//! Bindings for QtCore C++ library.
//!
//! Starting guide and examples are available at
//! &#32;   &#32;   &#32;   [https://github.com/rust-qt/examples](https://github.com/rust-qt/examples).
//!
//! This crate was generated for Qt 5.9.7, 5.11.3, 5.12.2, 5.13.0.
//! &#32;   &#32;   &#32;   &#32;   It should work with other Qt versions later than 5.9.7,
//! &#32;   &#32;   &#32;   &#32;   but API specific to other versions will not be available.
//!
//! This crate was generated by `ritual`.
//! &#32;   &#32;   &#32;   &#32;   &#32;   See [README](https://github.com/rust-qt/ritual) for more information.

mod connect;
mod impl_arguments_compatible;
mod impl_q_byte_array;
mod impl_q_core_application;
mod impl_q_string;
mod q_debug_shim;
mod q_flags;
mod q_message_logger_macros;

pub use crate::connect::{ArgumentsCompatible, AsReceiver, Receiver, Signal};
pub use crate::impl_q_core_application::QCoreApplicationArgs;
pub use crate::q_debug_shim::{qdebug, QDebugShim};
pub use crate::q_flags::QFlags;

mod __ffi {
    //! Functions provided by the C++ wrapper library

    include!(concat!(env!("OUT_DIR"), "/ffi.rs"));
}
pub mod ops;
pub use ::cpp_utils;
pub mod q_sys_info {
    //! C++ type: <span style='color: green;'>```QSysInfo```</span>

    /// <p>This enum provides platform-specific information about the sizes of data structures used by the underlying architecture.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QSysInfo::Sizes```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsysinfo.html#Sizes-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum provides platform-specific information about the sizes of data structures used by the underlying architecture.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Sizes(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Sizes {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Sizes(value)
        }
    }

    impl From<Sizes> for ::std::os::raw::c_int {
        fn from(value: Sizes) -> Self {
            value.0
        }
    }

    impl Sizes {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Sizes {
        /// The size in bits of a pointer for the platform on which the application is compiled (32 or 64). (C++ enum variant: <span style='color: green;'>```WordSize = 64```</span>)
        #[allow(non_upper_case_globals)]
        pub const WordSize: crate::q_sys_info::Sizes = crate::q_sys_info::Sizes(64);
    }

    /// C++ enum: <span style='color: green;'>```QSysInfo::Endian```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsysinfo.html#Endian-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Endian(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Endian {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Endian(value)
        }
    }

    impl From<Endian> for ::std::os::raw::c_int {
        fn from(value: Endian) -> Self {
            value.0
        }
    }

    impl Endian {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Endian {
        /// Big-endian byte order (also called Network byte order) (C++ enum variant: <span style='color: green;'>```BigEndian = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const BigEndian: crate::q_sys_info::Endian = crate::q_sys_info::Endian(0);
        /// Little-endian byte order (C++ enum variant: <span style='color: green;'>```LittleEndian = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const LittleEndian: crate::q_sys_info::Endian = crate::q_sys_info::Endian(1);
        /// Equals BigEndian or LittleEndian, depending on the platform's byte order. (C++ enum variant: <span style='color: green;'>```ByteOrder = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const ByteOrder: crate::q_sys_info::Endian = crate::q_sys_info::Endian(1);
    }

    /// <p>This enum provides symbolic names for the various versions of the Windows operating system. On Windows, the <a href="http://doc.qt.io/qt-5/qsysinfo-obsolete.html#WindowsVersion-var">QSysInfo::WindowsVersion</a> variable gives the version of the system on which the application is run.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QSysInfo::WinVersion```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsysinfo-obsolete.html#WinVersion-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum provides symbolic names for the various versions of the Windows operating system. On Windows, the <a href="http://doc.qt.io/qt-5/qsysinfo-obsolete.html#WindowsVersion-var">QSysInfo::WindowsVersion</a> variable gives the version of the system on which the application is run.</p>
    /// <p>MS-DOS-based versions:</p>
    ///
    /// <p>NT-based versions (note that each operating system version is only represented once rather than each Windows edition):</p>
    ///
    /// <p>Alternatively, you may use the following macros which correspond directly to the Windows operating system version number:</p>
    ///
    /// <p>The following masks can be used for testing whether a Windows version is MS-DOS-based or NT-based:</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsysinfo-obsolete.html#MacVersion-enum">MacVersion</a>.</p>
    ///
    /// <h2>Member Function Documentation</h2></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct WinVersion(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for WinVersion {
        fn from(value: ::std::os::raw::c_int) -> Self {
            WinVersion(value)
        }
    }

    impl From<WinVersion> for ::std::os::raw::c_int {
        fn from(value: WinVersion) -> Self {
            value.0
        }
    }

    impl WinVersion {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl WinVersion {
        /// Operating system other than Windows. (C++ enum variant: <span style='color: green;'>```WV_None = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const WVNone: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(0);
        /// Windows 3.1 with Win 32s (C++ enum variant: <span style='color: green;'>```WV_32s = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const WV32S: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(1);
        /// Windows 95 (C++ enum variant: <span style='color: green;'>```WV_95 = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const WV95: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(2);
        /// Windows 98 (C++ enum variant: <span style='color: green;'>```WV_98 = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const WV98: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(3);
        /// Windows Me (C++ enum variant: <span style='color: green;'>```WV_Me = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const WVMe: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(4);
        /// MS-DOS-based version of Windows (C++ enum variant: <span style='color: green;'>```WV_DOS_based = 15```</span>)
        #[allow(non_upper_case_globals)]
        pub const WVDOSBased: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(15);
        /// Windows NT (operating system version 4.0) (C++ enum variant: <span style='color: green;'>```WV_NT = 16```</span>)
        #[allow(non_upper_case_globals)]
        pub const WVNT: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(16);
        /// Windows 2000 (operating system version 5.0) (C++ enum variant: <span style='color: green;'>```WV_2000 = 32```</span>)
        #[allow(non_upper_case_globals)]
        pub const WV2000: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(32);
        /// Windows XP (operating system version 5.1) (C++ enum variant: <span style='color: green;'>```WV_XP = 48```</span>)
        #[allow(non_upper_case_globals)]
        pub const WVXP: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(48);
        /// Windows Server 2003, Windows Server 2003 R2, Windows Home Server, Windows XP Professional x64 Edition (operating system version 5.2) (C++ enum variant: <span style='color: green;'>```WV_2003 = 64```</span>)
        #[allow(non_upper_case_globals)]
        pub const WV2003: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(64);
        /// Windows Vista, Windows Server 2008 (operating system version 6.0) (C++ enum variant: <span style='color: green;'>```WV_VISTA = 128```</span>)
        #[allow(non_upper_case_globals)]
        pub const WVVISTA: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(128);
        /// Windows 7, Windows Server 2008 R2 (operating system version 6.1) (C++ enum variant: <span style='color: green;'>```WV_WINDOWS7 = 144```</span>)
        #[allow(non_upper_case_globals)]
        pub const WVWINDOWS7: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(144);
        /// Windows 8 (operating system version 6.2) (C++ enum variant: <span style='color: green;'>```WV_WINDOWS8 = 160```</span>)
        #[allow(non_upper_case_globals)]
        pub const WVWINDOWS8: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(160);
        /// Windows 8.1 (operating system version 6.3), introduced in Qt 5.2 (C++ enum variant: <span style='color: green;'>```WV_WINDOWS8_1 = 176```</span>)
        #[allow(non_upper_case_globals)]
        pub const WVWINDOWS81: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(176);
        /// Windows 10 (operating system version 10.0), introduced in Qt 5.5 (C++ enum variant: <span style='color: green;'>```WV_WINDOWS10 = 192```</span>)
        #[allow(non_upper_case_globals)]
        pub const WVWINDOWS10: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(192);
        /// NT-based version of Windows (C++ enum variant: <span style='color: green;'>```WV_NT_based = 240```</span>)
        #[allow(non_upper_case_globals)]
        pub const WVNTBased: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(240);
        /// Operating system version 4.0, corresponds to Windows NT (C++ enum variant: <span style='color: green;'>```WV_4_0 = 16```</span>)
        #[allow(non_upper_case_globals)]
        pub const WV40: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(16);
        /// Operating system version 5.0, corresponds to Windows 2000 (C++ enum variant: <span style='color: green;'>```WV_5_0 = 32```</span>)
        #[allow(non_upper_case_globals)]
        pub const WV50: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(32);
        /// Operating system version 5.1, corresponds to Windows XP (C++ enum variant: <span style='color: green;'>```WV_5_1 = 48```</span>)
        #[allow(non_upper_case_globals)]
        pub const WV51: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(48);
        /// Operating system version 5.2, corresponds to Windows Server 2003, Windows Server 2003 R2, Windows Home Server, and Windows XP Professional x64 Edition (C++ enum variant: <span style='color: green;'>```WV_5_2 = 64```</span>)
        #[allow(non_upper_case_globals)]
        pub const WV52: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(64);
        /// Operating system version 6.0, corresponds to Windows Vista and Windows Server 2008 (C++ enum variant: <span style='color: green;'>```WV_6_0 = 128```</span>)
        #[allow(non_upper_case_globals)]
        pub const WV60: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(128);
        /// Operating system version 6.1, corresponds to Windows 7 and Windows Server 2008 R2 (C++ enum variant: <span style='color: green;'>```WV_6_1 = 144```</span>)
        #[allow(non_upper_case_globals)]
        pub const WV61: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(144);
        /// Operating system version 6.2, corresponds to Windows 8 (C++ enum variant: <span style='color: green;'>```WV_6_2 = 160```</span>)
        #[allow(non_upper_case_globals)]
        pub const WV62: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(160);
        /// Operating system version 6.3, corresponds to Windows 8.1, introduced in Qt 5.2 (C++ enum variant: <span style='color: green;'>```WV_6_3 = 176```</span>)
        #[allow(non_upper_case_globals)]
        pub const WV63: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(176);
        /// Operating system version 10.0, corresponds to Windows 10, introduced in Qt 5.5 (C++ enum variant: <span style='color: green;'>```WV_10_0 = 192```</span>)
        #[allow(non_upper_case_globals)]
        pub const WV100: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(192);
        /// C++ enum variant: <span style='color: green;'>```WV_CE = 256```</span>
        #[allow(non_upper_case_globals)]
        pub const WVCE: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(256);
        /// C++ enum variant: <span style='color: green;'>```WV_CENET = 512```</span>
        #[allow(non_upper_case_globals)]
        pub const WVCENET: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(512);
        /// C++ enum variant: <span style='color: green;'>```WV_CE_5 = 768```</span>
        #[allow(non_upper_case_globals)]
        pub const WVCE5: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(768);
        /// C++ enum variant: <span style='color: green;'>```WV_CE_6 = 1024```</span>
        #[allow(non_upper_case_globals)]
        pub const WVCE6: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(1024);
        /// C++ enum variant: <span style='color: green;'>```WV_CE_based = 3840```</span>
        #[allow(non_upper_case_globals)]
        pub const WVCEBased: crate::q_sys_info::WinVersion = crate::q_sys_info::WinVersion(3840);
    }

    /// <p>This enum provides symbolic names for the various versions of the Darwin operating system, covering both <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS. The <a href="http://doc.qt.io/qt-5/qsysinfo-obsolete.html#MacintoshVersion-var">QSysInfo::MacintoshVersion</a> variable gives the version of the system on which the application is run.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QSysInfo::MacVersion```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsysinfo-obsolete.html#MacVersion-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum provides symbolic names for the various versions of the Darwin operating system, covering both <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS. The <a href="http://doc.qt.io/qt-5/qsysinfo-obsolete.html#MacintoshVersion-var">QSysInfo::MacintoshVersion</a> variable gives the version of the system on which the application is run.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsysinfo-obsolete.html#WinVersion-enum">WinVersion</a>.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct MacVersion(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for MacVersion {
        fn from(value: ::std::os::raw::c_int) -> Self {
            MacVersion(value)
        }
    }

    impl From<MacVersion> for ::std::os::raw::c_int {
        fn from(value: MacVersion) -> Self {
            value.0
        }
    }

    impl MacVersion {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl MacVersion {
        /// Not a Darwin operating system (C++ enum variant: <span style='color: green;'>```MV_None = 65535```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVNone: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(65535);
        /// An unknown and currently unsupported platform (C++ enum variant: <span style='color: green;'>```MV_Unknown = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVUnknown: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(0);
        /// <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> 9 (C++ enum variant: <span style='color: green;'>```MV_9 = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const MV9: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(1);
        /// <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> 10.0 (C++ enum variant: <span style='color: green;'>```MV_10_0 = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const MV100: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(2);
        /// <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> 10.1 (C++ enum variant: <span style='color: green;'>```MV_10_1 = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const MV101: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(3);
        /// <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> 10.2 (C++ enum variant: <span style='color: green;'>```MV_10_2 = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const MV102: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(4);
        /// <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> 10.3 (C++ enum variant: <span style='color: green;'>```MV_10_3 = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const MV103: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(5);
        /// <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> 10.4 (C++ enum variant: <span style='color: green;'>```MV_10_4 = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const MV104: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(6);
        /// <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> 10.5 (C++ enum variant: <span style='color: green;'>```MV_10_5 = 7```</span>)
        #[allow(non_upper_case_globals)]
        pub const MV105: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(7);
        /// <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> 10.6 (C++ enum variant: <span style='color: green;'>```MV_10_6 = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const MV106: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(8);
        /// <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> 10.7 (C++ enum variant: <span style='color: green;'>```MV_10_7 = 9```</span>)
        #[allow(non_upper_case_globals)]
        pub const MV107: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(9);
        /// <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> 10.8 (C++ enum variant: <span style='color: green;'>```MV_10_8 = 10```</span>)
        #[allow(non_upper_case_globals)]
        pub const MV108: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(10);
        /// <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> 10.9 (C++ enum variant: <span style='color: green;'>```MV_10_9 = 11```</span>)
        #[allow(non_upper_case_globals)]
        pub const MV109: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(11);
        /// <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> 10.10 (C++ enum variant: <span style='color: green;'>```MV_10_10 = 12```</span>)
        #[allow(non_upper_case_globals)]
        pub const MV1010: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(12);
        /// <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> 10.11 (C++ enum variant: <span style='color: green;'>```MV_10_11 = 13```</span>)
        #[allow(non_upper_case_globals)]
        pub const MV1011: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(13);
        /// <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> 10.12 (C++ enum variant: <span style='color: green;'>```MV_10_12 = 14```</span>)
        #[allow(non_upper_case_globals)]
        pub const MV1012: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(14);
        /// Apple codename for MV_10_0 (C++ enum variant: <span style='color: green;'>```MV_CHEETAH = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVCHEETAH: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(2);
        /// Apple codename for MV_10_1 (C++ enum variant: <span style='color: green;'>```MV_PUMA = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVPUMA: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(3);
        /// Apple codename for MV_10_2 (C++ enum variant: <span style='color: green;'>```MV_JAGUAR = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVJAGUAR: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(4);
        /// Apple codename for MV_10_3 (C++ enum variant: <span style='color: green;'>```MV_PANTHER = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVPANTHER: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(5);
        /// Apple codename for MV_10_4 (C++ enum variant: <span style='color: green;'>```MV_TIGER = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVTIGER: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(6);
        /// Apple codename for MV_10_5 (C++ enum variant: <span style='color: green;'>```MV_LEOPARD = 7```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVLEOPARD: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(7);
        /// Apple codename for MV_10_6 (C++ enum variant: <span style='color: green;'>```MV_SNOWLEOPARD = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVSNOWLEOPARD: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(8);
        /// Apple codename for MV_10_7 (C++ enum variant: <span style='color: green;'>```MV_LION = 9```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVLION: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(9);
        /// Apple codename for MV_10_8 (C++ enum variant: <span style='color: green;'>```MV_MOUNTAINLION = 10```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVMOUNTAINLION: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(10);
        /// Apple codename for MV_10_9 (C++ enum variant: <span style='color: green;'>```MV_MAVERICKS = 11```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVMAVERICKS: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(11);
        /// Apple codename for MV_10_10 (C++ enum variant: <span style='color: green;'>```MV_YOSEMITE = 12```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVYOSEMITE: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(12);
        /// Apple codename for MV_10_11 (C++ enum variant: <span style='color: green;'>```MV_ELCAPITAN = 13```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVELCAPITAN: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(13);
        /// Apple codename for MV_10_12 (C++ enum variant: <span style='color: green;'>```MV_SIERRA = 14```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVSIERRA: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(14);
        /// iOS (any) (C++ enum variant: <span style='color: green;'>```MV_IOS = 256```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVIOS: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(256);
        /// iOS 4.3 (C++ enum variant: <span style='color: green;'>```MV_IOS_4_3 = 323```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVIOS43: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(323);
        /// iOS 5.0 (C++ enum variant: <span style='color: green;'>```MV_IOS_5_0 = 336```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVIOS50: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(336);
        /// iOS 5.1 (C++ enum variant: <span style='color: green;'>```MV_IOS_5_1 = 337```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVIOS51: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(337);
        /// iOS 6.0 (C++ enum variant: <span style='color: green;'>```MV_IOS_6_0 = 352```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVIOS60: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(352);
        /// iOS 6.1 (C++ enum variant: <span style='color: green;'>```MV_IOS_6_1 = 353```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVIOS61: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(353);
        /// iOS 7.0 (C++ enum variant: <span style='color: green;'>```MV_IOS_7_0 = 368```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVIOS70: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(368);
        /// iOS 7.1 (C++ enum variant: <span style='color: green;'>```MV_IOS_7_1 = 369```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVIOS71: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(369);
        /// iOS 8.0 (C++ enum variant: <span style='color: green;'>```MV_IOS_8_0 = 384```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVIOS80: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(384);
        /// iOS 8.1 (C++ enum variant: <span style='color: green;'>```MV_IOS_8_1 = 385```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVIOS81: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(385);
        /// iOS 8.2 (C++ enum variant: <span style='color: green;'>```MV_IOS_8_2 = 386```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVIOS82: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(386);
        /// iOS 8.3 (C++ enum variant: <span style='color: green;'>```MV_IOS_8_3 = 387```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVIOS83: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(387);
        /// iOS 8.4 (C++ enum variant: <span style='color: green;'>```MV_IOS_8_4 = 388```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVIOS84: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(388);
        /// iOS 9.0 (C++ enum variant: <span style='color: green;'>```MV_IOS_9_0 = 400```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVIOS90: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(400);
        /// iOS 9.1 (C++ enum variant: <span style='color: green;'>```MV_IOS_9_1 = 401```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVIOS91: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(401);
        /// iOS 9.2 (C++ enum variant: <span style='color: green;'>```MV_IOS_9_2 = 402```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVIOS92: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(402);
        /// iOS 9.3 (C++ enum variant: <span style='color: green;'>```MV_IOS_9_3 = 403```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVIOS93: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(403);
        /// iOS 10.0 (C++ enum variant: <span style='color: green;'>```MV_IOS_10_0 = 416```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVIOS100: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(416);
        /// tvOS (any) (C++ enum variant: <span style='color: green;'>```MV_TVOS = 512```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVTVOS: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(512);
        /// tvOS 9.0 (C++ enum variant: <span style='color: green;'>```MV_TVOS_9_0 = 656```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVTVOS90: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(656);
        /// tvOS 9.1 (C++ enum variant: <span style='color: green;'>```MV_TVOS_9_1 = 657```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVTVOS91: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(657);
        /// tvOS 9.2 (C++ enum variant: <span style='color: green;'>```MV_TVOS_9_2 = 658```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVTVOS92: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(658);
        /// tvOS 10.0 (C++ enum variant: <span style='color: green;'>```MV_TVOS_10_0 = 672```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVTVOS100: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(672);
        /// watchOS (any) (C++ enum variant: <span style='color: green;'>```MV_WATCHOS = 1024```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVWATCHOS: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(1024);
        /// watchOS 2.0 (C++ enum variant: <span style='color: green;'>```MV_WATCHOS_2_0 = 1056```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVWATCHOS20: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(1056);
        /// watchOS 2.1 (C++ enum variant: <span style='color: green;'>```MV_WATCHOS_2_1 = 1057```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVWATCHOS21: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(1057);
        /// watchOS 2.2 (C++ enum variant: <span style='color: green;'>```MV_WATCHOS_2_2 = 1058```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVWATCHOS22: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(1058);
        /// watchOS 3.0 (C++ enum variant: <span style='color: green;'>```MV_WATCHOS_3_0 = 1072```</span>)
        #[allow(non_upper_case_globals)]
        pub const MVWATCHOS30: crate::q_sys_info::MacVersion = crate::q_sys_info::MacVersion(1072);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qsysinfo.html">QSysInfo</a> class provides information about the system.</p>
///
/// C++ class: <span style='color: green;'>```QSysInfo```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qsysinfo.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsysinfo.html">QSysInfo</a> class provides information about the system.</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qsysinfo.html#Sizes-enum">WordSize</a> specifies the size of a pointer for the platform on which the application is compiled.</li>
/// <li><a href="http://doc.qt.io/qt-5/qsysinfo.html#Endian-enum">ByteOrder</a> specifies whether the platform is big-endian or little-endian.</li>
/// </ul>
/// <p>Some constants are defined only on certain platforms. You can use the preprocessor symbols <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_OS_WIN">Q_OS_WIN</a> and <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_OS_MACOS">Q_OS_MACOS</a> to test that the application is compiled under Windows or <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>.</p></div>
#[repr(C)]
pub struct QSysInfo {
    _unused: u8,
}
impl QSysInfo {
    /// <p>Returns a unique ID for this machine's boot, if one can be determined. If no unique ID could be determined, this function returns an empty byte array. This value is expected to change after every boot and can be considered globally unique.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QSysInfo::bootUniqueId()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsysinfo.html#bootUniqueId">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a unique ID for this machine's boot, if one can be determined. If no unique ID could be determined, this function returns an empty byte array. This value is expected to change after every boot and can be considered globally unique.</p>
    /// <p>This function is currently only implemented for Linux and Apple operating systems.</p>
    /// <p>This function was introduced in  Qt 5.11.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsysinfo.html#machineUniqueId">machineUniqueId</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn boot_unique_id() -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSysInfo_bootUniqueId();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the full architecture string that Qt was compiled for. This string is useful for identifying different, incompatible builds. For example, it can be used as an identifier to request an upgrade package from a server.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QSysInfo::buildAbi()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsysinfo.html#buildAbi">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the full architecture string that Qt was compiled for. This string is useful for identifying different, incompatible builds. For example, it can be used as an identifier to request an upgrade package from a server.</p>
    /// <p>The values returned from this function are kept stable as follows: the mandatory components of the result will not change in future versions of Qt, but optional suffixes may be added.</p>
    /// <p>The returned value is composed of three or more parts, separated by dashes ("-"). They are:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Component</th><th>Value</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>CPU Architecture</td><td>The same as <a href="http://doc.qt.io/qt-5/qsysinfo.html#buildCpuArchitecture">QSysInfo::buildCpuArchitecture</a>(), such as "arm", "i386", "mips" or "x86_64"</td></tr>
    /// <tr valign="top" class="even"><td>Endianness</td><td>"little_endian" or "big_endian"</td></tr>
    /// <tr valign="top" class="odd"><td>Word size</td><td>Whether it's a 32- or 64-bit application. Possible values are: "llp64" (Windows 64-bit), "lp64" (Unix 64-bit), "ilp32" (32-bit)</td></tr>
    /// <tr valign="top" class="even"><td>(Optional) ABI</td><td>Zero or more components identifying different ABIs possible in this architecture. Currently, Qt has optional ABI components for ARM and MIPS processors: one component is the main ABI (such as "eabi", "o32", "n32", "o64"); another is whether the calling convention is using hardware floating point registers ("hardfloat" is present).<p>Additionally, if Qt was configured with <code>-qreal float</code>, the ABI option tag "qreal_float" will be present. If Qt was configured with another type as qreal, that type is present after "qreal_", with all characters other than letters and digits escaped by an underscore, followed by two hex digits. For example, <code>-qreal long double</code> becomes "qreal_long_20double".</p>
    /// </td></tr>
    /// </tbody></table></div>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsysinfo.html#buildCpuArchitecture">QSysInfo::buildCpuArchitecture</a>().</p></div>
    pub unsafe fn build_abi() -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSysInfo_buildAbi();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the architecture of the CPU that Qt was compiled for, in text format. Note that this may not match the actual CPU that the application is running on if there's an emulation layer or if the CPU supports multiple architectures (like x86-64 processors supporting i386 applications). To detect that, use <a href="http://doc.qt.io/qt-5/qsysinfo.html#currentCpuArchitecture">currentCpuArchitecture</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QSysInfo::buildCpuArchitecture()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsysinfo.html#buildCpuArchitecture">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the architecture of the CPU that Qt was compiled for, in text format. Note that this may not match the actual CPU that the application is running on if there's an emulation layer or if the CPU supports multiple architectures (like x86-64 processors supporting i386 applications). To detect that, use <a href="http://doc.qt.io/qt-5/qsysinfo.html#currentCpuArchitecture">currentCpuArchitecture</a>().</p>
    /// <p>Values returned by this function are stable and will not change over time, so applications can rely on the returned value as an identifier, except that new CPU types may be added over time.</p>
    /// <p>Typical returned values are (note: list not exhaustive):</p>
    /// <ul>
    /// <li>"arm"</li>
    /// <li>"arm64"</li>
    /// <li>"i386"</li>
    /// <li>"ia64"</li>
    /// <li>"mips"</li>
    /// <li>"mips64"</li>
    /// <li>"power"</li>
    /// <li>"power64"</li>
    /// <li>"sparc"</li>
    /// <li>"sparcv9"</li>
    /// <li>"x86_64"</li>
    /// </ul>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsysinfo.html#buildAbi">QSysInfo::buildAbi</a>() and <a href="http://doc.qt.io/qt-5/qsysinfo.html#currentCpuArchitecture">QSysInfo::currentCpuArchitecture</a>().</p></div>
    pub unsafe fn build_cpu_architecture() -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSysInfo_buildCpuArchitecture();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qsysinfo.html">QSysInfo</a> class provides information about the system.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSysInfo& QSysInfo::operator=(const QSysInfo& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsysinfo.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsysinfo.html">QSysInfo</a> class provides information about the system.</p>
    /// <ul>
    /// <li><a href="http://doc.qt.io/qt-5/qsysinfo.html#Sizes-enum">WordSize</a> specifies the size of a pointer for the platform on which the application is compiled.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qsysinfo.html#Endian-enum">ByteOrder</a> specifies whether the platform is big-endian or little-endian.</li>
    /// </ul>
    /// <p>Some constants are defined only on certain platforms. You can use the preprocessor symbols <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_OS_WIN">Q_OS_WIN</a> and <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_OS_MACOS">Q_OS_MACOS</a> to test that the application is compiled under Windows or <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSysInfo>>,
    ) -> ::cpp_utils::MutRef<crate::QSysInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSysInfo_operator_(
            self as *mut crate::QSysInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSysInfo>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the architecture of the CPU that the application is running on, in text format. Note that this function depends on what the OS will report and may not detect the actual CPU architecture if the OS hides that information or is unable to provide it. For example, a 32-bit OS running on a 64-bit CPU is usually unable to determine the CPU is actually capable of running 64-bit programs.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QSysInfo::currentCpuArchitecture()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsysinfo.html#currentCpuArchitecture">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the architecture of the CPU that the application is running on, in text format. Note that this function depends on what the OS will report and may not detect the actual CPU architecture if the OS hides that information or is unable to provide it. For example, a 32-bit OS running on a 64-bit CPU is usually unable to determine the CPU is actually capable of running 64-bit programs.</p>
    /// <p>Values returned by this function are mostly stable: an attempt will be made to ensure that they stay constant over time and match the values returned by QSysInfo::builldCpuArchitecture(). However, due to the nature of the operating system functions being used, there may be discrepancies.</p>
    /// <p>Typical returned values are (note: list not exhaustive):</p>
    /// <ul>
    /// <li>"arm"</li>
    /// <li>"arm64"</li>
    /// <li>"i386"</li>
    /// <li>"ia64"</li>
    /// <li>"mips"</li>
    /// <li>"mips64"</li>
    /// <li>"power"</li>
    /// <li>"power64"</li>
    /// <li>"sparc"</li>
    /// <li>"sparcv9"</li>
    /// <li>"x86_64"</li>
    /// </ul>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsysinfo.html#buildAbi">QSysInfo::buildAbi</a>() and <a href="http://doc.qt.io/qt-5/qsysinfo.html#buildCpuArchitecture">QSysInfo::buildCpuArchitecture</a>().</p></div>
    pub unsafe fn current_cpu_architecture() -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSysInfo_currentCpuArchitecture();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the type of the operating system kernel Qt was compiled for. It's also the kernel the application is running on, unless the host operating system is running a form of compatibility or virtualization layer.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QSysInfo::kernelType()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsysinfo.html#kernelType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the type of the operating system kernel Qt was compiled for. It's also the kernel the application is running on, unless the host operating system is running a form of compatibility or virtualization layer.</p>
    /// <p>Values returned by this function are stable and will not change over time, so applications can rely on the returned value as an identifier, except that new OS kernel types may be added over time.</p>
    /// <p>On Windows, this function returns the type of Windows kernel, like "winnt". On Unix systems, it returns the same as the output of <code>uname -s</code> (lowercased).</p>
    /// <p><b>Note: </b>This function may return surprising values: it returns "linux" for all operating systems running Linux (including Android), "qnx" for all operating systems running QNX, "freebsd" for Debian/kFreeBSD, and "darwin" for <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS. For information on the type of product the application is running on, see <a href="http://doc.qt.io/qt-5/qsysinfo.html#productType">productType</a>().</p><p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileselector.html">QFileSelector</a>, <a href="http://doc.qt.io/qt-5/qsysinfo.html#kernelVersion">kernelVersion</a>(), <a href="http://doc.qt.io/qt-5/qsysinfo.html#productType">productType</a>(), <a href="http://doc.qt.io/qt-5/qsysinfo.html#productVersion">productVersion</a>(), and <a href="http://doc.qt.io/qt-5/qsysinfo.html#prettyProductName">prettyProductName</a>().</p></div>
    pub unsafe fn kernel_type() -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSysInfo_kernelType();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the release version of the operating system kernel. On Windows, it returns the version of the NT kernel. On Unix systems, including Android and <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, it returns the same as the <code>uname -r</code> command would return.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QSysInfo::kernelVersion()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsysinfo.html#kernelVersion">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the release version of the operating system kernel. On Windows, it returns the version of the NT kernel. On Unix systems, including Android and <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, it returns the same as the <code>uname -r</code> command would return.</p>
    /// <p>If the version could not be determined, this function may return an empty string.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsysinfo.html#kernelType">kernelType</a>(), <a href="http://doc.qt.io/qt-5/qsysinfo.html#productType">productType</a>(), <a href="http://doc.qt.io/qt-5/qsysinfo.html#productVersion">productVersion</a>(), and <a href="http://doc.qt.io/qt-5/qsysinfo.html#prettyProductName">prettyProductName</a>().</p></div>
    pub unsafe fn kernel_version() -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSysInfo_kernelVersion();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the version of Darwin (<a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> or iOS) on which the application is run, or <a href="http://doc.qt.io/qt-5/qsysinfo-obsolete.html#MacVersion-enum">MV_None</a> if the operating system is not a version of Darwin.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QSysInfo::MacVersion QSysInfo::macVersion()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsysinfo-obsolete.html#macVersion">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the version of Darwin (<a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> or iOS) on which the application is run, or <a href="http://doc.qt.io/qt-5/qsysinfo-obsolete.html#MacVersion-enum">MV_None</a> if the operating system is not a version of Darwin.</p></div>
    pub unsafe fn mac_version() -> crate::q_sys_info::MacVersion {
        crate::__ffi::ctr_qt_core_ffi_QSysInfo_macVersion()
    }

    /// <p>Returns this machine's host name, if one is configured. Note that hostnames are not guaranteed to be globally unique, especially if they were configured automatically.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QSysInfo::machineHostName()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsysinfo.html#machineHostName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns this machine's host name, if one is configured. Note that hostnames are not guaranteed to be globally unique, especially if they were configured automatically.</p>
    /// <p>This function does not guarantee the returned host name is a Fully Qualified Domain Name (FQDN). For that, use <a href="http://doc.qt.io/qt-5/qhostinfo.html">QHostInfo</a> to resolve the returned name to an FQDN.</p>
    /// <p>This function returns the same as <a href="http://doc.qt.io/qt-5/qhostinfo.html#localHostName">QHostInfo::localHostName</a>().</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhostinfo.html#localDomainName">QHostInfo::localDomainName</a>.</p></div>
    pub unsafe fn machine_host_name() -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSysInfo_machineHostName();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a unique ID for this machine, if one can be determined. If no unique ID could be determined, this function returns an empty byte array. Unlike <a href="http://doc.qt.io/qt-5/qsysinfo.html#machineHostName">machineHostName</a>(), the value returned by this function is likely globally unique.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QSysInfo::machineUniqueId()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsysinfo.html#machineUniqueId">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a unique ID for this machine, if one can be determined. If no unique ID could be determined, this function returns an empty byte array. Unlike <a href="http://doc.qt.io/qt-5/qsysinfo.html#machineHostName">machineHostName</a>(), the value returned by this function is likely globally unique.</p>
    /// <p>A unique ID is useful in network operations to identify this machine for an extended period of time, when the IP address could change or if this machine could have more than one IP address. For example, the ID could be used when communicating with a server or when storing device-specific data in shared network storage.</p>
    /// <p>Note that on some systems, this value will persist across reboots and on some it will not. Applications should not blindly depend on this fact without verifying the OS capabilities. In particular, on Linux systems, this ID is usually permanent and it matches the D-Bus machine ID, except for nodes without their own storage (replicated nodes).</p>
    /// <p>This function was introduced in  Qt 5.11.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsysinfo.html#machineHostName">machineHostName</a>() and <a href="http://doc.qt.io/qt-5/qsysinfo.html#bootUniqueId">bootUniqueId</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn machine_unique_id() -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSysInfo_machineUniqueId();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Returns the value of the <span style='color: green;'>```MacintoshVersion```</span> field.
    pub unsafe fn macintosh_version() -> crate::q_sys_info::MacVersion {
        crate::__ffi::ctr_qt_core_ffi_QSysInfo_MacintoshVersion()
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qsysinfo.html">QSysInfo</a> class provides information about the system.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSysInfo::QSysInfo()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsysinfo.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsysinfo.html">QSysInfo</a> class provides information about the system.</p>
    /// <ul>
    /// <li><a href="http://doc.qt.io/qt-5/qsysinfo.html#Sizes-enum">WordSize</a> specifies the size of a pointer for the platform on which the application is compiled.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qsysinfo.html#Endian-enum">ByteOrder</a> specifies whether the platform is big-endian or little-endian.</li>
    /// </ul>
    /// <p>Some constants are defined only on certain platforms. You can use the preprocessor symbols <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_OS_WIN">Q_OS_WIN</a> and <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_OS_MACOS">Q_OS_MACOS</a> to test that the application is compiled under Windows or <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QSysInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSysInfo_QSysInfo();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qsysinfo.html">QSysInfo</a> class provides information about the system.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSysInfo::QSysInfo(const QSysInfo& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsysinfo.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsysinfo.html">QSysInfo</a> class provides information about the system.</p>
    /// <ul>
    /// <li><a href="http://doc.qt.io/qt-5/qsysinfo.html#Sizes-enum">WordSize</a> specifies the size of a pointer for the platform on which the application is compiled.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qsysinfo.html#Endian-enum">ByteOrder</a> specifies whether the platform is big-endian or little-endian.</li>
    /// </ul>
    /// <p>Some constants are defined only on certain platforms. You can use the preprocessor symbols <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_OS_WIN">Q_OS_WIN</a> and <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_OS_MACOS">Q_OS_MACOS</a> to test that the application is compiled under Windows or <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSysInfo>>,
    ) -> ::cpp_utils::CppBox<crate::QSysInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSysInfo_QSysInfo1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSysInfo>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a prettier form of <a href="http://doc.qt.io/qt-5/qsysinfo.html#productType">productType</a>() and <a href="http://doc.qt.io/qt-5/qsysinfo.html#productVersion">productVersion</a>(), containing other tokens like the operating system type, codenames and other information. The result of this function is suitable for displaying to the user, but not for long-term storage, as the string may change with updates to Qt.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QSysInfo::prettyProductName()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsysinfo.html#prettyProductName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a prettier form of <a href="http://doc.qt.io/qt-5/qsysinfo.html#productType">productType</a>() and <a href="http://doc.qt.io/qt-5/qsysinfo.html#productVersion">productVersion</a>(), containing other tokens like the operating system type, codenames and other information. The result of this function is suitable for displaying to the user, but not for long-term storage, as the string may change with updates to Qt.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qsysinfo.html#productType">productType</a>() is "unknown", this function will instead use the <a href="http://doc.qt.io/qt-5/qsysinfo.html#kernelType">kernelType</a>() and <a href="http://doc.qt.io/qt-5/qsysinfo.html#kernelVersion">kernelVersion</a>() functions.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsysinfo.html#kernelType">kernelType</a>(), <a href="http://doc.qt.io/qt-5/qsysinfo.html#kernelVersion">kernelVersion</a>(), <a href="http://doc.qt.io/qt-5/qsysinfo.html#productType">productType</a>(), and <a href="http://doc.qt.io/qt-5/qsysinfo.html#productVersion">productVersion</a>().</p></div>
    pub unsafe fn pretty_product_name() -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSysInfo_prettyProductName();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the product name of the operating system this application is running in. If the application is running on some sort of emulation or virtualization layer (such as WINE on a Unix system), this function will inspect the emulation / virtualization layer.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QSysInfo::productType()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsysinfo.html#productType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the product name of the operating system this application is running in. If the application is running on some sort of emulation or virtualization layer (such as WINE on a Unix system), this function will inspect the emulation / virtualization layer.</p>
    /// <p>Values returned by this function are stable and will not change over time, so applications can rely on the returned value as an identifier, except that new OS types may be added over time.</p>
    /// <p><b>Linux and Android note</b>: this function returns "android" for Linux systems running Android userspace, notably when using the Bionic library. For all other Linux systems, regardless of C library being used, it tries to determine the distribution name and returns that. If determining the distribution name failed, it returns "unknown".</p>
    /// <p><b><a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> note</b>: this function returns "osx" for all <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> systems, regardless of Apple naming convention. The returned string will be updated for Qt 6. Note that this function erroneously returned "macos" for <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> 10.12 in Qt versions 5.6.2, 5.7.1, and 5.8.0.</p>
    /// <p><b>Darwin, iOS, tvOS, and watchOS note</b>: this function returns "ios" for iOS systems, "tvos" for tvOS systems, "watchos" for watchOS systems, and "darwin" in case the system could not be determined.</p>
    /// <p><b><a href="http://doc.qt.io/qt-5/qtcore-attribution-freebsd.html#freebsd">FreeBSD</a> note</b>: this function returns "debian" for Debian/kFreeBSD and "unknown" otherwise.</p>
    /// <p><b>Windows note</b>: this function "winrt" for <a href="http://doc.qt.io/qt-5/build-sources.html#winrt">WinRT</a> builds, and "windows" for normal desktop builds.</p>
    /// <p>For other Unix-type systems, this function usually returns "unknown".</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileselector.html">QFileSelector</a>, <a href="http://doc.qt.io/qt-5/qsysinfo.html#kernelType">kernelType</a>(), <a href="http://doc.qt.io/qt-5/qsysinfo.html#kernelVersion">kernelVersion</a>(), <a href="http://doc.qt.io/qt-5/qsysinfo.html#productVersion">productVersion</a>(), and <a href="http://doc.qt.io/qt-5/qsysinfo.html#prettyProductName">prettyProductName</a>().</p></div>
    pub unsafe fn product_type() -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSysInfo_productType();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the product version of the operating system in string form. If the version could not be determined, this function returns "unknown".</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QSysInfo::productVersion()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsysinfo.html#productVersion">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the product version of the operating system in string form. If the version could not be determined, this function returns "unknown".</p>
    /// <p>It will return the Android, iOS, <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, Windows full-product versions on those systems.</p>
    /// <p>Typical returned values are (note: list not exhaustive):</p>
    /// <ul>
    /// <li>"2016.09" (Amazon Linux AMI 2016.09)</li>
    /// <li>"7.1" (Android Nougat)</li>
    /// <li>"25" (Fedora 25)</li>
    /// <li>"10.1" (iOS 10.1)</li>
    /// <li>"10.12" (<a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> Sierra)</li>
    /// <li>"10.0" (tvOS 10)</li>
    /// <li>"16.10" (Ubuntu 16.10)</li>
    /// <li>"3.1" (watchOS 3.1)</li>
    /// <li>"7 SP 1" (Windows 7 Service Pack 1)</li>
    /// <li>"8.1" (Windows 8.1)</li>
    /// <li>"10" (Windows 10)</li>
    /// <li>"Server 2016" (Windows Server 2016)</li>
    /// </ul>
    /// <p>On Linux systems, it will try to determine the distribution version and will return that. This is also done on Debian/kFreeBSD, so this function will return Debian version in that case.</p>
    /// <p>In all other Unix-type systems, this function always returns "unknown".</p>
    /// <p><b>Note: </b>The version string returned from this function is not guaranteed to be orderable. On Linux, the version of the distribution may jump unexpectedly, please refer to the distribution's documentation for versioning practices.</p><p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsysinfo.html#kernelType">kernelType</a>(), <a href="http://doc.qt.io/qt-5/qsysinfo.html#kernelVersion">kernelVersion</a>(), <a href="http://doc.qt.io/qt-5/qsysinfo.html#productType">productType</a>(), and <a href="http://doc.qt.io/qt-5/qsysinfo.html#prettyProductName">prettyProductName</a>().</p></div>
    pub unsafe fn product_version() -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSysInfo_productVersion();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Returns the value of the <span style='color: green;'>```WindowsVersion```</span> field.
    pub unsafe fn windows_version() -> crate::q_sys_info::WinVersion {
        crate::__ffi::ctr_qt_core_ffi_QSysInfo_WindowsVersion()
    }

    /// <p>Returns the version of the Windows operating system on which the application is run, or <a href="http://doc.qt.io/qt-5/qsysinfo-obsolete.html#WinVersion-enum">WV_None</a> if the operating system is not Windows.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QSysInfo::WinVersion QSysInfo::windowsVersion()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsysinfo-obsolete.html#windowsVersion">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the version of the Windows operating system on which the application is run, or <a href="http://doc.qt.io/qt-5/qsysinfo-obsolete.html#WinVersion-enum">WV_None</a> if the operating system is not Windows.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    ///
    /// <h2>Member Variable Documentation</h2></div>
    pub unsafe fn windows_version2() -> crate::q_sys_info::WinVersion {
        crate::__ffi::ctr_qt_core_ffi_QSysInfo_windowsVersion()
    }
}

/// <p>This enum describes the messages that can be sent to a message handler (<a href="http://doc.qt.io/qt-5/qtglobal.html#QtMessageHandler-typedef">QtMessageHandler</a>). You can use the enum to identify and associate the various message types with the appropriate actions.</p>
///
/// C++ enum: <span style='color: green;'>```QtMsgType```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtglobal.html#QtMsgType-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the messages that can be sent to a message handler (<a href="http://doc.qt.io/qt-5/qtglobal.html#QtMessageHandler-typedef">QtMessageHandler</a>). You can use the enum to identify and associate the various message types with the appropriate actions.</p>
///
/// <p><code>QtInfoMsg</code> was added in Qt 5.5.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtglobal.html#QtMessageHandler-typedef">QtMessageHandler</a> and <a href="http://doc.qt.io/qt-5/qtglobal.html#qInstallMessageHandler">qInstallMessageHandler</a>().</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct QtMsgType(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for QtMsgType {
    fn from(value: ::std::os::raw::c_int) -> Self {
        QtMsgType(value)
    }
}

impl From<QtMsgType> for ::std::os::raw::c_int {
    fn from(value: QtMsgType) -> Self {
        value.0
    }
}

impl QtMsgType {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl QtMsgType {
    /// A message generated by the <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() function. (C++ enum variant: <span style='color: green;'>```QtDebugMsg = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const QtDebugMsg: crate::QtMsgType = crate::QtMsgType(0);
    /// A message generated by the <a href="http://doc.qt.io/qt-5/qtglobal.html#qWarning">qWarning</a>() function. (C++ enum variant: <span style='color: green;'>```QtWarningMsg = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const QtWarningMsg: crate::QtMsgType = crate::QtMsgType(1);
    /// A message generated by the <a href="http://doc.qt.io/qt-5/qtglobal.html#qCritical">qCritical</a>() function. (C++ enum variant: <span style='color: green;'>```QtCriticalMsg = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const QtCriticalMsg: crate::QtMsgType = crate::QtMsgType(2);
    /// A message generated by the <a href="http://doc.qt.io/qt-5/qtglobal.html#qFatal">qFatal</a>() function. (C++ enum variant: <span style='color: green;'>```QtFatalMsg = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const QtFatalMsg: crate::QtMsgType = crate::QtMsgType(3);
    /// A message generated by the <a href="http://doc.qt.io/qt-5/qtglobal.html#qInfo">qInfo</a>() function. (C++ enum variant: <span style='color: green;'>```QtInfoMsg = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const QtInfoMsg: crate::QtMsgType = crate::QtMsgType(4);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```QtSystemMsg = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const QtSystemMsg: crate::QtMsgType = crate::QtMsgType(2);
}

/// <p>The <a href="http://doc.qt.io/qt-5/qmessagelogcontext.html">QMessageLogContext</a> class provides additional information about a log message.</p>
///
/// C++ class: <span style='color: green;'>```QMessageLogContext```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qmessagelogcontext.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmessagelogcontext.html">QMessageLogContext</a> class provides additional information about a log message.</p>
/// <p>The class provides information about the source code location a <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>(), <a href="http://doc.qt.io/qt-5/qtglobal.html#qInfo">qInfo</a>(), <a href="http://doc.qt.io/qt-5/qtglobal.html#qWarning">qWarning</a>(), <a href="http://doc.qt.io/qt-5/qtglobal.html#qCritical">qCritical</a>() or <a href="http://doc.qt.io/qt-5/qtglobal.html#qFatal">qFatal</a>() message was generated.</p>
/// <p><b>Note: </b>By default, this information is recorded only in debug builds. You can overwrite this explicitly by defining <code>QT_MESSAGELOGCONTEXT</code> or <code>QT_NO_MESSAGELOGCONTEXT</code>.</p></div>
#[repr(C)]
pub struct QMessageLogContext {
    _unused: u8,
}
impl QMessageLogContext {
    /// Returns the value of the <span style='color: green;'>```category```</span> field.
    pub unsafe fn category(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMessageLogContext_category(
            self as *const crate::QMessageLogContext,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Returns the value of the <span style='color: green;'>```file```</span> field.
    pub unsafe fn file(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMessageLogContext_file(
            self as *const crate::QMessageLogContext,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Returns the value of the <span style='color: green;'>```function```</span> field.
    pub unsafe fn function(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMessageLogContext_function(
            self as *const crate::QMessageLogContext,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Returns the value of the <span style='color: green;'>```line```</span> field.
    pub unsafe fn line(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMessageLogContext_line(
            self as *const crate::QMessageLogContext,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMessageLogContext::QMessageLogContext()```</span>.
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QMessageLogContext> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMessageLogContext_QMessageLogContext();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMessageLogContext::QMessageLogContext(const char* fileName, int lineNumber, const char* functionName, const char* categoryName)```</span>.
    pub unsafe fn new_4a(
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        line_number: ::std::os::raw::c_int,
        function_name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        category_name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QMessageLogContext> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMessageLogContext_QMessageLogContext1(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(file_name)
                .as_raw_ptr(),
            line_number,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(
                function_name,
            )
            .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(
                category_name,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Sets the value of the <span style='color: green;'>```category```</span> field.
    pub unsafe fn set_category(
        &mut self,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_set_QMessageLogContext_category(
            self as *mut crate::QMessageLogContext,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// Sets the value of the <span style='color: green;'>```file```</span> field.
    pub unsafe fn set_file(
        &mut self,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_set_QMessageLogContext_file(
            self as *mut crate::QMessageLogContext,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// Sets the value of the <span style='color: green;'>```function```</span> field.
    pub unsafe fn set_function(
        &mut self,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_set_QMessageLogContext_function(
            self as *mut crate::QMessageLogContext,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// Sets the value of the <span style='color: green;'>```line```</span> field.
    pub unsafe fn set_line(&mut self, value: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_set_QMessageLogContext_line(
            self as *mut crate::QMessageLogContext,
            value,
        )
    }

    /// Sets the value of the <span style='color: green;'>```version```</span> field.
    pub unsafe fn set_version(&mut self, value: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_set_QMessageLogContext_version(
            self as *mut crate::QMessageLogContext,
            value,
        )
    }

    /// Returns the value of the <span style='color: green;'>```version```</span> field.
    pub unsafe fn version(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMessageLogContext_version(
            self as *const crate::QMessageLogContext,
        )
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qmessagelogger.html">QMessageLogger</a> class generates log messages.</p>
///
/// C++ class: <span style='color: green;'>```QMessageLogger```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qmessagelogger.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmessagelogger.html">QMessageLogger</a> class generates log messages.</p>
/// <p><a href="http://doc.qt.io/qt-5/qmessagelogger.html">QMessageLogger</a> is used to generate messages for the Qt logging framework. Usually one uses it through <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>(), <a href="http://doc.qt.io/qt-5/qtglobal.html#qInfo">qInfo</a>(), <a href="http://doc.qt.io/qt-5/qtglobal.html#qWarning">qWarning</a>(), <a href="http://doc.qt.io/qt-5/qtglobal.html#qCritical">qCritical</a>, or <a href="http://doc.qt.io/qt-5/qtglobal.html#qFatal">qFatal</a>() functions, which are actually macros: For example <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() expands to <a href="http://doc.qt.io/qt-5/qmessagelogger.html">QMessageLogger</a>(__FILE__, __LINE__, <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_FUNC_INFO">Q_FUNC_INFO</a>).<a href="http://doc.qt.io/qt-5/qmessagelogger.html#debug-3">debug</a>() for debug builds, and <a href="http://doc.qt.io/qt-5/qmessagelogger.html">QMessageLogger</a>(0, 0, 0).<a href="http://doc.qt.io/qt-5/qmessagelogger.html#debug-3">debug</a>() for release builds.</p>
/// <p>One example of direct use is to forward errors that stem from a scripting language, e.g. QML:</p>
/// <pre class="cpp">
///
/// &#32;     <span class="type">void</span> statusChanged(<span class="type">QDeclarativeComponent</span><span class="operator">::</span>Status status) {
/// &#32;   &#32;     <span class="keyword">if</span> (status <span class="operator">=</span><span class="operator">=</span> <span class="type">QDeclarativeComponent</span><span class="operator">::</span>Error) {
/// &#32;   &#32;   &#32;     foreach (<span class="keyword">const</span> <span class="type">QDeclarativeError</span> <span class="operator">&amp;</span>error<span class="operator">,</span> component<span class="operator">-</span><span class="operator">&gt;</span>errors()) {
/// &#32;   &#32;   &#32;   &#32;     <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span> file <span class="operator">=</span> error<span class="operator">.</span>url()<span class="operator">.</span>toEncoded();
/// &#32;   &#32;   &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qmessagelogger.html#QMessageLogger">QMessageLogger</a></span>(file<span class="operator">.</span>constData()<span class="operator">,</span> error<span class="operator">.</span>line()<span class="operator">,</span> <span class="number">0</span>)<span class="operator">.</span>debug() <span class="operator">&lt;</span><span class="operator">&lt;</span> error<span class="operator">.</span>description();
/// &#32;   &#32;   &#32;     }
/// &#32;   &#32;     }
/// &#32;     }
///
/// </pre></div>
#[repr(C)]
pub struct QMessageLogger {
    _unused: u8,
}
impl QMessageLogger {
    /// <p>Logs a critical message using a <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> stream</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug QMessageLogger::critical() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmessagelogger.html#critical-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Logs a critical message using a <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> stream</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtglobal.html#qCritical">qCritical</a>() and <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a>.</p></div>
    pub unsafe fn critical_0a(&self) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMessageLogger_critical3(
            self as *const crate::QMessageLogger,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Logs a critical message into category <i>cat</i> using a <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug QMessageLogger::critical(const QLoggingCategory& cat) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmessagelogger.html#critical-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Logs a critical message into category <i>cat</i> using a <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> stream.</p>
    /// <p>This function was introduced in  Qt 5.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qloggingcategory.html#qCCritical">qCCritical</a>() and <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a>.</p></div>
    pub unsafe fn critical_1a(
        &self,
        cat: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLoggingCategory>>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMessageLogger_critical4(
            self as *const crate::QMessageLogger,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLoggingCategory>>::cast_into(cat)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Logs a debug message using a <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> stream</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug QMessageLogger::debug() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmessagelogger.html#debug-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Logs a debug message using a <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> stream</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() and <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a>.</p></div>
    pub unsafe fn debug_0a(&self) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMessageLogger_debug3(
            self as *const crate::QMessageLogger,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Logs a debug message into category <i>cat</i> using a <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug QMessageLogger::debug(const QLoggingCategory& cat) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmessagelogger.html#debug-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Logs a debug message into category <i>cat</i> using a <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> stream.</p>
    /// <p>This function was introduced in  Qt 5.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qloggingcategory.html#qCDebug">qCDebug</a>() and <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a>.</p></div>
    pub unsafe fn debug_1a(
        &self,
        cat: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLoggingCategory>>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMessageLogger_debug4(
            self as *const crate::QMessageLogger,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLoggingCategory>>::cast_into(cat)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Logs an informational message using a <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug QMessageLogger::info() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmessagelogger.html#info-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Logs an informational message using a <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> stream.</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtglobal.html#qInfo">qInfo</a>() and <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a>.</p></div>
    pub unsafe fn info_0a(&self) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMessageLogger_info3(
            self as *const crate::QMessageLogger,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Logs an informational message into the category <i>cat</i> using a <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug QMessageLogger::info(const QLoggingCategory& cat) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmessagelogger.html#info-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Logs an informational message into the category <i>cat</i> using a <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> stream.</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qloggingcategory.html#qCInfo">qCInfo</a>() and <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a>.</p></div>
    pub unsafe fn info_1a(
        &self,
        cat: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLoggingCategory>>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMessageLogger_info4(
            self as *const crate::QMessageLogger,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLoggingCategory>>::cast_into(cat)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a default <a href="http://doc.qt.io/qt-5/qmessagelogger.html">QMessageLogger</a>. See the other constructors to specify context information.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMessageLogger::QMessageLogger()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmessagelogger.html#QMessageLogger">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a default <a href="http://doc.qt.io/qt-5/qmessagelogger.html">QMessageLogger</a>. See the other constructors to specify context information.</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QMessageLogger> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMessageLogger_QMessageLogger();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qmessagelogger.html">QMessageLogger</a> to record log messages for <i>file</i> at <i>line</i> in <i>function</i>. The is equivalent to <a href="http://doc.qt.io/qt-5/qmessagelogger.html">QMessageLogger</a>(file, line, function, "default")</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMessageLogger::QMessageLogger(const char* file, int line, const char* function)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmessagelogger.html#QMessageLogger-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qmessagelogger.html">QMessageLogger</a> to record log messages for <i>file</i> at <i>line</i> in <i>function</i>. The is equivalent to <a href="http://doc.qt.io/qt-5/qmessagelogger.html">QMessageLogger</a>(file, line, function, "default")</p></div>
    pub unsafe fn new_3a(
        file: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        line: ::std::os::raw::c_int,
        function: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QMessageLogger> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMessageLogger_QMessageLogger1(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(file)
                .as_raw_ptr(),
            line,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(function)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qmessagelogger.html">QMessageLogger</a> to record <i>category</i> messages for <i>file</i> at <i>line</i> in <i>function</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMessageLogger::QMessageLogger(const char* file, int line, const char* function, const char* category)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmessagelogger.html#QMessageLogger-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qmessagelogger.html">QMessageLogger</a> to record <i>category</i> messages for <i>file</i> at <i>line</i> in <i>function</i>.</p></div>
    pub unsafe fn new_4a(
        file: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        line: ::std::os::raw::c_int,
        function: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        category: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QMessageLogger> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMessageLogger_QMessageLogger2(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(file)
                .as_raw_ptr(),
            line,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(function)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(category)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Logs a warning message using a <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> stream</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug QMessageLogger::warning() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmessagelogger.html#warning-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Logs a warning message using a <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> stream</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtglobal.html#qWarning">qWarning</a>() and <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a>.</p></div>
    pub unsafe fn warning_0a(&self) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMessageLogger_warning3(
            self as *const crate::QMessageLogger,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Logs a warning message into category <i>cat</i> using a <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug QMessageLogger::warning(const QLoggingCategory& cat) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmessagelogger.html#warning-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Logs a warning message into category <i>cat</i> using a <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> stream.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qloggingcategory.html#qCWarning">qCWarning</a>() and <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a>.</p></div>
    pub unsafe fn warning_1a(
        &self,
        cat: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLoggingCategory>>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMessageLogger_warning4(
            self as *const crate::QMessageLogger,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLoggingCategory>>::cast_into(cat)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>Qt's predefined <a href="http://doc.qt.io/qt-5/qcolor.html">QColor</a> objects:</p>
///
/// C++ enum: <span style='color: green;'>```Qt::GlobalColor```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#GlobalColor-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Qt's predefined <a href="http://doc.qt.io/qt-5/qcolor.html">QColor</a> objects:</p>
///
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcolor.html">QColor</a>.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct GlobalColor(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for GlobalColor {
    fn from(value: ::std::os::raw::c_int) -> Self {
        GlobalColor(value)
    }
}

impl From<GlobalColor> for ::std::os::raw::c_int {
    fn from(value: GlobalColor) -> Self {
        value.0
    }
}

impl GlobalColor {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl GlobalColor {
    /// 0 pixel value (for bitmaps) (C++ enum variant: <span style='color: green;'>```color0 = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const Color0: crate::GlobalColor = crate::GlobalColor(0);
    /// 1 pixel value (for bitmaps) (C++ enum variant: <span style='color: green;'>```color1 = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const Color1: crate::GlobalColor = crate::GlobalColor(1);
    /// <span id="color-black">Black (#000000) </span> (C++ enum variant: <span style='color: green;'>```black = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const Black: crate::GlobalColor = crate::GlobalColor(2);
    /// <span id="color-white">White (#ffffff) </span> (C++ enum variant: <span style='color: green;'>```white = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const White: crate::GlobalColor = crate::GlobalColor(3);
    /// <span id="color-darkGray">Dark gray (#808080) </span> (C++ enum variant: <span style='color: green;'>```darkGray = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const DarkGray: crate::GlobalColor = crate::GlobalColor(4);
    /// <span id="color-gray">Gray (#a0a0a4) </span> (C++ enum variant: <span style='color: green;'>```gray = 5```</span>)
    #[allow(non_upper_case_globals)]
    pub const Gray: crate::GlobalColor = crate::GlobalColor(5);
    /// <span id="color-lightGray">Light gray (#c0c0c0) </span> (C++ enum variant: <span style='color: green;'>```lightGray = 6```</span>)
    #[allow(non_upper_case_globals)]
    pub const LightGray: crate::GlobalColor = crate::GlobalColor(6);
    /// <span id="color-red">Red (#ff0000) </span> (C++ enum variant: <span style='color: green;'>```red = 7```</span>)
    #[allow(non_upper_case_globals)]
    pub const Red: crate::GlobalColor = crate::GlobalColor(7);
    /// <span id="color-green">Green (#00ff00) </span> (C++ enum variant: <span style='color: green;'>```green = 8```</span>)
    #[allow(non_upper_case_globals)]
    pub const Green: crate::GlobalColor = crate::GlobalColor(8);
    /// <span id="color-blue">Blue (#0000ff) </span> (C++ enum variant: <span style='color: green;'>```blue = 9```</span>)
    #[allow(non_upper_case_globals)]
    pub const Blue: crate::GlobalColor = crate::GlobalColor(9);
    /// <span id="color-cyan">Cyan (#00ffff) </span> (C++ enum variant: <span style='color: green;'>```cyan = 10```</span>)
    #[allow(non_upper_case_globals)]
    pub const Cyan: crate::GlobalColor = crate::GlobalColor(10);
    /// <span id="color-magenta">Magenta (#ff00ff) </span> (C++ enum variant: <span style='color: green;'>```magenta = 11```</span>)
    #[allow(non_upper_case_globals)]
    pub const Magenta: crate::GlobalColor = crate::GlobalColor(11);
    /// <span id="color-yellow">Yellow (#ffff00) </span> (C++ enum variant: <span style='color: green;'>```yellow = 12```</span>)
    #[allow(non_upper_case_globals)]
    pub const Yellow: crate::GlobalColor = crate::GlobalColor(12);
    /// <span id="color-darkRed">Dark red (#800000) </span> (C++ enum variant: <span style='color: green;'>```darkRed = 13```</span>)
    #[allow(non_upper_case_globals)]
    pub const DarkRed: crate::GlobalColor = crate::GlobalColor(13);
    /// <span id="color-darkGreen">Dark green (#008000) </span> (C++ enum variant: <span style='color: green;'>```darkGreen = 14```</span>)
    #[allow(non_upper_case_globals)]
    pub const DarkGreen: crate::GlobalColor = crate::GlobalColor(14);
    /// <span id="color-darkBlue">Dark blue (#000080) </span> (C++ enum variant: <span style='color: green;'>```darkBlue = 15```</span>)
    #[allow(non_upper_case_globals)]
    pub const DarkBlue: crate::GlobalColor = crate::GlobalColor(15);
    /// <span id="color-darkCyan">Dark cyan (#008080) </span> (C++ enum variant: <span style='color: green;'>```darkCyan = 16```</span>)
    #[allow(non_upper_case_globals)]
    pub const DarkCyan: crate::GlobalColor = crate::GlobalColor(16);
    /// <span id="color-darkMagenta">Dark magenta (#800080) </span> (C++ enum variant: <span style='color: green;'>```darkMagenta = 17```</span>)
    #[allow(non_upper_case_globals)]
    pub const DarkMagenta: crate::GlobalColor = crate::GlobalColor(17);
    /// <span id="color-darkYellow">Dark yellow (#808000) </span> (C++ enum variant: <span style='color: green;'>```darkYellow = 18```</span>)
    #[allow(non_upper_case_globals)]
    pub const DarkYellow: crate::GlobalColor = crate::GlobalColor(18);
    /// a transparent black value (i.e., <a href="http://doc.qt.io/qt-5/qcolor.html">QColor</a>(0, 0, 0, 0)) (C++ enum variant: <span style='color: green;'>```transparent = 19```</span>)
    #[allow(non_upper_case_globals)]
    pub const Transparent: crate::GlobalColor = crate::GlobalColor(19);
}

/// <p>This enum describes the modifier keys.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::KeyboardModifier```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#KeyboardModifier-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the modifier keys.</p>
///
/// <p><b>Note: </b>On <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, the <code>ControlModifier</code> value corresponds to the Command keys on the keyboard, and the <code>MetaModifier</code> value corresponds to the Control keys. The <code>KeypadModifier</code> value will also be set when an arrow key is pressed as the arrow keys are considered part of the keypad.</p><p><b>Note: </b>On Windows Keyboards, Qt::MetaModifier and <a href="http://doc.qt.io/qt-5/qt.html#Key-enum">Qt::Key_Meta</a> are mapped to the Windows key.</p><p>The KeyboardModifiers type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;KeyboardModifier&gt;. It stores an OR combination of KeyboardModifier values.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qt.html#MouseButton-enum">MouseButton</a> and <a href="http://doc.qt.io/qt-5/qt.html#Modifier-enum">Modifier</a>.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct KeyboardModifier(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for KeyboardModifier {
    fn from(value: ::std::os::raw::c_int) -> Self {
        KeyboardModifier(value)
    }
}

impl From<KeyboardModifier> for ::std::os::raw::c_int {
    fn from(value: KeyboardModifier) -> Self {
        value.0
    }
}

impl KeyboardModifier {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl KeyboardModifier {
    /// No modifier key is pressed. (C++ enum variant: <span style='color: green;'>```NoModifier = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const NoModifier: crate::KeyboardModifier = crate::KeyboardModifier(0);
    /// A Shift key on the keyboard is pressed. (C++ enum variant: <span style='color: green;'>```ShiftModifier = 33554432```</span>)
    #[allow(non_upper_case_globals)]
    pub const ShiftModifier: crate::KeyboardModifier = crate::KeyboardModifier(33554432);
    /// A Ctrl key on the keyboard is pressed. (C++ enum variant: <span style='color: green;'>```ControlModifier = 67108864```</span>)
    #[allow(non_upper_case_globals)]
    pub const ControlModifier: crate::KeyboardModifier = crate::KeyboardModifier(67108864);
    /// An Alt key on the keyboard is pressed. (C++ enum variant: <span style='color: green;'>```AltModifier = 134217728```</span>)
    #[allow(non_upper_case_globals)]
    pub const AltModifier: crate::KeyboardModifier = crate::KeyboardModifier(134217728);
    /// A Meta key on the keyboard is pressed. (C++ enum variant: <span style='color: green;'>```MetaModifier = 268435456```</span>)
    #[allow(non_upper_case_globals)]
    pub const MetaModifier: crate::KeyboardModifier = crate::KeyboardModifier(268435456);
    /// A keypad button is pressed. (C++ enum variant: <span style='color: green;'>```KeypadModifier = 536870912```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeypadModifier: crate::KeyboardModifier = crate::KeyboardModifier(536870912);
    /// X11 only. A Mode_switch key on the keyboard is pressed. (C++ enum variant: <span style='color: green;'>```GroupSwitchModifier = 1073741824```</span>)
    #[allow(non_upper_case_globals)]
    pub const GroupSwitchModifier: crate::KeyboardModifier = crate::KeyboardModifier(1073741824);
    /// C++ enum variant: <span style='color: green;'>```KeyboardModifierMask = -33554432```</span>
    #[allow(non_upper_case_globals)]
    pub const KeyboardModifierMask: crate::KeyboardModifier = crate::KeyboardModifier(-33554432);
}

/// <p>This enum provides shorter names for the keyboard modifier keys supported by Qt.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::Modifier```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#Modifier-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum provides shorter names for the keyboard modifier keys supported by Qt.</p>
/// <p><b>Note: </b>On <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, the <code>CTRL</code> value corresponds to the Command keys on the keyboard, and the <code>META</code> value corresponds to the Control keys.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qt.html#KeyboardModifier-enum">KeyboardModifier</a> and <a href="http://doc.qt.io/qt-5/qt.html#MouseButton-enum">MouseButton</a>.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct Modifier(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for Modifier {
    fn from(value: ::std::os::raw::c_int) -> Self {
        Modifier(value)
    }
}

impl From<Modifier> for ::std::os::raw::c_int {
    fn from(value: Modifier) -> Self {
        value.0
    }
}

impl Modifier {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl Modifier {
    /// The Meta keys. (C++ enum variant: <span style='color: green;'>```META = 268435456```</span>)
    #[allow(non_upper_case_globals)]
    pub const META: crate::Modifier = crate::Modifier(268435456);
    /// The Shift keys provided on all standard keyboards. (C++ enum variant: <span style='color: green;'>```SHIFT = 33554432```</span>)
    #[allow(non_upper_case_globals)]
    pub const SHIFT: crate::Modifier = crate::Modifier(33554432);
    /// The Ctrl keys. (C++ enum variant: <span style='color: green;'>```CTRL = 67108864```</span>)
    #[allow(non_upper_case_globals)]
    pub const CTRL: crate::Modifier = crate::Modifier(67108864);
    /// The normal Alt keys, but not keys like AltGr. (C++ enum variant: <span style='color: green;'>```ALT = 134217728```</span>)
    #[allow(non_upper_case_globals)]
    pub const ALT: crate::Modifier = crate::Modifier(134217728);
    /// C++ enum variant: <span style='color: green;'>```MODIFIER_MASK = -33554432```</span>
    #[allow(non_upper_case_globals)]
    pub const MODIFIERMASK: crate::Modifier = crate::Modifier(-33554432);
    /// The shortcut is specified as a Unicode code point, not as a Qt Key. (C++ enum variant: <span style='color: green;'>```UNICODE_ACCEL = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const UNICODEACCEL: crate::Modifier = crate::Modifier(0);
}

/// <p>This enum type describes the different mouse buttons.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::MouseButton```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#MouseButton-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type describes the different mouse buttons.</p>
///
/// <p><b>Note: </b>Some models of multi-button mice are pre-configured with high-numbered Buttons emulating keyboard sequences, for use in specific games. In order for these Buttons to be seen as actual 'Mouse Buttons', the device must be re-configured (using the vendor's configuration tool).</p><p>The MouseButtons type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;MouseButton&gt;. It stores an OR combination of MouseButton values.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qt.html#KeyboardModifier-enum">KeyboardModifier</a> and <a href="http://doc.qt.io/qt-5/qt.html#Modifier-enum">Modifier</a>.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct MouseButton(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for MouseButton {
    fn from(value: ::std::os::raw::c_int) -> Self {
        MouseButton(value)
    }
}

impl From<MouseButton> for ::std::os::raw::c_int {
    fn from(value: MouseButton) -> Self {
        value.0
    }
}

impl MouseButton {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl MouseButton {
    /// The button state does not refer to any button (see <a href="http://doc.qt.io/qt-5/qmouseevent.html#button">QMouseEvent::button</a>()). (C++ enum variant: <span style='color: green;'>```NoButton = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const NoButton: crate::MouseButton = crate::MouseButton(0);
    /// The left button is pressed, or an event refers to the left button. (The left button may be the right button on left-handed mice.) (C++ enum variant: <span style='color: green;'>```LeftButton = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const LeftButton: crate::MouseButton = crate::MouseButton(1);
    /// The right button. (C++ enum variant: <span style='color: green;'>```RightButton = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const RightButton: crate::MouseButton = crate::MouseButton(2);
    /// The middle button. (C++ enum variant: <span style='color: green;'>```MidButton = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const MidButton: crate::MouseButton = crate::MouseButton(4);
    /// The middle button. (C++ enum variant: <span style='color: green;'>```MiddleButton = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const MiddleButton: crate::MouseButton = crate::MouseButton(4);
    /// The 'Back' button. (Typically present on the 'thumb' side of a mouse with extra buttons. This is NOT the tilt wheel.) (C++ enum variant: <span style='color: green;'>```BackButton = 8```</span>)
    #[allow(non_upper_case_globals)]
    pub const BackButton: crate::MouseButton = crate::MouseButton(8);
    /// The 'Back' Button. (C++ enum variant: <span style='color: green;'>```XButton1 = 8```</span>)
    #[allow(non_upper_case_globals)]
    pub const XButton1: crate::MouseButton = crate::MouseButton(8);
    /// The 'Back' Button. (C++ enum variant: <span style='color: green;'>```ExtraButton1 = 8```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExtraButton1: crate::MouseButton = crate::MouseButton(8);
    /// The 'Forward' Button. (Typically present beside the 'Back' button, and also pressed by the thumb.) (C++ enum variant: <span style='color: green;'>```ForwardButton = 16```</span>)
    #[allow(non_upper_case_globals)]
    pub const ForwardButton: crate::MouseButton = crate::MouseButton(16);
    /// The 'Forward Button. (C++ enum variant: <span style='color: green;'>```XButton2 = 16```</span>)
    #[allow(non_upper_case_globals)]
    pub const XButton2: crate::MouseButton = crate::MouseButton(16);
    /// The 'Forward' Button. (C++ enum variant: <span style='color: green;'>```ExtraButton2 = 16```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExtraButton2: crate::MouseButton = crate::MouseButton(16);
    /// The 'Task' Button. (C++ enum variant: <span style='color: green;'>```TaskButton = 32```</span>)
    #[allow(non_upper_case_globals)]
    pub const TaskButton: crate::MouseButton = crate::MouseButton(32);
    /// The 'Task' Button. (C++ enum variant: <span style='color: green;'>```ExtraButton3 = 32```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExtraButton3: crate::MouseButton = crate::MouseButton(32);
    /// The 7th non-wheel Mouse Button. (C++ enum variant: <span style='color: green;'>```ExtraButton4 = 64```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExtraButton4: crate::MouseButton = crate::MouseButton(64);
    /// The 8th non-wheel Mouse Button. (C++ enum variant: <span style='color: green;'>```ExtraButton5 = 128```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExtraButton5: crate::MouseButton = crate::MouseButton(128);
    /// The 9th non-wheel Mouse Button. (C++ enum variant: <span style='color: green;'>```ExtraButton6 = 256```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExtraButton6: crate::MouseButton = crate::MouseButton(256);
    /// The 10th non-wheel Mouse Button. (C++ enum variant: <span style='color: green;'>```ExtraButton7 = 512```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExtraButton7: crate::MouseButton = crate::MouseButton(512);
    /// The 11th non-wheel Mouse Button. (C++ enum variant: <span style='color: green;'>```ExtraButton8 = 1024```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExtraButton8: crate::MouseButton = crate::MouseButton(1024);
    /// The 12th non-wheel Mouse Button. (C++ enum variant: <span style='color: green;'>```ExtraButton9 = 2048```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExtraButton9: crate::MouseButton = crate::MouseButton(2048);
    /// The 13th non-wheel Mouse Button. (C++ enum variant: <span style='color: green;'>```ExtraButton10 = 4096```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExtraButton10: crate::MouseButton = crate::MouseButton(4096);
    /// The 14th non-wheel Mouse Button. (C++ enum variant: <span style='color: green;'>```ExtraButton11 = 8192```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExtraButton11: crate::MouseButton = crate::MouseButton(8192);
    /// The 15th non-wheel Mouse Button. (C++ enum variant: <span style='color: green;'>```ExtraButton12 = 16384```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExtraButton12: crate::MouseButton = crate::MouseButton(16384);
    /// The 16th non-wheel Mouse Button. (C++ enum variant: <span style='color: green;'>```ExtraButton13 = 32768```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExtraButton13: crate::MouseButton = crate::MouseButton(32768);
    /// The 17th non-wheel Mouse Button. (C++ enum variant: <span style='color: green;'>```ExtraButton14 = 65536```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExtraButton14: crate::MouseButton = crate::MouseButton(65536);
    /// The 18th non-wheel Mouse Button. (C++ enum variant: <span style='color: green;'>```ExtraButton15 = 131072```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExtraButton15: crate::MouseButton = crate::MouseButton(131072);
    /// The 19th non-wheel Mouse Button. (C++ enum variant: <span style='color: green;'>```ExtraButton16 = 262144```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExtraButton16: crate::MouseButton = crate::MouseButton(262144);
    /// The 20th non-wheel Mouse Button. (C++ enum variant: <span style='color: green;'>```ExtraButton17 = 524288```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExtraButton17: crate::MouseButton = crate::MouseButton(524288);
    /// The 21st non-wheel Mouse Button. (C++ enum variant: <span style='color: green;'>```ExtraButton18 = 1048576```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExtraButton18: crate::MouseButton = crate::MouseButton(1048576);
    /// The 22nd non-wheel Mouse Button. (C++ enum variant: <span style='color: green;'>```ExtraButton19 = 2097152```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExtraButton19: crate::MouseButton = crate::MouseButton(2097152);
    /// The 23rd non-wheel Mouse Button. (C++ enum variant: <span style='color: green;'>```ExtraButton20 = 4194304```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExtraButton20: crate::MouseButton = crate::MouseButton(4194304);
    /// The 24th non-wheel Mouse Button. (C++ enum variant: <span style='color: green;'>```ExtraButton21 = 8388608```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExtraButton21: crate::MouseButton = crate::MouseButton(8388608);
    /// The 25th non-wheel Mouse Button. (C++ enum variant: <span style='color: green;'>```ExtraButton22 = 16777216```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExtraButton22: crate::MouseButton = crate::MouseButton(16777216);
    /// The 26th non-wheel Mouse Button. (C++ enum variant: <span style='color: green;'>```ExtraButton23 = 33554432```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExtraButton23: crate::MouseButton = crate::MouseButton(33554432);
    /// The 27th non-wheel Mouse Button. (C++ enum variant: <span style='color: green;'>```ExtraButton24 = 67108864```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExtraButton24: crate::MouseButton = crate::MouseButton(67108864);
    /// This value corresponds to a mask of all possible mouse buttons. Use to set the 'acceptedButtons' property of a <a href="http://doc.qt.io/qt-5/qml-qtquick-mousearea.html">MouseArea</a> to accept ALL mouse buttons. (C++ enum variant: <span style='color: green;'>```AllButtons = 134217727```</span>)
    #[allow(non_upper_case_globals)]
    pub const AllButtons: crate::MouseButton = crate::MouseButton(134217727);
    /// C++ enum variant: <span style='color: green;'>```MaxMouseButton = 67108864```</span>
    #[allow(non_upper_case_globals)]
    pub const MaxMouseButton: crate::MouseButton = crate::MouseButton(67108864);
    /// C++ enum variant: <span style='color: green;'>```MouseButtonMask = -1```</span>
    #[allow(non_upper_case_globals)]
    pub const MouseButtonMask: crate::MouseButton = crate::MouseButton(-1);
}

/// <p>This type is used to signify an object's orientation.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::Orientation```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#Orientation-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This type is used to signify an object's orientation.</p>
///
/// <p>Orientation is used with <a href="http://doc.qt.io/qt-5/qscrollbar.html">QScrollBar</a> for example.</p>
/// <p>The Orientations type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;Orientation&gt;. It stores an OR combination of Orientation values.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct Orientation(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for Orientation {
    fn from(value: ::std::os::raw::c_int) -> Self {
        Orientation(value)
    }
}

impl From<Orientation> for ::std::os::raw::c_int {
    fn from(value: Orientation) -> Self {
        value.0
    }
}

impl Orientation {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl Orientation {
    /// C++ enum variant: <span style='color: green;'>```Horizontal = 1```</span>
    #[allow(non_upper_case_globals)]
    pub const Horizontal: crate::Orientation = crate::Orientation(1);
    /// C++ enum variant: <span style='color: green;'>```Vertical = 2```</span>
    #[allow(non_upper_case_globals)]
    pub const Vertical: crate::Orientation = crate::Orientation(2);
}

/// <p>This enum type defines the various policies a widget can have with respect to acquiring keyboard focus.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::FocusPolicy```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#FocusPolicy-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type defines the various policies a widget can have with respect to acquiring keyboard focus.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct FocusPolicy(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for FocusPolicy {
    fn from(value: ::std::os::raw::c_int) -> Self {
        FocusPolicy(value)
    }
}

impl From<FocusPolicy> for ::std::os::raw::c_int {
    fn from(value: FocusPolicy) -> Self {
        value.0
    }
}

impl FocusPolicy {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl FocusPolicy {
    /// the widget does not accept focus. (C++ enum variant: <span style='color: green;'>```NoFocus = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const NoFocus: crate::FocusPolicy = crate::FocusPolicy(0);
    /// the widget accepts focus by tabbing. (C++ enum variant: <span style='color: green;'>```TabFocus = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const TabFocus: crate::FocusPolicy = crate::FocusPolicy(1);
    /// the widget accepts focus by clicking. (C++ enum variant: <span style='color: green;'>```ClickFocus = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const ClickFocus: crate::FocusPolicy = crate::FocusPolicy(2);
    /// the widget accepts focus by both tabbing and clicking. On <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> this will also be indicate that the widget accepts tab focus when in 'Text/List focus mode'. (C++ enum variant: <span style='color: green;'>```StrongFocus = 11```</span>)
    #[allow(non_upper_case_globals)]
    pub const StrongFocus: crate::FocusPolicy = crate::FocusPolicy(11);
    /// like Qt::StrongFocus plus the widget accepts focus by using the mouse wheel. (C++ enum variant: <span style='color: green;'>```WheelFocus = 15```</span>)
    #[allow(non_upper_case_globals)]
    pub const WheelFocus: crate::FocusPolicy = crate::FocusPolicy(15);
}

/// <p>This enum type provides different focus behaviors for tab navigation.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::TabFocusBehavior```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#TabFocusBehavior-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type provides different focus behaviors for tab navigation.</p>
///
/// <p>This enum was introduced or modified in  Qt 5.5.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct TabFocusBehavior(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for TabFocusBehavior {
    fn from(value: ::std::os::raw::c_int) -> Self {
        TabFocusBehavior(value)
    }
}

impl From<TabFocusBehavior> for ::std::os::raw::c_int {
    fn from(value: TabFocusBehavior) -> Self {
        value.0
    }
}

impl TabFocusBehavior {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl TabFocusBehavior {
    /// iterate nothing. (C++ enum variant: <span style='color: green;'>```NoTabFocus = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const NoTabFocus: crate::TabFocusBehavior = crate::TabFocusBehavior(0);
    /// iterate text controls and widgets. (C++ enum variant: <span style='color: green;'>```TabFocusTextControls = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const TabFocusTextControls: crate::TabFocusBehavior = crate::TabFocusBehavior(1);
    /// iterate list controls and widgets. (C++ enum variant: <span style='color: green;'>```TabFocusListControls = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const TabFocusListControls: crate::TabFocusBehavior = crate::TabFocusBehavior(2);
    /// iterate all controls and widgets. (C++ enum variant: <span style='color: green;'>```TabFocusAllControls = 255```</span>)
    #[allow(non_upper_case_globals)]
    pub const TabFocusAllControls: crate::TabFocusBehavior = crate::TabFocusBehavior(255);
}

/// <p>This enum describes how the items in a widget are sorted.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::SortOrder```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#SortOrder-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes how the items in a widget are sorted.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct SortOrder(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for SortOrder {
    fn from(value: ::std::os::raw::c_int) -> Self {
        SortOrder(value)
    }
}

impl From<SortOrder> for ::std::os::raw::c_int {
    fn from(value: SortOrder) -> Self {
        value.0
    }
}

impl SortOrder {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl SortOrder {
    /// The items are sorted ascending e.g. starts with 'AAA' ends with 'ZZZ' in Latin-1 locales (C++ enum variant: <span style='color: green;'>```AscendingOrder = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const AscendingOrder: crate::SortOrder = crate::SortOrder(0);
    /// The items are sorted descending e.g. starts with 'ZZZ' ends with 'AAA' in Latin-1 locales (C++ enum variant: <span style='color: green;'>```DescendingOrder = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const DescendingOrder: crate::SortOrder = crate::SortOrder(1);
}

/// <p>This enum describes how to repeat or stretch the parts of an image when drawing.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::TileRule```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#TileRule-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes how to repeat or stretch the parts of an image when drawing.</p>
///
/// <p>This enum was introduced or modified in  Qt 4.6.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct TileRule(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for TileRule {
    fn from(value: ::std::os::raw::c_int) -> Self {
        TileRule(value)
    }
}

impl From<TileRule> for ::std::os::raw::c_int {
    fn from(value: TileRule) -> Self {
        value.0
    }
}

impl TileRule {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl TileRule {
    /// Scale the image to fit to the available area. (C++ enum variant: <span style='color: green;'>```StretchTile = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const StretchTile: crate::TileRule = crate::TileRule(0);
    /// Repeat the image until there is no more space. May crop the last image. (C++ enum variant: <span style='color: green;'>```RepeatTile = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const RepeatTile: crate::TileRule = crate::TileRule(1);
    /// Similar to Repeat, but scales the image down to ensure that the last tile is not cropped. (C++ enum variant: <span style='color: green;'>```RoundTile = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const RoundTile: crate::TileRule = crate::TileRule(2);
}

/// <p>This enum type is used to describe alignment. It contains horizontal and vertical flags that can be combined to produce the required effect.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::AlignmentFlag```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#AlignmentFlag-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type is used to describe alignment. It contains horizontal and vertical flags that can be combined to produce the required effect.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qt.html#TextElideMode-enum">TextElideMode</a> enum can also be used in many situations to fine-tune the appearance of aligned text.</p>
/// <p>The horizontal flags are:</p>
///
/// <p>The vertical flags are:</p>
///
/// <p>You can use only one of the horizontal flags at a time. There is one two-dimensional flag:</p>
///
/// <p>You can use at most one horizontal and one vertical flag at a time. Qt::AlignCenter counts as both horizontal and vertical.</p>
/// <p>Three enum values are useful in applications that can be run in right-to-left mode:</p>
///
/// <p>Masks:</p>
///
/// <p>Conflicting combinations of flags have undefined meanings.</p>
/// <p>The Alignment type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;AlignmentFlag&gt;. It stores an OR combination of AlignmentFlag values.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct AlignmentFlag(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for AlignmentFlag {
    fn from(value: ::std::os::raw::c_int) -> Self {
        AlignmentFlag(value)
    }
}

impl From<AlignmentFlag> for ::std::os::raw::c_int {
    fn from(value: AlignmentFlag) -> Self {
        value.0
    }
}

impl AlignmentFlag {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl AlignmentFlag {
    /// Aligns with the left edge. (C++ enum variant: <span style='color: green;'>```AlignLeft = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const AlignLeft: crate::AlignmentFlag = crate::AlignmentFlag(1);
    /// Synonym for Qt::AlignLeft. (C++ enum variant: <span style='color: green;'>```AlignLeading = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const AlignLeading: crate::AlignmentFlag = crate::AlignmentFlag(1);
    /// Aligns with the right edge. (C++ enum variant: <span style='color: green;'>```AlignRight = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const AlignRight: crate::AlignmentFlag = crate::AlignmentFlag(2);
    /// Synonym for Qt::AlignRight. (C++ enum variant: <span style='color: green;'>```AlignTrailing = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const AlignTrailing: crate::AlignmentFlag = crate::AlignmentFlag(2);
    /// Centers horizontally in the available space. (C++ enum variant: <span style='color: green;'>```AlignHCenter = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const AlignHCenter: crate::AlignmentFlag = crate::AlignmentFlag(4);
    /// Justifies the text in the available space. (C++ enum variant: <span style='color: green;'>```AlignJustify = 8```</span>)
    #[allow(non_upper_case_globals)]
    pub const AlignJustify: crate::AlignmentFlag = crate::AlignmentFlag(8);
    /// If the widget's layout direction is <a href="http://doc.qt.io/qt-5/qt.html#LayoutDirection-enum">Qt::RightToLeft</a> (instead of <a href="http://doc.qt.io/qt-5/qt.html#LayoutDirection-enum">Qt::LeftToRight</a>, the default), Qt::AlignLeft refers to the <i>right</i> edge and Qt::AlignRight to the <i>left</i> edge. This is normally the desired behavior. If you want Qt::AlignLeft to always mean "left" and Qt::AlignRight to always mean "right", combine the flag with Qt::AlignAbsolute. (C++ enum variant: <span style='color: green;'>```AlignAbsolute = 16```</span>)
    #[allow(non_upper_case_globals)]
    pub const AlignAbsolute: crate::AlignmentFlag = crate::AlignmentFlag(16);
    /// C++ enum variant: <span style='color: green;'>```AlignHorizontal_Mask = 31```</span>
    #[allow(non_upper_case_globals)]
    pub const AlignHorizontalMask: crate::AlignmentFlag = crate::AlignmentFlag(31);
    /// Aligns with the top. (C++ enum variant: <span style='color: green;'>```AlignTop = 32```</span>)
    #[allow(non_upper_case_globals)]
    pub const AlignTop: crate::AlignmentFlag = crate::AlignmentFlag(32);
    /// Aligns with the bottom. (C++ enum variant: <span style='color: green;'>```AlignBottom = 64```</span>)
    #[allow(non_upper_case_globals)]
    pub const AlignBottom: crate::AlignmentFlag = crate::AlignmentFlag(64);
    /// Centers vertically in the available space. (C++ enum variant: <span style='color: green;'>```AlignVCenter = 128```</span>)
    #[allow(non_upper_case_globals)]
    pub const AlignVCenter: crate::AlignmentFlag = crate::AlignmentFlag(128);
    /// Aligns with the baseline. (C++ enum variant: <span style='color: green;'>```AlignBaseline = 256```</span>)
    #[allow(non_upper_case_globals)]
    pub const AlignBaseline: crate::AlignmentFlag = crate::AlignmentFlag(256);
    /// C++ enum variant: <span style='color: green;'>```AlignVertical_Mask = 480```</span>
    #[allow(non_upper_case_globals)]
    pub const AlignVerticalMask: crate::AlignmentFlag = crate::AlignmentFlag(480);
    /// Centers in both dimensions. (C++ enum variant: <span style='color: green;'>```AlignCenter = 132```</span>)
    #[allow(non_upper_case_globals)]
    pub const AlignCenter: crate::AlignmentFlag = crate::AlignmentFlag(132);
}

/// <p>This enum type is used to define some modifier flags. Some of these flags only make sense in the context of printing:</p>
///
/// C++ enum: <span style='color: green;'>```Qt::TextFlag```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#TextFlag-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type is used to define some modifier flags. Some of these flags only make sense in the context of printing:</p>
///
/// <p>You can use as many modifier flags as you want, except that Qt::TextSingleLine and Qt::TextWordWrap cannot be combined.</p>
/// <p>Flags that are inappropriate for a given use are generally ignored.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct TextFlag(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for TextFlag {
    fn from(value: ::std::os::raw::c_int) -> Self {
        TextFlag(value)
    }
}

impl From<TextFlag> for ::std::os::raw::c_int {
    fn from(value: TextFlag) -> Self {
        value.0
    }
}

impl TextFlag {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl TextFlag {
    /// Treats all whitespace as spaces and prints just one line. (C++ enum variant: <span style='color: green;'>```TextSingleLine = 256```</span>)
    #[allow(non_upper_case_globals)]
    pub const TextSingleLine: crate::TextFlag = crate::TextFlag(256);
    /// If it's impossible to stay within the given bounds, it prints outside. (C++ enum variant: <span style='color: green;'>```TextDontClip = 512```</span>)
    #[allow(non_upper_case_globals)]
    pub const TextDontClip: crate::TextFlag = crate::TextFlag(512);
    /// Makes the U+0009 (ASCII tab) character move to the next tab stop. (C++ enum variant: <span style='color: green;'>```TextExpandTabs = 1024```</span>)
    #[allow(non_upper_case_globals)]
    pub const TextExpandTabs: crate::TextFlag = crate::TextFlag(1024);
    /// Displays the string "&amp;P" as <u>P</u> For an ampersand, use "&amp;&amp;". (C++ enum variant: <span style='color: green;'>```TextShowMnemonic = 2048```</span>)
    #[allow(non_upper_case_globals)]
    pub const TextShowMnemonic: crate::TextFlag = crate::TextFlag(2048);
    /// Breaks lines at appropriate points, e.g. at word boundaries. (C++ enum variant: <span style='color: green;'>```TextWordWrap = 4096```</span>)
    #[allow(non_upper_case_globals)]
    pub const TextWordWrap: crate::TextFlag = crate::TextFlag(4096);
    /// Breaks lines anywhere, even within words. (C++ enum variant: <span style='color: green;'>```TextWrapAnywhere = 8192```</span>)
    #[allow(non_upper_case_globals)]
    pub const TextWrapAnywhere: crate::TextFlag = crate::TextFlag(8192);
    /// Treat this text as "hidden" and don't print it. (C++ enum variant: <span style='color: green;'>```TextDontPrint = 16384```</span>)
    #[allow(non_upper_case_globals)]
    pub const TextDontPrint: crate::TextFlag = crate::TextFlag(16384);
    /// When this option is set, <a href="http://doc.qt.io/qt-5/qtextline.html#naturalTextWidth">QTextLine::naturalTextWidth</a>() and <a href="http://doc.qt.io/qt-5/qtextline.html#naturalTextRect">QTextLine::naturalTextRect</a>() will return a value that includes the width of trailing spaces in the text; otherwise this width is excluded. (C++ enum variant: <span style='color: green;'>```TextIncludeTrailingSpaces = 134217728```</span>)
    #[allow(non_upper_case_globals)]
    pub const TextIncludeTrailingSpaces: crate::TextFlag = crate::TextFlag(134217728);
    /// Same as Qt::TextShowMnemonic but doesn't draw the underlines. (C++ enum variant: <span style='color: green;'>```TextHideMnemonic = 32768```</span>)
    #[allow(non_upper_case_globals)]
    pub const TextHideMnemonic: crate::TextFlag = crate::TextFlag(32768);
    /// Ensures that text lines are justified. (C++ enum variant: <span style='color: green;'>```TextJustificationForced = 65536```</span>)
    #[allow(non_upper_case_globals)]
    pub const TextJustificationForced: crate::TextFlag = crate::TextFlag(65536);
    /// C++ enum variant: <span style='color: green;'>```TextForceLeftToRight = 131072```</span>
    #[allow(non_upper_case_globals)]
    pub const TextForceLeftToRight: crate::TextFlag = crate::TextFlag(131072);
    /// C++ enum variant: <span style='color: green;'>```TextForceRightToLeft = 262144```</span>
    #[allow(non_upper_case_globals)]
    pub const TextForceRightToLeft: crate::TextFlag = crate::TextFlag(262144);
    /// C++ enum variant: <span style='color: green;'>```TextLongestVariant = 524288```</span>
    #[allow(non_upper_case_globals)]
    pub const TextLongestVariant: crate::TextFlag = crate::TextFlag(524288);
    /// C++ enum variant: <span style='color: green;'>```TextBypassShaping = 1048576```</span>
    #[allow(non_upper_case_globals)]
    pub const TextBypassShaping: crate::TextFlag = crate::TextFlag(1048576);
}

/// <p>This enum specifies where the ellipsis should appear when displaying texts that don't fit:</p>
///
/// C++ enum: <span style='color: green;'>```Qt::TextElideMode```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#TextElideMode-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum specifies where the ellipsis should appear when displaying texts that don't fit:</p>
///
/// <p>Qt::ElideMiddle is normally the most appropriate choice for URLs (e.g., "<a href="http://bugreports.qt.io/browse/QTWEBSITE-13">http://bugreports.qt.../QTWEBSITE-13/</a>"), whereas Qt::ElideRight is appropriate for other strings (e.g., "<a href="http://doc.qt.io/archives/qq/qq09-mac-deployment.html">Deploying Applications on Ma...</a>").</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemview.html#textElideMode-prop">QAbstractItemView::textElideMode</a>, <a href="http://doc.qt.io/qt-5/qfontmetrics.html#elidedText">QFontMetrics::elidedText</a>(), <a href="http://doc.qt.io/qt-5/qt.html#AlignmentFlag-enum">AlignmentFlag</a>, and <a href="http://doc.qt.io/qt-5/qtabbar.html#elideMode-prop">QTabBar::elideMode</a>.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct TextElideMode(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for TextElideMode {
    fn from(value: ::std::os::raw::c_int) -> Self {
        TextElideMode(value)
    }
}

impl From<TextElideMode> for ::std::os::raw::c_int {
    fn from(value: TextElideMode) -> Self {
        value.0
    }
}

impl TextElideMode {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl TextElideMode {
    /// The ellipsis should appear at the beginning of the text. (C++ enum variant: <span style='color: green;'>```ElideLeft = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const ElideLeft: crate::TextElideMode = crate::TextElideMode(0);
    /// The ellipsis should appear at the end of the text. (C++ enum variant: <span style='color: green;'>```ElideRight = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const ElideRight: crate::TextElideMode = crate::TextElideMode(1);
    /// The ellipsis should appear in the middle of the text. (C++ enum variant: <span style='color: green;'>```ElideMiddle = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const ElideMiddle: crate::TextElideMode = crate::TextElideMode(2);
    /// Ellipsis should NOT appear in the text. (C++ enum variant: <span style='color: green;'>```ElideNone = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const ElideNone: crate::TextElideMode = crate::TextElideMode(3);
}

/// <p>This enum describes the types of whitespace mode that are used by the <a href="http://doc.qt.io/qt-5/qtextdocument.html">QTextDocument</a> class to meet the requirements of different kinds of textual information.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::WhiteSpaceMode```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#WhiteSpaceMode-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the types of whitespace mode that are used by the <a href="http://doc.qt.io/qt-5/qtextdocument.html">QTextDocument</a> class to meet the requirements of different kinds of textual information.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct WhiteSpaceMode(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for WhiteSpaceMode {
    fn from(value: ::std::os::raw::c_int) -> Self {
        WhiteSpaceMode(value)
    }
}

impl From<WhiteSpaceMode> for ::std::os::raw::c_int {
    fn from(value: WhiteSpaceMode) -> Self {
        value.0
    }
}

impl WhiteSpaceMode {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl WhiteSpaceMode {
    /// The whitespace mode used to display normal word wrapped text in paragraphs. (C++ enum variant: <span style='color: green;'>```WhiteSpaceNormal = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const WhiteSpaceNormal: crate::WhiteSpaceMode = crate::WhiteSpaceMode(0);
    /// A preformatted text mode in which whitespace is reproduced exactly. (C++ enum variant: <span style='color: green;'>```WhiteSpacePre = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const WhiteSpacePre: crate::WhiteSpaceMode = crate::WhiteSpaceMode(1);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```WhiteSpaceNoWrap = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const WhiteSpaceNoWrap: crate::WhiteSpaceMode = crate::WhiteSpaceMode(2);
    /// C++ enum variant: <span style='color: green;'>```WhiteSpaceModeUndefined = -1```</span>
    #[allow(non_upper_case_globals)]
    pub const WhiteSpaceModeUndefined: crate::WhiteSpaceMode = crate::WhiteSpaceMode(-1);
}

/// <p>This enum contains the types of accuracy that can be used by the <a href="http://doc.qt.io/qt-5/qtextdocument.html">QTextDocument</a> class when testing for mouse clicks on text documents.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::HitTestAccuracy```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#HitTestAccuracy-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum contains the types of accuracy that can be used by the <a href="http://doc.qt.io/qt-5/qtextdocument.html">QTextDocument</a> class when testing for mouse clicks on text documents.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct HitTestAccuracy(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for HitTestAccuracy {
    fn from(value: ::std::os::raw::c_int) -> Self {
        HitTestAccuracy(value)
    }
}

impl From<HitTestAccuracy> for ::std::os::raw::c_int {
    fn from(value: HitTestAccuracy) -> Self {
        value.0
    }
}

impl HitTestAccuracy {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl HitTestAccuracy {
    /// The point at which input occurred must coincide exactly with input-sensitive parts of the document. (C++ enum variant: <span style='color: green;'>```ExactHit = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExactHit: crate::HitTestAccuracy = crate::HitTestAccuracy(0);
    /// The point at which input occurred can lie close to input-sensitive parts of the document. (C++ enum variant: <span style='color: green;'>```FuzzyHit = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const FuzzyHit: crate::HitTestAccuracy = crate::HitTestAccuracy(1);
}

/// <p>This enum type is used to specify various window-system properties for the widget. They are fairly unusual but necessary in a few cases. Some of these flags depend on whether the underlying window manager supports them.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::WindowType```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#WindowType-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type is used to specify various window-system properties for the widget. They are fairly unusual but necessary in a few cases. Some of these flags depend on whether the underlying window manager supports them.</p>
/// <p>The main types are</p>
///
/// <p>There are also a number of flags which you can use to customize the appearance of top-level windows. These have no effect on other windows:</p>
///
/// <p>The <code>CustomizeWindowHint</code> flag is used to enable customization of the window controls. This flag must be set to allow the <code>WindowTitleHint</code>, <code>WindowSystemMenuHint</code>, <code>WindowMinimizeButtonHint</code>, <code>WindowMaximizeButtonHint</code> and <code>WindowCloseButtonHint</code> flags to be changed.</p>
///
/// <p>The WindowFlags type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;WindowType&gt;. It stores an OR combination of WindowType values.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qwidget.html#windowFlags-prop">QWidget::windowFlags</a> and <a href="http://doc.qt.io/qt-5/qtwidgets-widgets-windowflags-example.html">Window Flags Example</a>.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct WindowType(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for WindowType {
    fn from(value: ::std::os::raw::c_int) -> Self {
        WindowType(value)
    }
}

impl From<WindowType> for ::std::os::raw::c_int {
    fn from(value: WindowType) -> Self {
        value.0
    }
}

impl WindowType {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl WindowType {
    /// This is the default type for <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a>. Widgets of this type are child widgets if they have a parent, and independent windows if they have no parent. See also Qt::Window and Qt::SubWindow. (C++ enum variant: <span style='color: green;'>```Widget = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const Widget: crate::WindowType = crate::WindowType(0);
    /// Indicates that the widget is a window, usually with a window system frame and a title bar, irrespective of whether the widget has a parent or not. Note that it is not possible to unset this flag if the widget does not have a parent. (C++ enum variant: <span style='color: green;'>```Window = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const Window: crate::WindowType = crate::WindowType(1);
    /// Indicates that the widget is a window that should be decorated as a dialog (i.e., typically no maximize or minimize buttons in the title bar). This is the default type for <a href="http://doc.qt.io/qt-5/qdialog.html">QDialog</a>. If you want to use it as a modal dialog, it should be launched from another window, or have a parent and used with the <a href="http://doc.qt.io/qt-5/qwidget.html#windowModality-prop">QWidget::windowModality</a> property. If you make it modal, the dialog will prevent other top-level windows in the application from getting any input. We refer to a top-level window that has a parent as a <i>secondary</i> window. (C++ enum variant: <span style='color: green;'>```Dialog = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const Dialog: crate::WindowType = crate::WindowType(3);
    /// Indicates that the window is a sheet on <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>. Since using a sheet implies window modality, the recommended way is to use <a href="http://doc.qt.io/qt-5/qwidget.html#windowModality-prop">QWidget::setWindowModality</a>(), or <a href="http://doc.qt.io/qt-5/qdialog.html#open">QDialog::open</a>(), instead. (C++ enum variant: <span style='color: green;'>```Sheet = 5```</span>)
    #[allow(non_upper_case_globals)]
    pub const Sheet: crate::WindowType = crate::WindowType(5);
    /// Indicates that the widget is a drawer on <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>. (C++ enum variant: <span style='color: green;'>```Drawer = 7```</span>)
    #[allow(non_upper_case_globals)]
    pub const Drawer: crate::WindowType = crate::WindowType(7);
    /// Indicates that the widget is a pop-up top-level window, i.e. that it is modal, but has a window system frame appropriate for pop-up menus. (C++ enum variant: <span style='color: green;'>```Popup = 9```</span>)
    #[allow(non_upper_case_globals)]
    pub const Popup: crate::WindowType = crate::WindowType(9);
    /// Indicates that the widget is a tool window. A tool window is often a small window with a smaller than usual title bar and decoration, typically used for collections of tool buttons. If there is a parent, the tool window will always be kept on top of it. If there isn't a parent, you may consider using Qt::WindowStaysOnTopHint as well. If the window system supports it, a tool window can be decorated with a somewhat lighter frame. It can also be combined with Qt::FramelessWindowHint. On <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, tool windows correspond to the <a href="http://developer.apple.com/documentation/Carbon/Conceptual/HandlingWindowsControls/hitb-wind_cont_concept/chapter_2_section_2.html">Floating</a> class of windows. This means that the window lives on a level above normal windows making it impossible to put a normal window on top of it. By default, tool windows will disappear when the application is inactive. This can be controlled by the <a href="http://doc.qt.io/qt-5/qt.html#WidgetAttribute-enum">Qt::WA_MacAlwaysShowToolWindow</a> attribute. (C++ enum variant: <span style='color: green;'>```Tool = 11```</span>)
    #[allow(non_upper_case_globals)]
    pub const Tool: crate::WindowType = crate::WindowType(11);
    /// Indicates that the widget is a tooltip. This is used internally to implement <a href="http://doc.qt.io/qt-5/qwidget.html#toolTip-prop">tooltips</a>. (C++ enum variant: <span style='color: green;'>```ToolTip = 13```</span>)
    #[allow(non_upper_case_globals)]
    pub const ToolTip: crate::WindowType = crate::WindowType(13);
    /// Indicates that the window is a splash screen. This is the default type for <a href="http://doc.qt.io/qt-5/qsplashscreen.html">QSplashScreen</a>. (C++ enum variant: <span style='color: green;'>```SplashScreen = 15```</span>)
    #[allow(non_upper_case_globals)]
    pub const SplashScreen: crate::WindowType = crate::WindowType(15);
    /// Indicates that this widget is the desktop. This is the type for <a href="http://doc.qt.io/qt-5/qdesktopwidget.html">QDesktopWidget</a>. (C++ enum variant: <span style='color: green;'>```Desktop = 17```</span>)
    #[allow(non_upper_case_globals)]
    pub const Desktop: crate::WindowType = crate::WindowType(17);
    /// Indicates that this widget is a sub-window, such as a <a href="http://doc.qt.io/qt-5/qmdisubwindow.html">QMdiSubWindow</a> widget. (C++ enum variant: <span style='color: green;'>```SubWindow = 18```</span>)
    #[allow(non_upper_case_globals)]
    pub const SubWindow: crate::WindowType = crate::WindowType(18);
    /// Indicates that this window object is a handle representing a native platform window created by another process or by manually using native code. (C++ enum variant: <span style='color: green;'>```ForeignWindow = 33```</span>)
    #[allow(non_upper_case_globals)]
    pub const ForeignWindow: crate::WindowType = crate::WindowType(33);
    /// Indicates that the window represents a cover window, which is shown when the application is minimized on some platforms. (C++ enum variant: <span style='color: green;'>```CoverWindow = 65```</span>)
    #[allow(non_upper_case_globals)]
    pub const CoverWindow: crate::WindowType = crate::WindowType(65);
    /// A mask for extracting the window type part of the window flags. (C++ enum variant: <span style='color: green;'>```WindowType_Mask = 255```</span>)
    #[allow(non_upper_case_globals)]
    pub const WindowTypeMask: crate::WindowType = crate::WindowType(255);
    /// Gives the window a thin dialog border on Windows. This style is traditionally used for fixed-size dialogs. (C++ enum variant: <span style='color: green;'>```MSWindowsFixedSizeDialogHint = 256```</span>)
    #[allow(non_upper_case_globals)]
    pub const MSWindowsFixedSizeDialogHint: crate::WindowType = crate::WindowType(256);
    /// Gives the window its own display context on Windows. (C++ enum variant: <span style='color: green;'>```MSWindowsOwnDC = 512```</span>)
    #[allow(non_upper_case_globals)]
    pub const MSWindowsOwnDC: crate::WindowType = crate::WindowType(512);
    /// This flag can be used to indicate to the platform plugin that "all" window manager protocols should be disabled. This flag will behave different depending on what operating system the application is running on and what window manager is running. The flag can be used to get a native window with no configuration set. (C++ enum variant: <span style='color: green;'>```BypassWindowManagerHint = 1024```</span>)
    #[allow(non_upper_case_globals)]
    pub const BypassWindowManagerHint: crate::WindowType = crate::WindowType(1024);
    /// Bypass the window manager completely. This results in a borderless window that is not managed at all (i.e., no keyboard input unless you call <a href="http://doc.qt.io/qt-5/qwidget.html#activateWindow">QWidget::activateWindow</a>() manually). (C++ enum variant: <span style='color: green;'>```X11BypassWindowManagerHint = 1024```</span>)
    #[allow(non_upper_case_globals)]
    pub const X11BypassWindowManagerHint: crate::WindowType = crate::WindowType(1024);
    /// Produces a borderless window. The user cannot move or resize a borderless window via the window system. On X11, the result of the flag is dependent on the window manager and its ability to understand Motif and/or NETWM hints. Most existing modern window managers can handle this. (C++ enum variant: <span style='color: green;'>```FramelessWindowHint = 2048```</span>)
    #[allow(non_upper_case_globals)]
    pub const FramelessWindowHint: crate::WindowType = crate::WindowType(2048);
    /// Gives the window a title bar. (C++ enum variant: <span style='color: green;'>```WindowTitleHint = 4096```</span>)
    #[allow(non_upper_case_globals)]
    pub const WindowTitleHint: crate::WindowType = crate::WindowType(4096);
    /// Adds a window system menu, and possibly a close button (for example on Mac). If you need to hide or show a close button, it is more portable to use <code>WindowCloseButtonHint</code>. (C++ enum variant: <span style='color: green;'>```WindowSystemMenuHint = 8192```</span>)
    #[allow(non_upper_case_globals)]
    pub const WindowSystemMenuHint: crate::WindowType = crate::WindowType(8192);
    /// Adds a minimize button. On some platforms this implies Qt::WindowSystemMenuHint for it to work. (C++ enum variant: <span style='color: green;'>```WindowMinimizeButtonHint = 16384```</span>)
    #[allow(non_upper_case_globals)]
    pub const WindowMinimizeButtonHint: crate::WindowType = crate::WindowType(16384);
    /// Adds a maximize button. On some platforms this implies Qt::WindowSystemMenuHint for it to work. (C++ enum variant: <span style='color: green;'>```WindowMaximizeButtonHint = 32768```</span>)
    #[allow(non_upper_case_globals)]
    pub const WindowMaximizeButtonHint: crate::WindowType = crate::WindowType(32768);
    /// Adds a minimize and a maximize button. On some platforms this implies Qt::WindowSystemMenuHint for it to work. (C++ enum variant: <span style='color: green;'>```WindowMinMaxButtonsHint = 49152```</span>)
    #[allow(non_upper_case_globals)]
    pub const WindowMinMaxButtonsHint: crate::WindowType = crate::WindowType(49152);
    /// Adds a context help button to dialogs. On some platforms this implies Qt::WindowSystemMenuHint for it to work. (C++ enum variant: <span style='color: green;'>```WindowContextHelpButtonHint = 65536```</span>)
    #[allow(non_upper_case_globals)]
    pub const WindowContextHelpButtonHint: crate::WindowType = crate::WindowType(65536);
    /// Adds a shade button in place of the minimize button if the underlying window manager supports it. (C++ enum variant: <span style='color: green;'>```WindowShadeButtonHint = 131072```</span>)
    #[allow(non_upper_case_globals)]
    pub const WindowShadeButtonHint: crate::WindowType = crate::WindowType(131072);
    /// Informs the window system that the window should stay on top of all other windows. Note that on some window managers on X11 you also have to pass Qt::X11BypassWindowManagerHint for this flag to work correctly. (C++ enum variant: <span style='color: green;'>```WindowStaysOnTopHint = 262144```</span>)
    #[allow(non_upper_case_globals)]
    pub const WindowStaysOnTopHint: crate::WindowType = crate::WindowType(262144);
    /// Informs the window system that this window is used only for output (displaying something) and does not take input. Therefore input events should pass through as if it wasn't there. (C++ enum variant: <span style='color: green;'>```WindowTransparentForInput = 524288```</span>)
    #[allow(non_upper_case_globals)]
    pub const WindowTransparentForInput: crate::WindowType = crate::WindowType(524288);
    /// Informs the window system that this window implements its own set of gestures and that system level gestures, like for instance three-finger desktop switching, should be disabled. (C++ enum variant: <span style='color: green;'>```WindowOverridesSystemGestures = 1048576```</span>)
    #[allow(non_upper_case_globals)]
    pub const WindowOverridesSystemGestures: crate::WindowType = crate::WindowType(1048576);
    /// Informs the window system that this window should not receive the input focus. (C++ enum variant: <span style='color: green;'>```WindowDoesNotAcceptFocus = 2097152```</span>)
    #[allow(non_upper_case_globals)]
    pub const WindowDoesNotAcceptFocus: crate::WindowType = crate::WindowType(2097152);
    /// Informs the window system that when maximizing the window it should use as much of the available screen geometry as possible, including areas that may be covered by system UI such as status bars or application launchers. This may result in the window being placed under these system UIs, but does not guarantee it, depending on whether or not the platform supports it. When the flag is enabled the user is responsible for taking <a href="http://doc.qt.io/qt-5/qscreen.html#availableGeometry-prop">QScreen::availableGeometry</a>() into account, so that any UI elements in the application that require user interaction are not covered by system UI. (C++ enum variant: <span style='color: green;'>```MaximizeUsingFullscreenGeometryHint = 4194304```</span>)
    #[allow(non_upper_case_globals)]
    pub const MaximizeUsingFullscreenGeometryHint: crate::WindowType = crate::WindowType(4194304);
    /// Turns off the default window title hints. (C++ enum variant: <span style='color: green;'>```CustomizeWindowHint = 33554432```</span>)
    #[allow(non_upper_case_globals)]
    pub const CustomizeWindowHint: crate::WindowType = crate::WindowType(33554432);
    /// Informs the window system that the window should stay on bottom of all other windows. Note that on X11 this hint will work only in window managers that support _NET_WM_STATE_BELOW atom. If a window always on the bottom has a parent, the parent will also be left on the bottom. This window hint is currently not implemented for <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>. (C++ enum variant: <span style='color: green;'>```WindowStaysOnBottomHint = 67108864```</span>)
    #[allow(non_upper_case_globals)]
    pub const WindowStaysOnBottomHint: crate::WindowType = crate::WindowType(67108864);
    /// Adds a close button. On some platforms this implies Qt::WindowSystemMenuHint for it to work. (C++ enum variant: <span style='color: green;'>```WindowCloseButtonHint = 134217728```</span>)
    #[allow(non_upper_case_globals)]
    pub const WindowCloseButtonHint: crate::WindowType = crate::WindowType(134217728);
    /// On <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> adds a tool bar button (i.e., the oblong button that is on the top right of windows that have toolbars). (C++ enum variant: <span style='color: green;'>```MacWindowToolBarButtonHint = 268435456```</span>)
    #[allow(non_upper_case_globals)]
    pub const MacWindowToolBarButtonHint: crate::WindowType = crate::WindowType(268435456);
    /// Prevents the window and its children from automatically embedding themselves into a <a href="http://doc.qt.io/qt-5/qgraphicsproxywidget.html">QGraphicsProxyWidget</a> if the parent widget is already embedded. You can set this flag if you want your widget to always be a toplevel widget on the desktop, regardless of whether the parent widget is embedded in a scene or not. (C++ enum variant: <span style='color: green;'>```BypassGraphicsProxyWidget = 536870912```</span>)
    #[allow(non_upper_case_globals)]
    pub const BypassGraphicsProxyWidget: crate::WindowType = crate::WindowType(536870912);
    /// Disables window drop shadow on supporting platforms. (C++ enum variant: <span style='color: green;'>```NoDropShadowWindowHint = 1073741824```</span>)
    #[allow(non_upper_case_globals)]
    pub const NoDropShadowWindowHint: crate::WindowType = crate::WindowType(1073741824);
    /// On <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> adds a fullscreen button. (C++ enum variant: <span style='color: green;'>```WindowFullscreenButtonHint = -2147483648```</span>)
    #[allow(non_upper_case_globals)]
    pub const WindowFullscreenButtonHint: crate::WindowType = crate::WindowType(-2147483648);
}

/// <p>This enum type is used to specify the current state of a top-level window.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::WindowState```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#WindowState-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type is used to specify the current state of a top-level window.</p>
/// <p>The states are</p>
///
/// <p>The WindowStates type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;WindowState&gt;. It stores an OR combination of WindowState values.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct WindowState(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for WindowState {
    fn from(value: ::std::os::raw::c_int) -> Self {
        WindowState(value)
    }
}

impl From<WindowState> for ::std::os::raw::c_int {
    fn from(value: WindowState) -> Self {
        value.0
    }
}

impl WindowState {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl WindowState {
    /// The window has no state set (in normal state). (C++ enum variant: <span style='color: green;'>```WindowNoState = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const WindowNoState: crate::WindowState = crate::WindowState(0);
    /// The window is minimized (i.e. iconified). (C++ enum variant: <span style='color: green;'>```WindowMinimized = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const WindowMinimized: crate::WindowState = crate::WindowState(1);
    /// The window is maximized with a frame around it. (C++ enum variant: <span style='color: green;'>```WindowMaximized = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const WindowMaximized: crate::WindowState = crate::WindowState(2);
    /// The window fills the entire screen without any frame around it. (C++ enum variant: <span style='color: green;'>```WindowFullScreen = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const WindowFullScreen: crate::WindowState = crate::WindowState(4);
    /// The window is the active window, i.e. it has keyboard focus. (C++ enum variant: <span style='color: green;'>```WindowActive = 8```</span>)
    #[allow(non_upper_case_globals)]
    pub const WindowActive: crate::WindowState = crate::WindowState(8);
}

/// <p>This enum type is used to specify the current state of the application.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::ApplicationState```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#ApplicationState-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type is used to specify the current state of the application.</p>
/// <p>The states are</p>
///
/// <p>This enum was introduced or modified in  Qt 5.1.</p>
/// <p>The ApplicationStates type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;ApplicationState&gt;. It stores an OR combination of ApplicationState values.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct ApplicationState(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for ApplicationState {
    fn from(value: ::std::os::raw::c_int) -> Self {
        ApplicationState(value)
    }
}

impl From<ApplicationState> for ::std::os::raw::c_int {
    fn from(value: ApplicationState) -> Self {
        value.0
    }
}

impl ApplicationState {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl ApplicationState {
    /// The application is about to suspend. When entering this state, the application should save its state, cease all activities, and be prepared for code execution to stop. While suspended, the application can be killed at any time without further warnings (e.g. when low memory forces the OS to purge suspended applications). (C++ enum variant: <span style='color: green;'>```ApplicationSuspended = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const ApplicationSuspended: crate::ApplicationState = crate::ApplicationState(0);
    /// The application is hidden and runs in the background. This is the normal state for applications that need to do background processing, like playing music, while the user interacts with other applications. The application should free up all graphical resources when entering this state. (C++ enum variant: <span style='color: green;'>```ApplicationHidden = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const ApplicationHidden: crate::ApplicationState = crate::ApplicationState(1);
    /// The application is visible, but not selected to be in front. On desktop platforms, this typically means that the user activated another application. On mobile platforms, it is more common to enter this state when the OS is interrupting the user with e.g. incoming calls or SMS-messages. While in this state, consider reducing CPU-intensive tasks. (C++ enum variant: <span style='color: green;'>```ApplicationInactive = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const ApplicationInactive: crate::ApplicationState = crate::ApplicationState(2);
    /// The application is visible and selected to be in front. (C++ enum variant: <span style='color: green;'>```ApplicationActive = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const ApplicationActive: crate::ApplicationState = crate::ApplicationState(4);
}

/// <p>This enum type specifies the various orientations a screen might have.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::ScreenOrientation```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#ScreenOrientation-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type specifies the various orientations a screen might have.</p>
///
/// <p>The ScreenOrientations type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;ScreenOrientation&gt;. It stores an OR combination of ScreenOrientation values.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct ScreenOrientation(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for ScreenOrientation {
    fn from(value: ::std::os::raw::c_int) -> Self {
        ScreenOrientation(value)
    }
}

impl From<ScreenOrientation> for ::std::os::raw::c_int {
    fn from(value: ScreenOrientation) -> Self {
        value.0
    }
}

impl ScreenOrientation {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl ScreenOrientation {
    /// The display's primary orientation. (C++ enum variant: <span style='color: green;'>```PrimaryOrientation = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const PrimaryOrientation: crate::ScreenOrientation = crate::ScreenOrientation(0);
    /// Portrait orientation, display height is greater than display width, rotated 90 degree clockwise relative to landscape. (C++ enum variant: <span style='color: green;'>```PortraitOrientation = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const PortraitOrientation: crate::ScreenOrientation = crate::ScreenOrientation(1);
    /// Landscape orientation, display width is greater than display height. (C++ enum variant: <span style='color: green;'>```LandscapeOrientation = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const LandscapeOrientation: crate::ScreenOrientation = crate::ScreenOrientation(2);
    /// Inverted portrait orientation, rotated 180 degrees relative to portrait. (C++ enum variant: <span style='color: green;'>```InvertedPortraitOrientation = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const InvertedPortraitOrientation: crate::ScreenOrientation = crate::ScreenOrientation(4);
    /// Inverted landscape orientation, rotated 180 degrees relative to landscape. (C++ enum variant: <span style='color: green;'>```InvertedLandscapeOrientation = 8```</span>)
    #[allow(non_upper_case_globals)]
    pub const InvertedLandscapeOrientation: crate::ScreenOrientation = crate::ScreenOrientation(8);
}

/// <p>This enum type is used to specify various widget attributes. Attributes are set and cleared with <a href="http://doc.qt.io/qt-5/qwidget.html#setAttribute">QWidget::setAttribute</a>(), and queried with <a href="http://doc.qt.io/qt-5/qwidget.html#testAttribute">QWidget::testAttribute</a>(), although some have special convenience functions which are mentioned below.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::WidgetAttribute```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#WidgetAttribute-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type is used to specify various widget attributes. Attributes are set and cleared with <a href="http://doc.qt.io/qt-5/qwidget.html#setAttribute">QWidget::setAttribute</a>(), and queried with <a href="http://doc.qt.io/qt-5/qwidget.html#testAttribute">QWidget::testAttribute</a>(), although some have special convenience functions which are mentioned below.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct WidgetAttribute(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for WidgetAttribute {
    fn from(value: ::std::os::raw::c_int) -> Self {
        WidgetAttribute(value)
    }
}

impl From<WidgetAttribute> for ::std::os::raw::c_int {
    fn from(value: WidgetAttribute) -> Self {
        value.0
    }
}

impl WidgetAttribute {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl WidgetAttribute {
    /// Indicates that the widget is disabled, i.e. it does not receive any mouse or keyboard events. There is also a getter functions <a href="http://doc.qt.io/qt-5/qwidget.html#enabled-prop">QWidget::isEnabled</a>(). This is set/cleared by the Qt kernel. (C++ enum variant: <span style='color: green;'>```WA_Disabled = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const WADisabled: crate::WidgetAttribute = crate::WidgetAttribute(0);
    /// Indicates that the widget is under the mouse cursor. The value is not updated correctly during drag and drop operations. There is also a getter function, <a href="http://doc.qt.io/qt-5/qwidget.html#underMouse">QWidget::underMouse</a>(). This flag is set or cleared by the Qt kernel. (C++ enum variant: <span style='color: green;'>```WA_UnderMouse = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAUnderMouse: crate::WidgetAttribute = crate::WidgetAttribute(1);
    /// Indicates that the widget has mouse tracking enabled. See <a href="http://doc.qt.io/qt-5/qwidget.html#mouseTracking-prop">QWidget::mouseTracking</a>. (C++ enum variant: <span style='color: green;'>```WA_MouseTracking = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAMouseTracking: crate::WidgetAttribute = crate::WidgetAttribute(2);
    /// This flag is superfluous and obsolete; it no longer has any effect. Since Qt 4.1, all widgets that do not set WA_PaintOnScreen propagate their contents. (C++ enum variant: <span style='color: green;'>```WA_ContentsPropagated = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAContentsPropagated: crate::WidgetAttribute = crate::WidgetAttribute(3);
    /// Indicates that the widget paints all its pixels when it receives a paint event. Thus, it is not required for operations like updating, resizing, scrolling and focus changes to erase the widget before generating paint events. The use of WA_OpaquePaintEvent provides a small optimization by helping to reduce flicker on systems that do not support double buffering and avoiding computational cycles necessary to erase the background prior to painting. <b>Note</b>: Unlike WA_NoSystemBackground, WA_OpaquePaintEvent makes an effort to avoid transparent window backgrounds. This flag is set or cleared by the widget's author. (C++ enum variant: <span style='color: green;'>```WA_OpaquePaintEvent = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAOpaquePaintEvent: crate::WidgetAttribute = crate::WidgetAttribute(4);
    /// This value is obsolete. Use WA_OpaquePaintEvent instead. (C++ enum variant: <span style='color: green;'>```WA_NoBackground = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const WANoBackground: crate::WidgetAttribute = crate::WidgetAttribute(4);
    /// Indicates that the widget contents are north-west aligned and static. On resize, such a widget will receive paint events only for parts of itself that are newly visible. This flag is set or cleared by the widget's author. (C++ enum variant: <span style='color: green;'>```WA_StaticContents = 5```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAStaticContents: crate::WidgetAttribute = crate::WidgetAttribute(5);
    /// C++ enum variant: <span style='color: green;'>```WA_LaidOut = 7```</span>
    #[allow(non_upper_case_globals)]
    pub const WALaidOut: crate::WidgetAttribute = crate::WidgetAttribute(7);
    /// Indicates that the widget wants to draw directly onto the screen. Widgets with this attribute set do not participate in composition management, i.e. they cannot be semi-transparent or shine through semi-transparent overlapping widgets. <b>Note</b>: This flag is only supported on X11 and it disables double buffering. On Qt for Embedded Linux, the flag only works when set on a top-level widget and it relies on support from the active screen driver. This flag is set or cleared by the widget's author. To render outside of Qt's paint system, e.g., if you require native painting primitives, you need to reimplement <a href="http://doc.qt.io/qt-5/qwidget.html#paintEngine">QWidget::paintEngine</a>() to return 0 and set this flag. (C++ enum variant: <span style='color: green;'>```WA_PaintOnScreen = 8```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAPaintOnScreen: crate::WidgetAttribute = crate::WidgetAttribute(8);
    /// Indicates that the widget has no background, i.e. when the widget receives paint events, the background is not automatically repainted. <b>Note</b>: Unlike WA_OpaquePaintEvent, newly exposed areas are <b>never</b> filled with the background (e.g., after showing a window for the first time the user can see "through" it until the application processes the paint events). This flag is set or cleared by the widget's author. (C++ enum variant: <span style='color: green;'>```WA_NoSystemBackground = 9```</span>)
    #[allow(non_upper_case_globals)]
    pub const WANoSystemBackground: crate::WidgetAttribute = crate::WidgetAttribute(9);
    /// Indicates that updates are blocked (including the system background). This flag is set or cleared by the Qt kernel. <b>Warning</b>: This flag must <i>never</i> be set or cleared by the widget's author. (C++ enum variant: <span style='color: green;'>```WA_UpdatesDisabled = 10```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAUpdatesDisabled: crate::WidgetAttribute = crate::WidgetAttribute(10);
    /// Indicates that the widget is mapped on screen. This is set/cleared by the Qt kernel. (C++ enum variant: <span style='color: green;'>```WA_Mapped = 11```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAMapped: crate::WidgetAttribute = crate::WidgetAttribute(11);
    /// When a widget that has this attribute set is clicked, and its window is inactive, the click will make the window active but won't be seen by the widget. Typical use of this attribute is on widgets with "destructive" actions, such as a "Delete" button. WA_MacNoClickThrough also applies to all child widgets of the widget that has it set. (C++ enum variant: <span style='color: green;'>```WA_MacNoClickThrough = 12```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAMacNoClickThrough: crate::WidgetAttribute = crate::WidgetAttribute(12);
    /// Enables input methods for Asian languages. Must be set when creating custom text editing widgets. (C++ enum variant: <span style='color: green;'>```WA_InputMethodEnabled = 14```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAInputMethodEnabled: crate::WidgetAttribute = crate::WidgetAttribute(14);
    /// C++ enum variant: <span style='color: green;'>```WA_WState_Visible = 15```</span>
    #[allow(non_upper_case_globals)]
    pub const WAWStateVisible: crate::WidgetAttribute = crate::WidgetAttribute(15);
    /// C++ enum variant: <span style='color: green;'>```WA_WState_Hidden = 16```</span>
    #[allow(non_upper_case_globals)]
    pub const WAWStateHidden: crate::WidgetAttribute = crate::WidgetAttribute(16);
    /// Indicates that the widget is explicitly disabled, i.e. it will remain disabled even when all its ancestors are set to the enabled state. This implies WA_Disabled. This is set/cleared by <a href="http://doc.qt.io/qt-5/qwidget.html#enabled-prop">QWidget::setEnabled</a>() and <a href="http://doc.qt.io/qt-5/qwidget.html#setDisabled">QWidget::setDisabled</a>(). (C++ enum variant: <span style='color: green;'>```WA_ForceDisabled = 32```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAForceDisabled: crate::WidgetAttribute = crate::WidgetAttribute(32);
    /// Enables key event compression if set, and disables it if not set. By default key compression is off, so widgets receive one key press event for each key press (or more, since autorepeat is usually on). If you turn it on and your program doesn't keep up with key input, Qt may try to compress key events so that more than one character can be processed in each event. For example, a word processor widget might receive 2, 3 or more characters in each <a href="http://doc.qt.io/qt-5/qkeyevent.html#text">QKeyEvent::text</a>(), if the layout recalculation takes too long for the CPU. If a widget supports multiple character unicode input, it is always safe to turn the compression on. Qt performs key event compression only for printable characters. <a href="http://doc.qt.io/qt-5/qt.html#Modifier-enum">Qt::Modifier</a> keys, cursor movement keys, function keys and miscellaneous action keys (e.g. Escape, Enter, Backspace, PrintScreen) will stop key event compression, even if there are more compressible key events available. Platforms other than Mac and X11 do not support this compression, in which case turning it on will have no effect. This is set/cleared by the widget's author. (C++ enum variant: <span style='color: green;'>```WA_KeyCompression = 33```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAKeyCompression: crate::WidgetAttribute = crate::WidgetAttribute(33);
    /// Indicates that a move event is pending, e.g., when a hidden widget was moved. This flag is set or cleared by the Qt kernel. (C++ enum variant: <span style='color: green;'>```WA_PendingMoveEvent = 34```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAPendingMoveEvent: crate::WidgetAttribute = crate::WidgetAttribute(34);
    /// Indicates that a resize event is pending, e.g., when a hidden widget was resized. This flag is set or cleared by the Qt kernel. (C++ enum variant: <span style='color: green;'>```WA_PendingResizeEvent = 35```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAPendingResizeEvent: crate::WidgetAttribute = crate::WidgetAttribute(35);
    /// Indicates that the widget has a palette of its own. This flag is set or cleared by <a href="http://doc.qt.io/qt-5/qwidget.html#palette-prop">QWidget::setPalette</a>(). (C++ enum variant: <span style='color: green;'>```WA_SetPalette = 36```</span>)
    #[allow(non_upper_case_globals)]
    pub const WASetPalette: crate::WidgetAttribute = crate::WidgetAttribute(36);
    /// Indicates that the widget has a font of its own. This flag is set or cleared by <a href="http://doc.qt.io/qt-5/qwidget.html#font-prop">QWidget::setFont</a>(). (C++ enum variant: <span style='color: green;'>```WA_SetFont = 37```</span>)
    #[allow(non_upper_case_globals)]
    pub const WASetFont: crate::WidgetAttribute = crate::WidgetAttribute(37);
    /// Indicates that the widget has a cursor of its own. This flag is set or cleared by <a href="http://doc.qt.io/qt-5/qwidget.html#cursor-prop">QWidget::setCursor</a>() and <a href="http://doc.qt.io/qt-5/qwidget.html#cursor-prop">QWidget::unsetCursor</a>(). (C++ enum variant: <span style='color: green;'>```WA_SetCursor = 38```</span>)
    #[allow(non_upper_case_globals)]
    pub const WASetCursor: crate::WidgetAttribute = crate::WidgetAttribute(38);
    /// Indicates that the widget does not want to receive ChildAdded or ChildRemoved events sent from its children. This is set by a widget's author. (C++ enum variant: <span style='color: green;'>```WA_NoChildEventsFromChildren = 39```</span>)
    #[allow(non_upper_case_globals)]
    pub const WANoChildEventsFromChildren: crate::WidgetAttribute = crate::WidgetAttribute(39);
    /// Indicates that the window is marked as modified. On some platforms this flag will do nothing, on others (including <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and Windows) the window will take a modified appearance. This flag is set or cleared by <a href="http://doc.qt.io/qt-5/qwidget.html#windowModified-prop">QWidget::setWindowModified</a>(). (C++ enum variant: <span style='color: green;'>```WA_WindowModified = 41```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAWindowModified: crate::WidgetAttribute = crate::WidgetAttribute(41);
    /// Indicates that the widget has an explicit size. This flag is set or cleared by <a href="http://doc.qt.io/qt-5/qwidget.html#size-prop">QWidget::resize</a>() and <a href="http://doc.qt.io/qt-5/qwidget.html#geometry-prop">QWidget::setGeometry</a>(). (C++ enum variant: <span style='color: green;'>```WA_Resized = 42```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAResized: crate::WidgetAttribute = crate::WidgetAttribute(42);
    /// Indicates that the widget has an explicit position. This is set/cleared by <a href="http://doc.qt.io/qt-5/qwidget.html#pos-prop">QWidget::move</a>() and by <a href="http://doc.qt.io/qt-5/qwidget.html#geometry-prop">QWidget::setGeometry</a>(). (C++ enum variant: <span style='color: green;'>```WA_Moved = 43```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAMoved: crate::WidgetAttribute = crate::WidgetAttribute(43);
    /// C++ enum variant: <span style='color: green;'>```WA_PendingUpdate = 44```</span>
    #[allow(non_upper_case_globals)]
    pub const WAPendingUpdate: crate::WidgetAttribute = crate::WidgetAttribute(44);
    /// C++ enum variant: <span style='color: green;'>```WA_InvalidSize = 45```</span>
    #[allow(non_upper_case_globals)]
    pub const WAInvalidSize: crate::WidgetAttribute = crate::WidgetAttribute(45);
    /// Indicates the widget should be drawn in the brushed metal style as supported by the windowing system. This attribute is only applicable to <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>. (C++ enum variant: <span style='color: green;'>```WA_MacBrushedMetal = 46```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAMacBrushedMetal: crate::WidgetAttribute = crate::WidgetAttribute(46);
    /// C++ enum variant: <span style='color: green;'>```WA_MacMetalStyle = 46```</span>
    #[allow(non_upper_case_globals)]
    pub const WAMacMetalStyle: crate::WidgetAttribute = crate::WidgetAttribute(46);
    /// Indicates that the widget wants to continue operating normally in "What's This?" mode. This is set by the widget's author. (C++ enum variant: <span style='color: green;'>```WA_CustomWhatsThis = 47```</span>)
    #[allow(non_upper_case_globals)]
    pub const WACustomWhatsThis: crate::WidgetAttribute = crate::WidgetAttribute(47);
    /// Indicates that the widget wants <a href="http://doc.qt.io/qt-5/qlayout.html">QLayout</a> to operate on the entire <a href="http://doc.qt.io/qt-5/qwidget.html#rect-prop">QWidget::rect</a>(), not only on <a href="http://doc.qt.io/qt-5/qwidget.html#contentsRect">QWidget::contentsRect</a>(). This is set by the widget's author. (C++ enum variant: <span style='color: green;'>```WA_LayoutOnEntireRect = 48```</span>)
    #[allow(non_upper_case_globals)]
    pub const WALayoutOnEntireRect: crate::WidgetAttribute = crate::WidgetAttribute(48);
    /// Indicates that the widget is outside the valid range of the window system's coordinate system. A widget outside the valid range cannot be mapped on screen. This is set/cleared by the Qt kernel. (C++ enum variant: <span style='color: green;'>```WA_OutsideWSRange = 49```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAOutsideWSRange: crate::WidgetAttribute = crate::WidgetAttribute(49);
    /// C++ enum variant: <span style='color: green;'>```WA_GrabbedShortcut = 50```</span>
    #[allow(non_upper_case_globals)]
    pub const WAGrabbedShortcut: crate::WidgetAttribute = crate::WidgetAttribute(50);
    /// When enabled, this attribute disables the delivery of mouse events to the widget and its children. Mouse events are delivered to other widgets as if the widget and its children were not present in the widget hierarchy; mouse clicks and other events effectively "pass through" them. This attribute is disabled by default. (C++ enum variant: <span style='color: green;'>```WA_TransparentForMouseEvents = 51```</span>)
    #[allow(non_upper_case_globals)]
    pub const WATransparentForMouseEvents: crate::WidgetAttribute = crate::WidgetAttribute(51);
    /// Makes all painters operating on this widget unclipped. Children of this widget or other widgets in front of it do not clip the area the painter can paint on. This flag is only supported for widgets with the WA_PaintOnScreen flag set. The preferred way to do this in a cross platform way is to create a transparent widget that lies in front of the other widgets. (C++ enum variant: <span style='color: green;'>```WA_PaintUnclipped = 52```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAPaintUnclipped: crate::WidgetAttribute = crate::WidgetAttribute(52);
    /// C++ enum variant: <span style='color: green;'>```WA_SetWindowIcon = 53```</span>
    #[allow(non_upper_case_globals)]
    pub const WASetWindowIcon: crate::WidgetAttribute = crate::WidgetAttribute(53);
    /// Used for pop-up widgets. Indicates that the most recent mouse press event should not be replayed when the pop-up widget closes. The flag is set by the widget's author and cleared by the Qt kernel every time the widget receives a new mouse event. (C++ enum variant: <span style='color: green;'>```WA_NoMouseReplay = 54```</span>)
    #[allow(non_upper_case_globals)]
    pub const WANoMouseReplay: crate::WidgetAttribute = crate::WidgetAttribute(54);
    /// Makes Qt delete this widget when the widget has accepted the close event (see <a href="http://doc.qt.io/qt-5/qwidget.html#closeEvent">QWidget::closeEvent</a>()). (C++ enum variant: <span style='color: green;'>```WA_DeleteOnClose = 55```</span>)
    #[allow(non_upper_case_globals)]
    pub const WADeleteOnClose: crate::WidgetAttribute = crate::WidgetAttribute(55);
    /// Indicates that the layout direction for the widget is right to left. (C++ enum variant: <span style='color: green;'>```WA_RightToLeft = 56```</span>)
    #[allow(non_upper_case_globals)]
    pub const WARightToLeft: crate::WidgetAttribute = crate::WidgetAttribute(56);
    /// C++ enum variant: <span style='color: green;'>```WA_SetLayoutDirection = 57```</span>
    #[allow(non_upper_case_globals)]
    pub const WASetLayoutDirection: crate::WidgetAttribute = crate::WidgetAttribute(57);
    /// Indicates that the widget does not want ChildAdded or ChildRemoved events sent to its parent. This is rarely necessary but can help to avoid automatic insertion widgets like splitters and layouts. This is set by a widget's author. (C++ enum variant: <span style='color: green;'>```WA_NoChildEventsForParent = 58```</span>)
    #[allow(non_upper_case_globals)]
    pub const WANoChildEventsForParent: crate::WidgetAttribute = crate::WidgetAttribute(58);
    /// Indicates that updates are explicitly disabled for the widget; i.e. it will remain disabled even when all its ancestors are set to the updates-enabled state. This implies WA_UpdatesDisabled. This is set/cleared by <a href="http://doc.qt.io/qt-5/qwidget.html#updatesEnabled-prop">QWidget::setUpdatesEnabled</a>(). (C++ enum variant: <span style='color: green;'>```WA_ForceUpdatesDisabled = 59```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAForceUpdatesDisabled: crate::WidgetAttribute = crate::WidgetAttribute(59);
    /// C++ enum variant: <span style='color: green;'>```WA_WState_Created = 60```</span>
    #[allow(non_upper_case_globals)]
    pub const WAWStateCreated: crate::WidgetAttribute = crate::WidgetAttribute(60);
    /// C++ enum variant: <span style='color: green;'>```WA_WState_CompressKeys = 61```</span>
    #[allow(non_upper_case_globals)]
    pub const WAWStateCompressKeys: crate::WidgetAttribute = crate::WidgetAttribute(61);
    /// C++ enum variant: <span style='color: green;'>```WA_WState_InPaintEvent = 62```</span>
    #[allow(non_upper_case_globals)]
    pub const WAWStateInPaintEvent: crate::WidgetAttribute = crate::WidgetAttribute(62);
    /// C++ enum variant: <span style='color: green;'>```WA_WState_Reparented = 63```</span>
    #[allow(non_upper_case_globals)]
    pub const WAWStateReparented: crate::WidgetAttribute = crate::WidgetAttribute(63);
    /// C++ enum variant: <span style='color: green;'>```WA_WState_ConfigPending = 64```</span>
    #[allow(non_upper_case_globals)]
    pub const WAWStateConfigPending: crate::WidgetAttribute = crate::WidgetAttribute(64);
    /// C++ enum variant: <span style='color: green;'>```WA_WState_Polished = 66```</span>
    #[allow(non_upper_case_globals)]
    pub const WAWStatePolished: crate::WidgetAttribute = crate::WidgetAttribute(66);
    /// C++ enum variant: <span style='color: green;'>```WA_WState_DND = 67```</span>
    #[allow(non_upper_case_globals)]
    pub const WAWStateDND: crate::WidgetAttribute = crate::WidgetAttribute(67);
    /// C++ enum variant: <span style='color: green;'>```WA_WState_OwnSizePolicy = 68```</span>
    #[allow(non_upper_case_globals)]
    pub const WAWStateOwnSizePolicy: crate::WidgetAttribute = crate::WidgetAttribute(68);
    /// C++ enum variant: <span style='color: green;'>```WA_WState_ExplicitShowHide = 69```</span>
    #[allow(non_upper_case_globals)]
    pub const WAWStateExplicitShowHide: crate::WidgetAttribute = crate::WidgetAttribute(69);
    /// <i>This attribute has been deprecated.</i> Use <a href="http://doc.qt.io/qt-5/qwidget.html#windowModality-prop">QWidget::windowModality</a> instead. (C++ enum variant: <span style='color: green;'>```WA_ShowModal = 70```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAShowModal: crate::WidgetAttribute = crate::WidgetAttribute(70);
    /// Makes the widget receive mouse events for the entire widget regardless of the currently set mask, overriding <a href="http://doc.qt.io/qt-5/qwidget.html#setMask">QWidget::setMask</a>(). This is not applicable for top-level windows. (C++ enum variant: <span style='color: green;'>```WA_MouseNoMask = 71```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAMouseNoMask: crate::WidgetAttribute = crate::WidgetAttribute(71);
    /// <i>This attribute has been deprecated.</i> Use <a href="http://doc.qt.io/qt-5/qwidget.html#windowModality-prop">QWidget::windowModality</a> instead. (C++ enum variant: <span style='color: green;'>```WA_GroupLeader = 72```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAGroupLeader: crate::WidgetAttribute = crate::WidgetAttribute(72);
    /// Prohibits mouse events from being propagated to the widget's parent. This attribute is disabled by default. (C++ enum variant: <span style='color: green;'>```WA_NoMousePropagation = 73```</span>)
    #[allow(non_upper_case_globals)]
    pub const WANoMousePropagation: crate::WidgetAttribute = crate::WidgetAttribute(73);
    /// Forces Qt to generate paint events when the mouse enters or leaves the widget. This feature is typically used when implementing custom styles; see the <a href="http://doc.qt.io/qt-5/qtwidgets-widgets-styles-example.html">Styles</a> example for details. (C++ enum variant: <span style='color: green;'>```WA_Hover = 74```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAHover: crate::WidgetAttribute = crate::WidgetAttribute(74);
    /// C++ enum variant: <span style='color: green;'>```WA_InputMethodTransparent = 75```</span>
    #[allow(non_upper_case_globals)]
    pub const WAInputMethodTransparent: crate::WidgetAttribute = crate::WidgetAttribute(75);
    /// Makes Qt quit the application when the last widget with the attribute set has accepted closeEvent(). This behavior can be modified with the QApplication::quitOnLastWindowClosed property. By default this attribute is set for all widgets of type <a href="http://doc.qt.io/qt-5/qt.html#WindowType-enum">Qt::Window</a>. (C++ enum variant: <span style='color: green;'>```WA_QuitOnClose = 76```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAQuitOnClose: crate::WidgetAttribute = crate::WidgetAttribute(76);
    /// Set on a toplevel window when the users changes focus with the keyboard (tab, backtab, or shortcut). (C++ enum variant: <span style='color: green;'>```WA_KeyboardFocusChange = 77```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAKeyboardFocusChange: crate::WidgetAttribute = crate::WidgetAttribute(77);
    /// Allows data from drag and drop operations to be dropped onto the widget (see <a href="http://doc.qt.io/qt-5/qwidget.html#acceptDrops-prop">QWidget::setAcceptDrops</a>()). (C++ enum variant: <span style='color: green;'>```WA_AcceptDrops = 78```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAAcceptDrops: crate::WidgetAttribute = crate::WidgetAttribute(78);
    /// C++ enum variant: <span style='color: green;'>```WA_DropSiteRegistered = 79```</span>
    #[allow(non_upper_case_globals)]
    pub const WADropSiteRegistered: crate::WidgetAttribute = crate::WidgetAttribute(79);
    /// C++ enum variant: <span style='color: green;'>```WA_ForceAcceptDrops = 79```</span>
    #[allow(non_upper_case_globals)]
    pub const WAForceAcceptDrops: crate::WidgetAttribute = crate::WidgetAttribute(79);
    /// Makes a toplevel window inherit font, palette and locale from its parent. (C++ enum variant: <span style='color: green;'>```WA_WindowPropagation = 80```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAWindowPropagation: crate::WidgetAttribute = crate::WidgetAttribute(80);
    /// C++ enum variant: <span style='color: green;'>```WA_NoX11EventCompression = 81```</span>
    #[allow(non_upper_case_globals)]
    pub const WANoX11EventCompression: crate::WidgetAttribute = crate::WidgetAttribute(81);
    /// C++ enum variant: <span style='color: green;'>```WA_TintedBackground = 82```</span>
    #[allow(non_upper_case_globals)]
    pub const WATintedBackground: crate::WidgetAttribute = crate::WidgetAttribute(82);
    /// C++ enum variant: <span style='color: green;'>```WA_X11OpenGLOverlay = 83```</span>
    #[allow(non_upper_case_globals)]
    pub const WAX11OpenGLOverlay: crate::WidgetAttribute = crate::WidgetAttribute(83);
    /// Enables tooltips for inactive windows. (C++ enum variant: <span style='color: green;'>```WA_AlwaysShowToolTips = 84```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAAlwaysShowToolTips: crate::WidgetAttribute = crate::WidgetAttribute(84);
    /// Indicates that the native Carbon size grip should be opaque instead of transparent (the default). This attribute is only applicable to <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and is set by the widget's author. (C++ enum variant: <span style='color: green;'>```WA_MacOpaqueSizeGrip = 85```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAMacOpaqueSizeGrip: crate::WidgetAttribute = crate::WidgetAttribute(85);
    /// Indicates that the widget has a style of its own. This flag is set or cleared by <a href="http://doc.qt.io/qt-5/qwidget.html#setStyle">QWidget::setStyle</a>(). (C++ enum variant: <span style='color: green;'>```WA_SetStyle = 86```</span>)
    #[allow(non_upper_case_globals)]
    pub const WASetStyle: crate::WidgetAttribute = crate::WidgetAttribute(86);
    /// Indicates the locale should be taken into consideration in the widget. (C++ enum variant: <span style='color: green;'>```WA_SetLocale = 87```</span>)
    #[allow(non_upper_case_globals)]
    pub const WASetLocale: crate::WidgetAttribute = crate::WidgetAttribute(87);
    /// Indicates that this widget should get a <a href="http://doc.qt.io/qt-5/qfocusframe.html">QFocusFrame</a> around it. Some widgets draw their own focus halo regardless of this attribute. Not that the <a href="http://doc.qt.io/qt-5/qwidget.html#focusPolicy-prop">QWidget::focusPolicy</a> also plays the main role in whether something is given focus or not, this only controls whether or not this gets the focus frame. This attribute is only applicable to <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>. (C++ enum variant: <span style='color: green;'>```WA_MacShowFocusRect = 88```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAMacShowFocusRect: crate::WidgetAttribute = crate::WidgetAttribute(88);
    /// Indicates the widget should have the normal size for widgets in <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>. This attribute is only applicable to <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>. (C++ enum variant: <span style='color: green;'>```WA_MacNormalSize = 89```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAMacNormalSize: crate::WidgetAttribute = crate::WidgetAttribute(89);
    /// Indicates the widget should have the small size for widgets in <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>. This attribute is only applicable to <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>. (C++ enum variant: <span style='color: green;'>```WA_MacSmallSize = 90```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAMacSmallSize: crate::WidgetAttribute = crate::WidgetAttribute(90);
    /// Indicates the widget should have the mini size for widgets in <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>. This attribute is only applicable to <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>. (C++ enum variant: <span style='color: green;'>```WA_MacMiniSize = 91```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAMacMiniSize: crate::WidgetAttribute = crate::WidgetAttribute(91);
    /// Ignore the layout item rect from the style when laying out this widget with <a href="http://doc.qt.io/qt-5/qlayout.html">QLayout</a>. (C++ enum variant: <span style='color: green;'>```WA_LayoutUsesWidgetRect = 92```</span>)
    #[allow(non_upper_case_globals)]
    pub const WALayoutUsesWidgetRect: crate::WidgetAttribute = crate::WidgetAttribute(92);
    /// Indicates the widget should be drawn using a styled background. (C++ enum variant: <span style='color: green;'>```WA_StyledBackground = 93```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAStyledBackground: crate::WidgetAttribute = crate::WidgetAttribute(93);
    /// This value is obsolete and has no effect. (C++ enum variant: <span style='color: green;'>```WA_MSWindowsUseDirect3D = 94```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAMSWindowsUseDirect3D: crate::WidgetAttribute = crate::WidgetAttribute(94);
    /// C++ enum variant: <span style='color: green;'>```WA_CanHostQMdiSubWindowTitleBar = 95```</span>
    #[allow(non_upper_case_globals)]
    pub const WACanHostQMdiSubWindowTitleBar: crate::WidgetAttribute = crate::WidgetAttribute(95);
    /// On <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, show the tool window even when the application is not active. By default, all tool windows are hidden when the application is inactive. (C++ enum variant: <span style='color: green;'>```WA_MacAlwaysShowToolWindow = 96```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAMacAlwaysShowToolWindow: crate::WidgetAttribute = crate::WidgetAttribute(96);
    /// Indicates that the widget is styled using a <a href="http://doc.qt.io/qt-5/stylesheet.html">style sheet</a>. (C++ enum variant: <span style='color: green;'>```WA_StyleSheet = 97```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAStyleSheet: crate::WidgetAttribute = crate::WidgetAttribute(97);
    /// Show the widget without making it active. (C++ enum variant: <span style='color: green;'>```WA_ShowWithoutActivating = 98```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAShowWithoutActivating: crate::WidgetAttribute = crate::WidgetAttribute(98);
    /// C++ enum variant: <span style='color: green;'>```WA_X11BypassTransientForHint = 99```</span>
    #[allow(non_upper_case_globals)]
    pub const WAX11BypassTransientForHint: crate::WidgetAttribute = crate::WidgetAttribute(99);
    /// Indicates that a native window is created for the widget. Enabling this flag will also force a native window for the widget's ancestors unless Qt::WA_DontCreateNativeAncestors is set. (C++ enum variant: <span style='color: green;'>```WA_NativeWindow = 100```</span>)
    #[allow(non_upper_case_globals)]
    pub const WANativeWindow: crate::WidgetAttribute = crate::WidgetAttribute(100);
    /// Indicates that the widget's ancestors are kept non-native even though the widget itself is native. (C++ enum variant: <span style='color: green;'>```WA_DontCreateNativeAncestors = 101```</span>)
    #[allow(non_upper_case_globals)]
    pub const WADontCreateNativeAncestors: crate::WidgetAttribute = crate::WidgetAttribute(101);
    /// Indicates the widget can choose between alternative sizes for widgets to avoid clipping. This attribute is only applicable to <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>. (C++ enum variant: <span style='color: green;'>```WA_MacVariableSize = 102```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAMacVariableSize: crate::WidgetAttribute = crate::WidgetAttribute(102);
    /// Indicates that the widget is hidden or is not a part of the viewable Desktop. (C++ enum variant: <span style='color: green;'>```WA_DontShowOnScreen = 103```</span>)
    #[allow(non_upper_case_globals)]
    pub const WADontShowOnScreen: crate::WidgetAttribute = crate::WidgetAttribute(103);
    /// Adds _NET_WM_WINDOW_TYPE_DESKTOP to the window's _NET_WM_WINDOW_TYPE X11 window property. See http://standards.freedesktop.org/wm-spec/ for more details. This attribute has no effect on non-X11 platforms. (C++ enum variant: <span style='color: green;'>```WA_X11NetWmWindowTypeDesktop = 104```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAX11NetWmWindowTypeDesktop: crate::WidgetAttribute = crate::WidgetAttribute(104);
    /// Adds _NET_WM_WINDOW_TYPE_DOCK to the window's _NET_WM_WINDOW_TYPE X11 window property. See http://standards.freedesktop.org/wm-spec/ for more details. This attribute has no effect on non-X11 platforms. (C++ enum variant: <span style='color: green;'>```WA_X11NetWmWindowTypeDock = 105```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAX11NetWmWindowTypeDock: crate::WidgetAttribute = crate::WidgetAttribute(105);
    /// Adds _NET_WM_WINDOW_TYPE_TOOLBAR to the window's _NET_WM_WINDOW_TYPE X11 window property. See http://standards.freedesktop.org/wm-spec/ for more details. This attribute has no effect on non-X11 platforms. <b>Note</b>: Qt automatically sets this attribute for <a href="http://doc.qt.io/qt-5/qtoolbar.html">QToolBar</a>. (C++ enum variant: <span style='color: green;'>```WA_X11NetWmWindowTypeToolBar = 106```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAX11NetWmWindowTypeToolBar: crate::WidgetAttribute = crate::WidgetAttribute(106);
    /// Adds _NET_WM_WINDOW_TYPE_MENU to the window's _NET_WM_WINDOW_TYPE X11 window property. See http://standards.freedesktop.org/wm-spec/ for more details. This attribute has no effect on non-X11 platforms. <b>Note</b>: Qt automatically sets this attribute for <a href="http://doc.qt.io/qt-5/qmenu.html">QMenu</a> when torn-off. (C++ enum variant: <span style='color: green;'>```WA_X11NetWmWindowTypeMenu = 107```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAX11NetWmWindowTypeMenu: crate::WidgetAttribute = crate::WidgetAttribute(107);
    /// Adds _NET_WM_WINDOW_TYPE_UTILITY to the window's _NET_WM_WINDOW_TYPE X11 window property. See http://standards.freedesktop.org/wm-spec/ for more details. This attribute has no effect on non-X11 platforms. <b>Note</b>: Qt automatically sets this attribute for the <a href="http://doc.qt.io/qt-5/qt.html#WindowType-enum">Qt::Tool</a> window type. (C++ enum variant: <span style='color: green;'>```WA_X11NetWmWindowTypeUtility = 108```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAX11NetWmWindowTypeUtility: crate::WidgetAttribute = crate::WidgetAttribute(108);
    /// Adds _NET_WM_WINDOW_TYPE_SPLASH to the window's _NET_WM_WINDOW_TYPE X11 window property. See http://standards.freedesktop.org/wm-spec/ for more details. This attribute has no effect on non-X11 platforms. <b>Note</b>: Qt automatically sets this attribute for the <a href="http://doc.qt.io/qt-5/qt.html#WindowType-enum">Qt::SplashScreen</a> window type. (C++ enum variant: <span style='color: green;'>```WA_X11NetWmWindowTypeSplash = 109```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAX11NetWmWindowTypeSplash: crate::WidgetAttribute = crate::WidgetAttribute(109);
    /// Adds _NET_WM_WINDOW_TYPE_DIALOG to the window's _NET_WM_WINDOW_TYPE X11 window property. See http://standards.freedesktop.org/wm-spec/ for more details. This attribute has no effect on non-X11 platforms. <b>Note</b>: Qt automatically sets this attribute for the <a href="http://doc.qt.io/qt-5/qt.html#WindowType-enum">Qt::Dialog</a> and <a href="http://doc.qt.io/qt-5/qt.html#WindowType-enum">Qt::Sheet</a> window types. (C++ enum variant: <span style='color: green;'>```WA_X11NetWmWindowTypeDialog = 110```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAX11NetWmWindowTypeDialog: crate::WidgetAttribute = crate::WidgetAttribute(110);
    /// Adds _NET_WM_WINDOW_TYPE_DROPDOWN_MENU to the window's _NET_WM_WINDOW_TYPE X11 window property. See http://standards.freedesktop.org/wm-spec/ for more details. This attribute has no effect on non-X11 platforms. <b>Note</b>: Qt automatically sets this attribute for <a href="http://doc.qt.io/qt-5/qmenu.html">QMenu</a> objects added to a <a href="http://doc.qt.io/qt-5/qmenubar.html">QMenuBar</a>. (C++ enum variant: <span style='color: green;'>```WA_X11NetWmWindowTypeDropDownMenu = 111```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAX11NetWmWindowTypeDropDownMenu: crate::WidgetAttribute =
        crate::WidgetAttribute(111);
    /// Adds _NET_WM_WINDOW_TYPE_POPUP_MENU to the window's _NET_WM_WINDOW_TYPE X11 window property. See http://standards.freedesktop.org/wm-spec/ for more details. This attribute has no effect on non-X11 platforms. <b>Note</b>: Qt automatically sets this attribute for <a href="http://doc.qt.io/qt-5/qmenu.html">QMenu</a>. (C++ enum variant: <span style='color: green;'>```WA_X11NetWmWindowTypePopupMenu = 112```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAX11NetWmWindowTypePopupMenu: crate::WidgetAttribute = crate::WidgetAttribute(112);
    /// Adds _NET_WM_WINDOW_TYPE_TOOLTIP to the window's _NET_WM_WINDOW_TYPE X11 window property. See http://standards.freedesktop.org/wm-spec/ for more details. This attribute has no effect on non-X11 platforms. <b>Note</b>: Qt automatically sets this attribute for the <a href="http://doc.qt.io/qt-5/qt.html#WindowType-enum">Qt::ToolTip</a> window type. (C++ enum variant: <span style='color: green;'>```WA_X11NetWmWindowTypeToolTip = 113```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAX11NetWmWindowTypeToolTip: crate::WidgetAttribute = crate::WidgetAttribute(113);
    /// Adds _NET_WM_WINDOW_TYPE_NOTIFICATION to the window's _NET_WM_WINDOW_TYPE X11 window property. See http://standards.freedesktop.org/wm-spec/ for more details. This attribute has no effect on non-X11 platforms. (C++ enum variant: <span style='color: green;'>```WA_X11NetWmWindowTypeNotification = 114```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAX11NetWmWindowTypeNotification: crate::WidgetAttribute =
        crate::WidgetAttribute(114);
    /// Adds _NET_WM_WINDOW_TYPE_COMBO to the window's _NET_WM_WINDOW_TYPE X11 window property. See http://standards.freedesktop.org/wm-spec/ for more details. This attribute has no effect on non-X11 platforms. <b>Note</b>: Qt automatically sets this attribute for the <a href="http://doc.qt.io/qt-5/qcombobox.html">QComboBox</a> pop-up. (C++ enum variant: <span style='color: green;'>```WA_X11NetWmWindowTypeCombo = 115```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAX11NetWmWindowTypeCombo: crate::WidgetAttribute = crate::WidgetAttribute(115);
    /// Adds _NET_WM_WINDOW_TYPE_DND to the window's _NET_WM_WINDOW_TYPE X11 window property. See http://standards.freedesktop.org/wm-spec/ for more details. This attribute has no effect on non-X11 platforms. <b>Note</b>: Qt automatically sets this attribute on the feedback widget used during a drag. (C++ enum variant: <span style='color: green;'>```WA_X11NetWmWindowTypeDND = 116```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAX11NetWmWindowTypeDND: crate::WidgetAttribute = crate::WidgetAttribute(116);
    /// Enables resolution independence aware mode on Mac when using Carbon. This attribute has no effect on Cocoa. The attribute is off by default and can be enabled on a per-window basis. (C++ enum variant: <span style='color: green;'>```WA_MacFrameworkScaled = 117```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAMacFrameworkScaled: crate::WidgetAttribute = crate::WidgetAttribute(117);
    /// C++ enum variant: <span style='color: green;'>```WA_SetWindowModality = 118```</span>
    #[allow(non_upper_case_globals)]
    pub const WASetWindowModality: crate::WidgetAttribute = crate::WidgetAttribute(118);
    /// C++ enum variant: <span style='color: green;'>```WA_WState_WindowOpacitySet = 119```</span>
    #[allow(non_upper_case_globals)]
    pub const WAWStateWindowOpacitySet: crate::WidgetAttribute = crate::WidgetAttribute(119);
    /// Indicates that the widget should have a translucent background, i.e., any non-opaque regions of the widgets will be translucent because the widget will have an alpha channel. Setting this flag causes WA_NoSystemBackground to be set. On Windows the widget also needs the <a href="http://doc.qt.io/qt-5/qt.html#WindowType-enum">Qt::FramelessWindowHint</a> window flag to be set. This flag is set or cleared by the widget's author. (C++ enum variant: <span style='color: green;'>```WA_TranslucentBackground = 120```</span>)
    #[allow(non_upper_case_globals)]
    pub const WATranslucentBackground: crate::WidgetAttribute = crate::WidgetAttribute(120);
    /// Allows touch events (see <a href="http://doc.qt.io/qt-5/qtouchevent.html">QTouchEvent</a>) to be sent to the widget. Must be set on all widgets that can handle touch events. Without this attribute set, events from a touch device will be sent as mouse events. (C++ enum variant: <span style='color: green;'>```WA_AcceptTouchEvents = 121```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAAcceptTouchEvents: crate::WidgetAttribute = crate::WidgetAttribute(121);
    /// C++ enum variant: <span style='color: green;'>```WA_WState_AcceptedTouchBeginEvent = 122```</span>
    #[allow(non_upper_case_globals)]
    pub const WAWStateAcceptedTouchBeginEvent: crate::WidgetAttribute = crate::WidgetAttribute(122);
    /// Allows touchpad single touch events to be sent to the widget. (C++ enum variant: <span style='color: green;'>```WA_TouchPadAcceptSingleTouchEvents = 123```</span>)
    #[allow(non_upper_case_globals)]
    pub const WATouchPadAcceptSingleTouchEvents: crate::WidgetAttribute =
        crate::WidgetAttribute(123);
    /// Asks the window manager to not give focus to this top level window. This attribute has no effect on non-X11 platforms. (C++ enum variant: <span style='color: green;'>```WA_X11DoNotAcceptFocus = 126```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAX11DoNotAcceptFocus: crate::WidgetAttribute = crate::WidgetAttribute(126);
    /// C++ enum variant: <span style='color: green;'>```WA_MacNoShadow = 127```</span>
    #[allow(non_upper_case_globals)]
    pub const WAMacNoShadow: crate::WidgetAttribute = crate::WidgetAttribute(127);
    /// Since Qt 5.4, this value forces <a href="http://doc.qt.io/qt-5/qopenglwidget.html">QOpenGLWidget</a> and <a href="http://doc.qt.io/qt-5/qquickwidget.html">QQuickWidget</a> to be drawn last, on top of other widgets. Ignored for other type of widgets. Setting this attribute breaks the stacking order, but allows having a semi-transparent OpenGL widget with other widgets visible underneath. It is strongly recommended to call update() on the widget's top-level window after enabling or disabling this attribute. (C++ enum variant: <span style='color: green;'>```WA_AlwaysStackOnTop = 128```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAAlwaysStackOnTop: crate::WidgetAttribute = crate::WidgetAttribute(128);
    /// Indicates that the widget has tablet tracking enabled. See <a href="http://doc.qt.io/qt-5/qwidget.html#tabletTracking-prop">QWidget::tabletTracking</a>. (C++ enum variant: <span style='color: green;'>```WA_TabletTracking = 129```</span>)
    #[allow(non_upper_case_globals)]
    pub const WATabletTracking: crate::WidgetAttribute = crate::WidgetAttribute(129);
    /// A <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> respects the safe area margins of a window by incorporating the margins into its contents' margins by default. This means, that a <a href="http://doc.qt.io/qt-5/qlayout.html">QLayout</a> will use the content area of a widget for its layout, unless the Qt::WA_LayoutOnEntireRect attribute is set. This along with a contents margin of 0 can be used on the actual layout, to allow for example a background image to underlay the status bar and other system areas on an iOS device, while still allowing child widgets of that background to be inset based on the safe area. (C++ enum variant: <span style='color: green;'>```WA_ContentsMarginsRespectsSafeArea = 130```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAContentsMarginsRespectsSafeArea: crate::WidgetAttribute =
        crate::WidgetAttribute(130);
    /// C++ enum variant: <span style='color: green;'>```WA_AttributeCount = 131```</span>
    #[allow(non_upper_case_globals)]
    pub const WAAttributeCount: crate::WidgetAttribute = crate::WidgetAttribute(131);
    /// Indicates that the widget appearance was modified by a <a href="http://doc.qt.io/qt-5/stylesheet.html">style sheet</a>. WA_StyleSheet will also be set. (C++ enum variant: <span style='color: green;'>```WA_StyleSheetTarget = 131```</span>)
    #[allow(non_upper_case_globals)]
    pub const WAStyleSheetTarget: crate::WidgetAttribute = crate::WidgetAttribute(131);
    /// C++ enum variant: <span style='color: green;'>```WA_AttributeCount = 132```</span>
    #[allow(non_upper_case_globals)]
    pub const WAAttributeCount2: crate::WidgetAttribute = crate::WidgetAttribute(132);
}

/// <p>This enum describes attributes that change the behavior of application-wide features. These are enabled and disabled using <a href="http://doc.qt.io/qt-5/qcoreapplication.html#setAttribute">QCoreApplication::setAttribute</a>(), and can be tested for with <a href="http://doc.qt.io/qt-5/qcoreapplication.html#testAttribute">QCoreApplication::testAttribute</a>().</p>
///
/// C++ enum: <span style='color: green;'>```Qt::ApplicationAttribute```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#ApplicationAttribute-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes attributes that change the behavior of application-wide features. These are enabled and disabled using <a href="http://doc.qt.io/qt-5/qcoreapplication.html#setAttribute">QCoreApplication::setAttribute</a>(), and can be tested for with <a href="http://doc.qt.io/qt-5/qcoreapplication.html#testAttribute">QCoreApplication::testAttribute</a>().</p>
///
/// <p>The following values are deprecated or obsolete:</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct ApplicationAttribute(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for ApplicationAttribute {
    fn from(value: ::std::os::raw::c_int) -> Self {
        ApplicationAttribute(value)
    }
}

impl From<ApplicationAttribute> for ::std::os::raw::c_int {
    fn from(value: ApplicationAttribute) -> Self {
        value.0
    }
}

impl ApplicationAttribute {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl ApplicationAttribute {
    /// This attribute is no longer fully supported in Qt 5. It ensures that widgets are created as soon as they are constructed. By default, resources for widgets are allocated on demand to improve efficiency and minimize resource usage. Setting or clearing this attribute affects widgets constructed after the change. Setting it tells Qt to create toplevel windows immediately. Therefore, if it is important to minimize resource consumption, do not set this attribute. (C++ enum variant: <span style='color: green;'>```AA_ImmediateWidgetCreation = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const AAImmediateWidgetCreation: crate::ApplicationAttribute =
        crate::ApplicationAttribute(0);
    /// This value is obsolete and has no effect. (C++ enum variant: <span style='color: green;'>```AA_MSWindowsUseDirect3DByDefault = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const AAMSWindowsUseDirect3DByDefault: crate::ApplicationAttribute =
        crate::ApplicationAttribute(1);
    /// Actions with the Icon property won't be shown in any menus unless specifically set by the <a href="http://doc.qt.io/qt-5/qaction.html#iconVisibleInMenu-prop">QAction::iconVisibleInMenu</a> property. Menus that are currently open or menus already created in the native <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> menubar <i>may not</i> pick up a change in this attribute. Changes in the <a href="http://doc.qt.io/qt-5/qaction.html#iconVisibleInMenu-prop">QAction::iconVisibleInMenu</a> property will always be picked up. (C++ enum variant: <span style='color: green;'>```AA_DontShowIconsInMenus = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const AADontShowIconsInMenus: crate::ApplicationAttribute = crate::ApplicationAttribute(2);
    /// Ensures that widgets have native windows. (C++ enum variant: <span style='color: green;'>```AA_NativeWindows = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const AANativeWindows: crate::ApplicationAttribute = crate::ApplicationAttribute(3);
    /// Ensures that siblings of native widgets stay non-native unless specifically set by the <a href="http://doc.qt.io/qt-5/qt.html#WidgetAttribute-enum">Qt::WA_NativeWindow</a> attribute. (C++ enum variant: <span style='color: green;'>```AA_DontCreateNativeWidgetSiblings = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const AADontCreateNativeWidgetSiblings: crate::ApplicationAttribute =
        crate::ApplicationAttribute(4);
    /// Indicates that Qt is used to author a plugin. Depending on the operating system, it suppresses specific initializations that do not necessarily make sense in the plugin case. For example on <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, this includes avoiding loading our nib for the main menu and not taking possession of the native menu bar. Setting this attribute to true will also set the AA_DontUseNativeMenuBar attribute to true. It also disables native event filters. This attribute has been added in Qt 5.7. It must be set before <a href="http://doc.qt.io/qt-5/qguiapplication.html">Q</a>(Gui)Application is constructed. (C++ enum variant: <span style='color: green;'>```AA_PluginApplication = 5```</span>)
    #[allow(non_upper_case_globals)]
    pub const AAPluginApplication: crate::ApplicationAttribute = crate::ApplicationAttribute(5);
    /// This attribute has been deprecated. Use AA_PluginApplication instead. (C++ enum variant: <span style='color: green;'>```AA_MacPluginApplication = 5```</span>)
    #[allow(non_upper_case_globals)]
    pub const AAMacPluginApplication: crate::ApplicationAttribute = crate::ApplicationAttribute(5);
    /// All menubars created while this attribute is set to true won't be used as a native menubar (e.g, the menubar at the top of the main screen on <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>). (C++ enum variant: <span style='color: green;'>```AA_DontUseNativeMenuBar = 6```</span>)
    #[allow(non_upper_case_globals)]
    pub const AADontUseNativeMenuBar: crate::ApplicationAttribute = crate::ApplicationAttribute(6);
    /// On <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> by default, Qt swaps the Control and Meta (Command) keys (i.e., whenever Control is pressed, Qt sends Meta, and whenever Meta is pressed Control is sent). When this attribute is true, Qt will not do the flip. <a href="http://doc.qt.io/qt-5/qkeysequence.html#StandardKey-enum">QKeySequence::StandardKey</a> will also flip accordingly (i.e., <a href="http://doc.qt.io/qt-5/qkeysequence.html#StandardKey-enum">QKeySequence::Copy</a> will be Command+C on the keyboard regardless of the value set, though what is output for <a href="http://doc.qt.io/qt-5/qkeysequence.html#toString">QKeySequence::toString</a>() will be different). (C++ enum variant: <span style='color: green;'>```AA_MacDontSwapCtrlAndMeta = 7```</span>)
    #[allow(non_upper_case_globals)]
    pub const AAMacDontSwapCtrlAndMeta: crate::ApplicationAttribute =
        crate::ApplicationAttribute(7);
    /// Assume the screen has a resolution of 96 DPI rather than using the OS-provided resolution. This will cause font rendering to be consistent in pixels-per-point across devices rather than defining 1 point as 1/72 inch. (C++ enum variant: <span style='color: green;'>```AA_Use96Dpi = 8```</span>)
    #[allow(non_upper_case_globals)]
    pub const AAUse96Dpi: crate::ApplicationAttribute = crate::ApplicationAttribute(8);
    /// This value is obsolete and has no effect. (C++ enum variant: <span style='color: green;'>```AA_X11InitThreads = 10```</span>)
    #[allow(non_upper_case_globals)]
    pub const AAX11InitThreads: crate::ApplicationAttribute = crate::ApplicationAttribute(10);
    /// All mouse events that are not accepted by the application will be translated to touch events instead. (C++ enum variant: <span style='color: green;'>```AA_SynthesizeTouchForUnhandledMouseEvents = 11```</span>)
    #[allow(non_upper_case_globals)]
    pub const AASynthesizeTouchForUnhandledMouseEvents: crate::ApplicationAttribute =
        crate::ApplicationAttribute(11);
    /// All touch events that are not accepted by the application will be translated to left button mouse events instead. This attribute is enabled by default. (C++ enum variant: <span style='color: green;'>```AA_SynthesizeMouseForUnhandledTouchEvents = 12```</span>)
    #[allow(non_upper_case_globals)]
    pub const AASynthesizeMouseForUnhandledTouchEvents: crate::ApplicationAttribute =
        crate::ApplicationAttribute(12);
    /// Make <a href="http://doc.qt.io/qt-5/qicon.html#pixmap-3">QIcon::pixmap</a>() generate high-dpi pixmaps that can be larger than the requested size. Such pixmaps will have <a href="http://doc.qt.io/qt-5/qpixmap.html#devicePixelRatio">devicePixelRatio()</a> set to a value higher than 1. After setting this attribute, application code that uses pixmap sizes in layout geometry calculations should typically divide by <a href="http://doc.qt.io/qt-5/qpixmap.html#devicePixelRatio">devicePixelRatio()</a> to get device-independent layout geometry. (C++ enum variant: <span style='color: green;'>```AA_UseHighDpiPixmaps = 13```</span>)
    #[allow(non_upper_case_globals)]
    pub const AAUseHighDpiPixmaps: crate::ApplicationAttribute = crate::ApplicationAttribute(13);
    /// Make top-level widgets use pure raster surfaces, and do not support non-native GL-based child widgets. (C++ enum variant: <span style='color: green;'>```AA_ForceRasterWidgets = 14```</span>)
    #[allow(non_upper_case_globals)]
    pub const AAForceRasterWidgets: crate::ApplicationAttribute = crate::ApplicationAttribute(14);
    /// Forces the usage of desktop OpenGL (for example, <i>opengl32.dll</i> or <i>libGL.so</i>) on platforms that use dynamic loading of the OpenGL implementation. This value has been added in Qt 5.3. This attribute must be set before <a href="http://doc.qt.io/qt-5/qguiapplication.html">Q</a>(Gui)Application is constructed. (C++ enum variant: <span style='color: green;'>```AA_UseDesktopOpenGL = 15```</span>)
    #[allow(non_upper_case_globals)]
    pub const AAUseDesktopOpenGL: crate::ApplicationAttribute = crate::ApplicationAttribute(15);
    /// Forces the usage of OpenGL ES 2.0 or higher on platforms that use dynamic loading of the OpenGL implementation. This value has been added in Qt 5.3. This attribute must be set before <a href="http://doc.qt.io/qt-5/qguiapplication.html">Q</a>(Gui)Application is constructed. (C++ enum variant: <span style='color: green;'>```AA_UseOpenGLES = 16```</span>)
    #[allow(non_upper_case_globals)]
    pub const AAUseOpenGLES: crate::ApplicationAttribute = crate::ApplicationAttribute(16);
    /// Forces the usage of a software based OpenGL implementation on platforms that use dynamic loading of the OpenGL implementation. This will typically be a patched build of <a href="http://www.mesa3d.org/llvmpipe.html">Mesa llvmpipe</a>, providing OpenGL 2.1. The value may have no effect if no such OpenGL implementation is available. The default name of this library is <code>opengl32sw.dll</code> and can be overridden by setting the environment variable <i>QT_OPENGL_DLL</i>. See the platform-specific pages, for instance <a href="http://doc.qt.io/qt-5/windows-support.html">Qt for Windows</a>, for more information. This value has been added in Qt 5.4. This attribute must be set before <a href="http://doc.qt.io/qt-5/qguiapplication.html">Q</a>(Gui)Application is constructed. (C++ enum variant: <span style='color: green;'>```AA_UseSoftwareOpenGL = 17```</span>)
    #[allow(non_upper_case_globals)]
    pub const AAUseSoftwareOpenGL: crate::ApplicationAttribute = crate::ApplicationAttribute(17);
    /// Enables resource sharing between the OpenGL contexts used by classes like <a href="http://doc.qt.io/qt-5/qopenglwidget.html">QOpenGLWidget</a> and <a href="http://doc.qt.io/qt-5/qquickwidget.html">QQuickWidget</a>. This allows sharing OpenGL resources, like textures, between <a href="http://doc.qt.io/qt-5/qopenglwidget.html">QOpenGLWidget</a> instances that belong to different top-level windows. This value has been added in Qt 5.4. This attribute must be set before <a href="http://doc.qt.io/qt-5/qguiapplication.html">Q</a>(Gui)Application is constructed. (C++ enum variant: <span style='color: green;'>```AA_ShareOpenGLContexts = 18```</span>)
    #[allow(non_upper_case_globals)]
    pub const AAShareOpenGLContexts: crate::ApplicationAttribute = crate::ApplicationAttribute(18);
    /// Indicates whether a palette was explicitly set on the <a href="http://doc.qt.io/qt-5/qguiapplication.html">Q</a>(Gui)Application. This value has been added in Qt 5.5. (C++ enum variant: <span style='color: green;'>```AA_SetPalette = 19```</span>)
    #[allow(non_upper_case_globals)]
    pub const AASetPalette: crate::ApplicationAttribute = crate::ApplicationAttribute(19);
    /// Enables high-DPI scaling in Qt on supported platforms (see also <a href="http://doc.qt.io/qt-5/highdpi.html#high-dpi-displays">High DPI Displays</a>). Supported platforms are X11, Windows and Android. Enabling makes Qt scale the main (device independent) coordinate system according to display scale factors provided by the operating system. This corresponds to setting the QT_AUTO_SCREEN​_SCALE_FACTOR environment variable to 1. This value has been added in Qt 5.6. This attribute must be set before Q(Gui)Application is constructed. (C++ enum variant: <span style='color: green;'>```AA_EnableHighDpiScaling = 20```</span>)
    #[allow(non_upper_case_globals)]
    pub const AAEnableHighDpiScaling: crate::ApplicationAttribute = crate::ApplicationAttribute(20);
    /// Disables high-DPI scaling in Qt, exposing window system coordinates. Note that the window system may do its own scaling, so this does not guarantee that <a href="http://doc.qt.io/qt-5/qpaintdevice.html#devicePixelRatio">QPaintDevice::devicePixelRatio</a>() will be equal to 1. In addition, scale factors set by QT_SCALE_FACTOR will not be affected. This corresponds to setting the QT_AUTO_SCREEN​_SCALE_FACTOR environment variable to 0. This value has been added in Qt 5.6. This attribute must be set before Q(Gui)Application is constructed. (C++ enum variant: <span style='color: green;'>```AA_DisableHighDpiScaling = 21```</span>)
    #[allow(non_upper_case_globals)]
    pub const AADisableHighDpiScaling: crate::ApplicationAttribute =
        crate::ApplicationAttribute(21);
    /// By default, Qt Style Sheets disable regular <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> palette and font propagation. When this flag is enabled, font and palette changes propagate as though the user had manually called the corresponding <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> methods. See <a href="http://doc.qt.io/qt-5/stylesheet-syntax.html#inheritance">The Style Sheet Syntax - Inheritance</a> for more details. This value has been added in Qt 5.7. (C++ enum variant: <span style='color: green;'>```AA_UseStyleSheetPropagationInWidgetStyles = 22```</span>)
    #[allow(non_upper_case_globals)]
    pub const AAUseStyleSheetPropagationInWidgetStyles: crate::ApplicationAttribute =
        crate::ApplicationAttribute(22);
    /// All dialogs created while this attribute is set to true won't use the native dialogs provided by the platform. This value has been added in Qt 5.7. (C++ enum variant: <span style='color: green;'>```AA_DontUseNativeDialogs = 23```</span>)
    #[allow(non_upper_case_globals)]
    pub const AADontUseNativeDialogs: crate::ApplicationAttribute = crate::ApplicationAttribute(23);
    /// All tablet events that are not accepted by the application will be translated to mouse events instead. This attribute is enabled by default. This value has been added in Qt 5.7. (C++ enum variant: <span style='color: green;'>```AA_SynthesizeMouseForUnhandledTabletEvents = 24```</span>)
    #[allow(non_upper_case_globals)]
    pub const AASynthesizeMouseForUnhandledTabletEvents: crate::ApplicationAttribute =
        crate::ApplicationAttribute(24);
    /// Enables compression of certain frequent events. On the X11 windowing system, the default value is true, which means that <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::MouseMove</a>, <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::TouchUpdate</a>, and changes in window size and position will be combined whenever they occur more frequently than the application handles them, so that they don't accumulate and overwhelm the application later. On other platforms, the default is false. (In the future, the compression feature may be implemented across platforms.) You can test the attribute to see whether compression is enabled. If your application needs to handle all events with no compression, you can unset this attribute. This value has been added in Qt 5.7. (C++ enum variant: <span style='color: green;'>```AA_CompressHighFrequencyEvents = 25```</span>)
    #[allow(non_upper_case_globals)]
    pub const AACompressHighFrequencyEvents: crate::ApplicationAttribute =
        crate::ApplicationAttribute(25);
    /// When making a context current using <a href="http://doc.qt.io/qt-5/qopenglcontext.html">QOpenGLContext</a>, do not check that the <a href="http://doc.qt.io/qt-5/qobject.html#thread-affinity">QObject thread affinity</a> of the <a href="http://doc.qt.io/qt-5/qopenglcontext.html">QOpenGLContext</a> object is the same thread calling <a href="http://doc.qt.io/qt-5/qopenglcontext.html#makeCurrent">makeCurrent()</a>. This value has been added in Qt 5.8. (C++ enum variant: <span style='color: green;'>```AA_DontCheckOpenGLContextThreadAffinity = 26```</span>)
    #[allow(non_upper_case_globals)]
    pub const AADontCheckOpenGLContextThreadAffinity: crate::ApplicationAttribute =
        crate::ApplicationAttribute(26);
    /// Disables caching of shader program binaries on disk. By default Qt Quick, <a href="http://doc.qt.io/qt-5/qpainter.html">QPainter</a>'s OpenGL backend, and any application using <a href="http://doc.qt.io/qt-5/qopenglshaderprogram.html">QOpenGLShaderProgram</a> with one of its <i>addCacheableShaderFromSource</i> overloads will employ a disk-based <a href="http://doc.qt.io/qt-5/qopenglshaderprogram.html#caching-program-binaries">program binary cache</a> in either the shared or per-process cache storage location, on systems that support <i>glProgramBinary()</i>. In the unlikely event of this being problematic, set this attribute to disable all disk-based caching of shaders. (C++ enum variant: <span style='color: green;'>```AA_DisableShaderDiskCache = 27```</span>)
    #[allow(non_upper_case_globals)]
    pub const AADisableShaderDiskCache: crate::ApplicationAttribute =
        crate::ApplicationAttribute(27);
    /// C++ enum variant: <span style='color: green;'>```AA_AttributeCount = 28```</span>
    #[allow(non_upper_case_globals)]
    pub const AAAttributeCount: crate::ApplicationAttribute = crate::ApplicationAttribute(28);
    /// Actions with the Shortcut property won't be shown in any shortcut menus unless specifically set by the <a href="http://doc.qt.io/qt-5/qaction.html#shortcutVisibleInContextMenu-prop">QAction::shortcutVisibleInContextMenu</a> property. This value was added in Qt 5.10. (C++ enum variant: <span style='color: green;'>```AA_DontShowShortcutsInContextMenus = 28```</span>)
    #[allow(non_upper_case_globals)]
    pub const AADontShowShortcutsInContextMenus: crate::ApplicationAttribute =
        crate::ApplicationAttribute(28);
    /// Enables compression of input events from tablet devices. Notice that AA_CompressHighFrequencyEvents must be true for events compression to be enabled, and that this flag extends the former to tablet events. Its default value is false. This value was added in Qt 5.10. (C++ enum variant: <span style='color: green;'>```AA_CompressTabletEvents = 29```</span>)
    #[allow(non_upper_case_globals)]
    pub const AACompressTabletEvents: crate::ApplicationAttribute = crate::ApplicationAttribute(29);
    /// Disables the <a href="http://doc.qt.io/qt-5/qt.html#WindowType-enum">WindowContextHelpButtonHint</a> by default on <a href="http://doc.qt.io/qt-5/qt.html#WindowType-enum">Qt::Sheet</a> and <a href="http://doc.qt.io/qt-5/qt.html#WindowType-enum">Qt::Dialog</a> widgets. This hides the <b>?</b> button on Windows, which only makes sense if you use <a href="http://doc.qt.io/qt-5/qwhatsthis.html">QWhatsThis</a> functionality. This value was added in Qt 5.10. In Qt 6, <a href="http://doc.qt.io/qt-5/qt.html#WindowType-enum">WindowContextHelpButtonHint</a> will not be set by default. (C++ enum variant: <span style='color: green;'>```AA_DisableWindowContextHelpButton = 30```</span>)
    #[allow(non_upper_case_globals)]
    pub const AADisableWindowContextHelpButton: crate::ApplicationAttribute =
        crate::ApplicationAttribute(30);
    /// C++ enum variant: <span style='color: green;'>```AA_AttributeCount = 31```</span>
    #[allow(non_upper_case_globals)]
    pub const AAAttributeCount2: crate::ApplicationAttribute = crate::ApplicationAttribute(31);
}

/// <p>The options marked "(default)" are set if no other values from the list are included (since the defaults are zero):</p>
///
/// C++ enum: <span style='color: green;'>```Qt::ImageConversionFlag```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#ImageConversionFlag-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The options marked "(default)" are set if no other values from the list are included (since the defaults are zero):</p>
/// <p>Color/Mono preference (ignored for <a href="http://doc.qt.io/qt-5/qbitmap.html">QBitmap</a>):</p>
///
/// <p>Dithering mode preference:</p>
///
/// <p>Dithering mode preference for 1-bit alpha masks:</p>
///
/// <p>Color matching versus dithering preference:</p>
///
/// <p>The ImageConversionFlags type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;ImageConversionFlag&gt;. It stores an OR combination of ImageConversionFlag values.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct ImageConversionFlag(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for ImageConversionFlag {
    fn from(value: ::std::os::raw::c_int) -> Self {
        ImageConversionFlag(value)
    }
}

impl From<ImageConversionFlag> for ::std::os::raw::c_int {
    fn from(value: ImageConversionFlag) -> Self {
        value.0
    }
}

impl ImageConversionFlag {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl ImageConversionFlag {
    /// C++ enum variant: <span style='color: green;'>```ColorMode_Mask = 3```</span>
    #[allow(non_upper_case_globals)]
    pub const ColorModeMask: crate::ImageConversionFlag = crate::ImageConversionFlag(3);
    /// (default) - If the image has <a href="http://doc.qt.io/qt-5/qimage.html#depth">depth</a> 1 and contains only black and white pixels, the pixmap becomes monochrome. (C++ enum variant: <span style='color: green;'>```AutoColor = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const AutoColor: crate::ImageConversionFlag = crate::ImageConversionFlag(0);
    /// The pixmap is dithered/converted to the <a href="http://doc.qt.io/qt-5/qpixmap.html#defaultDepth">native display depth</a>. (C++ enum variant: <span style='color: green;'>```ColorOnly = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const ColorOnly: crate::ImageConversionFlag = crate::ImageConversionFlag(3);
    /// The pixmap becomes monochrome. If necessary, it is dithered using the chosen dithering algorithm. (C++ enum variant: <span style='color: green;'>```MonoOnly = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const MonoOnly: crate::ImageConversionFlag = crate::ImageConversionFlag(2);
    /// C++ enum variant: <span style='color: green;'>```AlphaDither_Mask = 12```</span>
    #[allow(non_upper_case_globals)]
    pub const AlphaDitherMask: crate::ImageConversionFlag = crate::ImageConversionFlag(12);
    /// (default) - No dithering. (C++ enum variant: <span style='color: green;'>```ThresholdAlphaDither = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const ThresholdAlphaDither: crate::ImageConversionFlag = crate::ImageConversionFlag(0);
    /// A faster, ordered dither. (C++ enum variant: <span style='color: green;'>```OrderedAlphaDither = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const OrderedAlphaDither: crate::ImageConversionFlag = crate::ImageConversionFlag(4);
    /// A high-quality dither using error diffusion. (C++ enum variant: <span style='color: green;'>```DiffuseAlphaDither = 8```</span>)
    #[allow(non_upper_case_globals)]
    pub const DiffuseAlphaDither: crate::ImageConversionFlag = crate::ImageConversionFlag(8);
    /// C++ enum variant: <span style='color: green;'>```NoAlpha = 12```</span>
    #[allow(non_upper_case_globals)]
    pub const NoAlpha: crate::ImageConversionFlag = crate::ImageConversionFlag(12);
    /// C++ enum variant: <span style='color: green;'>```Dither_Mask = 48```</span>
    #[allow(non_upper_case_globals)]
    pub const DitherMask: crate::ImageConversionFlag = crate::ImageConversionFlag(48);
    /// (default) - A high-quality dither using error diffusion. (C++ enum variant: <span style='color: green;'>```DiffuseDither = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const DiffuseDither: crate::ImageConversionFlag = crate::ImageConversionFlag(0);
    /// A faster, ordered dither. (C++ enum variant: <span style='color: green;'>```OrderedDither = 16```</span>)
    #[allow(non_upper_case_globals)]
    pub const OrderedDither: crate::ImageConversionFlag = crate::ImageConversionFlag(16);
    /// No dithering; closest color is used. (C++ enum variant: <span style='color: green;'>```ThresholdDither = 32```</span>)
    #[allow(non_upper_case_globals)]
    pub const ThresholdDither: crate::ImageConversionFlag = crate::ImageConversionFlag(32);
    /// C++ enum variant: <span style='color: green;'>```DitherMode_Mask = 192```</span>
    #[allow(non_upper_case_globals)]
    pub const DitherModeMask: crate::ImageConversionFlag = crate::ImageConversionFlag(192);
    /// (default) - Only dither when down-converting to 1 or 8-bit indexed formats. (C++ enum variant: <span style='color: green;'>```AutoDither = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const AutoDither: crate::ImageConversionFlag = crate::ImageConversionFlag(0);
    /// Always dither images when converting to smaller color-spaces. (C++ enum variant: <span style='color: green;'>```PreferDither = 64```</span>)
    #[allow(non_upper_case_globals)]
    pub const PreferDither: crate::ImageConversionFlag = crate::ImageConversionFlag(64);
    /// Only dither to indexed formats if the source image uses more different colors than the size of the color table of the destination format. (C++ enum variant: <span style='color: green;'>```AvoidDither = 128```</span>)
    #[allow(non_upper_case_globals)]
    pub const AvoidDither: crate::ImageConversionFlag = crate::ImageConversionFlag(128);
    /// Do not check whether the image contains non-opaque pixels. Use this if you know that the image is semi-transparent and you want to avoid the overhead of checking the pixels in the image until a non-opaque pixel is found, or if you want the pixmap to retain an alpha channel for some other reason. If the image has no alpha channel this flag has no effect. (C++ enum variant: <span style='color: green;'>```NoOpaqueDetection = 256```</span>)
    #[allow(non_upper_case_globals)]
    pub const NoOpaqueDetection: crate::ImageConversionFlag = crate::ImageConversionFlag(256);
    /// Don't do any format conversions on the image. Can be useful when converting a <a href="http://doc.qt.io/qt-5/qimage.html">QImage</a> to a <a href="http://doc.qt.io/qt-5/qpixmap.html">QPixmap</a> for a one-time rendering operation for example. Note that a <a href="http://doc.qt.io/qt-5/qpixmap.html">QPixmap</a> not in the preferred format will be much slower as a paint device. (C++ enum variant: <span style='color: green;'>```NoFormatConversion = 512```</span>)
    #[allow(non_upper_case_globals)]
    pub const NoFormatConversion: crate::ImageConversionFlag = crate::ImageConversionFlag(512);
}

/// <p>Background mode:</p>
///
/// C++ enum: <span style='color: green;'>```Qt::BGMode```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#BGMode-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Background mode:</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct BGMode(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for BGMode {
    fn from(value: ::std::os::raw::c_int) -> Self {
        BGMode(value)
    }
}

impl From<BGMode> for ::std::os::raw::c_int {
    fn from(value: BGMode) -> Self {
        value.0
    }
}

impl BGMode {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl BGMode {
    /// C++ enum variant: <span style='color: green;'>```TransparentMode = 0```</span>
    #[allow(non_upper_case_globals)]
    pub const TransparentMode: crate::BGMode = crate::BGMode(0);
    /// C++ enum variant: <span style='color: green;'>```OpaqueMode = 1```</span>
    #[allow(non_upper_case_globals)]
    pub const OpaqueMode: crate::BGMode = crate::BGMode(1);
}

/// <p>The key names used by Qt.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::Key```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#Key-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The key names used by Qt.</p>
///
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qkeyevent.html#key">QKeyEvent::key</a>().</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct Key(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for Key {
    fn from(value: ::std::os::raw::c_int) -> Self {
        Key(value)
    }
}

impl From<Key> for ::std::os::raw::c_int {
    fn from(value: Key) -> Self {
        value.0
    }
}

impl Key {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl Key {
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Escape = 16777216```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyEscape: crate::Key = crate::Key(16777216);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Tab = 16777217```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyTab: crate::Key = crate::Key(16777217);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Backtab = 16777218```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyBacktab: crate::Key = crate::Key(16777218);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Backspace = 16777219```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyBackspace: crate::Key = crate::Key(16777219);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Return = 16777220```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyReturn: crate::Key = crate::Key(16777220);
    /// Typically located on the keypad. (C++ enum variant: <span style='color: green;'>```Key_Enter = 16777221```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyEnter: crate::Key = crate::Key(16777221);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Insert = 16777222```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyInsert: crate::Key = crate::Key(16777222);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Delete = 16777223```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDelete: crate::Key = crate::Key(16777223);
    /// The Pause/Break key (<b>Note</b>: Not related to pausing media) (C++ enum variant: <span style='color: green;'>```Key_Pause = 16777224```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyPause: crate::Key = crate::Key(16777224);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Print = 16777225```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyPrint: crate::Key = crate::Key(16777225);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_SysReq = 16777226```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeySysReq: crate::Key = crate::Key(16777226);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Clear = 16777227```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyClear: crate::Key = crate::Key(16777227);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Home = 16777232```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyHome: crate::Key = crate::Key(16777232);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_End = 16777233```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyEnd: crate::Key = crate::Key(16777233);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Left = 16777234```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyLeft: crate::Key = crate::Key(16777234);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Up = 16777235```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyUp: crate::Key = crate::Key(16777235);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Right = 16777236```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyRight: crate::Key = crate::Key(16777236);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Down = 16777237```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDown: crate::Key = crate::Key(16777237);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_PageUp = 16777238```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyPageUp: crate::Key = crate::Key(16777238);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_PageDown = 16777239```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyPageDown: crate::Key = crate::Key(16777239);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Shift = 16777248```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyShift: crate::Key = crate::Key(16777248);
    /// On <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, this corresponds to the Command keys. (C++ enum variant: <span style='color: green;'>```Key_Control = 16777249```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyControl: crate::Key = crate::Key(16777249);
    /// On <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, this corresponds to the Control keys. On Windows keyboards, this key is mapped to the Windows key. (C++ enum variant: <span style='color: green;'>```Key_Meta = 16777250```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMeta: crate::Key = crate::Key(16777250);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Alt = 16777251```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyAlt: crate::Key = crate::Key(16777251);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_CapsLock = 16777252```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyCapsLock: crate::Key = crate::Key(16777252);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_NumLock = 16777253```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyNumLock: crate::Key = crate::Key(16777253);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_ScrollLock = 16777254```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyScrollLock: crate::Key = crate::Key(16777254);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F1 = 16777264```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF1: crate::Key = crate::Key(16777264);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F2 = 16777265```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF2: crate::Key = crate::Key(16777265);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F3 = 16777266```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF3: crate::Key = crate::Key(16777266);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F4 = 16777267```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF4: crate::Key = crate::Key(16777267);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F5 = 16777268```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF5: crate::Key = crate::Key(16777268);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F6 = 16777269```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF6: crate::Key = crate::Key(16777269);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F7 = 16777270```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF7: crate::Key = crate::Key(16777270);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F8 = 16777271```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF8: crate::Key = crate::Key(16777271);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F9 = 16777272```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF9: crate::Key = crate::Key(16777272);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F10 = 16777273```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF10: crate::Key = crate::Key(16777273);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F11 = 16777274```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF11: crate::Key = crate::Key(16777274);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F12 = 16777275```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF12: crate::Key = crate::Key(16777275);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F13 = 16777276```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF13: crate::Key = crate::Key(16777276);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F14 = 16777277```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF14: crate::Key = crate::Key(16777277);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F15 = 16777278```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF15: crate::Key = crate::Key(16777278);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F16 = 16777279```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF16: crate::Key = crate::Key(16777279);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F17 = 16777280```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF17: crate::Key = crate::Key(16777280);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F18 = 16777281```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF18: crate::Key = crate::Key(16777281);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F19 = 16777282```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF19: crate::Key = crate::Key(16777282);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F20 = 16777283```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF20: crate::Key = crate::Key(16777283);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F21 = 16777284```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF21: crate::Key = crate::Key(16777284);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F22 = 16777285```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF22: crate::Key = crate::Key(16777285);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F23 = 16777286```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF23: crate::Key = crate::Key(16777286);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F24 = 16777287```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF24: crate::Key = crate::Key(16777287);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F25 = 16777288```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF25: crate::Key = crate::Key(16777288);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F26 = 16777289```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF26: crate::Key = crate::Key(16777289);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F27 = 16777290```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF27: crate::Key = crate::Key(16777290);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F28 = 16777291```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF28: crate::Key = crate::Key(16777291);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F29 = 16777292```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF29: crate::Key = crate::Key(16777292);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F30 = 16777293```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF30: crate::Key = crate::Key(16777293);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F31 = 16777294```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF31: crate::Key = crate::Key(16777294);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F32 = 16777295```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF32: crate::Key = crate::Key(16777295);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F33 = 16777296```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF33: crate::Key = crate::Key(16777296);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F34 = 16777297```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF34: crate::Key = crate::Key(16777297);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F35 = 16777298```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF35: crate::Key = crate::Key(16777298);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Super_L = 16777299```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeySuperL: crate::Key = crate::Key(16777299);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Super_R = 16777300```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeySuperR: crate::Key = crate::Key(16777300);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Menu = 16777301```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMenu: crate::Key = crate::Key(16777301);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Hyper_L = 16777302```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyHyperL: crate::Key = crate::Key(16777302);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Hyper_R = 16777303```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyHyperR: crate::Key = crate::Key(16777303);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Help = 16777304```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyHelp: crate::Key = crate::Key(16777304);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Direction_L = 16777305```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDirectionL: crate::Key = crate::Key(16777305);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Direction_R = 16777312```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDirectionR: crate::Key = crate::Key(16777312);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Space = 32```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeySpace: crate::Key = crate::Key(32);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Any = 32```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyAny: crate::Key = crate::Key(32);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Exclam = 33```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyExclam: crate::Key = crate::Key(33);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_QuoteDbl = 34```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyQuoteDbl: crate::Key = crate::Key(34);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_NumberSign = 35```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyNumberSign: crate::Key = crate::Key(35);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dollar = 36```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDollar: crate::Key = crate::Key(36);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Percent = 37```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyPercent: crate::Key = crate::Key(37);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Ampersand = 38```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyAmpersand: crate::Key = crate::Key(38);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Apostrophe = 39```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyApostrophe: crate::Key = crate::Key(39);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_ParenLeft = 40```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyParenLeft: crate::Key = crate::Key(40);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_ParenRight = 41```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyParenRight: crate::Key = crate::Key(41);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Asterisk = 42```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyAsterisk: crate::Key = crate::Key(42);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Plus = 43```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyPlus: crate::Key = crate::Key(43);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Comma = 44```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyComma: crate::Key = crate::Key(44);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Minus = 45```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMinus: crate::Key = crate::Key(45);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Period = 46```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyPeriod: crate::Key = crate::Key(46);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Slash = 47```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeySlash: crate::Key = crate::Key(47);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_0 = 48```</span>)
    #[allow(non_upper_case_globals)]
    pub const Key0: crate::Key = crate::Key(48);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_1 = 49```</span>)
    #[allow(non_upper_case_globals)]
    pub const Key1: crate::Key = crate::Key(49);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_2 = 50```</span>)
    #[allow(non_upper_case_globals)]
    pub const Key2: crate::Key = crate::Key(50);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_3 = 51```</span>)
    #[allow(non_upper_case_globals)]
    pub const Key3: crate::Key = crate::Key(51);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_4 = 52```</span>)
    #[allow(non_upper_case_globals)]
    pub const Key4: crate::Key = crate::Key(52);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_5 = 53```</span>)
    #[allow(non_upper_case_globals)]
    pub const Key5: crate::Key = crate::Key(53);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_6 = 54```</span>)
    #[allow(non_upper_case_globals)]
    pub const Key6: crate::Key = crate::Key(54);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_7 = 55```</span>)
    #[allow(non_upper_case_globals)]
    pub const Key7: crate::Key = crate::Key(55);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_8 = 56```</span>)
    #[allow(non_upper_case_globals)]
    pub const Key8: crate::Key = crate::Key(56);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_9 = 57```</span>)
    #[allow(non_upper_case_globals)]
    pub const Key9: crate::Key = crate::Key(57);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Colon = 58```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyColon: crate::Key = crate::Key(58);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Semicolon = 59```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeySemicolon: crate::Key = crate::Key(59);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Less = 60```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyLess: crate::Key = crate::Key(60);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Equal = 61```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyEqual: crate::Key = crate::Key(61);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Greater = 62```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyGreater: crate::Key = crate::Key(62);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Question = 63```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyQuestion: crate::Key = crate::Key(63);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_At = 64```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyAt: crate::Key = crate::Key(64);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_A = 65```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyA: crate::Key = crate::Key(65);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_B = 66```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyB: crate::Key = crate::Key(66);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_C = 67```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyC: crate::Key = crate::Key(67);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_D = 68```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyD: crate::Key = crate::Key(68);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_E = 69```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyE: crate::Key = crate::Key(69);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_F = 70```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyF: crate::Key = crate::Key(70);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_G = 71```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyG: crate::Key = crate::Key(71);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_H = 72```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyH: crate::Key = crate::Key(72);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_I = 73```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyI: crate::Key = crate::Key(73);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_J = 74```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyJ: crate::Key = crate::Key(74);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_K = 75```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyK: crate::Key = crate::Key(75);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_L = 76```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyL: crate::Key = crate::Key(76);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_M = 77```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyM: crate::Key = crate::Key(77);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_N = 78```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyN: crate::Key = crate::Key(78);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_O = 79```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyO: crate::Key = crate::Key(79);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_P = 80```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyP: crate::Key = crate::Key(80);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Q = 81```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyQ: crate::Key = crate::Key(81);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_R = 82```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyR: crate::Key = crate::Key(82);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_S = 83```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyS: crate::Key = crate::Key(83);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_T = 84```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyT: crate::Key = crate::Key(84);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_U = 85```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyU: crate::Key = crate::Key(85);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_V = 86```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyV: crate::Key = crate::Key(86);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_W = 87```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyW: crate::Key = crate::Key(87);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_X = 88```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyX: crate::Key = crate::Key(88);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Y = 89```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyY: crate::Key = crate::Key(89);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Z = 90```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyZ: crate::Key = crate::Key(90);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_BracketLeft = 91```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyBracketLeft: crate::Key = crate::Key(91);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Backslash = 92```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyBackslash: crate::Key = crate::Key(92);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_BracketRight = 93```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyBracketRight: crate::Key = crate::Key(93);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_AsciiCircum = 94```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyAsciiCircum: crate::Key = crate::Key(94);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Underscore = 95```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyUnderscore: crate::Key = crate::Key(95);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_QuoteLeft = 96```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyQuoteLeft: crate::Key = crate::Key(96);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_BraceLeft = 123```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyBraceLeft: crate::Key = crate::Key(123);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Bar = 124```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyBar: crate::Key = crate::Key(124);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_BraceRight = 125```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyBraceRight: crate::Key = crate::Key(125);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_AsciiTilde = 126```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyAsciiTilde: crate::Key = crate::Key(126);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_nobreakspace = 160```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyNobreakspace: crate::Key = crate::Key(160);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_exclamdown = 161```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyExclamdown: crate::Key = crate::Key(161);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_cent = 162```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyCent: crate::Key = crate::Key(162);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_sterling = 163```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeySterling: crate::Key = crate::Key(163);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_currency = 164```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyCurrency: crate::Key = crate::Key(164);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_yen = 165```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyYen: crate::Key = crate::Key(165);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_brokenbar = 166```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyBrokenbar: crate::Key = crate::Key(166);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_section = 167```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeySection: crate::Key = crate::Key(167);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_diaeresis = 168```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDiaeresis: crate::Key = crate::Key(168);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_copyright = 169```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyCopyright: crate::Key = crate::Key(169);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_ordfeminine = 170```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyOrdfeminine: crate::Key = crate::Key(170);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_guillemotleft = 171```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyGuillemotleft: crate::Key = crate::Key(171);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_notsign = 172```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyNotsign: crate::Key = crate::Key(172);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_hyphen = 173```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyHyphen: crate::Key = crate::Key(173);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_registered = 174```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyRegistered: crate::Key = crate::Key(174);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_macron = 175```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMacron: crate::Key = crate::Key(175);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_degree = 176```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDegree: crate::Key = crate::Key(176);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_plusminus = 177```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyPlusminus: crate::Key = crate::Key(177);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_twosuperior = 178```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyTwosuperior: crate::Key = crate::Key(178);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_threesuperior = 179```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyThreesuperior: crate::Key = crate::Key(179);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_acute = 180```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyAcute: crate::Key = crate::Key(180);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_mu = 181```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMu: crate::Key = crate::Key(181);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_paragraph = 182```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyParagraph: crate::Key = crate::Key(182);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_periodcentered = 183```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyPeriodcentered: crate::Key = crate::Key(183);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_cedilla = 184```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyCedilla: crate::Key = crate::Key(184);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_onesuperior = 185```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyOnesuperior: crate::Key = crate::Key(185);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_masculine = 186```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMasculine: crate::Key = crate::Key(186);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_guillemotright = 187```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyGuillemotright: crate::Key = crate::Key(187);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_onequarter = 188```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyOnequarter: crate::Key = crate::Key(188);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_onehalf = 189```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyOnehalf: crate::Key = crate::Key(189);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_threequarters = 190```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyThreequarters: crate::Key = crate::Key(190);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_questiondown = 191```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyQuestiondown: crate::Key = crate::Key(191);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Agrave = 192```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyAgrave: crate::Key = crate::Key(192);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Aacute = 193```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyAacute: crate::Key = crate::Key(193);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Acircumflex = 194```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyAcircumflex: crate::Key = crate::Key(194);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Atilde = 195```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyAtilde: crate::Key = crate::Key(195);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Adiaeresis = 196```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyAdiaeresis: crate::Key = crate::Key(196);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Aring = 197```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyAring: crate::Key = crate::Key(197);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_AE = 198```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyAE: crate::Key = crate::Key(198);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Ccedilla = 199```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyCcedilla: crate::Key = crate::Key(199);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Egrave = 200```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyEgrave: crate::Key = crate::Key(200);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Eacute = 201```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyEacute: crate::Key = crate::Key(201);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Ecircumflex = 202```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyEcircumflex: crate::Key = crate::Key(202);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Ediaeresis = 203```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyEdiaeresis: crate::Key = crate::Key(203);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Igrave = 204```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyIgrave: crate::Key = crate::Key(204);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Iacute = 205```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyIacute: crate::Key = crate::Key(205);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Icircumflex = 206```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyIcircumflex: crate::Key = crate::Key(206);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Idiaeresis = 207```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyIdiaeresis: crate::Key = crate::Key(207);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_ETH = 208```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyETH: crate::Key = crate::Key(208);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Ntilde = 209```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyNtilde: crate::Key = crate::Key(209);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Ograve = 210```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyOgrave: crate::Key = crate::Key(210);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Oacute = 211```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyOacute: crate::Key = crate::Key(211);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Ocircumflex = 212```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyOcircumflex: crate::Key = crate::Key(212);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Otilde = 213```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyOtilde: crate::Key = crate::Key(213);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Odiaeresis = 214```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyOdiaeresis: crate::Key = crate::Key(214);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_multiply = 215```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMultiply: crate::Key = crate::Key(215);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Ooblique = 216```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyOoblique: crate::Key = crate::Key(216);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Ugrave = 217```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyUgrave: crate::Key = crate::Key(217);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Uacute = 218```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyUacute: crate::Key = crate::Key(218);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Ucircumflex = 219```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyUcircumflex: crate::Key = crate::Key(219);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Udiaeresis = 220```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyUdiaeresis: crate::Key = crate::Key(220);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Yacute = 221```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyYacute: crate::Key = crate::Key(221);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_THORN = 222```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyTHORN: crate::Key = crate::Key(222);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_ssharp = 223```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeySsharp: crate::Key = crate::Key(223);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_division = 247```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDivision: crate::Key = crate::Key(247);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_ydiaeresis = 255```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyYdiaeresis: crate::Key = crate::Key(255);
    /// On Windows, when the KeyDown event for this key is sent, the Ctrl+Alt modifiers are also set. (C++ enum variant: <span style='color: green;'>```Key_AltGr = 16781571```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyAltGr: crate::Key = crate::Key(16781571);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Multi_key = 16781600```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMultiKey: crate::Key = crate::Key(16781600);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Codeinput = 16781623```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyCodeinput: crate::Key = crate::Key(16781623);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_SingleCandidate = 16781628```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeySingleCandidate: crate::Key = crate::Key(16781628);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_MultipleCandidate = 16781629```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMultipleCandidate: crate::Key = crate::Key(16781629);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_PreviousCandidate = 16781630```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyPreviousCandidate: crate::Key = crate::Key(16781630);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Mode_switch = 16781694```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyModeSwitch: crate::Key = crate::Key(16781694);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Kanji = 16781601```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyKanji: crate::Key = crate::Key(16781601);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Muhenkan = 16781602```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMuhenkan: crate::Key = crate::Key(16781602);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Henkan = 16781603```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyHenkan: crate::Key = crate::Key(16781603);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Romaji = 16781604```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyRomaji: crate::Key = crate::Key(16781604);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Hiragana = 16781605```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyHiragana: crate::Key = crate::Key(16781605);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Katakana = 16781606```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyKatakana: crate::Key = crate::Key(16781606);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Hiragana_Katakana = 16781607```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyHiraganaKatakana: crate::Key = crate::Key(16781607);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Zenkaku = 16781608```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyZenkaku: crate::Key = crate::Key(16781608);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Hankaku = 16781609```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyHankaku: crate::Key = crate::Key(16781609);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Zenkaku_Hankaku = 16781610```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyZenkakuHankaku: crate::Key = crate::Key(16781610);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Touroku = 16781611```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyTouroku: crate::Key = crate::Key(16781611);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Massyo = 16781612```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMassyo: crate::Key = crate::Key(16781612);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Kana_Lock = 16781613```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyKanaLock: crate::Key = crate::Key(16781613);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Kana_Shift = 16781614```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyKanaShift: crate::Key = crate::Key(16781614);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Eisu_Shift = 16781615```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyEisuShift: crate::Key = crate::Key(16781615);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Eisu_toggle = 16781616```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyEisuToggle: crate::Key = crate::Key(16781616);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Hangul = 16781617```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyHangul: crate::Key = crate::Key(16781617);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Hangul_Start = 16781618```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyHangulStart: crate::Key = crate::Key(16781618);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Hangul_End = 16781619```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyHangulEnd: crate::Key = crate::Key(16781619);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Hangul_Hanja = 16781620```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyHangulHanja: crate::Key = crate::Key(16781620);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Hangul_Jamo = 16781621```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyHangulJamo: crate::Key = crate::Key(16781621);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Hangul_Romaja = 16781622```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyHangulRomaja: crate::Key = crate::Key(16781622);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Hangul_Jeonja = 16781624```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyHangulJeonja: crate::Key = crate::Key(16781624);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Hangul_Banja = 16781625```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyHangulBanja: crate::Key = crate::Key(16781625);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Hangul_PreHanja = 16781626```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyHangulPreHanja: crate::Key = crate::Key(16781626);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Hangul_PostHanja = 16781627```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyHangulPostHanja: crate::Key = crate::Key(16781627);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Hangul_Special = 16781631```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyHangulSpecial: crate::Key = crate::Key(16781631);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Grave = 16781904```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadGrave: crate::Key = crate::Key(16781904);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Acute = 16781905```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadAcute: crate::Key = crate::Key(16781905);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Circumflex = 16781906```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadCircumflex: crate::Key = crate::Key(16781906);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Tilde = 16781907```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadTilde: crate::Key = crate::Key(16781907);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Macron = 16781908```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadMacron: crate::Key = crate::Key(16781908);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Breve = 16781909```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadBreve: crate::Key = crate::Key(16781909);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Abovedot = 16781910```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadAbovedot: crate::Key = crate::Key(16781910);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Diaeresis = 16781911```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadDiaeresis: crate::Key = crate::Key(16781911);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Abovering = 16781912```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadAbovering: crate::Key = crate::Key(16781912);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Doubleacute = 16781913```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadDoubleacute: crate::Key = crate::Key(16781913);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Caron = 16781914```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadCaron: crate::Key = crate::Key(16781914);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Cedilla = 16781915```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadCedilla: crate::Key = crate::Key(16781915);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Ogonek = 16781916```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadOgonek: crate::Key = crate::Key(16781916);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Iota = 16781917```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadIota: crate::Key = crate::Key(16781917);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Voiced_Sound = 16781918```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadVoicedSound: crate::Key = crate::Key(16781918);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Semivoiced_Sound = 16781919```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadSemivoicedSound: crate::Key = crate::Key(16781919);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Belowdot = 16781920```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadBelowdot: crate::Key = crate::Key(16781920);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Hook = 16781921```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadHook: crate::Key = crate::Key(16781921);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Horn = 16781922```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadHorn: crate::Key = crate::Key(16781922);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Back = 16777313```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyBack: crate::Key = crate::Key(16777313);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Forward = 16777314```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyForward: crate::Key = crate::Key(16777314);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Stop = 16777315```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyStop: crate::Key = crate::Key(16777315);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Refresh = 16777316```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyRefresh: crate::Key = crate::Key(16777316);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_VolumeDown = 16777328```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyVolumeDown: crate::Key = crate::Key(16777328);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_VolumeMute = 16777329```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyVolumeMute: crate::Key = crate::Key(16777329);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_VolumeUp = 16777330```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyVolumeUp: crate::Key = crate::Key(16777330);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_BassBoost = 16777331```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyBassBoost: crate::Key = crate::Key(16777331);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_BassUp = 16777332```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyBassUp: crate::Key = crate::Key(16777332);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_BassDown = 16777333```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyBassDown: crate::Key = crate::Key(16777333);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_TrebleUp = 16777334```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyTrebleUp: crate::Key = crate::Key(16777334);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_TrebleDown = 16777335```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyTrebleDown: crate::Key = crate::Key(16777335);
    /// A key setting the state of the media player to play (C++ enum variant: <span style='color: green;'>```Key_MediaPlay = 16777344```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMediaPlay: crate::Key = crate::Key(16777344);
    /// A key setting the state of the media player to stop (C++ enum variant: <span style='color: green;'>```Key_MediaStop = 16777345```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMediaStop: crate::Key = crate::Key(16777345);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_MediaPrevious = 16777346```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMediaPrevious: crate::Key = crate::Key(16777346);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_MediaNext = 16777347```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMediaNext: crate::Key = crate::Key(16777347);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_MediaRecord = 16777348```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMediaRecord: crate::Key = crate::Key(16777348);
    /// A key setting the state of the media player to pause (<b>Note</b>: not the pause/break key) (C++ enum variant: <span style='color: green;'>```Key_MediaPause = 16777349```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMediaPause: crate::Key = crate::Key(16777349);
    /// A key to toggle the play/pause state in the media player (rather than setting an absolute state) (C++ enum variant: <span style='color: green;'>```Key_MediaTogglePlayPause = 16777350```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMediaTogglePlayPause: crate::Key = crate::Key(16777350);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_HomePage = 16777360```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyHomePage: crate::Key = crate::Key(16777360);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Favorites = 16777361```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyFavorites: crate::Key = crate::Key(16777361);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Search = 16777362```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeySearch: crate::Key = crate::Key(16777362);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Standby = 16777363```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyStandby: crate::Key = crate::Key(16777363);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_OpenUrl = 16777364```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyOpenUrl: crate::Key = crate::Key(16777364);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_LaunchMail = 16777376```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyLaunchMail: crate::Key = crate::Key(16777376);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_LaunchMedia = 16777377```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyLaunchMedia: crate::Key = crate::Key(16777377);
    /// On X11 this key is mapped to "My Computer" (XF86XK_MyComputer) key for legacy reasons. (C++ enum variant: <span style='color: green;'>```Key_Launch0 = 16777378```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyLaunch0: crate::Key = crate::Key(16777378);
    /// On X11 this key is mapped to "Calculator" (XF86XK_Calculator) key for legacy reasons. (C++ enum variant: <span style='color: green;'>```Key_Launch1 = 16777379```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyLaunch1: crate::Key = crate::Key(16777379);
    /// On X11 this key is mapped to XF86XK_Launch0 key for legacy reasons. (C++ enum variant: <span style='color: green;'>```Key_Launch2 = 16777380```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyLaunch2: crate::Key = crate::Key(16777380);
    /// On X11 this key is mapped to XF86XK_Launch1 key for legacy reasons. (C++ enum variant: <span style='color: green;'>```Key_Launch3 = 16777381```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyLaunch3: crate::Key = crate::Key(16777381);
    /// On X11 this key is mapped to XF86XK_Launch2 key for legacy reasons. (C++ enum variant: <span style='color: green;'>```Key_Launch4 = 16777382```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyLaunch4: crate::Key = crate::Key(16777382);
    /// On X11 this key is mapped to XF86XK_Launch3 key for legacy reasons. (C++ enum variant: <span style='color: green;'>```Key_Launch5 = 16777383```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyLaunch5: crate::Key = crate::Key(16777383);
    /// On X11 this key is mapped to XF86XK_Launch4 key for legacy reasons. (C++ enum variant: <span style='color: green;'>```Key_Launch6 = 16777384```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyLaunch6: crate::Key = crate::Key(16777384);
    /// On X11 this key is mapped to XF86XK_Launch5 key for legacy reasons. (C++ enum variant: <span style='color: green;'>```Key_Launch7 = 16777385```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyLaunch7: crate::Key = crate::Key(16777385);
    /// On X11 this key is mapped to XF86XK_Launch6 key for legacy reasons. (C++ enum variant: <span style='color: green;'>```Key_Launch8 = 16777386```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyLaunch8: crate::Key = crate::Key(16777386);
    /// On X11 this key is mapped to XF86XK_Launch7 key for legacy reasons. (C++ enum variant: <span style='color: green;'>```Key_Launch9 = 16777387```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyLaunch9: crate::Key = crate::Key(16777387);
    /// On X11 this key is mapped to XF86XK_Launch8 key for legacy reasons. (C++ enum variant: <span style='color: green;'>```Key_LaunchA = 16777388```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyLaunchA: crate::Key = crate::Key(16777388);
    /// On X11 this key is mapped to XF86XK_Launch9 key for legacy reasons. (C++ enum variant: <span style='color: green;'>```Key_LaunchB = 16777389```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyLaunchB: crate::Key = crate::Key(16777389);
    /// On X11 this key is mapped to XF86XK_LaunchA key for legacy reasons. (C++ enum variant: <span style='color: green;'>```Key_LaunchC = 16777390```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyLaunchC: crate::Key = crate::Key(16777390);
    /// On X11 this key is mapped to XF86XK_LaunchB key for legacy reasons. (C++ enum variant: <span style='color: green;'>```Key_LaunchD = 16777391```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyLaunchD: crate::Key = crate::Key(16777391);
    /// On X11 this key is mapped to XF86XK_LaunchC key for legacy reasons. (C++ enum variant: <span style='color: green;'>```Key_LaunchE = 16777392```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyLaunchE: crate::Key = crate::Key(16777392);
    /// On X11 this key is mapped to XF86XK_LaunchD key for legacy reasons. (C++ enum variant: <span style='color: green;'>```Key_LaunchF = 16777393```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyLaunchF: crate::Key = crate::Key(16777393);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_MonBrightnessUp = 16777394```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMonBrightnessUp: crate::Key = crate::Key(16777394);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_MonBrightnessDown = 16777395```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMonBrightnessDown: crate::Key = crate::Key(16777395);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_KeyboardLightOnOff = 16777396```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyKeyboardLightOnOff: crate::Key = crate::Key(16777396);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_KeyboardBrightnessUp = 16777397```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyKeyboardBrightnessUp: crate::Key = crate::Key(16777397);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_KeyboardBrightnessDown = 16777398```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyKeyboardBrightnessDown: crate::Key = crate::Key(16777398);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_PowerOff = 16777399```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyPowerOff: crate::Key = crate::Key(16777399);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_WakeUp = 16777400```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyWakeUp: crate::Key = crate::Key(16777400);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Eject = 16777401```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyEject: crate::Key = crate::Key(16777401);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_ScreenSaver = 16777402```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyScreenSaver: crate::Key = crate::Key(16777402);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_WWW = 16777403```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyWWW: crate::Key = crate::Key(16777403);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Memo = 16777404```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMemo: crate::Key = crate::Key(16777404);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_LightBulb = 16777405```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyLightBulb: crate::Key = crate::Key(16777405);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Shop = 16777406```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyShop: crate::Key = crate::Key(16777406);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_History = 16777407```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyHistory: crate::Key = crate::Key(16777407);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_AddFavorite = 16777408```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyAddFavorite: crate::Key = crate::Key(16777408);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_HotLinks = 16777409```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyHotLinks: crate::Key = crate::Key(16777409);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_BrightnessAdjust = 16777410```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyBrightnessAdjust: crate::Key = crate::Key(16777410);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Finance = 16777411```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyFinance: crate::Key = crate::Key(16777411);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Community = 16777412```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyCommunity: crate::Key = crate::Key(16777412);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_AudioRewind = 16777413```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyAudioRewind: crate::Key = crate::Key(16777413);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_BackForward = 16777414```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyBackForward: crate::Key = crate::Key(16777414);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_ApplicationLeft = 16777415```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyApplicationLeft: crate::Key = crate::Key(16777415);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_ApplicationRight = 16777416```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyApplicationRight: crate::Key = crate::Key(16777416);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Book = 16777417```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyBook: crate::Key = crate::Key(16777417);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_CD = 16777418```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyCD: crate::Key = crate::Key(16777418);
    /// On X11 this key is not mapped for legacy reasons. Use Qt::Key_Launch1 instead. (C++ enum variant: <span style='color: green;'>```Key_Calculator = 16777419```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyCalculator: crate::Key = crate::Key(16777419);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_ToDoList = 16777420```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyToDoList: crate::Key = crate::Key(16777420);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_ClearGrab = 16777421```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyClearGrab: crate::Key = crate::Key(16777421);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Close = 16777422```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyClose: crate::Key = crate::Key(16777422);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Copy = 16777423```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyCopy: crate::Key = crate::Key(16777423);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Cut = 16777424```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyCut: crate::Key = crate::Key(16777424);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Display = 16777425```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDisplay: crate::Key = crate::Key(16777425);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_DOS = 16777426```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDOS: crate::Key = crate::Key(16777426);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Documents = 16777427```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDocuments: crate::Key = crate::Key(16777427);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Excel = 16777428```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyExcel: crate::Key = crate::Key(16777428);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Explorer = 16777429```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyExplorer: crate::Key = crate::Key(16777429);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Game = 16777430```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyGame: crate::Key = crate::Key(16777430);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Go = 16777431```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyGo: crate::Key = crate::Key(16777431);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_iTouch = 16777432```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyITouch: crate::Key = crate::Key(16777432);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_LogOff = 16777433```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyLogOff: crate::Key = crate::Key(16777433);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Market = 16777434```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMarket: crate::Key = crate::Key(16777434);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Meeting = 16777435```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMeeting: crate::Key = crate::Key(16777435);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_MenuKB = 16777436```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMenuKB: crate::Key = crate::Key(16777436);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_MenuPB = 16777437```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMenuPB: crate::Key = crate::Key(16777437);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_MySites = 16777438```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMySites: crate::Key = crate::Key(16777438);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_News = 16777439```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyNews: crate::Key = crate::Key(16777439);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_OfficeHome = 16777440```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyOfficeHome: crate::Key = crate::Key(16777440);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Option = 16777441```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyOption: crate::Key = crate::Key(16777441);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Paste = 16777442```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyPaste: crate::Key = crate::Key(16777442);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Phone = 16777443```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyPhone: crate::Key = crate::Key(16777443);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Calendar = 16777444```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyCalendar: crate::Key = crate::Key(16777444);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Reply = 16777445```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyReply: crate::Key = crate::Key(16777445);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Reload = 16777446```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyReload: crate::Key = crate::Key(16777446);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_RotateWindows = 16777447```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyRotateWindows: crate::Key = crate::Key(16777447);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_RotationPB = 16777448```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyRotationPB: crate::Key = crate::Key(16777448);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_RotationKB = 16777449```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyRotationKB: crate::Key = crate::Key(16777449);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Save = 16777450```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeySave: crate::Key = crate::Key(16777450);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Send = 16777451```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeySend: crate::Key = crate::Key(16777451);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Spell = 16777452```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeySpell: crate::Key = crate::Key(16777452);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_SplitScreen = 16777453```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeySplitScreen: crate::Key = crate::Key(16777453);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Support = 16777454```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeySupport: crate::Key = crate::Key(16777454);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_TaskPane = 16777455```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyTaskPane: crate::Key = crate::Key(16777455);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Terminal = 16777456```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyTerminal: crate::Key = crate::Key(16777456);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Tools = 16777457```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyTools: crate::Key = crate::Key(16777457);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Travel = 16777458```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyTravel: crate::Key = crate::Key(16777458);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Video = 16777459```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyVideo: crate::Key = crate::Key(16777459);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Word = 16777460```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyWord: crate::Key = crate::Key(16777460);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Xfer = 16777461```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyXfer: crate::Key = crate::Key(16777461);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_ZoomIn = 16777462```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyZoomIn: crate::Key = crate::Key(16777462);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_ZoomOut = 16777463```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyZoomOut: crate::Key = crate::Key(16777463);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Away = 16777464```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyAway: crate::Key = crate::Key(16777464);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Messenger = 16777465```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMessenger: crate::Key = crate::Key(16777465);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_WebCam = 16777466```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyWebCam: crate::Key = crate::Key(16777466);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_MailForward = 16777467```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMailForward: crate::Key = crate::Key(16777467);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Pictures = 16777468```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyPictures: crate::Key = crate::Key(16777468);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Music = 16777469```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMusic: crate::Key = crate::Key(16777469);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Battery = 16777470```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyBattery: crate::Key = crate::Key(16777470);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Bluetooth = 16777471```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyBluetooth: crate::Key = crate::Key(16777471);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_WLAN = 16777472```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyWLAN: crate::Key = crate::Key(16777472);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_UWB = 16777473```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyUWB: crate::Key = crate::Key(16777473);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_AudioForward = 16777474```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyAudioForward: crate::Key = crate::Key(16777474);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_AudioRepeat = 16777475```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyAudioRepeat: crate::Key = crate::Key(16777475);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_AudioRandomPlay = 16777476```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyAudioRandomPlay: crate::Key = crate::Key(16777476);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Subtitle = 16777477```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeySubtitle: crate::Key = crate::Key(16777477);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_AudioCycleTrack = 16777478```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyAudioCycleTrack: crate::Key = crate::Key(16777478);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Time = 16777479```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyTime: crate::Key = crate::Key(16777479);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Hibernate = 16777480```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyHibernate: crate::Key = crate::Key(16777480);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_View = 16777481```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyView: crate::Key = crate::Key(16777481);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_TopMenu = 16777482```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyTopMenu: crate::Key = crate::Key(16777482);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_PowerDown = 16777483```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyPowerDown: crate::Key = crate::Key(16777483);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Suspend = 16777484```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeySuspend: crate::Key = crate::Key(16777484);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_ContrastAdjust = 16777485```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyContrastAdjust: crate::Key = crate::Key(16777485);
    /// On X11 this key is mapped to XF86XK_LaunchE key for legacy reasons. (C++ enum variant: <span style='color: green;'>```Key_LaunchG = 16777486```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyLaunchG: crate::Key = crate::Key(16777486);
    /// On X11 this key is mapped to XF86XK_LaunchF key for legacy reasons. (C++ enum variant: <span style='color: green;'>```Key_LaunchH = 16777487```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyLaunchH: crate::Key = crate::Key(16777487);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_TouchpadToggle = 16777488```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyTouchpadToggle: crate::Key = crate::Key(16777488);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_TouchpadOn = 16777489```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyTouchpadOn: crate::Key = crate::Key(16777489);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_TouchpadOff = 16777490```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyTouchpadOff: crate::Key = crate::Key(16777490);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_MicMute = 16777491```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMicMute: crate::Key = crate::Key(16777491);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Red = 16777492```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyRed: crate::Key = crate::Key(16777492);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Green = 16777493```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyGreen: crate::Key = crate::Key(16777493);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Yellow = 16777494```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyYellow: crate::Key = crate::Key(16777494);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Blue = 16777495```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyBlue: crate::Key = crate::Key(16777495);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_ChannelUp = 16777496```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyChannelUp: crate::Key = crate::Key(16777496);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_ChannelDown = 16777497```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyChannelDown: crate::Key = crate::Key(16777497);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Guide = 16777498```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyGuide: crate::Key = crate::Key(16777498);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Info = 16777499```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyInfo: crate::Key = crate::Key(16777499);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Settings = 16777500```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeySettings: crate::Key = crate::Key(16777500);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_MicVolumeUp = 16777501```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMicVolumeUp: crate::Key = crate::Key(16777501);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_MicVolumeDown = 16777502```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMicVolumeDown: crate::Key = crate::Key(16777502);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_New = 16777504```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyNew: crate::Key = crate::Key(16777504);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Open = 16777505```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyOpen: crate::Key = crate::Key(16777505);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Find = 16777506```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyFind: crate::Key = crate::Key(16777506);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Undo = 16777507```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyUndo: crate::Key = crate::Key(16777507);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Redo = 16777508```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyRedo: crate::Key = crate::Key(16777508);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_MediaLast = 16842751```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyMediaLast: crate::Key = crate::Key(16842751);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Select = 16842752```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeySelect: crate::Key = crate::Key(16842752);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Yes = 16842753```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyYes: crate::Key = crate::Key(16842753);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_No = 16842754```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyNo: crate::Key = crate::Key(16842754);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Cancel = 16908289```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyCancel: crate::Key = crate::Key(16908289);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Printer = 16908290```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyPrinter: crate::Key = crate::Key(16908290);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Execute = 16908291```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyExecute: crate::Key = crate::Key(16908291);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Sleep = 16908292```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeySleep: crate::Key = crate::Key(16908292);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Play = 16908293```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyPlay: crate::Key = crate::Key(16908293);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Zoom = 16908294```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyZoom: crate::Key = crate::Key(16908294);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Exit = 16908298```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyExit: crate::Key = crate::Key(16908298);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Context1 = 17825792```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyContext1: crate::Key = crate::Key(17825792);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Context2 = 17825793```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyContext2: crate::Key = crate::Key(17825793);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Context3 = 17825794```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyContext3: crate::Key = crate::Key(17825794);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Context4 = 17825795```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyContext4: crate::Key = crate::Key(17825795);
    /// A key to answer or initiate a call (see Qt::Key_ToggleCallHangup for a key to toggle current call state) (C++ enum variant: <span style='color: green;'>```Key_Call = 17825796```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyCall: crate::Key = crate::Key(17825796);
    /// A key to end an ongoing call (see Qt::Key_ToggleCallHangup for a key to toggle current call state) (C++ enum variant: <span style='color: green;'>```Key_Hangup = 17825797```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyHangup: crate::Key = crate::Key(17825797);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Flip = 17825798```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyFlip: crate::Key = crate::Key(17825798);
    /// A key to toggle the current call state (ie. either answer, or hangup) depending on current call state (C++ enum variant: <span style='color: green;'>```Key_ToggleCallHangup = 17825799```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyToggleCallHangup: crate::Key = crate::Key(17825799);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_VoiceDial = 17825800```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyVoiceDial: crate::Key = crate::Key(17825800);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_LastNumberRedial = 17825801```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyLastNumberRedial: crate::Key = crate::Key(17825801);
    /// A key to activate the camera shutter. On Windows Runtime, the environment variable QT_QPA_ENABLE_CAMERA_KEYS must be set to receive the event. (C++ enum variant: <span style='color: green;'>```Key_Camera = 17825824```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyCamera: crate::Key = crate::Key(17825824);
    /// A key to focus the camera. On Windows Runtime, the environment variable QT_QPA_ENABLE_CAMERA_KEYS must be set to receive the event. (C++ enum variant: <span style='color: green;'>```Key_CameraFocus = 17825825```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyCameraFocus: crate::Key = crate::Key(17825825);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_unknown = 33554431```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyUnknown: crate::Key = crate::Key(33554431);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Stroke = 16781923```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadStroke: crate::Key = crate::Key(16781923);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Abovecomma = 16781924```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadAbovecomma: crate::Key = crate::Key(16781924);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Abovereversedcomma = 16781925```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadAbovereversedcomma: crate::Key = crate::Key(16781925);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Doublegrave = 16781926```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadDoublegrave: crate::Key = crate::Key(16781926);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Belowring = 16781927```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadBelowring: crate::Key = crate::Key(16781927);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Belowmacron = 16781928```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadBelowmacron: crate::Key = crate::Key(16781928);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Belowcircumflex = 16781929```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadBelowcircumflex: crate::Key = crate::Key(16781929);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Belowtilde = 16781930```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadBelowtilde: crate::Key = crate::Key(16781930);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Belowbreve = 16781931```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadBelowbreve: crate::Key = crate::Key(16781931);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Belowdiaeresis = 16781932```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadBelowdiaeresis: crate::Key = crate::Key(16781932);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Invertedbreve = 16781933```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadInvertedbreve: crate::Key = crate::Key(16781933);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Belowcomma = 16781934```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadBelowcomma: crate::Key = crate::Key(16781934);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Currency = 16781935```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadCurrency: crate::Key = crate::Key(16781935);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_a = 16781952```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadA: crate::Key = crate::Key(16781952);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_A = 16781953```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadA2: crate::Key = crate::Key(16781953);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_e = 16781954```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadE: crate::Key = crate::Key(16781954);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_E = 16781955```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadE2: crate::Key = crate::Key(16781955);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_i = 16781956```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadI: crate::Key = crate::Key(16781956);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_I = 16781957```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadI2: crate::Key = crate::Key(16781957);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_o = 16781958```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadO: crate::Key = crate::Key(16781958);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_O = 16781959```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadO2: crate::Key = crate::Key(16781959);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_u = 16781960```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadU: crate::Key = crate::Key(16781960);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_U = 16781961```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadU2: crate::Key = crate::Key(16781961);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Small_Schwa = 16781962```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadSmallSchwa: crate::Key = crate::Key(16781962);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Capital_Schwa = 16781963```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadCapitalSchwa: crate::Key = crate::Key(16781963);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Greek = 16781964```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadGreek: crate::Key = crate::Key(16781964);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Lowline = 16781968```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadLowline: crate::Key = crate::Key(16781968);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Aboveverticalline = 16781969```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadAboveverticalline: crate::Key = crate::Key(16781969);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Belowverticalline = 16781970```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadBelowverticalline: crate::Key = crate::Key(16781970);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```Key_Dead_Longsolidusoverlay = 16781971```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeyDeadLongsolidusoverlay: crate::Key = crate::Key(16781971);
}

/// C++ enum: <span style='color: green;'>```Qt::ArrowType```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#ArrowType-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct ArrowType(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for ArrowType {
    fn from(value: ::std::os::raw::c_int) -> Self {
        ArrowType(value)
    }
}

impl From<ArrowType> for ::std::os::raw::c_int {
    fn from(value: ArrowType) -> Self {
        value.0
    }
}

impl ArrowType {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl ArrowType {
    /// C++ enum variant: <span style='color: green;'>```NoArrow = 0```</span>
    #[allow(non_upper_case_globals)]
    pub const NoArrow: crate::ArrowType = crate::ArrowType(0);
    /// C++ enum variant: <span style='color: green;'>```UpArrow = 1```</span>
    #[allow(non_upper_case_globals)]
    pub const UpArrow: crate::ArrowType = crate::ArrowType(1);
    /// C++ enum variant: <span style='color: green;'>```DownArrow = 2```</span>
    #[allow(non_upper_case_globals)]
    pub const DownArrow: crate::ArrowType = crate::ArrowType(2);
    /// C++ enum variant: <span style='color: green;'>```LeftArrow = 3```</span>
    #[allow(non_upper_case_globals)]
    pub const LeftArrow: crate::ArrowType = crate::ArrowType(3);
    /// C++ enum variant: <span style='color: green;'>```RightArrow = 4```</span>
    #[allow(non_upper_case_globals)]
    pub const RightArrow: crate::ArrowType = crate::ArrowType(4);
}

/// <p>This enum type defines the pen styles that can be drawn using <a href="http://doc.qt.io/qt-5/qpainter.html">QPainter</a>. The styles are:</p>
///
/// C++ enum: <span style='color: green;'>```Qt::PenStyle```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#PenStyle-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type defines the pen styles that can be drawn using <a href="http://doc.qt.io/qt-5/qpainter.html">QPainter</a>. The styles are:</p>
/// <div class="table"><table class="generic">
///  <tbody><tr valign="top" class="odd"><td><img src="http://doc.qt.io/qt-5/images/qpen-solid.png" alt=""></td><td><img src="http://doc.qt.io/qt-5/images/qpen-dash.png" alt=""></td><td><img src="http://doc.qt.io/qt-5/images/qpen-dot.png" alt=""></td></tr>
/// <tr valign="top" class="even"><td>Qt::SolidLine</td><td>Qt::DashLine</td><td>Qt::DotLine</td></tr>
/// <tr valign="top" class="odd"><td><img src="http://doc.qt.io/qt-5/images/qpen-dashdot.png" alt=""></td><td><img src="http://doc.qt.io/qt-5/images/qpen-dashdotdot.png" alt=""></td><td><img src="http://doc.qt.io/qt-5/images/qpen-custom.png" alt=""></td></tr>
/// <tr valign="top" class="even"><td>Qt::DashDotLine</td><td>Qt::DashDotDotLine</td><td>Qt::CustomDashLine</td></tr>
/// </tbody></table></div>
///
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpen.html">QPen</a>.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct PenStyle(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for PenStyle {
    fn from(value: ::std::os::raw::c_int) -> Self {
        PenStyle(value)
    }
}

impl From<PenStyle> for ::std::os::raw::c_int {
    fn from(value: PenStyle) -> Self {
        value.0
    }
}

impl PenStyle {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl PenStyle {
    /// no line at all. For example, <a href="http://doc.qt.io/qt-5/qpainter.html#drawRect-2">QPainter::drawRect</a>() fills but does not draw any boundary line. (C++ enum variant: <span style='color: green;'>```NoPen = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const NoPen: crate::PenStyle = crate::PenStyle(0);
    /// A plain line. (C++ enum variant: <span style='color: green;'>```SolidLine = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const SolidLine: crate::PenStyle = crate::PenStyle(1);
    /// Dashes separated by a few pixels. (C++ enum variant: <span style='color: green;'>```DashLine = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const DashLine: crate::PenStyle = crate::PenStyle(2);
    /// Dots separated by a few pixels. (C++ enum variant: <span style='color: green;'>```DotLine = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const DotLine: crate::PenStyle = crate::PenStyle(3);
    /// Alternate dots and dashes. (C++ enum variant: <span style='color: green;'>```DashDotLine = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const DashDotLine: crate::PenStyle = crate::PenStyle(4);
    /// One dash, two dots, one dash, two dots. (C++ enum variant: <span style='color: green;'>```DashDotDotLine = 5```</span>)
    #[allow(non_upper_case_globals)]
    pub const DashDotDotLine: crate::PenStyle = crate::PenStyle(5);
    /// A custom pattern defined using <a href="http://doc.qt.io/qt-5/qpainterpathstroker.html#setDashPattern">QPainterPathStroker::setDashPattern</a>(). (C++ enum variant: <span style='color: green;'>```CustomDashLine = 6```</span>)
    #[allow(non_upper_case_globals)]
    pub const CustomDashLine: crate::PenStyle = crate::PenStyle(6);
    /// C++ enum variant: <span style='color: green;'>```MPenStyle = 15```</span>
    #[allow(non_upper_case_globals)]
    pub const MPenStyle: crate::PenStyle = crate::PenStyle(15);
}

/// <p>This enum type defines the pen cap styles supported by Qt, i.e. the line end caps that can be drawn using <a href="http://doc.qt.io/qt-5/qpainter.html">QPainter</a>.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::PenCapStyle```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#PenCapStyle-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type defines the pen cap styles supported by Qt, i.e. the line end caps that can be drawn using <a href="http://doc.qt.io/qt-5/qpainter.html">QPainter</a>.</p>
/// <div class="table"><table class="generic">
///  <tbody><tr valign="top" class="odd"><td><img src="http://doc.qt.io/qt-5/images/qpen-square.png" alt=""></td><td><img src="http://doc.qt.io/qt-5/images/qpen-flat.png" alt=""></td><td><img src="http://doc.qt.io/qt-5/images/qpen-roundcap.png" alt=""></td></tr>
/// <tr valign="top" class="even"><td>Qt::SquareCap</td><td>Qt::FlatCap</td><td>Qt::RoundCap</td></tr>
/// </tbody></table></div>
///
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpen.html">QPen</a>.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct PenCapStyle(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for PenCapStyle {
    fn from(value: ::std::os::raw::c_int) -> Self {
        PenCapStyle(value)
    }
}

impl From<PenCapStyle> for ::std::os::raw::c_int {
    fn from(value: PenCapStyle) -> Self {
        value.0
    }
}

impl PenCapStyle {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl PenCapStyle {
    /// a square line end that does not cover the end point of the line. (C++ enum variant: <span style='color: green;'>```FlatCap = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const FlatCap: crate::PenCapStyle = crate::PenCapStyle(0);
    /// a square line end that covers the end point and extends beyond it by half the line width. (C++ enum variant: <span style='color: green;'>```SquareCap = 16```</span>)
    #[allow(non_upper_case_globals)]
    pub const SquareCap: crate::PenCapStyle = crate::PenCapStyle(16);
    /// a rounded line end. (C++ enum variant: <span style='color: green;'>```RoundCap = 32```</span>)
    #[allow(non_upper_case_globals)]
    pub const RoundCap: crate::PenCapStyle = crate::PenCapStyle(32);
    /// C++ enum variant: <span style='color: green;'>```MPenCapStyle = 48```</span>
    #[allow(non_upper_case_globals)]
    pub const MPenCapStyle: crate::PenCapStyle = crate::PenCapStyle(48);
}

/// <p>This enum type defines the pen join styles supported by Qt, i.e. which joins between two connected lines can be drawn using <a href="http://doc.qt.io/qt-5/qpainter.html">QPainter</a>.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::PenJoinStyle```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#PenJoinStyle-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type defines the pen join styles supported by Qt, i.e. which joins between two connected lines can be drawn using <a href="http://doc.qt.io/qt-5/qpainter.html">QPainter</a>.</p>
/// <div class="table"><table class="generic">
///  <tbody><tr valign="top" class="odd"><td><img src="http://doc.qt.io/qt-5/images/qpen-bevel.png" alt=""></td><td><img src="http://doc.qt.io/qt-5/images/qpen-miter.png" alt=""></td><td><img src="http://doc.qt.io/qt-5/images/qpen-roundjoin.png" alt=""></td></tr>
/// <tr valign="top" class="even"><td>Qt::BevelJoin</td><td>Qt::MiterJoin</td><td>Qt::RoundJoin</td></tr>
/// </tbody></table></div>
///
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpen.html">QPen</a>.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct PenJoinStyle(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for PenJoinStyle {
    fn from(value: ::std::os::raw::c_int) -> Self {
        PenJoinStyle(value)
    }
}

impl From<PenJoinStyle> for ::std::os::raw::c_int {
    fn from(value: PenJoinStyle) -> Self {
        value.0
    }
}

impl PenJoinStyle {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl PenJoinStyle {
    /// The outer edges of the lines are extended to meet at an angle, and this area is filled. (C++ enum variant: <span style='color: green;'>```MiterJoin = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const MiterJoin: crate::PenJoinStyle = crate::PenJoinStyle(0);
    /// The triangular notch between the two lines is filled. (C++ enum variant: <span style='color: green;'>```BevelJoin = 64```</span>)
    #[allow(non_upper_case_globals)]
    pub const BevelJoin: crate::PenJoinStyle = crate::PenJoinStyle(64);
    /// A circular arc between the two lines is filled. (C++ enum variant: <span style='color: green;'>```RoundJoin = 128```</span>)
    #[allow(non_upper_case_globals)]
    pub const RoundJoin: crate::PenJoinStyle = crate::PenJoinStyle(128);
    /// A miter join corresponding to the definition of a miter join in the <a href="http://www.w3.org/TR/SVGMobile12/">SVG 1.2 Tiny</a> specification. (C++ enum variant: <span style='color: green;'>```SvgMiterJoin = 256```</span>)
    #[allow(non_upper_case_globals)]
    pub const SvgMiterJoin: crate::PenJoinStyle = crate::PenJoinStyle(256);
    /// C++ enum variant: <span style='color: green;'>```MPenJoinStyle = 448```</span>
    #[allow(non_upper_case_globals)]
    pub const MPenJoinStyle: crate::PenJoinStyle = crate::PenJoinStyle(448);
}

/// <p>This enum type defines the brush styles supported by Qt, i.e. the fill pattern of shapes drawn using <a href="http://doc.qt.io/qt-5/qpainter.html">QPainter</a>.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::BrushStyle```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#BrushStyle-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type defines the brush styles supported by Qt, i.e. the fill pattern of shapes drawn using <a href="http://doc.qt.io/qt-5/qpainter.html">QPainter</a>.</p>
/// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/brush-styles.png" alt="Brush Styles"></p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbrush.html">QBrush</a>.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct BrushStyle(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for BrushStyle {
    fn from(value: ::std::os::raw::c_int) -> Self {
        BrushStyle(value)
    }
}

impl From<BrushStyle> for ::std::os::raw::c_int {
    fn from(value: BrushStyle) -> Self {
        value.0
    }
}

impl BrushStyle {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl BrushStyle {
    /// No brush pattern. (C++ enum variant: <span style='color: green;'>```NoBrush = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const NoBrush: crate::BrushStyle = crate::BrushStyle(0);
    /// Uniform color. (C++ enum variant: <span style='color: green;'>```SolidPattern = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const SolidPattern: crate::BrushStyle = crate::BrushStyle(1);
    /// Extremely dense brush pattern. (C++ enum variant: <span style='color: green;'>```Dense1Pattern = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const Dense1Pattern: crate::BrushStyle = crate::BrushStyle(2);
    /// Very dense brush pattern. (C++ enum variant: <span style='color: green;'>```Dense2Pattern = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const Dense2Pattern: crate::BrushStyle = crate::BrushStyle(3);
    /// Somewhat dense brush pattern. (C++ enum variant: <span style='color: green;'>```Dense3Pattern = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const Dense3Pattern: crate::BrushStyle = crate::BrushStyle(4);
    /// Half dense brush pattern. (C++ enum variant: <span style='color: green;'>```Dense4Pattern = 5```</span>)
    #[allow(non_upper_case_globals)]
    pub const Dense4Pattern: crate::BrushStyle = crate::BrushStyle(5);
    /// Somewhat sparse brush pattern. (C++ enum variant: <span style='color: green;'>```Dense5Pattern = 6```</span>)
    #[allow(non_upper_case_globals)]
    pub const Dense5Pattern: crate::BrushStyle = crate::BrushStyle(6);
    /// Very sparse brush pattern. (C++ enum variant: <span style='color: green;'>```Dense6Pattern = 7```</span>)
    #[allow(non_upper_case_globals)]
    pub const Dense6Pattern: crate::BrushStyle = crate::BrushStyle(7);
    /// Extremely sparse brush pattern. (C++ enum variant: <span style='color: green;'>```Dense7Pattern = 8```</span>)
    #[allow(non_upper_case_globals)]
    pub const Dense7Pattern: crate::BrushStyle = crate::BrushStyle(8);
    /// Horizontal lines. (C++ enum variant: <span style='color: green;'>```HorPattern = 9```</span>)
    #[allow(non_upper_case_globals)]
    pub const HorPattern: crate::BrushStyle = crate::BrushStyle(9);
    /// Vertical lines. (C++ enum variant: <span style='color: green;'>```VerPattern = 10```</span>)
    #[allow(non_upper_case_globals)]
    pub const VerPattern: crate::BrushStyle = crate::BrushStyle(10);
    /// Crossing horizontal and vertical lines. (C++ enum variant: <span style='color: green;'>```CrossPattern = 11```</span>)
    #[allow(non_upper_case_globals)]
    pub const CrossPattern: crate::BrushStyle = crate::BrushStyle(11);
    /// Backward diagonal lines. (C++ enum variant: <span style='color: green;'>```BDiagPattern = 12```</span>)
    #[allow(non_upper_case_globals)]
    pub const BDiagPattern: crate::BrushStyle = crate::BrushStyle(12);
    /// Forward diagonal lines. (C++ enum variant: <span style='color: green;'>```FDiagPattern = 13```</span>)
    #[allow(non_upper_case_globals)]
    pub const FDiagPattern: crate::BrushStyle = crate::BrushStyle(13);
    /// Crossing diagonal lines. (C++ enum variant: <span style='color: green;'>```DiagCrossPattern = 14```</span>)
    #[allow(non_upper_case_globals)]
    pub const DiagCrossPattern: crate::BrushStyle = crate::BrushStyle(14);
    /// Linear gradient (set using a dedicated <a href="http://doc.qt.io/qt-5/qbrush.html">QBrush</a> constructor). (C++ enum variant: <span style='color: green;'>```LinearGradientPattern = 15```</span>)
    #[allow(non_upper_case_globals)]
    pub const LinearGradientPattern: crate::BrushStyle = crate::BrushStyle(15);
    /// Radial gradient (set using a dedicated <a href="http://doc.qt.io/qt-5/qbrush.html">QBrush</a> constructor). (C++ enum variant: <span style='color: green;'>```RadialGradientPattern = 16```</span>)
    #[allow(non_upper_case_globals)]
    pub const RadialGradientPattern: crate::BrushStyle = crate::BrushStyle(16);
    /// Conical gradient (set using a dedicated <a href="http://doc.qt.io/qt-5/qbrush.html">QBrush</a> constructor). (C++ enum variant: <span style='color: green;'>```ConicalGradientPattern = 17```</span>)
    #[allow(non_upper_case_globals)]
    pub const ConicalGradientPattern: crate::BrushStyle = crate::BrushStyle(17);
    /// Custom pattern (see <a href="http://doc.qt.io/qt-5/qbrush.html#setTexture">QBrush::setTexture</a>()). (C++ enum variant: <span style='color: green;'>```TexturePattern = 24```</span>)
    #[allow(non_upper_case_globals)]
    pub const TexturePattern: crate::BrushStyle = crate::BrushStyle(24);
}

/// <p>This enum is used by <a href="http://doc.qt.io/qt-5/qpainter.html#drawRoundedRect-2">QPainter::drawRoundedRect</a>() and <a href="http://doc.qt.io/qt-5/qpainterpath.html#addRoundedRect">QPainterPath::addRoundedRect</a>() functions to specify the radii of rectangle corners with respect to the dimensions of the bounding rectangles specified.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::SizeMode```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#SizeMode-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum is used by <a href="http://doc.qt.io/qt-5/qpainter.html#drawRoundedRect-2">QPainter::drawRoundedRect</a>() and <a href="http://doc.qt.io/qt-5/qpainterpath.html#addRoundedRect">QPainterPath::addRoundedRect</a>() functions to specify the radii of rectangle corners with respect to the dimensions of the bounding rectangles specified.</p>
///
/// <p>This enum was introduced or modified in  Qt 4.4.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct SizeMode(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for SizeMode {
    fn from(value: ::std::os::raw::c_int) -> Self {
        SizeMode(value)
    }
}

impl From<SizeMode> for ::std::os::raw::c_int {
    fn from(value: SizeMode) -> Self {
        value.0
    }
}

impl SizeMode {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl SizeMode {
    /// Specifies the size using absolute measurements. (C++ enum variant: <span style='color: green;'>```AbsoluteSize = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const AbsoluteSize: crate::SizeMode = crate::SizeMode(0);
    /// Specifies the size relative to the bounding rectangle, typically using percentage measurements. (C++ enum variant: <span style='color: green;'>```RelativeSize = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const RelativeSize: crate::SizeMode = crate::SizeMode(1);
}

/// <p>This enum describes the available UI effects.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::UIEffect```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#UIEffect-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the available UI effects.</p>
/// <p>By default, Qt will try to use the platform specific desktop settings for each effect. Use the QApplication::setDesktopSettingsAware() function (passing <code>false</code> as argument) to prevent this, and the <a href="http://doc.qt.io/qt-5/qapplication.html#setEffectEnabled">QApplication::setEffectEnabled</a>() to enable or disable a particular effect.</p>
/// <p>Note that all effects are disabled on screens running at less than 16-bit color depth.</p>
///
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qapplication.html#setEffectEnabled">QApplication::setEffectEnabled</a>() and <a href="http://doc.qt.io/qt-5/qguiapplication.html#setDesktopSettingsAware">QGuiApplication::setDesktopSettingsAware</a>().</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct UIEffect(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for UIEffect {
    fn from(value: ::std::os::raw::c_int) -> Self {
        UIEffect(value)
    }
}

impl From<UIEffect> for ::std::os::raw::c_int {
    fn from(value: UIEffect) -> Self {
        value.0
    }
}

impl UIEffect {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl UIEffect {
    /// C++ enum variant: <span style='color: green;'>```UI_General = 0```</span>
    #[allow(non_upper_case_globals)]
    pub const UIGeneral: crate::UIEffect = crate::UIEffect(0);
    /// Show animated menus. (C++ enum variant: <span style='color: green;'>```UI_AnimateMenu = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const UIAnimateMenu: crate::UIEffect = crate::UIEffect(1);
    /// Show faded menus. (C++ enum variant: <span style='color: green;'>```UI_FadeMenu = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const UIFadeMenu: crate::UIEffect = crate::UIEffect(2);
    /// Show animated comboboxes. (C++ enum variant: <span style='color: green;'>```UI_AnimateCombo = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const UIAnimateCombo: crate::UIEffect = crate::UIEffect(3);
    /// Show tooltip animations. (C++ enum variant: <span style='color: green;'>```UI_AnimateTooltip = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const UIAnimateTooltip: crate::UIEffect = crate::UIEffect(4);
    /// Show tooltip fading effects. (C++ enum variant: <span style='color: green;'>```UI_FadeTooltip = 5```</span>)
    #[allow(non_upper_case_globals)]
    pub const UIFadeTooltip: crate::UIEffect = crate::UIEffect(5);
    /// Reserved (C++ enum variant: <span style='color: green;'>```UI_AnimateToolBox = 6```</span>)
    #[allow(non_upper_case_globals)]
    pub const UIAnimateToolBox: crate::UIEffect = crate::UIEffect(6);
}

/// <p>This enum type defines the various cursors that can be used.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::CursorShape```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#CursorShape-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type defines the various cursors that can be used.</p>
/// <p>The standard arrow cursor is the default for widgets in a normal state.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct CursorShape(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for CursorShape {
    fn from(value: ::std::os::raw::c_int) -> Self {
        CursorShape(value)
    }
}

impl From<CursorShape> for ::std::os::raw::c_int {
    fn from(value: CursorShape) -> Self {
        value.0
    }
}

impl CursorShape {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl CursorShape {
    /// <img src="http://doc.qt.io/qt-5/images/cursor-arrow.png" alt=""> The standard arrow cursor. (C++ enum variant: <span style='color: green;'>```ArrowCursor = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const ArrowCursor: crate::CursorShape = crate::CursorShape(0);
    /// <img src="http://doc.qt.io/qt-5/images/cursor-uparrow.png" alt=""> An arrow pointing upwards toward the top of the screen. (C++ enum variant: <span style='color: green;'>```UpArrowCursor = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const UpArrowCursor: crate::CursorShape = crate::CursorShape(1);
    /// <img src="http://doc.qt.io/qt-5/images/cursor-cross.png" alt=""> A crosshair cursor, typically used to help the user accurately select a point on the screen. (C++ enum variant: <span style='color: green;'>```CrossCursor = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const CrossCursor: crate::CursorShape = crate::CursorShape(2);
    /// <img src="http://doc.qt.io/qt-5/images/cursor-wait.png" alt=""> An hourglass or watch cursor, usually shown during operations that prevent the user from interacting with the application. (C++ enum variant: <span style='color: green;'>```WaitCursor = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const WaitCursor: crate::CursorShape = crate::CursorShape(3);
    /// <img src="http://doc.qt.io/qt-5/images/cursor-ibeam.png" alt=""> A caret or ibeam cursor, indicating that a widget can accept and display text input. (C++ enum variant: <span style='color: green;'>```IBeamCursor = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const IBeamCursor: crate::CursorShape = crate::CursorShape(4);
    /// <img src="http://doc.qt.io/qt-5/images/cursor-sizev.png" alt=""> A cursor used for elements that are used to vertically resize top-level windows. (C++ enum variant: <span style='color: green;'>```SizeVerCursor = 5```</span>)
    #[allow(non_upper_case_globals)]
    pub const SizeVerCursor: crate::CursorShape = crate::CursorShape(5);
    /// <img src="http://doc.qt.io/qt-5/images/cursor-sizeh.png" alt=""> A cursor used for elements that are used to horizontally resize top-level windows. (C++ enum variant: <span style='color: green;'>```SizeHorCursor = 6```</span>)
    #[allow(non_upper_case_globals)]
    pub const SizeHorCursor: crate::CursorShape = crate::CursorShape(6);
    /// <img src="http://doc.qt.io/qt-5/images/cursor-sizeb.png" alt=""> A cursor used for elements that are used to diagonally resize top-level windows at their top-right and bottom-left corners. (C++ enum variant: <span style='color: green;'>```SizeBDiagCursor = 7```</span>)
    #[allow(non_upper_case_globals)]
    pub const SizeBDiagCursor: crate::CursorShape = crate::CursorShape(7);
    /// <img src="http://doc.qt.io/qt-5/images/cursor-sizef.png" alt=""> A cursor used for elements that are used to diagonally resize top-level windows at their top-left and bottom-right corners. (C++ enum variant: <span style='color: green;'>```SizeFDiagCursor = 8```</span>)
    #[allow(non_upper_case_globals)]
    pub const SizeFDiagCursor: crate::CursorShape = crate::CursorShape(8);
    /// <img src="http://doc.qt.io/qt-5/images/cursor-sizeall.png" alt=""> A cursor used for elements that are used to resize top-level windows in any direction. (C++ enum variant: <span style='color: green;'>```SizeAllCursor = 9```</span>)
    #[allow(non_upper_case_globals)]
    pub const SizeAllCursor: crate::CursorShape = crate::CursorShape(9);
    /// A blank/invisible cursor, typically used when the cursor shape needs to be hidden. (C++ enum variant: <span style='color: green;'>```BlankCursor = 10```</span>)
    #[allow(non_upper_case_globals)]
    pub const BlankCursor: crate::CursorShape = crate::CursorShape(10);
    /// <img src="http://doc.qt.io/qt-5/images/cursor-vsplit.png" alt=""> A cursor used for vertical splitters, indicating that a handle can be dragged horizontally to adjust the use of available space. (C++ enum variant: <span style='color: green;'>```SplitVCursor = 11```</span>)
    #[allow(non_upper_case_globals)]
    pub const SplitVCursor: crate::CursorShape = crate::CursorShape(11);
    /// <img src="http://doc.qt.io/qt-5/images/cursor-hsplit.png" alt=""> A cursor used for horizontal splitters, indicating that a handle can be dragged vertically to adjust the use of available space. (C++ enum variant: <span style='color: green;'>```SplitHCursor = 12```</span>)
    #[allow(non_upper_case_globals)]
    pub const SplitHCursor: crate::CursorShape = crate::CursorShape(12);
    /// <img src="http://doc.qt.io/qt-5/images/cursor-hand.png" alt=""> A pointing hand cursor that is typically used for clickable elements such as hyperlinks. (C++ enum variant: <span style='color: green;'>```PointingHandCursor = 13```</span>)
    #[allow(non_upper_case_globals)]
    pub const PointingHandCursor: crate::CursorShape = crate::CursorShape(13);
    /// <img src="http://doc.qt.io/qt-5/images/cursor-forbidden.png" alt=""> A slashed circle cursor, typically used during drag and drop operations to indicate that dragged content cannot be dropped on particular widgets or inside certain regions. (C++ enum variant: <span style='color: green;'>```ForbiddenCursor = 14```</span>)
    #[allow(non_upper_case_globals)]
    pub const ForbiddenCursor: crate::CursorShape = crate::CursorShape(14);
    /// <img src="http://doc.qt.io/qt-5/images/cursor-whatsthis.png" alt=""> An arrow with a question mark, typically used to indicate the presence of What's This? help for a widget. (C++ enum variant: <span style='color: green;'>```WhatsThisCursor = 15```</span>)
    #[allow(non_upper_case_globals)]
    pub const WhatsThisCursor: crate::CursorShape = crate::CursorShape(15);
    /// <img src="http://doc.qt.io/qt-5/images/cursor-busy.png" alt=""> An hourglass or watch cursor, usually shown during operations that allow the user to interact with the application while they are performed in the background. (C++ enum variant: <span style='color: green;'>```BusyCursor = 16```</span>)
    #[allow(non_upper_case_globals)]
    pub const BusyCursor: crate::CursorShape = crate::CursorShape(16);
    /// <img src="http://doc.qt.io/qt-5/images/cursor-openhand.png" alt=""> A cursor representing an open hand, typically used to indicate that the area under the cursor is the visible part of a canvas that the user can click and drag in order to scroll around. (C++ enum variant: <span style='color: green;'>```OpenHandCursor = 17```</span>)
    #[allow(non_upper_case_globals)]
    pub const OpenHandCursor: crate::CursorShape = crate::CursorShape(17);
    /// <img src="http://doc.qt.io/qt-5/images/cursor-closedhand.png" alt=""> A cursor representing a closed hand, typically used to indicate that a dragging operation is in progress that involves scrolling. (C++ enum variant: <span style='color: green;'>```ClosedHandCursor = 18```</span>)
    #[allow(non_upper_case_globals)]
    pub const ClosedHandCursor: crate::CursorShape = crate::CursorShape(18);
    /// A cursor that is usually used when dragging an item to copy it. (C++ enum variant: <span style='color: green;'>```DragCopyCursor = 19```</span>)
    #[allow(non_upper_case_globals)]
    pub const DragCopyCursor: crate::CursorShape = crate::CursorShape(19);
    /// A cursor that is usually used when dragging an item. (C++ enum variant: <span style='color: green;'>```DragMoveCursor = 20```</span>)
    #[allow(non_upper_case_globals)]
    pub const DragMoveCursor: crate::CursorShape = crate::CursorShape(20);
    /// A cursor that is usually used when dragging an item to make a link to it. (C++ enum variant: <span style='color: green;'>```DragLinkCursor = 21```</span>)
    #[allow(non_upper_case_globals)]
    pub const DragLinkCursor: crate::CursorShape = crate::CursorShape(21);
    /// C++ enum variant: <span style='color: green;'>```LastCursor = 21```</span>
    #[allow(non_upper_case_globals)]
    pub const LastCursor: crate::CursorShape = crate::CursorShape(21);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```BitmapCursor = 24```</span>)
    #[allow(non_upper_case_globals)]
    pub const BitmapCursor: crate::CursorShape = crate::CursorShape(24);
    /// C++ enum variant: <span style='color: green;'>```CustomCursor = 25```</span>
    #[allow(non_upper_case_globals)]
    pub const CustomCursor: crate::CursorShape = crate::CursorShape(25);
}

/// <p>This enum is used in widgets that can display both plain text and rich text, for example <a href="http://doc.qt.io/qt-5/qlabel.html">QLabel</a>. It is used for deciding whether a text string should be interpreted as one or the other. This is normally done by passing one of the enum values to a QTextEdit::setTextFormat() function.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::TextFormat```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#TextFormat-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum is used in widgets that can display both plain text and rich text, for example <a href="http://doc.qt.io/qt-5/qlabel.html">QLabel</a>. It is used for deciding whether a text string should be interpreted as one or the other. This is normally done by passing one of the enum values to a QTextEdit::setTextFormat() function.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct TextFormat(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for TextFormat {
    fn from(value: ::std::os::raw::c_int) -> Self {
        TextFormat(value)
    }
}

impl From<TextFormat> for ::std::os::raw::c_int {
    fn from(value: TextFormat) -> Self {
        value.0
    }
}

impl TextFormat {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl TextFormat {
    /// The text string is interpreted as a plain text string. (C++ enum variant: <span style='color: green;'>```PlainText = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const PlainText: crate::TextFormat = crate::TextFormat(0);
    /// The text string is interpreted as a rich text string. See <a href="http://doc.qt.io/qt-5/richtext-html-subset.html">Supported HTML Subset</a> for the definition of rich text. (C++ enum variant: <span style='color: green;'>```RichText = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const RichText: crate::TextFormat = crate::TextFormat(1);
    /// The text string is interpreted as for Qt::RichText if <a href="http://doc.qt.io/qt-5/qt.html#mightBeRichText">Qt::mightBeRichText</a>() returns <code>true</code>, otherwise as Qt::PlainText. (C++ enum variant: <span style='color: green;'>```AutoText = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const AutoText: crate::TextFormat = crate::TextFormat(2);
}

/// <p>This enum type defines what happens to the aspect ratio when scaling an rectangle.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::AspectRatioMode```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#AspectRatioMode-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type defines what happens to the aspect ratio when scaling an rectangle.</p>
/// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qimage-scaling.png" alt=""></p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsize.html#scale">QSize::scale</a>() and <a href="http://doc.qt.io/qt-5/qimage.html#scaled">QImage::scaled</a>().</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct AspectRatioMode(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for AspectRatioMode {
    fn from(value: ::std::os::raw::c_int) -> Self {
        AspectRatioMode(value)
    }
}

impl From<AspectRatioMode> for ::std::os::raw::c_int {
    fn from(value: AspectRatioMode) -> Self {
        value.0
    }
}

impl AspectRatioMode {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl AspectRatioMode {
    /// The size is scaled freely. The aspect ratio is not preserved. (C++ enum variant: <span style='color: green;'>```IgnoreAspectRatio = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const IgnoreAspectRatio: crate::AspectRatioMode = crate::AspectRatioMode(0);
    /// The size is scaled to a rectangle as large as possible inside a given rectangle, preserving the aspect ratio. (C++ enum variant: <span style='color: green;'>```KeepAspectRatio = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeepAspectRatio: crate::AspectRatioMode = crate::AspectRatioMode(1);
    /// The size is scaled to a rectangle as small as possible outside a given rectangle, preserving the aspect ratio. (C++ enum variant: <span style='color: green;'>```KeepAspectRatioByExpanding = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const KeepAspectRatioByExpanding: crate::AspectRatioMode = crate::AspectRatioMode(2);
}

/// <p>The DockWidgetAreas type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;DockWidgetArea&gt;. It stores an OR combination of DockWidgetArea values.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::DockWidgetArea```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#DockWidgetArea-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The DockWidgetAreas type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;DockWidgetArea&gt;. It stores an OR combination of DockWidgetArea values.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct DockWidgetArea(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for DockWidgetArea {
    fn from(value: ::std::os::raw::c_int) -> Self {
        DockWidgetArea(value)
    }
}

impl From<DockWidgetArea> for ::std::os::raw::c_int {
    fn from(value: DockWidgetArea) -> Self {
        value.0
    }
}

impl DockWidgetArea {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl DockWidgetArea {
    /// C++ enum variant: <span style='color: green;'>```LeftDockWidgetArea = 1```</span>
    #[allow(non_upper_case_globals)]
    pub const LeftDockWidgetArea: crate::DockWidgetArea = crate::DockWidgetArea(1);
    /// C++ enum variant: <span style='color: green;'>```RightDockWidgetArea = 2```</span>
    #[allow(non_upper_case_globals)]
    pub const RightDockWidgetArea: crate::DockWidgetArea = crate::DockWidgetArea(2);
    /// C++ enum variant: <span style='color: green;'>```TopDockWidgetArea = 4```</span>
    #[allow(non_upper_case_globals)]
    pub const TopDockWidgetArea: crate::DockWidgetArea = crate::DockWidgetArea(4);
    /// C++ enum variant: <span style='color: green;'>```BottomDockWidgetArea = 8```</span>
    #[allow(non_upper_case_globals)]
    pub const BottomDockWidgetArea: crate::DockWidgetArea = crate::DockWidgetArea(8);
    /// C++ enum variant: <span style='color: green;'>```DockWidgetArea_Mask = 15```</span>
    #[allow(non_upper_case_globals)]
    pub const DockWidgetAreaMask: crate::DockWidgetArea = crate::DockWidgetArea(15);
    /// C++ enum variant: <span style='color: green;'>```AllDockWidgetAreas = 15```</span>
    #[allow(non_upper_case_globals)]
    pub const AllDockWidgetAreas: crate::DockWidgetArea = crate::DockWidgetArea(15);
    /// C++ enum variant: <span style='color: green;'>```NoDockWidgetArea = 0```</span>
    #[allow(non_upper_case_globals)]
    pub const NoDockWidgetArea: crate::DockWidgetArea = crate::DockWidgetArea(0);
}

/// C++ enum: <span style='color: green;'>```Qt::DockWidgetAreaSizes```</span>.
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct DockWidgetAreaSizes(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for DockWidgetAreaSizes {
    fn from(value: ::std::os::raw::c_int) -> Self {
        DockWidgetAreaSizes(value)
    }
}

impl From<DockWidgetAreaSizes> for ::std::os::raw::c_int {
    fn from(value: DockWidgetAreaSizes) -> Self {
        value.0
    }
}

impl DockWidgetAreaSizes {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl DockWidgetAreaSizes {
    /// C++ enum variant: <span style='color: green;'>```NDockWidgetAreas = 4```</span>
    #[allow(non_upper_case_globals)]
    pub const NDockWidgetAreas: crate::DockWidgetAreaSizes = crate::DockWidgetAreaSizes(4);
}

/// <p>The ToolBarAreas type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;ToolBarArea&gt;. It stores an OR combination of ToolBarArea values.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::ToolBarArea```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#ToolBarArea-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The ToolBarAreas type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;ToolBarArea&gt;. It stores an OR combination of ToolBarArea values.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct ToolBarArea(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for ToolBarArea {
    fn from(value: ::std::os::raw::c_int) -> Self {
        ToolBarArea(value)
    }
}

impl From<ToolBarArea> for ::std::os::raw::c_int {
    fn from(value: ToolBarArea) -> Self {
        value.0
    }
}

impl ToolBarArea {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl ToolBarArea {
    /// C++ enum variant: <span style='color: green;'>```LeftToolBarArea = 1```</span>
    #[allow(non_upper_case_globals)]
    pub const LeftToolBarArea: crate::ToolBarArea = crate::ToolBarArea(1);
    /// C++ enum variant: <span style='color: green;'>```RightToolBarArea = 2```</span>
    #[allow(non_upper_case_globals)]
    pub const RightToolBarArea: crate::ToolBarArea = crate::ToolBarArea(2);
    /// C++ enum variant: <span style='color: green;'>```TopToolBarArea = 4```</span>
    #[allow(non_upper_case_globals)]
    pub const TopToolBarArea: crate::ToolBarArea = crate::ToolBarArea(4);
    /// C++ enum variant: <span style='color: green;'>```BottomToolBarArea = 8```</span>
    #[allow(non_upper_case_globals)]
    pub const BottomToolBarArea: crate::ToolBarArea = crate::ToolBarArea(8);
    /// C++ enum variant: <span style='color: green;'>```ToolBarArea_Mask = 15```</span>
    #[allow(non_upper_case_globals)]
    pub const ToolBarAreaMask: crate::ToolBarArea = crate::ToolBarArea(15);
    /// C++ enum variant: <span style='color: green;'>```AllToolBarAreas = 15```</span>
    #[allow(non_upper_case_globals)]
    pub const AllToolBarAreas: crate::ToolBarArea = crate::ToolBarArea(15);
    /// C++ enum variant: <span style='color: green;'>```NoToolBarArea = 0```</span>
    #[allow(non_upper_case_globals)]
    pub const NoToolBarArea: crate::ToolBarArea = crate::ToolBarArea(0);
}

/// C++ enum: <span style='color: green;'>```Qt::ToolBarAreaSizes```</span>.
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct ToolBarAreaSizes(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for ToolBarAreaSizes {
    fn from(value: ::std::os::raw::c_int) -> Self {
        ToolBarAreaSizes(value)
    }
}

impl From<ToolBarAreaSizes> for ::std::os::raw::c_int {
    fn from(value: ToolBarAreaSizes) -> Self {
        value.0
    }
}

impl ToolBarAreaSizes {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl ToolBarAreaSizes {
    /// C++ enum variant: <span style='color: green;'>```NToolBarAreas = 4```</span>
    #[allow(non_upper_case_globals)]
    pub const NToolBarAreas: crate::ToolBarAreaSizes = crate::ToolBarAreaSizes(4);
}

/// <p><b>Note: </b>For <code>ISODate</code> formats, each <code>Y</code>, <code>M</code> and <code>D</code> represents a single digit of the year, month and day used to specify the date. Each <code>H</code>, <code>M</code> and <code>S</code> represents a single digit of the hour, minute and second used to specify the time. The presence of a literal <code>T</code> character is used to separate the date and time when both are specified.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::DateFormat```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p><b>Note: </b>For <code>ISODate</code> formats, each <code>Y</code>, <code>M</code> and <code>D</code> represents a single digit of the year, month and day used to specify the date. Each <code>H</code>, <code>M</code> and <code>S</code> represents a single digit of the hour, minute and second used to specify the time. The presence of a literal <code>T</code> character is used to separate the date and time when both are specified.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct DateFormat(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for DateFormat {
    fn from(value: ::std::os::raw::c_int) -> Self {
        DateFormat(value)
    }
}

impl From<DateFormat> for ::std::os::raw::c_int {
    fn from(value: DateFormat) -> Self {
        value.0
    }
}

impl DateFormat {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl DateFormat {
    /// The default Qt format, which includes the day and month name, the day number in the month, and the year in full. The day and month names will be short, localized names. This is basically equivalent to using the date format string, "ddd MMM d yyyy". See <a href="http://doc.qt.io/qt-5/qdate.html#toString">QDate::toString</a>() for more information. (C++ enum variant: <span style='color: green;'>```TextDate = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const TextDate: crate::DateFormat = crate::DateFormat(0);
    /// <a href="http://www.iso.org/iso/catalogue_detail?csnumber=40874">ISO 8601</a> extended format: either <code>YYYY-MM-DD</code> for dates or <code>YYYY-MM-DDTHH:mm:ss</code>, <code>YYYY-MM-DDTHH:mm:ssTZD</code> (e.g., 1997-07-16T19:20:30+01:00) for combined dates and times. (C++ enum variant: <span style='color: green;'>```ISODate = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const ISODate: crate::DateFormat = crate::DateFormat(1);
    /// <i>This enum value is deprecated.</i> Use Qt::SystemLocaleShortDate instead (or Qt::SystemLocaleLongDate if you want long dates). (C++ enum variant: <span style='color: green;'>```SystemLocaleDate = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const SystemLocaleDate: crate::DateFormat = crate::DateFormat(2);
    /// <i>This enum value is deprecated.</i> Use Qt::SystemLocaleShortDate instead (or Qt::SystemLocaleLongDate if you want long dates). (C++ enum variant: <span style='color: green;'>```LocalDate = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const LocalDate: crate::DateFormat = crate::DateFormat(2);
    /// <i>This enum value is deprecated.</i> Use Qt::DefaultLocaleShortDate instead (or Qt::DefaultLocaleLongDate if you want long dates). (C++ enum variant: <span style='color: green;'>```LocaleDate = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const LocaleDate: crate::DateFormat = crate::DateFormat(3);
    /// The <a href="http://doc.qt.io/qt-5/qlocale.html#FormatType-enum">short format</a> used by the <a href="http://doc.qt.io/qt-5/qlocale.html#system">operating system</a>. (C++ enum variant: <span style='color: green;'>```SystemLocaleShortDate = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const SystemLocaleShortDate: crate::DateFormat = crate::DateFormat(4);
    /// The <a href="http://doc.qt.io/qt-5/qlocale.html#FormatType-enum">long format</a> used by the <a href="http://doc.qt.io/qt-5/qlocale.html#system">operating system</a>. (C++ enum variant: <span style='color: green;'>```SystemLocaleLongDate = 5```</span>)
    #[allow(non_upper_case_globals)]
    pub const SystemLocaleLongDate: crate::DateFormat = crate::DateFormat(5);
    /// The <a href="http://doc.qt.io/qt-5/qlocale.html#FormatType-enum">short format</a> specified by the <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">application's locale</a>. (C++ enum variant: <span style='color: green;'>```DefaultLocaleShortDate = 6```</span>)
    #[allow(non_upper_case_globals)]
    pub const DefaultLocaleShortDate: crate::DateFormat = crate::DateFormat(6);
    /// The <a href="http://doc.qt.io/qt-5/qlocale.html#FormatType-enum">long format</a> used by the <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">application's locale</a>. (C++ enum variant: <span style='color: green;'>```DefaultLocaleLongDate = 7```</span>)
    #[allow(non_upper_case_globals)]
    pub const DefaultLocaleLongDate: crate::DateFormat = crate::DateFormat(7);
    /// <a href="http://www.rfc-editor.org/rfc/rfc2822.txt">RFC 2822</a>, <a href="http://www.rfc-editor.org/rfc/rfc850.txt">RFC 850</a> and <a href="http://www.rfc-editor.org/rfc/rfc1036.txt">RFC 1036</a> format: either <code>[ddd,] dd MMM yyyy hh:mm[:ss] +/-TZ</code> or <code>ddd MMM dd yyyy hh:mm[:ss] +/-TZ</code> for combined dates and times. (C++ enum variant: <span style='color: green;'>```RFC2822Date = 8```</span>)
    #[allow(non_upper_case_globals)]
    pub const RFC2822Date: crate::DateFormat = crate::DateFormat(8);
    /// <a href="http://www.iso.org/iso/catalogue_detail?csnumber=40874">ISO 8601</a> extended format, including milliseconds if applicable. (C++ enum variant: <span style='color: green;'>```ISODateWithMs = 9```</span>)
    #[allow(non_upper_case_globals)]
    pub const ISODateWithMs: crate::DateFormat = crate::DateFormat(9);
}

/// C++ enum: <span style='color: green;'>```Qt::TimeSpec```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct TimeSpec(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for TimeSpec {
    fn from(value: ::std::os::raw::c_int) -> Self {
        TimeSpec(value)
    }
}

impl From<TimeSpec> for ::std::os::raw::c_int {
    fn from(value: TimeSpec) -> Self {
        value.0
    }
}

impl TimeSpec {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl TimeSpec {
    /// Locale dependent time (Timezones and Daylight Savings Time). (C++ enum variant: <span style='color: green;'>```LocalTime = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const LocalTime: crate::TimeSpec = crate::TimeSpec(0);
    /// Coordinated Universal Time, replaces Greenwich Mean Time. (C++ enum variant: <span style='color: green;'>```UTC = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const UTC: crate::TimeSpec = crate::TimeSpec(1);
    /// An offset in seconds from Coordinated Universal Time. (C++ enum variant: <span style='color: green;'>```OffsetFromUTC = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const OffsetFromUTC: crate::TimeSpec = crate::TimeSpec(2);
    /// A named time zone using a specific set of Daylight Savings rules. (C++ enum variant: <span style='color: green;'>```TimeZone = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const TimeZone: crate::TimeSpec = crate::TimeSpec(3);
}

/// C++ enum: <span style='color: green;'>```Qt::DayOfWeek```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#DayOfWeek-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct DayOfWeek(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for DayOfWeek {
    fn from(value: ::std::os::raw::c_int) -> Self {
        DayOfWeek(value)
    }
}

impl From<DayOfWeek> for ::std::os::raw::c_int {
    fn from(value: DayOfWeek) -> Self {
        value.0
    }
}

impl DayOfWeek {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl DayOfWeek {
    /// C++ enum variant: <span style='color: green;'>```Monday = 1```</span>
    #[allow(non_upper_case_globals)]
    pub const Monday: crate::DayOfWeek = crate::DayOfWeek(1);
    /// C++ enum variant: <span style='color: green;'>```Tuesday = 2```</span>
    #[allow(non_upper_case_globals)]
    pub const Tuesday: crate::DayOfWeek = crate::DayOfWeek(2);
    /// C++ enum variant: <span style='color: green;'>```Wednesday = 3```</span>
    #[allow(non_upper_case_globals)]
    pub const Wednesday: crate::DayOfWeek = crate::DayOfWeek(3);
    /// C++ enum variant: <span style='color: green;'>```Thursday = 4```</span>
    #[allow(non_upper_case_globals)]
    pub const Thursday: crate::DayOfWeek = crate::DayOfWeek(4);
    /// C++ enum variant: <span style='color: green;'>```Friday = 5```</span>
    #[allow(non_upper_case_globals)]
    pub const Friday: crate::DayOfWeek = crate::DayOfWeek(5);
    /// C++ enum variant: <span style='color: green;'>```Saturday = 6```</span>
    #[allow(non_upper_case_globals)]
    pub const Saturday: crate::DayOfWeek = crate::DayOfWeek(6);
    /// C++ enum variant: <span style='color: green;'>```Sunday = 7```</span>
    #[allow(non_upper_case_globals)]
    pub const Sunday: crate::DayOfWeek = crate::DayOfWeek(7);
}

/// <p>This enum type describes the various modes of <a href="http://doc.qt.io/qt-5/qabstractscrollarea.html">QAbstractScrollArea</a>'s scroll bars.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::ScrollBarPolicy```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#ScrollBarPolicy-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type describes the various modes of <a href="http://doc.qt.io/qt-5/qabstractscrollarea.html">QAbstractScrollArea</a>'s scroll bars.</p>
///
/// <p>(The modes for the horizontal and vertical scroll bars are independent.)</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct ScrollBarPolicy(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for ScrollBarPolicy {
    fn from(value: ::std::os::raw::c_int) -> Self {
        ScrollBarPolicy(value)
    }
}

impl From<ScrollBarPolicy> for ::std::os::raw::c_int {
    fn from(value: ScrollBarPolicy) -> Self {
        value.0
    }
}

impl ScrollBarPolicy {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl ScrollBarPolicy {
    /// <a href="http://doc.qt.io/qt-5/qabstractscrollarea.html">QAbstractScrollArea</a> shows a scroll bar when the content is too large to fit and not otherwise. This is the default. (C++ enum variant: <span style='color: green;'>```ScrollBarAsNeeded = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const ScrollBarAsNeeded: crate::ScrollBarPolicy = crate::ScrollBarPolicy(0);
    /// <a href="http://doc.qt.io/qt-5/qabstractscrollarea.html">QAbstractScrollArea</a> never shows a scroll bar. (C++ enum variant: <span style='color: green;'>```ScrollBarAlwaysOff = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const ScrollBarAlwaysOff: crate::ScrollBarPolicy = crate::ScrollBarPolicy(1);
    /// <a href="http://doc.qt.io/qt-5/qabstractscrollarea.html">QAbstractScrollArea</a> always shows a scroll bar. This property is ignored on systems with transient scroll bars (e.g., on Mac from version 10.7). (C++ enum variant: <span style='color: green;'>```ScrollBarAlwaysOn = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const ScrollBarAlwaysOn: crate::ScrollBarPolicy = crate::ScrollBarPolicy(2);
}

/// C++ enum: <span style='color: green;'>```Qt::CaseSensitivity```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct CaseSensitivity(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for CaseSensitivity {
    fn from(value: ::std::os::raw::c_int) -> Self {
        CaseSensitivity(value)
    }
}

impl From<CaseSensitivity> for ::std::os::raw::c_int {
    fn from(value: CaseSensitivity) -> Self {
        value.0
    }
}

impl CaseSensitivity {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl CaseSensitivity {
    /// C++ enum variant: <span style='color: green;'>```CaseInsensitive = 0```</span>
    #[allow(non_upper_case_globals)]
    pub const CaseInsensitive: crate::CaseSensitivity = crate::CaseSensitivity(0);
    /// C++ enum variant: <span style='color: green;'>```CaseSensitive = 1```</span>
    #[allow(non_upper_case_globals)]
    pub const CaseSensitive: crate::CaseSensitivity = crate::CaseSensitivity(1);
}

/// <p>This enum type specifies a corner in a rectangle:</p>
///
/// C++ enum: <span style='color: green;'>```Qt::Corner```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#Corner-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type specifies a corner in a rectangle:</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct Corner(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for Corner {
    fn from(value: ::std::os::raw::c_int) -> Self {
        Corner(value)
    }
}

impl From<Corner> for ::std::os::raw::c_int {
    fn from(value: Corner) -> Self {
        value.0
    }
}

impl Corner {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl Corner {
    /// The top-left corner of the rectangle. (C++ enum variant: <span style='color: green;'>```TopLeftCorner = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const TopLeftCorner: crate::Corner = crate::Corner(0);
    /// The top-right corner of the rectangle. (C++ enum variant: <span style='color: green;'>```TopRightCorner = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const TopRightCorner: crate::Corner = crate::Corner(1);
    /// The bottom-left corner of the rectangle. (C++ enum variant: <span style='color: green;'>```BottomLeftCorner = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const BottomLeftCorner: crate::Corner = crate::Corner(2);
    /// The bottom-right corner of the rectangle. (C++ enum variant: <span style='color: green;'>```BottomRightCorner = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const BottomRightCorner: crate::Corner = crate::Corner(3);
}

/// <p>This enum type specifies an edge in a rectangle:</p>
///
/// C++ enum: <span style='color: green;'>```Qt::Edge```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#Edge-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type specifies an edge in a rectangle:</p>
///
/// <p>This enum was introduced or modified in  Qt 5.1.</p>
/// <p>The Edges type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;Edge&gt;. It stores an OR combination of Edge values.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct Edge(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for Edge {
    fn from(value: ::std::os::raw::c_int) -> Self {
        Edge(value)
    }
}

impl From<Edge> for ::std::os::raw::c_int {
    fn from(value: Edge) -> Self {
        value.0
    }
}

impl Edge {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl Edge {
    /// The top edge of the rectangle. (C++ enum variant: <span style='color: green;'>```TopEdge = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const TopEdge: crate::Edge = crate::Edge(1);
    /// The left edge of the rectangle. (C++ enum variant: <span style='color: green;'>```LeftEdge = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const LeftEdge: crate::Edge = crate::Edge(2);
    /// The right edge of the rectangle. (C++ enum variant: <span style='color: green;'>```RightEdge = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const RightEdge: crate::Edge = crate::Edge(4);
    /// The bottom edge of the rectangle. (C++ enum variant: <span style='color: green;'>```BottomEdge = 8```</span>)
    #[allow(non_upper_case_globals)]
    pub const BottomEdge: crate::Edge = crate::Edge(8);
}

/// <p>This enum describes the types of connection that can be used between signals and slots. In particular, it determines whether a particular signal is delivered to a slot immediately or queued for delivery at a later time.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::ConnectionType```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#ConnectionType-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the types of connection that can be used between signals and slots. In particular, it determines whether a particular signal is delivered to a slot immediately or queued for delivery at a later time.</p>
///
/// <p>With queued connections, the parameters must be of types that are known to Qt's meta-object system, because Qt needs to copy the arguments to store them in an event behind the scenes. If you try to use a queued connection and get the error message:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qobject.html">QObject</a></span><span class="operator">::</span>connect: Cannot queue arguments of type <span class="char">'MyType'</span>
///
/// </pre>
/// <p>Call <a href="http://doc.qt.io/qt-5/qmetatype.html#qRegisterMetaType-1">qRegisterMetaType</a>() to register the data type before you establish the connection.</p>
/// <p>When using signals and slots with multiple threads, see <a href="http://doc.qt.io/qt-5/threads-qobject.html#signals-and-slots-across-threads">Signals and Slots Across Threads</a>.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/threads.html">Thread Support in Qt</a>, <a href="http://doc.qt.io/qt-5/qobject.html#connect">QObject::connect</a>(), <a href="http://doc.qt.io/qt-5/qmetatype.html#qRegisterMetaType-1">qRegisterMetaType</a>(), and <a href="http://doc.qt.io/qt-5/qmetatype.html#Q_DECLARE_METATYPE">Q_DECLARE_METATYPE</a>().</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct ConnectionType(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for ConnectionType {
    fn from(value: ::std::os::raw::c_int) -> Self {
        ConnectionType(value)
    }
}

impl From<ConnectionType> for ::std::os::raw::c_int {
    fn from(value: ConnectionType) -> Self {
        value.0
    }
}

impl ConnectionType {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl ConnectionType {
    /// <b>(Default)</b> If the receiver <a href="http://doc.qt.io/qt-5/qobject.html#thread-affinity">lives in</a> the thread that emits the signal, Qt::DirectConnection is used. Otherwise, Qt::QueuedConnection is used. The connection type is determined when the signal is emitted. (C++ enum variant: <span style='color: green;'>```AutoConnection = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const AutoConnection: crate::ConnectionType = crate::ConnectionType(0);
    /// The slot is invoked immediately when the signal is emitted. The slot is executed in the signalling thread. (C++ enum variant: <span style='color: green;'>```DirectConnection = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const DirectConnection: crate::ConnectionType = crate::ConnectionType(1);
    /// The slot is invoked when control returns to the event loop of the receiver's thread. The slot is executed in the receiver's thread. (C++ enum variant: <span style='color: green;'>```QueuedConnection = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const QueuedConnection: crate::ConnectionType = crate::ConnectionType(2);
    /// Same as Qt::QueuedConnection, except that the signalling thread blocks until the slot returns. This connection must <i>not</i> be used if the receiver lives in the signalling thread, or else the application will deadlock. (C++ enum variant: <span style='color: green;'>```BlockingQueuedConnection = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const BlockingQueuedConnection: crate::ConnectionType = crate::ConnectionType(3);
    /// This is a flag that can be combined with any one of the above connection types, using a bitwise OR. When Qt::UniqueConnection is set, <a href="http://doc.qt.io/qt-5/qobject.html#connect">QObject::connect</a>() will fail if the connection already exists (i.e. if the same signal is already connected to the same slot for the same pair of objects). This flag was introduced in Qt 4.6. (C++ enum variant: <span style='color: green;'>```UniqueConnection = 128```</span>)
    #[allow(non_upper_case_globals)]
    pub const UniqueConnection: crate::ConnectionType = crate::ConnectionType(128);
}

/// <p>For a <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::Shortcut</a> event to occur, the shortcut's key sequence must be entered by the user in a context where the shortcut is active. The possible contexts are these:</p>
///
/// C++ enum: <span style='color: green;'>```Qt::ShortcutContext```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#ShortcutContext-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>For a <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::Shortcut</a> event to occur, the shortcut's key sequence must be entered by the user in a context where the shortcut is active. The possible contexts are these:</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct ShortcutContext(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for ShortcutContext {
    fn from(value: ::std::os::raw::c_int) -> Self {
        ShortcutContext(value)
    }
}

impl From<ShortcutContext> for ::std::os::raw::c_int {
    fn from(value: ShortcutContext) -> Self {
        value.0
    }
}

impl ShortcutContext {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl ShortcutContext {
    /// The shortcut is active when its parent widget has focus. (C++ enum variant: <span style='color: green;'>```WidgetShortcut = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const WidgetShortcut: crate::ShortcutContext = crate::ShortcutContext(0);
    /// The shortcut is active when its parent widget is a logical subwidget of the active top-level window. (C++ enum variant: <span style='color: green;'>```WindowShortcut = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const WindowShortcut: crate::ShortcutContext = crate::ShortcutContext(1);
    /// The shortcut is active when one of the applications windows are active. (C++ enum variant: <span style='color: green;'>```ApplicationShortcut = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const ApplicationShortcut: crate::ShortcutContext = crate::ShortcutContext(2);
    /// The shortcut is active when its parent widget, or any of its children has focus. Children which are top-level widgets, except pop-ups, are not affected by this shortcut context. (C++ enum variant: <span style='color: green;'>```WidgetWithChildrenShortcut = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const WidgetWithChildrenShortcut: crate::ShortcutContext = crate::ShortcutContext(3);
}

/// <p>Specifies which method should be used to fill the paths and polygons.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::FillRule```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#FillRule-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Specifies which method should be used to fill the paths and polygons.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct FillRule(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for FillRule {
    fn from(value: ::std::os::raw::c_int) -> Self {
        FillRule(value)
    }
}

impl From<FillRule> for ::std::os::raw::c_int {
    fn from(value: FillRule) -> Self {
        value.0
    }
}

impl FillRule {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl FillRule {
    /// Specifies that the region is filled using the odd even fill rule. With this rule, we determine whether a point is inside the shape by using the following method. Draw a horizontal line from the point to a location outside the shape, and count the number of intersections. If the number of intersections is an odd number, the point is inside the shape. This mode is the default. (C++ enum variant: <span style='color: green;'>```OddEvenFill = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const OddEvenFill: crate::FillRule = crate::FillRule(0);
    /// Specifies that the region is filled using the non zero winding rule. With this rule, we determine whether a point is inside the shape by using the following method. Draw a horizontal line from the point to a location outside the shape. Determine whether the direction of the line at each intersection point is up or down. The winding number is determined by summing the direction of each intersection. If the number is non zero, the point is inside the shape. This fill mode can also in most cases be considered as the intersection of closed shapes. (C++ enum variant: <span style='color: green;'>```WindingFill = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const WindingFill: crate::FillRule = crate::FillRule(1);
}

/// <p>This enum specifies the behavior of the <a href="http://doc.qt.io/qt-5/qpixmap.html#createMaskFromColor">QPixmap::createMaskFromColor</a>() and <a href="http://doc.qt.io/qt-5/qimage.html#createMaskFromColor">QImage::createMaskFromColor</a>() functions.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::MaskMode```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#MaskMode-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum specifies the behavior of the <a href="http://doc.qt.io/qt-5/qpixmap.html#createMaskFromColor">QPixmap::createMaskFromColor</a>() and <a href="http://doc.qt.io/qt-5/qimage.html#createMaskFromColor">QImage::createMaskFromColor</a>() functions.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct MaskMode(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for MaskMode {
    fn from(value: ::std::os::raw::c_int) -> Self {
        MaskMode(value)
    }
}

impl From<MaskMode> for ::std::os::raw::c_int {
    fn from(value: MaskMode) -> Self {
        value.0
    }
}

impl MaskMode {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl MaskMode {
    /// Creates a mask where all pixels matching the given color are opaque. (C++ enum variant: <span style='color: green;'>```MaskInColor = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const MaskInColor: crate::MaskMode = crate::MaskMode(0);
    /// Creates a mask where all pixels matching the given color are transparent. (C++ enum variant: <span style='color: green;'>```MaskOutColor = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const MaskOutColor: crate::MaskMode = crate::MaskMode(1);
}

/// C++ enum: <span style='color: green;'>```Qt::ClipOperation```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#ClipOperation-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct ClipOperation(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for ClipOperation {
    fn from(value: ::std::os::raw::c_int) -> Self {
        ClipOperation(value)
    }
}

impl From<ClipOperation> for ::std::os::raw::c_int {
    fn from(value: ClipOperation) -> Self {
        value.0
    }
}

impl ClipOperation {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl ClipOperation {
    /// This operation turns clipping off. (C++ enum variant: <span style='color: green;'>```NoClip = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const NoClip: crate::ClipOperation = crate::ClipOperation(0);
    /// Replaces the current clip path/rect/region with the one supplied in the function call. (C++ enum variant: <span style='color: green;'>```ReplaceClip = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const ReplaceClip: crate::ClipOperation = crate::ClipOperation(1);
    /// Intersects the current clip path/rect/region with the one supplied in the function call. (C++ enum variant: <span style='color: green;'>```IntersectClip = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const IntersectClip: crate::ClipOperation = crate::ClipOperation(2);
}

/// <p>This enum is used in <a href="http://doc.qt.io/qt-5/qgraphicsitem.html">QGraphicsItem</a>, <a href="http://doc.qt.io/qt-5/qgraphicsscene.html">QGraphicsScene</a> and <a href="http://doc.qt.io/qt-5/qgraphicsview.html">QGraphicsView</a> to specify how items are selected, or how to determine if shapes and items collide.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::ItemSelectionMode```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#ItemSelectionMode-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum is used in <a href="http://doc.qt.io/qt-5/qgraphicsitem.html">QGraphicsItem</a>, <a href="http://doc.qt.io/qt-5/qgraphicsscene.html">QGraphicsScene</a> and <a href="http://doc.qt.io/qt-5/qgraphicsview.html">QGraphicsView</a> to specify how items are selected, or how to determine if shapes and items collide.</p>
///
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qgraphicsscene.html#items">QGraphicsScene::items</a>(), <a href="http://doc.qt.io/qt-5/qgraphicsscene.html#collidingItems">QGraphicsScene::collidingItems</a>(), <a href="http://doc.qt.io/qt-5/qgraphicsview.html#items">QGraphicsView::items</a>(), <a href="http://doc.qt.io/qt-5/qgraphicsitem.html#collidesWithItem">QGraphicsItem::collidesWithItem</a>(), and <a href="http://doc.qt.io/qt-5/qgraphicsitem.html#collidesWithPath">QGraphicsItem::collidesWithPath</a>().</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct ItemSelectionMode(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for ItemSelectionMode {
    fn from(value: ::std::os::raw::c_int) -> Self {
        ItemSelectionMode(value)
    }
}

impl From<ItemSelectionMode> for ::std::os::raw::c_int {
    fn from(value: ItemSelectionMode) -> Self {
        value.0
    }
}

impl ItemSelectionMode {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl ItemSelectionMode {
    /// The output list contains only items whose <a href="http://doc.qt.io/qt-5/qgraphicsitem.html#shape">shape</a> is fully contained inside the selection area. Items that intersect with the area's outline are not included. (C++ enum variant: <span style='color: green;'>```ContainsItemShape = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const ContainsItemShape: crate::ItemSelectionMode = crate::ItemSelectionMode(0);
    /// The output list contains both items whose <a href="http://doc.qt.io/qt-5/qgraphicsitem.html#shape">shape</a> is fully contained inside the selection area, and items that intersect with the area's outline. This is a common mode for rubber band selection. (C++ enum variant: <span style='color: green;'>```IntersectsItemShape = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const IntersectsItemShape: crate::ItemSelectionMode = crate::ItemSelectionMode(1);
    /// The output list contains only items whose <a href="http://doc.qt.io/qt-5/qgraphicsitem.html#boundingRect">bounding rectangle</a> is fully contained inside the selection area. Items that intersect with the area's outline are not included. (C++ enum variant: <span style='color: green;'>```ContainsItemBoundingRect = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const ContainsItemBoundingRect: crate::ItemSelectionMode = crate::ItemSelectionMode(2);
    /// The output list contains both items whose <a href="http://doc.qt.io/qt-5/qgraphicsitem.html#boundingRect">bounding rectangle</a> is fully contained inside the selection area, and items that intersect with the area's outline. This method is commonly used for determining areas that need redrawing. (C++ enum variant: <span style='color: green;'>```IntersectsItemBoundingRect = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const IntersectsItemBoundingRect: crate::ItemSelectionMode = crate::ItemSelectionMode(3);
}

/// <p>This enum is used in <a href="http://doc.qt.io/qt-5/qgraphicsscene.html">QGraphicsScene</a> to specify what to do with currently selected items when setting a selection area.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::ItemSelectionOperation```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#ItemSelectionOperation-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum is used in <a href="http://doc.qt.io/qt-5/qgraphicsscene.html">QGraphicsScene</a> to specify what to do with currently selected items when setting a selection area.</p>
///
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qgraphicsscene.html#setSelectionArea-1">QGraphicsScene::setSelectionArea</a>().</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct ItemSelectionOperation(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for ItemSelectionOperation {
    fn from(value: ::std::os::raw::c_int) -> Self {
        ItemSelectionOperation(value)
    }
}

impl From<ItemSelectionOperation> for ::std::os::raw::c_int {
    fn from(value: ItemSelectionOperation) -> Self {
        value.0
    }
}

impl ItemSelectionOperation {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl ItemSelectionOperation {
    /// The currently selected items are replaced by items in the selection area. (C++ enum variant: <span style='color: green;'>```ReplaceSelection = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const ReplaceSelection: crate::ItemSelectionOperation = crate::ItemSelectionOperation(0);
    /// The items in the selection area are added to the currently selected items. (C++ enum variant: <span style='color: green;'>```AddToSelection = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const AddToSelection: crate::ItemSelectionOperation = crate::ItemSelectionOperation(1);
}

/// <p>This enum type defines whether image transformations (e.g., scaling) should be smooth or not.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::TransformationMode```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#TransformationMode-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type defines whether image transformations (e.g., scaling) should be smooth or not.</p>
///
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qimage.html#scaled">QImage::scaled</a>().</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct TransformationMode(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for TransformationMode {
    fn from(value: ::std::os::raw::c_int) -> Self {
        TransformationMode(value)
    }
}

impl From<TransformationMode> for ::std::os::raw::c_int {
    fn from(value: TransformationMode) -> Self {
        value.0
    }
}

impl TransformationMode {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl TransformationMode {
    /// The transformation is performed quickly, with no smoothing. (C++ enum variant: <span style='color: green;'>```FastTransformation = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const FastTransformation: crate::TransformationMode = crate::TransformationMode(0);
    /// The resulting image is transformed using bilinear filtering. (C++ enum variant: <span style='color: green;'>```SmoothTransformation = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const SmoothTransformation: crate::TransformationMode = crate::TransformationMode(1);
}

/// <p>This enum type defines three values to represent the three axes in the cartesian coordinate system.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::Axis```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#Axis-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type defines three values to represent the three axes in the cartesian coordinate system.</p>
///
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtransform.html#rotate">QTransform::rotate</a>() and <a href="http://doc.qt.io/qt-5/qtransform.html#rotateRadians">QTransform::rotateRadians</a>().</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct Axis(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for Axis {
    fn from(value: ::std::os::raw::c_int) -> Self {
        Axis(value)
    }
}

impl From<Axis> for ::std::os::raw::c_int {
    fn from(value: Axis) -> Self {
        value.0
    }
}

impl Axis {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl Axis {
    /// The X axis. (C++ enum variant: <span style='color: green;'>```XAxis = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const XAxis: crate::Axis = crate::Axis(0);
    /// The Y axis. (C++ enum variant: <span style='color: green;'>```YAxis = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const YAxis: crate::Axis = crate::Axis(1);
    /// The Z axis. (C++ enum variant: <span style='color: green;'>```ZAxis = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const ZAxis: crate::Axis = crate::Axis(2);
}

/// <p>This enum specifies why the focus changed. It will be passed through <a href="http://doc.qt.io/qt-5/qwidget.html#setFocus-1">QWidget::setFocus</a> and can be retrieved in the <a href="http://doc.qt.io/qt-5/qfocusevent.html">QFocusEvent</a> sent to the widget upon focus change.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::FocusReason```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#FocusReason-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum specifies why the focus changed. It will be passed through <a href="http://doc.qt.io/qt-5/qwidget.html#setFocus-1">QWidget::setFocus</a> and can be retrieved in the <a href="http://doc.qt.io/qt-5/qfocusevent.html">QFocusEvent</a> sent to the widget upon focus change.</p>
///
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/focus.html">Keyboard Focus in Widgets</a>.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct FocusReason(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for FocusReason {
    fn from(value: ::std::os::raw::c_int) -> Self {
        FocusReason(value)
    }
}

impl From<FocusReason> for ::std::os::raw::c_int {
    fn from(value: FocusReason) -> Self {
        value.0
    }
}

impl FocusReason {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl FocusReason {
    /// A mouse action occurred. (C++ enum variant: <span style='color: green;'>```MouseFocusReason = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const MouseFocusReason: crate::FocusReason = crate::FocusReason(0);
    /// The Tab key was pressed. (C++ enum variant: <span style='color: green;'>```TabFocusReason = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const TabFocusReason: crate::FocusReason = crate::FocusReason(1);
    /// A Backtab occurred. The input for this may include the Shift or Control keys; e.g. Shift+Tab. (C++ enum variant: <span style='color: green;'>```BacktabFocusReason = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const BacktabFocusReason: crate::FocusReason = crate::FocusReason(2);
    /// The window system made this window either active or inactive. (C++ enum variant: <span style='color: green;'>```ActiveWindowFocusReason = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const ActiveWindowFocusReason: crate::FocusReason = crate::FocusReason(3);
    /// The application opened/closed a pop-up that grabbed/released the keyboard focus. (C++ enum variant: <span style='color: green;'>```PopupFocusReason = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const PopupFocusReason: crate::FocusReason = crate::FocusReason(4);
    /// The user typed a label's buddy shortcut (C++ enum variant: <span style='color: green;'>```ShortcutFocusReason = 5```</span>)
    #[allow(non_upper_case_globals)]
    pub const ShortcutFocusReason: crate::FocusReason = crate::FocusReason(5);
    /// The menu bar took focus. (C++ enum variant: <span style='color: green;'>```MenuBarFocusReason = 6```</span>)
    #[allow(non_upper_case_globals)]
    pub const MenuBarFocusReason: crate::FocusReason = crate::FocusReason(6);
    /// Another reason, usually application-specific. (C++ enum variant: <span style='color: green;'>```OtherFocusReason = 7```</span>)
    #[allow(non_upper_case_globals)]
    pub const OtherFocusReason: crate::FocusReason = crate::FocusReason(7);
    /// C++ enum variant: <span style='color: green;'>```NoFocusReason = 8```</span>
    #[allow(non_upper_case_globals)]
    pub const NoFocusReason: crate::FocusReason = crate::FocusReason(8);
}

/// <p>This enum type defines the various policies a widget can have with respect to showing a context menu.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::ContextMenuPolicy```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#ContextMenuPolicy-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type defines the various policies a widget can have with respect to showing a context menu.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct ContextMenuPolicy(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for ContextMenuPolicy {
    fn from(value: ::std::os::raw::c_int) -> Self {
        ContextMenuPolicy(value)
    }
}

impl From<ContextMenuPolicy> for ::std::os::raw::c_int {
    fn from(value: ContextMenuPolicy) -> Self {
        value.0
    }
}

impl ContextMenuPolicy {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl ContextMenuPolicy {
    /// the widget does not feature a context menu, context menu handling is deferred to the widget's parent. (C++ enum variant: <span style='color: green;'>```NoContextMenu = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const NoContextMenu: crate::ContextMenuPolicy = crate::ContextMenuPolicy(0);
    /// the widget's <a href="http://doc.qt.io/qt-5/qwidget.html#contextMenuEvent">QWidget::contextMenuEvent</a>() handler is called. (C++ enum variant: <span style='color: green;'>```DefaultContextMenu = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const DefaultContextMenu: crate::ContextMenuPolicy = crate::ContextMenuPolicy(1);
    /// the widget displays its <a href="http://doc.qt.io/qt-5/qwidget.html#actions">QWidget::actions</a>() as context menu. (C++ enum variant: <span style='color: green;'>```ActionsContextMenu = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const ActionsContextMenu: crate::ContextMenuPolicy = crate::ContextMenuPolicy(2);
    /// the widget emits the <a href="http://doc.qt.io/qt-5/qwidget.html#customContextMenuRequested">QWidget::customContextMenuRequested</a>() signal. (C++ enum variant: <span style='color: green;'>```CustomContextMenu = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const CustomContextMenu: crate::ContextMenuPolicy = crate::ContextMenuPolicy(3);
    /// the widget does not feature a context menu, and in contrast to <code>NoContextMenu</code>, the handling is <i>not</i> deferred to the widget's parent. This means that all right mouse button events are guaranteed to be delivered to the widget itself through <a href="http://doc.qt.io/qt-5/qwidget.html#mousePressEvent">QWidget::mousePressEvent</a>(), and <a href="http://doc.qt.io/qt-5/qwidget.html#mouseReleaseEvent">QWidget::mouseReleaseEvent</a>(). (C++ enum variant: <span style='color: green;'>```PreventContextMenu = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const PreventContextMenu: crate::ContextMenuPolicy = crate::ContextMenuPolicy(4);
}

/// <p>Masks:</p>
///
/// C++ enum: <span style='color: green;'>```Qt::InputMethodQuery```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#InputMethodQuery-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Masks:</p>
///
/// <p>The InputMethodQueries type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;InputMethodQuery&gt;. It stores an OR combination of InputMethodQuery values.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qinputmethodqueryevent.html">QInputMethodQueryEvent</a>.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct InputMethodQuery(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for InputMethodQuery {
    fn from(value: ::std::os::raw::c_int) -> Self {
        InputMethodQuery(value)
    }
}

impl From<InputMethodQuery> for ::std::os::raw::c_int {
    fn from(value: InputMethodQuery) -> Self {
        value.0
    }
}

impl InputMethodQuery {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl InputMethodQuery {
    /// The widget accepts input method input. (C++ enum variant: <span style='color: green;'>```ImEnabled = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImEnabled: crate::InputMethodQuery = crate::InputMethodQuery(1);
    /// The rectangle covering the area of the input cursor in widget coordinates. (C++ enum variant: <span style='color: green;'>```ImCursorRectangle = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImCursorRectangle: crate::InputMethodQuery = crate::InputMethodQuery(2);
    /// This query is obsolete. Use <code>ImCursorRectangle</code> instead. (C++ enum variant: <span style='color: green;'>```ImMicroFocus = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImMicroFocus: crate::InputMethodQuery = crate::InputMethodQuery(2);
    /// The currently used font for text input. (C++ enum variant: <span style='color: green;'>```ImFont = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImFont: crate::InputMethodQuery = crate::InputMethodQuery(4);
    /// The logical position of the cursor within the text surrounding the input area (see <code>ImSurroundingText</code>). (C++ enum variant: <span style='color: green;'>```ImCursorPosition = 8```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImCursorPosition: crate::InputMethodQuery = crate::InputMethodQuery(8);
    /// The plain text around the input area, for example the current paragraph. (C++ enum variant: <span style='color: green;'>```ImSurroundingText = 16```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImSurroundingText: crate::InputMethodQuery = crate::InputMethodQuery(16);
    /// The currently selected text. (C++ enum variant: <span style='color: green;'>```ImCurrentSelection = 32```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImCurrentSelection: crate::InputMethodQuery = crate::InputMethodQuery(32);
    /// The maximum number of characters that the widget can hold. If there is no limit, <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant">QVariant::QVariant</a>() is returned. (C++ enum variant: <span style='color: green;'>```ImMaximumTextLength = 64```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImMaximumTextLength: crate::InputMethodQuery = crate::InputMethodQuery(64);
    /// The position of the selection anchor. This may be less or greater than <code>ImCursorPosition</code>, depending on which side of selection the cursor is. If there is no selection, it returns the same as <code>ImCursorPosition</code>. (C++ enum variant: <span style='color: green;'>```ImAnchorPosition = 128```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImAnchorPosition: crate::InputMethodQuery = crate::InputMethodQuery(128);
    /// The hints for input method on expected input. (See <a href="http://doc.qt.io/qt-5/qt.html#InputMethodHint-enum">Qt::InputMethodHints</a>) (C++ enum variant: <span style='color: green;'>```ImHints = 256```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImHints: crate::InputMethodQuery = crate::InputMethodQuery(256);
    /// The preferred input language. (C++ enum variant: <span style='color: green;'>```ImPreferredLanguage = 512```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImPreferredLanguage: crate::InputMethodQuery = crate::InputMethodQuery(512);
    /// The logical position of the cursor within the entire document. (C++ enum variant: <span style='color: green;'>```ImAbsolutePosition = 1024```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImAbsolutePosition: crate::InputMethodQuery = crate::InputMethodQuery(1024);
    /// The plain text before the cursor. The widget can decide how much text to return, but <b>must</b> not return an empty string unless the cursor is at the start of the document. (C++ enum variant: <span style='color: green;'>```ImTextBeforeCursor = 2048```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImTextBeforeCursor: crate::InputMethodQuery = crate::InputMethodQuery(2048);
    /// The plain text after the cursor. The widget can decide how much text to return, but <b>must</b> not return an empty string unless the cursor is at the end of the document. (C++ enum variant: <span style='color: green;'>```ImTextAfterCursor = 4096```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImTextAfterCursor: crate::InputMethodQuery = crate::InputMethodQuery(4096);
    /// The Enter key type. (C++ enum variant: <span style='color: green;'>```ImEnterKeyType = 8192```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImEnterKeyType: crate::InputMethodQuery = crate::InputMethodQuery(8192);
    /// The bounding rectangle of the selection anchor. This value has been added in Qt 5.7. (C++ enum variant: <span style='color: green;'>```ImAnchorRectangle = 16384```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImAnchorRectangle: crate::InputMethodQuery = crate::InputMethodQuery(16384);
    /// The actual exposed input item rectangle. Parts of the input item might be clipped. This value will take clipping into consideration and return the actual painted item rectangle. The rectangle is in widget coordinates. (C++ enum variant: <span style='color: green;'>```ImInputItemClipRectangle = 32768```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImInputItemClipRectangle: crate::InputMethodQuery = crate::InputMethodQuery(32768);
    /// Platform specific data for input method. (C++ enum variant: <span style='color: green;'>```ImPlatformData = -2147483648```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImPlatformData: crate::InputMethodQuery = crate::InputMethodQuery(-2147483648);
    /// Commonly changed properties on input. (C++ enum variant: <span style='color: green;'>```ImQueryInput = 16570```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImQueryInput: crate::InputMethodQuery = crate::InputMethodQuery(16570);
    /// Query for all input method properties. (C++ enum variant: <span style='color: green;'>```ImQueryAll = -1```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImQueryAll: crate::InputMethodQuery = crate::InputMethodQuery(-1);
}

/// <p>Flags that alter the behavior:</p>
///
/// C++ enum: <span style='color: green;'>```Qt::InputMethodHint```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#InputMethodHint-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Flags that alter the behavior:</p>
///
/// <p>Flags that restrict input (exclusive flags):</p>
///
/// <p>Masks:</p>
///
/// <p><b>Note: </b>If several exclusive flags are OR-ed together, the resulting character set will consist of the union of the specified sets. For instance specifying <code>ImhNumbersOnly</code> and <code>ImhUppercaseOnly</code> would yield a set consisting of numbers and uppercase letters.</p><p>The InputMethodHints type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;InputMethodHint&gt;. It stores an OR combination of InputMethodHint values.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qgraphicsitem.html#inputMethodHints">QGraphicsItem::inputMethodHints</a>().</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct InputMethodHint(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for InputMethodHint {
    fn from(value: ::std::os::raw::c_int) -> Self {
        InputMethodHint(value)
    }
}

impl From<InputMethodHint> for ::std::os::raw::c_int {
    fn from(value: InputMethodHint) -> Self {
        value.0
    }
}

impl InputMethodHint {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl InputMethodHint {
    /// No hints. (C++ enum variant: <span style='color: green;'>```ImhNone = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImhNone: crate::InputMethodHint = crate::InputMethodHint(0);
    /// The input method should not show the characters while typing. This is automatically set when setting <a href="http://doc.qt.io/qt-5/qlineedit.html#echoMode-prop">QLineEdit::echoMode</a> to <code>Password</code>. Note that setting <code>ImhHiddenText</code> does not change the echo mode. (C++ enum variant: <span style='color: green;'>```ImhHiddenText = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImhHiddenText: crate::InputMethodHint = crate::InputMethodHint(1);
    /// Typed text should not be stored by the active input method in any persistent storage like predictive user dictionary. (C++ enum variant: <span style='color: green;'>```ImhSensitiveData = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImhSensitiveData: crate::InputMethodHint = crate::InputMethodHint(2);
    /// The input method should not try to automatically switch to upper case when a sentence ends. (C++ enum variant: <span style='color: green;'>```ImhNoAutoUppercase = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImhNoAutoUppercase: crate::InputMethodHint = crate::InputMethodHint(4);
    /// Numbers are preferred (but not required). (C++ enum variant: <span style='color: green;'>```ImhPreferNumbers = 8```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImhPreferNumbers: crate::InputMethodHint = crate::InputMethodHint(8);
    /// Upper case letters are preferred (but not required). (C++ enum variant: <span style='color: green;'>```ImhPreferUppercase = 16```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImhPreferUppercase: crate::InputMethodHint = crate::InputMethodHint(16);
    /// Lower case letters are preferred (but not required). (C++ enum variant: <span style='color: green;'>```ImhPreferLowercase = 32```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImhPreferLowercase: crate::InputMethodHint = crate::InputMethodHint(32);
    /// Do not use predictive text (i.e. dictionary lookup) while typing. (C++ enum variant: <span style='color: green;'>```ImhNoPredictiveText = 64```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImhNoPredictiveText: crate::InputMethodHint = crate::InputMethodHint(64);
    /// The text editor functions as a date field. (C++ enum variant: <span style='color: green;'>```ImhDate = 128```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImhDate: crate::InputMethodHint = crate::InputMethodHint(128);
    /// The text editor functions as a time field. (C++ enum variant: <span style='color: green;'>```ImhTime = 256```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImhTime: crate::InputMethodHint = crate::InputMethodHint(256);
    /// Latin characters are preferred (but not required). (C++ enum variant: <span style='color: green;'>```ImhPreferLatin = 512```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImhPreferLatin: crate::InputMethodHint = crate::InputMethodHint(512);
    /// Multiple lines can be entered into the text field. (C++ enum variant: <span style='color: green;'>```ImhMultiLine = 1024```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImhMultiLine: crate::InputMethodHint = crate::InputMethodHint(1024);
    /// Only digits are allowed. (C++ enum variant: <span style='color: green;'>```ImhDigitsOnly = 65536```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImhDigitsOnly: crate::InputMethodHint = crate::InputMethodHint(65536);
    /// Only number input is allowed. This includes decimal point and minus sign. (C++ enum variant: <span style='color: green;'>```ImhFormattedNumbersOnly = 131072```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImhFormattedNumbersOnly: crate::InputMethodHint = crate::InputMethodHint(131072);
    /// Only upper case letter input is allowed. (C++ enum variant: <span style='color: green;'>```ImhUppercaseOnly = 262144```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImhUppercaseOnly: crate::InputMethodHint = crate::InputMethodHint(262144);
    /// Only lower case letter input is allowed. (C++ enum variant: <span style='color: green;'>```ImhLowercaseOnly = 524288```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImhLowercaseOnly: crate::InputMethodHint = crate::InputMethodHint(524288);
    /// Only characters suitable for phone dialing are allowed. (C++ enum variant: <span style='color: green;'>```ImhDialableCharactersOnly = 1048576```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImhDialableCharactersOnly: crate::InputMethodHint = crate::InputMethodHint(1048576);
    /// Only characters suitable for email addresses are allowed. (C++ enum variant: <span style='color: green;'>```ImhEmailCharactersOnly = 2097152```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImhEmailCharactersOnly: crate::InputMethodHint = crate::InputMethodHint(2097152);
    /// Only characters suitable for URLs are allowed. (C++ enum variant: <span style='color: green;'>```ImhUrlCharactersOnly = 4194304```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImhUrlCharactersOnly: crate::InputMethodHint = crate::InputMethodHint(4194304);
    /// Only latin based input is allowed. (C++ enum variant: <span style='color: green;'>```ImhLatinOnly = 8388608```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImhLatinOnly: crate::InputMethodHint = crate::InputMethodHint(8388608);
    /// This mask yields nonzero if any of the exclusive flags are used. (C++ enum variant: <span style='color: green;'>```ImhExclusiveInputMask = -65536```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImhExclusiveInputMask: crate::InputMethodHint = crate::InputMethodHint(-65536);
    /// Do not use built-in edit menu. This flag was introduced in Qt 5.11. (C++ enum variant: <span style='color: green;'>```ImhNoEditMenu = 2048```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImhNoEditMenu: crate::InputMethodHint = crate::InputMethodHint(2048);
    /// Do not use built-in text cursor and selection handles. This flag was introduced in Qt 5.11. (C++ enum variant: <span style='color: green;'>```ImhNoTextHandles = 4096```</span>)
    #[allow(non_upper_case_globals)]
    pub const ImhNoTextHandles: crate::InputMethodHint = crate::InputMethodHint(4096);
}

/// <p>This can be used to alter the appearance of the Return key on an on-screen keyboard.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::EnterKeyType```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#EnterKeyType-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This can be used to alter the appearance of the Return key on an on-screen keyboard.</p>
/// <p><b>Note: </b>Not all of these values are supported on all platforms. For unsupported values the default key will be used instead.</p>
/// <p>This enum was introduced or modified in  Qt 5.6.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct EnterKeyType(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for EnterKeyType {
    fn from(value: ::std::os::raw::c_int) -> Self {
        EnterKeyType(value)
    }
}

impl From<EnterKeyType> for ::std::os::raw::c_int {
    fn from(value: EnterKeyType) -> Self {
        value.0
    }
}

impl EnterKeyType {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl EnterKeyType {
    /// The default Enter key. This can either be a button closing the keyboard, or a Return button causing a new line in case of a multi-line input field. (C++ enum variant: <span style='color: green;'>```EnterKeyDefault = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const EnterKeyDefault: crate::EnterKeyType = crate::EnterKeyType(0);
    /// Show a Return button that inserts a new line. The keyboard will not close when this button is pressed. (C++ enum variant: <span style='color: green;'>```EnterKeyReturn = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const EnterKeyReturn: crate::EnterKeyType = crate::EnterKeyType(1);
    /// Show a "Done" button. The keyboard will close when this button is pressed. (C++ enum variant: <span style='color: green;'>```EnterKeyDone = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const EnterKeyDone: crate::EnterKeyType = crate::EnterKeyType(2);
    /// Show a "Go" button. Typically used in an address bar when entering a URL; the keyboard will close when this button is pressed. (C++ enum variant: <span style='color: green;'>```EnterKeyGo = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const EnterKeyGo: crate::EnterKeyType = crate::EnterKeyType(3);
    /// Show a "Send" button. The keyboard will close when this button is pressed. (C++ enum variant: <span style='color: green;'>```EnterKeySend = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const EnterKeySend: crate::EnterKeyType = crate::EnterKeyType(4);
    /// Show a "Search" button. The keyboard will close when this button is pressed. (C++ enum variant: <span style='color: green;'>```EnterKeySearch = 5```</span>)
    #[allow(non_upper_case_globals)]
    pub const EnterKeySearch: crate::EnterKeyType = crate::EnterKeyType(5);
    /// Show a "Next" button. Typically used in a form to allow navigating to the next input field; the keyboard will not close when this button is pressed. (C++ enum variant: <span style='color: green;'>```EnterKeyNext = 6```</span>)
    #[allow(non_upper_case_globals)]
    pub const EnterKeyNext: crate::EnterKeyType = crate::EnterKeyType(6);
    /// Show a "Previous" button. The keyboard will not close when this button is pressed. (C++ enum variant: <span style='color: green;'>```EnterKeyPrevious = 7```</span>)
    #[allow(non_upper_case_globals)]
    pub const EnterKeyPrevious: crate::EnterKeyType = crate::EnterKeyType(7);
}

/// <p>The style of the tool button, describing how the button's text and icon should be displayed.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::ToolButtonStyle```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#ToolButtonStyle-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The style of the tool button, describing how the button's text and icon should be displayed.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct ToolButtonStyle(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for ToolButtonStyle {
    fn from(value: ::std::os::raw::c_int) -> Self {
        ToolButtonStyle(value)
    }
}

impl From<ToolButtonStyle> for ::std::os::raw::c_int {
    fn from(value: ToolButtonStyle) -> Self {
        value.0
    }
}

impl ToolButtonStyle {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl ToolButtonStyle {
    /// Only display the icon. (C++ enum variant: <span style='color: green;'>```ToolButtonIconOnly = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const ToolButtonIconOnly: crate::ToolButtonStyle = crate::ToolButtonStyle(0);
    /// Only display the text. (C++ enum variant: <span style='color: green;'>```ToolButtonTextOnly = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const ToolButtonTextOnly: crate::ToolButtonStyle = crate::ToolButtonStyle(1);
    /// The text appears beside the icon. (C++ enum variant: <span style='color: green;'>```ToolButtonTextBesideIcon = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const ToolButtonTextBesideIcon: crate::ToolButtonStyle = crate::ToolButtonStyle(2);
    /// The text appears under the icon. (C++ enum variant: <span style='color: green;'>```ToolButtonTextUnderIcon = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const ToolButtonTextUnderIcon: crate::ToolButtonStyle = crate::ToolButtonStyle(3);
    /// Follow the <a href="http://doc.qt.io/qt-5/qstyle.html#StyleHint-enum">style</a>. (C++ enum variant: <span style='color: green;'>```ToolButtonFollowStyle = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const ToolButtonFollowStyle: crate::ToolButtonStyle = crate::ToolButtonStyle(4);
}

/// <p>Specifies the direction of Qt's layouts and text handling.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::LayoutDirection```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#LayoutDirection-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Specifies the direction of Qt's layouts and text handling.</p>
///
/// <p>Right-to-left layouts are necessary for certain languages, notably Arabic and Hebrew.</p>
/// <p>LayoutDirectionAuto serves two purposes. When used in conjunction with widgets and layouts, it will imply to use the layout direction set on the parent widget or <a href="http://doc.qt.io/qt-5/qapplication.html">QApplication</a>. This has the same effect as <a href="http://doc.qt.io/qt-5/qwidget.html#layoutDirection-prop">QWidget::unsetLayoutDirection</a>().</p>
/// <p>When LayoutDirectionAuto is used in conjunction with text layouting, it will imply that the text directionality is determined from the content of the string to be layouted.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qguiapplication.html#layoutDirection-prop">QGuiApplication::setLayoutDirection</a>(), <a href="http://doc.qt.io/qt-5/qwidget.html#layoutDirection-prop">QWidget::setLayoutDirection</a>(), <a href="http://doc.qt.io/qt-5/qtextoption.html#setTextDirection">QTextOption::setTextDirection</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#isRightToLeft">QString::isRightToLeft</a>().</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct LayoutDirection(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for LayoutDirection {
    fn from(value: ::std::os::raw::c_int) -> Self {
        LayoutDirection(value)
    }
}

impl From<LayoutDirection> for ::std::os::raw::c_int {
    fn from(value: LayoutDirection) -> Self {
        value.0
    }
}

impl LayoutDirection {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl LayoutDirection {
    /// Left-to-right layout. (C++ enum variant: <span style='color: green;'>```LeftToRight = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const LeftToRight: crate::LayoutDirection = crate::LayoutDirection(0);
    /// Right-to-left layout. (C++ enum variant: <span style='color: green;'>```RightToLeft = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const RightToLeft: crate::LayoutDirection = crate::LayoutDirection(1);
    /// Automatic layout. (C++ enum variant: <span style='color: green;'>```LayoutDirectionAuto = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const LayoutDirectionAuto: crate::LayoutDirection = crate::LayoutDirection(2);
}

/// <p>Specifies a side of a layout item that can be anchored. This is used by <a href="http://doc.qt.io/qt-5/qgraphicsanchorlayout.html">QGraphicsAnchorLayout</a>.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::AnchorPoint```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#AnchorPoint-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Specifies a side of a layout item that can be anchored. This is used by <a href="http://doc.qt.io/qt-5/qgraphicsanchorlayout.html">QGraphicsAnchorLayout</a>.</p>
///
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qgraphicsanchorlayout.html">QGraphicsAnchorLayout</a>.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct AnchorPoint(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for AnchorPoint {
    fn from(value: ::std::os::raw::c_int) -> Self {
        AnchorPoint(value)
    }
}

impl From<AnchorPoint> for ::std::os::raw::c_int {
    fn from(value: AnchorPoint) -> Self {
        value.0
    }
}

impl AnchorPoint {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl AnchorPoint {
    /// The left side of a layout item. (C++ enum variant: <span style='color: green;'>```AnchorLeft = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const AnchorLeft: crate::AnchorPoint = crate::AnchorPoint(0);
    /// A "virtual" side that is centered between the left and the right side of a layout item. (C++ enum variant: <span style='color: green;'>```AnchorHorizontalCenter = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const AnchorHorizontalCenter: crate::AnchorPoint = crate::AnchorPoint(1);
    /// The right side of a layout item. (C++ enum variant: <span style='color: green;'>```AnchorRight = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const AnchorRight: crate::AnchorPoint = crate::AnchorPoint(2);
    /// The top side of a layout item. (C++ enum variant: <span style='color: green;'>```AnchorTop = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const AnchorTop: crate::AnchorPoint = crate::AnchorPoint(3);
    /// A "virtual" side that is centered between the top and the bottom side of a layout item. (C++ enum variant: <span style='color: green;'>```AnchorVerticalCenter = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const AnchorVerticalCenter: crate::AnchorPoint = crate::AnchorPoint(4);
    /// The bottom side of a layout item. (C++ enum variant: <span style='color: green;'>```AnchorBottom = 5```</span>)
    #[allow(non_upper_case_globals)]
    pub const AnchorBottom: crate::AnchorPoint = crate::AnchorPoint(5);
}

/// <p>The FindChildOptions type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;FindChildOption&gt;. It stores an OR combination of FindChildOption values.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::FindChildOption```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#FindChildOption-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The FindChildOptions type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;FindChildOption&gt;. It stores an OR combination of FindChildOption values.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct FindChildOption(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for FindChildOption {
    fn from(value: ::std::os::raw::c_int) -> Self {
        FindChildOption(value)
    }
}

impl From<FindChildOption> for ::std::os::raw::c_int {
    fn from(value: FindChildOption) -> Self {
        value.0
    }
}

impl FindChildOption {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl FindChildOption {
    /// Looks only at the direct children of the object. (C++ enum variant: <span style='color: green;'>```FindDirectChildrenOnly = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const FindDirectChildrenOnly: crate::FindChildOption = crate::FindChildOption(0);
    /// Looks at all children of the object (recursive search). (C++ enum variant: <span style='color: green;'>```FindChildrenRecursively = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const FindChildrenRecursively: crate::FindChildOption = crate::FindChildOption(1);
}

/// <p>The DropActions type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;DropAction&gt;. It stores an OR combination of DropAction values.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::DropAction```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#DropAction-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The DropActions type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;DropAction&gt;. It stores an OR combination of DropAction values.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct DropAction(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for DropAction {
    fn from(value: ::std::os::raw::c_int) -> Self {
        DropAction(value)
    }
}

impl From<DropAction> for ::std::os::raw::c_int {
    fn from(value: DropAction) -> Self {
        value.0
    }
}

impl DropAction {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl DropAction {
    /// Copy the data to the target. (C++ enum variant: <span style='color: green;'>```CopyAction = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const CopyAction: crate::DropAction = crate::DropAction(1);
    /// Move the data from the source to the target. (C++ enum variant: <span style='color: green;'>```MoveAction = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const MoveAction: crate::DropAction = crate::DropAction(2);
    /// Create a link from the source to the target. (C++ enum variant: <span style='color: green;'>```LinkAction = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const LinkAction: crate::DropAction = crate::DropAction(4);
    /// &nbsp; (C++ enum variant: <span style='color: green;'>```ActionMask = 255```</span>)
    #[allow(non_upper_case_globals)]
    pub const ActionMask: crate::DropAction = crate::DropAction(255);
    /// On Windows, this value is used when the ownership of the D&amp;D data should be taken over by the target application, i.e., the source application should not delete the data. On X11 this value is used to do a move. TargetMoveAction is not used on the Mac. (C++ enum variant: <span style='color: green;'>```TargetMoveAction = 32770```</span>)
    #[allow(non_upper_case_globals)]
    pub const TargetMoveAction: crate::DropAction = crate::DropAction(32770);
    /// Ignore the action (do nothing with the data). (C++ enum variant: <span style='color: green;'>```IgnoreAction = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const IgnoreAction: crate::DropAction = crate::DropAction(0);
}

/// <p>This enum describes the state of checkable items, controls, and widgets.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::CheckState```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#CheckState-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the state of checkable items, controls, and widgets.</p>
///
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcheckbox.html">QCheckBox</a>, <a href="http://doc.qt.io/qt-5/qt.html#ItemFlag-enum">Qt::ItemFlags</a>, and <a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::ItemDataRole</a>.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct CheckState(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for CheckState {
    fn from(value: ::std::os::raw::c_int) -> Self {
        CheckState(value)
    }
}

impl From<CheckState> for ::std::os::raw::c_int {
    fn from(value: CheckState) -> Self {
        value.0
    }
}

impl CheckState {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl CheckState {
    /// The item is unchecked. (C++ enum variant: <span style='color: green;'>```Unchecked = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const Unchecked: crate::CheckState = crate::CheckState(0);
    /// The item is partially checked. Items in hierarchical models may be partially checked if some, but not all, of their children are checked. (C++ enum variant: <span style='color: green;'>```PartiallyChecked = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const PartiallyChecked: crate::CheckState = crate::CheckState(1);
    /// The item is checked. (C++ enum variant: <span style='color: green;'>```Checked = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const Checked: crate::CheckState = crate::CheckState(2);
}

/// <p>Each item in the model has a set of data elements associated with it, each with its own role. The roles are used by the view to indicate to the model which type of data it needs. Custom models should return data in these types.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::ItemDataRole```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Each item in the model has a set of data elements associated with it, each with its own role. The roles are used by the view to indicate to the model which type of data it needs. Custom models should return data in these types.</p>
/// <p>The general purpose roles (and the associated types) are:</p>
///
/// <p>Roles describing appearance and meta data (with associated types):</p>
///
/// <p>Accessibility roles (with associated types):</p>
///
/// <p>User roles:</p>
///
/// <p>For user roles, it is up to the developer to decide which types to use and ensure that components use the correct types when accessing and setting data.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct ItemDataRole(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for ItemDataRole {
    fn from(value: ::std::os::raw::c_int) -> Self {
        ItemDataRole(value)
    }
}

impl From<ItemDataRole> for ::std::os::raw::c_int {
    fn from(value: ItemDataRole) -> Self {
        value.0
    }
}

impl ItemDataRole {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl ItemDataRole {
    /// The key data to be rendered in the form of text. (<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>) (C++ enum variant: <span style='color: green;'>```DisplayRole = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const DisplayRole: crate::ItemDataRole = crate::ItemDataRole(0);
    /// The data to be rendered as a decoration in the form of an icon. (<a href="http://doc.qt.io/qt-5/qcolor.html">QColor</a>, <a href="http://doc.qt.io/qt-5/qicon.html">QIcon</a> or <a href="http://doc.qt.io/qt-5/qpixmap.html">QPixmap</a>) (C++ enum variant: <span style='color: green;'>```DecorationRole = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const DecorationRole: crate::ItemDataRole = crate::ItemDataRole(1);
    /// The data in a form suitable for editing in an editor. (<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>) (C++ enum variant: <span style='color: green;'>```EditRole = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const EditRole: crate::ItemDataRole = crate::ItemDataRole(2);
    /// The data displayed in the item's tooltip. (<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>) (C++ enum variant: <span style='color: green;'>```ToolTipRole = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const ToolTipRole: crate::ItemDataRole = crate::ItemDataRole(3);
    /// The data displayed in the status bar. (<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>) (C++ enum variant: <span style='color: green;'>```StatusTipRole = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const StatusTipRole: crate::ItemDataRole = crate::ItemDataRole(4);
    /// The data displayed for the item in "What's This?" mode. (<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>) (C++ enum variant: <span style='color: green;'>```WhatsThisRole = 5```</span>)
    #[allow(non_upper_case_globals)]
    pub const WhatsThisRole: crate::ItemDataRole = crate::ItemDataRole(5);
    /// The font used for items rendered with the default delegate. (<a href="http://doc.qt.io/qt-5/qfont.html">QFont</a>) (C++ enum variant: <span style='color: green;'>```FontRole = 6```</span>)
    #[allow(non_upper_case_globals)]
    pub const FontRole: crate::ItemDataRole = crate::ItemDataRole(6);
    /// The alignment of the text for items rendered with the default delegate. (<a href="http://doc.qt.io/qt-5/qt.html#AlignmentFlag-enum">Qt::Alignment</a>) (C++ enum variant: <span style='color: green;'>```TextAlignmentRole = 7```</span>)
    #[allow(non_upper_case_globals)]
    pub const TextAlignmentRole: crate::ItemDataRole = crate::ItemDataRole(7);
    /// This role is obsolete. Use BackgroundRole instead. (C++ enum variant: <span style='color: green;'>```BackgroundColorRole = 8```</span>)
    #[allow(non_upper_case_globals)]
    pub const BackgroundColorRole: crate::ItemDataRole = crate::ItemDataRole(8);
    /// The background brush used for items rendered with the default delegate. (<a href="http://doc.qt.io/qt-5/qbrush.html">QBrush</a>) (C++ enum variant: <span style='color: green;'>```BackgroundRole = 8```</span>)
    #[allow(non_upper_case_globals)]
    pub const BackgroundRole: crate::ItemDataRole = crate::ItemDataRole(8);
    /// This role is obsolete. Use ForegroundRole instead. (C++ enum variant: <span style='color: green;'>```TextColorRole = 9```</span>)
    #[allow(non_upper_case_globals)]
    pub const TextColorRole: crate::ItemDataRole = crate::ItemDataRole(9);
    /// The foreground brush (text color, typically) used for items rendered with the default delegate. (<a href="http://doc.qt.io/qt-5/qbrush.html">QBrush</a>) (C++ enum variant: <span style='color: green;'>```ForegroundRole = 9```</span>)
    #[allow(non_upper_case_globals)]
    pub const ForegroundRole: crate::ItemDataRole = crate::ItemDataRole(9);
    /// This role is used to obtain the checked state of an item. (<a href="http://doc.qt.io/qt-5/qt.html#CheckState-enum">Qt::CheckState</a>) (C++ enum variant: <span style='color: green;'>```CheckStateRole = 10```</span>)
    #[allow(non_upper_case_globals)]
    pub const CheckStateRole: crate::ItemDataRole = crate::ItemDataRole(10);
    /// The text to be used by accessibility extensions and plugins, such as screen readers. (<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>) (C++ enum variant: <span style='color: green;'>```AccessibleTextRole = 11```</span>)
    #[allow(non_upper_case_globals)]
    pub const AccessibleTextRole: crate::ItemDataRole = crate::ItemDataRole(11);
    /// A description of the item for accessibility purposes. (<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>) (C++ enum variant: <span style='color: green;'>```AccessibleDescriptionRole = 12```</span>)
    #[allow(non_upper_case_globals)]
    pub const AccessibleDescriptionRole: crate::ItemDataRole = crate::ItemDataRole(12);
    /// The size hint for the item that will be supplied to views. (<a href="http://doc.qt.io/qt-5/qsize.html">QSize</a>) (C++ enum variant: <span style='color: green;'>```SizeHintRole = 13```</span>)
    #[allow(non_upper_case_globals)]
    pub const SizeHintRole: crate::ItemDataRole = crate::ItemDataRole(13);
    /// This role is used to obtain the initial sort order of a header view section. (<a href="http://doc.qt.io/qt-5/qt.html#SortOrder-enum">Qt::SortOrder</a>). This role was introduced in Qt 4.8. (C++ enum variant: <span style='color: green;'>```InitialSortOrderRole = 14```</span>)
    #[allow(non_upper_case_globals)]
    pub const InitialSortOrderRole: crate::ItemDataRole = crate::ItemDataRole(14);
    /// C++ enum variant: <span style='color: green;'>```DisplayPropertyRole = 27```</span>
    #[allow(non_upper_case_globals)]
    pub const DisplayPropertyRole: crate::ItemDataRole = crate::ItemDataRole(27);
    /// C++ enum variant: <span style='color: green;'>```DecorationPropertyRole = 28```</span>
    #[allow(non_upper_case_globals)]
    pub const DecorationPropertyRole: crate::ItemDataRole = crate::ItemDataRole(28);
    /// C++ enum variant: <span style='color: green;'>```ToolTipPropertyRole = 29```</span>
    #[allow(non_upper_case_globals)]
    pub const ToolTipPropertyRole: crate::ItemDataRole = crate::ItemDataRole(29);
    /// C++ enum variant: <span style='color: green;'>```StatusTipPropertyRole = 30```</span>
    #[allow(non_upper_case_globals)]
    pub const StatusTipPropertyRole: crate::ItemDataRole = crate::ItemDataRole(30);
    /// C++ enum variant: <span style='color: green;'>```WhatsThisPropertyRole = 31```</span>
    #[allow(non_upper_case_globals)]
    pub const WhatsThisPropertyRole: crate::ItemDataRole = crate::ItemDataRole(31);
    /// The first role that can be used for application-specific purposes. (C++ enum variant: <span style='color: green;'>```UserRole = 256```</span>)
    #[allow(non_upper_case_globals)]
    pub const UserRole: crate::ItemDataRole = crate::ItemDataRole(256);
}

/// <p>This enum describes the properties of an item:</p>
///
/// C++ enum: <span style='color: green;'>```Qt::ItemFlag```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#ItemFlag-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the properties of an item:</p>
///
/// <p>Note that checkable items need to be given both a suitable set of flags and an initial state, indicating whether the item is checked or not. This is handled automatically for model/view components, but needs to be explicitly set for instances of <a href="http://doc.qt.io/qt-5/qlistwidgetitem.html">QListWidgetItem</a>, <a href="http://doc.qt.io/qt-5/qtablewidgetitem.html">QTableWidgetItem</a>, and <a href="http://doc.qt.io/qt-5/qtreewidgetitem.html">QTreeWidgetItem</a>.</p>
/// <p>Note that it is undefined behavior to reimplement <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#hasChildren">QAbstractItemModel::hasChildren</a> to return true for an index if that index has the Qt::ItemNeverHasChildren flag set.</p>
/// <p>The ItemFlags type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;ItemFlag&gt;. It stores an OR combination of ItemFlag values.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a>.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct ItemFlag(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for ItemFlag {
    fn from(value: ::std::os::raw::c_int) -> Self {
        ItemFlag(value)
    }
}

impl From<ItemFlag> for ::std::os::raw::c_int {
    fn from(value: ItemFlag) -> Self {
        value.0
    }
}

impl ItemFlag {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl ItemFlag {
    /// It does not have any properties set. (C++ enum variant: <span style='color: green;'>```NoItemFlags = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const NoItemFlags: crate::ItemFlag = crate::ItemFlag(0);
    /// It can be selected. (C++ enum variant: <span style='color: green;'>```ItemIsSelectable = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const ItemIsSelectable: crate::ItemFlag = crate::ItemFlag(1);
    /// It can be edited. (C++ enum variant: <span style='color: green;'>```ItemIsEditable = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const ItemIsEditable: crate::ItemFlag = crate::ItemFlag(2);
    /// It can be dragged. (C++ enum variant: <span style='color: green;'>```ItemIsDragEnabled = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const ItemIsDragEnabled: crate::ItemFlag = crate::ItemFlag(4);
    /// It can be used as a drop target. (C++ enum variant: <span style='color: green;'>```ItemIsDropEnabled = 8```</span>)
    #[allow(non_upper_case_globals)]
    pub const ItemIsDropEnabled: crate::ItemFlag = crate::ItemFlag(8);
    /// It can be checked or unchecked by the user. (C++ enum variant: <span style='color: green;'>```ItemIsUserCheckable = 16```</span>)
    #[allow(non_upper_case_globals)]
    pub const ItemIsUserCheckable: crate::ItemFlag = crate::ItemFlag(16);
    /// The user can interact with the item. (C++ enum variant: <span style='color: green;'>```ItemIsEnabled = 32```</span>)
    #[allow(non_upper_case_globals)]
    pub const ItemIsEnabled: crate::ItemFlag = crate::ItemFlag(32);
    /// The item's state depends on the state of its children. This enables automatic management of the state of parent items in <a href="http://doc.qt.io/qt-5/qtreewidget.html">QTreeWidget</a> (checked if all children are checked, unchecked if all children are unchecked, or partially checked if only some children are checked). (C++ enum variant: <span style='color: green;'>```ItemIsAutoTristate = 64```</span>)
    #[allow(non_upper_case_globals)]
    pub const ItemIsAutoTristate: crate::ItemFlag = crate::ItemFlag(64);
    /// <i>This enum value is deprecated.</i> Use Qt::ItemIsAutoTristate instead. (C++ enum variant: <span style='color: green;'>```ItemIsTristate = 64```</span>)
    #[allow(non_upper_case_globals)]
    pub const ItemIsTristate: crate::ItemFlag = crate::ItemFlag(64);
    /// The item never has child items. This is used for optimization purposes only. (C++ enum variant: <span style='color: green;'>```ItemNeverHasChildren = 128```</span>)
    #[allow(non_upper_case_globals)]
    pub const ItemNeverHasChildren: crate::ItemFlag = crate::ItemFlag(128);
    /// The user can cycle through three separate states. This value has been added in Qt 5.5. (C++ enum variant: <span style='color: green;'>```ItemIsUserTristate = 256```</span>)
    #[allow(non_upper_case_globals)]
    pub const ItemIsUserTristate: crate::ItemFlag = crate::ItemFlag(256);
}

/// <p>This enum describes the type of matches that can be used when searching for items in a model.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::MatchFlag```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#MatchFlag-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the type of matches that can be used when searching for items in a model.</p>
///
/// <p>The MatchFlags type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;MatchFlag&gt;. It stores an OR combination of MatchFlag values.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#compare">QString::compare</a>() and <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a>.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct MatchFlag(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for MatchFlag {
    fn from(value: ::std::os::raw::c_int) -> Self {
        MatchFlag(value)
    }
}

impl From<MatchFlag> for ::std::os::raw::c_int {
    fn from(value: MatchFlag) -> Self {
        value.0
    }
}

impl MatchFlag {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl MatchFlag {
    /// Performs <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>-based matching. (C++ enum variant: <span style='color: green;'>```MatchExactly = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const MatchExactly: crate::MatchFlag = crate::MatchFlag(0);
    /// The search term is contained in the item. (C++ enum variant: <span style='color: green;'>```MatchContains = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const MatchContains: crate::MatchFlag = crate::MatchFlag(1);
    /// The search term matches the start of the item. (C++ enum variant: <span style='color: green;'>```MatchStartsWith = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const MatchStartsWith: crate::MatchFlag = crate::MatchFlag(2);
    /// The search term matches the end of the item. (C++ enum variant: <span style='color: green;'>```MatchEndsWith = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const MatchEndsWith: crate::MatchFlag = crate::MatchFlag(3);
    /// Performs string-based matching using a regular expression as the search term. (C++ enum variant: <span style='color: green;'>```MatchRegExp = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const MatchRegExp: crate::MatchFlag = crate::MatchFlag(4);
    /// Performs string-based matching using a string with wildcards as the search term. (C++ enum variant: <span style='color: green;'>```MatchWildcard = 5```</span>)
    #[allow(non_upper_case_globals)]
    pub const MatchWildcard: crate::MatchFlag = crate::MatchFlag(5);
    /// Performs string-based matching. String-based comparisons are case-insensitive unless the <code>MatchCaseSensitive</code> flag is also specified. (C++ enum variant: <span style='color: green;'>```MatchFixedString = 8```</span>)
    #[allow(non_upper_case_globals)]
    pub const MatchFixedString: crate::MatchFlag = crate::MatchFlag(8);
    /// The search is case sensitive. (C++ enum variant: <span style='color: green;'>```MatchCaseSensitive = 16```</span>)
    #[allow(non_upper_case_globals)]
    pub const MatchCaseSensitive: crate::MatchFlag = crate::MatchFlag(16);
    /// Perform a search that wraps around, so that when the search reaches the last item in the model, it begins again at the first item and continues until all items have been examined. (C++ enum variant: <span style='color: green;'>```MatchWrap = 32```</span>)
    #[allow(non_upper_case_globals)]
    pub const MatchWrap: crate::MatchFlag = crate::MatchFlag(32);
    /// Searches the entire hierarchy. (C++ enum variant: <span style='color: green;'>```MatchRecursive = 64```</span>)
    #[allow(non_upper_case_globals)]
    pub const MatchRecursive: crate::MatchFlag = crate::MatchFlag(64);
}

/// <p>This enum specifies the behavior of a modal window. A modal window is one that blocks input to other windows. Note that windows that are children of a modal window are not blocked.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::WindowModality```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#WindowModality-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum specifies the behavior of a modal window. A modal window is one that blocks input to other windows. Note that windows that are children of a modal window are not blocked.</p>
/// <p>The values are:</p>
///
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qwidget.html#windowModality-prop">QWidget::windowModality</a> and <a href="http://doc.qt.io/qt-5/qdialog.html">QDialog</a>.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct WindowModality(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for WindowModality {
    fn from(value: ::std::os::raw::c_int) -> Self {
        WindowModality(value)
    }
}

impl From<WindowModality> for ::std::os::raw::c_int {
    fn from(value: WindowModality) -> Self {
        value.0
    }
}

impl WindowModality {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl WindowModality {
    /// The window is not modal and does not block input to other windows. (C++ enum variant: <span style='color: green;'>```NonModal = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const NonModal: crate::WindowModality = crate::WindowModality(0);
    /// The window is modal to a single window hierarchy and blocks input to its parent window, all grandparent windows, and all siblings of its parent and grandparent windows. (C++ enum variant: <span style='color: green;'>```WindowModal = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const WindowModal: crate::WindowModality = crate::WindowModality(1);
    /// The window is modal to the application and blocks input to all windows. (C++ enum variant: <span style='color: green;'>```ApplicationModal = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const ApplicationModal: crate::WindowModality = crate::WindowModality(2);
}

/// <p>This enum specifies how a text displaying widget reacts to user input.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::TextInteractionFlag```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#TextInteractionFlag-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum specifies how a text displaying widget reacts to user input.</p>
///
/// <p>The TextInteractionFlags type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;TextInteractionFlag&gt;. It stores an OR combination of TextInteractionFlag values.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct TextInteractionFlag(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for TextInteractionFlag {
    fn from(value: ::std::os::raw::c_int) -> Self {
        TextInteractionFlag(value)
    }
}

impl From<TextInteractionFlag> for ::std::os::raw::c_int {
    fn from(value: TextInteractionFlag) -> Self {
        value.0
    }
}

impl TextInteractionFlag {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl TextInteractionFlag {
    /// No interaction with the text is possible. (C++ enum variant: <span style='color: green;'>```NoTextInteraction = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const NoTextInteraction: crate::TextInteractionFlag = crate::TextInteractionFlag(0);
    /// Text can be selected with the mouse and copied to the clipboard using a context menu or standard keyboard shortcuts. (C++ enum variant: <span style='color: green;'>```TextSelectableByMouse = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const TextSelectableByMouse: crate::TextInteractionFlag = crate::TextInteractionFlag(1);
    /// Text can be selected with the cursor keys on the keyboard. A text cursor is shown. (C++ enum variant: <span style='color: green;'>```TextSelectableByKeyboard = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const TextSelectableByKeyboard: crate::TextInteractionFlag = crate::TextInteractionFlag(2);
    /// Links can be highlighted and activated with the mouse. (C++ enum variant: <span style='color: green;'>```LinksAccessibleByMouse = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const LinksAccessibleByMouse: crate::TextInteractionFlag = crate::TextInteractionFlag(4);
    /// Links can be focused using tab and activated with enter. (C++ enum variant: <span style='color: green;'>```LinksAccessibleByKeyboard = 8```</span>)
    #[allow(non_upper_case_globals)]
    pub const LinksAccessibleByKeyboard: crate::TextInteractionFlag = crate::TextInteractionFlag(8);
    /// The text is fully editable. (C++ enum variant: <span style='color: green;'>```TextEditable = 16```</span>)
    #[allow(non_upper_case_globals)]
    pub const TextEditable: crate::TextInteractionFlag = crate::TextInteractionFlag(16);
    /// The default for a text editor. (C++ enum variant: <span style='color: green;'>```TextEditorInteraction = 19```</span>)
    #[allow(non_upper_case_globals)]
    pub const TextEditorInteraction: crate::TextInteractionFlag = crate::TextInteractionFlag(19);
    /// The default for <a href="http://doc.qt.io/qt-5/qtextbrowser.html">QTextBrowser</a>. (C++ enum variant: <span style='color: green;'>```TextBrowserInteraction = 13```</span>)
    #[allow(non_upper_case_globals)]
    pub const TextBrowserInteraction: crate::TextInteractionFlag = crate::TextInteractionFlag(13);
}

/// <p>This enum can be used to specify event priorities.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::EventPriority```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#EventPriority-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum can be used to specify event priorities.</p>
///
/// <p>Note that these values are provided purely for convenience, since event priorities can be any value between <code>INT_MAX</code> and <code>INT_MIN</code>, inclusive. For example, you can define custom priorities as being relative to each other:</p>
/// <pre class="cpp">
///
///   <span class="keyword">enum</span> CustomEventPriority
///   {
/// &#32;     <span class="comment">// An important event</span>
/// &#32;     ImportantEventPriority <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>HighEventPriority<span class="operator">,</span>
///
/// &#32;     <span class="comment">// A more important event</span>
/// &#32;     MoreImportantEventPriority <span class="operator">=</span> ImportantEventPriority <span class="operator">+</span> <span class="number">1</span><span class="operator">,</span>
///
/// &#32;     <span class="comment">// A critical event</span>
/// &#32;     CriticalEventPriority <span class="operator">=</span> <span class="number">100</span> <span class="operator">*</span> MoreImportantEventPriority<span class="operator">,</span>
///
/// &#32;     <span class="comment">// Not that important</span>
/// &#32;     StatusEventPriority <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>LowEventPriority<span class="operator">,</span>
///
/// &#32;     <span class="comment">// These are less important than Status events</span>
/// &#32;     IdleProcessingDoneEventPriority <span class="operator">=</span> StatusEventPriority <span class="operator">-</span> <span class="number">1</span>
///   };
///
/// </pre>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#postEvent">QCoreApplication::postEvent</a>().</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct EventPriority(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for EventPriority {
    fn from(value: ::std::os::raw::c_int) -> Self {
        EventPriority(value)
    }
}

impl From<EventPriority> for ::std::os::raw::c_int {
    fn from(value: EventPriority) -> Self {
        value.0
    }
}

impl EventPriority {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl EventPriority {
    /// Events with this priority are sent before events with NormalEventPriority or LowEventPriority. (C++ enum variant: <span style='color: green;'>```HighEventPriority = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const HighEventPriority: crate::EventPriority = crate::EventPriority(1);
    /// Events with this priority are sent after events with HighEventPriority, but before events with LowEventPriority. (C++ enum variant: <span style='color: green;'>```NormalEventPriority = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const NormalEventPriority: crate::EventPriority = crate::EventPriority(0);
    /// Events with this priority are sent after events with HighEventPriority or NormalEventPriority. (C++ enum variant: <span style='color: green;'>```LowEventPriority = -1```</span>)
    #[allow(non_upper_case_globals)]
    pub const LowEventPriority: crate::EventPriority = crate::EventPriority(-1);
}

/// <p>This enum is used by <a href="http://doc.qt.io/qt-5/qgraphicslayoutitem.html#sizeHint">QGraphicsLayoutItem::sizeHint</a>()</p>
///
/// C++ enum: <span style='color: green;'>```Qt::SizeHint```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#SizeHint-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum is used by <a href="http://doc.qt.io/qt-5/qgraphicslayoutitem.html#sizeHint">QGraphicsLayoutItem::sizeHint</a>()</p>
///
/// <p>This enum was introduced or modified in  Qt 4.4.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qgraphicslayoutitem.html#sizeHint">QGraphicsLayoutItem::sizeHint</a>().</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct SizeHint(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for SizeHint {
    fn from(value: ::std::os::raw::c_int) -> Self {
        SizeHint(value)
    }
}

impl From<SizeHint> for ::std::os::raw::c_int {
    fn from(value: SizeHint) -> Self {
        value.0
    }
}

impl SizeHint {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl SizeHint {
    /// is used to specify the minimum size of a graphics layout item. (C++ enum variant: <span style='color: green;'>```MinimumSize = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const MinimumSize: crate::SizeHint = crate::SizeHint(0);
    /// is used to specify the preferred size of a graphics layout item. (C++ enum variant: <span style='color: green;'>```PreferredSize = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const PreferredSize: crate::SizeHint = crate::SizeHint(1);
    /// is used to specify the maximum size of a graphics layout item. (C++ enum variant: <span style='color: green;'>```MaximumSize = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const MaximumSize: crate::SizeHint = crate::SizeHint(2);
    /// is used to specify the minimum descent of a text string in a graphics layout item. (C++ enum variant: <span style='color: green;'>```MinimumDescent = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const MinimumDescent: crate::SizeHint = crate::SizeHint(3);
    /// C++ enum variant: <span style='color: green;'>```NSizeHints = 4```</span>
    #[allow(non_upper_case_globals)]
    pub const NSizeHints: crate::SizeHint = crate::SizeHint(4);
}

/// <p>This enum is used to describe parts of a window frame. It is returned by <a href="http://doc.qt.io/qt-5/qgraphicswidget.html#windowFrameSectionAt">QGraphicsWidget::windowFrameSectionAt</a>() to describe what section of the window frame is under the mouse.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::WindowFrameSection```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#WindowFrameSection-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum is used to describe parts of a window frame. It is returned by <a href="http://doc.qt.io/qt-5/qgraphicswidget.html#windowFrameSectionAt">QGraphicsWidget::windowFrameSectionAt</a>() to describe what section of the window frame is under the mouse.</p>
///
/// <p>This enum was introduced or modified in  Qt 4.4.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qgraphicswidget.html#windowFrameEvent">QGraphicsWidget::windowFrameEvent</a>(), <a href="http://doc.qt.io/qt-5/qgraphicswidget.html#paintWindowFrame">QGraphicsWidget::paintWindowFrame</a>(), and <a href="http://doc.qt.io/qt-5/qgraphicswidget.html#windowFrameSectionAt">QGraphicsWidget::windowFrameSectionAt</a>().</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct WindowFrameSection(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for WindowFrameSection {
    fn from(value: ::std::os::raw::c_int) -> Self {
        WindowFrameSection(value)
    }
}

impl From<WindowFrameSection> for ::std::os::raw::c_int {
    fn from(value: WindowFrameSection) -> Self {
        value.0
    }
}

impl WindowFrameSection {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl WindowFrameSection {
    /// C++ enum variant: <span style='color: green;'>```NoSection = 0```</span>
    #[allow(non_upper_case_globals)]
    pub const NoSection: crate::WindowFrameSection = crate::WindowFrameSection(0);
    /// C++ enum variant: <span style='color: green;'>```LeftSection = 1```</span>
    #[allow(non_upper_case_globals)]
    pub const LeftSection: crate::WindowFrameSection = crate::WindowFrameSection(1);
    /// C++ enum variant: <span style='color: green;'>```TopLeftSection = 2```</span>
    #[allow(non_upper_case_globals)]
    pub const TopLeftSection: crate::WindowFrameSection = crate::WindowFrameSection(2);
    /// C++ enum variant: <span style='color: green;'>```TopSection = 3```</span>
    #[allow(non_upper_case_globals)]
    pub const TopSection: crate::WindowFrameSection = crate::WindowFrameSection(3);
    /// C++ enum variant: <span style='color: green;'>```TopRightSection = 4```</span>
    #[allow(non_upper_case_globals)]
    pub const TopRightSection: crate::WindowFrameSection = crate::WindowFrameSection(4);
    /// C++ enum variant: <span style='color: green;'>```RightSection = 5```</span>
    #[allow(non_upper_case_globals)]
    pub const RightSection: crate::WindowFrameSection = crate::WindowFrameSection(5);
    /// C++ enum variant: <span style='color: green;'>```BottomRightSection = 6```</span>
    #[allow(non_upper_case_globals)]
    pub const BottomRightSection: crate::WindowFrameSection = crate::WindowFrameSection(6);
    /// C++ enum variant: <span style='color: green;'>```BottomSection = 7```</span>
    #[allow(non_upper_case_globals)]
    pub const BottomSection: crate::WindowFrameSection = crate::WindowFrameSection(7);
    /// C++ enum variant: <span style='color: green;'>```BottomLeftSection = 8```</span>
    #[allow(non_upper_case_globals)]
    pub const BottomLeftSection: crate::WindowFrameSection = crate::WindowFrameSection(8);
    /// C++ enum variant: <span style='color: green;'>```TitleBarArea = 9```</span>
    #[allow(non_upper_case_globals)]
    pub const TitleBarArea: crate::WindowFrameSection = crate::WindowFrameSection(9);
}

/// <p>This enum specifies the coordinate system.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::CoordinateSystem```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#CoordinateSystem-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum specifies the coordinate system.</p>
///
/// <p>This enum was introduced or modified in  Qt 4.6.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct CoordinateSystem(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for CoordinateSystem {
    fn from(value: ::std::os::raw::c_int) -> Self {
        CoordinateSystem(value)
    }
}

impl From<CoordinateSystem> for ::std::os::raw::c_int {
    fn from(value: CoordinateSystem) -> Self {
        value.0
    }
}

impl CoordinateSystem {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl CoordinateSystem {
    /// Coordinates are relative to the upper-left corner of the object's paint device. (C++ enum variant: <span style='color: green;'>```DeviceCoordinates = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const DeviceCoordinates: crate::CoordinateSystem = crate::CoordinateSystem(0);
    /// Coordinates are relative to the upper-left corner of the object. (C++ enum variant: <span style='color: green;'>```LogicalCoordinates = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const LogicalCoordinates: crate::CoordinateSystem = crate::CoordinateSystem(1);
}

/// <p>This enum represents the state of a touch point at the time a <a href="http://doc.qt.io/qt-5/qtouchevent.html">QTouchEvent</a> occurred.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::TouchPointState```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#TouchPointState-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum represents the state of a touch point at the time a <a href="http://doc.qt.io/qt-5/qtouchevent.html">QTouchEvent</a> occurred.</p>
///
/// <p>This enum was introduced or modified in  Qt 4.6.</p>
/// <p>The TouchPointStates type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;TouchPointState&gt;. It stores an OR combination of TouchPointState values.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct TouchPointState(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for TouchPointState {
    fn from(value: ::std::os::raw::c_int) -> Self {
        TouchPointState(value)
    }
}

impl From<TouchPointState> for ::std::os::raw::c_int {
    fn from(value: TouchPointState) -> Self {
        value.0
    }
}

impl TouchPointState {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl TouchPointState {
    /// The touch point is now pressed. (C++ enum variant: <span style='color: green;'>```TouchPointPressed = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const TouchPointPressed: crate::TouchPointState = crate::TouchPointState(1);
    /// The touch point moved. (C++ enum variant: <span style='color: green;'>```TouchPointMoved = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const TouchPointMoved: crate::TouchPointState = crate::TouchPointState(2);
    /// The touch point did not move. (C++ enum variant: <span style='color: green;'>```TouchPointStationary = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const TouchPointStationary: crate::TouchPointState = crate::TouchPointState(4);
    /// The touch point was released. (C++ enum variant: <span style='color: green;'>```TouchPointReleased = 8```</span>)
    #[allow(non_upper_case_globals)]
    pub const TouchPointReleased: crate::TouchPointState = crate::TouchPointState(8);
}

/// <p>This enum type describes the state of a gesture.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::GestureState```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#GestureState-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type describes the state of a gesture.</p>
///
/// <p>This enum was introduced or modified in  Qt 4.6.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qgesture.html">QGesture</a>.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct GestureState(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for GestureState {
    fn from(value: ::std::os::raw::c_int) -> Self {
        GestureState(value)
    }
}

impl From<GestureState> for ::std::os::raw::c_int {
    fn from(value: GestureState) -> Self {
        value.0
    }
}

impl GestureState {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl GestureState {
    /// No gesture has been detected. (C++ enum variant: <span style='color: green;'>```NoGesture = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const NoGesture: crate::GestureState = crate::GestureState(0);
    /// A continuous gesture has started. (C++ enum variant: <span style='color: green;'>```GestureStarted = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const GestureStarted: crate::GestureState = crate::GestureState(1);
    /// A gesture continues. (C++ enum variant: <span style='color: green;'>```GestureUpdated = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const GestureUpdated: crate::GestureState = crate::GestureState(2);
    /// A gesture has finished. (C++ enum variant: <span style='color: green;'>```GestureFinished = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const GestureFinished: crate::GestureState = crate::GestureState(3);
    /// A gesture was canceled. (C++ enum variant: <span style='color: green;'>```GestureCanceled = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const GestureCanceled: crate::GestureState = crate::GestureState(4);
}

/// <p>This enum type describes the standard gestures.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::GestureType```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#GestureType-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type describes the standard gestures.</p>
///
/// <p>User-defined gestures are registered with the <a href="http://doc.qt.io/qt-5/qgesturerecognizer.html#registerRecognizer">QGestureRecognizer::registerRecognizer</a>() function which generates a custom gesture ID with the Qt::CustomGesture flag set.</p>
/// <p>This enum was introduced or modified in  Qt 4.6.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qgesture.html">QGesture</a>, <a href="http://doc.qt.io/qt-5/qwidget.html#grabGesture">QWidget::grabGesture</a>(), and <a href="http://doc.qt.io/qt-5/qgraphicsobject.html#grabGesture">QGraphicsObject::grabGesture</a>().</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct GestureType(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for GestureType {
    fn from(value: ::std::os::raw::c_int) -> Self {
        GestureType(value)
    }
}

impl From<GestureType> for ::std::os::raw::c_int {
    fn from(value: GestureType) -> Self {
        value.0
    }
}

impl GestureType {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl GestureType {
    /// A Tap gesture. (C++ enum variant: <span style='color: green;'>```TapGesture = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const TapGesture: crate::GestureType = crate::GestureType(1);
    /// A Tap-And-Hold (Long-Tap) gesture. (C++ enum variant: <span style='color: green;'>```TapAndHoldGesture = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const TapAndHoldGesture: crate::GestureType = crate::GestureType(2);
    /// A Pan gesture. (C++ enum variant: <span style='color: green;'>```PanGesture = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const PanGesture: crate::GestureType = crate::GestureType(3);
    /// A Pinch gesture. (C++ enum variant: <span style='color: green;'>```PinchGesture = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const PinchGesture: crate::GestureType = crate::GestureType(4);
    /// A Swipe gesture. (C++ enum variant: <span style='color: green;'>```SwipeGesture = 5```</span>)
    #[allow(non_upper_case_globals)]
    pub const SwipeGesture: crate::GestureType = crate::GestureType(5);
    /// A flag that can be used to test if the gesture is a user-defined gesture ID. (C++ enum variant: <span style='color: green;'>```CustomGesture = 256```</span>)
    #[allow(non_upper_case_globals)]
    pub const CustomGesture: crate::GestureType = crate::GestureType(256);
    /// C++ enum variant: <span style='color: green;'>```LastGestureType = -1```</span>
    #[allow(non_upper_case_globals)]
    pub const LastGestureType: crate::GestureType = crate::GestureType(-1);
}

/// <p>This enum type describes additional flags that can be used when subscribing to a gesture.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::GestureFlag```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#GestureFlag-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type describes additional flags that can be used when subscribing to a gesture.</p>
///
/// <p>This enum was introduced or modified in  Qt 4.6.</p>
/// <p>The GestureFlags type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;GestureFlag&gt;. It stores an OR combination of GestureFlag values.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qwidget.html#grabGesture">QWidget::grabGesture</a>() and <a href="http://doc.qt.io/qt-5/qgraphicsobject.html#grabGesture">QGraphicsObject::grabGesture</a>().</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct GestureFlag(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for GestureFlag {
    fn from(value: ::std::os::raw::c_int) -> Self {
        GestureFlag(value)
    }
}

impl From<GestureFlag> for ::std::os::raw::c_int {
    fn from(value: GestureFlag) -> Self {
        value.0
    }
}

impl GestureFlag {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl GestureFlag {
    /// By default gestures can start on the widget or over any of its children. Use this flag to disable this and allow a gesture to start on the widget only. (C++ enum variant: <span style='color: green;'>```DontStartGestureOnChildren = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const DontStartGestureOnChildren: crate::GestureFlag = crate::GestureFlag(1);
    /// Allows any ignored gesture events to be propagated to parent widgets which have specified this hint. By default only gestures that are in the <a href="http://doc.qt.io/qt-5/qt.html#GestureState-enum">Qt::GestureStarted</a> state are propagated and the widget always gets the full gesture sequence starting with a gesture in the <a href="http://doc.qt.io/qt-5/qt.html#GestureState-enum">Qt::GestureStarted</a> state and ending with a gesture in the <a href="http://doc.qt.io/qt-5/qt.html#GestureState-enum">Qt::GestureFinished</a> or <a href="http://doc.qt.io/qt-5/qt.html#GestureState-enum">Qt::GestureCanceled</a> states. (C++ enum variant: <span style='color: green;'>```ReceivePartialGestures = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const ReceivePartialGestures: crate::GestureFlag = crate::GestureFlag(2);
    /// Since Qt 4.7, this flag allows you to fine-tune gesture event propagation. By setting the flag when <a href="http://doc.qt.io/qt-5/qgraphicsobject.html#grabGesture">grabbing</a> a gesture all ignored partial gestures will propagate to their parent items. (C++ enum variant: <span style='color: green;'>```IgnoredGesturesPropagateToParent = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const IgnoredGesturesPropagateToParent: crate::GestureFlag = crate::GestureFlag(4);
}

/// <p>This enum returns the gesture type.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::NativeGestureType```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#NativeGestureType-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum returns the gesture type.</p>
///
/// <p>This enum was introduced or modified in  Qt 5.2.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct NativeGestureType(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for NativeGestureType {
    fn from(value: ::std::os::raw::c_int) -> Self {
        NativeGestureType(value)
    }
}

impl From<NativeGestureType> for ::std::os::raw::c_int {
    fn from(value: NativeGestureType) -> Self {
        value.0
    }
}

impl NativeGestureType {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl NativeGestureType {
    /// Sent before gesture event stream. (C++ enum variant: <span style='color: green;'>```BeginNativeGesture = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const BeginNativeGesture: crate::NativeGestureType = crate::NativeGestureType(0);
    /// Sent after gesture event stream. (C++ enum variant: <span style='color: green;'>```EndNativeGesture = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const EndNativeGesture: crate::NativeGestureType = crate::NativeGestureType(1);
    /// Sent after a panning gesture. Similar to a click-and-drag mouse movement. (C++ enum variant: <span style='color: green;'>```PanNativeGesture = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const PanNativeGesture: crate::NativeGestureType = crate::NativeGestureType(2);
    /// Specifies the magnification delta in percent. (C++ enum variant: <span style='color: green;'>```ZoomNativeGesture = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const ZoomNativeGesture: crate::NativeGestureType = crate::NativeGestureType(3);
    /// Boolean magnification state. (C++ enum variant: <span style='color: green;'>```SmartZoomNativeGesture = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const SmartZoomNativeGesture: crate::NativeGestureType = crate::NativeGestureType(4);
    /// Rotation delta in degrees. (C++ enum variant: <span style='color: green;'>```RotateNativeGesture = 5```</span>)
    #[allow(non_upper_case_globals)]
    pub const RotateNativeGesture: crate::NativeGestureType = crate::NativeGestureType(5);
    /// Sent after a swipe movements. (C++ enum variant: <span style='color: green;'>```SwipeNativeGesture = 6```</span>)
    #[allow(non_upper_case_globals)]
    pub const SwipeNativeGesture: crate::NativeGestureType = crate::NativeGestureType(6);
}

/// <p>This enum type describes the mode for moving focus.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::NavigationMode```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#NavigationMode-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type describes the mode for moving focus.</p>
///
/// <p><b>Note: </b>In 4.6, cursor navigation is only implemented for Symbian OS. On other platforms, it behaves as NavigationModeNone.</p><p>This enum was introduced or modified in  Qt 4.6.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qapplication.html#setNavigationMode">QApplication::setNavigationMode</a>() and <a href="http://doc.qt.io/qt-5/qapplication.html#navigationMode">QApplication::navigationMode</a>().</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct NavigationMode(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for NavigationMode {
    fn from(value: ::std::os::raw::c_int) -> Self {
        NavigationMode(value)
    }
}

impl From<NavigationMode> for ::std::os::raw::c_int {
    fn from(value: NavigationMode) -> Self {
        value.0
    }
}

impl NavigationMode {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl NavigationMode {
    /// Only the touch screen is used. (C++ enum variant: <span style='color: green;'>```NavigationModeNone = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const NavigationModeNone: crate::NavigationMode = crate::NavigationMode(0);
    /// <a href="http://doc.qt.io/qt-5/qt.html#Key-enum">Qt::Key_Up</a> and <a href="http://doc.qt.io/qt-5/qt.html#Key-enum">Qt::Key_Down</a> are used to change focus. (C++ enum variant: <span style='color: green;'>```NavigationModeKeypadTabOrder = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const NavigationModeKeypadTabOrder: crate::NavigationMode = crate::NavigationMode(1);
    /// <a href="http://doc.qt.io/qt-5/qt.html#Key-enum">Qt::Key_Up</a>, <a href="http://doc.qt.io/qt-5/qt.html#Key-enum">Qt::Key_Down</a>, <a href="http://doc.qt.io/qt-5/qt.html#Key-enum">Qt::Key_Left</a> and <a href="http://doc.qt.io/qt-5/qt.html#Key-enum">Qt::Key_Right</a> are used to change focus. (C++ enum variant: <span style='color: green;'>```NavigationModeKeypadDirectional = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const NavigationModeKeypadDirectional: crate::NavigationMode = crate::NavigationMode(2);
    /// The mouse cursor is used to change focus, it is displayed only on non touchscreen devices. The keypad is used to implement a virtual cursor, unless the device has an analog mouse type of input device (e.g. touchpad). This is the recommended setting for an application such as a web browser that needs pointer control on both touch and non-touch devices. (C++ enum variant: <span style='color: green;'>```NavigationModeCursorAuto = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const NavigationModeCursorAuto: crate::NavigationMode = crate::NavigationMode(3);
    /// The mouse cursor is used to change focus, it is displayed regardless of device type. The keypad is used to implement a virtual cursor, unless the device has an analog mouse type of input device (e.g. touchpad) (C++ enum variant: <span style='color: green;'>```NavigationModeCursorForceVisible = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const NavigationModeCursorForceVisible: crate::NavigationMode = crate::NavigationMode(4);
}

/// <p>This enum describes the movement style available to text cursors. The options are:</p>
///
/// C++ enum: <span style='color: green;'>```Qt::CursorMoveStyle```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#CursorMoveStyle-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the movement style available to text cursors. The options are:</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct CursorMoveStyle(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for CursorMoveStyle {
    fn from(value: ::std::os::raw::c_int) -> Self {
        CursorMoveStyle(value)
    }
}

impl From<CursorMoveStyle> for ::std::os::raw::c_int {
    fn from(value: CursorMoveStyle) -> Self {
        value.0
    }
}

impl CursorMoveStyle {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl CursorMoveStyle {
    /// Within a left-to-right text block, decrease cursor position when pressing left arrow key, increase cursor position when pressing the right arrow key. If the text block is right-to-left, the opposite behavior applies. (C++ enum variant: <span style='color: green;'>```LogicalMoveStyle = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const LogicalMoveStyle: crate::CursorMoveStyle = crate::CursorMoveStyle(0);
    /// Pressing the left arrow key will always cause the cursor to move left, regardless of the text's writing direction. Pressing the right arrow key will always cause the cursor to move right. (C++ enum variant: <span style='color: green;'>```VisualMoveStyle = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const VisualMoveStyle: crate::CursorMoveStyle = crate::CursorMoveStyle(1);
}

/// <p>The timer type indicates how accurate a timer can be.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::TimerType```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#TimerType-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The timer type indicates how accurate a timer can be.</p>
///
/// <p>On UNIX (including Linux, <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, and iOS), Qt will keep millisecond accuracy for Qt::PreciseTimer. For Qt::CoarseTimer, the interval will be adjusted up to 5% to align the timer with other timers that are expected to fire at or around the same time. The objective is to make most timers wake up at the same time, thereby reducing CPU wakeups and power consumption.</p>
/// <p>On Windows, Qt will use Windows's Multimedia timer facility (if available) for Qt::PreciseTimer and normal Windows timers for Qt::CoarseTimer and Qt::VeryCoarseTimer.</p>
/// <p>On all platforms, the interval for Qt::VeryCoarseTimer is rounded to the nearest full second (e.g. an interval of 23500ms will be rounded to 24000ms, and 20300ms to 20000ms).</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct TimerType(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for TimerType {
    fn from(value: ::std::os::raw::c_int) -> Self {
        TimerType(value)
    }
}

impl From<TimerType> for ::std::os::raw::c_int {
    fn from(value: TimerType) -> Self {
        value.0
    }
}

impl TimerType {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl TimerType {
    /// Precise timers try to keep millisecond accuracy (C++ enum variant: <span style='color: green;'>```PreciseTimer = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const PreciseTimer: crate::TimerType = crate::TimerType(0);
    /// Coarse timers try to keep accuracy within 5% of the desired interval (C++ enum variant: <span style='color: green;'>```CoarseTimer = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const CoarseTimer: crate::TimerType = crate::TimerType(1);
    /// Very coarse timers only keep full second accuracy (C++ enum variant: <span style='color: green;'>```VeryCoarseTimer = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const VeryCoarseTimer: crate::TimerType = crate::TimerType(2);
}

/// <p>This enum describes the phase of scrolling.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::ScrollPhase```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#ScrollPhase-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the phase of scrolling.</p>
///
/// <p>This enum was introduced or modified in  Qt 5.2.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct ScrollPhase(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for ScrollPhase {
    fn from(value: ::std::os::raw::c_int) -> Self {
        ScrollPhase(value)
    }
}

impl From<ScrollPhase> for ::std::os::raw::c_int {
    fn from(value: ScrollPhase) -> Self {
        value.0
    }
}

impl ScrollPhase {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl ScrollPhase {
    /// The input device doesn't support scroll phase. This value was introduced in Qt 5.7. (C++ enum variant: <span style='color: green;'>```NoScrollPhase = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const NoScrollPhase: crate::ScrollPhase = crate::ScrollPhase(0);
    /// Scrolling is about to begin, but the scrolling distance did not yet change. (C++ enum variant: <span style='color: green;'>```ScrollBegin = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const ScrollBegin: crate::ScrollPhase = crate::ScrollPhase(1);
    /// The scrolling distance has changed (default). (C++ enum variant: <span style='color: green;'>```ScrollUpdate = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const ScrollUpdate: crate::ScrollPhase = crate::ScrollPhase(2);
    /// Scrolling has ended, but the scrolling distance did not change anymore. (C++ enum variant: <span style='color: green;'>```ScrollEnd = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const ScrollEnd: crate::ScrollPhase = crate::ScrollPhase(3);
    /// The user no longer touches the input device, but scrolling continues due to scroll momentum. This value was introduced in Qt 5.12. (C++ enum variant: <span style='color: green;'>```ScrollMomentum = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const ScrollMomentum: crate::ScrollPhase = crate::ScrollPhase(4);
}

/// <p>This enum describes the source of a mouse event and can be useful to determine if the event is an artificial mouse event originating from another device such as a touchscreen.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::MouseEventSource```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#MouseEventSource-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the source of a mouse event and can be useful to determine if the event is an artificial mouse event originating from another device such as a touchscreen.</p>
///
/// <p>This enum was introduced or modified in  Qt 5.3.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qt.html#ApplicationAttribute-enum">Qt::AA_SynthesizeMouseForUnhandledTouchEvents</a>.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct MouseEventSource(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for MouseEventSource {
    fn from(value: ::std::os::raw::c_int) -> Self {
        MouseEventSource(value)
    }
}

impl From<MouseEventSource> for ::std::os::raw::c_int {
    fn from(value: MouseEventSource) -> Self {
        value.0
    }
}

impl MouseEventSource {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl MouseEventSource {
    /// The most common value. On platforms where such information is available this value indicates that the event was generated in response to a genuine mouse event in the system. (C++ enum variant: <span style='color: green;'>```MouseEventNotSynthesized = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const MouseEventNotSynthesized: crate::MouseEventSource = crate::MouseEventSource(0);
    /// Indicates that the mouse event was synthesized from a touch event by the platform. (C++ enum variant: <span style='color: green;'>```MouseEventSynthesizedBySystem = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const MouseEventSynthesizedBySystem: crate::MouseEventSource = crate::MouseEventSource(1);
    /// Indicates that the mouse event was synthesized from an unhandled touch event by Qt. (C++ enum variant: <span style='color: green;'>```MouseEventSynthesizedByQt = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const MouseEventSynthesizedByQt: crate::MouseEventSource = crate::MouseEventSource(2);
    /// Indicates that the mouse event was synthesized by the application. This allows distinguishing application-generated mouse events from the ones that are coming from the system or are synthesized by Qt. This value was introduced in Qt 5.6 (C++ enum variant: <span style='color: green;'>```MouseEventSynthesizedByApplication = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const MouseEventSynthesizedByApplication: crate::MouseEventSource =
        crate::MouseEventSource(3);
}

/// <p>This enum provides additional information concerning a <a href="http://doc.qt.io/qt-5/qmouseevent.html">QMouseEvent</a>.</p>
///
/// C++ enum: <span style='color: green;'>```Qt::MouseEventFlag```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#MouseEventFlag-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum provides additional information concerning a <a href="http://doc.qt.io/qt-5/qmouseevent.html">QMouseEvent</a>.</p>
///
/// <p>This enum was introduced or modified in  Qt 5.3.</p>
/// <p>The MouseEventFlags type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;MouseEventFlag&gt;. It stores an OR combination of MouseEventFlag values.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct MouseEventFlag(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for MouseEventFlag {
    fn from(value: ::std::os::raw::c_int) -> Self {
        MouseEventFlag(value)
    }
}

impl From<MouseEventFlag> for ::std::os::raw::c_int {
    fn from(value: MouseEventFlag) -> Self {
        value.0
    }
}

impl MouseEventFlag {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl MouseEventFlag {
    /// Indicates that Qt has created a <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">MouseButtonDblClick</a> event from this event. The flag is set in the causing <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">MouseButtonPress</a>, and not in the resulting <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">MouseButtonDblClick</a>. (C++ enum variant: <span style='color: green;'>```MouseEventCreatedDoubleClick = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const MouseEventCreatedDoubleClick: crate::MouseEventFlag = crate::MouseEventFlag(1);
    /// C++ enum variant: <span style='color: green;'>```MouseEventFlagMask = 255```</span>
    #[allow(non_upper_case_globals)]
    pub const MouseEventFlagMask: crate::MouseEventFlag = crate::MouseEventFlag(255);
}

/// <p>This enum describes the possible standards used by <a href="http://doc.qt.io/qt-5/qbytearray.html#qChecksum">qChecksum</a>().</p>
///
/// C++ enum: <span style='color: green;'>```Qt::ChecksumType```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qt.html#ChecksumType-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the possible standards used by <a href="http://doc.qt.io/qt-5/qbytearray.html#qChecksum">qChecksum</a>().</p>
///
/// <p>This enum was introduced or modified in  Qt 5.9.</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct ChecksumType(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for ChecksumType {
    fn from(value: ::std::os::raw::c_int) -> Self {
        ChecksumType(value)
    }
}

impl From<ChecksumType> for ::std::os::raw::c_int {
    fn from(value: ChecksumType) -> Self {
        value.0
    }
}

impl ChecksumType {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl ChecksumType {
    /// Checksum calculation based on ISO 3309. (C++ enum variant: <span style='color: green;'>```ChecksumIso3309 = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const ChecksumIso3309: crate::ChecksumType = crate::ChecksumType(0);
    /// Checksum calculation based on ITU-V.41. (C++ enum variant: <span style='color: green;'>```ChecksumItuV41 = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const ChecksumItuV41: crate::ChecksumType = crate::ChecksumType(1);
}

pub mod q_meta_object {
    //! C++ type: <span style='color: green;'>```QMetaObject```</span>

    /// C++ enum: <span style='color: green;'>```QMetaObject::Call```</span>.
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Call(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Call {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Call(value)
        }
    }

    impl From<Call> for ::std::os::raw::c_int {
        fn from(value: Call) -> Self {
            value.0
        }
    }

    impl Call {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Call {
        /// C++ enum variant: <span style='color: green;'>```InvokeMetaMethod = 0```</span>
        #[allow(non_upper_case_globals)]
        pub const InvokeMetaMethod: crate::q_meta_object::Call = crate::q_meta_object::Call(0);
        /// C++ enum variant: <span style='color: green;'>```ReadProperty = 1```</span>
        #[allow(non_upper_case_globals)]
        pub const ReadProperty: crate::q_meta_object::Call = crate::q_meta_object::Call(1);
        /// C++ enum variant: <span style='color: green;'>```WriteProperty = 2```</span>
        #[allow(non_upper_case_globals)]
        pub const WriteProperty: crate::q_meta_object::Call = crate::q_meta_object::Call(2);
        /// C++ enum variant: <span style='color: green;'>```ResetProperty = 3```</span>
        #[allow(non_upper_case_globals)]
        pub const ResetProperty: crate::q_meta_object::Call = crate::q_meta_object::Call(3);
        /// C++ enum variant: <span style='color: green;'>```QueryPropertyDesignable = 4```</span>
        #[allow(non_upper_case_globals)]
        pub const QueryPropertyDesignable: crate::q_meta_object::Call =
            crate::q_meta_object::Call(4);
        /// C++ enum variant: <span style='color: green;'>```QueryPropertyScriptable = 5```</span>
        #[allow(non_upper_case_globals)]
        pub const QueryPropertyScriptable: crate::q_meta_object::Call =
            crate::q_meta_object::Call(5);
        /// C++ enum variant: <span style='color: green;'>```QueryPropertyStored = 6```</span>
        #[allow(non_upper_case_globals)]
        pub const QueryPropertyStored: crate::q_meta_object::Call = crate::q_meta_object::Call(6);
        /// C++ enum variant: <span style='color: green;'>```QueryPropertyEditable = 7```</span>
        #[allow(non_upper_case_globals)]
        pub const QueryPropertyEditable: crate::q_meta_object::Call = crate::q_meta_object::Call(7);
        /// C++ enum variant: <span style='color: green;'>```QueryPropertyUser = 8```</span>
        #[allow(non_upper_case_globals)]
        pub const QueryPropertyUser: crate::q_meta_object::Call = crate::q_meta_object::Call(8);
        /// C++ enum variant: <span style='color: green;'>```CreateInstance = 9```</span>
        #[allow(non_upper_case_globals)]
        pub const CreateInstance: crate::q_meta_object::Call = crate::q_meta_object::Call(9);
        /// C++ enum variant: <span style='color: green;'>```IndexOfMethod = 10```</span>
        #[allow(non_upper_case_globals)]
        pub const IndexOfMethod: crate::q_meta_object::Call = crate::q_meta_object::Call(10);
        /// C++ enum variant: <span style='color: green;'>```RegisterPropertyMetaType = 11```</span>
        #[allow(non_upper_case_globals)]
        pub const RegisterPropertyMetaType: crate::q_meta_object::Call =
            crate::q_meta_object::Call(11);
        /// C++ enum variant: <span style='color: green;'>```RegisterMethodArgumentMetaType = 12```</span>
        #[allow(non_upper_case_globals)]
        pub const RegisterMethodArgumentMetaType: crate::q_meta_object::Call =
            crate::q_meta_object::Call(12);
    }

    /// <p>Represents a handle to a signal-slot connection. It can be used to disconnect that connection, or check if the connection was successful</p>
    ///
    /// C++ class: <span style='color: green;'>```QMetaObject::Connection```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject-connection.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Represents a handle to a signal-slot connection. It can be used to disconnect that connection, or check if the connection was successful</p></div>
    #[repr(C)]
    pub struct Connection {
        _unused: u8,
    }
    impl Connection {
        /// <p>Assigns <i>other</i> to this connection and returns a reference to this connection.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMetaObject::Connection& QMetaObject::Connection::operator=(const QMetaObject::Connection& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmetaobject-connection.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this connection and returns a reference to this connection.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_meta_object::Connection>>,
        ) -> ::cpp_utils::MutRef<crate::q_meta_object::Connection> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaObject_Connection_operator_(self as *mut crate::q_meta_object::Connection, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_meta_object::Connection>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Creates a Connection instance.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QMetaObject::Connection::Connection()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmetaobject-connection.html#Connection">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a Connection instance.</p></div>
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_meta_object::Connection> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaObject_Connection_Connection();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs a copy of <i>other</i>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QMetaObject::Connection::Connection(const QMetaObject::Connection& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmetaobject-connection.html#Connection-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p></div>
        pub unsafe fn new_copy(
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_meta_object::Connection>>,
        ) -> ::cpp_utils::CppBox<crate::q_meta_object::Connection> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaObject_Connection_Connection1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_meta_object::Connection>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_meta_object::Connection {
        /// <p>Destructor for <a href="http://doc.qt.io/qt-5/qmetaobject-connection.html">QMetaObject::Connection</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QMetaObject::Connection::~Connection()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmetaobject-connection.html#dtor.Connection">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destructor for <a href="http://doc.qt.io/qt-5/qmetaobject-connection.html">QMetaObject::Connection</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QMetaObject_Connection_dConnection(
                self as *mut crate::q_meta_object::Connection,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qmetaobject.html">QMetaObject</a> class contains meta-information about Qt objects.</p>
///
/// C++ class: <span style='color: green;'>```QMetaObject```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qmetaobject.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmetaobject.html">QMetaObject</a> class contains meta-information about Qt objects.</p>
/// <p>The Qt <a href="http://doc.qt.io/qt-5/metaobjects.html">Meta-Object System</a> in Qt is responsible for the signals and slots inter-object communication mechanism, runtime type information, and the Qt property system. A single <a href="http://doc.qt.io/qt-5/qmetaobject.html">QMetaObject</a> instance is created for each <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> subclass that is used in an application, and this instance stores all the meta-information for the <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> subclass. This object is available as <a href="http://doc.qt.io/qt-5/qobject.html#metaObject">QObject::metaObject</a>().</p>
/// <p>This class is not normally required for application programming, but it is useful if you write meta-applications, such as scripting engines or GUI builders.</p>
/// <p>The functions you are most likely to find useful are these:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#className">className</a>() returns the name of a class.</li>
/// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#superClass">superClass</a>() returns the superclass's meta-object.</li>
/// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#method">method</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#methodCount">methodCount</a>() provide information about a class's meta-methods (signals, slots and other <a href="http://doc.qt.io/qt-5/qobject.html#Q_INVOKABLE">invokable</a> member functions).</li>
/// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#enumerator">enumerator</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#enumeratorCount">enumeratorCount</a>() and provide information about a class's enumerators.</li>
/// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#propertyCount">propertyCount</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#property">property</a>() provide information about a class's properties.</li>
/// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#constructor">constructor</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#constructorCount">constructorCount</a>() provide information about a class's meta-constructors.</li>
/// </ul>
/// <p>The index functions <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfConstructor">indexOfConstructor</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfMethod">indexOfMethod</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfEnumerator">indexOfEnumerator</a>(), and <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfProperty">indexOfProperty</a>() map names of constructors, member functions, enumerators, or properties to indexes in the meta-object. For example, Qt uses <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfMethod">indexOfMethod</a>() internally when you connect a signal to a slot.</p>
/// <p>Classes can also have a list of <i>name</i>--<i>value</i> pairs of additional class information, stored in <a href="http://doc.qt.io/qt-5/qmetaclassinfo.html">QMetaClassInfo</a> objects. The number of pairs is returned by <a href="http://doc.qt.io/qt-5/qmetaobject.html#classInfoCount">classInfoCount</a>(), single pairs are returned by <a href="http://doc.qt.io/qt-5/qmetaobject.html#classInfo">classInfo</a>(), and you can search for pairs with <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfClassInfo">indexOfClassInfo</a>().</p></div>
#[repr(C)]
pub struct QMetaObject {
    _unused: u8,
}
impl QMetaObject {
    /// Calls C++ function: <span style='color: green;'>```static void QMetaObject::activate(QObject* sender, int signal_index, void** argv)```</span>.
    pub unsafe fn activate_q_object_int_void_ptr(
        sender: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
        signal_index: ::std::os::raw::c_int,
        argv: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_activate(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(sender)
                .as_mut_raw_ptr(),
            signal_index,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(argv)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```static void QMetaObject::activate(QObject* sender, const QMetaObject* arg2, int local_signal_index, void** argv)```</span>.
    pub unsafe fn activate_q_object_q_meta_object_int_void_ptr(
        sender: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
        arg2: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QMetaObject>>,
        local_signal_index: ::std::os::raw::c_int,
        argv: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_activate1(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(sender)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QMetaObject>>::cast_into(arg2)
                .as_raw_ptr(),
            local_signal_index,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(argv)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```static void QMetaObject::activate(QObject* sender, int signal_offset, int local_signal_index, void** argv)```</span>.
    pub unsafe fn activate_q_object2_int_void_ptr(
        sender: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
        signal_offset: ::std::os::raw::c_int,
        local_signal_index: ::std::os::raw::c_int,
        argv: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_activate2(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(sender)
                .as_mut_raw_ptr(),
            signal_offset,
            local_signal_index,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(argv)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* QMetaObject::cast(QObject* obj) const```</span>.
    pub unsafe fn cast_q_object(
        &self,
        obj: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaObject_cast(
            self as *const crate::QMetaObject,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(obj)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```const QObject* QMetaObject::cast(const QObject* obj) const```</span>.
    pub unsafe fn cast_q_object2(
        &self,
        obj: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaObject_cast1(
            self as *const crate::QMetaObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(obj).as_raw_ptr(),
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns <code>true</code> if the <i>signal</i> and <i>method</i> arguments are compatible; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QMetaObject::checkConnectArgs(const char* signal, const char* method)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#checkConnectArgs">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the <i>signal</i> and <i>method</i> arguments are compatible; otherwise returns <code>false</code>.</p>
    /// <p>Both <i>signal</i> and <i>method</i> are expected to be normalized.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#normalizedSignature">normalizedSignature</a>().</p></div>
    pub unsafe fn check_connect_args_2_char(
        signal: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        method: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_checkConnectArgs(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(signal)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(method)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QMetaObject::checkConnectArgs(const QMetaMethod& signal, const QMetaMethod& method)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#checkConnectArgs-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the <i>signal</i> and <i>method</i> arguments are compatible; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn check_connect_args_2_q_meta_method(
        signal: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMetaMethod>>,
        method: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMetaMethod>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_checkConnectArgs1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMetaMethod>>::cast_into(signal)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMetaMethod>>::cast_into(method)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the meta-data for the item of class information with the given <i>index</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMetaClassInfo QMetaObject::classInfo(int index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#classInfo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the meta-data for the item of class information with the given <i>index</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">class</span> MyClass : <span class="keyword">public</span> <span class="type"><a href="http://doc.qt.io/qt-5/qobject.html">QObject</a></span>
    ///   {
    /// &#32;     Q_OBJECT
    /// &#32;     Q_CLASSINFO(<span class="string">"author"</span><span class="operator">,</span> <span class="string">"Sabrina Schweinsteiger"</span>)
    /// &#32;     Q_CLASSINFO(<span class="string">"url"</span><span class="operator">,</span> <span class="string">"http://doc.moosesoft.co.uk/1.0/"</span>)
    ///
    ///   <span class="keyword">public</span>:
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   };
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#classInfoCount">classInfoCount</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#classInfoOffset">classInfoOffset</a>(), and <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfClassInfo">indexOfClassInfo</a>().</p></div>
    pub unsafe fn class_info(
        &self,
        index: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QMetaClassInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaObject_classInfo(
            self as *const crate::QMetaObject,
            index,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the number of items of class information in this class.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaObject::classInfoCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#classInfoCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items of class information in this class.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#classInfo">classInfo</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#classInfoOffset">classInfoOffset</a>(), and <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfClassInfo">indexOfClassInfo</a>().</p></div>
    pub unsafe fn class_info_count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_classInfoCount(self as *const crate::QMetaObject)
    }

    /// <p>Returns the class information offset for this class; i.e. the index position of this class's first class information item.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaObject::classInfoOffset() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#classInfoOffset">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the class information offset for this class; i.e. the index position of this class's first class information item.</p>
    /// <p>If the class has no superclasses with class information, the offset is 0; otherwise the offset is the sum of all the class information items in the class's superclasses.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#classInfo">classInfo</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#classInfoCount">classInfoCount</a>(), and <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfClassInfo">indexOfClassInfo</a>().</p></div>
    pub unsafe fn class_info_offset(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_classInfoOffset(self as *const crate::QMetaObject)
    }

    /// <p>Returns the class name.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QMetaObject::className() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#className">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the class name.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#superClass">superClass</a>().</p></div>
    pub unsafe fn class_name(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMetaObject_className(self as *const crate::QMetaObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```static QMetaObject::Connection QMetaObject::connect(const QObject* sender, int signal_index, const QObject* receiver, int method_index, int type = …, int* types = …)```</span>.
    pub unsafe fn connect_6a(
        sender: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        signal_index: ::std::os::raw::c_int,
        receiver: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        method_index: ::std::os::raw::c_int,
        type_: ::std::os::raw::c_int,
        types: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::CppBox<crate::q_meta_object::Connection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaObject_connect(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(sender)
                .as_raw_ptr(),
            signal_index,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(receiver)
                .as_raw_ptr(),
            method_index,
            type_,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_int>>::cast_into(types)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QMetaObject::Connection QMetaObject::connect(const QObject* sender, int signal_index, const QObject* receiver, int method_index, int type = …)```</span>.
    pub unsafe fn connect_5a(
        sender: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        signal_index: ::std::os::raw::c_int,
        receiver: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        method_index: ::std::os::raw::c_int,
        type_: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::q_meta_object::Connection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaObject_connect1(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(sender)
                .as_raw_ptr(),
            signal_index,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(receiver)
                .as_raw_ptr(),
            method_index,
            type_,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QMetaObject::Connection QMetaObject::connect(const QObject* sender, int signal_index, const QObject* receiver, int method_index)```</span>.
    pub unsafe fn connect_4a(
        sender: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        signal_index: ::std::os::raw::c_int,
        receiver: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        method_index: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::q_meta_object::Connection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaObject_connect2(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(sender)
                .as_raw_ptr(),
            signal_index,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(receiver)
                .as_raw_ptr(),
            method_index,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Searches recursively for all child objects of the given <i>object</i>, and connects matching signals from them to slots of <i>object</i> that follow the following form:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QMetaObject::connectSlotsByName(QObject* o)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#connectSlotsByName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Searches recursively for all child objects of the given <i>object</i>, and connects matching signals from them to slots of <i>object</i> that follow the following form:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">void</span> on_<span class="operator">&lt;</span>object name<span class="operator">&gt;</span>_<span class="operator">&lt;</span>signal name<span class="operator">&gt;</span>(<span class="operator">&lt;</span>signal parameters<span class="operator">&gt;</span>);
    ///
    /// </pre>
    /// <p>Let's assume our object has a child object of type <code>QPushButton</code> with the <a href="http://doc.qt.io/qt-5/qobject.html#objectName-prop">object name</a> <code>button1</code>. The slot to catch the button's <code>clicked()</code> signal would be:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">void</span> on_button1_clicked();
    ///
    /// </pre>
    /// <p>If <i>object</i> itself has a properly set object name, its own signals are also connected to its respective slots.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#objectName-prop">QObject::setObjectName</a>().</p></div>
    pub unsafe fn connect_slots_by_name(
        o: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_connectSlotsByName(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(o)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the meta-data for the constructor with the given <i>index</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMetaMethod QMetaObject::constructor(int index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#constructor">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the meta-data for the constructor with the given <i>index</i>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#constructorCount">constructorCount</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#newInstance">newInstance</a>().</p></div>
    pub unsafe fn constructor(
        &self,
        index: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QMetaMethod> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaObject_constructor(
            self as *const crate::QMetaObject,
            index,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the number of constructors in this class.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaObject::constructorCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#constructorCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of constructors in this class.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#constructor">constructor</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfConstructor">indexOfConstructor</a>().</p></div>
    pub unsafe fn constructor_count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_constructorCount(
            self as *const crate::QMetaObject,
        )
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qmetaobject.html">QMetaObject</a> class contains meta-information about Qt objects.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMetaObject& QMetaObject::operator=(const QMetaObject& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmetaobject.html">QMetaObject</a> class contains meta-information about Qt objects.</p>
    /// <p>The Qt <a href="http://doc.qt.io/qt-5/metaobjects.html">Meta-Object System</a> in Qt is responsible for the signals and slots inter-object communication mechanism, runtime type information, and the Qt property system. A single <a href="http://doc.qt.io/qt-5/qmetaobject.html">QMetaObject</a> instance is created for each <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> subclass that is used in an application, and this instance stores all the meta-information for the <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> subclass. This object is available as <a href="http://doc.qt.io/qt-5/qobject.html#metaObject">QObject::metaObject</a>().</p>
    /// <p>This class is not normally required for application programming, but it is useful if you write meta-applications, such as scripting engines or GUI builders.</p>
    /// <p>The functions you are most likely to find useful are these:</p>
    /// <ul>
    /// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#className">className</a>() returns the name of a class.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#superClass">superClass</a>() returns the superclass's meta-object.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#method">method</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#methodCount">methodCount</a>() provide information about a class's meta-methods (signals, slots and other <a href="http://doc.qt.io/qt-5/qobject.html#Q_INVOKABLE">invokable</a> member functions).</li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#enumerator">enumerator</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#enumeratorCount">enumeratorCount</a>() and provide information about a class's enumerators.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#propertyCount">propertyCount</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#property">property</a>() provide information about a class's properties.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#constructor">constructor</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#constructorCount">constructorCount</a>() provide information about a class's meta-constructors.</li>
    /// </ul>
    /// <p>The index functions <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfConstructor">indexOfConstructor</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfMethod">indexOfMethod</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfEnumerator">indexOfEnumerator</a>(), and <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfProperty">indexOfProperty</a>() map names of constructors, member functions, enumerators, or properties to indexes in the meta-object. For example, Qt uses <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfMethod">indexOfMethod</a>() internally when you connect a signal to a slot.</p>
    /// <p>Classes can also have a list of <i>name</i>--<i>value</i> pairs of additional class information, stored in <a href="http://doc.qt.io/qt-5/qmetaclassinfo.html">QMetaClassInfo</a> objects. The number of pairs is returned by <a href="http://doc.qt.io/qt-5/qmetaobject.html#classInfoCount">classInfoCount</a>(), single pairs are returned by <a href="http://doc.qt.io/qt-5/qmetaobject.html#classInfo">classInfo</a>(), and you can search for pairs with <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfClassInfo">indexOfClassInfo</a>().</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMetaObject>>,
    ) -> ::cpp_utils::MutRef<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaObject_operator_(
            self as *mut crate::QMetaObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMetaObject>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static bool QMetaObject::disconnect(const QObject* sender, int signal_index, const QObject* receiver, int method_index)```</span>.
    pub unsafe fn disconnect(
        sender: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        signal_index: ::std::os::raw::c_int,
        receiver: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        method_index: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_disconnect(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(sender)
                .as_raw_ptr(),
            signal_index,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(receiver)
                .as_raw_ptr(),
            method_index,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```static bool QMetaObject::disconnectOne(const QObject* sender, int signal_index, const QObject* receiver, int method_index)```</span>.
    pub unsafe fn disconnect_one(
        sender: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        signal_index: ::std::os::raw::c_int,
        receiver: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        method_index: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_disconnectOne(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(sender)
                .as_raw_ptr(),
            signal_index,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(receiver)
                .as_raw_ptr(),
            method_index,
        )
    }

    /// <p>Returns the meta-data for the enumerator with the given <i>index</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMetaEnum QMetaObject::enumerator(int index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#enumerator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the meta-data for the enumerator with the given <i>index</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#enumeratorCount">enumeratorCount</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#enumeratorOffset">enumeratorOffset</a>(), and <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfEnumerator">indexOfEnumerator</a>().</p></div>
    pub unsafe fn enumerator(
        &self,
        index: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QMetaEnum> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaObject_enumerator(
            self as *const crate::QMetaObject,
            index,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the number of enumerators in this class.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaObject::enumeratorCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#enumeratorCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of enumerators in this class.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#enumerator">enumerator</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#enumeratorOffset">enumeratorOffset</a>(), and <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfEnumerator">indexOfEnumerator</a>().</p></div>
    pub unsafe fn enumerator_count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_enumeratorCount(self as *const crate::QMetaObject)
    }

    /// <p>Returns the enumerator offset for this class; i.e. the index position of this class's first enumerator.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaObject::enumeratorOffset() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#enumeratorOffset">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the enumerator offset for this class; i.e. the index position of this class's first enumerator.</p>
    /// <p>If the class has no superclasses with enumerators, the offset is 0; otherwise the offset is the sum of all the enumerators in the class's superclasses.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#enumerator">enumerator</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#enumeratorCount">enumeratorCount</a>(), and <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfEnumerator">indexOfEnumerator</a>().</p></div>
    pub unsafe fn enumerator_offset(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_enumeratorOffset(
            self as *const crate::QMetaObject,
        )
    }

    /// <p>Finds class information item <i>name</i> and returns its index; otherwise returns -1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaObject::indexOfClassInfo(const char* name) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfClassInfo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Finds class information item <i>name</i> and returns its index; otherwise returns -1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#classInfo">classInfo</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#classInfoCount">classInfoCount</a>(), and <a href="http://doc.qt.io/qt-5/qmetaobject.html#classInfoOffset">classInfoOffset</a>().</p></div>
    pub unsafe fn index_of_class_info(
        &self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_indexOfClassInfo(
            self as *const crate::QMetaObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(name)
                .as_raw_ptr(),
        )
    }

    /// <p>Finds <i>constructor</i> and returns its index; otherwise returns -1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaObject::indexOfConstructor(const char* constructor) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfConstructor">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Finds <i>constructor</i> and returns its index; otherwise returns -1.</p>
    /// <p>Note that the <i>constructor</i> has to be in normalized form, as returned by <a href="http://doc.qt.io/qt-5/qmetaobject.html#normalizedSignature">normalizedSignature</a>().</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#constructor">constructor</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#constructorCount">constructorCount</a>(), and <a href="http://doc.qt.io/qt-5/qmetaobject.html#normalizedSignature">normalizedSignature</a>().</p></div>
    pub unsafe fn index_of_constructor(
        &self,
        constructor: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_indexOfConstructor(
            self as *const crate::QMetaObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(
                constructor,
            )
            .as_raw_ptr(),
        )
    }

    /// <p>Finds enumerator <i>name</i> and returns its index; otherwise returns -1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaObject::indexOfEnumerator(const char* name) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfEnumerator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Finds enumerator <i>name</i> and returns its index; otherwise returns -1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#enumerator">enumerator</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#enumeratorCount">enumeratorCount</a>(), and <a href="http://doc.qt.io/qt-5/qmetaobject.html#enumeratorOffset">enumeratorOffset</a>().</p></div>
    pub unsafe fn index_of_enumerator(
        &self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_indexOfEnumerator(
            self as *const crate::QMetaObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(name)
                .as_raw_ptr(),
        )
    }

    /// <p>Finds <i>method</i> and returns its index; otherwise returns -1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaObject::indexOfMethod(const char* method) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfMethod">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Finds <i>method</i> and returns its index; otherwise returns -1.</p>
    /// <p>Note that the <i>method</i> has to be in normalized form, as returned by <a href="http://doc.qt.io/qt-5/qmetaobject.html#normalizedSignature">normalizedSignature</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#method">method</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#methodCount">methodCount</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#methodOffset">methodOffset</a>(), and <a href="http://doc.qt.io/qt-5/qmetaobject.html#normalizedSignature">normalizedSignature</a>().</p></div>
    pub unsafe fn index_of_method(
        &self,
        method: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_indexOfMethod(
            self as *const crate::QMetaObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(method)
                .as_raw_ptr(),
        )
    }

    /// <p>Finds property <i>name</i> and returns its index; otherwise returns -1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaObject::indexOfProperty(const char* name) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfProperty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Finds property <i>name</i> and returns its index; otherwise returns -1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#property">property</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#propertyCount">propertyCount</a>(), and <a href="http://doc.qt.io/qt-5/qmetaobject.html#propertyOffset">propertyOffset</a>().</p></div>
    pub unsafe fn index_of_property(
        &self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_indexOfProperty(
            self as *const crate::QMetaObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(name)
                .as_raw_ptr(),
        )
    }

    /// <p>Finds <i>signal</i> and returns its index; otherwise returns -1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaObject::indexOfSignal(const char* signal) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfSignal">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Finds <i>signal</i> and returns its index; otherwise returns -1.</p>
    /// <p>This is the same as <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfMethod">indexOfMethod</a>(), except that it will return -1 if the method exists but isn't a signal.</p>
    /// <p>Note that the <i>signal</i> has to be in normalized form, as returned by <a href="http://doc.qt.io/qt-5/qmetaobject.html#normalizedSignature">normalizedSignature</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfMethod">indexOfMethod</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#normalizedSignature">normalizedSignature</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#method">method</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#methodCount">methodCount</a>(), and <a href="http://doc.qt.io/qt-5/qmetaobject.html#methodOffset">methodOffset</a>().</p></div>
    pub unsafe fn index_of_signal(
        &self,
        signal: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_indexOfSignal(
            self as *const crate::QMetaObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(signal)
                .as_raw_ptr(),
        )
    }

    /// <p>Finds <i>slot</i> and returns its index; otherwise returns -1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaObject::indexOfSlot(const char* slot) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfSlot">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Finds <i>slot</i> and returns its index; otherwise returns -1.</p>
    /// <p>This is the same as <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfMethod">indexOfMethod</a>(), except that it will return -1 if the method exists but isn't a slot.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfMethod">indexOfMethod</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#method">method</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#methodCount">methodCount</a>(), and <a href="http://doc.qt.io/qt-5/qmetaobject.html#methodOffset">methodOffset</a>().</p></div>
    pub unsafe fn index_of_slot(
        &self,
        slot: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_indexOfSlot(
            self as *const crate::QMetaObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(slot)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if the class described by this <a href="http://doc.qt.io/qt-5/qmetaobject.html">QMetaObject</a> inherits the type described by <i>metaObject</i>; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaObject::inherits(const QMetaObject* metaObject) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#inherits">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the class described by this <a href="http://doc.qt.io/qt-5/qmetaobject.html">QMetaObject</a> inherits the type described by <i>metaObject</i>; otherwise returns false.</p>
    /// <p>A type is considered to inherit itself.</p>
    /// <p>This function was introduced in  Qt 5.7.</p></div>
    pub unsafe fn inherits(
        &self,
        meta_object: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QMetaObject>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_inherits(
            self as *const crate::QMetaObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QMetaObject>>::cast_into(meta_object)
                .as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```static int QMetaObject::metacall(QObject* arg1, QMetaObject::Call arg2, int arg3, void** arg4)```</span>.
    pub unsafe fn metacall(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
        arg2: crate::q_meta_object::Call,
        arg3: ::std::os::raw::c_int,
        arg4: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_metacall(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(arg1)
                .as_mut_raw_ptr(),
            arg2,
            arg3,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg4)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the meta-data for the method with the given <i>index</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMetaMethod QMetaObject::method(int index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#method">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the meta-data for the method with the given <i>index</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#methodCount">methodCount</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#methodOffset">methodOffset</a>(), and <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfMethod">indexOfMethod</a>().</p></div>
    pub unsafe fn method(
        &self,
        index: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QMetaMethod> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaObject_method(
            self as *const crate::QMetaObject,
            index,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the number of methods in this class, including the number of methods provided by each base class. These include signals and slots as well as normal member functions.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaObject::methodCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#methodCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of methods in this class, including the number of methods provided by each base class. These include signals and slots as well as normal member functions.</p>
    /// <p>Use code like the following to obtain a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> containing the methods specific to a given class:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qmetaobject.html">QMetaObject</a></span><span class="operator">*</span> metaObject <span class="operator">=</span> obj<span class="operator">-</span><span class="operator">&gt;</span>metaObject();
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> methods;
    ///   <span class="keyword">for</span>(<span class="type">int</span> i <span class="operator">=</span> metaObject<span class="operator">-</span><span class="operator">&gt;</span>methodOffset(); i <span class="operator">&lt;</span> metaObject<span class="operator">-</span><span class="operator">&gt;</span>methodCount(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     methods <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">::</span>fromLatin1(metaObject<span class="operator">-</span><span class="operator">&gt;</span>method(i)<span class="operator">.</span>methodSignature());
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#method">method</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#methodOffset">methodOffset</a>(), and <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfMethod">indexOfMethod</a>().</p></div>
    pub unsafe fn method_count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_methodCount(self as *const crate::QMetaObject)
    }

    /// <p>Returns the method offset for this class; i.e. the index position of this class's first member function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaObject::methodOffset() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#methodOffset">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the method offset for this class; i.e. the index position of this class's first member function.</p>
    /// <p>The offset is the sum of all the methods in the class's superclasses (which is always positive since <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> has the deleteLater() slot and a destroyed() signal).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#method">method</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#methodCount">methodCount</a>(), and <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfMethod">indexOfMethod</a>().</p></div>
    pub unsafe fn method_offset(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_methodOffset(self as *const crate::QMetaObject)
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qmetaobject.html">QMetaObject</a> class contains meta-information about Qt objects.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMetaObject::QMetaObject()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmetaobject.html">QMetaObject</a> class contains meta-information about Qt objects.</p>
    /// <p>The Qt <a href="http://doc.qt.io/qt-5/metaobjects.html">Meta-Object System</a> in Qt is responsible for the signals and slots inter-object communication mechanism, runtime type information, and the Qt property system. A single <a href="http://doc.qt.io/qt-5/qmetaobject.html">QMetaObject</a> instance is created for each <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> subclass that is used in an application, and this instance stores all the meta-information for the <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> subclass. This object is available as <a href="http://doc.qt.io/qt-5/qobject.html#metaObject">QObject::metaObject</a>().</p>
    /// <p>This class is not normally required for application programming, but it is useful if you write meta-applications, such as scripting engines or GUI builders.</p>
    /// <p>The functions you are most likely to find useful are these:</p>
    /// <ul>
    /// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#className">className</a>() returns the name of a class.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#superClass">superClass</a>() returns the superclass's meta-object.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#method">method</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#methodCount">methodCount</a>() provide information about a class's meta-methods (signals, slots and other <a href="http://doc.qt.io/qt-5/qobject.html#Q_INVOKABLE">invokable</a> member functions).</li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#enumerator">enumerator</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#enumeratorCount">enumeratorCount</a>() and provide information about a class's enumerators.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#propertyCount">propertyCount</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#property">property</a>() provide information about a class's properties.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#constructor">constructor</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#constructorCount">constructorCount</a>() provide information about a class's meta-constructors.</li>
    /// </ul>
    /// <p>The index functions <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfConstructor">indexOfConstructor</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfMethod">indexOfMethod</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfEnumerator">indexOfEnumerator</a>(), and <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfProperty">indexOfProperty</a>() map names of constructors, member functions, enumerators, or properties to indexes in the meta-object. For example, Qt uses <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfMethod">indexOfMethod</a>() internally when you connect a signal to a slot.</p>
    /// <p>Classes can also have a list of <i>name</i>--<i>value</i> pairs of additional class information, stored in <a href="http://doc.qt.io/qt-5/qmetaclassinfo.html">QMetaClassInfo</a> objects. The number of pairs is returned by <a href="http://doc.qt.io/qt-5/qmetaobject.html#classInfoCount">classInfoCount</a>(), single pairs are returned by <a href="http://doc.qt.io/qt-5/qmetaobject.html#classInfo">classInfo</a>(), and you can search for pairs with <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfClassInfo">indexOfClassInfo</a>().</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaObject_QMetaObject();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qmetaobject.html">QMetaObject</a> class contains meta-information about Qt objects.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMetaObject::QMetaObject(const QMetaObject& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmetaobject.html">QMetaObject</a> class contains meta-information about Qt objects.</p>
    /// <p>The Qt <a href="http://doc.qt.io/qt-5/metaobjects.html">Meta-Object System</a> in Qt is responsible for the signals and slots inter-object communication mechanism, runtime type information, and the Qt property system. A single <a href="http://doc.qt.io/qt-5/qmetaobject.html">QMetaObject</a> instance is created for each <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> subclass that is used in an application, and this instance stores all the meta-information for the <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> subclass. This object is available as <a href="http://doc.qt.io/qt-5/qobject.html#metaObject">QObject::metaObject</a>().</p>
    /// <p>This class is not normally required for application programming, but it is useful if you write meta-applications, such as scripting engines or GUI builders.</p>
    /// <p>The functions you are most likely to find useful are these:</p>
    /// <ul>
    /// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#className">className</a>() returns the name of a class.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#superClass">superClass</a>() returns the superclass's meta-object.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#method">method</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#methodCount">methodCount</a>() provide information about a class's meta-methods (signals, slots and other <a href="http://doc.qt.io/qt-5/qobject.html#Q_INVOKABLE">invokable</a> member functions).</li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#enumerator">enumerator</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#enumeratorCount">enumeratorCount</a>() and provide information about a class's enumerators.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#propertyCount">propertyCount</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#property">property</a>() provide information about a class's properties.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#constructor">constructor</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#constructorCount">constructorCount</a>() provide information about a class's meta-constructors.</li>
    /// </ul>
    /// <p>The index functions <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfConstructor">indexOfConstructor</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfMethod">indexOfMethod</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfEnumerator">indexOfEnumerator</a>(), and <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfProperty">indexOfProperty</a>() map names of constructors, member functions, enumerators, or properties to indexes in the meta-object. For example, Qt uses <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfMethod">indexOfMethod</a>() internally when you connect a signal to a slot.</p>
    /// <p>Classes can also have a list of <i>name</i>--<i>value</i> pairs of additional class information, stored in <a href="http://doc.qt.io/qt-5/qmetaclassinfo.html">QMetaClassInfo</a> objects. The number of pairs is returned by <a href="http://doc.qt.io/qt-5/qmetaobject.html#classInfoCount">classInfoCount</a>(), single pairs are returned by <a href="http://doc.qt.io/qt-5/qmetaobject.html#classInfo">classInfo</a>(), and you can search for pairs with <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfClassInfo">indexOfClassInfo</a>().</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMetaObject>>,
    ) -> ::cpp_utils::CppBox<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaObject_QMetaObject1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMetaObject>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Normalizes the signature of the given <i>method</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QMetaObject::normalizedSignature(const char* method)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#normalizedSignature">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Normalizes the signature of the given <i>method</i>.</p>
    /// <p>Qt uses normalized signatures to decide whether two given signals and slots are compatible. Normalization reduces whitespace to a minimum, moves 'const' to the front where appropriate, removes 'const' from value types and replaces const references with values.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#checkConnectArgs">checkConnectArgs</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#normalizedType">normalizedType</a>().</p></div>
    pub unsafe fn normalized_signature(
        method: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaObject_normalizedSignature(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(method)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Normalizes a <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QMetaObject::normalizedType(const char* type)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#normalizedType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Normalizes a <i>type</i>.</p>
    /// <p>See <a href="http://doc.qt.io/qt-5/qmetaobject.html#normalizedSignature">QMetaObject::normalizedSignature</a>() for a description on how Qt normalizes.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span> normType <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qmetaobject.html">QMetaObject</a></span><span class="operator">::</span>normalizedType(<span class="string">" int    const  *"</span>);
    ///   <span class="comment">// normType is now "const int*"</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#normalizedSignature">normalizedSignature</a>().</p></div>
    pub unsafe fn normalized_type(
        type_: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaObject_normalizedType(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(type_)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the meta-data for the property with the given <i>index</i>. If no such property exists, a null <a href="http://doc.qt.io/qt-5/qmetaproperty.html">QMetaProperty</a> is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMetaProperty QMetaObject::property(int index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#property">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the meta-data for the property with the given <i>index</i>. If no such property exists, a null <a href="http://doc.qt.io/qt-5/qmetaproperty.html">QMetaProperty</a> is returned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#propertyCount">propertyCount</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#propertyOffset">propertyOffset</a>(), and <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfProperty">indexOfProperty</a>().</p></div>
    pub unsafe fn property(
        &self,
        index: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QMetaProperty> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaObject_property(
            self as *const crate::QMetaObject,
            index,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the number of properties in this class, including the number of properties provided by each base class.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaObject::propertyCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#propertyCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of properties in this class, including the number of properties provided by each base class.</p>
    /// <p>Use code like the following to obtain a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> containing the properties specific to a given class:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qmetaobject.html">QMetaObject</a></span><span class="operator">*</span> metaObject <span class="operator">=</span> obj<span class="operator">-</span><span class="operator">&gt;</span>metaObject();
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> properties;
    ///   <span class="keyword">for</span>(<span class="type">int</span> i <span class="operator">=</span> metaObject<span class="operator">-</span><span class="operator">&gt;</span>propertyOffset(); i <span class="operator">&lt;</span> metaObject<span class="operator">-</span><span class="operator">&gt;</span>propertyCount(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     properties <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">::</span>fromLatin1(metaObject<span class="operator">-</span><span class="operator">&gt;</span>property(i)<span class="operator">.</span>name());
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#property">property</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#propertyOffset">propertyOffset</a>(), and <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfProperty">indexOfProperty</a>().</p></div>
    pub unsafe fn property_count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_propertyCount(self as *const crate::QMetaObject)
    }

    /// <p>Returns the property offset for this class; i.e. the index position of this class's first property.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaObject::propertyOffset() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#propertyOffset">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the property offset for this class; i.e. the index position of this class's first property.</p>
    /// <p>The offset is the sum of all the properties in the class's superclasses (which is always positive since <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> has the name() property).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#property">property</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#propertyCount">propertyCount</a>(), and <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfProperty">indexOfProperty</a>().</p></div>
    pub unsafe fn property_offset(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_propertyOffset(self as *const crate::QMetaObject)
    }

    /// Calls C++ function: <span style='color: green;'>```int QMetaObject::static_metacall(QMetaObject::Call arg1, int arg2, void** arg3) const```</span>.
    pub unsafe fn static_metacall(
        &self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_static_metacall(
            self as *const crate::QMetaObject,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the meta-object of the superclass, or 0 if there is no such object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QMetaObject* QMetaObject::superClass() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#superClass">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the meta-object of the superclass, or 0 if there is no such object.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#className">className</a>().</p></div>
    pub unsafe fn super_class(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMetaObject_superClass(self as *const crate::QMetaObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QString QMetaObject::tr(const char* s, const char* c, int n = …) const```</span>.
    pub unsafe fn tr_3a(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaObject_tr(
            self as *const crate::QMetaObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QString QMetaObject::tr(const char* s, const char* c) const```</span>.
    pub unsafe fn tr_2a(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaObject_tr1(
            self as *const crate::QMetaObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the property that has the <code>USER</code> flag set to true.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMetaProperty QMetaObject::userProperty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html#userProperty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the property that has the <code>USER</code> flag set to true.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaproperty.html#isUser">QMetaProperty::isUser</a>().</p></div>
    pub unsafe fn user_property(&self) -> ::cpp_utils::CppBox<crate::QMetaProperty> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaObject_userProperty(
            self as *const crate::QMetaObject,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qlatin1char.html">QLatin1Char</a> class provides an 8-bit ASCII/Latin-1 character.</p>
///
/// C++ class: <span style='color: green;'>```QLatin1Char```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlatin1char.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlatin1char.html">QLatin1Char</a> class provides an 8-bit ASCII/Latin-1 character.</p>
/// <p>This class is only useful to construct a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> with 8-bit character.</p></div>
#[repr(C)]
pub struct QLatin1Char {
    _unused: u8,
}
impl QLatin1Char {
    /// <p>The <a href="http://doc.qt.io/qt-5/qlatin1char.html">QLatin1Char</a> class provides an 8-bit ASCII/Latin-1 character.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLatin1Char& QLatin1Char::operator=(const QLatin1Char& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1char.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlatin1char.html">QLatin1Char</a> class provides an 8-bit ASCII/Latin-1 character.</p>
    /// <p>This class is only useful to construct a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> with 8-bit character.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1Char>>,
    ) -> ::cpp_utils::MutRef<crate::QLatin1Char> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLatin1Char_operator_(
            self as *mut crate::QLatin1Char,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1Char>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Constructs a Latin-1 character for <i>c</i>. This constructor should be used when the encoding of the input character is known to be Latin-1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLatin1Char::QLatin1Char(char c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1char.html#QLatin1Char">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a Latin-1 character for <i>c</i>. This constructor should be used when the encoding of the input character is known to be Latin-1.</p></div>
    pub unsafe fn new(c: ::std::os::raw::c_char) -> ::cpp_utils::CppBox<crate::QLatin1Char> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLatin1Char_QLatin1Char(c);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlatin1char.html">QLatin1Char</a> class provides an 8-bit ASCII/Latin-1 character.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLatin1Char::QLatin1Char(const QLatin1Char& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1char.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlatin1char.html">QLatin1Char</a> class provides an 8-bit ASCII/Latin-1 character.</p>
    /// <p>This class is only useful to construct a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> with 8-bit character.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1Char>>,
    ) -> ::cpp_utils::CppBox<crate::QLatin1Char> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLatin1Char_QLatin1Char2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1Char>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts a Latin-1 character to an 8-bit ASCII representation of the character.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```char QLatin1Char::toLatin1() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1char.html#toLatin1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts a Latin-1 character to an 8-bit ASCII representation of the character.</p></div>
    pub unsafe fn to_latin1(&self) -> ::std::os::raw::c_char {
        crate::__ffi::ctr_qt_core_ffi_QLatin1Char_toLatin1(self as *const crate::QLatin1Char)
    }

    /// <p>Converts a Latin-1 character to an 16-bit-encoded Unicode representation of the character.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned short QLatin1Char::unicode() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1char.html#unicode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts a Latin-1 character to an 16-bit-encoded Unicode representation of the character.</p></div>
    pub unsafe fn unicode(&self) -> ::std::os::raw::c_ushort {
        crate::__ffi::ctr_qt_core_ffi_QLatin1Char_unicode(self as *const crate::QLatin1Char)
    }
}

pub mod q_char {
    //! C++ type: <span style='color: green;'>```QChar```</span>

    /// C++ enum: <span style='color: green;'>```QChar::SpecialCharacter```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#SpecialCharacter-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct SpecialCharacter(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for SpecialCharacter {
        fn from(value: ::std::os::raw::c_int) -> Self {
            SpecialCharacter(value)
        }
    }

    impl From<SpecialCharacter> for ::std::os::raw::c_int {
        fn from(value: SpecialCharacter) -> Self {
            value.0
        }
    }

    impl SpecialCharacter {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl SpecialCharacter {
        /// A <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> with this value <a href="http://doc.qt.io/qt-5/qchar.html#isNull">isNull</a>(). (C++ enum variant: <span style='color: green;'>```Null = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const Null: crate::q_char::SpecialCharacter = crate::q_char::SpecialCharacter(0);
        /// Character tabulation. (C++ enum variant: <span style='color: green;'>```Tabulation = 9```</span>)
        #[allow(non_upper_case_globals)]
        pub const Tabulation: crate::q_char::SpecialCharacter = crate::q_char::SpecialCharacter(9);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```LineFeed = 10```</span>)
        #[allow(non_upper_case_globals)]
        pub const LineFeed: crate::q_char::SpecialCharacter = crate::q_char::SpecialCharacter(10);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```CarriageReturn = 13```</span>)
        #[allow(non_upper_case_globals)]
        pub const CarriageReturn: crate::q_char::SpecialCharacter =
            crate::q_char::SpecialCharacter(13);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Space = 32```</span>)
        #[allow(non_upper_case_globals)]
        pub const Space: crate::q_char::SpecialCharacter = crate::q_char::SpecialCharacter(32);
        /// Non-breaking space. (C++ enum variant: <span style='color: green;'>```Nbsp = 160```</span>)
        #[allow(non_upper_case_globals)]
        pub const Nbsp: crate::q_char::SpecialCharacter = crate::q_char::SpecialCharacter(160);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SoftHyphen = 173```</span>)
        #[allow(non_upper_case_globals)]
        pub const SoftHyphen: crate::q_char::SpecialCharacter =
            crate::q_char::SpecialCharacter(173);
        /// The character shown when a font has no glyph for a certain codepoint. A special question mark character is often used. Codecs use this codepoint when input data cannot be represented in Unicode. (C++ enum variant: <span style='color: green;'>```ReplacementCharacter = 65533```</span>)
        #[allow(non_upper_case_globals)]
        pub const ReplacementCharacter: crate::q_char::SpecialCharacter =
            crate::q_char::SpecialCharacter(65533);
        /// Used to represent an object such as an image when such objects cannot be presented. (C++ enum variant: <span style='color: green;'>```ObjectReplacementCharacter = 65532```</span>)
        #[allow(non_upper_case_globals)]
        pub const ObjectReplacementCharacter: crate::q_char::SpecialCharacter =
            crate::q_char::SpecialCharacter(65532);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```ByteOrderMark = 65279```</span>)
        #[allow(non_upper_case_globals)]
        pub const ByteOrderMark: crate::q_char::SpecialCharacter =
            crate::q_char::SpecialCharacter(65279);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```ByteOrderSwapped = 65534```</span>)
        #[allow(non_upper_case_globals)]
        pub const ByteOrderSwapped: crate::q_char::SpecialCharacter =
            crate::q_char::SpecialCharacter(65534);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```ParagraphSeparator = 8233```</span>)
        #[allow(non_upper_case_globals)]
        pub const ParagraphSeparator: crate::q_char::SpecialCharacter =
            crate::q_char::SpecialCharacter(8233);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```LineSeparator = 8232```</span>)
        #[allow(non_upper_case_globals)]
        pub const LineSeparator: crate::q_char::SpecialCharacter =
            crate::q_char::SpecialCharacter(8232);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```LastValidCodePoint = 1114111```</span>)
        #[allow(non_upper_case_globals)]
        pub const LastValidCodePoint: crate::q_char::SpecialCharacter =
            crate::q_char::SpecialCharacter(1114111);
    }

    /// <p>This enum maps the Unicode character categories.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QChar::Category```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#Category-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum maps the Unicode character categories.</p>
    /// <p>The following characters are normative in Unicode:</p>
    ///
    /// <p>The following categories are informative in Unicode:</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#category">category</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Category(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Category {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Category(value)
        }
    }

    impl From<Category> for ::std::os::raw::c_int {
        fn from(value: Category) -> Self {
            value.0
        }
    }

    impl Category {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Category {
        /// Unicode class name Mn (C++ enum variant: <span style='color: green;'>```Mark_NonSpacing = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const MarkNonSpacing: crate::q_char::Category = crate::q_char::Category(0);
        /// Unicode class name Mc (C++ enum variant: <span style='color: green;'>```Mark_SpacingCombining = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const MarkSpacingCombining: crate::q_char::Category = crate::q_char::Category(1);
        /// Unicode class name Me (C++ enum variant: <span style='color: green;'>```Mark_Enclosing = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const MarkEnclosing: crate::q_char::Category = crate::q_char::Category(2);
        /// Unicode class name Nd (C++ enum variant: <span style='color: green;'>```Number_DecimalDigit = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const NumberDecimalDigit: crate::q_char::Category = crate::q_char::Category(3);
        /// Unicode class name Nl (C++ enum variant: <span style='color: green;'>```Number_Letter = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const NumberLetter: crate::q_char::Category = crate::q_char::Category(4);
        /// Unicode class name No (C++ enum variant: <span style='color: green;'>```Number_Other = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const NumberOther: crate::q_char::Category = crate::q_char::Category(5);
        /// Unicode class name Zs (C++ enum variant: <span style='color: green;'>```Separator_Space = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const SeparatorSpace: crate::q_char::Category = crate::q_char::Category(6);
        /// Unicode class name Zl (C++ enum variant: <span style='color: green;'>```Separator_Line = 7```</span>)
        #[allow(non_upper_case_globals)]
        pub const SeparatorLine: crate::q_char::Category = crate::q_char::Category(7);
        /// Unicode class name Zp (C++ enum variant: <span style='color: green;'>```Separator_Paragraph = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const SeparatorParagraph: crate::q_char::Category = crate::q_char::Category(8);
        /// Unicode class name Cc (C++ enum variant: <span style='color: green;'>```Other_Control = 9```</span>)
        #[allow(non_upper_case_globals)]
        pub const OtherControl: crate::q_char::Category = crate::q_char::Category(9);
        /// Unicode class name Cf (C++ enum variant: <span style='color: green;'>```Other_Format = 10```</span>)
        #[allow(non_upper_case_globals)]
        pub const OtherFormat: crate::q_char::Category = crate::q_char::Category(10);
        /// Unicode class name Cs (C++ enum variant: <span style='color: green;'>```Other_Surrogate = 11```</span>)
        #[allow(non_upper_case_globals)]
        pub const OtherSurrogate: crate::q_char::Category = crate::q_char::Category(11);
        /// Unicode class name Co (C++ enum variant: <span style='color: green;'>```Other_PrivateUse = 12```</span>)
        #[allow(non_upper_case_globals)]
        pub const OtherPrivateUse: crate::q_char::Category = crate::q_char::Category(12);
        /// Unicode class name Cn (C++ enum variant: <span style='color: green;'>```Other_NotAssigned = 13```</span>)
        #[allow(non_upper_case_globals)]
        pub const OtherNotAssigned: crate::q_char::Category = crate::q_char::Category(13);
        /// Unicode class name Lu (C++ enum variant: <span style='color: green;'>```Letter_Uppercase = 14```</span>)
        #[allow(non_upper_case_globals)]
        pub const LetterUppercase: crate::q_char::Category = crate::q_char::Category(14);
        /// Unicode class name Ll (C++ enum variant: <span style='color: green;'>```Letter_Lowercase = 15```</span>)
        #[allow(non_upper_case_globals)]
        pub const LetterLowercase: crate::q_char::Category = crate::q_char::Category(15);
        /// Unicode class name Lt (C++ enum variant: <span style='color: green;'>```Letter_Titlecase = 16```</span>)
        #[allow(non_upper_case_globals)]
        pub const LetterTitlecase: crate::q_char::Category = crate::q_char::Category(16);
        /// Unicode class name Lm (C++ enum variant: <span style='color: green;'>```Letter_Modifier = 17```</span>)
        #[allow(non_upper_case_globals)]
        pub const LetterModifier: crate::q_char::Category = crate::q_char::Category(17);
        /// Unicode class name Lo (C++ enum variant: <span style='color: green;'>```Letter_Other = 18```</span>)
        #[allow(non_upper_case_globals)]
        pub const LetterOther: crate::q_char::Category = crate::q_char::Category(18);
        /// Unicode class name Pc (C++ enum variant: <span style='color: green;'>```Punctuation_Connector = 19```</span>)
        #[allow(non_upper_case_globals)]
        pub const PunctuationConnector: crate::q_char::Category = crate::q_char::Category(19);
        /// Unicode class name Pd (C++ enum variant: <span style='color: green;'>```Punctuation_Dash = 20```</span>)
        #[allow(non_upper_case_globals)]
        pub const PunctuationDash: crate::q_char::Category = crate::q_char::Category(20);
        /// Unicode class name Ps (C++ enum variant: <span style='color: green;'>```Punctuation_Open = 21```</span>)
        #[allow(non_upper_case_globals)]
        pub const PunctuationOpen: crate::q_char::Category = crate::q_char::Category(21);
        /// Unicode class name Pe (C++ enum variant: <span style='color: green;'>```Punctuation_Close = 22```</span>)
        #[allow(non_upper_case_globals)]
        pub const PunctuationClose: crate::q_char::Category = crate::q_char::Category(22);
        /// Unicode class name Pi (C++ enum variant: <span style='color: green;'>```Punctuation_InitialQuote = 23```</span>)
        #[allow(non_upper_case_globals)]
        pub const PunctuationInitialQuote: crate::q_char::Category = crate::q_char::Category(23);
        /// Unicode class name Pf (C++ enum variant: <span style='color: green;'>```Punctuation_FinalQuote = 24```</span>)
        #[allow(non_upper_case_globals)]
        pub const PunctuationFinalQuote: crate::q_char::Category = crate::q_char::Category(24);
        /// Unicode class name Po (C++ enum variant: <span style='color: green;'>```Punctuation_Other = 25```</span>)
        #[allow(non_upper_case_globals)]
        pub const PunctuationOther: crate::q_char::Category = crate::q_char::Category(25);
        /// Unicode class name Sm (C++ enum variant: <span style='color: green;'>```Symbol_Math = 26```</span>)
        #[allow(non_upper_case_globals)]
        pub const SymbolMath: crate::q_char::Category = crate::q_char::Category(26);
        /// Unicode class name Sc (C++ enum variant: <span style='color: green;'>```Symbol_Currency = 27```</span>)
        #[allow(non_upper_case_globals)]
        pub const SymbolCurrency: crate::q_char::Category = crate::q_char::Category(27);
        /// Unicode class name Sk (C++ enum variant: <span style='color: green;'>```Symbol_Modifier = 28```</span>)
        #[allow(non_upper_case_globals)]
        pub const SymbolModifier: crate::q_char::Category = crate::q_char::Category(28);
        /// Unicode class name So (C++ enum variant: <span style='color: green;'>```Symbol_Other = 29```</span>)
        #[allow(non_upper_case_globals)]
        pub const SymbolOther: crate::q_char::Category = crate::q_char::Category(29);
    }

    /// <p>This enum type defines the Unicode script property values.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QChar::Script```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#Script-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type defines the Unicode script property values.</p>
    /// <p>For details about the Unicode script property values see <a href="http://www.unicode.org/reports/tr24/">Unicode Standard Annex #24</a>.</p>
    /// <p>In order to conform to C/C++ naming conventions "Script_" is prepended to the codes used in the Unicode Standard.</p>
    ///
    /// <p>This enum was introduced or modified in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#script">script</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Script(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Script {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Script(value)
        }
    }

    impl From<Script> for ::std::os::raw::c_int {
        fn from(value: Script) -> Self {
            value.0
        }
    }

    impl Script {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Script {
        /// For unassigned, private-use, noncharacter, and surrogate code points. (C++ enum variant: <span style='color: green;'>```Script_Unknown = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptUnknown: crate::q_char::Script = crate::q_char::Script(0);
        /// For characters that may be used with multiple scripts and that inherit their script from the preceding characters. These include nonspacing marks, enclosing marks, and zero width joiner/non-joiner characters. (C++ enum variant: <span style='color: green;'>```Script_Inherited = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptInherited: crate::q_char::Script = crate::q_char::Script(1);
        /// For characters that may be used with multiple scripts and that do not inherit their script from the preceding characters. (C++ enum variant: <span style='color: green;'>```Script_Common = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptCommon: crate::q_char::Script = crate::q_char::Script(2);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Latin = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptLatin: crate::q_char::Script = crate::q_char::Script(3);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Greek = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptGreek: crate::q_char::Script = crate::q_char::Script(4);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Cyrillic = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptCyrillic: crate::q_char::Script = crate::q_char::Script(5);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Armenian = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptArmenian: crate::q_char::Script = crate::q_char::Script(6);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Hebrew = 7```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptHebrew: crate::q_char::Script = crate::q_char::Script(7);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Arabic = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptArabic: crate::q_char::Script = crate::q_char::Script(8);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Syriac = 9```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptSyriac: crate::q_char::Script = crate::q_char::Script(9);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Thaana = 10```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptThaana: crate::q_char::Script = crate::q_char::Script(10);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Devanagari = 11```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptDevanagari: crate::q_char::Script = crate::q_char::Script(11);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Bengali = 12```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptBengali: crate::q_char::Script = crate::q_char::Script(12);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Gurmukhi = 13```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptGurmukhi: crate::q_char::Script = crate::q_char::Script(13);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Gujarati = 14```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptGujarati: crate::q_char::Script = crate::q_char::Script(14);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Oriya = 15```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptOriya: crate::q_char::Script = crate::q_char::Script(15);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Tamil = 16```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptTamil: crate::q_char::Script = crate::q_char::Script(16);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Telugu = 17```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptTelugu: crate::q_char::Script = crate::q_char::Script(17);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Kannada = 18```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptKannada: crate::q_char::Script = crate::q_char::Script(18);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Malayalam = 19```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptMalayalam: crate::q_char::Script = crate::q_char::Script(19);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Sinhala = 20```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptSinhala: crate::q_char::Script = crate::q_char::Script(20);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Thai = 21```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptThai: crate::q_char::Script = crate::q_char::Script(21);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Lao = 22```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptLao: crate::q_char::Script = crate::q_char::Script(22);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Tibetan = 23```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptTibetan: crate::q_char::Script = crate::q_char::Script(23);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Myanmar = 24```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptMyanmar: crate::q_char::Script = crate::q_char::Script(24);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Georgian = 25```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptGeorgian: crate::q_char::Script = crate::q_char::Script(25);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Hangul = 26```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptHangul: crate::q_char::Script = crate::q_char::Script(26);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Ethiopic = 27```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptEthiopic: crate::q_char::Script = crate::q_char::Script(27);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Cherokee = 28```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptCherokee: crate::q_char::Script = crate::q_char::Script(28);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_CanadianAboriginal = 29```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptCanadianAboriginal: crate::q_char::Script = crate::q_char::Script(29);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Ogham = 30```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptOgham: crate::q_char::Script = crate::q_char::Script(30);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Runic = 31```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptRunic: crate::q_char::Script = crate::q_char::Script(31);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Khmer = 32```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptKhmer: crate::q_char::Script = crate::q_char::Script(32);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Mongolian = 33```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptMongolian: crate::q_char::Script = crate::q_char::Script(33);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Hiragana = 34```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptHiragana: crate::q_char::Script = crate::q_char::Script(34);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Katakana = 35```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptKatakana: crate::q_char::Script = crate::q_char::Script(35);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Bopomofo = 36```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptBopomofo: crate::q_char::Script = crate::q_char::Script(36);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Han = 37```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptHan: crate::q_char::Script = crate::q_char::Script(37);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Yi = 38```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptYi: crate::q_char::Script = crate::q_char::Script(38);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_OldItalic = 39```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptOldItalic: crate::q_char::Script = crate::q_char::Script(39);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Gothic = 40```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptGothic: crate::q_char::Script = crate::q_char::Script(40);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Deseret = 41```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptDeseret: crate::q_char::Script = crate::q_char::Script(41);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Tagalog = 42```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptTagalog: crate::q_char::Script = crate::q_char::Script(42);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Hanunoo = 43```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptHanunoo: crate::q_char::Script = crate::q_char::Script(43);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Buhid = 44```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptBuhid: crate::q_char::Script = crate::q_char::Script(44);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Tagbanwa = 45```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptTagbanwa: crate::q_char::Script = crate::q_char::Script(45);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Coptic = 46```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptCoptic: crate::q_char::Script = crate::q_char::Script(46);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Limbu = 47```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptLimbu: crate::q_char::Script = crate::q_char::Script(47);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_TaiLe = 48```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptTaiLe: crate::q_char::Script = crate::q_char::Script(48);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_LinearB = 49```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptLinearB: crate::q_char::Script = crate::q_char::Script(49);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Ugaritic = 50```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptUgaritic: crate::q_char::Script = crate::q_char::Script(50);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Shavian = 51```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptShavian: crate::q_char::Script = crate::q_char::Script(51);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Osmanya = 52```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptOsmanya: crate::q_char::Script = crate::q_char::Script(52);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Cypriot = 53```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptCypriot: crate::q_char::Script = crate::q_char::Script(53);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Braille = 54```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptBraille: crate::q_char::Script = crate::q_char::Script(54);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Buginese = 55```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptBuginese: crate::q_char::Script = crate::q_char::Script(55);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_NewTaiLue = 56```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptNewTaiLue: crate::q_char::Script = crate::q_char::Script(56);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Glagolitic = 57```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptGlagolitic: crate::q_char::Script = crate::q_char::Script(57);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Tifinagh = 58```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptTifinagh: crate::q_char::Script = crate::q_char::Script(58);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_SylotiNagri = 59```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptSylotiNagri: crate::q_char::Script = crate::q_char::Script(59);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_OldPersian = 60```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptOldPersian: crate::q_char::Script = crate::q_char::Script(60);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Kharoshthi = 61```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptKharoshthi: crate::q_char::Script = crate::q_char::Script(61);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Balinese = 62```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptBalinese: crate::q_char::Script = crate::q_char::Script(62);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Cuneiform = 63```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptCuneiform: crate::q_char::Script = crate::q_char::Script(63);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Phoenician = 64```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptPhoenician: crate::q_char::Script = crate::q_char::Script(64);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_PhagsPa = 65```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptPhagsPa: crate::q_char::Script = crate::q_char::Script(65);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Nko = 66```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptNko: crate::q_char::Script = crate::q_char::Script(66);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Sundanese = 67```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptSundanese: crate::q_char::Script = crate::q_char::Script(67);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Lepcha = 68```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptLepcha: crate::q_char::Script = crate::q_char::Script(68);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_OlChiki = 69```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptOlChiki: crate::q_char::Script = crate::q_char::Script(69);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Vai = 70```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptVai: crate::q_char::Script = crate::q_char::Script(70);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Saurashtra = 71```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptSaurashtra: crate::q_char::Script = crate::q_char::Script(71);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_KayahLi = 72```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptKayahLi: crate::q_char::Script = crate::q_char::Script(72);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Rejang = 73```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptRejang: crate::q_char::Script = crate::q_char::Script(73);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Lycian = 74```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptLycian: crate::q_char::Script = crate::q_char::Script(74);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Carian = 75```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptCarian: crate::q_char::Script = crate::q_char::Script(75);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Lydian = 76```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptLydian: crate::q_char::Script = crate::q_char::Script(76);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Cham = 77```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptCham: crate::q_char::Script = crate::q_char::Script(77);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_TaiTham = 78```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptTaiTham: crate::q_char::Script = crate::q_char::Script(78);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_TaiViet = 79```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptTaiViet: crate::q_char::Script = crate::q_char::Script(79);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Avestan = 80```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptAvestan: crate::q_char::Script = crate::q_char::Script(80);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_EgyptianHieroglyphs = 81```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptEgyptianHieroglyphs: crate::q_char::Script = crate::q_char::Script(81);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Samaritan = 82```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptSamaritan: crate::q_char::Script = crate::q_char::Script(82);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Lisu = 83```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptLisu: crate::q_char::Script = crate::q_char::Script(83);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Bamum = 84```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptBamum: crate::q_char::Script = crate::q_char::Script(84);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Javanese = 85```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptJavanese: crate::q_char::Script = crate::q_char::Script(85);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_MeeteiMayek = 86```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptMeeteiMayek: crate::q_char::Script = crate::q_char::Script(86);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_ImperialAramaic = 87```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptImperialAramaic: crate::q_char::Script = crate::q_char::Script(87);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_OldSouthArabian = 88```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptOldSouthArabian: crate::q_char::Script = crate::q_char::Script(88);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_InscriptionalParthian = 89```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptInscriptionalParthian: crate::q_char::Script = crate::q_char::Script(89);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_InscriptionalPahlavi = 90```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptInscriptionalPahlavi: crate::q_char::Script = crate::q_char::Script(90);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_OldTurkic = 91```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptOldTurkic: crate::q_char::Script = crate::q_char::Script(91);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Kaithi = 92```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptKaithi: crate::q_char::Script = crate::q_char::Script(92);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Batak = 93```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptBatak: crate::q_char::Script = crate::q_char::Script(93);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Brahmi = 94```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptBrahmi: crate::q_char::Script = crate::q_char::Script(94);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Mandaic = 95```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptMandaic: crate::q_char::Script = crate::q_char::Script(95);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Chakma = 96```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptChakma: crate::q_char::Script = crate::q_char::Script(96);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_MeroiticCursive = 97```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptMeroiticCursive: crate::q_char::Script = crate::q_char::Script(97);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_MeroiticHieroglyphs = 98```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptMeroiticHieroglyphs: crate::q_char::Script = crate::q_char::Script(98);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Miao = 99```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptMiao: crate::q_char::Script = crate::q_char::Script(99);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Sharada = 100```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptSharada: crate::q_char::Script = crate::q_char::Script(100);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_SoraSompeng = 101```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptSoraSompeng: crate::q_char::Script = crate::q_char::Script(101);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Takri = 102```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptTakri: crate::q_char::Script = crate::q_char::Script(102);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_CaucasianAlbanian = 103```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptCaucasianAlbanian: crate::q_char::Script = crate::q_char::Script(103);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_BassaVah = 104```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptBassaVah: crate::q_char::Script = crate::q_char::Script(104);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Duployan = 105```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptDuployan: crate::q_char::Script = crate::q_char::Script(105);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Elbasan = 106```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptElbasan: crate::q_char::Script = crate::q_char::Script(106);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Grantha = 107```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptGrantha: crate::q_char::Script = crate::q_char::Script(107);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_PahawhHmong = 108```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptPahawhHmong: crate::q_char::Script = crate::q_char::Script(108);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Khojki = 109```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptKhojki: crate::q_char::Script = crate::q_char::Script(109);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_LinearA = 110```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptLinearA: crate::q_char::Script = crate::q_char::Script(110);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Mahajani = 111```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptMahajani: crate::q_char::Script = crate::q_char::Script(111);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Manichaean = 112```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptManichaean: crate::q_char::Script = crate::q_char::Script(112);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_MendeKikakui = 113```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptMendeKikakui: crate::q_char::Script = crate::q_char::Script(113);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Modi = 114```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptModi: crate::q_char::Script = crate::q_char::Script(114);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Mro = 115```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptMro: crate::q_char::Script = crate::q_char::Script(115);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_OldNorthArabian = 116```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptOldNorthArabian: crate::q_char::Script = crate::q_char::Script(116);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Nabataean = 117```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptNabataean: crate::q_char::Script = crate::q_char::Script(117);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Palmyrene = 118```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptPalmyrene: crate::q_char::Script = crate::q_char::Script(118);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_PauCinHau = 119```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptPauCinHau: crate::q_char::Script = crate::q_char::Script(119);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_OldPermic = 120```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptOldPermic: crate::q_char::Script = crate::q_char::Script(120);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_PsalterPahlavi = 121```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptPsalterPahlavi: crate::q_char::Script = crate::q_char::Script(121);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Siddham = 122```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptSiddham: crate::q_char::Script = crate::q_char::Script(122);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Khudawadi = 123```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptKhudawadi: crate::q_char::Script = crate::q_char::Script(123);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Tirhuta = 124```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptTirhuta: crate::q_char::Script = crate::q_char::Script(124);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_WarangCiti = 125```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptWarangCiti: crate::q_char::Script = crate::q_char::Script(125);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Ahom = 126```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptAhom: crate::q_char::Script = crate::q_char::Script(126);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_AnatolianHieroglyphs = 127```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptAnatolianHieroglyphs: crate::q_char::Script = crate::q_char::Script(127);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Hatran = 128```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptHatran: crate::q_char::Script = crate::q_char::Script(128);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Multani = 129```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptMultani: crate::q_char::Script = crate::q_char::Script(129);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_OldHungarian = 130```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptOldHungarian: crate::q_char::Script = crate::q_char::Script(130);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_SignWriting = 131```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptSignWriting: crate::q_char::Script = crate::q_char::Script(131);
        /// C++ enum variant: <span style='color: green;'>```ScriptCount = 132```</span>
        #[allow(non_upper_case_globals)]
        pub const ScriptCount: crate::q_char::Script = crate::q_char::Script(132);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Adlam = 132```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptAdlam: crate::q_char::Script = crate::q_char::Script(132);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Bhaiksuki = 133```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptBhaiksuki: crate::q_char::Script = crate::q_char::Script(133);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Marchen = 134```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptMarchen: crate::q_char::Script = crate::q_char::Script(134);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Newa = 135```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptNewa: crate::q_char::Script = crate::q_char::Script(135);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Osage = 136```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptOsage: crate::q_char::Script = crate::q_char::Script(136);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Tangut = 137```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptTangut: crate::q_char::Script = crate::q_char::Script(137);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_MasaramGondi = 138```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptMasaramGondi: crate::q_char::Script = crate::q_char::Script(138);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Nushu = 139```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptNushu: crate::q_char::Script = crate::q_char::Script(139);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_Soyombo = 140```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptSoyombo: crate::q_char::Script = crate::q_char::Script(140);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Script_ZanabazarSquare = 141```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScriptZanabazarSquare: crate::q_char::Script = crate::q_char::Script(141);
        /// C++ enum variant: <span style='color: green;'>```ScriptCount = 142```</span>
        #[allow(non_upper_case_globals)]
        pub const ScriptCount2: crate::q_char::Script = crate::q_char::Script(142);
    }

    /// <p>This enum type defines the Unicode direction attributes. See the <a href="http://www.unicode.org/reports/tr9/tr9-35.html#Table_Bidirectional_Character_Types">Unicode Standard</a> for a description of the values.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QChar::Direction```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#Direction-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type defines the Unicode direction attributes. See the <a href="http://www.unicode.org/reports/tr9/tr9-35.html#Table_Bidirectional_Character_Types">Unicode Standard</a> for a description of the values.</p>
    /// <p>In order to conform to C/C++ naming conventions "Dir" is prepended to the codes used in the Unicode Standard.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#direction">direction</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Direction(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Direction {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Direction(value)
        }
    }

    impl From<Direction> for ::std::os::raw::c_int {
        fn from(value: Direction) -> Self {
            value.0
        }
    }

    impl Direction {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Direction {
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```DirL = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const DirL: crate::q_char::Direction = crate::q_char::Direction(0);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```DirR = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const DirR: crate::q_char::Direction = crate::q_char::Direction(1);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```DirEN = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const DirEN: crate::q_char::Direction = crate::q_char::Direction(2);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```DirES = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const DirES: crate::q_char::Direction = crate::q_char::Direction(3);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```DirET = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const DirET: crate::q_char::Direction = crate::q_char::Direction(4);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```DirAN = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const DirAN: crate::q_char::Direction = crate::q_char::Direction(5);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```DirCS = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const DirCS: crate::q_char::Direction = crate::q_char::Direction(6);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```DirB = 7```</span>)
        #[allow(non_upper_case_globals)]
        pub const DirB: crate::q_char::Direction = crate::q_char::Direction(7);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```DirS = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const DirS: crate::q_char::Direction = crate::q_char::Direction(8);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```DirWS = 9```</span>)
        #[allow(non_upper_case_globals)]
        pub const DirWS: crate::q_char::Direction = crate::q_char::Direction(9);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```DirON = 10```</span>)
        #[allow(non_upper_case_globals)]
        pub const DirON: crate::q_char::Direction = crate::q_char::Direction(10);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```DirLRE = 11```</span>)
        #[allow(non_upper_case_globals)]
        pub const DirLRE: crate::q_char::Direction = crate::q_char::Direction(11);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```DirLRO = 12```</span>)
        #[allow(non_upper_case_globals)]
        pub const DirLRO: crate::q_char::Direction = crate::q_char::Direction(12);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```DirAL = 13```</span>)
        #[allow(non_upper_case_globals)]
        pub const DirAL: crate::q_char::Direction = crate::q_char::Direction(13);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```DirRLE = 14```</span>)
        #[allow(non_upper_case_globals)]
        pub const DirRLE: crate::q_char::Direction = crate::q_char::Direction(14);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```DirRLO = 15```</span>)
        #[allow(non_upper_case_globals)]
        pub const DirRLO: crate::q_char::Direction = crate::q_char::Direction(15);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```DirPDF = 16```</span>)
        #[allow(non_upper_case_globals)]
        pub const DirPDF: crate::q_char::Direction = crate::q_char::Direction(16);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```DirNSM = 17```</span>)
        #[allow(non_upper_case_globals)]
        pub const DirNSM: crate::q_char::Direction = crate::q_char::Direction(17);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```DirBN = 18```</span>)
        #[allow(non_upper_case_globals)]
        pub const DirBN: crate::q_char::Direction = crate::q_char::Direction(18);
        /// Since Qt 5.3 (C++ enum variant: <span style='color: green;'>```DirLRI = 19```</span>)
        #[allow(non_upper_case_globals)]
        pub const DirLRI: crate::q_char::Direction = crate::q_char::Direction(19);
        /// Since Qt 5.3 (C++ enum variant: <span style='color: green;'>```DirRLI = 20```</span>)
        #[allow(non_upper_case_globals)]
        pub const DirRLI: crate::q_char::Direction = crate::q_char::Direction(20);
        /// Since Qt 5.3 (C++ enum variant: <span style='color: green;'>```DirFSI = 21```</span>)
        #[allow(non_upper_case_globals)]
        pub const DirFSI: crate::q_char::Direction = crate::q_char::Direction(21);
        /// Since Qt 5.3 (C++ enum variant: <span style='color: green;'>```DirPDI = 22```</span>)
        #[allow(non_upper_case_globals)]
        pub const DirPDI: crate::q_char::Direction = crate::q_char::Direction(22);
    }

    /// <p>This enum type defines the Unicode decomposition attributes. See the <a href="http://www.unicode.org/">Unicode Standard</a> for a description of the values.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QChar::Decomposition```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#Decomposition-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type defines the Unicode decomposition attributes. See the <a href="http://www.unicode.org/">Unicode Standard</a> for a description of the values.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#decomposition">decomposition</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Decomposition(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Decomposition {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Decomposition(value)
        }
    }

    impl From<Decomposition> for ::std::os::raw::c_int {
        fn from(value: Decomposition) -> Self {
            value.0
        }
    }

    impl Decomposition {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Decomposition {
        /// C++ enum variant: <span style='color: green;'>```NoDecomposition = 0```</span>
        #[allow(non_upper_case_globals)]
        pub const NoDecomposition: crate::q_char::Decomposition = crate::q_char::Decomposition(0);
        /// C++ enum variant: <span style='color: green;'>```Canonical = 1```</span>
        #[allow(non_upper_case_globals)]
        pub const Canonical: crate::q_char::Decomposition = crate::q_char::Decomposition(1);
        /// C++ enum variant: <span style='color: green;'>```Font = 2```</span>
        #[allow(non_upper_case_globals)]
        pub const Font: crate::q_char::Decomposition = crate::q_char::Decomposition(2);
        /// C++ enum variant: <span style='color: green;'>```NoBreak = 3```</span>
        #[allow(non_upper_case_globals)]
        pub const NoBreak: crate::q_char::Decomposition = crate::q_char::Decomposition(3);
        /// C++ enum variant: <span style='color: green;'>```Initial = 4```</span>
        #[allow(non_upper_case_globals)]
        pub const Initial: crate::q_char::Decomposition = crate::q_char::Decomposition(4);
        /// C++ enum variant: <span style='color: green;'>```Medial = 5```</span>
        #[allow(non_upper_case_globals)]
        pub const Medial: crate::q_char::Decomposition = crate::q_char::Decomposition(5);
        /// C++ enum variant: <span style='color: green;'>```Final = 6```</span>
        #[allow(non_upper_case_globals)]
        pub const Final: crate::q_char::Decomposition = crate::q_char::Decomposition(6);
        /// C++ enum variant: <span style='color: green;'>```Isolated = 7```</span>
        #[allow(non_upper_case_globals)]
        pub const Isolated: crate::q_char::Decomposition = crate::q_char::Decomposition(7);
        /// C++ enum variant: <span style='color: green;'>```Circle = 8```</span>
        #[allow(non_upper_case_globals)]
        pub const Circle: crate::q_char::Decomposition = crate::q_char::Decomposition(8);
        /// C++ enum variant: <span style='color: green;'>```Super = 9```</span>
        #[allow(non_upper_case_globals)]
        pub const Super: crate::q_char::Decomposition = crate::q_char::Decomposition(9);
        /// C++ enum variant: <span style='color: green;'>```Sub = 10```</span>
        #[allow(non_upper_case_globals)]
        pub const Sub: crate::q_char::Decomposition = crate::q_char::Decomposition(10);
        /// C++ enum variant: <span style='color: green;'>```Vertical = 11```</span>
        #[allow(non_upper_case_globals)]
        pub const Vertical: crate::q_char::Decomposition = crate::q_char::Decomposition(11);
        /// C++ enum variant: <span style='color: green;'>```Wide = 12```</span>
        #[allow(non_upper_case_globals)]
        pub const Wide: crate::q_char::Decomposition = crate::q_char::Decomposition(12);
        /// C++ enum variant: <span style='color: green;'>```Narrow = 13```</span>
        #[allow(non_upper_case_globals)]
        pub const Narrow: crate::q_char::Decomposition = crate::q_char::Decomposition(13);
        /// C++ enum variant: <span style='color: green;'>```Small = 14```</span>
        #[allow(non_upper_case_globals)]
        pub const Small: crate::q_char::Decomposition = crate::q_char::Decomposition(14);
        /// C++ enum variant: <span style='color: green;'>```Square = 15```</span>
        #[allow(non_upper_case_globals)]
        pub const Square: crate::q_char::Decomposition = crate::q_char::Decomposition(15);
        /// C++ enum variant: <span style='color: green;'>```Compat = 16```</span>
        #[allow(non_upper_case_globals)]
        pub const Compat: crate::q_char::Decomposition = crate::q_char::Decomposition(16);
        /// C++ enum variant: <span style='color: green;'>```Fraction = 17```</span>
        #[allow(non_upper_case_globals)]
        pub const Fraction: crate::q_char::Decomposition = crate::q_char::Decomposition(17);
    }

    /// <p>since 5.3</p>
    ///
    /// C++ enum: <span style='color: green;'>```QChar::JoiningType```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#JoiningType-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>since 5.3</p>
    /// <p>This enum type defines the Unicode joining type attributes. See the <a href="http://www.unicode.org/">Unicode Standard</a> for a description of the values.</p>
    /// <p>In order to conform to C/C++ naming conventions "Joining_" is prepended to the codes used in the Unicode Standard.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#joiningType">joiningType</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct JoiningType(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for JoiningType {
        fn from(value: ::std::os::raw::c_int) -> Self {
            JoiningType(value)
        }
    }

    impl From<JoiningType> for ::std::os::raw::c_int {
        fn from(value: JoiningType) -> Self {
            value.0
        }
    }

    impl JoiningType {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl JoiningType {
        /// C++ enum variant: <span style='color: green;'>```Joining_None = 0```</span>
        #[allow(non_upper_case_globals)]
        pub const JoiningNone: crate::q_char::JoiningType = crate::q_char::JoiningType(0);
        /// C++ enum variant: <span style='color: green;'>```Joining_Causing = 1```</span>
        #[allow(non_upper_case_globals)]
        pub const JoiningCausing: crate::q_char::JoiningType = crate::q_char::JoiningType(1);
        /// C++ enum variant: <span style='color: green;'>```Joining_Dual = 2```</span>
        #[allow(non_upper_case_globals)]
        pub const JoiningDual: crate::q_char::JoiningType = crate::q_char::JoiningType(2);
        /// C++ enum variant: <span style='color: green;'>```Joining_Right = 3```</span>
        #[allow(non_upper_case_globals)]
        pub const JoiningRight: crate::q_char::JoiningType = crate::q_char::JoiningType(3);
        /// C++ enum variant: <span style='color: green;'>```Joining_Left = 4```</span>
        #[allow(non_upper_case_globals)]
        pub const JoiningLeft: crate::q_char::JoiningType = crate::q_char::JoiningType(4);
        /// C++ enum variant: <span style='color: green;'>```Joining_Transparent = 5```</span>
        #[allow(non_upper_case_globals)]
        pub const JoiningTransparent: crate::q_char::JoiningType = crate::q_char::JoiningType(5);
    }

    /// <p>This enum type defines the Unicode joining attributes. See the <a href="http://www.unicode.org/">Unicode Standard</a> for a description of the values.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QChar::Joining```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar-obsolete.html#Joining-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type defines the Unicode joining attributes. See the <a href="http://www.unicode.org/">Unicode Standard</a> for a description of the values.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar-obsolete.html#joining">joining</a>().</p>
    ///
    /// <h2>Member Function Documentation</h2></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Joining(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Joining {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Joining(value)
        }
    }

    impl From<Joining> for ::std::os::raw::c_int {
        fn from(value: Joining) -> Self {
            value.0
        }
    }

    impl Joining {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Joining {
        /// C++ enum variant: <span style='color: green;'>```OtherJoining = 0```</span>
        #[allow(non_upper_case_globals)]
        pub const OtherJoining: crate::q_char::Joining = crate::q_char::Joining(0);
        /// C++ enum variant: <span style='color: green;'>```Dual = 1```</span>
        #[allow(non_upper_case_globals)]
        pub const Dual: crate::q_char::Joining = crate::q_char::Joining(1);
        /// C++ enum variant: <span style='color: green;'>```Right = 2```</span>
        #[allow(non_upper_case_globals)]
        pub const Right: crate::q_char::Joining = crate::q_char::Joining(2);
        /// C++ enum variant: <span style='color: green;'>```Center = 3```</span>
        #[allow(non_upper_case_globals)]
        pub const Center: crate::q_char::Joining = crate::q_char::Joining(3);
    }

    /// C++ enum: <span style='color: green;'>```QChar::CombiningClass```</span>.
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct CombiningClass(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for CombiningClass {
        fn from(value: ::std::os::raw::c_int) -> Self {
            CombiningClass(value)
        }
    }

    impl From<CombiningClass> for ::std::os::raw::c_int {
        fn from(value: CombiningClass) -> Self {
            value.0
        }
    }

    impl CombiningClass {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl CombiningClass {
        /// C++ enum variant: <span style='color: green;'>```Combining_BelowLeftAttached = 200```</span>
        #[allow(non_upper_case_globals)]
        pub const CombiningBelowLeftAttached: crate::q_char::CombiningClass =
            crate::q_char::CombiningClass(200);
        /// C++ enum variant: <span style='color: green;'>```Combining_BelowAttached = 202```</span>
        #[allow(non_upper_case_globals)]
        pub const CombiningBelowAttached: crate::q_char::CombiningClass =
            crate::q_char::CombiningClass(202);
        /// C++ enum variant: <span style='color: green;'>```Combining_BelowRightAttached = 204```</span>
        #[allow(non_upper_case_globals)]
        pub const CombiningBelowRightAttached: crate::q_char::CombiningClass =
            crate::q_char::CombiningClass(204);
        /// C++ enum variant: <span style='color: green;'>```Combining_LeftAttached = 208```</span>
        #[allow(non_upper_case_globals)]
        pub const CombiningLeftAttached: crate::q_char::CombiningClass =
            crate::q_char::CombiningClass(208);
        /// C++ enum variant: <span style='color: green;'>```Combining_RightAttached = 210```</span>
        #[allow(non_upper_case_globals)]
        pub const CombiningRightAttached: crate::q_char::CombiningClass =
            crate::q_char::CombiningClass(210);
        /// C++ enum variant: <span style='color: green;'>```Combining_AboveLeftAttached = 212```</span>
        #[allow(non_upper_case_globals)]
        pub const CombiningAboveLeftAttached: crate::q_char::CombiningClass =
            crate::q_char::CombiningClass(212);
        /// C++ enum variant: <span style='color: green;'>```Combining_AboveAttached = 214```</span>
        #[allow(non_upper_case_globals)]
        pub const CombiningAboveAttached: crate::q_char::CombiningClass =
            crate::q_char::CombiningClass(214);
        /// C++ enum variant: <span style='color: green;'>```Combining_AboveRightAttached = 216```</span>
        #[allow(non_upper_case_globals)]
        pub const CombiningAboveRightAttached: crate::q_char::CombiningClass =
            crate::q_char::CombiningClass(216);
        /// C++ enum variant: <span style='color: green;'>```Combining_BelowLeft = 218```</span>
        #[allow(non_upper_case_globals)]
        pub const CombiningBelowLeft: crate::q_char::CombiningClass =
            crate::q_char::CombiningClass(218);
        /// C++ enum variant: <span style='color: green;'>```Combining_Below = 220```</span>
        #[allow(non_upper_case_globals)]
        pub const CombiningBelow: crate::q_char::CombiningClass =
            crate::q_char::CombiningClass(220);
        /// C++ enum variant: <span style='color: green;'>```Combining_BelowRight = 222```</span>
        #[allow(non_upper_case_globals)]
        pub const CombiningBelowRight: crate::q_char::CombiningClass =
            crate::q_char::CombiningClass(222);
        /// C++ enum variant: <span style='color: green;'>```Combining_Left = 224```</span>
        #[allow(non_upper_case_globals)]
        pub const CombiningLeft: crate::q_char::CombiningClass = crate::q_char::CombiningClass(224);
        /// C++ enum variant: <span style='color: green;'>```Combining_Right = 226```</span>
        #[allow(non_upper_case_globals)]
        pub const CombiningRight: crate::q_char::CombiningClass =
            crate::q_char::CombiningClass(226);
        /// C++ enum variant: <span style='color: green;'>```Combining_AboveLeft = 228```</span>
        #[allow(non_upper_case_globals)]
        pub const CombiningAboveLeft: crate::q_char::CombiningClass =
            crate::q_char::CombiningClass(228);
        /// C++ enum variant: <span style='color: green;'>```Combining_Above = 230```</span>
        #[allow(non_upper_case_globals)]
        pub const CombiningAbove: crate::q_char::CombiningClass =
            crate::q_char::CombiningClass(230);
        /// C++ enum variant: <span style='color: green;'>```Combining_AboveRight = 232```</span>
        #[allow(non_upper_case_globals)]
        pub const CombiningAboveRight: crate::q_char::CombiningClass =
            crate::q_char::CombiningClass(232);
        /// C++ enum variant: <span style='color: green;'>```Combining_DoubleBelow = 233```</span>
        #[allow(non_upper_case_globals)]
        pub const CombiningDoubleBelow: crate::q_char::CombiningClass =
            crate::q_char::CombiningClass(233);
        /// C++ enum variant: <span style='color: green;'>```Combining_DoubleAbove = 234```</span>
        #[allow(non_upper_case_globals)]
        pub const CombiningDoubleAbove: crate::q_char::CombiningClass =
            crate::q_char::CombiningClass(234);
        /// C++ enum variant: <span style='color: green;'>```Combining_IotaSubscript = 240```</span>
        #[allow(non_upper_case_globals)]
        pub const CombiningIotaSubscript: crate::q_char::CombiningClass =
            crate::q_char::CombiningClass(240);
    }

    /// <p>Specifies which version of the <a href="http://www.unicode.org/">Unicode standard</a> introduced a certain character.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QChar::UnicodeVersion```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#UnicodeVersion-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Specifies which version of the <a href="http://www.unicode.org/">Unicode standard</a> introduced a certain character.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#unicodeVersion">unicodeVersion</a>() and <a href="http://doc.qt.io/qt-5/qchar.html#currentUnicodeVersion">currentUnicodeVersion</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct UnicodeVersion(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for UnicodeVersion {
        fn from(value: ::std::os::raw::c_int) -> Self {
            UnicodeVersion(value)
        }
    }

    impl From<UnicodeVersion> for ::std::os::raw::c_int {
        fn from(value: UnicodeVersion) -> Self {
            value.0
        }
    }

    impl UnicodeVersion {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl UnicodeVersion {
        /// The value is not assigned to any character in version 8.0 of Unicode. (C++ enum variant: <span style='color: green;'>```Unicode_Unassigned = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const UnicodeUnassigned: crate::q_char::UnicodeVersion =
            crate::q_char::UnicodeVersion(0);
        /// Version 1.1 (C++ enum variant: <span style='color: green;'>```Unicode_1_1 = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Unicode11: crate::q_char::UnicodeVersion = crate::q_char::UnicodeVersion(1);
        /// Version 2.0 (C++ enum variant: <span style='color: green;'>```Unicode_2_0 = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const Unicode20: crate::q_char::UnicodeVersion = crate::q_char::UnicodeVersion(2);
        /// Version 2.1.2 (C++ enum variant: <span style='color: green;'>```Unicode_2_1_2 = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const Unicode212: crate::q_char::UnicodeVersion = crate::q_char::UnicodeVersion(3);
        /// Version 3.0 (C++ enum variant: <span style='color: green;'>```Unicode_3_0 = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const Unicode30: crate::q_char::UnicodeVersion = crate::q_char::UnicodeVersion(4);
        /// Version 3.1 (C++ enum variant: <span style='color: green;'>```Unicode_3_1 = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const Unicode31: crate::q_char::UnicodeVersion = crate::q_char::UnicodeVersion(5);
        /// Version 3.2 (C++ enum variant: <span style='color: green;'>```Unicode_3_2 = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const Unicode32: crate::q_char::UnicodeVersion = crate::q_char::UnicodeVersion(6);
        /// Version 4.0 (C++ enum variant: <span style='color: green;'>```Unicode_4_0 = 7```</span>)
        #[allow(non_upper_case_globals)]
        pub const Unicode40: crate::q_char::UnicodeVersion = crate::q_char::UnicodeVersion(7);
        /// Version 4.1 (C++ enum variant: <span style='color: green;'>```Unicode_4_1 = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const Unicode41: crate::q_char::UnicodeVersion = crate::q_char::UnicodeVersion(8);
        /// Version 5.0 (C++ enum variant: <span style='color: green;'>```Unicode_5_0 = 9```</span>)
        #[allow(non_upper_case_globals)]
        pub const Unicode50: crate::q_char::UnicodeVersion = crate::q_char::UnicodeVersion(9);
        /// Version 5.1 (C++ enum variant: <span style='color: green;'>```Unicode_5_1 = 10```</span>)
        #[allow(non_upper_case_globals)]
        pub const Unicode51: crate::q_char::UnicodeVersion = crate::q_char::UnicodeVersion(10);
        /// Version 5.2 (C++ enum variant: <span style='color: green;'>```Unicode_5_2 = 11```</span>)
        #[allow(non_upper_case_globals)]
        pub const Unicode52: crate::q_char::UnicodeVersion = crate::q_char::UnicodeVersion(11);
        /// Version 6.0 (C++ enum variant: <span style='color: green;'>```Unicode_6_0 = 12```</span>)
        #[allow(non_upper_case_globals)]
        pub const Unicode60: crate::q_char::UnicodeVersion = crate::q_char::UnicodeVersion(12);
        /// Version 6.1 (C++ enum variant: <span style='color: green;'>```Unicode_6_1 = 13```</span>)
        #[allow(non_upper_case_globals)]
        pub const Unicode61: crate::q_char::UnicodeVersion = crate::q_char::UnicodeVersion(13);
        /// Version 6.2 (C++ enum variant: <span style='color: green;'>```Unicode_6_2 = 14```</span>)
        #[allow(non_upper_case_globals)]
        pub const Unicode62: crate::q_char::UnicodeVersion = crate::q_char::UnicodeVersion(14);
        /// Version 6.3 Since Qt 5.3 (C++ enum variant: <span style='color: green;'>```Unicode_6_3 = 15```</span>)
        #[allow(non_upper_case_globals)]
        pub const Unicode63: crate::q_char::UnicodeVersion = crate::q_char::UnicodeVersion(15);
        /// Version 7.0 Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```Unicode_7_0 = 16```</span>)
        #[allow(non_upper_case_globals)]
        pub const Unicode70: crate::q_char::UnicodeVersion = crate::q_char::UnicodeVersion(16);
        /// Version 8.0 Since Qt 5.6 (C++ enum variant: <span style='color: green;'>```Unicode_8_0 = 17```</span>)
        #[allow(non_upper_case_globals)]
        pub const Unicode80: crate::q_char::UnicodeVersion = crate::q_char::UnicodeVersion(17);
        /// Version 9.0 Since Qt 5.11 (C++ enum variant: <span style='color: green;'>```Unicode_9_0 = 18```</span>)
        #[allow(non_upper_case_globals)]
        pub const Unicode90: crate::q_char::UnicodeVersion = crate::q_char::UnicodeVersion(18);
        /// Version 10.0 Since Qt 5.11 (C++ enum variant: <span style='color: green;'>```Unicode_10_0 = 19```</span>)
        #[allow(non_upper_case_globals)]
        pub const Unicode100: crate::q_char::UnicodeVersion = crate::q_char::UnicodeVersion(19);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> class provides a 16-bit Unicode character.</p>
///
/// C++ class: <span style='color: green;'>```QChar```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qchar.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> class provides a 16-bit Unicode character.</p>
/// <p>In Qt, Unicode characters are 16-bit entities without any markup or structure. This class represents such an entity. It is lightweight, so it can be used everywhere. Most compilers treat it like an <code>unsigned short</code>.</p>
/// <p><a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> provides a full complement of testing/classification functions, converting to and from other formats, converting from composed to decomposed Unicode, and trying to compare and case-convert if you ask it to.</p>
/// <p>The classification functions include functions like those in the standard C++ header &lt;cctype&gt; (formerly &lt;ctype.h&gt;), but operating on the full range of Unicode characters, not just for the ASCII range. They all return true if the character is a certain type of character; otherwise they return false. These classification functions are <a href="http://doc.qt.io/qt-5/qchar.html#isNull">isNull</a>() (returns <code>true</code> if the character is '\0'), <a href="http://doc.qt.io/qt-5/qchar.html#isPrint">isPrint</a>() (true if the character is any sort of printable character, including whitespace), <a href="http://doc.qt.io/qt-5/qchar.html#isPunct">isPunct</a>() (any sort of punctation), <a href="http://doc.qt.io/qt-5/qchar.html#isMark">isMark</a>() (Unicode Mark), <a href="http://doc.qt.io/qt-5/qchar.html#isLetter">isLetter</a>() (a letter), <a href="http://doc.qt.io/qt-5/qchar.html#isNumber">isNumber</a>() (any sort of numeric character, not just 0-9), <a href="http://doc.qt.io/qt-5/qchar.html#isLetterOrNumber">isLetterOrNumber</a>(), and <a href="http://doc.qt.io/qt-5/qchar.html#isDigit">isDigit</a>() (decimal digits). All of these are wrappers around <a href="http://doc.qt.io/qt-5/qchar.html#category">category</a>() which return the Unicode-defined category of each character. Some of these also calculate the derived properties (for example <a href="http://doc.qt.io/qt-5/qchar.html#isSpace">isSpace</a>() returns <code>true</code> if the character is of category Separator_* or an exceptional code point from <a href="http://doc.qt.io/qt-5/qchar.html#Category-enum">Other_Control</a> category).</p>
/// <p><a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> also provides <a href="http://doc.qt.io/qt-5/qchar.html#direction">direction</a>(), which indicates the "natural" writing direction of this character. The <a href="http://doc.qt.io/qt-5/qchar.html#joiningType">joiningType</a>() function indicates how the character joins with it's neighbors (needed mostly for Arabic or Syriac) and finally <a href="http://doc.qt.io/qt-5/qchar.html#hasMirrored">hasMirrored</a>(), which indicates whether the character needs to be mirrored when it is printed in it's "unnatural" writing direction.</p>
/// <p>Composed Unicode characters (like <i>ring</i>) can be converted to decomposed Unicode ("a" followed by "ring above") by using <a href="http://doc.qt.io/qt-5/qchar.html#decomposition">decomposition</a>().</p>
/// <p>In Unicode, comparison is not necessarily possible and case conversion is very difficult at best. Unicode, covering the "entire" world, also includes most of the world's case and sorting problems. operator==() and friends will do comparison based purely on the numeric Unicode value (code point) of the characters, and <a href="http://doc.qt.io/qt-5/qchar.html#toUpper">toUpper</a>() and <a href="http://doc.qt.io/qt-5/qchar.html#toLower">toLower</a>() will do case changes when the character has a well-defined uppercase/lowercase equivalent. For locale-dependent comparisons, use <a href="http://doc.qt.io/qt-5/qstring.html#localeAwareCompare">QString::localeAwareCompare</a>().</p>
/// <p>The conversion functions include <a href="http://doc.qt.io/qt-5/qchar.html#unicode">unicode</a>() (to a scalar), <a href="http://doc.qt.io/qt-5/qchar.html#toLatin1">toLatin1</a>() (to scalar, but converts all non-Latin-1 characters to 0), <a href="http://doc.qt.io/qt-5/qchar.html#row">row</a>() (gives the Unicode row), <a href="http://doc.qt.io/qt-5/qchar.html#cell">cell</a>() (gives the Unicode cell), <a href="http://doc.qt.io/qt-5/qchar.html#digitValue">digitValue</a>() (gives the integer value of any of the numerous digit characters), and a host of constructors.</p>
/// <p><a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> provides constructors and cast operators that make it easy to convert to and from traditional 8-bit <code>char</code>s. If you defined <code>QT_NO_CAST_FROM_ASCII</code> and <code>QT_NO_CAST_TO_ASCII</code>, as explained in the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> documentation, you will need to explicitly call <a href="http://doc.qt.io/qt-5/qchar.html#fromLatin1">fromLatin1</a>(), or use <a href="http://doc.qt.io/qt-5/qlatin1char.html">QLatin1Char</a>, to construct a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> from an 8-bit <code>char</code>, and you will need to call <a href="http://doc.qt.io/qt-5/qchar.html#toLatin1">toLatin1</a>() to get the 8-bit value back.</p>
/// <p>For more information see <a href="http://www.unicode.org/ucd/">"About the Unicode Character Database"</a>.</p></div>
#[repr(C)]
pub struct QChar {
    _unused: u8,
}
impl QChar {
    /// <p>Returns the character's category.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar::Category QChar::category() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#category">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the character's category.</p></div>
    pub unsafe fn category_0a(&self) -> crate::q_char::Category {
        crate::__ffi::ctr_qt_core_ffi_QChar_category(self as *const crate::QChar)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QChar::Category QChar::category(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#category-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the category of the UCS-4-encoded character specified by <i>ucs4</i>.</p></div>
    pub unsafe fn category_1a(ucs4: ::std::os::raw::c_uint) -> crate::q_char::Category {
        crate::__ffi::ctr_qt_core_ffi_QChar_category1(ucs4)
    }

    /// <p>Returns the cell (least significant byte) of the Unicode character.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned char QChar::cell() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#cell">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the cell (least significant byte) of the Unicode character.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#row">row</a>().</p></div>
    pub unsafe fn cell(&self) -> ::std::os::raw::c_uchar {
        crate::__ffi::ctr_qt_core_ffi_QChar_cell(self as *const crate::QChar)
    }

    /// <p>Returns the combining class for the character as defined in the Unicode standard. This is mainly useful as a positioning hint for marks attached to a base character.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned char QChar::combiningClass() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#combiningClass">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the combining class for the character as defined in the Unicode standard. This is mainly useful as a positioning hint for marks attached to a base character.</p>
    /// <p>The Qt text rendering engine uses this information to correctly position non-spacing marks around a base character.</p></div>
    pub unsafe fn combining_class_0a(&self) -> ::std::os::raw::c_uchar {
        crate::__ffi::ctr_qt_core_ffi_QChar_combiningClass(self as *const crate::QChar)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static unsigned char QChar::combiningClass(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#combiningClass-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the combining class for the UCS-4-encoded character specified by <i>ucs4</i>, as defined in the Unicode standard.</p></div>
    pub unsafe fn combining_class_1a(ucs4: ::std::os::raw::c_uint) -> ::std::os::raw::c_uchar {
        crate::__ffi::ctr_qt_core_ffi_QChar_combiningClass1(ucs4)
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> class provides a 16-bit Unicode character.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar& QChar::operator=(const QChar& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> class provides a 16-bit Unicode character.</p>
    /// <p>In Qt, Unicode characters are 16-bit entities without any markup or structure. This class represents such an entity. It is lightweight, so it can be used everywhere. Most compilers treat it like an <code>unsigned short</code>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> provides a full complement of testing/classification functions, converting to and from other formats, converting from composed to decomposed Unicode, and trying to compare and case-convert if you ask it to.</p>
    /// <p>The classification functions include functions like those in the standard C++ header &lt;cctype&gt; (formerly &lt;ctype.h&gt;), but operating on the full range of Unicode characters, not just for the ASCII range. They all return true if the character is a certain type of character; otherwise they return false. These classification functions are <a href="http://doc.qt.io/qt-5/qchar.html#isNull">isNull</a>() (returns <code>true</code> if the character is '\0'), <a href="http://doc.qt.io/qt-5/qchar.html#isPrint">isPrint</a>() (true if the character is any sort of printable character, including whitespace), <a href="http://doc.qt.io/qt-5/qchar.html#isPunct">isPunct</a>() (any sort of punctation), <a href="http://doc.qt.io/qt-5/qchar.html#isMark">isMark</a>() (Unicode Mark), <a href="http://doc.qt.io/qt-5/qchar.html#isLetter">isLetter</a>() (a letter), <a href="http://doc.qt.io/qt-5/qchar.html#isNumber">isNumber</a>() (any sort of numeric character, not just 0-9), <a href="http://doc.qt.io/qt-5/qchar.html#isLetterOrNumber">isLetterOrNumber</a>(), and <a href="http://doc.qt.io/qt-5/qchar.html#isDigit">isDigit</a>() (decimal digits). All of these are wrappers around <a href="http://doc.qt.io/qt-5/qchar.html#category">category</a>() which return the Unicode-defined category of each character. Some of these also calculate the derived properties (for example <a href="http://doc.qt.io/qt-5/qchar.html#isSpace">isSpace</a>() returns <code>true</code> if the character is of category Separator_* or an exceptional code point from <a href="http://doc.qt.io/qt-5/qchar.html#Category-enum">Other_Control</a> category).</p>
    /// <p><a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> also provides <a href="http://doc.qt.io/qt-5/qchar.html#direction">direction</a>(), which indicates the "natural" writing direction of this character. The <a href="http://doc.qt.io/qt-5/qchar.html#joiningType">joiningType</a>() function indicates how the character joins with it's neighbors (needed mostly for Arabic or Syriac) and finally <a href="http://doc.qt.io/qt-5/qchar.html#hasMirrored">hasMirrored</a>(), which indicates whether the character needs to be mirrored when it is printed in it's "unnatural" writing direction.</p>
    /// <p>Composed Unicode characters (like <i>ring</i>) can be converted to decomposed Unicode ("a" followed by "ring above") by using <a href="http://doc.qt.io/qt-5/qchar.html#decomposition">decomposition</a>().</p>
    /// <p>In Unicode, comparison is not necessarily possible and case conversion is very difficult at best. Unicode, covering the "entire" world, also includes most of the world's case and sorting problems. operator==() and friends will do comparison based purely on the numeric Unicode value (code point) of the characters, and <a href="http://doc.qt.io/qt-5/qchar.html#toUpper">toUpper</a>() and <a href="http://doc.qt.io/qt-5/qchar.html#toLower">toLower</a>() will do case changes when the character has a well-defined uppercase/lowercase equivalent. For locale-dependent comparisons, use <a href="http://doc.qt.io/qt-5/qstring.html#localeAwareCompare">QString::localeAwareCompare</a>().</p>
    /// <p>The conversion functions include <a href="http://doc.qt.io/qt-5/qchar.html#unicode">unicode</a>() (to a scalar), <a href="http://doc.qt.io/qt-5/qchar.html#toLatin1">toLatin1</a>() (to scalar, but converts all non-Latin-1 characters to 0), <a href="http://doc.qt.io/qt-5/qchar.html#row">row</a>() (gives the Unicode row), <a href="http://doc.qt.io/qt-5/qchar.html#cell">cell</a>() (gives the Unicode cell), <a href="http://doc.qt.io/qt-5/qchar.html#digitValue">digitValue</a>() (gives the integer value of any of the numerous digit characters), and a host of constructors.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> provides constructors and cast operators that make it easy to convert to and from traditional 8-bit <code>char</code>s. If you defined <code>QT_NO_CAST_FROM_ASCII</code> and <code>QT_NO_CAST_TO_ASCII</code>, as explained in the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> documentation, you will need to explicitly call <a href="http://doc.qt.io/qt-5/qchar.html#fromLatin1">fromLatin1</a>(), or use <a href="http://doc.qt.io/qt-5/qlatin1char.html">QLatin1Char</a>, to construct a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> from an 8-bit <code>char</code>, and you will need to call <a href="http://doc.qt.io/qt-5/qchar.html#toLatin1">toLatin1</a>() to get the 8-bit value back.</p>
    /// <p>For more information see <a href="http://www.unicode.org/ucd/">"About the Unicode Character Database"</a>.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::MutRef<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QChar_operator_(
            self as *mut crate::QChar,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the most recent supported Unicode version.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QChar::UnicodeVersion QChar::currentUnicodeVersion()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#currentUnicodeVersion">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the most recent supported Unicode version.</p></div>
    pub unsafe fn current_unicode_version() -> crate::q_char::UnicodeVersion {
        crate::__ffi::ctr_qt_core_ffi_QChar_currentUnicodeVersion()
    }

    /// <p>Decomposes a character into it's constituent parts. Returns an empty string if no decomposition exists.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QChar::decomposition() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#decomposition">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Decomposes a character into it's constituent parts. Returns an empty string if no decomposition exists.</p></div>
    pub unsafe fn decomposition_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QChar_decomposition(self as *const crate::QChar);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QChar::decomposition(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#decomposition-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Decomposes the UCS-4-encoded character specified by <i>ucs4</i> into it's constituent parts. Returns an empty string if no decomposition exists.</p></div>
    pub unsafe fn decomposition_1a(
        ucs4: ::std::os::raw::c_uint,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QChar_decomposition1(ucs4);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the tag defining the composition of the character. Returns <a href="http://doc.qt.io/qt-5/qchar.html#Decomposition-enum">QChar::NoDecomposition</a> if no decomposition exists.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar::Decomposition QChar::decompositionTag() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#decompositionTag">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the tag defining the composition of the character. Returns <a href="http://doc.qt.io/qt-5/qchar.html#Decomposition-enum">QChar::NoDecomposition</a> if no decomposition exists.</p></div>
    pub unsafe fn decomposition_tag_0a(&self) -> crate::q_char::Decomposition {
        crate::__ffi::ctr_qt_core_ffi_QChar_decompositionTag(self as *const crate::QChar)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QChar::Decomposition QChar::decompositionTag(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#decompositionTag-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the tag defining the composition of the UCS-4-encoded character specified by <i>ucs4</i>. Returns <a href="http://doc.qt.io/qt-5/qchar.html#Decomposition-enum">QChar::NoDecomposition</a> if no decomposition exists.</p></div>
    pub unsafe fn decomposition_tag_1a(
        ucs4: ::std::os::raw::c_uint,
    ) -> crate::q_char::Decomposition {
        crate::__ffi::ctr_qt_core_ffi_QChar_decompositionTag1(ucs4)
    }

    /// <p>Returns the numeric value of the digit, or -1 if the character is not a digit.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QChar::digitValue() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#digitValue">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the numeric value of the digit, or -1 if the character is not a digit.</p></div>
    pub unsafe fn digit_value_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QChar_digitValue(self as *const crate::QChar)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QChar::digitValue(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#digitValue-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the numeric value of the digit specified by the UCS-4-encoded character, <i>ucs4</i>, or -1 if the character is not a digit.</p></div>
    pub unsafe fn digit_value_1a(ucs4: ::std::os::raw::c_uint) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QChar_digitValue1(ucs4)
    }

    /// <p>Returns the character's direction.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar::Direction QChar::direction() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#direction">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the character's direction.</p></div>
    pub unsafe fn direction_0a(&self) -> crate::q_char::Direction {
        crate::__ffi::ctr_qt_core_ffi_QChar_direction(self as *const crate::QChar)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QChar::Direction QChar::direction(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#direction-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the direction of the UCS-4-encoded character specified by <i>ucs4</i>.</p></div>
    pub unsafe fn direction_1a(ucs4: ::std::os::raw::c_uint) -> crate::q_char::Direction {
        crate::__ffi::ctr_qt_core_ffi_QChar_direction1(ucs4)
    }

    /// <p>Converts the Latin-1 character <i>c</i> to its equivalent <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>. This is mainly useful for non-internationalized software.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QChar QChar::fromLatin1(char c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#fromLatin1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the Latin-1 character <i>c</i> to its equivalent <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>. This is mainly useful for non-internationalized software.</p>
    /// <p>An alternative is to use <a href="http://doc.qt.io/qt-5/qlatin1char.html">QLatin1Char</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#toLatin1">toLatin1</a>() and <a href="http://doc.qt.io/qt-5/qchar.html#unicode">unicode</a>().</p></div>
    pub unsafe fn from_latin1(c: ::std::os::raw::c_char) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QChar_fromLatin1(c);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if the character should be reversed if the text direction is reversed; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QChar::hasMirrored() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#hasMirrored">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the character should be reversed if the text direction is reversed; otherwise returns <code>false</code>.</p>
    /// <p>A bit faster equivalent of (ch.<a href="http://doc.qt.io/qt-5/qchar.html#mirroredChar">mirroredChar</a>() != ch).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#mirroredChar">mirroredChar</a>().</p></div>
    pub unsafe fn has_mirrored_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_hasMirrored(self as *const crate::QChar)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QChar::hasMirrored(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#hasMirrored-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the UCS-4-encoded character specified by <i>ucs4</i> should be reversed if the text direction is reversed; otherwise returns <code>false</code>.</p>
    /// <p>A bit faster equivalent of (<a href="http://doc.qt.io/qt-5/qchar.html#mirroredChar">QChar::mirroredChar</a>(ucs4) != ucs4).</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#mirroredChar">mirroredChar</a>().</p></div>
    pub unsafe fn has_mirrored_1a(ucs4: ::std::os::raw::c_uint) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_hasMirrored1(ucs4)
    }

    /// <p>Returns the high surrogate part of a UCS-4-encoded code point. The returned result is undefined if <i>ucs4</i> is smaller than 0x10000.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static unsigned short QChar::highSurrogate(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#highSurrogate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the high surrogate part of a UCS-4-encoded code point. The returned result is undefined if <i>ucs4</i> is smaller than 0x10000.</p></div>
    pub unsafe fn high_surrogate(ucs4: ::std::os::raw::c_uint) -> ::std::os::raw::c_ushort {
        crate::__ffi::ctr_qt_core_ffi_QChar_highSurrogate(ucs4)
    }

    /// <p>Returns <code>true</code> if the character is a decimal digit (<a href="http://doc.qt.io/qt-5/qchar.html#Category-enum">Number_DecimalDigit</a>); otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QChar::isDigit() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isDigit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the character is a decimal digit (<a href="http://doc.qt.io/qt-5/qchar.html#Category-enum">Number_DecimalDigit</a>); otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#isNumber">isNumber</a>().</p></div>
    pub unsafe fn is_digit_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isDigit(self as *const crate::QChar)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QChar::isDigit(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isDigit-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the UCS-4-encoded character specified by <i>ucs4</i> is a decimal digit (<a href="http://doc.qt.io/qt-5/qchar.html#Category-enum">Number_DecimalDigit</a>); otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#isNumber">isNumber</a>().</p></div>
    pub unsafe fn is_digit_1a(ucs4: ::std::os::raw::c_uint) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isDigit1(ucs4)
    }

    /// <p>Returns <code>true</code> if the <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> is the high part of a UTF16 surrogate (for example if its code point is in range [0xd800..0xdbff]); false otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QChar::isHighSurrogate() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isHighSurrogate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> is the high part of a UTF16 surrogate (for example if its code point is in range [0xd800..0xdbff]); false otherwise.</p></div>
    pub unsafe fn is_high_surrogate_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isHighSurrogate(self as *const crate::QChar)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QChar::isHighSurrogate(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isHighSurrogate-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the UCS-4-encoded character specified by <i>ucs4</i> is the high part of a UTF16 surrogate (for example if its code point is in range [0xd800..0xdbff]); false otherwise.</p></div>
    pub unsafe fn is_high_surrogate_1a(ucs4: ::std::os::raw::c_uint) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isHighSurrogate1(ucs4)
    }

    /// <p>Returns <code>true</code> if the character is a letter (Letter_* categories); otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QChar::isLetter() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isLetter">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the character is a letter (Letter_* categories); otherwise returns <code>false</code>.</p></div>
    pub unsafe fn is_letter_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isLetter(self as *const crate::QChar)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QChar::isLetter(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isLetter-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the UCS-4-encoded character specified by <i>ucs4</i> is a letter (Letter_* categories); otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn is_letter_1a(ucs4: ::std::os::raw::c_uint) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isLetter1(ucs4)
    }

    /// <p>Returns <code>true</code> if the character is a letter or number (Letter_* or <a href="http://doc.qt.io/qt-5/stylesheet-reference.html#number">Number_</a>* categories); otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QChar::isLetterOrNumber() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isLetterOrNumber">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the character is a letter or number (Letter_* or <a href="http://doc.qt.io/qt-5/stylesheet-reference.html#number">Number_</a>* categories); otherwise returns <code>false</code>.</p></div>
    pub unsafe fn is_letter_or_number_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isLetterOrNumber(self as *const crate::QChar)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QChar::isLetterOrNumber(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isLetterOrNumber-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the UCS-4-encoded character specified by <i>ucs4</i> is a letter or number (Letter_* or <a href="http://doc.qt.io/qt-5/stylesheet-reference.html#number">Number_</a>* categories); otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn is_letter_or_number_1a(ucs4: ::std::os::raw::c_uint) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isLetterOrNumber1(ucs4)
    }

    /// <p>Returns <code>true</code> if the <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> is the low part of a UTF16 surrogate (for example if its code point is in range [0xdc00..0xdfff]); false otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QChar::isLowSurrogate() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isLowSurrogate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> is the low part of a UTF16 surrogate (for example if its code point is in range [0xdc00..0xdfff]); false otherwise.</p></div>
    pub unsafe fn is_low_surrogate_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isLowSurrogate(self as *const crate::QChar)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QChar::isLowSurrogate(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isLowSurrogate-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the UCS-4-encoded character specified by <i>ucs4</i> is the low part of a UTF16 surrogate (for example if its code point is in range [0xdc00..0xdfff]); false otherwise.</p></div>
    pub unsafe fn is_low_surrogate_1a(ucs4: ::std::os::raw::c_uint) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isLowSurrogate1(ucs4)
    }

    /// <p>Returns <code>true</code> if the character is a lowercase letter, for example <a href="http://doc.qt.io/qt-5/qchar.html#category">category</a>() is <a href="http://doc.qt.io/qt-5/qchar.html#Category-enum">Letter_Lowercase</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QChar::isLower() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isLower">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the character is a lowercase letter, for example <a href="http://doc.qt.io/qt-5/qchar.html#category">category</a>() is <a href="http://doc.qt.io/qt-5/qchar.html#Category-enum">Letter_Lowercase</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#isUpper">isUpper</a>(), <a href="http://doc.qt.io/qt-5/qchar.html#toLower">toLower</a>(), and <a href="http://doc.qt.io/qt-5/qchar.html#toUpper">toUpper</a>().</p></div>
    pub unsafe fn is_lower_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isLower(self as *const crate::QChar)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QChar::isLower(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isLower-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the UCS-4-encoded character specified by <i>ucs4</i> is a lowercase letter, for example <a href="http://doc.qt.io/qt-5/qchar.html#category">category</a>() is <a href="http://doc.qt.io/qt-5/qchar.html#Category-enum">Letter_Lowercase</a>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#isUpper">isUpper</a>(), <a href="http://doc.qt.io/qt-5/qchar.html#toLower">toLower</a>(), and <a href="http://doc.qt.io/qt-5/qchar.html#toUpper">toUpper</a>().</p></div>
    pub unsafe fn is_lower_1a(ucs4: ::std::os::raw::c_uint) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isLower1(ucs4)
    }

    /// <p>Returns <code>true</code> if the character is a mark (Mark_* categories); otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QChar::isMark() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isMark">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the character is a mark (Mark_* categories); otherwise returns <code>false</code>.</p>
    /// <p>See <a href="http://doc.qt.io/qt-5/qchar.html#Category-enum">QChar::Category</a> for more information regarding marks.</p></div>
    pub unsafe fn is_mark_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isMark(self as *const crate::QChar)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QChar::isMark(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isMark-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the UCS-4-encoded character specified by <i>ucs4</i> is a mark (Mark_* categories); otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn is_mark_1a(ucs4: ::std::os::raw::c_uint) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isMark1(ucs4)
    }

    /// <p>Returns <code>true</code> if the <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> is a non-character; false otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QChar::isNonCharacter() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isNonCharacter">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> is a non-character; false otherwise.</p>
    /// <p>Unicode has a certain number of code points that are classified as "non-characters:" that is, they can be used for internal purposes in applications but cannot be used for text interchange. Those are the last two entries each Unicode Plane ([0xfffe..0xffff], [0x1fffe..0x1ffff], etc.) as well as the entries in range [0xfdd0..0xfdef].</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn is_non_character_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isNonCharacter(self as *const crate::QChar)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QChar::isNonCharacter(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isNonCharacter-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the UCS-4-encoded character specified by <i>ucs4</i> is a non-character; false otherwise.</p>
    /// <p>Unicode has a certain number of code points that are classified as "non-characters:" that is, they can be used for internal purposes in applications but cannot be used for text interchange. Those are the last two entries each Unicode Plane ([0xfffe..0xffff], [0x1fffe..0x1ffff], etc.) as well as the entries in range [0xfdd0..0xfdef].</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn is_non_character_1a(ucs4: ::std::os::raw::c_uint) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isNonCharacter1(ucs4)
    }

    /// <p>Returns <code>true</code> if the character is the Unicode character 0x0000 ('\0'); otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QChar::isNull() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the character is the Unicode character 0x0000 ('\0'); otherwise returns <code>false</code>.</p></div>
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isNull(self as *const crate::QChar)
    }

    /// <p>Returns <code>true</code> if the character is a number (<a href="http://doc.qt.io/qt-5/stylesheet-reference.html#number">Number_</a>* categories, not just 0-9); otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QChar::isNumber() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isNumber">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the character is a number (<a href="http://doc.qt.io/qt-5/stylesheet-reference.html#number">Number_</a>* categories, not just 0-9); otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#isDigit">isDigit</a>().</p></div>
    pub unsafe fn is_number_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isNumber(self as *const crate::QChar)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QChar::isNumber(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isNumber-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the UCS-4-encoded character specified by <i>ucs4</i> is a number (<a href="http://doc.qt.io/qt-5/stylesheet-reference.html#number">Number_</a>* categories, not just 0-9); otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#isDigit">isDigit</a>().</p></div>
    pub unsafe fn is_number_1a(ucs4: ::std::os::raw::c_uint) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isNumber1(ucs4)
    }

    /// <p>Returns <code>true</code> if the character is a printable character; otherwise returns <code>false</code>. This is any character not of category <a href="http://doc.qt.io/qt-5/whatsnew50.html#other">Other_</a>*.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QChar::isPrint() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isPrint">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the character is a printable character; otherwise returns <code>false</code>. This is any character not of category <a href="http://doc.qt.io/qt-5/whatsnew50.html#other">Other_</a>*.</p>
    /// <p>Note that this gives no indication of whether the character is available in a particular font.</p></div>
    pub unsafe fn is_print_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isPrint(self as *const crate::QChar)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QChar::isPrint(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isPrint-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the UCS-4-encoded character specified by <i>ucs4</i> is a printable character; otherwise returns <code>false</code>. This is any character not of category <a href="http://doc.qt.io/qt-5/whatsnew50.html#other">Other_</a>*.</p>
    /// <p>Note that this gives no indication of whether the character is available in a particular font.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn is_print_1a(ucs4: ::std::os::raw::c_uint) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isPrint1(ucs4)
    }

    /// <p>Returns <code>true</code> if the character is a punctuation mark (Punctuation_* categories); otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QChar::isPunct() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isPunct">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the character is a punctuation mark (Punctuation_* categories); otherwise returns <code>false</code>.</p></div>
    pub unsafe fn is_punct_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isPunct(self as *const crate::QChar)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QChar::isPunct(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isPunct-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the UCS-4-encoded character specified by <i>ucs4</i> is a punctuation mark (Punctuation_* categories); otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn is_punct_1a(ucs4: ::std::os::raw::c_uint) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isPunct1(ucs4)
    }

    /// <p>Returns <code>true</code> if the character is a separator character (Separator_* categories or certain code points from <a href="http://doc.qt.io/qt-5/qchar.html#Category-enum">Other_Control</a> category); otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QChar::isSpace() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isSpace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the character is a separator character (Separator_* categories or certain code points from <a href="http://doc.qt.io/qt-5/qchar.html#Category-enum">Other_Control</a> category); otherwise returns <code>false</code>.</p></div>
    pub unsafe fn is_space_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isSpace(self as *const crate::QChar)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QChar::isSpace(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isSpace-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the UCS-4-encoded character specified by <i>ucs4</i> is a separator character (Separator_* categories or certain code points from <a href="http://doc.qt.io/qt-5/qchar.html#Category-enum">Other_Control</a> category); otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn is_space_1a(ucs4: ::std::os::raw::c_uint) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isSpace1(ucs4)
    }

    /// <p>Returns <code>true</code> if the <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> contains a code point that is in either the high or the low part of the UTF-16 surrogate range (for example if its code point is in range [0xd800..0xdfff]); false otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QChar::isSurrogate() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isSurrogate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> contains a code point that is in either the high or the low part of the UTF-16 surrogate range (for example if its code point is in range [0xd800..0xdfff]); false otherwise.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn is_surrogate_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isSurrogate(self as *const crate::QChar)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QChar::isSurrogate(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isSurrogate-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the UCS-4-encoded character specified by <i>ucs4</i> contains a code point that is in either the high or the low part of the UTF-16 surrogate range (for example if its code point is in range [0xd800..0xdfff]); false otherwise.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn is_surrogate_1a(ucs4: ::std::os::raw::c_uint) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isSurrogate1(ucs4)
    }

    /// <p>Returns <code>true</code> if the character is a symbol (Symbol_* categories); otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QChar::isSymbol() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isSymbol">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the character is a symbol (Symbol_* categories); otherwise returns <code>false</code>.</p></div>
    pub unsafe fn is_symbol_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isSymbol(self as *const crate::QChar)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QChar::isSymbol(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isSymbol-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the UCS-4-encoded character specified by <i>ucs4</i> is a symbol (Symbol_* categories); otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn is_symbol_1a(ucs4: ::std::os::raw::c_uint) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isSymbol1(ucs4)
    }

    /// <p>Returns <code>true</code> if the character is a titlecase letter, for example <a href="http://doc.qt.io/qt-5/qchar.html#category">category</a>() is <a href="http://doc.qt.io/qt-5/qchar.html#Category-enum">Letter_Titlecase</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QChar::isTitleCase() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isTitleCase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the character is a titlecase letter, for example <a href="http://doc.qt.io/qt-5/qchar.html#category">category</a>() is <a href="http://doc.qt.io/qt-5/qchar.html#Category-enum">Letter_Titlecase</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#isLower">isLower</a>(), <a href="http://doc.qt.io/qt-5/qchar.html#toUpper">toUpper</a>(), <a href="http://doc.qt.io/qt-5/qchar.html#toLower">toLower</a>(), and <a href="http://doc.qt.io/qt-5/qchar.html#toTitleCase">toTitleCase</a>().</p></div>
    pub unsafe fn is_title_case_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isTitleCase(self as *const crate::QChar)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QChar::isTitleCase(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isTitleCase-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the UCS-4-encoded character specified by <i>ucs4</i> is a titlecase letter, for example <a href="http://doc.qt.io/qt-5/qchar.html#category">category</a>() is <a href="http://doc.qt.io/qt-5/qchar.html#Category-enum">Letter_Titlecase</a>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#isLower">isLower</a>(), <a href="http://doc.qt.io/qt-5/qchar.html#toUpper">toUpper</a>(), <a href="http://doc.qt.io/qt-5/qchar.html#toLower">toLower</a>(), and <a href="http://doc.qt.io/qt-5/qchar.html#toTitleCase">toTitleCase</a>().</p></div>
    pub unsafe fn is_title_case_1a(ucs4: ::std::os::raw::c_uint) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isTitleCase1(ucs4)
    }

    /// <p>Returns <code>true</code> if the character is an uppercase letter, for example <a href="http://doc.qt.io/qt-5/qchar.html#category">category</a>() is <a href="http://doc.qt.io/qt-5/qchar.html#Category-enum">Letter_Uppercase</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QChar::isUpper() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isUpper">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the character is an uppercase letter, for example <a href="http://doc.qt.io/qt-5/qchar.html#category">category</a>() is <a href="http://doc.qt.io/qt-5/qchar.html#Category-enum">Letter_Uppercase</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#isLower">isLower</a>(), <a href="http://doc.qt.io/qt-5/qchar.html#toUpper">toUpper</a>(), and <a href="http://doc.qt.io/qt-5/qchar.html#toLower">toLower</a>().</p></div>
    pub unsafe fn is_upper_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isUpper(self as *const crate::QChar)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QChar::isUpper(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#isUpper-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the UCS-4-encoded character specified by <i>ucs4</i> is an uppercase letter, for example <a href="http://doc.qt.io/qt-5/qchar.html#category">category</a>() is <a href="http://doc.qt.io/qt-5/qchar.html#Category-enum">Letter_Uppercase</a>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#isLower">isLower</a>(), <a href="http://doc.qt.io/qt-5/qchar.html#toUpper">toUpper</a>(), and <a href="http://doc.qt.io/qt-5/qchar.html#toLower">toLower</a>().</p></div>
    pub unsafe fn is_upper_1a(ucs4: ::std::os::raw::c_uint) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_isUpper1(ucs4)
    }

    /// <p>Returns information about the joining properties of the character (needed for certain languages such as Arabic).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar::Joining QChar::joining() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar-obsolete.html#joining">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns information about the joining properties of the character (needed for certain languages such as Arabic).</p></div>
    pub unsafe fn joining_0a(&self) -> crate::q_char::Joining {
        crate::__ffi::ctr_qt_core_ffi_QChar_joining(self as *const crate::QChar)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QChar::Joining QChar::joining(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar-obsolete.html#joining-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns information about the joining properties of the UCS-4-encoded character specified by <i>ucs4</i> (needed for certain languages such as Arabic).</p></div>
    pub unsafe fn joining_1a(ucs4: ::std::os::raw::c_uint) -> crate::q_char::Joining {
        crate::__ffi::ctr_qt_core_ffi_QChar_joining1(ucs4)
    }

    /// <p>Returns information about the joining type attributes of the character (needed for certain languages such as Arabic or Syriac).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar::JoiningType QChar::joiningType() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#joiningType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns information about the joining type attributes of the character (needed for certain languages such as Arabic or Syriac).</p>
    /// <p>This function was introduced in  Qt 5.3.</p></div>
    pub unsafe fn joining_type_0a(&self) -> crate::q_char::JoiningType {
        crate::__ffi::ctr_qt_core_ffi_QChar_joiningType(self as *const crate::QChar)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QChar::JoiningType QChar::joiningType(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#joiningType-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns information about the joining type attributes of the UCS-4-encoded character specified by <i>ucs4</i> (needed for certain languages such as Arabic or Syriac).</p>
    /// <p>This function was introduced in  Qt 5.3.</p></div>
    pub unsafe fn joining_type_1a(ucs4: ::std::os::raw::c_uint) -> crate::q_char::JoiningType {
        crate::__ffi::ctr_qt_core_ffi_QChar_joiningType1(ucs4)
    }

    /// <p>Returns the low surrogate part of a UCS-4-encoded code point. The returned result is undefined if <i>ucs4</i> is smaller than 0x10000.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static unsigned short QChar::lowSurrogate(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#lowSurrogate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the low surrogate part of a UCS-4-encoded code point. The returned result is undefined if <i>ucs4</i> is smaller than 0x10000.</p></div>
    pub unsafe fn low_surrogate(ucs4: ::std::os::raw::c_uint) -> ::std::os::raw::c_ushort {
        crate::__ffi::ctr_qt_core_ffi_QChar_lowSurrogate(ucs4)
    }

    /// <p>Returns the mirrored character if this character is a mirrored character; otherwise returns the character itself.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QChar::mirroredChar() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#mirroredChar">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the mirrored character if this character is a mirrored character; otherwise returns the character itself.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#hasMirrored">hasMirrored</a>().</p></div>
    pub unsafe fn mirrored_char_0a(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QChar_mirroredChar(self as *const crate::QChar);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static unsigned int QChar::mirroredChar(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#mirroredChar-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the mirrored character if the UCS-4-encoded character specified by <i>ucs4</i> is a mirrored character; otherwise returns the character itself.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#hasMirrored">hasMirrored</a>().</p></div>
    pub unsafe fn mirrored_char_1a(ucs4: ::std::os::raw::c_uint) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QChar_mirroredChar1(ucs4)
    }

    /// <p>Constructs a null <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> ('\0').</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QChar::QChar()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#QChar">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a null <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> ('\0').</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#isNull">isNull</a>().</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QChar_QChar();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QChar::QChar(unsigned short rc)```</span>.
    pub unsafe fn from_ushort(rc: ::std::os::raw::c_ushort) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QChar_QChar1(rc);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QChar::QChar(unsigned char c, unsigned char r)```</span>.
    pub unsafe fn from_2_uchar(
        c: ::std::os::raw::c_uchar,
        r: ::std::os::raw::c_uchar,
    ) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QChar_QChar2(c, r);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> for the character with Unicode code point <i>code</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QChar::QChar(short rc)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#QChar-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> for the character with Unicode code point <i>code</i>.</p></div>
    pub unsafe fn from_short(rc: ::std::os::raw::c_short) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QChar_QChar3(rc);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QChar::QChar(unsigned int rc)```</span>.
    pub unsafe fn from_uint(rc: ::std::os::raw::c_uint) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QChar_QChar4(rc);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> for the character with Unicode code point <i>code</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QChar::QChar(int rc)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#QChar-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> for the character with Unicode code point <i>code</i>.</p></div>
    pub unsafe fn from_int(rc: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QChar_QChar5(rc);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> for the predefined character value <i>ch</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QChar::QChar(QChar::SpecialCharacter s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#QChar-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> for the predefined character value <i>ch</i>.</p></div>
    pub unsafe fn from_special_character(
        s: crate::q_char::SpecialCharacter,
    ) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QChar_QChar6(s);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> corresponding to ASCII/Latin-1 character <i>ch</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QChar::QChar(QLatin1Char ch)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#QChar-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> corresponding to ASCII/Latin-1 character <i>ch</i>.</p></div>
    pub unsafe fn from_q_latin1_char(
        ch: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1Char>>,
    ) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QChar_QChar7(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1Char>>::cast_into(ch)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> corresponding to ASCII/Latin-1 character <i>ch</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QChar::QChar(char c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#QChar-8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> corresponding to ASCII/Latin-1 character <i>ch</i>.</p>
    /// <p><b>Note: </b>This constructor is not available when <code>QT_NO_CAST_FROM_ASCII</code> is defined.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#QT_NO_CAST_FROM_ASCII">QT_NO_CAST_FROM_ASCII</a>.</p></div>
    pub unsafe fn from_char(c: ::std::os::raw::c_char) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QChar_QChar8(c);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QChar::QChar(unsigned char c)```</span>.
    pub unsafe fn from_uchar(c: ::std::os::raw::c_uchar) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QChar_QChar9(c);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> class provides a 16-bit Unicode character.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QChar::QChar(const QChar& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> class provides a 16-bit Unicode character.</p>
    /// <p>In Qt, Unicode characters are 16-bit entities without any markup or structure. This class represents such an entity. It is lightweight, so it can be used everywhere. Most compilers treat it like an <code>unsigned short</code>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> provides a full complement of testing/classification functions, converting to and from other formats, converting from composed to decomposed Unicode, and trying to compare and case-convert if you ask it to.</p>
    /// <p>The classification functions include functions like those in the standard C++ header &lt;cctype&gt; (formerly &lt;ctype.h&gt;), but operating on the full range of Unicode characters, not just for the ASCII range. They all return true if the character is a certain type of character; otherwise they return false. These classification functions are <a href="http://doc.qt.io/qt-5/qchar.html#isNull">isNull</a>() (returns <code>true</code> if the character is '\0'), <a href="http://doc.qt.io/qt-5/qchar.html#isPrint">isPrint</a>() (true if the character is any sort of printable character, including whitespace), <a href="http://doc.qt.io/qt-5/qchar.html#isPunct">isPunct</a>() (any sort of punctation), <a href="http://doc.qt.io/qt-5/qchar.html#isMark">isMark</a>() (Unicode Mark), <a href="http://doc.qt.io/qt-5/qchar.html#isLetter">isLetter</a>() (a letter), <a href="http://doc.qt.io/qt-5/qchar.html#isNumber">isNumber</a>() (any sort of numeric character, not just 0-9), <a href="http://doc.qt.io/qt-5/qchar.html#isLetterOrNumber">isLetterOrNumber</a>(), and <a href="http://doc.qt.io/qt-5/qchar.html#isDigit">isDigit</a>() (decimal digits). All of these are wrappers around <a href="http://doc.qt.io/qt-5/qchar.html#category">category</a>() which return the Unicode-defined category of each character. Some of these also calculate the derived properties (for example <a href="http://doc.qt.io/qt-5/qchar.html#isSpace">isSpace</a>() returns <code>true</code> if the character is of category Separator_* or an exceptional code point from <a href="http://doc.qt.io/qt-5/qchar.html#Category-enum">Other_Control</a> category).</p>
    /// <p><a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> also provides <a href="http://doc.qt.io/qt-5/qchar.html#direction">direction</a>(), which indicates the "natural" writing direction of this character. The <a href="http://doc.qt.io/qt-5/qchar.html#joiningType">joiningType</a>() function indicates how the character joins with it's neighbors (needed mostly for Arabic or Syriac) and finally <a href="http://doc.qt.io/qt-5/qchar.html#hasMirrored">hasMirrored</a>(), which indicates whether the character needs to be mirrored when it is printed in it's "unnatural" writing direction.</p>
    /// <p>Composed Unicode characters (like <i>ring</i>) can be converted to decomposed Unicode ("a" followed by "ring above") by using <a href="http://doc.qt.io/qt-5/qchar.html#decomposition">decomposition</a>().</p>
    /// <p>In Unicode, comparison is not necessarily possible and case conversion is very difficult at best. Unicode, covering the "entire" world, also includes most of the world's case and sorting problems. operator==() and friends will do comparison based purely on the numeric Unicode value (code point) of the characters, and <a href="http://doc.qt.io/qt-5/qchar.html#toUpper">toUpper</a>() and <a href="http://doc.qt.io/qt-5/qchar.html#toLower">toLower</a>() will do case changes when the character has a well-defined uppercase/lowercase equivalent. For locale-dependent comparisons, use <a href="http://doc.qt.io/qt-5/qstring.html#localeAwareCompare">QString::localeAwareCompare</a>().</p>
    /// <p>The conversion functions include <a href="http://doc.qt.io/qt-5/qchar.html#unicode">unicode</a>() (to a scalar), <a href="http://doc.qt.io/qt-5/qchar.html#toLatin1">toLatin1</a>() (to scalar, but converts all non-Latin-1 characters to 0), <a href="http://doc.qt.io/qt-5/qchar.html#row">row</a>() (gives the Unicode row), <a href="http://doc.qt.io/qt-5/qchar.html#cell">cell</a>() (gives the Unicode cell), <a href="http://doc.qt.io/qt-5/qchar.html#digitValue">digitValue</a>() (gives the integer value of any of the numerous digit characters), and a host of constructors.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> provides constructors and cast operators that make it easy to convert to and from traditional 8-bit <code>char</code>s. If you defined <code>QT_NO_CAST_FROM_ASCII</code> and <code>QT_NO_CAST_TO_ASCII</code>, as explained in the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> documentation, you will need to explicitly call <a href="http://doc.qt.io/qt-5/qchar.html#fromLatin1">fromLatin1</a>(), or use <a href="http://doc.qt.io/qt-5/qlatin1char.html">QLatin1Char</a>, to construct a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> from an 8-bit <code>char</code>, and you will need to call <a href="http://doc.qt.io/qt-5/qchar.html#toLatin1">toLatin1</a>() to get the 8-bit value back.</p>
    /// <p>For more information see <a href="http://www.unicode.org/ucd/">"About the Unicode Character Database"</a>.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QChar_QChar10(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if the UCS-4-encoded character specified by <i>ucs4</i> can be split into the high and low parts of a UTF16 surrogate (for example if its code point is greater than or equals to 0x10000); false otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QChar::requiresSurrogates(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#requiresSurrogates">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the UCS-4-encoded character specified by <i>ucs4</i> can be split into the high and low parts of a UTF16 surrogate (for example if its code point is greater than or equals to 0x10000); false otherwise.</p></div>
    pub unsafe fn requires_surrogates(ucs4: ::std::os::raw::c_uint) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChar_requiresSurrogates(ucs4)
    }

    /// <p>Returns the row (most significant byte) of the Unicode character.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned char QChar::row() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#row">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the row (most significant byte) of the Unicode character.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#cell">cell</a>().</p></div>
    pub unsafe fn row(&self) -> ::std::os::raw::c_uchar {
        crate::__ffi::ctr_qt_core_ffi_QChar_row(self as *const crate::QChar)
    }

    /// <p>Returns the Unicode script property value for this character.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar::Script QChar::script() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#script">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the Unicode script property value for this character.</p>
    /// <p>This function was introduced in  Qt 5.1.</p></div>
    pub unsafe fn script_0a(&self) -> crate::q_char::Script {
        crate::__ffi::ctr_qt_core_ffi_QChar_script(self as *const crate::QChar)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QChar::Script QChar::script(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#script-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the Unicode script property value for the character specified in its UCS-4-encoded form as <i>ucs4</i>.</p>
    /// <p>This function was introduced in  Qt 5.1.</p></div>
    pub unsafe fn script_1a(ucs4: ::std::os::raw::c_uint) -> crate::q_char::Script {
        crate::__ffi::ctr_qt_core_ffi_QChar_script1(ucs4)
    }

    /// Calls C++ function: <span style='color: green;'>```void QChar::setCell(unsigned char acell)```</span>.
    pub unsafe fn set_cell(&mut self, acell: ::std::os::raw::c_uchar) {
        crate::__ffi::ctr_qt_core_ffi_QChar_setCell(self as *mut crate::QChar, acell)
    }

    /// Calls C++ function: <span style='color: green;'>```void QChar::setRow(unsigned char arow)```</span>.
    pub unsafe fn set_row(&mut self, arow: ::std::os::raw::c_uchar) {
        crate::__ffi::ctr_qt_core_ffi_QChar_setRow(self as *mut crate::QChar, arow)
    }

    /// <p>Converts a UTF16 surrogate pair with the given <i>high</i> and <i>low</i> values to it's UCS-4-encoded code point.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static unsigned int QChar::surrogateToUcs4(unsigned short high, unsigned short low)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#surrogateToUcs4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts a UTF16 surrogate pair with the given <i>high</i> and <i>low</i> values to it's UCS-4-encoded code point.</p></div>
    pub unsafe fn surrogate_to_ucs4_2_ushort(
        high: ::std::os::raw::c_ushort,
        low: ::std::os::raw::c_ushort,
    ) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QChar_surrogateToUcs4(high, low)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static unsigned int QChar::surrogateToUcs4(QChar high, QChar low)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#surrogateToUcs4-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Converts a UTF16 surrogate pair (<i>high</i>, <i>low</i>) to it's UCS-4-encoded code point.</p></div>
    pub unsafe fn surrogate_to_ucs4_2_q_char(
        high: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        low: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QChar_surrogateToUcs41(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(high).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(low).as_raw_ptr(),
        )
    }

    /// <p>Returns the case folded equivalent of the character. For most Unicode characters this is the same as <a href="http://doc.qt.io/qt-5/qchar.html#toLower">toLower</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QChar::toCaseFolded() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#toCaseFolded">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the case folded equivalent of the character. For most Unicode characters this is the same as <a href="http://doc.qt.io/qt-5/qchar.html#toLower">toLower</a>().</p></div>
    pub unsafe fn to_case_folded_0a(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QChar_toCaseFolded(self as *const crate::QChar);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static unsigned int QChar::toCaseFolded(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#toCaseFolded-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the case folded equivalent of the UCS-4-encoded character specified by <i>ucs4</i>. For most Unicode characters this is the same as <a href="http://doc.qt.io/qt-5/qchar.html#toLower">toLower</a>().</p></div>
    pub unsafe fn to_case_folded_1a(ucs4: ::std::os::raw::c_uint) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QChar_toCaseFolded1(ucs4)
    }

    /// <p>Returns the Latin-1 character equivalent to the <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>, or 0. This is mainly useful for non-internationalized software.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```char QChar::toLatin1() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#toLatin1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the Latin-1 character equivalent to the <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>, or 0. This is mainly useful for non-internationalized software.</p>
    /// <p><b>Note: </b>It is not possible to distinguish a non-Latin-1 character from a Latin-1 0 (NUL) character. Prefer to use <a href="http://doc.qt.io/qt-5/qchar.html#unicode">unicode</a>(), which does not have this ambiguity.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#unicode">unicode</a>().</p></div>
    pub unsafe fn to_latin1(&self) -> ::std::os::raw::c_char {
        crate::__ffi::ctr_qt_core_ffi_QChar_toLatin1(self as *const crate::QChar)
    }

    /// <p>Returns the lowercase equivalent if the character is uppercase or titlecase; otherwise returns the character itself.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QChar::toLower() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#toLower">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the lowercase equivalent if the character is uppercase or titlecase; otherwise returns the character itself.</p></div>
    pub unsafe fn to_lower_0a(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QChar_toLower(self as *const crate::QChar);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static unsigned int QChar::toLower(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#toLower-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the lowercase equivalent of the UCS-4-encoded character specified by <i>ucs4</i> if the character is uppercase or titlecase; otherwise returns the character itself.</p></div>
    pub unsafe fn to_lower_1a(ucs4: ::std::os::raw::c_uint) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QChar_toLower1(ucs4)
    }

    /// <p>Returns the title case equivalent if the character is lowercase or uppercase; otherwise returns the character itself.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QChar::toTitleCase() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#toTitleCase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the title case equivalent if the character is lowercase or uppercase; otherwise returns the character itself.</p></div>
    pub unsafe fn to_title_case_0a(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QChar_toTitleCase(self as *const crate::QChar);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static unsigned int QChar::toTitleCase(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#toTitleCase-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the title case equivalent of the UCS-4-encoded character specified by <i>ucs4</i> if the character is lowercase or uppercase; otherwise returns the character itself.</p></div>
    pub unsafe fn to_title_case_1a(ucs4: ::std::os::raw::c_uint) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QChar_toTitleCase1(ucs4)
    }

    /// <p>Returns the uppercase equivalent if the character is lowercase or titlecase; otherwise returns the character itself.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QChar::toUpper() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#toUpper">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the uppercase equivalent if the character is lowercase or titlecase; otherwise returns the character itself.</p></div>
    pub unsafe fn to_upper_0a(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QChar_toUpper(self as *const crate::QChar);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static unsigned int QChar::toUpper(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#toUpper-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the uppercase equivalent of the UCS-4-encoded character specified by <i>ucs4</i> if the character is lowercase or titlecase; otherwise returns the character itself.</p></div>
    pub unsafe fn to_upper_1a(ucs4: ::std::os::raw::c_uint) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QChar_toUpper1(ucs4)
    }

    /// <p>Returns the numeric Unicode value of the <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned short QChar::unicode() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#unicode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the numeric Unicode value of the <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>.</p></div>
    pub unsafe fn unicode(&self) -> ::std::os::raw::c_ushort {
        crate::__ffi::ctr_qt_core_ffi_QChar_unicode(self as *const crate::QChar)
    }

    /// <p>Returns a reference to the numeric Unicode value of the <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned short& QChar::unicode()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#unicode-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the numeric Unicode value of the <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>.</p></div>
    pub unsafe fn unicode_mut(&mut self) -> ::cpp_utils::MutRef<::std::os::raw::c_ushort> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QChar_unicode1(self as *mut crate::QChar);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the Unicode version that introduced this character.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar::UnicodeVersion QChar::unicodeVersion() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#unicodeVersion">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the Unicode version that introduced this character.</p></div>
    pub unsafe fn unicode_version_0a(&self) -> crate::q_char::UnicodeVersion {
        crate::__ffi::ctr_qt_core_ffi_QChar_unicodeVersion(self as *const crate::QChar)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QChar::UnicodeVersion QChar::unicodeVersion(unsigned int ucs4)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#unicodeVersion-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the Unicode version that introduced the character specified in its UCS-4-encoded form as <i>ucs4</i>.</p></div>
    pub unsafe fn unicode_version_1a(
        ucs4: ::std::os::raw::c_uint,
    ) -> crate::q_char::UnicodeVersion {
        crate::__ffi::ctr_qt_core_ffi_QChar_unicodeVersion1(ucs4)
    }
}

pub mod q_byte_array {
    //! C++ type: <span style='color: green;'>```QByteArray```</span>

    /// <p>This enum contains the options available for encoding and decoding Base64. Base64 is defined by <a href="http://www.ietf.org/rfc/rfc4648.txt">RFC 4648</a>, with the following options:</p>
    ///
    /// C++ enum: <span style='color: green;'>```QByteArray::Base64Option```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#Base64Option-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum contains the options available for encoding and decoding Base64. Base64 is defined by <a href="http://www.ietf.org/rfc/rfc4648.txt">RFC 4648</a>, with the following options:</p>
    ///
    /// <p><a href="http://doc.qt.io/qt-5/qbytearray.html#fromBase64">QByteArray::fromBase64</a>() ignores the KeepTrailingEquals and OmitTrailingEquals options and will not flag errors in case they are missing or if there are too many of them.</p>
    /// <p>This enum was introduced or modified in  Qt 5.2.</p>
    /// <p>The Base64Options type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;Base64Option&gt;. It stores an OR combination of Base64Option values.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Base64Option(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Base64Option {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Base64Option(value)
        }
    }

    impl From<Base64Option> for ::std::os::raw::c_int {
        fn from(value: Base64Option) -> Self {
            value.0
        }
    }

    impl Base64Option {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Base64Option {
        /// (default) The regular Base64 alphabet, called simply "base64" (C++ enum variant: <span style='color: green;'>```Base64Encoding = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const Base64Encoding: crate::q_byte_array::Base64Option =
            crate::q_byte_array::Base64Option(0);
        /// An alternate alphabet, called "base64url", which replaces two characters in the alphabet to be more friendly to URLs. (C++ enum variant: <span style='color: green;'>```Base64UrlEncoding = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Base64UrlEncoding: crate::q_byte_array::Base64Option =
            crate::q_byte_array::Base64Option(1);
        /// (default) Keeps the trailing padding equal signs at the end of the encoded data, so the data is always a size multiple of four. (C++ enum variant: <span style='color: green;'>```KeepTrailingEquals = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const KeepTrailingEquals: crate::q_byte_array::Base64Option =
            crate::q_byte_array::Base64Option(0);
        /// Omits adding the padding equal signs at the end of the encoded data. (C++ enum variant: <span style='color: green;'>```OmitTrailingEquals = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const OmitTrailingEquals: crate::q_byte_array::Base64Option =
            crate::q_byte_array::Base64Option(2);
    }

    impl From<crate::q_byte_array::Base64Option> for crate::QFlags<crate::q_byte_array::Base64Option> {
        fn from(value: crate::q_byte_array::Base64Option) -> Self {
            Self::from(value.to_int())
        }
    }

    impl<T: Into<crate::QFlags<crate::q_byte_array::Base64Option>>> std::ops::BitOr<T>
        for crate::q_byte_array::Base64Option
    {
        type Output = crate::QFlags<crate::q_byte_array::Base64Option>;
        fn bitor(self, rhs: T) -> crate::QFlags<crate::q_byte_array::Base64Option> {
            Into::<crate::QFlags<crate::q_byte_array::Base64Option>>::into(self) | rhs
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> class provides an array of bytes.</p>
///
/// C++ class: <span style='color: green;'>```QByteArray```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qbytearray.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> class provides an array of bytes.</p>
/// <p><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> can be used to store both raw bytes (including '\0's) and traditional 8-bit '\0'-terminated strings. Using <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> is much more convenient than using <code>const char *</code>. Behind the scenes, it always ensures that the data is followed by a '\0' terminator, and uses <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicit sharing</a> (copy-on-write) to reduce memory usage and avoid needless copying of data.</p>
/// <p>In addition to <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>, Qt also provides the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> class to store string data. For most purposes, <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> is the class you want to use. It stores 16-bit Unicode characters, making it easy to store non-ASCII/non-Latin-1 characters in your application. Furthermore, <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> is used throughout in the Qt API. The two main cases where <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> is appropriate are when you need to store raw binary data, and when memory conservation is critical (e.g., with Qt for Embedded Linux).</p>
/// <p>One way to initialize a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> is simply to pass a <code>const char *</code> to its constructor. For example, the following code creates a byte array of size 5 containing the data "Hello":</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba(<span class="string">"Hello"</span>);
///
/// </pre>
/// <p>Although the <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>() is 5, the byte array also maintains an extra '\0' character at the end so that if a function is used that asks for a pointer to the underlying data (e.g. a call to <a href="http://doc.qt.io/qt-5/qbytearray.html#data">data</a>()), the data pointed to is guaranteed to be '\0'-terminated.</p>
/// <p><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> makes a deep copy of the <code>const char *</code> data, so you can modify it later without experiencing side effects. (If for performance reasons you don't want to take a deep copy of the character data, use <a href="http://doc.qt.io/qt-5/qbytearray.html#fromRawData">QByteArray::fromRawData</a>() instead.)</p>
/// <p>Another approach is to set the size of the array using <a href="http://doc.qt.io/qt-5/qbytearray.html#resize">resize</a>() and to initialize the data byte per byte. <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> uses 0-based indexes, just like C++ arrays. To access the byte at a particular index position, you can use operator[](). On non-const byte arrays, operator[]() returns a reference to a byte that can be used on the left side of an assignment. For example:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba;
///   ba<span class="operator">.</span>resize(<span class="number">5</span>);
///   ba<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">0x3c</span>;
///   ba<span class="operator">[</span><span class="number">1</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">0xb8</span>;
///   ba<span class="operator">[</span><span class="number">2</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">0x64</span>;
///   ba<span class="operator">[</span><span class="number">3</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">0x18</span>;
///   ba<span class="operator">[</span><span class="number">4</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">0xca</span>;
///
/// </pre>
/// <p>For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qbytearray.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> ba<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (ba<span class="operator">.</span>at(i) <span class="operator">&gt;</span><span class="operator">=</span> <span class="char">'a'</span> <span class="operator">&amp;</span><span class="operator">&amp;</span> ba<span class="operator">.</span>at(i) <span class="operator">&lt;</span><span class="operator">=</span> <span class="char">'f'</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found character in range [a-f]"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qbytearray.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>To extract many bytes at a time, use <a href="http://doc.qt.io/qt-5/qbytearray.html#left">left</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#right">right</a>(), or <a href="http://doc.qt.io/qt-5/qbytearray.html#mid">mid</a>().</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> can embed '\0' bytes. The <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>() function always returns the size of the whole array, including embedded '\0' bytes, but excluding the terminating '\0' added by <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>. For example:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba1(<span class="string">"ca\0r\0t"</span>);
///   ba1<span class="operator">.</span>size();                     <span class="comment">// Returns 2.</span>
///   ba1<span class="operator">.</span>constData();                <span class="comment">// Returns "ca" with terminating \0.</span>
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba2(<span class="string">"ca\0r\0t"</span><span class="operator">,</span> <span class="number">3</span>);
///   ba2<span class="operator">.</span>size();                     <span class="comment">// Returns 3.</span>
///   ba2<span class="operator">.</span>constData();                <span class="comment">// Returns "ca\0" with terminating \0.</span>
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba3(<span class="string">"ca\0r\0t"</span><span class="operator">,</span> <span class="number">4</span>);
///   ba3<span class="operator">.</span>size();                     <span class="comment">// Returns 4.</span>
///   ba3<span class="operator">.</span>constData();                <span class="comment">// Returns "ca\0r" with terminating \0.</span>
///
///   <span class="keyword">const</span> <span class="type">char</span> cart<span class="operator">[</span><span class="operator">]</span> <span class="operator">=</span> {<span class="char">'c'</span><span class="operator">,</span> <span class="char">'a'</span><span class="operator">,</span> <span class="char">'\0'</span><span class="operator">,</span> <span class="char">'r'</span><span class="operator">,</span> <span class="char">'\0'</span><span class="operator">,</span> <span class="char">'t'</span>};
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba4(<span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span><span class="operator">::</span>fromRawData(cart<span class="operator">,</span> <span class="number">6</span>));
///   ba4<span class="operator">.</span>size();                     <span class="comment">// Returns 6.</span>
///   ba4<span class="operator">.</span>constData();                <span class="comment">// Returns "ca\0r\0t" without terminating \0.</span>
///
/// </pre>
/// <p>If you want to obtain the length of the data up to and excluding the first '\0' character, call <a href="http://doc.qt.io/qt-5/qbytearray.html#qstrlen">qstrlen</a>() on the byte array.</p>
/// <p>After a call to <a href="http://doc.qt.io/qt-5/qbytearray.html#resize">resize</a>(), newly allocated bytes have undefined values. To set all the bytes to a particular value, call <a href="http://doc.qt.io/qt-5/qbytearray.html#fill">fill</a>().</p>
/// <p>To obtain a pointer to the actual character data, call <a href="http://doc.qt.io/qt-5/qbytearray.html#data">data</a>() or <a href="http://doc.qt.io/qt-5/qbytearray.html#constData">constData</a>(). These functions return a pointer to the beginning of the data. The pointer is guaranteed to remain valid until a non-const function is called on the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>. It is also guaranteed that the data ends with a '\0' byte unless the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> was created from a <a href="http://doc.qt.io/qt-5/qbytearray.html#fromRawData">raw data</a>. This '\0' byte is automatically provided by <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> and is not counted in <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> provides the following basic functions for modifying the byte data: <a href="http://doc.qt.io/qt-5/qbytearray.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qbytearray.html#remove">remove</a>(). For example:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> x(<span class="string">"and"</span>);
///   x<span class="operator">.</span>prepend(<span class="string">"rock "</span>);         <span class="comment">// x == "rock and"</span>
///   x<span class="operator">.</span>append(<span class="string">" roll"</span>);          <span class="comment">// x == "rock and roll"</span>
///   x<span class="operator">.</span>replace(<span class="number">5</span><span class="operator">,</span> <span class="number">3</span><span class="operator">,</span> <span class="string">"&amp;"</span>);       <span class="comment">// x == "rock &amp; roll"</span>
///
/// </pre>
/// <p>The <a href="http://doc.qt.io/qt-5/qbytearray.html#replace">replace</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#remove">remove</a>() functions' first two arguments are the position from which to start erasing and the number of bytes that should be erased.</p>
/// <p>When you <a href="http://doc.qt.io/qt-5/qbytearray.html#append">append</a>() data to a non-empty array, the array will be reallocated and the new data copied to it. You can avoid this behavior by calling <a href="http://doc.qt.io/qt-5/qbytearray.html#reserve">reserve</a>(), which preallocates a certain amount of memory. You can also call <a href="http://doc.qt.io/qt-5/qbytearray.html#capacity">capacity</a>() to find out how much memory <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> actually allocated. Data appended to an empty array is not copied.</p>
/// <p>A frequent requirement is to remove whitespace characters from a byte array ('\n', '\t', ' ', etc.). If you want to remove whitespace from both ends of a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>, use <a href="http://doc.qt.io/qt-5/qbytearray.html#trimmed">trimmed</a>(). If you want to remove whitespace from both ends and replace multiple consecutive whitespaces with a single space character within the byte array, use <a href="http://doc.qt.io/qt-5/qbytearray.html#simplified">simplified</a>().</p>
/// <p>If you want to find all occurrences of a particular character or substring in a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>, use <a href="http://doc.qt.io/qt-5/qbytearray.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qbytearray.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index position of the character or substring if they find it; otherwise, they return -1. For example, here's a typical loop that finds all occurrences of a particular substring:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba(<span class="string">"We must be &lt;b&gt;bold&lt;/b&gt;, very &lt;b&gt;bold&lt;/b&gt;"</span>);
///   <span class="type">int</span> j <span class="operator">=</span> <span class="number">0</span>;
///   <span class="keyword">while</span> ((j <span class="operator">=</span> ba<span class="operator">.</span>indexOf(<span class="string">"&lt;b&gt;"</span><span class="operator">,</span> j)) <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>) {
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found &lt;b&gt; tag at index position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> j <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
/// &#32;     <span class="operator">+</span><span class="operator">+</span>j;
///   }
///
/// </pre>
/// <p>If you simply want to check whether a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> contains a particular character or substring, use <a href="http://doc.qt.io/qt-5/qbytearray.html#contains">contains</a>(). If you want to find out how many times a particular character or substring occurs in the byte array, use <a href="http://doc.qt.io/qt-5/qbytearray.html#count-3">count</a>(). If you want to replace all occurrences of a particular value with another, use one of the two-parameter <a href="http://doc.qt.io/qt-5/qbytearray.html#replace">replace</a>() overloads.</p>
/// <p><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>s can be compared using overloaded operators such as operator&lt;(), operator&lt;=(), operator==(), operator&gt;=(), and so on. The comparison is based exclusively on the numeric values of the characters and is very fast, but is not what a human would expect. <a href="http://doc.qt.io/qt-5/qstring.html#localeAwareCompare">QString::localeAwareCompare</a>() is a better choice for sorting user-interface strings.</p>
/// <p>For historical reasons, <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> distinguishes between a null byte array and an empty byte array. A <i>null</i> byte array is a byte array that is initialized using <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>'s default constructor or by passing (const char *)0 to the constructor. An <i>empty</i> byte array is any byte array with size 0. A null byte array is always empty, but an empty byte array isn't necessarily null:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span>()<span class="operator">.</span>isNull();          <span class="comment">// returns true</span>
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span>()<span class="operator">.</span>isEmpty();         <span class="comment">// returns true</span>
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span>(<span class="string">""</span>)<span class="operator">.</span>isNull();        <span class="comment">// returns false</span>
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span>(<span class="string">""</span>)<span class="operator">.</span>isEmpty();       <span class="comment">// returns true</span>
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span>(<span class="string">"abc"</span>)<span class="operator">.</span>isNull();     <span class="comment">// returns false</span>
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span>(<span class="string">"abc"</span>)<span class="operator">.</span>isEmpty();    <span class="comment">// returns false</span>
///
/// </pre>
/// <p>All functions except <a href="http://doc.qt.io/qt-5/qbytearray.html#isNull">isNull</a>() treat null byte arrays the same as empty byte arrays. For example, <a href="http://doc.qt.io/qt-5/qbytearray.html#data">data</a>() returns a pointer to a '\0' character for a null byte array (<i>not</i> a null pointer), and <a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a>() compares equal to <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>(""). We recommend that you always use <a href="http://doc.qt.io/qt-5/qbytearray.html#isEmpty">isEmpty</a>() and avoid <a href="http://doc.qt.io/qt-5/qbytearray.html#isNull">isNull</a>().</p>
/// <a name="notes-on-locale"></a></div>
#[repr(C)]
pub struct QByteArray {
    _unused: u8,
}
impl QByteArray {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::append(char c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#append-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the character <i>ch</i> to this byte array.</p></div>
    pub unsafe fn append_char(
        &mut self,
        c: ::std::os::raw::c_char,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_append(self as *mut crate::QByteArray, c);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::append(int count, char c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#append-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>count</i> copies of character <i>ch</i> to this byte array and returns a reference to this byte array.</p>
    /// <p>If <i>count</i> is negative or zero nothing is appended to the byte array.</p>
    /// <p>This function was introduced in  Qt 5.7.</p></div>
    pub unsafe fn append_int_char(
        &mut self,
        count: ::std::os::raw::c_int,
        c: ::std::os::raw::c_char,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_append1(
            self as *mut crate::QByteArray,
            count,
            c,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::append(const char* s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#append-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the string <i>str</i> to this byte array.</p></div>
    pub unsafe fn append_char2(
        &mut self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_append2(
            self as *mut crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qbytearray.html#append">append</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::append(const char* s, int len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#append-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qbytearray.html#append">append</a>().</p>
    /// <p>Appends the first <i>len</i> characters of the string <i>str</i> to this byte array and returns a reference to this byte array.</p>
    /// <p>If <i>len</i> is negative, the length of the string will be determined automatically using <a href="http://doc.qt.io/qt-5/qbytearray.html#qstrlen">qstrlen</a>(). If <i>len</i> is zero or <i>str</i> is null, nothing is appended to the byte array. Ensure that <i>len</i> is <i>not</i> longer than <i>str</i>.</p></div>
    pub unsafe fn append_char_int(
        &mut self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        len: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_append3(
            self as *mut crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            len,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Appends the byte array <i>ba</i> onto the end of this byte array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::append(const QByteArray& a)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the byte array <i>ba</i> onto the end of this byte array.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> x(<span class="string">"free"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> y(<span class="string">"dom"</span>);
    ///   x<span class="operator">.</span>append(y);
    ///   <span class="comment">// x == "freedom"</span>
    ///
    /// </pre>
    /// <p>This is the same as insert(<a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>(), <i>ba</i>).</p>
    /// <p>Note: <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> is an <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a> class. Consequently, if you append to an empty byte array, then the byte array will just share the data held in <i>ba</i>. In this case, no copying of data is done, taking <a href="http://doc.qt.io/qt-5/containers.html#constant-time">constant time</a>. If a shared instance is modified, it will be copied (copy-on-write), taking <a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>.</p>
    /// <p>If the byte array being appended to is not empty, a deep copy of the data is performed, taking <a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>.</p>
    /// <p>This operation typically does not suffer from allocation overhead, because <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> preallocates extra space at the end of the data so that it may grow without reallocating for each append operation.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#operator-2b-eq">operator+=</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qbytearray.html#insert">insert</a>().</p></div>
    pub unsafe fn append_q_byte_array(
        &mut self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_append4(
            self as *mut crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(a).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::append(const QString& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#append-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the string <i>str</i> to this byte array. The Unicode data is converted into 8-bit characters using <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">QString::toUtf8</a>().</p>
    /// <p>You can disable this function by defining <code>QT_NO_CAST_TO_ASCII</code> when you compile your applications. You then need to call <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">QString::toUtf8</a>() (or <a href="http://doc.qt.io/qt-5/qstring.html#toLatin1">QString::toLatin1</a>() or <a href="http://doc.qt.io/qt-5/qstring.html#toLocal8Bit">QString::toLocal8Bit</a>()) explicitly if you want to convert the data to <code>const char *</code>.</p></div>
    pub unsafe fn append_q_string(
        &mut self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_append5(
            self as *mut crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the character at index position <i>i</i> in the byte array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```char QByteArray::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the character at index position <i>i</i> in the byte array.</p>
    /// <p><i>i</i> must be a valid index position in the byte array (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(&self, i: ::std::os::raw::c_int) -> ::std::os::raw::c_char {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_at(self as *const crate::QByteArray, i)
    }

    /// <p>Returns the last character in the byte array. Same as <code>at(size() - 1)</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```char QByteArray::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the last character in the byte array. Same as <code>at(size() - 1)</code>.</p>
    /// <p>This function is provided for STL compatibility.</p>
    /// <p><b>Warning:</b> Calling this function on an empty byte array constitutes undefined behavior.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#front">front</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#at">at</a>(), and <a href="http://doc.qt.io/qt-5/qbytearray.html#operator-5b-5d">operator[]</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn back(&self) -> ::std::os::raw::c_char {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_back(self as *const crate::QByteArray)
    }

    /// <p>Returns a reference to the last character in the byte array. Same as <code>operator[](size() - 1)</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteRef QByteArray::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last character in the byte array. Same as <code>operator[](size() - 1)</code>.</p>
    /// <p>This function is provided for STL compatibility.</p>
    /// <p><b>Warning:</b> Calling this function on an empty byte array constitutes undefined behavior.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#front">front</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#at">at</a>(), and <a href="http://doc.qt.io/qt-5/qbytearray.html#operator-5b-5d">operator[]</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::CppBox<crate::QByteRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_back1(self as *mut crate::QByteArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the maximum number of bytes that can be stored in the byte array without forcing a reallocation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::capacity() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#capacity">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the maximum number of bytes that can be stored in the byte array without forcing a reallocation.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>'s memory usage. In general, you will rarely ever need to call this function. If you want to know how many bytes are in the byte array, call <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#squeeze">squeeze</a>().</p></div>
    pub unsafe fn capacity(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_capacity(self as *const crate::QByteArray)
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first character in the byte-array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QByteArray::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first character in the byte-array.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_cbegin(self as *const crate::QByteArray);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary character after the last character in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QByteArray::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary character after the last character in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#end">end</a>().</p></div>
    pub unsafe fn cend(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_cend(self as *const crate::QByteArray);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Removes <i>n</i> bytes from the end of the byte array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QByteArray::chop(int n)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#chop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes <i>n</i> bytes from the end of the byte array.</p>
    /// <p>If <i>n</i> is greater than <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>(), the result is an empty byte array.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba(<span class="string">"STARTTLS\r\n"</span>);
    ///   ba<span class="operator">.</span>chop(<span class="number">2</span>);                 <span class="comment">// ba == "STARTTLS"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#truncate">truncate</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#resize">resize</a>(), and <a href="http://doc.qt.io/qt-5/qbytearray.html#left">left</a>().</p></div>
    pub unsafe fn chop(&mut self, n: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_chop(self as *mut crate::QByteArray, n)
    }

    /// <p>Returns a byte array that contains the leftmost <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>() - <i>len</i> bytes of this byte array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::chopped(int len) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#chopped">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a byte array that contains the leftmost <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>() - <i>len</i> bytes of this byte array.</p>
    /// <p><b>Note: </b>The behavior is undefined if <i>len</i> is negative or greater than <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>().</p><p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#endsWith">endsWith</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#left">left</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#right">right</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#mid">mid</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#chop">chop</a>(), and <a href="http://doc.qt.io/qt-5/qbytearray.html#truncate">truncate</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn chopped(
        &self,
        len: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_chopped(self as *const crate::QByteArray, len);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Clears the contents of the byte array and makes it null.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QByteArray::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Clears the contents of the byte array and makes it null.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#resize">resize</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#isNull">isNull</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_clear(self as *mut crate::QByteArray)
    }

    /// <p>Returns an integer less than, equal to, or greater than zero depending on whether this <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> sorts before, at the same position, or after the string pointed to by <i>c</i>. The comparison is performed according to case sensitivity <i>cs</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::compare(const char* c, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#compare">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an integer less than, equal to, or greater than zero depending on whether this <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> sorts before, at the same position, or after the string pointed to by <i>c</i>. The comparison is performed according to case sensitivity <i>cs</i>.</p>
    /// <p>This function was introduced in  Qt 5.12.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#operator-eq-eq">operator==</a>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn compare_char_case_sensitivity(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_compare(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            cs,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::compare(const QByteArray& a, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#compare-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns an integer less than, equal to, or greater than zero depending on whether this <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> sorts before, at the same position, or after the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> <i>a</i>. The comparison is performed according to case sensitivity <i>cs</i>.</p>
    /// <p>This function was introduced in  Qt 5.12.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#operator-eq-eq">operator==</a>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn compare_q_byte_array_case_sensitivity(
        &self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_compare1(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(a).as_raw_ptr(),
            cs,
        )
    }

    /// <p>Returns an integer less than, equal to, or greater than zero depending on whether this <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> sorts before, at the same position, or after the string pointed to by <i>c</i>. The comparison is performed according to case sensitivity <i>cs</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::compare(const char* c) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#compare">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an integer less than, equal to, or greater than zero depending on whether this <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> sorts before, at the same position, or after the string pointed to by <i>c</i>. The comparison is performed according to case sensitivity <i>cs</i>.</p>
    /// <p>This function was introduced in  Qt 5.12.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#operator-eq-eq">operator==</a>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn compare_char(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_compare2(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::compare(const QByteArray& a) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#compare-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns an integer less than, equal to, or greater than zero depending on whether this <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> sorts before, at the same position, or after the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> <i>a</i>. The comparison is performed according to case sensitivity <i>cs</i>.</p>
    /// <p>This function was introduced in  Qt 5.12.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#operator-eq-eq">operator==</a>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn compare_q_byte_array(
        &self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_compare3(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(a).as_raw_ptr(),
        )
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first character in the byte-array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QByteArray::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first character in the byte-array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_constBegin(self as *const crate::QByteArray);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns a pointer to the data stored in the byte array. The pointer can be used to access the bytes that compose the array. The data is '\0'-terminated unless the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> object was created from raw data. The pointer remains valid as long as the byte array isn't reallocated or destroyed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QByteArray::constData() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#constData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the data stored in the byte array. The pointer can be used to access the bytes that compose the array. The data is '\0'-terminated unless the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> object was created from raw data. The pointer remains valid as long as the byte array isn't reallocated or destroyed.</p>
    /// <p>This function is mostly useful to pass a byte array to a function that accepts a <code>const char *</code>.</p>
    /// <p>Note: A <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> can store any byte values including '\0's, but most functions that take <code>char *</code> arguments assume that the data ends at the first '\0' they encounter.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#data">data</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#operator-5b-5d">operator[]</a>(), and <a href="http://doc.qt.io/qt-5/qbytearray.html#fromRawData">fromRawData</a>().</p></div>
    pub unsafe fn const_data(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_constData(self as *const crate::QByteArray);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary character after the last character in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QByteArray::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary character after the last character in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#end">end</a>().</p></div>
    pub unsafe fn const_end(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_constEnd(self as *const crate::QByteArray);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QByteArray::contains(char c) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#contains-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the byte array contains the character <i>ch</i>; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn contains_char(&self, c: ::std::os::raw::c_char) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_contains(self as *const crate::QByteArray, c)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QByteArray::contains(const char* a) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#contains-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the byte array contains the string <i>str</i>; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn contains_char2(
        &self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_contains1(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(a)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if the byte array contains an occurrence of the byte array <i>ba</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QByteArray::contains(const QByteArray& a) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the byte array contains an occurrence of the byte array <i>ba</i>; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#count-3">count</a>().</p></div>
    pub unsafe fn contains_q_byte_array(
        &self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_contains2(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(a).as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this byte array and returns a reference to this byte array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::operator=(const QByteArray& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this byte array and returns a reference to this byte array.</p></div>
    pub unsafe fn copy_from_q_byte_array(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_operator_(
            self as *mut crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::operator=(const char* str)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#operator-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Assigns <i>str</i> to this byte array.</p></div>
    pub unsafe fn copy_from_char(
        &mut self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_operator_1(
            self as *mut crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(str)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::count(char c) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#count-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the number of occurrences of character <i>ch</i> in the byte array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_char(&self, c: ::std::os::raw::c_char) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_count(self as *const crate::QByteArray, c)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::count(const char* a) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the number of (potentially overlapping) occurrences of string <i>str</i> in the byte array.</p></div>
    pub unsafe fn count_char2(
        &self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_count1(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(a)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the number of (potentially overlapping) occurrences of byte array <i>ba</i> in this byte array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::count(const QByteArray& a) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of (potentially overlapping) occurrences of byte array <i>ba</i> in this byte array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_q_byte_array(
        &self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_count2(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(a).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#count-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>().</p></div>
    pub unsafe fn count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_count3(self as *const crate::QByteArray)
    }

    /// <p>Returns a pointer to the data stored in the byte array. The pointer can be used to access and modify the bytes that compose the array. The data is '\0'-terminated, i.e. the number of bytes in the returned character string is <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>() + 1 for the '\0' terminator.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```char* QByteArray::data()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the data stored in the byte array. The pointer can be used to access and modify the bytes that compose the array. The data is '\0'-terminated, i.e. the number of bytes in the returned character string is <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>() + 1 for the '\0' terminator.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba(<span class="string">"Hello world"</span>);
    ///   <span class="type">char</span> <span class="operator">*</span>data <span class="operator">=</span> ba<span class="operator">.</span>data();
    ///   <span class="keyword">while</span> (<span class="operator">*</span>data) {
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"["</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>data <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"]"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>data;
    ///   }
    ///
    /// </pre>
    /// <p>The pointer remains valid as long as the byte array isn't reallocated or destroyed. For read-only access, <a href="http://doc.qt.io/qt-5/qbytearray.html#constData">constData</a>() is faster because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
    /// <p>This function is mostly useful to pass a byte array to a function that accepts a <code>const char *</code>.</p>
    /// <p>The following example makes a copy of the char* returned by data(), but it will corrupt the heap and cause a crash because it does not allocate a byte for the '\0' at the end:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> tmp <span class="operator">=</span> <span class="string">"test"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> text <span class="operator">=</span> tmp<span class="operator">.</span>toLocal8Bit();
    ///   <span class="type">char</span> <span class="operator">*</span>data <span class="operator">=</span> <span class="keyword">new</span> <span class="type">char</span><span class="operator">[</span>text<span class="operator">.</span>size()<span class="operator">]</span>;
    ///   strcpy(data<span class="operator">,</span> text<span class="operator">.</span>data());
    ///   <span class="keyword">delete</span> <span class="operator">[</span><span class="operator">]</span> data;
    ///
    /// </pre>
    /// <p>This one allocates the correct amount of space:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> tmp <span class="operator">=</span> <span class="string">"test"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> text <span class="operator">=</span> tmp<span class="operator">.</span>toLocal8Bit();
    ///   <span class="type">char</span> <span class="operator">*</span>data <span class="operator">=</span> <span class="keyword">new</span> <span class="type">char</span><span class="operator">[</span>text<span class="operator">.</span>size() <span class="operator">+</span> <span class="number">1</span><span class="operator">]</span>;
    ///   strcpy(data<span class="operator">,</span> text<span class="operator">.</span>data());
    ///   <span class="keyword">delete</span> <span class="operator">[</span><span class="operator">]</span> data;
    ///
    /// </pre>
    /// <p>Note: A <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> can store any byte values including '\0's, but most functions that take <code>char *</code> arguments assume that the data ends at the first '\0' they encounter.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#constData">constData</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn data_mut(&mut self) -> ::cpp_utils::MutPtr<::std::os::raw::c_char> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_data(self as *mut crate::QByteArray);
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QByteArray::data() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#data-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn data(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_data1(self as *const crate::QByteArray);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```void QByteArray::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_detach(self as *mut crate::QByteArray)
    }

    /// <p>Returns <code>true</code> if this byte array ends with byte array <i>ba</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QByteArray::endsWith(const QByteArray& a) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this byte array ends with byte array <i>ba</i>; otherwise returns <code>false</code>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> url(<span class="string">"http://qt-project.org/doc/qt-5.0/qtdoc/index.html"</span>);
    ///   <span class="keyword">if</span> (url<span class="operator">.</span>endsWith(<span class="string">".html"</span>))
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#startsWith">startsWith</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#right">right</a>().</p></div>
    pub unsafe fn ends_with_q_byte_array(
        &self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_endsWith(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(a).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QByteArray::endsWith(char c) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#endsWith-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if this byte array ends with character <i>ch</i>; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn ends_with_char(&self, c: ::std::os::raw::c_char) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_endsWith1(self as *const crate::QByteArray, c)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QByteArray::endsWith(const char* c) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#endsWith-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if this byte array ends with string <i>str</i>; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn ends_with_char2(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_endsWith2(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets every byte in the byte array to character <i>ch</i>. If <i>size</i> is different from -1 (the default), the byte array is resized to size <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::fill(char c, int size = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets every byte in the byte array to character <i>ch</i>. If <i>size</i> is different from -1 (the default), the byte array is resized to size <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba(<span class="string">"Istambul"</span>);
    ///   ba<span class="operator">.</span>fill(<span class="char">'o'</span>);
    ///   <span class="comment">// ba == "oooooooo"</span>
    ///
    ///   ba<span class="operator">.</span>fill(<span class="char">'X'</span><span class="operator">,</span> <span class="number">2</span>);
    ///   <span class="comment">// ba == "XX"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_2a(
        &mut self,
        c: ::std::os::raw::c_char,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_fill(self as *mut crate::QByteArray, c, size);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Sets every byte in the byte array to character <i>ch</i>. If <i>size</i> is different from -1 (the default), the byte array is resized to size <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::fill(char c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets every byte in the byte array to character <i>ch</i>. If <i>size</i> is different from -1 (the default), the byte array is resized to size <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba(<span class="string">"Istambul"</span>);
    ///   ba<span class="operator">.</span>fill(<span class="char">'o'</span>);
    ///   <span class="comment">// ba == "oooooooo"</span>
    ///
    ///   ba<span class="operator">.</span>fill(<span class="char">'X'</span><span class="operator">,</span> <span class="number">2</span>);
    ///   <span class="comment">// ba == "XX"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_1a(
        &mut self,
        c: ::std::os::raw::c_char,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_fill1(self as *mut crate::QByteArray, c);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QByteArray::fromBase64(const QByteArray& base64, QFlags<QByteArray::Base64Option> options)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#fromBase64-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a decoded copy of the Base64 array <i>base64</i>, using the alphabet defined by <i>options</i>. Input is not checked for validity; invalid characters in the input are skipped, enabling the decoding process to continue with subsequent characters.</p>
    /// <p>For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span><span class="operator">::</span>fromBase64(<span class="string">"PHA+SGVsbG8/PC9wPg=="</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span><span class="operator">::</span>Base64Encoding); <span class="comment">// returns "&lt;p&gt;Hello?&lt;/p&gt;"</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span><span class="operator">::</span>fromBase64(<span class="string">"PHA-SGVsbG8_PC9wPg=="</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span><span class="operator">::</span>Base64UrlEncoding); <span class="comment">// returns "&lt;p&gt;Hello?&lt;/p&gt;"</span>
    ///
    /// </pre>
    /// <p>The algorithm used to decode Base64-encoded data is defined in <a href="http://www.ietf.org/rfc/rfc4648.txt">RFC 4648</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toBase64">toBase64</a>().</p></div>
    pub unsafe fn from_base64_2a(
        base64: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        options: crate::QFlags<crate::q_byte_array::Base64Option>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_fromBase64(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(base64)
                .as_raw_ptr(),
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a decoded copy of the Base64 array <i>base64</i>. Input is not checked for validity; invalid characters in the input are skipped, enabling the decoding process to continue with subsequent characters.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QByteArray::fromBase64(const QByteArray& base64)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#fromBase64">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a decoded copy of the Base64 array <i>base64</i>. Input is not checked for validity; invalid characters in the input are skipped, enabling the decoding process to continue with subsequent characters.</p>
    /// <p>For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> text <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span><span class="operator">::</span>fromBase64(<span class="string">"UXQgaXMgZ3JlYXQh"</span>);
    ///   text<span class="operator">.</span>data();            <span class="comment">// returns "Qt is great!"</span>
    ///
    /// </pre>
    /// <p>The algorithm used to decode Base64-encoded data is defined in <a href="http://www.ietf.org/rfc/rfc4648.txt">RFC 4648</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toBase64">toBase64</a>().</p></div>
    pub unsafe fn from_base64_1a(
        base64: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_fromBase641(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(base64)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a decoded copy of the hex encoded array <i>hexEncoded</i>. Input is not checked for validity; invalid characters in the input are skipped, enabling the decoding process to continue with subsequent characters.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QByteArray::fromHex(const QByteArray& hexEncoded)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#fromHex">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a decoded copy of the hex encoded array <i>hexEncoded</i>. Input is not checked for validity; invalid characters in the input are skipped, enabling the decoding process to continue with subsequent characters.</p>
    /// <p>For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> text <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span><span class="operator">::</span>fromHex(<span class="string">"517420697320677265617421"</span>);
    ///   text<span class="operator">.</span>data();            <span class="comment">// returns "Qt is great!"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toHex">toHex</a>().</p></div>
    pub unsafe fn from_hex(
        hex_encoded: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_fromHex(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(hex_encoded)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a decoded copy of the URI/URL-style percent-encoded <i>input</i>. The <i>percent</i> parameter allows you to replace the '%' character for another (for instance, '_' or '=').</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QByteArray::fromPercentEncoding(const QByteArray& pctEncoded, char percent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#fromPercentEncoding">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a decoded copy of the URI/URL-style percent-encoded <i>input</i>. The <i>percent</i> parameter allows you to replace the '%' character for another (for instance, '_' or '=').</p>
    /// <p>For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> text <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span><span class="operator">::</span>fromPercentEncoding(<span class="string">"Qt%20is%20great%33"</span>);
    ///   text<span class="operator">.</span>data();            <span class="comment">// returns "Qt is great!"</span>
    ///
    /// </pre>
    /// <p><b>Note: </b>Given invalid input (such as a string containing the sequence "%G5", which is not a valid hexadecimal number) the output will be invalid as well. As an example: the sequence "%G5" could be decoded to 'W'.</p><p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toPercentEncoding">toPercentEncoding</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#fromPercentEncoding">QUrl::fromPercentEncoding</a>().</p></div>
    pub unsafe fn from_percent_encoding_2a(
        pct_encoded: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        percent: ::std::os::raw::c_char,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_fromPercentEncoding(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(pct_encoded)
                .as_raw_ptr(),
            percent,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a decoded copy of the URI/URL-style percent-encoded <i>input</i>. The <i>percent</i> parameter allows you to replace the '%' character for another (for instance, '_' or '=').</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QByteArray::fromPercentEncoding(const QByteArray& pctEncoded)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#fromPercentEncoding">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a decoded copy of the URI/URL-style percent-encoded <i>input</i>. The <i>percent</i> parameter allows you to replace the '%' character for another (for instance, '_' or '=').</p>
    /// <p>For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> text <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span><span class="operator">::</span>fromPercentEncoding(<span class="string">"Qt%20is%20great%33"</span>);
    ///   text<span class="operator">.</span>data();            <span class="comment">// returns "Qt is great!"</span>
    ///
    /// </pre>
    /// <p><b>Note: </b>Given invalid input (such as a string containing the sequence "%G5", which is not a valid hexadecimal number) the output will be invalid as well. As an example: the sequence "%G5" could be decoded to 'W'.</p><p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toPercentEncoding">toPercentEncoding</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#fromPercentEncoding">QUrl::fromPercentEncoding</a>().</p></div>
    pub unsafe fn from_percent_encoding_1a(
        pct_encoded: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_fromPercentEncoding1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(pct_encoded)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> that uses the first <i>size</i> bytes of the <i>data</i> array. The bytes are <i>not</i> copied. The <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> will contain the <i>data</i> pointer. The caller guarantees that <i>data</i> will not be deleted or modified as long as this <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> and any copies of it exist that have not been modified. In other words, because <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> is an <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a> class and the instance returned by this function contains the <i>data</i> pointer, the caller must not delete <i>data</i> or modify it directly as long as the returned <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> and any copies exist. However, <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> does not take ownership of <i>data</i>, so the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> destructor will never delete the raw <i>data</i>, even when the last <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> referring to <i>data</i> is destroyed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QByteArray::fromRawData(const char* arg1, int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#fromRawData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> that uses the first <i>size</i> bytes of the <i>data</i> array. The bytes are <i>not</i> copied. The <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> will contain the <i>data</i> pointer. The caller guarantees that <i>data</i> will not be deleted or modified as long as this <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> and any copies of it exist that have not been modified. In other words, because <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> is an <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a> class and the instance returned by this function contains the <i>data</i> pointer, the caller must not delete <i>data</i> or modify it directly as long as the returned <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> and any copies exist. However, <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> does not take ownership of <i>data</i>, so the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> destructor will never delete the raw <i>data</i>, even when the last <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> referring to <i>data</i> is destroyed.</p>
    /// <p>A subsequent attempt to modify the contents of the returned <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> or any copy made from it will cause it to create a deep copy of the <i>data</i> array before doing the modification. This ensures that the raw <i>data</i> array itself will never be modified by <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    /// <p>Here is an example of how to read data using a <a href="http://doc.qt.io/qt-5/qdatastream.html">QDataStream</a> on raw data in memory without copying the raw data into a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>:</p>
    /// <pre class="cpp">
    ///
    ///    <span class="keyword">static</span> <span class="keyword">const</span> <span class="type">char</span> mydata<span class="operator">[</span><span class="operator">]</span> <span class="operator">=</span> {
    /// &#32;     <span class="number">0x00</span><span class="operator">,</span> <span class="number">0x00</span><span class="operator">,</span> <span class="number">0x03</span><span class="operator">,</span> <span class="number">0x84</span><span class="operator">,</span> <span class="number">0x78</span><span class="operator">,</span> <span class="number">0x9c</span><span class="operator">,</span> <span class="number">0x3b</span><span class="operator">,</span> <span class="number">0x76</span><span class="operator">,</span>
    /// &#32;     <span class="number">0xec</span><span class="operator">,</span> <span class="number">0x18</span><span class="operator">,</span> <span class="number">0xc3</span><span class="operator">,</span> <span class="number">0x31</span><span class="operator">,</span> <span class="number">0x0a</span><span class="operator">,</span> <span class="number">0xf1</span><span class="operator">,</span> <span class="number">0xcc</span><span class="operator">,</span> <span class="number">0x99</span><span class="operator">,</span>
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    /// &#32;     <span class="number">0x6d</span><span class="operator">,</span> <span class="number">0x5b</span>
    ///   };
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> data <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span><span class="operator">::</span>fromRawData(mydata<span class="operator">,</span> <span class="keyword">sizeof</span>(mydata));
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdatastream.html">QDataStream</a></span> in(<span class="operator">&amp;</span>data<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a></span><span class="operator">::</span>ReadOnly);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///
    /// </pre>
    /// <p><b>Warning:</b> A byte array created with fromRawData() is <i>not</i> null-terminated, unless the raw data contains a 0 character at position <i>size</i>. While that does not matter for <a href="http://doc.qt.io/qt-5/qdatastream.html">QDataStream</a> or functions like <a href="http://doc.qt.io/qt-5/qbytearray.html#indexOf">indexOf</a>(), passing the byte array to a function accepting a <code>const char *</code> expected to be '\0'-terminated will fail.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#setRawData">setRawData</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#data">data</a>(), and <a href="http://doc.qt.io/qt-5/qbytearray.html#constData">constData</a>().</p></div>
    pub unsafe fn from_raw_data(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_fromRawData(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
            size,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the first character in the byte array. Same as <code>at(0)</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```char QByteArray::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the first character in the byte array. Same as <code>at(0)</code>.</p>
    /// <p>This function is provided for STL compatibility.</p>
    /// <p><b>Warning:</b> Calling this function on an empty byte array constitutes undefined behavior.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#back">back</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#at">at</a>(), and <a href="http://doc.qt.io/qt-5/qbytearray.html#operator-5b-5d">operator[]</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn front(&self) -> ::std::os::raw::c_char {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_front(self as *const crate::QByteArray)
    }

    /// <p>Returns a reference to the first character in the byte array. Same as <code>operator[](0)</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteRef QByteArray::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first character in the byte array. Same as <code>operator[](0)</code>.</p>
    /// <p>This function is provided for STL compatibility.</p>
    /// <p><b>Warning:</b> Calling this function on an empty byte array constitutes undefined behavior.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#back">back</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#at">at</a>(), and <a href="http://doc.qt.io/qt-5/qbytearray.html#operator-5b-5d">operator[]</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::CppBox<crate::QByteRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_front1(self as *mut crate::QByteArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```char QByteArray::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#operator-5b-5d-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as at(<i>i</i>).</p></div>
    pub unsafe fn index_int(&self, i: ::std::os::raw::c_int) -> ::std::os::raw::c_char {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_operator__(self as *const crate::QByteArray, i)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```char QByteArray::operator[](unsigned int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn index_uint(&self, i: ::std::os::raw::c_uint) -> ::std::os::raw::c_char {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_operator__1(self as *const crate::QByteArray, i)
    }

    /// <p>Returns the byte at index position <i>i</i> as a modifiable reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteRef QByteArray::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte at index position <i>i</i> as a modifiable reference.</p>
    /// <p>If an assignment is made beyond the end of the byte array, the array is extended with <a href="http://doc.qt.io/qt-5/qbytearray.html#resize">resize</a>() before the assignment takes place.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba;
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> <span class="number">10</span>; <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     ba<span class="operator">[</span>i<span class="operator">]</span> <span class="operator">=</span> <span class="char">'A'</span> <span class="operator">+</span> i;
    ///   <span class="comment">// ba == "ABCDEFGHIJ"</span>
    ///
    /// </pre>
    /// <p>The return value is of type QByteRef, a helper class for <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>. When you get an object of type QByteRef, you can use it as if it were a char &amp;. If you assign to it, the assignment will apply to the character in the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> from which you got the reference.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#at">at</a>().</p></div>
    pub unsafe fn index_int_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QByteRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_operator__2(self as *mut crate::QByteArray, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteRef QByteArray::operator[](unsigned int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#operator-5b-5d-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn index_uint_mut(
        &mut self,
        i: ::std::os::raw::c_uint,
    ) -> ::cpp_utils::CppBox<crate::QByteRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_operator__3(self as *mut crate::QByteArray, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::indexOf(char c, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#indexOf-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the index position of the first occurrence of the character <i>ch</i> in the byte array, searching forward from index position <i>from</i>. Returns -1 if <i>ch</i> could not be found.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba(<span class="string">"ABCBA"</span>);
    ///   ba<span class="operator">.</span>indexOf(<span class="string">"B"</span>);            <span class="comment">// returns 1</span>
    ///   ba<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);         <span class="comment">// returns 1</span>
    ///   ba<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);         <span class="comment">// returns 3</span>
    ///   ba<span class="operator">.</span>indexOf(<span class="string">"X"</span>);            <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_char_int(
        &self,
        c: ::std::os::raw::c_char,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_indexOf(self as *const crate::QByteArray, c, from)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::indexOf(const char* c, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#indexOf-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the index position of the first occurrence of the string <i>str</i> in the byte array, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> could not be found.</p></div>
    pub unsafe fn index_of_char_int2(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_indexOf1(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first occurrence of the byte array <i>ba</i> in this byte array, searching forward from index position <i>from</i>. Returns -1 if <i>ba</i> could not be found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::indexOf(const QByteArray& a, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of the byte array <i>ba</i> in this byte array, searching forward from index position <i>from</i>. Returns -1 if <i>ba</i> could not be found.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> x(<span class="string">"sticky question"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> y(<span class="string">"sti"</span>);
    ///   x<span class="operator">.</span>indexOf(y);               <span class="comment">// returns 0</span>
    ///   x<span class="operator">.</span>indexOf(y<span class="operator">,</span> <span class="number">1</span>);            <span class="comment">// returns 10</span>
    ///   x<span class="operator">.</span>indexOf(y<span class="operator">,</span> <span class="number">10</span>);           <span class="comment">// returns 10</span>
    ///   x<span class="operator">.</span>indexOf(y<span class="operator">,</span> <span class="number">11</span>);           <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#lastIndexOf">lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qbytearray.html#count-3">count</a>().</p></div>
    pub unsafe fn index_of_q_byte_array_int(
        &self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_indexOf2(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(a).as_raw_ptr(),
            from,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::indexOf(const QString& s, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#indexOf-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the index position of the first occurrence of the string <i>str</i> in the byte array, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> could not be found.</p>
    /// <p>The Unicode data is converted into 8-bit characters using <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">QString::toUtf8</a>().</p>
    /// <p>You can disable this function by defining <code>QT_NO_CAST_TO_ASCII</code> when you compile your applications. You then need to call <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">QString::toUtf8</a>() (or <a href="http://doc.qt.io/qt-5/qstring.html#toLatin1">QString::toLatin1</a>() or <a href="http://doc.qt.io/qt-5/qstring.html#toLocal8Bit">QString::toLocal8Bit</a>()) explicitly if you want to convert the data to <code>const char *</code>.</p></div>
    pub unsafe fn index_of_q_string_int(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_indexOf3(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            from,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::indexOf(char c) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#indexOf-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the index position of the first occurrence of the character <i>ch</i> in the byte array, searching forward from index position <i>from</i>. Returns -1 if <i>ch</i> could not be found.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba(<span class="string">"ABCBA"</span>);
    ///   ba<span class="operator">.</span>indexOf(<span class="string">"B"</span>);            <span class="comment">// returns 1</span>
    ///   ba<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);         <span class="comment">// returns 1</span>
    ///   ba<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);         <span class="comment">// returns 3</span>
    ///   ba<span class="operator">.</span>indexOf(<span class="string">"X"</span>);            <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_char(&self, c: ::std::os::raw::c_char) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_indexOf4(self as *const crate::QByteArray, c)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::indexOf(const char* c) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#indexOf-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the index position of the first occurrence of the string <i>str</i> in the byte array, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> could not be found.</p></div>
    pub unsafe fn index_of_char2(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_indexOf5(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the index position of the first occurrence of the byte array <i>ba</i> in this byte array, searching forward from index position <i>from</i>. Returns -1 if <i>ba</i> could not be found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::indexOf(const QByteArray& a) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of the byte array <i>ba</i> in this byte array, searching forward from index position <i>from</i>. Returns -1 if <i>ba</i> could not be found.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> x(<span class="string">"sticky question"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> y(<span class="string">"sti"</span>);
    ///   x<span class="operator">.</span>indexOf(y);               <span class="comment">// returns 0</span>
    ///   x<span class="operator">.</span>indexOf(y<span class="operator">,</span> <span class="number">1</span>);            <span class="comment">// returns 10</span>
    ///   x<span class="operator">.</span>indexOf(y<span class="operator">,</span> <span class="number">10</span>);           <span class="comment">// returns 10</span>
    ///   x<span class="operator">.</span>indexOf(y<span class="operator">,</span> <span class="number">11</span>);           <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#lastIndexOf">lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qbytearray.html#count-3">count</a>().</p></div>
    pub unsafe fn index_of_q_byte_array(
        &self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_indexOf6(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(a).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::indexOf(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#indexOf-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the index position of the first occurrence of the string <i>str</i> in the byte array, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> could not be found.</p>
    /// <p>The Unicode data is converted into 8-bit characters using <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">QString::toUtf8</a>().</p>
    /// <p>You can disable this function by defining <code>QT_NO_CAST_TO_ASCII</code> when you compile your applications. You then need to call <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">QString::toUtf8</a>() (or <a href="http://doc.qt.io/qt-5/qstring.html#toLatin1">QString::toLatin1</a>() or <a href="http://doc.qt.io/qt-5/qstring.html#toLocal8Bit">QString::toLocal8Bit</a>()) explicitly if you want to convert the data to <code>const char *</code>.</p></div>
    pub unsafe fn index_of_q_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_indexOf7(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::insert(int i, char c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#insert-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts character <i>ch</i> at index position <i>i</i> in the byte array. If <i>i</i> is greater than <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>(), the array is first extended using <a href="http://doc.qt.io/qt-5/qbytearray.html#resize">resize</a>().</p></div>
    pub unsafe fn insert_int_char(
        &mut self,
        i: ::std::os::raw::c_int,
        c: ::std::os::raw::c_char,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_insert(self as *mut crate::QByteArray, i, c);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::insert(int i, int count, char c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>count</i> copies of character <i>ch</i> at index position <i>i</i> in the byte array.</p>
    /// <p>If <i>i</i> is greater than <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>(), the array is first extended using <a href="http://doc.qt.io/qt-5/qbytearray.html#resize">resize</a>().</p>
    /// <p>This function was introduced in  Qt 5.7.</p></div>
    pub unsafe fn insert_2_int_char(
        &mut self,
        i: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
        c: ::std::os::raw::c_char,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_insert1(
            self as *mut crate::QByteArray,
            i,
            count,
            c,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::insert(int i, const char* s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#insert-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts the string <i>str</i> at position <i>i</i> in the byte array.</p>
    /// <p>If <i>i</i> is greater than <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>(), the array is first extended using <a href="http://doc.qt.io/qt-5/qbytearray.html#resize">resize</a>().</p></div>
    pub unsafe fn insert_int_char2(
        &mut self,
        i: ::std::os::raw::c_int,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_insert2(
            self as *mut crate::QByteArray,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::insert(int i, const char* s, int len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#insert-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>len</i> bytes of the string <i>str</i> at position <i>i</i> in the byte array.</p>
    /// <p>If <i>i</i> is greater than <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>(), the array is first extended using <a href="http://doc.qt.io/qt-5/qbytearray.html#resize">resize</a>().</p>
    /// <p>This function was introduced in  Qt 4.6.</p></div>
    pub unsafe fn insert_int_char_int(
        &mut self,
        i: ::std::os::raw::c_int,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        len: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_insert3(
            self as *mut crate::QByteArray,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            len,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Inserts the byte array <i>ba</i> at index position <i>i</i> and returns a reference to this byte array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::insert(int i, const QByteArray& a)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts the byte array <i>ba</i> at index position <i>i</i> and returns a reference to this byte array.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba(<span class="string">"Meal"</span>);
    ///   ba<span class="operator">.</span>insert(<span class="number">1</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span>(<span class="string">"ontr"</span>));
    ///   <span class="comment">// ba == "Montreal"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qbytearray.html#remove">remove</a>().</p></div>
    pub unsafe fn insert_int_q_byte_array(
        &mut self,
        i: ::std::os::raw::c_int,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_insert4(
            self as *mut crate::QByteArray,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(a).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::insert(int i, const QString& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#insert-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts the string <i>str</i> at index position <i>i</i> in the byte array. The Unicode data is converted into 8-bit characters using <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">QString::toUtf8</a>().</p>
    /// <p>If <i>i</i> is greater than <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>(), the array is first extended using <a href="http://doc.qt.io/qt-5/qbytearray.html#resize">resize</a>().</p>
    /// <p>You can disable this function by defining <code>QT_NO_CAST_TO_ASCII</code> when you compile your applications. You then need to call <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">QString::toUtf8</a>() (or <a href="http://doc.qt.io/qt-5/qstring.html#toLatin1">QString::toLatin1</a>() or <a href="http://doc.qt.io/qt-5/qstring.html#toLocal8Bit">QString::toLocal8Bit</a>()) explicitly if you want to convert the data to <code>const char *</code>.</p></div>
    pub unsafe fn insert_int_q_string(
        &mut self,
        i: ::std::os::raw::c_int,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_insert5(
            self as *mut crate::QByteArray,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QByteArray::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_isDetached(self as *const crate::QByteArray)
    }

    /// <p>Returns <code>true</code> if the byte array has size 0; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QByteArray::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the byte array has size 0; otherwise returns <code>false</code>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span>()<span class="operator">.</span>isEmpty();         <span class="comment">// returns true</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span>(<span class="string">""</span>)<span class="operator">.</span>isEmpty();       <span class="comment">// returns true</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span>(<span class="string">"abc"</span>)<span class="operator">.</span>isEmpty();    <span class="comment">// returns false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_isEmpty(self as *const crate::QByteArray)
    }

    /// <p>Returns <code>true</code> if this byte array contains only lowercase letters, otherwise returns <code>false</code>. The byte array is interpreted as a Latin-1 encoded string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QByteArray::isLower() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#isLower">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this byte array contains only lowercase letters, otherwise returns <code>false</code>. The byte array is interpreted as a Latin-1 encoded string.</p>
    /// <p>This function was introduced in  Qt 5.12.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#isUpper">isUpper</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#toLower">toLower</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_lower(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_isLower(self as *const crate::QByteArray)
    }

    /// <p>Returns <code>true</code> if this byte array is null; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QByteArray::isNull() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#isNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this byte array is null; otherwise returns <code>false</code>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span>()<span class="operator">.</span>isNull();          <span class="comment">// returns true</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span>(<span class="string">""</span>)<span class="operator">.</span>isNull();        <span class="comment">// returns false</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span>(<span class="string">"abc"</span>)<span class="operator">.</span>isNull();     <span class="comment">// returns false</span>
    ///
    /// </pre>
    /// <p>Qt makes a distinction between null byte arrays and empty byte arrays for historical reasons. For most applications, what matters is whether or not a byte array contains any data, and this can be determined using <a href="http://doc.qt.io/qt-5/qbytearray.html#isEmpty">isEmpty</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_isNull(self as *const crate::QByteArray)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QByteArray::isSharedWith(const QByteArray& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_isSharedWith(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if this byte array contains only uppercase letters, otherwise returns <code>false</code>. The byte array is interpreted as a Latin-1 encoded string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QByteArray::isUpper() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#isUpper">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this byte array contains only uppercase letters, otherwise returns <code>false</code>. The byte array is interpreted as a Latin-1 encoded string.</p>
    /// <p>This function was introduced in  Qt 5.12.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#isLower">isLower</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#toUpper">toUpper</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_upper(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_isUpper(self as *const crate::QByteArray)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::lastIndexOf(char c, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#lastIndexOf-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the index position of the last occurrence of character <i>ch</i> in the byte array, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last (<a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>() - 1) byte. Returns -1 if <i>ch</i> could not be found.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba(<span class="string">"ABCBA"</span>);
    ///   ba<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);        <span class="comment">// returns 3</span>
    ///   ba<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);     <span class="comment">// returns 3</span>
    ///   ba<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);     <span class="comment">// returns 1</span>
    ///   ba<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#contains">contains</a>().</p></div>
    pub unsafe fn last_index_of_char_int(
        &self,
        c: ::std::os::raw::c_char,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_lastIndexOf(
            self as *const crate::QByteArray,
            c,
            from,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::lastIndexOf(const char* c, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#lastIndexOf-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the index position of the last occurrence of the string <i>str</i> in the byte array, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last (<a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>() - 1) byte. Returns -1 if <i>str</i> could not be found.</p></div>
    pub unsafe fn last_index_of_char_int2(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_lastIndexOf1(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the last occurrence of the byte array <i>ba</i> in this byte array, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last byte. Returns -1 if <i>ba</i> could not be found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::lastIndexOf(const QByteArray& a, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the byte array <i>ba</i> in this byte array, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last byte. Returns -1 if <i>ba</i> could not be found.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> x(<span class="string">"crazy azimuths"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> y(<span class="string">"az"</span>);
    ///   x<span class="operator">.</span>lastIndexOf(y);           <span class="comment">// returns 6</span>
    ///   x<span class="operator">.</span>lastIndexOf(y<span class="operator">,</span> <span class="number">6</span>);        <span class="comment">// returns 6</span>
    ///   x<span class="operator">.</span>lastIndexOf(y<span class="operator">,</span> <span class="number">5</span>);        <span class="comment">// returns 2</span>
    ///   x<span class="operator">.</span>lastIndexOf(y<span class="operator">,</span> <span class="number">1</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qbytearray.html#count-3">count</a>().</p></div>
    pub unsafe fn last_index_of_q_byte_array_int(
        &self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_lastIndexOf2(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(a).as_raw_ptr(),
            from,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::lastIndexOf(const QString& s, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#lastIndexOf-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the index position of the last occurrence of the string <i>str</i> in the byte array, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last (<a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>() - 1) byte. Returns -1 if <i>str</i> could not be found.</p>
    /// <p>The Unicode data is converted into 8-bit characters using <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">QString::toUtf8</a>().</p>
    /// <p>You can disable this function by defining <code>QT_NO_CAST_TO_ASCII</code> when you compile your applications. You then need to call <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">QString::toUtf8</a>() (or <a href="http://doc.qt.io/qt-5/qstring.html#toLatin1">QString::toLatin1</a>() or <a href="http://doc.qt.io/qt-5/qstring.html#toLocal8Bit">QString::toLocal8Bit</a>()) explicitly if you want to convert the data to <code>const char *</code>.</p></div>
    pub unsafe fn last_index_of_q_string_int(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_lastIndexOf3(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            from,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::lastIndexOf(char c) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#lastIndexOf-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the index position of the last occurrence of character <i>ch</i> in the byte array, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last (<a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>() - 1) byte. Returns -1 if <i>ch</i> could not be found.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba(<span class="string">"ABCBA"</span>);
    ///   ba<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);        <span class="comment">// returns 3</span>
    ///   ba<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);     <span class="comment">// returns 3</span>
    ///   ba<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);     <span class="comment">// returns 1</span>
    ///   ba<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#contains">contains</a>().</p></div>
    pub unsafe fn last_index_of_char(&self, c: ::std::os::raw::c_char) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_lastIndexOf4(self as *const crate::QByteArray, c)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::lastIndexOf(const char* c) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#lastIndexOf-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the index position of the last occurrence of the string <i>str</i> in the byte array, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last (<a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>() - 1) byte. Returns -1 if <i>str</i> could not be found.</p></div>
    pub unsafe fn last_index_of_char2(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_lastIndexOf5(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the index position of the last occurrence of the byte array <i>ba</i> in this byte array, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last byte. Returns -1 if <i>ba</i> could not be found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::lastIndexOf(const QByteArray& a) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the byte array <i>ba</i> in this byte array, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last byte. Returns -1 if <i>ba</i> could not be found.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> x(<span class="string">"crazy azimuths"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> y(<span class="string">"az"</span>);
    ///   x<span class="operator">.</span>lastIndexOf(y);           <span class="comment">// returns 6</span>
    ///   x<span class="operator">.</span>lastIndexOf(y<span class="operator">,</span> <span class="number">6</span>);        <span class="comment">// returns 6</span>
    ///   x<span class="operator">.</span>lastIndexOf(y<span class="operator">,</span> <span class="number">5</span>);        <span class="comment">// returns 2</span>
    ///   x<span class="operator">.</span>lastIndexOf(y<span class="operator">,</span> <span class="number">1</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qbytearray.html#count-3">count</a>().</p></div>
    pub unsafe fn last_index_of_q_byte_array(
        &self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_lastIndexOf6(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(a).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::lastIndexOf(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#lastIndexOf-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the index position of the last occurrence of the string <i>str</i> in the byte array, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last (<a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>() - 1) byte. Returns -1 if <i>str</i> could not be found.</p>
    /// <p>The Unicode data is converted into 8-bit characters using <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">QString::toUtf8</a>().</p>
    /// <p>You can disable this function by defining <code>QT_NO_CAST_TO_ASCII</code> when you compile your applications. You then need to call <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">QString::toUtf8</a>() (or <a href="http://doc.qt.io/qt-5/qstring.html#toLatin1">QString::toLatin1</a>() or <a href="http://doc.qt.io/qt-5/qstring.html#toLocal8Bit">QString::toLocal8Bit</a>()) explicitly if you want to convert the data to <code>const char *</code>.</p></div>
    pub unsafe fn last_index_of_q_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_lastIndexOf7(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Returns a byte array that contains the leftmost <i>len</i> bytes of this byte array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::left(int len) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#left">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a byte array that contains the leftmost <i>len</i> bytes of this byte array.</p>
    /// <p>The entire byte array is returned if <i>len</i> is greater than <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> x(<span class="string">"Pineapple"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> y <span class="operator">=</span> x<span class="operator">.</span>left(<span class="number">4</span>);
    ///   <span class="comment">// y == "Pine"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#right">right</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#mid">mid</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#startsWith">startsWith</a>(), and <a href="http://doc.qt.io/qt-5/qbytearray.html#truncate">truncate</a>().</p></div>
    pub unsafe fn left(
        &self,
        len: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_left(self as *const crate::QByteArray, len);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a byte array of size <i>width</i> that contains this byte array padded by the <i>fill</i> character.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::leftJustified(int width, char fill = …, bool truncate = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#leftJustified">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a byte array of size <i>width</i> that contains this byte array padded by the <i>fill</i> character.</p>
    /// <p>If <i>truncate</i> is false and the <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>() of the byte array is more than <i>width</i>, then the returned byte array is a copy of this byte array.</p>
    /// <p>If <i>truncate</i> is true and the <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>() of the byte array is more than <i>width</i>, then any bytes in a copy of the byte array after position <i>width</i> are removed, and the copy is returned.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> x(<span class="string">"apple"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> y <span class="operator">=</span> x<span class="operator">.</span>leftJustified(<span class="number">8</span><span class="operator">,</span> <span class="char">'.'</span>);   <span class="comment">// y == "apple..."</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#rightJustified">rightJustified</a>().</p></div>
    pub unsafe fn left_justified_3a(
        &self,
        width: ::std::os::raw::c_int,
        fill: ::std::os::raw::c_char,
        truncate: bool,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_leftJustified(
            self as *const crate::QByteArray,
            width,
            fill,
            truncate,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a byte array of size <i>width</i> that contains this byte array padded by the <i>fill</i> character.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::leftJustified(int width, char fill = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#leftJustified">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a byte array of size <i>width</i> that contains this byte array padded by the <i>fill</i> character.</p>
    /// <p>If <i>truncate</i> is false and the <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>() of the byte array is more than <i>width</i>, then the returned byte array is a copy of this byte array.</p>
    /// <p>If <i>truncate</i> is true and the <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>() of the byte array is more than <i>width</i>, then any bytes in a copy of the byte array after position <i>width</i> are removed, and the copy is returned.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> x(<span class="string">"apple"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> y <span class="operator">=</span> x<span class="operator">.</span>leftJustified(<span class="number">8</span><span class="operator">,</span> <span class="char">'.'</span>);   <span class="comment">// y == "apple..."</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#rightJustified">rightJustified</a>().</p></div>
    pub unsafe fn left_justified_2a(
        &self,
        width: ::std::os::raw::c_int,
        fill: ::std::os::raw::c_char,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_leftJustified1(
            self as *const crate::QByteArray,
            width,
            fill,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a byte array of size <i>width</i> that contains this byte array padded by the <i>fill</i> character.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::leftJustified(int width) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#leftJustified">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a byte array of size <i>width</i> that contains this byte array padded by the <i>fill</i> character.</p>
    /// <p>If <i>truncate</i> is false and the <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>() of the byte array is more than <i>width</i>, then the returned byte array is a copy of this byte array.</p>
    /// <p>If <i>truncate</i> is true and the <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>() of the byte array is more than <i>width</i>, then any bytes in a copy of the byte array after position <i>width</i> are removed, and the copy is returned.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> x(<span class="string">"apple"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> y <span class="operator">=</span> x<span class="operator">.</span>leftJustified(<span class="number">8</span><span class="operator">,</span> <span class="char">'.'</span>);   <span class="comment">// y == "apple..."</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#rightJustified">rightJustified</a>().</p></div>
    pub unsafe fn left_justified_1a(
        &self,
        width: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_leftJustified2(
            self as *const crate::QByteArray,
            width,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Same as <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_length(self as *const crate::QByteArray)
    }

    /// <p>Returns a byte array containing <i>len</i> bytes from this byte array, starting at position <i>pos</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::mid(int index, int len = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a byte array containing <i>len</i> bytes from this byte array, starting at position <i>pos</i>.</p>
    /// <p>If <i>len</i> is -1 (the default), or <i>pos</i> + <i>len</i> &gt;= <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>(), returns a byte array containing all bytes starting at position <i>pos</i> until the end of the byte array.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> x(<span class="string">"Five pineapples"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> y <span class="operator">=</span> x<span class="operator">.</span>mid(<span class="number">5</span><span class="operator">,</span> <span class="number">4</span>);     <span class="comment">// y == "pine"</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> z <span class="operator">=</span> x<span class="operator">.</span>mid(<span class="number">5</span>);        <span class="comment">// z == "pineapples"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#left">left</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#right">right</a>().</p></div>
    pub unsafe fn mid_2a(
        &self,
        index: ::std::os::raw::c_int,
        len: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_mid(
            self as *const crate::QByteArray,
            index,
            len,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a byte array containing <i>len</i> bytes from this byte array, starting at position <i>pos</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::mid(int index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a byte array containing <i>len</i> bytes from this byte array, starting at position <i>pos</i>.</p>
    /// <p>If <i>len</i> is -1 (the default), or <i>pos</i> + <i>len</i> &gt;= <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>(), returns a byte array containing all bytes starting at position <i>pos</i> until the end of the byte array.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> x(<span class="string">"Five pineapples"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> y <span class="operator">=</span> x<span class="operator">.</span>mid(<span class="number">5</span><span class="operator">,</span> <span class="number">4</span>);     <span class="comment">// y == "pine"</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> z <span class="operator">=</span> x<span class="operator">.</span>mid(<span class="number">5</span>);        <span class="comment">// z == "pineapples"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#left">left</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#right">right</a>().</p></div>
    pub unsafe fn mid_1a(
        &self,
        index: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_mid1(self as *const crate::QByteArray, index);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs an empty byte array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QByteArray::QByteArray()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty byte array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_QByteArray();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a byte array containing the first <i>size</i> bytes of array <i>data</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QByteArray::QByteArray(const char* arg1, int size = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a byte array containing the first <i>size</i> bytes of array <i>data</i>.</p>
    /// <p>If <i>data</i> is 0, a null byte array is constructed.</p>
    /// <p>If <i>size</i> is negative, <i>data</i> is assumed to point to a nul-terminated string and its length is determined dynamically. The terminating nul-character is not considered part of the byte array.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> makes a deep copy of the string data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#fromRawData">fromRawData</a>().</p></div>
    pub unsafe fn from_char_int(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_QByteArray1(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
            size,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a byte array of size <i>size</i> with every byte set to character <i>ch</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QByteArray::QByteArray(int size, char c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a byte array of size <i>size</i> with every byte set to character <i>ch</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#fill">fill</a>().</p></div>
    pub unsafe fn from_int_char(
        size: ::std::os::raw::c_int,
        c: ::std::os::raw::c_char,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_QByteArray2(size, c);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a byte array containing the first <i>size</i> bytes of array <i>data</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QByteArray::QByteArray(const char* arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a byte array containing the first <i>size</i> bytes of array <i>data</i>.</p>
    /// <p>If <i>data</i> is 0, a null byte array is constructed.</p>
    /// <p>If <i>size</i> is negative, <i>data</i> is assumed to point to a nul-terminated string and its length is determined dynamically. The terminating nul-character is not considered part of the byte array.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> makes a deep copy of the string data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#fromRawData">fromRawData</a>().</p></div>
    pub unsafe fn from_char(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_QByteArray5(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QByteArray::QByteArray(const QByteArray& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#constant-time">constant time</a>, because <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), taking <a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_QByteArray3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a byte array containing the string equivalent of the number <i>n</i> to base <i>base</i> (10 by default). The <i>base</i> can be any value between 2 and 36.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QByteArray::number(int arg1, int base = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#number">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a byte array containing the string equivalent of the number <i>n</i> to base <i>base</i> (10 by default). The <i>base</i> can be any value between 2 and 36.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">int</span> n <span class="operator">=</span> <span class="number">63</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span><span class="operator">::</span>number(n);              <span class="comment">// returns "63"</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span><span class="operator">::</span>number(n<span class="operator">,</span> <span class="number">16</span>);          <span class="comment">// returns "3f"</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span><span class="operator">::</span>number(n<span class="operator">,</span> <span class="number">16</span>)<span class="operator">.</span>toUpper();  <span class="comment">// returns "3F"</span>
    ///
    /// </pre>
    /// <p><b>Note: </b>The format of the number is not localized; the default C locale is used irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#setNum">setNum</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#toInt">toInt</a>().</p></div>
    pub unsafe fn number_2_int(
        arg1: ::std::os::raw::c_int,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_number(arg1, base);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QByteArray::number(unsigned int arg1, int base = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#number-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toUInt">toUInt</a>().</p></div>
    pub unsafe fn number_uint_int(
        arg1: ::std::os::raw::c_uint,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_number1(arg1, base);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QByteArray::number(qlonglong arg1, int base = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#number-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toLongLong">toLongLong</a>().</p></div>
    pub unsafe fn number_i64_int(
        arg1: i64,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_number2(arg1, base);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QByteArray::number(qulonglong arg1, int base = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#number-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toULongLong">toULongLong</a>().</p></div>
    pub unsafe fn number_u64_int(
        arg1: u64,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_number3(arg1, base);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QByteArray::number(double arg1, char f = …, int prec = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#number-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a byte array that contains the printed value of <i>n</i>, formatted in format <i>f</i> with precision <i>prec</i>.</p>
    /// <p>Argument <i>n</i> is formatted according to the <i>f</i> format specified, which is <code>g</code> by default, and can be any of the following:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Format</th><th>Meaning</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td><code>e</code></td><td>format as [-]9.9e[+|-]999</td></tr>
    /// <tr valign="top" class="even"><td><code>E</code></td><td>format as [-]9.9E[+|-]999</td></tr>
    /// <tr valign="top" class="odd"><td><code>f</code></td><td>format as [-]9.9</td></tr>
    /// <tr valign="top" class="even"><td><code>g</code></td><td>use <code>e</code> or <code>f</code> format, whichever is the most concise</td></tr>
    /// <tr valign="top" class="odd"><td><code>G</code></td><td>use <code>E</code> or <code>f</code> format, whichever is the most concise</td></tr>
    /// </tbody></table></div>
    /// <p>With 'e', 'E', and 'f', <i>prec</i> is the number of digits after the decimal point. With 'g' and 'G', <i>prec</i> is the maximum number of significant digits (trailing zeroes are omitted).</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span><span class="operator">::</span>number(<span class="number">12.3456</span><span class="operator">,</span> <span class="char">'E'</span><span class="operator">,</span> <span class="number">3</span>);
    ///   <span class="comment">// ba == 1.235E+01</span>
    ///
    /// </pre>
    /// <p><b>Note: </b>The format of the number is not localized; the default C locale is used irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toDouble">toDouble</a>().</p></div>
    pub unsafe fn number_double_char_int(
        arg1: ::std::os::raw::c_double,
        f: ::std::os::raw::c_char,
        prec: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_number4(arg1, f, prec);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a byte array containing the string equivalent of the number <i>n</i> to base <i>base</i> (10 by default). The <i>base</i> can be any value between 2 and 36.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QByteArray::number(int arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#number">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a byte array containing the string equivalent of the number <i>n</i> to base <i>base</i> (10 by default). The <i>base</i> can be any value between 2 and 36.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">int</span> n <span class="operator">=</span> <span class="number">63</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span><span class="operator">::</span>number(n);              <span class="comment">// returns "63"</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span><span class="operator">::</span>number(n<span class="operator">,</span> <span class="number">16</span>);          <span class="comment">// returns "3f"</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span><span class="operator">::</span>number(n<span class="operator">,</span> <span class="number">16</span>)<span class="operator">.</span>toUpper();  <span class="comment">// returns "3F"</span>
    ///
    /// </pre>
    /// <p><b>Note: </b>The format of the number is not localized; the default C locale is used irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#setNum">setNum</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#toInt">toInt</a>().</p></div>
    pub unsafe fn number_int(
        arg1: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_number5(arg1);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QByteArray::number(unsigned int arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#number-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toUInt">toUInt</a>().</p></div>
    pub unsafe fn number_uint(
        arg1: ::std::os::raw::c_uint,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_number6(arg1);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QByteArray::number(qlonglong arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#number-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toLongLong">toLongLong</a>().</p></div>
    pub unsafe fn number_i64(arg1: i64) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_number7(arg1);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QByteArray::number(qulonglong arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#number-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toULongLong">toULongLong</a>().</p></div>
    pub unsafe fn number_u64(arg1: u64) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_number8(arg1);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QByteArray::number(double arg1, char f = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#number-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a byte array that contains the printed value of <i>n</i>, formatted in format <i>f</i> with precision <i>prec</i>.</p>
    /// <p>Argument <i>n</i> is formatted according to the <i>f</i> format specified, which is <code>g</code> by default, and can be any of the following:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Format</th><th>Meaning</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td><code>e</code></td><td>format as [-]9.9e[+|-]999</td></tr>
    /// <tr valign="top" class="even"><td><code>E</code></td><td>format as [-]9.9E[+|-]999</td></tr>
    /// <tr valign="top" class="odd"><td><code>f</code></td><td>format as [-]9.9</td></tr>
    /// <tr valign="top" class="even"><td><code>g</code></td><td>use <code>e</code> or <code>f</code> format, whichever is the most concise</td></tr>
    /// <tr valign="top" class="odd"><td><code>G</code></td><td>use <code>E</code> or <code>f</code> format, whichever is the most concise</td></tr>
    /// </tbody></table></div>
    /// <p>With 'e', 'E', and 'f', <i>prec</i> is the number of digits after the decimal point. With 'g' and 'G', <i>prec</i> is the maximum number of significant digits (trailing zeroes are omitted).</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span><span class="operator">::</span>number(<span class="number">12.3456</span><span class="operator">,</span> <span class="char">'E'</span><span class="operator">,</span> <span class="number">3</span>);
    ///   <span class="comment">// ba == 1.235E+01</span>
    ///
    /// </pre>
    /// <p><b>Note: </b>The format of the number is not localized; the default C locale is used irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toDouble">toDouble</a>().</p></div>
    pub unsafe fn number_double_char(
        arg1: ::std::os::raw::c_double,
        f: ::std::os::raw::c_char,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_number9(arg1, f);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QByteArray::number(double arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#number-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a byte array that contains the printed value of <i>n</i>, formatted in format <i>f</i> with precision <i>prec</i>.</p>
    /// <p>Argument <i>n</i> is formatted according to the <i>f</i> format specified, which is <code>g</code> by default, and can be any of the following:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Format</th><th>Meaning</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td><code>e</code></td><td>format as [-]9.9e[+|-]999</td></tr>
    /// <tr valign="top" class="even"><td><code>E</code></td><td>format as [-]9.9E[+|-]999</td></tr>
    /// <tr valign="top" class="odd"><td><code>f</code></td><td>format as [-]9.9</td></tr>
    /// <tr valign="top" class="even"><td><code>g</code></td><td>use <code>e</code> or <code>f</code> format, whichever is the most concise</td></tr>
    /// <tr valign="top" class="odd"><td><code>G</code></td><td>use <code>E</code> or <code>f</code> format, whichever is the most concise</td></tr>
    /// </tbody></table></div>
    /// <p>With 'e', 'E', and 'f', <i>prec</i> is the number of digits after the decimal point. With 'g' and 'G', <i>prec</i> is the maximum number of significant digits (trailing zeroes are omitted).</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span><span class="operator">::</span>number(<span class="number">12.3456</span><span class="operator">,</span> <span class="char">'E'</span><span class="operator">,</span> <span class="number">3</span>);
    ///   <span class="comment">// ba == 1.235E+01</span>
    ///
    /// </pre>
    /// <p><b>Note: </b>The format of the number is not localized; the default C locale is used irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toDouble">toDouble</a>().</p></div>
    pub unsafe fn number_double(
        arg1: ::std::os::raw::c_double,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_number10(arg1);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::prepend(char c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#prepend-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Prepends the character <i>ch</i> to this byte array.</p></div>
    pub unsafe fn prepend_char(
        &mut self,
        c: ::std::os::raw::c_char,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_prepend(self as *mut crate::QByteArray, c);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::prepend(int count, char c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#prepend-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Prepends <i>count</i> copies of character <i>ch</i> to this byte array.</p>
    /// <p>This function was introduced in  Qt 5.7.</p></div>
    pub unsafe fn prepend_int_char(
        &mut self,
        count: ::std::os::raw::c_int,
        c: ::std::os::raw::c_char,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_prepend1(
            self as *mut crate::QByteArray,
            count,
            c,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::prepend(const char* s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#prepend-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Prepends the string <i>str</i> to this byte array.</p></div>
    pub unsafe fn prepend_char2(
        &mut self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_prepend2(
            self as *mut crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::prepend(const char* s, int len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#prepend-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Prepends <i>len</i> bytes of the string <i>str</i> to this byte array.</p>
    /// <p>This function was introduced in  Qt 4.6.</p></div>
    pub unsafe fn prepend_char_int(
        &mut self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        len: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_prepend3(
            self as *mut crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            len,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Prepends the byte array <i>ba</i> to this byte array and returns a reference to this byte array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::prepend(const QByteArray& a)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Prepends the byte array <i>ba</i> to this byte array and returns a reference to this byte array.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> x(<span class="string">"ship"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> y(<span class="string">"air"</span>);
    ///   x<span class="operator">.</span>prepend(y);
    ///   <span class="comment">// x == "airship"</span>
    ///
    /// </pre>
    /// <p>This is the same as insert(0, <i>ba</i>).</p>
    /// <p>Note: <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> is an <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a> class. Consequently, if you prepend to an empty byte array, then the byte array will just share the data held in <i>ba</i>. In this case, no copying of data is done, taking <a href="http://doc.qt.io/qt-5/containers.html#constant-time">constant time</a>. If a shared instance is modified, it will be copied (copy-on-write), taking <a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>.</p>
    /// <p>If the byte array being prepended to is not empty, a deep copy of the data is performed, taking <a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend_q_byte_array(
        &mut self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_prepend4(
            self as *mut crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(a).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QByteArray::push_back(char c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#push_back-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as append(<i>ch</i>).</p></div>
    pub unsafe fn push_back_char(&mut self, c: ::std::os::raw::c_char) {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_push_back(self as *mut crate::QByteArray, c)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QByteArray::push_back(const char* c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#push_back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as append(<i>str</i>).</p></div>
    pub unsafe fn push_back_char2(
        &mut self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_push_back1(
            self as *mut crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to append(<i>other</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QByteArray::push_back(const QByteArray& a)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to append(<i>other</i>).</p></div>
    pub unsafe fn push_back_q_byte_array(
        &mut self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_push_back2(
            self as *mut crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(a).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QByteArray::push_front(char c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#push_front-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as prepend(<i>ch</i>).</p></div>
    pub unsafe fn push_front_char(&mut self, c: ::std::os::raw::c_char) {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_push_front(self as *mut crate::QByteArray, c)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QByteArray::push_front(const char* c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#push_front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as prepend(<i>str</i>).</p></div>
    pub unsafe fn push_front_char2(
        &mut self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_push_front1(
            self as *mut crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to prepend(<i>other</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QByteArray::push_front(const QByteArray& a)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to prepend(<i>other</i>).</p></div>
    pub unsafe fn push_front_q_byte_array(
        &mut self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_push_front2(
            self as *mut crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(a).as_raw_ptr(),
        )
    }

    /// <p>Removes <i>len</i> bytes from the array, starting at index position <i>pos</i>, and returns a reference to the array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::remove(int index, int len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#remove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes <i>len</i> bytes from the array, starting at index position <i>pos</i>, and returns a reference to the array.</p>
    /// <p>If <i>pos</i> is out of range, nothing happens. If <i>pos</i> is valid, but <i>pos</i> + <i>len</i> is larger than the size of the array, the array is truncated at position <i>pos</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba(<span class="string">"Montreal"</span>);
    ///   ba<span class="operator">.</span>remove(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// ba == "Meal"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#replace">replace</a>().</p></div>
    pub unsafe fn remove(
        &mut self,
        index: ::std::os::raw::c_int,
        len: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_remove(
            self as *mut crate::QByteArray,
            index,
            len,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a copy of this byte array repeated the specified number of <i>times</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::repeated(int times) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#repeated">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a copy of this byte array repeated the specified number of <i>times</i>.</p>
    /// <p>If <i>times</i> is less than 1, an empty byte array is returned.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba(<span class="string">"ab"</span>);
    ///   ba<span class="operator">.</span>repeated(<span class="number">4</span>);             <span class="comment">// returns "abababab"</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn repeated(
        &self,
        times: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_repeated(
            self as *const crate::QByteArray,
            times,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::replace(int index, int len, const char* s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#replace-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Replaces <i>len</i> bytes from index position <i>pos</i> with the zero terminated string <i>after</i>.</p>
    /// <p>Notice: this can change the length of the byte array.</p></div>
    pub unsafe fn replace_2_int_char(
        &mut self,
        index: ::std::os::raw::c_int,
        len: ::std::os::raw::c_int,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_replace(
            self as *mut crate::QByteArray,
            index,
            len,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::replace(int index, int len, const char* s, int alen)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#replace-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Replaces <i>len</i> bytes from index position <i>pos</i> with <i>alen</i> bytes from the string <i>after</i>. <i>after</i> is allowed to have '\0' characters.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn replace_2_int_char_int(
        &mut self,
        index: ::std::os::raw::c_int,
        len: ::std::os::raw::c_int,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        alen: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_replace1(
            self as *mut crate::QByteArray,
            index,
            len,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            alen,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Replaces <i>len</i> bytes from index position <i>pos</i> with the byte array <i>after</i>, and returns a reference to this byte array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::replace(int index, int len, const QByteArray& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces <i>len</i> bytes from index position <i>pos</i> with the byte array <i>after</i>, and returns a reference to this byte array.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> x(<span class="string">"Say yes!"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> y(<span class="string">"no"</span>);
    ///   x<span class="operator">.</span>replace(<span class="number">4</span><span class="operator">,</span> <span class="number">3</span><span class="operator">,</span> y);
    ///   <span class="comment">// x == "Say no!"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#remove">remove</a>().</p></div>
    pub unsafe fn replace_2_int_q_byte_array(
        &mut self,
        index: ::std::os::raw::c_int,
        len: ::std::os::raw::c_int,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_replace2(
            self as *mut crate::QByteArray,
            index,
            len,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(s).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::replace(char before, const char* after)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#replace-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Replaces every occurrence of the character <i>before</i> with the string <i>after</i>.</p></div>
    pub unsafe fn replace_char_char(
        &mut self,
        before: ::std::os::raw::c_char,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_replace3(
            self as *mut crate::QByteArray,
            before,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(after)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::replace(char before, const QByteArray& after)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#replace-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Replaces every occurrence of the character <i>before</i> with the byte array <i>after</i>.</p></div>
    pub unsafe fn replace_char_q_byte_array(
        &mut self,
        before: ::std::os::raw::c_char,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_replace4(
            self as *mut crate::QByteArray,
            before,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(after)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::replace(const char* before, const char* after)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#replace-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Replaces every occurrence of the string <i>before</i> with the string <i>after</i>.</p></div>
    pub unsafe fn replace_2_char(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_replace5(
            self as *mut crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(before)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(after)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::replace(const char* before, int bsize, const char* after, int asize)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#replace-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Replaces every occurrence of the string <i>before</i> with the string <i>after</i>. Since the sizes of the strings are given by <i>bsize</i> and <i>asize</i>, they may contain zero characters and do not need to be zero-terminated.</p></div>
    pub unsafe fn replace_char_int_char_int(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        bsize: ::std::os::raw::c_int,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        asize: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_replace6(
            self as *mut crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(before)
                .as_raw_ptr(),
            bsize,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(after)
                .as_raw_ptr(),
            asize,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::replace(const QByteArray& before, const QByteArray& after)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#replace-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Replaces every occurrence of the byte array <i>before</i> with the byte array <i>after</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba(<span class="string">"colour behaviour flavour neighbour"</span>);
    ///   ba<span class="operator">.</span>replace(<span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span>(<span class="string">"ou"</span>)<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span>(<span class="string">"o"</span>));
    ///   <span class="comment">// ba == "color behavior flavor neighbor"</span>
    ///
    /// </pre></div>
    pub unsafe fn replace_2_q_byte_array(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_replace7(
            self as *mut crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(before)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(after)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::replace(const QByteArray& before, const char* after)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#replace-8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Replaces every occurrence of the byte array <i>before</i> with the string <i>after</i>.</p></div>
    pub unsafe fn replace_q_byte_array_char(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_replace8(
            self as *mut crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(before)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(after)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::replace(const char* before, const QByteArray& after)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#replace-9">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Replaces every occurrence of the string <i>before</i> with the byte array <i>after</i>.</p></div>
    pub unsafe fn replace_char_q_byte_array2(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_replace9(
            self as *mut crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(before)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(after)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::replace(char before, char after)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#replace-10">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Replaces every occurrence of the character <i>before</i> with the character <i>after</i>.</p></div>
    pub unsafe fn replace_2_char2(
        &mut self,
        before: ::std::os::raw::c_char,
        after: ::std::os::raw::c_char,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_replace10(
            self as *mut crate::QByteArray,
            before,
            after,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::replace(const QString& before, const char* after)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#replace-11">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Replaces every occurrence of the string <i>before</i> with the string <i>after</i>.</p></div>
    pub unsafe fn replace_q_string_char(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_replace11(
            self as *mut crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(before)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(after)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::replace(char c, const QString& after)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#replace-12">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Replaces every occurrence of the character <i>before</i> with the string <i>after</i>. The Unicode data is converted into 8-bit characters using <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">QString::toUtf8</a>().</p>
    /// <p>You can disable this function by defining <code>QT_NO_CAST_TO_ASCII</code> when you compile your applications. You then need to call <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">QString::toUtf8</a>() (or <a href="http://doc.qt.io/qt-5/qstring.html#toLatin1">QString::toLatin1</a>() or <a href="http://doc.qt.io/qt-5/qstring.html#toLocal8Bit">QString::toLocal8Bit</a>()) explicitly if you want to convert the data to <code>const char *</code>.</p></div>
    pub unsafe fn replace_char_q_string(
        &mut self,
        c: ::std::os::raw::c_char,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_replace12(
            self as *mut crate::QByteArray,
            c,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(after)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::replace(const QString& before, const QByteArray& after)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#replace-13">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Replaces every occurrence of the string <i>before</i> with the byte array <i>after</i>. The Unicode data is converted into 8-bit characters using <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">QString::toUtf8</a>().</p>
    /// <p>You can disable this function by defining <code>QT_NO_CAST_TO_ASCII</code> when you compile your applications. You then need to call <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">QString::toUtf8</a>() (or <a href="http://doc.qt.io/qt-5/qstring.html#toLatin1">QString::toLatin1</a>() or <a href="http://doc.qt.io/qt-5/qstring.html#toLocal8Bit">QString::toLocal8Bit</a>()) explicitly if you want to convert the data to <code>const char *</code>.</p></div>
    pub unsafe fn replace_q_string_q_byte_array(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_replace13(
            self as *mut crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(before)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(after)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Attempts to allocate memory for at least <i>size</i> bytes. If you know in advance how large the byte array will be, you can call this function, and if you call <a href="http://doc.qt.io/qt-5/qbytearray.html#resize">resize</a>() often you are likely to get better performance. If <i>size</i> is an underestimate, the worst that will happen is that the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> will be a bit slower.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QByteArray::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to allocate memory for at least <i>size</i> bytes. If you know in advance how large the byte array will be, you can call this function, and if you call <a href="http://doc.qt.io/qt-5/qbytearray.html#resize">resize</a>() often you are likely to get better performance. If <i>size</i> is an underestimate, the worst that will happen is that the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> will be a bit slower.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>'s memory usage. In general, you will rarely ever need to call this function. If you want to change the size of the byte array, call <a href="http://doc.qt.io/qt-5/qbytearray.html#resize">resize</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#squeeze">squeeze</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#capacity">capacity</a>().</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_reserve(self as *mut crate::QByteArray, size)
    }

    /// <p>Sets the size of the byte array to <i>size</i> bytes.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QByteArray::resize(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#resize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the size of the byte array to <i>size</i> bytes.</p>
    /// <p>If <i>size</i> is greater than the current size, the byte array is extended to make it <i>size</i> bytes with the extra bytes added to the end. The new bytes are uninitialized.</p>
    /// <p>If <i>size</i> is less than the current size, bytes are removed from the end.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#truncate">truncate</a>().</p></div>
    pub unsafe fn resize(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_resize(self as *mut crate::QByteArray, size)
    }

    /// <p>Returns a byte array that contains the rightmost <i>len</i> bytes of this byte array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::right(int len) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#right">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a byte array that contains the rightmost <i>len</i> bytes of this byte array.</p>
    /// <p>The entire byte array is returned if <i>len</i> is greater than <a href="http://doc.qt.io/qt-5/qbytearray.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> x(<span class="string">"Pineapple"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> y <span class="operator">=</span> x<span class="operator">.</span>right(<span class="number">5</span>);
    ///   <span class="comment">// y == "apple"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#endsWith">endsWith</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#left">left</a>(), and <a href="http://doc.qt.io/qt-5/qbytearray.html#mid">mid</a>().</p></div>
    pub unsafe fn right(
        &self,
        len: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_right(self as *const crate::QByteArray, len);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a byte array of size <i>width</i> that contains the <i>fill</i> character followed by this byte array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::rightJustified(int width, char fill = …, bool truncate = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#rightJustified">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a byte array of size <i>width</i> that contains the <i>fill</i> character followed by this byte array.</p>
    /// <p>If <i>truncate</i> is false and the size of the byte array is more than <i>width</i>, then the returned byte array is a copy of this byte array.</p>
    /// <p>If <i>truncate</i> is true and the size of the byte array is more than <i>width</i>, then the resulting byte array is truncated at position <i>width</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> x(<span class="string">"apple"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> y <span class="operator">=</span> x<span class="operator">.</span>rightJustified(<span class="number">8</span><span class="operator">,</span> <span class="char">'.'</span>);    <span class="comment">// y == "...apple"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#leftJustified">leftJustified</a>().</p></div>
    pub unsafe fn right_justified_3a(
        &self,
        width: ::std::os::raw::c_int,
        fill: ::std::os::raw::c_char,
        truncate: bool,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_rightJustified(
            self as *const crate::QByteArray,
            width,
            fill,
            truncate,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a byte array of size <i>width</i> that contains the <i>fill</i> character followed by this byte array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::rightJustified(int width, char fill = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#rightJustified">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a byte array of size <i>width</i> that contains the <i>fill</i> character followed by this byte array.</p>
    /// <p>If <i>truncate</i> is false and the size of the byte array is more than <i>width</i>, then the returned byte array is a copy of this byte array.</p>
    /// <p>If <i>truncate</i> is true and the size of the byte array is more than <i>width</i>, then the resulting byte array is truncated at position <i>width</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> x(<span class="string">"apple"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> y <span class="operator">=</span> x<span class="operator">.</span>rightJustified(<span class="number">8</span><span class="operator">,</span> <span class="char">'.'</span>);    <span class="comment">// y == "...apple"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#leftJustified">leftJustified</a>().</p></div>
    pub unsafe fn right_justified_2a(
        &self,
        width: ::std::os::raw::c_int,
        fill: ::std::os::raw::c_char,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_rightJustified1(
            self as *const crate::QByteArray,
            width,
            fill,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a byte array of size <i>width</i> that contains the <i>fill</i> character followed by this byte array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::rightJustified(int width) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#rightJustified">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a byte array of size <i>width</i> that contains the <i>fill</i> character followed by this byte array.</p>
    /// <p>If <i>truncate</i> is false and the size of the byte array is more than <i>width</i>, then the returned byte array is a copy of this byte array.</p>
    /// <p>If <i>truncate</i> is true and the size of the byte array is more than <i>width</i>, then the resulting byte array is truncated at position <i>width</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> x(<span class="string">"apple"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> y <span class="operator">=</span> x<span class="operator">.</span>rightJustified(<span class="number">8</span><span class="operator">,</span> <span class="char">'.'</span>);    <span class="comment">// y == "...apple"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#leftJustified">leftJustified</a>().</p></div>
    pub unsafe fn right_justified_1a(
        &self,
        width: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_rightJustified2(
            self as *const crate::QByteArray,
            width,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::setNum(short arg1, int base = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#setNum-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toShort">toShort</a>().</p></div>
    pub unsafe fn set_num_short_int(
        &mut self,
        arg1: ::std::os::raw::c_short,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_setNum(
            self as *mut crate::QByteArray,
            arg1,
            base,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::setNum(unsigned short arg1, int base = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#setNum-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toUShort">toUShort</a>().</p></div>
    pub unsafe fn set_num_ushort_int(
        &mut self,
        arg1: ::std::os::raw::c_ushort,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_setNum1(
            self as *mut crate::QByteArray,
            arg1,
            base,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Sets the byte array to the printed value of <i>n</i> in base <i>base</i> (10 by default) and returns a reference to the byte array. The <i>base</i> can be any value between 2 and 36. For bases other than 10, n is treated as an unsigned integer.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::setNum(int arg1, int base = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#setNum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the byte array to the printed value of <i>n</i> in base <i>base</i> (10 by default) and returns a reference to the byte array. The <i>base</i> can be any value between 2 and 36. For bases other than 10, n is treated as an unsigned integer.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba;
    ///   <span class="type">int</span> n <span class="operator">=</span> <span class="number">63</span>;
    ///   ba<span class="operator">.</span>setNum(n);           <span class="comment">// ba == "63"</span>
    ///   ba<span class="operator">.</span>setNum(n<span class="operator">,</span> <span class="number">16</span>);       <span class="comment">// ba == "3f"</span>
    ///
    /// </pre>
    /// <p><b>Note: </b>The format of the number is not localized; the default C locale is used irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#toInt">toInt</a>().</p></div>
    pub unsafe fn set_num_2_int(
        &mut self,
        arg1: ::std::os::raw::c_int,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_setNum2(
            self as *mut crate::QByteArray,
            arg1,
            base,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::setNum(unsigned int arg1, int base = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#setNum-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toUInt">toUInt</a>().</p></div>
    pub unsafe fn set_num_uint_int(
        &mut self,
        arg1: ::std::os::raw::c_uint,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_setNum3(
            self as *mut crate::QByteArray,
            arg1,
            base,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::setNum(qlonglong arg1, int base = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#setNum-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toLongLong">toLongLong</a>().</p></div>
    pub unsafe fn set_num_i64_int(
        &mut self,
        arg1: i64,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_setNum4(
            self as *mut crate::QByteArray,
            arg1,
            base,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::setNum(qulonglong arg1, int base = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#setNum-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toULongLong">toULongLong</a>().</p></div>
    pub unsafe fn set_num_u64_int(
        &mut self,
        arg1: u64,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_setNum5(
            self as *mut crate::QByteArray,
            arg1,
            base,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::setNum(float arg1, char f = …, int prec = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#setNum-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Sets the byte array to the printed value of <i>n</i>, formatted in format <i>f</i> with precision <i>prec</i>, and returns a reference to the byte array.</p>
    /// <p><b>Note: </b>The format of the number is not localized; the default C locale is used irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toFloat">toFloat</a>().</p></div>
    pub unsafe fn set_num_float_char_int(
        &mut self,
        arg1: ::std::os::raw::c_float,
        f: ::std::os::raw::c_char,
        prec: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_setNum6(
            self as *mut crate::QByteArray,
            arg1,
            f,
            prec,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::setNum(double arg1, char f = …, int prec = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#setNum-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Sets the byte array to the printed value of <i>n</i>, formatted in format <i>f</i> with precision <i>prec</i>, and returns a reference to the byte array.</p>
    /// <p>The format <i>f</i> can be any of the following:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Format</th><th>Meaning</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td><code>e</code></td><td>format as [-]9.9e[+|-]999</td></tr>
    /// <tr valign="top" class="even"><td><code>E</code></td><td>format as [-]9.9E[+|-]999</td></tr>
    /// <tr valign="top" class="odd"><td><code>f</code></td><td>format as [-]9.9</td></tr>
    /// <tr valign="top" class="even"><td><code>g</code></td><td>use <code>e</code> or <code>f</code> format, whichever is the most concise</td></tr>
    /// <tr valign="top" class="odd"><td><code>G</code></td><td>use <code>E</code> or <code>f</code> format, whichever is the most concise</td></tr>
    /// </tbody></table></div>
    /// <p>With 'e', 'E', and 'f', <i>prec</i> is the number of digits after the decimal point. With 'g' and 'G', <i>prec</i> is the maximum number of significant digits (trailing zeroes are omitted).</p>
    /// <p><b>Note: </b>The format of the number is not localized; the default C locale is used irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toDouble">toDouble</a>().</p></div>
    pub unsafe fn set_num_double_char_int(
        &mut self,
        arg1: ::std::os::raw::c_double,
        f: ::std::os::raw::c_char,
        prec: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_setNum7(
            self as *mut crate::QByteArray,
            arg1,
            f,
            prec,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::setNum(short arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#setNum-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toShort">toShort</a>().</p></div>
    pub unsafe fn set_num_short(
        &mut self,
        arg1: ::std::os::raw::c_short,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_setNum8(self as *mut crate::QByteArray, arg1);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::setNum(unsigned short arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#setNum-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toUShort">toUShort</a>().</p></div>
    pub unsafe fn set_num_ushort(
        &mut self,
        arg1: ::std::os::raw::c_ushort,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_setNum9(self as *mut crate::QByteArray, arg1);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Sets the byte array to the printed value of <i>n</i> in base <i>base</i> (10 by default) and returns a reference to the byte array. The <i>base</i> can be any value between 2 and 36. For bases other than 10, n is treated as an unsigned integer.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::setNum(int arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#setNum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the byte array to the printed value of <i>n</i> in base <i>base</i> (10 by default) and returns a reference to the byte array. The <i>base</i> can be any value between 2 and 36. For bases other than 10, n is treated as an unsigned integer.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba;
    ///   <span class="type">int</span> n <span class="operator">=</span> <span class="number">63</span>;
    ///   ba<span class="operator">.</span>setNum(n);           <span class="comment">// ba == "63"</span>
    ///   ba<span class="operator">.</span>setNum(n<span class="operator">,</span> <span class="number">16</span>);       <span class="comment">// ba == "3f"</span>
    ///
    /// </pre>
    /// <p><b>Note: </b>The format of the number is not localized; the default C locale is used irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#toInt">toInt</a>().</p></div>
    pub unsafe fn set_num_int(
        &mut self,
        arg1: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_setNum10(self as *mut crate::QByteArray, arg1);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::setNum(unsigned int arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#setNum-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toUInt">toUInt</a>().</p></div>
    pub unsafe fn set_num_uint(
        &mut self,
        arg1: ::std::os::raw::c_uint,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_setNum11(self as *mut crate::QByteArray, arg1);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::setNum(qlonglong arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#setNum-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toLongLong">toLongLong</a>().</p></div>
    pub unsafe fn set_num_i64(&mut self, arg1: i64) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_setNum12(self as *mut crate::QByteArray, arg1);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::setNum(qulonglong arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#setNum-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toULongLong">toULongLong</a>().</p></div>
    pub unsafe fn set_num_u64(&mut self, arg1: u64) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_setNum13(self as *mut crate::QByteArray, arg1);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::setNum(float arg1, char f = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#setNum-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Sets the byte array to the printed value of <i>n</i>, formatted in format <i>f</i> with precision <i>prec</i>, and returns a reference to the byte array.</p>
    /// <p><b>Note: </b>The format of the number is not localized; the default C locale is used irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toFloat">toFloat</a>().</p></div>
    pub unsafe fn set_num_float_char(
        &mut self,
        arg1: ::std::os::raw::c_float,
        f: ::std::os::raw::c_char,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_setNum14(
            self as *mut crate::QByteArray,
            arg1,
            f,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::setNum(float arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#setNum-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Sets the byte array to the printed value of <i>n</i>, formatted in format <i>f</i> with precision <i>prec</i>, and returns a reference to the byte array.</p>
    /// <p><b>Note: </b>The format of the number is not localized; the default C locale is used irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toFloat">toFloat</a>().</p></div>
    pub unsafe fn set_num_float(
        &mut self,
        arg1: ::std::os::raw::c_float,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_setNum15(self as *mut crate::QByteArray, arg1);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::setNum(double arg1, char f = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#setNum-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Sets the byte array to the printed value of <i>n</i>, formatted in format <i>f</i> with precision <i>prec</i>, and returns a reference to the byte array.</p>
    /// <p>The format <i>f</i> can be any of the following:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Format</th><th>Meaning</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td><code>e</code></td><td>format as [-]9.9e[+|-]999</td></tr>
    /// <tr valign="top" class="even"><td><code>E</code></td><td>format as [-]9.9E[+|-]999</td></tr>
    /// <tr valign="top" class="odd"><td><code>f</code></td><td>format as [-]9.9</td></tr>
    /// <tr valign="top" class="even"><td><code>g</code></td><td>use <code>e</code> or <code>f</code> format, whichever is the most concise</td></tr>
    /// <tr valign="top" class="odd"><td><code>G</code></td><td>use <code>E</code> or <code>f</code> format, whichever is the most concise</td></tr>
    /// </tbody></table></div>
    /// <p>With 'e', 'E', and 'f', <i>prec</i> is the number of digits after the decimal point. With 'g' and 'G', <i>prec</i> is the maximum number of significant digits (trailing zeroes are omitted).</p>
    /// <p><b>Note: </b>The format of the number is not localized; the default C locale is used irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toDouble">toDouble</a>().</p></div>
    pub unsafe fn set_num_double_char(
        &mut self,
        arg1: ::std::os::raw::c_double,
        f: ::std::os::raw::c_char,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_setNum16(
            self as *mut crate::QByteArray,
            arg1,
            f,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::setNum(double arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#setNum-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Sets the byte array to the printed value of <i>n</i>, formatted in format <i>f</i> with precision <i>prec</i>, and returns a reference to the byte array.</p>
    /// <p>The format <i>f</i> can be any of the following:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Format</th><th>Meaning</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td><code>e</code></td><td>format as [-]9.9e[+|-]999</td></tr>
    /// <tr valign="top" class="even"><td><code>E</code></td><td>format as [-]9.9E[+|-]999</td></tr>
    /// <tr valign="top" class="odd"><td><code>f</code></td><td>format as [-]9.9</td></tr>
    /// <tr valign="top" class="even"><td><code>g</code></td><td>use <code>e</code> or <code>f</code> format, whichever is the most concise</td></tr>
    /// <tr valign="top" class="odd"><td><code>G</code></td><td>use <code>E</code> or <code>f</code> format, whichever is the most concise</td></tr>
    /// </tbody></table></div>
    /// <p>With 'e', 'E', and 'f', <i>prec</i> is the number of digits after the decimal point. With 'g' and 'G', <i>prec</i> is the maximum number of significant digits (trailing zeroes are omitted).</p>
    /// <p><b>Note: </b>The format of the number is not localized; the default C locale is used irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toDouble">toDouble</a>().</p></div>
    pub unsafe fn set_num_double(
        &mut self,
        arg1: ::std::os::raw::c_double,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_setNum17(self as *mut crate::QByteArray, arg1);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Resets the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> to use the first <i>size</i> bytes of the <i>data</i> array. The bytes are <i>not</i> copied. The <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> will contain the <i>data</i> pointer. The caller guarantees that <i>data</i> will not be deleted or modified as long as this <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> and any copies of it exist that have not been modified.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::setRawData(const char* a, unsigned int n)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#setRawData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Resets the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> to use the first <i>size</i> bytes of the <i>data</i> array. The bytes are <i>not</i> copied. The <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> will contain the <i>data</i> pointer. The caller guarantees that <i>data</i> will not be deleted or modified as long as this <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> and any copies of it exist that have not been modified.</p>
    /// <p>This function can be used instead of <a href="http://doc.qt.io/qt-5/qbytearray.html#fromRawData">fromRawData</a>() to re-use existing <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> objects to save memory re-allocations.</p>
    /// <p>This function was introduced in  Qt 4.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#fromRawData">fromRawData</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#data">data</a>(), and <a href="http://doc.qt.io/qt-5/qbytearray.html#constData">constData</a>().</p></div>
    pub unsafe fn set_raw_data(
        &mut self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_uint,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_setRawData(
            self as *mut crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(a)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qbytearray.html#squeeze">squeeze</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QByteArray::shrink_to_fit()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#shrink_to_fit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qbytearray.html#squeeze">squeeze</a>().</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn shrink_to_fit(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_shrink_to_fit(self as *mut crate::QByteArray)
    }

    /// <p>Returns a byte array that has whitespace removed from the start and the end, and which has each sequence of internal whitespace replaced with a single space.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::simplified() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#simplified">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a byte array that has whitespace removed from the start and the end, and which has each sequence of internal whitespace replaced with a single space.</p>
    /// <p>Whitespace means any character for which the standard C++ <code>isspace()</code> function returns <code>true</code> in the C locale. This includes the ASCII isspace() function returns <code>true</code> in the C locale. This includes the ASCII characters '\t', '\n', '\v', '\f', '\r', and ' '.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba(<span class="string">"  lots\t of\nwhitespace\r\n "</span>);
    ///   ba <span class="operator">=</span> ba<span class="operator">.</span>simplified();
    ///   <span class="comment">// ba == "lots of whitespace";</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#trimmed">trimmed</a>().</p></div>
    pub unsafe fn simplified(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_simplified(self as *const crate::QByteArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a byte array that has whitespace removed from the start and the end, and which has each sequence of internal whitespace replaced with a single space.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::simplified()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#simplified">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a byte array that has whitespace removed from the start and the end, and which has each sequence of internal whitespace replaced with a single space.</p>
    /// <p>Whitespace means any character for which the standard C++ <code>isspace()</code> function returns <code>true</code> in the C locale. This includes the ASCII isspace() function returns <code>true</code> in the C locale. This includes the ASCII characters '\t', '\n', '\v', '\f', '\r', and ' '.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba(<span class="string">"  lots\t of\nwhitespace\r\n "</span>);
    ///   ba <span class="operator">=</span> ba<span class="operator">.</span>simplified();
    ///   <span class="comment">// ba == "lots of whitespace";</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#trimmed">trimmed</a>().</p></div>
    pub unsafe fn simplified_mut(&mut self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_simplified1(self as *mut crate::QByteArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the number of bytes in this byte array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of bytes in this byte array.</p>
    /// <p>The last byte in the byte array is at position size() - 1. In addition, <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> ensures that the byte at position size() is always '\0', so that you can use the return value of <a href="http://doc.qt.io/qt-5/qbytearray.html#data">data</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#constData">constData</a>() as arguments to functions that expect '\0'-terminated strings. If the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> object was created from a <a href="http://doc.qt.io/qt-5/qbytearray.html#fromRawData">raw data</a> that didn't include the trailing null-termination character then <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> doesn't add it automaticall unless the <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> is created.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba(<span class="string">"Hello"</span>);
    ///   <span class="type">int</span> n <span class="operator">=</span> ba<span class="operator">.</span>size();          <span class="comment">// n == 5</span>
    ///   ba<span class="operator">.</span>data()<span class="operator">[</span><span class="number">0</span><span class="operator">]</span>;               <span class="comment">// returns 'H'</span>
    ///   ba<span class="operator">.</span>data()<span class="operator">[</span><span class="number">4</span><span class="operator">]</span>;               <span class="comment">// returns 'o'</span>
    ///   ba<span class="operator">.</span>data()<span class="operator">[</span><span class="number">5</span><span class="operator">]</span>;               <span class="comment">// returns '\0'</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#resize">resize</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_size(self as *const crate::QByteArray)
    }

    /// <p>Splits the byte array into subarrays wherever <i>sep</i> occurs, and returns the list of those arrays. If <i>sep</i> does not match anywhere in the byte array, split() returns a single-element list containing this byte array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QByteArray> QByteArray::split(char sep) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#split">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Splits the byte array into subarrays wherever <i>sep</i> occurs, and returns the list of those arrays. If <i>sep</i> does not match anywhere in the byte array, split() returns a single-element list containing this byte array.</p></div>
    pub unsafe fn split(
        &self,
        sep: ::std::os::raw::c_char,
    ) -> ::cpp_utils::CppBox<crate::QListOfQByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_split(self as *const crate::QByteArray, sep);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Releases any memory not required to store the array's data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QByteArray::squeeze()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#squeeze">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Releases any memory not required to store the array's data.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>'s memory usage. In general, you will rarely ever need to call this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#capacity">capacity</a>().</p></div>
    pub unsafe fn squeeze(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_squeeze(self as *mut crate::QByteArray)
    }

    /// <p>Returns <code>true</code> if this byte array starts with byte array <i>ba</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QByteArray::startsWith(const QByteArray& a) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this byte array starts with byte array <i>ba</i>; otherwise returns <code>false</code>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> url(<span class="string">"ftp://ftp.qt-project.org/"</span>);
    ///   <span class="keyword">if</span> (url<span class="operator">.</span>startsWith(<span class="string">"ftp:"</span>))
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#endsWith">endsWith</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#left">left</a>().</p></div>
    pub unsafe fn starts_with_q_byte_array(
        &self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_startsWith(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(a).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QByteArray::startsWith(char c) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#startsWith-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if this byte array starts with character <i>ch</i>; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn starts_with_char(&self, c: ::std::os::raw::c_char) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_startsWith1(self as *const crate::QByteArray, c)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QByteArray::startsWith(const char* c) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#startsWith-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if this byte array starts with string <i>str</i>; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn starts_with_char2(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_startsWith2(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
        )
    }

    /// <p>Swaps byte array <i>other</i> with this byte array. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QByteArray::swap(QByteArray& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps byte array <i>other</i> with this byte array. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QByteArray>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_swap(
            self as *mut crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QByteArray>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::toBase64(QFlags<QByteArray::Base64Option> options) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toBase64-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a copy of the byte array, encoded using the options <i>options</i>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> text(<span class="string">"&lt;p&gt;Hello?&lt;/p&gt;"</span>);
    ///   text<span class="operator">.</span>toBase64(<span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span><span class="operator">::</span>Base64Encoding <span class="operator">|</span> <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span><span class="operator">::</span>OmitTrailingEquals);      <span class="comment">// returns "PHA+SGVsbG8/PC9wPg"</span>
    ///   text<span class="operator">.</span>toBase64(<span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span><span class="operator">::</span>Base64Encoding);                                       <span class="comment">// returns "PHA+SGVsbG8/PC9wPg=="</span>
    ///   text<span class="operator">.</span>toBase64(<span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span><span class="operator">::</span>Base64UrlEncoding);                                    <span class="comment">// returns "PHA-SGVsbG8_PC9wPg=="</span>
    ///   text<span class="operator">.</span>toBase64(<span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span><span class="operator">::</span>Base64UrlEncoding <span class="operator">|</span> <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span><span class="operator">::</span>OmitTrailingEquals);   <span class="comment">// returns "PHA-SGVsbG8_PC9wPg"</span>
    ///
    /// </pre>
    /// <p>The algorithm used to encode Base64-encoded data is defined in <a href="http://www.ietf.org/rfc/rfc4648.txt">RFC 4648</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#fromBase64">fromBase64</a>().</p></div>
    pub unsafe fn to_base64_1a(
        &self,
        options: crate::QFlags<crate::q_byte_array::Base64Option>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_toBase64(
            self as *const crate::QByteArray,
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a copy of the byte array, encoded as Base64.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::toBase64() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toBase64">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a copy of the byte array, encoded as Base64.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> text(<span class="string">"Qt is great!"</span>);
    ///   text<span class="operator">.</span>toBase64();        <span class="comment">// returns "UXQgaXMgZ3JlYXQh"</span>
    ///
    /// </pre>
    /// <p>The algorithm used to encode Base64-encoded data is defined in <a href="http://www.ietf.org/rfc/rfc4648.txt">RFC 4648</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#fromBase64">fromBase64</a>().</p></div>
    pub unsafe fn to_base64_0a(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_toBase641(self as *const crate::QByteArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```const char* QByteArray::operator const char*() const```</span>.
    pub unsafe fn to_char(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_operator_const_char_(
            self as *const crate::QByteArray,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns the byte array converted to a <code>double</code> value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QByteArray::toDouble(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toDouble">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to a <code>double</code> value.</p>
    /// <p>Returns 0.0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> string(<span class="string">"1234.56"</span>);
    ///   <span class="type">double</span> a <span class="operator">=</span> string<span class="operator">.</span>toDouble();   <span class="comment">// a == 1234.56</span>
    ///
    /// </pre>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_double_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toDouble(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the byte array converted to a <code>double</code> value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QByteArray::toDouble() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toDouble">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to a <code>double</code> value.</p>
    /// <p>Returns 0.0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> string(<span class="string">"1234.56"</span>);
    ///   <span class="type">double</span> a <span class="operator">=</span> string<span class="operator">.</span>toDouble();   <span class="comment">// a == 1234.56</span>
    ///
    /// </pre>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_double_0a(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toDouble1(self as *const crate::QByteArray)
    }

    /// <p>Returns the byte array converted to a <code>float</code> value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```float QByteArray::toFloat(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toFloat">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to a <code>float</code> value.</p>
    /// <p>Returns 0.0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_float_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_float {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toFloat(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the byte array converted to a <code>float</code> value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```float QByteArray::toFloat() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toFloat">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to a <code>float</code> value.</p>
    /// <p>Returns 0.0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_float_0a(&self) -> ::std::os::raw::c_float {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toFloat1(self as *const crate::QByteArray)
    }

    /// <p>Returns a hex encoded copy of the byte array. The hex encoding uses the numbers 0-9 and the letters a-f.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::toHex() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toHex">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a hex encoded copy of the byte array. The hex encoding uses the numbers 0-9 and the letters a-f.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#fromHex">fromHex</a>().</p></div>
    pub unsafe fn to_hex_0a(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_toHex(self as *const crate::QByteArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::toHex(char separator) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toHex-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a hex encoded copy of the byte array. The hex encoding uses the numbers 0-9 and the letters a-f.</p>
    /// <p>If <i>separator</i> is not '\0', the separator character is inserted between the hex bytes.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> macAddress <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span><span class="operator">::</span>fromHex(<span class="string">"123456abcdef"</span>);
    ///   macAddress<span class="operator">.</span>toHex(<span class="char">':'</span>); <span class="comment">// returns "12:34:56:ab:cd:ef"</span>
    ///   macAddress<span class="operator">.</span>toHex(<span class="number">0</span>);   <span class="comment">// returns "123456abcdef"</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.9.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#fromHex">fromHex</a>().</p></div>
    pub unsafe fn to_hex_1a(
        &self,
        separator: ::std::os::raw::c_char,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_toHex1(
            self as *const crate::QByteArray,
            separator,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the byte array converted to an <code>int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::toInt(bool* ok = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to an <code>int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    /// <p>If <i>base</i> is 0, the base is determined automatically using the following rules: If the byte array begins with "0x", it is assumed to be hexadecimal; if it begins with "0", it is assumed to be octal; otherwise it is assumed to be decimal.</p>
    /// <p>Returns 0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> str(<span class="string">"FF"</span>);
    ///   bool ok;
    ///   <span class="type">int</span> hex <span class="operator">=</span> str<span class="operator">.</span>toInt(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);     <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type">int</span> dec <span class="operator">=</span> str<span class="operator">.</span>toInt(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);     <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_int_2a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
        base: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toInt(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
            base,
        )
    }

    /// <p>Returns the byte array converted to an <code>int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::toInt(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to an <code>int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    /// <p>If <i>base</i> is 0, the base is determined automatically using the following rules: If the byte array begins with "0x", it is assumed to be hexadecimal; if it begins with "0", it is assumed to be octal; otherwise it is assumed to be decimal.</p>
    /// <p>Returns 0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> str(<span class="string">"FF"</span>);
    ///   bool ok;
    ///   <span class="type">int</span> hex <span class="operator">=</span> str<span class="operator">.</span>toInt(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);     <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type">int</span> dec <span class="operator">=</span> str<span class="operator">.</span>toInt(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);     <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_int_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toInt1(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the byte array converted to an <code>int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArray::toInt() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to an <code>int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    /// <p>If <i>base</i> is 0, the base is determined automatically using the following rules: If the byte array begins with "0x", it is assumed to be hexadecimal; if it begins with "0", it is assumed to be octal; otherwise it is assumed to be decimal.</p>
    /// <p>Returns 0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> str(<span class="string">"FF"</span>);
    ///   bool ok;
    ///   <span class="type">int</span> hex <span class="operator">=</span> str<span class="operator">.</span>toInt(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);     <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type">int</span> dec <span class="operator">=</span> str<span class="operator">.</span>toInt(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);     <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_int_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toInt2(self as *const crate::QByteArray)
    }

    /// <p>Returns the byte array converted to a <code>long</code> int using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```long QByteArray::toLong(bool* ok = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to a <code>long</code> int using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    /// <p>If <i>base</i> is 0, the base is determined automatically using the following rules: If the byte array begins with "0x", it is assumed to be hexadecimal; if it begins with "0", it is assumed to be octal; otherwise it is assumed to be decimal.</p>
    /// <p>Returns 0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> str(<span class="string">"FF"</span>);
    ///   bool ok;
    ///   <span class="type">long</span> hex <span class="operator">=</span> str<span class="operator">.</span>toLong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);   <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type">long</span> dec <span class="operator">=</span> str<span class="operator">.</span>toLong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);   <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_long_2a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
        base: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_long {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toLong(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
            base,
        )
    }

    /// <p>Returns the byte array converted to a <code>long</code> int using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```long QByteArray::toLong(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to a <code>long</code> int using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    /// <p>If <i>base</i> is 0, the base is determined automatically using the following rules: If the byte array begins with "0x", it is assumed to be hexadecimal; if it begins with "0", it is assumed to be octal; otherwise it is assumed to be decimal.</p>
    /// <p>Returns 0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> str(<span class="string">"FF"</span>);
    ///   bool ok;
    ///   <span class="type">long</span> hex <span class="operator">=</span> str<span class="operator">.</span>toLong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);   <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type">long</span> dec <span class="operator">=</span> str<span class="operator">.</span>toLong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);   <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_long_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_long {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toLong1(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the byte array converted to a <code>long</code> int using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```long QByteArray::toLong() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to a <code>long</code> int using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    /// <p>If <i>base</i> is 0, the base is determined automatically using the following rules: If the byte array begins with "0x", it is assumed to be hexadecimal; if it begins with "0", it is assumed to be octal; otherwise it is assumed to be decimal.</p>
    /// <p>Returns 0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> str(<span class="string">"FF"</span>);
    ///   bool ok;
    ///   <span class="type">long</span> hex <span class="operator">=</span> str<span class="operator">.</span>toLong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);   <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type">long</span> dec <span class="operator">=</span> str<span class="operator">.</span>toLong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);   <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_long_0a(&self) -> ::std::os::raw::c_long {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toLong2(self as *const crate::QByteArray)
    }

    /// <p>Returns the byte array converted to a <code>long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qlonglong QByteArray::toLongLong(bool* ok = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toLongLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to a <code>long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    /// <p>If <i>base</i> is 0, the base is determined automatically using the following rules: If the byte array begins with "0x", it is assumed to be hexadecimal; if it begins with "0", it is assumed to be octal; otherwise it is assumed to be decimal.</p>
    /// <p>Returns 0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_long_long_2a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
        base: ::std::os::raw::c_int,
    ) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toLongLong(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
            base,
        )
    }

    /// <p>Returns the byte array converted to a <code>long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qlonglong QByteArray::toLongLong(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toLongLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to a <code>long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    /// <p>If <i>base</i> is 0, the base is determined automatically using the following rules: If the byte array begins with "0x", it is assumed to be hexadecimal; if it begins with "0", it is assumed to be octal; otherwise it is assumed to be decimal.</p>
    /// <p>Returns 0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_long_long_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toLongLong1(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the byte array converted to a <code>long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qlonglong QByteArray::toLongLong() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toLongLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to a <code>long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    /// <p>If <i>base</i> is 0, the base is determined automatically using the following rules: If the byte array begins with "0x", it is assumed to be hexadecimal; if it begins with "0", it is assumed to be octal; otherwise it is assumed to be decimal.</p>
    /// <p>Returns 0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_long_long_0a(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toLongLong2(self as *const crate::QByteArray)
    }

    /// <p>Returns a lowercase copy of the byte array. The bytearray is interpreted as a Latin-1 encoded string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::toLower() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toLower">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a lowercase copy of the byte array. The bytearray is interpreted as a Latin-1 encoded string.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> x(<span class="string">"Qt by THE QT COMPANY"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> y <span class="operator">=</span> x<span class="operator">.</span>toLower();
    ///   <span class="comment">// y == "qt by the qt company"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toUpper">toUpper</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#8-bit-character-comparisons">8-bit Character Comparisons</a>.</p></div>
    pub unsafe fn to_lower(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_toLower(self as *const crate::QByteArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a lowercase copy of the byte array. The bytearray is interpreted as a Latin-1 encoded string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::toLower()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toLower">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a lowercase copy of the byte array. The bytearray is interpreted as a Latin-1 encoded string.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> x(<span class="string">"Qt by THE QT COMPANY"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> y <span class="operator">=</span> x<span class="operator">.</span>toLower();
    ///   <span class="comment">// y == "qt by the qt company"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toUpper">toUpper</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#8-bit-character-comparisons">8-bit Character Comparisons</a>.</p></div>
    pub unsafe fn to_lower_mut(&mut self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_toLower1(self as *mut crate::QByteArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a URI/URL-style percent-encoded copy of this byte array. The <i>percent</i> parameter allows you to override the default '%' character for another.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::toPercentEncoding(const QByteArray& exclude = …, const QByteArray& include = …, char percent = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toPercentEncoding">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a URI/URL-style percent-encoded copy of this byte array. The <i>percent</i> parameter allows you to override the default '%' character for another.</p>
    /// <p>By default, this function will encode all characters that are not one of the following:</p>
    /// <p>ALPHA ("a" to "z" and "A" to "Z") / DIGIT (0 to 9) / "-" / "." / "_" / "~"</p>
    /// <p>To prevent characters from being encoded pass them to <i>exclude</i>. To force characters to be encoded pass them to <i>include</i>. The <i>percent</i> character is always encoded.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> text <span class="operator">=</span> <span class="string">"{a fishy string?}"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba <span class="operator">=</span> text<span class="operator">.</span>toPercentEncoding(<span class="string">"{}"</span><span class="operator">,</span> <span class="string">"s"</span>);
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>(ba<span class="operator">.</span>constData());
    ///   <span class="comment">// prints "{a fi%73hy %73tring%3F}"</span>
    ///
    /// </pre>
    /// <p>The hex encoding uses the numbers 0-9 and the uppercase letters A-F.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#fromPercentEncoding">fromPercentEncoding</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#toPercentEncoding">QUrl::toPercentEncoding</a>().</p></div>
    pub unsafe fn to_percent_encoding_3a(
        &self,
        exclude: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        include: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        percent: ::std::os::raw::c_char,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_toPercentEncoding(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(exclude)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(include)
                .as_raw_ptr(),
            percent,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a URI/URL-style percent-encoded copy of this byte array. The <i>percent</i> parameter allows you to override the default '%' character for another.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::toPercentEncoding(const QByteArray& exclude = …, const QByteArray& include = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toPercentEncoding">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a URI/URL-style percent-encoded copy of this byte array. The <i>percent</i> parameter allows you to override the default '%' character for another.</p>
    /// <p>By default, this function will encode all characters that are not one of the following:</p>
    /// <p>ALPHA ("a" to "z" and "A" to "Z") / DIGIT (0 to 9) / "-" / "." / "_" / "~"</p>
    /// <p>To prevent characters from being encoded pass them to <i>exclude</i>. To force characters to be encoded pass them to <i>include</i>. The <i>percent</i> character is always encoded.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> text <span class="operator">=</span> <span class="string">"{a fishy string?}"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba <span class="operator">=</span> text<span class="operator">.</span>toPercentEncoding(<span class="string">"{}"</span><span class="operator">,</span> <span class="string">"s"</span>);
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>(ba<span class="operator">.</span>constData());
    ///   <span class="comment">// prints "{a fi%73hy %73tring%3F}"</span>
    ///
    /// </pre>
    /// <p>The hex encoding uses the numbers 0-9 and the uppercase letters A-F.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#fromPercentEncoding">fromPercentEncoding</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#toPercentEncoding">QUrl::toPercentEncoding</a>().</p></div>
    pub unsafe fn to_percent_encoding_2a(
        &self,
        exclude: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        include: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_toPercentEncoding1(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(exclude)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(include)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a URI/URL-style percent-encoded copy of this byte array. The <i>percent</i> parameter allows you to override the default '%' character for another.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::toPercentEncoding(const QByteArray& exclude = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toPercentEncoding">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a URI/URL-style percent-encoded copy of this byte array. The <i>percent</i> parameter allows you to override the default '%' character for another.</p>
    /// <p>By default, this function will encode all characters that are not one of the following:</p>
    /// <p>ALPHA ("a" to "z" and "A" to "Z") / DIGIT (0 to 9) / "-" / "." / "_" / "~"</p>
    /// <p>To prevent characters from being encoded pass them to <i>exclude</i>. To force characters to be encoded pass them to <i>include</i>. The <i>percent</i> character is always encoded.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> text <span class="operator">=</span> <span class="string">"{a fishy string?}"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba <span class="operator">=</span> text<span class="operator">.</span>toPercentEncoding(<span class="string">"{}"</span><span class="operator">,</span> <span class="string">"s"</span>);
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>(ba<span class="operator">.</span>constData());
    ///   <span class="comment">// prints "{a fi%73hy %73tring%3F}"</span>
    ///
    /// </pre>
    /// <p>The hex encoding uses the numbers 0-9 and the uppercase letters A-F.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#fromPercentEncoding">fromPercentEncoding</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#toPercentEncoding">QUrl::toPercentEncoding</a>().</p></div>
    pub unsafe fn to_percent_encoding_1a(
        &self,
        exclude: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_toPercentEncoding2(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(exclude)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a URI/URL-style percent-encoded copy of this byte array. The <i>percent</i> parameter allows you to override the default '%' character for another.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::toPercentEncoding() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toPercentEncoding">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a URI/URL-style percent-encoded copy of this byte array. The <i>percent</i> parameter allows you to override the default '%' character for another.</p>
    /// <p>By default, this function will encode all characters that are not one of the following:</p>
    /// <p>ALPHA ("a" to "z" and "A" to "Z") / DIGIT (0 to 9) / "-" / "." / "_" / "~"</p>
    /// <p>To prevent characters from being encoded pass them to <i>exclude</i>. To force characters to be encoded pass them to <i>include</i>. The <i>percent</i> character is always encoded.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> text <span class="operator">=</span> <span class="string">"{a fishy string?}"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba <span class="operator">=</span> text<span class="operator">.</span>toPercentEncoding(<span class="string">"{}"</span><span class="operator">,</span> <span class="string">"s"</span>);
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>(ba<span class="operator">.</span>constData());
    ///   <span class="comment">// prints "{a fi%73hy %73tring%3F}"</span>
    ///
    /// </pre>
    /// <p>The hex encoding uses the numbers 0-9 and the uppercase letters A-F.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#fromPercentEncoding">fromPercentEncoding</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#toPercentEncoding">QUrl::toPercentEncoding</a>().</p></div>
    pub unsafe fn to_percent_encoding_0a(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_toPercentEncoding3(
            self as *const crate::QByteArray,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the byte array converted to a <code>short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```short QByteArray::toShort(bool* ok = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toShort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to a <code>short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    /// <p>If <i>base</i> is 0, the base is determined automatically using the following rules: If the byte array begins with "0x", it is assumed to be hexadecimal; if it begins with "0", it is assumed to be octal; otherwise it is assumed to be decimal.</p>
    /// <p>Returns 0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_short_2a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
        base: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_short {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toShort(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
            base,
        )
    }

    /// <p>Returns the byte array converted to a <code>short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```short QByteArray::toShort(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toShort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to a <code>short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    /// <p>If <i>base</i> is 0, the base is determined automatically using the following rules: If the byte array begins with "0x", it is assumed to be hexadecimal; if it begins with "0", it is assumed to be octal; otherwise it is assumed to be decimal.</p>
    /// <p>Returns 0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_short_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_short {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toShort1(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the byte array converted to a <code>short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```short QByteArray::toShort() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toShort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to a <code>short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    /// <p>If <i>base</i> is 0, the base is determined automatically using the following rules: If the byte array begins with "0x", it is assumed to be hexadecimal; if it begins with "0", it is assumed to be octal; otherwise it is assumed to be decimal.</p>
    /// <p>Returns 0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_short_0a(&self) -> ::std::os::raw::c_short {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toShort2(self as *const crate::QByteArray)
    }

    /// <p>Returns the byte array converted to an <code>unsigned int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QByteArray::toUInt(bool* ok = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toUInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to an <code>unsigned int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    /// <p>If <i>base</i> is 0, the base is determined automatically using the following rules: If the byte array begins with "0x", it is assumed to be hexadecimal; if it begins with "0", it is assumed to be octal; otherwise it is assumed to be decimal.</p>
    /// <p>Returns 0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_u_int_2a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
        base: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toUInt(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
            base,
        )
    }

    /// <p>Returns the byte array converted to an <code>unsigned int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QByteArray::toUInt(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toUInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to an <code>unsigned int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    /// <p>If <i>base</i> is 0, the base is determined automatically using the following rules: If the byte array begins with "0x", it is assumed to be hexadecimal; if it begins with "0", it is assumed to be octal; otherwise it is assumed to be decimal.</p>
    /// <p>Returns 0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_u_int_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toUInt1(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the byte array converted to an <code>unsigned int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QByteArray::toUInt() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toUInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to an <code>unsigned int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    /// <p>If <i>base</i> is 0, the base is determined automatically using the following rules: If the byte array begins with "0x", it is assumed to be hexadecimal; if it begins with "0", it is assumed to be octal; otherwise it is assumed to be decimal.</p>
    /// <p>Returns 0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_u_int_0a(&self) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toUInt2(self as *const crate::QByteArray)
    }

    /// <p>Returns the byte array converted to an <code>unsigned long int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned long QByteArray::toULong(bool* ok = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toULong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to an <code>unsigned long int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    /// <p>If <i>base</i> is 0, the base is determined automatically using the following rules: If the byte array begins with "0x", it is assumed to be hexadecimal; if it begins with "0", it is assumed to be octal; otherwise it is assumed to be decimal.</p>
    /// <p>Returns 0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_u_long_2a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
        base: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_ulong {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toULong(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
            base,
        )
    }

    /// <p>Returns the byte array converted to an <code>unsigned long int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned long QByteArray::toULong(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toULong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to an <code>unsigned long int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    /// <p>If <i>base</i> is 0, the base is determined automatically using the following rules: If the byte array begins with "0x", it is assumed to be hexadecimal; if it begins with "0", it is assumed to be octal; otherwise it is assumed to be decimal.</p>
    /// <p>Returns 0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_u_long_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_ulong {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toULong1(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the byte array converted to an <code>unsigned long int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned long QByteArray::toULong() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toULong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to an <code>unsigned long int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    /// <p>If <i>base</i> is 0, the base is determined automatically using the following rules: If the byte array begins with "0x", it is assumed to be hexadecimal; if it begins with "0", it is assumed to be octal; otherwise it is assumed to be decimal.</p>
    /// <p>Returns 0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_u_long_0a(&self) -> ::std::os::raw::c_ulong {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toULong2(self as *const crate::QByteArray)
    }

    /// <p>Returns the byte array converted to an <code>unsigned long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qulonglong QByteArray::toULongLong(bool* ok = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toULongLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to an <code>unsigned long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    /// <p>If <i>base</i> is 0, the base is determined automatically using the following rules: If the byte array begins with "0x", it is assumed to be hexadecimal; if it begins with "0", it is assumed to be octal; otherwise it is assumed to be decimal.</p>
    /// <p>Returns 0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_u_long_long_2a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
        base: ::std::os::raw::c_int,
    ) -> u64 {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toULongLong(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
            base,
        )
    }

    /// <p>Returns the byte array converted to an <code>unsigned long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qulonglong QByteArray::toULongLong(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toULongLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to an <code>unsigned long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    /// <p>If <i>base</i> is 0, the base is determined automatically using the following rules: If the byte array begins with "0x", it is assumed to be hexadecimal; if it begins with "0", it is assumed to be octal; otherwise it is assumed to be decimal.</p>
    /// <p>Returns 0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_u_long_long_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> u64 {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toULongLong1(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the byte array converted to an <code>unsigned long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qulonglong QByteArray::toULongLong() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toULongLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to an <code>unsigned long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    /// <p>If <i>base</i> is 0, the base is determined automatically using the following rules: If the byte array begins with "0x", it is assumed to be hexadecimal; if it begins with "0", it is assumed to be octal; otherwise it is assumed to be decimal.</p>
    /// <p>Returns 0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_u_long_long_0a(&self) -> u64 {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toULongLong2(self as *const crate::QByteArray)
    }

    /// <p>Returns the byte array converted to an <code>unsigned short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned short QByteArray::toUShort(bool* ok = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toUShort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to an <code>unsigned short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    /// <p>If <i>base</i> is 0, the base is determined automatically using the following rules: If the byte array begins with "0x", it is assumed to be hexadecimal; if it begins with "0", it is assumed to be octal; otherwise it is assumed to be decimal.</p>
    /// <p>Returns 0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_u_short_2a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
        base: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_ushort {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toUShort(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
            base,
        )
    }

    /// <p>Returns the byte array converted to an <code>unsigned short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned short QByteArray::toUShort(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toUShort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to an <code>unsigned short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    /// <p>If <i>base</i> is 0, the base is determined automatically using the following rules: If the byte array begins with "0x", it is assumed to be hexadecimal; if it begins with "0", it is assumed to be octal; otherwise it is assumed to be decimal.</p>
    /// <p>Returns 0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_u_short_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_ushort {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toUShort1(
            self as *const crate::QByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the byte array converted to an <code>unsigned short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned short QByteArray::toUShort() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toUShort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array converted to an <code>unsigned short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0.</p>
    /// <p>If <i>base</i> is 0, the base is determined automatically using the following rules: If the byte array begins with "0x", it is assumed to be hexadecimal; if it begins with "0", it is assumed to be octal; otherwise it is assumed to be decimal.</p>
    /// <p>Returns 0 if the conversion fails.</p>
    /// <p>If <i>ok</i> is not 0: if a conversion error occurs, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <p><b>Note: </b>The conversion of the number is performed in the default C locale, irrespective of the user's locale.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#number">number</a>().</p></div>
    pub unsafe fn to_u_short_0a(&self) -> ::std::os::raw::c_ushort {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_toUShort2(self as *const crate::QByteArray)
    }

    /// <p>Returns an uppercase copy of the byte array. The bytearray is interpreted as a Latin-1 encoded string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::toUpper() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toUpper">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an uppercase copy of the byte array. The bytearray is interpreted as a Latin-1 encoded string.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> x(<span class="string">"Qt by THE QT COMPANY"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> y <span class="operator">=</span> x<span class="operator">.</span>toUpper();
    ///   <span class="comment">// y == "QT BY THE QT COMPANY"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toLower">toLower</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#8-bit-character-comparisons">8-bit Character Comparisons</a>.</p></div>
    pub unsafe fn to_upper(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_toUpper(self as *const crate::QByteArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns an uppercase copy of the byte array. The bytearray is interpreted as a Latin-1 encoded string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::toUpper()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#toUpper">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an uppercase copy of the byte array. The bytearray is interpreted as a Latin-1 encoded string.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> x(<span class="string">"Qt by THE QT COMPANY"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> y <span class="operator">=</span> x<span class="operator">.</span>toUpper();
    ///   <span class="comment">// y == "QT BY THE QT COMPANY"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toLower">toLower</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#8-bit-character-comparisons">8-bit Character Comparisons</a>.</p></div>
    pub unsafe fn to_upper_mut(&mut self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_toUpper1(self as *mut crate::QByteArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```const void* QByteArray::operator const void*() const```</span>.
    pub unsafe fn to_void(&self) -> ::cpp_utils::Ptr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArray_operator_const_void_(
            self as *const crate::QByteArray,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns a byte array that has whitespace removed from the start and the end.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::trimmed() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#trimmed">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a byte array that has whitespace removed from the start and the end.</p>
    /// <p>Whitespace means any character for which the standard C++ <code>isspace()</code> function returns <code>true</code> in the C locale. This includes the ASCII characters '\t', '\n', '\v', '\f', '\r', and ' '.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba(<span class="string">"  lots\t of\nwhitespace\r\n "</span>);
    ///   ba <span class="operator">=</span> ba<span class="operator">.</span>trimmed();
    ///   <span class="comment">// ba == "lots\t of\nwhitespace";</span>
    ///
    /// </pre>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qbytearray.html#simplified">simplified</a>(), trimmed() leaves internal whitespace alone.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#simplified">simplified</a>().</p></div>
    pub unsafe fn trimmed(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_trimmed(self as *const crate::QByteArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a byte array that has whitespace removed from the start and the end.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArray::trimmed()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#trimmed">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a byte array that has whitespace removed from the start and the end.</p>
    /// <p>Whitespace means any character for which the standard C++ <code>isspace()</code> function returns <code>true</code> in the C locale. This includes the ASCII characters '\t', '\n', '\v', '\f', '\r', and ' '.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba(<span class="string">"  lots\t of\nwhitespace\r\n "</span>);
    ///   ba <span class="operator">=</span> ba<span class="operator">.</span>trimmed();
    ///   <span class="comment">// ba == "lots\t of\nwhitespace";</span>
    ///
    /// </pre>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qbytearray.html#simplified">simplified</a>(), trimmed() leaves internal whitespace alone.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#simplified">simplified</a>().</p></div>
    pub unsafe fn trimmed_mut(&mut self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteArray_trimmed1(self as *mut crate::QByteArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Truncates the byte array at index position <i>pos</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QByteArray::truncate(int pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#truncate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Truncates the byte array at index position <i>pos</i>.</p>
    /// <p>If <i>pos</i> is beyond the end of the array, nothing happens.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> ba(<span class="string">"Stockholm"</span>);
    ///   ba<span class="operator">.</span>truncate(<span class="number">5</span>);             <span class="comment">// ba == "Stock"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#chop">chop</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#resize">resize</a>(), and <a href="http://doc.qt.io/qt-5/qbytearray.html#left">left</a>().</p></div>
    pub unsafe fn truncate(&mut self, pos: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_truncate(self as *mut crate::QByteArray, pos)
    }
}

/// C++ class: <span style='color: green;'>```QByteRef```</span>.
#[repr(C)]
pub struct QByteRef {
    _unused: u8,
}
impl QByteRef {
    /// Calls C++ function: <span style='color: green;'>```QByteRef& QByteRef::operator=(char c)```</span>.
    pub unsafe fn copy_from_char(
        &mut self,
        c: ::std::os::raw::c_char,
    ) -> ::cpp_utils::MutRef<crate::QByteRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QByteRef_operator_(self as *mut crate::QByteRef, c);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```QByteRef& QByteRef::operator=(const QByteRef& c)```</span>.
    pub unsafe fn copy_from_q_byte_ref(
        &mut self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteRef>>,
    ) -> ::cpp_utils::MutRef<crate::QByteRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteRef_operator_1(
            self as *mut crate::QByteRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteRef>>::cast_into(c).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QByteRef::QByteRef(const QByteRef& other)```</span>.
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteRef>>,
    ) -> ::cpp_utils::CppBox<crate::QByteRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteRef_QByteRef1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteRef>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```char QByteRef::operator char() const```</span>.
    pub unsafe fn to_char(&self) -> ::std::os::raw::c_char {
        crate::__ffi::ctr_qt_core_ffi_QByteRef_operator_char(self as *const crate::QByteRef)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> class provides a thin wrapper around an US-ASCII/Latin-1 encoded string literal.</p>
///
/// C++ class: <span style='color: green;'>```QLatin1String```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlatin1string.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> class provides a thin wrapper around an US-ASCII/Latin-1 encoded string literal.</p>
/// <p>Many of <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>'s member functions are overloaded to accept <code>const char *</code> instead of <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. This includes the copy constructor, the assignment operator, the comparison operators, and various other functions such as <a href="http://doc.qt.io/qt-5/qstring.html#insert">insert()</a>, <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace()</a>, and <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf()</a>. These functions are usually optimized to avoid constructing a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> object for the <code>const char *</code> data. For example, assuming <code>str</code> is a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>,</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (str <span class="operator">=</span><span class="operator">=</span> <span class="string">"auto"</span> <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> <span class="string">"extern"</span>
/// &#32;   &#32;     <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> <span class="string">"static"</span> <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> <span class="string">"register"</span>) {
/// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   }
///
/// </pre>
/// <p>is much faster than</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (str <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>(<span class="string">"auto"</span>) <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>(<span class="string">"extern"</span>)
/// &#32;   &#32;     <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>(<span class="string">"static"</span>) <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>(<span class="string">"register"</span>)) {
/// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   }
///
/// </pre>
/// <p>because it doesn't construct four temporary <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> objects and make a deep copy of the character data.</p>
/// <p>Applications that define <code>QT_NO_CAST_FROM_ASCII</code> (as explained in the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> documentation) don't have access to <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>'s <code>const char *</code> API. To provide an efficient way of specifying constant Latin-1 strings, Qt provides the <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>, which is just a very thin wrapper around a <code>const char *</code>. Using <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>, the example code above becomes</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (str <span class="operator">=</span><span class="operator">=</span> QLatin1String(<span class="string">"auto"</span>)
/// &#32;   &#32;     <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> QLatin1String(<span class="string">"extern"</span>)
/// &#32;   &#32;     <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> QLatin1String(<span class="string">"static"</span>)
/// &#32;   &#32;     <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> QLatin1String(<span class="string">"register"</span>) {
/// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   }
///
/// </pre>
/// <p>This is a bit longer to type, but it provides exactly the same benefits as the first version of the code, and is faster than converting the Latin-1 strings using <a href="http://doc.qt.io/qt-5/qstring.html#fromLatin1">QString::fromLatin1</a>().</p>
/// <p>Thanks to the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>(<a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>) constructor, <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> can be used everywhere a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> is expected. For example:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlabel.html">QLabel</a></span> <span class="operator">*</span>label <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qlabel.html">QLabel</a></span>(QLatin1String(<span class="string">"MOD"</span>)<span class="operator">,</span> <span class="keyword">this</span>);
///
/// </pre>
/// <p><b>Note: </b>If the function you're calling with a <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> argument isn't actually overloaded to take <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>, the implicit conversion to <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> will trigger a memory allocation, which is usually what you want to avoid by using <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> in the first place. In those cases, using <a href="http://doc.qt.io/qt-5/qstring.html#QStringLiteral">QStringLiteral</a> may be the better option.</p></div>
#[repr(C)]
pub struct QLatin1String {
    _unused: u8,
}
impl QLatin1String {
    /// <p>Returns the character at position <i>pos</i> in this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLatin1Char QLatin1String::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the character at position <i>pos</i> in this object.</p>
    /// <p><b>Note: </b>This function performs no error checking. The behavior is undefined when <i>pos</i> &lt; 0 or <i>pos</i> &gt;= <a href="http://doc.qt.io/qt-5/qlatin1string.html#size">size</a>().</p><p>This function was introduced in  Qt 5.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlatin1string.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QLatin1Char> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLatin1String_at(self as *const crate::QLatin1String, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the last character in the string. Same as <code>at(size() - 1)</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLatin1Char QLatin1String::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the last character in the string. Same as <code>at(size() - 1)</code>.</p>
    /// <p>This function is provided for STL compatibility.</p>
    /// <p><b>Warning:</b> Calling this function on an empty string constitutes undefined behavior.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlatin1string.html#front">front</a>(), <a href="http://doc.qt.io/qt-5/qlatin1string.html#at">at</a>(), and <a href="http://doc.qt.io/qt-5/qlatin1string.html#operator-5b-5d">operator[]</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn back(&self) -> ::cpp_utils::CppBox<crate::QLatin1Char> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLatin1String_back(self as *const crate::QLatin1String);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Same as <a href="http://doc.qt.io/qt-5/qlatin1string.html#begin">begin</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QLatin1String::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as <a href="http://doc.qt.io/qt-5/qlatin1string.html#begin">begin</a>().</p>
    /// <p>This function is provided for STL compatibility.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlatin1string.html#cend">cend</a>(), <a href="http://doc.qt.io/qt-5/qlatin1string.html#begin">begin</a>(), <a href="http://doc.qt.io/qt-5/qlatin1string.html#crbegin">crbegin</a>(), and <a href="http://doc.qt.io/qt-5/qlatin1string.html#data">data</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn cbegin(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLatin1String_cbegin(self as *const crate::QLatin1String);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Same as <a href="http://doc.qt.io/qt-5/qlatin1string.html#end">end</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QLatin1String::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as <a href="http://doc.qt.io/qt-5/qlatin1string.html#end">end</a>().</p>
    /// <p>This function is provided for STL compatibility.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlatin1string.html#cbegin">cbegin</a>(), <a href="http://doc.qt.io/qt-5/qlatin1string.html#end">end</a>(), and <a href="http://doc.qt.io/qt-5/qlatin1string.html#crend">crend</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn cend(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLatin1String_cend(self as *const crate::QLatin1String);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Truncates this string by <i>length</i> characters.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QLatin1String::chop(int n)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#chop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Truncates this string by <i>length</i> characters.</p>
    /// <p>Same as <code>*this = left(size() - length)</code>.</p>
    /// <p><b>Note: </b>The behavior is undefined when <i>length</i> &lt; 0 or <i>length</i> &gt; <a href="http://doc.qt.io/qt-5/qlatin1string.html#size">size</a>().</p><p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlatin1string.html#mid">mid</a>(), <a href="http://doc.qt.io/qt-5/qlatin1string.html#left">left</a>(), <a href="http://doc.qt.io/qt-5/qlatin1string.html#right">right</a>(), <a href="http://doc.qt.io/qt-5/qlatin1string.html#chopped">chopped</a>(), and <a href="http://doc.qt.io/qt-5/qlatin1string.html#truncate">truncate</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn chop(&mut self, n: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QLatin1String_chop(self as *mut crate::QLatin1String, n)
    }

    /// <p>Returns the substring of length <a href="http://doc.qt.io/qt-5/qlatin1string.html#size">size</a>() - <i>length</i> starting at the beginning of this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLatin1String QLatin1String::chopped(int n) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#chopped">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the substring of length <a href="http://doc.qt.io/qt-5/qlatin1string.html#size">size</a>() - <i>length</i> starting at the beginning of this object.</p>
    /// <p>Same as <code>left(size() - length)</code>.</p>
    /// <p><b>Note: </b>The behavior is undefined when <i>length</i> &lt; 0 or <i>length</i> &gt; <a href="http://doc.qt.io/qt-5/qlatin1string.html#size">size</a>().</p><p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlatin1string.html#mid">mid</a>(), <a href="http://doc.qt.io/qt-5/qlatin1string.html#left">left</a>(), <a href="http://doc.qt.io/qt-5/qlatin1string.html#right">right</a>(), <a href="http://doc.qt.io/qt-5/qlatin1string.html#chop">chop</a>(), and <a href="http://doc.qt.io/qt-5/qlatin1string.html#truncate">truncate</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn chopped(
        &self,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QLatin1String> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLatin1String_chopped(
            self as *const crate::QLatin1String,
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> class provides a thin wrapper around an US-ASCII/Latin-1 encoded string literal.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLatin1String& QLatin1String::operator=(const QLatin1String& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> class provides a thin wrapper around an US-ASCII/Latin-1 encoded string literal.</p>
    /// <p>Many of <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>'s member functions are overloaded to accept <code>const char *</code> instead of <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. This includes the copy constructor, the assignment operator, the comparison operators, and various other functions such as <a href="http://doc.qt.io/qt-5/qstring.html#insert">insert()</a>, <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace()</a>, and <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf()</a>. These functions are usually optimized to avoid constructing a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> object for the <code>const char *</code> data. For example, assuming <code>str</code> is a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>,</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">if</span> (str <span class="operator">=</span><span class="operator">=</span> <span class="string">"auto"</span> <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> <span class="string">"extern"</span>
    /// &#32;   &#32;     <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> <span class="string">"static"</span> <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> <span class="string">"register"</span>) {
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   }
    ///
    /// </pre>
    /// <p>is much faster than</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">if</span> (str <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>(<span class="string">"auto"</span>) <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>(<span class="string">"extern"</span>)
    /// &#32;   &#32;     <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>(<span class="string">"static"</span>) <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>(<span class="string">"register"</span>)) {
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   }
    ///
    /// </pre>
    /// <p>because it doesn't construct four temporary <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> objects and make a deep copy of the character data.</p>
    /// <p>Applications that define <code>QT_NO_CAST_FROM_ASCII</code> (as explained in the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> documentation) don't have access to <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>'s <code>const char *</code> API. To provide an efficient way of specifying constant Latin-1 strings, Qt provides the <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>, which is just a very thin wrapper around a <code>const char *</code>. Using <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>, the example code above becomes</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">if</span> (str <span class="operator">=</span><span class="operator">=</span> QLatin1String(<span class="string">"auto"</span>)
    /// &#32;   &#32;     <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> QLatin1String(<span class="string">"extern"</span>)
    /// &#32;   &#32;     <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> QLatin1String(<span class="string">"static"</span>)
    /// &#32;   &#32;     <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> QLatin1String(<span class="string">"register"</span>) {
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   }
    ///
    /// </pre>
    /// <p>This is a bit longer to type, but it provides exactly the same benefits as the first version of the code, and is faster than converting the Latin-1 strings using <a href="http://doc.qt.io/qt-5/qstring.html#fromLatin1">QString::fromLatin1</a>().</p>
    /// <p>Thanks to the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>(<a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>) constructor, <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> can be used everywhere a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> is expected. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlabel.html">QLabel</a></span> <span class="operator">*</span>label <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qlabel.html">QLabel</a></span>(QLatin1String(<span class="string">"MOD"</span>)<span class="operator">,</span> <span class="keyword">this</span>);
    ///
    /// </pre>
    /// <p><b>Note: </b>If the function you're calling with a <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> argument isn't actually overloaded to take <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>, the implicit conversion to <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> will trigger a memory allocation, which is usually what you want to avoid by using <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> in the first place. In those cases, using <a href="http://doc.qt.io/qt-5/qstring.html#QStringLiteral">QStringLiteral</a> may be the better option.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::MutRef<crate::QLatin1String> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLatin1String_operator_6(
            self as *mut crate::QLatin1String,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the Latin-1 string stored in this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QLatin1String::data() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the Latin-1 string stored in this object.</p></div>
    pub unsafe fn data(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLatin1String_data(self as *const crate::QLatin1String);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::endsWith(QStringView s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn ends_with_q_string_view_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLatin1String_endsWith(
            self as *const crate::QLatin1String,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
            cs,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::endsWith(QLatin1String s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn ends_with_q_latin1_string_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLatin1String_endsWith1(
            self as *const crate::QLatin1String,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
            cs,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::endsWith(QChar c) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn ends_with_q_char(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLatin1String_endsWith2(
            self as *const crate::QLatin1String,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::endsWith(QChar c, Qt::CaseSensitivity cs) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn ends_with_q_char_case_sensitivity(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLatin1String_endsWith3(
            self as *const crate::QLatin1String,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
            cs,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::endsWith(QStringView s) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn ends_with_q_string_view(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLatin1String_endsWith4(
            self as *const crate::QLatin1String,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::endsWith(QLatin1String s) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn ends_with_q_latin1_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLatin1String_endsWith5(
            self as *const crate::QLatin1String,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the first character in the string. Same as <code>at(0)</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLatin1Char QLatin1String::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the first character in the string. Same as <code>at(0)</code>.</p>
    /// <p>This function is provided for STL compatibility.</p>
    /// <p><b>Warning:</b> Calling this function on an empty string constitutes undefined behavior.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlatin1string.html#back">back</a>(), <a href="http://doc.qt.io/qt-5/qlatin1string.html#at">at</a>(), and <a href="http://doc.qt.io/qt-5/qlatin1string.html#operator-5b-5d">operator[]</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn front(&self) -> ::cpp_utils::CppBox<crate::QLatin1Char> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLatin1String_front(self as *const crate::QLatin1String);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the character at position <i>pos</i> in this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLatin1Char QLatin1String::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the character at position <i>pos</i> in this object.</p>
    /// <p><b>Note: </b>This function performs no error checking. The behavior is undefined when <i>pos</i> &lt; 0 or <i>pos</i> &gt;= <a href="http://doc.qt.io/qt-5/qlatin1string.html#size">size</a>().</p><p>This function was introduced in  Qt 5.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlatin1string.html#at">at</a>().</p></div>
    pub unsafe fn index(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QLatin1Char> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLatin1String_operator__(
            self as *const crate::QLatin1String,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns whether the Latin-1 string stored in this object is empty (<code>size() == 0</code>) or not.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns whether the Latin-1 string stored in this object is empty (<code>size() == 0</code>) or not.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlatin1string.html#isNull">isNull</a>() and <a href="http://doc.qt.io/qt-5/qlatin1string.html#size">size</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLatin1String_isEmpty(self as *const crate::QLatin1String)
    }

    /// <p>Returns whether the Latin-1 string stored in this object is null (<code>data() == nullptr</code>) or not.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::isNull() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#isNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns whether the Latin-1 string stored in this object is null (<code>data() == nullptr</code>) or not.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlatin1string.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlatin1string.html#data">data</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLatin1String_isNull(self as *const crate::QLatin1String)
    }

    /// <p>Returns the Latin-1 string stored in this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QLatin1String::latin1() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#latin1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the Latin-1 string stored in this object.</p></div>
    pub unsafe fn latin1(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLatin1String_latin1(self as *const crate::QLatin1String);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns the substring of length <i>length</i> starting at position 0 in this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLatin1String QLatin1String::left(int n) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#left">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the substring of length <i>length</i> starting at position 0 in this object.</p>
    /// <p><b>Note: </b>This function performs no error checking. The behavior is undefined when <i>length</i> &lt; 0 or <i>length</i> &gt; <a href="http://doc.qt.io/qt-5/qlatin1string.html#size">size</a>().</p><p>This function was introduced in  Qt 5.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlatin1string.html#mid">mid</a>() and <a href="http://doc.qt.io/qt-5/qlatin1string.html#right">right</a>().</p></div>
    pub unsafe fn left(
        &self,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QLatin1String> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLatin1String_left(
            self as *const crate::QLatin1String,
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the substring starting at position <i>start</i> in this object, and extending to the end of the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLatin1String QLatin1String::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the substring starting at position <i>start</i> in this object, and extending to the end of the string.</p>
    /// <p><b>Note: </b>This function performs no error checking. The behavior is undefined when <i>start</i> &lt; 0 or <i>start</i> &gt; <a href="http://doc.qt.io/qt-5/qlatin1string.html#size">size</a>().</p><p>This function was introduced in  Qt 5.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlatin1string.html#left">left</a>() and <a href="http://doc.qt.io/qt-5/qlatin1string.html#right">right</a>().</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QLatin1String> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLatin1String_mid(
            self as *const crate::QLatin1String,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLatin1String QLatin1String::mid(int pos, int n) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#mid-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the substring of length <i>length</i> starting at position <i>start</i> in this object.</p>
    /// <p><b>Note: </b>This function performs no error checking. The behavior is undefined when <i>start</i> &lt; 0, <i>length</i> &lt; 0, or <i>start</i> + <i>length</i> &gt; <a href="http://doc.qt.io/qt-5/qlatin1string.html#size">size</a>().</p><p>This function was introduced in  Qt 5.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlatin1string.html#left">left</a>() and <a href="http://doc.qt.io/qt-5/qlatin1string.html#right">right</a>().</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QLatin1String> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLatin1String_mid1(
            self as *const crate::QLatin1String,
            pos,
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> object that stores a nullptr.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLatin1String::QLatin1String()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#QLatin1String">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> object that stores a nullptr.</p>
    /// <p>This function was introduced in  Qt 5.6.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QLatin1String> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLatin1String_QLatin1String();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> object that stores <i>str</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLatin1String::QLatin1String(const char* s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#QLatin1String-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> object that stores <i>str</i>.</p>
    /// <p>The string data is <i>not</i> copied. The caller must be able to guarantee that <i>str</i> will not be deleted or modified as long as the <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> object exists.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlatin1string.html#latin1">latin1</a>().</p></div>
    pub unsafe fn from_char(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QLatin1String> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLatin1String_QLatin1String1(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> object that stores <i>str</i> with <i>size</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLatin1String::QLatin1String(const char* s, int sz)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#QLatin1String-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> object that stores <i>str</i> with <i>size</i>.</p>
    /// <p>The string data is <i>not</i> copied. The caller must be able to guarantee that <i>str</i> will not be deleted or modified as long as the <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> object exists.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlatin1string.html#latin1">latin1</a>().</p></div>
    pub unsafe fn from_char_int(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        sz: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QLatin1String> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLatin1String_QLatin1String2(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            sz,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> object that stores <i>str</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLatin1String::QLatin1String(const QByteArray& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#QLatin1String-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> object that stores <i>str</i>.</p>
    /// <p>The string data is <i>not</i> copied. The caller must be able to guarantee that <i>str</i> will not be deleted or modified as long as the <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> object exists.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlatin1string.html#latin1">latin1</a>().</p></div>
    pub unsafe fn from_q_byte_array(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QLatin1String> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLatin1String_QLatin1String3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(s).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> object that stores <i>first</i> with length (<i>last</i> - <i>first</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLatin1String::QLatin1String(const char* f, const char* l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#QLatin1String-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> object that stores <i>first</i> with length (<i>last</i> - <i>first</i>).</p>
    /// <p>The range <code>[first,last)</code> must remain valid for the lifetime of this Latin-1 string object.</p>
    /// <p>Passing <code>nullptr</code> as <i>first</i> is safe if <i>last</i> is <code>nullptr</code>, too, and results in a null Latin-1 string.</p>
    /// <p>The behavior is undefined if <i>last</i> precedes <i>first</i>, <i>first</i> is <code>nullptr</code> and <i>last</i> is not, or if <code>last - first &gt; INT_MAX</code>.</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn from_2_char(
        f: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QLatin1String> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLatin1String_QLatin1String10(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(f)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> class provides a thin wrapper around an US-ASCII/Latin-1 encoded string literal.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLatin1String::QLatin1String(const QLatin1String& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> class provides a thin wrapper around an US-ASCII/Latin-1 encoded string literal.</p>
    /// <p>Many of <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>'s member functions are overloaded to accept <code>const char *</code> instead of <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. This includes the copy constructor, the assignment operator, the comparison operators, and various other functions such as <a href="http://doc.qt.io/qt-5/qstring.html#insert">insert()</a>, <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace()</a>, and <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf()</a>. These functions are usually optimized to avoid constructing a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> object for the <code>const char *</code> data. For example, assuming <code>str</code> is a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>,</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">if</span> (str <span class="operator">=</span><span class="operator">=</span> <span class="string">"auto"</span> <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> <span class="string">"extern"</span>
    /// &#32;   &#32;     <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> <span class="string">"static"</span> <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> <span class="string">"register"</span>) {
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   }
    ///
    /// </pre>
    /// <p>is much faster than</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">if</span> (str <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>(<span class="string">"auto"</span>) <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>(<span class="string">"extern"</span>)
    /// &#32;   &#32;     <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>(<span class="string">"static"</span>) <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>(<span class="string">"register"</span>)) {
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   }
    ///
    /// </pre>
    /// <p>because it doesn't construct four temporary <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> objects and make a deep copy of the character data.</p>
    /// <p>Applications that define <code>QT_NO_CAST_FROM_ASCII</code> (as explained in the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> documentation) don't have access to <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>'s <code>const char *</code> API. To provide an efficient way of specifying constant Latin-1 strings, Qt provides the <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>, which is just a very thin wrapper around a <code>const char *</code>. Using <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>, the example code above becomes</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">if</span> (str <span class="operator">=</span><span class="operator">=</span> QLatin1String(<span class="string">"auto"</span>)
    /// &#32;   &#32;     <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> QLatin1String(<span class="string">"extern"</span>)
    /// &#32;   &#32;     <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> QLatin1String(<span class="string">"static"</span>)
    /// &#32;   &#32;     <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> QLatin1String(<span class="string">"register"</span>) {
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   }
    ///
    /// </pre>
    /// <p>This is a bit longer to type, but it provides exactly the same benefits as the first version of the code, and is faster than converting the Latin-1 strings using <a href="http://doc.qt.io/qt-5/qstring.html#fromLatin1">QString::fromLatin1</a>().</p>
    /// <p>Thanks to the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>(<a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>) constructor, <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> can be used everywhere a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> is expected. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlabel.html">QLabel</a></span> <span class="operator">*</span>label <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qlabel.html">QLabel</a></span>(QLatin1String(<span class="string">"MOD"</span>)<span class="operator">,</span> <span class="keyword">this</span>);
    ///
    /// </pre>
    /// <p><b>Note: </b>If the function you're calling with a <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> argument isn't actually overloaded to take <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>, the implicit conversion to <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> will trigger a memory allocation, which is usually what you want to avoid by using <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> in the first place. In those cases, using <a href="http://doc.qt.io/qt-5/qstring.html#QStringLiteral">QStringLiteral</a> may be the better option.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::QLatin1String> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLatin1String_QLatin1String4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the substring of length <i>length</i> starting at position <a href="http://doc.qt.io/qt-5/qlatin1string.html#size">size</a>() - <i>length</i> in this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLatin1String QLatin1String::right(int n) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#right">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the substring of length <i>length</i> starting at position <a href="http://doc.qt.io/qt-5/qlatin1string.html#size">size</a>() - <i>length</i> in this object.</p>
    /// <p><b>Note: </b>This function performs no error checking. The behavior is undefined when <i>length</i> &lt; 0 or <i>length</i> &gt; <a href="http://doc.qt.io/qt-5/qlatin1string.html#size">size</a>().</p><p>This function was introduced in  Qt 5.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlatin1string.html#mid">mid</a>() and <a href="http://doc.qt.io/qt-5/qlatin1string.html#left">left</a>().</p></div>
    pub unsafe fn right(
        &self,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QLatin1String> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLatin1String_right(
            self as *const crate::QLatin1String,
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the size of the Latin-1 string stored in this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QLatin1String::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the size of the Latin-1 string stored in this object.</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QLatin1String_size(self as *const crate::QLatin1String)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::startsWith(QStringView s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn starts_with_q_string_view_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLatin1String_startsWith(
            self as *const crate::QLatin1String,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
            cs,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::startsWith(QLatin1String s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn starts_with_q_latin1_string_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLatin1String_startsWith1(
            self as *const crate::QLatin1String,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
            cs,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::startsWith(QChar c) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn starts_with_q_char(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLatin1String_startsWith2(
            self as *const crate::QLatin1String,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::startsWith(QChar c, Qt::CaseSensitivity cs) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn starts_with_q_char_case_sensitivity(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLatin1String_startsWith3(
            self as *const crate::QLatin1String,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
            cs,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::startsWith(QStringView s) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn starts_with_q_string_view(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLatin1String_startsWith4(
            self as *const crate::QLatin1String,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::startsWith(QLatin1String s) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn starts_with_q_latin1_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLatin1String_startsWith5(
            self as *const crate::QLatin1String,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// <p>Strips leading and trailing whitespace and returns the result.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLatin1String QLatin1String::trimmed() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#trimmed">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Strips leading and trailing whitespace and returns the result.</p>
    /// <p>Whitespace means any character for which <a href="http://doc.qt.io/qt-5/qchar.html#isSpace">QChar::isSpace</a>() returns <code>true</code>. This includes the ASCII characters '\t', '\n', '\v', '\f', '\r', and ' '.</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn trimmed(&self) -> ::cpp_utils::CppBox<crate::QLatin1String> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLatin1String_trimmed(
            self as *const crate::QLatin1String,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Truncates this string to length <i>length</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QLatin1String::truncate(int n)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#truncate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Truncates this string to length <i>length</i>.</p>
    /// <p>Same as <code>*this = left(length)</code>.</p>
    /// <p><b>Note: </b>The behavior is undefined when <i>length</i> &lt; 0 or <i>length</i> &gt; <a href="http://doc.qt.io/qt-5/qlatin1string.html#size">size</a>().</p><p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlatin1string.html#mid">mid</a>(), <a href="http://doc.qt.io/qt-5/qlatin1string.html#left">left</a>(), <a href="http://doc.qt.io/qt-5/qlatin1string.html#right">right</a>(), <a href="http://doc.qt.io/qt-5/qlatin1string.html#chopped">chopped</a>(), and <a href="http://doc.qt.io/qt-5/qlatin1string.html#chop">chop</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn truncate(&mut self, n: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QLatin1String_truncate(self as *mut crate::QLatin1String, n)
    }
}

pub mod q_string {
    //! C++ type: <span style='color: green;'>```QString```</span>

    /// <p>This enum specifies flags that can be used to affect various aspects of the <a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>() function's behavior with respect to separators and empty fields.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QString::SectionFlag```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#SectionFlag-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum specifies flags that can be used to affect various aspects of the <a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>() function's behavior with respect to separators and empty fields.</p>
    ///
    /// <p>The SectionFlags type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;SectionFlag&gt;. It stores an OR combination of SectionFlag values.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct SectionFlag(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for SectionFlag {
        fn from(value: ::std::os::raw::c_int) -> Self {
            SectionFlag(value)
        }
    }

    impl From<SectionFlag> for ::std::os::raw::c_int {
        fn from(value: SectionFlag) -> Self {
            value.0
        }
    }

    impl SectionFlag {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl SectionFlag {
        /// Empty fields are counted, leading and trailing separators are not included, and the separator is compared case sensitively. (C++ enum variant: <span style='color: green;'>```SectionDefault = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const SectionDefault: crate::q_string::SectionFlag = crate::q_string::SectionFlag(0);
        /// Treat empty fields as if they don't exist, i.e. they are not considered as far as <i>start</i> and <i>end</i> are concerned. (C++ enum variant: <span style='color: green;'>```SectionSkipEmpty = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const SectionSkipEmpty: crate::q_string::SectionFlag = crate::q_string::SectionFlag(1);
        /// Include the leading separator (if any) in the result string. (C++ enum variant: <span style='color: green;'>```SectionIncludeLeadingSep = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const SectionIncludeLeadingSep: crate::q_string::SectionFlag =
            crate::q_string::SectionFlag(2);
        /// Include the trailing separator (if any) in the result string. (C++ enum variant: <span style='color: green;'>```SectionIncludeTrailingSep = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const SectionIncludeTrailingSep: crate::q_string::SectionFlag =
            crate::q_string::SectionFlag(4);
        /// Compare the separator case-insensitively. (C++ enum variant: <span style='color: green;'>```SectionCaseInsensitiveSeps = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const SectionCaseInsensitiveSeps: crate::q_string::SectionFlag =
            crate::q_string::SectionFlag(8);
    }

    /// <p>This enum specifies how the <a href="http://doc.qt.io/qt-5/qstring.html#split">split</a>() function should behave with respect to empty strings.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QString::SplitBehavior```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#SplitBehavior-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum specifies how the <a href="http://doc.qt.io/qt-5/qstring.html#split">split</a>() function should behave with respect to empty strings.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#split">split</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct SplitBehavior(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for SplitBehavior {
        fn from(value: ::std::os::raw::c_int) -> Self {
            SplitBehavior(value)
        }
    }

    impl From<SplitBehavior> for ::std::os::raw::c_int {
        fn from(value: SplitBehavior) -> Self {
            value.0
        }
    }

    impl SplitBehavior {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl SplitBehavior {
        /// If a field is empty, keep it in the result. (C++ enum variant: <span style='color: green;'>```KeepEmptyParts = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const KeepEmptyParts: crate::q_string::SplitBehavior =
            crate::q_string::SplitBehavior(0);
        /// If a field is empty, don't include it in the result. (C++ enum variant: <span style='color: green;'>```SkipEmptyParts = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const SkipEmptyParts: crate::q_string::SplitBehavior =
            crate::q_string::SplitBehavior(1);
    }

    /// <p>This enum describes the various normalized forms of Unicode text.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QString::NormalizationForm```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#NormalizationForm-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the various normalized forms of Unicode text.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#normalized">normalized</a>() and <a href="http://www.unicode.org/reports/tr15/">Unicode Standard Annex #15</a>.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct NormalizationForm(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for NormalizationForm {
        fn from(value: ::std::os::raw::c_int) -> Self {
            NormalizationForm(value)
        }
    }

    impl From<NormalizationForm> for ::std::os::raw::c_int {
        fn from(value: NormalizationForm) -> Self {
            value.0
        }
    }

    impl NormalizationForm {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl NormalizationForm {
        /// Canonical Decomposition (C++ enum variant: <span style='color: green;'>```NormalizationForm_D = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NormalizationFormD: crate::q_string::NormalizationForm =
            crate::q_string::NormalizationForm(0);
        /// Canonical Decomposition followed by Canonical Composition (C++ enum variant: <span style='color: green;'>```NormalizationForm_C = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const NormalizationFormC: crate::q_string::NormalizationForm =
            crate::q_string::NormalizationForm(1);
        /// Compatibility Decomposition (C++ enum variant: <span style='color: green;'>```NormalizationForm_KD = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const NormalizationFormKD: crate::q_string::NormalizationForm =
            crate::q_string::NormalizationForm(2);
        /// Compatibility Decomposition followed by Canonical Composition (C++ enum variant: <span style='color: green;'>```NormalizationForm_KC = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const NormalizationFormKC: crate::q_string::NormalizationForm =
            crate::q_string::NormalizationForm(3);
    }

    impl From<crate::q_string::SectionFlag> for crate::QFlags<crate::q_string::SectionFlag> {
        fn from(value: crate::q_string::SectionFlag) -> Self {
            Self::from(value.to_int())
        }
    }

    impl<T: Into<crate::QFlags<crate::q_string::SectionFlag>>> std::ops::BitOr<T>
        for crate::q_string::SectionFlag
    {
        type Output = crate::QFlags<crate::q_string::SectionFlag>;
        fn bitor(self, rhs: T) -> crate::QFlags<crate::q_string::SectionFlag> {
            Into::<crate::QFlags<crate::q_string::SectionFlag>>::into(self) | rhs
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> class provides a Unicode character string.</p>
///
/// C++ class: <span style='color: green;'>```QString```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qstring.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> class provides a Unicode character string.</p>
/// <p><a href="http://doc.qt.io/qt-5/qstring.html">QString</a> stores a string of 16-bit <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>s, where each <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> corresponds one Unicode 4.0 character. (Unicode characters with code values above 65535 are stored using surrogate pairs, i.e., two consecutive <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>s.)</p>
/// <p><a href="http://doc.qt.io/qt-5/unicode.html">Unicode</a> is an international standard that supports most of the writing systems in use today. It is a superset of US-ASCII (ANSI X3.4-1986) and Latin-1 (ISO 8859-1), and all the US-ASCII/Latin-1 characters are available at the same code positions.</p>
/// <p>Behind the scenes, <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> uses <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicit sharing</a> (copy-on-write) to reduce memory usage and to avoid the needless copying of data. This also helps reduce the inherent overhead of storing 16-bit characters instead of 8-bit characters.</p>
/// <p>In addition to <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, Qt also provides the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> class to store raw bytes and traditional 8-bit '\0'-terminated strings. For most purposes, <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> is the class you want to use. It is used throughout the Qt API, and the Unicode support ensures that your applications will be easy to translate if you want to expand your application's market at some point. The two main cases where <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> is appropriate are when you need to store raw binary data, and when memory conservation is critical (like in embedded systems).</p>
/// <a name="initializing-a-string"></a></div>
#[repr(C)]
pub struct QString {
    _unused: u8,
}
impl QString {
    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#append">append</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::append(QChar c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#append-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#append">append</a>().</p>
    /// <p>Appends the character <i>ch</i> to this string.</p></div>
    pub unsafe fn append_q_char(
        &mut self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_append(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#append">append</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::append(const QChar* uc, int len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#append-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#append">append</a>().</p>
    /// <p>Appends <i>len</i> characters from the <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> array <i>str</i> to this string.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn append_q_char_int(
        &mut self,
        uc: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QChar>>,
        len: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_append1(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QChar>>::cast_into(uc).as_raw_ptr(),
            len,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Appends the string <i>str</i> onto the end of this string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::append(const QString& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the string <i>str</i> onto the end of this string.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> x <span class="operator">=</span> <span class="string">"free"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> y <span class="operator">=</span> <span class="string">"dom"</span>;
    ///
    ///   x<span class="operator">.</span>append(y);
    ///   <span class="comment">// x == "freedom"</span>
    ///
    /// </pre>
    /// <p>This is the same as using the <a href="http://doc.qt.io/qt-5/qstring.html#insert">insert</a>() function:</p>
    /// <pre class="cpp">
    ///
    ///   x<span class="operator">.</span>insert(x<span class="operator">.</span>size()<span class="operator">,</span> y);
    ///
    /// </pre>
    /// <p>The append() function is typically very fast (<a href="http://doc.qt.io/qt-5/containers.html#constant-time">constant time</a>), because <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> preallocates extra space at the end of the string data so it can grow without reallocating the entire string each time.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#operator-2b-eq">operator+=</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#insert">insert</a>().</p></div>
    pub unsafe fn append_q_string(
        &mut self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_append2(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Appends the given string <i>reference</i> to this string and returns the result.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::append(const QStringRef& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#append-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the given string <i>reference</i> to this string and returns the result.</p>
    /// <p>This function was introduced in  Qt 4.4.</p></div>
    pub unsafe fn append_q_string_ref(
        &mut self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_append3(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#append">append</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::append(QLatin1String s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#append-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#append">append</a>().</p>
    /// <p>Appends the Latin-1 string <i>str</i> to this string.</p></div>
    pub unsafe fn append_q_latin1_string(
        &mut self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_append4(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#append">append</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::append(const char* s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#append-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#append">append</a>().</p>
    /// <p>Appends the string <i>str</i> to this string. The given const char pointer is converted to Unicode using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>() function.</p>
    /// <p>You can disable this function by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p></div>
    pub unsafe fn append_char(
        &mut self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_append5(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#append">append</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::append(const QByteArray& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#append-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#append">append</a>().</p>
    /// <p>Appends the byte array <i>ba</i> to this string. The given byte array is converted to Unicode using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>() function.</p>
    /// <p>You can disable this function by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p></div>
    pub unsafe fn append_q_byte_array(
        &mut self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_append6(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(s).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(qlonglong a, int fieldwidth = …, int base = …, QChar fillChar = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-11">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The <i>base</i> argument specifies the base to use when converting the integer <i>a</i> into a string. The base must be between 2 and 36, with 8 giving octal, 10 decimal, and 16 hexadecimal numbers.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_i64_2_int_q_char(
        &self,
        a: i64,
        fieldwidth: ::std::os::raw::c_int,
        base: ::std::os::raw::c_int,
        fill_char: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg(
            self as *const crate::QString,
            a,
            fieldwidth,
            base,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(fill_char)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(qulonglong a, int fieldwidth = …, int base = …, QChar fillChar = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The <i>base</i> argument specifies the base to use when converting the integer <i>a</i> into a string. <i>base</i> must be between 2 and 36, with 8 giving octal, 10 decimal, and 16 hexadecimal numbers.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_u64_2_int_q_char(
        &self,
        a: u64,
        fieldwidth: ::std::os::raw::c_int,
        base: ::std::os::raw::c_int,
        fill_char: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg1(
            self as *const crate::QString,
            a,
            fieldwidth,
            base,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(fill_char)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(long a, int fieldwidth = …, int base = …, QChar fillChar = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The <i>a</i> argument is expressed in the given <i>base</i>, which is 10 by default and must be between 2 and 36.</p>
    /// <p>The '%' can be followed by an 'L', in which case the sequence is replaced with a localized representation of <i>a</i>. The conversion uses the default locale. The default locale is determined from the system's locale settings at application startup. It can be changed using <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">QLocale::setDefault</a>(). The 'L' flag is ignored if <i>base</i> is not 10.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str;
    ///   str <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"Decimal 63 is %1 in hexadecimal"</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">63</span><span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="number">16</span>);
    ///   <span class="comment">// str == "Decimal 63 is 3f in hexadecimal"</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span><span class="operator">::</span>setDefault(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span>(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span><span class="operator">::</span>English<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span><span class="operator">::</span>UnitedStates));
    ///   str <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"%1 %L2 %L3"</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">12345</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">12345</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">12345</span><span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="number">16</span>);
    ///   <span class="comment">// str == "12345 12,345 3039"</span>
    ///
    /// </pre>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_long2_int_q_char(
        &self,
        a: ::std::os::raw::c_long,
        fieldwidth: ::std::os::raw::c_int,
        base: ::std::os::raw::c_int,
        fill_char: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg2(
            self as *const crate::QString,
            a,
            fieldwidth,
            base,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(fill_char)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(unsigned long a, int fieldwidth = …, int base = …, QChar fillChar = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The <i>base</i> argument specifies the base to use when converting the integer <i>a</i> to a string. The base must be between 2 and 36, with 8 giving octal, 10 decimal, and 16 hexadecimal numbers.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_ulong2_int_q_char(
        &self,
        a: ::std::os::raw::c_ulong,
        fieldwidth: ::std::os::raw::c_int,
        base: ::std::os::raw::c_int,
        fill_char: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg3(
            self as *const crate::QString,
            a,
            fieldwidth,
            base,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(fill_char)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(int a, int fieldWidth = …, int base = …, QChar fillChar = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p>The <i>a</i> argument is expressed in base <i>base</i>, which is 10 by default and must be between 2 and 36. For bases other than 10, <i>a</i> is treated as an unsigned integer.</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The '%' can be followed by an 'L', in which case the sequence is replaced with a localized representation of <i>a</i>. The conversion uses the default locale, set by <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">QLocale::setDefault</a>(). If no default locale was specified, the "C" locale is used. The 'L' flag is ignored if <i>base</i> is not 10.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str;
    ///   str <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"Decimal 63 is %1 in hexadecimal"</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">63</span><span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="number">16</span>);
    ///   <span class="comment">// str == "Decimal 63 is 3f in hexadecimal"</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span><span class="operator">::</span>setDefault(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span>(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span><span class="operator">::</span>English<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span><span class="operator">::</span>UnitedStates));
    ///   str <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"%1 %L2 %L3"</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">12345</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">12345</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">12345</span><span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="number">16</span>);
    ///   <span class="comment">// str == "12345 12,345 3039"</span>
    ///
    /// </pre>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_3_int_q_char(
        &self,
        a: ::std::os::raw::c_int,
        field_width: ::std::os::raw::c_int,
        base: ::std::os::raw::c_int,
        fill_char: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg4(
            self as *const crate::QString,
            a,
            field_width,
            base,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(fill_char)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(unsigned int a, int fieldWidth = …, int base = …, QChar fillChar = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p>The <i>base</i> argument specifies the base to use when converting the integer <i>a</i> into a string. The base must be between 2 and 36.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_uint2_int_q_char(
        &self,
        a: ::std::os::raw::c_uint,
        field_width: ::std::os::raw::c_int,
        base: ::std::os::raw::c_int,
        fill_char: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg5(
            self as *const crate::QString,
            a,
            field_width,
            base,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(fill_char)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(short a, int fieldWidth = …, int base = …, QChar fillChar = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The <i>base</i> argument specifies the base to use when converting the integer <i>a</i> into a string. The base must be between 2 and 36, with 8 giving octal, 10 decimal, and 16 hexadecimal numbers.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_short2_int_q_char(
        &self,
        a: ::std::os::raw::c_short,
        field_width: ::std::os::raw::c_int,
        base: ::std::os::raw::c_int,
        fill_char: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg6(
            self as *const crate::QString,
            a,
            field_width,
            base,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(fill_char)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(unsigned short a, int fieldWidth = …, int base = …, QChar fillChar = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The <i>base</i> argument specifies the base to use when converting the integer <i>a</i> into a string. The base must be between 2 and 36, with 8 giving octal, 10 decimal, and 16 hexadecimal numbers.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_ushort2_int_q_char(
        &self,
        a: ::std::os::raw::c_ushort,
        field_width: ::std::os::raw::c_int,
        base: ::std::os::raw::c_int,
        fill_char: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg7(
            self as *const crate::QString,
            a,
            field_width,
            base,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(fill_char)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(double a, int fieldWidth = …, char fmt = …, int prec = …, QChar fillChar = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p>Argument <i>a</i> is formatted according to the specified <i>format</i> and <i>precision</i>. See <a href="http://doc.qt.io/qt-5/qstring.html#argument-formats">Argument Formats</a> for details.</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">double</span> d <span class="operator">=</span> <span class="number">12.34</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"delta: %1"</span>)<span class="operator">.</span>arg(d<span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="char">'E'</span><span class="operator">,</span> <span class="number">3</span>);
    ///   <span class="comment">// str == "delta: 1.234E+01"</span>
    ///
    /// </pre>
    /// <p>The '%' can be followed by an 'L', in which case the sequence is replaced with a localized representation of <i>a</i>. The conversion uses the default locale, set by <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">QLocale::setDefault</a>(). If no default locale was specified, the "C" locale is used.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), this function will use the locale's zero to pad. For negative numbers, the zero padding will probably appear before the minus sign.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>().</p></div>
    pub unsafe fn arg_double_int_char_int_q_char(
        &self,
        a: ::std::os::raw::c_double,
        field_width: ::std::os::raw::c_int,
        fmt: ::std::os::raw::c_char,
        prec: ::std::os::raw::c_int,
        fill_char: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg8(
            self as *const crate::QString,
            a,
            field_width,
            fmt,
            prec,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(fill_char)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(char a, int fieldWidth = …, QChar fillChar = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-9">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p>The <i>a</i> argument is interpreted as a Latin-1 character.</p></div>
    pub unsafe fn arg_char_int_q_char(
        &self,
        a: ::std::os::raw::c_char,
        field_width: ::std::os::raw::c_int,
        fill_char: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg9(
            self as *const crate::QString,
            a,
            field_width,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(fill_char)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(QChar a, int fieldWidth = …, QChar fillChar = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-10">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p></div>
    pub unsafe fn arg_q_char_int_q_char(
        &self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        field_width: ::std::os::raw::c_int,
        fill_char: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg10(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(a).as_raw_ptr(),
            field_width,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(fill_char)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a copy of this string with the lowest numbered place marker replaced by string <i>a</i>, i.e., <code>%1</code>, <code>%2</code>, ..., <code>%99</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(const QString& a, int fieldWidth = …, QChar fillChar = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a copy of this string with the lowest numbered place marker replaced by string <i>a</i>, i.e., <code>%1</code>, <code>%2</code>, ..., <code>%99</code>.</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that argument <i>a</i> shall occupy. If <i>a</i> requires less space than <i>fieldWidth</i>, it is padded to <i>fieldWidth</i> with character <i>fillChar</i>. A positive <i>fieldWidth</i> produces right-aligned text. A negative <i>fieldWidth</i> produces left-aligned text.</p>
    /// <p>This example shows how we might create a <code>status</code> string for reporting progress while processing a list of files:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> i;           <span class="comment">// current file's number</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> total;       <span class="comment">// number of files to process</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> fileName;    <span class="comment">// current file's name</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> status <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"Processing file %1 of %2: %3"</span>)
    /// &#32;   &#32;   &#32;   &#32;     <span class="operator">.</span>arg(i)<span class="operator">.</span>arg(total)<span class="operator">.</span>arg(fileName);
    ///
    /// </pre>
    /// <p>First, <code>arg(i)</code> replaces <code>%1</code>. Then <code>arg(total)</code> replaces <code>%2</code>. Finally, <code>arg(fileName)</code> replaces <code>%3</code>.</p>
    /// <p>One advantage of using arg() over <a href="http://doc.qt.io/qt-5/qstring.html#asprintf">asprintf</a>() is that the order of the numbered place markers can change, if the application's strings are translated into other languages, but each arg() will still replace the lowest numbered unreplaced place marker, no matter where it appears. Also, if place marker <code>%i</code> appears more than once in the string, the arg() replaces all of them.</p>
    /// <p>If there is no unreplaced place marker remaining, a warning message is output and the result is undefined. Place marker numbers must be in the range 1 to 99.</p></div>
    pub unsafe fn arg_q_string_int_q_char(
        &self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        field_width: ::std::os::raw::c_int,
        fill_char: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg11(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a).as_raw_ptr(),
            field_width,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(fill_char)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(const QString& a1, const QString& a2) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-12">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p>This is the same as <code>str.arg(a1).arg(a2)</code>, except that the strings <i>a1</i> and <i>a2</i> are replaced in one pass. This can make a difference if <i>a1</i> contains e.g. <code>%1</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str;
    ///   str <span class="operator">=</span> <span class="string">"%1 %2"</span>;
    ///
    ///   str<span class="operator">.</span>arg(<span class="string">"%1f"</span><span class="operator">,</span> <span class="string">"Hello"</span>);        <span class="comment">// returns "%1f Hello"</span>
    ///   str<span class="operator">.</span>arg(<span class="string">"%1f"</span>)<span class="operator">.</span>arg(<span class="string">"Hello"</span>);    <span class="comment">// returns "Hellof %2"</span>
    ///
    /// </pre>
    /// <p>A similar problem occurs when the numbered place markers are not white space separated:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str;
    ///   str <span class="operator">=</span> <span class="string">"%1%3%2"</span>;
    ///   str<span class="operator">.</span>arg(<span class="string">"Hello"</span>)<span class="operator">.</span>arg(<span class="number">20</span>)<span class="operator">.</span>arg(<span class="number">50</span>); <span class="comment">// returns "Hello500"</span>
    ///
    ///   str <span class="operator">=</span> <span class="string">"%1%2%3"</span>;
    ///   str<span class="operator">.</span>arg(<span class="string">"Hello"</span>)<span class="operator">.</span>arg(<span class="number">50</span>)<span class="operator">.</span>arg(<span class="number">20</span>); <span class="comment">// returns "Hello5020"</span>
    ///
    /// </pre>
    /// <p>Let's look at the substitutions:</p>
    /// <ul>
    /// <li>First, <code>Hello</code> replaces <code>%1</code> so the string becomes <code>"Hello%3%2"</code>.</li>
    /// <li>Then, <code>20</code> replaces <code>%2</code> so the string becomes <code>"Hello%320"</code>.</li>
    /// <li>Since the maximum numbered place marker value is 99, <code>50</code> replaces <code>%32</code>.</li>
    /// </ul>
    /// <p>Thus the string finally becomes <code>"Hello500"</code>.</p>
    /// <p>In such cases, the following yields the expected results:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str;
    ///   str <span class="operator">=</span> <span class="string">"%1%2%3"</span>;
    ///   str<span class="operator">.</span>arg(<span class="string">"Hello"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span><span class="operator">::</span>number(<span class="number">20</span>)<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span><span class="operator">::</span>number(<span class="number">50</span>)); <span class="comment">// returns "Hello5020"</span>
    ///
    /// </pre></div>
    pub unsafe fn arg_2_q_string(
        &self,
        a1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg12(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a2).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(const QString& a1, const QString& a2, const QString& a3) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-13">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p>This is the same as calling <code>str.arg(a1).arg(a2).arg(a3)</code>, except that the strings <i>a1</i>, <i>a2</i> and <i>a3</i> are replaced in one pass.</p></div>
    pub unsafe fn arg_3_q_string(
        &self,
        a1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a3: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg13(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a2).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a3).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(const QString& a1, const QString& a2, const QString& a3, const QString& a4) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-14">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p>This is the same as calling <code>str.arg(a1).arg(a2).arg(a3).arg(a4)</code>, except that the strings <i>a1</i>, <i>a2</i>, <i>a3</i> and <i>a4</i> are replaced in one pass.</p></div>
    pub unsafe fn arg_4_q_string(
        &self,
        a1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a3: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a4: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg14(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a2).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a3).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a4).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(const QString& a1, const QString& a2, const QString& a3, const QString& a4, const QString& a5) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-15">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p>This is the same as calling <code>str.arg(a1).arg(a2).arg(a3).arg(a4).arg(a5)</code>, except that the strings <i>a1</i>, <i>a2</i>, <i>a3</i>, <i>a4</i>, and <i>a5</i> are replaced in one pass.</p></div>
    pub unsafe fn arg_5_q_string(
        &self,
        a1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a3: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a4: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a5: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg15(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a2).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a3).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a4).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a5).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(const QString& a1, const QString& a2, const QString& a3, const QString& a4, const QString& a5, const QString& a6) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-16">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p>This is the same as calling <code>str.arg(a1).arg(a2).arg(a3).arg(a4).arg(a5).arg(a6))</code>, except that the strings <i>a1</i>, <i>a2</i>, <i>a3</i>, <i>a4</i>, <i>a5</i>, and <i>a6</i> are replaced in one pass.</p></div>
    pub unsafe fn arg_6_q_string(
        &self,
        a1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a3: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a4: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a5: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a6: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg16(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a2).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a3).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a4).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a5).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a6).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(const QString& a1, const QString& a2, const QString& a3, const QString& a4, const QString& a5, const QString& a6, const QString& a7) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-17">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p>This is the same as calling <code>str.arg(a1).arg(a2).arg(a3).arg(a4).arg(a5).arg(a6).arg(a7)</code>, except that the strings <i>a1</i>, <i>a2</i>, <i>a3</i>, <i>a4</i>, <i>a5</i>, <i>a6</i>, and <i>a7</i> are replaced in one pass.</p></div>
    pub unsafe fn arg_7_q_string(
        &self,
        a1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a3: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a4: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a5: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a6: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a7: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg17(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a2).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a3).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a4).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a5).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a6).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a7).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(const QString& a1, const QString& a2, const QString& a3, const QString& a4, const QString& a5, const QString& a6, const QString& a7, const QString& a8) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-18">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p>This is the same as calling <code>str.arg(a1).arg(a2).arg(a3).arg(a4).arg(a5).arg(a6).arg(a7).arg(a8)</code>, except that the strings <i>a1</i>, <i>a2</i>, <i>a3</i>, <i>a4</i>, <i>a5</i>, <i>a6</i>, <i>a7</i>, and <i>a8</i> are replaced in one pass.</p></div>
    pub unsafe fn arg_8_q_string(
        &self,
        a1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a3: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a4: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a5: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a6: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a7: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a8: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg18(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a2).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a3).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a4).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a5).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a6).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a7).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a8).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(const QString& a1, const QString& a2, const QString& a3, const QString& a4, const QString& a5, const QString& a6, const QString& a7, const QString& a8, const QString& a9) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-19">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p>This is the same as calling <code>str.arg(a1).arg(a2).arg(a3).arg(a4).arg(a5).arg(a6).arg(a7).arg(a8).arg(a9)</code>, except that the strings <i>a1</i>, <i>a2</i>, <i>a3</i>, <i>a4</i>, <i>a5</i>, <i>a6</i>, <i>a7</i>, <i>a8</i>, and <i>a9</i> are replaced in one pass.</p></div>
    pub unsafe fn arg_9_q_string(
        &self,
        a1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a3: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a4: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a5: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a6: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a7: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a8: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        a9: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg19(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a2).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a3).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a4).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a5).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a6).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a7).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a8).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a9).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(qlonglong a, int fieldwidth = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-11">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The <i>base</i> argument specifies the base to use when converting the integer <i>a</i> into a string. The base must be between 2 and 36, with 8 giving octal, 10 decimal, and 16 hexadecimal numbers.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_i64_2_int(
        &self,
        a: i64,
        fieldwidth: ::std::os::raw::c_int,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg20(
            self as *const crate::QString,
            a,
            fieldwidth,
            base,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(qlonglong a, int fieldwidth = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-11">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The <i>base</i> argument specifies the base to use when converting the integer <i>a</i> into a string. The base must be between 2 and 36, with 8 giving octal, 10 decimal, and 16 hexadecimal numbers.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_i64_int(
        &self,
        a: i64,
        fieldwidth: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg21(
            self as *const crate::QString,
            a,
            fieldwidth,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(qlonglong a) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-11">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The <i>base</i> argument specifies the base to use when converting the integer <i>a</i> into a string. The base must be between 2 and 36, with 8 giving octal, 10 decimal, and 16 hexadecimal numbers.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_i64(&self, a: i64) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_arg22(self as *const crate::QString, a);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(qulonglong a, int fieldwidth = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The <i>base</i> argument specifies the base to use when converting the integer <i>a</i> into a string. <i>base</i> must be between 2 and 36, with 8 giving octal, 10 decimal, and 16 hexadecimal numbers.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_u64_2_int(
        &self,
        a: u64,
        fieldwidth: ::std::os::raw::c_int,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg23(
            self as *const crate::QString,
            a,
            fieldwidth,
            base,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(qulonglong a, int fieldwidth = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The <i>base</i> argument specifies the base to use when converting the integer <i>a</i> into a string. <i>base</i> must be between 2 and 36, with 8 giving octal, 10 decimal, and 16 hexadecimal numbers.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_u64_int(
        &self,
        a: u64,
        fieldwidth: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg24(
            self as *const crate::QString,
            a,
            fieldwidth,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(qulonglong a) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The <i>base</i> argument specifies the base to use when converting the integer <i>a</i> into a string. <i>base</i> must be between 2 and 36, with 8 giving octal, 10 decimal, and 16 hexadecimal numbers.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_u64(&self, a: u64) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_arg25(self as *const crate::QString, a);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(long a, int fieldwidth = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The <i>a</i> argument is expressed in the given <i>base</i>, which is 10 by default and must be between 2 and 36.</p>
    /// <p>The '%' can be followed by an 'L', in which case the sequence is replaced with a localized representation of <i>a</i>. The conversion uses the default locale. The default locale is determined from the system's locale settings at application startup. It can be changed using <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">QLocale::setDefault</a>(). The 'L' flag is ignored if <i>base</i> is not 10.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str;
    ///   str <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"Decimal 63 is %1 in hexadecimal"</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">63</span><span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="number">16</span>);
    ///   <span class="comment">// str == "Decimal 63 is 3f in hexadecimal"</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span><span class="operator">::</span>setDefault(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span>(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span><span class="operator">::</span>English<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span><span class="operator">::</span>UnitedStates));
    ///   str <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"%1 %L2 %L3"</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">12345</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">12345</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">12345</span><span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="number">16</span>);
    ///   <span class="comment">// str == "12345 12,345 3039"</span>
    ///
    /// </pre>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_long2_int(
        &self,
        a: ::std::os::raw::c_long,
        fieldwidth: ::std::os::raw::c_int,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg26(
            self as *const crate::QString,
            a,
            fieldwidth,
            base,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(long a, int fieldwidth = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The <i>a</i> argument is expressed in the given <i>base</i>, which is 10 by default and must be between 2 and 36.</p>
    /// <p>The '%' can be followed by an 'L', in which case the sequence is replaced with a localized representation of <i>a</i>. The conversion uses the default locale. The default locale is determined from the system's locale settings at application startup. It can be changed using <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">QLocale::setDefault</a>(). The 'L' flag is ignored if <i>base</i> is not 10.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str;
    ///   str <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"Decimal 63 is %1 in hexadecimal"</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">63</span><span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="number">16</span>);
    ///   <span class="comment">// str == "Decimal 63 is 3f in hexadecimal"</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span><span class="operator">::</span>setDefault(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span>(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span><span class="operator">::</span>English<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span><span class="operator">::</span>UnitedStates));
    ///   str <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"%1 %L2 %L3"</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">12345</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">12345</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">12345</span><span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="number">16</span>);
    ///   <span class="comment">// str == "12345 12,345 3039"</span>
    ///
    /// </pre>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_long_int(
        &self,
        a: ::std::os::raw::c_long,
        fieldwidth: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg27(
            self as *const crate::QString,
            a,
            fieldwidth,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(long a) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The <i>a</i> argument is expressed in the given <i>base</i>, which is 10 by default and must be between 2 and 36.</p>
    /// <p>The '%' can be followed by an 'L', in which case the sequence is replaced with a localized representation of <i>a</i>. The conversion uses the default locale. The default locale is determined from the system's locale settings at application startup. It can be changed using <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">QLocale::setDefault</a>(). The 'L' flag is ignored if <i>base</i> is not 10.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str;
    ///   str <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"Decimal 63 is %1 in hexadecimal"</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">63</span><span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="number">16</span>);
    ///   <span class="comment">// str == "Decimal 63 is 3f in hexadecimal"</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span><span class="operator">::</span>setDefault(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span>(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span><span class="operator">::</span>English<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span><span class="operator">::</span>UnitedStates));
    ///   str <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"%1 %L2 %L3"</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">12345</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">12345</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">12345</span><span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="number">16</span>);
    ///   <span class="comment">// str == "12345 12,345 3039"</span>
    ///
    /// </pre>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_long(
        &self,
        a: ::std::os::raw::c_long,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_arg28(self as *const crate::QString, a);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(unsigned long a, int fieldwidth = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The <i>base</i> argument specifies the base to use when converting the integer <i>a</i> to a string. The base must be between 2 and 36, with 8 giving octal, 10 decimal, and 16 hexadecimal numbers.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_ulong2_int(
        &self,
        a: ::std::os::raw::c_ulong,
        fieldwidth: ::std::os::raw::c_int,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg29(
            self as *const crate::QString,
            a,
            fieldwidth,
            base,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(unsigned long a, int fieldwidth = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The <i>base</i> argument specifies the base to use when converting the integer <i>a</i> to a string. The base must be between 2 and 36, with 8 giving octal, 10 decimal, and 16 hexadecimal numbers.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_ulong_int(
        &self,
        a: ::std::os::raw::c_ulong,
        fieldwidth: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg30(
            self as *const crate::QString,
            a,
            fieldwidth,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(unsigned long a) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The <i>base</i> argument specifies the base to use when converting the integer <i>a</i> to a string. The base must be between 2 and 36, with 8 giving octal, 10 decimal, and 16 hexadecimal numbers.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_ulong(
        &self,
        a: ::std::os::raw::c_ulong,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_arg31(self as *const crate::QString, a);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(int a, int fieldWidth = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p>The <i>a</i> argument is expressed in base <i>base</i>, which is 10 by default and must be between 2 and 36. For bases other than 10, <i>a</i> is treated as an unsigned integer.</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The '%' can be followed by an 'L', in which case the sequence is replaced with a localized representation of <i>a</i>. The conversion uses the default locale, set by <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">QLocale::setDefault</a>(). If no default locale was specified, the "C" locale is used. The 'L' flag is ignored if <i>base</i> is not 10.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str;
    ///   str <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"Decimal 63 is %1 in hexadecimal"</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">63</span><span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="number">16</span>);
    ///   <span class="comment">// str == "Decimal 63 is 3f in hexadecimal"</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span><span class="operator">::</span>setDefault(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span>(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span><span class="operator">::</span>English<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span><span class="operator">::</span>UnitedStates));
    ///   str <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"%1 %L2 %L3"</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">12345</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">12345</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">12345</span><span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="number">16</span>);
    ///   <span class="comment">// str == "12345 12,345 3039"</span>
    ///
    /// </pre>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_3_int(
        &self,
        a: ::std::os::raw::c_int,
        field_width: ::std::os::raw::c_int,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg32(
            self as *const crate::QString,
            a,
            field_width,
            base,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(int a, int fieldWidth = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p>The <i>a</i> argument is expressed in base <i>base</i>, which is 10 by default and must be between 2 and 36. For bases other than 10, <i>a</i> is treated as an unsigned integer.</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The '%' can be followed by an 'L', in which case the sequence is replaced with a localized representation of <i>a</i>. The conversion uses the default locale, set by <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">QLocale::setDefault</a>(). If no default locale was specified, the "C" locale is used. The 'L' flag is ignored if <i>base</i> is not 10.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str;
    ///   str <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"Decimal 63 is %1 in hexadecimal"</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">63</span><span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="number">16</span>);
    ///   <span class="comment">// str == "Decimal 63 is 3f in hexadecimal"</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span><span class="operator">::</span>setDefault(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span>(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span><span class="operator">::</span>English<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span><span class="operator">::</span>UnitedStates));
    ///   str <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"%1 %L2 %L3"</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">12345</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">12345</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">12345</span><span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="number">16</span>);
    ///   <span class="comment">// str == "12345 12,345 3039"</span>
    ///
    /// </pre>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_2_int(
        &self,
        a: ::std::os::raw::c_int,
        field_width: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg33(
            self as *const crate::QString,
            a,
            field_width,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(int a) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p>The <i>a</i> argument is expressed in base <i>base</i>, which is 10 by default and must be between 2 and 36. For bases other than 10, <i>a</i> is treated as an unsigned integer.</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The '%' can be followed by an 'L', in which case the sequence is replaced with a localized representation of <i>a</i>. The conversion uses the default locale, set by <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">QLocale::setDefault</a>(). If no default locale was specified, the "C" locale is used. The 'L' flag is ignored if <i>base</i> is not 10.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str;
    ///   str <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"Decimal 63 is %1 in hexadecimal"</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">63</span><span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="number">16</span>);
    ///   <span class="comment">// str == "Decimal 63 is 3f in hexadecimal"</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span><span class="operator">::</span>setDefault(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span>(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span><span class="operator">::</span>English<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span><span class="operator">::</span>UnitedStates));
    ///   str <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"%1 %L2 %L3"</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">12345</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">12345</span>)
    /// &#32;   &#32;     <span class="operator">.</span>arg(<span class="number">12345</span><span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="number">16</span>);
    ///   <span class="comment">// str == "12345 12,345 3039"</span>
    ///
    /// </pre>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_int(&self, a: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_arg34(self as *const crate::QString, a);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(unsigned int a, int fieldWidth = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p>The <i>base</i> argument specifies the base to use when converting the integer <i>a</i> into a string. The base must be between 2 and 36.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_uint2_int(
        &self,
        a: ::std::os::raw::c_uint,
        field_width: ::std::os::raw::c_int,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg35(
            self as *const crate::QString,
            a,
            field_width,
            base,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(unsigned int a, int fieldWidth = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p>The <i>base</i> argument specifies the base to use when converting the integer <i>a</i> into a string. The base must be between 2 and 36.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_uint_int(
        &self,
        a: ::std::os::raw::c_uint,
        field_width: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg36(
            self as *const crate::QString,
            a,
            field_width,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(unsigned int a) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p>The <i>base</i> argument specifies the base to use when converting the integer <i>a</i> into a string. The base must be between 2 and 36.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_uint(
        &self,
        a: ::std::os::raw::c_uint,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_arg37(self as *const crate::QString, a);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(short a, int fieldWidth = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The <i>base</i> argument specifies the base to use when converting the integer <i>a</i> into a string. The base must be between 2 and 36, with 8 giving octal, 10 decimal, and 16 hexadecimal numbers.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_short2_int(
        &self,
        a: ::std::os::raw::c_short,
        field_width: ::std::os::raw::c_int,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg38(
            self as *const crate::QString,
            a,
            field_width,
            base,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(short a, int fieldWidth = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The <i>base</i> argument specifies the base to use when converting the integer <i>a</i> into a string. The base must be between 2 and 36, with 8 giving octal, 10 decimal, and 16 hexadecimal numbers.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_short_int(
        &self,
        a: ::std::os::raw::c_short,
        field_width: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg39(
            self as *const crate::QString,
            a,
            field_width,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(short a) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The <i>base</i> argument specifies the base to use when converting the integer <i>a</i> into a string. The base must be between 2 and 36, with 8 giving octal, 10 decimal, and 16 hexadecimal numbers.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_short(
        &self,
        a: ::std::os::raw::c_short,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_arg40(self as *const crate::QString, a);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(unsigned short a, int fieldWidth = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The <i>base</i> argument specifies the base to use when converting the integer <i>a</i> into a string. The base must be between 2 and 36, with 8 giving octal, 10 decimal, and 16 hexadecimal numbers.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_ushort2_int(
        &self,
        a: ::std::os::raw::c_ushort,
        field_width: ::std::os::raw::c_int,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg41(
            self as *const crate::QString,
            a,
            field_width,
            base,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(unsigned short a, int fieldWidth = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The <i>base</i> argument specifies the base to use when converting the integer <i>a</i> into a string. The base must be between 2 and 36, with 8 giving octal, 10 decimal, and 16 hexadecimal numbers.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_ushort_int(
        &self,
        a: ::std::os::raw::c_ushort,
        field_width: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg42(
            self as *const crate::QString,
            a,
            field_width,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(unsigned short a) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <p>The <i>base</i> argument specifies the base to use when converting the integer <i>a</i> into a string. The base must be between 2 and 36, with 8 giving octal, 10 decimal, and 16 hexadecimal numbers.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), the locale's zero is used. For negative numbers, zero padding might appear before the minus sign.</p></div>
    pub unsafe fn arg_ushort(
        &self,
        a: ::std::os::raw::c_ushort,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_arg43(self as *const crate::QString, a);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(double a, int fieldWidth = …, char fmt = …, int prec = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p>Argument <i>a</i> is formatted according to the specified <i>format</i> and <i>precision</i>. See <a href="http://doc.qt.io/qt-5/qstring.html#argument-formats">Argument Formats</a> for details.</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">double</span> d <span class="operator">=</span> <span class="number">12.34</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"delta: %1"</span>)<span class="operator">.</span>arg(d<span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="char">'E'</span><span class="operator">,</span> <span class="number">3</span>);
    ///   <span class="comment">// str == "delta: 1.234E+01"</span>
    ///
    /// </pre>
    /// <p>The '%' can be followed by an 'L', in which case the sequence is replaced with a localized representation of <i>a</i>. The conversion uses the default locale, set by <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">QLocale::setDefault</a>(). If no default locale was specified, the "C" locale is used.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), this function will use the locale's zero to pad. For negative numbers, the zero padding will probably appear before the minus sign.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>().</p></div>
    pub unsafe fn arg_double_int_char_int(
        &self,
        a: ::std::os::raw::c_double,
        field_width: ::std::os::raw::c_int,
        fmt: ::std::os::raw::c_char,
        prec: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg44(
            self as *const crate::QString,
            a,
            field_width,
            fmt,
            prec,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(double a, int fieldWidth = …, char fmt = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p>Argument <i>a</i> is formatted according to the specified <i>format</i> and <i>precision</i>. See <a href="http://doc.qt.io/qt-5/qstring.html#argument-formats">Argument Formats</a> for details.</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">double</span> d <span class="operator">=</span> <span class="number">12.34</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"delta: %1"</span>)<span class="operator">.</span>arg(d<span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="char">'E'</span><span class="operator">,</span> <span class="number">3</span>);
    ///   <span class="comment">// str == "delta: 1.234E+01"</span>
    ///
    /// </pre>
    /// <p>The '%' can be followed by an 'L', in which case the sequence is replaced with a localized representation of <i>a</i>. The conversion uses the default locale, set by <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">QLocale::setDefault</a>(). If no default locale was specified, the "C" locale is used.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), this function will use the locale's zero to pad. For negative numbers, the zero padding will probably appear before the minus sign.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>().</p></div>
    pub unsafe fn arg_double_int_char(
        &self,
        a: ::std::os::raw::c_double,
        field_width: ::std::os::raw::c_int,
        fmt: ::std::os::raw::c_char,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg45(
            self as *const crate::QString,
            a,
            field_width,
            fmt,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(double a, int fieldWidth = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p>Argument <i>a</i> is formatted according to the specified <i>format</i> and <i>precision</i>. See <a href="http://doc.qt.io/qt-5/qstring.html#argument-formats">Argument Formats</a> for details.</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">double</span> d <span class="operator">=</span> <span class="number">12.34</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"delta: %1"</span>)<span class="operator">.</span>arg(d<span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="char">'E'</span><span class="operator">,</span> <span class="number">3</span>);
    ///   <span class="comment">// str == "delta: 1.234E+01"</span>
    ///
    /// </pre>
    /// <p>The '%' can be followed by an 'L', in which case the sequence is replaced with a localized representation of <i>a</i>. The conversion uses the default locale, set by <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">QLocale::setDefault</a>(). If no default locale was specified, the "C" locale is used.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), this function will use the locale's zero to pad. For negative numbers, the zero padding will probably appear before the minus sign.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>().</p></div>
    pub unsafe fn arg_double_int(
        &self,
        a: ::std::os::raw::c_double,
        field_width: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg46(
            self as *const crate::QString,
            a,
            field_width,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(double a) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p>Argument <i>a</i> is formatted according to the specified <i>format</i> and <i>precision</i>. See <a href="http://doc.qt.io/qt-5/qstring.html#argument-formats">Argument Formats</a> for details.</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> is padded to and filled with the character <i>fillChar</i>. A positive value produces right-aligned text; a negative value produces left-aligned text.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">double</span> d <span class="operator">=</span> <span class="number">12.34</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"delta: %1"</span>)<span class="operator">.</span>arg(d<span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="char">'E'</span><span class="operator">,</span> <span class="number">3</span>);
    ///   <span class="comment">// str == "delta: 1.234E+01"</span>
    ///
    /// </pre>
    /// <p>The '%' can be followed by an 'L', in which case the sequence is replaced with a localized representation of <i>a</i>. The conversion uses the default locale, set by <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">QLocale::setDefault</a>(). If no default locale was specified, the "C" locale is used.</p>
    /// <p>If <i>fillChar</i> is '0' (the number 0, ASCII 48), this function will use the locale's zero to pad. For negative numbers, the zero padding will probably appear before the minus sign.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>().</p></div>
    pub unsafe fn arg_double(
        &self,
        a: ::std::os::raw::c_double,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_arg47(self as *const crate::QString, a);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(char a, int fieldWidth = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-9">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p>The <i>a</i> argument is interpreted as a Latin-1 character.</p></div>
    pub unsafe fn arg_char_int(
        &self,
        a: ::std::os::raw::c_char,
        field_width: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg48(
            self as *const crate::QString,
            a,
            field_width,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(char a) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-9">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    /// <p>The <i>a</i> argument is interpreted as a Latin-1 character.</p></div>
    pub unsafe fn arg_char(
        &self,
        a: ::std::os::raw::c_char,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_arg49(self as *const crate::QString, a);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(QChar a, int fieldWidth = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-10">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p></div>
    pub unsafe fn arg_q_char_int(
        &self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        field_width: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg50(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(a).as_raw_ptr(),
            field_width,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(QChar a) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-10">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>().</p></div>
    pub unsafe fn arg_q_char(
        &self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg51(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(a).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a copy of this string with the lowest numbered place marker replaced by string <i>a</i>, i.e., <code>%1</code>, <code>%2</code>, ..., <code>%99</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(const QString& a, int fieldWidth = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a copy of this string with the lowest numbered place marker replaced by string <i>a</i>, i.e., <code>%1</code>, <code>%2</code>, ..., <code>%99</code>.</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that argument <i>a</i> shall occupy. If <i>a</i> requires less space than <i>fieldWidth</i>, it is padded to <i>fieldWidth</i> with character <i>fillChar</i>. A positive <i>fieldWidth</i> produces right-aligned text. A negative <i>fieldWidth</i> produces left-aligned text.</p>
    /// <p>This example shows how we might create a <code>status</code> string for reporting progress while processing a list of files:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> i;           <span class="comment">// current file's number</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> total;       <span class="comment">// number of files to process</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> fileName;    <span class="comment">// current file's name</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> status <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"Processing file %1 of %2: %3"</span>)
    /// &#32;   &#32;   &#32;   &#32;     <span class="operator">.</span>arg(i)<span class="operator">.</span>arg(total)<span class="operator">.</span>arg(fileName);
    ///
    /// </pre>
    /// <p>First, <code>arg(i)</code> replaces <code>%1</code>. Then <code>arg(total)</code> replaces <code>%2</code>. Finally, <code>arg(fileName)</code> replaces <code>%3</code>.</p>
    /// <p>One advantage of using arg() over <a href="http://doc.qt.io/qt-5/qstring.html#asprintf">asprintf</a>() is that the order of the numbered place markers can change, if the application's strings are translated into other languages, but each arg() will still replace the lowest numbered unreplaced place marker, no matter where it appears. Also, if place marker <code>%i</code> appears more than once in the string, the arg() replaces all of them.</p>
    /// <p>If there is no unreplaced place marker remaining, a warning message is output and the result is undefined. Place marker numbers must be in the range 1 to 99.</p></div>
    pub unsafe fn arg_q_string_int(
        &self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        field_width: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg52(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a).as_raw_ptr(),
            field_width,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a copy of this string with the lowest numbered place marker replaced by string <i>a</i>, i.e., <code>%1</code>, <code>%2</code>, ..., <code>%99</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(const QString& a) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a copy of this string with the lowest numbered place marker replaced by string <i>a</i>, i.e., <code>%1</code>, <code>%2</code>, ..., <code>%99</code>.</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that argument <i>a</i> shall occupy. If <i>a</i> requires less space than <i>fieldWidth</i>, it is padded to <i>fieldWidth</i> with character <i>fillChar</i>. A positive <i>fieldWidth</i> produces right-aligned text. A negative <i>fieldWidth</i> produces left-aligned text.</p>
    /// <p>This example shows how we might create a <code>status</code> string for reporting progress while processing a list of files:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> i;           <span class="comment">// current file's number</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> total;       <span class="comment">// number of files to process</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> fileName;    <span class="comment">// current file's name</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> status <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"Processing file %1 of %2: %3"</span>)
    /// &#32;   &#32;   &#32;   &#32;     <span class="operator">.</span>arg(i)<span class="operator">.</span>arg(total)<span class="operator">.</span>arg(fileName);
    ///
    /// </pre>
    /// <p>First, <code>arg(i)</code> replaces <code>%1</code>. Then <code>arg(total)</code> replaces <code>%2</code>. Finally, <code>arg(fileName)</code> replaces <code>%3</code>.</p>
    /// <p>One advantage of using arg() over <a href="http://doc.qt.io/qt-5/qstring.html#asprintf">asprintf</a>() is that the order of the numbered place markers can change, if the application's strings are translated into other languages, but each arg() will still replace the lowest numbered unreplaced place marker, no matter where it appears. Also, if place marker <code>%i</code> appears more than once in the string, the arg() replaces all of them.</p>
    /// <p>If there is no unreplaced place marker remaining, a warning message is output and the result is undefined. Place marker numbers must be in the range 1 to 99.</p></div>
    pub unsafe fn arg_q_string(
        &self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg53(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(QStringView a, int fieldWidth = …, QChar fillChar = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-12">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a copy of this string with the lowest-numbered place-marker replaced by string <i>a</i>, i.e., <code>%1</code>, <code>%2</code>, ..., <code>%99</code>.</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> shall occupy. If <i>a</i> requires less space than <i>fieldWidth</i>, it is padded to <i>fieldWidth</i> with character <i>fillChar</i>. A positive <i>fieldWidth</i> produces right-aligned text. A negative <i>fieldWidth</i> produces left-aligned text.</p>
    /// <p>This example shows how we might create a <code>status</code> string for reporting progress while processing a list of files:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">int</span> i;                <span class="comment">// current file's number</span>
    ///   <span class="type">int</span> total;            <span class="comment">// number of files to process</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a></span> fileName; <span class="comment">// current file's name</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> status <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"Processing file %1 of %2: %3"</span>)
    /// &#32;   &#32;   &#32;   &#32;     <span class="operator">.</span>arg(i)<span class="operator">.</span>arg(total)<span class="operator">.</span>arg(fileName);
    ///
    /// </pre>
    /// <p>First, <code>arg(i)</code> replaces <code>%1</code>. Then <code>arg(total)</code> replaces <code>%2</code>. Finally, <code>arg(fileName)</code> replaces <code>%3</code>.</p>
    /// <p>One advantage of using <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>() over <a href="http://doc.qt.io/qt-5/qstring.html#asprintf">asprintf</a>() is that the order of the numbered place markers can change, if the application's strings are translated into other languages, but each <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>() will still replace the lowest-numbered unreplaced place-marker, no matter where it appears. Also, if place-marker <code>%i</code> appears more than once in the string, <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>() replaces all of them.</p>
    /// <p>If there is no unreplaced place-marker remaining, a warning message is printed and the result is undefined. Place-marker numbers must be in the range 1 to 99.</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn arg_q_string_view_int_q_char(
        &self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
        field_width: ::std::os::raw::c_int,
        fill_char: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg108(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(a)
                .as_raw_ptr(),
            field_width,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(fill_char)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(QLatin1String a, int fieldWidth = …, QChar fillChar = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-13">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a copy of this string with the lowest-numbered place-marker replaced by string <i>a</i>, i.e., <code>%1</code>, <code>%2</code>, ..., <code>%99</code>.</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> shall occupy. If <i>a</i> requires less space than <i>fieldWidth</i>, it is padded to <i>fieldWidth</i> with character <i>fillChar</i>. A positive <i>fieldWidth</i> produces right-aligned text. A negative <i>fieldWidth</i> produces left-aligned text.</p>
    /// <p>One advantage of using <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>() over <a href="http://doc.qt.io/qt-5/qstring.html#asprintf">asprintf</a>() is that the order of the numbered place markers can change, if the application's strings are translated into other languages, but each <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>() will still replace the lowest-numbered unreplaced place-marker, no matter where it appears. Also, if place-marker <code>%i</code> appears more than once in the string, <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>() replaces all of them.</p>
    /// <p>If there is no unreplaced place-marker remaining, a warning message is printed and the result is undefined. Place-marker numbers must be in the range 1 to 99.</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn arg_q_latin1_string_int_q_char(
        &self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        field_width: ::std::os::raw::c_int,
        fill_char: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg109(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(a)
                .as_raw_ptr(),
            field_width,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(fill_char)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(QStringView a, int fieldWidth = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-12">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a copy of this string with the lowest-numbered place-marker replaced by string <i>a</i>, i.e., <code>%1</code>, <code>%2</code>, ..., <code>%99</code>.</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> shall occupy. If <i>a</i> requires less space than <i>fieldWidth</i>, it is padded to <i>fieldWidth</i> with character <i>fillChar</i>. A positive <i>fieldWidth</i> produces right-aligned text. A negative <i>fieldWidth</i> produces left-aligned text.</p>
    /// <p>This example shows how we might create a <code>status</code> string for reporting progress while processing a list of files:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">int</span> i;                <span class="comment">// current file's number</span>
    ///   <span class="type">int</span> total;            <span class="comment">// number of files to process</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a></span> fileName; <span class="comment">// current file's name</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> status <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"Processing file %1 of %2: %3"</span>)
    /// &#32;   &#32;   &#32;   &#32;     <span class="operator">.</span>arg(i)<span class="operator">.</span>arg(total)<span class="operator">.</span>arg(fileName);
    ///
    /// </pre>
    /// <p>First, <code>arg(i)</code> replaces <code>%1</code>. Then <code>arg(total)</code> replaces <code>%2</code>. Finally, <code>arg(fileName)</code> replaces <code>%3</code>.</p>
    /// <p>One advantage of using <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>() over <a href="http://doc.qt.io/qt-5/qstring.html#asprintf">asprintf</a>() is that the order of the numbered place markers can change, if the application's strings are translated into other languages, but each <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>() will still replace the lowest-numbered unreplaced place-marker, no matter where it appears. Also, if place-marker <code>%i</code> appears more than once in the string, <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>() replaces all of them.</p>
    /// <p>If there is no unreplaced place-marker remaining, a warning message is printed and the result is undefined. Place-marker numbers must be in the range 1 to 99.</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn arg_q_string_view_int(
        &self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
        field_width: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg110(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(a)
                .as_raw_ptr(),
            field_width,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(QStringView a) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-12">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a copy of this string with the lowest-numbered place-marker replaced by string <i>a</i>, i.e., <code>%1</code>, <code>%2</code>, ..., <code>%99</code>.</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> shall occupy. If <i>a</i> requires less space than <i>fieldWidth</i>, it is padded to <i>fieldWidth</i> with character <i>fillChar</i>. A positive <i>fieldWidth</i> produces right-aligned text. A negative <i>fieldWidth</i> produces left-aligned text.</p>
    /// <p>This example shows how we might create a <code>status</code> string for reporting progress while processing a list of files:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">int</span> i;                <span class="comment">// current file's number</span>
    ///   <span class="type">int</span> total;            <span class="comment">// number of files to process</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a></span> fileName; <span class="comment">// current file's name</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> status <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"Processing file %1 of %2: %3"</span>)
    /// &#32;   &#32;   &#32;   &#32;     <span class="operator">.</span>arg(i)<span class="operator">.</span>arg(total)<span class="operator">.</span>arg(fileName);
    ///
    /// </pre>
    /// <p>First, <code>arg(i)</code> replaces <code>%1</code>. Then <code>arg(total)</code> replaces <code>%2</code>. Finally, <code>arg(fileName)</code> replaces <code>%3</code>.</p>
    /// <p>One advantage of using <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>() over <a href="http://doc.qt.io/qt-5/qstring.html#asprintf">asprintf</a>() is that the order of the numbered place markers can change, if the application's strings are translated into other languages, but each <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>() will still replace the lowest-numbered unreplaced place-marker, no matter where it appears. Also, if place-marker <code>%i</code> appears more than once in the string, <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>() replaces all of them.</p>
    /// <p>If there is no unreplaced place-marker remaining, a warning message is printed and the result is undefined. Place-marker numbers must be in the range 1 to 99.</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn arg_q_string_view(
        &self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg111(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(a)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(QLatin1String a, int fieldWidth = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-13">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a copy of this string with the lowest-numbered place-marker replaced by string <i>a</i>, i.e., <code>%1</code>, <code>%2</code>, ..., <code>%99</code>.</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> shall occupy. If <i>a</i> requires less space than <i>fieldWidth</i>, it is padded to <i>fieldWidth</i> with character <i>fillChar</i>. A positive <i>fieldWidth</i> produces right-aligned text. A negative <i>fieldWidth</i> produces left-aligned text.</p>
    /// <p>One advantage of using <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>() over <a href="http://doc.qt.io/qt-5/qstring.html#asprintf">asprintf</a>() is that the order of the numbered place markers can change, if the application's strings are translated into other languages, but each <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>() will still replace the lowest-numbered unreplaced place-marker, no matter where it appears. Also, if place-marker <code>%i</code> appears more than once in the string, <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>() replaces all of them.</p>
    /// <p>If there is no unreplaced place-marker remaining, a warning message is printed and the result is undefined. Place-marker numbers must be in the range 1 to 99.</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn arg_q_latin1_string_int(
        &self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        field_width: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg112(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(a)
                .as_raw_ptr(),
            field_width,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::arg(QLatin1String a) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#arg-13">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a copy of this string with the lowest-numbered place-marker replaced by string <i>a</i>, i.e., <code>%1</code>, <code>%2</code>, ..., <code>%99</code>.</p>
    /// <p><i>fieldWidth</i> specifies the minimum amount of space that <i>a</i> shall occupy. If <i>a</i> requires less space than <i>fieldWidth</i>, it is padded to <i>fieldWidth</i> with character <i>fillChar</i>. A positive <i>fieldWidth</i> produces right-aligned text. A negative <i>fieldWidth</i> produces left-aligned text.</p>
    /// <p>One advantage of using <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>() over <a href="http://doc.qt.io/qt-5/qstring.html#asprintf">asprintf</a>() is that the order of the numbered place markers can change, if the application's strings are translated into other languages, but each <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>() will still replace the lowest-numbered unreplaced place-marker, no matter where it appears. Also, if place-marker <code>%i</code> appears more than once in the string, <a href="http://doc.qt.io/qt-5/qstring.html#arg">arg</a>() replaces all of them.</p>
    /// <p>If there is no unreplaced place-marker remaining, a warning message is printed and the result is undefined. Place-marker numbers must be in the range 1 to 99.</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn arg_q_latin1_string(
        &self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_arg113(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(a)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the character at the given index <i>position</i> in the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QString::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the character at the given index <i>position</i> in the string.</p>
    /// <p>The <i>position</i> must be a valid index position in the string (i.e., 0 &lt;= <i>position</i> &lt; <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_at(self as *const crate::QString, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the last character in the string. Same as <code>at(size() - 1)</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QString::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the last character in the string. Same as <code>at(size() - 1)</code>.</p>
    /// <p>This function is provided for STL compatibility.</p>
    /// <p><b>Warning:</b> Calling this function on an empty string constitutes undefined behavior.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#front">front</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#at">at</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#operator-5b-5d">operator[]</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn back(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_back(self as *const crate::QString);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the last character in the string. Same as <code>operator[](size() - 1)</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCharRef QString::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last character in the string. Same as <code>operator[](size() - 1)</code>.</p>
    /// <p>This function is provided for STL compatibility.</p>
    /// <p><b>Warning:</b> Calling this function on an empty string constitutes undefined behavior.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#front">front</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#at">at</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#operator-5b-5d">operator[]</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::CppBox<crate::QCharRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_back1(self as *mut crate::QString);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the maximum number of characters that can be stored in the string without forcing a reallocation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::capacity() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#capacity">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the maximum number of characters that can be stored in the string without forcing a reallocation.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>'s memory usage. In general, you will rarely ever need to call this function. If you want to know how many characters are in the string, call <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#squeeze">squeeze</a>().</p></div>
    pub unsafe fn capacity(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_capacity(self as *const crate::QString)
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first character in the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QChar* QString::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first character in the string.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(&self) -> ::cpp_utils::Ptr<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_cbegin(self as *const crate::QString);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary character after the last character in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QChar* QString::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary character after the last character in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#end">end</a>().</p></div>
    pub unsafe fn cend(&self) -> ::cpp_utils::Ptr<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_cend(self as *const crate::QString);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Removes <i>n</i> characters from the end of the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QString::chop(int n)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#chop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes <i>n</i> characters from the end of the string.</p>
    /// <p>If <i>n</i> is greater than or equal to <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>(), the result is an empty string; if <i>n</i> is negative, it is equivalent to passing zero.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str(<span class="string">"LOGOUT\r\n"</span>);
    ///   str<span class="operator">.</span>chop(<span class="number">2</span>);
    ///   <span class="comment">// str == "LOGOUT"</span>
    ///
    /// </pre>
    /// <p>If you want to remove characters from the <i>beginning</i> of the string, use <a href="http://doc.qt.io/qt-5/qstring.html#remove">remove</a>() instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#truncate">truncate</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#resize">resize</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#remove">remove</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#chop">QStringRef::chop</a>().</p></div>
    pub unsafe fn chop(&mut self, n: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QString_chop(self as *mut crate::QString, n)
    }

    /// <p>Returns a substring that contains the <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>() - <i>len</i> leftmost characters of this string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::chopped(int n) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#chopped">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a substring that contains the <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>() - <i>len</i> leftmost characters of this string.</p>
    /// <p><b>Note: </b>The behavior is undefined if <i>len</i> is negative or greater than <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>().</p><p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#endsWith">endsWith</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#left">left</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#right">right</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#mid">mid</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#chop">chop</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#truncate">truncate</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn chopped(&self, n: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_chopped(self as *const crate::QString, n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Clears the contents of the string and makes it null.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QString::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Clears the contents of the string and makes it null.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#resize">resize</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#isNull">isNull</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QString_clear(self as *mut crate::QString)
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::compare(const QString& s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#compare-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p>
    /// <p>Lexically compares this string with the <i>other</i> string and returns an integer less than, equal to, or greater than zero if this string is less than, equal to, or greater than the other string.</p>
    /// <p>Same as compare(*this, <i>other</i>, <i>cs</i>).</p>
    /// <p>This function was introduced in  Qt 4.2.</p></div>
    pub unsafe fn compare_q_string_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_compare(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::compare(QLatin1String other, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#compare-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p>
    /// <p>Same as compare(*this, <i>other</i>, <i>cs</i>).</p>
    /// <p>This function was introduced in  Qt 4.2.</p></div>
    pub unsafe fn compare_q_latin1_string_case_sensitivity(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_compare1(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(other)
                .as_raw_ptr(),
            cs,
        )
    }

    /// <p>Compares <i>s1</i> with <i>s2</i> and returns an integer less than, equal to, or greater than zero if <i>s1</i> is less than, equal to, or greater than <i>s2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QString::compare(const QString& s1, const QString& s2, Qt::CaseSensitivity cs = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#compare">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares <i>s1</i> with <i>s2</i> and returns an integer less than, equal to, or greater than zero if <i>s1</i> is less than, equal to, or greater than <i>s2</i>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a>, the comparison is case sensitive; otherwise the comparison is case insensitive.</p>
    /// <p>Case sensitive comparison is based exclusively on the numeric Unicode values of the characters and is very fast, but is not what a human would expect. Consider sorting user-visible strings with <a href="http://doc.qt.io/qt-5/qstring.html#localeAwareCompare">localeAwareCompare</a>().</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">int</span> x <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span><span class="operator">::</span>compare(<span class="string">"aUtO"</span><span class="operator">,</span> <span class="string">"AuTo"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>CaseInsensitive);  <span class="comment">// x == 0</span>
    ///   <span class="type">int</span> y <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span><span class="operator">::</span>compare(<span class="string">"auto"</span><span class="operator">,</span> <span class="string">"Car"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>CaseSensitive);     <span class="comment">// y &gt; 0</span>
    ///   <span class="type">int</span> z <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span><span class="operator">::</span>compare(<span class="string">"auto"</span><span class="operator">,</span> <span class="string">"Car"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>CaseInsensitive);   <span class="comment">// z &lt; 0</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#operator-eq-eq">operator==</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#operator-lt">operator&lt;</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#operator-gt">operator&gt;</a>().</p></div>
    pub unsafe fn compare_2_q_string_case_sensitivity(
        s1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        s2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_compare2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s2).as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QString::compare(const QString& s1, QLatin1String s2, Qt::CaseSensitivity cs = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#compare-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p>
    /// <p>Performs a comparison of <i>s1</i> and <i>s2</i>, using the case sensitivity setting <i>cs</i>.</p>
    /// <p>This function was introduced in  Qt 4.2.</p></div>
    pub unsafe fn compare_q_string_q_latin1_string_case_sensitivity(
        s1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        s2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_compare3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s2)
                .as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QString::compare(QLatin1String s1, const QString& s2, Qt::CaseSensitivity cs = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#compare-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p>
    /// <p>Performs a comparison of <i>s1</i> and <i>s2</i>, using the case sensitivity setting <i>cs</i>.</p>
    /// <p>This function was introduced in  Qt 4.2.</p></div>
    pub unsafe fn compare_q_latin1_string_q_string_case_sensitivity(
        s1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        s2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_compare4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s1)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s2).as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::compare(const QStringRef& s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#compare-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p>
    /// <p>Compares the string reference, <i>ref</i>, with the string and returns an integer less than, equal to, or greater than zero if the string is less than, equal to, or greater than <i>ref</i>.</p></div>
    pub unsafe fn compare_q_string_ref_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_compare5(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QString::compare(const QString& s1, const QStringRef& s2, Qt::CaseSensitivity arg3 = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#compare-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p></div>
    pub unsafe fn compare_q_string_q_string_ref_case_sensitivity(
        s1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        s2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        arg3: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_compare6(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s2)
                .as_raw_ptr(),
            arg3,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::compare(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#compare-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p>
    /// <p>Lexically compares this string with the <i>other</i> string and returns an integer less than, equal to, or greater than zero if this string is less than, equal to, or greater than the other string.</p>
    /// <p>Same as compare(*this, <i>other</i>, <i>cs</i>).</p>
    /// <p>This function was introduced in  Qt 4.2.</p></div>
    pub unsafe fn compare_q_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_compare7(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::compare(QLatin1String other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#compare-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p>
    /// <p>Same as compare(*this, <i>other</i>, <i>cs</i>).</p>
    /// <p>This function was introduced in  Qt 4.2.</p></div>
    pub unsafe fn compare_q_latin1_string(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_compare8(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Compares <i>s1</i> with <i>s2</i> and returns an integer less than, equal to, or greater than zero if <i>s1</i> is less than, equal to, or greater than <i>s2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QString::compare(const QString& s1, const QString& s2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#compare">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares <i>s1</i> with <i>s2</i> and returns an integer less than, equal to, or greater than zero if <i>s1</i> is less than, equal to, or greater than <i>s2</i>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a>, the comparison is case sensitive; otherwise the comparison is case insensitive.</p>
    /// <p>Case sensitive comparison is based exclusively on the numeric Unicode values of the characters and is very fast, but is not what a human would expect. Consider sorting user-visible strings with <a href="http://doc.qt.io/qt-5/qstring.html#localeAwareCompare">localeAwareCompare</a>().</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">int</span> x <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span><span class="operator">::</span>compare(<span class="string">"aUtO"</span><span class="operator">,</span> <span class="string">"AuTo"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>CaseInsensitive);  <span class="comment">// x == 0</span>
    ///   <span class="type">int</span> y <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span><span class="operator">::</span>compare(<span class="string">"auto"</span><span class="operator">,</span> <span class="string">"Car"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>CaseSensitive);     <span class="comment">// y &gt; 0</span>
    ///   <span class="type">int</span> z <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span><span class="operator">::</span>compare(<span class="string">"auto"</span><span class="operator">,</span> <span class="string">"Car"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>CaseInsensitive);   <span class="comment">// z &lt; 0</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#operator-eq-eq">operator==</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#operator-lt">operator&lt;</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#operator-gt">operator&gt;</a>().</p></div>
    pub unsafe fn compare_2_q_string(
        s1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        s2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_compare9(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s2).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QString::compare(const QString& s1, QLatin1String s2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#compare-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p>
    /// <p>Performs a comparison of <i>s1</i> and <i>s2</i>, using the case sensitivity setting <i>cs</i>.</p>
    /// <p>This function was introduced in  Qt 4.2.</p></div>
    pub unsafe fn compare_q_string_q_latin1_string(
        s1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        s2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_compare10(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s2)
                .as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QString::compare(QLatin1String s1, const QString& s2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#compare-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p>
    /// <p>Performs a comparison of <i>s1</i> and <i>s2</i>, using the case sensitivity setting <i>cs</i>.</p>
    /// <p>This function was introduced in  Qt 4.2.</p></div>
    pub unsafe fn compare_q_latin1_string_q_string(
        s1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        s2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_compare11(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s1)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s2).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::compare(const QStringRef& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#compare-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p>
    /// <p>Compares the string reference, <i>ref</i>, with the string and returns an integer less than, equal to, or greater than zero if the string is less than, equal to, or greater than <i>ref</i>.</p></div>
    pub unsafe fn compare_q_string_ref(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_compare12(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QString::compare(const QString& s1, const QStringRef& s2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#compare-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p></div>
    pub unsafe fn compare_q_string_q_string_ref(
        s1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        s2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_compare13(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s2)
                .as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::compare(QStringView s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#compare-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p>
    /// <p>Performs a comparison of this with <i>s</i>, using the case sensitivity setting <i>cs</i>.</p>
    /// <p>This function was introduced in  Qt 5.12.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn compare_q_string_view_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_compare28(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::compare(QStringView s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#compare-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>().</p>
    /// <p>Performs a comparison of this with <i>s</i>, using the case sensitivity setting <i>cs</i>.</p>
    /// <p>This function was introduced in  Qt 5.12.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn compare_q_string_view(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_compare29(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first character in the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QChar* QString::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first character in the string.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(&self) -> ::cpp_utils::Ptr<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_constBegin(self as *const crate::QString);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns a pointer to the data stored in the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. The pointer can be used to access the characters that compose the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QChar* QString::constData() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#constData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the data stored in the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. The pointer can be used to access the characters that compose the string.</p>
    /// <p>Note that the pointer remains valid only as long as the string is not modified.</p>
    /// <p><b>Note: </b>The returned string may not be '\0'-terminated. Use <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>() to determine the length of the array.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#data">data</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#operator-5b-5d">operator[]</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#fromRawData">fromRawData</a>().</p></div>
    pub unsafe fn const_data(&self) -> ::cpp_utils::Ptr<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_constData(self as *const crate::QString);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary character after the last character in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QChar* QString::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary character after the last character in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#end">end</a>().</p></div>
    pub unsafe fn const_end(&self) -> ::cpp_utils::Ptr<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_constEnd(self as *const crate::QString);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::contains(QChar c, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#contains-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>().</p>
    /// <p>Returns <code>true</code> if this string contains an occurrence of the character <i>ch</i>; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn contains_q_char_case_sensitivity(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_contains(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
            cs,
        )
    }

    /// <p>Returns <code>true</code> if this string contains an occurrence of the string <i>str</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::contains(const QString& s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this string contains an occurrence of the string <i>str</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"Peter Pan"</span>;
    ///   str<span class="operator">.</span>contains(<span class="string">"peter"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>CaseInsensitive);    <span class="comment">// returns true</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p></div>
    pub unsafe fn contains_q_string_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_contains1(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::contains(QLatin1String s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#contains-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>().</p>
    /// <p>Returns <code>true</code> if this string contains an occurrence of the latin-1 string <i>str</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 5.3.</p></div>
    pub unsafe fn contains_q_latin1_string_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_contains2(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
            cs,
        )
    }

    /// <p>Returns <code>true</code> if this string contains an occurrence of the string reference <i>str</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::contains(const QStringRef& s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#contains-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this string contains an occurrence of the string reference <i>str</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p></div>
    pub unsafe fn contains_q_string_ref_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_contains3(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::contains(const QRegExp& rx) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#contains-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>().</p>
    /// <p>Returns <code>true</code> if the regular expression <i>rx</i> matches somewhere in this string; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn contains_q_reg_exp(
        &self,
        rx: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_contains4(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(rx).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::contains(QRegExp& rx) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#contains-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>().</p>
    /// <p>Returns <code>true</code> if the regular expression <i>rx</i> matches somewhere in this string; otherwise returns <code>false</code>.</p>
    /// <p>If there is a match, the <i>rx</i> regular expression will contain the matched captures (see <a href="http://doc.qt.io/qt-5/qregexp.html#matchedLength">QRegExp::matchedLength</a>, <a href="http://doc.qt.io/qt-5/qregexp.html#cap">QRegExp::cap</a>).</p>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn contains_q_reg_exp2(
        &self,
        rx: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QRegExp>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_contains5(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QRegExp>>::cast_into(rx)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::contains(const QRegularExpression& re) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#contains-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>().</p>
    /// <p>Returns <code>true</code> if the regular expression <i>re</i> matches somewhere in this string; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn contains_q_regular_expression(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_contains6(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::contains(const QRegularExpression& re, QRegularExpressionMatch* match) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#contains-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>().</p>
    /// <p>Returns <code>true</code> if the regular expression <i>re</i> matches somewhere in this string; otherwise returns <code>false</code>.</p>
    /// <p>If the match is successful and <i>match</i> is not a null pointer, it also writes the results of the match into the <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> object pointed to by <i>match</i>.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#match">QRegularExpression::match</a>().</p></div>
    pub unsafe fn contains_q_regular_expression_q_regular_expression_match(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
        match_: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QRegularExpressionMatch>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_contains7(self as *const crate::QString, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QRegularExpressionMatch>>::cast_into(match_).as_mut_raw_ptr())
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::contains(QChar c) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#contains-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>().</p>
    /// <p>Returns <code>true</code> if this string contains an occurrence of the character <i>ch</i>; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn contains_q_char(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_contains8(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if this string contains an occurrence of the string <i>str</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::contains(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this string contains an occurrence of the string <i>str</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"Peter Pan"</span>;
    ///   str<span class="operator">.</span>contains(<span class="string">"peter"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>CaseInsensitive);    <span class="comment">// returns true</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p></div>
    pub unsafe fn contains_q_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_contains9(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::contains(QLatin1String s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#contains-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>().</p>
    /// <p>Returns <code>true</code> if this string contains an occurrence of the latin-1 string <i>str</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 5.3.</p></div>
    pub unsafe fn contains_q_latin1_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_contains10(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if this string contains an occurrence of the string reference <i>str</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::contains(const QStringRef& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#contains-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this string contains an occurrence of the string reference <i>str</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p></div>
    pub unsafe fn contains_q_string_ref(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_contains11(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-eq">operator=</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::operator=(QChar c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-eq">operator=</a>().</p>
    /// <p>Sets the string to contain the single character <i>ch</i>.</p></div>
    pub unsafe fn copy_from_q_char(
        &mut self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_operator_(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Assigns <i>other</i> to this string and returns a reference to this string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::operator=(const QString& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this string and returns a reference to this string.</p></div>
    pub unsafe fn copy_from_q_string(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_operator_1(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(arg1).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-eq">operator=</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::operator=(QLatin1String latin1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-eq-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-eq">operator=</a>().</p>
    /// <p>Assigns the Latin-1 string <i>str</i> to this string.</p></div>
    pub unsafe fn copy_from_q_latin1_string(
        &mut self,
        latin1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_operator_2(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(latin1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-eq">operator=</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::operator=(const char* ch)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-eq-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-eq">operator=</a>().</p>
    /// <p>Assigns <i>str</i> to this string. The const char pointer is converted to Unicode using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>() function.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> or <code>QT_RESTRICTED_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p></div>
    pub unsafe fn copy_from_char(
        &mut self,
        ch: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_operator_6(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(ch)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-eq">operator=</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::operator=(const QByteArray& a)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-eq-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-eq">operator=</a>().</p>
    /// <p>Assigns <i>ba</i> to this string. The byte array is converted to Unicode using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>() function. This function stops conversion at the first NUL character found, or the end of the <i>ba</i> byte array.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p></div>
    pub unsafe fn copy_from_q_byte_array(
        &mut self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_operator_7(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(a).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-eq">operator=</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::operator=(char c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-eq-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-eq">operator=</a>().</p>
    /// <p>Assigns character <i>ch</i> to this string. Note that the character is converted to Unicode using the <a href="http://doc.qt.io/qt-5/qstring.html#fromLatin1">fromLatin1</a>() function, unlike other 8-bit functions that operate on UTF-8 data.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p></div>
    pub unsafe fn copy_from_char2(
        &mut self,
        c: ::std::os::raw::c_char,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_operator_8(self as *mut crate::QString, c);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads count().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#count-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads count().</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>().</p></div>
    pub unsafe fn count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_count(self as *const crate::QString)
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::count(QChar c, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p>
    /// <p>Returns the number of occurrences of character <i>ch</i> in the string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_q_char_case_sensitivity(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_count1(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
            cs,
        )
    }

    /// <p>Returns the number of (potentially overlapping) occurrences of the string <i>str</i> in this string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::count(const QString& s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of (potentially overlapping) occurrences of the string <i>str</i> in this string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_q_string_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_count2(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::count(const QStringRef& s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#count-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p>
    /// <p>Returns the number of (potentially overlapping) occurrences of the string reference <i>str</i> in this string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_q_string_ref_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_count3(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::count(const QRegExp& arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#count-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p>
    /// <p>Returns the number of times the regular expression <i>rx</i> matches in the string.</p>
    /// <p>This function counts overlapping matches, so in the example below, there are four instances of "ana" or "ama":</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"banana and panama"</span>;
    ///   str<span class="operator">.</span>count(<span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a></span>(<span class="string">"a[nm]a"</span>));    <span class="comment">// returns 4</span>
    ///
    /// </pre></div>
    pub unsafe fn count_q_reg_exp(
        &self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_count4(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(arg1).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::count(const QRegularExpression& re) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#count-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p>
    /// <p>Returns the number of times the regular expression <i>re</i> matches in the string.</p>
    /// <p>This function counts overlapping matches, so in the example below, there are four instances of "ana" or "ama":</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"banana and panama"</span>;
    ///   str<span class="operator">.</span>count(<span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a></span>(<span class="string">"a[nm]a"</span>));    <span class="comment">// returns 4</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn count_q_regular_expression(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_count5(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::count(QChar c) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p>
    /// <p>Returns the number of occurrences of character <i>ch</i> in the string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_q_char(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_count6(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
        )
    }

    /// <p>Returns the number of (potentially overlapping) occurrences of the string <i>str</i> in this string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::count(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of (potentially overlapping) occurrences of the string <i>str</i> in this string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_q_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_count7(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::count(const QStringRef& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#count-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p>
    /// <p>Returns the number of (potentially overlapping) occurrences of the string reference <i>str</i> in this string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_q_string_ref(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_count8(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Returns a pointer to the data stored in the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. The pointer can be used to access and modify the characters that compose the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar* QString::data()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the data stored in the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. The pointer can be used to access and modify the characters that compose the string.</p>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qstring.html#constData">constData</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#unicode">unicode</a>(), the returned data is always '\0'-terminated.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"Hello world"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qchar.html">QChar</a></span> <span class="operator">*</span>data <span class="operator">=</span> str<span class="operator">.</span>data();
    ///   <span class="keyword">while</span> (<span class="operator">!</span>data<span class="operator">-</span><span class="operator">&gt;</span>isNull()) {
    /// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> data<span class="operator">-</span><span class="operator">&gt;</span>unicode();
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>data;
    ///   }
    ///
    /// </pre>
    /// <p>Note that the pointer remains valid only as long as the string is not modified by other means. For read-only access, <a href="http://doc.qt.io/qt-5/qstring.html#constData">constData</a>() is faster because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#constData">constData</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn data_mut(&mut self) -> ::cpp_utils::MutPtr<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_data(self as *mut crate::QString);
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QChar* QString::data() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#data-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><b>Note: </b>The returned string may not be '\0'-terminated. Use <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>() to determine the length of the array.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#fromRawData">fromRawData</a>().</p></div>
    pub unsafe fn data(&self) -> ::cpp_utils::Ptr<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_data1(self as *const crate::QString);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```void QString::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QString_detach(self as *mut crate::QString)
    }

    /// <p>Returns <code>true</code> if the string ends with <i>s</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::endsWith(const QString& s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the string ends with <i>s</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"Bananas"</span>;
    ///   str<span class="operator">.</span>endsWith(<span class="string">"anas"</span>);         <span class="comment">// returns true</span>
    ///   str<span class="operator">.</span>endsWith(<span class="string">"pple"</span>);         <span class="comment">// returns false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#startsWith">startsWith</a>().</p></div>
    pub unsafe fn ends_with_q_string_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_endsWith(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#endsWith">endsWith</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::endsWith(const QStringRef& s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#endsWith-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#endsWith">endsWith</a>().</p>
    /// <p>Returns <code>true</code> if the string ends with the string reference <i>s</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#startsWith">startsWith</a>().</p></div>
    pub unsafe fn ends_with_q_string_ref_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_endsWith1(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#endsWith">endsWith</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::endsWith(QLatin1String s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#endsWith-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#endsWith">endsWith</a>().</p></div>
    pub unsafe fn ends_with_q_latin1_string_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_endsWith2(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
            cs,
        )
    }

    /// <p>Returns <code>true</code> if the string ends with <i>c</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::endsWith(QChar c, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#endsWith-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the string ends with <i>c</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#endsWith">endsWith</a>().</p></div>
    pub unsafe fn ends_with_q_char_case_sensitivity(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_endsWith3(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
            cs,
        )
    }

    /// <p>Returns <code>true</code> if the string ends with <i>s</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::endsWith(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the string ends with <i>s</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"Bananas"</span>;
    ///   str<span class="operator">.</span>endsWith(<span class="string">"anas"</span>);         <span class="comment">// returns true</span>
    ///   str<span class="operator">.</span>endsWith(<span class="string">"pple"</span>);         <span class="comment">// returns false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#startsWith">startsWith</a>().</p></div>
    pub unsafe fn ends_with_q_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_endsWith4(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#endsWith">endsWith</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::endsWith(const QStringRef& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#endsWith-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#endsWith">endsWith</a>().</p>
    /// <p>Returns <code>true</code> if the string ends with the string reference <i>s</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#startsWith">startsWith</a>().</p></div>
    pub unsafe fn ends_with_q_string_ref(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_endsWith5(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#endsWith">endsWith</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::endsWith(QLatin1String s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#endsWith-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#endsWith">endsWith</a>().</p></div>
    pub unsafe fn ends_with_q_latin1_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_endsWith6(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if the string ends with <i>c</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::endsWith(QChar c) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#endsWith-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the string ends with <i>c</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#endsWith">endsWith</a>().</p></div>
    pub unsafe fn ends_with_q_char(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_endsWith7(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#endsWith">endsWith</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::endsWith(QStringView s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#endsWith-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#endsWith">endsWith</a>().</p>
    /// <p>Returns <code>true</code> if the string ends with the string view <i>str</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#startsWith">startsWith</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn ends_with_q_string_view_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_endsWith16(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#endsWith">endsWith</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::endsWith(QStringView s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#endsWith-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#endsWith">endsWith</a>().</p>
    /// <p>Returns <code>true</code> if the string ends with the string view <i>str</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#startsWith">startsWith</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn ends_with_q_string_view(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_endsWith17(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets every character in the string to character <i>ch</i>. If <i>size</i> is different from -1 (default), the string is resized to <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::fill(QChar c, int size = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets every character in the string to character <i>ch</i>. If <i>size</i> is different from -1 (default), the string is resized to <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"Berlin"</span>;
    ///   str<span class="operator">.</span>fill(<span class="char">'z'</span>);
    ///   <span class="comment">// str == "zzzzzz"</span>
    ///
    ///   str<span class="operator">.</span>fill(<span class="char">'A'</span><span class="operator">,</span> <span class="number">2</span>);
    ///   <span class="comment">// str == "AA"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_2a(
        &mut self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_fill(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
            size,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Sets every character in the string to character <i>ch</i>. If <i>size</i> is different from -1 (default), the string is resized to <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::fill(QChar c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets every character in the string to character <i>ch</i>. If <i>size</i> is different from -1 (default), the string is resized to <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"Berlin"</span>;
    ///   str<span class="operator">.</span>fill(<span class="char">'z'</span>);
    ///   <span class="comment">// str == "zzzzzz"</span>
    ///
    ///   str<span class="operator">.</span>fill(<span class="char">'A'</span><span class="operator">,</span> <span class="number">2</span>);
    ///   <span class="comment">// str == "AA"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_1a(
        &mut self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_fill1(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> initialized with the first <i>size</i> characters of the Latin-1 string <i>str</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::fromLatin1(const char* str, int size = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#fromLatin1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> initialized with the first <i>size</i> characters of the Latin-1 string <i>str</i>.</p>
    /// <p>If <i>size</i> is -1 (default), it is taken to be strlen(<i>str</i>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toLatin1">toLatin1</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#fromLocal8Bit">fromLocal8Bit</a>().</p></div>
    pub unsafe fn from_latin1_char_int(
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_fromLatin1(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(str)
                .as_raw_ptr(),
            size,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::fromLatin1(const QByteArray& str)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#fromLatin1-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> initialized with the Latin-1 string <i>str</i>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn from_latin1_q_byte_array(
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_fromLatin11(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(str)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> initialized with the first <i>size</i> characters of the Latin-1 string <i>str</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::fromLatin1(const char* str)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#fromLatin1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> initialized with the first <i>size</i> characters of the Latin-1 string <i>str</i>.</p>
    /// <p>If <i>size</i> is -1 (default), it is taken to be strlen(<i>str</i>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toLatin1">toLatin1</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#fromLocal8Bit">fromLocal8Bit</a>().</p></div>
    pub unsafe fn from_latin1_char(
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_fromLatin12(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(str)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> initialized with the first <i>size</i> characters of the 8-bit string <i>str</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::fromLocal8Bit(const char* str, int size = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#fromLocal8Bit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> initialized with the first <i>size</i> characters of the 8-bit string <i>str</i>.</p>
    /// <p>If <i>size</i> is -1 (default), it is taken to be strlen(<i>str</i>).</p>
    /// <p><a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForLocale">QTextCodec::codecForLocale</a>() is used to perform the conversion.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toLocal8Bit">toLocal8Bit</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#fromLatin1">fromLatin1</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>().</p></div>
    pub unsafe fn from_local8_bit_char_int(
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_fromLocal8Bit(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(str)
                .as_raw_ptr(),
            size,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::fromLocal8Bit(const QByteArray& str)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#fromLocal8Bit-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> initialized with the 8-bit string <i>str</i>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn from_local8_bit_q_byte_array(
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_fromLocal8Bit1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(str)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> initialized with the first <i>size</i> characters of the 8-bit string <i>str</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::fromLocal8Bit(const char* str)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#fromLocal8Bit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> initialized with the first <i>size</i> characters of the 8-bit string <i>str</i>.</p>
    /// <p>If <i>size</i> is -1 (default), it is taken to be strlen(<i>str</i>).</p>
    /// <p><a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForLocale">QTextCodec::codecForLocale</a>() is used to perform the conversion.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toLocal8Bit">toLocal8Bit</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#fromLatin1">fromLatin1</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>().</p></div>
    pub unsafe fn from_local8_bit_char(
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_fromLocal8Bit2(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(str)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> that uses the first <i>size</i> Unicode characters in the array <i>unicode</i>. The data in <i>unicode</i> is <i>not</i> copied. The caller must be able to guarantee that <i>unicode</i> will not be deleted or modified as long as the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> (or an unmodified copy of it) exists.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::fromRawData(const QChar* arg1, int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#fromRawData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> that uses the first <i>size</i> Unicode characters in the array <i>unicode</i>. The data in <i>unicode</i> is <i>not</i> copied. The caller must be able to guarantee that <i>unicode</i> will not be deleted or modified as long as the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> (or an unmodified copy of it) exists.</p>
    /// <p>Any attempts to modify the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> or copies of it will cause it to create a deep copy of the data, ensuring that the raw data isn't modified.</p>
    /// <p>Here's an example of how we can use a <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> on raw data in memory without requiring to copy the data into a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a></span> pattern(<span class="string">"\u00A4"</span>);
    ///   <span class="keyword">static</span> <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qchar.html">QChar</a></span> unicode<span class="operator">[</span><span class="operator">]</span> <span class="operator">=</span> {
    /// &#32;   &#32;     <span class="number">0x005A</span><span class="operator">,</span> <span class="number">0x007F</span><span class="operator">,</span> <span class="number">0x00A4</span><span class="operator">,</span> <span class="number">0x0060</span><span class="operator">,</span>
    /// &#32;   &#32;     <span class="number">0x1009</span><span class="operator">,</span> <span class="number">0x0020</span><span class="operator">,</span> <span class="number">0x0020</span>};
    ///   <span class="type">int</span> size <span class="operator">=</span> <span class="keyword">sizeof</span>(unicode) <span class="operator">/</span> <span class="keyword">sizeof</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qchar.html">QChar</a></span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span><span class="operator">::</span>fromRawData(unicode<span class="operator">,</span> size);
    ///   <span class="keyword">if</span> (str<span class="operator">.</span>contains(pattern) {
    /// &#32;     <span class="comment">// ...</span>
    ///   }
    ///
    /// </pre>
    /// <p><b>Warning:</b> A string created with fromRawData() is <i>not</i> '\0'-terminated, unless the raw data contains a '\0' character at position <i>size</i>. This means <a href="http://doc.qt.io/qt-5/qstring.html#unicode">unicode</a>() will <i>not</i> return a '\0'-terminated string (although <a href="http://doc.qt.io/qt-5/qstring.html#utf16">utf16</a>() does, at the cost of copying the raw data).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#fromUtf16">fromUtf16</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#setRawData">setRawData</a>().</p></div>
    pub unsafe fn from_raw_data(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QChar>>,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_fromRawData(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QChar>>::cast_into(arg1).as_raw_ptr(),
            size,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> initialized with the first <i>size</i> characters of the Unicode string <i>unicode</i> (ISO-10646-UCS-4 encoded).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::fromUcs4(const unsigned int* arg1, int size = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#fromUcs4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> initialized with the first <i>size</i> characters of the Unicode string <i>unicode</i> (ISO-10646-UCS-4 encoded).</p>
    /// <p>If <i>size</i> is -1 (default), <i>unicode</i> must be terminated with a 0.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toUcs4">toUcs4</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf16">fromUtf16</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#utf16">utf16</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#setUtf16">setUtf16</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#fromWCharArray">fromWCharArray</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#fromStdU32String">fromStdU32String</a>().</p></div>
    pub unsafe fn from_ucs4_2a(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_uint>>,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_fromUcs4(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_uint>>::cast_into(arg1)
                .as_raw_ptr(),
            size,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> initialized with the first <i>size</i> characters of the Unicode string <i>unicode</i> (ISO-10646-UCS-4 encoded).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::fromUcs4(const unsigned int* arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#fromUcs4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> initialized with the first <i>size</i> characters of the Unicode string <i>unicode</i> (ISO-10646-UCS-4 encoded).</p>
    /// <p>If <i>size</i> is -1 (default), <i>unicode</i> must be terminated with a 0.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toUcs4">toUcs4</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf16">fromUtf16</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#utf16">utf16</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#setUtf16">setUtf16</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#fromWCharArray">fromWCharArray</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#fromStdU32String">fromStdU32String</a>().</p></div>
    pub unsafe fn from_ucs4_1a(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_uint>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_fromUcs42(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_uint>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> initialized with the first <i>size</i> characters of the Unicode string <i>unicode</i> (ISO-10646-UTF-16 encoded).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::fromUtf16(const unsigned short* arg1, int size = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf16">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> initialized with the first <i>size</i> characters of the Unicode string <i>unicode</i> (ISO-10646-UTF-16 encoded).</p>
    /// <p>If <i>size</i> is -1 (default), <i>unicode</i> must be terminated with a 0.</p>
    /// <p>This function checks for a Byte Order Mark (BOM). If it is missing, host byte order is assumed.</p>
    /// <p>This function is slow compared to the other Unicode conversions. Use <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>(const <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> *, int) or <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>(const <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> *) if possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qstring.html">QString</a> makes a deep copy of the Unicode data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#utf16">utf16</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#setUtf16">setUtf16</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#fromStdU16String">fromStdU16String</a>().</p></div>
    pub unsafe fn from_utf16_2a(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_ushort>>,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_fromUtf16(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_ushort>>::cast_into(arg1)
                .as_raw_ptr(),
            size,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> initialized with the first <i>size</i> characters of the Unicode string <i>unicode</i> (ISO-10646-UTF-16 encoded).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::fromUtf16(const unsigned short* arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf16">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> initialized with the first <i>size</i> characters of the Unicode string <i>unicode</i> (ISO-10646-UTF-16 encoded).</p>
    /// <p>If <i>size</i> is -1 (default), <i>unicode</i> must be terminated with a 0.</p>
    /// <p>This function checks for a Byte Order Mark (BOM). If it is missing, host byte order is assumed.</p>
    /// <p>This function is slow compared to the other Unicode conversions. Use <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>(const <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> *, int) or <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>(const <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> *) if possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qstring.html">QString</a> makes a deep copy of the Unicode data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#utf16">utf16</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#setUtf16">setUtf16</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#fromStdU16String">fromStdU16String</a>().</p></div>
    pub unsafe fn from_utf16_1a(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_ushort>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_fromUtf162(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_ushort>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> initialized with the first <i>size</i> bytes of the UTF-8 string <i>str</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::fromUtf8(const char* str, int size = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> initialized with the first <i>size</i> bytes of the UTF-8 string <i>str</i>.</p>
    /// <p>If <i>size</i> is -1 (default), it is taken to be strlen(<i>str</i>).</p>
    /// <p>UTF-8 is a Unicode codec and can represent all characters in a Unicode string like <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. However, invalid sequences are possible with UTF-8 and, if any such are found, they will be replaced with one or more "replacement characters", or suppressed. These include non-Unicode sequences, non-characters, overlong sequences or surrogate codepoints encoded into UTF-8.</p>
    /// <p>This function can be used to process incoming data incrementally as long as all UTF-8 characters are terminated within the incoming data. Any unterminated characters at the end of the string will be replaced or suppressed. In order to do stateful decoding, please use <a href="http://doc.qt.io/qt-5/qtextdecoder.html">QTextDecoder</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">toUtf8</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#fromLatin1">fromLatin1</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#fromLocal8Bit">fromLocal8Bit</a>().</p></div>
    pub unsafe fn from_utf8_char_int(
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_fromUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(str)
                .as_raw_ptr(),
            size,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::fromUtf8(const QByteArray& str)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> initialized with the UTF-8 string <i>str</i>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn from_utf8_q_byte_array(
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_fromUtf81(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(str)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> initialized with the first <i>size</i> bytes of the UTF-8 string <i>str</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::fromUtf8(const char* str)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> initialized with the first <i>size</i> bytes of the UTF-8 string <i>str</i>.</p>
    /// <p>If <i>size</i> is -1 (default), it is taken to be strlen(<i>str</i>).</p>
    /// <p>UTF-8 is a Unicode codec and can represent all characters in a Unicode string like <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. However, invalid sequences are possible with UTF-8 and, if any such are found, they will be replaced with one or more "replacement characters", or suppressed. These include non-Unicode sequences, non-characters, overlong sequences or surrogate codepoints encoded into UTF-8.</p>
    /// <p>This function can be used to process incoming data incrementally as long as all UTF-8 characters are terminated within the incoming data. Any unterminated characters at the end of the string will be replaced or suppressed. In order to do stateful decoding, please use <a href="http://doc.qt.io/qt-5/qtextdecoder.html">QTextDecoder</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">toUtf8</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#fromLatin1">fromLatin1</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#fromLocal8Bit">fromLocal8Bit</a>().</p></div>
    pub unsafe fn from_utf8_char(
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_fromUtf82(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(str)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the first character in the string. Same as <code>at(0)</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QString::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the first character in the string. Same as <code>at(0)</code>.</p>
    /// <p>This function is provided for STL compatibility.</p>
    /// <p><b>Warning:</b> Calling this function on an empty string constitutes undefined behavior.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#back">back</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#at">at</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#operator-5b-5d">operator[]</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn front(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_front(self as *const crate::QString);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the first character in the string. Same as <code>operator[](0)</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCharRef QString::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first character in the string. Same as <code>operator[](0)</code>.</p>
    /// <p>This function is provided for STL compatibility.</p>
    /// <p><b>Warning:</b> Calling this function on an empty string constitutes undefined behavior.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#back">back</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#at">at</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#operator-5b-5d">operator[]</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::CppBox<crate::QCharRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_front1(self as *mut crate::QString);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the character at the specified <i>position</i> in the string as a modifiable reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QString::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the character at the specified <i>position</i> in the string as a modifiable reference.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str;
    ///
    ///   <span class="keyword">if</span> (str<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qchar.html">QChar</a></span>(<span class="char">'?'</span>))
    /// &#32;     str<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qchar.html">QChar</a></span>(<span class="char">'_'</span>);
    ///
    /// </pre>
    /// <p>The return value is of type QCharRef, a helper class for <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. When you get an object of type QCharRef, you can use it as if it were a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> &amp;. If you assign to it, the assignment will apply to the character in the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> from which you got the reference.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#at">at</a>().</p></div>
    pub unsafe fn index_int(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_operator__(self as *const crate::QString, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the character at the specified <i>position</i> in the string as a modifiable reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCharRef QString::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the character at the specified <i>position</i> in the string as a modifiable reference.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str;
    ///
    ///   <span class="keyword">if</span> (str<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qchar.html">QChar</a></span>(<span class="char">'?'</span>))
    /// &#32;     str<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qchar.html">QChar</a></span>(<span class="char">'_'</span>);
    ///
    /// </pre>
    /// <p>The return value is of type QCharRef, a helper class for <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. When you get an object of type QCharRef, you can use it as if it were a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> &amp;. If you assign to it, the assignment will apply to the character in the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> from which you got the reference.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#at">at</a>().</p></div>
    pub unsafe fn index_int_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QCharRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_operator__1(self as *mut crate::QString, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Equivalent to <code>at(position)</code>. </p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QString::operator[](unsigned int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-5b-5d-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Equivalent to <code>at(position)</code>. </p><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn index_uint(
        &self,
        i: ::std::os::raw::c_uint,
    ) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_operator__2(self as *const crate::QString, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Equivalent to <code>at(position)</code>. </p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCharRef QString::operator[](unsigned int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-5b-5d-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Equivalent to <code>at(position)</code>. </p><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn index_uint_mut(
        &mut self,
        i: ::std::os::raw::c_uint,
    ) -> ::cpp_utils::CppBox<crate::QCharRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_operator__3(self as *mut crate::QString, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::indexOf(QChar c, int from = …, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#indexOf-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p>
    /// <p>Returns the index position of the first occurrence of the character <i>ch</i> in the string, searching forward from index position <i>from</i>. Returns -1 if <i>ch</i> could not be found.</p></div>
    pub unsafe fn index_of_q_char_int_case_sensitivity(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        from: ::std::os::raw::c_int,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_indexOf(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
            from,
            cs,
        )
    }

    /// <p>Returns the index position of the first occurrence of the string <i>str</i> in this string, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::indexOf(const QString& s, int from = …, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of the string <i>str</i> in this string, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> x <span class="operator">=</span> <span class="string">"sticky question"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> y <span class="operator">=</span> <span class="string">"sti"</span>;
    ///   x<span class="operator">.</span>indexOf(y);               <span class="comment">// returns 0</span>
    ///   x<span class="operator">.</span>indexOf(y<span class="operator">,</span> <span class="number">1</span>);            <span class="comment">// returns 10</span>
    ///   x<span class="operator">.</span>indexOf(y<span class="operator">,</span> <span class="number">10</span>);           <span class="comment">// returns 10</span>
    ///   x<span class="operator">.</span>indexOf(y<span class="operator">,</span> <span class="number">11</span>);           <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>If <i>from</i> is -1, the search starts at the last character; if it is -2, at the next to last character and so on.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p></div>
    pub unsafe fn index_of_q_string_int_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        from: ::std::os::raw::c_int,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_indexOf1(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            from,
            cs,
        )
    }

    /// <p>Returns the index position of the first occurrence of the string <i>str</i> in this string, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::indexOf(QLatin1String s, int from = …, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#indexOf-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of the string <i>str</i> in this string, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> x <span class="operator">=</span> <span class="string">"sticky question"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> y <span class="operator">=</span> <span class="string">"sti"</span>;
    ///   x<span class="operator">.</span>indexOf(y);               <span class="comment">// returns 0</span>
    ///   x<span class="operator">.</span>indexOf(y<span class="operator">,</span> <span class="number">1</span>);            <span class="comment">// returns 10</span>
    ///   x<span class="operator">.</span>indexOf(y<span class="operator">,</span> <span class="number">10</span>);           <span class="comment">// returns 10</span>
    ///   x<span class="operator">.</span>indexOf(y<span class="operator">,</span> <span class="number">11</span>);           <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>If <i>from</i> is -1, the search starts at the last character; if it is -2, at the next to last character and so on.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p></div>
    pub unsafe fn index_of_q_latin1_string_int_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        from: ::std::os::raw::c_int,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_indexOf2(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
            from,
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::indexOf(const QStringRef& s, int from = …, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#indexOf-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p>
    /// <p>Returns the index position of the first occurrence of the string reference <i>str</i> in this string, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn index_of_q_string_ref_int_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        from: ::std::os::raw::c_int,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_indexOf3(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
            from,
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::indexOf(const QRegExp& arg1, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#indexOf-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p>
    /// <p>Returns the index position of the first match of the regular expression <i>rx</i> in the string, searching forward from index position <i>from</i>. Returns -1 if <i>rx</i> didn't match anywhere.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"the minimum"</span>;
    ///   str<span class="operator">.</span>indexOf(<span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a></span>(<span class="string">"m[aeiou]"</span>)<span class="operator">,</span> <span class="number">0</span>);       <span class="comment">// returns 4</span>
    ///
    /// </pre></div>
    pub unsafe fn index_of_q_reg_exp_int(
        &self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_indexOf4(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(arg1).as_raw_ptr(),
            from,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::indexOf(QRegExp& arg1, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#indexOf-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p>
    /// <p>Returns the index position of the first match of the regular expression <i>rx</i> in the string, searching forward from index position <i>from</i>. Returns -1 if <i>rx</i> didn't match anywhere.</p>
    /// <p>If there is a match, the <i>rx</i> regular expression will contain the matched captures (see <a href="http://doc.qt.io/qt-5/qregexp.html#matchedLength">QRegExp::matchedLength</a>, <a href="http://doc.qt.io/qt-5/qregexp.html#cap">QRegExp::cap</a>).</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"the minimum"</span>;
    ///   str<span class="operator">.</span>indexOf(<span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a></span>(<span class="string">"m[aeiou]"</span>)<span class="operator">,</span> <span class="number">0</span>);       <span class="comment">// returns 4</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn index_of_q_reg_exp_int2(
        &self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QRegExp>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_indexOf5(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QRegExp>>::cast_into(arg1)
                .as_mut_raw_ptr(),
            from,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::indexOf(const QRegularExpression& re, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#indexOf-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p>
    /// <p>Returns the index position of the first match of the regular expression <i>re</i> in the string, searching forward from index position <i>from</i>. Returns -1 if <i>re</i> didn't match anywhere.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"the minimum"</span>;
    ///   str<span class="operator">.</span>indexOf(<span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a></span>(<span class="string">"m[aeiou]"</span>)<span class="operator">,</span> <span class="number">0</span>);       <span class="comment">// returns 4</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn index_of_q_regular_expression_int(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_indexOf6(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::indexOf(const QRegularExpression& re, int from, QRegularExpressionMatch* rmatch) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#indexOf-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the index position of the first match of the regular expression <i>re</i> in the string, searching forward from index position <i>from</i>. Returns -1 if <i>re</i> didn't match anywhere.</p>
    /// <p>If the match is successful and <i>rmatch</i> is not a null pointer, it also writes the results of the match into the <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> object pointed to by <i>rmatch</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"the minimum"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a></span> match;
    ///   str<span class="operator">.</span>indexOf(<span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a></span>(<span class="string">"m[aeiou]"</span>)<span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="operator">&amp;</span>match);       <span class="comment">// returns 4</span>
    ///   <span class="comment">// match.captured() == mi</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.5.</p></div>
    pub unsafe fn index_of_q_regular_expression_int_q_regular_expression_match(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
        from: ::std::os::raw::c_int,
        rmatch: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QRegularExpressionMatch>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_indexOf7(self as *const crate::QString, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re).as_raw_ptr(), from, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QRegularExpressionMatch>>::cast_into(rmatch).as_mut_raw_ptr())
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::indexOf(QChar c, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#indexOf-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p>
    /// <p>Returns the index position of the first occurrence of the character <i>ch</i> in the string, searching forward from index position <i>from</i>. Returns -1 if <i>ch</i> could not be found.</p></div>
    pub unsafe fn index_of_q_char_int(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_indexOf8(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
            from,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::indexOf(QChar c) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#indexOf-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p>
    /// <p>Returns the index position of the first occurrence of the character <i>ch</i> in the string, searching forward from index position <i>from</i>. Returns -1 if <i>ch</i> could not be found.</p></div>
    pub unsafe fn index_of_q_char(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_indexOf9(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
        )
    }

    /// <p>Returns the index position of the first occurrence of the string <i>str</i> in this string, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::indexOf(const QString& s, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of the string <i>str</i> in this string, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> x <span class="operator">=</span> <span class="string">"sticky question"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> y <span class="operator">=</span> <span class="string">"sti"</span>;
    ///   x<span class="operator">.</span>indexOf(y);               <span class="comment">// returns 0</span>
    ///   x<span class="operator">.</span>indexOf(y<span class="operator">,</span> <span class="number">1</span>);            <span class="comment">// returns 10</span>
    ///   x<span class="operator">.</span>indexOf(y<span class="operator">,</span> <span class="number">10</span>);           <span class="comment">// returns 10</span>
    ///   x<span class="operator">.</span>indexOf(y<span class="operator">,</span> <span class="number">11</span>);           <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>If <i>from</i> is -1, the search starts at the last character; if it is -2, at the next to last character and so on.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p></div>
    pub unsafe fn index_of_q_string_int(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_indexOf10(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first occurrence of the string <i>str</i> in this string, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::indexOf(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of the string <i>str</i> in this string, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> x <span class="operator">=</span> <span class="string">"sticky question"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> y <span class="operator">=</span> <span class="string">"sti"</span>;
    ///   x<span class="operator">.</span>indexOf(y);               <span class="comment">// returns 0</span>
    ///   x<span class="operator">.</span>indexOf(y<span class="operator">,</span> <span class="number">1</span>);            <span class="comment">// returns 10</span>
    ///   x<span class="operator">.</span>indexOf(y<span class="operator">,</span> <span class="number">10</span>);           <span class="comment">// returns 10</span>
    ///   x<span class="operator">.</span>indexOf(y<span class="operator">,</span> <span class="number">11</span>);           <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>If <i>from</i> is -1, the search starts at the last character; if it is -2, at the next to last character and so on.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p></div>
    pub unsafe fn index_of_q_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_indexOf11(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Returns the index position of the first occurrence of the string <i>str</i> in this string, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::indexOf(QLatin1String s, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#indexOf-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of the string <i>str</i> in this string, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> x <span class="operator">=</span> <span class="string">"sticky question"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> y <span class="operator">=</span> <span class="string">"sti"</span>;
    ///   x<span class="operator">.</span>indexOf(y);               <span class="comment">// returns 0</span>
    ///   x<span class="operator">.</span>indexOf(y<span class="operator">,</span> <span class="number">1</span>);            <span class="comment">// returns 10</span>
    ///   x<span class="operator">.</span>indexOf(y<span class="operator">,</span> <span class="number">10</span>);           <span class="comment">// returns 10</span>
    ///   x<span class="operator">.</span>indexOf(y<span class="operator">,</span> <span class="number">11</span>);           <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>If <i>from</i> is -1, the search starts at the last character; if it is -2, at the next to last character and so on.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p></div>
    pub unsafe fn index_of_q_latin1_string_int(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_indexOf12(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first occurrence of the string <i>str</i> in this string, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::indexOf(QLatin1String s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#indexOf-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of the string <i>str</i> in this string, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> x <span class="operator">=</span> <span class="string">"sticky question"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> y <span class="operator">=</span> <span class="string">"sti"</span>;
    ///   x<span class="operator">.</span>indexOf(y);               <span class="comment">// returns 0</span>
    ///   x<span class="operator">.</span>indexOf(y<span class="operator">,</span> <span class="number">1</span>);            <span class="comment">// returns 10</span>
    ///   x<span class="operator">.</span>indexOf(y<span class="operator">,</span> <span class="number">10</span>);           <span class="comment">// returns 10</span>
    ///   x<span class="operator">.</span>indexOf(y<span class="operator">,</span> <span class="number">11</span>);           <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>If <i>from</i> is -1, the search starts at the last character; if it is -2, at the next to last character and so on.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p></div>
    pub unsafe fn index_of_q_latin1_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_indexOf13(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::indexOf(const QStringRef& s, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#indexOf-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p>
    /// <p>Returns the index position of the first occurrence of the string reference <i>str</i> in this string, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn index_of_q_string_ref_int(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_indexOf14(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
            from,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::indexOf(const QStringRef& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#indexOf-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p>
    /// <p>Returns the index position of the first occurrence of the string reference <i>str</i> in this string, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn index_of_q_string_ref(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_indexOf15(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::indexOf(const QRegExp& arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#indexOf-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p>
    /// <p>Returns the index position of the first match of the regular expression <i>rx</i> in the string, searching forward from index position <i>from</i>. Returns -1 if <i>rx</i> didn't match anywhere.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"the minimum"</span>;
    ///   str<span class="operator">.</span>indexOf(<span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a></span>(<span class="string">"m[aeiou]"</span>)<span class="operator">,</span> <span class="number">0</span>);       <span class="comment">// returns 4</span>
    ///
    /// </pre></div>
    pub unsafe fn index_of_q_reg_exp(
        &self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_indexOf16(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(arg1).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::indexOf(QRegExp& arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#indexOf-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p>
    /// <p>Returns the index position of the first match of the regular expression <i>rx</i> in the string, searching forward from index position <i>from</i>. Returns -1 if <i>rx</i> didn't match anywhere.</p>
    /// <p>If there is a match, the <i>rx</i> regular expression will contain the matched captures (see <a href="http://doc.qt.io/qt-5/qregexp.html#matchedLength">QRegExp::matchedLength</a>, <a href="http://doc.qt.io/qt-5/qregexp.html#cap">QRegExp::cap</a>).</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"the minimum"</span>;
    ///   str<span class="operator">.</span>indexOf(<span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a></span>(<span class="string">"m[aeiou]"</span>)<span class="operator">,</span> <span class="number">0</span>);       <span class="comment">// returns 4</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn index_of_q_reg_exp2(
        &self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QRegExp>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_indexOf17(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QRegExp>>::cast_into(arg1)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::indexOf(const QRegularExpression& re) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#indexOf-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>().</p>
    /// <p>Returns the index position of the first match of the regular expression <i>re</i> in the string, searching forward from index position <i>from</i>. Returns -1 if <i>re</i> didn't match anywhere.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"the minimum"</span>;
    ///   str<span class="operator">.</span>indexOf(<span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a></span>(<span class="string">"m[aeiou]"</span>)<span class="operator">,</span> <span class="number">0</span>);       <span class="comment">// returns 4</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn index_of_q_regular_expression(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_indexOf18(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#insert">insert</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::insert(int i, QChar c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#insert-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#insert">insert</a>().</p>
    /// <p>Inserts <i>ch</i> at the given index <i>position</i> in the string.</p></div>
    pub unsafe fn insert_int_q_char(
        &mut self,
        i: ::std::os::raw::c_int,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_insert(
            self as *mut crate::QString,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#insert">insert</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::insert(int i, const QChar* uc, int len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#insert">insert</a>().</p>
    /// <p>Inserts the first <i>size</i> characters of the <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> array <i>unicode</i> at the given index <i>position</i> in the string.</p></div>
    pub unsafe fn insert_int_q_char_int(
        &mut self,
        i: ::std::os::raw::c_int,
        uc: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QChar>>,
        len: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_insert1(
            self as *mut crate::QString,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QChar>>::cast_into(uc).as_raw_ptr(),
            len,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Inserts the string <i>str</i> at the given index <i>position</i> and returns a reference to this string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::insert(int i, const QString& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts the string <i>str</i> at the given index <i>position</i> and returns a reference to this string.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"Meal"</span>;
    ///   str<span class="operator">.</span>insert(<span class="number">1</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"ontr"</span>));
    ///   <span class="comment">// str == "Montreal"</span>
    ///
    /// </pre>
    /// <p>If the given <i>position</i> is greater than <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>(), the array is first extended using <a href="http://doc.qt.io/qt-5/qstring.html#resize">resize</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#remove">remove</a>().</p></div>
    pub unsafe fn insert_int_q_string(
        &mut self,
        i: ::std::os::raw::c_int,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_insert2(
            self as *mut crate::QString,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#insert">insert</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::insert(int i, const QStringRef& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#insert-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#insert">insert</a>().</p>
    /// <p>Inserts the string reference <i>str</i> at the given index <i>position</i> and returns a reference to this string.</p>
    /// <p>If the given <i>position</i> is greater than <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>(), the array is first extended using <a href="http://doc.qt.io/qt-5/qstring.html#resize">resize</a>().</p>
    /// <p>This function was introduced in  Qt 5.5.</p></div>
    pub unsafe fn insert_int_q_string_ref(
        &mut self,
        i: ::std::os::raw::c_int,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_insert3(
            self as *mut crate::QString,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#insert">insert</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::insert(int i, QLatin1String s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#insert-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#insert">insert</a>().</p>
    /// <p>Inserts the Latin-1 string <i>str</i> at the given index <i>position</i>.</p></div>
    pub unsafe fn insert_int_q_latin1_string(
        &mut self,
        i: ::std::os::raw::c_int,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_insert4(
            self as *mut crate::QString,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#insert">insert</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::insert(int i, const char* s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#insert-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#insert">insert</a>().</p>
    /// <p>Inserts the C string <i>str</i> at the given index <i>position</i> and returns a reference to this string.</p>
    /// <p>If the given <i>position</i> is greater than <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>(), the array is first extended using <a href="http://doc.qt.io/qt-5/qstring.html#resize">resize</a>().</p>
    /// <p>This function is not available when <a href="http://doc.qt.io/qt-5/qstring.html#QT_NO_CAST_FROM_ASCII">QT_NO_CAST_FROM_ASCII</a> is defined.</p>
    /// <p>This function was introduced in  Qt 5.5.</p></div>
    pub unsafe fn insert_int_char(
        &mut self,
        i: ::std::os::raw::c_int,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_insert5(
            self as *mut crate::QString,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#insert">insert</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::insert(int i, const QByteArray& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#insert-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#insert">insert</a>().</p>
    /// <p>Inserts the byte array <i>str</i> at the given index <i>position</i> and returns a reference to this string.</p>
    /// <p>If the given <i>position</i> is greater than <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>(), the array is first extended using <a href="http://doc.qt.io/qt-5/qstring.html#resize">resize</a>().</p>
    /// <p>This function is not available when <a href="http://doc.qt.io/qt-5/qstring.html#QT_NO_CAST_FROM_ASCII">QT_NO_CAST_FROM_ASCII</a> is defined.</p>
    /// <p>This function was introduced in  Qt 5.5.</p></div>
    pub unsafe fn insert_int_q_byte_array(
        &mut self,
        i: ::std::os::raw::c_int,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_insert6(
            self as *mut crate::QString,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(s).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QString::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_isDetached(self as *const crate::QString)
    }

    /// <p>Returns <code>true</code> if the string has no characters; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the string has no characters; otherwise returns <code>false</code>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>()<span class="operator">.</span>isEmpty();            <span class="comment">// returns true</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">""</span>)<span class="operator">.</span>isEmpty();          <span class="comment">// returns true</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"x"</span>)<span class="operator">.</span>isEmpty();         <span class="comment">// returns false</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"abc"</span>)<span class="operator">.</span>isEmpty();       <span class="comment">// returns false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_isEmpty(self as *const crate::QString)
    }

    /// <p>Returns <code>true</code> if the string only contains lowercase letters, otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::isLower() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#isLower">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the string only contains lowercase letters, otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 5.12.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#isLower">QChar::isLower</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#isUpper">isUpper</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_lower(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_isLower(self as *const crate::QString)
    }

    /// <p>Returns <code>true</code> if this string is null; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::isNull() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#isNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this string is null; otherwise returns <code>false</code>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>()<span class="operator">.</span>isNull();             <span class="comment">// returns true</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">""</span>)<span class="operator">.</span>isNull();           <span class="comment">// returns false</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"abc"</span>)<span class="operator">.</span>isNull();        <span class="comment">// returns false</span>
    ///
    /// </pre>
    /// <p>Qt makes a distinction between null strings and empty strings for historical reasons. For most applications, what matters is whether or not a string contains any data, and this can be determined using the <a href="http://doc.qt.io/qt-5/qstring.html#isEmpty">isEmpty</a>() function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_isNull(self as *const crate::QString)
    }

    /// <p>Returns <code>true</code> if the string is read right to left.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::isRightToLeft() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#isRightToLeft">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the string is read right to left.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#isRightToLeft">QStringRef::isRightToLeft</a>().</p></div>
    pub unsafe fn is_right_to_left(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_isRightToLeft(self as *const crate::QString)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QString::isSharedWith(const QString& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_isSharedWith(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QString::isSimpleText() const```</span>.
    pub unsafe fn is_simple_text(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_isSimpleText(self as *const crate::QString)
    }

    /// <p>Returns <code>true</code> if the string only contains uppercase letters, otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::isUpper() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#isUpper">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the string only contains uppercase letters, otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 5.12.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#isUpper">QChar::isUpper</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#isLower">isLower</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_upper(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_isUpper(self as *const crate::QString)
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::lastIndexOf(QChar c, int from = …, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    /// <p>Returns the index position of the last occurrence of the character <i>ch</i>, searching backward from position <i>from</i>.</p></div>
    pub unsafe fn last_index_of_q_char_int_case_sensitivity(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        from: ::std::os::raw::c_int,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_lastIndexOf(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
            from,
            cs,
        )
    }

    /// <p>Returns the index position of the last occurrence of the string <i>str</i> in this string, searching backward from index position <i>from</i>. If <i>from</i> is -1 (default), the search starts at the last character; if <i>from</i> is -2, at the next to last character and so on. Returns -1 if <i>str</i> is not found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::lastIndexOf(const QString& s, int from = …, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the string <i>str</i> in this string, searching backward from index position <i>from</i>. If <i>from</i> is -1 (default), the search starts at the last character; if <i>from</i> is -2, at the next to last character and so on. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> x <span class="operator">=</span> <span class="string">"crazy azimuths"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> y <span class="operator">=</span> <span class="string">"az"</span>;
    ///   x<span class="operator">.</span>lastIndexOf(y);           <span class="comment">// returns 6</span>
    ///   x<span class="operator">.</span>lastIndexOf(y<span class="operator">,</span> <span class="number">6</span>);        <span class="comment">// returns 6</span>
    ///   x<span class="operator">.</span>lastIndexOf(y<span class="operator">,</span> <span class="number">5</span>);        <span class="comment">// returns 2</span>
    ///   x<span class="operator">.</span>lastIndexOf(y<span class="operator">,</span> <span class="number">1</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p></div>
    pub unsafe fn last_index_of_q_string_int_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        from: ::std::os::raw::c_int,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_lastIndexOf1(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            from,
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::lastIndexOf(QLatin1String s, int from = …, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    /// <p>Returns the index position of the last occurrence of the string <i>str</i> in this string, searching backward from index position <i>from</i>. If <i>from</i> is -1 (default), the search starts at the last character; if <i>from</i> is -2, at the next to last character and so on. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> x <span class="operator">=</span> <span class="string">"crazy azimuths"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> y <span class="operator">=</span> <span class="string">"az"</span>;
    ///   x<span class="operator">.</span>lastIndexOf(y);           <span class="comment">// returns 6</span>
    ///   x<span class="operator">.</span>lastIndexOf(y<span class="operator">,</span> <span class="number">6</span>);        <span class="comment">// returns 6</span>
    ///   x<span class="operator">.</span>lastIndexOf(y<span class="operator">,</span> <span class="number">5</span>);        <span class="comment">// returns 2</span>
    ///   x<span class="operator">.</span>lastIndexOf(y<span class="operator">,</span> <span class="number">1</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p></div>
    pub unsafe fn last_index_of_q_latin1_string_int_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        from: ::std::os::raw::c_int,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_lastIndexOf2(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
            from,
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::lastIndexOf(const QStringRef& s, int from = …, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    /// <p>Returns the index position of the last occurrence of the string reference <i>str</i> in this string, searching backward from index position <i>from</i>. If <i>from</i> is -1 (default), the search starts at the last character; if <i>from</i> is -2, at the next to last character and so on. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p></div>
    pub unsafe fn last_index_of_q_string_ref_int_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        from: ::std::os::raw::c_int,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_lastIndexOf3(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
            from,
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::lastIndexOf(const QRegExp& arg1, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    /// <p>Returns the index position of the last match of the regular expression <i>rx</i> in the string, searching backward from index position <i>from</i>. Returns -1 if <i>rx</i> didn't match anywhere.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"the minimum"</span>;
    ///   str<span class="operator">.</span>lastIndexOf(<span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a></span>(<span class="string">"m[aeiou]"</span>));      <span class="comment">// returns 8</span>
    ///
    /// </pre></div>
    pub unsafe fn last_index_of_q_reg_exp_int(
        &self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_lastIndexOf4(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(arg1).as_raw_ptr(),
            from,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::lastIndexOf(QRegExp& arg1, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    /// <p>Returns the index position of the last match of the regular expression <i>rx</i> in the string, searching backward from index position <i>from</i>. Returns -1 if <i>rx</i> didn't match anywhere.</p>
    /// <p>If there is a match, the <i>rx</i> regular expression will contain the matched captures (see <a href="http://doc.qt.io/qt-5/qregexp.html#matchedLength">QRegExp::matchedLength</a>, <a href="http://doc.qt.io/qt-5/qregexp.html#cap">QRegExp::cap</a>).</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"the minimum"</span>;
    ///   str<span class="operator">.</span>lastIndexOf(<span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a></span>(<span class="string">"m[aeiou]"</span>));      <span class="comment">// returns 8</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn last_index_of_q_reg_exp_int2(
        &self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QRegExp>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_lastIndexOf5(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QRegExp>>::cast_into(arg1)
                .as_mut_raw_ptr(),
            from,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::lastIndexOf(const QRegularExpression& re, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    /// <p>Returns the index position of the last match of the regular expression <i>re</i> in the string, which starts before the index position <i>from</i>. Returns -1 if <i>re</i> didn't match anywhere.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"the minimum"</span>;
    ///   str<span class="operator">.</span>lastIndexOf(<span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a></span>(<span class="string">"m[aeiou]"</span>));      <span class="comment">// returns 8</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn last_index_of_q_regular_expression_int(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_lastIndexOf6(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::lastIndexOf(const QRegularExpression& re, int from, QRegularExpressionMatch* rmatch) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the index position of the last match of the regular expression <i>re</i> in the string, which starts before the index position <i>from</i>. Returns -1 if <i>re</i> didn't match anywhere.</p>
    /// <p>If the match is successful and <i>rmatch</i> is not a null pointer, it also writes the results of the match into the <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> object pointed to by <i>rmatch</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"the minimum"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a></span> match;
    ///   str<span class="operator">.</span>lastIndexOf(<span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a></span>(<span class="string">"m[aeiou]"</span>)<span class="operator">,</span> <span class="operator">-</span><span class="number">1</span><span class="operator">,</span> <span class="operator">&amp;</span>match);      <span class="comment">// returns 8</span>
    ///   <span class="comment">// match.captured() == mu</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.5.</p></div>
    pub unsafe fn last_index_of_q_regular_expression_int_q_regular_expression_match(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
        from: ::std::os::raw::c_int,
        rmatch: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QRegularExpressionMatch>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_lastIndexOf7(self as *const crate::QString, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re).as_raw_ptr(), from, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QRegularExpressionMatch>>::cast_into(rmatch).as_mut_raw_ptr())
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::lastIndexOf(QChar c, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    /// <p>Returns the index position of the last occurrence of the character <i>ch</i>, searching backward from position <i>from</i>.</p></div>
    pub unsafe fn last_index_of_q_char_int(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_lastIndexOf8(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
            from,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::lastIndexOf(QChar c) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    /// <p>Returns the index position of the last occurrence of the character <i>ch</i>, searching backward from position <i>from</i>.</p></div>
    pub unsafe fn last_index_of_q_char(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_lastIndexOf9(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
        )
    }

    /// <p>Returns the index position of the last occurrence of the string <i>str</i> in this string, searching backward from index position <i>from</i>. If <i>from</i> is -1 (default), the search starts at the last character; if <i>from</i> is -2, at the next to last character and so on. Returns -1 if <i>str</i> is not found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::lastIndexOf(const QString& s, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the string <i>str</i> in this string, searching backward from index position <i>from</i>. If <i>from</i> is -1 (default), the search starts at the last character; if <i>from</i> is -2, at the next to last character and so on. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> x <span class="operator">=</span> <span class="string">"crazy azimuths"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> y <span class="operator">=</span> <span class="string">"az"</span>;
    ///   x<span class="operator">.</span>lastIndexOf(y);           <span class="comment">// returns 6</span>
    ///   x<span class="operator">.</span>lastIndexOf(y<span class="operator">,</span> <span class="number">6</span>);        <span class="comment">// returns 6</span>
    ///   x<span class="operator">.</span>lastIndexOf(y<span class="operator">,</span> <span class="number">5</span>);        <span class="comment">// returns 2</span>
    ///   x<span class="operator">.</span>lastIndexOf(y<span class="operator">,</span> <span class="number">1</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p></div>
    pub unsafe fn last_index_of_q_string_int(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_lastIndexOf10(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the last occurrence of the string <i>str</i> in this string, searching backward from index position <i>from</i>. If <i>from</i> is -1 (default), the search starts at the last character; if <i>from</i> is -2, at the next to last character and so on. Returns -1 if <i>str</i> is not found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::lastIndexOf(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the string <i>str</i> in this string, searching backward from index position <i>from</i>. If <i>from</i> is -1 (default), the search starts at the last character; if <i>from</i> is -2, at the next to last character and so on. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> x <span class="operator">=</span> <span class="string">"crazy azimuths"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> y <span class="operator">=</span> <span class="string">"az"</span>;
    ///   x<span class="operator">.</span>lastIndexOf(y);           <span class="comment">// returns 6</span>
    ///   x<span class="operator">.</span>lastIndexOf(y<span class="operator">,</span> <span class="number">6</span>);        <span class="comment">// returns 6</span>
    ///   x<span class="operator">.</span>lastIndexOf(y<span class="operator">,</span> <span class="number">5</span>);        <span class="comment">// returns 2</span>
    ///   x<span class="operator">.</span>lastIndexOf(y<span class="operator">,</span> <span class="number">1</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p></div>
    pub unsafe fn last_index_of_q_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_lastIndexOf11(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::lastIndexOf(QLatin1String s, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    /// <p>Returns the index position of the last occurrence of the string <i>str</i> in this string, searching backward from index position <i>from</i>. If <i>from</i> is -1 (default), the search starts at the last character; if <i>from</i> is -2, at the next to last character and so on. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> x <span class="operator">=</span> <span class="string">"crazy azimuths"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> y <span class="operator">=</span> <span class="string">"az"</span>;
    ///   x<span class="operator">.</span>lastIndexOf(y);           <span class="comment">// returns 6</span>
    ///   x<span class="operator">.</span>lastIndexOf(y<span class="operator">,</span> <span class="number">6</span>);        <span class="comment">// returns 6</span>
    ///   x<span class="operator">.</span>lastIndexOf(y<span class="operator">,</span> <span class="number">5</span>);        <span class="comment">// returns 2</span>
    ///   x<span class="operator">.</span>lastIndexOf(y<span class="operator">,</span> <span class="number">1</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p></div>
    pub unsafe fn last_index_of_q_latin1_string_int(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_lastIndexOf12(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::lastIndexOf(QLatin1String s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    /// <p>Returns the index position of the last occurrence of the string <i>str</i> in this string, searching backward from index position <i>from</i>. If <i>from</i> is -1 (default), the search starts at the last character; if <i>from</i> is -2, at the next to last character and so on. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> x <span class="operator">=</span> <span class="string">"crazy azimuths"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> y <span class="operator">=</span> <span class="string">"az"</span>;
    ///   x<span class="operator">.</span>lastIndexOf(y);           <span class="comment">// returns 6</span>
    ///   x<span class="operator">.</span>lastIndexOf(y<span class="operator">,</span> <span class="number">6</span>);        <span class="comment">// returns 6</span>
    ///   x<span class="operator">.</span>lastIndexOf(y<span class="operator">,</span> <span class="number">5</span>);        <span class="comment">// returns 2</span>
    ///   x<span class="operator">.</span>lastIndexOf(y<span class="operator">,</span> <span class="number">1</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p></div>
    pub unsafe fn last_index_of_q_latin1_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_lastIndexOf13(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::lastIndexOf(const QStringRef& s, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    /// <p>Returns the index position of the last occurrence of the string reference <i>str</i> in this string, searching backward from index position <i>from</i>. If <i>from</i> is -1 (default), the search starts at the last character; if <i>from</i> is -2, at the next to last character and so on. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p></div>
    pub unsafe fn last_index_of_q_string_ref_int(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_lastIndexOf14(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
            from,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::lastIndexOf(const QStringRef& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    /// <p>Returns the index position of the last occurrence of the string reference <i>str</i> in this string, searching backward from index position <i>from</i>. If <i>from</i> is -1 (default), the search starts at the last character; if <i>from</i> is -2, at the next to last character and so on. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#count-2">count</a>().</p></div>
    pub unsafe fn last_index_of_q_string_ref(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_lastIndexOf15(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::lastIndexOf(const QRegExp& arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    /// <p>Returns the index position of the last match of the regular expression <i>rx</i> in the string, searching backward from index position <i>from</i>. Returns -1 if <i>rx</i> didn't match anywhere.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"the minimum"</span>;
    ///   str<span class="operator">.</span>lastIndexOf(<span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a></span>(<span class="string">"m[aeiou]"</span>));      <span class="comment">// returns 8</span>
    ///
    /// </pre></div>
    pub unsafe fn last_index_of_q_reg_exp(
        &self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_lastIndexOf16(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(arg1).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::lastIndexOf(QRegExp& arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    /// <p>Returns the index position of the last match of the regular expression <i>rx</i> in the string, searching backward from index position <i>from</i>. Returns -1 if <i>rx</i> didn't match anywhere.</p>
    /// <p>If there is a match, the <i>rx</i> regular expression will contain the matched captures (see <a href="http://doc.qt.io/qt-5/qregexp.html#matchedLength">QRegExp::matchedLength</a>, <a href="http://doc.qt.io/qt-5/qregexp.html#cap">QRegExp::cap</a>).</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"the minimum"</span>;
    ///   str<span class="operator">.</span>lastIndexOf(<span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a></span>(<span class="string">"m[aeiou]"</span>));      <span class="comment">// returns 8</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn last_index_of_q_reg_exp2(
        &self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QRegExp>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_lastIndexOf17(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QRegExp>>::cast_into(arg1)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::lastIndexOf(const QRegularExpression& re) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>().</p>
    /// <p>Returns the index position of the last match of the regular expression <i>re</i> in the string, which starts before the index position <i>from</i>. Returns -1 if <i>re</i> didn't match anywhere.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"the minimum"</span>;
    ///   str<span class="operator">.</span>lastIndexOf(<span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a></span>(<span class="string">"m[aeiou]"</span>));      <span class="comment">// returns 8</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn last_index_of_q_regular_expression(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_lastIndexOf18(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a substring that contains the <i>n</i> leftmost characters of the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::left(int n) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#left">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a substring that contains the <i>n</i> leftmost characters of the string.</p>
    /// <p>The entire string is returned if <i>n</i> is greater than or equal to <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>(), or less than zero.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> x <span class="operator">=</span> <span class="string">"Pineapple"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> y <span class="operator">=</span> x<span class="operator">.</span>left(<span class="number">4</span>);      <span class="comment">// y == "Pine"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#right">right</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#mid">mid</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#startsWith">startsWith</a>().</p></div>
    pub unsafe fn left(&self, n: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_left(self as *const crate::QString, n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a string of size <i>width</i> that contains this string padded by the <i>fill</i> character.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::leftJustified(int width, QChar fill = …, bool trunc = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#leftJustified">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string of size <i>width</i> that contains this string padded by the <i>fill</i> character.</p>
    /// <p>If <i>truncate</i> is <code>false</code> and the <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>() of the string is more than <i>width</i>, then the returned string is a copy of the string.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> s <span class="operator">=</span> <span class="string">"apple"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> t <span class="operator">=</span> s<span class="operator">.</span>leftJustified(<span class="number">8</span><span class="operator">,</span> <span class="char">'.'</span>);    <span class="comment">// t == "apple..."</span>
    ///
    /// </pre>
    /// <p>If <i>truncate</i> is <code>true</code> and the <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>() of the string is more than <i>width</i>, then any characters in a copy of the string after position <i>width</i> are removed, and the copy is returned.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"Pineapple"</span>;
    ///   str <span class="operator">=</span> str<span class="operator">.</span>leftJustified(<span class="number">5</span><span class="operator">,</span> <span class="char">'.'</span><span class="operator">,</span> <span class="keyword">true</span>);    <span class="comment">// str == "Pinea"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#rightJustified">rightJustified</a>().</p></div>
    pub unsafe fn left_justified_3a(
        &self,
        width: ::std::os::raw::c_int,
        fill: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        trunc: bool,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_leftJustified(
            self as *const crate::QString,
            width,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(fill).as_raw_ptr(),
            trunc,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a string of size <i>width</i> that contains this string padded by the <i>fill</i> character.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::leftJustified(int width, QChar fill = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#leftJustified">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string of size <i>width</i> that contains this string padded by the <i>fill</i> character.</p>
    /// <p>If <i>truncate</i> is <code>false</code> and the <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>() of the string is more than <i>width</i>, then the returned string is a copy of the string.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> s <span class="operator">=</span> <span class="string">"apple"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> t <span class="operator">=</span> s<span class="operator">.</span>leftJustified(<span class="number">8</span><span class="operator">,</span> <span class="char">'.'</span>);    <span class="comment">// t == "apple..."</span>
    ///
    /// </pre>
    /// <p>If <i>truncate</i> is <code>true</code> and the <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>() of the string is more than <i>width</i>, then any characters in a copy of the string after position <i>width</i> are removed, and the copy is returned.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"Pineapple"</span>;
    ///   str <span class="operator">=</span> str<span class="operator">.</span>leftJustified(<span class="number">5</span><span class="operator">,</span> <span class="char">'.'</span><span class="operator">,</span> <span class="keyword">true</span>);    <span class="comment">// str == "Pinea"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#rightJustified">rightJustified</a>().</p></div>
    pub unsafe fn left_justified_2a(
        &self,
        width: ::std::os::raw::c_int,
        fill: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_leftJustified1(
            self as *const crate::QString,
            width,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(fill).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a string of size <i>width</i> that contains this string padded by the <i>fill</i> character.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::leftJustified(int width) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#leftJustified">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string of size <i>width</i> that contains this string padded by the <i>fill</i> character.</p>
    /// <p>If <i>truncate</i> is <code>false</code> and the <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>() of the string is more than <i>width</i>, then the returned string is a copy of the string.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> s <span class="operator">=</span> <span class="string">"apple"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> t <span class="operator">=</span> s<span class="operator">.</span>leftJustified(<span class="number">8</span><span class="operator">,</span> <span class="char">'.'</span>);    <span class="comment">// t == "apple..."</span>
    ///
    /// </pre>
    /// <p>If <i>truncate</i> is <code>true</code> and the <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>() of the string is more than <i>width</i>, then any characters in a copy of the string after position <i>width</i> are removed, and the copy is returned.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"Pineapple"</span>;
    ///   str <span class="operator">=</span> str<span class="operator">.</span>leftJustified(<span class="number">5</span><span class="operator">,</span> <span class="char">'.'</span><span class="operator">,</span> <span class="keyword">true</span>);    <span class="comment">// str == "Pinea"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#rightJustified">rightJustified</a>().</p></div>
    pub unsafe fn left_justified_1a(
        &self,
        width: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_leftJustified2(
            self as *const crate::QString,
            width,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a substring reference to the <i>n</i> leftmost characters of the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QString::leftRef(int n) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#leftRef">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a substring reference to the <i>n</i> leftmost characters of the string.</p>
    /// <p>If <i>n</i> is greater than or equal to <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>(), or less than zero, a reference to the entire string is returned.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> x <span class="operator">=</span> <span class="string">"Pineapple"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a></span> y <span class="operator">=</span> x<span class="operator">.</span>leftRef(<span class="number">4</span>);        <span class="comment">// y == "Pine"</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#left">left</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#rightRef">rightRef</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#midRef">midRef</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#startsWith">startsWith</a>().</p></div>
    pub unsafe fn left_ref(
        &self,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_leftRef(self as *const crate::QString, n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the number of characters in this string. Equivalent to <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of characters in this string. Equivalent to <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#resize">resize</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_length(self as *const crate::QString)
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#localeAwareCompare">localeAwareCompare</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::localeAwareCompare(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#localeAwareCompare-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#localeAwareCompare">localeAwareCompare</a>().</p>
    /// <p>Compares this string with the <i>other</i> string and returns an integer less than, equal to, or greater than zero if this string is less than, equal to, or greater than the <i>other</i> string.</p>
    /// <p>The comparison is performed in a locale- and also platform-dependent manner. Use this function to present sorted lists of strings to the user.</p>
    /// <p>Same as <code>localeAwareCompare(*this, other)</code>.</p></div>
    pub unsafe fn locale_aware_compare_q_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_localeAwareCompare(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Compares <i>s1</i> with <i>s2</i> and returns an integer less than, equal to, or greater than zero if <i>s1</i> is less than, equal to, or greater than <i>s2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QString::localeAwareCompare(const QString& s1, const QString& s2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#localeAwareCompare">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares <i>s1</i> with <i>s2</i> and returns an integer less than, equal to, or greater than zero if <i>s1</i> is less than, equal to, or greater than <i>s2</i>.</p>
    /// <p>The comparison is performed in a locale- and also platform-dependent manner. Use this function to present sorted lists of strings to the user.</p>
    /// <p>On <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS this function compares according the "Order for sorted lists" setting in the International preferences panel.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#compare">compare</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a>.</p></div>
    pub unsafe fn locale_aware_compare_2_q_string(
        s1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        s2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_localeAwareCompare1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s2).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#localeAwareCompare">localeAwareCompare</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::localeAwareCompare(const QStringRef& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#localeAwareCompare-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#localeAwareCompare">localeAwareCompare</a>().</p>
    /// <p>Compares this string with the <i>other</i> string and returns an integer less than, equal to, or greater than zero if this string is less than, equal to, or greater than the <i>other</i> string.</p>
    /// <p>The comparison is performed in a locale- and also platform-dependent manner. Use this function to present sorted lists of strings to the user.</p>
    /// <p>Same as <code>localeAwareCompare(*this, other)</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn locale_aware_compare_q_string_ref(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_localeAwareCompare2(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#localeAwareCompare">localeAwareCompare</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QString::localeAwareCompare(const QString& s1, const QStringRef& s2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#localeAwareCompare-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#localeAwareCompare">localeAwareCompare</a>().</p>
    /// <p>Compares <i>s1</i> with <i>s2</i> and returns an integer less than, equal to, or greater than zero if <i>s1</i> is less than, equal to, or greater than <i>s2</i>.</p>
    /// <p>The comparison is performed in a locale- and also platform-dependent manner. Use this function to present sorted lists of strings to the user.</p>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn locale_aware_compare_q_string_q_string_ref(
        s1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        s2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_localeAwareCompare3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s2)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a string that contains <i>n</i> characters of this string, starting at the specified <i>position</i> index.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::mid(int position, int n = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string that contains <i>n</i> characters of this string, starting at the specified <i>position</i> index.</p>
    /// <p>Returns a null string if the <i>position</i> index exceeds the length of the string. If there are less than <i>n</i> characters available in the string starting at the given <i>position</i>, or if <i>n</i> is -1 (default), the function returns all characters that are available from the specified <i>position</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> x <span class="operator">=</span> <span class="string">"Nine pineapples"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> y <span class="operator">=</span> x<span class="operator">.</span>mid(<span class="number">5</span><span class="operator">,</span> <span class="number">4</span>);            <span class="comment">// y == "pine"</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> z <span class="operator">=</span> x<span class="operator">.</span>mid(<span class="number">5</span>);               <span class="comment">// z == "pineapples"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#left">left</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#right">right</a>().</p></div>
    pub unsafe fn mid_2a(
        &self,
        position: ::std::os::raw::c_int,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_mid(self as *const crate::QString, position, n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a string that contains <i>n</i> characters of this string, starting at the specified <i>position</i> index.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::mid(int position) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string that contains <i>n</i> characters of this string, starting at the specified <i>position</i> index.</p>
    /// <p>Returns a null string if the <i>position</i> index exceeds the length of the string. If there are less than <i>n</i> characters available in the string starting at the given <i>position</i>, or if <i>n</i> is -1 (default), the function returns all characters that are available from the specified <i>position</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> x <span class="operator">=</span> <span class="string">"Nine pineapples"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> y <span class="operator">=</span> x<span class="operator">.</span>mid(<span class="number">5</span><span class="operator">,</span> <span class="number">4</span>);            <span class="comment">// y == "pine"</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> z <span class="operator">=</span> x<span class="operator">.</span>mid(<span class="number">5</span>);               <span class="comment">// z == "pineapples"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#left">left</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#right">right</a>().</p></div>
    pub unsafe fn mid_1a(
        &self,
        position: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_mid1(self as *const crate::QString, position);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a substring reference to <i>n</i> characters of this string, starting at the specified <i>position</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QString::midRef(int position, int n = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#midRef">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a substring reference to <i>n</i> characters of this string, starting at the specified <i>position</i>.</p>
    /// <p>If the <i>position</i> exceeds the length of the string, a null reference is returned.</p>
    /// <p>If there are less than <i>n</i> characters available in the string, starting at the given <i>position</i>, or if <i>n</i> is -1 (default), the function returns all characters from the specified <i>position</i> onwards.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> x <span class="operator">=</span> <span class="string">"Nine pineapples"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a></span> y <span class="operator">=</span> x<span class="operator">.</span>midRef(<span class="number">5</span><span class="operator">,</span> <span class="number">4</span>);      <span class="comment">// y == "pine"</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a></span> z <span class="operator">=</span> x<span class="operator">.</span>midRef(<span class="number">5</span>);         <span class="comment">// z == "pineapples"</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#mid">mid</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#leftRef">leftRef</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#rightRef">rightRef</a>().</p></div>
    pub unsafe fn mid_ref_2a(
        &self,
        position: ::std::os::raw::c_int,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_midRef(
            self as *const crate::QString,
            position,
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a substring reference to <i>n</i> characters of this string, starting at the specified <i>position</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QString::midRef(int position) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#midRef">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a substring reference to <i>n</i> characters of this string, starting at the specified <i>position</i>.</p>
    /// <p>If the <i>position</i> exceeds the length of the string, a null reference is returned.</p>
    /// <p>If there are less than <i>n</i> characters available in the string, starting at the given <i>position</i>, or if <i>n</i> is -1 (default), the function returns all characters from the specified <i>position</i> onwards.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> x <span class="operator">=</span> <span class="string">"Nine pineapples"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a></span> y <span class="operator">=</span> x<span class="operator">.</span>midRef(<span class="number">5</span><span class="operator">,</span> <span class="number">4</span>);      <span class="comment">// y == "pine"</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a></span> z <span class="operator">=</span> x<span class="operator">.</span>midRef(<span class="number">5</span>);         <span class="comment">// z == "pineapples"</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#mid">mid</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#leftRef">leftRef</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#rightRef">rightRef</a>().</p></div>
    pub unsafe fn mid_ref_1a(
        &self,
        position: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_midRef1(self as *const crate::QString, position);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a null string. Null strings are also empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QString::QString()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#QString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a null string. Null strings are also empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_QString();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a string initialized with the first <i>size</i> characters of the <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> array <i>unicode</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QString::QString(const QChar* unicode, int size = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#QString-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a string initialized with the first <i>size</i> characters of the <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> array <i>unicode</i>.</p>
    /// <p>If <i>unicode</i> is 0, a null string is constructed.</p>
    /// <p>If <i>size</i> is negative, <i>unicode</i> is assumed to point to a nul-terminated array and its length is determined dynamically. The terminating nul-character is not considered part of the string.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qstring.html">QString</a> makes a deep copy of the string data. The unicode data is copied as is and the Byte Order Mark is preserved if present.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#fromRawData">fromRawData</a>().</p></div>
    pub unsafe fn from_q_char_int(
        unicode: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QChar>>,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_QString1(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QChar>>::cast_into(unicode)
                .as_raw_ptr(),
            size,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a string of size 1 containing the character <i>ch</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QString::QString(QChar c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#QString-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a string of size 1 containing the character <i>ch</i>.</p></div>
    pub unsafe fn from_q_char(
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_QString2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a string of the given <i>size</i> with every character set to <i>ch</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QString::QString(int size, QChar c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#QString-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a string of the given <i>size</i> with every character set to <i>ch</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#fill">fill</a>().</p></div>
    pub unsafe fn from_int_q_char(
        size: ::std::os::raw::c_int,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_QString3(
            size,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of the Latin-1 string <i>str</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QString::QString(QLatin1String latin1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#QString-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of the Latin-1 string <i>str</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#fromLatin1">fromLatin1</a>().</p></div>
    pub unsafe fn from_q_latin1_string(
        latin1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_QString4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(latin1)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a string initialized with the 8-bit string <i>str</i>. The given const char pointer is converted to Unicode using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>() function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QString::QString(const char* ch)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#QString-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a string initialized with the 8-bit string <i>str</i>. The given const char pointer is converted to Unicode using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>() function.</p>
    /// <p>You can disable this constructor by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p>
    /// <p><b>Note: </b>Defining <a href="http://doc.qt.io/qt-5/qstring.html#QT_RESTRICTED_CAST_FROM_ASCII">QT_RESTRICTED_CAST_FROM_ASCII</a> also disables this constructor, but enables a <code>QString(const char (&amp;ch)[N])</code> constructor instead. Using non-literal input, or input with embedded NUL characters, or non-7-bit characters is undefined in this case.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#fromLatin1">fromLatin1</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#fromLocal8Bit">fromLocal8Bit</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>().</p></div>
    pub unsafe fn from_char(
        ch: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_QString7(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(ch)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a string initialized with the byte array <i>ba</i>. The given byte array is converted to Unicode using <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>(). Stops copying at the first 0 character, otherwise copies the entire byte array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QString::QString(const QByteArray& a)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#QString-8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a string initialized with the byte array <i>ba</i>. The given byte array is converted to Unicode using <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>(). Stops copying at the first 0 character, otherwise copies the entire byte array.</p>
    /// <p>You can disable this constructor by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#fromLatin1">fromLatin1</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#fromLocal8Bit">fromLocal8Bit</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>().</p></div>
    pub unsafe fn from_q_byte_array(
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_QString8(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(a).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a string initialized with the first <i>size</i> characters of the <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> array <i>unicode</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QString::QString(const QChar* unicode)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#QString-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a string initialized with the first <i>size</i> characters of the <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> array <i>unicode</i>.</p>
    /// <p>If <i>unicode</i> is 0, a null string is constructed.</p>
    /// <p>If <i>size</i> is negative, <i>unicode</i> is assumed to point to a nul-terminated array and its length is determined dynamically. The terminating nul-character is not considered part of the string.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qstring.html">QString</a> makes a deep copy of the string data. The unicode data is copied as is and the Byte Order Mark is preserved if present.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#fromRawData">fromRawData</a>().</p></div>
    pub unsafe fn from_q_char2(
        unicode: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QChar>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_QString9(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QChar>>::cast_into(unicode)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QString::QString(const QString& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#QString-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#constant-time">constant time</a>, because <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_QString5(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(arg1).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the string in the given Unicode normalization <i>mode</i>, according to the given <i>version</i> of the Unicode standard.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::normalized(QString::NormalizationForm mode, QChar::UnicodeVersion version = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#normalized">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string in the given Unicode normalization <i>mode</i>, according to the given <i>version</i> of the Unicode standard.</p></div>
    pub unsafe fn normalized_2a(
        &self,
        mode: crate::q_string::NormalizationForm,
        version: crate::q_char::UnicodeVersion,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_normalized(
            self as *const crate::QString,
            mode,
            version,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the string in the given Unicode normalization <i>mode</i>, according to the given <i>version</i> of the Unicode standard.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::normalized(QString::NormalizationForm mode) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#normalized">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string in the given Unicode normalization <i>mode</i>, according to the given <i>version</i> of the Unicode standard.</p></div>
    pub unsafe fn normalized_1a(
        &self,
        mode: crate::q_string::NormalizationForm,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_normalized1(self as *const crate::QString, mode);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::number(int arg1, int base = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#number-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn number_2_int(
        arg1: ::std::os::raw::c_int,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_number(arg1, base);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::number(unsigned int arg1, int base = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#number-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn number_uint_int(
        arg1: ::std::os::raw::c_uint,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_number1(arg1, base);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a string equivalent of the number <i>n</i> according to the specified <i>base</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::number(long arg1, int base = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#number">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string equivalent of the number <i>n</i> according to the specified <i>base</i>.</p>
    /// <p>The base is 10 by default and must be between 2 and 36. For bases other than 10, <i>n</i> is treated as an unsigned integer.</p>
    /// <p>The formatting always uses <a href="http://doc.qt.io/qt-5/qlocale.html#Language-enum">QLocale::C</a>, i.e., English/UnitedStates. To get a localized string representation of a number, use <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>() with the appropriate locale.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">long</span> a <span class="operator">=</span> <span class="number">63</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> s <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span><span class="operator">::</span>number(a<span class="operator">,</span> <span class="number">16</span>);             <span class="comment">// s == "3f"</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> t <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span><span class="operator">::</span>number(a<span class="operator">,</span> <span class="number">16</span>)<span class="operator">.</span>toUpper();     <span class="comment">// t == "3F"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#setNum">setNum</a>().</p></div>
    pub unsafe fn number_long_int(
        arg1: ::std::os::raw::c_long,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_number2(arg1, base);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::number(unsigned long arg1, int base = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#number-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn number_ulong_int(
        arg1: ::std::os::raw::c_ulong,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_number3(arg1, base);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::number(qlonglong arg1, int base = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#number-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn number_i64_int(
        arg1: i64,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_number4(arg1, base);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::number(qulonglong arg1, int base = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#number-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn number_u64_int(
        arg1: u64,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_number5(arg1, base);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a string equivalent of the number <i>n</i>, formatted according to the specified <i>format</i> and <i>precision</i>. See <a href="http://doc.qt.io/qt-5/qstring.html#argument-formats">Argument Formats</a> for details.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::number(double arg1, char f = …, int prec = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#number-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string equivalent of the number <i>n</i>, formatted according to the specified <i>format</i> and <i>precision</i>. See <a href="http://doc.qt.io/qt-5/qstring.html#argument-formats">Argument Formats</a> for details.</p>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>(), this function does not honor the user's locale settings.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#setNum">setNum</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>().</p></div>
    pub unsafe fn number_double_char_int(
        arg1: ::std::os::raw::c_double,
        f: ::std::os::raw::c_char,
        prec: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_number6(arg1, f, prec);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::number(int arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#number-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn number_int(arg1: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_number7(arg1);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::number(unsigned int arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#number-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn number_uint(arg1: ::std::os::raw::c_uint) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_number8(arg1);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a string equivalent of the number <i>n</i> according to the specified <i>base</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::number(long arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#number">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string equivalent of the number <i>n</i> according to the specified <i>base</i>.</p>
    /// <p>The base is 10 by default and must be between 2 and 36. For bases other than 10, <i>n</i> is treated as an unsigned integer.</p>
    /// <p>The formatting always uses <a href="http://doc.qt.io/qt-5/qlocale.html#Language-enum">QLocale::C</a>, i.e., English/UnitedStates. To get a localized string representation of a number, use <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>() with the appropriate locale.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">long</span> a <span class="operator">=</span> <span class="number">63</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> s <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span><span class="operator">::</span>number(a<span class="operator">,</span> <span class="number">16</span>);             <span class="comment">// s == "3f"</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> t <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span><span class="operator">::</span>number(a<span class="operator">,</span> <span class="number">16</span>)<span class="operator">.</span>toUpper();     <span class="comment">// t == "3F"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#setNum">setNum</a>().</p></div>
    pub unsafe fn number_long(arg1: ::std::os::raw::c_long) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_number9(arg1);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::number(unsigned long arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#number-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn number_ulong(
        arg1: ::std::os::raw::c_ulong,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_number10(arg1);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::number(qlonglong arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#number-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn number_i64(arg1: i64) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_number11(arg1);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::number(qulonglong arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#number-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn number_u64(arg1: u64) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_number12(arg1);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a string equivalent of the number <i>n</i>, formatted according to the specified <i>format</i> and <i>precision</i>. See <a href="http://doc.qt.io/qt-5/qstring.html#argument-formats">Argument Formats</a> for details.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::number(double arg1, char f = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#number-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string equivalent of the number <i>n</i>, formatted according to the specified <i>format</i> and <i>precision</i>. See <a href="http://doc.qt.io/qt-5/qstring.html#argument-formats">Argument Formats</a> for details.</p>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>(), this function does not honor the user's locale settings.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#setNum">setNum</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>().</p></div>
    pub unsafe fn number_double_char(
        arg1: ::std::os::raw::c_double,
        f: ::std::os::raw::c_char,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_number13(arg1, f);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a string equivalent of the number <i>n</i>, formatted according to the specified <i>format</i> and <i>precision</i>. See <a href="http://doc.qt.io/qt-5/qstring.html#argument-formats">Argument Formats</a> for details.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QString::number(double arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#number-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string equivalent of the number <i>n</i>, formatted according to the specified <i>format</i> and <i>precision</i>. See <a href="http://doc.qt.io/qt-5/qstring.html#argument-formats">Argument Formats</a> for details.</p>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>(), this function does not honor the user's locale settings.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#setNum">setNum</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>().</p></div>
    pub unsafe fn number_double(
        arg1: ::std::os::raw::c_double,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_number14(arg1);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#prepend">prepend</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::prepend(QChar c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#prepend-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#prepend">prepend</a>().</p>
    /// <p>Prepends the character <i>ch</i> to this string.</p></div>
    pub unsafe fn prepend_q_char(
        &mut self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_prepend(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#prepend">prepend</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::prepend(const QChar* uc, int len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#prepend-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#prepend">prepend</a>().</p>
    /// <p>Prepends <i>len</i> characters from the <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> array <i>str</i> to this string and returns a reference to this string.</p>
    /// <p>This function was introduced in  Qt 5.5.</p></div>
    pub unsafe fn prepend_q_char_int(
        &mut self,
        uc: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QChar>>,
        len: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_prepend1(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QChar>>::cast_into(uc).as_raw_ptr(),
            len,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Prepends the string <i>str</i> to the beginning of this string and returns a reference to this string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::prepend(const QString& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Prepends the string <i>str</i> to the beginning of this string and returns a reference to this string.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> x <span class="operator">=</span> <span class="string">"ship"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> y <span class="operator">=</span> <span class="string">"air"</span>;
    ///   x<span class="operator">.</span>prepend(y);
    ///   <span class="comment">// x == "airship"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend_q_string(
        &mut self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_prepend2(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#prepend">prepend</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::prepend(const QStringRef& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#prepend-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#prepend">prepend</a>().</p>
    /// <p>Prepends the string reference <i>str</i> to the beginning of this string and returns a reference to this string.</p>
    /// <p>This function was introduced in  Qt 5.5.</p></div>
    pub unsafe fn prepend_q_string_ref(
        &mut self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_prepend3(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#prepend">prepend</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::prepend(QLatin1String s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#prepend-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#prepend">prepend</a>().</p>
    /// <p>Prepends the Latin-1 string <i>str</i> to this string.</p></div>
    pub unsafe fn prepend_q_latin1_string(
        &mut self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_prepend4(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#prepend">prepend</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::prepend(const char* s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#prepend-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#prepend">prepend</a>().</p>
    /// <p>Prepends the string <i>str</i> to this string. The const char pointer is converted to Unicode using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>() function.</p>
    /// <p>You can disable this function by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p></div>
    pub unsafe fn prepend_char(
        &mut self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_prepend5(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#prepend">prepend</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::prepend(const QByteArray& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#prepend-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#prepend">prepend</a>().</p>
    /// <p>Prepends the byte array <i>ba</i> to this string. The byte array is converted to Unicode using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>() function.</p>
    /// <p>You can disable this function by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p></div>
    pub unsafe fn prepend_q_byte_array(
        &mut self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_prepend6(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(s).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QString::push_back(QChar c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#push_back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the given <i>ch</i> character onto the end of this string.</p></div>
    pub unsafe fn push_back_q_char(
        &mut self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QString_push_back(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility, appending the given <i>other</i> string onto the end of this string. It is equivalent to <code>append(other)</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QString::push_back(const QString& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility, appending the given <i>other</i> string onto the end of this string. It is equivalent to <code>append(other)</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#append">append</a>().</p></div>
    pub unsafe fn push_back_q_string(
        &mut self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QString_push_back1(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QString::push_front(QChar c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#push_front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Prepends the given <i>ch</i> character to the beginning of this string.</p></div>
    pub unsafe fn push_front_q_char(
        &mut self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QString_push_front(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility, prepending the given <i>other</i> string to the beginning of this string. It is equivalent to <code>prepend(other)</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QString::push_front(const QString& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility, prepending the given <i>other</i> string to the beginning of this string. It is equivalent to <code>prepend(other)</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#prepend">prepend</a>().</p></div>
    pub unsafe fn push_front_q_string(
        &mut self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QString_push_front1(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Removes <i>n</i> characters from the string, starting at the given <i>position</i> index, and returns a reference to the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::remove(int i, int len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#remove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes <i>n</i> characters from the string, starting at the given <i>position</i> index, and returns a reference to the string.</p>
    /// <p>If the specified <i>position</i> index is within the string, but <i>position</i> + <i>n</i> is beyond the end of the string, the string is truncated at the specified <i>position</i>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> s <span class="operator">=</span> <span class="string">"Montreal"</span>;
    ///   s<span class="operator">.</span>remove(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// s == "Meal"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_2_int(
        &mut self,
        i: ::std::os::raw::c_int,
        len: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_remove(self as *mut crate::QString, i, len);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Removes every occurrence of the character <i>ch</i> in this string, and returns a reference to this string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::remove(QChar c, Qt::CaseSensitivity cs = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#remove-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes every occurrence of the character <i>ch</i> in this string, and returns a reference to this string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> t <span class="operator">=</span> <span class="string">"Ali Baba"</span>;
    ///   t<span class="operator">.</span>remove(<span class="type"><a href="http://doc.qt.io/qt-5/qchar.html">QChar</a></span>(<span class="char">'a'</span>)<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>CaseInsensitive);
    ///   <span class="comment">// t == "li Bb"</span>
    ///
    /// </pre>
    /// <p>This is the same as <code>replace(ch, "", cs)</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_q_char_case_sensitivity(
        &mut self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        cs: crate::CaseSensitivity,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_remove1(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
            cs,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Removes every occurrence of the given <i>str</i> string in this string, and returns a reference to this string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::remove(const QString& s, Qt::CaseSensitivity cs = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#remove-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes every occurrence of the given <i>str</i> string in this string, and returns a reference to this string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This is the same as <code>replace(str, "", cs)</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_q_string_case_sensitivity(
        &mut self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        cs: crate::CaseSensitivity,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_remove2(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            cs,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Removes every occurrence of the regular expression <i>rx</i> in the string, and returns a reference to the string. For example:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::remove(const QRegExp& rx)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#remove-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes every occurrence of the regular expression <i>rx</i> in the string, and returns a reference to the string. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> r <span class="operator">=</span> <span class="string">"Telephone"</span>;
    ///   r<span class="operator">.</span>remove(<span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a></span>(<span class="string">"[aeiou]."</span>));
    ///   <span class="comment">// r == "The"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_q_reg_exp(
        &mut self,
        rx: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_remove3(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(rx).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Removes every occurrence of the regular expression <i>re</i> in the string, and returns a reference to the string. For example:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::remove(const QRegularExpression& re)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#remove-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes every occurrence of the regular expression <i>re</i> in the string, and returns a reference to the string. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> r <span class="operator">=</span> <span class="string">"Telephone"</span>;
    ///   r<span class="operator">.</span>remove(<span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a></span>(<span class="string">"[aeiou]."</span>));
    ///   <span class="comment">// r == "The"</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_q_regular_expression(
        &mut self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_remove4(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Removes every occurrence of the character <i>ch</i> in this string, and returns a reference to this string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::remove(QChar c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#remove-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes every occurrence of the character <i>ch</i> in this string, and returns a reference to this string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> t <span class="operator">=</span> <span class="string">"Ali Baba"</span>;
    ///   t<span class="operator">.</span>remove(<span class="type"><a href="http://doc.qt.io/qt-5/qchar.html">QChar</a></span>(<span class="char">'a'</span>)<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>CaseInsensitive);
    ///   <span class="comment">// t == "li Bb"</span>
    ///
    /// </pre>
    /// <p>This is the same as <code>replace(ch, "", cs)</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_q_char(
        &mut self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_remove5(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Removes every occurrence of the given <i>str</i> string in this string, and returns a reference to this string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::remove(const QString& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#remove-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes every occurrence of the given <i>str</i> string in this string, and returns a reference to this string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This is the same as <code>replace(str, "", cs)</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_q_string(
        &mut self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_remove6(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::remove(QLatin1String s, Qt::CaseSensitivity cs = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#remove-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes every occurrence of the given <i>str</i> string in this string, and returns a reference to this string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This is the same as <code>replace(str, "", cs)</code>.</p>
    /// <p>This function was introduced in  Qt 5.11.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn remove_q_latin1_string_case_sensitivity(
        &mut self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        cs: crate::CaseSensitivity,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_remove14(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
            cs,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::remove(QLatin1String s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#remove-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes every occurrence of the given <i>str</i> string in this string, and returns a reference to this string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This is the same as <code>replace(str, "", cs)</code>.</p>
    /// <p>This function was introduced in  Qt 5.11.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn remove_q_latin1_string(
        &mut self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_remove15(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a copy of this string repeated the specified number of <i>times</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::repeated(int times) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#repeated">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a copy of this string repeated the specified number of <i>times</i>.</p>
    /// <p>If <i>times</i> is less than 1, an empty string is returned.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str(<span class="string">"ab"</span>);
    ///   str<span class="operator">.</span>repeated(<span class="number">4</span>);            <span class="comment">// returns "abababab"</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn repeated(
        &self,
        times: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_repeated(self as *const crate::QString, times);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::replace(int i, int len, QChar after)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#replace-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    /// <p>Replaces <i>n</i> characters beginning at index <i>position</i> with the character <i>after</i> and returns a reference to this string.</p></div>
    pub unsafe fn replace_2_int_q_char(
        &mut self,
        i: ::std::os::raw::c_int,
        len: ::std::os::raw::c_int,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_replace(
            self as *mut crate::QString,
            i,
            len,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(after).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::replace(int i, int len, const QChar* s, int slen)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#replace-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    /// <p>Replaces <i>n</i> characters beginning at index <i>position</i> with the first <i>size</i> characters of the <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> array <i>unicode</i> and returns a reference to this string.</p></div>
    pub unsafe fn replace_2_int_q_char_int(
        &mut self,
        i: ::std::os::raw::c_int,
        len: ::std::os::raw::c_int,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QChar>>,
        slen: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_replace1(
            self as *mut crate::QString,
            i,
            len,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QChar>>::cast_into(s).as_raw_ptr(),
            slen,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Replaces <i>n</i> characters beginning at index <i>position</i> with the string <i>after</i> and returns a reference to this string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::replace(int i, int len, const QString& after)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces <i>n</i> characters beginning at index <i>position</i> with the string <i>after</i> and returns a reference to this string.</p>
    /// <p><b>Note: </b>If the specified <i>position</i> index is within the string, but <i>position</i> + <i>n</i> goes outside the strings range, then <i>n</i> will be adjusted to stop at the end of the string.</p><p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> x <span class="operator">=</span> <span class="string">"Say yes!"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> y <span class="operator">=</span> <span class="string">"no"</span>;
    ///   x<span class="operator">.</span>replace(<span class="number">4</span><span class="operator">,</span> <span class="number">3</span><span class="operator">,</span> y);
    ///   <span class="comment">// x == "Say no!"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#remove">remove</a>().</p></div>
    pub unsafe fn replace_2_int_q_string(
        &mut self,
        i: ::std::os::raw::c_int,
        len: ::std::os::raw::c_int,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_replace2(
            self as *mut crate::QString,
            i,
            len,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(after)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::replace(QChar before, QChar after, Qt::CaseSensitivity cs = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#replace-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    /// <p>Replaces every occurrence of the character <i>before</i> with the character <i>after</i> and returns a reference to this string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p></div>
    pub unsafe fn replace_2_q_char_case_sensitivity(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        cs: crate::CaseSensitivity,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_replace3(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(before).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(after).as_raw_ptr(),
            cs,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::replace(const QChar* before, int blen, const QChar* after, int alen, Qt::CaseSensitivity cs = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#replace-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    /// <p>Replaces each occurrence in this string of the first <i>blen</i> characters of <i>before</i> with the first <i>alen</i> characters of <i>after</i> and returns a reference to this string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn replace_q_char_int_q_char_int_case_sensitivity(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QChar>>,
        blen: ::std::os::raw::c_int,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QChar>>,
        alen: ::std::os::raw::c_int,
        cs: crate::CaseSensitivity,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_replace4(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QChar>>::cast_into(before).as_raw_ptr(),
            blen,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QChar>>::cast_into(after).as_raw_ptr(),
            alen,
            cs,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::replace(QLatin1String before, QLatin1String after, Qt::CaseSensitivity cs = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#replace-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    /// <p>Replaces every occurrence of the string <i>before</i> with the string <i>after</i> and returns a reference to this string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p><b>Note: </b>The text is not rescanned after a replacement.</p><p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn replace_2_q_latin1_string_case_sensitivity(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        cs: crate::CaseSensitivity,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_replace5(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(before)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(after)
                .as_raw_ptr(),
            cs,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::replace(QLatin1String before, const QString& after, Qt::CaseSensitivity cs = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#replace-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    /// <p>Replaces every occurrence of the string <i>before</i> with the string <i>after</i> and returns a reference to this string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p><b>Note: </b>The text is not rescanned after a replacement.</p><p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn replace_q_latin1_string_q_string_case_sensitivity(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        cs: crate::CaseSensitivity,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_replace6(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(before)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(after)
                .as_raw_ptr(),
            cs,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::replace(const QString& before, QLatin1String after, Qt::CaseSensitivity cs = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#replace-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    /// <p>Replaces every occurrence of the string <i>before</i> with the string <i>after</i> and returns a reference to this string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p><b>Note: </b>The text is not rescanned after a replacement.</p><p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn replace_q_string_q_latin1_string_case_sensitivity(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        cs: crate::CaseSensitivity,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_replace7(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(before)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(after)
                .as_raw_ptr(),
            cs,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::replace(const QString& before, const QString& after, Qt::CaseSensitivity cs = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#replace-8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    /// <p>Replaces every occurrence of the string <i>before</i> with the string <i>after</i> and returns a reference to this string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"colour behaviour flavour neighbour"</span>;
    ///   str<span class="operator">.</span>replace(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"ou"</span>)<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"o"</span>));
    ///   <span class="comment">// str == "color behavior flavor neighbor"</span>
    ///
    /// </pre>
    /// <p><b>Note: </b>The replacement text is not rescanned after it is inserted.</p><p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> equis <span class="operator">=</span> <span class="string">"xxxxxx"</span>;
    ///   equis<span class="operator">.</span>replace(<span class="string">"xx"</span><span class="operator">,</span> <span class="string">"x"</span>);
    ///   <span class="comment">// equis == "xxx"</span>
    ///
    /// </pre></div>
    pub unsafe fn replace_2_q_string_case_sensitivity(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        cs: crate::CaseSensitivity,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_replace8(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(before)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(after)
                .as_raw_ptr(),
            cs,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::replace(QChar c, const QString& after, Qt::CaseSensitivity cs = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#replace-9">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    /// <p>Replaces every occurrence of the character <i>ch</i> in the string with <i>after</i> and returns a reference to this string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p></div>
    pub unsafe fn replace_q_char_q_string_case_sensitivity(
        &mut self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        cs: crate::CaseSensitivity,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_replace9(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(after)
                .as_raw_ptr(),
            cs,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::replace(QChar c, QLatin1String after, Qt::CaseSensitivity cs = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#replace-10">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    /// <p>Replaces every occurrence of the character <i>c</i> with the string <i>after</i> and returns a reference to this string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p><b>Note: </b>The text is not rescanned after a replacement.</p><p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn replace_q_char_q_latin1_string_case_sensitivity(
        &mut self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        cs: crate::CaseSensitivity,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_replace10(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(after)
                .as_raw_ptr(),
            cs,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::replace(const QRegExp& rx, const QString& after)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#replace-11">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    /// <p>Replaces every occurrence of the regular expression <i>rx</i> in the string with <i>after</i>. Returns a reference to the string. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> s <span class="operator">=</span> <span class="string">"Banana"</span>;
    ///   s<span class="operator">.</span>replace(<span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a></span>(<span class="string">"a[mn]"</span>)<span class="operator">,</span> <span class="string">"ox"</span>);
    ///   <span class="comment">// s == "Boxoxa"</span>
    ///
    /// </pre>
    /// <p>For regular expressions containing <a href="http://doc.qt.io/qt-5/qregexp.html#capturing-parentheses">capturing parentheses</a>, occurrences of <b>\1</b>, <b>\2</b>, ..., in <i>after</i> are replaced with <i>rx</i>.cap(1), cap(2), ...</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> t <span class="operator">=</span> <span class="string">"A &lt;i&gt;bon mot&lt;/i&gt;."</span>;
    ///   t<span class="operator">.</span>replace(<span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a></span>(<span class="string">"&lt;i&gt;([^&lt;]*)&lt;/i&gt;"</span>)<span class="operator">,</span> <span class="string">"\\emph{\\1}"</span>);
    ///   <span class="comment">// t == "A \\emph{bon mot}."</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#remove">remove</a>(), and <a href="http://doc.qt.io/qt-5/qregexp.html#cap">QRegExp::cap</a>().</p></div>
    pub unsafe fn replace_q_reg_exp_q_string(
        &mut self,
        rx: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_replace11(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(rx).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(after)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::replace(const QRegularExpression& re, const QString& after)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#replace-12">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    /// <p>Replaces every occurrence of the regular expression <i>re</i> in the string with <i>after</i>. Returns a reference to the string. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> s <span class="operator">=</span> <span class="string">"Banana"</span>;
    ///   s<span class="operator">.</span>replace(<span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a></span>(<span class="string">"a[mn]"</span>)<span class="operator">,</span> <span class="string">"ox"</span>);
    ///   <span class="comment">// s == "Boxoxa"</span>
    ///
    /// </pre>
    /// <p>For regular expressions containing capturing groups, occurrences of <b>\1</b>, <b>\2</b>, ..., in <i>after</i> are replaced with the string captured by the corresponding capturing group.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> t <span class="operator">=</span> <span class="string">"A &lt;i&gt;bon mot&lt;/i&gt;."</span>;
    ///   t<span class="operator">.</span>replace(<span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a></span>(<span class="string">"&lt;i&gt;([^&lt;]*)&lt;/i&gt;"</span>)<span class="operator">,</span> <span class="string">"\\emph{\\1}"</span>);
    ///   <span class="comment">// t == "A \\emph{bon mot}."</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a>, and <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a>.</p></div>
    pub unsafe fn replace_q_regular_expression_q_string(
        &mut self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_replace12(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(after)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::replace(QChar before, QChar after)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#replace-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    /// <p>Replaces every occurrence of the character <i>before</i> with the character <i>after</i> and returns a reference to this string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p></div>
    pub unsafe fn replace_2_q_char(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_replace13(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(before).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(after).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::replace(const QChar* before, int blen, const QChar* after, int alen)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#replace-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    /// <p>Replaces each occurrence in this string of the first <i>blen</i> characters of <i>before</i> with the first <i>alen</i> characters of <i>after</i> and returns a reference to this string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn replace_q_char_int_q_char_int(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QChar>>,
        blen: ::std::os::raw::c_int,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QChar>>,
        alen: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_replace14(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QChar>>::cast_into(before).as_raw_ptr(),
            blen,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QChar>>::cast_into(after).as_raw_ptr(),
            alen,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::replace(QLatin1String before, QLatin1String after)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#replace-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    /// <p>Replaces every occurrence of the string <i>before</i> with the string <i>after</i> and returns a reference to this string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p><b>Note: </b>The text is not rescanned after a replacement.</p><p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn replace_2_q_latin1_string(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_replace15(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(before)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(after)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::replace(QLatin1String before, const QString& after)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#replace-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    /// <p>Replaces every occurrence of the string <i>before</i> with the string <i>after</i> and returns a reference to this string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p><b>Note: </b>The text is not rescanned after a replacement.</p><p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn replace_q_latin1_string_q_string(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_replace16(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(before)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(after)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::replace(const QString& before, QLatin1String after)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#replace-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    /// <p>Replaces every occurrence of the string <i>before</i> with the string <i>after</i> and returns a reference to this string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p><b>Note: </b>The text is not rescanned after a replacement.</p><p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn replace_q_string_q_latin1_string(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_replace17(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(before)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(after)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::replace(const QString& before, const QString& after)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#replace-8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    /// <p>Replaces every occurrence of the string <i>before</i> with the string <i>after</i> and returns a reference to this string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"colour behaviour flavour neighbour"</span>;
    ///   str<span class="operator">.</span>replace(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"ou"</span>)<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>(<span class="string">"o"</span>));
    ///   <span class="comment">// str == "color behavior flavor neighbor"</span>
    ///
    /// </pre>
    /// <p><b>Note: </b>The replacement text is not rescanned after it is inserted.</p><p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> equis <span class="operator">=</span> <span class="string">"xxxxxx"</span>;
    ///   equis<span class="operator">.</span>replace(<span class="string">"xx"</span><span class="operator">,</span> <span class="string">"x"</span>);
    ///   <span class="comment">// equis == "xxx"</span>
    ///
    /// </pre></div>
    pub unsafe fn replace_2_q_string(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_replace18(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(before)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(after)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::replace(QChar c, const QString& after)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#replace-9">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    /// <p>Replaces every occurrence of the character <i>ch</i> in the string with <i>after</i> and returns a reference to this string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p></div>
    pub unsafe fn replace_q_char_q_string(
        &mut self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_replace19(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(after)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::replace(QChar c, QLatin1String after)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#replace-10">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace</a>().</p>
    /// <p>Replaces every occurrence of the character <i>c</i> with the string <i>after</i> and returns a reference to this string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p><b>Note: </b>The text is not rescanned after a replacement.</p><p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn replace_q_char_q_latin1_string(
        &mut self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        after: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_replace20(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(after)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Attempts to allocate memory for at least <i>size</i> characters. If you know in advance how large the string will be, you can call this function, and if you resize the string often you are likely to get better performance. If <i>size</i> is an underestimate, the worst that will happen is that the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> will be a bit slower.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QString::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to allocate memory for at least <i>size</i> characters. If you know in advance how large the string will be, you can call this function, and if you resize the string often you are likely to get better performance. If <i>size</i> is an underestimate, the worst that will happen is that the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> will be a bit slower.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>'s memory usage. In general, you will rarely ever need to call this function. If you want to change the size of the string, call <a href="http://doc.qt.io/qt-5/qstring.html#resize">resize</a>().</p>
    /// <p>This function is useful for code that needs to build up a long string and wants to avoid repeated reallocation. In this example, we want to add to the string until some condition is <code>true</code>, and we're fairly sure that size is large enough to make a call to reserve() worthwhile:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> result;
    ///   <span class="type">int</span> maxSize;
    ///   bool condition;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qchar.html">QChar</a></span> nextChar;
    ///
    ///   result<span class="operator">.</span>reserve(maxSize);
    ///
    ///   <span class="keyword">while</span> (condition)
    /// &#32;     result<span class="operator">.</span>append(nextChar);
    ///
    ///   result<span class="operator">.</span>squeeze();
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#squeeze">squeeze</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#capacity">capacity</a>().</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QString_reserve(self as *mut crate::QString, size)
    }

    /// <p>Sets the size of the string to <i>size</i> characters.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QString::resize(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#resize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the size of the string to <i>size</i> characters.</p>
    /// <p>If <i>size</i> is greater than the current size, the string is extended to make it <i>size</i> characters long with the extra characters added to the end. The new characters are uninitialized.</p>
    /// <p>If <i>size</i> is less than the current size, characters are removed from the end.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> s <span class="operator">=</span> <span class="string">"Hello world"</span>;
    ///   s<span class="operator">.</span>resize(<span class="number">5</span>);
    ///   <span class="comment">// s == "Hello"</span>
    ///
    ///   s<span class="operator">.</span>resize(<span class="number">8</span>);
    ///   <span class="comment">// s == "Hello???" (where ? stands for any character)</span>
    ///
    /// </pre>
    /// <p>If you want to append a certain number of identical characters to the string, use the <a href="http://doc.qt.io/qt-5/qstring.html#resize-1">resize</a>(int, <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>) overload.</p>
    /// <p>If you want to expand the string so that it reaches a certain width and fill the new positions with a particular character, use the <a href="http://doc.qt.io/qt-5/qstring.html#leftJustified">leftJustified</a>() function:</p>
    /// <p>If <i>size</i> is negative, it is equivalent to passing zero.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> r <span class="operator">=</span> <span class="string">"Hello"</span>;
    ///   r <span class="operator">=</span> r<span class="operator">.</span>leftJustified(<span class="number">10</span><span class="operator">,</span> <span class="char">' '</span>);
    ///   <span class="comment">// r == "Hello     "</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#truncate">truncate</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#reserve">reserve</a>().</p></div>
    pub unsafe fn resize_1a(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QString_resize(self as *mut crate::QString, size)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QString::resize(int size, QChar fillChar)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#resize-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qstring.html#resize">resize</a>(int), this overload initializes the new characters to <i>fillChar</i>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> t <span class="operator">=</span> <span class="string">"Hello"</span>;
    ///   r<span class="operator">.</span>resize(t<span class="operator">.</span>size() <span class="operator">+</span> <span class="number">10</span><span class="operator">,</span> <span class="char">'X'</span>);
    ///   <span class="comment">// t == "HelloXXXXXXXXXX"</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.7.</p></div>
    pub unsafe fn resize_2a(
        &mut self,
        size: ::std::os::raw::c_int,
        fill_char: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QString_resize1(
            self as *mut crate::QString,
            size,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(fill_char)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a substring that contains the <i>n</i> rightmost characters of the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::right(int n) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#right">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a substring that contains the <i>n</i> rightmost characters of the string.</p>
    /// <p>The entire string is returned if <i>n</i> is greater than or equal to <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>(), or less than zero.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> x <span class="operator">=</span> <span class="string">"Pineapple"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> y <span class="operator">=</span> x<span class="operator">.</span>right(<span class="number">5</span>);      <span class="comment">// y == "apple"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#left">left</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#mid">mid</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#endsWith">endsWith</a>().</p></div>
    pub unsafe fn right(&self, n: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_right(self as *const crate::QString, n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a string of <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>() <i>width</i> that contains the <i>fill</i> character followed by the string. For example:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::rightJustified(int width, QChar fill = …, bool trunc = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#rightJustified">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string of <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>() <i>width</i> that contains the <i>fill</i> character followed by the string. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> s <span class="operator">=</span> <span class="string">"apple"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> t <span class="operator">=</span> s<span class="operator">.</span>rightJustified(<span class="number">8</span><span class="operator">,</span> <span class="char">'.'</span>);    <span class="comment">// t == "...apple"</span>
    ///
    /// </pre>
    /// <p>If <i>truncate</i> is <code>false</code> and the <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>() of the string is more than <i>width</i>, then the returned string is a copy of the string.</p>
    /// <p>If <i>truncate</i> is true and the <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>() of the string is more than <i>width</i>, then the resulting string is truncated at position <i>width</i>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"Pineapple"</span>;
    ///   str <span class="operator">=</span> str<span class="operator">.</span>rightJustified(<span class="number">5</span><span class="operator">,</span> <span class="char">'.'</span><span class="operator">,</span> <span class="keyword">true</span>);    <span class="comment">// str == "Pinea"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#leftJustified">leftJustified</a>().</p></div>
    pub unsafe fn right_justified_3a(
        &self,
        width: ::std::os::raw::c_int,
        fill: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        trunc: bool,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_rightJustified(
            self as *const crate::QString,
            width,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(fill).as_raw_ptr(),
            trunc,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a string of <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>() <i>width</i> that contains the <i>fill</i> character followed by the string. For example:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::rightJustified(int width, QChar fill = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#rightJustified">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string of <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>() <i>width</i> that contains the <i>fill</i> character followed by the string. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> s <span class="operator">=</span> <span class="string">"apple"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> t <span class="operator">=</span> s<span class="operator">.</span>rightJustified(<span class="number">8</span><span class="operator">,</span> <span class="char">'.'</span>);    <span class="comment">// t == "...apple"</span>
    ///
    /// </pre>
    /// <p>If <i>truncate</i> is <code>false</code> and the <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>() of the string is more than <i>width</i>, then the returned string is a copy of the string.</p>
    /// <p>If <i>truncate</i> is true and the <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>() of the string is more than <i>width</i>, then the resulting string is truncated at position <i>width</i>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"Pineapple"</span>;
    ///   str <span class="operator">=</span> str<span class="operator">.</span>rightJustified(<span class="number">5</span><span class="operator">,</span> <span class="char">'.'</span><span class="operator">,</span> <span class="keyword">true</span>);    <span class="comment">// str == "Pinea"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#leftJustified">leftJustified</a>().</p></div>
    pub unsafe fn right_justified_2a(
        &self,
        width: ::std::os::raw::c_int,
        fill: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_rightJustified1(
            self as *const crate::QString,
            width,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(fill).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a string of <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>() <i>width</i> that contains the <i>fill</i> character followed by the string. For example:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::rightJustified(int width) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#rightJustified">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string of <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>() <i>width</i> that contains the <i>fill</i> character followed by the string. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> s <span class="operator">=</span> <span class="string">"apple"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> t <span class="operator">=</span> s<span class="operator">.</span>rightJustified(<span class="number">8</span><span class="operator">,</span> <span class="char">'.'</span>);    <span class="comment">// t == "...apple"</span>
    ///
    /// </pre>
    /// <p>If <i>truncate</i> is <code>false</code> and the <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>() of the string is more than <i>width</i>, then the returned string is a copy of the string.</p>
    /// <p>If <i>truncate</i> is true and the <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>() of the string is more than <i>width</i>, then the resulting string is truncated at position <i>width</i>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"Pineapple"</span>;
    ///   str <span class="operator">=</span> str<span class="operator">.</span>rightJustified(<span class="number">5</span><span class="operator">,</span> <span class="char">'.'</span><span class="operator">,</span> <span class="keyword">true</span>);    <span class="comment">// str == "Pinea"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#leftJustified">leftJustified</a>().</p></div>
    pub unsafe fn right_justified_1a(
        &self,
        width: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_rightJustified2(
            self as *const crate::QString,
            width,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a substring reference to the <i>n</i> rightmost characters of the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QString::rightRef(int n) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#rightRef">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a substring reference to the <i>n</i> rightmost characters of the string.</p>
    /// <p>If <i>n</i> is greater than or equal to <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>(), or less than zero, a reference to the entire string is returned.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> x <span class="operator">=</span> <span class="string">"Pineapple"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a></span> y <span class="operator">=</span> x<span class="operator">.</span>rightRef(<span class="number">5</span>);       <span class="comment">// y == "apple"</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#right">right</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#leftRef">leftRef</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#midRef">midRef</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#endsWith">endsWith</a>().</p></div>
    pub unsafe fn right_ref(
        &self,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_rightRef(self as *const crate::QString, n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function returns a section of the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::section(QChar sep, int start, int end = …, QFlags<QString::SectionFlag> flags = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#section">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function returns a section of the string.</p>
    /// <p>This string is treated as a sequence of fields separated by the character, <i>sep</i>. The returned string consists of the fields from position <i>start</i> to position <i>end</i> inclusive. If <i>end</i> is not specified, all fields from position <i>start</i> to the end of the string are included. Fields are numbered 0, 1, 2, etc., counting from the left, and -1, -2, etc., counting from right to left.</p>
    /// <p>The <i>flags</i> argument can be used to affect some aspects of the function's behavior, e.g. whether to be case sensitive, whether to skip empty fields and how to deal with leading and trailing separators; see <a href="http://doc.qt.io/qt-5/qstring.html#SectionFlag-enum">SectionFlags</a>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> csv <span class="operator">=</span> <span class="string">"forename,middlename,surname,phone"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> path <span class="operator">=</span> <span class="string">"/usr/local/bin/myapp"</span>; <span class="comment">// First field is empty</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span><span class="operator">::</span>SectionFlag flag <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span><span class="operator">::</span>SectionSkipEmpty;
    ///
    ///   str <span class="operator">=</span> csv<span class="operator">.</span>section(<span class="char">','</span><span class="operator">,</span> <span class="number">2</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// str == "surname"</span>
    ///   str <span class="operator">=</span> path<span class="operator">.</span>section(<span class="char">'/'</span><span class="operator">,</span> <span class="number">3</span><span class="operator">,</span> <span class="number">4</span>);  <span class="comment">// str == "bin/myapp"</span>
    ///   str <span class="operator">=</span> path<span class="operator">.</span>section(<span class="char">'/'</span><span class="operator">,</span> <span class="number">3</span><span class="operator">,</span> <span class="number">3</span><span class="operator">,</span> flag); <span class="comment">// str == "myapp"</span>
    ///
    /// </pre>
    /// <p>If <i>start</i> or <i>end</i> is negative, we count fields from the right of the string, the right-most field being -1, the one from right-most field being -2, and so on.</p>
    /// <pre class="cpp">
    ///
    ///   str <span class="operator">=</span> csv<span class="operator">.</span>section(<span class="char">','</span><span class="operator">,</span> <span class="operator">-</span><span class="number">3</span><span class="operator">,</span> <span class="operator">-</span><span class="number">2</span>);  <span class="comment">// str == "middlename,surname"</span>
    ///   str <span class="operator">=</span> path<span class="operator">.</span>section(<span class="char">'/'</span><span class="operator">,</span> <span class="operator">-</span><span class="number">1</span>); <span class="comment">// str == "myapp"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#split">split</a>().</p></div>
    pub unsafe fn section_q_char2_int_q_flags_section_flag(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        start: ::std::os::raw::c_int,
        end: ::std::os::raw::c_int,
        flags: crate::QFlags<crate::q_string::SectionFlag>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_section(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(sep).as_raw_ptr(),
            start,
            end,
            flags.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::section(const QString& in_sep, int start, int end = …, QFlags<QString::SectionFlag> flags = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#section-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>().</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> data <span class="operator">=</span> <span class="string">"forename**middlename**surname**phone"</span>;
    ///
    ///   str <span class="operator">=</span> data<span class="operator">.</span>section(<span class="string">"**"</span><span class="operator">,</span> <span class="number">2</span><span class="operator">,</span> <span class="number">2</span>); <span class="comment">// str == "surname"</span>
    ///   str <span class="operator">=</span> data<span class="operator">.</span>section(<span class="string">"**"</span><span class="operator">,</span> <span class="operator">-</span><span class="number">3</span><span class="operator">,</span> <span class="operator">-</span><span class="number">2</span>); <span class="comment">// str == "middlename**surname"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#split">split</a>().</p></div>
    pub unsafe fn section_q_string2_int_q_flags_section_flag(
        &self,
        in_sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        start: ::std::os::raw::c_int,
        end: ::std::os::raw::c_int,
        flags: crate::QFlags<crate::q_string::SectionFlag>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_section1(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(in_sep)
                .as_raw_ptr(),
            start,
            end,
            flags.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::section(const QRegExp& reg, int start, int end = …, QFlags<QString::SectionFlag> flags = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#section-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>().</p>
    /// <p>This string is treated as a sequence of fields separated by the regular expression, <i>reg</i>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> line <span class="operator">=</span> <span class="string">"forename\tmiddlename  surname \t \t phone"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a></span> sep(<span class="string">"\\s+"</span>);
    ///   str <span class="operator">=</span> line<span class="operator">.</span>section(sep<span class="operator">,</span> <span class="number">2</span><span class="operator">,</span> <span class="number">2</span>); <span class="comment">// str == "surname"</span>
    ///   str <span class="operator">=</span> line<span class="operator">.</span>section(sep<span class="operator">,</span> <span class="operator">-</span><span class="number">3</span><span class="operator">,</span> <span class="operator">-</span><span class="number">2</span>); <span class="comment">// str == "middlename  surname"</span>
    ///
    /// </pre>
    /// <p><b>Warning:</b> Using this <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> version is much more expensive than the overloaded string and character versions.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#split">split</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#simplified">simplified</a>().</p></div>
    pub unsafe fn section_q_reg_exp2_int_q_flags_section_flag(
        &self,
        reg: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
        start: ::std::os::raw::c_int,
        end: ::std::os::raw::c_int,
        flags: crate::QFlags<crate::q_string::SectionFlag>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_section2(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(reg).as_raw_ptr(),
            start,
            end,
            flags.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::section(const QRegularExpression& re, int start, int end = …, QFlags<QString::SectionFlag> flags = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#section-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>().</p>
    /// <p>This string is treated as a sequence of fields separated by the regular expression, <i>re</i>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> line <span class="operator">=</span> <span class="string">"forename\tmiddlename  surname \t \t phone"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a></span> sep(<span class="string">"\\s+"</span>);
    ///   str <span class="operator">=</span> line<span class="operator">.</span>section(sep<span class="operator">,</span> <span class="number">2</span><span class="operator">,</span> <span class="number">2</span>); <span class="comment">// str == "surname"</span>
    ///   str <span class="operator">=</span> line<span class="operator">.</span>section(sep<span class="operator">,</span> <span class="operator">-</span><span class="number">3</span><span class="operator">,</span> <span class="operator">-</span><span class="number">2</span>); <span class="comment">// str == "middlename  surname"</span>
    ///
    /// </pre>
    /// <p><b>Warning:</b> Using this <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> version is much more expensive than the overloaded string and character versions.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#split">split</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#simplified">simplified</a>().</p></div>
    pub unsafe fn section_q_regular_expression2_int_q_flags_section_flag(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
        start: ::std::os::raw::c_int,
        end: ::std::os::raw::c_int,
        flags: crate::QFlags<crate::q_string::SectionFlag>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_section3(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
            start,
            end,
            flags.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function returns a section of the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::section(QChar sep, int start, int end = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#section">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function returns a section of the string.</p>
    /// <p>This string is treated as a sequence of fields separated by the character, <i>sep</i>. The returned string consists of the fields from position <i>start</i> to position <i>end</i> inclusive. If <i>end</i> is not specified, all fields from position <i>start</i> to the end of the string are included. Fields are numbered 0, 1, 2, etc., counting from the left, and -1, -2, etc., counting from right to left.</p>
    /// <p>The <i>flags</i> argument can be used to affect some aspects of the function's behavior, e.g. whether to be case sensitive, whether to skip empty fields and how to deal with leading and trailing separators; see <a href="http://doc.qt.io/qt-5/qstring.html#SectionFlag-enum">SectionFlags</a>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> csv <span class="operator">=</span> <span class="string">"forename,middlename,surname,phone"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> path <span class="operator">=</span> <span class="string">"/usr/local/bin/myapp"</span>; <span class="comment">// First field is empty</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span><span class="operator">::</span>SectionFlag flag <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span><span class="operator">::</span>SectionSkipEmpty;
    ///
    ///   str <span class="operator">=</span> csv<span class="operator">.</span>section(<span class="char">','</span><span class="operator">,</span> <span class="number">2</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// str == "surname"</span>
    ///   str <span class="operator">=</span> path<span class="operator">.</span>section(<span class="char">'/'</span><span class="operator">,</span> <span class="number">3</span><span class="operator">,</span> <span class="number">4</span>);  <span class="comment">// str == "bin/myapp"</span>
    ///   str <span class="operator">=</span> path<span class="operator">.</span>section(<span class="char">'/'</span><span class="operator">,</span> <span class="number">3</span><span class="operator">,</span> <span class="number">3</span><span class="operator">,</span> flag); <span class="comment">// str == "myapp"</span>
    ///
    /// </pre>
    /// <p>If <i>start</i> or <i>end</i> is negative, we count fields from the right of the string, the right-most field being -1, the one from right-most field being -2, and so on.</p>
    /// <pre class="cpp">
    ///
    ///   str <span class="operator">=</span> csv<span class="operator">.</span>section(<span class="char">','</span><span class="operator">,</span> <span class="operator">-</span><span class="number">3</span><span class="operator">,</span> <span class="operator">-</span><span class="number">2</span>);  <span class="comment">// str == "middlename,surname"</span>
    ///   str <span class="operator">=</span> path<span class="operator">.</span>section(<span class="char">'/'</span><span class="operator">,</span> <span class="operator">-</span><span class="number">1</span>); <span class="comment">// str == "myapp"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#split">split</a>().</p></div>
    pub unsafe fn section_q_char2_int(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        start: ::std::os::raw::c_int,
        end: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_section4(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(sep).as_raw_ptr(),
            start,
            end,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function returns a section of the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::section(QChar sep, int start) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#section">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function returns a section of the string.</p>
    /// <p>This string is treated as a sequence of fields separated by the character, <i>sep</i>. The returned string consists of the fields from position <i>start</i> to position <i>end</i> inclusive. If <i>end</i> is not specified, all fields from position <i>start</i> to the end of the string are included. Fields are numbered 0, 1, 2, etc., counting from the left, and -1, -2, etc., counting from right to left.</p>
    /// <p>The <i>flags</i> argument can be used to affect some aspects of the function's behavior, e.g. whether to be case sensitive, whether to skip empty fields and how to deal with leading and trailing separators; see <a href="http://doc.qt.io/qt-5/qstring.html#SectionFlag-enum">SectionFlags</a>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> csv <span class="operator">=</span> <span class="string">"forename,middlename,surname,phone"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> path <span class="operator">=</span> <span class="string">"/usr/local/bin/myapp"</span>; <span class="comment">// First field is empty</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span><span class="operator">::</span>SectionFlag flag <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span><span class="operator">::</span>SectionSkipEmpty;
    ///
    ///   str <span class="operator">=</span> csv<span class="operator">.</span>section(<span class="char">','</span><span class="operator">,</span> <span class="number">2</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// str == "surname"</span>
    ///   str <span class="operator">=</span> path<span class="operator">.</span>section(<span class="char">'/'</span><span class="operator">,</span> <span class="number">3</span><span class="operator">,</span> <span class="number">4</span>);  <span class="comment">// str == "bin/myapp"</span>
    ///   str <span class="operator">=</span> path<span class="operator">.</span>section(<span class="char">'/'</span><span class="operator">,</span> <span class="number">3</span><span class="operator">,</span> <span class="number">3</span><span class="operator">,</span> flag); <span class="comment">// str == "myapp"</span>
    ///
    /// </pre>
    /// <p>If <i>start</i> or <i>end</i> is negative, we count fields from the right of the string, the right-most field being -1, the one from right-most field being -2, and so on.</p>
    /// <pre class="cpp">
    ///
    ///   str <span class="operator">=</span> csv<span class="operator">.</span>section(<span class="char">','</span><span class="operator">,</span> <span class="operator">-</span><span class="number">3</span><span class="operator">,</span> <span class="operator">-</span><span class="number">2</span>);  <span class="comment">// str == "middlename,surname"</span>
    ///   str <span class="operator">=</span> path<span class="operator">.</span>section(<span class="char">'/'</span><span class="operator">,</span> <span class="operator">-</span><span class="number">1</span>); <span class="comment">// str == "myapp"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#split">split</a>().</p></div>
    pub unsafe fn section_q_char_int(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        start: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_section5(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(sep).as_raw_ptr(),
            start,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::section(const QString& in_sep, int start, int end = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#section-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>().</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> data <span class="operator">=</span> <span class="string">"forename**middlename**surname**phone"</span>;
    ///
    ///   str <span class="operator">=</span> data<span class="operator">.</span>section(<span class="string">"**"</span><span class="operator">,</span> <span class="number">2</span><span class="operator">,</span> <span class="number">2</span>); <span class="comment">// str == "surname"</span>
    ///   str <span class="operator">=</span> data<span class="operator">.</span>section(<span class="string">"**"</span><span class="operator">,</span> <span class="operator">-</span><span class="number">3</span><span class="operator">,</span> <span class="operator">-</span><span class="number">2</span>); <span class="comment">// str == "middlename**surname"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#split">split</a>().</p></div>
    pub unsafe fn section_q_string2_int(
        &self,
        in_sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        start: ::std::os::raw::c_int,
        end: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_section6(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(in_sep)
                .as_raw_ptr(),
            start,
            end,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::section(const QString& in_sep, int start) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#section-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>().</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> data <span class="operator">=</span> <span class="string">"forename**middlename**surname**phone"</span>;
    ///
    ///   str <span class="operator">=</span> data<span class="operator">.</span>section(<span class="string">"**"</span><span class="operator">,</span> <span class="number">2</span><span class="operator">,</span> <span class="number">2</span>); <span class="comment">// str == "surname"</span>
    ///   str <span class="operator">=</span> data<span class="operator">.</span>section(<span class="string">"**"</span><span class="operator">,</span> <span class="operator">-</span><span class="number">3</span><span class="operator">,</span> <span class="operator">-</span><span class="number">2</span>); <span class="comment">// str == "middlename**surname"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#split">split</a>().</p></div>
    pub unsafe fn section_q_string_int(
        &self,
        in_sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        start: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_section7(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(in_sep)
                .as_raw_ptr(),
            start,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::section(const QRegExp& reg, int start, int end = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#section-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>().</p>
    /// <p>This string is treated as a sequence of fields separated by the regular expression, <i>reg</i>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> line <span class="operator">=</span> <span class="string">"forename\tmiddlename  surname \t \t phone"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a></span> sep(<span class="string">"\\s+"</span>);
    ///   str <span class="operator">=</span> line<span class="operator">.</span>section(sep<span class="operator">,</span> <span class="number">2</span><span class="operator">,</span> <span class="number">2</span>); <span class="comment">// str == "surname"</span>
    ///   str <span class="operator">=</span> line<span class="operator">.</span>section(sep<span class="operator">,</span> <span class="operator">-</span><span class="number">3</span><span class="operator">,</span> <span class="operator">-</span><span class="number">2</span>); <span class="comment">// str == "middlename  surname"</span>
    ///
    /// </pre>
    /// <p><b>Warning:</b> Using this <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> version is much more expensive than the overloaded string and character versions.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#split">split</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#simplified">simplified</a>().</p></div>
    pub unsafe fn section_q_reg_exp2_int(
        &self,
        reg: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
        start: ::std::os::raw::c_int,
        end: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_section8(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(reg).as_raw_ptr(),
            start,
            end,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::section(const QRegExp& reg, int start) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#section-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>().</p>
    /// <p>This string is treated as a sequence of fields separated by the regular expression, <i>reg</i>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> line <span class="operator">=</span> <span class="string">"forename\tmiddlename  surname \t \t phone"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a></span> sep(<span class="string">"\\s+"</span>);
    ///   str <span class="operator">=</span> line<span class="operator">.</span>section(sep<span class="operator">,</span> <span class="number">2</span><span class="operator">,</span> <span class="number">2</span>); <span class="comment">// str == "surname"</span>
    ///   str <span class="operator">=</span> line<span class="operator">.</span>section(sep<span class="operator">,</span> <span class="operator">-</span><span class="number">3</span><span class="operator">,</span> <span class="operator">-</span><span class="number">2</span>); <span class="comment">// str == "middlename  surname"</span>
    ///
    /// </pre>
    /// <p><b>Warning:</b> Using this <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> version is much more expensive than the overloaded string and character versions.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#split">split</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#simplified">simplified</a>().</p></div>
    pub unsafe fn section_q_reg_exp_int(
        &self,
        reg: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
        start: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_section9(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(reg).as_raw_ptr(),
            start,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::section(const QRegularExpression& re, int start, int end = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#section-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>().</p>
    /// <p>This string is treated as a sequence of fields separated by the regular expression, <i>re</i>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> line <span class="operator">=</span> <span class="string">"forename\tmiddlename  surname \t \t phone"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a></span> sep(<span class="string">"\\s+"</span>);
    ///   str <span class="operator">=</span> line<span class="operator">.</span>section(sep<span class="operator">,</span> <span class="number">2</span><span class="operator">,</span> <span class="number">2</span>); <span class="comment">// str == "surname"</span>
    ///   str <span class="operator">=</span> line<span class="operator">.</span>section(sep<span class="operator">,</span> <span class="operator">-</span><span class="number">3</span><span class="operator">,</span> <span class="operator">-</span><span class="number">2</span>); <span class="comment">// str == "middlename  surname"</span>
    ///
    /// </pre>
    /// <p><b>Warning:</b> Using this <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> version is much more expensive than the overloaded string and character versions.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#split">split</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#simplified">simplified</a>().</p></div>
    pub unsafe fn section_q_regular_expression2_int(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
        start: ::std::os::raw::c_int,
        end: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_section10(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
            start,
            end,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::section(const QRegularExpression& re, int start) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#section-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>().</p>
    /// <p>This string is treated as a sequence of fields separated by the regular expression, <i>re</i>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> line <span class="operator">=</span> <span class="string">"forename\tmiddlename  surname \t \t phone"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a></span> sep(<span class="string">"\\s+"</span>);
    ///   str <span class="operator">=</span> line<span class="operator">.</span>section(sep<span class="operator">,</span> <span class="number">2</span><span class="operator">,</span> <span class="number">2</span>); <span class="comment">// str == "surname"</span>
    ///   str <span class="operator">=</span> line<span class="operator">.</span>section(sep<span class="operator">,</span> <span class="operator">-</span><span class="number">3</span><span class="operator">,</span> <span class="operator">-</span><span class="number">2</span>); <span class="comment">// str == "middlename  surname"</span>
    ///
    /// </pre>
    /// <p><b>Warning:</b> Using this <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> version is much more expensive than the overloaded string and character versions.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#split">split</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#simplified">simplified</a>().</p></div>
    pub unsafe fn section_q_regular_expression_int(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
        start: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_section11(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
            start,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::setNum(short arg1, int base = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#setNum-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn set_num_short_int(
        &mut self,
        arg1: ::std::os::raw::c_short,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_setNum(self as *mut crate::QString, arg1, base);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::setNum(unsigned short arg1, int base = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#setNum-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn set_num_ushort_int(
        &mut self,
        arg1: ::std::os::raw::c_ushort,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_setNum1(self as *mut crate::QString, arg1, base);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Sets the string to the printed value of <i>n</i> in the specified <i>base</i>, and returns a reference to the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::setNum(int arg1, int base = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#setNum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the string to the printed value of <i>n</i> in the specified <i>base</i>, and returns a reference to the string.</p>
    /// <p>The base is 10 by default and must be between 2 and 36. For bases other than 10, <i>n</i> is treated as an unsigned integer.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str;
    ///   str<span class="operator">.</span>setNum(<span class="number">1234</span>);       <span class="comment">// str == "1234"</span>
    ///
    /// </pre>
    /// <p>The formatting always uses <a href="http://doc.qt.io/qt-5/qlocale.html#Language-enum">QLocale::C</a>, i.e., English/UnitedStates. To get a localized string representation of a number, use <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>() with the appropriate locale.</p></div>
    pub unsafe fn set_num_2_int(
        &mut self,
        arg1: ::std::os::raw::c_int,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_setNum2(self as *mut crate::QString, arg1, base);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::setNum(unsigned int arg1, int base = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#setNum-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn set_num_uint_int(
        &mut self,
        arg1: ::std::os::raw::c_uint,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_setNum3(self as *mut crate::QString, arg1, base);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::setNum(long arg1, int base = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#setNum-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn set_num_long_int(
        &mut self,
        arg1: ::std::os::raw::c_long,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_setNum4(self as *mut crate::QString, arg1, base);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::setNum(unsigned long arg1, int base = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#setNum-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn set_num_ulong_int(
        &mut self,
        arg1: ::std::os::raw::c_ulong,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_setNum5(self as *mut crate::QString, arg1, base);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::setNum(qlonglong arg1, int base = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#setNum-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn set_num_i64_int(
        &mut self,
        arg1: i64,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_setNum6(self as *mut crate::QString, arg1, base);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::setNum(qulonglong arg1, int base = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#setNum-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn set_num_u64_int(
        &mut self,
        arg1: u64,
        base: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_setNum7(self as *mut crate::QString, arg1, base);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::setNum(float arg1, char f = …, int prec = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#setNum-8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Sets the string to the printed value of <i>n</i>, formatted according to the given <i>format</i> and <i>precision</i>, and returns a reference to the string.</p>
    /// <p>The formatting always uses <a href="http://doc.qt.io/qt-5/qlocale.html#Language-enum">QLocale::C</a>, i.e., English/UnitedStates. To get a localized string representation of a number, use <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>() with the appropriate locale.</p></div>
    pub unsafe fn set_num_float_char_int(
        &mut self,
        arg1: ::std::os::raw::c_float,
        f: ::std::os::raw::c_char,
        prec: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_setNum8(
            self as *mut crate::QString,
            arg1,
            f,
            prec,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::setNum(double arg1, char f = …, int prec = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#setNum-9">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Sets the string to the printed value of <i>n</i>, formatted according to the given <i>format</i> and <i>precision</i>, and returns a reference to the string.</p>
    /// <p>The <i>format</i> can be 'e', 'E', 'f', 'g' or 'G' (see <a href="http://doc.qt.io/qt-5/qstring.html#argument-formats">Argument Formats</a> for an explanation of the formats).</p>
    /// <p>The formatting always uses <a href="http://doc.qt.io/qt-5/qlocale.html#Language-enum">QLocale::C</a>, i.e., English/UnitedStates. To get a localized string representation of a number, use <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>() with the appropriate locale.</p></div>
    pub unsafe fn set_num_double_char_int(
        &mut self,
        arg1: ::std::os::raw::c_double,
        f: ::std::os::raw::c_char,
        prec: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_setNum9(
            self as *mut crate::QString,
            arg1,
            f,
            prec,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::setNum(short arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#setNum-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn set_num_short(
        &mut self,
        arg1: ::std::os::raw::c_short,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_setNum10(self as *mut crate::QString, arg1);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::setNum(unsigned short arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#setNum-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn set_num_ushort(
        &mut self,
        arg1: ::std::os::raw::c_ushort,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_setNum11(self as *mut crate::QString, arg1);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Sets the string to the printed value of <i>n</i> in the specified <i>base</i>, and returns a reference to the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::setNum(int arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#setNum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the string to the printed value of <i>n</i> in the specified <i>base</i>, and returns a reference to the string.</p>
    /// <p>The base is 10 by default and must be between 2 and 36. For bases other than 10, <i>n</i> is treated as an unsigned integer.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str;
    ///   str<span class="operator">.</span>setNum(<span class="number">1234</span>);       <span class="comment">// str == "1234"</span>
    ///
    /// </pre>
    /// <p>The formatting always uses <a href="http://doc.qt.io/qt-5/qlocale.html#Language-enum">QLocale::C</a>, i.e., English/UnitedStates. To get a localized string representation of a number, use <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>() with the appropriate locale.</p></div>
    pub unsafe fn set_num_int(
        &mut self,
        arg1: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_setNum12(self as *mut crate::QString, arg1);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::setNum(unsigned int arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#setNum-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn set_num_uint(
        &mut self,
        arg1: ::std::os::raw::c_uint,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_setNum13(self as *mut crate::QString, arg1);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::setNum(long arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#setNum-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn set_num_long(
        &mut self,
        arg1: ::std::os::raw::c_long,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_setNum14(self as *mut crate::QString, arg1);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::setNum(unsigned long arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#setNum-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn set_num_ulong(
        &mut self,
        arg1: ::std::os::raw::c_ulong,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_setNum15(self as *mut crate::QString, arg1);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::setNum(qlonglong arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#setNum-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn set_num_i64(&mut self, arg1: i64) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_setNum16(self as *mut crate::QString, arg1);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::setNum(qulonglong arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#setNum-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn set_num_u64(&mut self, arg1: u64) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_setNum17(self as *mut crate::QString, arg1);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::setNum(float arg1, char f = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#setNum-8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Sets the string to the printed value of <i>n</i>, formatted according to the given <i>format</i> and <i>precision</i>, and returns a reference to the string.</p>
    /// <p>The formatting always uses <a href="http://doc.qt.io/qt-5/qlocale.html#Language-enum">QLocale::C</a>, i.e., English/UnitedStates. To get a localized string representation of a number, use <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>() with the appropriate locale.</p></div>
    pub unsafe fn set_num_float_char(
        &mut self,
        arg1: ::std::os::raw::c_float,
        f: ::std::os::raw::c_char,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_setNum18(self as *mut crate::QString, arg1, f);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::setNum(float arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#setNum-8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Sets the string to the printed value of <i>n</i>, formatted according to the given <i>format</i> and <i>precision</i>, and returns a reference to the string.</p>
    /// <p>The formatting always uses <a href="http://doc.qt.io/qt-5/qlocale.html#Language-enum">QLocale::C</a>, i.e., English/UnitedStates. To get a localized string representation of a number, use <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>() with the appropriate locale.</p></div>
    pub unsafe fn set_num_float(
        &mut self,
        arg1: ::std::os::raw::c_float,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_setNum19(self as *mut crate::QString, arg1);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::setNum(double arg1, char f = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#setNum-9">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Sets the string to the printed value of <i>n</i>, formatted according to the given <i>format</i> and <i>precision</i>, and returns a reference to the string.</p>
    /// <p>The <i>format</i> can be 'e', 'E', 'f', 'g' or 'G' (see <a href="http://doc.qt.io/qt-5/qstring.html#argument-formats">Argument Formats</a> for an explanation of the formats).</p>
    /// <p>The formatting always uses <a href="http://doc.qt.io/qt-5/qlocale.html#Language-enum">QLocale::C</a>, i.e., English/UnitedStates. To get a localized string representation of a number, use <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>() with the appropriate locale.</p></div>
    pub unsafe fn set_num_double_char(
        &mut self,
        arg1: ::std::os::raw::c_double,
        f: ::std::os::raw::c_char,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_setNum20(self as *mut crate::QString, arg1, f);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::setNum(double arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#setNum-9">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Sets the string to the printed value of <i>n</i>, formatted according to the given <i>format</i> and <i>precision</i>, and returns a reference to the string.</p>
    /// <p>The <i>format</i> can be 'e', 'E', 'f', 'g' or 'G' (see <a href="http://doc.qt.io/qt-5/qstring.html#argument-formats">Argument Formats</a> for an explanation of the formats).</p>
    /// <p>The formatting always uses <a href="http://doc.qt.io/qt-5/qlocale.html#Language-enum">QLocale::C</a>, i.e., English/UnitedStates. To get a localized string representation of a number, use <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>() with the appropriate locale.</p></div>
    pub unsafe fn set_num_double(
        &mut self,
        arg1: ::std::os::raw::c_double,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_setNum21(self as *mut crate::QString, arg1);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Resets the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> to use the first <i>size</i> Unicode characters in the array <i>unicode</i>. The data in <i>unicode</i> is <i>not</i> copied. The caller must be able to guarantee that <i>unicode</i> will not be deleted or modified as long as the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> (or an unmodified copy of it) exists.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::setRawData(const QChar* unicode, int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#setRawData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Resets the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> to use the first <i>size</i> Unicode characters in the array <i>unicode</i>. The data in <i>unicode</i> is <i>not</i> copied. The caller must be able to guarantee that <i>unicode</i> will not be deleted or modified as long as the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> (or an unmodified copy of it) exists.</p>
    /// <p>This function can be used instead of <a href="http://doc.qt.io/qt-5/qstring.html#fromRawData">fromRawData</a>() to re-use existings <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> objects to save memory re-allocations.</p>
    /// <p>This function was introduced in  Qt 4.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#fromRawData">fromRawData</a>().</p></div>
    pub unsafe fn set_raw_data(
        &mut self,
        unicode: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QChar>>,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_setRawData(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QChar>>::cast_into(unicode)
                .as_raw_ptr(),
            size,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Resizes the string to <i>size</i> characters and copies <i>unicode</i> into the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::setUnicode(const QChar* unicode, int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#setUnicode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Resizes the string to <i>size</i> characters and copies <i>unicode</i> into the string.</p>
    /// <p>If <i>unicode</i> is 0, nothing is copied, but the string is still resized to <i>size</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#unicode">unicode</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#setUtf16">setUtf16</a>().</p></div>
    pub unsafe fn set_unicode(
        &mut self,
        unicode: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QChar>>,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_setUnicode(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QChar>>::cast_into(unicode)
                .as_raw_ptr(),
            size,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Resizes the string to <i>size</i> characters and copies <i>unicode</i> into the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::setUtf16(const unsigned short* utf16, int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#setUtf16">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Resizes the string to <i>size</i> characters and copies <i>unicode</i> into the string.</p>
    /// <p>If <i>unicode</i> is 0, nothing is copied, but the string is still resized to <i>size</i>.</p>
    /// <p>Note that unlike <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf16">fromUtf16</a>(), this function does not consider BOMs and possibly differing byte ordering.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#utf16">utf16</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#setUnicode">setUnicode</a>().</p></div>
    pub unsafe fn set_utf16(
        &mut self,
        utf16: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_ushort>>,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_setUtf16(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_ushort>>::cast_into(utf16)
                .as_raw_ptr(),
            size,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qstring.html#squeeze">squeeze</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QString::shrink_to_fit()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#shrink_to_fit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qstring.html#squeeze">squeeze</a>().</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#squeeze">squeeze</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn shrink_to_fit(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QString_shrink_to_fit(self as *mut crate::QString)
    }

    /// <p>Returns a string that has whitespace removed from the start and the end, and that has each sequence of internal whitespace replaced with a single space.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::simplified() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#simplified">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string that has whitespace removed from the start and the end, and that has each sequence of internal whitespace replaced with a single space.</p>
    /// <p>Whitespace means any character for which <a href="http://doc.qt.io/qt-5/qchar.html#isSpace">QChar::isSpace</a>() returns <code>true</code>. This includes the ASCII characters '\t', '\n', '\v', '\f', '\r', and ' '.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"  lots\t of\nwhitespace\r\n "</span>;
    ///   str <span class="operator">=</span> str<span class="operator">.</span>simplified();
    ///   <span class="comment">// str == "lots of whitespace";</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#trimmed">trimmed</a>().</p></div>
    pub unsafe fn simplified(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_simplified(self as *const crate::QString);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a string that has whitespace removed from the start and the end, and that has each sequence of internal whitespace replaced with a single space.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::simplified()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#simplified">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string that has whitespace removed from the start and the end, and that has each sequence of internal whitespace replaced with a single space.</p>
    /// <p>Whitespace means any character for which <a href="http://doc.qt.io/qt-5/qchar.html#isSpace">QChar::isSpace</a>() returns <code>true</code>. This includes the ASCII characters '\t', '\n', '\v', '\f', '\r', and ' '.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"  lots\t of\nwhitespace\r\n "</span>;
    ///   str <span class="operator">=</span> str<span class="operator">.</span>simplified();
    ///   <span class="comment">// str == "lots of whitespace";</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#trimmed">trimmed</a>().</p></div>
    pub unsafe fn simplified_mut(&mut self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_simplified1(self as *mut crate::QString);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the number of characters in this string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of characters in this string.</p>
    /// <p>The last character in the string is at position size() - 1.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"World"</span>;
    ///   <span class="type">int</span> n <span class="operator">=</span> str<span class="operator">.</span>size();         <span class="comment">// n == 5</span>
    ///   str<span class="operator">.</span>data()<span class="operator">[</span><span class="number">0</span><span class="operator">]</span>;              <span class="comment">// returns 'W'</span>
    ///   str<span class="operator">.</span>data()<span class="operator">[</span><span class="number">4</span><span class="operator">]</span>;              <span class="comment">// returns 'd'</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#resize">resize</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_size(self as *const crate::QString)
    }

    /// <p>Splits the string into substrings wherever <i>sep</i> occurs, and returns the list of those strings. If <i>sep</i> does not match anywhere in the string, split() returns a single-element list containing this string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QString::split(const QString& sep, QString::SplitBehavior behavior = …, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#split">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Splits the string into substrings wherever <i>sep</i> occurs, and returns the list of those strings. If <i>sep</i> does not match anywhere in the string, split() returns a single-element list containing this string.</p>
    /// <p><i>cs</i> specifies whether <i>sep</i> should be matched case sensitively or case insensitively.</p>
    /// <p>If <i>behavior</i> is <a href="http://doc.qt.io/qt-5/qstring.html#SplitBehavior-enum">QString::SkipEmptyParts</a>, empty entries don't appear in the result. By default, empty entries are kept.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"a,,b,c"</span>;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> list1 <span class="operator">=</span> str<span class="operator">.</span>split(<span class="char">','</span>);
    ///   <span class="comment">// list1: [ "a", "", "b", "c" ]</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> list2 <span class="operator">=</span> str<span class="operator">.</span>split(<span class="char">','</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span><span class="operator">::</span>SkipEmptyParts);
    ///   <span class="comment">// list2: [ "a", "b", "c" ]</span>
    ///
    /// </pre>
    /// <p>If <i>sep</i> is empty, split() returns an empty string, followed by each of the string's characters, followed by another empty string:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"abc"</span>;
    ///   <span class="keyword">auto</span> parts <span class="operator">=</span> str<span class="operator">.</span>split(<span class="string">""</span>);
    ///   <span class="comment">// parts: {"", "a", "b", "c", ""}</span>
    ///
    /// </pre>
    /// <p>To understand this behavior, recall that the empty string matches everywhere, so the above is qualitatively the same as:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"/a/b/c/"</span>;
    ///   <span class="keyword">auto</span> parts <span class="operator">=</span> str<span class="operator">.</span>split(<span class="char">'/'</span>);
    ///   <span class="comment">// parts: {"", "a", "b", "c", ""}</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>().</p></div>
    pub unsafe fn split_q_string_split_behavior_case_sensitivity(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        behavior: crate::q_string::SplitBehavior,
        cs: crate::CaseSensitivity,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_split(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(sep).as_raw_ptr(),
            behavior,
            cs,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QString::split(QChar sep, QString::SplitBehavior behavior = …, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#split-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn split_q_char_split_behavior_case_sensitivity(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        behavior: crate::q_string::SplitBehavior,
        cs: crate::CaseSensitivity,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_split1(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(sep).as_raw_ptr(),
            behavior,
            cs,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QString::split(const QRegExp& sep, QString::SplitBehavior behavior = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#split-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Splits the string into substrings wherever the regular expression <i>rx</i> matches, and returns the list of those strings. If <i>rx</i> does not match anywhere in the string, <a href="http://doc.qt.io/qt-5/qstring.html#split">split</a>() returns a single-element list containing this string.</p>
    /// <p>Here's an example where we extract the words in a sentence using one or more whitespace characters as the separator:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> list;
    ///
    ///   str <span class="operator">=</span> <span class="string">"Some  text\n\twith  strange whitespace."</span>;
    ///   list <span class="operator">=</span> str<span class="operator">.</span>split(<span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a></span>(<span class="string">"\\s+"</span>));
    ///   <span class="comment">// list: [ "Some", "text", "with", "strange", "whitespace." ]</span>
    ///
    /// </pre>
    /// <p>Here's a similar example, but this time we use any sequence of non-word characters as the separator:</p>
    /// <pre class="cpp">
    ///
    ///   str <span class="operator">=</span> <span class="string">"This time, a normal English sentence."</span>;
    ///   list <span class="operator">=</span> str<span class="operator">.</span>split(<span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a></span>(<span class="string">"\\W+"</span>)<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span><span class="operator">::</span>SkipEmptyParts);
    ///   <span class="comment">// list: [ "This", "time", "a", "normal", "English", "sentence" ]</span>
    ///
    /// </pre>
    /// <p>Here's a third example where we use a zero-length assertion, <b>\b</b> (word boundary), to split the string into an alternating sequence of non-word and word tokens:</p>
    /// <pre class="cpp">
    ///
    ///   str <span class="operator">=</span> <span class="string">"Now: this sentence fragment."</span>;
    ///   list <span class="operator">=</span> str<span class="operator">.</span>split(<span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a></span>(<span class="string">"\\b"</span>));
    ///   <span class="comment">// list: [ "", "Now", ": ", "this", " ", "sentence", " ", "fragment", "." ]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>().</p></div>
    pub unsafe fn split_q_reg_exp_split_behavior(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
        behavior: crate::q_string::SplitBehavior,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_split2(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(sep).as_raw_ptr(),
            behavior,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QString::split(const QRegularExpression& sep, QString::SplitBehavior behavior = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#split-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Splits the string into substrings wherever the regular expression <i>re</i> matches, and returns the list of those strings. If <i>re</i> does not match anywhere in the string, <a href="http://doc.qt.io/qt-5/qstring.html#split">split</a>() returns a single-element list containing this string.</p>
    /// <p>Here's an example where we extract the words in a sentence using one or more whitespace characters as the separator:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> list;
    ///
    ///   str <span class="operator">=</span> <span class="string">"Some  text\n\twith  strange whitespace."</span>;
    ///   list <span class="operator">=</span> str<span class="operator">.</span>split(<span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a></span>(<span class="string">"\\s+"</span>));
    ///   <span class="comment">// list: [ "Some", "text", "with", "strange", "whitespace." ]</span>
    ///
    /// </pre>
    /// <p>Here's a similar example, but this time we use any sequence of non-word characters as the separator:</p>
    /// <pre class="cpp">
    ///
    ///   str <span class="operator">=</span> <span class="string">"This time, a normal English sentence."</span>;
    ///   list <span class="operator">=</span> str<span class="operator">.</span>split(<span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a></span>(<span class="string">"\\W+"</span>)<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span><span class="operator">::</span>SkipEmptyParts);
    ///   <span class="comment">// list: [ "This", "time", "a", "normal", "English", "sentence" ]</span>
    ///
    /// </pre>
    /// <p>Here's a third example where we use a zero-length assertion, <b>\b</b> (word boundary), to split the string into an alternating sequence of non-word and word tokens:</p>
    /// <pre class="cpp">
    ///
    ///   str <span class="operator">=</span> <span class="string">"Now: this sentence fragment."</span>;
    ///   list <span class="operator">=</span> str<span class="operator">.</span>split(<span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a></span>(<span class="string">"\\b"</span>));
    ///   <span class="comment">// list: [ "", "Now", ": ", "this", " ", "sentence", " ", "fragment", "." ]</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>().</p></div>
    pub unsafe fn split_q_regular_expression_split_behavior(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
        behavior: crate::q_string::SplitBehavior,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_split3(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(sep)
                .as_raw_ptr(),
            behavior,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Splits the string into substrings wherever <i>sep</i> occurs, and returns the list of those strings. If <i>sep</i> does not match anywhere in the string, split() returns a single-element list containing this string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QString::split(const QString& sep, QString::SplitBehavior behavior = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#split">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Splits the string into substrings wherever <i>sep</i> occurs, and returns the list of those strings. If <i>sep</i> does not match anywhere in the string, split() returns a single-element list containing this string.</p>
    /// <p><i>cs</i> specifies whether <i>sep</i> should be matched case sensitively or case insensitively.</p>
    /// <p>If <i>behavior</i> is <a href="http://doc.qt.io/qt-5/qstring.html#SplitBehavior-enum">QString::SkipEmptyParts</a>, empty entries don't appear in the result. By default, empty entries are kept.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"a,,b,c"</span>;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> list1 <span class="operator">=</span> str<span class="operator">.</span>split(<span class="char">','</span>);
    ///   <span class="comment">// list1: [ "a", "", "b", "c" ]</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> list2 <span class="operator">=</span> str<span class="operator">.</span>split(<span class="char">','</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span><span class="operator">::</span>SkipEmptyParts);
    ///   <span class="comment">// list2: [ "a", "b", "c" ]</span>
    ///
    /// </pre>
    /// <p>If <i>sep</i> is empty, split() returns an empty string, followed by each of the string's characters, followed by another empty string:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"abc"</span>;
    ///   <span class="keyword">auto</span> parts <span class="operator">=</span> str<span class="operator">.</span>split(<span class="string">""</span>);
    ///   <span class="comment">// parts: {"", "a", "b", "c", ""}</span>
    ///
    /// </pre>
    /// <p>To understand this behavior, recall that the empty string matches everywhere, so the above is qualitatively the same as:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"/a/b/c/"</span>;
    ///   <span class="keyword">auto</span> parts <span class="operator">=</span> str<span class="operator">.</span>split(<span class="char">'/'</span>);
    ///   <span class="comment">// parts: {"", "a", "b", "c", ""}</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>().</p></div>
    pub unsafe fn split_q_string_split_behavior(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        behavior: crate::q_string::SplitBehavior,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_split4(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(sep).as_raw_ptr(),
            behavior,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Splits the string into substrings wherever <i>sep</i> occurs, and returns the list of those strings. If <i>sep</i> does not match anywhere in the string, split() returns a single-element list containing this string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QString::split(const QString& sep) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#split">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Splits the string into substrings wherever <i>sep</i> occurs, and returns the list of those strings. If <i>sep</i> does not match anywhere in the string, split() returns a single-element list containing this string.</p>
    /// <p><i>cs</i> specifies whether <i>sep</i> should be matched case sensitively or case insensitively.</p>
    /// <p>If <i>behavior</i> is <a href="http://doc.qt.io/qt-5/qstring.html#SplitBehavior-enum">QString::SkipEmptyParts</a>, empty entries don't appear in the result. By default, empty entries are kept.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"a,,b,c"</span>;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> list1 <span class="operator">=</span> str<span class="operator">.</span>split(<span class="char">','</span>);
    ///   <span class="comment">// list1: [ "a", "", "b", "c" ]</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> list2 <span class="operator">=</span> str<span class="operator">.</span>split(<span class="char">','</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span><span class="operator">::</span>SkipEmptyParts);
    ///   <span class="comment">// list2: [ "a", "b", "c" ]</span>
    ///
    /// </pre>
    /// <p>If <i>sep</i> is empty, split() returns an empty string, followed by each of the string's characters, followed by another empty string:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"abc"</span>;
    ///   <span class="keyword">auto</span> parts <span class="operator">=</span> str<span class="operator">.</span>split(<span class="string">""</span>);
    ///   <span class="comment">// parts: {"", "a", "b", "c", ""}</span>
    ///
    /// </pre>
    /// <p>To understand this behavior, recall that the empty string matches everywhere, so the above is qualitatively the same as:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"/a/b/c/"</span>;
    ///   <span class="keyword">auto</span> parts <span class="operator">=</span> str<span class="operator">.</span>split(<span class="char">'/'</span>);
    ///   <span class="comment">// parts: {"", "a", "b", "c", ""}</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>().</p></div>
    pub unsafe fn split_q_string(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_split5(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(sep).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QString::split(QChar sep, QString::SplitBehavior behavior = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#split-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn split_q_char_split_behavior(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        behavior: crate::q_string::SplitBehavior,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_split6(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(sep).as_raw_ptr(),
            behavior,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QString::split(QChar sep) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#split-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn split_q_char(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_split7(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(sep).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QString::split(const QRegExp& sep) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#split-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Splits the string into substrings wherever the regular expression <i>rx</i> matches, and returns the list of those strings. If <i>rx</i> does not match anywhere in the string, <a href="http://doc.qt.io/qt-5/qstring.html#split">split</a>() returns a single-element list containing this string.</p>
    /// <p>Here's an example where we extract the words in a sentence using one or more whitespace characters as the separator:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> list;
    ///
    ///   str <span class="operator">=</span> <span class="string">"Some  text\n\twith  strange whitespace."</span>;
    ///   list <span class="operator">=</span> str<span class="operator">.</span>split(<span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a></span>(<span class="string">"\\s+"</span>));
    ///   <span class="comment">// list: [ "Some", "text", "with", "strange", "whitespace." ]</span>
    ///
    /// </pre>
    /// <p>Here's a similar example, but this time we use any sequence of non-word characters as the separator:</p>
    /// <pre class="cpp">
    ///
    ///   str <span class="operator">=</span> <span class="string">"This time, a normal English sentence."</span>;
    ///   list <span class="operator">=</span> str<span class="operator">.</span>split(<span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a></span>(<span class="string">"\\W+"</span>)<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span><span class="operator">::</span>SkipEmptyParts);
    ///   <span class="comment">// list: [ "This", "time", "a", "normal", "English", "sentence" ]</span>
    ///
    /// </pre>
    /// <p>Here's a third example where we use a zero-length assertion, <b>\b</b> (word boundary), to split the string into an alternating sequence of non-word and word tokens:</p>
    /// <pre class="cpp">
    ///
    ///   str <span class="operator">=</span> <span class="string">"Now: this sentence fragment."</span>;
    ///   list <span class="operator">=</span> str<span class="operator">.</span>split(<span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a></span>(<span class="string">"\\b"</span>));
    ///   <span class="comment">// list: [ "", "Now", ": ", "this", " ", "sentence", " ", "fragment", "." ]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>().</p></div>
    pub unsafe fn split_q_reg_exp(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_split8(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(sep).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QString::split(const QRegularExpression& sep) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#split-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Splits the string into substrings wherever the regular expression <i>re</i> matches, and returns the list of those strings. If <i>re</i> does not match anywhere in the string, <a href="http://doc.qt.io/qt-5/qstring.html#split">split</a>() returns a single-element list containing this string.</p>
    /// <p>Here's an example where we extract the words in a sentence using one or more whitespace characters as the separator:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> list;
    ///
    ///   str <span class="operator">=</span> <span class="string">"Some  text\n\twith  strange whitespace."</span>;
    ///   list <span class="operator">=</span> str<span class="operator">.</span>split(<span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a></span>(<span class="string">"\\s+"</span>));
    ///   <span class="comment">// list: [ "Some", "text", "with", "strange", "whitespace." ]</span>
    ///
    /// </pre>
    /// <p>Here's a similar example, but this time we use any sequence of non-word characters as the separator:</p>
    /// <pre class="cpp">
    ///
    ///   str <span class="operator">=</span> <span class="string">"This time, a normal English sentence."</span>;
    ///   list <span class="operator">=</span> str<span class="operator">.</span>split(<span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a></span>(<span class="string">"\\W+"</span>)<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span><span class="operator">::</span>SkipEmptyParts);
    ///   <span class="comment">// list: [ "This", "time", "a", "normal", "English", "sentence" ]</span>
    ///
    /// </pre>
    /// <p>Here's a third example where we use a zero-length assertion, <b>\b</b> (word boundary), to split the string into an alternating sequence of non-word and word tokens:</p>
    /// <pre class="cpp">
    ///
    ///   str <span class="operator">=</span> <span class="string">"Now: this sentence fragment."</span>;
    ///   list <span class="operator">=</span> str<span class="operator">.</span>split(<span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a></span>(<span class="string">"\\b"</span>));
    ///   <span class="comment">// list: [ "", "Now", ": ", "this", " ", "sentence", " ", "fragment", "." ]</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#section">section</a>().</p></div>
    pub unsafe fn split_q_regular_expression(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_split9(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(sep)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Splits the string into substring references wherever <i>sep</i> occurs, and returns the list of those strings.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStringRef> QString::splitRef(const QString& sep, QString::SplitBehavior behavior = …, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#splitRef">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Splits the string into substring references wherever <i>sep</i> occurs, and returns the list of those strings.</p>
    /// <p>See <a href="http://doc.qt.io/qt-5/qstring.html#split">QString::split</a>() for how <i>sep</i>, <i>behavior</i> and <i>cs</i> interact to form the result.</p>
    /// <p><b>Note: </b>All references are valid as long this string is alive. Destroying this string will cause all references be dangling pointers.</p><p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a> and <a href="http://doc.qt.io/qt-5/qstring.html#split">split</a>().</p></div>
    pub unsafe fn split_ref_q_string_split_behavior_case_sensitivity(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        behavior: crate::q_string::SplitBehavior,
        cs: crate::CaseSensitivity,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_splitRef(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(sep).as_raw_ptr(),
            behavior,
            cs,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStringRef> QString::splitRef(QChar sep, QString::SplitBehavior behavior = …, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#splitRef-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 5.4.</p></div>
    pub unsafe fn split_ref_q_char_split_behavior_case_sensitivity(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        behavior: crate::q_string::SplitBehavior,
        cs: crate::CaseSensitivity,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_splitRef1(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(sep).as_raw_ptr(),
            behavior,
            cs,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStringRef> QString::splitRef(const QRegExp& sep, QString::SplitBehavior behavior = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#splitRef-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Splits the string into substring references wherever the regular expression <i>rx</i> matches, and returns the list of those strings. If <i>rx</i> does not match anywhere in the string, <a href="http://doc.qt.io/qt-5/qstring.html#splitRef">splitRef</a>() returns a single-element vector containing this string reference.</p>
    /// <p><b>Note: </b>All references are valid as long this string is alive. Destroying this string will cause all references be dangling pointers.</p><p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a> and <a href="http://doc.qt.io/qt-5/qstring.html#split">split</a>().</p></div>
    pub unsafe fn split_ref_q_reg_exp_split_behavior(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
        behavior: crate::q_string::SplitBehavior,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_splitRef2(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(sep).as_raw_ptr(),
            behavior,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStringRef> QString::splitRef(const QRegularExpression& sep, QString::SplitBehavior behavior = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#splitRef-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Splits the string into substring references wherever the regular expression <i>re</i> matches, and returns the list of those strings. If <i>re</i> does not match anywhere in the string, <a href="http://doc.qt.io/qt-5/qstring.html#splitRef">splitRef</a>() returns a single-element vector containing this string reference.</p>
    /// <p><b>Note: </b>All references are valid as long this string is alive. Destroying this string will cause all references be dangling pointers.</p><p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#split">split</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a>.</p></div>
    pub unsafe fn split_ref_q_regular_expression_split_behavior(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
        behavior: crate::q_string::SplitBehavior,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_splitRef3(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(sep)
                .as_raw_ptr(),
            behavior,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Splits the string into substring references wherever <i>sep</i> occurs, and returns the list of those strings.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStringRef> QString::splitRef(const QString& sep, QString::SplitBehavior behavior = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#splitRef">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Splits the string into substring references wherever <i>sep</i> occurs, and returns the list of those strings.</p>
    /// <p>See <a href="http://doc.qt.io/qt-5/qstring.html#split">QString::split</a>() for how <i>sep</i>, <i>behavior</i> and <i>cs</i> interact to form the result.</p>
    /// <p><b>Note: </b>All references are valid as long this string is alive. Destroying this string will cause all references be dangling pointers.</p><p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a> and <a href="http://doc.qt.io/qt-5/qstring.html#split">split</a>().</p></div>
    pub unsafe fn split_ref_q_string_split_behavior(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        behavior: crate::q_string::SplitBehavior,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_splitRef4(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(sep).as_raw_ptr(),
            behavior,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Splits the string into substring references wherever <i>sep</i> occurs, and returns the list of those strings.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStringRef> QString::splitRef(const QString& sep) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#splitRef">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Splits the string into substring references wherever <i>sep</i> occurs, and returns the list of those strings.</p>
    /// <p>See <a href="http://doc.qt.io/qt-5/qstring.html#split">QString::split</a>() for how <i>sep</i>, <i>behavior</i> and <i>cs</i> interact to form the result.</p>
    /// <p><b>Note: </b>All references are valid as long this string is alive. Destroying this string will cause all references be dangling pointers.</p><p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a> and <a href="http://doc.qt.io/qt-5/qstring.html#split">split</a>().</p></div>
    pub unsafe fn split_ref_q_string(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_splitRef5(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(sep).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStringRef> QString::splitRef(QChar sep, QString::SplitBehavior behavior = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#splitRef-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 5.4.</p></div>
    pub unsafe fn split_ref_q_char_split_behavior(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        behavior: crate::q_string::SplitBehavior,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_splitRef6(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(sep).as_raw_ptr(),
            behavior,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStringRef> QString::splitRef(QChar sep) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#splitRef-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 5.4.</p></div>
    pub unsafe fn split_ref_q_char(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_splitRef7(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(sep).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStringRef> QString::splitRef(const QRegExp& sep) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#splitRef-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Splits the string into substring references wherever the regular expression <i>rx</i> matches, and returns the list of those strings. If <i>rx</i> does not match anywhere in the string, <a href="http://doc.qt.io/qt-5/qstring.html#splitRef">splitRef</a>() returns a single-element vector containing this string reference.</p>
    /// <p><b>Note: </b>All references are valid as long this string is alive. Destroying this string will cause all references be dangling pointers.</p><p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a> and <a href="http://doc.qt.io/qt-5/qstring.html#split">split</a>().</p></div>
    pub unsafe fn split_ref_q_reg_exp(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_splitRef8(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(sep).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStringRef> QString::splitRef(const QRegularExpression& sep) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#splitRef-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Splits the string into substring references wherever the regular expression <i>re</i> matches, and returns the list of those strings. If <i>re</i> does not match anywhere in the string, <a href="http://doc.qt.io/qt-5/qstring.html#splitRef">splitRef</a>() returns a single-element vector containing this string reference.</p>
    /// <p><b>Note: </b>All references are valid as long this string is alive. Destroying this string will cause all references be dangling pointers.</p><p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#split">split</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a>.</p></div>
    pub unsafe fn split_ref_q_regular_expression(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_splitRef9(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(sep)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Releases any memory not required to store the character data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QString::squeeze()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#squeeze">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Releases any memory not required to store the character data.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>'s memory usage. In general, you will rarely ever need to call this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#capacity">capacity</a>().</p></div>
    pub unsafe fn squeeze(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QString_squeeze(self as *mut crate::QString)
    }

    /// <p>Returns <code>true</code> if the string starts with <i>s</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::startsWith(const QString& s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the string starts with <i>s</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"Bananas"</span>;
    ///   str<span class="operator">.</span>startsWith(<span class="string">"Ban"</span>);     <span class="comment">// returns true</span>
    ///   str<span class="operator">.</span>startsWith(<span class="string">"Car"</span>);     <span class="comment">// returns false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#endsWith">endsWith</a>().</p></div>
    pub unsafe fn starts_with_q_string_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_startsWith(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            cs,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::startsWith(const QStringRef& s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#startsWith-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the string starts with the string reference <i>s</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#endsWith">endsWith</a>().</p></div>
    pub unsafe fn starts_with_q_string_ref_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_startsWith1(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#startsWith">startsWith</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::startsWith(QLatin1String s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#startsWith-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#startsWith">startsWith</a>().</p></div>
    pub unsafe fn starts_with_q_latin1_string_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_startsWith2(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#startsWith">startsWith</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::startsWith(QChar c, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#startsWith-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#startsWith">startsWith</a>().</p>
    /// <p>Returns <code>true</code> if the string starts with <i>c</i>; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn starts_with_q_char_case_sensitivity(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_startsWith3(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
            cs,
        )
    }

    /// <p>Returns <code>true</code> if the string starts with <i>s</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::startsWith(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the string starts with <i>s</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"Bananas"</span>;
    ///   str<span class="operator">.</span>startsWith(<span class="string">"Ban"</span>);     <span class="comment">// returns true</span>
    ///   str<span class="operator">.</span>startsWith(<span class="string">"Car"</span>);     <span class="comment">// returns false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#endsWith">endsWith</a>().</p></div>
    pub unsafe fn starts_with_q_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_startsWith4(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::startsWith(const QStringRef& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#startsWith-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the string starts with the string reference <i>s</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#endsWith">endsWith</a>().</p></div>
    pub unsafe fn starts_with_q_string_ref(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_startsWith5(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#startsWith">startsWith</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::startsWith(QLatin1String s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#startsWith-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#startsWith">startsWith</a>().</p></div>
    pub unsafe fn starts_with_q_latin1_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_startsWith6(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#startsWith">startsWith</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::startsWith(QChar c) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#startsWith-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#startsWith">startsWith</a>().</p>
    /// <p>Returns <code>true</code> if the string starts with <i>c</i>; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn starts_with_q_char(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_startsWith7(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::startsWith(QStringView s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#startsWith-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the string starts with the string-view <i>str</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case-sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#endsWith">endsWith</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn starts_with_q_string_view_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_startsWith16(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
            cs,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::startsWith(QStringView s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#startsWith-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the string starts with the string-view <i>str</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case-sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#endsWith">endsWith</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn starts_with_q_string_view(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QString_startsWith17(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// <p>Swaps string <i>other</i> with this string. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QString::swap(QString& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps string <i>other</i> with this string. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QString_swap(
            self as *mut crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QString>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the case folded equivalent of the string. For most Unicode characters this is the same as <a href="http://doc.qt.io/qt-5/qstring.html#toLower">toLower</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::toCaseFolded() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toCaseFolded">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the case folded equivalent of the string. For most Unicode characters this is the same as <a href="http://doc.qt.io/qt-5/qstring.html#toLower">toLower</a>().</p></div>
    pub unsafe fn to_case_folded(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_toCaseFolded(self as *const crate::QString);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the case folded equivalent of the string. For most Unicode characters this is the same as <a href="http://doc.qt.io/qt-5/qstring.html#toLower">toLower</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::toCaseFolded()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toCaseFolded">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the case folded equivalent of the string. For most Unicode characters this is the same as <a href="http://doc.qt.io/qt-5/qstring.html#toLower">toLower</a>().</p></div>
    pub unsafe fn to_case_folded_mut(&mut self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_toCaseFolded1(self as *mut crate::QString);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the string converted to a <code>double</code> value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QString::toDouble(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toDouble">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to a <code>double</code> value.</p>
    /// <p>Returns 0.0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, <code>*</code><i>ok</i> is set to <code>false</code>; otherwise <code>*</code><i>ok</i> is set to <code>true</code>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"1234.56"</span>;
    ///   <span class="type">double</span> val <span class="operator">=</span> str<span class="operator">.</span>toDouble();   <span class="comment">// val == 1234.56</span>
    ///
    /// </pre>
    /// <p><b>Warning:</b> The <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> content may only contain valid numerical characters which includes the plus/minus sign, the characters g and e used in scientific notation, and the decimal point. Including the unit or additional characters leads to a conversion error.</p>
    /// <pre class="cpp">
    ///
    ///   bool ok;
    ///   <span class="type">double</span> d;
    ///
    ///   d <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>( <span class="string">"1234.56e-02"</span> )<span class="operator">.</span>toDouble(<span class="operator">&amp;</span>ok); <span class="comment">// ok == true, d == 12.3456</span>
    ///
    /// </pre>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">QLocale::toDouble</a>()</p>
    /// <pre class="cpp">
    ///
    ///   d <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>( <span class="string">"1234,56"</span> )<span class="operator">.</span>toDouble(<span class="operator">&amp;</span>ok); <span class="comment">// ok == false</span>
    ///   d <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>( <span class="string">"1234.56"</span> )<span class="operator">.</span>toDouble(<span class="operator">&amp;</span>ok); <span class="comment">// ok == true, d == 1234.56</span>
    ///
    /// </pre>
    /// <p>For historical reasons, this function does not handle thousands group separators. If you need to convert such numbers, use <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">QLocale::toDouble</a>().</p>
    /// <pre class="cpp">
    ///
    ///   d <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>( <span class="string">"1,234,567.89"</span> )<span class="operator">.</span>toDouble(<span class="operator">&amp;</span>ok); <span class="comment">// ok == false</span>
    ///   d <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>( <span class="string">"1234567.89"</span> )<span class="operator">.</span>toDouble(<span class="operator">&amp;</span>ok); <span class="comment">// ok == true</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">QLocale::setDefault</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">QLocale::toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#trimmed">trimmed</a>().</p></div>
    pub unsafe fn to_double_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QString_toDouble(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the string converted to a <code>double</code> value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QString::toDouble() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toDouble">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to a <code>double</code> value.</p>
    /// <p>Returns 0.0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, <code>*</code><i>ok</i> is set to <code>false</code>; otherwise <code>*</code><i>ok</i> is set to <code>true</code>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"1234.56"</span>;
    ///   <span class="type">double</span> val <span class="operator">=</span> str<span class="operator">.</span>toDouble();   <span class="comment">// val == 1234.56</span>
    ///
    /// </pre>
    /// <p><b>Warning:</b> The <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> content may only contain valid numerical characters which includes the plus/minus sign, the characters g and e used in scientific notation, and the decimal point. Including the unit or additional characters leads to a conversion error.</p>
    /// <pre class="cpp">
    ///
    ///   bool ok;
    ///   <span class="type">double</span> d;
    ///
    ///   d <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>( <span class="string">"1234.56e-02"</span> )<span class="operator">.</span>toDouble(<span class="operator">&amp;</span>ok); <span class="comment">// ok == true, d == 12.3456</span>
    ///
    /// </pre>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">QLocale::toDouble</a>()</p>
    /// <pre class="cpp">
    ///
    ///   d <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>( <span class="string">"1234,56"</span> )<span class="operator">.</span>toDouble(<span class="operator">&amp;</span>ok); <span class="comment">// ok == false</span>
    ///   d <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>( <span class="string">"1234.56"</span> )<span class="operator">.</span>toDouble(<span class="operator">&amp;</span>ok); <span class="comment">// ok == true, d == 1234.56</span>
    ///
    /// </pre>
    /// <p>For historical reasons, this function does not handle thousands group separators. If you need to convert such numbers, use <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">QLocale::toDouble</a>().</p>
    /// <pre class="cpp">
    ///
    ///   d <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>( <span class="string">"1,234,567.89"</span> )<span class="operator">.</span>toDouble(<span class="operator">&amp;</span>ok); <span class="comment">// ok == false</span>
    ///   d <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span>( <span class="string">"1234567.89"</span> )<span class="operator">.</span>toDouble(<span class="operator">&amp;</span>ok); <span class="comment">// ok == true</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">QLocale::setDefault</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">QLocale::toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#trimmed">trimmed</a>().</p></div>
    pub unsafe fn to_double_0a(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QString_toDouble1(self as *const crate::QString)
    }

    /// <p>Returns the string converted to a <code>float</code> value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```float QString::toFloat(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toFloat">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to a <code>float</code> value.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>. Returns 0.0 if the conversion fails.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toFloat">QLocale::toFloat</a>()</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str1 <span class="operator">=</span> <span class="string">"1234.56"</span>;
    ///   str1<span class="operator">.</span>toFloat();             <span class="comment">// returns 1234.56</span>
    ///
    ///   bool ok;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str2 <span class="operator">=</span> <span class="string">"R2D2"</span>;
    ///   str2<span class="operator">.</span>toFloat(<span class="operator">&amp;</span>ok);          <span class="comment">// returns 0.0, sets ok to false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toDouble">toDouble</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toInt">toInt</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toFloat">QLocale::toFloat</a>().</p></div>
    pub unsafe fn to_float_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_float {
        crate::__ffi::ctr_qt_core_ffi_QString_toFloat(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the string converted to a <code>float</code> value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```float QString::toFloat() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toFloat">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to a <code>float</code> value.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>. Returns 0.0 if the conversion fails.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toFloat">QLocale::toFloat</a>()</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str1 <span class="operator">=</span> <span class="string">"1234.56"</span>;
    ///   str1<span class="operator">.</span>toFloat();             <span class="comment">// returns 1234.56</span>
    ///
    ///   bool ok;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str2 <span class="operator">=</span> <span class="string">"R2D2"</span>;
    ///   str2<span class="operator">.</span>toFloat(<span class="operator">&amp;</span>ok);          <span class="comment">// returns 0.0, sets ok to false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toDouble">toDouble</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toInt">toInt</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toFloat">QLocale::toFloat</a>().</p></div>
    pub unsafe fn to_float_0a(&self) -> ::std::os::raw::c_float {
        crate::__ffi::ctr_qt_core_ffi_QString_toFloat1(self as *const crate::QString)
    }

    /// <p>Converts a plain text string to an HTML string with HTML metacharacters <code>&lt;</code>, <code>&gt;</code>, <code>&amp;</code>, and <code>"</code> replaced by HTML entities.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::toHtmlEscaped() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toHtmlEscaped">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts a plain text string to an HTML string with HTML metacharacters <code>&lt;</code>, <code>&gt;</code>, <code>&amp;</code>, and <code>"</code> replaced by HTML entities.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> plain <span class="operator">=</span> <span class="string">"#include &lt;QtCore&gt;"</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> html <span class="operator">=</span> plain<span class="operator">.</span>toHtmlEscaped();
    ///   <span class="comment">// html == "#include &amp;lt;QtCore&amp;gt;"</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn to_html_escaped(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_toHtmlEscaped(self as *const crate::QString);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the string converted to an <code>int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::toInt(bool* ok = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">QLocale::toInt</a>()</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"FF"</span>;
    ///   bool ok;
    ///   <span class="type">int</span> hex <span class="operator">=</span> str<span class="operator">.</span>toInt(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);       <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type">int</span> dec <span class="operator">=</span> str<span class="operator">.</span>toInt(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);       <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toUInt">toUInt</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toDouble">toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">QLocale::toInt</a>().</p></div>
    pub unsafe fn to_int_2a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
        base: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_toInt(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
            base,
        )
    }

    /// <p>Returns the string converted to an <code>int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::toInt(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">QLocale::toInt</a>()</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"FF"</span>;
    ///   bool ok;
    ///   <span class="type">int</span> hex <span class="operator">=</span> str<span class="operator">.</span>toInt(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);       <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type">int</span> dec <span class="operator">=</span> str<span class="operator">.</span>toInt(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);       <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toUInt">toUInt</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toDouble">toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">QLocale::toInt</a>().</p></div>
    pub unsafe fn to_int_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_toInt1(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the string converted to an <code>int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QString::toInt() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">QLocale::toInt</a>()</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"FF"</span>;
    ///   bool ok;
    ///   <span class="type">int</span> hex <span class="operator">=</span> str<span class="operator">.</span>toInt(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);       <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type">int</span> dec <span class="operator">=</span> str<span class="operator">.</span>toInt(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);       <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toUInt">toUInt</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toDouble">toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">QLocale::toInt</a>().</p></div>
    pub unsafe fn to_int_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QString_toInt2(self as *const crate::QString)
    }

    /// <p>Returns a Latin-1 representation of the string as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QString::toLatin1() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toLatin1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a Latin-1 representation of the string as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    /// <p>The returned byte array is undefined if the string contains non-Latin1 characters. Those characters may be suppressed or replaced with a question mark.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#fromLatin1">fromLatin1</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">toUtf8</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toLocal8Bit">toLocal8Bit</a>(), and <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a>.</p></div>
    pub unsafe fn to_latin1(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_toLatin1(self as *const crate::QString);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a Latin-1 representation of the string as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QString::toLatin1()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toLatin1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a Latin-1 representation of the string as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    /// <p>The returned byte array is undefined if the string contains non-Latin1 characters. Those characters may be suppressed or replaced with a question mark.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#fromLatin1">fromLatin1</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">toUtf8</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toLocal8Bit">toLocal8Bit</a>(), and <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a>.</p></div>
    pub unsafe fn to_latin1_mut(&mut self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_toLatin11(self as *mut crate::QString);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the local 8-bit representation of the string as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>. The returned byte array is undefined if the string contains characters not supported by the local 8-bit encoding.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QString::toLocal8Bit() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toLocal8Bit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the local 8-bit representation of the string as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>. The returned byte array is undefined if the string contains characters not supported by the local 8-bit encoding.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForLocale">QTextCodec::codecForLocale</a>() is used to perform the conversion from Unicode. If the locale encoding could not be determined, this function does the same as <a href="http://doc.qt.io/qt-5/qstring.html#toLatin1">toLatin1</a>().</p>
    /// <p>If this string contains any characters that cannot be encoded in the locale, the returned byte array is undefined. Those characters may be suppressed or replaced by another.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#fromLocal8Bit">fromLocal8Bit</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toLatin1">toLatin1</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">toUtf8</a>(), and <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a>.</p></div>
    pub unsafe fn to_local8_bit(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_toLocal8Bit(self as *const crate::QString);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the local 8-bit representation of the string as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>. The returned byte array is undefined if the string contains characters not supported by the local 8-bit encoding.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QString::toLocal8Bit()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toLocal8Bit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the local 8-bit representation of the string as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>. The returned byte array is undefined if the string contains characters not supported by the local 8-bit encoding.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForLocale">QTextCodec::codecForLocale</a>() is used to perform the conversion from Unicode. If the locale encoding could not be determined, this function does the same as <a href="http://doc.qt.io/qt-5/qstring.html#toLatin1">toLatin1</a>().</p>
    /// <p>If this string contains any characters that cannot be encoded in the locale, the returned byte array is undefined. Those characters may be suppressed or replaced by another.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#fromLocal8Bit">fromLocal8Bit</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toLatin1">toLatin1</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">toUtf8</a>(), and <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a>.</p></div>
    pub unsafe fn to_local8_bit_mut(&mut self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_toLocal8Bit1(self as *mut crate::QString);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the string converted to a <code>long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```long QString::toLong(bool* ok = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to a <code>long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toLongLong">QLocale::toLongLong</a>()</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"FF"</span>;
    ///   bool ok;
    ///
    ///   <span class="type">long</span> hex <span class="operator">=</span> str<span class="operator">.</span>toLong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);     <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type">long</span> dec <span class="operator">=</span> str<span class="operator">.</span>toLong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);     <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toULong">toULong</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toInt">toInt</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">QLocale::toInt</a>().</p></div>
    pub unsafe fn to_long_2a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
        base: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_long {
        crate::__ffi::ctr_qt_core_ffi_QString_toLong(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
            base,
        )
    }

    /// <p>Returns the string converted to a <code>long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```long QString::toLong(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to a <code>long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toLongLong">QLocale::toLongLong</a>()</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"FF"</span>;
    ///   bool ok;
    ///
    ///   <span class="type">long</span> hex <span class="operator">=</span> str<span class="operator">.</span>toLong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);     <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type">long</span> dec <span class="operator">=</span> str<span class="operator">.</span>toLong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);     <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toULong">toULong</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toInt">toInt</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">QLocale::toInt</a>().</p></div>
    pub unsafe fn to_long_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_long {
        crate::__ffi::ctr_qt_core_ffi_QString_toLong1(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the string converted to a <code>long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```long QString::toLong() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to a <code>long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toLongLong">QLocale::toLongLong</a>()</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"FF"</span>;
    ///   bool ok;
    ///
    ///   <span class="type">long</span> hex <span class="operator">=</span> str<span class="operator">.</span>toLong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);     <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type">long</span> dec <span class="operator">=</span> str<span class="operator">.</span>toLong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);     <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toULong">toULong</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toInt">toInt</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">QLocale::toInt</a>().</p></div>
    pub unsafe fn to_long_0a(&self) -> ::std::os::raw::c_long {
        crate::__ffi::ctr_qt_core_ffi_QString_toLong2(self as *const crate::QString)
    }

    /// <p>Returns the string converted to a <code>long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qlonglong QString::toLongLong(bool* ok = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toLongLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to a <code>long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toLongLong">QLocale::toLongLong</a>()</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"FF"</span>;
    ///   bool ok;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#qint64-typedef">qint64</a></span> hex <span class="operator">=</span> str<span class="operator">.</span>toLongLong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);      <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#qint64-typedef">qint64</a></span> dec <span class="operator">=</span> str<span class="operator">.</span>toLongLong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);      <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toULongLong">toULongLong</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toInt">toInt</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toLongLong">QLocale::toLongLong</a>().</p></div>
    pub unsafe fn to_long_long_2a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
        base: ::std::os::raw::c_int,
    ) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QString_toLongLong(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
            base,
        )
    }

    /// <p>Returns the string converted to a <code>long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qlonglong QString::toLongLong(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toLongLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to a <code>long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toLongLong">QLocale::toLongLong</a>()</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"FF"</span>;
    ///   bool ok;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#qint64-typedef">qint64</a></span> hex <span class="operator">=</span> str<span class="operator">.</span>toLongLong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);      <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#qint64-typedef">qint64</a></span> dec <span class="operator">=</span> str<span class="operator">.</span>toLongLong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);      <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toULongLong">toULongLong</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toInt">toInt</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toLongLong">QLocale::toLongLong</a>().</p></div>
    pub unsafe fn to_long_long_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QString_toLongLong1(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the string converted to a <code>long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qlonglong QString::toLongLong() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toLongLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to a <code>long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toLongLong">QLocale::toLongLong</a>()</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"FF"</span>;
    ///   bool ok;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#qint64-typedef">qint64</a></span> hex <span class="operator">=</span> str<span class="operator">.</span>toLongLong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);      <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#qint64-typedef">qint64</a></span> dec <span class="operator">=</span> str<span class="operator">.</span>toLongLong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);      <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toULongLong">toULongLong</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toInt">toInt</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toLongLong">QLocale::toLongLong</a>().</p></div>
    pub unsafe fn to_long_long_0a(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QString_toLongLong2(self as *const crate::QString)
    }

    /// <p>Returns a lowercase copy of the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::toLower() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toLower">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a lowercase copy of the string.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"The Qt PROJECT"</span>;
    ///   str <span class="operator">=</span> str<span class="operator">.</span>toLower();        <span class="comment">// str == "the qt project"</span>
    ///
    /// </pre>
    /// <p>The case conversion will always happen in the 'C' locale. For locale dependent case folding use <a href="http://doc.qt.io/qt-5/qlocale.html#toLower">QLocale::toLower</a>()</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toUpper">toUpper</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toLower">QLocale::toLower</a>().</p></div>
    pub unsafe fn to_lower(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_toLower(self as *const crate::QString);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a lowercase copy of the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::toLower()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toLower">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a lowercase copy of the string.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"The Qt PROJECT"</span>;
    ///   str <span class="operator">=</span> str<span class="operator">.</span>toLower();        <span class="comment">// str == "the qt project"</span>
    ///
    /// </pre>
    /// <p>The case conversion will always happen in the 'C' locale. For locale dependent case folding use <a href="http://doc.qt.io/qt-5/qlocale.html#toLower">QLocale::toLower</a>()</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toUpper">toUpper</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toLower">QLocale::toLower</a>().</p></div>
    pub unsafe fn to_lower_mut(&mut self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_toLower1(self as *mut crate::QString);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the string converted to a <code>short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```short QString::toShort(bool* ok = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toShort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to a <code>short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toShort">QLocale::toShort</a>()</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"FF"</span>;
    ///   bool ok;
    ///
    ///   <span class="type">short</span> hex <span class="operator">=</span> str<span class="operator">.</span>toShort(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);   <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type">short</span> dec <span class="operator">=</span> str<span class="operator">.</span>toShort(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);   <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toUShort">toUShort</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toInt">toInt</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toShort">QLocale::toShort</a>().</p></div>
    pub unsafe fn to_short_2a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
        base: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_short {
        crate::__ffi::ctr_qt_core_ffi_QString_toShort(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
            base,
        )
    }

    /// <p>Returns the string converted to a <code>short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```short QString::toShort(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toShort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to a <code>short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toShort">QLocale::toShort</a>()</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"FF"</span>;
    ///   bool ok;
    ///
    ///   <span class="type">short</span> hex <span class="operator">=</span> str<span class="operator">.</span>toShort(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);   <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type">short</span> dec <span class="operator">=</span> str<span class="operator">.</span>toShort(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);   <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toUShort">toUShort</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toInt">toInt</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toShort">QLocale::toShort</a>().</p></div>
    pub unsafe fn to_short_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_short {
        crate::__ffi::ctr_qt_core_ffi_QString_toShort1(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the string converted to a <code>short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```short QString::toShort() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toShort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to a <code>short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toShort">QLocale::toShort</a>()</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"FF"</span>;
    ///   bool ok;
    ///
    ///   <span class="type">short</span> hex <span class="operator">=</span> str<span class="operator">.</span>toShort(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);   <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type">short</span> dec <span class="operator">=</span> str<span class="operator">.</span>toShort(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);   <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toUShort">toUShort</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toInt">toInt</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toShort">QLocale::toShort</a>().</p></div>
    pub unsafe fn to_short_0a(&self) -> ::std::os::raw::c_short {
        crate::__ffi::ctr_qt_core_ffi_QString_toShort2(self as *const crate::QString)
    }

    /// <p>Returns the string converted to an <code>unsigned int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QString::toUInt(bool* ok = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toUInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>unsigned int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toUInt">QLocale::toUInt</a>()</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"FF"</span>;
    ///   bool ok;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> hex <span class="operator">=</span> str<span class="operator">.</span>toUInt(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);     <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> dec <span class="operator">=</span> str<span class="operator">.</span>toUInt(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);     <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toInt">toInt</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toUInt">QLocale::toUInt</a>().</p></div>
    pub unsafe fn to_u_int_2a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
        base: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QString_toUInt(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
            base,
        )
    }

    /// <p>Returns the string converted to an <code>unsigned int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QString::toUInt(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toUInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>unsigned int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toUInt">QLocale::toUInt</a>()</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"FF"</span>;
    ///   bool ok;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> hex <span class="operator">=</span> str<span class="operator">.</span>toUInt(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);     <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> dec <span class="operator">=</span> str<span class="operator">.</span>toUInt(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);     <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toInt">toInt</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toUInt">QLocale::toUInt</a>().</p></div>
    pub unsafe fn to_u_int_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QString_toUInt1(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the string converted to an <code>unsigned int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QString::toUInt() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toUInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>unsigned int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toUInt">QLocale::toUInt</a>()</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"FF"</span>;
    ///   bool ok;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> hex <span class="operator">=</span> str<span class="operator">.</span>toUInt(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);     <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> dec <span class="operator">=</span> str<span class="operator">.</span>toUInt(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);     <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toInt">toInt</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toUInt">QLocale::toUInt</a>().</p></div>
    pub unsafe fn to_u_int_0a(&self) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QString_toUInt2(self as *const crate::QString)
    }

    /// <p>Returns the string converted to an <code>unsigned long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned long QString::toULong(bool* ok = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toULong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>unsigned long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong">QLocale::toULongLong</a>()</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"FF"</span>;
    ///   bool ok;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#ulong-typedef">ulong</a></span> hex <span class="operator">=</span> str<span class="operator">.</span>toULong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);   <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#ulong-typedef">ulong</a></span> dec <span class="operator">=</span> str<span class="operator">.</span>toULong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);   <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toUInt">QLocale::toUInt</a>().</p></div>
    pub unsafe fn to_u_long_2a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
        base: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_ulong {
        crate::__ffi::ctr_qt_core_ffi_QString_toULong(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
            base,
        )
    }

    /// <p>Returns the string converted to an <code>unsigned long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned long QString::toULong(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toULong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>unsigned long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong">QLocale::toULongLong</a>()</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"FF"</span>;
    ///   bool ok;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#ulong-typedef">ulong</a></span> hex <span class="operator">=</span> str<span class="operator">.</span>toULong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);   <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#ulong-typedef">ulong</a></span> dec <span class="operator">=</span> str<span class="operator">.</span>toULong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);   <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toUInt">QLocale::toUInt</a>().</p></div>
    pub unsafe fn to_u_long_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_ulong {
        crate::__ffi::ctr_qt_core_ffi_QString_toULong1(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the string converted to an <code>unsigned long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned long QString::toULong() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toULong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>unsigned long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong">QLocale::toULongLong</a>()</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"FF"</span>;
    ///   bool ok;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#ulong-typedef">ulong</a></span> hex <span class="operator">=</span> str<span class="operator">.</span>toULong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);   <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#ulong-typedef">ulong</a></span> dec <span class="operator">=</span> str<span class="operator">.</span>toULong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);   <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toUInt">QLocale::toUInt</a>().</p></div>
    pub unsafe fn to_u_long_0a(&self) -> ::std::os::raw::c_ulong {
        crate::__ffi::ctr_qt_core_ffi_QString_toULong2(self as *const crate::QString)
    }

    /// <p>Returns the string converted to an <code>unsigned long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qulonglong QString::toULongLong(bool* ok = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toULongLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>unsigned long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong">QLocale::toULongLong</a>()</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"FF"</span>;
    ///   bool ok;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#quint64-typedef">quint64</a></span> hex <span class="operator">=</span> str<span class="operator">.</span>toULongLong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);    <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#quint64-typedef">quint64</a></span> dec <span class="operator">=</span> str<span class="operator">.</span>toULongLong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);    <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toLongLong">toLongLong</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong">QLocale::toULongLong</a>().</p></div>
    pub unsafe fn to_u_long_long_2a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
        base: ::std::os::raw::c_int,
    ) -> u64 {
        crate::__ffi::ctr_qt_core_ffi_QString_toULongLong(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
            base,
        )
    }

    /// <p>Returns the string converted to an <code>unsigned long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qulonglong QString::toULongLong(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toULongLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>unsigned long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong">QLocale::toULongLong</a>()</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"FF"</span>;
    ///   bool ok;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#quint64-typedef">quint64</a></span> hex <span class="operator">=</span> str<span class="operator">.</span>toULongLong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);    <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#quint64-typedef">quint64</a></span> dec <span class="operator">=</span> str<span class="operator">.</span>toULongLong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);    <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toLongLong">toLongLong</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong">QLocale::toULongLong</a>().</p></div>
    pub unsafe fn to_u_long_long_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> u64 {
        crate::__ffi::ctr_qt_core_ffi_QString_toULongLong1(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the string converted to an <code>unsigned long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qulonglong QString::toULongLong() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toULongLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>unsigned long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong">QLocale::toULongLong</a>()</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"FF"</span>;
    ///   bool ok;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#quint64-typedef">quint64</a></span> hex <span class="operator">=</span> str<span class="operator">.</span>toULongLong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);    <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#quint64-typedef">quint64</a></span> dec <span class="operator">=</span> str<span class="operator">.</span>toULongLong(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);    <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toLongLong">toLongLong</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong">QLocale::toULongLong</a>().</p></div>
    pub unsafe fn to_u_long_long_0a(&self) -> u64 {
        crate::__ffi::ctr_qt_core_ffi_QString_toULongLong2(self as *const crate::QString)
    }

    /// <p>Returns the string converted to an <code>unsigned short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned short QString::toUShort(bool* ok = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toUShort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>unsigned short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toUShort">QLocale::toUShort</a>()</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"FF"</span>;
    ///   bool ok;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#ushort-typedef">ushort</a></span> hex <span class="operator">=</span> str<span class="operator">.</span>toUShort(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);     <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#ushort-typedef">ushort</a></span> dec <span class="operator">=</span> str<span class="operator">.</span>toUShort(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);     <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toShort">toShort</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toUShort">QLocale::toUShort</a>().</p></div>
    pub unsafe fn to_u_short_2a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
        base: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_ushort {
        crate::__ffi::ctr_qt_core_ffi_QString_toUShort(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
            base,
        )
    }

    /// <p>Returns the string converted to an <code>unsigned short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned short QString::toUShort(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toUShort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>unsigned short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toUShort">QLocale::toUShort</a>()</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"FF"</span>;
    ///   bool ok;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#ushort-typedef">ushort</a></span> hex <span class="operator">=</span> str<span class="operator">.</span>toUShort(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);     <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#ushort-typedef">ushort</a></span> dec <span class="operator">=</span> str<span class="operator">.</span>toUShort(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);     <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toShort">toShort</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toUShort">QLocale::toUShort</a>().</p></div>
    pub unsafe fn to_u_short_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_ushort {
        crate::__ffi::ctr_qt_core_ffi_QString_toUShort1(
            self as *const crate::QString,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the string converted to an <code>unsigned short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned short QString::toUShort() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toUShort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>unsigned short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toUShort">QLocale::toUShort</a>()</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"FF"</span>;
    ///   bool ok;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#ushort-typedef">ushort</a></span> hex <span class="operator">=</span> str<span class="operator">.</span>toUShort(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">16</span>);     <span class="comment">// hex == 255, ok == true</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#ushort-typedef">ushort</a></span> dec <span class="operator">=</span> str<span class="operator">.</span>toUShort(<span class="operator">&amp;</span>ok<span class="operator">,</span> <span class="number">10</span>);     <span class="comment">// dec == 0, ok == false</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#number">number</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toShort">toShort</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toUShort">QLocale::toUShort</a>().</p></div>
    pub unsafe fn to_u_short_0a(&self) -> ::std::os::raw::c_ushort {
        crate::__ffi::ctr_qt_core_ffi_QString_toUShort2(self as *const crate::QString)
    }

    /// <p>Returns a UCS-4/UTF-32 representation of the string as a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;uint&gt;.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<unsigned int> QString::toUcs4() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toUcs4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a UCS-4/UTF-32 representation of the string as a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;uint&gt;.</p>
    /// <p>UCS-4 is a Unicode codec and therefore it is lossless. All characters from this string will be encoded in UCS-4. Any invalid sequence of code units in this string is replaced by the Unicode's replacement character (<a href="http://doc.qt.io/qt-5/qchar.html#SpecialCharacter-enum">QChar::ReplacementCharacter</a>, which corresponds to <code>U+FFFD</code>).</p>
    /// <p>The returned vector is not NUL terminated.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">toUtf8</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toLatin1">toLatin1</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toLocal8Bit">toLocal8Bit</a>(), <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a>, <a href="http://doc.qt.io/qt-5/qstring.html#fromUcs4">fromUcs4</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#toWCharArray">toWCharArray</a>().</p></div>
    pub unsafe fn to_ucs4(&self) -> ::cpp_utils::CppBox<crate::QVectorOfUint> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_toUcs4(self as *const crate::QString);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns an uppercase copy of the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::toUpper() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toUpper">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an uppercase copy of the string.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"TeXt"</span>;
    ///   str <span class="operator">=</span> str<span class="operator">.</span>toUpper();        <span class="comment">// str == "TEXT"</span>
    ///
    /// </pre>
    /// <p>The case conversion will always happen in the 'C' locale. For locale dependent case folding use <a href="http://doc.qt.io/qt-5/qlocale.html#toUpper">QLocale::toUpper</a>()</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toLower">toLower</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toLower">QLocale::toLower</a>().</p></div>
    pub unsafe fn to_upper(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_toUpper(self as *const crate::QString);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns an uppercase copy of the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::toUpper()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toUpper">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an uppercase copy of the string.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"TeXt"</span>;
    ///   str <span class="operator">=</span> str<span class="operator">.</span>toUpper();        <span class="comment">// str == "TEXT"</span>
    ///
    /// </pre>
    /// <p>The case conversion will always happen in the 'C' locale. For locale dependent case folding use <a href="http://doc.qt.io/qt-5/qlocale.html#toUpper">QLocale::toUpper</a>()</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toLower">toLower</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toLower">QLocale::toLower</a>().</p></div>
    pub unsafe fn to_upper_mut(&mut self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_toUpper1(self as *mut crate::QString);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a UTF-8 representation of the string as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QString::toUtf8() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a UTF-8 representation of the string as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    /// <p>UTF-8 is a Unicode codec and can represent all characters in a Unicode string like <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toLatin1">toLatin1</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toLocal8Bit">toLocal8Bit</a>(), and <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a>.</p></div>
    pub unsafe fn to_utf8(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_toUtf8(self as *const crate::QString);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a UTF-8 representation of the string as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QString::toUtf8()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a UTF-8 representation of the string as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    /// <p>UTF-8 is a Unicode codec and can represent all characters in a Unicode string like <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toLatin1">toLatin1</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toLocal8Bit">toLocal8Bit</a>(), and <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a>.</p></div>
    pub unsafe fn to_utf8_mut(&mut self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_toUtf81(self as *mut crate::QString);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a string that has whitespace removed from the start and the end.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::trimmed() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#trimmed">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string that has whitespace removed from the start and the end.</p>
    /// <p>Whitespace means any character for which <a href="http://doc.qt.io/qt-5/qchar.html#isSpace">QChar::isSpace</a>() returns <code>true</code>. This includes the ASCII characters '\t', '\n', '\v', '\f', '\r', and ' '.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"  lots\t of\nwhitespace\r\n "</span>;
    ///   str <span class="operator">=</span> str<span class="operator">.</span>trimmed();
    ///   <span class="comment">// str == "lots\t of\nwhitespace"</span>
    ///
    /// </pre>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qstring.html#simplified">simplified</a>(), trimmed() leaves internal whitespace alone.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#simplified">simplified</a>().</p></div>
    pub unsafe fn trimmed(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_trimmed(self as *const crate::QString);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a string that has whitespace removed from the start and the end.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QString::trimmed()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#trimmed">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string that has whitespace removed from the start and the end.</p>
    /// <p>Whitespace means any character for which <a href="http://doc.qt.io/qt-5/qchar.html#isSpace">QChar::isSpace</a>() returns <code>true</code>. This includes the ASCII characters '\t', '\n', '\v', '\f', '\r', and ' '.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"  lots\t of\nwhitespace\r\n "</span>;
    ///   str <span class="operator">=</span> str<span class="operator">.</span>trimmed();
    ///   <span class="comment">// str == "lots\t of\nwhitespace"</span>
    ///
    /// </pre>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qstring.html#simplified">simplified</a>(), trimmed() leaves internal whitespace alone.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#simplified">simplified</a>().</p></div>
    pub unsafe fn trimmed_mut(&mut self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_trimmed1(self as *mut crate::QString);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Truncates the string at the given <i>position</i> index.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QString::truncate(int pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#truncate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Truncates the string at the given <i>position</i> index.</p>
    /// <p>If the specified <i>position</i> index is beyond the end of the string, nothing happens.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> str <span class="operator">=</span> <span class="string">"Vladivostok"</span>;
    ///   str<span class="operator">.</span>truncate(<span class="number">4</span>);
    ///   <span class="comment">// str == "Vlad"</span>
    ///
    /// </pre>
    /// <p>If <i>position</i> is negative, it is equivalent to passing zero.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#chop">chop</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#resize">resize</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#left">left</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#truncate">QStringRef::truncate</a>().</p></div>
    pub unsafe fn truncate(&mut self, pos: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QString_truncate(self as *mut crate::QString, pos)
    }

    /// <p>Returns a Unicode representation of the string. The result remains valid until the string is modified.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QChar* QString::unicode() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#unicode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a Unicode representation of the string. The result remains valid until the string is modified.</p>
    /// <p><b>Note: </b>The returned string may not be '\0'-terminated. Use <a href="http://doc.qt.io/qt-5/qstring.html#size">size</a>() to determine the length of the array.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#setUnicode">setUnicode</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#utf16">utf16</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#fromRawData">fromRawData</a>().</p></div>
    pub unsafe fn unicode(&self) -> ::cpp_utils::Ptr<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QString_unicode(self as *const crate::QString);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> as a '\0'-terminated array of unsigned shorts. The result remains valid until the string is modified.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const unsigned short* QString::utf16() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#utf16">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> as a '\0'-terminated array of unsigned shorts. The result remains valid until the string is modified.</p>
    /// <p>The returned string is in host byte order.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#setUtf16">setUtf16</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#unicode">unicode</a>().</p></div>
    pub unsafe fn utf16(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_ushort> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QString_utf16(self as *const crate::QString);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }
}

/// C++ class: <span style='color: green;'>```QCharRef```</span>.
#[repr(C)]
pub struct QCharRef {
    _unused: u8,
}
impl QCharRef {
    /// Calls C++ function: <span style='color: green;'>```QChar::Category QCharRef::category() const```</span>.
    pub unsafe fn category(&self) -> crate::q_char::Category {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_category(self as *const crate::QCharRef)
    }

    /// Calls C++ function: <span style='color: green;'>```unsigned char QCharRef::cell() const```</span>.
    pub unsafe fn cell(&self) -> ::std::os::raw::c_uchar {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_cell(self as *const crate::QCharRef)
    }

    /// Calls C++ function: <span style='color: green;'>```unsigned char QCharRef::combiningClass() const```</span>.
    pub unsafe fn combining_class(&self) -> ::std::os::raw::c_uchar {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_combiningClass(self as *const crate::QCharRef)
    }

    /// Calls C++ function: <span style='color: green;'>```QCharRef& QCharRef::operator=(QChar c)```</span>.
    pub unsafe fn copy_from_q_char(
        &mut self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::MutRef<crate::QCharRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCharRef_operator_(
            self as *mut crate::QCharRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```QCharRef& QCharRef::operator=(char c)```</span>.
    pub unsafe fn copy_from_char(
        &mut self,
        c: ::std::os::raw::c_char,
    ) -> ::cpp_utils::MutRef<crate::QCharRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCharRef_operator_1(self as *mut crate::QCharRef, c);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```QCharRef& QCharRef::operator=(unsigned char c)```</span>.
    pub unsafe fn copy_from_uchar(
        &mut self,
        c: ::std::os::raw::c_uchar,
    ) -> ::cpp_utils::MutRef<crate::QCharRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCharRef_operator_2(self as *mut crate::QCharRef, c);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```QCharRef& QCharRef::operator=(const QCharRef& c)```</span>.
    pub unsafe fn copy_from_q_char_ref(
        &mut self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCharRef>>,
    ) -> ::cpp_utils::MutRef<crate::QCharRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCharRef_operator_3(
            self as *mut crate::QCharRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCharRef>>::cast_into(c).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```QCharRef& QCharRef::operator=(unsigned short rc)```</span>.
    pub unsafe fn copy_from_ushort(
        &mut self,
        rc: ::std::os::raw::c_ushort,
    ) -> ::cpp_utils::MutRef<crate::QCharRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCharRef_operator_4(self as *mut crate::QCharRef, rc);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```QCharRef& QCharRef::operator=(short rc)```</span>.
    pub unsafe fn copy_from_short(
        &mut self,
        rc: ::std::os::raw::c_short,
    ) -> ::cpp_utils::MutRef<crate::QCharRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCharRef_operator_5(self as *mut crate::QCharRef, rc);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```QCharRef& QCharRef::operator=(unsigned int rc)```</span>.
    pub unsafe fn copy_from_uint(
        &mut self,
        rc: ::std::os::raw::c_uint,
    ) -> ::cpp_utils::MutRef<crate::QCharRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCharRef_operator_6(self as *mut crate::QCharRef, rc);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```QCharRef& QCharRef::operator=(int rc)```</span>.
    pub unsafe fn copy_from_int(
        &mut self,
        rc: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QCharRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCharRef_operator_7(self as *mut crate::QCharRef, rc);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```QString QCharRef::decomposition() const```</span>.
    pub unsafe fn decomposition(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCharRef_decomposition(self as *const crate::QCharRef);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QChar::Decomposition QCharRef::decompositionTag() const```</span>.
    pub unsafe fn decomposition_tag(&self) -> crate::q_char::Decomposition {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_decompositionTag(self as *const crate::QCharRef)
    }

    /// Calls C++ function: <span style='color: green;'>```int QCharRef::digitValue() const```</span>.
    pub unsafe fn digit_value(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_digitValue(self as *const crate::QCharRef)
    }

    /// Calls C++ function: <span style='color: green;'>```QChar::Direction QCharRef::direction() const```</span>.
    pub unsafe fn direction(&self) -> crate::q_char::Direction {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_direction(self as *const crate::QCharRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCharRef::hasMirrored() const```</span>.
    pub unsafe fn has_mirrored(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_hasMirrored(self as *const crate::QCharRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCharRef::isDigit() const```</span>.
    pub unsafe fn is_digit(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_isDigit(self as *const crate::QCharRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCharRef::isLetter() const```</span>.
    pub unsafe fn is_letter(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_isLetter(self as *const crate::QCharRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCharRef::isLetterOrNumber()```</span>.
    pub unsafe fn is_letter_or_number(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_isLetterOrNumber(self as *mut crate::QCharRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCharRef::isLower() const```</span>.
    pub unsafe fn is_lower(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_isLower(self as *const crate::QCharRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCharRef::isMark() const```</span>.
    pub unsafe fn is_mark(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_isMark(self as *const crate::QCharRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCharRef::isNull() const```</span>.
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_isNull(self as *const crate::QCharRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCharRef::isNumber() const```</span>.
    pub unsafe fn is_number(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_isNumber(self as *const crate::QCharRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCharRef::isPrint() const```</span>.
    pub unsafe fn is_print(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_isPrint(self as *const crate::QCharRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCharRef::isPunct() const```</span>.
    pub unsafe fn is_punct(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_isPunct(self as *const crate::QCharRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCharRef::isSpace() const```</span>.
    pub unsafe fn is_space(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_isSpace(self as *const crate::QCharRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCharRef::isTitleCase() const```</span>.
    pub unsafe fn is_title_case(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_isTitleCase(self as *const crate::QCharRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCharRef::isUpper() const```</span>.
    pub unsafe fn is_upper(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_isUpper(self as *const crate::QCharRef)
    }

    /// Calls C++ function: <span style='color: green;'>```QChar::Joining QCharRef::joining() const```</span>.
    pub unsafe fn joining(&self) -> crate::q_char::Joining {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_joining(self as *const crate::QCharRef)
    }

    /// Calls C++ function: <span style='color: green;'>```QChar::JoiningType QCharRef::joiningType() const```</span>.
    pub unsafe fn joining_type(&self) -> crate::q_char::JoiningType {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_joiningType(self as *const crate::QCharRef)
    }

    /// Calls C++ function: <span style='color: green;'>```QChar QCharRef::mirroredChar() const```</span>.
    pub unsafe fn mirrored_char(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCharRef_mirroredChar(self as *const crate::QCharRef);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCharRef::QCharRef(const QCharRef& other)```</span>.
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCharRef>>,
    ) -> ::cpp_utils::CppBox<crate::QCharRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCharRef_QCharRef1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCharRef>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```unsigned char QCharRef::row() const```</span>.
    pub unsafe fn row(&self) -> ::std::os::raw::c_uchar {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_row(self as *const crate::QCharRef)
    }

    /// Calls C++ function: <span style='color: green;'>```QChar::Script QCharRef::script() const```</span>.
    pub unsafe fn script(&self) -> crate::q_char::Script {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_script(self as *const crate::QCharRef)
    }

    /// Calls C++ function: <span style='color: green;'>```void QCharRef::setCell(unsigned char cell)```</span>.
    pub unsafe fn set_cell(&mut self, cell: ::std::os::raw::c_uchar) {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_setCell(self as *mut crate::QCharRef, cell)
    }

    /// Calls C++ function: <span style='color: green;'>```void QCharRef::setRow(unsigned char row)```</span>.
    pub unsafe fn set_row(&mut self, row: ::std::os::raw::c_uchar) {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_setRow(self as *mut crate::QCharRef, row)
    }

    /// Calls C++ function: <span style='color: green;'>```char QCharRef::toLatin1() const```</span>.
    pub unsafe fn to_latin1(&self) -> ::std::os::raw::c_char {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_toLatin1(self as *const crate::QCharRef)
    }

    /// Calls C++ function: <span style='color: green;'>```QChar QCharRef::toLower() const```</span>.
    pub unsafe fn to_lower(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCharRef_toLower(self as *const crate::QCharRef);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QChar QCharRef::operator QChar() const```</span>.
    pub unsafe fn to_q_char(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCharRef_operator_QChar(self as *const crate::QCharRef);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QChar QCharRef::toTitleCase() const```</span>.
    pub unsafe fn to_title_case(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCharRef_toTitleCase(self as *const crate::QCharRef);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QChar QCharRef::toUpper() const```</span>.
    pub unsafe fn to_upper(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCharRef_toUpper(self as *const crate::QCharRef);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```unsigned short QCharRef::unicode() const```</span>.
    pub unsafe fn unicode(&self) -> ::std::os::raw::c_ushort {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_unicode(self as *const crate::QCharRef)
    }

    /// Calls C++ function: <span style='color: green;'>```unsigned short& QCharRef::unicode()```</span>.
    pub unsafe fn unicode_mut(&mut self) -> ::cpp_utils::MutRef<::std::os::raw::c_ushort> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCharRef_unicode1(self as *mut crate::QCharRef);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```QChar::UnicodeVersion QCharRef::unicodeVersion() const```</span>.
    pub unsafe fn unicode_version(&self) -> crate::q_char::UnicodeVersion {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_unicodeVersion(self as *const crate::QCharRef)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a> class provides a thin wrapper around <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> substrings.</p>
///
/// C++ class: <span style='color: green;'>```QStringRef```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qstringref.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a> class provides a thin wrapper around <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> substrings.</p>
/// <p><a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a> provides a read-only subset of the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> API.</p>
/// <p>A string reference explicitly references a portion of a <a href="http://doc.qt.io/qt-5/qstringref.html#string">string</a>() with a given <a href="http://doc.qt.io/qt-5/qstringref.html#size">size</a>(), starting at a specific <a href="http://doc.qt.io/qt-5/qstringref.html#position">position</a>(). Calling <a href="http://doc.qt.io/qt-5/qstringref.html#toString">toString</a>() returns a copy of the data as a real <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> instance.</p>
/// <p>This class is designed to improve the performance of substring handling when manipulating substrings obtained from existing <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> instances. <a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a> avoids the memory allocation and reference counting overhead of a standard <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> by simply referencing a part of the original string. This can prove to be advantageous in low level code, such as that used in a parser, at the expense of potentially more complex code.</p>
/// <p>For most users, there are no semantic benefits to using <a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a> instead of <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> since <a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a> requires attention to be paid to memory management issues, potentially making code more complex to write and maintain.</p>
/// <p><b>Warning:</b> A <a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a> is only valid as long as the referenced string exists. If the original string is deleted, the string reference points to an invalid memory location.</p>
/// <p>We suggest that you only use this class in stable code where profiling has clearly identified that performance improvements can be made by replacing standard string operations with the optimized substring handling provided by this class.</p></div>
#[repr(C)]
pub struct QStringRef {
    _unused: u8,
}
impl QStringRef {
    /// <p>Appends the string reference to <i>string</i>, and returns a new reference to the combined string data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QStringRef::appendTo(QString* string) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#appendTo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the string reference to <i>string</i>, and returns a new reference to the combined string data.</p></div>
    pub unsafe fn append_to(
        &self,
        string: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringRef_appendTo(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QString>>::cast_into(string)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the character at the given index <i>position</i> in the string reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QStringRef::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the character at the given index <i>position</i> in the string reference.</p>
    /// <p>The <i>position</i> must be a valid index position in the string (i.e., 0 &lt;= <i>position</i> &lt; <a href="http://doc.qt.io/qt-5/qstringref.html#size">size</a>()).</p></div>
    pub unsafe fn at(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringRef_at(self as *const crate::QStringRef, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the last character in the string. Same as <code>at(size() - 1)</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QStringRef::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the last character in the string. Same as <code>at(size() - 1)</code>.</p>
    /// <p>This function is provided for STL compatibility.</p>
    /// <p><b>Warning:</b> Calling this function on an empty string constitutes undefined behavior.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#front">front</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#at">at</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#operator-5b-5d">operator[]</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn back(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringRef_back(self as *const crate::QStringRef);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Same as <a href="http://doc.qt.io/qt-5/qstringref.html#begin">begin</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QChar* QStringRef::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as <a href="http://doc.qt.io/qt-5/qstringref.html#begin">begin</a>().</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#begin">begin</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#constBegin">constBegin</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#cend">cend</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#constEnd">constEnd</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#rbegin">rbegin</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#rend">rend</a>().</p></div>
    pub unsafe fn cbegin(&self) -> ::cpp_utils::Ptr<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringRef_cbegin(self as *const crate::QStringRef);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Same as <a href="http://doc.qt.io/qt-5/qstringref.html#end">end</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QChar* QStringRef::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as <a href="http://doc.qt.io/qt-5/qstringref.html#end">end</a>().</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#end">end</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#constEnd">constEnd</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#cbegin">cbegin</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#constBegin">constBegin</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#rbegin">rbegin</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#rend">rend</a>().</p></div>
    pub unsafe fn cend(&self) -> ::cpp_utils::Ptr<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringRef_cend(self as *const crate::QStringRef);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Removes <i>n</i> characters from the end of the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QStringRef::chop(int n)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#chop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes <i>n</i> characters from the end of the string.</p>
    /// <p>If <i>n</i> is greater than or equal to <a href="http://doc.qt.io/qt-5/qstringref.html#size">size</a>(), the result is an empty string; if <i>n</i> is negative, it is equivalent to passing zero.</p>
    /// <p>This function was introduced in  Qt 5.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#chop">QString::chop</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#truncate">truncate</a>().</p></div>
    pub unsafe fn chop(&mut self, n: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_chop(self as *mut crate::QStringRef, n)
    }

    /// <p>Returns a substring reference to the <a href="http://doc.qt.io/qt-5/qstringref.html#size">size</a>() - <i>len</i> leftmost characters of this string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QStringRef::chopped(int n) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#chopped">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a substring reference to the <a href="http://doc.qt.io/qt-5/qstringref.html#size">size</a>() - <i>len</i> leftmost characters of this string.</p>
    /// <p><b>Note: </b>The behavior is undefined if <i>len</i> is negative or greater than <a href="http://doc.qt.io/qt-5/qstringref.html#size">size</a>().</p><p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#left">left</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#right">right</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#mid">mid</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#chop">chop</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#truncate">truncate</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn chopped(
        &self,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringRef_chopped(self as *const crate::QStringRef, n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Clears the contents of the string reference by making it null and empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QStringRef::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Clears the contents of the string reference by making it null and empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#isNull">isNull</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_clear(self as *mut crate::QStringRef)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::compare(const QString& s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#compare-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Compares this string with the <i>other</i> string and returns an integer less than, equal to, or greater than zero if this string is less than, equal to, or greater than the <i>other</i> string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a>, the comparison is case sensitive; otherwise the comparison is case insensitive.</p>
    /// <p>Equivalent to <code>compare(*this, other, cs)</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#compare">QString::compare</a>().</p></div>
    pub unsafe fn compare_q_string_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_compare(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            cs,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::compare(const QStringRef& s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#compare-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Compares this string with the <i>other</i> string and returns an integer less than, equal to, or greater than zero if this string is less than, equal to, or greater than the <i>other</i> string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a>, the comparison is case sensitive; otherwise the comparison is case insensitive.</p>
    /// <p>Equivalent to <code>compare(*this, other, cs)</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#compare">QString::compare</a>().</p></div>
    pub unsafe fn compare_q_string_ref_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_compare1(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
            cs,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::compare(QLatin1String s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#compare-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Compares this string with the <i>other</i> string and returns an integer less than, equal to, or greater than zero if this string is less than, equal to, or greater than the <i>other</i> string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a>, the comparison is case sensitive; otherwise the comparison is case insensitive.</p>
    /// <p>Equivalent to <code>compare(*this, other, cs)</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#compare">QString::compare</a>().</p></div>
    pub unsafe fn compare_q_latin1_string_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_compare2(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
            cs,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::compare(const QByteArray& s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#compare-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Compares this string with <i>other</i> and returns an integer less than, equal to, or greater than zero if this string is less than, equal to, or greater than the <i>other</i> byte array, interpreted as a UTF-8 sequence.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a>, the comparison is case sensitive; otherwise the comparison is case insensitive.</p>
    /// <p>Equivalent to <code>compare(*this, other, cs)</code>.</p>
    /// <p>This function was introduced in  Qt 5.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#compare">QString::compare</a>().</p></div>
    pub unsafe fn compare_q_byte_array_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_compare3(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(s).as_raw_ptr(),
            cs,
        )
    }

    /// <p>Compares the string <i>s1</i> with the string <i>s2</i> and returns an integer less than, equal to, or greater than zero if <i>s1</i> is less than, equal to, or greater than <i>s2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QStringRef::compare(const QStringRef& s1, const QString& s2, Qt::CaseSensitivity arg3 = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#compare">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares the string <i>s1</i> with the string <i>s2</i> and returns an integer less than, equal to, or greater than zero if <i>s1</i> is less than, equal to, or greater than <i>s2</i>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a>, the comparison is case sensitive; otherwise the comparison is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn compare_q_string_ref_q_string_case_sensitivity(
        s1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        s2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        arg3: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_compare4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s1)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s2).as_raw_ptr(),
            arg3,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QStringRef::compare(const QStringRef& s1, const QStringRef& s2, Qt::CaseSensitivity arg3 = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#compare-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Compares the string <i>s1</i> with the string <i>s2</i> and returns an integer less than, equal to, or greater than zero if <i>s1</i> is less than, equal to, or greater than <i>s2</i>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a>, the comparison is case sensitive; otherwise the comparison is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn compare_2_q_string_ref_case_sensitivity(
        s1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        s2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        arg3: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_compare5(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s1)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s2)
                .as_raw_ptr(),
            arg3,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QStringRef::compare(const QStringRef& s1, QLatin1String s2, Qt::CaseSensitivity cs = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#compare-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Compares the string <i>s1</i> with the string <i>s2</i> and returns an integer less than, equal to, or greater than zero if <i>s1</i> is less than, equal to, or greater than <i>s2</i>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a>, the comparison is case sensitive; otherwise the comparison is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn compare_q_string_ref_q_latin1_string_case_sensitivity(
        s1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        s2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_compare6(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s1)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s2)
                .as_raw_ptr(),
            cs,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::compare(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#compare-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Compares this string with the <i>other</i> string and returns an integer less than, equal to, or greater than zero if this string is less than, equal to, or greater than the <i>other</i> string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a>, the comparison is case sensitive; otherwise the comparison is case insensitive.</p>
    /// <p>Equivalent to <code>compare(*this, other, cs)</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#compare">QString::compare</a>().</p></div>
    pub unsafe fn compare_q_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_compare7(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::compare(const QStringRef& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#compare-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Compares this string with the <i>other</i> string and returns an integer less than, equal to, or greater than zero if this string is less than, equal to, or greater than the <i>other</i> string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a>, the comparison is case sensitive; otherwise the comparison is case insensitive.</p>
    /// <p>Equivalent to <code>compare(*this, other, cs)</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#compare">QString::compare</a>().</p></div>
    pub unsafe fn compare_q_string_ref(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_compare8(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::compare(QLatin1String s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#compare-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Compares this string with the <i>other</i> string and returns an integer less than, equal to, or greater than zero if this string is less than, equal to, or greater than the <i>other</i> string.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a>, the comparison is case sensitive; otherwise the comparison is case insensitive.</p>
    /// <p>Equivalent to <code>compare(*this, other, cs)</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#compare">QString::compare</a>().</p></div>
    pub unsafe fn compare_q_latin1_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_compare9(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::compare(const QByteArray& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#compare-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Compares this string with <i>other</i> and returns an integer less than, equal to, or greater than zero if this string is less than, equal to, or greater than the <i>other</i> byte array, interpreted as a UTF-8 sequence.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a>, the comparison is case sensitive; otherwise the comparison is case insensitive.</p>
    /// <p>Equivalent to <code>compare(*this, other, cs)</code>.</p>
    /// <p>This function was introduced in  Qt 5.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#compare">QString::compare</a>().</p></div>
    pub unsafe fn compare_q_byte_array(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_compare10(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Compares the string <i>s1</i> with the string <i>s2</i> and returns an integer less than, equal to, or greater than zero if <i>s1</i> is less than, equal to, or greater than <i>s2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QStringRef::compare(const QStringRef& s1, const QString& s2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#compare">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares the string <i>s1</i> with the string <i>s2</i> and returns an integer less than, equal to, or greater than zero if <i>s1</i> is less than, equal to, or greater than <i>s2</i>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a>, the comparison is case sensitive; otherwise the comparison is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn compare_q_string_ref_q_string(
        s1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        s2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_compare11(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s1)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s2).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QStringRef::compare(const QStringRef& s1, const QStringRef& s2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#compare-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Compares the string <i>s1</i> with the string <i>s2</i> and returns an integer less than, equal to, or greater than zero if <i>s1</i> is less than, equal to, or greater than <i>s2</i>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a>, the comparison is case sensitive; otherwise the comparison is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn compare_2_q_string_ref(
        s1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        s2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_compare12(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s1)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s2)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QStringRef::compare(const QStringRef& s1, QLatin1String s2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#compare-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Compares the string <i>s1</i> with the string <i>s2</i> and returns an integer less than, equal to, or greater than zero if <i>s1</i> is less than, equal to, or greater than <i>s2</i>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a>, the comparison is case sensitive; otherwise the comparison is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn compare_q_string_ref_q_latin1_string(
        s1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        s2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_compare13(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s1)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s2)
                .as_raw_ptr(),
        )
    }

    /// <p>Same as <a href="http://doc.qt.io/qt-5/qstringref.html#begin">begin</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QChar* QStringRef::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as <a href="http://doc.qt.io/qt-5/qstringref.html#begin">begin</a>().</p>
    /// <p>This function was introduced in  Qt 5.9.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#begin">begin</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#cend">cend</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#constEnd">constEnd</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#rbegin">rbegin</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#rend">rend</a>().</p></div>
    pub unsafe fn const_begin(&self) -> ::cpp_utils::Ptr<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringRef_constBegin(self as *const crate::QStringRef);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Same as <a href="http://doc.qt.io/qt-5/qstringref.html#unicode">unicode</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QChar* QStringRef::constData() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#constData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as <a href="http://doc.qt.io/qt-5/qstringref.html#unicode">unicode</a>().</p></div>
    pub unsafe fn const_data(&self) -> ::cpp_utils::Ptr<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringRef_constData(self as *const crate::QStringRef);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Same as <a href="http://doc.qt.io/qt-5/qstringref.html#end">end</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QChar* QStringRef::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as <a href="http://doc.qt.io/qt-5/qstringref.html#end">end</a>().</p>
    /// <p>This function was introduced in  Qt 5.9.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#end">end</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#cend">cend</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#cbegin">cbegin</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#constBegin">constBegin</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#rbegin">rbegin</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#rend">rend</a>().</p></div>
    pub unsafe fn const_end(&self) -> ::cpp_utils::Ptr<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringRef_constEnd(self as *const crate::QStringRef);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns <code>true</code> if this string reference contains an occurrence of the string <i>str</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::contains(const QString& str, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this string reference contains an occurrence of the string <i>str</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn contains_q_string_case_sensitivity(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_contains(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(str).as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::contains(QChar ch, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#contains-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>().</p>
    /// <p>Returns <code>true</code> if this string contains an occurrence of the character <i>ch</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn contains_q_char_case_sensitivity(
        &self,
        ch: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_contains1(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(ch).as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::contains(QLatin1String str, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#contains-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>().</p>
    /// <p>Returns <code>true</code> if this string reference contains an occurrence of the string <i>str</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn contains_q_latin1_string_case_sensitivity(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_contains2(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(str)
                .as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::contains(const QStringRef& str, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#contains-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>().</p>
    /// <p>Returns <code>true</code> if this string reference contains an occurrence of the string reference <i>str</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn contains_q_string_ref_case_sensitivity(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_contains3(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(str)
                .as_raw_ptr(),
            cs,
        )
    }

    /// <p>Returns <code>true</code> if this string reference contains an occurrence of the string <i>str</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::contains(const QString& str) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this string reference contains an occurrence of the string <i>str</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn contains_q_string(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_contains4(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(str).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::contains(QChar ch) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#contains-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>().</p>
    /// <p>Returns <code>true</code> if this string contains an occurrence of the character <i>ch</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn contains_q_char(
        &self,
        ch: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_contains5(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(ch).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::contains(QLatin1String str) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#contains-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>().</p>
    /// <p>Returns <code>true</code> if this string reference contains an occurrence of the string <i>str</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn contains_q_latin1_string(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_contains6(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(str)
                .as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::contains(const QStringRef& str) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#contains-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>().</p>
    /// <p>Returns <code>true</code> if this string reference contains an occurrence of the string reference <i>str</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn contains_q_string_ref(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_contains7(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(str)
                .as_raw_ptr(),
        )
    }

    /// <p>Assigns the <i>other</i> string reference to this string reference, and returns the result.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef& QStringRef::operator=(const QStringRef& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#operator-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns the <i>other</i> string reference to this string reference, and returns the result.</p></div>
    pub unsafe fn copy_from_q_string_ref(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::cpp_utils::MutRef<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringRef_operator_1(
            self as *mut crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Constructs a string reference to the given <i>string</i> and assigns it to this string reference, returning the result.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef& QStringRef::operator=(const QString* string)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#operator-eq-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a string reference to the given <i>string</i> and assigns it to this string reference, returning the result.</p></div>
    pub unsafe fn copy_from_q_string(
        &mut self,
        string: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QString>>,
    ) -> ::cpp_utils::MutRef<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringRef_operator_2(
            self as *mut crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QString>>::cast_into(string)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of characters referred to by the string reference. Equivalent to <a href="http://doc.qt.io/qt-5/qstringref.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#length">length</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of characters referred to by the string reference. Equivalent to <a href="http://doc.qt.io/qt-5/qstringref.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#length">length</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#position">position</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#string">string</a>().</p></div>
    pub unsafe fn count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_count(self as *const crate::QStringRef)
    }

    /// <p>Returns the number of (potentially overlapping) occurrences of the string <i>str</i> in this string reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::count(const QString& s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of (potentially overlapping) occurrences of the string <i>str</i> in this string reference.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#count-2">QString::count</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_q_string_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_count1(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::count(QChar c, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#count-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p>
    /// <p>Returns the number of occurrences of the character <i>ch</i> in the string reference.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#count-2">QString::count</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_q_char_case_sensitivity(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_count2(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::count(const QStringRef& s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#count-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p>
    /// <p>Returns the number of (potentially overlapping) occurrences of the string reference <i>str</i> in this string reference.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#count-2">QString::count</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_q_string_ref_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_count3(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
            cs,
        )
    }

    /// <p>Returns the number of (potentially overlapping) occurrences of the string <i>str</i> in this string reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::count(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of (potentially overlapping) occurrences of the string <i>str</i> in this string reference.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#count-2">QString::count</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_q_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_count4(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::count(QChar c) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#count-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p>
    /// <p>Returns the number of occurrences of the character <i>ch</i> in the string reference.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#count-2">QString::count</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_q_char(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_count5(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::count(const QStringRef& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#count-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p>
    /// <p>Returns the number of (potentially overlapping) occurrences of the string reference <i>str</i> in this string reference.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#count-2">QString::count</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_q_string_ref(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_count6(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Same as <a href="http://doc.qt.io/qt-5/qstringref.html#unicode">unicode</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QChar* QStringRef::data() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as <a href="http://doc.qt.io/qt-5/qstringref.html#unicode">unicode</a>().</p></div>
    pub unsafe fn data(&self) -> ::cpp_utils::Ptr<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringRef_data(self as *const crate::QStringRef);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns <code>true</code> if the string reference ends with <i>str</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::endsWith(const QString& s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the string reference ends with <i>str</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#endsWith">QString::endsWith</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith">startsWith</a>().</p></div>
    pub unsafe fn ends_with_q_string_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_endsWith(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::endsWith(QLatin1String s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#endsWith">QString::endsWith</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p></div>
    pub unsafe fn ends_with_q_latin1_string_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_endsWith1(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::endsWith(QChar c, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p>
    /// <p>Returns <code>true</code> if the string reference ends with <i>ch</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#endsWith">QString::endsWith</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p></div>
    pub unsafe fn ends_with_q_char_case_sensitivity(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_endsWith2(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::endsWith(const QStringRef& c, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#endsWith">QString::endsWith</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p></div>
    pub unsafe fn ends_with_q_string_ref_case_sensitivity(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_endsWith3(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(c).as_raw_ptr(),
            cs,
        )
    }

    /// <p>Returns <code>true</code> if the string reference ends with <i>str</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::endsWith(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the string reference ends with <i>str</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#endsWith">QString::endsWith</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith">startsWith</a>().</p></div>
    pub unsafe fn ends_with_q_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_endsWith4(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::endsWith(QLatin1String s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#endsWith">QString::endsWith</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p></div>
    pub unsafe fn ends_with_q_latin1_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_endsWith5(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::endsWith(QChar c) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p>
    /// <p>Returns <code>true</code> if the string reference ends with <i>ch</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#endsWith">QString::endsWith</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p></div>
    pub unsafe fn ends_with_q_char(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_endsWith6(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::endsWith(const QStringRef& c) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#endsWith">QString::endsWith</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p></div>
    pub unsafe fn ends_with_q_string_ref(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_endsWith7(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(c).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::endsWith(QStringView s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#endsWith">QString::endsWith</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith">startsWith</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn ends_with_q_string_view_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_endsWith16(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::endsWith(QStringView s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#endsWith">QString::endsWith</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith">startsWith</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn ends_with_q_string_view(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_endsWith17(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the first character in the string. Same as <code>at(0)</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QStringRef::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the first character in the string. Same as <code>at(0)</code>.</p>
    /// <p>This function is provided for STL compatibility.</p>
    /// <p><b>Warning:</b> Calling this function on an empty string constitutes undefined behavior.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#back">back</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#at">at</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#operator-5b-5d">operator[]</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn front(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringRef_front(self as *const crate::QStringRef);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the character at the given index <i>position</i> in the string reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QStringRef::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the character at the given index <i>position</i> in the string reference.</p>
    /// <p>The <i>position</i> must be a valid index position in the string reference (i.e., 0 &lt;= <i>position</i> &lt; <a href="http://doc.qt.io/qt-5/qstringref.html#size">size</a>()).</p>
    /// <p>This function was introduced in  Qt 5.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#at">at</a>().</p></div>
    pub unsafe fn index(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringRef_operator__(
            self as *const crate::QStringRef,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the index position of the first occurrence of the string <i>str</i> in this string reference, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::indexOf(const QString& str, int from = …, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of the string <i>str</i> in this string reference, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>If <i>from</i> is -1, the search starts at the last character; if it is -2, at the next to last character and so on.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">QString::indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn index_of_q_string_int_case_sensitivity(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        from: ::std::os::raw::c_int,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_indexOf(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(str).as_raw_ptr(),
            from,
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::indexOf(QChar ch, int from = …, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>().</p>
    /// <p>Returns the index position of the first occurrence of the character <i>ch</i> in the string reference, searching forward from index position <i>from</i>. Returns -1 if <i>ch</i> could not be found.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">QString::indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn index_of_q_char_int_case_sensitivity(
        &self,
        ch: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        from: ::std::os::raw::c_int,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_indexOf1(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(ch).as_raw_ptr(),
            from,
            cs,
        )
    }

    /// <p>Returns the index position of the first occurrence of the string <i>str</i> in this string reference, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::indexOf(QLatin1String str, int from = …, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of the string <i>str</i> in this string reference, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>If <i>from</i> is -1, the search starts at the last character; if it is -2, at the next to last character and so on.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">QString::indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn index_of_q_latin1_string_int_case_sensitivity(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        from: ::std::os::raw::c_int,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_indexOf2(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(str)
                .as_raw_ptr(),
            from,
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::indexOf(const QStringRef& str, int from = …, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>().</p>
    /// <p>Returns the index position of the first occurrence of the string reference <i>str</i> in this string reference, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">QString::indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn index_of_q_string_ref_int_case_sensitivity(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        from: ::std::os::raw::c_int,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_indexOf3(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(str)
                .as_raw_ptr(),
            from,
            cs,
        )
    }

    /// <p>Returns the index position of the first occurrence of the string <i>str</i> in this string reference, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::indexOf(const QString& str, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of the string <i>str</i> in this string reference, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>If <i>from</i> is -1, the search starts at the last character; if it is -2, at the next to last character and so on.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">QString::indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn index_of_q_string_int(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_indexOf4(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(str).as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first occurrence of the string <i>str</i> in this string reference, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::indexOf(const QString& str) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of the string <i>str</i> in this string reference, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>If <i>from</i> is -1, the search starts at the last character; if it is -2, at the next to last character and so on.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">QString::indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn index_of_q_string(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_indexOf5(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(str).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::indexOf(QChar ch, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>().</p>
    /// <p>Returns the index position of the first occurrence of the character <i>ch</i> in the string reference, searching forward from index position <i>from</i>. Returns -1 if <i>ch</i> could not be found.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">QString::indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn index_of_q_char_int(
        &self,
        ch: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_indexOf6(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(ch).as_raw_ptr(),
            from,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::indexOf(QChar ch) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>().</p>
    /// <p>Returns the index position of the first occurrence of the character <i>ch</i> in the string reference, searching forward from index position <i>from</i>. Returns -1 if <i>ch</i> could not be found.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">QString::indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn index_of_q_char(
        &self,
        ch: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_indexOf7(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(ch).as_raw_ptr(),
        )
    }

    /// <p>Returns the index position of the first occurrence of the string <i>str</i> in this string reference, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::indexOf(QLatin1String str, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of the string <i>str</i> in this string reference, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>If <i>from</i> is -1, the search starts at the last character; if it is -2, at the next to last character and so on.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">QString::indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn index_of_q_latin1_string_int(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_indexOf8(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(str)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first occurrence of the string <i>str</i> in this string reference, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::indexOf(QLatin1String str) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of the string <i>str</i> in this string reference, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>If <i>from</i> is -1, the search starts at the last character; if it is -2, at the next to last character and so on.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">QString::indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn index_of_q_latin1_string(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_indexOf9(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(str)
                .as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::indexOf(const QStringRef& str, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>().</p>
    /// <p>Returns the index position of the first occurrence of the string reference <i>str</i> in this string reference, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">QString::indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn index_of_q_string_ref_int(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_indexOf10(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(str)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::indexOf(const QStringRef& str) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>().</p>
    /// <p>Returns the index position of the first occurrence of the string reference <i>str</i> in this string reference, searching forward from index position <i>from</i>. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#indexOf">QString::indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn index_of_q_string_ref(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_indexOf11(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(str)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if the string reference has no characters; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the string reference has no characters; otherwise returns <code>false</code>.</p>
    /// <p>A string reference is empty if its size is zero.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_isEmpty(self as *const crate::QStringRef)
    }

    /// <p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qstringref.html#string">string</a>() returns a null pointer or a pointer to a null string; otherwise returns <code>true</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::isNull() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#isNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qstringref.html#string">string</a>() returns a null pointer or a pointer to a null string; otherwise returns <code>true</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#size">size</a>().</p></div>
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_isNull(self as *const crate::QStringRef)
    }

    /// <p>Returns <code>true</code> if the string is read right to left.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::isRightToLeft() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#isRightToLeft">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the string is read right to left.</p>
    /// <p>This function was introduced in  Qt 5.9.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#isRightToLeft">QString::isRightToLeft</a>().</p></div>
    pub unsafe fn is_right_to_left(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_isRightToLeft(self as *const crate::QStringRef)
    }

    /// <p>Returns the index position of the last occurrence of the string <i>str</i> in this string reference, searching backward from index position <i>from</i>. If <i>from</i> is -1 (default), the search starts at the last character; if <i>from</i> is -2, at the next to last character and so on. Returns -1 if <i>str</i> is not found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::lastIndexOf(const QString& str, int from = …, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the string <i>str</i> in this string reference, searching backward from index position <i>from</i>. If <i>from</i> is -1 (default), the search starts at the last character; if <i>from</i> is -2, at the next to last character and so on. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">QString::lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn last_index_of_q_string_int_case_sensitivity(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        from: ::std::os::raw::c_int,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_lastIndexOf(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(str).as_raw_ptr(),
            from,
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::lastIndexOf(QChar ch, int from = …, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>().</p>
    /// <p>Returns the index position of the last occurrence of the character <i>ch</i>, searching backward from position <i>from</i>.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">QString::lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn last_index_of_q_char_int_case_sensitivity(
        &self,
        ch: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        from: ::std::os::raw::c_int,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_lastIndexOf1(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(ch).as_raw_ptr(),
            from,
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::lastIndexOf(QLatin1String str, int from = …, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>().</p>
    /// <p>Returns the index position of the last occurrence of the string <i>str</i> in this string reference, searching backward from index position <i>from</i>. If <i>from</i> is -1 (default), the search starts at the last character; if <i>from</i> is -2, at the next to last character and so on. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">QString::lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn last_index_of_q_latin1_string_int_case_sensitivity(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        from: ::std::os::raw::c_int,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_lastIndexOf2(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(str)
                .as_raw_ptr(),
            from,
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::lastIndexOf(const QStringRef& str, int from = …, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>().</p>
    /// <p>Returns the index position of the last occurrence of the string reference <i>str</i> in this string reference, searching backward from index position <i>from</i>. If <i>from</i> is -1 (default), the search starts at the last character; if <i>from</i> is -2, at the next to last character and so on. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">QString::lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn last_index_of_q_string_ref_int_case_sensitivity(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        from: ::std::os::raw::c_int,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_lastIndexOf3(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(str)
                .as_raw_ptr(),
            from,
            cs,
        )
    }

    /// <p>Returns the index position of the last occurrence of the string <i>str</i> in this string reference, searching backward from index position <i>from</i>. If <i>from</i> is -1 (default), the search starts at the last character; if <i>from</i> is -2, at the next to last character and so on. Returns -1 if <i>str</i> is not found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::lastIndexOf(const QString& str, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the string <i>str</i> in this string reference, searching backward from index position <i>from</i>. If <i>from</i> is -1 (default), the search starts at the last character; if <i>from</i> is -2, at the next to last character and so on. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">QString::lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn last_index_of_q_string_int(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_lastIndexOf4(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(str).as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the last occurrence of the string <i>str</i> in this string reference, searching backward from index position <i>from</i>. If <i>from</i> is -1 (default), the search starts at the last character; if <i>from</i> is -2, at the next to last character and so on. Returns -1 if <i>str</i> is not found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::lastIndexOf(const QString& str) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the string <i>str</i> in this string reference, searching backward from index position <i>from</i>. If <i>from</i> is -1 (default), the search starts at the last character; if <i>from</i> is -2, at the next to last character and so on. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">QString::lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn last_index_of_q_string(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_lastIndexOf5(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(str).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::lastIndexOf(QChar ch, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>().</p>
    /// <p>Returns the index position of the last occurrence of the character <i>ch</i>, searching backward from position <i>from</i>.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">QString::lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn last_index_of_q_char_int(
        &self,
        ch: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_lastIndexOf6(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(ch).as_raw_ptr(),
            from,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::lastIndexOf(QChar ch) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>().</p>
    /// <p>Returns the index position of the last occurrence of the character <i>ch</i>, searching backward from position <i>from</i>.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">QString::lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn last_index_of_q_char(
        &self,
        ch: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_lastIndexOf7(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(ch).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::lastIndexOf(QLatin1String str, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>().</p>
    /// <p>Returns the index position of the last occurrence of the string <i>str</i> in this string reference, searching backward from index position <i>from</i>. If <i>from</i> is -1 (default), the search starts at the last character; if <i>from</i> is -2, at the next to last character and so on. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">QString::lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn last_index_of_q_latin1_string_int(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_lastIndexOf8(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(str)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::lastIndexOf(QLatin1String str) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>().</p>
    /// <p>Returns the index position of the last occurrence of the string <i>str</i> in this string reference, searching backward from index position <i>from</i>. If <i>from</i> is -1 (default), the search starts at the last character; if <i>from</i> is -2, at the next to last character and so on. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">QString::lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn last_index_of_q_latin1_string(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_lastIndexOf9(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(str)
                .as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::lastIndexOf(const QStringRef& str, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>().</p>
    /// <p>Returns the index position of the last occurrence of the string reference <i>str</i> in this string reference, searching backward from index position <i>from</i>. If <i>from</i> is -1 (default), the search starts at the last character; if <i>from</i> is -2, at the next to last character and so on. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">QString::lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn last_index_of_q_string_ref_int(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_lastIndexOf10(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(str)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::lastIndexOf(const QStringRef& str) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#lastIndexOf">lastIndexOf</a>().</p>
    /// <p>Returns the index position of the last occurrence of the string reference <i>str</i> in this string reference, searching backward from index position <i>from</i>. If <i>from</i> is -1 (default), the search starts at the last character; if <i>from</i> is -2, at the next to last character and so on. Returns -1 if <i>str</i> is not found.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#lastIndexOf">QString::lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p></div>
    pub unsafe fn last_index_of_q_string_ref(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_lastIndexOf11(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(str)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a substring reference to the <i>n</i> leftmost characters of the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QStringRef::left(int n) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#left">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a substring reference to the <i>n</i> leftmost characters of the string.</p>
    /// <p>If <i>n</i> is greater than or equal to <a href="http://doc.qt.io/qt-5/qstringref.html#size">size</a>(), or less than zero, a reference to the entire string is returned.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#right">right</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#mid">mid</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith">startsWith</a>().</p></div>
    pub unsafe fn left(&self, n: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringRef_left(self as *const crate::QStringRef, n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the number of characters referred to by the string reference. Equivalent to <a href="http://doc.qt.io/qt-5/qstringref.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of characters referred to by the string reference. Equivalent to <a href="http://doc.qt.io/qt-5/qstringref.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#position">position</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#string">string</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_length(self as *const crate::QStringRef)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::localeAwareCompare(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#localeAwareCompare-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Compares this string with the <i>other</i> string and returns an integer less than, equal to, or greater than zero if this string is less than, equal to, or greater than the <i>other</i> string.</p>
    /// <p>The comparison is performed in a locale- and also platform-dependent manner. Use this function to present sorted lists of strings to the user.</p>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn locale_aware_compare_q_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_localeAwareCompare(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::localeAwareCompare(const QStringRef& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#localeAwareCompare-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Compares this string with the <i>other</i> string and returns an integer less than, equal to, or greater than zero if this string is less than, equal to, or greater than the <i>other</i> string.</p>
    /// <p>The comparison is performed in a locale- and also platform-dependent manner. Use this function to present sorted lists of strings to the user.</p>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn locale_aware_compare_q_string_ref(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_localeAwareCompare1(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Compares <i>s1</i> with <i>s2</i> and returns an integer less than, equal to, or greater than zero if <i>s1</i> is less than, equal to, or greater than <i>s2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QStringRef::localeAwareCompare(const QStringRef& s1, const QString& s2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#localeAwareCompare">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares <i>s1</i> with <i>s2</i> and returns an integer less than, equal to, or greater than zero if <i>s1</i> is less than, equal to, or greater than <i>s2</i>.</p>
    /// <p>The comparison is performed in a locale- and also platform-dependent manner. Use this function to present sorted lists of strings to the user.</p>
    /// <p>On <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS, this function compares according the "Order for sorted lists" setting in the International prefereces panel.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#compare">compare</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a>.</p></div>
    pub unsafe fn locale_aware_compare_q_string_ref_q_string(
        s1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        s2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_localeAwareCompare2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s1)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s2).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QStringRef::localeAwareCompare(const QStringRef& s1, const QStringRef& s2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#localeAwareCompare-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Compares <i>s1</i> with <i>s2</i> and returns an integer less than, equal to, or greater than zero if <i>s1</i> is less than, equal to, or greater than <i>s2</i>.</p>
    /// <p>The comparison is performed in a locale- and also platform-dependent manner. Use this function to present sorted lists of strings to the user.</p>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn locale_aware_compare_2_q_string_ref(
        s1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        s2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_localeAwareCompare3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s1)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s2)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a substring reference to <i>n</i> characters of this string, starting at the specified <i>position</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QStringRef::mid(int pos, int n = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a substring reference to <i>n</i> characters of this string, starting at the specified <i>position</i>.</p>
    /// <p>If the <i>position</i> exceeds the length of the string, a null reference is returned.</p>
    /// <p>If there are less than <i>n</i> characters available in the string, starting at the given <i>position</i>, or if <i>n</i> is -1 (default), the function returns all characters from the specified <i>position</i> onwards.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#left">left</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#right">right</a>().</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringRef_mid(self as *const crate::QStringRef, pos, n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a substring reference to <i>n</i> characters of this string, starting at the specified <i>position</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QStringRef::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a substring reference to <i>n</i> characters of this string, starting at the specified <i>position</i>.</p>
    /// <p>If the <i>position</i> exceeds the length of the string, a null reference is returned.</p>
    /// <p>If there are less than <i>n</i> characters available in the string, starting at the given <i>position</i>, or if <i>n</i> is -1 (default), the function returns all characters from the specified <i>position</i> onwards.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#left">left</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#right">right</a>().</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringRef_mid1(self as *const crate::QStringRef, pos);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs an empty string reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStringRef::QStringRef()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#QStringRef">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty string reference.</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringRef_QStringRef();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a string reference to the range of characters in the given <i>string</i> specified by the starting <i>position</i> and <i>length</i> in characters.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStringRef::QStringRef(const QString* string, int position, int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#QStringRef-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a string reference to the range of characters in the given <i>string</i> specified by the starting <i>position</i> and <i>length</i> in characters.</p>
    /// <p><b>Warning:</b> This function exists to improve performance as much as possible, and performs no bounds checking. For program correctness, <i>position</i> and <i>length</i> must describe a valid substring of <i>string</i>.</p>
    /// <p>This means that the starting <i>position</i> must be positive or 0 and smaller than <i>string</i>'s length, and <i>length</i> must be positive or 0 but smaller than the string's length minus the starting <i>position</i>; i.e, 0 &lt;= position &lt; string-&gt;<a href="http://doc.qt.io/qt-5/qstringref.html#length">length</a>() and 0 &lt;= length &lt;= string-&gt;<a href="http://doc.qt.io/qt-5/qstringref.html#length">length</a>() - position must both be satisfied.</p></div>
    pub unsafe fn new_3a(
        string: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QString>>,
        position: ::std::os::raw::c_int,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringRef_QStringRef1(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QString>>::cast_into(string)
                .as_raw_ptr(),
            position,
            size,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a string reference to the given <i>string</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStringRef::QStringRef(const QString* string)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#QStringRef-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a string reference to the given <i>string</i>.</p></div>
    pub unsafe fn new_1a(
        string: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringRef_QStringRef2(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QString>>::cast_into(string)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of the <i>other</i> string reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStringRef::QStringRef(const QStringRef& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#QStringRef-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of the <i>other</i> string reference.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringRef_QStringRef3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the starting position in the referenced string that is referred to by the string reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::position() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#position">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the starting position in the referenced string that is referred to by the string reference.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#string">string</a>().</p></div>
    pub unsafe fn position(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_position(self as *const crate::QStringRef)
    }

    /// <p>Returns a substring reference to the <i>n</i> rightmost characters of the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QStringRef::right(int n) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#right">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a substring reference to the <i>n</i> rightmost characters of the string.</p>
    /// <p>If <i>n</i> is greater than or equal to <a href="http://doc.qt.io/qt-5/qstringref.html#size">size</a>(), or less than zero, a reference to the entire string is returned.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#left">left</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#mid">mid</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p></div>
    pub unsafe fn right(&self, n: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringRef_right(self as *const crate::QStringRef, n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the number of characters referred to by the string reference. Equivalent to <a href="http://doc.qt.io/qt-5/qstringref.html#length">length</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of characters referred to by the string reference. Equivalent to <a href="http://doc.qt.io/qt-5/qstringref.html#length">length</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#count">count</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#position">position</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#string">string</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_size(self as *const crate::QStringRef)
    }

    /// <p>Splits the string into substrings references wherever <i>sep</i> occurs, and returns the list of those strings.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStringRef> QStringRef::split(const QString& sep, QString::SplitBehavior behavior = …, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#split">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Splits the string into substrings references wherever <i>sep</i> occurs, and returns the list of those strings.</p>
    /// <p>See <a href="http://doc.qt.io/qt-5/qstring.html#split">QString::split</a>() for how <i>sep</i>, <i>behavior</i> and <i>cs</i> interact to form the result.</p>
    /// <p><b>Note: </b>All references are valid as long this string is alive. Destroying this string will cause all references be dangling pointers.</p><p>This function was introduced in  Qt 5.4.</p></div>
    pub unsafe fn split_q_string_split_behavior_case_sensitivity(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        behavior: crate::q_string::SplitBehavior,
        cs: crate::CaseSensitivity,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringRef_split(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(sep).as_raw_ptr(),
            behavior,
            cs,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStringRef> QStringRef::split(QChar sep, QString::SplitBehavior behavior = …, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#split-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 5.4.</p></div>
    pub unsafe fn split_q_char_split_behavior_case_sensitivity(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        behavior: crate::q_string::SplitBehavior,
        cs: crate::CaseSensitivity,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringRef_split1(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(sep).as_raw_ptr(),
            behavior,
            cs,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Splits the string into substrings references wherever <i>sep</i> occurs, and returns the list of those strings.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStringRef> QStringRef::split(const QString& sep, QString::SplitBehavior behavior = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#split">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Splits the string into substrings references wherever <i>sep</i> occurs, and returns the list of those strings.</p>
    /// <p>See <a href="http://doc.qt.io/qt-5/qstring.html#split">QString::split</a>() for how <i>sep</i>, <i>behavior</i> and <i>cs</i> interact to form the result.</p>
    /// <p><b>Note: </b>All references are valid as long this string is alive. Destroying this string will cause all references be dangling pointers.</p><p>This function was introduced in  Qt 5.4.</p></div>
    pub unsafe fn split_q_string_split_behavior(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        behavior: crate::q_string::SplitBehavior,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringRef_split2(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(sep).as_raw_ptr(),
            behavior,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Splits the string into substrings references wherever <i>sep</i> occurs, and returns the list of those strings.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStringRef> QStringRef::split(const QString& sep) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#split">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Splits the string into substrings references wherever <i>sep</i> occurs, and returns the list of those strings.</p>
    /// <p>See <a href="http://doc.qt.io/qt-5/qstring.html#split">QString::split</a>() for how <i>sep</i>, <i>behavior</i> and <i>cs</i> interact to form the result.</p>
    /// <p><b>Note: </b>All references are valid as long this string is alive. Destroying this string will cause all references be dangling pointers.</p><p>This function was introduced in  Qt 5.4.</p></div>
    pub unsafe fn split_q_string(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringRef_split3(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(sep).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStringRef> QStringRef::split(QChar sep, QString::SplitBehavior behavior = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#split-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 5.4.</p></div>
    pub unsafe fn split_q_char_split_behavior(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        behavior: crate::q_string::SplitBehavior,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringRef_split4(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(sep).as_raw_ptr(),
            behavior,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStringRef> QStringRef::split(QChar sep) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#split-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 5.4.</p></div>
    pub unsafe fn split_q_char(
        &self,
        sep: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringRef_split5(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(sep).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if the string reference starts with <i>str</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::startsWith(const QString& s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the string reference starts with <i>str</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#startsWith">QString::startsWith</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p></div>
    pub unsafe fn starts_with_q_string_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_startsWith(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith">startsWith</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::startsWith(QLatin1String s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith">startsWith</a>().</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#startsWith">QString::startsWith</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p></div>
    pub unsafe fn starts_with_q_latin1_string_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_startsWith1(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith">startsWith</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::startsWith(QChar c, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith">startsWith</a>().</p>
    /// <p>Returns <code>true</code> if the string reference starts with <i>ch</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#startsWith">QString::startsWith</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p></div>
    pub unsafe fn starts_with_q_char_case_sensitivity(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_startsWith2(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith">startsWith</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::startsWith(const QStringRef& c, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith">startsWith</a>().</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#startsWith">QString::startsWith</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p></div>
    pub unsafe fn starts_with_q_string_ref_case_sensitivity(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_startsWith3(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(c).as_raw_ptr(),
            cs,
        )
    }

    /// <p>Returns <code>true</code> if the string reference starts with <i>str</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::startsWith(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the string reference starts with <i>str</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#startsWith">QString::startsWith</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p></div>
    pub unsafe fn starts_with_q_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_startsWith4(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith">startsWith</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::startsWith(QLatin1String s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith">startsWith</a>().</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#startsWith">QString::startsWith</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p></div>
    pub unsafe fn starts_with_q_latin1_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_startsWith5(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith">startsWith</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::startsWith(QChar c) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith">startsWith</a>().</p>
    /// <p>Returns <code>true</code> if the string reference starts with <i>ch</i>; otherwise returns <code>false</code>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> (default), the search is case sensitive; otherwise the search is case insensitive.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#startsWith">QString::startsWith</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p></div>
    pub unsafe fn starts_with_q_char(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_startsWith6(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith">startsWith</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::startsWith(const QStringRef& c) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith">startsWith</a>().</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#startsWith">QString::startsWith</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p></div>
    pub unsafe fn starts_with_q_string_ref(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_startsWith7(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(c).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith">startsWith</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::startsWith(QStringView s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith">startsWith</a>().</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#startsWith">QString::startsWith</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn starts_with_q_string_view_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_startsWith16(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
            cs,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith">startsWith</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::startsWith(QStringView s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringref.html#startsWith">startsWith</a>().</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#startsWith">QString::startsWith</a>() and <a href="http://doc.qt.io/qt-5/qstringref.html#endsWith">endsWith</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn starts_with_q_string_view(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_startsWith17(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a pointer to the string referred to by the string reference, or 0 if it does not reference a string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QString* QStringRef::string() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#string">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the string referred to by the string reference, or 0 if it does not reference a string.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#unicode">unicode</a>().</p></div>
    pub unsafe fn string(&self) -> ::cpp_utils::Ptr<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringRef_string(self as *const crate::QStringRef);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns the string converted to a <code>double</code> value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QStringRef::toDouble(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toDouble">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to a <code>double</code> value.</p>
    /// <p>Returns 0.0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, <code>*</code><i>ok</i> is set to <code>false</code>; otherwise <code>*</code><i>ok</i> is set to <code>true</code>.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">QLocale::toDouble</a>()</p>
    /// <p>For historic reasons, this function does not handle thousands group separators. If you need to convert such numbers, use <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">QLocale::toDouble</a>().</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toDouble">QString::toDouble</a>().</p></div>
    pub unsafe fn to_double_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toDouble(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the string converted to a <code>double</code> value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QStringRef::toDouble() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toDouble">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to a <code>double</code> value.</p>
    /// <p>Returns 0.0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, <code>*</code><i>ok</i> is set to <code>false</code>; otherwise <code>*</code><i>ok</i> is set to <code>true</code>.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">QLocale::toDouble</a>()</p>
    /// <p>For historic reasons, this function does not handle thousands group separators. If you need to convert such numbers, use <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">QLocale::toDouble</a>().</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toDouble">QString::toDouble</a>().</p></div>
    pub unsafe fn to_double_0a(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toDouble1(self as *const crate::QStringRef)
    }

    /// <p>Returns the string converted to a <code>float</code> value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```float QStringRef::toFloat(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toFloat">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to a <code>float</code> value.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>. Returns 0.0 if the conversion fails.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toFloat">QLocale::toFloat</a>()</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toFloat">QString::toFloat</a>().</p></div>
    pub unsafe fn to_float_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_float {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toFloat(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the string converted to a <code>float</code> value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```float QStringRef::toFloat() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toFloat">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to a <code>float</code> value.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>. Returns 0.0 if the conversion fails.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toFloat">QLocale::toFloat</a>()</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toFloat">QString::toFloat</a>().</p></div>
    pub unsafe fn to_float_0a(&self) -> ::std::os::raw::c_float {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toFloat1(self as *const crate::QStringRef)
    }

    /// <p>Returns the string converted to an <code>int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::toInt(bool* ok = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">QLocale::toInt</a>()</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toInt">QString::toInt</a>().</p></div>
    pub unsafe fn to_int_2a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
        base: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toInt(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
            base,
        )
    }

    /// <p>Returns the string converted to an <code>int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::toInt(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">QLocale::toInt</a>()</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toInt">QString::toInt</a>().</p></div>
    pub unsafe fn to_int_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toInt1(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the string converted to an <code>int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringRef::toInt() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">QLocale::toInt</a>()</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toInt">QString::toInt</a>().</p></div>
    pub unsafe fn to_int_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toInt2(self as *const crate::QStringRef)
    }

    /// <p>Returns a Latin-1 representation of the string as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QStringRef::toLatin1() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toLatin1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a Latin-1 representation of the string as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    /// <p>The returned byte array is undefined if the string contains non-Latin1 characters. Those characters may be suppressed or replaced with a question mark.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#toUtf8">toUtf8</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#toLocal8Bit">toLocal8Bit</a>(), and <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a>.</p></div>
    pub unsafe fn to_latin1(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringRef_toLatin1(self as *const crate::QStringRef);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the local 8-bit representation of the string as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>. The returned byte array is undefined if the string contains characters not supported by the local 8-bit encoding.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QStringRef::toLocal8Bit() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toLocal8Bit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the local 8-bit representation of the string as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>. The returned byte array is undefined if the string contains characters not supported by the local 8-bit encoding.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForLocale">QTextCodec::codecForLocale</a>() is used to perform the conversion from Unicode. If the locale encoding could not be determined, this function does the same as <a href="http://doc.qt.io/qt-5/qstringref.html#toLatin1">toLatin1</a>().</p>
    /// <p>If this string contains any characters that cannot be encoded in the locale, the returned byte array is undefined. Those characters may be suppressed or replaced by another.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#toLatin1">toLatin1</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#toUtf8">toUtf8</a>(), and <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a>.</p></div>
    pub unsafe fn to_local8_bit(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringRef_toLocal8Bit(self as *const crate::QStringRef);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the string converted to a <code>long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```long QStringRef::toLong(bool* ok = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to a <code>long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use QLocale::toLong()</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toLong">QString::toLong</a>().</p></div>
    pub unsafe fn to_long_2a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
        base: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_long {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toLong(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
            base,
        )
    }

    /// <p>Returns the string converted to a <code>long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```long QStringRef::toLong(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to a <code>long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use QLocale::toLong()</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toLong">QString::toLong</a>().</p></div>
    pub unsafe fn to_long_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_long {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toLong1(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the string converted to a <code>long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```long QStringRef::toLong() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to a <code>long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use QLocale::toLong()</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toLong">QString::toLong</a>().</p></div>
    pub unsafe fn to_long_0a(&self) -> ::std::os::raw::c_long {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toLong2(self as *const crate::QStringRef)
    }

    /// <p>Returns the string converted to a <code>long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qlonglong QStringRef::toLongLong(bool* ok = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toLongLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to a <code>long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toLongLong">QLocale::toLongLong</a>()</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toLongLong">QString::toLongLong</a>().</p></div>
    pub unsafe fn to_long_long_2a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
        base: ::std::os::raw::c_int,
    ) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toLongLong(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
            base,
        )
    }

    /// <p>Returns the string converted to a <code>long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qlonglong QStringRef::toLongLong(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toLongLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to a <code>long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toLongLong">QLocale::toLongLong</a>()</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toLongLong">QString::toLongLong</a>().</p></div>
    pub unsafe fn to_long_long_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toLongLong1(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the string converted to a <code>long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qlonglong QStringRef::toLongLong() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toLongLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to a <code>long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toLongLong">QLocale::toLongLong</a>()</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toLongLong">QString::toLongLong</a>().</p></div>
    pub unsafe fn to_long_long_0a(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toLongLong2(self as *const crate::QStringRef)
    }

    /// <p>Returns the string converted to a <code>short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```short QStringRef::toShort(bool* ok = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toShort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to a <code>short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toShort">QLocale::toShort</a>()</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toShort">QString::toShort</a>().</p></div>
    pub unsafe fn to_short_2a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
        base: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_short {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toShort(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
            base,
        )
    }

    /// <p>Returns the string converted to a <code>short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```short QStringRef::toShort(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toShort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to a <code>short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toShort">QLocale::toShort</a>()</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toShort">QString::toShort</a>().</p></div>
    pub unsafe fn to_short_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_short {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toShort1(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the string converted to a <code>short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```short QStringRef::toShort() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toShort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to a <code>short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toShort">QLocale::toShort</a>()</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toShort">QString::toShort</a>().</p></div>
    pub unsafe fn to_short_0a(&self) -> ::std::os::raw::c_short {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toShort2(self as *const crate::QStringRef)
    }

    /// <p>Returns a copy of the string reference as a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QStringRef::toString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a copy of the string reference as a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> object.</p>
    /// <p>If the string reference is not a complete reference of the string (meaning that <a href="http://doc.qt.io/qt-5/qstringref.html#position">position</a>() is 0 and <a href="http://doc.qt.io/qt-5/qstringref.html#size">size</a>() equals <a href="http://doc.qt.io/qt-5/qstringref.html#string">string</a>()-&gt;<a href="http://doc.qt.io/qt-5/qstringref.html#size">size</a>()), this function will allocate a new string to return.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#string">string</a>().</p></div>
    pub unsafe fn to_string(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringRef_toString(self as *const crate::QStringRef);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the string converted to an <code>unsigned int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QStringRef::toUInt(bool* ok = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toUInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>unsigned int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toUInt">QLocale::toUInt</a>()</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toUInt">QString::toUInt</a>().</p></div>
    pub unsafe fn to_u_int_2a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
        base: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toUInt(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
            base,
        )
    }

    /// <p>Returns the string converted to an <code>unsigned int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QStringRef::toUInt(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toUInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>unsigned int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toUInt">QLocale::toUInt</a>()</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toUInt">QString::toUInt</a>().</p></div>
    pub unsafe fn to_u_int_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toUInt1(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the string converted to an <code>unsigned int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QStringRef::toUInt() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toUInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>unsigned int</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toUInt">QLocale::toUInt</a>()</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toUInt">QString::toUInt</a>().</p></div>
    pub unsafe fn to_u_int_0a(&self) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toUInt2(self as *const crate::QStringRef)
    }

    /// <p>Returns the string converted to an <code>unsigned long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned long QStringRef::toULong(bool* ok = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toULong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>unsigned long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong">QLocale::toULongLong</a>()</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toULong">QString::toULong</a>().</p></div>
    pub unsafe fn to_u_long_2a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
        base: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_ulong {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toULong(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
            base,
        )
    }

    /// <p>Returns the string converted to an <code>unsigned long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned long QStringRef::toULong(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toULong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>unsigned long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong">QLocale::toULongLong</a>()</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toULong">QString::toULong</a>().</p></div>
    pub unsafe fn to_u_long_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_ulong {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toULong1(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the string converted to an <code>unsigned long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned long QStringRef::toULong() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toULong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>unsigned long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong">QLocale::toULongLong</a>()</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toULong">QString::toULong</a>().</p></div>
    pub unsafe fn to_u_long_0a(&self) -> ::std::os::raw::c_ulong {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toULong2(self as *const crate::QStringRef)
    }

    /// <p>Returns the string converted to an <code>unsigned long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qulonglong QStringRef::toULongLong(bool* ok = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toULongLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>unsigned long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong">QLocale::toULongLong</a>()</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toULongLong">QString::toULongLong</a>().</p></div>
    pub unsafe fn to_u_long_long_2a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
        base: ::std::os::raw::c_int,
    ) -> u64 {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toULongLong(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
            base,
        )
    }

    /// <p>Returns the string converted to an <code>unsigned long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qulonglong QStringRef::toULongLong(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toULongLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>unsigned long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong">QLocale::toULongLong</a>()</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toULongLong">QString::toULongLong</a>().</p></div>
    pub unsafe fn to_u_long_long_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> u64 {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toULongLong1(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the string converted to an <code>unsigned long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qulonglong QStringRef::toULongLong() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toULongLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>unsigned long long</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong">QLocale::toULongLong</a>()</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toULongLong">QString::toULongLong</a>().</p></div>
    pub unsafe fn to_u_long_long_0a(&self) -> u64 {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toULongLong2(self as *const crate::QStringRef)
    }

    /// <p>Returns the string converted to an <code>unsigned short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned short QStringRef::toUShort(bool* ok = …, int base = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toUShort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>unsigned short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toUShort">QLocale::toUShort</a>()</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toUShort">QString::toUShort</a>().</p></div>
    pub unsafe fn to_u_short_2a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
        base: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_ushort {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toUShort(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
            base,
        )
    }

    /// <p>Returns the string converted to an <code>unsigned short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned short QStringRef::toUShort(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toUShort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>unsigned short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toUShort">QLocale::toUShort</a>()</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toUShort">QString::toUShort</a>().</p></div>
    pub unsafe fn to_u_short_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_ushort {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toUShort1(
            self as *const crate::QStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the string converted to an <code>unsigned short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned short QStringRef::toUShort() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toUShort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string converted to an <code>unsigned short</code> using base <i>base</i>, which is 10 by default and must be between 2 and 36, or 0. Returns 0 if the conversion fails.</p>
    /// <p>If a conversion error occurs, *<i>ok</i> is set to <code>false</code>; otherwise *<i>ok</i> is set to <code>true</code>.</p>
    /// <p>If <i>base</i> is 0, the C language convention is used: If the string begins with "0x", base 16 is used; if the string begins with "0", base 8 is used; otherwise, base 10 is used.</p>
    /// <p>The string conversion will always happen in the 'C' locale. For locale dependent conversion use <a href="http://doc.qt.io/qt-5/qlocale.html#toUShort">QLocale::toUShort</a>()</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toUShort">QString::toUShort</a>().</p></div>
    pub unsafe fn to_u_short_0a(&self) -> ::std::os::raw::c_ushort {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_toUShort2(self as *const crate::QStringRef)
    }

    /// <p>Returns a UCS-4/UTF-32 representation of the string as a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;uint&gt;.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<unsigned int> QStringRef::toUcs4() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toUcs4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a UCS-4/UTF-32 representation of the string as a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;uint&gt;.</p>
    /// <p>UCS-4 is a Unicode codec and therefore it is lossless. All characters from this string will be encoded in UCS-4. Any invalid sequence of code units in this string is replaced by the Unicode's replacement character (<a href="http://doc.qt.io/qt-5/qchar.html#SpecialCharacter-enum">QChar::ReplacementCharacter</a>, which corresponds to <code>U+FFFD</code>).</p>
    /// <p>The returned vector is not NUL terminated.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#toUtf8">toUtf8</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#toLatin1">toLatin1</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#toLocal8Bit">toLocal8Bit</a>(), and <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a>.</p></div>
    pub unsafe fn to_ucs4(&self) -> ::cpp_utils::CppBox<crate::QVectorOfUint> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringRef_toUcs4(self as *const crate::QStringRef);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a UTF-8 representation of the string as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QStringRef::toUtf8() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#toUtf8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a UTF-8 representation of the string as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    /// <p>UTF-8 is a Unicode codec and can represent all characters in a Unicode string like <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#toLatin1">toLatin1</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#toLocal8Bit">toLocal8Bit</a>(), and <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a>.</p></div>
    pub unsafe fn to_utf8(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringRef_toUtf8(self as *const crate::QStringRef);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a string that has whitespace removed from the start and the end.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QStringRef::trimmed() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#trimmed">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string that has whitespace removed from the start and the end.</p>
    /// <p>Whitespace means any character for which <a href="http://doc.qt.io/qt-5/qchar.html#isSpace">QChar::isSpace</a>() returns <code>true</code>. This includes the ASCII characters '\t', '\n', '\v', '\f', '\r', and ' '.</p>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qstring.html#simplified">QString::simplified</a>(), trimmed() leaves internal whitespace alone.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#trimmed">QString::trimmed</a>().</p></div>
    pub unsafe fn trimmed(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringRef_trimmed(self as *const crate::QStringRef);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Truncates the string at the given <i>position</i> index.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QStringRef::truncate(int pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#truncate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Truncates the string at the given <i>position</i> index.</p>
    /// <p>If the specified <i>position</i> index is beyond the end of the string, nothing happens.</p>
    /// <p>If <i>position</i> is negative, it is equivalent to passing zero.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#truncate">QString::truncate</a>().</p></div>
    pub unsafe fn truncate(&mut self, pos: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_truncate(self as *mut crate::QStringRef, pos)
    }

    /// <p>Returns a Unicode representation of the string reference. Since the data stems directly from the referenced string, it is not null-terminated unless the string reference includes the string's null terminator.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QChar* QStringRef::unicode() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#unicode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a Unicode representation of the string reference. Since the data stems directly from the referenced string, it is not null-terminated unless the string reference includes the string's null terminator.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#string">string</a>().</p></div>
    pub unsafe fn unicode(&self) -> ::cpp_utils::Ptr<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringRef_unicode(self as *const crate::QStringRef);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }
}

pub mod q_reg_exp {
    //! C++ type: <span style='color: green;'>```QRegExp```</span>

    /// <p>The syntax used to interpret the meaning of the pattern.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QRegExp::PatternSyntax```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#PatternSyntax-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The syntax used to interpret the meaning of the pattern.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#setPatternSyntax">setPatternSyntax</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct PatternSyntax(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for PatternSyntax {
        fn from(value: ::std::os::raw::c_int) -> Self {
            PatternSyntax(value)
        }
    }

    impl From<PatternSyntax> for ::std::os::raw::c_int {
        fn from(value: PatternSyntax) -> Self {
            value.0
        }
    }

    impl PatternSyntax {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl PatternSyntax {
        /// A rich Perl-like pattern matching syntax. This is the default. (C++ enum variant: <span style='color: green;'>```RegExp = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const RegExp: crate::q_reg_exp::PatternSyntax = crate::q_reg_exp::PatternSyntax(0);
        /// This provides a simple pattern matching syntax similar to that used by shells (command interpreters) for "file globbing". See <a href="http://doc.qt.io/qt-5/qregexp.html#qregexp-wildcard-matching">QRegExp wildcard matching</a>. (C++ enum variant: <span style='color: green;'>```Wildcard = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Wildcard: crate::q_reg_exp::PatternSyntax = crate::q_reg_exp::PatternSyntax(1);
        /// The pattern is a fixed string. This is equivalent to using the RegExp pattern on a string in which all metacharacters are escaped using <a href="http://doc.qt.io/qt-5/qregexp.html#escape">escape</a>(). (C++ enum variant: <span style='color: green;'>```FixedString = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const FixedString: crate::q_reg_exp::PatternSyntax = crate::q_reg_exp::PatternSyntax(2);
        /// Like RegExp, but with <a href="http://doc.qt.io/qt-5/qregexp.html#greedy-quantifiers">greedy quantifiers</a>. (Introduced in Qt 4.2.) (C++ enum variant: <span style='color: green;'>```RegExp2 = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const RegExp2: crate::q_reg_exp::PatternSyntax = crate::q_reg_exp::PatternSyntax(3);
        /// This is similar to Wildcard but with the behavior of a Unix shell. The wildcard characters can be escaped with the character "\". (C++ enum variant: <span style='color: green;'>```WildcardUnix = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const WildcardUnix: crate::q_reg_exp::PatternSyntax =
            crate::q_reg_exp::PatternSyntax(4);
        /// The pattern is a regular expression as defined by the W3C XML Schema 1.1 specification. (C++ enum variant: <span style='color: green;'>```W3CXmlSchema11 = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const W3CXmlSchema11: crate::q_reg_exp::PatternSyntax =
            crate::q_reg_exp::PatternSyntax(5);
    }

    /// <p>The CaretMode enum defines the different meanings of the caret (<b>^</b>) in a regular expression. The possible values are:</p>
    ///
    /// C++ enum: <span style='color: green;'>```QRegExp::CaretMode```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#CaretMode-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The CaretMode enum defines the different meanings of the caret (<b>^</b>) in a regular expression. The possible values are:</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct CaretMode(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for CaretMode {
        fn from(value: ::std::os::raw::c_int) -> Self {
            CaretMode(value)
        }
    }

    impl From<CaretMode> for ::std::os::raw::c_int {
        fn from(value: CaretMode) -> Self {
            value.0
        }
    }

    impl CaretMode {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl CaretMode {
        /// The caret corresponds to index 0 in the searched string. (C++ enum variant: <span style='color: green;'>```CaretAtZero = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const CaretAtZero: crate::q_reg_exp::CaretMode = crate::q_reg_exp::CaretMode(0);
        /// The caret corresponds to the start offset of the search. (C++ enum variant: <span style='color: green;'>```CaretAtOffset = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const CaretAtOffset: crate::q_reg_exp::CaretMode = crate::q_reg_exp::CaretMode(1);
        /// The caret never matches. (C++ enum variant: <span style='color: green;'>```CaretWontMatch = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const CaretWontMatch: crate::q_reg_exp::CaretMode = crate::q_reg_exp::CaretMode(2);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> class provides pattern matching using regular expressions.</p>
///
/// C++ class: <span style='color: green;'>```QRegExp```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qregexp.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> class provides pattern matching using regular expressions.</p>
/// <p>A regular expression, or "regexp", is a pattern for matching substrings in a text. This is useful in many contexts, e.g.,</p>
/// <div class="table"><table class="generic">
///  <tbody><tr valign="top" class="odd"><td>Validation</td><td>A regexp can test whether a substring meets some criteria, e.g. is an integer or contains no whitespace.</td></tr>
/// <tr valign="top" class="even"><td>Searching</td><td>A regexp provides more powerful pattern matching than simple substring matching, e.g., match one of the words <i>mail</i>, <i>letter</i> or <i>correspondence</i>, but none of the words <i>email</i>, <i>mailman</i>, <i>mailer</i>, <i>letterbox</i>, etc.</td></tr>
/// <tr valign="top" class="odd"><td>Search and Replace</td><td>A regexp can replace all occurrences of a substring with a different substring, e.g., replace all occurrences of <i>&amp;</i> with <i>&amp;amp;</i> except where the <i>&amp;</i> is already followed by an <i>amp;</i>.</td></tr>
/// <tr valign="top" class="even"><td>String Splitting</td><td>A regexp can be used to identify where a string should be split apart, e.g. splitting tab-delimited strings.</td></tr>
/// </tbody></table></div>
/// <p>A brief introduction to regexps is presented, a description of Qt's regexp language, some examples, and the function documentation itself. <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> is modeled on Perl's regexp language. It fully supports Unicode. <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> can also be used in a simpler, <i>wildcard mode</i> that is similar to the functionality found in command shells. The syntax rules used by <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> can be changed with <a href="http://doc.qt.io/qt-5/qregexp.html#setPatternSyntax">setPatternSyntax</a>(). In particular, the pattern syntax can be set to <a href="http://doc.qt.io/qt-5/qregexp.html#PatternSyntax-enum">QRegExp::FixedString</a>, which means the pattern to be matched is interpreted as a plain string, i.e., special characters (e.g., backslash) are not escaped.</p>
/// <p>A good text on regexps is <i>Mastering Regular Expressions</i> (Third Edition) by Jeffrey E. F. Friedl, ISBN 0-596-52812-4.</p>
/// <p><b>Note: </b>In Qt 5, the new <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> class provides a Perl compatible implementation of regular expressions and is recommended in place of <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a>.</p><a name="introduction"></a></div>
#[repr(C)]
pub struct QRegExp {
    _unused: u8,
}
impl QRegExp {
    /// <p>Returns the text captured by the <i>nth</i> subexpression. The entire match has index 0 and the parenthesized subexpressions have indexes starting from 1 (excluding non-capturing parentheses).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QRegExp::cap(int nth = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#cap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the text captured by the <i>nth</i> subexpression. The entire match has index 0 and the parenthesized subexpressions have indexes starting from 1 (excluding non-capturing parentheses).</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html#QRegExp">QRegExp</a></span> rxlen(<span class="string">"(\\d+)(?:\\s*)(cm|inch)"</span>);
    ///   <span class="type">int</span> pos <span class="operator">=</span> rxlen<span class="operator">.</span>indexIn(<span class="string">"Length: 189cm"</span>);
    ///   <span class="keyword">if</span> (pos <span class="operator">&gt;</span> <span class="operator">-</span><span class="number">1</span>) {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> value <span class="operator">=</span> rxlen<span class="operator">.</span>cap(<span class="number">1</span>); <span class="comment">// "189"</span>
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> unit <span class="operator">=</span> rxlen<span class="operator">.</span>cap(<span class="number">2</span>);  <span class="comment">// "cm"</span>
    /// &#32;     <span class="comment">// ...</span>
    ///   }
    ///
    /// </pre>
    /// <p>The order of elements matched by cap() is as follows. The first element, cap(0), is the entire matching string. Each subsequent element corresponds to the next capturing open left parentheses. Thus cap(1) is the text of the first capturing parentheses, cap(2) is the text of the second, and so on.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#capturedTexts">capturedTexts</a>() and <a href="http://doc.qt.io/qt-5/qregexp.html#pos">pos</a>().</p></div>
    pub unsafe fn cap_1a(&self, nth: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QRegExp_cap(self as *const crate::QRegExp, nth);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the text captured by the <i>nth</i> subexpression. The entire match has index 0 and the parenthesized subexpressions have indexes starting from 1 (excluding non-capturing parentheses).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QRegExp::cap(int nth = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#cap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the text captured by the <i>nth</i> subexpression. The entire match has index 0 and the parenthesized subexpressions have indexes starting from 1 (excluding non-capturing parentheses).</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html#QRegExp">QRegExp</a></span> rxlen(<span class="string">"(\\d+)(?:\\s*)(cm|inch)"</span>);
    ///   <span class="type">int</span> pos <span class="operator">=</span> rxlen<span class="operator">.</span>indexIn(<span class="string">"Length: 189cm"</span>);
    ///   <span class="keyword">if</span> (pos <span class="operator">&gt;</span> <span class="operator">-</span><span class="number">1</span>) {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> value <span class="operator">=</span> rxlen<span class="operator">.</span>cap(<span class="number">1</span>); <span class="comment">// "189"</span>
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> unit <span class="operator">=</span> rxlen<span class="operator">.</span>cap(<span class="number">2</span>);  <span class="comment">// "cm"</span>
    /// &#32;     <span class="comment">// ...</span>
    ///   }
    ///
    /// </pre>
    /// <p>The order of elements matched by cap() is as follows. The first element, cap(0), is the entire matching string. Each subsequent element corresponds to the next capturing open left parentheses. Thus cap(1) is the text of the first capturing parentheses, cap(2) is the text of the second, and so on.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#capturedTexts">capturedTexts</a>() and <a href="http://doc.qt.io/qt-5/qregexp.html#pos">pos</a>().</p></div>
    pub unsafe fn cap_1a_mut(
        &mut self,
        nth: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QRegExp_cap1(self as *mut crate::QRegExp, nth);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the text captured by the <i>nth</i> subexpression. The entire match has index 0 and the parenthesized subexpressions have indexes starting from 1 (excluding non-capturing parentheses).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QRegExp::cap() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#cap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the text captured by the <i>nth</i> subexpression. The entire match has index 0 and the parenthesized subexpressions have indexes starting from 1 (excluding non-capturing parentheses).</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html#QRegExp">QRegExp</a></span> rxlen(<span class="string">"(\\d+)(?:\\s*)(cm|inch)"</span>);
    ///   <span class="type">int</span> pos <span class="operator">=</span> rxlen<span class="operator">.</span>indexIn(<span class="string">"Length: 189cm"</span>);
    ///   <span class="keyword">if</span> (pos <span class="operator">&gt;</span> <span class="operator">-</span><span class="number">1</span>) {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> value <span class="operator">=</span> rxlen<span class="operator">.</span>cap(<span class="number">1</span>); <span class="comment">// "189"</span>
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> unit <span class="operator">=</span> rxlen<span class="operator">.</span>cap(<span class="number">2</span>);  <span class="comment">// "cm"</span>
    /// &#32;     <span class="comment">// ...</span>
    ///   }
    ///
    /// </pre>
    /// <p>The order of elements matched by cap() is as follows. The first element, cap(0), is the entire matching string. Each subsequent element corresponds to the next capturing open left parentheses. Thus cap(1) is the text of the first capturing parentheses, cap(2) is the text of the second, and so on.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#capturedTexts">capturedTexts</a>() and <a href="http://doc.qt.io/qt-5/qregexp.html#pos">pos</a>().</p></div>
    pub unsafe fn cap_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegExp_cap2(self as *const crate::QRegExp);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the text captured by the <i>nth</i> subexpression. The entire match has index 0 and the parenthesized subexpressions have indexes starting from 1 (excluding non-capturing parentheses).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QRegExp::cap()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#cap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the text captured by the <i>nth</i> subexpression. The entire match has index 0 and the parenthesized subexpressions have indexes starting from 1 (excluding non-capturing parentheses).</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html#QRegExp">QRegExp</a></span> rxlen(<span class="string">"(\\d+)(?:\\s*)(cm|inch)"</span>);
    ///   <span class="type">int</span> pos <span class="operator">=</span> rxlen<span class="operator">.</span>indexIn(<span class="string">"Length: 189cm"</span>);
    ///   <span class="keyword">if</span> (pos <span class="operator">&gt;</span> <span class="operator">-</span><span class="number">1</span>) {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> value <span class="operator">=</span> rxlen<span class="operator">.</span>cap(<span class="number">1</span>); <span class="comment">// "189"</span>
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> unit <span class="operator">=</span> rxlen<span class="operator">.</span>cap(<span class="number">2</span>);  <span class="comment">// "cm"</span>
    /// &#32;     <span class="comment">// ...</span>
    ///   }
    ///
    /// </pre>
    /// <p>The order of elements matched by cap() is as follows. The first element, cap(0), is the entire matching string. Each subsequent element corresponds to the next capturing open left parentheses. Thus cap(1) is the text of the first capturing parentheses, cap(2) is the text of the second, and so on.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#capturedTexts">capturedTexts</a>() and <a href="http://doc.qt.io/qt-5/qregexp.html#pos">pos</a>().</p></div>
    pub unsafe fn cap_0a_mut(&mut self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegExp_cap3(self as *mut crate::QRegExp);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the number of captures contained in the regular expression.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRegExp::captureCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#captureCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of captures contained in the regular expression.</p>
    /// <p>This function was introduced in  Qt 4.6.</p></div>
    pub unsafe fn capture_count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRegExp_captureCount(self as *const crate::QRegExp)
    }

    /// <p>Returns a list of the captured text strings.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QRegExp::capturedTexts() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#capturedTexts">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of the captured text strings.</p>
    /// <p>The first string in the list is the entire matched string. Each subsequent list element contains a string that matched a (capturing) subexpression of the regexp.</p>
    /// <p>For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html#QRegExp">QRegExp</a></span> rx(<span class="string">"(\\d+)(\\s*)(cm|inch(es)?)"</span>);
    ///   <span class="type">int</span> pos <span class="operator">=</span> rx<span class="operator">.</span>indexIn(<span class="string">"Length: 36 inches"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> list <span class="operator">=</span> rx<span class="operator">.</span>capturedTexts();
    ///   <span class="comment">// list is now ("36 inches", "36", " ", "inches", "es")</span>
    ///
    /// </pre>
    /// <p>The above example also captures elements that may be present but which we have no interest in. This problem can be solved by using non-capturing parentheses:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html#QRegExp">QRegExp</a></span> rx(<span class="string">"(\\d+)(?:\\s*)(cm|inch(?:es)?)"</span>);
    ///   <span class="type">int</span> pos <span class="operator">=</span> rx<span class="operator">.</span>indexIn(<span class="string">"Length: 36 inches"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> list <span class="operator">=</span> rx<span class="operator">.</span>capturedTexts();
    ///   <span class="comment">// list is now ("36 inches", "36", "inches")</span>
    ///
    /// </pre>
    /// <p>Note that if you want to iterate over the list, you should iterate over a copy, e.g.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> list <span class="operator">=</span> rx<span class="operator">.</span>capturedTexts();
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span><span class="operator">::</span>iterator it <span class="operator">=</span> list<span class="operator">.</span>begin();
    ///   <span class="keyword">while</span> (it <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end()) {
    /// &#32;     myProcessing(<span class="operator">*</span>it);
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>it;
    ///   }
    ///
    /// </pre>
    /// <p>Some regexps can match an indeterminate number of times. For example if the input string is "Offsets: 12 14 99 231 7" and the regexp, <code>rx</code>, is <b>(\d+)+</b>, we would hope to get a list of all the numbers matched. However, after calling <code>rx.indexIn(str)</code>, capturedTexts() will return the list ("12", "12"), i.e. the entire match was "12" and the first subexpression matched was "12". The correct approach is to use <a href="http://doc.qt.io/qt-5/qregexp.html#cap">cap</a>() in a <a href="http://doc.qt.io/qt-5/qregexp.html#cap-in-a-loop">loop</a>.</p>
    /// <p>The order of elements in the string list is as follows. The first element is the entire matching string. Each subsequent element corresponds to the next capturing open left parentheses. Thus capturedTexts()[1] is the text of the first capturing parentheses, capturedTexts()[2] is the text of the second and so on (corresponding to $1, $2, etc., in some other regexp languages).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#cap">cap</a>() and <a href="http://doc.qt.io/qt-5/qregexp.html#pos">pos</a>().</p></div>
    pub unsafe fn captured_texts(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QRegExp_capturedTexts(self as *const crate::QRegExp);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list of the captured text strings.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QRegExp::capturedTexts()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#capturedTexts">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of the captured text strings.</p>
    /// <p>The first string in the list is the entire matched string. Each subsequent list element contains a string that matched a (capturing) subexpression of the regexp.</p>
    /// <p>For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html#QRegExp">QRegExp</a></span> rx(<span class="string">"(\\d+)(\\s*)(cm|inch(es)?)"</span>);
    ///   <span class="type">int</span> pos <span class="operator">=</span> rx<span class="operator">.</span>indexIn(<span class="string">"Length: 36 inches"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> list <span class="operator">=</span> rx<span class="operator">.</span>capturedTexts();
    ///   <span class="comment">// list is now ("36 inches", "36", " ", "inches", "es")</span>
    ///
    /// </pre>
    /// <p>The above example also captures elements that may be present but which we have no interest in. This problem can be solved by using non-capturing parentheses:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html#QRegExp">QRegExp</a></span> rx(<span class="string">"(\\d+)(?:\\s*)(cm|inch(?:es)?)"</span>);
    ///   <span class="type">int</span> pos <span class="operator">=</span> rx<span class="operator">.</span>indexIn(<span class="string">"Length: 36 inches"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> list <span class="operator">=</span> rx<span class="operator">.</span>capturedTexts();
    ///   <span class="comment">// list is now ("36 inches", "36", "inches")</span>
    ///
    /// </pre>
    /// <p>Note that if you want to iterate over the list, you should iterate over a copy, e.g.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> list <span class="operator">=</span> rx<span class="operator">.</span>capturedTexts();
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span><span class="operator">::</span>iterator it <span class="operator">=</span> list<span class="operator">.</span>begin();
    ///   <span class="keyword">while</span> (it <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end()) {
    /// &#32;     myProcessing(<span class="operator">*</span>it);
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>it;
    ///   }
    ///
    /// </pre>
    /// <p>Some regexps can match an indeterminate number of times. For example if the input string is "Offsets: 12 14 99 231 7" and the regexp, <code>rx</code>, is <b>(\d+)+</b>, we would hope to get a list of all the numbers matched. However, after calling <code>rx.indexIn(str)</code>, capturedTexts() will return the list ("12", "12"), i.e. the entire match was "12" and the first subexpression matched was "12". The correct approach is to use <a href="http://doc.qt.io/qt-5/qregexp.html#cap">cap</a>() in a <a href="http://doc.qt.io/qt-5/qregexp.html#cap-in-a-loop">loop</a>.</p>
    /// <p>The order of elements in the string list is as follows. The first element is the entire matching string. Each subsequent element corresponds to the next capturing open left parentheses. Thus capturedTexts()[1] is the text of the first capturing parentheses, capturedTexts()[2] is the text of the second and so on (corresponding to $1, $2, etc., in some other regexp languages).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#cap">cap</a>() and <a href="http://doc.qt.io/qt-5/qregexp.html#pos">pos</a>().</p></div>
    pub unsafe fn captured_texts_mut(&mut self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QRegExp_capturedTexts1(self as *mut crate::QRegExp);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> if the regexp is matched case sensitively; otherwise returns <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseInsensitive</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```Qt::CaseSensitivity QRegExp::caseSensitivity() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#caseSensitivity">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a> if the regexp is matched case sensitively; otherwise returns <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseInsensitive</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#setCaseSensitivity">setCaseSensitivity</a>(), <a href="http://doc.qt.io/qt-5/qregexp.html#patternSyntax">patternSyntax</a>(), <a href="http://doc.qt.io/qt-5/qregexp.html#pattern">pattern</a>(), and <a href="http://doc.qt.io/qt-5/qregexp.html#isMinimal">isMinimal</a>().</p></div>
    pub unsafe fn case_sensitivity(&self) -> crate::CaseSensitivity {
        crate::__ffi::ctr_qt_core_ffi_QRegExp_caseSensitivity(self as *const crate::QRegExp)
    }

    /// <p>Copies the regular expression <i>rx</i> and returns a reference to the copy. The case sensitivity, wildcard, and minimal matching options are also copied.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegExp& QRegExp::operator=(const QRegExp& rx)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Copies the regular expression <i>rx</i> and returns a reference to the copy. The case sensitivity, wildcard, and minimal matching options are also copied.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        rx: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
    ) -> ::cpp_utils::MutRef<crate::QRegExp> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegExp_operator_(
            self as *mut crate::QRegExp,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(rx).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a text string that explains why a regexp pattern is invalid the case being; otherwise returns "no error occurred".</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QRegExp::errorString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#errorString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a text string that explains why a regexp pattern is invalid the case being; otherwise returns "no error occurred".</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#isValid">isValid</a>().</p></div>
    pub unsafe fn error_string(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QRegExp_errorString(self as *const crate::QRegExp);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a text string that explains why a regexp pattern is invalid the case being; otherwise returns "no error occurred".</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QRegExp::errorString()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#errorString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a text string that explains why a regexp pattern is invalid the case being; otherwise returns "no error occurred".</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#isValid">isValid</a>().</p></div>
    pub unsafe fn error_string_mut(&mut self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QRegExp_errorString1(self as *mut crate::QRegExp);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the string <i>str</i> with every regexp special character escaped with a backslash. The special characters are $, (,), *, +, ., ?, [, ,], ^, {, | and }.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QRegExp::escape(const QString& str)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#escape">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string <i>str</i> with every regexp special character escaped with a backslash. The special characters are $, (,), *, +, ., ?, [, ,], ^, {, | and }.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   s1 <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html#QRegExp">QRegExp</a></span><span class="operator">::</span>escape(<span class="string">"bingo"</span>);   <span class="comment">// s1 == "bingo"</span>
    ///   s2 <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html#QRegExp">QRegExp</a></span><span class="operator">::</span>escape(<span class="string">"f(x)"</span>);    <span class="comment">// s2 == "f\\(x\\)"</span>
    ///
    /// </pre>
    /// <p>This function is useful to construct regexp patterns dynamically:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html#QRegExp">QRegExp</a></span> rx(<span class="string">"("</span> <span class="operator">+</span> <span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html#QRegExp">QRegExp</a></span><span class="operator">::</span>escape(name) <span class="operator">+</span>
    /// &#32;   &#32;   &#32;    <span class="string">"|"</span> <span class="operator">+</span> <span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html#QRegExp">QRegExp</a></span><span class="operator">::</span>escape(alias) <span class="operator">+</span> <span class="string">")"</span>);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#setPatternSyntax">setPatternSyntax</a>().</p></div>
    pub unsafe fn escape(
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegExp_escape(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(str).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if <i>str</i> is matched exactly by this regular expression; otherwise returns <code>false</code>. You can determine how much of the string was matched by calling <a href="http://doc.qt.io/qt-5/qregexp.html#matchedLength">matchedLength</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRegExp::exactMatch(const QString& str) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#exactMatch">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>str</i> is matched exactly by this regular expression; otherwise returns <code>false</code>. You can determine how much of the string was matched by calling <a href="http://doc.qt.io/qt-5/qregexp.html#matchedLength">matchedLength</a>().</p>
    /// <p>For a given regexp string R, exactMatch("R") is the equivalent of <a href="http://doc.qt.io/qt-5/qregexp.html#indexIn">indexIn</a>("^R$") since exactMatch() effectively encloses the regexp in the start of string and end of string anchors, except that it sets <a href="http://doc.qt.io/qt-5/qregexp.html#matchedLength">matchedLength</a>() differently.</p>
    /// <p>For example, if the regular expression is <b>blue</b>, then exactMatch() returns <code>true</code> only for input <code>blue</code>. For inputs <code>bluebell</code>, <code>blutak</code> and <code>lightblue</code>, exactMatch() returns <code>false</code> and <a href="http://doc.qt.io/qt-5/qregexp.html#matchedLength">matchedLength</a>() will return 4, 3 and 0 respectively.</p>
    /// <p>Although const, this function sets <a href="http://doc.qt.io/qt-5/qregexp.html#matchedLength">matchedLength</a>(), <a href="http://doc.qt.io/qt-5/qregexp.html#capturedTexts">capturedTexts</a>(), and <a href="http://doc.qt.io/qt-5/qregexp.html#pos">pos</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#indexIn">indexIn</a>() and <a href="http://doc.qt.io/qt-5/qregexp.html#lastIndexIn">lastIndexIn</a>().</p></div>
    pub unsafe fn exact_match(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRegExp_exactMatch(
            self as *const crate::QRegExp,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(str).as_raw_ptr(),
        )
    }

    /// <p>Attempts to find a match in <i>str</i> from position <i>offset</i> (0 by default). If <i>offset</i> is -1, the search starts at the last character; if -2, at the next to last character; etc.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRegExp::indexIn(const QString& str, int offset = …, QRegExp::CaretMode caretMode = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#indexIn">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to find a match in <i>str</i> from position <i>offset</i> (0 by default). If <i>offset</i> is -1, the search starts at the last character; if -2, at the next to last character; etc.</p>
    /// <p>Returns the position of the first match, or -1 if there was no match.</p>
    /// <p>The <i>caretMode</i> parameter can be used to instruct whether <b>^</b> should match at index 0 or at <i>offset</i>.</p>
    /// <p>You might prefer to use <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">QString::indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#contains">QString::contains</a>(), or even <a href="http://doc.qt.io/qt-5/qstringlist.html#filter">QStringList::filter</a>(). To replace matches use <a href="http://doc.qt.io/qt-5/qstring.html#replace">QString::replace</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> str <span class="operator">=</span> <span class="string">"offsets: 1.23 .50 71.00 6.00"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html#QRegExp">QRegExp</a></span> rx(<span class="string">"\\d*\\.\\d+"</span>);    <span class="comment">// primitive floating point matching</span>
    ///   <span class="type">int</span> count <span class="operator">=</span> <span class="number">0</span>;
    ///   <span class="type">int</span> pos <span class="operator">=</span> <span class="number">0</span>;
    ///   <span class="keyword">while</span> ((pos <span class="operator">=</span> rx<span class="operator">.</span>indexIn(str<span class="operator">,</span> pos)) <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>) {
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>count;
    /// &#32;     pos <span class="operator">+</span><span class="operator">=</span> rx<span class="operator">.</span>matchedLength();
    ///   }
    ///   <span class="comment">// pos will be 9, 14, 18 and finally 24; count will end up as 4</span>
    ///
    /// </pre>
    /// <p>Although const, this function sets <a href="http://doc.qt.io/qt-5/qregexp.html#matchedLength">matchedLength</a>(), <a href="http://doc.qt.io/qt-5/qregexp.html#capturedTexts">capturedTexts</a>() and <a href="http://doc.qt.io/qt-5/qregexp.html#pos">pos</a>().</p>
    /// <p>If the <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> is a wildcard expression (see <a href="http://doc.qt.io/qt-5/qregexp.html#setPatternSyntax">setPatternSyntax</a>()) and want to test a string against the whole wildcard expression, use <a href="http://doc.qt.io/qt-5/qregexp.html#exactMatch">exactMatch</a>() instead of this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#lastIndexIn">lastIndexIn</a>() and <a href="http://doc.qt.io/qt-5/qregexp.html#exactMatch">exactMatch</a>().</p></div>
    pub unsafe fn index_in_3a(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        offset: ::std::os::raw::c_int,
        caret_mode: crate::q_reg_exp::CaretMode,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRegExp_indexIn(
            self as *const crate::QRegExp,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(str).as_raw_ptr(),
            offset,
            caret_mode,
        )
    }

    /// <p>Attempts to find a match in <i>str</i> from position <i>offset</i> (0 by default). If <i>offset</i> is -1, the search starts at the last character; if -2, at the next to last character; etc.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRegExp::indexIn(const QString& str, int offset = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#indexIn">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to find a match in <i>str</i> from position <i>offset</i> (0 by default). If <i>offset</i> is -1, the search starts at the last character; if -2, at the next to last character; etc.</p>
    /// <p>Returns the position of the first match, or -1 if there was no match.</p>
    /// <p>The <i>caretMode</i> parameter can be used to instruct whether <b>^</b> should match at index 0 or at <i>offset</i>.</p>
    /// <p>You might prefer to use <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">QString::indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#contains">QString::contains</a>(), or even <a href="http://doc.qt.io/qt-5/qstringlist.html#filter">QStringList::filter</a>(). To replace matches use <a href="http://doc.qt.io/qt-5/qstring.html#replace">QString::replace</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> str <span class="operator">=</span> <span class="string">"offsets: 1.23 .50 71.00 6.00"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html#QRegExp">QRegExp</a></span> rx(<span class="string">"\\d*\\.\\d+"</span>);    <span class="comment">// primitive floating point matching</span>
    ///   <span class="type">int</span> count <span class="operator">=</span> <span class="number">0</span>;
    ///   <span class="type">int</span> pos <span class="operator">=</span> <span class="number">0</span>;
    ///   <span class="keyword">while</span> ((pos <span class="operator">=</span> rx<span class="operator">.</span>indexIn(str<span class="operator">,</span> pos)) <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>) {
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>count;
    /// &#32;     pos <span class="operator">+</span><span class="operator">=</span> rx<span class="operator">.</span>matchedLength();
    ///   }
    ///   <span class="comment">// pos will be 9, 14, 18 and finally 24; count will end up as 4</span>
    ///
    /// </pre>
    /// <p>Although const, this function sets <a href="http://doc.qt.io/qt-5/qregexp.html#matchedLength">matchedLength</a>(), <a href="http://doc.qt.io/qt-5/qregexp.html#capturedTexts">capturedTexts</a>() and <a href="http://doc.qt.io/qt-5/qregexp.html#pos">pos</a>().</p>
    /// <p>If the <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> is a wildcard expression (see <a href="http://doc.qt.io/qt-5/qregexp.html#setPatternSyntax">setPatternSyntax</a>()) and want to test a string against the whole wildcard expression, use <a href="http://doc.qt.io/qt-5/qregexp.html#exactMatch">exactMatch</a>() instead of this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#lastIndexIn">lastIndexIn</a>() and <a href="http://doc.qt.io/qt-5/qregexp.html#exactMatch">exactMatch</a>().</p></div>
    pub unsafe fn index_in_2a(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        offset: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRegExp_indexIn1(
            self as *const crate::QRegExp,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(str).as_raw_ptr(),
            offset,
        )
    }

    /// <p>Attempts to find a match in <i>str</i> from position <i>offset</i> (0 by default). If <i>offset</i> is -1, the search starts at the last character; if -2, at the next to last character; etc.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRegExp::indexIn(const QString& str) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#indexIn">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to find a match in <i>str</i> from position <i>offset</i> (0 by default). If <i>offset</i> is -1, the search starts at the last character; if -2, at the next to last character; etc.</p>
    /// <p>Returns the position of the first match, or -1 if there was no match.</p>
    /// <p>The <i>caretMode</i> parameter can be used to instruct whether <b>^</b> should match at index 0 or at <i>offset</i>.</p>
    /// <p>You might prefer to use <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">QString::indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#contains">QString::contains</a>(), or even <a href="http://doc.qt.io/qt-5/qstringlist.html#filter">QStringList::filter</a>(). To replace matches use <a href="http://doc.qt.io/qt-5/qstring.html#replace">QString::replace</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> str <span class="operator">=</span> <span class="string">"offsets: 1.23 .50 71.00 6.00"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html#QRegExp">QRegExp</a></span> rx(<span class="string">"\\d*\\.\\d+"</span>);    <span class="comment">// primitive floating point matching</span>
    ///   <span class="type">int</span> count <span class="operator">=</span> <span class="number">0</span>;
    ///   <span class="type">int</span> pos <span class="operator">=</span> <span class="number">0</span>;
    ///   <span class="keyword">while</span> ((pos <span class="operator">=</span> rx<span class="operator">.</span>indexIn(str<span class="operator">,</span> pos)) <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>) {
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>count;
    /// &#32;     pos <span class="operator">+</span><span class="operator">=</span> rx<span class="operator">.</span>matchedLength();
    ///   }
    ///   <span class="comment">// pos will be 9, 14, 18 and finally 24; count will end up as 4</span>
    ///
    /// </pre>
    /// <p>Although const, this function sets <a href="http://doc.qt.io/qt-5/qregexp.html#matchedLength">matchedLength</a>(), <a href="http://doc.qt.io/qt-5/qregexp.html#capturedTexts">capturedTexts</a>() and <a href="http://doc.qt.io/qt-5/qregexp.html#pos">pos</a>().</p>
    /// <p>If the <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> is a wildcard expression (see <a href="http://doc.qt.io/qt-5/qregexp.html#setPatternSyntax">setPatternSyntax</a>()) and want to test a string against the whole wildcard expression, use <a href="http://doc.qt.io/qt-5/qregexp.html#exactMatch">exactMatch</a>() instead of this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#lastIndexIn">lastIndexIn</a>() and <a href="http://doc.qt.io/qt-5/qregexp.html#exactMatch">exactMatch</a>().</p></div>
    pub unsafe fn index_in_1a(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRegExp_indexIn2(
            self as *const crate::QRegExp,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(str).as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if the pattern string is empty; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRegExp::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the pattern string is empty; otherwise returns false.</p>
    /// <p>If you call <a href="http://doc.qt.io/qt-5/qregexp.html#exactMatch">exactMatch</a>() with an empty pattern on an empty string it will return true; otherwise it returns <code>false</code> since it operates over the whole string. If you call <a href="http://doc.qt.io/qt-5/qregexp.html#indexIn">indexIn</a>() with an empty pattern on <i>any</i> string it will return the start offset (0 by default) because the empty pattern matches the 'emptiness' at the start of the string. In this case the length of the match returned by <a href="http://doc.qt.io/qt-5/qregexp.html#matchedLength">matchedLength</a>() will be 0.</p>
    /// <p>See <a href="http://doc.qt.io/qt-5/qstring.html#isEmpty">QString::isEmpty</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRegExp_isEmpty(self as *const crate::QRegExp)
    }

    /// <p>Returns <code>true</code> if minimal (non-greedy) matching is enabled; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRegExp::isMinimal() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#isMinimal">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if minimal (non-greedy) matching is enabled; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#caseSensitivity">caseSensitivity</a>() and <a href="http://doc.qt.io/qt-5/qregexp.html#setMinimal">setMinimal</a>().</p></div>
    pub unsafe fn is_minimal(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRegExp_isMinimal(self as *const crate::QRegExp)
    }

    /// <p>Returns <code>true</code> if the regular expression is valid; otherwise returns false. An invalid regular expression never matches.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRegExp::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the regular expression is valid; otherwise returns false. An invalid regular expression never matches.</p>
    /// <p>The pattern <b>[a-z</b> is an example of an invalid pattern, since it lacks a closing square bracket.</p>
    /// <p>Note that the validity of a regexp may also depend on the setting of the wildcard flag, for example <b>*.html</b> is a valid wildcard regexp but an invalid full regexp.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#errorString">errorString</a>().</p></div>
    pub unsafe fn is_valid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRegExp_isValid(self as *const crate::QRegExp)
    }

    /// <p>Attempts to find a match backwards in <i>str</i> from position <i>offset</i>. If <i>offset</i> is -1 (the default), the search starts at the last character; if -2, at the next to last character; etc.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRegExp::lastIndexIn(const QString& str, int offset = …, QRegExp::CaretMode caretMode = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#lastIndexIn">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to find a match backwards in <i>str</i> from position <i>offset</i>. If <i>offset</i> is -1 (the default), the search starts at the last character; if -2, at the next to last character; etc.</p>
    /// <p>Returns the position of the first match, or -1 if there was no match.</p>
    /// <p>The <i>caretMode</i> parameter can be used to instruct whether <b>^</b> should match at index 0 or at <i>offset</i>.</p>
    /// <p>Although const, this function sets <a href="http://doc.qt.io/qt-5/qregexp.html#matchedLength">matchedLength</a>(), <a href="http://doc.qt.io/qt-5/qregexp.html#capturedTexts">capturedTexts</a>() and <a href="http://doc.qt.io/qt-5/qregexp.html#pos">pos</a>().</p>
    /// <p><b>Warning:</b> Searching backwards is much slower than searching forwards.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#indexIn">indexIn</a>() and <a href="http://doc.qt.io/qt-5/qregexp.html#exactMatch">exactMatch</a>().</p></div>
    pub unsafe fn last_index_in_3a(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        offset: ::std::os::raw::c_int,
        caret_mode: crate::q_reg_exp::CaretMode,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRegExp_lastIndexIn(
            self as *const crate::QRegExp,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(str).as_raw_ptr(),
            offset,
            caret_mode,
        )
    }

    /// <p>Attempts to find a match backwards in <i>str</i> from position <i>offset</i>. If <i>offset</i> is -1 (the default), the search starts at the last character; if -2, at the next to last character; etc.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRegExp::lastIndexIn(const QString& str, int offset = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#lastIndexIn">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to find a match backwards in <i>str</i> from position <i>offset</i>. If <i>offset</i> is -1 (the default), the search starts at the last character; if -2, at the next to last character; etc.</p>
    /// <p>Returns the position of the first match, or -1 if there was no match.</p>
    /// <p>The <i>caretMode</i> parameter can be used to instruct whether <b>^</b> should match at index 0 or at <i>offset</i>.</p>
    /// <p>Although const, this function sets <a href="http://doc.qt.io/qt-5/qregexp.html#matchedLength">matchedLength</a>(), <a href="http://doc.qt.io/qt-5/qregexp.html#capturedTexts">capturedTexts</a>() and <a href="http://doc.qt.io/qt-5/qregexp.html#pos">pos</a>().</p>
    /// <p><b>Warning:</b> Searching backwards is much slower than searching forwards.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#indexIn">indexIn</a>() and <a href="http://doc.qt.io/qt-5/qregexp.html#exactMatch">exactMatch</a>().</p></div>
    pub unsafe fn last_index_in_2a(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        offset: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRegExp_lastIndexIn1(
            self as *const crate::QRegExp,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(str).as_raw_ptr(),
            offset,
        )
    }

    /// <p>Attempts to find a match backwards in <i>str</i> from position <i>offset</i>. If <i>offset</i> is -1 (the default), the search starts at the last character; if -2, at the next to last character; etc.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRegExp::lastIndexIn(const QString& str) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#lastIndexIn">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to find a match backwards in <i>str</i> from position <i>offset</i>. If <i>offset</i> is -1 (the default), the search starts at the last character; if -2, at the next to last character; etc.</p>
    /// <p>Returns the position of the first match, or -1 if there was no match.</p>
    /// <p>The <i>caretMode</i> parameter can be used to instruct whether <b>^</b> should match at index 0 or at <i>offset</i>.</p>
    /// <p>Although const, this function sets <a href="http://doc.qt.io/qt-5/qregexp.html#matchedLength">matchedLength</a>(), <a href="http://doc.qt.io/qt-5/qregexp.html#capturedTexts">capturedTexts</a>() and <a href="http://doc.qt.io/qt-5/qregexp.html#pos">pos</a>().</p>
    /// <p><b>Warning:</b> Searching backwards is much slower than searching forwards.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#indexIn">indexIn</a>() and <a href="http://doc.qt.io/qt-5/qregexp.html#exactMatch">exactMatch</a>().</p></div>
    pub unsafe fn last_index_in_1a(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRegExp_lastIndexIn2(
            self as *const crate::QRegExp,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(str).as_raw_ptr(),
        )
    }

    /// <p>Returns the length of the last matched string, or -1 if there was no match.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRegExp::matchedLength() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#matchedLength">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the length of the last matched string, or -1 if there was no match.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#exactMatch">exactMatch</a>(), <a href="http://doc.qt.io/qt-5/qregexp.html#indexIn">indexIn</a>(), and <a href="http://doc.qt.io/qt-5/qregexp.html#lastIndexIn">lastIndexIn</a>().</p></div>
    pub unsafe fn matched_length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRegExp_matchedLength(self as *const crate::QRegExp)
    }

    /// <p>Constructs an empty regexp.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRegExp::QRegExp()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#QRegExp">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty regexp.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#isValid">isValid</a>() and <a href="http://doc.qt.io/qt-5/qregexp.html#errorString">errorString</a>().</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QRegExp> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegExp_QRegExp();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a regular expression object for the given <i>pattern</i> string. The pattern must be given using wildcard notation if <i>syntax</i> is <a href="http://doc.qt.io/qt-5/qregexp.html#PatternSyntax-enum">Wildcard</a>; the default is <a href="http://doc.qt.io/qt-5/qregexp.html#PatternSyntax-enum">RegExp</a>. The pattern is case sensitive, unless <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseInsensitive</a>. Matching is greedy (maximal), but can be changed by calling <a href="http://doc.qt.io/qt-5/qregexp.html#setMinimal">setMinimal</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRegExp::QRegExp(const QString& pattern, Qt::CaseSensitivity cs = …, QRegExp::PatternSyntax syntax = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#QRegExp-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a regular expression object for the given <i>pattern</i> string. The pattern must be given using wildcard notation if <i>syntax</i> is <a href="http://doc.qt.io/qt-5/qregexp.html#PatternSyntax-enum">Wildcard</a>; the default is <a href="http://doc.qt.io/qt-5/qregexp.html#PatternSyntax-enum">RegExp</a>. The pattern is case sensitive, unless <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseInsensitive</a>. Matching is greedy (maximal), but can be changed by calling <a href="http://doc.qt.io/qt-5/qregexp.html#setMinimal">setMinimal</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#setPattern">setPattern</a>(), <a href="http://doc.qt.io/qt-5/qregexp.html#setCaseSensitivity">setCaseSensitivity</a>(), and <a href="http://doc.qt.io/qt-5/qregexp.html#setPatternSyntax">setPatternSyntax</a>().</p></div>
    pub unsafe fn new_3a(
        pattern: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        cs: crate::CaseSensitivity,
        syntax: crate::q_reg_exp::PatternSyntax,
    ) -> ::cpp_utils::CppBox<crate::QRegExp> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegExp_QRegExp1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(pattern)
                .as_raw_ptr(),
            cs,
            syntax,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a regular expression object for the given <i>pattern</i> string. The pattern must be given using wildcard notation if <i>syntax</i> is <a href="http://doc.qt.io/qt-5/qregexp.html#PatternSyntax-enum">Wildcard</a>; the default is <a href="http://doc.qt.io/qt-5/qregexp.html#PatternSyntax-enum">RegExp</a>. The pattern is case sensitive, unless <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseInsensitive</a>. Matching is greedy (maximal), but can be changed by calling <a href="http://doc.qt.io/qt-5/qregexp.html#setMinimal">setMinimal</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRegExp::QRegExp(const QString& pattern, Qt::CaseSensitivity cs = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#QRegExp-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a regular expression object for the given <i>pattern</i> string. The pattern must be given using wildcard notation if <i>syntax</i> is <a href="http://doc.qt.io/qt-5/qregexp.html#PatternSyntax-enum">Wildcard</a>; the default is <a href="http://doc.qt.io/qt-5/qregexp.html#PatternSyntax-enum">RegExp</a>. The pattern is case sensitive, unless <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseInsensitive</a>. Matching is greedy (maximal), but can be changed by calling <a href="http://doc.qt.io/qt-5/qregexp.html#setMinimal">setMinimal</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#setPattern">setPattern</a>(), <a href="http://doc.qt.io/qt-5/qregexp.html#setCaseSensitivity">setCaseSensitivity</a>(), and <a href="http://doc.qt.io/qt-5/qregexp.html#setPatternSyntax">setPatternSyntax</a>().</p></div>
    pub unsafe fn new_2a(
        pattern: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        cs: crate::CaseSensitivity,
    ) -> ::cpp_utils::CppBox<crate::QRegExp> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegExp_QRegExp3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(pattern)
                .as_raw_ptr(),
            cs,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a regular expression object for the given <i>pattern</i> string. The pattern must be given using wildcard notation if <i>syntax</i> is <a href="http://doc.qt.io/qt-5/qregexp.html#PatternSyntax-enum">Wildcard</a>; the default is <a href="http://doc.qt.io/qt-5/qregexp.html#PatternSyntax-enum">RegExp</a>. The pattern is case sensitive, unless <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseInsensitive</a>. Matching is greedy (maximal), but can be changed by calling <a href="http://doc.qt.io/qt-5/qregexp.html#setMinimal">setMinimal</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRegExp::QRegExp(const QString& pattern)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#QRegExp-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a regular expression object for the given <i>pattern</i> string. The pattern must be given using wildcard notation if <i>syntax</i> is <a href="http://doc.qt.io/qt-5/qregexp.html#PatternSyntax-enum">Wildcard</a>; the default is <a href="http://doc.qt.io/qt-5/qregexp.html#PatternSyntax-enum">RegExp</a>. The pattern is case sensitive, unless <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseInsensitive</a>. Matching is greedy (maximal), but can be changed by calling <a href="http://doc.qt.io/qt-5/qregexp.html#setMinimal">setMinimal</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#setPattern">setPattern</a>(), <a href="http://doc.qt.io/qt-5/qregexp.html#setCaseSensitivity">setCaseSensitivity</a>(), and <a href="http://doc.qt.io/qt-5/qregexp.html#setPatternSyntax">setPatternSyntax</a>().</p></div>
    pub unsafe fn new_1a(
        pattern: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QRegExp> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegExp_QRegExp4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(pattern)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a regular expression as a copy of <i>rx</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRegExp::QRegExp(const QRegExp& rx)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#QRegExp-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a regular expression as a copy of <i>rx</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        rx: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
    ) -> ::cpp_utils::CppBox<crate::QRegExp> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegExp_QRegExp2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(rx).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the pattern string of the regular expression. The pattern has either regular expression syntax or wildcard syntax, depending on <a href="http://doc.qt.io/qt-5/qregexp.html#patternSyntax">patternSyntax</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QRegExp::pattern() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#pattern">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the pattern string of the regular expression. The pattern has either regular expression syntax or wildcard syntax, depending on <a href="http://doc.qt.io/qt-5/qregexp.html#patternSyntax">patternSyntax</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#setPattern">setPattern</a>(), <a href="http://doc.qt.io/qt-5/qregexp.html#patternSyntax">patternSyntax</a>(), and <a href="http://doc.qt.io/qt-5/qregexp.html#caseSensitivity">caseSensitivity</a>().</p></div>
    pub unsafe fn pattern(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QRegExp_pattern(self as *const crate::QRegExp);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the syntax used by the regular expression. The default is <a href="http://doc.qt.io/qt-5/qregexp.html#PatternSyntax-enum">QRegExp::RegExp</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegExp::PatternSyntax QRegExp::patternSyntax() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#patternSyntax">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the syntax used by the regular expression. The default is <a href="http://doc.qt.io/qt-5/qregexp.html#PatternSyntax-enum">QRegExp::RegExp</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#setPatternSyntax">setPatternSyntax</a>(), <a href="http://doc.qt.io/qt-5/qregexp.html#pattern">pattern</a>(), and <a href="http://doc.qt.io/qt-5/qregexp.html#caseSensitivity">caseSensitivity</a>().</p></div>
    pub unsafe fn pattern_syntax(&self) -> crate::q_reg_exp::PatternSyntax {
        crate::__ffi::ctr_qt_core_ffi_QRegExp_patternSyntax(self as *const crate::QRegExp)
    }

    /// <p>Returns the position of the <i>nth</i> captured text in the searched string. If <i>nth</i> is 0 (the default), pos() returns the position of the whole match.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRegExp::pos(int nth = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#pos">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the position of the <i>nth</i> captured text in the searched string. If <i>nth</i> is 0 (the default), pos() returns the position of the whole match.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html#QRegExp">QRegExp</a></span> rx(<span class="string">"/([a-z]+)/([a-z]+)"</span>);
    ///   rx<span class="operator">.</span>indexIn(<span class="string">"Output /dev/null"</span>);   <span class="comment">// returns 7 (position of /dev/null)</span>
    ///   rx<span class="operator">.</span>pos(<span class="number">0</span>);                        <span class="comment">// returns 7 (position of /dev/null)</span>
    ///   rx<span class="operator">.</span>pos(<span class="number">1</span>);                        <span class="comment">// returns 8 (position of dev)</span>
    ///   rx<span class="operator">.</span>pos(<span class="number">2</span>);                        <span class="comment">// returns 12 (position of null)</span>
    ///
    /// </pre>
    /// <p>For zero-length matches, pos() always returns -1. (For example, if cap(4) would return an empty string, pos(4) returns -1.) This is a feature of the implementation.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#cap">cap</a>() and <a href="http://doc.qt.io/qt-5/qregexp.html#capturedTexts">capturedTexts</a>().</p></div>
    pub unsafe fn pos_1a(&self, nth: ::std::os::raw::c_int) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRegExp_pos(self as *const crate::QRegExp, nth)
    }

    /// <p>Returns the position of the <i>nth</i> captured text in the searched string. If <i>nth</i> is 0 (the default), pos() returns the position of the whole match.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRegExp::pos(int nth = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#pos">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the position of the <i>nth</i> captured text in the searched string. If <i>nth</i> is 0 (the default), pos() returns the position of the whole match.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html#QRegExp">QRegExp</a></span> rx(<span class="string">"/([a-z]+)/([a-z]+)"</span>);
    ///   rx<span class="operator">.</span>indexIn(<span class="string">"Output /dev/null"</span>);   <span class="comment">// returns 7 (position of /dev/null)</span>
    ///   rx<span class="operator">.</span>pos(<span class="number">0</span>);                        <span class="comment">// returns 7 (position of /dev/null)</span>
    ///   rx<span class="operator">.</span>pos(<span class="number">1</span>);                        <span class="comment">// returns 8 (position of dev)</span>
    ///   rx<span class="operator">.</span>pos(<span class="number">2</span>);                        <span class="comment">// returns 12 (position of null)</span>
    ///
    /// </pre>
    /// <p>For zero-length matches, pos() always returns -1. (For example, if cap(4) would return an empty string, pos(4) returns -1.) This is a feature of the implementation.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#cap">cap</a>() and <a href="http://doc.qt.io/qt-5/qregexp.html#capturedTexts">capturedTexts</a>().</p></div>
    pub unsafe fn pos_1a_mut(&mut self, nth: ::std::os::raw::c_int) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRegExp_pos1(self as *mut crate::QRegExp, nth)
    }

    /// <p>Returns the position of the <i>nth</i> captured text in the searched string. If <i>nth</i> is 0 (the default), pos() returns the position of the whole match.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRegExp::pos() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#pos">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the position of the <i>nth</i> captured text in the searched string. If <i>nth</i> is 0 (the default), pos() returns the position of the whole match.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html#QRegExp">QRegExp</a></span> rx(<span class="string">"/([a-z]+)/([a-z]+)"</span>);
    ///   rx<span class="operator">.</span>indexIn(<span class="string">"Output /dev/null"</span>);   <span class="comment">// returns 7 (position of /dev/null)</span>
    ///   rx<span class="operator">.</span>pos(<span class="number">0</span>);                        <span class="comment">// returns 7 (position of /dev/null)</span>
    ///   rx<span class="operator">.</span>pos(<span class="number">1</span>);                        <span class="comment">// returns 8 (position of dev)</span>
    ///   rx<span class="operator">.</span>pos(<span class="number">2</span>);                        <span class="comment">// returns 12 (position of null)</span>
    ///
    /// </pre>
    /// <p>For zero-length matches, pos() always returns -1. (For example, if cap(4) would return an empty string, pos(4) returns -1.) This is a feature of the implementation.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#cap">cap</a>() and <a href="http://doc.qt.io/qt-5/qregexp.html#capturedTexts">capturedTexts</a>().</p></div>
    pub unsafe fn pos_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRegExp_pos2(self as *const crate::QRegExp)
    }

    /// <p>Returns the position of the <i>nth</i> captured text in the searched string. If <i>nth</i> is 0 (the default), pos() returns the position of the whole match.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRegExp::pos()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#pos">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the position of the <i>nth</i> captured text in the searched string. If <i>nth</i> is 0 (the default), pos() returns the position of the whole match.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qregexp.html#QRegExp">QRegExp</a></span> rx(<span class="string">"/([a-z]+)/([a-z]+)"</span>);
    ///   rx<span class="operator">.</span>indexIn(<span class="string">"Output /dev/null"</span>);   <span class="comment">// returns 7 (position of /dev/null)</span>
    ///   rx<span class="operator">.</span>pos(<span class="number">0</span>);                        <span class="comment">// returns 7 (position of /dev/null)</span>
    ///   rx<span class="operator">.</span>pos(<span class="number">1</span>);                        <span class="comment">// returns 8 (position of dev)</span>
    ///   rx<span class="operator">.</span>pos(<span class="number">2</span>);                        <span class="comment">// returns 12 (position of null)</span>
    ///
    /// </pre>
    /// <p>For zero-length matches, pos() always returns -1. (For example, if cap(4) would return an empty string, pos(4) returns -1.) This is a feature of the implementation.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#cap">cap</a>() and <a href="http://doc.qt.io/qt-5/qregexp.html#capturedTexts">capturedTexts</a>().</p></div>
    pub unsafe fn pos_0a_mut(&mut self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRegExp_pos3(self as *mut crate::QRegExp)
    }

    /// <p>Sets case sensitive matching to <i>cs</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRegExp::setCaseSensitivity(Qt::CaseSensitivity cs)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#setCaseSensitivity">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets case sensitive matching to <i>cs</i>.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a>, <b>\.txt$</b> matches <code>readme.txt</code> but not <code>README.TXT</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#caseSensitivity">caseSensitivity</a>(), <a href="http://doc.qt.io/qt-5/qregexp.html#setPatternSyntax">setPatternSyntax</a>(), <a href="http://doc.qt.io/qt-5/qregexp.html#setPattern">setPattern</a>(), and <a href="http://doc.qt.io/qt-5/qregexp.html#setMinimal">setMinimal</a>().</p></div>
    pub unsafe fn set_case_sensitivity(&mut self, cs: crate::CaseSensitivity) {
        crate::__ffi::ctr_qt_core_ffi_QRegExp_setCaseSensitivity(self as *mut crate::QRegExp, cs)
    }

    /// <p>Enables or disables minimal matching. If <i>minimal</i> is false, matching is greedy (maximal) which is the default.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRegExp::setMinimal(bool minimal)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#setMinimal">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Enables or disables minimal matching. If <i>minimal</i> is false, matching is greedy (maximal) which is the default.</p>
    /// <p>For example, suppose we have the input string "We must be &lt;b&gt;bold&lt;/b&gt;, very &lt;b&gt;bold&lt;/b&gt;!" and the pattern <b>&lt;b&gt;.*&lt;/b&gt;</b>. With the default greedy (maximal) matching, the match is "We must be <u>&lt;b&gt;bold&lt;/b&gt;, very &lt;b&gt;bold&lt;/b&gt;</u>!". But with minimal (non-greedy) matching, the first match is: "We must be <u>&lt;b&gt;bold&lt;/b&gt;</u>, very &lt;b&gt;bold&lt;/b&gt;!" and the second match is "We must be &lt;b&gt;bold&lt;/b&gt;, very <u>&lt;b&gt;bold&lt;/b&gt;</u>!". In practice we might use the pattern <b>&lt;b&gt;[^&lt;]*&lt;/b&gt;</b> instead, although this will still fail for nested tags.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#isMinimal">isMinimal</a>() and <a href="http://doc.qt.io/qt-5/qregexp.html#setCaseSensitivity">setCaseSensitivity</a>().</p></div>
    pub unsafe fn set_minimal(&mut self, minimal: bool) {
        crate::__ffi::ctr_qt_core_ffi_QRegExp_setMinimal(self as *mut crate::QRegExp, minimal)
    }

    /// <p>Sets the pattern string to <i>pattern</i>. The case sensitivity, wildcard, and minimal matching options are not changed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRegExp::setPattern(const QString& pattern)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#setPattern">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the pattern string to <i>pattern</i>. The case sensitivity, wildcard, and minimal matching options are not changed.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#pattern">pattern</a>(), <a href="http://doc.qt.io/qt-5/qregexp.html#setPatternSyntax">setPatternSyntax</a>(), and <a href="http://doc.qt.io/qt-5/qregexp.html#setCaseSensitivity">setCaseSensitivity</a>().</p></div>
    pub unsafe fn set_pattern(
        &mut self,
        pattern: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRegExp_setPattern(
            self as *mut crate::QRegExp,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(pattern)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the syntax mode for the regular expression. The default is <a href="http://doc.qt.io/qt-5/qregexp.html#PatternSyntax-enum">QRegExp::RegExp</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRegExp::setPatternSyntax(QRegExp::PatternSyntax syntax)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#setPatternSyntax">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the syntax mode for the regular expression. The default is <a href="http://doc.qt.io/qt-5/qregexp.html#PatternSyntax-enum">QRegExp::RegExp</a>.</p>
    /// <p>Setting <i>syntax</i> to <a href="http://doc.qt.io/qt-5/qregexp.html#PatternSyntax-enum">QRegExp::Wildcard</a> enables simple shell-like <a href="http://doc.qt.io/qt-5/qregexp.html#qregexp-wildcard-matching">QRegExp wildcard matching</a>. For example, <b>r*.txt</b> matches the string <code>readme.txt</code> in wildcard mode, but does not match <code>readme</code>.</p>
    /// <p>Setting <i>syntax</i> to <a href="http://doc.qt.io/qt-5/qregexp.html#PatternSyntax-enum">QRegExp::FixedString</a> means that the pattern is interpreted as a plain string. Special characters (e.g., backslash) don't need to be escaped then.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#patternSyntax">patternSyntax</a>(), <a href="http://doc.qt.io/qt-5/qregexp.html#setPattern">setPattern</a>(), <a href="http://doc.qt.io/qt-5/qregexp.html#setCaseSensitivity">setCaseSensitivity</a>(), and <a href="http://doc.qt.io/qt-5/qregexp.html#escape">escape</a>().</p></div>
    pub unsafe fn set_pattern_syntax(&mut self, syntax: crate::q_reg_exp::PatternSyntax) {
        crate::__ffi::ctr_qt_core_ffi_QRegExp_setPatternSyntax(self as *mut crate::QRegExp, syntax)
    }

    /// <p>Swaps regular expression <i>other</i> with this regular expression. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRegExp::swap(QRegExp& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps regular expression <i>other</i> with this regular expression. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QRegExp>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRegExp_swap(
            self as *mut crate::QRegExp,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QRegExp>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qstringmatcher.html">QStringMatcher</a> class holds a sequence of characters that can be quickly matched in a Unicode string.</p>
///
/// C++ class: <span style='color: green;'>```QStringMatcher```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qstringmatcher.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qstringmatcher.html">QStringMatcher</a> class holds a sequence of characters that can be quickly matched in a Unicode string.</p>
/// <p>This class is useful when you have a sequence of <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>s that you want to repeatedly match against some strings (perhaps in a loop), or when you want to search for the same sequence of characters multiple times in the same string. Using a matcher object and <a href="http://doc.qt.io/qt-5/qstringmatcher.html#indexIn">indexIn</a>() is faster than matching a plain <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> with <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">QString::indexOf</a>() if repeated matching takes place. This class offers no benefit if you are doing one-off string matches.</p>
/// <p>Create the <a href="http://doc.qt.io/qt-5/qstringmatcher.html">QStringMatcher</a> with the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> you want to search for. Then call <a href="http://doc.qt.io/qt-5/qstringmatcher.html#indexIn">indexIn</a>() on the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> that you want to search.</p></div>
#[repr(C)]
pub struct QStringMatcher {
    _unused: u8,
}
impl QStringMatcher {
    /// <p>Returns the case sensitivity setting for this string matcher.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```Qt::CaseSensitivity QStringMatcher::caseSensitivity() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringmatcher.html#caseSensitivity">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the case sensitivity setting for this string matcher.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringmatcher.html#setCaseSensitivity">setCaseSensitivity</a>().</p></div>
    pub unsafe fn case_sensitivity(&self) -> crate::CaseSensitivity {
        crate::__ffi::ctr_qt_core_ffi_QStringMatcher_caseSensitivity(
            self as *const crate::QStringMatcher,
        )
    }

    /// <p>Assigns the <i>other</i> string matcher to this string matcher.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringMatcher& QStringMatcher::operator=(const QStringMatcher& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringmatcher.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns the <i>other</i> string matcher to this string matcher.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringMatcher>>,
    ) -> ::cpp_utils::MutRef<crate::QStringMatcher> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringMatcher_operator_(
            self as *mut crate::QStringMatcher,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringMatcher>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Searches the string <i>str</i> from character position <i>from</i> (default 0, i.e. from the first character), for the string <a href="http://doc.qt.io/qt-5/qstringmatcher.html#pattern">pattern</a>() that was set in the constructor or in the most recent call to <a href="http://doc.qt.io/qt-5/qstringmatcher.html#setPattern">setPattern</a>(). Returns the position where the <a href="http://doc.qt.io/qt-5/qstringmatcher.html#pattern">pattern</a>() matched in <i>str</i>, or -1 if no match was found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringMatcher::indexIn(const QString& str, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringmatcher.html#indexIn">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Searches the string <i>str</i> from character position <i>from</i> (default 0, i.e. from the first character), for the string <a href="http://doc.qt.io/qt-5/qstringmatcher.html#pattern">pattern</a>() that was set in the constructor or in the most recent call to <a href="http://doc.qt.io/qt-5/qstringmatcher.html#setPattern">setPattern</a>(). Returns the position where the <a href="http://doc.qt.io/qt-5/qstringmatcher.html#pattern">pattern</a>() matched in <i>str</i>, or -1 if no match was found.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringmatcher.html#setPattern">setPattern</a>() and <a href="http://doc.qt.io/qt-5/qstringmatcher.html#setCaseSensitivity">setCaseSensitivity</a>().</p></div>
    pub unsafe fn index_in_q_string_int(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringMatcher_indexIn(
            self as *const crate::QStringMatcher,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(str).as_raw_ptr(),
            from,
        )
    }

    /// <p>Searches the string starting at <i>str</i> (of length <i>length</i>) from character position <i>from</i> (default 0, i.e. from the first character), for the string <a href="http://doc.qt.io/qt-5/qstringmatcher.html#pattern">pattern</a>() that was set in the constructor or in the most recent call to <a href="http://doc.qt.io/qt-5/qstringmatcher.html#setPattern">setPattern</a>(). Returns the position where the <a href="http://doc.qt.io/qt-5/qstringmatcher.html#pattern">pattern</a>() matched in <i>str</i>, or -1 if no match was found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringMatcher::indexIn(const QChar* str, int length, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringmatcher.html#indexIn-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Searches the string starting at <i>str</i> (of length <i>length</i>) from character position <i>from</i> (default 0, i.e. from the first character), for the string <a href="http://doc.qt.io/qt-5/qstringmatcher.html#pattern">pattern</a>() that was set in the constructor or in the most recent call to <a href="http://doc.qt.io/qt-5/qstringmatcher.html#setPattern">setPattern</a>(). Returns the position where the <a href="http://doc.qt.io/qt-5/qstringmatcher.html#pattern">pattern</a>() matched in <i>str</i>, or -1 if no match was found.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringmatcher.html#setPattern">setPattern</a>() and <a href="http://doc.qt.io/qt-5/qstringmatcher.html#setCaseSensitivity">setCaseSensitivity</a>().</p></div>
    pub unsafe fn index_in_q_char2_int(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QChar>>,
        length: ::std::os::raw::c_int,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringMatcher_indexIn1(
            self as *const crate::QStringMatcher,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QChar>>::cast_into(str).as_raw_ptr(),
            length,
            from,
        )
    }

    /// <p>Searches the string <i>str</i> from character position <i>from</i> (default 0, i.e. from the first character), for the string <a href="http://doc.qt.io/qt-5/qstringmatcher.html#pattern">pattern</a>() that was set in the constructor or in the most recent call to <a href="http://doc.qt.io/qt-5/qstringmatcher.html#setPattern">setPattern</a>(). Returns the position where the <a href="http://doc.qt.io/qt-5/qstringmatcher.html#pattern">pattern</a>() matched in <i>str</i>, or -1 if no match was found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringMatcher::indexIn(const QString& str) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringmatcher.html#indexIn">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Searches the string <i>str</i> from character position <i>from</i> (default 0, i.e. from the first character), for the string <a href="http://doc.qt.io/qt-5/qstringmatcher.html#pattern">pattern</a>() that was set in the constructor or in the most recent call to <a href="http://doc.qt.io/qt-5/qstringmatcher.html#setPattern">setPattern</a>(). Returns the position where the <a href="http://doc.qt.io/qt-5/qstringmatcher.html#pattern">pattern</a>() matched in <i>str</i>, or -1 if no match was found.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringmatcher.html#setPattern">setPattern</a>() and <a href="http://doc.qt.io/qt-5/qstringmatcher.html#setCaseSensitivity">setCaseSensitivity</a>().</p></div>
    pub unsafe fn index_in_q_string(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringMatcher_indexIn2(
            self as *const crate::QStringMatcher,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(str).as_raw_ptr(),
        )
    }

    /// <p>Searches the string starting at <i>str</i> (of length <i>length</i>) from character position <i>from</i> (default 0, i.e. from the first character), for the string <a href="http://doc.qt.io/qt-5/qstringmatcher.html#pattern">pattern</a>() that was set in the constructor or in the most recent call to <a href="http://doc.qt.io/qt-5/qstringmatcher.html#setPattern">setPattern</a>(). Returns the position where the <a href="http://doc.qt.io/qt-5/qstringmatcher.html#pattern">pattern</a>() matched in <i>str</i>, or -1 if no match was found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringMatcher::indexIn(const QChar* str, int length) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringmatcher.html#indexIn-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Searches the string starting at <i>str</i> (of length <i>length</i>) from character position <i>from</i> (default 0, i.e. from the first character), for the string <a href="http://doc.qt.io/qt-5/qstringmatcher.html#pattern">pattern</a>() that was set in the constructor or in the most recent call to <a href="http://doc.qt.io/qt-5/qstringmatcher.html#setPattern">setPattern</a>(). Returns the position where the <a href="http://doc.qt.io/qt-5/qstringmatcher.html#pattern">pattern</a>() matched in <i>str</i>, or -1 if no match was found.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringmatcher.html#setPattern">setPattern</a>() and <a href="http://doc.qt.io/qt-5/qstringmatcher.html#setCaseSensitivity">setCaseSensitivity</a>().</p></div>
    pub unsafe fn index_in_q_char_int(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QChar>>,
        length: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringMatcher_indexIn3(
            self as *const crate::QStringMatcher,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QChar>>::cast_into(str).as_raw_ptr(),
            length,
        )
    }

    /// <p>Constructs an empty string matcher that won't match anything. Call <a href="http://doc.qt.io/qt-5/qstringmatcher.html#setPattern">setPattern</a>() to give it a pattern to match.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStringMatcher::QStringMatcher()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringmatcher.html#QStringMatcher">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty string matcher that won't match anything. Call <a href="http://doc.qt.io/qt-5/qstringmatcher.html#setPattern">setPattern</a>() to give it a pattern to match.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QStringMatcher> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringMatcher_QStringMatcher();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a string matcher that will search for <i>pattern</i>, with case sensitivity <i>cs</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStringMatcher::QStringMatcher(const QString& pattern, Qt::CaseSensitivity cs = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringmatcher.html#QStringMatcher-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a string matcher that will search for <i>pattern</i>, with case sensitivity <i>cs</i>.</p>
    /// <p>Call <a href="http://doc.qt.io/qt-5/qstringmatcher.html#indexIn">indexIn</a>() to perform a search.</p></div>
    pub unsafe fn from_q_string_case_sensitivity(
        pattern: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        cs: crate::CaseSensitivity,
    ) -> ::cpp_utils::CppBox<crate::QStringMatcher> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringMatcher_QStringMatcher1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(pattern)
                .as_raw_ptr(),
            cs,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a string matcher that will search for the pattern referred to by <i>uc</i> with the given <i>length</i> and case sensitivity specified by <i>cs</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStringMatcher::QStringMatcher(const QChar* uc, int len, Qt::CaseSensitivity cs = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringmatcher.html#QStringMatcher-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a string matcher that will search for the pattern referred to by <i>uc</i> with the given <i>length</i> and case sensitivity specified by <i>cs</i>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn from_q_char_int_case_sensitivity(
        uc: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QChar>>,
        len: ::std::os::raw::c_int,
        cs: crate::CaseSensitivity,
    ) -> ::cpp_utils::CppBox<crate::QStringMatcher> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringMatcher_QStringMatcher2(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QChar>>::cast_into(uc).as_raw_ptr(),
            len,
            cs,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a string matcher that will search for <i>pattern</i>, with case sensitivity <i>cs</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStringMatcher::QStringMatcher(const QString& pattern)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringmatcher.html#QStringMatcher-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a string matcher that will search for <i>pattern</i>, with case sensitivity <i>cs</i>.</p>
    /// <p>Call <a href="http://doc.qt.io/qt-5/qstringmatcher.html#indexIn">indexIn</a>() to perform a search.</p></div>
    pub unsafe fn from_q_string(
        pattern: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QStringMatcher> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringMatcher_QStringMatcher4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(pattern)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a string matcher that will search for the pattern referred to by <i>uc</i> with the given <i>length</i> and case sensitivity specified by <i>cs</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStringMatcher::QStringMatcher(const QChar* uc, int len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringmatcher.html#QStringMatcher-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a string matcher that will search for the pattern referred to by <i>uc</i> with the given <i>length</i> and case sensitivity specified by <i>cs</i>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn from_q_char_int(
        uc: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QChar>>,
        len: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QStringMatcher> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringMatcher_QStringMatcher5(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QChar>>::cast_into(uc).as_raw_ptr(),
            len,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Copies the <i>other</i> string matcher to this string matcher.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStringMatcher::QStringMatcher(const QStringMatcher& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringmatcher.html#QStringMatcher-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Copies the <i>other</i> string matcher to this string matcher.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringMatcher>>,
    ) -> ::cpp_utils::CppBox<crate::QStringMatcher> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringMatcher_QStringMatcher3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringMatcher>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the string pattern that this string matcher will search for.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QStringMatcher::pattern() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringmatcher.html#pattern">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string pattern that this string matcher will search for.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringmatcher.html#setPattern">setPattern</a>().</p></div>
    pub unsafe fn pattern(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringMatcher_pattern(
            self as *const crate::QStringMatcher,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Sets the case sensitivity setting of this string matcher to <i>cs</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QStringMatcher::setCaseSensitivity(Qt::CaseSensitivity cs)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringmatcher.html#setCaseSensitivity">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the case sensitivity setting of this string matcher to <i>cs</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringmatcher.html#caseSensitivity">caseSensitivity</a>(), <a href="http://doc.qt.io/qt-5/qstringmatcher.html#setPattern">setPattern</a>(), and <a href="http://doc.qt.io/qt-5/qstringmatcher.html#indexIn">indexIn</a>().</p></div>
    pub unsafe fn set_case_sensitivity(&mut self, cs: crate::CaseSensitivity) {
        crate::__ffi::ctr_qt_core_ffi_QStringMatcher_setCaseSensitivity(
            self as *mut crate::QStringMatcher,
            cs,
        )
    }

    /// <p>Sets the string that this string matcher will search for to <i>pattern</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QStringMatcher::setPattern(const QString& pattern)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringmatcher.html#setPattern">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the string that this string matcher will search for to <i>pattern</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringmatcher.html#pattern">pattern</a>(), <a href="http://doc.qt.io/qt-5/qstringmatcher.html#setCaseSensitivity">setCaseSensitivity</a>(), and <a href="http://doc.qt.io/qt-5/qstringmatcher.html#indexIn">indexIn</a>().</p></div>
    pub unsafe fn set_pattern(
        &mut self,
        pattern: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QStringMatcher_setPattern(
            self as *mut crate::QStringMatcher,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(pattern)
                .as_raw_ptr(),
        )
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class provides a list of strings.</p>
///
/// C++ class: <span style='color: green;'>```QStringList```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qstringlist.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class provides a list of strings.</p>
/// <p><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> inherits from <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>&gt;. Like <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. It provides fast index-based access as well as fast insertions and removals. Passing string lists as value parameters is both fast and safe.</p>
/// <p>All of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s functionality also applies to <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a>. For example, you can use <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() to test whether the list is empty, and you can call functions like <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>() to modify a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a>. In addition, <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> provides a few convenience functions that make handling lists of strings easier:</p>
/// <a name="adding-strings"></a></div>
#[repr(C)]
pub struct QStringList {
    _unused: u8,
}
impl QStringList {
    /// <p>Returns <code>true</code> if the list contains the string <i>str</i>; otherwise returns <code>false</code>. The search is case insensitive if <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseInsensitive</a>; the search is case sensitive by default.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringList::contains(const QString& str, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains the string <i>str</i>; otherwise returns <code>false</code>. The search is case insensitive if <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseInsensitive</a>; the search is case sensitive by default.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf">lastIndexOf</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#contains">QString::contains</a>().</p></div>
    pub unsafe fn contains_q_string_case_sensitivity(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringList_contains(
            self as *const crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(str).as_raw_ptr(),
            cs,
        )
    }

    /// <p>Returns <code>true</code> if the list contains the string <i>str</i>; otherwise returns <code>false</code>. The search is case insensitive if <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseInsensitive</a>; the search is case sensitive by default.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringList::contains(const QString& str) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains the string <i>str</i>; otherwise returns <code>false</code>. The search is case insensitive if <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseInsensitive</a>; the search is case sensitive by default.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf">lastIndexOf</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#contains">QString::contains</a>().</p></div>
    pub unsafe fn contains_q_string(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringList_contains1(
            self as *const crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(str).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringList::contains(QLatin1String str, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#contains-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the list contains the string <i>str</i>; otherwise returns <code>false</code>. The search is case insensitive if <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseInsensitive</a>; the search is case sensitive by default.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf">lastIndexOf</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#contains">QString::contains</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn contains_q_latin1_string_case_sensitivity(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringList_contains4(
            self as *const crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(str)
                .as_raw_ptr(),
            cs,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringList::contains(QLatin1String str) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#contains-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the list contains the string <i>str</i>; otherwise returns <code>false</code>. The search is case insensitive if <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseInsensitive</a>; the search is case sensitive by default.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf">lastIndexOf</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#contains">QString::contains</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn contains_q_latin1_string(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringList_contains5(
            self as *const crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(str)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringList::contains(QStringView str, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#contains-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the list contains the string <i>str</i>; otherwise returns <code>false</code>. The search is case insensitive if <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseInsensitive</a>; the search is case sensitive by default.</p>
    /// <p>This function was introduced in  Qt 5.12.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn contains_q_string_view_case_sensitivity(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringList_contains8(
            self as *const crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(str)
                .as_raw_ptr(),
            cs,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringList::contains(QStringView str) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#contains-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the list contains the string <i>str</i>; otherwise returns <code>false</code>. The search is case insensitive if <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseInsensitive</a>; the search is case sensitive by default.</p>
    /// <p>This function was introduced in  Qt 5.12.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn contains_q_string_view(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringList_contains9(
            self as *const crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(str)
                .as_raw_ptr(),
        )
    }

    /// <p>Copy assignment operator from <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>&gt;. Assigns the <i>other</i> list of strings to this string list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList& QStringList::operator=(const QList<QString>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Copy assignment operator from <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>&gt;. Assigns the <i>other</i> list of strings to this string list.</p>
    /// <p>After the operation, <i>other</i> and <code>*this</code> will be equal.</p>
    /// <p>This function was introduced in  Qt 5.4.</p></div>
    pub unsafe fn copy_from_q_list_of_q_string(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQString>>,
    ) -> ::cpp_utils::MutRef<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringList_operator_(
            self as *mut crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQString>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class provides a list of strings.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList& QStringList::operator=(const QStringList& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class provides a list of strings.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> inherits from <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>&gt;. Like <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. It provides fast index-based access as well as fast insertions and removals. Passing string lists as value parameters is both fast and safe.</p>
    /// <p>All of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s functionality also applies to <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a>. For example, you can use <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() to test whether the list is empty, and you can call functions like <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>() to modify a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a>. In addition, <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> provides a few convenience functions that make handling lists of strings easier:</p>
    /// <a name="adding-strings"></a></div>
    pub unsafe fn copy_from_q_string_list(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) -> ::cpp_utils::MutRef<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringList_operator_3(
            self as *mut crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the first exact match of <i>rx</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringList::indexOf(const QRegExp& rx, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first exact match of <i>rx</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>By default, this function is case sensitive.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf">lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringlist.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qregexp.html#exactMatch">QRegExp::exactMatch</a>().</p></div>
    pub unsafe fn index_of_q_reg_exp_int(
        &self,
        rx: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringList_indexOf(
            self as *const crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(rx).as_raw_ptr(),
            from,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf">indexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringList::indexOf(QRegExp& rx, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf">indexOf</a>().</p>
    /// <p>Returns the index position of the first exact match of <i>rx</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>By default, this function is case sensitive.</p>
    /// <p>If an item matched, the <i>rx</i> regular expression will contain the matched objects (see <a href="http://doc.qt.io/qt-5/qregexp.html#matchedLength">QRegExp::matchedLength</a>, <a href="http://doc.qt.io/qt-5/qregexp.html#cap">QRegExp::cap</a>).</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf">lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringlist.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qregexp.html#exactMatch">QRegExp::exactMatch</a>().</p></div>
    pub unsafe fn index_of_q_reg_exp_int2(
        &self,
        rx: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QRegExp>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringList_indexOf1(
            self as *const crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QRegExp>>::cast_into(rx)
                .as_mut_raw_ptr(),
            from,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringList::indexOf(const QRegularExpression& re, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the index position of the first match of <i>re</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf">lastIndexOf</a>().</p></div>
    pub unsafe fn index_of_q_regular_expression_int(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringList_indexOf2(
            self as *const crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first exact match of <i>rx</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringList::indexOf(const QRegExp& rx) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first exact match of <i>rx</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>By default, this function is case sensitive.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf">lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringlist.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qregexp.html#exactMatch">QRegExp::exactMatch</a>().</p></div>
    pub unsafe fn index_of_q_reg_exp(
        &self,
        rx: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringList_indexOf3(
            self as *const crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(rx).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf">indexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringList::indexOf(QRegExp& rx) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf">indexOf</a>().</p>
    /// <p>Returns the index position of the first exact match of <i>rx</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>By default, this function is case sensitive.</p>
    /// <p>If an item matched, the <i>rx</i> regular expression will contain the matched objects (see <a href="http://doc.qt.io/qt-5/qregexp.html#matchedLength">QRegExp::matchedLength</a>, <a href="http://doc.qt.io/qt-5/qregexp.html#cap">QRegExp::cap</a>).</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf">lastIndexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringlist.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qregexp.html#exactMatch">QRegExp::exactMatch</a>().</p></div>
    pub unsafe fn index_of_q_reg_exp2(
        &self,
        rx: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QRegExp>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringList_indexOf4(
            self as *const crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QRegExp>>::cast_into(rx)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringList::indexOf(const QRegularExpression& re) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the index position of the first match of <i>re</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf">lastIndexOf</a>().</p></div>
    pub unsafe fn index_of_q_regular_expression(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringList_indexOf5(
            self as *const crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringList::indexOf(QStringView str, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the index position of the first occurrence of <i>str</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#contains">contains</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn index_of_q_string_view_int(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringList_indexOf12(
            self as *const crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(str)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringList::indexOf(QLatin1String str, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the index position of the first occurrence of <i>str</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#contains">contains</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn index_of_q_latin1_string_int(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringList_indexOf13(
            self as *const crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(str)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringList::indexOf(QStringView str) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the index position of the first occurrence of <i>str</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#contains">contains</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn index_of_q_string_view(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringList_indexOf14(
            self as *const crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(str)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringList::indexOf(QLatin1String str) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the index position of the first occurrence of <i>str</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#contains">contains</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn index_of_q_latin1_string(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringList_indexOf15(
            self as *const crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(str)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the index position of the last exact match of <i>rx</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringList::lastIndexOf(const QRegExp& rx, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last exact match of <i>rx</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>By default, this function is case sensitive.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringlist.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qregexp.html#exactMatch">QRegExp::exactMatch</a>().</p></div>
    pub unsafe fn last_index_of_q_reg_exp_int(
        &self,
        rx: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringList_lastIndexOf(
            self as *const crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(rx).as_raw_ptr(),
            from,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf">lastIndexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringList::lastIndexOf(QRegExp& rx, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf">lastIndexOf</a>().</p>
    /// <p>Returns the index position of the last exact match of <i>rx</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>By default, this function is case sensitive.</p>
    /// <p>If an item matched, the <i>rx</i> regular expression will contain the matched objects (see <a href="http://doc.qt.io/qt-5/qregexp.html#matchedLength">QRegExp::matchedLength</a>, <a href="http://doc.qt.io/qt-5/qregexp.html#cap">QRegExp::cap</a>).</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringlist.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qregexp.html#exactMatch">QRegExp::exactMatch</a>().</p></div>
    pub unsafe fn last_index_of_q_reg_exp_int2(
        &self,
        rx: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QRegExp>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringList_lastIndexOf1(
            self as *const crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QRegExp>>::cast_into(rx)
                .as_mut_raw_ptr(),
            from,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringList::lastIndexOf(const QRegularExpression& re, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the index position of the last exact match of <i>re</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_q_regular_expression_int(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringList_lastIndexOf2(
            self as *const crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the last exact match of <i>rx</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringList::lastIndexOf(const QRegExp& rx) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last exact match of <i>rx</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>By default, this function is case sensitive.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringlist.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qregexp.html#exactMatch">QRegExp::exactMatch</a>().</p></div>
    pub unsafe fn last_index_of_q_reg_exp(
        &self,
        rx: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringList_lastIndexOf3(
            self as *const crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(rx).as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf">lastIndexOf</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringList::lastIndexOf(QRegExp& rx) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf">lastIndexOf</a>().</p>
    /// <p>Returns the index position of the last exact match of <i>rx</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>By default, this function is case sensitive.</p>
    /// <p>If an item matched, the <i>rx</i> regular expression will contain the matched objects (see <a href="http://doc.qt.io/qt-5/qregexp.html#matchedLength">QRegExp::matchedLength</a>, <a href="http://doc.qt.io/qt-5/qregexp.html#cap">QRegExp::cap</a>).</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf">indexOf</a>(), <a href="http://doc.qt.io/qt-5/qstringlist.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qregexp.html#exactMatch">QRegExp::exactMatch</a>().</p></div>
    pub unsafe fn last_index_of_q_reg_exp2(
        &self,
        rx: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QRegExp>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringList_lastIndexOf4(
            self as *const crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QRegExp>>::cast_into(rx)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringList::lastIndexOf(const QRegularExpression& re) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the index position of the last exact match of <i>re</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_q_regular_expression(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringList_lastIndexOf5(
            self as *const crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringList::lastIndexOf(QStringView str, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the index position of the last occurrence of <i>str</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#contains">contains</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn last_index_of_q_string_view_int(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringList_lastIndexOf12(
            self as *const crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(str)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringList::lastIndexOf(QLatin1String str, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the index position of the last occurrence of <i>str</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#contains">contains</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn last_index_of_q_latin1_string_int(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringList_lastIndexOf13(
            self as *const crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(str)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringList::lastIndexOf(QStringView str) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the index position of the last occurrence of <i>str</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#contains">contains</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn last_index_of_q_string_view(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringList_lastIndexOf14(
            self as *const crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(str)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringList::lastIndexOf(QLatin1String str) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#lastIndexOf-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the index position of the last occurrence of <i>str</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#contains">contains</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn last_index_of_q_latin1_string(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringList_lastIndexOf15(
            self as *const crate::QStringList,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(str)
                .as_raw_ptr(),
        )
    }

    /// <p>Constructs an empty string list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStringList::QStringList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#QStringList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty string list.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringList_QStringList();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a string list that contains the given string, <i>str</i>. Longer lists are easily created like this:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStringList::QStringList(const QString& i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#QStringList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a string list that contains the given string, <i>str</i>. Longer lists are easily created like this:</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html#QStringList">QStringList</a></span> longerList <span class="operator">=</span> (<span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html#QStringList">QStringList</a></span>() <span class="operator">&lt;</span><span class="operator">&lt;</span> str1 <span class="operator">&lt;</span><span class="operator">&lt;</span> str2 <span class="operator">&lt;</span><span class="operator">&lt;</span> str3);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    pub unsafe fn from_q_string(
        i: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringList_QStringList1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(i).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStringList::QStringList(const QList<QString>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#QStringList-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#constant-time">constant time</a>, because <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlist.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn from_q_list_of_q_string(
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQString>>,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringList_QStringList2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQString>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class provides a list of strings.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStringList::QStringList(const QStringList& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class provides a list of strings.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> inherits from <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>&gt;. Like <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. It provides fast index-based access as well as fast insertions and removals. Passing string lists as value parameters is both fast and safe.</p>
    /// <p>All of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s functionality also applies to <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a>. For example, you can use <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() to test whether the list is empty, and you can call functions like <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>() to modify a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a>. In addition, <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> provides a few convenience functions that make handling lists of strings easier:</p>
    /// <a name="adding-strings"></a></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringList_QStringList5(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_meta_type {
    //! C++ type: <span style='color: green;'>```QMetaType```</span>

    /// <p>These are the built-in types supported by <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a>:</p>
    ///
    /// C++ enum: <span style='color: green;'>```QMetaType::Type```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>These are the built-in types supported by <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a>:</p>
    ///
    /// <p>Additional types can be registered using <a href="http://doc.qt.io/qt-5/qmetatype.html#Q_DECLARE_METATYPE">Q_DECLARE_METATYPE</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetatype.html#type">type</a>() and <a href="http://doc.qt.io/qt-5/qmetatype.html#typeName">typeName</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Type(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Type {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Type(value)
        }
    }

    impl From<Type> for ::std::os::raw::c_int {
        fn from(value: Type) -> Self {
            value.0
        }
    }

    impl Type {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Type {
        /// <code>void</code> (C++ enum variant: <span style='color: green;'>```Void = 43```</span>)
        #[allow(non_upper_case_globals)]
        pub const Void: crate::q_meta_type::Type = crate::q_meta_type::Type(43);
        /// <code>bool</code> (C++ enum variant: <span style='color: green;'>```Bool = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Bool: crate::q_meta_type::Type = crate::q_meta_type::Type(1);
        /// <code>int</code> (C++ enum variant: <span style='color: green;'>```Int = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const Int: crate::q_meta_type::Type = crate::q_meta_type::Type(2);
        /// <code>unsigned int</code> (C++ enum variant: <span style='color: green;'>```UInt = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const UInt: crate::q_meta_type::Type = crate::q_meta_type::Type(3);
        /// LongLong (C++ enum variant: <span style='color: green;'>```LongLong = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const LongLong: crate::q_meta_type::Type = crate::q_meta_type::Type(4);
        /// ULongLong (C++ enum variant: <span style='color: green;'>```ULongLong = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const ULongLong: crate::q_meta_type::Type = crate::q_meta_type::Type(5);
        /// <code>double</code> (C++ enum variant: <span style='color: green;'>```Double = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const Double: crate::q_meta_type::Type = crate::q_meta_type::Type(6);
        /// <code>long</code> (C++ enum variant: <span style='color: green;'>```Long = 32```</span>)
        #[allow(non_upper_case_globals)]
        pub const Long: crate::q_meta_type::Type = crate::q_meta_type::Type(32);
        /// <code>short</code> (C++ enum variant: <span style='color: green;'>```Short = 33```</span>)
        #[allow(non_upper_case_globals)]
        pub const Short: crate::q_meta_type::Type = crate::q_meta_type::Type(33);
        /// <code>char</code> (C++ enum variant: <span style='color: green;'>```Char = 34```</span>)
        #[allow(non_upper_case_globals)]
        pub const Char: crate::q_meta_type::Type = crate::q_meta_type::Type(34);
        /// <code>unsigned long</code> (C++ enum variant: <span style='color: green;'>```ULong = 35```</span>)
        #[allow(non_upper_case_globals)]
        pub const ULong: crate::q_meta_type::Type = crate::q_meta_type::Type(35);
        /// <code>unsigned short</code> (C++ enum variant: <span style='color: green;'>```UShort = 36```</span>)
        #[allow(non_upper_case_globals)]
        pub const UShort: crate::q_meta_type::Type = crate::q_meta_type::Type(36);
        /// <code>unsigned char</code> (C++ enum variant: <span style='color: green;'>```UChar = 37```</span>)
        #[allow(non_upper_case_globals)]
        pub const UChar: crate::q_meta_type::Type = crate::q_meta_type::Type(37);
        /// <code>float</code> (C++ enum variant: <span style='color: green;'>```Float = 38```</span>)
        #[allow(non_upper_case_globals)]
        pub const Float: crate::q_meta_type::Type = crate::q_meta_type::Type(38);
        /// <code>signed char</code> (C++ enum variant: <span style='color: green;'>```SChar = 40```</span>)
        #[allow(non_upper_case_globals)]
        pub const SChar: crate::q_meta_type::Type = crate::q_meta_type::Type(40);
        /// <code>std::nullptr_t</code> (C++ enum variant: <span style='color: green;'>```Nullptr = 51```</span>)
        #[allow(non_upper_case_globals)]
        pub const Nullptr: crate::q_meta_type::Type = crate::q_meta_type::Type(51);
        /// <code>void *</code> (C++ enum variant: <span style='color: green;'>```VoidStar = 31```</span>)
        #[allow(non_upper_case_globals)]
        pub const VoidStar: crate::q_meta_type::Type = crate::q_meta_type::Type(31);
        /// QChar (C++ enum variant: <span style='color: green;'>```QChar = 7```</span>)
        #[allow(non_upper_case_globals)]
        pub const QChar: crate::q_meta_type::Type = crate::q_meta_type::Type(7);
        /// QString (C++ enum variant: <span style='color: green;'>```QString = 10```</span>)
        #[allow(non_upper_case_globals)]
        pub const QString: crate::q_meta_type::Type = crate::q_meta_type::Type(10);
        /// QStringList (C++ enum variant: <span style='color: green;'>```QStringList = 11```</span>)
        #[allow(non_upper_case_globals)]
        pub const QStringList: crate::q_meta_type::Type = crate::q_meta_type::Type(11);
        /// QByteArray (C++ enum variant: <span style='color: green;'>```QByteArray = 12```</span>)
        #[allow(non_upper_case_globals)]
        pub const QByteArray: crate::q_meta_type::Type = crate::q_meta_type::Type(12);
        /// QBitArray (C++ enum variant: <span style='color: green;'>```QBitArray = 13```</span>)
        #[allow(non_upper_case_globals)]
        pub const QBitArray: crate::q_meta_type::Type = crate::q_meta_type::Type(13);
        /// QDate (C++ enum variant: <span style='color: green;'>```QDate = 14```</span>)
        #[allow(non_upper_case_globals)]
        pub const QDate: crate::q_meta_type::Type = crate::q_meta_type::Type(14);
        /// QTime (C++ enum variant: <span style='color: green;'>```QTime = 15```</span>)
        #[allow(non_upper_case_globals)]
        pub const QTime: crate::q_meta_type::Type = crate::q_meta_type::Type(15);
        /// QDateTime (C++ enum variant: <span style='color: green;'>```QDateTime = 16```</span>)
        #[allow(non_upper_case_globals)]
        pub const QDateTime: crate::q_meta_type::Type = crate::q_meta_type::Type(16);
        /// QUrl (C++ enum variant: <span style='color: green;'>```QUrl = 17```</span>)
        #[allow(non_upper_case_globals)]
        pub const QUrl: crate::q_meta_type::Type = crate::q_meta_type::Type(17);
        /// QLocale (C++ enum variant: <span style='color: green;'>```QLocale = 18```</span>)
        #[allow(non_upper_case_globals)]
        pub const QLocale: crate::q_meta_type::Type = crate::q_meta_type::Type(18);
        /// QRect (C++ enum variant: <span style='color: green;'>```QRect = 19```</span>)
        #[allow(non_upper_case_globals)]
        pub const QRect: crate::q_meta_type::Type = crate::q_meta_type::Type(19);
        /// QRectF (C++ enum variant: <span style='color: green;'>```QRectF = 20```</span>)
        #[allow(non_upper_case_globals)]
        pub const QRectF: crate::q_meta_type::Type = crate::q_meta_type::Type(20);
        /// QSize (C++ enum variant: <span style='color: green;'>```QSize = 21```</span>)
        #[allow(non_upper_case_globals)]
        pub const QSize: crate::q_meta_type::Type = crate::q_meta_type::Type(21);
        /// QSizeF (C++ enum variant: <span style='color: green;'>```QSizeF = 22```</span>)
        #[allow(non_upper_case_globals)]
        pub const QSizeF: crate::q_meta_type::Type = crate::q_meta_type::Type(22);
        /// QLine (C++ enum variant: <span style='color: green;'>```QLine = 23```</span>)
        #[allow(non_upper_case_globals)]
        pub const QLine: crate::q_meta_type::Type = crate::q_meta_type::Type(23);
        /// QLineF (C++ enum variant: <span style='color: green;'>```QLineF = 24```</span>)
        #[allow(non_upper_case_globals)]
        pub const QLineF: crate::q_meta_type::Type = crate::q_meta_type::Type(24);
        /// QPoint (C++ enum variant: <span style='color: green;'>```QPoint = 25```</span>)
        #[allow(non_upper_case_globals)]
        pub const QPoint: crate::q_meta_type::Type = crate::q_meta_type::Type(25);
        /// QPointF (C++ enum variant: <span style='color: green;'>```QPointF = 26```</span>)
        #[allow(non_upper_case_globals)]
        pub const QPointF: crate::q_meta_type::Type = crate::q_meta_type::Type(26);
        /// QRegExp (C++ enum variant: <span style='color: green;'>```QRegExp = 27```</span>)
        #[allow(non_upper_case_globals)]
        pub const QRegExp: crate::q_meta_type::Type = crate::q_meta_type::Type(27);
        /// QEasingCurve (C++ enum variant: <span style='color: green;'>```QEasingCurve = 29```</span>)
        #[allow(non_upper_case_globals)]
        pub const QEasingCurve: crate::q_meta_type::Type = crate::q_meta_type::Type(29);
        /// QUuid (C++ enum variant: <span style='color: green;'>```QUuid = 30```</span>)
        #[allow(non_upper_case_globals)]
        pub const QUuid: crate::q_meta_type::Type = crate::q_meta_type::Type(30);
        /// QVariant (C++ enum variant: <span style='color: green;'>```QVariant = 41```</span>)
        #[allow(non_upper_case_globals)]
        pub const QVariant: crate::q_meta_type::Type = crate::q_meta_type::Type(41);
        /// QModelIndex (C++ enum variant: <span style='color: green;'>```QModelIndex = 42```</span>)
        #[allow(non_upper_case_globals)]
        pub const QModelIndex: crate::q_meta_type::Type = crate::q_meta_type::Type(42);
        /// QRegularExpression (C++ enum variant: <span style='color: green;'>```QRegularExpression = 44```</span>)
        #[allow(non_upper_case_globals)]
        pub const QRegularExpression: crate::q_meta_type::Type = crate::q_meta_type::Type(44);
        /// QJsonValue (C++ enum variant: <span style='color: green;'>```QJsonValue = 45```</span>)
        #[allow(non_upper_case_globals)]
        pub const QJsonValue: crate::q_meta_type::Type = crate::q_meta_type::Type(45);
        /// QJsonObject (C++ enum variant: <span style='color: green;'>```QJsonObject = 46```</span>)
        #[allow(non_upper_case_globals)]
        pub const QJsonObject: crate::q_meta_type::Type = crate::q_meta_type::Type(46);
        /// QJsonArray (C++ enum variant: <span style='color: green;'>```QJsonArray = 47```</span>)
        #[allow(non_upper_case_globals)]
        pub const QJsonArray: crate::q_meta_type::Type = crate::q_meta_type::Type(47);
        /// QJsonDocument (C++ enum variant: <span style='color: green;'>```QJsonDocument = 48```</span>)
        #[allow(non_upper_case_globals)]
        pub const QJsonDocument: crate::q_meta_type::Type = crate::q_meta_type::Type(48);
        /// QPersistentModelIndex (since 5.5) (C++ enum variant: <span style='color: green;'>```QPersistentModelIndex = 50```</span>)
        #[allow(non_upper_case_globals)]
        pub const QPersistentModelIndex: crate::q_meta_type::Type = crate::q_meta_type::Type(50);
        /// <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> * (C++ enum variant: <span style='color: green;'>```QObjectStar = 39```</span>)
        #[allow(non_upper_case_globals)]
        pub const QObjectStar: crate::q_meta_type::Type = crate::q_meta_type::Type(39);
        /// QVariantMap (C++ enum variant: <span style='color: green;'>```QVariantMap = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const QVariantMap: crate::q_meta_type::Type = crate::q_meta_type::Type(8);
        /// QVariantList (C++ enum variant: <span style='color: green;'>```QVariantList = 9```</span>)
        #[allow(non_upper_case_globals)]
        pub const QVariantList: crate::q_meta_type::Type = crate::q_meta_type::Type(9);
        /// QVariantHash (C++ enum variant: <span style='color: green;'>```QVariantHash = 28```</span>)
        #[allow(non_upper_case_globals)]
        pub const QVariantHash: crate::q_meta_type::Type = crate::q_meta_type::Type(28);
        /// QByteArrayList (C++ enum variant: <span style='color: green;'>```QByteArrayList = 49```</span>)
        #[allow(non_upper_case_globals)]
        pub const QByteArrayList: crate::q_meta_type::Type = crate::q_meta_type::Type(49);
        /// QFont (C++ enum variant: <span style='color: green;'>```QFont = 64```</span>)
        #[allow(non_upper_case_globals)]
        pub const QFont: crate::q_meta_type::Type = crate::q_meta_type::Type(64);
        /// QPixmap (C++ enum variant: <span style='color: green;'>```QPixmap = 65```</span>)
        #[allow(non_upper_case_globals)]
        pub const QPixmap: crate::q_meta_type::Type = crate::q_meta_type::Type(65);
        /// QBrush (C++ enum variant: <span style='color: green;'>```QBrush = 66```</span>)
        #[allow(non_upper_case_globals)]
        pub const QBrush: crate::q_meta_type::Type = crate::q_meta_type::Type(66);
        /// QColor (C++ enum variant: <span style='color: green;'>```QColor = 67```</span>)
        #[allow(non_upper_case_globals)]
        pub const QColor: crate::q_meta_type::Type = crate::q_meta_type::Type(67);
        /// QPalette (C++ enum variant: <span style='color: green;'>```QPalette = 68```</span>)
        #[allow(non_upper_case_globals)]
        pub const QPalette: crate::q_meta_type::Type = crate::q_meta_type::Type(68);
        /// QIcon (C++ enum variant: <span style='color: green;'>```QIcon = 69```</span>)
        #[allow(non_upper_case_globals)]
        pub const QIcon: crate::q_meta_type::Type = crate::q_meta_type::Type(69);
        /// QImage (C++ enum variant: <span style='color: green;'>```QImage = 70```</span>)
        #[allow(non_upper_case_globals)]
        pub const QImage: crate::q_meta_type::Type = crate::q_meta_type::Type(70);
        /// QPolygon (C++ enum variant: <span style='color: green;'>```QPolygon = 71```</span>)
        #[allow(non_upper_case_globals)]
        pub const QPolygon: crate::q_meta_type::Type = crate::q_meta_type::Type(71);
        /// QRegion (C++ enum variant: <span style='color: green;'>```QRegion = 72```</span>)
        #[allow(non_upper_case_globals)]
        pub const QRegion: crate::q_meta_type::Type = crate::q_meta_type::Type(72);
        /// QBitmap (C++ enum variant: <span style='color: green;'>```QBitmap = 73```</span>)
        #[allow(non_upper_case_globals)]
        pub const QBitmap: crate::q_meta_type::Type = crate::q_meta_type::Type(73);
        /// QCursor (C++ enum variant: <span style='color: green;'>```QCursor = 74```</span>)
        #[allow(non_upper_case_globals)]
        pub const QCursor: crate::q_meta_type::Type = crate::q_meta_type::Type(74);
        /// QKeySequence (C++ enum variant: <span style='color: green;'>```QKeySequence = 75```</span>)
        #[allow(non_upper_case_globals)]
        pub const QKeySequence: crate::q_meta_type::Type = crate::q_meta_type::Type(75);
        /// QPen (C++ enum variant: <span style='color: green;'>```QPen = 76```</span>)
        #[allow(non_upper_case_globals)]
        pub const QPen: crate::q_meta_type::Type = crate::q_meta_type::Type(76);
        /// QTextLength (C++ enum variant: <span style='color: green;'>```QTextLength = 77```</span>)
        #[allow(non_upper_case_globals)]
        pub const QTextLength: crate::q_meta_type::Type = crate::q_meta_type::Type(77);
        /// QTextFormat (C++ enum variant: <span style='color: green;'>```QTextFormat = 78```</span>)
        #[allow(non_upper_case_globals)]
        pub const QTextFormat: crate::q_meta_type::Type = crate::q_meta_type::Type(78);
        /// QMatrix (C++ enum variant: <span style='color: green;'>```QMatrix = 79```</span>)
        #[allow(non_upper_case_globals)]
        pub const QMatrix: crate::q_meta_type::Type = crate::q_meta_type::Type(79);
        /// QTransform (C++ enum variant: <span style='color: green;'>```QTransform = 80```</span>)
        #[allow(non_upper_case_globals)]
        pub const QTransform: crate::q_meta_type::Type = crate::q_meta_type::Type(80);
        /// QMatrix4x4 (C++ enum variant: <span style='color: green;'>```QMatrix4x4 = 81```</span>)
        #[allow(non_upper_case_globals)]
        pub const QMatrix4X4: crate::q_meta_type::Type = crate::q_meta_type::Type(81);
        /// QVector2D (C++ enum variant: <span style='color: green;'>```QVector2D = 82```</span>)
        #[allow(non_upper_case_globals)]
        pub const QVector2D: crate::q_meta_type::Type = crate::q_meta_type::Type(82);
        /// QVector3D (C++ enum variant: <span style='color: green;'>```QVector3D = 83```</span>)
        #[allow(non_upper_case_globals)]
        pub const QVector3D: crate::q_meta_type::Type = crate::q_meta_type::Type(83);
        /// QVector4D (C++ enum variant: <span style='color: green;'>```QVector4D = 84```</span>)
        #[allow(non_upper_case_globals)]
        pub const QVector4D: crate::q_meta_type::Type = crate::q_meta_type::Type(84);
        /// QQuaternion (C++ enum variant: <span style='color: green;'>```QQuaternion = 85```</span>)
        #[allow(non_upper_case_globals)]
        pub const QQuaternion: crate::q_meta_type::Type = crate::q_meta_type::Type(85);
        /// QPolygonF (C++ enum variant: <span style='color: green;'>```QPolygonF = 86```</span>)
        #[allow(non_upper_case_globals)]
        pub const QPolygonF: crate::q_meta_type::Type = crate::q_meta_type::Type(86);
        /// QSizePolicy (C++ enum variant: <span style='color: green;'>```QSizePolicy = 121```</span>)
        #[allow(non_upper_case_globals)]
        pub const QSizePolicy: crate::q_meta_type::Type = crate::q_meta_type::Type(121);
        /// C++ enum variant: <span style='color: green;'>```FirstCoreType = 1```</span>
        #[allow(non_upper_case_globals)]
        pub const FirstCoreType: crate::q_meta_type::Type = crate::q_meta_type::Type(1);
        /// C++ enum variant: <span style='color: green;'>```LastCoreType = 51```</span>
        #[allow(non_upper_case_globals)]
        pub const LastCoreType: crate::q_meta_type::Type = crate::q_meta_type::Type(51);
        /// C++ enum variant: <span style='color: green;'>```FirstGuiType = 64```</span>
        #[allow(non_upper_case_globals)]
        pub const FirstGuiType: crate::q_meta_type::Type = crate::q_meta_type::Type(64);
        /// C++ enum variant: <span style='color: green;'>```LastGuiType = 86```</span>
        #[allow(non_upper_case_globals)]
        pub const LastGuiType: crate::q_meta_type::Type = crate::q_meta_type::Type(86);
        /// C++ enum variant: <span style='color: green;'>```FirstWidgetsType = 121```</span>
        #[allow(non_upper_case_globals)]
        pub const FirstWidgetsType: crate::q_meta_type::Type = crate::q_meta_type::Type(121);
        /// C++ enum variant: <span style='color: green;'>```LastWidgetsType = 121```</span>
        #[allow(non_upper_case_globals)]
        pub const LastWidgetsType: crate::q_meta_type::Type = crate::q_meta_type::Type(121);
        /// C++ enum variant: <span style='color: green;'>```HighestInternalId = 121```</span>
        #[allow(non_upper_case_globals)]
        pub const HighestInternalId: crate::q_meta_type::Type = crate::q_meta_type::Type(121);
        /// C++ enum variant: <span style='color: green;'>```QReal = 6```</span>
        #[allow(non_upper_case_globals)]
        pub const QReal: crate::q_meta_type::Type = crate::q_meta_type::Type(6);
        /// This is an invalid type id. It is returned from <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a> for types that are not registered (C++ enum variant: <span style='color: green;'>```UnknownType = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const UnknownType: crate::q_meta_type::Type = crate::q_meta_type::Type(0);
        /// Base value for user types (C++ enum variant: <span style='color: green;'>```User = 1024```</span>)
        #[allow(non_upper_case_globals)]
        pub const User: crate::q_meta_type::Type = crate::q_meta_type::Type(1024);
        /// QCborSimpleType (C++ enum variant: <span style='color: green;'>```QCborSimpleType = 52```</span>)
        #[allow(non_upper_case_globals)]
        pub const QCborSimpleType: crate::q_meta_type::Type = crate::q_meta_type::Type(52);
        /// QCborValue (C++ enum variant: <span style='color: green;'>```QCborValue = 53```</span>)
        #[allow(non_upper_case_globals)]
        pub const QCborValue: crate::q_meta_type::Type = crate::q_meta_type::Type(53);
        /// QCborArray (C++ enum variant: <span style='color: green;'>```QCborArray = 54```</span>)
        #[allow(non_upper_case_globals)]
        pub const QCborArray: crate::q_meta_type::Type = crate::q_meta_type::Type(54);
        /// QCborMap (C++ enum variant: <span style='color: green;'>```QCborMap = 55```</span>)
        #[allow(non_upper_case_globals)]
        pub const QCborMap: crate::q_meta_type::Type = crate::q_meta_type::Type(55);
        /// C++ enum variant: <span style='color: green;'>```LastCoreType = 55```</span>
        #[allow(non_upper_case_globals)]
        pub const LastCoreType2: crate::q_meta_type::Type = crate::q_meta_type::Type(55);
    }

    /// <p>The enum describes attributes of a type supported by <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a>.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QMetaType::TypeFlag```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#TypeFlag-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The enum describes attributes of a type supported by <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a>.</p>
    ///
    /// <p>The TypeFlags type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;TypeFlag&gt;. It stores an OR combination of TypeFlag values.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct TypeFlag(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for TypeFlag {
        fn from(value: ::std::os::raw::c_int) -> Self {
            TypeFlag(value)
        }
    }

    impl From<TypeFlag> for ::std::os::raw::c_int {
        fn from(value: TypeFlag) -> Self {
            value.0
        }
    }

    impl TypeFlag {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl TypeFlag {
        /// This type has non-trivial constructors. If the flag is not set instances can be safely initialized with memset to 0. (C++ enum variant: <span style='color: green;'>```NeedsConstruction = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const NeedsConstruction: crate::q_meta_type::TypeFlag = crate::q_meta_type::TypeFlag(1);
        /// This type has a non-trivial destructor. If the flag is not set calls to the destructor are not necessary before discarding objects. (C++ enum variant: <span style='color: green;'>```NeedsDestruction = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const NeedsDestruction: crate::q_meta_type::TypeFlag = crate::q_meta_type::TypeFlag(2);
        /// An instance of a type having this attribute can be safely moved by memcpy. (C++ enum variant: <span style='color: green;'>```MovableType = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const MovableType: crate::q_meta_type::TypeFlag = crate::q_meta_type::TypeFlag(4);
        /// This type is a pointer to a derived of <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> This type is a <a href="http://doc.qt.io/qt-5/qobject.html#Q_GADGET">Q_GADGET</a> and it's corresponding <a href="http://doc.qt.io/qt-5/qmetaobject.html">QMetaObject</a> can be accessed with <a href="http://doc.qt.io/qt-5/qmetatype.html#metaObject">QMetaType::metaObject</a> Since 5.5. (C++ enum variant: <span style='color: green;'>```PointerToQObject = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const PointerToQObject: crate::q_meta_type::TypeFlag = crate::q_meta_type::TypeFlag(8);
        /// This type is an enumeration (C++ enum variant: <span style='color: green;'>```IsEnumeration = 16```</span>)
        #[allow(non_upper_case_globals)]
        pub const IsEnumeration: crate::q_meta_type::TypeFlag = crate::q_meta_type::TypeFlag(16);
        /// C++ enum variant: <span style='color: green;'>```SharedPointerToQObject = 32```</span>
        #[allow(non_upper_case_globals)]
        pub const SharedPointerToQObject: crate::q_meta_type::TypeFlag =
            crate::q_meta_type::TypeFlag(32);
        /// C++ enum variant: <span style='color: green;'>```WeakPointerToQObject = 64```</span>
        #[allow(non_upper_case_globals)]
        pub const WeakPointerToQObject: crate::q_meta_type::TypeFlag =
            crate::q_meta_type::TypeFlag(64);
        /// C++ enum variant: <span style='color: green;'>```TrackingPointerToQObject = 128```</span>
        #[allow(non_upper_case_globals)]
        pub const TrackingPointerToQObject: crate::q_meta_type::TypeFlag =
            crate::q_meta_type::TypeFlag(128);
        /// C++ enum variant: <span style='color: green;'>```WasDeclaredAsMetaType = 256```</span>
        #[allow(non_upper_case_globals)]
        pub const WasDeclaredAsMetaType: crate::q_meta_type::TypeFlag =
            crate::q_meta_type::TypeFlag(256);
        /// C++ enum variant: <span style='color: green;'>```IsGadget = 512```</span>
        #[allow(non_upper_case_globals)]
        pub const IsGadget: crate::q_meta_type::TypeFlag = crate::q_meta_type::TypeFlag(512);
        /// C++ enum variant: <span style='color: green;'>```PointerToGadget = 1024```</span>
        #[allow(non_upper_case_globals)]
        pub const PointerToGadget: crate::q_meta_type::TypeFlag =
            crate::q_meta_type::TypeFlag(1024);
    }

    impl From<crate::q_meta_type::TypeFlag> for crate::QFlags<crate::q_meta_type::TypeFlag> {
        fn from(value: crate::q_meta_type::TypeFlag) -> Self {
            Self::from(value.to_int())
        }
    }

    impl<T: Into<crate::QFlags<crate::q_meta_type::TypeFlag>>> std::ops::BitOr<T>
        for crate::q_meta_type::TypeFlag
    {
        type Output = crate::QFlags<crate::q_meta_type::TypeFlag>;
        fn bitor(self, rhs: T) -> crate::QFlags<crate::q_meta_type::TypeFlag> {
            Into::<crate::QFlags<crate::q_meta_type::TypeFlag>>::into(self) | rhs
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a> class manages named types in the meta-object system.</p>
///
/// C++ class: <span style='color: green;'>```QMetaType```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qmetatype.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a> class manages named types in the meta-object system.</p>
/// <p>The class is used as a helper to marshall types in <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QVariant</a> and in queued signals and slots connections. It associates a type name to a type so that it can be created and destructed dynamically at run-time. Declare new types with <a href="http://doc.qt.io/qt-5/qmetatype.html#Q_DECLARE_METATYPE">Q_DECLARE_METATYPE</a>() to make them available to <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QVariant</a> and other template-based functions. Call <a href="http://doc.qt.io/qt-5/qmetatype.html#qRegisterMetaType-1">qRegisterMetaType</a>() to make types available to non-template based functions, such as the queued signal and slot connections.</p>
/// <p>Any class or struct that has a public default constructor, a public copy constructor, and a public destructor can be registered.</p>
/// <p>The following code allocates and destructs an instance of <code>MyClass</code>:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> id <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qmetatype.html#QMetaType">QMetaType</a></span><span class="operator">::</span>type(<span class="string">"MyClass"</span>);
///   <span class="keyword">if</span> (id <span class="operator">!</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qmetatype.html#QMetaType">QMetaType</a></span><span class="operator">::</span>UnknownType) {
/// &#32;     <span class="type">void</span> <span class="operator">*</span>myClassPtr <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qmetatype.html#QMetaType">QMetaType</a></span><span class="operator">::</span>create(id);
/// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qmetatype.html#QMetaType">QMetaType</a></span><span class="operator">::</span>destroy(id<span class="operator">,</span> myClassPtr);
/// &#32;     myClassPtr <span class="operator">=</span> <span class="number">0</span>;
///   }
///
/// </pre>
/// <p>If we want the stream operators <code>operator&lt;&lt;()</code> and <code>operator&gt;&gt;()</code> to work on <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QVariant</a> objects that store custom types, the custom type must provide <code>operator&lt;&lt;()</code> and <code>operator&gt;&gt;()</code> operators.</p></div>
#[repr(C)]
pub struct QMetaType {
    _unused: u8,
}
impl QMetaType {
    /// <p>Compares the objects at <i>lhs</i> and <i>rhs</i>. Both objects need to be of type <i>typeId</i>. <i>result</i> is set to less than, equal to or greater than zero, if <i>lhs</i> is less than, equal to or greater than <i>rhs</i>. Returns <code>true</code>, if the comparison succeeded, otherwise <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QMetaType::compare(const void* lhs, const void* rhs, int typeId, int* result)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#compare">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares the objects at <i>lhs</i> and <i>rhs</i>. Both objects need to be of type <i>typeId</i>. <i>result</i> is set to less than, equal to or greater than zero, if <i>lhs</i> is less than, equal to or greater than <i>rhs</i>. Returns <code>true</code>, if the comparison succeeded, otherwise <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p></div>
    pub unsafe fn compare(
        lhs: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::ffi::c_void>>,
        rhs: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::ffi::c_void>>,
        type_id: ::std::os::raw::c_int,
        result: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_int>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_compare(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::ffi::c_void>>::cast_into(lhs)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::ffi::c_void>>::cast_into(rhs)
                .as_raw_ptr(),
            type_id,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_int>>::cast_into(result)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Constructs a value of the given type which is a copy of <i>copy</i>. The default value for <i>copy</i> is 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void* QMetaType::construct(int type, void* where, const void* copy)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qmetatype-obsolete.html#construct-1">C++ documentation</a> for <span style='color: green;'>```static void *QMetaType::construct(int type, const void *copy = Q_NULLPTR)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a value of the given type which is a copy of <i>copy</i>. The default value for <i>copy</i> is 0.</p>
    /// <p>Deprecated, use the static function <a href="http://doc.qt.io/qt-5/qmetatype.html#create">QMetaType::create</a>(int type, const void *copy) instead.</p></div>
    pub unsafe fn construct_3a(
        type_: ::std::os::raw::c_int,
        where_: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
        copy: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::ffi::c_void>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaType_construct(
            type_,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(where_)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::ffi::c_void>>::cast_into(copy)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Constructs a value of the given type which is a copy of <i>copy</i>. The default value for <i>copy</i> is 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void* QMetaType::construct(void* where, const void* copy = …) const```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qmetatype-obsolete.html#construct-1">C++ documentation</a> for <span style='color: green;'>```static void *QMetaType::construct(int type, const void *copy = Q_NULLPTR)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a value of the given type which is a copy of <i>copy</i>. The default value for <i>copy</i> is 0.</p>
    /// <p>Deprecated, use the static function <a href="http://doc.qt.io/qt-5/qmetatype.html#create">QMetaType::create</a>(int type, const void *copy) instead.</p></div>
    pub unsafe fn construct_2a(
        &self,
        where_: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
        copy: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::ffi::c_void>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaType_construct1(
            self as *const crate::QMetaType,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(where_)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::ffi::c_void>>::cast_into(copy)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Constructs a value of the given type which is a copy of <i>copy</i>. The default value for <i>copy</i> is 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void* QMetaType::construct(void* where) const```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qmetatype-obsolete.html#construct-1">C++ documentation</a> for <span style='color: green;'>```static void *QMetaType::construct(int type, const void *copy = Q_NULLPTR)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a value of the given type which is a copy of <i>copy</i>. The default value for <i>copy</i> is 0.</p>
    /// <p>Deprecated, use the static function <a href="http://doc.qt.io/qt-5/qmetatype.html#create">QMetaType::create</a>(int type, const void *copy) instead.</p></div>
    pub unsafe fn construct_1a(
        &self,
        where_: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaType_construct2(
            self as *const crate::QMetaType,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(where_)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Converts the object at <i>from</i> from <i>fromTypeId</i> to the preallocated space at <i>to</i> typed <i>toTypeId</i>. Returns <code>true</code>, if the conversion succeeded, otherwise false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QMetaType::convert(const void* from, int fromTypeId, void* to, int toTypeId)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#convert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the object at <i>from</i> from <i>fromTypeId</i> to the preallocated space at <i>to</i> typed <i>toTypeId</i>. Returns <code>true</code>, if the conversion succeeded, otherwise false.</p>
    /// <p>This function was introduced in  Qt 5.2.</p></div>
    pub unsafe fn convert(
        from: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::ffi::c_void>>,
        from_type_id: ::std::os::raw::c_int,
        to: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
        to_type_id: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_convert(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::ffi::c_void>>::cast_into(from)
                .as_raw_ptr(),
            from_type_id,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(to)
                .as_mut_raw_ptr(),
            to_type_id,
        )
    }

    /// <p>Returns a copy of <i>copy</i>, assuming it is of type <i>type</i>. If <i>copy</i> is zero, creates a default constructed instance.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void* QMetaType::create(int type, const void* copy = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#create">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a copy of <i>copy</i>, assuming it is of type <i>type</i>. If <i>copy</i> is zero, creates a default constructed instance.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetatype.html#destroy">destroy</a>(), <a href="http://doc.qt.io/qt-5/qmetatype.html#isRegistered-1">isRegistered</a>(), and <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">Type</a>.</p></div>
    pub unsafe fn create_int_void(
        type_: ::std::os::raw::c_int,
        copy: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::ffi::c_void>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaType_create(
            type_,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::ffi::c_void>>::cast_into(copy)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns a copy of <i>copy</i>, assuming it is of the type that this <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a> instance was created for. If <i>copy</i> is null, creates a default constructed instance.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void* QMetaType::create(const void* copy = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#create-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a copy of <i>copy</i>, assuming it is of the type that this <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a> instance was created for. If <i>copy</i> is null, creates a default constructed instance.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetatype.html#destroy">QMetaType::destroy</a>().</p></div>
    pub unsafe fn create_void(
        &self,
        copy: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::ffi::c_void>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaType_create1(
            self as *const crate::QMetaType,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::ffi::c_void>>::cast_into(copy)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns a copy of <i>copy</i>, assuming it is of type <i>type</i>. If <i>copy</i> is zero, creates a default constructed instance.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void* QMetaType::create(int type)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#create">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a copy of <i>copy</i>, assuming it is of type <i>type</i>. If <i>copy</i> is zero, creates a default constructed instance.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetatype.html#destroy">destroy</a>(), <a href="http://doc.qt.io/qt-5/qmetatype.html#isRegistered-1">isRegistered</a>(), and <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">Type</a>.</p></div>
    pub unsafe fn create_int(
        type_: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaType_create2(type_);
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns a copy of <i>copy</i>, assuming it is of the type that this <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a> instance was created for. If <i>copy</i> is null, creates a default constructed instance.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void* QMetaType::create() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#create-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a copy of <i>copy</i>, assuming it is of the type that this <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a> instance was created for. If <i>copy</i> is null, creates a default constructed instance.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetatype.html#destroy">QMetaType::destroy</a>().</p></div>
    pub unsafe fn create(&self) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMetaType_create3(self as *const crate::QMetaType);
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Streams the object at <i>rhs</i> of type <i>typeId</i> to the debug stream <i>dbg</i>. Returns <code>true</code> on success, otherwise false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QMetaType::debugStream(QDebug& dbg, const void* rhs, int typeId)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#debugStream">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Streams the object at <i>rhs</i> of type <i>typeId</i> to the debug stream <i>dbg</i>. Returns <code>true</code> on success, otherwise false.</p>
    /// <p>This function was introduced in  Qt 5.2.</p></div>
    pub unsafe fn debug_stream(
        dbg: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QDebug>>,
        rhs: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::ffi::c_void>>,
        type_id: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_debugStream(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QDebug>>::cast_into(dbg)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::ffi::c_void>>::cast_into(rhs)
                .as_raw_ptr(),
            type_id,
        )
    }

    /// <p>Destroys the <i>data</i>, assuming it is of the <i>type</i> given.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QMetaType::destroy(int type, void* data)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#destroy">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the <i>data</i>, assuming it is of the <i>type</i> given.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetatype.html#create">create</a>(), <a href="http://doc.qt.io/qt-5/qmetatype.html#isRegistered-1">isRegistered</a>(), and <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">Type</a>.</p></div>
    pub unsafe fn destroy_2a(
        type_: ::std::os::raw::c_int,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_destroy(
            type_,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Destroys the <i>data</i>, assuming it is of the type that this <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a> instance was created for.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMetaType::destroy(void* data) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#destroy-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the <i>data</i>, assuming it is of the type that this <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a> instance was created for.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetatype.html#create">QMetaType::create</a>().</p></div>
    pub unsafe fn destroy_1a(
        &self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_destroy1(
            self as *const crate::QMetaType,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Destructs the value of the given <i>type</i>, located at <i>where</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QMetaType::destruct(int type, void* where)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#destruct">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destructs the value of the given <i>type</i>, located at <i>where</i>.</p>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qmetatype.html#destroy">destroy</a>(), this function only invokes the type's destructor, it doesn't invoke the delete operator.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetatype.html#construct">construct</a>().</p></div>
    pub unsafe fn destruct_2a(
        type_: ::std::os::raw::c_int,
        where_: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_destruct(
            type_,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(where_)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Destructs the value, located at <i>data</i>, assuming that it is of the type for which this <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a> instance was constructed for.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMetaType::destruct(void* data) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#destruct-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destructs the value, located at <i>data</i>, assuming that it is of the type for which this <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a> instance was constructed for.</p>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qmetatype.html#destroy">destroy</a>(), this function only invokes the type's destructor, it doesn't invoke the delete operator.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetatype.html#construct">QMetaType::construct</a>().</p></div>
    pub unsafe fn destruct_1a(
        &self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_destruct1(
            self as *const crate::QMetaType,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Compares the objects at <i>lhs</i> and <i>rhs</i>. Both objects need to be of type <i>typeId</i>. <i>result</i> is set to zero, if <i>lhs</i> equals to rhs. Returns <code>true</code>, if the comparison succeeded, otherwise <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QMetaType::equals(const void* lhs, const void* rhs, int typeId, int* result)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#equals">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares the objects at <i>lhs</i> and <i>rhs</i>. Both objects need to be of type <i>typeId</i>. <i>result</i> is set to zero, if <i>lhs</i> equals to rhs. Returns <code>true</code>, if the comparison succeeded, otherwise <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 5.5.</p></div>
    pub unsafe fn equals(
        lhs: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::ffi::c_void>>,
        rhs: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::ffi::c_void>>,
        type_id: ::std::os::raw::c_int,
        result: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_int>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_equals(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::ffi::c_void>>::cast_into(lhs)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::ffi::c_void>>::cast_into(rhs)
                .as_raw_ptr(),
            type_id,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_int>>::cast_into(result)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns flags of the type for which this <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a> instance was constructed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFlags<QMetaType::TypeFlag> QMetaType::flags() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#flags">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns flags of the type for which this <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a> instance was constructed.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetatype.html#TypeFlag-enum">QMetaType::TypeFlags</a> and <a href="http://doc.qt.io/qt-5/qmetatype.html#typeFlags">QMetaType::typeFlags</a>().</p></div>
    pub unsafe fn flags(&self) -> crate::QFlags<crate::q_meta_type::TypeFlag> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMetaType_flags(self as *const crate::QMetaType);
        crate::QFlags::from(ffi_result)
    }

    /// <p>Returns <code>true</code>, if the meta type system has registered comparators for type id <i>typeId</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QMetaType::hasRegisteredComparators(int typeId)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#hasRegisteredComparators-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code>, if the meta type system has registered comparators for type id <i>typeId</i>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p></div>
    pub unsafe fn has_registered_comparators(type_id: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_hasRegisteredComparators(type_id)
    }

    /// <p>Returns <code>true</code>, if the meta type system has a registered conversion from meta type id <i>fromTypeId</i> to <i>toTypeId</i></p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QMetaType::hasRegisteredConverterFunction(int fromTypeId, int toTypeId)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#hasRegisteredConverterFunction">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code>, if the meta type system has a registered conversion from meta type id <i>fromTypeId</i> to <i>toTypeId</i></p>
    /// <p>This function was introduced in  Qt 5.2.</p></div>
    pub unsafe fn has_registered_converter_function(
        from_type_id: ::std::os::raw::c_int,
        to_type_id: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_hasRegisteredConverterFunction(
            from_type_id,
            to_type_id,
        )
    }

    /// <p>Returns <code>true</code>, if the meta type system has a registered debug stream operator for type id <i>typeId</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QMetaType::hasRegisteredDebugStreamOperator(int typeId)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#hasRegisteredDebugStreamOperator-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code>, if the meta type system has a registered debug stream operator for type id <i>typeId</i>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p></div>
    pub unsafe fn has_registered_debug_stream_operator(type_id: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_hasRegisteredDebugStreamOperator(type_id)
    }

    /// <p>Returns id type hold by this QMetatype instance.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaType::id() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#id">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns id type hold by this QMetatype instance.</p>
    /// <p>This function was introduced in  Qt 5.13.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn id(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_id(self as *const crate::QMetaType)
    }

    /// <p>Returns <code>true</code> if the datatype with ID <i>type</i> is registered; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QMetaType::isRegistered(int type)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#isRegistered">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the datatype with ID <i>type</i> is registered; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetatype.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qmetatype.html#typeName">typeName</a>(), and <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">Type</a>.</p></div>
    pub unsafe fn is_registered_1a(type_: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_isRegistered(type_)
    }

    /// <p>Returns <code>true</code> if this <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a> object contains valid information about a type, false otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaType::isRegistered() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#isRegistered-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a> object contains valid information about a type, false otherwise.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn is_registered_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_isRegistered1(self as *const crate::QMetaType)
    }

    /// <p>Returns <code>true</code> if this <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a> object contains valid information about a type, false otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaType::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a> object contains valid information about a type, false otherwise.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn is_valid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_isValid(self as *const crate::QMetaType)
    }

    /// <p>Reads the object of the specified <i>type</i> from the given <i>stream</i> into <i>data</i>. Returns <code>true</code> if the object is loaded successfully; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QMetaType::load(QDataStream& stream, int type, void* data)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#load">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads the object of the specified <i>type</i> from the given <i>stream</i> into <i>data</i>. Returns <code>true</code> if the object is loaded successfully; otherwise returns <code>false</code>.</p>
    /// <p>The type must have been registered with <a href="http://doc.qt.io/qt-5/qmetatype.html#qRegisterMetaType-1">qRegisterMetaType</a>() and <a href="http://doc.qt.io/qt-5/qmetatype.html#qRegisterMetaTypeStreamOperators">qRegisterMetaTypeStreamOperators</a>() beforehand.</p>
    /// <p>Normally, you should not need to call this function directly. Instead, use <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QVariant</a>'s <code>operator&gt;&gt;()</code>, which relies on load() to stream custom types.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetatype.html#save">save</a>() and <a href="http://doc.qt.io/qt-5/qmetatype.html#qRegisterMetaTypeStreamOperators">qRegisterMetaTypeStreamOperators</a>().</p></div>
    pub unsafe fn load(
        stream: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QDataStream>>,
        type_: ::std::os::raw::c_int,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_load(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QDataStream>>::cast_into(stream)
                .as_mut_raw_ptr(),
            type_,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>return a <a href="http://doc.qt.io/qt-5/qmetaobject.html">QMetaObject</a> relative to this type.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QMetaObject* QMetaType::metaObject() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#metaObject">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>return a <a href="http://doc.qt.io/qt-5/qmetaobject.html">QMetaObject</a> relative to this type.</p>
    /// <p>If the type is a pointer type to a subclass of <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#flags">flags</a>() contains <a href="http://doc.qt.io/qt-5/qmetatype.html#TypeFlag-enum">QMetaType::PointerToQObject</a> and this function returns the corresponding <a href="http://doc.qt.io/qt-5/qmetaobject.html">QMetaObject</a>. This can be used to in combinaison with QMetaObject::construct to create <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> of this type.</p>
    /// <p>If the type is a <a href="http://doc.qt.io/qt-5/qobject.html#Q_GADGET">Q_GADGET</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#flags">flags</a>() contains <a href="http://doc.qt.io/qt-5/qmetatype.html#TypeFlag-enum">QMetaType::IsGadget</a>, and this function returns its <a href="http://doc.qt.io/qt-5/qmetaobject.html">QMetaObject</a>. This can be used to retrieve <a href="http://doc.qt.io/qt-5/qmetamethod.html">QMetaMethod</a> and <a href="http://doc.qt.io/qt-5/qmetaproperty.html">QMetaProperty</a> and use them on a pointer of this type. (given by QVariant::data for example)</p>
    /// <p>If the type is an enumeration, <a href="http://doc.qt.io/qt-5/qmetatype.html#flags">flags</a>() contains <a href="http://doc.qt.io/qt-5/qmetatype.html#TypeFlag-enum">QMetaType::IsEnumeration</a>, and this function returns the <a href="http://doc.qt.io/qt-5/qmetaobject.html">QMetaObject</a> of the enclosing object if the enum was registered as a <a href="http://doc.qt.io/qt-5/qobject.html#Q_ENUM">Q_ENUM</a> or 0 otherwise</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetatype.html#metaObjectForType">QMetaType::metaObjectForType</a>() and <a href="http://doc.qt.io/qt-5/qmetatype.html#flags">QMetaType::flags</a>().</p></div>
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMetaType_metaObject(self as *const crate::QMetaType);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>returns <a href="http://doc.qt.io/qt-5/qmetatype.html#metaObject">QMetaType::metaObject</a> for <i>type</i></p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static const QMetaObject* QMetaType::metaObjectForType(int type)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#metaObjectForType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>returns <a href="http://doc.qt.io/qt-5/qmetatype.html#metaObject">QMetaType::metaObject</a> for <i>type</i></p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetatype.html#metaObject">metaObject</a>().</p></div>
    pub unsafe fn meta_object_for_type(
        type_: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaType_metaObjectForType(type_);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a> object that contains all information about type <i>typeId</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMetaType::QMetaType(int type)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#QMetaType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a> object that contains all information about type <i>typeId</i>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn new(type_: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QMetaType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaType_QMetaType(type_);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static int QMetaType::registerNormalizedType(const QByteArray& normalizedTypeName, void (*FN_PTR)(void*) deleter, void* (*FN_PTR)(const void*) creator, void (*FN_PTR)(void*) destructor, void* (*FN_PTR)(void*, const void*) constructor, int size, QFlags<QMetaType::TypeFlag> flags, const QMetaObject* metaObject)```</span>.
    pub unsafe fn register_normalized_type_q_byte_array_option_fn_option_fn_option_fn_option_fn_int_q_flags_type_flag_q_meta_object(
        normalized_type_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        deleter: ::std::option::Option<extern "C" fn(*mut ::std::ffi::c_void)>,
        creator: ::std::option::Option<
            extern "C" fn(*const ::std::ffi::c_void) -> *mut ::std::ffi::c_void,
        >,
        destructor: ::std::option::Option<extern "C" fn(*mut ::std::ffi::c_void)>,
        constructor: ::std::option::Option<
            extern "C" fn(
                *mut ::std::ffi::c_void,
                *const ::std::ffi::c_void,
            ) -> *mut ::std::ffi::c_void,
        >,
        size: ::std::os::raw::c_int,
        flags: crate::QFlags<crate::q_meta_type::TypeFlag>,
        meta_object: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QMetaObject>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_registerNormalizedType(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(
                normalized_type_name,
            )
            .as_raw_ptr(),
            deleter,
            creator,
            destructor,
            constructor,
            size,
            flags.to_int(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QMetaObject>>::cast_into(meta_object)
                .as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```static int QMetaType::registerNormalizedType(const QByteArray& normalizedTypeName, void (*FN_PTR)(void*) destructor, void* (*FN_PTR)(void*, const void*) constructor, int size, QFlags<QMetaType::TypeFlag> flags, const QMetaObject* metaObject)```</span>.
    pub unsafe fn register_normalized_type_q_byte_array_option_fn_option_fn_int_q_flags_type_flag_q_meta_object(
        normalized_type_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        destructor: ::std::option::Option<extern "C" fn(*mut ::std::ffi::c_void)>,
        constructor: ::std::option::Option<
            extern "C" fn(
                *mut ::std::ffi::c_void,
                *const ::std::ffi::c_void,
            ) -> *mut ::std::ffi::c_void,
        >,
        size: ::std::os::raw::c_int,
        flags: crate::QFlags<crate::q_meta_type::TypeFlag>,
        meta_object: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QMetaObject>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_registerNormalizedType1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(
                normalized_type_name,
            )
            .as_raw_ptr(),
            destructor,
            constructor,
            size,
            flags.to_int(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QMetaObject>>::cast_into(meta_object)
                .as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```static int QMetaType::registerNormalizedType(const QByteArray& normalizedTypeName, void (*FN_PTR)(int, void*) destructor, void* (*FN_PTR)(int, void*, const void*) constructor, int size, QFlags<QMetaType::TypeFlag> flags, const QMetaObject* metaObject)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn register_normalized_type_q_byte_array_option_fn_option_fn_int_q_flags_type_flag_q_meta_object2(
        normalized_type_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        destructor: ::std::option::Option<
            extern "C" fn(::std::os::raw::c_int, *mut ::std::ffi::c_void),
        >,
        constructor: ::std::option::Option<
            extern "C" fn(
                ::std::os::raw::c_int,
                *mut ::std::ffi::c_void,
                *const ::std::ffi::c_void,
            ) -> *mut ::std::ffi::c_void,
        >,
        size: ::std::os::raw::c_int,
        flags: crate::QFlags<crate::q_meta_type::TypeFlag>,
        meta_object: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QMetaObject>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_registerNormalizedType4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(
                normalized_type_name,
            )
            .as_raw_ptr(),
            destructor,
            constructor,
            size,
            flags.to_int(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QMetaObject>>::cast_into(meta_object)
                .as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```static int QMetaType::registerNormalizedTypedef(const QByteArray& normalizedTypeName, int aliasId)```</span>.
    pub unsafe fn register_normalized_typedef(
        normalized_type_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        alias_id: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_registerNormalizedTypedef(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(
                normalized_type_name,
            )
            .as_raw_ptr(),
            alias_id,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```static int QMetaType::registerType(const char* typeName, void (*FN_PTR)(void*) deleter, void* (*FN_PTR)(const void*) creator)```</span>.
    pub unsafe fn register_type_3a(
        type_name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        deleter: ::std::option::Option<extern "C" fn(*mut ::std::ffi::c_void)>,
        creator: ::std::option::Option<
            extern "C" fn(*const ::std::ffi::c_void) -> *mut ::std::ffi::c_void,
        >,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_registerType(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(type_name)
                .as_raw_ptr(),
            deleter,
            creator,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```static int QMetaType::registerType(const char* typeName, void (*FN_PTR)(void*) deleter, void* (*FN_PTR)(const void*) creator, void (*FN_PTR)(void*) destructor, void* (*FN_PTR)(void*, const void*) constructor, int size, QFlags<QMetaType::TypeFlag> flags, const QMetaObject* metaObject)```</span>.
    pub unsafe fn register_type_8a(
        type_name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        deleter: ::std::option::Option<extern "C" fn(*mut ::std::ffi::c_void)>,
        creator: ::std::option::Option<
            extern "C" fn(*const ::std::ffi::c_void) -> *mut ::std::ffi::c_void,
        >,
        destructor: ::std::option::Option<extern "C" fn(*mut ::std::ffi::c_void)>,
        constructor: ::std::option::Option<
            extern "C" fn(
                *mut ::std::ffi::c_void,
                *const ::std::ffi::c_void,
            ) -> *mut ::std::ffi::c_void,
        >,
        size: ::std::os::raw::c_int,
        flags: crate::QFlags<crate::q_meta_type::TypeFlag>,
        meta_object: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QMetaObject>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_registerType1(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(type_name)
                .as_raw_ptr(),
            deleter,
            creator,
            destructor,
            constructor,
            size,
            flags.to_int(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QMetaObject>>::cast_into(meta_object)
                .as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```static int QMetaType::registerType(const char* typeName, void (*FN_PTR)(int, void*) destructor, void* (*FN_PTR)(int, void*, const void*) constructor, int size, QFlags<QMetaType::TypeFlag> flags, const QMetaObject* metaObject)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn register_type_6a(
        type_name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        destructor: ::std::option::Option<
            extern "C" fn(::std::os::raw::c_int, *mut ::std::ffi::c_void),
        >,
        constructor: ::std::option::Option<
            extern "C" fn(
                ::std::os::raw::c_int,
                *mut ::std::ffi::c_void,
                *const ::std::ffi::c_void,
            ) -> *mut ::std::ffi::c_void,
        >,
        size: ::std::os::raw::c_int,
        flags: crate::QFlags<crate::q_meta_type::TypeFlag>,
        meta_object: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QMetaObject>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_registerType4(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(type_name)
                .as_raw_ptr(),
            destructor,
            constructor,
            size,
            flags.to_int(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QMetaObject>>::cast_into(meta_object)
                .as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```static int QMetaType::registerTypedef(const char* typeName, int aliasId)```</span>.
    pub unsafe fn register_typedef(
        type_name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        alias_id: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_registerTypedef(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(type_name)
                .as_raw_ptr(),
            alias_id,
        )
    }

    /// <p>Writes the object pointed to by <i>data</i> with the ID <i>type</i> to the given <i>stream</i>. Returns <code>true</code> if the object is saved successfully; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QMetaType::save(QDataStream& stream, int type, const void* data)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#save">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes the object pointed to by <i>data</i> with the ID <i>type</i> to the given <i>stream</i>. Returns <code>true</code> if the object is saved successfully; otherwise returns <code>false</code>.</p>
    /// <p>The type must have been registered with <a href="http://doc.qt.io/qt-5/qmetatype.html#qRegisterMetaType-1">qRegisterMetaType</a>() and <a href="http://doc.qt.io/qt-5/qmetatype.html#qRegisterMetaTypeStreamOperators">qRegisterMetaTypeStreamOperators</a>() beforehand.</p>
    /// <p>Normally, you should not need to call this function directly. Instead, use <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QVariant</a>'s <code>operator&lt;&lt;()</code>, which relies on save() to stream custom types.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetatype.html#load">load</a>() and <a href="http://doc.qt.io/qt-5/qmetatype.html#qRegisterMetaTypeStreamOperators">qRegisterMetaTypeStreamOperators</a>().</p></div>
    pub unsafe fn save(
        stream: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QDataStream>>,
        type_: ::std::os::raw::c_int,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::ffi::c_void>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_save(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QDataStream>>::cast_into(stream)
                .as_mut_raw_ptr(),
            type_,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::ffi::c_void>>::cast_into(data)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the size of the given <i>type</i> in bytes (i.e. sizeof(T), where T is the actual type identified by the <i>type</i> argument).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QMetaType::sizeOf(int type)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#sizeOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the size of the given <i>type</i> in bytes (i.e. sizeof(T), where T is the actual type identified by the <i>type</i> argument).</p>
    /// <p>This function is typically used together with <a href="http://doc.qt.io/qt-5/qmetatype.html#construct">construct</a>() to perform low-level management of the memory used by a type.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetatype.html#construct">construct</a>().</p></div>
    pub unsafe fn size_of_1a(type_: ::std::os::raw::c_int) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_sizeOf(type_)
    }

    /// <p>Returns the size of the type in bytes (i.e. sizeof(T), where T is the actual type for which this <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a> instance was constructed for).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaType::sizeOf() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#sizeOf-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the size of the type in bytes (i.e. sizeof(T), where T is the actual type for which this <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a> instance was constructed for).</p>
    /// <p>This function is typically used together with <a href="http://doc.qt.io/qt-5/qmetatype.html#construct">construct</a>() to perform low-level management of the memory used by a type.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetatype.html#construct">QMetaType::construct</a>() and QMetaType::sizeOf().</p></div>
    pub unsafe fn size_of_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_sizeOf1(self as *const crate::QMetaType)
    }

    /// <p>Returns a handle to the type called <i>typeName</i>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UnknownType</a> if there is no such type.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QMetaType::type(const char* typeName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#type">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a handle to the type called <i>typeName</i>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UnknownType</a> if there is no such type.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetatype.html#isRegistered-1">isRegistered</a>(), <a href="http://doc.qt.io/qt-5/qmetatype.html#typeName">typeName</a>(), and <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">Type</a>.</p></div>
    pub unsafe fn type_char(
        type_name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_type(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(type_name)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QMetaType::type(const QByteArray& typeName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#type-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a handle to the type called <i>typeName</i>, or 0 if there is no such type.</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetatype.html#isRegistered-1">isRegistered</a>() and <a href="http://doc.qt.io/qt-5/qmetatype.html#typeName">typeName</a>().</p></div>
    pub unsafe fn type_q_byte_array(
        type_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_type1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(type_name)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns flags of the given <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QFlags<QMetaType::TypeFlag> QMetaType::typeFlags(int type)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#typeFlags">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns flags of the given <i>type</i>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetatype.html#TypeFlag-enum">QMetaType::TypeFlags</a>.</p></div>
    pub unsafe fn type_flags(
        type_: ::std::os::raw::c_int,
    ) -> crate::QFlags<crate::q_meta_type::TypeFlag> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaType_typeFlags(type_);
        crate::QFlags::from(ffi_result)
    }

    /// <p>Returns the type name associated with the given <i>typeId</i>, or a null pointer if no matching type was found. The returned pointer must not be deleted.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static const char* QMetaType::typeName(int type)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#typeName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the type name associated with the given <i>typeId</i>, or a null pointer if no matching type was found. The returned pointer must not be deleted.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetatype.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qmetatype.html#isRegistered-1">isRegistered</a>(), and <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">Type</a>.</p></div>
    pub unsafe fn type_name(
        type_: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaType_typeName(type_);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```static bool QMetaType::unregisterType(int type)```</span>.
    pub unsafe fn unregister_type(type_: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_unregisterType(type_)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> class is the base class of all Qt objects.</p>
///
/// C++ class: <span style='color: green;'>```QObject```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qobject.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> class is the base class of all Qt objects.</p>
/// <p><a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> is the heart of the Qt <a href="http://doc.qt.io/qt-5/object.html">Object Model</a>. The central feature in this model is a very powerful mechanism for seamless object communication called <a href="http://doc.qt.io/qt-5/signalsandslots.html">signals and slots</a>. You can connect a signal to a slot with <a href="http://doc.qt.io/qt-5/qobject.html#connect">connect</a>() and destroy the connection with <a href="http://doc.qt.io/qt-5/qobject.html#disconnect">disconnect</a>(). To avoid never ending notification loops you can temporarily block signals with <a href="http://doc.qt.io/qt-5/qobject.html#blockSignals">blockSignals</a>(). The protected functions <a href="http://doc.qt.io/qt-5/qobject.html#connectNotify">connectNotify</a>() and <a href="http://doc.qt.io/qt-5/qobject.html#disconnectNotify">disconnectNotify</a>() make it possible to track connections.</p>
/// <p>QObjects organize themselves in <a href="http://doc.qt.io/qt-5/objecttrees.html">object trees</a>. When you create a <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> with another object as parent, the object will automatically add itself to the parent's <a href="http://doc.qt.io/qt-5/qobject.html#children">children</a>() list. The parent takes ownership of the object; i.e., it will automatically delete its children in its destructor. You can look for an object by name and optionally type using <a href="http://doc.qt.io/qt-5/qobject.html#findChild">findChild</a>() or <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
/// <p>Every object has an <a href="http://doc.qt.io/qt-5/qobject.html#objectName-prop">objectName</a>() and its class name can be found via the corresponding <a href="http://doc.qt.io/qt-5/qobject.html#metaObject">metaObject</a>() (see <a href="http://doc.qt.io/qt-5/qmetaobject.html#className">QMetaObject::className</a>()). You can determine whether the object's class inherits another class in the <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> inheritance hierarchy by using the <a href="http://doc.qt.io/qt-5/qobject.html#inherits">inherits</a>() function.</p>
/// <p>When an object is deleted, it emits a <a href="http://doc.qt.io/qt-5/qobject.html#destroyed">destroyed</a>() signal. You can catch this signal to avoid dangling references to QObjects.</p>
/// <p>QObjects can receive events through <a href="http://doc.qt.io/qt-5/qobject.html#event">event</a>() and filter the events of other objects. See <a href="http://doc.qt.io/qt-5/qobject.html#installEventFilter">installEventFilter</a>() and <a href="http://doc.qt.io/qt-5/qobject.html#eventFilter">eventFilter</a>() for details. A convenience handler, <a href="http://doc.qt.io/qt-5/qobject.html#childEvent">childEvent</a>(), can be reimplemented to catch child events.</p>
/// <p>Last but not least, <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> provides the basic timer support in Qt; see <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> for high-level support for timers.</p>
/// <p>Notice that the <a href="http://doc.qt.io/qt-5/qobject.html#Q_OBJECT">Q_OBJECT</a> macro is mandatory for any object that implements signals, slots or properties. You also need to run the <a href="http://doc.qt.io/qt-5/moc.html">Meta Object Compiler</a> on the source file. We strongly recommend the use of this macro in all subclasses of <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> regardless of whether or not they actually use signals, slots and properties, since failure to do so may lead certain functions to exhibit strange behavior.</p>
/// <p>All Qt widgets inherit <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>. The convenience function <a href="http://doc.qt.io/qt-5/qobject.html#isWidgetType">isWidgetType</a>() returns whether an object is actually a widget. It is much faster than <a href="http://doc.qt.io/qt-5/qobject.html#qobject_cast">qobject_cast</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;(<i>obj</i>) or <i>obj</i>-&gt;<a href="http://doc.qt.io/qt-5/qobject.html#inherits">inherits</a>("<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a>").</p>
/// <p>Some <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> functions, e.g. <a href="http://doc.qt.io/qt-5/qobject.html#children">children</a>(), return a <a href="http://doc.qt.io/qt-5/qobject.html#QObjectList-typedef">QObjectList</a>. <a href="http://doc.qt.io/qt-5/qobject.html#QObjectList-typedef">QObjectList</a> is a typedef for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> *&gt;.</p>
/// <a name="thread-affinity"></a></div>
#[repr(C)]
pub struct QObject {
    _unused: u8,
}
impl QObject {
    /// <p>This signal is emitted immediately before the object <i>obj</i> is destroyed, and can not be blocked.</p>
    ///
    /// Returns a built-in Qt signal `QObject::destroyed` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#destroyed">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted immediately before the object <i>obj</i> is destroyed, and can not be blocked.</p>
    /// <p>All the objects's children are destroyed immediately after this signal is emitted.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#deleteLater">deleteLater</a>() and <a href="http://doc.qt.io/qt-5/qpointer.html">QPointer</a>.</p></div>
    pub fn destroyed(&self) -> crate::Signal<(*mut crate::QObject,)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2destroyed(QObject*)\0"),
            )
        }
    }

    /// <p>This signal is emitted after the object's name has been changed. The new object name is passed as <i>objectName</i>.</p>
    ///
    /// Returns a built-in Qt signal `QObject::objectNameChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#objectNameChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted after the object's name has been changed. The new object name is passed as <i>objectName</i>.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>Note:</b> Notifier signal for property <a href="http://doc.qt.io/qt-5/qobject.html#objectName-prop">objectName</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#objectName-prop">QObject::objectName</a>.</p></div>
    pub fn object_name_changed(&self) -> crate::Signal<(*const crate::QString,)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2objectNameChanged(const QString&)\0",
                ),
            )
        }
    }

    /// <p>Schedules this object for deletion.</p>
    ///
    /// Returns a built-in Qt slot `QObject::deleteLater` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#deleteLater">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Schedules this object for deletion.</p>
    /// <p>The object will be deleted when control returns to the event loop. If the event loop is not running when this function is called (e.g. deleteLater() is called on an object before <a href="http://doc.qt.io/qt-5/qcoreapplication.html#exec">QCoreApplication::exec</a>()), the object will be deleted once the event loop is started. If deleteLater() is called after the main event loop has stopped, the object will not be deleted. Since Qt 4.8, if deleteLater() is called on an object that lives in a thread with no running event loop, the object will be destroyed when the thread finishes.</p>
    /// <p>Note that entering and leaving a new event loop (e.g., by opening a modal dialog) will <i>not</i> perform the deferred deletion; for the object to be deleted, the control must return to the event loop from which deleteLater() was called.</p>
    /// <p><b>Note:</b> It is safe to call this function more than once; when the first deferred deletion event is delivered, any pending events for the object are removed from the event queue.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#destroyed">destroyed</a>() and <a href="http://doc.qt.io/qt-5/qpointer.html">QPointer</a>.</p></div>
    pub fn slot_delete_later(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1deleteLater()\0"),
            )
        }
    }

    /// <p>If <i>block</i> is true, signals emitted by this object are blocked (i.e., emitting a signal will not invoke anything connected to it). If <i>block</i> is false, no such blocking will occur.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QObject::blockSignals(bool b)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#blockSignals">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If <i>block</i> is true, signals emitted by this object are blocked (i.e., emitting a signal will not invoke anything connected to it). If <i>block</i> is false, no such blocking will occur.</p>
    /// <p>The return value is the previous value of <a href="http://doc.qt.io/qt-5/qobject.html#signalsBlocked">signalsBlocked</a>().</p>
    /// <p>Note that the <a href="http://doc.qt.io/qt-5/qobject.html#destroyed">destroyed</a>() signal will be emitted even if the signals for this object have been blocked.</p>
    /// <p>Signals emitted while being blocked are not buffered.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#signalsBlocked">signalsBlocked</a>() and <a href="http://doc.qt.io/qt-5/qsignalblocker.html">QSignalBlocker</a>.</p></div>
    pub unsafe fn block_signals(&mut self, b: bool) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QObject_blockSignals(self as *mut crate::QObject, b)
    }

    /// <p>Returns a list of child objects. The <a href="http://doc.qt.io/qt-5/qobject.html#QObjectList-typedef">QObjectList</a> class is defined in the <code>&lt;QObject&gt;</code> header file as the following:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QList<QObject*>& QObject::children() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#children">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of child objects. The <a href="http://doc.qt.io/qt-5/qobject.html#QObjectList-typedef">QObjectList</a> class is defined in the <code>&lt;QObject&gt;</code> header file as the following:</p>
    /// <pre class="qml">
    ///
    ///   <span class="keyword">typedef</span> <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span><span class="operator">*</span><span class="operator">&gt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObjectList-typedef">QObjectList</a></span>;
    ///
    /// </pre>
    /// <p>The first child added is the <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a> object in the list and the last child added is the <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a> object in the list, i.e. new children are appended at the end.</p>
    /// <p>Note that the list order changes when <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> children are <a href="http://doc.qt.io/qt-5/qwidget.html#raise">raised</a> or <a href="http://doc.qt.io/qt-5/qwidget.html#lower">lowered</a>. A widget that is raised becomes the last object in the list, and a widget that is lowered becomes the first object in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#findChild">findChild</a>(), <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>(), <a href="http://doc.qt.io/qt-5/qobject.html#parent">parent</a>(), and <a href="http://doc.qt.io/qt-5/qobject.html#setParent">setParent</a>().</p></div>
    pub unsafe fn children(&self) -> ::cpp_utils::Ref<crate::QListOfQObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QObject_children(self as *const crate::QObject);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Creates a connection of the given <i>type</i> from the <i>signal</i> in the <i>sender</i> object to the <i>method</i> in the <i>receiver</i> object. Returns a handle to the connection that can be used to disconnect it later.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QMetaObject::Connection QObject::connect(const QObject* sender, const char* signal, const QObject* receiver, const char* member, Qt::ConnectionType arg5 = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#connect">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a connection of the given <i>type</i> from the <i>signal</i> in the <i>sender</i> object to the <i>method</i> in the <i>receiver</i> object. Returns a handle to the connection that can be used to disconnect it later.</p>
    /// <p>You must use the <code>SIGNAL()</code> and <code>SLOT()</code> macros when specifying the <i>signal</i> and the <i>method</i>, for example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlabel.html">QLabel</a></span> <span class="operator">*</span>label <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qlabel.html">QLabel</a></span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qscrollbar.html">QScrollBar</a></span> <span class="operator">*</span>scrollBar <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qscrollbar.html">QScrollBar</a></span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span><span class="operator">::</span>connect(scrollBar<span class="operator">,</span> SIGNAL(valueChanged(<span class="type">int</span>))<span class="operator">,</span>
    /// &#32;   &#32;   &#32;   &#32;      label<span class="operator">,</span>  SLOT(setNum(<span class="type">int</span>)));
    ///
    /// </pre>
    /// <p>This example ensures that the label always displays the current scroll bar value. Note that the signal and slots parameters must not contain any variable names, only the type. E.g. the following would not work and return false:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="comment">// WRONG</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span><span class="operator">::</span>connect(scrollBar<span class="operator">,</span> SIGNAL(valueChanged(<span class="type">int</span> value))<span class="operator">,</span>
    /// &#32;   &#32;   &#32;   &#32;      label<span class="operator">,</span> SLOT(setNum(<span class="type">int</span> value)));
    ///
    /// </pre>
    /// <p>A signal can also be connected to another signal:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">class</span> MyWidget : <span class="keyword">public</span> <span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span>
    ///   {
    /// &#32;     Q_OBJECT
    ///
    ///   <span class="keyword">public</span>:
    /// &#32;     MyWidget();
    ///
    ///   <span class="keyword">signals</span>:
    /// &#32;     <span class="type">void</span> buttonClicked();
    ///
    ///   <span class="keyword">private</span>:
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span>myButton;
    ///   };
    ///
    ///   MyWidget<span class="operator">::</span>MyWidget()
    ///   {
    /// &#32;     myButton <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span>(<span class="keyword">this</span>);
    /// &#32;     connect(myButton<span class="operator">,</span> SIGNAL(clicked())<span class="operator">,</span>
    /// &#32;   &#32;   &#32;     <span class="keyword">this</span><span class="operator">,</span> SIGNAL(buttonClicked()));
    ///   }
    ///
    /// </pre>
    /// <p>In this example, the <code>MyWidget</code> constructor relays a signal from a private member variable, and makes it available under a name that relates to <code>MyWidget</code>.</p>
    /// <p>A signal can be connected to many slots and signals. Many signals can be connected to one slot.</p>
    /// <p>If a signal is connected to several slots, the slots are activated in the same order in which the connections were made, when the signal is emitted.</p>
    /// <p>The function returns a <a href="http://doc.qt.io/qt-5/qmetaobject-connection.html">QMetaObject::Connection</a> that represents a handle to a connection if it successfully connects the signal to the slot. The connection handle will be invalid if it cannot create the connection, for example, if <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> is unable to verify the existence of either <i>signal</i> or <i>method</i>, or if their signatures aren't compatible. You can check if the handle is valid by casting it to a bool.</p>
    /// <p>By default, a signal is emitted for every connection you make; two signals are emitted for duplicate connections. You can break all of these connections with a single <a href="http://doc.qt.io/qt-5/qobject.html#disconnect">disconnect</a>() call. If you pass the <a href="http://doc.qt.io/qt-5/qt.html#ConnectionType-enum">Qt::UniqueConnection</a> <i>type</i>, the connection will only be made if it is not a duplicate. If there is already a duplicate (exact same signal to the exact same slot on the same objects), the connection will fail and connect will return an invalid <a href="http://doc.qt.io/qt-5/qmetaobject-connection.html">QMetaObject::Connection</a>.</p>
    /// <p><b>Note: </b>Qt::UniqueConnections do not work for lambdas, non-member functions and functors; they only apply to connecting to member functions.</p><p>The optional <i>type</i> parameter describes the type of connection to establish. In particular, it determines whether a particular signal is delivered to a slot immediately or queued for delivery at a later time. If the signal is queued, the parameters must be of types that are known to Qt's meta-object system, because Qt needs to copy the arguments to store them in an event behind the scenes. If you try to use a queued connection and get the error message</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span><span class="operator">::</span>connect: Cannot queue arguments of type <span class="char">'MyType'</span>
    ///   (Make sure <span class="char">'MyType'</span> is registered <span class="keyword">using</span> <a href="http://doc.qt.io/qt-5/qmetatype.html#qRegisterMetaType-1">qRegisterMetaType</a>()<span class="operator">.</span>)
    ///
    /// </pre>
    /// <p>call <a href="http://doc.qt.io/qt-5/qmetatype.html#qRegisterMetaType-1">qRegisterMetaType</a>() to register the data type before you establish the connection.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#disconnect">disconnect</a>(), <a href="http://doc.qt.io/qt-5/qobject.html#sender">sender</a>(), <a href="http://doc.qt.io/qt-5/qmetatype.html#qRegisterMetaType-1">qRegisterMetaType</a>(), <a href="http://doc.qt.io/qt-5/qmetatype.html#Q_DECLARE_METATYPE">Q_DECLARE_METATYPE</a>(), and <a href="http://doc.qt.io/qt-5/signalsandslots-syntaxes.html">Differences between String-Based and Functor-Based Connections</a>.</p></div>
    pub unsafe fn connect_5a(
        sender: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        signal: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        receiver: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        member: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        arg5: crate::ConnectionType,
    ) -> ::cpp_utils::CppBox<crate::q_meta_object::Connection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_connect(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(sender)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(signal)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(receiver)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(member)
                .as_raw_ptr(),
            arg5,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a connection of the given <i>type</i> from the <i>signal</i> in the <i>sender</i> object to the <i>method</i> in the <i>receiver</i> object. Returns a handle to the connection that can be used to disconnect it later.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QMetaObject::Connection QObject::connect(const QObject* sender, const char* signal, const QObject* receiver, const char* member)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#connect">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a connection of the given <i>type</i> from the <i>signal</i> in the <i>sender</i> object to the <i>method</i> in the <i>receiver</i> object. Returns a handle to the connection that can be used to disconnect it later.</p>
    /// <p>You must use the <code>SIGNAL()</code> and <code>SLOT()</code> macros when specifying the <i>signal</i> and the <i>method</i>, for example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlabel.html">QLabel</a></span> <span class="operator">*</span>label <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qlabel.html">QLabel</a></span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qscrollbar.html">QScrollBar</a></span> <span class="operator">*</span>scrollBar <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qscrollbar.html">QScrollBar</a></span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span><span class="operator">::</span>connect(scrollBar<span class="operator">,</span> SIGNAL(valueChanged(<span class="type">int</span>))<span class="operator">,</span>
    /// &#32;   &#32;   &#32;   &#32;      label<span class="operator">,</span>  SLOT(setNum(<span class="type">int</span>)));
    ///
    /// </pre>
    /// <p>This example ensures that the label always displays the current scroll bar value. Note that the signal and slots parameters must not contain any variable names, only the type. E.g. the following would not work and return false:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="comment">// WRONG</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span><span class="operator">::</span>connect(scrollBar<span class="operator">,</span> SIGNAL(valueChanged(<span class="type">int</span> value))<span class="operator">,</span>
    /// &#32;   &#32;   &#32;   &#32;      label<span class="operator">,</span> SLOT(setNum(<span class="type">int</span> value)));
    ///
    /// </pre>
    /// <p>A signal can also be connected to another signal:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">class</span> MyWidget : <span class="keyword">public</span> <span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span>
    ///   {
    /// &#32;     Q_OBJECT
    ///
    ///   <span class="keyword">public</span>:
    /// &#32;     MyWidget();
    ///
    ///   <span class="keyword">signals</span>:
    /// &#32;     <span class="type">void</span> buttonClicked();
    ///
    ///   <span class="keyword">private</span>:
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span>myButton;
    ///   };
    ///
    ///   MyWidget<span class="operator">::</span>MyWidget()
    ///   {
    /// &#32;     myButton <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span>(<span class="keyword">this</span>);
    /// &#32;     connect(myButton<span class="operator">,</span> SIGNAL(clicked())<span class="operator">,</span>
    /// &#32;   &#32;   &#32;     <span class="keyword">this</span><span class="operator">,</span> SIGNAL(buttonClicked()));
    ///   }
    ///
    /// </pre>
    /// <p>In this example, the <code>MyWidget</code> constructor relays a signal from a private member variable, and makes it available under a name that relates to <code>MyWidget</code>.</p>
    /// <p>A signal can be connected to many slots and signals. Many signals can be connected to one slot.</p>
    /// <p>If a signal is connected to several slots, the slots are activated in the same order in which the connections were made, when the signal is emitted.</p>
    /// <p>The function returns a <a href="http://doc.qt.io/qt-5/qmetaobject-connection.html">QMetaObject::Connection</a> that represents a handle to a connection if it successfully connects the signal to the slot. The connection handle will be invalid if it cannot create the connection, for example, if <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> is unable to verify the existence of either <i>signal</i> or <i>method</i>, or if their signatures aren't compatible. You can check if the handle is valid by casting it to a bool.</p>
    /// <p>By default, a signal is emitted for every connection you make; two signals are emitted for duplicate connections. You can break all of these connections with a single <a href="http://doc.qt.io/qt-5/qobject.html#disconnect">disconnect</a>() call. If you pass the <a href="http://doc.qt.io/qt-5/qt.html#ConnectionType-enum">Qt::UniqueConnection</a> <i>type</i>, the connection will only be made if it is not a duplicate. If there is already a duplicate (exact same signal to the exact same slot on the same objects), the connection will fail and connect will return an invalid <a href="http://doc.qt.io/qt-5/qmetaobject-connection.html">QMetaObject::Connection</a>.</p>
    /// <p><b>Note: </b>Qt::UniqueConnections do not work for lambdas, non-member functions and functors; they only apply to connecting to member functions.</p><p>The optional <i>type</i> parameter describes the type of connection to establish. In particular, it determines whether a particular signal is delivered to a slot immediately or queued for delivery at a later time. If the signal is queued, the parameters must be of types that are known to Qt's meta-object system, because Qt needs to copy the arguments to store them in an event behind the scenes. If you try to use a queued connection and get the error message</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span><span class="operator">::</span>connect: Cannot queue arguments of type <span class="char">'MyType'</span>
    ///   (Make sure <span class="char">'MyType'</span> is registered <span class="keyword">using</span> <a href="http://doc.qt.io/qt-5/qmetatype.html#qRegisterMetaType-1">qRegisterMetaType</a>()<span class="operator">.</span>)
    ///
    /// </pre>
    /// <p>call <a href="http://doc.qt.io/qt-5/qmetatype.html#qRegisterMetaType-1">qRegisterMetaType</a>() to register the data type before you establish the connection.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#disconnect">disconnect</a>(), <a href="http://doc.qt.io/qt-5/qobject.html#sender">sender</a>(), <a href="http://doc.qt.io/qt-5/qmetatype.html#qRegisterMetaType-1">qRegisterMetaType</a>(), <a href="http://doc.qt.io/qt-5/qmetatype.html#Q_DECLARE_METATYPE">Q_DECLARE_METATYPE</a>(), and <a href="http://doc.qt.io/qt-5/signalsandslots-syntaxes.html">Differences between String-Based and Functor-Based Connections</a>.</p></div>
    pub unsafe fn connect_4a(
        sender: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        signal: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        receiver: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        member: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::q_meta_object::Connection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_connect3(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(sender)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(signal)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(receiver)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(member)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a connection of the given <i>type</i> from the <i>signal</i> in the <i>sender</i> object to the <i>method</i> in the <i>receiver</i> object. Returns a handle to the connection that can be used to disconnect it later.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QMetaObject::Connection QObject::connect(const QObject* sender, const QMetaMethod& signal, const QObject* receiver, const QMetaMethod& method, Qt::ConnectionType type = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#connect-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a connection of the given <i>type</i> from the <i>signal</i> in the <i>sender</i> object to the <i>method</i> in the <i>receiver</i> object. Returns a handle to the connection that can be used to disconnect it later.</p>
    /// <p>The Connection handle will be invalid if it cannot create the connection, for example, the parameters were invalid. You can check if the <a href="http://doc.qt.io/qt-5/qmetaobject-connection.html">QMetaObject::Connection</a> is valid by casting it to a bool.</p>
    /// <p>This function works in the same way as <code>connect(const QObject *sender, const char *signal, const QObject *receiver, const char *method, Qt::ConnectionType type)</code> but it uses <a href="http://doc.qt.io/qt-5/qmetamethod.html">QMetaMethod</a> to specify signal and method.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#connect">connect</a>(const QObject *sender, const char *signal, const QObject *receiver, const char *method, Qt::ConnectionType type).</p></div>
    pub unsafe fn connect_by_meta_methods_5a(
        sender: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        signal: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMetaMethod>>,
        receiver: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        method: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMetaMethod>>,
        type_: crate::ConnectionType,
    ) -> ::cpp_utils::CppBox<crate::q_meta_object::Connection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_connect1(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(sender)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMetaMethod>>::cast_into(signal)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(receiver)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMetaMethod>>::cast_into(method)
                .as_raw_ptr(),
            type_,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a connection of the given <i>type</i> from the <i>signal</i> in the <i>sender</i> object to the <i>method</i> in the <i>receiver</i> object. Returns a handle to the connection that can be used to disconnect it later.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QMetaObject::Connection QObject::connect(const QObject* sender, const QMetaMethod& signal, const QObject* receiver, const QMetaMethod& method)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#connect-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a connection of the given <i>type</i> from the <i>signal</i> in the <i>sender</i> object to the <i>method</i> in the <i>receiver</i> object. Returns a handle to the connection that can be used to disconnect it later.</p>
    /// <p>The Connection handle will be invalid if it cannot create the connection, for example, the parameters were invalid. You can check if the <a href="http://doc.qt.io/qt-5/qmetaobject-connection.html">QMetaObject::Connection</a> is valid by casting it to a bool.</p>
    /// <p>This function works in the same way as <code>connect(const QObject *sender, const char *signal, const QObject *receiver, const char *method, Qt::ConnectionType type)</code> but it uses <a href="http://doc.qt.io/qt-5/qmetamethod.html">QMetaMethod</a> to specify signal and method.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#connect">connect</a>(const QObject *sender, const char *signal, const QObject *receiver, const char *method, Qt::ConnectionType type).</p></div>
    pub unsafe fn connect_by_meta_methods_4a(
        sender: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        signal: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMetaMethod>>,
        receiver: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        method: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMetaMethod>>,
    ) -> ::cpp_utils::CppBox<crate::q_meta_object::Connection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_connect4(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(sender)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMetaMethod>>::cast_into(signal)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(receiver)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMetaMethod>>::cast_into(method)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Schedules this object for deletion.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QObject::deleteLater()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#deleteLater">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Schedules this object for deletion.</p>
    /// <p>The object will be deleted when control returns to the event loop. If the event loop is not running when this function is called (e.g. deleteLater() is called on an object before <a href="http://doc.qt.io/qt-5/qcoreapplication.html#exec">QCoreApplication::exec</a>()), the object will be deleted once the event loop is started. If deleteLater() is called after the main event loop has stopped, the object will not be deleted. Since Qt 4.8, if deleteLater() is called on an object that lives in a thread with no running event loop, the object will be destroyed when the thread finishes.</p>
    /// <p>Note that entering and leaving a new event loop (e.g., by opening a modal dialog) will <i>not</i> perform the deferred deletion; for the object to be deleted, the control must return to the event loop from which deleteLater() was called.</p>
    /// <p><b>Note:</b> It is safe to call this function more than once; when the first deferred deletion event is delivered, any pending events for the object are removed from the event queue.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#destroyed">destroyed</a>() and <a href="http://doc.qt.io/qt-5/qpointer.html">QPointer</a>.</p></div>
    pub unsafe fn delete_later(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QObject_deleteLater(self as *mut crate::QObject)
    }

    /// <p>Disconnects <i>signal</i> in object <i>sender</i> from <i>method</i> in object <i>receiver</i>. Returns <code>true</code> if the connection is successfully broken; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QObject::disconnect(const QObject* sender, const char* signal, const QObject* receiver, const char* member)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#disconnect">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Disconnects <i>signal</i> in object <i>sender</i> from <i>method</i> in object <i>receiver</i>. Returns <code>true</code> if the connection is successfully broken; otherwise returns <code>false</code>.</p>
    /// <p>A signal-slot connection is removed when either of the objects involved are destroyed.</p>
    /// <p>disconnect() is typically used in three ways, as the following examples demonstrate.</p>
    /// <ol class="1" type="1"><li>Disconnect everything connected to an object's signals:<pre class="cpp">
    ///
    ///   disconnect(myObject<span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="number">0</span>);
    ///
    /// </pre>
    /// <p>equivalent to the non-static overloaded function</p>
    /// <pre class="cpp">
    ///
    ///   myObject<span class="operator">-</span><span class="operator">&gt;</span>disconnect();
    ///
    /// </pre>
    /// </li>
    /// <li>Disconnect everything connected to a specific signal:<pre class="cpp">
    ///
    ///   disconnect(myObject<span class="operator">,</span> SIGNAL(mySignal())<span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="number">0</span>);
    ///
    /// </pre>
    /// <p>equivalent to the non-static overloaded function</p>
    /// <pre class="cpp">
    ///
    ///   myObject<span class="operator">-</span><span class="operator">&gt;</span>disconnect(SIGNAL(mySignal()));
    ///
    /// </pre>
    /// </li>
    /// <li>Disconnect a specific receiver:<pre class="cpp">
    ///
    ///   disconnect(myObject<span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> myReceiver<span class="operator">,</span> <span class="number">0</span>);
    ///
    /// </pre>
    /// <p>equivalent to the non-static overloaded function</p>
    /// <pre class="cpp">
    ///
    ///   myObject<span class="operator">-</span><span class="operator">&gt;</span>disconnect(myReceiver);
    ///
    /// </pre>
    /// </li>
    /// </ol>
    /// <p>0 may be used as a wildcard, meaning "any signal", "any receiving object", or "any slot in the receiving object", respectively.</p>
    /// <p>The <i>sender</i> may never be 0. (You cannot disconnect signals from more than one object in a single call.)</p>
    /// <p>If <i>signal</i> is 0, it disconnects <i>receiver</i> and <i>method</i> from any signal. If not, only the specified signal is disconnected.</p>
    /// <p>If <i>receiver</i> is 0, it disconnects anything connected to <i>signal</i>. If not, slots in objects other than <i>receiver</i> are not disconnected.</p>
    /// <p>If <i>method</i> is 0, it disconnects anything that is connected to <i>receiver</i>. If not, only slots named <i>method</i> will be disconnected, and all other slots are left alone. The <i>method</i> must be 0 if <i>receiver</i> is left out, so you cannot disconnect a specifically-named slot on all objects.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#connect">connect</a>().</p></div>
    pub unsafe fn disconnect_q_object_char_q_object_char(
        sender: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        signal: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        receiver: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        member: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QObject_disconnect(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(sender)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(signal)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(receiver)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(member)
                .as_raw_ptr(),
        )
    }

    /// <p>Disconnects <i>signal</i> in object <i>sender</i> from <i>method</i> in object <i>receiver</i>. Returns <code>true</code> if the connection is successfully broken; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QObject::disconnect(const QObject* sender, const QMetaMethod& signal, const QObject* receiver, const QMetaMethod& member)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#disconnect-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Disconnects <i>signal</i> in object <i>sender</i> from <i>method</i> in object <i>receiver</i>. Returns <code>true</code> if the connection is successfully broken; otherwise returns <code>false</code>.</p>
    /// <p>This function provides the same possibilities like <code>disconnect(const QObject *sender, const char *signal, const QObject *receiver, const char *method)</code> but uses <a href="http://doc.qt.io/qt-5/qmetamethod.html">QMetaMethod</a> to represent the signal and the method to be disconnected.</p>
    /// <p>Additionally this function returnsfalse and no signals and slots disconnected if:</p>
    /// <ol class="1" type="1"><li><i>signal</i> is not a member of sender class or one of its parent classes.</li>
    /// <li><i>method</i> is not a member of receiver class or one of its parent classes.</li>
    /// <li><i>signal</i> instance represents not a signal.</li>
    /// </ol>
    /// <p>QMetaMethod() may be used as wildcard in the meaning "any signal" or "any slot in receiving object". In the same way 0 can be used for <i>receiver</i> in the meaning "any receiving object". In this case method should also be QMetaMethod(). <i>sender</i> parameter should be never 0.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#disconnect">disconnect</a>(const QObject *sender, const char *signal, const QObject *receiver, const char *method).</p></div>
    pub unsafe fn disconnect_q_object_q_meta_method_q_object_q_meta_method(
        sender: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        signal: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMetaMethod>>,
        receiver: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        member: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMetaMethod>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QObject_disconnect1(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(sender)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMetaMethod>>::cast_into(signal)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(receiver)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMetaMethod>>::cast_into(member)
                .as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#disconnect">disconnect</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QObject::disconnect(const char* signal = …, const QObject* receiver = …, const char* member = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#disconnect-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#disconnect">disconnect</a>().</p>
    /// <p>Disconnects <i>signal</i> from <i>method</i> of <i>receiver</i>.</p>
    /// <p>A signal-slot connection is removed when either of the objects involved are destroyed.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p></div>
    pub unsafe fn disconnect_char_q_object_char(
        &self,
        signal: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        receiver: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        member: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QObject_disconnect2(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(signal)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(receiver)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(member)
                .as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#disconnect">disconnect</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QObject::disconnect(const QObject* receiver, const char* member = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#disconnect-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#disconnect">disconnect</a>().</p>
    /// <p>Disconnects all signals in this object from <i>receiver</i>'s <i>method</i>.</p>
    /// <p>A signal-slot connection is removed when either of the objects involved are destroyed.</p></div>
    pub unsafe fn disconnect_q_object_char(
        &self,
        receiver: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        member: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QObject_disconnect3(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(receiver)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(member)
                .as_raw_ptr(),
        )
    }

    /// <p>Disconnect a connection.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QObject::disconnect(const QMetaObject::Connection& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#disconnect-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Disconnect a connection.</p>
    /// <p>If the <i>connection</i> is invalid or has already been disconnected, do nothing and return false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#connect">connect</a>().</p></div>
    pub unsafe fn disconnect_connection(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_meta_object::Connection>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QObject_disconnect4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_meta_object::Connection>>::cast_into(
                arg1,
            )
            .as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#disconnect">disconnect</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QObject::disconnect(const char* signal = …, const QObject* receiver = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#disconnect-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#disconnect">disconnect</a>().</p>
    /// <p>Disconnects <i>signal</i> from <i>method</i> of <i>receiver</i>.</p>
    /// <p>A signal-slot connection is removed when either of the objects involved are destroyed.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p></div>
    pub unsafe fn disconnect_char_q_object(
        &self,
        signal: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        receiver: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QObject_disconnect5(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(signal)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(receiver)
                .as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#disconnect">disconnect</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QObject::disconnect(const char* signal = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#disconnect-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#disconnect">disconnect</a>().</p>
    /// <p>Disconnects <i>signal</i> from <i>method</i> of <i>receiver</i>.</p>
    /// <p>A signal-slot connection is removed when either of the objects involved are destroyed.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p></div>
    pub unsafe fn disconnect_char(
        &self,
        signal: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QObject_disconnect6(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(signal)
                .as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#disconnect">disconnect</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QObject::disconnect() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#disconnect-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#disconnect">disconnect</a>().</p>
    /// <p>Disconnects <i>signal</i> from <i>method</i> of <i>receiver</i>.</p>
    /// <p>A signal-slot connection is removed when either of the objects involved are destroyed.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p></div>
    pub unsafe fn disconnect(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QObject_disconnect7(self as *const crate::QObject)
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#disconnect">disconnect</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QObject::disconnect(const QObject* receiver) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#disconnect-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#disconnect">disconnect</a>().</p>
    /// <p>Disconnects all signals in this object from <i>receiver</i>'s <i>method</i>.</p>
    /// <p>A signal-slot connection is removed when either of the objects involved are destroyed.</p></div>
    pub unsafe fn disconnect_q_object(
        &self,
        receiver: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QObject_disconnect8(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(receiver)
                .as_raw_ptr(),
        )
    }

    /// <p>Dumps information about signal connections, etc. for this object to the debug output.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QObject::dumpObjectInfo()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#dumpObjectInfo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Dumps information about signal connections, etc. for this object to the debug output.</p>
    /// <p><b>Note: </b>before Qt 5.9, this function was not const.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#dumpObjectTree">dumpObjectTree</a>().</p></div>
    pub unsafe fn dump_object_info_mut(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QObject_dumpObjectInfo(self as *mut crate::QObject)
    }

    /// <p>Dumps information about signal connections, etc. for this object to the debug output.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QObject::dumpObjectInfo() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#dumpObjectInfo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Dumps information about signal connections, etc. for this object to the debug output.</p>
    /// <p><b>Note: </b>before Qt 5.9, this function was not const.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#dumpObjectTree">dumpObjectTree</a>().</p></div>
    pub unsafe fn dump_object_info(&self) {
        crate::__ffi::ctr_qt_core_ffi_QObject_dumpObjectInfo1(self as *const crate::QObject)
    }

    /// <p>Dumps a tree of children to the debug output.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QObject::dumpObjectTree()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#dumpObjectTree">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Dumps a tree of children to the debug output.</p>
    /// <p><b>Note: </b>before Qt 5.9, this function was not const.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#dumpObjectInfo">dumpObjectInfo</a>().</p></div>
    pub unsafe fn dump_object_tree_mut(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QObject_dumpObjectTree(self as *mut crate::QObject)
    }

    /// <p>Dumps a tree of children to the debug output.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QObject::dumpObjectTree() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#dumpObjectTree">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Dumps a tree of children to the debug output.</p>
    /// <p><b>Note: </b>before Qt 5.9, this function was not const.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#dumpObjectInfo">dumpObjectInfo</a>().</p></div>
    pub unsafe fn dump_object_tree(&self) {
        crate::__ffi::ctr_qt_core_ffi_QObject_dumpObjectTree1(self as *const crate::QObject)
    }

    /// <p>Returns the names of all properties that were dynamically added to the object using <a href="http://doc.qt.io/qt-5/qobject.html#setProperty">setProperty</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QByteArray> QObject::dynamicPropertyNames() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#dynamicPropertyNames">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the names of all properties that were dynamically added to the object using <a href="http://doc.qt.io/qt-5/qobject.html#setProperty">setProperty</a>().</p>
    /// <p>This function was introduced in  Qt 4.2.</p></div>
    pub unsafe fn dynamic_property_names(&self) -> ::cpp_utils::CppBox<crate::QListOfQByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_dynamicPropertyNames(
            self as *const crate::QObject,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This virtual function receives events to an object and should return true if the event <i>e</i> was recognized and processed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QObject::event(QEvent* event)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#event">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This virtual function receives events to an object and should return true if the event <i>e</i> was recognized and processed.</p>
    /// <p>The event() function can be reimplemented to customize the behavior of an object.</p>
    /// <p>Make sure you call the parent event class implementation for all the events you did not handle.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">class</span> MyClass : <span class="keyword">public</span> <span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span>
    ///   {
    /// &#32;     Q_OBJECT
    ///
    ///   <span class="keyword">public</span>:
    /// &#32;     MyClass(<span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span>parent <span class="operator">=</span> <span class="number">0</span>);
    /// &#32;     <span class="operator">~</span>MyClass();
    ///
    /// &#32;     bool event(<span class="type"><a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a></span><span class="operator">*</span> ev)
    /// &#32;     {
    /// &#32;   &#32;     <span class="keyword">if</span> (ev<span class="operator">-</span><span class="operator">&gt;</span>type() <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a></span><span class="operator">::</span>PolishRequest) {
    /// &#32;   &#32;   &#32;     <span class="comment">// overwrite handling of PolishRequest if any</span>
    /// &#32;   &#32;   &#32;     doThings();
    /// &#32;   &#32;   &#32;     <span class="keyword">return</span> <span class="keyword">true</span>;
    /// &#32;   &#32;     } <span class="keyword">else</span>  <span class="keyword">if</span> (ev<span class="operator">-</span><span class="operator">&gt;</span>type() <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a></span><span class="operator">::</span>Show) {
    /// &#32;   &#32;   &#32;     <span class="comment">// complement handling of Show if any</span>
    /// &#32;   &#32;   &#32;     doThings2();
    /// &#32;   &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span><span class="operator">::</span>event(ev);
    /// &#32;   &#32;   &#32;     <span class="keyword">return</span> <span class="keyword">true</span>;
    /// &#32;   &#32;     }
    /// &#32;   &#32;     <span class="comment">// Make sure the rest of events are handled</span>
    /// &#32;   &#32;     <span class="keyword">return</span> <span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span><span class="operator">::</span>event(ev);
    /// &#32;     }
    ///   };
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#installEventFilter">installEventFilter</a>(), <a href="http://doc.qt.io/qt-5/qobject.html#timerEvent">timerEvent</a>(), <a href="http://doc.qt.io/qt-5/qcoreapplication.html#sendEvent">QCoreApplication::sendEvent</a>(), and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#postEvent">QCoreApplication::postEvent</a>().</p></div>
    pub unsafe fn event(
        &mut self,
        event: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QEvent>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QObject_event(
            self as *mut crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QEvent>>::cast_into(event)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Filters events if this object has been installed as an event filter for the <i>watched</i> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QObject::eventFilter(QObject* watched, QEvent* event)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#eventFilter">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Filters events if this object has been installed as an event filter for the <i>watched</i> object.</p>
    /// <p>In your reimplementation of this function, if you want to filter the <i>event</i> out, i.e. stop it being handled further, return true; otherwise return false.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">class</span> MainWindow : <span class="keyword">public</span> <span class="type"><a href="http://doc.qt.io/qt-5/qmainwindow.html">QMainWindow</a></span>
    ///   {
    ///   <span class="keyword">public</span>:
    /// &#32;     MainWindow();
    ///
    ///   <span class="keyword">protected</span>:
    /// &#32;     bool eventFilter(<span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span> <span class="operator">*</span>obj<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a></span> <span class="operator">*</span>ev);
    ///
    ///   <span class="keyword">private</span>:
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qtextedit.html">QTextEdit</a></span> <span class="operator">*</span>textEdit;
    ///   };
    ///
    ///   MainWindow<span class="operator">::</span>MainWindow()
    ///   {
    /// &#32;     textEdit <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtextedit.html">QTextEdit</a></span>;
    /// &#32;     setCentralWidget(textEdit);
    ///
    /// &#32;     textEdit<span class="operator">-</span><span class="operator">&gt;</span>installEventFilter(<span class="keyword">this</span>);
    ///   }
    ///
    ///   bool MainWindow<span class="operator">::</span>eventFilter(<span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span> <span class="operator">*</span>obj<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a></span> <span class="operator">*</span>event)
    ///   {
    /// &#32;     <span class="keyword">if</span> (obj <span class="operator">=</span><span class="operator">=</span> textEdit) {
    /// &#32;   &#32;     <span class="keyword">if</span> (event<span class="operator">-</span><span class="operator">&gt;</span>type() <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a></span><span class="operator">::</span>KeyPress) {
    /// &#32;   &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qkeyevent.html">QKeyEvent</a></span> <span class="operator">*</span>keyEvent <span class="operator">=</span> <span class="keyword">static_cast</span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qkeyevent.html">QKeyEvent</a></span><span class="operator">*</span><span class="operator">&gt;</span>(event);
    /// &#32;   &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Ate key press"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> keyEvent<span class="operator">-</span><span class="operator">&gt;</span>key();
    /// &#32;   &#32;   &#32;     <span class="keyword">return</span> <span class="keyword">true</span>;
    /// &#32;   &#32;     } <span class="keyword">else</span> {
    /// &#32;   &#32;   &#32;     <span class="keyword">return</span> <span class="keyword">false</span>;
    /// &#32;   &#32;     }
    /// &#32;     } <span class="keyword">else</span> {
    /// &#32;   &#32;     <span class="comment">// pass the event on to the parent class</span>
    /// &#32;   &#32;     <span class="keyword">return</span> <span class="type"><a href="http://doc.qt.io/qt-5/qmainwindow.html">QMainWindow</a></span><span class="operator">::</span>eventFilter(obj<span class="operator">,</span> event);
    /// &#32;     }
    ///   }
    ///
    /// </pre>
    /// <p>Notice in the example above that unhandled events are passed to the base class's eventFilter() function, since the base class might have reimplemented eventFilter() for its own internal purposes.</p>
    /// <p><b>Warning:</b> If you delete the receiver object in this function, be sure to return true. Otherwise, Qt will forward the event to the deleted object and the program might crash.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#installEventFilter">installEventFilter</a>().</p></div>
    pub unsafe fn event_filter(
        &mut self,
        watched: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
        event: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QEvent>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QObject_eventFilter(
            self as *mut crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(watched)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QEvent>>::cast_into(event)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the child of this object that can be cast into type T and that is called <i>name</i>, or 0 if there is no such object. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QObject* QObject::findChild<QObject*>(const QString& aName = …, QFlags<Qt::FindChildOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChild">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the child of this object that can be cast into type T and that is called <i>name</i>, or 0 if there is no such object. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>If there is more than one child matching the search, the most direct ancestor is returned. If there are several direct ancestors, it is undefined which one will be returned. In that case, <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>() should be used.</p>
    /// <p>This example returns a child <code>QPushButton</code> of <code>parentWidget</code> named <code>"button1"</code>, even if the button isn't a direct child of the parent:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span>button <span class="operator">=</span> parentWidget<span class="operator">-</span><span class="operator">&gt;</span>findChild<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="string">"button1"</span>);
    ///
    /// </pre>
    /// <p>This example returns a <code>QListWidget</code> child of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlistwidget.html">QListWidget</a></span> <span class="operator">*</span>list <span class="operator">=</span> parentWidget<span class="operator">-</span><span class="operator">&gt;</span>findChild<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qlistwidget.html">QListWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span>();
    ///
    /// </pre>
    /// <p>This example returns a child <code>QPushButton</code> of <code>parentWidget</code> (its direct parent) named <code>"button1"</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span>button <span class="operator">=</span> parentWidget<span class="operator">-</span><span class="operator">&gt;</span>findChild<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="string">"button1"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>FindDirectChildrenOnly);
    ///
    /// </pre>
    /// <p>This example returns a <code>QListWidget</code> child of <code>parentWidget</code>, its direct parent:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlistwidget.html">QListWidget</a></span> <span class="operator">*</span>list <span class="operator">=</span> parentWidget<span class="operator">-</span><span class="operator">&gt;</span>findChild<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qlistwidget.html">QListWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>()<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>FindDirectChildrenOnly);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p></div>
    pub unsafe fn find_child_q_object_2a(
        &self,
        a_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        options: crate::QFlags<crate::FindChildOption>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChild_QObject_ptr(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a_name)
                .as_raw_ptr(),
            options.to_int(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns the child of this object that can be cast into type T and that is called <i>name</i>, or 0 if there is no such object. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QObject* QObject::findChild<QObject*>(const QString& aName = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChild">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the child of this object that can be cast into type T and that is called <i>name</i>, or 0 if there is no such object. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>If there is more than one child matching the search, the most direct ancestor is returned. If there are several direct ancestors, it is undefined which one will be returned. In that case, <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>() should be used.</p>
    /// <p>This example returns a child <code>QPushButton</code> of <code>parentWidget</code> named <code>"button1"</code>, even if the button isn't a direct child of the parent:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span>button <span class="operator">=</span> parentWidget<span class="operator">-</span><span class="operator">&gt;</span>findChild<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="string">"button1"</span>);
    ///
    /// </pre>
    /// <p>This example returns a <code>QListWidget</code> child of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlistwidget.html">QListWidget</a></span> <span class="operator">*</span>list <span class="operator">=</span> parentWidget<span class="operator">-</span><span class="operator">&gt;</span>findChild<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qlistwidget.html">QListWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span>();
    ///
    /// </pre>
    /// <p>This example returns a child <code>QPushButton</code> of <code>parentWidget</code> (its direct parent) named <code>"button1"</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span>button <span class="operator">=</span> parentWidget<span class="operator">-</span><span class="operator">&gt;</span>findChild<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="string">"button1"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>FindDirectChildrenOnly);
    ///
    /// </pre>
    /// <p>This example returns a <code>QListWidget</code> child of <code>parentWidget</code>, its direct parent:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlistwidget.html">QListWidget</a></span> <span class="operator">*</span>list <span class="operator">=</span> parentWidget<span class="operator">-</span><span class="operator">&gt;</span>findChild<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qlistwidget.html">QListWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>()<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>FindDirectChildrenOnly);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p></div>
    pub unsafe fn find_child_q_object_1a(
        &self,
        a_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChild_QObject_ptr1(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns the child of this object that can be cast into type T and that is called <i>name</i>, or 0 if there is no such object. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QObject* QObject::findChild<QObject*>() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChild">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the child of this object that can be cast into type T and that is called <i>name</i>, or 0 if there is no such object. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>If there is more than one child matching the search, the most direct ancestor is returned. If there are several direct ancestors, it is undefined which one will be returned. In that case, <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>() should be used.</p>
    /// <p>This example returns a child <code>QPushButton</code> of <code>parentWidget</code> named <code>"button1"</code>, even if the button isn't a direct child of the parent:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span>button <span class="operator">=</span> parentWidget<span class="operator">-</span><span class="operator">&gt;</span>findChild<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="string">"button1"</span>);
    ///
    /// </pre>
    /// <p>This example returns a <code>QListWidget</code> child of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlistwidget.html">QListWidget</a></span> <span class="operator">*</span>list <span class="operator">=</span> parentWidget<span class="operator">-</span><span class="operator">&gt;</span>findChild<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qlistwidget.html">QListWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span>();
    ///
    /// </pre>
    /// <p>This example returns a child <code>QPushButton</code> of <code>parentWidget</code> (its direct parent) named <code>"button1"</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span>button <span class="operator">=</span> parentWidget<span class="operator">-</span><span class="operator">&gt;</span>findChild<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="string">"button1"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>FindDirectChildrenOnly);
    ///
    /// </pre>
    /// <p>This example returns a <code>QListWidget</code> child of <code>parentWidget</code>, its direct parent:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlistwidget.html">QListWidget</a></span> <span class="operator">*</span>list <span class="operator">=</span> parentWidget<span class="operator">-</span><span class="operator">&gt;</span>findChild<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qlistwidget.html">QListWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>()<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>FindDirectChildrenOnly);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p></div>
    pub unsafe fn find_child_q_object_0a(&self) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChild_QObject_ptr2(
            self as *const crate::QObject,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*> QObject::findChildren<QAbstractAnimation*>(const QString& aName = …, QFlags<Qt::FindChildOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>The following example shows how to find a list of child <code>QWidget</code>s of the specified <code>parentWidget</code> named <code>widgetname</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> widgets <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="string">"widgetname"</span>);
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> allPButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>();
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are immediate children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> childButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>()<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>FindDirectChildrenOnly);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#findChild">findChild</a>().</p></div>
    pub unsafe fn find_children_q_abstract_animation_q_string_q_flags_find_child_option(
        &self,
        a_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        options: crate::QFlags<crate::FindChildOption>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QAbstractAnimation_ptr(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a_name)
                .as_raw_ptr(),
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*> QObject::findChildren<QAbstractAnimation*>(const QRegExp& re, QFlags<Qt::FindChildOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    /// <p>Returns the children of this object that can be cast to type T and that have names matching the regular expression <i>regExp</i>, or an empty list if there are no such objects. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p></div>
    pub unsafe fn find_children_q_abstract_animation_q_reg_exp_q_flags_find_child_option(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
        options: crate::QFlags<crate::FindChildOption>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QAbstractAnimation_ptr1(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(re).as_raw_ptr(),
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*> QObject::findChildren<QAbstractAnimation*>(const QRegularExpression& re, QFlags<Qt::FindChildOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    /// <p>Returns the children of this object that can be cast to type T and that have names matching the regular expression <i>re</i>, or an empty list if there are no such objects. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn find_children_q_abstract_animation_q_regular_expression_q_flags_find_child_option(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
        options: crate::QFlags<crate::FindChildOption>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QAbstractAnimation_ptr2(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*> QObject::findChildren<QAbstractAnimation*>(const QString& aName = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>The following example shows how to find a list of child <code>QWidget</code>s of the specified <code>parentWidget</code> named <code>widgetname</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> widgets <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="string">"widgetname"</span>);
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> allPButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>();
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are immediate children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> childButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>()<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>FindDirectChildrenOnly);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#findChild">findChild</a>().</p></div>
    pub unsafe fn find_children_q_abstract_animation_q_string(
        &self,
        a_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QAbstractAnimation_ptr3(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*> QObject::findChildren<QAbstractAnimation*>() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>The following example shows how to find a list of child <code>QWidget</code>s of the specified <code>parentWidget</code> named <code>widgetname</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> widgets <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="string">"widgetname"</span>);
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> allPButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>();
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are immediate children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> childButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>()<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>FindDirectChildrenOnly);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#findChild">findChild</a>().</p></div>
    pub unsafe fn find_children_q_abstract_animation(
        &self,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QAbstractAnimation_ptr4(
            self as *const crate::QObject,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*> QObject::findChildren<QAbstractAnimation*>(const QRegExp& re) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    /// <p>Returns the children of this object that can be cast to type T and that have names matching the regular expression <i>regExp</i>, or an empty list if there are no such objects. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p></div>
    pub unsafe fn find_children_q_abstract_animation_q_reg_exp(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QAbstractAnimation_ptr5(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(re).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*> QObject::findChildren<QAbstractAnimation*>(const QRegularExpression& re) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    /// <p>Returns the children of this object that can be cast to type T and that have names matching the regular expression <i>re</i>, or an empty list if there are no such objects. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn find_children_q_abstract_animation_q_regular_expression(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QAbstractAnimation_ptr6(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*> QObject::findChildren<QAbstractState*>(const QString& aName = …, QFlags<Qt::FindChildOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>The following example shows how to find a list of child <code>QWidget</code>s of the specified <code>parentWidget</code> named <code>widgetname</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> widgets <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="string">"widgetname"</span>);
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> allPButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>();
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are immediate children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> childButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>()<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>FindDirectChildrenOnly);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#findChild">findChild</a>().</p></div>
    pub unsafe fn find_children_q_abstract_state_q_string_q_flags_find_child_option(
        &self,
        a_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        options: crate::QFlags<crate::FindChildOption>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QAbstractState_ptr(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a_name)
                .as_raw_ptr(),
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*> QObject::findChildren<QAbstractState*>(const QRegExp& re, QFlags<Qt::FindChildOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    /// <p>Returns the children of this object that can be cast to type T and that have names matching the regular expression <i>regExp</i>, or an empty list if there are no such objects. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p></div>
    pub unsafe fn find_children_q_abstract_state_q_reg_exp_q_flags_find_child_option(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
        options: crate::QFlags<crate::FindChildOption>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QAbstractState_ptr1(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(re).as_raw_ptr(),
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*> QObject::findChildren<QAbstractState*>(const QRegularExpression& re, QFlags<Qt::FindChildOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    /// <p>Returns the children of this object that can be cast to type T and that have names matching the regular expression <i>re</i>, or an empty list if there are no such objects. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn find_children_q_abstract_state_q_regular_expression_q_flags_find_child_option(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
        options: crate::QFlags<crate::FindChildOption>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QAbstractState_ptr2(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*> QObject::findChildren<QAbstractState*>(const QString& aName = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>The following example shows how to find a list of child <code>QWidget</code>s of the specified <code>parentWidget</code> named <code>widgetname</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> widgets <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="string">"widgetname"</span>);
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> allPButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>();
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are immediate children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> childButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>()<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>FindDirectChildrenOnly);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#findChild">findChild</a>().</p></div>
    pub unsafe fn find_children_q_abstract_state_q_string(
        &self,
        a_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QAbstractState_ptr3(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*> QObject::findChildren<QAbstractState*>() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>The following example shows how to find a list of child <code>QWidget</code>s of the specified <code>parentWidget</code> named <code>widgetname</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> widgets <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="string">"widgetname"</span>);
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> allPButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>();
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are immediate children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> childButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>()<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>FindDirectChildrenOnly);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#findChild">findChild</a>().</p></div>
    pub unsafe fn find_children_q_abstract_state(
        &self,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QAbstractState_ptr4(
            self as *const crate::QObject,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*> QObject::findChildren<QAbstractState*>(const QRegExp& re) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    /// <p>Returns the children of this object that can be cast to type T and that have names matching the regular expression <i>regExp</i>, or an empty list if there are no such objects. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p></div>
    pub unsafe fn find_children_q_abstract_state_q_reg_exp(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QAbstractState_ptr5(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(re).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*> QObject::findChildren<QAbstractState*>(const QRegularExpression& re) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    /// <p>Returns the children of this object that can be cast to type T and that have names matching the regular expression <i>re</i>, or an empty list if there are no such objects. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn find_children_q_abstract_state_q_regular_expression(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QAbstractState_ptr6(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*> QObject::findChildren<QAbstractTransition*>(const QString& aName = …, QFlags<Qt::FindChildOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>The following example shows how to find a list of child <code>QWidget</code>s of the specified <code>parentWidget</code> named <code>widgetname</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> widgets <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="string">"widgetname"</span>);
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> allPButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>();
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are immediate children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> childButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>()<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>FindDirectChildrenOnly);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#findChild">findChild</a>().</p></div>
    pub unsafe fn find_children_q_abstract_transition_q_string_q_flags_find_child_option(
        &self,
        a_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        options: crate::QFlags<crate::FindChildOption>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QAbstractTransition_ptr(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a_name)
                .as_raw_ptr(),
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*> QObject::findChildren<QAbstractTransition*>(const QRegExp& re, QFlags<Qt::FindChildOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    /// <p>Returns the children of this object that can be cast to type T and that have names matching the regular expression <i>regExp</i>, or an empty list if there are no such objects. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p></div>
    pub unsafe fn find_children_q_abstract_transition_q_reg_exp_q_flags_find_child_option(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
        options: crate::QFlags<crate::FindChildOption>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractTransition> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QAbstractTransition_ptr1(
                self as *const crate::QObject,
                ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(re)
                    .as_raw_ptr(),
                options.to_int(),
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*> QObject::findChildren<QAbstractTransition*>(const QRegularExpression& re, QFlags<Qt::FindChildOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    /// <p>Returns the children of this object that can be cast to type T and that have names matching the regular expression <i>re</i>, or an empty list if there are no such objects. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn find_children_q_abstract_transition_q_regular_expression_q_flags_find_child_option(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
        options: crate::QFlags<crate::FindChildOption>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractTransition> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QAbstractTransition_ptr2(
                self as *const crate::QObject,
                ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                    .as_raw_ptr(),
                options.to_int(),
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*> QObject::findChildren<QAbstractTransition*>(const QString& aName = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>The following example shows how to find a list of child <code>QWidget</code>s of the specified <code>parentWidget</code> named <code>widgetname</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> widgets <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="string">"widgetname"</span>);
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> allPButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>();
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are immediate children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> childButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>()<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>FindDirectChildrenOnly);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#findChild">findChild</a>().</p></div>
    pub unsafe fn find_children_q_abstract_transition_q_string(
        &self,
        a_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractTransition> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QAbstractTransition_ptr3(
                self as *const crate::QObject,
                ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a_name)
                    .as_raw_ptr(),
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*> QObject::findChildren<QAbstractTransition*>() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>The following example shows how to find a list of child <code>QWidget</code>s of the specified <code>parentWidget</code> named <code>widgetname</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> widgets <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="string">"widgetname"</span>);
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> allPButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>();
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are immediate children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> childButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>()<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>FindDirectChildrenOnly);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#findChild">findChild</a>().</p></div>
    pub unsafe fn find_children_q_abstract_transition(
        &self,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractTransition> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QAbstractTransition_ptr4(
                self as *const crate::QObject,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*> QObject::findChildren<QAbstractTransition*>(const QRegExp& re) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    /// <p>Returns the children of this object that can be cast to type T and that have names matching the regular expression <i>regExp</i>, or an empty list if there are no such objects. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p></div>
    pub unsafe fn find_children_q_abstract_transition_q_reg_exp(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractTransition> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QAbstractTransition_ptr5(
                self as *const crate::QObject,
                ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(re)
                    .as_raw_ptr(),
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*> QObject::findChildren<QAbstractTransition*>(const QRegularExpression& re) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    /// <p>Returns the children of this object that can be cast to type T and that have names matching the regular expression <i>re</i>, or an empty list if there are no such objects. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn find_children_q_abstract_transition_q_regular_expression(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractTransition> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QAbstractTransition_ptr6(
                self as *const crate::QObject,
                ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                    .as_raw_ptr(),
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale> QObject::findChildren<QLocale>(const QString& aName = …, QFlags<Qt::FindChildOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>The following example shows how to find a list of child <code>QWidget</code>s of the specified <code>parentWidget</code> named <code>widgetname</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> widgets <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="string">"widgetname"</span>);
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> allPButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>();
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are immediate children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> childButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>()<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>FindDirectChildrenOnly);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#findChild">findChild</a>().</p></div>
    pub unsafe fn find_children_q_locale_q_string_q_flags_find_child_option(
        &self,
        a_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        options: crate::QFlags<crate::FindChildOption>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QLocale(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a_name)
                .as_raw_ptr(),
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale> QObject::findChildren<QLocale>(const QRegExp& re, QFlags<Qt::FindChildOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    /// <p>Returns the children of this object that can be cast to type T and that have names matching the regular expression <i>regExp</i>, or an empty list if there are no such objects. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p></div>
    pub unsafe fn find_children_q_locale_q_reg_exp_q_flags_find_child_option(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
        options: crate::QFlags<crate::FindChildOption>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QLocale1(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(re).as_raw_ptr(),
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale> QObject::findChildren<QLocale>(const QRegularExpression& re, QFlags<Qt::FindChildOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    /// <p>Returns the children of this object that can be cast to type T and that have names matching the regular expression <i>re</i>, or an empty list if there are no such objects. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn find_children_q_locale_q_regular_expression_q_flags_find_child_option(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
        options: crate::QFlags<crate::FindChildOption>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QLocale2(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale> QObject::findChildren<QLocale>(const QString& aName = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>The following example shows how to find a list of child <code>QWidget</code>s of the specified <code>parentWidget</code> named <code>widgetname</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> widgets <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="string">"widgetname"</span>);
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> allPButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>();
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are immediate children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> childButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>()<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>FindDirectChildrenOnly);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#findChild">findChild</a>().</p></div>
    pub unsafe fn find_children_q_locale_q_string(
        &self,
        a_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QLocale3(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale> QObject::findChildren<QLocale>() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>The following example shows how to find a list of child <code>QWidget</code>s of the specified <code>parentWidget</code> named <code>widgetname</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> widgets <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="string">"widgetname"</span>);
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> allPButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>();
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are immediate children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> childButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>()<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>FindDirectChildrenOnly);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#findChild">findChild</a>().</p></div>
    pub unsafe fn find_children_q_locale(&self) -> ::cpp_utils::CppBox<crate::QListOfQLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QLocale4(
            self as *const crate::QObject,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale> QObject::findChildren<QLocale>(const QRegExp& re) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    /// <p>Returns the children of this object that can be cast to type T and that have names matching the regular expression <i>regExp</i>, or an empty list if there are no such objects. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p></div>
    pub unsafe fn find_children_q_locale_q_reg_exp(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QLocale5(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(re).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale> QObject::findChildren<QLocale>(const QRegularExpression& re) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    /// <p>Returns the children of this object that can be cast to type T and that have names matching the regular expression <i>re</i>, or an empty list if there are no such objects. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn find_children_q_locale_q_regular_expression(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QLocale6(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType> QObject::findChildren<QMimeType>(const QString& aName = …, QFlags<Qt::FindChildOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>The following example shows how to find a list of child <code>QWidget</code>s of the specified <code>parentWidget</code> named <code>widgetname</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> widgets <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="string">"widgetname"</span>);
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> allPButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>();
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are immediate children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> childButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>()<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>FindDirectChildrenOnly);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#findChild">findChild</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn find_children_q_mime_type_q_string_q_flags_find_child_option(
        &self,
        a_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        options: crate::QFlags<crate::FindChildOption>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QMimeType(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a_name)
                .as_raw_ptr(),
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType> QObject::findChildren<QMimeType>(const QRegExp& re, QFlags<Qt::FindChildOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    /// <p>Returns the children of this object that can be cast to type T and that have names matching the regular expression <i>regExp</i>, or an empty list if there are no such objects. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn find_children_q_mime_type_q_reg_exp_q_flags_find_child_option(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
        options: crate::QFlags<crate::FindChildOption>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QMimeType1(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(re).as_raw_ptr(),
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType> QObject::findChildren<QMimeType>(const QRegularExpression& re, QFlags<Qt::FindChildOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    /// <p>Returns the children of this object that can be cast to type T and that have names matching the regular expression <i>re</i>, or an empty list if there are no such objects. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn find_children_q_mime_type_q_regular_expression_q_flags_find_child_option(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
        options: crate::QFlags<crate::FindChildOption>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QMimeType2(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType> QObject::findChildren<QMimeType>(const QString& aName = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>The following example shows how to find a list of child <code>QWidget</code>s of the specified <code>parentWidget</code> named <code>widgetname</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> widgets <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="string">"widgetname"</span>);
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> allPButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>();
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are immediate children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> childButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>()<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>FindDirectChildrenOnly);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#findChild">findChild</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn find_children_q_mime_type_q_string(
        &self,
        a_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QMimeType3(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType> QObject::findChildren<QMimeType>() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>The following example shows how to find a list of child <code>QWidget</code>s of the specified <code>parentWidget</code> named <code>widgetname</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> widgets <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="string">"widgetname"</span>);
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> allPButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>();
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are immediate children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> childButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>()<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>FindDirectChildrenOnly);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#findChild">findChild</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn find_children_q_mime_type(&self) -> ::cpp_utils::CppBox<crate::QListOfQMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QMimeType4(
            self as *const crate::QObject,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType> QObject::findChildren<QMimeType>(const QRegExp& re) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    /// <p>Returns the children of this object that can be cast to type T and that have names matching the regular expression <i>regExp</i>, or an empty list if there are no such objects. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn find_children_q_mime_type_q_reg_exp(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QMimeType5(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(re).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType> QObject::findChildren<QMimeType>(const QRegularExpression& re) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    /// <p>Returns the children of this object that can be cast to type T and that have names matching the regular expression <i>re</i>, or an empty list if there are no such objects. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn find_children_q_mime_type_q_regular_expression(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QMimeType6(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QObject*> QObject::findChildren<QObject*>(const QString& aName = …, QFlags<Qt::FindChildOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>The following example shows how to find a list of child <code>QWidget</code>s of the specified <code>parentWidget</code> named <code>widgetname</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> widgets <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="string">"widgetname"</span>);
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> allPButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>();
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are immediate children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> childButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>()<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>FindDirectChildrenOnly);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#findChild">findChild</a>().</p></div>
    pub unsafe fn find_children_q_object_q_string_q_flags_find_child_option(
        &self,
        a_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        options: crate::QFlags<crate::FindChildOption>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QObject_ptr(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a_name)
                .as_raw_ptr(),
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QObject*> QObject::findChildren<QObject*>(const QRegExp& re, QFlags<Qt::FindChildOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    /// <p>Returns the children of this object that can be cast to type T and that have names matching the regular expression <i>regExp</i>, or an empty list if there are no such objects. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p></div>
    pub unsafe fn find_children_q_object_q_reg_exp_q_flags_find_child_option(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
        options: crate::QFlags<crate::FindChildOption>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QObject_ptr1(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(re).as_raw_ptr(),
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QObject*> QObject::findChildren<QObject*>(const QRegularExpression& re, QFlags<Qt::FindChildOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    /// <p>Returns the children of this object that can be cast to type T and that have names matching the regular expression <i>re</i>, or an empty list if there are no such objects. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn find_children_q_object_q_regular_expression_q_flags_find_child_option(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
        options: crate::QFlags<crate::FindChildOption>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QObject_ptr2(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QObject*> QObject::findChildren<QObject*>(const QString& aName = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>The following example shows how to find a list of child <code>QWidget</code>s of the specified <code>parentWidget</code> named <code>widgetname</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> widgets <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="string">"widgetname"</span>);
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> allPButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>();
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are immediate children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> childButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>()<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>FindDirectChildrenOnly);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#findChild">findChild</a>().</p></div>
    pub unsafe fn find_children_q_object_q_string(
        &self,
        a_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QObject_ptr3(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QObject*> QObject::findChildren<QObject*>() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns all children of this object with the given <i>name</i> that can be cast to type T, or an empty list if there are no such objects. Omitting the <i>name</i> argument causes all object names to be matched. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>The following example shows how to find a list of child <code>QWidget</code>s of the specified <code>parentWidget</code> named <code>widgetname</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> widgets <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="string">"widgetname"</span>);
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> allPButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>();
    ///
    /// </pre>
    /// <p>This example returns all <code>QPushButton</code>s that are immediate children of <code>parentWidget</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span> childButtons <span class="operator">=</span> parentWidget<span class="operator">.</span>findChildren<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>()<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>FindDirectChildrenOnly);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#findChild">findChild</a>().</p></div>
    pub unsafe fn find_children_q_object(&self) -> ::cpp_utils::CppBox<crate::QListOfQObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QObject_ptr4(
            self as *const crate::QObject,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QObject*> QObject::findChildren<QObject*>(const QRegExp& re) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    /// <p>Returns the children of this object that can be cast to type T and that have names matching the regular expression <i>regExp</i>, or an empty list if there are no such objects. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p></div>
    pub unsafe fn find_children_q_object_q_reg_exp(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QObject_ptr5(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(re).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QObject*> QObject::findChildren<QObject*>(const QRegularExpression& re) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#findChildren-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    /// <p>Returns the children of this object that can be cast to type T and that have names matching the regular expression <i>re</i>, or an empty list if there are no such objects. The search is performed recursively, unless <i>options</i> specifies the option FindDirectChildrenOnly.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn find_children_q_object_q_regular_expression(
        &self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_findChildren_QObject_ptr6(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if this object is an instance of a class that inherits <i>className</i> or a <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> subclass that inherits <i>className</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QObject::inherits(const char* classname) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#inherits">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this object is an instance of a class that inherits <i>className</i> or a <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> subclass that inherits <i>className</i>; otherwise returns <code>false</code>.</p>
    /// <p>A class is considered to inherit itself.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a></span> <span class="operator">*</span>timer <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a></span>;         <span class="comment">// QTimer inherits QObject</span>
    ///   timer<span class="operator">-</span><span class="operator">&gt;</span>inherits(<span class="string">"QTimer"</span>);          <span class="comment">// returns true</span>
    ///   timer<span class="operator">-</span><span class="operator">&gt;</span>inherits(<span class="string">"QObject"</span>);         <span class="comment">// returns true</span>
    ///   timer<span class="operator">-</span><span class="operator">&gt;</span>inherits(<span class="string">"QAbstractButton"</span>); <span class="comment">// returns false</span>
    ///
    ///   <span class="comment">// QVBoxLayout inherits QObject and QLayoutItem</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvboxlayout.html">QVBoxLayout</a></span> <span class="operator">*</span>layout <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvboxlayout.html">QVBoxLayout</a></span>;
    ///   layout<span class="operator">-</span><span class="operator">&gt;</span>inherits(<span class="string">"QObject"</span>);        <span class="comment">// returns true</span>
    ///   layout<span class="operator">-</span><span class="operator">&gt;</span>inherits(<span class="string">"QLayoutItem"</span>);    <span class="comment">// returns true (even though QLayoutItem is not a QObject)</span>
    ///
    /// </pre>
    /// <p>If you need to determine whether an object is an instance of a particular class for the purpose of casting it, consider using <a href="http://doc.qt.io/qt-5/qobject.html#qobject_cast">qobject_cast</a>&lt;Type *&gt;(object) instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#metaObject">metaObject</a>() and <a href="http://doc.qt.io/qt-5/qobject.html#qobject_cast">qobject_cast</a>().</p></div>
    pub unsafe fn inherits(
        &self,
        classname: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QObject_inherits(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(classname)
                .as_raw_ptr(),
        )
    }

    /// <p>Installs an event filter <i>filterObj</i> on this object. For example:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QObject::installEventFilter(QObject* filterObj)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#installEventFilter">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Installs an event filter <i>filterObj</i> on this object. For example:</p>
    /// <pre class="cpp">
    ///
    ///   monitoredObj<span class="operator">-</span><span class="operator">&gt;</span>installEventFilter(filterObj);
    ///
    /// </pre>
    /// <p>An event filter is an object that receives all events that are sent to this object. The filter can either stop the event or forward it to this object. The event filter <i>filterObj</i> receives events via its <a href="http://doc.qt.io/qt-5/qobject.html#eventFilter">eventFilter</a>() function. The <a href="http://doc.qt.io/qt-5/qobject.html#eventFilter">eventFilter</a>() function must return true if the event should be filtered, (i.e. stopped); otherwise it must return false.</p>
    /// <p>If multiple event filters are installed on a single object, the filter that was installed last is activated first.</p>
    /// <p>Here's a <code>KeyPressEater</code> class that eats the key presses of its monitored objects:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">class</span> KeyPressEater : <span class="keyword">public</span> <span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span>
    ///   {
    /// &#32;     Q_OBJECT
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///
    ///   <span class="keyword">protected</span>:
    /// &#32;     bool eventFilter(<span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span> <span class="operator">*</span>obj<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a></span> <span class="operator">*</span>event);
    ///   };
    ///
    ///   bool KeyPressEater<span class="operator">::</span>eventFilter(<span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span> <span class="operator">*</span>obj<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a></span> <span class="operator">*</span>event)
    ///   {
    /// &#32;     <span class="keyword">if</span> (event<span class="operator">-</span><span class="operator">&gt;</span>type() <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a></span><span class="operator">::</span>KeyPress) {
    /// &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qkeyevent.html">QKeyEvent</a></span> <span class="operator">*</span>keyEvent <span class="operator">=</span> <span class="keyword">static_cast</span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qkeyevent.html">QKeyEvent</a></span> <span class="operator">*</span><span class="operator">&gt;</span>(event);
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>(<span class="string">"Ate key press %d"</span><span class="operator">,</span> keyEvent<span class="operator">-</span><span class="operator">&gt;</span>key());
    /// &#32;   &#32;     <span class="keyword">return</span> <span class="keyword">true</span>;
    /// &#32;     } <span class="keyword">else</span> {
    /// &#32;   &#32;     <span class="comment">// standard event processing</span>
    /// &#32;   &#32;     <span class="keyword">return</span> <span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span><span class="operator">::</span>eventFilter(obj<span class="operator">,</span> event);
    /// &#32;     }
    ///   }
    ///
    /// </pre>
    /// <p>And here's how to install it on two widgets:</p>
    /// <pre class="cpp">
    ///
    ///   KeyPressEater <span class="operator">*</span>keyPressEater <span class="operator">=</span> <span class="keyword">new</span> KeyPressEater(<span class="keyword">this</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span>pushButton <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span>(<span class="keyword">this</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlistview.html">QListView</a></span> <span class="operator">*</span>listView <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qlistview.html">QListView</a></span>(<span class="keyword">this</span>);
    ///
    ///   pushButton<span class="operator">-</span><span class="operator">&gt;</span>installEventFilter(keyPressEater);
    ///   listView<span class="operator">-</span><span class="operator">&gt;</span>installEventFilter(keyPressEater);
    ///
    /// </pre>
    /// <p>The <a href="http://doc.qt.io/qt-5/qshortcut.html">QShortcut</a> class, for example, uses this technique to intercept shortcut key presses.</p>
    /// <p><b>Warning:</b> If you delete the receiver object in your <a href="http://doc.qt.io/qt-5/qobject.html#eventFilter">eventFilter</a>() function, be sure to return true. If you return false, Qt sends the event to the deleted object and the program will crash.</p>
    /// <p>Note that the filtering object must be in the same thread as this object. If <i>filterObj</i> is in a different thread, this function does nothing. If either <i>filterObj</i> or this object are moved to a different thread after calling this function, the event filter will not be called until both objects have the same thread affinity again (it is <i>not</i> removed).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#removeEventFilter">removeEventFilter</a>(), <a href="http://doc.qt.io/qt-5/qobject.html#eventFilter">eventFilter</a>(), and <a href="http://doc.qt.io/qt-5/qobject.html#event">event</a>().</p></div>
    pub unsafe fn install_event_filter(
        &mut self,
        filter_obj: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QObject_installEventFilter(
            self as *mut crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(filter_obj)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if the object is a widget; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QObject::isWidgetType() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#isWidgetType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the object is a widget; otherwise returns <code>false</code>.</p>
    /// <p>Calling this function is equivalent to calling <code>inherits("QWidget")</code>, except that it is much faster.</p></div>
    pub unsafe fn is_widget_type(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QObject_isWidgetType(self as *const crate::QObject)
    }

    /// <p>Returns <code>true</code> if the object is a window; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QObject::isWindowType() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#isWindowType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the object is a window; otherwise returns <code>false</code>.</p>
    /// <p>Calling this function is equivalent to calling <code>inherits("QWindow")</code>, except that it is much faster.</p></div>
    pub unsafe fn is_window_type(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QObject_isWindowType(self as *const crate::QObject)
    }

    /// <p>Kills the timer with timer identifier, <i>id</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QObject::killTimer(int id)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#killTimer">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Kills the timer with timer identifier, <i>id</i>.</p>
    /// <p>The timer identifier is returned by <a href="http://doc.qt.io/qt-5/qobject.html#startTimer">startTimer</a>() when a timer event is started.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#timerEvent">timerEvent</a>() and <a href="http://doc.qt.io/qt-5/qobject.html#startTimer">startTimer</a>().</p></div>
    pub unsafe fn kill_timer(&mut self, id: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QObject_killTimer(self as *mut crate::QObject, id)
    }

    /// <p>Returns a pointer to the meta-object of this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QObject::metaObject() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#metaObject">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the meta-object of this object.</p>
    /// <p>A meta-object contains information about a class that inherits <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>, e.g. class name, superclass name, properties, signals and slots. Every <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> subclass that contains the <a href="http://doc.qt.io/qt-5/qobject.html#Q_OBJECT">Q_OBJECT</a> macro will have a meta-object.</p>
    /// <p>The meta-object information is required by the signal/slot connection mechanism and the property system. The <a href="http://doc.qt.io/qt-5/qobject.html#inherits">inherits</a>() function also makes use of the meta-object.</p>
    /// <p>If you have no pointer to an actual object instance but still want to access the meta-object of a class, you can use <a href="http://doc.qt.io/qt-5/qobject.html#staticMetaObject-var">staticMetaObject</a>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span> <span class="operator">*</span>obj <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span>;
    ///   obj<span class="operator">-</span><span class="operator">&gt;</span>metaObject()<span class="operator">-</span><span class="operator">&gt;</span>className();             <span class="comment">// returns "QPushButton"</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span><span class="operator">::</span>staticMetaObject<span class="operator">.</span>className();  <span class="comment">// returns "QPushButton"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#staticMetaObject-var">staticMetaObject</a>.</p></div>
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QObject_metaObject(self as *const crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Changes the thread affinity for this object and its children. The object cannot be moved if it has a parent. Event processing will continue in the <i>targetThread</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QObject::moveToThread(QThread* thread)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#moveToThread">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Changes the thread affinity for this object and its children. The object cannot be moved if it has a parent. Event processing will continue in the <i>targetThread</i>.</p>
    /// <p>To move an object to the main thread, use QApplication::instance() to retrieve a pointer to the current application, and then use QApplication::thread() to retrieve the thread in which the application lives. For example:</p>
    /// <pre class="cpp">
    ///
    ///   myObject<span class="operator">-</span><span class="operator">&gt;</span>moveToThread(<span class="type"><a href="http://doc.qt.io/qt-5/qapplication.html">QApplication</a></span><span class="operator">::</span>instance()<span class="operator">-</span><span class="operator">&gt;</span>thread());
    ///
    /// </pre>
    /// <p>If <i>targetThread</i> is zero, all event processing for this object and its children stops.</p>
    /// <p>Note that all active timers for the object will be reset. The timers are first stopped in the current thread and restarted (with the same interval) in the <i>targetThread</i>. As a result, constantly moving an object between threads can postpone timer events indefinitely.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::ThreadChange</a> event is sent to this object just before the thread affinity is changed. You can handle this event to perform any special processing. Note that any new events that are posted to this object will be handled in the <i>targetThread</i>.</p>
    /// <p><b>Warning:</b> This function is <i>not</i> thread-safe; the current thread must be same as the current thread affinity. In other words, this function can only "push" an object from the current thread to another thread, it cannot "pull" an object from any arbitrary thread to the current thread.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#thread">thread</a>().</p></div>
    pub unsafe fn move_to_thread(
        &mut self,
        thread: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QThread>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QObject_moveToThread(
            self as *mut crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QThread>>::cast_into(thread)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Constructs an object with parent object <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QObject::QObject(QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#QObject">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an object with parent object <i>parent</i>.</p>
    /// <p>The parent of an object may be viewed as the object's owner. For instance, a <a href="http://doc.qt.io/qt-5/qdialog.html">dialog box</a> is the parent of the <b>OK</b> and <b>Cancel</b> buttons it contains.</p>
    /// <p>The destructor of a parent object destroys all child objects.</p>
    /// <p>Setting <i>parent</i> to 0 constructs an object with no parent. If the object is a widget, it will become a top-level window.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#parent">parent</a>(), <a href="http://doc.qt.io/qt-5/qobject.html#findChild">findChild</a>(), and <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p></div>
    pub unsafe fn new_1a(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_QObject(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> class is the base class of all Qt objects.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QObject::QObject()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> class is the base class of all Qt objects.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> is the heart of the Qt <a href="http://doc.qt.io/qt-5/object.html">Object Model</a>. The central feature in this model is a very powerful mechanism for seamless object communication called <a href="http://doc.qt.io/qt-5/signalsandslots.html">signals and slots</a>. You can connect a signal to a slot with <a href="http://doc.qt.io/qt-5/qobject.html#connect">connect</a>() and destroy the connection with <a href="http://doc.qt.io/qt-5/qobject.html#disconnect">disconnect</a>(). To avoid never ending notification loops you can temporarily block signals with <a href="http://doc.qt.io/qt-5/qobject.html#blockSignals">blockSignals</a>(). The protected functions <a href="http://doc.qt.io/qt-5/qobject.html#connectNotify">connectNotify</a>() and <a href="http://doc.qt.io/qt-5/qobject.html#disconnectNotify">disconnectNotify</a>() make it possible to track connections.</p>
    /// <p>QObjects organize themselves in <a href="http://doc.qt.io/qt-5/objecttrees.html">object trees</a>. When you create a <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> with another object as parent, the object will automatically add itself to the parent's <a href="http://doc.qt.io/qt-5/qobject.html#children">children</a>() list. The parent takes ownership of the object; i.e., it will automatically delete its children in its destructor. You can look for an object by name and optionally type using <a href="http://doc.qt.io/qt-5/qobject.html#findChild">findChild</a>() or <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    /// <p>Every object has an <a href="http://doc.qt.io/qt-5/qobject.html#objectName-prop">objectName</a>() and its class name can be found via the corresponding <a href="http://doc.qt.io/qt-5/qobject.html#metaObject">metaObject</a>() (see <a href="http://doc.qt.io/qt-5/qmetaobject.html#className">QMetaObject::className</a>()). You can determine whether the object's class inherits another class in the <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> inheritance hierarchy by using the <a href="http://doc.qt.io/qt-5/qobject.html#inherits">inherits</a>() function.</p>
    /// <p>When an object is deleted, it emits a <a href="http://doc.qt.io/qt-5/qobject.html#destroyed">destroyed</a>() signal. You can catch this signal to avoid dangling references to QObjects.</p>
    /// <p>QObjects can receive events through <a href="http://doc.qt.io/qt-5/qobject.html#event">event</a>() and filter the events of other objects. See <a href="http://doc.qt.io/qt-5/qobject.html#installEventFilter">installEventFilter</a>() and <a href="http://doc.qt.io/qt-5/qobject.html#eventFilter">eventFilter</a>() for details. A convenience handler, <a href="http://doc.qt.io/qt-5/qobject.html#childEvent">childEvent</a>(), can be reimplemented to catch child events.</p>
    /// <p>Last but not least, <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> provides the basic timer support in Qt; see <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> for high-level support for timers.</p>
    /// <p>Notice that the <a href="http://doc.qt.io/qt-5/qobject.html#Q_OBJECT">Q_OBJECT</a> macro is mandatory for any object that implements signals, slots or properties. You also need to run the <a href="http://doc.qt.io/qt-5/moc.html">Meta Object Compiler</a> on the source file. We strongly recommend the use of this macro in all subclasses of <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> regardless of whether or not they actually use signals, slots and properties, since failure to do so may lead certain functions to exhibit strange behavior.</p>
    /// <p>All Qt widgets inherit <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>. The convenience function <a href="http://doc.qt.io/qt-5/qobject.html#isWidgetType">isWidgetType</a>() returns whether an object is actually a widget. It is much faster than <a href="http://doc.qt.io/qt-5/qobject.html#qobject_cast">qobject_cast</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;(<i>obj</i>) or <i>obj</i>-&gt;<a href="http://doc.qt.io/qt-5/qobject.html#inherits">inherits</a>("<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a>").</p>
    /// <p>Some <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> functions, e.g. <a href="http://doc.qt.io/qt-5/qobject.html#children">children</a>(), return a <a href="http://doc.qt.io/qt-5/qobject.html#QObjectList-typedef">QObjectList</a>. <a href="http://doc.qt.io/qt-5/qobject.html#QObjectList-typedef">QObjectList</a> is a typedef for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> *&gt;.</p>
    /// <a name="thread-affinity"></a></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_QObject1();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This property holds the name of this object</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QObject::objectName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#objectName-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the name of this object</p>
    /// <p>You can find an object by name (and type) using <a href="http://doc.qt.io/qt-5/qobject.html#findChild">findChild</a>(). You can find a set of objects with <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    /// <pre class="cpp">
    ///
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>(<span class="string">"MyClass::setPrecision(): (%s) invalid precision %f"</span><span class="operator">,</span>
    /// &#32;   &#32;    <a href="http://doc.qt.io/qt-5/qtglobal.html#qPrintable">qPrintable</a>(objectName())<span class="operator">,</span> newPrecision);
    ///
    /// </pre>
    /// <p>By default, this property contains an empty string.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QString </td><td class="memItemRight bottomAlign"><span class="name"><b>objectName</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setObjectName</b></span>(const QString &amp;<i>name</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qobject.html#objectNameChanged">objectNameChanged</a></b></span>(const QString &amp;<i>objectName</i>)</td><td class="memItemRight bottomAlign">[see note below]</td></tr>
    /// </tbody></table></div>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#metaObject">metaObject</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#className">QMetaObject::className</a>().</p></div>
    pub unsafe fn object_name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QObject_objectName(self as *const crate::QObject);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a pointer to the parent object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QObject* QObject::parent() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#parent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the parent object.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#setParent">setParent</a>() and <a href="http://doc.qt.io/qt-5/qobject.html#children">children</a>().</p></div>
    pub unsafe fn parent(&self) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QObject_parent(self as *const crate::QObject);
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns the value of the object's <i>name</i> property.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QObject::property(const char* name) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#property">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value of the object's <i>name</i> property.</p>
    /// <p>If no such property exists, the returned variant is invalid.</p>
    /// <p>Information about all available properties is provided through the <a href="http://doc.qt.io/qt-5/qobject.html#metaObject">metaObject</a>() and <a href="http://doc.qt.io/qt-5/qobject.html#dynamicPropertyNames">dynamicPropertyNames</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#setProperty">setProperty</a>(), <a href="http://doc.qt.io/qt-5/qvariant.html#isValid">QVariant::isValid</a>(), <a href="http://doc.qt.io/qt-5/qobject.html#metaObject">metaObject</a>(), and <a href="http://doc.qt.io/qt-5/qobject.html#dynamicPropertyNames">dynamicPropertyNames</a>().</p></div>
    pub unsafe fn property(
        &self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_property(
            self as *const crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QObject::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QObject_qt_metacall(
            self as *mut crate::QObject,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QObject::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_qt_metacast(
            self as *mut crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Removes an event filter object <i>obj</i> from this object. The request is ignored if such an event filter has not been installed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QObject::removeEventFilter(QObject* obj)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#removeEventFilter">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes an event filter object <i>obj</i> from this object. The request is ignored if such an event filter has not been installed.</p>
    /// <p>All event filters for this object are automatically removed when this object is destroyed.</p>
    /// <p>It is always safe to remove an event filter, even during event filter activation (i.e. from the <a href="http://doc.qt.io/qt-5/qobject.html#eventFilter">eventFilter</a>() function).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#installEventFilter">installEventFilter</a>(), <a href="http://doc.qt.io/qt-5/qobject.html#eventFilter">eventFilter</a>(), and <a href="http://doc.qt.io/qt-5/qobject.html#event">event</a>().</p></div>
    pub unsafe fn remove_event_filter(
        &mut self,
        obj: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QObject_removeEventFilter(
            self as *mut crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(obj)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>This property holds the name of this object</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QObject::setObjectName(const QString& name)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#objectName-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the name of this object</p>
    /// <p>You can find an object by name (and type) using <a href="http://doc.qt.io/qt-5/qobject.html#findChild">findChild</a>(). You can find a set of objects with <a href="http://doc.qt.io/qt-5/qobject.html#findChildren">findChildren</a>().</p>
    /// <pre class="cpp">
    ///
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>(<span class="string">"MyClass::setPrecision(): (%s) invalid precision %f"</span><span class="operator">,</span>
    /// &#32;   &#32;    <a href="http://doc.qt.io/qt-5/qtglobal.html#qPrintable">qPrintable</a>(objectName())<span class="operator">,</span> newPrecision);
    ///
    /// </pre>
    /// <p>By default, this property contains an empty string.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QString </td><td class="memItemRight bottomAlign"><span class="name"><b>objectName</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setObjectName</b></span>(const QString &amp;<i>name</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qobject.html#objectNameChanged">objectNameChanged</a></b></span>(const QString &amp;<i>objectName</i>)</td><td class="memItemRight bottomAlign">[see note below]</td></tr>
    /// </tbody></table></div>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#metaObject">metaObject</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#className">QMetaObject::className</a>().</p></div>
    pub unsafe fn set_object_name(
        &mut self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QObject_setObjectName(
            self as *mut crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
        )
    }

    /// <p>Makes the object a child of <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QObject::setParent(QObject* parent)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#setParent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the object a child of <i>parent</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#parent">parent</a>() and <a href="http://doc.qt.io/qt-5/qobject.html#children">children</a>().</p></div>
    pub unsafe fn set_parent(
        &mut self,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QObject_setParent(
            self as *mut crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Sets the value of the object's <i>name</i> property to <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QObject::setProperty(const char* name, const QVariant& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#setProperty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the value of the object's <i>name</i> property to <i>value</i>.</p>
    /// <p>If the property is defined in the class using <a href="http://doc.qt.io/qt-5/qobject.html#Q_PROPERTY">Q_PROPERTY</a> then true is returned on success and false otherwise. If the property is not defined using <a href="http://doc.qt.io/qt-5/qobject.html#Q_PROPERTY">Q_PROPERTY</a>, and therefore not listed in the meta-object, it is added as a dynamic property and false is returned.</p>
    /// <p>Information about all available properties is provided through the <a href="http://doc.qt.io/qt-5/qobject.html#metaObject">metaObject</a>() and <a href="http://doc.qt.io/qt-5/qobject.html#dynamicPropertyNames">dynamicPropertyNames</a>().</p>
    /// <p>Dynamic properties can be queried again using <a href="http://doc.qt.io/qt-5/qobject.html#property">property</a>() and can be removed by setting the property value to an invalid <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>. Changing the value of a dynamic property causes a <a href="http://doc.qt.io/qt-5/qdynamicpropertychangeevent.html">QDynamicPropertyChangeEvent</a> to be sent to the object.</p>
    /// <p><b>Note:</b> Dynamic properties starting with "_q_" are reserved for internal purposes.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#property">property</a>(), <a href="http://doc.qt.io/qt-5/qobject.html#metaObject">metaObject</a>(), <a href="http://doc.qt.io/qt-5/qobject.html#dynamicPropertyNames">dynamicPropertyNames</a>(), and <a href="http://doc.qt.io/qt-5/qmetaproperty.html#write">QMetaProperty::write</a>().</p></div>
    pub unsafe fn set_property(
        &mut self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QObject_setProperty(
            self as *mut crate::QObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(name)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if signals are blocked; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QObject::signalsBlocked() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#signalsBlocked">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if signals are blocked; otherwise returns <code>false</code>.</p>
    /// <p>Signals are not blocked by default.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#blockSignals">blockSignals</a>() and <a href="http://doc.qt.io/qt-5/qsignalblocker.html">QSignalBlocker</a>.</p></div>
    pub unsafe fn signals_blocked(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QObject_signalsBlocked(self as *const crate::QObject)
    }

    /// <p>Starts a timer and returns a timer identifier, or returns zero if it could not start a timer.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QObject::startTimer(int interval, Qt::TimerType timerType = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#startTimer">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Starts a timer and returns a timer identifier, or returns zero if it could not start a timer.</p>
    /// <p>A timer event will occur every <i>interval</i> milliseconds until <a href="http://doc.qt.io/qt-5/qobject.html#killTimer">killTimer</a>() is called. If <i>interval</i> is 0, then the timer event occurs once every time there are no more window system events to process.</p>
    /// <p>The virtual <a href="http://doc.qt.io/qt-5/qobject.html#timerEvent">timerEvent</a>() function is called with the <a href="http://doc.qt.io/qt-5/qtimerevent.html">QTimerEvent</a> event parameter class when a timer event occurs. Reimplement this function to get timer events.</p>
    /// <p>If multiple timers are running, the <a href="http://doc.qt.io/qt-5/qtimerevent.html#timerId">QTimerEvent::timerId</a>() can be used to find out which timer was activated.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">class</span> MyObject : <span class="keyword">public</span> <span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span>
    ///   {
    /// &#32;     Q_OBJECT
    ///
    ///   <span class="keyword">public</span>:
    /// &#32;     MyObject(<span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span> <span class="operator">*</span>parent <span class="operator">=</span> <span class="number">0</span>);
    ///
    ///   <span class="keyword">protected</span>:
    /// &#32;     <span class="type">void</span> timerEvent(<span class="type"><a href="http://doc.qt.io/qt-5/qtimerevent.html">QTimerEvent</a></span> <span class="operator">*</span>event);
    ///   };
    ///
    ///   MyObject<span class="operator">::</span>MyObject(<span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span> <span class="operator">*</span>parent)
    /// &#32;     : <span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span>(parent)
    ///   {
    /// &#32;     startTimer(<span class="number">50</span>);&#32;    <span class="comment">// 50-millisecond timer</span>
    /// &#32;     startTimer(<span class="number">1000</span>);   <span class="comment">// 1-second timer</span>
    /// &#32;     startTimer(<span class="number">60000</span>);  <span class="comment">// 1-minute timer</span>
    ///
    /// &#32;     <span class="keyword">using</span> <span class="keyword">namespace</span> std<span class="operator">::</span>chrono;
    /// &#32;     startTimer(milliseconds(<span class="number">50</span>));
    /// &#32;     startTimer(seconds(<span class="number">1</span>));
    /// &#32;     startTimer(minutes(<span class="number">1</span>));
    ///
    /// &#32;     <span class="comment">// since C++14 we can use std::chrono::duration literals, e.g.:</span>
    /// &#32;     startTimer(<span class="number">100ms</span>);
    /// &#32;     startTimer(<span class="number">5s</span>);
    /// &#32;     startTimer(<span class="number">2min</span>);
    /// &#32;     startTimer(<span class="number">1h</span>);
    ///   }
    ///
    ///   <span class="type">void</span> MyObject<span class="operator">::</span>timerEvent(<span class="type"><a href="http://doc.qt.io/qt-5/qtimerevent.html">QTimerEvent</a></span> <span class="operator">*</span>event)
    ///   {
    /// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Timer ID:"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> event<span class="operator">-</span><span class="operator">&gt;</span>timerId();
    ///   }
    ///
    /// </pre>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a>'s accuracy depends on the underlying operating system and hardware. The <i>timerType</i> argument allows you to customize the accuracy of the timer. See <a href="http://doc.qt.io/qt-5/qt.html#TimerType-enum">Qt::TimerType</a> for information on the different timer types. Most platforms support an accuracy of 20 milliseconds; some provide more. If Qt is unable to deliver the requested number of timer events, it will silently discard some.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> class provides a high-level programming interface with single-shot timers and timer signals instead of events. There is also a <a href="http://doc.qt.io/qt-5/qbasictimer.html">QBasicTimer</a> class that is more lightweight than <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> and less clumsy than using timer IDs directly.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#timerEvent">timerEvent</a>(), <a href="http://doc.qt.io/qt-5/qobject.html#killTimer">killTimer</a>(), and <a href="http://doc.qt.io/qt-5/qtimer.html#singleShot">QTimer::singleShot</a>().</p></div>
    pub unsafe fn start_timer_2a(
        &mut self,
        interval: ::std::os::raw::c_int,
        timer_type: crate::TimerType,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QObject_startTimer(
            self as *mut crate::QObject,
            interval,
            timer_type,
        )
    }

    /// <p>Starts a timer and returns a timer identifier, or returns zero if it could not start a timer.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QObject::startTimer(int interval)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#startTimer">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Starts a timer and returns a timer identifier, or returns zero if it could not start a timer.</p>
    /// <p>A timer event will occur every <i>interval</i> milliseconds until <a href="http://doc.qt.io/qt-5/qobject.html#killTimer">killTimer</a>() is called. If <i>interval</i> is 0, then the timer event occurs once every time there are no more window system events to process.</p>
    /// <p>The virtual <a href="http://doc.qt.io/qt-5/qobject.html#timerEvent">timerEvent</a>() function is called with the <a href="http://doc.qt.io/qt-5/qtimerevent.html">QTimerEvent</a> event parameter class when a timer event occurs. Reimplement this function to get timer events.</p>
    /// <p>If multiple timers are running, the <a href="http://doc.qt.io/qt-5/qtimerevent.html#timerId">QTimerEvent::timerId</a>() can be used to find out which timer was activated.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">class</span> MyObject : <span class="keyword">public</span> <span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span>
    ///   {
    /// &#32;     Q_OBJECT
    ///
    ///   <span class="keyword">public</span>:
    /// &#32;     MyObject(<span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span> <span class="operator">*</span>parent <span class="operator">=</span> <span class="number">0</span>);
    ///
    ///   <span class="keyword">protected</span>:
    /// &#32;     <span class="type">void</span> timerEvent(<span class="type"><a href="http://doc.qt.io/qt-5/qtimerevent.html">QTimerEvent</a></span> <span class="operator">*</span>event);
    ///   };
    ///
    ///   MyObject<span class="operator">::</span>MyObject(<span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span> <span class="operator">*</span>parent)
    /// &#32;     : <span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span>(parent)
    ///   {
    /// &#32;     startTimer(<span class="number">50</span>);&#32;    <span class="comment">// 50-millisecond timer</span>
    /// &#32;     startTimer(<span class="number">1000</span>);   <span class="comment">// 1-second timer</span>
    /// &#32;     startTimer(<span class="number">60000</span>);  <span class="comment">// 1-minute timer</span>
    ///
    /// &#32;     <span class="keyword">using</span> <span class="keyword">namespace</span> std<span class="operator">::</span>chrono;
    /// &#32;     startTimer(milliseconds(<span class="number">50</span>));
    /// &#32;     startTimer(seconds(<span class="number">1</span>));
    /// &#32;     startTimer(minutes(<span class="number">1</span>));
    ///
    /// &#32;     <span class="comment">// since C++14 we can use std::chrono::duration literals, e.g.:</span>
    /// &#32;     startTimer(<span class="number">100ms</span>);
    /// &#32;     startTimer(<span class="number">5s</span>);
    /// &#32;     startTimer(<span class="number">2min</span>);
    /// &#32;     startTimer(<span class="number">1h</span>);
    ///   }
    ///
    ///   <span class="type">void</span> MyObject<span class="operator">::</span>timerEvent(<span class="type"><a href="http://doc.qt.io/qt-5/qtimerevent.html">QTimerEvent</a></span> <span class="operator">*</span>event)
    ///   {
    /// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Timer ID:"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> event<span class="operator">-</span><span class="operator">&gt;</span>timerId();
    ///   }
    ///
    /// </pre>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a>'s accuracy depends on the underlying operating system and hardware. The <i>timerType</i> argument allows you to customize the accuracy of the timer. See <a href="http://doc.qt.io/qt-5/qt.html#TimerType-enum">Qt::TimerType</a> for information on the different timer types. Most platforms support an accuracy of 20 milliseconds; some provide more. If Qt is unable to deliver the requested number of timer events, it will silently discard some.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> class provides a high-level programming interface with single-shot timers and timer signals instead of events. There is also a <a href="http://doc.qt.io/qt-5/qbasictimer.html">QBasicTimer</a> class that is more lightweight than <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> and less clumsy than using timer IDs directly.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#timerEvent">timerEvent</a>(), <a href="http://doc.qt.io/qt-5/qobject.html#killTimer">killTimer</a>(), and <a href="http://doc.qt.io/qt-5/qtimer.html#singleShot">QTimer::singleShot</a>().</p></div>
    pub unsafe fn start_timer_1a(
        &mut self,
        interval: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QObject_startTimer1(self as *mut crate::QObject, interval)
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the thread in which the object lives.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QThread* QObject::thread() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#thread">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the thread in which the object lives.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#moveToThread">moveToThread</a>().</p></div>
    pub unsafe fn thread(&self) -> ::cpp_utils::MutPtr<crate::QThread> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QObject_thread(self as *const crate::QObject);
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns a translated version of <i>sourceText</i>, optionally based on a <i>disambiguation</i> string and value of <i>n</i> for strings containing plurals; otherwise returns <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">QString::fromUtf8</a>(<i>sourceText</i>) if no appropriate translated string is available.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QObject::tr(const char* s, const char* c, int n)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#tr">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a translated version of <i>sourceText</i>, optionally based on a <i>disambiguation</i> string and value of <i>n</i> for strings containing plurals; otherwise returns <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">QString::fromUtf8</a>(<i>sourceText</i>) if no appropriate translated string is available.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">void</span> MainWindow<span class="operator">::</span>createActions()
    ///   {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qmenu.html">QMenu</a></span> <span class="operator">*</span>fileMenu <span class="operator">=</span> menuBar()<span class="operator">-</span><span class="operator">&gt;</span>addMenu(tr(<span class="string">"&amp;File"</span>));
    /// &#32;     ...
    ///
    /// </pre>
    /// <p>If the same <i>sourceText</i> is used in different roles within the same context, an additional identifying string may be passed in <i>disambiguation</i> (0 by default). In Qt 4.4 and earlier, this was the preferred way to pass comments to translators.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   MyWindow<span class="operator">::</span>MyWindow()
    ///   {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qlabel.html">QLabel</a></span> <span class="operator">*</span>senderLabel <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qlabel.html">QLabel</a></span>(tr(<span class="string">"Name:"</span>));
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qlabel.html">QLabel</a></span> <span class="operator">*</span>recipientLabel <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qlabel.html">QLabel</a></span>(tr(<span class="string">"Name:"</span><span class="operator">,</span> <span class="string">"recipient"</span>));
    /// &#32;     ...
    ///
    /// </pre>
    /// <p>See <a href="http://doc.qt.io/qt-5/i18n-source-translation.html">Writing Source Code for Translation</a> for a detailed description of Qt's translation mechanisms in general, and the <a href="http://doc.qt.io/qt-5/i18n-source-translation.html#disambiguation">Disambiguation</a> section for information on disambiguation.</p>
    /// <p><b>Warning:</b> This method is reentrant only if all translators are installed <i>before</i> calling this method. Installing or removing translators while performing translations is not supported. Doing so will probably result in crashes or other undesirable behavior.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#translate">QCoreApplication::translate</a>() and <a href="http://doc.qt.io/qt-5/internationalization.html">Internationalization with Qt</a>.</p></div>
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a translated version of <i>sourceText</i>, or <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">QString::fromUtf8</a>(<i>sourceText</i>) if there is no appropriate version. It is otherwise identical to tr(<i>sourceText</i>, <i>disambiguation</i>, <i>n</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QObject::trUtf8(const char* s, const char* c, int n)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject-obsolete.html#trUtf8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a translated version of <i>sourceText</i>, or <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">QString::fromUtf8</a>(<i>sourceText</i>) if there is no appropriate version. It is otherwise identical to tr(<i>sourceText</i>, <i>disambiguation</i>, <i>n</i>).</p>
    /// <p><b>Warning:</b> This method is reentrant only if all translators are installed <i>before</i> calling this method. Installing or removing translators while performing translations is not supported. Doing so will probably result in crashes or other undesirable behavior.</p>
    /// <p><b>Warning:</b> For portability reasons, we recommend that you use escape sequences for specifying non-ASCII characters in string literals to trUtf8(). For example:</p>
    /// <pre class="cpp">
    ///
    ///   label<span class="operator">-</span><span class="operator">&gt;</span>setText(tr(<span class="string">"F\374r \310lise"</span>));
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#tr">tr</a>(), <a href="http://doc.qt.io/qt-5/qcoreapplication.html#translate">QCoreApplication::translate</a>(), and <a href="http://doc.qt.io/qt-5/internationalization.html">Internationalization with Qt</a>.</p>
    ///
    /// <h2>Related Non-Members</h2></div>
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObject_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>Exception-safe wrapper around <a href="http://doc.qt.io/qt-5/qobject.html#blockSignals">QObject::blockSignals</a>()</p>
///
/// C++ class: <span style='color: green;'>```QSignalBlocker```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qsignalblocker.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exception-safe wrapper around <a href="http://doc.qt.io/qt-5/qobject.html#blockSignals">QObject::blockSignals</a>()</p>
/// <p><a href="http://doc.qt.io/qt-5/qsignalblocker.html">QSignalBlocker</a> can be used whereever you would otherwise use a pair of calls to blockSignals(). It blocks signals in its constructor and in the destructor it resets the state to what it was before the constructor ran.</p>
/// <pre class="cpp">
///
///   {
///   <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsignalblocker.html#QSignalBlocker">QSignalBlocker</a></span> blocker(someQObject);
///   <span class="comment">// no signals here</span>
///   }
///
/// </pre>
/// <p>is thus equivalent to</p>
/// <pre class="cpp">
///
///   <span class="keyword">const</span> bool wasBlocked <span class="operator">=</span> someQObject<span class="operator">-</span><span class="operator">&gt;</span>blockSignals(<span class="keyword">true</span>);
///   <span class="comment">// no signals here</span>
///   someQObject<span class="operator">-</span><span class="operator">&gt;</span>blockSignals(wasBlocked);
///
/// </pre>
/// <p>except the code using <a href="http://doc.qt.io/qt-5/qsignalblocker.html">QSignalBlocker</a> is safe in the face of exceptions.</p></div>
#[repr(C)]
pub struct QSignalBlocker {
    _unused: u8,
}
impl QSignalBlocker {
    /// <p>Constructor. Calls <i>object</i>-&gt;blockSignals(true).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSignalBlocker::QSignalBlocker(QObject* o)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignalblocker.html#QSignalBlocker">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructor. Calls <i>object</i>-&gt;blockSignals(true).</p></div>
    pub unsafe fn from_q_object(
        o: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QSignalBlocker> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSignalBlocker_QSignalBlocker(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(o)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSignalBlocker::QSignalBlocker(QObject& o)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignalblocker.html#QSignalBlocker-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Calls <i>object</i>.blockSignals(true).</p></div>
    pub unsafe fn from_q_object2(
        o: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QSignalBlocker> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSignalBlocker_QSignalBlocker1(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QObject>>::cast_into(o)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Re-blocks signals after a previous <a href="http://doc.qt.io/qt-5/qsignalblocker.html#unblock">unblock</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSignalBlocker::reblock()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignalblocker.html#reblock">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Re-blocks signals after a previous <a href="http://doc.qt.io/qt-5/qsignalblocker.html#unblock">unblock</a>().</p>
    /// <p>The numbers of reblock() and <a href="http://doc.qt.io/qt-5/qsignalblocker.html#unblock">unblock</a>() calls are not counted, so every reblock() undoes any number of <a href="http://doc.qt.io/qt-5/qsignalblocker.html#unblock">unblock</a>() calls.</p></div>
    pub unsafe fn reblock(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSignalBlocker_reblock(self as *mut crate::QSignalBlocker)
    }

    /// <p>Temporarily restores the <a href="http://doc.qt.io/qt-5/qobject.html#signalsBlocked">QObject::signalsBlocked</a>() state to what it was before this QSignaBlocker's constructor ran. To undo, use <a href="http://doc.qt.io/qt-5/qsignalblocker.html#reblock">reblock</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSignalBlocker::unblock()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignalblocker.html#unblock">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Temporarily restores the <a href="http://doc.qt.io/qt-5/qobject.html#signalsBlocked">QObject::signalsBlocked</a>() state to what it was before this QSignaBlocker's constructor ran. To undo, use <a href="http://doc.qt.io/qt-5/qsignalblocker.html#reblock">reblock</a>().</p>
    /// <p>The numbers of <a href="http://doc.qt.io/qt-5/qsignalblocker.html#reblock">reblock</a>() and unblock() calls are not counted, so every unblock() undoes any number of <a href="http://doc.qt.io/qt-5/qsignalblocker.html#reblock">reblock</a>() calls.</p></div>
    pub unsafe fn unblock(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSignalBlocker_unblock(self as *mut crate::QSignalBlocker)
    }
}

pub mod q_abstract_animation {
    //! C++ type: <span style='color: green;'>```QAbstractAnimation```</span>

    /// <p>This enum describes the direction of the animation when in <a href="http://doc.qt.io/qt-5/qabstractanimation.html#State-enum">Running</a> state.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QAbstractAnimation::Direction```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#Direction-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the direction of the animation when in <a href="http://doc.qt.io/qt-5/qabstractanimation.html#State-enum">Running</a> state.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#direction-prop">direction</a>.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Direction(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Direction {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Direction(value)
        }
    }

    impl From<Direction> for ::std::os::raw::c_int {
        fn from(value: Direction) -> Self {
            value.0
        }
    }

    impl Direction {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Direction {
        /// The current time of the animation increases with time (i.e., moves from 0 and towards the end / duration). (C++ enum variant: <span style='color: green;'>```Forward = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const Forward: crate::q_abstract_animation::Direction =
            crate::q_abstract_animation::Direction(0);
        /// The current time of the animation decreases with time (i.e., moves from the end / duration and towards 0). (C++ enum variant: <span style='color: green;'>```Backward = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Backward: crate::q_abstract_animation::Direction =
            crate::q_abstract_animation::Direction(1);
    }

    /// <p>This enum describes the state of the animation.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QAbstractAnimation::State```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#State-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the state of the animation.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">state</a>() and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#stateChanged">stateChanged</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct State(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for State {
        fn from(value: ::std::os::raw::c_int) -> Self {
            State(value)
        }
    }

    impl From<State> for ::std::os::raw::c_int {
        fn from(value: State) -> Self {
            value.0
        }
    }

    impl State {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl State {
        /// The animation is not running. This is the initial state of <a href="http://doc.qt.io/qt-5/qabstractanimation.html">QAbstractAnimation</a>, and the state <a href="http://doc.qt.io/qt-5/qabstractanimation.html">QAbstractAnimation</a> reenters when finished. The current time remain unchanged until either <a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentTime-prop">setCurrentTime</a>() is called, or the animation is started by calling <a href="http://doc.qt.io/qt-5/qabstractanimation.html#start">start</a>(). (C++ enum variant: <span style='color: green;'>```Stopped = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const Stopped: crate::q_abstract_animation::State =
            crate::q_abstract_animation::State(0);
        /// The animation is paused (i.e., temporarily suspended). Calling <a href="http://doc.qt.io/qt-5/qabstractanimation.html#resume">resume</a>() will resume animation activity. (C++ enum variant: <span style='color: green;'>```Paused = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Paused: crate::q_abstract_animation::State =
            crate::q_abstract_animation::State(1);
        /// The animation is running. While control is in the event loop, <a href="http://doc.qt.io/qt-5/qabstractanimation.html">QAbstractAnimation</a> will update its current time at regular intervals, calling <a href="http://doc.qt.io/qt-5/qabstractanimation.html#updateCurrentTime">updateCurrentTime</a>() when appropriate. (C++ enum variant: <span style='color: green;'>```Running = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const Running: crate::q_abstract_animation::State =
            crate::q_abstract_animation::State(2);
    }

    /// C++ enum: <span style='color: green;'>```QAbstractAnimation::DeletionPolicy```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#DeletionPolicy-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct DeletionPolicy(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for DeletionPolicy {
        fn from(value: ::std::os::raw::c_int) -> Self {
            DeletionPolicy(value)
        }
    }

    impl From<DeletionPolicy> for ::std::os::raw::c_int {
        fn from(value: DeletionPolicy) -> Self {
            value.0
        }
    }

    impl DeletionPolicy {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl DeletionPolicy {
        /// The animation will not be deleted when stopped. (C++ enum variant: <span style='color: green;'>```KeepWhenStopped = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const KeepWhenStopped: crate::q_abstract_animation::DeletionPolicy =
            crate::q_abstract_animation::DeletionPolicy(0);
        /// The animation will be automatically deleted when stopped. (C++ enum variant: <span style='color: green;'>```DeleteWhenStopped = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const DeleteWhenStopped: crate::q_abstract_animation::DeletionPolicy =
            crate::q_abstract_animation::DeletionPolicy(1);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qabstractanimation.html">QAbstractAnimation</a> class is the base of all animations.</p>
///
/// C++ class: <span style='color: green;'>```QAbstractAnimation```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qabstractanimation.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qabstractanimation.html">QAbstractAnimation</a> class is the base of all animations.</p>
/// <p>The class defines the functions for the functionality shared by all animations. By inheriting this class, you can create custom animations that plug into the rest of the animation framework.</p>
/// <p>The progress of an animation is given by its current time (<a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentLoopTime">currentLoopTime</a>()), which is measured in milliseconds from the start of the animation (0) to its end (<a href="http://doc.qt.io/qt-5/qabstractanimation.html#duration">duration</a>()). The value is updated automatically while the animation is running. It can also be set directly with <a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentTime-prop">setCurrentTime</a>().</p>
/// <p>At any point an animation is in one of three states: <a href="http://doc.qt.io/qt-5/qabstractanimation.html#State-enum">Running</a>, <a href="http://doc.qt.io/qt-5/qabstractanimation.html#State-enum">Stopped</a>, or <a href="http://doc.qt.io/qt-5/qabstractanimation.html#State-enum">Paused</a>--as defined by the <a href="http://doc.qt.io/qt-5/qabstractanimation.html#State-enum">State</a> enum. The current state can be changed by calling <a href="http://doc.qt.io/qt-5/qabstractanimation.html#start">start</a>(), <a href="http://doc.qt.io/qt-5/qabstractanimation.html#stop">stop</a>(), <a href="http://doc.qt.io/qt-5/qabstractanimation.html#pause">pause</a>(), or <a href="http://doc.qt.io/qt-5/qabstractanimation.html#resume">resume</a>(). An animation will always reset its <a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentTime-prop">current time</a> when it is started. If paused, it will continue with the same current time when resumed. When an animation is stopped, it cannot be resumed, but will keep its current time (until started again). <a href="http://doc.qt.io/qt-5/qabstractanimation.html">QAbstractAnimation</a> will emit <a href="http://doc.qt.io/qt-5/qabstractanimation.html#stateChanged">stateChanged</a>() whenever its state changes.</p>
/// <p>An animation can loop any number of times by setting the <a href="http://doc.qt.io/qt-5/qabstractanimation.html#loopCount-prop">loopCount</a> property. When an animation's current time reaches its <a href="http://doc.qt.io/qt-5/qabstractanimation.html#duration">duration</a>(), it will reset the current time and keep running. A loop count of 1 (the default value) means that the animation will run one time. Note that a duration of -1 means that the animation will run until stopped; the current time will increase indefinitely. When the current time equals <a href="http://doc.qt.io/qt-5/qabstractanimation.html#duration">duration</a>() and the animation is in its final loop, the <a href="http://doc.qt.io/qt-5/qabstractanimation.html#State-enum">Stopped</a> state is entered, and the <a href="http://doc.qt.io/qt-5/qabstractanimation.html#finished">finished</a>() signal is emitted.</p>
/// <p><a href="http://doc.qt.io/qt-5/qabstractanimation.html">QAbstractAnimation</a> provides pure virtual functions used by subclasses to track the progress of the animation: <a href="http://doc.qt.io/qt-5/qabstractanimation.html#duration">duration</a>() and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#updateCurrentTime">updateCurrentTime</a>(). The <a href="http://doc.qt.io/qt-5/qabstractanimation.html#duration">duration</a>() function lets you report a duration for the animation (as discussed above). The animation framework calls <a href="http://doc.qt.io/qt-5/qabstractanimation.html#updateCurrentTime">updateCurrentTime</a>() when current time has changed. By reimplementing this function, you can track the animation progress. Note that neither the interval between calls nor the number of calls to this function are defined; though, it will normally be 60 updates per second.</p>
/// <p>By reimplementing <a href="http://doc.qt.io/qt-5/qabstractanimation.html#updateState">updateState</a>(), you can track the animation's state changes, which is particularly useful for animations that are not driven by time.</p></div>
#[repr(C)]
pub struct QAbstractAnimation {
    _unused: u8,
}
impl QAbstractAnimation {
    /// <p><a href="http://doc.qt.io/qt-5/qabstractanimation.html">QAbstractAnimation</a> emits this signal after the animation has stopped and has reached the end.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractAnimation::finished` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#finished">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p><a href="http://doc.qt.io/qt-5/qabstractanimation.html">QAbstractAnimation</a> emits this signal after the animation has stopped and has reached the end.</p>
    /// <p>This signal is emitted after <a href="http://doc.qt.io/qt-5/qabstractanimation.html#stateChanged">stateChanged</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#stateChanged">stateChanged</a>().</p></div>
    pub fn finished(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2finished()\0"),
            )
        }
    }

    /// <p><a href="http://doc.qt.io/qt-5/qabstractanimation.html">QAbstractAnimation</a> emits this signal whenever the state of the animation has changed from <i>oldState</i> to <i>newState</i>. This signal is emitted after the virtual <a href="http://doc.qt.io/qt-5/qabstractanimation.html#updateState">updateState</a>() function is called.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractAnimation::stateChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#stateChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p><a href="http://doc.qt.io/qt-5/qabstractanimation.html">QAbstractAnimation</a> emits this signal whenever the state of the animation has changed from <i>oldState</i> to <i>newState</i>. This signal is emitted after the virtual <a href="http://doc.qt.io/qt-5/qabstractanimation.html#updateState">updateState</a>() function is called.</p>
    /// <p><b>Note:</b> Notifier signal for property <a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">state</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#updateState">updateState</a>().</p></div>
    pub fn state_changed(
        &self,
    ) -> crate::Signal<(
        crate::q_abstract_animation::State,
        crate::q_abstract_animation::State,
    )> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2stateChanged(QAbstractAnimation::State,QAbstractAnimation::State)\0",
                ),
            )
        }
    }

    /// <p><a href="http://doc.qt.io/qt-5/qabstractanimation.html">QAbstractAnimation</a> emits this signal whenever the current loop changes. <i>currentLoop</i> is the current loop.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractAnimation::currentLoopChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentLoopChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p><a href="http://doc.qt.io/qt-5/qabstractanimation.html">QAbstractAnimation</a> emits this signal whenever the current loop changes. <i>currentLoop</i> is the current loop.</p>
    /// <p><b>Note:</b> Notifier signal for property <a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentLoop-prop">currentLoop</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentLoop-prop">currentLoop</a>() and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#loopCount-prop">loopCount</a>().</p></div>
    pub fn current_loop_changed(&self) -> crate::Signal<(::std::os::raw::c_int,)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2currentLoopChanged(int)\0"),
            )
        }
    }

    /// <p><a href="http://doc.qt.io/qt-5/qabstractanimation.html">QAbstractAnimation</a> emits this signal whenever the direction has been changed. <i>newDirection</i> is the new direction.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractAnimation::directionChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#directionChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p><a href="http://doc.qt.io/qt-5/qabstractanimation.html">QAbstractAnimation</a> emits this signal whenever the direction has been changed. <i>newDirection</i> is the new direction.</p>
    /// <p><b>Note:</b> Notifier signal for property <a href="http://doc.qt.io/qt-5/qabstractanimation.html#direction-prop">direction</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#direction-prop">direction</a>.</p></div>
    pub fn direction_changed(&self) -> crate::Signal<(crate::q_abstract_animation::Direction,)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2directionChanged(QAbstractAnimation::Direction)\0",
                ),
            )
        }
    }

    /// <p>Starts the animation. The <i>policy</i> argument says whether or not the animation should be deleted when it's done. When the animation starts, the <a href="http://doc.qt.io/qt-5/qabstractanimation.html#stateChanged">stateChanged</a>() signal is emitted, and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">state</a>() returns Running. When control reaches the event loop, the animation will run by itself, periodically calling <a href="http://doc.qt.io/qt-5/qabstractanimation.html#updateCurrentTime">updateCurrentTime</a>() as the animation progresses.</p>
    ///
    /// Returns a built-in Qt slot `QAbstractAnimation::start` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#start">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Starts the animation. The <i>policy</i> argument says whether or not the animation should be deleted when it's done. When the animation starts, the <a href="http://doc.qt.io/qt-5/qabstractanimation.html#stateChanged">stateChanged</a>() signal is emitted, and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">state</a>() returns Running. When control reaches the event loop, the animation will run by itself, periodically calling <a href="http://doc.qt.io/qt-5/qabstractanimation.html#updateCurrentTime">updateCurrentTime</a>() as the animation progresses.</p>
    /// <p>If the animation is currently stopped or has already reached the end, calling start() will rewind the animation and start again from the beginning. When the animation reaches the end, the animation will either stop, or if the loop level is more than 1, it will rewind and continue from the beginning.</p>
    /// <p>If the animation is already running, this function does nothing.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#stop">stop</a>() and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">state</a>().</p></div>
    pub fn slot_start(&self) -> crate::Receiver<(crate::q_abstract_animation::DeletionPolicy,)> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"1start(QAbstractAnimation::DeletionPolicy)\0",
                ),
            )
        }
    }

    /// <p>Pauses the animation. When the animation is paused, <a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">state</a>() returns Paused. The value of <a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentTime-prop">currentTime</a> will remain unchanged until <a href="http://doc.qt.io/qt-5/qabstractanimation.html#resume">resume</a>() or <a href="http://doc.qt.io/qt-5/qabstractanimation.html#start">start</a>() is called. If you want to continue from the current time, call <a href="http://doc.qt.io/qt-5/qabstractanimation.html#resume">resume</a>().</p>
    ///
    /// Returns a built-in Qt slot `QAbstractAnimation::pause` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#pause">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Pauses the animation. When the animation is paused, <a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">state</a>() returns Paused. The value of <a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentTime-prop">currentTime</a> will remain unchanged until <a href="http://doc.qt.io/qt-5/qabstractanimation.html#resume">resume</a>() or <a href="http://doc.qt.io/qt-5/qabstractanimation.html#start">start</a>() is called. If you want to continue from the current time, call <a href="http://doc.qt.io/qt-5/qabstractanimation.html#resume">resume</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#start">start</a>(), <a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">state</a>(), and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#resume">resume</a>().</p></div>
    pub fn slot_pause(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1pause()\0"),
            )
        }
    }

    /// <p>Resumes the animation after it was paused. When the animation is resumed, it emits the resumed() and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#stateChanged">stateChanged</a>() signals. The currenttime is not changed.</p>
    ///
    /// Returns a built-in Qt slot `QAbstractAnimation::resume` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#resume">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Resumes the animation after it was paused. When the animation is resumed, it emits the resumed() and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#stateChanged">stateChanged</a>() signals. The currenttime is not changed.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#start">start</a>(), <a href="http://doc.qt.io/qt-5/qabstractanimation.html#pause">pause</a>(), and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">state</a>().</p></div>
    pub fn slot_resume(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1resume()\0"),
            )
        }
    }

    /// <p>If <i>paused</i> is true, the animation is paused. If <i>paused</i> is false, the animation is resumed.</p>
    ///
    /// Returns a built-in Qt slot `QAbstractAnimation::setPaused` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#setPaused">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If <i>paused</i> is true, the animation is paused. If <i>paused</i> is false, the animation is resumed.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">state</a>(), <a href="http://doc.qt.io/qt-5/qabstractanimation.html#pause">pause</a>(), and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#resume">resume</a>().</p></div>
    pub fn slot_set_paused(&self) -> crate::Receiver<(bool,)> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1setPaused(bool)\0"),
            )
        }
    }

    /// <p>Stops the animation. When the animation is stopped, it emits the <a href="http://doc.qt.io/qt-5/qabstractanimation.html#stateChanged">stateChanged</a>() signal, and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">state</a>() returns Stopped. The current time is not changed.</p>
    ///
    /// Returns a built-in Qt slot `QAbstractAnimation::stop` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#stop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Stops the animation. When the animation is stopped, it emits the <a href="http://doc.qt.io/qt-5/qabstractanimation.html#stateChanged">stateChanged</a>() signal, and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">state</a>() returns Stopped. The current time is not changed.</p>
    /// <p>If the animation stops by itself after reaching the end (i.e., <a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentLoopTime">currentLoopTime</a>() == <a href="http://doc.qt.io/qt-5/qabstractanimation.html#duration">duration</a>() and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentLoop-prop">currentLoop</a>() &gt; <a href="http://doc.qt.io/qt-5/qabstractanimation.html#loopCount-prop">loopCount</a>() - 1), the <a href="http://doc.qt.io/qt-5/qabstractanimation.html#finished">finished</a>() signal is emitted.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#start">start</a>() and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">state</a>().</p></div>
    pub fn slot_stop(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1stop()\0"),
            )
        }
    }

    /// <p>This property holds the current time and progress of the animation</p>
    ///
    /// Returns a built-in Qt slot `QAbstractAnimation::setCurrentTime` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentTime-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the current time and progress of the animation</p>
    /// <p>This property describes the animation's current time. You can change the current time by calling setCurrentTime, or you can call <a href="http://doc.qt.io/qt-5/qabstractanimation.html#start">start</a>() and let the animation run, setting the current time automatically as the animation progresses.</p>
    /// <p>The animation's current time starts at 0, and ends at <a href="http://doc.qt.io/qt-5/qabstractanimation.html#totalDuration">totalDuration</a>().</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>currentTime</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setCurrentTime</b></span>(int <i>msecs</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#loopCount-prop">loopCount</a> and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentLoopTime">currentLoopTime</a>().</p></div>
    pub fn slot_set_current_time(&self) -> crate::Receiver<(::std::os::raw::c_int,)> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1setCurrentTime(int)\0"),
            )
        }
    }

    /// <p>This property holds the current loop of the animation</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QAbstractAnimation::currentLoop() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentLoop-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the current loop of the animation</p>
    /// <p>This property describes the current loop of the animation. By default, the animation's loop count is 1, and so the current loop will always be 0. If the loop count is 2 and the animation runs past its duration, it will automatically rewind and restart at current time 0, and current loop 1, and so on.</p>
    /// <p>When the current loop changes, <a href="http://doc.qt.io/qt-5/qabstractanimation.html">QAbstractAnimation</a> emits the <a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentLoopChanged">currentLoopChanged</a>() signal.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>currentLoop</b></span>() const</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentLoopChanged">currentLoopChanged</a></b></span>(int <i>currentLoop</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn current_loop(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QAbstractAnimation_currentLoop(
            self as *const crate::QAbstractAnimation,
        )
    }

    /// <p>Returns the current time inside the current loop. It can go from 0 to <a href="http://doc.qt.io/qt-5/qabstractanimation.html#duration">duration</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QAbstractAnimation::currentLoopTime() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentLoopTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current time inside the current loop. It can go from 0 to <a href="http://doc.qt.io/qt-5/qabstractanimation.html#duration">duration</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#duration">duration</a>() and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentTime-prop">currentTime</a>.</p></div>
    pub unsafe fn current_loop_time(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QAbstractAnimation_currentLoopTime(
            self as *const crate::QAbstractAnimation,
        )
    }

    /// <p>This property holds the current time and progress of the animation</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QAbstractAnimation::currentTime() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentTime-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the current time and progress of the animation</p>
    /// <p>This property describes the animation's current time. You can change the current time by calling setCurrentTime, or you can call <a href="http://doc.qt.io/qt-5/qabstractanimation.html#start">start</a>() and let the animation run, setting the current time automatically as the animation progresses.</p>
    /// <p>The animation's current time starts at 0, and ends at <a href="http://doc.qt.io/qt-5/qabstractanimation.html#totalDuration">totalDuration</a>().</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>currentTime</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setCurrentTime</b></span>(int <i>msecs</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#loopCount-prop">loopCount</a> and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentLoopTime">currentLoopTime</a>().</p></div>
    pub unsafe fn current_time(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QAbstractAnimation_currentTime(
            self as *const crate::QAbstractAnimation,
        )
    }

    /// <p>This property holds the direction of the animation when it is in <a href="http://doc.qt.io/qt-5/qabstractanimation.html#State-enum">Running</a> state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation::Direction QAbstractAnimation::direction() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#direction-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the direction of the animation when it is in <a href="http://doc.qt.io/qt-5/qabstractanimation.html#State-enum">Running</a> state.</p>
    /// <p>This direction indicates whether the time moves from 0 towards the animation duration, or from the value of the duration and towards 0 after <a href="http://doc.qt.io/qt-5/qabstractanimation.html#start">start</a>() has been called.</p>
    /// <p>By default, this property is set to <a href="http://doc.qt.io/qt-5/qabstractanimation.html#Direction-enum">Forward</a>.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> Direction </td><td class="memItemRight bottomAlign"><span class="name"><b>direction</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setDirection</b></span>(Direction <i>direction</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#directionChanged">directionChanged</a></b></span>(QAbstractAnimation::Direction <i>newDirection</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn direction(&self) -> crate::q_abstract_animation::Direction {
        crate::__ffi::ctr_qt_core_ffi_QAbstractAnimation_direction(
            self as *const crate::QAbstractAnimation,
        )
    }

    /// <p>This property holds the duration of the animation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual int QAbstractAnimation::duration() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#duration-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the duration of the animation.</p>
    /// <p>If the duration is -1, it means that the duration is undefined. In this case, <a href="http://doc.qt.io/qt-5/qabstractanimation.html#loopCount-prop">loopCount</a> is ignored.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> virtual int </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#duration">duration</a></b></span>() const = 0</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn duration(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QAbstractAnimation_duration(
            self as *const crate::QAbstractAnimation,
        )
    }

    /// <p>If this animation is part of a <a href="http://doc.qt.io/qt-5/qanimationgroup.html">QAnimationGroup</a>, this function returns a pointer to the group; otherwise, it returns 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAnimationGroup* QAbstractAnimation::group() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#group">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If this animation is part of a <a href="http://doc.qt.io/qt-5/qanimationgroup.html">QAnimationGroup</a>, this function returns a pointer to the group; otherwise, it returns 0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qanimationgroup.html#addAnimation">QAnimationGroup::addAnimation</a>().</p></div>
    pub unsafe fn group(&self) -> ::cpp_utils::MutPtr<crate::QAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractAnimation_group(
            self as *const crate::QAbstractAnimation,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This property holds the loop count of the animation</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QAbstractAnimation::loopCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#loopCount-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the loop count of the animation</p>
    /// <p>This property describes the loop count of the animation as an integer. By default this value is 1, indicating that the animation should run once only, and then stop. By changing it you can let the animation loop several times. With a value of 0, the animation will not run at all, and with a value of -1, the animation will loop forever until stopped. It is not supported to have loop on an animation that has an undefined duration. It will only run once.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>loopCount</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setLoopCount</b></span>(int <i>loopCount</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn loop_count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QAbstractAnimation_loopCount(
            self as *const crate::QAbstractAnimation,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QAbstractAnimation::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractAnimation_metaObject(
            self as *const crate::QAbstractAnimation,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Pauses the animation. When the animation is paused, <a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">state</a>() returns Paused. The value of <a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentTime-prop">currentTime</a> will remain unchanged until <a href="http://doc.qt.io/qt-5/qabstractanimation.html#resume">resume</a>() or <a href="http://doc.qt.io/qt-5/qabstractanimation.html#start">start</a>() is called. If you want to continue from the current time, call <a href="http://doc.qt.io/qt-5/qabstractanimation.html#resume">resume</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QAbstractAnimation::pause()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#pause">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Pauses the animation. When the animation is paused, <a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">state</a>() returns Paused. The value of <a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentTime-prop">currentTime</a> will remain unchanged until <a href="http://doc.qt.io/qt-5/qabstractanimation.html#resume">resume</a>() or <a href="http://doc.qt.io/qt-5/qabstractanimation.html#start">start</a>() is called. If you want to continue from the current time, call <a href="http://doc.qt.io/qt-5/qabstractanimation.html#resume">resume</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#start">start</a>(), <a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">state</a>(), and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#resume">resume</a>().</p></div>
    pub unsafe fn pause(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractAnimation_pause(
            self as *mut crate::QAbstractAnimation,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QAbstractAnimation::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QAbstractAnimation_qt_metacall(
            self as *mut crate::QAbstractAnimation,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QAbstractAnimation::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractAnimation_qt_metacast(
            self as *mut crate::QAbstractAnimation,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Resumes the animation after it was paused. When the animation is resumed, it emits the resumed() and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#stateChanged">stateChanged</a>() signals. The currenttime is not changed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QAbstractAnimation::resume()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#resume">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Resumes the animation after it was paused. When the animation is resumed, it emits the resumed() and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#stateChanged">stateChanged</a>() signals. The currenttime is not changed.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#start">start</a>(), <a href="http://doc.qt.io/qt-5/qabstractanimation.html#pause">pause</a>(), and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">state</a>().</p></div>
    pub unsafe fn resume(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractAnimation_resume(
            self as *mut crate::QAbstractAnimation,
        )
    }

    /// <p>This property holds the current time and progress of the animation</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QAbstractAnimation::setCurrentTime(int msecs)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentTime-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the current time and progress of the animation</p>
    /// <p>This property describes the animation's current time. You can change the current time by calling setCurrentTime, or you can call <a href="http://doc.qt.io/qt-5/qabstractanimation.html#start">start</a>() and let the animation run, setting the current time automatically as the animation progresses.</p>
    /// <p>The animation's current time starts at 0, and ends at <a href="http://doc.qt.io/qt-5/qabstractanimation.html#totalDuration">totalDuration</a>().</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>currentTime</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setCurrentTime</b></span>(int <i>msecs</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#loopCount-prop">loopCount</a> and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentLoopTime">currentLoopTime</a>().</p></div>
    pub unsafe fn set_current_time(&mut self, msecs: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractAnimation_setCurrentTime(
            self as *mut crate::QAbstractAnimation,
            msecs,
        )
    }

    /// <p>This property holds the direction of the animation when it is in <a href="http://doc.qt.io/qt-5/qabstractanimation.html#State-enum">Running</a> state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QAbstractAnimation::setDirection(QAbstractAnimation::Direction direction)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#direction-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the direction of the animation when it is in <a href="http://doc.qt.io/qt-5/qabstractanimation.html#State-enum">Running</a> state.</p>
    /// <p>This direction indicates whether the time moves from 0 towards the animation duration, or from the value of the duration and towards 0 after <a href="http://doc.qt.io/qt-5/qabstractanimation.html#start">start</a>() has been called.</p>
    /// <p>By default, this property is set to <a href="http://doc.qt.io/qt-5/qabstractanimation.html#Direction-enum">Forward</a>.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> Direction </td><td class="memItemRight bottomAlign"><span class="name"><b>direction</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setDirection</b></span>(Direction <i>direction</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#directionChanged">directionChanged</a></b></span>(QAbstractAnimation::Direction <i>newDirection</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn set_direction(&mut self, direction: crate::q_abstract_animation::Direction) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractAnimation_setDirection(
            self as *mut crate::QAbstractAnimation,
            direction,
        )
    }

    /// <p>This property holds the loop count of the animation</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QAbstractAnimation::setLoopCount(int loopCount)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#loopCount-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the loop count of the animation</p>
    /// <p>This property describes the loop count of the animation as an integer. By default this value is 1, indicating that the animation should run once only, and then stop. By changing it you can let the animation loop several times. With a value of 0, the animation will not run at all, and with a value of -1, the animation will loop forever until stopped. It is not supported to have loop on an animation that has an undefined duration. It will only run once.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>loopCount</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setLoopCount</b></span>(int <i>loopCount</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn set_loop_count(&mut self, loop_count: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractAnimation_setLoopCount(
            self as *mut crate::QAbstractAnimation,
            loop_count,
        )
    }

    /// <p>If <i>paused</i> is true, the animation is paused. If <i>paused</i> is false, the animation is resumed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QAbstractAnimation::setPaused(bool arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#setPaused">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If <i>paused</i> is true, the animation is paused. If <i>paused</i> is false, the animation is resumed.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">state</a>(), <a href="http://doc.qt.io/qt-5/qabstractanimation.html#pause">pause</a>(), and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#resume">resume</a>().</p></div>
    pub unsafe fn set_paused(&mut self, arg1: bool) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractAnimation_setPaused(
            self as *mut crate::QAbstractAnimation,
            arg1,
        )
    }

    /// <p>Starts the animation. The <i>policy</i> argument says whether or not the animation should be deleted when it's done. When the animation starts, the <a href="http://doc.qt.io/qt-5/qabstractanimation.html#stateChanged">stateChanged</a>() signal is emitted, and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">state</a>() returns Running. When control reaches the event loop, the animation will run by itself, periodically calling <a href="http://doc.qt.io/qt-5/qabstractanimation.html#updateCurrentTime">updateCurrentTime</a>() as the animation progresses.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QAbstractAnimation::start(QAbstractAnimation::DeletionPolicy policy = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#start">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Starts the animation. The <i>policy</i> argument says whether or not the animation should be deleted when it's done. When the animation starts, the <a href="http://doc.qt.io/qt-5/qabstractanimation.html#stateChanged">stateChanged</a>() signal is emitted, and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">state</a>() returns Running. When control reaches the event loop, the animation will run by itself, periodically calling <a href="http://doc.qt.io/qt-5/qabstractanimation.html#updateCurrentTime">updateCurrentTime</a>() as the animation progresses.</p>
    /// <p>If the animation is currently stopped or has already reached the end, calling start() will rewind the animation and start again from the beginning. When the animation reaches the end, the animation will either stop, or if the loop level is more than 1, it will rewind and continue from the beginning.</p>
    /// <p>If the animation is already running, this function does nothing.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#stop">stop</a>() and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">state</a>().</p></div>
    pub unsafe fn start_1a(&mut self, policy: crate::q_abstract_animation::DeletionPolicy) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractAnimation_start(
            self as *mut crate::QAbstractAnimation,
            policy,
        )
    }

    /// <p>Starts the animation. The <i>policy</i> argument says whether or not the animation should be deleted when it's done. When the animation starts, the <a href="http://doc.qt.io/qt-5/qabstractanimation.html#stateChanged">stateChanged</a>() signal is emitted, and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">state</a>() returns Running. When control reaches the event loop, the animation will run by itself, periodically calling <a href="http://doc.qt.io/qt-5/qabstractanimation.html#updateCurrentTime">updateCurrentTime</a>() as the animation progresses.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QAbstractAnimation::start()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#start">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Starts the animation. The <i>policy</i> argument says whether or not the animation should be deleted when it's done. When the animation starts, the <a href="http://doc.qt.io/qt-5/qabstractanimation.html#stateChanged">stateChanged</a>() signal is emitted, and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">state</a>() returns Running. When control reaches the event loop, the animation will run by itself, periodically calling <a href="http://doc.qt.io/qt-5/qabstractanimation.html#updateCurrentTime">updateCurrentTime</a>() as the animation progresses.</p>
    /// <p>If the animation is currently stopped or has already reached the end, calling start() will rewind the animation and start again from the beginning. When the animation reaches the end, the animation will either stop, or if the loop level is more than 1, it will rewind and continue from the beginning.</p>
    /// <p>If the animation is already running, this function does nothing.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#stop">stop</a>() and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">state</a>().</p></div>
    pub unsafe fn start_0a(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractAnimation_start1(
            self as *mut crate::QAbstractAnimation,
        )
    }

    /// <p>state of the animation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation::State QAbstractAnimation::state() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>state of the animation.</p>
    /// <p>This property describes the current state of the animation. When the animation state changes, <a href="http://doc.qt.io/qt-5/qabstractanimation.html">QAbstractAnimation</a> emits the <a href="http://doc.qt.io/qt-5/qabstractanimation.html#stateChanged">stateChanged</a>() signal.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> State </td><td class="memItemRight bottomAlign"><span class="name"><b>state</b></span>() const</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#stateChanged">stateChanged</a></b></span>(QAbstractAnimation::State <i>newState</i>, QAbstractAnimation::State <i>oldState</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn state(&self) -> crate::q_abstract_animation::State {
        crate::__ffi::ctr_qt_core_ffi_QAbstractAnimation_state(
            self as *const crate::QAbstractAnimation,
        )
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractAnimation_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Stops the animation. When the animation is stopped, it emits the <a href="http://doc.qt.io/qt-5/qabstractanimation.html#stateChanged">stateChanged</a>() signal, and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">state</a>() returns Stopped. The current time is not changed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QAbstractAnimation::stop()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#stop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Stops the animation. When the animation is stopped, it emits the <a href="http://doc.qt.io/qt-5/qabstractanimation.html#stateChanged">stateChanged</a>() signal, and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">state</a>() returns Stopped. The current time is not changed.</p>
    /// <p>If the animation stops by itself after reaching the end (i.e., <a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentLoopTime">currentLoopTime</a>() == <a href="http://doc.qt.io/qt-5/qabstractanimation.html#duration">duration</a>() and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentLoop-prop">currentLoop</a>() &gt; <a href="http://doc.qt.io/qt-5/qabstractanimation.html#loopCount-prop">loopCount</a>() - 1), the <a href="http://doc.qt.io/qt-5/qabstractanimation.html#finished">finished</a>() signal is emitted.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#start">start</a>() and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#state-prop">state</a>().</p></div>
    pub unsafe fn stop(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractAnimation_stop(
            self as *mut crate::QAbstractAnimation,
        )
    }

    /// <p>Returns the total and effective duration of the animation, including the loop count.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QAbstractAnimation::totalDuration() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#totalDuration">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the total and effective duration of the animation, including the loop count.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#duration">duration</a>() and <a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentTime-prop">currentTime</a>.</p></div>
    pub unsafe fn total_duration(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QAbstractAnimation_totalDuration(
            self as *const crate::QAbstractAnimation,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QAbstractAnimation::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractAnimation_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QAbstractAnimation::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractAnimation_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// C++ class: <span style='color: green;'>```QAnimationDriver```</span>.
#[repr(C)]
pub struct QAnimationDriver {
    _unused: u8,
}
impl QAnimationDriver {
    /// Returns a built-in Qt signal `QAnimationDriver::started` that can be passed to `qt_core::Signal::connect`.
    pub fn started(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2started()\0"),
            )
        }
    }

    /// Returns a built-in Qt signal `QAnimationDriver::stopped` that can be passed to `qt_core::Signal::connect`.
    pub fn stopped(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2stopped()\0"),
            )
        }
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void QAnimationDriver::advance()```</span>.
    pub unsafe fn advance(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QAnimationDriver_advance(self as *mut crate::QAnimationDriver)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual qint64 QAnimationDriver::elapsed() const```</span>.
    pub unsafe fn elapsed(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QAnimationDriver_elapsed(
            self as *const crate::QAnimationDriver,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QAnimationDriver::install()```</span>.
    pub unsafe fn install(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QAnimationDriver_install(self as *mut crate::QAnimationDriver)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QAnimationDriver::isRunning() const```</span>.
    pub unsafe fn is_running(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAnimationDriver_isRunning(
            self as *const crate::QAnimationDriver,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QAnimationDriver::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAnimationDriver_metaObject(
            self as *const crate::QAnimationDriver,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QAnimationDriver::QAnimationDriver(QObject* parent = …)```</span>.
    pub unsafe fn new_1a(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QAnimationDriver> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAnimationDriver_QAnimationDriver(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QAnimationDriver::QAnimationDriver()```</span>.
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QAnimationDriver> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAnimationDriver_QAnimationDriver1();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QAnimationDriver::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QAnimationDriver_qt_metacall(
            self as *mut crate::QAnimationDriver,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QAnimationDriver::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAnimationDriver_qt_metacast(
            self as *mut crate::QAnimationDriver,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```void QAnimationDriver::setStartTime(qint64 startTime)```</span>.
    pub unsafe fn set_start_time(&mut self, start_time: i64) {
        crate::__ffi::ctr_qt_core_ffi_QAnimationDriver_setStartTime(
            self as *mut crate::QAnimationDriver,
            start_time,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```qint64 QAnimationDriver::startTime() const```</span>.
    pub unsafe fn start_time(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QAnimationDriver_startTime(
            self as *const crate::QAnimationDriver,
        )
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAnimationDriver_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QAnimationDriver::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAnimationDriver_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QAnimationDriver::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAnimationDriver_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```void QAnimationDriver::uninstall()```</span>.
    pub unsafe fn uninstall(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QAnimationDriver_uninstall(
            self as *mut crate::QAnimationDriver,
        )
    }
}

pub mod q_event_loop {
    //! C++ type: <span style='color: green;'>```QEventLoop```</span>

    /// <p>This enum controls the types of events processed by the <a href="http://doc.qt.io/qt-5/qeventloop.html#processEvents">processEvents</a>() functions.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QEventLoop::ProcessEventsFlag```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventloop.html#ProcessEventsFlag-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum controls the types of events processed by the <a href="http://doc.qt.io/qt-5/qeventloop.html#processEvents">processEvents</a>() functions.</p>
    ///
    /// <p>The ProcessEventsFlags type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;ProcessEventsFlag&gt;. It stores an OR combination of ProcessEventsFlag values.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qeventloop.html#processEvents">processEvents</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct ProcessEventsFlag(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for ProcessEventsFlag {
        fn from(value: ::std::os::raw::c_int) -> Self {
            ProcessEventsFlag(value)
        }
    }

    impl From<ProcessEventsFlag> for ::std::os::raw::c_int {
        fn from(value: ProcessEventsFlag) -> Self {
            value.0
        }
    }

    impl ProcessEventsFlag {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl ProcessEventsFlag {
        /// All events. Note that <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">DeferredDelete</a> events are processed specially. See <a href="http://doc.qt.io/qt-5/qobject.html#deleteLater">QObject::deleteLater</a>() for more details. (C++ enum variant: <span style='color: green;'>```AllEvents = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const AllEvents: crate::q_event_loop::ProcessEventsFlag =
            crate::q_event_loop::ProcessEventsFlag(0);
        /// Do not process user input events, such as ButtonPress and KeyPress. Note that the events are not discarded; they will be delivered the next time <a href="http://doc.qt.io/qt-5/qeventloop.html#processEvents">processEvents</a>() is called without the ExcludeUserInputEvents flag. (C++ enum variant: <span style='color: green;'>```ExcludeUserInputEvents = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const ExcludeUserInputEvents: crate::q_event_loop::ProcessEventsFlag =
            crate::q_event_loop::ProcessEventsFlag(1);
        /// Do not process socket notifier events. Note that the events are not discarded; they will be delivered the next time <a href="http://doc.qt.io/qt-5/qeventloop.html#processEvents">processEvents</a>() is called without the ExcludeSocketNotifiers flag. (C++ enum variant: <span style='color: green;'>```ExcludeSocketNotifiers = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const ExcludeSocketNotifiers: crate::q_event_loop::ProcessEventsFlag =
            crate::q_event_loop::ProcessEventsFlag(2);
        /// Wait for events if no pending events are available. (C++ enum variant: <span style='color: green;'>```WaitForMoreEvents = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const WaitForMoreEvents: crate::q_event_loop::ProcessEventsFlag =
            crate::q_event_loop::ProcessEventsFlag(4);
        /// C++ enum variant: <span style='color: green;'>```X11ExcludeTimers = 8```</span>
        #[allow(non_upper_case_globals)]
        pub const X11ExcludeTimers: crate::q_event_loop::ProcessEventsFlag =
            crate::q_event_loop::ProcessEventsFlag(8);
        /// C++ enum variant: <span style='color: green;'>```EventLoopExec = 32```</span>
        #[allow(non_upper_case_globals)]
        pub const EventLoopExec: crate::q_event_loop::ProcessEventsFlag =
            crate::q_event_loop::ProcessEventsFlag(32);
        /// C++ enum variant: <span style='color: green;'>```DialogExec = 64```</span>
        #[allow(non_upper_case_globals)]
        pub const DialogExec: crate::q_event_loop::ProcessEventsFlag =
            crate::q_event_loop::ProcessEventsFlag(64);
    }

    impl From<crate::q_event_loop::ProcessEventsFlag>
        for crate::QFlags<crate::q_event_loop::ProcessEventsFlag>
    {
        fn from(value: crate::q_event_loop::ProcessEventsFlag) -> Self {
            Self::from(value.to_int())
        }
    }

    impl<T: Into<crate::QFlags<crate::q_event_loop::ProcessEventsFlag>>> std::ops::BitOr<T>
        for crate::q_event_loop::ProcessEventsFlag
    {
        type Output = crate::QFlags<crate::q_event_loop::ProcessEventsFlag>;
        fn bitor(self, rhs: T) -> crate::QFlags<crate::q_event_loop::ProcessEventsFlag> {
            Into::<crate::QFlags<crate::q_event_loop::ProcessEventsFlag>>::into(self) | rhs
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qeventloop.html">QEventLoop</a> class provides a means of entering and leaving an event loop.</p>
///
/// C++ class: <span style='color: green;'>```QEventLoop```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qeventloop.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qeventloop.html">QEventLoop</a> class provides a means of entering and leaving an event loop.</p>
/// <p>At any time, you can create a <a href="http://doc.qt.io/qt-5/qeventloop.html">QEventLoop</a> object and call <a href="http://doc.qt.io/qt-5/qeventloop.html#exec">exec</a>() on it to start a local event loop. From within the event loop, calling <a href="http://doc.qt.io/qt-5/qeventloop.html#exit">exit</a>() will force <a href="http://doc.qt.io/qt-5/qeventloop.html#exec">exec</a>() to return.</p></div>
#[repr(C)]
pub struct QEventLoop {
    _unused: u8,
}
impl QEventLoop {
    /// <p>Tells the event loop to exit normally.</p>
    ///
    /// Returns a built-in Qt slot `QEventLoop::quit` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventloop.html#quit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Tells the event loop to exit normally.</p>
    /// <p>Same as exit(0).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#quit">QCoreApplication::quit</a>() and <a href="http://doc.qt.io/qt-5/qeventloop.html#exit">exit</a>().</p></div>
    pub fn slot_quit(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1quit()\0"),
            )
        }
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qobject.html#event">QObject::event</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QEventLoop::event(QEvent* event)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventloop.html#event">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qobject.html#event">QObject::event</a>().</p></div>
    pub unsafe fn event(
        &mut self,
        event: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QEvent>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QEventLoop_event(
            self as *mut crate::QEventLoop,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QEvent>>::cast_into(event)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Enters the main event loop and waits until <a href="http://doc.qt.io/qt-5/qeventloop.html#exit">exit</a>() is called. Returns the value that was passed to <a href="http://doc.qt.io/qt-5/qeventloop.html#exit">exit</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QEventLoop::exec(QFlags<QEventLoop::ProcessEventsFlag> flags = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventloop.html#exec">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Enters the main event loop and waits until <a href="http://doc.qt.io/qt-5/qeventloop.html#exit">exit</a>() is called. Returns the value that was passed to <a href="http://doc.qt.io/qt-5/qeventloop.html#exit">exit</a>().</p>
    /// <p>If <i>flags</i> are specified, only events of the types allowed by the <i>flags</i> will be processed.</p>
    /// <p>It is necessary to call this function to start event handling. The main event loop receives events from the window system and dispatches these to the application widgets.</p>
    /// <p>Generally speaking, no user interaction can take place before calling exec(). As a special case, modal widgets like <a href="http://doc.qt.io/qt-5/qmessagebox.html">QMessageBox</a> can be used before calling exec(), because modal widgets use their own local event loop.</p>
    /// <p>To make your application perform idle processing (i.e. executing a special function whenever there are no pending events), use a <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> with 0 timeout. More sophisticated idle processing schemes can be achieved using <a href="http://doc.qt.io/qt-5/qeventloop.html#processEvents">processEvents</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#quit">QCoreApplication::quit</a>(), <a href="http://doc.qt.io/qt-5/qeventloop.html#exit">exit</a>(), and <a href="http://doc.qt.io/qt-5/qeventloop.html#processEvents">processEvents</a>().</p></div>
    pub unsafe fn exec_1a(
        &mut self,
        flags: crate::QFlags<crate::q_event_loop::ProcessEventsFlag>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QEventLoop_exec(
            self as *mut crate::QEventLoop,
            flags.to_int(),
        )
    }

    /// <p>Enters the main event loop and waits until <a href="http://doc.qt.io/qt-5/qeventloop.html#exit">exit</a>() is called. Returns the value that was passed to <a href="http://doc.qt.io/qt-5/qeventloop.html#exit">exit</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QEventLoop::exec()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventloop.html#exec">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Enters the main event loop and waits until <a href="http://doc.qt.io/qt-5/qeventloop.html#exit">exit</a>() is called. Returns the value that was passed to <a href="http://doc.qt.io/qt-5/qeventloop.html#exit">exit</a>().</p>
    /// <p>If <i>flags</i> are specified, only events of the types allowed by the <i>flags</i> will be processed.</p>
    /// <p>It is necessary to call this function to start event handling. The main event loop receives events from the window system and dispatches these to the application widgets.</p>
    /// <p>Generally speaking, no user interaction can take place before calling exec(). As a special case, modal widgets like <a href="http://doc.qt.io/qt-5/qmessagebox.html">QMessageBox</a> can be used before calling exec(), because modal widgets use their own local event loop.</p>
    /// <p>To make your application perform idle processing (i.e. executing a special function whenever there are no pending events), use a <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> with 0 timeout. More sophisticated idle processing schemes can be achieved using <a href="http://doc.qt.io/qt-5/qeventloop.html#processEvents">processEvents</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#quit">QCoreApplication::quit</a>(), <a href="http://doc.qt.io/qt-5/qeventloop.html#exit">exit</a>(), and <a href="http://doc.qt.io/qt-5/qeventloop.html#processEvents">processEvents</a>().</p></div>
    pub unsafe fn exec_0a(&mut self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QEventLoop_exec1(self as *mut crate::QEventLoop)
    }

    /// <p>Tells the event loop to exit with a return code.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QEventLoop::exit(int returnCode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventloop.html#exit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Tells the event loop to exit with a return code.</p>
    /// <p>After this function has been called, the event loop returns from the call to <a href="http://doc.qt.io/qt-5/qeventloop.html#exec">exec</a>(). The <a href="http://doc.qt.io/qt-5/qeventloop.html#exec">exec</a>() function returns <i>returnCode</i>.</p>
    /// <p>By convention, a <i>returnCode</i> of 0 means success, and any non-zero value indicates an error.</p>
    /// <p>Note that unlike the C library function of the same name, this function <i>does</i> return to the caller -- it is event processing that stops.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#quit">QCoreApplication::quit</a>(), <a href="http://doc.qt.io/qt-5/qeventloop.html#quit">quit</a>(), and <a href="http://doc.qt.io/qt-5/qeventloop.html#exec">exec</a>().</p></div>
    pub unsafe fn exit_1a(&mut self, return_code: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QEventLoop_exit(self as *mut crate::QEventLoop, return_code)
    }

    /// <p>Tells the event loop to exit with a return code.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QEventLoop::exit()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventloop.html#exit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Tells the event loop to exit with a return code.</p>
    /// <p>After this function has been called, the event loop returns from the call to <a href="http://doc.qt.io/qt-5/qeventloop.html#exec">exec</a>(). The <a href="http://doc.qt.io/qt-5/qeventloop.html#exec">exec</a>() function returns <i>returnCode</i>.</p>
    /// <p>By convention, a <i>returnCode</i> of 0 means success, and any non-zero value indicates an error.</p>
    /// <p>Note that unlike the C library function of the same name, this function <i>does</i> return to the caller -- it is event processing that stops.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#quit">QCoreApplication::quit</a>(), <a href="http://doc.qt.io/qt-5/qeventloop.html#quit">quit</a>(), and <a href="http://doc.qt.io/qt-5/qeventloop.html#exec">exec</a>().</p></div>
    pub unsafe fn exit_0a(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QEventLoop_exit1(self as *mut crate::QEventLoop)
    }

    /// <p>Returns <code>true</code> if the event loop is running; otherwise returns false. The event loop is considered running from the time when <a href="http://doc.qt.io/qt-5/qeventloop.html#exec">exec</a>() is called until <a href="http://doc.qt.io/qt-5/qeventloop.html#exit">exit</a>() is called.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QEventLoop::isRunning() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventloop.html#isRunning">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the event loop is running; otherwise returns false. The event loop is considered running from the time when <a href="http://doc.qt.io/qt-5/qeventloop.html#exec">exec</a>() is called until <a href="http://doc.qt.io/qt-5/qeventloop.html#exit">exit</a>() is called.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qeventloop.html#exec">exec</a>() and <a href="http://doc.qt.io/qt-5/qeventloop.html#exit">exit</a>().</p></div>
    pub unsafe fn is_running(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QEventLoop_isRunning(self as *const crate::QEventLoop)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QEventLoop::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QEventLoop_metaObject(self as *const crate::QEventLoop);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs an event loop object with the given <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QEventLoop::QEventLoop(QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventloop.html#QEventLoop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an event loop object with the given <i>parent</i>.</p></div>
    pub unsafe fn new_1a(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QEventLoop> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEventLoop_QEventLoop(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qeventloop.html">QEventLoop</a> class provides a means of entering and leaving an event loop.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QEventLoop::QEventLoop()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventloop.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qeventloop.html">QEventLoop</a> class provides a means of entering and leaving an event loop.</p>
    /// <p>At any time, you can create a <a href="http://doc.qt.io/qt-5/qeventloop.html">QEventLoop</a> object and call <a href="http://doc.qt.io/qt-5/qeventloop.html#exec">exec</a>() on it to start a local event loop. From within the event loop, calling <a href="http://doc.qt.io/qt-5/qeventloop.html#exit">exit</a>() will force <a href="http://doc.qt.io/qt-5/qeventloop.html#exec">exec</a>() to return.</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QEventLoop> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEventLoop_QEventLoop1();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Processes pending events that match <i>flags</i> until there are no more events to process. Returns <code>true</code> if pending events were handled; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QEventLoop::processEvents(QFlags<QEventLoop::ProcessEventsFlag> flags = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventloop.html#processEvents">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Processes pending events that match <i>flags</i> until there are no more events to process. Returns <code>true</code> if pending events were handled; otherwise returns <code>false</code>.</p>
    /// <p>This function is especially useful if you have a long running operation and want to show its progress without allowing user input; i.e. by using the <a href="http://doc.qt.io/qt-5/qeventloop.html#ProcessEventsFlag-enum">ExcludeUserInputEvents</a> flag.</p>
    /// <p>This function is simply a wrapper for <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#processEvents">QAbstractEventDispatcher::processEvents</a>(). See the documentation for that function for details.</p></div>
    pub unsafe fn process_events_1a(
        &mut self,
        flags: crate::QFlags<crate::q_event_loop::ProcessEventsFlag>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QEventLoop_processEvents(
            self as *mut crate::QEventLoop,
            flags.to_int(),
        )
    }

    /// <p>Process pending events that match <i>flags</i> for a maximum of <i>maxTime</i> milliseconds, or until there are no more events to process, whichever is shorter. This function is especially useful if you have a long running operation and want to show its progress without allowing user input, i.e. by using the <a href="http://doc.qt.io/qt-5/qeventloop.html#ProcessEventsFlag-enum">ExcludeUserInputEvents</a> flag.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QEventLoop::processEvents(QFlags<QEventLoop::ProcessEventsFlag> flags, int maximumTime)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventloop.html#processEvents-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Process pending events that match <i>flags</i> for a maximum of <i>maxTime</i> milliseconds, or until there are no more events to process, whichever is shorter. This function is especially useful if you have a long running operation and want to show its progress without allowing user input, i.e. by using the <a href="http://doc.qt.io/qt-5/qeventloop.html#ProcessEventsFlag-enum">ExcludeUserInputEvents</a> flag.</p>
    /// <p><b>Notes:</b></p>
    /// <ul>
    /// <li>This function does not process events continuously; it returns after all available events are processed.</li>
    /// <li>Specifying the <a href="http://doc.qt.io/qt-5/qeventloop.html#ProcessEventsFlag-enum">WaitForMoreEvents</a> flag makes no sense and will be ignored.</li>
    /// </ul></div>
    pub unsafe fn process_events_2a(
        &mut self,
        flags: crate::QFlags<crate::q_event_loop::ProcessEventsFlag>,
        maximum_time: ::std::os::raw::c_int,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QEventLoop_processEvents1(
            self as *mut crate::QEventLoop,
            flags.to_int(),
            maximum_time,
        )
    }

    /// <p>Processes pending events that match <i>flags</i> until there are no more events to process. Returns <code>true</code> if pending events were handled; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QEventLoop::processEvents()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventloop.html#processEvents">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Processes pending events that match <i>flags</i> until there are no more events to process. Returns <code>true</code> if pending events were handled; otherwise returns <code>false</code>.</p>
    /// <p>This function is especially useful if you have a long running operation and want to show its progress without allowing user input; i.e. by using the <a href="http://doc.qt.io/qt-5/qeventloop.html#ProcessEventsFlag-enum">ExcludeUserInputEvents</a> flag.</p>
    /// <p>This function is simply a wrapper for <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#processEvents">QAbstractEventDispatcher::processEvents</a>(). See the documentation for that function for details.</p></div>
    pub unsafe fn process_events_0a(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QEventLoop_processEvents2(self as *mut crate::QEventLoop)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QEventLoop::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QEventLoop_qt_metacall(
            self as *mut crate::QEventLoop,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QEventLoop::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEventLoop_qt_metacast(
            self as *mut crate::QEventLoop,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Tells the event loop to exit normally.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QEventLoop::quit()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventloop.html#quit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Tells the event loop to exit normally.</p>
    /// <p>Same as exit(0).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#quit">QCoreApplication::quit</a>() and <a href="http://doc.qt.io/qt-5/qeventloop.html#exit">exit</a>().</p></div>
    pub unsafe fn quit(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QEventLoop_quit(self as *mut crate::QEventLoop)
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEventLoop_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QEventLoop::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEventLoop_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QEventLoop::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEventLoop_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Wakes up the event loop.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QEventLoop::wakeUp()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventloop.html#wakeUp">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Wakes up the event loop.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#wakeUp">QAbstractEventDispatcher::wakeUp</a>().</p></div>
    pub unsafe fn wake_up(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QEventLoop_wakeUp(self as *mut crate::QEventLoop)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qeventlooplocker.html">QEventLoopLocker</a> class provides a means to quit an event loop when it is no longer needed.</p>
///
/// C++ class: <span style='color: green;'>```QEventLoopLocker```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qeventlooplocker.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qeventlooplocker.html">QEventLoopLocker</a> class provides a means to quit an event loop when it is no longer needed.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qeventlooplocker.html">QEventLoopLocker</a> operates on particular objects - either a <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> instance, a <a href="http://doc.qt.io/qt-5/qeventloop.html">QEventLoop</a> instance or a <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a> instance.</p>
/// <p>This makes it possible to, for example, run a batch of jobs with an event loop and exit that event loop after the last job is finished. That is accomplished by keeping a <a href="http://doc.qt.io/qt-5/qeventlooplocker.html">QEventLoopLocker</a> with each job instance.</p>
/// <p>The variant which operates on <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> makes it possible to finish asynchronously running jobs after the last gui window has been closed. This can be useful for example for running a job which uploads data to a network.</p></div>
#[repr(C)]
pub struct QEventLoopLocker {
    _unused: u8,
}
impl QEventLoopLocker {
    /// <p>Creates an event locker operating on the <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QEventLoopLocker::QEventLoopLocker()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventlooplocker.html#QEventLoopLocker">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates an event locker operating on the <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a>.</p>
    /// <p>The application will quit when there are no more QEventLoopLockers operating on it.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#quit">QCoreApplication::quit</a>() and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#quitLockEnabled-prop">QCoreApplication::isQuitLockEnabled</a>().</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QEventLoopLocker> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEventLoopLocker_QEventLoopLocker();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates an event locker operating on the <i>loop</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QEventLoopLocker::QEventLoopLocker(QEventLoop* loop)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventlooplocker.html#QEventLoopLocker-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates an event locker operating on the <i>loop</i>.</p>
    /// <p>This particular <a href="http://doc.qt.io/qt-5/qeventloop.html">QEventLoop</a> will quit when there are no more QEventLoopLockers operating on it.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qeventloop.html#quit">QEventLoop::quit</a>().</p></div>
    pub unsafe fn from_q_event_loop(
        loop_: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QEventLoop>>,
    ) -> ::cpp_utils::CppBox<crate::QEventLoopLocker> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEventLoopLocker_QEventLoopLocker1(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QEventLoop>>::cast_into(loop_)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates an event locker operating on the <i>thread</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QEventLoopLocker::QEventLoopLocker(QThread* thread)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventlooplocker.html#QEventLoopLocker-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates an event locker operating on the <i>thread</i>.</p>
    /// <p>This particular <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a> will quit when there are no more QEventLoopLockers operating on it.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#quit">QThread::quit</a>().</p></div>
    pub unsafe fn from_q_thread(
        thread: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QThread>>,
    ) -> ::cpp_utils::CppBox<crate::QEventLoopLocker> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEventLoopLocker_QEventLoopLocker2(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QThread>>::cast_into(thread)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_abstract_event_dispatcher {
    //! C++ type: <span style='color: green;'>```QAbstractEventDispatcher```</span>

    /// <p>This struct represents information about a timer: <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html#timerId-var">timerId</a>, <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html#interval-var">interval</a>, and <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html#timerType-var">timerType</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QAbstractEventDispatcher::TimerInfo```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This struct represents information about a timer: <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html#timerId-var">timerId</a>, <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html#interval-var">interval</a>, and <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html#timerType-var">timerType</a>.</p></div>
    #[repr(C)]
    pub struct TimerInfo {
        _unused: u8,
    }
    impl TimerInfo {
        /// <p>This struct represents information about a timer: <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html#timerId-var">timerId</a>, <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html#interval-var">interval</a>, and <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html#timerType-var">timerType</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QAbstractEventDispatcher::TimerInfo& QAbstractEventDispatcher::TimerInfo::operator=(const QAbstractEventDispatcher::TimerInfo& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This struct represents information about a timer: <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html#timerId-var">timerId</a>, <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html#interval-var">interval</a>, and <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html#timerType-var">timerType</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_abstract_event_dispatcher::TimerInfo> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_TimerInfo_operator_(
                    self as *mut crate::q_abstract_event_dispatcher::TimerInfo,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// Returns the value of the <span style='color: green;'>```interval```</span> field.
        pub unsafe fn interval(&self) -> ::std::os::raw::c_int {
            crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_TimerInfo_interval(
                self as *const crate::q_abstract_event_dispatcher::TimerInfo,
            )
        }

        /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html">TimerInfo</a> struct with the given <i>timerId</i>, <i>interval</i>, and <i>timerType</i>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QAbstractEventDispatcher::TimerInfo::TimerInfo(int id, int i, Qt::TimerType t)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html#TimerInfo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html">TimerInfo</a> struct with the given <i>timerId</i>, <i>interval</i>, and <i>timerType</i>.</p></div>
        pub unsafe fn new(
            id: ::std::os::raw::c_int,
            i: ::std::os::raw::c_int,
            t: crate::TimerType,
        ) -> ::cpp_utils::CppBox<crate::q_abstract_event_dispatcher::TimerInfo> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_TimerInfo_TimerInfo(
                    id, i, t,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This struct represents information about a timer: <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html#timerId-var">timerId</a>, <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html#interval-var">interval</a>, and <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html#timerType-var">timerType</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QAbstractEventDispatcher::TimerInfo::TimerInfo(const QAbstractEventDispatcher::TimerInfo& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This struct represents information about a timer: <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html#timerId-var">timerId</a>, <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html#interval-var">interval</a>, and <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html#timerType-var">timerType</a>.</p></div>
        pub unsafe fn new_copy(
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_abstract_event_dispatcher::TimerInfo> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_TimerInfo_TimerInfo2(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Sets the value of the <span style='color: green;'>```interval```</span> field.
        pub unsafe fn set_interval(&mut self, value: ::std::os::raw::c_int) {
            crate::__ffi::ctr_qt_core_ffi_set_QAbstractEventDispatcher_TimerInfo_interval(
                self as *mut crate::q_abstract_event_dispatcher::TimerInfo,
                value,
            )
        }

        /// Sets the value of the <span style='color: green;'>```timerId```</span> field.
        pub unsafe fn set_timer_id(&mut self, value: ::std::os::raw::c_int) {
            crate::__ffi::ctr_qt_core_ffi_set_QAbstractEventDispatcher_TimerInfo_timerId(
                self as *mut crate::q_abstract_event_dispatcher::TimerInfo,
                value,
            )
        }

        /// Sets the value of the <span style='color: green;'>```timerType```</span> field.
        pub unsafe fn set_timer_type(&mut self, value: crate::TimerType) {
            crate::__ffi::ctr_qt_core_ffi_set_QAbstractEventDispatcher_TimerInfo_timerType(
                self as *mut crate::q_abstract_event_dispatcher::TimerInfo,
                value,
            )
        }

        /// Returns the value of the <span style='color: green;'>```timerId```</span> field.
        pub unsafe fn timer_id(&self) -> ::std::os::raw::c_int {
            crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_TimerInfo_timerId(
                self as *const crate::q_abstract_event_dispatcher::TimerInfo,
            )
        }

        /// Returns the value of the <span style='color: green;'>```timerType```</span> field.
        pub unsafe fn timer_type(&self) -> crate::TimerType {
            crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_TimerInfo_timerType(
                self as *const crate::q_abstract_event_dispatcher::TimerInfo,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_abstract_event_dispatcher::TimerInfo {
        /// <p>This struct represents information about a timer: <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html#timerId-var">timerId</a>, <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html#interval-var">interval</a>, and <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html#timerType-var">timerType</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QAbstractEventDispatcher::TimerInfo::~TimerInfo()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This struct represents information about a timer: <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html#timerId-var">timerId</a>, <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html#interval-var">interval</a>, and <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html#timerType-var">timerType</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_TimerInfo_dTimerInfo(
                self as *mut crate::q_abstract_event_dispatcher::TimerInfo,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html">QAbstractEventDispatcher</a> class provides an interface to manage Qt's event queue.</p>
///
/// C++ class: <span style='color: green;'>```QAbstractEventDispatcher```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html">QAbstractEventDispatcher</a> class provides an interface to manage Qt's event queue.</p>
/// <p>An event dispatcher receives events from the window system and other sources. It then sends them to the <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> or <a href="http://doc.qt.io/qt-5/qapplication.html">QApplication</a> instance for processing and delivery. <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html">QAbstractEventDispatcher</a> provides fine-grained control over event delivery.</p>
/// <p>For simple control of event processing use <a href="http://doc.qt.io/qt-5/qcoreapplication.html#processEvents">QCoreApplication::processEvents</a>().</p>
/// <p>For finer control of the application's event loop, call <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#instance">instance</a>() and call functions on the <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html">QAbstractEventDispatcher</a> object that is returned. If you want to use your own instance of <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html">QAbstractEventDispatcher</a> or of a <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html">QAbstractEventDispatcher</a> subclass, you must install it with <a href="http://doc.qt.io/qt-5/qcoreapplication.html#setEventDispatcher">QCoreApplication::setEventDispatcher</a>() or <a href="http://doc.qt.io/qt-5/qthread.html#setEventDispatcher">QThread::setEventDispatcher</a>() <i>before</i> a default event dispatcher has been installed.</p>
/// <p>The main event loop is started by calling <a href="http://doc.qt.io/qt-5/qcoreapplication.html#exec">QCoreApplication::exec</a>(), and stopped by calling <a href="http://doc.qt.io/qt-5/qcoreapplication.html#exit">QCoreApplication::exit</a>(). Local event loops can be created using <a href="http://doc.qt.io/qt-5/qeventloop.html">QEventLoop</a>.</p>
/// <p>Programs that perform long operations can call <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#processEvents">processEvents</a>() with a bitwise OR combination of various <a href="http://doc.qt.io/qt-5/qeventloop.html#ProcessEventsFlag-enum">QEventLoop::ProcessEventsFlag</a> values to control which events should be delivered.</p>
/// <p><a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html">QAbstractEventDispatcher</a> also allows the integration of an external event loop with the Qt event loop.</p></div>
#[repr(C)]
pub struct QAbstractEventDispatcher {
    _unused: u8,
}
impl QAbstractEventDispatcher {
    /// <p>This signal is emitted before the event loop calls a function that could block.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractEventDispatcher::aboutToBlock` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#aboutToBlock">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted before the event loop calls a function that could block.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#awake">awake</a>().</p></div>
    pub fn about_to_block(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2aboutToBlock()\0"),
            )
        }
    }

    /// <p>This signal is emitted after the event loop returns from a function that could block.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractEventDispatcher::awake` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#awake">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted after the event loop returns from a function that could block.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#wakeUp">wakeUp</a>() and <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#aboutToBlock">aboutToBlock</a>().</p></div>
    pub fn awake(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2awake()\0"),
            )
        }
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void QAbstractEventDispatcher::closingDown()```</span>.
    pub unsafe fn closing_down(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_closingDown(
            self as *mut crate::QAbstractEventDispatcher,
        )
    }

    /// <p>Sends <i>message</i> through the event filters that were set by <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#installNativeEventFilter">installNativeEventFilter</a>(). This function returns <code>true</code> as soon as an event filter returns <code>true</code>, and false otherwise to indicate that the processing of the event should continue.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QAbstractEventDispatcher::filterNativeEvent(const QByteArray& eventType, void* message, long* result)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#filterNativeEvent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sends <i>message</i> through the event filters that were set by <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#installNativeEventFilter">installNativeEventFilter</a>(). This function returns <code>true</code> as soon as an event filter returns <code>true</code>, and false otherwise to indicate that the processing of the event should continue.</p>
    /// <p>Subclasses of <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html">QAbstractEventDispatcher</a> <i>must</i> call this function for <i>all</i> messages received from the system to ensure compatibility with any extensions that may be used in the application. The type of event <i>eventType</i> is specific to the platform plugin chosen at run-time, and can be used to cast message to the right type. The <i>result</i> pointer is only used on Windows, and corresponds to the LRESULT pointer.</p>
    /// <p>Note that the type of <i>message</i> is platform dependent. See <a href="http://doc.qt.io/qt-5/qabstractnativeeventfilter.html">QAbstractNativeEventFilter</a> for details.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#installNativeEventFilter">installNativeEventFilter</a>() and <a href="http://doc.qt.io/qt-5/qabstractnativeeventfilter.html#nativeEventFilter">QAbstractNativeEventFilter::nativeEventFilter</a>().</p></div>
    pub unsafe fn filter_native_event(
        &mut self,
        event_type: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        message: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
        result: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_long>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_filterNativeEvent(
            self as *mut crate::QAbstractEventDispatcher,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(event_type)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(message)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_long>>::cast_into(result)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Depending from the event dispatcher implementation does nothing or calls QApplication::sendPostedEvents().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual void QAbstractEventDispatcher::flush()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-obsolete.html#flush">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Depending from the event dispatcher implementation does nothing or calls QApplication::sendPostedEvents().</p></div>
    pub unsafe fn flush(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_flush(
            self as *mut crate::QAbstractEventDispatcher,
        )
    }

    /// <p>Returns <code>true</code> if there is an event waiting; otherwise returns false. This function is an implementation detail for <a href="http://doc.qt.io/qt-5/qcoreapplication-obsolete.html#hasPendingEvents">QCoreApplication::hasPendingEvents</a>() and must not be called directly.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual bool QAbstractEventDispatcher::hasPendingEvents()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-obsolete.html#hasPendingEvents">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if there is an event waiting; otherwise returns false. This function is an implementation detail for <a href="http://doc.qt.io/qt-5/qcoreapplication-obsolete.html#hasPendingEvents">QCoreApplication::hasPendingEvents</a>() and must not be called directly.</p></div>
    pub unsafe fn has_pending_events(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_hasPendingEvents(
            self as *mut crate::QAbstractEventDispatcher,
        )
    }

    /// <p>Installs an event filter <i>filterObj</i> for all native event filters received by the application.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QAbstractEventDispatcher::installNativeEventFilter(QAbstractNativeEventFilter* filterObj)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#installNativeEventFilter">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Installs an event filter <i>filterObj</i> for all native event filters received by the application.</p>
    /// <p>The event filter <i>filterObj</i> receives events via its <a href="http://doc.qt.io/qt-5/qabstractnativeeventfilter.html#nativeEventFilter">nativeEventFilter()</a> function, which is called for all events received by all threads.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qabstractnativeeventfilter.html#nativeEventFilter">nativeEventFilter()</a> function should return true if the event should be filtered, (in this case, stopped). It should return false to allow normal Qt processing to continue: the native event can then be translated into a <a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a> and handled by the standard Qt <a href="http://doc.qt.io/qt-5/qevent.html">event</a> filtering, e.g. <a href="http://doc.qt.io/qt-5/qobject.html#installEventFilter">QObject::installEventFilter</a>().</p>
    /// <p>If multiple event filters are installed, the filter that was installed last is activated first.</p>
    /// <p><b>Note: </b>The filter function set here receives native messages, that is, MSG or XEvent structs.</p><p>For maximum portability, you should always try to use <a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a> objects and <a href="http://doc.qt.io/qt-5/qobject.html#installEventFilter">QObject::installEventFilter</a>() whenever possible.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#installEventFilter">QObject::installEventFilter</a>().</p></div>
    pub unsafe fn install_native_event_filter(
        &mut self,
        filter_obj: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractNativeEventFilter>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_installNativeEventFilter(self as *mut crate::QAbstractEventDispatcher, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractNativeEventFilter>>::cast_into(filter_obj).as_mut_raw_ptr())
    }

    /// <p>Returns a pointer to the event dispatcher object for the specified <i>thread</i>. If <i>thread</i> is zero, the current thread is used. If no event dispatcher exists for the specified thread, this function returns 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QAbstractEventDispatcher* QAbstractEventDispatcher::instance(QThread* thread = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#instance">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the event dispatcher object for the specified <i>thread</i>. If <i>thread</i> is zero, the current thread is used. If no event dispatcher exists for the specified thread, this function returns 0.</p>
    /// <p><b>Note:</b> If Qt is built without thread support, the <i>thread</i> argument is ignored.</p></div>
    pub unsafe fn instance_1a(
        thread: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QThread>>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractEventDispatcher> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_instance(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QThread>>::cast_into(thread)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns a pointer to the event dispatcher object for the specified <i>thread</i>. If <i>thread</i> is zero, the current thread is used. If no event dispatcher exists for the specified thread, this function returns 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QAbstractEventDispatcher* QAbstractEventDispatcher::instance()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#instance">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the event dispatcher object for the specified <i>thread</i>. If <i>thread</i> is zero, the current thread is used. If no event dispatcher exists for the specified thread, this function returns 0.</p>
    /// <p><b>Note:</b> If Qt is built without thread support, the <i>thread</i> argument is ignored.</p></div>
    pub unsafe fn instance_0a() -> ::cpp_utils::MutPtr<crate::QAbstractEventDispatcher> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_instance1();
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Interrupts event dispatching. The event dispatcher will return from <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#processEvents">processEvents</a>() as soon as possible.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual void QAbstractEventDispatcher::interrupt()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#interrupt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Interrupts event dispatching. The event dispatcher will return from <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#processEvents">processEvents</a>() as soon as possible.</p></div>
    pub unsafe fn interrupt(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_interrupt(
            self as *mut crate::QAbstractEventDispatcher,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QAbstractEventDispatcher::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_metaObject(
            self as *const crate::QAbstractEventDispatcher,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Processes pending events that match <i>flags</i> until there are no more events to process. Returns <code>true</code> if an event was processed; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual bool QAbstractEventDispatcher::processEvents(QFlags<QEventLoop::ProcessEventsFlag> flags)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#processEvents">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Processes pending events that match <i>flags</i> until there are no more events to process. Returns <code>true</code> if an event was processed; otherwise returns <code>false</code>.</p>
    /// <p>This function is especially useful if you have a long running operation, and want to show its progress without allowing user input by using the <a href="http://doc.qt.io/qt-5/qeventloop.html#ProcessEventsFlag-enum">QEventLoop::ExcludeUserInputEvents</a> flag.</p>
    /// <p>If the <a href="http://doc.qt.io/qt-5/qeventloop.html#ProcessEventsFlag-enum">QEventLoop::WaitForMoreEvents</a> flag is set in <i>flags</i>, the behavior of this function is as follows:</p>
    /// <ul>
    /// <li>If events are available, this function returns after processing them.</li>
    /// <li>If no events are available, this function will wait until more are available and return after processing newly available events.</li>
    /// </ul>
    /// <p>If the <a href="http://doc.qt.io/qt-5/qeventloop.html#ProcessEventsFlag-enum">QEventLoop::WaitForMoreEvents</a> flag is not set in <i>flags</i>, and no events are available, this function will return immediately.</p>
    /// <p><b>Note:</b> This function does not process events continuously; it returns after all available events are processed.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-obsolete.html#hasPendingEvents">hasPendingEvents</a>().</p></div>
    pub unsafe fn process_events(
        &mut self,
        flags: crate::QFlags<crate::q_event_loop::ProcessEventsFlag>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_processEvents(
            self as *mut crate::QAbstractEventDispatcher,
            flags.to_int(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QAbstractEventDispatcher::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_qt_metacall(
            self as *mut crate::QAbstractEventDispatcher,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QAbstractEventDispatcher::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_qt_metacast(
            self as *mut crate::QAbstractEventDispatcher,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Registers <i>notifier</i> with the event loop. Subclasses must implement this method to tie a socket notifier into another event loop.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual void QAbstractEventDispatcher::registerSocketNotifier(QSocketNotifier* notifier)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#registerSocketNotifier">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Registers <i>notifier</i> with the event loop. Subclasses must implement this method to tie a socket notifier into another event loop.</p></div>
    pub unsafe fn register_socket_notifier(
        &mut self,
        notifier: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QSocketNotifier>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_registerSocketNotifier(
            self as *mut crate::QAbstractEventDispatcher,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QSocketNotifier>>::cast_into(
                notifier,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Register a timer with the specified <i>timerId</i> and <i>interval</i> for the given <i>object</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QAbstractEventDispatcher::registerTimer(int interval, Qt::TimerType timerType, QObject* object)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-obsolete.html#registerTimer-1">C++ documentation</a> for <span style='color: green;'>```void QAbstractEventDispatcher::registerTimer(int timerId, int interval, QObject *object)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Register a timer with the specified <i>timerId</i> and <i>interval</i> for the given <i>object</i>.</p></div>
    pub unsafe fn register_timer_3a(
        &mut self,
        interval: ::std::os::raw::c_int,
        timer_type: crate::TimerType,
        object: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_registerTimer(
            self as *mut crate::QAbstractEventDispatcher,
            interval,
            timer_type,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(object)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Register a timer with the specified <i>timerId</i> and <i>interval</i> for the given <i>object</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual void QAbstractEventDispatcher::registerTimer(int timerId, int interval, Qt::TimerType timerType, QObject* object)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-obsolete.html#registerTimer-1">C++ documentation</a> for <span style='color: green;'>```void QAbstractEventDispatcher::registerTimer(int timerId, int interval, QObject *object)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Register a timer with the specified <i>timerId</i> and <i>interval</i> for the given <i>object</i>.</p></div>
    pub unsafe fn register_timer_4a(
        &mut self,
        timer_id: ::std::os::raw::c_int,
        interval: ::std::os::raw::c_int,
        timer_type: crate::TimerType,
        object: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_registerTimer1(
            self as *mut crate::QAbstractEventDispatcher,
            timer_id,
            interval,
            timer_type,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(object)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns a list of registered timers for <i>object</i>. The <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html">TimerInfo</a> struct has <code>timerId</code>, <code>interval</code>, and <code>timerType</code> members.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual QList<QAbstractEventDispatcher::TimerInfo> QAbstractEventDispatcher::registeredTimers(QObject* object) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#registeredTimers">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of registered timers for <i>object</i>. The <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-timerinfo.html">TimerInfo</a> struct has <code>timerId</code>, <code>interval</code>, and <code>timerType</code> members.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qt.html#TimerType-enum">Qt::TimerType</a>.</p></div>
    pub unsafe fn registered_timers(
        &self,
        object: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfTimerInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_registeredTimers(
            self as *const crate::QAbstractEventDispatcher,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(object)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the remaining time in milliseconds with the given <i>timerId</i>. If the timer is inactive, the returned value will be -1. If the timer is overdue, the returned value will be 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual int QAbstractEventDispatcher::remainingTime(int timerId)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#remainingTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the remaining time in milliseconds with the given <i>timerId</i>. If the timer is inactive, the returned value will be -1. If the timer is overdue, the returned value will be 0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qt.html#TimerType-enum">Qt::TimerType</a>.</p></div>
    pub unsafe fn remaining_time(
        &mut self,
        timer_id: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_remainingTime(
            self as *mut crate::QAbstractEventDispatcher,
            timer_id,
        )
    }

    /// <p>Removes the event filter <i>filter</i> from this object. The request is ignored if such an event filter has not been installed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QAbstractEventDispatcher::removeNativeEventFilter(QAbstractNativeEventFilter* filterObj)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#removeNativeEventFilter">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the event filter <i>filter</i> from this object. The request is ignored if such an event filter has not been installed.</p>
    /// <p>All event filters for this object are automatically removed when this object is destroyed.</p>
    /// <p>It is always safe to remove an event filter, even during event filter filter activation (that is, even from within the <a href="http://doc.qt.io/qt-5/qabstractnativeeventfilter.html#nativeEventFilter">nativeEventFilter()</a> function).</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#installNativeEventFilter">installNativeEventFilter</a>() and <a href="http://doc.qt.io/qt-5/qabstractnativeeventfilter.html">QAbstractNativeEventFilter</a>.</p></div>
    pub unsafe fn remove_native_event_filter(
        &mut self,
        filter_obj: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractNativeEventFilter>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_removeNativeEventFilter(self as *mut crate::QAbstractEventDispatcher, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractNativeEventFilter>>::cast_into(filter_obj).as_mut_raw_ptr())
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void QAbstractEventDispatcher::startingUp()```</span>.
    pub unsafe fn starting_up(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_startingUp(
            self as *mut crate::QAbstractEventDispatcher,
        )
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QAbstractEventDispatcher::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QAbstractEventDispatcher::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Unregisters <i>notifier</i> from the event dispatcher. Subclasses must reimplement this method to tie a socket notifier into another event loop. Reimplementations must call the base implementation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual void QAbstractEventDispatcher::unregisterSocketNotifier(QSocketNotifier* notifier)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#unregisterSocketNotifier">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Unregisters <i>notifier</i> from the event dispatcher. Subclasses must reimplement this method to tie a socket notifier into another event loop. Reimplementations must call the base implementation.</p></div>
    pub unsafe fn unregister_socket_notifier(
        &mut self,
        notifier: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QSocketNotifier>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_unregisterSocketNotifier(
            self as *mut crate::QAbstractEventDispatcher,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QSocketNotifier>>::cast_into(
                notifier,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Unregisters the timer with the given <i>timerId</i>. Returns <code>true</code> if successful; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual bool QAbstractEventDispatcher::unregisterTimer(int timerId)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#unregisterTimer">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Unregisters the timer with the given <i>timerId</i>. Returns <code>true</code> if successful; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#registerTimer">registerTimer</a>() and <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#unregisterTimers">unregisterTimers</a>().</p></div>
    pub unsafe fn unregister_timer(&mut self, timer_id: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_unregisterTimer(
            self as *mut crate::QAbstractEventDispatcher,
            timer_id,
        )
    }

    /// <p>Unregisters all the timers associated with the given <i>object</i>. Returns <code>true</code> if all timers were successful removed; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual bool QAbstractEventDispatcher::unregisterTimers(QObject* object)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#unregisterTimers">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Unregisters all the timers associated with the given <i>object</i>. Returns <code>true</code> if all timers were successful removed; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#unregisterTimer">unregisterTimer</a>() and <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#registeredTimers">registeredTimers</a>().</p></div>
    pub unsafe fn unregister_timers(
        &mut self,
        object: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_unregisterTimers(
            self as *mut crate::QAbstractEventDispatcher,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(object)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Wakes up the event loop.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual void QAbstractEventDispatcher::wakeUp()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#wakeUp">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Wakes up the event loop.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#awake">awake</a>().</p></div>
    pub unsafe fn wake_up(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_wakeUp(
            self as *mut crate::QAbstractEventDispatcher,
        )
    }
}

pub mod q_variant {
    //! C++ type: <span style='color: green;'>```QVariant```</span>

    /// <p>This enum type defines the types of variable that a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> can contain.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QVariant::Type```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant-obsolete.html#Type-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type defines the types of variable that a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> can contain.</p>
    ///
    ///
    /// <h2>Related Non-Members</h2></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Type(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Type {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Type(value)
        }
    }

    impl From<Type> for ::std::os::raw::c_int {
        fn from(value: Type) -> Self {
            value.0
        }
    }

    impl Type {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Type {
        /// no type (C++ enum variant: <span style='color: green;'>```Invalid = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const Invalid: crate::q_variant::Type = crate::q_variant::Type(0);
        /// a bool (C++ enum variant: <span style='color: green;'>```Bool = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Bool: crate::q_variant::Type = crate::q_variant::Type(1);
        /// an int (C++ enum variant: <span style='color: green;'>```Int = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const Int: crate::q_variant::Type = crate::q_variant::Type(2);
        /// a <a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a> (C++ enum variant: <span style='color: green;'>```UInt = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const UInt: crate::q_variant::Type = crate::q_variant::Type(3);
        /// a <a href="http://doc.qt.io/qt-5/qtglobal.html#qlonglong-typedef">qlonglong</a> (C++ enum variant: <span style='color: green;'>```LongLong = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const LongLong: crate::q_variant::Type = crate::q_variant::Type(4);
        /// a <a href="http://doc.qt.io/qt-5/qtglobal.html#qulonglong-typedef">qulonglong</a> (C++ enum variant: <span style='color: green;'>```ULongLong = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const ULongLong: crate::q_variant::Type = crate::q_variant::Type(5);
        /// a double (C++ enum variant: <span style='color: green;'>```Double = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const Double: crate::q_variant::Type = crate::q_variant::Type(6);
        /// a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> (C++ enum variant: <span style='color: green;'>```Char = 7```</span>)
        #[allow(non_upper_case_globals)]
        pub const Char: crate::q_variant::Type = crate::q_variant::Type(7);
        /// a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantMap-typedef">QVariantMap</a> (C++ enum variant: <span style='color: green;'>```Map = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const Map: crate::q_variant::Type = crate::q_variant::Type(8);
        /// a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a> (C++ enum variant: <span style='color: green;'>```List = 9```</span>)
        #[allow(non_upper_case_globals)]
        pub const List: crate::q_variant::Type = crate::q_variant::Type(9);
        /// a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> (C++ enum variant: <span style='color: green;'>```String = 10```</span>)
        #[allow(non_upper_case_globals)]
        pub const String: crate::q_variant::Type = crate::q_variant::Type(10);
        /// a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> (C++ enum variant: <span style='color: green;'>```StringList = 11```</span>)
        #[allow(non_upper_case_globals)]
        pub const StringList: crate::q_variant::Type = crate::q_variant::Type(11);
        /// a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> (C++ enum variant: <span style='color: green;'>```ByteArray = 12```</span>)
        #[allow(non_upper_case_globals)]
        pub const ByteArray: crate::q_variant::Type = crate::q_variant::Type(12);
        /// a <a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a> (C++ enum variant: <span style='color: green;'>```BitArray = 13```</span>)
        #[allow(non_upper_case_globals)]
        pub const BitArray: crate::q_variant::Type = crate::q_variant::Type(13);
        /// a <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> (C++ enum variant: <span style='color: green;'>```Date = 14```</span>)
        #[allow(non_upper_case_globals)]
        pub const Date: crate::q_variant::Type = crate::q_variant::Type(14);
        /// a <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> (C++ enum variant: <span style='color: green;'>```Time = 15```</span>)
        #[allow(non_upper_case_globals)]
        pub const Time: crate::q_variant::Type = crate::q_variant::Type(15);
        /// a <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> (C++ enum variant: <span style='color: green;'>```DateTime = 16```</span>)
        #[allow(non_upper_case_globals)]
        pub const DateTime: crate::q_variant::Type = crate::q_variant::Type(16);
        /// a <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> (C++ enum variant: <span style='color: green;'>```Url = 17```</span>)
        #[allow(non_upper_case_globals)]
        pub const Url: crate::q_variant::Type = crate::q_variant::Type(17);
        /// a <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> (C++ enum variant: <span style='color: green;'>```Locale = 18```</span>)
        #[allow(non_upper_case_globals)]
        pub const Locale: crate::q_variant::Type = crate::q_variant::Type(18);
        /// a <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> (C++ enum variant: <span style='color: green;'>```Rect = 19```</span>)
        #[allow(non_upper_case_globals)]
        pub const Rect: crate::q_variant::Type = crate::q_variant::Type(19);
        /// a <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> (C++ enum variant: <span style='color: green;'>```RectF = 20```</span>)
        #[allow(non_upper_case_globals)]
        pub const RectF: crate::q_variant::Type = crate::q_variant::Type(20);
        /// a <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a> (C++ enum variant: <span style='color: green;'>```Size = 21```</span>)
        #[allow(non_upper_case_globals)]
        pub const Size: crate::q_variant::Type = crate::q_variant::Type(21);
        /// a <a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a> (C++ enum variant: <span style='color: green;'>```SizeF = 22```</span>)
        #[allow(non_upper_case_globals)]
        pub const SizeF: crate::q_variant::Type = crate::q_variant::Type(22);
        /// a <a href="http://doc.qt.io/qt-5/qline.html">QLine</a> (C++ enum variant: <span style='color: green;'>```Line = 23```</span>)
        #[allow(non_upper_case_globals)]
        pub const Line: crate::q_variant::Type = crate::q_variant::Type(23);
        /// a <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> (C++ enum variant: <span style='color: green;'>```LineF = 24```</span>)
        #[allow(non_upper_case_globals)]
        pub const LineF: crate::q_variant::Type = crate::q_variant::Type(24);
        /// a <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> (C++ enum variant: <span style='color: green;'>```Point = 25```</span>)
        #[allow(non_upper_case_globals)]
        pub const Point: crate::q_variant::Type = crate::q_variant::Type(25);
        /// a <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> (C++ enum variant: <span style='color: green;'>```PointF = 26```</span>)
        #[allow(non_upper_case_globals)]
        pub const PointF: crate::q_variant::Type = crate::q_variant::Type(26);
        /// a <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> (C++ enum variant: <span style='color: green;'>```RegExp = 27```</span>)
        #[allow(non_upper_case_globals)]
        pub const RegExp: crate::q_variant::Type = crate::q_variant::Type(27);
        /// a <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> (C++ enum variant: <span style='color: green;'>```RegularExpression = 44```</span>)
        #[allow(non_upper_case_globals)]
        pub const RegularExpression: crate::q_variant::Type = crate::q_variant::Type(44);
        /// a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantHash-typedef">QVariantHash</a> (C++ enum variant: <span style='color: green;'>```Hash = 28```</span>)
        #[allow(non_upper_case_globals)]
        pub const Hash: crate::q_variant::Type = crate::q_variant::Type(28);
        /// a <a href="http://doc.qt.io/qt-5/qeasingcurve.html">QEasingCurve</a> (C++ enum variant: <span style='color: green;'>```EasingCurve = 29```</span>)
        #[allow(non_upper_case_globals)]
        pub const EasingCurve: crate::q_variant::Type = crate::q_variant::Type(29);
        /// a <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> (C++ enum variant: <span style='color: green;'>```Uuid = 30```</span>)
        #[allow(non_upper_case_globals)]
        pub const Uuid: crate::q_variant::Type = crate::q_variant::Type(30);
        /// a <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> (C++ enum variant: <span style='color: green;'>```ModelIndex = 42```</span>)
        #[allow(non_upper_case_globals)]
        pub const ModelIndex: crate::q_variant::Type = crate::q_variant::Type(42);
        /// a <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html">QPersistentModelIndex</a> (since 5.5) (C++ enum variant: <span style='color: green;'>```PersistentModelIndex = 50```</span>)
        #[allow(non_upper_case_globals)]
        pub const PersistentModelIndex: crate::q_variant::Type = crate::q_variant::Type(50);
        /// C++ enum variant: <span style='color: green;'>```LastCoreType = 51```</span>
        #[allow(non_upper_case_globals)]
        pub const LastCoreType: crate::q_variant::Type = crate::q_variant::Type(51);
        /// a <a href="http://doc.qt.io/qt-5/qfont.html">QFont</a> (C++ enum variant: <span style='color: green;'>```Font = 64```</span>)
        #[allow(non_upper_case_globals)]
        pub const Font: crate::q_variant::Type = crate::q_variant::Type(64);
        /// a <a href="http://doc.qt.io/qt-5/qpixmap.html">QPixmap</a> (C++ enum variant: <span style='color: green;'>```Pixmap = 65```</span>)
        #[allow(non_upper_case_globals)]
        pub const Pixmap: crate::q_variant::Type = crate::q_variant::Type(65);
        /// a <a href="http://doc.qt.io/qt-5/qbrush.html">QBrush</a> (C++ enum variant: <span style='color: green;'>```Brush = 66```</span>)
        #[allow(non_upper_case_globals)]
        pub const Brush: crate::q_variant::Type = crate::q_variant::Type(66);
        /// a <a href="http://doc.qt.io/qt-5/qcolor.html">QColor</a> (C++ enum variant: <span style='color: green;'>```Color = 67```</span>)
        #[allow(non_upper_case_globals)]
        pub const Color: crate::q_variant::Type = crate::q_variant::Type(67);
        /// a <a href="http://doc.qt.io/qt-5/qpalette.html">QPalette</a> (C++ enum variant: <span style='color: green;'>```Palette = 68```</span>)
        #[allow(non_upper_case_globals)]
        pub const Palette: crate::q_variant::Type = crate::q_variant::Type(68);
        /// a <a href="http://doc.qt.io/qt-5/qimage.html">QImage</a> (C++ enum variant: <span style='color: green;'>```Image = 70```</span>)
        #[allow(non_upper_case_globals)]
        pub const Image: crate::q_variant::Type = crate::q_variant::Type(70);
        /// a <a href="http://doc.qt.io/qt-5/qpolygon.html">QPolygon</a> (C++ enum variant: <span style='color: green;'>```Polygon = 71```</span>)
        #[allow(non_upper_case_globals)]
        pub const Polygon: crate::q_variant::Type = crate::q_variant::Type(71);
        /// a <a href="http://doc.qt.io/qt-5/qregion.html">QRegion</a> (C++ enum variant: <span style='color: green;'>```Region = 72```</span>)
        #[allow(non_upper_case_globals)]
        pub const Region: crate::q_variant::Type = crate::q_variant::Type(72);
        /// a <a href="http://doc.qt.io/qt-5/qbitmap.html">QBitmap</a> (C++ enum variant: <span style='color: green;'>```Bitmap = 73```</span>)
        #[allow(non_upper_case_globals)]
        pub const Bitmap: crate::q_variant::Type = crate::q_variant::Type(73);
        /// a <a href="http://doc.qt.io/qt-5/qcursor.html">QCursor</a> (C++ enum variant: <span style='color: green;'>```Cursor = 74```</span>)
        #[allow(non_upper_case_globals)]
        pub const Cursor: crate::q_variant::Type = crate::q_variant::Type(74);
        /// a <a href="http://doc.qt.io/qt-5/qkeysequence.html">QKeySequence</a> (C++ enum variant: <span style='color: green;'>```KeySequence = 75```</span>)
        #[allow(non_upper_case_globals)]
        pub const KeySequence: crate::q_variant::Type = crate::q_variant::Type(75);
        /// a <a href="http://doc.qt.io/qt-5/qpen.html">QPen</a> (C++ enum variant: <span style='color: green;'>```Pen = 76```</span>)
        #[allow(non_upper_case_globals)]
        pub const Pen: crate::q_variant::Type = crate::q_variant::Type(76);
        /// a <a href="http://doc.qt.io/qt-5/qtextlength.html">QTextLength</a> (C++ enum variant: <span style='color: green;'>```TextLength = 77```</span>)
        #[allow(non_upper_case_globals)]
        pub const TextLength: crate::q_variant::Type = crate::q_variant::Type(77);
        /// a <a href="http://doc.qt.io/qt-5/qtextformat.html">QTextFormat</a> (C++ enum variant: <span style='color: green;'>```TextFormat = 78```</span>)
        #[allow(non_upper_case_globals)]
        pub const TextFormat: crate::q_variant::Type = crate::q_variant::Type(78);
        /// a <a href="http://doc.qt.io/qt-5/qmatrix.html">QMatrix</a> (C++ enum variant: <span style='color: green;'>```Matrix = 79```</span>)
        #[allow(non_upper_case_globals)]
        pub const Matrix: crate::q_variant::Type = crate::q_variant::Type(79);
        /// a <a href="http://doc.qt.io/qt-5/qtransform.html">QTransform</a> (C++ enum variant: <span style='color: green;'>```Transform = 80```</span>)
        #[allow(non_upper_case_globals)]
        pub const Transform: crate::q_variant::Type = crate::q_variant::Type(80);
        /// a <a href="http://doc.qt.io/qt-5/qmatrix4x4.html">QMatrix4x4</a> (C++ enum variant: <span style='color: green;'>```Matrix4x4 = 81```</span>)
        #[allow(non_upper_case_globals)]
        pub const Matrix4X4: crate::q_variant::Type = crate::q_variant::Type(81);
        /// a <a href="http://doc.qt.io/qt-5/qvector2d.html">QVector2D</a> (C++ enum variant: <span style='color: green;'>```Vector2D = 82```</span>)
        #[allow(non_upper_case_globals)]
        pub const Vector2D: crate::q_variant::Type = crate::q_variant::Type(82);
        /// a <a href="http://doc.qt.io/qt-5/qvector3d.html">QVector3D</a> (C++ enum variant: <span style='color: green;'>```Vector3D = 83```</span>)
        #[allow(non_upper_case_globals)]
        pub const Vector3D: crate::q_variant::Type = crate::q_variant::Type(83);
        /// a <a href="http://doc.qt.io/qt-5/qvector4d.html">QVector4D</a> (C++ enum variant: <span style='color: green;'>```Vector4D = 84```</span>)
        #[allow(non_upper_case_globals)]
        pub const Vector4D: crate::q_variant::Type = crate::q_variant::Type(84);
        /// a <a href="http://doc.qt.io/qt-5/qquaternion.html">QQuaternion</a> (C++ enum variant: <span style='color: green;'>```Quaternion = 85```</span>)
        #[allow(non_upper_case_globals)]
        pub const Quaternion: crate::q_variant::Type = crate::q_variant::Type(85);
        /// a <a href="http://doc.qt.io/qt-5/qpolygonf.html">QPolygonF</a> (C++ enum variant: <span style='color: green;'>```PolygonF = 86```</span>)
        #[allow(non_upper_case_globals)]
        pub const PolygonF: crate::q_variant::Type = crate::q_variant::Type(86);
        /// a <a href="http://doc.qt.io/qt-5/qicon.html">QIcon</a> (C++ enum variant: <span style='color: green;'>```Icon = 69```</span>)
        #[allow(non_upper_case_globals)]
        pub const Icon: crate::q_variant::Type = crate::q_variant::Type(69);
        /// C++ enum variant: <span style='color: green;'>```LastGuiType = 86```</span>
        #[allow(non_upper_case_globals)]
        pub const LastGuiType: crate::q_variant::Type = crate::q_variant::Type(86);
        /// a <a href="http://doc.qt.io/qt-5/qsizepolicy.html">QSizePolicy</a> (C++ enum variant: <span style='color: green;'>```SizePolicy = 121```</span>)
        #[allow(non_upper_case_globals)]
        pub const SizePolicy: crate::q_variant::Type = crate::q_variant::Type(121);
        /// Base value for user-defined types. (C++ enum variant: <span style='color: green;'>```UserType = 1024```</span>)
        #[allow(non_upper_case_globals)]
        pub const UserType: crate::q_variant::Type = crate::q_variant::Type(1024);
        /// C++ enum variant: <span style='color: green;'>```LastType = -1```</span>
        #[allow(non_upper_case_globals)]
        pub const LastType: crate::q_variant::Type = crate::q_variant::Type(-1);
        /// C++ enum variant: <span style='color: green;'>```LastCoreType = 55```</span>
        #[allow(non_upper_case_globals)]
        pub const LastCoreType2: crate::q_variant::Type = crate::q_variant::Type(55);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> class acts like a union for the most common Qt data types.</p>
///
/// C++ class: <span style='color: green;'>```QVariant```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qvariant.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> class acts like a union for the most common Qt data types.</p>
/// <p>Because C++ forbids unions from including types that have non-default constructors or destructors, most interesting Qt classes cannot be used in unions. Without <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>, this would be a problem for <a href="http://doc.qt.io/qt-5/qobject.html#property">QObject::property</a>() and for database work, etc.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> object holds a single value of a single <a href="http://doc.qt.io/qt-5/qvariant.html#type">type</a>() at a time. (Some <a href="http://doc.qt.io/qt-5/qvariant.html#type">type</a>()s are multi-valued, for example a string list.) You can find out what type, T, the variant holds, convert it to a different type using <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>(), get its value using one of the toT() functions (e.g., <a href="http://doc.qt.io/qt-5/qvariant.html#toSize">toSize</a>()) and check whether the type can be converted to a particular type using <a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>().</p>
/// <p>The methods named toT() (e.g., <a href="http://doc.qt.io/qt-5/qvariant.html#toInt">toInt</a>(), <a href="http://doc.qt.io/qt-5/qvariant.html#toString">toString</a>()) are const. If you ask for the stored type, they return a copy of the stored object. If you ask for a type that can be generated from the stored type, toT() copies and converts and leaves the object itself unchanged. If you ask for a type that cannot be generated from the stored type, the result depends on the type; see the function documentation for details.</p>
/// <p>Here is some example code to demonstrate the use of <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qdatastream.html">QDataStream</a></span> out(<span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>);
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariant">QVariant</a></span> v(<span class="number">123</span>);                <span class="comment">// The variant now contains an int</span>
///   <span class="type">int</span> x <span class="operator">=</span> v<span class="operator">.</span>toInt();              <span class="comment">// x = 123</span>
///   out <span class="operator">&lt;</span><span class="operator">&lt;</span> v;                       <span class="comment">// Writes a type tag and an int to out</span>
///   v <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariant">QVariant</a></span>(<span class="string">"hello"</span>);          <span class="comment">// The variant now contains a QByteArray</span>
///   v <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariant">QVariant</a></span>(tr(<span class="string">"hello"</span>));      <span class="comment">// The variant now contains a QString</span>
///   <span class="type">int</span> y <span class="operator">=</span> v<span class="operator">.</span>toInt();              <span class="comment">// y = 0 since v cannot be converted to an int</span>
///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> s <span class="operator">=</span> v<span class="operator">.</span>toString();       <span class="comment">// s = tr("hello")  (see QObject::tr())</span>
///   out <span class="operator">&lt;</span><span class="operator">&lt;</span> v;                       <span class="comment">// Writes a type tag and a QString to out</span>
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   <span class="type"><a href="http://doc.qt.io/qt-5/qdatastream.html">QDataStream</a></span> in(<span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>);            <span class="comment">// (opening the previously written stream)</span>
///   in <span class="operator">&gt;</span><span class="operator">&gt;</span> v;                        <span class="comment">// Reads an Int variant</span>
///   <span class="type">int</span> z <span class="operator">=</span> v<span class="operator">.</span>toInt();              <span class="comment">// z = 123</span>
///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>(<span class="string">"Type is %s"</span><span class="operator">,</span>            <span class="comment">// prints "Type is int"</span>
/// &#32;   &#32;     v<span class="operator">.</span>typeName());
///   v <span class="operator">=</span> v<span class="operator">.</span>toInt() <span class="operator">+</span> <span class="number">100</span>;            <span class="comment">// The variant now hold the value 223</span>
///   v <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariant">QVariant</a></span>(<span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span>());
///
/// </pre>
/// <p>You can even store <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>&gt; and <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>&gt; values in a variant, so you can easily construct arbitrarily complex data structures of arbitrary types. This is very powerful and versatile, but may prove less memory and speed efficient than storing specific types in standard data structures.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> also supports the notion of null values, where you can have a defined type with no value set. However, note that <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> types can only be cast when they have had a value set.</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariant">QVariant</a></span> x<span class="operator">,</span> y(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>())<span class="operator">,</span> z(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>(<span class="string">""</span>));
///   x<span class="operator">.</span>convert(<span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariant">QVariant</a></span><span class="operator">::</span>Int);
///   <span class="comment">// x.isNull() == true</span>
///   <span class="comment">// y.isNull() == true, z.isNull() == false</span>
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> can be extended to support other types than those mentioned in the <a href="http://doc.qt.io/qt-5/qvariant-obsolete.html#Type-enum">Type</a> enum. See <a href="http://doc.qt.io/qt-5/custom-types.html">Creating Custom Qt Types</a> for details.</p>
/// <a name="a-note-on-gui-types"></a></div>
#[repr(C)]
pub struct QVariant {
    _unused: u8,
}
impl QVariant {
    /// <p>Returns <code>true</code> if the variant's type can be cast to the requested type, <i>targetTypeId</i>. Such casting is done automatically when calling the <a href="http://doc.qt.io/qt-5/qvariant.html#toInt">toInt</a>(), <a href="http://doc.qt.io/qt-5/qvariant.html#toBool">toBool</a>(), ... methods.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVariant::canConvert(int targetTypeId) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#canConvert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the variant's type can be cast to the requested type, <i>targetTypeId</i>. Such casting is done automatically when calling the <a href="http://doc.qt.io/qt-5/qvariant.html#toInt">toInt</a>(), <a href="http://doc.qt.io/qt-5/qvariant.html#toBool">toBool</a>(), ... methods.</p>
    /// <p>The following casts are done automatically:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Type</th><th>Automatically Cast To</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a></td><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a></td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a></td><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QUuid</a></td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a></td><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a></td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QColor</a></td><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a></td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QDate</a></td><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QDateTime</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a></td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QDateTime</a></td><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QDate</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QTime</a></td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a></td><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a></td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QFont</a></td><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a></td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a></td><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a></td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QKeySequence</a></td><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a></td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QVariantList</a></td><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QStringList</a> (if the list's items can be converted to QStrings)</td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a></td><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a></td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QPoint</a></td><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QPointF</a></td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QRect</a></td><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QRectF</a></td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a></td><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QColor</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QDate</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QDateTime</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QFont</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QKeySequence</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QStringList</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QTime</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QUuid</a></td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QStringList</a></td><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QVariantList</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a> (if the list contains exactly one item)</td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QTime</a></td><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a></td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a></td><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a></td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a></td><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a></td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QUuid</a></td><td><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a></td></tr>
    /// </tbody></table></div>
    /// <p>A <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> containing a pointer to a type derived from <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> will also return true for this function if a <a href="http://doc.qt.io/qt-5/qobject.html#qobject_cast">qobject_cast</a> to the type described by <i>targetTypeId</i> would succeed. Note that this only works for <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> subclasses which use the <a href="http://doc.qt.io/qt-5/qobject.html#Q_OBJECT">Q_OBJECT</a> macro.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> containing a sequential container will also return true for this function if the <i>targetTypeId</i> is <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a>. It is possible to iterate over the contents of the container without extracting it as a (copied) <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> intList <span class="operator">=</span> {<span class="number">7</span><span class="operator">,</span> <span class="number">11</span><span class="operator">,</span> <span class="number">42</span>};
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariant">QVariant</a></span> variant <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariant">QVariant</a></span><span class="operator">::</span>fromValue(intList);
    ///   <span class="keyword">if</span> (variant<span class="operator">.</span>canConvert<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a></span><span class="operator">&gt;</span>()) {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qsequentialiterable.html">QSequentialIterable</a></span> iterable <span class="operator">=</span> variant<span class="operator">.</span>value<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qsequentialiterable.html">QSequentialIterable</a></span><span class="operator">&gt;</span>();
    /// &#32;     <span class="comment">// Can use foreach:</span>
    /// &#32;     foreach (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariant">QVariant</a></span> <span class="operator">&amp;</span>v<span class="operator">,</span> iterable) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> v;
    /// &#32;     }
    /// &#32;     <span class="comment">// Can use C++11 range-for:</span>
    /// &#32;     <span class="keyword">for</span> (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariant">QVariant</a></span> <span class="operator">&amp;</span>v : iterable) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> v;
    /// &#32;     }
    /// &#32;     <span class="comment">// Can use iterators:</span>
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qsequentialiterable.html">QSequentialIterable</a></span><span class="operator">::</span>const_iterator it <span class="operator">=</span> iterable<span class="operator">.</span>begin();
    /// &#32;     <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsequentialiterable.html">QSequentialIterable</a></span><span class="operator">::</span>const_iterator end <span class="operator">=</span> iterable<span class="operator">.</span>end();
    /// &#32;     <span class="keyword">for</span> ( ; it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>it;
    /// &#32;     }
    ///   }
    ///
    /// </pre>
    /// <p>This requires that the value_type of the container is itself a metatype.</p>
    /// <p>Similarly, a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> containing a sequential container will also return true for this function the <i>targetTypeId</i> is <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantHash-typedef">QVariantHash</a> or <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantMap-typedef">QVariantMap</a>. It is possible to iterate over the contents of the container without extracting it as a (copied) <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantHash-typedef">QVariantHash</a> or <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantMap-typedef">QVariantMap</a>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html">QHash</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> mapping;
    ///   mapping<span class="operator">.</span>insert(<span class="number">7</span><span class="operator">,</span> <span class="string">"Seven"</span>);
    ///   mapping<span class="operator">.</span>insert(<span class="number">11</span><span class="operator">,</span> <span class="string">"Eleven"</span>);
    ///   mapping<span class="operator">.</span>insert(<span class="number">42</span><span class="operator">,</span> <span class="string">"Forty-two"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariant">QVariant</a></span> variant <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariant">QVariant</a></span><span class="operator">::</span>fromValue(mapping);
    ///   <span class="keyword">if</span> (variant<span class="operator">.</span>canConvert<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariantHash-typedef">QVariantHash</a></span><span class="operator">&gt;</span>()) {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qassociativeiterable.html">QAssociativeIterable</a></span> iterable <span class="operator">=</span> variant<span class="operator">.</span>value<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qassociativeiterable.html">QAssociativeIterable</a></span><span class="operator">&gt;</span>();
    /// &#32;     <span class="comment">// Can use foreach over the values:</span>
    /// &#32;     foreach (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariant">QVariant</a></span> <span class="operator">&amp;</span>v<span class="operator">,</span> iterable) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> v;
    /// &#32;     }
    /// &#32;     <span class="comment">// Can use C++11 range-for over the values:</span>
    /// &#32;     <span class="keyword">for</span> (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariant">QVariant</a></span> <span class="operator">&amp;</span>v : iterable) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> v;
    /// &#32;     }
    /// &#32;     <span class="comment">// Can use iterators:</span>
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qassociativeiterable.html">QAssociativeIterable</a></span><span class="operator">::</span>const_iterator it <span class="operator">=</span> iterable<span class="operator">.</span>begin();
    /// &#32;     <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qassociativeiterable.html">QAssociativeIterable</a></span><span class="operator">::</span>const_iterator end <span class="operator">=</span> iterable<span class="operator">.</span>end();
    /// &#32;     <span class="keyword">for</span> ( ; it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>it; <span class="comment">// The current value</span>
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>key();
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>value();
    /// &#32;     }
    ///   }
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>(), <a href="http://doc.qt.io/qt-5/qsequentialiterable.html">QSequentialIterable</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Q_DECLARE_SEQUENTIAL_CONTAINER_METATYPE">Q_DECLARE_SEQUENTIAL_CONTAINER_METATYPE</a>(), <a href="http://doc.qt.io/qt-5/qassociativeiterable.html">QAssociativeIterable</a>, and <a href="http://doc.qt.io/qt-5/qmetatype.html#Q_DECLARE_ASSOCIATIVE_CONTAINER_METATYPE">Q_DECLARE_ASSOCIATIVE_CONTAINER_METATYPE</a>().</p></div>
    pub unsafe fn can_convert(&self, target_type_id: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVariant_canConvert(
            self as *const crate::QVariant,
            target_type_id,
        )
    }

    /// <p>Convert this variant to type <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UnknownType</a> and free up any resources used.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVariant::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Convert this variant to type <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UnknownType</a> and free up any resources used.</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVariant_clear(self as *mut crate::QVariant)
    }

    /// Calls C++ function: <span style='color: green;'>```const void* QVariant::constData() const```</span>.
    pub unsafe fn const_data(&self) -> ::cpp_utils::Ptr<::std::ffi::c_void> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_constData(self as *const crate::QVariant);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Casts the variant to the requested type, <i>targetTypeId</i>. If the cast cannot be done, the variant is cleared. Returns <code>true</code> if the current type of the variant was successfully cast; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVariant::convert(int targetTypeId)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#convert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Casts the variant to the requested type, <i>targetTypeId</i>. If the cast cannot be done, the variant is cleared. Returns <code>true</code> if the current type of the variant was successfully cast; otherwise returns <code>false</code>.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> containing a pointer to a type derived from <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> will also convert and return true for this function if a <a href="http://doc.qt.io/qt-5/qobject.html#qobject_cast">qobject_cast</a> to the type described by <i>targetTypeId</i> would succeed. Note that this only works for <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> subclasses which use the <a href="http://doc.qt.io/qt-5/qobject.html#Q_OBJECT">Q_OBJECT</a> macro.</p>
    /// <p><b>Warning:</b> For historical reasons, converting a null <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> results in a null value of the desired type (e.g., an empty string for <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>) and a result of false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#clear">clear</a>().</p></div>
    pub unsafe fn convert(&mut self, target_type_id: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVariant_convert(self as *mut crate::QVariant, target_type_id)
    }

    /// <p>Assigns the value of the variant <i>variant</i> to this variant.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant& QVariant::operator=(const QVariant& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns the value of the variant <i>variant</i> to this variant.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::MutRef<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_operator_(
            self as *mut crate::QVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```void* QVariant::data()```</span>.
    pub unsafe fn data_mut(&mut self) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_data(self as *mut crate::QVariant);
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```const void* QVariant::data() const```</span>.
    pub unsafe fn data(&self) -> ::cpp_utils::Ptr<::std::ffi::c_void> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_data1(self as *const crate::QVariant);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```void QVariant::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVariant_detach(self as *mut crate::QVariant)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QVariant::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVariant_isDetached(self as *const crate::QVariant)
    }

    /// <p>Returns <code>true</code> if this is a null variant, false otherwise. A variant is considered null if it contains a default constructed value or a built-in type instance that has an isNull method, in which case the result would be the same as calling isNull on the wrapped object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVariant::isNull() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#isNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this is a null variant, false otherwise. A variant is considered null if it contains a default constructed value or a built-in type instance that has an isNull method, in which case the result would be the same as calling isNull on the wrapped object.</p>
    /// <p><b>Warning:</b> The result of the function doesn't affect == operator, which means that two values can be equal even if one of them is null and another is not.</p></div>
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVariant_isNull(self as *const crate::QVariant)
    }

    /// <p>Returns <code>true</code> if the storage type of this variant is not <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UnknownType</a>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVariant::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the storage type of this variant is not <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UnknownType</a>; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn is_valid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVariant_isValid(self as *const crate::QVariant)
    }

    /// Calls C++ function: <span style='color: green;'>```void QVariant::load(QDataStream& ds)```</span>.
    pub unsafe fn load(
        &mut self,
        ds: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QDataStream>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVariant_load(
            self as *mut crate::QVariant,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QDataStream>>::cast_into(ds)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Converts the string representation of the storage type given in <i>name</i>, to its enum representation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QVariant::Type QVariant::nameToType(const char* name)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#nameToType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the string representation of the storage type given in <i>name</i>, to its enum representation.</p>
    /// <p>If the string representation cannot be converted to any enum representation, the variant is set to <code>Invalid</code>.</p></div>
    pub unsafe fn name_to_type(
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> crate::q_variant::Type {
        crate::__ffi::ctr_qt_core_ffi_QVariant_nameToType(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(name)
                .as_raw_ptr(),
        )
    }

    /// <p>Constructs an invalid variant.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an invalid variant.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a null variant of type <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(QVariant::Type type)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a null variant of type <i>type</i>.</p></div>
    pub unsafe fn from_type(type_: crate::q_variant::Type) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant1(type_);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs variant of type <i>typeId</i>, and initializes with <i>copy</i> if <i>copy</i> is not 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(int typeId, const void* copy)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs variant of type <i>typeId</i>, and initializes with <i>copy</i> if <i>copy</i> is not 0.</p>
    /// <p>Note that you have to pass the address of the variable you want stored.</p>
    /// <p>Usually, you never have to use this constructor, use <a href="http://doc.qt.io/qt-5/qvariant.html#fromValue">QVariant::fromValue</a>() instead to construct variants from the pointer types represented by <code>QMetaType::VoidStar</code>, and <code>QMetaType::QObjectStar</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#fromValue">QVariant::fromValue</a>() and <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Type</a>.</p></div>
    pub unsafe fn from_int_void(
        type_id: ::std::os::raw::c_int,
        copy: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::ffi::c_void>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant2(
            type_id,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::ffi::c_void>>::cast_into(copy)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(int typeId, const void* copy, unsigned int flags)```</span>.
    pub unsafe fn from_int_void_uint(
        type_id: ::std::os::raw::c_int,
        copy: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::ffi::c_void>>,
        flags: ::std::os::raw::c_uint,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant3(
            type_id,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::ffi::c_void>>::cast_into(copy)
                .as_raw_ptr(),
            flags,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reads the variant from the data stream, <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(QDataStream& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads the variant from the data stream, <i>s</i>.</p></div>
    pub unsafe fn from_q_data_stream(
        s: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QDataStream>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant5(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QDataStream>>::cast_into(s)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with an integer value, <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with an integer value, <i>val</i>.</p></div>
    pub unsafe fn from_int(i: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant6(i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with an unsigned integer value, <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(unsigned int ui)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with an unsigned integer value, <i>val</i>.</p></div>
    pub unsafe fn from_uint(ui: ::std::os::raw::c_uint) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant7(ui);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a long long integer value, <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(qlonglong ll)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a long long integer value, <i>val</i>.</p></div>
    pub unsafe fn from_i64(ll: i64) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant8(ll);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with an unsigned long long integer value, <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(qulonglong ull)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-9">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with an unsigned long long integer value, <i>val</i>.</p></div>
    pub unsafe fn from_u64(ull: u64) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant9(ull);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a boolean value, <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(bool b)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-10">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a boolean value, <i>val</i>.</p></div>
    pub unsafe fn from_bool(b: bool) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant10(b);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a floating point value, <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(double d)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-11">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a floating point value, <i>val</i>.</p></div>
    pub unsafe fn from_double(d: ::std::os::raw::c_double) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant11(d);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a floating point value, <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(float f)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-12">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a floating point value, <i>val</i>.</p>
    /// <p>This function was introduced in  Qt 4.6.</p></div>
    pub unsafe fn from_float(f: ::std::os::raw::c_float) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant12(f);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a string value of <i>val</i>. The variant creates a deep copy of <i>val</i> into a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> assuming UTF-8 encoding on the input <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const char* str)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-13">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a string value of <i>val</i>. The variant creates a deep copy of <i>val</i> into a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> assuming UTF-8 encoding on the input <i>val</i>.</p>
    /// <p>Note that <i>val</i> is converted to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> for storing in the variant and <a href="http://doc.qt.io/qt-5/qvariant.html#userType">QVariant::userType</a>() will return <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a> for the variant.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications.</p></div>
    pub unsafe fn from_char(
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant13(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(str)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a bytearray value, <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QByteArray& bytearray)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-14">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a bytearray value, <i>val</i>.</p></div>
    pub unsafe fn from_q_byte_array(
        bytearray: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant14(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(bytearray)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a bitarray value, <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QBitArray& bitarray)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-15">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a bitarray value, <i>val</i>.</p></div>
    pub unsafe fn from_q_bit_array(
        bitarray: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QBitArray>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant15(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QBitArray>>::cast_into(bitarray)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a string value, <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QString& string)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-16">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a string value, <i>val</i>.</p></div>
    pub unsafe fn from_q_string(
        string: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant16(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(string)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a string value, <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(QLatin1String string)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-17">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a string value, <i>val</i>.</p></div>
    pub unsafe fn from_q_latin1_string(
        string: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant17(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(string)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a string list value, <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QStringList& stringlist)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-18">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a string list value, <i>val</i>.</p></div>
    pub unsafe fn from_q_string_list(
        stringlist: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant18(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(stringlist)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a char value, <i>c</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(QChar qchar)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-19">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a char value, <i>c</i>.</p></div>
    pub unsafe fn from_q_char(
        qchar: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant19(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(qchar).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a date value, <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QDate& date)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-20">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a date value, <i>val</i>.</p></div>
    pub unsafe fn from_q_date(
        date: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDate>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant20(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDate>>::cast_into(date).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a time value, <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QTime& time)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-21">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a time value, <i>val</i>.</p></div>
    pub unsafe fn from_q_time(
        time: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTime>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant21(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTime>>::cast_into(time).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a date/time value, <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QDateTime& datetime)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-22">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a date/time value, <i>val</i>.</p></div>
    pub unsafe fn from_q_date_time(
        datetime: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant22(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(datetime)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a list value, <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QList<QVariant>& list)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-23">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a list value, <i>val</i>.</p></div>
    pub unsafe fn from_q_list_of_q_variant(
        list: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant23(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQVariant>>::cast_into(list)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a map of <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>s, <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QMap<QString, QVariant>& map)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-24">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a map of <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>s, <i>val</i>.</p></div>
    pub unsafe fn from_q_map_of_q_string_q_variant(
        map: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMapOfQStringQVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant24(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMapOfQStringQVariant>>::cast_into(map)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a hash of <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>s, <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QHash<QString, QVariant>& hash)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-25">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a hash of <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>s, <i>val</i>.</p></div>
    pub unsafe fn from_q_hash_of_q_string_q_variant(
        hash: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QHashOfQStringQVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant25(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QHashOfQStringQVariant>>::cast_into(
                hash,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a size value of <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QSize& size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-26">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a size value of <i>val</i>.</p></div>
    pub unsafe fn from_q_size(
        size: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSize>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant26(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSize>>::cast_into(size).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a size value of <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QSizeF& size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-27">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a size value of <i>val</i>.</p></div>
    pub unsafe fn from_q_size_f(
        size: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSizeF>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant27(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSizeF>>::cast_into(size).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a point value of <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QPoint& pt)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-28">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a point value of <i>val</i>.</p></div>
    pub unsafe fn from_q_point(
        pt: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant28(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(pt).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a point value of <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QPointF& pt)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-29">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a point value of <i>val</i>.</p></div>
    pub unsafe fn from_q_point_f(
        pt: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant29(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(pt).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a line value of <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QLine& line)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-30">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a line value of <i>val</i>.</p></div>
    pub unsafe fn from_q_line(
        line: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLine>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant30(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLine>>::cast_into(line).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a line value of <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QLineF& line)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-31">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a line value of <i>val</i>.</p></div>
    pub unsafe fn from_q_line_f(
        line: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLineF>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant31(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLineF>>::cast_into(line).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a rect value of <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QRect& rect)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-32">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a rect value of <i>val</i>.</p></div>
    pub unsafe fn from_q_rect(
        rect: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRect>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant32(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRect>>::cast_into(rect).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a rect value of <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QRectF& rect)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-33">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a rect value of <i>val</i>.</p></div>
    pub unsafe fn from_q_rect_f(
        rect: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRectF>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant33(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRectF>>::cast_into(rect).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a locale value, <i>l</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QLocale& locale)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-34">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a locale value, <i>l</i>.</p></div>
    pub unsafe fn from_q_locale(
        locale: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant34(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(locale)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with the regexp value <i>regExp</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QRegExp& regExp)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-35">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with the regexp value <i>regExp</i>.</p></div>
    pub unsafe fn from_q_reg_exp(
        reg_exp: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant35(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(reg_exp)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with the regular expression value <i>re</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QRegularExpression& re)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-36">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with the regular expression value <i>re</i>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn from_q_regular_expression(
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant36(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a url value of <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QUrl& url)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-37">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a url value of <i>val</i>.</p></div>
    pub unsafe fn from_q_url(
        url: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant37(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(url).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with an easing curve value, <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QEasingCurve& easing)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-38">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with an easing curve value, <i>val</i>.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn from_q_easing_curve(
        easing: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QEasingCurve>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant38(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QEasingCurve>>::cast_into(easing)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with an uuid value, <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QUuid& uuid)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-39">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with an uuid value, <i>val</i>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn from_q_uuid(
        uuid: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUuid>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant39(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUuid>>::cast_into(uuid).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> value, <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QModelIndex& modelIndex)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-40">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> value, <i>val</i>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn from_q_model_index(
        model_index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant40(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(model_index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html">QPersistentModelIndex</a> value, <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QPersistentModelIndex& modelIndex)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-41">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html">QPersistentModelIndex</a> value, <i>val</i>.</p>
    /// <p>This function was introduced in  Qt 5.5.</p></div>
    pub unsafe fn from_q_persistent_model_index(
        model_index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPersistentModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant41(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPersistentModelIndex>>::cast_into(
                model_index,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a json value, <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QJsonValue& jsonValue)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-42">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a json value, <i>val</i>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn from_q_json_value(
        json_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonValue>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant42(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonValue>>::cast_into(json_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a json object value, <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QJsonObject& jsonObject)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-43">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a json object value, <i>val</i>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn from_q_json_object(
        json_object: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonObject>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant43(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonObject>>::cast_into(json_object)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a json array value, <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QJsonArray& jsonArray)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-44">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a json array value, <i>val</i>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn from_q_json_array(
        json_array: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonArray>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant44(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonArray>>::cast_into(json_array)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new variant with a json document value, <i>val</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QJsonDocument& jsonDocument)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-45">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new variant with a json document value, <i>val</i>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn from_q_json_document(
        json_document: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonDocument>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant45(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonDocument>>::cast_into(
                json_document,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of the variant, <i>p</i>, passed as the argument to this constructor.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariant::QVariant(const QVariant& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of the variant, <i>p</i>, passed as the argument to this constructor.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_QVariant4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```void QVariant::save(QDataStream& ds) const```</span>.
    pub unsafe fn save(
        &self,
        ds: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QDataStream>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVariant_save(
            self as *const crate::QVariant,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QDataStream>>::cast_into(ds)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Swaps variant <i>other</i> with this variant. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVariant::swap(QVariant& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps variant <i>other</i> with this variant. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVariant_swap(
            self as *mut crate::QVariant,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QVariant>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QBitArray</a>; otherwise returns an empty bit array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QBitArray QVariant::toBitArray() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toBitArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QBitArray</a>; otherwise returns an empty bit array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_bit_array(&self) -> ::cpp_utils::CppBox<crate::QBitArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toBitArray(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a bool if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() Bool.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVariant::toBool() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toBool">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a bool if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() Bool.</p>
    /// <p>Returns <code>true</code> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a> and the value is non-zero, or if the variant has type <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a> or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a> and its lower-case content is not one of the following: empty, "0" or "false"; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_bool(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVariant_toBool(self as *const crate::QVariant)
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a> or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a> (converted using <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">QString::fromUtf8</a>()); otherwise returns an empty byte array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QVariant::toByteArray() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toByteArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a> or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a> (converted using <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">QString::fromUtf8</a>()); otherwise returns an empty byte array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_byte_array(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toByteArray(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>; otherwise returns an invalid <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QVariant::toChar() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toChar">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>; otherwise returns an invalid <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_char(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toChar(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QDate</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QDateTime</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>; otherwise returns an invalid date.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDate QVariant::toDate() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toDate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QDate</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QDateTime</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>; otherwise returns an invalid date.</p>
    /// <p>If the <a href="http://doc.qt.io/qt-5/qvariant.html#type">type</a>() is <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, an invalid date will be returned if the string cannot be parsed as a <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::ISODate</a> format date.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_date(&self) -> ::cpp_utils::CppBox<crate::QDate> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toDate(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QDateTime</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QDate</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>; otherwise returns an invalid date/time.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDateTime QVariant::toDateTime() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toDateTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QDateTime</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QDate</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>; otherwise returns an invalid date/time.</p>
    /// <p>If the <a href="http://doc.qt.io/qt-5/qvariant.html#type">type</a>() is <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, an invalid date/time will be returned if the string cannot be parsed as a <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::ISODate</a> format date/time.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_date_time(&self) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toDateTime(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a double if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Float</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>; otherwise returns 0.0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QVariant::toDouble(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toDouble">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a double if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Float</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>; otherwise returns 0.0.</p>
    /// <p>If <i>ok</i> is non-null: <code>*</code><i>ok</i> is set to true if the value could be converted to a double; otherwise <code>*</code><i>ok</i> is set to false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_double_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QVariant_toDouble(
            self as *const crate::QVariant,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the variant as a double if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Float</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>; otherwise returns 0.0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QVariant::toDouble() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toDouble">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a double if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Float</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>; otherwise returns 0.0.</p>
    /// <p>If <i>ok</i> is non-null: <code>*</code><i>ok</i> is set to true if the value could be converted to a double; otherwise <code>*</code><i>ok</i> is set to false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_double_0a(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QVariant_toDouble1(self as *const crate::QVariant)
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qeasingcurve.html">QEasingCurve</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QEasingCurve</a>; otherwise returns a default easing curve.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QEasingCurve QVariant::toEasingCurve() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toEasingCurve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qeasingcurve.html">QEasingCurve</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QEasingCurve</a>; otherwise returns a default easing curve.</p>
    /// <p>This function was introduced in  Qt 4.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_easing_curve(&self) -> ::cpp_utils::CppBox<crate::QEasingCurve> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toEasingCurve(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a float if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Float</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>; otherwise returns 0.0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```float QVariant::toFloat(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toFloat">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a float if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Float</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>; otherwise returns 0.0.</p>
    /// <p>If <i>ok</i> is non-null: <code>*</code><i>ok</i> is set to true if the value could be converted to a double; otherwise <code>*</code><i>ok</i> is set to false.</p>
    /// <p>This function was introduced in  Qt 4.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_float_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_float {
        crate::__ffi::ctr_qt_core_ffi_QVariant_toFloat(
            self as *const crate::QVariant,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the variant as a float if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Float</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>; otherwise returns 0.0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```float QVariant::toFloat() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toFloat">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a float if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Float</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>; otherwise returns 0.0.</p>
    /// <p>If <i>ok</i> is non-null: <code>*</code><i>ok</i> is set to true if the value could be converted to a double; otherwise <code>*</code><i>ok</i> is set to false.</p>
    /// <p>This function was introduced in  Qt 4.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_float_0a(&self) -> ::std::os::raw::c_float {
        crate::__ffi::ctr_qt_core_ffi_QVariant_toFloat1(self as *const crate::QVariant)
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>&gt; if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#type">type</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QVariantHash</a>; otherwise returns an empty map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant> QVariant::toHash() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toHash">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>&gt; if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#type">type</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QVariantHash</a>; otherwise returns an empty map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_hash(&self) -> ::cpp_utils::CppBox<crate::QHashOfQStringQVariant> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toHash(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as an int if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>; otherwise returns 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVariant::toInt(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as an int if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>; otherwise returns 0.</p>
    /// <p>If <i>ok</i> is non-null: <code>*</code><i>ok</i> is set to true if the value could be converted to an int; otherwise <code>*</code><i>ok</i> is set to false.</p>
    /// <p><b>Warning:</b> If the value is convertible to a <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a> but is too large to be represented in an int, the resulting arithmetic overflow will not be reflected in <i>ok</i>. A simple workaround is to use <a href="http://doc.qt.io/qt-5/qstring.html#toInt">QString::toInt</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_int_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVariant_toInt(
            self as *const crate::QVariant,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the variant as an int if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>; otherwise returns 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVariant::toInt() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as an int if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>; otherwise returns 0.</p>
    /// <p>If <i>ok</i> is non-null: <code>*</code><i>ok</i> is set to true if the value could be converted to an int; otherwise <code>*</code><i>ok</i> is set to false.</p>
    /// <p><b>Warning:</b> If the value is convertible to a <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a> but is too large to be represented in an int, the resulting arithmetic overflow will not be reflected in <i>ok</i>. A simple workaround is to use <a href="http://doc.qt.io/qt-5/qstring.html#toInt">QString::toInt</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_int_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVariant_toInt1(self as *const crate::QVariant)
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>; otherwise returns a default constructed <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonArray QVariant::toJsonArray() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toJsonArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>; otherwise returns a default constructed <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_json_array(&self) -> ::cpp_utils::CppBox<crate::QJsonArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toJsonArray(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a>; otherwise returns a default constructed <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonDocument QVariant::toJsonDocument() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toJsonDocument">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a>; otherwise returns a default constructed <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_json_document(&self) -> ::cpp_utils::CppBox<crate::QJsonDocument> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toJsonDocument(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>; otherwise returns a default constructed <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonObject QVariant::toJsonObject() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toJsonObject">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>; otherwise returns a default constructed <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_json_object(&self) -> ::cpp_utils::CppBox<crate::QJsonObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toJsonObject(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>; otherwise returns a default constructed <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonValue QVariant::toJsonValue() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toJsonValue">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>; otherwise returns a default constructed <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_json_value(&self) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toJsonValue(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qline.html">QLine</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QLine</a>; otherwise returns an invalid <a href="http://doc.qt.io/qt-5/qline.html">QLine</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLine QVariant::toLine() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toLine">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qline.html">QLine</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QLine</a>; otherwise returns an invalid <a href="http://doc.qt.io/qt-5/qline.html">QLine</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_line(&self) -> ::cpp_utils::CppBox<crate::QLine> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toLine(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QLineF</a>; otherwise returns an invalid <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLineF QVariant::toLineF() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toLineF">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QLineF</a>; otherwise returns an invalid <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_line_f(&self) -> ::cpp_utils::CppBox<crate::QLineF> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toLineF(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QVariantList</a> or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QStringList</a>; otherwise returns an empty list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant> QVariant::toList() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QVariantList</a> or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QStringList</a>; otherwise returns an empty list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_list(&self) -> ::cpp_utils::CppBox<crate::QListOfQVariant> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toList(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QLocale</a>; otherwise returns an invalid <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale QVariant::toLocale() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toLocale">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QLocale</a>; otherwise returns an invalid <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_locale(&self) -> ::cpp_utils::CppBox<crate::QLocale> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toLocale(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a long long int if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>; otherwise returns 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qlonglong QVariant::toLongLong(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toLongLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a long long int if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>; otherwise returns 0.</p>
    /// <p>If <i>ok</i> is non-null: <code>*</code><code>ok</code> is set to true if the value could be converted to an int; otherwise <code>*</code><code>ok</code> is set to false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_long_long_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QVariant_toLongLong(
            self as *const crate::QVariant,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the variant as a long long int if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>; otherwise returns 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qlonglong QVariant::toLongLong() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toLongLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a long long int if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>; otherwise returns 0.</p>
    /// <p>If <i>ok</i> is non-null: <code>*</code><code>ok</code> is set to true if the value could be converted to an int; otherwise <code>*</code><code>ok</code> is set to false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_long_long_0a(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QVariant_toLongLong1(self as *const crate::QVariant)
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>&gt; if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#type">type</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QVariantMap</a>; otherwise returns an empty map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant> QVariant::toMap() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toMap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>&gt; if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#type">type</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QVariantMap</a>; otherwise returns an empty map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_map(&self) -> ::cpp_utils::CppBox<crate::QMapOfQStringQVariant> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toMap(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a>; otherwise returns a default constructed <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QModelIndex QVariant::toModelIndex() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toModelIndex">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a>; otherwise returns a default constructed <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>(), <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>(), and <a href="http://doc.qt.io/qt-5/qvariant.html#toPersistentModelIndex">toPersistentModelIndex</a>().</p></div>
    pub unsafe fn to_model_index(&self) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toModelIndex(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html">QPersistentModelIndex</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html">QPersistentModelIndex</a>; otherwise returns a default constructed <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html">QPersistentModelIndex</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPersistentModelIndex QVariant::toPersistentModelIndex() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toPersistentModelIndex">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html">QPersistentModelIndex</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html">QPersistentModelIndex</a>; otherwise returns a default constructed <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html">QPersistentModelIndex</a>.</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>(), <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>(), and <a href="http://doc.qt.io/qt-5/qvariant.html#toModelIndex">toModelIndex</a>().</p></div>
    pub unsafe fn to_persistent_model_index(
        &self,
    ) -> ::cpp_utils::CppBox<crate::QPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_toPersistentModelIndex(
            self as *const crate::QVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QPoint</a> or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QPointF</a>; otherwise returns a null <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPoint QVariant::toPoint() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toPoint">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QPoint</a> or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QPointF</a>; otherwise returns a null <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_point(&self) -> ::cpp_utils::CppBox<crate::QPoint> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toPoint(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QPoint</a> or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QPointF</a>; otherwise returns a null <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPointF QVariant::toPointF() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toPointF">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QPoint</a> or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QPointF</a>; otherwise returns a null <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_point_f(&self) -> ::cpp_utils::CppBox<crate::QPointF> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toPointF(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a qreal if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Float</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>; otherwise returns 0.0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QVariant::toReal(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toReal">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a qreal if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Float</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>; otherwise returns 0.0.</p>
    /// <p>If <i>ok</i> is non-null: <code>*</code><i>ok</i> is set to true if the value could be converted to a double; otherwise <code>*</code><i>ok</i> is set to false.</p>
    /// <p>This function was introduced in  Qt 4.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_real_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QVariant_toReal(
            self as *const crate::QVariant,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the variant as a qreal if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Float</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>; otherwise returns 0.0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QVariant::toReal() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toReal">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a qreal if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Float</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>; otherwise returns 0.0.</p>
    /// <p>If <i>ok</i> is non-null: <code>*</code><i>ok</i> is set to true if the value could be converted to a double; otherwise <code>*</code><i>ok</i> is set to false.</p>
    /// <p>This function was introduced in  Qt 4.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_real_0a(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QVariant_toReal1(self as *const crate::QVariant)
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QRect</a>; otherwise returns an invalid <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRect QVariant::toRect() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toRect">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QRect</a>; otherwise returns an invalid <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_rect(&self) -> ::cpp_utils::CppBox<crate::QRect> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toRect(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QRect</a> or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QRectF</a>; otherwise returns an invalid <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRectF QVariant::toRectF() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toRectF">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QRect</a> or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QRectF</a>; otherwise returns an invalid <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_rect_f(&self) -> ::cpp_utils::CppBox<crate::QRectF> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toRectF(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QRegExp</a>; otherwise returns an empty <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegExp QVariant::toRegExp() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toRegExp">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QRegExp</a>; otherwise returns an empty <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a>.</p>
    /// <p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_reg_exp(&self) -> ::cpp_utils::CppBox<crate::QRegExp> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toRegExp(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a>; otherwise returns an empty <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpression QVariant::toRegularExpression() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toRegularExpression">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a>; otherwise returns an empty <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_regular_expression(&self) -> ::cpp_utils::CppBox<crate::QRegularExpression> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_toRegularExpression(
            self as *const crate::QVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QSize</a>; otherwise returns an invalid <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSize QVariant::toSize() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toSize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QSize</a>; otherwise returns an invalid <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_size(&self) -> ::cpp_utils::CppBox<crate::QSize> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toSize(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QSizeF</a>; otherwise returns an invalid <a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSizeF QVariant::toSizeF() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toSizeF">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QSizeF</a>; otherwise returns an invalid <a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_size_f(&self) -> ::cpp_utils::CppBox<crate::QSizeF> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toSizeF(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QDate</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QDateTime</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QStringList</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QTime</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>; otherwise returns an empty string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QVariant::toString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QDate</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QDateTime</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QStringList</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QTime</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>; otherwise returns an empty string.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_string(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toString(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QStringList</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QVariantList</a> of a type that can be converted to <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>; otherwise returns an empty list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QVariant::toStringList() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toStringList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QStringList</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QVariantList</a> of a type that can be converted to <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>; otherwise returns an empty list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_string_list(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toStringList(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QTime</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QDateTime</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>; otherwise returns an invalid time.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTime QVariant::toTime() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QTime</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QDateTime</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>; otherwise returns an invalid time.</p>
    /// <p>If the <a href="http://doc.qt.io/qt-5/qvariant.html#type">type</a>() is <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, an invalid time will be returned if the string cannot be parsed as a <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::ISODate</a> format time.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_time(&self) -> ::cpp_utils::CppBox<crate::QTime> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toTime(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as an unsigned int if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>; otherwise returns 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QVariant::toUInt(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toUInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as an unsigned int if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>; otherwise returns 0.</p>
    /// <p>If <i>ok</i> is non-null: <code>*</code><i>ok</i> is set to true if the value could be converted to an unsigned int; otherwise <code>*</code><i>ok</i> is set to false.</p>
    /// <p><b>Warning:</b> If the value is convertible to a <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a> but is too large to be represented in an unsigned int, the resulting arithmetic overflow will not be reflected in <i>ok</i>. A simple workaround is to use <a href="http://doc.qt.io/qt-5/qstring.html#toUInt">QString::toUInt</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_u_int_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QVariant_toUInt(
            self as *const crate::QVariant,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the variant as an unsigned int if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>; otherwise returns 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QVariant::toUInt() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toUInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as an unsigned int if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>; otherwise returns 0.</p>
    /// <p>If <i>ok</i> is non-null: <code>*</code><i>ok</i> is set to true if the value could be converted to an unsigned int; otherwise <code>*</code><i>ok</i> is set to false.</p>
    /// <p><b>Warning:</b> If the value is convertible to a <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a> but is too large to be represented in an unsigned int, the resulting arithmetic overflow will not be reflected in <i>ok</i>. A simple workaround is to use <a href="http://doc.qt.io/qt-5/qstring.html#toUInt">QString::toUInt</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_u_int_0a(&self) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QVariant_toUInt1(self as *const crate::QVariant)
    }

    /// <p>Returns the variant as an unsigned long long int if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#type">type</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>; otherwise returns 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qulonglong QVariant::toULongLong(bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toULongLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as an unsigned long long int if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#type">type</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>; otherwise returns 0.</p>
    /// <p>If <i>ok</i> is non-null: <code>*</code><i>ok</i> is set to true if the value could be converted to an int; otherwise <code>*</code><i>ok</i> is set to false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_u_long_long_1a(
        &self,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> u64 {
        crate::__ffi::ctr_qt_core_ffi_QVariant_toULongLong(
            self as *const crate::QVariant,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the variant as an unsigned long long int if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#type">type</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>; otherwise returns 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qulonglong QVariant::toULongLong() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toULongLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as an unsigned long long int if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#type">type</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a>; otherwise returns 0.</p>
    /// <p>If <i>ok</i> is non-null: <code>*</code><i>ok</i> is set to true if the value could be converted to an int; otherwise <code>*</code><i>ok</i> is set to false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_u_long_long_0a(&self) -> u64 {
        crate::__ffi::ctr_qt_core_ffi_QVariant_toULongLong1(self as *const crate::QVariant)
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QUrl</a>; otherwise returns an invalid <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QUrl QVariant::toUrl() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toUrl">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#userType">userType</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QUrl</a>; otherwise returns an invalid <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_url(&self) -> ::cpp_utils::CppBox<crate::QUrl> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toUrl(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the variant as a <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#type">type</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QUuid</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a> or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>; otherwise returns a default-constructed <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QUuid QVariant::toUuid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#toUuid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the variant as a <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> if the variant has <a href="http://doc.qt.io/qt-5/qvariant.html#type">type</a>() <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QUuid</a>, <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QByteArray</a> or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a>; otherwise returns a default-constructed <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() and <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>().</p></div>
    pub unsafe fn to_uuid(&self) -> ::cpp_utils::CppBox<crate::QUuid> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_toUuid(self as *const crate::QVariant);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the storage type of the value stored in the variant. Although this function is declared as returning QVariant::Type, the return value should be interpreted as <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Type</a>. In particular, QVariant::UserType is returned here only if the value is equal or greater than <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::User</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant::Type QVariant::type() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#type">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the storage type of the value stored in the variant. Although this function is declared as returning QVariant::Type, the return value should be interpreted as <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Type</a>. In particular, QVariant::UserType is returned here only if the value is equal or greater than <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::User</a>.</p>
    /// <p>Note that return values in the ranges QVariant::Char through QVariant::RegExp and QVariant::Font through QVariant::Transform correspond to the values in the ranges <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a> through <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QRegExp</a> and <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QFont</a> through <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QQuaternion</a>.</p>
    /// <p>Pay particular attention when working with char and <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> variants. Note that there is no <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> constructor specifically for type char, but there is one for <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>. For a variant of type <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>, this function returns QVariant::Char, which is the same as <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QChar</a>, but for a variant of type <code>char</code>, this function returns <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Char</a>, which is <i>not</i> the same as QVariant::Char.</p>
    /// <p>Also note that the types <code>void*</code>, <code>long</code>, <code>short</code>, <code>unsigned</code> <code>long</code>, <code>unsigned</code> <code>short</code>, <code>unsigned</code> <code>char</code>, <code>float</code>, <code>QObject*</code>, and <code>QWidget*</code> are represented in <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Type</a> but not in QVariant::Type, and they can be returned by this function. However, they are considered to be user defined types when tested against QVariant::Type.</p>
    /// <p>To test whether an instance of <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> contains a data type that is compatible with the data type you are interested in, use <a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>().</p></div>
    pub unsafe fn type_(&self) -> crate::q_variant::Type {
        crate::__ffi::ctr_qt_core_ffi_QVariant_type(self as *const crate::QVariant)
    }

    /// <p>Returns the name of the type stored in the variant. The returned strings describe the C++ datatype used to store the data: for example, "<a href="http://doc.qt.io/qt-5/qfont.html">QFont</a>", "<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>", or "<a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a>". An Invalid variant returns 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QVariant::typeName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#typeName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the name of the type stored in the variant. The returned strings describe the C++ datatype used to store the data: for example, "<a href="http://doc.qt.io/qt-5/qfont.html">QFont</a>", "<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>", or "<a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a>". An Invalid variant returns 0.</p></div>
    pub unsafe fn type_name(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVariant_typeName(self as *const crate::QVariant);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Converts the int representation of the storage type, <i>typeId</i>, to its string representation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static const char* QVariant::typeToName(int typeId)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#typeToName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the int representation of the storage type, <i>typeId</i>, to its string representation.</p>
    /// <p>Returns a null pointer if the type is <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UnknownType</a> or doesn't exist.</p></div>
    pub unsafe fn type_to_name(
        type_id: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariant_typeToName(type_id);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns the storage type of the value stored in the variant. For non-user types, this is the same as <a href="http://doc.qt.io/qt-5/qvariant.html#type">type</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVariant::userType() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#userType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the storage type of the value stored in the variant. For non-user types, this is the same as <a href="http://doc.qt.io/qt-5/qvariant.html#type">type</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#type">type</a>().</p></div>
    pub unsafe fn user_type(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVariant_userType(self as *const crate::QVariant)
    }
}

pub mod q_sequential_iterable {
    //! C++ type: <span style='color: green;'>```QSequentialIterable```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qsequentialiterable-const-iterator.html">QSequentialIterable::const_iterator</a> allows iteration over a container in a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QSequentialIterable::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsequentialiterable-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsequentialiterable-const-iterator.html">QSequentialIterable::const_iterator</a> allows iteration over a container in a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qsequentialiterable-const-iterator.html">QSequentialIterable::const_iterator</a> can only be created by a <a href="http://doc.qt.io/qt-5/qsequentialiterable.html">QSequentialIterable</a> instance, and can be used in a way similar to other stl-style iterators.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> intList <span class="operator">=</span> {<span class="number">7</span><span class="operator">,</span> <span class="number">11</span><span class="operator">,</span> <span class="number">42</span>};
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> variant <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span><span class="operator">::</span>fromValue(intList);
    ///   <span class="keyword">if</span> (variant<span class="operator">.</span>canConvert<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a></span><span class="operator">&gt;</span>()) {
    /// &#32;     <span class="type">QSequentialIterable</span> iterable <span class="operator">=</span> variant<span class="operator">.</span>value<span class="operator">&lt;</span><span class="type">QSequentialIterable</span><span class="operator">&gt;</span>();
    /// &#32;     <span class="comment">// Can use foreach:</span>
    /// &#32;     foreach (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> <span class="operator">&amp;</span>v<span class="operator">,</span> iterable) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> v;
    /// &#32;     }
    /// &#32;     <span class="comment">// Can use C++11 range-for:</span>
    /// &#32;     <span class="keyword">for</span> (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> <span class="operator">&amp;</span>v : iterable) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> v;
    /// &#32;     }
    /// &#32;     <span class="comment">// Can use iterators:</span>
    /// &#32;     <span class="type">QSequentialIterable</span><span class="operator">::</span>const_iterator it <span class="operator">=</span> iterable<span class="operator">.</span>begin();
    /// &#32;     <span class="keyword">const</span> <span class="type">QSequentialIterable</span><span class="operator">::</span>const_iterator end <span class="operator">=</span> iterable<span class="operator">.</span>end();
    /// &#32;     <span class="keyword">for</span> ( ; it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>it;
    /// &#32;     }
    ///   }
    ///
    /// </pre></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>Assigns <i>other</i> to this.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QSequentialIterable::const_iterator& QSequentialIterable::const_iterator::operator=(const QSequentialIterable::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_sequential_iterable::ConstIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_sequential_iterable::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QSequentialIterable_const_iterator_operator_(
                    self as *mut crate::q_sequential_iterable::ConstIterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_sequential_iterable::ConstIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QSequentialIterable::const_iterator QSequentialIterable::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_sequential_iterable::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QSequentialIterable_const_iterator_operator__5(
                    self as *mut crate::q_sequential_iterable::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QSequentialIterable::const_iterator QSequentialIterable::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_sequential_iterable::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QSequentialIterable_const_iterator_operator__3(
                    self as *mut crate::q_sequential_iterable::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Creates a copy of <i>other</i>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QSequentialIterable::const_iterator::const_iterator(const QSequentialIterable::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qsequentialiterable-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a copy of <i>other</i>.</p></div>
        pub unsafe fn new_copy(
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_sequential_iterable::ConstIterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_sequential_iterable::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QSequentialIterable_const_iterator_const_iterator(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_sequential_iterable::ConstIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_sequential_iterable::ConstIterator {
        /// <p>Destroys the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html">QAssociativeIterable::const_iterator</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QSequentialIterable::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#dtor.const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html">QAssociativeIterable::const_iterator</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QSequentialIterable_const_iterator_dconst_iterator(
                self as *mut crate::q_sequential_iterable::ConstIterator,
            )
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_sequential_iterable::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::QVariant>;
        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QVariant QSequentialIterable::const_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        fn indirection(self) -> ::cpp_utils::CppBox<crate::QVariant> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QSequentialIterable_const_iterator_operator_1(
                    self as *const crate::q_sequential_iterable::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_sequential_iterable::ConstIterator>>
        for crate::q_sequential_iterable::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QSequentialIterable::const_iterator::operator==(const QSequentialIterable::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_sequential_iterable::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QSequentialIterable_const_iterator_operator__(
                    self as *const crate::q_sequential_iterable::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_sequential_iterable::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_sequential_iterable::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QSequentialIterable::const_iterator& QSequentialIterable::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_sequential_iterable::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QSequentialIterable_const_iterator_operator__2(
                    self as *mut crate::q_sequential_iterable::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_sequential_iterable::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_sequential_iterable::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QSequentialIterable::const_iterator& QSequentialIterable::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_sequential_iterable::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QSequentialIterable_const_iterator_operator__4(
                    self as *mut crate::q_sequential_iterable::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_int> for crate::q_sequential_iterable::ConstIterator {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QSequentialIterable::const_iterator& QSequentialIterable::const_iterator::operator+=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QSequentialIterable_const_iterator_operator__6(
                    self as *mut crate::q_sequential_iterable::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_int> for crate::q_sequential_iterable::ConstIterator {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QSequentialIterable::const_iterator& QSequentialIterable::const_iterator::operator-=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QSequentialIterable_const_iterator_operator__7(
                    self as *mut crate::q_sequential_iterable::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_int> for &crate::q_sequential_iterable::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_sequential_iterable::ConstIterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions forward from this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QSequentialIterable::const_iterator QSequentialIterable::const_iterator::operator+(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions forward from this iterator.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>().</p></div>
        fn add(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_sequential_iterable::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QSequentialIterable_const_iterator_operator_2(
                    self as *const crate::q_sequential_iterable::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_int> for &crate::q_sequential_iterable::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_sequential_iterable::ConstIterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions backward from this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QSequentialIterable::const_iterator QSequentialIterable::const_iterator::operator-(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions backward from this iterator.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>().</p></div>
        fn sub(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_sequential_iterable::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QSequentialIterable_const_iterator_operator_3(
                    self as *const crate::q_sequential_iterable::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qsequentialiterable.html">QSequentialIterable</a> class is an iterable interface for a container in a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
///
/// C++ class: <span style='color: green;'>```QSequentialIterable```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qsequentialiterable.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsequentialiterable.html">QSequentialIterable</a> class is an iterable interface for a container in a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
/// <p>This class allows several methods of accessing the elements of a container held within a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>. An instance of <a href="http://doc.qt.io/qt-5/qsequentialiterable.html">QSequentialIterable</a> can be extracted from a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> if it can be converted to a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a>.</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> intList <span class="operator">=</span> {<span class="number">7</span><span class="operator">,</span> <span class="number">11</span><span class="operator">,</span> <span class="number">42</span>};
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> variant <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span><span class="operator">::</span>fromValue(intList);
///   <span class="keyword">if</span> (variant<span class="operator">.</span>canConvert<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a></span><span class="operator">&gt;</span>()) {
/// &#32;     <span class="type">QSequentialIterable</span> iterable <span class="operator">=</span> variant<span class="operator">.</span>value<span class="operator">&lt;</span><span class="type">QSequentialIterable</span><span class="operator">&gt;</span>();
/// &#32;     <span class="comment">// Can use foreach:</span>
/// &#32;     foreach (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> <span class="operator">&amp;</span>v<span class="operator">,</span> iterable) {
/// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> v;
/// &#32;     }
/// &#32;     <span class="comment">// Can use C++11 range-for:</span>
/// &#32;     <span class="keyword">for</span> (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> <span class="operator">&amp;</span>v : iterable) {
/// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> v;
/// &#32;     }
/// &#32;     <span class="comment">// Can use iterators:</span>
/// &#32;     <span class="type">QSequentialIterable</span><span class="operator">::</span>const_iterator it <span class="operator">=</span> iterable<span class="operator">.</span>begin();
/// &#32;     <span class="keyword">const</span> <span class="type">QSequentialIterable</span><span class="operator">::</span>const_iterator end <span class="operator">=</span> iterable<span class="operator">.</span>end();
/// &#32;     <span class="keyword">for</span> ( ; it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
/// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>it;
/// &#32;     }
///   }
///
/// </pre>
/// <p>The container itself is not copied before iterating over it.</p></div>
#[repr(C)]
pub struct QSequentialIterable {
    _unused: u8,
}
impl QSequentialIterable {
    /// <p>Returns the element at position <i>idx</i> in the container.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QSequentialIterable::at(int idx) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsequentialiterable.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the element at position <i>idx</i> in the container.</p></div>
    pub unsafe fn at(&self, idx: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSequentialIterable_at(
            self as *const crate::QSequentialIterable,
            idx,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns whether it is possible to iterate over the container in reverse. This corresponds to the std::bidirectional_iterator_tag iterator trait of the <a href="http://doc.qt.io/qt-5/qsequentialiterable-const-iterator.html">const_iterator</a> of the container.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSequentialIterable::canReverseIterate() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsequentialiterable.html#canReverseIterate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns whether it is possible to iterate over the container in reverse. This corresponds to the std::bidirectional_iterator_tag iterator trait of the <a href="http://doc.qt.io/qt-5/qsequentialiterable-const-iterator.html">const_iterator</a> of the container.</p></div>
    pub unsafe fn can_reverse_iterate(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSequentialIterable_canReverseIterate(
            self as *const crate::QSequentialIterable,
        )
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qsequentialiterable.html">QSequentialIterable</a> class is an iterable interface for a container in a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSequentialIterable& QSequentialIterable::operator=(const QSequentialIterable& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsequentialiterable.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsequentialiterable.html">QSequentialIterable</a> class is an iterable interface for a container in a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
    /// <p>This class allows several methods of accessing the elements of a container held within a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>. An instance of <a href="http://doc.qt.io/qt-5/qsequentialiterable.html">QSequentialIterable</a> can be extracted from a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> if it can be converted to a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> intList <span class="operator">=</span> {<span class="number">7</span><span class="operator">,</span> <span class="number">11</span><span class="operator">,</span> <span class="number">42</span>};
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> variant <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span><span class="operator">::</span>fromValue(intList);
    ///   <span class="keyword">if</span> (variant<span class="operator">.</span>canConvert<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a></span><span class="operator">&gt;</span>()) {
    /// &#32;     <span class="type">QSequentialIterable</span> iterable <span class="operator">=</span> variant<span class="operator">.</span>value<span class="operator">&lt;</span><span class="type">QSequentialIterable</span><span class="operator">&gt;</span>();
    /// &#32;     <span class="comment">// Can use foreach:</span>
    /// &#32;     foreach (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> <span class="operator">&amp;</span>v<span class="operator">,</span> iterable) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> v;
    /// &#32;     }
    /// &#32;     <span class="comment">// Can use C++11 range-for:</span>
    /// &#32;     <span class="keyword">for</span> (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> <span class="operator">&amp;</span>v : iterable) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> v;
    /// &#32;     }
    /// &#32;     <span class="comment">// Can use iterators:</span>
    /// &#32;     <span class="type">QSequentialIterable</span><span class="operator">::</span>const_iterator it <span class="operator">=</span> iterable<span class="operator">.</span>begin();
    /// &#32;     <span class="keyword">const</span> <span class="type">QSequentialIterable</span><span class="operator">::</span>const_iterator end <span class="operator">=</span> iterable<span class="operator">.</span>end();
    /// &#32;     <span class="keyword">for</span> ( ; it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>it;
    /// &#32;     }
    ///   }
    ///
    /// </pre>
    /// <p>The container itself is not copied before iterating over it.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSequentialIterable>>,
    ) -> ::cpp_utils::MutRef<crate::QSequentialIterable> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSequentialIterable_operator_(
            self as *mut crate::QSequentialIterable,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSequentialIterable>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qsequentialiterable.html">QSequentialIterable</a> class is an iterable interface for a container in a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSequentialIterable::QSequentialIterable(const QSequentialIterable& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsequentialiterable.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsequentialiterable.html">QSequentialIterable</a> class is an iterable interface for a container in a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
    /// <p>This class allows several methods of accessing the elements of a container held within a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>. An instance of <a href="http://doc.qt.io/qt-5/qsequentialiterable.html">QSequentialIterable</a> can be extracted from a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> if it can be converted to a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> intList <span class="operator">=</span> {<span class="number">7</span><span class="operator">,</span> <span class="number">11</span><span class="operator">,</span> <span class="number">42</span>};
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> variant <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span><span class="operator">::</span>fromValue(intList);
    ///   <span class="keyword">if</span> (variant<span class="operator">.</span>canConvert<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a></span><span class="operator">&gt;</span>()) {
    /// &#32;     <span class="type">QSequentialIterable</span> iterable <span class="operator">=</span> variant<span class="operator">.</span>value<span class="operator">&lt;</span><span class="type">QSequentialIterable</span><span class="operator">&gt;</span>();
    /// &#32;     <span class="comment">// Can use foreach:</span>
    /// &#32;     foreach (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> <span class="operator">&amp;</span>v<span class="operator">,</span> iterable) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> v;
    /// &#32;     }
    /// &#32;     <span class="comment">// Can use C++11 range-for:</span>
    /// &#32;     <span class="keyword">for</span> (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> <span class="operator">&amp;</span>v : iterable) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> v;
    /// &#32;     }
    /// &#32;     <span class="comment">// Can use iterators:</span>
    /// &#32;     <span class="type">QSequentialIterable</span><span class="operator">::</span>const_iterator it <span class="operator">=</span> iterable<span class="operator">.</span>begin();
    /// &#32;     <span class="keyword">const</span> <span class="type">QSequentialIterable</span><span class="operator">::</span>const_iterator end <span class="operator">=</span> iterable<span class="operator">.</span>end();
    /// &#32;     <span class="keyword">for</span> ( ; it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>it;
    /// &#32;     }
    ///   }
    ///
    /// </pre>
    /// <p>The container itself is not copied before iterating over it.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSequentialIterable>>,
    ) -> ::cpp_utils::CppBox<crate::QSequentialIterable> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSequentialIterable_QSequentialIterable2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSequentialIterable>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the number of elements in the container.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QSequentialIterable::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsequentialiterable.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of elements in the container.</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QSequentialIterable_size(
            self as *const crate::QSequentialIterable,
        )
    }
}

pub mod q_associative_iterable {
    //! C++ type: <span style='color: green;'>```QAssociativeIterable```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html">QAssociativeIterable::const_iterator</a> allows iteration over a container in a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QAssociativeIterable::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html">QAssociativeIterable::const_iterator</a> allows iteration over a container in a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html">QAssociativeIterable::const_iterator</a> can only be created by a <a href="http://doc.qt.io/qt-5/qassociativeiterable.html">QAssociativeIterable</a> instance, and can be used in a way similar to other stl-style iterators.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html">QHash</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> mapping;
    ///   mapping<span class="operator">.</span>insert(<span class="number">7</span><span class="operator">,</span> <span class="string">"Seven"</span>);
    ///   mapping<span class="operator">.</span>insert(<span class="number">11</span><span class="operator">,</span> <span class="string">"Eleven"</span>);
    ///   mapping<span class="operator">.</span>insert(<span class="number">42</span><span class="operator">,</span> <span class="string">"Forty-two"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> variant <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span><span class="operator">::</span>fromValue(mapping);
    ///   <span class="keyword">if</span> (variant<span class="operator">.</span>canConvert<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariantHash-typedef">QVariantHash</a></span><span class="operator">&gt;</span>()) {
    /// &#32;     <span class="type">QAssociativeIterable</span> iterable <span class="operator">=</span> variant<span class="operator">.</span>value<span class="operator">&lt;</span><span class="type">QAssociativeIterable</span><span class="operator">&gt;</span>();
    /// &#32;     <span class="comment">// Can use foreach over the values:</span>
    /// &#32;     foreach (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> <span class="operator">&amp;</span>v<span class="operator">,</span> iterable) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> v;
    /// &#32;     }
    /// &#32;     <span class="comment">// Can use C++11 range-for over the values:</span>
    /// &#32;     <span class="keyword">for</span> (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> <span class="operator">&amp;</span>v : iterable) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> v;
    /// &#32;     }
    /// &#32;     <span class="comment">// Can use iterators:</span>
    /// &#32;     <span class="type">QAssociativeIterable</span><span class="operator">::</span>const_iterator it <span class="operator">=</span> iterable<span class="operator">.</span>begin();
    /// &#32;     <span class="keyword">const</span> <span class="type">QAssociativeIterable</span><span class="operator">::</span>const_iterator end <span class="operator">=</span> iterable<span class="operator">.</span>end();
    /// &#32;     <span class="keyword">for</span> ( ; it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>it; <span class="comment">// The current value</span>
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>key();
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>value();
    /// &#32;     }
    ///   }
    ///
    /// </pre></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>Assigns <i>other</i> to this.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QAssociativeIterable::const_iterator& QAssociativeIterable::const_iterator::operator=(const QAssociativeIterable::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_associative_iterable::ConstIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_associative_iterable::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QAssociativeIterable_const_iterator_operator_(
                    self as *mut crate::q_associative_iterable::ConstIterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_associative_iterable::ConstIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QAssociativeIterable::const_iterator QAssociativeIterable::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_associative_iterable::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QAssociativeIterable_const_iterator_operator__5(
                    self as *mut crate::q_associative_iterable::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QAssociativeIterable::const_iterator QAssociativeIterable::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_associative_iterable::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QAssociativeIterable_const_iterator_operator__3(
                    self as *mut crate::q_associative_iterable::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the current key, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QVariant QAssociativeIterable::const_iterator::key() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#key">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current key, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        pub unsafe fn key(&self) -> ::cpp_utils::CppBox<crate::QVariant> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAssociativeIterable_const_iterator_key(
                self as *const crate::q_associative_iterable::ConstIterator,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Creates a copy of <i>other</i>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QAssociativeIterable::const_iterator::const_iterator(const QAssociativeIterable::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a copy of <i>other</i>.</p></div>
        pub unsafe fn new_copy(
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_associative_iterable::ConstIterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_associative_iterable::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QAssociativeIterable_const_iterator_const_iterator(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_associative_iterable::ConstIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QVariant QAssociativeIterable::const_iterator::value() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        pub unsafe fn value(&self) -> ::cpp_utils::CppBox<crate::QVariant> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QAssociativeIterable_const_iterator_value(
                    self as *const crate::q_associative_iterable::ConstIterator,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_associative_iterable::ConstIterator {
        /// <p>Destroys the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html">QAssociativeIterable::const_iterator</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QAssociativeIterable::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#dtor.const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html">QAssociativeIterable::const_iterator</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QAssociativeIterable_const_iterator_dconst_iterator(
                self as *mut crate::q_associative_iterable::ConstIterator,
            )
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_associative_iterable::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::QVariant>;
        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QVariant QAssociativeIterable::const_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        fn indirection(self) -> ::cpp_utils::CppBox<crate::QVariant> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QAssociativeIterable_const_iterator_operator_1(
                    self as *const crate::q_associative_iterable::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_associative_iterable::ConstIterator>>
        for crate::q_associative_iterable::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QAssociativeIterable::const_iterator::operator==(const QAssociativeIterable::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_associative_iterable::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QAssociativeIterable_const_iterator_operator__(
                    self as *const crate::q_associative_iterable::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_associative_iterable::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_associative_iterable::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QAssociativeIterable::const_iterator& QAssociativeIterable::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_associative_iterable::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QAssociativeIterable_const_iterator_operator__2(
                    self as *mut crate::q_associative_iterable::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_associative_iterable::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_associative_iterable::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QAssociativeIterable::const_iterator& QAssociativeIterable::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_associative_iterable::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QAssociativeIterable_const_iterator_operator__4(
                    self as *mut crate::q_associative_iterable::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_int> for crate::q_associative_iterable::ConstIterator {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QAssociativeIterable::const_iterator& QAssociativeIterable::const_iterator::operator+=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QAssociativeIterable_const_iterator_operator__6(
                    self as *mut crate::q_associative_iterable::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_int> for crate::q_associative_iterable::ConstIterator {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QAssociativeIterable::const_iterator& QAssociativeIterable::const_iterator::operator-=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QAssociativeIterable_const_iterator_operator__7(
                    self as *mut crate::q_associative_iterable::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_int> for &crate::q_associative_iterable::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_associative_iterable::ConstIterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions forward from this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QAssociativeIterable::const_iterator QAssociativeIterable::const_iterator::operator+(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions forward from this iterator.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>().</p></div>
        fn add(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_associative_iterable::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QAssociativeIterable_const_iterator_operator_2(
                    self as *const crate::q_associative_iterable::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_int> for &crate::q_associative_iterable::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_associative_iterable::ConstIterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions backward from this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QAssociativeIterable::const_iterator QAssociativeIterable::const_iterator::operator-(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions backward from this iterator.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>().</p></div>
        fn sub(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_associative_iterable::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QAssociativeIterable_const_iterator_operator_3(
                    self as *const crate::q_associative_iterable::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qassociativeiterable.html">QAssociativeIterable</a> class is an iterable interface for an associative container in a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
///
/// C++ class: <span style='color: green;'>```QAssociativeIterable```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qassociativeiterable.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qassociativeiterable.html">QAssociativeIterable</a> class is an iterable interface for an associative container in a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
/// <p>This class allows several methods of accessing the elements of an associative container held within a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>. An instance of <a href="http://doc.qt.io/qt-5/qassociativeiterable.html">QAssociativeIterable</a> can be extracted from a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> if it can be converted to a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantHash-typedef">QVariantHash</a> or <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantMap-typedef">QVariantMap</a>.</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html">QHash</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> mapping;
///   mapping<span class="operator">.</span>insert(<span class="number">7</span><span class="operator">,</span> <span class="string">"Seven"</span>);
///   mapping<span class="operator">.</span>insert(<span class="number">11</span><span class="operator">,</span> <span class="string">"Eleven"</span>);
///   mapping<span class="operator">.</span>insert(<span class="number">42</span><span class="operator">,</span> <span class="string">"Forty-two"</span>);
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> variant <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span><span class="operator">::</span>fromValue(mapping);
///   <span class="keyword">if</span> (variant<span class="operator">.</span>canConvert<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariantHash-typedef">QVariantHash</a></span><span class="operator">&gt;</span>()) {
/// &#32;     <span class="type">QAssociativeIterable</span> iterable <span class="operator">=</span> variant<span class="operator">.</span>value<span class="operator">&lt;</span><span class="type">QAssociativeIterable</span><span class="operator">&gt;</span>();
/// &#32;     <span class="comment">// Can use foreach over the values:</span>
/// &#32;     foreach (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> <span class="operator">&amp;</span>v<span class="operator">,</span> iterable) {
/// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> v;
/// &#32;     }
/// &#32;     <span class="comment">// Can use C++11 range-for over the values:</span>
/// &#32;     <span class="keyword">for</span> (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> <span class="operator">&amp;</span>v : iterable) {
/// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> v;
/// &#32;     }
/// &#32;     <span class="comment">// Can use iterators:</span>
/// &#32;     <span class="type">QAssociativeIterable</span><span class="operator">::</span>const_iterator it <span class="operator">=</span> iterable<span class="operator">.</span>begin();
/// &#32;     <span class="keyword">const</span> <span class="type">QAssociativeIterable</span><span class="operator">::</span>const_iterator end <span class="operator">=</span> iterable<span class="operator">.</span>end();
/// &#32;     <span class="keyword">for</span> ( ; it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
/// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>it; <span class="comment">// The current value</span>
/// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>key();
/// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>value();
/// &#32;     }
///   }
///
/// </pre>
/// <p>The container itself is not copied before iterating over it.</p></div>
#[repr(C)]
pub struct QAssociativeIterable {
    _unused: u8,
}
impl QAssociativeIterable {
    /// <p>The <a href="http://doc.qt.io/qt-5/qassociativeiterable.html">QAssociativeIterable</a> class is an iterable interface for an associative container in a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAssociativeIterable& QAssociativeIterable::operator=(const QAssociativeIterable& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qassociativeiterable.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qassociativeiterable.html">QAssociativeIterable</a> class is an iterable interface for an associative container in a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
    /// <p>This class allows several methods of accessing the elements of an associative container held within a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>. An instance of <a href="http://doc.qt.io/qt-5/qassociativeiterable.html">QAssociativeIterable</a> can be extracted from a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> if it can be converted to a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantHash-typedef">QVariantHash</a> or <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantMap-typedef">QVariantMap</a>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html">QHash</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> mapping;
    ///   mapping<span class="operator">.</span>insert(<span class="number">7</span><span class="operator">,</span> <span class="string">"Seven"</span>);
    ///   mapping<span class="operator">.</span>insert(<span class="number">11</span><span class="operator">,</span> <span class="string">"Eleven"</span>);
    ///   mapping<span class="operator">.</span>insert(<span class="number">42</span><span class="operator">,</span> <span class="string">"Forty-two"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> variant <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span><span class="operator">::</span>fromValue(mapping);
    ///   <span class="keyword">if</span> (variant<span class="operator">.</span>canConvert<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariantHash-typedef">QVariantHash</a></span><span class="operator">&gt;</span>()) {
    /// &#32;     <span class="type">QAssociativeIterable</span> iterable <span class="operator">=</span> variant<span class="operator">.</span>value<span class="operator">&lt;</span><span class="type">QAssociativeIterable</span><span class="operator">&gt;</span>();
    /// &#32;     <span class="comment">// Can use foreach over the values:</span>
    /// &#32;     foreach (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> <span class="operator">&amp;</span>v<span class="operator">,</span> iterable) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> v;
    /// &#32;     }
    /// &#32;     <span class="comment">// Can use C++11 range-for over the values:</span>
    /// &#32;     <span class="keyword">for</span> (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> <span class="operator">&amp;</span>v : iterable) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> v;
    /// &#32;     }
    /// &#32;     <span class="comment">// Can use iterators:</span>
    /// &#32;     <span class="type">QAssociativeIterable</span><span class="operator">::</span>const_iterator it <span class="operator">=</span> iterable<span class="operator">.</span>begin();
    /// &#32;     <span class="keyword">const</span> <span class="type">QAssociativeIterable</span><span class="operator">::</span>const_iterator end <span class="operator">=</span> iterable<span class="operator">.</span>end();
    /// &#32;     <span class="keyword">for</span> ( ; it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>it; <span class="comment">// The current value</span>
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>key();
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>value();
    /// &#32;     }
    ///   }
    ///
    /// </pre>
    /// <p>The container itself is not copied before iterating over it.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QAssociativeIterable>>,
    ) -> ::cpp_utils::MutRef<crate::QAssociativeIterable> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAssociativeIterable_operator_(
            self as *mut crate::QAssociativeIterable,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QAssociativeIterable>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html">QAssociativeIterable::const_iterator</a> for the given key <i>key</i> in the container, if the types are convertible.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAssociativeIterable::const_iterator QAssociativeIterable::find(const QVariant& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#find">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html">QAssociativeIterable::const_iterator</a> for the given key <i>key</i> in the container, if the types are convertible.</p>
    /// <p>If the key is not found, returns <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">end</a>().</p>
    /// <p>This can be used in stl-style iteration.</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">begin</a>(), <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">end</a>(), and <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#value">value</a>().</p></div>
    pub unsafe fn find(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::q_associative_iterable::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAssociativeIterable_find(
            self as *const crate::QAssociativeIterable,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qassociativeiterable.html">QAssociativeIterable</a> class is an iterable interface for an associative container in a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QAssociativeIterable::QAssociativeIterable(const QAssociativeIterable& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qassociativeiterable.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qassociativeiterable.html">QAssociativeIterable</a> class is an iterable interface for an associative container in a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
    /// <p>This class allows several methods of accessing the elements of an associative container held within a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>. An instance of <a href="http://doc.qt.io/qt-5/qassociativeiterable.html">QAssociativeIterable</a> can be extracted from a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> if it can be converted to a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantHash-typedef">QVariantHash</a> or <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantMap-typedef">QVariantMap</a>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html">QHash</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> mapping;
    ///   mapping<span class="operator">.</span>insert(<span class="number">7</span><span class="operator">,</span> <span class="string">"Seven"</span>);
    ///   mapping<span class="operator">.</span>insert(<span class="number">11</span><span class="operator">,</span> <span class="string">"Eleven"</span>);
    ///   mapping<span class="operator">.</span>insert(<span class="number">42</span><span class="operator">,</span> <span class="string">"Forty-two"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> variant <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span><span class="operator">::</span>fromValue(mapping);
    ///   <span class="keyword">if</span> (variant<span class="operator">.</span>canConvert<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariantHash-typedef">QVariantHash</a></span><span class="operator">&gt;</span>()) {
    /// &#32;     <span class="type">QAssociativeIterable</span> iterable <span class="operator">=</span> variant<span class="operator">.</span>value<span class="operator">&lt;</span><span class="type">QAssociativeIterable</span><span class="operator">&gt;</span>();
    /// &#32;     <span class="comment">// Can use foreach over the values:</span>
    /// &#32;     foreach (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> <span class="operator">&amp;</span>v<span class="operator">,</span> iterable) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> v;
    /// &#32;     }
    /// &#32;     <span class="comment">// Can use C++11 range-for over the values:</span>
    /// &#32;     <span class="keyword">for</span> (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> <span class="operator">&amp;</span>v : iterable) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> v;
    /// &#32;     }
    /// &#32;     <span class="comment">// Can use iterators:</span>
    /// &#32;     <span class="type">QAssociativeIterable</span><span class="operator">::</span>const_iterator it <span class="operator">=</span> iterable<span class="operator">.</span>begin();
    /// &#32;     <span class="keyword">const</span> <span class="type">QAssociativeIterable</span><span class="operator">::</span>const_iterator end <span class="operator">=</span> iterable<span class="operator">.</span>end();
    /// &#32;     <span class="keyword">for</span> ( ; it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>it; <span class="comment">// The current value</span>
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>key();
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>value();
    /// &#32;     }
    ///   }
    ///
    /// </pre>
    /// <p>The container itself is not copied before iterating over it.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QAssociativeIterable>>,
    ) -> ::cpp_utils::CppBox<crate::QAssociativeIterable> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAssociativeIterable_QAssociativeIterable2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QAssociativeIterable>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the number of elements in the container.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QAssociativeIterable::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of elements in the container.</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QAssociativeIterable_size(
            self as *const crate::QAssociativeIterable,
        )
    }

    /// <p>Returns the value for the given <i>key</i> in the container, if the types are convertible.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QAssociativeIterable::value(const QVariant& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value for the given <i>key</i> in the container, if the types are convertible.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable.html#find">find</a>().</p></div>
    pub unsafe fn value(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAssociativeIterable_value(
            self as *const crate::QAssociativeIterable,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> class is used to locate data in a data model.</p>
///
/// C++ class: <span style='color: green;'>```QModelIndex```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qmodelindex.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> class is used to locate data in a data model.</p>
/// <p>This class is used as an index into item models derived from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a>. The index is used by item views, delegates, and selection models to locate an item in the model.</p>
/// <p>New <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> objects are created by the model using the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#createIndex">QAbstractItemModel::createIndex</a>() function. An <i>invalid</i> model index can be constructed with the <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> constructor. Invalid indexes are often used as parent indexes when referring to top-level items in a model.</p>
/// <p>Model indexes refer to items in models, and contain all the information required to specify their locations in those models. Each index is located in a given row and column, and may have a parent index; use <a href="http://doc.qt.io/qt-5/qmodelindex.html#row">row</a>(), <a href="http://doc.qt.io/qt-5/qmodelindex.html#column">column</a>(), and <a href="http://doc.qt.io/qt-5/qmodelindex.html#parent">parent</a>() to obtain this information. Each top-level item in a model is represented by a model index that does not have a parent index - in this case, <a href="http://doc.qt.io/qt-5/qmodelindex.html#parent">parent</a>() will return an invalid model index, equivalent to an index constructed with the zero argument form of the <a href="http://doc.qt.io/qt-5/qmodelindex.html#QModelIndex">QModelIndex</a>() constructor.</p>
/// <p>To obtain a model index that refers to an existing item in a model, call <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>() with the required row and column values, and the model index of the parent. When referring to top-level items in a model, supply <a href="http://doc.qt.io/qt-5/qmodelindex.html#QModelIndex">QModelIndex</a>() as the parent index.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qmodelindex.html#model">model</a>() function returns the model that the index references as a <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a>. The child() function is used to examine items held under the index in the model. The <a href="http://doc.qt.io/qt-5/qmodelindex.html#sibling">sibling</a>() function allows you to traverse items in the model on the same level as the index.</p>
/// <p><b>Note: </b>Model indexes should be used immediately and then discarded. You should not rely on indexes to remain valid after calling model functions that change the structure of the model or delete items. If you need to keep a model index over time use a <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html">QPersistentModelIndex</a>.</p></div>
#[repr(C)]
pub struct QModelIndex {
    _unused: u8,
}
impl QModelIndex {
    /// <p>Use <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>() instead.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QModelIndex QModelIndex::child(int row, int column) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmodelindex-obsolete.html#child">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Use <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>() instead.</p>
    /// <p>Returns the child of the model index that is stored in the given <i>row</i> and <i>column</i>.</p>
    /// <p><b>Note: </b>This function does not work for an invalid model index which is often used as the root index.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qmodelindex.html#parent">parent</a>() and <a href="http://doc.qt.io/qt-5/qmodelindex.html#sibling">sibling</a>().</p></div>
    pub unsafe fn child(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QModelIndex_child(
            self as *const crate::QModelIndex,
            row,
            column,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the column this model index refers to.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QModelIndex::column() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmodelindex.html#column">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the column this model index refers to.</p></div>
    pub unsafe fn column(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QModelIndex_column(self as *const crate::QModelIndex)
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> class is used to locate data in a data model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QModelIndex& QModelIndex::operator=(const QModelIndex& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmodelindex.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> class is used to locate data in a data model.</p>
    /// <p>This class is used as an index into item models derived from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a>. The index is used by item views, delegates, and selection models to locate an item in the model.</p>
    /// <p>New <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> objects are created by the model using the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#createIndex">QAbstractItemModel::createIndex</a>() function. An <i>invalid</i> model index can be constructed with the <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> constructor. Invalid indexes are often used as parent indexes when referring to top-level items in a model.</p>
    /// <p>Model indexes refer to items in models, and contain all the information required to specify their locations in those models. Each index is located in a given row and column, and may have a parent index; use <a href="http://doc.qt.io/qt-5/qmodelindex.html#row">row</a>(), <a href="http://doc.qt.io/qt-5/qmodelindex.html#column">column</a>(), and <a href="http://doc.qt.io/qt-5/qmodelindex.html#parent">parent</a>() to obtain this information. Each top-level item in a model is represented by a model index that does not have a parent index - in this case, <a href="http://doc.qt.io/qt-5/qmodelindex.html#parent">parent</a>() will return an invalid model index, equivalent to an index constructed with the zero argument form of the <a href="http://doc.qt.io/qt-5/qmodelindex.html#QModelIndex">QModelIndex</a>() constructor.</p>
    /// <p>To obtain a model index that refers to an existing item in a model, call <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>() with the required row and column values, and the model index of the parent. When referring to top-level items in a model, supply <a href="http://doc.qt.io/qt-5/qmodelindex.html#QModelIndex">QModelIndex</a>() as the parent index.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qmodelindex.html#model">model</a>() function returns the model that the index references as a <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a>. The child() function is used to examine items held under the index in the model. The <a href="http://doc.qt.io/qt-5/qmodelindex.html#sibling">sibling</a>() function allows you to traverse items in the model on the same level as the index.</p>
    /// <p><b>Note: </b>Model indexes should be used immediately and then discarded. You should not rely on indexes to remain valid after calling model functions that change the structure of the model or delete items. If you need to keep a model index over time use a <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html">QPersistentModelIndex</a>.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::MutRef<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QModelIndex_operator_1(
            self as *mut crate::QModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the data for the given <i>role</i> for the item referred to by the index.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QModelIndex::data(int role = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmodelindex.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the data for the given <i>role</i> for the item referred to by the index.</p></div>
    pub unsafe fn data_1a(
        &self,
        role: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QModelIndex_data(self as *const crate::QModelIndex, role);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the data for the given <i>role</i> for the item referred to by the index.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QModelIndex::data() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmodelindex.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the data for the given <i>role</i> for the item referred to by the index.</p></div>
    pub unsafe fn data_0a(&self) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QModelIndex_data1(self as *const crate::QModelIndex);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the flags for the item referred to by the index.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFlags<Qt::ItemFlag> QModelIndex::flags() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmodelindex.html#flags">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the flags for the item referred to by the index.</p>
    /// <p>This function was introduced in  Qt 4.2.</p></div>
    pub unsafe fn flags(&self) -> crate::QFlags<crate::ItemFlag> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QModelIndex_flags(self as *const crate::QModelIndex);
        crate::QFlags::from(ffi_result)
    }

    /// <p>Returns a <code>quintptr</code> used by the model to associate the index with the internal data structure.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```quintptr QModelIndex::internalId() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmodelindex.html#internalId">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <code>quintptr</code> used by the model to associate the index with the internal data structure.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#createIndex">QAbstractItemModel::createIndex</a>().</p></div>
    pub unsafe fn internal_id(&self) -> usize {
        crate::__ffi::ctr_qt_core_ffi_QModelIndex_internalId(self as *const crate::QModelIndex)
    }

    /// <p>Returns a <code>void</code> <code>*</code> pointer used by the model to associate the index with the internal data structure.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void* QModelIndex::internalPointer() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmodelindex.html#internalPointer">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <code>void</code> <code>*</code> pointer used by the model to associate the index with the internal data structure.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#createIndex">QAbstractItemModel::createIndex</a>().</p></div>
    pub unsafe fn internal_pointer(&self) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QModelIndex_internalPointer(
            self as *const crate::QModelIndex,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns <code>true</code> if this model index is valid; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QModelIndex::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmodelindex.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this model index is valid; otherwise returns <code>false</code>.</p>
    /// <p>A valid index belongs to a model, and has non-negative row and column numbers.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmodelindex.html#model">model</a>(), <a href="http://doc.qt.io/qt-5/qmodelindex.html#row">row</a>(), and <a href="http://doc.qt.io/qt-5/qmodelindex.html#column">column</a>().</p></div>
    pub unsafe fn is_valid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QModelIndex_isValid(self as *const crate::QModelIndex)
    }

    /// <p>Returns a pointer to the model containing the item that this index refers to.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QAbstractItemModel* QModelIndex::model() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmodelindex.html#model">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the model containing the item that this index refers to.</p>
    /// <p>A const pointer to the model is returned because calls to non-const functions of the model might invalidate the model index and possibly crash your application.</p></div>
    pub unsafe fn model(&self) -> ::cpp_utils::Ptr<crate::QAbstractItemModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QModelIndex_model(self as *const crate::QModelIndex);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Creates a new empty model index. This type of model index is used to indicate that the position in the model is invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QModelIndex::QModelIndex()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmodelindex.html#QModelIndex">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a new empty model index. This type of model index is used to indicate that the position in the model is invalid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmodelindex.html#isValid">isValid</a>() and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a>.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QModelIndex_QModelIndex();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> class is used to locate data in a data model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QModelIndex::QModelIndex(const QModelIndex& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmodelindex.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> class is used to locate data in a data model.</p>
    /// <p>This class is used as an index into item models derived from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a>. The index is used by item views, delegates, and selection models to locate an item in the model.</p>
    /// <p>New <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> objects are created by the model using the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#createIndex">QAbstractItemModel::createIndex</a>() function. An <i>invalid</i> model index can be constructed with the <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> constructor. Invalid indexes are often used as parent indexes when referring to top-level items in a model.</p>
    /// <p>Model indexes refer to items in models, and contain all the information required to specify their locations in those models. Each index is located in a given row and column, and may have a parent index; use <a href="http://doc.qt.io/qt-5/qmodelindex.html#row">row</a>(), <a href="http://doc.qt.io/qt-5/qmodelindex.html#column">column</a>(), and <a href="http://doc.qt.io/qt-5/qmodelindex.html#parent">parent</a>() to obtain this information. Each top-level item in a model is represented by a model index that does not have a parent index - in this case, <a href="http://doc.qt.io/qt-5/qmodelindex.html#parent">parent</a>() will return an invalid model index, equivalent to an index constructed with the zero argument form of the <a href="http://doc.qt.io/qt-5/qmodelindex.html#QModelIndex">QModelIndex</a>() constructor.</p>
    /// <p>To obtain a model index that refers to an existing item in a model, call <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>() with the required row and column values, and the model index of the parent. When referring to top-level items in a model, supply <a href="http://doc.qt.io/qt-5/qmodelindex.html#QModelIndex">QModelIndex</a>() as the parent index.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qmodelindex.html#model">model</a>() function returns the model that the index references as a <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a>. The child() function is used to examine items held under the index in the model. The <a href="http://doc.qt.io/qt-5/qmodelindex.html#sibling">sibling</a>() function allows you to traverse items in the model on the same level as the index.</p>
    /// <p><b>Note: </b>Model indexes should be used immediately and then discarded. You should not rely on indexes to remain valid after calling model functions that change the structure of the model or delete items. If you need to keep a model index over time use a <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html">QPersistentModelIndex</a>.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QModelIndex_QModelIndex1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the parent of the model index, or <a href="http://doc.qt.io/qt-5/qmodelindex.html#QModelIndex">QModelIndex</a>() if it has no parent.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QModelIndex QModelIndex::parent() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmodelindex.html#parent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the parent of the model index, or <a href="http://doc.qt.io/qt-5/qmodelindex.html#QModelIndex">QModelIndex</a>() if it has no parent.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmodelindex.html#sibling">sibling</a>() and <a href="http://doc.qt.io/qt-5/qmodelindex.html#model">model</a>().</p></div>
    pub unsafe fn parent(&self) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QModelIndex_parent(self as *const crate::QModelIndex);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the row this model index refers to.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QModelIndex::row() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmodelindex.html#row">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the row this model index refers to.</p></div>
    pub unsafe fn row(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QModelIndex_row(self as *const crate::QModelIndex)
    }

    /// <p>Returns the sibling at <i>row</i> and <i>column</i>. If there is no sibling at this position, an invalid <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QModelIndex QModelIndex::sibling(int row, int column) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmodelindex.html#sibling">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the sibling at <i>row</i> and <i>column</i>. If there is no sibling at this position, an invalid <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> is returned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmodelindex.html#parent">parent</a>().</p></div>
    pub unsafe fn sibling(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QModelIndex_sibling(
            self as *const crate::QModelIndex,
            row,
            column,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the sibling at <i>column</i> for the current row. If there is no sibling at this position, an invalid <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QModelIndex QModelIndex::siblingAtColumn(int column) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmodelindex.html#siblingAtColumn">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the sibling at <i>column</i> for the current row. If there is no sibling at this position, an invalid <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> is returned.</p>
    /// <p>This function was introduced in  Qt 5.11.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmodelindex.html#sibling">sibling</a>() and <a href="http://doc.qt.io/qt-5/qmodelindex.html#siblingAtRow">siblingAtRow</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn sibling_at_column(
        &self,
        column: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QModelIndex_siblingAtColumn(
            self as *const crate::QModelIndex,
            column,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the sibling at <i>row</i> for the current column. If there is no sibling at this position, an invalid <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QModelIndex QModelIndex::siblingAtRow(int row) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmodelindex.html#siblingAtRow">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the sibling at <i>row</i> for the current column. If there is no sibling at this position, an invalid <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> is returned.</p>
    /// <p>This function was introduced in  Qt 5.11.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmodelindex.html#sibling">sibling</a>() and <a href="http://doc.qt.io/qt-5/qmodelindex.html#siblingAtColumn">siblingAtColumn</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn sibling_at_row(
        &self,
        row: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QModelIndex_siblingAtRow(
            self as *const crate::QModelIndex,
            row,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html">QPersistentModelIndex</a> class is used to locate data in a data model.</p>
///
/// C++ class: <span style='color: green;'>```QPersistentModelIndex```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html">QPersistentModelIndex</a> class is used to locate data in a data model.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html">QPersistentModelIndex</a> is a model index that can be stored by an application, and later used to access information in a model. Unlike the <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> class, it is safe to store a <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html">QPersistentModelIndex</a> since the model will ensure that references to items will continue to be valid as long as they can be accessed by the model.</p>
/// <p>It is good practice to check that persistent model indexes are valid before using them.</p></div>
#[repr(C)]
pub struct QPersistentModelIndex {
    _unused: u8,
}
impl QPersistentModelIndex {
    /// <p>Use <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>() instead.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QModelIndex QPersistentModelIndex::child(int row, int column) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpersistentmodelindex-obsolete.html#child">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Use <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>() instead.</p>
    /// <p>Returns the child of the model index that is stored in the given <i>row</i> and <i>column</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html#parent">parent</a>() and <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html#sibling">sibling</a>().</p></div>
    pub unsafe fn child(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPersistentModelIndex_child(
            self as *const crate::QPersistentModelIndex,
            row,
            column,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the column this persistent model index refers to.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QPersistentModelIndex::column() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html#column">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the column this persistent model index refers to.</p></div>
    pub unsafe fn column(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QPersistentModelIndex_column(
            self as *const crate::QPersistentModelIndex,
        )
    }

    /// <p>Sets the persistent model index to refer to the same item in a model as the <i>other</i> persistent model index.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPersistentModelIndex& QPersistentModelIndex::operator=(const QPersistentModelIndex& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the persistent model index to refer to the same item in a model as the <i>other</i> persistent model index.</p></div>
    pub unsafe fn copy_from_q_persistent_model_index(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPersistentModelIndex>>,
    ) -> ::cpp_utils::MutRef<crate::QPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPersistentModelIndex_operator_1(
            self as *mut crate::QPersistentModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPersistentModelIndex>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Sets the persistent model index to refer to the same item in a model as the <i>other</i> model index.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPersistentModelIndex& QPersistentModelIndex::operator=(const QModelIndex& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html#operator-eq-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the persistent model index to refer to the same item in a model as the <i>other</i> model index.</p></div>
    pub unsafe fn copy_from_q_model_index(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::MutRef<crate::QPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPersistentModelIndex_operator_3(
            self as *mut crate::QPersistentModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the data for the given <i>role</i> for the item referred to by the index.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QPersistentModelIndex::data(int role = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the data for the given <i>role</i> for the item referred to by the index.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::ItemDataRole</a> and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">QAbstractItemModel::setData</a>().</p></div>
    pub unsafe fn data_1a(
        &self,
        role: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPersistentModelIndex_data(
            self as *const crate::QPersistentModelIndex,
            role,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the data for the given <i>role</i> for the item referred to by the index.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QPersistentModelIndex::data() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the data for the given <i>role</i> for the item referred to by the index.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::ItemDataRole</a> and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">QAbstractItemModel::setData</a>().</p></div>
    pub unsafe fn data_0a(&self) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPersistentModelIndex_data1(
            self as *const crate::QPersistentModelIndex,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the flags for the item referred to by the index.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFlags<Qt::ItemFlag> QPersistentModelIndex::flags() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html#flags">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the flags for the item referred to by the index.</p>
    /// <p>This function was introduced in  Qt 4.2.</p></div>
    pub unsafe fn flags(&self) -> crate::QFlags<crate::ItemFlag> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPersistentModelIndex_flags(
            self as *const crate::QPersistentModelIndex,
        );
        crate::QFlags::from(ffi_result)
    }

    /// <p>Returns <code>true</code> if this persistent model index is valid; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QPersistentModelIndex::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this persistent model index is valid; otherwise returns <code>false</code>.</p>
    /// <p>A valid index belongs to a model, and has non-negative row and column numbers.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html#model">model</a>(), <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html#row">row</a>(), and <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html#column">column</a>().</p></div>
    pub unsafe fn is_valid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QPersistentModelIndex_isValid(
            self as *const crate::QPersistentModelIndex,
        )
    }

    /// <p>Returns the model that the index belongs to.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QAbstractItemModel* QPersistentModelIndex::model() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html#model">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the model that the index belongs to.</p></div>
    pub unsafe fn model(&self) -> ::cpp_utils::Ptr<crate::QAbstractItemModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPersistentModelIndex_model(
            self as *const crate::QPersistentModelIndex,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPersistentModelIndex::QPersistentModelIndex()```</span>.
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QPersistentModelIndex> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QPersistentModelIndex_QPersistentModelIndex();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a new <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html">QPersistentModelIndex</a> that is a copy of the model <i>index</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPersistentModelIndex::QPersistentModelIndex(const QModelIndex& index)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html#QPersistentModelIndex">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a new <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html">QPersistentModelIndex</a> that is a copy of the model <i>index</i>.</p></div>
    pub unsafe fn new_1a(
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPersistentModelIndex_QPersistentModelIndex1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a new <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html">QPersistentModelIndex</a> that is a copy of the <i>other</i> persistent model index.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPersistentModelIndex::QPersistentModelIndex(const QPersistentModelIndex& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html#QPersistentModelIndex-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a new <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html">QPersistentModelIndex</a> that is a copy of the <i>other</i> persistent model index.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPersistentModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPersistentModelIndex_QPersistentModelIndex2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPersistentModelIndex>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the parent <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> for this persistent index, or an invalid <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> if it has no parent.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QModelIndex QPersistentModelIndex::parent() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html#parent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the parent <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> for this persistent index, or an invalid <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> if it has no parent.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html#sibling">sibling</a>() and <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html#model">model</a>().</p></div>
    pub unsafe fn parent(&self) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPersistentModelIndex_parent(
            self as *const crate::QPersistentModelIndex,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the row this persistent model index refers to.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QPersistentModelIndex::row() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html#row">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the row this persistent model index refers to.</p></div>
    pub unsafe fn row(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QPersistentModelIndex_row(
            self as *const crate::QPersistentModelIndex,
        )
    }

    /// <p>Returns the sibling at <i>row</i> and <i>column</i> or an invalid <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> if there is no sibling at this position.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QModelIndex QPersistentModelIndex::sibling(int row, int column) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html#sibling">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the sibling at <i>row</i> and <i>column</i> or an invalid <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> if there is no sibling at this position.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html#parent">parent</a>().</p></div>
    pub unsafe fn sibling(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPersistentModelIndex_sibling(
            self as *const crate::QPersistentModelIndex,
            row,
            column,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Swaps this persistent modelindex with <i>other</i>. This function is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QPersistentModelIndex::swap(QPersistentModelIndex& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps this persistent modelindex with <i>other</i>. This function is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QPersistentModelIndex>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QPersistentModelIndex_swap(
            self as *mut crate::QPersistentModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QPersistentModelIndex>>::cast_into(
                other,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```const QModelIndex& QPersistentModelIndex::operator const QModelIndex&() const```</span>.
    pub unsafe fn to_q_model_index(&self) -> ::cpp_utils::Ref<crate::QModelIndex> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QPersistentModelIndex_operator_const_QModelIndex_(
                self as *const crate::QPersistentModelIndex,
            );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }
}

pub mod q_abstract_item_model {
    //! C++ type: <span style='color: green;'>```QAbstractItemModel```</span>

    /// <p>This enum describes the way the model changes layout.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QAbstractItemModel::LayoutChangeHint```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#LayoutChangeHint-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the way the model changes layout.</p>
    ///
    /// <p>Note that VerticalSortHint and HorizontalSortHint carry the meaning that items are being moved within the same parent, not moved to a different parent in the model, and not filtered out or in.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct LayoutChangeHint(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for LayoutChangeHint {
        fn from(value: ::std::os::raw::c_int) -> Self {
            LayoutChangeHint(value)
        }
    }

    impl From<LayoutChangeHint> for ::std::os::raw::c_int {
        fn from(value: LayoutChangeHint) -> Self {
            value.0
        }
    }

    impl LayoutChangeHint {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl LayoutChangeHint {
        /// No hint is available. (C++ enum variant: <span style='color: green;'>```NoLayoutChangeHint = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoLayoutChangeHint: crate::q_abstract_item_model::LayoutChangeHint =
            crate::q_abstract_item_model::LayoutChangeHint(0);
        /// Rows are being sorted. (C++ enum variant: <span style='color: green;'>```VerticalSortHint = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const VerticalSortHint: crate::q_abstract_item_model::LayoutChangeHint =
            crate::q_abstract_item_model::LayoutChangeHint(1);
        /// Columns are being sorted. (C++ enum variant: <span style='color: green;'>```HorizontalSortHint = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const HorizontalSortHint: crate::q_abstract_item_model::LayoutChangeHint =
            crate::q_abstract_item_model::LayoutChangeHint(2);
    }

    /// <p>This enum can be used to control the checks performed by <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#checkIndex">QAbstractItemModel::checkIndex</a>().</p>
    ///
    /// C++ enum: <span style='color: green;'>```QAbstractItemModel::CheckIndexOption```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#CheckIndexOption-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum can be used to control the checks performed by <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#checkIndex">QAbstractItemModel::checkIndex</a>().</p>
    ///
    /// <p>This enum was introduced or modified in  Qt 5.11.</p>
    /// <p>The CheckIndexOptions type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;CheckIndexOption&gt;. It stores an OR combination of CheckIndexOption values.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct CheckIndexOption(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for CheckIndexOption {
        fn from(value: ::std::os::raw::c_int) -> Self {
            CheckIndexOption(value)
        }
    }

    impl From<CheckIndexOption> for ::std::os::raw::c_int {
        fn from(value: CheckIndexOption) -> Self {
            value.0
        }
    }

    impl CheckIndexOption {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl CheckIndexOption {
        /// No check options are specified. (C++ enum variant: <span style='color: green;'>```NoOption = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoOption: crate::q_abstract_item_model::CheckIndexOption =
            crate::q_abstract_item_model::CheckIndexOption(0);
        /// The model index passed to <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#checkIndex">QAbstractItemModel::checkIndex</a>() is checked to be a valid model index. (C++ enum variant: <span style='color: green;'>```IndexIsValid = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const IndexIsValid: crate::q_abstract_item_model::CheckIndexOption =
            crate::q_abstract_item_model::CheckIndexOption(1);
        /// Does not perform any check involving the usage of the parent of the index passed to <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#checkIndex">QAbstractItemModel::checkIndex</a>(). (C++ enum variant: <span style='color: green;'>```DoNotUseParent = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const DoNotUseParent: crate::q_abstract_item_model::CheckIndexOption =
            crate::q_abstract_item_model::CheckIndexOption(2);
        /// The parent of the model index passed to <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#checkIndex">QAbstractItemModel::checkIndex</a>() is checked to be an invalid model index. If both this option and DoNotUseParent are specified, then this option is ignored. (C++ enum variant: <span style='color: green;'>```ParentIsInvalid = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const ParentIsInvalid: crate::q_abstract_item_model::CheckIndexOption =
            crate::q_abstract_item_model::CheckIndexOption(4);
    }

    impl From<crate::q_abstract_item_model::CheckIndexOption>
        for crate::QFlags<crate::q_abstract_item_model::CheckIndexOption>
    {
        fn from(value: crate::q_abstract_item_model::CheckIndexOption) -> Self {
            Self::from(value.to_int())
        }
    }

    impl<T: Into<crate::QFlags<crate::q_abstract_item_model::CheckIndexOption>>> std::ops::BitOr<T>
        for crate::q_abstract_item_model::CheckIndexOption
    {
        type Output = crate::QFlags<crate::q_abstract_item_model::CheckIndexOption>;
        fn bitor(self, rhs: T) -> crate::QFlags<crate::q_abstract_item_model::CheckIndexOption> {
            Into::<crate::QFlags<crate::q_abstract_item_model::CheckIndexOption>>::into(self) | rhs
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a> class provides the abstract interface for item model classes.</p>
///
/// C++ class: <span style='color: green;'>```QAbstractItemModel```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a> class provides the abstract interface for item model classes.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a> class defines the standard interface that item models must use to be able to interoperate with other components in the model/view architecture. It is not supposed to be instantiated directly. Instead, you should subclass it to create new models.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a> class is one of the <a href="http://doc.qt.io/qt-5/model-view-programming.html#model-view-classes">Model/View Classes</a> and is part of Qt's <a href="http://doc.qt.io/qt-5/model-view-programming.html">model/view framework</a>. It can be used as the underlying data model for the item view elements in QML or the item view classes in the Qt Widgets module.</p>
/// <p>If you need a model to use with an item view such as QML's List View element or the C++ widgets <a href="http://doc.qt.io/qt-5/qlistview.html">QListView</a> or <a href="http://doc.qt.io/qt-5/qtableview.html">QTableView</a>, you should consider subclassing <a href="http://doc.qt.io/qt-5/qabstractlistmodel.html">QAbstractListModel</a> or <a href="http://doc.qt.io/qt-5/qabstracttablemodel.html">QAbstractTableModel</a> instead of this class.</p>
/// <p>The underlying data model is exposed to views and delegates as a hierarchy of tables. If you do not make use of the hierarchy, then the model is a simple table of rows and columns. Each item has a unique index specified by a <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a>.</p>
/// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/modelindex-no-parent.png" alt=""></p><p>Every item of data that can be accessed via a model has an associated model index. You can obtain this model index using the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">index</a>() function. Each index may have a <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sibling">sibling</a>() index; child items have a <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#parent">parent</a>() index.</p>
/// <p>Each item has a number of data elements associated with it and they can be retrieved by specifying a role (see <a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::ItemDataRole</a>) to the model's <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">data</a>() function. Data for all available roles can be obtained at the same time using the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#itemData">itemData</a>() function.</p>
/// <p>Data for each role is set using a particular <a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::ItemDataRole</a>. Data for individual roles are set individually with <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">setData</a>(), or they can be set for all roles with <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setItemData">setItemData</a>().</p>
/// <p>Items can be queried with <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#flags">flags</a>() (see <a href="http://doc.qt.io/qt-5/qt.html#ItemFlag-enum">Qt::ItemFlag</a>) to see if they can be selected, dragged, or manipulated in other ways.</p>
/// <p>If an item has child objects, <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#hasChildren">hasChildren</a>() returns <code>true</code> for the corresponding index.</p>
/// <p>The model has a <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">rowCount</a>() and a <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnCount">columnCount</a>() for each level of the hierarchy. Rows and columns can be inserted and removed with <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">insertRows</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumns">insertColumns</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">removeRows</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumns">removeColumns</a>().</p>
/// <p>The model emits signals to indicate changes. For example, <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dataChanged">dataChanged</a>() is emitted whenever items of data made available by the model are changed. Changes to the headers supplied by the model cause <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerDataChanged">headerDataChanged</a>() to be emitted. If the structure of the underlying data changes, the model can emit <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#layoutChanged">layoutChanged</a>() to indicate to any attached views that they should redisplay any items shown, taking the new structure into account.</p>
/// <p>The items available through the model can be searched for particular data using the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#match">match</a>() function.</p>
/// <p>To sort the model, you can use <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sort">sort</a>().</p>
/// <a name="subclassing"></a></div>
#[repr(C)]
pub struct QAbstractItemModel {
    _unused: u8,
}
impl QAbstractItemModel {
    /// <p>This signal is emitted whenever a header is changed. The <i>orientation</i> indicates whether the horizontal or vertical header has changed. The sections in the header from the <i>first</i> to the <i>last</i> need to be updated.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractItemModel::headerDataChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerDataChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted whenever a header is changed. The <i>orientation</i> indicates whether the horizontal or vertical header has changed. The sections in the header from the <i>first</i> to the <i>last</i> need to be updated.</p>
    /// <p>When reimplementing the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setHeaderData">setHeaderData</a>() function, this signal must be emitted explicitly.</p>
    /// <p>If you are changing the number of columns or rows you do not need to emit this signal, but use the begin/end functions (refer to the section on subclassing in the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a> class description for details).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerData">headerData</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setHeaderData">setHeaderData</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dataChanged">dataChanged</a>().</p></div>
    pub fn header_data_changed(
        &self,
    ) -> crate::Signal<(
        crate::Orientation,
        ::std::os::raw::c_int,
        ::std::os::raw::c_int,
    )> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2headerDataChanged(Qt::Orientation,int,int)\0",
                ),
            )
        }
    }

    /// <p>This signal is emitted just before rows are inserted into the model. The new items will be positioned between <i>start</i> and <i>end</i> inclusive, under the given <i>parent</i> item.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractItemModel::rowsAboutToBeInserted` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowsAboutToBeInserted">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted just before rows are inserted into the model. The new items will be positioned between <i>start</i> and <i>end</i> inclusive, under the given <i>parent</i> item.</p>
    /// <p><b>Note: </b>Components connected to this signal use it to adapt to changes in the model's dimensions. It can only be emitted by the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a> implementation, and cannot be explicitly emitted in subclass code.</p><p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">insertRows</a>() and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginInsertRows">beginInsertRows</a>().</p></div>
    pub fn rows_about_to_be_inserted(
        &self,
    ) -> crate::Signal<(
        *const crate::QModelIndex,
        ::std::os::raw::c_int,
        ::std::os::raw::c_int,
    )> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2rowsAboutToBeInserted(const QModelIndex&,int,int)\0",
                ),
            )
        }
    }

    /// <p>This signal is emitted after rows have been inserted into the model. The new items are those between <i>first</i> and <i>last</i> inclusive, under the given <i>parent</i> item.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractItemModel::rowsInserted` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowsInserted">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted after rows have been inserted into the model. The new items are those between <i>first</i> and <i>last</i> inclusive, under the given <i>parent</i> item.</p>
    /// <p><b>Note: </b>Components connected to this signal use it to adapt to changes in the model's dimensions. It can only be emitted by the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a> implementation, and cannot be explicitly emitted in subclass code.</p><p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">insertRows</a>() and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginInsertRows">beginInsertRows</a>().</p></div>
    pub fn rows_inserted(
        &self,
    ) -> crate::Signal<(
        *const crate::QModelIndex,
        ::std::os::raw::c_int,
        ::std::os::raw::c_int,
    )> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2rowsInserted(const QModelIndex&,int,int)\0",
                ),
            )
        }
    }

    /// <p>This signal is emitted just before rows are removed from the model. The items that will be removed are those between <i>first</i> and <i>last</i> inclusive, under the given <i>parent</i> item.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractItemModel::rowsAboutToBeRemoved` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowsAboutToBeRemoved">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted just before rows are removed from the model. The items that will be removed are those between <i>first</i> and <i>last</i> inclusive, under the given <i>parent</i> item.</p>
    /// <p><b>Note: </b>Components connected to this signal use it to adapt to changes in the model's dimensions. It can only be emitted by the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a> implementation, and cannot be explicitly emitted in subclass code.</p><p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">removeRows</a>() and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginRemoveRows">beginRemoveRows</a>().</p></div>
    pub fn rows_about_to_be_removed(
        &self,
    ) -> crate::Signal<(
        *const crate::QModelIndex,
        ::std::os::raw::c_int,
        ::std::os::raw::c_int,
    )> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2rowsAboutToBeRemoved(const QModelIndex&,int,int)\0",
                ),
            )
        }
    }

    /// <p>This signal is emitted after rows have been removed from the model. The removed items are those between <i>first</i> and <i>last</i> inclusive, under the given <i>parent</i> item.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractItemModel::rowsRemoved` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowsRemoved">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted after rows have been removed from the model. The removed items are those between <i>first</i> and <i>last</i> inclusive, under the given <i>parent</i> item.</p>
    /// <p><b>Note: </b>Components connected to this signal use it to adapt to changes in the model's dimensions. It can only be emitted by the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a> implementation, and cannot be explicitly emitted in subclass code.</p><p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">removeRows</a>() and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginRemoveRows">beginRemoveRows</a>().</p></div>
    pub fn rows_removed(
        &self,
    ) -> crate::Signal<(
        *const crate::QModelIndex,
        ::std::os::raw::c_int,
        ::std::os::raw::c_int,
    )> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2rowsRemoved(const QModelIndex&,int,int)\0",
                ),
            )
        }
    }

    /// <p>This signal is emitted just before columns are inserted into the model. The new items will be positioned between <i>first</i> and <i>last</i> inclusive, under the given <i>parent</i> item.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractItemModel::columnsAboutToBeInserted` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnsAboutToBeInserted">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted just before columns are inserted into the model. The new items will be positioned between <i>first</i> and <i>last</i> inclusive, under the given <i>parent</i> item.</p>
    /// <p><b>Note: </b>Components connected to this signal use it to adapt to changes in the model's dimensions. It can only be emitted by the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a> implementation, and cannot be explicitly emitted in subclass code.</p><p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumns">insertColumns</a>() and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginInsertColumns">beginInsertColumns</a>().</p></div>
    pub fn columns_about_to_be_inserted(
        &self,
    ) -> crate::Signal<(
        *const crate::QModelIndex,
        ::std::os::raw::c_int,
        ::std::os::raw::c_int,
    )> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2columnsAboutToBeInserted(const QModelIndex&,int,int)\0",
                ),
            )
        }
    }

    /// <p>This signal is emitted after columns have been inserted into the model. The new items are those between <i>first</i> and <i>last</i> inclusive, under the given <i>parent</i> item.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractItemModel::columnsInserted` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnsInserted">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted after columns have been inserted into the model. The new items are those between <i>first</i> and <i>last</i> inclusive, under the given <i>parent</i> item.</p>
    /// <p><b>Note: </b>Components connected to this signal use it to adapt to changes in the model's dimensions. It can only be emitted by the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a> implementation, and cannot be explicitly emitted in subclass code.</p><p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumns">insertColumns</a>() and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginInsertColumns">beginInsertColumns</a>().</p></div>
    pub fn columns_inserted(
        &self,
    ) -> crate::Signal<(
        *const crate::QModelIndex,
        ::std::os::raw::c_int,
        ::std::os::raw::c_int,
    )> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2columnsInserted(const QModelIndex&,int,int)\0",
                ),
            )
        }
    }

    /// <p>This signal is emitted just before columns are removed from the model. The items to be removed are those between <i>first</i> and <i>last</i> inclusive, under the given <i>parent</i> item.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractItemModel::columnsAboutToBeRemoved` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnsAboutToBeRemoved">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted just before columns are removed from the model. The items to be removed are those between <i>first</i> and <i>last</i> inclusive, under the given <i>parent</i> item.</p>
    /// <p><b>Note: </b>Components connected to this signal use it to adapt to changes in the model's dimensions. It can only be emitted by the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a> implementation, and cannot be explicitly emitted in subclass code.</p><p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumns">removeColumns</a>() and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginRemoveColumns">beginRemoveColumns</a>().</p></div>
    pub fn columns_about_to_be_removed(
        &self,
    ) -> crate::Signal<(
        *const crate::QModelIndex,
        ::std::os::raw::c_int,
        ::std::os::raw::c_int,
    )> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2columnsAboutToBeRemoved(const QModelIndex&,int,int)\0",
                ),
            )
        }
    }

    /// <p>This signal is emitted after columns have been removed from the model. The removed items are those between <i>first</i> and <i>last</i> inclusive, under the given <i>parent</i> item.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractItemModel::columnsRemoved` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnsRemoved">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted after columns have been removed from the model. The removed items are those between <i>first</i> and <i>last</i> inclusive, under the given <i>parent</i> item.</p>
    /// <p><b>Note: </b>Components connected to this signal use it to adapt to changes in the model's dimensions. It can only be emitted by the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a> implementation, and cannot be explicitly emitted in subclass code.</p><p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumns">removeColumns</a>() and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginRemoveColumns">beginRemoveColumns</a>().</p></div>
    pub fn columns_removed(
        &self,
    ) -> crate::Signal<(
        *const crate::QModelIndex,
        ::std::os::raw::c_int,
        ::std::os::raw::c_int,
    )> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2columnsRemoved(const QModelIndex&,int,int)\0",
                ),
            )
        }
    }

    /// <p>This signal is emitted when <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginResetModel">beginResetModel</a>() is called, before the model's internal state (e.g. persistent model indexes) has been invalidated.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractItemModel::modelAboutToBeReset` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#modelAboutToBeReset">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginResetModel">beginResetModel</a>() is called, before the model's internal state (e.g. persistent model indexes) has been invalidated.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginResetModel">beginResetModel</a>() and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#modelReset">modelReset</a>().</p></div>
    pub fn model_about_to_be_reset(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2modelAboutToBeReset()\0"),
            )
        }
    }

    /// <p>This signal is emitted when <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#endResetModel">endResetModel</a>() is called, after the model's internal state (e.g. persistent model indexes) has been invalidated.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractItemModel::modelReset` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#modelReset">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#endResetModel">endResetModel</a>() is called, after the model's internal state (e.g. persistent model indexes) has been invalidated.</p>
    /// <p>Note that if a model is reset it should be considered that all information previously retrieved from it is invalid. This includes but is not limited to the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">rowCount</a>() and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnCount">columnCount</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#flags">flags</a>(), data retrieved through <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">data</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#roleNames">roleNames</a>().</p>
    /// <p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#endResetModel">endResetModel</a>() and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#modelAboutToBeReset">modelAboutToBeReset</a>().</p></div>
    pub fn model_reset(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2modelReset()\0"),
            )
        }
    }

    /// <p>This signal is emitted just before rows are moved within the model. The items that will be moved are those between <i>sourceStart</i> and <i>sourceEnd</i> inclusive, under the given <i>sourceParent</i> item. They will be moved to <i>destinationParent</i> starting at the row <i>destinationRow</i>.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractItemModel::rowsAboutToBeMoved` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowsAboutToBeMoved">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted just before rows are moved within the model. The items that will be moved are those between <i>sourceStart</i> and <i>sourceEnd</i> inclusive, under the given <i>sourceParent</i> item. They will be moved to <i>destinationParent</i> starting at the row <i>destinationRow</i>.</p>
    /// <p><b>Note:</b> Components connected to this signal use it to adapt to changes in the model's dimensions. It can only be emitted by the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a> implementation, and cannot be explicitly emitted in subclass code.</p>
    /// <p>This function was introduced in  Qt 4.6.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginMoveRows">beginMoveRows</a>().</p></div>
    pub fn rows_about_to_be_moved(
        &self,
    ) -> crate::Signal<(
        *const crate::QModelIndex,
        ::std::os::raw::c_int,
        ::std::os::raw::c_int,
        *const crate::QModelIndex,
        ::std::os::raw::c_int,
    )> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2rowsAboutToBeMoved(const QModelIndex&,int,int,const QModelIndex&,int)\0",
                ),
            )
        }
    }

    /// <p>This signal is emitted after rows have been moved within the model. The items between <i>start</i> and <i>end</i> inclusive, under the given <i>parent</i> item have been moved to <i>destination</i> starting at the row <i>row</i>.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractItemModel::rowsMoved` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowsMoved">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted after rows have been moved within the model. The items between <i>start</i> and <i>end</i> inclusive, under the given <i>parent</i> item have been moved to <i>destination</i> starting at the row <i>row</i>.</p>
    /// <p><b>Note:</b> Components connected to this signal use it to adapt to changes in the model's dimensions. It can only be emitted by the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a> implementation, and cannot be explicitly emitted in subclass code.</p>
    /// <p>This function was introduced in  Qt 4.6.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginMoveRows">beginMoveRows</a>().</p></div>
    pub fn rows_moved(
        &self,
    ) -> crate::Signal<(
        *const crate::QModelIndex,
        ::std::os::raw::c_int,
        ::std::os::raw::c_int,
        *const crate::QModelIndex,
        ::std::os::raw::c_int,
    )> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2rowsMoved(const QModelIndex&,int,int,const QModelIndex&,int)\0",
                ),
            )
        }
    }

    /// <p>This signal is emitted just before columns are moved within the model. The items that will be moved are those between <i>sourceStart</i> and <i>sourceEnd</i> inclusive, under the given <i>sourceParent</i> item. They will be moved to <i>destinationParent</i> starting at the column <i>destinationColumn</i>.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractItemModel::columnsAboutToBeMoved` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnsAboutToBeMoved">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted just before columns are moved within the model. The items that will be moved are those between <i>sourceStart</i> and <i>sourceEnd</i> inclusive, under the given <i>sourceParent</i> item. They will be moved to <i>destinationParent</i> starting at the column <i>destinationColumn</i>.</p>
    /// <p><b>Note:</b> Components connected to this signal use it to adapt to changes in the model's dimensions. It can only be emitted by the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a> implementation, and cannot be explicitly emitted in subclass code.</p>
    /// <p>This function was introduced in  Qt 4.6.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginMoveRows">beginMoveRows</a>().</p></div>
    pub fn columns_about_to_be_moved(
        &self,
    ) -> crate::Signal<(
        *const crate::QModelIndex,
        ::std::os::raw::c_int,
        ::std::os::raw::c_int,
        *const crate::QModelIndex,
        ::std::os::raw::c_int,
    )> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2columnsAboutToBeMoved(const QModelIndex&,int,int,const QModelIndex&,int)\0",
                ),
            )
        }
    }

    /// <p>This signal is emitted after columns have been moved within the model. The items between <i>start</i> and <i>end</i> inclusive, under the given <i>parent</i> item have been moved to <i>destination</i> starting at the column <i>column</i>.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractItemModel::columnsMoved` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnsMoved">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted after columns have been moved within the model. The items between <i>start</i> and <i>end</i> inclusive, under the given <i>parent</i> item have been moved to <i>destination</i> starting at the column <i>column</i>.</p>
    /// <p><b>Note:</b> Components connected to this signal use it to adapt to changes in the model's dimensions. It can only be emitted by the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a> implementation, and cannot be explicitly emitted in subclass code.</p>
    /// <p>This function was introduced in  Qt 4.6.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginMoveRows">beginMoveRows</a>().</p></div>
    pub fn columns_moved(
        &self,
    ) -> crate::Signal<(
        *const crate::QModelIndex,
        ::std::os::raw::c_int,
        ::std::os::raw::c_int,
        *const crate::QModelIndex,
        ::std::os::raw::c_int,
    )> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2columnsMoved(const QModelIndex&,int,int,const QModelIndex&,int)\0",
                ),
            )
        }
    }

    /// <p>Lets the model know that it should submit cached information to permanent storage. This function is typically used for row editing.</p>
    ///
    /// Returns a built-in Qt slot `QAbstractItemModel::submit` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#submit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Lets the model know that it should submit cached information to permanent storage. This function is typically used for row editing.</p>
    /// <p>Returns <code>true</code> if there is no error; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#revert">revert</a>().</p></div>
    pub fn slot_submit(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1submit()\0"),
            )
        }
    }

    /// <p>Lets the model know that it should discard cached information. This function is typically used for row editing.</p>
    ///
    /// Returns a built-in Qt slot `QAbstractItemModel::revert` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#revert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Lets the model know that it should discard cached information. This function is typically used for row editing.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#submit">submit</a>().</p></div>
    pub fn slot_revert(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1revert()\0"),
            )
        }
    }

    /// <p>This slot is called just after the internal data of a model is cleared while it is being reset.</p>
    ///
    /// Returns a built-in Qt slot `QAbstractItemModel::resetInternalData` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#resetInternalData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This slot is called just after the internal data of a model is cleared while it is being reset.</p>
    /// <p>This slot is provided the convenience of subclasses of concrete proxy models, such as subclasses of <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html">QSortFilterProxyModel</a> which maintain extra data.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">class</span> CustomDataProxy : <span class="keyword">public</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html">QSortFilterProxyModel</a></span>
    ///   {
    /// &#32;     Q_OBJECT
    ///   <span class="keyword">public</span>:
    /// &#32;     CustomDataProxy(<span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span> <span class="operator">*</span>parent)
    /// &#32;   &#32;   : <span class="type"><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html">QSortFilterProxyModel</a></span>(parent)
    /// &#32;     {
    /// &#32;     }
    ///
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> data(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a></span> <span class="operator">&amp;</span>index<span class="operator">,</span> <span class="type">int</span> role)
    /// &#32;     {
    /// &#32;   &#32;     <span class="keyword">if</span> (role <span class="operator">!</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>BackgroundRole)
    /// &#32;   &#32;   &#32;     <span class="keyword">return</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html">QSortFilterProxyModel</a></span><span class="operator">::</span>data(index<span class="operator">,</span> role);
    ///
    /// &#32;   &#32;     <span class="keyword">if</span> (m_customData<span class="operator">.</span>contains(index<span class="operator">.</span>row()))
    /// &#32;   &#32;   &#32;     <span class="keyword">return</span> m_customData<span class="operator">.</span>value(index<span class="operator">.</span>row());
    /// &#32;   &#32;     <span class="keyword">return</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html">QSortFilterProxyModel</a></span><span class="operator">::</span>data(index<span class="operator">,</span> role);
    /// &#32;     }
    ///
    ///   <span class="keyword">private</span> <span class="keyword">slots</span>:
    /// &#32;     <span class="type">void</span> resetInternalData()
    /// &#32;     {
    /// &#32;   &#32;     m_customData<span class="operator">.</span>clear();
    /// &#32;     }
    ///
    ///   <span class="keyword">private</span>:
    /// &#32;   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html">QHash</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span><span class="operator">&gt;</span> m_customData;
    ///   };
    ///
    /// </pre>
    /// <p><b>Note: </b>Due to a mistake, this slot is missing in Qt 5.0.</p><p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#modelAboutToBeReset">modelAboutToBeReset</a>() and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#modelReset">modelReset</a>().</p></div>
    pub fn slot_reset_internal_data(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1resetInternalData()\0"),
            )
        }
    }

    /// <p>This signal is emitted whenever the data in an existing item changes.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractItemModel::dataChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dataChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted whenever the data in an existing item changes.</p>
    /// <p>If the items are of the same parent, the affected ones are those between <i>topLeft</i> and <i>bottomRight</i> inclusive. If the items do not have the same parent, the behavior is undefined.</p>
    /// <p>When reimplementing the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">setData</a>() function, this signal must be emitted explicitly.</p>
    /// <p>The optional <i>roles</i> argument can be used to specify which data roles have actually been modified. An empty vector in the roles argument means that all roles should be considered modified. The order of elements in the roles argument does not have any relevance.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerDataChanged">headerDataChanged</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">setData</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#layoutChanged">layoutChanged</a>().</p></div>
    pub fn data_changed(
        &self,
    ) -> crate::Signal<(
        *const crate::QModelIndex,
        *const crate::QModelIndex,
        *const crate::QVectorOfInt,
    )> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2dataChanged(const QModelIndex&,const QModelIndex&,const QVector< int >&)\0",
                ),
            )
        }
    }

    /// <p>This signal is emitted whenever the layout of items exposed by the model has changed; for example, when the model has been sorted. When this signal is received by a view, it should update the layout of items to reflect this change.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractItemModel::layoutChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#layoutChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted whenever the layout of items exposed by the model has changed; for example, when the model has been sorted. When this signal is received by a view, it should update the layout of items to reflect this change.</p>
    /// <p>When subclassing <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a> or <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html">QAbstractProxyModel</a>, ensure that you emit <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#layoutAboutToBeChanged">layoutAboutToBeChanged</a>() before changing the order of items or altering the structure of the data you expose to views, and emit layoutChanged() after changing the layout.</p>
    /// <p>The optional <i>parents</i> parameter is used to give a more specific notification about what parts of the layout of the model are changing. An empty list indicates a change to the layout of the entire model. The order of elements in the <i>parents</i> list is not significant. The optional <i>hint</i> parameter is used to give a hint about what is happening while the model is relayouting.</p>
    /// <p>Subclasses should update any persistent model indexes before emitting layoutChanged(). In other words, when the structure changes:</p>
    /// <ul>
    /// <li>emit <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#layoutAboutToBeChanged">layoutAboutToBeChanged</a></li>
    /// <li>Remember the <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> that will change</li>
    /// <li>Update your internal data</li>
    /// <li>Call <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#changePersistentIndex">changePersistentIndex</a>()</li>
    /// <li>emit layoutChanged</li>
    /// </ul>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#layoutAboutToBeChanged">layoutAboutToBeChanged</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dataChanged">dataChanged</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerDataChanged">headerDataChanged</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#modelReset">modelReset</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#changePersistentIndex">changePersistentIndex</a>().</p></div>
    pub fn layout_changed(
        &self,
    ) -> crate::Signal<(
        *const crate::QListOfQPersistentModelIndex,
        crate::q_abstract_item_model::LayoutChangeHint,
    )> {
        unsafe {
            crate::Signal::new(::cpp_utils::Ref::from_raw_ref(self), ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2layoutChanged(const QList< QPersistentModelIndex >&,QAbstractItemModel::LayoutChangeHint)\0"))
        }
    }

    /// <p>This signal is emitted just before the layout of a model is changed. Components connected to this signal use it to adapt to changes in the model's layout.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractItemModel::layoutAboutToBeChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#layoutAboutToBeChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted just before the layout of a model is changed. Components connected to this signal use it to adapt to changes in the model's layout.</p>
    /// <p>Subclasses should update any persistent model indexes after emitting layoutAboutToBeChanged().</p>
    /// <p>The optional <i>parents</i> parameter is used to give a more specific notification about what parts of the layout of the model are changing. An empty list indicates a change to the layout of the entire model. The order of elements in the <i>parents</i> list is not significant. The optional <i>hint</i> parameter is used to give a hint about what is happening while the model is relayouting.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#layoutChanged">layoutChanged</a>() and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#changePersistentIndex">changePersistentIndex</a>().</p></div>
    pub fn layout_about_to_be_changed(
        &self,
    ) -> crate::Signal<(
        *const crate::QListOfQPersistentModelIndex,
        crate::q_abstract_item_model::LayoutChangeHint,
    )> {
        unsafe {
            crate::Signal::new(::cpp_utils::Ref::from_raw_ref(self), ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2layoutAboutToBeChanged(const QList< QPersistentModelIndex >&,QAbstractItemModel::LayoutChangeHint)\0"))
        }
    }

    /// <p>Returns a model index for the buddy of the item represented by <i>index</i>. When the user wants to edit an item, the view will call this function to check whether another item in the model should be edited instead. Then, the view will construct a delegate using the model index returned by the buddy item.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QAbstractItemModel::buddy(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#buddy">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a model index for the buddy of the item represented by <i>index</i>. When the user wants to edit an item, the view will call this function to check whether another item in the model should be edited instead. Then, the view will construct a delegate using the model index returned by the buddy item.</p>
    /// <p>The default implementation of this function has each item as its own buddy.</p></div>
    pub unsafe fn buddy(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_buddy(
            self as *const crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if a model can accept a drop of the <i>data</i>. This default implementation only checks if <i>data</i> has at least one format in the list of <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeTypes">mimeTypes</a>() and if <i>action</i> is among the model's <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#supportedDropActions">supportedDropActions</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractItemModel::canDropMimeData(const QMimeData* data, Qt::DropAction action, int row, int column, const QModelIndex& parent) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#canDropMimeData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if a model can accept a drop of the <i>data</i>. This default implementation only checks if <i>data</i> has at least one format in the list of <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeTypes">mimeTypes</a>() and if <i>action</i> is among the model's <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#supportedDropActions">supportedDropActions</a>().</p>
    /// <p>Reimplement this function in your custom model, if you want to test whether the <i>data</i> can be dropped at <i>row</i>, <i>column</i>, <i>parent</i> with <i>action</i>. If you don't need that test, it is not necessary to reimplement this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dropMimeData">dropMimeData</a>() and <a href="http://doc.qt.io/qt-5/model-view-programming.html#using-drag-and-drop-with-item-views">Using drag and drop with item views</a>.</p></div>
    pub unsafe fn can_drop_mime_data(
        &self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QMimeData>>,
        action: crate::DropAction,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_canDropMimeData(
            self as *const crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QMimeData>>::cast_into(data)
                .as_raw_ptr(),
            action,
            row,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if there is more data available for <i>parent</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractItemModel::canFetchMore(const QModelIndex& parent) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#canFetchMore">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if there is more data available for <i>parent</i>; otherwise returns <code>false</code>.</p>
    /// <p>The default implementation always returns <code>false</code>.</p>
    /// <p>If canFetchMore() returns <code>true</code>, the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#fetchMore">fetchMore</a>() function should be called. This is the behavior of <a href="http://doc.qt.io/qt-5/qabstractitemview.html">QAbstractItemView</a>, for example.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#fetchMore">fetchMore</a>().</p></div>
    pub unsafe fn can_fetch_more(
        &self,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_canFetchMore(
            self as *const crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>This function checks whether <i>index</i> is a legal model index for this model. A legal model index is either an invalid model index, or a valid model index for which all the following holds:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QAbstractItemModel::checkIndex(const QModelIndex& index, QFlags<QAbstractItemModel::CheckIndexOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#checkIndex">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function checks whether <i>index</i> is a legal model index for this model. A legal model index is either an invalid model index, or a valid model index for which all the following holds:</p>
    /// <ul>
    /// <li>the index' model is <code>this</code>;</li>
    /// <li>the index' row is greater or equal than zero;</li>
    /// <li>the index' row is less than the row count for the index' parent;</li>
    /// <li>the index' column is greater or equal than zero;</li>
    /// <li>the index' column is less than the column count for the index' parent.</li>
    /// </ul>
    /// <p>The <i>options</i> argument may change some of these checks. If <i>options</i> contains <code>IndexIsValid</code>, then <i>index</i> must be a valid index; this is useful when reimplementing functions such as <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">data</a>() or <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">setData</a>(), which expect valid indexes.</p>
    /// <p>If <i>options</i> contains <code>DoNotUseParent</code>, then the checks that would call <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#parent">parent</a>() are omitted; this allows calling this function from a <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#parent">parent</a>() reimplementation (otherwise, this would result in endless recursion and a crash).</p>
    /// <p>If <i>options</i> does not contain <code>DoNotUseParent</code>, and it contains <code>ParentIsInvalid</code>, then an additional check is performed: the parent index is checked for not being valid. This is useful when implementing flat models such as lists or tables, where no model index should have a valid parent index.</p>
    /// <p>This function returns true if all the checks succeeded, and false otherwise. This allows to use the function in <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT">Q_ASSERT</a> and similar other debugging mechanisms. If some check failed, a warning message will be printed in the <code>qt.core.qabstractitemmodel.checkindex</code> logging category, containing some information that may be useful for debugging the failure.</p>
    /// <p><b>Note: </b>This function is a debugging helper for implementing your own item models. When developing complex models, as well as when building complicated model hierarchies (e.g. using proxy models), it is useful to call this function in order to catch bugs relative to illegal model indices (as defined above) accidentally passed to some <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a> API.</p><p><b>Warning:</b> Note that it's undefined behavior to pass illegal indices to item models, so applications must refrain from doing so, and not rely on any "defensive" programming that item models could employ to handle illegal indexes gracefully.</p>
    /// <p>This function was introduced in  Qt 5.11.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn check_index_2a(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        options: crate::QFlags<crate::q_abstract_item_model::CheckIndexOption>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_checkIndex(
            self as *const crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
            options.to_int(),
        )
    }

    /// <p>This function checks whether <i>index</i> is a legal model index for this model. A legal model index is either an invalid model index, or a valid model index for which all the following holds:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QAbstractItemModel::checkIndex(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#checkIndex">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function checks whether <i>index</i> is a legal model index for this model. A legal model index is either an invalid model index, or a valid model index for which all the following holds:</p>
    /// <ul>
    /// <li>the index' model is <code>this</code>;</li>
    /// <li>the index' row is greater or equal than zero;</li>
    /// <li>the index' row is less than the row count for the index' parent;</li>
    /// <li>the index' column is greater or equal than zero;</li>
    /// <li>the index' column is less than the column count for the index' parent.</li>
    /// </ul>
    /// <p>The <i>options</i> argument may change some of these checks. If <i>options</i> contains <code>IndexIsValid</code>, then <i>index</i> must be a valid index; this is useful when reimplementing functions such as <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">data</a>() or <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">setData</a>(), which expect valid indexes.</p>
    /// <p>If <i>options</i> contains <code>DoNotUseParent</code>, then the checks that would call <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#parent">parent</a>() are omitted; this allows calling this function from a <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#parent">parent</a>() reimplementation (otherwise, this would result in endless recursion and a crash).</p>
    /// <p>If <i>options</i> does not contain <code>DoNotUseParent</code>, and it contains <code>ParentIsInvalid</code>, then an additional check is performed: the parent index is checked for not being valid. This is useful when implementing flat models such as lists or tables, where no model index should have a valid parent index.</p>
    /// <p>This function returns true if all the checks succeeded, and false otherwise. This allows to use the function in <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT">Q_ASSERT</a> and similar other debugging mechanisms. If some check failed, a warning message will be printed in the <code>qt.core.qabstractitemmodel.checkindex</code> logging category, containing some information that may be useful for debugging the failure.</p>
    /// <p><b>Note: </b>This function is a debugging helper for implementing your own item models. When developing complex models, as well as when building complicated model hierarchies (e.g. using proxy models), it is useful to call this function in order to catch bugs relative to illegal model indices (as defined above) accidentally passed to some <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a> API.</p><p><b>Warning:</b> Note that it's undefined behavior to pass illegal indices to item models, so applications must refrain from doing so, and not rely on any "defensive" programming that item models could employ to handle illegal indexes gracefully.</p>
    /// <p>This function was introduced in  Qt 5.11.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn check_index_1a(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_checkIndex1(
            self as *const crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the number of columns for the children of the given <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual int QAbstractItemModel::columnCount(const QModelIndex& parent = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of columns for the children of the given <i>parent</i>.</p>
    /// <p>In most subclasses, the number of columns is independent of the <i>parent</i>.</p>
    /// <p>For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">int</span> DomModel<span class="operator">::</span>columnCount(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a></span> <span class="operator">&amp;</span><span class="comment">/*parent*/</span>) <span class="keyword">const</span>
    ///   {
    /// &#32;     <span class="keyword">return</span> <span class="number">3</span>;
    ///   }
    ///
    /// </pre>
    /// <p><b>Note: </b>When implementing a table based model, columnCount() should return 0 when the parent is valid.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">rowCount</a>().</p></div>
    pub unsafe fn column_count_1a(
        &self,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_columnCount(
            self as *const crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the number of columns for the children of the given <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual int QAbstractItemModel::columnCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of columns for the children of the given <i>parent</i>.</p>
    /// <p>In most subclasses, the number of columns is independent of the <i>parent</i>.</p>
    /// <p>For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">int</span> DomModel<span class="operator">::</span>columnCount(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a></span> <span class="operator">&amp;</span><span class="comment">/*parent*/</span>) <span class="keyword">const</span>
    ///   {
    /// &#32;     <span class="keyword">return</span> <span class="number">3</span>;
    ///   }
    ///
    /// </pre>
    /// <p><b>Note: </b>When implementing a table based model, columnCount() should return 0 when the parent is valid.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">rowCount</a>().</p></div>
    pub unsafe fn column_count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_columnCount1(
            self as *const crate::QAbstractItemModel,
        )
    }

    /// <p>Returns the data stored under the given <i>role</i> for the item referred to by the <i>index</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual QVariant QAbstractItemModel::data(const QModelIndex& index, int role = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the data stored under the given <i>role</i> for the item referred to by the <i>index</i>.</p>
    /// <p><b>Note: </b>If you do not have a value to return, return an <b>invalid</b> <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> instead of returning 0.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::ItemDataRole</a>, <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">setData</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerData">headerData</a>().</p></div>
    pub unsafe fn data_2a(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        role: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_data(
            self as *const crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
            role,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the data stored under the given <i>role</i> for the item referred to by the <i>index</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual QVariant QAbstractItemModel::data(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the data stored under the given <i>role</i> for the item referred to by the <i>index</i>.</p>
    /// <p><b>Note: </b>If you do not have a value to return, return an <b>invalid</b> <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> instead of returning 0.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::ItemDataRole</a>, <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">setData</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerData">headerData</a>().</p></div>
    pub unsafe fn data_1a(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_data1(
            self as *const crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Handles the <i>data</i> supplied by a drag and drop operation that ended with the given <i>action</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractItemModel::dropMimeData(const QMimeData* data, Qt::DropAction action, int row, int column, const QModelIndex& parent)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dropMimeData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Handles the <i>data</i> supplied by a drag and drop operation that ended with the given <i>action</i>.</p>
    /// <p>Returns <code>true</code> if the data and action were handled by the model; otherwise returns <code>false</code>.</p>
    /// <p>The specified <i>row</i>, <i>column</i> and <i>parent</i> indicate the location of an item in the model where the operation ended. It is the responsibility of the model to complete the action at the correct location.</p>
    /// <p>For instance, a drop action on an item in a <a href="http://doc.qt.io/qt-5/qtreeview.html">QTreeView</a> can result in new items either being inserted as children of the item specified by <i>row</i>, <i>column</i>, and <i>parent</i>, or as siblings of the item.</p>
    /// <p>When <i>row</i> and <i>column</i> are -1 it means that the dropped data should be considered as dropped directly on <i>parent</i>. Usually this will mean appending the data as child items of <i>parent</i>. If <i>row</i> and <i>column</i> are greater than or equal zero, it means that the drop occurred just before the specified <i>row</i> and <i>column</i> in the specified <i>parent</i>.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeTypes">mimeTypes</a>() member is called to get the list of acceptable MIME types. This default implementation assumes the default implementation of <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeTypes">mimeTypes</a>(), which returns a single default MIME type. If you reimplement <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeTypes">mimeTypes</a>() in your custom model to return multiple MIME types, you must reimplement this function to make use of them.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#supportedDropActions">supportedDropActions</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#canDropMimeData">canDropMimeData</a>(), and <a href="http://doc.qt.io/qt-5/model-view-programming.html#using-drag-and-drop-with-item-views">Using drag and drop with item views</a>.</p></div>
    pub unsafe fn drop_mime_data(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QMimeData>>,
        action: crate::DropAction,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_dropMimeData(
            self as *mut crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QMimeData>>::cast_into(data)
                .as_raw_ptr(),
            action,
            row,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Fetches any available data for the items with the parent specified by the <i>parent</i> index.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual void QAbstractItemModel::fetchMore(const QModelIndex& parent)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#fetchMore">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Fetches any available data for the items with the parent specified by the <i>parent</i> index.</p>
    /// <p>Reimplement this if you are populating your model incrementally.</p>
    /// <p>The default implementation does nothing.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#canFetchMore">canFetchMore</a>().</p></div>
    pub unsafe fn fetch_more(
        &mut self,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_fetchMore(
            self as *mut crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the item flags for the given <i>index</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QFlags<Qt::ItemFlag> QAbstractItemModel::flags(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#flags">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item flags for the given <i>index</i>.</p>
    /// <p>The base class implementation returns a combination of flags that enables the item (<code>ItemIsEnabled</code>) and allows it to be selected (<code>ItemIsSelectable</code>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qt.html#ItemFlag-enum">Qt::ItemFlags</a>.</p></div>
    pub unsafe fn flags(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> crate::QFlags<crate::ItemFlag> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_flags(
            self as *const crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        );
        crate::QFlags::from(ffi_result)
    }

    /// <p>Returns <code>true</code> if <i>parent</i> has any children; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractItemModel::hasChildren(const QModelIndex& parent = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#hasChildren">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>parent</i> has any children; otherwise returns <code>false</code>.</p>
    /// <p>Use <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">rowCount</a>() on the parent to find out the number of children.</p>
    /// <p>Note that it is undefined behavior to report that a particular index hasChildren with this method if the same index has the flag <a href="http://doc.qt.io/qt-5/qt.html#ItemFlag-enum">Qt::ItemNeverHasChildren</a> set.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#parent">parent</a>() and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">index</a>().</p></div>
    pub unsafe fn has_children_1a(
        &self,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_hasChildren(
            self as *const crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if <i>parent</i> has any children; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractItemModel::hasChildren() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#hasChildren">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>parent</i> has any children; otherwise returns <code>false</code>.</p>
    /// <p>Use <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">rowCount</a>() on the parent to find out the number of children.</p>
    /// <p>Note that it is undefined behavior to report that a particular index hasChildren with this method if the same index has the flag <a href="http://doc.qt.io/qt-5/qt.html#ItemFlag-enum">Qt::ItemNeverHasChildren</a> set.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#parent">parent</a>() and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">index</a>().</p></div>
    pub unsafe fn has_children_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_hasChildren1(
            self as *const crate::QAbstractItemModel,
        )
    }

    /// <p>Returns <code>true</code> if the model returns a valid <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> for <i>row</i> and <i>column</i> with <i>parent</i>, otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QAbstractItemModel::hasIndex(int row, int column, const QModelIndex& parent = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#hasIndex">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the model returns a valid <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> for <i>row</i> and <i>column</i> with <i>parent</i>, otherwise returns <code>false</code>.</p></div>
    pub unsafe fn has_index_3a(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_hasIndex(
            self as *const crate::QAbstractItemModel,
            row,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if the model returns a valid <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> for <i>row</i> and <i>column</i> with <i>parent</i>, otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QAbstractItemModel::hasIndex(int row, int column) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#hasIndex">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the model returns a valid <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> for <i>row</i> and <i>column</i> with <i>parent</i>, otherwise returns <code>false</code>.</p></div>
    pub unsafe fn has_index_2a(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_hasIndex1(
            self as *const crate::QAbstractItemModel,
            row,
            column,
        )
    }

    /// <p>Returns the data for the given <i>role</i> and <i>section</i> in the header with the specified <i>orientation</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QVariant QAbstractItemModel::headerData(int section, Qt::Orientation orientation, int role = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the data for the given <i>role</i> and <i>section</i> in the header with the specified <i>orientation</i>.</p>
    /// <p>For horizontal headers, the section number corresponds to the column number. Similarly, for vertical headers, the section number corresponds to the row number.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::ItemDataRole</a>, <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setHeaderData">setHeaderData</a>(), and <a href="http://doc.qt.io/qt-5/qheaderview.html">QHeaderView</a>.</p></div>
    pub unsafe fn header_data_3a(
        &self,
        section: ::std::os::raw::c_int,
        orientation: crate::Orientation,
        role: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_headerData(
            self as *const crate::QAbstractItemModel,
            section,
            orientation,
            role,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the data for the given <i>role</i> and <i>section</i> in the header with the specified <i>orientation</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QVariant QAbstractItemModel::headerData(int section, Qt::Orientation orientation) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the data for the given <i>role</i> and <i>section</i> in the header with the specified <i>orientation</i>.</p>
    /// <p>For horizontal headers, the section number corresponds to the column number. Similarly, for vertical headers, the section number corresponds to the row number.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::ItemDataRole</a>, <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setHeaderData">setHeaderData</a>(), and <a href="http://doc.qt.io/qt-5/qheaderview.html">QHeaderView</a>.</p></div>
    pub unsafe fn header_data_2a(
        &self,
        section: ::std::os::raw::c_int,
        orientation: crate::Orientation,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_headerData1(
            self as *const crate::QAbstractItemModel,
            section,
            orientation,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the index of the item in the model specified by the given <i>row</i>, <i>column</i> and <i>parent</i> index.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual QModelIndex QAbstractItemModel::index(int row, int column, const QModelIndex& parent = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index of the item in the model specified by the given <i>row</i>, <i>column</i> and <i>parent</i> index.</p>
    /// <p>When reimplementing this function in a subclass, call <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#createIndex">createIndex</a>() to generate model indexes that other components can use to refer to items in your model.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#createIndex">createIndex</a>().</p></div>
    pub unsafe fn index_3a(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_index(
            self as *const crate::QAbstractItemModel,
            row,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the index of the item in the model specified by the given <i>row</i>, <i>column</i> and <i>parent</i> index.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual QModelIndex QAbstractItemModel::index(int row, int column) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index of the item in the model specified by the given <i>row</i>, <i>column</i> and <i>parent</i> index.</p>
    /// <p>When reimplementing this function in a subclass, call <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#createIndex">createIndex</a>() to generate model indexes that other components can use to refer to items in your model.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#createIndex">createIndex</a>().</p></div>
    pub unsafe fn index_2a(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_index1(
            self as *const crate::QAbstractItemModel,
            row,
            column,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Inserts a single column before the given <i>column</i> in the child items of the <i>parent</i> specified.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QAbstractItemModel::insertColumn(int column, const QModelIndex& parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumn">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts a single column before the given <i>column</i> in the child items of the <i>parent</i> specified.</p>
    /// <p>Returns <code>true</code> if the column is inserted; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumns">insertColumns</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRow">insertRow</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumn">removeColumn</a>().</p></div>
    pub unsafe fn insert_column_2a(
        &mut self,
        column: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_insertColumn(
            self as *mut crate::QAbstractItemModel,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Inserts a single column before the given <i>column</i> in the child items of the <i>parent</i> specified.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QAbstractItemModel::insertColumn(int column)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumn">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts a single column before the given <i>column</i> in the child items of the <i>parent</i> specified.</p>
    /// <p>Returns <code>true</code> if the column is inserted; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumns">insertColumns</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRow">insertRow</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumn">removeColumn</a>().</p></div>
    pub unsafe fn insert_column_1a(&mut self, column: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_insertColumn1(
            self as *mut crate::QAbstractItemModel,
            column,
        )
    }

    /// <p>On models that support this, inserts <i>count</i> new columns into the model before the given <i>column</i>. The items in each new column will be children of the item represented by the <i>parent</i> model index.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractItemModel::insertColumns(int column, int count, const QModelIndex& parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumns">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>On models that support this, inserts <i>count</i> new columns into the model before the given <i>column</i>. The items in each new column will be children of the item represented by the <i>parent</i> model index.</p>
    /// <p>If <i>column</i> is 0, the columns are prepended to any existing columns.</p>
    /// <p>If <i>column</i> is <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnCount">columnCount</a>(), the columns are appended to any existing columns.</p>
    /// <p>If <i>parent</i> has no children, a single row with <i>count</i> columns is inserted.</p>
    /// <p>Returns <code>true</code> if the columns were successfully inserted; otherwise returns <code>false</code>.</p>
    /// <p>The base class implementation does nothing and returns <code>false</code>.</p>
    /// <p>If you implement your own model, you can reimplement this function if you want to support insertions. Alternatively, you can provide your own API for altering the data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">insertRows</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumns">removeColumns</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginInsertColumns">beginInsertColumns</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#endInsertColumns">endInsertColumns</a>().</p></div>
    pub unsafe fn insert_columns_3a(
        &mut self,
        column: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_insertColumns(
            self as *mut crate::QAbstractItemModel,
            column,
            count,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>On models that support this, inserts <i>count</i> new columns into the model before the given <i>column</i>. The items in each new column will be children of the item represented by the <i>parent</i> model index.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractItemModel::insertColumns(int column, int count)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumns">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>On models that support this, inserts <i>count</i> new columns into the model before the given <i>column</i>. The items in each new column will be children of the item represented by the <i>parent</i> model index.</p>
    /// <p>If <i>column</i> is 0, the columns are prepended to any existing columns.</p>
    /// <p>If <i>column</i> is <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnCount">columnCount</a>(), the columns are appended to any existing columns.</p>
    /// <p>If <i>parent</i> has no children, a single row with <i>count</i> columns is inserted.</p>
    /// <p>Returns <code>true</code> if the columns were successfully inserted; otherwise returns <code>false</code>.</p>
    /// <p>The base class implementation does nothing and returns <code>false</code>.</p>
    /// <p>If you implement your own model, you can reimplement this function if you want to support insertions. Alternatively, you can provide your own API for altering the data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">insertRows</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumns">removeColumns</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginInsertColumns">beginInsertColumns</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#endInsertColumns">endInsertColumns</a>().</p></div>
    pub unsafe fn insert_columns_2a(
        &mut self,
        column: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_insertColumns1(
            self as *mut crate::QAbstractItemModel,
            column,
            count,
        )
    }

    /// <p>Inserts a single row before the given <i>row</i> in the child items of the <i>parent</i> specified.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QAbstractItemModel::insertRow(int row, const QModelIndex& parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRow">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts a single row before the given <i>row</i> in the child items of the <i>parent</i> specified.</p>
    /// <p><b>Note: </b>This function calls the virtual method <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">insertRows</a>.</p><p>Returns <code>true</code> if the row is inserted; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">insertRows</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumn">insertColumn</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRow">removeRow</a>().</p></div>
    pub unsafe fn insert_row_2a(
        &mut self,
        row: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_insertRow(
            self as *mut crate::QAbstractItemModel,
            row,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Inserts a single row before the given <i>row</i> in the child items of the <i>parent</i> specified.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QAbstractItemModel::insertRow(int row)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRow">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts a single row before the given <i>row</i> in the child items of the <i>parent</i> specified.</p>
    /// <p><b>Note: </b>This function calls the virtual method <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">insertRows</a>.</p><p>Returns <code>true</code> if the row is inserted; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">insertRows</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumn">insertColumn</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRow">removeRow</a>().</p></div>
    pub unsafe fn insert_row_1a(&mut self, row: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_insertRow1(
            self as *mut crate::QAbstractItemModel,
            row,
        )
    }

    /// <p><b>Note: </b>The base class implementation of this function does nothing and returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractItemModel::insertRows(int row, int count, const QModelIndex& parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p><b>Note: </b>The base class implementation of this function does nothing and returns <code>false</code>.</p><p>On models that support this, inserts <i>count</i> rows into the model before the given <i>row</i>. Items in the new row will be children of the item represented by the <i>parent</i> model index.</p>
    /// <p>If <i>row</i> is 0, the rows are prepended to any existing rows in the parent.</p>
    /// <p>If <i>row</i> is <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">rowCount</a>(), the rows are appended to any existing rows in the parent.</p>
    /// <p>If <i>parent</i> has no children, a single column with <i>count</i> rows is inserted.</p>
    /// <p>Returns <code>true</code> if the rows were successfully inserted; otherwise returns <code>false</code>.</p>
    /// <p>If you implement your own model, you can reimplement this function if you want to support insertions. Alternatively, you can provide your own API for altering the data. In either case, you will need to call <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginInsertRows">beginInsertRows</a>() and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#endInsertRows">endInsertRows</a>() to notify other components that the model has changed.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumns">insertColumns</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">removeRows</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginInsertRows">beginInsertRows</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#endInsertRows">endInsertRows</a>().</p></div>
    pub unsafe fn insert_rows_3a(
        &mut self,
        row: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_insertRows(
            self as *mut crate::QAbstractItemModel,
            row,
            count,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p><b>Note: </b>The base class implementation of this function does nothing and returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractItemModel::insertRows(int row, int count)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p><b>Note: </b>The base class implementation of this function does nothing and returns <code>false</code>.</p><p>On models that support this, inserts <i>count</i> rows into the model before the given <i>row</i>. Items in the new row will be children of the item represented by the <i>parent</i> model index.</p>
    /// <p>If <i>row</i> is 0, the rows are prepended to any existing rows in the parent.</p>
    /// <p>If <i>row</i> is <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">rowCount</a>(), the rows are appended to any existing rows in the parent.</p>
    /// <p>If <i>parent</i> has no children, a single column with <i>count</i> rows is inserted.</p>
    /// <p>Returns <code>true</code> if the rows were successfully inserted; otherwise returns <code>false</code>.</p>
    /// <p>If you implement your own model, you can reimplement this function if you want to support insertions. Alternatively, you can provide your own API for altering the data. In either case, you will need to call <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginInsertRows">beginInsertRows</a>() and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#endInsertRows">endInsertRows</a>() to notify other components that the model has changed.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumns">insertColumns</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">removeRows</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginInsertRows">beginInsertRows</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#endInsertRows">endInsertRows</a>().</p></div>
    pub unsafe fn insert_rows_2a(
        &mut self,
        row: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_insertRows1(
            self as *mut crate::QAbstractItemModel,
            row,
            count,
        )
    }

    /// <p>Returns a map with values for all predefined roles in the model for the item at the given <i>index</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QMap<int, QVariant> QAbstractItemModel::itemData(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#itemData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a map with values for all predefined roles in the model for the item at the given <i>index</i>.</p>
    /// <p>Reimplement this function if you want to extend the default behavior of this function to include custom roles in the map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setItemData">setItemData</a>(), <a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::ItemDataRole</a>, and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">data</a>().</p></div>
    pub unsafe fn item_data(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QMapOfIntQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_itemData(
            self as *const crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list of indexes for the items in the column of the <i>start</i> index where data stored under the given <i>role</i> matches the specified <i>value</i>. The way the search is performed is defined by the <i>flags</i> given. The list that is returned may be empty. Note also that the order of results in the list may not correspond to the order in the model, if for example a proxy model is used. The order of the results can not be relied upon.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QList<QModelIndex> QAbstractItemModel::match(const QModelIndex& start, int role, const QVariant& value, int hits = …, QFlags<Qt::MatchFlag> flags = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#match">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of indexes for the items in the column of the <i>start</i> index where data stored under the given <i>role</i> matches the specified <i>value</i>. The way the search is performed is defined by the <i>flags</i> given. The list that is returned may be empty. Note also that the order of results in the list may not correspond to the order in the model, if for example a proxy model is used. The order of the results can not be relied upon.</p>
    /// <p>The search begins from the <i>start</i> index, and continues until the number of matching data items equals <i>hits</i>, the search reaches the last row, or the search reaches <i>start</i> again - depending on whether <code>MatchWrap</code> is specified in <i>flags</i>. If you want to search for all matching items, use <i>hits</i> = -1.</p>
    /// <p>By default, this function will perform a wrapping, string-based comparison on all items, searching for items that begin with the search term specified by <i>value</i>.</p>
    /// <p><b>Note: </b>The default implementation of this function only searches columns. Reimplement this function to include a different search behavior.</p></div>
    pub unsafe fn match_5a(
        &self,
        start: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        role: ::std::os::raw::c_int,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
        hits: ::std::os::raw::c_int,
        flags: crate::QFlags<crate::MatchFlag>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_match(
            self as *const crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(start)
                .as_raw_ptr(),
            role,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
            hits,
            flags.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list of indexes for the items in the column of the <i>start</i> index where data stored under the given <i>role</i> matches the specified <i>value</i>. The way the search is performed is defined by the <i>flags</i> given. The list that is returned may be empty. Note also that the order of results in the list may not correspond to the order in the model, if for example a proxy model is used. The order of the results can not be relied upon.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QList<QModelIndex> QAbstractItemModel::match(const QModelIndex& start, int role, const QVariant& value, int hits = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#match">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of indexes for the items in the column of the <i>start</i> index where data stored under the given <i>role</i> matches the specified <i>value</i>. The way the search is performed is defined by the <i>flags</i> given. The list that is returned may be empty. Note also that the order of results in the list may not correspond to the order in the model, if for example a proxy model is used. The order of the results can not be relied upon.</p>
    /// <p>The search begins from the <i>start</i> index, and continues until the number of matching data items equals <i>hits</i>, the search reaches the last row, or the search reaches <i>start</i> again - depending on whether <code>MatchWrap</code> is specified in <i>flags</i>. If you want to search for all matching items, use <i>hits</i> = -1.</p>
    /// <p>By default, this function will perform a wrapping, string-based comparison on all items, searching for items that begin with the search term specified by <i>value</i>.</p>
    /// <p><b>Note: </b>The default implementation of this function only searches columns. Reimplement this function to include a different search behavior.</p></div>
    pub unsafe fn match_4a(
        &self,
        start: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        role: ::std::os::raw::c_int,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
        hits: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_match1(
            self as *const crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(start)
                .as_raw_ptr(),
            role,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
            hits,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list of indexes for the items in the column of the <i>start</i> index where data stored under the given <i>role</i> matches the specified <i>value</i>. The way the search is performed is defined by the <i>flags</i> given. The list that is returned may be empty. Note also that the order of results in the list may not correspond to the order in the model, if for example a proxy model is used. The order of the results can not be relied upon.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QList<QModelIndex> QAbstractItemModel::match(const QModelIndex& start, int role, const QVariant& value) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#match">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of indexes for the items in the column of the <i>start</i> index where data stored under the given <i>role</i> matches the specified <i>value</i>. The way the search is performed is defined by the <i>flags</i> given. The list that is returned may be empty. Note also that the order of results in the list may not correspond to the order in the model, if for example a proxy model is used. The order of the results can not be relied upon.</p>
    /// <p>The search begins from the <i>start</i> index, and continues until the number of matching data items equals <i>hits</i>, the search reaches the last row, or the search reaches <i>start</i> again - depending on whether <code>MatchWrap</code> is specified in <i>flags</i>. If you want to search for all matching items, use <i>hits</i> = -1.</p>
    /// <p>By default, this function will perform a wrapping, string-based comparison on all items, searching for items that begin with the search term specified by <i>value</i>.</p>
    /// <p><b>Note: </b>The default implementation of this function only searches columns. Reimplement this function to include a different search behavior.</p></div>
    pub unsafe fn match_3a(
        &self,
        start: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        role: ::std::os::raw::c_int,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_match2(
            self as *const crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(start)
                .as_raw_ptr(),
            role,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QAbstractItemModel::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_metaObject(
            self as *const crate::QAbstractItemModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns an object that contains serialized items of data corresponding to the list of <i>indexes</i> specified. The format used to describe the encoded data is obtained from the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeTypes">mimeTypes</a>() function. This default implementation uses the default MIME type returned by the default implementation of <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeTypes">mimeTypes</a>(). If you reimplement <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeTypes">mimeTypes</a>() in your custom model to return more MIME types, reimplement this function to make use of them.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QMimeData* QAbstractItemModel::mimeData(const QList<QModelIndex>& indexes) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an object that contains serialized items of data corresponding to the list of <i>indexes</i> specified. The format used to describe the encoded data is obtained from the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeTypes">mimeTypes</a>() function. This default implementation uses the default MIME type returned by the default implementation of <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeTypes">mimeTypes</a>(). If you reimplement <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeTypes">mimeTypes</a>() in your custom model to return more MIME types, reimplement this function to make use of them.</p>
    /// <p>If the list of <i>indexes</i> is empty, or there are no supported MIME types, 0 is returned rather than a serialized empty list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeTypes">mimeTypes</a>() and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dropMimeData">dropMimeData</a>().</p></div>
    pub unsafe fn mime_data(
        &self,
        indexes: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQModelIndex>>,
    ) -> ::cpp_utils::MutPtr<crate::QMimeData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_mimeData(
            self as *const crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQModelIndex>>::cast_into(
                indexes,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns the list of allowed MIME types. By default, the built-in models and views use an internal MIME type: <code>application/x-qabstractitemmodeldatalist</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QStringList QAbstractItemModel::mimeTypes() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeTypes">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the list of allowed MIME types. By default, the built-in models and views use an internal MIME type: <code>application/x-qabstractitemmodeldatalist</code>.</p>
    /// <p>When implementing drag and drop support in a custom model, if you will return data in formats other than the default internal MIME type, reimplement this function to return your list of MIME types.</p>
    /// <p>If you reimplement this function in your custom model, you must also reimplement the member functions that call it: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeData">mimeData</a>() and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dropMimeData">dropMimeData</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeData">mimeData</a>() and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dropMimeData">dropMimeData</a>().</p></div>
    pub unsafe fn mime_types(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_mimeTypes(
            self as *const crate::QAbstractItemModel,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>On models that support this, moves <i>sourceColumn</i> from <i>sourceParent</i> to <i>destinationChild</i> under <i>destinationParent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QAbstractItemModel::moveColumn(const QModelIndex& sourceParent, int sourceColumn, const QModelIndex& destinationParent, int destinationChild)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#moveColumn">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>On models that support this, moves <i>sourceColumn</i> from <i>sourceParent</i> to <i>destinationChild</i> under <i>destinationParent</i>.</p>
    /// <p>Returns <code>true</code> if the columns were successfully moved; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#moveColumns">moveColumns</a>() and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#moveRow">moveRow</a>().</p></div>
    pub unsafe fn move_column(
        &mut self,
        source_parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        source_column: ::std::os::raw::c_int,
        destination_parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        destination_child: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_moveColumn(
            self as *mut crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(source_parent)
                .as_raw_ptr(),
            source_column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(
                destination_parent,
            )
            .as_raw_ptr(),
            destination_child,
        )
    }

    /// <p>On models that support this, moves <i>count</i> columns starting with the given <i>sourceColumn</i> under parent <i>sourceParent</i> to column <i>destinationChild</i> under parent <i>destinationParent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractItemModel::moveColumns(const QModelIndex& sourceParent, int sourceColumn, int count, const QModelIndex& destinationParent, int destinationChild)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#moveColumns">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>On models that support this, moves <i>count</i> columns starting with the given <i>sourceColumn</i> under parent <i>sourceParent</i> to column <i>destinationChild</i> under parent <i>destinationParent</i>.</p>
    /// <p>Returns <code>true</code> if the columns were successfully moved; otherwise returns <code>false</code>.</p>
    /// <p>The base class implementation does nothing and returns <code>false</code>.</p>
    /// <p>If you implement your own model, you can reimplement this function if you want to support moving. Alternatively, you can provide your own API for altering the data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginMoveColumns">beginMoveColumns</a>() and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#endMoveColumns">endMoveColumns</a>().</p></div>
    pub unsafe fn move_columns(
        &mut self,
        source_parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        source_column: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
        destination_parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        destination_child: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_moveColumns(
            self as *mut crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(source_parent)
                .as_raw_ptr(),
            source_column,
            count,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(
                destination_parent,
            )
            .as_raw_ptr(),
            destination_child,
        )
    }

    /// <p>On models that support this, moves <i>sourceRow</i> from <i>sourceParent</i> to <i>destinationChild</i> under <i>destinationParent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QAbstractItemModel::moveRow(const QModelIndex& sourceParent, int sourceRow, const QModelIndex& destinationParent, int destinationChild)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#moveRow">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>On models that support this, moves <i>sourceRow</i> from <i>sourceParent</i> to <i>destinationChild</i> under <i>destinationParent</i>.</p>
    /// <p>Returns <code>true</code> if the rows were successfully moved; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#moveRows">moveRows</a>() and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#moveColumn">moveColumn</a>().</p></div>
    pub unsafe fn move_row(
        &mut self,
        source_parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        source_row: ::std::os::raw::c_int,
        destination_parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        destination_child: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_moveRow(
            self as *mut crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(source_parent)
                .as_raw_ptr(),
            source_row,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(
                destination_parent,
            )
            .as_raw_ptr(),
            destination_child,
        )
    }

    /// <p>On models that support this, moves <i>count</i> rows starting with the given <i>sourceRow</i> under parent <i>sourceParent</i> to row <i>destinationChild</i> under parent <i>destinationParent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractItemModel::moveRows(const QModelIndex& sourceParent, int sourceRow, int count, const QModelIndex& destinationParent, int destinationChild)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#moveRows">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>On models that support this, moves <i>count</i> rows starting with the given <i>sourceRow</i> under parent <i>sourceParent</i> to row <i>destinationChild</i> under parent <i>destinationParent</i>.</p>
    /// <p>Returns <code>true</code> if the rows were successfully moved; otherwise returns <code>false</code>.</p>
    /// <p>The base class implementation does nothing and returns <code>false</code>.</p>
    /// <p>If you implement your own model, you can reimplement this function if you want to support moving. Alternatively, you can provide your own API for altering the data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginMoveRows">beginMoveRows</a>() and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#endMoveRows">endMoveRows</a>().</p></div>
    pub unsafe fn move_rows(
        &mut self,
        source_parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        source_row: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
        destination_parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        destination_child: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_moveRows(
            self as *mut crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(source_parent)
                .as_raw_ptr(),
            source_row,
            count,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(
                destination_parent,
            )
            .as_raw_ptr(),
            destination_child,
        )
    }

    /// <p>Returns the parent of the model item with the given <i>index</i>. If the item has no parent, an invalid <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual QModelIndex QAbstractItemModel::parent(const QModelIndex& child) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#parent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the parent of the model item with the given <i>index</i>. If the item has no parent, an invalid <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> is returned.</p>
    /// <p>A common convention used in models that expose tree data structures is that only items in the first column have children. For that case, when reimplementing this function in a subclass the column of the returned <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> would be 0.</p>
    /// <p>When reimplementing this function in a subclass, be careful to avoid calling <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> member functions, such as <a href="http://doc.qt.io/qt-5/qmodelindex.html#parent">QModelIndex::parent</a>(), since indexes belonging to your model will simply call your implementation, leading to infinite recursion.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#createIndex">createIndex</a>().</p></div>
    pub unsafe fn parent(
        &self,
        child: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_parent(
            self as *const crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(child)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QAbstractItemModel::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_qt_metacall(
            self as *mut crate::QAbstractItemModel,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QAbstractItemModel::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_qt_metacast(
            self as *mut crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Removes the given <i>column</i> from the child items of the <i>parent</i> specified.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QAbstractItemModel::removeColumn(int column, const QModelIndex& parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumn">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the given <i>column</i> from the child items of the <i>parent</i> specified.</p>
    /// <p>Returns <code>true</code> if the column is removed; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumns">removeColumns</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRow">removeRow</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumn">insertColumn</a>().</p></div>
    pub unsafe fn remove_column_2a(
        &mut self,
        column: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_removeColumn(
            self as *mut crate::QAbstractItemModel,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes the given <i>column</i> from the child items of the <i>parent</i> specified.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QAbstractItemModel::removeColumn(int column)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumn">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the given <i>column</i> from the child items of the <i>parent</i> specified.</p>
    /// <p>Returns <code>true</code> if the column is removed; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumns">removeColumns</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRow">removeRow</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumn">insertColumn</a>().</p></div>
    pub unsafe fn remove_column_1a(&mut self, column: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_removeColumn1(
            self as *mut crate::QAbstractItemModel,
            column,
        )
    }

    /// <p>On models that support this, removes <i>count</i> columns starting with the given <i>column</i> under parent <i>parent</i> from the model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractItemModel::removeColumns(int column, int count, const QModelIndex& parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumns">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>On models that support this, removes <i>count</i> columns starting with the given <i>column</i> under parent <i>parent</i> from the model.</p>
    /// <p>Returns <code>true</code> if the columns were successfully removed; otherwise returns <code>false</code>.</p>
    /// <p>The base class implementation does nothing and returns <code>false</code>.</p>
    /// <p>If you implement your own model, you can reimplement this function if you want to support removing. Alternatively, you can provide your own API for altering the data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumn">removeColumn</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">removeRows</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumns">insertColumns</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginRemoveColumns">beginRemoveColumns</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#endRemoveColumns">endRemoveColumns</a>().</p></div>
    pub unsafe fn remove_columns_3a(
        &mut self,
        column: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_removeColumns(
            self as *mut crate::QAbstractItemModel,
            column,
            count,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>On models that support this, removes <i>count</i> columns starting with the given <i>column</i> under parent <i>parent</i> from the model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractItemModel::removeColumns(int column, int count)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumns">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>On models that support this, removes <i>count</i> columns starting with the given <i>column</i> under parent <i>parent</i> from the model.</p>
    /// <p>Returns <code>true</code> if the columns were successfully removed; otherwise returns <code>false</code>.</p>
    /// <p>The base class implementation does nothing and returns <code>false</code>.</p>
    /// <p>If you implement your own model, you can reimplement this function if you want to support removing. Alternatively, you can provide your own API for altering the data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumn">removeColumn</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">removeRows</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumns">insertColumns</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginRemoveColumns">beginRemoveColumns</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#endRemoveColumns">endRemoveColumns</a>().</p></div>
    pub unsafe fn remove_columns_2a(
        &mut self,
        column: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_removeColumns1(
            self as *mut crate::QAbstractItemModel,
            column,
            count,
        )
    }

    /// <p>Removes the given <i>row</i> from the child items of the <i>parent</i> specified.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QAbstractItemModel::removeRow(int row, const QModelIndex& parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRow">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the given <i>row</i> from the child items of the <i>parent</i> specified.</p>
    /// <p>Returns <code>true</code> if the row is removed; otherwise returns <code>false</code>.</p>
    /// <p>This is a convenience function that calls <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">removeRows</a>(). The <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a> implementation of <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">removeRows</a>() does nothing.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">removeRows</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumn">removeColumn</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRow">insertRow</a>().</p></div>
    pub unsafe fn remove_row_2a(
        &mut self,
        row: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_removeRow(
            self as *mut crate::QAbstractItemModel,
            row,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes the given <i>row</i> from the child items of the <i>parent</i> specified.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QAbstractItemModel::removeRow(int row)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRow">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the given <i>row</i> from the child items of the <i>parent</i> specified.</p>
    /// <p>Returns <code>true</code> if the row is removed; otherwise returns <code>false</code>.</p>
    /// <p>This is a convenience function that calls <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">removeRows</a>(). The <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a> implementation of <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">removeRows</a>() does nothing.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">removeRows</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumn">removeColumn</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRow">insertRow</a>().</p></div>
    pub unsafe fn remove_row_1a(&mut self, row: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_removeRow1(
            self as *mut crate::QAbstractItemModel,
            row,
        )
    }

    /// <p>On models that support this, removes <i>count</i> rows starting with the given <i>row</i> under parent <i>parent</i> from the model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractItemModel::removeRows(int row, int count, const QModelIndex& parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>On models that support this, removes <i>count</i> rows starting with the given <i>row</i> under parent <i>parent</i> from the model.</p>
    /// <p>Returns <code>true</code> if the rows were successfully removed; otherwise returns <code>false</code>.</p>
    /// <p>The base class implementation does nothing and returns <code>false</code>.</p>
    /// <p>If you implement your own model, you can reimplement this function if you want to support removing. Alternatively, you can provide your own API for altering the data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRow">removeRow</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumns">removeColumns</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumns">insertColumns</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginRemoveRows">beginRemoveRows</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#endRemoveRows">endRemoveRows</a>().</p></div>
    pub unsafe fn remove_rows_3a(
        &mut self,
        row: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_removeRows(
            self as *mut crate::QAbstractItemModel,
            row,
            count,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>On models that support this, removes <i>count</i> rows starting with the given <i>row</i> under parent <i>parent</i> from the model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractItemModel::removeRows(int row, int count)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>On models that support this, removes <i>count</i> rows starting with the given <i>row</i> under parent <i>parent</i> from the model.</p>
    /// <p>Returns <code>true</code> if the rows were successfully removed; otherwise returns <code>false</code>.</p>
    /// <p>The base class implementation does nothing and returns <code>false</code>.</p>
    /// <p>If you implement your own model, you can reimplement this function if you want to support removing. Alternatively, you can provide your own API for altering the data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRow">removeRow</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumns">removeColumns</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumns">insertColumns</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginRemoveRows">beginRemoveRows</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#endRemoveRows">endRemoveRows</a>().</p></div>
    pub unsafe fn remove_rows_2a(
        &mut self,
        row: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_removeRows1(
            self as *mut crate::QAbstractItemModel,
            row,
            count,
        )
    }

    /// <p>Lets the model know that it should discard cached information. This function is typically used for row editing.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [slot] void QAbstractItemModel::revert()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#revert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Lets the model know that it should discard cached information. This function is typically used for row editing.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#submit">submit</a>().</p></div>
    pub unsafe fn revert(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_revert(
            self as *mut crate::QAbstractItemModel,
        )
    }

    /// <p>Returns the model's role names.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QHash<int, QByteArray> QAbstractItemModel::roleNames() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#roleNames">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the model's role names.</p>
    /// <p>The default role names set by Qt are:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Qt Role</th><th>QML Role Name</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::DisplayRole</a></td><td>display</td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::DecorationRole</a></td><td>decoration</td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::EditRole</a></td><td>edit</td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::ToolTipRole</a></td><td>toolTip</td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::StatusTipRole</a></td><td>statusTip</td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::WhatsThisRole</a></td><td>whatsThis</td></tr>
    /// </tbody></table></div>
    /// <p>This function was introduced in  Qt 4.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel-obsolete.html#setRoleNames">setRoleNames</a>().</p></div>
    pub unsafe fn role_names(&self) -> ::cpp_utils::CppBox<crate::QHashOfIntQByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_roleNames(
            self as *const crate::QAbstractItemModel,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the number of rows under the given <i>parent</i>. When the parent is valid it means that rowCount is returning the number of children of parent.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual int QAbstractItemModel::rowCount(const QModelIndex& parent = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of rows under the given <i>parent</i>. When the parent is valid it means that rowCount is returning the number of children of parent.</p>
    /// <p><b>Note: </b>When implementing a table based model, rowCount() should return 0 when the parent is valid.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnCount">columnCount</a>().</p></div>
    pub unsafe fn row_count_1a(
        &self,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_rowCount(
            self as *const crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the number of rows under the given <i>parent</i>. When the parent is valid it means that rowCount is returning the number of children of parent.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual int QAbstractItemModel::rowCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of rows under the given <i>parent</i>. When the parent is valid it means that rowCount is returning the number of children of parent.</p>
    /// <p><b>Note: </b>When implementing a table based model, rowCount() should return 0 when the parent is valid.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnCount">columnCount</a>().</p></div>
    pub unsafe fn row_count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_rowCount1(
            self as *const crate::QAbstractItemModel,
        )
    }

    /// <p>Sets the <i>role</i> data for the item at <i>index</i> to <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractItemModel::setData(const QModelIndex& index, const QVariant& value, int role = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the <i>role</i> data for the item at <i>index</i> to <i>value</i>.</p>
    /// <p>Returns <code>true</code> if successful; otherwise returns <code>false</code>.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dataChanged">dataChanged</a>() signal should be emitted if the data was successfully set.</p>
    /// <p>The base class implementation returns <code>false</code>. This function and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">data</a>() must be reimplemented for editable models.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::ItemDataRole</a>, <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">data</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#itemData">itemData</a>().</p></div>
    pub unsafe fn set_data_3a(
        &mut self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
        role: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_setData(
            self as *mut crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
            role,
        )
    }

    /// <p>Sets the <i>role</i> data for the item at <i>index</i> to <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractItemModel::setData(const QModelIndex& index, const QVariant& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the <i>role</i> data for the item at <i>index</i> to <i>value</i>.</p>
    /// <p>Returns <code>true</code> if successful; otherwise returns <code>false</code>.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dataChanged">dataChanged</a>() signal should be emitted if the data was successfully set.</p>
    /// <p>The base class implementation returns <code>false</code>. This function and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">data</a>() must be reimplemented for editable models.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::ItemDataRole</a>, <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">data</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#itemData">itemData</a>().</p></div>
    pub unsafe fn set_data_2a(
        &mut self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_setData1(
            self as *mut crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the data for the given <i>role</i> and <i>section</i> in the header with the specified <i>orientation</i> to the <i>value</i> supplied.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractItemModel::setHeaderData(int section, Qt::Orientation orientation, const QVariant& value, int role = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setHeaderData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the data for the given <i>role</i> and <i>section</i> in the header with the specified <i>orientation</i> to the <i>value</i> supplied.</p>
    /// <p>Returns <code>true</code> if the header's data was updated; otherwise returns <code>false</code>.</p>
    /// <p>When reimplementing this function, the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerDataChanged">headerDataChanged</a>() signal must be emitted explicitly.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::ItemDataRole</a> and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerData">headerData</a>().</p></div>
    pub unsafe fn set_header_data_4a(
        &mut self,
        section: ::std::os::raw::c_int,
        orientation: crate::Orientation,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
        role: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_setHeaderData(
            self as *mut crate::QAbstractItemModel,
            section,
            orientation,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
            role,
        )
    }

    /// <p>Sets the data for the given <i>role</i> and <i>section</i> in the header with the specified <i>orientation</i> to the <i>value</i> supplied.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractItemModel::setHeaderData(int section, Qt::Orientation orientation, const QVariant& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setHeaderData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the data for the given <i>role</i> and <i>section</i> in the header with the specified <i>orientation</i> to the <i>value</i> supplied.</p>
    /// <p>Returns <code>true</code> if the header's data was updated; otherwise returns <code>false</code>.</p>
    /// <p>When reimplementing this function, the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerDataChanged">headerDataChanged</a>() signal must be emitted explicitly.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::ItemDataRole</a> and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerData">headerData</a>().</p></div>
    pub unsafe fn set_header_data_3a(
        &mut self,
        section: ::std::os::raw::c_int,
        orientation: crate::Orientation,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_setHeaderData1(
            self as *mut crate::QAbstractItemModel,
            section,
            orientation,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the role data for the item at <i>index</i> to the associated value in <i>roles</i>, for every <a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::ItemDataRole</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractItemModel::setItemData(const QModelIndex& index, const QMap<int, QVariant>& roles)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setItemData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the role data for the item at <i>index</i> to the associated value in <i>roles</i>, for every <a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::ItemDataRole</a>.</p>
    /// <p>Returns <code>true</code> if successful; otherwise returns <code>false</code>.</p>
    /// <p>Roles that are not in <i>roles</i> will not be modified.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">setData</a>(), <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">data</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#itemData">itemData</a>().</p></div>
    pub unsafe fn set_item_data(
        &mut self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        roles: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMapOfIntQVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_setItemData(
            self as *mut crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMapOfIntQVariant>>::cast_into(roles)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the sibling at <i>row</i> and <i>column</i> for the item at <i>index</i>, or an invalid <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> if there is no sibling at that location.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QAbstractItemModel::sibling(int row, int column, const QModelIndex& idx) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sibling">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the sibling at <i>row</i> and <i>column</i> for the item at <i>index</i>, or an invalid <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> if there is no sibling at that location.</p>
    /// <p>sibling() is just a convenience function that finds the item's parent, and uses it to retrieve the index of the child item in the specified <i>row</i> and <i>column</i>.</p>
    /// <p>This method can optionally be overridden for implementation-specific optimization.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">index</a>(), <a href="http://doc.qt.io/qt-5/qmodelindex.html#row">QModelIndex::row</a>(), and <a href="http://doc.qt.io/qt-5/qmodelindex.html#column">QModelIndex::column</a>().</p></div>
    pub unsafe fn sibling(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
        idx: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_sibling(
            self as *const crate::QAbstractItemModel,
            row,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(idx)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Sorts the model by <i>column</i> in the given <i>order</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual void QAbstractItemModel::sort(int column, Qt::SortOrder order = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sorts the model by <i>column</i> in the given <i>order</i>.</p>
    /// <p>The base class implementation does nothing.</p></div>
    pub unsafe fn sort_2a(&mut self, column: ::std::os::raw::c_int, order: crate::SortOrder) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_sort(
            self as *mut crate::QAbstractItemModel,
            column,
            order,
        )
    }

    /// <p>Sorts the model by <i>column</i> in the given <i>order</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual void QAbstractItemModel::sort(int column)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sorts the model by <i>column</i> in the given <i>order</i>.</p>
    /// <p>The base class implementation does nothing.</p></div>
    pub unsafe fn sort_1a(&mut self, column: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_sort1(
            self as *mut crate::QAbstractItemModel,
            column,
        )
    }

    /// <p>Returns the row and column span of the item represented by <i>index</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QSize QAbstractItemModel::span(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#span">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the row and column span of the item represented by <i>index</i>.</p>
    /// <p><b>Note: </b>Currently, span is not used.</p></div>
    pub unsafe fn span(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QSize> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_span(
            self as *const crate::QAbstractItemModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Lets the model know that it should submit cached information to permanent storage. This function is typically used for row editing.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [slot] bool QAbstractItemModel::submit()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#submit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Lets the model know that it should submit cached information to permanent storage. This function is typically used for row editing.</p>
    /// <p>Returns <code>true</code> if there is no error; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#revert">revert</a>().</p></div>
    pub unsafe fn submit(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_submit(
            self as *mut crate::QAbstractItemModel,
        )
    }

    /// <p>Returns the actions supported by the data in this model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QFlags<Qt::DropAction> QAbstractItemModel::supportedDragActions() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#supportedDragActions">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the actions supported by the data in this model.</p>
    /// <p>The default implementation returns <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#supportedDropActions">supportedDropActions</a>(). Reimplement this function if you wish to support additional actions.</p>
    /// <p>supportedDragActions() is used by <a href="http://doc.qt.io/qt-5/qabstractitemview.html#startDrag">QAbstractItemView::startDrag</a>() as the default values when a drag occurs.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel-obsolete.html#setSupportedDragActions">setSupportedDragActions</a>(), <a href="http://doc.qt.io/qt-5/qt.html#DropAction-enum">Qt::DropActions</a>, and <a href="http://doc.qt.io/qt-5/model-view-programming.html#using-drag-and-drop-with-item-views">Using drag and drop with item views</a>.</p></div>
    pub unsafe fn supported_drag_actions(&self) -> crate::QFlags<crate::DropAction> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_supportedDragActions(
            self as *const crate::QAbstractItemModel,
        );
        crate::QFlags::from(ffi_result)
    }

    /// <p>Returns the drop actions supported by this model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QFlags<Qt::DropAction> QAbstractItemModel::supportedDropActions() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#supportedDropActions">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the drop actions supported by this model.</p>
    /// <p>The default implementation returns <a href="http://doc.qt.io/qt-5/qt.html#DropAction-enum">Qt::CopyAction</a>. Reimplement this function if you wish to support additional actions. You must also reimplement the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dropMimeData">dropMimeData</a>() function to handle the additional operations.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dropMimeData">dropMimeData</a>(), <a href="http://doc.qt.io/qt-5/qt.html#DropAction-enum">Qt::DropActions</a>, and <a href="http://doc.qt.io/qt-5/model-view-programming.html#using-drag-and-drop-with-item-views">Using drag and drop with item views</a>.</p></div>
    pub unsafe fn supported_drop_actions(&self) -> crate::QFlags<crate::DropAction> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_supportedDropActions(
            self as *const crate::QAbstractItemModel,
        );
        crate::QFlags::from(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QAbstractItemModel::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QAbstractItemModel::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qabstracttablemodel.html">QAbstractTableModel</a> class provides an abstract model that can be subclassed to create table models.</p>
///
/// C++ class: <span style='color: green;'>```QAbstractTableModel```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qabstracttablemodel.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qabstracttablemodel.html">QAbstractTableModel</a> class provides an abstract model that can be subclassed to create table models.</p>
/// <p><a href="http://doc.qt.io/qt-5/qabstracttablemodel.html">QAbstractTableModel</a> provides a standard interface for models that represent their data as a two-dimensional array of items. It is not used directly, but must be subclassed.</p>
/// <p>Since the model provides a more specialized interface than <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a>, it is not suitable for use with tree views, although it can be used to provide data to a <a href="http://doc.qt.io/qt-5/qlistview.html">QListView</a>. If you need to represent a simple list of items, and only need a model to contain a single column of data, subclassing the <a href="http://doc.qt.io/qt-5/qabstractlistmodel.html">QAbstractListModel</a> may be more appropriate.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">rowCount</a>() and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnCount">columnCount</a>() functions return the dimensions of the table. To retrieve a model index corresponding to an item in the model, use <a href="http://doc.qt.io/qt-5/qabstracttablemodel.html#index">index</a>() and provide only the row and column numbers.</p>
/// <a name="subclassing"></a></div>
#[repr(C)]
pub struct QAbstractTableModel {
    _unused: u8,
}
impl QAbstractTableModel {
    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dropMimeData">QAbstractItemModel::dropMimeData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractTableModel::dropMimeData(const QMimeData* data, Qt::DropAction action, int row, int column, const QModelIndex& parent)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracttablemodel.html#dropMimeData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dropMimeData">QAbstractItemModel::dropMimeData</a>().</p></div>
    pub unsafe fn drop_mime_data(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QMimeData>>,
        action: crate::DropAction,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractTableModel_dropMimeData(
            self as *mut crate::QAbstractTableModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QMimeData>>::cast_into(data)
                .as_raw_ptr(),
            action,
            row,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#flags">QAbstractItemModel::flags</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QFlags<Qt::ItemFlag> QAbstractTableModel::flags(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracttablemodel.html#flags">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#flags">QAbstractItemModel::flags</a>().</p></div>
    pub unsafe fn flags(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> crate::QFlags<crate::ItemFlag> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractTableModel_flags(
            self as *const crate::QAbstractTableModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        );
        crate::QFlags::from(ffi_result)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QAbstractTableModel::index(int row, int column, const QModelIndex& parent = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracttablemodel.html#index">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>().</p>
    /// <p>Returns the index of the data in <i>row</i> and <i>column</i> with <i>parent</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#parent">parent</a>().</p></div>
    pub unsafe fn index_3a(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractTableModel_index(
            self as *const crate::QAbstractTableModel,
            row,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QAbstractTableModel::index(int row, int column) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracttablemodel.html#index">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>().</p>
    /// <p>Returns the index of the data in <i>row</i> and <i>column</i> with <i>parent</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#parent">parent</a>().</p></div>
    pub unsafe fn index_2a(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractTableModel_index1(
            self as *const crate::QAbstractTableModel,
            row,
            column,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QAbstractTableModel::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractTableModel_metaObject(
            self as *const crate::QAbstractTableModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QAbstractTableModel::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QAbstractTableModel_qt_metacall(
            self as *mut crate::QAbstractTableModel,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QAbstractTableModel::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractTableModel_qt_metacast(
            self as *mut crate::QAbstractTableModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sibling">QAbstractItemModel::sibling</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QAbstractTableModel::sibling(int row, int column, const QModelIndex& idx) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracttablemodel.html#sibling">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sibling">QAbstractItemModel::sibling</a>().</p></div>
    pub unsafe fn sibling(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
        idx: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractTableModel_sibling(
            self as *const crate::QAbstractTableModel,
            row,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(idx)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractTableModel_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QAbstractTableModel::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractTableModel_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QAbstractTableModel::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractTableModel_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qabstractlistmodel.html">QAbstractListModel</a> class provides an abstract model that can be subclassed to create one-dimensional list models.</p>
///
/// C++ class: <span style='color: green;'>```QAbstractListModel```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qabstractlistmodel.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qabstractlistmodel.html">QAbstractListModel</a> class provides an abstract model that can be subclassed to create one-dimensional list models.</p>
/// <p><a href="http://doc.qt.io/qt-5/qabstractlistmodel.html">QAbstractListModel</a> provides a standard interface for models that represent their data as a simple non-hierarchical sequence of items. It is not used directly, but must be subclassed.</p>
/// <p>Since the model provides a more specialized interface than <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a>, it is not suitable for use with tree views; you will need to subclass <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a> if you want to provide a model for that purpose. If you need to use a number of list models to manage data, it may be more appropriate to subclass <a href="http://doc.qt.io/qt-5/qabstracttablemodel.html">QAbstractTableModel</a> instead.</p>
/// <p>Simple models can be created by subclassing this class and implementing the minimum number of required functions. For example, we could implement a simple read-only <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a>-based model that provides a list of strings to a <a href="http://doc.qt.io/qt-5/qlistview.html">QListView</a> widget. In such a case, we only need to implement the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">rowCount</a>() function to return the number of items in the list, and the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">data</a>() function to retrieve items from the list.</p>
/// <p>Since the model represents a one-dimensional structure, the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">rowCount</a>() function returns the total number of items in the model. The <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnCount">columnCount</a>() function is implemented for interoperability with all kinds of views, but by default informs views that the model contains only one column.</p>
/// <a name="subclassing"></a></div>
#[repr(C)]
pub struct QAbstractListModel {
    _unused: u8,
}
impl QAbstractListModel {
    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dropMimeData">QAbstractItemModel::dropMimeData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractListModel::dropMimeData(const QMimeData* data, Qt::DropAction action, int row, int column, const QModelIndex& parent)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractlistmodel.html#dropMimeData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dropMimeData">QAbstractItemModel::dropMimeData</a>().</p></div>
    pub unsafe fn drop_mime_data(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QMimeData>>,
        action: crate::DropAction,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractListModel_dropMimeData(
            self as *mut crate::QAbstractListModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QMimeData>>::cast_into(data)
                .as_raw_ptr(),
            action,
            row,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#flags">QAbstractItemModel::flags</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QFlags<Qt::ItemFlag> QAbstractListModel::flags(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractlistmodel.html#flags">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#flags">QAbstractItemModel::flags</a>().</p></div>
    pub unsafe fn flags(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> crate::QFlags<crate::ItemFlag> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractListModel_flags(
            self as *const crate::QAbstractListModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        );
        crate::QFlags::from(ffi_result)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QAbstractListModel::index(int row, int column = …, const QModelIndex& parent = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractlistmodel.html#index">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>().</p>
    /// <p>Returns the index of the data in <i>row</i> and <i>column</i> with <i>parent</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#parent">parent</a>().</p></div>
    pub unsafe fn index_3a(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractListModel_index(
            self as *const crate::QAbstractListModel,
            row,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QAbstractListModel::index(int row, int column = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractlistmodel.html#index">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>().</p>
    /// <p>Returns the index of the data in <i>row</i> and <i>column</i> with <i>parent</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#parent">parent</a>().</p></div>
    pub unsafe fn index_2a(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractListModel_index1(
            self as *const crate::QAbstractListModel,
            row,
            column,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QAbstractListModel::index(int row) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractlistmodel.html#index">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>().</p>
    /// <p>Returns the index of the data in <i>row</i> and <i>column</i> with <i>parent</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#parent">parent</a>().</p></div>
    pub unsafe fn index_1a(
        &self,
        row: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractListModel_index2(
            self as *const crate::QAbstractListModel,
            row,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QAbstractListModel::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractListModel_metaObject(
            self as *const crate::QAbstractListModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QAbstractListModel::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QAbstractListModel_qt_metacall(
            self as *mut crate::QAbstractListModel,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QAbstractListModel::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractListModel_qt_metacast(
            self as *mut crate::QAbstractListModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sibling">QAbstractItemModel::sibling</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QAbstractListModel::sibling(int row, int column, const QModelIndex& idx) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractlistmodel.html#sibling">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sibling">QAbstractItemModel::sibling</a>().</p></div>
    pub unsafe fn sibling(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
        idx: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractListModel_sibling(
            self as *const crate::QAbstractListModel,
            row,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(idx)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractListModel_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QAbstractListModel::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractListModel_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QAbstractListModel::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractListModel_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qabstractnativeeventfilter.html">QAbstractNativeEventFilter</a> class provides an interface for receiving native events, such as MSG or XCB event structs.</p>
///
/// C++ class: <span style='color: green;'>```QAbstractNativeEventFilter```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qabstractnativeeventfilter.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qabstractnativeeventfilter.html">QAbstractNativeEventFilter</a> class provides an interface for receiving native events, such as MSG or XCB event structs.</p></div>
#[repr(C)]
pub struct QAbstractNativeEventFilter {
    _unused: u8,
}
impl QAbstractNativeEventFilter {
    /// <p>This method is called for every native event.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual bool QAbstractNativeEventFilter::nativeEventFilter(const QByteArray& eventType, void* message, long* result)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractnativeeventfilter.html#nativeEventFilter">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This method is called for every native event.</p>
    /// <p><b>Note: </b>The filter function here receives native messages, for example, MSG or XCB event structs.</p><p>It is called by the QPA platform plugin. On Windows, it is called by the event dispatcher.</p>
    /// <p>The type of event <i>eventType</i> is specific to the platform plugin chosen at run-time, and can be used to cast <i>message</i> to the right type.</p>
    /// <p>On X11, <i>eventType</i> is set to "xcb_generic_event_t", and the <i>message</i> can be casted to a xcb_generic_event_t pointer.</p>
    /// <p>On Windows, <i>eventType</i> is set to "windows_generic_MSG" for messages sent to toplevel windows, and "windows_dispatcher_MSG" for system-wide messages such as messages from a registered hot key. In both cases, the <i>message</i> can be casted to a MSG pointer. The <i>result</i> pointer is only used on Windows, and corresponds to the LRESULT pointer.</p>
    /// <p>On <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, <i>eventType</i> is set to "mac_generic_NSEvent", and the <i>message</i> can be casted to an NSEvent pointer.</p>
    /// <p>In your reimplementation of this function, if you want to filter the <i>message</i> out, i.e. stop it being handled further, return true; otherwise return false.</p>
    /// <p><b>Linux example</b></p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">class</span> MyXcbEventFilter : <span class="keyword">public</span> <span class="type"><a href="http://doc.qt.io/qt-5/qabstractnativeeventfilter.html#QAbstractNativeEventFilter">QAbstractNativeEventFilter</a></span>
    ///   {
    ///   <span class="keyword">public</span>:
    /// &#32;     bool nativeEventFilter(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span> <span class="operator">&amp;</span>eventType<span class="operator">,</span> <span class="type">void</span> <span class="operator">*</span>message<span class="operator">,</span> <span class="type">long</span> <span class="operator">*</span>) override
    /// &#32;     {
    /// &#32;   &#32;     <span class="keyword">if</span> (eventType <span class="operator">=</span><span class="operator">=</span> <span class="string">"xcb_generic_event_t"</span>) {
    /// &#32;   &#32;   &#32;     xcb_generic_event_t<span class="operator">*</span> ev <span class="operator">=</span> <span class="keyword">static_cast</span><span class="operator">&lt;</span>xcb_generic_event_t <span class="operator">*</span><span class="operator">&gt;</span>(message);
    /// &#32;   &#32;   &#32;     <span class="comment">// ...</span>
    /// &#32;   &#32;     }
    /// &#32;   &#32;     <span class="keyword">return</span> <span class="keyword">false</span>;
    /// &#32;     }
    ///   };
    ///
    /// </pre>
    /// <p><b><a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> example</b></p>
    /// <p>mycocoaeventfilter.h:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="preprocessor">#include &lt;QAbstractNativeEventFilter&gt;</span>
    ///
    ///   <span class="keyword">class</span> MyCocoaEventFilter : <span class="keyword">public</span> <span class="type"><a href="http://doc.qt.io/qt-5/qabstractnativeeventfilter.html#QAbstractNativeEventFilter">QAbstractNativeEventFilter</a></span>
    ///   {
    ///   <span class="keyword">public</span>:
    /// &#32;     bool nativeEventFilter(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span> <span class="operator">&amp;</span>eventType<span class="operator">,</span> <span class="type">void</span> <span class="operator">*</span>message<span class="operator">,</span> <span class="type">long</span> <span class="operator">*</span>) override;
    ///   };
    ///
    /// </pre>
    /// <p>mycocoaeventfilter.mm:</p>
    /// <pre class="cpp">
    ///
    ///   #include "mycocoaeventfilter.h"
    ///
    ///   #import &lt;AppKit/AppKit.h&gt;
    ///
    ///   bool CocoaNativeEventFilter::nativeEventFilter(const QByteArray &amp;eventType, void *message, long *)
    ///   {
    /// &#32;     if (eventType == "mac_generic_NSEvent") {
    /// &#32;   &#32;     NSEvent *event = static_cast&lt;NSEvent *&gt;(message);
    /// &#32;   &#32;     if ([event type] == NSKeyDown) {
    /// &#32;   &#32;   &#32;     // Handle key event
    /// &#32;   &#32;   &#32;     qDebug() &lt;&lt; QString::fromNSString([event characters]);
    /// &#32;   &#32;     }
    /// &#32;     }
    /// &#32;     return false;
    ///   }
    ///
    /// </pre>
    /// <p>myapp.pro:</p>
    /// <pre class="cpp">
    ///
    ///   HEADERS += mycocoaeventfilter.h
    ///   OBJECTIVE_SOURCES += mycocoaeventfilter.mm
    ///   LIBS += -framework AppKit
    ///
    /// </pre></div>
    pub unsafe fn native_event_filter(
        &mut self,
        event_type: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        message: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
        result: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_long>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractNativeEventFilter_nativeEventFilter(
            self as *mut crate::QAbstractNativeEventFilter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(event_type)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(message)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_long>>::cast_into(result)
                .as_mut_raw_ptr(),
        )
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html">QAbstractProxyModel</a> class provides a base class for proxy item models that can do sorting, filtering or other data processing tasks.</p>
///
/// C++ class: <span style='color: green;'>```QAbstractProxyModel```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html">QAbstractProxyModel</a> class provides a base class for proxy item models that can do sorting, filtering or other data processing tasks.</p>
/// <p>This class defines the standard interface that proxy models must use to be able to interoperate correctly with other model/view components. It is not supposed to be instantiated directly.</p>
/// <p>All standard proxy models are derived from the <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html">QAbstractProxyModel</a> class. If you need to create a new proxy model class, it is usually better to subclass an existing class that provides the closest behavior to the one you want to provide.</p>
/// <p>Proxy models that filter or sort items of data from a source model should be created by using or subclassing <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html">QSortFilterProxyModel</a>.</p>
/// <p>To subclass <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html">QAbstractProxyModel</a>, you need to implement <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapFromSource">mapFromSource</a>() and <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapToSource">mapToSource</a>(). The <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapSelectionFromSource">mapSelectionFromSource</a>() and <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapSelectionToSource">mapSelectionToSource</a>() functions only need to be reimplemented if you need a behavior different from the default behavior.</p>
/// <p><b>Note: </b>If the source model is deleted or no source model is specified, the proxy model operates on a empty placeholder model.</p></div>
#[repr(C)]
pub struct QAbstractProxyModel {
    _unused: u8,
}
impl QAbstractProxyModel {
    /// <p>This property holds the source model of this proxy model.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractProxyModel::sourceModelChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#sourceModel-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the source model of this proxy model.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QAbstractItemModel *</td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#sourceModel">sourceModel</a></b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> virtual void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#setSourceModel">setSourceModel</a></b></span>(QAbstractItemModel *<i>sourceModel</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>sourceModelChanged</b></span>()</td><td class="memItemRight bottomAlign">[see note below]</td></tr>
    /// </tbody></table></div>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p></div>
    pub fn source_model_changed(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2sourceModelChanged()\0"),
            )
        }
    }

    /// <p>Clears the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#roleNames">roleNames</a> of this proxy model.</p>
    ///
    /// Returns a built-in Qt slot `QAbstractProxyModel::resetInternalData` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#resetInternalData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Clears the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#roleNames">roleNames</a> of this proxy model.</p></div>
    pub fn slot_reset_internal_data(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1resetInternalData()\0"),
            )
        }
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#buddy">QAbstractItemModel::buddy</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QAbstractProxyModel::buddy(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#buddy">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#buddy">QAbstractItemModel::buddy</a>().</p></div>
    pub unsafe fn buddy(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_buddy(
            self as *const crate::QAbstractProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#canDropMimeData">QAbstractItemModel::canDropMimeData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractProxyModel::canDropMimeData(const QMimeData* data, Qt::DropAction action, int row, int column, const QModelIndex& parent) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#canDropMimeData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#canDropMimeData">QAbstractItemModel::canDropMimeData</a>().</p>
    /// <p>This function was introduced in  Qt 5.4.</p></div>
    pub unsafe fn can_drop_mime_data(
        &self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QMimeData>>,
        action: crate::DropAction,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_canDropMimeData(
            self as *const crate::QAbstractProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QMimeData>>::cast_into(data)
                .as_raw_ptr(),
            action,
            row,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#canFetchMore">QAbstractItemModel::canFetchMore</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractProxyModel::canFetchMore(const QModelIndex& parent) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#canFetchMore">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#canFetchMore">QAbstractItemModel::canFetchMore</a>().</p></div>
    pub unsafe fn can_fetch_more(
        &self,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_canFetchMore(
            self as *const crate::QAbstractProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">QAbstractItemModel::data</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QVariant QAbstractProxyModel::data(const QModelIndex& proxyIndex, int role = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">QAbstractItemModel::data</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#setData">setData</a>().</p></div>
    pub unsafe fn data_2a(
        &self,
        proxy_index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        role: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_data(
            self as *const crate::QAbstractProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(proxy_index)
                .as_raw_ptr(),
            role,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">QAbstractItemModel::data</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QVariant QAbstractProxyModel::data(const QModelIndex& proxyIndex) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">QAbstractItemModel::data</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#setData">setData</a>().</p></div>
    pub unsafe fn data_1a(
        &self,
        proxy_index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_data1(
            self as *const crate::QAbstractProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(proxy_index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dropMimeData">QAbstractItemModel::dropMimeData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractProxyModel::dropMimeData(const QMimeData* data, Qt::DropAction action, int row, int column, const QModelIndex& parent)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#dropMimeData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dropMimeData">QAbstractItemModel::dropMimeData</a>().</p>
    /// <p>This function was introduced in  Qt 5.4.</p></div>
    pub unsafe fn drop_mime_data(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QMimeData>>,
        action: crate::DropAction,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_dropMimeData(
            self as *mut crate::QAbstractProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QMimeData>>::cast_into(data)
                .as_raw_ptr(),
            action,
            row,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#fetchMore">QAbstractItemModel::fetchMore</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual void QAbstractProxyModel::fetchMore(const QModelIndex& parent)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#fetchMore">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#fetchMore">QAbstractItemModel::fetchMore</a>().</p></div>
    pub unsafe fn fetch_more(
        &mut self,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_fetchMore(
            self as *mut crate::QAbstractProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#flags">QAbstractItemModel::flags</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QFlags<Qt::ItemFlag> QAbstractProxyModel::flags(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#flags">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#flags">QAbstractItemModel::flags</a>().</p></div>
    pub unsafe fn flags(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> crate::QFlags<crate::ItemFlag> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_flags(
            self as *const crate::QAbstractProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        );
        crate::QFlags::from(ffi_result)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#hasChildren">QAbstractItemModel::hasChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractProxyModel::hasChildren(const QModelIndex& parent = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#hasChildren">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#hasChildren">QAbstractItemModel::hasChildren</a>().</p></div>
    pub unsafe fn has_children_1a(
        &self,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_hasChildren(
            self as *const crate::QAbstractProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#hasChildren">QAbstractItemModel::hasChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractProxyModel::hasChildren() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#hasChildren">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#hasChildren">QAbstractItemModel::hasChildren</a>().</p></div>
    pub unsafe fn has_children_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_hasChildren1(
            self as *const crate::QAbstractProxyModel,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerData">QAbstractItemModel::headerData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QVariant QAbstractProxyModel::headerData(int section, Qt::Orientation orientation, int role = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#headerData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerData">QAbstractItemModel::headerData</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#setHeaderData">setHeaderData</a>().</p></div>
    pub unsafe fn header_data_3a(
        &self,
        section: ::std::os::raw::c_int,
        orientation: crate::Orientation,
        role: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_headerData(
            self as *const crate::QAbstractProxyModel,
            section,
            orientation,
            role,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerData">QAbstractItemModel::headerData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QVariant QAbstractProxyModel::headerData(int section, Qt::Orientation orientation) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#headerData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerData">QAbstractItemModel::headerData</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#setHeaderData">setHeaderData</a>().</p></div>
    pub unsafe fn header_data_2a(
        &self,
        section: ::std::os::raw::c_int,
        orientation: crate::Orientation,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_headerData1(
            self as *const crate::QAbstractProxyModel,
            section,
            orientation,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#itemData">QAbstractItemModel::itemData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QMap<int, QVariant> QAbstractProxyModel::itemData(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#itemData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#itemData">QAbstractItemModel::itemData</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#setItemData">setItemData</a>().</p></div>
    pub unsafe fn item_data(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QMapOfIntQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_itemData(
            self as *const crate::QAbstractProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplement this function to return the model index in the proxy model that corresponds to the <i>sourceIndex</i> from the source model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual QModelIndex QAbstractProxyModel::mapFromSource(const QModelIndex& sourceIndex) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapFromSource">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplement this function to return the model index in the proxy model that corresponds to the <i>sourceIndex</i> from the source model.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapToSource">mapToSource</a>().</p></div>
    pub unsafe fn map_from_source(
        &self,
        source_index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_mapFromSource(
            self as *const crate::QAbstractProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(source_index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a proxy selection mapped from the specified <i>sourceSelection</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QItemSelection QAbstractProxyModel::mapSelectionFromSource(const QItemSelection& selection) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapSelectionFromSource">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a proxy selection mapped from the specified <i>sourceSelection</i>.</p>
    /// <p>Reimplement this method to map source selections to proxy selections.</p></div>
    pub unsafe fn map_selection_from_source(
        &self,
        selection: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelection>>,
    ) -> ::cpp_utils::CppBox<crate::QItemSelection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_mapSelectionFromSource(
            self as *const crate::QAbstractProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelection>>::cast_into(selection)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a source selection mapped from the specified <i>proxySelection</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QItemSelection QAbstractProxyModel::mapSelectionToSource(const QItemSelection& selection) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapSelectionToSource">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a source selection mapped from the specified <i>proxySelection</i>.</p>
    /// <p>Reimplement this method to map proxy selections to source selections.</p></div>
    pub unsafe fn map_selection_to_source(
        &self,
        selection: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelection>>,
    ) -> ::cpp_utils::CppBox<crate::QItemSelection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_mapSelectionToSource(
            self as *const crate::QAbstractProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelection>>::cast_into(selection)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplement this function to return the model index in the source model that corresponds to the <i>proxyIndex</i> in the proxy model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual QModelIndex QAbstractProxyModel::mapToSource(const QModelIndex& proxyIndex) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapToSource">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplement this function to return the model index in the source model that corresponds to the <i>proxyIndex</i> in the proxy model.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapFromSource">mapFromSource</a>().</p></div>
    pub unsafe fn map_to_source(
        &self,
        proxy_index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_mapToSource(
            self as *const crate::QAbstractProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(proxy_index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QAbstractProxyModel::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_metaObject(
            self as *const crate::QAbstractProxyModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeData">QAbstractItemModel::mimeData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QMimeData* QAbstractProxyModel::mimeData(const QList<QModelIndex>& indexes) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mimeData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeData">QAbstractItemModel::mimeData</a>().</p></div>
    pub unsafe fn mime_data(
        &self,
        indexes: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQModelIndex>>,
    ) -> ::cpp_utils::MutPtr<crate::QMimeData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_mimeData(
            self as *const crate::QAbstractProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQModelIndex>>::cast_into(
                indexes,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeTypes">QAbstractItemModel::mimeTypes</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QStringList QAbstractProxyModel::mimeTypes() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mimeTypes">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeTypes">QAbstractItemModel::mimeTypes</a>().</p></div>
    pub unsafe fn mime_types(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_mimeTypes(
            self as *const crate::QAbstractProxyModel,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QAbstractProxyModel::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_qt_metacall(
            self as *mut crate::QAbstractProxyModel,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QAbstractProxyModel::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_qt_metacast(
            self as *mut crate::QAbstractProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#revert">QAbstractItemModel::revert</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual void QAbstractProxyModel::revert()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#revert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#revert">QAbstractItemModel::revert</a>().</p></div>
    pub unsafe fn revert(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_revert(
            self as *mut crate::QAbstractProxyModel,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">QAbstractItemModel::setData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractProxyModel::setData(const QModelIndex& index, const QVariant& value, int role = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#setData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">QAbstractItemModel::setData</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#data">data</a>().</p></div>
    pub unsafe fn set_data_3a(
        &mut self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
        role: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_setData(
            self as *mut crate::QAbstractProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
            role,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">QAbstractItemModel::setData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractProxyModel::setData(const QModelIndex& index, const QVariant& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#setData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">QAbstractItemModel::setData</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#data">data</a>().</p></div>
    pub unsafe fn set_data_2a(
        &mut self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_setData1(
            self as *mut crate::QAbstractProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setHeaderData">QAbstractItemModel::setHeaderData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractProxyModel::setHeaderData(int section, Qt::Orientation orientation, const QVariant& value, int role = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#setHeaderData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setHeaderData">QAbstractItemModel::setHeaderData</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#headerData">headerData</a>().</p></div>
    pub unsafe fn set_header_data_4a(
        &mut self,
        section: ::std::os::raw::c_int,
        orientation: crate::Orientation,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
        role: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_setHeaderData(
            self as *mut crate::QAbstractProxyModel,
            section,
            orientation,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
            role,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setHeaderData">QAbstractItemModel::setHeaderData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractProxyModel::setHeaderData(int section, Qt::Orientation orientation, const QVariant& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#setHeaderData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setHeaderData">QAbstractItemModel::setHeaderData</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#headerData">headerData</a>().</p></div>
    pub unsafe fn set_header_data_3a(
        &mut self,
        section: ::std::os::raw::c_int,
        orientation: crate::Orientation,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_setHeaderData1(
            self as *mut crate::QAbstractProxyModel,
            section,
            orientation,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setItemData">QAbstractItemModel::setItemData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractProxyModel::setItemData(const QModelIndex& index, const QMap<int, QVariant>& roles)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#setItemData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setItemData">QAbstractItemModel::setItemData</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#itemData">itemData</a>().</p></div>
    pub unsafe fn set_item_data(
        &mut self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        roles: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMapOfIntQVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_setItemData(
            self as *mut crate::QAbstractProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMapOfIntQVariant>>::cast_into(roles)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the given <i>sourceModel</i> to be processed by the proxy model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual void QAbstractProxyModel::setSourceModel(QAbstractItemModel* sourceModel)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#setSourceModel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the given <i>sourceModel</i> to be processed by the proxy model.</p>
    /// <p>Subclasses should call <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#beginResetModel">beginResetModel</a>() at the beginning of the method, disconnect from the old model, call this method, connect to the new model, and call <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#endResetModel">endResetModel</a>().</p>
    /// <p><b>Note:</b> Setter function for property <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#sourceModel-prop">sourceModel</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#sourceModel">sourceModel</a>().</p></div>
    pub unsafe fn set_source_model(
        &mut self,
        source_model: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractItemModel>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_setSourceModel(
            self as *mut crate::QAbstractProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractItemModel>>::cast_into(
                source_model,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sibling">QAbstractItemModel::sibling</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QAbstractProxyModel::sibling(int row, int column, const QModelIndex& idx) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#sibling">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sibling">QAbstractItemModel::sibling</a>().</p></div>
    pub unsafe fn sibling(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
        idx: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_sibling(
            self as *const crate::QAbstractProxyModel,
            row,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(idx)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sort">QAbstractItemModel::sort</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual void QAbstractProxyModel::sort(int column, Qt::SortOrder order = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#sort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sort">QAbstractItemModel::sort</a>().</p></div>
    pub unsafe fn sort_2a(&mut self, column: ::std::os::raw::c_int, order: crate::SortOrder) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_sort(
            self as *mut crate::QAbstractProxyModel,
            column,
            order,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sort">QAbstractItemModel::sort</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual void QAbstractProxyModel::sort(int column)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#sort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sort">QAbstractItemModel::sort</a>().</p></div>
    pub unsafe fn sort_1a(&mut self, column: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_sort1(
            self as *mut crate::QAbstractProxyModel,
            column,
        )
    }

    /// <p>This property holds the source model of this proxy model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* QAbstractProxyModel::sourceModel() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#sourceModel-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the source model of this proxy model.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QAbstractItemModel *</td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#sourceModel">sourceModel</a></b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> virtual void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#setSourceModel">setSourceModel</a></b></span>(QAbstractItemModel *<i>sourceModel</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>sourceModelChanged</b></span>()</td><td class="memItemRight bottomAlign">[see note below]</td></tr>
    /// </tbody></table></div>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p></div>
    pub unsafe fn source_model(&self) -> ::cpp_utils::MutPtr<crate::QAbstractItemModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_sourceModel(
            self as *const crate::QAbstractProxyModel,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#span">QAbstractItemModel::span</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QSize QAbstractProxyModel::span(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#span">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#span">QAbstractItemModel::span</a>().</p></div>
    pub unsafe fn span(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QSize> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_span(
            self as *const crate::QAbstractProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#submit">QAbstractItemModel::submit</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QAbstractProxyModel::submit()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#submit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#submit">QAbstractItemModel::submit</a>().</p></div>
    pub unsafe fn submit(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_submit(
            self as *mut crate::QAbstractProxyModel,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#supportedDragActions">QAbstractItemModel::supportedDragActions</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QFlags<Qt::DropAction> QAbstractProxyModel::supportedDragActions() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#supportedDragActions">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#supportedDragActions">QAbstractItemModel::supportedDragActions</a>().</p></div>
    pub unsafe fn supported_drag_actions(&self) -> crate::QFlags<crate::DropAction> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_supportedDragActions(
            self as *const crate::QAbstractProxyModel,
        );
        crate::QFlags::from(ffi_result)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#supportedDropActions">QAbstractItemModel::supportedDropActions</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QFlags<Qt::DropAction> QAbstractProxyModel::supportedDropActions() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#supportedDropActions">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#supportedDropActions">QAbstractItemModel::supportedDropActions</a>().</p></div>
    pub unsafe fn supported_drop_actions(&self) -> crate::QFlags<crate::DropAction> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_supportedDropActions(
            self as *const crate::QAbstractProxyModel,
        );
        crate::QFlags::from(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QAbstractProxyModel::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QAbstractProxyModel::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qabstractstate.html">QAbstractState</a> class is the base class of states of a <a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a>.</p>
///
/// C++ class: <span style='color: green;'>```QAbstractState```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qabstractstate.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qabstractstate.html">QAbstractState</a> class is the base class of states of a <a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a>.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qabstractstate.html">QAbstractState</a> class is the abstract base class of states that are part of a <a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a>. It defines the interface that all state objects have in common. <a href="http://doc.qt.io/qt-5/qabstractstate.html">QAbstractState</a> is part of <a href="http://doc.qt.io/qt-5/statemachine-api.html">The State Machine Framework</a>.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qabstractstate.html#entered">entered</a>() signal is emitted when the state has been entered. The <a href="http://doc.qt.io/qt-5/qabstractstate.html#exited">exited</a>() signal is emitted when the state has been exited.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qabstractstate.html#parentState">parentState</a>() function returns the state's parent state. The <a href="http://doc.qt.io/qt-5/qabstractstate.html#machine">machine</a>() function returns the state machine that the state is part of.</p>
/// <a name="subclassing"></a></div>
#[repr(C)]
pub struct QAbstractState {
    _unused: u8,
}
impl QAbstractState {
    /// <p>This signal is emitted when the state has been entered (after <a href="http://doc.qt.io/qt-5/qabstractstate.html#onEntry">onEntry</a>() has been called).</p>
    ///
    /// Returns a built-in Qt signal `QAbstractState::entered` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractstate.html#entered">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the state has been entered (after <a href="http://doc.qt.io/qt-5/qabstractstate.html#onEntry">onEntry</a>() has been called).</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p></div>
    pub fn entered(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2entered()\0"),
            )
        }
    }

    /// <p>This signal is emitted when the state has been exited (after <a href="http://doc.qt.io/qt-5/qabstractstate.html#onExit">onExit</a>() has been called).</p>
    ///
    /// Returns a built-in Qt signal `QAbstractState::exited` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractstate.html#exited">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the state has been exited (after <a href="http://doc.qt.io/qt-5/qabstractstate.html#onExit">onExit</a>() has been called).</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p></div>
    pub fn exited(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2exited()\0"),
            )
        }
    }

    /// <p>This signal is emitted when the active property is changed with <i>active</i> as argument.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractState::activeChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractstate.html#activeChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the active property is changed with <i>active</i> as argument.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>Note:</b> Notifier signal for property <a href="http://doc.qt.io/qt-5/qabstractstate.html#active-prop">active</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractstate.html#active-prop">QAbstractState::active</a>, <a href="http://doc.qt.io/qt-5/qabstractstate.html#entered">entered</a>(), and <a href="http://doc.qt.io/qt-5/qabstractstate.html#exited">exited</a>().</p></div>
    pub fn active_changed(&self) -> crate::Signal<(bool,)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2activeChanged(bool)\0"),
            )
        }
    }

    /// <p>This property holds the active property of this state. A state is active between <a href="http://doc.qt.io/qt-5/qabstractstate.html#entered">entered</a>() and <a href="http://doc.qt.io/qt-5/qabstractstate.html#exited">exited</a>() signals.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QAbstractState::active() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractstate.html#active-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the active property of this state. A state is active between <a href="http://doc.qt.io/qt-5/qabstractstate.html#entered">entered</a>() and <a href="http://doc.qt.io/qt-5/qabstractstate.html#exited">exited</a>() signals.</p>
    /// <p>This property was introduced in  Qt 5.4.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> bool </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qabstractstate.html#active">active</a></b></span>() const</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qabstractstate.html#activeChanged">activeChanged</a></b></span>(bool <i>active</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn active(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QAbstractState_active(self as *const crate::QAbstractState)
    }

    /// <p>Returns the state machine that this state is part of, or 0 if the state is not part of a state machine.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStateMachine* QAbstractState::machine() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractstate.html#machine">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the state machine that this state is part of, or 0 if the state is not part of a state machine.</p></div>
    pub unsafe fn machine(&self) -> ::cpp_utils::MutPtr<crate::QStateMachine> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractState_machine(
            self as *const crate::QAbstractState,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QAbstractState::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractState_metaObject(
            self as *const crate::QAbstractState,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns this state's parent state, or 0 if the state has no parent state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QState* QAbstractState::parentState() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractstate.html#parentState">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns this state's parent state, or 0 if the state has no parent state.</p></div>
    pub unsafe fn parent_state(&self) -> ::cpp_utils::MutPtr<crate::QState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractState_parentState(
            self as *const crate::QAbstractState,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QAbstractState::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QAbstractState_qt_metacall(
            self as *mut crate::QAbstractState,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QAbstractState::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractState_qt_metacast(
            self as *mut crate::QAbstractState,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractState_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QAbstractState::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractState_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QAbstractState::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractState_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_abstract_transition {
    //! C++ type: <span style='color: green;'>```QAbstractTransition```</span>

    /// <p>This enum specifies the kind of transition. By default, the type is an external transition.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QAbstractTransition::TransitionType```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracttransition.html#TransitionType-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum specifies the kind of transition. By default, the type is an external transition.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstracttransition.html#transitionType-prop">QAbstractTransition::transitionType</a>.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct TransitionType(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for TransitionType {
        fn from(value: ::std::os::raw::c_int) -> Self {
            TransitionType(value)
        }
    }

    impl From<TransitionType> for ::std::os::raw::c_int {
        fn from(value: TransitionType) -> Self {
            value.0
        }
    }

    impl TransitionType {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl TransitionType {
        /// Any state that is the source state of a transition (which is not a target-less transition) is left, and re-entered when necessary. (C++ enum variant: <span style='color: green;'>```ExternalTransition = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const ExternalTransition: crate::q_abstract_transition::TransitionType =
            crate::q_abstract_transition::TransitionType(0);
        /// If the target state of a transition is a sub-state of a compound state, and that compound state is the source state, an internal transition will not leave the source state. (C++ enum variant: <span style='color: green;'>```InternalTransition = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const InternalTransition: crate::q_abstract_transition::TransitionType =
            crate::q_abstract_transition::TransitionType(1);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qabstracttransition.html">QAbstractTransition</a> class is the base class of transitions between <a href="http://doc.qt.io/qt-5/qabstractstate.html">QAbstractState</a> objects.</p>
///
/// C++ class: <span style='color: green;'>```QAbstractTransition```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qabstracttransition.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qabstracttransition.html">QAbstractTransition</a> class is the base class of transitions between <a href="http://doc.qt.io/qt-5/qabstractstate.html">QAbstractState</a> objects.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qabstracttransition.html">QAbstractTransition</a> class is the abstract base class of transitions between states (<a href="http://doc.qt.io/qt-5/qabstractstate.html">QAbstractState</a> objects) of a <a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a>. <a href="http://doc.qt.io/qt-5/qabstracttransition.html">QAbstractTransition</a> is part of <a href="http://doc.qt.io/qt-5/statemachine-api.html">The State Machine Framework</a>.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qabstracttransition.html#sourceState">sourceState</a>() function returns the source of the transition. The <a href="http://doc.qt.io/qt-5/qabstracttransition.html#targetStates">targetStates</a>() function returns the targets of the transition. The <a href="http://doc.qt.io/qt-5/qabstracttransition.html#machine">machine</a>() function returns the state machine that the transition is part of.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qabstracttransition.html#triggered">triggered</a>() signal is emitted when the transition has been triggered.</p>
/// <p>Transitions can cause animations to be played. Use the <a href="http://doc.qt.io/qt-5/qabstracttransition.html#addAnimation">addAnimation</a>() function to add an animation to the transition.</p>
/// <a name="subclassing"></a></div>
#[repr(C)]
pub struct QAbstractTransition {
    _unused: u8,
}
impl QAbstractTransition {
    /// <p>This signal is emitted when the transition has been triggered (after <a href="http://doc.qt.io/qt-5/qabstracttransition.html#onTransition">onTransition</a>() has been called).</p>
    ///
    /// Returns a built-in Qt signal `QAbstractTransition::triggered` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracttransition.html#triggered">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the transition has been triggered (after <a href="http://doc.qt.io/qt-5/qabstracttransition.html#onTransition">onTransition</a>() has been called).</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p></div>
    pub fn triggered(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2triggered()\0"),
            )
        }
    }

    /// <p>This signal is emitted when the <a href="http://doc.qt.io/qt-5/qabstracttransition.html#targetState-prop">targetState</a> property is changed.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractTransition::targetStateChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracttransition.html#targetStateChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the <a href="http://doc.qt.io/qt-5/qabstracttransition.html#targetState-prop">targetState</a> property is changed.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>Note:</b> Notifier signal for property <a href="http://doc.qt.io/qt-5/qabstracttransition.html#targetState-prop">targetState</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstracttransition.html#targetState-prop">QAbstractTransition::targetState</a>.</p></div>
    pub fn target_state_changed(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2targetStateChanged()\0"),
            )
        }
    }

    /// <p>This signal is emitted when the <a href="http://doc.qt.io/qt-5/qabstracttransition.html#targetStates-prop">targetStates</a> property is changed.</p>
    ///
    /// Returns a built-in Qt signal `QAbstractTransition::targetStatesChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracttransition.html#targetStatesChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the <a href="http://doc.qt.io/qt-5/qabstracttransition.html#targetStates-prop">targetStates</a> property is changed.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>Note:</b> Notifier signal for property <a href="http://doc.qt.io/qt-5/qabstracttransition.html#targetStates-prop">targetStates</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstracttransition.html#targetStates-prop">QAbstractTransition::targetStates</a>.</p></div>
    pub fn target_states_changed(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2targetStatesChanged()\0"),
            )
        }
    }

    /// <p>Adds the given <i>animation</i> to this transition. The transition does not take ownership of the animation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QAbstractTransition::addAnimation(QAbstractAnimation* animation)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracttransition.html#addAnimation">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds the given <i>animation</i> to this transition. The transition does not take ownership of the animation.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstracttransition.html#removeAnimation">removeAnimation</a>() and <a href="http://doc.qt.io/qt-5/qabstracttransition.html#animations">animations</a>().</p></div>
    pub unsafe fn add_animation(
        &mut self,
        animation: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractAnimation>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractTransition_addAnimation(
            self as *mut crate::QAbstractTransition,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractAnimation>>::cast_into(
                animation,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the list of animations associated with this transition, or an empty list if it has no animations.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*> QAbstractTransition::animations() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracttransition.html#animations">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the list of animations associated with this transition, or an empty list if it has no animations.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstracttransition.html#addAnimation">addAnimation</a>().</p></div>
    pub unsafe fn animations(&self) -> ::cpp_utils::CppBox<crate::QListOfQAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractTransition_animations(
            self as *const crate::QAbstractTransition,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the state machine that this transition is part of, or 0 if the transition is not part of a state machine.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStateMachine* QAbstractTransition::machine() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracttransition.html#machine">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the state machine that this transition is part of, or 0 if the transition is not part of a state machine.</p></div>
    pub unsafe fn machine(&self) -> ::cpp_utils::MutPtr<crate::QStateMachine> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractTransition_machine(
            self as *const crate::QAbstractTransition,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QAbstractTransition::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractTransition_metaObject(
            self as *const crate::QAbstractTransition,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QAbstractTransition::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QAbstractTransition_qt_metacall(
            self as *mut crate::QAbstractTransition,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QAbstractTransition::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractTransition_qt_metacast(
            self as *mut crate::QAbstractTransition,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Removes the given <i>animation</i> from this transition.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QAbstractTransition::removeAnimation(QAbstractAnimation* animation)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracttransition.html#removeAnimation">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the given <i>animation</i> from this transition.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstracttransition.html#addAnimation">addAnimation</a>().</p></div>
    pub unsafe fn remove_animation(
        &mut self,
        animation: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractAnimation>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractTransition_removeAnimation(
            self as *mut crate::QAbstractTransition,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractAnimation>>::cast_into(
                animation,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Sets the <i>target</i> state of this transition.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QAbstractTransition::setTargetState(QAbstractState* target)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracttransition.html#setTargetState">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the <i>target</i> state of this transition.</p>
    /// <p><b>Note:</b> Setter function for property <a href="http://doc.qt.io/qt-5/qabstracttransition.html#targetState-prop">targetState</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstracttransition.html#targetState">targetState</a>().</p></div>
    pub unsafe fn set_target_state(
        &mut self,
        target: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractState>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractTransition_setTargetState(
            self as *mut crate::QAbstractTransition,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractState>>::cast_into(target)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Sets the target states of this transition to be the given <i>targets</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QAbstractTransition::setTargetStates(const QList<QAbstractState*>& targets)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracttransition.html#setTargetStates">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the target states of this transition to be the given <i>targets</i>.</p>
    /// <p><b>Note:</b> Setter function for property <a href="http://doc.qt.io/qt-5/qabstracttransition.html#targetStates-prop">targetStates</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstracttransition.html#targetStates">targetStates</a>().</p></div>
    pub unsafe fn set_target_states(
        &mut self,
        targets: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQAbstractState>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractTransition_setTargetStates(
            self as *mut crate::QAbstractTransition,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQAbstractState>>::cast_into(
                targets,
            )
            .as_raw_ptr(),
        )
    }

    /// <p>Sets the type of the transition to <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QAbstractTransition::setTransitionType(QAbstractTransition::TransitionType type)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracttransition.html#setTransitionType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the type of the transition to <i>type</i>.</p>
    /// <p><b>Note:</b> Setter function for property <a href="http://doc.qt.io/qt-5/qabstracttransition.html#transitionType-prop">transitionType</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstracttransition.html#transitionType">transitionType</a>().</p></div>
    pub unsafe fn set_transition_type(
        &mut self,
        type_: crate::q_abstract_transition::TransitionType,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractTransition_setTransitionType(
            self as *mut crate::QAbstractTransition,
            type_,
        )
    }

    /// <p>This property holds the source state (parent) of this transition</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QState* QAbstractTransition::sourceState() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracttransition.html#sourceState-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the source state (parent) of this transition</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QState *</td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qabstracttransition.html#sourceState">sourceState</a></b></span>() const</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn source_state(&self) -> ::cpp_utils::MutPtr<crate::QState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractTransition_sourceState(
            self as *const crate::QAbstractTransition,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractTransition_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This property holds the target state of this transition</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractState* QAbstractTransition::targetState() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracttransition.html#targetState-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the target state of this transition</p>
    /// <p>If a transition has no target state, the transition may still be triggered, but this will not cause the state machine's configuration to change (i.e. the current state will not be exited and re-entered).</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QAbstractState *</td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qabstracttransition.html#targetState">targetState</a></b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qabstracttransition.html#setTargetState">setTargetState</a></b></span>(QAbstractState *<i>target</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qabstracttransition.html#targetStateChanged">targetStateChanged</a></b></span>()</td><td class="memItemRight bottomAlign">[see note below]</td></tr>
    /// </tbody></table></div>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p></div>
    pub unsafe fn target_state(&self) -> ::cpp_utils::MutPtr<crate::QAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractTransition_targetState(
            self as *const crate::QAbstractTransition,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This property holds the target states of this transition</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*> QAbstractTransition::targetStates() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracttransition.html#targetStates-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the target states of this transition</p>
    /// <p>If multiple states are specified, all must be descendants of the same parallel group state.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QList&lt;QAbstractState *&gt; </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qabstracttransition.html#targetStates">targetStates</a></b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qabstracttransition.html#setTargetStates">setTargetStates</a></b></span>(const QList&lt;QAbstractState *&gt; &amp;<i>targets</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qabstracttransition.html#targetStatesChanged">targetStatesChanged</a></b></span>()</td><td class="memItemRight bottomAlign">[see note below]</td></tr>
    /// </tbody></table></div>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p></div>
    pub unsafe fn target_states(&self) -> ::cpp_utils::CppBox<crate::QListOfQAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractTransition_targetStates(
            self as *const crate::QAbstractTransition,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QAbstractTransition::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractTransition_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QAbstractTransition::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAbstractTransition_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>indicates whether this transition is an internal transition, or an external transition.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractTransition::TransitionType QAbstractTransition::transitionType() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracttransition.html#transitionType-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>indicates whether this transition is an internal transition, or an external transition.</p>
    /// <p>Internal and external transitions behave the same, except for the case of a transition whose source state is a compound state and whose target(s) is a descendant of the source. In such a case, an internal transition will not exit and re-enter its source state, while an external one will.</p>
    /// <p>By default, the type is an external transition.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> TransitionType </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qabstracttransition.html#transitionType">transitionType</a></b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qabstracttransition.html#setTransitionType">setTransitionType</a></b></span>(TransitionType <i>type</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn transition_type(&self) -> crate::q_abstract_transition::TransitionType {
        crate::__ffi::ctr_qt_core_ffi_QAbstractTransition_transitionType(
            self as *const crate::QAbstractTransition,
        )
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qanimationgroup.html">QAnimationGroup</a> class is an abstract base class for groups of animations.</p>
///
/// C++ class: <span style='color: green;'>```QAnimationGroup```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qanimationgroup.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qanimationgroup.html">QAnimationGroup</a> class is an abstract base class for groups of animations.</p>
/// <p>An animation group is a container for animations (subclasses of <a href="http://doc.qt.io/qt-5/qabstractanimation.html">QAbstractAnimation</a>). A group is usually responsible for managing the <a href="http://doc.qt.io/qt-5/qabstractanimation.html#State-enum">state</a> of its animations, i.e., it decides when to start, stop, resume, and pause them. Currently, Qt provides two such groups: <a href="http://doc.qt.io/qt-5/qparallelanimationgroup.html">QParallelAnimationGroup</a> and <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html">QSequentialAnimationGroup</a>. Look up their class descriptions for details.</p>
/// <p>Since <a href="http://doc.qt.io/qt-5/qanimationgroup.html">QAnimationGroup</a> inherits from <a href="http://doc.qt.io/qt-5/qabstractanimation.html">QAbstractAnimation</a>, you can combine groups, and easily construct complex animation graphs. You can query <a href="http://doc.qt.io/qt-5/qabstractanimation.html">QAbstractAnimation</a> for the group it belongs to (using the <a href="http://doc.qt.io/qt-5/qabstractanimation.html#group">group()</a> function).</p>
/// <p>To start a top-level animation group, you simply use the <a href="http://doc.qt.io/qt-5/qabstractanimation.html#start">start()</a> function from <a href="http://doc.qt.io/qt-5/qabstractanimation.html">QAbstractAnimation</a>. By a top-level animation group, we think of a group that itself is not contained within another group. Starting sub groups directly is not supported, and may lead to unexpected behavior.</p>
/// <p><a href="http://doc.qt.io/qt-5/qanimationgroup.html">QAnimationGroup</a> provides methods for adding and retrieving animations. Besides that, you can remove animations by calling <a href="http://doc.qt.io/qt-5/qanimationgroup.html#removeAnimation">removeAnimation</a>(), and clear the animation group by calling <a href="http://doc.qt.io/qt-5/qanimationgroup.html#clear">clear</a>(). You may keep track of changes in the group's animations by listening to <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::ChildAdded</a> and <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::ChildRemoved</a> events.</p>
/// <p><a href="http://doc.qt.io/qt-5/qanimationgroup.html">QAnimationGroup</a> takes ownership of the animations it manages, and ensures that they are deleted when the animation group is deleted.</p></div>
#[repr(C)]
pub struct QAnimationGroup {
    _unused: u8,
}
impl QAnimationGroup {
    /// <p>Adds <i>animation</i> to this group. This will call <a href="http://doc.qt.io/qt-5/qanimationgroup.html#insertAnimation">insertAnimation</a> with index equals to <a href="http://doc.qt.io/qt-5/qanimationgroup.html#animationCount">animationCount</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QAnimationGroup::addAnimation(QAbstractAnimation* animation)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qanimationgroup.html#addAnimation">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds <i>animation</i> to this group. This will call <a href="http://doc.qt.io/qt-5/qanimationgroup.html#insertAnimation">insertAnimation</a> with index equals to <a href="http://doc.qt.io/qt-5/qanimationgroup.html#animationCount">animationCount</a>().</p>
    /// <p><b>Note: </b>The group takes ownership of the animation.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qanimationgroup.html#removeAnimation">removeAnimation</a>().</p></div>
    pub unsafe fn add_animation(
        &mut self,
        animation: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractAnimation>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QAnimationGroup_addAnimation(
            self as *mut crate::QAnimationGroup,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractAnimation>>::cast_into(
                animation,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns a pointer to the animation at <i>index</i> in this group. This function is useful when you need access to a particular animation. <i>index</i> is between 0 and <a href="http://doc.qt.io/qt-5/qanimationgroup.html#animationCount">animationCount</a>() - 1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* QAnimationGroup::animationAt(int index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qanimationgroup.html#animationAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the animation at <i>index</i> in this group. This function is useful when you need access to a particular animation. <i>index</i> is between 0 and <a href="http://doc.qt.io/qt-5/qanimationgroup.html#animationCount">animationCount</a>() - 1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qanimationgroup.html#animationCount">animationCount</a>() and <a href="http://doc.qt.io/qt-5/qanimationgroup.html#indexOfAnimation">indexOfAnimation</a>().</p></div>
    pub unsafe fn animation_at(
        &self,
        index: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAnimationGroup_animationAt(
            self as *const crate::QAnimationGroup,
            index,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns the number of animations managed by this group.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QAnimationGroup::animationCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qanimationgroup.html#animationCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of animations managed by this group.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qanimationgroup.html#indexOfAnimation">indexOfAnimation</a>(), <a href="http://doc.qt.io/qt-5/qanimationgroup.html#addAnimation">addAnimation</a>(), and <a href="http://doc.qt.io/qt-5/qanimationgroup.html#animationAt">animationAt</a>().</p></div>
    pub unsafe fn animation_count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QAnimationGroup_animationCount(
            self as *const crate::QAnimationGroup,
        )
    }

    /// <p>Removes and deletes all animations in this animation group, and resets the current time to 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QAnimationGroup::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qanimationgroup.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes and deletes all animations in this animation group, and resets the current time to 0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qanimationgroup.html#addAnimation">addAnimation</a>() and <a href="http://doc.qt.io/qt-5/qanimationgroup.html#removeAnimation">removeAnimation</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QAnimationGroup_clear(self as *mut crate::QAnimationGroup)
    }

    /// <p>Returns the index of <i>animation</i>. The returned index can be passed to the other functions that take an index as an argument.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QAnimationGroup::indexOfAnimation(QAbstractAnimation* animation) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qanimationgroup.html#indexOfAnimation">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index of <i>animation</i>. The returned index can be passed to the other functions that take an index as an argument.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qanimationgroup.html#insertAnimation">insertAnimation</a>(), <a href="http://doc.qt.io/qt-5/qanimationgroup.html#animationAt">animationAt</a>(), and <a href="http://doc.qt.io/qt-5/qanimationgroup.html#takeAnimation">takeAnimation</a>().</p></div>
    pub unsafe fn index_of_animation(
        &self,
        animation: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractAnimation>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QAnimationGroup_indexOfAnimation(
            self as *const crate::QAnimationGroup,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractAnimation>>::cast_into(
                animation,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Inserts <i>animation</i> into this animation group at <i>index</i>. If <i>index</i> is 0 the animation is inserted at the beginning. If <i>index</i> is <a href="http://doc.qt.io/qt-5/qanimationgroup.html#animationCount">animationCount</a>(), the animation is inserted at the end.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QAnimationGroup::insertAnimation(int index, QAbstractAnimation* animation)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qanimationgroup.html#insertAnimation">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>animation</i> into this animation group at <i>index</i>. If <i>index</i> is 0 the animation is inserted at the beginning. If <i>index</i> is <a href="http://doc.qt.io/qt-5/qanimationgroup.html#animationCount">animationCount</a>(), the animation is inserted at the end.</p>
    /// <p><b>Note: </b>The group takes ownership of the animation.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qanimationgroup.html#takeAnimation">takeAnimation</a>(), <a href="http://doc.qt.io/qt-5/qanimationgroup.html#addAnimation">addAnimation</a>(), <a href="http://doc.qt.io/qt-5/qanimationgroup.html#indexOfAnimation">indexOfAnimation</a>(), and <a href="http://doc.qt.io/qt-5/qanimationgroup.html#removeAnimation">removeAnimation</a>().</p></div>
    pub unsafe fn insert_animation(
        &mut self,
        index: ::std::os::raw::c_int,
        animation: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractAnimation>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QAnimationGroup_insertAnimation(
            self as *mut crate::QAnimationGroup,
            index,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractAnimation>>::cast_into(
                animation,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QAnimationGroup::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAnimationGroup_metaObject(
            self as *const crate::QAnimationGroup,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QAnimationGroup::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QAnimationGroup_qt_metacall(
            self as *mut crate::QAnimationGroup,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QAnimationGroup::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAnimationGroup_qt_metacast(
            self as *mut crate::QAnimationGroup,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Removes <i>animation</i> from this group. The ownership of <i>animation</i> is transferred to the caller.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QAnimationGroup::removeAnimation(QAbstractAnimation* animation)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qanimationgroup.html#removeAnimation">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes <i>animation</i> from this group. The ownership of <i>animation</i> is transferred to the caller.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qanimationgroup.html#takeAnimation">takeAnimation</a>(), <a href="http://doc.qt.io/qt-5/qanimationgroup.html#insertAnimation">insertAnimation</a>(), and <a href="http://doc.qt.io/qt-5/qanimationgroup.html#addAnimation">addAnimation</a>().</p></div>
    pub unsafe fn remove_animation(
        &mut self,
        animation: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractAnimation>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QAnimationGroup_removeAnimation(
            self as *mut crate::QAnimationGroup,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractAnimation>>::cast_into(
                animation,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAnimationGroup_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the animation at <i>index</i> and removes it from the animation group.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* QAnimationGroup::takeAnimation(int index)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qanimationgroup.html#takeAnimation">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the animation at <i>index</i> and removes it from the animation group.</p>
    /// <p><b>Note: </b>The ownership of the animation is transferred to the caller.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qanimationgroup.html#removeAnimation">removeAnimation</a>(), <a href="http://doc.qt.io/qt-5/qanimationgroup.html#addAnimation">addAnimation</a>(), <a href="http://doc.qt.io/qt-5/qanimationgroup.html#insertAnimation">insertAnimation</a>(), and <a href="http://doc.qt.io/qt-5/qanimationgroup.html#indexOfAnimation">indexOfAnimation</a>().</p></div>
    pub unsafe fn take_animation(
        &mut self,
        index: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAnimationGroup_takeAnimation(
            self as *mut crate::QAnimationGroup,
            index,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QAnimationGroup::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAnimationGroup_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QAnimationGroup::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QAnimationGroup_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qbasictimer.html">QBasicTimer</a> class provides timer events for objects.</p>
///
/// C++ class: <span style='color: green;'>```QBasicTimer```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qbasictimer.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qbasictimer.html">QBasicTimer</a> class provides timer events for objects.</p>
/// <p>This is a fast, lightweight, and low-level class used by Qt internally. We recommend using the higher-level <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> class rather than this class if you want to use timers in your applications. Note that this timer is a repeating timer that will send subsequent timer events unless the <a href="http://doc.qt.io/qt-5/qbasictimer.html#stop">stop</a>() function is called.</p>
/// <p>To use this class, create a <a href="http://doc.qt.io/qt-5/qbasictimer.html">QBasicTimer</a>, and call its <a href="http://doc.qt.io/qt-5/qbasictimer.html#start">start</a>() function with a timeout interval and with a pointer to a <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> subclass. When the timer times out it will send a timer event to the <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> subclass. The timer can be stopped at any time using <a href="http://doc.qt.io/qt-5/qbasictimer.html#stop">stop</a>(). <a href="http://doc.qt.io/qt-5/qbasictimer.html#isActive">isActive</a>() returns <code>true</code> for a timer that is running; i.e. it has been started, has not reached the timeout time, and has not been stopped. The timer's ID can be retrieved using <a href="http://doc.qt.io/qt-5/qbasictimer.html#timerId">timerId</a>().</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qtwidgets-widgets-wiggly-example.html">Wiggly</a> example uses <a href="http://doc.qt.io/qt-5/qbasictimer.html">QBasicTimer</a> to repaint a widget at regular intervals.</p></div>
#[repr(C)]
pub struct QBasicTimer {
    _unused: u8,
}
impl QBasicTimer {
    /// <p>The <a href="http://doc.qt.io/qt-5/qbasictimer.html">QBasicTimer</a> class provides timer events for objects.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QBasicTimer& QBasicTimer::operator=(const QBasicTimer& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbasictimer.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qbasictimer.html">QBasicTimer</a> class provides timer events for objects.</p>
    /// <p>This is a fast, lightweight, and low-level class used by Qt internally. We recommend using the higher-level <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> class rather than this class if you want to use timers in your applications. Note that this timer is a repeating timer that will send subsequent timer events unless the <a href="http://doc.qt.io/qt-5/qbasictimer.html#stop">stop</a>() function is called.</p>
    /// <p>To use this class, create a <a href="http://doc.qt.io/qt-5/qbasictimer.html">QBasicTimer</a>, and call its <a href="http://doc.qt.io/qt-5/qbasictimer.html#start">start</a>() function with a timeout interval and with a pointer to a <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> subclass. When the timer times out it will send a timer event to the <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> subclass. The timer can be stopped at any time using <a href="http://doc.qt.io/qt-5/qbasictimer.html#stop">stop</a>(). <a href="http://doc.qt.io/qt-5/qbasictimer.html#isActive">isActive</a>() returns <code>true</code> for a timer that is running; i.e. it has been started, has not reached the timeout time, and has not been stopped. The timer's ID can be retrieved using <a href="http://doc.qt.io/qt-5/qbasictimer.html#timerId">timerId</a>().</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qtwidgets-widgets-wiggly-example.html">Wiggly</a> example uses <a href="http://doc.qt.io/qt-5/qbasictimer.html">QBasicTimer</a> to repaint a widget at regular intervals.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QBasicTimer>>,
    ) -> ::cpp_utils::MutRef<crate::QBasicTimer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QBasicTimer_operator_(
            self as *mut crate::QBasicTimer,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QBasicTimer>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the timer is running and has not been stopped; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QBasicTimer::isActive() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbasictimer.html#isActive">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the timer is running and has not been stopped; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbasictimer.html#start">start</a>() and <a href="http://doc.qt.io/qt-5/qbasictimer.html#stop">stop</a>().</p></div>
    pub unsafe fn is_active(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QBasicTimer_isActive(self as *const crate::QBasicTimer)
    }

    /// <p>Contructs a basic timer.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QBasicTimer::QBasicTimer()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbasictimer.html#QBasicTimer">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Contructs a basic timer.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbasictimer.html#start">start</a>().</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QBasicTimer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QBasicTimer_QBasicTimer();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qbasictimer.html">QBasicTimer</a> class provides timer events for objects.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QBasicTimer::QBasicTimer(const QBasicTimer& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbasictimer.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qbasictimer.html">QBasicTimer</a> class provides timer events for objects.</p>
    /// <p>This is a fast, lightweight, and low-level class used by Qt internally. We recommend using the higher-level <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> class rather than this class if you want to use timers in your applications. Note that this timer is a repeating timer that will send subsequent timer events unless the <a href="http://doc.qt.io/qt-5/qbasictimer.html#stop">stop</a>() function is called.</p>
    /// <p>To use this class, create a <a href="http://doc.qt.io/qt-5/qbasictimer.html">QBasicTimer</a>, and call its <a href="http://doc.qt.io/qt-5/qbasictimer.html#start">start</a>() function with a timeout interval and with a pointer to a <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> subclass. When the timer times out it will send a timer event to the <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> subclass. The timer can be stopped at any time using <a href="http://doc.qt.io/qt-5/qbasictimer.html#stop">stop</a>(). <a href="http://doc.qt.io/qt-5/qbasictimer.html#isActive">isActive</a>() returns <code>true</code> for a timer that is running; i.e. it has been started, has not reached the timeout time, and has not been stopped. The timer's ID can be retrieved using <a href="http://doc.qt.io/qt-5/qbasictimer.html#timerId">timerId</a>().</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qtwidgets-widgets-wiggly-example.html">Wiggly</a> example uses <a href="http://doc.qt.io/qt-5/qbasictimer.html">QBasicTimer</a> to repaint a widget at regular intervals.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QBasicTimer>>,
    ) -> ::cpp_utils::CppBox<crate::QBasicTimer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QBasicTimer_QBasicTimer1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QBasicTimer>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Starts (or restarts) the timer with a <i>msec</i> milliseconds timeout. The timer will be a <a href="http://doc.qt.io/qt-5/qt.html#TimerType-enum">Qt::CoarseTimer</a>. See <a href="http://doc.qt.io/qt-5/qt.html#TimerType-enum">Qt::TimerType</a> for information on the different timer types.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QBasicTimer::start(int msec, QObject* obj)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbasictimer.html#start">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Starts (or restarts) the timer with a <i>msec</i> milliseconds timeout. The timer will be a <a href="http://doc.qt.io/qt-5/qt.html#TimerType-enum">Qt::CoarseTimer</a>. See <a href="http://doc.qt.io/qt-5/qt.html#TimerType-enum">Qt::TimerType</a> for information on the different timer types.</p>
    /// <p>The given <i>object</i> will receive timer events.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbasictimer.html#stop">stop</a>(), <a href="http://doc.qt.io/qt-5/qbasictimer.html#isActive">isActive</a>(), <a href="http://doc.qt.io/qt-5/qobject.html#timerEvent">QObject::timerEvent</a>(), and <a href="http://doc.qt.io/qt-5/qt.html#TimerType-enum">Qt::CoarseTimer</a>.</p></div>
    pub unsafe fn start_2a(
        &mut self,
        msec: ::std::os::raw::c_int,
        obj: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QBasicTimer_start(
            self as *mut crate::QBasicTimer,
            msec,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(obj)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QBasicTimer::start(int msec, Qt::TimerType timerType, QObject* obj)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbasictimer.html#start-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Starts (or restarts) the timer with a <i>msec</i> milliseconds timeout and the given <i>timerType</i>. See <a href="http://doc.qt.io/qt-5/qt.html#TimerType-enum">Qt::TimerType</a> for information on the different timer types.</p>
    /// <p><i>obj</i> will receive timer events.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbasictimer.html#stop">stop</a>(), <a href="http://doc.qt.io/qt-5/qbasictimer.html#isActive">isActive</a>(), <a href="http://doc.qt.io/qt-5/qobject.html#timerEvent">QObject::timerEvent</a>(), and <a href="http://doc.qt.io/qt-5/qt.html#TimerType-enum">Qt::TimerType</a>.</p></div>
    pub unsafe fn start_3a(
        &mut self,
        msec: ::std::os::raw::c_int,
        timer_type: crate::TimerType,
        obj: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QBasicTimer_start1(
            self as *mut crate::QBasicTimer,
            msec,
            timer_type,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(obj)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Stops the timer.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QBasicTimer::stop()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbasictimer.html#stop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Stops the timer.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbasictimer.html#start">start</a>() and <a href="http://doc.qt.io/qt-5/qbasictimer.html#isActive">isActive</a>().</p></div>
    pub unsafe fn stop(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QBasicTimer_stop(self as *mut crate::QBasicTimer)
    }

    /// <p>Returns the timer's ID.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QBasicTimer::timerId() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbasictimer.html#timerId">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the timer's ID.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimerevent.html#timerId">QTimerEvent::timerId</a>().</p></div>
    pub unsafe fn timer_id(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QBasicTimer_timerId(self as *const crate::QBasicTimer)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a> class provides an array of bits.</p>
///
/// C++ class: <span style='color: green;'>```QBitArray```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qbitarray.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a> class provides an array of bits.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a> is an array that gives access to individual bits and provides operators (<a href="http://doc.qt.io/qt-5/qbitarray.html#operator-and">AND</a>, <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-7c">OR</a>, <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-5e">XOR</a>, and <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-7e">NOT</a>) that work on entire arrays of bits. It uses <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicit sharing</a> (copy-on-write) to reduce memory usage and to avoid the needless copying of data.</p>
/// <p>The following code constructs a <a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a> containing 200 bits initialized to false (0):</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span> ba(<span class="number">200</span>);
///
/// </pre>
/// <p>To initialize the bits to true, either pass <code>true</code> as second argument to the constructor, or call <a href="http://doc.qt.io/qt-5/qbitarray.html#fill">fill</a>() later on.</p>
/// <p><a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a> uses 0-based indexes, just like C++ arrays. To access the bit at a particular index position, you can use operator[](). On non-const bit arrays, operator[]() returns a reference to a bit that can be used on the left side of an assignment. For example:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span> ba;
///   ba<span class="operator">.</span>resize(<span class="number">3</span>);
///   ba<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="keyword">true</span>;
///   ba<span class="operator">[</span><span class="number">1</span><span class="operator">]</span> <span class="operator">=</span> <span class="keyword">false</span>;
///   ba<span class="operator">[</span><span class="number">2</span><span class="operator">]</span> <span class="operator">=</span> <span class="keyword">true</span>;
///
/// </pre>
/// <p>For technical reasons, it is more efficient to use <a href="http://doc.qt.io/qt-5/qbitarray.html#testBit">testBit</a>() and <a href="http://doc.qt.io/qt-5/qbitarray.html#setBit">setBit</a>() to access bits in the array than operator[](). For example:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span> ba(<span class="number">3</span>);
///   ba<span class="operator">.</span>setBit(<span class="number">0</span><span class="operator">,</span> <span class="keyword">true</span>);
///   ba<span class="operator">.</span>setBit(<span class="number">1</span><span class="operator">,</span> <span class="keyword">false</span>);
///   ba<span class="operator">.</span>setBit(<span class="number">2</span><span class="operator">,</span> <span class="keyword">true</span>);
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a> supports <code>&amp;</code> (<a href="http://doc.qt.io/qt-5/qbitarray.html#operator-and">AND</a>), <code>|</code> (<a href="http://doc.qt.io/qt-5/qbitarray.html#operator-7c">OR</a>), <code>^</code> (<a href="http://doc.qt.io/qt-5/qbitarray.html#operator-5e">XOR</a>), <code>~</code> (<a href="http://doc.qt.io/qt-5/qbitarray.html#operator-7e">NOT</a>), as well as <code>&amp;=</code>, <code>|=</code>, and <code>^=</code>. These operators work in the same way as the built-in C++ bitwise operators of the same name. For example:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span> x(<span class="number">5</span>);
///   x<span class="operator">.</span>setBit(<span class="number">3</span><span class="operator">,</span> <span class="keyword">true</span>);
///   <span class="comment">// x: [ 0, 0, 0, 1, 0 ]</span>
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span> y(<span class="number">5</span>);
///   y<span class="operator">.</span>setBit(<span class="number">4</span><span class="operator">,</span> <span class="keyword">true</span>);
///   <span class="comment">// y: [ 0, 0, 0, 0, 1 ]</span>
///
///   x <span class="operator">|</span><span class="operator">=</span> y;
///   <span class="comment">// x: [ 0, 0, 0, 1, 1 ]</span>
///
/// </pre>
/// <p>For historical reasons, <a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a> distinguishes between a null bit array and an empty bit array. A <i>null</i> bit array is a bit array that is initialized using <a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a>'s default constructor. An <i>empty</i> bit array is any bit array with size 0. A null bit array is always empty, but an empty bit array isn't necessarily null:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span>()<span class="operator">.</span>isNull();           <span class="comment">// returns true</span>
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span>()<span class="operator">.</span>isEmpty();          <span class="comment">// returns true</span>
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span>(<span class="number">0</span>)<span class="operator">.</span>isNull();          <span class="comment">// returns false</span>
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span>(<span class="number">0</span>)<span class="operator">.</span>isEmpty();         <span class="comment">// returns true</span>
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span>(<span class="number">3</span>)<span class="operator">.</span>isNull();          <span class="comment">// returns false</span>
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span>(<span class="number">3</span>)<span class="operator">.</span>isEmpty();         <span class="comment">// returns false</span>
///
/// </pre>
/// <p>All functions except <a href="http://doc.qt.io/qt-5/qbitarray.html#isNull">isNull</a>() treat null bit arrays the same as empty bit arrays; for example, <a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a>() compares equal to <a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a>(0). We recommend that you always use <a href="http://doc.qt.io/qt-5/qbitarray.html#isEmpty">isEmpty</a>() and avoid <a href="http://doc.qt.io/qt-5/qbitarray.html#isNull">isNull</a>().</p></div>
#[repr(C)]
pub struct QBitArray {
    _unused: u8,
}
impl QBitArray {
    /// <p>Returns the value of the bit at index position <i>i</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QBitArray::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value of the bit at index position <i>i</i>.</p>
    /// <p><i>i</i> must be a valid index position in the bit array (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qbitarray.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbitarray.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(&self, i: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QBitArray_at(self as *const crate::QBitArray, i)
    }

    /// <p>Returns a bit array that contains the inverted bits of this bit array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QBitArray QBitArray::operator~() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-7e">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a bit array that contains the inverted bits of this bit array.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span> a(<span class="number">3</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span> b;
    ///   a<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>; a<span class="operator">[</span><span class="number">1</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">0</span>; a<span class="operator">[</span><span class="number">2</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>;   <span class="comment">// a: [ 1, 0, 1 ]</span>
    ///   b <span class="operator">=</span> <span class="operator">~</span>a;                         <span class="comment">// b: [ 0, 1, 0 ]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbitarray.html#operator-and">operator&amp;</a>(), <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-7c">operator|</a>(), and <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-5e">operator^</a>().</p></div>
    pub unsafe fn bit_not(&self) -> ::cpp_utils::CppBox<crate::QBitArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QBitArray_operatord(self as *const crate::QBitArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a pointer to a dense bit array for this <a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a>. Bits are counted upwards from the least significant bit in each byte. The the number of bits relevant in the last byte is given by <code>size() % 8</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QBitArray::bits() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#bits">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to a dense bit array for this <a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a>. Bits are counted upwards from the least significant bit in each byte. The the number of bits relevant in the last byte is given by <code>size() % 8</code>.</p>
    /// <p>This function was introduced in  Qt 5.11.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbitarray.html#fromBits">fromBits</a>() and <a href="http://doc.qt.io/qt-5/qbitarray.html#size">size</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn bits(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QBitArray_bits(self as *const crate::QBitArray);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Clears the contents of the bit array and makes it empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QBitArray::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Clears the contents of the bit array and makes it empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbitarray.html#resize">resize</a>() and <a href="http://doc.qt.io/qt-5/qbitarray.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QBitArray_clear(self as *mut crate::QBitArray)
    }

    /// <p>Sets the bit at index position <i>i</i> to 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QBitArray::clearBit(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#clearBit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the bit at index position <i>i</i> to 0.</p>
    /// <p><i>i</i> must be a valid index position in the bit array (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qbitarray.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbitarray.html#setBit">setBit</a>() and <a href="http://doc.qt.io/qt-5/qbitarray.html#toggleBit">toggleBit</a>().</p></div>
    pub unsafe fn clear_bit(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QBitArray_clearBit(self as *mut crate::QBitArray, i)
    }

    /// <p>Assigns <i>other</i> to this bit array and returns a reference to this bit array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QBitArray& QBitArray::operator=(const QBitArray& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this bit array and returns a reference to this bit array.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QBitArray>>,
    ) -> ::cpp_utils::MutRef<crate::QBitArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QBitArray_operator_(
            self as *mut crate::QBitArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QBitArray>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Same as <a href="http://doc.qt.io/qt-5/qbitarray.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QBitArray::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as <a href="http://doc.qt.io/qt-5/qbitarray.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QBitArray_count(self as *const crate::QBitArray)
    }

    /// <p>If <i>on</i> is true, this function returns the number of 1-bits stored in the bit array; otherwise the number of 0-bits is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QBitArray::count(bool on) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If <i>on</i> is true, this function returns the number of 1-bits stored in the bit array; otherwise the number of 0-bits is returned.</p></div>
    pub unsafe fn count_1a(&self, on: bool) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QBitArray_count1(self as *const crate::QBitArray, on)
    }

    /// Calls C++ function: <span style='color: green;'>```void QBitArray::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QBitArray_detach(self as *mut crate::QBitArray)
    }

    /// <p>Sets every bit in the bit array to <i>value</i>, returning true if successful; otherwise returns <code>false</code>. If <i>size</i> is different from -1 (the default), the bit array is resized to <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QBitArray::fill(bool val, int size = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets every bit in the bit array to <i>value</i>, returning true if successful; otherwise returns <code>false</code>. If <i>size</i> is different from -1 (the default), the bit array is resized to <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span> ba(<span class="number">8</span>);
    ///   ba<span class="operator">.</span>fill(<span class="keyword">true</span>);
    ///   <span class="comment">// ba: [ 1, 1, 1, 1, 1, 1, 1, 1 ]</span>
    ///
    ///   ba<span class="operator">.</span>fill(<span class="keyword">false</span><span class="operator">,</span> <span class="number">2</span>);
    ///   <span class="comment">// ba: [ 0, 0 ]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbitarray.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_2a(&mut self, val: bool, size: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QBitArray_fill(self as *mut crate::QBitArray, val, size)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QBitArray::fill(bool val, int first, int last)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#fill-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Sets bits at index positions <i>begin</i> up to (but not including) <i>end</i> to <i>value</i>.</p>
    /// <p><i>begin</i> must be a valid index position in the bit array (0 &lt;= <i>begin</i> &lt; <a href="http://doc.qt.io/qt-5/qbitarray.html#size">size</a>()).</p>
    /// <p><i>end</i> must be either a valid index position or equal to <a href="http://doc.qt.io/qt-5/qbitarray.html#size">size</a>(), in which case the fill operation runs until the end of the array (0 &lt;= <i>end</i> &lt;= <a href="http://doc.qt.io/qt-5/qbitarray.html#size">size</a>()).</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span> ba(<span class="number">4</span>);
    ///   ba<span class="operator">.</span>fill(<span class="keyword">true</span><span class="operator">,</span> <span class="number">1</span><span class="operator">,</span> <span class="number">2</span>);            <span class="comment">// ba: [ 0, 1, 0, 0 ]</span>
    ///   ba<span class="operator">.</span>fill(<span class="keyword">true</span><span class="operator">,</span> <span class="number">1</span><span class="operator">,</span> <span class="number">3</span>);            <span class="comment">// ba: [ 0, 1, 1, 0 ]</span>
    ///   ba<span class="operator">.</span>fill(<span class="keyword">true</span><span class="operator">,</span> <span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);            <span class="comment">// ba: [ 0, 1, 1, 1 ]</span>
    ///
    /// </pre></div>
    pub unsafe fn fill_3a(
        &mut self,
        val: bool,
        first: ::std::os::raw::c_int,
        last: ::std::os::raw::c_int,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QBitArray_fill1(
            self as *mut crate::QBitArray,
            val,
            first,
            last,
        )
    }

    /// <p>Sets every bit in the bit array to <i>value</i>, returning true if successful; otherwise returns <code>false</code>. If <i>size</i> is different from -1 (the default), the bit array is resized to <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QBitArray::fill(bool val)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets every bit in the bit array to <i>value</i>, returning true if successful; otherwise returns <code>false</code>. If <i>size</i> is different from -1 (the default), the bit array is resized to <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span> ba(<span class="number">8</span>);
    ///   ba<span class="operator">.</span>fill(<span class="keyword">true</span>);
    ///   <span class="comment">// ba: [ 1, 1, 1, 1, 1, 1, 1, 1 ]</span>
    ///
    ///   ba<span class="operator">.</span>fill(<span class="keyword">false</span><span class="operator">,</span> <span class="number">2</span>);
    ///   <span class="comment">// ba: [ 0, 0 ]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbitarray.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_1a(&mut self, val: bool) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QBitArray_fill2(self as *mut crate::QBitArray, val)
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a> with the dense bit array located at <i>data</i>, with <i>size</i> bits. The byte array at <i>data</i> must be at least <i>size</i> / 8 (rounded up) bytes long.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QBitArray QBitArray::fromBits(const char* data, long long len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#fromBits">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a> with the dense bit array located at <i>data</i>, with <i>size</i> bits. The byte array at <i>data</i> must be at least <i>size</i> / 8 (rounded up) bytes long.</p>
    /// <p>If <i>size</i> is not a multiple of 8, this function will include the lowest <i>size</i> % 8 bits from the last byte in <i>data</i>.</p>
    /// <p>This function was introduced in  Qt 5.11.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbitarray.html#bits">bits</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn from_bits(
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        len: ::std::os::raw::c_longlong,
    ) -> ::cpp_utils::CppBox<crate::QBitArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QBitArray_fromBits(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(data)
                .as_raw_ptr(),
            len,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the bit at index position <i>i</i> as a modifiable reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QBitRef QBitArray::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the bit at index position <i>i</i> as a modifiable reference.</p>
    /// <p><i>i</i> must be a valid index position in the bit array (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qbitarray.html#size">size</a>()).</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span> a(<span class="number">3</span>);
    ///   a<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="keyword">false</span>;
    ///   a<span class="operator">[</span><span class="number">1</span><span class="operator">]</span> <span class="operator">=</span> <span class="keyword">true</span>;
    ///   a<span class="operator">[</span><span class="number">2</span><span class="operator">]</span> <span class="operator">=</span> a<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">^</span> a<span class="operator">[</span><span class="number">1</span><span class="operator">]</span>;
    ///
    /// </pre>
    /// <p>The return value is of type QBitRef, a helper class for <a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a>. When you get an object of type QBitRef, you can assign to it, and the assignment will apply to the bit in the <a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a> from which you got the reference.</p>
    /// <p>The functions <a href="http://doc.qt.io/qt-5/qbitarray.html#testBit">testBit</a>(), <a href="http://doc.qt.io/qt-5/qbitarray.html#setBit">setBit</a>(), and <a href="http://doc.qt.io/qt-5/qbitarray.html#clearBit">clearBit</a>() are slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbitarray.html#at">at</a>(), <a href="http://doc.qt.io/qt-5/qbitarray.html#testBit">testBit</a>(), <a href="http://doc.qt.io/qt-5/qbitarray.html#setBit">setBit</a>(), and <a href="http://doc.qt.io/qt-5/qbitarray.html#clearBit">clearBit</a>().</p></div>
    pub unsafe fn index_int_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QBitRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QBitArray_operator__(self as *mut crate::QBitArray, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QBitArray::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn index_int(&self, i: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QBitArray_operator__1(self as *const crate::QBitArray, i)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QBitRef QBitArray::operator[](unsigned int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-5b-5d-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn index_uint_mut(
        &mut self,
        i: ::std::os::raw::c_uint,
    ) -> ::cpp_utils::CppBox<crate::QBitRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QBitArray_operator__2(self as *mut crate::QBitArray, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QBitArray::operator[](unsigned int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-5b-5d-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn index_uint(&self, i: ::std::os::raw::c_uint) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QBitArray_operator__3(self as *const crate::QBitArray, i)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QBitArray::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QBitArray_isDetached(self as *const crate::QBitArray)
    }

    /// <p>Returns <code>true</code> if this bit array has size 0; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QBitArray::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this bit array has size 0; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbitarray.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QBitArray_isEmpty(self as *const crate::QBitArray)
    }

    /// <p>Returns <code>true</code> if this bit array is null; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QBitArray::isNull() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#isNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this bit array is null; otherwise returns <code>false</code>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span>()<span class="operator">.</span>isNull();           <span class="comment">// returns true</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span>(<span class="number">0</span>)<span class="operator">.</span>isNull();          <span class="comment">// returns false</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span>(<span class="number">3</span>)<span class="operator">.</span>isNull();          <span class="comment">// returns false</span>
    ///
    /// </pre>
    /// <p>Qt makes a distinction between null bit arrays and empty bit arrays for historical reasons. For most applications, what matters is whether or not a bit array contains any data, and this can be determined using <a href="http://doc.qt.io/qt-5/qbitarray.html#isEmpty">isEmpty</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbitarray.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QBitArray_isNull(self as *const crate::QBitArray)
    }

    /// <p>Constructs an empty bit array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QBitArray::QBitArray()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty bit array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbitarray.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QBitArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QBitArray_QBitArray();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a bit array containing <i>size</i> bits. The bits are initialized with <i>value</i>, which defaults to false (0).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QBitArray::QBitArray(int size, bool val = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a bit array containing <i>size</i> bits. The bits are initialized with <i>value</i>, which defaults to false (0).</p></div>
    pub unsafe fn new_2a(
        size: ::std::os::raw::c_int,
        val: bool,
    ) -> ::cpp_utils::CppBox<crate::QBitArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QBitArray_QBitArray1(size, val);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a bit array containing <i>size</i> bits. The bits are initialized with <i>value</i>, which defaults to false (0).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QBitArray::QBitArray(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a bit array containing <i>size</i> bits. The bits are initialized with <i>value</i>, which defaults to false (0).</p></div>
    pub unsafe fn new_1a(size: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QBitArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QBitArray_QBitArray4(size);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QBitArray::QBitArray(const QBitArray& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#constant-time">constant time</a>, because <a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbitarray.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QBitArray>>,
    ) -> ::cpp_utils::CppBox<crate::QBitArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QBitArray_QBitArray2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QBitArray>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Resizes the bit array to <i>size</i> bits.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QBitArray::resize(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#resize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Resizes the bit array to <i>size</i> bits.</p>
    /// <p>If <i>size</i> is greater than the current size, the bit array is extended to make it <i>size</i> bits with the extra bits added to the end. The new bits are initialized to false (0).</p>
    /// <p>If <i>size</i> is less than the current size, bits are removed from the end.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbitarray.html#size">size</a>().</p></div>
    pub unsafe fn resize(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QBitArray_resize(self as *mut crate::QBitArray, size)
    }

    /// <p>Sets the bit at index position <i>i</i> to 1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QBitArray::setBit(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#setBit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the bit at index position <i>i</i> to 1.</p>
    /// <p><i>i</i> must be a valid index position in the bit array (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qbitarray.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbitarray.html#clearBit">clearBit</a>() and <a href="http://doc.qt.io/qt-5/qbitarray.html#toggleBit">toggleBit</a>().</p></div>
    pub unsafe fn set_bit_1a(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QBitArray_setBit(self as *mut crate::QBitArray, i)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QBitArray::setBit(int i, bool val)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#setBit-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Sets the bit at index position <i>i</i> to <i>value</i>.</p></div>
    pub unsafe fn set_bit_2a(&mut self, i: ::std::os::raw::c_int, val: bool) {
        crate::__ffi::ctr_qt_core_ffi_QBitArray_setBit1(self as *mut crate::QBitArray, i, val)
    }

    /// <p>Returns the number of bits stored in the bit array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QBitArray::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of bits stored in the bit array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbitarray.html#resize">resize</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QBitArray_size(self as *const crate::QBitArray)
    }

    /// <p>Swaps bit array <i>other</i> with this bit array. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QBitArray::swap(QBitArray& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps bit array <i>other</i> with this bit array. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QBitArray>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QBitArray_swap(
            self as *mut crate::QBitArray,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QBitArray>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if the bit at index position <i>i</i> is 1; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QBitArray::testBit(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#testBit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the bit at index position <i>i</i> is 1; otherwise returns <code>false</code>.</p>
    /// <p><i>i</i> must be a valid index position in the bit array (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qbitarray.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbitarray.html#setBit">setBit</a>() and <a href="http://doc.qt.io/qt-5/qbitarray.html#clearBit">clearBit</a>().</p></div>
    pub unsafe fn test_bit(&self, i: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QBitArray_testBit(self as *const crate::QBitArray, i)
    }

    /// <p>Inverts the value of the bit at index position <i>i</i>, returning the previous value of that bit as either true (if it was set) or false (if it was unset).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QBitArray::toggleBit(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#toggleBit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inverts the value of the bit at index position <i>i</i>, returning the previous value of that bit as either true (if it was set) or false (if it was unset).</p>
    /// <p>If the previous value was 0, the new value will be 1. If the previous value was 1, the new value will be 0.</p>
    /// <p><i>i</i> must be a valid index position in the bit array (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qbitarray.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbitarray.html#setBit">setBit</a>() and <a href="http://doc.qt.io/qt-5/qbitarray.html#clearBit">clearBit</a>().</p></div>
    pub unsafe fn toggle_bit(&mut self, i: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QBitArray_toggleBit(self as *mut crate::QBitArray, i)
    }

    /// <p>Truncates the bit array at index position <i>pos</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QBitArray::truncate(int pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#truncate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Truncates the bit array at index position <i>pos</i>.</p>
    /// <p>If <i>pos</i> is beyond the end of the array, nothing happens.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbitarray.html#resize">resize</a>().</p></div>
    pub unsafe fn truncate(&mut self, pos: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QBitArray_truncate(self as *mut crate::QBitArray, pos)
    }
}

/// C++ class: <span style='color: green;'>```QBitRef```</span>.
#[repr(C)]
pub struct QBitRef {
    _unused: u8,
}
impl QBitRef {
    /// Calls C++ function: <span style='color: green;'>```QBitRef& QBitRef::operator=(const QBitRef& val)```</span>.
    pub unsafe fn copy_from_q_bit_ref(
        &mut self,
        val: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QBitRef>>,
    ) -> ::cpp_utils::MutRef<crate::QBitRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QBitRef_operator_1(
            self as *mut crate::QBitRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QBitRef>>::cast_into(val).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```QBitRef& QBitRef::operator=(bool val)```</span>.
    pub unsafe fn copy_from_bool(&mut self, val: bool) -> ::cpp_utils::MutRef<crate::QBitRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QBitRef_operator_2(self as *mut crate::QBitRef, val);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QBitRef::QBitRef(const QBitRef& other)```</span>.
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QBitRef>>,
    ) -> ::cpp_utils::CppBox<crate::QBitRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QBitRef_QBitRef1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QBitRef>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QBitRef::operator bool() const```</span>.
    pub unsafe fn to_bool(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QBitRef_operator_bool(self as *const crate::QBitRef)
    }
}

pub mod q_io_device {
    //! C++ type: <span style='color: green;'>```QIODevice```</span>

    /// <p>This enum is used with <a href="http://doc.qt.io/qt-5/qiodevice.html#open">open</a>() to describe the mode in which a device is opened. It is also returned by <a href="http://doc.qt.io/qt-5/qiodevice.html#openMode">openMode</a>().</p>
    ///
    /// C++ enum: <span style='color: green;'>```QIODevice::OpenModeFlag```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum is used with <a href="http://doc.qt.io/qt-5/qiodevice.html#open">open</a>() to describe the mode in which a device is opened. It is also returned by <a href="http://doc.qt.io/qt-5/qiodevice.html#openMode">openMode</a>().</p>
    ///
    /// <p>Certain flags, such as <code>Unbuffered</code> and <code>Truncate</code>, are meaningless when used with some subclasses. Some of these restrictions are implied by the type of device that is represented by a subclass. In other cases, the restriction may be due to the implementation, or may be imposed by the underlying platform; for example, <a href="http://doc.qt.io/qt-5/qtcpsocket.html">QTcpSocket</a> does not support <code>Unbuffered</code> mode, and limitations in the native API prevent <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> from supporting <code>Unbuffered</code> on Windows.</p>
    /// <p>The OpenMode type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;OpenModeFlag&gt;. It stores an OR combination of OpenModeFlag values.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct OpenModeFlag(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for OpenModeFlag {
        fn from(value: ::std::os::raw::c_int) -> Self {
            OpenModeFlag(value)
        }
    }

    impl From<OpenModeFlag> for ::std::os::raw::c_int {
        fn from(value: OpenModeFlag) -> Self {
            value.0
        }
    }

    impl OpenModeFlag {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl OpenModeFlag {
        /// The device is not open. (C++ enum variant: <span style='color: green;'>```NotOpen = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NotOpen: crate::q_io_device::OpenModeFlag = crate::q_io_device::OpenModeFlag(0);
        /// The device is open for reading. (C++ enum variant: <span style='color: green;'>```ReadOnly = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const ReadOnly: crate::q_io_device::OpenModeFlag = crate::q_io_device::OpenModeFlag(1);
        /// The device is open for writing. Note that this mode implies Truncate. (C++ enum variant: <span style='color: green;'>```WriteOnly = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const WriteOnly: crate::q_io_device::OpenModeFlag = crate::q_io_device::OpenModeFlag(2);
        /// The device is open for reading and writing. (C++ enum variant: <span style='color: green;'>```ReadWrite = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const ReadWrite: crate::q_io_device::OpenModeFlag = crate::q_io_device::OpenModeFlag(3);
        /// The device is opened in append mode so that all data is written to the end of the file. (C++ enum variant: <span style='color: green;'>```Append = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const Append: crate::q_io_device::OpenModeFlag = crate::q_io_device::OpenModeFlag(4);
        /// If possible, the device is truncated before it is opened. All earlier contents of the device are lost. (C++ enum variant: <span style='color: green;'>```Truncate = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const Truncate: crate::q_io_device::OpenModeFlag = crate::q_io_device::OpenModeFlag(8);
        /// When reading, the end-of-line terminators are translated to '\n'. When writing, the end-of-line terminators are translated to the local encoding, for example '\r\n' for Win32. (C++ enum variant: <span style='color: green;'>```Text = 16```</span>)
        #[allow(non_upper_case_globals)]
        pub const Text: crate::q_io_device::OpenModeFlag = crate::q_io_device::OpenModeFlag(16);
        /// Any buffer in the device is bypassed. (C++ enum variant: <span style='color: green;'>```Unbuffered = 32```</span>)
        #[allow(non_upper_case_globals)]
        pub const Unbuffered: crate::q_io_device::OpenModeFlag =
            crate::q_io_device::OpenModeFlag(32);
        /// Fail if the file to be opened already exists. Create and open the file only if it does not exist. There is a guarantee from the operating system that you are the only one creating and opening the file. Note that this mode implies WriteOnly, and combining it with ReadWrite is allowed. This flag currently only affects <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a>. Other classes might use this flag in the future, but until then using this flag with any classes other than <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> may result in undefined behavior. (C++ enum variant: <span style='color: green;'>```NewOnly = 64```</span>)
        #[allow(non_upper_case_globals)]
        pub const NewOnly: crate::q_io_device::OpenModeFlag = crate::q_io_device::OpenModeFlag(64);
        /// Fail if the file to be opened does not exist. This flag must be specified alongside ReadOnly, WriteOnly, or ReadWrite. Note that using this flag with ReadOnly alone is redundant, as ReadOnly already fails when the file does not exist. This flag currently only affects <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a>. Other classes might use this flag in the future, but until then using this flag with any classes other than <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> may result in undefined behavior. (C++ enum variant: <span style='color: green;'>```ExistingOnly = 128```</span>)
        #[allow(non_upper_case_globals)]
        pub const ExistingOnly: crate::q_io_device::OpenModeFlag =
            crate::q_io_device::OpenModeFlag(128);
    }

    impl From<crate::q_io_device::OpenModeFlag> for crate::QFlags<crate::q_io_device::OpenModeFlag> {
        fn from(value: crate::q_io_device::OpenModeFlag) -> Self {
            Self::from(value.to_int())
        }
    }

    impl<T: Into<crate::QFlags<crate::q_io_device::OpenModeFlag>>> std::ops::BitOr<T>
        for crate::q_io_device::OpenModeFlag
    {
        type Output = crate::QFlags<crate::q_io_device::OpenModeFlag>;
        fn bitor(self, rhs: T) -> crate::QFlags<crate::q_io_device::OpenModeFlag> {
            Into::<crate::QFlags<crate::q_io_device::OpenModeFlag>>::into(self) | rhs
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> class is the base interface class of all I/O devices in Qt.</p>
///
/// C++ class: <span style='color: green;'>```QIODevice```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qiodevice.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> class is the base interface class of all I/O devices in Qt.</p>
/// <p><a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> provides both a common implementation and an abstract interface for devices that support reading and writing of blocks of data, such as <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a>, <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> and <a href="http://doc.qt.io/qt-5/qtcpsocket.html">QTcpSocket</a>. <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> is abstract and can not be instantiated, but it is common to use the interface it defines to provide device-independent I/O features. For example, Qt's XML classes operate on a <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> pointer, allowing them to be used with various devices (such as files and buffers).</p>
/// <p>Before accessing the device, <a href="http://doc.qt.io/qt-5/qiodevice.html#open">open</a>() must be called to set the correct <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">OpenMode</a> (such as <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">ReadOnly</a> or <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">ReadWrite</a>). You can then write to the device with <a href="http://doc.qt.io/qt-5/qiodevice.html#write">write</a>() or <a href="http://doc.qt.io/qt-5/qiodevice.html#putChar">putChar</a>(), and read by calling either <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#readLine">readLine</a>(), or <a href="http://doc.qt.io/qt-5/qiodevice.html#readAll">readAll</a>(). Call <a href="http://doc.qt.io/qt-5/qiodevice.html#close">close</a>() when you are done with the device.</p>
/// <p><a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> distinguishes between two types of devices: random-access devices and sequential devices.</p>
/// <ul>
/// <li>Random-access devices support seeking to arbitrary positions using <a href="http://doc.qt.io/qt-5/qiodevice.html#seek">seek</a>(). The current position in the file is available by calling <a href="http://doc.qt.io/qt-5/qiodevice.html#pos">pos</a>(). <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> and <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> are examples of random-access devices.</li>
/// <li>Sequential devices don't support seeking to arbitrary positions. The data must be read in one pass. The functions <a href="http://doc.qt.io/qt-5/qiodevice.html#pos">pos</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#size">size</a>() don't work for sequential devices. <a href="http://doc.qt.io/qt-5/qtcpsocket.html">QTcpSocket</a> and <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> are examples of sequential devices.</li>
/// </ul>
/// <p>You can use <a href="http://doc.qt.io/qt-5/qiodevice.html#isSequential">isSequential</a>() to determine the type of device.</p>
/// <p><a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> emits <a href="http://doc.qt.io/qt-5/qiodevice.html#readyRead">readyRead</a>() when new data is available for reading; for example, if new data has arrived on the network or if additional data is appended to a file that you are reading from. You can call <a href="http://doc.qt.io/qt-5/qiodevice.html#bytesAvailable">bytesAvailable</a>() to determine the number of bytes that are currently available for reading. It's common to use <a href="http://doc.qt.io/qt-5/qiodevice.html#bytesAvailable">bytesAvailable</a>() together with the <a href="http://doc.qt.io/qt-5/qiodevice.html#readyRead">readyRead</a>() signal when programming with asynchronous devices such as <a href="http://doc.qt.io/qt-5/qtcpsocket.html">QTcpSocket</a>, where fragments of data can arrive at arbitrary points in time. <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> emits the <a href="http://doc.qt.io/qt-5/qiodevice.html#bytesWritten">bytesWritten</a>() signal every time a payload of data has been written to the device. Use <a href="http://doc.qt.io/qt-5/qiodevice.html#bytesToWrite">bytesToWrite</a>() to determine the current amount of data waiting to be written.</p>
/// <p>Certain subclasses of <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>, such as <a href="http://doc.qt.io/qt-5/qtcpsocket.html">QTcpSocket</a> and <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>, are asynchronous. This means that I/O functions such as <a href="http://doc.qt.io/qt-5/qiodevice.html#write">write</a>() or <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>() always return immediately, while communication with the device itself may happen when control goes back to the event loop. <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> provides functions that allow you to force these operations to be performed immediately, while blocking the calling thread and without entering the event loop. This allows <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> subclasses to be used without an event loop, or in a separate thread:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qiodevice.html#waitForReadyRead">waitForReadyRead</a>() - This function suspends operation in the calling thread until new data is available for reading.</li>
/// <li><a href="http://doc.qt.io/qt-5/qiodevice.html#waitForBytesWritten">waitForBytesWritten</a>() - This function suspends operation in the calling thread until one payload of data has been written to the device.</li>
/// <li>waitFor....() - Subclasses of <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> implement blocking functions for device-specific operations. For example, <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> has a function called <a href="http://doc.qt.io/qt-5/qprocess.html#waitForStarted">waitForStarted()</a> which suspends operation in the calling thread until the process has started.</li>
/// </ul>
/// <p>Calling these functions from the main, GUI thread, may cause your user interface to freeze. Example:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a></span> gzip;
///   gzip<span class="operator">.</span>start(<span class="string">"gzip"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"-c"</span>);
///   <span class="keyword">if</span> (<span class="operator">!</span>gzip<span class="operator">.</span>waitForStarted())
/// &#32;     <span class="keyword">return</span> <span class="keyword">false</span>;
///
///   gzip<span class="operator">.</span>write(<span class="string">"uncompressed data"</span>);
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span> compressed;
///   <span class="keyword">while</span> (gzip<span class="operator">.</span>waitForReadyRead())
/// &#32;     compressed <span class="operator">+</span><span class="operator">=</span> gzip<span class="operator">.</span>readAll();
///
/// </pre>
/// <p>By subclassing <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>, you can provide the same interface to your own I/O devices. Subclasses of <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> are only required to implement the protected <a href="http://doc.qt.io/qt-5/qiodevice.html#readData">readData</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#writeData">writeData</a>() functions. <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> uses these functions to implement all its convenience functions, such as <a href="http://doc.qt.io/qt-5/qiodevice.html#getChar">getChar</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#readLine">readLine</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#write">write</a>(). <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> also handles access control for you, so you can safely assume that the device is opened in write mode if <a href="http://doc.qt.io/qt-5/qiodevice.html#writeData">writeData</a>() is called.</p>
/// <p>Some subclasses, such as <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> and <a href="http://doc.qt.io/qt-5/qtcpsocket.html">QTcpSocket</a>, are implemented using a memory buffer for intermediate storing of data. This reduces the number of required device accessing calls, which are often very slow. Buffering makes functions like <a href="http://doc.qt.io/qt-5/qiodevice.html#getChar">getChar</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#putChar">putChar</a>() fast, as they can operate on the memory buffer instead of directly on the device itself. Certain I/O operations, however, don't work well with a buffer. For example, if several users open the same device and read it character by character, they may end up reading the same data when they meant to read a separate chunk each. For this reason, <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> allows you to bypass any buffering by passing the Unbuffered flag to <a href="http://doc.qt.io/qt-5/qiodevice.html#open">open</a>(). When subclassing <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>, remember to bypass any buffer you may use when the device is open in Unbuffered mode.</p>
/// <p>Usually, the incoming data stream from an asynchronous device is fragmented, and chunks of data can arrive at arbitrary points in time. To handle incomplete reads of data structures, use the transaction mechanism implemented by <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>. See <a href="http://doc.qt.io/qt-5/qiodevice.html#startTransaction">startTransaction</a>() and related functions for more details.</p>
/// <p>Some sequential devices support communicating via multiple channels. These channels represent separate streams of data that have the property of independently sequenced delivery. Once the device is opened, you can determine the number of channels by calling the <a href="http://doc.qt.io/qt-5/qiodevice.html#readChannelCount">readChannelCount</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#writeChannelCount">writeChannelCount</a>() functions. To switch between channels, call <a href="http://doc.qt.io/qt-5/qiodevice.html#setCurrentReadChannel">setCurrentReadChannel</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#setCurrentWriteChannel">setCurrentWriteChannel</a>(), respectively. <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> also provides additional signals to handle asynchronous communication on a per-channel basis.</p></div>
#[repr(C)]
pub struct QIODevice {
    _unused: u8,
}
impl QIODevice {
    /// <p>This signal is emitted once every time new data is available for reading from the device's current read channel. It will only be emitted again once new data is available, such as when a new payload of network data has arrived on your network socket, or when a new block of data has been appended to your device.</p>
    ///
    /// Returns a built-in Qt signal `QIODevice::readyRead` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#readyRead">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted once every time new data is available for reading from the device's current read channel. It will only be emitted again once new data is available, such as when a new payload of network data has arrived on your network socket, or when a new block of data has been appended to your device.</p>
    /// <p>readyRead() is not emitted recursively; if you reenter the event loop or call <a href="http://doc.qt.io/qt-5/qiodevice.html#waitForReadyRead">waitForReadyRead</a>() inside a slot connected to the readyRead() signal, the signal will not be reemitted (although <a href="http://doc.qt.io/qt-5/qiodevice.html#waitForReadyRead">waitForReadyRead</a>() may still return true).</p>
    /// <p>Note for developers implementing classes derived from <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>: you should always emit readyRead() when new data has arrived (do not emit it only because there's data still to be read in your buffers). Do not emit readyRead() in other conditions.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#bytesWritten">bytesWritten</a>().</p></div>
    pub fn ready_read(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2readyRead()\0"),
            )
        }
    }

    /// <p>This signal is emitted when new data is available for reading from the device. The <i>channel</i> argument is set to the index of the read channel on which the data has arrived. Unlike <a href="http://doc.qt.io/qt-5/qiodevice.html#readyRead">readyRead</a>(), it is emitted regardless of the <a href="http://doc.qt.io/qt-5/qiodevice.html#currentReadChannel">current read channel</a>.</p>
    ///
    /// Returns a built-in Qt signal `QIODevice::channelReadyRead` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#channelReadyRead">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when new data is available for reading from the device. The <i>channel</i> argument is set to the index of the read channel on which the data has arrived. Unlike <a href="http://doc.qt.io/qt-5/qiodevice.html#readyRead">readyRead</a>(), it is emitted regardless of the <a href="http://doc.qt.io/qt-5/qiodevice.html#currentReadChannel">current read channel</a>.</p>
    /// <p>channelReadyRead() can be emitted recursively - even for the same channel.</p>
    /// <p>This function was introduced in  Qt 5.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#readyRead">readyRead</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#channelBytesWritten">channelBytesWritten</a>().</p></div>
    pub fn channel_ready_read(&self) -> crate::Signal<(::std::os::raw::c_int,)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2channelReadyRead(int)\0"),
            )
        }
    }

    /// <p>This signal is emitted every time a payload of data has been written to the device's current write channel. The <i>bytes</i> argument is set to the number of bytes that were written in this payload.</p>
    ///
    /// Returns a built-in Qt signal `QIODevice::bytesWritten` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#bytesWritten">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted every time a payload of data has been written to the device's current write channel. The <i>bytes</i> argument is set to the number of bytes that were written in this payload.</p>
    /// <p>bytesWritten() is not emitted recursively; if you reenter the event loop or call <a href="http://doc.qt.io/qt-5/qiodevice.html#waitForBytesWritten">waitForBytesWritten</a>() inside a slot connected to the bytesWritten() signal, the signal will not be reemitted (although <a href="http://doc.qt.io/qt-5/qiodevice.html#waitForBytesWritten">waitForBytesWritten</a>() may still return true).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#readyRead">readyRead</a>().</p></div>
    pub fn bytes_written(&self) -> crate::Signal<(i64,)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2bytesWritten(qint64)\0"),
            )
        }
    }

    /// <p>This signal is emitted every time a payload of data has been written to the device. The <i>bytes</i> argument is set to the number of bytes that were written in this payload, while <i>channel</i> is the channel they were written to. Unlike <a href="http://doc.qt.io/qt-5/qiodevice.html#bytesWritten">bytesWritten</a>(), it is emitted regardless of the <a href="http://doc.qt.io/qt-5/qiodevice.html#currentWriteChannel">current write channel</a>.</p>
    ///
    /// Returns a built-in Qt signal `QIODevice::channelBytesWritten` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#channelBytesWritten">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted every time a payload of data has been written to the device. The <i>bytes</i> argument is set to the number of bytes that were written in this payload, while <i>channel</i> is the channel they were written to. Unlike <a href="http://doc.qt.io/qt-5/qiodevice.html#bytesWritten">bytesWritten</a>(), it is emitted regardless of the <a href="http://doc.qt.io/qt-5/qiodevice.html#currentWriteChannel">current write channel</a>.</p>
    /// <p>channelBytesWritten() can be emitted recursively - even for the same channel.</p>
    /// <p>This function was introduced in  Qt 5.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#bytesWritten">bytesWritten</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#channelReadyRead">channelReadyRead</a>().</p></div>
    pub fn channel_bytes_written(&self) -> crate::Signal<(::std::os::raw::c_int, i64)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2channelBytesWritten(int,qint64)\0",
                ),
            )
        }
    }

    /// <p>This signal is emitted when the device is about to close. Connect this signal if you have operations that need to be performed before the device closes (e.g., if you have data in a separate buffer that needs to be written to the device).</p>
    ///
    /// Returns a built-in Qt signal `QIODevice::aboutToClose` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#aboutToClose">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the device is about to close. Connect this signal if you have operations that need to be performed before the device closes (e.g., if you have data in a separate buffer that needs to be written to the device).</p></div>
    pub fn about_to_close(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2aboutToClose()\0"),
            )
        }
    }

    /// <p>This signal is emitted when the input (reading) stream is closed in this device. It is emitted as soon as the closing is detected, which means that there might still be data available for reading with <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>().</p>
    ///
    /// Returns a built-in Qt signal `QIODevice::readChannelFinished` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#readChannelFinished">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the input (reading) stream is closed in this device. It is emitted as soon as the closing is detected, which means that there might still be data available for reading with <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>().</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#atEnd">atEnd</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>().</p></div>
    pub fn read_channel_finished(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2readChannelFinished()\0"),
            )
        }
    }

    /// <p>Returns <code>true</code> if the current read and write position is at the end of the device (i.e. there is no more data available for reading on the device); otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QIODevice::atEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#atEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the current read and write position is at the end of the device (i.e. there is no more data available for reading on the device); otherwise returns <code>false</code>.</p>
    /// <p>For some devices, atEnd() can return true even though there is more data to read. This special case only applies to devices that generate data in direct response to you calling <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>() (e.g., <code>/dev</code> or <code>/proc</code> files on Unix and <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, or console input / <code>stdin</code> on all platforms).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#bytesAvailable">bytesAvailable</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>(), and <a href="http://doc.qt.io/qt-5/qiodevice.html#isSequential">isSequential</a>().</p></div>
    pub unsafe fn at_end(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_atEnd(self as *const crate::QIODevice)
    }

    /// <p>Returns the number of bytes that are available for reading. This function is commonly used with sequential devices to determine the number of bytes to allocate in a buffer before reading.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual qint64 QIODevice::bytesAvailable() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#bytesAvailable">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of bytes that are available for reading. This function is commonly used with sequential devices to determine the number of bytes to allocate in a buffer before reading.</p>
    /// <p>Subclasses that reimplement this function must call the base implementation in order to include the size of the buffer of <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>. Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#qint64-typedef">qint64</a></span> CustomDevice<span class="operator">::</span>bytesAvailable() <span class="keyword">const</span>
    ///   {
    /// &#32;     <span class="keyword">return</span> buffer<span class="operator">.</span>size() <span class="operator">+</span> <span class="type"><a href="http://doc.qt.io/qt-5/qiodevice.html#QIODevice">QIODevice</a></span><span class="operator">::</span>bytesAvailable();
    ///   }
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#bytesToWrite">bytesToWrite</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#readyRead">readyRead</a>(), and <a href="http://doc.qt.io/qt-5/qiodevice.html#isSequential">isSequential</a>().</p></div>
    pub unsafe fn bytes_available(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_bytesAvailable(self as *const crate::QIODevice)
    }

    /// <p>For buffered devices, this function returns the number of bytes waiting to be written. For devices with no buffer, this function returns 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual qint64 QIODevice::bytesToWrite() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#bytesToWrite">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>For buffered devices, this function returns the number of bytes waiting to be written. For devices with no buffer, this function returns 0.</p>
    /// <p>Subclasses that reimplement this function must call the base implementation in order to include the size of the buffer of <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#bytesAvailable">bytesAvailable</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#bytesWritten">bytesWritten</a>(), and <a href="http://doc.qt.io/qt-5/qiodevice.html#isSequential">isSequential</a>().</p></div>
    pub unsafe fn bytes_to_write(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_bytesToWrite(self as *const crate::QIODevice)
    }

    /// <p>Returns <code>true</code> if a complete line of data can be read from the device; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QIODevice::canReadLine() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#canReadLine">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if a complete line of data can be read from the device; otherwise returns <code>false</code>.</p>
    /// <p>Note that unbuffered devices, which have no way of determining what can be read, always return false.</p>
    /// <p>This function is often called in conjunction with the <a href="http://doc.qt.io/qt-5/qiodevice.html#readyRead">readyRead</a>() signal.</p>
    /// <p>Subclasses that reimplement this function must call the base implementation in order to include the contents of the <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>'s buffer. Example:</p>
    /// <pre class="cpp">
    ///
    ///   bool CustomDevice<span class="operator">::</span>canReadLine() <span class="keyword">const</span>
    ///   {
    /// &#32;     <span class="keyword">return</span> buffer<span class="operator">.</span>contains(<span class="char">'\n'</span>) <span class="operator">|</span><span class="operator">|</span> <span class="type"><a href="http://doc.qt.io/qt-5/qiodevice.html#QIODevice">QIODevice</a></span><span class="operator">::</span>canReadLine();
    ///   }
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#readyRead">readyRead</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#readLine">readLine</a>().</p></div>
    pub unsafe fn can_read_line(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_canReadLine(self as *const crate::QIODevice)
    }

    /// <p>First emits <a href="http://doc.qt.io/qt-5/qiodevice.html#aboutToClose">aboutToClose</a>(), then closes the device and sets its <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">OpenMode</a> to <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">NotOpen</a>. The error string is also reset.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual void QIODevice::close()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#close">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>First emits <a href="http://doc.qt.io/qt-5/qiodevice.html#aboutToClose">aboutToClose</a>(), then closes the device and sets its <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">OpenMode</a> to <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">NotOpen</a>. The error string is also reset.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#setOpenMode">setOpenMode</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">OpenMode</a>.</p></div>
    pub unsafe fn close(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_close(self as *mut crate::QIODevice)
    }

    /// <p>Completes a read transaction.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QIODevice::commitTransaction()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#commitTransaction">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Completes a read transaction.</p>
    /// <p>For sequential devices, all data recorded in the internal buffer during the transaction will be discarded.</p>
    /// <p>This function was introduced in  Qt 5.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#startTransaction">startTransaction</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#rollbackTransaction">rollbackTransaction</a>().</p></div>
    pub unsafe fn commit_transaction(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_commitTransaction(self as *mut crate::QIODevice)
    }

    /// <p>Returns the index of the current read channel.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QIODevice::currentReadChannel() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#currentReadChannel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index of the current read channel.</p>
    /// <p>This function was introduced in  Qt 5.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#setCurrentReadChannel">setCurrentReadChannel</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#readChannelCount">readChannelCount</a>(), and <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>.</p></div>
    pub unsafe fn current_read_channel(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_currentReadChannel(self as *const crate::QIODevice)
    }

    /// <p>Returns the the index of the current write channel.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QIODevice::currentWriteChannel() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#currentWriteChannel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the the index of the current write channel.</p>
    /// <p>This function was introduced in  Qt 5.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#setCurrentWriteChannel">setCurrentWriteChannel</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#writeChannelCount">writeChannelCount</a>().</p></div>
    pub unsafe fn current_write_channel(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_currentWriteChannel(self as *const crate::QIODevice)
    }

    /// <p>Returns a human-readable description of the last device error that occurred.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QIODevice::errorString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#errorString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a human-readable description of the last device error that occurred.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#setErrorString">setErrorString</a>().</p></div>
    pub unsafe fn error_string(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QIODevice_errorString(self as *const crate::QIODevice);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reads one character from the device and stores it in <i>c</i>. If <i>c</i> is 0, the character is discarded. Returns <code>true</code> on success; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QIODevice::getChar(char* c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#getChar">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads one character from the device and stores it in <i>c</i>. If <i>c</i> is 0, the character is discarded. Returns <code>true</code> on success; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#putChar">putChar</a>(), and <a href="http://doc.qt.io/qt-5/qiodevice.html#ungetChar">ungetChar</a>().</p></div>
    pub unsafe fn get_char(
        &mut self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_char>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_getChar(
            self as *mut crate::QIODevice,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_char>>::cast_into(c)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if the device is open; otherwise returns <code>false</code>. A device is open if it can be read from and/or written to. By default, this function returns <code>false</code> if <a href="http://doc.qt.io/qt-5/qiodevice.html#openMode">openMode</a>() returns <code>NotOpen</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QIODevice::isOpen() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#isOpen">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the device is open; otherwise returns <code>false</code>. A device is open if it can be read from and/or written to. By default, this function returns <code>false</code> if <a href="http://doc.qt.io/qt-5/qiodevice.html#openMode">openMode</a>() returns <code>NotOpen</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#openMode">openMode</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">OpenMode</a>.</p></div>
    pub unsafe fn is_open(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_isOpen(self as *const crate::QIODevice)
    }

    /// <p>Returns <code>true</code> if data can be read from the device; otherwise returns false. Use <a href="http://doc.qt.io/qt-5/qiodevice.html#bytesAvailable">bytesAvailable</a>() to determine how many bytes can be read.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QIODevice::isReadable() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#isReadable">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if data can be read from the device; otherwise returns false. Use <a href="http://doc.qt.io/qt-5/qiodevice.html#bytesAvailable">bytesAvailable</a>() to determine how many bytes can be read.</p>
    /// <p>This is a convenience function which checks if the <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">OpenMode</a> of the device contains the <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">ReadOnly</a> flag.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#openMode">openMode</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">OpenMode</a>.</p></div>
    pub unsafe fn is_readable(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_isReadable(self as *const crate::QIODevice)
    }

    /// <p>Returns <code>true</code> if this device is sequential; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QIODevice::isSequential() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#isSequential">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this device is sequential; otherwise returns false.</p>
    /// <p>Sequential devices, as opposed to a random-access devices, have no concept of a start, an end, a size, or a current position, and they do not support seeking. You can only read from the device when it reports that data is available. The most common example of a sequential device is a network socket. On Unix, special files such as /dev/zero and fifo pipes are sequential.</p>
    /// <p>Regular files, on the other hand, do support random access. They have both a size and a current position, and they also support seeking backwards and forwards in the data stream. Regular files are non-sequential.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#bytesAvailable">bytesAvailable</a>().</p></div>
    pub unsafe fn is_sequential(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_isSequential(self as *const crate::QIODevice)
    }

    /// <p>Returns <code>true</code> if the <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">Text</a> flag is enabled; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QIODevice::isTextModeEnabled() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#isTextModeEnabled">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">Text</a> flag is enabled; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#setTextModeEnabled">setTextModeEnabled</a>().</p></div>
    pub unsafe fn is_text_mode_enabled(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_isTextModeEnabled(self as *const crate::QIODevice)
    }

    /// <p>Returns <code>true</code> if a transaction is in progress on the device, otherwise <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QIODevice::isTransactionStarted() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#isTransactionStarted">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if a transaction is in progress on the device, otherwise <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 5.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#startTransaction">startTransaction</a>().</p></div>
    pub unsafe fn is_transaction_started(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_isTransactionStarted(
            self as *const crate::QIODevice,
        )
    }

    /// <p>Returns <code>true</code> if data can be written to the device; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QIODevice::isWritable() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#isWritable">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if data can be written to the device; otherwise returns false.</p>
    /// <p>This is a convenience function which checks if the <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">OpenMode</a> of the device contains the <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">WriteOnly</a> flag.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#openMode">openMode</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">OpenMode</a>.</p></div>
    pub unsafe fn is_writable(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_isWritable(self as *const crate::QIODevice)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QIODevice::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QIODevice_metaObject(self as *const crate::QIODevice);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Opens the device and sets its <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">OpenMode</a> to <i>mode</i>. Returns <code>true</code> if successful; otherwise returns <code>false</code>. This function should be called from any reimplementations of open() or other functions that open the device.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QIODevice::open(QFlags<QIODevice::OpenModeFlag> mode)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#open">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Opens the device and sets its <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">OpenMode</a> to <i>mode</i>. Returns <code>true</code> if successful; otherwise returns <code>false</code>. This function should be called from any reimplementations of open() or other functions that open the device.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#openMode">openMode</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">OpenMode</a>.</p></div>
    pub unsafe fn open(&mut self, mode: crate::QFlags<crate::q_io_device::OpenModeFlag>) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_open(self as *mut crate::QIODevice, mode.to_int())
    }

    /// <p>Returns the mode in which the device has been opened; i.e. <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">ReadOnly</a> or <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">WriteOnly</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFlags<QIODevice::OpenModeFlag> QIODevice::openMode() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#openMode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the mode in which the device has been opened; i.e. <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">ReadOnly</a> or <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">WriteOnly</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#setOpenMode">setOpenMode</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">OpenMode</a>.</p></div>
    pub unsafe fn open_mode(&self) -> crate::QFlags<crate::q_io_device::OpenModeFlag> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QIODevice_openMode(self as *const crate::QIODevice);
        crate::QFlags::from(ffi_result)
    }

    /// <p>Reads at most <i>maxSize</i> bytes from the device into <i>data</i>, without side effects (i.e., if you call <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>() after peek(), you will get the same data). Returns the number of bytes read. If an error occurs, such as when attempting to peek a device opened in <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">WriteOnly</a> mode, this function returns -1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QIODevice::peek(char* data, qint64 maxlen)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#peek">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads at most <i>maxSize</i> bytes from the device into <i>data</i>, without side effects (i.e., if you call <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>() after peek(), you will get the same data). Returns the number of bytes read. If an error occurs, such as when attempting to peek a device opened in <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">WriteOnly</a> mode, this function returns -1.</p>
    /// <p>0 is returned when no more data is available for reading.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   bool isExeFile(<span class="type"><a href="http://doc.qt.io/qt-5/qfile.html">QFile</a></span> <span class="operator">*</span>file)
    ///   {
    /// &#32;     <span class="type">char</span> buf<span class="operator">[</span><span class="number">2</span><span class="operator">]</span>;
    /// &#32;     <span class="keyword">if</span> (file<span class="operator">-</span><span class="operator">&gt;</span>peek(buf<span class="operator">,</span> <span class="keyword">sizeof</span>(buf)) <span class="operator">=</span><span class="operator">=</span> <span class="keyword">sizeof</span>(buf))
    /// &#32;   &#32;     <span class="keyword">return</span> (buf<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="char">'M'</span> <span class="operator">&amp;</span><span class="operator">&amp;</span> buf<span class="operator">[</span><span class="number">1</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="char">'Z'</span>);
    /// &#32;     <span class="keyword">return</span> <span class="keyword">false</span>;
    ///   }
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>().</p></div>
    pub unsafe fn peek_2a(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_char>>,
        maxlen: i64,
    ) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_peek(
            self as *mut crate::QIODevice,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_char>>::cast_into(data)
                .as_mut_raw_ptr(),
            maxlen,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QIODevice::peek(qint64 maxlen)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#peek-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Peeks at most <i>maxSize</i> bytes from the device, returning the data peeked as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   bool isExeFile(<span class="type"><a href="http://doc.qt.io/qt-5/qfile.html">QFile</a></span> <span class="operator">*</span>file)
    ///   {
    /// &#32;     <span class="keyword">return</span> file<span class="operator">-</span><span class="operator">&gt;</span>peek(<span class="number">2</span>) <span class="operator">=</span><span class="operator">=</span> <span class="string">"MZ"</span>;
    ///   }
    ///
    /// </pre>
    /// <p>This function has no way of reporting errors; returning an empty <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> can mean either that no data was currently available for peeking, or that an error occurred.</p>
    /// <p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>().</p></div>
    pub unsafe fn peek_1a(&mut self, maxlen: i64) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QIODevice_peek1(self as *mut crate::QIODevice, maxlen);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>For random-access devices, this function returns the position that data is written to or read from. For sequential devices or closed devices, where there is no concept of a "current position", 0 is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual qint64 QIODevice::pos() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#pos">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>For random-access devices, this function returns the position that data is written to or read from. For sequential devices or closed devices, where there is no concept of a "current position", 0 is returned.</p>
    /// <p>The current read/write position of the device is maintained internally by <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>, so reimplementing this function is not necessary. When subclassing <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>, use <a href="http://doc.qt.io/qt-5/qiodevice.html#seek">QIODevice::seek</a>() to notify <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> about changes in the device position.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#isSequential">isSequential</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#seek">seek</a>().</p></div>
    pub unsafe fn pos(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_pos(self as *const crate::QIODevice)
    }

    /// <p>Writes the character <i>c</i> to the device. Returns <code>true</code> on success; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QIODevice::putChar(char c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#putChar">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes the character <i>c</i> to the device. Returns <code>true</code> on success; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#write">write</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#getChar">getChar</a>(), and <a href="http://doc.qt.io/qt-5/qiodevice.html#ungetChar">ungetChar</a>().</p></div>
    pub unsafe fn put_char(&mut self, c: ::std::os::raw::c_char) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_putChar(self as *mut crate::QIODevice, c)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QIODevice::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_qt_metacall(
            self as *mut crate::QIODevice,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QIODevice::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QIODevice_qt_metacast(
            self as *mut crate::QIODevice,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Reads at most <i>maxSize</i> bytes from the device into <i>data</i>, and returns the number of bytes read. If an error occurs, such as when attempting to read from a device opened in <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">WriteOnly</a> mode, this function returns -1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QIODevice::read(char* data, qint64 maxlen)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#read">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads at most <i>maxSize</i> bytes from the device into <i>data</i>, and returns the number of bytes read. If an error occurs, such as when attempting to read from a device opened in <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">WriteOnly</a> mode, this function returns -1.</p>
    /// <p>0 is returned when no more data is available for reading. However, reading past the end of the stream is considered an error, so this function returns -1 in those cases (that is, reading on a closed socket or after a process has died).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#readData">readData</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#readLine">readLine</a>(), and <a href="http://doc.qt.io/qt-5/qiodevice.html#write">write</a>().</p></div>
    pub unsafe fn read_2a(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_char>>,
        maxlen: i64,
    ) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_read(
            self as *mut crate::QIODevice,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_char>>::cast_into(data)
                .as_mut_raw_ptr(),
            maxlen,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QIODevice::read(qint64 maxlen)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#read-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Reads at most <i>maxSize</i> bytes from the device, and returns the data read as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    /// <p>This function has no way of reporting errors; returning an empty <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> can mean either that no data was currently available for reading, or that an error occurred.</p></div>
    pub unsafe fn read_1a(&mut self, maxlen: i64) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QIODevice_read1(self as *mut crate::QIODevice, maxlen);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reads all remaining data from the device, and returns it as a byte array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QIODevice::readAll()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#readAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads all remaining data from the device, and returns it as a byte array.</p>
    /// <p>This function has no way of reporting errors; returning an empty <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> can mean either that no data was currently available for reading, or that an error occurred.</p></div>
    pub unsafe fn read_all(&mut self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QIODevice_readAll(self as *mut crate::QIODevice);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the number of available read channels if the device is open; otherwise returns 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QIODevice::readChannelCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#readChannelCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of available read channels if the device is open; otherwise returns 0.</p>
    /// <p>This function was introduced in  Qt 5.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#writeChannelCount">writeChannelCount</a>() and <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>.</p></div>
    pub unsafe fn read_channel_count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_readChannelCount(self as *const crate::QIODevice)
    }

    /// <p>This function reads a line of ASCII characters from the device, up to a maximum of <i>maxSize</i> - 1 bytes, stores the characters in <i>data</i>, and returns the number of bytes read. If a line could not be read but no error ocurred, this function returns 0. If an error occurs, this function returns the length of what could be read, or -1 if nothing was read.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QIODevice::readLine(char* data, qint64 maxlen)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#readLine">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function reads a line of ASCII characters from the device, up to a maximum of <i>maxSize</i> - 1 bytes, stores the characters in <i>data</i>, and returns the number of bytes read. If a line could not be read but no error ocurred, this function returns 0. If an error occurs, this function returns the length of what could be read, or -1 if nothing was read.</p>
    /// <p>A terminating '\0' byte is always appended to <i>data</i>, so <i>maxSize</i> must be larger than 1.</p>
    /// <p>Data is read until either of the following conditions are met:</p>
    /// <ul>
    /// <li>The first '\n' character is read.</li>
    /// <li><i>maxSize</i> - 1 bytes are read.</li>
    /// <li>The end of the device data is detected.</li>
    /// </ul>
    /// <p>For example, the following code reads a line of characters from a file:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qfile.html">QFile</a></span> file(<span class="string">"box.txt"</span>);
    ///   <span class="keyword">if</span> (file<span class="operator">.</span>open(<span class="type"><a href="http://doc.qt.io/qt-5/qfile.html">QFile</a></span><span class="operator">::</span>ReadOnly)) {
    /// &#32;     <span class="type">char</span> buf<span class="operator">[</span><span class="number">1024</span><span class="operator">]</span>;
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#qint64-typedef">qint64</a></span> lineLength <span class="operator">=</span> file<span class="operator">.</span>readLine(buf<span class="operator">,</span> <span class="keyword">sizeof</span>(buf));
    /// &#32;     <span class="keyword">if</span> (lineLength <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>) {
    /// &#32;   &#32;     <span class="comment">// the line is available in buf</span>
    /// &#32;     }
    ///   }
    ///
    /// </pre>
    /// <p>The newline character ('\n') is included in the buffer. If a newline is not encountered before maxSize - 1 bytes are read, a newline will not be inserted into the buffer. On windows newline characters are replaced with '\n'.</p>
    /// <p>This function calls <a href="http://doc.qt.io/qt-5/qiodevice.html#readLineData">readLineData</a>(), which is implemented using repeated calls to <a href="http://doc.qt.io/qt-5/qiodevice.html#getChar">getChar</a>(). You can provide a more efficient implementation by reimplementing <a href="http://doc.qt.io/qt-5/qiodevice.html#readLineData">readLineData</a>() in your own subclass.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#getChar">getChar</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>(), and <a href="http://doc.qt.io/qt-5/qiodevice.html#write">write</a>().</p></div>
    pub unsafe fn read_line_2a(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_char>>,
        maxlen: i64,
    ) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_readLine(
            self as *mut crate::QIODevice,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_char>>::cast_into(data)
                .as_mut_raw_ptr(),
            maxlen,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QIODevice::readLine(qint64 maxlen = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#readLine-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Reads a line from the device, but no more than <i>maxSize</i> characters, and returns the result as a byte array.</p>
    /// <p>This function has no way of reporting errors; returning an empty <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> can mean either that no data was currently available for reading, or that an error occurred.</p></div>
    pub unsafe fn read_line_1a(&mut self, maxlen: i64) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QIODevice_readLine1(
            self as *mut crate::QIODevice,
            maxlen,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QIODevice::readLine()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#readLine-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Reads a line from the device, but no more than <i>maxSize</i> characters, and returns the result as a byte array.</p>
    /// <p>This function has no way of reporting errors; returning an empty <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> can mean either that no data was currently available for reading, or that an error occurred.</p></div>
    pub unsafe fn read_line_0a(&mut self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QIODevice_readLine2(self as *mut crate::QIODevice);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Seeks to the start of input for random-access devices. Returns true on success; otherwise returns <code>false</code> (for example, if the device is not open).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QIODevice::reset()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#reset">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Seeks to the start of input for random-access devices. Returns true on success; otherwise returns <code>false</code> (for example, if the device is not open).</p>
    /// <p>Note that when using a <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> on a <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a>, calling reset() on the <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> will not have the expected result because <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> buffers the file. Use the <a href="http://doc.qt.io/qt-5/qtextstream.html#seek">QTextStream::seek</a>() function instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#seek">seek</a>().</p></div>
    pub unsafe fn reset(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_reset(self as *mut crate::QIODevice)
    }

    /// <p>Rolls back a read transaction.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QIODevice::rollbackTransaction()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#rollbackTransaction">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Rolls back a read transaction.</p>
    /// <p>Restores the input stream to the point of the <a href="http://doc.qt.io/qt-5/qiodevice.html#startTransaction">startTransaction</a>() call. This function is commonly used to rollback the transaction when an incomplete read was detected prior to committing the transaction.</p>
    /// <p>This function was introduced in  Qt 5.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#startTransaction">startTransaction</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#commitTransaction">commitTransaction</a>().</p></div>
    pub unsafe fn rollback_transaction(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_rollbackTransaction(self as *mut crate::QIODevice)
    }

    /// <p>For random-access devices, this function sets the current position to <i>pos</i>, returning true on success, or false if an error occurred. For sequential devices, the default behavior is to produce a warning and return false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QIODevice::seek(qint64 pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#seek">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>For random-access devices, this function sets the current position to <i>pos</i>, returning true on success, or false if an error occurred. For sequential devices, the default behavior is to produce a warning and return false.</p>
    /// <p>When subclassing <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>, you must call QIODevice::seek() at the start of your function to ensure integrity with <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>'s built-in buffer.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#pos">pos</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#isSequential">isSequential</a>().</p></div>
    pub unsafe fn seek(&mut self, pos: i64) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_seek(self as *mut crate::QIODevice, pos)
    }

    /// <p>Sets the current read channel of the <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> to the given <i>channel</i>. The current input channel is used by the functions <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#readAll">readAll</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#readLine">readLine</a>(), and <a href="http://doc.qt.io/qt-5/qiodevice.html#getChar">getChar</a>(). It also determines which channel triggers <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> to emit <a href="http://doc.qt.io/qt-5/qiodevice.html#readyRead">readyRead</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QIODevice::setCurrentReadChannel(int channel)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#setCurrentReadChannel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the current read channel of the <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> to the given <i>channel</i>. The current input channel is used by the functions <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#readAll">readAll</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#readLine">readLine</a>(), and <a href="http://doc.qt.io/qt-5/qiodevice.html#getChar">getChar</a>(). It also determines which channel triggers <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> to emit <a href="http://doc.qt.io/qt-5/qiodevice.html#readyRead">readyRead</a>().</p>
    /// <p>This function was introduced in  Qt 5.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#currentReadChannel">currentReadChannel</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#readChannelCount">readChannelCount</a>(), and <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>.</p></div>
    pub unsafe fn set_current_read_channel(&mut self, channel: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_setCurrentReadChannel(
            self as *mut crate::QIODevice,
            channel,
        )
    }

    /// <p>Sets the current write channel of the <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> to the given <i>channel</i>. The current output channel is used by the functions <a href="http://doc.qt.io/qt-5/qiodevice.html#write">write</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#putChar">putChar</a>(). It also determines which channel triggers <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> to emit <a href="http://doc.qt.io/qt-5/qiodevice.html#bytesWritten">bytesWritten</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QIODevice::setCurrentWriteChannel(int channel)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#setCurrentWriteChannel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the current write channel of the <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> to the given <i>channel</i>. The current output channel is used by the functions <a href="http://doc.qt.io/qt-5/qiodevice.html#write">write</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#putChar">putChar</a>(). It also determines which channel triggers <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> to emit <a href="http://doc.qt.io/qt-5/qiodevice.html#bytesWritten">bytesWritten</a>().</p>
    /// <p>This function was introduced in  Qt 5.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#currentWriteChannel">currentWriteChannel</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#writeChannelCount">writeChannelCount</a>().</p></div>
    pub unsafe fn set_current_write_channel(&mut self, channel: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_setCurrentWriteChannel(
            self as *mut crate::QIODevice,
            channel,
        )
    }

    /// <p>If <i>enabled</i> is true, this function sets the <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">Text</a> flag on the device; otherwise the <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">Text</a> flag is removed. This feature is useful for classes that provide custom end-of-line handling on a <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QIODevice::setTextModeEnabled(bool enabled)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#setTextModeEnabled">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If <i>enabled</i> is true, this function sets the <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">Text</a> flag on the device; otherwise the <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">Text</a> flag is removed. This feature is useful for classes that provide custom end-of-line handling on a <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>.</p>
    /// <p>The IO device should be opened before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#isTextModeEnabled">isTextModeEnabled</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#open">open</a>(), and <a href="http://doc.qt.io/qt-5/qiodevice.html#setOpenMode">setOpenMode</a>().</p></div>
    pub unsafe fn set_text_mode_enabled(&mut self, enabled: bool) {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_setTextModeEnabled(
            self as *mut crate::QIODevice,
            enabled,
        )
    }

    /// <p>For open random-access devices, this function returns the size of the device. For open sequential devices, <a href="http://doc.qt.io/qt-5/qiodevice.html#bytesAvailable">bytesAvailable</a>() is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual qint64 QIODevice::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>For open random-access devices, this function returns the size of the device. For open sequential devices, <a href="http://doc.qt.io/qt-5/qiodevice.html#bytesAvailable">bytesAvailable</a>() is returned.</p>
    /// <p>If the device is closed, the size returned will not reflect the actual size of the device.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#isSequential">isSequential</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#pos">pos</a>().</p></div>
    pub unsafe fn size(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_size(self as *const crate::QIODevice)
    }

    /// <p>Skips up to <i>maxSize</i> bytes from the device. Returns the number of bytes actually skipped, or -1 on error.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QIODevice::skip(qint64 maxSize)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#skip">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Skips up to <i>maxSize</i> bytes from the device. Returns the number of bytes actually skipped, or -1 on error.</p>
    /// <p>This function does not wait and only discards the data that is already available for reading.</p>
    /// <p>If the device is opened in text mode, end-of-line terminators are translated to '\n' symbols and count as a single byte identically to the <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#peek">peek</a>() behavior.</p>
    /// <p>This function works for all devices, including sequential ones that cannot <a href="http://doc.qt.io/qt-5/qiodevice.html#seek">seek</a>(). It is optimized to skip unwanted data after a <a href="http://doc.qt.io/qt-5/qiodevice.html#peek">peek</a>() call.</p>
    /// <p>For random-access devices, skip() can be used to seek forward from the current position. Negative <i>maxSize</i> values are not allowed.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#peek">peek</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#seek">seek</a>(), and <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn skip(&mut self, max_size: i64) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_skip(self as *mut crate::QIODevice, max_size)
    }

    /// <p>Starts a new read transaction on the device.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QIODevice::startTransaction()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#startTransaction">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Starts a new read transaction on the device.</p>
    /// <p>Defines a restorable point within the sequence of read operations. For sequential devices, read data will be duplicated internally to allow recovery in case of incomplete reads. For random-access devices, this function saves the current position. Call <a href="http://doc.qt.io/qt-5/qiodevice.html#commitTransaction">commitTransaction</a>() or <a href="http://doc.qt.io/qt-5/qiodevice.html#rollbackTransaction">rollbackTransaction</a>() to finish the transaction.</p>
    /// <p><b>Note: </b>Nesting transactions is not supported.</p><p>This function was introduced in  Qt 5.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#commitTransaction">commitTransaction</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#rollbackTransaction">rollbackTransaction</a>().</p></div>
    pub unsafe fn start_transaction(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_startTransaction(self as *mut crate::QIODevice)
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QIODevice_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QIODevice::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QIODevice_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QIODevice::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QIODevice_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Puts the character <i>c</i> back into the device, and decrements the current position unless the position is 0. This function is usually called to "undo" a <a href="http://doc.qt.io/qt-5/qiodevice.html#getChar">getChar</a>() operation, such as when writing a backtracking parser.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QIODevice::ungetChar(char c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#ungetChar">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Puts the character <i>c</i> back into the device, and decrements the current position unless the position is 0. This function is usually called to "undo" a <a href="http://doc.qt.io/qt-5/qiodevice.html#getChar">getChar</a>() operation, such as when writing a backtracking parser.</p>
    /// <p>If <i>c</i> was not previously read from the device, the behavior is undefined.</p>
    /// <p><b>Note: </b>This function is not available while a transaction is in progress.</p></div>
    pub unsafe fn unget_char(&mut self, c: ::std::os::raw::c_char) {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_ungetChar(self as *mut crate::QIODevice, c)
    }

    /// <p>For buffered devices, this function waits until a payload of buffered written data has been written to the device and the <a href="http://doc.qt.io/qt-5/qiodevice.html#bytesWritten">bytesWritten</a>() signal has been emitted, or until <i>msecs</i> milliseconds have passed. If msecs is -1, this function will not time out. For unbuffered devices, it returns immediately.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QIODevice::waitForBytesWritten(int msecs)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#waitForBytesWritten">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>For buffered devices, this function waits until a payload of buffered written data has been written to the device and the <a href="http://doc.qt.io/qt-5/qiodevice.html#bytesWritten">bytesWritten</a>() signal has been emitted, or until <i>msecs</i> milliseconds have passed. If msecs is -1, this function will not time out. For unbuffered devices, it returns immediately.</p>
    /// <p>Returns <code>true</code> if a payload of data was written to the device; otherwise returns <code>false</code> (i.e. if the operation timed out, or if an error occurred).</p>
    /// <p>This function can operate without an event loop. It is useful when writing non-GUI applications and when performing I/O operations in a non-GUI thread.</p>
    /// <p>If called from within a slot connected to the <a href="http://doc.qt.io/qt-5/qiodevice.html#bytesWritten">bytesWritten</a>() signal, <a href="http://doc.qt.io/qt-5/qiodevice.html#bytesWritten">bytesWritten</a>() will not be reemitted.</p>
    /// <p>Reimplement this function to provide a blocking API for a custom device. The default implementation does nothing, and returns <code>false</code>.</p>
    /// <p><b>Warning:</b> Calling this function from the main (GUI) thread might cause your user interface to freeze.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#waitForReadyRead">waitForReadyRead</a>().</p></div>
    pub unsafe fn wait_for_bytes_written(&mut self, msecs: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_waitForBytesWritten(
            self as *mut crate::QIODevice,
            msecs,
        )
    }

    /// <p>Blocks until new data is available for reading and the <a href="http://doc.qt.io/qt-5/qiodevice.html#readyRead">readyRead</a>() signal has been emitted, or until <i>msecs</i> milliseconds have passed. If msecs is -1, this function will not time out.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QIODevice::waitForReadyRead(int msecs)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#waitForReadyRead">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Blocks until new data is available for reading and the <a href="http://doc.qt.io/qt-5/qiodevice.html#readyRead">readyRead</a>() signal has been emitted, or until <i>msecs</i> milliseconds have passed. If msecs is -1, this function will not time out.</p>
    /// <p>Returns <code>true</code> if new data is available for reading; otherwise returns false (if the operation timed out or if an error occurred).</p>
    /// <p>This function can operate without an event loop. It is useful when writing non-GUI applications and when performing I/O operations in a non-GUI thread.</p>
    /// <p>If called from within a slot connected to the <a href="http://doc.qt.io/qt-5/qiodevice.html#readyRead">readyRead</a>() signal, <a href="http://doc.qt.io/qt-5/qiodevice.html#readyRead">readyRead</a>() will not be reemitted.</p>
    /// <p>Reimplement this function to provide a blocking API for a custom device. The default implementation does nothing, and returns <code>false</code>.</p>
    /// <p><b>Warning:</b> Calling this function from the main (GUI) thread might cause your user interface to freeze.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#waitForBytesWritten">waitForBytesWritten</a>().</p></div>
    pub unsafe fn wait_for_ready_read(&mut self, msecs: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_waitForReadyRead(
            self as *mut crate::QIODevice,
            msecs,
        )
    }

    /// <p>Writes at most <i>maxSize</i> bytes of data from <i>data</i> to the device. Returns the number of bytes that were actually written, or -1 if an error occurred.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QIODevice::write(const char* data, qint64 len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#write">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes at most <i>maxSize</i> bytes of data from <i>data</i> to the device. Returns the number of bytes that were actually written, or -1 if an error occurred.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#writeData">writeData</a>().</p></div>
    pub unsafe fn write_char_i64(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        len: i64,
    ) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_write(
            self as *mut crate::QIODevice,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(data)
                .as_raw_ptr(),
            len,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QIODevice::write(const char* data)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#write-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes data from a zero-terminated string of 8-bit characters to the device. Returns the number of bytes that were actually written, or -1 if an error occurred. This is equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qiodevice.html#QIODevice">QIODevice</a></span><span class="operator">::</span>write(data<span class="operator">,</span> qstrlen(data));
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#writeData">writeData</a>().</p></div>
    pub unsafe fn write_char(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_write1(
            self as *mut crate::QIODevice,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(data)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QIODevice::write(const QByteArray& data)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#write-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes the content of <i>byteArray</i> to the device. Returns the number of bytes that were actually written, or -1 if an error occurred.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#writeData">writeData</a>().</p></div>
    pub unsafe fn write_q_byte_array(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_write2(
            self as *mut crate::QIODevice,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(data)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the number of available write channels if the device is open; otherwise returns 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QIODevice::writeChannelCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#writeChannelCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of available write channels if the device is open; otherwise returns 0.</p>
    /// <p>This function was introduced in  Qt 5.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#readChannelCount">readChannelCount</a>().</p></div>
    pub unsafe fn write_channel_count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_writeChannelCount(self as *const crate::QIODevice)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> class provides a <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> interface for a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
///
/// C++ class: <span style='color: green;'>```QBuffer```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qbuffer.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> class provides a <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> interface for a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
/// <p><a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> allows you to access a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> using the <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> interface. The <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> is treated just as a standard random-accessed file. Example:</p>
/// <pre class="cpp">
///
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qbuffer.html#QBuffer">QBuffer</a></span> buffer;
/// &#32;     <span class="type">char</span> ch;
///
/// &#32;     buffer<span class="operator">.</span>open(<span class="type"><a href="http://doc.qt.io/qt-5/qbuffer.html#QBuffer">QBuffer</a></span><span class="operator">::</span>ReadWrite);
/// &#32;     buffer<span class="operator">.</span>write(<span class="string">"Qt rocks!"</span>);
/// &#32;     buffer<span class="operator">.</span>seek(<span class="number">0</span>);
/// &#32;     buffer<span class="operator">.</span>getChar(<span class="operator">&amp;</span>ch);  <span class="comment">// ch == 'Q'</span>
/// &#32;     buffer<span class="operator">.</span>getChar(<span class="operator">&amp;</span>ch);  <span class="comment">// ch == 't'</span>
/// &#32;     buffer<span class="operator">.</span>getChar(<span class="operator">&amp;</span>ch);  <span class="comment">// ch == ' '</span>
/// &#32;     buffer<span class="operator">.</span>getChar(<span class="operator">&amp;</span>ch);  <span class="comment">// ch == 'r'</span>
///
/// </pre>
/// <p>By default, an internal <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> buffer is created for you when you create a <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a>. You can access this buffer directly by calling <a href="http://doc.qt.io/qt-5/qbuffer.html#buffer">buffer</a>(). You can also use <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> with an existing <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> by calling <a href="http://doc.qt.io/qt-5/qbuffer.html#setBuffer">setBuffer</a>(), or by passing your array to <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a>'s constructor.</p>
/// <p>Call <a href="http://doc.qt.io/qt-5/qbuffer.html#open">open</a>() to open the buffer. Then call <a href="http://doc.qt.io/qt-5/qiodevice.html#write">write</a>() or <a href="http://doc.qt.io/qt-5/qiodevice.html#putChar">putChar</a>() to write to the buffer, and <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#readLine">readLine</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#readAll">readAll</a>(), or <a href="http://doc.qt.io/qt-5/qiodevice.html#getChar">getChar</a>() to read from it. <a href="http://doc.qt.io/qt-5/qbuffer.html#size">size</a>() returns the current size of the buffer, and you can seek to arbitrary positions in the buffer by calling <a href="http://doc.qt.io/qt-5/qbuffer.html#seek">seek</a>(). When you are done with accessing the buffer, call <a href="http://doc.qt.io/qt-5/qbuffer.html#close">close</a>().</p>
/// <p>The following code snippet shows how to write data to a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> using <a href="http://doc.qt.io/qt-5/qdatastream.html">QDataStream</a> and <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a>:</p>
/// <pre class="cpp">
///
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span> byteArray;
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qbuffer.html#QBuffer">QBuffer</a></span> buffer(<span class="operator">&amp;</span>byteArray);
/// &#32;     buffer<span class="operator">.</span>open(<span class="type"><a href="http://doc.qt.io/qt-5/qiodevice.html#QIODevice">QIODevice</a></span><span class="operator">::</span>WriteOnly);
///
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qdatastream.html">QDataStream</a></span> out(<span class="operator">&amp;</span>buffer);
/// &#32;     out <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qapplication.html">QApplication</a></span><span class="operator">::</span>palette();
///
/// </pre>
/// <p>Effectively, we convert the application's <a href="http://doc.qt.io/qt-5/qpalette.html">QPalette</a> into a byte array. Here's how to read the data from the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>:</p>
/// <pre class="cpp">
///
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qpalette.html">QPalette</a></span> palette;
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qbuffer.html#QBuffer">QBuffer</a></span> buffer(<span class="operator">&amp;</span>byteArray);
/// &#32;     buffer<span class="operator">.</span>open(<span class="type"><a href="http://doc.qt.io/qt-5/qiodevice.html#QIODevice">QIODevice</a></span><span class="operator">::</span>ReadOnly);
///
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qdatastream.html">QDataStream</a></span> in(<span class="operator">&amp;</span>buffer);
/// &#32;     in <span class="operator">&gt;</span><span class="operator">&gt;</span> palette;
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> and <a href="http://doc.qt.io/qt-5/qdatastream.html">QDataStream</a> also provide convenience constructors that take a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> and that create a <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> behind the scenes.</p>
/// <p><a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> emits <a href="http://doc.qt.io/qt-5/qiodevice.html#readyRead">readyRead</a>() when new data has arrived in the buffer. By connecting to this signal, you can use <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> to store temporary data before processing it. <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> also emits <a href="http://doc.qt.io/qt-5/qiodevice.html#bytesWritten">bytesWritten</a>() every time new data has been written to the buffer.</p></div>
#[repr(C)]
pub struct QBuffer {
    _unused: u8,
}
impl QBuffer {
    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#atEnd">QIODevice::atEnd</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QBuffer::atEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbuffer.html#atEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#atEnd">QIODevice::atEnd</a>().</p></div>
    pub unsafe fn at_end(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QBuffer_atEnd(self as *const crate::QBuffer)
    }

    /// <p>Returns a reference to the <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a>'s internal buffer. You can use it to modify the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> behind the <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a>'s back.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QBuffer::buffer()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbuffer.html#buffer">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a>'s internal buffer. You can use it to modify the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> behind the <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a>'s back.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbuffer.html#setBuffer">setBuffer</a>() and <a href="http://doc.qt.io/qt-5/qbuffer.html#data">data</a>().</p></div>
    pub unsafe fn buffer_mut(&mut self) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QBuffer_buffer(self as *mut crate::QBuffer);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QByteArray& QBuffer::buffer() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbuffer.html#buffer-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This is the same as <a href="http://doc.qt.io/qt-5/qbuffer.html#data">data</a>().</p></div>
    pub unsafe fn buffer(&self) -> ::cpp_utils::Ref<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QBuffer_buffer1(self as *const crate::QBuffer);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#canReadLine">QIODevice::canReadLine</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QBuffer::canReadLine() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbuffer.html#canReadLine">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#canReadLine">QIODevice::canReadLine</a>().</p></div>
    pub unsafe fn can_read_line(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QBuffer_canReadLine(self as *const crate::QBuffer)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#close">QIODevice::close</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual void QBuffer::close()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbuffer.html#close">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#close">QIODevice::close</a>().</p></div>
    pub unsafe fn close(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QBuffer_close(self as *mut crate::QBuffer)
    }

    /// <p>Returns the data contained in the buffer.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QByteArray& QBuffer::data() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbuffer.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the data contained in the buffer.</p>
    /// <p>This is the same as <a href="http://doc.qt.io/qt-5/qbuffer.html#buffer">buffer</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbuffer.html#setData">setData</a>() and <a href="http://doc.qt.io/qt-5/qbuffer.html#setBuffer">setBuffer</a>().</p></div>
    pub unsafe fn data(&self) -> ::cpp_utils::Ref<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QBuffer_data(self as *const crate::QBuffer);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QBuffer::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QBuffer_metaObject(self as *const crate::QBuffer);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs an empty buffer with the given <i>parent</i>. You can call <a href="http://doc.qt.io/qt-5/qbuffer.html#setData">setData</a>() to fill the buffer with data, or you can open it in write mode and use <a href="http://doc.qt.io/qt-5/qiodevice.html#write">write</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QBuffer::QBuffer(QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbuffer.html#QBuffer">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty buffer with the given <i>parent</i>. You can call <a href="http://doc.qt.io/qt-5/qbuffer.html#setData">setData</a>() to fill the buffer with data, or you can open it in write mode and use <a href="http://doc.qt.io/qt-5/qiodevice.html#write">write</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbuffer.html#open">open</a>().</p></div>
    pub unsafe fn from_q_object(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QBuffer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QBuffer_QBuffer(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> that uses the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> pointed to by <i>byteArray</i> as its internal buffer, and with the given <i>parent</i>. The caller is responsible for ensuring that <i>byteArray</i> remains valid until the <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> is destroyed, or until <a href="http://doc.qt.io/qt-5/qbuffer.html#setBuffer">setBuffer</a>() is called to change the buffer. <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> doesn't take ownership of the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QBuffer::QBuffer(QByteArray* buf, QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbuffer.html#QBuffer-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> that uses the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> pointed to by <i>byteArray</i> as its internal buffer, and with the given <i>parent</i>. The caller is responsible for ensuring that <i>byteArray</i> remains valid until the <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> is destroyed, or until <a href="http://doc.qt.io/qt-5/qbuffer.html#setBuffer">setBuffer</a>() is called to change the buffer. <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> doesn't take ownership of the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    /// <p>If you open the buffer in write-only mode or read-write mode and write something into the <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a>, <i>byteArray</i> will be modified.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span> byteArray(<span class="string">"abc"</span>);
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qbuffer.html#QBuffer">QBuffer</a></span> buffer(<span class="operator">&amp;</span>byteArray);
    /// &#32;     buffer<span class="operator">.</span>open(<span class="type"><a href="http://doc.qt.io/qt-5/qiodevice.html#QIODevice">QIODevice</a></span><span class="operator">::</span>WriteOnly);
    /// &#32;     buffer<span class="operator">.</span>seek(<span class="number">3</span>);
    /// &#32;     buffer<span class="operator">.</span>write(<span class="string">"def"</span><span class="operator">,</span> <span class="number">3</span>);
    /// &#32;     buffer<span class="operator">.</span>close();
    /// &#32;     <span class="comment">// byteArray == "abcdef"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbuffer.html#open">open</a>(), <a href="http://doc.qt.io/qt-5/qbuffer.html#setBuffer">setBuffer</a>(), and <a href="http://doc.qt.io/qt-5/qbuffer.html#setData">setData</a>().</p></div>
    pub unsafe fn from_q_byte_array_q_object(
        buf: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QByteArray>>,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QBuffer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QBuffer_QBuffer1(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QByteArray>>::cast_into(buf)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> class provides a <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> interface for a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QBuffer::QBuffer()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbuffer.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> class provides a <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> interface for a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> allows you to access a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> using the <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> interface. The <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> is treated just as a standard random-accessed file. Example:</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qbuffer.html#QBuffer">QBuffer</a></span> buffer;
    /// &#32;     <span class="type">char</span> ch;
    ///
    /// &#32;     buffer<span class="operator">.</span>open(<span class="type"><a href="http://doc.qt.io/qt-5/qbuffer.html#QBuffer">QBuffer</a></span><span class="operator">::</span>ReadWrite);
    /// &#32;     buffer<span class="operator">.</span>write(<span class="string">"Qt rocks!"</span>);
    /// &#32;     buffer<span class="operator">.</span>seek(<span class="number">0</span>);
    /// &#32;     buffer<span class="operator">.</span>getChar(<span class="operator">&amp;</span>ch);  <span class="comment">// ch == 'Q'</span>
    /// &#32;     buffer<span class="operator">.</span>getChar(<span class="operator">&amp;</span>ch);  <span class="comment">// ch == 't'</span>
    /// &#32;     buffer<span class="operator">.</span>getChar(<span class="operator">&amp;</span>ch);  <span class="comment">// ch == ' '</span>
    /// &#32;     buffer<span class="operator">.</span>getChar(<span class="operator">&amp;</span>ch);  <span class="comment">// ch == 'r'</span>
    ///
    /// </pre>
    /// <p>By default, an internal <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> buffer is created for you when you create a <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a>. You can access this buffer directly by calling <a href="http://doc.qt.io/qt-5/qbuffer.html#buffer">buffer</a>(). You can also use <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> with an existing <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> by calling <a href="http://doc.qt.io/qt-5/qbuffer.html#setBuffer">setBuffer</a>(), or by passing your array to <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a>'s constructor.</p>
    /// <p>Call <a href="http://doc.qt.io/qt-5/qbuffer.html#open">open</a>() to open the buffer. Then call <a href="http://doc.qt.io/qt-5/qiodevice.html#write">write</a>() or <a href="http://doc.qt.io/qt-5/qiodevice.html#putChar">putChar</a>() to write to the buffer, and <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#readLine">readLine</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#readAll">readAll</a>(), or <a href="http://doc.qt.io/qt-5/qiodevice.html#getChar">getChar</a>() to read from it. <a href="http://doc.qt.io/qt-5/qbuffer.html#size">size</a>() returns the current size of the buffer, and you can seek to arbitrary positions in the buffer by calling <a href="http://doc.qt.io/qt-5/qbuffer.html#seek">seek</a>(). When you are done with accessing the buffer, call <a href="http://doc.qt.io/qt-5/qbuffer.html#close">close</a>().</p>
    /// <p>The following code snippet shows how to write data to a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> using <a href="http://doc.qt.io/qt-5/qdatastream.html">QDataStream</a> and <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a>:</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span> byteArray;
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qbuffer.html#QBuffer">QBuffer</a></span> buffer(<span class="operator">&amp;</span>byteArray);
    /// &#32;     buffer<span class="operator">.</span>open(<span class="type"><a href="http://doc.qt.io/qt-5/qiodevice.html#QIODevice">QIODevice</a></span><span class="operator">::</span>WriteOnly);
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qdatastream.html">QDataStream</a></span> out(<span class="operator">&amp;</span>buffer);
    /// &#32;     out <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qapplication.html">QApplication</a></span><span class="operator">::</span>palette();
    ///
    /// </pre>
    /// <p>Effectively, we convert the application's <a href="http://doc.qt.io/qt-5/qpalette.html">QPalette</a> into a byte array. Here's how to read the data from the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>:</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qpalette.html">QPalette</a></span> palette;
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qbuffer.html#QBuffer">QBuffer</a></span> buffer(<span class="operator">&amp;</span>byteArray);
    /// &#32;     buffer<span class="operator">.</span>open(<span class="type"><a href="http://doc.qt.io/qt-5/qiodevice.html#QIODevice">QIODevice</a></span><span class="operator">::</span>ReadOnly);
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qdatastream.html">QDataStream</a></span> in(<span class="operator">&amp;</span>buffer);
    /// &#32;     in <span class="operator">&gt;</span><span class="operator">&gt;</span> palette;
    ///
    /// </pre>
    /// <p><a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> and <a href="http://doc.qt.io/qt-5/qdatastream.html">QDataStream</a> also provide convenience constructors that take a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> and that create a <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> behind the scenes.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> emits <a href="http://doc.qt.io/qt-5/qiodevice.html#readyRead">readyRead</a>() when new data has arrived in the buffer. By connecting to this signal, you can use <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> to store temporary data before processing it. <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> also emits <a href="http://doc.qt.io/qt-5/qiodevice.html#bytesWritten">bytesWritten</a>() every time new data has been written to the buffer.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QBuffer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QBuffer_QBuffer2();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> that uses the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> pointed to by <i>byteArray</i> as its internal buffer, and with the given <i>parent</i>. The caller is responsible for ensuring that <i>byteArray</i> remains valid until the <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> is destroyed, or until <a href="http://doc.qt.io/qt-5/qbuffer.html#setBuffer">setBuffer</a>() is called to change the buffer. <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> doesn't take ownership of the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QBuffer::QBuffer(QByteArray* buf)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbuffer.html#QBuffer-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> that uses the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> pointed to by <i>byteArray</i> as its internal buffer, and with the given <i>parent</i>. The caller is responsible for ensuring that <i>byteArray</i> remains valid until the <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> is destroyed, or until <a href="http://doc.qt.io/qt-5/qbuffer.html#setBuffer">setBuffer</a>() is called to change the buffer. <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> doesn't take ownership of the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    /// <p>If you open the buffer in write-only mode or read-write mode and write something into the <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a>, <i>byteArray</i> will be modified.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span> byteArray(<span class="string">"abc"</span>);
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qbuffer.html#QBuffer">QBuffer</a></span> buffer(<span class="operator">&amp;</span>byteArray);
    /// &#32;     buffer<span class="operator">.</span>open(<span class="type"><a href="http://doc.qt.io/qt-5/qiodevice.html#QIODevice">QIODevice</a></span><span class="operator">::</span>WriteOnly);
    /// &#32;     buffer<span class="operator">.</span>seek(<span class="number">3</span>);
    /// &#32;     buffer<span class="operator">.</span>write(<span class="string">"def"</span><span class="operator">,</span> <span class="number">3</span>);
    /// &#32;     buffer<span class="operator">.</span>close();
    /// &#32;     <span class="comment">// byteArray == "abcdef"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbuffer.html#open">open</a>(), <a href="http://doc.qt.io/qt-5/qbuffer.html#setBuffer">setBuffer</a>(), and <a href="http://doc.qt.io/qt-5/qbuffer.html#setData">setData</a>().</p></div>
    pub unsafe fn from_q_byte_array(
        buf: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QBuffer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QBuffer_QBuffer3(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QByteArray>>::cast_into(buf)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#open">QIODevice::open</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QBuffer::open(QFlags<QIODevice::OpenModeFlag> openMode)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbuffer.html#open">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#open">QIODevice::open</a>().</p></div>
    pub unsafe fn open(
        &mut self,
        open_mode: crate::QFlags<crate::q_io_device::OpenModeFlag>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QBuffer_open(self as *mut crate::QBuffer, open_mode.to_int())
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#pos">QIODevice::pos</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual qint64 QBuffer::pos() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbuffer.html#pos">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#pos">QIODevice::pos</a>().</p></div>
    pub unsafe fn pos(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QBuffer_pos(self as *const crate::QBuffer)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QBuffer::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QBuffer_qt_metacall(
            self as *mut crate::QBuffer,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QBuffer::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QBuffer_qt_metacast(
            self as *mut crate::QBuffer,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#seek">QIODevice::seek</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QBuffer::seek(qint64 off)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbuffer.html#seek">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#seek">QIODevice::seek</a>().</p></div>
    pub unsafe fn seek(&mut self, off: i64) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QBuffer_seek(self as *mut crate::QBuffer, off)
    }

    /// <p>Makes <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> uses the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> pointed to by <i>byteArray</i> as its internal buffer. The caller is responsible for ensuring that <i>byteArray</i> remains valid until the <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> is destroyed, or until setBuffer() is called to change the buffer. <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> doesn't take ownership of the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QBuffer::setBuffer(QByteArray* a)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbuffer.html#setBuffer">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> uses the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> pointed to by <i>byteArray</i> as its internal buffer. The caller is responsible for ensuring that <i>byteArray</i> remains valid until the <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> is destroyed, or until setBuffer() is called to change the buffer. <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> doesn't take ownership of the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    /// <p>Does nothing if <a href="http://doc.qt.io/qt-5/qiodevice.html#isOpen">isOpen</a>() is true.</p>
    /// <p>If you open the buffer in write-only mode or read-write mode and write something into the <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a>, <i>byteArray</i> will be modified.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span> byteArray(<span class="string">"abc"</span>);
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qbuffer.html#QBuffer">QBuffer</a></span> buffer;
    /// &#32;     buffer<span class="operator">.</span>setBuffer(<span class="operator">&amp;</span>byteArray);
    /// &#32;     buffer<span class="operator">.</span>open(<span class="type"><a href="http://doc.qt.io/qt-5/qiodevice.html#QIODevice">QIODevice</a></span><span class="operator">::</span>WriteOnly);
    /// &#32;     buffer<span class="operator">.</span>seek(<span class="number">3</span>);
    /// &#32;     buffer<span class="operator">.</span>write(<span class="string">"def"</span><span class="operator">,</span> <span class="number">3</span>);
    /// &#32;     buffer<span class="operator">.</span>close();
    /// &#32;     <span class="comment">// byteArray == "abcdef"</span>
    ///
    /// </pre>
    /// <p>If <i>byteArray</i> is 0, the buffer creates its own internal <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> to work on. This byte array is initially empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbuffer.html#buffer">buffer</a>(), <a href="http://doc.qt.io/qt-5/qbuffer.html#setData">setData</a>(), and <a href="http://doc.qt.io/qt-5/qbuffer.html#open">open</a>().</p></div>
    pub unsafe fn set_buffer(
        &mut self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QByteArray>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QBuffer_setBuffer(
            self as *mut crate::QBuffer,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QByteArray>>::cast_into(a)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Sets the contents of the internal buffer to be <i>data</i>. This is the same as assigning <i>data</i> to <a href="http://doc.qt.io/qt-5/qbuffer.html#buffer">buffer</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QBuffer::setData(const QByteArray& data)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbuffer.html#setData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the contents of the internal buffer to be <i>data</i>. This is the same as assigning <i>data</i> to <a href="http://doc.qt.io/qt-5/qbuffer.html#buffer">buffer</a>().</p>
    /// <p>Does nothing if <a href="http://doc.qt.io/qt-5/qiodevice.html#isOpen">isOpen</a>() is true.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbuffer.html#data">data</a>() and <a href="http://doc.qt.io/qt-5/qbuffer.html#setBuffer">setBuffer</a>().</p></div>
    pub unsafe fn set_data_1a(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QBuffer_setData(
            self as *mut crate::QBuffer,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(data)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QBuffer::setData(const char* data, int len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbuffer.html#setData-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Sets the contents of the internal buffer to be the first <i>size</i> bytes of <i>data</i>.</p></div>
    pub unsafe fn set_data_2a(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        len: ::std::os::raw::c_int,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QBuffer_setData1(
            self as *mut crate::QBuffer,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(data)
                .as_raw_ptr(),
            len,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#size">QIODevice::size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual qint64 QBuffer::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbuffer.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#size">QIODevice::size</a>().</p></div>
    pub unsafe fn size(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QBuffer_size(self as *const crate::QBuffer)
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QBuffer_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QBuffer::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QBuffer_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QBuffer::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QBuffer_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html">QByteArrayMatcher</a> class holds a sequence of bytes that can be quickly matched in a byte array.</p>
///
/// C++ class: <span style='color: green;'>```QByteArrayMatcher```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html">QByteArrayMatcher</a> class holds a sequence of bytes that can be quickly matched in a byte array.</p>
/// <p>This class is useful when you have a sequence of bytes that you want to repeatedly match against some byte arrays (perhaps in a loop), or when you want to search for the same sequence of bytes multiple times in the same byte array. Using a matcher object and <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#indexIn">indexIn</a>() is faster than matching a plain <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> with <a href="http://doc.qt.io/qt-5/qbytearray.html#indexOf">QByteArray::indexOf</a>() if repeated matching takes place. This class offers no benefit if you are doing one-off byte array matches.</p>
/// <p>Create the <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html">QByteArrayMatcher</a> with the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> you want to search for. Then call <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#indexIn">indexIn</a>() on the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> that you want to search.</p></div>
#[repr(C)]
pub struct QByteArrayMatcher {
    _unused: u8,
}
impl QByteArrayMatcher {
    /// <p>Assigns the <i>other</i> byte array matcher to this byte array matcher.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArrayMatcher& QByteArrayMatcher::operator=(const QByteArrayMatcher& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns the <i>other</i> byte array matcher to this byte array matcher.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArrayMatcher>>,
    ) -> ::cpp_utils::MutRef<crate::QByteArrayMatcher> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArrayMatcher_operator_(
            self as *mut crate::QByteArrayMatcher,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArrayMatcher>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Searches the byte array <i>ba</i>, from byte position <i>from</i> (default 0, i.e. from the first byte), for the byte array <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#pattern">pattern</a>() that was set in the constructor or in the most recent call to <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#setPattern">setPattern</a>(). Returns the position where the <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#pattern">pattern</a>() matched in <i>ba</i>, or -1 if no match was found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArrayMatcher::indexIn(const QByteArray& ba, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#indexIn">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Searches the byte array <i>ba</i>, from byte position <i>from</i> (default 0, i.e. from the first byte), for the byte array <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#pattern">pattern</a>() that was set in the constructor or in the most recent call to <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#setPattern">setPattern</a>(). Returns the position where the <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#pattern">pattern</a>() matched in <i>ba</i>, or -1 if no match was found.</p></div>
    pub unsafe fn index_in_q_byte_array_int(
        &self,
        ba: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArrayMatcher_indexIn(
            self as *const crate::QByteArrayMatcher,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(ba)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Searches the char string <i>str</i>, which has length <i>len</i>, from byte position <i>from</i> (default 0, i.e. from the first byte), for the byte array <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#pattern">pattern</a>() that was set in the constructor or in the most recent call to <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#setPattern">setPattern</a>(). Returns the position where the <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#pattern">pattern</a>() matched in <i>str</i>, or -1 if no match was found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArrayMatcher::indexIn(const char* str, int len, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#indexIn-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Searches the char string <i>str</i>, which has length <i>len</i>, from byte position <i>from</i> (default 0, i.e. from the first byte), for the byte array <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#pattern">pattern</a>() that was set in the constructor or in the most recent call to <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#setPattern">setPattern</a>(). Returns the position where the <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#pattern">pattern</a>() matched in <i>str</i>, or -1 if no match was found.</p></div>
    pub unsafe fn index_in_char2_int(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        len: ::std::os::raw::c_int,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArrayMatcher_indexIn1(
            self as *const crate::QByteArrayMatcher,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(str)
                .as_raw_ptr(),
            len,
            from,
        )
    }

    /// <p>Searches the byte array <i>ba</i>, from byte position <i>from</i> (default 0, i.e. from the first byte), for the byte array <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#pattern">pattern</a>() that was set in the constructor or in the most recent call to <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#setPattern">setPattern</a>(). Returns the position where the <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#pattern">pattern</a>() matched in <i>ba</i>, or -1 if no match was found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArrayMatcher::indexIn(const QByteArray& ba) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#indexIn">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Searches the byte array <i>ba</i>, from byte position <i>from</i> (default 0, i.e. from the first byte), for the byte array <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#pattern">pattern</a>() that was set in the constructor or in the most recent call to <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#setPattern">setPattern</a>(). Returns the position where the <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#pattern">pattern</a>() matched in <i>ba</i>, or -1 if no match was found.</p></div>
    pub unsafe fn index_in_q_byte_array(
        &self,
        ba: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArrayMatcher_indexIn2(
            self as *const crate::QByteArrayMatcher,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(ba)
                .as_raw_ptr(),
        )
    }

    /// <p>Searches the char string <i>str</i>, which has length <i>len</i>, from byte position <i>from</i> (default 0, i.e. from the first byte), for the byte array <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#pattern">pattern</a>() that was set in the constructor or in the most recent call to <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#setPattern">setPattern</a>(). Returns the position where the <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#pattern">pattern</a>() matched in <i>str</i>, or -1 if no match was found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QByteArrayMatcher::indexIn(const char* str, int len) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#indexIn-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Searches the char string <i>str</i>, which has length <i>len</i>, from byte position <i>from</i> (default 0, i.e. from the first byte), for the byte array <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#pattern">pattern</a>() that was set in the constructor or in the most recent call to <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#setPattern">setPattern</a>(). Returns the position where the <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#pattern">pattern</a>() matched in <i>str</i>, or -1 if no match was found.</p></div>
    pub unsafe fn index_in_char_int(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        len: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QByteArrayMatcher_indexIn3(
            self as *const crate::QByteArrayMatcher,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(str)
                .as_raw_ptr(),
            len,
        )
    }

    /// <p>Constructs an empty byte array matcher that won't match anything. Call <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#setPattern">setPattern</a>() to give it a pattern to match.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QByteArrayMatcher::QByteArrayMatcher()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#QByteArrayMatcher">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty byte array matcher that won't match anything. Call <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#setPattern">setPattern</a>() to give it a pattern to match.</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QByteArrayMatcher> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArrayMatcher_QByteArrayMatcher();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a byte array matcher that will search for <i>pattern</i>. Call <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#indexIn">indexIn</a>() to perform a search.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QByteArrayMatcher::QByteArrayMatcher(const QByteArray& pattern)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#QByteArrayMatcher-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a byte array matcher that will search for <i>pattern</i>. Call <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#indexIn">indexIn</a>() to perform a search.</p></div>
    pub unsafe fn new_1a(
        pattern: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArrayMatcher> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArrayMatcher_QByteArrayMatcher1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(pattern)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a byte array matcher from <i>pattern</i>. <i>pattern</i> has the given <i>length</i>. <i>pattern</i> must remain in scope, but the destructor does not delete <i>pattern</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QByteArrayMatcher::QByteArrayMatcher(const char* pattern, int length)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#QByteArrayMatcher-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a byte array matcher from <i>pattern</i>. <i>pattern</i> has the given <i>length</i>. <i>pattern</i> must remain in scope, but the destructor does not delete <i>pattern</i>.</p></div>
    pub unsafe fn new_2a(
        pattern: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        length: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QByteArrayMatcher> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArrayMatcher_QByteArrayMatcher2(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(pattern)
                .as_raw_ptr(),
            length,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Copies the <i>other</i> byte array matcher to this byte array matcher.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QByteArrayMatcher::QByteArrayMatcher(const QByteArrayMatcher& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#QByteArrayMatcher-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Copies the <i>other</i> byte array matcher to this byte array matcher.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArrayMatcher>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArrayMatcher> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArrayMatcher_QByteArrayMatcher3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArrayMatcher>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the byte array pattern that this byte array matcher will search for.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QByteArrayMatcher::pattern() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#pattern">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array pattern that this byte array matcher will search for.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#setPattern">setPattern</a>().</p></div>
    pub unsafe fn pattern(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QByteArrayMatcher_pattern(
            self as *const crate::QByteArrayMatcher,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Sets the byte array that this byte array matcher will search for to <i>pattern</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QByteArrayMatcher::setPattern(const QByteArray& pattern)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#setPattern">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the byte array that this byte array matcher will search for to <i>pattern</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#pattern">pattern</a>() and <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#indexIn">indexIn</a>().</p></div>
    pub unsafe fn set_pattern(
        &mut self,
        pattern: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QByteArrayMatcher_setPattern(
            self as *mut crate::QByteArrayMatcher,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(pattern)
                .as_raw_ptr(),
        )
    }
}

/// C++ class: <span style='color: green;'>```QStaticByteArrayMatcherBase```</span>.
#[repr(C)]
pub struct QStaticByteArrayMatcherBase {
    _unused: u8,
}
impl QStaticByteArrayMatcherBase {
    /// Calls C++ function: <span style='color: green;'>```QStaticByteArrayMatcherBase& QStaticByteArrayMatcherBase::operator=(const QStaticByteArrayMatcherBase& other)```</span>.
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStaticByteArrayMatcherBase>>,
    ) -> ::cpp_utils::MutRef<crate::QStaticByteArrayMatcherBase> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStaticByteArrayMatcherBase_operator_(self as *mut crate::QStaticByteArrayMatcherBase, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStaticByteArrayMatcherBase>>::cast_into(other).as_raw_ptr());
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStaticByteArrayMatcherBase::QStaticByteArrayMatcherBase(const QStaticByteArrayMatcherBase& other)```</span>.
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStaticByteArrayMatcherBase>>,
    ) -> ::cpp_utils::CppBox<crate::QStaticByteArrayMatcherBase> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStaticByteArrayMatcherBase_QStaticByteArrayMatcherBase1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStaticByteArrayMatcherBase>>::cast_into(other).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_locale {
    //! C++ type: <span style='color: green;'>```QLocale```</span>

    /// <p>This enumerated type is used to specify a language.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QLocale::Language```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#Language-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enumerated type is used to specify a language.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#language">language</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#languageToString">languageToString</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Language(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Language {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Language(value)
        }
    }

    impl From<Language> for ::std::os::raw::c_int {
        fn from(value: Language) -> Self {
            value.0
        }
    }

    impl Language {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Language {
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```AnyLanguage = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const AnyLanguage: crate::q_locale::Language = crate::q_locale::Language(0);
        /// The "C" locale is identical in behavior to English/<a href="http://doc.qt.io/qt-5/qlocale.html#Country-enum">UnitedStates</a>. (C++ enum variant: <span style='color: green;'>```C = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const C: crate::q_locale::Language = crate::q_locale::Language(1);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Abkhazian = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const Abkhazian: crate::q_locale::Language = crate::q_locale::Language(2);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Oromo = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const Oromo: crate::q_locale::Language = crate::q_locale::Language(3);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Afar = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const Afar: crate::q_locale::Language = crate::q_locale::Language(4);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Afrikaans = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const Afrikaans: crate::q_locale::Language = crate::q_locale::Language(5);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Albanian = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const Albanian: crate::q_locale::Language = crate::q_locale::Language(6);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Amharic = 7```</span>)
        #[allow(non_upper_case_globals)]
        pub const Amharic: crate::q_locale::Language = crate::q_locale::Language(7);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Arabic = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const Arabic: crate::q_locale::Language = crate::q_locale::Language(8);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Armenian = 9```</span>)
        #[allow(non_upper_case_globals)]
        pub const Armenian: crate::q_locale::Language = crate::q_locale::Language(9);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Assamese = 10```</span>)
        #[allow(non_upper_case_globals)]
        pub const Assamese: crate::q_locale::Language = crate::q_locale::Language(10);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Aymara = 11```</span>)
        #[allow(non_upper_case_globals)]
        pub const Aymara: crate::q_locale::Language = crate::q_locale::Language(11);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Azerbaijani = 12```</span>)
        #[allow(non_upper_case_globals)]
        pub const Azerbaijani: crate::q_locale::Language = crate::q_locale::Language(12);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Bashkir = 13```</span>)
        #[allow(non_upper_case_globals)]
        pub const Bashkir: crate::q_locale::Language = crate::q_locale::Language(13);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Basque = 14```</span>)
        #[allow(non_upper_case_globals)]
        pub const Basque: crate::q_locale::Language = crate::q_locale::Language(14);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Bengali = 15```</span>)
        #[allow(non_upper_case_globals)]
        pub const Bengali: crate::q_locale::Language = crate::q_locale::Language(15);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Dzongkha = 16```</span>)
        #[allow(non_upper_case_globals)]
        pub const Dzongkha: crate::q_locale::Language = crate::q_locale::Language(16);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Bihari = 17```</span>)
        #[allow(non_upper_case_globals)]
        pub const Bihari: crate::q_locale::Language = crate::q_locale::Language(17);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Bislama = 18```</span>)
        #[allow(non_upper_case_globals)]
        pub const Bislama: crate::q_locale::Language = crate::q_locale::Language(18);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Breton = 19```</span>)
        #[allow(non_upper_case_globals)]
        pub const Breton: crate::q_locale::Language = crate::q_locale::Language(19);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Bulgarian = 20```</span>)
        #[allow(non_upper_case_globals)]
        pub const Bulgarian: crate::q_locale::Language = crate::q_locale::Language(20);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Burmese = 21```</span>)
        #[allow(non_upper_case_globals)]
        pub const Burmese: crate::q_locale::Language = crate::q_locale::Language(21);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Belarusian = 22```</span>)
        #[allow(non_upper_case_globals)]
        pub const Belarusian: crate::q_locale::Language = crate::q_locale::Language(22);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Khmer = 23```</span>)
        #[allow(non_upper_case_globals)]
        pub const Khmer: crate::q_locale::Language = crate::q_locale::Language(23);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Catalan = 24```</span>)
        #[allow(non_upper_case_globals)]
        pub const Catalan: crate::q_locale::Language = crate::q_locale::Language(24);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Chinese = 25```</span>)
        #[allow(non_upper_case_globals)]
        pub const Chinese: crate::q_locale::Language = crate::q_locale::Language(25);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Corsican = 26```</span>)
        #[allow(non_upper_case_globals)]
        pub const Corsican: crate::q_locale::Language = crate::q_locale::Language(26);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Croatian = 27```</span>)
        #[allow(non_upper_case_globals)]
        pub const Croatian: crate::q_locale::Language = crate::q_locale::Language(27);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Czech = 28```</span>)
        #[allow(non_upper_case_globals)]
        pub const Czech: crate::q_locale::Language = crate::q_locale::Language(28);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Danish = 29```</span>)
        #[allow(non_upper_case_globals)]
        pub const Danish: crate::q_locale::Language = crate::q_locale::Language(29);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Dutch = 30```</span>)
        #[allow(non_upper_case_globals)]
        pub const Dutch: crate::q_locale::Language = crate::q_locale::Language(30);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```English = 31```</span>)
        #[allow(non_upper_case_globals)]
        pub const English: crate::q_locale::Language = crate::q_locale::Language(31);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Esperanto = 32```</span>)
        #[allow(non_upper_case_globals)]
        pub const Esperanto: crate::q_locale::Language = crate::q_locale::Language(32);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Estonian = 33```</span>)
        #[allow(non_upper_case_globals)]
        pub const Estonian: crate::q_locale::Language = crate::q_locale::Language(33);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Faroese = 34```</span>)
        #[allow(non_upper_case_globals)]
        pub const Faroese: crate::q_locale::Language = crate::q_locale::Language(34);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Fijian = 35```</span>)
        #[allow(non_upper_case_globals)]
        pub const Fijian: crate::q_locale::Language = crate::q_locale::Language(35);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Finnish = 36```</span>)
        #[allow(non_upper_case_globals)]
        pub const Finnish: crate::q_locale::Language = crate::q_locale::Language(36);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```French = 37```</span>)
        #[allow(non_upper_case_globals)]
        pub const French: crate::q_locale::Language = crate::q_locale::Language(37);
        /// same as Frisian (C++ enum variant: <span style='color: green;'>```WesternFrisian = 38```</span>)
        #[allow(non_upper_case_globals)]
        pub const WesternFrisian: crate::q_locale::Language = crate::q_locale::Language(38);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Gaelic = 39```</span>)
        #[allow(non_upper_case_globals)]
        pub const Gaelic: crate::q_locale::Language = crate::q_locale::Language(39);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Galician = 40```</span>)
        #[allow(non_upper_case_globals)]
        pub const Galician: crate::q_locale::Language = crate::q_locale::Language(40);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Georgian = 41```</span>)
        #[allow(non_upper_case_globals)]
        pub const Georgian: crate::q_locale::Language = crate::q_locale::Language(41);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```German = 42```</span>)
        #[allow(non_upper_case_globals)]
        pub const German: crate::q_locale::Language = crate::q_locale::Language(42);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Greek = 43```</span>)
        #[allow(non_upper_case_globals)]
        pub const Greek: crate::q_locale::Language = crate::q_locale::Language(43);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Greenlandic = 44```</span>)
        #[allow(non_upper_case_globals)]
        pub const Greenlandic: crate::q_locale::Language = crate::q_locale::Language(44);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Guarani = 45```</span>)
        #[allow(non_upper_case_globals)]
        pub const Guarani: crate::q_locale::Language = crate::q_locale::Language(45);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Gujarati = 46```</span>)
        #[allow(non_upper_case_globals)]
        pub const Gujarati: crate::q_locale::Language = crate::q_locale::Language(46);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Hausa = 47```</span>)
        #[allow(non_upper_case_globals)]
        pub const Hausa: crate::q_locale::Language = crate::q_locale::Language(47);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Hebrew = 48```</span>)
        #[allow(non_upper_case_globals)]
        pub const Hebrew: crate::q_locale::Language = crate::q_locale::Language(48);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Hindi = 49```</span>)
        #[allow(non_upper_case_globals)]
        pub const Hindi: crate::q_locale::Language = crate::q_locale::Language(49);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Hungarian = 50```</span>)
        #[allow(non_upper_case_globals)]
        pub const Hungarian: crate::q_locale::Language = crate::q_locale::Language(50);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Icelandic = 51```</span>)
        #[allow(non_upper_case_globals)]
        pub const Icelandic: crate::q_locale::Language = crate::q_locale::Language(51);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Indonesian = 52```</span>)
        #[allow(non_upper_case_globals)]
        pub const Indonesian: crate::q_locale::Language = crate::q_locale::Language(52);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Interlingua = 53```</span>)
        #[allow(non_upper_case_globals)]
        pub const Interlingua: crate::q_locale::Language = crate::q_locale::Language(53);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Interlingue = 54```</span>)
        #[allow(non_upper_case_globals)]
        pub const Interlingue: crate::q_locale::Language = crate::q_locale::Language(54);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Inuktitut = 55```</span>)
        #[allow(non_upper_case_globals)]
        pub const Inuktitut: crate::q_locale::Language = crate::q_locale::Language(55);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Inupiak = 56```</span>)
        #[allow(non_upper_case_globals)]
        pub const Inupiak: crate::q_locale::Language = crate::q_locale::Language(56);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Irish = 57```</span>)
        #[allow(non_upper_case_globals)]
        pub const Irish: crate::q_locale::Language = crate::q_locale::Language(57);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Italian = 58```</span>)
        #[allow(non_upper_case_globals)]
        pub const Italian: crate::q_locale::Language = crate::q_locale::Language(58);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Japanese = 59```</span>)
        #[allow(non_upper_case_globals)]
        pub const Japanese: crate::q_locale::Language = crate::q_locale::Language(59);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Javanese = 60```</span>)
        #[allow(non_upper_case_globals)]
        pub const Javanese: crate::q_locale::Language = crate::q_locale::Language(60);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Kannada = 61```</span>)
        #[allow(non_upper_case_globals)]
        pub const Kannada: crate::q_locale::Language = crate::q_locale::Language(61);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Kashmiri = 62```</span>)
        #[allow(non_upper_case_globals)]
        pub const Kashmiri: crate::q_locale::Language = crate::q_locale::Language(62);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Kazakh = 63```</span>)
        #[allow(non_upper_case_globals)]
        pub const Kazakh: crate::q_locale::Language = crate::q_locale::Language(63);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Kinyarwanda = 64```</span>)
        #[allow(non_upper_case_globals)]
        pub const Kinyarwanda: crate::q_locale::Language = crate::q_locale::Language(64);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Kirghiz = 65```</span>)
        #[allow(non_upper_case_globals)]
        pub const Kirghiz: crate::q_locale::Language = crate::q_locale::Language(65);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Korean = 66```</span>)
        #[allow(non_upper_case_globals)]
        pub const Korean: crate::q_locale::Language = crate::q_locale::Language(66);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Kurdish = 67```</span>)
        #[allow(non_upper_case_globals)]
        pub const Kurdish: crate::q_locale::Language = crate::q_locale::Language(67);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Rundi = 68```</span>)
        #[allow(non_upper_case_globals)]
        pub const Rundi: crate::q_locale::Language = crate::q_locale::Language(68);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Lao = 69```</span>)
        #[allow(non_upper_case_globals)]
        pub const Lao: crate::q_locale::Language = crate::q_locale::Language(69);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Latin = 70```</span>)
        #[allow(non_upper_case_globals)]
        pub const Latin: crate::q_locale::Language = crate::q_locale::Language(70);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Latvian = 71```</span>)
        #[allow(non_upper_case_globals)]
        pub const Latvian: crate::q_locale::Language = crate::q_locale::Language(71);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Lingala = 72```</span>)
        #[allow(non_upper_case_globals)]
        pub const Lingala: crate::q_locale::Language = crate::q_locale::Language(72);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Lithuanian = 73```</span>)
        #[allow(non_upper_case_globals)]
        pub const Lithuanian: crate::q_locale::Language = crate::q_locale::Language(73);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Macedonian = 74```</span>)
        #[allow(non_upper_case_globals)]
        pub const Macedonian: crate::q_locale::Language = crate::q_locale::Language(74);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Malagasy = 75```</span>)
        #[allow(non_upper_case_globals)]
        pub const Malagasy: crate::q_locale::Language = crate::q_locale::Language(75);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Malay = 76```</span>)
        #[allow(non_upper_case_globals)]
        pub const Malay: crate::q_locale::Language = crate::q_locale::Language(76);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Malayalam = 77```</span>)
        #[allow(non_upper_case_globals)]
        pub const Malayalam: crate::q_locale::Language = crate::q_locale::Language(77);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Maltese = 78```</span>)
        #[allow(non_upper_case_globals)]
        pub const Maltese: crate::q_locale::Language = crate::q_locale::Language(78);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Maori = 79```</span>)
        #[allow(non_upper_case_globals)]
        pub const Maori: crate::q_locale::Language = crate::q_locale::Language(79);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Marathi = 80```</span>)
        #[allow(non_upper_case_globals)]
        pub const Marathi: crate::q_locale::Language = crate::q_locale::Language(80);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Marshallese = 81```</span>)
        #[allow(non_upper_case_globals)]
        pub const Marshallese: crate::q_locale::Language = crate::q_locale::Language(81);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Mongolian = 82```</span>)
        #[allow(non_upper_case_globals)]
        pub const Mongolian: crate::q_locale::Language = crate::q_locale::Language(82);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```NauruLanguage = 83```</span>)
        #[allow(non_upper_case_globals)]
        pub const NauruLanguage: crate::q_locale::Language = crate::q_locale::Language(83);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Nepali = 84```</span>)
        #[allow(non_upper_case_globals)]
        pub const Nepali: crate::q_locale::Language = crate::q_locale::Language(84);
        /// same as Norwegian (C++ enum variant: <span style='color: green;'>```NorwegianBokmal = 85```</span>)
        #[allow(non_upper_case_globals)]
        pub const NorwegianBokmal: crate::q_locale::Language = crate::q_locale::Language(85);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Occitan = 86```</span>)
        #[allow(non_upper_case_globals)]
        pub const Occitan: crate::q_locale::Language = crate::q_locale::Language(86);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Oriya = 87```</span>)
        #[allow(non_upper_case_globals)]
        pub const Oriya: crate::q_locale::Language = crate::q_locale::Language(87);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Pashto = 88```</span>)
        #[allow(non_upper_case_globals)]
        pub const Pashto: crate::q_locale::Language = crate::q_locale::Language(88);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Persian = 89```</span>)
        #[allow(non_upper_case_globals)]
        pub const Persian: crate::q_locale::Language = crate::q_locale::Language(89);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Polish = 90```</span>)
        #[allow(non_upper_case_globals)]
        pub const Polish: crate::q_locale::Language = crate::q_locale::Language(90);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Portuguese = 91```</span>)
        #[allow(non_upper_case_globals)]
        pub const Portuguese: crate::q_locale::Language = crate::q_locale::Language(91);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Punjabi = 92```</span>)
        #[allow(non_upper_case_globals)]
        pub const Punjabi: crate::q_locale::Language = crate::q_locale::Language(92);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Quechua = 93```</span>)
        #[allow(non_upper_case_globals)]
        pub const Quechua: crate::q_locale::Language = crate::q_locale::Language(93);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Romansh = 94```</span>)
        #[allow(non_upper_case_globals)]
        pub const Romansh: crate::q_locale::Language = crate::q_locale::Language(94);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Romanian = 95```</span>)
        #[allow(non_upper_case_globals)]
        pub const Romanian: crate::q_locale::Language = crate::q_locale::Language(95);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Russian = 96```</span>)
        #[allow(non_upper_case_globals)]
        pub const Russian: crate::q_locale::Language = crate::q_locale::Language(96);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Samoan = 97```</span>)
        #[allow(non_upper_case_globals)]
        pub const Samoan: crate::q_locale::Language = crate::q_locale::Language(97);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Sango = 98```</span>)
        #[allow(non_upper_case_globals)]
        pub const Sango: crate::q_locale::Language = crate::q_locale::Language(98);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Sanskrit = 99```</span>)
        #[allow(non_upper_case_globals)]
        pub const Sanskrit: crate::q_locale::Language = crate::q_locale::Language(99);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Serbian = 100```</span>)
        #[allow(non_upper_case_globals)]
        pub const Serbian: crate::q_locale::Language = crate::q_locale::Language(100);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Ossetic = 101```</span>)
        #[allow(non_upper_case_globals)]
        pub const Ossetic: crate::q_locale::Language = crate::q_locale::Language(101);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SouthernSotho = 102```</span>)
        #[allow(non_upper_case_globals)]
        pub const SouthernSotho: crate::q_locale::Language = crate::q_locale::Language(102);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Tswana = 103```</span>)
        #[allow(non_upper_case_globals)]
        pub const Tswana: crate::q_locale::Language = crate::q_locale::Language(103);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Shona = 104```</span>)
        #[allow(non_upper_case_globals)]
        pub const Shona: crate::q_locale::Language = crate::q_locale::Language(104);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Sindhi = 105```</span>)
        #[allow(non_upper_case_globals)]
        pub const Sindhi: crate::q_locale::Language = crate::q_locale::Language(105);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Sinhala = 106```</span>)
        #[allow(non_upper_case_globals)]
        pub const Sinhala: crate::q_locale::Language = crate::q_locale::Language(106);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Swati = 107```</span>)
        #[allow(non_upper_case_globals)]
        pub const Swati: crate::q_locale::Language = crate::q_locale::Language(107);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Slovak = 108```</span>)
        #[allow(non_upper_case_globals)]
        pub const Slovak: crate::q_locale::Language = crate::q_locale::Language(108);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Slovenian = 109```</span>)
        #[allow(non_upper_case_globals)]
        pub const Slovenian: crate::q_locale::Language = crate::q_locale::Language(109);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Somali = 110```</span>)
        #[allow(non_upper_case_globals)]
        pub const Somali: crate::q_locale::Language = crate::q_locale::Language(110);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Spanish = 111```</span>)
        #[allow(non_upper_case_globals)]
        pub const Spanish: crate::q_locale::Language = crate::q_locale::Language(111);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Sundanese = 112```</span>)
        #[allow(non_upper_case_globals)]
        pub const Sundanese: crate::q_locale::Language = crate::q_locale::Language(112);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Swahili = 113```</span>)
        #[allow(non_upper_case_globals)]
        pub const Swahili: crate::q_locale::Language = crate::q_locale::Language(113);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Swedish = 114```</span>)
        #[allow(non_upper_case_globals)]
        pub const Swedish: crate::q_locale::Language = crate::q_locale::Language(114);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Sardinian = 115```</span>)
        #[allow(non_upper_case_globals)]
        pub const Sardinian: crate::q_locale::Language = crate::q_locale::Language(115);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Tajik = 116```</span>)
        #[allow(non_upper_case_globals)]
        pub const Tajik: crate::q_locale::Language = crate::q_locale::Language(116);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Tamil = 117```</span>)
        #[allow(non_upper_case_globals)]
        pub const Tamil: crate::q_locale::Language = crate::q_locale::Language(117);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Tatar = 118```</span>)
        #[allow(non_upper_case_globals)]
        pub const Tatar: crate::q_locale::Language = crate::q_locale::Language(118);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Telugu = 119```</span>)
        #[allow(non_upper_case_globals)]
        pub const Telugu: crate::q_locale::Language = crate::q_locale::Language(119);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Thai = 120```</span>)
        #[allow(non_upper_case_globals)]
        pub const Thai: crate::q_locale::Language = crate::q_locale::Language(120);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Tibetan = 121```</span>)
        #[allow(non_upper_case_globals)]
        pub const Tibetan: crate::q_locale::Language = crate::q_locale::Language(121);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Tigrinya = 122```</span>)
        #[allow(non_upper_case_globals)]
        pub const Tigrinya: crate::q_locale::Language = crate::q_locale::Language(122);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Tongan = 123```</span>)
        #[allow(non_upper_case_globals)]
        pub const Tongan: crate::q_locale::Language = crate::q_locale::Language(123);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Tsonga = 124```</span>)
        #[allow(non_upper_case_globals)]
        pub const Tsonga: crate::q_locale::Language = crate::q_locale::Language(124);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Turkish = 125```</span>)
        #[allow(non_upper_case_globals)]
        pub const Turkish: crate::q_locale::Language = crate::q_locale::Language(125);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Turkmen = 126```</span>)
        #[allow(non_upper_case_globals)]
        pub const Turkmen: crate::q_locale::Language = crate::q_locale::Language(126);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Tahitian = 127```</span>)
        #[allow(non_upper_case_globals)]
        pub const Tahitian: crate::q_locale::Language = crate::q_locale::Language(127);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Uighur = 128```</span>)
        #[allow(non_upper_case_globals)]
        pub const Uighur: crate::q_locale::Language = crate::q_locale::Language(128);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Ukrainian = 129```</span>)
        #[allow(non_upper_case_globals)]
        pub const Ukrainian: crate::q_locale::Language = crate::q_locale::Language(129);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Urdu = 130```</span>)
        #[allow(non_upper_case_globals)]
        pub const Urdu: crate::q_locale::Language = crate::q_locale::Language(130);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Uzbek = 131```</span>)
        #[allow(non_upper_case_globals)]
        pub const Uzbek: crate::q_locale::Language = crate::q_locale::Language(131);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Vietnamese = 132```</span>)
        #[allow(non_upper_case_globals)]
        pub const Vietnamese: crate::q_locale::Language = crate::q_locale::Language(132);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Volapuk = 133```</span>)
        #[allow(non_upper_case_globals)]
        pub const Volapuk: crate::q_locale::Language = crate::q_locale::Language(133);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Welsh = 134```</span>)
        #[allow(non_upper_case_globals)]
        pub const Welsh: crate::q_locale::Language = crate::q_locale::Language(134);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Wolof = 135```</span>)
        #[allow(non_upper_case_globals)]
        pub const Wolof: crate::q_locale::Language = crate::q_locale::Language(135);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Xhosa = 136```</span>)
        #[allow(non_upper_case_globals)]
        pub const Xhosa: crate::q_locale::Language = crate::q_locale::Language(136);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Yiddish = 137```</span>)
        #[allow(non_upper_case_globals)]
        pub const Yiddish: crate::q_locale::Language = crate::q_locale::Language(137);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Yoruba = 138```</span>)
        #[allow(non_upper_case_globals)]
        pub const Yoruba: crate::q_locale::Language = crate::q_locale::Language(138);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Zhuang = 139```</span>)
        #[allow(non_upper_case_globals)]
        pub const Zhuang: crate::q_locale::Language = crate::q_locale::Language(139);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Zulu = 140```</span>)
        #[allow(non_upper_case_globals)]
        pub const Zulu: crate::q_locale::Language = crate::q_locale::Language(140);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```NorwegianNynorsk = 141```</span>)
        #[allow(non_upper_case_globals)]
        pub const NorwegianNynorsk: crate::q_locale::Language = crate::q_locale::Language(141);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Bosnian = 142```</span>)
        #[allow(non_upper_case_globals)]
        pub const Bosnian: crate::q_locale::Language = crate::q_locale::Language(142);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Divehi = 143```</span>)
        #[allow(non_upper_case_globals)]
        pub const Divehi: crate::q_locale::Language = crate::q_locale::Language(143);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Manx = 144```</span>)
        #[allow(non_upper_case_globals)]
        pub const Manx: crate::q_locale::Language = crate::q_locale::Language(144);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Cornish = 145```</span>)
        #[allow(non_upper_case_globals)]
        pub const Cornish: crate::q_locale::Language = crate::q_locale::Language(145);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Akan = 146```</span>)
        #[allow(non_upper_case_globals)]
        pub const Akan: crate::q_locale::Language = crate::q_locale::Language(146);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Konkani = 147```</span>)
        #[allow(non_upper_case_globals)]
        pub const Konkani: crate::q_locale::Language = crate::q_locale::Language(147);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Ga = 148```</span>)
        #[allow(non_upper_case_globals)]
        pub const Ga: crate::q_locale::Language = crate::q_locale::Language(148);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Igbo = 149```</span>)
        #[allow(non_upper_case_globals)]
        pub const Igbo: crate::q_locale::Language = crate::q_locale::Language(149);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Kamba = 150```</span>)
        #[allow(non_upper_case_globals)]
        pub const Kamba: crate::q_locale::Language = crate::q_locale::Language(150);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Syriac = 151```</span>)
        #[allow(non_upper_case_globals)]
        pub const Syriac: crate::q_locale::Language = crate::q_locale::Language(151);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Blin = 152```</span>)
        #[allow(non_upper_case_globals)]
        pub const Blin: crate::q_locale::Language = crate::q_locale::Language(152);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Geez = 153```</span>)
        #[allow(non_upper_case_globals)]
        pub const Geez: crate::q_locale::Language = crate::q_locale::Language(153);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Koro = 154```</span>)
        #[allow(non_upper_case_globals)]
        pub const Koro: crate::q_locale::Language = crate::q_locale::Language(154);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Sidamo = 155```</span>)
        #[allow(non_upper_case_globals)]
        pub const Sidamo: crate::q_locale::Language = crate::q_locale::Language(155);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Atsam = 156```</span>)
        #[allow(non_upper_case_globals)]
        pub const Atsam: crate::q_locale::Language = crate::q_locale::Language(156);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Tigre = 157```</span>)
        #[allow(non_upper_case_globals)]
        pub const Tigre: crate::q_locale::Language = crate::q_locale::Language(157);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Jju = 158```</span>)
        #[allow(non_upper_case_globals)]
        pub const Jju: crate::q_locale::Language = crate::q_locale::Language(158);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Friulian = 159```</span>)
        #[allow(non_upper_case_globals)]
        pub const Friulian: crate::q_locale::Language = crate::q_locale::Language(159);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Venda = 160```</span>)
        #[allow(non_upper_case_globals)]
        pub const Venda: crate::q_locale::Language = crate::q_locale::Language(160);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Ewe = 161```</span>)
        #[allow(non_upper_case_globals)]
        pub const Ewe: crate::q_locale::Language = crate::q_locale::Language(161);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Walamo = 162```</span>)
        #[allow(non_upper_case_globals)]
        pub const Walamo: crate::q_locale::Language = crate::q_locale::Language(162);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Hawaiian = 163```</span>)
        #[allow(non_upper_case_globals)]
        pub const Hawaiian: crate::q_locale::Language = crate::q_locale::Language(163);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Tyap = 164```</span>)
        #[allow(non_upper_case_globals)]
        pub const Tyap: crate::q_locale::Language = crate::q_locale::Language(164);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Nyanja = 165```</span>)
        #[allow(non_upper_case_globals)]
        pub const Nyanja: crate::q_locale::Language = crate::q_locale::Language(165);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Filipino = 166```</span>)
        #[allow(non_upper_case_globals)]
        pub const Filipino: crate::q_locale::Language = crate::q_locale::Language(166);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SwissGerman = 167```</span>)
        #[allow(non_upper_case_globals)]
        pub const SwissGerman: crate::q_locale::Language = crate::q_locale::Language(167);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SichuanYi = 168```</span>)
        #[allow(non_upper_case_globals)]
        pub const SichuanYi: crate::q_locale::Language = crate::q_locale::Language(168);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Kpelle = 169```</span>)
        #[allow(non_upper_case_globals)]
        pub const Kpelle: crate::q_locale::Language = crate::q_locale::Language(169);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```LowGerman = 170```</span>)
        #[allow(non_upper_case_globals)]
        pub const LowGerman: crate::q_locale::Language = crate::q_locale::Language(170);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SouthNdebele = 171```</span>)
        #[allow(non_upper_case_globals)]
        pub const SouthNdebele: crate::q_locale::Language = crate::q_locale::Language(171);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```NorthernSotho = 172```</span>)
        #[allow(non_upper_case_globals)]
        pub const NorthernSotho: crate::q_locale::Language = crate::q_locale::Language(172);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```NorthernSami = 173```</span>)
        #[allow(non_upper_case_globals)]
        pub const NorthernSami: crate::q_locale::Language = crate::q_locale::Language(173);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Taroko = 174```</span>)
        #[allow(non_upper_case_globals)]
        pub const Taroko: crate::q_locale::Language = crate::q_locale::Language(174);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Gusii = 175```</span>)
        #[allow(non_upper_case_globals)]
        pub const Gusii: crate::q_locale::Language = crate::q_locale::Language(175);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Taita = 176```</span>)
        #[allow(non_upper_case_globals)]
        pub const Taita: crate::q_locale::Language = crate::q_locale::Language(176);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Fulah = 177```</span>)
        #[allow(non_upper_case_globals)]
        pub const Fulah: crate::q_locale::Language = crate::q_locale::Language(177);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Kikuyu = 178```</span>)
        #[allow(non_upper_case_globals)]
        pub const Kikuyu: crate::q_locale::Language = crate::q_locale::Language(178);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Samburu = 179```</span>)
        #[allow(non_upper_case_globals)]
        pub const Samburu: crate::q_locale::Language = crate::q_locale::Language(179);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Sena = 180```</span>)
        #[allow(non_upper_case_globals)]
        pub const Sena: crate::q_locale::Language = crate::q_locale::Language(180);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```NorthNdebele = 181```</span>)
        #[allow(non_upper_case_globals)]
        pub const NorthNdebele: crate::q_locale::Language = crate::q_locale::Language(181);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Rombo = 182```</span>)
        #[allow(non_upper_case_globals)]
        pub const Rombo: crate::q_locale::Language = crate::q_locale::Language(182);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Tachelhit = 183```</span>)
        #[allow(non_upper_case_globals)]
        pub const Tachelhit: crate::q_locale::Language = crate::q_locale::Language(183);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Kabyle = 184```</span>)
        #[allow(non_upper_case_globals)]
        pub const Kabyle: crate::q_locale::Language = crate::q_locale::Language(184);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Nyankole = 185```</span>)
        #[allow(non_upper_case_globals)]
        pub const Nyankole: crate::q_locale::Language = crate::q_locale::Language(185);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Bena = 186```</span>)
        #[allow(non_upper_case_globals)]
        pub const Bena: crate::q_locale::Language = crate::q_locale::Language(186);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Vunjo = 187```</span>)
        #[allow(non_upper_case_globals)]
        pub const Vunjo: crate::q_locale::Language = crate::q_locale::Language(187);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Bambara = 188```</span>)
        #[allow(non_upper_case_globals)]
        pub const Bambara: crate::q_locale::Language = crate::q_locale::Language(188);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Embu = 189```</span>)
        #[allow(non_upper_case_globals)]
        pub const Embu: crate::q_locale::Language = crate::q_locale::Language(189);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Cherokee = 190```</span>)
        #[allow(non_upper_case_globals)]
        pub const Cherokee: crate::q_locale::Language = crate::q_locale::Language(190);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Morisyen = 191```</span>)
        #[allow(non_upper_case_globals)]
        pub const Morisyen: crate::q_locale::Language = crate::q_locale::Language(191);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Makonde = 192```</span>)
        #[allow(non_upper_case_globals)]
        pub const Makonde: crate::q_locale::Language = crate::q_locale::Language(192);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Langi = 193```</span>)
        #[allow(non_upper_case_globals)]
        pub const Langi: crate::q_locale::Language = crate::q_locale::Language(193);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Ganda = 194```</span>)
        #[allow(non_upper_case_globals)]
        pub const Ganda: crate::q_locale::Language = crate::q_locale::Language(194);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Bemba = 195```</span>)
        #[allow(non_upper_case_globals)]
        pub const Bemba: crate::q_locale::Language = crate::q_locale::Language(195);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Kabuverdianu = 196```</span>)
        #[allow(non_upper_case_globals)]
        pub const Kabuverdianu: crate::q_locale::Language = crate::q_locale::Language(196);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Meru = 197```</span>)
        #[allow(non_upper_case_globals)]
        pub const Meru: crate::q_locale::Language = crate::q_locale::Language(197);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Kalenjin = 198```</span>)
        #[allow(non_upper_case_globals)]
        pub const Kalenjin: crate::q_locale::Language = crate::q_locale::Language(198);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Nama = 199```</span>)
        #[allow(non_upper_case_globals)]
        pub const Nama: crate::q_locale::Language = crate::q_locale::Language(199);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Machame = 200```</span>)
        #[allow(non_upper_case_globals)]
        pub const Machame: crate::q_locale::Language = crate::q_locale::Language(200);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Colognian = 201```</span>)
        #[allow(non_upper_case_globals)]
        pub const Colognian: crate::q_locale::Language = crate::q_locale::Language(201);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Masai = 202```</span>)
        #[allow(non_upper_case_globals)]
        pub const Masai: crate::q_locale::Language = crate::q_locale::Language(202);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Soga = 203```</span>)
        #[allow(non_upper_case_globals)]
        pub const Soga: crate::q_locale::Language = crate::q_locale::Language(203);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Luyia = 204```</span>)
        #[allow(non_upper_case_globals)]
        pub const Luyia: crate::q_locale::Language = crate::q_locale::Language(204);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Asu = 205```</span>)
        #[allow(non_upper_case_globals)]
        pub const Asu: crate::q_locale::Language = crate::q_locale::Language(205);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Teso = 206```</span>)
        #[allow(non_upper_case_globals)]
        pub const Teso: crate::q_locale::Language = crate::q_locale::Language(206);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Saho = 207```</span>)
        #[allow(non_upper_case_globals)]
        pub const Saho: crate::q_locale::Language = crate::q_locale::Language(207);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```KoyraChiini = 208```</span>)
        #[allow(non_upper_case_globals)]
        pub const KoyraChiini: crate::q_locale::Language = crate::q_locale::Language(208);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Rwa = 209```</span>)
        #[allow(non_upper_case_globals)]
        pub const Rwa: crate::q_locale::Language = crate::q_locale::Language(209);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Luo = 210```</span>)
        #[allow(non_upper_case_globals)]
        pub const Luo: crate::q_locale::Language = crate::q_locale::Language(210);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Chiga = 211```</span>)
        #[allow(non_upper_case_globals)]
        pub const Chiga: crate::q_locale::Language = crate::q_locale::Language(211);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```CentralMoroccoTamazight = 212```</span>)
        #[allow(non_upper_case_globals)]
        pub const CentralMoroccoTamazight: crate::q_locale::Language =
            crate::q_locale::Language(212);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```KoyraboroSenni = 213```</span>)
        #[allow(non_upper_case_globals)]
        pub const KoyraboroSenni: crate::q_locale::Language = crate::q_locale::Language(213);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Shambala = 214```</span>)
        #[allow(non_upper_case_globals)]
        pub const Shambala: crate::q_locale::Language = crate::q_locale::Language(214);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Bodo = 215```</span>)
        #[allow(non_upper_case_globals)]
        pub const Bodo: crate::q_locale::Language = crate::q_locale::Language(215);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Avaric = 216```</span>)
        #[allow(non_upper_case_globals)]
        pub const Avaric: crate::q_locale::Language = crate::q_locale::Language(216);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Chamorro = 217```</span>)
        #[allow(non_upper_case_globals)]
        pub const Chamorro: crate::q_locale::Language = crate::q_locale::Language(217);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Chechen = 218```</span>)
        #[allow(non_upper_case_globals)]
        pub const Chechen: crate::q_locale::Language = crate::q_locale::Language(218);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Church = 219```</span>)
        #[allow(non_upper_case_globals)]
        pub const Church: crate::q_locale::Language = crate::q_locale::Language(219);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Chuvash = 220```</span>)
        #[allow(non_upper_case_globals)]
        pub const Chuvash: crate::q_locale::Language = crate::q_locale::Language(220);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Cree = 221```</span>)
        #[allow(non_upper_case_globals)]
        pub const Cree: crate::q_locale::Language = crate::q_locale::Language(221);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Haitian = 222```</span>)
        #[allow(non_upper_case_globals)]
        pub const Haitian: crate::q_locale::Language = crate::q_locale::Language(222);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Herero = 223```</span>)
        #[allow(non_upper_case_globals)]
        pub const Herero: crate::q_locale::Language = crate::q_locale::Language(223);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```HiriMotu = 224```</span>)
        #[allow(non_upper_case_globals)]
        pub const HiriMotu: crate::q_locale::Language = crate::q_locale::Language(224);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Kanuri = 225```</span>)
        #[allow(non_upper_case_globals)]
        pub const Kanuri: crate::q_locale::Language = crate::q_locale::Language(225);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Komi = 226```</span>)
        #[allow(non_upper_case_globals)]
        pub const Komi: crate::q_locale::Language = crate::q_locale::Language(226);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Kongo = 227```</span>)
        #[allow(non_upper_case_globals)]
        pub const Kongo: crate::q_locale::Language = crate::q_locale::Language(227);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Kwanyama = 228```</span>)
        #[allow(non_upper_case_globals)]
        pub const Kwanyama: crate::q_locale::Language = crate::q_locale::Language(228);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Limburgish = 229```</span>)
        #[allow(non_upper_case_globals)]
        pub const Limburgish: crate::q_locale::Language = crate::q_locale::Language(229);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```LubaKatanga = 230```</span>)
        #[allow(non_upper_case_globals)]
        pub const LubaKatanga: crate::q_locale::Language = crate::q_locale::Language(230);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Luxembourgish = 231```</span>)
        #[allow(non_upper_case_globals)]
        pub const Luxembourgish: crate::q_locale::Language = crate::q_locale::Language(231);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Navaho = 232```</span>)
        #[allow(non_upper_case_globals)]
        pub const Navaho: crate::q_locale::Language = crate::q_locale::Language(232);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Ndonga = 233```</span>)
        #[allow(non_upper_case_globals)]
        pub const Ndonga: crate::q_locale::Language = crate::q_locale::Language(233);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Ojibwa = 234```</span>)
        #[allow(non_upper_case_globals)]
        pub const Ojibwa: crate::q_locale::Language = crate::q_locale::Language(234);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Pali = 235```</span>)
        #[allow(non_upper_case_globals)]
        pub const Pali: crate::q_locale::Language = crate::q_locale::Language(235);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Walloon = 236```</span>)
        #[allow(non_upper_case_globals)]
        pub const Walloon: crate::q_locale::Language = crate::q_locale::Language(236);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Aghem = 237```</span>)
        #[allow(non_upper_case_globals)]
        pub const Aghem: crate::q_locale::Language = crate::q_locale::Language(237);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Basaa = 238```</span>)
        #[allow(non_upper_case_globals)]
        pub const Basaa: crate::q_locale::Language = crate::q_locale::Language(238);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Zarma = 239```</span>)
        #[allow(non_upper_case_globals)]
        pub const Zarma: crate::q_locale::Language = crate::q_locale::Language(239);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Duala = 240```</span>)
        #[allow(non_upper_case_globals)]
        pub const Duala: crate::q_locale::Language = crate::q_locale::Language(240);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```JolaFonyi = 241```</span>)
        #[allow(non_upper_case_globals)]
        pub const JolaFonyi: crate::q_locale::Language = crate::q_locale::Language(241);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Ewondo = 242```</span>)
        #[allow(non_upper_case_globals)]
        pub const Ewondo: crate::q_locale::Language = crate::q_locale::Language(242);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Bafia = 243```</span>)
        #[allow(non_upper_case_globals)]
        pub const Bafia: crate::q_locale::Language = crate::q_locale::Language(243);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```MakhuwaMeetto = 244```</span>)
        #[allow(non_upper_case_globals)]
        pub const MakhuwaMeetto: crate::q_locale::Language = crate::q_locale::Language(244);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Mundang = 245```</span>)
        #[allow(non_upper_case_globals)]
        pub const Mundang: crate::q_locale::Language = crate::q_locale::Language(245);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Kwasio = 246```</span>)
        #[allow(non_upper_case_globals)]
        pub const Kwasio: crate::q_locale::Language = crate::q_locale::Language(246);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Nuer = 247```</span>)
        #[allow(non_upper_case_globals)]
        pub const Nuer: crate::q_locale::Language = crate::q_locale::Language(247);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Sakha = 248```</span>)
        #[allow(non_upper_case_globals)]
        pub const Sakha: crate::q_locale::Language = crate::q_locale::Language(248);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Sangu = 249```</span>)
        #[allow(non_upper_case_globals)]
        pub const Sangu: crate::q_locale::Language = crate::q_locale::Language(249);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```CongoSwahili = 250```</span>)
        #[allow(non_upper_case_globals)]
        pub const CongoSwahili: crate::q_locale::Language = crate::q_locale::Language(250);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Tasawaq = 251```</span>)
        #[allow(non_upper_case_globals)]
        pub const Tasawaq: crate::q_locale::Language = crate::q_locale::Language(251);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Vai = 252```</span>)
        #[allow(non_upper_case_globals)]
        pub const Vai: crate::q_locale::Language = crate::q_locale::Language(252);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Walser = 253```</span>)
        #[allow(non_upper_case_globals)]
        pub const Walser: crate::q_locale::Language = crate::q_locale::Language(253);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Yangben = 254```</span>)
        #[allow(non_upper_case_globals)]
        pub const Yangben: crate::q_locale::Language = crate::q_locale::Language(254);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Avestan = 255```</span>)
        #[allow(non_upper_case_globals)]
        pub const Avestan: crate::q_locale::Language = crate::q_locale::Language(255);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Asturian = 256```</span>)
        #[allow(non_upper_case_globals)]
        pub const Asturian: crate::q_locale::Language = crate::q_locale::Language(256);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Ngomba = 257```</span>)
        #[allow(non_upper_case_globals)]
        pub const Ngomba: crate::q_locale::Language = crate::q_locale::Language(257);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Kako = 258```</span>)
        #[allow(non_upper_case_globals)]
        pub const Kako: crate::q_locale::Language = crate::q_locale::Language(258);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Meta = 259```</span>)
        #[allow(non_upper_case_globals)]
        pub const Meta: crate::q_locale::Language = crate::q_locale::Language(259);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Ngiemboon = 260```</span>)
        #[allow(non_upper_case_globals)]
        pub const Ngiemboon: crate::q_locale::Language = crate::q_locale::Language(260);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Aragonese = 261```</span>)
        #[allow(non_upper_case_globals)]
        pub const Aragonese: crate::q_locale::Language = crate::q_locale::Language(261);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Akkadian = 262```</span>)
        #[allow(non_upper_case_globals)]
        pub const Akkadian: crate::q_locale::Language = crate::q_locale::Language(262);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```AncientEgyptian = 263```</span>)
        #[allow(non_upper_case_globals)]
        pub const AncientEgyptian: crate::q_locale::Language = crate::q_locale::Language(263);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```AncientGreek = 264```</span>)
        #[allow(non_upper_case_globals)]
        pub const AncientGreek: crate::q_locale::Language = crate::q_locale::Language(264);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Aramaic = 265```</span>)
        #[allow(non_upper_case_globals)]
        pub const Aramaic: crate::q_locale::Language = crate::q_locale::Language(265);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Balinese = 266```</span>)
        #[allow(non_upper_case_globals)]
        pub const Balinese: crate::q_locale::Language = crate::q_locale::Language(266);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Bamun = 267```</span>)
        #[allow(non_upper_case_globals)]
        pub const Bamun: crate::q_locale::Language = crate::q_locale::Language(267);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```BatakToba = 268```</span>)
        #[allow(non_upper_case_globals)]
        pub const BatakToba: crate::q_locale::Language = crate::q_locale::Language(268);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Buginese = 269```</span>)
        #[allow(non_upper_case_globals)]
        pub const Buginese: crate::q_locale::Language = crate::q_locale::Language(269);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Buhid = 270```</span>)
        #[allow(non_upper_case_globals)]
        pub const Buhid: crate::q_locale::Language = crate::q_locale::Language(270);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Carian = 271```</span>)
        #[allow(non_upper_case_globals)]
        pub const Carian: crate::q_locale::Language = crate::q_locale::Language(271);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Chakma = 272```</span>)
        #[allow(non_upper_case_globals)]
        pub const Chakma: crate::q_locale::Language = crate::q_locale::Language(272);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```ClassicalMandaic = 273```</span>)
        #[allow(non_upper_case_globals)]
        pub const ClassicalMandaic: crate::q_locale::Language = crate::q_locale::Language(273);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Coptic = 274```</span>)
        #[allow(non_upper_case_globals)]
        pub const Coptic: crate::q_locale::Language = crate::q_locale::Language(274);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Dogri = 275```</span>)
        #[allow(non_upper_case_globals)]
        pub const Dogri: crate::q_locale::Language = crate::q_locale::Language(275);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```EasternCham = 276```</span>)
        #[allow(non_upper_case_globals)]
        pub const EasternCham: crate::q_locale::Language = crate::q_locale::Language(276);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```EasternKayah = 277```</span>)
        #[allow(non_upper_case_globals)]
        pub const EasternKayah: crate::q_locale::Language = crate::q_locale::Language(277);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Etruscan = 278```</span>)
        #[allow(non_upper_case_globals)]
        pub const Etruscan: crate::q_locale::Language = crate::q_locale::Language(278);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Gothic = 279```</span>)
        #[allow(non_upper_case_globals)]
        pub const Gothic: crate::q_locale::Language = crate::q_locale::Language(279);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Hanunoo = 280```</span>)
        #[allow(non_upper_case_globals)]
        pub const Hanunoo: crate::q_locale::Language = crate::q_locale::Language(280);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Ingush = 281```</span>)
        #[allow(non_upper_case_globals)]
        pub const Ingush: crate::q_locale::Language = crate::q_locale::Language(281);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```LargeFloweryMiao = 282```</span>)
        #[allow(non_upper_case_globals)]
        pub const LargeFloweryMiao: crate::q_locale::Language = crate::q_locale::Language(282);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Lepcha = 283```</span>)
        #[allow(non_upper_case_globals)]
        pub const Lepcha: crate::q_locale::Language = crate::q_locale::Language(283);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Limbu = 284```</span>)
        #[allow(non_upper_case_globals)]
        pub const Limbu: crate::q_locale::Language = crate::q_locale::Language(284);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Lisu = 285```</span>)
        #[allow(non_upper_case_globals)]
        pub const Lisu: crate::q_locale::Language = crate::q_locale::Language(285);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Lu = 286```</span>)
        #[allow(non_upper_case_globals)]
        pub const Lu: crate::q_locale::Language = crate::q_locale::Language(286);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Lycian = 287```</span>)
        #[allow(non_upper_case_globals)]
        pub const Lycian: crate::q_locale::Language = crate::q_locale::Language(287);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Lydian = 288```</span>)
        #[allow(non_upper_case_globals)]
        pub const Lydian: crate::q_locale::Language = crate::q_locale::Language(288);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Mandingo = 289```</span>)
        #[allow(non_upper_case_globals)]
        pub const Mandingo: crate::q_locale::Language = crate::q_locale::Language(289);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Manipuri = 290```</span>)
        #[allow(non_upper_case_globals)]
        pub const Manipuri: crate::q_locale::Language = crate::q_locale::Language(290);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Meroitic = 291```</span>)
        #[allow(non_upper_case_globals)]
        pub const Meroitic: crate::q_locale::Language = crate::q_locale::Language(291);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```NorthernThai = 292```</span>)
        #[allow(non_upper_case_globals)]
        pub const NorthernThai: crate::q_locale::Language = crate::q_locale::Language(292);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```OldIrish = 293```</span>)
        #[allow(non_upper_case_globals)]
        pub const OldIrish: crate::q_locale::Language = crate::q_locale::Language(293);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```OldNorse = 294```</span>)
        #[allow(non_upper_case_globals)]
        pub const OldNorse: crate::q_locale::Language = crate::q_locale::Language(294);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```OldPersian = 295```</span>)
        #[allow(non_upper_case_globals)]
        pub const OldPersian: crate::q_locale::Language = crate::q_locale::Language(295);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```OldTurkish = 296```</span>)
        #[allow(non_upper_case_globals)]
        pub const OldTurkish: crate::q_locale::Language = crate::q_locale::Language(296);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Pahlavi = 297```</span>)
        #[allow(non_upper_case_globals)]
        pub const Pahlavi: crate::q_locale::Language = crate::q_locale::Language(297);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Parthian = 298```</span>)
        #[allow(non_upper_case_globals)]
        pub const Parthian: crate::q_locale::Language = crate::q_locale::Language(298);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Phoenician = 299```</span>)
        #[allow(non_upper_case_globals)]
        pub const Phoenician: crate::q_locale::Language = crate::q_locale::Language(299);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```PrakritLanguage = 300```</span>)
        #[allow(non_upper_case_globals)]
        pub const PrakritLanguage: crate::q_locale::Language = crate::q_locale::Language(300);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Rejang = 301```</span>)
        #[allow(non_upper_case_globals)]
        pub const Rejang: crate::q_locale::Language = crate::q_locale::Language(301);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Sabaean = 302```</span>)
        #[allow(non_upper_case_globals)]
        pub const Sabaean: crate::q_locale::Language = crate::q_locale::Language(302);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Samaritan = 303```</span>)
        #[allow(non_upper_case_globals)]
        pub const Samaritan: crate::q_locale::Language = crate::q_locale::Language(303);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Santali = 304```</span>)
        #[allow(non_upper_case_globals)]
        pub const Santali: crate::q_locale::Language = crate::q_locale::Language(304);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Saurashtra = 305```</span>)
        #[allow(non_upper_case_globals)]
        pub const Saurashtra: crate::q_locale::Language = crate::q_locale::Language(305);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Sora = 306```</span>)
        #[allow(non_upper_case_globals)]
        pub const Sora: crate::q_locale::Language = crate::q_locale::Language(306);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Sylheti = 307```</span>)
        #[allow(non_upper_case_globals)]
        pub const Sylheti: crate::q_locale::Language = crate::q_locale::Language(307);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Tagbanwa = 308```</span>)
        #[allow(non_upper_case_globals)]
        pub const Tagbanwa: crate::q_locale::Language = crate::q_locale::Language(308);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```TaiDam = 309```</span>)
        #[allow(non_upper_case_globals)]
        pub const TaiDam: crate::q_locale::Language = crate::q_locale::Language(309);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```TaiNua = 310```</span>)
        #[allow(non_upper_case_globals)]
        pub const TaiNua: crate::q_locale::Language = crate::q_locale::Language(310);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Ugaritic = 311```</span>)
        #[allow(non_upper_case_globals)]
        pub const Ugaritic: crate::q_locale::Language = crate::q_locale::Language(311);
        /// Since Qt 5.3 (C++ enum variant: <span style='color: green;'>```Akoose = 312```</span>)
        #[allow(non_upper_case_globals)]
        pub const Akoose: crate::q_locale::Language = crate::q_locale::Language(312);
        /// Since Qt 5.3 (C++ enum variant: <span style='color: green;'>```Lakota = 313```</span>)
        #[allow(non_upper_case_globals)]
        pub const Lakota: crate::q_locale::Language = crate::q_locale::Language(313);
        /// Since Qt 5.3 (C++ enum variant: <span style='color: green;'>```StandardMoroccanTamazight = 314```</span>)
        #[allow(non_upper_case_globals)]
        pub const StandardMoroccanTamazight: crate::q_locale::Language =
            crate::q_locale::Language(314);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```Mapuche = 315```</span>)
        #[allow(non_upper_case_globals)]
        pub const Mapuche: crate::q_locale::Language = crate::q_locale::Language(315);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```CentralKurdish = 316```</span>)
        #[allow(non_upper_case_globals)]
        pub const CentralKurdish: crate::q_locale::Language = crate::q_locale::Language(316);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```LowerSorbian = 317```</span>)
        #[allow(non_upper_case_globals)]
        pub const LowerSorbian: crate::q_locale::Language = crate::q_locale::Language(317);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```UpperSorbian = 318```</span>)
        #[allow(non_upper_case_globals)]
        pub const UpperSorbian: crate::q_locale::Language = crate::q_locale::Language(318);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```Kenyang = 319```</span>)
        #[allow(non_upper_case_globals)]
        pub const Kenyang: crate::q_locale::Language = crate::q_locale::Language(319);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```Mohawk = 320```</span>)
        #[allow(non_upper_case_globals)]
        pub const Mohawk: crate::q_locale::Language = crate::q_locale::Language(320);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```Nko = 321```</span>)
        #[allow(non_upper_case_globals)]
        pub const Nko: crate::q_locale::Language = crate::q_locale::Language(321);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```Prussian = 322```</span>)
        #[allow(non_upper_case_globals)]
        pub const Prussian: crate::q_locale::Language = crate::q_locale::Language(322);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```Kiche = 323```</span>)
        #[allow(non_upper_case_globals)]
        pub const Kiche: crate::q_locale::Language = crate::q_locale::Language(323);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```SouthernSami = 324```</span>)
        #[allow(non_upper_case_globals)]
        pub const SouthernSami: crate::q_locale::Language = crate::q_locale::Language(324);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```LuleSami = 325```</span>)
        #[allow(non_upper_case_globals)]
        pub const LuleSami: crate::q_locale::Language = crate::q_locale::Language(325);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```InariSami = 326```</span>)
        #[allow(non_upper_case_globals)]
        pub const InariSami: crate::q_locale::Language = crate::q_locale::Language(326);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```SkoltSami = 327```</span>)
        #[allow(non_upper_case_globals)]
        pub const SkoltSami: crate::q_locale::Language = crate::q_locale::Language(327);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```Warlpiri = 328```</span>)
        #[allow(non_upper_case_globals)]
        pub const Warlpiri: crate::q_locale::Language = crate::q_locale::Language(328);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```ManichaeanMiddlePersian = 329```</span>)
        #[allow(non_upper_case_globals)]
        pub const ManichaeanMiddlePersian: crate::q_locale::Language =
            crate::q_locale::Language(329);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```Mende = 330```</span>)
        #[allow(non_upper_case_globals)]
        pub const Mende: crate::q_locale::Language = crate::q_locale::Language(330);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```AncientNorthArabian = 331```</span>)
        #[allow(non_upper_case_globals)]
        pub const AncientNorthArabian: crate::q_locale::Language = crate::q_locale::Language(331);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```LinearA = 332```</span>)
        #[allow(non_upper_case_globals)]
        pub const LinearA: crate::q_locale::Language = crate::q_locale::Language(332);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```HmongNjua = 333```</span>)
        #[allow(non_upper_case_globals)]
        pub const HmongNjua: crate::q_locale::Language = crate::q_locale::Language(333);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```Ho = 334```</span>)
        #[allow(non_upper_case_globals)]
        pub const Ho: crate::q_locale::Language = crate::q_locale::Language(334);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```Lezghian = 335```</span>)
        #[allow(non_upper_case_globals)]
        pub const Lezghian: crate::q_locale::Language = crate::q_locale::Language(335);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```Bassa = 336```</span>)
        #[allow(non_upper_case_globals)]
        pub const Bassa: crate::q_locale::Language = crate::q_locale::Language(336);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```Mono = 337```</span>)
        #[allow(non_upper_case_globals)]
        pub const Mono: crate::q_locale::Language = crate::q_locale::Language(337);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```TedimChin = 338```</span>)
        #[allow(non_upper_case_globals)]
        pub const TedimChin: crate::q_locale::Language = crate::q_locale::Language(338);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```Maithili = 339```</span>)
        #[allow(non_upper_case_globals)]
        pub const Maithili: crate::q_locale::Language = crate::q_locale::Language(339);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```Ahom = 340```</span>)
        #[allow(non_upper_case_globals)]
        pub const Ahom: crate::q_locale::Language = crate::q_locale::Language(340);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```AmericanSignLanguage = 341```</span>)
        #[allow(non_upper_case_globals)]
        pub const AmericanSignLanguage: crate::q_locale::Language = crate::q_locale::Language(341);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```ArdhamagadhiPrakrit = 342```</span>)
        #[allow(non_upper_case_globals)]
        pub const ArdhamagadhiPrakrit: crate::q_locale::Language = crate::q_locale::Language(342);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```Bhojpuri = 343```</span>)
        #[allow(non_upper_case_globals)]
        pub const Bhojpuri: crate::q_locale::Language = crate::q_locale::Language(343);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```HieroglyphicLuwian = 344```</span>)
        #[allow(non_upper_case_globals)]
        pub const HieroglyphicLuwian: crate::q_locale::Language = crate::q_locale::Language(344);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```LiteraryChinese = 345```</span>)
        #[allow(non_upper_case_globals)]
        pub const LiteraryChinese: crate::q_locale::Language = crate::q_locale::Language(345);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```Mazanderani = 346```</span>)
        #[allow(non_upper_case_globals)]
        pub const Mazanderani: crate::q_locale::Language = crate::q_locale::Language(346);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```Mru = 347```</span>)
        #[allow(non_upper_case_globals)]
        pub const Mru: crate::q_locale::Language = crate::q_locale::Language(347);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```Newari = 348```</span>)
        #[allow(non_upper_case_globals)]
        pub const Newari: crate::q_locale::Language = crate::q_locale::Language(348);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```NorthernLuri = 349```</span>)
        #[allow(non_upper_case_globals)]
        pub const NorthernLuri: crate::q_locale::Language = crate::q_locale::Language(349);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```Palauan = 350```</span>)
        #[allow(non_upper_case_globals)]
        pub const Palauan: crate::q_locale::Language = crate::q_locale::Language(350);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```Papiamento = 351```</span>)
        #[allow(non_upper_case_globals)]
        pub const Papiamento: crate::q_locale::Language = crate::q_locale::Language(351);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```Saraiki = 352```</span>)
        #[allow(non_upper_case_globals)]
        pub const Saraiki: crate::q_locale::Language = crate::q_locale::Language(352);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```TokelauLanguage = 353```</span>)
        #[allow(non_upper_case_globals)]
        pub const TokelauLanguage: crate::q_locale::Language = crate::q_locale::Language(353);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```TokPisin = 354```</span>)
        #[allow(non_upper_case_globals)]
        pub const TokPisin: crate::q_locale::Language = crate::q_locale::Language(354);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```TuvaluLanguage = 355```</span>)
        #[allow(non_upper_case_globals)]
        pub const TuvaluLanguage: crate::q_locale::Language = crate::q_locale::Language(355);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```UncodedLanguages = 356```</span>)
        #[allow(non_upper_case_globals)]
        pub const UncodedLanguages: crate::q_locale::Language = crate::q_locale::Language(356);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```Cantonese = 357```</span>)
        #[allow(non_upper_case_globals)]
        pub const Cantonese: crate::q_locale::Language = crate::q_locale::Language(357);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```Osage = 358```</span>)
        #[allow(non_upper_case_globals)]
        pub const Osage: crate::q_locale::Language = crate::q_locale::Language(358);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```Tangut = 359```</span>)
        #[allow(non_upper_case_globals)]
        pub const Tangut: crate::q_locale::Language = crate::q_locale::Language(359);
        /// same as NorwegianBokmal (C++ enum variant: <span style='color: green;'>```Norwegian = 85```</span>)
        #[allow(non_upper_case_globals)]
        pub const Norwegian: crate::q_locale::Language = crate::q_locale::Language(85);
        /// Obsolete, please use Romanian (C++ enum variant: <span style='color: green;'>```Moldavian = 95```</span>)
        #[allow(non_upper_case_globals)]
        pub const Moldavian: crate::q_locale::Language = crate::q_locale::Language(95);
        /// Obsolete, please use Serbian (C++ enum variant: <span style='color: green;'>```SerboCroatian = 100```</span>)
        #[allow(non_upper_case_globals)]
        pub const SerboCroatian: crate::q_locale::Language = crate::q_locale::Language(100);
        /// Obsolete, please use Filipino (C++ enum variant: <span style='color: green;'>```Tagalog = 166```</span>)
        #[allow(non_upper_case_globals)]
        pub const Tagalog: crate::q_locale::Language = crate::q_locale::Language(166);
        /// Obsolete, please use Akan (C++ enum variant: <span style='color: green;'>```Twi = 146```</span>)
        #[allow(non_upper_case_globals)]
        pub const Twi: crate::q_locale::Language = crate::q_locale::Language(146);
        /// Obsolete, please use Oromo (C++ enum variant: <span style='color: green;'>```Afan = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const Afan: crate::q_locale::Language = crate::q_locale::Language(3);
        /// Obsolete, please use Belarusian (C++ enum variant: <span style='color: green;'>```Byelorussian = 22```</span>)
        #[allow(non_upper_case_globals)]
        pub const Byelorussian: crate::q_locale::Language = crate::q_locale::Language(22);
        /// Obsolete, please use Dzongkha (C++ enum variant: <span style='color: green;'>```Bhutani = 16```</span>)
        #[allow(non_upper_case_globals)]
        pub const Bhutani: crate::q_locale::Language = crate::q_locale::Language(16);
        /// Obsolete, please use Khmer (C++ enum variant: <span style='color: green;'>```Cambodian = 23```</span>)
        #[allow(non_upper_case_globals)]
        pub const Cambodian: crate::q_locale::Language = crate::q_locale::Language(23);
        /// Obsolete, please use Rundi (C++ enum variant: <span style='color: green;'>```Kurundi = 68```</span>)
        #[allow(non_upper_case_globals)]
        pub const Kurundi: crate::q_locale::Language = crate::q_locale::Language(68);
        /// Obsolete, please use Romansh (C++ enum variant: <span style='color: green;'>```RhaetoRomance = 94```</span>)
        #[allow(non_upper_case_globals)]
        pub const RhaetoRomance: crate::q_locale::Language = crate::q_locale::Language(94);
        /// Obsolete, please use Nyanja (C++ enum variant: <span style='color: green;'>```Chewa = 165```</span>)
        #[allow(non_upper_case_globals)]
        pub const Chewa: crate::q_locale::Language = crate::q_locale::Language(165);
        /// same as WesternFrisian (C++ enum variant: <span style='color: green;'>```Frisian = 38```</span>)
        #[allow(non_upper_case_globals)]
        pub const Frisian: crate::q_locale::Language = crate::q_locale::Language(38);
        /// Obsolete, please use Uighur (C++ enum variant: <span style='color: green;'>```Uigur = 128```</span>)
        #[allow(non_upper_case_globals)]
        pub const Uigur: crate::q_locale::Language = crate::q_locale::Language(128);
        /// C++ enum variant: <span style='color: green;'>```LastLanguage = 359```</span>
        #[allow(non_upper_case_globals)]
        pub const LastLanguage: crate::q_locale::Language = crate::q_locale::Language(359);
        /// Since Qt 5.12 (C++ enum variant: <span style='color: green;'>```Ido = 360```</span>)
        #[allow(non_upper_case_globals)]
        pub const Ido: crate::q_locale::Language = crate::q_locale::Language(360);
        /// Since Qt 5.12 (C++ enum variant: <span style='color: green;'>```Lojban = 361```</span>)
        #[allow(non_upper_case_globals)]
        pub const Lojban: crate::q_locale::Language = crate::q_locale::Language(361);
        /// Since Qt 5.12 (C++ enum variant: <span style='color: green;'>```Sicilian = 362```</span>)
        #[allow(non_upper_case_globals)]
        pub const Sicilian: crate::q_locale::Language = crate::q_locale::Language(362);
        /// Since Qt 5.12 (C++ enum variant: <span style='color: green;'>```SouthernKurdish = 363```</span>)
        #[allow(non_upper_case_globals)]
        pub const SouthernKurdish: crate::q_locale::Language = crate::q_locale::Language(363);
        /// Since Qt 5.12 (C++ enum variant: <span style='color: green;'>```WesternBalochi = 364```</span>)
        #[allow(non_upper_case_globals)]
        pub const WesternBalochi: crate::q_locale::Language = crate::q_locale::Language(364);
        /// C++ enum variant: <span style='color: green;'>```LastLanguage = 364```</span>
        #[allow(non_upper_case_globals)]
        pub const LastLanguage2: crate::q_locale::Language = crate::q_locale::Language(364);
    }

    /// <p>This enumerated type is used to specify a script.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QLocale::Script```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#Script-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enumerated type is used to specify a script.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#script">script</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#scriptToString">scriptToString</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#languageToString">languageToString</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Script(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Script {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Script(value)
        }
    }

    impl From<Script> for ::std::os::raw::c_int {
        fn from(value: Script) -> Self {
            value.0
        }
    }

    impl Script {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Script {
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```AnyScript = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const AnyScript: crate::q_locale::Script = crate::q_locale::Script(0);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```ArabicScript = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const ArabicScript: crate::q_locale::Script = crate::q_locale::Script(1);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```CyrillicScript = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const CyrillicScript: crate::q_locale::Script = crate::q_locale::Script(2);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```DeseretScript = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const DeseretScript: crate::q_locale::Script = crate::q_locale::Script(3);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```GurmukhiScript = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const GurmukhiScript: crate::q_locale::Script = crate::q_locale::Script(4);
        /// same as SimplifiedChineseScript (C++ enum variant: <span style='color: green;'>```SimplifiedHanScript = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const SimplifiedHanScript: crate::q_locale::Script = crate::q_locale::Script(5);
        /// same as TraditionalChineseScript (C++ enum variant: <span style='color: green;'>```TraditionalHanScript = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const TraditionalHanScript: crate::q_locale::Script = crate::q_locale::Script(6);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```LatinScript = 7```</span>)
        #[allow(non_upper_case_globals)]
        pub const LatinScript: crate::q_locale::Script = crate::q_locale::Script(7);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```MongolianScript = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const MongolianScript: crate::q_locale::Script = crate::q_locale::Script(8);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```TifinaghScript = 9```</span>)
        #[allow(non_upper_case_globals)]
        pub const TifinaghScript: crate::q_locale::Script = crate::q_locale::Script(9);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```ArmenianScript = 10```</span>)
        #[allow(non_upper_case_globals)]
        pub const ArmenianScript: crate::q_locale::Script = crate::q_locale::Script(10);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```BengaliScript = 11```</span>)
        #[allow(non_upper_case_globals)]
        pub const BengaliScript: crate::q_locale::Script = crate::q_locale::Script(11);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```CherokeeScript = 12```</span>)
        #[allow(non_upper_case_globals)]
        pub const CherokeeScript: crate::q_locale::Script = crate::q_locale::Script(12);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```DevanagariScript = 13```</span>)
        #[allow(non_upper_case_globals)]
        pub const DevanagariScript: crate::q_locale::Script = crate::q_locale::Script(13);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```EthiopicScript = 14```</span>)
        #[allow(non_upper_case_globals)]
        pub const EthiopicScript: crate::q_locale::Script = crate::q_locale::Script(14);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```GeorgianScript = 15```</span>)
        #[allow(non_upper_case_globals)]
        pub const GeorgianScript: crate::q_locale::Script = crate::q_locale::Script(15);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```GreekScript = 16```</span>)
        #[allow(non_upper_case_globals)]
        pub const GreekScript: crate::q_locale::Script = crate::q_locale::Script(16);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```GujaratiScript = 17```</span>)
        #[allow(non_upper_case_globals)]
        pub const GujaratiScript: crate::q_locale::Script = crate::q_locale::Script(17);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```HebrewScript = 18```</span>)
        #[allow(non_upper_case_globals)]
        pub const HebrewScript: crate::q_locale::Script = crate::q_locale::Script(18);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```JapaneseScript = 19```</span>)
        #[allow(non_upper_case_globals)]
        pub const JapaneseScript: crate::q_locale::Script = crate::q_locale::Script(19);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```KhmerScript = 20```</span>)
        #[allow(non_upper_case_globals)]
        pub const KhmerScript: crate::q_locale::Script = crate::q_locale::Script(20);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```KannadaScript = 21```</span>)
        #[allow(non_upper_case_globals)]
        pub const KannadaScript: crate::q_locale::Script = crate::q_locale::Script(21);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```KoreanScript = 22```</span>)
        #[allow(non_upper_case_globals)]
        pub const KoreanScript: crate::q_locale::Script = crate::q_locale::Script(22);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```LaoScript = 23```</span>)
        #[allow(non_upper_case_globals)]
        pub const LaoScript: crate::q_locale::Script = crate::q_locale::Script(23);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```MalayalamScript = 24```</span>)
        #[allow(non_upper_case_globals)]
        pub const MalayalamScript: crate::q_locale::Script = crate::q_locale::Script(24);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```MyanmarScript = 25```</span>)
        #[allow(non_upper_case_globals)]
        pub const MyanmarScript: crate::q_locale::Script = crate::q_locale::Script(25);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```OriyaScript = 26```</span>)
        #[allow(non_upper_case_globals)]
        pub const OriyaScript: crate::q_locale::Script = crate::q_locale::Script(26);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```TamilScript = 27```</span>)
        #[allow(non_upper_case_globals)]
        pub const TamilScript: crate::q_locale::Script = crate::q_locale::Script(27);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```TeluguScript = 28```</span>)
        #[allow(non_upper_case_globals)]
        pub const TeluguScript: crate::q_locale::Script = crate::q_locale::Script(28);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```ThaanaScript = 29```</span>)
        #[allow(non_upper_case_globals)]
        pub const ThaanaScript: crate::q_locale::Script = crate::q_locale::Script(29);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```ThaiScript = 30```</span>)
        #[allow(non_upper_case_globals)]
        pub const ThaiScript: crate::q_locale::Script = crate::q_locale::Script(30);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```TibetanScript = 31```</span>)
        #[allow(non_upper_case_globals)]
        pub const TibetanScript: crate::q_locale::Script = crate::q_locale::Script(31);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SinhalaScript = 32```</span>)
        #[allow(non_upper_case_globals)]
        pub const SinhalaScript: crate::q_locale::Script = crate::q_locale::Script(32);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SyriacScript = 33```</span>)
        #[allow(non_upper_case_globals)]
        pub const SyriacScript: crate::q_locale::Script = crate::q_locale::Script(33);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```YiScript = 34```</span>)
        #[allow(non_upper_case_globals)]
        pub const YiScript: crate::q_locale::Script = crate::q_locale::Script(34);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```VaiScript = 35```</span>)
        #[allow(non_upper_case_globals)]
        pub const VaiScript: crate::q_locale::Script = crate::q_locale::Script(35);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```AvestanScript = 36```</span>)
        #[allow(non_upper_case_globals)]
        pub const AvestanScript: crate::q_locale::Script = crate::q_locale::Script(36);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```BalineseScript = 37```</span>)
        #[allow(non_upper_case_globals)]
        pub const BalineseScript: crate::q_locale::Script = crate::q_locale::Script(37);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```BamumScript = 38```</span>)
        #[allow(non_upper_case_globals)]
        pub const BamumScript: crate::q_locale::Script = crate::q_locale::Script(38);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```BatakScript = 39```</span>)
        #[allow(non_upper_case_globals)]
        pub const BatakScript: crate::q_locale::Script = crate::q_locale::Script(39);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```BopomofoScript = 40```</span>)
        #[allow(non_upper_case_globals)]
        pub const BopomofoScript: crate::q_locale::Script = crate::q_locale::Script(40);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```BrahmiScript = 41```</span>)
        #[allow(non_upper_case_globals)]
        pub const BrahmiScript: crate::q_locale::Script = crate::q_locale::Script(41);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```BugineseScript = 42```</span>)
        #[allow(non_upper_case_globals)]
        pub const BugineseScript: crate::q_locale::Script = crate::q_locale::Script(42);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```BuhidScript = 43```</span>)
        #[allow(non_upper_case_globals)]
        pub const BuhidScript: crate::q_locale::Script = crate::q_locale::Script(43);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```CanadianAboriginalScript = 44```</span>)
        #[allow(non_upper_case_globals)]
        pub const CanadianAboriginalScript: crate::q_locale::Script = crate::q_locale::Script(44);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```CarianScript = 45```</span>)
        #[allow(non_upper_case_globals)]
        pub const CarianScript: crate::q_locale::Script = crate::q_locale::Script(45);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```ChakmaScript = 46```</span>)
        #[allow(non_upper_case_globals)]
        pub const ChakmaScript: crate::q_locale::Script = crate::q_locale::Script(46);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```ChamScript = 47```</span>)
        #[allow(non_upper_case_globals)]
        pub const ChamScript: crate::q_locale::Script = crate::q_locale::Script(47);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```CopticScript = 48```</span>)
        #[allow(non_upper_case_globals)]
        pub const CopticScript: crate::q_locale::Script = crate::q_locale::Script(48);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```CypriotScript = 49```</span>)
        #[allow(non_upper_case_globals)]
        pub const CypriotScript: crate::q_locale::Script = crate::q_locale::Script(49);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```EgyptianHieroglyphsScript = 50```</span>)
        #[allow(non_upper_case_globals)]
        pub const EgyptianHieroglyphsScript: crate::q_locale::Script = crate::q_locale::Script(50);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```FraserScript = 51```</span>)
        #[allow(non_upper_case_globals)]
        pub const FraserScript: crate::q_locale::Script = crate::q_locale::Script(51);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```GlagoliticScript = 52```</span>)
        #[allow(non_upper_case_globals)]
        pub const GlagoliticScript: crate::q_locale::Script = crate::q_locale::Script(52);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```GothicScript = 53```</span>)
        #[allow(non_upper_case_globals)]
        pub const GothicScript: crate::q_locale::Script = crate::q_locale::Script(53);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```HanScript = 54```</span>)
        #[allow(non_upper_case_globals)]
        pub const HanScript: crate::q_locale::Script = crate::q_locale::Script(54);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```HangulScript = 55```</span>)
        #[allow(non_upper_case_globals)]
        pub const HangulScript: crate::q_locale::Script = crate::q_locale::Script(55);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```HanunooScript = 56```</span>)
        #[allow(non_upper_case_globals)]
        pub const HanunooScript: crate::q_locale::Script = crate::q_locale::Script(56);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```ImperialAramaicScript = 57```</span>)
        #[allow(non_upper_case_globals)]
        pub const ImperialAramaicScript: crate::q_locale::Script = crate::q_locale::Script(57);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```InscriptionalPahlaviScript = 58```</span>)
        #[allow(non_upper_case_globals)]
        pub const InscriptionalPahlaviScript: crate::q_locale::Script = crate::q_locale::Script(58);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```InscriptionalParthianScript = 59```</span>)
        #[allow(non_upper_case_globals)]
        pub const InscriptionalParthianScript: crate::q_locale::Script =
            crate::q_locale::Script(59);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```JavaneseScript = 60```</span>)
        #[allow(non_upper_case_globals)]
        pub const JavaneseScript: crate::q_locale::Script = crate::q_locale::Script(60);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```KaithiScript = 61```</span>)
        #[allow(non_upper_case_globals)]
        pub const KaithiScript: crate::q_locale::Script = crate::q_locale::Script(61);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```KatakanaScript = 62```</span>)
        #[allow(non_upper_case_globals)]
        pub const KatakanaScript: crate::q_locale::Script = crate::q_locale::Script(62);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```KayahLiScript = 63```</span>)
        #[allow(non_upper_case_globals)]
        pub const KayahLiScript: crate::q_locale::Script = crate::q_locale::Script(63);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```KharoshthiScript = 64```</span>)
        #[allow(non_upper_case_globals)]
        pub const KharoshthiScript: crate::q_locale::Script = crate::q_locale::Script(64);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```LannaScript = 65```</span>)
        #[allow(non_upper_case_globals)]
        pub const LannaScript: crate::q_locale::Script = crate::q_locale::Script(65);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```LepchaScript = 66```</span>)
        #[allow(non_upper_case_globals)]
        pub const LepchaScript: crate::q_locale::Script = crate::q_locale::Script(66);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```LimbuScript = 67```</span>)
        #[allow(non_upper_case_globals)]
        pub const LimbuScript: crate::q_locale::Script = crate::q_locale::Script(67);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```LinearBScript = 68```</span>)
        #[allow(non_upper_case_globals)]
        pub const LinearBScript: crate::q_locale::Script = crate::q_locale::Script(68);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```LycianScript = 69```</span>)
        #[allow(non_upper_case_globals)]
        pub const LycianScript: crate::q_locale::Script = crate::q_locale::Script(69);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```LydianScript = 70```</span>)
        #[allow(non_upper_case_globals)]
        pub const LydianScript: crate::q_locale::Script = crate::q_locale::Script(70);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```MandaeanScript = 71```</span>)
        #[allow(non_upper_case_globals)]
        pub const MandaeanScript: crate::q_locale::Script = crate::q_locale::Script(71);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```MeiteiMayekScript = 72```</span>)
        #[allow(non_upper_case_globals)]
        pub const MeiteiMayekScript: crate::q_locale::Script = crate::q_locale::Script(72);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```MeroiticScript = 73```</span>)
        #[allow(non_upper_case_globals)]
        pub const MeroiticScript: crate::q_locale::Script = crate::q_locale::Script(73);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```MeroiticCursiveScript = 74```</span>)
        #[allow(non_upper_case_globals)]
        pub const MeroiticCursiveScript: crate::q_locale::Script = crate::q_locale::Script(74);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```NkoScript = 75```</span>)
        #[allow(non_upper_case_globals)]
        pub const NkoScript: crate::q_locale::Script = crate::q_locale::Script(75);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```NewTaiLueScript = 76```</span>)
        #[allow(non_upper_case_globals)]
        pub const NewTaiLueScript: crate::q_locale::Script = crate::q_locale::Script(76);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```OghamScript = 77```</span>)
        #[allow(non_upper_case_globals)]
        pub const OghamScript: crate::q_locale::Script = crate::q_locale::Script(77);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```OlChikiScript = 78```</span>)
        #[allow(non_upper_case_globals)]
        pub const OlChikiScript: crate::q_locale::Script = crate::q_locale::Script(78);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```OldItalicScript = 79```</span>)
        #[allow(non_upper_case_globals)]
        pub const OldItalicScript: crate::q_locale::Script = crate::q_locale::Script(79);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```OldPersianScript = 80```</span>)
        #[allow(non_upper_case_globals)]
        pub const OldPersianScript: crate::q_locale::Script = crate::q_locale::Script(80);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```OldSouthArabianScript = 81```</span>)
        #[allow(non_upper_case_globals)]
        pub const OldSouthArabianScript: crate::q_locale::Script = crate::q_locale::Script(81);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```OrkhonScript = 82```</span>)
        #[allow(non_upper_case_globals)]
        pub const OrkhonScript: crate::q_locale::Script = crate::q_locale::Script(82);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```OsmanyaScript = 83```</span>)
        #[allow(non_upper_case_globals)]
        pub const OsmanyaScript: crate::q_locale::Script = crate::q_locale::Script(83);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```PhagsPaScript = 84```</span>)
        #[allow(non_upper_case_globals)]
        pub const PhagsPaScript: crate::q_locale::Script = crate::q_locale::Script(84);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```PhoenicianScript = 85```</span>)
        #[allow(non_upper_case_globals)]
        pub const PhoenicianScript: crate::q_locale::Script = crate::q_locale::Script(85);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```PollardPhoneticScript = 86```</span>)
        #[allow(non_upper_case_globals)]
        pub const PollardPhoneticScript: crate::q_locale::Script = crate::q_locale::Script(86);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```RejangScript = 87```</span>)
        #[allow(non_upper_case_globals)]
        pub const RejangScript: crate::q_locale::Script = crate::q_locale::Script(87);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```RunicScript = 88```</span>)
        #[allow(non_upper_case_globals)]
        pub const RunicScript: crate::q_locale::Script = crate::q_locale::Script(88);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SamaritanScript = 89```</span>)
        #[allow(non_upper_case_globals)]
        pub const SamaritanScript: crate::q_locale::Script = crate::q_locale::Script(89);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SaurashtraScript = 90```</span>)
        #[allow(non_upper_case_globals)]
        pub const SaurashtraScript: crate::q_locale::Script = crate::q_locale::Script(90);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SharadaScript = 91```</span>)
        #[allow(non_upper_case_globals)]
        pub const SharadaScript: crate::q_locale::Script = crate::q_locale::Script(91);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```ShavianScript = 92```</span>)
        #[allow(non_upper_case_globals)]
        pub const ShavianScript: crate::q_locale::Script = crate::q_locale::Script(92);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SoraSompengScript = 93```</span>)
        #[allow(non_upper_case_globals)]
        pub const SoraSompengScript: crate::q_locale::Script = crate::q_locale::Script(93);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```CuneiformScript = 94```</span>)
        #[allow(non_upper_case_globals)]
        pub const CuneiformScript: crate::q_locale::Script = crate::q_locale::Script(94);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SundaneseScript = 95```</span>)
        #[allow(non_upper_case_globals)]
        pub const SundaneseScript: crate::q_locale::Script = crate::q_locale::Script(95);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SylotiNagriScript = 96```</span>)
        #[allow(non_upper_case_globals)]
        pub const SylotiNagriScript: crate::q_locale::Script = crate::q_locale::Script(96);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```TagalogScript = 97```</span>)
        #[allow(non_upper_case_globals)]
        pub const TagalogScript: crate::q_locale::Script = crate::q_locale::Script(97);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```TagbanwaScript = 98```</span>)
        #[allow(non_upper_case_globals)]
        pub const TagbanwaScript: crate::q_locale::Script = crate::q_locale::Script(98);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```TaiLeScript = 99```</span>)
        #[allow(non_upper_case_globals)]
        pub const TaiLeScript: crate::q_locale::Script = crate::q_locale::Script(99);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```TaiVietScript = 100```</span>)
        #[allow(non_upper_case_globals)]
        pub const TaiVietScript: crate::q_locale::Script = crate::q_locale::Script(100);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```TakriScript = 101```</span>)
        #[allow(non_upper_case_globals)]
        pub const TakriScript: crate::q_locale::Script = crate::q_locale::Script(101);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```UgariticScript = 102```</span>)
        #[allow(non_upper_case_globals)]
        pub const UgariticScript: crate::q_locale::Script = crate::q_locale::Script(102);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```BrailleScript = 103```</span>)
        #[allow(non_upper_case_globals)]
        pub const BrailleScript: crate::q_locale::Script = crate::q_locale::Script(103);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```HiraganaScript = 104```</span>)
        #[allow(non_upper_case_globals)]
        pub const HiraganaScript: crate::q_locale::Script = crate::q_locale::Script(104);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```CaucasianAlbanianScript = 105```</span>)
        #[allow(non_upper_case_globals)]
        pub const CaucasianAlbanianScript: crate::q_locale::Script = crate::q_locale::Script(105);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```BassaVahScript = 106```</span>)
        #[allow(non_upper_case_globals)]
        pub const BassaVahScript: crate::q_locale::Script = crate::q_locale::Script(106);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```DuployanScript = 107```</span>)
        #[allow(non_upper_case_globals)]
        pub const DuployanScript: crate::q_locale::Script = crate::q_locale::Script(107);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```ElbasanScript = 108```</span>)
        #[allow(non_upper_case_globals)]
        pub const ElbasanScript: crate::q_locale::Script = crate::q_locale::Script(108);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```GranthaScript = 109```</span>)
        #[allow(non_upper_case_globals)]
        pub const GranthaScript: crate::q_locale::Script = crate::q_locale::Script(109);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```PahawhHmongScript = 110```</span>)
        #[allow(non_upper_case_globals)]
        pub const PahawhHmongScript: crate::q_locale::Script = crate::q_locale::Script(110);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```KhojkiScript = 111```</span>)
        #[allow(non_upper_case_globals)]
        pub const KhojkiScript: crate::q_locale::Script = crate::q_locale::Script(111);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```LinearAScript = 112```</span>)
        #[allow(non_upper_case_globals)]
        pub const LinearAScript: crate::q_locale::Script = crate::q_locale::Script(112);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```MahajaniScript = 113```</span>)
        #[allow(non_upper_case_globals)]
        pub const MahajaniScript: crate::q_locale::Script = crate::q_locale::Script(113);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```ManichaeanScript = 114```</span>)
        #[allow(non_upper_case_globals)]
        pub const ManichaeanScript: crate::q_locale::Script = crate::q_locale::Script(114);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```MendeKikakuiScript = 115```</span>)
        #[allow(non_upper_case_globals)]
        pub const MendeKikakuiScript: crate::q_locale::Script = crate::q_locale::Script(115);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```ModiScript = 116```</span>)
        #[allow(non_upper_case_globals)]
        pub const ModiScript: crate::q_locale::Script = crate::q_locale::Script(116);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```MroScript = 117```</span>)
        #[allow(non_upper_case_globals)]
        pub const MroScript: crate::q_locale::Script = crate::q_locale::Script(117);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```OldNorthArabianScript = 118```</span>)
        #[allow(non_upper_case_globals)]
        pub const OldNorthArabianScript: crate::q_locale::Script = crate::q_locale::Script(118);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```NabataeanScript = 119```</span>)
        #[allow(non_upper_case_globals)]
        pub const NabataeanScript: crate::q_locale::Script = crate::q_locale::Script(119);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```PalmyreneScript = 120```</span>)
        #[allow(non_upper_case_globals)]
        pub const PalmyreneScript: crate::q_locale::Script = crate::q_locale::Script(120);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```PauCinHauScript = 121```</span>)
        #[allow(non_upper_case_globals)]
        pub const PauCinHauScript: crate::q_locale::Script = crate::q_locale::Script(121);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```OldPermicScript = 122```</span>)
        #[allow(non_upper_case_globals)]
        pub const OldPermicScript: crate::q_locale::Script = crate::q_locale::Script(122);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```PsalterPahlaviScript = 123```</span>)
        #[allow(non_upper_case_globals)]
        pub const PsalterPahlaviScript: crate::q_locale::Script = crate::q_locale::Script(123);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```SiddhamScript = 124```</span>)
        #[allow(non_upper_case_globals)]
        pub const SiddhamScript: crate::q_locale::Script = crate::q_locale::Script(124);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```KhudawadiScript = 125```</span>)
        #[allow(non_upper_case_globals)]
        pub const KhudawadiScript: crate::q_locale::Script = crate::q_locale::Script(125);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```TirhutaScript = 126```</span>)
        #[allow(non_upper_case_globals)]
        pub const TirhutaScript: crate::q_locale::Script = crate::q_locale::Script(126);
        /// Since Qt 5.5 (C++ enum variant: <span style='color: green;'>```VarangKshitiScript = 127```</span>)
        #[allow(non_upper_case_globals)]
        pub const VarangKshitiScript: crate::q_locale::Script = crate::q_locale::Script(127);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```AhomScript = 128```</span>)
        #[allow(non_upper_case_globals)]
        pub const AhomScript: crate::q_locale::Script = crate::q_locale::Script(128);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```AnatolianHieroglyphsScript = 129```</span>)
        #[allow(non_upper_case_globals)]
        pub const AnatolianHieroglyphsScript: crate::q_locale::Script =
            crate::q_locale::Script(129);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```HatranScript = 130```</span>)
        #[allow(non_upper_case_globals)]
        pub const HatranScript: crate::q_locale::Script = crate::q_locale::Script(130);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```MultaniScript = 131```</span>)
        #[allow(non_upper_case_globals)]
        pub const MultaniScript: crate::q_locale::Script = crate::q_locale::Script(131);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```OldHungarianScript = 132```</span>)
        #[allow(non_upper_case_globals)]
        pub const OldHungarianScript: crate::q_locale::Script = crate::q_locale::Script(132);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```SignWritingScript = 133```</span>)
        #[allow(non_upper_case_globals)]
        pub const SignWritingScript: crate::q_locale::Script = crate::q_locale::Script(133);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```AdlamScript = 134```</span>)
        #[allow(non_upper_case_globals)]
        pub const AdlamScript: crate::q_locale::Script = crate::q_locale::Script(134);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```BhaiksukiScript = 135```</span>)
        #[allow(non_upper_case_globals)]
        pub const BhaiksukiScript: crate::q_locale::Script = crate::q_locale::Script(135);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```MarchenScript = 136```</span>)
        #[allow(non_upper_case_globals)]
        pub const MarchenScript: crate::q_locale::Script = crate::q_locale::Script(136);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```NewaScript = 137```</span>)
        #[allow(non_upper_case_globals)]
        pub const NewaScript: crate::q_locale::Script = crate::q_locale::Script(137);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```OsageScript = 138```</span>)
        #[allow(non_upper_case_globals)]
        pub const OsageScript: crate::q_locale::Script = crate::q_locale::Script(138);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```TangutScript = 139```</span>)
        #[allow(non_upper_case_globals)]
        pub const TangutScript: crate::q_locale::Script = crate::q_locale::Script(139);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```HanWithBopomofoScript = 140```</span>)
        #[allow(non_upper_case_globals)]
        pub const HanWithBopomofoScript: crate::q_locale::Script = crate::q_locale::Script(140);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```JamoScript = 141```</span>)
        #[allow(non_upper_case_globals)]
        pub const JamoScript: crate::q_locale::Script = crate::q_locale::Script(141);
        /// same as SimplifiedHanScript (C++ enum variant: <span style='color: green;'>```SimplifiedChineseScript = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const SimplifiedChineseScript: crate::q_locale::Script = crate::q_locale::Script(5);
        /// same as TraditionalHanScript (C++ enum variant: <span style='color: green;'>```TraditionalChineseScript = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const TraditionalChineseScript: crate::q_locale::Script = crate::q_locale::Script(6);
        /// C++ enum variant: <span style='color: green;'>```LastScript = 141```</span>
        #[allow(non_upper_case_globals)]
        pub const LastScript: crate::q_locale::Script = crate::q_locale::Script(141);
    }

    /// <p>This enumerated type is used to specify a country.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QLocale::Country```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#Country-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enumerated type is used to specify a country.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#country">country</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#countryToString">countryToString</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Country(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Country {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Country(value)
        }
    }

    impl From<Country> for ::std::os::raw::c_int {
        fn from(value: Country) -> Self {
            value.0
        }
    }

    impl Country {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Country {
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```AnyCountry = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const AnyCountry: crate::q_locale::Country = crate::q_locale::Country(0);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Afghanistan = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Afghanistan: crate::q_locale::Country = crate::q_locale::Country(1);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Albania = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const Albania: crate::q_locale::Country = crate::q_locale::Country(2);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Algeria = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const Algeria: crate::q_locale::Country = crate::q_locale::Country(3);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```AmericanSamoa = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const AmericanSamoa: crate::q_locale::Country = crate::q_locale::Country(4);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Andorra = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const Andorra: crate::q_locale::Country = crate::q_locale::Country(5);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Angola = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const Angola: crate::q_locale::Country = crate::q_locale::Country(6);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Anguilla = 7```</span>)
        #[allow(non_upper_case_globals)]
        pub const Anguilla: crate::q_locale::Country = crate::q_locale::Country(7);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Antarctica = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const Antarctica: crate::q_locale::Country = crate::q_locale::Country(8);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```AntiguaAndBarbuda = 9```</span>)
        #[allow(non_upper_case_globals)]
        pub const AntiguaAndBarbuda: crate::q_locale::Country = crate::q_locale::Country(9);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Argentina = 10```</span>)
        #[allow(non_upper_case_globals)]
        pub const Argentina: crate::q_locale::Country = crate::q_locale::Country(10);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Armenia = 11```</span>)
        #[allow(non_upper_case_globals)]
        pub const Armenia: crate::q_locale::Country = crate::q_locale::Country(11);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Aruba = 12```</span>)
        #[allow(non_upper_case_globals)]
        pub const Aruba: crate::q_locale::Country = crate::q_locale::Country(12);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Australia = 13```</span>)
        #[allow(non_upper_case_globals)]
        pub const Australia: crate::q_locale::Country = crate::q_locale::Country(13);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Austria = 14```</span>)
        #[allow(non_upper_case_globals)]
        pub const Austria: crate::q_locale::Country = crate::q_locale::Country(14);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Azerbaijan = 15```</span>)
        #[allow(non_upper_case_globals)]
        pub const Azerbaijan: crate::q_locale::Country = crate::q_locale::Country(15);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Bahamas = 16```</span>)
        #[allow(non_upper_case_globals)]
        pub const Bahamas: crate::q_locale::Country = crate::q_locale::Country(16);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Bahrain = 17```</span>)
        #[allow(non_upper_case_globals)]
        pub const Bahrain: crate::q_locale::Country = crate::q_locale::Country(17);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Bangladesh = 18```</span>)
        #[allow(non_upper_case_globals)]
        pub const Bangladesh: crate::q_locale::Country = crate::q_locale::Country(18);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Barbados = 19```</span>)
        #[allow(non_upper_case_globals)]
        pub const Barbados: crate::q_locale::Country = crate::q_locale::Country(19);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Belarus = 20```</span>)
        #[allow(non_upper_case_globals)]
        pub const Belarus: crate::q_locale::Country = crate::q_locale::Country(20);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Belgium = 21```</span>)
        #[allow(non_upper_case_globals)]
        pub const Belgium: crate::q_locale::Country = crate::q_locale::Country(21);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Belize = 22```</span>)
        #[allow(non_upper_case_globals)]
        pub const Belize: crate::q_locale::Country = crate::q_locale::Country(22);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Benin = 23```</span>)
        #[allow(non_upper_case_globals)]
        pub const Benin: crate::q_locale::Country = crate::q_locale::Country(23);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Bermuda = 24```</span>)
        #[allow(non_upper_case_globals)]
        pub const Bermuda: crate::q_locale::Country = crate::q_locale::Country(24);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Bhutan = 25```</span>)
        #[allow(non_upper_case_globals)]
        pub const Bhutan: crate::q_locale::Country = crate::q_locale::Country(25);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Bolivia = 26```</span>)
        #[allow(non_upper_case_globals)]
        pub const Bolivia: crate::q_locale::Country = crate::q_locale::Country(26);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```BosniaAndHerzegowina = 27```</span>)
        #[allow(non_upper_case_globals)]
        pub const BosniaAndHerzegowina: crate::q_locale::Country = crate::q_locale::Country(27);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Botswana = 28```</span>)
        #[allow(non_upper_case_globals)]
        pub const Botswana: crate::q_locale::Country = crate::q_locale::Country(28);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```BouvetIsland = 29```</span>)
        #[allow(non_upper_case_globals)]
        pub const BouvetIsland: crate::q_locale::Country = crate::q_locale::Country(29);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Brazil = 30```</span>)
        #[allow(non_upper_case_globals)]
        pub const Brazil: crate::q_locale::Country = crate::q_locale::Country(30);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```BritishIndianOceanTerritory = 31```</span>)
        #[allow(non_upper_case_globals)]
        pub const BritishIndianOceanTerritory: crate::q_locale::Country =
            crate::q_locale::Country(31);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Brunei = 32```</span>)
        #[allow(non_upper_case_globals)]
        pub const Brunei: crate::q_locale::Country = crate::q_locale::Country(32);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Bulgaria = 33```</span>)
        #[allow(non_upper_case_globals)]
        pub const Bulgaria: crate::q_locale::Country = crate::q_locale::Country(33);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```BurkinaFaso = 34```</span>)
        #[allow(non_upper_case_globals)]
        pub const BurkinaFaso: crate::q_locale::Country = crate::q_locale::Country(34);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Burundi = 35```</span>)
        #[allow(non_upper_case_globals)]
        pub const Burundi: crate::q_locale::Country = crate::q_locale::Country(35);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Cambodia = 36```</span>)
        #[allow(non_upper_case_globals)]
        pub const Cambodia: crate::q_locale::Country = crate::q_locale::Country(36);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Cameroon = 37```</span>)
        #[allow(non_upper_case_globals)]
        pub const Cameroon: crate::q_locale::Country = crate::q_locale::Country(37);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Canada = 38```</span>)
        #[allow(non_upper_case_globals)]
        pub const Canada: crate::q_locale::Country = crate::q_locale::Country(38);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```CapeVerde = 39```</span>)
        #[allow(non_upper_case_globals)]
        pub const CapeVerde: crate::q_locale::Country = crate::q_locale::Country(39);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```CaymanIslands = 40```</span>)
        #[allow(non_upper_case_globals)]
        pub const CaymanIslands: crate::q_locale::Country = crate::q_locale::Country(40);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```CentralAfricanRepublic = 41```</span>)
        #[allow(non_upper_case_globals)]
        pub const CentralAfricanRepublic: crate::q_locale::Country = crate::q_locale::Country(41);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Chad = 42```</span>)
        #[allow(non_upper_case_globals)]
        pub const Chad: crate::q_locale::Country = crate::q_locale::Country(42);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Chile = 43```</span>)
        #[allow(non_upper_case_globals)]
        pub const Chile: crate::q_locale::Country = crate::q_locale::Country(43);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```China = 44```</span>)
        #[allow(non_upper_case_globals)]
        pub const China: crate::q_locale::Country = crate::q_locale::Country(44);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```ChristmasIsland = 45```</span>)
        #[allow(non_upper_case_globals)]
        pub const ChristmasIsland: crate::q_locale::Country = crate::q_locale::Country(45);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```CocosIslands = 46```</span>)
        #[allow(non_upper_case_globals)]
        pub const CocosIslands: crate::q_locale::Country = crate::q_locale::Country(46);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Colombia = 47```</span>)
        #[allow(non_upper_case_globals)]
        pub const Colombia: crate::q_locale::Country = crate::q_locale::Country(47);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Comoros = 48```</span>)
        #[allow(non_upper_case_globals)]
        pub const Comoros: crate::q_locale::Country = crate::q_locale::Country(48);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```CongoKinshasa = 49```</span>)
        #[allow(non_upper_case_globals)]
        pub const CongoKinshasa: crate::q_locale::Country = crate::q_locale::Country(49);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```CongoBrazzaville = 50```</span>)
        #[allow(non_upper_case_globals)]
        pub const CongoBrazzaville: crate::q_locale::Country = crate::q_locale::Country(50);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```CookIslands = 51```</span>)
        #[allow(non_upper_case_globals)]
        pub const CookIslands: crate::q_locale::Country = crate::q_locale::Country(51);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```CostaRica = 52```</span>)
        #[allow(non_upper_case_globals)]
        pub const CostaRica: crate::q_locale::Country = crate::q_locale::Country(52);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```IvoryCoast = 53```</span>)
        #[allow(non_upper_case_globals)]
        pub const IvoryCoast: crate::q_locale::Country = crate::q_locale::Country(53);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Croatia = 54```</span>)
        #[allow(non_upper_case_globals)]
        pub const Croatia: crate::q_locale::Country = crate::q_locale::Country(54);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Cuba = 55```</span>)
        #[allow(non_upper_case_globals)]
        pub const Cuba: crate::q_locale::Country = crate::q_locale::Country(55);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Cyprus = 56```</span>)
        #[allow(non_upper_case_globals)]
        pub const Cyprus: crate::q_locale::Country = crate::q_locale::Country(56);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```CzechRepublic = 57```</span>)
        #[allow(non_upper_case_globals)]
        pub const CzechRepublic: crate::q_locale::Country = crate::q_locale::Country(57);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Denmark = 58```</span>)
        #[allow(non_upper_case_globals)]
        pub const Denmark: crate::q_locale::Country = crate::q_locale::Country(58);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Djibouti = 59```</span>)
        #[allow(non_upper_case_globals)]
        pub const Djibouti: crate::q_locale::Country = crate::q_locale::Country(59);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Dominica = 60```</span>)
        #[allow(non_upper_case_globals)]
        pub const Dominica: crate::q_locale::Country = crate::q_locale::Country(60);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```DominicanRepublic = 61```</span>)
        #[allow(non_upper_case_globals)]
        pub const DominicanRepublic: crate::q_locale::Country = crate::q_locale::Country(61);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```EastTimor = 62```</span>)
        #[allow(non_upper_case_globals)]
        pub const EastTimor: crate::q_locale::Country = crate::q_locale::Country(62);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Ecuador = 63```</span>)
        #[allow(non_upper_case_globals)]
        pub const Ecuador: crate::q_locale::Country = crate::q_locale::Country(63);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Egypt = 64```</span>)
        #[allow(non_upper_case_globals)]
        pub const Egypt: crate::q_locale::Country = crate::q_locale::Country(64);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```ElSalvador = 65```</span>)
        #[allow(non_upper_case_globals)]
        pub const ElSalvador: crate::q_locale::Country = crate::q_locale::Country(65);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```EquatorialGuinea = 66```</span>)
        #[allow(non_upper_case_globals)]
        pub const EquatorialGuinea: crate::q_locale::Country = crate::q_locale::Country(66);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Eritrea = 67```</span>)
        #[allow(non_upper_case_globals)]
        pub const Eritrea: crate::q_locale::Country = crate::q_locale::Country(67);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Estonia = 68```</span>)
        #[allow(non_upper_case_globals)]
        pub const Estonia: crate::q_locale::Country = crate::q_locale::Country(68);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Ethiopia = 69```</span>)
        #[allow(non_upper_case_globals)]
        pub const Ethiopia: crate::q_locale::Country = crate::q_locale::Country(69);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```FalklandIslands = 70```</span>)
        #[allow(non_upper_case_globals)]
        pub const FalklandIslands: crate::q_locale::Country = crate::q_locale::Country(70);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```FaroeIslands = 71```</span>)
        #[allow(non_upper_case_globals)]
        pub const FaroeIslands: crate::q_locale::Country = crate::q_locale::Country(71);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Fiji = 72```</span>)
        #[allow(non_upper_case_globals)]
        pub const Fiji: crate::q_locale::Country = crate::q_locale::Country(72);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Finland = 73```</span>)
        #[allow(non_upper_case_globals)]
        pub const Finland: crate::q_locale::Country = crate::q_locale::Country(73);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```France = 74```</span>)
        #[allow(non_upper_case_globals)]
        pub const France: crate::q_locale::Country = crate::q_locale::Country(74);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Guernsey = 75```</span>)
        #[allow(non_upper_case_globals)]
        pub const Guernsey: crate::q_locale::Country = crate::q_locale::Country(75);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```FrenchGuiana = 76```</span>)
        #[allow(non_upper_case_globals)]
        pub const FrenchGuiana: crate::q_locale::Country = crate::q_locale::Country(76);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```FrenchPolynesia = 77```</span>)
        #[allow(non_upper_case_globals)]
        pub const FrenchPolynesia: crate::q_locale::Country = crate::q_locale::Country(77);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```FrenchSouthernTerritories = 78```</span>)
        #[allow(non_upper_case_globals)]
        pub const FrenchSouthernTerritories: crate::q_locale::Country =
            crate::q_locale::Country(78);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Gabon = 79```</span>)
        #[allow(non_upper_case_globals)]
        pub const Gabon: crate::q_locale::Country = crate::q_locale::Country(79);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Gambia = 80```</span>)
        #[allow(non_upper_case_globals)]
        pub const Gambia: crate::q_locale::Country = crate::q_locale::Country(80);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Georgia = 81```</span>)
        #[allow(non_upper_case_globals)]
        pub const Georgia: crate::q_locale::Country = crate::q_locale::Country(81);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Germany = 82```</span>)
        #[allow(non_upper_case_globals)]
        pub const Germany: crate::q_locale::Country = crate::q_locale::Country(82);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Ghana = 83```</span>)
        #[allow(non_upper_case_globals)]
        pub const Ghana: crate::q_locale::Country = crate::q_locale::Country(83);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Gibraltar = 84```</span>)
        #[allow(non_upper_case_globals)]
        pub const Gibraltar: crate::q_locale::Country = crate::q_locale::Country(84);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Greece = 85```</span>)
        #[allow(non_upper_case_globals)]
        pub const Greece: crate::q_locale::Country = crate::q_locale::Country(85);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Greenland = 86```</span>)
        #[allow(non_upper_case_globals)]
        pub const Greenland: crate::q_locale::Country = crate::q_locale::Country(86);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Grenada = 87```</span>)
        #[allow(non_upper_case_globals)]
        pub const Grenada: crate::q_locale::Country = crate::q_locale::Country(87);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Guadeloupe = 88```</span>)
        #[allow(non_upper_case_globals)]
        pub const Guadeloupe: crate::q_locale::Country = crate::q_locale::Country(88);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Guam = 89```</span>)
        #[allow(non_upper_case_globals)]
        pub const Guam: crate::q_locale::Country = crate::q_locale::Country(89);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Guatemala = 90```</span>)
        #[allow(non_upper_case_globals)]
        pub const Guatemala: crate::q_locale::Country = crate::q_locale::Country(90);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Guinea = 91```</span>)
        #[allow(non_upper_case_globals)]
        pub const Guinea: crate::q_locale::Country = crate::q_locale::Country(91);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```GuineaBissau = 92```</span>)
        #[allow(non_upper_case_globals)]
        pub const GuineaBissau: crate::q_locale::Country = crate::q_locale::Country(92);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Guyana = 93```</span>)
        #[allow(non_upper_case_globals)]
        pub const Guyana: crate::q_locale::Country = crate::q_locale::Country(93);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Haiti = 94```</span>)
        #[allow(non_upper_case_globals)]
        pub const Haiti: crate::q_locale::Country = crate::q_locale::Country(94);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```HeardAndMcDonaldIslands = 95```</span>)
        #[allow(non_upper_case_globals)]
        pub const HeardAndMcDonaldIslands: crate::q_locale::Country = crate::q_locale::Country(95);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Honduras = 96```</span>)
        #[allow(non_upper_case_globals)]
        pub const Honduras: crate::q_locale::Country = crate::q_locale::Country(96);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```HongKong = 97```</span>)
        #[allow(non_upper_case_globals)]
        pub const HongKong: crate::q_locale::Country = crate::q_locale::Country(97);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Hungary = 98```</span>)
        #[allow(non_upper_case_globals)]
        pub const Hungary: crate::q_locale::Country = crate::q_locale::Country(98);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Iceland = 99```</span>)
        #[allow(non_upper_case_globals)]
        pub const Iceland: crate::q_locale::Country = crate::q_locale::Country(99);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```India = 100```</span>)
        #[allow(non_upper_case_globals)]
        pub const India: crate::q_locale::Country = crate::q_locale::Country(100);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Indonesia = 101```</span>)
        #[allow(non_upper_case_globals)]
        pub const Indonesia: crate::q_locale::Country = crate::q_locale::Country(101);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Iran = 102```</span>)
        #[allow(non_upper_case_globals)]
        pub const Iran: crate::q_locale::Country = crate::q_locale::Country(102);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Iraq = 103```</span>)
        #[allow(non_upper_case_globals)]
        pub const Iraq: crate::q_locale::Country = crate::q_locale::Country(103);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Ireland = 104```</span>)
        #[allow(non_upper_case_globals)]
        pub const Ireland: crate::q_locale::Country = crate::q_locale::Country(104);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Israel = 105```</span>)
        #[allow(non_upper_case_globals)]
        pub const Israel: crate::q_locale::Country = crate::q_locale::Country(105);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Italy = 106```</span>)
        #[allow(non_upper_case_globals)]
        pub const Italy: crate::q_locale::Country = crate::q_locale::Country(106);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Jamaica = 107```</span>)
        #[allow(non_upper_case_globals)]
        pub const Jamaica: crate::q_locale::Country = crate::q_locale::Country(107);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Japan = 108```</span>)
        #[allow(non_upper_case_globals)]
        pub const Japan: crate::q_locale::Country = crate::q_locale::Country(108);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Jordan = 109```</span>)
        #[allow(non_upper_case_globals)]
        pub const Jordan: crate::q_locale::Country = crate::q_locale::Country(109);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Kazakhstan = 110```</span>)
        #[allow(non_upper_case_globals)]
        pub const Kazakhstan: crate::q_locale::Country = crate::q_locale::Country(110);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Kenya = 111```</span>)
        #[allow(non_upper_case_globals)]
        pub const Kenya: crate::q_locale::Country = crate::q_locale::Country(111);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Kiribati = 112```</span>)
        #[allow(non_upper_case_globals)]
        pub const Kiribati: crate::q_locale::Country = crate::q_locale::Country(112);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```NorthKorea = 113```</span>)
        #[allow(non_upper_case_globals)]
        pub const NorthKorea: crate::q_locale::Country = crate::q_locale::Country(113);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SouthKorea = 114```</span>)
        #[allow(non_upper_case_globals)]
        pub const SouthKorea: crate::q_locale::Country = crate::q_locale::Country(114);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Kuwait = 115```</span>)
        #[allow(non_upper_case_globals)]
        pub const Kuwait: crate::q_locale::Country = crate::q_locale::Country(115);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Kyrgyzstan = 116```</span>)
        #[allow(non_upper_case_globals)]
        pub const Kyrgyzstan: crate::q_locale::Country = crate::q_locale::Country(116);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Laos = 117```</span>)
        #[allow(non_upper_case_globals)]
        pub const Laos: crate::q_locale::Country = crate::q_locale::Country(117);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Latvia = 118```</span>)
        #[allow(non_upper_case_globals)]
        pub const Latvia: crate::q_locale::Country = crate::q_locale::Country(118);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Lebanon = 119```</span>)
        #[allow(non_upper_case_globals)]
        pub const Lebanon: crate::q_locale::Country = crate::q_locale::Country(119);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Lesotho = 120```</span>)
        #[allow(non_upper_case_globals)]
        pub const Lesotho: crate::q_locale::Country = crate::q_locale::Country(120);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Liberia = 121```</span>)
        #[allow(non_upper_case_globals)]
        pub const Liberia: crate::q_locale::Country = crate::q_locale::Country(121);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Libya = 122```</span>)
        #[allow(non_upper_case_globals)]
        pub const Libya: crate::q_locale::Country = crate::q_locale::Country(122);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Liechtenstein = 123```</span>)
        #[allow(non_upper_case_globals)]
        pub const Liechtenstein: crate::q_locale::Country = crate::q_locale::Country(123);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Lithuania = 124```</span>)
        #[allow(non_upper_case_globals)]
        pub const Lithuania: crate::q_locale::Country = crate::q_locale::Country(124);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Luxembourg = 125```</span>)
        #[allow(non_upper_case_globals)]
        pub const Luxembourg: crate::q_locale::Country = crate::q_locale::Country(125);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Macau = 126```</span>)
        #[allow(non_upper_case_globals)]
        pub const Macau: crate::q_locale::Country = crate::q_locale::Country(126);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Macedonia = 127```</span>)
        #[allow(non_upper_case_globals)]
        pub const Macedonia: crate::q_locale::Country = crate::q_locale::Country(127);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Madagascar = 128```</span>)
        #[allow(non_upper_case_globals)]
        pub const Madagascar: crate::q_locale::Country = crate::q_locale::Country(128);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Malawi = 129```</span>)
        #[allow(non_upper_case_globals)]
        pub const Malawi: crate::q_locale::Country = crate::q_locale::Country(129);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Malaysia = 130```</span>)
        #[allow(non_upper_case_globals)]
        pub const Malaysia: crate::q_locale::Country = crate::q_locale::Country(130);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Maldives = 131```</span>)
        #[allow(non_upper_case_globals)]
        pub const Maldives: crate::q_locale::Country = crate::q_locale::Country(131);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Mali = 132```</span>)
        #[allow(non_upper_case_globals)]
        pub const Mali: crate::q_locale::Country = crate::q_locale::Country(132);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Malta = 133```</span>)
        #[allow(non_upper_case_globals)]
        pub const Malta: crate::q_locale::Country = crate::q_locale::Country(133);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```MarshallIslands = 134```</span>)
        #[allow(non_upper_case_globals)]
        pub const MarshallIslands: crate::q_locale::Country = crate::q_locale::Country(134);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Martinique = 135```</span>)
        #[allow(non_upper_case_globals)]
        pub const Martinique: crate::q_locale::Country = crate::q_locale::Country(135);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Mauritania = 136```</span>)
        #[allow(non_upper_case_globals)]
        pub const Mauritania: crate::q_locale::Country = crate::q_locale::Country(136);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Mauritius = 137```</span>)
        #[allow(non_upper_case_globals)]
        pub const Mauritius: crate::q_locale::Country = crate::q_locale::Country(137);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Mayotte = 138```</span>)
        #[allow(non_upper_case_globals)]
        pub const Mayotte: crate::q_locale::Country = crate::q_locale::Country(138);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Mexico = 139```</span>)
        #[allow(non_upper_case_globals)]
        pub const Mexico: crate::q_locale::Country = crate::q_locale::Country(139);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Micronesia = 140```</span>)
        #[allow(non_upper_case_globals)]
        pub const Micronesia: crate::q_locale::Country = crate::q_locale::Country(140);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Moldova = 141```</span>)
        #[allow(non_upper_case_globals)]
        pub const Moldova: crate::q_locale::Country = crate::q_locale::Country(141);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Monaco = 142```</span>)
        #[allow(non_upper_case_globals)]
        pub const Monaco: crate::q_locale::Country = crate::q_locale::Country(142);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Mongolia = 143```</span>)
        #[allow(non_upper_case_globals)]
        pub const Mongolia: crate::q_locale::Country = crate::q_locale::Country(143);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Montserrat = 144```</span>)
        #[allow(non_upper_case_globals)]
        pub const Montserrat: crate::q_locale::Country = crate::q_locale::Country(144);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Morocco = 145```</span>)
        #[allow(non_upper_case_globals)]
        pub const Morocco: crate::q_locale::Country = crate::q_locale::Country(145);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Mozambique = 146```</span>)
        #[allow(non_upper_case_globals)]
        pub const Mozambique: crate::q_locale::Country = crate::q_locale::Country(146);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Myanmar = 147```</span>)
        #[allow(non_upper_case_globals)]
        pub const Myanmar: crate::q_locale::Country = crate::q_locale::Country(147);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Namibia = 148```</span>)
        #[allow(non_upper_case_globals)]
        pub const Namibia: crate::q_locale::Country = crate::q_locale::Country(148);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```NauruCountry = 149```</span>)
        #[allow(non_upper_case_globals)]
        pub const NauruCountry: crate::q_locale::Country = crate::q_locale::Country(149);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Nepal = 150```</span>)
        #[allow(non_upper_case_globals)]
        pub const Nepal: crate::q_locale::Country = crate::q_locale::Country(150);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Netherlands = 151```</span>)
        #[allow(non_upper_case_globals)]
        pub const Netherlands: crate::q_locale::Country = crate::q_locale::Country(151);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```CuraSao = 152```</span>)
        #[allow(non_upper_case_globals)]
        pub const CuraSao: crate::q_locale::Country = crate::q_locale::Country(152);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```NewCaledonia = 153```</span>)
        #[allow(non_upper_case_globals)]
        pub const NewCaledonia: crate::q_locale::Country = crate::q_locale::Country(153);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```NewZealand = 154```</span>)
        #[allow(non_upper_case_globals)]
        pub const NewZealand: crate::q_locale::Country = crate::q_locale::Country(154);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Nicaragua = 155```</span>)
        #[allow(non_upper_case_globals)]
        pub const Nicaragua: crate::q_locale::Country = crate::q_locale::Country(155);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Niger = 156```</span>)
        #[allow(non_upper_case_globals)]
        pub const Niger: crate::q_locale::Country = crate::q_locale::Country(156);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Nigeria = 157```</span>)
        #[allow(non_upper_case_globals)]
        pub const Nigeria: crate::q_locale::Country = crate::q_locale::Country(157);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Niue = 158```</span>)
        #[allow(non_upper_case_globals)]
        pub const Niue: crate::q_locale::Country = crate::q_locale::Country(158);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```NorfolkIsland = 159```</span>)
        #[allow(non_upper_case_globals)]
        pub const NorfolkIsland: crate::q_locale::Country = crate::q_locale::Country(159);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```NorthernMarianaIslands = 160```</span>)
        #[allow(non_upper_case_globals)]
        pub const NorthernMarianaIslands: crate::q_locale::Country = crate::q_locale::Country(160);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Norway = 161```</span>)
        #[allow(non_upper_case_globals)]
        pub const Norway: crate::q_locale::Country = crate::q_locale::Country(161);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Oman = 162```</span>)
        #[allow(non_upper_case_globals)]
        pub const Oman: crate::q_locale::Country = crate::q_locale::Country(162);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Pakistan = 163```</span>)
        #[allow(non_upper_case_globals)]
        pub const Pakistan: crate::q_locale::Country = crate::q_locale::Country(163);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Palau = 164```</span>)
        #[allow(non_upper_case_globals)]
        pub const Palau: crate::q_locale::Country = crate::q_locale::Country(164);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```PalestinianTerritories = 165```</span>)
        #[allow(non_upper_case_globals)]
        pub const PalestinianTerritories: crate::q_locale::Country = crate::q_locale::Country(165);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Panama = 166```</span>)
        #[allow(non_upper_case_globals)]
        pub const Panama: crate::q_locale::Country = crate::q_locale::Country(166);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```PapuaNewGuinea = 167```</span>)
        #[allow(non_upper_case_globals)]
        pub const PapuaNewGuinea: crate::q_locale::Country = crate::q_locale::Country(167);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Paraguay = 168```</span>)
        #[allow(non_upper_case_globals)]
        pub const Paraguay: crate::q_locale::Country = crate::q_locale::Country(168);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Peru = 169```</span>)
        #[allow(non_upper_case_globals)]
        pub const Peru: crate::q_locale::Country = crate::q_locale::Country(169);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Philippines = 170```</span>)
        #[allow(non_upper_case_globals)]
        pub const Philippines: crate::q_locale::Country = crate::q_locale::Country(170);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Pitcairn = 171```</span>)
        #[allow(non_upper_case_globals)]
        pub const Pitcairn: crate::q_locale::Country = crate::q_locale::Country(171);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Poland = 172```</span>)
        #[allow(non_upper_case_globals)]
        pub const Poland: crate::q_locale::Country = crate::q_locale::Country(172);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Portugal = 173```</span>)
        #[allow(non_upper_case_globals)]
        pub const Portugal: crate::q_locale::Country = crate::q_locale::Country(173);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```PuertoRico = 174```</span>)
        #[allow(non_upper_case_globals)]
        pub const PuertoRico: crate::q_locale::Country = crate::q_locale::Country(174);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Qatar = 175```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qatar: crate::q_locale::Country = crate::q_locale::Country(175);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Reunion = 176```</span>)
        #[allow(non_upper_case_globals)]
        pub const Reunion: crate::q_locale::Country = crate::q_locale::Country(176);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Romania = 177```</span>)
        #[allow(non_upper_case_globals)]
        pub const Romania: crate::q_locale::Country = crate::q_locale::Country(177);
        /// same as RussianFederation (C++ enum variant: <span style='color: green;'>```Russia = 178```</span>)
        #[allow(non_upper_case_globals)]
        pub const Russia: crate::q_locale::Country = crate::q_locale::Country(178);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Rwanda = 179```</span>)
        #[allow(non_upper_case_globals)]
        pub const Rwanda: crate::q_locale::Country = crate::q_locale::Country(179);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SaintKittsAndNevis = 180```</span>)
        #[allow(non_upper_case_globals)]
        pub const SaintKittsAndNevis: crate::q_locale::Country = crate::q_locale::Country(180);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SaintLucia = 181```</span>)
        #[allow(non_upper_case_globals)]
        pub const SaintLucia: crate::q_locale::Country = crate::q_locale::Country(181);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SaintVincentAndTheGrenadines = 182```</span>)
        #[allow(non_upper_case_globals)]
        pub const SaintVincentAndTheGrenadines: crate::q_locale::Country =
            crate::q_locale::Country(182);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Samoa = 183```</span>)
        #[allow(non_upper_case_globals)]
        pub const Samoa: crate::q_locale::Country = crate::q_locale::Country(183);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SanMarino = 184```</span>)
        #[allow(non_upper_case_globals)]
        pub const SanMarino: crate::q_locale::Country = crate::q_locale::Country(184);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SaoTomeAndPrincipe = 185```</span>)
        #[allow(non_upper_case_globals)]
        pub const SaoTomeAndPrincipe: crate::q_locale::Country = crate::q_locale::Country(185);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SaudiArabia = 186```</span>)
        #[allow(non_upper_case_globals)]
        pub const SaudiArabia: crate::q_locale::Country = crate::q_locale::Country(186);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Senegal = 187```</span>)
        #[allow(non_upper_case_globals)]
        pub const Senegal: crate::q_locale::Country = crate::q_locale::Country(187);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Seychelles = 188```</span>)
        #[allow(non_upper_case_globals)]
        pub const Seychelles: crate::q_locale::Country = crate::q_locale::Country(188);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SierraLeone = 189```</span>)
        #[allow(non_upper_case_globals)]
        pub const SierraLeone: crate::q_locale::Country = crate::q_locale::Country(189);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Singapore = 190```</span>)
        #[allow(non_upper_case_globals)]
        pub const Singapore: crate::q_locale::Country = crate::q_locale::Country(190);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Slovakia = 191```</span>)
        #[allow(non_upper_case_globals)]
        pub const Slovakia: crate::q_locale::Country = crate::q_locale::Country(191);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Slovenia = 192```</span>)
        #[allow(non_upper_case_globals)]
        pub const Slovenia: crate::q_locale::Country = crate::q_locale::Country(192);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SolomonIslands = 193```</span>)
        #[allow(non_upper_case_globals)]
        pub const SolomonIslands: crate::q_locale::Country = crate::q_locale::Country(193);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Somalia = 194```</span>)
        #[allow(non_upper_case_globals)]
        pub const Somalia: crate::q_locale::Country = crate::q_locale::Country(194);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SouthAfrica = 195```</span>)
        #[allow(non_upper_case_globals)]
        pub const SouthAfrica: crate::q_locale::Country = crate::q_locale::Country(195);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SouthGeorgiaAndTheSouthSandwichIslands = 196```</span>)
        #[allow(non_upper_case_globals)]
        pub const SouthGeorgiaAndTheSouthSandwichIslands: crate::q_locale::Country =
            crate::q_locale::Country(196);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Spain = 197```</span>)
        #[allow(non_upper_case_globals)]
        pub const Spain: crate::q_locale::Country = crate::q_locale::Country(197);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SriLanka = 198```</span>)
        #[allow(non_upper_case_globals)]
        pub const SriLanka: crate::q_locale::Country = crate::q_locale::Country(198);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SaintHelena = 199```</span>)
        #[allow(non_upper_case_globals)]
        pub const SaintHelena: crate::q_locale::Country = crate::q_locale::Country(199);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SaintPierreAndMiquelon = 200```</span>)
        #[allow(non_upper_case_globals)]
        pub const SaintPierreAndMiquelon: crate::q_locale::Country = crate::q_locale::Country(200);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Sudan = 201```</span>)
        #[allow(non_upper_case_globals)]
        pub const Sudan: crate::q_locale::Country = crate::q_locale::Country(201);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Suriname = 202```</span>)
        #[allow(non_upper_case_globals)]
        pub const Suriname: crate::q_locale::Country = crate::q_locale::Country(202);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SvalbardAndJanMayenIslands = 203```</span>)
        #[allow(non_upper_case_globals)]
        pub const SvalbardAndJanMayenIslands: crate::q_locale::Country =
            crate::q_locale::Country(203);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Swaziland = 204```</span>)
        #[allow(non_upper_case_globals)]
        pub const Swaziland: crate::q_locale::Country = crate::q_locale::Country(204);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Sweden = 205```</span>)
        #[allow(non_upper_case_globals)]
        pub const Sweden: crate::q_locale::Country = crate::q_locale::Country(205);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Switzerland = 206```</span>)
        #[allow(non_upper_case_globals)]
        pub const Switzerland: crate::q_locale::Country = crate::q_locale::Country(206);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Syria = 207```</span>)
        #[allow(non_upper_case_globals)]
        pub const Syria: crate::q_locale::Country = crate::q_locale::Country(207);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Taiwan = 208```</span>)
        #[allow(non_upper_case_globals)]
        pub const Taiwan: crate::q_locale::Country = crate::q_locale::Country(208);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Tajikistan = 209```</span>)
        #[allow(non_upper_case_globals)]
        pub const Tajikistan: crate::q_locale::Country = crate::q_locale::Country(209);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Tanzania = 210```</span>)
        #[allow(non_upper_case_globals)]
        pub const Tanzania: crate::q_locale::Country = crate::q_locale::Country(210);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Thailand = 211```</span>)
        #[allow(non_upper_case_globals)]
        pub const Thailand: crate::q_locale::Country = crate::q_locale::Country(211);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Togo = 212```</span>)
        #[allow(non_upper_case_globals)]
        pub const Togo: crate::q_locale::Country = crate::q_locale::Country(212);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```TokelauCountry = 213```</span>)
        #[allow(non_upper_case_globals)]
        pub const TokelauCountry: crate::q_locale::Country = crate::q_locale::Country(213);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Tonga = 214```</span>)
        #[allow(non_upper_case_globals)]
        pub const Tonga: crate::q_locale::Country = crate::q_locale::Country(214);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```TrinidadAndTobago = 215```</span>)
        #[allow(non_upper_case_globals)]
        pub const TrinidadAndTobago: crate::q_locale::Country = crate::q_locale::Country(215);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Tunisia = 216```</span>)
        #[allow(non_upper_case_globals)]
        pub const Tunisia: crate::q_locale::Country = crate::q_locale::Country(216);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Turkey = 217```</span>)
        #[allow(non_upper_case_globals)]
        pub const Turkey: crate::q_locale::Country = crate::q_locale::Country(217);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Turkmenistan = 218```</span>)
        #[allow(non_upper_case_globals)]
        pub const Turkmenistan: crate::q_locale::Country = crate::q_locale::Country(218);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```TurksAndCaicosIslands = 219```</span>)
        #[allow(non_upper_case_globals)]
        pub const TurksAndCaicosIslands: crate::q_locale::Country = crate::q_locale::Country(219);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```TuvaluCountry = 220```</span>)
        #[allow(non_upper_case_globals)]
        pub const TuvaluCountry: crate::q_locale::Country = crate::q_locale::Country(220);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Uganda = 221```</span>)
        #[allow(non_upper_case_globals)]
        pub const Uganda: crate::q_locale::Country = crate::q_locale::Country(221);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Ukraine = 222```</span>)
        #[allow(non_upper_case_globals)]
        pub const Ukraine: crate::q_locale::Country = crate::q_locale::Country(222);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```UnitedArabEmirates = 223```</span>)
        #[allow(non_upper_case_globals)]
        pub const UnitedArabEmirates: crate::q_locale::Country = crate::q_locale::Country(223);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```UnitedKingdom = 224```</span>)
        #[allow(non_upper_case_globals)]
        pub const UnitedKingdom: crate::q_locale::Country = crate::q_locale::Country(224);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```UnitedStates = 225```</span>)
        #[allow(non_upper_case_globals)]
        pub const UnitedStates: crate::q_locale::Country = crate::q_locale::Country(225);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```UnitedStatesMinorOutlyingIslands = 226```</span>)
        #[allow(non_upper_case_globals)]
        pub const UnitedStatesMinorOutlyingIslands: crate::q_locale::Country =
            crate::q_locale::Country(226);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Uruguay = 227```</span>)
        #[allow(non_upper_case_globals)]
        pub const Uruguay: crate::q_locale::Country = crate::q_locale::Country(227);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Uzbekistan = 228```</span>)
        #[allow(non_upper_case_globals)]
        pub const Uzbekistan: crate::q_locale::Country = crate::q_locale::Country(228);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Vanuatu = 229```</span>)
        #[allow(non_upper_case_globals)]
        pub const Vanuatu: crate::q_locale::Country = crate::q_locale::Country(229);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```VaticanCityState = 230```</span>)
        #[allow(non_upper_case_globals)]
        pub const VaticanCityState: crate::q_locale::Country = crate::q_locale::Country(230);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Venezuela = 231```</span>)
        #[allow(non_upper_case_globals)]
        pub const Venezuela: crate::q_locale::Country = crate::q_locale::Country(231);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Vietnam = 232```</span>)
        #[allow(non_upper_case_globals)]
        pub const Vietnam: crate::q_locale::Country = crate::q_locale::Country(232);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```BritishVirginIslands = 233```</span>)
        #[allow(non_upper_case_globals)]
        pub const BritishVirginIslands: crate::q_locale::Country = crate::q_locale::Country(233);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```UnitedStatesVirginIslands = 234```</span>)
        #[allow(non_upper_case_globals)]
        pub const UnitedStatesVirginIslands: crate::q_locale::Country =
            crate::q_locale::Country(234);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```WallisAndFutunaIslands = 235```</span>)
        #[allow(non_upper_case_globals)]
        pub const WallisAndFutunaIslands: crate::q_locale::Country = crate::q_locale::Country(235);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```WesternSahara = 236```</span>)
        #[allow(non_upper_case_globals)]
        pub const WesternSahara: crate::q_locale::Country = crate::q_locale::Country(236);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Yemen = 237```</span>)
        #[allow(non_upper_case_globals)]
        pub const Yemen: crate::q_locale::Country = crate::q_locale::Country(237);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```CanaryIslands = 238```</span>)
        #[allow(non_upper_case_globals)]
        pub const CanaryIslands: crate::q_locale::Country = crate::q_locale::Country(238);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Zambia = 239```</span>)
        #[allow(non_upper_case_globals)]
        pub const Zambia: crate::q_locale::Country = crate::q_locale::Country(239);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Zimbabwe = 240```</span>)
        #[allow(non_upper_case_globals)]
        pub const Zimbabwe: crate::q_locale::Country = crate::q_locale::Country(240);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```ClippertonIsland = 241```</span>)
        #[allow(non_upper_case_globals)]
        pub const ClippertonIsland: crate::q_locale::Country = crate::q_locale::Country(241);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Montenegro = 242```</span>)
        #[allow(non_upper_case_globals)]
        pub const Montenegro: crate::q_locale::Country = crate::q_locale::Country(242);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Serbia = 243```</span>)
        #[allow(non_upper_case_globals)]
        pub const Serbia: crate::q_locale::Country = crate::q_locale::Country(243);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SaintBarthelemy = 244```</span>)
        #[allow(non_upper_case_globals)]
        pub const SaintBarthelemy: crate::q_locale::Country = crate::q_locale::Country(244);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SaintMartin = 245```</span>)
        #[allow(non_upper_case_globals)]
        pub const SaintMartin: crate::q_locale::Country = crate::q_locale::Country(245);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```LatinAmericaAndTheCaribbean = 246```</span>)
        #[allow(non_upper_case_globals)]
        pub const LatinAmericaAndTheCaribbean: crate::q_locale::Country =
            crate::q_locale::Country(246);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```AscensionIsland = 247```</span>)
        #[allow(non_upper_case_globals)]
        pub const AscensionIsland: crate::q_locale::Country = crate::q_locale::Country(247);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```AlandIslands = 248```</span>)
        #[allow(non_upper_case_globals)]
        pub const AlandIslands: crate::q_locale::Country = crate::q_locale::Country(248);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```DiegoGarcia = 249```</span>)
        #[allow(non_upper_case_globals)]
        pub const DiegoGarcia: crate::q_locale::Country = crate::q_locale::Country(249);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```CeutaAndMelilla = 250```</span>)
        #[allow(non_upper_case_globals)]
        pub const CeutaAndMelilla: crate::q_locale::Country = crate::q_locale::Country(250);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```IsleOfMan = 251```</span>)
        #[allow(non_upper_case_globals)]
        pub const IsleOfMan: crate::q_locale::Country = crate::q_locale::Country(251);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Jersey = 252```</span>)
        #[allow(non_upper_case_globals)]
        pub const Jersey: crate::q_locale::Country = crate::q_locale::Country(252);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```TristanDaCunha = 253```</span>)
        #[allow(non_upper_case_globals)]
        pub const TristanDaCunha: crate::q_locale::Country = crate::q_locale::Country(253);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SouthSudan = 254```</span>)
        #[allow(non_upper_case_globals)]
        pub const SouthSudan: crate::q_locale::Country = crate::q_locale::Country(254);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Bonaire = 255```</span>)
        #[allow(non_upper_case_globals)]
        pub const Bonaire: crate::q_locale::Country = crate::q_locale::Country(255);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```SintMaarten = 256```</span>)
        #[allow(non_upper_case_globals)]
        pub const SintMaarten: crate::q_locale::Country = crate::q_locale::Country(256);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Kosovo = 257```</span>)
        #[allow(non_upper_case_globals)]
        pub const Kosovo: crate::q_locale::Country = crate::q_locale::Country(257);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```EuropeanUnion = 258```</span>)
        #[allow(non_upper_case_globals)]
        pub const EuropeanUnion: crate::q_locale::Country = crate::q_locale::Country(258);
        /// Since Qt 5.7 (C++ enum variant: <span style='color: green;'>```OutlyingOceania = 259```</span>)
        #[allow(non_upper_case_globals)]
        pub const OutlyingOceania: crate::q_locale::Country = crate::q_locale::Country(259);
        /// Obsolete, please use TokelauCountry (C++ enum variant: <span style='color: green;'>```Tokelau = 213```</span>)
        #[allow(non_upper_case_globals)]
        pub const Tokelau: crate::q_locale::Country = crate::q_locale::Country(213);
        /// Obsolete, please use TuvaluCountry (C++ enum variant: <span style='color: green;'>```Tuvalu = 220```</span>)
        #[allow(non_upper_case_globals)]
        pub const Tuvalu: crate::q_locale::Country = crate::q_locale::Country(220);
        /// Obsolete, please use CongoKinshasa (C++ enum variant: <span style='color: green;'>```DemocraticRepublicOfCongo = 49```</span>)
        #[allow(non_upper_case_globals)]
        pub const DemocraticRepublicOfCongo: crate::q_locale::Country =
            crate::q_locale::Country(49);
        /// Obsolete, please use CongoBrazzaville (C++ enum variant: <span style='color: green;'>```PeoplesRepublicOfCongo = 50```</span>)
        #[allow(non_upper_case_globals)]
        pub const PeoplesRepublicOfCongo: crate::q_locale::Country = crate::q_locale::Country(50);
        /// Obsolete, please use NorthKorea (C++ enum variant: <span style='color: green;'>```DemocraticRepublicOfKorea = 113```</span>)
        #[allow(non_upper_case_globals)]
        pub const DemocraticRepublicOfKorea: crate::q_locale::Country =
            crate::q_locale::Country(113);
        /// Obsolete, please use SouthKorea (C++ enum variant: <span style='color: green;'>```RepublicOfKorea = 114```</span>)
        #[allow(non_upper_case_globals)]
        pub const RepublicOfKorea: crate::q_locale::Country = crate::q_locale::Country(114);
        /// same as Russia (C++ enum variant: <span style='color: green;'>```RussianFederation = 178```</span>)
        #[allow(non_upper_case_globals)]
        pub const RussianFederation: crate::q_locale::Country = crate::q_locale::Country(178);
        /// Obsolete, please use Syria (C++ enum variant: <span style='color: green;'>```SyrianArabRepublic = 207```</span>)
        #[allow(non_upper_case_globals)]
        pub const SyrianArabRepublic: crate::q_locale::Country = crate::q_locale::Country(207);
        /// C++ enum variant: <span style='color: green;'>```LastCountry = 259```</span>
        #[allow(non_upper_case_globals)]
        pub const LastCountry: crate::q_locale::Country = crate::q_locale::Country(259);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```LatinAmerica = 246```</span>)
        #[allow(non_upper_case_globals)]
        pub const LatinAmerica: crate::q_locale::Country = crate::q_locale::Country(246);
        /// Since Qt 5.12 (C++ enum variant: <span style='color: green;'>```World = 260```</span>)
        #[allow(non_upper_case_globals)]
        pub const World: crate::q_locale::Country = crate::q_locale::Country(260);
        /// Since Qt 5.12 (C++ enum variant: <span style='color: green;'>```Europe = 261```</span>)
        #[allow(non_upper_case_globals)]
        pub const Europe: crate::q_locale::Country = crate::q_locale::Country(261);
        /// C++ enum variant: <span style='color: green;'>```LastCountry = 261```</span>
        #[allow(non_upper_case_globals)]
        pub const LastCountry2: crate::q_locale::Country = crate::q_locale::Country(261);
    }

    /// <p>This enum defines which units are used for measurement.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QLocale::MeasurementSystem```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#MeasurementSystem-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum defines which units are used for measurement.</p>
    ///
    /// <p>This enum was introduced or modified in  Qt 4.4.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct MeasurementSystem(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for MeasurementSystem {
        fn from(value: ::std::os::raw::c_int) -> Self {
            MeasurementSystem(value)
        }
    }

    impl From<MeasurementSystem> for ::std::os::raw::c_int {
        fn from(value: MeasurementSystem) -> Self {
            value.0
        }
    }

    impl MeasurementSystem {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl MeasurementSystem {
        /// This value indicates metric units, such as meters, centimeters and millimeters. (C++ enum variant: <span style='color: green;'>```MetricSystem = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const MetricSystem: crate::q_locale::MeasurementSystem =
            crate::q_locale::MeasurementSystem(0);
        /// This value indicates imperial units, such as inches and miles as they are used in the United States. (C++ enum variant: <span style='color: green;'>```ImperialUSSystem = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const ImperialUSSystem: crate::q_locale::MeasurementSystem =
            crate::q_locale::MeasurementSystem(1);
        /// This value indicates imperial units, such as inches and miles as they are used in the United Kingdom. (C++ enum variant: <span style='color: green;'>```ImperialUKSystem = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const ImperialUKSystem: crate::q_locale::MeasurementSystem =
            crate::q_locale::MeasurementSystem(2);
        /// Provided for compatibility. Same as ImperialUSSystem (C++ enum variant: <span style='color: green;'>```ImperialSystem = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const ImperialSystem: crate::q_locale::MeasurementSystem =
            crate::q_locale::MeasurementSystem(1);
    }

    /// <p>This enum describes the types of format that can be used when converting <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> and <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> objects to strings.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QLocale::FormatType```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#FormatType-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the types of format that can be used when converting <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> and <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> objects to strings.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct FormatType(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for FormatType {
        fn from(value: ::std::os::raw::c_int) -> Self {
            FormatType(value)
        }
    }

    impl From<FormatType> for ::std::os::raw::c_int {
        fn from(value: FormatType) -> Self {
            value.0
        }
    }

    impl FormatType {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl FormatType {
        /// The long version of day and month names; for example, returning "January" as a month name. (C++ enum variant: <span style='color: green;'>```LongFormat = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const LongFormat: crate::q_locale::FormatType = crate::q_locale::FormatType(0);
        /// The short version of day and month names; for example, returning "Jan" as a month name. (C++ enum variant: <span style='color: green;'>```ShortFormat = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const ShortFormat: crate::q_locale::FormatType = crate::q_locale::FormatType(1);
        /// A special version of day and month names for use when space is limited; for example, returning "J" as a month name. Note that the narrow format might contain the same text for different months and days or it can even be an empty string if the locale doesn't support narrow names, so you should avoid using it for date formatting. Also, for the system locale this format is the same as ShortFormat. (C++ enum variant: <span style='color: green;'>```NarrowFormat = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const NarrowFormat: crate::q_locale::FormatType = crate::q_locale::FormatType(2);
    }

    /// <p>This enum defines a set of options for number-to-string and string-to-number conversions. They can be retrieved with <a href="http://doc.qt.io/qt-5/qlocale.html#numberOptions">numberOptions</a>() and set with <a href="http://doc.qt.io/qt-5/qlocale.html#setNumberOptions">setNumberOptions</a>().</p>
    ///
    /// C++ enum: <span style='color: green;'>```QLocale::NumberOption```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#NumberOption-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum defines a set of options for number-to-string and string-to-number conversions. They can be retrieved with <a href="http://doc.qt.io/qt-5/qlocale.html#numberOptions">numberOptions</a>() and set with <a href="http://doc.qt.io/qt-5/qlocale.html#setNumberOptions">setNumberOptions</a>().</p>
    ///
    /// <p>The NumberOptions type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;NumberOption&gt;. It stores an OR combination of NumberOption values.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#setNumberOptions">setNumberOptions</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#numberOptions">numberOptions</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct NumberOption(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for NumberOption {
        fn from(value: ::std::os::raw::c_int) -> Self {
            NumberOption(value)
        }
    }

    impl From<NumberOption> for ::std::os::raw::c_int {
        fn from(value: NumberOption) -> Self {
            value.0
        }
    }

    impl NumberOption {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl NumberOption {
        /// This option represents the default behavior, with group separators, with one leading zero in single digit exponents, and without trailing zeroes after the decimal dot. (C++ enum variant: <span style='color: green;'>```DefaultNumberOptions = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const DefaultNumberOptions: crate::q_locale::NumberOption =
            crate::q_locale::NumberOption(0);
        /// If this option is set, the number-to-string functions will not insert group separators in their return values. The default is to insert group separators. (C++ enum variant: <span style='color: green;'>```OmitGroupSeparator = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const OmitGroupSeparator: crate::q_locale::NumberOption =
            crate::q_locale::NumberOption(1);
        /// If this option is set, the string-to-number functions will fail if they encounter group separators in their input. The default is to accept numbers containing correctly placed group separators. (C++ enum variant: <span style='color: green;'>```RejectGroupSeparator = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const RejectGroupSeparator: crate::q_locale::NumberOption =
            crate::q_locale::NumberOption(2);
        /// If this option is set, the number-to-string functions will not pad exponents with zeroes when printing floating point numbers in scientific notation. The default is to add one leading zero to single digit exponents. (C++ enum variant: <span style='color: green;'>```OmitLeadingZeroInExponent = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const OmitLeadingZeroInExponent: crate::q_locale::NumberOption =
            crate::q_locale::NumberOption(4);
        /// If this option is set, the string-to-number functions will fail if they encounter an exponent padded with zeroes when parsing a floating point number in scientific notation. The default is to accept such padding. (C++ enum variant: <span style='color: green;'>```RejectLeadingZeroInExponent = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const RejectLeadingZeroInExponent: crate::q_locale::NumberOption =
            crate::q_locale::NumberOption(8);
        /// If this option is set, the number-to-string functions will pad numbers with zeroes to the requested precision in "g" or "most concise" mode, even if the number of significant digits is lower than the requested precision. The default is to omit trailing zeroes. (C++ enum variant: <span style='color: green;'>```IncludeTrailingZeroesAfterDot = 16```</span>)
        #[allow(non_upper_case_globals)]
        pub const IncludeTrailingZeroesAfterDot: crate::q_locale::NumberOption =
            crate::q_locale::NumberOption(16);
        /// If this option is set, the string-to-number functions will fail if they encounter trailing zeroes after the decimal dot when parsing a number in scientific or decimal representation. The default is to accept trailing zeroes. (C++ enum variant: <span style='color: green;'>```RejectTrailingZeroesAfterDot = 32```</span>)
        #[allow(non_upper_case_globals)]
        pub const RejectTrailingZeroesAfterDot: crate::q_locale::NumberOption =
            crate::q_locale::NumberOption(32);
    }

    /// <p>This enum defines constants that can be given as precision to <a href="http://doc.qt.io/qt-5/qstring.html#number">QString::number</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#number">QByteArray::number</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>() when converting floats or doubles, in order to express a variable number of digits as precision.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QLocale::FloatingPointPrecisionOption```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#FloatingPointPrecisionOption-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum defines constants that can be given as precision to <a href="http://doc.qt.io/qt-5/qstring.html#number">QString::number</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#number">QByteArray::number</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>() when converting floats or doubles, in order to express a variable number of digits as precision.</p>
    ///
    /// <p>This enum was introduced or modified in  Qt 5.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>(), <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, and <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct FloatingPointPrecisionOption(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for FloatingPointPrecisionOption {
        fn from(value: ::std::os::raw::c_int) -> Self {
            FloatingPointPrecisionOption(value)
        }
    }

    impl From<FloatingPointPrecisionOption> for ::std::os::raw::c_int {
        fn from(value: FloatingPointPrecisionOption) -> Self {
            value.0
        }
    }

    impl FloatingPointPrecisionOption {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl FloatingPointPrecisionOption {
        /// The conversion algorithm will try to find the shortest accurate representation for the given number. "Accurate" means that you get the exact same number back from an inverse conversion on the generated string representation. (C++ enum variant: <span style='color: green;'>```FloatingPointShortest = -128```</span>)
        #[allow(non_upper_case_globals)]
        pub const FloatingPointShortest: crate::q_locale::FloatingPointPrecisionOption =
            crate::q_locale::FloatingPointPrecisionOption(-128);
    }

    /// <p>Specifies the format of the currency symbol.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QLocale::CurrencySymbolFormat```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#CurrencySymbolFormat-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Specifies the format of the currency symbol.</p>
    ///
    /// <p>This enum was introduced or modified in  Qt 4.8.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct CurrencySymbolFormat(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for CurrencySymbolFormat {
        fn from(value: ::std::os::raw::c_int) -> Self {
            CurrencySymbolFormat(value)
        }
    }

    impl From<CurrencySymbolFormat> for ::std::os::raw::c_int {
        fn from(value: CurrencySymbolFormat) -> Self {
            value.0
        }
    }

    impl CurrencySymbolFormat {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl CurrencySymbolFormat {
        /// a ISO-4217 code of the currency. (C++ enum variant: <span style='color: green;'>```CurrencyIsoCode = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const CurrencyIsoCode: crate::q_locale::CurrencySymbolFormat =
            crate::q_locale::CurrencySymbolFormat(0);
        /// a currency symbol. (C++ enum variant: <span style='color: green;'>```CurrencySymbol = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const CurrencySymbol: crate::q_locale::CurrencySymbolFormat =
            crate::q_locale::CurrencySymbolFormat(1);
        /// a user readable name of the currency. (C++ enum variant: <span style='color: green;'>```CurrencyDisplayName = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const CurrencyDisplayName: crate::q_locale::CurrencySymbolFormat =
            crate::q_locale::CurrencySymbolFormat(2);
    }

    /// <p>This enum defines a set of possible styles for locale specific quotation.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QLocale::QuotationStyle```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#QuotationStyle-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum defines a set of possible styles for locale specific quotation.</p>
    ///
    /// <p>This enum was introduced or modified in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#quoteString">quoteString</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct QuotationStyle(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for QuotationStyle {
        fn from(value: ::std::os::raw::c_int) -> Self {
            QuotationStyle(value)
        }
    }

    impl From<QuotationStyle> for ::std::os::raw::c_int {
        fn from(value: QuotationStyle) -> Self {
            value.0
        }
    }

    impl QuotationStyle {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl QuotationStyle {
        /// If this option is set, the standard quotation marks will be used to quote strings. (C++ enum variant: <span style='color: green;'>```StandardQuotation = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const StandardQuotation: crate::q_locale::QuotationStyle =
            crate::q_locale::QuotationStyle(0);
        /// If this option is set, the alternate quotation marks will be used to quote strings. (C++ enum variant: <span style='color: green;'>```AlternateQuotation = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const AlternateQuotation: crate::q_locale::QuotationStyle =
            crate::q_locale::QuotationStyle(1);
    }

    impl From<crate::q_locale::NumberOption> for crate::QFlags<crate::q_locale::NumberOption> {
        fn from(value: crate::q_locale::NumberOption) -> Self {
            Self::from(value.to_int())
        }
    }

    impl<T: Into<crate::QFlags<crate::q_locale::NumberOption>>> std::ops::BitOr<T>
        for crate::q_locale::NumberOption
    {
        type Output = crate::QFlags<crate::q_locale::NumberOption>;
        fn bitor(self, rhs: T) -> crate::QFlags<crate::q_locale::NumberOption> {
            Into::<crate::QFlags<crate::q_locale::NumberOption>>::into(self) | rhs
        }
    }

    /// <p>Specifies the format for representation of data quantities.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QLocale::DataSizeFormat```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#DataSizeFormat-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Specifies the format for representation of data quantities.</p>
    ///
    /// <p>This enum was introduced or modified in  Qt 5.10.</p>
    /// <p>The DataSizeFormats type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;DataSizeFormat&gt;. It stores an OR combination of DataSizeFormat values.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#formattedDataSize">formattedDataSize</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct DataSizeFormat(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for DataSizeFormat {
        fn from(value: ::std::os::raw::c_int) -> Self {
            DataSizeFormat(value)
        }
    }

    impl From<DataSizeFormat> for ::std::os::raw::c_int {
        fn from(value: DataSizeFormat) -> Self {
            value.0
        }
    }

    impl DataSizeFormat {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl DataSizeFormat {
        /// C++ enum variant: <span style='color: green;'>```DataSizeBase1000 = 1```</span>
        #[allow(non_upper_case_globals)]
        pub const DataSizeBase1000: crate::q_locale::DataSizeFormat =
            crate::q_locale::DataSizeFormat(1);
        /// C++ enum variant: <span style='color: green;'>```DataSizeSIQuantifiers = 2```</span>
        #[allow(non_upper_case_globals)]
        pub const DataSizeSIQuantifiers: crate::q_locale::DataSizeFormat =
            crate::q_locale::DataSizeFormat(2);
        /// format using base 1024 and IEC prefixes: KiB, MiB, GiB, ... (C++ enum variant: <span style='color: green;'>```DataSizeIecFormat = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const DataSizeIecFormat: crate::q_locale::DataSizeFormat =
            crate::q_locale::DataSizeFormat(0);
        /// format using base 1024 and SI prefixes: kB, MB, GB, ... (C++ enum variant: <span style='color: green;'>```DataSizeTraditionalFormat = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const DataSizeTraditionalFormat: crate::q_locale::DataSizeFormat =
            crate::q_locale::DataSizeFormat(2);
        /// format using base 1000 and SI prefixes: kB, MB, GB, ... (C++ enum variant: <span style='color: green;'>```DataSizeSIFormat = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const DataSizeSIFormat: crate::q_locale::DataSizeFormat =
            crate::q_locale::DataSizeFormat(3);
    }

    impl From<crate::q_locale::DataSizeFormat> for crate::QFlags<crate::q_locale::DataSizeFormat> {
        fn from(value: crate::q_locale::DataSizeFormat) -> Self {
            Self::from(value.to_int())
        }
    }

    impl<T: Into<crate::QFlags<crate::q_locale::DataSizeFormat>>> std::ops::BitOr<T>
        for crate::q_locale::DataSizeFormat
    {
        type Output = crate::QFlags<crate::q_locale::DataSizeFormat>;
        fn bitor(self, rhs: T) -> crate::QFlags<crate::q_locale::DataSizeFormat> {
            Into::<crate::QFlags<crate::q_locale::DataSizeFormat>>::into(self) | rhs
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> class converts between numbers and their string representations in various languages.</p>
///
/// C++ class: <span style='color: green;'>```QLocale```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlocale.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> class converts between numbers and their string representations in various languages.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> is initialized with a language/country pair in its constructor and offers number-to-string and string-to-number conversion functions similar to those in <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
/// <p>Example:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span> egyptian(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span><span class="operator">::</span>Arabic<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span><span class="operator">::</span>Egypt);
///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> s1 <span class="operator">=</span> egyptian<span class="operator">.</span>toString(<span class="number">1.571429E</span><span class="operator">+</span><span class="number">07</span><span class="operator">,</span> <span class="char">'e'</span>);
///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> s2 <span class="operator">=</span> egyptian<span class="operator">.</span>toString(<span class="number">10</span>);
///
///   <span class="type">double</span> d <span class="operator">=</span> egyptian<span class="operator">.</span>toDouble(s1);
///   <span class="type">int</span> i <span class="operator">=</span> egyptian<span class="operator">.</span>toInt(s2);
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> supports the concept of a default locale, which is determined from the system's locale settings at application startup. The default locale can be changed by calling the static member <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">setDefault</a>(). Setting the default locale has the following effects:</p>
/// <ul>
/// <li>If a <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object is constructed with the default constructor, it will use the default locale's settings.</li>
/// <li><a href="http://doc.qt.io/qt-5/qstring.html#toInt">QString::toInt</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#toDouble">QString::toDouble</a>(), etc., interpret the string according to the default locale. If this fails, it falls back on the "C" locale.</li>
/// <li><a href="http://doc.qt.io/qt-5/qstring.html#arg">QString::arg</a>() uses the default locale to format a number when its position specifier in the format string contains an 'L', e.g. "%L1".</li>
/// </ul>
/// <p>The following example illustrates how to use <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> directly:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span><span class="operator">::</span>setDefault(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span>(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span><span class="operator">::</span>Hebrew<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span><span class="operator">::</span>Israel));
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span> hebrew; <span class="comment">// Constructs a default QLocale</span>
///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> s1 <span class="operator">=</span> hebrew<span class="operator">.</span>toString(<span class="number">15714.3</span><span class="operator">,</span> <span class="char">'e'</span>);
///
///   bool ok;
///   <span class="type">double</span> d;
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span><span class="operator">::</span>setDefault(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span><span class="operator">::</span>C);
///   d <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>(<span class="string">"1234,56"</span>)<span class="operator">.</span>toDouble(<span class="operator">&amp;</span>ok);   <span class="comment">// ok == false</span>
///   d <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>(<span class="string">"1234.56"</span>)<span class="operator">.</span>toDouble(<span class="operator">&amp;</span>ok);   <span class="comment">// ok == true, d == 1234.56</span>
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span><span class="operator">::</span>setDefault(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span><span class="operator">::</span>German);
///   d <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>(<span class="string">"1234,56"</span>)<span class="operator">.</span>toDouble(<span class="operator">&amp;</span>ok);   <span class="comment">// ok == true, d == 1234.56</span>
///   d <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>(<span class="string">"1234.56"</span>)<span class="operator">.</span>toDouble(<span class="operator">&amp;</span>ok);   <span class="comment">// ok == true, d == 1234.56</span>
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span><span class="operator">::</span>setDefault(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span>(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span><span class="operator">::</span>English<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span><span class="operator">::</span>UnitedStates));
///   str <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>(<span class="string">"%1 %L2 %L3"</span>)
/// &#32;   &#32;   <span class="operator">.</span>arg(<span class="number">12345</span>)<span class="operator">.</span>arg(<span class="number">12345</span>)<span class="operator">.</span>arg(<span class="number">12345</span><span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="number">16</span>);
///   <span class="comment">// str == "12345 12,345 3039"</span>
///
/// </pre>
/// <p>When a language/country pair is specified in the constructor, one of three things can happen:</p>
/// <ul>
/// <li>If the language/country pair is found in the database, it is used.</li>
/// <li>If the language is found but the country is not, or if the country is <code>AnyCountry</code>, the language is used with the most appropriate available country (for example, Germany for German),</li>
/// <li>If neither the language nor the country are found, <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> defaults to the default locale (see <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">setDefault</a>()).</li>
/// </ul>
/// <p>Use <a href="http://doc.qt.io/qt-5/qlocale.html#language">language</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#country">country</a>() to determine the actual language and country values used.</p>
/// <p>An alternative method for constructing a <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object is by specifying the locale name.</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span> korean(<span class="string">"ko"</span>);
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span> swiss(<span class="string">"de_CH"</span>);
///
/// </pre>
/// <p>This constructor converts the locale name to a language/country pair; it does not use the system locale database.</p>
/// <p><b>Note: </b>For the current keyboard input locale take a look at <a href="http://doc.qt.io/qt-5/qinputmethod.html#locale-prop">QInputMethod::locale</a>().</p><p><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a>'s data is based on Common Locale Data Repository v31.0.1.</p></div>
#[repr(C)]
pub struct QLocale {
    _unused: u8,
}
impl QLocale {
    /// <p>Returns the localized name of the "AM" suffix for times specified using the conventions of the 12-hour clock.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::amText() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#amText">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the localized name of the "AM" suffix for times specified using the conventions of the 12-hour clock.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#pmText">pmText</a>().</p></div>
    pub unsafe fn am_text(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_amText(self as *const crate::QLocale);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the dash-separated language, script and country (and possibly other BCP47 fields) of this locale as a string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::bcp47Name() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#bcp47Name">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the dash-separated language, script and country (and possibly other BCP47 fields) of this locale as a string.</p>
    /// <p>Unlike the <a href="http://doc.qt.io/qt-5/qlocale.html#uiLanguages">uiLanguages</a>() the returned value of the bcp47Name() represents the locale name of the <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> data but not the language the user-interface should be in.</p>
    /// <p>This function tries to conform the locale name to BCP47.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#language">language</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#country">country</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#script">script</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#uiLanguages">uiLanguages</a>().</p></div>
    pub unsafe fn bcp47_name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_bcp47Name(self as *const crate::QLocale);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object initialized to the "C" locale.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QLocale QLocale::c()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#c">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object initialized to the "C" locale.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#system">system</a>().</p></div>
    pub unsafe fn c() -> ::cpp_utils::CppBox<crate::QLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_c();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Assigns <i>other</i> to this <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object and returns a reference to this <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale& QLocale::operator=(const QLocale& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#operator-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object and returns a reference to this <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    ) -> ::cpp_utils::MutRef<crate::QLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_operator_1(
            self as *mut crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the list of countries that have entries for <i>language</i> in Qt's locale database. If the result is an empty list, then <i>language</i> is not represented in Qt's locale database.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QList<QLocale::Country> QLocale::countriesForLanguage(QLocale::Language lang)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale-obsolete.html#countriesForLanguage">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the list of countries that have entries for <i>language</i> in Qt's locale database. If the result is an empty list, then <i>language</i> is not represented in Qt's locale database.</p>
    /// <p>This function was introduced in  Qt 4.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#matchingLocales">matchingLocales</a>().</p></div>
    pub unsafe fn countries_for_language(
        lang: crate::q_locale::Language,
    ) -> ::cpp_utils::CppBox<crate::QListOfCountry> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_countriesForLanguage(lang);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the country of this locale.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale::Country QLocale::country() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#country">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the country of this locale.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#language">language</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#script">script</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#countryToString">countryToString</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#bcp47Name">bcp47Name</a>().</p></div>
    pub unsafe fn country(&self) -> crate::q_locale::Country {
        crate::__ffi::ctr_qt_core_ffi_QLocale_country(self as *const crate::QLocale)
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> containing the name of <i>country</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QLocale::countryToString(QLocale::Country country)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#countryToString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> containing the name of <i>country</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#languageToString">languageToString</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#scriptToString">scriptToString</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#country">country</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#bcp47Name">bcp47Name</a>().</p></div>
    pub unsafe fn country_to_string(
        country: crate::q_locale::Country,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_countryToString(country);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a string that represents a join of a given <i>list</i> of strings with a separator defined by the locale.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::createSeparatedList(const QStringList& strl) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#createSeparatedList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string that represents a join of a given <i>list</i> of strings with a separator defined by the locale.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn create_separated_list(
        &self,
        strl: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_createSeparatedList(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(strl)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a currency symbol according to the <i>format</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::currencySymbol(QLocale::CurrencySymbolFormat arg1 = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#currencySymbol">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a currency symbol according to the <i>format</i>.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn currency_symbol_1a(
        &self,
        arg1: crate::q_locale::CurrencySymbolFormat,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_currencySymbol(
            self as *const crate::QLocale,
            arg1,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a currency symbol according to the <i>format</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::currencySymbol() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#currencySymbol">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a currency symbol according to the <i>format</i>.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn currency_symbol_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_currencySymbol1(self as *const crate::QLocale);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the date format used for the current locale.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::dateFormat(QLocale::FormatType format = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#dateFormat">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the date format used for the current locale.</p>
    /// <p>If <i>format</i> is <a href="http://doc.qt.io/qt-5/qlocale.html#FormatType-enum">LongFormat</a> the format will be a long version. Otherwise it uses a shorter version.</p>
    /// <p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#toString">QDate::toString</a>() and <a href="http://doc.qt.io/qt-5/qdate.html#fromString">QDate::fromString</a>().</p></div>
    pub unsafe fn date_format_1a(
        &self,
        format: crate::q_locale::FormatType,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_dateFormat(self as *const crate::QLocale, format);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the date format used for the current locale.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::dateFormat() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#dateFormat">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the date format used for the current locale.</p>
    /// <p>If <i>format</i> is <a href="http://doc.qt.io/qt-5/qlocale.html#FormatType-enum">LongFormat</a> the format will be a long version. Otherwise it uses a shorter version.</p>
    /// <p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#toString">QDate::toString</a>() and <a href="http://doc.qt.io/qt-5/qdate.html#fromString">QDate::fromString</a>().</p></div>
    pub unsafe fn date_format_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_dateFormat1(self as *const crate::QLocale);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the date time format used for the current locale.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::dateTimeFormat(QLocale::FormatType format = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#dateTimeFormat">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the date time format used for the current locale.</p>
    /// <p>If <i>format</i> is <a href="http://doc.qt.io/qt-5/qlocale.html#FormatType-enum">ShortFormat</a> the format will be a short version. Otherwise it uses a longer version.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#toString">QDateTime::toString</a>() and <a href="http://doc.qt.io/qt-5/qdatetime.html#fromString">QDateTime::fromString</a>().</p></div>
    pub unsafe fn date_time_format_1a(
        &self,
        format: crate::q_locale::FormatType,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_dateTimeFormat(
            self as *const crate::QLocale,
            format,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the date time format used for the current locale.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::dateTimeFormat() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#dateTimeFormat">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the date time format used for the current locale.</p>
    /// <p>If <i>format</i> is <a href="http://doc.qt.io/qt-5/qlocale.html#FormatType-enum">ShortFormat</a> the format will be a short version. Otherwise it uses a longer version.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#toString">QDateTime::toString</a>() and <a href="http://doc.qt.io/qt-5/qdatetime.html#fromString">QDateTime::fromString</a>().</p></div>
    pub unsafe fn date_time_format_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_dateTimeFormat1(self as *const crate::QLocale);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the localized name of the <i>day</i> (where 1 represents Monday, 2 represents Tuesday and so on), in the format specified by <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::dayName(int arg1, QLocale::FormatType format = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#dayName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the localized name of the <i>day</i> (where 1 represents Monday, 2 represents Tuesday and so on), in the format specified by <i>type</i>.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#monthName">monthName</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#standaloneDayName">standaloneDayName</a>().</p></div>
    pub unsafe fn day_name_2a(
        &self,
        arg1: ::std::os::raw::c_int,
        format: crate::q_locale::FormatType,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_dayName(
            self as *const crate::QLocale,
            arg1,
            format,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the localized name of the <i>day</i> (where 1 represents Monday, 2 represents Tuesday and so on), in the format specified by <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::dayName(int arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#dayName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the localized name of the <i>day</i> (where 1 represents Monday, 2 represents Tuesday and so on), in the format specified by <i>type</i>.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#monthName">monthName</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#standaloneDayName">standaloneDayName</a>().</p></div>
    pub unsafe fn day_name_1a(
        &self,
        arg1: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_dayName1(self as *const crate::QLocale, arg1);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the decimal point character of this locale.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QLocale::decimalPoint() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#decimalPoint">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the decimal point character of this locale.</p>
    /// <p>This function was introduced in  Qt 4.1.</p></div>
    pub unsafe fn decimal_point(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_decimalPoint(self as *const crate::QLocale);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the exponential character of this locale.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QLocale::exponential() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#exponential">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the exponential character of this locale.</p>
    /// <p>This function was introduced in  Qt 4.1.</p></div>
    pub unsafe fn exponential(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_exponential(self as *const crate::QLocale);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the first day of the week according to the current locale.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```Qt::DayOfWeek QLocale::firstDayOfWeek() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#firstDayOfWeek">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the first day of the week according to the current locale.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn first_day_of_week(&self) -> crate::DayOfWeek {
        crate::__ffi::ctr_qt_core_ffi_QLocale_firstDayOfWeek(self as *const crate::QLocale)
    }

    /// <p>Converts a size in bytes to a human-readable localized string, comprising a number and a quantified unit. The quantifier is chosen such that the number is at least one, and as small as possible. For example if <i>bytes</i> is 16384, <i>precision</i> is 2, and <i>format</i> is <a href="http://doc.qt.io/qt-5/qlocale.html#DataSizeFormat-enum">DataSizeIecFormat</a> (the default), this function returns "16.00 KiB"; for 1330409069609 bytes it returns "1.21 GiB"; and so on. If <i>format</i> is <a href="http://doc.qt.io/qt-5/qlocale.html#DataSizeFormat-enum">DataSizeIecFormat</a> or <a href="http://doc.qt.io/qt-5/qlocale.html#DataSizeFormat-enum">DataSizeTraditionalFormat</a>, the given number of bytes is divided by a power of 1024, with result less than 1024; for <a href="http://doc.qt.io/qt-5/qlocale.html#DataSizeFormat-enum">DataSizeSIFormat</a>, it is divided by a power of 1000, with result less than 1000. <code>DataSizeIecFormat</code> uses the new IEC standard quantifiers Ki, Mi and so on, whereas <code>DataSizeSIFormat</code> uses the older SI quantifiers k, M, etc., and <code>DataSizeTraditionalFormat</code> abuses them.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::formattedDataSize(qint64 bytes, int precision = …, QFlags<QLocale::DataSizeFormat> format = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#formattedDataSize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts a size in bytes to a human-readable localized string, comprising a number and a quantified unit. The quantifier is chosen such that the number is at least one, and as small as possible. For example if <i>bytes</i> is 16384, <i>precision</i> is 2, and <i>format</i> is <a href="http://doc.qt.io/qt-5/qlocale.html#DataSizeFormat-enum">DataSizeIecFormat</a> (the default), this function returns "16.00 KiB"; for 1330409069609 bytes it returns "1.21 GiB"; and so on. If <i>format</i> is <a href="http://doc.qt.io/qt-5/qlocale.html#DataSizeFormat-enum">DataSizeIecFormat</a> or <a href="http://doc.qt.io/qt-5/qlocale.html#DataSizeFormat-enum">DataSizeTraditionalFormat</a>, the given number of bytes is divided by a power of 1024, with result less than 1024; for <a href="http://doc.qt.io/qt-5/qlocale.html#DataSizeFormat-enum">DataSizeSIFormat</a>, it is divided by a power of 1000, with result less than 1000. <code>DataSizeIecFormat</code> uses the new IEC standard quantifiers Ki, Mi and so on, whereas <code>DataSizeSIFormat</code> uses the older SI quantifiers k, M, etc., and <code>DataSizeTraditionalFormat</code> abuses them.</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn formatted_data_size_3a_mut(
        &mut self,
        bytes: i64,
        precision: ::std::os::raw::c_int,
        format: crate::QFlags<crate::q_locale::DataSizeFormat>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_formattedDataSize(
            self as *mut crate::QLocale,
            bytes,
            precision,
            format.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts a size in bytes to a human-readable localized string, comprising a number and a quantified unit. The quantifier is chosen such that the number is at least one, and as small as possible. For example if <i>bytes</i> is 16384, <i>precision</i> is 2, and <i>format</i> is <a href="http://doc.qt.io/qt-5/qlocale.html#DataSizeFormat-enum">DataSizeIecFormat</a> (the default), this function returns "16.00 KiB"; for 1330409069609 bytes it returns "1.21 GiB"; and so on. If <i>format</i> is <a href="http://doc.qt.io/qt-5/qlocale.html#DataSizeFormat-enum">DataSizeIecFormat</a> or <a href="http://doc.qt.io/qt-5/qlocale.html#DataSizeFormat-enum">DataSizeTraditionalFormat</a>, the given number of bytes is divided by a power of 1024, with result less than 1024; for <a href="http://doc.qt.io/qt-5/qlocale.html#DataSizeFormat-enum">DataSizeSIFormat</a>, it is divided by a power of 1000, with result less than 1000. <code>DataSizeIecFormat</code> uses the new IEC standard quantifiers Ki, Mi and so on, whereas <code>DataSizeSIFormat</code> uses the older SI quantifiers k, M, etc., and <code>DataSizeTraditionalFormat</code> abuses them.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::formattedDataSize(qint64 bytes, int precision = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#formattedDataSize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts a size in bytes to a human-readable localized string, comprising a number and a quantified unit. The quantifier is chosen such that the number is at least one, and as small as possible. For example if <i>bytes</i> is 16384, <i>precision</i> is 2, and <i>format</i> is <a href="http://doc.qt.io/qt-5/qlocale.html#DataSizeFormat-enum">DataSizeIecFormat</a> (the default), this function returns "16.00 KiB"; for 1330409069609 bytes it returns "1.21 GiB"; and so on. If <i>format</i> is <a href="http://doc.qt.io/qt-5/qlocale.html#DataSizeFormat-enum">DataSizeIecFormat</a> or <a href="http://doc.qt.io/qt-5/qlocale.html#DataSizeFormat-enum">DataSizeTraditionalFormat</a>, the given number of bytes is divided by a power of 1024, with result less than 1024; for <a href="http://doc.qt.io/qt-5/qlocale.html#DataSizeFormat-enum">DataSizeSIFormat</a>, it is divided by a power of 1000, with result less than 1000. <code>DataSizeIecFormat</code> uses the new IEC standard quantifiers Ki, Mi and so on, whereas <code>DataSizeSIFormat</code> uses the older SI quantifiers k, M, etc., and <code>DataSizeTraditionalFormat</code> abuses them.</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn formatted_data_size_2a_mut(
        &mut self,
        bytes: i64,
        precision: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_formattedDataSize1(
            self as *mut crate::QLocale,
            bytes,
            precision,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts a size in bytes to a human-readable localized string, comprising a number and a quantified unit. The quantifier is chosen such that the number is at least one, and as small as possible. For example if <i>bytes</i> is 16384, <i>precision</i> is 2, and <i>format</i> is <a href="http://doc.qt.io/qt-5/qlocale.html#DataSizeFormat-enum">DataSizeIecFormat</a> (the default), this function returns "16.00 KiB"; for 1330409069609 bytes it returns "1.21 GiB"; and so on. If <i>format</i> is <a href="http://doc.qt.io/qt-5/qlocale.html#DataSizeFormat-enum">DataSizeIecFormat</a> or <a href="http://doc.qt.io/qt-5/qlocale.html#DataSizeFormat-enum">DataSizeTraditionalFormat</a>, the given number of bytes is divided by a power of 1024, with result less than 1024; for <a href="http://doc.qt.io/qt-5/qlocale.html#DataSizeFormat-enum">DataSizeSIFormat</a>, it is divided by a power of 1000, with result less than 1000. <code>DataSizeIecFormat</code> uses the new IEC standard quantifiers Ki, Mi and so on, whereas <code>DataSizeSIFormat</code> uses the older SI quantifiers k, M, etc., and <code>DataSizeTraditionalFormat</code> abuses them.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::formattedDataSize(qint64 bytes)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#formattedDataSize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts a size in bytes to a human-readable localized string, comprising a number and a quantified unit. The quantifier is chosen such that the number is at least one, and as small as possible. For example if <i>bytes</i> is 16384, <i>precision</i> is 2, and <i>format</i> is <a href="http://doc.qt.io/qt-5/qlocale.html#DataSizeFormat-enum">DataSizeIecFormat</a> (the default), this function returns "16.00 KiB"; for 1330409069609 bytes it returns "1.21 GiB"; and so on. If <i>format</i> is <a href="http://doc.qt.io/qt-5/qlocale.html#DataSizeFormat-enum">DataSizeIecFormat</a> or <a href="http://doc.qt.io/qt-5/qlocale.html#DataSizeFormat-enum">DataSizeTraditionalFormat</a>, the given number of bytes is divided by a power of 1024, with result less than 1024; for <a href="http://doc.qt.io/qt-5/qlocale.html#DataSizeFormat-enum">DataSizeSIFormat</a>, it is divided by a power of 1000, with result less than 1000. <code>DataSizeIecFormat</code> uses the new IEC standard quantifiers Ki, Mi and so on, whereas <code>DataSizeSIFormat</code> uses the older SI quantifiers k, M, etc., and <code>DataSizeTraditionalFormat</code> abuses them.</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn formatted_data_size_1a_mut(
        &mut self,
        bytes: i64,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_formattedDataSize2(
            self as *mut crate::QLocale,
            bytes,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is obsolete. It is provided to keep old source code working. We strongly advise against using it in new code.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::formattedDataSize(qint64 bytes, int precision = …, QFlags<QLocale::DataSizeFormat> format = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale-obsolete.html#formattedDataSize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is obsolete. It is provided to keep old source code working. We strongly advise against using it in new code.</p>
    /// <p>Use the const version instead.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn formatted_data_size_3a(
        &self,
        bytes: i64,
        precision: ::std::os::raw::c_int,
        format: crate::QFlags<crate::q_locale::DataSizeFormat>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_formattedDataSize6(
            self as *const crate::QLocale,
            bytes,
            precision,
            format.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is obsolete. It is provided to keep old source code working. We strongly advise against using it in new code.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::formattedDataSize(qint64 bytes, int precision = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale-obsolete.html#formattedDataSize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is obsolete. It is provided to keep old source code working. We strongly advise against using it in new code.</p>
    /// <p>Use the const version instead.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn formatted_data_size_2a(
        &self,
        bytes: i64,
        precision: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_formattedDataSize7(
            self as *const crate::QLocale,
            bytes,
            precision,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is obsolete. It is provided to keep old source code working. We strongly advise against using it in new code.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::formattedDataSize(qint64 bytes) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale-obsolete.html#formattedDataSize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is obsolete. It is provided to keep old source code working. We strongly advise against using it in new code.</p>
    /// <p>Use the const version instead.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn formatted_data_size_1a(&self, bytes: i64) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_formattedDataSize8(
            self as *const crate::QLocale,
            bytes,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the group separator character of this locale.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QLocale::groupSeparator() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#groupSeparator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the group separator character of this locale.</p>
    /// <p>This function was introduced in  Qt 4.1.</p></div>
    pub unsafe fn group_separator(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_groupSeparator(self as *const crate::QLocale);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the language of this locale.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale::Language QLocale::language() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#language">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the language of this locale.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#script">script</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#country">country</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#languageToString">languageToString</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#bcp47Name">bcp47Name</a>().</p></div>
    pub unsafe fn language(&self) -> crate::q_locale::Language {
        crate::__ffi::ctr_qt_core_ffi_QLocale_language(self as *const crate::QLocale)
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> containing the name of <i>language</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QLocale::languageToString(QLocale::Language language)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#languageToString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> containing the name of <i>language</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#countryToString">countryToString</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#scriptToString">scriptToString</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#bcp47Name">bcp47Name</a>().</p></div>
    pub unsafe fn language_to_string(
        language: crate::q_locale::Language,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_languageToString(language);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list of valid locale objects that match the given <i>language</i>, <i>script</i> and <i>country</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QList<QLocale> QLocale::matchingLocales(QLocale::Language language, QLocale::Script script, QLocale::Country country)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#matchingLocales">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of valid locale objects that match the given <i>language</i>, <i>script</i> and <i>country</i>.</p>
    /// <p>Getting a list of all locales: <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a>&gt; allLocales = QLocale::matchingLocales(<a href="http://doc.qt.io/qt-5/qlocale.html#Language-enum">QLocale::AnyLanguage</a>, <a href="http://doc.qt.io/qt-5/qlocale.html#Script-enum">QLocale::AnyScript</a>, <a href="http://doc.qt.io/qt-5/qlocale.html#Country-enum">QLocale::AnyCountry</a>);</p>
    /// <p>Getting a list of locales suitable for Russia: <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a>&gt; locales = QLocale::matchingLocales(<a href="http://doc.qt.io/qt-5/qlocale.html#Language-enum">QLocale::AnyLanguage</a>, <a href="http://doc.qt.io/qt-5/qlocale.html#Script-enum">QLocale::AnyScript</a>, <a href="http://doc.qt.io/qt-5/qlocale.html#Country-enum">QLocale::Russia</a>);</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn matching_locales(
        language: crate::q_locale::Language,
        script: crate::q_locale::Script,
        country: crate::q_locale::Country,
    ) -> ::cpp_utils::CppBox<crate::QListOfQLocale> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_matchingLocales(language, script, country);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the measurement system for the locale.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale::MeasurementSystem QLocale::measurementSystem() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#measurementSystem">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the measurement system for the locale.</p>
    /// <p>This function was introduced in  Qt 4.4.</p></div>
    pub unsafe fn measurement_system(&self) -> crate::q_locale::MeasurementSystem {
        crate::__ffi::ctr_qt_core_ffi_QLocale_measurementSystem(self as *const crate::QLocale)
    }

    /// <p>Returns the localized name of <i>month</i>, in the format specified by <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::monthName(int arg1, QLocale::FormatType format = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#monthName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the localized name of <i>month</i>, in the format specified by <i>type</i>.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#dayName">dayName</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#standaloneMonthName">standaloneMonthName</a>().</p></div>
    pub unsafe fn month_name_2a(
        &self,
        arg1: ::std::os::raw::c_int,
        format: crate::q_locale::FormatType,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_monthName(
            self as *const crate::QLocale,
            arg1,
            format,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the localized name of <i>month</i>, in the format specified by <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::monthName(int arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#monthName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the localized name of <i>month</i>, in the format specified by <i>type</i>.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#dayName">dayName</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#standaloneMonthName">standaloneMonthName</a>().</p></div>
    pub unsafe fn month_name_1a(
        &self,
        arg1: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_monthName1(self as *const crate::QLocale, arg1);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the language and country of this locale as a string of the form "language_country", where language is a lowercase, two-letter ISO 639 language code, and country is an uppercase, two- or three-letter ISO 3166 country code.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::name() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#name">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the language and country of this locale as a string of the form "language_country", where language is a lowercase, two-letter ISO 639 language code, and country is an uppercase, two- or three-letter ISO 3166 country code.</p>
    /// <p>Note that even if <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object was constructed with an explicit script, name() will not contain it for compatibility reasons. Use <a href="http://doc.qt.io/qt-5/qlocale.html#bcp47Name">bcp47Name</a>() instead if you need a full locale name.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#language">language</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#script">script</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#country">country</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#bcp47Name">bcp47Name</a>().</p></div>
    pub unsafe fn name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_name(self as *const crate::QLocale);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a native name of the country for the locale. For example "España" for Spanish/Spain locale.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::nativeCountryName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#nativeCountryName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a native name of the country for the locale. For example "España" for Spanish/Spain locale.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#nativeLanguageName">nativeLanguageName</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#countryToString">countryToString</a>().</p></div>
    pub unsafe fn native_country_name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_nativeCountryName(self as *const crate::QLocale);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a native name of the language for the locale. For example "Schwiizertüütsch" for Swiss-German locale.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::nativeLanguageName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#nativeLanguageName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a native name of the language for the locale. For example "Schwiizertüütsch" for Swiss-German locale.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#nativeCountryName">nativeCountryName</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#languageToString">languageToString</a>().</p></div>
    pub unsafe fn native_language_name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_nativeLanguageName(self as *const crate::QLocale);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the negative sign character of this locale.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QLocale::negativeSign() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#negativeSign">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the negative sign character of this locale.</p>
    /// <p>This function was introduced in  Qt 4.1.</p></div>
    pub unsafe fn negative_sign(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_negativeSign(self as *const crate::QLocale);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object initialized with the default locale. If no default locale was set using <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">setDefault</a>(), this locale will be the same as the one returned by <a href="http://doc.qt.io/qt-5/qlocale.html#system">system</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLocale::QLocale()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object initialized with the default locale. If no default locale was set using <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">setDefault</a>(), this locale will be the same as the one returned by <a href="http://doc.qt.io/qt-5/qlocale.html#system">system</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">setDefault</a>().</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_QLocale();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object with the specified <i>name</i>, which has the format "language[_script][_country][.codeset][@modifier]" or "C", where:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLocale::QLocale(const QString& name)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#QLocale-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object with the specified <i>name</i>, which has the format "language[_script][_country][.codeset][@modifier]" or "C", where:</p>
    /// <ul>
    /// <li>language is a lowercase, two-letter, ISO 639 language code (also some three-letter codes),</li>
    /// <li>script is a titlecase, four-letter, ISO 15924 script code,</li>
    /// <li>country is an uppercase, two-letter, ISO 3166 country code (also "419" as defined by United Nations),</li>
    /// <li>and codeset and modifier are ignored.</li>
    /// </ul>
    /// <p>The separator can be either underscore or a minus sign.</p>
    /// <p>If the string violates the locale format, or language is not a valid ISO 639 code, the "C" locale is used instead. If country is not present, or is not a valid ISO 3166 code, the most appropriate country is chosen for the specified language.</p>
    /// <p>The language, script and country codes are converted to their respective <code>Language</code>, <code>Script</code> and <code>Country</code> enums. After this conversion is performed, the constructor behaves exactly like <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a>(Country, Script, Language).</p>
    /// <p>This constructor is much slower than <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a>(Country, Script, Language).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#bcp47Name">bcp47Name</a>().</p></div>
    pub unsafe fn from_q_string(
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_QLocale1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object with the specified <i>language</i> and <i>country</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLocale::QLocale(QLocale::Language language, QLocale::Country country = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#QLocale-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object with the specified <i>language</i> and <i>country</i>.</p>
    /// <ul>
    /// <li>If the language/country pair is found in the database, it is used.</li>
    /// <li>If the language is found but the country is not, or if the country is <code>AnyCountry</code>, the language is used with the most appropriate available country (for example, Germany for German),</li>
    /// <li>If neither the language nor the country are found, <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> defaults to the default locale (see <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">setDefault</a>()).</li>
    /// </ul>
    /// <p>The language and country that are actually used can be queried using <a href="http://doc.qt.io/qt-5/qlocale.html#language">language</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#country">country</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">setDefault</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#language">language</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#country">country</a>().</p></div>
    pub unsafe fn from_language_country(
        language: crate::q_locale::Language,
        country: crate::q_locale::Country,
    ) -> ::cpp_utils::CppBox<crate::QLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_QLocale2(language, country);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object with the specified <i>language</i>, <i>script</i> and <i>country</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLocale::QLocale(QLocale::Language language, QLocale::Script script, QLocale::Country country)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#QLocale-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object with the specified <i>language</i>, <i>script</i> and <i>country</i>.</p>
    /// <ul>
    /// <li>If the language/script/country is found in the database, it is used.</li>
    /// <li>If both <i>script</i> is <a href="http://doc.qt.io/qt-5/qlocale.html#Script-enum">AnyScript</a> and <i>country</i> is <a href="http://doc.qt.io/qt-5/qlocale.html#Country-enum">AnyCountry</a>, the language is used with the most appropriate available script and country (for example, Germany for German),</li>
    /// <li>If either <i>script</i> is <a href="http://doc.qt.io/qt-5/qlocale.html#Script-enum">AnyScript</a> or <i>country</i> is <a href="http://doc.qt.io/qt-5/qlocale.html#Country-enum">AnyCountry</a>, the language is used with the first locale that matches the given <i>script</i> and <i>country</i>.</li>
    /// <li>If neither the language nor the country are found, <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> defaults to the default locale (see <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">setDefault</a>()).</li>
    /// </ul>
    /// <p>The language, script and country that are actually used can be queried using <a href="http://doc.qt.io/qt-5/qlocale.html#language">language</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#script">script</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#country">country</a>().</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">setDefault</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#language">language</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#script">script</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#country">country</a>().</p></div>
    pub unsafe fn from_language_script_country(
        language: crate::q_locale::Language,
        script: crate::q_locale::Script,
        country: crate::q_locale::Country,
    ) -> ::cpp_utils::CppBox<crate::QLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_QLocale3(language, script, country);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object with the specified <i>language</i> and <i>country</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLocale::QLocale(QLocale::Language language)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#QLocale-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object with the specified <i>language</i> and <i>country</i>.</p>
    /// <ul>
    /// <li>If the language/country pair is found in the database, it is used.</li>
    /// <li>If the language is found but the country is not, or if the country is <code>AnyCountry</code>, the language is used with the most appropriate available country (for example, Germany for German),</li>
    /// <li>If neither the language nor the country are found, <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> defaults to the default locale (see <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">setDefault</a>()).</li>
    /// </ul>
    /// <p>The language and country that are actually used can be queried using <a href="http://doc.qt.io/qt-5/qlocale.html#language">language</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#country">country</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">setDefault</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#language">language</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#country">country</a>().</p></div>
    pub unsafe fn from_language(
        language: crate::q_locale::Language,
    ) -> ::cpp_utils::CppBox<crate::QLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_QLocale5(language);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object as a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLocale::QLocale(const QLocale& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#QLocale-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object as a copy of <i>other</i>.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    ) -> ::cpp_utils::CppBox<crate::QLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_QLocale4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the options related to number conversions for this <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> instance.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFlags<QLocale::NumberOption> QLocale::numberOptions() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#numberOptions">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the options related to number conversions for this <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> instance.</p>
    /// <p>By default, no options are set for the standard locales.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#setNumberOptions">setNumberOptions</a>().</p></div>
    pub unsafe fn number_options(&self) -> crate::QFlags<crate::q_locale::NumberOption> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_numberOptions(self as *const crate::QLocale);
        crate::QFlags::from(ffi_result)
    }

    /// <p>Returns the percent character of this locale.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QLocale::percent() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#percent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the percent character of this locale.</p>
    /// <p>This function was introduced in  Qt 4.1.</p></div>
    pub unsafe fn percent(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_percent(self as *const crate::QLocale);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the localized name of the "PM" suffix for times specified using the conventions of the 12-hour clock.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::pmText() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#pmText">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the localized name of the "PM" suffix for times specified using the conventions of the 12-hour clock.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#amText">amText</a>().</p></div>
    pub unsafe fn pm_text(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_pmText(self as *const crate::QLocale);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the positive sign character of this locale.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QLocale::positiveSign() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#positiveSign">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the positive sign character of this locale.</p>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn positive_sign(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_positiveSign(self as *const crate::QLocale);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <i>str</i> quoted according to the current locale using the given quotation <i>style</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::quoteString(const QString& str, QLocale::QuotationStyle style = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#quoteString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <i>str</i> quoted according to the current locale using the given quotation <i>style</i>.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn quote_string_q_string_quotation_style(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        style: crate::q_locale::QuotationStyle,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_quoteString(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(str).as_raw_ptr(),
            style,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::quoteString(const QStringRef& str, QLocale::QuotationStyle style = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#quoteString-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn quote_string_q_string_ref_quotation_style(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        style: crate::q_locale::QuotationStyle,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_quoteString1(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(str)
                .as_raw_ptr(),
            style,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <i>str</i> quoted according to the current locale using the given quotation <i>style</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::quoteString(const QString& str) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#quoteString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <i>str</i> quoted according to the current locale using the given quotation <i>style</i>.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn quote_string_q_string(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_quoteString2(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(str).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::quoteString(const QStringRef& str) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#quoteString-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn quote_string_q_string_ref(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_quoteString3(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(str)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the script of this locale.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale::Script QLocale::script() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#script">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the script of this locale.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#language">language</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#country">country</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#languageToString">languageToString</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#scriptToString">scriptToString</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#bcp47Name">bcp47Name</a>().</p></div>
    pub unsafe fn script(&self) -> crate::q_locale::Script {
        crate::__ffi::ctr_qt_core_ffi_QLocale_script(self as *const crate::QLocale)
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> containing the name of <i>script</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QLocale::scriptToString(QLocale::Script script)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#scriptToString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> containing the name of <i>script</i>.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#languageToString">languageToString</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#countryToString">countryToString</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#script">script</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#bcp47Name">bcp47Name</a>().</p></div>
    pub unsafe fn script_to_string(
        script: crate::q_locale::Script,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_scriptToString(script);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Sets the global default locale to <i>locale</i>. These values are used when a <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object is constructed with no arguments. If this function is not called, the system's locale is used.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QLocale::setDefault(const QLocale& locale)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the global default locale to <i>locale</i>. These values are used when a <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object is constructed with no arguments. If this function is not called, the system's locale is used.</p>
    /// <p><b>Warning:</b> In a multithreaded application, the default locale should be set at application startup, before any non-GUI threads are created.</p>
    /// <p><b>Warning:</b> This function is not <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">reentrant</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#system">system</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#c">c</a>().</p></div>
    pub unsafe fn set_default(
        locale: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QLocale_setDefault(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(locale)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the <i>options</i> related to number conversions for this <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> instance.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QLocale::setNumberOptions(QFlags<QLocale::NumberOption> options)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#setNumberOptions">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the <i>options</i> related to number conversions for this <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> instance.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#numberOptions">numberOptions</a>().</p></div>
    pub unsafe fn set_number_options(
        &mut self,
        options: crate::QFlags<crate::q_locale::NumberOption>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QLocale_setNumberOptions(
            self as *mut crate::QLocale,
            options.to_int(),
        )
    }

    /// <p>Returns the localized name of the <i>day</i> (where 1 represents Monday, 2 represents Tuesday and so on) that is used as a standalone text, in the format specified by <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::standaloneDayName(int arg1, QLocale::FormatType format = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#standaloneDayName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the localized name of the <i>day</i> (where 1 represents Monday, 2 represents Tuesday and so on) that is used as a standalone text, in the format specified by <i>type</i>.</p>
    /// <p>If the locale information does not specify the standalone day name then return value is the same as in <a href="http://doc.qt.io/qt-5/qlocale.html#dayName">dayName</a>().</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#dayName">dayName</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#standaloneMonthName">standaloneMonthName</a>().</p></div>
    pub unsafe fn standalone_day_name_2a(
        &self,
        arg1: ::std::os::raw::c_int,
        format: crate::q_locale::FormatType,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_standaloneDayName(
            self as *const crate::QLocale,
            arg1,
            format,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the localized name of the <i>day</i> (where 1 represents Monday, 2 represents Tuesday and so on) that is used as a standalone text, in the format specified by <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::standaloneDayName(int arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#standaloneDayName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the localized name of the <i>day</i> (where 1 represents Monday, 2 represents Tuesday and so on) that is used as a standalone text, in the format specified by <i>type</i>.</p>
    /// <p>If the locale information does not specify the standalone day name then return value is the same as in <a href="http://doc.qt.io/qt-5/qlocale.html#dayName">dayName</a>().</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#dayName">dayName</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#standaloneMonthName">standaloneMonthName</a>().</p></div>
    pub unsafe fn standalone_day_name_1a(
        &self,
        arg1: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_standaloneDayName1(
            self as *const crate::QLocale,
            arg1,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the localized name of <i>month</i> that is used as a standalone text, in the format specified by <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::standaloneMonthName(int arg1, QLocale::FormatType format = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#standaloneMonthName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the localized name of <i>month</i> that is used as a standalone text, in the format specified by <i>type</i>.</p>
    /// <p>If the locale information doesn't specify the standalone month name then return value is the same as in <a href="http://doc.qt.io/qt-5/qlocale.html#monthName">monthName</a>().</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#monthName">monthName</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#standaloneDayName">standaloneDayName</a>().</p></div>
    pub unsafe fn standalone_month_name_2a(
        &self,
        arg1: ::std::os::raw::c_int,
        format: crate::q_locale::FormatType,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_standaloneMonthName(
            self as *const crate::QLocale,
            arg1,
            format,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the localized name of <i>month</i> that is used as a standalone text, in the format specified by <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::standaloneMonthName(int arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#standaloneMonthName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the localized name of <i>month</i> that is used as a standalone text, in the format specified by <i>type</i>.</p>
    /// <p>If the locale information doesn't specify the standalone month name then return value is the same as in <a href="http://doc.qt.io/qt-5/qlocale.html#monthName">monthName</a>().</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#monthName">monthName</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#standaloneDayName">standaloneDayName</a>().</p></div>
    pub unsafe fn standalone_month_name_1a(
        &self,
        arg1: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_standaloneMonthName1(
            self as *const crate::QLocale,
            arg1,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Swaps locale <i>other</i> with this locale. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QLocale::swap(QLocale& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps locale <i>other</i> with this locale. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 5.6.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QLocale>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QLocale_swap(
            self as *mut crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QLocale>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object initialized to the system locale.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QLocale QLocale::system()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#system">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object initialized to the system locale.</p>
    /// <p>On Windows and Mac, this locale will use the decimal/grouping characters and date/time formats specified in the system configuration panel.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#c">c</a>().</p></div>
    pub unsafe fn system() -> ::cpp_utils::CppBox<crate::QLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_system();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the text direction of the language.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```Qt::LayoutDirection QLocale::textDirection() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#textDirection">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the text direction of the language.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn text_direction(&self) -> crate::LayoutDirection {
        crate::__ffi::ctr_qt_core_ffi_QLocale_textDirection(self as *const crate::QLocale)
    }

    /// <p>Returns the time format used for the current locale.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::timeFormat(QLocale::FormatType format = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#timeFormat">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the time format used for the current locale.</p>
    /// <p>If <i>format</i> is <a href="http://doc.qt.io/qt-5/qlocale.html#FormatType-enum">LongFormat</a> the format will be a long version. Otherwise it uses a shorter version.</p>
    /// <p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#toString">QTime::toString</a>() and <a href="http://doc.qt.io/qt-5/qtime.html#fromString">QTime::fromString</a>().</p></div>
    pub unsafe fn time_format_1a(
        &self,
        format: crate::q_locale::FormatType,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_timeFormat(self as *const crate::QLocale, format);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the time format used for the current locale.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::timeFormat() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#timeFormat">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the time format used for the current locale.</p>
    /// <p>If <i>format</i> is <a href="http://doc.qt.io/qt-5/qlocale.html#FormatType-enum">LongFormat</a> the format will be a long version. Otherwise it uses a shorter version.</p>
    /// <p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#toString">QTime::toString</a>() and <a href="http://doc.qt.io/qt-5/qtime.html#fromString">QTime::fromString</a>().</p></div>
    pub unsafe fn time_format_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_timeFormat1(self as *const crate::QLocale);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a localized string representation of <i>value</i> as a currency. If the <i>symbol</i> is provided it is used instead of the default currency symbol.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toCurrencyString(qlonglong arg1, const QString& symbol = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toCurrencyString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a localized string representation of <i>value</i> as a currency. If the <i>symbol</i> is provided it is used instead of the default currency symbol.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#currencySymbol">currencySymbol</a>().</p></div>
    pub unsafe fn to_currency_string_i64_q_string(
        &self,
        arg1: i64,
        symbol: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toCurrencyString(
            self as *const crate::QLocale,
            arg1,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(symbol)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toCurrencyString(qulonglong arg1, const QString& symbol = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toCurrencyString-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn to_currency_string_u64_q_string(
        &self,
        arg1: u64,
        symbol: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toCurrencyString1(
            self as *const crate::QLocale,
            arg1,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(symbol)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toCurrencyString(short arg1, const QString& symbol = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toCurrencyString-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn to_currency_string_short_q_string(
        &self,
        arg1: ::std::os::raw::c_short,
        symbol: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toCurrencyString2(
            self as *const crate::QLocale,
            arg1,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(symbol)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toCurrencyString(unsigned short arg1, const QString& symbol = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toCurrencyString-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn to_currency_string_ushort_q_string(
        &self,
        arg1: ::std::os::raw::c_ushort,
        symbol: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toCurrencyString3(
            self as *const crate::QLocale,
            arg1,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(symbol)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toCurrencyString(int arg1, const QString& symbol = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toCurrencyString-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn to_currency_string_int_q_string(
        &self,
        arg1: ::std::os::raw::c_int,
        symbol: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toCurrencyString4(
            self as *const crate::QLocale,
            arg1,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(symbol)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toCurrencyString(unsigned int arg1, const QString& symbol = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toCurrencyString-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn to_currency_string_uint_q_string(
        &self,
        arg1: ::std::os::raw::c_uint,
        symbol: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toCurrencyString5(
            self as *const crate::QLocale,
            arg1,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(symbol)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toCurrencyString(double arg1, const QString& symbol = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toCurrencyString-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn to_currency_string_double_q_string(
        &self,
        arg1: ::std::os::raw::c_double,
        symbol: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toCurrencyString6(
            self as *const crate::QLocale,
            arg1,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(symbol)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qlocale.html#toCurrencyString">toCurrencyString</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toCurrencyString(double arg1, const QString& symbol, int precision) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toCurrencyString-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qlocale.html#toCurrencyString">toCurrencyString</a>().</p>
    /// <p>Returns a localized string representation of <i>value</i> as a currency. If the <i>symbol</i> is provided it is used instead of the default currency symbol. If the <i>precision</i> is provided it is used to set the precision of the currency value.</p>
    /// <p>This function was introduced in  Qt 5.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#currencySymbol">currencySymbol</a>().</p></div>
    pub unsafe fn to_currency_string_double_q_string_int(
        &self,
        arg1: ::std::os::raw::c_double,
        symbol: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        precision: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toCurrencyString7(
            self as *const crate::QLocale,
            arg1,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(symbol)
                .as_raw_ptr(),
            precision,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toCurrencyString(float i, const QString& symbol = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toCurrencyString-8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn to_currency_string_float_q_string(
        &self,
        i: ::std::os::raw::c_float,
        symbol: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toCurrencyString8(
            self as *const crate::QLocale,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(symbol)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toCurrencyString(float i, const QString& symbol, int precision) const```</span>.
    pub unsafe fn to_currency_string_float_q_string_int(
        &self,
        i: ::std::os::raw::c_float,
        symbol: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        precision: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toCurrencyString9(
            self as *const crate::QLocale,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(symbol)
                .as_raw_ptr(),
            precision,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a localized string representation of <i>value</i> as a currency. If the <i>symbol</i> is provided it is used instead of the default currency symbol.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toCurrencyString(qlonglong arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toCurrencyString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a localized string representation of <i>value</i> as a currency. If the <i>symbol</i> is provided it is used instead of the default currency symbol.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#currencySymbol">currencySymbol</a>().</p></div>
    pub unsafe fn to_currency_string_i64(&self, arg1: i64) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toCurrencyString10(
            self as *const crate::QLocale,
            arg1,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toCurrencyString(qulonglong arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toCurrencyString-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn to_currency_string_u64(&self, arg1: u64) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toCurrencyString11(
            self as *const crate::QLocale,
            arg1,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toCurrencyString(short arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toCurrencyString-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn to_currency_string_short(
        &self,
        arg1: ::std::os::raw::c_short,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toCurrencyString12(
            self as *const crate::QLocale,
            arg1,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toCurrencyString(unsigned short arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toCurrencyString-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn to_currency_string_ushort(
        &self,
        arg1: ::std::os::raw::c_ushort,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toCurrencyString13(
            self as *const crate::QLocale,
            arg1,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toCurrencyString(int arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toCurrencyString-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn to_currency_string_int(
        &self,
        arg1: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toCurrencyString14(
            self as *const crate::QLocale,
            arg1,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toCurrencyString(unsigned int arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toCurrencyString-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn to_currency_string_uint(
        &self,
        arg1: ::std::os::raw::c_uint,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toCurrencyString15(
            self as *const crate::QLocale,
            arg1,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toCurrencyString(double arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toCurrencyString-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn to_currency_string_double(
        &self,
        arg1: ::std::os::raw::c_double,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toCurrencyString16(
            self as *const crate::QLocale,
            arg1,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toCurrencyString(float i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toCurrencyString-8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn to_currency_string_float(
        &self,
        i: ::std::os::raw::c_float,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toCurrencyString17(
            self as *const crate::QLocale,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Parses the date string given in <i>string</i> and returns the date. The format of the date string is chosen according to the <i>format</i> parameter (see <a href="http://doc.qt.io/qt-5/qlocale.html#dateFormat">dateFormat</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDate QLocale::toDate(const QString& string, QLocale::FormatType arg2 = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toDate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Parses the date string given in <i>string</i> and returns the date. The format of the date string is chosen according to the <i>format</i> parameter (see <a href="http://doc.qt.io/qt-5/qlocale.html#dateFormat">dateFormat</a>()).</p>
    /// <p>If the date could not be parsed, returns an invalid date.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#dateFormat">dateFormat</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toTime">toTime</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDateTime">toDateTime</a>(), and <a href="http://doc.qt.io/qt-5/qdate.html#fromString">QDate::fromString</a>().</p></div>
    pub unsafe fn to_date_q_string_format_type(
        &self,
        string: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        arg2: crate::q_locale::FormatType,
    ) -> ::cpp_utils::CppBox<crate::QDate> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toDate(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(string)
                .as_raw_ptr(),
            arg2,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Parses the date string given in <i>string</i> and returns the date. See <a href="http://doc.qt.io/qt-5/qdate.html#fromString">QDate::fromString</a>() for information on the expressions that can be used with this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDate QLocale::toDate(const QString& string, const QString& format) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toDate-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Parses the date string given in <i>string</i> and returns the date. See <a href="http://doc.qt.io/qt-5/qdate.html#fromString">QDate::fromString</a>() for information on the expressions that can be used with this function.</p>
    /// <p>This function searches month names and the names of the days of the week in the current locale.</p>
    /// <p>If the date could not be parsed, returns an invalid date.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#dateFormat">dateFormat</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toTime">toTime</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDateTime">toDateTime</a>(), and <a href="http://doc.qt.io/qt-5/qdate.html#fromString">QDate::fromString</a>().</p></div>
    pub unsafe fn to_date_2_q_string(
        &self,
        string: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        format: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QDate> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toDate1(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(string)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(format)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Parses the date string given in <i>string</i> and returns the date. The format of the date string is chosen according to the <i>format</i> parameter (see <a href="http://doc.qt.io/qt-5/qlocale.html#dateFormat">dateFormat</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDate QLocale::toDate(const QString& string) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toDate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Parses the date string given in <i>string</i> and returns the date. The format of the date string is chosen according to the <i>format</i> parameter (see <a href="http://doc.qt.io/qt-5/qlocale.html#dateFormat">dateFormat</a>()).</p>
    /// <p>If the date could not be parsed, returns an invalid date.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#dateFormat">dateFormat</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toTime">toTime</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDateTime">toDateTime</a>(), and <a href="http://doc.qt.io/qt-5/qdate.html#fromString">QDate::fromString</a>().</p></div>
    pub unsafe fn to_date_q_string(
        &self,
        string: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QDate> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toDate2(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(string)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Parses the date/time string given in <i>string</i> and returns the time. The format of the date/time string is chosen according to the <i>format</i> parameter (see <a href="http://doc.qt.io/qt-5/qlocale.html#dateTimeFormat">dateTimeFormat</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDateTime QLocale::toDateTime(const QString& string, QLocale::FormatType format = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toDateTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Parses the date/time string given in <i>string</i> and returns the time. The format of the date/time string is chosen according to the <i>format</i> parameter (see <a href="http://doc.qt.io/qt-5/qlocale.html#dateTimeFormat">dateTimeFormat</a>()).</p>
    /// <p>If the string could not be parsed, returns an invalid <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#dateTimeFormat">dateTimeFormat</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toTime">toTime</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDate">toDate</a>(), and <a href="http://doc.qt.io/qt-5/qdatetime.html#fromString">QDateTime::fromString</a>().</p></div>
    pub unsafe fn to_date_time_q_string_format_type(
        &self,
        string: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        format: crate::q_locale::FormatType,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toDateTime(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(string)
                .as_raw_ptr(),
            format,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Parses the date/time string given in <i>string</i> and returns the time. See <a href="http://doc.qt.io/qt-5/qdatetime.html#fromString">QDateTime::fromString</a>() for information on the expressions that can be used with this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDateTime QLocale::toDateTime(const QString& string, const QString& format) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toDateTime-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Parses the date/time string given in <i>string</i> and returns the time. See <a href="http://doc.qt.io/qt-5/qdatetime.html#fromString">QDateTime::fromString</a>() for information on the expressions that can be used with this function.</p>
    /// <p><b>Note: </b>The month and day names used must be given in the user's local language.</p><p>If the string could not be parsed, returns an invalid <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#dateTimeFormat">dateTimeFormat</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toTime">toTime</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDate">toDate</a>(), and <a href="http://doc.qt.io/qt-5/qdatetime.html#fromString">QDateTime::fromString</a>().</p></div>
    pub unsafe fn to_date_time_2_q_string(
        &self,
        string: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        format: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toDateTime1(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(string)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(format)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Parses the date/time string given in <i>string</i> and returns the time. The format of the date/time string is chosen according to the <i>format</i> parameter (see <a href="http://doc.qt.io/qt-5/qlocale.html#dateTimeFormat">dateTimeFormat</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDateTime QLocale::toDateTime(const QString& string) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toDateTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Parses the date/time string given in <i>string</i> and returns the time. The format of the date/time string is chosen according to the <i>format</i> parameter (see <a href="http://doc.qt.io/qt-5/qlocale.html#dateTimeFormat">dateTimeFormat</a>()).</p>
    /// <p>If the string could not be parsed, returns an invalid <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#dateTimeFormat">dateTimeFormat</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toTime">toTime</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDate">toDate</a>(), and <a href="http://doc.qt.io/qt-5/qdatetime.html#fromString">QDateTime::fromString</a>().</p></div>
    pub unsafe fn to_date_time_q_string(
        &self,
        string: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toDateTime2(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(string)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the double represented by the localized string <i>s</i>, or 0.0 if the conversion failed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QLocale::toDouble(const QString& s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the double represented by the localized string <i>s</i>, or 0.0 if the conversion failed.</p>
    /// <p>If <i>ok</i> is not 0, reports failure by setting *ok to false and success by setting *ok to true.</p>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qstring.html#toDouble">QString::toDouble</a>(), this function does not use the 'C' locale if the string cannot be interpreted in this locale.</p>
    /// <pre class="cpp">
    ///
    ///   bool ok;
    ///   <span class="type">double</span> d;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span> c(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span><span class="operator">::</span>C);
    ///   d <span class="operator">=</span> c<span class="operator">.</span>toDouble( <span class="string">"1234.56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok );  <span class="comment">// ok == true, d == 1234.56</span>
    ///   d <span class="operator">=</span> c<span class="operator">.</span>toDouble( <span class="string">"1,234.56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok ); <span class="comment">// ok == true, d == 1234.56</span>
    ///   d <span class="operator">=</span> c<span class="operator">.</span>toDouble( <span class="string">"1234,56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok );  <span class="comment">// ok == false</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span> german(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span><span class="operator">::</span>German);
    ///   d <span class="operator">=</span> german<span class="operator">.</span>toDouble( <span class="string">"1234,56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok );  <span class="comment">// ok == true, d == 1234.56</span>
    ///   d <span class="operator">=</span> german<span class="operator">.</span>toDouble( <span class="string">"1.234,56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok ); <span class="comment">// ok == true, d == 1234.56</span>
    ///   d <span class="operator">=</span> german<span class="operator">.</span>toDouble( <span class="string">"1234.56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok );  <span class="comment">// ok == false</span>
    ///
    ///   d <span class="operator">=</span> german<span class="operator">.</span>toDouble( <span class="string">"1.234"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok );    <span class="comment">// ok == true, d == 1234.0</span>
    ///
    /// </pre>
    /// <p>Notice that the last conversion returns 1234.0, because '.' is the thousands group separator in the German locale.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toFloat">toFloat</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_double_q_string_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toDouble(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the double represented by the localized string <i>s</i>, or 0.0 if the conversion failed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QLocale::toDouble(const QStringRef& s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the double represented by the localized string <i>s</i>, or 0.0 if the conversion failed.</p>
    /// <p>If <i>ok</i> is not null, reports failure by setting *ok to false and success by setting *ok to true.</p>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qstring.html#toDouble">QString::toDouble</a>(), this function does not fall back to the "C" locale if the string cannot be interpreted in this locale.</p>
    /// <pre class="cpp">
    ///
    ///   bool ok;
    ///   <span class="type">double</span> d;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span> c(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span><span class="operator">::</span>C);
    ///   d <span class="operator">=</span> c<span class="operator">.</span>toDouble( <span class="string">"1234.56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok );  <span class="comment">// ok == true, d == 1234.56</span>
    ///   d <span class="operator">=</span> c<span class="operator">.</span>toDouble( <span class="string">"1,234.56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok ); <span class="comment">// ok == true, d == 1234.56</span>
    ///   d <span class="operator">=</span> c<span class="operator">.</span>toDouble( <span class="string">"1234,56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok );  <span class="comment">// ok == false</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span> german(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span><span class="operator">::</span>German);
    ///   d <span class="operator">=</span> german<span class="operator">.</span>toDouble( <span class="string">"1234,56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok );  <span class="comment">// ok == true, d == 1234.56</span>
    ///   d <span class="operator">=</span> german<span class="operator">.</span>toDouble( <span class="string">"1.234,56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok ); <span class="comment">// ok == true, d == 1234.56</span>
    ///   d <span class="operator">=</span> german<span class="operator">.</span>toDouble( <span class="string">"1234.56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok );  <span class="comment">// ok == false</span>
    ///
    ///   d <span class="operator">=</span> german<span class="operator">.</span>toDouble( <span class="string">"1.234"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok );    <span class="comment">// ok == true, d == 1234.0</span>
    ///
    /// </pre>
    /// <p>Notice that the last conversion returns 1234.0, because '.' is the thousands group separator in the German locale.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toFloat">toFloat</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_double_q_string_ref_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toDouble1(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the double represented by the localized string <i>s</i>, or 0.0 if the conversion failed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QLocale::toDouble(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the double represented by the localized string <i>s</i>, or 0.0 if the conversion failed.</p>
    /// <p>If <i>ok</i> is not 0, reports failure by setting *ok to false and success by setting *ok to true.</p>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qstring.html#toDouble">QString::toDouble</a>(), this function does not use the 'C' locale if the string cannot be interpreted in this locale.</p>
    /// <pre class="cpp">
    ///
    ///   bool ok;
    ///   <span class="type">double</span> d;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span> c(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span><span class="operator">::</span>C);
    ///   d <span class="operator">=</span> c<span class="operator">.</span>toDouble( <span class="string">"1234.56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok );  <span class="comment">// ok == true, d == 1234.56</span>
    ///   d <span class="operator">=</span> c<span class="operator">.</span>toDouble( <span class="string">"1,234.56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok ); <span class="comment">// ok == true, d == 1234.56</span>
    ///   d <span class="operator">=</span> c<span class="operator">.</span>toDouble( <span class="string">"1234,56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok );  <span class="comment">// ok == false</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span> german(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span><span class="operator">::</span>German);
    ///   d <span class="operator">=</span> german<span class="operator">.</span>toDouble( <span class="string">"1234,56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok );  <span class="comment">// ok == true, d == 1234.56</span>
    ///   d <span class="operator">=</span> german<span class="operator">.</span>toDouble( <span class="string">"1.234,56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok ); <span class="comment">// ok == true, d == 1234.56</span>
    ///   d <span class="operator">=</span> german<span class="operator">.</span>toDouble( <span class="string">"1234.56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok );  <span class="comment">// ok == false</span>
    ///
    ///   d <span class="operator">=</span> german<span class="operator">.</span>toDouble( <span class="string">"1.234"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok );    <span class="comment">// ok == true, d == 1234.0</span>
    ///
    /// </pre>
    /// <p>Notice that the last conversion returns 1234.0, because '.' is the thousands group separator in the German locale.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toFloat">toFloat</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_double_q_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toDouble2(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Returns the double represented by the localized string <i>s</i>, or 0.0 if the conversion failed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QLocale::toDouble(const QStringRef& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the double represented by the localized string <i>s</i>, or 0.0 if the conversion failed.</p>
    /// <p>If <i>ok</i> is not null, reports failure by setting *ok to false and success by setting *ok to true.</p>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qstring.html#toDouble">QString::toDouble</a>(), this function does not fall back to the "C" locale if the string cannot be interpreted in this locale.</p>
    /// <pre class="cpp">
    ///
    ///   bool ok;
    ///   <span class="type">double</span> d;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span> c(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span><span class="operator">::</span>C);
    ///   d <span class="operator">=</span> c<span class="operator">.</span>toDouble( <span class="string">"1234.56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok );  <span class="comment">// ok == true, d == 1234.56</span>
    ///   d <span class="operator">=</span> c<span class="operator">.</span>toDouble( <span class="string">"1,234.56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok ); <span class="comment">// ok == true, d == 1234.56</span>
    ///   d <span class="operator">=</span> c<span class="operator">.</span>toDouble( <span class="string">"1234,56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok );  <span class="comment">// ok == false</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span> german(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span><span class="operator">::</span>German);
    ///   d <span class="operator">=</span> german<span class="operator">.</span>toDouble( <span class="string">"1234,56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok );  <span class="comment">// ok == true, d == 1234.56</span>
    ///   d <span class="operator">=</span> german<span class="operator">.</span>toDouble( <span class="string">"1.234,56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok ); <span class="comment">// ok == true, d == 1234.56</span>
    ///   d <span class="operator">=</span> german<span class="operator">.</span>toDouble( <span class="string">"1234.56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok );  <span class="comment">// ok == false</span>
    ///
    ///   d <span class="operator">=</span> german<span class="operator">.</span>toDouble( <span class="string">"1.234"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok );    <span class="comment">// ok == true, d == 1234.0</span>
    ///
    /// </pre>
    /// <p>Notice that the last conversion returns 1234.0, because '.' is the thousands group separator in the German locale.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toFloat">toFloat</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_double_q_string_ref(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toDouble3(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Returns the double represented by the localized string <i>s</i>, or 0.0 if the conversion failed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QLocale::toDouble(QStringView s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the double represented by the localized string <i>s</i>, or 0.0 if the conversion failed.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qstring.html#toDouble">QString::toDouble</a>(), this function does not fall back to the "C" locale if the string cannot be interpreted in this locale.</p>
    /// <pre class="cpp">
    ///
    ///   bool ok;
    ///   <span class="type">double</span> d;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span> c(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span><span class="operator">::</span>C);
    ///   d <span class="operator">=</span> c<span class="operator">.</span>toDouble(u<span class="string">"1234.56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok);  <span class="comment">// ok == true, d == 1234.56</span>
    ///   d <span class="operator">=</span> c<span class="operator">.</span>toDouble(u<span class="string">"1,234.56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok); <span class="comment">// ok == true, d == 1234.56</span>
    ///   d <span class="operator">=</span> c<span class="operator">.</span>toDouble(u<span class="string">"1234,56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok);  <span class="comment">// ok == false</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span> german(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span><span class="operator">::</span>German);
    ///   d <span class="operator">=</span> german<span class="operator">.</span>toDouble(u<span class="string">"1234,56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok);  <span class="comment">// ok == true, d == 1234.56</span>
    ///   d <span class="operator">=</span> german<span class="operator">.</span>toDouble(u<span class="string">"1.234,56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok); <span class="comment">// ok == true, d == 1234.56</span>
    ///   d <span class="operator">=</span> german<span class="operator">.</span>toDouble(u<span class="string">"1234.56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok);  <span class="comment">// ok == false</span>
    ///
    ///   d <span class="operator">=</span> german<span class="operator">.</span>toDouble(u<span class="string">"1.234"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok);    <span class="comment">// ok == true, d == 1234.0</span>
    ///
    /// </pre>
    /// <p>Notice that the last conversion returns 1234.0, because '.' is the thousands group separator in the German locale.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toFloat">toFloat</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_double_q_string_view_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toDouble8(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the double represented by the localized string <i>s</i>, or 0.0 if the conversion failed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QLocale::toDouble(QStringView s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the double represented by the localized string <i>s</i>, or 0.0 if the conversion failed.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qstring.html#toDouble">QString::toDouble</a>(), this function does not fall back to the "C" locale if the string cannot be interpreted in this locale.</p>
    /// <pre class="cpp">
    ///
    ///   bool ok;
    ///   <span class="type">double</span> d;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span> c(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span><span class="operator">::</span>C);
    ///   d <span class="operator">=</span> c<span class="operator">.</span>toDouble(u<span class="string">"1234.56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok);  <span class="comment">// ok == true, d == 1234.56</span>
    ///   d <span class="operator">=</span> c<span class="operator">.</span>toDouble(u<span class="string">"1,234.56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok); <span class="comment">// ok == true, d == 1234.56</span>
    ///   d <span class="operator">=</span> c<span class="operator">.</span>toDouble(u<span class="string">"1234,56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok);  <span class="comment">// ok == false</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span> german(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html#QLocale">QLocale</a></span><span class="operator">::</span>German);
    ///   d <span class="operator">=</span> german<span class="operator">.</span>toDouble(u<span class="string">"1234,56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok);  <span class="comment">// ok == true, d == 1234.56</span>
    ///   d <span class="operator">=</span> german<span class="operator">.</span>toDouble(u<span class="string">"1.234,56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok); <span class="comment">// ok == true, d == 1234.56</span>
    ///   d <span class="operator">=</span> german<span class="operator">.</span>toDouble(u<span class="string">"1234.56"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok);  <span class="comment">// ok == false</span>
    ///
    ///   d <span class="operator">=</span> german<span class="operator">.</span>toDouble(u<span class="string">"1.234"</span><span class="operator">,</span> <span class="operator">&amp;</span>ok);    <span class="comment">// ok == true, d == 1234.0</span>
    ///
    /// </pre>
    /// <p>Notice that the last conversion returns 1234.0, because '.' is the thousands group separator in the German locale.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toFloat">toFloat</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_double_q_string_view(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toDouble9(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the float represented by the localized string <i>s</i>, or 0.0 if the conversion failed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```float QLocale::toFloat(const QString& s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toFloat">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the float represented by the localized string <i>s</i>, or 0.0 if the conversion failed.</p>
    /// <p>If <i>ok</i> is not 0, reports failure by setting *ok to false and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_float_q_string_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_float {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toFloat(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the float represented by the localized string <i>s</i>, or 0.0 if the conversion failed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```float QLocale::toFloat(const QStringRef& s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toFloat-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the float represented by the localized string <i>s</i>, or 0.0 if the conversion failed.</p>
    /// <p>If <i>ok</i> is not null, reports failure by setting *ok to false and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_float_q_string_ref_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_float {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toFloat1(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the float represented by the localized string <i>s</i>, or 0.0 if the conversion failed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```float QLocale::toFloat(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toFloat">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the float represented by the localized string <i>s</i>, or 0.0 if the conversion failed.</p>
    /// <p>If <i>ok</i> is not 0, reports failure by setting *ok to false and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_float_q_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_float {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toFloat2(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Returns the float represented by the localized string <i>s</i>, or 0.0 if the conversion failed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```float QLocale::toFloat(const QStringRef& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toFloat-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the float represented by the localized string <i>s</i>, or 0.0 if the conversion failed.</p>
    /// <p>If <i>ok</i> is not null, reports failure by setting *ok to false and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_float_q_string_ref(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::std::os::raw::c_float {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toFloat3(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Returns the float represented by the localized string <i>s</i>, or 0.0 if the conversion failed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```float QLocale::toFloat(QStringView s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toFloat-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the float represented by the localized string <i>s</i>, or 0.0 if the conversion failed.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_float_q_string_view_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_float {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toFloat8(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the float represented by the localized string <i>s</i>, or 0.0 if the conversion failed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```float QLocale::toFloat(QStringView s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toFloat-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the float represented by the localized string <i>s</i>, or 0.0 if the conversion failed.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_float_q_string_view(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::std::os::raw::c_float {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toFloat9(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QLocale::toInt(const QString& s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not 0, failure is reported by setting *ok to false, and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toUInt">toUInt</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_int_q_string_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toInt(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QLocale::toInt(const QStringRef& s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toInt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not null, failure is reported by setting *ok to false, and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toUInt">toUInt</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_int_q_string_ref_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toInt1(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QLocale::toInt(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not 0, failure is reported by setting *ok to false, and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toUInt">toUInt</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_int_q_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toInt2(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Returns the int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QLocale::toInt(const QStringRef& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toInt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not null, failure is reported by setting *ok to false, and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toUInt">toUInt</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_int_q_string_ref(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toInt3(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Returns the int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QLocale::toInt(QStringView s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toInt-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails, the function returns 0.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toUInt">toUInt</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_int_q_string_view_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toInt8(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QLocale::toInt(QStringView s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toInt-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails, the function returns 0.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toUInt">toUInt</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_int_q_string_view(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toInt9(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the long int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```long QLocale::toLong(const QString& s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the long int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toULong">toULong</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn to_long_q_string_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_long {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toLong(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the long int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```long QLocale::toLong(const QStringRef& s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toLong-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the long int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toULong">toULong</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn to_long_q_string_ref_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_long {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toLong1(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the long int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```long QLocale::toLong(QStringView s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toLong-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the long int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toULong">toULong</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn to_long_q_string_view_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_long {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toLong2(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the long int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```long QLocale::toLong(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the long int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toULong">toULong</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn to_long_q_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_long {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toLong3(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Returns the long int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```long QLocale::toLong(const QStringRef& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toLong-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the long int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toULong">toULong</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn to_long_q_string_ref(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::std::os::raw::c_long {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toLong4(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Returns the long int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```long QLocale::toLong(QStringView s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toLong-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the long int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toULong">toULong</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn to_long_q_string_view(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::std::os::raw::c_long {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toLong5(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the long long int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qlonglong QLocale::toLongLong(const QString& s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toLongLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the long long int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not 0, failure is reported by setting *ok to false, and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong">toULongLong</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_long_long_q_string_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toLongLong(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the long long int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qlonglong QLocale::toLongLong(const QStringRef& s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toLongLong-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the long long int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not null, failure is reported by setting *ok to false, and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong">toULongLong</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_long_long_q_string_ref_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toLongLong1(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the long long int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qlonglong QLocale::toLongLong(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toLongLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the long long int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not 0, failure is reported by setting *ok to false, and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong">toULongLong</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_long_long_q_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toLongLong2(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Returns the long long int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qlonglong QLocale::toLongLong(const QStringRef& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toLongLong-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the long long int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not null, failure is reported by setting *ok to false, and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong">toULongLong</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_long_long_q_string_ref(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toLongLong3(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Returns the long long int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qlonglong QLocale::toLongLong(QStringView s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toLongLong-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the long long int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails, the function returns 0.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong">toULongLong</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_long_long_q_string_view_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toLongLong8(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the long long int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qlonglong QLocale::toLongLong(QStringView s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toLongLong-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the long long int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails, the function returns 0.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong">toULongLong</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_long_long_q_string_view(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toLongLong9(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a lowercase copy of <i>str</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toLower(const QString& str) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toLower">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a lowercase copy of <i>str</i>.</p>
    /// <p>If Qt Core is using the ICU libraries, they will be used to perform the transformation according to the rules of the current locale. Otherwise the conversion may be done in a platform-dependent manner, with <a href="http://doc.qt.io/qt-5/qstring.html#toLower">QString::toLower</a>() as a generic fallback.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toLower">QString::toLower</a>().</p></div>
    pub unsafe fn to_lower(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toLower(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(str).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the short int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```short QLocale::toShort(const QString& s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toShort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the short int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not 0, failure is reported by setting *ok to false, and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toUShort">toUShort</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_short_q_string_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_short {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toShort(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the short int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```short QLocale::toShort(const QStringRef& s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toShort-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the short int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not null, failure is reported by setting *ok to false, and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toUShort">toUShort</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_short_q_string_ref_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_short {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toShort1(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the short int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```short QLocale::toShort(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toShort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the short int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not 0, failure is reported by setting *ok to false, and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toUShort">toUShort</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_short_q_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_short {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toShort2(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Returns the short int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```short QLocale::toShort(const QStringRef& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toShort-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the short int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not null, failure is reported by setting *ok to false, and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toUShort">toUShort</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_short_q_string_ref(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::std::os::raw::c_short {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toShort3(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Returns the short int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```short QLocale::toShort(QStringView s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toShort-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the short int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails, the function returns 0.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toUShort">toUShort</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_short_q_string_view_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_short {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toShort8(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the short int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```short QLocale::toShort(QStringView s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toShort-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the short int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails, the function returns 0.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toUShort">toUShort</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_short_q_string_view(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::std::os::raw::c_short {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toShort9(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a localized string representation of <i>i</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toString(qlonglong i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a localized string representation of <i>i</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toLongLong">toLongLong</a>().</p></div>
    pub unsafe fn to_string_i64(&self, i: i64) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_toString(self as *const crate::QLocale, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toString(qulonglong i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toString-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong">toULongLong</a>().</p></div>
    pub unsafe fn to_string_u64(&self, i: u64) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_toString1(self as *const crate::QLocale, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toString(short i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toString-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toShort">toShort</a>().</p></div>
    pub unsafe fn to_string_short(
        &self,
        i: ::std::os::raw::c_short,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_toString2(self as *const crate::QLocale, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toString(unsigned short i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toString-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toUShort">toUShort</a>().</p></div>
    pub unsafe fn to_string_ushort(
        &self,
        i: ::std::os::raw::c_ushort,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_toString3(self as *const crate::QLocale, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toString(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toString-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>().</p></div>
    pub unsafe fn to_string_int(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_toString4(self as *const crate::QLocale, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toString(unsigned int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toString-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toUInt">toUInt</a>().</p></div>
    pub unsafe fn to_string_uint(
        &self,
        i: ::std::os::raw::c_uint,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_toString5(self as *const crate::QLocale, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toString(double i, char f = …, int prec = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toString-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><i>f</i> and <i>prec</i> have the same meaning as in <a href="http://doc.qt.io/qt-5/qstring.html#number">QString::number</a>(double, char, int).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>().</p></div>
    pub unsafe fn to_string_double_char_int(
        &self,
        i: ::std::os::raw::c_double,
        f: ::std::os::raw::c_char,
        prec: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toString6(
            self as *const crate::QLocale,
            i,
            f,
            prec,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toString(float i, char f = …, int prec = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toString-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><i>f</i> and <i>prec</i> have the same meaning as in <a href="http://doc.qt.io/qt-5/qstring.html#number">QString::number</a>(double, char, int).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>().</p></div>
    pub unsafe fn to_string_float_char_int(
        &self,
        i: ::std::os::raw::c_float,
        f: ::std::os::raw::c_char,
        prec: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toString7(
            self as *const crate::QLocale,
            i,
            f,
            prec,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a localized string representation of the given <i>date</i> in the specified <i>format</i>. If <i>format</i> is an empty string, an empty string is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toString(const QDate& date, const QString& formatStr) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toString-8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a localized string representation of the given <i>date</i> in the specified <i>format</i>. If <i>format</i> is an empty string, an empty string is returned.</p></div>
    pub unsafe fn to_string_q_date_q_string(
        &self,
        date: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDate>>,
        format_str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toString8(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDate>>::cast_into(date).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(format_str)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a localized string representation of the given <i>date</i> according to the specified <i>format</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toString(const QDate& date, QLocale::FormatType format = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toString-9">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a localized string representation of the given <i>date</i> according to the specified <i>format</i>.</p></div>
    pub unsafe fn to_string_q_date_format_type(
        &self,
        date: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDate>>,
        format: crate::q_locale::FormatType,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toString9(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDate>>::cast_into(date).as_raw_ptr(),
            format,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a localized string representation of the given <i>time</i> according to the specified <i>format</i>. If <i>format</i> is an empty string, an empty string is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toString(const QTime& time, const QString& formatStr) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toString-10">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a localized string representation of the given <i>time</i> according to the specified <i>format</i>. If <i>format</i> is an empty string, an empty string is returned.</p></div>
    pub unsafe fn to_string_q_time_q_string(
        &self,
        time: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTime>>,
        format_str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toString10(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTime>>::cast_into(time).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(format_str)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a localized string representation of the given <i>time</i> in the specified <i>format</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toString(const QTime& time, QLocale::FormatType format = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toString-11">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a localized string representation of the given <i>time</i> in the specified <i>format</i>.</p></div>
    pub unsafe fn to_string_q_time_format_type(
        &self,
        time: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTime>>,
        format: crate::q_locale::FormatType,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toString11(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTime>>::cast_into(time).as_raw_ptr(),
            format,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a localized string representation of the given <i>dateTime</i> according to the specified <i>format</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toString(const QDateTime& dateTime, QLocale::FormatType format = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toString-12">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a localized string representation of the given <i>dateTime</i> according to the specified <i>format</i>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p></div>
    pub unsafe fn to_string_q_date_time_format_type(
        &self,
        date_time: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
        format: crate::q_locale::FormatType,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toString12(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(date_time)
                .as_raw_ptr(),
            format,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a localized string representation of the given <i>dateTime</i> according to the specified <i>format</i>. If <i>format</i> is an empty string, an empty string is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toString(const QDateTime& dateTime, const QString& format) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toString-13">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a localized string representation of the given <i>dateTime</i> according to the specified <i>format</i>. If <i>format</i> is an empty string, an empty string is returned.</p>
    /// <p>This function was introduced in  Qt 4.4.</p></div>
    pub unsafe fn to_string_q_date_time_q_string(
        &self,
        date_time: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
        format: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toString13(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(date_time)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(format)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toString(double i, char f = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toString-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><i>f</i> and <i>prec</i> have the same meaning as in <a href="http://doc.qt.io/qt-5/qstring.html#number">QString::number</a>(double, char, int).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>().</p></div>
    pub unsafe fn to_string_double_char(
        &self,
        i: ::std::os::raw::c_double,
        f: ::std::os::raw::c_char,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_toString14(self as *const crate::QLocale, i, f);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toString(double i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toString-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><i>f</i> and <i>prec</i> have the same meaning as in <a href="http://doc.qt.io/qt-5/qstring.html#number">QString::number</a>(double, char, int).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>().</p></div>
    pub unsafe fn to_string_double(
        &self,
        i: ::std::os::raw::c_double,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_toString15(self as *const crate::QLocale, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toString(float i, char f = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toString-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><i>f</i> and <i>prec</i> have the same meaning as in <a href="http://doc.qt.io/qt-5/qstring.html#number">QString::number</a>(double, char, int).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>().</p></div>
    pub unsafe fn to_string_float_char(
        &self,
        i: ::std::os::raw::c_float,
        f: ::std::os::raw::c_char,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_toString16(self as *const crate::QLocale, i, f);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toString(float i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toString-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><i>f</i> and <i>prec</i> have the same meaning as in <a href="http://doc.qt.io/qt-5/qstring.html#number">QString::number</a>(double, char, int).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>().</p></div>
    pub unsafe fn to_string_float(
        &self,
        i: ::std::os::raw::c_float,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_toString17(self as *const crate::QLocale, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a localized string representation of the given <i>date</i> according to the specified <i>format</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toString(const QDate& date) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toString-9">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a localized string representation of the given <i>date</i> according to the specified <i>format</i>.</p></div>
    pub unsafe fn to_string_q_date(
        &self,
        date: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDate>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toString18(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDate>>::cast_into(date).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a localized string representation of the given <i>time</i> in the specified <i>format</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toString(const QTime& time) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toString-11">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a localized string representation of the given <i>time</i> in the specified <i>format</i>.</p></div>
    pub unsafe fn to_string_q_time(
        &self,
        time: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTime>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toString19(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTime>>::cast_into(time).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a localized string representation of the given <i>dateTime</i> according to the specified <i>format</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toString(const QDateTime& dateTime) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toString-12">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a localized string representation of the given <i>dateTime</i> according to the specified <i>format</i>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p></div>
    pub unsafe fn to_string_q_date_time(
        &self,
        date_time: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toString20(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(date_time)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a localized string representation of the given <i>date</i> in the specified <i>format</i>. If <i>format</i> is an empty string, an empty string is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toString(const QDate& date, QStringView formatStr) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toString-11">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a localized string representation of the given <i>date</i> in the specified <i>format</i>. If <i>format</i> is an empty string, an empty string is returned.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#toString">QDate::toString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_string_q_date_q_string_view(
        &self,
        date: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDate>>,
        format_str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toString42(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDate>>::cast_into(date).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(format_str)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a localized string representation of the given <i>time</i> according to the specified <i>format</i>. If <i>format</i> is an empty string, an empty string is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toString(const QTime& time, QStringView formatStr) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toString-12">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a localized string representation of the given <i>time</i> according to the specified <i>format</i>. If <i>format</i> is an empty string, an empty string is returned.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#toString">QTime::toString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_string_q_time_q_string_view(
        &self,
        time: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTime>>,
        format_str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toString43(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTime>>::cast_into(time).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(format_str)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a localized string representation of the given <i>dateTime</i> according to the specified <i>format</i>. If <i>format</i> is an empty string, an empty string is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toString(const QDateTime& dateTime, QStringView format) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toString-13">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a localized string representation of the given <i>dateTime</i> according to the specified <i>format</i>. If <i>format</i> is an empty string, an empty string is returned.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#toString">QDateTime::toString</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#toString">QDate::toString</a>(), and <a href="http://doc.qt.io/qt-5/qtime.html#toString">QTime::toString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_string_q_date_time_q_string_view(
        &self,
        date_time: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
        format: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toString44(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(date_time)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(format)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toString(long i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toString-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toLong">toLong</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn to_string_long(
        &self,
        i: ::std::os::raw::c_long,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_toString48(self as *const crate::QLocale, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toString(unsigned long i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toString-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toULong">toULong</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn to_string_ulong(
        &self,
        i: ::std::os::raw::c_ulong,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_toString49(self as *const crate::QLocale, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Parses the time string given in <i>string</i> and returns the time. The format of the time string is chosen according to the <i>format</i> parameter (see <a href="http://doc.qt.io/qt-5/qlocale.html#timeFormat">timeFormat</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTime QLocale::toTime(const QString& string, QLocale::FormatType arg2 = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Parses the time string given in <i>string</i> and returns the time. The format of the time string is chosen according to the <i>format</i> parameter (see <a href="http://doc.qt.io/qt-5/qlocale.html#timeFormat">timeFormat</a>()).</p>
    /// <p>If the time could not be parsed, returns an invalid time.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#timeFormat">timeFormat</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDate">toDate</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDateTime">toDateTime</a>(), and <a href="http://doc.qt.io/qt-5/qtime.html#fromString">QTime::fromString</a>().</p></div>
    pub unsafe fn to_time_q_string_format_type(
        &self,
        string: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        arg2: crate::q_locale::FormatType,
    ) -> ::cpp_utils::CppBox<crate::QTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toTime(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(string)
                .as_raw_ptr(),
            arg2,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Parses the time string given in <i>string</i> and returns the time. See <a href="http://doc.qt.io/qt-5/qtime.html#fromString">QTime::fromString</a>() for information on what is a valid format string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTime QLocale::toTime(const QString& string, const QString& format) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toTime-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Parses the time string given in <i>string</i> and returns the time. See <a href="http://doc.qt.io/qt-5/qtime.html#fromString">QTime::fromString</a>() for information on what is a valid format string.</p>
    /// <p>If the time could not be parsed, returns an invalid time.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#timeFormat">timeFormat</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDate">toDate</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDateTime">toDateTime</a>(), and <a href="http://doc.qt.io/qt-5/qtime.html#fromString">QTime::fromString</a>().</p></div>
    pub unsafe fn to_time_2_q_string(
        &self,
        string: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        format: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toTime1(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(string)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(format)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Parses the time string given in <i>string</i> and returns the time. The format of the time string is chosen according to the <i>format</i> parameter (see <a href="http://doc.qt.io/qt-5/qlocale.html#timeFormat">timeFormat</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTime QLocale::toTime(const QString& string) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Parses the time string given in <i>string</i> and returns the time. The format of the time string is chosen according to the <i>format</i> parameter (see <a href="http://doc.qt.io/qt-5/qlocale.html#timeFormat">timeFormat</a>()).</p>
    /// <p>If the time could not be parsed, returns an invalid time.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#timeFormat">timeFormat</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDate">toDate</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDateTime">toDateTime</a>(), and <a href="http://doc.qt.io/qt-5/qtime.html#fromString">QTime::fromString</a>().</p></div>
    pub unsafe fn to_time_q_string(
        &self,
        string: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toTime2(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(string)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the unsigned int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QLocale::toUInt(const QString& s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toUInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the unsigned int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not 0, failure is reported by setting *ok to false, and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_u_int_q_string_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toUInt(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the unsigned int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QLocale::toUInt(const QStringRef& s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toUInt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the unsigned int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not null, failure is reported by setting *ok to false, and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_u_int_q_string_ref_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toUInt1(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the unsigned int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QLocale::toUInt(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toUInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the unsigned int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not 0, failure is reported by setting *ok to false, and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_u_int_q_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toUInt2(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Returns the unsigned int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QLocale::toUInt(const QStringRef& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toUInt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the unsigned int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not null, failure is reported by setting *ok to false, and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_u_int_q_string_ref(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toUInt3(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Returns the unsigned int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QLocale::toUInt(QStringView s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toUInt-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the unsigned int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails, the function returns 0.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_u_int_q_string_view_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toUInt8(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the unsigned int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QLocale::toUInt(QStringView s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toUInt-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the unsigned int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails, the function returns 0.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_u_int_q_string_view(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toUInt9(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the unsigned long int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned long QLocale::toULong(const QString& s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toULong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the unsigned long int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toLong">toLong</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn to_u_long_q_string_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_ulong {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toULong(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the unsigned long int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned long QLocale::toULong(const QStringRef& s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toULong-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the unsigned long int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toLong">toLong</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn to_u_long_q_string_ref_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_ulong {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toULong1(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the unsigned long int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned long QLocale::toULong(QStringView s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toULong-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the unsigned long int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toLong">toLong</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn to_u_long_q_string_view_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_ulong {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toULong2(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the unsigned long int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned long QLocale::toULong(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toULong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the unsigned long int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toLong">toLong</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn to_u_long_q_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_ulong {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toULong3(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Returns the unsigned long int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned long QLocale::toULong(const QStringRef& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toULong-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the unsigned long int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toLong">toLong</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn to_u_long_q_string_ref(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::std::os::raw::c_ulong {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toULong4(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Returns the unsigned long int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned long QLocale::toULong(QStringView s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toULong-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the unsigned long int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toLong">toLong</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn to_u_long_q_string_view(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::std::os::raw::c_ulong {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toULong5(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the unsigned long long int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qulonglong QLocale::toULongLong(const QString& s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the unsigned long long int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not 0, failure is reported by setting *ok to false, and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toLongLong">toLongLong</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_u_long_long_q_string_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> u64 {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toULongLong(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the unsigned long long int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qulonglong QLocale::toULongLong(const QStringRef& s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the unsigned long long int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not null, failure is reported by setting *ok to false, and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toLongLong">toLongLong</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_u_long_long_q_string_ref_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> u64 {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toULongLong1(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the unsigned long long int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qulonglong QLocale::toULongLong(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the unsigned long long int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not 0, failure is reported by setting *ok to false, and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toLongLong">toLongLong</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_u_long_long_q_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> u64 {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toULongLong2(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Returns the unsigned long long int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qulonglong QLocale::toULongLong(const QStringRef& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the unsigned long long int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not null, failure is reported by setting *ok to false, and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toLongLong">toLongLong</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_u_long_long_q_string_ref(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> u64 {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toULongLong3(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Returns the unsigned long long int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qulonglong QLocale::toULongLong(QStringView s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the unsigned long long int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails, the function returns 0.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toLongLong">toLongLong</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_u_long_long_q_string_view_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> u64 {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toULongLong8(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the unsigned long long int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qulonglong QLocale::toULongLong(QStringView s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toULongLong-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the unsigned long long int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails, the function returns 0.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toLongLong">toLongLong</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toInt">toInt</a>(), <a href="http://doc.qt.io/qt-5/qlocale.html#toDouble">toDouble</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_u_long_long_q_string_view(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> u64 {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toULongLong9(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the unsigned short int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned short QLocale::toUShort(const QString& s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toUShort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the unsigned short int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not 0, failure is reported by setting *ok to false, and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toShort">toShort</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_u_short_q_string_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_ushort {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toUShort(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the unsigned short int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned short QLocale::toUShort(const QStringRef& s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toUShort-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the unsigned short int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not null, failure is reported by setting *ok to false, and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toShort">toShort</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_u_short_q_string_ref_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_ushort {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toUShort1(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the unsigned short int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned short QLocale::toUShort(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toUShort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the unsigned short int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not 0, failure is reported by setting *ok to false, and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toShort">toShort</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_u_short_q_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_ushort {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toUShort2(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Returns the unsigned short int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned short QLocale::toUShort(const QStringRef& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toUShort-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the unsigned short int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails the function returns 0.</p>
    /// <p>If <i>ok</i> is not null, failure is reported by setting *ok to false, and success by setting *ok to true.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toShort">toShort</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    pub unsafe fn to_u_short_q_string_ref(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::std::os::raw::c_ushort {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toUShort3(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Returns the unsigned short int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned short QLocale::toUShort(QStringView s, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toUShort-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the unsigned short int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails, the function returns 0.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toShort">toShort</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_u_short_q_string_view_bool(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_ushort {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toUShort8(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the unsigned short int represented by the localized string <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned short QLocale::toUShort(QStringView s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toUShort-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the unsigned short int represented by the localized string <i>s</i>.</p>
    /// <p>If the conversion fails, the function returns 0.</p>
    /// <p>If <i>ok</i> is not <code>nullptr</code>, failure is reported by setting *<i>ok</i> to <code>false</code>, and success by setting *<i>ok</i> to <code>true</code>.</p>
    /// <p>This function ignores leading and trailing whitespace.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlocale.html#toShort">toShort</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_u_short_q_string_view(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::std::os::raw::c_ushort {
        crate::__ffi::ctr_qt_core_ffi_QLocale_toUShort9(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns an uppercase copy of <i>str</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLocale::toUpper(const QString& str) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#toUpper">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an uppercase copy of <i>str</i>.</p>
    /// <p>If Qt Core is using the ICU libraries, they will be used to perform the transformation according to the rules of the current locale. Otherwise the conversion may be done in a platform-dependent manner, with <a href="http://doc.qt.io/qt-5/qstring.html#toUpper">QString::toUpper</a>() as a generic fallback.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toUpper">QString::toUpper</a>().</p></div>
    pub unsafe fn to_upper(
        &self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLocale_toUpper(
            self as *const crate::QLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(str).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns an ordered list of locale names for translation purposes in preference order (like "en-Latn-US", "en-US", "en").</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QLocale::uiLanguages() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#uiLanguages">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an ordered list of locale names for translation purposes in preference order (like "en-Latn-US", "en-US", "en").</p>
    /// <p>The return value represents locale names that the user expects to see the UI translation in.</p>
    /// <p>Most like you do not need to use this function directly, but just pass the <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object to the <a href="http://doc.qt.io/qt-5/qtranslator.html#load">QTranslator::load</a>() function.</p>
    /// <p>The first item in the list is the most preferred one.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtranslator.html">QTranslator</a> and <a href="http://doc.qt.io/qt-5/qlocale.html#bcp47Name">bcp47Name</a>().</p></div>
    pub unsafe fn ui_languages(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_uiLanguages(self as *const crate::QLocale);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list of days that are considered weekdays according to the current locale.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek> QLocale::weekdays() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#weekdays">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of days that are considered weekdays according to the current locale.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn weekdays(&self) -> ::cpp_utils::CppBox<crate::QListOfDayOfWeek> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_weekdays(self as *const crate::QLocale);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the zero digit character of this locale.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QLocale::zeroDigit() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#zeroDigit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the zero digit character of this locale.</p>
    /// <p>This function was introduced in  Qt 4.1.</p></div>
    pub unsafe fn zero_digit(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLocale_zeroDigit(self as *const crate::QLocale);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qcollatorsortkey.html">QCollatorSortKey</a> class can be used to speed up string collation.</p>
///
/// C++ class: <span style='color: green;'>```QCollatorSortKey```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qcollatorsortkey.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qcollatorsortkey.html">QCollatorSortKey</a> class can be used to speed up string collation.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qcollatorsortkey.html">QCollatorSortKey</a> class is always created by <a href="http://doc.qt.io/qt-5/qcollator.html#sortKey">QCollator::sortKey</a>() and is used for fast strings collation, for example when collating many strings.</p></div>
#[repr(C)]
pub struct QCollatorSortKey {
    _unused: u8,
}
impl QCollatorSortKey {
    /// <p>Compares the key to <i>otherKey</i>. Returns a negative value if the key is less than <i>otherKey</i>, 0 if the key is equal to <i>otherKey</i> or a positive value if the key is greater than <i>otherKey</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QCollatorSortKey::compare(const QCollatorSortKey& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcollatorsortkey.html#compare">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares the key to <i>otherKey</i>. Returns a negative value if the key is less than <i>otherKey</i>, 0 if the key is equal to <i>otherKey</i> or a positive value if the key is greater than <i>otherKey</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcollatorsortkey.html#operator-lt">operator&lt;</a>().</p></div>
    pub unsafe fn compare(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCollatorSortKey>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QCollatorSortKey_compare(
            self as *const crate::QCollatorSortKey,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCollatorSortKey>>::cast_into(key)
                .as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this collator key.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCollatorSortKey& QCollatorSortKey::operator=(const QCollatorSortKey& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcollatorsortkey.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this collator key.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCollatorSortKey>>,
    ) -> ::cpp_utils::MutRef<crate::QCollatorSortKey> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCollatorSortKey_operator_(
            self as *mut crate::QCollatorSortKey,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCollatorSortKey>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Constructs a copy of the <i>other</i> collator key.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCollatorSortKey::QCollatorSortKey(const QCollatorSortKey& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcollatorsortkey.html#QCollatorSortKey">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of the <i>other</i> collator key.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCollatorSortKey>>,
    ) -> ::cpp_utils::CppBox<crate::QCollatorSortKey> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCollatorSortKey_QCollatorSortKey(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCollatorSortKey>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Swaps this collator key with <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCollatorSortKey::swap(QCollatorSortKey& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcollatorsortkey.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps this collator key with <i>other</i>.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QCollatorSortKey>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCollatorSortKey_swap(
            self as *mut crate::QCollatorSortKey,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QCollatorSortKey>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qcollator.html">QCollator</a> class compares strings according to a localized collation algorithm.</p>
///
/// C++ class: <span style='color: green;'>```QCollator```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qcollator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qcollator.html">QCollator</a> class compares strings according to a localized collation algorithm.</p>
/// <p><a href="http://doc.qt.io/qt-5/qcollator.html">QCollator</a> is initialized with a <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> and an optional collation strategy. It tries to initialize the collator with the specified values. The collator can then be used to compare and sort strings in a locale dependent fashion.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qcollator.html">QCollator</a> object can be used together with template based sorting algorithms such as std::sort to sort a list of QStrings.</p>
/// <p>In addition to the locale and collation strategy, several optional flags can be set that influence the result of the collation.</p></div>
#[repr(C)]
pub struct QCollator {
    _unused: u8,
}
impl QCollator {
    /// Calls C++ function: <span style='color: green;'>```bool QCollator::operator()(const QString& s1, const QString& s2) const```</span>.
    pub unsafe fn call(
        &self,
        s1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        s2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCollator_operator__(
            self as *const crate::QCollator,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s2).as_raw_ptr(),
        )
    }

    /// <p>Returns case sensitivity of the collator.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```Qt::CaseSensitivity QCollator::caseSensitivity() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcollator.html#caseSensitivity">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns case sensitivity of the collator.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcollator.html#setCaseSensitivity">setCaseSensitivity</a>().</p></div>
    pub unsafe fn case_sensitivity(&self) -> crate::CaseSensitivity {
        crate::__ffi::ctr_qt_core_ffi_QCollator_caseSensitivity(self as *const crate::QCollator)
    }

    /// <p>Compares <i>s1</i> with <i>s2</i>. Returns an integer less than, equal to, or greater than zero depending on whether <i>s1</i> is smaller, equal or larger than <i>s2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QCollator::compare(const QString& s1, const QString& s2) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcollator.html#compare">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares <i>s1</i> with <i>s2</i>. Returns an integer less than, equal to, or greater than zero depending on whether <i>s1</i> is smaller, equal or larger than <i>s2</i>.</p></div>
    pub unsafe fn compare_2_q_string(
        &self,
        s1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        s2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QCollator_compare(
            self as *const crate::QCollator,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s2).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QCollator::compare(const QStringRef& s1, const QStringRef& s2) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcollator.html#compare-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Compares <i>s1</i> with <i>s2</i>. Returns an integer less than, equal to, or greater than zero depending on whether <i>s1</i> is smaller, equal or larger than <i>s2</i>.</p></div>
    pub unsafe fn compare_2_q_string_ref(
        &self,
        s1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        s2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QCollator_compare1(
            self as *const crate::QCollator,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s1)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(s2)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QCollator::compare(const QChar* s1, int len1, const QChar* s2, int len2) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcollator.html#compare-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Compares <i>s1</i> with <i>s2</i>. <i>len1</i> and <i>len2</i> specify the length of the <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> arrays pointer to by <i>s1</i> and <i>s2</i>.</p>
    /// <p>Returns an integer less than, equal to, or greater than zero depending on whether <i>s1</i> is smaller, equal or larger than <i>s2</i>.</p></div>
    pub unsafe fn compare_q_char_int_q_char_int(
        &self,
        s1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QChar>>,
        len1: ::std::os::raw::c_int,
        s2: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QChar>>,
        len2: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QCollator_compare2(
            self as *const crate::QCollator,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QChar>>::cast_into(s1).as_raw_ptr(),
            len1,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QChar>>::cast_into(s2).as_raw_ptr(),
            len2,
        )
    }

    /// <p>Assigns <i>other</i> to this collator.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCollator& QCollator::operator=(const QCollator& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcollator.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this collator.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCollator>>,
    ) -> ::cpp_utils::MutRef<crate::QCollator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCollator_operator_(
            self as *mut crate::QCollator,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCollator>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if punctuation characters and symbols are ignored when determining sort order.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCollator::ignorePunctuation() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcollator.html#ignorePunctuation">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if punctuation characters and symbols are ignored when determining sort order.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcollator.html#setIgnorePunctuation">setIgnorePunctuation</a>().</p></div>
    pub unsafe fn ignore_punctuation(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCollator_ignorePunctuation(self as *const crate::QCollator)
    }

    /// <p>Returns the locale of the collator.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale QCollator::locale() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcollator.html#locale">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the locale of the collator.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcollator.html#setLocale">setLocale</a>().</p></div>
    pub unsafe fn locale(&self) -> ::cpp_utils::CppBox<crate::QLocale> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCollator_locale(self as *const crate::QCollator);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qcollator.html">QCollator</a> from <i>locale</i>. If <i>locale</i> is not specified the system's default locale is used.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCollator::QCollator(const QLocale& locale = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcollator.html#QCollator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qcollator.html">QCollator</a> from <i>locale</i>. If <i>locale</i> is not specified the system's default locale is used.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcollator.html#setLocale">setLocale</a>().</p></div>
    pub unsafe fn new_1a(
        locale: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    ) -> ::cpp_utils::CppBox<crate::QCollator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCollator_QCollator(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(locale)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qcollator.html">QCollator</a> class compares strings according to a localized collation algorithm.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCollator::QCollator()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcollator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qcollator.html">QCollator</a> class compares strings according to a localized collation algorithm.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qcollator.html">QCollator</a> is initialized with a <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> and an optional collation strategy. It tries to initialize the collator with the specified values. The collator can then be used to compare and sort strings in a locale dependent fashion.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qcollator.html">QCollator</a> object can be used together with template based sorting algorithms such as std::sort to sort a list of QStrings.</p>
    /// <p>In addition to the locale and collation strategy, several optional flags can be set that influence the result of the collation.</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QCollator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCollator_QCollator3();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCollator::QCollator(const QCollator& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcollator.html#QCollator-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a copy of <i>other</i>.</p></div>
    pub unsafe fn new_copy(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCollator>>,
    ) -> ::cpp_utils::CppBox<crate::QCollator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCollator_QCollator1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCollator>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if numeric sorting is enabled, false otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCollator::numericMode() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcollator.html#numericMode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if numeric sorting is enabled, false otherwise.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcollator.html#setNumericMode">setNumericMode</a>().</p></div>
    pub unsafe fn numeric_mode(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCollator_numericMode(self as *const crate::QCollator)
    }

    /// <p>Sets the case <i>sensitivity</i> of the collator.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCollator::setCaseSensitivity(Qt::CaseSensitivity cs)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcollator.html#setCaseSensitivity">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the case <i>sensitivity</i> of the collator.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcollator.html#caseSensitivity">caseSensitivity</a>().</p></div>
    pub unsafe fn set_case_sensitivity(&mut self, cs: crate::CaseSensitivity) {
        crate::__ffi::ctr_qt_core_ffi_QCollator_setCaseSensitivity(
            self as *mut crate::QCollator,
            cs,
        )
    }

    /// <p>If <i>on</i> is set to true, punctuation characters and symbols are ignored when determining sort order.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCollator::setIgnorePunctuation(bool on)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcollator.html#setIgnorePunctuation">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If <i>on</i> is set to true, punctuation characters and symbols are ignored when determining sort order.</p>
    /// <p>The default is locale dependent.</p>
    /// <p><b>Note: </b>This method is not currently supported if Qt is configured to not use ICU on Linux.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qcollator.html#ignorePunctuation">ignorePunctuation</a>().</p></div>
    pub unsafe fn set_ignore_punctuation(&mut self, on: bool) {
        crate::__ffi::ctr_qt_core_ffi_QCollator_setIgnorePunctuation(
            self as *mut crate::QCollator,
            on,
        )
    }

    /// <p>Sets the locale of the collator to <i>locale</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCollator::setLocale(const QLocale& locale)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcollator.html#setLocale">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the locale of the collator to <i>locale</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcollator.html#locale">locale</a>().</p></div>
    pub unsafe fn set_locale(
        &mut self,
        locale: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCollator_setLocale(
            self as *mut crate::QCollator,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(locale)
                .as_raw_ptr(),
        )
    }

    /// <p>Enables numeric sorting mode when <i>on</i> is set to true.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCollator::setNumericMode(bool on)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcollator.html#setNumericMode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Enables numeric sorting mode when <i>on</i> is set to true.</p>
    /// <p>This will enable proper sorting of numeric digits, so that e.g. 100 sorts after 99.</p>
    /// <p>By default this mode is off.</p>
    /// <p><b>Note: </b>On Windows, this functionality makes use of the <a href="http://doc.qt.io/qt-5/windows-requirements.html#icu">ICU</a> library. If Qt was compiled without ICU support, it falls back to code using native Windows API, which only works from Windows 7 onwards. On older versions of Windows, it will not work and a warning will be emitted at runtime.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qcollator.html#numericMode">numericMode</a>().</p></div>
    pub unsafe fn set_numeric_mode(&mut self, on: bool) {
        crate::__ffi::ctr_qt_core_ffi_QCollator_setNumericMode(self as *mut crate::QCollator, on)
    }

    /// <p>Returns a sortKey for <i>string</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCollatorSortKey QCollator::sortKey(const QString& string) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcollator.html#sortKey">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sortKey for <i>string</i>.</p>
    /// <p>Creating the sort key is usually somewhat slower, than using the <a href="http://doc.qt.io/qt-5/qcollator.html#compare">compare</a>() methods directly. But if the string is compared repeatedly (e.g. when sorting a whole list of strings), it's usually faster to create the sort keys for each string and then sort using the keys.</p></div>
    pub unsafe fn sort_key(
        &self,
        string: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QCollatorSortKey> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCollator_sortKey(
            self as *const crate::QCollator,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(string)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Swaps this collator with <i>other</i>. This function is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCollator::swap(QCollator& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcollator.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps this collator with <i>other</i>. This function is very fast and never fails.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QCollator>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCollator_swap(
            self as *mut crate::QCollator,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QCollator>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }
}

pub mod q_command_line_option {
    //! C++ type: <span style='color: green;'>```QCommandLineOption```</span>

    /// <p>The Flags type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;Flag&gt;. It stores an OR combination of Flag values.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QCommandLineOption::Flag```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#Flag-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The Flags type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;Flag&gt;. It stores an OR combination of Flag values.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setFlags">QCommandLineOption::setFlags</a>() and <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#flags">QCommandLineOption::flags</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Flag(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Flag {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Flag(value)
        }
    }

    impl From<Flag> for ::std::os::raw::c_int {
        fn from(value: Flag) -> Self {
            value.0
        }
    }

    impl Flag {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Flag {
        /// Hide this option in the user-visible help output. All options are visible by default. Setting this flag for a particular option makes it internal, i.e. not listed in the help output. (C++ enum variant: <span style='color: green;'>```HiddenFromHelp = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const HiddenFromHelp: crate::q_command_line_option::Flag =
            crate::q_command_line_option::Flag(1);
        /// The option will always be understood as a short option, regardless of what was set by <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#setSingleDashWordOptionMode">QCommandLineParser::setSingleDashWordOptionMode</a>. This allows flags such as <code>-DDEFINE=VALUE</code> or <code>-I/include/path</code> to be interpreted as short flags even when the parser is in <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#SingleDashWordOptionMode-enum">QCommandLineParser::ParseAsLongOptions</a> mode. (C++ enum variant: <span style='color: green;'>```ShortOptionStyle = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const ShortOptionStyle: crate::q_command_line_option::Flag =
            crate::q_command_line_option::Flag(2);
    }

    impl From<crate::q_command_line_option::Flag>
        for crate::QFlags<crate::q_command_line_option::Flag>
    {
        fn from(value: crate::q_command_line_option::Flag) -> Self {
            Self::from(value.to_int())
        }
    }

    impl<T: Into<crate::QFlags<crate::q_command_line_option::Flag>>> std::ops::BitOr<T>
        for crate::q_command_line_option::Flag
    {
        type Output = crate::QFlags<crate::q_command_line_option::Flag>;
        fn bitor(self, rhs: T) -> crate::QFlags<crate::q_command_line_option::Flag> {
            Into::<crate::QFlags<crate::q_command_line_option::Flag>>::into(self) | rhs
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qcommandlineoption.html">QCommandLineOption</a> class defines a possible command-line option.</p>
///
/// C++ class: <span style='color: green;'>```QCommandLineOption```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qcommandlineoption.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qcommandlineoption.html">QCommandLineOption</a> class defines a possible command-line option.</p>
/// <p>This class is used to describe an option on the command line. It allows different ways of defining the same option with multiple aliases possible. It is also used to describe how the option is used - it may be a flag (e.g. <code>-v</code>) or take a value (e.g. <code>-o file</code>).</p>
/// <p>Examples:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qcommandlineoption.html#QCommandLineOption">QCommandLineOption</a></span> verboseOption(<span class="string">"verbose"</span><span class="operator">,</span> <span class="string">"Verbose mode. Prints out more information."</span>);
///   <span class="type"><a href="http://doc.qt.io/qt-5/qcommandlineoption.html#QCommandLineOption">QCommandLineOption</a></span> outputOption(<span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"o"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"output"</span><span class="operator">,</span> <span class="string">"Write generated data into &lt;file&gt;."</span><span class="operator">,</span> <span class="string">"file"</span>);
///
/// </pre></div>
#[repr(C)]
pub struct QCommandLineOption {
    _unused: u8,
}
impl QCommandLineOption {
    /// <p>Makes a copy of the <i>other</i> object and assigns it to this <a href="http://doc.qt.io/qt-5/qcommandlineoption.html">QCommandLineOption</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCommandLineOption& QCommandLineOption::operator=(const QCommandLineOption& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes a copy of the <i>other</i> object and assigns it to this <a href="http://doc.qt.io/qt-5/qcommandlineoption.html">QCommandLineOption</a> object.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCommandLineOption>>,
    ) -> ::cpp_utils::MutRef<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineOption_operator_(
            self as *mut crate::QCommandLineOption,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCommandLineOption>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the default values set for this option.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QCommandLineOption::defaultValues() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#defaultValues">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the default values set for this option.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDefaultValues">setDefaultValues</a>().</p></div>
    pub unsafe fn default_values(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineOption_defaultValues(
            self as *const crate::QCommandLineOption,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the description set for this option.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QCommandLineOption::description() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#description">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the description set for this option.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDescription">setDescription</a>().</p></div>
    pub unsafe fn description(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineOption_description(
            self as *const crate::QCommandLineOption,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a set of flags that affect this command-line option.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFlags<QCommandLineOption::Flag> QCommandLineOption::flags() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#flags">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a set of flags that affect this command-line option.</p>
    /// <p>This function was introduced in  Qt 5.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setFlags">setFlags</a>() and <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#Flag-enum">QCommandLineOption::Flags</a>.</p></div>
    pub unsafe fn flags(&self) -> crate::QFlags<crate::q_command_line_option::Flag> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineOption_flags(
            self as *const crate::QCommandLineOption,
        );
        crate::QFlags::from(ffi_result)
    }

    /// <p>Returns true if this option is omitted from the help output, false if the option is listed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCommandLineOption::isHidden() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineoption-obsolete.html#isHidden">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if this option is omitted from the help output, false if the option is listed.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineoption-obsolete.html#setHidden">setHidden</a>() and <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#Flag-enum">QCommandLineOption::HiddenFromHelp</a>.</p></div>
    pub unsafe fn is_hidden(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCommandLineOption_isHidden(
            self as *const crate::QCommandLineOption,
        )
    }

    /// <p>Returns the names set for this option.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QCommandLineOption::names() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#names">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the names set for this option.</p></div>
    pub unsafe fn names(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineOption_names(
            self as *const crate::QCommandLineOption,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a command line option object with the name <i>name</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCommandLineOption::QCommandLineOption(const QString& name)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#QCommandLineOption">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a command line option object with the name <i>name</i>.</p>
    /// <p>The name can be either short or long. If the name is one character in length, it is considered a short name. Option names must not be empty, must not start with a dash or a slash character, must not contain a <code>=</code> and cannot be repeated.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDescription">setDescription</a>(), <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setValueName">setValueName</a>(), and <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDefaultValues">setDefaultValues</a>().</p></div>
    pub unsafe fn from_q_string(
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineOption_QCommandLineOption(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a command line option object with the names <i>names</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCommandLineOption::QCommandLineOption(const QStringList& names)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#QCommandLineOption-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a command line option object with the names <i>names</i>.</p>
    /// <p>This overload allows to set multiple names for the option, for instance <code>o</code> and <code>output</code>.</p>
    /// <p>The names can be either short or long. Any name in the list that is one character in length is a short name. Option names must not be empty, must not start with a dash or a slash character, must not contain a <code>=</code> and cannot be repeated.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDescription">setDescription</a>(), <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setValueName">setValueName</a>(), and <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDefaultValues">setDefaultValues</a>().</p></div>
    pub unsafe fn from_q_string_list(
        names: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) -> ::cpp_utils::CppBox<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineOption_QCommandLineOption1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(names)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a command line option object with the given arguments.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCommandLineOption::QCommandLineOption(const QString& name, const QString& description, const QString& valueName = …, const QString& defaultValue = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#QCommandLineOption-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a command line option object with the given arguments.</p>
    /// <p>The name of the option is set to <i>name</i>. The name can be either short or long. If the name is one character in length, it is considered a short name. Option names must not be empty, must not start with a dash or a slash character, must not contain a <code>=</code> and cannot be repeated.</p>
    /// <p>The description is set to <i>description</i>. It is customary to add a "." at the end of the description.</p>
    /// <p>In addition, the <i>valueName</i> needs to be set if the option expects a value. The default value for the option is set to <i>defaultValue</i>.</p>
    /// <p>In Qt versions before 5.4, this constructor was <code>explicit</code>. In Qt 5.4 and later, it no longer is and can be used for C++11-style uniform initialization:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qcommandlineparser.html">QCommandLineParser</a></span> parser;
    ///   parser<span class="operator">.</span>addOption({<span class="string">"verbose"</span><span class="operator">,</span> <span class="string">"Verbose mode. Prints out more information."</span>});
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDescription">setDescription</a>(), <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setValueName">setValueName</a>(), and <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDefaultValues">setDefaultValues</a>().</p></div>
    pub unsafe fn from_4_q_string(
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        description: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        value_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineOption_QCommandLineOption2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(description)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(value_name)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a command line option object with the given arguments.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCommandLineOption::QCommandLineOption(const QStringList& names, const QString& description, const QString& valueName = …, const QString& defaultValue = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#QCommandLineOption-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a command line option object with the given arguments.</p>
    /// <p>This overload allows to set multiple names for the option, for instance <code>o</code> and <code>output</code>.</p>
    /// <p>The names of the option are set to <i>names</i>. The names can be either short or long. Any name in the list that is one character in length is a short name. Option names must not be empty, must not start with a dash or a slash character, must not contain a <code>=</code> and cannot be repeated.</p>
    /// <p>The description is set to <i>description</i>. It is customary to add a "." at the end of the description.</p>
    /// <p>In addition, the <i>valueName</i> needs to be set if the option expects a value. The default value for the option is set to <i>defaultValue</i>.</p>
    /// <p>In Qt versions before 5.4, this constructor was <code>explicit</code>. In Qt 5.4 and later, it no longer is and can be used for C++11-style uniform initialization:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qcommandlineparser.html">QCommandLineParser</a></span> parser;
    ///   parser<span class="operator">.</span>addOption({{<span class="string">"o"</span><span class="operator">,</span> <span class="string">"output"</span>}<span class="operator">,</span> <span class="string">"Write generated data into &lt;file&gt;."</span><span class="operator">,</span> <span class="string">"file"</span>});
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDescription">setDescription</a>(), <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setValueName">setValueName</a>(), and <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDefaultValues">setDefaultValues</a>().</p></div>
    pub unsafe fn from_q_string_list3_q_string(
        names: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
        description: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        value_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineOption_QCommandLineOption3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(names)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(description)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(value_name)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a command line option object with the given arguments.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCommandLineOption::QCommandLineOption(const QString& name, const QString& description, const QString& valueName = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#QCommandLineOption-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a command line option object with the given arguments.</p>
    /// <p>The name of the option is set to <i>name</i>. The name can be either short or long. If the name is one character in length, it is considered a short name. Option names must not be empty, must not start with a dash or a slash character, must not contain a <code>=</code> and cannot be repeated.</p>
    /// <p>The description is set to <i>description</i>. It is customary to add a "." at the end of the description.</p>
    /// <p>In addition, the <i>valueName</i> needs to be set if the option expects a value. The default value for the option is set to <i>defaultValue</i>.</p>
    /// <p>In Qt versions before 5.4, this constructor was <code>explicit</code>. In Qt 5.4 and later, it no longer is and can be used for C++11-style uniform initialization:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qcommandlineparser.html">QCommandLineParser</a></span> parser;
    ///   parser<span class="operator">.</span>addOption({<span class="string">"verbose"</span><span class="operator">,</span> <span class="string">"Verbose mode. Prints out more information."</span>});
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDescription">setDescription</a>(), <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setValueName">setValueName</a>(), and <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDefaultValues">setDefaultValues</a>().</p></div>
    pub unsafe fn from_3_q_string(
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        description: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        value_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineOption_QCommandLineOption6(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(description)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(value_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a command line option object with the given arguments.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCommandLineOption::QCommandLineOption(const QString& name, const QString& description)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#QCommandLineOption-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a command line option object with the given arguments.</p>
    /// <p>The name of the option is set to <i>name</i>. The name can be either short or long. If the name is one character in length, it is considered a short name. Option names must not be empty, must not start with a dash or a slash character, must not contain a <code>=</code> and cannot be repeated.</p>
    /// <p>The description is set to <i>description</i>. It is customary to add a "." at the end of the description.</p>
    /// <p>In addition, the <i>valueName</i> needs to be set if the option expects a value. The default value for the option is set to <i>defaultValue</i>.</p>
    /// <p>In Qt versions before 5.4, this constructor was <code>explicit</code>. In Qt 5.4 and later, it no longer is and can be used for C++11-style uniform initialization:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qcommandlineparser.html">QCommandLineParser</a></span> parser;
    ///   parser<span class="operator">.</span>addOption({<span class="string">"verbose"</span><span class="operator">,</span> <span class="string">"Verbose mode. Prints out more information."</span>});
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDescription">setDescription</a>(), <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setValueName">setValueName</a>(), and <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDefaultValues">setDefaultValues</a>().</p></div>
    pub unsafe fn from_2_q_string(
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        description: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineOption_QCommandLineOption7(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(description)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a command line option object with the given arguments.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCommandLineOption::QCommandLineOption(const QStringList& names, const QString& description, const QString& valueName = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#QCommandLineOption-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a command line option object with the given arguments.</p>
    /// <p>This overload allows to set multiple names for the option, for instance <code>o</code> and <code>output</code>.</p>
    /// <p>The names of the option are set to <i>names</i>. The names can be either short or long. Any name in the list that is one character in length is a short name. Option names must not be empty, must not start with a dash or a slash character, must not contain a <code>=</code> and cannot be repeated.</p>
    /// <p>The description is set to <i>description</i>. It is customary to add a "." at the end of the description.</p>
    /// <p>In addition, the <i>valueName</i> needs to be set if the option expects a value. The default value for the option is set to <i>defaultValue</i>.</p>
    /// <p>In Qt versions before 5.4, this constructor was <code>explicit</code>. In Qt 5.4 and later, it no longer is and can be used for C++11-style uniform initialization:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qcommandlineparser.html">QCommandLineParser</a></span> parser;
    ///   parser<span class="operator">.</span>addOption({{<span class="string">"o"</span><span class="operator">,</span> <span class="string">"output"</span>}<span class="operator">,</span> <span class="string">"Write generated data into &lt;file&gt;."</span><span class="operator">,</span> <span class="string">"file"</span>});
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDescription">setDescription</a>(), <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setValueName">setValueName</a>(), and <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDefaultValues">setDefaultValues</a>().</p></div>
    pub unsafe fn from_q_string_list2_q_string(
        names: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
        description: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        value_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineOption_QCommandLineOption8(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(names)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(description)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(value_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a command line option object with the given arguments.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCommandLineOption::QCommandLineOption(const QStringList& names, const QString& description)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#QCommandLineOption-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a command line option object with the given arguments.</p>
    /// <p>This overload allows to set multiple names for the option, for instance <code>o</code> and <code>output</code>.</p>
    /// <p>The names of the option are set to <i>names</i>. The names can be either short or long. Any name in the list that is one character in length is a short name. Option names must not be empty, must not start with a dash or a slash character, must not contain a <code>=</code> and cannot be repeated.</p>
    /// <p>The description is set to <i>description</i>. It is customary to add a "." at the end of the description.</p>
    /// <p>In addition, the <i>valueName</i> needs to be set if the option expects a value. The default value for the option is set to <i>defaultValue</i>.</p>
    /// <p>In Qt versions before 5.4, this constructor was <code>explicit</code>. In Qt 5.4 and later, it no longer is and can be used for C++11-style uniform initialization:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qcommandlineparser.html">QCommandLineParser</a></span> parser;
    ///   parser<span class="operator">.</span>addOption({{<span class="string">"o"</span><span class="operator">,</span> <span class="string">"output"</span>}<span class="operator">,</span> <span class="string">"Write generated data into &lt;file&gt;."</span><span class="operator">,</span> <span class="string">"file"</span>});
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDescription">setDescription</a>(), <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setValueName">setValueName</a>(), and <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDefaultValues">setDefaultValues</a>().</p></div>
    pub unsafe fn from_q_string_list_q_string(
        names: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
        description: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineOption_QCommandLineOption9(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(names)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(description)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qcommandlineoption.html">QCommandLineOption</a> object that is a copy of the <a href="http://doc.qt.io/qt-5/qcommandlineoption.html">QCommandLineOption</a> object <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCommandLineOption::QCommandLineOption(const QCommandLineOption& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#QCommandLineOption-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qcommandlineoption.html">QCommandLineOption</a> object that is a copy of the <a href="http://doc.qt.io/qt-5/qcommandlineoption.html">QCommandLineOption</a> object <i>other</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineoption.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCommandLineOption>>,
    ) -> ::cpp_utils::CppBox<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineOption_QCommandLineOption4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCommandLineOption>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Sets the default value used for this option to <i>defaultValue</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCommandLineOption::setDefaultValue(const QString& defaultValue)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDefaultValue">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the default value used for this option to <i>defaultValue</i>.</p>
    /// <p>The default value is used if the user of the application does not specify the option on the command line.</p>
    /// <p>If <i>defaultValue</i> is empty, the option has no default values.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineoption.html#defaultValues">defaultValues</a>() and <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDefaultValues">setDefaultValues</a>().</p></div>
    pub unsafe fn set_default_value(
        &mut self,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCommandLineOption_setDefaultValue(
            self as *mut crate::QCommandLineOption,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(default_value)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the list of default values used for this option to <i>defaultValues</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCommandLineOption::setDefaultValues(const QStringList& defaultValues)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDefaultValues">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the list of default values used for this option to <i>defaultValues</i>.</p>
    /// <p>The default values are used if the user of the application does not specify the option on the command line.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineoption.html#defaultValues">defaultValues</a>() and <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDefaultValue">setDefaultValue</a>().</p></div>
    pub unsafe fn set_default_values(
        &mut self,
        default_values: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCommandLineOption_setDefaultValues(
            self as *mut crate::QCommandLineOption,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(
                default_values,
            )
            .as_raw_ptr(),
        )
    }

    /// <p>Sets the description used for this option to <i>description</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCommandLineOption::setDescription(const QString& description)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDescription">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the description used for this option to <i>description</i>.</p>
    /// <p>It is customary to add a "." at the end of the description.</p>
    /// <p>The description is used by <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#showHelp">QCommandLineParser::showHelp</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineoption.html#description">description</a>().</p></div>
    pub unsafe fn set_description(
        &mut self,
        description: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCommandLineOption_setDescription(
            self as *mut crate::QCommandLineOption,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(description)
                .as_raw_ptr(),
        )
    }

    /// <p>Set the set of flags that affect this command-line option to <i>flags</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCommandLineOption::setFlags(QFlags<QCommandLineOption::Flag> aflags)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setFlags">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Set the set of flags that affect this command-line option to <i>flags</i>.</p>
    /// <p>This function was introduced in  Qt 5.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineoption.html#flags">flags</a>() and <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#Flag-enum">QCommandLineOption::Flags</a>.</p></div>
    pub unsafe fn set_flags(&mut self, aflags: crate::QFlags<crate::q_command_line_option::Flag>) {
        crate::__ffi::ctr_qt_core_ffi_QCommandLineOption_setFlags(
            self as *mut crate::QCommandLineOption,
            aflags.to_int(),
        )
    }

    /// <p>Sets whether to hide this option in the user-visible help output.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCommandLineOption::setHidden(bool hidden)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineoption-obsolete.html#setHidden">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets whether to hide this option in the user-visible help output.</p>
    /// <p>All options are visible by default. Setting <i>hide</i> to true for a particular option makes it internal, i.e. not listed in the help output.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineoption-obsolete.html#isHidden">isHidden</a>.</p></div>
    pub unsafe fn set_hidden(&mut self, hidden: bool) {
        crate::__ffi::ctr_qt_core_ffi_QCommandLineOption_setHidden(
            self as *mut crate::QCommandLineOption,
            hidden,
        )
    }

    /// <p>Sets the name of the expected value, for the documentation, to <i>valueName</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCommandLineOption::setValueName(const QString& name)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setValueName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the name of the expected value, for the documentation, to <i>valueName</i>.</p>
    /// <p>Options without a value assigned have a boolean-like behavior: either the user specifies --option or they don't.</p>
    /// <p>Options with a value assigned need to set a name for the expected value, for the documentation of the option in the help output. An option with names <code>o</code> and <code>output</code>, and a value name of <code>file</code> will appear as <code>-o, --output &lt;file&gt;</code>.</p>
    /// <p>Call <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#value">QCommandLineParser::value</a>() if you expect the option to be present only once, and <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#values">QCommandLineParser::values</a>() if you expect that option to be present multiple times.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineoption.html#valueName">valueName</a>().</p></div>
    pub unsafe fn set_value_name(
        &mut self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCommandLineOption_setValueName(
            self as *mut crate::QCommandLineOption,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
        )
    }

    /// <p>Swaps option <i>other</i> with this option. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCommandLineOption::swap(QCommandLineOption& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps option <i>other</i> with this option. This operation is very fast and never fails.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QCommandLineOption>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCommandLineOption_swap(
            self as *mut crate::QCommandLineOption,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QCommandLineOption>>::cast_into(
                other,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the name of the expected value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QCommandLineOption::valueName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#valueName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the name of the expected value.</p>
    /// <p>If empty, the option doesn't take a value.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setValueName">setValueName</a>().</p></div>
    pub unsafe fn value_name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineOption_valueName(
            self as *const crate::QCommandLineOption,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_event {
    //! C++ type: <span style='color: green;'>```QEvent```</span>

    /// <p>This enum type defines the valid event types in Qt. The event types and the specialized classes for each type are as follows:</p>
    ///
    /// C++ enum: <span style='color: green;'>```QEvent::Type```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type defines the valid event types in Qt. The event types and the specialized classes for each type are as follows:</p>
    ///
    /// <p>User events should have values between <code>User</code> and <code>MaxUser</code>:</p>
    ///
    /// <p>For convenience, you can use the <a href="http://doc.qt.io/qt-5/qevent.html#registerEventType">registerEventType</a>() function to register and reserve a custom event type for your application. Doing so will allow you to avoid accidentally re-using a custom event type already in use elsewhere in your application.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Type(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Type {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Type(value)
        }
    }

    impl From<Type> for ::std::os::raw::c_int {
        fn from(value: Type) -> Self {
            value.0
        }
    }

    impl Type {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Type {
        /// Not an event. (C++ enum variant: <span style='color: green;'>```None = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const None: crate::q_event::Type = crate::q_event::Type(0);
        /// Regular timer events (<a href="http://doc.qt.io/qt-5/qtimerevent.html">QTimerEvent</a>). (C++ enum variant: <span style='color: green;'>```Timer = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Timer: crate::q_event::Type = crate::q_event::Type(1);
        /// Mouse press (<a href="http://doc.qt.io/qt-5/qmouseevent.html">QMouseEvent</a>). (C++ enum variant: <span style='color: green;'>```MouseButtonPress = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const MouseButtonPress: crate::q_event::Type = crate::q_event::Type(2);
        /// Mouse release (<a href="http://doc.qt.io/qt-5/qmouseevent.html">QMouseEvent</a>). (C++ enum variant: <span style='color: green;'>```MouseButtonRelease = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const MouseButtonRelease: crate::q_event::Type = crate::q_event::Type(3);
        /// Mouse press again (<a href="http://doc.qt.io/qt-5/qmouseevent.html">QMouseEvent</a>). (C++ enum variant: <span style='color: green;'>```MouseButtonDblClick = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const MouseButtonDblClick: crate::q_event::Type = crate::q_event::Type(4);
        /// Mouse move (<a href="http://doc.qt.io/qt-5/qmouseevent.html">QMouseEvent</a>). (C++ enum variant: <span style='color: green;'>```MouseMove = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const MouseMove: crate::q_event::Type = crate::q_event::Type(5);
        /// Key press (<a href="http://doc.qt.io/qt-5/qkeyevent.html">QKeyEvent</a>). (C++ enum variant: <span style='color: green;'>```KeyPress = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const KeyPress: crate::q_event::Type = crate::q_event::Type(6);
        /// Key release (<a href="http://doc.qt.io/qt-5/qkeyevent.html">QKeyEvent</a>). (C++ enum variant: <span style='color: green;'>```KeyRelease = 7```</span>)
        #[allow(non_upper_case_globals)]
        pub const KeyRelease: crate::q_event::Type = crate::q_event::Type(7);
        /// Widget or Window gains keyboard focus (<a href="http://doc.qt.io/qt-5/qfocusevent.html">QFocusEvent</a>). (C++ enum variant: <span style='color: green;'>```FocusIn = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const FocusIn: crate::q_event::Type = crate::q_event::Type(8);
        /// Widget or Window loses keyboard focus (<a href="http://doc.qt.io/qt-5/qfocusevent.html">QFocusEvent</a>). (C++ enum variant: <span style='color: green;'>```FocusOut = 9```</span>)
        #[allow(non_upper_case_globals)]
        pub const FocusOut: crate::q_event::Type = crate::q_event::Type(9);
        /// Widget or Window focus is about to change (<a href="http://doc.qt.io/qt-5/qfocusevent.html">QFocusEvent</a>) (C++ enum variant: <span style='color: green;'>```FocusAboutToChange = 23```</span>)
        #[allow(non_upper_case_globals)]
        pub const FocusAboutToChange: crate::q_event::Type = crate::q_event::Type(23);
        /// Mouse enters widget's boundaries (<a href="http://doc.qt.io/qt-5/qenterevent.html">QEnterEvent</a>). (C++ enum variant: <span style='color: green;'>```Enter = 10```</span>)
        #[allow(non_upper_case_globals)]
        pub const Enter: crate::q_event::Type = crate::q_event::Type(10);
        /// Mouse leaves widget's boundaries. (C++ enum variant: <span style='color: green;'>```Leave = 11```</span>)
        #[allow(non_upper_case_globals)]
        pub const Leave: crate::q_event::Type = crate::q_event::Type(11);
        /// Screen update necessary (<a href="http://doc.qt.io/qt-5/qpaintevent.html">QPaintEvent</a>). (C++ enum variant: <span style='color: green;'>```Paint = 12```</span>)
        #[allow(non_upper_case_globals)]
        pub const Paint: crate::q_event::Type = crate::q_event::Type(12);
        /// Widget's position changed (<a href="http://doc.qt.io/qt-5/qmoveevent.html">QMoveEvent</a>). (C++ enum variant: <span style='color: green;'>```Move = 13```</span>)
        #[allow(non_upper_case_globals)]
        pub const Move: crate::q_event::Type = crate::q_event::Type(13);
        /// Widget's size changed (<a href="http://doc.qt.io/qt-5/qresizeevent.html">QResizeEvent</a>). (C++ enum variant: <span style='color: green;'>```Resize = 14```</span>)
        #[allow(non_upper_case_globals)]
        pub const Resize: crate::q_event::Type = crate::q_event::Type(14);
        /// C++ enum variant: <span style='color: green;'>```Create = 15```</span>
        #[allow(non_upper_case_globals)]
        pub const Create: crate::q_event::Type = crate::q_event::Type(15);
        /// C++ enum variant: <span style='color: green;'>```Destroy = 16```</span>
        #[allow(non_upper_case_globals)]
        pub const Destroy: crate::q_event::Type = crate::q_event::Type(16);
        /// Widget was shown on screen (<a href="http://doc.qt.io/qt-5/qshowevent.html">QShowEvent</a>). (C++ enum variant: <span style='color: green;'>```Show = 17```</span>)
        #[allow(non_upper_case_globals)]
        pub const Show: crate::q_event::Type = crate::q_event::Type(17);
        /// Widget was hidden (<a href="http://doc.qt.io/qt-5/qhideevent.html">QHideEvent</a>). (C++ enum variant: <span style='color: green;'>```Hide = 18```</span>)
        #[allow(non_upper_case_globals)]
        pub const Hide: crate::q_event::Type = crate::q_event::Type(18);
        /// Widget was closed (<a href="http://doc.qt.io/qt-5/qcloseevent.html">QCloseEvent</a>). (C++ enum variant: <span style='color: green;'>```Close = 19```</span>)
        #[allow(non_upper_case_globals)]
        pub const Close: crate::q_event::Type = crate::q_event::Type(19);
        /// C++ enum variant: <span style='color: green;'>```Quit = 20```</span>
        #[allow(non_upper_case_globals)]
        pub const Quit: crate::q_event::Type = crate::q_event::Type(20);
        /// The widget parent has changed. (C++ enum variant: <span style='color: green;'>```ParentChange = 21```</span>)
        #[allow(non_upper_case_globals)]
        pub const ParentChange: crate::q_event::Type = crate::q_event::Type(21);
        /// The widget parent is about to change. (C++ enum variant: <span style='color: green;'>```ParentAboutToChange = 131```</span>)
        #[allow(non_upper_case_globals)]
        pub const ParentAboutToChange: crate::q_event::Type = crate::q_event::Type(131);
        /// The object is moved to another thread. This is the last event sent to this object in the previous thread. See <a href="http://doc.qt.io/qt-5/qobject.html#moveToThread">QObject::moveToThread</a>(). (C++ enum variant: <span style='color: green;'>```ThreadChange = 22```</span>)
        #[allow(non_upper_case_globals)]
        pub const ThreadChange: crate::q_event::Type = crate::q_event::Type(22);
        /// Window was activated. (C++ enum variant: <span style='color: green;'>```WindowActivate = 24```</span>)
        #[allow(non_upper_case_globals)]
        pub const WindowActivate: crate::q_event::Type = crate::q_event::Type(24);
        /// Window was deactivated. (C++ enum variant: <span style='color: green;'>```WindowDeactivate = 25```</span>)
        #[allow(non_upper_case_globals)]
        pub const WindowDeactivate: crate::q_event::Type = crate::q_event::Type(25);
        /// A child widget has been shown. (C++ enum variant: <span style='color: green;'>```ShowToParent = 26```</span>)
        #[allow(non_upper_case_globals)]
        pub const ShowToParent: crate::q_event::Type = crate::q_event::Type(26);
        /// A child widget has been hidden. (C++ enum variant: <span style='color: green;'>```HideToParent = 27```</span>)
        #[allow(non_upper_case_globals)]
        pub const HideToParent: crate::q_event::Type = crate::q_event::Type(27);
        /// Mouse wheel rolled (<a href="http://doc.qt.io/qt-5/qwheelevent.html">QWheelEvent</a>). (C++ enum variant: <span style='color: green;'>```Wheel = 31```</span>)
        #[allow(non_upper_case_globals)]
        pub const Wheel: crate::q_event::Type = crate::q_event::Type(31);
        /// The window title has changed. (C++ enum variant: <span style='color: green;'>```WindowTitleChange = 33```</span>)
        #[allow(non_upper_case_globals)]
        pub const WindowTitleChange: crate::q_event::Type = crate::q_event::Type(33);
        /// The window's icon has changed. (C++ enum variant: <span style='color: green;'>```WindowIconChange = 34```</span>)
        #[allow(non_upper_case_globals)]
        pub const WindowIconChange: crate::q_event::Type = crate::q_event::Type(34);
        /// The application's icon has changed. (C++ enum variant: <span style='color: green;'>```ApplicationWindowIconChange = 35```</span>)
        #[allow(non_upper_case_globals)]
        pub const ApplicationWindowIconChange: crate::q_event::Type = crate::q_event::Type(35);
        /// The default application font has changed. (C++ enum variant: <span style='color: green;'>```ApplicationFontChange = 36```</span>)
        #[allow(non_upper_case_globals)]
        pub const ApplicationFontChange: crate::q_event::Type = crate::q_event::Type(36);
        /// The default application layout direction has changed. (C++ enum variant: <span style='color: green;'>```ApplicationLayoutDirectionChange = 37```</span>)
        #[allow(non_upper_case_globals)]
        pub const ApplicationLayoutDirectionChange: crate::q_event::Type = crate::q_event::Type(37);
        /// The default application palette has changed. (C++ enum variant: <span style='color: green;'>```ApplicationPaletteChange = 38```</span>)
        #[allow(non_upper_case_globals)]
        pub const ApplicationPaletteChange: crate::q_event::Type = crate::q_event::Type(38);
        /// Palette of the widget changed. (C++ enum variant: <span style='color: green;'>```PaletteChange = 39```</span>)
        #[allow(non_upper_case_globals)]
        pub const PaletteChange: crate::q_event::Type = crate::q_event::Type(39);
        /// The clipboard contents have changed. (C++ enum variant: <span style='color: green;'>```Clipboard = 40```</span>)
        #[allow(non_upper_case_globals)]
        pub const Clipboard: crate::q_event::Type = crate::q_event::Type(40);
        /// C++ enum variant: <span style='color: green;'>```Speech = 42```</span>
        #[allow(non_upper_case_globals)]
        pub const Speech: crate::q_event::Type = crate::q_event::Type(42);
        /// An asynchronous method invocation via <a href="http://doc.qt.io/qt-5/qmetaobject.html#invokeMethod">QMetaObject::invokeMethod</a>(). (C++ enum variant: <span style='color: green;'>```MetaCall = 43```</span>)
        #[allow(non_upper_case_globals)]
        pub const MetaCall: crate::q_event::Type = crate::q_event::Type(43);
        /// Socket activated, used to implement <a href="http://doc.qt.io/qt-5/qsocketnotifier.html">QSocketNotifier</a>. (C++ enum variant: <span style='color: green;'>```SockAct = 50```</span>)
        #[allow(non_upper_case_globals)]
        pub const SockAct: crate::q_event::Type = crate::q_event::Type(50);
        /// A Windows-specific activation event has occurred. (C++ enum variant: <span style='color: green;'>```WinEventAct = 132```</span>)
        #[allow(non_upper_case_globals)]
        pub const WinEventAct: crate::q_event::Type = crate::q_event::Type(132);
        /// The object will be deleted after it has cleaned up (QDeferredDeleteEvent) (C++ enum variant: <span style='color: green;'>```DeferredDelete = 52```</span>)
        #[allow(non_upper_case_globals)]
        pub const DeferredDelete: crate::q_event::Type = crate::q_event::Type(52);
        /// The cursor enters a widget during a drag and drop operation (<a href="http://doc.qt.io/qt-5/qdragenterevent.html">QDragEnterEvent</a>). (C++ enum variant: <span style='color: green;'>```DragEnter = 60```</span>)
        #[allow(non_upper_case_globals)]
        pub const DragEnter: crate::q_event::Type = crate::q_event::Type(60);
        /// A drag and drop operation is in progress (<a href="http://doc.qt.io/qt-5/qdragmoveevent.html">QDragMoveEvent</a>). (C++ enum variant: <span style='color: green;'>```DragMove = 61```</span>)
        #[allow(non_upper_case_globals)]
        pub const DragMove: crate::q_event::Type = crate::q_event::Type(61);
        /// The cursor leaves a widget during a drag and drop operation (<a href="http://doc.qt.io/qt-5/qdragleaveevent.html">QDragLeaveEvent</a>). (C++ enum variant: <span style='color: green;'>```DragLeave = 62```</span>)
        #[allow(non_upper_case_globals)]
        pub const DragLeave: crate::q_event::Type = crate::q_event::Type(62);
        /// A drag and drop operation is completed (<a href="http://doc.qt.io/qt-5/qdropevent.html">QDropEvent</a>). (C++ enum variant: <span style='color: green;'>```Drop = 63```</span>)
        #[allow(non_upper_case_globals)]
        pub const Drop: crate::q_event::Type = crate::q_event::Type(63);
        /// C++ enum variant: <span style='color: green;'>```DragResponse = 64```</span>
        #[allow(non_upper_case_globals)]
        pub const DragResponse: crate::q_event::Type = crate::q_event::Type(64);
        /// An object gets a child (<a href="http://doc.qt.io/qt-5/qchildevent.html">QChildEvent</a>). (C++ enum variant: <span style='color: green;'>```ChildAdded = 68```</span>)
        #[allow(non_upper_case_globals)]
        pub const ChildAdded: crate::q_event::Type = crate::q_event::Type(68);
        /// A widget child gets polished (<a href="http://doc.qt.io/qt-5/qchildevent.html">QChildEvent</a>). (C++ enum variant: <span style='color: green;'>```ChildPolished = 69```</span>)
        #[allow(non_upper_case_globals)]
        pub const ChildPolished: crate::q_event::Type = crate::q_event::Type(69);
        /// An object loses a child (<a href="http://doc.qt.io/qt-5/qchildevent.html">QChildEvent</a>). (C++ enum variant: <span style='color: green;'>```ChildRemoved = 71```</span>)
        #[allow(non_upper_case_globals)]
        pub const ChildRemoved: crate::q_event::Type = crate::q_event::Type(71);
        /// C++ enum variant: <span style='color: green;'>```ShowWindowRequest = 73```</span>
        #[allow(non_upper_case_globals)]
        pub const ShowWindowRequest: crate::q_event::Type = crate::q_event::Type(73);
        /// The widget should be polished. (C++ enum variant: <span style='color: green;'>```PolishRequest = 74```</span>)
        #[allow(non_upper_case_globals)]
        pub const PolishRequest: crate::q_event::Type = crate::q_event::Type(74);
        /// The widget is polished. (C++ enum variant: <span style='color: green;'>```Polish = 75```</span>)
        #[allow(non_upper_case_globals)]
        pub const Polish: crate::q_event::Type = crate::q_event::Type(75);
        /// Widget layout needs to be redone. (C++ enum variant: <span style='color: green;'>```LayoutRequest = 76```</span>)
        #[allow(non_upper_case_globals)]
        pub const LayoutRequest: crate::q_event::Type = crate::q_event::Type(76);
        /// The widget should be repainted. (C++ enum variant: <span style='color: green;'>```UpdateRequest = 77```</span>)
        #[allow(non_upper_case_globals)]
        pub const UpdateRequest: crate::q_event::Type = crate::q_event::Type(77);
        /// The widget should be queued to be repainted at a later time. (C++ enum variant: <span style='color: green;'>```UpdateLater = 78```</span>)
        #[allow(non_upper_case_globals)]
        pub const UpdateLater: crate::q_event::Type = crate::q_event::Type(78);
        /// C++ enum variant: <span style='color: green;'>```EmbeddingControl = 79```</span>
        #[allow(non_upper_case_globals)]
        pub const EmbeddingControl: crate::q_event::Type = crate::q_event::Type(79);
        /// C++ enum variant: <span style='color: green;'>```ActivateControl = 80```</span>
        #[allow(non_upper_case_globals)]
        pub const ActivateControl: crate::q_event::Type = crate::q_event::Type(80);
        /// C++ enum variant: <span style='color: green;'>```DeactivateControl = 81```</span>
        #[allow(non_upper_case_globals)]
        pub const DeactivateControl: crate::q_event::Type = crate::q_event::Type(81);
        /// Context popup menu (<a href="http://doc.qt.io/qt-5/qcontextmenuevent.html">QContextMenuEvent</a>). (C++ enum variant: <span style='color: green;'>```ContextMenu = 82```</span>)
        #[allow(non_upper_case_globals)]
        pub const ContextMenu: crate::q_event::Type = crate::q_event::Type(82);
        /// An input method is being used (<a href="http://doc.qt.io/qt-5/qinputmethodevent.html">QInputMethodEvent</a>). (C++ enum variant: <span style='color: green;'>```InputMethod = 83```</span>)
        #[allow(non_upper_case_globals)]
        pub const InputMethod: crate::q_event::Type = crate::q_event::Type(83);
        /// Wacom tablet move (<a href="http://doc.qt.io/qt-5/qtabletevent.html">QTabletEvent</a>). (C++ enum variant: <span style='color: green;'>```TabletMove = 87```</span>)
        #[allow(non_upper_case_globals)]
        pub const TabletMove: crate::q_event::Type = crate::q_event::Type(87);
        /// The system locale has changed. (C++ enum variant: <span style='color: green;'>```LocaleChange = 88```</span>)
        #[allow(non_upper_case_globals)]
        pub const LocaleChange: crate::q_event::Type = crate::q_event::Type(88);
        /// The application translation changed. (C++ enum variant: <span style='color: green;'>```LanguageChange = 89```</span>)
        #[allow(non_upper_case_globals)]
        pub const LanguageChange: crate::q_event::Type = crate::q_event::Type(89);
        /// The direction of layouts changed. (C++ enum variant: <span style='color: green;'>```LayoutDirectionChange = 90```</span>)
        #[allow(non_upper_case_globals)]
        pub const LayoutDirectionChange: crate::q_event::Type = crate::q_event::Type(90);
        /// C++ enum variant: <span style='color: green;'>```Style = 91```</span>
        #[allow(non_upper_case_globals)]
        pub const Style: crate::q_event::Type = crate::q_event::Type(91);
        /// Wacom tablet press (<a href="http://doc.qt.io/qt-5/qtabletevent.html">QTabletEvent</a>). (C++ enum variant: <span style='color: green;'>```TabletPress = 92```</span>)
        #[allow(non_upper_case_globals)]
        pub const TabletPress: crate::q_event::Type = crate::q_event::Type(92);
        /// Wacom tablet release (<a href="http://doc.qt.io/qt-5/qtabletevent.html">QTabletEvent</a>). (C++ enum variant: <span style='color: green;'>```TabletRelease = 93```</span>)
        #[allow(non_upper_case_globals)]
        pub const TabletRelease: crate::q_event::Type = crate::q_event::Type(93);
        /// C++ enum variant: <span style='color: green;'>```OkRequest = 94```</span>
        #[allow(non_upper_case_globals)]
        pub const OkRequest: crate::q_event::Type = crate::q_event::Type(94);
        /// C++ enum variant: <span style='color: green;'>```HelpRequest = 95```</span>
        #[allow(non_upper_case_globals)]
        pub const HelpRequest: crate::q_event::Type = crate::q_event::Type(95);
        /// The main icon of a window has been dragged away (<a href="http://doc.qt.io/qt-5/qicondragevent.html">QIconDragEvent</a>). (C++ enum variant: <span style='color: green;'>```IconDrag = 96```</span>)
        #[allow(non_upper_case_globals)]
        pub const IconDrag: crate::q_event::Type = crate::q_event::Type(96);
        /// Widget's font has changed. (C++ enum variant: <span style='color: green;'>```FontChange = 97```</span>)
        #[allow(non_upper_case_globals)]
        pub const FontChange: crate::q_event::Type = crate::q_event::Type(97);
        /// Widget's enabled state has changed. (C++ enum variant: <span style='color: green;'>```EnabledChange = 98```</span>)
        #[allow(non_upper_case_globals)]
        pub const EnabledChange: crate::q_event::Type = crate::q_event::Type(98);
        /// A widget's top-level window activation state has changed. (C++ enum variant: <span style='color: green;'>```ActivationChange = 99```</span>)
        #[allow(non_upper_case_globals)]
        pub const ActivationChange: crate::q_event::Type = crate::q_event::Type(99);
        /// Widget's style has been changed. (C++ enum variant: <span style='color: green;'>```StyleChange = 100```</span>)
        #[allow(non_upper_case_globals)]
        pub const StyleChange: crate::q_event::Type = crate::q_event::Type(100);
        /// Widget's icon text has been changed. (Deprecated) (C++ enum variant: <span style='color: green;'>```IconTextChange = 101```</span>)
        #[allow(non_upper_case_globals)]
        pub const IconTextChange: crate::q_event::Type = crate::q_event::Type(101);
        /// Widgets modification state has been changed. (C++ enum variant: <span style='color: green;'>```ModifiedChange = 102```</span>)
        #[allow(non_upper_case_globals)]
        pub const ModifiedChange: crate::q_event::Type = crate::q_event::Type(102);
        /// The mouse tracking state has changed. (C++ enum variant: <span style='color: green;'>```MouseTrackingChange = 109```</span>)
        #[allow(non_upper_case_globals)]
        pub const MouseTrackingChange: crate::q_event::Type = crate::q_event::Type(109);
        /// The window is blocked by a modal dialog. (C++ enum variant: <span style='color: green;'>```WindowBlocked = 103```</span>)
        #[allow(non_upper_case_globals)]
        pub const WindowBlocked: crate::q_event::Type = crate::q_event::Type(103);
        /// The window is unblocked after a modal dialog exited. (C++ enum variant: <span style='color: green;'>```WindowUnblocked = 104```</span>)
        #[allow(non_upper_case_globals)]
        pub const WindowUnblocked: crate::q_event::Type = crate::q_event::Type(104);
        /// The <a href="http://doc.qt.io/qt-5/qwindow.html#windowState">window's state</a> (minimized, maximized or full-screen) has changed (<a href="http://doc.qt.io/qt-5/qwindowstatechangeevent.html">QWindowStateChangeEvent</a>). (C++ enum variant: <span style='color: green;'>```WindowStateChange = 105```</span>)
        #[allow(non_upper_case_globals)]
        pub const WindowStateChange: crate::q_event::Type = crate::q_event::Type(105);
        /// Widget's read-only state has changed (since Qt 5.4). (C++ enum variant: <span style='color: green;'>```ReadOnlyChange = 106```</span>)
        #[allow(non_upper_case_globals)]
        pub const ReadOnlyChange: crate::q_event::Type = crate::q_event::Type(106);
        /// A tooltip was requested (<a href="http://doc.qt.io/qt-5/qhelpevent.html">QHelpEvent</a>). (C++ enum variant: <span style='color: green;'>```ToolTip = 110```</span>)
        #[allow(non_upper_case_globals)]
        pub const ToolTip: crate::q_event::Type = crate::q_event::Type(110);
        /// The widget should reveal "What's This?" help (<a href="http://doc.qt.io/qt-5/qhelpevent.html">QHelpEvent</a>). (C++ enum variant: <span style='color: green;'>```WhatsThis = 111```</span>)
        #[allow(non_upper_case_globals)]
        pub const WhatsThis: crate::q_event::Type = crate::q_event::Type(111);
        /// A status tip is requested (<a href="http://doc.qt.io/qt-5/qstatustipevent.html">QStatusTipEvent</a>). (C++ enum variant: <span style='color: green;'>```StatusTip = 112```</span>)
        #[allow(non_upper_case_globals)]
        pub const StatusTip: crate::q_event::Type = crate::q_event::Type(112);
        /// An action has been changed (<a href="http://doc.qt.io/qt-5/qactionevent.html">QActionEvent</a>). (C++ enum variant: <span style='color: green;'>```ActionChanged = 113```</span>)
        #[allow(non_upper_case_globals)]
        pub const ActionChanged: crate::q_event::Type = crate::q_event::Type(113);
        /// A new action has been added (<a href="http://doc.qt.io/qt-5/qactionevent.html">QActionEvent</a>). (C++ enum variant: <span style='color: green;'>```ActionAdded = 114```</span>)
        #[allow(non_upper_case_globals)]
        pub const ActionAdded: crate::q_event::Type = crate::q_event::Type(114);
        /// An action has been removed (<a href="http://doc.qt.io/qt-5/qactionevent.html">QActionEvent</a>). (C++ enum variant: <span style='color: green;'>```ActionRemoved = 115```</span>)
        #[allow(non_upper_case_globals)]
        pub const ActionRemoved: crate::q_event::Type = crate::q_event::Type(115);
        /// File open request (<a href="http://doc.qt.io/qt-5/qfileopenevent.html">QFileOpenEvent</a>). (C++ enum variant: <span style='color: green;'>```FileOpen = 116```</span>)
        #[allow(non_upper_case_globals)]
        pub const FileOpen: crate::q_event::Type = crate::q_event::Type(116);
        /// Key press in child for shortcut key handling (<a href="http://doc.qt.io/qt-5/qshortcutevent.html">QShortcutEvent</a>). (C++ enum variant: <span style='color: green;'>```Shortcut = 117```</span>)
        #[allow(non_upper_case_globals)]
        pub const Shortcut: crate::q_event::Type = crate::q_event::Type(117);
        /// Key press in child, for overriding shortcut key handling (<a href="http://doc.qt.io/qt-5/qkeyevent.html">QKeyEvent</a>). (C++ enum variant: <span style='color: green;'>```ShortcutOverride = 51```</span>)
        #[allow(non_upper_case_globals)]
        pub const ShortcutOverride: crate::q_event::Type = crate::q_event::Type(51);
        /// A link in a widget's "What's This?" help was clicked. (C++ enum variant: <span style='color: green;'>```WhatsThisClicked = 118```</span>)
        #[allow(non_upper_case_globals)]
        pub const WhatsThisClicked: crate::q_event::Type = crate::q_event::Type(118);
        /// The toolbar button is toggled on <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>. (C++ enum variant: <span style='color: green;'>```ToolBarChange = 120```</span>)
        #[allow(non_upper_case_globals)]
        pub const ToolBarChange: crate::q_event::Type = crate::q_event::Type(120);
        /// This enum has been deprecated. Use ApplicationStateChange instead. (C++ enum variant: <span style='color: green;'>```ApplicationActivate = 121```</span>)
        #[allow(non_upper_case_globals)]
        pub const ApplicationActivate: crate::q_event::Type = crate::q_event::Type(121);
        /// This enum has been deprecated. Use ApplicationStateChange instead. (C++ enum variant: <span style='color: green;'>```ApplicationActivated = 121```</span>)
        #[allow(non_upper_case_globals)]
        pub const ApplicationActivated: crate::q_event::Type = crate::q_event::Type(121);
        /// This enum has been deprecated. Use ApplicationStateChange instead. (C++ enum variant: <span style='color: green;'>```ApplicationDeactivate = 122```</span>)
        #[allow(non_upper_case_globals)]
        pub const ApplicationDeactivate: crate::q_event::Type = crate::q_event::Type(122);
        /// C++ enum variant: <span style='color: green;'>```ApplicationDeactivated = 122```</span>
        #[allow(non_upper_case_globals)]
        pub const ApplicationDeactivated: crate::q_event::Type = crate::q_event::Type(122);
        /// The widget should accept the event if it has "What's This?" help. (C++ enum variant: <span style='color: green;'>```QueryWhatsThis = 123```</span>)
        #[allow(non_upper_case_globals)]
        pub const QueryWhatsThis: crate::q_event::Type = crate::q_event::Type(123);
        /// Send to toplevel widgets when the application enters "What's This?" mode. (C++ enum variant: <span style='color: green;'>```EnterWhatsThisMode = 124```</span>)
        #[allow(non_upper_case_globals)]
        pub const EnterWhatsThisMode: crate::q_event::Type = crate::q_event::Type(124);
        /// Send to toplevel widgets when the application leaves "What's This?" mode. (C++ enum variant: <span style='color: green;'>```LeaveWhatsThisMode = 125```</span>)
        #[allow(non_upper_case_globals)]
        pub const LeaveWhatsThisMode: crate::q_event::Type = crate::q_event::Type(125);
        /// The widget's z-order has changed. This event is never sent to top level windows. (C++ enum variant: <span style='color: green;'>```ZOrderChange = 126```</span>)
        #[allow(non_upper_case_globals)]
        pub const ZOrderChange: crate::q_event::Type = crate::q_event::Type(126);
        /// The mouse cursor enters a hover widget (<a href="http://doc.qt.io/qt-5/qhoverevent.html">QHoverEvent</a>). (C++ enum variant: <span style='color: green;'>```HoverEnter = 127```</span>)
        #[allow(non_upper_case_globals)]
        pub const HoverEnter: crate::q_event::Type = crate::q_event::Type(127);
        /// The mouse cursor leaves a hover widget (<a href="http://doc.qt.io/qt-5/qhoverevent.html">QHoverEvent</a>). (C++ enum variant: <span style='color: green;'>```HoverLeave = 128```</span>)
        #[allow(non_upper_case_globals)]
        pub const HoverLeave: crate::q_event::Type = crate::q_event::Type(128);
        /// The mouse cursor moves inside a hover widget (<a href="http://doc.qt.io/qt-5/qhoverevent.html">QHoverEvent</a>). (C++ enum variant: <span style='color: green;'>```HoverMove = 129```</span>)
        #[allow(non_upper_case_globals)]
        pub const HoverMove: crate::q_event::Type = crate::q_event::Type(129);
        /// C++ enum variant: <span style='color: green;'>```AcceptDropsChange = 152```</span>
        #[allow(non_upper_case_globals)]
        pub const AcceptDropsChange: crate::q_event::Type = crate::q_event::Type(152);
        /// C++ enum variant: <span style='color: green;'>```ZeroTimerEvent = 154```</span>
        #[allow(non_upper_case_globals)]
        pub const ZeroTimerEvent: crate::q_event::Type = crate::q_event::Type(154);
        /// Move mouse in a graphics scene (<a href="http://doc.qt.io/qt-5/qgraphicsscenemouseevent.html">QGraphicsSceneMouseEvent</a>). (C++ enum variant: <span style='color: green;'>```GraphicsSceneMouseMove = 155```</span>)
        #[allow(non_upper_case_globals)]
        pub const GraphicsSceneMouseMove: crate::q_event::Type = crate::q_event::Type(155);
        /// Mouse press in a graphics scene (<a href="http://doc.qt.io/qt-5/qgraphicsscenemouseevent.html">QGraphicsSceneMouseEvent</a>). (C++ enum variant: <span style='color: green;'>```GraphicsSceneMousePress = 156```</span>)
        #[allow(non_upper_case_globals)]
        pub const GraphicsSceneMousePress: crate::q_event::Type = crate::q_event::Type(156);
        /// Mouse release in a graphics scene (<a href="http://doc.qt.io/qt-5/qgraphicsscenemouseevent.html">QGraphicsSceneMouseEvent</a>). (C++ enum variant: <span style='color: green;'>```GraphicsSceneMouseRelease = 157```</span>)
        #[allow(non_upper_case_globals)]
        pub const GraphicsSceneMouseRelease: crate::q_event::Type = crate::q_event::Type(157);
        /// Mouse press again (double click) in a graphics scene (<a href="http://doc.qt.io/qt-5/qgraphicsscenemouseevent.html">QGraphicsSceneMouseEvent</a>). (C++ enum variant: <span style='color: green;'>```GraphicsSceneMouseDoubleClick = 158```</span>)
        #[allow(non_upper_case_globals)]
        pub const GraphicsSceneMouseDoubleClick: crate::q_event::Type = crate::q_event::Type(158);
        /// Context popup menu over a graphics scene (<a href="http://doc.qt.io/qt-5/qgraphicsscenecontextmenuevent.html">QGraphicsSceneContextMenuEvent</a>). (C++ enum variant: <span style='color: green;'>```GraphicsSceneContextMenu = 159```</span>)
        #[allow(non_upper_case_globals)]
        pub const GraphicsSceneContextMenu: crate::q_event::Type = crate::q_event::Type(159);
        /// The mouse cursor enters a hover item in a graphics scene (<a href="http://doc.qt.io/qt-5/qgraphicsscenehoverevent.html">QGraphicsSceneHoverEvent</a>). (C++ enum variant: <span style='color: green;'>```GraphicsSceneHoverEnter = 160```</span>)
        #[allow(non_upper_case_globals)]
        pub const GraphicsSceneHoverEnter: crate::q_event::Type = crate::q_event::Type(160);
        /// The mouse cursor moves inside a hover item in a graphics scene (<a href="http://doc.qt.io/qt-5/qgraphicsscenehoverevent.html">QGraphicsSceneHoverEvent</a>). (C++ enum variant: <span style='color: green;'>```GraphicsSceneHoverMove = 161```</span>)
        #[allow(non_upper_case_globals)]
        pub const GraphicsSceneHoverMove: crate::q_event::Type = crate::q_event::Type(161);
        /// The mouse cursor leaves a hover item in a graphics scene (<a href="http://doc.qt.io/qt-5/qgraphicsscenehoverevent.html">QGraphicsSceneHoverEvent</a>). (C++ enum variant: <span style='color: green;'>```GraphicsSceneHoverLeave = 162```</span>)
        #[allow(non_upper_case_globals)]
        pub const GraphicsSceneHoverLeave: crate::q_event::Type = crate::q_event::Type(162);
        /// The user requests help for a graphics scene (<a href="http://doc.qt.io/qt-5/qhelpevent.html">QHelpEvent</a>). (C++ enum variant: <span style='color: green;'>```GraphicsSceneHelp = 163```</span>)
        #[allow(non_upper_case_globals)]
        pub const GraphicsSceneHelp: crate::q_event::Type = crate::q_event::Type(163);
        /// The cursor enters a graphics scene during a drag and drop operation (<a href="http://doc.qt.io/qt-5/qgraphicsscenedragdropevent.html">QGraphicsSceneDragDropEvent</a>). (C++ enum variant: <span style='color: green;'>```GraphicsSceneDragEnter = 164```</span>)
        #[allow(non_upper_case_globals)]
        pub const GraphicsSceneDragEnter: crate::q_event::Type = crate::q_event::Type(164);
        /// A drag and drop operation is in progress over a scene (<a href="http://doc.qt.io/qt-5/qgraphicsscenedragdropevent.html">QGraphicsSceneDragDropEvent</a>). (C++ enum variant: <span style='color: green;'>```GraphicsSceneDragMove = 165```</span>)
        #[allow(non_upper_case_globals)]
        pub const GraphicsSceneDragMove: crate::q_event::Type = crate::q_event::Type(165);
        /// The cursor leaves a graphics scene during a drag and drop operation (<a href="http://doc.qt.io/qt-5/qgraphicsscenedragdropevent.html">QGraphicsSceneDragDropEvent</a>). (C++ enum variant: <span style='color: green;'>```GraphicsSceneDragLeave = 166```</span>)
        #[allow(non_upper_case_globals)]
        pub const GraphicsSceneDragLeave: crate::q_event::Type = crate::q_event::Type(166);
        /// A drag and drop operation is completed over a scene (<a href="http://doc.qt.io/qt-5/qgraphicsscenedragdropevent.html">QGraphicsSceneDragDropEvent</a>). (C++ enum variant: <span style='color: green;'>```GraphicsSceneDrop = 167```</span>)
        #[allow(non_upper_case_globals)]
        pub const GraphicsSceneDrop: crate::q_event::Type = crate::q_event::Type(167);
        /// Mouse wheel rolled in a graphics scene (<a href="http://doc.qt.io/qt-5/qgraphicsscenewheelevent.html">QGraphicsSceneWheelEvent</a>). (C++ enum variant: <span style='color: green;'>```GraphicsSceneWheel = 168```</span>)
        #[allow(non_upper_case_globals)]
        pub const GraphicsSceneWheel: crate::q_event::Type = crate::q_event::Type(168);
        /// The keyboard layout has changed. (C++ enum variant: <span style='color: green;'>```KeyboardLayoutChange = 169```</span>)
        #[allow(non_upper_case_globals)]
        pub const KeyboardLayoutChange: crate::q_event::Type = crate::q_event::Type(169);
        /// A dynamic property was added, changed, or removed from the object. (C++ enum variant: <span style='color: green;'>```DynamicPropertyChange = 170```</span>)
        #[allow(non_upper_case_globals)]
        pub const DynamicPropertyChange: crate::q_event::Type = crate::q_event::Type(170);
        /// Wacom tablet enter proximity event (<a href="http://doc.qt.io/qt-5/qtabletevent.html">QTabletEvent</a>), sent to <a href="http://doc.qt.io/qt-5/qapplication.html">QApplication</a>. (C++ enum variant: <span style='color: green;'>```TabletEnterProximity = 171```</span>)
        #[allow(non_upper_case_globals)]
        pub const TabletEnterProximity: crate::q_event::Type = crate::q_event::Type(171);
        /// Wacom tablet leave proximity event (<a href="http://doc.qt.io/qt-5/qtabletevent.html">QTabletEvent</a>), sent to <a href="http://doc.qt.io/qt-5/qapplication.html">QApplication</a>. (C++ enum variant: <span style='color: green;'>```TabletLeaveProximity = 172```</span>)
        #[allow(non_upper_case_globals)]
        pub const TabletLeaveProximity: crate::q_event::Type = crate::q_event::Type(172);
        /// A mouse move occurred outside the client area (<a href="http://doc.qt.io/qt-5/qmouseevent.html">QMouseEvent</a>). (C++ enum variant: <span style='color: green;'>```NonClientAreaMouseMove = 173```</span>)
        #[allow(non_upper_case_globals)]
        pub const NonClientAreaMouseMove: crate::q_event::Type = crate::q_event::Type(173);
        /// A mouse button press occurred outside the client area (<a href="http://doc.qt.io/qt-5/qmouseevent.html">QMouseEvent</a>). (C++ enum variant: <span style='color: green;'>```NonClientAreaMouseButtonPress = 174```</span>)
        #[allow(non_upper_case_globals)]
        pub const NonClientAreaMouseButtonPress: crate::q_event::Type = crate::q_event::Type(174);
        /// A mouse button release occurred outside the client area (<a href="http://doc.qt.io/qt-5/qmouseevent.html">QMouseEvent</a>). (C++ enum variant: <span style='color: green;'>```NonClientAreaMouseButtonRelease = 175```</span>)
        #[allow(non_upper_case_globals)]
        pub const NonClientAreaMouseButtonRelease: crate::q_event::Type = crate::q_event::Type(175);
        /// A mouse double click occurred outside the client area (<a href="http://doc.qt.io/qt-5/qmouseevent.html">QMouseEvent</a>). (C++ enum variant: <span style='color: green;'>```NonClientAreaMouseButtonDblClick = 176```</span>)
        #[allow(non_upper_case_globals)]
        pub const NonClientAreaMouseButtonDblClick: crate::q_event::Type =
            crate::q_event::Type(176);
        /// The user changed his widget sizes (<a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> only). (C++ enum variant: <span style='color: green;'>```MacSizeChange = 177```</span>)
        #[allow(non_upper_case_globals)]
        pub const MacSizeChange: crate::q_event::Type = crate::q_event::Type(177);
        /// The margins of the widget's content rect changed. (C++ enum variant: <span style='color: green;'>```ContentsRectChange = 178```</span>)
        #[allow(non_upper_case_globals)]
        pub const ContentsRectChange: crate::q_event::Type = crate::q_event::Type(178);
        /// C++ enum variant: <span style='color: green;'>```MacGLWindowChange = 179```</span>
        #[allow(non_upper_case_globals)]
        pub const MacGLWindowChange: crate::q_event::Type = crate::q_event::Type(179);
        /// C++ enum variant: <span style='color: green;'>```FutureCallOut = 180```</span>
        #[allow(non_upper_case_globals)]
        pub const FutureCallOut: crate::q_event::Type = crate::q_event::Type(180);
        /// Widget was resized (<a href="http://doc.qt.io/qt-5/qgraphicssceneresizeevent.html">QGraphicsSceneResizeEvent</a>). (C++ enum variant: <span style='color: green;'>```GraphicsSceneResize = 181```</span>)
        #[allow(non_upper_case_globals)]
        pub const GraphicsSceneResize: crate::q_event::Type = crate::q_event::Type(181);
        /// Widget was moved (<a href="http://doc.qt.io/qt-5/qgraphicsscenemoveevent.html">QGraphicsSceneMoveEvent</a>). (C++ enum variant: <span style='color: green;'>```GraphicsSceneMove = 182```</span>)
        #[allow(non_upper_case_globals)]
        pub const GraphicsSceneMove: crate::q_event::Type = crate::q_event::Type(182);
        /// The widget's cursor has changed. (C++ enum variant: <span style='color: green;'>```CursorChange = 183```</span>)
        #[allow(non_upper_case_globals)]
        pub const CursorChange: crate::q_event::Type = crate::q_event::Type(183);
        /// The widget's tooltip has changed. (C++ enum variant: <span style='color: green;'>```ToolTipChange = 184```</span>)
        #[allow(non_upper_case_globals)]
        pub const ToolTipChange: crate::q_event::Type = crate::q_event::Type(184);
        /// C++ enum variant: <span style='color: green;'>```NetworkReplyUpdated = 185```</span>
        #[allow(non_upper_case_globals)]
        pub const NetworkReplyUpdated: crate::q_event::Type = crate::q_event::Type(185);
        /// Item gains mouse grab (<a href="http://doc.qt.io/qt-5/qgraphicsitem.html">QGraphicsItem</a> only). (C++ enum variant: <span style='color: green;'>```GrabMouse = 186```</span>)
        #[allow(non_upper_case_globals)]
        pub const GrabMouse: crate::q_event::Type = crate::q_event::Type(186);
        /// Item loses mouse grab (<a href="http://doc.qt.io/qt-5/qgraphicsitem.html">QGraphicsItem</a>, <a href="http://doc.qt.io/qt-5/qquickitem.html">QQuickItem</a>). (C++ enum variant: <span style='color: green;'>```UngrabMouse = 187```</span>)
        #[allow(non_upper_case_globals)]
        pub const UngrabMouse: crate::q_event::Type = crate::q_event::Type(187);
        /// Item gains keyboard grab (<a href="http://doc.qt.io/qt-5/qgraphicsitem.html">QGraphicsItem</a> only). (C++ enum variant: <span style='color: green;'>```GrabKeyboard = 188```</span>)
        #[allow(non_upper_case_globals)]
        pub const GrabKeyboard: crate::q_event::Type = crate::q_event::Type(188);
        /// Item loses keyboard grab (<a href="http://doc.qt.io/qt-5/qgraphicsitem.html">QGraphicsItem</a> only). (C++ enum variant: <span style='color: green;'>```UngrabKeyboard = 189```</span>)
        #[allow(non_upper_case_globals)]
        pub const UngrabKeyboard: crate::q_event::Type = crate::q_event::Type(189);
        /// C++ enum variant: <span style='color: green;'>```MacGLClearDrawable = 191```</span>
        #[allow(non_upper_case_globals)]
        pub const MacGLClearDrawable: crate::q_event::Type = crate::q_event::Type(191);
        /// A signal delivered to a state machine (<a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html">QStateMachine::SignalEvent</a>). (C++ enum variant: <span style='color: green;'>```StateMachineSignal = 192```</span>)
        #[allow(non_upper_case_globals)]
        pub const StateMachineSignal: crate::q_event::Type = crate::q_event::Type(192);
        /// The event is a wrapper for, i.e., contains, another event (<a href="http://doc.qt.io/qt-5/qstatemachine-wrappedevent.html">QStateMachine::WrappedEvent</a>). (C++ enum variant: <span style='color: green;'>```StateMachineWrapped = 193```</span>)
        #[allow(non_upper_case_globals)]
        pub const StateMachineWrapped: crate::q_event::Type = crate::q_event::Type(193);
        /// Beginning of a sequence of touch-screen or track-pad events (<a href="http://doc.qt.io/qt-5/qtouchevent.html">QTouchEvent</a>). (C++ enum variant: <span style='color: green;'>```TouchBegin = 194```</span>)
        #[allow(non_upper_case_globals)]
        pub const TouchBegin: crate::q_event::Type = crate::q_event::Type(194);
        /// Touch-screen event (<a href="http://doc.qt.io/qt-5/qtouchevent.html">QTouchEvent</a>). (C++ enum variant: <span style='color: green;'>```TouchUpdate = 195```</span>)
        #[allow(non_upper_case_globals)]
        pub const TouchUpdate: crate::q_event::Type = crate::q_event::Type(195);
        /// End of touch-event sequence (<a href="http://doc.qt.io/qt-5/qtouchevent.html">QTouchEvent</a>). (C++ enum variant: <span style='color: green;'>```TouchEnd = 196```</span>)
        #[allow(non_upper_case_globals)]
        pub const TouchEnd: crate::q_event::Type = crate::q_event::Type(196);
        /// The system has detected a gesture (<a href="http://doc.qt.io/qt-5/qnativegestureevent.html">QNativeGestureEvent</a>). (C++ enum variant: <span style='color: green;'>```NativeGesture = 197```</span>)
        #[allow(non_upper_case_globals)]
        pub const NativeGesture: crate::q_event::Type = crate::q_event::Type(197);
        /// A widget wants to open a software input panel (SIP). (C++ enum variant: <span style='color: green;'>```RequestSoftwareInputPanel = 199```</span>)
        #[allow(non_upper_case_globals)]
        pub const RequestSoftwareInputPanel: crate::q_event::Type = crate::q_event::Type(199);
        /// A widget wants to close the software input panel (SIP). (C++ enum variant: <span style='color: green;'>```CloseSoftwareInputPanel = 200```</span>)
        #[allow(non_upper_case_globals)]
        pub const CloseSoftwareInputPanel: crate::q_event::Type = crate::q_event::Type(200);
        /// The window system identifer for this native widget has changed. (C++ enum variant: <span style='color: green;'>```WinIdChange = 203```</span>)
        #[allow(non_upper_case_globals)]
        pub const WinIdChange: crate::q_event::Type = crate::q_event::Type(203);
        /// A gesture was triggered (<a href="http://doc.qt.io/qt-5/qgestureevent.html">QGestureEvent</a>). (C++ enum variant: <span style='color: green;'>```Gesture = 198```</span>)
        #[allow(non_upper_case_globals)]
        pub const Gesture: crate::q_event::Type = crate::q_event::Type(198);
        /// A gesture override was triggered (<a href="http://doc.qt.io/qt-5/qgestureevent.html">QGestureEvent</a>). (C++ enum variant: <span style='color: green;'>```GestureOverride = 202```</span>)
        #[allow(non_upper_case_globals)]
        pub const GestureOverride: crate::q_event::Type = crate::q_event::Type(202);
        /// The object needs to fill in its geometry information (<a href="http://doc.qt.io/qt-5/qscrollprepareevent.html">QScrollPrepareEvent</a>). (C++ enum variant: <span style='color: green;'>```ScrollPrepare = 204```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScrollPrepare: crate::q_event::Type = crate::q_event::Type(204);
        /// The object needs to scroll to the supplied position (<a href="http://doc.qt.io/qt-5/qscrollevent.html">QScrollEvent</a>). (C++ enum variant: <span style='color: green;'>```Scroll = 205```</span>)
        #[allow(non_upper_case_globals)]
        pub const Scroll: crate::q_event::Type = crate::q_event::Type(205);
        /// Sent to a window when its on-screen contents are invalidated and need to be flushed from the backing store. (C++ enum variant: <span style='color: green;'>```Expose = 206```</span>)
        #[allow(non_upper_case_globals)]
        pub const Expose: crate::q_event::Type = crate::q_event::Type(206);
        /// A input method query event (<a href="http://doc.qt.io/qt-5/qinputmethodqueryevent.html">QInputMethodQueryEvent</a>) (C++ enum variant: <span style='color: green;'>```InputMethodQuery = 207```</span>)
        #[allow(non_upper_case_globals)]
        pub const InputMethodQuery: crate::q_event::Type = crate::q_event::Type(207);
        /// The screens orientation has changes (QScreenOrientationChangeEvent). (C++ enum variant: <span style='color: green;'>```OrientationChange = 208```</span>)
        #[allow(non_upper_case_globals)]
        pub const OrientationChange: crate::q_event::Type = crate::q_event::Type(208);
        /// Cancellation of touch-event sequence (<a href="http://doc.qt.io/qt-5/qtouchevent.html">QTouchEvent</a>). (C++ enum variant: <span style='color: green;'>```TouchCancel = 209```</span>)
        #[allow(non_upper_case_globals)]
        pub const TouchCancel: crate::q_event::Type = crate::q_event::Type(209);
        /// C++ enum variant: <span style='color: green;'>```ThemeChange = 210```</span>
        #[allow(non_upper_case_globals)]
        pub const ThemeChange: crate::q_event::Type = crate::q_event::Type(210);
        /// C++ enum variant: <span style='color: green;'>```SockClose = 211```</span>
        #[allow(non_upper_case_globals)]
        pub const SockClose: crate::q_event::Type = crate::q_event::Type(211);
        /// A platform specific panel has been requested. (C++ enum variant: <span style='color: green;'>```PlatformPanel = 212```</span>)
        #[allow(non_upper_case_globals)]
        pub const PlatformPanel: crate::q_event::Type = crate::q_event::Type(212);
        /// C++ enum variant: <span style='color: green;'>```StyleAnimationUpdate = 213```</span>
        #[allow(non_upper_case_globals)]
        pub const StyleAnimationUpdate: crate::q_event::Type = crate::q_event::Type(213);
        /// The state of the application has changed. (C++ enum variant: <span style='color: green;'>```ApplicationStateChange = 214```</span>)
        #[allow(non_upper_case_globals)]
        pub const ApplicationStateChange: crate::q_event::Type = crate::q_event::Type(214);
        /// C++ enum variant: <span style='color: green;'>```WindowChangeInternal = 215```</span>
        #[allow(non_upper_case_globals)]
        pub const WindowChangeInternal: crate::q_event::Type = crate::q_event::Type(215);
        /// C++ enum variant: <span style='color: green;'>```ScreenChangeInternal = 216```</span>
        #[allow(non_upper_case_globals)]
        pub const ScreenChangeInternal: crate::q_event::Type = crate::q_event::Type(216);
        /// A native platform surface has been created or is about to be destroyed (<a href="http://doc.qt.io/qt-5/qplatformsurfaceevent.html">QPlatformSurfaceEvent</a>). (C++ enum variant: <span style='color: green;'>```PlatformSurface = 217```</span>)
        #[allow(non_upper_case_globals)]
        pub const PlatformSurface: crate::q_event::Type = crate::q_event::Type(217);
        /// C++ enum variant: <span style='color: green;'>```Pointer = 218```</span>
        #[allow(non_upper_case_globals)]
        pub const Pointer: crate::q_event::Type = crate::q_event::Type(218);
        /// The Wacom tablet tracking state has changed (since Qt 5.9). (C++ enum variant: <span style='color: green;'>```TabletTrackingChange = 219```</span>)
        #[allow(non_upper_case_globals)]
        pub const TabletTrackingChange: crate::q_event::Type = crate::q_event::Type(219);
        /// User-defined event. (C++ enum variant: <span style='color: green;'>```User = 1000```</span>)
        #[allow(non_upper_case_globals)]
        pub const User: crate::q_event::Type = crate::q_event::Type(1000);
        /// Last user event ID. (C++ enum variant: <span style='color: green;'>```MaxUser = 65535```</span>)
        #[allow(non_upper_case_globals)]
        pub const MaxUser: crate::q_event::Type = crate::q_event::Type(65535);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a> class is the base class of all event classes. Event objects contain event parameters.</p>
///
/// C++ class: <span style='color: green;'>```QEvent```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qevent.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a> class is the base class of all event classes. Event objects contain event parameters.</p>
/// <p>Qt's main event loop (<a href="http://doc.qt.io/qt-5/qcoreapplication.html#exec">QCoreApplication::exec</a>()) fetches native window system events from the event queue, translates them into QEvents, and sends the translated events to <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>s.</p>
/// <p>In general, events come from the underlying window system (<a href="http://doc.qt.io/qt-5/qevent.html#spontaneous">spontaneous</a>() returns <code>true</code>), but it is also possible to manually send events using <a href="http://doc.qt.io/qt-5/qcoreapplication.html#sendEvent">QCoreApplication::sendEvent</a>() and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#postEvent">QCoreApplication::postEvent</a>() (<a href="http://doc.qt.io/qt-5/qevent.html#spontaneous">spontaneous</a>() returns <code>false</code>).</p>
/// <p><a href="http://doc.qt.io/qt-5/qobject.html">QObjects</a> receive events by having their <a href="http://doc.qt.io/qt-5/qobject.html#event">QObject::event</a>() function called. The function can be reimplemented in subclasses to customize event handling and add additional event types; <a href="http://doc.qt.io/qt-5/qwidget.html#event">QWidget::event</a>() is a notable example. By default, events are dispatched to event handlers like <a href="http://doc.qt.io/qt-5/qobject.html#timerEvent">QObject::timerEvent</a>() and <a href="http://doc.qt.io/qt-5/qwidget.html#mouseMoveEvent">QWidget::mouseMoveEvent</a>(). <a href="http://doc.qt.io/qt-5/qobject.html#installEventFilter">QObject::installEventFilter</a>() allows an object to intercept events destined for another object.</p>
/// <p>The basic <a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a> contains only an event type parameter and an "accept" flag. The accept flag set with <a href="http://doc.qt.io/qt-5/qevent.html#accept">accept</a>(), and cleared with <a href="http://doc.qt.io/qt-5/qevent.html#ignore">ignore</a>(). It is set by default, but don't rely on this as subclasses may choose to clear it in their constructor.</p>
/// <p>Subclasses of <a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a> contain additional parameters that describe the particular event.</p></div>
#[repr(C)]
pub struct QEvent {
    _unused: u8,
}
impl QEvent {
    /// <p>Sets the accept flag of the event object, the equivalent of calling <a href="http://doc.qt.io/qt-5/qevent.html#accepted-prop">setAccepted</a>(true).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QEvent::accept()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qevent.html#accept">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the accept flag of the event object, the equivalent of calling <a href="http://doc.qt.io/qt-5/qevent.html#accepted-prop">setAccepted</a>(true).</p>
    /// <p>Setting the accept parameter indicates that the event receiver wants the event. Unwanted events might be propagated to the parent widget.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qevent.html#ignore">ignore</a>().</p></div>
    pub unsafe fn accept(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QEvent_accept(self as *mut crate::QEvent)
    }

    /// Calls C++ function: <span style='color: green;'>```QEvent& QEvent::operator=(const QEvent& other)```</span>.
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QEvent>>,
    ) -> ::cpp_utils::MutRef<crate::QEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEvent_operator_(
            self as *mut crate::QEvent,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QEvent>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Clears the accept flag parameter of the event object, the equivalent of calling <a href="http://doc.qt.io/qt-5/qevent.html#accepted-prop">setAccepted</a>(false).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QEvent::ignore()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qevent.html#ignore">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Clears the accept flag parameter of the event object, the equivalent of calling <a href="http://doc.qt.io/qt-5/qevent.html#accepted-prop">setAccepted</a>(false).</p>
    /// <p>Clearing the accept parameter indicates that the event receiver does not want the event. Unwanted events might be propagated to the parent widget.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qevent.html#accept">accept</a>().</p></div>
    pub unsafe fn ignore(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QEvent_ignore(self as *mut crate::QEvent)
    }

    /// <p>the accept flag of the event object</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QEvent::isAccepted() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qevent.html#accepted-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>the accept flag of the event object</p>
    /// <p>Setting the accept parameter indicates that the event receiver wants the event. Unwanted events might be propagated to the parent widget. By default, isAccepted() is set to true, but don't rely on this as subclasses may choose to clear it in their constructor.</p>
    /// <p>For convenience, the accept flag can also be set with <a href="http://doc.qt.io/qt-5/qevent.html#accept">accept</a>(), and cleared with <a href="http://doc.qt.io/qt-5/qevent.html#ignore">ignore</a>().</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> bool </td><td class="memItemRight bottomAlign"><span class="name"><b>isAccepted</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setAccepted</b></span>(bool <i>accepted</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn is_accepted(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QEvent_isAccepted(self as *const crate::QEvent)
    }

    /// <p>Contructs an event object of type <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QEvent::QEvent(QEvent::Type type)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qevent.html#QEvent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Contructs an event object of type <i>type</i>.</p></div>
    pub unsafe fn new(type_: crate::q_event::Type) -> ::cpp_utils::CppBox<crate::QEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEvent_QEvent(type_);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Contructs an event object of type <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QEvent::QEvent(const QEvent& other)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qevent.html#QEvent">C++ documentation</a> for <span style='color: green;'>```QEvent::QEvent(Type type)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Contructs an event object of type <i>type</i>.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QEvent>>,
    ) -> ::cpp_utils::CppBox<crate::QEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEvent_QEvent1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QEvent>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Registers and returns a custom event type. The <i>hint</i> provided will be used if it is available, otherwise it will return a value between <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::User</a> and <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::MaxUser</a> that has not yet been registered. The <i>hint</i> is ignored if its value is not between <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::User</a> and <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::MaxUser</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QEvent::registerEventType(int hint = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qevent.html#registerEventType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Registers and returns a custom event type. The <i>hint</i> provided will be used if it is available, otherwise it will return a value between <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::User</a> and <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::MaxUser</a> that has not yet been registered. The <i>hint</i> is ignored if its value is not between <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::User</a> and <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::MaxUser</a>.</p>
    /// <p>Returns -1 if all available values are already taken or the program is shutting down.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p></div>
    pub unsafe fn register_event_type_1a(hint: ::std::os::raw::c_int) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QEvent_registerEventType(hint)
    }

    /// <p>Registers and returns a custom event type. The <i>hint</i> provided will be used if it is available, otherwise it will return a value between <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::User</a> and <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::MaxUser</a> that has not yet been registered. The <i>hint</i> is ignored if its value is not between <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::User</a> and <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::MaxUser</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QEvent::registerEventType()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qevent.html#registerEventType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Registers and returns a custom event type. The <i>hint</i> provided will be used if it is available, otherwise it will return a value between <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::User</a> and <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::MaxUser</a> that has not yet been registered. The <i>hint</i> is ignored if its value is not between <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::User</a> and <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::MaxUser</a>.</p>
    /// <p>Returns -1 if all available values are already taken or the program is shutting down.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p></div>
    pub unsafe fn register_event_type_0a() -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QEvent_registerEventType1()
    }

    /// <p>the accept flag of the event object</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QEvent::setAccepted(bool accepted)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qevent.html#accepted-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>the accept flag of the event object</p>
    /// <p>Setting the accept parameter indicates that the event receiver wants the event. Unwanted events might be propagated to the parent widget. By default, isAccepted() is set to true, but don't rely on this as subclasses may choose to clear it in their constructor.</p>
    /// <p>For convenience, the accept flag can also be set with <a href="http://doc.qt.io/qt-5/qevent.html#accept">accept</a>(), and cleared with <a href="http://doc.qt.io/qt-5/qevent.html#ignore">ignore</a>().</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> bool </td><td class="memItemRight bottomAlign"><span class="name"><b>isAccepted</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setAccepted</b></span>(bool <i>accepted</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn set_accepted(&mut self, accepted: bool) {
        crate::__ffi::ctr_qt_core_ffi_QEvent_setAccepted(self as *mut crate::QEvent, accepted)
    }

    /// <p>Returns <code>true</code> if the event originated outside the application (a system event); otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QEvent::spontaneous() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qevent.html#spontaneous">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the event originated outside the application (a system event); otherwise returns <code>false</code>.</p>
    /// <p>The return value of this function is not defined for paint events.</p></div>
    pub unsafe fn spontaneous(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QEvent_spontaneous(self as *const crate::QEvent)
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEvent_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the event type.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QEvent::Type QEvent::type() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qevent.html#type">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the event type.</p></div>
    pub unsafe fn type_(&self) -> crate::q_event::Type {
        crate::__ffi::ctr_qt_core_ffi_QEvent_type(self as *const crate::QEvent)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qtimerevent.html">QTimerEvent</a> class contains parameters that describe a timer event.</p>
///
/// C++ class: <span style='color: green;'>```QTimerEvent```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtimerevent.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qtimerevent.html">QTimerEvent</a> class contains parameters that describe a timer event.</p>
/// <p>Timer events are sent at regular intervals to objects that have started one or more timers. Each timer has a unique identifier. A timer is started with <a href="http://doc.qt.io/qt-5/qobject.html#startTimer">QObject::startTimer</a>().</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> class provides a high-level programming interface that uses signals instead of events. It also provides single-shot timers.</p>
/// <p>The event handler <a href="http://doc.qt.io/qt-5/qobject.html#timerEvent">QObject::timerEvent</a>() receives timer events.</p></div>
#[repr(C)]
pub struct QTimerEvent {
    _unused: u8,
}
impl QTimerEvent {
    /// <p>The <a href="http://doc.qt.io/qt-5/qtimerevent.html">QTimerEvent</a> class contains parameters that describe a timer event.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTimerEvent& QTimerEvent::operator=(const QTimerEvent& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimerevent.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qtimerevent.html">QTimerEvent</a> class contains parameters that describe a timer event.</p>
    /// <p>Timer events are sent at regular intervals to objects that have started one or more timers. Each timer has a unique identifier. A timer is started with <a href="http://doc.qt.io/qt-5/qobject.html#startTimer">QObject::startTimer</a>().</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> class provides a high-level programming interface that uses signals instead of events. It also provides single-shot timers.</p>
    /// <p>The event handler <a href="http://doc.qt.io/qt-5/qobject.html#timerEvent">QObject::timerEvent</a>() receives timer events.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTimerEvent>>,
    ) -> ::cpp_utils::MutRef<crate::QTimerEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimerEvent_operator_(
            self as *mut crate::QTimerEvent,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTimerEvent>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Constructs a timer event object with the timer identifier set to <i>timerId</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTimerEvent::QTimerEvent(int timerId)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimerevent.html#QTimerEvent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a timer event object with the timer identifier set to <i>timerId</i>.</p></div>
    pub unsafe fn new(timer_id: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QTimerEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimerEvent_QTimerEvent(timer_id);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qtimerevent.html">QTimerEvent</a> class contains parameters that describe a timer event.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTimerEvent::QTimerEvent(const QTimerEvent& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimerevent.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qtimerevent.html">QTimerEvent</a> class contains parameters that describe a timer event.</p>
    /// <p>Timer events are sent at regular intervals to objects that have started one or more timers. Each timer has a unique identifier. A timer is started with <a href="http://doc.qt.io/qt-5/qobject.html#startTimer">QObject::startTimer</a>().</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> class provides a high-level programming interface that uses signals instead of events. It also provides single-shot timers.</p>
    /// <p>The event handler <a href="http://doc.qt.io/qt-5/qobject.html#timerEvent">QObject::timerEvent</a>() receives timer events.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTimerEvent>>,
    ) -> ::cpp_utils::CppBox<crate::QTimerEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimerEvent_QTimerEvent2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTimerEvent>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the unique timer identifier, which is the same identifier as returned from <a href="http://doc.qt.io/qt-5/qobject.html#startTimer">QObject::startTimer</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTimerEvent::timerId() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimerevent.html#timerId">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the unique timer identifier, which is the same identifier as returned from <a href="http://doc.qt.io/qt-5/qobject.html#startTimer">QObject::startTimer</a>().</p></div>
    pub unsafe fn timer_id(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTimerEvent_timerId(self as *const crate::QTimerEvent)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qchildevent.html">QChildEvent</a> class contains event parameters for child object events.</p>
///
/// C++ class: <span style='color: green;'>```QChildEvent```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qchildevent.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qchildevent.html">QChildEvent</a> class contains event parameters for child object events.</p>
/// <p>Child events are sent immediately to objects when children are added or removed.</p>
/// <p>In both cases you can only rely on the child being a <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> (or, if <a href="http://doc.qt.io/qt-5/qobject.html#isWidgetType">QObject::isWidgetType</a>() returns <code>true</code>, a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a>). This is because in the <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::ChildAdded</a> case the child is not yet fully constructed; in the <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::ChildRemoved</a> case it might have already been destructed.</p>
/// <p>The handler for these events is <a href="http://doc.qt.io/qt-5/qobject.html#childEvent">QObject::childEvent</a>().</p></div>
#[repr(C)]
pub struct QChildEvent {
    _unused: u8,
}
impl QChildEvent {
    /// <p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qevent.html#type">type</a>() is <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::ChildAdded</a>; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QChildEvent::added() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchildevent.html#added">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qevent.html#type">type</a>() is <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::ChildAdded</a>; otherwise returns false.</p></div>
    pub unsafe fn added(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChildEvent_added(self as *const crate::QChildEvent)
    }

    /// <p>Returns the child object that was added or removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QObject* QChildEvent::child() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchildevent.html#child">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the child object that was added or removed.</p></div>
    pub unsafe fn child(&self) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QChildEvent_child(self as *const crate::QChildEvent);
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qchildevent.html">QChildEvent</a> class contains event parameters for child object events.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChildEvent& QChildEvent::operator=(const QChildEvent& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchildevent.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qchildevent.html">QChildEvent</a> class contains event parameters for child object events.</p>
    /// <p>Child events are sent immediately to objects when children are added or removed.</p>
    /// <p>In both cases you can only rely on the child being a <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> (or, if <a href="http://doc.qt.io/qt-5/qobject.html#isWidgetType">QObject::isWidgetType</a>() returns <code>true</code>, a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a>). This is because in the <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::ChildAdded</a> case the child is not yet fully constructed; in the <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::ChildRemoved</a> case it might have already been destructed.</p>
    /// <p>The handler for these events is <a href="http://doc.qt.io/qt-5/qobject.html#childEvent">QObject::childEvent</a>().</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChildEvent>>,
    ) -> ::cpp_utils::MutRef<crate::QChildEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QChildEvent_operator_(
            self as *mut crate::QChildEvent,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChildEvent>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Constructs a child event object of a particular <i>type</i> for the <i>child</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QChildEvent::QChildEvent(QEvent::Type type, QObject* child)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchildevent.html#QChildEvent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a child event object of a particular <i>type</i> for the <i>child</i>.</p>
    /// <p><i>type</i> can be <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::ChildAdded</a>, <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::ChildRemoved</a>, or <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::ChildPolished</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchildevent.html#child">child</a>().</p></div>
    pub unsafe fn new(
        type_: crate::q_event::Type,
        child: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QChildEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QChildEvent_QChildEvent(
            type_,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(child)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qchildevent.html">QChildEvent</a> class contains event parameters for child object events.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QChildEvent::QChildEvent(const QChildEvent& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchildevent.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qchildevent.html">QChildEvent</a> class contains event parameters for child object events.</p>
    /// <p>Child events are sent immediately to objects when children are added or removed.</p>
    /// <p>In both cases you can only rely on the child being a <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> (or, if <a href="http://doc.qt.io/qt-5/qobject.html#isWidgetType">QObject::isWidgetType</a>() returns <code>true</code>, a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a>). This is because in the <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::ChildAdded</a> case the child is not yet fully constructed; in the <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::ChildRemoved</a> case it might have already been destructed.</p>
    /// <p>The handler for these events is <a href="http://doc.qt.io/qt-5/qobject.html#childEvent">QObject::childEvent</a>().</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChildEvent>>,
    ) -> ::cpp_utils::CppBox<crate::QChildEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QChildEvent_QChildEvent2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChildEvent>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qevent.html#type">type</a>() is <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::ChildPolished</a>; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QChildEvent::polished() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchildevent.html#polished">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qevent.html#type">type</a>() is <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::ChildPolished</a>; otherwise returns false.</p></div>
    pub unsafe fn polished(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChildEvent_polished(self as *const crate::QChildEvent)
    }

    /// <p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qevent.html#type">type</a>() is <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::ChildRemoved</a>; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QChildEvent::removed() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchildevent.html#removed">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qevent.html#type">type</a>() is <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">QEvent::ChildRemoved</a>; otherwise returns false.</p></div>
    pub unsafe fn removed(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QChildEvent_removed(self as *const crate::QChildEvent)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qdynamicpropertychangeevent.html">QDynamicPropertyChangeEvent</a> class contains event parameters for dynamic property change events.</p>
///
/// C++ class: <span style='color: green;'>```QDynamicPropertyChangeEvent```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qdynamicpropertychangeevent.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qdynamicpropertychangeevent.html">QDynamicPropertyChangeEvent</a> class contains event parameters for dynamic property change events.</p>
/// <p>Dynamic property change events are sent to objects when properties are dynamically added, changed or removed using <a href="http://doc.qt.io/qt-5/qobject.html#setProperty">QObject::setProperty</a>().</p></div>
#[repr(C)]
pub struct QDynamicPropertyChangeEvent {
    _unused: u8,
}
impl QDynamicPropertyChangeEvent {
    /// <p>The <a href="http://doc.qt.io/qt-5/qdynamicpropertychangeevent.html">QDynamicPropertyChangeEvent</a> class contains event parameters for dynamic property change events.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDynamicPropertyChangeEvent& QDynamicPropertyChangeEvent::operator=(const QDynamicPropertyChangeEvent& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdynamicpropertychangeevent.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qdynamicpropertychangeevent.html">QDynamicPropertyChangeEvent</a> class contains event parameters for dynamic property change events.</p>
    /// <p>Dynamic property change events are sent to objects when properties are dynamically added, changed or removed using <a href="http://doc.qt.io/qt-5/qobject.html#setProperty">QObject::setProperty</a>().</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDynamicPropertyChangeEvent>>,
    ) -> ::cpp_utils::MutRef<crate::QDynamicPropertyChangeEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDynamicPropertyChangeEvent_operator_(self as *mut crate::QDynamicPropertyChangeEvent, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDynamicPropertyChangeEvent>>::cast_into(other).as_raw_ptr());
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Constructs a dynamic property change event object with the property name set to <i>name</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDynamicPropertyChangeEvent::QDynamicPropertyChangeEvent(const QByteArray& name)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdynamicpropertychangeevent.html#QDynamicPropertyChangeEvent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a dynamic property change event object with the property name set to <i>name</i>.</p></div>
    pub unsafe fn new(
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QDynamicPropertyChangeEvent> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDynamicPropertyChangeEvent_QDynamicPropertyChangeEvent(
                ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(name)
                    .as_raw_ptr(),
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qdynamicpropertychangeevent.html">QDynamicPropertyChangeEvent</a> class contains event parameters for dynamic property change events.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDynamicPropertyChangeEvent::QDynamicPropertyChangeEvent(const QDynamicPropertyChangeEvent& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdynamicpropertychangeevent.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qdynamicpropertychangeevent.html">QDynamicPropertyChangeEvent</a> class contains event parameters for dynamic property change events.</p>
    /// <p>Dynamic property change events are sent to objects when properties are dynamically added, changed or removed using <a href="http://doc.qt.io/qt-5/qobject.html#setProperty">QObject::setProperty</a>().</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDynamicPropertyChangeEvent>>,
    ) -> ::cpp_utils::CppBox<crate::QDynamicPropertyChangeEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDynamicPropertyChangeEvent_QDynamicPropertyChangeEvent2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDynamicPropertyChangeEvent>>::cast_into(other).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the name of the dynamic property that was added, changed or removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QDynamicPropertyChangeEvent::propertyName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdynamicpropertychangeevent.html#propertyName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the name of the dynamic property that was added, changed or removed.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#setProperty">QObject::setProperty</a>() and <a href="http://doc.qt.io/qt-5/qobject.html#dynamicPropertyNames">QObject::dynamicPropertyNames</a>().</p></div>
    pub unsafe fn property_name(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDynamicPropertyChangeEvent_propertyName(
            self as *const crate::QDynamicPropertyChangeEvent,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// C++ class: <span style='color: green;'>```QDeferredDeleteEvent```</span>.
#[repr(C)]
pub struct QDeferredDeleteEvent {
    _unused: u8,
}
impl QDeferredDeleteEvent {
    /// Calls C++ function: <span style='color: green;'>```QDeferredDeleteEvent& QDeferredDeleteEvent::operator=(const QDeferredDeleteEvent& other)```</span>.
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDeferredDeleteEvent>>,
    ) -> ::cpp_utils::MutRef<crate::QDeferredDeleteEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDeferredDeleteEvent_operator_(
            self as *mut crate::QDeferredDeleteEvent,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDeferredDeleteEvent>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the loop-level in which the event was posted. The loop-level is set by <a href="http://doc.qt.io/qt-5/qcoreapplication.html#postEvent">QCoreApplication::postEvent</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QDeferredDeleteEvent::loopLevel() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeferreddeleteevent.html#loopLevel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the loop-level in which the event was posted. The loop-level is set by <a href="http://doc.qt.io/qt-5/qcoreapplication.html#postEvent">QCoreApplication::postEvent</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#deleteLater">QObject::deleteLater</a>().</p></div>
    pub unsafe fn loop_level(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QDeferredDeleteEvent_loopLevel(
            self as *const crate::QDeferredDeleteEvent,
        )
    }

    /// <p>Constructs a deferred delete event with an initial <a href="http://doc.qt.io/qt-5/qdeferreddeleteevent.html#loopLevel">loopLevel</a>() of zero.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDeferredDeleteEvent::QDeferredDeleteEvent()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeferreddeleteevent.html#QDeferredDeleteEvent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a deferred delete event with an initial <a href="http://doc.qt.io/qt-5/qdeferreddeleteevent.html#loopLevel">loopLevel</a>() of zero.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QDeferredDeleteEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDeferredDeleteEvent_QDeferredDeleteEvent();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDeferredDeleteEvent::QDeferredDeleteEvent(const QDeferredDeleteEvent& other)```</span>.
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDeferredDeleteEvent>>,
    ) -> ::cpp_utils::CppBox<crate::QDeferredDeleteEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDeferredDeleteEvent_QDeferredDeleteEvent1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDeferredDeleteEvent>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> class provides an event loop for Qt applications without UI.</p>
///
/// C++ class: <span style='color: green;'>```QCoreApplication```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qcoreapplication.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> class provides an event loop for Qt applications without UI.</p>
/// <p>This class is used by non-GUI applications to provide their event loop. For non-GUI application that uses Qt, there should be exactly one <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> object. For GUI applications, see <a href="http://doc.qt.io/qt-5/qguiapplication.html">QGuiApplication</a>. For applications that use the Qt Widgets module, see <a href="http://doc.qt.io/qt-5/qapplication.html">QApplication</a>.</p>
/// <p><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> contains the main event loop, where all events from the operating system (e.g., timer and network events) and other sources are processed and dispatched. It also handles the application's initialization and finalization, as well as system-wide and application-wide settings.</p>
/// <a name="the-event-loop-and-event-handling"></a></div>
#[repr(C)]
pub struct QCoreApplication {
    _unused: u8,
}
impl QCoreApplication {
    /// <p>Tells the application to exit with return code 0 (success). Equivalent to calling <a href="http://doc.qt.io/qt-5/qcoreapplication.html#exit">QCoreApplication::exit</a>(0).</p>
    ///
    /// Returns a built-in Qt slot `QCoreApplication::quit` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#quit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Tells the application to exit with return code 0 (success). Equivalent to calling <a href="http://doc.qt.io/qt-5/qcoreapplication.html#exit">QCoreApplication::exit</a>(0).</p>
    /// <p>It's common to connect the <a href="http://doc.qt.io/qt-5/qguiapplication.html#lastWindowClosed">QGuiApplication::lastWindowClosed</a>() signal to quit(), and you also often connect e.g. <a href="http://doc.qt.io/qt-5/qabstractbutton.html#clicked">QAbstractButton::clicked</a>() or signals in <a href="http://doc.qt.io/qt-5/qaction.html">QAction</a>, <a href="http://doc.qt.io/qt-5/qmenu.html">QMenu</a>, or <a href="http://doc.qt.io/qt-5/qmenubar.html">QMenuBar</a> to it.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span>quitButton <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span>(<span class="string">"Quit"</span>);
    ///   connect(quitButton<span class="operator">,</span> SIGNAL(clicked())<span class="operator">,</span> <span class="operator">&amp;</span>app<span class="operator">,</span> SLOT(quit()));
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#exit">exit</a>(), <a href="http://doc.qt.io/qt-5/qcoreapplication.html#aboutToQuit">aboutToQuit</a>(), and <a href="http://doc.qt.io/qt-5/qguiapplication.html#lastWindowClosed">QGuiApplication::lastWindowClosed</a>().</p></div>
    pub fn slot_quit(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1quit()\0"),
            )
        }
    }

    /// <p>This signal is emitted when the application is about to quit the main event loop, e.g. when the event loop level drops to zero. This may happen either after a call to <a href="http://doc.qt.io/qt-5/qcoreapplication.html#quit">quit</a>() from inside the application or when the user shuts down the entire desktop session.</p>
    ///
    /// Returns a built-in Qt signal `QCoreApplication::aboutToQuit` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#aboutToQuit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the application is about to quit the main event loop, e.g. when the event loop level drops to zero. This may happen either after a call to <a href="http://doc.qt.io/qt-5/qcoreapplication.html#quit">quit</a>() from inside the application or when the user shuts down the entire desktop session.</p>
    /// <p>The signal is particularly useful if your application has to do some last-second cleanup. Note that no user interaction is possible in this state.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#quit">quit</a>().</p></div>
    pub fn about_to_quit(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2aboutToQuit()\0"),
            )
        }
    }

    /// <p>This property holds the name of the organization that wrote this application</p>
    ///
    /// Returns a built-in Qt signal `QCoreApplication::organizationNameChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationName-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the name of the organization that wrote this application</p>
    /// <p>The value is used by the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> class when it is constructed using the empty constructor. This saves having to repeat this information each time a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object is created.</p>
    /// <p>On Mac, <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> uses <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationDomain-prop">organizationDomain()</a> as the organization if it's not an empty string; otherwise it uses organizationName(). On all other platforms, <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> uses organizationName() as the organization.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QString </td><td class="memItemRight bottomAlign"><span class="name"><b>organizationName</b></span>()</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setOrganizationName</b></span>(const QString &amp;<i>orgName</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>organizationNameChanged</b></span>()</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationDomain-prop">organizationDomain</a> and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationName-prop">applicationName</a>.</p></div>
    pub fn organization_name_changed(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2organizationNameChanged()\0"),
            )
        }
    }

    /// <p>This property holds the Internet domain of the organization that wrote this application</p>
    ///
    /// Returns a built-in Qt signal `QCoreApplication::organizationDomainChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationDomain-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the Internet domain of the organization that wrote this application</p>
    /// <p>The value is used by the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> class when it is constructed using the empty constructor. This saves having to repeat this information each time a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object is created.</p>
    /// <p>On Mac, <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> uses organizationDomain() as the organization if it's not an empty string; otherwise it uses <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationName-prop">organizationName</a>(). On all other platforms, <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> uses <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationName-prop">organizationName</a>() as the organization.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QString </td><td class="memItemRight bottomAlign"><span class="name"><b>organizationDomain</b></span>()</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setOrganizationDomain</b></span>(const QString &amp;<i>orgDomain</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>organizationDomainChanged</b></span>()</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationName-prop">organizationName</a>, <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationName-prop">applicationName</a>, and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationVersion-prop">applicationVersion</a>.</p></div>
    pub fn organization_domain_changed(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2organizationDomainChanged()\0"),
            )
        }
    }

    /// <p>This property holds the name of this application</p>
    ///
    /// Returns a built-in Qt signal `QCoreApplication::applicationNameChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationName-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the name of this application</p>
    /// <p>The value is used by the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> class when it is constructed using the empty constructor. This saves having to repeat this information each time a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object is created.</p>
    /// <p>If not set, the application name defaults to the executable name (since 5.0).</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QString </td><td class="memItemRight bottomAlign"><span class="name"><b>applicationName</b></span>()</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setApplicationName</b></span>(const QString &amp;<i>application</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>applicationNameChanged</b></span>()</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationName-prop">organizationName</a>, <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationDomain-prop">organizationDomain</a>, <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationVersion-prop">applicationVersion</a>, and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationFilePath">applicationFilePath</a>().</p></div>
    pub fn application_name_changed(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2applicationNameChanged()\0"),
            )
        }
    }

    /// <p>This property holds the version of this application</p>
    ///
    /// Returns a built-in Qt signal `QCoreApplication::applicationVersionChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationVersion-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the version of this application</p>
    /// <p>If not set, the application version defaults to a platform-specific value determined from the main application executable or package (since Qt 5.9):</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Platform</th><th>Source</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>Windows (classic desktop)</td><td>PRODUCTVERSION parameter of the VERSIONINFO resource</td></tr>
    /// <tr valign="top" class="even"><td>Universal Windows Platform</td><td>version attribute of the application package manifest</td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, iOS, tvOS, watchOS</td><td>CFBundleVersion property of the information property list</td></tr>
    /// <tr valign="top" class="even"><td>Android</td><td>android:versionName property of the AndroidManifest.xml manifest element</td></tr>
    /// </tbody></table></div>
    /// <p>On other platforms, the default is the empty string.</p>
    /// <p>This property was introduced in  Qt 4.4.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QString </td><td class="memItemRight bottomAlign"><span class="name"><b>applicationVersion</b></span>()</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setApplicationVersion</b></span>(const QString &amp;<i>version</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>applicationVersionChanged</b></span>()</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationName-prop">applicationName</a>, <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationName-prop">organizationName</a>, and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationDomain-prop">organizationDomain</a>.</p></div>
    pub fn application_version_changed(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2applicationVersionChanged()\0"),
            )
        }
    }

    /// <p>Prepends <i>path</i> to the beginning of the library path list, ensuring that it is searched for libraries first. If <i>path</i> is empty or already in the path list, the path list is not changed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QCoreApplication::addLibraryPath(const QString& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#addLibraryPath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Prepends <i>path</i> to the beginning of the library path list, ensuring that it is searched for libraries first. If <i>path</i> is empty or already in the path list, the path list is not changed.</p>
    /// <p>The default path list consists of a single entry, the installation directory for plugins. The default installation directory for plugins is <code>INSTALL/plugins</code>, where <code>INSTALL</code> is the directory where Qt was installed.</p>
    /// <p>The library paths are reset to the default when an instance of <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> is destructed.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#removeLibraryPath">removeLibraryPath</a>(), <a href="http://doc.qt.io/qt-5/qcoreapplication.html#libraryPaths">libraryPaths</a>(), and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#setLibraryPaths">setLibraryPaths</a>().</p></div>
    pub unsafe fn add_library_path(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_addLibraryPath(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(arg1).as_raw_ptr(),
        )
    }

    /// <p>Returns the directory that contains the application executable.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QCoreApplication::applicationDirPath()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationDirPath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the directory that contains the application executable.</p>
    /// <p>For example, if you have installed Qt in the <code>C:\Qt</code> directory, and you run the <code>regexp</code> example, this function will return "C:/Qt/examples/tools/regexp".</p>
    /// <p>On <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS this will point to the directory actually containing the executable, which may be inside an application bundle (if the application is bundled).</p>
    /// <p><b>Warning:</b> On Linux, this function will try to get the path from the <code>/proc</code> file system. If that fails, it assumes that <code>argv[0]</code> contains the absolute file name of the executable. The function also assumes that the current directory has not been changed by the application.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationFilePath">applicationFilePath</a>().</p></div>
    pub unsafe fn application_dir_path() -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCoreApplication_applicationDirPath();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the file path of the application executable.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QCoreApplication::applicationFilePath()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationFilePath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the file path of the application executable.</p>
    /// <p>For example, if you have installed Qt in the <code>/usr/local/qt</code> directory, and you run the <code>regexp</code> example, this function will return "/usr/local/qt/examples/tools/regexp/regexp".</p>
    /// <p><b>Warning:</b> On Linux, this function will try to get the path from the <code>/proc</code> file system. If that fails, it assumes that <code>argv[0]</code> contains the absolute file name of the executable. The function also assumes that the current directory has not been changed by the application.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationDirPath">applicationDirPath</a>().</p></div>
    pub unsafe fn application_file_path() -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCoreApplication_applicationFilePath();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This property holds the name of this application</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QCoreApplication::applicationName()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationName-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the name of this application</p>
    /// <p>The value is used by the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> class when it is constructed using the empty constructor. This saves having to repeat this information each time a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object is created.</p>
    /// <p>If not set, the application name defaults to the executable name (since 5.0).</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QString </td><td class="memItemRight bottomAlign"><span class="name"><b>applicationName</b></span>()</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setApplicationName</b></span>(const QString &amp;<i>application</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationName-prop">organizationName</a>, <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationDomain-prop">organizationDomain</a>, <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationVersion-prop">applicationVersion</a>, and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationFilePath">applicationFilePath</a>().</p></div>
    pub unsafe fn application_name() -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCoreApplication_applicationName();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the current process ID for the application.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static qint64 QCoreApplication::applicationPid()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationPid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current process ID for the application.</p>
    /// <p>This function was introduced in  Qt 4.4.</p></div>
    pub unsafe fn application_pid() -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_applicationPid()
    }

    /// <p>This property holds the version of this application</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QCoreApplication::applicationVersion()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationVersion-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the version of this application</p>
    /// <p>If not set, the application version defaults to a platform-specific value determined from the main application executable or package (since Qt 5.9):</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Platform</th><th>Source</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>Windows (classic desktop)</td><td>PRODUCTVERSION parameter of the VERSIONINFO resource</td></tr>
    /// <tr valign="top" class="even"><td>Universal Windows Platform</td><td>version attribute of the application package manifest</td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, iOS, tvOS, watchOS</td><td>CFBundleVersion property of the information property list</td></tr>
    /// <tr valign="top" class="even"><td>Android</td><td>android:versionName property of the AndroidManifest.xml manifest element</td></tr>
    /// </tbody></table></div>
    /// <p>On other platforms, the default is the empty string.</p>
    /// <p>This property was introduced in  Qt 4.4.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QString </td><td class="memItemRight bottomAlign"><span class="name"><b>applicationVersion</b></span>()</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setApplicationVersion</b></span>(const QString &amp;<i>version</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationName-prop">applicationName</a>, <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationName-prop">organizationName</a>, and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationDomain-prop">organizationDomain</a>.</p></div>
    pub unsafe fn application_version() -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCoreApplication_applicationVersion();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the list of command-line arguments.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QStringList QCoreApplication::arguments()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#arguments">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the list of command-line arguments.</p>
    /// <p>Usually arguments().at(0) is the program name, arguments().at(1) is the first argument, and arguments().last() is the last argument. See the note below about Windows.</p>
    /// <p>Calling this function is slow - you should store the result in a variable when parsing the command line.</p>
    /// <p><b>Warning:</b> On Unix, this list is built from the argc and argv parameters passed to the constructor in the main() function. The string-data in argv is interpreted using <a href="http://doc.qt.io/qt-5/qstring.html#fromLocal8Bit">QString::fromLocal8Bit</a>(); hence it is not possible to pass, for example, Japanese command line arguments on a system that runs in a Latin1 locale. Most modern Unix systems do not have this limitation, as they are Unicode-based.</p>
    /// <p>On Windows, the list is built from the argc and argv parameters only if modified argv/argc parameters are passed to the constructor. In that case, encoding problems might occur.</p>
    /// <p>Otherwise, the arguments() are constructed from the return value of <a href="http://msdn2.microsoft.com/en-us/library/ms683156(VS.85).aspx">GetCommandLine()</a>. As a result of this, the string given by arguments().at(0) might not be the program name on Windows, depending on how the application was started.</p>
    /// <p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationFilePath">applicationFilePath</a>() and <a href="http://doc.qt.io/qt-5/qcommandlineparser.html">QCommandLineParser</a>.</p></div>
    pub unsafe fn arguments() -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCoreApplication_arguments();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if the application objects are being destroyed; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QCoreApplication::closingDown()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#closingDown">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the application objects are being destroyed; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#startingUp">startingUp</a>().</p></div>
    pub unsafe fn closing_down() -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_closingDown()
    }

    /// <p>Returns a pointer to the event dispatcher object for the main thread. If no event dispatcher exists for the thread, this function returns 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QAbstractEventDispatcher* QCoreApplication::eventDispatcher()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#eventDispatcher">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the event dispatcher object for the main thread. If no event dispatcher exists for the thread, this function returns 0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#setEventDispatcher">setEventDispatcher</a>().</p></div>
    pub unsafe fn event_dispatcher() -> ::cpp_utils::MutPtr<crate::QAbstractEventDispatcher> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCoreApplication_eventDispatcher();
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Enters the main event loop and waits until <a href="http://doc.qt.io/qt-5/qcoreapplication.html#exit">exit</a>() is called. Returns the value that was passed to <a href="http://doc.qt.io/qt-5/qcoreapplication.html#exit">exit</a>() (which is 0 if <a href="http://doc.qt.io/qt-5/qcoreapplication.html#exit">exit</a>() is called via <a href="http://doc.qt.io/qt-5/qcoreapplication.html#quit">quit</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QCoreApplication::exec()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#exec">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Enters the main event loop and waits until <a href="http://doc.qt.io/qt-5/qcoreapplication.html#exit">exit</a>() is called. Returns the value that was passed to <a href="http://doc.qt.io/qt-5/qcoreapplication.html#exit">exit</a>() (which is 0 if <a href="http://doc.qt.io/qt-5/qcoreapplication.html#exit">exit</a>() is called via <a href="http://doc.qt.io/qt-5/qcoreapplication.html#quit">quit</a>()).</p>
    /// <p>It is necessary to call this function to start event handling. The main event loop receives events from the window system and dispatches these to the application widgets.</p>
    /// <p>To make your application perform idle processing (by executing a special function whenever there are no pending events), use a <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> with 0 timeout. More advanced idle processing schemes can be achieved using <a href="http://doc.qt.io/qt-5/qcoreapplication.html#processEvents">processEvents</a>().</p>
    /// <p>We recommend that you connect clean-up code to the <a href="http://doc.qt.io/qt-5/qcoreapplication.html#aboutToQuit">aboutToQuit()</a> signal, instead of putting it in your application's <code>main()</code> function because on some platforms the exec() call may not return. For example, on Windows when the user logs off, the system terminates the process after Qt closes all top-level windows. Hence, there is no guarantee that the application will have time to exit its event loop and execute code at the end of the <code>main()</code> function after the exec() call.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#quit">quit</a>(), <a href="http://doc.qt.io/qt-5/qcoreapplication.html#exit">exit</a>(), <a href="http://doc.qt.io/qt-5/qcoreapplication.html#processEvents">processEvents</a>(), and <a href="http://doc.qt.io/qt-5/qapplication.html#exec">QApplication::exec</a>().</p></div>
    pub unsafe fn exec() -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_exec()
    }

    /// <p>Tells the application to exit with a return code.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QCoreApplication::exit(int retcode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#exit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Tells the application to exit with a return code.</p>
    /// <p>After this function has been called, the application leaves the main event loop and returns from the call to <a href="http://doc.qt.io/qt-5/qcoreapplication.html#exec">exec</a>(). The <a href="http://doc.qt.io/qt-5/qcoreapplication.html#exec">exec</a>() function returns <i>returnCode</i>. If the event loop is not running, this function does nothing.</p>
    /// <p>By convention, a <i>returnCode</i> of 0 means success, and any non-zero value indicates an error.</p>
    /// <p>Note that unlike the C library function of the same name, this function <i>does</i> return to the caller -- it is event processing that stops.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#quit">quit</a>() and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#exec">exec</a>().</p></div>
    pub unsafe fn exit_1a(retcode: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_exit(retcode)
    }

    /// <p>Tells the application to exit with a return code.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QCoreApplication::exit()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#exit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Tells the application to exit with a return code.</p>
    /// <p>After this function has been called, the application leaves the main event loop and returns from the call to <a href="http://doc.qt.io/qt-5/qcoreapplication.html#exec">exec</a>(). The <a href="http://doc.qt.io/qt-5/qcoreapplication.html#exec">exec</a>() function returns <i>returnCode</i>. If the event loop is not running, this function does nothing.</p>
    /// <p>By convention, a <i>returnCode</i> of 0 means success, and any non-zero value indicates an error.</p>
    /// <p>Note that unlike the C library function of the same name, this function <i>does</i> return to the caller -- it is event processing that stops.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#quit">quit</a>() and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#exec">exec</a>().</p></div>
    pub unsafe fn exit_0a() {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_exit1()
    }

    /// <p>This function is equivalent to calling <code>QCoreApplication::eventDispatcher()-&gt;flush()</code>, which also is deprecated, see <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-obsolete.html#flush">QAbstractEventDispatcher::flush</a>(). Use <a href="http://doc.qt.io/qt-5/qcoreapplication.html#sendPostedEvents">sendPostedEvents</a>() and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#processEvents">processEvents</a>() for more fine-grained control of the event loop instead.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QCoreApplication::flush()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication-obsolete.html#flush">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is equivalent to calling <code>QCoreApplication::eventDispatcher()-&gt;flush()</code>, which also is deprecated, see <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-obsolete.html#flush">QAbstractEventDispatcher::flush</a>(). Use <a href="http://doc.qt.io/qt-5/qcoreapplication.html#sendPostedEvents">sendPostedEvents</a>() and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#processEvents">processEvents</a>() for more fine-grained control of the event loop instead.</p>
    /// <p>Historically this functions was used to flush the platform-specific native event queues.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#sendPostedEvents">sendPostedEvents</a>(), <a href="http://doc.qt.io/qt-5/qcoreapplication.html#processEvents">processEvents</a>(), and <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-obsolete.html#flush">QAbstractEventDispatcher::flush</a>().</p></div>
    pub unsafe fn flush() {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_flush()
    }

    /// <p>This function returns <code>true</code> if there are pending events; otherwise returns <code>false</code>. Pending events can be either from the window system or posted events using <a href="http://doc.qt.io/qt-5/qcoreapplication.html#postEvent">postEvent</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QCoreApplication::hasPendingEvents()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication-obsolete.html#hasPendingEvents">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function returns <code>true</code> if there are pending events; otherwise returns <code>false</code>. Pending events can be either from the window system or posted events using <a href="http://doc.qt.io/qt-5/qcoreapplication.html#postEvent">postEvent</a>().</p>
    /// <p><b>Note: </b>this function is not thread-safe. It may only be called in the main thread and only if there are no other threads running in the application (including threads Qt starts for its own purposes).</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstracteventdispatcher-obsolete.html#hasPendingEvents">QAbstractEventDispatcher::hasPendingEvents</a>().</p></div>
    pub unsafe fn has_pending_events() -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_hasPendingEvents()
    }

    /// <p>Installs an event filter <i>filterObj</i> for all native events received by the application in the main thread.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCoreApplication::installNativeEventFilter(QAbstractNativeEventFilter* filterObj)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#installNativeEventFilter">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Installs an event filter <i>filterObj</i> for all native events received by the application in the main thread.</p>
    /// <p>The event filter <i>filterObj</i> receives events via its <a href="http://doc.qt.io/qt-5/qabstractnativeeventfilter.html#nativeEventFilter">nativeEventFilter()</a> function, which is called for all native events received in the main thread.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qabstractnativeeventfilter.html#nativeEventFilter">QAbstractNativeEventFilter::nativeEventFilter</a>() function should return true if the event should be filtered, i.e. stopped. It should return false to allow normal Qt processing to continue: the native event can then be translated into a <a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a> and handled by the standard Qt <a href="http://doc.qt.io/qt-5/qevent.html">event</a> filtering, e.g. <a href="http://doc.qt.io/qt-5/qobject.html#installEventFilter">QObject::installEventFilter</a>().</p>
    /// <p>If multiple event filters are installed, the filter that was installed last is activated first.</p>
    /// <p><b>Note: </b>The filter function set here receives native messages, i.e. MSG or XCB event structs.</p><p><b>Note: </b>Native event filters will be disabled in the application when the <a href="http://doc.qt.io/qt-5/qt.html#ApplicationAttribute-enum">Qt::AA_PluginApplication</a> attribute is set.</p><p>For maximum portability, you should always try to use <a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a> and <a href="http://doc.qt.io/qt-5/qobject.html#installEventFilter">QObject::installEventFilter</a>() whenever possible.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#installEventFilter">QObject::installEventFilter</a>().</p></div>
    pub unsafe fn install_native_event_filter(
        &mut self,
        filter_obj: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractNativeEventFilter>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_installNativeEventFilter(self as *mut crate::QCoreApplication, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractNativeEventFilter>>::cast_into(filter_obj).as_mut_raw_ptr())
    }

    /// <p>Adds the translation file <i>translationFile</i> to the list of translation files to be used for translations.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QCoreApplication::installTranslator(QTranslator* messageFile)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#installTranslator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds the translation file <i>translationFile</i> to the list of translation files to be used for translations.</p>
    /// <p>Multiple translation files can be installed. Translations are searched for in the reverse order in which they were installed, so the most recently installed translation file is searched first and the first translation file installed is searched last. The search stops as soon as a translation containing a matching string is found.</p>
    /// <p>Installing or removing a <a href="http://doc.qt.io/qt-5/qtranslator.html">QTranslator</a>, or changing an installed <a href="http://doc.qt.io/qt-5/qtranslator.html">QTranslator</a> generates a <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">LanguageChange</a> event for the <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> instance. A <a href="http://doc.qt.io/qt-5/qapplication.html">QApplication</a> instance will propagate the event to all toplevel widgets, where a reimplementation of changeEvent can re-translate the user interface by passing user-visible strings via the <a href="http://doc.qt.io/qt-5/qobject.html#tr">tr</a>() function to the respective property setters. User-interface classes generated by Qt Designer provide a <code>retranslateUi()</code> function that can be called.</p>
    /// <p>The function returns <code>true</code> on success and false on failure.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#removeTranslator">removeTranslator</a>(), <a href="http://doc.qt.io/qt-5/qcoreapplication.html#translate">translate</a>(), <a href="http://doc.qt.io/qt-5/qtranslator.html#load">QTranslator::load</a>(), and <a href="http://doc.qt.io/qt-5/internationalization.html#dynamic-translation">Dynamic Translation</a>.</p></div>
    pub unsafe fn install_translator(
        message_file: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QTranslator>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_installTranslator(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QTranslator>>::cast_into(
                message_file,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns a pointer to the application's <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> (or <a href="http://doc.qt.io/qt-5/qguiapplication.html">QGuiApplication</a>/<a href="http://doc.qt.io/qt-5/qapplication.html">QApplication</a>) instance.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QCoreApplication* QCoreApplication::instance()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#instance">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the application's <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> (or <a href="http://doc.qt.io/qt-5/qguiapplication.html">QGuiApplication</a>/<a href="http://doc.qt.io/qt-5/qapplication.html">QApplication</a>) instance.</p>
    /// <p>If no instance has been allocated, <code>null</code> is returned.</p></div>
    pub unsafe fn instance() -> ::cpp_utils::MutPtr<crate::QCoreApplication> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCoreApplication_instance();
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This property holds whether the use of the <a href="http://doc.qt.io/qt-5/qeventlooplocker.html">QEventLoopLocker</a> feature can cause the application to quit.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QCoreApplication::isQuitLockEnabled()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#quitLockEnabled-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds whether the use of the <a href="http://doc.qt.io/qt-5/qeventlooplocker.html">QEventLoopLocker</a> feature can cause the application to quit.</p>
    /// <p>The default is <code>true</code>.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> bool </td><td class="memItemRight bottomAlign"><span class="name"><b>isQuitLockEnabled</b></span>()</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setQuitLockEnabled</b></span>(bool <i>enabled</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qeventlooplocker.html">QEventLoopLocker</a>.</p></div>
    pub unsafe fn is_quit_lock_enabled() -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_isQuitLockEnabled()
    }

    /// <p>Returns true if the application is allowed to run setuid on UNIX platforms.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QCoreApplication::isSetuidAllowed()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#isSetuidAllowed">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the application is allowed to run setuid on UNIX platforms.</p>
    /// <p>This function was introduced in  Qt 5.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#setSetuidAllowed">QCoreApplication::setSetuidAllowed</a>().</p></div>
    pub unsafe fn is_setuid_allowed() -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_isSetuidAllowed()
    }

    /// <p>Returns a list of paths that the application will search when dynamically loading libraries.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QStringList QCoreApplication::libraryPaths()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#libraryPaths">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of paths that the application will search when dynamically loading libraries.</p>
    /// <p>The return value of this function may change when a <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> is created. It is not recommended to call it before creating a <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a>. The directory of the application executable (<b>not</b> the working directory) is part of the list if it is known. In order to make it known a <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> has to be constructed as it will use <code>argv[0]</code> to find it.</p>
    /// <p>Qt provides default library paths, but they can also be set using a <a href="http://doc.qt.io/qt-5/qt-conf.html">qt.conf</a> file. Paths specified in this file will override default values. Note that if the qt.conf file is in the directory of the application executable, it may not be found until a <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> is created. If it is not found when calling this function, the default library paths will be used.</p>
    /// <p>The list will include the installation directory for plugins if it exists (the default installation directory for plugins is <code>INSTALL/plugins</code>, where <code>INSTALL</code> is the directory where Qt was installed). The colon separated entries of the <code>QT_PLUGIN_PATH</code> environment variable are always added. The plugin installation directory (and its existence) may change when the directory of the application executable becomes known.</p>
    /// <p>If you want to iterate over the list, you can use the <a href="http://doc.qt.io/qt-5/containers.html#foreach">foreach</a> pseudo-keyword:</p>
    /// <pre class="cpp">
    ///
    ///   foreach (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>path<span class="operator">,</span> app<span class="operator">.</span>libraryPaths())
    /// &#32;     do_something(path);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#setLibraryPaths">setLibraryPaths</a>(), <a href="http://doc.qt.io/qt-5/qcoreapplication.html#addLibraryPath">addLibraryPath</a>(), <a href="http://doc.qt.io/qt-5/qcoreapplication.html#removeLibraryPath">removeLibraryPath</a>(), <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a>, and <a href="http://doc.qt.io/qt-5/plugins-howto.html">How to Create Qt Plugins</a>.</p></div>
    pub unsafe fn library_paths() -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCoreApplication_libraryPaths();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QCoreApplication::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCoreApplication_metaObject(
            self as *const crate::QCoreApplication,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs a Qt core application. Core applications are applications without a graphical user interface. Such applications are used at the console or as server processes.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCoreApplication::QCoreApplication(int& argc, char** argv, int arg3 = …)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qcoreapplication.html#QCoreApplication">C++ documentation</a> for <span style='color: green;'>```QCoreApplication::QCoreApplication(int &argc, char **argv)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a Qt core application. Core applications are applications without a graphical user interface. Such applications are used at the console or as server processes.</p>
    /// <p>The <i>argc</i> and <i>argv</i> arguments are processed by the application, and made available in a more convenient form by the <a href="http://doc.qt.io/qt-5/qcoreapplication.html#arguments">arguments</a>() function.</p>
    /// <p><b>Warning:</b> The data referred to by <i>argc</i> and <i>argv</i> must stay valid for the entire lifetime of the <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> object. In addition, <i>argc</i> must be greater than zero and <i>argv</i> must contain at least one valid character string.</p></div>
    pub unsafe fn new_3a(
        argc: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<::std::os::raw::c_int>>,
        argv: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::os::raw::c_char>>,
        arg3: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QCoreApplication> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCoreApplication_QCoreApplication(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<::std::os::raw::c_int>>::cast_into(argc)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::os::raw::c_char>>::cast_into(
                argv,
            )
            .as_mut_raw_ptr(),
            arg3,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a Qt core application. Core applications are applications without a graphical user interface. Such applications are used at the console or as server processes.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCoreApplication::QCoreApplication(int& argc, char** argv)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qcoreapplication.html#QCoreApplication">C++ documentation</a> for <span style='color: green;'>```QCoreApplication::QCoreApplication(int &argc, char **argv)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a Qt core application. Core applications are applications without a graphical user interface. Such applications are used at the console or as server processes.</p>
    /// <p>The <i>argc</i> and <i>argv</i> arguments are processed by the application, and made available in a more convenient form by the <a href="http://doc.qt.io/qt-5/qcoreapplication.html#arguments">arguments</a>() function.</p>
    /// <p><b>Warning:</b> The data referred to by <i>argc</i> and <i>argv</i> must stay valid for the entire lifetime of the <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> object. In addition, <i>argc</i> must be greater than zero and <i>argv</i> must contain at least one valid character string.</p></div>
    pub unsafe fn new_2a(
        argc: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<::std::os::raw::c_int>>,
        argv: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QCoreApplication> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCoreApplication_QCoreApplication2(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<::std::os::raw::c_int>>::cast_into(argc)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::os::raw::c_char>>::cast_into(
                argv,
            )
            .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Sends <i>event</i> to <i>receiver</i>: <i>receiver</i>-&gt;event(<i>event</i>). Returns the value that is returned from the receiver's event handler. Note that this function is called for all events sent to any object in any thread.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QCoreApplication::notify(QObject* arg1, QEvent* arg2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#notify">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sends <i>event</i> to <i>receiver</i>: <i>receiver</i>-&gt;event(<i>event</i>). Returns the value that is returned from the receiver's event handler. Note that this function is called for all events sent to any object in any thread.</p>
    /// <p>For certain types of events (e.g. mouse and key events), the event will be propagated to the receiver's parent and so on up to the top-level object if the receiver is not interested in the event (i.e., it returns <code>false</code>).</p>
    /// <p>There are five different ways that events can be processed; reimplementing this virtual function is just one of them. All five approaches are listed below:</p>
    /// <ol class="1" type="1"><li>Reimplementing <a href="http://doc.qt.io/qt-5/qwidget.html#paintEvent">paintEvent()</a>, <a href="http://doc.qt.io/qt-5/qwidget.html#mousePressEvent">mousePressEvent()</a> and so on. This is the most common, easiest, and least powerful way.</li>
    /// <li>Reimplementing this function. This is very powerful, providing complete control; but only one subclass can be active at a time.</li>
    /// <li>Installing an event filter on <a href="http://doc.qt.io/qt-5/qcoreapplication.html#instance">QCoreApplication::instance</a>(). Such an event filter is able to process all events for all widgets, so it's just as powerful as reimplementing notify(); furthermore, it's possible to have more than one application-global event filter. Global event filters even see mouse events for <a href="http://doc.qt.io/qt-5/qwidget.html#enabled-prop">disabled widgets</a>. Note that application event filters are only called for objects that live in the main thread.</li>
    /// <li>Reimplementing <a href="http://doc.qt.io/qt-5/qobject.html#event">QObject::event</a>() (as <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> does). If you do this you get Tab key presses, and you get to see the events before any widget-specific event filters.</li>
    /// <li>Installing an event filter on the object. Such an event filter gets all the events, including Tab and Shift+Tab key press events, as long as they do not change the focus widget.</li>
    /// </ol>
    /// <p><b>Future direction:</b> This function will not be called for objects that live outside the main thread in Qt 6. Applications that need that functionality should find other solutions for their event inspection needs in the meantime. The change may be extended to the main thread, causing this function to be deprecated.</p>
    /// <p><b>Warning:</b> If you override this function, you must ensure all threads that process events stop doing so before your application object begins destruction. This includes threads started by other libraries that you may be using, but does not apply to Qt's own threads.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#event">QObject::event</a>() and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#installNativeEventFilter">installNativeEventFilter</a>().</p></div>
    pub unsafe fn notify(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
        arg2: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QEvent>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_notify(
            self as *mut crate::QCoreApplication,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(arg1)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QEvent>>::cast_into(arg2)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>This property holds the Internet domain of the organization that wrote this application</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QCoreApplication::organizationDomain()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationDomain-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the Internet domain of the organization that wrote this application</p>
    /// <p>The value is used by the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> class when it is constructed using the empty constructor. This saves having to repeat this information each time a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object is created.</p>
    /// <p>On Mac, <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> uses organizationDomain() as the organization if it's not an empty string; otherwise it uses <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationName-prop">organizationName</a>(). On all other platforms, <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> uses <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationName-prop">organizationName</a>() as the organization.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QString </td><td class="memItemRight bottomAlign"><span class="name"><b>organizationDomain</b></span>()</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setOrganizationDomain</b></span>(const QString &amp;<i>orgDomain</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationName-prop">organizationName</a>, <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationName-prop">applicationName</a>, and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationVersion-prop">applicationVersion</a>.</p></div>
    pub unsafe fn organization_domain() -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCoreApplication_organizationDomain();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This property holds the name of the organization that wrote this application</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QCoreApplication::organizationName()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationName-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the name of the organization that wrote this application</p>
    /// <p>The value is used by the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> class when it is constructed using the empty constructor. This saves having to repeat this information each time a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object is created.</p>
    /// <p>On Mac, <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> uses <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationDomain-prop">organizationDomain()</a> as the organization if it's not an empty string; otherwise it uses organizationName(). On all other platforms, <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> uses organizationName() as the organization.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QString </td><td class="memItemRight bottomAlign"><span class="name"><b>organizationName</b></span>()</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setOrganizationName</b></span>(const QString &amp;<i>orgName</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationDomain-prop">organizationDomain</a> and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationName-prop">applicationName</a>.</p></div>
    pub unsafe fn organization_name() -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCoreApplication_organizationName();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Adds the event <i>event</i>, with the object <i>receiver</i> as the receiver of the event, to an event queue and returns immediately.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QCoreApplication::postEvent(QObject* receiver, QEvent* event, int priority = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#postEvent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds the event <i>event</i>, with the object <i>receiver</i> as the receiver of the event, to an event queue and returns immediately.</p>
    /// <p>The event must be allocated on the heap since the post event queue will take ownership of the event and delete it once it has been posted. It is <i>not safe</i> to access the event after it has been posted.</p>
    /// <p>When control returns to the main event loop, all events that are stored in the queue will be sent using the <a href="http://doc.qt.io/qt-5/qcoreapplication.html#notify">notify</a>() function.</p>
    /// <p>Events are sorted in descending <i>priority</i> order, i.e. events with a high <i>priority</i> are queued before events with a lower <i>priority</i>. The <i>priority</i> can be any integer value, i.e. between INT_MAX and INT_MIN, inclusive; see <a href="http://doc.qt.io/qt-5/qt.html#EventPriority-enum">Qt::EventPriority</a> for more details. Events with equal <i>priority</i> will be processed in the order posted.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p>
    /// <p>This function was introduced in  Qt 4.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#sendEvent">sendEvent</a>(), <a href="http://doc.qt.io/qt-5/qcoreapplication.html#notify">notify</a>(), <a href="http://doc.qt.io/qt-5/qcoreapplication.html#sendPostedEvents">sendPostedEvents</a>(), and <a href="http://doc.qt.io/qt-5/qt.html#EventPriority-enum">Qt::EventPriority</a>.</p></div>
    pub unsafe fn post_event_3a(
        receiver: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
        event: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QEvent>>,
        priority: ::std::os::raw::c_int,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_postEvent(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(receiver)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QEvent>>::cast_into(event)
                .as_mut_raw_ptr(),
            priority,
        )
    }

    /// <p>Adds the event <i>event</i>, with the object <i>receiver</i> as the receiver of the event, to an event queue and returns immediately.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QCoreApplication::postEvent(QObject* receiver, QEvent* event)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#postEvent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds the event <i>event</i>, with the object <i>receiver</i> as the receiver of the event, to an event queue and returns immediately.</p>
    /// <p>The event must be allocated on the heap since the post event queue will take ownership of the event and delete it once it has been posted. It is <i>not safe</i> to access the event after it has been posted.</p>
    /// <p>When control returns to the main event loop, all events that are stored in the queue will be sent using the <a href="http://doc.qt.io/qt-5/qcoreapplication.html#notify">notify</a>() function.</p>
    /// <p>Events are sorted in descending <i>priority</i> order, i.e. events with a high <i>priority</i> are queued before events with a lower <i>priority</i>. The <i>priority</i> can be any integer value, i.e. between INT_MAX and INT_MIN, inclusive; see <a href="http://doc.qt.io/qt-5/qt.html#EventPriority-enum">Qt::EventPriority</a> for more details. Events with equal <i>priority</i> will be processed in the order posted.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p>
    /// <p>This function was introduced in  Qt 4.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#sendEvent">sendEvent</a>(), <a href="http://doc.qt.io/qt-5/qcoreapplication.html#notify">notify</a>(), <a href="http://doc.qt.io/qt-5/qcoreapplication.html#sendPostedEvents">sendPostedEvents</a>(), and <a href="http://doc.qt.io/qt-5/qt.html#EventPriority-enum">Qt::EventPriority</a>.</p></div>
    pub unsafe fn post_event_2a(
        receiver: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
        event: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QEvent>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_postEvent1(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(receiver)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QEvent>>::cast_into(event)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Processes all pending events for the calling thread according to the specified <i>flags</i> until there are no more events to process.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QCoreApplication::processEvents(QFlags<QEventLoop::ProcessEventsFlag> flags = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#processEvents">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Processes all pending events for the calling thread according to the specified <i>flags</i> until there are no more events to process.</p>
    /// <p>You can call this function occasionally when your program is busy performing a long operation (e.g. copying a file).</p>
    /// <p>In the event that you are running a local loop which calls this function continuously, without an event loop, the <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">DeferredDelete</a> events will not be processed. This can affect the behaviour of widgets, e.g. <a href="http://doc.qt.io/qt-5/qtooltip.html">QToolTip</a>, that rely on <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">DeferredDelete</a> events to function properly. An alternative would be to call <a href="http://doc.qt.io/qt-5/qcoreapplication.html#sendPostedEvents">sendPostedEvents()</a> from within that local loop.</p>
    /// <p>Calling this function processes events only for the calling thread.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#exec">exec</a>(), <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a>, <a href="http://doc.qt.io/qt-5/qeventloop.html#processEvents">QEventLoop::processEvents</a>(), <a href="http://doc.qt.io/qt-5/qcoreapplication-obsolete.html#flush">flush</a>(), and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#sendPostedEvents">sendPostedEvents</a>().</p></div>
    pub unsafe fn process_events_1a(flags: crate::QFlags<crate::q_event_loop::ProcessEventsFlag>) {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_processEvents(flags.to_int())
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qcoreapplication.html#processEvents">processEvents</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QCoreApplication::processEvents(QFlags<QEventLoop::ProcessEventsFlag> flags, int maxtime)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#processEvents-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qcoreapplication.html#processEvents">processEvents</a>().</p>
    /// <p>Processes pending events for the calling thread for <i>maxtime</i> milliseconds or until there are no more events to process, whichever is shorter.</p>
    /// <p>You can call this function occasionally when your program is busy doing a long operation (e.g. copying a file).</p>
    /// <p>Calling this function processes events only for the calling thread.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#exec">exec</a>(), <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a>, and <a href="http://doc.qt.io/qt-5/qeventloop.html#processEvents">QEventLoop::processEvents</a>().</p></div>
    pub unsafe fn process_events_2a(
        flags: crate::QFlags<crate::q_event_loop::ProcessEventsFlag>,
        maxtime: ::std::os::raw::c_int,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_processEvents1(flags.to_int(), maxtime)
    }

    /// <p>Processes all pending events for the calling thread according to the specified <i>flags</i> until there are no more events to process.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QCoreApplication::processEvents()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#processEvents">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Processes all pending events for the calling thread according to the specified <i>flags</i> until there are no more events to process.</p>
    /// <p>You can call this function occasionally when your program is busy performing a long operation (e.g. copying a file).</p>
    /// <p>In the event that you are running a local loop which calls this function continuously, without an event loop, the <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">DeferredDelete</a> events will not be processed. This can affect the behaviour of widgets, e.g. <a href="http://doc.qt.io/qt-5/qtooltip.html">QToolTip</a>, that rely on <a href="http://doc.qt.io/qt-5/qevent.html#Type-enum">DeferredDelete</a> events to function properly. An alternative would be to call <a href="http://doc.qt.io/qt-5/qcoreapplication.html#sendPostedEvents">sendPostedEvents()</a> from within that local loop.</p>
    /// <p>Calling this function processes events only for the calling thread.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#exec">exec</a>(), <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a>, <a href="http://doc.qt.io/qt-5/qeventloop.html#processEvents">QEventLoop::processEvents</a>(), <a href="http://doc.qt.io/qt-5/qcoreapplication-obsolete.html#flush">flush</a>(), and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#sendPostedEvents">sendPostedEvents</a>().</p></div>
    pub unsafe fn process_events_0a() {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_processEvents2()
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QCoreApplication::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_qt_metacall(
            self as *mut crate::QCoreApplication,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QCoreApplication::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCoreApplication_qt_metacast(
            self as *mut crate::QCoreApplication,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Tells the application to exit with return code 0 (success). Equivalent to calling <a href="http://doc.qt.io/qt-5/qcoreapplication.html#exit">QCoreApplication::exit</a>(0).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static [slot] void QCoreApplication::quit()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#quit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Tells the application to exit with return code 0 (success). Equivalent to calling <a href="http://doc.qt.io/qt-5/qcoreapplication.html#exit">QCoreApplication::exit</a>(0).</p>
    /// <p>It's common to connect the <a href="http://doc.qt.io/qt-5/qguiapplication.html#lastWindowClosed">QGuiApplication::lastWindowClosed</a>() signal to quit(), and you also often connect e.g. <a href="http://doc.qt.io/qt-5/qabstractbutton.html#clicked">QAbstractButton::clicked</a>() or signals in <a href="http://doc.qt.io/qt-5/qaction.html">QAction</a>, <a href="http://doc.qt.io/qt-5/qmenu.html">QMenu</a>, or <a href="http://doc.qt.io/qt-5/qmenubar.html">QMenuBar</a> to it.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span>quitButton <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span>(<span class="string">"Quit"</span>);
    ///   connect(quitButton<span class="operator">,</span> SIGNAL(clicked())<span class="operator">,</span> <span class="operator">&amp;</span>app<span class="operator">,</span> SLOT(quit()));
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#exit">exit</a>(), <a href="http://doc.qt.io/qt-5/qcoreapplication.html#aboutToQuit">aboutToQuit</a>(), and <a href="http://doc.qt.io/qt-5/qguiapplication.html#lastWindowClosed">QGuiApplication::lastWindowClosed</a>().</p></div>
    pub unsafe fn quit() {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_quit()
    }

    /// <p>Removes <i>path</i> from the library path list. If <i>path</i> is empty or not in the path list, the list is not changed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QCoreApplication::removeLibraryPath(const QString& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#removeLibraryPath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes <i>path</i> from the library path list. If <i>path</i> is empty or not in the path list, the list is not changed.</p>
    /// <p>The library paths are reset to the default when an instance of <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> is destructed.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#addLibraryPath">addLibraryPath</a>(), <a href="http://doc.qt.io/qt-5/qcoreapplication.html#libraryPaths">libraryPaths</a>(), and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#setLibraryPaths">setLibraryPaths</a>().</p></div>
    pub unsafe fn remove_library_path(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_removeLibraryPath(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(arg1).as_raw_ptr(),
        )
    }

    /// <p>Removes an event <i>filterObject</i> from this object. The request is ignored if such an event filter has not been installed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCoreApplication::removeNativeEventFilter(QAbstractNativeEventFilter* filterObj)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#removeNativeEventFilter">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes an event <i>filterObject</i> from this object. The request is ignored if such an event filter has not been installed.</p>
    /// <p>All event filters for this object are automatically removed when this object is destroyed.</p>
    /// <p>It is always safe to remove an event filter, even during event filter activation (i.e. from the nativeEventFilter() function).</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#installNativeEventFilter">installNativeEventFilter</a>().</p></div>
    pub unsafe fn remove_native_event_filter(
        &mut self,
        filter_obj: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractNativeEventFilter>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_removeNativeEventFilter(self as *mut crate::QCoreApplication, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractNativeEventFilter>>::cast_into(filter_obj).as_mut_raw_ptr())
    }

    /// <p>Removes all events of the given <i>eventType</i> that were posted using <a href="http://doc.qt.io/qt-5/qcoreapplication.html#postEvent">postEvent</a>() for <i>receiver</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QCoreApplication::removePostedEvents(QObject* receiver, int eventType = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#removePostedEvents">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all events of the given <i>eventType</i> that were posted using <a href="http://doc.qt.io/qt-5/qcoreapplication.html#postEvent">postEvent</a>() for <i>receiver</i>.</p>
    /// <p>The events are <i>not</i> dispatched, instead they are removed from the queue. You should never need to call this function. If you do call it, be aware that killing events may cause <i>receiver</i> to break one or more invariants.</p>
    /// <p>If <i>receiver</i> is null, the events of <i>eventType</i> are removed for all objects. If <i>eventType</i> is 0, all the events are removed for <i>receiver</i>. You should never call this function with <i>eventType</i> of 0. If you do call it in this way, be aware that killing events may cause <i>receiver</i> to break one or more invariants.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p>
    /// <p>This function was introduced in  Qt 4.3.</p></div>
    pub unsafe fn remove_posted_events_2a(
        receiver: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
        event_type: ::std::os::raw::c_int,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_removePostedEvents(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(receiver)
                .as_mut_raw_ptr(),
            event_type,
        )
    }

    /// <p>Removes all events of the given <i>eventType</i> that were posted using <a href="http://doc.qt.io/qt-5/qcoreapplication.html#postEvent">postEvent</a>() for <i>receiver</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QCoreApplication::removePostedEvents(QObject* receiver)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#removePostedEvents">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all events of the given <i>eventType</i> that were posted using <a href="http://doc.qt.io/qt-5/qcoreapplication.html#postEvent">postEvent</a>() for <i>receiver</i>.</p>
    /// <p>The events are <i>not</i> dispatched, instead they are removed from the queue. You should never need to call this function. If you do call it, be aware that killing events may cause <i>receiver</i> to break one or more invariants.</p>
    /// <p>If <i>receiver</i> is null, the events of <i>eventType</i> are removed for all objects. If <i>eventType</i> is 0, all the events are removed for <i>receiver</i>. You should never call this function with <i>eventType</i> of 0. If you do call it in this way, be aware that killing events may cause <i>receiver</i> to break one or more invariants.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p>
    /// <p>This function was introduced in  Qt 4.3.</p></div>
    pub unsafe fn remove_posted_events_1a(
        receiver: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_removePostedEvents1(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(receiver)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Removes the translation file <i>translationFile</i> from the list of translation files used by this application. (It does not delete the translation file from the file system.)</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QCoreApplication::removeTranslator(QTranslator* messageFile)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#removeTranslator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the translation file <i>translationFile</i> from the list of translation files used by this application. (It does not delete the translation file from the file system.)</p>
    /// <p>The function returns <code>true</code> on success and false on failure.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#installTranslator">installTranslator</a>(), <a href="http://doc.qt.io/qt-5/qcoreapplication.html#translate">translate</a>(), and <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>().</p></div>
    pub unsafe fn remove_translator(
        message_file: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QTranslator>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_removeTranslator(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QTranslator>>::cast_into(
                message_file,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Sends event <i>event</i> directly to receiver <i>receiver</i>, using the <a href="http://doc.qt.io/qt-5/qcoreapplication.html#notify">notify</a>() function. Returns the value that was returned from the event handler.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QCoreApplication::sendEvent(QObject* receiver, QEvent* event)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#sendEvent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sends event <i>event</i> directly to receiver <i>receiver</i>, using the <a href="http://doc.qt.io/qt-5/qcoreapplication.html#notify">notify</a>() function. Returns the value that was returned from the event handler.</p>
    /// <p>The event is <i>not</i> deleted when the event has been sent. The normal approach is to create the event on the stack, for example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmouseevent.html">QMouseEvent</a></span> event(<span class="type"><a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a></span><span class="operator">::</span>MouseButtonPress<span class="operator">,</span> pos<span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="number">0</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qapplication.html">QApplication</a></span><span class="operator">::</span>sendEvent(mainWindow<span class="operator">,</span> <span class="operator">&amp;</span>event);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#postEvent">postEvent</a>() and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#notify">notify</a>().</p></div>
    pub unsafe fn send_event(
        receiver: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
        event: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QEvent>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_sendEvent(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(receiver)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QEvent>>::cast_into(event)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Immediately dispatches all events which have been previously queued with <a href="http://doc.qt.io/qt-5/qcoreapplication.html#postEvent">QCoreApplication::postEvent</a>() and which are for the object <i>receiver</i> and have the event type <i>event_type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QCoreApplication::sendPostedEvents(QObject* receiver = …, int event_type = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#sendPostedEvents">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Immediately dispatches all events which have been previously queued with <a href="http://doc.qt.io/qt-5/qcoreapplication.html#postEvent">QCoreApplication::postEvent</a>() and which are for the object <i>receiver</i> and have the event type <i>event_type</i>.</p>
    /// <p>Events from the window system are <i>not</i> dispatched by this function, but by <a href="http://doc.qt.io/qt-5/qcoreapplication.html#processEvents">processEvents</a>().</p>
    /// <p>If <i>receiver</i> is null, the events of <i>event_type</i> are sent for all objects. If <i>event_type</i> is 0, all the events are sent for <i>receiver</i>.</p>
    /// <p><b>Note: </b>This method must be called from the thread in which its <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> parameter, <i>receiver</i>, lives.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication-obsolete.html#flush">flush</a>() and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#postEvent">postEvent</a>().</p></div>
    pub unsafe fn send_posted_events_2a(
        receiver: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
        event_type: ::std::os::raw::c_int,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_sendPostedEvents(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(receiver)
                .as_mut_raw_ptr(),
            event_type,
        )
    }

    /// <p>Immediately dispatches all events which have been previously queued with <a href="http://doc.qt.io/qt-5/qcoreapplication.html#postEvent">QCoreApplication::postEvent</a>() and which are for the object <i>receiver</i> and have the event type <i>event_type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QCoreApplication::sendPostedEvents(QObject* receiver = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#sendPostedEvents">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Immediately dispatches all events which have been previously queued with <a href="http://doc.qt.io/qt-5/qcoreapplication.html#postEvent">QCoreApplication::postEvent</a>() and which are for the object <i>receiver</i> and have the event type <i>event_type</i>.</p>
    /// <p>Events from the window system are <i>not</i> dispatched by this function, but by <a href="http://doc.qt.io/qt-5/qcoreapplication.html#processEvents">processEvents</a>().</p>
    /// <p>If <i>receiver</i> is null, the events of <i>event_type</i> are sent for all objects. If <i>event_type</i> is 0, all the events are sent for <i>receiver</i>.</p>
    /// <p><b>Note: </b>This method must be called from the thread in which its <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> parameter, <i>receiver</i>, lives.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication-obsolete.html#flush">flush</a>() and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#postEvent">postEvent</a>().</p></div>
    pub unsafe fn send_posted_events_1a(
        receiver: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_sendPostedEvents1(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(receiver)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Immediately dispatches all events which have been previously queued with <a href="http://doc.qt.io/qt-5/qcoreapplication.html#postEvent">QCoreApplication::postEvent</a>() and which are for the object <i>receiver</i> and have the event type <i>event_type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QCoreApplication::sendPostedEvents()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#sendPostedEvents">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Immediately dispatches all events which have been previously queued with <a href="http://doc.qt.io/qt-5/qcoreapplication.html#postEvent">QCoreApplication::postEvent</a>() and which are for the object <i>receiver</i> and have the event type <i>event_type</i>.</p>
    /// <p>Events from the window system are <i>not</i> dispatched by this function, but by <a href="http://doc.qt.io/qt-5/qcoreapplication.html#processEvents">processEvents</a>().</p>
    /// <p>If <i>receiver</i> is null, the events of <i>event_type</i> are sent for all objects. If <i>event_type</i> is 0, all the events are sent for <i>receiver</i>.</p>
    /// <p><b>Note: </b>This method must be called from the thread in which its <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> parameter, <i>receiver</i>, lives.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication-obsolete.html#flush">flush</a>() and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#postEvent">postEvent</a>().</p></div>
    pub unsafe fn send_posted_events_0a() {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_sendPostedEvents2()
    }

    /// <p>This property holds the name of this application</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QCoreApplication::setApplicationName(const QString& application)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationName-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the name of this application</p>
    /// <p>The value is used by the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> class when it is constructed using the empty constructor. This saves having to repeat this information each time a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object is created.</p>
    /// <p>If not set, the application name defaults to the executable name (since 5.0).</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QString </td><td class="memItemRight bottomAlign"><span class="name"><b>applicationName</b></span>()</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setApplicationName</b></span>(const QString &amp;<i>application</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationName-prop">organizationName</a>, <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationDomain-prop">organizationDomain</a>, <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationVersion-prop">applicationVersion</a>, and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationFilePath">applicationFilePath</a>().</p></div>
    pub unsafe fn set_application_name(
        application: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_setApplicationName(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(application)
                .as_raw_ptr(),
        )
    }

    /// <p>This property holds the version of this application</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QCoreApplication::setApplicationVersion(const QString& version)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationVersion-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the version of this application</p>
    /// <p>If not set, the application version defaults to a platform-specific value determined from the main application executable or package (since Qt 5.9):</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Platform</th><th>Source</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>Windows (classic desktop)</td><td>PRODUCTVERSION parameter of the VERSIONINFO resource</td></tr>
    /// <tr valign="top" class="even"><td>Universal Windows Platform</td><td>version attribute of the application package manifest</td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, iOS, tvOS, watchOS</td><td>CFBundleVersion property of the information property list</td></tr>
    /// <tr valign="top" class="even"><td>Android</td><td>android:versionName property of the AndroidManifest.xml manifest element</td></tr>
    /// </tbody></table></div>
    /// <p>On other platforms, the default is the empty string.</p>
    /// <p>This property was introduced in  Qt 4.4.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QString </td><td class="memItemRight bottomAlign"><span class="name"><b>applicationVersion</b></span>()</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setApplicationVersion</b></span>(const QString &amp;<i>version</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationName-prop">applicationName</a>, <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationName-prop">organizationName</a>, and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationDomain-prop">organizationDomain</a>.</p></div>
    pub unsafe fn set_application_version(
        version: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_setApplicationVersion(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(version)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the attribute <i>attribute</i> if <i>on</i> is true; otherwise clears the attribute.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QCoreApplication::setAttribute(Qt::ApplicationAttribute attribute, bool on = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#setAttribute">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the attribute <i>attribute</i> if <i>on</i> is true; otherwise clears the attribute.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#testAttribute">testAttribute</a>().</p></div>
    pub unsafe fn set_attribute_2a(attribute: crate::ApplicationAttribute, on: bool) {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_setAttribute(attribute, on)
    }

    /// <p>Sets the attribute <i>attribute</i> if <i>on</i> is true; otherwise clears the attribute.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QCoreApplication::setAttribute(Qt::ApplicationAttribute attribute)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#setAttribute">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the attribute <i>attribute</i> if <i>on</i> is true; otherwise clears the attribute.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#testAttribute">testAttribute</a>().</p></div>
    pub unsafe fn set_attribute_1a(attribute: crate::ApplicationAttribute) {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_setAttribute1(attribute)
    }

    /// <p>Sets the event dispatcher for the main thread to <i>eventDispatcher</i>. This is only possible as long as there is no event dispatcher installed yet. That is, before <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> has been instantiated. This method takes ownership of the object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QCoreApplication::setEventDispatcher(QAbstractEventDispatcher* eventDispatcher)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#setEventDispatcher">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the event dispatcher for the main thread to <i>eventDispatcher</i>. This is only possible as long as there is no event dispatcher installed yet. That is, before <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> has been instantiated. This method takes ownership of the object.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#eventDispatcher">eventDispatcher</a>().</p></div>
    pub unsafe fn set_event_dispatcher(
        event_dispatcher: impl ::cpp_utils::CastInto<
            ::cpp_utils::MutPtr<crate::QAbstractEventDispatcher>,
        >,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_setEventDispatcher(::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractEventDispatcher>>::cast_into(event_dispatcher).as_mut_raw_ptr())
    }

    /// <p>Sets the list of directories to search when loading libraries to <i>paths</i>. All existing paths will be deleted and the path list will consist of the paths given in <i>paths</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QCoreApplication::setLibraryPaths(const QStringList& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#setLibraryPaths">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the list of directories to search when loading libraries to <i>paths</i>. All existing paths will be deleted and the path list will consist of the paths given in <i>paths</i>.</p>
    /// <p>The library paths are reset to the default when an instance of <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> is destructed.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#libraryPaths">libraryPaths</a>(), <a href="http://doc.qt.io/qt-5/qcoreapplication.html#addLibraryPath">addLibraryPath</a>(), <a href="http://doc.qt.io/qt-5/qcoreapplication.html#removeLibraryPath">removeLibraryPath</a>(), and <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a>.</p></div>
    pub unsafe fn set_library_paths(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_setLibraryPaths(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(arg1)
                .as_raw_ptr(),
        )
    }

    /// <p>This property holds the Internet domain of the organization that wrote this application</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QCoreApplication::setOrganizationDomain(const QString& orgDomain)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationDomain-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the Internet domain of the organization that wrote this application</p>
    /// <p>The value is used by the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> class when it is constructed using the empty constructor. This saves having to repeat this information each time a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object is created.</p>
    /// <p>On Mac, <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> uses organizationDomain() as the organization if it's not an empty string; otherwise it uses <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationName-prop">organizationName</a>(). On all other platforms, <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> uses <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationName-prop">organizationName</a>() as the organization.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QString </td><td class="memItemRight bottomAlign"><span class="name"><b>organizationDomain</b></span>()</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setOrganizationDomain</b></span>(const QString &amp;<i>orgDomain</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationName-prop">organizationName</a>, <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationName-prop">applicationName</a>, and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationVersion-prop">applicationVersion</a>.</p></div>
    pub unsafe fn set_organization_domain(
        org_domain: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_setOrganizationDomain(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(org_domain)
                .as_raw_ptr(),
        )
    }

    /// <p>This property holds the name of the organization that wrote this application</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QCoreApplication::setOrganizationName(const QString& orgName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationName-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the name of the organization that wrote this application</p>
    /// <p>The value is used by the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> class when it is constructed using the empty constructor. This saves having to repeat this information each time a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object is created.</p>
    /// <p>On Mac, <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> uses <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationDomain-prop">organizationDomain()</a> as the organization if it's not an empty string; otherwise it uses organizationName(). On all other platforms, <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> uses organizationName() as the organization.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QString </td><td class="memItemRight bottomAlign"><span class="name"><b>organizationName</b></span>()</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setOrganizationName</b></span>(const QString &amp;<i>orgName</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationDomain-prop">organizationDomain</a> and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationName-prop">applicationName</a>.</p></div>
    pub unsafe fn set_organization_name(
        org_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_setOrganizationName(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(org_name)
                .as_raw_ptr(),
        )
    }

    /// <p>This property holds whether the use of the <a href="http://doc.qt.io/qt-5/qeventlooplocker.html">QEventLoopLocker</a> feature can cause the application to quit.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QCoreApplication::setQuitLockEnabled(bool enabled)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#quitLockEnabled-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds whether the use of the <a href="http://doc.qt.io/qt-5/qeventlooplocker.html">QEventLoopLocker</a> feature can cause the application to quit.</p>
    /// <p>The default is <code>true</code>.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> bool </td><td class="memItemRight bottomAlign"><span class="name"><b>isQuitLockEnabled</b></span>()</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setQuitLockEnabled</b></span>(bool <i>enabled</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qeventlooplocker.html">QEventLoopLocker</a>.</p></div>
    pub unsafe fn set_quit_lock_enabled(enabled: bool) {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_setQuitLockEnabled(enabled)
    }

    /// <p>Allows the application to run setuid on UNIX platforms if <i>allow</i> is true.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QCoreApplication::setSetuidAllowed(bool allow)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#setSetuidAllowed">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Allows the application to run setuid on UNIX platforms if <i>allow</i> is true.</p>
    /// <p>If <i>allow</i> is false (the default) and Qt detects the application is running with an effective user id different than the real user id, the application will be aborted when a <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> instance is created.</p>
    /// <p>Qt is not an appropriate solution for setuid programs due to its large attack surface. However some applications may be required to run in this manner for historical reasons. This flag will prevent Qt from aborting the application when this is detected, and must be set before a <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> instance is created.</p>
    /// <p><b>Note: </b>It is strongly recommended not to enable this option since it introduces security risks.</p><p>This function was introduced in  Qt 5.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#isSetuidAllowed">isSetuidAllowed</a>().</p></div>
    pub unsafe fn set_setuid_allowed(allow: bool) {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_setSetuidAllowed(allow)
    }

    /// <p>Returns <code>true</code> if an application object has not been created yet; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QCoreApplication::startingUp()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#startingUp">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if an application object has not been created yet; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#closingDown">closingDown</a>().</p></div>
    pub unsafe fn starting_up() -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_startingUp()
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCoreApplication_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if attribute <i>attribute</i> is set; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QCoreApplication::testAttribute(Qt::ApplicationAttribute attribute)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#testAttribute">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if attribute <i>attribute</i> is set; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#setAttribute">setAttribute</a>().</p></div>
    pub unsafe fn test_attribute(attribute: crate::ApplicationAttribute) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_testAttribute(attribute)
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QCoreApplication::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCoreApplication_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QCoreApplication::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCoreApplication_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is obsolete. It is provided to keep old source code working. We strongly advise against using it in new code.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QCoreApplication::translate(const char* context, const char* key, const char* disambiguation = …, int n = …)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qcoreapplication-obsolete.html#translate-1">C++ documentation</a> for <span style='color: green;'>```static QString QCoreApplication::translate(const char *context, const char *key, const char *disambiguation, QCoreApplication::Encoding encoding, int n = ...)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is obsolete. It is provided to keep old source code working. We strongly advise against using it in new code.</p></div>
    pub unsafe fn translate_4a(
        context: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        disambiguation: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCoreApplication_translate(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(context)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(key)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(
                disambiguation,
            )
            .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is obsolete. It is provided to keep old source code working. We strongly advise against using it in new code.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QCoreApplication::translate(const char* context, const char* key, const char* disambiguation = …)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qcoreapplication-obsolete.html#translate-1">C++ documentation</a> for <span style='color: green;'>```static QString QCoreApplication::translate(const char *context, const char *key, const char *disambiguation, QCoreApplication::Encoding encoding, int n = ...)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is obsolete. It is provided to keep old source code working. We strongly advise against using it in new code.</p></div>
    pub unsafe fn translate_3a(
        context: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        disambiguation: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCoreApplication_translate1(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(context)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(key)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(
                disambiguation,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is obsolete. It is provided to keep old source code working. We strongly advise against using it in new code.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QCoreApplication::translate(const char* context, const char* key)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qcoreapplication-obsolete.html#translate-1">C++ documentation</a> for <span style='color: green;'>```static QString QCoreApplication::translate(const char *context, const char *key, const char *disambiguation, QCoreApplication::Encoding encoding, int n = ...)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is obsolete. It is provided to keep old source code working. We strongly advise against using it in new code.</p></div>
    pub unsafe fn translate_2a(
        context: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCoreApplication_translate2(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(context)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_command_line_parser {
    //! C++ type: <span style='color: green;'>```QCommandLineParser```</span>

    /// <p>This enum describes the way the parser interprets command-line options that use a single dash followed by multiple letters, as as <code>-abc</code>.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QCommandLineParser::SingleDashWordOptionMode```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#SingleDashWordOptionMode-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the way the parser interprets command-line options that use a single dash followed by multiple letters, as as <code>-abc</code>.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineparser.html#setSingleDashWordOptionMode">setSingleDashWordOptionMode</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct SingleDashWordOptionMode(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for SingleDashWordOptionMode {
        fn from(value: ::std::os::raw::c_int) -> Self {
            SingleDashWordOptionMode(value)
        }
    }

    impl From<SingleDashWordOptionMode> for ::std::os::raw::c_int {
        fn from(value: SingleDashWordOptionMode) -> Self {
            value.0
        }
    }

    impl SingleDashWordOptionMode {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl SingleDashWordOptionMode {
        /// <code>-abc</code> is interpreted as <code>-a -b -c</code>, i.e. as three short options that have been compacted on the command-line, if none of the options take a value. If <code>a</code> takes a value, then it is interpreted as <code>-a bc</code>, i.e. the short option <code>a</code> followed by the value <code>bc</code>. This is typically used in tools that behave like compilers, in order to handle options such as <code>-DDEFINE=VALUE</code> or <code>-I/include/path</code>. This is the default parsing mode. New applications are recommended to use this mode. (C++ enum variant: <span style='color: green;'>```ParseAsCompactedShortOptions = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const ParseAsCompactedShortOptions:
            crate::q_command_line_parser::SingleDashWordOptionMode =
            crate::q_command_line_parser::SingleDashWordOptionMode(0);
        /// <code>-abc</code> is interpreted as <code>--abc</code>, i.e. as the long option named <code>abc</code>. This is how Qt's own tools (uic, rcc...) have always been parsing arguments. This mode should be used for preserving compatibility in applications that were parsing arguments in such a way. There is an exception if the <code>a</code> option has the <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#Flag-enum">QCommandLineOption::ShortOptionStyle</a> flag set, in which case it is still interpreted as <code>-a bc</code>. (C++ enum variant: <span style='color: green;'>```ParseAsLongOptions = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const ParseAsLongOptions: crate::q_command_line_parser::SingleDashWordOptionMode =
            crate::q_command_line_parser::SingleDashWordOptionMode(1);
    }

    /// <p>This enum describes the way the parser interprets options that occur after positional arguments.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QCommandLineParser::OptionsAfterPositionalArgumentsMode```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#OptionsAfterPositionalArgumentsMode-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the way the parser interprets options that occur after positional arguments.</p>
    ///
    /// <p>This enum was introduced or modified in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineparser.html#setOptionsAfterPositionalArgumentsMode">setOptionsAfterPositionalArgumentsMode</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct OptionsAfterPositionalArgumentsMode(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for OptionsAfterPositionalArgumentsMode {
        fn from(value: ::std::os::raw::c_int) -> Self {
            OptionsAfterPositionalArgumentsMode(value)
        }
    }

    impl From<OptionsAfterPositionalArgumentsMode> for ::std::os::raw::c_int {
        fn from(value: OptionsAfterPositionalArgumentsMode) -> Self {
            value.0
        }
    }

    impl OptionsAfterPositionalArgumentsMode {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl OptionsAfterPositionalArgumentsMode {
        /// <code>application argument --opt -t</code> is interpreted as setting the options <code>opt</code> and <code>t</code>, just like <code>application --opt -t argument</code> would do. This is the default parsing mode. In order to specify that <code>--opt</code> and <code>-t</code> are positional arguments instead, the user can use <code>--</code>, as in <code>application argument -- --opt -t</code>. (C++ enum variant: <span style='color: green;'>```ParseAsOptions = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const ParseAsOptions:
            crate::q_command_line_parser::OptionsAfterPositionalArgumentsMode =
            crate::q_command_line_parser::OptionsAfterPositionalArgumentsMode(0);
        /// <code>application argument --opt</code> is interpreted as having two positional arguments, <code>argument</code> and <code>--opt</code>. This mode is useful for executables that aim to launch other executables (e.g. wrappers, debugging tools, etc.) or that support internal commands followed by options for the command. <code>argument</code> is the name of the command, and all options occurring after it can be collected and parsed by another command line parser, possibly in another executable. (C++ enum variant: <span style='color: green;'>```ParseAsPositionalArguments = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const ParseAsPositionalArguments:
            crate::q_command_line_parser::OptionsAfterPositionalArgumentsMode =
            crate::q_command_line_parser::OptionsAfterPositionalArgumentsMode(1);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qcommandlineparser.html">QCommandLineParser</a> class provides a means for handling the command line options.</p>
///
/// C++ class: <span style='color: green;'>```QCommandLineParser```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qcommandlineparser.html">QCommandLineParser</a> class provides a means for handling the command line options.</p>
/// <p><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> provides the command-line arguments as a simple list of strings. <a href="http://doc.qt.io/qt-5/qcommandlineparser.html">QCommandLineParser</a> provides the ability to define a set of options, parse the command-line arguments, and store which options have actually been used, as well as option values.</p>
/// <p>Any argument that isn't an option (i.e. doesn't start with a <code>-</code>) is stored as a "positional argument".</p>
/// <p>The parser handles short names, long names, more than one name for the same option, and option values.</p>
/// <p>Options on the command line are recognized as starting with a single or double <code>-</code> character(s). The option <code>-</code> (single dash alone) is a special case, often meaning standard input, and not treated as an option. The parser will treat everything after the option <code>--</code> (double dash) as positional arguments.</p>
/// <p>Short options are single letters. The option <code>v</code> would be specified by passing <code>-v</code> on the command line. In the default parsing mode, short options can be written in a compact form, for instance <code>-abc</code> is equivalent to <code>-a -b -c</code>. The parsing mode for can be set to <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#SingleDashWordOptionMode-enum">ParseAsLongOptions</a>, in which case <code>-abc</code> will be parsed as the long option <code>abc</code>.</p>
/// <p>Long options are more than one letter long and cannot be compacted together. The long option <code>verbose</code> would be passed as <code>--verbose</code> or <code>-verbose</code>.</p>
/// <p>Passing values to options can be done using the assignment operator: <code>-v=value</code> <code>--verbose=value</code>, or a space: <code>-v value</code> <code>--verbose value</code>, i.e. the next argument is used as value (even if it starts with a <code>-</code>).</p>
/// <p>The parser does not support optional values - if an option is set to require a value, one must be present. If such an option is placed last and has no value, the option will be treated as if it had not been specified.</p>
/// <p>The parser does not automatically support negating or disabling long options by using the format <code>--disable-option</code> or <code>--no-option</code>. However, it is possible to handle this case explicitly by making an option with <code>no-option</code> as one of its names, and handling the option explicitly.</p>
/// <p>Example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> main(<span class="type">int</span> argc<span class="operator">,</span> <span class="type">char</span> <span class="operator">*</span>argv<span class="operator">[</span><span class="operator">]</span>)
///   {
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span> app(argc<span class="operator">,</span> argv);
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>setApplicationName(<span class="string">"my-copy-program"</span>);
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>setApplicationVersion(<span class="string">"1.0"</span>);
///
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcommandlineparser.html#QCommandLineParser">QCommandLineParser</a></span> parser;
/// &#32;     parser<span class="operator">.</span>setApplicationDescription(<span class="string">"Test helper"</span>);
/// &#32;     parser<span class="operator">.</span>addHelpOption();
/// &#32;     parser<span class="operator">.</span>addVersionOption();
/// &#32;     parser<span class="operator">.</span>addPositionalArgument(<span class="string">"source"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"main"</span><span class="operator">,</span> <span class="string">"Source file to copy."</span>));
/// &#32;     parser<span class="operator">.</span>addPositionalArgument(<span class="string">"destination"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"main"</span><span class="operator">,</span> <span class="string">"Destination directory."</span>));
///
/// &#32;     <span class="comment">// A boolean option with a single name (-p)</span>
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcommandlineoption.html">QCommandLineOption</a></span> showProgressOption(<span class="string">"p"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"main"</span><span class="operator">,</span> <span class="string">"Show progress during copy"</span>));
/// &#32;     parser<span class="operator">.</span>addOption(showProgressOption);
///
/// &#32;     <span class="comment">// A boolean option with multiple names (-f, --force)</span>
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcommandlineoption.html">QCommandLineOption</a></span> forceOption(<span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"f"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"force"</span><span class="operator">,</span>
/// &#32;   &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"main"</span><span class="operator">,</span> <span class="string">"Overwrite existing files."</span>));
/// &#32;     parser<span class="operator">.</span>addOption(forceOption);
///
/// &#32;     <span class="comment">// An option with a value</span>
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcommandlineoption.html">QCommandLineOption</a></span> targetDirectoryOption(<span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"t"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"target-directory"</span><span class="operator">,</span>
/// &#32;   &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"main"</span><span class="operator">,</span> <span class="string">"Copy all source files into &lt;directory&gt;."</span>)<span class="operator">,</span>
/// &#32;   &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"main"</span><span class="operator">,</span> <span class="string">"directory"</span>));
/// &#32;     parser<span class="operator">.</span>addOption(targetDirectoryOption);
///
/// &#32;     <span class="comment">// Process the actual command line arguments given by the user</span>
/// &#32;     parser<span class="operator">.</span>process(app);
///
/// &#32;     <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> args <span class="operator">=</span> parser<span class="operator">.</span>positionalArguments();
/// &#32;     <span class="comment">// source is args.at(0), destination is args.at(1)</span>
///
/// &#32;     bool showProgress <span class="operator">=</span> parser<span class="operator">.</span>isSet(showProgressOption);
/// &#32;     bool force <span class="operator">=</span> parser<span class="operator">.</span>isSet(forceOption);
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> targetDir <span class="operator">=</span> parser<span class="operator">.</span>value(targetDirectoryOption);
/// &#32;     <span class="comment">// ...</span>
///   }
///
/// </pre>
/// <p>If your compiler supports the C++11 standard, the three <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#addOption">addOption</a>() calls in the above example can be simplified:</p>
/// <pre class="cpp">
///
/// &#32;     parser<span class="operator">.</span>addOptions({
/// &#32;   &#32;     <span class="comment">// A boolean option with a single name (-p)</span>
/// &#32;   &#32;     {<span class="string">"p"</span><span class="operator">,</span>
/// &#32;   &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"main"</span><span class="operator">,</span> <span class="string">"Show progress during copy"</span>)}<span class="operator">,</span>
/// &#32;   &#32;     <span class="comment">// A boolean option with multiple names (-f, --force)</span>
/// &#32;   &#32;     {{<span class="string">"f"</span><span class="operator">,</span> <span class="string">"force"</span>}<span class="operator">,</span>
/// &#32;   &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"main"</span><span class="operator">,</span> <span class="string">"Overwrite existing files."</span>)}<span class="operator">,</span>
/// &#32;   &#32;     <span class="comment">// An option with a value</span>
/// &#32;   &#32;     {{<span class="string">"t"</span><span class="operator">,</span> <span class="string">"target-directory"</span>}<span class="operator">,</span>
/// &#32;   &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"main"</span><span class="operator">,</span> <span class="string">"Copy all source files into &lt;directory&gt;."</span>)<span class="operator">,</span>
/// &#32;   &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"main"</span><span class="operator">,</span> <span class="string">"directory"</span>)}<span class="operator">,</span>
/// &#32;     });
///
/// </pre>
/// <p>Known limitation: the parsing of Qt options inside <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> and subclasses happens before <a href="http://doc.qt.io/qt-5/qcommandlineparser.html">QCommandLineParser</a> exists, so it can't take it into account. This means any option value that looks like a builtin Qt option, will be treated by <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> as a builtin Qt option. Example: <code>--profile -reverse</code> will lead to <a href="http://doc.qt.io/qt-5/qguiapplication.html">QGuiApplication</a> seeing the -reverse option set, and removing it from <a href="http://doc.qt.io/qt-5/qcoreapplication.html#arguments">QCoreApplication::arguments</a>() before <a href="http://doc.qt.io/qt-5/qcommandlineparser.html">QCommandLineParser</a> defines the <code>profile</code> option and parses the command line.</p>
/// <a name="how-to-use-qcommandlineparser-in-complex-applications"></a>
/// <h4>How to Use QCommandLineParser in Complex Applications</h4>
/// <p>In practice, additional error checking needs to be performed on the positional arguments and option values. For example, ranges of numbers should be checked.</p>
/// <p>It is then advisable to introduce a function to do the command line parsing which takes a struct or class receiving the option values returning an enumeration representing the result. The dnslookup example of the <a href="http://doc.qt.io/qt-5/qtnetwork-module.html">QtNetwork</a> module illustrates this:</p>
/// <pre class="cpp">
///
///   <span class="keyword">struct</span> DnsQuery
///   {
/// &#32;     DnsQuery() : type(<span class="type"><a href="http://doc.qt.io/qt-5/qdnslookup.html">QDnsLookup</a></span><span class="operator">::</span>A) {}
///
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qdnslookup.html">QDnsLookup</a></span><span class="operator">::</span>Type type;
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qhostaddress.html">QHostAddress</a></span> nameServer;
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> name;
///   };
///
///   <span class="keyword">enum</span> CommandLineParseResult
///   {
/// &#32;     CommandLineOk<span class="operator">,</span>
/// &#32;     CommandLineError<span class="operator">,</span>
/// &#32;     CommandLineVersionRequested<span class="operator">,</span>
/// &#32;     CommandLineHelpRequested
///   };
///
///   CommandLineParseResult parseCommandLine(<span class="type"><a href="http://doc.qt.io/qt-5/qcommandlineparser.html#QCommandLineParser">QCommandLineParser</a></span> <span class="operator">&amp;</span>parser<span class="operator">,</span> DnsQuery <span class="operator">*</span>query<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">*</span>errorMessage)
///   {
/// &#32;     parser<span class="operator">.</span>setSingleDashWordOptionMode(<span class="type"><a href="http://doc.qt.io/qt-5/qcommandlineparser.html#QCommandLineParser">QCommandLineParser</a></span><span class="operator">::</span>ParseAsLongOptions);
/// &#32;     <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcommandlineoption.html">QCommandLineOption</a></span> nameServerOption(<span class="string">"n"</span><span class="operator">,</span> <span class="string">"The name server to use."</span><span class="operator">,</span> <span class="string">"nameserver"</span>);
/// &#32;     parser<span class="operator">.</span>addOption(nameServerOption);
/// &#32;     <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcommandlineoption.html">QCommandLineOption</a></span> typeOption(<span class="string">"t"</span><span class="operator">,</span> <span class="string">"The lookup type."</span><span class="operator">,</span> <span class="string">"type"</span>);
/// &#32;     parser<span class="operator">.</span>addOption(typeOption);
/// &#32;     parser<span class="operator">.</span>addPositionalArgument(<span class="string">"name"</span><span class="operator">,</span> <span class="string">"The name to look up."</span>);
/// &#32;     <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcommandlineoption.html">QCommandLineOption</a></span> helpOption <span class="operator">=</span> parser<span class="operator">.</span>addHelpOption();
/// &#32;     <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcommandlineoption.html">QCommandLineOption</a></span> versionOption <span class="operator">=</span> parser<span class="operator">.</span>addVersionOption();
///
/// &#32;     <span class="keyword">if</span> (<span class="operator">!</span>parser<span class="operator">.</span>parse(<span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>arguments())) {
/// &#32;   &#32;     <span class="operator">*</span>errorMessage <span class="operator">=</span> parser<span class="operator">.</span>errorText();
/// &#32;   &#32;     <span class="keyword">return</span> CommandLineError;
/// &#32;     }
///
/// &#32;     <span class="keyword">if</span> (parser<span class="operator">.</span>isSet(versionOption))
/// &#32;   &#32;     <span class="keyword">return</span> CommandLineVersionRequested;
///
/// &#32;     <span class="keyword">if</span> (parser<span class="operator">.</span>isSet(helpOption))
/// &#32;   &#32;     <span class="keyword">return</span> CommandLineHelpRequested;
///
/// &#32;     <span class="keyword">if</span> (parser<span class="operator">.</span>isSet(nameServerOption)) {
/// &#32;   &#32;     <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> nameserver <span class="operator">=</span> parser<span class="operator">.</span>value(nameServerOption);
/// &#32;   &#32;     query<span class="operator">-</span><span class="operator">&gt;</span>nameServer <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qhostaddress.html">QHostAddress</a></span>(nameserver);
/// &#32;   &#32;     <span class="keyword">if</span> (query<span class="operator">-</span><span class="operator">&gt;</span>nameServer<span class="operator">.</span>isNull() <span class="operator">|</span><span class="operator">|</span> query<span class="operator">-</span><span class="operator">&gt;</span>nameServer<span class="operator">.</span>protocol() <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qabstractsocket.html">QAbstractSocket</a></span><span class="operator">::</span>UnknownNetworkLayerProtocol) {
/// &#32;   &#32;   &#32;     <span class="operator">*</span>errorMessage <span class="operator">=</span> <span class="string">"Bad nameserver address: "</span> <span class="operator">+</span> nameserver;
/// &#32;   &#32;   &#32;     <span class="keyword">return</span> CommandLineError;
/// &#32;   &#32;     }
/// &#32;     }
///
/// &#32;     <span class="keyword">if</span> (parser<span class="operator">.</span>isSet(typeOption)) {
/// &#32;   &#32;     <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> typeParameter <span class="operator">=</span> parser<span class="operator">.</span>value(typeOption);
/// &#32;   &#32;     <span class="keyword">const</span> <span class="type">int</span> type <span class="operator">=</span> typeFromParameter(typeParameter<span class="operator">.</span>toLower());
/// &#32;   &#32;     <span class="keyword">if</span> (type <span class="operator">&lt;</span> <span class="number">0</span>) {
/// &#32;   &#32;   &#32;     <span class="operator">*</span>errorMessage <span class="operator">=</span> <span class="string">"Bad record type: "</span> <span class="operator">+</span> typeParameter;
/// &#32;   &#32;   &#32;     <span class="keyword">return</span> CommandLineError;
/// &#32;   &#32;     }
/// &#32;   &#32;     query<span class="operator">-</span><span class="operator">&gt;</span>type <span class="operator">=</span> <span class="keyword">static_cast</span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qdnslookup.html">QDnsLookup</a></span><span class="operator">::</span>Type<span class="operator">&gt;</span>(type);
/// &#32;     }
///
/// &#32;     <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> positionalArguments <span class="operator">=</span> parser<span class="operator">.</span>positionalArguments();
/// &#32;     <span class="keyword">if</span> (positionalArguments<span class="operator">.</span>isEmpty()) {
/// &#32;   &#32;     <span class="operator">*</span>errorMessage <span class="operator">=</span> <span class="string">"Argument 'name' missing."</span>;
/// &#32;   &#32;     <span class="keyword">return</span> CommandLineError;
/// &#32;     }
/// &#32;     <span class="keyword">if</span> (positionalArguments<span class="operator">.</span>size() <span class="operator">&gt;</span> <span class="number">1</span>) {
/// &#32;   &#32;     <span class="operator">*</span>errorMessage <span class="operator">=</span> <span class="string">"Several 'name' arguments specified."</span>;
/// &#32;   &#32;     <span class="keyword">return</span> CommandLineError;
/// &#32;     }
/// &#32;     query<span class="operator">-</span><span class="operator">&gt;</span>name <span class="operator">=</span> positionalArguments<span class="operator">.</span>first();
///
/// &#32;     <span class="keyword">return</span> CommandLineOk;
///   }
///
/// </pre>
/// <p>In the main function, help should be printed to the standard output if the help option was passed and the application should return the exit code 0.</p>
/// <p>If an error was detected, the error message should be printed to the standard error output and the application should return an exit code other than 0.</p>
/// <pre class="cpp">
///
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>setApplicationVersion(QT_VERSION_STR);
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>setApplicationName(<span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"QDnsLookupExample"</span><span class="operator">,</span> <span class="string">"DNS Lookup Example"</span>));
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcommandlineparser.html#QCommandLineParser">QCommandLineParser</a></span> parser;
/// &#32;     parser<span class="operator">.</span>setApplicationDescription(<span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"QDnsLookupExample"</span><span class="operator">,</span> <span class="string">"An example demonstrating the class QDnsLookup."</span>));
/// &#32;     DnsQuery query;
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> errorMessage;
/// &#32;     <span class="keyword">switch</span> (parseCommandLine(parser<span class="operator">,</span> <span class="operator">&amp;</span>query<span class="operator">,</span> <span class="operator">&amp;</span>errorMessage)) {
/// &#32;     <span class="keyword">case</span> CommandLineOk:
/// &#32;   &#32;     <span class="keyword">break</span>;
/// &#32;     <span class="keyword">case</span> CommandLineError:
/// &#32;   &#32;     fputs(<a href="http://doc.qt.io/qt-5/qtglobal.html#qPrintable">qPrintable</a>(errorMessage)<span class="operator">,</span> stderr);
/// &#32;   &#32;     fputs(<span class="string">"\n\n"</span><span class="operator">,</span> stderr);
/// &#32;   &#32;     fputs(<a href="http://doc.qt.io/qt-5/qtglobal.html#qPrintable">qPrintable</a>(parser<span class="operator">.</span>helpText())<span class="operator">,</span> stderr);
/// &#32;   &#32;     <span class="keyword">return</span> <span class="number">1</span>;
/// &#32;     <span class="keyword">case</span> CommandLineVersionRequested:
/// &#32;   &#32;     printf(<span class="string">"%s %s\n"</span><span class="operator">,</span> <a href="http://doc.qt.io/qt-5/qtglobal.html#qPrintable">qPrintable</a>(<span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>applicationName())<span class="operator">,</span>
/// &#32;   &#32;   &#32;   &#32;    <a href="http://doc.qt.io/qt-5/qtglobal.html#qPrintable">qPrintable</a>(<span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>applicationVersion()));
/// &#32;   &#32;     <span class="keyword">return</span> <span class="number">0</span>;
/// &#32;     <span class="keyword">case</span> CommandLineHelpRequested:
/// &#32;   &#32;     parser<span class="operator">.</span>showHelp();
/// &#32;   &#32;     Q_UNREACHABLE();
/// &#32;     }
///
/// </pre>
/// <p>A special case to consider here are GUI applications on Windows and mobile platforms. These applications may not use the standard output or error channels since the output is either discarded or not accessible.</p>
/// <p>On Windows, <a href="http://doc.qt.io/qt-5/qcommandlineparser.html">QCommandLineParser</a> uses message boxes to display usage information and errors if no console window can be obtained.</p>
/// <p>For other platforms, it is recommended to display help texts and error messages using a <a href="http://doc.qt.io/qt-5/qmessagebox.html">QMessageBox</a>. To preserve the formatting of the help text, rich text with <code>&lt;pre&gt;</code> elements should be used:</p>
/// <pre class="cpp">
///
///   <span class="keyword">switch</span> (parseCommandLine(parser<span class="operator">,</span> <span class="operator">&amp;</span>query<span class="operator">,</span> <span class="operator">&amp;</span>errorMessage)) {
///   <span class="keyword">case</span> CommandLineOk:
/// &#32;     <span class="keyword">break</span>;
///   <span class="keyword">case</span> CommandLineError:
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qmessagebox.html">QMessageBox</a></span><span class="operator">::</span>warning(<span class="number">0</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qguiapplication.html">QGuiApplication</a></span><span class="operator">::</span>applicationDisplayName()<span class="operator">,</span>
/// &#32;   &#32;   &#32;   &#32;   &#32;   &#32;      <span class="string">"&lt;html&gt;&lt;head/&gt;&lt;body&gt;&lt;h2&gt;"</span> <span class="operator">+</span> errorMessage <span class="operator">+</span> <span class="string">"&lt;/h2&gt;&lt;pre&gt;"</span>
/// &#32;   &#32;   &#32;   &#32;   &#32;   &#32;      <span class="operator">+</span> parser<span class="operator">.</span>helpText() <span class="operator">+</span> <span class="string">"&lt;/pre&gt;&lt;/body&gt;&lt;/html&gt;"</span>);
/// &#32;     <span class="keyword">return</span> <span class="number">1</span>;
///   <span class="keyword">case</span> CommandLineVersionRequested:
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qmessagebox.html">QMessageBox</a></span><span class="operator">::</span>information(<span class="number">0</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qguiapplication.html">QGuiApplication</a></span><span class="operator">::</span>applicationDisplayName()<span class="operator">,</span>
/// &#32;   &#32;   &#32;   &#32;   &#32;   &#32;   &#32;      <span class="type"><a href="http://doc.qt.io/qt-5/qguiapplication.html">QGuiApplication</a></span><span class="operator">::</span>applicationDisplayName() <span class="operator">+</span> <span class="char">' '</span>
/// &#32;   &#32;   &#32;   &#32;   &#32;   &#32;   &#32;      <span class="operator">+</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>applicationVersion());
/// &#32;     <span class="keyword">return</span> <span class="number">0</span>;
///   <span class="keyword">case</span> CommandLineHelpRequested:
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qmessagebox.html">QMessageBox</a></span><span class="operator">::</span>warning(<span class="number">0</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qguiapplication.html">QGuiApplication</a></span><span class="operator">::</span>applicationDisplayName()<span class="operator">,</span>
/// &#32;   &#32;   &#32;   &#32;   &#32;   &#32;      <span class="string">"&lt;html&gt;&lt;head/&gt;&lt;body&gt;&lt;pre&gt;"</span>
/// &#32;   &#32;   &#32;   &#32;   &#32;   &#32;      <span class="operator">+</span> parser<span class="operator">.</span>helpText() <span class="operator">+</span> <span class="string">"&lt;/pre&gt;&lt;/body&gt;&lt;/html&gt;"</span>);
/// &#32;     <span class="keyword">return</span> <span class="number">0</span>;
///   }
///
/// </pre>
/// <p>However, this does not apply to the dnslookup example, because it is a console application.</p></div>
#[repr(C)]
pub struct QCommandLineParser {
    _unused: u8,
}
impl QCommandLineParser {
    /// <p>Adds the help option (<code>-h</code>, <code>--help</code> and <code>-?</code> on Windows) This option is handled automatically by <a href="http://doc.qt.io/qt-5/qcommandlineparser.html">QCommandLineParser</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCommandLineOption QCommandLineParser::addHelpOption()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#addHelpOption">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds the help option (<code>-h</code>, <code>--help</code> and <code>-?</code> on Windows) This option is handled automatically by <a href="http://doc.qt.io/qt-5/qcommandlineparser.html">QCommandLineParser</a>.</p>
    /// <p>Remember to use <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#setApplicationDescription">setApplicationDescription</a> to set the application description, which will be displayed when this option is used.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">int</span> main(<span class="type">int</span> argc<span class="operator">,</span> <span class="type">char</span> <span class="operator">*</span>argv<span class="operator">[</span><span class="operator">]</span>)
    ///   {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span> app(argc<span class="operator">,</span> argv);
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>setApplicationName(<span class="string">"my-copy-program"</span>);
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>setApplicationVersion(<span class="string">"1.0"</span>);
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcommandlineparser.html#QCommandLineParser">QCommandLineParser</a></span> parser;
    /// &#32;     parser<span class="operator">.</span>setApplicationDescription(<span class="string">"Test helper"</span>);
    /// &#32;     parser<span class="operator">.</span>addHelpOption();
    /// &#32;     parser<span class="operator">.</span>addVersionOption();
    /// &#32;     parser<span class="operator">.</span>addPositionalArgument(<span class="string">"source"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"main"</span><span class="operator">,</span> <span class="string">"Source file to copy."</span>));
    /// &#32;     parser<span class="operator">.</span>addPositionalArgument(<span class="string">"destination"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"main"</span><span class="operator">,</span> <span class="string">"Destination directory."</span>));
    ///
    /// &#32;     <span class="comment">// A boolean option with a single name (-p)</span>
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcommandlineoption.html">QCommandLineOption</a></span> showProgressOption(<span class="string">"p"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"main"</span><span class="operator">,</span> <span class="string">"Show progress during copy"</span>));
    /// &#32;     parser<span class="operator">.</span>addOption(showProgressOption);
    ///
    /// &#32;     <span class="comment">// A boolean option with multiple names (-f, --force)</span>
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcommandlineoption.html">QCommandLineOption</a></span> forceOption(<span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"f"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"force"</span><span class="operator">,</span>
    /// &#32;   &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"main"</span><span class="operator">,</span> <span class="string">"Overwrite existing files."</span>));
    /// &#32;     parser<span class="operator">.</span>addOption(forceOption);
    ///
    /// &#32;     <span class="comment">// An option with a value</span>
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcommandlineoption.html">QCommandLineOption</a></span> targetDirectoryOption(<span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"t"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"target-directory"</span><span class="operator">,</span>
    /// &#32;   &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"main"</span><span class="operator">,</span> <span class="string">"Copy all source files into &lt;directory&gt;."</span>)<span class="operator">,</span>
    /// &#32;   &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"main"</span><span class="operator">,</span> <span class="string">"directory"</span>));
    /// &#32;     parser<span class="operator">.</span>addOption(targetDirectoryOption);
    ///
    /// &#32;     <span class="comment">// Process the actual command line arguments given by the user</span>
    /// &#32;     parser<span class="operator">.</span>process(app);
    ///
    /// &#32;     <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> args <span class="operator">=</span> parser<span class="operator">.</span>positionalArguments();
    /// &#32;     <span class="comment">// source is args.at(0), destination is args.at(1)</span>
    ///
    /// &#32;     bool showProgress <span class="operator">=</span> parser<span class="operator">.</span>isSet(showProgressOption);
    /// &#32;     bool force <span class="operator">=</span> parser<span class="operator">.</span>isSet(forceOption);
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> targetDir <span class="operator">=</span> parser<span class="operator">.</span>value(targetDirectoryOption);
    /// &#32;     <span class="comment">// ...</span>
    ///   }
    ///
    /// </pre>
    /// <p>Returns the option instance, which can be used to call <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#isSet">isSet</a>().</p></div>
    pub unsafe fn add_help_option(&mut self) -> ::cpp_utils::CppBox<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_addHelpOption(
            self as *mut crate::QCommandLineParser,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Adds the option <i>option</i> to look for while parsing.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCommandLineParser::addOption(const QCommandLineOption& commandLineOption)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#addOption">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds the option <i>option</i> to look for while parsing.</p>
    /// <p>Returns <code>true</code> if adding the option was successful; otherwise returns <code>false</code>.</p>
    /// <p>Adding the option fails if there is no name attached to the option, or the option has a name that clashes with an option name added before.</p></div>
    pub unsafe fn add_option(
        &mut self,
        command_line_option: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCommandLineOption>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_addOption(
            self as *mut crate::QCommandLineParser,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCommandLineOption>>::cast_into(
                command_line_option,
            )
            .as_raw_ptr(),
        )
    }

    /// <p>Adds the options to look for while parsing. The options are specified by the parameter <i>options</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCommandLineParser::addOptions(const QList<QCommandLineOption>& options)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#addOptions">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds the options to look for while parsing. The options are specified by the parameter <i>options</i>.</p>
    /// <p>Returns <code>true</code> if adding all of the options was successful; otherwise returns <code>false</code>.</p>
    /// <p>See the documentation for <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#addOption">addOption</a>() for when this function may fail.</p>
    /// <p>This function was introduced in  Qt 5.4.</p></div>
    pub unsafe fn add_options(
        &mut self,
        options: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQCommandLineOption>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_addOptions(
            self as *mut crate::QCommandLineParser,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQCommandLineOption>>::cast_into(
                options,
            )
            .as_raw_ptr(),
        )
    }

    /// <p>Defines an additional argument to the application, for the benefit of the help text.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCommandLineParser::addPositionalArgument(const QString& name, const QString& description, const QString& syntax = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#addPositionalArgument">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Defines an additional argument to the application, for the benefit of the help text.</p>
    /// <p>The argument <i>name</i> and <i>description</i> will appear under the <code>Arguments:</code> section of the help. If <i>syntax</i> is specified, it will be appended to the Usage line, otherwise the <i>name</i> will be appended.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="comment">// Usage: image-editor file</span>
    ///   <span class="comment">//</span>
    ///   <span class="comment">// Arguments:</span>
    ///   <span class="comment">//   file                  The file to open.</span>
    ///   parser<span class="operator">.</span>addPositionalArgument(<span class="string">"file"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"main"</span><span class="operator">,</span> <span class="string">"The file to open."</span>));
    ///
    ///   <span class="comment">// Usage: web-browser [urls...]</span>
    ///   <span class="comment">//</span>
    ///   <span class="comment">// Arguments:</span>
    ///   <span class="comment">//   urls                URLs to open, optionally.</span>
    ///   parser<span class="operator">.</span>addPositionalArgument(<span class="string">"urls"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"main"</span><span class="operator">,</span> <span class="string">"URLs to open, optionally."</span>)<span class="operator">,</span> <span class="string">"[urls...]"</span>);
    ///
    ///   <span class="comment">// Usage: cp source destination</span>
    ///   <span class="comment">//</span>
    ///   <span class="comment">// Arguments:</span>
    ///   <span class="comment">//   source                Source file to copy.</span>
    ///   <span class="comment">//   destination           Destination directory.</span>
    ///   parser<span class="operator">.</span>addPositionalArgument(<span class="string">"source"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"main"</span><span class="operator">,</span> <span class="string">"Source file to copy."</span>));
    ///   parser<span class="operator">.</span>addPositionalArgument(<span class="string">"destination"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"main"</span><span class="operator">,</span> <span class="string">"Destination directory."</span>));
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineparser.html#addHelpOption">addHelpOption</a>() and <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#helpText">helpText</a>().</p></div>
    pub unsafe fn add_positional_argument_3a(
        &mut self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        description: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        syntax: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_addPositionalArgument(
            self as *mut crate::QCommandLineParser,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(description)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(syntax)
                .as_raw_ptr(),
        )
    }

    /// <p>Defines an additional argument to the application, for the benefit of the help text.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCommandLineParser::addPositionalArgument(const QString& name, const QString& description)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#addPositionalArgument">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Defines an additional argument to the application, for the benefit of the help text.</p>
    /// <p>The argument <i>name</i> and <i>description</i> will appear under the <code>Arguments:</code> section of the help. If <i>syntax</i> is specified, it will be appended to the Usage line, otherwise the <i>name</i> will be appended.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="comment">// Usage: image-editor file</span>
    ///   <span class="comment">//</span>
    ///   <span class="comment">// Arguments:</span>
    ///   <span class="comment">//   file                  The file to open.</span>
    ///   parser<span class="operator">.</span>addPositionalArgument(<span class="string">"file"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"main"</span><span class="operator">,</span> <span class="string">"The file to open."</span>));
    ///
    ///   <span class="comment">// Usage: web-browser [urls...]</span>
    ///   <span class="comment">//</span>
    ///   <span class="comment">// Arguments:</span>
    ///   <span class="comment">//   urls                URLs to open, optionally.</span>
    ///   parser<span class="operator">.</span>addPositionalArgument(<span class="string">"urls"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"main"</span><span class="operator">,</span> <span class="string">"URLs to open, optionally."</span>)<span class="operator">,</span> <span class="string">"[urls...]"</span>);
    ///
    ///   <span class="comment">// Usage: cp source destination</span>
    ///   <span class="comment">//</span>
    ///   <span class="comment">// Arguments:</span>
    ///   <span class="comment">//   source                Source file to copy.</span>
    ///   <span class="comment">//   destination           Destination directory.</span>
    ///   parser<span class="operator">.</span>addPositionalArgument(<span class="string">"source"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"main"</span><span class="operator">,</span> <span class="string">"Source file to copy."</span>));
    ///   parser<span class="operator">.</span>addPositionalArgument(<span class="string">"destination"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"main"</span><span class="operator">,</span> <span class="string">"Destination directory."</span>));
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineparser.html#addHelpOption">addHelpOption</a>() and <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#helpText">helpText</a>().</p></div>
    pub unsafe fn add_positional_argument_2a(
        &mut self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        description: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_addPositionalArgument1(
            self as *mut crate::QCommandLineParser,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(description)
                .as_raw_ptr(),
        )
    }

    /// <p>Adds the <code>-v</code> / <code>--version</code> option, which displays the version string of the application.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCommandLineOption QCommandLineParser::addVersionOption()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#addVersionOption">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds the <code>-v</code> / <code>--version</code> option, which displays the version string of the application.</p>
    /// <p>This option is handled automatically by <a href="http://doc.qt.io/qt-5/qcommandlineparser.html">QCommandLineParser</a>.</p>
    /// <p>You can set the actual version string by using <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationVersion-prop">QCoreApplication::setApplicationVersion</a>().</p>
    /// <p>Returns the option instance, which can be used to call <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#isSet">isSet</a>().</p></div>
    pub unsafe fn add_version_option(&mut self) -> ::cpp_utils::CppBox<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_addVersionOption(
            self as *mut crate::QCommandLineParser,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the application description set in <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#setApplicationDescription">setApplicationDescription</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QCommandLineParser::applicationDescription() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#applicationDescription">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the application description set in <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#setApplicationDescription">setApplicationDescription</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineparser.html#setApplicationDescription">setApplicationDescription</a>().</p></div>
    pub unsafe fn application_description(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_applicationDescription(
            self as *const crate::QCommandLineParser,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Clears the definitions of additional arguments from the help text.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCommandLineParser::clearPositionalArguments()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#clearPositionalArguments">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Clears the definitions of additional arguments from the help text.</p>
    /// <p>This is only needed for the special case of tools which support multiple commands with different options. Once the actual command has been identified, the options for this command can be defined, and the help text for the command can be adjusted accordingly.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span> app(argc<span class="operator">,</span> argv);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qcommandlineparser.html#QCommandLineParser">QCommandLineParser</a></span> parser;
    ///
    ///   parser<span class="operator">.</span>addPositionalArgument(<span class="string">"command"</span><span class="operator">,</span> <span class="string">"The command to execute."</span>);
    ///
    ///   <span class="comment">// Call parse() to find out the positional arguments.</span>
    ///   parser<span class="operator">.</span>parse(<span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>arguments());
    ///
    ///   <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> args <span class="operator">=</span> parser<span class="operator">.</span>positionalArguments();
    ///   <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> command <span class="operator">=</span> args<span class="operator">.</span>isEmpty() <span class="operator">?</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>() : args<span class="operator">.</span>first();
    ///   <span class="keyword">if</span> (command <span class="operator">=</span><span class="operator">=</span> <span class="string">"resize"</span>) {
    /// &#32;     parser<span class="operator">.</span>clearPositionalArguments();
    /// &#32;     parser<span class="operator">.</span>addPositionalArgument(<span class="string">"resize"</span><span class="operator">,</span> <span class="string">"Resize the object to a new size."</span><span class="operator">,</span> <span class="string">"resize [resize_options]"</span>);
    /// &#32;     parser<span class="operator">.</span>addOption(<span class="type"><a href="http://doc.qt.io/qt-5/qcommandlineoption.html">QCommandLineOption</a></span>(<span class="string">"size"</span><span class="operator">,</span> <span class="string">"New size."</span><span class="operator">,</span> <span class="string">"new_size"</span>));
    /// &#32;     parser<span class="operator">.</span>process(app);
    /// &#32;     <span class="comment">// ...</span>
    ///   }
    ///
    ///   <span class="comment">/*
    ///   This code results in context-dependent help:
    ///
    ///   $ tool --help
    ///   Usage: tool command
    ///
    ///   Arguments:
    /// &#32;   command  The command to execute.
    ///
    ///   $ tool resize --help
    ///   Usage: tool resize [resize_options]
    ///
    ///   Options:
    /// &#32;   --size &lt;size&gt;  New size.
    ///
    ///   Arguments:
    /// &#32;   resize&#32;   &#32;    Resize the object to a new size.
    ///   */</span>
    ///
    /// </pre></div>
    pub unsafe fn clear_positional_arguments(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_clearPositionalArguments(
            self as *mut crate::QCommandLineParser,
        )
    }

    /// <p>Returns a translated error text for the user. This should only be called when <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#parse">parse</a>() returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QCommandLineParser::errorText() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#errorText">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a translated error text for the user. This should only be called when <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#parse">parse</a>() returns <code>false</code>.</p></div>
    pub unsafe fn error_text(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_errorText(
            self as *const crate::QCommandLineParser,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a string containing the complete help information.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QCommandLineParser::helpText() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#helpText">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string containing the complete help information.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineparser.html#showHelp">showHelp</a>().</p></div>
    pub unsafe fn help_text(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_helpText(
            self as *const crate::QCommandLineParser,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Checks whether the option <i>name</i> was passed to the application.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCommandLineParser::isSet(const QString& name) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#isSet">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Checks whether the option <i>name</i> was passed to the application.</p>
    /// <p>Returns <code>true</code> if the option <i>name</i> was set, false otherwise.</p>
    /// <p>The name provided can be any long or short name of any option that was added with <code>addOption()</code>. All the options names are treated as being equivalent. If the name is not recognized or that option was not present, false is returned.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   bool verbose <span class="operator">=</span> parser<span class="operator">.</span>isSet(<span class="string">"verbose"</span>);
    ///
    /// </pre></div>
    pub unsafe fn is_set_q_string(
        &self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_isSet(
            self as *const crate::QCommandLineParser,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCommandLineParser::isSet(const QCommandLineOption& option) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#isSet-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Checks whether the <i>option</i> was passed to the application.</p>
    /// <p>Returns <code>true</code> if the <i>option</i> was set, false otherwise.</p>
    /// <p>This is the recommended way to check for options with no values.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span> app(argc<span class="operator">,</span> argv);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qcommandlineparser.html#QCommandLineParser">QCommandLineParser</a></span> parser;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qcommandlineoption.html">QCommandLineOption</a></span> verboseOption(<span class="string">"verbose"</span>);
    ///   parser<span class="operator">.</span>addOption(verboseOption);
    ///   parser<span class="operator">.</span>process(app);
    ///   bool verbose <span class="operator">=</span> parser<span class="operator">.</span>isSet(verboseOption);
    ///
    /// </pre></div>
    pub unsafe fn is_set_q_command_line_option(
        &self,
        option: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCommandLineOption>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_isSet1(
            self as *const crate::QCommandLineParser,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCommandLineOption>>::cast_into(option)
                .as_raw_ptr(),
        )
    }

    /// <p>Constructs a command line parser object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCommandLineParser::QCommandLineParser()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#QCommandLineParser">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a command line parser object.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QCommandLineParser> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_QCommandLineParser();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list of option names that were found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QCommandLineParser::optionNames() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#optionNames">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of option names that were found.</p>
    /// <p>This returns a list of all the recognized option names found by the parser, in the order in which they were found. For any long options that were in the form {--option=value}, the value part will have been dropped.</p>
    /// <p>The names in this list do not include the preceding dash characters. Names may appear more than once in this list if they were encountered more than once by the parser.</p>
    /// <p>Any entry in the list can be used with <code>value()</code> or with <code>values()</code> to get any relevant option values.</p></div>
    pub unsafe fn option_names(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_optionNames(
            self as *const crate::QCommandLineParser,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Parses the command line <i>arguments</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCommandLineParser::parse(const QStringList& arguments)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#parse">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Parses the command line <i>arguments</i>.</p>
    /// <p>Most programs don't need to call this, a simple call to <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#process">process</a>() is enough.</p>
    /// <p>parse() is more low-level, and only does the parsing. The application will have to take care of the error handling, using <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#errorText">errorText</a>() if parse() returns <code>false</code>. This can be useful for instance to show a graphical error message in graphical programs.</p>
    /// <p>Calling parse() instead of <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#process">process</a>() can also be useful in order to ignore unknown options temporarily, because more option definitions will be provided later on (depending on one of the arguments), before calling <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#process">process</a>().</p>
    /// <p>Don't forget that <i>arguments</i> must start with the name of the executable (ignored, though).</p>
    /// <p>Returns <code>false</code> in case of a parse error (unknown option or missing value); returns <code>true</code> otherwise.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineparser.html#process">process</a>().</p></div>
    pub unsafe fn parse(
        &mut self,
        arguments: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_parse(
            self as *mut crate::QCommandLineParser,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(arguments)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a list of positional arguments.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QCommandLineParser::positionalArguments() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#positionalArguments">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of positional arguments.</p>
    /// <p>These are all of the arguments that were not recognized as part of an option.</p></div>
    pub unsafe fn positional_arguments(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_positionalArguments(
            self as *const crate::QCommandLineParser,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Processes the command line <i>arguments</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCommandLineParser::process(const QStringList& arguments)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#process">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Processes the command line <i>arguments</i>.</p>
    /// <p>In addition to parsing the options (like <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#parse">parse</a>()), this function also handles the builtin options and handles errors.</p>
    /// <p>The builtin options are <code>--version</code> if <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#addVersionOption">addVersionOption</a> was called and <code>--help</code> if <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#addHelpOption">addHelpOption</a> was called.</p>
    /// <p>When invoking one of these options, or when an error happens (for instance an unknown option was passed), the current process will then stop, using the exit() function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#arguments">QCoreApplication::arguments</a>() and <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#parse">parse</a>().</p></div>
    pub unsafe fn process_q_string_list(
        &mut self,
        arguments: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_process(
            self as *mut crate::QCommandLineParser,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(arguments)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCommandLineParser::process(const QCoreApplication& app)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#process-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>The command line is obtained from the <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> instance <i>app</i>.</p></div>
    pub unsafe fn process_q_core_application(
        &mut self,
        app: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCoreApplication>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_process1(
            self as *mut crate::QCommandLineParser,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCoreApplication>>::cast_into(app)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the application <i>description</i> shown by <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#helpText">helpText</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCommandLineParser::setApplicationDescription(const QString& description)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#setApplicationDescription">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the application <i>description</i> shown by <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#helpText">helpText</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineparser.html#applicationDescription">applicationDescription</a>().</p></div>
    pub unsafe fn set_application_description(
        &mut self,
        description: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_setApplicationDescription(
            self as *mut crate::QCommandLineParser,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(description)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the parsing mode to <i>parsingMode</i>. This must be called before <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#process">process</a>() or <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#parse">parse</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCommandLineParser::setOptionsAfterPositionalArgumentsMode(QCommandLineParser::OptionsAfterPositionalArgumentsMode mode)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#setOptionsAfterPositionalArgumentsMode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the parsing mode to <i>parsingMode</i>. This must be called before <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#process">process</a>() or <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#parse">parse</a>().</p>
    /// <p>This function was introduced in  Qt 5.6.</p></div>
    pub unsafe fn set_options_after_positional_arguments_mode(
        &mut self,
        mode: crate::q_command_line_parser::OptionsAfterPositionalArgumentsMode,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_setOptionsAfterPositionalArgumentsMode(
            self as *mut crate::QCommandLineParser,
            mode,
        )
    }

    /// <p>Sets the parsing mode to <i>singleDashWordOptionMode</i>. This must be called before <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#process">process</a>() or <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#parse">parse</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCommandLineParser::setSingleDashWordOptionMode(QCommandLineParser::SingleDashWordOptionMode parsingMode)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#setSingleDashWordOptionMode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the parsing mode to <i>singleDashWordOptionMode</i>. This must be called before <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#process">process</a>() or <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#parse">parse</a>().</p></div>
    pub unsafe fn set_single_dash_word_option_mode(
        &mut self,
        parsing_mode: crate::q_command_line_parser::SingleDashWordOptionMode,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_setSingleDashWordOptionMode(
            self as *mut crate::QCommandLineParser,
            parsing_mode,
        )
    }

    /// <p>Displays the help information, and exits the application. This is automatically triggered by the --help option, but can also be used to display the help when the user is not invoking the application correctly. The exit code is set to <i>exitCode</i>. It should be set to 0 if the user requested to see the help, and to any other value in case of an error.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCommandLineParser::showHelp(int exitCode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#showHelp">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Displays the help information, and exits the application. This is automatically triggered by the --help option, but can also be used to display the help when the user is not invoking the application correctly. The exit code is set to <i>exitCode</i>. It should be set to 0 if the user requested to see the help, and to any other value in case of an error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineparser.html#helpText">helpText</a>().</p></div>
    pub unsafe fn show_help_1a(&mut self, exit_code: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_showHelp(
            self as *mut crate::QCommandLineParser,
            exit_code,
        )
    }

    /// <p>Displays the help information, and exits the application. This is automatically triggered by the --help option, but can also be used to display the help when the user is not invoking the application correctly. The exit code is set to <i>exitCode</i>. It should be set to 0 if the user requested to see the help, and to any other value in case of an error.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCommandLineParser::showHelp()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#showHelp">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Displays the help information, and exits the application. This is automatically triggered by the --help option, but can also be used to display the help when the user is not invoking the application correctly. The exit code is set to <i>exitCode</i>. It should be set to 0 if the user requested to see the help, and to any other value in case of an error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineparser.html#helpText">helpText</a>().</p></div>
    pub unsafe fn show_help_0a(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_showHelp1(
            self as *mut crate::QCommandLineParser,
        )
    }

    /// <p>Displays the version information from <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationVersion-prop">QCoreApplication::applicationVersion</a>(), and exits the application. This is automatically triggered by the --version option, but can also be used to display the version when not using <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#process">process</a>(). The exit code is set to EXIT_SUCCESS (0).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCommandLineParser::showVersion()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#showVersion">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Displays the version information from <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationVersion-prop">QCoreApplication::applicationVersion</a>(), and exits the application. This is automatically triggered by the --version option, but can also be used to display the version when not using <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#process">process</a>(). The exit code is set to EXIT_SUCCESS (0).</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineparser.html#addVersionOption">addVersionOption</a>().</p></div>
    pub unsafe fn show_version(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_showVersion(
            self as *mut crate::QCommandLineParser,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QCommandLineParser::tr(const char* sourceText, const char* disambiguation, int n)```</span>.
    pub unsafe fn tr(
        source_text: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        disambiguation: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(
                source_text,
            )
            .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(
                disambiguation,
            )
            .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QCommandLineParser::trUtf8(const char* sourceText, const char* disambiguation, int n)```</span>.
    pub unsafe fn tr_utf8(
        source_text: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        disambiguation: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(
                source_text,
            )
            .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(
                disambiguation,
            )
            .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list of unknown option names.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QCommandLineParser::unknownOptionNames() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#unknownOptionNames">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of unknown option names.</p>
    /// <p>This list will include both long an short name options that were not recognized. For any long options that were in the form {--option=value}, the value part will have been dropped and only the long name is added.</p>
    /// <p>The names in this list do not include the preceding dash characters. Names may appear more than once in this list if they were encountered more than once by the parser.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineparser.html#optionNames">optionNames</a>().</p></div>
    pub unsafe fn unknown_option_names(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_unknownOptionNames(
            self as *const crate::QCommandLineParser,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the option value found for the given option name <i>optionName</i>, or an empty string if not found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QCommandLineParser::value(const QString& name) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the option value found for the given option name <i>optionName</i>, or an empty string if not found.</p>
    /// <p>The name provided can be any long or short name of any option that was added with <code>addOption()</code>. All the option names are treated as being equivalent. If the name is not recognized or that option was not present, an empty string is returned.</p>
    /// <p>For options found by the parser, the last value found for that option is returned. If the option wasn't specified on the command line, the default value is returned.</p>
    /// <p>An empty string is returned if the option does not take a value.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineparser.html#values">values</a>(), <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDefaultValue">QCommandLineOption::setDefaultValue</a>(), and <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDefaultValues">QCommandLineOption::setDefaultValues</a>().</p></div>
    pub unsafe fn value_q_string(
        &self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_value(
            self as *const crate::QCommandLineParser,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QCommandLineParser::value(const QCommandLineOption& option) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the option value found for the given <i>option</i>, or an empty string if not found.</p>
    /// <p>For options found by the parser, the last value found for that option is returned. If the option wasn't specified on the command line, the default value is returned.</p>
    /// <p>An empty string is returned if the option does not take a value.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineparser.html#values">values</a>(), <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDefaultValue">QCommandLineOption::setDefaultValue</a>(), and <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDefaultValues">QCommandLineOption::setDefaultValues</a>().</p></div>
    pub unsafe fn value_q_command_line_option(
        &self,
        option: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCommandLineOption>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_value1(
            self as *const crate::QCommandLineParser,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCommandLineOption>>::cast_into(option)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list of option values found for the given option name <i>optionName</i>, or an empty list if not found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QCommandLineParser::values(const QString& name) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#values">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of option values found for the given option name <i>optionName</i>, or an empty list if not found.</p>
    /// <p>The name provided can be any long or short name of any option that was added with <code>addOption()</code>. All the options names are treated as being equivalent. If the name is not recognized or that option was not present, an empty list is returned.</p>
    /// <p>For options found by the parser, the list will contain an entry for each time the option was encountered by the parser. If the option wasn't specified on the command line, the default values are returned.</p>
    /// <p>An empty list is returned if the option does not take a value.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineparser.html#value">value</a>(), <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDefaultValue">QCommandLineOption::setDefaultValue</a>(), and <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDefaultValues">QCommandLineOption::setDefaultValues</a>().</p></div>
    pub unsafe fn values_q_string(
        &self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_values(
            self as *const crate::QCommandLineParser,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QCommandLineParser::values(const QCommandLineOption& option) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#values-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a list of option values found for the given <i>option</i>, or an empty list if not found.</p>
    /// <p>For options found by the parser, the list will contain an entry for each time the option was encountered by the parser. If the option wasn't specified on the command line, the default values are returned.</p>
    /// <p>An empty list is returned if the option does not take a value.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcommandlineparser.html#value">value</a>(), <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDefaultValue">QCommandLineOption::setDefaultValue</a>(), and <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#setDefaultValues">QCommandLineOption::setDefaultValues</a>().</p></div>
    pub unsafe fn values_q_command_line_option(
        &self,
        option: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCommandLineOption>>,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_values1(
            self as *const crate::QCommandLineParser,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCommandLineOption>>::cast_into(option)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_cryptographic_hash {
    //! C++ type: <span style='color: green;'>```QCryptographicHash```</span>

    /// <p><b>Note: </b>In Qt versions before 5.9, when asked to generate a SHA3 hash sum, <a href="http://doc.qt.io/qt-5/qcryptographichash.html">QCryptographicHash</a> actually calculated Keccak. If you need compatibility with SHA-3 hashes produced by those versions of Qt, use the <code>Keccak_</code> enumerators. Alternatively, if source compatibility is required, define the macro <code>QT_SHA3_KECCAK_COMPAT</code>.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QCryptographicHash::Algorithm```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcryptographichash.html#Algorithm-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p><b>Note: </b>In Qt versions before 5.9, when asked to generate a SHA3 hash sum, <a href="http://doc.qt.io/qt-5/qcryptographichash.html">QCryptographicHash</a> actually calculated Keccak. If you need compatibility with SHA-3 hashes produced by those versions of Qt, use the <code>Keccak_</code> enumerators. Alternatively, if source compatibility is required, define the macro <code>QT_SHA3_KECCAK_COMPAT</code>.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Algorithm(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Algorithm {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Algorithm(value)
        }
    }

    impl From<Algorithm> for ::std::os::raw::c_int {
        fn from(value: Algorithm) -> Self {
            value.0
        }
    }

    impl Algorithm {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Algorithm {
        /// Generate an MD4 hash sum (C++ enum variant: <span style='color: green;'>```Md4 = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const Md4: crate::q_cryptographic_hash::Algorithm =
            crate::q_cryptographic_hash::Algorithm(0);
        /// Generate an MD5 hash sum (C++ enum variant: <span style='color: green;'>```Md5 = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Md5: crate::q_cryptographic_hash::Algorithm =
            crate::q_cryptographic_hash::Algorithm(1);
        /// Generate an SHA-1 hash sum (C++ enum variant: <span style='color: green;'>```Sha1 = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const Sha1: crate::q_cryptographic_hash::Algorithm =
            crate::q_cryptographic_hash::Algorithm(2);
        /// Generate an SHA-224 hash sum (SHA-2). Introduced in Qt 5.0 (C++ enum variant: <span style='color: green;'>```Sha224 = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const Sha224: crate::q_cryptographic_hash::Algorithm =
            crate::q_cryptographic_hash::Algorithm(3);
        /// Generate an SHA-256 hash sum (SHA-2). Introduced in Qt 5.0 (C++ enum variant: <span style='color: green;'>```Sha256 = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const Sha256: crate::q_cryptographic_hash::Algorithm =
            crate::q_cryptographic_hash::Algorithm(4);
        /// Generate an SHA-384 hash sum (SHA-2). Introduced in Qt 5.0 (C++ enum variant: <span style='color: green;'>```Sha384 = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const Sha384: crate::q_cryptographic_hash::Algorithm =
            crate::q_cryptographic_hash::Algorithm(5);
        /// Generate an SHA-512 hash sum (SHA-2). Introduced in Qt 5.0 (C++ enum variant: <span style='color: green;'>```Sha512 = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const Sha512: crate::q_cryptographic_hash::Algorithm =
            crate::q_cryptographic_hash::Algorithm(6);
        /// Generate a Keccak-224 hash sum. Introduced in Qt 5.9.2 (C++ enum variant: <span style='color: green;'>```Keccak_224 = 7```</span>)
        #[allow(non_upper_case_globals)]
        pub const Keccak224: crate::q_cryptographic_hash::Algorithm =
            crate::q_cryptographic_hash::Algorithm(7);
        /// Generate a Keccak-256 hash sum. Introduced in Qt 5.9.2 (C++ enum variant: <span style='color: green;'>```Keccak_256 = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const Keccak256: crate::q_cryptographic_hash::Algorithm =
            crate::q_cryptographic_hash::Algorithm(8);
        /// Generate a Keccak-384 hash sum. Introduced in Qt 5.9.2 (C++ enum variant: <span style='color: green;'>```Keccak_384 = 9```</span>)
        #[allow(non_upper_case_globals)]
        pub const Keccak384: crate::q_cryptographic_hash::Algorithm =
            crate::q_cryptographic_hash::Algorithm(9);
        /// Generate a Keccak-512 hash sum. Introduced in Qt 5.9.2 (C++ enum variant: <span style='color: green;'>```Keccak_512 = 10```</span>)
        #[allow(non_upper_case_globals)]
        pub const Keccak512: crate::q_cryptographic_hash::Algorithm =
            crate::q_cryptographic_hash::Algorithm(10);
        /// C++ enum variant: <span style='color: green;'>```RealSha3_224 = 11```</span>
        #[allow(non_upper_case_globals)]
        pub const RealSha3224: crate::q_cryptographic_hash::Algorithm =
            crate::q_cryptographic_hash::Algorithm(11);
        /// C++ enum variant: <span style='color: green;'>```RealSha3_256 = 12```</span>
        #[allow(non_upper_case_globals)]
        pub const RealSha3256: crate::q_cryptographic_hash::Algorithm =
            crate::q_cryptographic_hash::Algorithm(12);
        /// C++ enum variant: <span style='color: green;'>```RealSha3_384 = 13```</span>
        #[allow(non_upper_case_globals)]
        pub const RealSha3384: crate::q_cryptographic_hash::Algorithm =
            crate::q_cryptographic_hash::Algorithm(13);
        /// C++ enum variant: <span style='color: green;'>```RealSha3_512 = 14```</span>
        #[allow(non_upper_case_globals)]
        pub const RealSha3512: crate::q_cryptographic_hash::Algorithm =
            crate::q_cryptographic_hash::Algorithm(14);
        /// Generate an SHA3-224 hash sum. Introduced in Qt 5.1 (C++ enum variant: <span style='color: green;'>```Sha3_224 = 11```</span>)
        #[allow(non_upper_case_globals)]
        pub const Sha3224: crate::q_cryptographic_hash::Algorithm =
            crate::q_cryptographic_hash::Algorithm(11);
        /// Generate an SHA3-256 hash sum. Introduced in Qt 5.1 (C++ enum variant: <span style='color: green;'>```Sha3_256 = 12```</span>)
        #[allow(non_upper_case_globals)]
        pub const Sha3256: crate::q_cryptographic_hash::Algorithm =
            crate::q_cryptographic_hash::Algorithm(12);
        /// Generate an SHA3-384 hash sum. Introduced in Qt 5.1 (C++ enum variant: <span style='color: green;'>```Sha3_384 = 13```</span>)
        #[allow(non_upper_case_globals)]
        pub const Sha3384: crate::q_cryptographic_hash::Algorithm =
            crate::q_cryptographic_hash::Algorithm(13);
        /// Generate an SHA3-512 hash sum. Introduced in Qt 5.1 (C++ enum variant: <span style='color: green;'>```Sha3_512 = 14```</span>)
        #[allow(non_upper_case_globals)]
        pub const Sha3512: crate::q_cryptographic_hash::Algorithm =
            crate::q_cryptographic_hash::Algorithm(14);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qcryptographichash.html">QCryptographicHash</a> class provides a way to generate cryptographic hashes.</p>
///
/// C++ class: <span style='color: green;'>```QCryptographicHash```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qcryptographichash.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qcryptographichash.html">QCryptographicHash</a> class provides a way to generate cryptographic hashes.</p>
/// <p><a href="http://doc.qt.io/qt-5/qcryptographichash.html">QCryptographicHash</a> can be used to generate cryptographic hashes of binary or text data.</p>
/// <p>Refer to the documentation of the <a href="http://doc.qt.io/qt-5/qcryptographichash.html#Algorithm-enum">QCryptographicHash::Algorithm</a> enum for a list of the supported algorithms.</p></div>
#[repr(C)]
pub struct QCryptographicHash {
    _unused: u8,
}
impl QCryptographicHash {
    /// <p>Adds the first <i>length</i> chars of <i>data</i> to the cryptographic hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCryptographicHash::addData(const char* data, int length)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcryptographichash.html#addData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds the first <i>length</i> chars of <i>data</i> to the cryptographic hash.</p></div>
    pub unsafe fn add_data_char_int(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        length: ::std::os::raw::c_int,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCryptographicHash_addData(
            self as *mut crate::QCryptographicHash,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(data)
                .as_raw_ptr(),
            length,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qcryptographichash.html#addData">addData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCryptographicHash::addData(const QByteArray& data)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcryptographichash.html#addData-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qcryptographichash.html#addData">addData</a>().</p></div>
    pub unsafe fn add_data_q_byte_array(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCryptographicHash_addData1(
            self as *mut crate::QCryptographicHash,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(data)
                .as_raw_ptr(),
        )
    }

    /// <p>Reads the data from the open <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> <i>device</i> until it ends and hashes it. Returns <code>true</code> if reading was successful.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCryptographicHash::addData(QIODevice* device)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcryptographichash.html#addData-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads the data from the open <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> <i>device</i> until it ends and hashes it. Returns <code>true</code> if reading was successful.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn add_data_q_io_device(
        &mut self,
        device: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QIODevice>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCryptographicHash_addData2(
            self as *mut crate::QCryptographicHash,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QIODevice>>::cast_into(device)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the hash of <i>data</i> using <i>method</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QCryptographicHash::hash(const QByteArray& data, QCryptographicHash::Algorithm method)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcryptographichash.html#hash">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash of <i>data</i> using <i>method</i>.</p></div>
    pub unsafe fn hash(
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        method: crate::q_cryptographic_hash::Algorithm,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCryptographicHash_hash(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(data)
                .as_raw_ptr(),
            method,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the size of the output of the selected hash <i>method</i> in bytes.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QCryptographicHash::hashLength(QCryptographicHash::Algorithm method)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcryptographichash.html#hashLength">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the size of the output of the selected hash <i>method</i> in bytes.</p>
    /// <p>This function was introduced in  Qt 5.12.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn hash_length(
        method: crate::q_cryptographic_hash::Algorithm,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QCryptographicHash_hashLength(method)
    }

    /// <p>Constructs an object that can be used to create a cryptographic hash from data using <i>method</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCryptographicHash::QCryptographicHash(QCryptographicHash::Algorithm method)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcryptographichash.html#QCryptographicHash">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an object that can be used to create a cryptographic hash from data using <i>method</i>.</p></div>
    pub unsafe fn new(
        method: crate::q_cryptographic_hash::Algorithm,
    ) -> ::cpp_utils::CppBox<crate::QCryptographicHash> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCryptographicHash_QCryptographicHash(method);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Resets the object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCryptographicHash::reset()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcryptographichash.html#reset">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Resets the object.</p></div>
    pub unsafe fn reset(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCryptographicHash_reset(
            self as *mut crate::QCryptographicHash,
        )
    }

    /// <p>Returns the final hash value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QCryptographicHash::result() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcryptographichash.html#result">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the final hash value.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toHex">QByteArray::toHex</a>().</p></div>
    pub unsafe fn result(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCryptographicHash_result(
            self as *const crate::QCryptographicHash,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCryptographicHash_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }
}

pub mod q_data_stream {
    //! C++ type: <span style='color: green;'>```QDataStream```</span>

    /// <p>This enum provides symbolic synonyms for the data serialization format version numbers.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QDataStream::Version```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#Version-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum provides symbolic synonyms for the data serialization format version numbers.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatastream.html#setVersion">setVersion</a>() and <a href="http://doc.qt.io/qt-5/qdatastream.html#version">version</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Version(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Version {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Version(value)
        }
    }

    impl From<Version> for ::std::os::raw::c_int {
        fn from(value: Version) -> Self {
            value.0
        }
    }

    impl Version {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Version {
        /// Version 1 (Qt 1.x) (C++ enum variant: <span style='color: green;'>```Qt_1_0 = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt10: crate::q_data_stream::Version = crate::q_data_stream::Version(1);
        /// Version 2 (Qt 2.0) (C++ enum variant: <span style='color: green;'>```Qt_2_0 = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt20: crate::q_data_stream::Version = crate::q_data_stream::Version(2);
        /// Version 3 (Qt 2.1, 2.2, 2.3) (C++ enum variant: <span style='color: green;'>```Qt_2_1 = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt21: crate::q_data_stream::Version = crate::q_data_stream::Version(3);
        /// Version 4 (Qt 3.0) (C++ enum variant: <span style='color: green;'>```Qt_3_0 = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt30: crate::q_data_stream::Version = crate::q_data_stream::Version(4);
        /// Version 5 (Qt 3.1, 3.2) (C++ enum variant: <span style='color: green;'>```Qt_3_1 = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt31: crate::q_data_stream::Version = crate::q_data_stream::Version(5);
        /// Version 6 (Qt 3.3) (C++ enum variant: <span style='color: green;'>```Qt_3_3 = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt33: crate::q_data_stream::Version = crate::q_data_stream::Version(6);
        /// Version 7 (Qt 4.0, Qt 4.1) (C++ enum variant: <span style='color: green;'>```Qt_4_0 = 7```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt40: crate::q_data_stream::Version = crate::q_data_stream::Version(7);
        /// Version 7 (Qt 4.0, Qt 4.1) (C++ enum variant: <span style='color: green;'>```Qt_4_1 = 7```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt41: crate::q_data_stream::Version = crate::q_data_stream::Version(7);
        /// Version 8 (Qt 4.2) (C++ enum variant: <span style='color: green;'>```Qt_4_2 = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt42: crate::q_data_stream::Version = crate::q_data_stream::Version(8);
        /// Version 9 (Qt 4.3) (C++ enum variant: <span style='color: green;'>```Qt_4_3 = 9```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt43: crate::q_data_stream::Version = crate::q_data_stream::Version(9);
        /// Version 10 (Qt 4.4) (C++ enum variant: <span style='color: green;'>```Qt_4_4 = 10```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt44: crate::q_data_stream::Version = crate::q_data_stream::Version(10);
        /// Version 11 (Qt 4.5) (C++ enum variant: <span style='color: green;'>```Qt_4_5 = 11```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt45: crate::q_data_stream::Version = crate::q_data_stream::Version(11);
        /// Version 12 (Qt 4.6, Qt 4.7, Qt 4.8) (C++ enum variant: <span style='color: green;'>```Qt_4_6 = 12```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt46: crate::q_data_stream::Version = crate::q_data_stream::Version(12);
        /// Same as Qt_4_6. (C++ enum variant: <span style='color: green;'>```Qt_4_7 = 12```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt47: crate::q_data_stream::Version = crate::q_data_stream::Version(12);
        /// Same as Qt_4_6. (C++ enum variant: <span style='color: green;'>```Qt_4_8 = 12```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt48: crate::q_data_stream::Version = crate::q_data_stream::Version(12);
        /// Same as Qt_4_6. (C++ enum variant: <span style='color: green;'>```Qt_4_9 = 12```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt49: crate::q_data_stream::Version = crate::q_data_stream::Version(12);
        /// Version 13 (Qt 5.0) (C++ enum variant: <span style='color: green;'>```Qt_5_0 = 13```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt50: crate::q_data_stream::Version = crate::q_data_stream::Version(13);
        /// Version 14 (Qt 5.1) (C++ enum variant: <span style='color: green;'>```Qt_5_1 = 14```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt51: crate::q_data_stream::Version = crate::q_data_stream::Version(14);
        /// Version 15 (Qt 5.2) (C++ enum variant: <span style='color: green;'>```Qt_5_2 = 15```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt52: crate::q_data_stream::Version = crate::q_data_stream::Version(15);
        /// Same as Qt_5_2 (C++ enum variant: <span style='color: green;'>```Qt_5_3 = 15```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt53: crate::q_data_stream::Version = crate::q_data_stream::Version(15);
        /// Version 16 (Qt 5.4) (C++ enum variant: <span style='color: green;'>```Qt_5_4 = 16```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt54: crate::q_data_stream::Version = crate::q_data_stream::Version(16);
        /// Same as Qt_5_4 (C++ enum variant: <span style='color: green;'>```Qt_5_5 = 16```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt55: crate::q_data_stream::Version = crate::q_data_stream::Version(16);
        /// Version 17 (Qt 5.6) (C++ enum variant: <span style='color: green;'>```Qt_5_6 = 17```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt56: crate::q_data_stream::Version = crate::q_data_stream::Version(17);
        /// Same as Qt_5_6 (C++ enum variant: <span style='color: green;'>```Qt_5_7 = 17```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt57: crate::q_data_stream::Version = crate::q_data_stream::Version(17);
        /// Same as Qt_5_6 (C++ enum variant: <span style='color: green;'>```Qt_5_8 = 17```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt58: crate::q_data_stream::Version = crate::q_data_stream::Version(17);
        /// Same as Qt_5_6 (C++ enum variant: <span style='color: green;'>```Qt_5_9 = 17```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt59: crate::q_data_stream::Version = crate::q_data_stream::Version(17);
        /// C++ enum variant: <span style='color: green;'>```Qt_DefaultCompiledVersion = 17```</span>
        #[allow(non_upper_case_globals)]
        pub const QtDefaultCompiledVersion: crate::q_data_stream::Version =
            crate::q_data_stream::Version(17);
        /// Same as Qt_5_6 (C++ enum variant: <span style='color: green;'>```Qt_5_10 = 17```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt510: crate::q_data_stream::Version = crate::q_data_stream::Version(17);
        /// Same as Qt_5_6 (C++ enum variant: <span style='color: green;'>```Qt_5_11 = 17```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt511: crate::q_data_stream::Version = crate::q_data_stream::Version(17);
        /// Version 18 (Qt 5.12) (C++ enum variant: <span style='color: green;'>```Qt_5_12 = 18```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt512: crate::q_data_stream::Version = crate::q_data_stream::Version(18);
        /// C++ enum variant: <span style='color: green;'>```Qt_DefaultCompiledVersion = 18```</span>
        #[allow(non_upper_case_globals)]
        pub const QtDefaultCompiledVersion2: crate::q_data_stream::Version =
            crate::q_data_stream::Version(18);
        /// Version 19 (Qt 5.13) (C++ enum variant: <span style='color: green;'>```Qt_5_13 = 19```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qt513: crate::q_data_stream::Version = crate::q_data_stream::Version(19);
        /// C++ enum variant: <span style='color: green;'>```Qt_DefaultCompiledVersion = 19```</span>
        #[allow(non_upper_case_globals)]
        pub const QtDefaultCompiledVersion3: crate::q_data_stream::Version =
            crate::q_data_stream::Version(19);
    }

    /// <p>The byte order used for reading/writing the data.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QDataStream::ByteOrder```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#ByteOrder-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The byte order used for reading/writing the data.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct ByteOrder(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for ByteOrder {
        fn from(value: ::std::os::raw::c_int) -> Self {
            ByteOrder(value)
        }
    }

    impl From<ByteOrder> for ::std::os::raw::c_int {
        fn from(value: ByteOrder) -> Self {
            value.0
        }
    }

    impl ByteOrder {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl ByteOrder {
        /// Most significant byte first (the default) (C++ enum variant: <span style='color: green;'>```BigEndian = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const BigEndian: crate::q_data_stream::ByteOrder = crate::q_data_stream::ByteOrder(0);
        /// Least significant byte first (C++ enum variant: <span style='color: green;'>```LittleEndian = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const LittleEndian: crate::q_data_stream::ByteOrder =
            crate::q_data_stream::ByteOrder(1);
    }

    /// <p>This enum describes the current status of the data stream.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QDataStream::Status```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#Status-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the current status of the data stream.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Status(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Status {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Status(value)
        }
    }

    impl From<Status> for ::std::os::raw::c_int {
        fn from(value: Status) -> Self {
            value.0
        }
    }

    impl Status {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Status {
        /// The data stream is operating normally. (C++ enum variant: <span style='color: green;'>```Ok = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const Ok: crate::q_data_stream::Status = crate::q_data_stream::Status(0);
        /// The data stream has read past the end of the data in the underlying device. (C++ enum variant: <span style='color: green;'>```ReadPastEnd = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const ReadPastEnd: crate::q_data_stream::Status = crate::q_data_stream::Status(1);
        /// The data stream has read corrupt data. (C++ enum variant: <span style='color: green;'>```ReadCorruptData = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const ReadCorruptData: crate::q_data_stream::Status = crate::q_data_stream::Status(2);
        /// The data stream cannot write to the underlying device. (C++ enum variant: <span style='color: green;'>```WriteFailed = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const WriteFailed: crate::q_data_stream::Status = crate::q_data_stream::Status(3);
    }

    /// <p>The precision of floating point numbers used for reading/writing the data. This will only have an effect if the version of the data stream is <a href="http://doc.qt.io/qt-5/qdatastream.html#Version-enum">Qt_4_6</a> or higher.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QDataStream::FloatingPointPrecision```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#FloatingPointPrecision-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The precision of floating point numbers used for reading/writing the data. This will only have an effect if the version of the data stream is <a href="http://doc.qt.io/qt-5/qdatastream.html#Version-enum">Qt_4_6</a> or higher.</p>
    /// <p><b>Warning:</b> The floating point precision must be set to the same value on the object that writes and the object that reads the data stream.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatastream.html#setFloatingPointPrecision">setFloatingPointPrecision</a>() and <a href="http://doc.qt.io/qt-5/qdatastream.html#floatingPointPrecision">floatingPointPrecision</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct FloatingPointPrecision(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for FloatingPointPrecision {
        fn from(value: ::std::os::raw::c_int) -> Self {
            FloatingPointPrecision(value)
        }
    }

    impl From<FloatingPointPrecision> for ::std::os::raw::c_int {
        fn from(value: FloatingPointPrecision) -> Self {
            value.0
        }
    }

    impl FloatingPointPrecision {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl FloatingPointPrecision {
        /// All floating point numbers in the data stream have 32-bit precision. (C++ enum variant: <span style='color: green;'>```SinglePrecision = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const SinglePrecision: crate::q_data_stream::FloatingPointPrecision =
            crate::q_data_stream::FloatingPointPrecision(0);
        /// All floating point numbers in the data stream have 64-bit precision. (C++ enum variant: <span style='color: green;'>```DoublePrecision = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const DoublePrecision: crate::q_data_stream::FloatingPointPrecision =
            crate::q_data_stream::FloatingPointPrecision(1);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qdatastream.html">QDataStream</a> class provides serialization of binary data to a <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>.</p>
///
/// C++ class: <span style='color: green;'>```QDataStream```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qdatastream.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qdatastream.html">QDataStream</a> class provides serialization of binary data to a <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>.</p>
/// <p>A data stream is a binary stream of encoded information which is 100% independent of the host computer's operating system, CPU or byte order. For example, a data stream that is written by a PC under Windows can be read by a Sun SPARC running Solaris.</p>
/// <p>You can also use a data stream to read/write <a href="http://doc.qt.io/qt-5/qdatastream.html#raw">raw unencoded binary data</a>. If you want a "parsing" input stream, see <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qdatastream.html">QDataStream</a> class implements the serialization of C++'s basic data types, like <code>char</code>, <code>short</code>, <code>int</code>, <code>char *</code>, etc. Serialization of more complex data is accomplished by breaking up the data into primitive units.</p>
/// <p>A data stream cooperates closely with a <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>. A <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> represents an input/output medium one can read data from and write data to. The <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> class is an example of an I/O device.</p>
/// <p>Example (write binary data to a stream):</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qfile.html">QFile</a></span> file(<span class="string">"file.dat"</span>);
///   file<span class="operator">.</span>open(<span class="type"><a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a></span><span class="operator">::</span>WriteOnly);
///   <span class="type"><a href="http://doc.qt.io/qt-5/qdatastream.html#QDataStream">QDataStream</a></span> out(<span class="operator">&amp;</span>file);   <span class="comment">// we will serialize the data into the file</span>
///   out <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>(<span class="string">"the answer is"</span>);   <span class="comment">// serialize a string</span>
///   out <span class="operator">&lt;</span><span class="operator">&lt;</span> (<span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#qint32-typedef">qint32</a></span>)<span class="number">42</span>;        <span class="comment">// serialize an integer</span>
///
/// </pre>
/// <p>Example (read binary data from a stream):</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qfile.html">QFile</a></span> file(<span class="string">"file.dat"</span>);
///   file<span class="operator">.</span>open(<span class="type"><a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a></span><span class="operator">::</span>ReadOnly);
///   <span class="type"><a href="http://doc.qt.io/qt-5/qdatastream.html#QDataStream">QDataStream</a></span> in(<span class="operator">&amp;</span>file);    <span class="comment">// read the data serialized from the file</span>
///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> str;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#qint32-typedef">qint32</a></span> a;
///   in <span class="operator">&gt;</span><span class="operator">&gt;</span> str <span class="operator">&gt;</span><span class="operator">&gt;</span> a;           <span class="comment">// extract "the answer is" and 42</span>
///
/// </pre>
/// <p>Each item written to the stream is written in a predefined binary format that varies depending on the item's type. Supported Qt types include <a href="http://doc.qt.io/qt-5/qbrush.html">QBrush</a>, <a href="http://doc.qt.io/qt-5/qcolor.html">QColor</a>, <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>, <a href="http://doc.qt.io/qt-5/qfont.html">QFont</a>, <a href="http://doc.qt.io/qt-5/qpixmap.html">QPixmap</a>, <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> and many others. For the complete list of all Qt types supporting data streaming see <a href="http://doc.qt.io/qt-5/datastreamformat.html">Serializing Qt Data Types</a>.</p>
/// <p>For integers it is best to always cast to a Qt integer type for writing, and to read back into the same Qt integer type. This ensures that you get integers of the size you want and insulates you from compiler and platform differences.</p>
/// <p>To take one example, a <code>char *</code> string is written as a 32-bit integer equal to the length of the string including the '\0' byte, followed by all the characters of the string including the '\0' byte. When reading a <code>char *</code> string, 4 bytes are read to create the 32-bit length value, then that many characters for the <code>char *</code> string including the '\0' terminator are read.</p>
/// <p>The initial I/O device is usually set in the constructor, but can be changed with <a href="http://doc.qt.io/qt-5/qdatastream.html#setDevice">setDevice</a>(). If you've reached the end of the data (or if there is no I/O device set) <a href="http://doc.qt.io/qt-5/qdatastream.html#atEnd">atEnd</a>() will return true.</p>
/// <a name="versioning"></a></div>
#[repr(C)]
pub struct QDataStream {
    _unused: u8,
}
impl QDataStream {
    /// <p>Aborts a read transaction.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDataStream::abortTransaction()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#abortTransaction">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Aborts a read transaction.</p>
    /// <p>This function is commonly used to discard the transaction after higher-level protocol errors or loss of stream synchronization.</p>
    /// <p>If called on an inner transaction, aborting is delegated to the outermost transaction, and subsequently started inner transactions are forced to fail.</p>
    /// <p>For the outermost transaction, discards the restoration point and any internally duplicated data of the stream. Will not affect the current read position of the stream.</p>
    /// <p>Sets the status of the data stream to</p></div>
    pub unsafe fn abort_transaction(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QDataStream_abortTransaction(self as *mut crate::QDataStream)
    }

    /// <p>Returns <code>true</code> if the I/O device has reached the end position (end of the stream or file) or if there is no I/O device set; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDataStream::atEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#atEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the I/O device has reached the end position (end of the stream or file) or if there is no I/O device set; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#atEnd">QIODevice::atEnd</a>().</p></div>
    pub unsafe fn at_end(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDataStream_atEnd(self as *const crate::QDataStream)
    }

    /// <p>Returns the current byte order setting -- either <a href="http://doc.qt.io/qt-5/qdatastream.html#ByteOrder-enum">BigEndian</a> or <a href="http://doc.qt.io/qt-5/qdatastream.html#ByteOrder-enum">LittleEndian</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream::ByteOrder QDataStream::byteOrder() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#byteOrder">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current byte order setting -- either <a href="http://doc.qt.io/qt-5/qdatastream.html#ByteOrder-enum">BigEndian</a> or <a href="http://doc.qt.io/qt-5/qdatastream.html#ByteOrder-enum">LittleEndian</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatastream.html#setByteOrder">setByteOrder</a>().</p></div>
    pub unsafe fn byte_order(&self) -> crate::q_data_stream::ByteOrder {
        crate::__ffi::ctr_qt_core_ffi_QDataStream_byteOrder(self as *const crate::QDataStream)
    }

    /// <p>Completes a read transaction. Returns <code>true</code> if no read errors have occurred during the transaction; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDataStream::commitTransaction()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#commitTransaction">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Completes a read transaction. Returns <code>true</code> if no read errors have occurred during the transaction; otherwise returns <code>false</code>.</p>
    /// <p>If called on an inner transaction, committing will be postponed until the outermost commitTransaction(), <a href="http://doc.qt.io/qt-5/qdatastream.html#rollbackTransaction">rollbackTransaction</a>(), or <a href="http://doc.qt.io/qt-5/qdatastream.html#abortTransaction">abortTransaction</a>() call occurs.</p>
    /// <p>Otherwise, if the stream status indicates reading past the end of the data, this function restores the stream data to the point of the <a href="http://doc.qt.io/qt-5/qdatastream.html#startTransaction">startTransaction</a>() call. When this situation occurs, you need to wait for more data to arrive, after which you start a new transaction. If the data stream has read corrupt data or any of the inner transactions was aborted, this function aborts the transaction.</p>
    /// <p>This function was introduced in  Qt 5.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatastream.html#startTransaction">startTransaction</a>(), <a href="http://doc.qt.io/qt-5/qdatastream.html#rollbackTransaction">rollbackTransaction</a>(), and <a href="http://doc.qt.io/qt-5/qdatastream.html#abortTransaction">abortTransaction</a>().</p></div>
    pub unsafe fn commit_transaction(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDataStream_commitTransaction(self as *mut crate::QDataStream)
    }

    /// <p>Returns the I/O device currently set, or 0 if no device is currently set.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QIODevice* QDataStream::device() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#device">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the I/O device currently set, or 0 if no device is currently set.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatastream.html#setDevice">setDevice</a>().</p></div>
    pub unsafe fn device(&self) -> ::cpp_utils::MutPtr<crate::QIODevice> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDataStream_device(self as *const crate::QDataStream);
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns the floating point precision of the data stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream::FloatingPointPrecision QDataStream::floatingPointPrecision() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#floatingPointPrecision">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the floating point precision of the data stream.</p>
    /// <p>This function was introduced in  Qt 4.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatastream.html#FloatingPointPrecision-enum">FloatingPointPrecision</a> and <a href="http://doc.qt.io/qt-5/qdatastream.html#setFloatingPointPrecision">setFloatingPointPrecision</a>().</p></div>
    pub unsafe fn floating_point_precision(&self) -> crate::q_data_stream::FloatingPointPrecision {
        crate::__ffi::ctr_qt_core_ffi_QDataStream_floatingPointPrecision(
            self as *const crate::QDataStream,
        )
    }

    /// <p>Constructs a data stream that has no I/O device.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDataStream::QDataStream()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#QDataStream">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a data stream that has no I/O device.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatastream.html#setDevice">setDevice</a>().</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QDataStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDataStream_QDataStream();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a data stream that uses the I/O device <i>d</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDataStream::QDataStream(QIODevice* arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#QDataStream-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a data stream that uses the I/O device <i>d</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatastream.html#setDevice">setDevice</a>() and <a href="http://doc.qt.io/qt-5/qdatastream.html#device">device</a>().</p></div>
    pub unsafe fn from_q_io_device(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QIODevice>>,
    ) -> ::cpp_utils::CppBox<crate::QDataStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDataStream_QDataStream1(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QIODevice>>::cast_into(arg1)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a data stream that operates on a byte array, <i>a</i>. The <i>mode</i> describes how the device is to be used.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDataStream::QDataStream(QByteArray* arg1, QFlags<QIODevice::OpenModeFlag> flags)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#QDataStream-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a data stream that operates on a byte array, <i>a</i>. The <i>mode</i> describes how the device is to be used.</p>
    /// <p>Alternatively, you can use <a href="http://doc.qt.io/qt-5/qdatastream.html">QDataStream</a>(const <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> &amp;) if you just want to read from a byte array.</p>
    /// <p>Since <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> is not a <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> subclass, internally a <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> is created to wrap the byte array.</p></div>
    pub unsafe fn from_q_byte_array_q_flags_open_mode_flag(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QByteArray>>,
        flags: crate::QFlags<crate::q_io_device::OpenModeFlag>,
    ) -> ::cpp_utils::CppBox<crate::QDataStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDataStream_QDataStream2(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QByteArray>>::cast_into(arg1)
                .as_mut_raw_ptr(),
            flags.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a read-only data stream that operates on byte array <i>a</i>. Use <a href="http://doc.qt.io/qt-5/qdatastream.html">QDataStream</a>(<a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>*, int) if you want to write to a byte array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDataStream::QDataStream(const QByteArray& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#QDataStream-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a read-only data stream that operates on byte array <i>a</i>. Use <a href="http://doc.qt.io/qt-5/qdatastream.html">QDataStream</a>(<a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>*, int) if you want to write to a byte array.</p>
    /// <p>Since <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> is not a <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> subclass, internally a <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> is created to wrap the byte array.</p></div>
    pub unsafe fn from_q_byte_array(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QDataStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDataStream_QDataStream3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reads the buffer <i>s</i> from the stream and returns a reference to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::readBytes(char*& arg1, unsigned int& len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#readBytes">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads the buffer <i>s</i> from the stream and returns a reference to the stream.</p>
    /// <p>The buffer <i>s</i> is allocated using <code>new []</code>. Destroy it with the <code>delete []</code> operator.</p>
    /// <p>The <i>l</i> parameter is set to the length of the buffer. If the string read is empty, <i>l</i> is set to 0 and <i>s</i> is set to a null pointer.</p>
    /// <p>The serialization format is a quint32 length specifier first, then <i>l</i> bytes of data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatastream.html#readRawData">readRawData</a>() and <a href="http://doc.qt.io/qt-5/qdatastream.html#writeBytes">writeBytes</a>().</p></div>
    pub unsafe fn read_bytes(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<*mut ::std::os::raw::c_char>>,
        len: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<::std::os::raw::c_uint>>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDataStream_readBytes(
            self as *mut crate::QDataStream,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<*mut ::std::os::raw::c_char>>::cast_into(
                arg1,
            )
            .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<::std::os::raw::c_uint>>::cast_into(len)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Reads at most <i>len</i> bytes from the stream into <i>s</i> and returns the number of bytes read. If an error occurs, this function returns -1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QDataStream::readRawData(char* arg1, int len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#readRawData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads at most <i>len</i> bytes from the stream into <i>s</i> and returns the number of bytes read. If an error occurs, this function returns -1.</p>
    /// <p>The buffer <i>s</i> must be preallocated. The data is <i>not</i> encoded.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatastream.html#readBytes">readBytes</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#read">QIODevice::read</a>(), and <a href="http://doc.qt.io/qt-5/qdatastream.html#writeRawData">writeRawData</a>().</p></div>
    pub unsafe fn read_raw_data(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_char>>,
        len: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QDataStream_readRawData(
            self as *mut crate::QDataStream,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_mut_raw_ptr(),
            len,
        )
    }

    /// <p>Resets the status of the data stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDataStream::resetStatus()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#resetStatus">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Resets the status of the data stream.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatastream.html#Status-enum">Status</a>, <a href="http://doc.qt.io/qt-5/qdatastream.html#status">status</a>(), and <a href="http://doc.qt.io/qt-5/qdatastream.html#setStatus">setStatus</a>().</p></div>
    pub unsafe fn reset_status(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QDataStream_resetStatus(self as *mut crate::QDataStream)
    }

    /// <p>Reverts a read transaction.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDataStream::rollbackTransaction()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#rollbackTransaction">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reverts a read transaction.</p>
    /// <p>This function is commonly used to rollback the transaction when an incomplete read was detected prior to committing the transaction.</p>
    /// <p>If called on an inner transaction, reverting is delegated to the outermost transaction, and subsequently started inner transactions are forced to fail.</p>
    /// <p>For the outermost transaction, restores the stream data to the point of the <a href="http://doc.qt.io/qt-5/qdatastream.html#startTransaction">startTransaction</a>() call. If the data stream has read corrupt data or any of the inner transactions was aborted, this function aborts the transaction.</p>
    /// <p>If the preceding stream operations were successful, sets the status of the data stream to</p></div>
    pub unsafe fn rollback_transaction(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QDataStream_rollbackTransaction(
            self as *mut crate::QDataStream,
        )
    }

    /// <p>Sets the serialization byte order to <i>bo</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDataStream::setByteOrder(QDataStream::ByteOrder arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#setByteOrder">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the serialization byte order to <i>bo</i>.</p>
    /// <p>The <i>bo</i> parameter can be <a href="http://doc.qt.io/qt-5/qdatastream.html#ByteOrder-enum">QDataStream::BigEndian</a> or <a href="http://doc.qt.io/qt-5/qdatastream.html#ByteOrder-enum">QDataStream::LittleEndian</a>.</p>
    /// <p>The default setting is big endian. We recommend leaving this setting unless you have special requirements.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatastream.html#byteOrder">byteOrder</a>().</p></div>
    pub unsafe fn set_byte_order(&mut self, arg1: crate::q_data_stream::ByteOrder) {
        crate::__ffi::ctr_qt_core_ffi_QDataStream_setByteOrder(
            self as *mut crate::QDataStream,
            arg1,
        )
    }

    /// <p>void QDataStream::setDevice(<a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> *d)</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDataStream::setDevice(QIODevice* arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#setDevice">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>void QDataStream::setDevice(<a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> *d)</p>
    /// <p>Sets the I/O device to <i>d</i>, which can be 0 to unset to current I/O device.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatastream.html#device">device</a>().</p></div>
    pub unsafe fn set_device(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QIODevice>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QDataStream_setDevice(
            self as *mut crate::QDataStream,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QIODevice>>::cast_into(arg1)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Sets the floating point precision of the data stream to <i>precision</i>. If the floating point precision is <a href="http://doc.qt.io/qt-5/qdatastream.html#FloatingPointPrecision-enum">DoublePrecision</a> and the version of the data stream is <a href="http://doc.qt.io/qt-5/qdatastream.html#Version-enum">Qt_4_6</a> or higher, all floating point numbers will be written and read with 64-bit precision. If the floating point precision is <a href="http://doc.qt.io/qt-5/qdatastream.html#FloatingPointPrecision-enum">SinglePrecision</a> and the version is <a href="http://doc.qt.io/qt-5/qdatastream.html#Version-enum">Qt_4_6</a> or higher, all floating point numbers will be written and read with 32-bit precision.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDataStream::setFloatingPointPrecision(QDataStream::FloatingPointPrecision precision)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#setFloatingPointPrecision">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the floating point precision of the data stream to <i>precision</i>. If the floating point precision is <a href="http://doc.qt.io/qt-5/qdatastream.html#FloatingPointPrecision-enum">DoublePrecision</a> and the version of the data stream is <a href="http://doc.qt.io/qt-5/qdatastream.html#Version-enum">Qt_4_6</a> or higher, all floating point numbers will be written and read with 64-bit precision. If the floating point precision is <a href="http://doc.qt.io/qt-5/qdatastream.html#FloatingPointPrecision-enum">SinglePrecision</a> and the version is <a href="http://doc.qt.io/qt-5/qdatastream.html#Version-enum">Qt_4_6</a> or higher, all floating point numbers will be written and read with 32-bit precision.</p>
    /// <p>For versions prior to <a href="http://doc.qt.io/qt-5/qdatastream.html#Version-enum">Qt_4_6</a>, the precision of floating point numbers in the data stream depends on the stream operator called.</p>
    /// <p>The default is <a href="http://doc.qt.io/qt-5/qdatastream.html#FloatingPointPrecision-enum">DoublePrecision</a>.</p>
    /// <p>Note that this property does not affect the serialization or deserialization of <code>qfloat16</code> instances.</p>
    /// <p><b>Warning:</b> This property must be set to the same value on the object that writes and the object that reads the data stream.</p>
    /// <p>This function was introduced in  Qt 4.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatastream.html#floatingPointPrecision">floatingPointPrecision</a>().</p></div>
    pub unsafe fn set_floating_point_precision(
        &mut self,
        precision: crate::q_data_stream::FloatingPointPrecision,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QDataStream_setFloatingPointPrecision(
            self as *mut crate::QDataStream,
            precision,
        )
    }

    /// <p>Sets the status of the data stream to the <i>status</i> given.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDataStream::setStatus(QDataStream::Status status)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#setStatus">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the status of the data stream to the <i>status</i> given.</p>
    /// <p>Subsequent calls to setStatus() are ignored until <a href="http://doc.qt.io/qt-5/qdatastream.html#resetStatus">resetStatus</a>() is called.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatastream.html#Status-enum">Status</a>, <a href="http://doc.qt.io/qt-5/qdatastream.html#status">status</a>(), and <a href="http://doc.qt.io/qt-5/qdatastream.html#resetStatus">resetStatus</a>().</p></div>
    pub unsafe fn set_status(&mut self, status: crate::q_data_stream::Status) {
        crate::__ffi::ctr_qt_core_ffi_QDataStream_setStatus(self as *mut crate::QDataStream, status)
    }

    /// <p>Sets the version number of the data serialization format to <i>v</i>, a value of the <a href="http://doc.qt.io/qt-5/qdatastream.html#Version-enum">Version</a> enum.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDataStream::setVersion(int arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#setVersion">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the version number of the data serialization format to <i>v</i>, a value of the <a href="http://doc.qt.io/qt-5/qdatastream.html#Version-enum">Version</a> enum.</p>
    /// <p>You don't <i>have</i> to set a version if you are using the current version of Qt, but for your own custom binary formats we recommend that you do; see <a href="http://doc.qt.io/qt-5/qdatastream.html#versioning">Versioning</a> in the Detailed Description.</p>
    /// <p>To accommodate new functionality, the datastream serialization format of some Qt classes has changed in some versions of Qt. If you want to read data that was created by an earlier version of Qt, or write data that can be read by a program that was compiled with an earlier version of Qt, use this function to modify the serialization format used by <a href="http://doc.qt.io/qt-5/qdatastream.html">QDataStream</a>.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qdatastream.html#Version-enum">Version</a> enum provides symbolic constants for the different versions of Qt. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdatastream.html#QDataStream">QDataStream</a></span> out(file);
    ///   out<span class="operator">.</span>setVersion(<span class="type"><a href="http://doc.qt.io/qt-5/qdatastream.html#QDataStream">QDataStream</a></span><span class="operator">::</span>Qt_4_0);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatastream.html#version">version</a>() and <a href="http://doc.qt.io/qt-5/qdatastream.html#Version-enum">Version</a>.</p></div>
    pub unsafe fn set_version(&mut self, arg1: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QDataStream_setVersion(self as *mut crate::QDataStream, arg1)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::operator<<(double f)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#operator-lt-lt-12">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes a floating point number, <i>f</i>, to the stream using the standard IEEE 754 format. Returns a reference to the stream.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatastream.html#setFloatingPointPrecision">setFloatingPointPrecision</a>().</p></div>
    pub unsafe fn shl(
        &mut self,
        f: ::std::os::raw::c_double,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDataStream_operator__24(
            self as *mut crate::QDataStream,
            f,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::operator>>(double& f)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#operator-gt-gt-12">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Reads a floating point number from the stream into <i>f</i>, using the standard IEEE 754 format. Returns a reference to the stream.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatastream.html#setFloatingPointPrecision">setFloatingPointPrecision</a>().</p></div>
    pub unsafe fn shr(
        &mut self,
        f: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<::std::os::raw::c_double>>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDataStream_operator__11(
            self as *mut crate::QDataStream,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<::std::os::raw::c_double>>::cast_into(f)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Skips <i>len</i> bytes from the device. Returns the number of bytes actually skipped, or -1 on error.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QDataStream::skipRawData(int len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#skipRawData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Skips <i>len</i> bytes from the device. Returns the number of bytes actually skipped, or -1 on error.</p>
    /// <p>This is equivalent to calling <a href="http://doc.qt.io/qt-5/qdatastream.html#readRawData">readRawData</a>() on a buffer of length <i>len</i> and ignoring the buffer.</p>
    /// <p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#seek">QIODevice::seek</a>().</p></div>
    pub unsafe fn skip_raw_data(&mut self, len: ::std::os::raw::c_int) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QDataStream_skipRawData(self as *mut crate::QDataStream, len)
    }

    /// <p>Starts a new read transaction on the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDataStream::startTransaction()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#startTransaction">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Starts a new read transaction on the stream.</p>
    /// <p>Defines a restorable point within the sequence of read operations. For sequential devices, read data will be duplicated internally to allow recovery in case of incomplete reads. For random-access devices, this function saves the current position of the stream. Call <a href="http://doc.qt.io/qt-5/qdatastream.html#commitTransaction">commitTransaction</a>(), <a href="http://doc.qt.io/qt-5/qdatastream.html#rollbackTransaction">rollbackTransaction</a>(), or <a href="http://doc.qt.io/qt-5/qdatastream.html#abortTransaction">abortTransaction</a>() to finish the current transaction.</p>
    /// <p>Once a transaction is started, subsequent calls to this function will make the transaction recursive. Inner transactions act as agents of the outermost transaction (i.e., report the status of read operations to the outermost transaction, which can restore the position of the stream).</p>
    /// <p><b>Note: </b>Restoring to the point of the nested startTransaction() call is not supported.</p><p>When an error occurs during a transaction (including an inner transaction failing), reading from the data stream is suspended (all subsequent read operations return empty/zero values) and subsequent inner transactions are forced to fail. Starting a new outermost transaction recovers from this state. This behavior makes it unnecessary to error-check every read operation separately.</p>
    /// <p>This function was introduced in  Qt 5.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatastream.html#commitTransaction">commitTransaction</a>(), <a href="http://doc.qt.io/qt-5/qdatastream.html#rollbackTransaction">rollbackTransaction</a>(), and <a href="http://doc.qt.io/qt-5/qdatastream.html#abortTransaction">abortTransaction</a>().</p></div>
    pub unsafe fn start_transaction(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QDataStream_startTransaction(self as *mut crate::QDataStream)
    }

    /// <p>Returns the status of the data stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream::Status QDataStream::status() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#status">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the status of the data stream.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatastream.html#Status-enum">Status</a>, <a href="http://doc.qt.io/qt-5/qdatastream.html#setStatus">setStatus</a>(), and <a href="http://doc.qt.io/qt-5/qdatastream.html#resetStatus">resetStatus</a>().</p></div>
    pub unsafe fn status(&self) -> crate::q_data_stream::Status {
        crate::__ffi::ctr_qt_core_ffi_QDataStream_status(self as *const crate::QDataStream)
    }

    /// <p>Unsets the I/O device. Use <a href="http://doc.qt.io/qt-5/qdatastream.html#setDevice">setDevice</a>(0) instead.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDataStream::unsetDevice()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream-obsolete.html#unsetDevice">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Unsets the I/O device. Use <a href="http://doc.qt.io/qt-5/qdatastream.html#setDevice">setDevice</a>(0) instead.</p></div>
    pub unsafe fn unset_device(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QDataStream_unsetDevice(self as *mut crate::QDataStream)
    }

    /// <p>Returns the version number of the data serialization format.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QDataStream::version() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#version">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the version number of the data serialization format.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatastream.html#setVersion">setVersion</a>() and <a href="http://doc.qt.io/qt-5/qdatastream.html#Version-enum">Version</a>.</p></div>
    pub unsafe fn version(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QDataStream_version(self as *const crate::QDataStream)
    }

    /// <p>Writes the length specifier <i>len</i> and the buffer <i>s</i> to the stream and returns a reference to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::writeBytes(const char* arg1, unsigned int len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#writeBytes">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes the length specifier <i>len</i> and the buffer <i>s</i> to the stream and returns a reference to the stream.</p>
    /// <p>The <i>len</i> is serialized as a quint32, followed by <i>len</i> bytes from <i>s</i>. Note that the data is <i>not</i> encoded.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatastream.html#writeRawData">writeRawData</a>() and <a href="http://doc.qt.io/qt-5/qdatastream.html#readBytes">readBytes</a>().</p></div>
    pub unsafe fn write_bytes(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        len: ::std::os::raw::c_uint,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDataStream_writeBytes(
            self as *mut crate::QDataStream,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
            len,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Writes <i>len</i> bytes from <i>s</i> to the stream. Returns the number of bytes actually written, or -1 on error. The data is <i>not</i> encoded.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QDataStream::writeRawData(const char* arg1, int len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#writeRawData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes <i>len</i> bytes from <i>s</i> to the stream. Returns the number of bytes actually written, or -1 on error. The data is <i>not</i> encoded.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatastream.html#writeBytes">writeBytes</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#write">QIODevice::write</a>(), and <a href="http://doc.qt.io/qt-5/qdatastream.html#readRawData">readRawData</a>().</p></div>
    pub unsafe fn write_raw_data(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        len: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QDataStream_writeRawData(
            self as *mut crate::QDataStream,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
            len,
        )
    }
}

pub mod q_date {
    //! C++ type: <span style='color: green;'>```QDate```</span>

    /// <p>This enum describes the types of the string representation used for the month name.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QDate::MonthNameType```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#MonthNameType-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the types of the string representation used for the month name.</p>
    ///
    /// <p>This enum was introduced or modified in  Qt 4.5.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct MonthNameType(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for MonthNameType {
        fn from(value: ::std::os::raw::c_int) -> Self {
            MonthNameType(value)
        }
    }

    impl From<MonthNameType> for ::std::os::raw::c_int {
        fn from(value: MonthNameType) -> Self {
            value.0
        }
    }

    impl MonthNameType {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl MonthNameType {
        /// This type of name can be used for date-to-string formatting. (C++ enum variant: <span style='color: green;'>```DateFormat = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const DateFormat: crate::q_date::MonthNameType = crate::q_date::MonthNameType(0);
        /// This type is used when you need to enumerate months or weekdays. Usually standalone names are represented in singular forms with capitalized first letter. (C++ enum variant: <span style='color: green;'>```StandaloneFormat = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const StandaloneFormat: crate::q_date::MonthNameType = crate::q_date::MonthNameType(1);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> class provides date functions.</p>
///
/// C++ class: <span style='color: green;'>```QDate```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qdate.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> class provides date functions.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> object contains a calendar date, i.e. year, month, and day numbers, in the Gregorian calendar. It can read the current date from the system clock. It provides functions for comparing dates, and for manipulating dates. For example, it is possible to add and subtract days, months, and years to dates.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> object is typically created by giving the year, month, and day numbers explicitly. Note that <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> interprets two digit years as is, i.e., years 0 - 99. A <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> can also be constructed with the static function <a href="http://doc.qt.io/qt-5/qdate.html#currentDate">currentDate</a>(), which creates a <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> object containing the system clock's date. An explicit date can also be set using <a href="http://doc.qt.io/qt-5/qdate.html#setDate">setDate</a>(). The <a href="http://doc.qt.io/qt-5/qdate.html#fromString">fromString</a>() function returns a <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> given a string and a date format which is used to interpret the date within the string.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qdate.html#year">year</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#month">month</a>(), and <a href="http://doc.qt.io/qt-5/qdate.html#day">day</a>() functions provide access to the year, month, and day numbers. Also, <a href="http://doc.qt.io/qt-5/qdate.html#dayOfWeek">dayOfWeek</a>() and <a href="http://doc.qt.io/qt-5/qdate.html#dayOfYear">dayOfYear</a>() functions are provided. The same information is provided in textual format by the <a href="http://doc.qt.io/qt-5/qdate.html#toString">toString</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#shortDayName">shortDayName</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#longDayName">longDayName</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#shortMonthName">shortMonthName</a>(), and <a href="http://doc.qt.io/qt-5/qdate.html#longMonthName">longMonthName</a>() functions.</p>
/// <p><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> provides a full set of operators to compare two <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> objects where smaller means earlier, and larger means later.</p>
/// <p>You can increment (or decrement) a date by a given number of days using <a href="http://doc.qt.io/qt-5/qdate.html#addDays">addDays</a>(). Similarly you can use <a href="http://doc.qt.io/qt-5/qdate.html#addMonths">addMonths</a>() and <a href="http://doc.qt.io/qt-5/qdate.html#addYears">addYears</a>(). The <a href="http://doc.qt.io/qt-5/qdate.html#daysTo">daysTo</a>() function returns the number of days between two dates.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qdate.html#daysInMonth">daysInMonth</a>() and <a href="http://doc.qt.io/qt-5/qdate.html#daysInYear">daysInYear</a>() functions return how many days there are in this date's month and year, respectively. The <a href="http://doc.qt.io/qt-5/qdate.html#isLeapYear">isLeapYear</a>() function indicates whether a date is in a leap year.</p>
/// <a name="no-year-0"></a>
/// <a name="no-year-0"></a>
/// <h4>No Year 0</h4>
/// <p>There is no year 0. Dates in that year are considered invalid. The year -1 is the year "1 before Christ" or "1 before current era." The day before 1 January 1 CE is 31 December 1 BCE.</p>
/// <a name="range-of-valid-dates"></a>
/// <h4>Range of Valid Dates</h4>
/// <p>Dates are stored internally as a Julian Day number, an integer count of every day in a contiguous range, with 24 November 4714 BCE in the Gregorian calendar being Julian Day 0 (1 January 4713 BCE in the Julian calendar). As well as being an efficient and accurate way of storing an absolute date, it is suitable for converting a Date into other calendar systems such as Hebrew, Islamic or Chinese. The Julian Day number can be obtained using <a href="http://doc.qt.io/qt-5/qdate.html#toJulianDay">QDate::toJulianDay</a>() and can be set using <a href="http://doc.qt.io/qt-5/qdate.html#fromJulianDay">QDate::fromJulianDay</a>().</p>
/// <p>The range of dates able to be stored by <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> as a Julian Day number is for technical reasons limited to between -784350574879 and 784354017364, which means from before 2 billion BCE to after 2 billion CE.</p></div>
#[repr(C)]
pub struct QDate {
    _unused: u8,
}
impl QDate {
    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> object containing a date <i>ndays</i> later than the date of this object (or earlier if <i>ndays</i> is negative).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDate QDate::addDays(qint64 days) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#addDays">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> object containing a date <i>ndays</i> later than the date of this object (or earlier if <i>ndays</i> is negative).</p>
    /// <p>Returns a null date if the current date is invalid or the new date is out of range.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#addMonths">addMonths</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#addYears">addYears</a>(), and <a href="http://doc.qt.io/qt-5/qdate.html#daysTo">daysTo</a>().</p></div>
    pub unsafe fn add_days(&self, days: i64) -> ::cpp_utils::CppBox<crate::QDate> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDate_addDays(self as *const crate::QDate, days);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> object containing a date <i>nmonths</i> later than the date of this object (or earlier if <i>nmonths</i> is negative).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDate QDate::addMonths(int months) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#addMonths">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> object containing a date <i>nmonths</i> later than the date of this object (or earlier if <i>nmonths</i> is negative).</p>
    /// <p><b>Note: </b>If the ending day/month combination does not exist in the resulting month/year, this function will return a date that is the latest valid date.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#addDays">addDays</a>() and <a href="http://doc.qt.io/qt-5/qdate.html#addYears">addYears</a>().</p></div>
    pub unsafe fn add_months(
        &self,
        months: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QDate> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDate_addMonths(self as *const crate::QDate, months);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> object containing a date <i>nyears</i> later than the date of this object (or earlier if <i>nyears</i> is negative).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDate QDate::addYears(int years) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#addYears">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> object containing a date <i>nyears</i> later than the date of this object (or earlier if <i>nyears</i> is negative).</p>
    /// <p><b>Note: </b>If the ending day/month combination does not exist in the resulting year (i.e., if the date was Feb 29 and the final year is not a leap year), this function will return a date that is the latest valid date (that is, Feb 28).</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#addDays">addDays</a>() and <a href="http://doc.qt.io/qt-5/qdate.html#addMonths">addMonths</a>().</p></div>
    pub unsafe fn add_years(
        &self,
        years: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QDate> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDate_addYears(self as *const crate::QDate, years);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> class provides date functions.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDate& QDate::operator=(const QDate& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> class provides date functions.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> object contains a calendar date, i.e. year, month, and day numbers, in the Gregorian calendar. It can read the current date from the system clock. It provides functions for comparing dates, and for manipulating dates. For example, it is possible to add and subtract days, months, and years to dates.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> object is typically created by giving the year, month, and day numbers explicitly. Note that <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> interprets two digit years as is, i.e., years 0 - 99. A <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> can also be constructed with the static function <a href="http://doc.qt.io/qt-5/qdate.html#currentDate">currentDate</a>(), which creates a <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> object containing the system clock's date. An explicit date can also be set using <a href="http://doc.qt.io/qt-5/qdate.html#setDate">setDate</a>(). The <a href="http://doc.qt.io/qt-5/qdate.html#fromString">fromString</a>() function returns a <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> given a string and a date format which is used to interpret the date within the string.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qdate.html#year">year</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#month">month</a>(), and <a href="http://doc.qt.io/qt-5/qdate.html#day">day</a>() functions provide access to the year, month, and day numbers. Also, <a href="http://doc.qt.io/qt-5/qdate.html#dayOfWeek">dayOfWeek</a>() and <a href="http://doc.qt.io/qt-5/qdate.html#dayOfYear">dayOfYear</a>() functions are provided. The same information is provided in textual format by the <a href="http://doc.qt.io/qt-5/qdate.html#toString">toString</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#shortDayName">shortDayName</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#longDayName">longDayName</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#shortMonthName">shortMonthName</a>(), and <a href="http://doc.qt.io/qt-5/qdate.html#longMonthName">longMonthName</a>() functions.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> provides a full set of operators to compare two <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> objects where smaller means earlier, and larger means later.</p>
    /// <p>You can increment (or decrement) a date by a given number of days using <a href="http://doc.qt.io/qt-5/qdate.html#addDays">addDays</a>(). Similarly you can use <a href="http://doc.qt.io/qt-5/qdate.html#addMonths">addMonths</a>() and <a href="http://doc.qt.io/qt-5/qdate.html#addYears">addYears</a>(). The <a href="http://doc.qt.io/qt-5/qdate.html#daysTo">daysTo</a>() function returns the number of days between two dates.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qdate.html#daysInMonth">daysInMonth</a>() and <a href="http://doc.qt.io/qt-5/qdate.html#daysInYear">daysInYear</a>() functions return how many days there are in this date's month and year, respectively. The <a href="http://doc.qt.io/qt-5/qdate.html#isLeapYear">isLeapYear</a>() function indicates whether a date is in a leap year.</p>
    /// <a name="no-year-0"></a>
    /// <a name="no-year-0"></a>
    /// <h4>No Year 0</h4>
    /// <p>There is no year 0. Dates in that year are considered invalid. The year -1 is the year "1 before Christ" or "1 before current era." The day before 1 January 1 CE is 31 December 1 BCE.</p>
    /// <a name="range-of-valid-dates"></a>
    /// <h4>Range of Valid Dates</h4>
    /// <p>Dates are stored internally as a Julian Day number, an integer count of every day in a contiguous range, with 24 November 4714 BCE in the Gregorian calendar being Julian Day 0 (1 January 4713 BCE in the Julian calendar). As well as being an efficient and accurate way of storing an absolute date, it is suitable for converting a Date into other calendar systems such as Hebrew, Islamic or Chinese. The Julian Day number can be obtained using <a href="http://doc.qt.io/qt-5/qdate.html#toJulianDay">QDate::toJulianDay</a>() and can be set using <a href="http://doc.qt.io/qt-5/qdate.html#fromJulianDay">QDate::fromJulianDay</a>().</p>
    /// <p>The range of dates able to be stored by <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> as a Julian Day number is for technical reasons limited to between -784350574879 and 784354017364, which means from before 2 billion BCE to after 2 billion CE.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDate>>,
    ) -> ::cpp_utils::MutRef<crate::QDate> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDate_operator_2(
            self as *mut crate::QDate,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDate>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the current date, as reported by the system clock.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDate QDate::currentDate()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#currentDate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current date, as reported by the system clock.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#currentTime">QTime::currentTime</a>() and <a href="http://doc.qt.io/qt-5/qdatetime.html#currentDateTime">QDateTime::currentDateTime</a>().</p></div>
    pub unsafe fn current_date() -> ::cpp_utils::CppBox<crate::QDate> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDate_currentDate();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the day of the month (1 to 31) of this date.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QDate::day() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#day">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the day of the month (1 to 31) of this date.</p>
    /// <p>Returns 0 if the date is invalid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#year">year</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#month">month</a>(), and <a href="http://doc.qt.io/qt-5/qdate.html#dayOfWeek">dayOfWeek</a>().</p></div>
    pub unsafe fn day(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QDate_day(self as *const crate::QDate)
    }

    /// <p>Returns the weekday (1 = Monday to 7 = Sunday) for this date.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QDate::dayOfWeek() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#dayOfWeek">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the weekday (1 = Monday to 7 = Sunday) for this date.</p>
    /// <p>Returns 0 if the date is invalid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#day">day</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#dayOfYear">dayOfYear</a>(), and <a href="http://doc.qt.io/qt-5/qt.html#DayOfWeek-enum">Qt::DayOfWeek</a>.</p></div>
    pub unsafe fn day_of_week(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QDate_dayOfWeek(self as *const crate::QDate)
    }

    /// <p>Returns the day of the year (1 to 365 or 366 on leap years) for this date.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QDate::dayOfYear() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#dayOfYear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the day of the year (1 to 365 or 366 on leap years) for this date.</p>
    /// <p>Returns 0 if the date is invalid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#day">day</a>() and <a href="http://doc.qt.io/qt-5/qdate.html#dayOfWeek">dayOfWeek</a>().</p></div>
    pub unsafe fn day_of_year(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QDate_dayOfYear(self as *const crate::QDate)
    }

    /// <p>Returns the number of days in the month (28 to 31) for this date.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QDate::daysInMonth() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#daysInMonth">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of days in the month (28 to 31) for this date.</p>
    /// <p>Returns 0 if the date is invalid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#day">day</a>() and <a href="http://doc.qt.io/qt-5/qdate.html#daysInYear">daysInYear</a>().</p></div>
    pub unsafe fn days_in_month(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QDate_daysInMonth(self as *const crate::QDate)
    }

    /// <p>Returns the number of days in the year (365 or 366) for this date.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QDate::daysInYear() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#daysInYear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of days in the year (365 or 366) for this date.</p>
    /// <p>Returns 0 if the date is invalid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#day">day</a>() and <a href="http://doc.qt.io/qt-5/qdate.html#daysInMonth">daysInMonth</a>().</p></div>
    pub unsafe fn days_in_year(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QDate_daysInYear(self as *const crate::QDate)
    }

    /// <p>Returns the number of days from this date to <i>d</i> (which is negative if <i>d</i> is earlier than this date).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QDate::daysTo(const QDate& arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#daysTo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of days from this date to <i>d</i> (which is negative if <i>d</i> is earlier than this date).</p>
    /// <p>Returns 0 if either date is invalid.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html#QDate">QDate</a></span> d1(<span class="number">1995</span><span class="operator">,</span> <span class="number">5</span><span class="operator">,</span> <span class="number">17</span>);  <span class="comment">// May 17, 1995</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html#QDate">QDate</a></span> d2(<span class="number">1995</span><span class="operator">,</span> <span class="number">5</span><span class="operator">,</span> <span class="number">20</span>);  <span class="comment">// May 20, 1995</span>
    ///   d1<span class="operator">.</span>daysTo(d2);          <span class="comment">// returns 3</span>
    ///   d2<span class="operator">.</span>daysTo(d1);          <span class="comment">// returns -3</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#addDays">addDays</a>().</p></div>
    pub unsafe fn days_to(
        &self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDate>>,
    ) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QDate_daysTo(
            self as *const crate::QDate,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDate>>::cast_into(arg1).as_raw_ptr(),
        )
    }

    /// <p>Converts the Julian day <i>jd</i> to a <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDate QDate::fromJulianDay(qint64 jd_)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#fromJulianDay">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the Julian day <i>jd</i> to a <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#toJulianDay">toJulianDay</a>().</p></div>
    pub unsafe fn from_julian_day(jd: i64) -> ::cpp_utils::CppBox<crate::QDate> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDate_fromJulianDay(jd);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> represented by the <i>string</i>, using the <i>format</i> given, or an invalid date if the string cannot be parsed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDate QDate::fromString(const QString& s, Qt::DateFormat f = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#fromString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> represented by the <i>string</i>, using the <i>format</i> given, or an invalid date if the string cannot be parsed.</p>
    /// <p>Note for <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::TextDate</a>: It is recommended that you use the English short month names (e.g. "Jan"). Although localized month names can also be used, they depend on the user's locale settings.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#toString">toString</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toDate">QLocale::toDate</a>().</p></div>
    pub unsafe fn from_string_q_string_date_format(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        f: crate::DateFormat,
    ) -> ::cpp_utils::CppBox<crate::QDate> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDate_fromString(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            f,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> represented by the <i>string</i>, using the <i>format</i> given, or an invalid date if the string cannot be parsed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDate QDate::fromString(const QString& s, const QString& format)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#fromString-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> represented by the <i>string</i>, using the <i>format</i> given, or an invalid date if the string cannot be parsed.</p>
    /// <p>These expressions may be used for the format:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Expression</th><th>Output</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>d</td><td>The day as a number without a leading zero (1 to 31)</td></tr>
    /// <tr valign="top" class="even"><td>dd</td><td>The day as a number with a leading zero (01 to 31)</td></tr>
    /// <tr valign="top" class="odd"><td>ddd</td><td>The abbreviated localized day name (e.g. 'Mon' to 'Sun'). Uses the system locale to localize the name, i.e. <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().</td></tr>
    /// <tr valign="top" class="even"><td>dddd</td><td>The long localized day name (e.g. 'Monday' to 'Sunday'). Uses the system locale to localize the name, i.e. <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().</td></tr>
    /// <tr valign="top" class="odd"><td>M</td><td>The month as a number without a leading zero (1 to 12)</td></tr>
    /// <tr valign="top" class="even"><td>MM</td><td>The month as a number with a leading zero (01 to 12)</td></tr>
    /// <tr valign="top" class="odd"><td>MMM</td><td>The abbreviated localized month name (e.g. 'Jan' to 'Dec'). Uses the system locale to localize the name, i.e. <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().</td></tr>
    /// <tr valign="top" class="even"><td>MMMM</td><td>The long localized month name (e.g. 'January' to 'December'). Uses the system locale to localize the name, i.e. <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().</td></tr>
    /// <tr valign="top" class="odd"><td>yy</td><td>The year as two digit number (00 to 99)</td></tr>
    /// <tr valign="top" class="even"><td>yyyy</td><td>The year as four digit number. If the year is negative, a minus sign is prepended in addition.</td></tr>
    /// </tbody></table></div>
    /// <p>All other input characters will be treated as text. Any sequence of characters that are enclosed in single quotes will also be treated as text and will not be used as an expression. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html#QDate">QDate</a></span> date <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html#QDate">QDate</a></span><span class="operator">::</span>fromString(<span class="string">"1MM12car2003"</span><span class="operator">,</span> <span class="string">"d'MM'MMcaryyyy"</span>);
    ///   <span class="comment">// date is 1 December 2003</span>
    ///
    /// </pre>
    /// <p>If the format is not satisfied, an invalid <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> is returned. The expressions that don't expect leading zeroes (d, M) will be greedy. This means that they will use two digits even if this will put them outside the accepted range of values and leaves too few digits for other sections. For example, the following format string could have meant January 30 but the M will grab two digits, resulting in an invalid date:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html#QDate">QDate</a></span> date <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html#QDate">QDate</a></span><span class="operator">::</span>fromString(<span class="string">"130"</span><span class="operator">,</span> <span class="string">"Md"</span>); <span class="comment">// invalid</span>
    ///
    /// </pre>
    /// <p>For any field that is not represented in the format the following defaults are used:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Field</th><th>Default value</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>Year</td><td>1900</td></tr>
    /// <tr valign="top" class="even"><td>Month</td><td>1</td></tr>
    /// <tr valign="top" class="odd"><td>Day</td><td>1</td></tr>
    /// </tbody></table></div>
    /// <p>The following examples demonstrate the default values:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html#QDate">QDate</a></span><span class="operator">::</span>fromString(<span class="string">"1.30"</span><span class="operator">,</span> <span class="string">"M.d"</span>);           <span class="comment">// January 30 1900</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html#QDate">QDate</a></span><span class="operator">::</span>fromString(<span class="string">"20000110"</span><span class="operator">,</span> <span class="string">"yyyyMMdd"</span>);  <span class="comment">// January 10, 2000</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html#QDate">QDate</a></span><span class="operator">::</span>fromString(<span class="string">"20000110"</span><span class="operator">,</span> <span class="string">"yyyyMd"</span>);    <span class="comment">// January 10, 2000</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#toString">toString</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#fromString">QDateTime::fromString</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#fromString">QTime::fromString</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toDate">QLocale::toDate</a>().</p></div>
    pub unsafe fn from_string_2_q_string(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        format: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QDate> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDate_fromString1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(format)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> represented by the <i>string</i>, using the <i>format</i> given, or an invalid date if the string cannot be parsed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDate QDate::fromString(const QString& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#fromString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> represented by the <i>string</i>, using the <i>format</i> given, or an invalid date if the string cannot be parsed.</p>
    /// <p>Note for <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::TextDate</a>: It is recommended that you use the English short month names (e.g. "Jan"). Although localized month names can also be used, they depend on the user's locale settings.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#toString">toString</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toDate">QLocale::toDate</a>().</p></div>
    pub unsafe fn from_string_q_string(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QDate> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDate_fromString2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Extracts the date's year, month, and day, and assigns them to *<i>year</i>, *<i>month</i>, and *<i>day</i>. The pointers may be null.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDate::getDate(int* year, int* month, int* day)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#getDate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Extracts the date's year, month, and day, and assigns them to *<i>year</i>, *<i>month</i>, and *<i>day</i>. The pointers may be null.</p>
    /// <p>Returns 0 if the date is invalid.</p>
    /// <p><b>Note: </b>In Qt versions prior to 5.7, this function is marked as non-<code>const</code>.</p><p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#year">year</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#month">month</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#day">day</a>(), and <a href="http://doc.qt.io/qt-5/qdate.html#isValid">isValid</a>().</p></div>
    pub unsafe fn get_date_mut(
        &mut self,
        year: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_int>>,
        month: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_int>>,
        day: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_int>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QDate_getDate(
            self as *mut crate::QDate,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_int>>::cast_into(year)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_int>>::cast_into(month)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_int>>::cast_into(day)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Extracts the date's year, month, and day, and assigns them to *<i>year</i>, *<i>month</i>, and *<i>day</i>. The pointers may be null.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDate::getDate(int* year, int* month, int* day) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#getDate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Extracts the date's year, month, and day, and assigns them to *<i>year</i>, *<i>month</i>, and *<i>day</i>. The pointers may be null.</p>
    /// <p>Returns 0 if the date is invalid.</p>
    /// <p><b>Note: </b>In Qt versions prior to 5.7, this function is marked as non-<code>const</code>.</p><p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#year">year</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#month">month</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#day">day</a>(), and <a href="http://doc.qt.io/qt-5/qdate.html#isValid">isValid</a>().</p></div>
    pub unsafe fn get_date(
        &self,
        year: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_int>>,
        month: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_int>>,
        day: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_int>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QDate_getDate1(
            self as *const crate::QDate,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_int>>::cast_into(year)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_int>>::cast_into(month)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_int>>::cast_into(day)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if the specified <i>year</i> is a leap year; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QDate::isLeapYear(int year)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#isLeapYear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the specified <i>year</i> is a leap year; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn is_leap_year(year: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDate_isLeapYear(year)
    }

    /// <p>Returns <code>true</code> if the date is null; otherwise returns <code>false</code>. A null date is invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDate::isNull() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#isNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the date is null; otherwise returns <code>false</code>. A null date is invalid.</p>
    /// <p><b>Note: </b>The behavior of this function is equivalent to <a href="http://doc.qt.io/qt-5/qdate.html#isValid">isValid</a>().</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#isValid">isValid</a>().</p></div>
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDate_isNull(self as *const crate::QDate)
    }

    /// <p>Returns <code>true</code> if this date is valid; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDate::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this date is valid; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#isNull">isNull</a>().</p></div>
    pub unsafe fn is_valid_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDate_isValid(self as *const crate::QDate)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QDate::isValid(int y, int m, int d)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#isValid-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the specified date (<i>year</i>, <i>month</i>, and <i>day</i>) is valid; otherwise returns <code>false</code>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html#QDate">QDate</a></span><span class="operator">::</span>isValid(<span class="number">2002</span><span class="operator">,</span> <span class="number">5</span><span class="operator">,</span> <span class="number">17</span>);  <span class="comment">// true</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html#QDate">QDate</a></span><span class="operator">::</span>isValid(<span class="number">2002</span><span class="operator">,</span> <span class="number">2</span><span class="operator">,</span> <span class="number">30</span>);  <span class="comment">// false (Feb 30 does not exist)</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html#QDate">QDate</a></span><span class="operator">::</span>isValid(<span class="number">2004</span><span class="operator">,</span> <span class="number">2</span><span class="operator">,</span> <span class="number">29</span>);  <span class="comment">// true (2004 is a leap year)</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html#QDate">QDate</a></span><span class="operator">::</span>isValid(<span class="number">2000</span><span class="operator">,</span> <span class="number">2</span><span class="operator">,</span> <span class="number">29</span>);  <span class="comment">// true (2000 is a leap year)</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html#QDate">QDate</a></span><span class="operator">::</span>isValid(<span class="number">2006</span><span class="operator">,</span> <span class="number">2</span><span class="operator">,</span> <span class="number">29</span>);  <span class="comment">// false (2006 is not a leap year)</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html#QDate">QDate</a></span><span class="operator">::</span>isValid(<span class="number">2100</span><span class="operator">,</span> <span class="number">2</span><span class="operator">,</span> <span class="number">29</span>);  <span class="comment">// false (2100 is not a leap year)</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html#QDate">QDate</a></span><span class="operator">::</span>isValid(<span class="number">1202</span><span class="operator">,</span> <span class="number">6</span><span class="operator">,</span> <span class="number">6</span>);   <span class="comment">// true (even though 1202 is pre-Gregorian)</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#isNull">isNull</a>() and <a href="http://doc.qt.io/qt-5/qdate.html#setDate">setDate</a>().</p></div>
    pub unsafe fn is_valid_3a(
        y: ::std::os::raw::c_int,
        m: ::std::os::raw::c_int,
        d: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDate_isValid1(y, m, d)
    }

    /// <p>Returns the long name of the <i>weekday</i> for the representation specified by <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QDate::longDayName(int weekday, QDate::MonthNameType type = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#longDayName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the long name of the <i>weekday</i> for the representation specified by <i>type</i>.</p>
    /// <p>The days are enumerated using the following convention:</p>
    /// <ul>
    /// <li>1 = "Monday"</li>
    /// <li>2 = "Tuesday"</li>
    /// <li>3 = "Wednesday"</li>
    /// <li>4 = "Thursday"</li>
    /// <li>5 = "Friday"</li>
    /// <li>6 = "Saturday"</li>
    /// <li>7 = "Sunday"</li>
    /// </ul>
    /// <p>The day names will be localized according to the system's locale settings, i.e. using <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().</p>
    /// <p>Returns an empty string if the date is invalid.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#toString">toString</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#shortDayName">shortDayName</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#shortMonthName">shortMonthName</a>(), and <a href="http://doc.qt.io/qt-5/qdate.html#longMonthName">longMonthName</a>().</p></div>
    pub unsafe fn long_day_name_2a(
        weekday: ::std::os::raw::c_int,
        type_: crate::q_date::MonthNameType,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDate_longDayName(weekday, type_);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the long name of the <i>weekday</i> for the representation specified by <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QDate::longDayName(int weekday)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#longDayName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the long name of the <i>weekday</i> for the representation specified by <i>type</i>.</p>
    /// <p>The days are enumerated using the following convention:</p>
    /// <ul>
    /// <li>1 = "Monday"</li>
    /// <li>2 = "Tuesday"</li>
    /// <li>3 = "Wednesday"</li>
    /// <li>4 = "Thursday"</li>
    /// <li>5 = "Friday"</li>
    /// <li>6 = "Saturday"</li>
    /// <li>7 = "Sunday"</li>
    /// </ul>
    /// <p>The day names will be localized according to the system's locale settings, i.e. using <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().</p>
    /// <p>Returns an empty string if the date is invalid.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#toString">toString</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#shortDayName">shortDayName</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#shortMonthName">shortMonthName</a>(), and <a href="http://doc.qt.io/qt-5/qdate.html#longMonthName">longMonthName</a>().</p></div>
    pub unsafe fn long_day_name_1a(
        weekday: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDate_longDayName1(weekday);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the long name of the <i>month</i> for the representation specified by <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QDate::longMonthName(int month, QDate::MonthNameType type = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#longMonthName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the long name of the <i>month</i> for the representation specified by <i>type</i>.</p>
    /// <p>The months are enumerated using the following convention:</p>
    /// <ul>
    /// <li>1 = "January"</li>
    /// <li>2 = "February"</li>
    /// <li>3 = "March"</li>
    /// <li>4 = "April"</li>
    /// <li>5 = "May"</li>
    /// <li>6 = "June"</li>
    /// <li>7 = "July"</li>
    /// <li>8 = "August"</li>
    /// <li>9 = "September"</li>
    /// <li>10 = "October"</li>
    /// <li>11 = "November"</li>
    /// <li>12 = "December"</li>
    /// </ul>
    /// <p>The month names will be localized according to the system's locale settings, i.e. using <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().</p>
    /// <p>Returns an empty string if the date is invalid.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#toString">toString</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#shortMonthName">shortMonthName</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#shortDayName">shortDayName</a>(), and <a href="http://doc.qt.io/qt-5/qdate.html#longDayName">longDayName</a>().</p></div>
    pub unsafe fn long_month_name_2a(
        month: ::std::os::raw::c_int,
        type_: crate::q_date::MonthNameType,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDate_longMonthName(month, type_);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the long name of the <i>month</i> for the representation specified by <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QDate::longMonthName(int month)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#longMonthName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the long name of the <i>month</i> for the representation specified by <i>type</i>.</p>
    /// <p>The months are enumerated using the following convention:</p>
    /// <ul>
    /// <li>1 = "January"</li>
    /// <li>2 = "February"</li>
    /// <li>3 = "March"</li>
    /// <li>4 = "April"</li>
    /// <li>5 = "May"</li>
    /// <li>6 = "June"</li>
    /// <li>7 = "July"</li>
    /// <li>8 = "August"</li>
    /// <li>9 = "September"</li>
    /// <li>10 = "October"</li>
    /// <li>11 = "November"</li>
    /// <li>12 = "December"</li>
    /// </ul>
    /// <p>The month names will be localized according to the system's locale settings, i.e. using <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().</p>
    /// <p>Returns an empty string if the date is invalid.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#toString">toString</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#shortMonthName">shortMonthName</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#shortDayName">shortDayName</a>(), and <a href="http://doc.qt.io/qt-5/qdate.html#longDayName">longDayName</a>().</p></div>
    pub unsafe fn long_month_name_1a(
        month: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDate_longMonthName1(month);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the number corresponding to the month of this date, using the following convention:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QDate::month() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#month">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number corresponding to the month of this date, using the following convention:</p>
    /// <ul>
    /// <li>1 = "January"</li>
    /// <li>2 = "February"</li>
    /// <li>3 = "March"</li>
    /// <li>4 = "April"</li>
    /// <li>5 = "May"</li>
    /// <li>6 = "June"</li>
    /// <li>7 = "July"</li>
    /// <li>8 = "August"</li>
    /// <li>9 = "September"</li>
    /// <li>10 = "October"</li>
    /// <li>11 = "November"</li>
    /// <li>12 = "December"</li>
    /// </ul>
    /// <p>Returns 0 if the date is invalid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#year">year</a>() and <a href="http://doc.qt.io/qt-5/qdate.html#day">day</a>().</p></div>
    pub unsafe fn month(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QDate_month(self as *const crate::QDate)
    }

    /// <p>Constructs a null date. Null dates are invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDate::QDate()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#QDate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a null date. Null dates are invalid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#isNull">isNull</a>() and <a href="http://doc.qt.io/qt-5/qdate.html#isValid">isValid</a>().</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QDate> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDate_QDate();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a date with year <i>y</i>, month <i>m</i> and day <i>d</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDate::QDate(int y, int m, int d)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#QDate-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a date with year <i>y</i>, month <i>m</i> and day <i>d</i>.</p>
    /// <p>If the specified date is invalid, the date is not set and <a href="http://doc.qt.io/qt-5/qdate.html#isValid">isValid</a>() returns <code>false</code>.</p>
    /// <p><b>Warning:</b> Years 1 to 99 are interpreted as is. Year 0 is invalid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#isValid">isValid</a>().</p></div>
    pub unsafe fn new_3a(
        y: ::std::os::raw::c_int,
        m: ::std::os::raw::c_int,
        d: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QDate> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDate_QDate1(y, m, d);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> class provides date functions.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDate::QDate(const QDate& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> class provides date functions.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> object contains a calendar date, i.e. year, month, and day numbers, in the Gregorian calendar. It can read the current date from the system clock. It provides functions for comparing dates, and for manipulating dates. For example, it is possible to add and subtract days, months, and years to dates.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> object is typically created by giving the year, month, and day numbers explicitly. Note that <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> interprets two digit years as is, i.e., years 0 - 99. A <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> can also be constructed with the static function <a href="http://doc.qt.io/qt-5/qdate.html#currentDate">currentDate</a>(), which creates a <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> object containing the system clock's date. An explicit date can also be set using <a href="http://doc.qt.io/qt-5/qdate.html#setDate">setDate</a>(). The <a href="http://doc.qt.io/qt-5/qdate.html#fromString">fromString</a>() function returns a <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> given a string and a date format which is used to interpret the date within the string.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qdate.html#year">year</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#month">month</a>(), and <a href="http://doc.qt.io/qt-5/qdate.html#day">day</a>() functions provide access to the year, month, and day numbers. Also, <a href="http://doc.qt.io/qt-5/qdate.html#dayOfWeek">dayOfWeek</a>() and <a href="http://doc.qt.io/qt-5/qdate.html#dayOfYear">dayOfYear</a>() functions are provided. The same information is provided in textual format by the <a href="http://doc.qt.io/qt-5/qdate.html#toString">toString</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#shortDayName">shortDayName</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#longDayName">longDayName</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#shortMonthName">shortMonthName</a>(), and <a href="http://doc.qt.io/qt-5/qdate.html#longMonthName">longMonthName</a>() functions.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> provides a full set of operators to compare two <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> objects where smaller means earlier, and larger means later.</p>
    /// <p>You can increment (or decrement) a date by a given number of days using <a href="http://doc.qt.io/qt-5/qdate.html#addDays">addDays</a>(). Similarly you can use <a href="http://doc.qt.io/qt-5/qdate.html#addMonths">addMonths</a>() and <a href="http://doc.qt.io/qt-5/qdate.html#addYears">addYears</a>(). The <a href="http://doc.qt.io/qt-5/qdate.html#daysTo">daysTo</a>() function returns the number of days between two dates.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qdate.html#daysInMonth">daysInMonth</a>() and <a href="http://doc.qt.io/qt-5/qdate.html#daysInYear">daysInYear</a>() functions return how many days there are in this date's month and year, respectively. The <a href="http://doc.qt.io/qt-5/qdate.html#isLeapYear">isLeapYear</a>() function indicates whether a date is in a leap year.</p>
    /// <a name="no-year-0"></a>
    /// <a name="no-year-0"></a>
    /// <h4>No Year 0</h4>
    /// <p>There is no year 0. Dates in that year are considered invalid. The year -1 is the year "1 before Christ" or "1 before current era." The day before 1 January 1 CE is 31 December 1 BCE.</p>
    /// <a name="range-of-valid-dates"></a>
    /// <h4>Range of Valid Dates</h4>
    /// <p>Dates are stored internally as a Julian Day number, an integer count of every day in a contiguous range, with 24 November 4714 BCE in the Gregorian calendar being Julian Day 0 (1 January 4713 BCE in the Julian calendar). As well as being an efficient and accurate way of storing an absolute date, it is suitable for converting a Date into other calendar systems such as Hebrew, Islamic or Chinese. The Julian Day number can be obtained using <a href="http://doc.qt.io/qt-5/qdate.html#toJulianDay">QDate::toJulianDay</a>() and can be set using <a href="http://doc.qt.io/qt-5/qdate.html#fromJulianDay">QDate::fromJulianDay</a>().</p>
    /// <p>The range of dates able to be stored by <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> as a Julian Day number is for technical reasons limited to between -784350574879 and 784354017364, which means from before 2 billion BCE to after 2 billion CE.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDate>>,
    ) -> ::cpp_utils::CppBox<crate::QDate> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDate_QDate2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDate>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Sets the date's <i>year</i>, <i>month</i>, and <i>day</i>. Returns <code>true</code> if the date is valid; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDate::setDate(int year, int month, int day)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#setDate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the date's <i>year</i>, <i>month</i>, and <i>day</i>. Returns <code>true</code> if the date is valid; otherwise returns <code>false</code>.</p>
    /// <p>If the specified date is invalid, the <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> object is set to be invalid.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#isValid">isValid</a>().</p></div>
    pub unsafe fn set_date(
        &mut self,
        year: ::std::os::raw::c_int,
        month: ::std::os::raw::c_int,
        day: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDate_setDate(self as *mut crate::QDate, year, month, day)
    }

    /// <p>Returns the short name of the <i>weekday</i> for the representation specified by <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QDate::shortDayName(int weekday, QDate::MonthNameType type = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#shortDayName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the short name of the <i>weekday</i> for the representation specified by <i>type</i>.</p>
    /// <p>The days are enumerated using the following convention:</p>
    /// <ul>
    /// <li>1 = "Mon"</li>
    /// <li>2 = "Tue"</li>
    /// <li>3 = "Wed"</li>
    /// <li>4 = "Thu"</li>
    /// <li>5 = "Fri"</li>
    /// <li>6 = "Sat"</li>
    /// <li>7 = "Sun"</li>
    /// </ul>
    /// <p>The day names will be localized according to the system's locale settings, i.e. using <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().</p>
    /// <p>Returns an empty string if the date is invalid.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#toString">toString</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#shortMonthName">shortMonthName</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#longMonthName">longMonthName</a>(), and <a href="http://doc.qt.io/qt-5/qdate.html#longDayName">longDayName</a>().</p></div>
    pub unsafe fn short_day_name_2a(
        weekday: ::std::os::raw::c_int,
        type_: crate::q_date::MonthNameType,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDate_shortDayName(weekday, type_);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the short name of the <i>weekday</i> for the representation specified by <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QDate::shortDayName(int weekday)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#shortDayName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the short name of the <i>weekday</i> for the representation specified by <i>type</i>.</p>
    /// <p>The days are enumerated using the following convention:</p>
    /// <ul>
    /// <li>1 = "Mon"</li>
    /// <li>2 = "Tue"</li>
    /// <li>3 = "Wed"</li>
    /// <li>4 = "Thu"</li>
    /// <li>5 = "Fri"</li>
    /// <li>6 = "Sat"</li>
    /// <li>7 = "Sun"</li>
    /// </ul>
    /// <p>The day names will be localized according to the system's locale settings, i.e. using <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().</p>
    /// <p>Returns an empty string if the date is invalid.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#toString">toString</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#shortMonthName">shortMonthName</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#longMonthName">longMonthName</a>(), and <a href="http://doc.qt.io/qt-5/qdate.html#longDayName">longDayName</a>().</p></div>
    pub unsafe fn short_day_name_1a(
        weekday: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDate_shortDayName1(weekday);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the short name of the <i>month</i> for the representation specified by <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QDate::shortMonthName(int month, QDate::MonthNameType type = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#shortMonthName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the short name of the <i>month</i> for the representation specified by <i>type</i>.</p>
    /// <p>The months are enumerated using the following convention:</p>
    /// <ul>
    /// <li>1 = "Jan"</li>
    /// <li>2 = "Feb"</li>
    /// <li>3 = "Mar"</li>
    /// <li>4 = "Apr"</li>
    /// <li>5 = "May"</li>
    /// <li>6 = "Jun"</li>
    /// <li>7 = "Jul"</li>
    /// <li>8 = "Aug"</li>
    /// <li>9 = "Sep"</li>
    /// <li>10 = "Oct"</li>
    /// <li>11 = "Nov"</li>
    /// <li>12 = "Dec"</li>
    /// </ul>
    /// <p>The month names will be localized according to the system's locale settings, i.e. using <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().</p>
    /// <p>Returns an empty string if the date is invalid.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#toString">toString</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#longMonthName">longMonthName</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#shortDayName">shortDayName</a>(), and <a href="http://doc.qt.io/qt-5/qdate.html#longDayName">longDayName</a>().</p></div>
    pub unsafe fn short_month_name_2a(
        month: ::std::os::raw::c_int,
        type_: crate::q_date::MonthNameType,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDate_shortMonthName(month, type_);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the short name of the <i>month</i> for the representation specified by <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QDate::shortMonthName(int month)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#shortMonthName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the short name of the <i>month</i> for the representation specified by <i>type</i>.</p>
    /// <p>The months are enumerated using the following convention:</p>
    /// <ul>
    /// <li>1 = "Jan"</li>
    /// <li>2 = "Feb"</li>
    /// <li>3 = "Mar"</li>
    /// <li>4 = "Apr"</li>
    /// <li>5 = "May"</li>
    /// <li>6 = "Jun"</li>
    /// <li>7 = "Jul"</li>
    /// <li>8 = "Aug"</li>
    /// <li>9 = "Sep"</li>
    /// <li>10 = "Oct"</li>
    /// <li>11 = "Nov"</li>
    /// <li>12 = "Dec"</li>
    /// </ul>
    /// <p>The month names will be localized according to the system's locale settings, i.e. using <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().</p>
    /// <p>Returns an empty string if the date is invalid.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#toString">toString</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#longMonthName">longMonthName</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#shortDayName">shortDayName</a>(), and <a href="http://doc.qt.io/qt-5/qdate.html#longDayName">longDayName</a>().</p></div>
    pub unsafe fn short_month_name_1a(
        month: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDate_shortMonthName1(month);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts the date to a Julian day.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QDate::toJulianDay() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#toJulianDay">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the date to a Julian day.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#fromJulianDay">fromJulianDay</a>().</p></div>
    pub unsafe fn to_julian_day(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QDate_toJulianDay(self as *const crate::QDate)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QDate::toString(Qt::DateFormat f = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#toString-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the date as a string. The <i>format</i> parameter determines the format of the string.</p>
    /// <p>If the <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::TextDate</a>, the string is formatted in the default way. <a href="http://doc.qt.io/qt-5/qdate.html#shortDayName">QDate::shortDayName</a>() and <a href="http://doc.qt.io/qt-5/qdate.html#shortMonthName">QDate::shortMonthName</a>() are used to generate the string, so the day and month names will be localized names using the system locale, i.e. <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>(). An example of this formatting is "Sat May 20 1995".</p>
    /// <p>If the <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::ISODate</a>, the string format corresponds to the ISO 8601 extended specification for representations of dates and times, taking the form YYYY-MM-DD, where YYYY is the year, MM is the month of the year (between 01 and 12), and DD is the day of the month between 01 and 31.</p>
    /// <p>If the <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::SystemLocaleShortDate</a> or <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::SystemLocaleLongDate</a>, the string format depends on the locale settings of the system. Identical to calling <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().<a href="http://doc.qt.io/qt-5/qdate.html#toString">toString</a>(date, <a href="http://doc.qt.io/qt-5/qlocale.html#FormatType-enum">QLocale::ShortFormat</a>) or <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().<a href="http://doc.qt.io/qt-5/qdate.html#toString">toString</a>(date, <a href="http://doc.qt.io/qt-5/qlocale.html#FormatType-enum">QLocale::LongFormat</a>).</p>
    /// <p>If the <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::DefaultLocaleShortDate</a> or <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::DefaultLocaleLongDate</a>, the string format depends on the default application locale. This is the locale set with <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">QLocale::setDefault</a>(), or the system locale if no default locale has been set. Identical to calling <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale().toString(date, QLocale::ShortFormat) </a> or <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale().toString(date, QLocale::LongFormat)</a>.</p>
    /// <p>If the <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::RFC2822Date</a>, the string is formatted in an <a href="http://www.rfc-editor.org/rfc/rfc2822.txt">RFC 2822</a> compatible way. An example of this formatting is "20 May 1995".</p>
    /// <p>If the date is invalid, an empty string will be returned.</p>
    /// <p><b>Warning:</b> The <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::ISODate</a> format is only valid for years in the range 0 to 9999. This restriction may apply to locale-aware formats as well, depending on the locale settings.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#fromString">fromString</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#shortDayName">shortDayName</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#shortMonthName">shortMonthName</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>().</p></div>
    pub unsafe fn to_string_date_format(
        &self,
        f: crate::DateFormat,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDate_toString(self as *const crate::QDate, f);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the date as a string. The <i>format</i> parameter determines the format of the result string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QDate::toString(const QString& format) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#toString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the date as a string. The <i>format</i> parameter determines the format of the result string.</p>
    /// <p>These expressions may be used:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Expression</th><th>Output</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>d</td><td>the day as number without a leading zero (1 to 31)</td></tr>
    /// <tr valign="top" class="even"><td>dd</td><td>the day as number with a leading zero (01 to 31)</td></tr>
    /// <tr valign="top" class="odd"><td>ddd</td><td>the abbreviated localized day name (e.g. 'Mon' to 'Sun'). Uses the system locale to localize the name, i.e. <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().</td></tr>
    /// <tr valign="top" class="even"><td>dddd</td><td>the long localized day name (e.g. 'Monday' to 'Sunday'). Uses the system locale to localize the name, i.e. <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().</td></tr>
    /// <tr valign="top" class="odd"><td>M</td><td>the month as number without a leading zero (1 to 12)</td></tr>
    /// <tr valign="top" class="even"><td>MM</td><td>the month as number with a leading zero (01 to 12)</td></tr>
    /// <tr valign="top" class="odd"><td>MMM</td><td>the abbreviated localized month name (e.g. 'Jan' to 'Dec'). Uses the system locale to localize the name, i.e. <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().</td></tr>
    /// <tr valign="top" class="even"><td>MMMM</td><td>the long localized month name (e.g. 'January' to 'December'). Uses the system locale to localize the name, i.e. <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().</td></tr>
    /// <tr valign="top" class="odd"><td>yy</td><td>the year as two digit number (00 to 99)</td></tr>
    /// <tr valign="top" class="even"><td>yyyy</td><td>the year as four digit number. If the year is negative, a minus sign is prepended in addition.</td></tr>
    /// </tbody></table></div>
    /// <p>All other input characters will be ignored. Any sequence of characters that are enclosed in single quotes will be treated as text and not be used as an expression. Two consecutive single quotes ("''") are replaced by a singlequote in the output. Formats without separators (e.g. "ddMM") are currently not supported.</p>
    /// <p>Example format strings (assuming that the <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> is the 20 July 1969):</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Format</th><th>Result</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>dd.MM.yyyy</td><td>20.07.1969</td></tr>
    /// <tr valign="top" class="even"><td>ddd MMMM d yy</td><td>Sun July 20 69</td></tr>
    /// <tr valign="top" class="odd"><td>'The day is' dddd</td><td>The day is Sunday</td></tr>
    /// </tbody></table></div>
    /// <p>If the datetime is invalid, an empty string will be returned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#fromString">fromString</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#toString">QDateTime::toString</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#toString">QTime::toString</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>().</p></div>
    pub unsafe fn to_string_q_string(
        &self,
        format: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDate_toString1(
            self as *const crate::QDate,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(format)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QDate::toString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#toString-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the date as a string. The <i>format</i> parameter determines the format of the string.</p>
    /// <p>If the <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::TextDate</a>, the string is formatted in the default way. <a href="http://doc.qt.io/qt-5/qdate.html#shortDayName">QDate::shortDayName</a>() and <a href="http://doc.qt.io/qt-5/qdate.html#shortMonthName">QDate::shortMonthName</a>() are used to generate the string, so the day and month names will be localized names using the system locale, i.e. <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>(). An example of this formatting is "Sat May 20 1995".</p>
    /// <p>If the <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::ISODate</a>, the string format corresponds to the ISO 8601 extended specification for representations of dates and times, taking the form YYYY-MM-DD, where YYYY is the year, MM is the month of the year (between 01 and 12), and DD is the day of the month between 01 and 31.</p>
    /// <p>If the <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::SystemLocaleShortDate</a> or <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::SystemLocaleLongDate</a>, the string format depends on the locale settings of the system. Identical to calling <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().<a href="http://doc.qt.io/qt-5/qdate.html#toString">toString</a>(date, <a href="http://doc.qt.io/qt-5/qlocale.html#FormatType-enum">QLocale::ShortFormat</a>) or <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().<a href="http://doc.qt.io/qt-5/qdate.html#toString">toString</a>(date, <a href="http://doc.qt.io/qt-5/qlocale.html#FormatType-enum">QLocale::LongFormat</a>).</p>
    /// <p>If the <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::DefaultLocaleShortDate</a> or <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::DefaultLocaleLongDate</a>, the string format depends on the default application locale. This is the locale set with <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">QLocale::setDefault</a>(), or the system locale if no default locale has been set. Identical to calling <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale().toString(date, QLocale::ShortFormat) </a> or <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale().toString(date, QLocale::LongFormat)</a>.</p>
    /// <p>If the <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::RFC2822Date</a>, the string is formatted in an <a href="http://www.rfc-editor.org/rfc/rfc2822.txt">RFC 2822</a> compatible way. An example of this formatting is "20 May 1995".</p>
    /// <p>If the date is invalid, an empty string will be returned.</p>
    /// <p><b>Warning:</b> The <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::ISODate</a> format is only valid for years in the range 0 to 9999. This restriction may apply to locale-aware formats as well, depending on the locale settings.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#fromString">fromString</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#shortDayName">shortDayName</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#shortMonthName">shortMonthName</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>().</p></div>
    pub unsafe fn to_string(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDate_toString2(self as *const crate::QDate);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QString QDate::toString(QStringView format) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_string_q_string_view(
        &self,
        format: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDate_toString6(
            self as *const crate::QDate,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(format)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the week number (1 to 53), and stores the year in *<i>yearNumber</i> unless <i>yearNumber</i> is null (the default).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QDate::weekNumber(int* yearNum = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#weekNumber">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the week number (1 to 53), and stores the year in *<i>yearNumber</i> unless <i>yearNumber</i> is null (the default).</p>
    /// <p>Returns 0 if the date is invalid.</p>
    /// <p>In accordance with ISO 8601, weeks start on Monday and the first Thursday of a year is always in week 1 of that year. Most years have 52 weeks, but some have 53.</p>
    /// <p>*<i>yearNumber</i> is not always the same as <a href="http://doc.qt.io/qt-5/qdate.html#year">year</a>(). For example, 1 January 2000 has week number 52 in the year 1999, and 31 December 2002 has week number 1 in the year 2003.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#isValid">isValid</a>().</p></div>
    pub unsafe fn week_number_1a(
        &self,
        year_num: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_int>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QDate_weekNumber(
            self as *const crate::QDate,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_int>>::cast_into(
                year_num,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the week number (1 to 53), and stores the year in *<i>yearNumber</i> unless <i>yearNumber</i> is null (the default).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QDate::weekNumber() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#weekNumber">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the week number (1 to 53), and stores the year in *<i>yearNumber</i> unless <i>yearNumber</i> is null (the default).</p>
    /// <p>Returns 0 if the date is invalid.</p>
    /// <p>In accordance with ISO 8601, weeks start on Monday and the first Thursday of a year is always in week 1 of that year. Most years have 52 weeks, but some have 53.</p>
    /// <p>*<i>yearNumber</i> is not always the same as <a href="http://doc.qt.io/qt-5/qdate.html#year">year</a>(). For example, 1 January 2000 has week number 52 in the year 1999, and 31 December 2002 has week number 1 in the year 2003.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#isValid">isValid</a>().</p></div>
    pub unsafe fn week_number_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QDate_weekNumber1(self as *const crate::QDate)
    }

    /// <p>Returns the year of this date. Negative numbers indicate years before 1 CE, such that year -44 is 44 BCE.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QDate::year() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#year">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the year of this date. Negative numbers indicate years before 1 CE, such that year -44 is 44 BCE.</p>
    /// <p>Returns 0 if the date is invalid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#month">month</a>() and <a href="http://doc.qt.io/qt-5/qdate.html#day">day</a>().</p></div>
    pub unsafe fn year(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QDate_year(self as *const crate::QDate)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> class provides clock time functions.</p>
///
/// C++ class: <span style='color: green;'>```QTime```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtime.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> class provides clock time functions.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> object contains a clock time, i.e. the number of hours, minutes, seconds, and milliseconds since midnight. It can read the current time from the system clock and measure a span of elapsed time. It provides functions for comparing times and for manipulating a time by adding a number of milliseconds.</p>
/// <p><a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> uses the 24-hour clock format; it has no concept of AM/PM. Unlike <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>, <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> knows nothing about time zones or daylight-saving time (DST).</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> object is typically created either by giving the number of hours, minutes, seconds, and milliseconds explicitly, or by using the static function <a href="http://doc.qt.io/qt-5/qtime.html#currentTime">currentTime</a>(), which creates a <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> object that contains the system's local time. Note that the accuracy depends on the accuracy of the underlying operating system; not all systems provide 1-millisecond accuracy.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qtime.html#hour">hour</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#minute">minute</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#second">second</a>(), and <a href="http://doc.qt.io/qt-5/qtime.html#msec">msec</a>() functions provide access to the number of hours, minutes, seconds, and milliseconds of the time. The same information is provided in textual format by the <a href="http://doc.qt.io/qt-5/qtime.html#toString">toString</a>() function.</p>
/// <p><a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> provides a full set of operators to compare two <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> objects. <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> A is considered smaller than <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> B if A is earlier than B.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qtime.html#addSecs">addSecs</a>() and <a href="http://doc.qt.io/qt-5/qtime.html#addMSecs">addMSecs</a>() functions provide the time a given number of seconds or milliseconds later than a given time. Correspondingly, the number of seconds or milliseconds between two times can be found using <a href="http://doc.qt.io/qt-5/qtime.html#secsTo">secsTo</a>() or <a href="http://doc.qt.io/qt-5/qtime.html#msecsTo">msecsTo</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> can be used to measure a span of elapsed time using the <a href="http://doc.qt.io/qt-5/qtime.html#start">start</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#restart">restart</a>(), and <a href="http://doc.qt.io/qt-5/qtime.html#elapsed">elapsed</a>() functions.</p></div>
#[repr(C)]
pub struct QTime {
    _unused: u8,
}
impl QTime {
    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> object containing a time <i>ms</i> milliseconds later than the time of this object (or earlier if <i>ms</i> is negative).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTime QTime::addMSecs(int ms) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#addMSecs">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> object containing a time <i>ms</i> milliseconds later than the time of this object (or earlier if <i>ms</i> is negative).</p>
    /// <p>Note that the time will wrap if it passes midnight. See <a href="http://doc.qt.io/qt-5/qtime.html#addSecs">addSecs</a>() for an example.</p>
    /// <p>Returns a null time if this time is invalid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#addSecs">addSecs</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#msecsTo">msecsTo</a>(), and <a href="http://doc.qt.io/qt-5/qdatetime.html#addMSecs">QDateTime::addMSecs</a>().</p></div>
    pub unsafe fn add_m_secs(
        &self,
        ms: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QTime> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QTime_addMSecs(self as *const crate::QTime, ms);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> object containing a time <i>s</i> seconds later than the time of this object (or earlier if <i>s</i> is negative).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTime QTime::addSecs(int secs) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#addSecs">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> object containing a time <i>s</i> seconds later than the time of this object (or earlier if <i>s</i> is negative).</p>
    /// <p>Note that the time will wrap if it passes midnight.</p>
    /// <p>Returns a null time if this time is invalid.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtime.html#QTime">QTime</a></span> n(<span class="number">14</span><span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="number">0</span>);                <span class="comment">// n == 14:00:00</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtime.html#QTime">QTime</a></span> t;
    ///   t <span class="operator">=</span> n<span class="operator">.</span>addSecs(<span class="number">70</span>);                <span class="comment">// t == 14:01:10</span>
    ///   t <span class="operator">=</span> n<span class="operator">.</span>addSecs(<span class="operator">-</span><span class="number">70</span>);               <span class="comment">// t == 13:58:50</span>
    ///   t <span class="operator">=</span> n<span class="operator">.</span>addSecs(<span class="number">10</span> <span class="operator">*</span> <span class="number">60</span> <span class="operator">*</span> <span class="number">60</span> <span class="operator">+</span> <span class="number">5</span>);  <span class="comment">// t == 00:00:05</span>
    ///   t <span class="operator">=</span> n<span class="operator">.</span>addSecs(<span class="operator">-</span><span class="number">15</span> <span class="operator">*</span> <span class="number">60</span> <span class="operator">*</span> <span class="number">60</span>);     <span class="comment">// t == 23:00:00</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#addMSecs">addMSecs</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#secsTo">secsTo</a>(), and <a href="http://doc.qt.io/qt-5/qdatetime.html#addSecs">QDateTime::addSecs</a>().</p></div>
    pub unsafe fn add_secs(
        &self,
        secs: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QTime> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QTime_addSecs(self as *const crate::QTime, secs);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> class provides clock time functions.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTime& QTime::operator=(const QTime& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> class provides clock time functions.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> object contains a clock time, i.e. the number of hours, minutes, seconds, and milliseconds since midnight. It can read the current time from the system clock and measure a span of elapsed time. It provides functions for comparing times and for manipulating a time by adding a number of milliseconds.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> uses the 24-hour clock format; it has no concept of AM/PM. Unlike <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>, <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> knows nothing about time zones or daylight-saving time (DST).</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> object is typically created either by giving the number of hours, minutes, seconds, and milliseconds explicitly, or by using the static function <a href="http://doc.qt.io/qt-5/qtime.html#currentTime">currentTime</a>(), which creates a <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> object that contains the system's local time. Note that the accuracy depends on the accuracy of the underlying operating system; not all systems provide 1-millisecond accuracy.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qtime.html#hour">hour</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#minute">minute</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#second">second</a>(), and <a href="http://doc.qt.io/qt-5/qtime.html#msec">msec</a>() functions provide access to the number of hours, minutes, seconds, and milliseconds of the time. The same information is provided in textual format by the <a href="http://doc.qt.io/qt-5/qtime.html#toString">toString</a>() function.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> provides a full set of operators to compare two <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> objects. <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> A is considered smaller than <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> B if A is earlier than B.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qtime.html#addSecs">addSecs</a>() and <a href="http://doc.qt.io/qt-5/qtime.html#addMSecs">addMSecs</a>() functions provide the time a given number of seconds or milliseconds later than a given time. Correspondingly, the number of seconds or milliseconds between two times can be found using <a href="http://doc.qt.io/qt-5/qtime.html#secsTo">secsTo</a>() or <a href="http://doc.qt.io/qt-5/qtime.html#msecsTo">msecsTo</a>().</p>
    /// <p><a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> can be used to measure a span of elapsed time using the <a href="http://doc.qt.io/qt-5/qtime.html#start">start</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#restart">restart</a>(), and <a href="http://doc.qt.io/qt-5/qtime.html#elapsed">elapsed</a>() functions.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTime>>,
    ) -> ::cpp_utils::MutRef<crate::QTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTime_operator_2(
            self as *mut crate::QTime,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTime>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the current time as reported by the system clock.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QTime QTime::currentTime()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#currentTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current time as reported by the system clock.</p>
    /// <p>Note that the accuracy depends on the accuracy of the underlying operating system; not all systems provide 1-millisecond accuracy.</p></div>
    pub unsafe fn current_time() -> ::cpp_utils::CppBox<crate::QTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTime_currentTime();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the number of milliseconds that have elapsed since the last time <a href="http://doc.qt.io/qt-5/qtime.html#start">start</a>() or <a href="http://doc.qt.io/qt-5/qtime.html#restart">restart</a>() was called.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTime::elapsed() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#elapsed">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of milliseconds that have elapsed since the last time <a href="http://doc.qt.io/qt-5/qtime.html#start">start</a>() or <a href="http://doc.qt.io/qt-5/qtime.html#restart">restart</a>() was called.</p>
    /// <p>Note that the counter wraps to zero 24 hours after the last call to <a href="http://doc.qt.io/qt-5/qtime.html#start">start</a>() or restart.</p>
    /// <p>Note that the accuracy depends on the accuracy of the underlying operating system; not all systems provide 1-millisecond accuracy.</p>
    /// <p><b>Warning:</b> If the system's clock setting has been changed since the last time <a href="http://doc.qt.io/qt-5/qtime.html#start">start</a>() or <a href="http://doc.qt.io/qt-5/qtime.html#restart">restart</a>() was called, the result is undefined. This can happen when daylight-saving time is turned on or off.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#start">start</a>() and <a href="http://doc.qt.io/qt-5/qtime.html#restart">restart</a>().</p></div>
    pub unsafe fn elapsed(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTime_elapsed(self as *const crate::QTime)
    }

    /// <p>Returns a new <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> instance with the time set to the number of <i>msecs</i> since the start of the day, i.e. since 00:00:00.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QTime QTime::fromMSecsSinceStartOfDay(int msecs)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#fromMSecsSinceStartOfDay">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a new <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> instance with the time set to the number of <i>msecs</i> since the start of the day, i.e. since 00:00:00.</p>
    /// <p>If <i>msecs</i> falls outside the valid range an invalid <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> will be returned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#msecsSinceStartOfDay">msecsSinceStartOfDay</a>().</p></div>
    pub unsafe fn from_m_secs_since_start_of_day(
        msecs: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTime_fromMSecsSinceStartOfDay(msecs);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the time represented in the <i>string</i> as a <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> using the <i>format</i> given, or an invalid time if this is not possible.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QTime QTime::fromString(const QString& s, Qt::DateFormat f = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#fromString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the time represented in the <i>string</i> as a <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> using the <i>format</i> given, or an invalid time if this is not possible.</p>
    /// <p>Note that fromString() uses a "C" locale encoded string to convert milliseconds to a float value. If the default locale is not "C", this may result in two conversion attempts (if the conversion fails for the default locale). This should be considered an implementation detail.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#toString">toString</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toTime">QLocale::toTime</a>().</p></div>
    pub unsafe fn from_string_q_string_date_format(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        f: crate::DateFormat,
    ) -> ::cpp_utils::CppBox<crate::QTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTime_fromString(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            f,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> represented by the <i>string</i>, using the <i>format</i> given, or an invalid time if the string cannot be parsed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QTime QTime::fromString(const QString& s, const QString& format)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#fromString-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> represented by the <i>string</i>, using the <i>format</i> given, or an invalid time if the string cannot be parsed.</p>
    /// <p>These expressions may be used for the format:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Expression</th><th>Output</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>h</td><td>the hour without a leading zero (0 to 23 or 1 to 12 if AM/PM display)</td></tr>
    /// <tr valign="top" class="even"><td>hh</td><td>the hour with a leading zero (00 to 23 or 01 to 12 if AM/PM display)</td></tr>
    /// <tr valign="top" class="odd"><td>m</td><td>the minute without a leading zero (0 to 59)</td></tr>
    /// <tr valign="top" class="even"><td>mm</td><td>the minute with a leading zero (00 to 59)</td></tr>
    /// <tr valign="top" class="odd"><td>s</td><td>the whole second, without any leading zero (0 to 59)</td></tr>
    /// <tr valign="top" class="even"><td>ss</td><td>the whole second, with a leading zero where applicable (00 to 59)</td></tr>
    /// <tr valign="top" class="odd"><td>z</td><td>the fractional part of the second, to go after a decimal point, without trailing zeroes (0 to 999). Thus "<code>s.z</code>" reports the seconds to full available (millisecond) precision without trailing zeroes.</td></tr>
    /// <tr valign="top" class="even"><td>zzz</td><td>the fractional part of the second, to millisecond precision, including trailing zeroes where applicable (000 to 999).</td></tr>
    /// <tr valign="top" class="odd"><td>AP</td><td>interpret as an AM/PM time. <i>AP</i> must be either "AM" or "PM".</td></tr>
    /// <tr valign="top" class="even"><td>ap</td><td>Interpret as an AM/PM time. <i>ap</i> must be either "am" or "pm".</td></tr>
    /// </tbody></table></div>
    /// <p>All other input characters will be treated as text. Any sequence of characters that are enclosed in single quotes will also be treated as text and not be used as an expression.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtime.html#QTime">QTime</a></span> time <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtime.html#QTime">QTime</a></span><span class="operator">::</span>fromString(<span class="string">"1mm12car00"</span><span class="operator">,</span> <span class="string">"m'mm'hcarss"</span>);
    ///   <span class="comment">// time is 12:01.00</span>
    ///
    /// </pre>
    /// <p>If the format is not satisfied, an invalid <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> is returned. Expressions that do not expect leading zeroes to be given (h, m, s and z) are greedy. This means that they will use two digits even if this puts them outside the range of accepted values and leaves too few digits for other sections. For example, the following string could have meant 00:07:10, but the m will grab two digits, resulting in an invalid time:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtime.html#QTime">QTime</a></span> time <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtime.html#QTime">QTime</a></span><span class="operator">::</span>fromString(<span class="string">"00:710"</span><span class="operator">,</span> <span class="string">"hh:ms"</span>); <span class="comment">// invalid</span>
    ///
    /// </pre>
    /// <p>Any field that is not represented in the format will be set to zero. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtime.html#QTime">QTime</a></span> time <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtime.html#QTime">QTime</a></span><span class="operator">::</span>fromString(<span class="string">"1.30"</span><span class="operator">,</span> <span class="string">"m.s"</span>);
    ///   <span class="comment">// time is 00:01:30.000</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#toString">toString</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#fromString">QDateTime::fromString</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#fromString">QDate::fromString</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toTime">QLocale::toTime</a>().</p></div>
    pub unsafe fn from_string_2_q_string(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        format: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTime_fromString1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(format)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the time represented in the <i>string</i> as a <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> using the <i>format</i> given, or an invalid time if this is not possible.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QTime QTime::fromString(const QString& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#fromString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the time represented in the <i>string</i> as a <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> using the <i>format</i> given, or an invalid time if this is not possible.</p>
    /// <p>Note that fromString() uses a "C" locale encoded string to convert milliseconds to a float value. If the default locale is not "C", this may result in two conversion attempts (if the conversion fails for the default locale). This should be considered an implementation detail.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#toString">toString</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toTime">QLocale::toTime</a>().</p></div>
    pub unsafe fn from_string_q_string(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTime_fromString2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the hour part (0 to 23) of the time.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTime::hour() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#hour">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hour part (0 to 23) of the time.</p>
    /// <p>Returns -1 if the time is invalid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#minute">minute</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#second">second</a>(), and <a href="http://doc.qt.io/qt-5/qtime.html#msec">msec</a>().</p></div>
    pub unsafe fn hour(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTime_hour(self as *const crate::QTime)
    }

    /// <p>Returns <code>true</code> if the time is null (i.e., the <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> object was constructed using the default constructor); otherwise returns false. A null time is also an invalid time.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTime::isNull() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#isNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the time is null (i.e., the <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> object was constructed using the default constructor); otherwise returns false. A null time is also an invalid time.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#isValid">isValid</a>().</p></div>
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTime_isNull(self as *const crate::QTime)
    }

    /// <p>Returns <code>true</code> if the time is valid; otherwise returns <code>false</code>. For example, the time 23:30:55.746 is valid, but 24:12:30 is invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTime::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the time is valid; otherwise returns <code>false</code>. For example, the time 23:30:55.746 is valid, but 24:12:30 is invalid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#isNull">isNull</a>().</p></div>
    pub unsafe fn is_valid_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTime_isValid(self as *const crate::QTime)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QTime::isValid(int h, int m, int s, int ms = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#isValid-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the specified time is valid; otherwise returns false.</p>
    /// <p>The time is valid if <i>h</i> is in the range 0 to 23, <i>m</i> and <i>s</i> are in the range 0 to 59, and <i>ms</i> is in the range 0 to 999.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtime.html#QTime">QTime</a></span><span class="operator">::</span>isValid(<span class="number">21</span><span class="operator">,</span> <span class="number">10</span><span class="operator">,</span> <span class="number">30</span>); <span class="comment">// returns true</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtime.html#QTime">QTime</a></span><span class="operator">::</span>isValid(<span class="number">22</span><span class="operator">,</span> <span class="number">5</span><span class="operator">,</span>  <span class="number">62</span>); <span class="comment">// returns false</span>
    ///
    /// </pre></div>
    pub unsafe fn is_valid_4a(
        h: ::std::os::raw::c_int,
        m: ::std::os::raw::c_int,
        s: ::std::os::raw::c_int,
        ms: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTime_isValid1(h, m, s, ms)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QTime::isValid(int h, int m, int s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#isValid-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the specified time is valid; otherwise returns false.</p>
    /// <p>The time is valid if <i>h</i> is in the range 0 to 23, <i>m</i> and <i>s</i> are in the range 0 to 59, and <i>ms</i> is in the range 0 to 999.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtime.html#QTime">QTime</a></span><span class="operator">::</span>isValid(<span class="number">21</span><span class="operator">,</span> <span class="number">10</span><span class="operator">,</span> <span class="number">30</span>); <span class="comment">// returns true</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtime.html#QTime">QTime</a></span><span class="operator">::</span>isValid(<span class="number">22</span><span class="operator">,</span> <span class="number">5</span><span class="operator">,</span>  <span class="number">62</span>); <span class="comment">// returns false</span>
    ///
    /// </pre></div>
    pub unsafe fn is_valid_3a(
        h: ::std::os::raw::c_int,
        m: ::std::os::raw::c_int,
        s: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTime_isValid2(h, m, s)
    }

    /// <p>Returns the minute part (0 to 59) of the time.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTime::minute() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#minute">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the minute part (0 to 59) of the time.</p>
    /// <p>Returns -1 if the time is invalid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#hour">hour</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#second">second</a>(), and <a href="http://doc.qt.io/qt-5/qtime.html#msec">msec</a>().</p></div>
    pub unsafe fn minute(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTime_minute(self as *const crate::QTime)
    }

    /// <p>Returns the millisecond part (0 to 999) of the time.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTime::msec() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#msec">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the millisecond part (0 to 999) of the time.</p>
    /// <p>Returns -1 if the time is invalid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#hour">hour</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#minute">minute</a>(), and <a href="http://doc.qt.io/qt-5/qtime.html#second">second</a>().</p></div>
    pub unsafe fn msec(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTime_msec(self as *const crate::QTime)
    }

    /// <p>Returns the number of msecs since the start of the day, i.e. since 00:00:00.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTime::msecsSinceStartOfDay() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#msecsSinceStartOfDay">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of msecs since the start of the day, i.e. since 00:00:00.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#fromMSecsSinceStartOfDay">fromMSecsSinceStartOfDay</a>().</p></div>
    pub unsafe fn msecs_since_start_of_day(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTime_msecsSinceStartOfDay(self as *const crate::QTime)
    }

    /// <p>Returns the number of milliseconds from this time to <i>t</i>. If <i>t</i> is earlier than this time, the number of milliseconds returned is negative.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTime::msecsTo(const QTime& arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#msecsTo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of milliseconds from this time to <i>t</i>. If <i>t</i> is earlier than this time, the number of milliseconds returned is negative.</p>
    /// <p>Because <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> measures time within a day and there are 86400 seconds in a day, the result is always between -86400000 and 86400000 ms.</p>
    /// <p>Returns 0 if either time is invalid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#secsTo">secsTo</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#addMSecs">addMSecs</a>(), and <a href="http://doc.qt.io/qt-5/qdatetime.html#msecsTo">QDateTime::msecsTo</a>().</p></div>
    pub unsafe fn msecs_to(
        &self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTime>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTime_msecsTo(
            self as *const crate::QTime,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTime>>::cast_into(arg1).as_raw_ptr(),
        )
    }

    /// <p>Constructs a null time object. A null time can be a <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a>(0, 0, 0, 0) (i.e., midnight) object, except that <a href="http://doc.qt.io/qt-5/qtime.html#isNull">isNull</a>() returns <code>true</code> and <a href="http://doc.qt.io/qt-5/qtime.html#isValid">isValid</a>() returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTime::QTime()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#QTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a null time object. A null time can be a <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a>(0, 0, 0, 0) (i.e., midnight) object, except that <a href="http://doc.qt.io/qt-5/qtime.html#isNull">isNull</a>() returns <code>true</code> and <a href="http://doc.qt.io/qt-5/qtime.html#isValid">isValid</a>() returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#isNull">isNull</a>() and <a href="http://doc.qt.io/qt-5/qtime.html#isValid">isValid</a>().</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTime_QTime();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a time with hour <i>h</i>, minute <i>m</i>, seconds <i>s</i> and milliseconds <i>ms</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTime::QTime(int h, int m, int s = …, int ms = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#QTime-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a time with hour <i>h</i>, minute <i>m</i>, seconds <i>s</i> and milliseconds <i>ms</i>.</p>
    /// <p><i>h</i> must be in the range 0 to 23, <i>m</i> and <i>s</i> must be in the range 0 to 59, and <i>ms</i> must be in the range 0 to 999.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#isValid">isValid</a>().</p></div>
    pub unsafe fn new_4a(
        h: ::std::os::raw::c_int,
        m: ::std::os::raw::c_int,
        s: ::std::os::raw::c_int,
        ms: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTime_QTime1(h, m, s, ms);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a time with hour <i>h</i>, minute <i>m</i>, seconds <i>s</i> and milliseconds <i>ms</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTime::QTime(int h, int m, int s = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#QTime-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a time with hour <i>h</i>, minute <i>m</i>, seconds <i>s</i> and milliseconds <i>ms</i>.</p>
    /// <p><i>h</i> must be in the range 0 to 23, <i>m</i> and <i>s</i> must be in the range 0 to 59, and <i>ms</i> must be in the range 0 to 999.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#isValid">isValid</a>().</p></div>
    pub unsafe fn new_3a(
        h: ::std::os::raw::c_int,
        m: ::std::os::raw::c_int,
        s: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTime_QTime3(h, m, s);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a time with hour <i>h</i>, minute <i>m</i>, seconds <i>s</i> and milliseconds <i>ms</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTime::QTime(int h, int m)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#QTime-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a time with hour <i>h</i>, minute <i>m</i>, seconds <i>s</i> and milliseconds <i>ms</i>.</p>
    /// <p><i>h</i> must be in the range 0 to 23, <i>m</i> and <i>s</i> must be in the range 0 to 59, and <i>ms</i> must be in the range 0 to 999.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#isValid">isValid</a>().</p></div>
    pub unsafe fn new_2a(
        h: ::std::os::raw::c_int,
        m: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTime_QTime4(h, m);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> class provides clock time functions.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTime::QTime(const QTime& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> class provides clock time functions.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> object contains a clock time, i.e. the number of hours, minutes, seconds, and milliseconds since midnight. It can read the current time from the system clock and measure a span of elapsed time. It provides functions for comparing times and for manipulating a time by adding a number of milliseconds.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> uses the 24-hour clock format; it has no concept of AM/PM. Unlike <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>, <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> knows nothing about time zones or daylight-saving time (DST).</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> object is typically created either by giving the number of hours, minutes, seconds, and milliseconds explicitly, or by using the static function <a href="http://doc.qt.io/qt-5/qtime.html#currentTime">currentTime</a>(), which creates a <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> object that contains the system's local time. Note that the accuracy depends on the accuracy of the underlying operating system; not all systems provide 1-millisecond accuracy.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qtime.html#hour">hour</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#minute">minute</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#second">second</a>(), and <a href="http://doc.qt.io/qt-5/qtime.html#msec">msec</a>() functions provide access to the number of hours, minutes, seconds, and milliseconds of the time. The same information is provided in textual format by the <a href="http://doc.qt.io/qt-5/qtime.html#toString">toString</a>() function.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> provides a full set of operators to compare two <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> objects. <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> A is considered smaller than <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> B if A is earlier than B.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qtime.html#addSecs">addSecs</a>() and <a href="http://doc.qt.io/qt-5/qtime.html#addMSecs">addMSecs</a>() functions provide the time a given number of seconds or milliseconds later than a given time. Correspondingly, the number of seconds or milliseconds between two times can be found using <a href="http://doc.qt.io/qt-5/qtime.html#secsTo">secsTo</a>() or <a href="http://doc.qt.io/qt-5/qtime.html#msecsTo">msecsTo</a>().</p>
    /// <p><a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> can be used to measure a span of elapsed time using the <a href="http://doc.qt.io/qt-5/qtime.html#start">start</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#restart">restart</a>(), and <a href="http://doc.qt.io/qt-5/qtime.html#elapsed">elapsed</a>() functions.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTime>>,
    ) -> ::cpp_utils::CppBox<crate::QTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTime_QTime2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTime>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Sets this time to the current time and returns the number of milliseconds that have elapsed since the last time <a href="http://doc.qt.io/qt-5/qtime.html#start">start</a>() or restart() was called.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTime::restart()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#restart">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets this time to the current time and returns the number of milliseconds that have elapsed since the last time <a href="http://doc.qt.io/qt-5/qtime.html#start">start</a>() or restart() was called.</p>
    /// <p>This function is guaranteed to be atomic and is thus very handy for repeated measurements. Call <a href="http://doc.qt.io/qt-5/qtime.html#start">start</a>() to start the first measurement, and restart() for each later measurement.</p>
    /// <p>Note that the counter wraps to zero 24 hours after the last call to <a href="http://doc.qt.io/qt-5/qtime.html#start">start</a>() or restart().</p>
    /// <p><b>Warning:</b> If the system's clock setting has been changed since the last time <a href="http://doc.qt.io/qt-5/qtime.html#start">start</a>() or restart() was called, the result is undefined. This can happen when daylight-saving time is turned on or off.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#start">start</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#elapsed">elapsed</a>(), and <a href="http://doc.qt.io/qt-5/qtime.html#currentTime">currentTime</a>().</p></div>
    pub unsafe fn restart(&mut self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTime_restart(self as *mut crate::QTime)
    }

    /// <p>Returns the second part (0 to 59) of the time.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTime::second() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#second">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the second part (0 to 59) of the time.</p>
    /// <p>Returns -1 if the time is invalid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#hour">hour</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#minute">minute</a>(), and <a href="http://doc.qt.io/qt-5/qtime.html#msec">msec</a>().</p></div>
    pub unsafe fn second(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTime_second(self as *const crate::QTime)
    }

    /// <p>Returns the number of seconds from this time to <i>t</i>. If <i>t</i> is earlier than this time, the number of seconds returned is negative.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTime::secsTo(const QTime& arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#secsTo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of seconds from this time to <i>t</i>. If <i>t</i> is earlier than this time, the number of seconds returned is negative.</p>
    /// <p>Because <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> measures time within a day and there are 86400 seconds in a day, the result is always between -86400 and 86400.</p>
    /// <p>secsTo() does not take into account any milliseconds.</p>
    /// <p>Returns 0 if either time is invalid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#addSecs">addSecs</a>() and <a href="http://doc.qt.io/qt-5/qdatetime.html#secsTo">QDateTime::secsTo</a>().</p></div>
    pub unsafe fn secs_to(
        &self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTime>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTime_secsTo(
            self as *const crate::QTime,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTime>>::cast_into(arg1).as_raw_ptr(),
        )
    }

    /// <p>Sets the time to hour <i>h</i>, minute <i>m</i>, seconds <i>s</i> and milliseconds <i>ms</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTime::setHMS(int h, int m, int s, int ms = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#setHMS">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the time to hour <i>h</i>, minute <i>m</i>, seconds <i>s</i> and milliseconds <i>ms</i>.</p>
    /// <p><i>h</i> must be in the range 0 to 23, <i>m</i> and <i>s</i> must be in the range 0 to 59, and <i>ms</i> must be in the range 0 to 999. Returns <code>true</code> if the set time is valid; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#isValid">isValid</a>().</p></div>
    pub unsafe fn set_h_m_s_4a(
        &mut self,
        h: ::std::os::raw::c_int,
        m: ::std::os::raw::c_int,
        s: ::std::os::raw::c_int,
        ms: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTime_setHMS(self as *mut crate::QTime, h, m, s, ms)
    }

    /// <p>Sets the time to hour <i>h</i>, minute <i>m</i>, seconds <i>s</i> and milliseconds <i>ms</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTime::setHMS(int h, int m, int s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#setHMS">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the time to hour <i>h</i>, minute <i>m</i>, seconds <i>s</i> and milliseconds <i>ms</i>.</p>
    /// <p><i>h</i> must be in the range 0 to 23, <i>m</i> and <i>s</i> must be in the range 0 to 59, and <i>ms</i> must be in the range 0 to 999. Returns <code>true</code> if the set time is valid; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#isValid">isValid</a>().</p></div>
    pub unsafe fn set_h_m_s_3a(
        &mut self,
        h: ::std::os::raw::c_int,
        m: ::std::os::raw::c_int,
        s: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTime_setHMS1(self as *mut crate::QTime, h, m, s)
    }

    /// <p>Sets this time to the current time. This is practical for timing:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTime::start()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#start">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets this time to the current time. This is practical for timing:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtime.html#QTime">QTime</a></span> t;
    ///   t<span class="operator">.</span>start();
    ///   some_lengthy_task();
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>(<span class="string">"Time elapsed: %d ms"</span><span class="operator">,</span> t<span class="operator">.</span>elapsed());
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#restart">restart</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#elapsed">elapsed</a>(), and <a href="http://doc.qt.io/qt-5/qtime.html#currentTime">currentTime</a>().</p></div>
    pub unsafe fn start(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QTime_start(self as *mut crate::QTime)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QTime::toString(Qt::DateFormat f = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#toString-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the time as a string. The <i>format</i> parameter determines the format of the string.</p>
    /// <p>If <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::TextDate</a>, the string format is HH:mm:ss; e.g. 1 second before midnight would be "23:59:59".</p>
    /// <p>If <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::ISODate</a>, the string format corresponds to the ISO 8601 extended specification for representations of dates, represented by HH:mm:ss. To include milliseconds in the ISO 8601 date, use the <i>format</i> <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::ISODateWithMs</a>, which corresponds to HH:mm:ss.zzz.</p>
    /// <p>If the <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::SystemLocaleShortDate</a> or <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::SystemLocaleLongDate</a>, the string format depends on the locale settings of the system. Identical to calling <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().<a href="http://doc.qt.io/qt-5/qtime.html#toString">toString</a>(time, <a href="http://doc.qt.io/qt-5/qlocale.html#FormatType-enum">QLocale::ShortFormat</a>) or <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().<a href="http://doc.qt.io/qt-5/qtime.html#toString">toString</a>(time, <a href="http://doc.qt.io/qt-5/qlocale.html#FormatType-enum">QLocale::LongFormat</a>).</p>
    /// <p>If the <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::DefaultLocaleShortDate</a> or <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::DefaultLocaleLongDate</a>, the string format depends on the default application locale. This is the locale set with <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">QLocale::setDefault</a>(), or the system locale if no default locale has been set. Identical to calling</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale().toString(time, QLocale::ShortFormat)</a> or <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale().toString(time, QLocale::LongFormat)</a>.</p>
    /// <p>If the <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::RFC2822Date</a>, the string is formatted in an <a href="http://www.rfc-editor.org/rfc/rfc2822.txt">RFC 2822</a> compatible way. An example of this formatting is "23:59:20".</p>
    /// <p>If the time is invalid, an empty string will be returned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#fromString">fromString</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#toString">QDate::toString</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#toString">QDateTime::toString</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>().</p></div>
    pub unsafe fn to_string_date_format(
        &self,
        f: crate::DateFormat,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QTime_toString(self as *const crate::QTime, f);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the time as a string. The <i>format</i> parameter determines the format of the result string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QTime::toString(const QString& format) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#toString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the time as a string. The <i>format</i> parameter determines the format of the result string.</p>
    /// <p>These expressions may be used:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Expression</th><th>Output</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>h</td><td>the hour without a leading zero (0 to 23 or 1 to 12 if AM/PM display)</td></tr>
    /// <tr valign="top" class="even"><td>hh</td><td>the hour with a leading zero (00 to 23 or 01 to 12 if AM/PM display)</td></tr>
    /// <tr valign="top" class="odd"><td>H</td><td>the hour without a leading zero (0 to 23, even with AM/PM display)</td></tr>
    /// <tr valign="top" class="even"><td>HH</td><td>the hour with a leading zero (00 to 23, even with AM/PM display)</td></tr>
    /// <tr valign="top" class="odd"><td>m</td><td>the minute without a leading zero (0 to 59)</td></tr>
    /// <tr valign="top" class="even"><td>mm</td><td>the minute with a leading zero (00 to 59)</td></tr>
    /// <tr valign="top" class="odd"><td>s</td><td>the whole second, without any leading zero (0 to 59)</td></tr>
    /// <tr valign="top" class="even"><td>ss</td><td>the whole second, with a leading zero where applicable (00 to 59)</td></tr>
    /// <tr valign="top" class="odd"><td>z</td><td>the fractional part of the second, to go after a decimal point, without trailing zeroes (0 to 999). Thus "<code>s.z</code>" reports the seconds to full available (millisecond) precision without trailing zeroes.</td></tr>
    /// <tr valign="top" class="even"><td>zzz</td><td>the fractional part of the second, to millisecond precision, including trailing zeroes where applicable (000 to 999).</td></tr>
    /// <tr valign="top" class="odd"><td>AP or A</td><td>use AM/PM display. <i>A/AP</i> will be replaced by either <a href="http://doc.qt.io/qt-5/qlocale.html#amText">QLocale::amText</a>() or <a href="http://doc.qt.io/qt-5/qlocale.html#pmText">QLocale::pmText</a>().</td></tr>
    /// <tr valign="top" class="even"><td>ap or a</td><td>use am/pm display. <i>a/ap</i> will be replaced by a lower-case version of <a href="http://doc.qt.io/qt-5/qlocale.html#amText">QLocale::amText</a>() or <a href="http://doc.qt.io/qt-5/qlocale.html#pmText">QLocale::pmText</a>().</td></tr>
    /// <tr valign="top" class="odd"><td>t</td><td>the timezone (for example "CEST")</td></tr>
    /// </tbody></table></div>
    /// <p>All other input characters will be ignored. Any sequence of characters that are enclosed in single quotes will be treated as text and not be used as an expression. Two consecutive single quotes ("''") are replaced by a singlequote in the output. Formats without separators (e.g. "HHmm") are currently not supported.</p>
    /// <p>Example format strings (assuming that the <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> is 14:13:09.042 and the system locale is <code>en_US</code>)</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Format</th><th>Result</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>hh:mm:ss.zzz</td><td>14:13:09.042</td></tr>
    /// <tr valign="top" class="even"><td>h:m:s ap</td><td>2:13:9 pm</td></tr>
    /// <tr valign="top" class="odd"><td>H:m:s a</td><td>14:13:9 pm</td></tr>
    /// </tbody></table></div>
    /// <p>If the time is invalid, an empty string will be returned. If <i>format</i> is empty, the default format "hh:mm:ss" is used.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#fromString">fromString</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#toString">QDate::toString</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#toString">QDateTime::toString</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>().</p></div>
    pub unsafe fn to_string_q_string(
        &self,
        format: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTime_toString1(
            self as *const crate::QTime,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(format)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QTime::toString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#toString-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the time as a string. The <i>format</i> parameter determines the format of the string.</p>
    /// <p>If <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::TextDate</a>, the string format is HH:mm:ss; e.g. 1 second before midnight would be "23:59:59".</p>
    /// <p>If <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::ISODate</a>, the string format corresponds to the ISO 8601 extended specification for representations of dates, represented by HH:mm:ss. To include milliseconds in the ISO 8601 date, use the <i>format</i> <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::ISODateWithMs</a>, which corresponds to HH:mm:ss.zzz.</p>
    /// <p>If the <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::SystemLocaleShortDate</a> or <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::SystemLocaleLongDate</a>, the string format depends on the locale settings of the system. Identical to calling <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().<a href="http://doc.qt.io/qt-5/qtime.html#toString">toString</a>(time, <a href="http://doc.qt.io/qt-5/qlocale.html#FormatType-enum">QLocale::ShortFormat</a>) or <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().<a href="http://doc.qt.io/qt-5/qtime.html#toString">toString</a>(time, <a href="http://doc.qt.io/qt-5/qlocale.html#FormatType-enum">QLocale::LongFormat</a>).</p>
    /// <p>If the <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::DefaultLocaleShortDate</a> or <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::DefaultLocaleLongDate</a>, the string format depends on the default application locale. This is the locale set with <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">QLocale::setDefault</a>(), or the system locale if no default locale has been set. Identical to calling</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale().toString(time, QLocale::ShortFormat)</a> or <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale().toString(time, QLocale::LongFormat)</a>.</p>
    /// <p>If the <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::RFC2822Date</a>, the string is formatted in an <a href="http://www.rfc-editor.org/rfc/rfc2822.txt">RFC 2822</a> compatible way. An example of this formatting is "23:59:20".</p>
    /// <p>If the time is invalid, an empty string will be returned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtime.html#fromString">fromString</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#toString">QDate::toString</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#toString">QDateTime::toString</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>().</p></div>
    pub unsafe fn to_string(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTime_toString2(self as *const crate::QTime);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QString QTime::toString(QStringView format) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_string_q_string_view(
        &self,
        format: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTime_toString6(
            self as *const crate::QTime,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(format)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> class provides date and time functions.</p>
///
/// C++ class: <span style='color: green;'>```QDateTime```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qdatetime.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> class provides date and time functions.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> object contains a calendar date and a clock time (a "datetime"). It is a combination of the <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> and <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> classes. It can read the current datetime from the system clock. It provides functions for comparing datetimes and for manipulating a datetime by adding a number of seconds, days, months, or years.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> object is typically created either by giving a date and time explicitly in the constructor, or by using the static function <a href="http://doc.qt.io/qt-5/qdatetime.html#currentDateTime">currentDateTime</a>() that returns a <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> object set to the system clock's time. The date and time can be changed with <a href="http://doc.qt.io/qt-5/qdatetime.html#setDate">setDate</a>() and <a href="http://doc.qt.io/qt-5/qdatetime.html#setTime">setTime</a>(). A datetime can also be set using the setTime_t() function that takes a POSIX-standard "number of seconds since 00:00:00 on January 1, 1970" value. The <a href="http://doc.qt.io/qt-5/qdatetime.html#fromString">fromString</a>() function returns a <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>, given a string and a date format used to interpret the date within the string.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qdatetime.html#date">date</a>() and <a href="http://doc.qt.io/qt-5/qdatetime.html#time">time</a>() functions provide access to the date and time parts of the datetime. The same information is provided in textual format by the <a href="http://doc.qt.io/qt-5/qdatetime.html#toString">toString</a>() function.</p>
/// <p><a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> provides a full set of operators to compare two <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> objects, where smaller means earlier and larger means later.</p>
/// <p>You can increment (or decrement) a datetime by a given number of milliseconds using <a href="http://doc.qt.io/qt-5/qdatetime.html#addMSecs">addMSecs</a>(), seconds using <a href="http://doc.qt.io/qt-5/qdatetime.html#addSecs">addSecs</a>(), or days using <a href="http://doc.qt.io/qt-5/qdatetime.html#addDays">addDays</a>(). Similarly, you can use <a href="http://doc.qt.io/qt-5/qdatetime.html#addMonths">addMonths</a>() and <a href="http://doc.qt.io/qt-5/qdatetime.html#addYears">addYears</a>(). The <a href="http://doc.qt.io/qt-5/qdatetime.html#daysTo">daysTo</a>() function returns the number of days between two datetimes, <a href="http://doc.qt.io/qt-5/qdatetime.html#secsTo">secsTo</a>() returns the number of seconds between two datetimes, and <a href="http://doc.qt.io/qt-5/qdatetime.html#msecsTo">msecsTo</a>() returns the number of milliseconds between two datetimes.</p>
/// <p><a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> can store datetimes as <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">local time</a> or as <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">UTC</a>. <a href="http://doc.qt.io/qt-5/qdatetime.html#currentDateTime">QDateTime::currentDateTime</a>() returns a <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> expressed as local time; use <a href="http://doc.qt.io/qt-5/qdatetime.html#toUTC">toUTC</a>() to convert it to UTC. You can also use <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>() to find out if a <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> object stores a UTC time or a local time. Operations such as <a href="http://doc.qt.io/qt-5/qdatetime.html#addSecs">addSecs</a>() and <a href="http://doc.qt.io/qt-5/qdatetime.html#secsTo">secsTo</a>() are aware of daylight-saving time (DST).</p>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> does not account for leap seconds.</p><a name="no-year-0"></a>
/// <a name="no-year-0"></a>
/// <h4>No Year 0</h4>
/// <p>There is no year 0. Dates in that year are considered invalid. The year -1 is the year "1 before Christ" or "1 before current era." The day before 1 January 1 CE is 31 December 1 BCE.</p>
/// <a name="range-of-valid-dates"></a>
/// <h4>Range of Valid Dates</h4>
/// <p>The range of valid values able to be stored in <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> is dependent on the internal storage implementation. <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> is currently stored in a qint64 as a serial msecs value encoding the date and time. This restricts the date range to about +/- 292 million years, compared to the <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> range of +/- 2 billion years. Care must be taken when creating a <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> with extreme values that you do not overflow the storage. The exact range of supported values varies depending on the <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeSpec</a> and time zone.</p>
/// <a name="use-of-system-timezone"></a>
/// <h4>Use of System Timezone</h4>
/// <p><a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> uses the system's time zone information to determine the offset of local time from UTC. If the system is not configured correctly or not up-to-date, <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> will give wrong results as well.</p>
/// <a name="daylight-saving-time-dst"></a>
/// <h4>Daylight-Saving Time (DST)</h4>
/// <p><a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> takes into account the system's time zone information when dealing with DST. On modern Unix systems, this means it applies the correct historical DST data whenever possible. On Windows, where the system doesn't support historical DST data, historical accuracy is not maintained with respect to DST.</p>
/// <p>The range of valid dates taking DST into account is 1970-01-01 to the present, and rules are in place for handling DST correctly until 2037-12-31, but these could change. For dates falling outside that range, <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> makes a <i>best guess</i> using the rules for year 1970 or 2037, but we can't guarantee accuracy. This means <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> doesn't take into account changes in a locale's time zone before 1970, even if the system's time zone database supports that information.</p>
/// <p><a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> takes into consideration the Standard Time to Daylight-Saving Time transition. For example if the transition is at 2am and the clock goes forward to 3am, then there is a "missing" hour from 02:00:00 to 02:59:59.999 which <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> considers to be invalid. Any date maths performed will take this missing hour into account and return a valid result.</p>
/// <a name="offset-from-utc"></a>
/// <h4>Offset From UTC</h4>
/// <p>A <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeSpec</a> of <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> is also supported. This allows you to define a <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> relative to UTC at a fixed offset of a given number of seconds from UTC. For example, an offset of +3600 seconds is one hour ahead of UTC and is usually written in ISO standard notation as "UTC+01:00". Daylight-Saving Time never applies with this TimeSpec.</p>
/// <p>There is no explicit size restriction to the offset seconds, but there is an implicit limit imposed when using the <a href="http://doc.qt.io/qt-5/qdatetime.html#toString">toString</a>() and <a href="http://doc.qt.io/qt-5/qdatetime.html#fromString">fromString</a>() methods which use a format of [+|-]hh:mm, effectively limiting the range to +/- 99 hours and 59 minutes and whole minutes only. Note that currently no time zone lies outside the range of +/- 14 hours.</p>
/// <a name="time-zone-support"></a>
/// <h4>Time Zone Support</h4>
/// <p>A <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeSpec</a> of <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeZone</a> is also supported in conjunction with the <a href="http://doc.qt.io/qt-5/qtimezone.html">QTimeZone</a> class. This allows you to define a datetime in a named time zone adhering to a consistent set of daylight-saving transition rules. For example a time zone of "Europe/Berlin" will apply the daylight-saving rules as used in Germany since 1970. Note that the transition rules applied depend on the platform support. See the <a href="http://doc.qt.io/qt-5/qtimezone.html">QTimeZone</a> documentation for more details.</p></div>
#[repr(C)]
pub struct QDateTime {
    _unused: u8,
}
impl QDateTime {
    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> object containing a datetime <i>ndays</i> days later than the datetime of this object (or earlier if <i>ndays</i> is negative).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDateTime QDateTime::addDays(qint64 days) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#addDays">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> object containing a datetime <i>ndays</i> days later than the datetime of this object (or earlier if <i>ndays</i> is negative).</p>
    /// <p>If the <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>() is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::LocalTime</a> and the resulting date and time fall in the Standard Time to Daylight-Saving Time transition hour then the result will be adjusted accordingly, i.e. if the transition is at 2am and the clock goes forward to 3am and the result falls between 2am and 3am then the result will be adjusted to fall after 3am.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#daysTo">daysTo</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#addMonths">addMonths</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#addYears">addYears</a>(), and <a href="http://doc.qt.io/qt-5/qdatetime.html#addSecs">addSecs</a>().</p></div>
    pub unsafe fn add_days(&self, days: i64) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDateTime_addDays(self as *const crate::QDateTime, days);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> object containing a datetime <i>msecs</i> miliseconds later than the datetime of this object (or earlier if <i>msecs</i> is negative).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDateTime QDateTime::addMSecs(qint64 msecs) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#addMSecs">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> object containing a datetime <i>msecs</i> miliseconds later than the datetime of this object (or earlier if <i>msecs</i> is negative).</p>
    /// <p>If this datetime is invalid, an invalid datetime will be returned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#addSecs">addSecs</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#msecsTo">msecsTo</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#addDays">addDays</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#addMonths">addMonths</a>(), and <a href="http://doc.qt.io/qt-5/qdatetime.html#addYears">addYears</a>().</p></div>
    pub unsafe fn add_m_secs(&self, msecs: i64) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_addMSecs(
            self as *const crate::QDateTime,
            msecs,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> object containing a datetime <i>nmonths</i> months later than the datetime of this object (or earlier if <i>nmonths</i> is negative).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDateTime QDateTime::addMonths(int months) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#addMonths">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> object containing a datetime <i>nmonths</i> months later than the datetime of this object (or earlier if <i>nmonths</i> is negative).</p>
    /// <p>If the <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>() is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::LocalTime</a> and the resulting date and time fall in the Standard Time to Daylight-Saving Time transition hour then the result will be adjusted accordingly, i.e. if the transition is at 2am and the clock goes forward to 3am and the result falls between 2am and 3am then the result will be adjusted to fall after 3am.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#daysTo">daysTo</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#addDays">addDays</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#addYears">addYears</a>(), and <a href="http://doc.qt.io/qt-5/qdatetime.html#addSecs">addSecs</a>().</p></div>
    pub unsafe fn add_months(
        &self,
        months: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_addMonths(
            self as *const crate::QDateTime,
            months,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> object containing a datetime <i>s</i> seconds later than the datetime of this object (or earlier if <i>s</i> is negative).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDateTime QDateTime::addSecs(qint64 secs) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#addSecs">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> object containing a datetime <i>s</i> seconds later than the datetime of this object (or earlier if <i>s</i> is negative).</p>
    /// <p>If this datetime is invalid, an invalid datetime will be returned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#addMSecs">addMSecs</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#secsTo">secsTo</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#addDays">addDays</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#addMonths">addMonths</a>(), and <a href="http://doc.qt.io/qt-5/qdatetime.html#addYears">addYears</a>().</p></div>
    pub unsafe fn add_secs(&self, secs: i64) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDateTime_addSecs(self as *const crate::QDateTime, secs);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> object containing a datetime <i>nyears</i> years later than the datetime of this object (or earlier if <i>nyears</i> is negative).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDateTime QDateTime::addYears(int years) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#addYears">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> object containing a datetime <i>nyears</i> years later than the datetime of this object (or earlier if <i>nyears</i> is negative).</p>
    /// <p>If the <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>() is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::LocalTime</a> and the resulting date and time fall in the Standard Time to Daylight-Saving Time transition hour then the result will be adjusted accordingly, i.e. if the transition is at 2am and the clock goes forward to 3am and the result falls between 2am and 3am then the result will be adjusted to fall after 3am.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#daysTo">daysTo</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#addDays">addDays</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#addMonths">addMonths</a>(), and <a href="http://doc.qt.io/qt-5/qdatetime.html#addSecs">addSecs</a>().</p></div>
    pub unsafe fn add_years(
        &self,
        years: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_addYears(
            self as *const crate::QDateTime,
            years,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Makes a copy of the <i>other</i> datetime and returns a reference to the copy.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDateTime& QDateTime::operator=(const QDateTime& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#operator-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes a copy of the <i>other</i> datetime and returns a reference to the copy.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
    ) -> ::cpp_utils::MutRef<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_operator_1(
            self as *mut crate::QDateTime,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the current datetime, as reported by the system clock, in the local time zone.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDateTime QDateTime::currentDateTime()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#currentDateTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current datetime, as reported by the system clock, in the local time zone.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#currentDateTimeUtc">currentDateTimeUtc</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#currentDate">QDate::currentDate</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#currentTime">QTime::currentTime</a>(), and <a href="http://doc.qt.io/qt-5/qdatetime.html#toTimeSpec">toTimeSpec</a>().</p></div>
    pub unsafe fn current_date_time() -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_currentDateTime();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the current datetime, as reported by the system clock, in UTC.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDateTime QDateTime::currentDateTimeUtc()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#currentDateTimeUtc">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current datetime, as reported by the system clock, in UTC.</p>
    /// <p>This function was introduced in  Qt 4.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#currentDateTime">currentDateTime</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#currentDate">QDate::currentDate</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#currentTime">QTime::currentTime</a>(), and <a href="http://doc.qt.io/qt-5/qdatetime.html#toTimeSpec">toTimeSpec</a>().</p></div>
    pub unsafe fn current_date_time_utc() -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_currentDateTimeUtc();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the number of milliseconds since 1970-01-01T00:00:00 Universal Coordinated Time. This number is like the POSIX time_t variable, but expressed in milliseconds instead.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static qint64 QDateTime::currentMSecsSinceEpoch()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#currentMSecsSinceEpoch">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of milliseconds since 1970-01-01T00:00:00 Universal Coordinated Time. This number is like the POSIX time_t variable, but expressed in milliseconds instead.</p>
    /// <p>This function was introduced in  Qt 4.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#currentDateTime">currentDateTime</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#currentDateTimeUtc">currentDateTimeUtc</a>(), <a href="http://doc.qt.io/qt-5/qdatetime-obsolete.html#toTime_t">toTime_t</a>(), and <a href="http://doc.qt.io/qt-5/qdatetime.html#toTimeSpec">toTimeSpec</a>().</p></div>
    pub unsafe fn current_m_secs_since_epoch() -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QDateTime_currentMSecsSinceEpoch()
    }

    /// <p>Returns the number of seconds since 1970-01-01T00:00:00 Universal Coordinated Time.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static qint64 QDateTime::currentSecsSinceEpoch()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#currentSecsSinceEpoch">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of seconds since 1970-01-01T00:00:00 Universal Coordinated Time.</p>
    /// <p>This function was introduced in  Qt 5.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#currentMSecsSinceEpoch">currentMSecsSinceEpoch</a>().</p></div>
    pub unsafe fn current_secs_since_epoch() -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QDateTime_currentSecsSinceEpoch()
    }

    /// <p>Returns the date part of the datetime.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDate QDateTime::date() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#date">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the date part of the datetime.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#setDate">setDate</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#time">time</a>(), and <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>().</p></div>
    pub unsafe fn date(&self) -> ::cpp_utils::CppBox<crate::QDate> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDateTime_date(self as *const crate::QDateTime);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the number of days from this datetime to the <i>other</i> datetime. The number of days is counted as the number of times midnight is reached between this datetime to the <i>other</i> datetime. This means that a 10 minute difference from 23:55 to 0:05 the next day counts as one day.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QDateTime::daysTo(const QDateTime& arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#daysTo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of days from this datetime to the <i>other</i> datetime. The number of days is counted as the number of times midnight is reached between this datetime to the <i>other</i> datetime. This means that a 10 minute difference from 23:55 to 0:05 the next day counts as one day.</p>
    /// <p>If the <i>other</i> datetime is earlier than this datetime, the value returned is negative.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span> startDate(<span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span>(<span class="number">2012</span><span class="operator">,</span> <span class="number">7</span><span class="operator">,</span> <span class="number">6</span>)<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtime.html">QTime</a></span>(<span class="number">8</span><span class="operator">,</span> <span class="number">30</span><span class="operator">,</span> <span class="number">0</span>));
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span> endDate(<span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span>(<span class="number">2012</span><span class="operator">,</span> <span class="number">7</span><span class="operator">,</span> <span class="number">7</span>)<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtime.html">QTime</a></span>(<span class="number">16</span><span class="operator">,</span> <span class="number">30</span><span class="operator">,</span> <span class="number">0</span>));
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Days from startDate to endDate: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> startDate<span class="operator">.</span>daysTo(endDate);
    ///
    ///   startDate <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span>(<span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span>(<span class="number">2012</span><span class="operator">,</span> <span class="number">7</span><span class="operator">,</span> <span class="number">6</span>)<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtime.html">QTime</a></span>(<span class="number">23</span><span class="operator">,</span> <span class="number">55</span><span class="operator">,</span> <span class="number">0</span>));
    ///   endDate <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span>(<span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span>(<span class="number">2012</span><span class="operator">,</span> <span class="number">7</span><span class="operator">,</span> <span class="number">7</span>)<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtime.html">QTime</a></span>(<span class="number">0</span><span class="operator">,</span> <span class="number">5</span><span class="operator">,</span> <span class="number">0</span>));
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Days from startDate to endDate: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> startDate<span class="operator">.</span>daysTo(endDate);
    ///
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms-obsolete.html#qSwap-1">qSwap</a>(startDate<span class="operator">,</span> endDate); <span class="comment">// Make endDate before startDate.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Days from startDate to endDate: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> startDate<span class="operator">.</span>daysTo(endDate);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#addDays">addDays</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#secsTo">secsTo</a>(), and <a href="http://doc.qt.io/qt-5/qdatetime.html#msecsTo">msecsTo</a>().</p></div>
    pub unsafe fn days_to(
        &self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
    ) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QDateTime_daysTo(
            self as *const crate::QDateTime,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(arg1)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a datetime whose date and time are the number of milliseconds, <i>msecs</i>, that have passed since 1970-01-01T00:00:00.000, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>), and converted to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::LocalTime</a>. On systems that do not support time zones, the time will be set as if local time were <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDateTime QDateTime::fromMSecsSinceEpoch(qint64 msecs)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#fromMSecsSinceEpoch">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a datetime whose date and time are the number of milliseconds, <i>msecs</i>, that have passed since 1970-01-01T00:00:00.000, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>), and converted to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::LocalTime</a>. On systems that do not support time zones, the time will be set as if local time were <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>.</p>
    /// <p>Note that there are possible values for <i>msecs</i> that lie outside the valid range of <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>, both negative and positive. The behavior of this function is undefined for those values.</p>
    /// <p>This function was introduced in  Qt 4.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#toMSecsSinceEpoch">toMSecsSinceEpoch</a>() and <a href="http://doc.qt.io/qt-5/qdatetime.html#setMSecsSinceEpoch">setMSecsSinceEpoch</a>().</p></div>
    pub unsafe fn from_m_secs_since_epoch_i64(msecs: i64) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_fromMSecsSinceEpoch(msecs);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a datetime whose date and time are the number of milliseconds <i>msecs</i> that have passed since 1970-01-01T00:00:00.000, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>) and converted to the given <i>spec</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDateTime QDateTime::fromMSecsSinceEpoch(qint64 msecs, Qt::TimeSpec spec, int offsetFromUtc = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#fromMSecsSinceEpoch-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a datetime whose date and time are the number of milliseconds <i>msecs</i> that have passed since 1970-01-01T00:00:00.000, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>) and converted to the given <i>spec</i>.</p>
    /// <p>Note that there are possible values for <i>msecs</i> that lie outside the valid range of <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>, both negative and positive. The behavior of this function is undefined for those values.</p>
    /// <p>If the <i>spec</i> is not <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> then the <i>offsetSeconds</i> will be ignored. If the <i>spec</i> is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> and the <i>offsetSeconds</i> is 0 then the spec will be set to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>, i.e. an offset of 0 seconds.</p>
    /// <p>If <i>spec</i> is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeZone</a> then the spec will be set to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::LocalTime</a>, i.e. the current system time zone.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#toMSecsSinceEpoch">toMSecsSinceEpoch</a>() and <a href="http://doc.qt.io/qt-5/qdatetime.html#setMSecsSinceEpoch">setMSecsSinceEpoch</a>().</p></div>
    pub unsafe fn from_m_secs_since_epoch_i64_time_spec_int(
        msecs: i64,
        spec: crate::TimeSpec,
        offset_from_utc: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_fromMSecsSinceEpoch1(
            msecs,
            spec,
            offset_from_utc,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a datetime whose date and time are the number of milliseconds <i>msecs</i> that have passed since 1970-01-01T00:00:00.000, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>) and with the given <i>timeZone</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDateTime QDateTime::fromMSecsSinceEpoch(qint64 msecs, const QTimeZone& timeZone)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#fromMSecsSinceEpoch-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a datetime whose date and time are the number of milliseconds <i>msecs</i> that have passed since 1970-01-01T00:00:00.000, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>) and with the given <i>timeZone</i>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#fromSecsSinceEpoch">fromSecsSinceEpoch</a>().</p></div>
    pub unsafe fn from_m_secs_since_epoch_i64_q_time_zone(
        msecs: i64,
        time_zone: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTimeZone>>,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_fromMSecsSinceEpoch2(
            msecs,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTimeZone>>::cast_into(time_zone)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a datetime whose date and time are the number of milliseconds <i>msecs</i> that have passed since 1970-01-01T00:00:00.000, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>) and converted to the given <i>spec</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDateTime QDateTime::fromMSecsSinceEpoch(qint64 msecs, Qt::TimeSpec spec)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#fromMSecsSinceEpoch-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a datetime whose date and time are the number of milliseconds <i>msecs</i> that have passed since 1970-01-01T00:00:00.000, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>) and converted to the given <i>spec</i>.</p>
    /// <p>Note that there are possible values for <i>msecs</i> that lie outside the valid range of <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>, both negative and positive. The behavior of this function is undefined for those values.</p>
    /// <p>If the <i>spec</i> is not <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> then the <i>offsetSeconds</i> will be ignored. If the <i>spec</i> is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> and the <i>offsetSeconds</i> is 0 then the spec will be set to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>, i.e. an offset of 0 seconds.</p>
    /// <p>If <i>spec</i> is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeZone</a> then the spec will be set to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::LocalTime</a>, i.e. the current system time zone.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#toMSecsSinceEpoch">toMSecsSinceEpoch</a>() and <a href="http://doc.qt.io/qt-5/qdatetime.html#setMSecsSinceEpoch">setMSecsSinceEpoch</a>().</p></div>
    pub unsafe fn from_m_secs_since_epoch_i64_time_spec(
        msecs: i64,
        spec: crate::TimeSpec,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_fromMSecsSinceEpoch3(msecs, spec);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a datetime whose date and time are the number of seconds <i>secs</i> that have passed since 1970-01-01T00:00:00.000, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>) and converted to the given <i>spec</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDateTime QDateTime::fromSecsSinceEpoch(qint64 secs, Qt::TimeSpec spe = …, int offsetFromUtc = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#fromSecsSinceEpoch">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a datetime whose date and time are the number of seconds <i>secs</i> that have passed since 1970-01-01T00:00:00.000, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>) and converted to the given <i>spec</i>.</p>
    /// <p>Note that there are possible values for <i>secs</i> that lie outside the valid range of <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>, both negative and positive. The behavior of this function is undefined for those values.</p>
    /// <p>If the <i>spec</i> is not <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> then the <i>offsetSeconds</i> will be ignored. If the <i>spec</i> is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> and the <i>offsetSeconds</i> is 0 then the spec will be set to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>, i.e. an offset of 0 seconds.</p>
    /// <p>If <i>spec</i> is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeZone</a> then the spec will be set to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::LocalTime</a>, i.e. the current system time zone.</p>
    /// <p>This function was introduced in  Qt 5.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#toSecsSinceEpoch">toSecsSinceEpoch</a>() and <a href="http://doc.qt.io/qt-5/qdatetime.html#setSecsSinceEpoch">setSecsSinceEpoch</a>().</p></div>
    pub unsafe fn from_secs_since_epoch_i64_time_spec_int(
        secs: i64,
        spe: crate::TimeSpec,
        offset_from_utc: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDateTime_fromSecsSinceEpoch(secs, spe, offset_from_utc);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a datetime whose date and time are the number of seconds <i>secs</i> that have passed since 1970-01-01T00:00:00.000, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>) and with the given <i>timeZone</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDateTime QDateTime::fromSecsSinceEpoch(qint64 secs, const QTimeZone& timeZone)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#fromSecsSinceEpoch-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a datetime whose date and time are the number of seconds <i>secs</i> that have passed since 1970-01-01T00:00:00.000, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>) and with the given <i>timeZone</i>.</p>
    /// <p>This function was introduced in  Qt 5.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#fromMSecsSinceEpoch">fromMSecsSinceEpoch</a>().</p></div>
    pub unsafe fn from_secs_since_epoch_i64_q_time_zone(
        secs: i64,
        time_zone: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTimeZone>>,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_fromSecsSinceEpoch1(
            secs,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTimeZone>>::cast_into(time_zone)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a datetime whose date and time are the number of seconds <i>secs</i> that have passed since 1970-01-01T00:00:00.000, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>) and converted to the given <i>spec</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDateTime QDateTime::fromSecsSinceEpoch(qint64 secs, Qt::TimeSpec spe = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#fromSecsSinceEpoch">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a datetime whose date and time are the number of seconds <i>secs</i> that have passed since 1970-01-01T00:00:00.000, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>) and converted to the given <i>spec</i>.</p>
    /// <p>Note that there are possible values for <i>secs</i> that lie outside the valid range of <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>, both negative and positive. The behavior of this function is undefined for those values.</p>
    /// <p>If the <i>spec</i> is not <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> then the <i>offsetSeconds</i> will be ignored. If the <i>spec</i> is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> and the <i>offsetSeconds</i> is 0 then the spec will be set to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>, i.e. an offset of 0 seconds.</p>
    /// <p>If <i>spec</i> is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeZone</a> then the spec will be set to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::LocalTime</a>, i.e. the current system time zone.</p>
    /// <p>This function was introduced in  Qt 5.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#toSecsSinceEpoch">toSecsSinceEpoch</a>() and <a href="http://doc.qt.io/qt-5/qdatetime.html#setSecsSinceEpoch">setSecsSinceEpoch</a>().</p></div>
    pub unsafe fn from_secs_since_epoch_i64_time_spec(
        secs: i64,
        spe: crate::TimeSpec,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_fromSecsSinceEpoch2(secs, spe);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a datetime whose date and time are the number of seconds <i>secs</i> that have passed since 1970-01-01T00:00:00.000, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>) and converted to the given <i>spec</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDateTime QDateTime::fromSecsSinceEpoch(qint64 secs)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#fromSecsSinceEpoch">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a datetime whose date and time are the number of seconds <i>secs</i> that have passed since 1970-01-01T00:00:00.000, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>) and converted to the given <i>spec</i>.</p>
    /// <p>Note that there are possible values for <i>secs</i> that lie outside the valid range of <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>, both negative and positive. The behavior of this function is undefined for those values.</p>
    /// <p>If the <i>spec</i> is not <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> then the <i>offsetSeconds</i> will be ignored. If the <i>spec</i> is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> and the <i>offsetSeconds</i> is 0 then the spec will be set to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>, i.e. an offset of 0 seconds.</p>
    /// <p>If <i>spec</i> is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeZone</a> then the spec will be set to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::LocalTime</a>, i.e. the current system time zone.</p>
    /// <p>This function was introduced in  Qt 5.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#toSecsSinceEpoch">toSecsSinceEpoch</a>() and <a href="http://doc.qt.io/qt-5/qdatetime.html#setSecsSinceEpoch">setSecsSinceEpoch</a>().</p></div>
    pub unsafe fn from_secs_since_epoch_i64(secs: i64) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_fromSecsSinceEpoch3(secs);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> represented by the <i>string</i>, using the <i>format</i> given, or an invalid datetime if this is not possible.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDateTime QDateTime::fromString(const QString& s, Qt::DateFormat f = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#fromString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> represented by the <i>string</i>, using the <i>format</i> given, or an invalid datetime if this is not possible.</p>
    /// <p>Note for <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::TextDate</a>: It is recommended that you use the English short month names (e.g. "Jan"). Although localized month names can also be used, they depend on the user's locale settings.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#toString">toString</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toDateTime">QLocale::toDateTime</a>().</p></div>
    pub unsafe fn from_string_q_string_date_format(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        f: crate::DateFormat,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_fromString(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            f,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> represented by the <i>string</i>, using the <i>format</i> given, or an invalid datetime if the string cannot be parsed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDateTime QDateTime::fromString(const QString& s, const QString& format)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#fromString-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> represented by the <i>string</i>, using the <i>format</i> given, or an invalid datetime if the string cannot be parsed.</p>
    /// <p>These expressions may be used for the date part of the format string:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Expression</th><th>Output</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>d</td><td>the day as number without a leading zero (1 to 31)</td></tr>
    /// <tr valign="top" class="even"><td>dd</td><td>the day as number with a leading zero (01 to 31)</td></tr>
    /// <tr valign="top" class="odd"><td>ddd</td><td>the abbreviated localized day name (e.g. 'Mon' to 'Sun'). Uses <a href="http://doc.qt.io/qt-5/qdate.html#shortDayName">QDate::shortDayName</a>().</td></tr>
    /// <tr valign="top" class="even"><td>dddd</td><td>the long localized day name (e.g. 'Monday' to 'Sunday'). Uses <a href="http://doc.qt.io/qt-5/qdate.html#longDayName">QDate::longDayName</a>().</td></tr>
    /// <tr valign="top" class="odd"><td>M</td><td>the month as number without a leading zero (1-12)</td></tr>
    /// <tr valign="top" class="even"><td>MM</td><td>the month as number with a leading zero (01-12)</td></tr>
    /// <tr valign="top" class="odd"><td>MMM</td><td>the abbreviated localized month name (e.g. 'Jan' to 'Dec'). Uses <a href="http://doc.qt.io/qt-5/qdate.html#shortMonthName">QDate::shortMonthName</a>().</td></tr>
    /// <tr valign="top" class="even"><td>MMMM</td><td>the long localized month name (e.g. 'January' to 'December'). Uses <a href="http://doc.qt.io/qt-5/qdate.html#longMonthName">QDate::longMonthName</a>().</td></tr>
    /// <tr valign="top" class="odd"><td>yy</td><td>the year as two digit number (00-99)</td></tr>
    /// <tr valign="top" class="even"><td>yyyy</td><td>the year as four digit number</td></tr>
    /// </tbody></table></div>
    /// <p><b>Note: </b>Unlike the other version of this function, day and month names must be given in the user's local language. It is only possible to use the English names if the user's language is English.</p><p>These expressions may be used for the time part of the format string:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Expression</th><th>Output</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>h</td><td>the hour without a leading zero (0 to 23 or 1 to 12 if AM/PM display)</td></tr>
    /// <tr valign="top" class="even"><td>hh</td><td>the hour with a leading zero (00 to 23 or 01 to 12 if AM/PM display)</td></tr>
    /// <tr valign="top" class="odd"><td>H</td><td>the hour without a leading zero (0 to 23, even with AM/PM display)</td></tr>
    /// <tr valign="top" class="even"><td>HH</td><td>the hour with a leading zero (00 to 23, even with AM/PM display)</td></tr>
    /// <tr valign="top" class="odd"><td>m</td><td>the minute without a leading zero (0 to 59)</td></tr>
    /// <tr valign="top" class="even"><td>mm</td><td>the minute with a leading zero (00 to 59)</td></tr>
    /// <tr valign="top" class="odd"><td>s</td><td>the whole second without a leading zero (0 to 59)</td></tr>
    /// <tr valign="top" class="even"><td>ss</td><td>the whole second with a leading zero where applicable (00 to 59)</td></tr>
    /// <tr valign="top" class="odd"><td>z</td><td>the fractional part of the second, to go after a decimal point, without trailing zeroes (0 to 999). Thus "<code>s.z</code>" reports the seconds to full available (millisecond) precision without trailing zeroes.</td></tr>
    /// <tr valign="top" class="even"><td>zzz</td><td>the fractional part of the second, to millisecond precision, including trailing zeroes where applicable (000 to 999).</td></tr>
    /// <tr valign="top" class="odd"><td>AP or A</td><td>interpret as an AM/PM time. <i>AP</i> must be either "AM" or "PM".</td></tr>
    /// <tr valign="top" class="even"><td>ap or a</td><td>Interpret as an AM/PM time. <i>ap</i> must be either "am" or "pm".</td></tr>
    /// </tbody></table></div>
    /// <p>All other input characters will be treated as text. Any sequence of characters that are enclosed in single quotes will also be treated as text and not be used as an expression.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtime.html">QTime</a></span> time1 <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtime.html">QTime</a></span><span class="operator">::</span>fromString(<span class="string">"131"</span><span class="operator">,</span> <span class="string">"HHh"</span>);
    ///   <span class="comment">// time1 is 13:00:00</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtime.html">QTime</a></span> time1 <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtime.html">QTime</a></span><span class="operator">::</span>fromString(<span class="string">"1apA"</span><span class="operator">,</span> <span class="string">"1amAM"</span>);
    ///   <span class="comment">// time1 is 01:00:00</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span> dateTime2 <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span><span class="operator">::</span>fromString(<span class="string">"M1d1y9800:01:02"</span><span class="operator">,</span>
    /// &#32;   &#32;   &#32;   &#32;   &#32;   &#32;   &#32;   &#32;   &#32;   &#32;   &#32;     <span class="string">"'M'M'd'd'y'yyhh:mm:ss"</span>);
    ///   <span class="comment">// dateTime is 1 January 1998 00:01:02</span>
    ///
    /// </pre>
    /// <p>If the format is not satisfied, an invalid <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> is returned. The expressions that don't have leading zeroes (d, M, h, m, s, z) will be greedy. This means that they will use two digits even if this will put them outside the range and/or leave too few digits for other sections.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span> dateTime <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span><span class="operator">::</span>fromString(<span class="string">"130"</span><span class="operator">,</span> <span class="string">"Mm"</span>); <span class="comment">// invalid</span>
    ///
    /// </pre>
    /// <p>This could have meant 1 January 00:30.00 but the M will grab two digits.</p>
    /// <p>Incorrectly specified fields of the <i>string</i> will cause an invalid <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> to be returned. For example, consider the following code, where the two digit year 12 is read as 1912 (see the table below for all field defaults); the resulting datetime is invalid because 23 April 1912 was a Tuesday, not a Monday:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> string <span class="operator">=</span> <span class="string">"Monday, 23 April 12 22:51:41"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> format <span class="operator">=</span> <span class="string">"dddd, d MMMM yy hh:mm:ss"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span> invalid <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span><span class="operator">::</span>fromString(string<span class="operator">,</span> format);
    ///
    /// </pre>
    /// <p>The correct code is:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> string <span class="operator">=</span> <span class="string">"Tuesday, 23 April 12 22:51:41"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> format <span class="operator">=</span> <span class="string">"dddd, d MMMM yy hh:mm:ss"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span> valid <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span><span class="operator">::</span>fromString(string<span class="operator">,</span> format);
    ///
    /// </pre>
    /// <p>For any field that is not represented in the format, the following defaults are used:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Field</th><th>Default value</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>Year</td><td>1900</td></tr>
    /// <tr valign="top" class="even"><td>Month</td><td>1 (January)</td></tr>
    /// <tr valign="top" class="odd"><td>Day</td><td>1</td></tr>
    /// <tr valign="top" class="even"><td>Hour</td><td>0</td></tr>
    /// <tr valign="top" class="odd"><td>Minute</td><td>0</td></tr>
    /// <tr valign="top" class="even"><td>Second</td><td>0</td></tr>
    /// </tbody></table></div>
    /// <p>For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span> dateTime <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span><span class="operator">::</span>fromString(<span class="string">"1.30.1"</span><span class="operator">,</span> <span class="string">"M.d.s"</span>);
    ///   <span class="comment">// dateTime is January 30 in 1900 at 00:00:01.</span>
    ///   dateTime <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span><span class="operator">::</span>fromString(<span class="string">"12"</span><span class="operator">,</span> <span class="string">"yy"</span>);
    ///   <span class="comment">// dateTime is January 1 in 1912 at 00:00:00.</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#toString">toString</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#fromString">QDate::fromString</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#fromString">QTime::fromString</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toDateTime">QLocale::toDateTime</a>().</p></div>
    pub unsafe fn from_string_2_q_string(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        format: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_fromString1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(format)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> represented by the <i>string</i>, using the <i>format</i> given, or an invalid datetime if this is not possible.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDateTime QDateTime::fromString(const QString& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#fromString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> represented by the <i>string</i>, using the <i>format</i> given, or an invalid datetime if this is not possible.</p>
    /// <p>Note for <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::TextDate</a>: It is recommended that you use the English short month names (e.g. "Jan"). Although localized month names can also be used, they depend on the user's locale settings.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#toString">toString</a>() and <a href="http://doc.qt.io/qt-5/qlocale.html#toDateTime">QLocale::toDateTime</a>().</p></div>
    pub unsafe fn from_string_q_string(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_fromString2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a datetime whose date and time are the number of <i>seconds</i> that have passed since 1970-01-01T00:00:00, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>) and converted to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::LocalTime</a>. On systems that do not support time zones, the time will be set as if local time were <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDateTime QDateTime::fromTime_t(unsigned int secsSince1Jan1970UTC)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime-obsolete.html#fromTime_t">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a datetime whose date and time are the number of <i>seconds</i> that have passed since 1970-01-01T00:00:00, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>) and converted to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::LocalTime</a>. On systems that do not support time zones, the time will be set as if local time were <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>.</p>
    /// <p><b>Note: </b>This function is deprecated. Please use <a href="http://doc.qt.io/qt-5/qdatetime.html#fromSecsSinceEpoch">fromSecsSinceEpoch</a>() in new code.</p><p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime-obsolete.html#toTime_t">toTime_t</a>() and <a href="http://doc.qt.io/qt-5/qdatetime-obsolete.html#setTime_t">setTime_t</a>().</p></div>
    pub unsafe fn from_time_t_uint(
        secs_since1_jan1970_u_t_c: ::std::os::raw::c_uint,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDateTime_fromTime_t(secs_since1_jan1970_u_t_c);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a datetime whose date and time are the number of <i>seconds</i> that have passed since 1970-01-01T00:00:00, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>) and converted to the given <i>spec</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDateTime QDateTime::fromTime_t(unsigned int secsSince1Jan1970UTC, Qt::TimeSpec spec, int offsetFromUtc = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime-obsolete.html#fromTime_t-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a datetime whose date and time are the number of <i>seconds</i> that have passed since 1970-01-01T00:00:00, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>) and converted to the given <i>spec</i>.</p>
    /// <p>If the <i>spec</i> is not <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> then the <i>offsetSeconds</i> will be ignored. If the <i>spec</i> is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> and the <i>offsetSeconds</i> is 0 then the spec will be set to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>, i.e. an offset of 0 seconds.</p>
    /// <p><b>Note: </b>This function is deprecated. Please use <a href="http://doc.qt.io/qt-5/qdatetime.html#fromSecsSinceEpoch">fromSecsSinceEpoch</a>() in new code.</p><p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime-obsolete.html#toTime_t">toTime_t</a>() and <a href="http://doc.qt.io/qt-5/qdatetime-obsolete.html#setTime_t">setTime_t</a>().</p></div>
    pub unsafe fn from_time_t_uint_time_spec_int(
        secs_since1_jan1970_u_t_c: ::std::os::raw::c_uint,
        spec: crate::TimeSpec,
        offset_from_utc: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_fromTime_t1(
            secs_since1_jan1970_u_t_c,
            spec,
            offset_from_utc,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a datetime whose date and time are the number of <i>seconds</i> that have passed since 1970-01-01T00:00:00, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>) and with the given <i>timeZone</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDateTime QDateTime::fromTime_t(unsigned int secsSince1Jan1970UTC, const QTimeZone& timeZone)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime-obsolete.html#fromTime_t-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a datetime whose date and time are the number of <i>seconds</i> that have passed since 1970-01-01T00:00:00, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>) and with the given <i>timeZone</i>.</p>
    /// <p><b>Note: </b>This function is deprecated. Please use <a href="http://doc.qt.io/qt-5/qdatetime.html#fromSecsSinceEpoch">fromSecsSinceEpoch</a>() in new code.</p><p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime-obsolete.html#toTime_t">toTime_t</a>() and <a href="http://doc.qt.io/qt-5/qdatetime-obsolete.html#setTime_t">setTime_t</a>().</p></div>
    pub unsafe fn from_time_t_uint_q_time_zone(
        secs_since1_jan1970_u_t_c: ::std::os::raw::c_uint,
        time_zone: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTimeZone>>,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_fromTime_t2(
            secs_since1_jan1970_u_t_c,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTimeZone>>::cast_into(time_zone)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a datetime whose date and time are the number of <i>seconds</i> that have passed since 1970-01-01T00:00:00, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>) and converted to the given <i>spec</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDateTime QDateTime::fromTime_t(unsigned int secsSince1Jan1970UTC, Qt::TimeSpec spec)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime-obsolete.html#fromTime_t-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a datetime whose date and time are the number of <i>seconds</i> that have passed since 1970-01-01T00:00:00, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>) and converted to the given <i>spec</i>.</p>
    /// <p>If the <i>spec</i> is not <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> then the <i>offsetSeconds</i> will be ignored. If the <i>spec</i> is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> and the <i>offsetSeconds</i> is 0 then the spec will be set to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>, i.e. an offset of 0 seconds.</p>
    /// <p><b>Note: </b>This function is deprecated. Please use <a href="http://doc.qt.io/qt-5/qdatetime.html#fromSecsSinceEpoch">fromSecsSinceEpoch</a>() in new code.</p><p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime-obsolete.html#toTime_t">toTime_t</a>() and <a href="http://doc.qt.io/qt-5/qdatetime-obsolete.html#setTime_t">setTime_t</a>().</p></div>
    pub unsafe fn from_time_t_uint_time_spec(
        secs_since1_jan1970_u_t_c: ::std::os::raw::c_uint,
        spec: crate::TimeSpec,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDateTime_fromTime_t3(secs_since1_jan1970_u_t_c, spec);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns if this datetime falls in Daylight-Saving Time.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDateTime::isDaylightTime() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#isDaylightTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns if this datetime falls in Daylight-Saving Time.</p>
    /// <p>If the <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeSpec</a> is not <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::LocalTime</a> or <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeZone</a> then will always return false.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>().</p></div>
    pub unsafe fn is_daylight_time(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDateTime_isDaylightTime(self as *const crate::QDateTime)
    }

    /// <p>Returns <code>true</code> if both the date and the time are null; otherwise returns <code>false</code>. A null datetime is invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDateTime::isNull() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#isNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if both the date and the time are null; otherwise returns <code>false</code>. A null datetime is invalid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#isNull">QDate::isNull</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#isNull">QTime::isNull</a>(), and <a href="http://doc.qt.io/qt-5/qdatetime.html#isValid">isValid</a>().</p></div>
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDateTime_isNull(self as *const crate::QDateTime)
    }

    /// <p>Returns <code>true</code> if both the date and the time are valid and they are valid in the current <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeSpec</a>, otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDateTime::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if both the date and the time are valid and they are valid in the current <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeSpec</a>, otherwise returns <code>false</code>.</p>
    /// <p>If the <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>() is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::LocalTime</a> or <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeZone</a> then the date and time are checked to see if they fall in the Standard Time to Daylight-Saving Time transition hour, i.e. if the transition is at 2am and the clock goes forward to 3am then the time from 02:00:00 to 02:59:59.999 is considered to be invalid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdate.html#isValid">QDate::isValid</a>() and <a href="http://doc.qt.io/qt-5/qtime.html#isValid">QTime::isValid</a>().</p></div>
    pub unsafe fn is_valid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDateTime_isValid(self as *const crate::QDateTime)
    }

    /// <p>Returns the number of milliseconds from this datetime to the <i>other</i> datetime. If the <i>other</i> datetime is earlier than this datetime, the value returned is negative.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QDateTime::msecsTo(const QDateTime& arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#msecsTo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of milliseconds from this datetime to the <i>other</i> datetime. If the <i>other</i> datetime is earlier than this datetime, the value returned is negative.</p>
    /// <p>Before performing the comparison, the two datetimes are converted to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a> to ensure that the result is correct if daylight-saving (DST) applies to one of the two datetimes and but not the other.</p>
    /// <p>Returns 0 if either datetime is invalid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#addMSecs">addMSecs</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#daysTo">daysTo</a>(), and <a href="http://doc.qt.io/qt-5/qtime.html#msecsTo">QTime::msecsTo</a>().</p></div>
    pub unsafe fn msecs_to(
        &self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
    ) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QDateTime_msecsTo(
            self as *const crate::QDateTime,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(arg1)
                .as_raw_ptr(),
        )
    }

    /// <p>Constructs a null datetime (i.e. null date and null time). A null datetime is invalid, since the date is invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDateTime::QDateTime()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a null datetime (i.e. null date and null time). A null datetime is invalid, since the date is invalid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#isValid">isValid</a>().</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_QDateTime();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a datetime with the given <i>date</i>, a valid time(00:00:00.000), and sets the <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>() to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::LocalTime</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDateTime::QDateTime(const QDate& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a datetime with the given <i>date</i>, a valid time(00:00:00.000), and sets the <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>() to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::LocalTime</a>.</p></div>
    pub unsafe fn from_q_date(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDate>>,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_QDateTime1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDate>>::cast_into(arg1).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a datetime with the given <i>date</i> and <i>time</i>, using the time specification defined by <i>spec</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDateTime::QDateTime(const QDate& arg1, const QTime& arg2, Qt::TimeSpec spec = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a datetime with the given <i>date</i> and <i>time</i>, using the time specification defined by <i>spec</i>.</p>
    /// <p>If <i>date</i> is valid and <i>time</i> is not, the time will be set to midnight.</p>
    /// <p>If <i>spec</i> is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> then it will be set to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>, i.e. an offset of 0 seconds. To create a <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> datetime use the correct constructor.</p>
    /// <p>If <i>spec</i> is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeZone</a> then the spec will be set to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::LocalTime</a>, i.e. the current system time zone. To create a <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeZone</a> datetime use the correct constructor.</p></div>
    pub unsafe fn from_q_date_q_time_time_spec(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDate>>,
        arg2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTime>>,
        spec: crate::TimeSpec,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_QDateTime2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDate>>::cast_into(arg1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTime>>::cast_into(arg2).as_raw_ptr(),
            spec,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a datetime with the given <i>date</i> and <i>time</i>, using the time specification defined by <i>spec</i> and <i>offsetSeconds</i> seconds.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDateTime::QDateTime(const QDate& date, const QTime& time, Qt::TimeSpec spec, int offsetSeconds)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a datetime with the given <i>date</i> and <i>time</i>, using the time specification defined by <i>spec</i> and <i>offsetSeconds</i> seconds.</p>
    /// <p>If <i>date</i> is valid and <i>time</i> is not, the time will be set to midnight.</p>
    /// <p>If the <i>spec</i> is not <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> then <i>offsetSeconds</i> will be ignored.</p>
    /// <p>If the <i>spec</i> is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> and <i>offsetSeconds</i> is 0 then the <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>() will be set to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>, i.e. an offset of 0 seconds.</p>
    /// <p>If <i>spec</i> is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeZone</a> then the spec will be set to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::LocalTime</a>, i.e. the current system time zone. To create a <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeZone</a> datetime use the correct constructor.</p>
    /// <p>This function was introduced in  Qt 5.2.</p></div>
    pub unsafe fn from_q_date_q_time_time_spec_int(
        date: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDate>>,
        time: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTime>>,
        spec: crate::TimeSpec,
        offset_seconds: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_QDateTime3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDate>>::cast_into(date).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTime>>::cast_into(time).as_raw_ptr(),
            spec,
            offset_seconds,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a datetime with the given <i>date</i> and <i>time</i>, using the Time Zone specified by <i>timeZone</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDateTime::QDateTime(const QDate& date, const QTime& time, const QTimeZone& timeZone)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a datetime with the given <i>date</i> and <i>time</i>, using the Time Zone specified by <i>timeZone</i>.</p>
    /// <p>If <i>date</i> is valid and <i>time</i> is not, the time will be set to 00:00:00.</p>
    /// <p>If <i>timeZone</i> is invalid then the datetime will be invalid.</p>
    /// <p>This function was introduced in  Qt 5.2.</p></div>
    pub unsafe fn from_q_date_q_time_q_time_zone(
        date: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDate>>,
        time: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTime>>,
        time_zone: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTimeZone>>,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_QDateTime4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDate>>::cast_into(date).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTime>>::cast_into(time).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTimeZone>>::cast_into(time_zone)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a datetime with the given <i>date</i> and <i>time</i>, using the time specification defined by <i>spec</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDateTime::QDateTime(const QDate& arg1, const QTime& arg2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a datetime with the given <i>date</i> and <i>time</i>, using the time specification defined by <i>spec</i>.</p>
    /// <p>If <i>date</i> is valid and <i>time</i> is not, the time will be set to midnight.</p>
    /// <p>If <i>spec</i> is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> then it will be set to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>, i.e. an offset of 0 seconds. To create a <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> datetime use the correct constructor.</p>
    /// <p>If <i>spec</i> is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeZone</a> then the spec will be set to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::LocalTime</a>, i.e. the current system time zone. To create a <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeZone</a> datetime use the correct constructor.</p></div>
    pub unsafe fn from_q_date_q_time(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDate>>,
        arg2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTime>>,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_QDateTime7(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDate>>::cast_into(arg1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTime>>::cast_into(arg2).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of the <i>other</i> datetime.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDateTime::QDateTime(const QDateTime& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of the <i>other</i> datetime.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_QDateTime5(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the current Offset From UTC in seconds.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QDateTime::offsetFromUtc() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#offsetFromUtc">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current Offset From UTC in seconds.</p>
    /// <p>If the <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>() is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> this will be the value originally set.</p>
    /// <p>If the <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>() is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeZone</a> this will be the offset effective in the Time Zone including any Daylight-Saving Offset.</p>
    /// <p>If the <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>() is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::LocalTime</a> this will be the difference between the Local Time and UTC including any Daylight-Saving Offset.</p>
    /// <p>If the <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>() is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a> this will be 0.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#setOffsetFromUtc">setOffsetFromUtc</a>().</p></div>
    pub unsafe fn offset_from_utc(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QDateTime_offsetFromUtc(self as *const crate::QDateTime)
    }

    /// <p>Returns the number of seconds from this datetime to the <i>other</i> datetime. If the <i>other</i> datetime is earlier than this datetime, the value returned is negative.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QDateTime::secsTo(const QDateTime& arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#secsTo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of seconds from this datetime to the <i>other</i> datetime. If the <i>other</i> datetime is earlier than this datetime, the value returned is negative.</p>
    /// <p>Before performing the comparison, the two datetimes are converted to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a> to ensure that the result is correct if daylight-saving (DST) applies to one of the two datetimes but not the other.</p>
    /// <p>Returns 0 if either datetime is invalid.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span> now <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span><span class="operator">::</span>currentDateTime();
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span> xmas(<span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span>(now<span class="operator">.</span>date()<span class="operator">.</span>year()<span class="operator">,</span> <span class="number">12</span><span class="operator">,</span> <span class="number">25</span>)<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtime.html">QTime</a></span>(<span class="number">0</span><span class="operator">,</span> <span class="number">0</span>));
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>(<span class="string">"There are %d seconds to Christmas"</span><span class="operator">,</span> now<span class="operator">.</span>secsTo(xmas));
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#addSecs">addSecs</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#daysTo">daysTo</a>(), and <a href="http://doc.qt.io/qt-5/qtime.html#secsTo">QTime::secsTo</a>().</p></div>
    pub unsafe fn secs_to(
        &self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
    ) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QDateTime_secsTo(
            self as *const crate::QDateTime,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(arg1)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the date part of this datetime to <i>date</i>. If no time is set yet, it is set to midnight. If <i>date</i> is invalid, this <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> becomes invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDateTime::setDate(const QDate& date)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#setDate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the date part of this datetime to <i>date</i>. If no time is set yet, it is set to midnight. If <i>date</i> is invalid, this <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> becomes invalid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#date">date</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#setTime">setTime</a>(), and <a href="http://doc.qt.io/qt-5/qdatetime.html#setTimeSpec">setTimeSpec</a>().</p></div>
    pub unsafe fn set_date(
        &mut self,
        date: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDate>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QDateTime_setDate(
            self as *mut crate::QDateTime,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDate>>::cast_into(date).as_raw_ptr(),
        )
    }

    /// <p>Sets the date and time given the number of milliseconds <i>msecs</i> that have passed since 1970-01-01T00:00:00.000, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>). On systems that do not support time zones this function will behave as if local time were <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDateTime::setMSecsSinceEpoch(qint64 msecs)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#setMSecsSinceEpoch">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the date and time given the number of milliseconds <i>msecs</i> that have passed since 1970-01-01T00:00:00.000, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>). On systems that do not support time zones this function will behave as if local time were <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>.</p>
    /// <p>Note that passing the minimum of <code>qint64</code> (<code>std::numeric_limits&lt;qint64&gt;::min()</code>) to <i>msecs</i> will result in undefined behavior.</p>
    /// <p>This function was introduced in  Qt 4.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#toMSecsSinceEpoch">toMSecsSinceEpoch</a>() and <a href="http://doc.qt.io/qt-5/qdatetime.html#setSecsSinceEpoch">setSecsSinceEpoch</a>().</p></div>
    pub unsafe fn set_m_secs_since_epoch(&mut self, msecs: i64) {
        crate::__ffi::ctr_qt_core_ffi_QDateTime_setMSecsSinceEpoch(
            self as *mut crate::QDateTime,
            msecs,
        )
    }

    /// <p>Sets the <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>() to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> and the offset to <i>offsetSeconds</i>. The datetime will refer to a different point in time.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDateTime::setOffsetFromUtc(int offsetSeconds)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#setOffsetFromUtc">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>() to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> and the offset to <i>offsetSeconds</i>. The datetime will refer to a different point in time.</p>
    /// <p>The maximum and minimum offset is 14 positive or negative hours. If <i>offsetSeconds</i> is larger or smaller than that, then the result is undefined.</p>
    /// <p>If <i>offsetSeconds</i> is 0 then the <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>() will be set to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#isValid">isValid</a>() and <a href="http://doc.qt.io/qt-5/qdatetime.html#offsetFromUtc">offsetFromUtc</a>().</p></div>
    pub unsafe fn set_offset_from_utc(&mut self, offset_seconds: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QDateTime_setOffsetFromUtc(
            self as *mut crate::QDateTime,
            offset_seconds,
        )
    }

    /// <p>Sets the date and time given the number of seconds <i>secs</i> that have passed since 1970-01-01T00:00:00.000, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>). On systems that do not support time zones this function will behave as if local time were <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDateTime::setSecsSinceEpoch(qint64 secs)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#setSecsSinceEpoch">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the date and time given the number of seconds <i>secs</i> that have passed since 1970-01-01T00:00:00.000, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>). On systems that do not support time zones this function will behave as if local time were <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>.</p>
    /// <p>This function was introduced in  Qt 5.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#toSecsSinceEpoch">toSecsSinceEpoch</a>() and <a href="http://doc.qt.io/qt-5/qdatetime.html#setMSecsSinceEpoch">setMSecsSinceEpoch</a>().</p></div>
    pub unsafe fn set_secs_since_epoch(&mut self, secs: i64) {
        crate::__ffi::ctr_qt_core_ffi_QDateTime_setSecsSinceEpoch(
            self as *mut crate::QDateTime,
            secs,
        )
    }

    /// <p>Sets the time part of this datetime to <i>time</i>. If <i>time</i> is not valid, this function sets it to midnight. Therefore, it's possible to clear any set time in a <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> by setting it to a default <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a>:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDateTime::setTime(const QTime& time)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#setTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the time part of this datetime to <i>time</i>. If <i>time</i> is not valid, this function sets it to midnight. Therefore, it's possible to clear any set time in a <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> by setting it to a default <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span> dt <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span><span class="operator">::</span>currentDateTime();
    ///   dt<span class="operator">.</span>setTime(<span class="type"><a href="http://doc.qt.io/qt-5/qtime.html">QTime</a></span>());
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#time">time</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#setDate">setDate</a>(), and <a href="http://doc.qt.io/qt-5/qdatetime.html#setTimeSpec">setTimeSpec</a>().</p></div>
    pub unsafe fn set_time(
        &mut self,
        time: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTime>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QDateTime_setTime(
            self as *mut crate::QDateTime,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTime>>::cast_into(time).as_raw_ptr(),
        )
    }

    /// <p>Sets the time specification used in this datetime to <i>spec</i>. The datetime will refer to a different point in time.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDateTime::setTimeSpec(Qt::TimeSpec spec)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#setTimeSpec">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the time specification used in this datetime to <i>spec</i>. The datetime will refer to a different point in time.</p>
    /// <p>If <i>spec</i> is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> then the <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>() will be set to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>, i.e. an effective offset of 0.</p>
    /// <p>If <i>spec</i> is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeZone</a> then the spec will be set to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::LocalTime</a>, i.e. the current system time zone.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span> local(<span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span><span class="operator">::</span>currentDateTime());
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Local time is:"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> local;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span> UTC(local);
    ///   UTC<span class="operator">.</span>setTimeSpec(<span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>UTC);
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"UTC time is:"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> UTC;
    ///
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"There are"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> local<span class="operator">.</span>secsTo(UTC) <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"seconds difference between the datetimes."</span>;
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#setDate">setDate</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#setTime">setTime</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#setTimeZone">setTimeZone</a>(), and <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeSpec</a>.</p></div>
    pub unsafe fn set_time_spec(&mut self, spec: crate::TimeSpec) {
        crate::__ffi::ctr_qt_core_ffi_QDateTime_setTimeSpec(self as *mut crate::QDateTime, spec)
    }

    /// <p>Sets the date and time given the number of <i>seconds</i> that have passed since 1970-01-01T00:00:00, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>). On systems that do not support time zones this function will behave as if local time were <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDateTime::setTime_t(unsigned int secsSince1Jan1970UTC)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime-obsolete.html#setTime_t">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the date and time given the number of <i>seconds</i> that have passed since 1970-01-01T00:00:00, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>). On systems that do not support time zones this function will behave as if local time were <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>.</p>
    /// <p><b>Note: </b>This function is deprecated. For new code, use <a href="http://doc.qt.io/qt-5/qdatetime.html#setSecsSinceEpoch">setSecsSinceEpoch</a>().</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime-obsolete.html#toTime_t">toTime_t</a>().</p></div>
    pub unsafe fn set_time_t(&mut self, secs_since1_jan1970_u_t_c: ::std::os::raw::c_uint) {
        crate::__ffi::ctr_qt_core_ffi_QDateTime_setTime_t(
            self as *mut crate::QDateTime,
            secs_since1_jan1970_u_t_c,
        )
    }

    /// <p>Sets the time zone used in this datetime to <i>toZone</i>. The datetime will refer to a different point in time.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDateTime::setTimeZone(const QTimeZone& toZone)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#setTimeZone">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the time zone used in this datetime to <i>toZone</i>. The datetime will refer to a different point in time.</p>
    /// <p>If <i>toZone</i> is invalid then the datetime will be invalid.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#timeZone">timeZone</a>() and <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeSpec</a>.</p></div>
    pub unsafe fn set_time_zone(
        &mut self,
        to_zone: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTimeZone>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QDateTime_setTimeZone(
            self as *mut crate::QDateTime,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTimeZone>>::cast_into(to_zone)
                .as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QDateTime::setUtcOffset(int seconds)```</span>.
    pub unsafe fn set_utc_offset(&mut self, seconds: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QDateTime_setUtcOffset(self as *mut crate::QDateTime, seconds)
    }

    /// <p>Swaps this datetime with <i>other</i>. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDateTime::swap(QDateTime& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps this datetime with <i>other</i>. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QDateTime>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QDateTime_swap(
            self as *mut crate::QDateTime,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QDateTime>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the time part of the datetime.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTime QDateTime::time() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#time">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the time part of the datetime.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#setTime">setTime</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#date">date</a>(), and <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>().</p></div>
    pub unsafe fn time(&self) -> ::cpp_utils::CppBox<crate::QTime> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDateTime_time(self as *const crate::QDateTime);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the time specification of the datetime.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```Qt::TimeSpec QDateTime::timeSpec() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the time specification of the datetime.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#setTimeSpec">setTimeSpec</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#date">date</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#time">time</a>(), and <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeSpec</a>.</p></div>
    pub unsafe fn time_spec(&self) -> crate::TimeSpec {
        crate::__ffi::ctr_qt_core_ffi_QDateTime_timeSpec(self as *const crate::QDateTime)
    }

    /// <p>Returns the time zone of the datetime.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTimeZone QDateTime::timeZone() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#timeZone">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the time zone of the datetime.</p>
    /// <p>If the <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>() is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::LocalTime</a> then an instance of the current system time zone will be returned. Note however that if you copy this time zone the instance will not remain in sync if the system time zone changes.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#setTimeZone">setTimeZone</a>() and <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeSpec</a>.</p></div>
    pub unsafe fn time_zone(&self) -> ::cpp_utils::CppBox<crate::QTimeZone> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDateTime_timeZone(self as *const crate::QDateTime);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the Time Zone Abbreviation for the datetime.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QDateTime::timeZoneAbbreviation() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#timeZoneAbbreviation">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the Time Zone Abbreviation for the datetime.</p>
    /// <p>If the <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>() is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a> this will be "UTC".</p>
    /// <p>If the <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>() is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> this will be in the format "UTC[+-]00:00".</p>
    /// <p>If the <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>() is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::LocalTime</a> then the host system is queried for the correct abbreviation.</p>
    /// <p>Note that abbreviations may or may not be localized.</p>
    /// <p>Note too that the abbreviation is not guaranteed to be a unique value, i.e. different time zones may have the same abbreviation.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>().</p></div>
    pub unsafe fn time_zone_abbreviation(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_timeZoneAbbreviation(
            self as *const crate::QDateTime,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a datetime containing the date and time information in this datetime, but specified using the <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::LocalTime</a> definition.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDateTime QDateTime::toLocalTime() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#toLocalTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a datetime containing the date and time information in this datetime, but specified using the <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::LocalTime</a> definition.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span> UTC(<span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span><span class="operator">::</span>currentDateTimeUtc());
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span> local(UTC<span class="operator">.</span>toLocalTime());
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"UTC time is:"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> UTC;
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Local time is:"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> local;
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"No difference between times:"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> UTC<span class="operator">.</span>secsTo(local);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#toTimeSpec">toTimeSpec</a>().</p></div>
    pub unsafe fn to_local_time(&self) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDateTime_toLocalTime(self as *const crate::QDateTime);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the datetime as the number of milliseconds that have passed since 1970-01-01T00:00:00.000, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QDateTime::toMSecsSinceEpoch() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#toMSecsSinceEpoch">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the datetime as the number of milliseconds that have passed since 1970-01-01T00:00:00.000, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>).</p>
    /// <p>On systems that do not support time zones, this function will behave as if local time were <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>.</p>
    /// <p>The behavior for this function is undefined if the datetime stored in this object is not valid. However, for all valid dates, this function returns a unique value.</p>
    /// <p>This function was introduced in  Qt 4.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#toSecsSinceEpoch">toSecsSinceEpoch</a>() and <a href="http://doc.qt.io/qt-5/qdatetime.html#setMSecsSinceEpoch">setMSecsSinceEpoch</a>().</p></div>
    pub unsafe fn to_m_secs_since_epoch(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QDateTime_toMSecsSinceEpoch(self as *const crate::QDateTime)
    }

    /// <p>Returns a copy of this datetime converted to a spec of <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> with the given <i>offsetSeconds</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDateTime QDateTime::toOffsetFromUtc(int offsetSeconds) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#toOffsetFromUtc">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a copy of this datetime converted to a spec of <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> with the given <i>offsetSeconds</i>.</p>
    /// <p>If the <i>offsetSeconds</i> equals 0 then a UTC datetime will be returned</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#setOffsetFromUtc">setOffsetFromUtc</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#offsetFromUtc">offsetFromUtc</a>(), and <a href="http://doc.qt.io/qt-5/qdatetime.html#toTimeSpec">toTimeSpec</a>().</p></div>
    pub unsafe fn to_offset_from_utc(
        &self,
        offset_seconds: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_toOffsetFromUtc(
            self as *const crate::QDateTime,
            offset_seconds,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the datetime as the number of seconds that have passed since 1970-01-01T00:00:00.000, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QDateTime::toSecsSinceEpoch() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#toSecsSinceEpoch">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the datetime as the number of seconds that have passed since 1970-01-01T00:00:00.000, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>).</p>
    /// <p>On systems that do not support time zones, this function will behave as if local time were <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>.</p>
    /// <p>The behavior for this function is undefined if the datetime stored in this object is not valid. However, for all valid dates, this function returns a unique value.</p>
    /// <p>This function was introduced in  Qt 5.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#toMSecsSinceEpoch">toMSecsSinceEpoch</a>() and <a href="http://doc.qt.io/qt-5/qdatetime.html#setSecsSinceEpoch">setSecsSinceEpoch</a>().</p></div>
    pub unsafe fn to_secs_since_epoch(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QDateTime_toSecsSinceEpoch(self as *const crate::QDateTime)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QDateTime::toString(Qt::DateFormat f = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#toString-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the datetime as a string in the <i>format</i> given.</p>
    /// <p>If the <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::TextDate</a>, the string is formatted in the default way. <a href="http://doc.qt.io/qt-5/qdate.html#shortDayName">QDate::shortDayName</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#shortMonthName">QDate::shortMonthName</a>(), and <a href="http://doc.qt.io/qt-5/qtime.html#toString">QTime::toString</a>() are used to generate the string, so the day and month names will be localized names using the system locale, i.e. <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>(). An example of this formatting is "Wed May 20 03:40:13 1998".</p>
    /// <p>If the <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::ISODate</a>, the string format corresponds to the ISO 8601 extended specification for representations of dates and times, taking the form YYYY-MM-DDTHH:mm:ss[Z|[+|-]HH:mm], depending on the <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>() of the <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>. If the <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>() is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>, Z will be appended to the string; if the <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>() is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a>, the offset in hours and minutes from UTC will be appended to the string. To include milliseconds in the ISO 8601 date, use the <i>format</i> <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::ISODateWithMs</a>, which corresponds to YYYY-MM-DDTHH:mm:ss.zzz[Z|[+|-]HH:mm].</p>
    /// <p>If the <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::SystemLocaleShortDate</a> or <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::SystemLocaleLongDate</a>, the string format depends on the locale settings of the system. Identical to calling <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().<a href="http://doc.qt.io/qt-5/qdatetime.html#toString">toString</a>(datetime, <a href="http://doc.qt.io/qt-5/qlocale.html#FormatType-enum">QLocale::ShortFormat</a>) or <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().<a href="http://doc.qt.io/qt-5/qdatetime.html#toString">toString</a>(datetime, <a href="http://doc.qt.io/qt-5/qlocale.html#FormatType-enum">QLocale::LongFormat</a>).</p>
    /// <p>If the <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::DefaultLocaleShortDate</a> or <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::DefaultLocaleLongDate</a>, the string format depends on the default application locale. This is the locale set with <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">QLocale::setDefault</a>(), or the system locale if no default locale has been set. Identical to calling QLocale().<a href="http://doc.qt.io/qt-5/qdatetime.html#toString">toString</a>(datetime, <a href="http://doc.qt.io/qt-5/qlocale.html#FormatType-enum">QLocale::ShortFormat</a>) or QLocale().<a href="http://doc.qt.io/qt-5/qdatetime.html#toString">toString</a>(datetime, <a href="http://doc.qt.io/qt-5/qlocale.html#FormatType-enum">QLocale::LongFormat</a>).</p>
    /// <p>If the <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::RFC2822Date</a>, the string is formatted following <a href="http://www.rfc-editor.org/rfc/rfc2822.txt">RFC 2822</a>.</p>
    /// <p>If the datetime is invalid, an empty string will be returned.</p>
    /// <p><b>Warning:</b> The <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::ISODate</a> format is only valid for years in the range 0 to 9999. This restriction may apply to locale-aware formats as well, depending on the locale settings.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#fromString">fromString</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#toString">QDate::toString</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#toString">QTime::toString</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>().</p></div>
    pub unsafe fn to_string_date_format(
        &self,
        f: crate::DateFormat,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDateTime_toString(self as *const crate::QDateTime, f);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the datetime as a string. The <i>format</i> parameter determines the format of the result string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QDateTime::toString(const QString& format) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#toString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the datetime as a string. The <i>format</i> parameter determines the format of the result string.</p>
    /// <p>These expressions may be used for the date:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Expression</th><th>Output</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>d</td><td>the day as number without a leading zero (1 to 31)</td></tr>
    /// <tr valign="top" class="even"><td>dd</td><td>the day as number with a leading zero (01 to 31)</td></tr>
    /// <tr valign="top" class="odd"><td>ddd</td><td>the abbreviated localized day name (e.g. 'Mon' to 'Sun'). Uses the system locale to localize the name, i.e. <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().</td></tr>
    /// <tr valign="top" class="even"><td>dddd</td><td>the long localized day name (e.g. 'Monday' to 'Sunday'). Uses the system locale to localize the name, i.e. <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().</td></tr>
    /// <tr valign="top" class="odd"><td>M</td><td>the month as number without a leading zero (1-12)</td></tr>
    /// <tr valign="top" class="even"><td>MM</td><td>the month as number with a leading zero (01-12)</td></tr>
    /// <tr valign="top" class="odd"><td>MMM</td><td>the abbreviated localized month name (e.g. 'Jan' to 'Dec'). Uses the system locale to localize the name, i.e. <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().</td></tr>
    /// <tr valign="top" class="even"><td>MMMM</td><td>the long localized month name (e.g. 'January' to 'December'). Uses the system locale to localize the name, i.e. <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().</td></tr>
    /// <tr valign="top" class="odd"><td>yy</td><td>the year as two digit number (00-99)</td></tr>
    /// <tr valign="top" class="even"><td>yyyy</td><td>the year as four digit number</td></tr>
    /// </tbody></table></div>
    /// <p>These expressions may be used for the time:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Expression</th><th>Output</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>h</td><td>the hour without a leading zero (0 to 23 or 1 to 12 if AM/PM display)</td></tr>
    /// <tr valign="top" class="even"><td>hh</td><td>the hour with a leading zero (00 to 23 or 01 to 12 if AM/PM display)</td></tr>
    /// <tr valign="top" class="odd"><td>H</td><td>the hour without a leading zero (0 to 23, even with AM/PM display)</td></tr>
    /// <tr valign="top" class="even"><td>HH</td><td>the hour with a leading zero (00 to 23, even with AM/PM display)</td></tr>
    /// <tr valign="top" class="odd"><td>m</td><td>the minute without a leading zero (0 to 59)</td></tr>
    /// <tr valign="top" class="even"><td>mm</td><td>the minute with a leading zero (00 to 59)</td></tr>
    /// <tr valign="top" class="odd"><td>s</td><td>the whole second without a leading zero (0 to 59)</td></tr>
    /// <tr valign="top" class="even"><td>ss</td><td>the whole second with a leading zero where applicable (00 to 59)</td></tr>
    /// <tr valign="top" class="odd"><td>z</td><td>the fractional part of the second, to go after a decimal point, without trailing zeroes (0 to 999). Thus "<code>s.z</code>" reports the seconds to full available (millisecond) precision without trailing zeroes.</td></tr>
    /// <tr valign="top" class="even"><td>zzz</td><td>the fractional part of the second, to millisecond precision, including trailing zeroes where applicable (000 to 999).</td></tr>
    /// <tr valign="top" class="odd"><td>AP or A</td><td>use AM/PM display. <i>A/AP</i> will be replaced by either "AM" or "PM".</td></tr>
    /// <tr valign="top" class="even"><td>ap or a</td><td>use am/pm display. <i>a/ap</i> will be replaced by either "am" or "pm".</td></tr>
    /// <tr valign="top" class="odd"><td>t</td><td>the timezone (for example "CEST")</td></tr>
    /// </tbody></table></div>
    /// <p>All other input characters will be ignored. Any sequence of characters that are enclosed in single quotes will be treated as text and not be used as an expression. Two consecutive single quotes ("''") are replaced by a singlequote in the output. Formats without separators (e.g. "HHmm") are currently not supported.</p>
    /// <p>Example format strings (assumed that the <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> is 21 May 2001 14:13:09.120):</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Format</th><th>Result</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>dd.MM.yyyy</td><td>21.05.2001</td></tr>
    /// <tr valign="top" class="even"><td>ddd MMMM d yy</td><td>Tue May 21 01</td></tr>
    /// <tr valign="top" class="odd"><td>hh:mm:ss.zzz</td><td>14:13:09.120</td></tr>
    /// <tr valign="top" class="even"><td>hh:mm:ss.z</td><td>14:13:09.12</td></tr>
    /// <tr valign="top" class="odd"><td>h:m:s ap</td><td>2:13:9 pm</td></tr>
    /// </tbody></table></div>
    /// <p>If the datetime is invalid, an empty string will be returned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#fromString">fromString</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#toString">QDate::toString</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#toString">QTime::toString</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>().</p></div>
    pub unsafe fn to_string_q_string(
        &self,
        format: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_toString1(
            self as *const crate::QDateTime,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(format)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QDateTime::toString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#toString-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the datetime as a string in the <i>format</i> given.</p>
    /// <p>If the <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::TextDate</a>, the string is formatted in the default way. <a href="http://doc.qt.io/qt-5/qdate.html#shortDayName">QDate::shortDayName</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#shortMonthName">QDate::shortMonthName</a>(), and <a href="http://doc.qt.io/qt-5/qtime.html#toString">QTime::toString</a>() are used to generate the string, so the day and month names will be localized names using the system locale, i.e. <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>(). An example of this formatting is "Wed May 20 03:40:13 1998".</p>
    /// <p>If the <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::ISODate</a>, the string format corresponds to the ISO 8601 extended specification for representations of dates and times, taking the form YYYY-MM-DDTHH:mm:ss[Z|[+|-]HH:mm], depending on the <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>() of the <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>. If the <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>() is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>, Z will be appended to the string; if the <a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>() is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a>, the offset in hours and minutes from UTC will be appended to the string. To include milliseconds in the ISO 8601 date, use the <i>format</i> <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::ISODateWithMs</a>, which corresponds to YYYY-MM-DDTHH:mm:ss.zzz[Z|[+|-]HH:mm].</p>
    /// <p>If the <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::SystemLocaleShortDate</a> or <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::SystemLocaleLongDate</a>, the string format depends on the locale settings of the system. Identical to calling <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().<a href="http://doc.qt.io/qt-5/qdatetime.html#toString">toString</a>(datetime, <a href="http://doc.qt.io/qt-5/qlocale.html#FormatType-enum">QLocale::ShortFormat</a>) or <a href="http://doc.qt.io/qt-5/qlocale.html#system">QLocale::system</a>().<a href="http://doc.qt.io/qt-5/qdatetime.html#toString">toString</a>(datetime, <a href="http://doc.qt.io/qt-5/qlocale.html#FormatType-enum">QLocale::LongFormat</a>).</p>
    /// <p>If the <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::DefaultLocaleShortDate</a> or <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::DefaultLocaleLongDate</a>, the string format depends on the default application locale. This is the locale set with <a href="http://doc.qt.io/qt-5/qlocale.html#setDefault">QLocale::setDefault</a>(), or the system locale if no default locale has been set. Identical to calling QLocale().<a href="http://doc.qt.io/qt-5/qdatetime.html#toString">toString</a>(datetime, <a href="http://doc.qt.io/qt-5/qlocale.html#FormatType-enum">QLocale::ShortFormat</a>) or QLocale().<a href="http://doc.qt.io/qt-5/qdatetime.html#toString">toString</a>(datetime, <a href="http://doc.qt.io/qt-5/qlocale.html#FormatType-enum">QLocale::LongFormat</a>).</p>
    /// <p>If the <i>format</i> is <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::RFC2822Date</a>, the string is formatted following <a href="http://www.rfc-editor.org/rfc/rfc2822.txt">RFC 2822</a>.</p>
    /// <p>If the datetime is invalid, an empty string will be returned.</p>
    /// <p><b>Warning:</b> The <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::ISODate</a> format is only valid for years in the range 0 to 9999. This restriction may apply to locale-aware formats as well, depending on the locale settings.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#fromString">fromString</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#toString">QDate::toString</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#toString">QTime::toString</a>(), and <a href="http://doc.qt.io/qt-5/qlocale.html#toString">QLocale::toString</a>().</p></div>
    pub unsafe fn to_string(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDateTime_toString2(self as *const crate::QDateTime);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QString QDateTime::toString(QStringView format) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_string_q_string_view(
        &self,
        format: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_toString6(
            self as *const crate::QDateTime,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(format)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a copy of this datetime converted to the given time <i>spec</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDateTime QDateTime::toTimeSpec(Qt::TimeSpec spec) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#toTimeSpec">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a copy of this datetime converted to the given time <i>spec</i>.</p>
    /// <p>If <i>spec</i> is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> then it is set to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>. To set to a spec of <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::OffsetFromUTC</a> use <a href="http://doc.qt.io/qt-5/qdatetime.html#toOffsetFromUtc">toOffsetFromUtc</a>().</p>
    /// <p>If <i>spec</i> is <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeZone</a> then it is set to <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::LocalTime</a>, i.e. the local Time Zone.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span> local(<span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span><span class="operator">::</span>currentDateTime());
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span> UTC(local<span class="operator">.</span>toTimeSpec(<span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>UTC));
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Local time is:"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> local;
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"UTC time is:"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> UTC;
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"No difference between times:"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> local<span class="operator">.</span>secsTo(UTC);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#timeSpec">timeSpec</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#toTimeZone">toTimeZone</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#toUTC">toUTC</a>(), and <a href="http://doc.qt.io/qt-5/qdatetime.html#toLocalTime">toLocalTime</a>().</p></div>
    pub unsafe fn to_time_spec(
        &self,
        spec: crate::TimeSpec,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_toTimeSpec(
            self as *const crate::QDateTime,
            spec,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the datetime as the number of seconds that have passed since 1970-01-01T00:00:00, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QDateTime::toTime_t() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime-obsolete.html#toTime_t">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the datetime as the number of seconds that have passed since 1970-01-01T00:00:00, Coordinated Universal Time (<a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>).</p>
    /// <p>On systems that do not support time zones, this function will behave as if local time were <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>.</p>
    /// <p><b>Note: </b>This function returns a 32-bit unsigned integer and is deprecated.</p><p>If the date is outside the range 1970-01-01T00:00:00 to 2106-02-07T06:28:14, this function returns -1 cast to an unsigned integer (i.e., 0xFFFFFFFF).</p>
    /// <p>To get an extended range, use <a href="http://doc.qt.io/qt-5/qdatetime.html#toMSecsSinceEpoch">toMSecsSinceEpoch</a>() or <a href="http://doc.qt.io/qt-5/qdatetime.html#toSecsSinceEpoch">toSecsSinceEpoch</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#toSecsSinceEpoch">toSecsSinceEpoch</a>(), <a href="http://doc.qt.io/qt-5/qdatetime.html#toMSecsSinceEpoch">toMSecsSinceEpoch</a>(), and <a href="http://doc.qt.io/qt-5/qdatetime-obsolete.html#setTime_t">setTime_t</a>().</p></div>
    pub unsafe fn to_time_t(&self) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QDateTime_toTime_t(self as *const crate::QDateTime)
    }

    /// <p>Returns a copy of this datetime converted to the given <i>timeZone</i></p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDateTime QDateTime::toTimeZone(const QTimeZone& toZone) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#toTimeZone">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a copy of this datetime converted to the given <i>timeZone</i></p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#timeZone">timeZone</a>() and <a href="http://doc.qt.io/qt-5/qdatetime.html#toTimeSpec">toTimeSpec</a>().</p></div>
    pub unsafe fn to_time_zone(
        &self,
        to_zone: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTimeZone>>,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDateTime_toTimeZone(
            self as *const crate::QDateTime,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTimeZone>>::cast_into(to_zone)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a datetime containing the date and time information in this datetime, but specified using the <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a> definition.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDateTime QDateTime::toUTC() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#toUTC">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a datetime containing the date and time information in this datetime, but specified using the <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a> definition.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span> local(<span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span><span class="operator">::</span>currentDateTime());
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html#QDateTime">QDateTime</a></span> UTC(local<span class="operator">.</span>toUTC());
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Local time is:"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> local;
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"UTC time is:"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> UTC;
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"No difference between times:"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> local<span class="operator">.</span>secsTo(UTC);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#toTimeSpec">toTimeSpec</a>().</p></div>
    pub unsafe fn to_u_t_c(&self) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDateTime_toUTC(self as *const crate::QDateTime);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```int QDateTime::utcOffset() const```</span>.
    pub unsafe fn utc_offset(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QDateTime_utcOffset(self as *const crate::QDateTime)
    }
}

pub mod q_elapsed_timer {
    //! C++ type: <span style='color: green;'>```QElapsedTimer```</span>

    /// <p>This enum contains the different clock types that <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> may use.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QElapsedTimer::ClockType```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#ClockType-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum contains the different clock types that <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> may use.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> will always use the same clock type in a particular machine, so this value will not change during the lifetime of a program. It is provided so that <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> can be used with other non-Qt implementations, to guarantee that the same reference clock is being used.</p>
    ///
    /// <a name="systemtime"></a>
    /// <h5>SystemTime</h5>
    /// <p>The system time clock is purely the real time, expressed in milliseconds since Jan 1, 1970 at 0:00 UTC. It's equivalent to the value returned by the C and POSIX <code>time</code> function, with the milliseconds added. This clock type is currently only used on Unix systems that do not support monotonic clocks (see below).</p>
    /// <p>This is the only non-monotonic clock that <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> may use.</p>
    /// <a name="monotonicclock"></a>
    /// <h5>MonotonicClock</h5>
    /// <p>This is the system's monotonic clock, expressed in milliseconds since an arbitrary point in the past. This clock type is used on Unix systems which support POSIX monotonic clocks (<code>_POSIX_MONOTONIC_CLOCK</code>).</p>
    /// <p>This clock does not overflow.</p>
    /// <a name="tickcounter"></a>
    /// <h5>TickCounter</h5>
    /// <p>The tick counter clock type is based on the system's or the processor's tick counter, multiplied by the duration of a tick. This clock type is used on Windows platforms. If the high-precision performance counter is available on Windows, the <code><a href="http://doc.qt.io/qt-5/#performancecounter">PerformanceCounter</a></code> clock type is used instead.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/#tickcounter">TickCounter</a> clock type is the only clock type that may overflow. Windows Vista and Windows Server 2008 support the extended 64-bit tick counter, which allows avoiding the overflow.</p>
    /// <p>On Windows systems, the clock overflows after 2^32 milliseconds, which corresponds to roughly 49.7 days. This means two processes' reckoning of the time since the reference may be different by multiples of 2^32 milliseconds. When comparing such values, it's recommended that the high 32 bits of the millisecond count be masked off.</p>
    /// <a name="machabsolutetime"></a>
    /// <h5>MachAbsoluteTime</h5>
    /// <p>This clock type is based on the absolute time presented by Mach kernels, such as that found on <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>. This clock type is presented separately from <a href="http://doc.qt.io/qt-5/#monotonicclock">MonotonicClock</a> since <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS are also Unix systems and may support a POSIX monotonic clock with values differing from the Mach absolute time.</p>
    /// <p>This clock is monotonic and does not overflow.</p>
    /// <a name="performancecounter"></a>
    /// <h5>PerformanceCounter</h5>
    /// <p>This clock uses the Windows functions <code>QueryPerformanceCounter</code> and <code>QueryPerformanceFrequency</code> to access the system's high-precision performance counter. Since this counter may not be available on all systems, <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> will fall back to the <code><a href="http://doc.qt.io/qt-5/#tickcounter">TickCounter</a></code> clock automatically, if this clock cannot be used.</p>
    /// <p>This clock is monotonic and does not overflow.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#clockType">clockType</a>() and <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#isMonotonic">isMonotonic</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct ClockType(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for ClockType {
        fn from(value: ::std::os::raw::c_int) -> Self {
            ClockType(value)
        }
    }

    impl From<ClockType> for ::std::os::raw::c_int {
        fn from(value: ClockType) -> Self {
            value.0
        }
    }

    impl ClockType {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl ClockType {
        /// The human-readable system time. This clock is not monotonic. (C++ enum variant: <span style='color: green;'>```SystemTime = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const SystemTime: crate::q_elapsed_timer::ClockType =
            crate::q_elapsed_timer::ClockType(0);
        /// The system's monotonic clock, usually found in Unix systems. This clock is monotonic and does not overflow. (C++ enum variant: <span style='color: green;'>```MonotonicClock = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const MonotonicClock: crate::q_elapsed_timer::ClockType =
            crate::q_elapsed_timer::ClockType(1);
        /// The system's tick counter, used on Windows systems. This clock may overflow. (C++ enum variant: <span style='color: green;'>```TickCounter = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const TickCounter: crate::q_elapsed_timer::ClockType =
            crate::q_elapsed_timer::ClockType(2);
        /// The Mach kernel's absolute time (<a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS). This clock is monotonic and does not overflow. (C++ enum variant: <span style='color: green;'>```MachAbsoluteTime = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const MachAbsoluteTime: crate::q_elapsed_timer::ClockType =
            crate::q_elapsed_timer::ClockType(3);
        /// The high-resolution performance counter provided by Windows. This clock is monotonic and does not overflow. (C++ enum variant: <span style='color: green;'>```PerformanceCounter = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const PerformanceCounter: crate::q_elapsed_timer::ClockType =
            crate::q_elapsed_timer::ClockType(4);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> class provides a fast way to calculate elapsed times.</p>
///
/// C++ class: <span style='color: green;'>```QElapsedTimer```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> class provides a fast way to calculate elapsed times.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> class is usually used to quickly calculate how much time has elapsed between two events. Its API is similar to that of <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a>, so code that was using that can be ported quickly to the new class.</p>
/// <p>However, unlike <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a>, <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> tries to use monotonic clocks if possible. This means it's not possible to convert <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> objects to a human-readable time.</p>
/// <p>The typical use-case for the class is to determine how much time was spent in a slow operation. The simplest example of such a case is for debugging purposes, as in the following example:</p>
/// <pre class="cpp">
///
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#QElapsedTimer">QElapsedTimer</a></span> timer;
/// &#32;     timer<span class="operator">.</span>start();
///
/// &#32;     slowOperation1();
///
/// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The slow operation took"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> timer<span class="operator">.</span>elapsed() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"milliseconds"</span>;
///
/// </pre>
/// <p>In this example, the timer is started by a call to <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#start">start</a>() and the elapsed timer is calculated by the <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#elapsed">elapsed</a>() function.</p>
/// <p>The time elapsed can also be used to recalculate the time available for another operation, after the first one is complete. This is useful when the execution must complete within a certain time period, but several steps are needed. The <code>waitFor</code>-type functions in <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> and its subclasses are good examples of such need. In that case, the code could be as follows:</p>
/// <pre class="cpp">
///
///   <span class="type">void</span> executeSlowOperations(<span class="type">int</span> timeout)
///   {
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#QElapsedTimer">QElapsedTimer</a></span> timer;
/// &#32;     timer<span class="operator">.</span>start();
/// &#32;     slowOperation1();
///
/// &#32;     <span class="type">int</span> remainingTime <span class="operator">=</span> timeout <span class="operator">-</span> timer<span class="operator">.</span>elapsed();
/// &#32;     <span class="keyword">if</span> (remainingTime <span class="operator">&gt;</span> <span class="number">0</span>)
/// &#32;   &#32;     slowOperation2(remainingTime);
///   }
///
/// </pre>
/// <p>Another use-case is to execute a certain operation for a specific timeslice. For this, <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> provides the <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#hasExpired">hasExpired</a>() convenience function, which can be used to determine if a certain number of milliseconds has already elapsed:</p>
/// <pre class="cpp">
///
///   <span class="type">void</span> executeOperationsForTime(<span class="type">int</span> ms)
///   {
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#QElapsedTimer">QElapsedTimer</a></span> timer;
/// &#32;     timer<span class="operator">.</span>start();
///
/// &#32;     <span class="keyword">while</span> (<span class="operator">!</span>timer<span class="operator">.</span>hasExpired(ms))
/// &#32;   &#32;     slowOperation1();
///   }
///
/// </pre>
/// <p>It is often more convenient to use <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> in this case, which counts towards a timeout in the future instead of tracking elapsed time.</p>
/// <a name="reference-clocks"></a></div>
#[repr(C)]
pub struct QElapsedTimer {
    _unused: u8,
}
impl QElapsedTimer {
    /// <p>Returns the clock type that this <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> implementation uses.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QElapsedTimer::ClockType QElapsedTimer::clockType()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#clockType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the clock type that this <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> implementation uses.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#isMonotonic">isMonotonic</a>().</p></div>
    pub unsafe fn clock_type() -> crate::q_elapsed_timer::ClockType {
        crate::__ffi::ctr_qt_core_ffi_QElapsedTimer_clockType()
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> class provides a fast way to calculate elapsed times.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QElapsedTimer& QElapsedTimer::operator=(const QElapsedTimer& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> class provides a fast way to calculate elapsed times.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> class is usually used to quickly calculate how much time has elapsed between two events. Its API is similar to that of <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a>, so code that was using that can be ported quickly to the new class.</p>
    /// <p>However, unlike <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a>, <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> tries to use monotonic clocks if possible. This means it's not possible to convert <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> objects to a human-readable time.</p>
    /// <p>The typical use-case for the class is to determine how much time was spent in a slow operation. The simplest example of such a case is for debugging purposes, as in the following example:</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#QElapsedTimer">QElapsedTimer</a></span> timer;
    /// &#32;     timer<span class="operator">.</span>start();
    ///
    /// &#32;     slowOperation1();
    ///
    /// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The slow operation took"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> timer<span class="operator">.</span>elapsed() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"milliseconds"</span>;
    ///
    /// </pre>
    /// <p>In this example, the timer is started by a call to <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#start">start</a>() and the elapsed timer is calculated by the <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#elapsed">elapsed</a>() function.</p>
    /// <p>The time elapsed can also be used to recalculate the time available for another operation, after the first one is complete. This is useful when the execution must complete within a certain time period, but several steps are needed. The <code>waitFor</code>-type functions in <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> and its subclasses are good examples of such need. In that case, the code could be as follows:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">void</span> executeSlowOperations(<span class="type">int</span> timeout)
    ///   {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#QElapsedTimer">QElapsedTimer</a></span> timer;
    /// &#32;     timer<span class="operator">.</span>start();
    /// &#32;     slowOperation1();
    ///
    /// &#32;     <span class="type">int</span> remainingTime <span class="operator">=</span> timeout <span class="operator">-</span> timer<span class="operator">.</span>elapsed();
    /// &#32;     <span class="keyword">if</span> (remainingTime <span class="operator">&gt;</span> <span class="number">0</span>)
    /// &#32;   &#32;     slowOperation2(remainingTime);
    ///   }
    ///
    /// </pre>
    /// <p>Another use-case is to execute a certain operation for a specific timeslice. For this, <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> provides the <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#hasExpired">hasExpired</a>() convenience function, which can be used to determine if a certain number of milliseconds has already elapsed:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">void</span> executeOperationsForTime(<span class="type">int</span> ms)
    ///   {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#QElapsedTimer">QElapsedTimer</a></span> timer;
    /// &#32;     timer<span class="operator">.</span>start();
    ///
    /// &#32;     <span class="keyword">while</span> (<span class="operator">!</span>timer<span class="operator">.</span>hasExpired(ms))
    /// &#32;   &#32;     slowOperation1();
    ///   }
    ///
    /// </pre>
    /// <p>It is often more convenient to use <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> in this case, which counts towards a timeout in the future instead of tracking elapsed time.</p>
    /// <a name="reference-clocks"></a></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QElapsedTimer>>,
    ) -> ::cpp_utils::MutRef<crate::QElapsedTimer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QElapsedTimer_operator_(
            self as *mut crate::QElapsedTimer,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QElapsedTimer>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of milliseconds since this <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> was last started.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QElapsedTimer::elapsed() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#elapsed">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of milliseconds since this <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> was last started.</p>
    /// <p>Calling this function on a <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> that is invalid results in undefined behavior.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#start">start</a>(), <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#restart">restart</a>(), <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#hasExpired">hasExpired</a>(), <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#isValid">isValid</a>(), and <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#invalidate">invalidate</a>().</p></div>
    pub unsafe fn elapsed(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QElapsedTimer_elapsed(self as *const crate::QElapsedTimer)
    }

    /// <p>Returns <code>true</code> if this <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> has already expired by <i>timeout</i> milliseconds (that is, more than <i>timeout</i> milliseconds have elapsed). The value of <i>timeout</i> can be -1 to indicate that this timer does not expire, in which case this function will always return false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QElapsedTimer::hasExpired(qint64 timeout) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#hasExpired">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> has already expired by <i>timeout</i> milliseconds (that is, more than <i>timeout</i> milliseconds have elapsed). The value of <i>timeout</i> can be -1 to indicate that this timer does not expire, in which case this function will always return false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#elapsed">elapsed</a>() and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a>.</p></div>
    pub unsafe fn has_expired(&self, timeout: i64) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QElapsedTimer_hasExpired(
            self as *const crate::QElapsedTimer,
            timeout,
        )
    }

    /// <p>Marks this <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> object as invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QElapsedTimer::invalidate()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#invalidate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Marks this <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> object as invalid.</p>
    /// <p>An invalid object can be checked with <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#isValid">isValid</a>(). Calculations of timer elapsed since invalid data are undefined and will likely produce bizarre results.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#isValid">isValid</a>(), <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#start">start</a>(), and <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#restart">restart</a>().</p></div>
    pub unsafe fn invalidate(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QElapsedTimer_invalidate(self as *mut crate::QElapsedTimer)
    }

    /// <p>Returns <code>true</code> if this is a monotonic clock, false otherwise. See the information on the different clock types to understand which ones are monotonic.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QElapsedTimer::isMonotonic()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#isMonotonic">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this is a monotonic clock, false otherwise. See the information on the different clock types to understand which ones are monotonic.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#clockType">clockType</a>() and <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#ClockType-enum">QElapsedTimer::ClockType</a>.</p></div>
    pub unsafe fn is_monotonic() -> bool {
        crate::__ffi::ctr_qt_core_ffi_QElapsedTimer_isMonotonic()
    }

    /// <p>Returns <code>false</code> if the timer has never been started or invalidated by a call to <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#invalidate">invalidate</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QElapsedTimer::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>false</code> if the timer has never been started or invalidated by a call to <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#invalidate">invalidate</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#invalidate">invalidate</a>(), <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#start">start</a>(), and <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#restart">restart</a>().</p></div>
    pub unsafe fn is_valid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QElapsedTimer_isValid(self as *const crate::QElapsedTimer)
    }

    /// <p>Returns the number of milliseconds between last time this <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> object was started and its reference clock's start.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QElapsedTimer::msecsSinceReference() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#msecsSinceReference">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of milliseconds between last time this <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> object was started and its reference clock's start.</p>
    /// <p>This number is usually arbitrary for all clocks except the <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#ClockType-enum">QElapsedTimer::SystemTime</a> clock. For that clock type, this number is the number of milliseconds since January 1st, 1970 at 0:00 UTC (that is, it is the Unix time expressed in milliseconds).</p>
    /// <p>On Linux, Windows and Apple platforms, this value is usually the time since the system boot, though it usually does not include the time the system has spent in sleep states.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#clockType">clockType</a>() and <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#elapsed">elapsed</a>().</p></div>
    pub unsafe fn msecs_since_reference(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QElapsedTimer_msecsSinceReference(
            self as *const crate::QElapsedTimer,
        )
    }

    /// <p>Returns the number of milliseconds between this <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> and <i>other</i>. If <i>other</i> was started before this object, the returned value will be negative. If it was started later, the returned value will be positive.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QElapsedTimer::msecsTo(const QElapsedTimer& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#msecsTo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of milliseconds between this <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> and <i>other</i>. If <i>other</i> was started before this object, the returned value will be negative. If it was started later, the returned value will be positive.</p>
    /// <p>The return value is undefined if this object or <i>other</i> were invalidated.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#secsTo">secsTo</a>() and <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#elapsed">elapsed</a>().</p></div>
    pub unsafe fn msecs_to(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QElapsedTimer>>,
    ) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QElapsedTimer_msecsTo(
            self as *const crate::QElapsedTimer,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QElapsedTimer>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Constructs an invalid <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a>. A timer becomes valid once it has been started.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QElapsedTimer::QElapsedTimer()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#QElapsedTimer">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an invalid <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a>. A timer becomes valid once it has been started.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#isValid">isValid</a>() and <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#start">start</a>().</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QElapsedTimer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QElapsedTimer_QElapsedTimer();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> class provides a fast way to calculate elapsed times.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QElapsedTimer::QElapsedTimer(const QElapsedTimer& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> class provides a fast way to calculate elapsed times.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> class is usually used to quickly calculate how much time has elapsed between two events. Its API is similar to that of <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a>, so code that was using that can be ported quickly to the new class.</p>
    /// <p>However, unlike <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a>, <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> tries to use monotonic clocks if possible. This means it's not possible to convert <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> objects to a human-readable time.</p>
    /// <p>The typical use-case for the class is to determine how much time was spent in a slow operation. The simplest example of such a case is for debugging purposes, as in the following example:</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#QElapsedTimer">QElapsedTimer</a></span> timer;
    /// &#32;     timer<span class="operator">.</span>start();
    ///
    /// &#32;     slowOperation1();
    ///
    /// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The slow operation took"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> timer<span class="operator">.</span>elapsed() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"milliseconds"</span>;
    ///
    /// </pre>
    /// <p>In this example, the timer is started by a call to <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#start">start</a>() and the elapsed timer is calculated by the <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#elapsed">elapsed</a>() function.</p>
    /// <p>The time elapsed can also be used to recalculate the time available for another operation, after the first one is complete. This is useful when the execution must complete within a certain time period, but several steps are needed. The <code>waitFor</code>-type functions in <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> and its subclasses are good examples of such need. In that case, the code could be as follows:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">void</span> executeSlowOperations(<span class="type">int</span> timeout)
    ///   {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#QElapsedTimer">QElapsedTimer</a></span> timer;
    /// &#32;     timer<span class="operator">.</span>start();
    /// &#32;     slowOperation1();
    ///
    /// &#32;     <span class="type">int</span> remainingTime <span class="operator">=</span> timeout <span class="operator">-</span> timer<span class="operator">.</span>elapsed();
    /// &#32;     <span class="keyword">if</span> (remainingTime <span class="operator">&gt;</span> <span class="number">0</span>)
    /// &#32;   &#32;     slowOperation2(remainingTime);
    ///   }
    ///
    /// </pre>
    /// <p>Another use-case is to execute a certain operation for a specific timeslice. For this, <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> provides the <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#hasExpired">hasExpired</a>() convenience function, which can be used to determine if a certain number of milliseconds has already elapsed:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">void</span> executeOperationsForTime(<span class="type">int</span> ms)
    ///   {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#QElapsedTimer">QElapsedTimer</a></span> timer;
    /// &#32;     timer<span class="operator">.</span>start();
    ///
    /// &#32;     <span class="keyword">while</span> (<span class="operator">!</span>timer<span class="operator">.</span>hasExpired(ms))
    /// &#32;   &#32;     slowOperation1();
    ///   }
    ///
    /// </pre>
    /// <p>It is often more convenient to use <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> in this case, which counts towards a timeout in the future instead of tracking elapsed time.</p>
    /// <a name="reference-clocks"></a></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QElapsedTimer>>,
    ) -> ::cpp_utils::CppBox<crate::QElapsedTimer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QElapsedTimer_QElapsedTimer1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QElapsedTimer>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the number of nanoseconds since this <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> was last started.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QElapsedTimer::nsecsElapsed() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#nsecsElapsed">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of nanoseconds since this <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> was last started.</p>
    /// <p>Calling this function on a <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> that is invalid results in undefined behavior.</p>
    /// <p>On platforms that do not provide nanosecond resolution, the value returned will be the best estimate available.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#start">start</a>(), <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#restart">restart</a>(), <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#hasExpired">hasExpired</a>(), and <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#invalidate">invalidate</a>().</p></div>
    pub unsafe fn nsecs_elapsed(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QElapsedTimer_nsecsElapsed(
            self as *const crate::QElapsedTimer,
        )
    }

    /// <p>Restarts the timer and returns the time elapsed since the previous start. This function is equivalent to obtaining the elapsed time with <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#elapsed">elapsed</a>() and then starting the timer again with <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#start">start</a>(), but it does so in one single operation, avoiding the need to obtain the clock value twice.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QElapsedTimer::restart()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#restart">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Restarts the timer and returns the time elapsed since the previous start. This function is equivalent to obtaining the elapsed time with <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#elapsed">elapsed</a>() and then starting the timer again with <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#start">start</a>(), but it does so in one single operation, avoiding the need to obtain the clock value twice.</p>
    /// <p>Calling this function on a <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> that is invalid results in undefined behavior.</p>
    /// <p>The following example illustrates how to use this function to calibrate a parameter to a slow operation (for example, an iteration count) so that this operation takes at least 250 milliseconds:</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#QElapsedTimer">QElapsedTimer</a></span> timer;
    ///
    /// &#32;     <span class="type">int</span> count <span class="operator">=</span> <span class="number">1</span>;
    /// &#32;     timer<span class="operator">.</span>start();
    /// &#32;     <span class="keyword">do</span> {
    /// &#32;   &#32;     count <span class="operator">*</span><span class="operator">=</span> <span class="number">2</span>;
    /// &#32;   &#32;     slowOperation2(count);
    /// &#32;     } <span class="keyword">while</span> (timer<span class="operator">.</span>restart() <span class="operator">&lt;</span> <span class="number">250</span>);
    ///
    /// &#32;     <span class="keyword">return</span> count;
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#start">start</a>(), <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#invalidate">invalidate</a>(), <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#elapsed">elapsed</a>(), and <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#isValid">isValid</a>().</p></div>
    pub unsafe fn restart(&mut self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QElapsedTimer_restart(self as *mut crate::QElapsedTimer)
    }

    /// <p>Returns the number of seconds between this <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> and <i>other</i>. If <i>other</i> was started before this object, the returned value will be negative. If it was started later, the returned value will be positive.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QElapsedTimer::secsTo(const QElapsedTimer& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#secsTo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of seconds between this <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> and <i>other</i>. If <i>other</i> was started before this object, the returned value will be negative. If it was started later, the returned value will be positive.</p>
    /// <p>Calling this function on or with a <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> that is invalid results in undefined behavior.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#msecsTo">msecsTo</a>() and <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#elapsed">elapsed</a>().</p></div>
    pub unsafe fn secs_to(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QElapsedTimer>>,
    ) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QElapsedTimer_secsTo(
            self as *const crate::QElapsedTimer,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QElapsedTimer>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Starts this timer. Once started, a timer value can be checked with <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#elapsed">elapsed</a>() or <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#msecsSinceReference">msecsSinceReference</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QElapsedTimer::start()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#start">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Starts this timer. Once started, a timer value can be checked with <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#elapsed">elapsed</a>() or <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#msecsSinceReference">msecsSinceReference</a>().</p>
    /// <p>Normally, a timer is started just before a lengthy operation, such as:</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#QElapsedTimer">QElapsedTimer</a></span> timer;
    /// &#32;     timer<span class="operator">.</span>start();
    ///
    /// &#32;     slowOperation1();
    ///
    /// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The slow operation took"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> timer<span class="operator">.</span>elapsed() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"milliseconds"</span>;
    ///
    /// </pre>
    /// <p>Also, starting a timer makes it valid again.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#restart">restart</a>(), <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#invalidate">invalidate</a>(), and <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#elapsed">elapsed</a>().</p></div>
    pub unsafe fn start(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QElapsedTimer_start(self as *mut crate::QElapsedTimer)
    }
}

pub mod q_deadline_timer {
    //! C++ type: <span style='color: green;'>```QDeadlineTimer```</span>

    /// C++ enum: <span style='color: green;'>```QDeadlineTimer::ForeverConstant```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#ForeverConstant-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct ForeverConstant(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for ForeverConstant {
        fn from(value: ::std::os::raw::c_int) -> Self {
            ForeverConstant(value)
        }
    }

    impl From<ForeverConstant> for ::std::os::raw::c_int {
        fn from(value: ForeverConstant) -> Self {
            value.0
        }
    }

    impl ForeverConstant {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl ForeverConstant {
        /// Used when creating a <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> to indicate the deadline should not expire (C++ enum variant: <span style='color: green;'>```Forever = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const Forever: crate::q_deadline_timer::ForeverConstant =
            crate::q_deadline_timer::ForeverConstant(0);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> class marks a deadline in the future.</p>
///
/// C++ class: <span style='color: green;'>```QDeadlineTimer```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> class marks a deadline in the future.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> class is usually used to calculate future deadlines and verify whether the deadline has expired. <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> can also be used for deadlines without expiration ("forever"). It forms a counterpart to <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a>, which calculates how much time has elapsed since <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#start">QElapsedTimer::start</a>() was called.</p>
/// <p><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> provides a more convenient API compared to <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#hasExpired">QElapsedTimer::hasExpired</a>().</p>
/// <p>The typical use-case for the class is to create a <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> before the operation in question is started, and then use <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>() or <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#hasExpired">hasExpired</a>() to determine whether to continue trying the operation. <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> objects can be passed to functions being called to execute this operation so they know how long to still operate.</p>
/// <pre class="cpp">
///
///   <span class="type">void</span> executeOperation(<span class="type">int</span> msecs)
///   {
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#QDeadlineTimer">QDeadlineTimer</a></span> deadline(msecs);
/// &#32;     <span class="keyword">do</span> {
/// &#32;   &#32;     <span class="keyword">if</span> (readFromDevice(deadline<span class="operator">.</span>remainingTime())
/// &#32;   &#32;   &#32;     <span class="keyword">break</span>;
/// &#32;   &#32;     waitForReadyRead(deadline);
/// &#32;     } <span class="keyword">while</span> (<span class="operator">!</span>deadline<span class="operator">.</span>hasExpired());
///   }
///
/// </pre>
/// <p>Many <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> functions deal with time out values, which all are measured in milliseconds. There are two special values, the same as many other Qt functions named <code>waitFor</code> or similar:</p>
/// <ul>
/// <li>0: no time left, expired</li>
/// <li>-1: infinite time left, timer never expires</li>
/// </ul>
/// <a name="reference-clocks"></a></div>
#[repr(C)]
pub struct QDeadlineTimer {
    _unused: u8,
}
impl QDeadlineTimer {
    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object whose deadline is extended from <i>dt</i>'s deadline by <i>nsecs</i> nanoseconds. If <i>dt</i> was set to never expire, this function returns a <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> that will not expire either.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDeadlineTimer QDeadlineTimer::addNSecs(QDeadlineTimer dt, qint64 nsecs)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#addNSecs">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object whose deadline is extended from <i>dt</i>'s deadline by <i>nsecs</i> nanoseconds. If <i>dt</i> was set to never expire, this function returns a <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> that will not expire either.</p>
    /// <p><b>Note: </b>if <i>dt</i> was created as expired, its deadline is indeterminate and adding an amount of time may or may not cause it to become unexpired.</p></div>
    pub unsafe fn add_n_secs(
        dt: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDeadlineTimer>>,
        nsecs: i64,
    ) -> ::cpp_utils::CppBox<crate::QDeadlineTimer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_addNSecs(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDeadlineTimer>>::cast_into(dt)
                .as_raw_ptr(),
            nsecs,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> class marks a deadline in the future.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDeadlineTimer& QDeadlineTimer::operator=(const QDeadlineTimer& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> class marks a deadline in the future.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> class is usually used to calculate future deadlines and verify whether the deadline has expired. <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> can also be used for deadlines without expiration ("forever"). It forms a counterpart to <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a>, which calculates how much time has elapsed since <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#start">QElapsedTimer::start</a>() was called.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> provides a more convenient API compared to <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#hasExpired">QElapsedTimer::hasExpired</a>().</p>
    /// <p>The typical use-case for the class is to create a <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> before the operation in question is started, and then use <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>() or <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#hasExpired">hasExpired</a>() to determine whether to continue trying the operation. <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> objects can be passed to functions being called to execute this operation so they know how long to still operate.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">void</span> executeOperation(<span class="type">int</span> msecs)
    ///   {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#QDeadlineTimer">QDeadlineTimer</a></span> deadline(msecs);
    /// &#32;     <span class="keyword">do</span> {
    /// &#32;   &#32;     <span class="keyword">if</span> (readFromDevice(deadline<span class="operator">.</span>remainingTime())
    /// &#32;   &#32;   &#32;     <span class="keyword">break</span>;
    /// &#32;   &#32;     waitForReadyRead(deadline);
    /// &#32;     } <span class="keyword">while</span> (<span class="operator">!</span>deadline<span class="operator">.</span>hasExpired());
    ///   }
    ///
    /// </pre>
    /// <p>Many <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> functions deal with time out values, which all are measured in milliseconds. There are two special values, the same as many other Qt functions named <code>waitFor</code> or similar:</p>
    /// <ul>
    /// <li>0: no time left, expired</li>
    /// <li>-1: infinite time left, timer never expires</li>
    /// </ul>
    /// <a name="reference-clocks"></a></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDeadlineTimer>>,
    ) -> ::cpp_utils::MutRef<crate::QDeadlineTimer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_operator_(
            self as *mut crate::QDeadlineTimer,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDeadlineTimer>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> that is expired but is guaranteed to contain the current time. Objects created by this function can participate in the calculation of how long a timer is overdue, using the <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>() function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDeadlineTimer QDeadlineTimer::current(Qt::TimerType timerType = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#current">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> that is expired but is guaranteed to contain the current time. Objects created by this function can participate in the calculation of how long a timer is overdue, using the <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>() function.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object will be constructed with the specified <i>timerType</i>.</p></div>
    pub unsafe fn current_1a(
        timer_type: crate::TimerType,
    ) -> ::cpp_utils::CppBox<crate::QDeadlineTimer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_current(timer_type);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> that is expired but is guaranteed to contain the current time. Objects created by this function can participate in the calculation of how long a timer is overdue, using the <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>() function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDeadlineTimer QDeadlineTimer::current()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#current">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> that is expired but is guaranteed to contain the current time. Objects created by this function can participate in the calculation of how long a timer is overdue, using the <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>() function.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object will be constructed with the specified <i>timerType</i>.</p></div>
    pub unsafe fn current_0a() -> ::cpp_utils::CppBox<crate::QDeadlineTimer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_current1();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the absolute time point for the deadline stored in <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object, calculated in milliseconds relative to the reference clock, the same as <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#msecsSinceReference">QElapsedTimer::msecsSinceReference</a>(). The value will be in the past if this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> has expired.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QDeadlineTimer::deadline() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the absolute time point for the deadline stored in <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object, calculated in milliseconds relative to the reference clock, the same as <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#msecsSinceReference">QElapsedTimer::msecsSinceReference</a>(). The value will be in the past if this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> has expired.</p>
    /// <p>If this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> never expires, this function returns <code>std::numeric_limits&lt;qint64&gt;::max()</code>.</p>
    /// <p>This function can be used to calculate the amount of time a timer is overdue, by subtracting <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#current">QDeadlineTimer::current</a>() or <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#msecsSinceReference">QElapsedTimer::msecsSinceReference</a>(), as in the following example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#qint64-typedef">qint64</a></span> realTimeLeft <span class="operator">=</span> deadline<span class="operator">.</span>deadline();
    ///   <span class="keyword">if</span> (realTimeLeft <span class="operator">!</span><span class="operator">=</span> (std<span class="operator">::</span>numeric_limits<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#qint64-typedef">qint64</a></span><span class="operator">&gt;</span><span class="operator">::</span>max)()) {
    /// &#32;     realTimeLeft <span class="operator">-</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#QDeadlineTimer">QDeadlineTimer</a></span><span class="operator">::</span>current()<span class="operator">.</span>deadline();
    /// &#32;     <span class="comment">// or:</span>
    /// &#32;     <span class="comment">//QElapsedTimer timer;</span>
    /// &#32;     <span class="comment">//timer.start();</span>
    /// &#32;     <span class="comment">//realTimeLeft -= timer.msecsSinceReference();</span>
    ///   }
    ///
    /// </pre>
    /// <p><b>Note: </b>Timers that were created as expired have an indetermine time point in the past as their deadline, so the above calculation may not work.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadlineNSecs">deadlineNSecs</a>(), and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setDeadline">setDeadline</a>().</p></div>
    pub unsafe fn deadline(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_deadline(self as *const crate::QDeadlineTimer)
    }

    /// <p>Returns the absolute time point for the deadline stored in <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object, calculated in nanoseconds relative to the reference clock, the same as <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#msecsSinceReference">QElapsedTimer::msecsSinceReference</a>(). The value will be in the past if this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> has expired.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QDeadlineTimer::deadlineNSecs() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadlineNSecs">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the absolute time point for the deadline stored in <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object, calculated in nanoseconds relative to the reference clock, the same as <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#msecsSinceReference">QElapsedTimer::msecsSinceReference</a>(). The value will be in the past if this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> has expired.</p>
    /// <p>If this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> never expires, this function returns <code>std::numeric_limits&lt;qint64&gt;::max()</code>.</p>
    /// <p>This function can be used to calculate the amount of time a timer is overdue, by subtracting <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#current">QDeadlineTimer::current</a>(), as in the following example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#qint64-typedef">qint64</a></span> realTimeLeft <span class="operator">=</span> deadline<span class="operator">.</span>deadlineNSecs();
    ///   <span class="keyword">if</span> (realTimeLeft <span class="operator">!</span><span class="operator">=</span> std<span class="operator">::</span>numeric_limits<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#qint64-typedef">qint64</a></span><span class="operator">&gt;</span><span class="operator">::</span>max())
    /// &#32;     realTimeLeft <span class="operator">-</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#QDeadlineTimer">QDeadlineTimer</a></span><span class="operator">::</span>current()<span class="operator">.</span>deadlineNSecs();
    ///
    /// </pre>
    /// <p><b>Note: </b>Timers that were created as expired have an indetermine time point in the past as their deadline, so the above calculation may not work.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>() and deadlineNSecs().</p></div>
    pub unsafe fn deadline_n_secs(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_deadlineNSecs(
            self as *const crate::QDeadlineTimer,
        )
    }

    /// <p>Returns true if this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object has expired, false if there remains time left. For objects that have expired, <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>() will return zero and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>() will return a time point in the past.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDeadlineTimer::hasExpired() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#hasExpired">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object has expired, false if there remains time left. For objects that have expired, <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>() will return zero and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>() will return a time point in the past.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> objects created with the <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#ForeverConstant-enum">ForeverConstant</a> never expire and this function always returns false for them.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#isForever">isForever</a>() and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>().</p></div>
    pub unsafe fn has_expired(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_hasExpired(
            self as *const crate::QDeadlineTimer,
        )
    }

    /// <p>Returns true if this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object never expires, false otherwise. For timers that never expire, <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>() always returns -1 and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>() returns the maximum value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDeadlineTimer::isForever() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#isForever">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object never expires, false otherwise. For timers that never expire, <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>() always returns -1 and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>() returns the maximum value.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#ForeverConstant-enum">ForeverConstant</a>, <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#hasExpired">hasExpired</a>(), and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>().</p></div>
    pub unsafe fn is_forever(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_isForever(self as *const crate::QDeadlineTimer)
    }

    /// <p>Constructs an expired <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object. For this object, <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>() will return 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDeadlineTimer::QDeadlineTimer(Qt::TimerType type_ = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#QDeadlineTimer">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an expired <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object. For this object, <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>() will return 0.</p>
    /// <p>The timer type <i>timerType</i> may be ignored, since the timer is already expired. Similarly, for optimization purposes, this function will not attempt to obtain the current time and will use a value known to be in the past. Therefore, <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>() may return an unexpected value and this object cannot be used in calculation of how long it is overdue. If that functionality is required, use <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#current">QDeadlineTimer::current</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#hasExpired">hasExpired</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>(), <a href="http://doc.qt.io/qt-5/qt.html#TimerType-enum">Qt::TimerType</a>, and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#current">current</a>().</p></div>
    pub unsafe fn from_timer_type(
        type_: crate::TimerType,
    ) -> ::cpp_utils::CppBox<crate::QDeadlineTimer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_QDeadlineTimer(type_);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> objects created with parameter <i>forever</i> never expire. For such objects, <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>() will return -1, <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>() will return the maximum value, and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#isForever">isForever</a>() will return true.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDeadlineTimer::QDeadlineTimer(QDeadlineTimer::ForeverConstant arg1, Qt::TimerType type_ = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#QDeadlineTimer-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> objects created with parameter <i>forever</i> never expire. For such objects, <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>() will return -1, <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>() will return the maximum value, and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#isForever">isForever</a>() will return true.</p>
    /// <p>The timer type <i>timerType</i> may be ignored, since the timer is already expired.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#ForeverConstant-enum">ForeverConstant</a>, <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#hasExpired">hasExpired</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#isForever">isForever</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>(), and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#timerType">timerType</a>().</p></div>
    pub unsafe fn from_forever_constant_timer_type(
        arg1: crate::q_deadline_timer::ForeverConstant,
        type_: crate::TimerType,
    ) -> ::cpp_utils::CppBox<crate::QDeadlineTimer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_QDeadlineTimer1(arg1, type_);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object with an expiry time of <i>msecs</i> msecs from the moment of the creation of this object, if msecs is positive. If <i>msecs</i> is zero, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be marked as expired, causing <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>() to return zero and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>() to return an indeterminate time point in the past. If <i>msecs</i> is -1, the timer will be set it to never expire, causing <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>() to return -1 and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>() to return the maximum value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDeadlineTimer::QDeadlineTimer(qint64 msecs, Qt::TimerType type = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#QDeadlineTimer-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object with an expiry time of <i>msecs</i> msecs from the moment of the creation of this object, if msecs is positive. If <i>msecs</i> is zero, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be marked as expired, causing <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>() to return zero and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>() to return an indeterminate time point in the past. If <i>msecs</i> is -1, the timer will be set it to never expire, causing <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>() to return -1 and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>() to return the maximum value.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object will be constructed with the specified timer <i>type</i>.</p>
    /// <p>For optimization purposes, if <i>msecs</i> is zero, this function may skip obtaining the current time and may instead use a value known to be in the past. If that happens, <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>() may return an unexpected value and this object cannot be used in calculation of how long it is overdue. If that functionality is required, use <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#current">QDeadlineTimer::current</a>() and add time to it.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#hasExpired">hasExpired</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#isForever">isForever</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>(), and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setRemainingTime">setRemainingTime</a>().</p></div>
    pub unsafe fn from_i64_timer_type(
        msecs: i64,
        type_: crate::TimerType,
    ) -> ::cpp_utils::CppBox<crate::QDeadlineTimer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_QDeadlineTimer2(msecs, type_);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> class marks a deadline in the future.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDeadlineTimer::QDeadlineTimer()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> class marks a deadline in the future.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> class is usually used to calculate future deadlines and verify whether the deadline has expired. <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> can also be used for deadlines without expiration ("forever"). It forms a counterpart to <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a>, which calculates how much time has elapsed since <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#start">QElapsedTimer::start</a>() was called.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> provides a more convenient API compared to <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#hasExpired">QElapsedTimer::hasExpired</a>().</p>
    /// <p>The typical use-case for the class is to create a <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> before the operation in question is started, and then use <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>() or <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#hasExpired">hasExpired</a>() to determine whether to continue trying the operation. <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> objects can be passed to functions being called to execute this operation so they know how long to still operate.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">void</span> executeOperation(<span class="type">int</span> msecs)
    ///   {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#QDeadlineTimer">QDeadlineTimer</a></span> deadline(msecs);
    /// &#32;     <span class="keyword">do</span> {
    /// &#32;   &#32;     <span class="keyword">if</span> (readFromDevice(deadline<span class="operator">.</span>remainingTime())
    /// &#32;   &#32;   &#32;     <span class="keyword">break</span>;
    /// &#32;   &#32;     waitForReadyRead(deadline);
    /// &#32;     } <span class="keyword">while</span> (<span class="operator">!</span>deadline<span class="operator">.</span>hasExpired());
    ///   }
    ///
    /// </pre>
    /// <p>Many <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> functions deal with time out values, which all are measured in milliseconds. There are two special values, the same as many other Qt functions named <code>waitFor</code> or similar:</p>
    /// <ul>
    /// <li>0: no time left, expired</li>
    /// <li>-1: infinite time left, timer never expires</li>
    /// </ul>
    /// <a name="reference-clocks"></a></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QDeadlineTimer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_QDeadlineTimer3();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> objects created with parameter <i>forever</i> never expire. For such objects, <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>() will return -1, <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>() will return the maximum value, and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#isForever">isForever</a>() will return true.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDeadlineTimer::QDeadlineTimer(QDeadlineTimer::ForeverConstant arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#QDeadlineTimer-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> objects created with parameter <i>forever</i> never expire. For such objects, <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>() will return -1, <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>() will return the maximum value, and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#isForever">isForever</a>() will return true.</p>
    /// <p>The timer type <i>timerType</i> may be ignored, since the timer is already expired.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#ForeverConstant-enum">ForeverConstant</a>, <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#hasExpired">hasExpired</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#isForever">isForever</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>(), and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#timerType">timerType</a>().</p></div>
    pub unsafe fn from_forever_constant(
        arg1: crate::q_deadline_timer::ForeverConstant,
    ) -> ::cpp_utils::CppBox<crate::QDeadlineTimer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_QDeadlineTimer5(arg1);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object with an expiry time of <i>msecs</i> msecs from the moment of the creation of this object, if msecs is positive. If <i>msecs</i> is zero, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be marked as expired, causing <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>() to return zero and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>() to return an indeterminate time point in the past. If <i>msecs</i> is -1, the timer will be set it to never expire, causing <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>() to return -1 and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>() to return the maximum value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDeadlineTimer::QDeadlineTimer(qint64 msecs)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#QDeadlineTimer-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object with an expiry time of <i>msecs</i> msecs from the moment of the creation of this object, if msecs is positive. If <i>msecs</i> is zero, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be marked as expired, causing <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>() to return zero and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>() to return an indeterminate time point in the past. If <i>msecs</i> is -1, the timer will be set it to never expire, causing <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>() to return -1 and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>() to return the maximum value.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object will be constructed with the specified timer <i>type</i>.</p>
    /// <p>For optimization purposes, if <i>msecs</i> is zero, this function may skip obtaining the current time and may instead use a value known to be in the past. If that happens, <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>() may return an unexpected value and this object cannot be used in calculation of how long it is overdue. If that functionality is required, use <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#current">QDeadlineTimer::current</a>() and add time to it.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#hasExpired">hasExpired</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#isForever">isForever</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>(), and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setRemainingTime">setRemainingTime</a>().</p></div>
    pub unsafe fn from_i64(msecs: i64) -> ::cpp_utils::CppBox<crate::QDeadlineTimer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_QDeadlineTimer6(msecs);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> class marks a deadline in the future.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDeadlineTimer::QDeadlineTimer(const QDeadlineTimer& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> class marks a deadline in the future.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> class is usually used to calculate future deadlines and verify whether the deadline has expired. <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> can also be used for deadlines without expiration ("forever"). It forms a counterpart to <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a>, which calculates how much time has elapsed since <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#start">QElapsedTimer::start</a>() was called.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> provides a more convenient API compared to <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#hasExpired">QElapsedTimer::hasExpired</a>().</p>
    /// <p>The typical use-case for the class is to create a <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> before the operation in question is started, and then use <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>() or <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#hasExpired">hasExpired</a>() to determine whether to continue trying the operation. <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> objects can be passed to functions being called to execute this operation so they know how long to still operate.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">void</span> executeOperation(<span class="type">int</span> msecs)
    ///   {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#QDeadlineTimer">QDeadlineTimer</a></span> deadline(msecs);
    /// &#32;     <span class="keyword">do</span> {
    /// &#32;   &#32;     <span class="keyword">if</span> (readFromDevice(deadline<span class="operator">.</span>remainingTime())
    /// &#32;   &#32;   &#32;     <span class="keyword">break</span>;
    /// &#32;   &#32;     waitForReadyRead(deadline);
    /// &#32;     } <span class="keyword">while</span> (<span class="operator">!</span>deadline<span class="operator">.</span>hasExpired());
    ///   }
    ///
    /// </pre>
    /// <p>Many <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> functions deal with time out values, which all are measured in milliseconds. There are two special values, the same as many other Qt functions named <code>waitFor</code> or similar:</p>
    /// <ul>
    /// <li>0: no time left, expired</li>
    /// <li>-1: infinite time left, timer never expires</li>
    /// </ul>
    /// <a name="reference-clocks"></a></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDeadlineTimer>>,
    ) -> ::cpp_utils::CppBox<crate::QDeadlineTimer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_QDeadlineTimer4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDeadlineTimer>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QPair<qint64, unsigned int> QDeadlineTimer::_q_data() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn q_data(&self) -> ::cpp_utils::CppBox<crate::QPairOfI64Uint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer__q_data(
            self as *const crate::QDeadlineTimer,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the remaining time in this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object in milliseconds. If the timer has already expired, this function will return zero and it is not possible to obtain the amount of time overdue with this function (to do that, see <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>()). If the timer was set to never expire, this function returns -1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QDeadlineTimer::remainingTime() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the remaining time in this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object in milliseconds. If the timer has already expired, this function will return zero and it is not possible to obtain the amount of time overdue with this function (to do that, see <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>()). If the timer was set to never expire, this function returns -1.</p>
    /// <p>This function is suitable for use in Qt APIs that take a millisecond timeout, such as the many <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> <code>waitFor</code> functions or the timed lock functions in <a href="http://doc.qt.io/qt-5/qmutex.html">QMutex</a>, <a href="http://doc.qt.io/qt-5/qwaitcondition.html">QWaitCondition</a>, <a href="http://doc.qt.io/qt-5/qsemaphore.html">QSemaphore</a>, or <a href="http://doc.qt.io/qt-5/qreadwritelock.html">QReadWriteLock</a>. For example:</p>
    /// <pre class="cpp">
    ///
    ///   mutex<span class="operator">.</span>tryLock(deadline<span class="operator">.</span>remainingTime());
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setRemainingTime">setRemainingTime</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTimeNSecs">remainingTimeNSecs</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#isForever">isForever</a>(), and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#hasExpired">hasExpired</a>().</p></div>
    pub unsafe fn remaining_time(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_remainingTime(
            self as *const crate::QDeadlineTimer,
        )
    }

    /// <p>Returns the remaining time in this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object in nanoseconds. If the timer has already expired, this function will return zero and it is not possible to obtain the amount of time overdue with this function. If the timer was set to never expire, this function returns -1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QDeadlineTimer::remainingTimeNSecs() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTimeNSecs">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the remaining time in this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object in nanoseconds. If the timer has already expired, this function will return zero and it is not possible to obtain the amount of time overdue with this function. If the timer was set to never expire, this function returns -1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#isForever">isForever</a>(), and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#hasExpired">hasExpired</a>().</p></div>
    pub unsafe fn remaining_time_n_secs(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_remainingTimeNSecs(
            self as *const crate::QDeadlineTimer,
        )
    }

    /// <p>Sets the deadline for this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object to be the <i>msecs</i> absolute time point, counted in milliseconds since the reference clock (the same as <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#msecsSinceReference">QElapsedTimer::msecsSinceReference</a>()), and the timer type to <i>timerType</i>. If the value is in the past, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be marked as expired.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDeadlineTimer::setDeadline(qint64 msecs, Qt::TimerType timerType = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setDeadline">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the deadline for this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object to be the <i>msecs</i> absolute time point, counted in milliseconds since the reference clock (the same as <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#msecsSinceReference">QElapsedTimer::msecsSinceReference</a>()), and the timer type to <i>timerType</i>. If the value is in the past, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be marked as expired.</p>
    /// <p>If <i>msecs</i> is <code>std::numeric_limits&lt;qint64&gt;::max()</code>, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be set to never expire.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setPreciseDeadline">setPreciseDeadline</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadlineNSecs">deadlineNSecs</a>(), and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setRemainingTime">setRemainingTime</a>().</p></div>
    pub unsafe fn set_deadline_2a(&mut self, msecs: i64, timer_type: crate::TimerType) {
        crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_setDeadline(
            self as *mut crate::QDeadlineTimer,
            msecs,
            timer_type,
        )
    }

    /// <p>Sets the deadline for this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object to be the <i>msecs</i> absolute time point, counted in milliseconds since the reference clock (the same as <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#msecsSinceReference">QElapsedTimer::msecsSinceReference</a>()), and the timer type to <i>timerType</i>. If the value is in the past, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be marked as expired.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDeadlineTimer::setDeadline(qint64 msecs)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setDeadline">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the deadline for this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object to be the <i>msecs</i> absolute time point, counted in milliseconds since the reference clock (the same as <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#msecsSinceReference">QElapsedTimer::msecsSinceReference</a>()), and the timer type to <i>timerType</i>. If the value is in the past, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be marked as expired.</p>
    /// <p>If <i>msecs</i> is <code>std::numeric_limits&lt;qint64&gt;::max()</code>, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be set to never expire.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setPreciseDeadline">setPreciseDeadline</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadlineNSecs">deadlineNSecs</a>(), and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setRemainingTime">setRemainingTime</a>().</p></div>
    pub unsafe fn set_deadline_1a(&mut self, msecs: i64) {
        crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_setDeadline1(
            self as *mut crate::QDeadlineTimer,
            msecs,
        )
    }

    /// <p>Sets the deadline for this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object to be <i>secs</i> seconds and <i>nsecs</i> nanoseconds since the reference clock epoch (the same as <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#msecsSinceReference">QElapsedTimer::msecsSinceReference</a>()), and the timer type to <i>timerType</i>. If the value is in the past, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be marked as expired.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDeadlineTimer::setPreciseDeadline(qint64 secs, qint64 nsecs = …, Qt::TimerType type = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setPreciseDeadline">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the deadline for this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object to be <i>secs</i> seconds and <i>nsecs</i> nanoseconds since the reference clock epoch (the same as <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#msecsSinceReference">QElapsedTimer::msecsSinceReference</a>()), and the timer type to <i>timerType</i>. If the value is in the past, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be marked as expired.</p>
    /// <p>If <i>secs</i> or <i>nsecs</i> is <code>std::numeric_limits&lt;qint64&gt;::max()</code>, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be set to never expire. If <i>nsecs</i> is more than 1 billion nanoseconds (1 second), then <i>secs</i> will be adjusted accordingly.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setDeadline">setDeadline</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadlineNSecs">deadlineNSecs</a>(), and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setRemainingTime">setRemainingTime</a>().</p></div>
    pub unsafe fn set_precise_deadline_3a(
        &mut self,
        secs: i64,
        nsecs: i64,
        type_: crate::TimerType,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_setPreciseDeadline(
            self as *mut crate::QDeadlineTimer,
            secs,
            nsecs,
            type_,
        )
    }

    /// <p>Sets the deadline for this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object to be <i>secs</i> seconds and <i>nsecs</i> nanoseconds since the reference clock epoch (the same as <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#msecsSinceReference">QElapsedTimer::msecsSinceReference</a>()), and the timer type to <i>timerType</i>. If the value is in the past, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be marked as expired.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDeadlineTimer::setPreciseDeadline(qint64 secs, qint64 nsecs = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setPreciseDeadline">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the deadline for this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object to be <i>secs</i> seconds and <i>nsecs</i> nanoseconds since the reference clock epoch (the same as <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#msecsSinceReference">QElapsedTimer::msecsSinceReference</a>()), and the timer type to <i>timerType</i>. If the value is in the past, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be marked as expired.</p>
    /// <p>If <i>secs</i> or <i>nsecs</i> is <code>std::numeric_limits&lt;qint64&gt;::max()</code>, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be set to never expire. If <i>nsecs</i> is more than 1 billion nanoseconds (1 second), then <i>secs</i> will be adjusted accordingly.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setDeadline">setDeadline</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadlineNSecs">deadlineNSecs</a>(), and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setRemainingTime">setRemainingTime</a>().</p></div>
    pub unsafe fn set_precise_deadline_2a(&mut self, secs: i64, nsecs: i64) {
        crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_setPreciseDeadline1(
            self as *mut crate::QDeadlineTimer,
            secs,
            nsecs,
        )
    }

    /// <p>Sets the deadline for this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object to be <i>secs</i> seconds and <i>nsecs</i> nanoseconds since the reference clock epoch (the same as <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#msecsSinceReference">QElapsedTimer::msecsSinceReference</a>()), and the timer type to <i>timerType</i>. If the value is in the past, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be marked as expired.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDeadlineTimer::setPreciseDeadline(qint64 secs)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setPreciseDeadline">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the deadline for this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object to be <i>secs</i> seconds and <i>nsecs</i> nanoseconds since the reference clock epoch (the same as <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#msecsSinceReference">QElapsedTimer::msecsSinceReference</a>()), and the timer type to <i>timerType</i>. If the value is in the past, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be marked as expired.</p>
    /// <p>If <i>secs</i> or <i>nsecs</i> is <code>std::numeric_limits&lt;qint64&gt;::max()</code>, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be set to never expire. If <i>nsecs</i> is more than 1 billion nanoseconds (1 second), then <i>secs</i> will be adjusted accordingly.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setDeadline">setDeadline</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadline">deadline</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#deadlineNSecs">deadlineNSecs</a>(), and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setRemainingTime">setRemainingTime</a>().</p></div>
    pub unsafe fn set_precise_deadline_1a(&mut self, secs: i64) {
        crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_setPreciseDeadline2(
            self as *mut crate::QDeadlineTimer,
            secs,
        )
    }

    /// <p>Sets the remaining time for this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object to <i>secs</i> seconds plus <i>nsecs</i> nanoseconds from now, if <i>secs</i> has a positive value. If <i>secs</i> is -1, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be set it to never expire. If both parameters are zero, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be marked as expired.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDeadlineTimer::setPreciseRemainingTime(qint64 secs, qint64 nsecs = …, Qt::TimerType type = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setPreciseRemainingTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the remaining time for this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object to <i>secs</i> seconds plus <i>nsecs</i> nanoseconds from now, if <i>secs</i> has a positive value. If <i>secs</i> is -1, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be set it to never expire. If both parameters are zero, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be marked as expired.</p>
    /// <p>The timer type for this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object will be set to the specified <i>timerType</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setRemainingTime">setRemainingTime</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#hasExpired">hasExpired</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#isForever">isForever</a>(), and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>().</p></div>
    pub unsafe fn set_precise_remaining_time_3a(
        &mut self,
        secs: i64,
        nsecs: i64,
        type_: crate::TimerType,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_setPreciseRemainingTime(
            self as *mut crate::QDeadlineTimer,
            secs,
            nsecs,
            type_,
        )
    }

    /// <p>Sets the remaining time for this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object to <i>secs</i> seconds plus <i>nsecs</i> nanoseconds from now, if <i>secs</i> has a positive value. If <i>secs</i> is -1, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be set it to never expire. If both parameters are zero, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be marked as expired.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDeadlineTimer::setPreciseRemainingTime(qint64 secs, qint64 nsecs = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setPreciseRemainingTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the remaining time for this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object to <i>secs</i> seconds plus <i>nsecs</i> nanoseconds from now, if <i>secs</i> has a positive value. If <i>secs</i> is -1, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be set it to never expire. If both parameters are zero, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be marked as expired.</p>
    /// <p>The timer type for this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object will be set to the specified <i>timerType</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setRemainingTime">setRemainingTime</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#hasExpired">hasExpired</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#isForever">isForever</a>(), and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>().</p></div>
    pub unsafe fn set_precise_remaining_time_2a(&mut self, secs: i64, nsecs: i64) {
        crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_setPreciseRemainingTime1(
            self as *mut crate::QDeadlineTimer,
            secs,
            nsecs,
        )
    }

    /// <p>Sets the remaining time for this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object to <i>secs</i> seconds plus <i>nsecs</i> nanoseconds from now, if <i>secs</i> has a positive value. If <i>secs</i> is -1, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be set it to never expire. If both parameters are zero, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be marked as expired.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDeadlineTimer::setPreciseRemainingTime(qint64 secs)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setPreciseRemainingTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the remaining time for this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object to <i>secs</i> seconds plus <i>nsecs</i> nanoseconds from now, if <i>secs</i> has a positive value. If <i>secs</i> is -1, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be set it to never expire. If both parameters are zero, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will be marked as expired.</p>
    /// <p>The timer type for this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object will be set to the specified <i>timerType</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setRemainingTime">setRemainingTime</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#hasExpired">hasExpired</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#isForever">isForever</a>(), and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>().</p></div>
    pub unsafe fn set_precise_remaining_time_1a(&mut self, secs: i64) {
        crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_setPreciseRemainingTime2(
            self as *mut crate::QDeadlineTimer,
            secs,
        )
    }

    /// <p>Sets the remaining time for this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object to <i>msecs</i> milliseconds from now, if <i>msecs</i> has a positive value. If <i>msecs</i> is zero, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object will be marked as expired, whereas a value of -1 will set it to never expire.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDeadlineTimer::setRemainingTime(qint64 msecs, Qt::TimerType type = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setRemainingTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the remaining time for this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object to <i>msecs</i> milliseconds from now, if <i>msecs</i> has a positive value. If <i>msecs</i> is zero, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object will be marked as expired, whereas a value of -1 will set it to never expire.</p>
    /// <p>The timer type for this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object will be set to the specified <i>timerType</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setPreciseRemainingTime">setPreciseRemainingTime</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#hasExpired">hasExpired</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#isForever">isForever</a>(), and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>().</p></div>
    pub unsafe fn set_remaining_time_2a(&mut self, msecs: i64, type_: crate::TimerType) {
        crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_setRemainingTime(
            self as *mut crate::QDeadlineTimer,
            msecs,
            type_,
        )
    }

    /// <p>Sets the remaining time for this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object to <i>msecs</i> milliseconds from now, if <i>msecs</i> has a positive value. If <i>msecs</i> is zero, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object will be marked as expired, whereas a value of -1 will set it to never expire.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDeadlineTimer::setRemainingTime(qint64 msecs)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setRemainingTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the remaining time for this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object to <i>msecs</i> milliseconds from now, if <i>msecs</i> has a positive value. If <i>msecs</i> is zero, this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object will be marked as expired, whereas a value of -1 will set it to never expire.</p>
    /// <p>The timer type for this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object will be set to the specified <i>timerType</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setPreciseRemainingTime">setPreciseRemainingTime</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#hasExpired">hasExpired</a>(), <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#isForever">isForever</a>(), and <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>().</p></div>
    pub unsafe fn set_remaining_time_1a(&mut self, msecs: i64) {
        crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_setRemainingTime1(
            self as *mut crate::QDeadlineTimer,
            msecs,
        )
    }

    /// <p>Changes the timer type for this object to <i>timerType</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDeadlineTimer::setTimerType(Qt::TimerType type)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setTimerType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Changes the timer type for this object to <i>timerType</i>.</p>
    /// <p>The behavior for each possible value of <i>timerType</i> is operating-system dependent. <a href="http://doc.qt.io/qt-5/qt.html#TimerType-enum">Qt::PreciseTimer</a> will use the most precise timer that Qt can find, with resolution of 1 millisecond or better, whereas <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> will try to use a more coarse timer for <a href="http://doc.qt.io/qt-5/qt.html#TimerType-enum">Qt::CoarseTimer</a> and <a href="http://doc.qt.io/qt-5/qt.html#TimerType-enum">Qt::VeryCoarseTimer</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#timerType">timerType</a>() and <a href="http://doc.qt.io/qt-5/qt.html#TimerType-enum">Qt::TimerType</a>.</p></div>
    pub unsafe fn set_timer_type(&mut self, type_: crate::TimerType) {
        crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_setTimerType(
            self as *mut crate::QDeadlineTimer,
            type_,
        )
    }

    /// <p>Swaps this deadline timer with the <i>other</i> deadline timer.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDeadlineTimer::swap(QDeadlineTimer& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps this deadline timer with the <i>other</i> deadline timer.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QDeadlineTimer>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_swap(
            self as *mut crate::QDeadlineTimer,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QDeadlineTimer>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the timer type is active for this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```Qt::TimerType QDeadlineTimer::timerType() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#timerType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the timer type is active for this object.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#setTimerType">setTimerType</a>().</p></div>
    pub unsafe fn timer_type(&self) -> crate::TimerType {
        crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_timerType(self as *const crate::QDeadlineTimer)
    }
}

pub mod q_text_stream {
    //! C++ type: <span style='color: green;'>```QTextStream```</span>

    /// <p>This enum specifies which notations to use for expressing <code>float</code> and <code>double</code> as strings.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QTextStream::RealNumberNotation```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#RealNumberNotation-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum specifies which notations to use for expressing <code>float</code> and <code>double</code> as strings.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#setRealNumberNotation">setRealNumberNotation</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct RealNumberNotation(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for RealNumberNotation {
        fn from(value: ::std::os::raw::c_int) -> Self {
            RealNumberNotation(value)
        }
    }

    impl From<RealNumberNotation> for ::std::os::raw::c_int {
        fn from(value: RealNumberNotation) -> Self {
            value.0
        }
    }

    impl RealNumberNotation {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl RealNumberNotation {
        /// Scientific or fixed-point notation, depending on which makes most sense (<code>printf()</code>'s <code>%g</code> flag). (C++ enum variant: <span style='color: green;'>```SmartNotation = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const SmartNotation: crate::q_text_stream::RealNumberNotation =
            crate::q_text_stream::RealNumberNotation(0);
        /// Fixed-point notation (<code>printf()</code>'s <code>%f</code> flag). (C++ enum variant: <span style='color: green;'>```FixedNotation = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const FixedNotation: crate::q_text_stream::RealNumberNotation =
            crate::q_text_stream::RealNumberNotation(1);
        /// Scientific notation (<code>printf()</code>'s <code>%e</code> flag). (C++ enum variant: <span style='color: green;'>```ScientificNotation = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const ScientificNotation: crate::q_text_stream::RealNumberNotation =
            crate::q_text_stream::RealNumberNotation(2);
    }

    /// <p>This enum specifies how to align text in fields when the field is wider than the text that occupies it.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QTextStream::FieldAlignment```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#FieldAlignment-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum specifies how to align text in fields when the field is wider than the text that occupies it.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#setFieldAlignment">setFieldAlignment</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct FieldAlignment(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for FieldAlignment {
        fn from(value: ::std::os::raw::c_int) -> Self {
            FieldAlignment(value)
        }
    }

    impl From<FieldAlignment> for ::std::os::raw::c_int {
        fn from(value: FieldAlignment) -> Self {
            value.0
        }
    }

    impl FieldAlignment {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl FieldAlignment {
        /// Pad on the right side of fields. (C++ enum variant: <span style='color: green;'>```AlignLeft = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const AlignLeft: crate::q_text_stream::FieldAlignment =
            crate::q_text_stream::FieldAlignment(0);
        /// Pad on the left side of fields. (C++ enum variant: <span style='color: green;'>```AlignRight = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const AlignRight: crate::q_text_stream::FieldAlignment =
            crate::q_text_stream::FieldAlignment(1);
        /// Pad on both sides of field. (C++ enum variant: <span style='color: green;'>```AlignCenter = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const AlignCenter: crate::q_text_stream::FieldAlignment =
            crate::q_text_stream::FieldAlignment(2);
        /// Same as AlignRight, except that the sign of a number is flush left. (C++ enum variant: <span style='color: green;'>```AlignAccountingStyle = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const AlignAccountingStyle: crate::q_text_stream::FieldAlignment =
            crate::q_text_stream::FieldAlignment(3);
    }

    /// <p>This enum describes the current status of the text stream.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QTextStream::Status```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#Status-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the current status of the text stream.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#status">status</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Status(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Status {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Status(value)
        }
    }

    impl From<Status> for ::std::os::raw::c_int {
        fn from(value: Status) -> Self {
            value.0
        }
    }

    impl Status {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Status {
        /// The text stream is operating normally. (C++ enum variant: <span style='color: green;'>```Ok = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const Ok: crate::q_text_stream::Status = crate::q_text_stream::Status(0);
        /// The text stream has read past the end of the data in the underlying device. (C++ enum variant: <span style='color: green;'>```ReadPastEnd = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const ReadPastEnd: crate::q_text_stream::Status = crate::q_text_stream::Status(1);
        /// The text stream has read corrupt data. (C++ enum variant: <span style='color: green;'>```ReadCorruptData = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const ReadCorruptData: crate::q_text_stream::Status = crate::q_text_stream::Status(2);
        /// The text stream cannot write to the underlying device. (C++ enum variant: <span style='color: green;'>```WriteFailed = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const WriteFailed: crate::q_text_stream::Status = crate::q_text_stream::Status(3);
    }

    /// <p>This enum specifies various flags that can be set to affect the output of integers, <code>float</code>s, and <code>double</code>s.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QTextStream::NumberFlag```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum specifies various flags that can be set to affect the output of integers, <code>float</code>s, and <code>double</code>s.</p>
    ///
    /// <p>The NumberFlags type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;NumberFlag&gt;. It stores an OR combination of NumberFlag values.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">setNumberFlags</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct NumberFlag(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for NumberFlag {
        fn from(value: ::std::os::raw::c_int) -> Self {
            NumberFlag(value)
        }
    }

    impl From<NumberFlag> for ::std::os::raw::c_int {
        fn from(value: NumberFlag) -> Self {
            value.0
        }
    }

    impl NumberFlag {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl NumberFlag {
        /// Show the base as a prefix if the base is 16 ("0x"), 8 ("0"), or 2 ("0b"). (C++ enum variant: <span style='color: green;'>```ShowBase = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const ShowBase: crate::q_text_stream::NumberFlag = crate::q_text_stream::NumberFlag(1);
        /// Always put the decimal separator in numbers, even if there are no decimals. (C++ enum variant: <span style='color: green;'>```ForcePoint = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const ForcePoint: crate::q_text_stream::NumberFlag =
            crate::q_text_stream::NumberFlag(2);
        /// Always put the sign in numbers, even for positive numbers. (C++ enum variant: <span style='color: green;'>```ForceSign = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const ForceSign: crate::q_text_stream::NumberFlag = crate::q_text_stream::NumberFlag(4);
        /// Use uppercase versions of base prefixes ("0X", "0B"). (C++ enum variant: <span style='color: green;'>```UppercaseBase = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const UppercaseBase: crate::q_text_stream::NumberFlag =
            crate::q_text_stream::NumberFlag(8);
        /// Use uppercase letters for expressing digits 10 to 35 instead of lowercase. (C++ enum variant: <span style='color: green;'>```UppercaseDigits = 16```</span>)
        #[allow(non_upper_case_globals)]
        pub const UppercaseDigits: crate::q_text_stream::NumberFlag =
            crate::q_text_stream::NumberFlag(16);
    }

    impl From<crate::q_text_stream::NumberFlag> for crate::QFlags<crate::q_text_stream::NumberFlag> {
        fn from(value: crate::q_text_stream::NumberFlag) -> Self {
            Self::from(value.to_int())
        }
    }

    impl<T: Into<crate::QFlags<crate::q_text_stream::NumberFlag>>> std::ops::BitOr<T>
        for crate::q_text_stream::NumberFlag
    {
        type Output = crate::QFlags<crate::q_text_stream::NumberFlag>;
        fn bitor(self, rhs: T) -> crate::QFlags<crate::q_text_stream::NumberFlag> {
            Into::<crate::QFlags<crate::q_text_stream::NumberFlag>>::into(self) | rhs
        }
    }

    /// <p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setIntegerBase">QTextStream::setIntegerBase</a>(2) on <i>stream</i> and returns <i>stream</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& bin(QTextStream& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#bin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setIntegerBase">QTextStream::setIntegerBase</a>(2) on <i>stream</i> and returns <i>stream</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#oct">oct</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#dec">dec</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#hex">hex</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#qtextstream-manipulators">QTextStream manipulators</a>.</p></div>
    pub unsafe fn bin(
        s: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QTextStream>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_bin(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QTextStream>>::cast_into(s)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Toggles insertion of the Byte Order Mark on <i>stream</i> when <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> is used with a UTF codec.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& bom(QTextStream& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#bom">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Toggles insertion of the Byte Order Mark on <i>stream</i> when <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> is used with a UTF codec.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#setGenerateByteOrderMark">QTextStream::setGenerateByteOrderMark</a>() and <a href="http://doc.qt.io/qt-5/qtextstream.html#qtextstream-manipulators">QTextStream manipulators</a>.</p></div>
    pub unsafe fn bom(
        s: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QTextStream>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_bom(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QTextStream>>::cast_into(s)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setFieldAlignment">QTextStream::setFieldAlignment</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#FieldAlignment-enum">QTextStream::AlignCenter</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& center(QTextStream& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#center">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setFieldAlignment">QTextStream::setFieldAlignment</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#FieldAlignment-enum">QTextStream::AlignCenter</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#left">left</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#right">right</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#qtextstream-manipulators">QTextStream manipulators</a>.</p></div>
    pub unsafe fn center(
        s: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QTextStream>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_center(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QTextStream>>::cast_into(s)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setIntegerBase">QTextStream::setIntegerBase</a>(10) on <i>stream</i> and returns <i>stream</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& dec(QTextStream& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#dec">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setIntegerBase">QTextStream::setIntegerBase</a>(10) on <i>stream</i> and returns <i>stream</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#bin">bin</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#oct">oct</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#hex">hex</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#qtextstream-manipulators">QTextStream manipulators</a>.</p></div>
    pub unsafe fn dec(
        s: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QTextStream>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dec(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QTextStream>>::cast_into(s)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Writes '\n' to the <i>stream</i> and flushes the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& endl(QTextStream& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#endl">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes '\n' to the <i>stream</i> and flushes the stream.</p>
    /// <p>Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   stream <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="char">'\n'</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> flush;
    ///
    /// </pre>
    /// <p>Note: On Windows, all '\n' characters are written as '\r\n' if <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>'s device or string is opened using the <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">QIODevice::Text</a> flag.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#flush-1">flush</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#reset-1">reset</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#qtextstream-manipulators">QTextStream manipulators</a>.</p></div>
    pub unsafe fn endl(
        s: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QTextStream>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_endl(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QTextStream>>::cast_into(s)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setRealNumberNotation">QTextStream::setRealNumberNotation</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#RealNumberNotation-enum">QTextStream::FixedNotation</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& fixed(QTextStream& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#fixed">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setRealNumberNotation">QTextStream::setRealNumberNotation</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#RealNumberNotation-enum">QTextStream::FixedNotation</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#scientific">scientific</a>() and <a href="http://doc.qt.io/qt-5/qtextstream.html#qtextstream-manipulators">QTextStream manipulators</a>.</p></div>
    pub unsafe fn fixed(
        s: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QTextStream>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_fixed(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QTextStream>>::cast_into(s)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```QTextStream& flush(QTextStream& s)```</span>.
    pub unsafe fn flush(
        s: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QTextStream>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_flush(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QTextStream>>::cast_into(s)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">QTextStream::setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">QTextStream::numberFlags</a>() | <a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">QTextStream::ForcePoint</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& forcepoint(QTextStream& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#forcepoint">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">QTextStream::setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">QTextStream::numberFlags</a>() | <a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">QTextStream::ForcePoint</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#noforcepoint">noforcepoint</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#forcesign">forcesign</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#showbase">showbase</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#qtextstream-manipulators">QTextStream manipulators</a>.</p></div>
    pub unsafe fn forcepoint(
        s: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QTextStream>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_forcepoint(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QTextStream>>::cast_into(s)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">QTextStream::setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">QTextStream::numberFlags</a>() | <a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">QTextStream::ForceSign</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& forcesign(QTextStream& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#forcesign">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">QTextStream::setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">QTextStream::numberFlags</a>() | <a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">QTextStream::ForceSign</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#noforcesign">noforcesign</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#forcepoint">forcepoint</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#showbase">showbase</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#qtextstream-manipulators">QTextStream manipulators</a>.</p></div>
    pub unsafe fn forcesign(
        s: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QTextStream>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_forcesign(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QTextStream>>::cast_into(s)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setIntegerBase">QTextStream::setIntegerBase</a>(16) on <i>stream</i> and returns <i>stream</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& hex(QTextStream& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#hex">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setIntegerBase">QTextStream::setIntegerBase</a>(16) on <i>stream</i> and returns <i>stream</i>.</p>
    /// <p><b>Note: </b>The hex modifier can only be used for writing to streams.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#bin">bin</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#oct">oct</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#dec">dec</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#qtextstream-manipulators">QTextStream manipulators</a>.</p></div>
    pub unsafe fn hex(
        s: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QTextStream>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_hex(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QTextStream>>::cast_into(s)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setFieldAlignment">QTextStream::setFieldAlignment</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#FieldAlignment-enum">QTextStream::AlignLeft</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& left(QTextStream& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#left">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setFieldAlignment">QTextStream::setFieldAlignment</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#FieldAlignment-enum">QTextStream::AlignLeft</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#right">right</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#center">center</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#qtextstream-manipulators">QTextStream manipulators</a>.</p></div>
    pub unsafe fn left(
        s: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QTextStream>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_left(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QTextStream>>::cast_into(s)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">QTextStream::setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">QTextStream::numberFlags</a>() &amp; ~<a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">QTextStream::UppercaseBase</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& lowercasebase(QTextStream& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#lowercasebase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">QTextStream::setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">QTextStream::numberFlags</a>() &amp; ~<a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">QTextStream::UppercaseBase</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#uppercasebase">uppercasebase</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#lowercasedigits">lowercasedigits</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#qtextstream-manipulators">QTextStream manipulators</a>.</p></div>
    pub unsafe fn lowercasebase(
        s: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QTextStream>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_lowercasebase(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QTextStream>>::cast_into(s)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">QTextStream::setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">QTextStream::numberFlags</a>() &amp; ~<a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">QTextStream::UppercaseDigits</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& lowercasedigits(QTextStream& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#lowercasedigits">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">QTextStream::setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">QTextStream::numberFlags</a>() &amp; ~<a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">QTextStream::UppercaseDigits</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#uppercasedigits">uppercasedigits</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#lowercasebase">lowercasebase</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#qtextstream-manipulators">QTextStream manipulators</a>.</p></div>
    pub unsafe fn lowercasedigits(
        s: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QTextStream>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_lowercasedigits(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QTextStream>>::cast_into(s)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">QTextStream::setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">QTextStream::numberFlags</a>() &amp; ~<a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">QTextStream::ForcePoint</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& noforcepoint(QTextStream& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#noforcepoint">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">QTextStream::setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">QTextStream::numberFlags</a>() &amp; ~<a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">QTextStream::ForcePoint</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#forcepoint">forcepoint</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#noforcesign">noforcesign</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#noshowbase">noshowbase</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#qtextstream-manipulators">QTextStream manipulators</a>.</p></div>
    pub unsafe fn noforcepoint(
        s: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QTextStream>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_noforcepoint(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QTextStream>>::cast_into(s)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">QTextStream::setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">QTextStream::numberFlags</a>() &amp; ~<a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">QTextStream::ForceSign</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& noforcesign(QTextStream& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#noforcesign">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">QTextStream::setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">QTextStream::numberFlags</a>() &amp; ~<a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">QTextStream::ForceSign</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#forcesign">forcesign</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#noforcepoint">noforcepoint</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#noshowbase">noshowbase</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#qtextstream-manipulators">QTextStream manipulators</a>.</p></div>
    pub unsafe fn noforcesign(
        s: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QTextStream>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_noforcesign(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QTextStream>>::cast_into(s)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">QTextStream::setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">QTextStream::numberFlags</a>() &amp; ~<a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">QTextStream::ShowBase</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& noshowbase(QTextStream& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#noshowbase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">QTextStream::setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">QTextStream::numberFlags</a>() &amp; ~<a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">QTextStream::ShowBase</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#showbase">showbase</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#noforcesign">noforcesign</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#noforcepoint">noforcepoint</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#qtextstream-manipulators">QTextStream manipulators</a>.</p></div>
    pub unsafe fn noshowbase(
        s: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QTextStream>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_noshowbase(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QTextStream>>::cast_into(s)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setIntegerBase">QTextStream::setIntegerBase</a>(8) on <i>stream</i> and returns <i>stream</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& oct(QTextStream& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#oct">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setIntegerBase">QTextStream::setIntegerBase</a>(8) on <i>stream</i> and returns <i>stream</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#bin">bin</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#dec">dec</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#hex">hex</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#qtextstream-manipulators">QTextStream manipulators</a>.</p></div>
    pub unsafe fn oct(
        s: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QTextStream>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_oct(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QTextStream>>::cast_into(s)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Equivalent to <a href="http://doc.qt.io/qt-5/qtextstream.html#setFieldWidth">QTextStream::setFieldWidth</a>(<i>width</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStreamManipulator qSetFieldWidth(int width)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#qSetFieldWidth">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Equivalent to <a href="http://doc.qt.io/qt-5/qtextstream.html#setFieldWidth">QTextStream::setFieldWidth</a>(<i>width</i>).</p></div>
    pub unsafe fn q_set_field_width(
        width: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QTextStreamManipulator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_qSetFieldWidth(width);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Equivalent to <a href="http://doc.qt.io/qt-5/qtextstream.html#setPadChar">QTextStream::setPadChar</a>(<i>ch</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStreamManipulator qSetPadChar(QChar ch)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#qSetPadChar">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Equivalent to <a href="http://doc.qt.io/qt-5/qtextstream.html#setPadChar">QTextStream::setPadChar</a>(<i>ch</i>).</p></div>
    pub unsafe fn q_set_pad_char(
        ch: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> ::cpp_utils::CppBox<crate::QTextStreamManipulator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_qSetPadChar(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(ch).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Equivalent to <a href="http://doc.qt.io/qt-5/qtextstream.html#setRealNumberPrecision">QTextStream::setRealNumberPrecision</a>(<i>precision</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStreamManipulator qSetRealNumberPrecision(int precision)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#qSetRealNumberPrecision">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Equivalent to <a href="http://doc.qt.io/qt-5/qtextstream.html#setRealNumberPrecision">QTextStream::setRealNumberPrecision</a>(<i>precision</i>).</p></div>
    pub unsafe fn q_set_real_number_precision(
        precision: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QTextStreamManipulator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_qSetRealNumberPrecision(precision);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QTextStream& reset(QTextStream& s)```</span>.
    pub unsafe fn reset(
        s: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QTextStream>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_reset(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QTextStream>>::cast_into(s)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setFieldAlignment">QTextStream::setFieldAlignment</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#FieldAlignment-enum">QTextStream::AlignRight</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& right(QTextStream& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#right">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setFieldAlignment">QTextStream::setFieldAlignment</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#FieldAlignment-enum">QTextStream::AlignRight</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#left">left</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#center">center</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#qtextstream-manipulators">QTextStream manipulators</a>.</p></div>
    pub unsafe fn right(
        s: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QTextStream>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_right(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QTextStream>>::cast_into(s)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setRealNumberNotation">QTextStream::setRealNumberNotation</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#RealNumberNotation-enum">QTextStream::ScientificNotation</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& scientific(QTextStream& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#scientific">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setRealNumberNotation">QTextStream::setRealNumberNotation</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#RealNumberNotation-enum">QTextStream::ScientificNotation</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#fixed">fixed</a>() and <a href="http://doc.qt.io/qt-5/qtextstream.html#qtextstream-manipulators">QTextStream manipulators</a>.</p></div>
    pub unsafe fn scientific(
        s: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QTextStream>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_scientific(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QTextStream>>::cast_into(s)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">QTextStream::setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">QTextStream::numberFlags</a>() | <a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">QTextStream::ShowBase</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& showbase(QTextStream& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#showbase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">QTextStream::setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">QTextStream::numberFlags</a>() | <a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">QTextStream::ShowBase</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#noshowbase">noshowbase</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#forcesign">forcesign</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#forcepoint">forcepoint</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#qtextstream-manipulators">QTextStream manipulators</a>.</p></div>
    pub unsafe fn showbase(
        s: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QTextStream>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_showbase(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QTextStream>>::cast_into(s)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">QTextStream::setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">QTextStream::numberFlags</a>() | <a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">QTextStream::UppercaseBase</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& uppercasebase(QTextStream& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#uppercasebase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">QTextStream::setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">QTextStream::numberFlags</a>() | <a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">QTextStream::UppercaseBase</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#lowercasebase">lowercasebase</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#uppercasedigits">uppercasedigits</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#qtextstream-manipulators">QTextStream manipulators</a>.</p></div>
    pub unsafe fn uppercasebase(
        s: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QTextStream>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_uppercasebase(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QTextStream>>::cast_into(s)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">QTextStream::setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">QTextStream::numberFlags</a>() | <a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">QTextStream::UppercaseDigits</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& uppercasedigits(QTextStream& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#uppercasedigits">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">QTextStream::setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">QTextStream::numberFlags</a>() | <a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">QTextStream::UppercaseDigits</a>) on <i>stream</i> and returns <i>stream</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#lowercasedigits">lowercasedigits</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#uppercasebase">uppercasebase</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#qtextstream-manipulators">QTextStream manipulators</a>.</p></div>
    pub unsafe fn uppercasedigits(
        s: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QTextStream>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_uppercasedigits(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QTextStream>>::cast_into(s)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#skipWhiteSpace">skipWhiteSpace()</a> on <i>stream</i> and returns <i>stream</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& ws(QTextStream& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#ws">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Calls <a href="http://doc.qt.io/qt-5/qtextstream.html#skipWhiteSpace">skipWhiteSpace()</a> on <i>stream</i> and returns <i>stream</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#qtextstream-manipulators">QTextStream manipulators</a>.</p></div>
    pub unsafe fn ws(
        s: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QTextStream>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ws(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QTextStream>>::cast_into(s)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> class provides a convenient interface for reading and writing text.</p>
///
/// C++ class: <span style='color: green;'>```QTextStream```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtextstream.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> class provides a convenient interface for reading and writing text.</p>
/// <p><a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> can operate on a <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>, a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> or a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. Using <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>'s streaming operators, you can conveniently read and write words, lines and numbers. For generating text, <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> supports formatting options for field padding and alignment, and formatting of numbers. Example:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qfile.html">QFile</a></span> data(<span class="string">"output.txt"</span>);
///   <span class="keyword">if</span> (data<span class="operator">.</span>open(<span class="type"><a href="http://doc.qt.io/qt-5/qfile.html">QFile</a></span><span class="operator">::</span>WriteOnly <span class="operator">|</span> <span class="type"><a href="http://doc.qt.io/qt-5/qfile.html">QFile</a></span><span class="operator">::</span>Truncate)) {
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qtextstream.html#QTextStream">QTextStream</a></span> out(<span class="operator">&amp;</span>data);
/// &#32;     out <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Result: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <a href="http://doc.qt.io/qt-5/qtextstream.html#qSetFieldWidth">qSetFieldWidth</a>(<span class="number">10</span>) <span class="operator">&lt;</span><span class="operator">&lt;</span> left <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">3.14</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">2.7</span>;
/// &#32;     <span class="comment">// writes "Result: 3.14&#32;     2.7&#32;      "</span>
///   }
///
/// </pre>
/// <p>It's also common to use <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> to read console input and write console output. <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> is locale aware, and will automatically decode standard input using the correct codec. Example:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qtextstream.html#QTextStream">QTextStream</a></span> stream(stdin);
///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> line;
///   <span class="keyword">while</span> (stream<span class="operator">.</span>readLineInto(<span class="operator">&amp;</span>line)) {
/// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   }
///
/// </pre>
/// <p>Besides using <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>'s constructors, you can also set the device or string <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> operates on by calling <a href="http://doc.qt.io/qt-5/qtextstream.html#setDevice">setDevice</a>() or <a href="http://doc.qt.io/qt-5/qtextstream.html#setString">setString</a>(). You can seek to a position by calling <a href="http://doc.qt.io/qt-5/qtextstream.html#seek">seek</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#atEnd">atEnd</a>() will return true when there is no data left to be read. If you call <a href="http://doc.qt.io/qt-5/qtextstream.html#flush">flush</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> will empty all data from its write buffer into the device and call <a href="http://doc.qt.io/qt-5/qtextstream.html#flush">flush</a>() on the device.</p>
/// <p>Internally, <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> uses a Unicode based buffer, and <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> is used by <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> to automatically support different character sets. By default, <a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForLocale">QTextCodec::codecForLocale</a>() is used for reading and writing, but you can also set the codec by calling <a href="http://doc.qt.io/qt-5/qtextstream.html#setCodec">setCodec</a>(). Automatic Unicode detection is also supported. When this feature is enabled (the default behavior), <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> will detect the UTF-16 or the UTF-32 BOM (Byte Order Mark) and switch to the appropriate UTF codec when reading. <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> does not write a BOM by default, but you can enable this by calling <a href="http://doc.qt.io/qt-5/qtextstream.html#setGenerateByteOrderMark">setGenerateByteOrderMark</a>(true). When <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> operates on a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> directly, the codec is disabled.</p>
/// <p>There are three general ways to use <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> when reading text files:</p>
/// <ul>
/// <li>Chunk by chunk, by calling <a href="http://doc.qt.io/qt-5/qtextstream.html#readLine">readLine</a>() or <a href="http://doc.qt.io/qt-5/qtextstream.html#readAll">readAll</a>().</li>
/// <li>Word by word. <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> supports streaming into <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>s, <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>s and char* buffers. Words are delimited by space, and leading white space is automatically skipped.</li>
/// <li>Character by character, by streaming into <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> or char types. This method is often used for convenient input handling when parsing files, independent of character encoding and end-of-line semantics. To skip white space, call <a href="http://doc.qt.io/qt-5/qtextstream.html#skipWhiteSpace">skipWhiteSpace</a>().</li>
/// </ul>
/// <p>Since the text stream uses a buffer, you should not read from the stream using the implementation of a superclass. For instance, if you have a <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> and read from it directly using <a href="http://doc.qt.io/qt-5/qiodevice.html#readLine">QFile::readLine</a>() instead of using the stream, the text stream's internal position will be out of sync with the file's position.</p>
/// <p>By default, when reading numbers from a stream of text, <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> will automatically detect the number's base representation. For example, if the number starts with "0x", it is assumed to be in hexadecimal form. If it starts with the digits 1-9, it is assumed to be in decimal form, and so on. You can set the integer base, thereby disabling the automatic detection, by calling <a href="http://doc.qt.io/qt-5/qtextstream.html#setIntegerBase">setIntegerBase</a>(). Example:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qtextstream.html#QTextStream">QTextStream</a></span> in(<span class="string">"0x50 0x20"</span>);
///   <span class="type">int</span> firstNumber<span class="operator">,</span> secondNumber;
///
///   in <span class="operator">&gt;</span><span class="operator">&gt;</span> firstNumber;             <span class="comment">// firstNumber == 80</span>
///   in <span class="operator">&gt;</span><span class="operator">&gt;</span> dec <span class="operator">&gt;</span><span class="operator">&gt;</span> secondNumber;     <span class="comment">// secondNumber == 0</span>
///
///   <span class="type">char</span> ch;
///   in <span class="operator">&gt;</span><span class="operator">&gt;</span> ch;                      <span class="comment">// ch == 'x'</span>
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> supports many formatting options for generating text. You can set the field width and pad character by calling <a href="http://doc.qt.io/qt-5/qtextstream.html#setFieldWidth">setFieldWidth</a>() and <a href="http://doc.qt.io/qt-5/qtextstream.html#setPadChar">setPadChar</a>(). Use <a href="http://doc.qt.io/qt-5/qtextstream.html#setFieldAlignment">setFieldAlignment</a>() to set the alignment within each field. For real numbers, call <a href="http://doc.qt.io/qt-5/qtextstream.html#setRealNumberNotation">setRealNumberNotation</a>() and <a href="http://doc.qt.io/qt-5/qtextstream.html#setRealNumberPrecision">setRealNumberPrecision</a>() to set the notation (<a href="http://doc.qt.io/qt-5/qtextstream.html#RealNumberNotation-enum">SmartNotation</a>, <a href="http://doc.qt.io/qt-5/qtextstream.html#RealNumberNotation-enum">ScientificNotation</a>, <a href="http://doc.qt.io/qt-5/qtextstream.html#RealNumberNotation-enum">FixedNotation</a>) and precision in digits of the generated number. Some extra number formatting options are also available through <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">setNumberFlags</a>().</p>
/// <a name="qtextstream-manipulators"></a><p>Like <code>&lt;iostream&gt;</code> in the standard C++ library, <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> also defines several global manipulator functions:</p>
/// <div class="table"><table class="generic">
///  <thead><tr class="qt-style"><th>Manipulator</th><th>Description</th></tr></thead>
/// <tbody><tr valign="top" class="odd"><td><code>bin</code></td><td>Same as <a href="http://doc.qt.io/qt-5/qtextstream.html#setIntegerBase">setIntegerBase</a>(2).</td></tr>
/// <tr valign="top" class="even"><td><code>oct</code></td><td>Same as <a href="http://doc.qt.io/qt-5/qtextstream.html#setIntegerBase">setIntegerBase</a>(8).</td></tr>
/// <tr valign="top" class="odd"><td><code>dec</code></td><td>Same as <a href="http://doc.qt.io/qt-5/qtextstream.html#setIntegerBase">setIntegerBase</a>(10).</td></tr>
/// <tr valign="top" class="even"><td><code>hex</code></td><td>Same as <a href="http://doc.qt.io/qt-5/qtextstream.html#setIntegerBase">setIntegerBase</a>(16).</td></tr>
/// <tr valign="top" class="odd"><td><code>showbase</code></td><td>Same as <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">numberFlags</a>() | <a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">ShowBase</a>).</td></tr>
/// <tr valign="top" class="even"><td><code>forcesign</code></td><td>Same as <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">numberFlags</a>() | <a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">ForceSign</a>).</td></tr>
/// <tr valign="top" class="odd"><td><code>forcepoint</code></td><td>Same as <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">numberFlags</a>() | <a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">ForcePoint</a>).</td></tr>
/// <tr valign="top" class="even"><td><code>noshowbase</code></td><td>Same as <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">numberFlags</a>() &amp; ~<a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">ShowBase</a>).</td></tr>
/// <tr valign="top" class="odd"><td><code>noforcesign</code></td><td>Same as <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">numberFlags</a>() &amp; ~<a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">ForceSign</a>).</td></tr>
/// <tr valign="top" class="even"><td><code>noforcepoint</code></td><td>Same as <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">numberFlags</a>() &amp; ~<a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">ForcePoint</a>).</td></tr>
/// <tr valign="top" class="odd"><td><code>uppercasebase</code></td><td>Same as <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">numberFlags</a>() | <a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">UppercaseBase</a>).</td></tr>
/// <tr valign="top" class="even"><td><code>uppercasedigits</code></td><td>Same as <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">numberFlags</a>() | <a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">UppercaseDigits</a>).</td></tr>
/// <tr valign="top" class="odd"><td><code>lowercasebase</code></td><td>Same as <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">numberFlags</a>() &amp; ~<a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">UppercaseBase</a>).</td></tr>
/// <tr valign="top" class="even"><td><code>lowercasedigits</code></td><td>Same as <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">setNumberFlags</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">numberFlags</a>() &amp; ~<a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">UppercaseDigits</a>).</td></tr>
/// <tr valign="top" class="odd"><td><code>fixed</code></td><td>Same as <a href="http://doc.qt.io/qt-5/qtextstream.html#setRealNumberNotation">setRealNumberNotation</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#RealNumberNotation-enum">FixedNotation</a>).</td></tr>
/// <tr valign="top" class="even"><td><code>scientific</code></td><td>Same as <a href="http://doc.qt.io/qt-5/qtextstream.html#setRealNumberNotation">setRealNumberNotation</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#RealNumberNotation-enum">ScientificNotation</a>).</td></tr>
/// <tr valign="top" class="odd"><td><code>left</code></td><td>Same as <a href="http://doc.qt.io/qt-5/qtextstream.html#setFieldAlignment">setFieldAlignment</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#FieldAlignment-enum">AlignLeft</a>).</td></tr>
/// <tr valign="top" class="even"><td><code>right</code></td><td>Same as <a href="http://doc.qt.io/qt-5/qtextstream.html#setFieldAlignment">setFieldAlignment</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#FieldAlignment-enum">AlignRight</a>).</td></tr>
/// <tr valign="top" class="odd"><td><code>center</code></td><td>Same as <a href="http://doc.qt.io/qt-5/qtextstream.html#setFieldAlignment">setFieldAlignment</a>(<a href="http://doc.qt.io/qt-5/qtextstream.html#FieldAlignment-enum">AlignCenter</a>).</td></tr>
/// <tr valign="top" class="even"><td><code>endl</code></td><td>Same as operator&lt;&lt;('\n') and <a href="http://doc.qt.io/qt-5/qtextstream.html#flush">flush</a>().</td></tr>
/// <tr valign="top" class="odd"><td><code>flush</code></td><td>Same as <a href="http://doc.qt.io/qt-5/qtextstream.html#flush">flush</a>().</td></tr>
/// <tr valign="top" class="even"><td><code>reset</code></td><td>Same as <a href="http://doc.qt.io/qt-5/qtextstream.html#reset">reset</a>().</td></tr>
/// <tr valign="top" class="odd"><td><code>ws</code></td><td>Same as <a href="http://doc.qt.io/qt-5/qtextstream.html#skipWhiteSpace">skipWhiteSpace</a>().</td></tr>
/// <tr valign="top" class="even"><td><code>bom</code></td><td>Same as <a href="http://doc.qt.io/qt-5/qtextstream.html#setGenerateByteOrderMark">setGenerateByteOrderMark</a>(true).</td></tr>
/// </tbody></table></div>
/// <p>In addition, Qt provides three global manipulators that take a parameter: <a href="http://doc.qt.io/qt-5/qtextstream.html#qSetFieldWidth">qSetFieldWidth</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#qSetPadChar">qSetPadChar</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#qSetRealNumberPrecision">qSetRealNumberPrecision</a>().</p></div>
#[repr(C)]
pub struct QTextStream {
    _unused: u8,
}
impl QTextStream {
    /// <p>Returns <code>true</code> if there is no more data to be read from the <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>; otherwise returns <code>false</code>. This is similar to, but not the same as calling <a href="http://doc.qt.io/qt-5/qiodevice.html#atEnd">QIODevice::atEnd</a>(), as <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> also takes into account its internal Unicode buffer.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTextStream::atEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#atEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if there is no more data to be read from the <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>; otherwise returns <code>false</code>. This is similar to, but not the same as calling <a href="http://doc.qt.io/qt-5/qiodevice.html#atEnd">QIODevice::atEnd</a>(), as <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> also takes into account its internal Unicode buffer.</p></div>
    pub unsafe fn at_end(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_atEnd(self as *const crate::QTextStream)
    }

    /// <p>Returns <code>true</code> if automatic Unicode detection is enabled, otherwise returns <code>false</code>. Automatic Unicode detection is enabled by default.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTextStream::autoDetectUnicode() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#autoDetectUnicode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if automatic Unicode detection is enabled, otherwise returns <code>false</code>. Automatic Unicode detection is enabled by default.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#setAutoDetectUnicode">setAutoDetectUnicode</a>() and <a href="http://doc.qt.io/qt-5/qtextstream.html#setCodec">setCodec</a>().</p></div>
    pub unsafe fn auto_detect_unicode(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_autoDetectUnicode(
            self as *const crate::QTextStream,
        )
    }

    /// <p>Returns the codec that is current assigned to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextCodec* QTextStream::codec() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#codec">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the codec that is current assigned to the stream.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#setCodec">setCodec</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#setAutoDetectUnicode">setAutoDetectUnicode</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#locale">locale</a>().</p></div>
    pub unsafe fn codec(&self) -> ::cpp_utils::MutPtr<crate::QTextCodec> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QTextStream_codec(self as *const crate::QTextStream);
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns the current device associated with the <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>, or 0 if no device has been assigned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QIODevice* QTextStream::device() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#device">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current device associated with the <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>, or 0 if no device has been assigned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#setDevice">setDevice</a>() and <a href="http://doc.qt.io/qt-5/qtextstream.html#string">string</a>().</p></div>
    pub unsafe fn device(&self) -> ::cpp_utils::MutPtr<crate::QIODevice> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QTextStream_device(self as *const crate::QTextStream);
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns the current field alignment.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream::FieldAlignment QTextStream::fieldAlignment() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#fieldAlignment">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current field alignment.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#setFieldAlignment">setFieldAlignment</a>() and <a href="http://doc.qt.io/qt-5/qtextstream.html#fieldWidth">fieldWidth</a>().</p></div>
    pub unsafe fn field_alignment(&self) -> crate::q_text_stream::FieldAlignment {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_fieldAlignment(self as *const crate::QTextStream)
    }

    /// <p>Returns the current field width.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTextStream::fieldWidth() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#fieldWidth">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current field width.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#setFieldWidth">setFieldWidth</a>().</p></div>
    pub unsafe fn field_width(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_fieldWidth(self as *const crate::QTextStream)
    }

    /// <p>Flushes any buffered data waiting to be written to the device.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTextStream::flush()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#flush">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Flushes any buffered data waiting to be written to the device.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> operates on a string, this function does nothing.</p></div>
    pub unsafe fn flush(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_flush(self as *mut crate::QTextStream)
    }

    /// <p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> is set to generate the UTF BOM (Byte Order Mark) when using a UTF codec; otherwise returns <code>false</code>. UTF BOM generation is set to false by default.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTextStream::generateByteOrderMark() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#generateByteOrderMark">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> is set to generate the UTF BOM (Byte Order Mark) when using a UTF codec; otherwise returns <code>false</code>. UTF BOM generation is set to false by default.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#setGenerateByteOrderMark">setGenerateByteOrderMark</a>().</p></div>
    pub unsafe fn generate_byte_order_mark(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_generateByteOrderMark(
            self as *const crate::QTextStream,
        )
    }

    /// <p>Returns the current base of integers. 0 means that the base is detected when reading, or 10 (decimal) when generating numbers.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTextStream::integerBase() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#integerBase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current base of integers. 0 means that the base is detected when reading, or 10 (decimal) when generating numbers.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#setIntegerBase">setIntegerBase</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#number">QString::number</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">numberFlags</a>().</p></div>
    pub unsafe fn integer_base(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_integerBase(self as *const crate::QTextStream)
    }

    /// <p>Returns the locale for this stream. The default locale is C.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale QTextStream::locale() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#locale">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the locale for this stream. The default locale is C.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#setLocale">setLocale</a>().</p></div>
    pub unsafe fn locale(&self) -> ::cpp_utils::CppBox<crate::QLocale> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QTextStream_locale(self as *const crate::QTextStream);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>. Before you can use it for reading or writing, you must assign a device or a string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTextStream::QTextStream()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#QTextStream">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>. Before you can use it for reading or writing, you must assign a device or a string.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#setDevice">setDevice</a>() and <a href="http://doc.qt.io/qt-5/qtextstream.html#setString">setString</a>().</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStream_QTextStream();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> that operates on <i>device</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTextStream::QTextStream(QIODevice* device)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#QTextStream-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> that operates on <i>device</i>.</p></div>
    pub unsafe fn from_q_io_device(
        device: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QIODevice>>,
    ) -> ::cpp_utils::CppBox<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStream_QTextStream1(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QIODevice>>::cast_into(device)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> that operates on <i>string</i>, using <i>openMode</i> to define the open mode.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTextStream::QTextStream(QString* string, QFlags<QIODevice::OpenModeFlag> openMode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#QTextStream-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> that operates on <i>string</i>, using <i>openMode</i> to define the open mode.</p></div>
    pub unsafe fn from_q_string_q_flags_open_mode_flag(
        string: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QString>>,
        open_mode: crate::QFlags<crate::q_io_device::OpenModeFlag>,
    ) -> ::cpp_utils::CppBox<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStream_QTextStream2(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QString>>::cast_into(string)
                .as_mut_raw_ptr(),
            open_mode.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> that operates on <i>array</i>, using <i>openMode</i> to define the open mode. Internally, the array is wrapped by a <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTextStream::QTextStream(QByteArray* array, QFlags<QIODevice::OpenModeFlag> openMode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#QTextStream-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> that operates on <i>array</i>, using <i>openMode</i> to define the open mode. Internally, the array is wrapped by a <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a>.</p></div>
    pub unsafe fn from_q_byte_array_q_flags_open_mode_flag(
        array: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QByteArray>>,
        open_mode: crate::QFlags<crate::q_io_device::OpenModeFlag>,
    ) -> ::cpp_utils::CppBox<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStream_QTextStream3(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QByteArray>>::cast_into(array)
                .as_mut_raw_ptr(),
            open_mode.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> that operates on <i>array</i>, using <i>openMode</i> to define the open mode. The array is accessed as read-only, regardless of the values in <i>openMode</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTextStream::QTextStream(const QByteArray& array, QFlags<QIODevice::OpenModeFlag> openMode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#QTextStream-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> that operates on <i>array</i>, using <i>openMode</i> to define the open mode. The array is accessed as read-only, regardless of the values in <i>openMode</i>.</p>
    /// <p>This constructor is convenient for working on constant strings. Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">int</span> main(<span class="type">int</span> argc<span class="operator">,</span> <span class="type">char</span> <span class="operator">*</span>argv<span class="operator">[</span><span class="operator">]</span>)
    ///   {
    /// &#32;     <span class="comment">// read numeric arguments (123, 0x20, 4.5...)</span>
    /// &#32;     <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">1</span>; i <span class="operator">&lt;</span> argc; <span class="operator">+</span><span class="operator">+</span>i) {
    /// &#32;   &#32;   &#32;   <span class="type">int</span> number;
    /// &#32;   &#32;   &#32;   <span class="type"><a href="http://doc.qt.io/qt-5/qtextstream.html#QTextStream">QTextStream</a></span> in(argv<span class="operator">[</span>i<span class="operator">]</span>);
    /// &#32;   &#32;   &#32;   in <span class="operator">&gt;</span><span class="operator">&gt;</span> number;
    /// &#32;   &#32;   &#32;   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    /// &#32;     }
    ///   }
    ///
    /// </pre></div>
    pub unsafe fn from_q_byte_array_q_flags_open_mode_flag2(
        array: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        open_mode: crate::QFlags<crate::q_io_device::OpenModeFlag>,
    ) -> ::cpp_utils::CppBox<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStream_QTextStream4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(array)
                .as_raw_ptr(),
            open_mode.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> that operates on <i>string</i>, using <i>openMode</i> to define the open mode.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTextStream::QTextStream(QString* string)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#QTextStream-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> that operates on <i>string</i>, using <i>openMode</i> to define the open mode.</p></div>
    pub unsafe fn from_q_string(
        string: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStream_QTextStream5(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QString>>::cast_into(string)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> that operates on <i>array</i>, using <i>openMode</i> to define the open mode. Internally, the array is wrapped by a <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTextStream::QTextStream(QByteArray* array)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#QTextStream-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> that operates on <i>array</i>, using <i>openMode</i> to define the open mode. Internally, the array is wrapped by a <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a>.</p></div>
    pub unsafe fn from_q_byte_array(
        array: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStream_QTextStream6(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QByteArray>>::cast_into(array)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> that operates on <i>array</i>, using <i>openMode</i> to define the open mode. The array is accessed as read-only, regardless of the values in <i>openMode</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTextStream::QTextStream(const QByteArray& array)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#QTextStream-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> that operates on <i>array</i>, using <i>openMode</i> to define the open mode. The array is accessed as read-only, regardless of the values in <i>openMode</i>.</p>
    /// <p>This constructor is convenient for working on constant strings. Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">int</span> main(<span class="type">int</span> argc<span class="operator">,</span> <span class="type">char</span> <span class="operator">*</span>argv<span class="operator">[</span><span class="operator">]</span>)
    ///   {
    /// &#32;     <span class="comment">// read numeric arguments (123, 0x20, 4.5...)</span>
    /// &#32;     <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">1</span>; i <span class="operator">&lt;</span> argc; <span class="operator">+</span><span class="operator">+</span>i) {
    /// &#32;   &#32;   &#32;   <span class="type">int</span> number;
    /// &#32;   &#32;   &#32;   <span class="type"><a href="http://doc.qt.io/qt-5/qtextstream.html#QTextStream">QTextStream</a></span> in(argv<span class="operator">[</span>i<span class="operator">]</span>);
    /// &#32;   &#32;   &#32;   in <span class="operator">&gt;</span><span class="operator">&gt;</span> number;
    /// &#32;   &#32;   &#32;   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    /// &#32;     }
    ///   }
    ///
    /// </pre></div>
    pub unsafe fn from_q_byte_array2(
        array: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStream_QTextStream7(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(array)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the current number flags.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFlags<QTextStream::NumberFlag> QTextStream::numberFlags() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current number flags.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">setNumberFlags</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#integerBase">integerBase</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#realNumberNotation">realNumberNotation</a>().</p></div>
    pub unsafe fn number_flags(&self) -> crate::QFlags<crate::q_text_stream::NumberFlag> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStream_numberFlags(
            self as *const crate::QTextStream,
        );
        crate::QFlags::from(ffi_result)
    }

    /// <p>Returns the current pad character.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QTextStream::padChar() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#padChar">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current pad character.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#setPadChar">setPadChar</a>() and <a href="http://doc.qt.io/qt-5/qtextstream.html#setFieldWidth">setFieldWidth</a>().</p></div>
    pub unsafe fn pad_char(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QTextStream_padChar(self as *const crate::QTextStream);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the device position corresponding to the current position of the stream, or -1 if an error occurs (e.g., if there is no device or string, or if there's a device error).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QTextStream::pos() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#pos">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the device position corresponding to the current position of the stream, or -1 if an error occurs (e.g., if there is no device or string, or if there's a device error).</p>
    /// <p>Because <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> is buffered, this function may have to seek the device to reconstruct a valid device position. This operation can be expensive, so you may want to avoid calling this function in a tight loop.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#seek">seek</a>().</p></div>
    pub unsafe fn pos(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_pos(self as *const crate::QTextStream)
    }

    /// <p>Reads at most <i>maxlen</i> characters from the stream, and returns the data read as a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QTextStream::read(qint64 maxlen)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#read">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads at most <i>maxlen</i> characters from the stream, and returns the data read as a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#readAll">readAll</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#readLine">readLine</a>(), and <a href="http://doc.qt.io/qt-5/qiodevice.html#read">QIODevice::read</a>().</p></div>
    pub unsafe fn read(&mut self, maxlen: i64) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QTextStream_read(self as *mut crate::QTextStream, maxlen);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reads the entire content of the stream, and returns it as a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. Avoid this function when working on large files, as it will consume a significant amount of memory.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QTextStream::readAll()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#readAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads the entire content of the stream, and returns it as a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. Avoid this function when working on large files, as it will consume a significant amount of memory.</p>
    /// <p>Calling <a href="http://doc.qt.io/qt-5/qtextstream.html#readLine">readLine()</a> is better if you do not know how much data is available.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#readLine">readLine</a>().</p></div>
    pub unsafe fn read_all(&mut self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QTextStream_readAll(self as *mut crate::QTextStream);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reads one line of text from the stream, and returns it as a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. The maximum allowed line length is set to <i>maxlen</i>. If the stream contains lines longer than this, then the lines will be split after <i>maxlen</i> characters and returned in parts.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QTextStream::readLine(qint64 maxlen = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#readLine">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads one line of text from the stream, and returns it as a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. The maximum allowed line length is set to <i>maxlen</i>. If the stream contains lines longer than this, then the lines will be split after <i>maxlen</i> characters and returned in parts.</p>
    /// <p>If <i>maxlen</i> is 0, the lines can be of any length.</p>
    /// <p>The returned line has no trailing end-of-line characters ("\n" or "\r\n"), so calling <a href="http://doc.qt.io/qt-5/qstring.html#trimmed">QString::trimmed</a>() can be unnecessary.</p>
    /// <p>If the stream has read to the end of the file, readLine() will return a null <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. For strings, or for devices that support it, you can explicitly test for the end of the stream using <a href="http://doc.qt.io/qt-5/qtextstream.html#atEnd">atEnd</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#readAll">readAll</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#readLine">QIODevice::readLine</a>().</p></div>
    pub unsafe fn read_line_1a(&mut self, maxlen: i64) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStream_readLine(
            self as *mut crate::QTextStream,
            maxlen,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reads one line of text from the stream, and returns it as a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. The maximum allowed line length is set to <i>maxlen</i>. If the stream contains lines longer than this, then the lines will be split after <i>maxlen</i> characters and returned in parts.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QTextStream::readLine()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#readLine">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads one line of text from the stream, and returns it as a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. The maximum allowed line length is set to <i>maxlen</i>. If the stream contains lines longer than this, then the lines will be split after <i>maxlen</i> characters and returned in parts.</p>
    /// <p>If <i>maxlen</i> is 0, the lines can be of any length.</p>
    /// <p>The returned line has no trailing end-of-line characters ("\n" or "\r\n"), so calling <a href="http://doc.qt.io/qt-5/qstring.html#trimmed">QString::trimmed</a>() can be unnecessary.</p>
    /// <p>If the stream has read to the end of the file, readLine() will return a null <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. For strings, or for devices that support it, you can explicitly test for the end of the stream using <a href="http://doc.qt.io/qt-5/qtextstream.html#atEnd">atEnd</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#readAll">readAll</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#readLine">QIODevice::readLine</a>().</p></div>
    pub unsafe fn read_line_0a(&mut self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QTextStream_readLine1(self as *mut crate::QTextStream);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reads one line of text from the stream into <i>line</i>. If <i>line</i> is 0, the read line is not stored.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTextStream::readLineInto(QString* line, qint64 maxlen = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#readLineInto">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads one line of text from the stream into <i>line</i>. If <i>line</i> is 0, the read line is not stored.</p>
    /// <p>The maximum allowed line length is set to <i>maxlen</i>. If the stream contains lines longer than this, then the lines will be split after <i>maxlen</i> characters and returned in parts.</p>
    /// <p>If <i>maxlen</i> is 0, the lines can be of any length.</p>
    /// <p>The resulting line has no trailing end-of-line characters ("\n" or "\r\n"), so calling <a href="http://doc.qt.io/qt-5/qstring.html#trimmed">QString::trimmed</a>() can be unnecessary.</p>
    /// <p>If <i>line</i> has sufficient capacity for the data that is about to be read, this function may not need to allocate new memory. Because of this, it can be faster than <a href="http://doc.qt.io/qt-5/qtextstream.html#readLine">readLine</a>().</p>
    /// <p>Returns <code>false</code> if the stream has read to the end of the file or an error has occurred; otherwise returns <code>true</code>. The contents in <i>line</i> before the call are discarded in any case.</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#readAll">readAll</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#readLine">QIODevice::readLine</a>().</p></div>
    pub unsafe fn read_line_into_2a(
        &mut self,
        line: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QString>>,
        maxlen: i64,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_readLineInto(
            self as *mut crate::QTextStream,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QString>>::cast_into(line)
                .as_mut_raw_ptr(),
            maxlen,
        )
    }

    /// <p>Reads one line of text from the stream into <i>line</i>. If <i>line</i> is 0, the read line is not stored.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTextStream::readLineInto(QString* line)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#readLineInto">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads one line of text from the stream into <i>line</i>. If <i>line</i> is 0, the read line is not stored.</p>
    /// <p>The maximum allowed line length is set to <i>maxlen</i>. If the stream contains lines longer than this, then the lines will be split after <i>maxlen</i> characters and returned in parts.</p>
    /// <p>If <i>maxlen</i> is 0, the lines can be of any length.</p>
    /// <p>The resulting line has no trailing end-of-line characters ("\n" or "\r\n"), so calling <a href="http://doc.qt.io/qt-5/qstring.html#trimmed">QString::trimmed</a>() can be unnecessary.</p>
    /// <p>If <i>line</i> has sufficient capacity for the data that is about to be read, this function may not need to allocate new memory. Because of this, it can be faster than <a href="http://doc.qt.io/qt-5/qtextstream.html#readLine">readLine</a>().</p>
    /// <p>Returns <code>false</code> if the stream has read to the end of the file or an error has occurred; otherwise returns <code>true</code>. The contents in <i>line</i> before the call are discarded in any case.</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#readAll">readAll</a>() and <a href="http://doc.qt.io/qt-5/qiodevice.html#readLine">QIODevice::readLine</a>().</p></div>
    pub unsafe fn read_line_into_1a(
        &mut self,
        line: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_readLineInto1(
            self as *mut crate::QTextStream,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QString>>::cast_into(line)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the current real number notation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream::RealNumberNotation QTextStream::realNumberNotation() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#realNumberNotation">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current real number notation.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#setRealNumberNotation">setRealNumberNotation</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#realNumberPrecision">realNumberPrecision</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">numberFlags</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#integerBase">integerBase</a>().</p></div>
    pub unsafe fn real_number_notation(&self) -> crate::q_text_stream::RealNumberNotation {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_realNumberNotation(
            self as *const crate::QTextStream,
        )
    }

    /// <p>Returns the current real number precision, or the number of fraction digits <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> will write when generating real numbers.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTextStream::realNumberPrecision() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#realNumberPrecision">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current real number precision, or the number of fraction digits <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> will write when generating real numbers.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#setRealNumberPrecision">setRealNumberPrecision</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#setRealNumberNotation">setRealNumberNotation</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#realNumberNotation">realNumberNotation</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">numberFlags</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#integerBase">integerBase</a>().</p></div>
    pub unsafe fn real_number_precision(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_realNumberPrecision(
            self as *const crate::QTextStream,
        )
    }

    /// <p>Resets <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>'s formatting options, bringing it back to its original constructed state. The device, string and any buffered data is left untouched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTextStream::reset()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#reset">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Resets <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>'s formatting options, bringing it back to its original constructed state. The device, string and any buffered data is left untouched.</p></div>
    pub unsafe fn reset(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_reset(self as *mut crate::QTextStream)
    }

    /// <p>Resets the status of the text stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTextStream::resetStatus()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#resetStatus">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Resets the status of the text stream.</p>
    /// <p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#Status-enum">QTextStream::Status</a>, <a href="http://doc.qt.io/qt-5/qtextstream.html#status">status</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#setStatus">setStatus</a>().</p></div>
    pub unsafe fn reset_status(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_resetStatus(self as *mut crate::QTextStream)
    }

    /// <p>Seeks to the position <i>pos</i> in the device. Returns <code>true</code> on success; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTextStream::seek(qint64 pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#seek">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Seeks to the position <i>pos</i> in the device. Returns <code>true</code> on success; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn seek(&mut self, pos: i64) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_seek(self as *mut crate::QTextStream, pos)
    }

    /// <p>If <i>enabled</i> is true, <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> will attempt to detect Unicode encoding by peeking into the stream data to see if it can find the UTF-16 or UTF-32 BOM (Byte Order Mark). If this mark is found, <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> will replace the current codec with the UTF codec.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTextStream::setAutoDetectUnicode(bool enabled)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#setAutoDetectUnicode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If <i>enabled</i> is true, <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> will attempt to detect Unicode encoding by peeking into the stream data to see if it can find the UTF-16 or UTF-32 BOM (Byte Order Mark). If this mark is found, <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> will replace the current codec with the UTF codec.</p>
    /// <p>This function can be used together with <a href="http://doc.qt.io/qt-5/qtextstream.html#setCodec">setCodec</a>(). It is common to set the codec to UTF-8, and then enable UTF-16 detection.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#autoDetectUnicode">autoDetectUnicode</a>() and <a href="http://doc.qt.io/qt-5/qtextstream.html#setCodec">setCodec</a>().</p></div>
    pub unsafe fn set_auto_detect_unicode(&mut self, enabled: bool) {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_setAutoDetectUnicode(
            self as *mut crate::QTextStream,
            enabled,
        )
    }

    /// <p>Sets the codec for this stream to <i>codec</i>. The codec is used for decoding any data that is read from the assigned device, and for encoding any data that is written. By default, <a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForLocale">QTextCodec::codecForLocale</a>() is used, and automatic unicode detection is enabled.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTextStream::setCodec(QTextCodec* codec)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#setCodec">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the codec for this stream to <i>codec</i>. The codec is used for decoding any data that is read from the assigned device, and for encoding any data that is written. By default, <a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForLocale">QTextCodec::codecForLocale</a>() is used, and automatic unicode detection is enabled.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> operates on a string, this function does nothing.</p>
    /// <p><b>Warning:</b> If you call this function while the text stream is reading from an open sequential socket, the internal buffer may still contain text decoded using the old codec.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#codec">codec</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#setAutoDetectUnicode">setAutoDetectUnicode</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#setLocale">setLocale</a>().</p></div>
    pub unsafe fn set_codec_q_text_codec(
        &mut self,
        codec: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QTextCodec>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_setCodec(
            self as *mut crate::QTextStream,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QTextCodec>>::cast_into(codec)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Sets the codec for this stream to the <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> for the encoding specified by <i>codecName</i>. Common values for <code>codecName</code> include "ISO 8859-1", "UTF-8", and "UTF-16". If the encoding isn't recognized, nothing happens.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTextStream::setCodec(const char* codecName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#setCodec-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the codec for this stream to the <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> for the encoding specified by <i>codecName</i>. Common values for <code>codecName</code> include "ISO 8859-1", "UTF-8", and "UTF-16". If the encoding isn't recognized, nothing happens.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtextstream.html#QTextStream">QTextStream</a></span> out(<span class="operator">&amp;</span>file);
    ///   out<span class="operator">.</span>setCodec(<span class="string">"UTF-8"</span>);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForName">QTextCodec::codecForName</a>() and <a href="http://doc.qt.io/qt-5/qtextstream.html#setLocale">setLocale</a>().</p></div>
    pub unsafe fn set_codec_char(
        &mut self,
        codec_name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_setCodec1(
            self as *mut crate::QTextStream,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(
                codec_name,
            )
            .as_raw_ptr(),
        )
    }

    /// <p>Sets the current device to <i>device</i>. If a device has already been assigned, <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> will call <a href="http://doc.qt.io/qt-5/qtextstream.html#flush">flush</a>() before the old device is replaced.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTextStream::setDevice(QIODevice* device)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#setDevice">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the current device to <i>device</i>. If a device has already been assigned, <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> will call <a href="http://doc.qt.io/qt-5/qtextstream.html#flush">flush</a>() before the old device is replaced.</p>
    /// <p><b>Note: </b>This function resets locale to the default locale ('C') and codec to the default codec, <a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForLocale">QTextCodec::codecForLocale</a>().</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#device">device</a>() and <a href="http://doc.qt.io/qt-5/qtextstream.html#setString">setString</a>().</p></div>
    pub unsafe fn set_device(
        &mut self,
        device: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QIODevice>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_setDevice(
            self as *mut crate::QTextStream,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QIODevice>>::cast_into(device)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Sets the field alignment to <i>mode</i>. When used together with <a href="http://doc.qt.io/qt-5/qtextstream.html#setFieldWidth">setFieldWidth</a>(), this function allows you to generate formatted output with text aligned to the left, to the right or center aligned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTextStream::setFieldAlignment(QTextStream::FieldAlignment alignment)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#setFieldAlignment">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the field alignment to <i>mode</i>. When used together with <a href="http://doc.qt.io/qt-5/qtextstream.html#setFieldWidth">setFieldWidth</a>(), this function allows you to generate formatted output with text aligned to the left, to the right or center aligned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#fieldAlignment">fieldAlignment</a>() and <a href="http://doc.qt.io/qt-5/qtextstream.html#setFieldWidth">setFieldWidth</a>().</p></div>
    pub unsafe fn set_field_alignment(&mut self, alignment: crate::q_text_stream::FieldAlignment) {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_setFieldAlignment(
            self as *mut crate::QTextStream,
            alignment,
        )
    }

    /// <p>Sets the current field width to <i>width</i>. If <i>width</i> is 0 (the default), the field width is equal to the length of the generated text.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTextStream::setFieldWidth(int width)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#setFieldWidth">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the current field width to <i>width</i>. If <i>width</i> is 0 (the default), the field width is equal to the length of the generated text.</p>
    /// <p><b>Note: </b>The field width applies to every element appended to this stream after this function has been called (e.g., it also pads endl). This behavior is different from similar classes in the STL, where the field width only applies to the next element.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#fieldWidth">fieldWidth</a>() and <a href="http://doc.qt.io/qt-5/qtextstream.html#setPadChar">setPadChar</a>().</p></div>
    pub unsafe fn set_field_width(&mut self, width: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_setFieldWidth(
            self as *mut crate::QTextStream,
            width,
        )
    }

    /// <p>If <i>generate</i> is true and a UTF codec is used, <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> will insert the BOM (Byte Order Mark) before any data has been written to the device. If <i>generate</i> is false, no BOM will be inserted. This function must be called before any data is written. Otherwise, it does nothing.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTextStream::setGenerateByteOrderMark(bool generate)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#setGenerateByteOrderMark">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If <i>generate</i> is true and a UTF codec is used, <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> will insert the BOM (Byte Order Mark) before any data has been written to the device. If <i>generate</i> is false, no BOM will be inserted. This function must be called before any data is written. Otherwise, it does nothing.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#generateByteOrderMark">generateByteOrderMark</a>() and <a href="http://doc.qt.io/qt-5/qtextstream.html#bom">bom</a>().</p></div>
    pub unsafe fn set_generate_byte_order_mark(&mut self, generate: bool) {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_setGenerateByteOrderMark(
            self as *mut crate::QTextStream,
            generate,
        )
    }

    /// <p>Sets the base of integers to <i>base</i>, both for reading and for generating numbers. <i>base</i> can be either 2 (binary), 8 (octal), 10 (decimal) or 16 (hexadecimal). If <i>base</i> is 0, <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> will attempt to detect the base by inspecting the data on the stream. When generating numbers, <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> assumes base is 10 unless the base has been set explicitly.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTextStream::setIntegerBase(int base)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#setIntegerBase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the base of integers to <i>base</i>, both for reading and for generating numbers. <i>base</i> can be either 2 (binary), 8 (octal), 10 (decimal) or 16 (hexadecimal). If <i>base</i> is 0, <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> will attempt to detect the base by inspecting the data on the stream. When generating numbers, <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> assumes base is 10 unless the base has been set explicitly.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#integerBase">integerBase</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#number">QString::number</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">setNumberFlags</a>().</p></div>
    pub unsafe fn set_integer_base(&mut self, base: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_setIntegerBase(
            self as *mut crate::QTextStream,
            base,
        )
    }

    /// <p>Sets the locale for this stream to <i>locale</i>. The specified locale is used for conversions between numbers and their string representations.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTextStream::setLocale(const QLocale& locale)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#setLocale">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the locale for this stream to <i>locale</i>. The specified locale is used for conversions between numbers and their string representations.</p>
    /// <p>The default locale is C and it is a special case - the thousands group separator is not used for backward compatibility reasons.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#locale">locale</a>().</p></div>
    pub unsafe fn set_locale(
        &mut self,
        locale: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_setLocale(
            self as *mut crate::QTextStream,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(locale)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the current number flags to <i>flags</i>. <i>flags</i> is a set of flags from the <a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">NumberFlag</a> enum, and describes options for formatting generated code (e.g., whether or not to always write the base or sign of a number).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTextStream::setNumberFlags(QFlags<QTextStream::NumberFlag> flags)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the current number flags to <i>flags</i>. <i>flags</i> is a set of flags from the <a href="http://doc.qt.io/qt-5/qtextstream.html#NumberFlag-enum">NumberFlag</a> enum, and describes options for formatting generated code (e.g., whether or not to always write the base or sign of a number).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#numberFlags">numberFlags</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#setIntegerBase">setIntegerBase</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#setRealNumberNotation">setRealNumberNotation</a>().</p></div>
    pub unsafe fn set_number_flags(
        &mut self,
        flags: crate::QFlags<crate::q_text_stream::NumberFlag>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_setNumberFlags(
            self as *mut crate::QTextStream,
            flags.to_int(),
        )
    }

    /// <p>Sets the pad character to <i>ch</i>. The default value is the ASCII space character (' '), or <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>(0x20). This character is used to fill in the space in fields when generating text.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTextStream::setPadChar(QChar ch)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#setPadChar">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the pad character to <i>ch</i>. The default value is the ASCII space character (' '), or <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>(0x20). This character is used to fill in the space in fields when generating text.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> s;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtextstream.html#QTextStream">QTextStream</a></span> out(<span class="operator">&amp;</span>s);
    ///   out<span class="operator">.</span>setFieldWidth(<span class="number">10</span>);
    ///   out<span class="operator">.</span>setFieldAlignment(<span class="type"><a href="http://doc.qt.io/qt-5/qtextstream.html#QTextStream">QTextStream</a></span><span class="operator">::</span>AlignCenter);
    ///   out<span class="operator">.</span>setPadChar(<span class="char">'-'</span>);
    ///   out <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Qt"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rocks!"</span>;
    ///
    /// </pre>
    /// <p>The string <code>s</code> contains:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="operator">-</span><span class="operator">-</span><span class="operator">-</span><span class="operator">-</span><span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">-</span><span class="operator">-</span><span class="operator">-</span><span class="operator">-</span><span class="operator">-</span><span class="operator">-</span>rocks<span class="operator">!</span><span class="operator">-</span><span class="operator">-</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#padChar">padChar</a>() and <a href="http://doc.qt.io/qt-5/qtextstream.html#setFieldWidth">setFieldWidth</a>().</p></div>
    pub unsafe fn set_pad_char(
        &mut self,
        ch: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_setPadChar(
            self as *mut crate::QTextStream,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(ch).as_raw_ptr(),
        )
    }

    /// <p>Sets the real number notation to <i>notation</i> (<a href="http://doc.qt.io/qt-5/qtextstream.html#RealNumberNotation-enum">SmartNotation</a>, <a href="http://doc.qt.io/qt-5/qtextstream.html#RealNumberNotation-enum">FixedNotation</a>, <a href="http://doc.qt.io/qt-5/qtextstream.html#RealNumberNotation-enum">ScientificNotation</a>). When reading and generating numbers, <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> uses this value to detect the formatting of real numbers.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTextStream::setRealNumberNotation(QTextStream::RealNumberNotation notation)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#setRealNumberNotation">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the real number notation to <i>notation</i> (<a href="http://doc.qt.io/qt-5/qtextstream.html#RealNumberNotation-enum">SmartNotation</a>, <a href="http://doc.qt.io/qt-5/qtextstream.html#RealNumberNotation-enum">FixedNotation</a>, <a href="http://doc.qt.io/qt-5/qtextstream.html#RealNumberNotation-enum">ScientificNotation</a>). When reading and generating numbers, <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> uses this value to detect the formatting of real numbers.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#realNumberNotation">realNumberNotation</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#setRealNumberPrecision">setRealNumberPrecision</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">setNumberFlags</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#setIntegerBase">setIntegerBase</a>().</p></div>
    pub unsafe fn set_real_number_notation(
        &mut self,
        notation: crate::q_text_stream::RealNumberNotation,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_setRealNumberNotation(
            self as *mut crate::QTextStream,
            notation,
        )
    }

    /// <p>Sets the precision of real numbers to <i>precision</i>. This value describes the number of fraction digits <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> should write when generating real numbers.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTextStream::setRealNumberPrecision(int precision)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#setRealNumberPrecision">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the precision of real numbers to <i>precision</i>. This value describes the number of fraction digits <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> should write when generating real numbers.</p>
    /// <p>The precision cannot be a negative value. The default value is 6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#realNumberPrecision">realNumberPrecision</a>() and <a href="http://doc.qt.io/qt-5/qtextstream.html#setRealNumberNotation">setRealNumberNotation</a>().</p></div>
    pub unsafe fn set_real_number_precision(&mut self, precision: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_setRealNumberPrecision(
            self as *mut crate::QTextStream,
            precision,
        )
    }

    /// <p>Sets the status of the text stream to the <i>status</i> given.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTextStream::setStatus(QTextStream::Status status)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#setStatus">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the status of the text stream to the <i>status</i> given.</p>
    /// <p>Subsequent calls to setStatus() are ignored until <a href="http://doc.qt.io/qt-5/qtextstream.html#resetStatus">resetStatus</a>() is called.</p>
    /// <p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#Status-enum">Status</a>, <a href="http://doc.qt.io/qt-5/qtextstream.html#status">status</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#resetStatus">resetStatus</a>().</p></div>
    pub unsafe fn set_status(&mut self, status: crate::q_text_stream::Status) {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_setStatus(self as *mut crate::QTextStream, status)
    }

    /// <p>Sets the current string to <i>string</i>, using the given <i>openMode</i>. If a device has already been assigned, <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> will call <a href="http://doc.qt.io/qt-5/qtextstream.html#flush">flush</a>() before replacing it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTextStream::setString(QString* string, QFlags<QIODevice::OpenModeFlag> openMode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#setString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the current string to <i>string</i>, using the given <i>openMode</i>. If a device has already been assigned, <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> will call <a href="http://doc.qt.io/qt-5/qtextstream.html#flush">flush</a>() before replacing it.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#string">string</a>() and <a href="http://doc.qt.io/qt-5/qtextstream.html#setDevice">setDevice</a>().</p></div>
    pub unsafe fn set_string_2a(
        &mut self,
        string: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QString>>,
        open_mode: crate::QFlags<crate::q_io_device::OpenModeFlag>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_setString(
            self as *mut crate::QTextStream,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QString>>::cast_into(string)
                .as_mut_raw_ptr(),
            open_mode.to_int(),
        )
    }

    /// <p>Sets the current string to <i>string</i>, using the given <i>openMode</i>. If a device has already been assigned, <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> will call <a href="http://doc.qt.io/qt-5/qtextstream.html#flush">flush</a>() before replacing it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTextStream::setString(QString* string)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#setString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the current string to <i>string</i>, using the given <i>openMode</i>. If a device has already been assigned, <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> will call <a href="http://doc.qt.io/qt-5/qtextstream.html#flush">flush</a>() before replacing it.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#string">string</a>() and <a href="http://doc.qt.io/qt-5/qtextstream.html#setDevice">setDevice</a>().</p></div>
    pub unsafe fn set_string_1a(
        &mut self,
        string: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_setString1(
            self as *mut crate::QTextStream,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QString>>::cast_into(string)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Writes the integer number <i>i</i> to the stream, then returns a reference to the <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>. By default, the number is stored in decimal form, but you can also set the base by calling <a href="http://doc.qt.io/qt-5/qtextstream.html#setIntegerBase">setIntegerBase</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator<<(short i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-lt-lt-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes the integer number <i>i</i> to the stream, then returns a reference to the <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>. By default, the number is stored in decimal form, but you can also set the base by calling <a href="http://doc.qt.io/qt-5/qtextstream.html#setIntegerBase">setIntegerBase</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#setFieldWidth">setFieldWidth</a>() and <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">setNumberFlags</a>().</p></div>
    pub unsafe fn shl_short(
        &mut self,
        i: ::std::os::raw::c_short,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__17(
            self as *mut crate::QTextStream,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator<<(unsigned short i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-lt-lt-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes the unsigned short <i>i</i> to the stream.</p></div>
    pub unsafe fn shl_ushort(
        &mut self,
        i: ::std::os::raw::c_ushort,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__18(
            self as *mut crate::QTextStream,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator<<(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-lt-lt-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes the signed int <i>i</i> to the stream.</p></div>
    pub unsafe fn shl_int(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__19(
            self as *mut crate::QTextStream,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator<<(unsigned int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-lt-lt-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes the unsigned int <i>i</i> to the stream.</p></div>
    pub unsafe fn shl_uint(
        &mut self,
        i: ::std::os::raw::c_uint,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__20(
            self as *mut crate::QTextStream,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator<<(long i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-lt-lt-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes the signed long <i>i</i> to the stream.</p></div>
    pub unsafe fn shl_long(
        &mut self,
        i: ::std::os::raw::c_long,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__21(
            self as *mut crate::QTextStream,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator<<(unsigned long i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-lt-lt-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes the unsigned long <i>i</i> to the stream.</p></div>
    pub unsafe fn shl_ulong(
        &mut self,
        i: ::std::os::raw::c_ulong,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__22(
            self as *mut crate::QTextStream,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator<<(qulonglong i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-lt-lt-9">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes the qulonglong <i>i</i> to the stream.</p></div>
    pub unsafe fn shl_u64(&mut self, i: u64) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__24(
            self as *mut crate::QTextStream,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator<<(double f)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-lt-lt-11">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes the double <i>f</i> to the stream.</p></div>
    pub unsafe fn shl_double(
        &mut self,
        f: ::std::os::raw::c_double,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__26(
            self as *mut crate::QTextStream,
            f,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Reads an integer from the stream and stores it in <i>i</i>, then returns a reference to the <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>. The number is cast to the correct type before it is stored. If no number was detected on the stream, <i>i</i> is set to 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator>>(short& i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-gt-gt-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads an integer from the stream and stores it in <i>i</i>, then returns a reference to the <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>. The number is cast to the correct type before it is stored. If no number was detected on the stream, <i>i</i> is set to 0.</p>
    /// <p>By default, <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> will attempt to detect the base of the number using the following rules:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Prefix</th><th>Base</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>"0b" or "0B"</td><td>2 (binary)</td></tr>
    /// <tr valign="top" class="even"><td>"0" followed by "0-7"</td><td>8 (octal)</td></tr>
    /// <tr valign="top" class="odd"><td>"0" otherwise</td><td>10 (decimal)</td></tr>
    /// <tr valign="top" class="even"><td>"0x" or "0X"</td><td>16 (hexadecimal)</td></tr>
    /// <tr valign="top" class="odd"><td>"1" to "9"</td><td>10 (decimal)</td></tr>
    /// </tbody></table></div>
    /// <p>By calling <a href="http://doc.qt.io/qt-5/qtextstream.html#setIntegerBase">setIntegerBase</a>(), you can specify the integer base explicitly. This will disable the auto-detection, and speed up <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> slightly.</p>
    /// <p>Leading whitespace is skipped.</p></div>
    pub unsafe fn shr_short(
        &mut self,
        i: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<::std::os::raw::c_short>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__2(
            self as *mut crate::QTextStream,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<::std::os::raw::c_short>>::cast_into(i)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator>>(unsigned short& i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-gt-gt-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Stores the integer in the unsigned short <i>i</i>.</p></div>
    pub unsafe fn shr_ushort(
        &mut self,
        i: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<::std::os::raw::c_ushort>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__3(
            self as *mut crate::QTextStream,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<::std::os::raw::c_ushort>>::cast_into(i)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator>>(int& i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-gt-gt-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Stores the integer in the signed int <i>i</i>.</p></div>
    pub unsafe fn shr_int(
        &mut self,
        i: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__4(
            self as *mut crate::QTextStream,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<::std::os::raw::c_int>>::cast_into(i)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator>>(unsigned int& i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-gt-gt-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Stores the integer in the unsigned int <i>i</i>.</p></div>
    pub unsafe fn shr_uint(
        &mut self,
        i: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<::std::os::raw::c_uint>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__5(
            self as *mut crate::QTextStream,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<::std::os::raw::c_uint>>::cast_into(i)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator>>(long& i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-gt-gt-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Stores the integer in the signed long <i>i</i>.</p></div>
    pub unsafe fn shr_long(
        &mut self,
        i: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<::std::os::raw::c_long>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__6(
            self as *mut crate::QTextStream,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<::std::os::raw::c_long>>::cast_into(i)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator>>(unsigned long& i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-gt-gt-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Stores the integer in the unsigned long <i>i</i>.</p></div>
    pub unsafe fn shr_ulong(
        &mut self,
        i: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<::std::os::raw::c_ulong>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__7(
            self as *mut crate::QTextStream,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<::std::os::raw::c_ulong>>::cast_into(i)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator>>(qulonglong& i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-gt-gt-9">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Stores the integer in the qulonglong <i>i</i>.</p></div>
    pub unsafe fn shr_u64(
        &mut self,
        i: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<u64>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__9(
            self as *mut crate::QTextStream,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<u64>>::cast_into(i).as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator>>(double& f)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-gt-gt-11">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Stores the real number in the double <i>f</i>.</p></div>
    pub unsafe fn shr_double(
        &mut self,
        f: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<::std::os::raw::c_double>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__11(
            self as *mut crate::QTextStream,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<::std::os::raw::c_double>>::cast_into(f)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Reads and discards whitespace from the stream until either a non-space character is detected, or until <a href="http://doc.qt.io/qt-5/qtextstream.html#atEnd">atEnd</a>() returns true. This function is useful when reading a stream character by character.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTextStream::skipWhiteSpace()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#skipWhiteSpace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads and discards whitespace from the stream until either a non-space character is detected, or until <a href="http://doc.qt.io/qt-5/qtextstream.html#atEnd">atEnd</a>() returns true. This function is useful when reading a stream character by character.</p>
    /// <p>Whitespace characters are all characters for which <a href="http://doc.qt.io/qt-5/qchar.html#isSpace">QChar::isSpace</a>() returns <code>true</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#operator-gt-gt">operator&gt;&gt;</a>().</p></div>
    pub unsafe fn skip_white_space(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_skipWhiteSpace(self as *mut crate::QTextStream)
    }

    /// <p>Returns the status of the text stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream::Status QTextStream::status() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#status">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the status of the text stream.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#Status-enum">QTextStream::Status</a>, <a href="http://doc.qt.io/qt-5/qtextstream.html#setStatus">setStatus</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#resetStatus">resetStatus</a>().</p></div>
    pub unsafe fn status(&self) -> crate::q_text_stream::Status {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_status(self as *const crate::QTextStream)
    }

    /// <p>Returns the current string assigned to the <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>, or 0 if no string has been assigned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString* QTextStream::string() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#string">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current string assigned to the <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>, or 0 if no string has been assigned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#setString">setString</a>() and <a href="http://doc.qt.io/qt-5/qtextstream.html#device">device</a>().</p></div>
    pub unsafe fn string(&self) -> ::cpp_utils::MutPtr<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QTextStream_string(self as *const crate::QTextStream);
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

/// C++ class: <span style='color: green;'>```QTextStreamManipulator```</span>.
#[repr(C)]
pub struct QTextStreamManipulator {
    _unused: u8,
}
impl QTextStreamManipulator {
    /// Calls C++ function: <span style='color: green;'>```QTextStreamManipulator& QTextStreamManipulator::operator=(const QTextStreamManipulator& other)```</span>.
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTextStreamManipulator>>,
    ) -> ::cpp_utils::MutRef<crate::QTextStreamManipulator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStreamManipulator_operator_(
            self as *mut crate::QTextStreamManipulator,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTextStreamManipulator>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```void QTextStreamManipulator::exec(QTextStream& s)```</span>.
    pub unsafe fn exec(
        &mut self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QTextStream>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QTextStreamManipulator_exec(
            self as *mut crate::QTextStreamManipulator,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QTextStream>>::cast_into(s)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTextStreamManipulator::QTextStreamManipulator(const QTextStreamManipulator& other)```</span>.
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTextStreamManipulator>>,
    ) -> ::cpp_utils::CppBox<crate::QTextStreamManipulator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextStreamManipulator_QTextStreamManipulator1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTextStreamManipulator>>::cast_into(other).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_debug {
    //! C++ type: <span style='color: green;'>```QDebug```</span>

    /// <p>This enum describes the range of verbosity levels.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QDebug::VerbosityLevel```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#VerbosityLevel-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the range of verbosity levels.</p>
    ///
    /// <p>This enum was introduced or modified in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdebug.html#verbosity-1">verbosity</a>() and <a href="http://doc.qt.io/qt-5/qdebug.html#setVerbosity">setVerbosity</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct VerbosityLevel(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for VerbosityLevel {
        fn from(value: ::std::os::raw::c_int) -> Self {
            VerbosityLevel(value)
        }
    }

    impl From<VerbosityLevel> for ::std::os::raw::c_int {
        fn from(value: VerbosityLevel) -> Self {
            value.0
        }
    }

    impl VerbosityLevel {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl VerbosityLevel {
        /// C++ enum variant: <span style='color: green;'>```MinimumVerbosity = 0```</span>
        #[allow(non_upper_case_globals)]
        pub const MinimumVerbosity: crate::q_debug::VerbosityLevel =
            crate::q_debug::VerbosityLevel(0);
        /// C++ enum variant: <span style='color: green;'>```DefaultVerbosity = 2```</span>
        #[allow(non_upper_case_globals)]
        pub const DefaultVerbosity: crate::q_debug::VerbosityLevel =
            crate::q_debug::VerbosityLevel(2);
        /// C++ enum variant: <span style='color: green;'>```MaximumVerbosity = 7```</span>
        #[allow(non_upper_case_globals)]
        pub const MaximumVerbosity: crate::q_debug::VerbosityLevel =
            crate::q_debug::VerbosityLevel(7);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> class provides an output stream for debugging information.</p>
///
/// C++ class: <span style='color: green;'>```QDebug```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qdebug.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> class provides an output stream for debugging information.</p>
/// <p><a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> is used whenever the developer needs to write out debugging or tracing information to a device, file, string or console.</p>
/// <a name="basic-use"></a></div>
#[repr(C)]
pub struct QDebug {
    _unused: u8,
}
impl QDebug {
    /// <p>Returns <code>true</code> if this <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> instance will automatically insert spaces between writes.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDebug::autoInsertSpaces() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#autoInsertSpaces">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> instance will automatically insert spaces between writes.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdebug.html#setAutoInsertSpaces">setAutoInsertSpaces</a>() and <a href="http://doc.qt.io/qt-5/qdebugstatesaver.html">QDebugStateSaver</a>.</p></div>
    pub unsafe fn auto_insert_spaces(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDebug_autoInsertSpaces(self as *const crate::QDebug)
    }

    /// <p>Assigns the <i>other</i> debug stream to this stream and returns a reference to this stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::operator=(const QDebug& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns the <i>other</i> debug stream to this stream and returns a reference to this stream.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDebug>>,
    ) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDebug_operator_(
            self as *mut crate::QDebug,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDebug>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Writes a character <i>c</i> to the debug stream, depending on the current setting for automatic insertion of quotes, and returns a reference to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::maybeQuote(char c = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#maybeQuote">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes a character <i>c</i> to the debug stream, depending on the current setting for automatic insertion of quotes, and returns a reference to the stream.</p>
    /// <p>The default character is a double quote <code>"</code>.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdebug.html#quote">quote</a>() and <a href="http://doc.qt.io/qt-5/qdebug.html#noquote">noquote</a>().</p></div>
    pub unsafe fn maybe_quote_1a(
        &mut self,
        c: ::std::os::raw::c_char,
    ) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDebug_maybeQuote(self as *mut crate::QDebug, c);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Writes a character <i>c</i> to the debug stream, depending on the current setting for automatic insertion of quotes, and returns a reference to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::maybeQuote()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#maybeQuote">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes a character <i>c</i> to the debug stream, depending on the current setting for automatic insertion of quotes, and returns a reference to the stream.</p>
    /// <p>The default character is a double quote <code>"</code>.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdebug.html#quote">quote</a>() and <a href="http://doc.qt.io/qt-5/qdebug.html#noquote">noquote</a>().</p></div>
    pub unsafe fn maybe_quote_0a(&mut self) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDebug_maybeQuote1(self as *mut crate::QDebug);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Writes a space character to the debug stream, depending on the current setting for automatic insertion of spaces, and returns a reference to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::maybeSpace()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#maybeSpace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes a space character to the debug stream, depending on the current setting for automatic insertion of spaces, and returns a reference to the stream.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdebug.html#space">space</a>() and <a href="http://doc.qt.io/qt-5/qdebug.html#nospace">nospace</a>().</p></div>
    pub unsafe fn maybe_space(&mut self) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDebug_maybeSpace(self as *mut crate::QDebug);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Constructs a debug stream that writes to the given <i>device</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDebug::QDebug(QIODevice* device)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#QDebug">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a debug stream that writes to the given <i>device</i>.</p></div>
    pub unsafe fn from_q_io_device(
        device: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QIODevice>>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDebug_QDebug(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QIODevice>>::cast_into(device)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a debug stream that writes to the given <i>string</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDebug::QDebug(QString* string)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#QDebug-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a debug stream that writes to the given <i>string</i>.</p></div>
    pub unsafe fn from_q_string(
        string: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDebug_QDebug1(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QString>>::cast_into(string)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a debug stream that writes to the handler for the message type specified by <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDebug::QDebug(QtMsgType t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#QDebug-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a debug stream that writes to the handler for the message type specified by <i>type</i>.</p></div>
    pub unsafe fn from_qt_msg_type(t: crate::QtMsgType) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDebug_QDebug2(t);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of the <i>other</i> debug stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDebug::QDebug(const QDebug& o)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#QDebug-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of the <i>other</i> debug stream.</p></div>
    pub unsafe fn new_copy(
        o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDebug>>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDebug_QDebug3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDebug>>::cast_into(o).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Disables automatic insertion of quotation characters around <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>, <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> contents and returns a reference to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::noquote()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#noquote">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Disables automatic insertion of quotation characters around <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>, <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> contents and returns a reference to the stream.</p>
    /// <p>When quoting is disabled, these types are printed without quotation characters and without escaping of non-printable characters.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdebug.html#quote">quote</a>() and <a href="http://doc.qt.io/qt-5/qdebug.html#maybeQuote">maybeQuote</a>().</p></div>
    pub unsafe fn noquote(&mut self) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDebug_noquote(self as *mut crate::QDebug);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Disables automatic insertion of spaces and returns a reference to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::nospace()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#nospace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Disables automatic insertion of spaces and returns a reference to the stream.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdebug.html#space">space</a>() and <a href="http://doc.qt.io/qt-5/qdebug.html#maybeSpace">maybeSpace</a>().</p></div>
    pub unsafe fn nospace(&mut self) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDebug_nospace(self as *mut crate::QDebug);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Enables automatic insertion of quotation characters around <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>, <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> contents and returns a reference to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::quote()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#quote">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Enables automatic insertion of quotation characters around <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>, <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> contents and returns a reference to the stream.</p>
    /// <p>Quoting is enabled by default.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdebug.html#noquote">noquote</a>() and <a href="http://doc.qt.io/qt-5/qdebug.html#maybeQuote">maybeQuote</a>().</p></div>
    pub unsafe fn quote(&mut self) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDebug_quote(self as *mut crate::QDebug);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Resets the stream formatting options, bringing it back to its original constructed state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::resetFormat()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#resetFormat">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Resets the stream formatting options, bringing it back to its original constructed state.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdebug.html#space">space</a>() and <a href="http://doc.qt.io/qt-5/qdebug.html#quote">quote</a>().</p></div>
    pub unsafe fn reset_format(&mut self) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDebug_resetFormat(self as *mut crate::QDebug);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Enables automatic insertion of spaces between writes if <i>b</i> is true; otherwise automatic insertion of spaces is disabled.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDebug::setAutoInsertSpaces(bool b)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#setAutoInsertSpaces">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Enables automatic insertion of spaces between writes if <i>b</i> is true; otherwise automatic insertion of spaces is disabled.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdebug.html#autoInsertSpaces">autoInsertSpaces</a>() and <a href="http://doc.qt.io/qt-5/qdebugstatesaver.html">QDebugStateSaver</a>.</p></div>
    pub unsafe fn set_auto_insert_spaces(&mut self, b: bool) {
        crate::__ffi::ctr_qt_core_ffi_QDebug_setAutoInsertSpaces(self as *mut crate::QDebug, b)
    }

    /// <p>Sets the verbosity of the stream to <i>verbosityLevel</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDebug::setVerbosity(int verbosityLevel)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#setVerbosity">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the verbosity of the stream to <i>verbosityLevel</i>.</p>
    /// <p>The allowed range is from 0 to 7. The default value is 2.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdebug.html#verbosity">verbosity</a>().</p></div>
    pub unsafe fn set_verbosity(&mut self, verbosity_level: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QDebug_setVerbosity(
            self as *mut crate::QDebug,
            verbosity_level,
        )
    }

    /// <p>Writes the signed short integer, <i>i</i>, to the stream and returns a reference to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::operator<<(short t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#operator-lt-lt-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes the signed short integer, <i>i</i>, to the stream and returns a reference to the stream.</p></div>
    pub unsafe fn shl_short(
        &mut self,
        t: ::std::os::raw::c_short,
    ) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDebug_operator__3(self as *mut crate::QDebug, t);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Writes then unsigned short integer, <i>i</i>, to the stream and returns a reference to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::operator<<(unsigned short t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#operator-lt-lt-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes then unsigned short integer, <i>i</i>, to the stream and returns a reference to the stream.</p></div>
    pub unsafe fn shl_ushort(
        &mut self,
        t: ::std::os::raw::c_ushort,
    ) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDebug_operator__4(self as *mut crate::QDebug, t);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Writes the signed integer, <i>i</i>, to the stream and returns a reference to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::operator<<(int t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#operator-lt-lt-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes the signed integer, <i>i</i>, to the stream and returns a reference to the stream.</p></div>
    pub unsafe fn shl_int(
        &mut self,
        t: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDebug_operator__7(self as *mut crate::QDebug, t);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Writes then unsigned integer, <i>i</i>, to the stream and returns a reference to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::operator<<(unsigned int t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#operator-lt-lt-8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes then unsigned integer, <i>i</i>, to the stream and returns a reference to the stream.</p></div>
    pub unsafe fn shl_uint(
        &mut self,
        t: ::std::os::raw::c_uint,
    ) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDebug_operator__8(self as *mut crate::QDebug, t);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Writes the signed long integer, <i>l</i>, to the stream and returns a reference to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::operator<<(long t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#operator-lt-lt-9">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes the signed long integer, <i>l</i>, to the stream and returns a reference to the stream.</p></div>
    pub unsafe fn shl_long(
        &mut self,
        t: ::std::os::raw::c_long,
    ) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDebug_operator__9(self as *mut crate::QDebug, t);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Writes then unsigned long integer, <i>l</i>, to the stream and returns a reference to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::operator<<(unsigned long t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#operator-lt-lt-10">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes then unsigned long integer, <i>l</i>, to the stream and returns a reference to the stream.</p></div>
    pub unsafe fn shl_ulong(
        &mut self,
        t: ::std::os::raw::c_ulong,
    ) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDebug_operator__10(self as *mut crate::QDebug, t);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Writes then unsigned 64-bit integer, <i>i</i>, to the stream and returns a reference to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::operator<<(quint64 t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#operator-lt-lt-12">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes then unsigned 64-bit integer, <i>i</i>, to the stream and returns a reference to the stream.</p></div>
    pub unsafe fn shl_u64(&mut self, t: u64) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDebug_operator__12(self as *mut crate::QDebug, t);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Writes the 64-bit floating point number, <i>f</i>, to the stream and returns a reference to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::operator<<(double t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#operator-lt-lt-14">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes the 64-bit floating point number, <i>f</i>, to the stream and returns a reference to the stream.</p></div>
    pub unsafe fn shl_double(
        &mut self,
        t: ::std::os::raw::c_double,
    ) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDebug_operator__14(self as *mut crate::QDebug, t);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Writes a space character to the debug stream and returns a reference to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::space()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#space">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes a space character to the debug stream and returns a reference to the stream.</p>
    /// <p>The stream remembers that automatic insertion of spaces is enabled for future writes.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdebug.html#nospace">nospace</a>() and <a href="http://doc.qt.io/qt-5/qdebug.html#maybeSpace">maybeSpace</a>().</p></div>
    pub unsafe fn space(&mut self) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDebug_space(self as *mut crate::QDebug);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Swaps this debug stream instance with <i>other</i>. This function is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDebug::swap(QDebug& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps this debug stream instance with <i>other</i>. This function is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QDebug>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QDebug_swap(
            self as *mut crate::QDebug,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QDebug>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the verbosity of the debug stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QDebug::verbosity() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#verbosity">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the verbosity of the debug stream.</p>
    /// <p>Streaming operators can check the value to decide whether verbose output is desired and print more information depending on the level. Higher values indicate that more information is desired.</p>
    /// <p>The allowed range is from 0 to 7. The default value is 2.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdebug.html#setVerbosity">setVerbosity</a>().</p></div>
    pub unsafe fn verbosity(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QDebug_verbosity(self as *const crate::QDebug)
    }

    /// <p>Sets the verbosity of the stream to <i>verbosityLevel</i> and returns a reference to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::verbosity(int verbosityLevel)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#verbosity">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the verbosity of the stream to <i>verbosityLevel</i> and returns a reference to the stream.</p>
    /// <p>The allowed range is from 0 to 7. The default value is 2.</p>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdebug.html#verbosity-1">verbosity</a>(), <a href="http://doc.qt.io/qt-5/qdebug.html#setVerbosity">setVerbosity</a>(), and <a href="http://doc.qt.io/qt-5/qdebug.html#VerbosityLevel-enum">VerbosityLevel</a>.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn verbosity_mut(
        &mut self,
        verbosity_level: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDebug_verbosity2(
            self as *mut crate::QDebug,
            verbosity_level,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }
}

/// <p>Convenience class for custom <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> operators</p>
///
/// C++ class: <span style='color: green;'>```QDebugStateSaver```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qdebugstatesaver.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Convenience class for custom <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> operators</p>
/// <p>Saves the settings used by <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a>, and restores them upon destruction, then calls <a href="http://doc.qt.io/qt-5/qdebug.html#maybeSpace">maybeSpace()</a>, to separate arguments with a space if <a href="http://doc.qt.io/qt-5/qdebug.html#autoInsertSpaces">autoInsertSpaces()</a> was true at the time of constructing the <a href="http://doc.qt.io/qt-5/qdebugstatesaver.html">QDebugStateSaver</a>.</p>
/// <p>The automatic insertion of spaces between writes is one of the settings that <a href="http://doc.qt.io/qt-5/qdebugstatesaver.html">QDebugStateSaver</a> stores for the duration of the current block.</p>
/// <p>The settings of the internal <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> are also saved and restored, so that using &lt;&lt; hex in a <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> operator doesn't affect other <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> operators.</p></div>
#[repr(C)]
pub struct QDebugStateSaver {
    _unused: u8,
}
impl QDebugStateSaver {
    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qdebugstatesaver.html">QDebugStateSaver</a> instance, which saves the settings currently used by <i>dbg</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDebugStateSaver::QDebugStateSaver(QDebug& dbg)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebugstatesaver.html#QDebugStateSaver">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qdebugstatesaver.html">QDebugStateSaver</a> instance, which saves the settings currently used by <i>dbg</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdebug.html#setAutoInsertSpaces">QDebug::setAutoInsertSpaces</a>() and <a href="http://doc.qt.io/qt-5/qdebug.html#autoInsertSpaces">QDebug::autoInsertSpaces</a>().</p></div>
    pub unsafe fn new(
        dbg: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QDebug>>,
    ) -> ::cpp_utils::CppBox<crate::QDebugStateSaver> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDebugStateSaver_QDebugStateSaver(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QDebug>>::cast_into(dbg)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_file_device {
    //! C++ type: <span style='color: green;'>```QFileDevice```</span>

    /// <p>This enum describes the errors that may be returned by the <a href="http://doc.qt.io/qt-5/qfiledevice.html#error">error</a>() function.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QFileDevice::FileError```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfiledevice.html#FileError-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the errors that may be returned by the <a href="http://doc.qt.io/qt-5/qfiledevice.html#error">error</a>() function.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct FileError(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for FileError {
        fn from(value: ::std::os::raw::c_int) -> Self {
            FileError(value)
        }
    }

    impl From<FileError> for ::std::os::raw::c_int {
        fn from(value: FileError) -> Self {
            value.0
        }
    }

    impl FileError {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl FileError {
        /// No error occurred. (C++ enum variant: <span style='color: green;'>```NoError = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoError: crate::q_file_device::FileError = crate::q_file_device::FileError(0);
        /// An error occurred when reading from the file. (C++ enum variant: <span style='color: green;'>```ReadError = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const ReadError: crate::q_file_device::FileError = crate::q_file_device::FileError(1);
        /// An error occurred when writing to the file. (C++ enum variant: <span style='color: green;'>```WriteError = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const WriteError: crate::q_file_device::FileError = crate::q_file_device::FileError(2);
        /// A fatal error occurred. (C++ enum variant: <span style='color: green;'>```FatalError = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const FatalError: crate::q_file_device::FileError = crate::q_file_device::FileError(3);
        /// Out of resources (e.g., too many open files, out of memory, etc.) (C++ enum variant: <span style='color: green;'>```ResourceError = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const ResourceError: crate::q_file_device::FileError =
            crate::q_file_device::FileError(4);
        /// The file could not be opened. (C++ enum variant: <span style='color: green;'>```OpenError = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const OpenError: crate::q_file_device::FileError = crate::q_file_device::FileError(5);
        /// The operation was aborted. (C++ enum variant: <span style='color: green;'>```AbortError = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const AbortError: crate::q_file_device::FileError = crate::q_file_device::FileError(6);
        /// A timeout occurred. (C++ enum variant: <span style='color: green;'>```TimeOutError = 7```</span>)
        #[allow(non_upper_case_globals)]
        pub const TimeOutError: crate::q_file_device::FileError =
            crate::q_file_device::FileError(7);
        /// An unspecified error occurred. (C++ enum variant: <span style='color: green;'>```UnspecifiedError = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const UnspecifiedError: crate::q_file_device::FileError =
            crate::q_file_device::FileError(8);
        /// The file could not be removed. (C++ enum variant: <span style='color: green;'>```RemoveError = 9```</span>)
        #[allow(non_upper_case_globals)]
        pub const RemoveError: crate::q_file_device::FileError = crate::q_file_device::FileError(9);
        /// The file could not be renamed. (C++ enum variant: <span style='color: green;'>```RenameError = 10```</span>)
        #[allow(non_upper_case_globals)]
        pub const RenameError: crate::q_file_device::FileError =
            crate::q_file_device::FileError(10);
        /// The position in the file could not be changed. (C++ enum variant: <span style='color: green;'>```PositionError = 11```</span>)
        #[allow(non_upper_case_globals)]
        pub const PositionError: crate::q_file_device::FileError =
            crate::q_file_device::FileError(11);
        /// The file could not be resized. (C++ enum variant: <span style='color: green;'>```ResizeError = 12```</span>)
        #[allow(non_upper_case_globals)]
        pub const ResizeError: crate::q_file_device::FileError =
            crate::q_file_device::FileError(12);
        /// The file could not be accessed. (C++ enum variant: <span style='color: green;'>```PermissionsError = 13```</span>)
        #[allow(non_upper_case_globals)]
        pub const PermissionsError: crate::q_file_device::FileError =
            crate::q_file_device::FileError(13);
        /// The file could not be copied. (C++ enum variant: <span style='color: green;'>```CopyError = 14```</span>)
        #[allow(non_upper_case_globals)]
        pub const CopyError: crate::q_file_device::FileError = crate::q_file_device::FileError(14);
    }

    /// <p>This enum is used by the permission() function to report the permissions and ownership of a file. The values may be OR-ed together to test multiple permissions and ownership values.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QFileDevice::Permission```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfiledevice.html#Permission-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum is used by the permission() function to report the permissions and ownership of a file. The values may be OR-ed together to test multiple permissions and ownership values.</p>
    ///
    /// <p><b>Warning:</b> Because of differences in the platforms supported by Qt, the semantics of ReadUser, WriteUser and ExeUser are platform-dependent: On Unix, the rights of the owner of the file are returned and on Windows the rights of the current user are returned. This behavior might change in a future Qt version.</p>
    /// <p><b>Note: </b>On NTFS file systems, ownership and permissions checking is disabled by default for performance reasons. To enable it, include the following line:</p><pre class="cpp">
    ///
    ///   <span class="keyword">extern</span> Q_CORE_EXPORT <span class="type">int</span> qt_ntfs_permission_lookup;
    ///
    /// </pre>
    /// <p>Permission checking is then turned on and off by incrementing and decrementing <code>qt_ntfs_permission_lookup</code> by 1.</p>
    /// <pre class="cpp">
    ///
    ///   qt_ntfs_permission_lookup<span class="operator">+</span><span class="operator">+</span>; <span class="comment">// turn checking on</span>
    ///   qt_ntfs_permission_lookup<span class="operator">-</span><span class="operator">-</span>; <span class="comment">// turn it off again</span>
    ///
    /// </pre>
    /// <p>The Permissions type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;Permission&gt;. It stores an OR combination of Permission values.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Permission(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Permission {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Permission(value)
        }
    }

    impl From<Permission> for ::std::os::raw::c_int {
        fn from(value: Permission) -> Self {
            value.0
        }
    }

    impl Permission {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Permission {
        /// The file is readable by the owner of the file. (C++ enum variant: <span style='color: green;'>```ReadOwner = 16384```</span>)
        #[allow(non_upper_case_globals)]
        pub const ReadOwner: crate::q_file_device::Permission =
            crate::q_file_device::Permission(16384);
        /// The file is writable by the owner of the file. (C++ enum variant: <span style='color: green;'>```WriteOwner = 8192```</span>)
        #[allow(non_upper_case_globals)]
        pub const WriteOwner: crate::q_file_device::Permission =
            crate::q_file_device::Permission(8192);
        /// The file is executable by the owner of the file. (C++ enum variant: <span style='color: green;'>```ExeOwner = 4096```</span>)
        #[allow(non_upper_case_globals)]
        pub const ExeOwner: crate::q_file_device::Permission =
            crate::q_file_device::Permission(4096);
        /// The file is readable by the user. (C++ enum variant: <span style='color: green;'>```ReadUser = 1024```</span>)
        #[allow(non_upper_case_globals)]
        pub const ReadUser: crate::q_file_device::Permission =
            crate::q_file_device::Permission(1024);
        /// The file is writable by the user. (C++ enum variant: <span style='color: green;'>```WriteUser = 512```</span>)
        #[allow(non_upper_case_globals)]
        pub const WriteUser: crate::q_file_device::Permission =
            crate::q_file_device::Permission(512);
        /// The file is executable by the user. (C++ enum variant: <span style='color: green;'>```ExeUser = 256```</span>)
        #[allow(non_upper_case_globals)]
        pub const ExeUser: crate::q_file_device::Permission = crate::q_file_device::Permission(256);
        /// The file is readable by the group. (C++ enum variant: <span style='color: green;'>```ReadGroup = 64```</span>)
        #[allow(non_upper_case_globals)]
        pub const ReadGroup: crate::q_file_device::Permission =
            crate::q_file_device::Permission(64);
        /// The file is writable by the group. (C++ enum variant: <span style='color: green;'>```WriteGroup = 32```</span>)
        #[allow(non_upper_case_globals)]
        pub const WriteGroup: crate::q_file_device::Permission =
            crate::q_file_device::Permission(32);
        /// The file is executable by the group. (C++ enum variant: <span style='color: green;'>```ExeGroup = 16```</span>)
        #[allow(non_upper_case_globals)]
        pub const ExeGroup: crate::q_file_device::Permission = crate::q_file_device::Permission(16);
        /// The file is readable by anyone. (C++ enum variant: <span style='color: green;'>```ReadOther = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const ReadOther: crate::q_file_device::Permission = crate::q_file_device::Permission(4);
        /// The file is writable by anyone. (C++ enum variant: <span style='color: green;'>```WriteOther = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const WriteOther: crate::q_file_device::Permission =
            crate::q_file_device::Permission(2);
        /// The file is executable by anyone. (C++ enum variant: <span style='color: green;'>```ExeOther = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const ExeOther: crate::q_file_device::Permission = crate::q_file_device::Permission(1);
    }

    /// <p>This enum is used when opening a file to specify additional options which only apply to files and not to a generic <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QFileDevice::FileHandleFlag```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfiledevice.html#FileHandleFlag-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum is used when opening a file to specify additional options which only apply to files and not to a generic <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>.</p>
    ///
    /// <p>The FileHandleFlags type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;FileHandleFlag&gt;. It stores an OR combination of FileHandleFlag values.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct FileHandleFlag(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for FileHandleFlag {
        fn from(value: ::std::os::raw::c_int) -> Self {
            FileHandleFlag(value)
        }
    }

    impl From<FileHandleFlag> for ::std::os::raw::c_int {
        fn from(value: FileHandleFlag) -> Self {
            value.0
        }
    }

    impl FileHandleFlag {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl FileHandleFlag {
        /// The file handle passed into <a href="http://doc.qt.io/qt-5/qiodevice.html#open">open</a>() should be closed by <a href="http://doc.qt.io/qt-5/qfiledevice.html#close">close</a>(), the default behavior is that close just flushes the file and the application is responsible for closing the file handle. When opening a file by name, this flag is ignored as Qt always owns the file handle and must close it. (C++ enum variant: <span style='color: green;'>```AutoCloseHandle = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const AutoCloseHandle: crate::q_file_device::FileHandleFlag =
            crate::q_file_device::FileHandleFlag(1);
        /// If not explicitly closed, the underlying file handle is left open when the <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> object is destroyed. (C++ enum variant: <span style='color: green;'>```DontCloseHandle = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const DontCloseHandle: crate::q_file_device::FileHandleFlag =
            crate::q_file_device::FileHandleFlag(0);
    }

    /// <p>This enum describes special options that may be used by the <a href="http://doc.qt.io/qt-5/qfiledevice.html#map">map</a>() function.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QFileDevice::MemoryMapFlags```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfiledevice.html#MemoryMapFlags-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes special options that may be used by the <a href="http://doc.qt.io/qt-5/qfiledevice.html#map">map</a>() function.</p>
    ///
    /// <p>This enum was introduced or modified in  Qt 4.4.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct MemoryMapFlags(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for MemoryMapFlags {
        fn from(value: ::std::os::raw::c_int) -> Self {
            MemoryMapFlags(value)
        }
    }

    impl From<MemoryMapFlags> for ::std::os::raw::c_int {
        fn from(value: MemoryMapFlags) -> Self {
            value.0
        }
    }

    impl MemoryMapFlags {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl MemoryMapFlags {
        /// No options. (C++ enum variant: <span style='color: green;'>```NoOptions = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoOptions: crate::q_file_device::MemoryMapFlags =
            crate::q_file_device::MemoryMapFlags(0);
        /// The mapped memory will be private, so any modifications will not be visible to other processes and will not be written to disk. Any such modifications will be lost when the memory is unmapped. It is unspecified whether modifications made to the file made after the mapping is created will be visible through the mapped memory. This enum value was introduced in Qt 5.4. (C++ enum variant: <span style='color: green;'>```MapPrivateOption = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const MapPrivateOption: crate::q_file_device::MemoryMapFlags =
            crate::q_file_device::MemoryMapFlags(1);
    }

    impl From<crate::q_file_device::Permission> for crate::QFlags<crate::q_file_device::Permission> {
        fn from(value: crate::q_file_device::Permission) -> Self {
            Self::from(value.to_int())
        }
    }

    impl<T: Into<crate::QFlags<crate::q_file_device::Permission>>> std::ops::BitOr<T>
        for crate::q_file_device::Permission
    {
        type Output = crate::QFlags<crate::q_file_device::Permission>;
        fn bitor(self, rhs: T) -> crate::QFlags<crate::q_file_device::Permission> {
            Into::<crate::QFlags<crate::q_file_device::Permission>>::into(self) | rhs
        }
    }

    impl From<crate::q_file_device::FileHandleFlag>
        for crate::QFlags<crate::q_file_device::FileHandleFlag>
    {
        fn from(value: crate::q_file_device::FileHandleFlag) -> Self {
            Self::from(value.to_int())
        }
    }

    impl<T: Into<crate::QFlags<crate::q_file_device::FileHandleFlag>>> std::ops::BitOr<T>
        for crate::q_file_device::FileHandleFlag
    {
        type Output = crate::QFlags<crate::q_file_device::FileHandleFlag>;
        fn bitor(self, rhs: T) -> crate::QFlags<crate::q_file_device::FileHandleFlag> {
            Into::<crate::QFlags<crate::q_file_device::FileHandleFlag>>::into(self) | rhs
        }
    }

    /// <p>This enum is used by the <a href="http://doc.qt.io/qt-5/qfiledevice.html#fileTime">fileTime</a>() and <a href="http://doc.qt.io/qt-5/qfiledevice.html#setFileTime">setFileTime</a>() functions.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QFileDevice::FileTime```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfiledevice.html#FileTime-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum is used by the <a href="http://doc.qt.io/qt-5/qfiledevice.html#fileTime">fileTime</a>() and <a href="http://doc.qt.io/qt-5/qfiledevice.html#setFileTime">setFileTime</a>() functions.</p>
    ///
    /// <p>This enum was introduced or modified in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfiledevice.html#setFileTime">setFileTime</a>(), <a href="http://doc.qt.io/qt-5/qfiledevice.html#fileTime">fileTime</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#fileTime">QFileInfo::fileTime</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct FileTime(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for FileTime {
        fn from(value: ::std::os::raw::c_int) -> Self {
            FileTime(value)
        }
    }

    impl From<FileTime> for ::std::os::raw::c_int {
        fn from(value: FileTime) -> Self {
            value.0
        }
    }

    impl FileTime {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl FileTime {
        /// When the file was most recently accessed (e.g. read or written to). (C++ enum variant: <span style='color: green;'>```FileAccessTime = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const FileAccessTime: crate::q_file_device::FileTime =
            crate::q_file_device::FileTime(0);
        /// When the file was created (may not be not supported on UNIX). (C++ enum variant: <span style='color: green;'>```FileBirthTime = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const FileBirthTime: crate::q_file_device::FileTime = crate::q_file_device::FileTime(1);
        /// When the file's metadata was last changed. (C++ enum variant: <span style='color: green;'>```FileMetadataChangeTime = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const FileMetadataChangeTime: crate::q_file_device::FileTime =
            crate::q_file_device::FileTime(2);
        /// When the file was most recently modified. (C++ enum variant: <span style='color: green;'>```FileModificationTime = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const FileModificationTime: crate::q_file_device::FileTime =
            crate::q_file_device::FileTime(3);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qfiledevice.html">QFileDevice</a> class provides an interface for reading from and writing to open files.</p>
///
/// C++ class: <span style='color: green;'>```QFileDevice```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qfiledevice.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qfiledevice.html">QFileDevice</a> class provides an interface for reading from and writing to open files.</p>
/// <p><a href="http://doc.qt.io/qt-5/qfiledevice.html">QFileDevice</a> is the base class for I/O devices that can read and write text and binary files and <a href="http://doc.qt.io/qt-5/resources.html">resources</a>. <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> offers the main functionality, <a href="http://doc.qt.io/qt-5/qfiledevice.html">QFileDevice</a> serves as a base class for sharing functionality with other file devices such as <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a>, by providing all the operations that can be done on files that have been opened by <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> or <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a>.</p></div>
#[repr(C)]
pub struct QFileDevice {
    _unused: u8,
}
impl QFileDevice {
    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#atEnd">QIODevice::atEnd</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QFileDevice::atEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfiledevice.html#atEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#atEnd">QIODevice::atEnd</a>().</p>
    /// <p>Returns <code>true</code> if the end of the file has been reached; otherwise returns false.</p>
    /// <p>For regular empty files on Unix (e.g. those in <code>/proc</code>), this function returns <code>true</code>, since the file system reports that the size of such a file is 0. Therefore, you should not depend on atEnd() when reading data from such a file, but rather call <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>() until no more data can be read.</p></div>
    pub unsafe fn at_end(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFileDevice_atEnd(self as *const crate::QFileDevice)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#close">QIODevice::close</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual void QFileDevice::close()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfiledevice.html#close">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#close">QIODevice::close</a>().</p>
    /// <p>Calls <a href="http://doc.qt.io/qt-5/qfiledevice.html#flush">QFileDevice::flush</a>() and closes the file. Errors from flush are ignored.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#close">QIODevice::close</a>().</p></div>
    pub unsafe fn close(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QFileDevice_close(self as *mut crate::QFileDevice)
    }

    /// <p>Returns the file error status.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFileDevice::FileError QFileDevice::error() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfiledevice.html#error">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the file error status.</p>
    /// <p>The I/O device status returns an error code. For example, if <a href="http://doc.qt.io/qt-5/qiodevice.html#open">open</a>() returns <code>false</code>, or a read/write operation returns -1, this function can be called to find out the reason why the operation failed.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfiledevice.html#unsetError">unsetError</a>().</p></div>
    pub unsafe fn error(&self) -> crate::q_file_device::FileError {
        crate::__ffi::ctr_qt_core_ffi_QFileDevice_error(self as *const crate::QFileDevice)
    }

    /// <p>Returns the name of the file. The default implementation in <a href="http://doc.qt.io/qt-5/qfiledevice.html">QFileDevice</a> returns a null string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QString QFileDevice::fileName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfiledevice.html#fileName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the name of the file. The default implementation in <a href="http://doc.qt.io/qt-5/qfiledevice.html">QFileDevice</a> returns a null string.</p></div>
    pub unsafe fn file_name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QFileDevice_fileName(self as *const crate::QFileDevice);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the file time specified by <i>time</i>. If the time cannot be determined return QDateTime() (an invalid date time).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDateTime QFileDevice::fileTime(QFileDevice::FileTime time) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfiledevice.html#fileTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the file time specified by <i>time</i>. If the time cannot be determined return QDateTime() (an invalid date time).</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfiledevice.html#setFileTime">setFileTime</a>(), <a href="http://doc.qt.io/qt-5/qfiledevice.html#FileTime-enum">FileTime</a>, and <a href="http://doc.qt.io/qt-5/qdatetime.html#isValid">QDateTime::isValid</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn file_time(
        &self,
        time: crate::q_file_device::FileTime,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileDevice_fileTime(
            self as *const crate::QFileDevice,
            time,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Flushes any buffered data to the file. Returns <code>true</code> if successful; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFileDevice::flush()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfiledevice.html#flush">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Flushes any buffered data to the file. Returns <code>true</code> if successful; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn flush(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFileDevice_flush(self as *mut crate::QFileDevice)
    }

    /// <p>Returns the file handle of the file.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QFileDevice::handle() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfiledevice.html#handle">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the file handle of the file.</p>
    /// <p>This is a small positive integer, suitable for use with C library functions such as <code>fdopen()</code> and <code>fcntl()</code>. On systems that use file descriptors for sockets (i.e. Unix systems, but not Windows) the handle can be used with <a href="http://doc.qt.io/qt-5/qsocketnotifier.html">QSocketNotifier</a> as well.</p>
    /// <p>If the file is not open, or there is an error, handle() returns -1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsocketnotifier.html">QSocketNotifier</a>.</p></div>
    pub unsafe fn handle(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QFileDevice_handle(self as *const crate::QFileDevice)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#isSequential">QIODevice::isSequential</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QFileDevice::isSequential() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfiledevice.html#isSequential">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#isSequential">QIODevice::isSequential</a>().</p>
    /// <p>Returns <code>true</code> if the file can only be manipulated sequentially; otherwise returns <code>false</code>.</p>
    /// <p>Most files support random-access, but some special files may not.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#isSequential">QIODevice::isSequential</a>().</p></div>
    pub unsafe fn is_sequential(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFileDevice_isSequential(self as *const crate::QFileDevice)
    }

    /// <p>Maps <i>size</i> bytes of the file into memory starting at <i>offset</i>. A file should be open for a map to succeed but the file does not need to stay open after the memory has been mapped. When the <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> is destroyed or a new file is opened with this object, any maps that have not been unmapped will automatically be unmapped.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned char* QFileDevice::map(qint64 offset, qint64 size, QFileDevice::MemoryMapFlags flags = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfiledevice.html#map">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Maps <i>size</i> bytes of the file into memory starting at <i>offset</i>. A file should be open for a map to succeed but the file does not need to stay open after the memory has been mapped. When the <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> is destroyed or a new file is opened with this object, any maps that have not been unmapped will automatically be unmapped.</p>
    /// <p>The mapping will have the same open mode as the file (read and/or write), except when using <a href="http://doc.qt.io/qt-5/qfiledevice.html#MemoryMapFlags-enum">MapPrivateOption</a>, in which case it is always possible to write to the mapped memory.</p>
    /// <p>Any mapping options can be passed through <i>flags</i>.</p>
    /// <p>Returns a pointer to the memory or 0 if there is an error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfiledevice.html#unmap">unmap</a>().</p></div>
    pub unsafe fn map_3a(
        &mut self,
        offset: i64,
        size: i64,
        flags: crate::q_file_device::MemoryMapFlags,
    ) -> ::cpp_utils::MutPtr<::std::os::raw::c_uchar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileDevice_map(
            self as *mut crate::QFileDevice,
            offset,
            size,
            flags,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Maps <i>size</i> bytes of the file into memory starting at <i>offset</i>. A file should be open for a map to succeed but the file does not need to stay open after the memory has been mapped. When the <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> is destroyed or a new file is opened with this object, any maps that have not been unmapped will automatically be unmapped.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned char* QFileDevice::map(qint64 offset, qint64 size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfiledevice.html#map">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Maps <i>size</i> bytes of the file into memory starting at <i>offset</i>. A file should be open for a map to succeed but the file does not need to stay open after the memory has been mapped. When the <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> is destroyed or a new file is opened with this object, any maps that have not been unmapped will automatically be unmapped.</p>
    /// <p>The mapping will have the same open mode as the file (read and/or write), except when using <a href="http://doc.qt.io/qt-5/qfiledevice.html#MemoryMapFlags-enum">MapPrivateOption</a>, in which case it is always possible to write to the mapped memory.</p>
    /// <p>Any mapping options can be passed through <i>flags</i>.</p>
    /// <p>Returns a pointer to the memory or 0 if there is an error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfiledevice.html#unmap">unmap</a>().</p></div>
    pub unsafe fn map_2a(
        &mut self,
        offset: i64,
        size: i64,
    ) -> ::cpp_utils::MutPtr<::std::os::raw::c_uchar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileDevice_map1(
            self as *mut crate::QFileDevice,
            offset,
            size,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QFileDevice::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QFileDevice_metaObject(self as *const crate::QFileDevice);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns the complete OR-ed together combination of <a href="http://doc.qt.io/qt-5/qfiledevice.html#Permission-enum">QFile::Permission</a> for the file.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QFlags<QFileDevice::Permission> QFileDevice::permissions() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfiledevice.html#permissions">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the complete OR-ed together combination of <a href="http://doc.qt.io/qt-5/qfiledevice.html#Permission-enum">QFile::Permission</a> for the file.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfiledevice.html#setPermissions">setPermissions</a>().</p></div>
    pub unsafe fn permissions(&self) -> crate::QFlags<crate::q_file_device::Permission> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileDevice_permissions(
            self as *const crate::QFileDevice,
        );
        crate::QFlags::from(ffi_result)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#pos">QIODevice::pos</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual qint64 QFileDevice::pos() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfiledevice.html#pos">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#pos">QIODevice::pos</a>().</p></div>
    pub unsafe fn pos(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QFileDevice_pos(self as *const crate::QFileDevice)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QFileDevice::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QFileDevice_qt_metacall(
            self as *mut crate::QFileDevice,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QFileDevice::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileDevice_qt_metacast(
            self as *mut crate::QFileDevice,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Sets the file size (in bytes) <i>sz</i>. Returns <code>true</code> if the resize succeeds; false otherwise. If <i>sz</i> is larger than the file currently is, the new bytes will be set to 0; if <i>sz</i> is smaller, the file is simply truncated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QFileDevice::resize(qint64 sz)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfiledevice.html#resize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the file size (in bytes) <i>sz</i>. Returns <code>true</code> if the resize succeeds; false otherwise. If <i>sz</i> is larger than the file currently is, the new bytes will be set to 0; if <i>sz</i> is smaller, the file is simply truncated.</p>
    /// <p><b>Warning:</b> This function can fail if the file doesn't exist.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfiledevice.html#size">size</a>().</p></div>
    pub unsafe fn resize(&mut self, sz: i64) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFileDevice_resize(self as *mut crate::QFileDevice, sz)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#seek">QIODevice::seek</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QFileDevice::seek(qint64 offset)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfiledevice.html#seek">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#seek">QIODevice::seek</a>().</p>
    /// <p>For random-access devices, this function sets the current position to <i>pos</i>, returning true on success, or false if an error occurred. For sequential devices, the default behavior is to do nothing and return false.</p>
    /// <p>Seeking beyond the end of a file: If the position is beyond the end of a file, then seek() will not immediately extend the file. If a write is performed at this position, then the file will be extended. The content of the file between the previous end of file and the newly written data is UNDEFINED and varies between platforms and file systems.</p></div>
    pub unsafe fn seek(&mut self, offset: i64) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFileDevice_seek(self as *mut crate::QFileDevice, offset)
    }

    /// <p>Sets the file time specified by <i>fileTime</i> to <i>newDate</i>, returning true if successful; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFileDevice::setFileTime(const QDateTime& newDate, QFileDevice::FileTime fileTime)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfiledevice.html#setFileTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the file time specified by <i>fileTime</i> to <i>newDate</i>, returning true if successful; otherwise returns false.</p>
    /// <p><b>Note: </b>The file must be open to use this function.</p><p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfiledevice.html#fileTime">fileTime</a>() and <a href="http://doc.qt.io/qt-5/qfiledevice.html#FileTime-enum">FileTime</a>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn set_file_time(
        &mut self,
        new_date: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
        file_time: crate::q_file_device::FileTime,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFileDevice_setFileTime(
            self as *mut crate::QFileDevice,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(new_date)
                .as_raw_ptr(),
            file_time,
        )
    }

    /// <p>Sets the permissions for the file to the <i>permissions</i> specified. Returns <code>true</code> if successful, or <code>false</code> if the permissions cannot be modified.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QFileDevice::setPermissions(QFlags<QFileDevice::Permission> permissionSpec)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfiledevice.html#setPermissions">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the permissions for the file to the <i>permissions</i> specified. Returns <code>true</code> if successful, or <code>false</code> if the permissions cannot be modified.</p>
    /// <p><b>Warning:</b> This function does not manipulate ACLs, which may limit its effectiveness.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfiledevice.html#permissions">permissions</a>().</p></div>
    pub unsafe fn set_permissions(
        &mut self,
        permission_spec: crate::QFlags<crate::q_file_device::Permission>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFileDevice_setPermissions(
            self as *mut crate::QFileDevice,
            permission_spec.to_int(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#size">QIODevice::size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual qint64 QFileDevice::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfiledevice.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#size">QIODevice::size</a>().</p>
    /// <p>Returns the size of the file.</p>
    /// <p>For regular empty files on Unix (e.g. those in <code>/proc</code>), this function returns 0; the contents of such a file are generated on demand in response to you calling <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>().</p></div>
    pub unsafe fn size(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QFileDevice_size(self as *const crate::QFileDevice)
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileDevice_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QFileDevice::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileDevice_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QFileDevice::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileDevice_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Unmaps the memory <i>address</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFileDevice::unmap(unsigned char* address)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfiledevice.html#unmap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Unmaps the memory <i>address</i>.</p>
    /// <p>Returns <code>true</code> if the unmap succeeds; false otherwise.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfiledevice.html#map">map</a>().</p></div>
    pub unsafe fn unmap(
        &mut self,
        address: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_uchar>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFileDevice_unmap(
            self as *mut crate::QFileDevice,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_uchar>>::cast_into(
                address,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Sets the file's error to <a href="http://doc.qt.io/qt-5/qfiledevice.html#FileError-enum">QFileDevice::NoError</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QFileDevice::unsetError()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfiledevice.html#unsetError">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the file's error to <a href="http://doc.qt.io/qt-5/qfiledevice.html#FileError-enum">QFileDevice::NoError</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfiledevice.html#error">error</a>().</p></div>
    pub unsafe fn unset_error(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QFileDevice_unsetError(self as *mut crate::QFileDevice)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> class provides an interface for reading from and writing to files.</p>
///
/// C++ class: <span style='color: green;'>```QFile```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qfile.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> class provides an interface for reading from and writing to files.</p>
/// <p><a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> is an I/O device for reading and writing text and binary files and <a href="http://doc.qt.io/qt-5/resources.html">resources</a>. A <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> may be used by itself or, more conveniently, with a <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> or <a href="http://doc.qt.io/qt-5/qdatastream.html">QDataStream</a>.</p>
/// <p>The file name is usually passed in the constructor, but it can be set at any time using <a href="http://doc.qt.io/qt-5/qfile.html#setFileName">setFileName</a>(). <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> expects the file separator to be '/' regardless of operating system. The use of other separators (e.g., '\') is not supported.</p>
/// <p>You can check for a file's existence using <a href="http://doc.qt.io/qt-5/qfile.html#exists-1">exists</a>(), and remove a file using <a href="http://doc.qt.io/qt-5/qfile.html#remove">remove</a>(). (More advanced file system related operations are provided by <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> and <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a>.)</p>
/// <p>The file is opened with <a href="http://doc.qt.io/qt-5/qfile.html#open">open</a>(), closed with <a href="http://doc.qt.io/qt-5/qfiledevice.html#close">close</a>(), and flushed with <a href="http://doc.qt.io/qt-5/qfiledevice.html#flush">flush</a>(). Data is usually read and written using <a href="http://doc.qt.io/qt-5/qdatastream.html">QDataStream</a> or <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>, but you can also call the <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>-inherited functions <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#readLine">readLine</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#readAll">readAll</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#write">write</a>(). <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> also inherits <a href="http://doc.qt.io/qt-5/qiodevice.html#getChar">getChar</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#putChar">putChar</a>(), and <a href="http://doc.qt.io/qt-5/qiodevice.html#ungetChar">ungetChar</a>(), which work one character at a time.</p>
/// <p>The size of the file is returned by <a href="http://doc.qt.io/qt-5/qfile.html#size">size</a>(). You can get the current file position using <a href="http://doc.qt.io/qt-5/qfiledevice.html#pos">pos</a>(), or move to a new file position using <a href="http://doc.qt.io/qt-5/qfiledevice.html#seek">seek</a>(). If you've reached the end of the file, <a href="http://doc.qt.io/qt-5/qfiledevice.html#atEnd">atEnd</a>() returns <code>true</code>.</p>
/// <a name="reading-files-directly"></a></div>
#[repr(C)]
pub struct QFile {
    _unused: u8,
}
impl QFile {
    /// <p>Copies the file currently specified by <a href="http://doc.qt.io/qt-5/qfile.html#fileName">fileName</a>() to a file called <i>newName</i>. Returns <code>true</code> if successful; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFile::copy(const QString& newName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#copy">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Copies the file currently specified by <a href="http://doc.qt.io/qt-5/qfile.html#fileName">fileName</a>() to a file called <i>newName</i>. Returns <code>true</code> if successful; otherwise returns <code>false</code>.</p>
    /// <p>Note that if a file with the name <i>newName</i> already exists, copy() returns <code>false</code> (i.e. <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> will not overwrite it).</p>
    /// <p>The source file is closed before it is copied.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfile.html#setFileName">setFileName</a>().</p></div>
    pub unsafe fn copy_mut(
        &mut self,
        new_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFile_copy(
            self as *mut crate::QFile,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(new_name)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QFile::copy(const QString& fileName, const QString& newName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#copy-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Copies the file <i>fileName</i> to <i>newName</i>. Returns <code>true</code> if successful; otherwise returns <code>false</code>.</p>
    /// <p>If a file with the name <i>newName</i> already exists, <a href="http://doc.qt.io/qt-5/qfile.html#copy">copy</a>() returns <code>false</code> (i.e., <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> will not overwrite it).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfile.html#rename">rename</a>().</p></div>
    pub unsafe fn copy(
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        new_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFile_copy1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(new_name)
                .as_raw_ptr(),
        )
    }

    /// <p>This does the reverse of <a href="http://doc.qt.io/qt-5/qfile.html#encodeName">QFile::encodeName</a>() using <i>localFileName</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QFile::decodeName(const QByteArray& localFileName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#decodeName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This does the reverse of <a href="http://doc.qt.io/qt-5/qfile.html#encodeName">QFile::encodeName</a>() using <i>localFileName</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfile.html#encodeName">encodeName</a>().</p></div>
    pub unsafe fn decode_name_q_byte_array(
        local_file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFile_decodeName(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(
                local_file_name,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QFile::decodeName(const char* localFileName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#decodeName-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the Unicode version of the given <i>localFileName</i>. See <a href="http://doc.qt.io/qt-5/qfile.html#encodeName">encodeName</a>() for details.</p></div>
    pub unsafe fn decode_name_char(
        local_file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFile_decodeName1(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(
                local_file_name,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts <i>fileName</i> to the local 8-bit encoding determined by the user's locale. This is sufficient for file names that the user chooses. File names hard-coded into the application should only use 7-bit ASCII filename characters.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QFile::encodeName(const QString& fileName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#encodeName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts <i>fileName</i> to the local 8-bit encoding determined by the user's locale. This is sufficient for file names that the user chooses. File names hard-coded into the application should only use 7-bit ASCII filename characters.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfile.html#decodeName">decodeName</a>().</p></div>
    pub unsafe fn encode_name(
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFile_encodeName(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFile::exists() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#exists-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the file specified by <a href="http://doc.qt.io/qt-5/qfile.html#fileName">fileName</a>() exists; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfile.html#fileName">fileName</a>() and <a href="http://doc.qt.io/qt-5/qfile.html#setFileName">setFileName</a>().</p></div>
    pub unsafe fn exists_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFile_exists(self as *const crate::QFile)
    }

    /// <p>Returns <code>true</code> if the file specified by <i>fileName</i> exists; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QFile::exists(const QString& fileName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#exists">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the file specified by <i>fileName</i> exists; otherwise returns <code>false</code>.</p>
    /// <p><b>Note: </b>If <i>fileName</i> is a symlink that points to a non-existing file, false is returned.</p></div>
    pub unsafe fn exists_1a(
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFile_exists1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qfiledevice.html#fileName">QFileDevice::fileName</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QString QFile::fileName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#fileName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qfiledevice.html#fileName">QFileDevice::fileName</a>().</p>
    /// <p>Returns the name set by <a href="http://doc.qt.io/qt-5/qfile.html#setFileName">setFileName</a>() or to the <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> constructors.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfile.html#setFileName">setFileName</a>() and <a href="http://doc.qt.io/qt-5/qfileinfo.html#fileName">QFileInfo::fileName</a>().</p></div>
    pub unsafe fn file_name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFile_fileName(self as *const crate::QFile);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a link named <i>linkName</i> that points to the file currently specified by <a href="http://doc.qt.io/qt-5/qfile.html#fileName">fileName</a>(). What a link is depends on the underlying filesystem (be it a shortcut on Windows or a symbolic link on Unix). Returns <code>true</code> if successful; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFile::link(const QString& newName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#link">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a link named <i>linkName</i> that points to the file currently specified by <a href="http://doc.qt.io/qt-5/qfile.html#fileName">fileName</a>(). What a link is depends on the underlying filesystem (be it a shortcut on Windows or a symbolic link on Unix). Returns <code>true</code> if successful; otherwise returns <code>false</code>.</p>
    /// <p>This function will not overwrite an already existing entity in the file system; in this case, <code>link()</code> will return false and set <a href="http://doc.qt.io/qt-5/qfiledevice.html#error">error()</a> to return <a href="http://doc.qt.io/qt-5/qfiledevice.html#FileError-enum">RenameError</a>.</p>
    /// <p><b>Note: </b>To create a valid link on Windows, <i>linkName</i> must have a <code>.lnk</code> file extension.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qfile.html#setFileName">setFileName</a>().</p></div>
    pub unsafe fn link_mut(
        &mut self,
        new_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFile_link(
            self as *mut crate::QFile,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(new_name)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QFile::link(const QString& oldname, const QString& newName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#link-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Creates a link named <i>linkName</i> that points to the file <i>fileName</i>. What a link is depends on the underlying filesystem (be it a shortcut on Windows or a symbolic link on Unix). Returns <code>true</code> if successful; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfile.html#link">link</a>().</p></div>
    pub unsafe fn link(
        oldname: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        new_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFile_link1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(oldname)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(new_name)
                .as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QFile::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QFile_metaObject(self as *const crate::QFile);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QFile::QFile()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#QFile">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> object.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFile_QFile();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new file object to represent the file with the given <i>name</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QFile::QFile(const QString& name)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#QFile-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new file object to represent the file with the given <i>name</i>.</p></div>
    pub unsafe fn from_q_string(
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFile_QFile1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new file object with the given <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QFile::QFile(QObject* parent)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#QFile-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new file object with the given <i>parent</i>.</p></div>
    pub unsafe fn from_q_object(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFile_QFile2(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new file object with the given <i>parent</i> to represent the file with the specified <i>name</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QFile::QFile(const QString& name, QObject* parent)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#QFile-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new file object with the given <i>parent</i> to represent the file with the specified <i>name</i>.</p></div>
    pub unsafe fn from_q_string_q_object(
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFile_QFile3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#open">QIODevice::open</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QFile::open(QFlags<QIODevice::OpenModeFlag> flags)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#open">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#open">QIODevice::open</a>().</p>
    /// <p>Opens the file using <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">OpenMode</a> <i>mode</i>, returning true if successful; otherwise false.</p>
    /// <p>The <i>mode</i> must be <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">QIODevice::ReadOnly</a>, <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">QIODevice::WriteOnly</a>, or <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">QIODevice::ReadWrite</a>. It may also have additional flags, such as <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">QIODevice::Text</a> and <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">QIODevice::Unbuffered</a>.</p>
    /// <p><b>Note: </b>In <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">WriteOnly</a> or <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">ReadWrite</a> mode, if the relevant file does not already exist, this function will try to create a new file before opening it.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">QIODevice::OpenMode</a> and <a href="http://doc.qt.io/qt-5/qfile.html#setFileName">setFileName</a>().</p></div>
    pub unsafe fn open_1a(
        &mut self,
        flags: crate::QFlags<crate::q_io_device::OpenModeFlag>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFile_open(self as *mut crate::QFile, flags.to_int())
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFile::open(int fd, QFlags<QIODevice::OpenModeFlag> ioFlags, QFlags<QFileDevice::FileHandleFlag> handleFlags = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#open-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Opens the existing file descriptor <i>fd</i> in the given <i>mode</i>. <i>handleFlags</i> may be used to specify additional options. Returns <code>true</code> if successful; otherwise returns <code>false</code>.</p>
    /// <p>When a <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> is opened using this function, behaviour of <a href="http://doc.qt.io/qt-5/qfiledevice.html#close">close</a>() is controlled by the <a href="http://doc.qt.io/qt-5/qfiledevice.html#FileHandleFlag-enum">AutoCloseHandle</a> flag. If <a href="http://doc.qt.io/qt-5/qfiledevice.html#FileHandleFlag-enum">AutoCloseHandle</a> is specified, and this function succeeds, then calling <a href="http://doc.qt.io/qt-5/qfiledevice.html#close">close</a>() closes the adopted handle. Otherwise, <a href="http://doc.qt.io/qt-5/qfiledevice.html#close">close</a>() does not actually close the file, but only flushes it.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> that is opened using this function is automatically set to be in raw mode; this means that the file input/output functions are slow. If you run into performance issues, you should try to use one of the other open functions.</p>
    /// <p><b>Warning:</b> If <i>fd</i> is not a regular file, e.g, it is 0 (<code>stdin</code>), 1 (<code>stdout</code>), or 2 (<code>stderr</code>), you may not be able to <a href="http://doc.qt.io/qt-5/qfiledevice.html#seek">seek</a>(). In those cases, <a href="http://doc.qt.io/qt-5/qfile.html#size">size</a>() returns <code>0</code>. See <a href="http://doc.qt.io/qt-5/qiodevice.html#isSequential">QIODevice::isSequential</a>() for more information.</p>
    /// <p><b>Warning:</b> Since this function opens the file without specifying the file name, you cannot use this <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> with a <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfiledevice.html#close">close</a>().</p></div>
    pub unsafe fn open_3a(
        &mut self,
        fd: ::std::os::raw::c_int,
        io_flags: crate::QFlags<crate::q_io_device::OpenModeFlag>,
        handle_flags: crate::QFlags<crate::q_file_device::FileHandleFlag>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFile_open1(
            self as *mut crate::QFile,
            fd,
            io_flags.to_int(),
            handle_flags.to_int(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFile::open(int fd, QFlags<QIODevice::OpenModeFlag> ioFlags)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#open-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Opens the existing file descriptor <i>fd</i> in the given <i>mode</i>. <i>handleFlags</i> may be used to specify additional options. Returns <code>true</code> if successful; otherwise returns <code>false</code>.</p>
    /// <p>When a <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> is opened using this function, behaviour of <a href="http://doc.qt.io/qt-5/qfiledevice.html#close">close</a>() is controlled by the <a href="http://doc.qt.io/qt-5/qfiledevice.html#FileHandleFlag-enum">AutoCloseHandle</a> flag. If <a href="http://doc.qt.io/qt-5/qfiledevice.html#FileHandleFlag-enum">AutoCloseHandle</a> is specified, and this function succeeds, then calling <a href="http://doc.qt.io/qt-5/qfiledevice.html#close">close</a>() closes the adopted handle. Otherwise, <a href="http://doc.qt.io/qt-5/qfiledevice.html#close">close</a>() does not actually close the file, but only flushes it.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> that is opened using this function is automatically set to be in raw mode; this means that the file input/output functions are slow. If you run into performance issues, you should try to use one of the other open functions.</p>
    /// <p><b>Warning:</b> If <i>fd</i> is not a regular file, e.g, it is 0 (<code>stdin</code>), 1 (<code>stdout</code>), or 2 (<code>stderr</code>), you may not be able to <a href="http://doc.qt.io/qt-5/qfiledevice.html#seek">seek</a>(). In those cases, <a href="http://doc.qt.io/qt-5/qfile.html#size">size</a>() returns <code>0</code>. See <a href="http://doc.qt.io/qt-5/qiodevice.html#isSequential">QIODevice::isSequential</a>() for more information.</p>
    /// <p><b>Warning:</b> Since this function opens the file without specifying the file name, you cannot use this <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> with a <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfiledevice.html#close">close</a>().</p></div>
    pub unsafe fn open_2a(
        &mut self,
        fd: ::std::os::raw::c_int,
        io_flags: crate::QFlags<crate::q_io_device::OpenModeFlag>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFile_open2(self as *mut crate::QFile, fd, io_flags.to_int())
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qfiledevice.html#permissions">QFileDevice::permissions</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QFlags<QFileDevice::Permission> QFile::permissions() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#permissions">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qfiledevice.html#permissions">QFileDevice::permissions</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfile.html#setPermissions">setPermissions</a>().</p></div>
    pub unsafe fn permissions_0a(&self) -> crate::QFlags<crate::q_file_device::Permission> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QFile_permissions(self as *const crate::QFile);
        crate::QFlags::from(ffi_result)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QFlags<QFileDevice::Permission> QFile::permissions(const QString& filename)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#permissions-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the complete OR-ed together combination of <a href="http://doc.qt.io/qt-5/qfiledevice.html#Permission-enum">QFile::Permission</a> for <i>fileName</i>.</p></div>
    pub unsafe fn permissions_1a(
        filename: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> crate::QFlags<crate::q_file_device::Permission> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFile_permissions1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(filename)
                .as_raw_ptr(),
        );
        crate::QFlags::from(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QFile::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QFile_qt_metacall(
            self as *mut crate::QFile,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QFile::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFile_qt_metacast(
            self as *mut crate::QFile,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Use <a href="http://doc.qt.io/qt-5/qfile.html#symLinkTarget-1">symLinkTarget</a>() instead.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QFile::readLink() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile-obsolete.html#readLink">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Use <a href="http://doc.qt.io/qt-5/qfile.html#symLinkTarget-1">symLinkTarget</a>() instead.</p></div>
    pub unsafe fn read_link_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFile_readLink(self as *const crate::QFile);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Use <a href="http://doc.qt.io/qt-5/qfile.html#symLinkTarget-1">symLinkTarget</a>() instead.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QFile::readLink(const QString& fileName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile-obsolete.html#readLink-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Use <a href="http://doc.qt.io/qt-5/qfile.html#symLinkTarget-1">symLinkTarget</a>() instead.</p></div>
    pub unsafe fn read_link_1a(
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFile_readLink1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the file specified by <a href="http://doc.qt.io/qt-5/qfile.html#fileName">fileName</a>(). Returns <code>true</code> if successful; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFile::remove()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#remove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the file specified by <a href="http://doc.qt.io/qt-5/qfile.html#fileName">fileName</a>(). Returns <code>true</code> if successful; otherwise returns <code>false</code>.</p>
    /// <p>The file is closed before it is removed.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfile.html#setFileName">setFileName</a>().</p></div>
    pub unsafe fn remove_mut(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFile_remove(self as *mut crate::QFile)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QFile::remove(const QString& fileName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#remove-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes the file specified by the <i>fileName</i> given.</p>
    /// <p>Returns <code>true</code> if successful; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfile.html#remove">remove</a>().</p></div>
    pub unsafe fn remove(
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFile_remove1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        )
    }

    /// <p>Renames the file currently specified by <a href="http://doc.qt.io/qt-5/qfile.html#fileName">fileName</a>() to <i>newName</i>. Returns <code>true</code> if successful; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFile::rename(const QString& newName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#rename">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Renames the file currently specified by <a href="http://doc.qt.io/qt-5/qfile.html#fileName">fileName</a>() to <i>newName</i>. Returns <code>true</code> if successful; otherwise returns <code>false</code>.</p>
    /// <p>If a file with the name <i>newName</i> already exists, rename() returns <code>false</code> (i.e., <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> will not overwrite it).</p>
    /// <p>The file is closed before it is renamed.</p>
    /// <p>If the rename operation fails, Qt will attempt to copy this file's contents to <i>newName</i>, and then remove this file, keeping only <i>newName</i>. If that copy operation fails or this file can't be removed, the destination file <i>newName</i> is removed to restore the old state.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfile.html#setFileName">setFileName</a>().</p></div>
    pub unsafe fn rename_mut(
        &mut self,
        new_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFile_rename(
            self as *mut crate::QFile,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(new_name)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QFile::rename(const QString& oldName, const QString& newName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#rename-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Renames the file <i>oldName</i> to <i>newName</i>. Returns <code>true</code> if successful; otherwise returns <code>false</code>.</p>
    /// <p>If a file with the name <i>newName</i> already exists, <a href="http://doc.qt.io/qt-5/qfile.html#rename">rename</a>() returns <code>false</code> (i.e., <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> will not overwrite it).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfile.html#rename">rename</a>().</p></div>
    pub unsafe fn rename(
        old_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        new_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFile_rename1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(old_name)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(new_name)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qfiledevice.html#resize">QFileDevice::resize</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QFile::resize(qint64 sz)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#resize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qfiledevice.html#resize">QFileDevice::resize</a>().</p></div>
    pub unsafe fn resize_mut(&mut self, sz: i64) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFile_resize(self as *mut crate::QFile, sz)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QFile::resize(const QString& filename, qint64 sz)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#resize-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Sets <i>fileName</i> to size (in bytes) <i>sz</i>. Returns <code>true</code> if the resize succeeds; false otherwise. If <i>sz</i> is larger than <i>fileName</i> currently is the new bytes will be set to 0, if <i>sz</i> is smaller the file is simply truncated.</p>
    /// <p><b>Warning:</b> This function can fail if the file doesn't exist.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfile.html#resize">resize</a>().</p></div>
    pub unsafe fn resize(
        filename: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        sz: i64,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFile_resize1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(filename)
                .as_raw_ptr(),
            sz,
        )
    }

    /// <p>Sets the <i>name</i> of the file. The name can have no path, a relative path, or an absolute path.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QFile::setFileName(const QString& name)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#setFileName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the <i>name</i> of the file. The name can have no path, a relative path, or an absolute path.</p>
    /// <p>Do not call this function if the file has already been opened.</p>
    /// <p>If the file name has no path or a relative path, the path used will be the application's current directory path <i>at the time of the <a href="http://doc.qt.io/qt-5/qfile.html#open">open</a>()</i> call.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qfile.html#QFile">QFile</a></span> file;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdir.html">QDir</a></span><span class="operator">::</span>setCurrent(<span class="string">"/tmp"</span>);
    ///   file<span class="operator">.</span>setFileName(<span class="string">"readme.txt"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdir.html">QDir</a></span><span class="operator">::</span>setCurrent(<span class="string">"/home"</span>);
    ///   file<span class="operator">.</span>open(<span class="type"><a href="http://doc.qt.io/qt-5/qiodevice.html#QIODevice">QIODevice</a></span><span class="operator">::</span>ReadOnly);      <span class="comment">// opens "/home/readme.txt" under Unix</span>
    ///
    /// </pre>
    /// <p>Note that the directory separator "/" works for all operating systems supported by Qt.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfile.html#fileName">fileName</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a>, and <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a>.</p></div>
    pub unsafe fn set_file_name(
        &mut self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QFile_setFileName(
            self as *mut crate::QFile,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qfiledevice.html#setPermissions">QFileDevice::setPermissions</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QFile::setPermissions(QFlags<QFileDevice::Permission> permissionSpec)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#setPermissions">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qfiledevice.html#setPermissions">QFileDevice::setPermissions</a>().</p>
    /// <p>Sets the permissions for the file to the <i>permissions</i> specified. Returns <code>true</code> if successful, or <code>false</code> if the permissions cannot be modified.</p>
    /// <p><b>Warning:</b> This function does not manipulate ACLs, which may limit its effectiveness.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfile.html#permissions">permissions</a>() and <a href="http://doc.qt.io/qt-5/qfile.html#setFileName">setFileName</a>().</p></div>
    pub unsafe fn set_permissions_mut(
        &mut self,
        permission_spec: crate::QFlags<crate::q_file_device::Permission>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFile_setPermissions(
            self as *mut crate::QFile,
            permission_spec.to_int(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QFile::setPermissions(const QString& filename, QFlags<QFileDevice::Permission> permissionSpec)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#setPermissions-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Sets the permissions for <i>fileName</i> file to <i>permissions</i>.</p></div>
    pub unsafe fn set_permissions(
        filename: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        permission_spec: crate::QFlags<crate::q_file_device::Permission>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFile_setPermissions1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(filename)
                .as_raw_ptr(),
            permission_spec.to_int(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#size">QIODevice::size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual qint64 QFile::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#size">QIODevice::size</a>().</p></div>
    pub unsafe fn size(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QFile_size(self as *const crate::QFile)
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFile_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QFile::symLinkTarget() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#symLinkTarget-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the absolute path of the file or directory a symlink (or shortcut on Windows) points to, or a an empty string if the object isn't a symbolic link.</p>
    /// <p>This name may not represent an existing file; it is only a string. <a href="http://doc.qt.io/qt-5/qfile.html#exists-1">QFile::exists</a>() returns <code>true</code> if the symlink points to an existing file.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfile.html#fileName">fileName</a>() and <a href="http://doc.qt.io/qt-5/qfile.html#setFileName">setFileName</a>().</p></div>
    pub unsafe fn sym_link_target_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QFile_symLinkTarget(self as *const crate::QFile);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the absolute path of the file or directory referred to by the symlink (or shortcut on Windows) specified by <i>fileName</i>, or returns an empty string if the <i>fileName</i> does not correspond to a symbolic link.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QFile::symLinkTarget(const QString& fileName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#symLinkTarget">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the absolute path of the file or directory referred to by the symlink (or shortcut on Windows) specified by <i>fileName</i>, or returns an empty string if the <i>fileName</i> does not correspond to a symbolic link.</p>
    /// <p>This name may not represent an existing file; it is only a string. <a href="http://doc.qt.io/qt-5/qfile.html#exists-1">QFile::exists</a>() returns <code>true</code> if the symlink points to an existing file.</p>
    /// <p>This function was introduced in  Qt 4.2.</p></div>
    pub unsafe fn sym_link_target_1a(
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFile_symLinkTarget1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QFile::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFile_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QFile::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFile_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> class provides system-independent file information.</p>
///
/// C++ class: <span style='color: green;'>```QFileInfo```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qfileinfo.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> class provides system-independent file information.</p>
/// <p><a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> provides information about a file's name and position (path) in the file system, its access rights and whether it is a directory or symbolic link, etc. The file's size and last modified/read times are also available. <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> can also be used to obtain information about a Qt <a href="http://doc.qt.io/qt-5/resources.html">resource</a>.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> can point to a file with either a relative or an absolute file path. Absolute file paths begin with the directory separator "/" (or with a drive specification on Windows). Relative file names begin with a directory name or a file name and specify a path relative to the current working directory. An example of an absolute path is the string "/tmp/quartz". A relative path might look like "src/fatlib". You can use the function <a href="http://doc.qt.io/qt-5/qfileinfo.html#isRelative">isRelative</a>() to check whether a <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> is using a relative or an absolute file path. You can call the function <a href="http://doc.qt.io/qt-5/qfileinfo.html#makeAbsolute">makeAbsolute</a>() to convert a relative <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a>'s path to an absolute path.</p>
/// <p>The file that the <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> works on is set in the constructor or later with <a href="http://doc.qt.io/qt-5/qfileinfo.html#setFile">setFile</a>(). Use <a href="http://doc.qt.io/qt-5/qfileinfo.html#exists">exists</a>() to see if the file exists and <a href="http://doc.qt.io/qt-5/qfileinfo.html#size">size</a>() to get its size.</p>
/// <p>The file's type is obtained with <a href="http://doc.qt.io/qt-5/qfileinfo.html#isFile">isFile</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#isDir">isDir</a>() and <a href="http://doc.qt.io/qt-5/qfileinfo.html#isSymLink">isSymLink</a>(). The <a href="http://doc.qt.io/qt-5/qfileinfo.html#symLinkTarget">symLinkTarget</a>() function provides the name of the file the symlink points to.</p>
/// <p>On Unix (including <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS), the symlink has the same <a href="http://doc.qt.io/qt-5/qfileinfo.html#size">size</a>() has the file it points to, because Unix handles symlinks transparently; similarly, opening a symlink using <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> effectively opens the link's target. For example:</p>
/// <pre class="cpp">
///
///   <span class="preprocessor">#ifdef Q_OS_UNIX</span>
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qfileinfo.html#QFileInfo">QFileInfo</a></span> info1(<span class="string">"/home/bob/bin/untabify"</span>);
///   info1<span class="operator">.</span>isSymLink();          <span class="comment">// returns true</span>
///   info1<span class="operator">.</span>absoluteFilePath();   <span class="comment">// returns "/home/bob/bin/untabify"</span>
///   info1<span class="operator">.</span>size();               <span class="comment">// returns 56201</span>
///   info1<span class="operator">.</span>symLinkTarget();      <span class="comment">// returns "/opt/pretty++/bin/untabify"</span>
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qfileinfo.html#QFileInfo">QFileInfo</a></span> info2(info1<span class="operator">.</span>symLinkTarget());
///   info2<span class="operator">.</span>isSymLink();          <span class="comment">// returns false</span>
///   info2<span class="operator">.</span>absoluteFilePath();   <span class="comment">// returns "/opt/pretty++/bin/untabify"</span>
///   info2<span class="operator">.</span>size();               <span class="comment">// returns 56201</span>
///
///   <span class="preprocessor">#endif</span>
///
/// </pre>
/// <p>On Windows, symlinks (shortcuts) are <code>.lnk</code> files. The reported <a href="http://doc.qt.io/qt-5/qfileinfo.html#size">size</a>() is that of the symlink (not the link's target), and opening a symlink using <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> opens the <code>.lnk</code> file. For example:</p>
/// <pre class="cpp">
///
///   <span class="preprocessor">#ifdef Q_OS_WIN</span>
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qfileinfo.html#QFileInfo">QFileInfo</a></span> info1(<span class="string">"C:\\Documents and Settings\\Bob\\untabify.lnk"</span>);
///   info1<span class="operator">.</span>isSymLink();          <span class="comment">// returns true</span>
///   info1<span class="operator">.</span>absoluteFilePath();   <span class="comment">// returns "C:/Documents and Settings/Bob/untabify.lnk"</span>
///   info1<span class="operator">.</span>size();               <span class="comment">// returns 743</span>
///   info1<span class="operator">.</span>symLinkTarget();      <span class="comment">// returns "C:/Pretty++/untabify"</span>
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qfileinfo.html#QFileInfo">QFileInfo</a></span> info2(info1<span class="operator">.</span>symLinkTarget());
///   info2<span class="operator">.</span>isSymLink();          <span class="comment">// returns false</span>
///   info2<span class="operator">.</span>absoluteFilePath();   <span class="comment">// returns "C:/Pretty++/untabify"</span>
///   info2<span class="operator">.</span>size();               <span class="comment">// returns 63942</span>
///
///   <span class="preprocessor">#endif</span>
///
/// </pre>
/// <p>Elements of the file's name can be extracted with <a href="http://doc.qt.io/qt-5/qfileinfo.html#path">path</a>() and <a href="http://doc.qt.io/qt-5/qfileinfo.html#fileName">fileName</a>(). The <a href="http://doc.qt.io/qt-5/qfileinfo.html#fileName">fileName</a>()'s parts can be extracted with <a href="http://doc.qt.io/qt-5/qfileinfo.html#baseName">baseName</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#suffix">suffix</a>() or <a href="http://doc.qt.io/qt-5/qfileinfo.html#completeSuffix">completeSuffix</a>(). <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> objects to directories created by Qt classes will not have a trailing file separator. If you wish to use trailing separators in your own file info objects, just append one to the file name given to the constructors or <a href="http://doc.qt.io/qt-5/qfileinfo.html#setFile">setFile</a>().</p>
/// <p>The file's dates are returned by <a href="http://doc.qt.io/qt-5/qfileinfo.html#created">created</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#lastModified">lastModified</a>() and <a href="http://doc.qt.io/qt-5/qfileinfo.html#lastRead">lastRead</a>(). Information about the file's access permissions is obtained with <a href="http://doc.qt.io/qt-5/qfileinfo.html#isReadable">isReadable</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#isWritable">isWritable</a>() and <a href="http://doc.qt.io/qt-5/qfileinfo.html#isExecutable">isExecutable</a>(). The file's ownership is available from <a href="http://doc.qt.io/qt-5/qfileinfo.html#owner">owner</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#ownerId">ownerId</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#group">group</a>() and <a href="http://doc.qt.io/qt-5/qfileinfo.html#groupId">groupId</a>(). You can examine a file's permissions and ownership in a single statement using the <a href="http://doc.qt.io/qt-5/qfileinfo.html#permission">permission</a>() function.</p>
/// <a name="ntfs-permissions"></a><p><b>Note: </b>On NTFS file systems, ownership and permissions checking is disabled by default for performance reasons. To enable it, include the following line:</p><pre class="cpp">
///
///   <span class="keyword">extern</span> Q_CORE_EXPORT <span class="type">int</span> qt_ntfs_permission_lookup;
///
/// </pre>
/// <p>Permission checking is then turned on and off by incrementing and decrementing <code>qt_ntfs_permission_lookup</code> by 1.</p>
/// <pre class="cpp">
///
///   qt_ntfs_permission_lookup<span class="operator">+</span><span class="operator">+</span>; <span class="comment">// turn checking on</span>
///   qt_ntfs_permission_lookup<span class="operator">-</span><span class="operator">-</span>; <span class="comment">// turn it off again</span>
///
/// </pre>
/// <a name="performance-issues"></a></div>
#[repr(C)]
pub struct QFileInfo {
    _unused: u8,
}
impl QFileInfo {
    /// <p>Returns the file's absolute path as a <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDir QFileInfo::absoluteDir() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#absoluteDir">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the file's absolute path as a <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> object.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#dir">dir</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#filePath">filePath</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#fileName">fileName</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#isRelative">isRelative</a>().</p></div>
    pub unsafe fn absolute_dir(&self) -> ::cpp_utils::CppBox<crate::QDir> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QFileInfo_absoluteDir(self as *const crate::QFileInfo);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns an absolute path including the file name.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QFileInfo::absoluteFilePath() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#absoluteFilePath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an absolute path including the file name.</p>
    /// <p>The absolute path name consists of the full path and the file name. On Unix this will always begin with the root, '/', directory. On Windows this will always begin 'D:/' where D is a drive letter, except for network shares that are not mapped to a drive letter, in which case the path will begin '//sharename/'. <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> will uppercase drive letters. Note that <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> does not do this. The code snippet below shows this.</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qfileinfo.html#QFileInfo">QFileInfo</a></span> fi(<span class="string">"c:/temp/foo"</span>); <span class="operator">=</span><span class="operator">&gt;</span> fi<span class="operator">.</span>absoluteFilePath() <span class="operator">=</span><span class="operator">&gt;</span> <span class="string">"C:/temp/foo"</span>
    ///
    /// </pre>
    /// <p>This function returns the same as <a href="http://doc.qt.io/qt-5/qfileinfo.html#filePath">filePath</a>(), unless <a href="http://doc.qt.io/qt-5/qfileinfo.html#isRelative">isRelative</a>() is true. In contrast to <a href="http://doc.qt.io/qt-5/qfileinfo.html#canonicalFilePath">canonicalFilePath</a>(), symbolic links or redundant "." or ".." elements are not necessarily removed.</p>
    /// <p><b>Warning:</b> If <a href="http://doc.qt.io/qt-5/qfileinfo.html#filePath">filePath</a>() is empty the behavior of this function is undefined.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#filePath">filePath</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#canonicalFilePath">canonicalFilePath</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#isRelative">isRelative</a>().</p></div>
    pub unsafe fn absolute_file_path(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileInfo_absoluteFilePath(
            self as *const crate::QFileInfo,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a file's path absolute path. This doesn't include the file name.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QFileInfo::absolutePath() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#absolutePath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a file's path absolute path. This doesn't include the file name.</p>
    /// <p>On Unix the absolute path will always begin with the root, '/', directory. On Windows this will always begin 'D:/' where D is a drive letter, except for network shares that are not mapped to a drive letter, in which case the path will begin '//sharename/'.</p>
    /// <p>In contrast to <a href="http://doc.qt.io/qt-5/qfileinfo.html#canonicalPath">canonicalPath</a>() symbolic links or redundant "." or ".." elements are not necessarily removed.</p>
    /// <p><b>Warning:</b> If <a href="http://doc.qt.io/qt-5/qfileinfo.html#filePath">filePath</a>() is empty the behavior of this function is undefined.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#absoluteFilePath">absoluteFilePath</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#path">path</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#canonicalPath">canonicalPath</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#fileName">fileName</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#isRelative">isRelative</a>().</p></div>
    pub unsafe fn absolute_path(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QFileInfo_absolutePath(self as *const crate::QFileInfo);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the base name of the file without the path.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QFileInfo::baseName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#baseName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the base name of the file without the path.</p>
    /// <p>The base name consists of all characters in the file up to (but not including) the <i>first</i> '.' character.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qfileinfo.html#QFileInfo">QFileInfo</a></span> fi(<span class="string">"/tmp/archive.tar.gz"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> base <span class="operator">=</span> fi<span class="operator">.</span>baseName();  <span class="comment">// base = "archive"</span>
    ///
    /// </pre>
    /// <p>The base name of a file is computed equally on all platforms, independent of file naming conventions (e.g., ".bashrc" on Unix has an empty base name, and the suffix is "bashrc").</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#fileName">fileName</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#suffix">suffix</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#completeSuffix">completeSuffix</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#completeBaseName">completeBaseName</a>().</p></div>
    pub unsafe fn base_name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QFileInfo_baseName(self as *const crate::QFileInfo);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the date and time when the file was created / born.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDateTime QFileInfo::birthTime() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#birthTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the date and time when the file was created / born.</p>
    /// <p>If the file birth time is not available, this function returns an invalid <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#lastModified">lastModified</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#lastRead">lastRead</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#metadataChangeTime">metadataChangeTime</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn birth_time(&self) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QFileInfo_birthTime(self as *const crate::QFileInfo);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the name of the bundle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QFileInfo::bundleName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#bundleName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the name of the bundle.</p>
    /// <p>On <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS this returns the proper localized name for a bundle if the path <a href="http://doc.qt.io/qt-5/qfileinfo.html#isBundle">isBundle</a>(). On all other platforms an empty <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> is returned.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qfileinfo.html#QFileInfo">QFileInfo</a></span> fi(<span class="string">"/Applications/Safari.app"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> bundle <span class="operator">=</span> fi<span class="operator">.</span>bundleName();                <span class="comment">// name = "Safari"</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 4.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#isBundle">isBundle</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#filePath">filePath</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#baseName">baseName</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#suffix">suffix</a>().</p></div>
    pub unsafe fn bundle_name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QFileInfo_bundleName(self as *const crate::QFileInfo);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if caching is enabled; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFileInfo::caching() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#caching">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if caching is enabled; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#setCaching">setCaching</a>() and <a href="http://doc.qt.io/qt-5/qfileinfo.html#refresh">refresh</a>().</p></div>
    pub unsafe fn caching(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFileInfo_caching(self as *const crate::QFileInfo)
    }

    /// <p>Returns the canonical path including the file name, i.e. an absolute path without symbolic links or redundant "." or ".." elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QFileInfo::canonicalFilePath() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#canonicalFilePath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the canonical path including the file name, i.e. an absolute path without symbolic links or redundant "." or ".." elements.</p>
    /// <p>If the file does not exist, canonicalFilePath() returns an empty string.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#filePath">filePath</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#absoluteFilePath">absoluteFilePath</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#dir">dir</a>().</p></div>
    pub unsafe fn canonical_file_path(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileInfo_canonicalFilePath(
            self as *const crate::QFileInfo,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the file's path canonical path (excluding the file name), i.e. an absolute path without symbolic links or redundant "." or ".." elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QFileInfo::canonicalPath() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#canonicalPath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the file's path canonical path (excluding the file name), i.e. an absolute path without symbolic links or redundant "." or ".." elements.</p>
    /// <p>If the file does not exist, canonicalPath() returns an empty string.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#path">path</a>() and <a href="http://doc.qt.io/qt-5/qfileinfo.html#absolutePath">absolutePath</a>().</p></div>
    pub unsafe fn canonical_path(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QFileInfo_canonicalPath(self as *const crate::QFileInfo);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the complete base name of the file without the path.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QFileInfo::completeBaseName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#completeBaseName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the complete base name of the file without the path.</p>
    /// <p>The complete base name consists of all characters in the file up to (but not including) the <i>last</i> '.' character.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qfileinfo.html#QFileInfo">QFileInfo</a></span> fi(<span class="string">"/tmp/archive.tar.gz"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> base <span class="operator">=</span> fi<span class="operator">.</span>completeBaseName();  <span class="comment">// base = "archive.tar"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#fileName">fileName</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#suffix">suffix</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#completeSuffix">completeSuffix</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#baseName">baseName</a>().</p></div>
    pub unsafe fn complete_base_name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileInfo_completeBaseName(
            self as *const crate::QFileInfo,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the complete suffix (extension) of the file.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QFileInfo::completeSuffix() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#completeSuffix">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the complete suffix (extension) of the file.</p>
    /// <p>The complete suffix consists of all characters in the file after (but not including) the first '.'.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qfileinfo.html#QFileInfo">QFileInfo</a></span> fi(<span class="string">"/tmp/archive.tar.gz"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> ext <span class="operator">=</span> fi<span class="operator">.</span>completeSuffix();  <span class="comment">// ext = "tar.gz"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#fileName">fileName</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#suffix">suffix</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#baseName">baseName</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#completeBaseName">completeBaseName</a>().</p></div>
    pub unsafe fn complete_suffix(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QFileInfo_completeSuffix(self as *const crate::QFileInfo);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Makes a copy of the given <i>fileinfo</i> and assigns it to this <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFileInfo& QFileInfo::operator=(const QFileInfo& fileinfo)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes a copy of the given <i>fileinfo</i> and assigns it to this <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a>.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        fileinfo: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QFileInfo>>,
    ) -> ::cpp_utils::MutRef<crate::QFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileInfo_operator_(
            self as *mut crate::QFileInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QFileInfo>>::cast_into(fileinfo)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the date and local time when the file was created.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDateTime QFileInfo::created() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#created">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the date and local time when the file was created.</p>
    /// <p>On most Unix systems, this function returns the time of the last status change. A status change occurs when the file is created, but it also occurs whenever the user writes or sets inode information (for example, changing the file permissions).</p>
    /// <p>If neither creation time nor "last status change" time are not available, returns the same as <a href="http://doc.qt.io/qt-5/qfileinfo.html#lastModified">lastModified</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#lastModified">lastModified</a>() and <a href="http://doc.qt.io/qt-5/qfileinfo.html#lastRead">lastRead</a>().</p></div>
    pub unsafe fn created(&self) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QFileInfo_created(self as *const crate::QFileInfo);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the path of the object's parent directory as a <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDir QFileInfo::dir() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#dir">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the path of the object's parent directory as a <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> object.</p>
    /// <p><b>Note:</b> The <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> returned always corresponds to the object's parent directory, even if the <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> represents a directory.</p>
    /// <p>For each of the following, dir() returns the <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> <code>"~/examples/191697"</code>.</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qfileinfo.html#QFileInfo">QFileInfo</a></span> fileInfo1(<span class="string">"~/examples/191697/."</span>);
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qfileinfo.html#QFileInfo">QFileInfo</a></span> fileInfo2(<span class="string">"~/examples/191697/.."</span>);
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qfileinfo.html#QFileInfo">QFileInfo</a></span> fileInfo3(<span class="string">"~/examples/191697/main.cpp"</span>);
    ///
    /// </pre>
    /// <p>For each of the following, dir() returns the <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> <code>"."</code>.</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qfileinfo.html#QFileInfo">QFileInfo</a></span> fileInfo4(<span class="string">"."</span>);
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qfileinfo.html#QFileInfo">QFileInfo</a></span> fileInfo5(<span class="string">".."</span>);
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qfileinfo.html#QFileInfo">QFileInfo</a></span> fileInfo6(<span class="string">"main.cpp"</span>);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#absolutePath">absolutePath</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#filePath">filePath</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#fileName">fileName</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#isRelative">isRelative</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#absoluteDir">absoluteDir</a>().</p></div>
    pub unsafe fn dir(&self) -> ::cpp_utils::CppBox<crate::QDir> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QFileInfo_dir(self as *const crate::QFileInfo);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if the file exists; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFileInfo::exists() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#exists">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the file exists; otherwise returns <code>false</code>.</p>
    /// <p><b>Note: </b>If the file is a symlink that points to a non-existing file, false is returned.</p></div>
    pub unsafe fn exists_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFileInfo_exists(self as *const crate::QFileInfo)
    }

    /// <p>Returns <code>true</code> if the <i>file</i> exists; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QFileInfo::exists(const QString& file)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#exists-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the <i>file</i> exists; otherwise returns <code>false</code>.</p>
    /// <p><b>Note: </b>If <i>file</i> is a symlink that points to a non-existing file, false is returned.</p><p><b>Note: </b>Using this function is faster than using <code>QFileInfo(file).exists()</code> for file system access.</p><p>This function was introduced in  Qt 5.2.</p></div>
    pub unsafe fn exists_1a(
        file: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFileInfo_exists1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file).as_raw_ptr(),
        )
    }

    /// <p>Returns the name of the file, excluding the path.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QFileInfo::fileName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#fileName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the name of the file, excluding the path.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qfileinfo.html#QFileInfo">QFileInfo</a></span> fi(<span class="string">"/tmp/archive.tar.gz"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> name <span class="operator">=</span> fi<span class="operator">.</span>fileName();                <span class="comment">// name = "archive.tar.gz"</span>
    ///
    /// </pre>
    /// <p>Note that, if this <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> object is given a path ending in a slash, the name of the file is considered empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#isRelative">isRelative</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#filePath">filePath</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#baseName">baseName</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#suffix">suffix</a>().</p></div>
    pub unsafe fn file_name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QFileInfo_fileName(self as *const crate::QFileInfo);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the file name, including the path (which may be absolute or relative).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QFileInfo::filePath() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#filePath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the file name, including the path (which may be absolute or relative).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#absoluteFilePath">absoluteFilePath</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#canonicalFilePath">canonicalFilePath</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#isRelative">isRelative</a>().</p></div>
    pub unsafe fn file_path(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QFileInfo_filePath(self as *const crate::QFileInfo);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the file time specified by <i>time</i>. If the time cannot be determined, an invalid date time is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDateTime QFileInfo::fileTime(QFileDevice::FileTime time) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#fileTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the file time specified by <i>time</i>. If the time cannot be determined, an invalid date time is returned.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfiledevice.html#FileTime-enum">QFile::FileTime</a> and <a href="http://doc.qt.io/qt-5/qdatetime.html#isValid">QDateTime::isValid</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn file_time(
        &self,
        time: crate::q_file_device::FileTime,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QFileInfo_fileTime(self as *const crate::QFileInfo, time);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the group of the file. On Windows, on systems where files do not have groups, or if an error occurs, an empty string is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QFileInfo::group() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#group">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the group of the file. On Windows, on systems where files do not have groups, or if an error occurs, an empty string is returned.</p>
    /// <p>This function can be time consuming under Unix (in the order of milliseconds).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#groupId">groupId</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#owner">owner</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#ownerId">ownerId</a>().</p></div>
    pub unsafe fn group(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QFileInfo_group(self as *const crate::QFileInfo);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the id of the group the file belongs to.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QFileInfo::groupId() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#groupId">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the id of the group the file belongs to.</p>
    /// <p>On Windows and on systems where files do not have groups this function always returns (uint) -2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#group">group</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#owner">owner</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#ownerId">ownerId</a>().</p></div>
    pub unsafe fn group_id(&self) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QFileInfo_groupId(self as *const crate::QFileInfo)
    }

    /// <p>Returns <code>true</code> if the file path name is absolute, otherwise returns false if the path is relative.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFileInfo::isAbsolute() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#isAbsolute">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the file path name is absolute, otherwise returns false if the path is relative.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#isRelative">isRelative</a>().</p></div>
    pub unsafe fn is_absolute(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFileInfo_isAbsolute(self as *const crate::QFileInfo)
    }

    /// <p>Returns <code>true</code> if this object points to a bundle or to a symbolic link to a bundle on <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFileInfo::isBundle() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#isBundle">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this object points to a bundle or to a symbolic link to a bundle on <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#isDir">isDir</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#isSymLink">isSymLink</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#isFile">isFile</a>().</p></div>
    pub unsafe fn is_bundle(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFileInfo_isBundle(self as *const crate::QFileInfo)
    }

    /// <p>Returns <code>true</code> if this object points to a directory or to a symbolic link to a directory; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFileInfo::isDir() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#isDir">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this object points to a directory or to a symbolic link to a directory; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#isFile">isFile</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#isSymLink">isSymLink</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#isBundle">isBundle</a>().</p></div>
    pub unsafe fn is_dir(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFileInfo_isDir(self as *const crate::QFileInfo)
    }

    /// <p>Returns <code>true</code> if the file is executable; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFileInfo::isExecutable() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#isExecutable">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the file is executable; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#isReadable">isReadable</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#isWritable">isWritable</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#permission">permission</a>().</p></div>
    pub unsafe fn is_executable(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFileInfo_isExecutable(self as *const crate::QFileInfo)
    }

    /// <p>Returns <code>true</code> if this object points to a file or to a symbolic link to a file. Returns <code>false</code> if the object points to something which isn't a file, such as a directory.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFileInfo::isFile() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#isFile">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this object points to a file or to a symbolic link to a file. Returns <code>false</code> if the object points to something which isn't a file, such as a directory.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#isDir">isDir</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#isSymLink">isSymLink</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#isBundle">isBundle</a>().</p></div>
    pub unsafe fn is_file(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFileInfo_isFile(self as *const crate::QFileInfo)
    }

    /// <p>Returns <code>true</code> if this is a `hidden' file; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFileInfo::isHidden() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#isHidden">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this is a `hidden' file; otherwise returns <code>false</code>.</p>
    /// <p><b>Note:</b> This function returns <code>true</code> for the special entries "." and ".." on Unix, even though <a href="http://doc.qt.io/qt-5/qdir.html#entryList">QDir::entryList</a> threats them as shown.</p></div>
    pub unsafe fn is_hidden(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFileInfo_isHidden(self as *const crate::QFileInfo)
    }

    /// <p>Returns <code>true</code> if the file path can be used directly with native APIs. Returns <code>false</code> if the file is otherwise supported by a virtual file system inside Qt, such as <a href="http://doc.qt.io/qt-5/resources.html">the Qt Resource System</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFileInfo::isNativePath() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#isNativePath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the file path can be used directly with native APIs. Returns <code>false</code> if the file is otherwise supported by a virtual file system inside Qt, such as <a href="http://doc.qt.io/qt-5/resources.html">the Qt Resource System</a>.</p>
    /// <p><b>Note:</b> Native paths may still require conversion of path separators and character encoding, depending on platform and input requirements of the native API.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#toNativeSeparators">QDir::toNativeSeparators</a>(), <a href="http://doc.qt.io/qt-5/qfile.html#encodeName">QFile::encodeName</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#filePath">filePath</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#absoluteFilePath">absoluteFilePath</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#canonicalFilePath">canonicalFilePath</a>().</p></div>
    pub unsafe fn is_native_path(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFileInfo_isNativePath(self as *const crate::QFileInfo)
    }

    /// <p>Returns <code>true</code> if the user can read the file; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFileInfo::isReadable() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#isReadable">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the user can read the file; otherwise returns <code>false</code>.</p>
    /// <p><b>Note: </b>If the <a href="http://doc.qt.io/qt-5/qfileinfo.html#ntfs-permissions">NTFS permissions</a> check has not been enabled, the result on Windows will merely reflect whether the file exists.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#isWritable">isWritable</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#isExecutable">isExecutable</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#permission">permission</a>().</p></div>
    pub unsafe fn is_readable(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFileInfo_isReadable(self as *const crate::QFileInfo)
    }

    /// <p>Returns <code>true</code> if the file path name is relative, otherwise returns false if the path is absolute (e.g. under Unix a path is absolute if it begins with a "/").</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFileInfo::isRelative() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#isRelative">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the file path name is relative, otherwise returns false if the path is absolute (e.g. under Unix a path is absolute if it begins with a "/").</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#isAbsolute">isAbsolute</a>().</p></div>
    pub unsafe fn is_relative(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFileInfo_isRelative(self as *const crate::QFileInfo)
    }

    /// <p>Returns <code>true</code> if the object points to a directory or to a symbolic link to a directory, and that directory is the root directory; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFileInfo::isRoot() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#isRoot">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the object points to a directory or to a symbolic link to a directory, and that directory is the root directory; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn is_root(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFileInfo_isRoot(self as *const crate::QFileInfo)
    }

    /// <p>Returns <code>true</code> if this object points to a symbolic link; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFileInfo::isSymLink() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#isSymLink">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this object points to a symbolic link; otherwise returns <code>false</code>.</p>
    /// <p>Symbolic links exist on Unix (including <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS) and Windows and are typically created by the <code>ln -s</code> or <code>mklink</code> commands, respectively. Opening a symbolic link effectively opens the <a href="http://doc.qt.io/qt-5/qfileinfo.html#symLinkTarget">link's target</a>.</p>
    /// <p>In addition, true will be returned for shortcuts (<code>*.lnk</code> files) on Windows. Opening those will open the <code>.lnk</code> file itself.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qfileinfo.html#QFileInfo">QFileInfo</a></span> info(fileName);
    ///   <span class="keyword">if</span> (info<span class="operator">.</span>isSymLink())
    /// &#32;     fileName <span class="operator">=</span> info<span class="operator">.</span>symLinkTarget();
    ///
    /// </pre>
    /// <p><b>Note: </b>If the symlink points to a non existing file, <a href="http://doc.qt.io/qt-5/qfileinfo.html#exists">exists</a>() returns false.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#isFile">isFile</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#isDir">isDir</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#symLinkTarget">symLinkTarget</a>().</p></div>
    pub unsafe fn is_sym_link(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFileInfo_isSymLink(self as *const crate::QFileInfo)
    }

    /// <p>Returns <code>true</code> if the user can write to the file; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFileInfo::isWritable() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#isWritable">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the user can write to the file; otherwise returns <code>false</code>.</p>
    /// <p><b>Note: </b>If the <a href="http://doc.qt.io/qt-5/qfileinfo.html#ntfs-permissions">NTFS permissions</a> check has not been enabled, the result on Windows will merely reflect whether the file is marked as Read Only.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#isReadable">isReadable</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#isExecutable">isExecutable</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#permission">permission</a>().</p></div>
    pub unsafe fn is_writable(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFileInfo_isWritable(self as *const crate::QFileInfo)
    }

    /// <p>Returns the date and local time when the file was last modified.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDateTime QFileInfo::lastModified() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#lastModified">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the date and local time when the file was last modified.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#created">created</a>() and <a href="http://doc.qt.io/qt-5/qfileinfo.html#lastRead">lastRead</a>().</p></div>
    pub unsafe fn last_modified(&self) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QFileInfo_lastModified(self as *const crate::QFileInfo);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the date and local time when the file was last read (accessed).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDateTime QFileInfo::lastRead() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#lastRead">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the date and local time when the file was last read (accessed).</p>
    /// <p>On platforms where this information is not available, returns the same as <a href="http://doc.qt.io/qt-5/qfileinfo.html#lastModified">lastModified</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#created">created</a>() and <a href="http://doc.qt.io/qt-5/qfileinfo.html#lastModified">lastModified</a>().</p></div>
    pub unsafe fn last_read(&self) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QFileInfo_lastRead(self as *const crate::QFileInfo);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts the file's path to an absolute path if it is not already in that form. Returns <code>true</code> to indicate that the path was converted; otherwise returns <code>false</code> to indicate that the path was already absolute.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFileInfo::makeAbsolute()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#makeAbsolute">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the file's path to an absolute path if it is not already in that form. Returns <code>true</code> to indicate that the path was converted; otherwise returns <code>false</code> to indicate that the path was already absolute.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#filePath">filePath</a>() and <a href="http://doc.qt.io/qt-5/qfileinfo.html#isRelative">isRelative</a>().</p></div>
    pub unsafe fn make_absolute(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFileInfo_makeAbsolute(self as *mut crate::QFileInfo)
    }

    /// <p>Returns the date and time when the file metadata was changed. A metadata change occurs when the file is created, but it also occurs whenever the user writes or sets inode information (for example, changing the file permissions).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDateTime QFileInfo::metadataChangeTime() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#metadataChangeTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the date and time when the file metadata was changed. A metadata change occurs when the file is created, but it also occurs whenever the user writes or sets inode information (for example, changing the file permissions).</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#lastModified">lastModified</a>() and <a href="http://doc.qt.io/qt-5/qfileinfo.html#lastRead">lastRead</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn metadata_change_time(&self) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileInfo_metadataChangeTime(
            self as *const crate::QFileInfo,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs an empty <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QFileInfo::QFileInfo()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#QFileInfo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> object.</p>
    /// <p>Note that an empty <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> object contain no file reference.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#setFile">setFile</a>().</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileInfo_QFileInfo1();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> that gives information about the given file. The <i>file</i> can also include an absolute or relative path.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QFileInfo::QFileInfo(const QString& file)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#QFileInfo-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> that gives information about the given file. The <i>file</i> can also include an absolute or relative path.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#setFile">setFile</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#isRelative">isRelative</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setCurrent">QDir::setCurrent</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#isRelativePath">QDir::isRelativePath</a>().</p></div>
    pub unsafe fn from_q_string(
        file: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileInfo_QFileInfo2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> that gives information about file <i>file</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QFileInfo::QFileInfo(const QFile& file)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#QFileInfo-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> that gives information about file <i>file</i>.</p>
    /// <p>If the <i>file</i> has a relative path, the <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> will also have a relative path.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#isRelative">isRelative</a>().</p></div>
    pub unsafe fn from_q_file(
        file: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QFile>>,
    ) -> ::cpp_utils::CppBox<crate::QFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileInfo_QFileInfo3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QFile>>::cast_into(file).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> that gives information about the given <i>file</i> in the directory <i>dir</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QFileInfo::QFileInfo(const QDir& dir, const QString& file)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#QFileInfo-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> that gives information about the given <i>file</i> in the directory <i>dir</i>.</p>
    /// <p>If <i>dir</i> has a relative path, the <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> will also have a relative path.</p>
    /// <p>If <i>file</i> is an absolute path, then the directory specified by <i>dir</i> will be disregarded.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#isRelative">isRelative</a>().</p></div>
    pub unsafe fn from_q_dir_q_string(
        dir: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDir>>,
        file: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileInfo_QFileInfo4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDir>>::cast_into(dir).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> that is a copy of the given <i>fileinfo</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QFileInfo::QFileInfo(const QFileInfo& fileinfo)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#QFileInfo-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> that is a copy of the given <i>fileinfo</i>.</p></div>
    pub unsafe fn new_copy(
        fileinfo: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QFileInfo>>,
    ) -> ::cpp_utils::CppBox<crate::QFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileInfo_QFileInfo5(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QFileInfo>>::cast_into(fileinfo)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the owner of the file. On systems where files do not have owners, or if an error occurs, an empty string is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QFileInfo::owner() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#owner">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the owner of the file. On systems where files do not have owners, or if an error occurs, an empty string is returned.</p>
    /// <p>This function can be time consuming under Unix (in the order of milliseconds). On Windows, it will return an empty string unless the <a href="http://doc.qt.io/qt-5/qfileinfo.html#ntfs-permissions">NTFS permissions</a> check has been enabled.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#ownerId">ownerId</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#group">group</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#groupId">groupId</a>().</p></div>
    pub unsafe fn owner(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QFileInfo_owner(self as *const crate::QFileInfo);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the id of the owner of the file.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QFileInfo::ownerId() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#ownerId">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the id of the owner of the file.</p>
    /// <p>On Windows and on systems where files do not have owners this function returns ((uint) -2).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#owner">owner</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#group">group</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#groupId">groupId</a>().</p></div>
    pub unsafe fn owner_id(&self) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QFileInfo_ownerId(self as *const crate::QFileInfo)
    }

    /// <p>Returns the file's path. This doesn't include the file name.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QFileInfo::path() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#path">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the file's path. This doesn't include the file name.</p>
    /// <p>Note that, if this <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> object is given a path ending in a slash, the name of the file is considered empty and this function will return the entire path.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#filePath">filePath</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#absolutePath">absolutePath</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#canonicalPath">canonicalPath</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#dir">dir</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#fileName">fileName</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#isRelative">isRelative</a>().</p></div>
    pub unsafe fn path(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QFileInfo_path(self as *const crate::QFileInfo);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Tests for file permissions. The <i>permissions</i> argument can be several flags of type <a href="http://doc.qt.io/qt-5/qfiledevice.html#Permission-enum">QFile::Permissions</a> OR-ed together to check for permission combinations.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFileInfo::permission(QFlags<QFileDevice::Permission> permissions) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#permission">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Tests for file permissions. The <i>permissions</i> argument can be several flags of type <a href="http://doc.qt.io/qt-5/qfiledevice.html#Permission-enum">QFile::Permissions</a> OR-ed together to check for permission combinations.</p>
    /// <p>On systems where files do not have permissions this function always returns <code>true</code>.</p>
    /// <p><b>Note: </b>The result might be inaccurate on Windows if the <a href="http://doc.qt.io/qt-5/qfileinfo.html#ntfs-permissions">NTFS permissions</a> check has not been enabled.</p><p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qfileinfo.html#QFileInfo">QFileInfo</a></span> fi(<span class="string">"/tmp/archive.tar.gz"</span>);
    ///   <span class="keyword">if</span> (fi<span class="operator">.</span>permission(<span class="type"><a href="http://doc.qt.io/qt-5/qfile.html">QFile</a></span><span class="operator">::</span>WriteUser <span class="operator">|</span> <span class="type"><a href="http://doc.qt.io/qt-5/qfile.html">QFile</a></span><span class="operator">::</span>ReadGroup))
    /// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qWarning">qWarning</a>(<span class="string">"I can change the file; my group can read the file"</span>);
    ///   <span class="keyword">if</span> (fi<span class="operator">.</span>permission(<span class="type"><a href="http://doc.qt.io/qt-5/qfile.html">QFile</a></span><span class="operator">::</span>WriteGroup <span class="operator">|</span> <span class="type"><a href="http://doc.qt.io/qt-5/qfile.html">QFile</a></span><span class="operator">::</span>WriteOther))
    /// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qWarning">qWarning</a>(<span class="string">"The group or others can change the file"</span>);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#isReadable">isReadable</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#isWritable">isWritable</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#isExecutable">isExecutable</a>().</p></div>
    pub unsafe fn permission(
        &self,
        permissions: crate::QFlags<crate::q_file_device::Permission>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFileInfo_permission(
            self as *const crate::QFileInfo,
            permissions.to_int(),
        )
    }

    /// <p>Returns the complete OR-ed together combination of <a href="http://doc.qt.io/qt-5/qfiledevice.html#Permission-enum">QFile::Permissions</a> for the file.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFlags<QFileDevice::Permission> QFileInfo::permissions() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#permissions">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the complete OR-ed together combination of <a href="http://doc.qt.io/qt-5/qfiledevice.html#Permission-enum">QFile::Permissions</a> for the file.</p>
    /// <p><b>Note: </b>The result might be inaccurate on Windows if the <a href="http://doc.qt.io/qt-5/qfileinfo.html#ntfs-permissions">NTFS permissions</a> check has not been enabled.</p></div>
    pub unsafe fn permissions(&self) -> crate::QFlags<crate::q_file_device::Permission> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QFileInfo_permissions(self as *const crate::QFileInfo);
        crate::QFlags::from(ffi_result)
    }

    /// <p>Use <a href="http://doc.qt.io/qt-5/qfileinfo.html#symLinkTarget">symLinkTarget</a>() instead.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QFileInfo::readLink() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo-obsolete.html#readLink">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Use <a href="http://doc.qt.io/qt-5/qfileinfo.html#symLinkTarget">symLinkTarget</a>() instead.</p></div>
    pub unsafe fn read_link(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QFileInfo_readLink(self as *const crate::QFileInfo);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Refreshes the information about the file, i.e. reads in information from the file system the next time a cached property is fetched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QFileInfo::refresh()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#refresh">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Refreshes the information about the file, i.e. reads in information from the file system the next time a cached property is fetched.</p></div>
    pub unsafe fn refresh(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QFileInfo_refresh(self as *mut crate::QFileInfo)
    }

    /// <p>If <i>enable</i> is true, enables caching of file information. If <i>enable</i> is false caching is disabled.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QFileInfo::setCaching(bool on)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#setCaching">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If <i>enable</i> is true, enables caching of file information. If <i>enable</i> is false caching is disabled.</p>
    /// <p>When caching is enabled, <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> reads the file information from the file system the first time it's needed, but generally not later.</p>
    /// <p>Caching is enabled by default.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#refresh">refresh</a>() and <a href="http://doc.qt.io/qt-5/qfileinfo.html#caching">caching</a>().</p></div>
    pub unsafe fn set_caching(&mut self, on: bool) {
        crate::__ffi::ctr_qt_core_ffi_QFileInfo_setCaching(self as *mut crate::QFileInfo, on)
    }

    /// <p>Sets the file that the <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> provides information about to <i>file</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QFileInfo::setFile(const QString& file)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#setFile">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the file that the <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> provides information about to <i>file</i>.</p>
    /// <p>The <i>file</i> can also include an absolute or relative file path. Absolute paths begin with the directory separator (e.g. "/" under Unix) or a drive specification (under Windows). Relative file names begin with a directory name or a file name and specify a path relative to the current directory.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> absolute <span class="operator">=</span> <span class="string">"/local/bin"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> relative <span class="operator">=</span> <span class="string">"local/bin"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qfileinfo.html#QFileInfo">QFileInfo</a></span> absFile(absolute);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qfileinfo.html#QFileInfo">QFileInfo</a></span> relFile(relative);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdir.html">QDir</a></span><span class="operator">::</span>setCurrent(<span class="type"><a href="http://doc.qt.io/qt-5/qdir.html">QDir</a></span><span class="operator">::</span>rootPath());
    ///   <span class="comment">// absFile and relFile now point to the same file</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdir.html">QDir</a></span><span class="operator">::</span>setCurrent(<span class="string">"/tmp"</span>);
    ///   <span class="comment">// absFile now points to "/local/bin",</span>
    ///   <span class="comment">// while relFile points to "/tmp/local/bin"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#isFile">isFile</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#isRelative">isRelative</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setCurrent">QDir::setCurrent</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#isRelativePath">QDir::isRelativePath</a>().</p></div>
    pub unsafe fn set_file_q_string(
        &mut self,
        file: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QFileInfo_setFile(
            self as *mut crate::QFileInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QFileInfo::setFile(const QFile& file)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#setFile-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Sets the file that the <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> provides information about to <i>file</i>.</p>
    /// <p>If <i>file</i> includes a relative path, the <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> will also have a relative path.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#isRelative">isRelative</a>().</p></div>
    pub unsafe fn set_file_q_file(
        &mut self,
        file: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QFile>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QFileInfo_setFile1(
            self as *mut crate::QFileInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QFile>>::cast_into(file).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QFileInfo::setFile(const QDir& dir, const QString& file)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#setFile-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Sets the file that the <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> provides information about to <i>file</i> in directory <i>dir</i>.</p>
    /// <p>If <i>file</i> includes a relative path, the <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> will also have a relative path.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#isRelative">isRelative</a>().</p></div>
    pub unsafe fn set_file_q_dir_q_string(
        &mut self,
        dir: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDir>>,
        file: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QFileInfo_setFile2(
            self as *mut crate::QFileInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDir>>::cast_into(dir).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file).as_raw_ptr(),
        )
    }

    /// <p>Returns the file size in bytes. If the file does not exist or cannot be fetched, 0 is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QFileInfo::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the file size in bytes. If the file does not exist or cannot be fetched, 0 is returned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#exists">exists</a>().</p></div>
    pub unsafe fn size(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QFileInfo_size(self as *const crate::QFileInfo)
    }

    /// <p>Returns the suffix (extension) of the file.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QFileInfo::suffix() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#suffix">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the suffix (extension) of the file.</p>
    /// <p>The suffix consists of all characters in the file after (but not including) the last '.'.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qfileinfo.html#QFileInfo">QFileInfo</a></span> fi(<span class="string">"/tmp/archive.tar.gz"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> ext <span class="operator">=</span> fi<span class="operator">.</span>suffix();  <span class="comment">// ext = "gz"</span>
    ///
    /// </pre>
    /// <p>The suffix of a file is computed equally on all platforms, independent of file naming conventions (e.g., ".bashrc" on Unix has an empty base name, and the suffix is "bashrc").</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#fileName">fileName</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#completeSuffix">completeSuffix</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#baseName">baseName</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#completeBaseName">completeBaseName</a>().</p></div>
    pub unsafe fn suffix(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QFileInfo_suffix(self as *const crate::QFileInfo);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Swaps this file info with <i>other</i>. This function is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QFileInfo::swap(QFileInfo& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps this file info with <i>other</i>. This function is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QFileInfo>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QFileInfo_swap(
            self as *mut crate::QFileInfo,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QFileInfo>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the absolute path to the file or directory a symbolic link points to, or an empty string if the object isn't a symbolic link.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QFileInfo::symLinkTarget() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#symLinkTarget">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the absolute path to the file or directory a symbolic link points to, or an empty string if the object isn't a symbolic link.</p>
    /// <p>This name may not represent an existing file; it is only a string. <a href="http://doc.qt.io/qt-5/qfileinfo.html#exists">QFileInfo::exists</a>() returns <code>true</code> if the symlink points to an existing file.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#exists">exists</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#isSymLink">isSymLink</a>(), <a href="http://doc.qt.io/qt-5/qfileinfo.html#isDir">isDir</a>(), and <a href="http://doc.qt.io/qt-5/qfileinfo.html#isFile">isFile</a>().</p></div>
    pub unsafe fn sym_link_target(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QFileInfo_symLinkTarget(self as *const crate::QFileInfo);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_dir {
    //! C++ type: <span style='color: green;'>```QDir```</span>

    /// <p>This enum describes the filtering options available to <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a>; e.g. for <a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList">entryInfoList</a>(). The filter value is specified by combining values from the following list using the bitwise OR operator:</p>
    ///
    /// C++ enum: <span style='color: green;'>```QDir::Filter```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#Filter-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the filtering options available to <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a>; e.g. for <a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList">entryInfoList</a>(). The filter value is specified by combining values from the following list using the bitwise OR operator:</p>
    ///
    /// <p>Functions that use Filter enum values to filter lists of files and directories will include symbolic links to files and directories unless you set the NoSymLinks value.</p>
    /// <p>A default constructed <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> will not filter out files based on their permissions, so <a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList">entryInfoList</a>() will return all files that are readable, writable, executable, or any combination of the three. This makes the default easy to write, and at the same time useful.</p>
    /// <p>For example, setting the <code>Readable</code>, <code>Writable</code>, and <code>Files</code> flags allows all files to be listed for which the application has read access, write access or both. If the <code>Dirs</code> and <code>Drives</code> flags are also included in this combination then all drives, directories, all files that the application can read, write, or execute, and symlinks to such files/directories can be listed.</p>
    /// <p>To retrieve the permissons for a directory, use the <a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList">entryInfoList</a>() function to get the associated <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> objects and then use the QFileInfo::permissons() to obtain the permissions and ownership for each file.</p>
    /// <p>The Filters type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;Filter&gt;. It stores an OR combination of Filter values.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Filter(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Filter {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Filter(value)
        }
    }

    impl From<Filter> for ::std::os::raw::c_int {
        fn from(value: Filter) -> Self {
            value.0
        }
    }

    impl Filter {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Filter {
        /// List directories that match the filters. (C++ enum variant: <span style='color: green;'>```Dirs = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Dirs: crate::q_dir::Filter = crate::q_dir::Filter(1);
        /// List files. (C++ enum variant: <span style='color: green;'>```Files = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const Files: crate::q_dir::Filter = crate::q_dir::Filter(2);
        /// List disk drives (ignored under Unix). (C++ enum variant: <span style='color: green;'>```Drives = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const Drives: crate::q_dir::Filter = crate::q_dir::Filter(4);
        /// Do not list symbolic links (ignored by operating systems that don't support symbolic links). (C++ enum variant: <span style='color: green;'>```NoSymLinks = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoSymLinks: crate::q_dir::Filter = crate::q_dir::Filter(8);
        /// List directories, files, drives and symlinks (this does not list broken symlinks unless you specify System). (C++ enum variant: <span style='color: green;'>```AllEntries = 7```</span>)
        #[allow(non_upper_case_globals)]
        pub const AllEntries: crate::q_dir::Filter = crate::q_dir::Filter(7);
        /// C++ enum variant: <span style='color: green;'>```TypeMask = 15```</span>
        #[allow(non_upper_case_globals)]
        pub const TypeMask: crate::q_dir::Filter = crate::q_dir::Filter(15);
        /// List files for which the application has read access. The Readable value needs to be combined with Dirs or Files. (C++ enum variant: <span style='color: green;'>```Readable = 16```</span>)
        #[allow(non_upper_case_globals)]
        pub const Readable: crate::q_dir::Filter = crate::q_dir::Filter(16);
        /// List files for which the application has write access. The Writable value needs to be combined with Dirs or Files. (C++ enum variant: <span style='color: green;'>```Writable = 32```</span>)
        #[allow(non_upper_case_globals)]
        pub const Writable: crate::q_dir::Filter = crate::q_dir::Filter(32);
        /// List files for which the application has execute access. The Executable value needs to be combined with Dirs or Files. (C++ enum variant: <span style='color: green;'>```Executable = 64```</span>)
        #[allow(non_upper_case_globals)]
        pub const Executable: crate::q_dir::Filter = crate::q_dir::Filter(64);
        /// C++ enum variant: <span style='color: green;'>```PermissionMask = 112```</span>
        #[allow(non_upper_case_globals)]
        pub const PermissionMask: crate::q_dir::Filter = crate::q_dir::Filter(112);
        /// Only list files that have been modified (ignored on Unix). (C++ enum variant: <span style='color: green;'>```Modified = 128```</span>)
        #[allow(non_upper_case_globals)]
        pub const Modified: crate::q_dir::Filter = crate::q_dir::Filter(128);
        /// List hidden files (on Unix, files starting with a "."). (C++ enum variant: <span style='color: green;'>```Hidden = 256```</span>)
        #[allow(non_upper_case_globals)]
        pub const Hidden: crate::q_dir::Filter = crate::q_dir::Filter(256);
        /// List system files (on Unix, FIFOs, sockets and device files are included; on Windows, <code>.lnk</code> files are included) (C++ enum variant: <span style='color: green;'>```System = 512```</span>)
        #[allow(non_upper_case_globals)]
        pub const System: crate::q_dir::Filter = crate::q_dir::Filter(512);
        /// C++ enum variant: <span style='color: green;'>```AccessMask = 1008```</span>
        #[allow(non_upper_case_globals)]
        pub const AccessMask: crate::q_dir::Filter = crate::q_dir::Filter(1008);
        /// List all directories; i.e. don't apply the filters to directory names. (C++ enum variant: <span style='color: green;'>```AllDirs = 1024```</span>)
        #[allow(non_upper_case_globals)]
        pub const AllDirs: crate::q_dir::Filter = crate::q_dir::Filter(1024);
        /// The filter should be case sensitive. (C++ enum variant: <span style='color: green;'>```CaseSensitive = 2048```</span>)
        #[allow(non_upper_case_globals)]
        pub const CaseSensitive: crate::q_dir::Filter = crate::q_dir::Filter(2048);
        /// Do not list the special entry ".". (C++ enum variant: <span style='color: green;'>```NoDot = 8192```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoDot: crate::q_dir::Filter = crate::q_dir::Filter(8192);
        /// Do not list the special entry "..". (C++ enum variant: <span style='color: green;'>```NoDotDot = 16384```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoDotDot: crate::q_dir::Filter = crate::q_dir::Filter(16384);
        /// Do not list the special entries "." and "..". (C++ enum variant: <span style='color: green;'>```NoDotAndDotDot = 24576```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoDotAndDotDot: crate::q_dir::Filter = crate::q_dir::Filter(24576);
        /// C++ enum variant: <span style='color: green;'>```NoFilter = -1```</span>
        #[allow(non_upper_case_globals)]
        pub const NoFilter: crate::q_dir::Filter = crate::q_dir::Filter(-1);
    }

    /// <p>This enum describes the sort options available to <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a>, e.g. for <a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList">entryInfoList</a>(). The sort value is specified by OR-ing together values from the following list:</p>
    ///
    /// C++ enum: <span style='color: green;'>```QDir::SortFlag```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#SortFlag-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the sort options available to <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a>, e.g. for <a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList">entryInfoList</a>(). The sort value is specified by OR-ing together values from the following list:</p>
    ///
    /// <p>You can only specify one of the first four.</p>
    /// <p>If you specify both DirsFirst and Reversed, directories are still put first, but in reverse order; the files will be listed after the directories, again in reverse order.</p>
    /// <p>The SortFlags type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;SortFlag&gt;. It stores an OR combination of SortFlag values.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct SortFlag(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for SortFlag {
        fn from(value: ::std::os::raw::c_int) -> Self {
            SortFlag(value)
        }
    }

    impl From<SortFlag> for ::std::os::raw::c_int {
        fn from(value: SortFlag) -> Self {
            value.0
        }
    }

    impl SortFlag {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl SortFlag {
        /// Sort by name. (C++ enum variant: <span style='color: green;'>```Name = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const Name: crate::q_dir::SortFlag = crate::q_dir::SortFlag(0);
        /// Sort by time (modification time). (C++ enum variant: <span style='color: green;'>```Time = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Time: crate::q_dir::SortFlag = crate::q_dir::SortFlag(1);
        /// Sort by file size. (C++ enum variant: <span style='color: green;'>```Size = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const Size: crate::q_dir::SortFlag = crate::q_dir::SortFlag(2);
        /// Do not sort. (C++ enum variant: <span style='color: green;'>```Unsorted = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const Unsorted: crate::q_dir::SortFlag = crate::q_dir::SortFlag(3);
        /// C++ enum variant: <span style='color: green;'>```SortByMask = 3```</span>
        #[allow(non_upper_case_globals)]
        pub const SortByMask: crate::q_dir::SortFlag = crate::q_dir::SortFlag(3);
        /// Put the directories first, then the files. (C++ enum variant: <span style='color: green;'>```DirsFirst = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const DirsFirst: crate::q_dir::SortFlag = crate::q_dir::SortFlag(4);
        /// Reverse the sort order. (C++ enum variant: <span style='color: green;'>```Reversed = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const Reversed: crate::q_dir::SortFlag = crate::q_dir::SortFlag(8);
        /// Sort case-insensitively. (C++ enum variant: <span style='color: green;'>```IgnoreCase = 16```</span>)
        #[allow(non_upper_case_globals)]
        pub const IgnoreCase: crate::q_dir::SortFlag = crate::q_dir::SortFlag(16);
        /// Put the files first, then the directories. (C++ enum variant: <span style='color: green;'>```DirsLast = 32```</span>)
        #[allow(non_upper_case_globals)]
        pub const DirsLast: crate::q_dir::SortFlag = crate::q_dir::SortFlag(32);
        /// Sort items appropriately using the current locale settings. (C++ enum variant: <span style='color: green;'>```LocaleAware = 64```</span>)
        #[allow(non_upper_case_globals)]
        pub const LocaleAware: crate::q_dir::SortFlag = crate::q_dir::SortFlag(64);
        /// Sort by file type (extension). (C++ enum variant: <span style='color: green;'>```Type = 128```</span>)
        #[allow(non_upper_case_globals)]
        pub const Type: crate::q_dir::SortFlag = crate::q_dir::SortFlag(128);
        /// Not sorted by default. (C++ enum variant: <span style='color: green;'>```NoSort = -1```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoSort: crate::q_dir::SortFlag = crate::q_dir::SortFlag(-1);
    }

    impl From<crate::q_dir::SortFlag> for crate::QFlags<crate::q_dir::SortFlag> {
        fn from(value: crate::q_dir::SortFlag) -> Self {
            Self::from(value.to_int())
        }
    }

    impl<T: Into<crate::QFlags<crate::q_dir::SortFlag>>> std::ops::BitOr<T> for crate::q_dir::SortFlag {
        type Output = crate::QFlags<crate::q_dir::SortFlag>;
        fn bitor(self, rhs: T) -> crate::QFlags<crate::q_dir::SortFlag> {
            Into::<crate::QFlags<crate::q_dir::SortFlag>>::into(self) | rhs
        }
    }

    impl From<crate::q_dir::Filter> for crate::QFlags<crate::q_dir::Filter> {
        fn from(value: crate::q_dir::Filter) -> Self {
            Self::from(value.to_int())
        }
    }

    impl<T: Into<crate::QFlags<crate::q_dir::Filter>>> std::ops::BitOr<T> for crate::q_dir::Filter {
        type Output = crate::QFlags<crate::q_dir::Filter>;
        fn bitor(self, rhs: T) -> crate::QFlags<crate::q_dir::Filter> {
            Into::<crate::QFlags<crate::q_dir::Filter>>::into(self) | rhs
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> class provides access to directory structures and their contents.</p>
///
/// C++ class: <span style='color: green;'>```QDir```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qdir.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> class provides access to directory structures and their contents.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> is used to manipulate path names, access information regarding paths and files, and manipulate the underlying file system. It can also be used to access Qt's <a href="http://doc.qt.io/qt-5/resources.html">resource system</a>.</p>
/// <p>Qt uses "/" as a universal directory separator in the same way that "/" is used as a path separator in URLs. If you always use "/" as a directory separator, Qt will translate your paths to conform to the underlying operating system.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> can point to a file using either a relative or an absolute path. Absolute paths begin with the directory separator (optionally preceded by a drive specification under Windows). Relative file names begin with a directory name or a file name and specify a path relative to the current directory.</p>
/// <p>Examples of absolute paths:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qdir.html#QDir">QDir</a></span>(<span class="string">"/home/user/Documents"</span>)
///   <span class="type"><a href="http://doc.qt.io/qt-5/qdir.html#QDir">QDir</a></span>(<span class="string">"C:/Documents and Settings"</span>)
///
/// </pre>
/// <p>On Windows, the second example above will be translated to <code>C:\Documents and Settings</code> when used to access files.</p>
/// <p>Examples of relative paths:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qdir.html#QDir">QDir</a></span>(<span class="string">"images/landscape.png"</span>)
///
/// </pre>
/// <p>You can use the <a href="http://doc.qt.io/qt-5/qdir.html#isRelative">isRelative</a>() or <a href="http://doc.qt.io/qt-5/qdir.html#isAbsolute">isAbsolute</a>() functions to check if a <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> is using a relative or an absolute file path. Call <a href="http://doc.qt.io/qt-5/qdir.html#makeAbsolute">makeAbsolute</a>() to convert a relative <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> to an absolute one.</p>
/// <a name="navigation-and-directory-operations"></a></div>
#[repr(C)]
pub struct QDir {
    _unused: u8,
}
impl QDir {
    /// <p>Returns the absolute path name of a file in the directory. Does <i>not</i> check if the file actually exists in the directory; but see <a href="http://doc.qt.io/qt-5/qdir.html#exists-1">exists</a>(). Redundant multiple separators or "." and ".." directories in <i>fileName</i> are not removed (see <a href="http://doc.qt.io/qt-5/qdir.html#cleanPath">cleanPath</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QDir::absoluteFilePath(const QString& fileName) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#absoluteFilePath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the absolute path name of a file in the directory. Does <i>not</i> check if the file actually exists in the directory; but see <a href="http://doc.qt.io/qt-5/qdir.html#exists-1">exists</a>(). Redundant multiple separators or "." and ".." directories in <i>fileName</i> are not removed (see <a href="http://doc.qt.io/qt-5/qdir.html#cleanPath">cleanPath</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#relativeFilePath">relativeFilePath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#filePath">filePath</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#canonicalPath">canonicalPath</a>().</p></div>
    pub unsafe fn absolute_file_path(
        &self,
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_absoluteFilePath(
            self as *const crate::QDir,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the absolute path (a path that starts with "/" or with a drive specification), which may contain symbolic links, but never contains redundant ".", ".." or multiple separators.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QDir::absolutePath() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#absolutePath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the absolute path (a path that starts with "/" or with a drive specification), which may contain symbolic links, but never contains redundant ".", ".." or multiple separators.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#setPath">setPath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#canonicalPath">canonicalPath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#exists-1">exists</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#cleanPath">cleanPath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#dirName">dirName</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#absoluteFilePath">absoluteFilePath</a>().</p></div>
    pub unsafe fn absolute_path(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDir_absolutePath(self as *const crate::QDir);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Use <a href="http://doc.qt.io/qt-5/qdir.html#addSearchPath">QDir::addSearchPath</a>() with a prefix instead.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QDir::addResourceSearchPath(const QString& path)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir-obsolete.html#addResourceSearchPath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Use <a href="http://doc.qt.io/qt-5/qdir.html#addSearchPath">QDir::addSearchPath</a>() with a prefix instead.</p>
    /// <p>Adds <i>path</i> to the search paths searched in to find resources that are not specified with an absolute path. The default search path is to search only in the root (<code>:/</code>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/resources.html">The Qt Resource System</a>.</p></div>
    pub unsafe fn add_resource_search_path(
        path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QDir_addResourceSearchPath(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(path).as_raw_ptr(),
        )
    }

    /// <p>Adds <i>path</i> to the search path for <i>prefix</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QDir::addSearchPath(const QString& prefix, const QString& path)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#addSearchPath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds <i>path</i> to the search path for <i>prefix</i>.</p>
    /// <p>This function was introduced in  Qt 4.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#setSearchPaths">setSearchPaths</a>().</p></div>
    pub unsafe fn add_search_path(
        prefix: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QDir_addSearchPath(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(prefix)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(path).as_raw_ptr(),
        )
    }

    /// <p>Returns the canonical path, i.e. a path without symbolic links or redundant "." or ".." elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QDir::canonicalPath() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#canonicalPath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the canonical path, i.e. a path without symbolic links or redundant "." or ".." elements.</p>
    /// <p>On systems that do not have symbolic links this function will always return the same string that <a href="http://doc.qt.io/qt-5/qdir.html#absolutePath">absolutePath</a>() returns. If the canonical path does not exist (normally due to dangling symbolic links) canonicalPath() returns an empty string.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> bin <span class="operator">=</span> <span class="string">"/local/bin"</span>;         <span class="comment">// where /local/bin is a symlink to /usr/bin</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdir.html#QDir">QDir</a></span> binDir(bin);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> canonicalBin <span class="operator">=</span> binDir<span class="operator">.</span>canonicalPath();
    ///   <span class="comment">// canonicalBin now equals "/usr/bin"</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> ls <span class="operator">=</span> <span class="string">"/local/bin/ls"</span>;       <span class="comment">// where ls is the executable "ls"</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdir.html#QDir">QDir</a></span> lsDir(ls);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> canonicalLs <span class="operator">=</span> lsDir<span class="operator">.</span>canonicalPath();
    ///   <span class="comment">// canonicalLS now equals "/usr/bin/ls".</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#path">path</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#absolutePath">absolutePath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#exists-1">exists</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#cleanPath">cleanPath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#dirName">dirName</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#absoluteFilePath">absoluteFilePath</a>().</p></div>
    pub unsafe fn canonical_path(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDir_canonicalPath(self as *const crate::QDir);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Changes the <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a>'s directory to <i>dirName</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDir::cd(const QString& dirName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#cd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Changes the <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a>'s directory to <i>dirName</i>.</p>
    /// <p>Returns <code>true</code> if the new directory exists; otherwise returns <code>false</code>. Note that the logical cd() operation is not performed if the new directory does not exist.</p>
    /// <p>Calling cd("..") is equivalent to calling <a href="http://doc.qt.io/qt-5/qdir.html#cdUp">cdUp</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#cdUp">cdUp</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#isReadable">isReadable</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#exists-1">exists</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#path">path</a>().</p></div>
    pub unsafe fn cd(
        &mut self,
        dir_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDir_cd(
            self as *mut crate::QDir,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(dir_name)
                .as_raw_ptr(),
        )
    }

    /// <p>Changes directory by moving one directory up from the <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a>'s current directory.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDir::cdUp()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#cdUp">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Changes directory by moving one directory up from the <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a>'s current directory.</p>
    /// <p>Returns <code>true</code> if the new directory exists; otherwise returns <code>false</code>. Note that the logical cdUp() operation is not performed if the new directory does not exist.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#cd">cd</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#isReadable">isReadable</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#exists-1">exists</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#path">path</a>().</p></div>
    pub unsafe fn cd_up(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDir_cdUp(self as *mut crate::QDir)
    }

    /// <p>Returns <i>path</i> with directory separators normalized (converted to "/") and redundant ones removed, and "."s and ".."s resolved (as far as possible).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QDir::cleanPath(const QString& path)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#cleanPath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <i>path</i> with directory separators normalized (converted to "/") and redundant ones removed, and "."s and ".."s resolved (as far as possible).</p>
    /// <p>Symbolic links are kept. This function does not return the canonical path, but rather the simplest version of the input. For example, "./local" becomes "local", "local/../bin" becomes "bin" and "/local/usr/../bin" becomes "/local/bin".</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#absolutePath">absolutePath</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#canonicalPath">canonicalPath</a>().</p></div>
    pub unsafe fn clean_path(
        path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_cleanPath(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(path).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Makes a copy of the <i>dir</i> object and assigns it to this <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDir& QDir::operator=(const QDir& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes a copy of the <i>dir</i> object and assigns it to this <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> object.</p></div>
    pub unsafe fn copy_from_q_dir(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDir>>,
    ) -> ::cpp_utils::MutRef<crate::QDir> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_operator_(
            self as *mut crate::QDir,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDir>>::cast_into(arg1).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```QDir& QDir::operator=(const QString& path)```</span>.
    pub unsafe fn copy_from_q_string(
        &mut self,
        path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::MutRef<crate::QDir> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_operator_1(
            self as *mut crate::QDir,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(path).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the total number of directories and files in the directory.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QDir::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the total number of directories and files in the directory.</p>
    /// <p>Equivalent to <a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>().count().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>().</p></div>
    pub unsafe fn count(&self) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QDir_count(self as *const crate::QDir)
    }

    /// <p>Returns the application's current directory.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDir QDir::current()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#current">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the application's current directory.</p>
    /// <p>The directory is constructed using the absolute path of the current directory, ensuring that its <a href="http://doc.qt.io/qt-5/qdir.html#path">path</a>() will be the same as its <a href="http://doc.qt.io/qt-5/qdir.html#absolutePath">absolutePath</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#currentPath">currentPath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setCurrent">setCurrent</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#home">home</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#root">root</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#temp">temp</a>().</p></div>
    pub unsafe fn current() -> ::cpp_utils::CppBox<crate::QDir> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_current();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the absolute path of the application's current directory. The current directory is the last directory set with <a href="http://doc.qt.io/qt-5/qdir.html#setCurrent">QDir::setCurrent</a>() or, if that was never called, the directory at which this application was started at by the parent process.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QDir::currentPath()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#currentPath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the absolute path of the application's current directory. The current directory is the last directory set with <a href="http://doc.qt.io/qt-5/qdir.html#setCurrent">QDir::setCurrent</a>() or, if that was never called, the directory at which this application was started at by the parent process.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#current">current</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setCurrent">setCurrent</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#homePath">homePath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#rootPath">rootPath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#tempPath">tempPath</a>(), and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationDirPath">QCoreApplication::applicationDirPath</a>().</p></div>
    pub unsafe fn current_path() -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_currentPath();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the name of the directory; this is <i>not</i> the same as the path, e.g. a directory with the name "mail", might have the path "/var/spool/mail". If the directory has no name (e.g. it is the root directory) an empty string is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QDir::dirName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#dirName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the name of the directory; this is <i>not</i> the same as the path, e.g. a directory with the name "mail", might have the path "/var/spool/mail". If the directory has no name (e.g. it is the root directory) an empty string is returned.</p>
    /// <p>No check is made to ensure that a directory with this name actually exists; but see <a href="http://doc.qt.io/qt-5/qdir.html#exists-1">exists</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#path">path</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#filePath">filePath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#absolutePath">absolutePath</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#absoluteFilePath">absoluteFilePath</a>().</p></div>
    pub unsafe fn dir_name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_dirName(self as *const crate::QDir);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list of the root directories on this system.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QList<QFileInfo> QDir::drives()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#drives">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of the root directories on this system.</p>
    /// <p>On Windows this returns a list of <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> objects containing "C:/", "D:/", etc. On other operating systems, it returns a list containing just one root directory (i.e. "/").</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#root">root</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#rootPath">rootPath</a>().</p></div>
    pub unsafe fn drives() -> ::cpp_utils::CppBox<crate::QListOfQFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_drives();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo> QDir::entryInfoList(QFlags<QDir::Filter> filters = …, QFlags<QDir::SortFlag> sort = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a list of <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> objects for all the files and directories in the directory, ordered according to the name and attribute filters previously set with <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>(), and sorted according to the flags set with <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>().</p>
    /// <p>The attribute filter and sorting specifications can be overridden using the <i>filters</i> and <i>sort</i> arguments.</p>
    /// <p>Returns an empty list if the directory is unreadable, does not exist, or if nothing matches the specification.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#isReadable">isReadable</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#exists-1">exists</a>().</p></div>
    pub unsafe fn entry_info_list_q_flags_filter_q_flags_sort_flag(
        &self,
        filters: crate::QFlags<crate::q_dir::Filter>,
        sort: crate::QFlags<crate::q_dir::SortFlag>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_entryInfoList(
            self as *const crate::QDir,
            filters.to_int(),
            sort.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list of <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> objects for all the files and directories in the directory, ordered according to the name and attribute filters previously set with <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>(), and sorted according to the flags set with <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo> QDir::entryInfoList(const QStringList& nameFilters, QFlags<QDir::Filter> filters = …, QFlags<QDir::SortFlag> sort = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> objects for all the files and directories in the directory, ordered according to the name and attribute filters previously set with <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>(), and sorted according to the flags set with <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>().</p>
    /// <p>The name filter, file attribute filter, and sorting specification can be overridden using the <i>nameFilters</i>, <i>filters</i>, and <i>sort</i> arguments.</p>
    /// <p>Returns an empty list if the directory is unreadable, does not exist, or if nothing matches the specification.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#isReadable">isReadable</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#exists-1">exists</a>().</p></div>
    pub unsafe fn entry_info_list_q_string_list_q_flags_filter_q_flags_sort_flag(
        &self,
        name_filters: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
        filters: crate::QFlags<crate::q_dir::Filter>,
        sort: crate::QFlags<crate::q_dir::SortFlag>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_entryInfoList1(
            self as *const crate::QDir,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(name_filters)
                .as_raw_ptr(),
            filters.to_int(),
            sort.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo> QDir::entryInfoList(QFlags<QDir::Filter> filters = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a list of <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> objects for all the files and directories in the directory, ordered according to the name and attribute filters previously set with <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>(), and sorted according to the flags set with <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>().</p>
    /// <p>The attribute filter and sorting specifications can be overridden using the <i>filters</i> and <i>sort</i> arguments.</p>
    /// <p>Returns an empty list if the directory is unreadable, does not exist, or if nothing matches the specification.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#isReadable">isReadable</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#exists-1">exists</a>().</p></div>
    pub unsafe fn entry_info_list_q_flags_filter(
        &self,
        filters: crate::QFlags<crate::q_dir::Filter>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_entryInfoList2(
            self as *const crate::QDir,
            filters.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo> QDir::entryInfoList() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a list of <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> objects for all the files and directories in the directory, ordered according to the name and attribute filters previously set with <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>(), and sorted according to the flags set with <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>().</p>
    /// <p>The attribute filter and sorting specifications can be overridden using the <i>filters</i> and <i>sort</i> arguments.</p>
    /// <p>Returns an empty list if the directory is unreadable, does not exist, or if nothing matches the specification.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#isReadable">isReadable</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#exists-1">exists</a>().</p></div>
    pub unsafe fn entry_info_list(&self) -> ::cpp_utils::CppBox<crate::QListOfQFileInfo> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDir_entryInfoList3(self as *const crate::QDir);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list of <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> objects for all the files and directories in the directory, ordered according to the name and attribute filters previously set with <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>(), and sorted according to the flags set with <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo> QDir::entryInfoList(const QStringList& nameFilters, QFlags<QDir::Filter> filters = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> objects for all the files and directories in the directory, ordered according to the name and attribute filters previously set with <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>(), and sorted according to the flags set with <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>().</p>
    /// <p>The name filter, file attribute filter, and sorting specification can be overridden using the <i>nameFilters</i>, <i>filters</i>, and <i>sort</i> arguments.</p>
    /// <p>Returns an empty list if the directory is unreadable, does not exist, or if nothing matches the specification.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#isReadable">isReadable</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#exists-1">exists</a>().</p></div>
    pub unsafe fn entry_info_list_q_string_list_q_flags_filter(
        &self,
        name_filters: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
        filters: crate::QFlags<crate::q_dir::Filter>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_entryInfoList4(
            self as *const crate::QDir,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(name_filters)
                .as_raw_ptr(),
            filters.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list of <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> objects for all the files and directories in the directory, ordered according to the name and attribute filters previously set with <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>(), and sorted according to the flags set with <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo> QDir::entryInfoList(const QStringList& nameFilters) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> objects for all the files and directories in the directory, ordered according to the name and attribute filters previously set with <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>(), and sorted according to the flags set with <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>().</p>
    /// <p>The name filter, file attribute filter, and sorting specification can be overridden using the <i>nameFilters</i>, <i>filters</i>, and <i>sort</i> arguments.</p>
    /// <p>Returns an empty list if the directory is unreadable, does not exist, or if nothing matches the specification.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#isReadable">isReadable</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#exists-1">exists</a>().</p></div>
    pub unsafe fn entry_info_list_q_string_list(
        &self,
        name_filters: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_entryInfoList5(
            self as *const crate::QDir,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(name_filters)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QDir::entryList(QFlags<QDir::Filter> filters = …, QFlags<QDir::SortFlag> sort = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#entryList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a list of the names of all the files and directories in the directory, ordered according to the name and attribute filters previously set with <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>(), and sorted according to the flags set with <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>().</p>
    /// <p>The attribute filter and sorting specifications can be overridden using the <i>filters</i> and <i>sort</i> arguments.</p>
    /// <p>Returns an empty list if the directory is unreadable, does not exist, or if nothing matches the specification.</p>
    /// <p><b>Note: </b>To list symlinks that point to non existing files, <a href="http://doc.qt.io/qt-5/qdir.html#Filter-enum">System</a> must be passed to the filter.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList">entryInfoList</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>().</p></div>
    pub unsafe fn entry_list_q_flags_filter_q_flags_sort_flag(
        &self,
        filters: crate::QFlags<crate::q_dir::Filter>,
        sort: crate::QFlags<crate::q_dir::SortFlag>,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_entryList(
            self as *const crate::QDir,
            filters.to_int(),
            sort.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list of the names of all the files and directories in the directory, ordered according to the name and attribute filters previously set with <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>(), and sorted according to the flags set with <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QDir::entryList(const QStringList& nameFilters, QFlags<QDir::Filter> filters = …, QFlags<QDir::SortFlag> sort = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#entryList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of the names of all the files and directories in the directory, ordered according to the name and attribute filters previously set with <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>(), and sorted according to the flags set with <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>().</p>
    /// <p>The name filter, file attribute filter, and sorting specification can be overridden using the <i>nameFilters</i>, <i>filters</i>, and <i>sort</i> arguments.</p>
    /// <p>Returns an empty list if the directory is unreadable, does not exist, or if nothing matches the specification.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList">entryInfoList</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>().</p></div>
    pub unsafe fn entry_list_q_string_list_q_flags_filter_q_flags_sort_flag(
        &self,
        name_filters: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
        filters: crate::QFlags<crate::q_dir::Filter>,
        sort: crate::QFlags<crate::q_dir::SortFlag>,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_entryList1(
            self as *const crate::QDir,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(name_filters)
                .as_raw_ptr(),
            filters.to_int(),
            sort.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QDir::entryList(QFlags<QDir::Filter> filters = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#entryList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a list of the names of all the files and directories in the directory, ordered according to the name and attribute filters previously set with <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>(), and sorted according to the flags set with <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>().</p>
    /// <p>The attribute filter and sorting specifications can be overridden using the <i>filters</i> and <i>sort</i> arguments.</p>
    /// <p>Returns an empty list if the directory is unreadable, does not exist, or if nothing matches the specification.</p>
    /// <p><b>Note: </b>To list symlinks that point to non existing files, <a href="http://doc.qt.io/qt-5/qdir.html#Filter-enum">System</a> must be passed to the filter.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList">entryInfoList</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>().</p></div>
    pub unsafe fn entry_list_q_flags_filter(
        &self,
        filters: crate::QFlags<crate::q_dir::Filter>,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_entryList2(
            self as *const crate::QDir,
            filters.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QDir::entryList() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#entryList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a list of the names of all the files and directories in the directory, ordered according to the name and attribute filters previously set with <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>(), and sorted according to the flags set with <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>().</p>
    /// <p>The attribute filter and sorting specifications can be overridden using the <i>filters</i> and <i>sort</i> arguments.</p>
    /// <p>Returns an empty list if the directory is unreadable, does not exist, or if nothing matches the specification.</p>
    /// <p><b>Note: </b>To list symlinks that point to non existing files, <a href="http://doc.qt.io/qt-5/qdir.html#Filter-enum">System</a> must be passed to the filter.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList">entryInfoList</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>().</p></div>
    pub unsafe fn entry_list(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_entryList3(self as *const crate::QDir);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list of the names of all the files and directories in the directory, ordered according to the name and attribute filters previously set with <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>(), and sorted according to the flags set with <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QDir::entryList(const QStringList& nameFilters, QFlags<QDir::Filter> filters = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#entryList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of the names of all the files and directories in the directory, ordered according to the name and attribute filters previously set with <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>(), and sorted according to the flags set with <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>().</p>
    /// <p>The name filter, file attribute filter, and sorting specification can be overridden using the <i>nameFilters</i>, <i>filters</i>, and <i>sort</i> arguments.</p>
    /// <p>Returns an empty list if the directory is unreadable, does not exist, or if nothing matches the specification.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList">entryInfoList</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>().</p></div>
    pub unsafe fn entry_list_q_string_list_q_flags_filter(
        &self,
        name_filters: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
        filters: crate::QFlags<crate::q_dir::Filter>,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_entryList4(
            self as *const crate::QDir,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(name_filters)
                .as_raw_ptr(),
            filters.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list of the names of all the files and directories in the directory, ordered according to the name and attribute filters previously set with <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>(), and sorted according to the flags set with <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QDir::entryList(const QStringList& nameFilters) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#entryList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of the names of all the files and directories in the directory, ordered according to the name and attribute filters previously set with <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>(), and sorted according to the flags set with <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>().</p>
    /// <p>The name filter, file attribute filter, and sorting specification can be overridden using the <i>nameFilters</i>, <i>filters</i>, and <i>sort</i> arguments.</p>
    /// <p>Returns an empty list if the directory is unreadable, does not exist, or if nothing matches the specification.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList">entryInfoList</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>().</p></div>
    pub unsafe fn entry_list_q_string_list(
        &self,
        name_filters: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_entryList5(
            self as *const crate::QDir,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(name_filters)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDir::exists() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#exists-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the directory exists; otherwise returns <code>false</code>. (If a file with the same name is found this function will return false).</p>
    /// <p>The overload of this function that accepts an argument is used to test for the presence of files and directories within a directory.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#exists">QFileInfo::exists</a>() and <a href="http://doc.qt.io/qt-5/qfile.html#exists-1">QFile::exists</a>().</p></div>
    pub unsafe fn exists_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDir_exists(self as *const crate::QDir)
    }

    /// <p>Returns <code>true</code> if the file called <i>name</i> exists; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDir::exists(const QString& name) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#exists">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the file called <i>name</i> exists; otherwise returns false.</p>
    /// <p>Unless <i>name</i> contains an absolute file path, the file name is assumed to be relative to the directory itself, so this function is typically used to check for the presence of files within a directory.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#exists">QFileInfo::exists</a>() and <a href="http://doc.qt.io/qt-5/qfile.html#exists-1">QFile::exists</a>().</p></div>
    pub unsafe fn exists_1a(
        &self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDir_exists1(
            self as *const crate::QDir,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
        )
    }

    /// <p>Returns the path name of a file in the directory. Does <i>not</i> check if the file actually exists in the directory; but see <a href="http://doc.qt.io/qt-5/qdir.html#exists-1">exists</a>(). If the <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> is relative the returned path name will also be relative. Redundant multiple separators or "." and ".." directories in <i>fileName</i> are not removed (see <a href="http://doc.qt.io/qt-5/qdir.html#cleanPath">cleanPath</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QDir::filePath(const QString& fileName) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#filePath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the path name of a file in the directory. Does <i>not</i> check if the file actually exists in the directory; but see <a href="http://doc.qt.io/qt-5/qdir.html#exists-1">exists</a>(). If the <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> is relative the returned path name will also be relative. Redundant multiple separators or "." and ".." directories in <i>fileName</i> are not removed (see <a href="http://doc.qt.io/qt-5/qdir.html#cleanPath">cleanPath</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#dirName">dirName</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#absoluteFilePath">absoluteFilePath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#isRelative">isRelative</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#canonicalPath">canonicalPath</a>().</p></div>
    pub unsafe fn file_path(
        &self,
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_filePath(
            self as *const crate::QDir,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value set by <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>()</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFlags<QDir::Filter> QDir::filter() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#filter">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value set by <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>()</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>().</p></div>
    pub unsafe fn filter(&self) -> crate::QFlags<crate::q_dir::Filter> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_filter(self as *const crate::QDir);
        crate::QFlags::from(ffi_result)
    }

    /// <p>Returns <i>pathName</i> using '/' as file separator. On Windows, for instance, fromNativeSeparators("<code>c:\\winnt\\system32</code>") returns "c:/winnt/system32".</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QDir::fromNativeSeparators(const QString& pathName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#fromNativeSeparators">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <i>pathName</i> using '/' as file separator. On Windows, for instance, fromNativeSeparators("<code>c:\\winnt\\system32</code>") returns "c:/winnt/system32".</p>
    /// <p>The returned string may be the same as the argument on some operating systems, for example on Unix.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#toNativeSeparators">toNativeSeparators</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#separator">separator</a>().</p></div>
    pub unsafe fn from_native_separators(
        path_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_fromNativeSeparators(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(path_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the user's home directory.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDir QDir::home()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#home">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the user's home directory.</p>
    /// <p>The directory is constructed using the absolute path of the home directory, ensuring that its <a href="http://doc.qt.io/qt-5/qdir.html#path">path</a>() will be the same as its <a href="http://doc.qt.io/qt-5/qdir.html#absolutePath">absolutePath</a>().</p>
    /// <p>See <a href="http://doc.qt.io/qt-5/qdir.html#homePath">homePath</a>() for details.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#drives">drives</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#current">current</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#root">root</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#temp">temp</a>().</p></div>
    pub unsafe fn home() -> ::cpp_utils::CppBox<crate::QDir> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_home();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the absolute path of the user's home directory.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QDir::homePath()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#homePath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the absolute path of the user's home directory.</p>
    /// <p>Under Windows this function will return the directory of the current user's profile. Typically, this is:</p>
    /// <pre class="cpp">
    ///
    ///   C:<span class="operator">/</span>Documents and Settings<span class="operator">/</span>Username
    ///
    /// </pre>
    /// <p>Use the <a href="http://doc.qt.io/qt-5/qdir.html#toNativeSeparators">toNativeSeparators</a>() function to convert the separators to the ones that are appropriate for the underlying operating system.</p>
    /// <p>If the directory of the current user's profile does not exist or cannot be retrieved, the following alternatives will be checked (in the given order) until an existing and available path is found:</p>
    /// <ol class="1" type="1"><li>The path specified by the <code>USERPROFILE</code> environment variable.</li>
    /// <li>The path formed by concatenating the <code>HOMEDRIVE</code> and <code>HOMEPATH</code> environment variables.</li>
    /// <li>The path specified by the <code>HOME</code> environment variable.</li>
    /// <li>The path returned by the <a href="http://doc.qt.io/qt-5/qdir.html#rootPath">rootPath</a>() function (which uses the <code>SystemDrive</code> environment variable)</li>
    /// <li>The <code>C:/</code> directory.</li>
    /// </ol>
    /// <p>Under non-Windows operating systems the <code>HOME</code> environment variable is used if it exists, otherwise the path returned by the <a href="http://doc.qt.io/qt-5/qdir.html#rootPath">rootPath</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#home">home</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#currentPath">currentPath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#rootPath">rootPath</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#tempPath">tempPath</a>().</p></div>
    pub unsafe fn home_path() -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_homePath();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the file name at position <i>pos</i> in the list of file names. Equivalent to <a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>().at(index). <i>pos</i> must be a valid index position in the list (i.e., 0 &lt;= pos &lt; <a href="http://doc.qt.io/qt-5/qdir.html#count">count</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QDir::operator[](int arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the file name at position <i>pos</i> in the list of file names. Equivalent to <a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>().at(index). <i>pos</i> must be a valid index position in the list (i.e., 0 &lt;= pos &lt; <a href="http://doc.qt.io/qt-5/qdir.html#count">count</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#count">count</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>().</p></div>
    pub unsafe fn index(&self, arg1: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDir_operator__(self as *const crate::QDir, arg1);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if the directory's path is absolute; otherwise returns <code>false</code>. See <a href="http://doc.qt.io/qt-5/qdir.html#isAbsolutePath">isAbsolutePath</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDir::isAbsolute() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#isAbsolute">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the directory's path is absolute; otherwise returns <code>false</code>. See <a href="http://doc.qt.io/qt-5/qdir.html#isAbsolutePath">isAbsolutePath</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#isRelative">isRelative</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#makeAbsolute">makeAbsolute</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#cleanPath">cleanPath</a>().</p></div>
    pub unsafe fn is_absolute(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDir_isAbsolute(self as *const crate::QDir)
    }

    /// <p>Returns <code>true</code> if <i>path</i> is absolute; returns <code>false</code> if it is relative.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QDir::isAbsolutePath(const QString& path)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#isAbsolutePath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>path</i> is absolute; returns <code>false</code> if it is relative.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#isAbsolute">isAbsolute</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#isRelativePath">isRelativePath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#makeAbsolute">makeAbsolute</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#cleanPath">cleanPath</a>().</p></div>
    pub unsafe fn is_absolute_path(
        path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDir_isAbsolutePath(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(path).as_raw_ptr(),
        )
    }

    /// <p>Returns whether the directory is empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDir::isEmpty(QFlags<QDir::Filter> filters = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns whether the directory is empty.</p>
    /// <p>Equivalent to <code>count() == 0</code> with filters <code>QDir::AllEntries | QDir::NoDotAndDotDot</code>, but faster as it just checks whether the directory contains at least one entry.</p>
    /// <p><b>Note: </b>Unless you set the <i>filters</i> flags to include <code>QDir::NoDotAndDotDot</code> (as the default value does), no directory is empty.</p><p>This function was introduced in  Qt 5.9.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#count">count</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>().</p></div>
    pub unsafe fn is_empty_1a(&self, filters: crate::QFlags<crate::q_dir::Filter>) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDir_isEmpty(self as *const crate::QDir, filters.to_int())
    }

    /// <p>Returns whether the directory is empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDir::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns whether the directory is empty.</p>
    /// <p>Equivalent to <code>count() == 0</code> with filters <code>QDir::AllEntries | QDir::NoDotAndDotDot</code>, but faster as it just checks whether the directory contains at least one entry.</p>
    /// <p><b>Note: </b>Unless you set the <i>filters</i> flags to include <code>QDir::NoDotAndDotDot</code> (as the default value does), no directory is empty.</p><p>This function was introduced in  Qt 5.9.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#count">count</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>().</p></div>
    pub unsafe fn is_empty_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDir_isEmpty1(self as *const crate::QDir)
    }

    /// <p>Returns <code>true</code> if the directory is readable <i>and</i> we can open files by name; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDir::isReadable() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#isReadable">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the directory is readable <i>and</i> we can open files by name; otherwise returns <code>false</code>.</p>
    /// <p><b>Warning:</b> A false value from this function is not a guarantee that files in the directory are not accessible.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#isReadable">QFileInfo::isReadable</a>().</p></div>
    pub unsafe fn is_readable(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDir_isReadable(self as *const crate::QDir)
    }

    /// <p>Returns <code>true</code> if the directory path is relative; otherwise returns false. (Under Unix a path is relative if it does not start with a "/").</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDir::isRelative() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#isRelative">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the directory path is relative; otherwise returns false. (Under Unix a path is relative if it does not start with a "/").</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#makeAbsolute">makeAbsolute</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#isAbsolute">isAbsolute</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#isAbsolutePath">isAbsolutePath</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#cleanPath">cleanPath</a>().</p></div>
    pub unsafe fn is_relative(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDir_isRelative(self as *const crate::QDir)
    }

    /// <p>Returns <code>true</code> if <i>path</i> is relative; returns <code>false</code> if it is absolute.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QDir::isRelativePath(const QString& path)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#isRelativePath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>path</i> is relative; returns <code>false</code> if it is absolute.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#isRelative">isRelative</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#isAbsolutePath">isAbsolutePath</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#makeAbsolute">makeAbsolute</a>().</p></div>
    pub unsafe fn is_relative_path(
        path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDir_isRelativePath(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(path).as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if the directory is the root directory; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDir::isRoot() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#isRoot">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the directory is the root directory; otherwise returns <code>false</code>.</p>
    /// <p>Note: If the directory is a symbolic link to the root directory this function returns <code>false</code>. If you want to test for this use <a href="http://doc.qt.io/qt-5/qdir.html#canonicalPath">canonicalPath</a>(), e.g.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdir.html#QDir">QDir</a></span> dir(<span class="string">"/tmp/root_link"</span>);
    ///   dir <span class="operator">=</span> dir<span class="operator">.</span>canonicalPath();
    ///   <span class="keyword">if</span> (dir<span class="operator">.</span>isRoot())
    /// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qWarning">qWarning</a>(<span class="string">"It is a root link"</span>);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#root">root</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#rootPath">rootPath</a>().</p></div>
    pub unsafe fn is_root(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDir_isRoot(self as *const crate::QDir)
    }

    /// <p>Returns the native path list separator: ':' under Unix and ';' under Windows.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QChar QDir::listSeparator()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#listSeparator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the native path list separator: ':' under Unix and ';' under Windows.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#separator">separator</a>().</p></div>
    pub unsafe fn list_separator() -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_listSeparator();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts the directory path to an absolute path. If it is already absolute nothing happens. Returns <code>true</code> if the conversion succeeded; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDir::makeAbsolute()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#makeAbsolute">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the directory path to an absolute path. If it is already absolute nothing happens. Returns <code>true</code> if the conversion succeeded; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#isAbsolute">isAbsolute</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#isAbsolutePath">isAbsolutePath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#isRelative">isRelative</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#cleanPath">cleanPath</a>().</p></div>
    pub unsafe fn make_absolute(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDir_makeAbsolute(self as *mut crate::QDir)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QDir::match(const QStringList& filters, const QString& fileName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#match-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the <i>fileName</i> matches any of the wildcard (glob) patterns in the list of <i>filters</i>; otherwise returns <code>false</code>. The matching is case insensitive.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#qregexp-wildcard-matching">QRegExp wildcard matching</a>, <a href="http://doc.qt.io/qt-5/qregexp.html#exactMatch">QRegExp::exactMatch</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList">entryInfoList</a>().</p></div>
    pub unsafe fn match_q_string_list_q_string(
        filters: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDir_match(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(filters)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if the <i>fileName</i> matches the wildcard (glob) pattern <i>filter</i>; otherwise returns <code>false</code>. The <i>filter</i> may contain multiple patterns separated by spaces or semicolons. The matching is case insensitive.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QDir::match(const QString& filter, const QString& fileName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#match">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the <i>fileName</i> matches the wildcard (glob) pattern <i>filter</i>; otherwise returns <code>false</code>. The <i>filter</i> may contain multiple patterns separated by spaces or semicolons. The matching is case insensitive.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregexp.html#qregexp-wildcard-matching">QRegExp wildcard matching</a>, <a href="http://doc.qt.io/qt-5/qregexp.html#exactMatch">QRegExp::exactMatch</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList">entryInfoList</a>().</p></div>
    pub unsafe fn match_2_q_string(
        filter: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDir_match1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(filter)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        )
    }

    /// <p>Creates a sub-directory called <i>dirName</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDir::mkdir(const QString& dirName) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#mkdir">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a sub-directory called <i>dirName</i>.</p>
    /// <p>Returns <code>true</code> on success; otherwise returns <code>false</code>.</p>
    /// <p>If the directory already exists when this function is called, it will return false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#rmdir">rmdir</a>().</p></div>
    pub unsafe fn mkdir(
        &self,
        dir_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDir_mkdir(
            self as *const crate::QDir,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(dir_name)
                .as_raw_ptr(),
        )
    }

    /// <p>Creates the directory path <i>dirPath</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDir::mkpath(const QString& dirPath) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#mkpath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates the directory path <i>dirPath</i>.</p>
    /// <p>The function will create all parent directories necessary to create the directory.</p>
    /// <p>Returns <code>true</code> if successful; otherwise returns <code>false</code>.</p>
    /// <p>If the path already exists when this function is called, it will return true.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#rmpath">rmpath</a>().</p></div>
    pub unsafe fn mkpath(
        &self,
        dir_path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDir_mkpath(
            self as *const crate::QDir,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(dir_path)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the string list set by <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>()</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QDir::nameFilters() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#nameFilters">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string list set by <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>()</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>().</p></div>
    pub unsafe fn name_filters(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_nameFilters(self as *const crate::QDir);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QStringList QDir::nameFiltersFromString(const QString& nameFilter)```</span>.
    pub unsafe fn name_filters_from_string(
        name_filter: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_nameFiltersFromString(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name_filter)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> pointing to the given directory <i>path</i>. If path is empty the program's working directory, ("."), is used.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDir::QDir(const QString& path = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#QDir-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> pointing to the given directory <i>path</i>. If path is empty the program's working directory, ("."), is used.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#currentPath">currentPath</a>().</p></div>
    pub unsafe fn new_1a(
        path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QDir> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_QDir1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(path).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> with path <i>path</i>, that filters its entries by name using <i>nameFilter</i> and by attributes using <i>filters</i>. It also sorts the names using <i>sort</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDir::QDir(const QString& path, const QString& nameFilter, QFlags<QDir::SortFlag> sort = …, QFlags<QDir::Filter> filter = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#QDir-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> with path <i>path</i>, that filters its entries by name using <i>nameFilter</i> and by attributes using <i>filters</i>. It also sorts the names using <i>sort</i>.</p>
    /// <p>The default <i>nameFilter</i> is an empty string, which excludes nothing; the default <i>filters</i> is <a href="http://doc.qt.io/qt-5/qdir.html#Filter-enum">AllEntries</a>, which also means exclude nothing. The default <i>sort</i> is <a href="http://doc.qt.io/qt-5/qdir.html#SortFlag-enum">Name</a> | <a href="http://doc.qt.io/qt-5/qdir.html#SortFlag-enum">IgnoreCase</a>, i.e. sort by name case-insensitively.</p>
    /// <p>If <i>path</i> is an empty string, <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> uses "." (the current directory). If <i>nameFilter</i> is an empty string, <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> uses the name filter "*" (all files).</p>
    /// <p>Note that <i>path</i> need not exist.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#exists-1">exists</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setPath">setPath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>().</p></div>
    pub unsafe fn new_4a(
        path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        name_filter: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        sort: crate::QFlags<crate::q_dir::SortFlag>,
        filter: crate::QFlags<crate::q_dir::Filter>,
    ) -> ::cpp_utils::CppBox<crate::QDir> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_QDir2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(path).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name_filter)
                .as_raw_ptr(),
            sort.to_int(),
            filter.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> class provides access to directory structures and their contents.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDir::QDir()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> class provides access to directory structures and their contents.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> is used to manipulate path names, access information regarding paths and files, and manipulate the underlying file system. It can also be used to access Qt's <a href="http://doc.qt.io/qt-5/resources.html">resource system</a>.</p>
    /// <p>Qt uses "/" as a universal directory separator in the same way that "/" is used as a path separator in URLs. If you always use "/" as a directory separator, Qt will translate your paths to conform to the underlying operating system.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> can point to a file using either a relative or an absolute path. Absolute paths begin with the directory separator (optionally preceded by a drive specification under Windows). Relative file names begin with a directory name or a file name and specify a path relative to the current directory.</p>
    /// <p>Examples of absolute paths:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdir.html#QDir">QDir</a></span>(<span class="string">"/home/user/Documents"</span>)
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdir.html#QDir">QDir</a></span>(<span class="string">"C:/Documents and Settings"</span>)
    ///
    /// </pre>
    /// <p>On Windows, the second example above will be translated to <code>C:\Documents and Settings</code> when used to access files.</p>
    /// <p>Examples of relative paths:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdir.html#QDir">QDir</a></span>(<span class="string">"images/landscape.png"</span>)
    ///
    /// </pre>
    /// <p>You can use the <a href="http://doc.qt.io/qt-5/qdir.html#isRelative">isRelative</a>() or <a href="http://doc.qt.io/qt-5/qdir.html#isAbsolute">isAbsolute</a>() functions to check if a <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> is using a relative or an absolute file path. Call <a href="http://doc.qt.io/qt-5/qdir.html#makeAbsolute">makeAbsolute</a>() to convert a relative <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> to an absolute one.</p>
    /// <a name="navigation-and-directory-operations"></a></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QDir> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_QDir3();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> with path <i>path</i>, that filters its entries by name using <i>nameFilter</i> and by attributes using <i>filters</i>. It also sorts the names using <i>sort</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDir::QDir(const QString& path, const QString& nameFilter, QFlags<QDir::SortFlag> sort = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#QDir-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> with path <i>path</i>, that filters its entries by name using <i>nameFilter</i> and by attributes using <i>filters</i>. It also sorts the names using <i>sort</i>.</p>
    /// <p>The default <i>nameFilter</i> is an empty string, which excludes nothing; the default <i>filters</i> is <a href="http://doc.qt.io/qt-5/qdir.html#Filter-enum">AllEntries</a>, which also means exclude nothing. The default <i>sort</i> is <a href="http://doc.qt.io/qt-5/qdir.html#SortFlag-enum">Name</a> | <a href="http://doc.qt.io/qt-5/qdir.html#SortFlag-enum">IgnoreCase</a>, i.e. sort by name case-insensitively.</p>
    /// <p>If <i>path</i> is an empty string, <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> uses "." (the current directory). If <i>nameFilter</i> is an empty string, <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> uses the name filter "*" (all files).</p>
    /// <p>Note that <i>path</i> need not exist.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#exists-1">exists</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setPath">setPath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>().</p></div>
    pub unsafe fn new_3a(
        path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        name_filter: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        sort: crate::QFlags<crate::q_dir::SortFlag>,
    ) -> ::cpp_utils::CppBox<crate::QDir> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_QDir4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(path).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name_filter)
                .as_raw_ptr(),
            sort.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> with path <i>path</i>, that filters its entries by name using <i>nameFilter</i> and by attributes using <i>filters</i>. It also sorts the names using <i>sort</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDir::QDir(const QString& path, const QString& nameFilter)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#QDir-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> with path <i>path</i>, that filters its entries by name using <i>nameFilter</i> and by attributes using <i>filters</i>. It also sorts the names using <i>sort</i>.</p>
    /// <p>The default <i>nameFilter</i> is an empty string, which excludes nothing; the default <i>filters</i> is <a href="http://doc.qt.io/qt-5/qdir.html#Filter-enum">AllEntries</a>, which also means exclude nothing. The default <i>sort</i> is <a href="http://doc.qt.io/qt-5/qdir.html#SortFlag-enum">Name</a> | <a href="http://doc.qt.io/qt-5/qdir.html#SortFlag-enum">IgnoreCase</a>, i.e. sort by name case-insensitively.</p>
    /// <p>If <i>path</i> is an empty string, <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> uses "." (the current directory). If <i>nameFilter</i> is an empty string, <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> uses the name filter "*" (all files).</p>
    /// <p>Note that <i>path</i> need not exist.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#exists-1">exists</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setPath">setPath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>().</p></div>
    pub unsafe fn new_2a(
        path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        name_filter: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QDir> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_QDir5(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(path).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name_filter)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> object that is a copy of the <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> object for directory <i>dir</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDir::QDir(const QDir& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#QDir">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> object that is a copy of the <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> object for directory <i>dir</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDir>>,
    ) -> ::cpp_utils::CppBox<crate::QDir> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_QDir(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDir>>::cast_into(arg1).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the path. This may contain symbolic links, but never contains redundant ".", ".." or multiple separators.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QDir::path() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#path">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the path. This may contain symbolic links, but never contains redundant ".", ".." or multiple separators.</p>
    /// <p>The returned path can be either absolute or relative (see <a href="http://doc.qt.io/qt-5/qdir.html#setPath">setPath</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#setPath">setPath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#absolutePath">absolutePath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#exists-1">exists</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#cleanPath">cleanPath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#dirName">dirName</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#absoluteFilePath">absoluteFilePath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#toNativeSeparators">toNativeSeparators</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#makeAbsolute">makeAbsolute</a>().</p></div>
    pub unsafe fn path(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_path(self as *const crate::QDir);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Refreshes the directory information.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDir::refresh() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#refresh">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Refreshes the directory information.</p></div>
    pub unsafe fn refresh(&self) {
        crate::__ffi::ctr_qt_core_ffi_QDir_refresh(self as *const crate::QDir)
    }

    /// <p>Returns the path to <i>fileName</i> relative to the directory.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QDir::relativeFilePath(const QString& fileName) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#relativeFilePath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the path to <i>fileName</i> relative to the directory.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdir.html#QDir">QDir</a></span> dir(<span class="string">"/home/bob"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> s;
    ///
    ///   s <span class="operator">=</span> dir<span class="operator">.</span>relativeFilePath(<span class="string">"images/file.jpg"</span>);     <span class="comment">// s is "images/file.jpg"</span>
    ///   s <span class="operator">=</span> dir<span class="operator">.</span>relativeFilePath(<span class="string">"/home/mary/file.txt"</span>); <span class="comment">// s is "../mary/file.txt"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#absoluteFilePath">absoluteFilePath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#filePath">filePath</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#canonicalPath">canonicalPath</a>().</p></div>
    pub unsafe fn relative_file_path(
        &self,
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_relativeFilePath(
            self as *const crate::QDir,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the file, <i>fileName</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDir::remove(const QString& fileName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#remove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the file, <i>fileName</i>.</p>
    /// <p>Returns <code>true</code> if the file is removed successfully; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn remove(
        &mut self,
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDir_remove(
            self as *mut crate::QDir,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes the directory, including all its contents.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDir::removeRecursively()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#removeRecursively">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the directory, including all its contents.</p>
    /// <p>Returns <code>true</code> if successful, otherwise false.</p>
    /// <p>If a file or directory cannot be removed, removeRecursively() keeps going and attempts to delete as many files and sub-directories as possible, then returns <code>false</code>.</p>
    /// <p>If the directory was already removed, the method returns <code>true</code> (expected result already reached).</p>
    /// <p>Note: this function is meant for removing a small application-internal directory (such as a temporary directory), but not user-visible directories. For user-visible operations, it is rather recommended to report errors more precisely to the user, to offer solutions in case of errors, to show progress during the deletion since it could take several minutes, etc.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn remove_recursively(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDir_removeRecursively(self as *mut crate::QDir)
    }

    /// <p>Renames a file or directory from <i>oldName</i> to <i>newName</i>, and returns true if successful; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDir::rename(const QString& oldName, const QString& newName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#rename">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Renames a file or directory from <i>oldName</i> to <i>newName</i>, and returns true if successful; otherwise returns <code>false</code>.</p>
    /// <p>On most file systems, rename() fails only if <i>oldName</i> does not exist, or if a file with the new name already exists. However, there are also other reasons why rename() can fail. For example, on at least one file system rename() fails if <i>newName</i> points to an open file.</p>
    /// <p>If <i>oldName</i> is a file (not a directory) that can't be renamed right away, Qt will try to copy <i>oldName</i> to <i>newName</i> and remove <i>oldName</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfile.html#rename">QFile::rename</a>().</p></div>
    pub unsafe fn rename(
        &mut self,
        old_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        new_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDir_rename(
            self as *mut crate::QDir,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(old_name)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(new_name)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes the directory specified by <i>dirName</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDir::rmdir(const QString& dirName) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#rmdir">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the directory specified by <i>dirName</i>.</p>
    /// <p>The directory must be empty for rmdir() to succeed.</p>
    /// <p>Returns <code>true</code> if successful; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#mkdir">mkdir</a>().</p></div>
    pub unsafe fn rmdir(
        &self,
        dir_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDir_rmdir(
            self as *const crate::QDir,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(dir_name)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes the directory path <i>dirPath</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDir::rmpath(const QString& dirPath) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#rmpath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the directory path <i>dirPath</i>.</p>
    /// <p>The function will remove all parent directories in <i>dirPath</i>, provided that they are empty. This is the opposite of mkpath(dirPath).</p>
    /// <p>Returns <code>true</code> if successful; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#mkpath">mkpath</a>().</p></div>
    pub unsafe fn rmpath(
        &self,
        dir_path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDir_rmpath(
            self as *const crate::QDir,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(dir_path)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the root directory.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDir QDir::root()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#root">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the root directory.</p>
    /// <p>The directory is constructed using the absolute path of the root directory, ensuring that its <a href="http://doc.qt.io/qt-5/qdir.html#path">path</a>() will be the same as its <a href="http://doc.qt.io/qt-5/qdir.html#absolutePath">absolutePath</a>().</p>
    /// <p>See <a href="http://doc.qt.io/qt-5/qdir.html#rootPath">rootPath</a>() for details.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#drives">drives</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#current">current</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#home">home</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#temp">temp</a>().</p></div>
    pub unsafe fn root() -> ::cpp_utils::CppBox<crate::QDir> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_root();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the absolute path of the root directory.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QDir::rootPath()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#rootPath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the absolute path of the root directory.</p>
    /// <p>For Unix operating systems this returns "/". For Windows file systems this normally returns "c:/".</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#root">root</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#drives">drives</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#currentPath">currentPath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#homePath">homePath</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#tempPath">tempPath</a>().</p></div>
    pub unsafe fn root_path() -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_rootPath();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the search paths for <i>prefix</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QStringList QDir::searchPaths(const QString& prefix)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#searchPaths">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the search paths for <i>prefix</i>.</p>
    /// <p>This function was introduced in  Qt 4.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#setSearchPaths">setSearchPaths</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#addSearchPath">addSearchPath</a>().</p></div>
    pub unsafe fn search_paths(
        prefix: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_searchPaths(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(prefix)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the native directory separator: "/" under Unix and "\" under Windows.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QChar QDir::separator()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#separator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the native directory separator: "/" under Unix and "\" under Windows.</p>
    /// <p>You do not need to use this function to build file paths. If you always use "/", Qt will translate your paths to conform to the underlying operating system. If you want to display paths to the user using their operating system's separator use <a href="http://doc.qt.io/qt-5/qdir.html#toNativeSeparators">toNativeSeparators</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#listSeparator">listSeparator</a>().</p></div>
    pub unsafe fn separator() -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_separator();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Sets the application's current working directory to <i>path</i>. Returns <code>true</code> if the directory was successfully changed; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QDir::setCurrent(const QString& path)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#setCurrent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the application's current working directory to <i>path</i>. Returns <code>true</code> if the directory was successfully changed; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#current">current</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#currentPath">currentPath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#home">home</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#root">root</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#temp">temp</a>().</p></div>
    pub unsafe fn set_current(
        path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDir_setCurrent(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(path).as_raw_ptr(),
        )
    }

    /// <p>Sets the filter used by <a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList">entryInfoList</a>() to <i>filters</i>. The filter is used to specify the kind of files that should be returned by <a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList">entryInfoList</a>(). See <a href="http://doc.qt.io/qt-5/qdir.html#Filter-enum">QDir::Filter</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDir::setFilter(QFlags<QDir::Filter> filter)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the filter used by <a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList">entryInfoList</a>() to <i>filters</i>. The filter is used to specify the kind of files that should be returned by <a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList">entryInfoList</a>(). See <a href="http://doc.qt.io/qt-5/qdir.html#Filter-enum">QDir::Filter</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#filter">filter</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">setNameFilters</a>().</p></div>
    pub unsafe fn set_filter(&mut self, filter: crate::QFlags<crate::q_dir::Filter>) {
        crate::__ffi::ctr_qt_core_ffi_QDir_setFilter(self as *mut crate::QDir, filter.to_int())
    }

    /// <p>Sets the name filters used by <a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList">entryInfoList</a>() to the list of filters specified by <i>nameFilters</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDir::setNameFilters(const QStringList& nameFilters)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#setNameFilters">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the name filters used by <a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList">entryInfoList</a>() to the list of filters specified by <i>nameFilters</i>.</p>
    /// <p>Each name filter is a wildcard (globbing) filter that understands <code>*</code> and <code>?</code> wildcards. (See <a href="http://doc.qt.io/qt-5/qregexp.html#qregexp-wildcard-matching">QRegExp wildcard matching</a>.)</p>
    /// <p>For example, the following code sets three name filters on a <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> to ensure that only files with extensions typically used for C++ source files are listed:</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> filters;
    /// &#32;     filters <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"*.cpp"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"*.cxx"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"*.cc"</span>;
    /// &#32;     dir<span class="operator">.</span>setNameFilters(filters);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#nameFilters">nameFilters</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#setFilter">setFilter</a>().</p></div>
    pub unsafe fn set_name_filters(
        &mut self,
        name_filters: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QDir_setNameFilters(
            self as *mut crate::QDir,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(name_filters)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the path of the directory to <i>path</i>. The path is cleaned of redundant ".", ".." and of multiple separators. No check is made to see whether a directory with this path actually exists; but you can check for yourself using <a href="http://doc.qt.io/qt-5/qdir.html#exists-1">exists</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDir::setPath(const QString& path)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#setPath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the path of the directory to <i>path</i>. The path is cleaned of redundant ".", ".." and of multiple separators. No check is made to see whether a directory with this path actually exists; but you can check for yourself using <a href="http://doc.qt.io/qt-5/qdir.html#exists-1">exists</a>().</p>
    /// <p>The path can be either absolute or relative. Absolute paths begin with the directory separator "/" (optionally preceded by a drive specification under Windows). Relative file names begin with a directory name or a file name and specify a path relative to the current directory. An example of an absolute path is the string "/tmp/quartz", a relative path might look like "src/fatlib".</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#path">path</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#absolutePath">absolutePath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#exists-1">exists</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#cleanPath">cleanPath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#dirName">dirName</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#absoluteFilePath">absoluteFilePath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#isRelative">isRelative</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#makeAbsolute">makeAbsolute</a>().</p></div>
    pub unsafe fn set_path(
        &mut self,
        path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QDir_setPath(
            self as *mut crate::QDir,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(path).as_raw_ptr(),
        )
    }

    /// <p>Sets or replaces Qt's search paths for file names with the prefix <i>prefix</i> to <i>searchPaths</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QDir::setSearchPaths(const QString& prefix, const QStringList& searchPaths)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#setSearchPaths">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets or replaces Qt's search paths for file names with the prefix <i>prefix</i> to <i>searchPaths</i>.</p>
    /// <p>To specify a prefix for a file name, prepend the prefix followed by a single colon (e.g., "images:undo.png", "xmldocs:books.xml"). <i>prefix</i> can only contain letters or numbers (e.g., it cannot contain a colon, nor a slash).</p>
    /// <p>Qt uses this search path to locate files with a known prefix. The search path entries are tested in order, starting with the first entry.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdir.html#QDir">QDir</a></span><span class="operator">::</span>setSearchPaths(<span class="string">"icons"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span>(<span class="type"><a href="http://doc.qt.io/qt-5/qdir.html#QDir">QDir</a></span><span class="operator">::</span>homePath() <span class="operator">+</span> <span class="string">"/images"</span>));
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdir.html#QDir">QDir</a></span><span class="operator">::</span>setSearchPaths(<span class="string">"docs"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span>(<span class="string">":/embeddedDocuments"</span>));
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpixmap.html">QPixmap</a></span> pixmap(<span class="string">"icons:undo.png"</span>); <span class="comment">// will look for undo.png in QDir::homePath() + "/images"</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qfile.html">QFile</a></span> file(<span class="string">"docs:design.odf"</span>); <span class="comment">// will look in the :/embeddedDocuments resource path</span>
    ///
    /// </pre>
    /// <p>File name prefix must be at least 2 characters long to avoid conflicts with Windows drive letters.</p>
    /// <p>Search paths may contain paths to <a href="http://doc.qt.io/qt-5/resources.html">The Qt Resource System</a>.</p>
    /// <p>This function was introduced in  Qt 4.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#searchPaths">searchPaths</a>().</p></div>
    pub unsafe fn set_search_paths(
        prefix: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        search_paths: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QDir_setSearchPaths(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(prefix)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(search_paths)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the sort order used by <a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList">entryInfoList</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDir::setSorting(QFlags<QDir::SortFlag> sort)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the sort order used by <a href="http://doc.qt.io/qt-5/qdir.html#entryList">entryList</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList">entryInfoList</a>().</p>
    /// <p>The <i>sort</i> is specified by OR-ing values from the enum <a href="http://doc.qt.io/qt-5/qdir.html#SortFlag-enum">QDir::SortFlag</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#sorting">sorting</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#SortFlag-enum">SortFlag</a>.</p></div>
    pub unsafe fn set_sorting(&mut self, sort: crate::QFlags<crate::q_dir::SortFlag>) {
        crate::__ffi::ctr_qt_core_ffi_QDir_setSorting(self as *mut crate::QDir, sort.to_int())
    }

    /// <p>Returns the value set by <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>()</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFlags<QDir::SortFlag> QDir::sorting() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#sorting">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value set by <a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>()</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#setSorting">setSorting</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#SortFlag-enum">SortFlag</a>.</p></div>
    pub unsafe fn sorting(&self) -> crate::QFlags<crate::q_dir::SortFlag> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_sorting(self as *const crate::QDir);
        crate::QFlags::from(ffi_result)
    }

    /// <p>Swaps this <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> instance with <i>other</i>. This function is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QDir::swap(QDir& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps this <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> instance with <i>other</i>. This function is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QDir>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QDir_swap(
            self as *mut crate::QDir,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QDir>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the system's temporary directory.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDir QDir::temp()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#temp">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the system's temporary directory.</p>
    /// <p>The directory is constructed using the absolute path of the temporary directory, ensuring that its <a href="http://doc.qt.io/qt-5/qdir.html#path">path</a>() will be the same as its <a href="http://doc.qt.io/qt-5/qdir.html#absolutePath">absolutePath</a>().</p>
    /// <p>See <a href="http://doc.qt.io/qt-5/qdir.html#tempPath">tempPath</a>() for details.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#drives">drives</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#current">current</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#home">home</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#root">root</a>().</p></div>
    pub unsafe fn temp() -> ::cpp_utils::CppBox<crate::QDir> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_temp();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the absolute path of the system's temporary directory.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QDir::tempPath()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#tempPath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the absolute path of the system's temporary directory.</p>
    /// <p>On Unix/Linux systems this is the path in the <code>TMPDIR</code> environment variable or <code>/tmp</code> if <code>TMPDIR</code> is not defined. On Windows this is usually the path in the <code>TEMP</code> or <code>TMP</code> environment variable. The path returned by this method doesn't end with a directory separator unless it is the root directory (of a drive).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#temp">temp</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#currentPath">currentPath</a>(), <a href="http://doc.qt.io/qt-5/qdir.html#homePath">homePath</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#rootPath">rootPath</a>().</p></div>
    pub unsafe fn temp_path() -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_tempPath();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <i>pathName</i> with the '/' separators converted to separators that are appropriate for the underlying operating system.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QDir::toNativeSeparators(const QString& pathName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#toNativeSeparators">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <i>pathName</i> with the '/' separators converted to separators that are appropriate for the underlying operating system.</p>
    /// <p>On Windows, toNativeSeparators("c:/winnt/system32") returns "c:\winnt\system32".</p>
    /// <p>The returned string may be the same as the argument on some operating systems, for example on Unix.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#fromNativeSeparators">fromNativeSeparators</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#separator">separator</a>().</p></div>
    pub unsafe fn to_native_separators(
        path_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDir_toNativeSeparators(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(path_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_dir_iterator {
    //! C++ type: <span style='color: green;'>```QDirIterator```</span>

    /// <p>This enum describes flags that you can combine to configure the behavior of <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a>.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QDirIterator::IteratorFlag```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdiriterator.html#IteratorFlag-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes flags that you can combine to configure the behavior of <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a>.</p>
    ///
    /// <p>The IteratorFlags type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;IteratorFlag&gt;. It stores an OR combination of IteratorFlag values.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct IteratorFlag(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for IteratorFlag {
        fn from(value: ::std::os::raw::c_int) -> Self {
            IteratorFlag(value)
        }
    }

    impl From<IteratorFlag> for ::std::os::raw::c_int {
        fn from(value: IteratorFlag) -> Self {
            value.0
        }
    }

    impl IteratorFlag {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl IteratorFlag {
        /// The default value, representing no flags. The iterator will return entries for the assigned path. (C++ enum variant: <span style='color: green;'>```NoIteratorFlags = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoIteratorFlags: crate::q_dir_iterator::IteratorFlag =
            crate::q_dir_iterator::IteratorFlag(0);
        /// When combined with Subdirectories, this flag enables iterating through all subdirectories of the assigned path, following all symbolic links. Symbolic link loops (e.g., "link" =&gt; "." or "link" =&gt; "..") are automatically detected and ignored. (C++ enum variant: <span style='color: green;'>```FollowSymlinks = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const FollowSymlinks: crate::q_dir_iterator::IteratorFlag =
            crate::q_dir_iterator::IteratorFlag(1);
        /// List entries inside all subdirectories as well. (C++ enum variant: <span style='color: green;'>```Subdirectories = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const Subdirectories: crate::q_dir_iterator::IteratorFlag =
            crate::q_dir_iterator::IteratorFlag(2);
    }

    impl From<crate::q_dir_iterator::IteratorFlag>
        for crate::QFlags<crate::q_dir_iterator::IteratorFlag>
    {
        fn from(value: crate::q_dir_iterator::IteratorFlag) -> Self {
            Self::from(value.to_int())
        }
    }

    impl<T: Into<crate::QFlags<crate::q_dir_iterator::IteratorFlag>>> std::ops::BitOr<T>
        for crate::q_dir_iterator::IteratorFlag
    {
        type Output = crate::QFlags<crate::q_dir_iterator::IteratorFlag>;
        fn bitor(self, rhs: T) -> crate::QFlags<crate::q_dir_iterator::IteratorFlag> {
            Into::<crate::QFlags<crate::q_dir_iterator::IteratorFlag>>::into(self) | rhs
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a> class provides an iterator for directory entrylists.</p>
///
/// C++ class: <span style='color: green;'>```QDirIterator```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qdiriterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a> class provides an iterator for directory entrylists.</p>
/// <p>You can use <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a> to navigate entries of a directory one at a time. It is similar to <a href="http://doc.qt.io/qt-5/qdir.html#entryList">QDir::entryList</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#entryInfoList">QDir::entryInfoList</a>(), but because it lists entries one at a time instead of all at once, it scales better and is more suitable for large directories. It also supports listing directory contents recursively, and following symbolic links. Unlike <a href="http://doc.qt.io/qt-5/qdir.html#entryList">QDir::entryList</a>(), <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a> does not support sorting.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a> constructor takes a <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> or a directory as argument. After construction, the iterator is located before the first directory entry. Here's how to iterate over all the entries sequentially:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qdiriterator.html#QDirIterator">QDirIterator</a></span> it(<span class="string">"/etc"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdiriterator.html#QDirIterator">QDirIterator</a></span><span class="operator">::</span>Subdirectories);
///   <span class="keyword">while</span> (it<span class="operator">.</span>hasNext()) {
/// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>next();
///
/// &#32;     <span class="comment">// /etc/.</span>
/// &#32;     <span class="comment">// /etc/..</span>
/// &#32;     <span class="comment">// /etc/X11</span>
/// &#32;     <span class="comment">// /etc/X11/fs</span>
/// &#32;     <span class="comment">// ...</span>
///   }
///
/// </pre>
/// <p>Here's how to find and read all files filtered by name, recursively:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qdiriterator.html#QDirIterator">QDirIterator</a></span> it(<span class="string">"/sys"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"scaling_cur_freq"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdir.html">QDir</a></span><span class="operator">::</span>NoFilter<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdiriterator.html#QDirIterator">QDirIterator</a></span><span class="operator">::</span>Subdirectories);
///   <span class="keyword">while</span> (it<span class="operator">.</span>hasNext()) {
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qfile.html">QFile</a></span> f(it<span class="operator">.</span>next());
/// &#32;     f<span class="operator">.</span>open(<span class="type"><a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a></span><span class="operator">::</span>ReadOnly);
/// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> f<span class="operator">.</span>fileName() <span class="operator">&lt;</span><span class="operator">&lt;</span> f<span class="operator">.</span>readAll()<span class="operator">.</span>trimmed()<span class="operator">.</span>toDouble() <span class="operator">/</span> <span class="number">1000</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"MHz"</span>;
///   }
///
/// </pre>
/// <p>The <a href="http://doc.qt.io/qt-5/qdiriterator.html#next">next</a>() function returns the path to the next directory entry and advances the iterator. You can also call <a href="http://doc.qt.io/qt-5/qdiriterator.html#filePath">filePath</a>() to get the current file path without advancing the iterator. The <a href="http://doc.qt.io/qt-5/qdiriterator.html#fileName">fileName</a>() function returns only the name of the file, similar to how <a href="http://doc.qt.io/qt-5/qdir.html#entryList">QDir::entryList</a>() works. You can also call <a href="http://doc.qt.io/qt-5/qdiriterator.html#fileInfo">fileInfo</a>() to get a <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> for the current entry.</p>
/// <p>Unlike Qt's container iterators, <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a> is uni-directional (i.e., you cannot iterate directories in reverse order) and does not allow random access.</p></div>
#[repr(C)]
pub struct QDirIterator {
    _unused: u8,
}
impl QDirIterator {
    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> for the current directory entry.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFileInfo QDirIterator::fileInfo() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdiriterator.html#fileInfo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> for the current directory entry.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdiriterator.html#filePath">filePath</a>() and <a href="http://doc.qt.io/qt-5/qdiriterator.html#fileName">fileName</a>().</p></div>
    pub unsafe fn file_info(&self) -> ::cpp_utils::CppBox<crate::QFileInfo> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDirIterator_fileInfo(self as *const crate::QDirIterator);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the file name for the current directory entry, without the path prepended.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QDirIterator::fileName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdiriterator.html#fileName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the file name for the current directory entry, without the path prepended.</p>
    /// <p>This function is convenient when iterating a single directory. When using the <a href="http://doc.qt.io/qt-5/qdiriterator.html#IteratorFlag-enum">QDirIterator::Subdirectories</a> flag, you can use <a href="http://doc.qt.io/qt-5/qdiriterator.html#filePath">filePath</a>() to get the full path.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdiriterator.html#filePath">filePath</a>() and <a href="http://doc.qt.io/qt-5/qdiriterator.html#fileInfo">fileInfo</a>().</p></div>
    pub unsafe fn file_name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDirIterator_fileName(self as *const crate::QDirIterator);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the full file path for the current directory entry.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QDirIterator::filePath() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdiriterator.html#filePath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the full file path for the current directory entry.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdiriterator.html#fileInfo">fileInfo</a>() and <a href="http://doc.qt.io/qt-5/qdiriterator.html#fileName">fileName</a>().</p></div>
    pub unsafe fn file_path(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDirIterator_filePath(self as *const crate::QDirIterator);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if there is at least one more entry in the directory; otherwise, false is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDirIterator::hasNext() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdiriterator.html#hasNext">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if there is at least one more entry in the directory; otherwise, false is returned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdiriterator.html#next">next</a>(), <a href="http://doc.qt.io/qt-5/qdiriterator.html#fileName">fileName</a>(), <a href="http://doc.qt.io/qt-5/qdiriterator.html#filePath">filePath</a>(), and <a href="http://doc.qt.io/qt-5/qdiriterator.html#fileInfo">fileInfo</a>().</p></div>
    pub unsafe fn has_next(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QDirIterator_hasNext(self as *const crate::QDirIterator)
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a> that can iterate over <i>dir</i>'s entrylist, using <i>dir</i>'s name filters and regular filters. You can pass options via <i>flags</i> to decide how the directory should be iterated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDirIterator::QDirIterator(const QDir& dir, QFlags<QDirIterator::IteratorFlag> flags = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdiriterator.html#QDirIterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a> that can iterate over <i>dir</i>'s entrylist, using <i>dir</i>'s name filters and regular filters. You can pass options via <i>flags</i> to decide how the directory should be iterated.</p>
    /// <p>By default, <i>flags</i> is <a href="http://doc.qt.io/qt-5/qdiriterator.html#IteratorFlag-enum">NoIteratorFlags</a>, which provides the same behavior as in <a href="http://doc.qt.io/qt-5/qdir.html#entryList">QDir::entryList</a>().</p>
    /// <p>The sorting in <i>dir</i> is ignored.</p>
    /// <p><b>Note: </b>To list symlinks that point to non existing files, <a href="http://doc.qt.io/qt-5/qdir.html#Filter-enum">QDir::System</a> must be passed to the flags.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qdiriterator.html#hasNext">hasNext</a>(), <a href="http://doc.qt.io/qt-5/qdiriterator.html#next">next</a>(), and <a href="http://doc.qt.io/qt-5/qdiriterator.html#IteratorFlag-enum">IteratorFlags</a>.</p></div>
    pub unsafe fn from_q_dir_q_flags_iterator_flag(
        dir: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDir>>,
        flags: crate::QFlags<crate::q_dir_iterator::IteratorFlag>,
    ) -> ::cpp_utils::CppBox<crate::QDirIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDirIterator_QDirIterator(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDir>>::cast_into(dir).as_raw_ptr(),
            flags.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a> that can iterate over <i>path</i>. You can pass options via <i>flags</i> to decide how the directory should be iterated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDirIterator::QDirIterator(const QString& path, QFlags<QDirIterator::IteratorFlag> flags = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdiriterator.html#QDirIterator-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a> that can iterate over <i>path</i>. You can pass options via <i>flags</i> to decide how the directory should be iterated.</p>
    /// <p>By default, <i>flags</i> is <a href="http://doc.qt.io/qt-5/qdiriterator.html#IteratorFlag-enum">NoIteratorFlags</a>, which provides the same behavior as in <a href="http://doc.qt.io/qt-5/qdir.html#entryList">QDir::entryList</a>().</p>
    /// <p><b>Note: </b>To list symlinks that point to non existing files, <a href="http://doc.qt.io/qt-5/qdir.html#Filter-enum">QDir::System</a> must be passed to the flags.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qdiriterator.html#hasNext">hasNext</a>(), <a href="http://doc.qt.io/qt-5/qdiriterator.html#next">next</a>(), and <a href="http://doc.qt.io/qt-5/qdiriterator.html#IteratorFlag-enum">IteratorFlags</a>.</p></div>
    pub unsafe fn from_q_string_q_flags_iterator_flag(
        path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        flags: crate::QFlags<crate::q_dir_iterator::IteratorFlag>,
    ) -> ::cpp_utils::CppBox<crate::QDirIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDirIterator_QDirIterator1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(path).as_raw_ptr(),
            flags.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a> that can iterate over <i>path</i>, with no name filtering and <i>filters</i> for entry filtering. You can pass options via <i>flags</i> to decide how the directory should be iterated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDirIterator::QDirIterator(const QString& path, QFlags<QDir::Filter> filter, QFlags<QDirIterator::IteratorFlag> flags = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdiriterator.html#QDirIterator-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a> that can iterate over <i>path</i>, with no name filtering and <i>filters</i> for entry filtering. You can pass options via <i>flags</i> to decide how the directory should be iterated.</p>
    /// <p>By default, <i>filters</i> is <a href="http://doc.qt.io/qt-5/qdir.html#Filter-enum">QDir::NoFilter</a>, and <i>flags</i> is <a href="http://doc.qt.io/qt-5/qdiriterator.html#IteratorFlag-enum">NoIteratorFlags</a>, which provides the same behavior as in <a href="http://doc.qt.io/qt-5/qdir.html#entryList">QDir::entryList</a>().</p>
    /// <p><b>Note: </b>To list symlinks that point to non existing files, <a href="http://doc.qt.io/qt-5/qdir.html#Filter-enum">QDir::System</a> must be passed to the flags.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qdiriterator.html#hasNext">hasNext</a>(), <a href="http://doc.qt.io/qt-5/qdiriterator.html#next">next</a>(), and <a href="http://doc.qt.io/qt-5/qdiriterator.html#IteratorFlag-enum">IteratorFlags</a>.</p></div>
    pub unsafe fn from_q_string_q_flags_filter_q_flags_iterator_flag(
        path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        filter: crate::QFlags<crate::q_dir::Filter>,
        flags: crate::QFlags<crate::q_dir_iterator::IteratorFlag>,
    ) -> ::cpp_utils::CppBox<crate::QDirIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDirIterator_QDirIterator2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(path).as_raw_ptr(),
            filter.to_int(),
            flags.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a> that can iterate over <i>path</i>, using <i>nameFilters</i> and <i>filters</i>. You can pass options via <i>flags</i> to decide how the directory should be iterated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDirIterator::QDirIterator(const QString& path, const QStringList& nameFilters, QFlags<QDir::Filter> filters = …, QFlags<QDirIterator::IteratorFlag> flags = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdiriterator.html#QDirIterator-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a> that can iterate over <i>path</i>, using <i>nameFilters</i> and <i>filters</i>. You can pass options via <i>flags</i> to decide how the directory should be iterated.</p>
    /// <p>By default, <i>flags</i> is <a href="http://doc.qt.io/qt-5/qdiriterator.html#IteratorFlag-enum">NoIteratorFlags</a>, which provides the same behavior as <a href="http://doc.qt.io/qt-5/qdir.html#entryList">QDir::entryList</a>().</p>
    /// <p><b>Note: </b>To list symlinks that point to non existing files, <a href="http://doc.qt.io/qt-5/qdir.html#Filter-enum">QDir::System</a> must be passed to the flags.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qdiriterator.html#hasNext">hasNext</a>(), <a href="http://doc.qt.io/qt-5/qdiriterator.html#next">next</a>(), and <a href="http://doc.qt.io/qt-5/qdiriterator.html#IteratorFlag-enum">IteratorFlags</a>.</p></div>
    pub unsafe fn from_q_string_q_string_list_q_flags_filter_q_flags_iterator_flag(
        path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        name_filters: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
        filters: crate::QFlags<crate::q_dir::Filter>,
        flags: crate::QFlags<crate::q_dir_iterator::IteratorFlag>,
    ) -> ::cpp_utils::CppBox<crate::QDirIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDirIterator_QDirIterator3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(path).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(name_filters)
                .as_raw_ptr(),
            filters.to_int(),
            flags.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a> that can iterate over <i>dir</i>'s entrylist, using <i>dir</i>'s name filters and regular filters. You can pass options via <i>flags</i> to decide how the directory should be iterated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDirIterator::QDirIterator(const QDir& dir)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdiriterator.html#QDirIterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a> that can iterate over <i>dir</i>'s entrylist, using <i>dir</i>'s name filters and regular filters. You can pass options via <i>flags</i> to decide how the directory should be iterated.</p>
    /// <p>By default, <i>flags</i> is <a href="http://doc.qt.io/qt-5/qdiriterator.html#IteratorFlag-enum">NoIteratorFlags</a>, which provides the same behavior as in <a href="http://doc.qt.io/qt-5/qdir.html#entryList">QDir::entryList</a>().</p>
    /// <p>The sorting in <i>dir</i> is ignored.</p>
    /// <p><b>Note: </b>To list symlinks that point to non existing files, <a href="http://doc.qt.io/qt-5/qdir.html#Filter-enum">QDir::System</a> must be passed to the flags.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qdiriterator.html#hasNext">hasNext</a>(), <a href="http://doc.qt.io/qt-5/qdiriterator.html#next">next</a>(), and <a href="http://doc.qt.io/qt-5/qdiriterator.html#IteratorFlag-enum">IteratorFlags</a>.</p></div>
    pub unsafe fn from_q_dir(
        dir: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDir>>,
    ) -> ::cpp_utils::CppBox<crate::QDirIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDirIterator_QDirIterator5(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDir>>::cast_into(dir).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a> that can iterate over <i>path</i>. You can pass options via <i>flags</i> to decide how the directory should be iterated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDirIterator::QDirIterator(const QString& path)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdiriterator.html#QDirIterator-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a> that can iterate over <i>path</i>. You can pass options via <i>flags</i> to decide how the directory should be iterated.</p>
    /// <p>By default, <i>flags</i> is <a href="http://doc.qt.io/qt-5/qdiriterator.html#IteratorFlag-enum">NoIteratorFlags</a>, which provides the same behavior as in <a href="http://doc.qt.io/qt-5/qdir.html#entryList">QDir::entryList</a>().</p>
    /// <p><b>Note: </b>To list symlinks that point to non existing files, <a href="http://doc.qt.io/qt-5/qdir.html#Filter-enum">QDir::System</a> must be passed to the flags.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qdiriterator.html#hasNext">hasNext</a>(), <a href="http://doc.qt.io/qt-5/qdiriterator.html#next">next</a>(), and <a href="http://doc.qt.io/qt-5/qdiriterator.html#IteratorFlag-enum">IteratorFlags</a>.</p></div>
    pub unsafe fn from_q_string(
        path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QDirIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDirIterator_QDirIterator6(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(path).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a> that can iterate over <i>path</i>, with no name filtering and <i>filters</i> for entry filtering. You can pass options via <i>flags</i> to decide how the directory should be iterated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDirIterator::QDirIterator(const QString& path, QFlags<QDir::Filter> filter)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdiriterator.html#QDirIterator-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a> that can iterate over <i>path</i>, with no name filtering and <i>filters</i> for entry filtering. You can pass options via <i>flags</i> to decide how the directory should be iterated.</p>
    /// <p>By default, <i>filters</i> is <a href="http://doc.qt.io/qt-5/qdir.html#Filter-enum">QDir::NoFilter</a>, and <i>flags</i> is <a href="http://doc.qt.io/qt-5/qdiriterator.html#IteratorFlag-enum">NoIteratorFlags</a>, which provides the same behavior as in <a href="http://doc.qt.io/qt-5/qdir.html#entryList">QDir::entryList</a>().</p>
    /// <p><b>Note: </b>To list symlinks that point to non existing files, <a href="http://doc.qt.io/qt-5/qdir.html#Filter-enum">QDir::System</a> must be passed to the flags.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qdiriterator.html#hasNext">hasNext</a>(), <a href="http://doc.qt.io/qt-5/qdiriterator.html#next">next</a>(), and <a href="http://doc.qt.io/qt-5/qdiriterator.html#IteratorFlag-enum">IteratorFlags</a>.</p></div>
    pub unsafe fn from_q_string_q_flags_filter(
        path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        filter: crate::QFlags<crate::q_dir::Filter>,
    ) -> ::cpp_utils::CppBox<crate::QDirIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDirIterator_QDirIterator7(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(path).as_raw_ptr(),
            filter.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a> that can iterate over <i>path</i>, using <i>nameFilters</i> and <i>filters</i>. You can pass options via <i>flags</i> to decide how the directory should be iterated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDirIterator::QDirIterator(const QString& path, const QStringList& nameFilters, QFlags<QDir::Filter> filters = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdiriterator.html#QDirIterator-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a> that can iterate over <i>path</i>, using <i>nameFilters</i> and <i>filters</i>. You can pass options via <i>flags</i> to decide how the directory should be iterated.</p>
    /// <p>By default, <i>flags</i> is <a href="http://doc.qt.io/qt-5/qdiriterator.html#IteratorFlag-enum">NoIteratorFlags</a>, which provides the same behavior as <a href="http://doc.qt.io/qt-5/qdir.html#entryList">QDir::entryList</a>().</p>
    /// <p><b>Note: </b>To list symlinks that point to non existing files, <a href="http://doc.qt.io/qt-5/qdir.html#Filter-enum">QDir::System</a> must be passed to the flags.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qdiriterator.html#hasNext">hasNext</a>(), <a href="http://doc.qt.io/qt-5/qdiriterator.html#next">next</a>(), and <a href="http://doc.qt.io/qt-5/qdiriterator.html#IteratorFlag-enum">IteratorFlags</a>.</p></div>
    pub unsafe fn from_q_string_q_string_list_q_flags_filter(
        path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        name_filters: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
        filters: crate::QFlags<crate::q_dir::Filter>,
    ) -> ::cpp_utils::CppBox<crate::QDirIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDirIterator_QDirIterator8(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(path).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(name_filters)
                .as_raw_ptr(),
            filters.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a> that can iterate over <i>path</i>, using <i>nameFilters</i> and <i>filters</i>. You can pass options via <i>flags</i> to decide how the directory should be iterated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QDirIterator::QDirIterator(const QString& path, const QStringList& nameFilters)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdiriterator.html#QDirIterator-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a> that can iterate over <i>path</i>, using <i>nameFilters</i> and <i>filters</i>. You can pass options via <i>flags</i> to decide how the directory should be iterated.</p>
    /// <p>By default, <i>flags</i> is <a href="http://doc.qt.io/qt-5/qdiriterator.html#IteratorFlag-enum">NoIteratorFlags</a>, which provides the same behavior as <a href="http://doc.qt.io/qt-5/qdir.html#entryList">QDir::entryList</a>().</p>
    /// <p><b>Note: </b>To list symlinks that point to non existing files, <a href="http://doc.qt.io/qt-5/qdir.html#Filter-enum">QDir::System</a> must be passed to the flags.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qdiriterator.html#hasNext">hasNext</a>(), <a href="http://doc.qt.io/qt-5/qdiriterator.html#next">next</a>(), and <a href="http://doc.qt.io/qt-5/qdiriterator.html#IteratorFlag-enum">IteratorFlags</a>.</p></div>
    pub unsafe fn from_q_string_q_string_list(
        path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        name_filters: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) -> ::cpp_utils::CppBox<crate::QDirIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QDirIterator_QDirIterator9(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(path).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(name_filters)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Advances the iterator to the next entry, and returns the file path of this new entry. If <a href="http://doc.qt.io/qt-5/qdiriterator.html#hasNext">hasNext</a>() returns <code>false</code>, this function does nothing, and returns an empty <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QDirIterator::next()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdiriterator.html#next">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator to the next entry, and returns the file path of this new entry. If <a href="http://doc.qt.io/qt-5/qdiriterator.html#hasNext">hasNext</a>() returns <code>false</code>, this function does nothing, and returns an empty <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p>You can call <a href="http://doc.qt.io/qt-5/qdiriterator.html#fileName">fileName</a>() or <a href="http://doc.qt.io/qt-5/qdiriterator.html#filePath">filePath</a>() to get the current entry file name or path, or <a href="http://doc.qt.io/qt-5/qdiriterator.html#fileInfo">fileInfo</a>() to get a <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> for the current entry.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdiriterator.html#hasNext">hasNext</a>(), <a href="http://doc.qt.io/qt-5/qdiriterator.html#fileName">fileName</a>(), <a href="http://doc.qt.io/qt-5/qdiriterator.html#filePath">filePath</a>(), and <a href="http://doc.qt.io/qt-5/qdiriterator.html#fileInfo">fileInfo</a>().</p></div>
    pub unsafe fn next(&mut self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDirIterator_next(self as *mut crate::QDirIterator);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the base directory of the iterator.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QDirIterator::path() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdiriterator.html#path">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the base directory of the iterator.</p></div>
    pub unsafe fn path(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QDirIterator_path(self as *const crate::QDirIterator);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_easing_curve {
    //! C++ type: <span style='color: green;'>```QEasingCurve```</span>

    /// <p>The type of easing curve.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QEasingCurve::Type```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The type of easing curve.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qeasingcurve.html#addCubicBezierSegment">addCubicBezierSegment</a>() and <a href="http://doc.qt.io/qt-5/qeasingcurve.html#addTCBSegment">addTCBSegment</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Type(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Type {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Type(value)
        }
    }

    impl From<Type> for ::std::os::raw::c_int {
        fn from(value: Type) -> Self {
            value.0
        }
    }

    impl Type {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Type {
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-linear.png" alt=""></p><p class="figCaption">Easing curve for a linear (t) function: velocity is constant.</p> (C++ enum variant: <span style='color: green;'>```Linear = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const Linear: crate::q_easing_curve::Type = crate::q_easing_curve::Type(0);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-inquad.png" alt=""></p><p class="figCaption">Easing curve for a quadratic (t^2) function: accelerating from zero velocity.</p> (C++ enum variant: <span style='color: green;'>```InQuad = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const InQuad: crate::q_easing_curve::Type = crate::q_easing_curve::Type(1);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-outquad.png" alt=""></p><p class="figCaption">Easing curve for a quadratic (t^2) function: decelerating to zero velocity.</p> (C++ enum variant: <span style='color: green;'>```OutQuad = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const OutQuad: crate::q_easing_curve::Type = crate::q_easing_curve::Type(2);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-inoutquad.png" alt=""></p><p class="figCaption">Easing curve for a quadratic (t^2) function: acceleration until halfway, then deceleration.</p> (C++ enum variant: <span style='color: green;'>```InOutQuad = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const InOutQuad: crate::q_easing_curve::Type = crate::q_easing_curve::Type(3);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-outinquad.png" alt=""></p><p class="figCaption">Easing curve for a quadratic (t^2) function: deceleration until halfway, then acceleration.</p> (C++ enum variant: <span style='color: green;'>```OutInQuad = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const OutInQuad: crate::q_easing_curve::Type = crate::q_easing_curve::Type(4);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-incubic.png" alt=""></p><p class="figCaption">Easing curve for a cubic (t^3) function: accelerating from zero velocity.</p> (C++ enum variant: <span style='color: green;'>```InCubic = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const InCubic: crate::q_easing_curve::Type = crate::q_easing_curve::Type(5);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-outcubic.png" alt=""></p><p class="figCaption">Easing curve for a cubic (t^3) function: decelerating to zero velocity.</p> (C++ enum variant: <span style='color: green;'>```OutCubic = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const OutCubic: crate::q_easing_curve::Type = crate::q_easing_curve::Type(6);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-inoutcubic.png" alt=""></p><p class="figCaption">Easing curve for a cubic (t^3) function: acceleration until halfway, then deceleration.</p> (C++ enum variant: <span style='color: green;'>```InOutCubic = 7```</span>)
        #[allow(non_upper_case_globals)]
        pub const InOutCubic: crate::q_easing_curve::Type = crate::q_easing_curve::Type(7);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-outincubic.png" alt=""></p><p class="figCaption">Easing curve for a cubic (t^3) function: deceleration until halfway, then acceleration.</p> (C++ enum variant: <span style='color: green;'>```OutInCubic = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const OutInCubic: crate::q_easing_curve::Type = crate::q_easing_curve::Type(8);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-inquart.png" alt=""></p><p class="figCaption">Easing curve for a quartic (t^4) function: accelerating from zero velocity.</p> (C++ enum variant: <span style='color: green;'>```InQuart = 9```</span>)
        #[allow(non_upper_case_globals)]
        pub const InQuart: crate::q_easing_curve::Type = crate::q_easing_curve::Type(9);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-outquart.png" alt=""></p><p class="figCaption">Easing curve for a quartic (t^4) function: decelerating to zero velocity.</p> (C++ enum variant: <span style='color: green;'>```OutQuart = 10```</span>)
        #[allow(non_upper_case_globals)]
        pub const OutQuart: crate::q_easing_curve::Type = crate::q_easing_curve::Type(10);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-inoutquart.png" alt=""></p><p class="figCaption">Easing curve for a quartic (t^4) function: acceleration until halfway, then deceleration.</p> (C++ enum variant: <span style='color: green;'>```InOutQuart = 11```</span>)
        #[allow(non_upper_case_globals)]
        pub const InOutQuart: crate::q_easing_curve::Type = crate::q_easing_curve::Type(11);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-outinquart.png" alt=""></p><p class="figCaption">Easing curve for a quartic (t^4) function: deceleration until halfway, then acceleration.</p> (C++ enum variant: <span style='color: green;'>```OutInQuart = 12```</span>)
        #[allow(non_upper_case_globals)]
        pub const OutInQuart: crate::q_easing_curve::Type = crate::q_easing_curve::Type(12);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-inquint.png" alt=""></p><p class="figCaption">Easing curve for a quintic (t^5) easing in: accelerating from zero velocity.</p> (C++ enum variant: <span style='color: green;'>```InQuint = 13```</span>)
        #[allow(non_upper_case_globals)]
        pub const InQuint: crate::q_easing_curve::Type = crate::q_easing_curve::Type(13);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-outquint.png" alt=""></p><p class="figCaption">Easing curve for a quintic (t^5) function: decelerating to zero velocity.</p> (C++ enum variant: <span style='color: green;'>```OutQuint = 14```</span>)
        #[allow(non_upper_case_globals)]
        pub const OutQuint: crate::q_easing_curve::Type = crate::q_easing_curve::Type(14);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-inoutquint.png" alt=""></p><p class="figCaption">Easing curve for a quintic (t^5) function: acceleration until halfway, then deceleration.</p> (C++ enum variant: <span style='color: green;'>```InOutQuint = 15```</span>)
        #[allow(non_upper_case_globals)]
        pub const InOutQuint: crate::q_easing_curve::Type = crate::q_easing_curve::Type(15);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-outinquint.png" alt=""></p><p class="figCaption">Easing curve for a quintic (t^5) function: deceleration until halfway, then acceleration.</p> (C++ enum variant: <span style='color: green;'>```OutInQuint = 16```</span>)
        #[allow(non_upper_case_globals)]
        pub const OutInQuint: crate::q_easing_curve::Type = crate::q_easing_curve::Type(16);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-insine.png" alt=""></p><p class="figCaption">Easing curve for a sinusoidal (sin(t)) function: accelerating from zero velocity.</p> (C++ enum variant: <span style='color: green;'>```InSine = 17```</span>)
        #[allow(non_upper_case_globals)]
        pub const InSine: crate::q_easing_curve::Type = crate::q_easing_curve::Type(17);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-outsine.png" alt=""></p><p class="figCaption">Easing curve for a sinusoidal (sin(t)) function: decelerating to zero velocity.</p> (C++ enum variant: <span style='color: green;'>```OutSine = 18```</span>)
        #[allow(non_upper_case_globals)]
        pub const OutSine: crate::q_easing_curve::Type = crate::q_easing_curve::Type(18);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-inoutsine.png" alt=""></p><p class="figCaption">Easing curve for a sinusoidal (sin(t)) function: acceleration until halfway, then deceleration.</p> (C++ enum variant: <span style='color: green;'>```InOutSine = 19```</span>)
        #[allow(non_upper_case_globals)]
        pub const InOutSine: crate::q_easing_curve::Type = crate::q_easing_curve::Type(19);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-outinsine.png" alt=""></p><p class="figCaption">Easing curve for a sinusoidal (sin(t)) function: deceleration until halfway, then acceleration.</p> (C++ enum variant: <span style='color: green;'>```OutInSine = 20```</span>)
        #[allow(non_upper_case_globals)]
        pub const OutInSine: crate::q_easing_curve::Type = crate::q_easing_curve::Type(20);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-inexpo.png" alt=""></p><p class="figCaption">Easing curve for an exponential (2^t) function: accelerating from zero velocity.</p> (C++ enum variant: <span style='color: green;'>```InExpo = 21```</span>)
        #[allow(non_upper_case_globals)]
        pub const InExpo: crate::q_easing_curve::Type = crate::q_easing_curve::Type(21);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-outexpo.png" alt=""></p><p class="figCaption">Easing curve for an exponential (2^t) function: decelerating to zero velocity.</p> (C++ enum variant: <span style='color: green;'>```OutExpo = 22```</span>)
        #[allow(non_upper_case_globals)]
        pub const OutExpo: crate::q_easing_curve::Type = crate::q_easing_curve::Type(22);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-inoutexpo.png" alt=""></p><p class="figCaption">Easing curve for an exponential (2^t) function: acceleration until halfway, then deceleration.</p> (C++ enum variant: <span style='color: green;'>```InOutExpo = 23```</span>)
        #[allow(non_upper_case_globals)]
        pub const InOutExpo: crate::q_easing_curve::Type = crate::q_easing_curve::Type(23);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-outinexpo.png" alt=""></p><p class="figCaption">Easing curve for an exponential (2^t) function: deceleration until halfway, then acceleration.</p> (C++ enum variant: <span style='color: green;'>```OutInExpo = 24```</span>)
        #[allow(non_upper_case_globals)]
        pub const OutInExpo: crate::q_easing_curve::Type = crate::q_easing_curve::Type(24);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-incirc.png" alt=""></p><p class="figCaption">Easing curve for a circular (sqrt(1-t^2)) function: accelerating from zero velocity.</p> (C++ enum variant: <span style='color: green;'>```InCirc = 25```</span>)
        #[allow(non_upper_case_globals)]
        pub const InCirc: crate::q_easing_curve::Type = crate::q_easing_curve::Type(25);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-outcirc.png" alt=""></p><p class="figCaption">Easing curve for a circular (sqrt(1-t^2)) function: decelerating to zero velocity.</p> (C++ enum variant: <span style='color: green;'>```OutCirc = 26```</span>)
        #[allow(non_upper_case_globals)]
        pub const OutCirc: crate::q_easing_curve::Type = crate::q_easing_curve::Type(26);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-inoutcirc.png" alt=""></p><p class="figCaption">Easing curve for a circular (sqrt(1-t^2)) function: acceleration until halfway, then deceleration.</p> (C++ enum variant: <span style='color: green;'>```InOutCirc = 27```</span>)
        #[allow(non_upper_case_globals)]
        pub const InOutCirc: crate::q_easing_curve::Type = crate::q_easing_curve::Type(27);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-outincirc.png" alt=""></p><p class="figCaption">Easing curve for a circular (sqrt(1-t^2)) function: deceleration until halfway, then acceleration.</p> (C++ enum variant: <span style='color: green;'>```OutInCirc = 28```</span>)
        #[allow(non_upper_case_globals)]
        pub const OutInCirc: crate::q_easing_curve::Type = crate::q_easing_curve::Type(28);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-inelastic.png" alt=""></p><p class="figCaption">Easing curve for an elastic (exponentially decaying sine wave) function: accelerating from zero velocity. The peak amplitude can be set with the <i>amplitude</i> parameter, and the period of decay by the <i>period</i> parameter.</p> (C++ enum variant: <span style='color: green;'>```InElastic = 29```</span>)
        #[allow(non_upper_case_globals)]
        pub const InElastic: crate::q_easing_curve::Type = crate::q_easing_curve::Type(29);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-outelastic.png" alt=""></p><p class="figCaption">Easing curve for an elastic (exponentially decaying sine wave) function: decelerating to zero velocity. The peak amplitude can be set with the <i>amplitude</i> parameter, and the period of decay by the <i>period</i> parameter.</p> (C++ enum variant: <span style='color: green;'>```OutElastic = 30```</span>)
        #[allow(non_upper_case_globals)]
        pub const OutElastic: crate::q_easing_curve::Type = crate::q_easing_curve::Type(30);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-inoutelastic.png" alt=""></p><p class="figCaption">Easing curve for an elastic (exponentially decaying sine wave) function: acceleration until halfway, then deceleration.</p> (C++ enum variant: <span style='color: green;'>```InOutElastic = 31```</span>)
        #[allow(non_upper_case_globals)]
        pub const InOutElastic: crate::q_easing_curve::Type = crate::q_easing_curve::Type(31);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-outinelastic.png" alt=""></p><p class="figCaption">Easing curve for an elastic (exponentially decaying sine wave) function: deceleration until halfway, then acceleration.</p> (C++ enum variant: <span style='color: green;'>```OutInElastic = 32```</span>)
        #[allow(non_upper_case_globals)]
        pub const OutInElastic: crate::q_easing_curve::Type = crate::q_easing_curve::Type(32);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-inback.png" alt=""></p><p class="figCaption">Easing curve for a back (overshooting cubic function: (s+1)*t^3 - s*t^2) easing in: accelerating from zero velocity.</p> (C++ enum variant: <span style='color: green;'>```InBack = 33```</span>)
        #[allow(non_upper_case_globals)]
        pub const InBack: crate::q_easing_curve::Type = crate::q_easing_curve::Type(33);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-outback.png" alt=""></p><p class="figCaption">Easing curve for a back (overshooting cubic function: (s+1)*t^3 - s*t^2) easing out: decelerating to zero velocity.</p> (C++ enum variant: <span style='color: green;'>```OutBack = 34```</span>)
        #[allow(non_upper_case_globals)]
        pub const OutBack: crate::q_easing_curve::Type = crate::q_easing_curve::Type(34);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-inoutback.png" alt=""></p><p class="figCaption">Easing curve for a back (overshooting cubic function: (s+1)*t^3 - s*t^2) easing in/out: acceleration until halfway, then deceleration.</p> (C++ enum variant: <span style='color: green;'>```InOutBack = 35```</span>)
        #[allow(non_upper_case_globals)]
        pub const InOutBack: crate::q_easing_curve::Type = crate::q_easing_curve::Type(35);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-outinback.png" alt=""></p><p class="figCaption">Easing curve for a back (overshooting cubic easing: (s+1)*t^3 - s*t^2) easing out/in: deceleration until halfway, then acceleration.</p> (C++ enum variant: <span style='color: green;'>```OutInBack = 36```</span>)
        #[allow(non_upper_case_globals)]
        pub const OutInBack: crate::q_easing_curve::Type = crate::q_easing_curve::Type(36);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-inbounce.png" alt=""></p><p class="figCaption">Easing curve for a bounce (exponentially decaying parabolic bounce) function: accelerating from zero velocity.</p> (C++ enum variant: <span style='color: green;'>```InBounce = 37```</span>)
        #[allow(non_upper_case_globals)]
        pub const InBounce: crate::q_easing_curve::Type = crate::q_easing_curve::Type(37);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-outbounce.png" alt=""></p><p class="figCaption">Easing curve for a bounce (exponentially decaying parabolic bounce) function: decelerating from zero velocity.</p> (C++ enum variant: <span style='color: green;'>```OutBounce = 38```</span>)
        #[allow(non_upper_case_globals)]
        pub const OutBounce: crate::q_easing_curve::Type = crate::q_easing_curve::Type(38);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-inoutbounce.png" alt=""></p><p class="figCaption">Easing curve for a bounce (exponentially decaying parabolic bounce) function easing in/out: acceleration until halfway, then deceleration.</p> (C++ enum variant: <span style='color: green;'>```InOutBounce = 39```</span>)
        #[allow(non_upper_case_globals)]
        pub const InOutBounce: crate::q_easing_curve::Type = crate::q_easing_curve::Type(39);
        /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qeasingcurve-outinbounce.png" alt=""></p><p class="figCaption">Easing curve for a bounce (exponentially decaying parabolic bounce) function easing out/in: deceleration until halfway, then acceleration.</p> (C++ enum variant: <span style='color: green;'>```OutInBounce = 40```</span>)
        #[allow(non_upper_case_globals)]
        pub const OutInBounce: crate::q_easing_curve::Type = crate::q_easing_curve::Type(40);
        /// C++ enum variant: <span style='color: green;'>```InCurve = 41```</span>
        #[allow(non_upper_case_globals)]
        pub const InCurve: crate::q_easing_curve::Type = crate::q_easing_curve::Type(41);
        /// C++ enum variant: <span style='color: green;'>```OutCurve = 42```</span>
        #[allow(non_upper_case_globals)]
        pub const OutCurve: crate::q_easing_curve::Type = crate::q_easing_curve::Type(42);
        /// C++ enum variant: <span style='color: green;'>```SineCurve = 43```</span>
        #[allow(non_upper_case_globals)]
        pub const SineCurve: crate::q_easing_curve::Type = crate::q_easing_curve::Type(43);
        /// C++ enum variant: <span style='color: green;'>```CosineCurve = 44```</span>
        #[allow(non_upper_case_globals)]
        pub const CosineCurve: crate::q_easing_curve::Type = crate::q_easing_curve::Type(44);
        /// Allows defining a custom easing curve using a cubic bezier spline (C++ enum variant: <span style='color: green;'>```BezierSpline = 45```</span>)
        #[allow(non_upper_case_globals)]
        pub const BezierSpline: crate::q_easing_curve::Type = crate::q_easing_curve::Type(45);
        /// Allows defining a custom easing curve using a TCB spline (C++ enum variant: <span style='color: green;'>```TCBSpline = 46```</span>)
        #[allow(non_upper_case_globals)]
        pub const TCBSpline: crate::q_easing_curve::Type = crate::q_easing_curve::Type(46);
        /// This is returned if the user specified a custom curve type with <a href="http://doc.qt.io/qt-5/qeasingcurve.html#setCustomType">setCustomType</a>(). Note that you cannot call <a href="http://doc.qt.io/qt-5/qeasingcurve.html#setType">setType</a>() with this value, but <a href="http://doc.qt.io/qt-5/qeasingcurve.html#type">type</a>() can return it. (C++ enum variant: <span style='color: green;'>```Custom = 47```</span>)
        #[allow(non_upper_case_globals)]
        pub const Custom: crate::q_easing_curve::Type = crate::q_easing_curve::Type(47);
        /// C++ enum variant: <span style='color: green;'>```NCurveTypes = 48```</span>
        #[allow(non_upper_case_globals)]
        pub const NCurveTypes: crate::q_easing_curve::Type = crate::q_easing_curve::Type(48);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qeasingcurve.html">QEasingCurve</a> class provides easing curves for controlling animation.</p>
///
/// C++ class: <span style='color: green;'>```QEasingCurve```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qeasingcurve.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qeasingcurve.html">QEasingCurve</a> class provides easing curves for controlling animation.</p>
/// <p>Easing curves describe a function that controls how the speed of the interpolation between 0 and 1 should be. Easing curves allow transitions from one value to another to appear more natural than a simple constant speed would allow. The <a href="http://doc.qt.io/qt-5/qeasingcurve.html">QEasingCurve</a> class is usually used in conjunction with the <a href="http://doc.qt.io/qt-5/qvariantanimation.html">QVariantAnimation</a> and <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a> classes but can be used on its own. It is usually used to accelerate the interpolation from zero velocity (ease in) or decelerate to zero velocity (ease out). Ease in and ease out can also be combined in the same easing curve.</p>
/// <p>To calculate the speed of the interpolation, the easing curve provides the function <a href="http://doc.qt.io/qt-5/qeasingcurve.html#valueForProgress">valueForProgress</a>(), where the <i>progress</i> argument specifies the progress of the interpolation: 0 is the start value of the interpolation, 1 is the end value of the interpolation. The returned value is the effective progress of the interpolation. If the returned value is the same as the input value for all input values the easing curve is a linear curve. This is the default behaviour.</p>
/// <p>For example,</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qeasingcurve.html#QEasingCurve">QEasingCurve</a></span> easing(<span class="type"><a href="http://doc.qt.io/qt-5/qeasingcurve.html#QEasingCurve">QEasingCurve</a></span><span class="operator">::</span>InOutQuad);
///
///   <span class="keyword">for</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#qreal-typedef">qreal</a></span> t <span class="operator">=</span> <span class="number">0.0</span>; t <span class="operator">&lt;</span> <span class="number">1.0</span>; t<span class="operator">+</span><span class="operator">=</span><span class="number">0.1</span>)
/// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qWarning">qWarning</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Effective progress"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> t <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">" is
/// &#32;   &#32;   &#32;   &#32;    &lt;&lt; easing.valueForProgress(t);
///   </span>
///
/// </pre>
/// <p>will print the effective progress of the interpolation between 0 and 1.</p>
/// <p>When using a <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a>, the associated easing curve will be used to control the progress of the interpolation between startValue and endValue:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a></span> animation;
///   animation<span class="operator">.</span>setStartValue(<span class="number">0</span>);
///   animation<span class="operator">.</span>setEndValue(<span class="number">1000</span>);
///   animation<span class="operator">.</span>setDuration(<span class="number">1000</span>);
///   animation<span class="operator">.</span>setEasingCurve(<span class="type"><a href="http://doc.qt.io/qt-5/qeasingcurve.html#QEasingCurve">QEasingCurve</a></span><span class="operator">::</span>InOutQuad);
///
/// </pre>
/// <p>The ability to set an amplitude, overshoot, or period depends on the <a href="http://doc.qt.io/qt-5/qeasingcurve.html">QEasingCurve</a> type. Amplitude access is available to curves that behave as springs such as elastic and bounce curves. Changing the amplitude changes the height of the curve. Period access is only available to elastic curves and setting a higher period slows the rate of bounce. Only curves that have "boomerang" behaviors such as the <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">InBack</a>, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">OutBack</a>, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">InOutBack</a>, and <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">OutInBack</a> have overshoot settings. These curves will interpolate beyond the end points and return to the end point, acting similar to a boomerang.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qtwidgets-animation-easing-example.html">Easing Curves Example</a> contains samples of <a href="http://doc.qt.io/qt-5/qeasingcurve.html">QEasingCurve</a> types and lets you change the curve settings.</p></div>
#[repr(C)]
pub struct QEasingCurve {
    _unused: u8,
}
impl QEasingCurve {
    /// <p>Adds a segment of a cubic bezier spline to define a custom easing curve. It is only applicable if <a href="http://doc.qt.io/qt-5/qeasingcurve.html#type">type</a>() is <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::BezierSpline</a>. Note that the spline implicitly starts at (0.0, 0.0) and has to end at (1.0, 1.0) to be a valid easing curve. <i>c1</i> and <i>c2</i> are the control points used for drawing the curve. <i>endPoint</i> is the endpoint of the curve.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QEasingCurve::addCubicBezierSegment(const QPointF& c1, const QPointF& c2, const QPointF& endPoint)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeasingcurve.html#addCubicBezierSegment">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds a segment of a cubic bezier spline to define a custom easing curve. It is only applicable if <a href="http://doc.qt.io/qt-5/qeasingcurve.html#type">type</a>() is <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::BezierSpline</a>. Note that the spline implicitly starts at (0.0, 0.0) and has to end at (1.0, 1.0) to be a valid easing curve. <i>c1</i> and <i>c2</i> are the control points used for drawing the curve. <i>endPoint</i> is the endpoint of the curve.</p></div>
    pub unsafe fn add_cubic_bezier_segment(
        &mut self,
        c1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
        c2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
        end_point: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QEasingCurve_addCubicBezierSegment(
            self as *mut crate::QEasingCurve,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(c1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(c2).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(end_point)
                .as_raw_ptr(),
        )
    }

    /// <p>Adds a segment of a TCB bezier spline to define a custom easing curve. It is only applicable if <a href="http://doc.qt.io/qt-5/qeasingcurve.html#type">type</a>() is <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::TCBSpline</a>. The spline has to start explitly at (0.0, 0.0) and has to end at (1.0, 1.0) to be a valid easing curve. The tension <i>t</i> changes the length of the tangent vector. The continuity <i>c</i> changes the sharpness in change between the tangents. The bias <i>b</i> changes the direction of the tangent vector. <i>nextPoint</i> is the sample position. All three parameters are valid between -1 and 1 and define the tangent of the control point. If all three parameters are 0 the resulting spline is a Catmull-Rom spline. The begin and endpoint always have a bias of -1 and 1, since the outer tangent is not defined.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QEasingCurve::addTCBSegment(const QPointF& nextPoint, double t, double c, double b)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeasingcurve.html#addTCBSegment">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds a segment of a TCB bezier spline to define a custom easing curve. It is only applicable if <a href="http://doc.qt.io/qt-5/qeasingcurve.html#type">type</a>() is <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::TCBSpline</a>. The spline has to start explitly at (0.0, 0.0) and has to end at (1.0, 1.0) to be a valid easing curve. The tension <i>t</i> changes the length of the tangent vector. The continuity <i>c</i> changes the sharpness in change between the tangents. The bias <i>b</i> changes the direction of the tangent vector. <i>nextPoint</i> is the sample position. All three parameters are valid between -1 and 1 and define the tangent of the control point. If all three parameters are 0 the resulting spline is a Catmull-Rom spline. The begin and endpoint always have a bias of -1 and 1, since the outer tangent is not defined.</p></div>
    pub unsafe fn add_t_c_b_segment(
        &mut self,
        next_point: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
        t: ::std::os::raw::c_double,
        c: ::std::os::raw::c_double,
        b: ::std::os::raw::c_double,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QEasingCurve_addTCBSegment(
            self as *mut crate::QEasingCurve,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(next_point)
                .as_raw_ptr(),
            t,
            c,
            b,
        )
    }

    /// <p>Returns the amplitude. This is not applicable for all curve types. It is only applicable for bounce and elastic curves (curves of <a href="http://doc.qt.io/qt-5/qeasingcurve.html#type">type</a>() <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::InBounce</a>, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::OutBounce</a>, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::InOutBounce</a>, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::OutInBounce</a>, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::InElastic</a>, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::OutElastic</a>, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::InOutElastic</a> or <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::OutInElastic</a>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QEasingCurve::amplitude() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeasingcurve.html#amplitude">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the amplitude. This is not applicable for all curve types. It is only applicable for bounce and elastic curves (curves of <a href="http://doc.qt.io/qt-5/qeasingcurve.html#type">type</a>() <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::InBounce</a>, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::OutBounce</a>, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::InOutBounce</a>, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::OutInBounce</a>, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::InElastic</a>, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::OutElastic</a>, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::InOutElastic</a> or <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::OutInElastic</a>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qeasingcurve.html#setAmplitude">setAmplitude</a>().</p></div>
    pub unsafe fn amplitude(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QEasingCurve_amplitude(self as *const crate::QEasingCurve)
    }

    /// <p>Copy <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QEasingCurve& QEasingCurve::operator=(const QEasingCurve& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeasingcurve.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Copy <i>other</i>.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QEasingCurve>>,
    ) -> ::cpp_utils::MutRef<crate::QEasingCurve> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEasingCurve_operator_(
            self as *mut crate::QEasingCurve,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QEasingCurve>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the function pointer to the custom easing curve. If <a href="http://doc.qt.io/qt-5/qeasingcurve.html#type">type</a>() does not return <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::Custom</a>, this function will return 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double (*FN_PTR)(double) QEasingCurve::customType() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeasingcurve.html#customType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the function pointer to the custom easing curve. If <a href="http://doc.qt.io/qt-5/qeasingcurve.html#type">type</a>() does not return <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::Custom</a>, this function will return 0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qeasingcurve.html#setCustomType">setCustomType</a>().</p></div>
    pub unsafe fn custom_type(
        &self,
    ) -> ::std::option::Option<extern "C" fn(::std::os::raw::c_double) -> ::std::os::raw::c_double>
    {
        crate::__ffi::ctr_qt_core_ffi_QEasingCurve_customType(self as *const crate::QEasingCurve)
    }

    /// <p>Constructs an easing curve of the given <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QEasingCurve::QEasingCurve(QEasingCurve::Type type = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeasingcurve.html#QEasingCurve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an easing curve of the given <i>type</i>.</p></div>
    pub unsafe fn new_1a(
        type_: crate::q_easing_curve::Type,
    ) -> ::cpp_utils::CppBox<crate::QEasingCurve> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEasingCurve_QEasingCurve(type_);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qeasingcurve.html">QEasingCurve</a> class provides easing curves for controlling animation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QEasingCurve::QEasingCurve()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeasingcurve.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qeasingcurve.html">QEasingCurve</a> class provides easing curves for controlling animation.</p>
    /// <p>Easing curves describe a function that controls how the speed of the interpolation between 0 and 1 should be. Easing curves allow transitions from one value to another to appear more natural than a simple constant speed would allow. The <a href="http://doc.qt.io/qt-5/qeasingcurve.html">QEasingCurve</a> class is usually used in conjunction with the <a href="http://doc.qt.io/qt-5/qvariantanimation.html">QVariantAnimation</a> and <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a> classes but can be used on its own. It is usually used to accelerate the interpolation from zero velocity (ease in) or decelerate to zero velocity (ease out). Ease in and ease out can also be combined in the same easing curve.</p>
    /// <p>To calculate the speed of the interpolation, the easing curve provides the function <a href="http://doc.qt.io/qt-5/qeasingcurve.html#valueForProgress">valueForProgress</a>(), where the <i>progress</i> argument specifies the progress of the interpolation: 0 is the start value of the interpolation, 1 is the end value of the interpolation. The returned value is the effective progress of the interpolation. If the returned value is the same as the input value for all input values the easing curve is a linear curve. This is the default behaviour.</p>
    /// <p>For example,</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qeasingcurve.html#QEasingCurve">QEasingCurve</a></span> easing(<span class="type"><a href="http://doc.qt.io/qt-5/qeasingcurve.html#QEasingCurve">QEasingCurve</a></span><span class="operator">::</span>InOutQuad);
    ///
    ///   <span class="keyword">for</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#qreal-typedef">qreal</a></span> t <span class="operator">=</span> <span class="number">0.0</span>; t <span class="operator">&lt;</span> <span class="number">1.0</span>; t<span class="operator">+</span><span class="operator">=</span><span class="number">0.1</span>)
    /// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qWarning">qWarning</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Effective progress"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> t <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">" is
    /// &#32;   &#32;   &#32;   &#32;    &lt;&lt; easing.valueForProgress(t);
    ///   </span>
    ///
    /// </pre>
    /// <p>will print the effective progress of the interpolation between 0 and 1.</p>
    /// <p>When using a <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a>, the associated easing curve will be used to control the progress of the interpolation between startValue and endValue:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a></span> animation;
    ///   animation<span class="operator">.</span>setStartValue(<span class="number">0</span>);
    ///   animation<span class="operator">.</span>setEndValue(<span class="number">1000</span>);
    ///   animation<span class="operator">.</span>setDuration(<span class="number">1000</span>);
    ///   animation<span class="operator">.</span>setEasingCurve(<span class="type"><a href="http://doc.qt.io/qt-5/qeasingcurve.html#QEasingCurve">QEasingCurve</a></span><span class="operator">::</span>InOutQuad);
    ///
    /// </pre>
    /// <p>The ability to set an amplitude, overshoot, or period depends on the <a href="http://doc.qt.io/qt-5/qeasingcurve.html">QEasingCurve</a> type. Amplitude access is available to curves that behave as springs such as elastic and bounce curves. Changing the amplitude changes the height of the curve. Period access is only available to elastic curves and setting a higher period slows the rate of bounce. Only curves that have "boomerang" behaviors such as the <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">InBack</a>, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">OutBack</a>, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">InOutBack</a>, and <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">OutInBack</a> have overshoot settings. These curves will interpolate beyond the end points and return to the end point, acting similar to a boomerang.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qtwidgets-animation-easing-example.html">Easing Curves Example</a> contains samples of <a href="http://doc.qt.io/qt-5/qeasingcurve.html">QEasingCurve</a> types and lets you change the curve settings.</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QEasingCurve> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEasingCurve_QEasingCurve3();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Construct a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QEasingCurve::QEasingCurve(const QEasingCurve& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeasingcurve.html#QEasingCurve-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Construct a copy of <i>other</i>.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QEasingCurve>>,
    ) -> ::cpp_utils::CppBox<crate::QEasingCurve> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEasingCurve_QEasingCurve1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QEasingCurve>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the overshoot. This is not applicable for all curve types. It is only applicable if <a href="http://doc.qt.io/qt-5/qeasingcurve.html#type">type</a>() is <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::InBack</a>, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::OutBack</a>, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::InOutBack</a> or <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::OutInBack</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QEasingCurve::overshoot() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeasingcurve.html#overshoot">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the overshoot. This is not applicable for all curve types. It is only applicable if <a href="http://doc.qt.io/qt-5/qeasingcurve.html#type">type</a>() is <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::InBack</a>, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::OutBack</a>, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::InOutBack</a> or <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::OutInBack</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qeasingcurve.html#setOvershoot">setOvershoot</a>().</p></div>
    pub unsafe fn overshoot(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QEasingCurve_overshoot(self as *const crate::QEasingCurve)
    }

    /// <p>Returns the period. This is not applicable for all curve types. It is only applicable if <a href="http://doc.qt.io/qt-5/qeasingcurve.html#type">type</a>() is <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::InElastic</a>, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::OutElastic</a>, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::InOutElastic</a> or <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::OutInElastic</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QEasingCurve::period() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeasingcurve.html#period">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the period. This is not applicable for all curve types. It is only applicable if <a href="http://doc.qt.io/qt-5/qeasingcurve.html#type">type</a>() is <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::InElastic</a>, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::OutElastic</a>, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::InOutElastic</a> or <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::OutInElastic</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qeasingcurve.html#setPeriod">setPeriod</a>().</p></div>
    pub unsafe fn period(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QEasingCurve_period(self as *const crate::QEasingCurve)
    }

    /// <p>Sets the amplitude to <i>amplitude</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QEasingCurve::setAmplitude(double amplitude)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeasingcurve.html#setAmplitude">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the amplitude to <i>amplitude</i>.</p>
    /// <p>This will set the amplitude of the bounce or the amplitude of the elastic "spring" effect. The higher the number, the higher the amplitude.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qeasingcurve.html#amplitude">amplitude</a>().</p></div>
    pub unsafe fn set_amplitude(&mut self, amplitude: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_QEasingCurve_setAmplitude(
            self as *mut crate::QEasingCurve,
            amplitude,
        )
    }

    /// <p>Sets a custom easing curve that is defined by the user in the function <i>func</i>. The signature of the function is qreal myEasingFunction(qreal progress), where <i>progress</i> and the return value are considered to be normalized between 0 and 1. (In some cases the return value can be outside that range) After calling this function <a href="http://doc.qt.io/qt-5/qeasingcurve.html#type">type</a>() will return <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::Custom</a>. <i>func</i> cannot be zero.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QEasingCurve::setCustomType(double (*FN_PTR)(double) func)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeasingcurve.html#setCustomType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets a custom easing curve that is defined by the user in the function <i>func</i>. The signature of the function is qreal myEasingFunction(qreal progress), where <i>progress</i> and the return value are considered to be normalized between 0 and 1. (In some cases the return value can be outside that range) After calling this function <a href="http://doc.qt.io/qt-5/qeasingcurve.html#type">type</a>() will return <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::Custom</a>. <i>func</i> cannot be zero.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qeasingcurve.html#customType">customType</a>() and <a href="http://doc.qt.io/qt-5/qeasingcurve.html#valueForProgress">valueForProgress</a>().</p></div>
    pub unsafe fn set_custom_type(
        &mut self,
        func: ::std::option::Option<
            extern "C" fn(::std::os::raw::c_double) -> ::std::os::raw::c_double,
        >,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QEasingCurve_setCustomType(
            self as *mut crate::QEasingCurve,
            func,
        )
    }

    /// <p>Sets the overshoot to <i>overshoot</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QEasingCurve::setOvershoot(double overshoot)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeasingcurve.html#setOvershoot">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the overshoot to <i>overshoot</i>.</p>
    /// <p>0 produces no overshoot, and the default value of 1.70158 produces an overshoot of 10 percent.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qeasingcurve.html#overshoot">overshoot</a>().</p></div>
    pub unsafe fn set_overshoot(&mut self, overshoot: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_QEasingCurve_setOvershoot(
            self as *mut crate::QEasingCurve,
            overshoot,
        )
    }

    /// <p>Sets the period to <i>period</i>. Setting a small period value will give a high frequency of the curve. A large period will give it a small frequency.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QEasingCurve::setPeriod(double period)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeasingcurve.html#setPeriod">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the period to <i>period</i>. Setting a small period value will give a high frequency of the curve. A large period will give it a small frequency.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qeasingcurve.html#period">period</a>().</p></div>
    pub unsafe fn set_period(&mut self, period: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_QEasingCurve_setPeriod(
            self as *mut crate::QEasingCurve,
            period,
        )
    }

    /// <p>Sets the type of the easing curve to <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QEasingCurve::setType(QEasingCurve::Type type)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeasingcurve.html#setType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the type of the easing curve to <i>type</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qeasingcurve.html#type">type</a>().</p></div>
    pub unsafe fn set_type(&mut self, type_: crate::q_easing_curve::Type) {
        crate::__ffi::ctr_qt_core_ffi_QEasingCurve_setType(self as *mut crate::QEasingCurve, type_)
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEasingCurve_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Swaps curve <i>other</i> with this curve. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QEasingCurve::swap(QEasingCurve& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeasingcurve.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps curve <i>other</i> with this curve. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QEasingCurve>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QEasingCurve_swap(
            self as *mut crate::QEasingCurve,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QEasingCurve>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the cubicBezierSpline that defines a custom easing curve. If the easing curve does not have a custom bezier easing curve the list is empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QPointF> QEasingCurve::toCubicSpline() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeasingcurve.html#toCubicSpline">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the cubicBezierSpline that defines a custom easing curve. If the easing curve does not have a custom bezier easing curve the list is empty.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn to_cubic_spline(&self) -> ::cpp_utils::CppBox<crate::QVectorOfQPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEasingCurve_toCubicSpline(
            self as *const crate::QEasingCurve,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the type of the easing curve.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QEasingCurve::Type QEasingCurve::type() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeasingcurve.html#type">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the type of the easing curve.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qeasingcurve.html#setType">setType</a>().</p></div>
    pub unsafe fn type_(&self) -> crate::q_easing_curve::Type {
        crate::__ffi::ctr_qt_core_ffi_QEasingCurve_type(self as *const crate::QEasingCurve)
    }

    /// <p>Return the effective progress for the easing curve at <i>progress</i>. Whereas <i>progress</i> must be between 0 and 1, the returned effective progress can be outside those bounds. For example, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::InBack</a> will return negative values in the beginning of the function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QEasingCurve::valueForProgress(double progress) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeasingcurve.html#valueForProgress">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Return the effective progress for the easing curve at <i>progress</i>. Whereas <i>progress</i> must be between 0 and 1, the returned effective progress can be outside those bounds. For example, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::InBack</a> will return negative values in the beginning of the function.</p></div>
    pub unsafe fn value_for_progress(
        &self,
        progress: ::std::os::raw::c_double,
    ) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QEasingCurve_valueForProgress(
            self as *const crate::QEasingCurve,
            progress,
        )
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qeventtransition.html">QEventTransition</a> class provides a <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>-specific transition for Qt events.</p>
///
/// C++ class: <span style='color: green;'>```QEventTransition```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qeventtransition.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qeventtransition.html">QEventTransition</a> class provides a <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>-specific transition for Qt events.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qeventtransition.html">QEventTransition</a> object binds an event to a particular <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>. <a href="http://doc.qt.io/qt-5/qeventtransition.html">QEventTransition</a> is part of <a href="http://doc.qt.io/qt-5/statemachine-api.html">The State Machine Framework</a>.</p>
/// <p>Example:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span>button <span class="operator">=</span> <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span> <span class="operator">*</span>s1 <span class="operator">=</span> <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span> <span class="operator">*</span>s2 <span class="operator">=</span> <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>;
///   <span class="comment">// If in s1 and the button receives an Enter event, transition to s2</span>
///   <span class="type"><a href="http://doc.qt.io/qt-5/qeventtransition.html#QEventTransition">QEventTransition</a></span> <span class="operator">*</span>enterTransition <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qeventtransition.html#QEventTransition">QEventTransition</a></span>(button<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a></span><span class="operator">::</span>Enter);
///   enterTransition<span class="operator">-</span><span class="operator">&gt;</span>setTargetState(s2);
///   s1<span class="operator">-</span><span class="operator">&gt;</span>addTransition(enterTransition);
///   <span class="comment">// If in s2 and the button receives an Exit event, transition back to s1</span>
///   <span class="type"><a href="http://doc.qt.io/qt-5/qeventtransition.html#QEventTransition">QEventTransition</a></span> <span class="operator">*</span>leaveTransition <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qeventtransition.html#QEventTransition">QEventTransition</a></span>(button<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a></span><span class="operator">::</span>Leave);
///   leaveTransition<span class="operator">-</span><span class="operator">&gt;</span>setTargetState(s1);
///   s2<span class="operator">-</span><span class="operator">&gt;</span>addTransition(leaveTransition);
///
/// </pre>
/// <a name="subclassing"></a></div>
#[repr(C)]
pub struct QEventTransition {
    _unused: u8,
}
impl QEventTransition {
    /// <p>This property holds the event source that this event transition is associated with</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QObject* QEventTransition::eventSource() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventtransition.html#eventSource-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the event source that this event transition is associated with</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QObject *</td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qeventtransition.html#eventSource">eventSource</a></b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qeventtransition.html#setEventSource">setEventSource</a></b></span>(QObject *<i>object</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn event_source(&self) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEventTransition_eventSource(
            self as *const crate::QEventTransition,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This property holds the type of event that this event transition is associated with</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QEvent::Type QEventTransition::eventType() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventtransition.html#eventType-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the type of event that this event transition is associated with</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QEvent::Type </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qeventtransition.html#eventType">eventType</a></b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qeventtransition.html#setEventType">setEventType</a></b></span>(QEvent::Type <i>type</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn event_type(&self) -> crate::q_event::Type {
        crate::__ffi::ctr_qt_core_ffi_QEventTransition_eventType(
            self as *const crate::QEventTransition,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QEventTransition::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEventTransition_metaObject(
            self as *const crate::QEventTransition,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs a new <a href="http://doc.qt.io/qt-5/qeventtransition.html">QEventTransition</a> object with the given <i>sourceState</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QEventTransition::QEventTransition(QState* sourceState = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventtransition.html#QEventTransition">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new <a href="http://doc.qt.io/qt-5/qeventtransition.html">QEventTransition</a> object with the given <i>sourceState</i>.</p></div>
    pub unsafe fn new_1a(
        source_state: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QState>>,
    ) -> ::cpp_utils::CppBox<crate::QEventTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEventTransition_QEventTransition(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QState>>::cast_into(source_state)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new <a href="http://doc.qt.io/qt-5/qeventtransition.html">QEventTransition</a> object associated with events of the given <i>type</i> for the given <i>object</i>, and with the given <i>sourceState</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QEventTransition::QEventTransition(QObject* object, QEvent::Type type, QState* sourceState = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventtransition.html#QEventTransition-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new <a href="http://doc.qt.io/qt-5/qeventtransition.html">QEventTransition</a> object associated with events of the given <i>type</i> for the given <i>object</i>, and with the given <i>sourceState</i>.</p></div>
    pub unsafe fn new_3a(
        object: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
        type_: crate::q_event::Type,
        source_state: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QState>>,
    ) -> ::cpp_utils::CppBox<crate::QEventTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEventTransition_QEventTransition1(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(object)
                .as_mut_raw_ptr(),
            type_,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QState>>::cast_into(source_state)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qeventtransition.html">QEventTransition</a> class provides a <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>-specific transition for Qt events.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QEventTransition::QEventTransition()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventtransition.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qeventtransition.html">QEventTransition</a> class provides a <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>-specific transition for Qt events.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qeventtransition.html">QEventTransition</a> object binds an event to a particular <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>. <a href="http://doc.qt.io/qt-5/qeventtransition.html">QEventTransition</a> is part of <a href="http://doc.qt.io/qt-5/statemachine-api.html">The State Machine Framework</a>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span>button <span class="operator">=</span> <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span> <span class="operator">*</span>s1 <span class="operator">=</span> <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span> <span class="operator">*</span>s2 <span class="operator">=</span> <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>;
    ///   <span class="comment">// If in s1 and the button receives an Enter event, transition to s2</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qeventtransition.html#QEventTransition">QEventTransition</a></span> <span class="operator">*</span>enterTransition <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qeventtransition.html#QEventTransition">QEventTransition</a></span>(button<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a></span><span class="operator">::</span>Enter);
    ///   enterTransition<span class="operator">-</span><span class="operator">&gt;</span>setTargetState(s2);
    ///   s1<span class="operator">-</span><span class="operator">&gt;</span>addTransition(enterTransition);
    ///   <span class="comment">// If in s2 and the button receives an Exit event, transition back to s1</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qeventtransition.html#QEventTransition">QEventTransition</a></span> <span class="operator">*</span>leaveTransition <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qeventtransition.html#QEventTransition">QEventTransition</a></span>(button<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a></span><span class="operator">::</span>Leave);
    ///   leaveTransition<span class="operator">-</span><span class="operator">&gt;</span>setTargetState(s1);
    ///   s2<span class="operator">-</span><span class="operator">&gt;</span>addTransition(leaveTransition);
    ///
    /// </pre>
    /// <a name="subclassing"></a></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QEventTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEventTransition_QEventTransition2();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new <a href="http://doc.qt.io/qt-5/qeventtransition.html">QEventTransition</a> object associated with events of the given <i>type</i> for the given <i>object</i>, and with the given <i>sourceState</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QEventTransition::QEventTransition(QObject* object, QEvent::Type type)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventtransition.html#QEventTransition-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new <a href="http://doc.qt.io/qt-5/qeventtransition.html">QEventTransition</a> object associated with events of the given <i>type</i> for the given <i>object</i>, and with the given <i>sourceState</i>.</p></div>
    pub unsafe fn new_2a(
        object: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
        type_: crate::q_event::Type,
    ) -> ::cpp_utils::CppBox<crate::QEventTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEventTransition_QEventTransition3(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(object)
                .as_mut_raw_ptr(),
            type_,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QEventTransition::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QEventTransition_qt_metacall(
            self as *mut crate::QEventTransition,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QEventTransition::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEventTransition_qt_metacast(
            self as *mut crate::QEventTransition,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Sets the event source associated with this event transition to be the given <i>object</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QEventTransition::setEventSource(QObject* object)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventtransition.html#setEventSource">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the event source associated with this event transition to be the given <i>object</i>.</p>
    /// <p><b>Note:</b> Setter function for property <a href="http://doc.qt.io/qt-5/qeventtransition.html#eventSource-prop">eventSource</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qeventtransition.html#eventSource">eventSource</a>().</p></div>
    pub unsafe fn set_event_source(
        &mut self,
        object: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QEventTransition_setEventSource(
            self as *mut crate::QEventTransition,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(object)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Sets the event <i>type</i> that this event transition is associated with.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QEventTransition::setEventType(QEvent::Type type)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventtransition.html#setEventType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the event <i>type</i> that this event transition is associated with.</p>
    /// <p><b>Note:</b> Setter function for property <a href="http://doc.qt.io/qt-5/qeventtransition.html#eventType-prop">eventType</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qeventtransition.html#eventType">eventType</a>().</p></div>
    pub unsafe fn set_event_type(&mut self, type_: crate::q_event::Type) {
        crate::__ffi::ctr_qt_core_ffi_QEventTransition_setEventType(
            self as *mut crate::QEventTransition,
            type_,
        )
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEventTransition_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QEventTransition::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEventTransition_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QEventTransition::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QEventTransition_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The UnhandledException class represents an unhandled exception in a worker thread.</p>
///
/// C++ class: <span style='color: green;'>```QUnhandledException```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qunhandledexception.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The UnhandledException class represents an unhandled exception in a worker thread.</p>
/// <p>If a worker thread throws an exception that is not a subclass of <a href="http://doc.qt.io/qt-5/qexception.html">QException</a>, the Qt functions will throw a <a href="http://doc.qt.io/qt-5/qunhandledexception.html">QUnhandledException</a> on the receiver thread side.</p>
/// <p>Inheriting from this class is not supported.</p></div>
#[repr(C)]
pub struct QUnhandledException {
    _unused: u8,
}
impl QUnhandledException {
    /// Calls C++ function: <span style='color: green;'>```virtual QUnhandledException* QUnhandledException::clone() const```</span>.
    pub unsafe fn clone(&self) -> ::cpp_utils::MutPtr<crate::QUnhandledException> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUnhandledException_clone(
            self as *const crate::QUnhandledException,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>The UnhandledException class represents an unhandled exception in a worker thread.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QUnhandledException& QUnhandledException::operator=(const QUnhandledException& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qunhandledexception.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The UnhandledException class represents an unhandled exception in a worker thread.</p>
    /// <p>If a worker thread throws an exception that is not a subclass of <a href="http://doc.qt.io/qt-5/qexception.html">QException</a>, the Qt functions will throw a <a href="http://doc.qt.io/qt-5/qunhandledexception.html">QUnhandledException</a> on the receiver thread side.</p>
    /// <p>Inheriting from this class is not supported.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUnhandledException>>,
    ) -> ::cpp_utils::MutRef<crate::QUnhandledException> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUnhandledException_operator_(
            self as *mut crate::QUnhandledException,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUnhandledException>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>The UnhandledException class represents an unhandled exception in a worker thread.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QUnhandledException::QUnhandledException()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qunhandledexception.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The UnhandledException class represents an unhandled exception in a worker thread.</p>
    /// <p>If a worker thread throws an exception that is not a subclass of <a href="http://doc.qt.io/qt-5/qexception.html">QException</a>, the Qt functions will throw a <a href="http://doc.qt.io/qt-5/qunhandledexception.html">QUnhandledException</a> on the receiver thread side.</p>
    /// <p>Inheriting from this class is not supported.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QUnhandledException> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUnhandledException_QUnhandledException();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The UnhandledException class represents an unhandled exception in a worker thread.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QUnhandledException::QUnhandledException(const QUnhandledException& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qunhandledexception.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The UnhandledException class represents an unhandled exception in a worker thread.</p>
    /// <p>If a worker thread throws an exception that is not a subclass of <a href="http://doc.qt.io/qt-5/qexception.html">QException</a>, the Qt functions will throw a <a href="http://doc.qt.io/qt-5/qunhandledexception.html">QUnhandledException</a> on the receiver thread side.</p>
    /// <p>Inheriting from this class is not supported.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUnhandledException>>,
    ) -> ::cpp_utils::CppBox<crate::QUnhandledException> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUnhandledException_QUnhandledException1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUnhandledException>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void QUnhandledException::raise() const```</span>.
    pub unsafe fn raise(&self) {
        crate::__ffi::ctr_qt_core_ffi_QUnhandledException_raise(
            self as *const crate::QUnhandledException,
        )
    }
}

/// C++ class: <span style='color: green;'>```QFactoryInterface```</span>.
#[repr(C)]
pub struct QFactoryInterface {
    _unused: u8,
}
impl QFactoryInterface {
    /// Calls C++ function: <span style='color: green;'>```QFactoryInterface& QFactoryInterface::operator=(const QFactoryInterface& other)```</span>.
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QFactoryInterface>>,
    ) -> ::cpp_utils::MutRef<crate::QFactoryInterface> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFactoryInterface_operator_(
            self as *mut crate::QFactoryInterface,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QFactoryInterface>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```pure virtual QStringList QFactoryInterface::keys() const```</span>.
    pub unsafe fn keys(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFactoryInterface_keys(
            self as *const crate::QFactoryInterface,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p><a href="http://doc.qt.io/qt-5/qfileselector.html">QFileSelector</a> provides a convenient way of selecting file variants.</p>
///
/// C++ class: <span style='color: green;'>```QFileSelector```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qfileselector.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p><a href="http://doc.qt.io/qt-5/qfileselector.html">QFileSelector</a> provides a convenient way of selecting file variants.</p>
/// <p><a href="http://doc.qt.io/qt-5/qfileselector.html">QFileSelector</a> is a convenience for selecting file variants based on platform or device characteristics. This allows you to develop and deploy one codebase containing all the different variants more easily in some circumstances, such as when the correct variant cannot be determined during the deploy step.</p>
/// <a name="using-qfileselector"></a></div>
#[repr(C)]
pub struct QFileSelector {
    _unused: u8,
}
impl QFileSelector {
    /// <p>Returns the complete, ordered list of selectors used by this instance</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QFileSelector::allSelectors() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileselector.html#allSelectors">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the complete, ordered list of selectors used by this instance</p></div>
    pub unsafe fn all_selectors(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileSelector_allSelectors(
            self as *const crate::QFileSelector,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the list of extra selectors which have been added programmatically to this instance.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QFileSelector::extraSelectors() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileselector.html#extraSelectors">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the list of extra selectors which have been added programmatically to this instance.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileselector.html#setExtraSelectors">setExtraSelectors</a>().</p></div>
    pub unsafe fn extra_selectors(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileSelector_extraSelectors(
            self as *const crate::QFileSelector,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QFileSelector::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileSelector_metaObject(
            self as *const crate::QFileSelector,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Create a <a href="http://doc.qt.io/qt-5/qfileselector.html">QFileSelector</a> instance. This instance will have the same static selectors as other <a href="http://doc.qt.io/qt-5/qfileselector.html">QFileSelector</a> instances, but its own set of extra selectors.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QFileSelector::QFileSelector(QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileselector.html#QFileSelector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Create a <a href="http://doc.qt.io/qt-5/qfileselector.html">QFileSelector</a> instance. This instance will have the same static selectors as other <a href="http://doc.qt.io/qt-5/qfileselector.html">QFileSelector</a> instances, but its own set of extra selectors.</p>
    /// <p>If supplied, it will have the given <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> <i>parent</i>.</p></div>
    pub unsafe fn new_1a(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QFileSelector> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileSelector_QFileSelector(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p><a href="http://doc.qt.io/qt-5/qfileselector.html">QFileSelector</a> provides a convenient way of selecting file variants.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QFileSelector::QFileSelector()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileselector.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p><a href="http://doc.qt.io/qt-5/qfileselector.html">QFileSelector</a> provides a convenient way of selecting file variants.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qfileselector.html">QFileSelector</a> is a convenience for selecting file variants based on platform or device characteristics. This allows you to develop and deploy one codebase containing all the different variants more easily in some circumstances, such as when the correct variant cannot be determined during the deploy step.</p>
    /// <a name="using-qfileselector"></a></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QFileSelector> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileSelector_QFileSelector1();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QFileSelector::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QFileSelector_qt_metacall(
            self as *mut crate::QFileSelector,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QFileSelector::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileSelector_qt_metacast(
            self as *mut crate::QFileSelector,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This function returns the selected version of the path, based on the conditions at runtime. If no selectable files are present, returns the original <i>filePath</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QFileSelector::select(const QString& filePath) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileselector.html#select">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function returns the selected version of the path, based on the conditions at runtime. If no selectable files are present, returns the original <i>filePath</i>.</p>
    /// <p>If the original file does not exist, the original <i>filePath</i> is returned. This means that you must have a base file to fall back on, you cannot have only files in selectable sub-directories.</p>
    /// <p>See the class overview for the selection algorithm.</p></div>
    pub unsafe fn select_q_string(
        &self,
        file_path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileSelector_select(
            self as *const crate::QFileSelector,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_path)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is a convenience version of select operating on <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> objects. If the scheme is not file or qrc, <i>filePath</i> is returned immediately. Otherwise selection is applied to the path of <i>filePath</i> and a <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> is returned with the selected path and other <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> parts the same as <i>filePath</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QUrl QFileSelector::select(const QUrl& filePath) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileselector.html#select-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is a convenience version of select operating on <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> objects. If the scheme is not file or qrc, <i>filePath</i> is returned immediately. Otherwise selection is applied to the path of <i>filePath</i> and a <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> is returned with the selected path and other <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> parts the same as <i>filePath</i>.</p>
    /// <p>See the class overview for the selection algorithm.</p></div>
    pub unsafe fn select_q_url(
        &self,
        file_path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
    ) -> ::cpp_utils::CppBox<crate::QUrl> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileSelector_select1(
            self as *const crate::QFileSelector,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(file_path)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Sets the <i>list</i> of extra selectors which have been added programmatically to this instance.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QFileSelector::setExtraSelectors(const QStringList& list)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileselector.html#setExtraSelectors">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the <i>list</i> of extra selectors which have been added programmatically to this instance.</p>
    /// <p>These selectors have priority over any which have been automatically picked up.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileselector.html#extraSelectors">extraSelectors</a>().</p></div>
    pub unsafe fn set_extra_selectors(
        &mut self,
        list: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QFileSelector_setExtraSelectors(
            self as *mut crate::QFileSelector,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(list)
                .as_raw_ptr(),
        )
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileSelector_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QFileSelector::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileSelector_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QFileSelector::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileSelector_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html">QFileSystemWatcher</a> class provides an interface for monitoring files and directories for modifications.</p>
///
/// C++ class: <span style='color: green;'>```QFileSystemWatcher```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html">QFileSystemWatcher</a> class provides an interface for monitoring files and directories for modifications.</p>
/// <p><a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html">QFileSystemWatcher</a> monitors the file system for changes to files and directories by watching a list of specified paths.</p>
/// <p>Call <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#addPath">addPath</a>() to watch a particular file or directory. Multiple paths can be added using the <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#addPaths">addPaths</a>() function. Existing paths can be removed by using the <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#removePath">removePath</a>() and <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#removePaths">removePaths</a>() functions.</p>
/// <p><a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html">QFileSystemWatcher</a> examines each path added to it. Files that have been added to the <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html">QFileSystemWatcher</a> can be accessed using the <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#files">files</a>() function, and directories using the <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#directories">directories</a>() function.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#fileChanged">fileChanged</a>() signal is emitted when a file has been modified, renamed or removed from disk. Similarly, the <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#directoryChanged">directoryChanged</a>() signal is emitted when a directory or its contents is modified or removed. Note that <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html">QFileSystemWatcher</a> stops monitoring files once they have been renamed or removed from disk, and directories once they have been removed from disk.</p>
/// <ul>
/// <li><b>Notes</b>:<ul>
/// <li>On systems running a Linux kernel without inotify support, file systems that contain watched paths cannot be unmounted.</li>
/// <li>The act of monitoring files and directories for modifications consumes system resources. This implies there is a limit to the number of files and directories your process can monitor simultaneously. On all BSD variants, for example, an open file descriptor is required for each monitored file. Some system limits the number of open file descriptors to 256 by default. This means that <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#addPath">addPath</a>() and <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#addPaths">addPaths</a>() will fail if your process tries to add more than 256 files or directories to the file system monitor. Also note that your process may have other file descriptors open in addition to the ones for files being monitored, and these other open descriptors also count in the total. <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> uses a different backend and does not suffer from this issue.</li>
/// </ul>
/// </li>
/// </ul></div>
#[repr(C)]
pub struct QFileSystemWatcher {
    _unused: u8,
}
impl QFileSystemWatcher {
    /// <p>This signal is emitted when the file at the specified <i>path</i> is modified, renamed or removed from disk.</p>
    ///
    /// Returns a built-in Qt signal `QFileSystemWatcher::fileChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#fileChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the file at the specified <i>path</i> is modified, renamed or removed from disk.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#directoryChanged">directoryChanged</a>().</p></div>
    pub fn file_changed(&self) -> crate::Signal<(*const crate::QString,)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2fileChanged(const QString&)\0"),
            )
        }
    }

    /// <p>This signal is emitted when the directory at a specified <i>path</i> is modified (e.g., when a file is added or deleted) or removed from disk. Note that if there are several changes during a short period of time, some of the changes might not emit this signal. However, the last change in the sequence of changes will always generate this signal.</p>
    ///
    /// Returns a built-in Qt signal `QFileSystemWatcher::directoryChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#directoryChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the directory at a specified <i>path</i> is modified (e.g., when a file is added or deleted) or removed from disk. Note that if there are several changes during a short period of time, some of the changes might not emit this signal. However, the last change in the sequence of changes will always generate this signal.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#fileChanged">fileChanged</a>().</p></div>
    pub fn directory_changed(&self) -> crate::Signal<(*const crate::QString,)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2directoryChanged(const QString&)\0",
                ),
            )
        }
    }

    /// <p>Adds <i>path</i> to the file system watcher if <i>path</i> exists. The path is not added if it does not exist, or if it is already being monitored by the file system watcher.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFileSystemWatcher::addPath(const QString& file)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#addPath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds <i>path</i> to the file system watcher if <i>path</i> exists. The path is not added if it does not exist, or if it is already being monitored by the file system watcher.</p>
    /// <p>If <i>path</i> specifies a directory, the <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#directoryChanged">directoryChanged</a>() signal will be emitted when <i>path</i> is modified or removed from disk; otherwise the <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#fileChanged">fileChanged</a>() signal is emitted when <i>path</i> is modified, renamed or removed.</p>
    /// <p>If the watch was successful, true is returned.</p>
    /// <p>Reasons for a watch failure are generally system-dependent, but may include the resource not existing, access failures, or the total watch count limit, if the platform has one.</p>
    /// <p><b>Note: </b>There may be a system dependent limit to the number of files and directories that can be monitored simultaneously. If this limit is been reached, <i>path</i> will not be monitored, and false is returned.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#addPaths">addPaths</a>() and <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#removePath">removePath</a>().</p></div>
    pub unsafe fn add_path(
        &mut self,
        file: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFileSystemWatcher_addPath(
            self as *mut crate::QFileSystemWatcher,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file).as_raw_ptr(),
        )
    }

    /// <p>Adds each path in <i>paths</i> to the file system watcher. Paths are not added if they not exist, or if they are already being monitored by the file system watcher.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QFileSystemWatcher::addPaths(const QStringList& files)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#addPaths">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds each path in <i>paths</i> to the file system watcher. Paths are not added if they not exist, or if they are already being monitored by the file system watcher.</p>
    /// <p>If a path specifies a directory, the <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#directoryChanged">directoryChanged</a>() signal will be emitted when the path is modified or removed from disk; otherwise the <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#fileChanged">fileChanged</a>() signal is emitted when the path is modified, renamed, or removed.</p>
    /// <p>The return value is a list of paths that could not be watched.</p>
    /// <p>Reasons for a watch failure are generally system-dependent, but may include the resource not existing, access failures, or the total watch count limit, if the platform has one.</p>
    /// <p><b>Note: </b>There may be a system dependent limit to the number of files and directories that can be monitored simultaneously. If this limit has been reached, the excess <i>paths</i> will not be monitored, and they will be added to the returned <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a>.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#addPath">addPath</a>() and <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#removePaths">removePaths</a>().</p></div>
    pub unsafe fn add_paths(
        &mut self,
        files: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileSystemWatcher_addPaths(
            self as *mut crate::QFileSystemWatcher,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(files)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list of paths to directories that are being watched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QFileSystemWatcher::directories() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#directories">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of paths to directories that are being watched.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#files">files</a>().</p></div>
    pub unsafe fn directories(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileSystemWatcher_directories(
            self as *const crate::QFileSystemWatcher,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list of paths to files that are being watched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QFileSystemWatcher::files() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#files">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of paths to files that are being watched.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#directories">directories</a>().</p></div>
    pub unsafe fn files(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileSystemWatcher_files(
            self as *const crate::QFileSystemWatcher,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QFileSystemWatcher::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileSystemWatcher_metaObject(
            self as *const crate::QFileSystemWatcher,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs a new file system watcher object with the given <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QFileSystemWatcher::QFileSystemWatcher(QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#QFileSystemWatcher">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new file system watcher object with the given <i>parent</i>.</p></div>
    pub unsafe fn from_q_object(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QFileSystemWatcher> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileSystemWatcher_QFileSystemWatcher(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new file system watcher object with the given <i>parent</i> which monitors the specified <i>paths</i> list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QFileSystemWatcher::QFileSystemWatcher(const QStringList& paths, QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#QFileSystemWatcher-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new file system watcher object with the given <i>parent</i> which monitors the specified <i>paths</i> list.</p></div>
    pub unsafe fn from_q_string_list_q_object(
        paths: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QFileSystemWatcher> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileSystemWatcher_QFileSystemWatcher1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(paths)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html">QFileSystemWatcher</a> class provides an interface for monitoring files and directories for modifications.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QFileSystemWatcher::QFileSystemWatcher()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html">QFileSystemWatcher</a> class provides an interface for monitoring files and directories for modifications.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html">QFileSystemWatcher</a> monitors the file system for changes to files and directories by watching a list of specified paths.</p>
    /// <p>Call <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#addPath">addPath</a>() to watch a particular file or directory. Multiple paths can be added using the <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#addPaths">addPaths</a>() function. Existing paths can be removed by using the <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#removePath">removePath</a>() and <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#removePaths">removePaths</a>() functions.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html">QFileSystemWatcher</a> examines each path added to it. Files that have been added to the <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html">QFileSystemWatcher</a> can be accessed using the <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#files">files</a>() function, and directories using the <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#directories">directories</a>() function.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#fileChanged">fileChanged</a>() signal is emitted when a file has been modified, renamed or removed from disk. Similarly, the <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#directoryChanged">directoryChanged</a>() signal is emitted when a directory or its contents is modified or removed. Note that <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html">QFileSystemWatcher</a> stops monitoring files once they have been renamed or removed from disk, and directories once they have been removed from disk.</p>
    /// <ul>
    /// <li><b>Notes</b>:<ul>
    /// <li>On systems running a Linux kernel without inotify support, file systems that contain watched paths cannot be unmounted.</li>
    /// <li>The act of monitoring files and directories for modifications consumes system resources. This implies there is a limit to the number of files and directories your process can monitor simultaneously. On all BSD variants, for example, an open file descriptor is required for each monitored file. Some system limits the number of open file descriptors to 256 by default. This means that <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#addPath">addPath</a>() and <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#addPaths">addPaths</a>() will fail if your process tries to add more than 256 files or directories to the file system monitor. Also note that your process may have other file descriptors open in addition to the ones for files being monitored, and these other open descriptors also count in the total. <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> uses a different backend and does not suffer from this issue.</li>
    /// </ul>
    /// </li>
    /// </ul></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QFileSystemWatcher> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileSystemWatcher_QFileSystemWatcher2();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new file system watcher object with the given <i>parent</i> which monitors the specified <i>paths</i> list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QFileSystemWatcher::QFileSystemWatcher(const QStringList& paths)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#QFileSystemWatcher-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new file system watcher object with the given <i>parent</i> which monitors the specified <i>paths</i> list.</p></div>
    pub unsafe fn from_q_string_list(
        paths: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) -> ::cpp_utils::CppBox<crate::QFileSystemWatcher> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileSystemWatcher_QFileSystemWatcher4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(paths)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QFileSystemWatcher::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QFileSystemWatcher_qt_metacall(
            self as *mut crate::QFileSystemWatcher,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QFileSystemWatcher::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileSystemWatcher_qt_metacast(
            self as *mut crate::QFileSystemWatcher,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Removes the specified <i>path</i> from the file system watcher.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFileSystemWatcher::removePath(const QString& file)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#removePath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the specified <i>path</i> from the file system watcher.</p>
    /// <p>If the watch is successfully removed, true is returned.</p>
    /// <p>Reasons for watch removal failing are generally system-dependent, but may be due to the path having already been deleted, for example.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#removePaths">removePaths</a>() and <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#addPath">addPath</a>().</p></div>
    pub unsafe fn remove_path(
        &mut self,
        file: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFileSystemWatcher_removePath(
            self as *mut crate::QFileSystemWatcher,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file).as_raw_ptr(),
        )
    }

    /// <p>Removes the specified <i>paths</i> from the file system watcher.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QFileSystemWatcher::removePaths(const QStringList& files)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#removePaths">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the specified <i>paths</i> from the file system watcher.</p>
    /// <p>The return value is a list of paths which were not able to be unwatched successfully.</p>
    /// <p>Reasons for watch removal failing are generally system-dependent, but may be due to the path having already been deleted, for example.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#removePath">removePath</a>() and <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#addPaths">addPaths</a>().</p></div>
    pub unsafe fn remove_paths(
        &mut self,
        files: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileSystemWatcher_removePaths(
            self as *mut crate::QFileSystemWatcher,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(files)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileSystemWatcher_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QFileSystemWatcher::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileSystemWatcher_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QFileSystemWatcher::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFileSystemWatcher_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qfinalstate.html">QFinalState</a> class provides a final state.</p>
///
/// C++ class: <span style='color: green;'>```QFinalState```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qfinalstate.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qfinalstate.html">QFinalState</a> class provides a final state.</p>
/// <p>A final state is used to communicate that (part of) a <a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a> has finished its work. When a final top-level state is entered, the state machine's <a href="http://doc.qt.io/qt-5/qstate.html#finished">finished</a>() signal is emitted. In general, when a final substate (a child of a <a href="http://doc.qt.io/qt-5/qstate.html">QState</a>) is entered, the parent state's <a href="http://doc.qt.io/qt-5/qstate.html#finished">finished</a>() signal is emitted. <a href="http://doc.qt.io/qt-5/qfinalstate.html">QFinalState</a> is part of <a href="http://doc.qt.io/qt-5/statemachine-api.html">The State Machine Framework</a>.</p>
/// <p>To use a final state, you create a <a href="http://doc.qt.io/qt-5/qfinalstate.html">QFinalState</a> object and add a transition to it from another state. Example:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> button;
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a></span> machine;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span> <span class="operator">*</span>s1 <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span>();
///   <span class="type"><a href="http://doc.qt.io/qt-5/qfinalstate.html#QFinalState">QFinalState</a></span> <span class="operator">*</span>s2 <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qfinalstate.html#QFinalState">QFinalState</a></span>();
///   s1<span class="operator">-</span><span class="operator">&gt;</span>addTransition(<span class="operator">&amp;</span>button<span class="operator">,</span> SIGNAL(clicked())<span class="operator">,</span> s2);
///   machine<span class="operator">.</span>addState(s1);
///   machine<span class="operator">.</span>addState(s2);
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span><span class="operator">::</span>connect(<span class="operator">&amp;</span>machine<span class="operator">,</span> SIGNAL(finished())<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qapplication.html">QApplication</a></span><span class="operator">::</span>instance()<span class="operator">,</span> SLOT(quit()));
///   machine<span class="operator">.</span>setInitialState(s1);
///   machine<span class="operator">.</span>start();
///
/// </pre></div>
#[repr(C)]
pub struct QFinalState {
    _unused: u8,
}
impl QFinalState {
    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QFinalState::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QFinalState_metaObject(self as *const crate::QFinalState);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs a new <a href="http://doc.qt.io/qt-5/qfinalstate.html">QFinalState</a> object with the given <i>parent</i> state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QFinalState::QFinalState(QState* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfinalstate.html#QFinalState">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new <a href="http://doc.qt.io/qt-5/qfinalstate.html">QFinalState</a> object with the given <i>parent</i> state.</p></div>
    pub unsafe fn new_1a(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QState>>,
    ) -> ::cpp_utils::CppBox<crate::QFinalState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFinalState_QFinalState(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QState>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qfinalstate.html">QFinalState</a> class provides a final state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QFinalState::QFinalState()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfinalstate.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qfinalstate.html">QFinalState</a> class provides a final state.</p>
    /// <p>A final state is used to communicate that (part of) a <a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a> has finished its work. When a final top-level state is entered, the state machine's <a href="http://doc.qt.io/qt-5/qstate.html#finished">finished</a>() signal is emitted. In general, when a final substate (a child of a <a href="http://doc.qt.io/qt-5/qstate.html">QState</a>) is entered, the parent state's <a href="http://doc.qt.io/qt-5/qstate.html#finished">finished</a>() signal is emitted. <a href="http://doc.qt.io/qt-5/qfinalstate.html">QFinalState</a> is part of <a href="http://doc.qt.io/qt-5/statemachine-api.html">The State Machine Framework</a>.</p>
    /// <p>To use a final state, you create a <a href="http://doc.qt.io/qt-5/qfinalstate.html">QFinalState</a> object and add a transition to it from another state. Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> button;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a></span> machine;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span> <span class="operator">*</span>s1 <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span>();
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qfinalstate.html#QFinalState">QFinalState</a></span> <span class="operator">*</span>s2 <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qfinalstate.html#QFinalState">QFinalState</a></span>();
    ///   s1<span class="operator">-</span><span class="operator">&gt;</span>addTransition(<span class="operator">&amp;</span>button<span class="operator">,</span> SIGNAL(clicked())<span class="operator">,</span> s2);
    ///   machine<span class="operator">.</span>addState(s1);
    ///   machine<span class="operator">.</span>addState(s2);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span><span class="operator">::</span>connect(<span class="operator">&amp;</span>machine<span class="operator">,</span> SIGNAL(finished())<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qapplication.html">QApplication</a></span><span class="operator">::</span>instance()<span class="operator">,</span> SLOT(quit()));
    ///   machine<span class="operator">.</span>setInitialState(s1);
    ///   machine<span class="operator">.</span>start();
    ///
    /// </pre></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QFinalState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFinalState_QFinalState1();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QFinalState::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QFinalState_qt_metacall(
            self as *mut crate::QFinalState,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QFinalState::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFinalState_qt_metacast(
            self as *mut crate::QFinalState,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFinalState_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QFinalState::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFinalState_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QFinalState::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFinalState_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a> class is the base class for all runnable objects.</p>
///
/// C++ class: <span style='color: green;'>```QRunnable```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qrunnable.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a> class is the base class for all runnable objects.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a> class is an interface for representing a task or piece of code that needs to be executed, represented by your reimplementation of the <a href="http://doc.qt.io/qt-5/qrunnable.html#run">run</a>() function.</p>
/// <p>You can use <a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a> to execute your code in a separate thread. <a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a> deletes the <a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a> automatically if <a href="http://doc.qt.io/qt-5/qrunnable.html#autoDelete">autoDelete</a>() returns <code>true</code> (the default). Use <a href="http://doc.qt.io/qt-5/qrunnable.html#setAutoDelete">setAutoDelete</a>() to change the auto-deletion flag.</p>
/// <p><a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a> supports executing the same <a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a> more than once by calling <a href="http://doc.qt.io/qt-5/qthreadpool.html#tryStart">QThreadPool::tryStart</a>(this) from within the <a href="http://doc.qt.io/qt-5/qrunnable.html#run">run</a>() function. If <a href="http://doc.qt.io/qt-5/qrunnable.html#autoDelete">autoDelete</a> is enabled the <a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a> will be deleted when the last thread exits the run function. Calling <a href="http://doc.qt.io/qt-5/qthreadpool.html#start">QThreadPool::start</a>() multiple times with the same <a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a> when <a href="http://doc.qt.io/qt-5/qrunnable.html#autoDelete">autoDelete</a> is enabled creates a race condition and is not recommended.</p></div>
#[repr(C)]
pub struct QRunnable {
    _unused: u8,
}
impl QRunnable {
    /// <p>Returns <code>true</code> is auto-deletion is enabled; false otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRunnable::autoDelete() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrunnable.html#autoDelete">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> is auto-deletion is enabled; false otherwise.</p>
    /// <p>If auto-deletion is enabled, <a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a> will automatically delete this runnable after calling <a href="http://doc.qt.io/qt-5/qrunnable.html#run">run</a>(); otherwise, ownership remains with the application programmer.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrunnable.html#setAutoDelete">setAutoDelete</a>() and <a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a>.</p></div>
    pub unsafe fn auto_delete(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRunnable_autoDelete(self as *const crate::QRunnable)
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a> class is the base class for all runnable objects.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRunnable& QRunnable::operator=(const QRunnable& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrunnable.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a> class is the base class for all runnable objects.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a> class is an interface for representing a task or piece of code that needs to be executed, represented by your reimplementation of the <a href="http://doc.qt.io/qt-5/qrunnable.html#run">run</a>() function.</p>
    /// <p>You can use <a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a> to execute your code in a separate thread. <a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a> deletes the <a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a> automatically if <a href="http://doc.qt.io/qt-5/qrunnable.html#autoDelete">autoDelete</a>() returns <code>true</code> (the default). Use <a href="http://doc.qt.io/qt-5/qrunnable.html#setAutoDelete">setAutoDelete</a>() to change the auto-deletion flag.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a> supports executing the same <a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a> more than once by calling <a href="http://doc.qt.io/qt-5/qthreadpool.html#tryStart">QThreadPool::tryStart</a>(this) from within the <a href="http://doc.qt.io/qt-5/qrunnable.html#run">run</a>() function. If <a href="http://doc.qt.io/qt-5/qrunnable.html#autoDelete">autoDelete</a> is enabled the <a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a> will be deleted when the last thread exits the run function. Calling <a href="http://doc.qt.io/qt-5/qthreadpool.html#start">QThreadPool::start</a>() multiple times with the same <a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a> when <a href="http://doc.qt.io/qt-5/qrunnable.html#autoDelete">autoDelete</a> is enabled creates a race condition and is not recommended.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRunnable>>,
    ) -> ::cpp_utils::MutRef<crate::QRunnable> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRunnable_operator_(
            self as *mut crate::QRunnable,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRunnable>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Implement this pure virtual function in your subclass.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual void QRunnable::run()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrunnable.html#run">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Implement this pure virtual function in your subclass.</p></div>
    pub unsafe fn run(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QRunnable_run(self as *mut crate::QRunnable)
    }

    /// <p>Enables auto-deletion if <i>autoDelete</i> is true; otherwise auto-deletion is disabled.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRunnable::setAutoDelete(bool _autoDelete)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrunnable.html#setAutoDelete">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Enables auto-deletion if <i>autoDelete</i> is true; otherwise auto-deletion is disabled.</p>
    /// <p>If auto-deletion is enabled, <a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a> will automatically delete this runnable after calling <a href="http://doc.qt.io/qt-5/qrunnable.html#run">run</a>(); otherwise, ownership remains with the application programmer.</p>
    /// <p>Note that this flag must be set before calling <a href="http://doc.qt.io/qt-5/qthreadpool.html#start">QThreadPool::start</a>(). Calling this function after <a href="http://doc.qt.io/qt-5/qthreadpool.html#start">QThreadPool::start</a>() results in undefined behavior.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrunnable.html#autoDelete">autoDelete</a>() and <a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a>.</p></div>
    pub unsafe fn set_auto_delete(&mut self, auto_delete: bool) {
        crate::__ffi::ctr_qt_core_ffi_QRunnable_setAutoDelete(
            self as *mut crate::QRunnable,
            auto_delete,
        )
    }
}

/// C++ class: <span style='color: green;'>```QBasicMutex```</span>.
#[repr(C)]
pub struct QBasicMutex {
    _unused: u8,
}
impl QBasicMutex {
    /// Calls C++ function: <span style='color: green;'>```bool QBasicMutex::isRecursive()```</span>.
    pub unsafe fn is_recursive_mut(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QBasicMutex_isRecursive(self as *mut crate::QBasicMutex)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QBasicMutex::isRecursive() const```</span>.
    pub unsafe fn is_recursive(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QBasicMutex_isRecursive1(self as *const crate::QBasicMutex)
    }

    /// Calls C++ function: <span style='color: green;'>```void QBasicMutex::lock()```</span>.
    pub unsafe fn lock(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QBasicMutex_lock(self as *mut crate::QBasicMutex)
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QBasicMutex::QBasicMutex()```</span>.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QBasicMutex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QBasicMutex_QBasicMutex();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QBasicMutex::tryLock()```</span>.
    pub unsafe fn try_lock(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QBasicMutex_tryLock(self as *mut crate::QBasicMutex)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QBasicMutex::try_lock()```</span>.
    pub unsafe fn try_lock2(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QBasicMutex_try_lock(self as *mut crate::QBasicMutex)
    }

    /// Calls C++ function: <span style='color: green;'>```void QBasicMutex::unlock()```</span>.
    pub unsafe fn unlock(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QBasicMutex_unlock(self as *mut crate::QBasicMutex)
    }
}

pub mod q_mutex {
    //! C++ type: <span style='color: green;'>```QMutex```</span>

    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmutex.html#QMutex">QMutex</a>().</p>
    ///
    /// C++ enum: <span style='color: green;'>```QMutex::RecursionMode```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmutex.html#RecursionMode-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p><b>See also </b><a href="http://doc.qt.io/qt-5/qmutex.html#QMutex">QMutex</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct RecursionMode(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for RecursionMode {
        fn from(value: ::std::os::raw::c_int) -> Self {
            RecursionMode(value)
        }
    }

    impl From<RecursionMode> for ::std::os::raw::c_int {
        fn from(value: RecursionMode) -> Self {
            value.0
        }
    }

    impl RecursionMode {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl RecursionMode {
        /// In this mode, a thread may only lock a mutex once. (C++ enum variant: <span style='color: green;'>```NonRecursive = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NonRecursive: crate::q_mutex::RecursionMode = crate::q_mutex::RecursionMode(0);
        /// In this mode, a thread can lock the same mutex multiple times and the mutex won't be unlocked until a corresponding number of <a href="http://doc.qt.io/qt-5/qmutex.html#unlock">unlock</a>() calls have been made. (C++ enum variant: <span style='color: green;'>```Recursive = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Recursive: crate::q_mutex::RecursionMode = crate::q_mutex::RecursionMode(1);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qmutex.html">QMutex</a> class provides access serialization between threads.</p>
///
/// C++ class: <span style='color: green;'>```QMutex```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qmutex.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmutex.html">QMutex</a> class provides access serialization between threads.</p>
/// <p>The purpose of a <a href="http://doc.qt.io/qt-5/qmutex.html">QMutex</a> is to protect an object, data structure or section of code so that only one thread can access it at a time (this is similar to the Java <code>synchronized</code> keyword). It is usually best to use a mutex with a <a href="http://doc.qt.io/qt-5/qmutexlocker.html">QMutexLocker</a> since this makes it easy to ensure that locking and unlocking are performed consistently.</p>
/// <p>For example, say there is a method that prints a message to the user on two lines:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> number <span class="operator">=</span> <span class="number">6</span>;
///
///   <span class="type">void</span> method1()
///   {
/// &#32;     number <span class="operator">*</span><span class="operator">=</span> <span class="number">5</span>;
/// &#32;     number <span class="operator">/</span><span class="operator">=</span> <span class="number">4</span>;
///   }
///
///   <span class="type">void</span> method2()
///   {
/// &#32;     number <span class="operator">*</span><span class="operator">=</span> <span class="number">3</span>;
/// &#32;     number <span class="operator">/</span><span class="operator">=</span> <span class="number">2</span>;
///   }
///
/// </pre>
/// <p>If these two methods are called in succession, the following happens:</p>
/// <pre class="cpp">
///
///   <span class="comment">// method1()</span>
///   number <span class="operator">*</span><span class="operator">=</span> <span class="number">5</span>;        <span class="comment">// number is now 30</span>
///   number <span class="operator">/</span><span class="operator">=</span> <span class="number">4</span>;        <span class="comment">// number is now 7</span>
///
///   <span class="comment">// method2()</span>
///   number <span class="operator">*</span><span class="operator">=</span> <span class="number">3</span>;        <span class="comment">// number is now 21</span>
///   number <span class="operator">/</span><span class="operator">=</span> <span class="number">2</span>;        <span class="comment">// number is now 10</span>
///
/// </pre>
/// <p>If these two methods are called simultaneously from two threads then the following sequence could result:</p>
/// <pre class="cpp">
///
///   <span class="comment">// Thread 1 calls method1()</span>
///   number <span class="operator">*</span><span class="operator">=</span> <span class="number">5</span>;        <span class="comment">// number is now 30</span>
///
///   <span class="comment">// Thread 2 calls method2().</span>
///   <span class="comment">//</span>
///   <span class="comment">// Most likely Thread 1 has been put to sleep by the operating</span>
///   <span class="comment">// system to allow Thread 2 to run.</span>
///   number <span class="operator">*</span><span class="operator">=</span> <span class="number">3</span>;        <span class="comment">// number is now 90</span>
///   number <span class="operator">/</span><span class="operator">=</span> <span class="number">2</span>;        <span class="comment">// number is now 45</span>
///
///   <span class="comment">// Thread 1 finishes executing.</span>
///   number <span class="operator">/</span><span class="operator">=</span> <span class="number">4</span>;        <span class="comment">// number is now 11, instead of 10</span>
///
/// </pre>
/// <p>If we add a mutex, we should get the result we want:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qmutex.html#QMutex">QMutex</a></span> mutex;
///   <span class="type">int</span> number <span class="operator">=</span> <span class="number">6</span>;
///
///   <span class="type">void</span> method1()
///   {
/// &#32;     mutex<span class="operator">.</span>lock();
/// &#32;     number <span class="operator">*</span><span class="operator">=</span> <span class="number">5</span>;
/// &#32;     number <span class="operator">/</span><span class="operator">=</span> <span class="number">4</span>;
/// &#32;     mutex<span class="operator">.</span>unlock();
///   }
///
///   <span class="type">void</span> method2()
///   {
/// &#32;     mutex<span class="operator">.</span>lock();
/// &#32;     number <span class="operator">*</span><span class="operator">=</span> <span class="number">3</span>;
/// &#32;     number <span class="operator">/</span><span class="operator">=</span> <span class="number">2</span>;
/// &#32;     mutex<span class="operator">.</span>unlock();
///   }
///
/// </pre>
/// <p>Then only one thread can modify <code>number</code> at any given time and the result is correct. This is a trivial example, of course, but applies to any other case where things need to happen in a particular sequence.</p>
/// <p>When you call <a href="http://doc.qt.io/qt-5/qmutex.html#lock">lock</a>() in a thread, other threads that try to call <a href="http://doc.qt.io/qt-5/qmutex.html#lock">lock</a>() in the same place will block until the thread that got the lock calls <a href="http://doc.qt.io/qt-5/qmutex.html#unlock">unlock</a>(). A non-blocking alternative to <a href="http://doc.qt.io/qt-5/qmutex.html#lock">lock</a>() is <a href="http://doc.qt.io/qt-5/qmutex.html#tryLock">tryLock</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qmutex.html">QMutex</a> is optimized to be fast in the non-contended case. A non-recursive <a href="http://doc.qt.io/qt-5/qmutex.html">QMutex</a> will not allocate memory if there is no contention on that mutex. It is constructed and destroyed with almost no overhead, which means it is fine to have many mutexes as part of other classes.</p></div>
#[repr(C)]
pub struct QMutex {
    _unused: u8,
}
impl QMutex {
    /// <p>Returns <code>true</code> if the mutex is recursive.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMutex::isRecursive() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmutex.html#isRecursive">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the mutex is recursive.</p>
    /// <p>This function was introduced in  Qt 5.7.</p></div>
    pub unsafe fn is_recursive(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMutex_isRecursive(self as *const crate::QMutex)
    }

    /// <p>Locks the mutex. If another thread has locked the mutex then this call will block until that thread has unlocked it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMutex::lock()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmutex.html#lock">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Locks the mutex. If another thread has locked the mutex then this call will block until that thread has unlocked it.</p>
    /// <p>Calling this function multiple times on the same mutex from the same thread is allowed if this mutex is a <a href="http://doc.qt.io/qt-5/qmutex.html#RecursionMode-enum">recursive mutex</a>. If this mutex is a <a href="http://doc.qt.io/qt-5/qmutex.html#RecursionMode-enum">non-recursive mutex</a>, this function will <i>dead-lock</i> when the mutex is locked recursively.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmutex.html#unlock">unlock</a>().</p></div>
    pub unsafe fn lock(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMutex_lock(self as *mut crate::QMutex)
    }

    /// <p>Constructs a new mutex. The mutex is created in an unlocked state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMutex::QMutex(QMutex::RecursionMode mode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmutex.html#QMutex">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new mutex. The mutex is created in an unlocked state.</p>
    /// <p>If <i>mode</i> is <a href="http://doc.qt.io/qt-5/qmutex.html#RecursionMode-enum">QMutex::Recursive</a>, a thread can lock the same mutex multiple times and the mutex won't be unlocked until a corresponding number of <a href="http://doc.qt.io/qt-5/qmutex.html#unlock">unlock</a>() calls have been made. Otherwise a thread may only lock a mutex once. The default is <a href="http://doc.qt.io/qt-5/qmutex.html#RecursionMode-enum">QMutex::NonRecursive</a>.</p>
    /// <p>Recursive mutexes are slower and take more memory than non-recursive ones.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmutex.html#lock">lock</a>() and <a href="http://doc.qt.io/qt-5/qmutex.html#unlock">unlock</a>().</p></div>
    pub unsafe fn new_1a(
        mode: crate::q_mutex::RecursionMode,
    ) -> ::cpp_utils::CppBox<crate::QMutex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMutex_QMutex(mode);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qmutex.html">QMutex</a> class provides access serialization between threads.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMutex::QMutex()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmutex.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmutex.html">QMutex</a> class provides access serialization between threads.</p>
    /// <p>The purpose of a <a href="http://doc.qt.io/qt-5/qmutex.html">QMutex</a> is to protect an object, data structure or section of code so that only one thread can access it at a time (this is similar to the Java <code>synchronized</code> keyword). It is usually best to use a mutex with a <a href="http://doc.qt.io/qt-5/qmutexlocker.html">QMutexLocker</a> since this makes it easy to ensure that locking and unlocking are performed consistently.</p>
    /// <p>For example, say there is a method that prints a message to the user on two lines:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">int</span> number <span class="operator">=</span> <span class="number">6</span>;
    ///
    ///   <span class="type">void</span> method1()
    ///   {
    /// &#32;     number <span class="operator">*</span><span class="operator">=</span> <span class="number">5</span>;
    /// &#32;     number <span class="operator">/</span><span class="operator">=</span> <span class="number">4</span>;
    ///   }
    ///
    ///   <span class="type">void</span> method2()
    ///   {
    /// &#32;     number <span class="operator">*</span><span class="operator">=</span> <span class="number">3</span>;
    /// &#32;     number <span class="operator">/</span><span class="operator">=</span> <span class="number">2</span>;
    ///   }
    ///
    /// </pre>
    /// <p>If these two methods are called in succession, the following happens:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="comment">// method1()</span>
    ///   number <span class="operator">*</span><span class="operator">=</span> <span class="number">5</span>;        <span class="comment">// number is now 30</span>
    ///   number <span class="operator">/</span><span class="operator">=</span> <span class="number">4</span>;        <span class="comment">// number is now 7</span>
    ///
    ///   <span class="comment">// method2()</span>
    ///   number <span class="operator">*</span><span class="operator">=</span> <span class="number">3</span>;        <span class="comment">// number is now 21</span>
    ///   number <span class="operator">/</span><span class="operator">=</span> <span class="number">2</span>;        <span class="comment">// number is now 10</span>
    ///
    /// </pre>
    /// <p>If these two methods are called simultaneously from two threads then the following sequence could result:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="comment">// Thread 1 calls method1()</span>
    ///   number <span class="operator">*</span><span class="operator">=</span> <span class="number">5</span>;        <span class="comment">// number is now 30</span>
    ///
    ///   <span class="comment">// Thread 2 calls method2().</span>
    ///   <span class="comment">//</span>
    ///   <span class="comment">// Most likely Thread 1 has been put to sleep by the operating</span>
    ///   <span class="comment">// system to allow Thread 2 to run.</span>
    ///   number <span class="operator">*</span><span class="operator">=</span> <span class="number">3</span>;        <span class="comment">// number is now 90</span>
    ///   number <span class="operator">/</span><span class="operator">=</span> <span class="number">2</span>;        <span class="comment">// number is now 45</span>
    ///
    ///   <span class="comment">// Thread 1 finishes executing.</span>
    ///   number <span class="operator">/</span><span class="operator">=</span> <span class="number">4</span>;        <span class="comment">// number is now 11, instead of 10</span>
    ///
    /// </pre>
    /// <p>If we add a mutex, we should get the result we want:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmutex.html#QMutex">QMutex</a></span> mutex;
    ///   <span class="type">int</span> number <span class="operator">=</span> <span class="number">6</span>;
    ///
    ///   <span class="type">void</span> method1()
    ///   {
    /// &#32;     mutex<span class="operator">.</span>lock();
    /// &#32;     number <span class="operator">*</span><span class="operator">=</span> <span class="number">5</span>;
    /// &#32;     number <span class="operator">/</span><span class="operator">=</span> <span class="number">4</span>;
    /// &#32;     mutex<span class="operator">.</span>unlock();
    ///   }
    ///
    ///   <span class="type">void</span> method2()
    ///   {
    /// &#32;     mutex<span class="operator">.</span>lock();
    /// &#32;     number <span class="operator">*</span><span class="operator">=</span> <span class="number">3</span>;
    /// &#32;     number <span class="operator">/</span><span class="operator">=</span> <span class="number">2</span>;
    /// &#32;     mutex<span class="operator">.</span>unlock();
    ///   }
    ///
    /// </pre>
    /// <p>Then only one thread can modify <code>number</code> at any given time and the result is correct. This is a trivial example, of course, but applies to any other case where things need to happen in a particular sequence.</p>
    /// <p>When you call <a href="http://doc.qt.io/qt-5/qmutex.html#lock">lock</a>() in a thread, other threads that try to call <a href="http://doc.qt.io/qt-5/qmutex.html#lock">lock</a>() in the same place will block until the thread that got the lock calls <a href="http://doc.qt.io/qt-5/qmutex.html#unlock">unlock</a>(). A non-blocking alternative to <a href="http://doc.qt.io/qt-5/qmutex.html#lock">lock</a>() is <a href="http://doc.qt.io/qt-5/qmutex.html#tryLock">tryLock</a>().</p>
    /// <p><a href="http://doc.qt.io/qt-5/qmutex.html">QMutex</a> is optimized to be fast in the non-contended case. A non-recursive <a href="http://doc.qt.io/qt-5/qmutex.html">QMutex</a> will not allocate memory if there is no contention on that mutex. It is constructed and destroyed with almost no overhead, which means it is fine to have many mutexes as part of other classes.</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QMutex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMutex_QMutex1();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Attempts to lock the mutex. This function returns <code>true</code> if the lock was obtained; otherwise it returns <code>false</code>. If another thread has locked the mutex, this function will wait for at most <i>timeout</i> milliseconds for the mutex to become available.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMutex::tryLock(int timeout = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmutex.html#tryLock">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to lock the mutex. This function returns <code>true</code> if the lock was obtained; otherwise it returns <code>false</code>. If another thread has locked the mutex, this function will wait for at most <i>timeout</i> milliseconds for the mutex to become available.</p>
    /// <p>Note: Passing a negative number as the <i>timeout</i> is equivalent to calling <a href="http://doc.qt.io/qt-5/qmutex.html#lock">lock</a>(), i.e. this function will wait forever until mutex can be locked if <i>timeout</i> is negative.</p>
    /// <p>If the lock was obtained, the mutex must be unlocked with <a href="http://doc.qt.io/qt-5/qmutex.html#unlock">unlock</a>() before another thread can successfully lock it.</p>
    /// <p>Calling this function multiple times on the same mutex from the same thread is allowed if this mutex is a <a href="http://doc.qt.io/qt-5/qmutex.html#RecursionMode-enum">recursive mutex</a>. If this mutex is a <a href="http://doc.qt.io/qt-5/qmutex.html#RecursionMode-enum">non-recursive mutex</a>, this function will <i>always</i> return false when attempting to lock the mutex recursively.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmutex.html#lock">lock</a>() and <a href="http://doc.qt.io/qt-5/qmutex.html#unlock">unlock</a>().</p></div>
    pub unsafe fn try_lock_int(&mut self, timeout: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMutex_tryLock(self as *mut crate::QMutex, timeout)
    }

    /// <p>Attempts to lock the mutex. This function returns <code>true</code> if the lock was obtained; otherwise it returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMutex::try_lock()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmutex.html#try_lock">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to lock the mutex. This function returns <code>true</code> if the lock was obtained; otherwise it returns <code>false</code>.</p>
    /// <p>This function is provided for compatibility with the Standard Library concept <code>Lockable</code>. It is equivalent to <a href="http://doc.qt.io/qt-5/qmutex.html#tryLock">tryLock</a>().</p>
    /// <p>The function returns <code>true</code> if the lock was obtained; otherwise it returns <code>false</code></p>
    /// <p>This function was introduced in  Qt 5.8.</p></div>
    pub unsafe fn try_lock(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMutex_try_lock(self as *mut crate::QMutex)
    }

    /// <p>Attempts to lock the mutex. This function returns <code>true</code> if the lock was obtained; otherwise it returns <code>false</code>. If another thread has locked the mutex, this function will wait for at most <i>timeout</i> milliseconds for the mutex to become available.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMutex::tryLock()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmutex.html#tryLock">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to lock the mutex. This function returns <code>true</code> if the lock was obtained; otherwise it returns <code>false</code>. If another thread has locked the mutex, this function will wait for at most <i>timeout</i> milliseconds for the mutex to become available.</p>
    /// <p>Note: Passing a negative number as the <i>timeout</i> is equivalent to calling <a href="http://doc.qt.io/qt-5/qmutex.html#lock">lock</a>(), i.e. this function will wait forever until mutex can be locked if <i>timeout</i> is negative.</p>
    /// <p>If the lock was obtained, the mutex must be unlocked with <a href="http://doc.qt.io/qt-5/qmutex.html#unlock">unlock</a>() before another thread can successfully lock it.</p>
    /// <p>Calling this function multiple times on the same mutex from the same thread is allowed if this mutex is a <a href="http://doc.qt.io/qt-5/qmutex.html#RecursionMode-enum">recursive mutex</a>. If this mutex is a <a href="http://doc.qt.io/qt-5/qmutex.html#RecursionMode-enum">non-recursive mutex</a>, this function will <i>always</i> return false when attempting to lock the mutex recursively.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmutex.html#lock">lock</a>() and <a href="http://doc.qt.io/qt-5/qmutex.html#unlock">unlock</a>().</p></div>
    pub unsafe fn try_lock2(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMutex_tryLock1(self as *mut crate::QMutex)
    }

    /// <p>Unlocks the mutex. Attempting to unlock a mutex in a different thread to the one that locked it results in an error. Unlocking a mutex that is not locked results in undefined behavior.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMutex::unlock()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmutex.html#unlock">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Unlocks the mutex. Attempting to unlock a mutex in a different thread to the one that locked it results in an error. Unlocking a mutex that is not locked results in undefined behavior.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmutex.html#lock">lock</a>().</p></div>
    pub unsafe fn unlock(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMutex_unlock(self as *mut crate::QMutex)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qmutexlocker.html">QMutexLocker</a> class is a convenience class that simplifies locking and unlocking mutexes.</p>
///
/// C++ class: <span style='color: green;'>```QMutexLocker```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qmutexlocker.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmutexlocker.html">QMutexLocker</a> class is a convenience class that simplifies locking and unlocking mutexes.</p>
/// <p>Locking and unlocking a <a href="http://doc.qt.io/qt-5/qmutex.html">QMutex</a> in complex functions and statements or in exception handling code is error-prone and difficult to debug. <a href="http://doc.qt.io/qt-5/qmutexlocker.html">QMutexLocker</a> can be used in such situations to ensure that the state of the mutex is always well-defined.</p>
/// <p><a href="http://doc.qt.io/qt-5/qmutexlocker.html">QMutexLocker</a> should be created within a function where a <a href="http://doc.qt.io/qt-5/qmutex.html">QMutex</a> needs to be locked. The mutex is locked when <a href="http://doc.qt.io/qt-5/qmutexlocker.html">QMutexLocker</a> is created. You can unlock and relock the mutex with <code>unlock()</code> and <code>relock()</code>. If locked, the mutex will be unlocked when the <a href="http://doc.qt.io/qt-5/qmutexlocker.html">QMutexLocker</a> is destroyed.</p>
/// <p>For example, this complex function locks a <a href="http://doc.qt.io/qt-5/qmutex.html">QMutex</a> upon entering the function and unlocks the mutex at all the exit points:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> complexFunction(<span class="type">int</span> flag)
///   {
/// &#32;     mutex<span class="operator">.</span>lock();
///
/// &#32;     <span class="type">int</span> retVal <span class="operator">=</span> <span class="number">0</span>;
///
/// &#32;     <span class="keyword">switch</span> (flag) {
/// &#32;     <span class="keyword">case</span> <span class="number">0</span>:
/// &#32;     <span class="keyword">case</span> <span class="number">1</span>:
/// &#32;   &#32;     retVal <span class="operator">=</span> moreComplexFunction(flag);
/// &#32;   &#32;     <span class="keyword">break</span>;
/// &#32;     <span class="keyword">case</span> <span class="number">2</span>:
/// &#32;   &#32;     {
/// &#32;   &#32;   &#32;     <span class="type">int</span> status <span class="operator">=</span> anotherFunction();
/// &#32;   &#32;   &#32;     <span class="keyword">if</span> (status <span class="operator">&lt;</span> <span class="number">0</span>) {
/// &#32;   &#32;   &#32;   &#32;     mutex<span class="operator">.</span>unlock();
/// &#32;   &#32;   &#32;   &#32;     <span class="keyword">return</span> <span class="operator">-</span><span class="number">2</span>;
/// &#32;   &#32;   &#32;     }
/// &#32;   &#32;   &#32;     retVal <span class="operator">=</span> status <span class="operator">+</span> flag;
/// &#32;   &#32;     }
/// &#32;   &#32;     <span class="keyword">break</span>;
/// &#32;     <span class="keyword">default</span>:
/// &#32;   &#32;     <span class="keyword">if</span> (flag <span class="operator">&gt;</span> <span class="number">10</span>) {
/// &#32;   &#32;   &#32;     mutex<span class="operator">.</span>unlock();
/// &#32;   &#32;   &#32;     <span class="keyword">return</span> <span class="operator">-</span><span class="number">1</span>;
/// &#32;   &#32;     }
/// &#32;   &#32;     <span class="keyword">break</span>;
/// &#32;     }
///
/// &#32;     mutex<span class="operator">.</span>unlock();
/// &#32;     <span class="keyword">return</span> retVal;
///   }
///
/// </pre>
/// <p>This example function will get more complicated as it is developed, which increases the likelihood that errors will occur.</p>
/// <p>Using <a href="http://doc.qt.io/qt-5/qmutexlocker.html">QMutexLocker</a> greatly simplifies the code, and makes it more readable:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> complexFunction(<span class="type">int</span> flag)
///   {
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qmutexlocker.html#QMutexLocker">QMutexLocker</a></span> locker(<span class="operator">&amp;</span>mutex);
///
/// &#32;     <span class="type">int</span> retVal <span class="operator">=</span> <span class="number">0</span>;
///
/// &#32;     <span class="keyword">switch</span> (flag) {
/// &#32;     <span class="keyword">case</span> <span class="number">0</span>:
/// &#32;     <span class="keyword">case</span> <span class="number">1</span>:
/// &#32;   &#32;     <span class="keyword">return</span> moreComplexFunction(flag);
/// &#32;     <span class="keyword">case</span> <span class="number">2</span>:
/// &#32;   &#32;     {
/// &#32;   &#32;   &#32;     <span class="type">int</span> status <span class="operator">=</span> anotherFunction();
/// &#32;   &#32;   &#32;     <span class="keyword">if</span> (status <span class="operator">&lt;</span> <span class="number">0</span>)
/// &#32;   &#32;   &#32;   &#32;     <span class="keyword">return</span> <span class="operator">-</span><span class="number">2</span>;
/// &#32;   &#32;   &#32;     retVal <span class="operator">=</span> status <span class="operator">+</span> flag;
/// &#32;   &#32;     }
/// &#32;   &#32;     <span class="keyword">break</span>;
/// &#32;     <span class="keyword">default</span>:
/// &#32;   &#32;     <span class="keyword">if</span> (flag <span class="operator">&gt;</span> <span class="number">10</span>)
/// &#32;   &#32;   &#32;     <span class="keyword">return</span> <span class="operator">-</span><span class="number">1</span>;
/// &#32;   &#32;     <span class="keyword">break</span>;
/// &#32;     }
///
/// &#32;     <span class="keyword">return</span> retVal;
///   }
///
/// </pre>
/// <p>Now, the mutex will always be unlocked when the <a href="http://doc.qt.io/qt-5/qmutexlocker.html">QMutexLocker</a> object is destroyed (when the function returns since <code>locker</code> is an auto variable).</p>
/// <p>The same principle applies to code that throws and catches exceptions. An exception that is not caught in the function that has locked the mutex has no way of unlocking the mutex before the exception is passed up the stack to the calling function.</p>
/// <p><a href="http://doc.qt.io/qt-5/qmutexlocker.html">QMutexLocker</a> also provides a <code>mutex()</code> member function that returns the mutex on which the <a href="http://doc.qt.io/qt-5/qmutexlocker.html">QMutexLocker</a> is operating. This is useful for code that needs access to the mutex, such as <a href="http://doc.qt.io/qt-5/qwaitcondition.html#wait">QWaitCondition::wait</a>(). For example:</p>
/// <pre class="cpp">
///
///   <span class="keyword">class</span> SignalWaiter
///   {
///   <span class="keyword">private</span>:
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qmutexlocker.html#QMutexLocker">QMutexLocker</a></span> locker;
///
///   <span class="keyword">public</span>:
/// &#32;     SignalWaiter(<span class="type"><a href="http://doc.qt.io/qt-5/qmutex.html">QMutex</a></span> <span class="operator">*</span>mutex)
/// &#32;   &#32;     : locker(mutex)
/// &#32;     {
/// &#32;     }
///
/// &#32;     <span class="type">void</span> waitForSignal()
/// &#32;     {
/// &#32;   &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
/// &#32;   &#32;     <span class="keyword">while</span> (<span class="operator">!</span>signalled)
/// &#32;   &#32;   &#32;     waitCondition<span class="operator">.</span>wait(locker<span class="operator">.</span>mutex());
/// &#32;   &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
/// &#32;     }
///   };
///
/// </pre></div>
#[repr(C)]
pub struct QMutexLocker {
    _unused: u8,
}
impl QMutexLocker {
    /// <p>Returns the mutex on which the <a href="http://doc.qt.io/qt-5/qmutexlocker.html">QMutexLocker</a> is operating.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMutex* QMutexLocker::mutex() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmutexlocker.html#mutex">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the mutex on which the <a href="http://doc.qt.io/qt-5/qmutexlocker.html">QMutexLocker</a> is operating.</p></div>
    pub unsafe fn mutex(&self) -> ::cpp_utils::MutPtr<crate::QMutex> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMutexLocker_mutex(self as *const crate::QMutexLocker);
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qmutexlocker.html">QMutexLocker</a> and locks <i>mutex</i>. The mutex will be unlocked when the <a href="http://doc.qt.io/qt-5/qmutexlocker.html">QMutexLocker</a> is destroyed. If <i>mutex</i> is zero, <a href="http://doc.qt.io/qt-5/qmutexlocker.html">QMutexLocker</a> does nothing.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMutexLocker::QMutexLocker(QBasicMutex* m)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qmutexlocker.html#QMutexLocker">C++ documentation</a> for <span style='color: green;'>```QMutexLocker::QMutexLocker(QMutex *mutex)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qmutexlocker.html">QMutexLocker</a> and locks <i>mutex</i>. The mutex will be unlocked when the <a href="http://doc.qt.io/qt-5/qmutexlocker.html">QMutexLocker</a> is destroyed. If <i>mutex</i> is zero, <a href="http://doc.qt.io/qt-5/qmutexlocker.html">QMutexLocker</a> does nothing.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmutex.html#lock">QMutex::lock</a>().</p></div>
    pub unsafe fn new(
        m: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QBasicMutex>>,
    ) -> ::cpp_utils::CppBox<crate::QMutexLocker> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMutexLocker_QMutexLocker(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QBasicMutex>>::cast_into(m)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Relocks an unlocked mutex locker.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMutexLocker::relock()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmutexlocker.html#relock">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Relocks an unlocked mutex locker.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmutexlocker.html#unlock">unlock</a>().</p></div>
    pub unsafe fn relock(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMutexLocker_relock(self as *mut crate::QMutexLocker)
    }

    /// <p>Unlocks this mutex locker. You can use <code>relock()</code> to lock it again. It does not need to be locked when destroyed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMutexLocker::unlock()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmutexlocker.html#unlock">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Unlocks this mutex locker. You can use <code>relock()</code> to lock it again. It does not need to be locked when destroyed.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmutexlocker.html#relock">relock</a>().</p></div>
    pub unsafe fn unlock(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMutexLocker_unlock(self as *mut crate::QMutexLocker)
    }
}

/// C++ class: <span style='color: green;'>```QFutureWatcherBase```</span>.
#[repr(C)]
pub struct QFutureWatcherBase {
    _unused: u8,
}
impl QFutureWatcherBase {
    /// Returns a built-in Qt signal `QFutureWatcherBase::started` that can be passed to `qt_core::Signal::connect`.
    pub fn started(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2started()\0"),
            )
        }
    }

    /// Returns a built-in Qt signal `QFutureWatcherBase::finished` that can be passed to `qt_core::Signal::connect`.
    pub fn finished(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2finished()\0"),
            )
        }
    }

    /// Returns a built-in Qt signal `QFutureWatcherBase::canceled` that can be passed to `qt_core::Signal::connect`.
    pub fn canceled(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2canceled()\0"),
            )
        }
    }

    /// Returns a built-in Qt signal `QFutureWatcherBase::paused` that can be passed to `qt_core::Signal::connect`.
    pub fn paused(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2paused()\0"),
            )
        }
    }

    /// Returns a built-in Qt signal `QFutureWatcherBase::resumed` that can be passed to `qt_core::Signal::connect`.
    pub fn resumed(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2resumed()\0"),
            )
        }
    }

    /// Returns a built-in Qt signal `QFutureWatcherBase::resultReadyAt` that can be passed to `qt_core::Signal::connect`.
    pub fn result_ready_at(&self) -> crate::Signal<(::std::os::raw::c_int,)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2resultReadyAt(int)\0"),
            )
        }
    }

    /// Returns a built-in Qt signal `QFutureWatcherBase::resultsReadyAt` that can be passed to `qt_core::Signal::connect`.
    pub fn results_ready_at(
        &self,
    ) -> crate::Signal<(::std::os::raw::c_int, ::std::os::raw::c_int)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2resultsReadyAt(int,int)\0"),
            )
        }
    }

    /// Returns a built-in Qt signal `QFutureWatcherBase::progressRangeChanged` that can be passed to `qt_core::Signal::connect`.
    pub fn progress_range_changed(
        &self,
    ) -> crate::Signal<(::std::os::raw::c_int, ::std::os::raw::c_int)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2progressRangeChanged(int,int)\0",
                ),
            )
        }
    }

    /// Returns a built-in Qt signal `QFutureWatcherBase::progressValueChanged` that can be passed to `qt_core::Signal::connect`.
    pub fn progress_value_changed(&self) -> crate::Signal<(::std::os::raw::c_int,)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2progressValueChanged(int)\0"),
            )
        }
    }

    /// Returns a built-in Qt signal `QFutureWatcherBase::progressTextChanged` that can be passed to `qt_core::Signal::connect`.
    pub fn progress_text_changed(&self) -> crate::Signal<(*const crate::QString,)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2progressTextChanged(const QString&)\0",
                ),
            )
        }
    }

    /// Returns a built-in Qt slot `QFutureWatcherBase::cancel` that can be passed to `qt_core::Signal::connect`.
    pub fn slot_cancel(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1cancel()\0"),
            )
        }
    }

    /// Returns a built-in Qt slot `QFutureWatcherBase::setPaused` that can be passed to `qt_core::Signal::connect`.
    pub fn slot_set_paused(&self) -> crate::Receiver<(bool,)> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1setPaused(bool)\0"),
            )
        }
    }

    /// Returns a built-in Qt slot `QFutureWatcherBase::pause` that can be passed to `qt_core::Signal::connect`.
    pub fn slot_pause(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1pause()\0"),
            )
        }
    }

    /// Returns a built-in Qt slot `QFutureWatcherBase::resume` that can be passed to `qt_core::Signal::connect`.
    pub fn slot_resume(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1resume()\0"),
            )
        }
    }

    /// Returns a built-in Qt slot `QFutureWatcherBase::togglePaused` that can be passed to `qt_core::Signal::connect`.
    pub fn slot_toggle_paused(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1togglePaused()\0"),
            )
        }
    }

    /// Calls C++ function: <span style='color: green;'>```[slot] void QFutureWatcherBase::cancel()```</span>.
    pub unsafe fn cancel(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QFutureWatcherBase_cancel(
            self as *mut crate::QFutureWatcherBase,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual bool QFutureWatcherBase::event(QEvent* event)```</span>.
    pub unsafe fn event(
        &mut self,
        event: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QEvent>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFutureWatcherBase_event(
            self as *mut crate::QFutureWatcherBase,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QEvent>>::cast_into(event)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QFutureWatcherBase::isCanceled() const```</span>.
    pub unsafe fn is_canceled(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFutureWatcherBase_isCanceled(
            self as *const crate::QFutureWatcherBase,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QFutureWatcherBase::isFinished() const```</span>.
    pub unsafe fn is_finished(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFutureWatcherBase_isFinished(
            self as *const crate::QFutureWatcherBase,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QFutureWatcherBase::isPaused() const```</span>.
    pub unsafe fn is_paused(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFutureWatcherBase_isPaused(
            self as *const crate::QFutureWatcherBase,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QFutureWatcherBase::isRunning() const```</span>.
    pub unsafe fn is_running(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFutureWatcherBase_isRunning(
            self as *const crate::QFutureWatcherBase,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QFutureWatcherBase::isStarted() const```</span>.
    pub unsafe fn is_started(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFutureWatcherBase_isStarted(
            self as *const crate::QFutureWatcherBase,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QFutureWatcherBase::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFutureWatcherBase_metaObject(
            self as *const crate::QFutureWatcherBase,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```[slot] void QFutureWatcherBase::pause()```</span>.
    pub unsafe fn pause(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QFutureWatcherBase_pause(
            self as *mut crate::QFutureWatcherBase,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```int QFutureWatcherBase::progressMaximum() const```</span>.
    pub unsafe fn progress_maximum(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QFutureWatcherBase_progressMaximum(
            self as *const crate::QFutureWatcherBase,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```int QFutureWatcherBase::progressMinimum() const```</span>.
    pub unsafe fn progress_minimum(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QFutureWatcherBase_progressMinimum(
            self as *const crate::QFutureWatcherBase,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```QString QFutureWatcherBase::progressText() const```</span>.
    pub unsafe fn progress_text(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFutureWatcherBase_progressText(
            self as *const crate::QFutureWatcherBase,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```int QFutureWatcherBase::progressValue() const```</span>.
    pub unsafe fn progress_value(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QFutureWatcherBase_progressValue(
            self as *const crate::QFutureWatcherBase,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QFutureWatcherBase::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QFutureWatcherBase_qt_metacall(
            self as *mut crate::QFutureWatcherBase,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QFutureWatcherBase::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFutureWatcherBase_qt_metacast(
            self as *mut crate::QFutureWatcherBase,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```[slot] void QFutureWatcherBase::resume()```</span>.
    pub unsafe fn resume(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QFutureWatcherBase_resume(
            self as *mut crate::QFutureWatcherBase,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```[slot] void QFutureWatcherBase::setPaused(bool paused)```</span>.
    pub unsafe fn set_paused(&mut self, paused: bool) {
        crate::__ffi::ctr_qt_core_ffi_QFutureWatcherBase_setPaused(
            self as *mut crate::QFutureWatcherBase,
            paused,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QFutureWatcherBase::setPendingResultsLimit(int limit)```</span>.
    pub unsafe fn set_pending_results_limit(&mut self, limit: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QFutureWatcherBase_setPendingResultsLimit(
            self as *mut crate::QFutureWatcherBase,
            limit,
        )
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFutureWatcherBase_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```[slot] void QFutureWatcherBase::togglePaused()```</span>.
    pub unsafe fn toggle_paused(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QFutureWatcherBase_togglePaused(
            self as *mut crate::QFutureWatcherBase,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QFutureWatcherBase::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFutureWatcherBase_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QFutureWatcherBase::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFutureWatcherBase_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```void QFutureWatcherBase::waitForFinished()```</span>.
    pub unsafe fn wait_for_finished(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QFutureWatcherBase_waitForFinished(
            self as *mut crate::QFutureWatcherBase,
        )
    }
}

pub mod q_history_state {
    //! C++ type: <span style='color: green;'>```QHistoryState```</span>

    /// <p>This enum specifies the type of history that a <a href="http://doc.qt.io/qt-5/qhistorystate.html">QHistoryState</a> records.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QHistoryState::HistoryType```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhistorystate.html#HistoryType-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum specifies the type of history that a <a href="http://doc.qt.io/qt-5/qhistorystate.html">QHistoryState</a> records.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct HistoryType(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for HistoryType {
        fn from(value: ::std::os::raw::c_int) -> Self {
            HistoryType(value)
        }
    }

    impl From<HistoryType> for ::std::os::raw::c_int {
        fn from(value: HistoryType) -> Self {
            value.0
        }
    }

    impl HistoryType {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl HistoryType {
        /// Only the immediate child states of the parent state are recorded. In this case a transition with the history state as its target will end up in the immediate child state that the parent was in the last time it was exited. This is the default. (C++ enum variant: <span style='color: green;'>```ShallowHistory = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const ShallowHistory: crate::q_history_state::HistoryType =
            crate::q_history_state::HistoryType(0);
        /// Nested states are recorded. In this case a transition with the history state as its target will end up in the most deeply nested descendant state the parent was in the last time it was exited. (C++ enum variant: <span style='color: green;'>```DeepHistory = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const DeepHistory: crate::q_history_state::HistoryType =
            crate::q_history_state::HistoryType(1);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qhistorystate.html">QHistoryState</a> class provides a means of returning to a previously active substate.</p>
///
/// C++ class: <span style='color: green;'>```QHistoryState```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhistorystate.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhistorystate.html">QHistoryState</a> class provides a means of returning to a previously active substate.</p>
/// <p>A history state is a pseudo-state that represents the child state that the parent state was in the last time the parent state was exited. A transition with a history state as its target is in fact a transition to one or more other child states of the parent state. <a href="http://doc.qt.io/qt-5/qhistorystate.html">QHistoryState</a> is part of <a href="http://doc.qt.io/qt-5/statemachine-api.html">The State Machine Framework</a>.</p>
/// <p>Use the <a href="http://doc.qt.io/qt-5/qhistorystate.html#setDefaultState">setDefaultState</a>() function to set the state that should be entered if the parent state has never been entered. Example:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a></span> machine;
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span> <span class="operator">*</span>s1 <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span>();
///   <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span> <span class="operator">*</span>s11 <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span>(s1);
///   <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span> <span class="operator">*</span>s12 <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span>(s1);
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qhistorystate.html#QHistoryState">QHistoryState</a></span> <span class="operator">*</span>s1h <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qhistorystate.html#QHistoryState">QHistoryState</a></span>(s1);
///   s1h<span class="operator">-</span><span class="operator">&gt;</span>setDefaultState(s11);
///
///   machine<span class="operator">.</span>addState(s1);
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span> <span class="operator">*</span>s2 <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span>();
///   machine<span class="operator">.</span>addState(s2);
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span>button <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span>();
///   <span class="comment">// Clicking the button will cause the state machine to enter the child state</span>
///   <span class="comment">// that s1 was in the last time s1 was exited, or the history state's default</span>
///   <span class="comment">// state if s1 has never been entered.</span>
///   s1<span class="operator">-</span><span class="operator">&gt;</span>addTransition(button<span class="operator">,</span> SIGNAL(clicked())<span class="operator">,</span> s1h);
///
/// </pre>
/// <p>If more than one default state has to be entered, or if the transition to the default state(s) has to be acted upon, the <a href="http://doc.qt.io/qt-5/qhistorystate.html#defaultTransition-prop">defaultTransition</a> should be set instead. Note that the eventTest() method of that transition will never be called: the selection and execution of the transition is done automatically when entering the history state.</p>
/// <p>By default a history state is shallow, meaning that it won't remember nested states. This can be configured through the <a href="http://doc.qt.io/qt-5/qhistorystate.html#historyType-prop">historyType</a> property.</p></div>
#[repr(C)]
pub struct QHistoryState {
    _unused: u8,
}
impl QHistoryState {
    /// <p>This signal is emitted when the <a href="http://doc.qt.io/qt-5/qhistorystate.html#defaultTransition-prop">defaultTransition</a> property is changed.</p>
    ///
    /// Returns a built-in Qt signal `QHistoryState::defaultTransitionChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhistorystate.html#defaultTransitionChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the <a href="http://doc.qt.io/qt-5/qhistorystate.html#defaultTransition-prop">defaultTransition</a> property is changed.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>Note:</b> Notifier signal for property <a href="http://doc.qt.io/qt-5/qhistorystate.html#defaultTransition-prop">defaultTransition</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qhistorystate.html#defaultTransition-prop">QHistoryState::defaultTransition</a>.</p></div>
    pub fn default_transition_changed(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2defaultTransitionChanged()\0"),
            )
        }
    }

    /// <p>This signal is emitted when the <a href="http://doc.qt.io/qt-5/qhistorystate.html#defaultState-prop">defaultState</a> property is changed.</p>
    ///
    /// Returns a built-in Qt signal `QHistoryState::defaultStateChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhistorystate.html#defaultStateChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the <a href="http://doc.qt.io/qt-5/qhistorystate.html#defaultState-prop">defaultState</a> property is changed.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>Note:</b> Notifier signal for property <a href="http://doc.qt.io/qt-5/qhistorystate.html#defaultState-prop">defaultState</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qhistorystate.html#defaultState-prop">QHistoryState::defaultState</a>.</p></div>
    pub fn default_state_changed(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2defaultStateChanged()\0"),
            )
        }
    }

    /// <p>This signal is emitted when the <a href="http://doc.qt.io/qt-5/qhistorystate.html#historyType-prop">historyType</a> property is changed.</p>
    ///
    /// Returns a built-in Qt signal `QHistoryState::historyTypeChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhistorystate.html#historyTypeChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the <a href="http://doc.qt.io/qt-5/qhistorystate.html#historyType-prop">historyType</a> property is changed.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>Note:</b> Notifier signal for property <a href="http://doc.qt.io/qt-5/qhistorystate.html#historyType-prop">historyType</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qhistorystate.html#historyType-prop">QHistoryState::historyType</a>.</p></div>
    pub fn history_type_changed(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2historyTypeChanged()\0"),
            )
        }
    }

    /// <p>This property holds the default state of this history state</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractState* QHistoryState::defaultState() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhistorystate.html#defaultState-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the default state of this history state</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QAbstractState *</td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qhistorystate.html#defaultState">defaultState</a></b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qhistorystate.html#setDefaultState">setDefaultState</a></b></span>(QAbstractState *<i>state</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qhistorystate.html#defaultStateChanged">defaultStateChanged</a></b></span>()</td><td class="memItemRight bottomAlign">[see note below]</td></tr>
    /// </tbody></table></div>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p></div>
    pub unsafe fn default_state(&self) -> ::cpp_utils::MutPtr<crate::QAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHistoryState_defaultState(
            self as *const crate::QHistoryState,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This property holds the default transition of this history state</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractTransition* QHistoryState::defaultTransition() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhistorystate.html#defaultTransition-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the default transition of this history state</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QAbstractTransition *</td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qhistorystate.html#defaultTransition">defaultTransition</a></b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qhistorystate.html#setDefaultTransition">setDefaultTransition</a></b></span>(QAbstractTransition *<i>transition</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qhistorystate.html#defaultTransitionChanged">defaultTransitionChanged</a></b></span>()</td><td class="memItemRight bottomAlign">[see note below]</td></tr>
    /// </tbody></table></div>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p></div>
    pub unsafe fn default_transition(&self) -> ::cpp_utils::MutPtr<crate::QAbstractTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHistoryState_defaultTransition(
            self as *const crate::QHistoryState,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This property holds the type of history that this history state records</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHistoryState::HistoryType QHistoryState::historyType() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhistorystate.html#historyType-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the type of history that this history state records</p>
    /// <p>The default value of this property is <a href="http://doc.qt.io/qt-5/qhistorystate.html#HistoryType-enum">QHistoryState::ShallowHistory</a>.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> HistoryType </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qhistorystate.html#historyType">historyType</a></b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qhistorystate.html#setHistoryType">setHistoryType</a></b></span>(HistoryType <i>type</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qhistorystate.html#historyTypeChanged">historyTypeChanged</a></b></span>()</td><td class="memItemRight bottomAlign">[see note below]</td></tr>
    /// </tbody></table></div>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p></div>
    pub unsafe fn history_type(&self) -> crate::q_history_state::HistoryType {
        crate::__ffi::ctr_qt_core_ffi_QHistoryState_historyType(self as *const crate::QHistoryState)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QHistoryState::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHistoryState_metaObject(
            self as *const crate::QHistoryState,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs a new shallow history state with the given <i>parent</i> state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QHistoryState::QHistoryState(QState* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhistorystate.html#QHistoryState">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new shallow history state with the given <i>parent</i> state.</p></div>
    pub unsafe fn from_q_state(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QState>>,
    ) -> ::cpp_utils::CppBox<crate::QHistoryState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHistoryState_QHistoryState(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QState>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new history state of the given <i>type</i>, with the given <i>parent</i> state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QHistoryState::QHistoryState(QHistoryState::HistoryType type, QState* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhistorystate.html#QHistoryState-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new history state of the given <i>type</i>, with the given <i>parent</i> state.</p></div>
    pub unsafe fn from_history_type_q_state(
        type_: crate::q_history_state::HistoryType,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QState>>,
    ) -> ::cpp_utils::CppBox<crate::QHistoryState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHistoryState_QHistoryState1(
            type_,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QState>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qhistorystate.html">QHistoryState</a> class provides a means of returning to a previously active substate.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QHistoryState::QHistoryState()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhistorystate.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhistorystate.html">QHistoryState</a> class provides a means of returning to a previously active substate.</p>
    /// <p>A history state is a pseudo-state that represents the child state that the parent state was in the last time the parent state was exited. A transition with a history state as its target is in fact a transition to one or more other child states of the parent state. <a href="http://doc.qt.io/qt-5/qhistorystate.html">QHistoryState</a> is part of <a href="http://doc.qt.io/qt-5/statemachine-api.html">The State Machine Framework</a>.</p>
    /// <p>Use the <a href="http://doc.qt.io/qt-5/qhistorystate.html#setDefaultState">setDefaultState</a>() function to set the state that should be entered if the parent state has never been entered. Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a></span> machine;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span> <span class="operator">*</span>s1 <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span>();
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span> <span class="operator">*</span>s11 <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span>(s1);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span> <span class="operator">*</span>s12 <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span>(s1);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhistorystate.html#QHistoryState">QHistoryState</a></span> <span class="operator">*</span>s1h <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qhistorystate.html#QHistoryState">QHistoryState</a></span>(s1);
    ///   s1h<span class="operator">-</span><span class="operator">&gt;</span>setDefaultState(s11);
    ///
    ///   machine<span class="operator">.</span>addState(s1);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span> <span class="operator">*</span>s2 <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span>();
    ///   machine<span class="operator">.</span>addState(s2);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span>button <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span>();
    ///   <span class="comment">// Clicking the button will cause the state machine to enter the child state</span>
    ///   <span class="comment">// that s1 was in the last time s1 was exited, or the history state's default</span>
    ///   <span class="comment">// state if s1 has never been entered.</span>
    ///   s1<span class="operator">-</span><span class="operator">&gt;</span>addTransition(button<span class="operator">,</span> SIGNAL(clicked())<span class="operator">,</span> s1h);
    ///
    /// </pre>
    /// <p>If more than one default state has to be entered, or if the transition to the default state(s) has to be acted upon, the <a href="http://doc.qt.io/qt-5/qhistorystate.html#defaultTransition-prop">defaultTransition</a> should be set instead. Note that the eventTest() method of that transition will never be called: the selection and execution of the transition is done automatically when entering the history state.</p>
    /// <p>By default a history state is shallow, meaning that it won't remember nested states. This can be configured through the <a href="http://doc.qt.io/qt-5/qhistorystate.html#historyType-prop">historyType</a> property.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QHistoryState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHistoryState_QHistoryState2();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new history state of the given <i>type</i>, with the given <i>parent</i> state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QHistoryState::QHistoryState(QHistoryState::HistoryType type)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhistorystate.html#QHistoryState-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new history state of the given <i>type</i>, with the given <i>parent</i> state.</p></div>
    pub unsafe fn from_history_type(
        type_: crate::q_history_state::HistoryType,
    ) -> ::cpp_utils::CppBox<crate::QHistoryState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHistoryState_QHistoryState3(type_);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QHistoryState::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QHistoryState_qt_metacall(
            self as *mut crate::QHistoryState,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QHistoryState::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHistoryState_qt_metacast(
            self as *mut crate::QHistoryState,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Sets this history state's default state to be the given <i>state</i>. <i>state</i> must be a sibling of this history state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QHistoryState::setDefaultState(QAbstractState* state)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhistorystate.html#setDefaultState">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets this history state's default state to be the given <i>state</i>. <i>state</i> must be a sibling of this history state.</p>
    /// <p>Note that this function does not set <i>state</i> as the initial state of its parent.</p>
    /// <p><b>Note:</b> Setter function for property <a href="http://doc.qt.io/qt-5/qhistorystate.html#defaultState-prop">defaultState</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qhistorystate.html#defaultState">defaultState</a>().</p></div>
    pub unsafe fn set_default_state(
        &mut self,
        state: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractState>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QHistoryState_setDefaultState(
            self as *mut crate::QHistoryState,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractState>>::cast_into(state)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Sets this history state's default transition to be the given <i>transition</i>. This will set the source state of the <i>transition</i> to the history state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QHistoryState::setDefaultTransition(QAbstractTransition* transition)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhistorystate.html#setDefaultTransition">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets this history state's default transition to be the given <i>transition</i>. This will set the source state of the <i>transition</i> to the history state.</p>
    /// <p>Note that the eventTest method of the <i>transition</i> will never be called.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>Note:</b> Setter function for property <a href="http://doc.qt.io/qt-5/qhistorystate.html#defaultTransition-prop">defaultTransition</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qhistorystate.html#defaultTransition">defaultTransition</a>().</p></div>
    pub unsafe fn set_default_transition(
        &mut self,
        transition: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractTransition>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QHistoryState_setDefaultTransition(
            self as *mut crate::QHistoryState,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractTransition>>::cast_into(
                transition,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Sets the <i>type</i> of history that this history state records.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QHistoryState::setHistoryType(QHistoryState::HistoryType type)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhistorystate.html#setHistoryType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the <i>type</i> of history that this history state records.</p>
    /// <p><b>Note:</b> Setter function for property <a href="http://doc.qt.io/qt-5/qhistorystate.html#historyType-prop">historyType</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qhistorystate.html#historyType">historyType</a>().</p></div>
    pub unsafe fn set_history_type(&mut self, type_: crate::q_history_state::HistoryType) {
        crate::__ffi::ctr_qt_core_ffi_QHistoryState_setHistoryType(
            self as *mut crate::QHistoryState,
            type_,
        )
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHistoryState_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QHistoryState::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHistoryState_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QHistoryState::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHistoryState_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html">QIdentityProxyModel</a> class proxies its source model unmodified</p>
///
/// C++ class: <span style='color: green;'>```QIdentityProxyModel```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html">QIdentityProxyModel</a> class proxies its source model unmodified</p>
/// <p><a href="http://doc.qt.io/qt-5/qidentityproxymodel.html">QIdentityProxyModel</a> can be used to forward the structure of a source model exactly, with no sorting, filtering or other transformation. This is similar in concept to an identity matrix where A.I = A.</p>
/// <p>Because it does no sorting or filtering, this class is most suitable to proxy models which transform the <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#data">data</a>() of the source model. For example, a proxy model could be created to define the font used, or the background colour, or the tooltip etc. This removes the need to implement all data handling in the same class that creates the structure of the model, and can also be used to create re-usable components.</p>
/// <p>This also provides a way to change the data in the case where a source model is supplied by a third party which can not be modified.</p>
/// <pre class="cpp">
///
///   <span class="keyword">class</span> DateFormatProxyModel : <span class="keyword">public</span> <span class="type"><a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#QIdentityProxyModel">QIdentityProxyModel</a></span>
///   {
/// &#32;   <span class="comment">// ...</span>
///
/// &#32;   <span class="type">void</span> setDateFormatString(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>formatString)
/// &#32;   {
/// &#32;     m_formatString <span class="operator">=</span> formatString;
/// &#32;   }
///
/// &#32;   <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> data(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a></span> <span class="operator">&amp;</span>index<span class="operator">,</span> <span class="type">int</span> role) <span class="keyword">const</span>
/// &#32;   {
/// &#32;     <span class="keyword">if</span> (role <span class="operator">!</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>DisplayRole)
/// &#32;   &#32;   <span class="keyword">return</span> <span class="type"><a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#QIdentityProxyModel">QIdentityProxyModel</a></span><span class="operator">::</span>data(index<span class="operator">,</span> role);
///
/// &#32;     <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a></span> dateTime <span class="operator">=</span> sourceModel()<span class="operator">-</span><span class="operator">&gt;</span>data(SourceClass<span class="operator">::</span>DateRole)<span class="operator">.</span>toDateTime();
///
/// &#32;     <span class="keyword">return</span> dateTime<span class="operator">.</span>toString(m_formatString);
/// &#32;   }
///
///   <span class="keyword">private</span>:
/// &#32;   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> m_formatString;
///   };
///
/// </pre></div>
#[repr(C)]
pub struct QIdentityProxyModel {
    _unused: u8,
}
impl QIdentityProxyModel {
    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnCount">QAbstractItemModel::columnCount</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual int QIdentityProxyModel::columnCount(const QModelIndex& parent = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#columnCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnCount">QAbstractItemModel::columnCount</a>().</p></div>
    pub unsafe fn column_count_1a(
        &self,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_columnCount(
            self as *const crate::QIdentityProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnCount">QAbstractItemModel::columnCount</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual int QIdentityProxyModel::columnCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#columnCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnCount">QAbstractItemModel::columnCount</a>().</p></div>
    pub unsafe fn column_count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_columnCount1(
            self as *const crate::QIdentityProxyModel,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dropMimeData">QAbstractItemModel::dropMimeData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QIdentityProxyModel::dropMimeData(const QMimeData* data, Qt::DropAction action, int row, int column, const QModelIndex& parent)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#dropMimeData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dropMimeData">QAbstractItemModel::dropMimeData</a>().</p></div>
    pub unsafe fn drop_mime_data(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QMimeData>>,
        action: crate::DropAction,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_dropMimeData(
            self as *mut crate::QIdentityProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QMimeData>>::cast_into(data)
                .as_raw_ptr(),
            action,
            row,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerData">QAbstractItemModel::headerData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QVariant QIdentityProxyModel::headerData(int section, Qt::Orientation orientation, int role = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#headerData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerData">QAbstractItemModel::headerData</a>().</p></div>
    pub unsafe fn header_data_3a(
        &self,
        section: ::std::os::raw::c_int,
        orientation: crate::Orientation,
        role: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_headerData(
            self as *const crate::QIdentityProxyModel,
            section,
            orientation,
            role,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerData">QAbstractItemModel::headerData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QVariant QIdentityProxyModel::headerData(int section, Qt::Orientation orientation) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#headerData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerData">QAbstractItemModel::headerData</a>().</p></div>
    pub unsafe fn header_data_2a(
        &self,
        section: ::std::os::raw::c_int,
        orientation: crate::Orientation,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_headerData1(
            self as *const crate::QIdentityProxyModel,
            section,
            orientation,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QIdentityProxyModel::index(int row, int column, const QModelIndex& parent = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#index">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>().</p></div>
    pub unsafe fn index_3a(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_index(
            self as *const crate::QIdentityProxyModel,
            row,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QIdentityProxyModel::index(int row, int column) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#index">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>().</p></div>
    pub unsafe fn index_2a(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_index1(
            self as *const crate::QIdentityProxyModel,
            row,
            column,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumns">QAbstractItemModel::insertColumns</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QIdentityProxyModel::insertColumns(int column, int count, const QModelIndex& parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#insertColumns">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumns">QAbstractItemModel::insertColumns</a>().</p></div>
    pub unsafe fn insert_columns_3a(
        &mut self,
        column: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_insertColumns(
            self as *mut crate::QIdentityProxyModel,
            column,
            count,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumns">QAbstractItemModel::insertColumns</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QIdentityProxyModel::insertColumns(int column, int count)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#insertColumns">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumns">QAbstractItemModel::insertColumns</a>().</p></div>
    pub unsafe fn insert_columns_2a(
        &mut self,
        column: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_insertColumns1(
            self as *mut crate::QIdentityProxyModel,
            column,
            count,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">QAbstractItemModel::insertRows</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QIdentityProxyModel::insertRows(int row, int count, const QModelIndex& parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#insertRows">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">QAbstractItemModel::insertRows</a>().</p></div>
    pub unsafe fn insert_rows_3a(
        &mut self,
        row: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_insertRows(
            self as *mut crate::QIdentityProxyModel,
            row,
            count,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">QAbstractItemModel::insertRows</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QIdentityProxyModel::insertRows(int row, int count)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#insertRows">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">QAbstractItemModel::insertRows</a>().</p></div>
    pub unsafe fn insert_rows_2a(
        &mut self,
        row: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_insertRows1(
            self as *mut crate::QIdentityProxyModel,
            row,
            count,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapFromSource">QAbstractProxyModel::mapFromSource</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QIdentityProxyModel::mapFromSource(const QModelIndex& sourceIndex) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#mapFromSource">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapFromSource">QAbstractProxyModel::mapFromSource</a>().</p></div>
    pub unsafe fn map_from_source(
        &self,
        source_index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_mapFromSource(
            self as *const crate::QIdentityProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(source_index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapSelectionFromSource">QAbstractProxyModel::mapSelectionFromSource</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QItemSelection QIdentityProxyModel::mapSelectionFromSource(const QItemSelection& selection) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#mapSelectionFromSource">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapSelectionFromSource">QAbstractProxyModel::mapSelectionFromSource</a>().</p></div>
    pub unsafe fn map_selection_from_source(
        &self,
        selection: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelection>>,
    ) -> ::cpp_utils::CppBox<crate::QItemSelection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_mapSelectionFromSource(
            self as *const crate::QIdentityProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelection>>::cast_into(selection)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapSelectionToSource">QAbstractProxyModel::mapSelectionToSource</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QItemSelection QIdentityProxyModel::mapSelectionToSource(const QItemSelection& selection) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#mapSelectionToSource">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapSelectionToSource">QAbstractProxyModel::mapSelectionToSource</a>().</p></div>
    pub unsafe fn map_selection_to_source(
        &self,
        selection: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelection>>,
    ) -> ::cpp_utils::CppBox<crate::QItemSelection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_mapSelectionToSource(
            self as *const crate::QIdentityProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelection>>::cast_into(selection)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapToSource">QAbstractProxyModel::mapToSource</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QIdentityProxyModel::mapToSource(const QModelIndex& proxyIndex) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#mapToSource">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapToSource">QAbstractProxyModel::mapToSource</a>().</p></div>
    pub unsafe fn map_to_source(
        &self,
        proxy_index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_mapToSource(
            self as *const crate::QIdentityProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(proxy_index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#match">QAbstractItemModel::match</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QList<QModelIndex> QIdentityProxyModel::match(const QModelIndex& start, int role, const QVariant& value, int hits = …, QFlags<Qt::MatchFlag> flags = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#match">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#match">QAbstractItemModel::match</a>().</p></div>
    pub unsafe fn match_5a(
        &self,
        start: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        role: ::std::os::raw::c_int,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
        hits: ::std::os::raw::c_int,
        flags: crate::QFlags<crate::MatchFlag>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_match(
            self as *const crate::QIdentityProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(start)
                .as_raw_ptr(),
            role,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
            hits,
            flags.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#match">QAbstractItemModel::match</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QList<QModelIndex> QIdentityProxyModel::match(const QModelIndex& start, int role, const QVariant& value, int hits = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#match">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#match">QAbstractItemModel::match</a>().</p></div>
    pub unsafe fn match_4a(
        &self,
        start: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        role: ::std::os::raw::c_int,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
        hits: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_match1(
            self as *const crate::QIdentityProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(start)
                .as_raw_ptr(),
            role,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
            hits,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#match">QAbstractItemModel::match</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QList<QModelIndex> QIdentityProxyModel::match(const QModelIndex& start, int role, const QVariant& value) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#match">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#match">QAbstractItemModel::match</a>().</p></div>
    pub unsafe fn match_3a(
        &self,
        start: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        role: ::std::os::raw::c_int,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_match2(
            self as *const crate::QIdentityProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(start)
                .as_raw_ptr(),
            role,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QIdentityProxyModel::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_metaObject(
            self as *const crate::QIdentityProxyModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs an identity model with the given <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QIdentityProxyModel::QIdentityProxyModel(QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#QIdentityProxyModel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an identity model with the given <i>parent</i>.</p></div>
    pub unsafe fn new_1a(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QIdentityProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_QIdentityProxyModel(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html">QIdentityProxyModel</a> class proxies its source model unmodified</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QIdentityProxyModel::QIdentityProxyModel()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html">QIdentityProxyModel</a> class proxies its source model unmodified</p>
    /// <p><a href="http://doc.qt.io/qt-5/qidentityproxymodel.html">QIdentityProxyModel</a> can be used to forward the structure of a source model exactly, with no sorting, filtering or other transformation. This is similar in concept to an identity matrix where A.I = A.</p>
    /// <p>Because it does no sorting or filtering, this class is most suitable to proxy models which transform the <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#data">data</a>() of the source model. For example, a proxy model could be created to define the font used, or the background colour, or the tooltip etc. This removes the need to implement all data handling in the same class that creates the structure of the model, and can also be used to create re-usable components.</p>
    /// <p>This also provides a way to change the data in the case where a source model is supplied by a third party which can not be modified.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">class</span> DateFormatProxyModel : <span class="keyword">public</span> <span class="type"><a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#QIdentityProxyModel">QIdentityProxyModel</a></span>
    ///   {
    /// &#32;   <span class="comment">// ...</span>
    ///
    /// &#32;   <span class="type">void</span> setDateFormatString(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>formatString)
    /// &#32;   {
    /// &#32;     m_formatString <span class="operator">=</span> formatString;
    /// &#32;   }
    ///
    /// &#32;   <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> data(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a></span> <span class="operator">&amp;</span>index<span class="operator">,</span> <span class="type">int</span> role) <span class="keyword">const</span>
    /// &#32;   {
    /// &#32;     <span class="keyword">if</span> (role <span class="operator">!</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>DisplayRole)
    /// &#32;   &#32;   <span class="keyword">return</span> <span class="type"><a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#QIdentityProxyModel">QIdentityProxyModel</a></span><span class="operator">::</span>data(index<span class="operator">,</span> role);
    ///
    /// &#32;     <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a></span> dateTime <span class="operator">=</span> sourceModel()<span class="operator">-</span><span class="operator">&gt;</span>data(SourceClass<span class="operator">::</span>DateRole)<span class="operator">.</span>toDateTime();
    ///
    /// &#32;     <span class="keyword">return</span> dateTime<span class="operator">.</span>toString(m_formatString);
    /// &#32;   }
    ///
    ///   <span class="keyword">private</span>:
    /// &#32;   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> m_formatString;
    ///   };
    ///
    /// </pre></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QIdentityProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_QIdentityProxyModel1();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#parent">QAbstractItemModel::parent</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QIdentityProxyModel::parent(const QModelIndex& child) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#parent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#parent">QAbstractItemModel::parent</a>().</p></div>
    pub unsafe fn parent(
        &self,
        child: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_parent(
            self as *const crate::QIdentityProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(child)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QIdentityProxyModel::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_qt_metacall(
            self as *mut crate::QIdentityProxyModel,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QIdentityProxyModel::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_qt_metacast(
            self as *mut crate::QIdentityProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumns">QAbstractItemModel::removeColumns</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QIdentityProxyModel::removeColumns(int column, int count, const QModelIndex& parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#removeColumns">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumns">QAbstractItemModel::removeColumns</a>().</p></div>
    pub unsafe fn remove_columns_3a(
        &mut self,
        column: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_removeColumns(
            self as *mut crate::QIdentityProxyModel,
            column,
            count,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumns">QAbstractItemModel::removeColumns</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QIdentityProxyModel::removeColumns(int column, int count)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#removeColumns">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumns">QAbstractItemModel::removeColumns</a>().</p></div>
    pub unsafe fn remove_columns_2a(
        &mut self,
        column: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_removeColumns1(
            self as *mut crate::QIdentityProxyModel,
            column,
            count,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">QAbstractItemModel::removeRows</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QIdentityProxyModel::removeRows(int row, int count, const QModelIndex& parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#removeRows">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">QAbstractItemModel::removeRows</a>().</p></div>
    pub unsafe fn remove_rows_3a(
        &mut self,
        row: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_removeRows(
            self as *mut crate::QIdentityProxyModel,
            row,
            count,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">QAbstractItemModel::removeRows</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QIdentityProxyModel::removeRows(int row, int count)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#removeRows">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">QAbstractItemModel::removeRows</a>().</p></div>
    pub unsafe fn remove_rows_2a(
        &mut self,
        row: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_removeRows1(
            self as *mut crate::QIdentityProxyModel,
            row,
            count,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">QAbstractItemModel::rowCount</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual int QIdentityProxyModel::rowCount(const QModelIndex& parent = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#rowCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">QAbstractItemModel::rowCount</a>().</p></div>
    pub unsafe fn row_count_1a(
        &self,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_rowCount(
            self as *const crate::QIdentityProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">QAbstractItemModel::rowCount</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual int QIdentityProxyModel::rowCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#rowCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">QAbstractItemModel::rowCount</a>().</p></div>
    pub unsafe fn row_count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_rowCount1(
            self as *const crate::QIdentityProxyModel,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#setSourceModel">QAbstractProxyModel::setSourceModel</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual void QIdentityProxyModel::setSourceModel(QAbstractItemModel* sourceModel)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#setSourceModel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#setSourceModel">QAbstractProxyModel::setSourceModel</a>().</p></div>
    pub unsafe fn set_source_model(
        &mut self,
        source_model: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractItemModel>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_setSourceModel(
            self as *mut crate::QIdentityProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractItemModel>>::cast_into(
                source_model,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sibling">QAbstractItemModel::sibling</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QIdentityProxyModel::sibling(int row, int column, const QModelIndex& idx) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#sibling">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sibling">QAbstractItemModel::sibling</a>().</p></div>
    pub unsafe fn sibling(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
        idx: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_sibling(
            self as *const crate::QIdentityProxyModel,
            row,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(idx)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QIdentityProxyModel::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QIdentityProxyModel::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qitemselectionrange.html">QItemSelectionRange</a> class manages information about a range of selected items in a model.</p>
///
/// C++ class: <span style='color: green;'>```QItemSelectionRange```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qitemselectionrange.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qitemselectionrange.html">QItemSelectionRange</a> class manages information about a range of selected items in a model.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qitemselectionrange.html">QItemSelectionRange</a> contains information about a range of selected items in a model. A range of items is a contiguous array of model items, extending to cover a number of adjacent rows and columns with a common parent item; this can be visualized as a two-dimensional block of cells in a table. A selection range has a <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#top">top</a>(), <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#left">left</a>() a <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#bottom">bottom</a>(), <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#right">right</a>() and a <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#parent">parent</a>().</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qitemselectionrange.html">QItemSelectionRange</a> class is one of the <a href="http://doc.qt.io/qt-5/model-view-programming.html#model-view-classes">Model/View Classes</a> and is part of Qt's <a href="http://doc.qt.io/qt-5/model-view-programming.html">model/view framework</a>.</p>
/// <p>The model items contained in the selection range can be obtained using the <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#indexes">indexes</a>() function. Use <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectedIndexes">QItemSelectionModel::selectedIndexes</a>() to get a list of all selected items for a view.</p>
/// <p>You can determine whether a given model item lies within a particular range by using the <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#contains">contains</a>() function. Ranges can also be compared using the overloaded operators for equality and inequality, and the <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#intersects">intersects</a>() function allows you to determine whether two ranges overlap.</p></div>
#[repr(C)]
pub struct QItemSelectionRange {
    _unused: u8,
}
impl QItemSelectionRange {
    /// <p>Returns the row index corresponding to the lowermost selected row in the selection range.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QItemSelectionRange::bottom() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#bottom">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the row index corresponding to the lowermost selected row in the selection range.</p></div>
    pub unsafe fn bottom(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionRange_bottom(
            self as *const crate::QItemSelectionRange,
        )
    }

    /// <p>Returns the index for the item located at the bottom-right corner of the selection range.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPersistentModelIndex& QItemSelectionRange::bottomRight() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#bottomRight">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index for the item located at the bottom-right corner of the selection range.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qitemselectionrange.html#bottom">bottom</a>(), <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#right">right</a>(), and <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#topLeft">topLeft</a>().</p></div>
    pub unsafe fn bottom_right(&self) -> ::cpp_utils::Ref<crate::QPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionRange_bottomRight(
            self as *const crate::QItemSelectionRange,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the model item specified by the <i>index</i> lies within the range of selected items; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QItemSelectionRange::contains(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the model item specified by the <i>index</i> lies within the range of selected items; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn contains_1a(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionRange_contains(
            self as *const crate::QItemSelectionRange,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QItemSelectionRange::contains(int row, int column, const QModelIndex& parentIndex) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#contains-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the model item specified by (<i>row</i>, <i>column</i>) and with <i>parentIndex</i> as the parent item lies within the range of selected items; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn contains_3a(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
        parent_index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionRange_contains1(
            self as *const crate::QItemSelectionRange,
            row,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent_index)
                .as_raw_ptr(),
        )
    }

    /// <p>Copy-assignment operator.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QItemSelectionRange& QItemSelectionRange::operator=(const QItemSelectionRange& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#operator-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Copy-assignment operator.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelectionRange>>,
    ) -> ::cpp_utils::MutRef<crate::QItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionRange_operator_1(
            self as *mut crate::QItemSelectionRange,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelectionRange>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of selected rows in the selection range.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QItemSelectionRange::height() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#height">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of selected rows in the selection range.</p></div>
    pub unsafe fn height(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionRange_height(
            self as *const crate::QItemSelectionRange,
        )
    }

    /// <p>Returns the list of model index items stored in the selection.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex> QItemSelectionRange::indexes() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#indexes">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the list of model index items stored in the selection.</p></div>
    pub unsafe fn indexes(&self) -> ::cpp_utils::CppBox<crate::QListOfQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionRange_indexes(
            self as *const crate::QItemSelectionRange,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a new selection range containing only the items that are found in both the selection range and the <i>other</i> selection range.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QItemSelectionRange QItemSelectionRange::intersected(const QItemSelectionRange& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#intersected">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a new selection range containing only the items that are found in both the selection range and the <i>other</i> selection range.</p>
    /// <p>This function was introduced in  Qt 4.2.</p></div>
    pub unsafe fn intersected(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelectionRange>>,
    ) -> ::cpp_utils::CppBox<crate::QItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionRange_intersected(
            self as *const crate::QItemSelectionRange,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelectionRange>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if this selection range intersects (overlaps with) the <i>other</i> range given; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QItemSelectionRange::intersects(const QItemSelectionRange& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#intersects">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this selection range intersects (overlaps with) the <i>other</i> range given; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn intersects(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelectionRange>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionRange_intersects(
            self as *const crate::QItemSelectionRange,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelectionRange>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if the selection range contains no selectable item</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QItemSelectionRange::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the selection range contains no selectable item</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionRange_isEmpty(
            self as *const crate::QItemSelectionRange,
        )
    }

    /// <p>Returns <code>true</code> if the selection range is valid; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QItemSelectionRange::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the selection range is valid; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn is_valid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionRange_isValid(
            self as *const crate::QItemSelectionRange,
        )
    }

    /// <p>Returns the column index corresponding to the leftmost selected column in the selection range.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QItemSelectionRange::left() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#left">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the column index corresponding to the leftmost selected column in the selection range.</p></div>
    pub unsafe fn left(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionRange_left(
            self as *const crate::QItemSelectionRange,
        )
    }

    /// <p>Returns the model that the items in the selection range belong to.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QAbstractItemModel* QItemSelectionRange::model() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#model">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the model that the items in the selection range belong to.</p></div>
    pub unsafe fn model(&self) -> ::cpp_utils::Ptr<crate::QAbstractItemModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionRange_model(
            self as *const crate::QItemSelectionRange,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs an empty selection range.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QItemSelectionRange::QItemSelectionRange()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#QItemSelectionRange">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty selection range.</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionRange_QItemSelectionRange();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new selection range containing only the index specified by the <i>topLeft</i> and the index <i>bottomRight</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QItemSelectionRange::QItemSelectionRange(const QModelIndex& topL, const QModelIndex& bottomR)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#QItemSelectionRange-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new selection range containing only the index specified by the <i>topLeft</i> and the index <i>bottomRight</i>.</p></div>
    pub unsafe fn new_2a(
        top_l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        bottom_r: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionRange_QItemSelectionRange3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(top_l)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(bottom_r)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new selection range containing only the model item specified by the model index <i>index</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QItemSelectionRange::QItemSelectionRange(const QModelIndex& index)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#QItemSelectionRange-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new selection range containing only the model item specified by the model index <i>index</i>.</p></div>
    pub unsafe fn new_1a(
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionRange_QItemSelectionRange4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Copy constructor. Constructs a new selection range with the same contents as the <i>other</i> range given.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QItemSelectionRange::QItemSelectionRange(const QItemSelectionRange& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#QItemSelectionRange-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Copy constructor. Constructs a new selection range with the same contents as the <i>other</i> range given.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelectionRange>>,
    ) -> ::cpp_utils::CppBox<crate::QItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionRange_QItemSelectionRange1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelectionRange>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the parent model item index of the items in the selection range.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QModelIndex QItemSelectionRange::parent() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#parent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the parent model item index of the items in the selection range.</p></div>
    pub unsafe fn parent(&self) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionRange_parent(
            self as *const crate::QItemSelectionRange,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the column index corresponding to the rightmost selected column in the selection range.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QItemSelectionRange::right() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#right">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the column index corresponding to the rightmost selected column in the selection range.</p></div>
    pub unsafe fn right(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionRange_right(
            self as *const crate::QItemSelectionRange,
        )
    }

    /// <p>Swaps this selection range's contents with <i>other</i>. This function is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QItemSelectionRange::swap(QItemSelectionRange& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps this selection range's contents with <i>other</i>. This function is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 5.6.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QItemSelectionRange>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionRange_swap(
            self as *mut crate::QItemSelectionRange,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QItemSelectionRange>>::cast_into(
                other,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the row index corresponding to the uppermost selected row in the selection range.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QItemSelectionRange::top() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#top">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the row index corresponding to the uppermost selected row in the selection range.</p></div>
    pub unsafe fn top(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionRange_top(
            self as *const crate::QItemSelectionRange,
        )
    }

    /// <p>Returns the index for the item located at the top-left corner of the selection range.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPersistentModelIndex& QItemSelectionRange::topLeft() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#topLeft">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index for the item located at the top-left corner of the selection range.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qitemselectionrange.html#top">top</a>(), <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#left">left</a>(), and <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#bottomRight">bottomRight</a>().</p></div>
    pub unsafe fn top_left(&self) -> ::cpp_utils::Ref<crate::QPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionRange_topLeft(
            self as *const crate::QItemSelectionRange,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of selected columns in the selection range.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QItemSelectionRange::width() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#width">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of selected columns in the selection range.</p></div>
    pub unsafe fn width(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionRange_width(
            self as *const crate::QItemSelectionRange,
        )
    }
}

pub mod q_item_selection_model {
    //! C++ type: <span style='color: green;'>```QItemSelectionModel```</span>

    /// <p>This enum describes the way the selection model will be updated.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QItemSelectionModel::SelectionFlag```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#SelectionFlag-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the way the selection model will be updated.</p>
    ///
    /// <p>The SelectionFlags type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;SelectionFlag&gt;. It stores an OR combination of SelectionFlag values.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct SelectionFlag(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for SelectionFlag {
        fn from(value: ::std::os::raw::c_int) -> Self {
            SelectionFlag(value)
        }
    }

    impl From<SelectionFlag> for ::std::os::raw::c_int {
        fn from(value: SelectionFlag) -> Self {
            value.0
        }
    }

    impl SelectionFlag {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl SelectionFlag {
        /// No selection will be made. (C++ enum variant: <span style='color: green;'>```NoUpdate = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoUpdate: crate::q_item_selection_model::SelectionFlag =
            crate::q_item_selection_model::SelectionFlag(0);
        /// The complete selection will be cleared. (C++ enum variant: <span style='color: green;'>```Clear = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Clear: crate::q_item_selection_model::SelectionFlag =
            crate::q_item_selection_model::SelectionFlag(1);
        /// All specified indexes will be selected. (C++ enum variant: <span style='color: green;'>```Select = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const Select: crate::q_item_selection_model::SelectionFlag =
            crate::q_item_selection_model::SelectionFlag(2);
        /// All specified indexes will be deselected. (C++ enum variant: <span style='color: green;'>```Deselect = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const Deselect: crate::q_item_selection_model::SelectionFlag =
            crate::q_item_selection_model::SelectionFlag(4);
        /// All specified indexes will be selected or deselected depending on their current state. (C++ enum variant: <span style='color: green;'>```Toggle = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const Toggle: crate::q_item_selection_model::SelectionFlag =
            crate::q_item_selection_model::SelectionFlag(8);
        /// The current selection will be updated. (C++ enum variant: <span style='color: green;'>```Current = 16```</span>)
        #[allow(non_upper_case_globals)]
        pub const Current: crate::q_item_selection_model::SelectionFlag =
            crate::q_item_selection_model::SelectionFlag(16);
        /// All indexes will be expanded to span rows. (C++ enum variant: <span style='color: green;'>```Rows = 32```</span>)
        #[allow(non_upper_case_globals)]
        pub const Rows: crate::q_item_selection_model::SelectionFlag =
            crate::q_item_selection_model::SelectionFlag(32);
        /// All indexes will be expanded to span columns. (C++ enum variant: <span style='color: green;'>```Columns = 64```</span>)
        #[allow(non_upper_case_globals)]
        pub const Columns: crate::q_item_selection_model::SelectionFlag =
            crate::q_item_selection_model::SelectionFlag(64);
        /// A combination of Select and Current, provided for convenience. (C++ enum variant: <span style='color: green;'>```SelectCurrent = 18```</span>)
        #[allow(non_upper_case_globals)]
        pub const SelectCurrent: crate::q_item_selection_model::SelectionFlag =
            crate::q_item_selection_model::SelectionFlag(18);
        /// A combination of Toggle and Current, provided for convenience. (C++ enum variant: <span style='color: green;'>```ToggleCurrent = 24```</span>)
        #[allow(non_upper_case_globals)]
        pub const ToggleCurrent: crate::q_item_selection_model::SelectionFlag =
            crate::q_item_selection_model::SelectionFlag(24);
        /// A combination of Clear and Select, provided for convenience. (C++ enum variant: <span style='color: green;'>```ClearAndSelect = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const ClearAndSelect: crate::q_item_selection_model::SelectionFlag =
            crate::q_item_selection_model::SelectionFlag(3);
    }

    impl From<crate::q_item_selection_model::SelectionFlag>
        for crate::QFlags<crate::q_item_selection_model::SelectionFlag>
    {
        fn from(value: crate::q_item_selection_model::SelectionFlag) -> Self {
            Self::from(value.to_int())
        }
    }

    impl<T: Into<crate::QFlags<crate::q_item_selection_model::SelectionFlag>>> std::ops::BitOr<T>
        for crate::q_item_selection_model::SelectionFlag
    {
        type Output = crate::QFlags<crate::q_item_selection_model::SelectionFlag>;
        fn bitor(self, rhs: T) -> crate::QFlags<crate::q_item_selection_model::SelectionFlag> {
            Into::<crate::QFlags<crate::q_item_selection_model::SelectionFlag>>::into(self) | rhs
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html">QItemSelectionModel</a> class keeps track of a view's selected items.</p>
///
/// C++ class: <span style='color: green;'>```QItemSelectionModel```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html">QItemSelectionModel</a> class keeps track of a view's selected items.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html">QItemSelectionModel</a> keeps track of the selected items in a view, or in several views onto the same model. It also keeps track of the currently selected item in a view.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html">QItemSelectionModel</a> class is one of the <a href="http://doc.qt.io/qt-5/model-view-programming.html#model-view-classes">Model/View Classes</a> and is part of Qt's <a href="http://doc.qt.io/qt-5/model-view-programming.html">model/view framework</a>.</p>
/// <p>The selected items are stored using ranges. Whenever you want to modify the selected items use <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#select">select</a>() and provide either a <a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a>, or a <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> and a <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#SelectionFlag-enum">QItemSelectionModel::SelectionFlag</a>.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html">QItemSelectionModel</a> takes a two layer approach to selection management, dealing with both selected items that have been committed and items that are part of the current selection. The current selected items are part of the current interactive selection (for example with rubber-band selection or keyboard-shift selections).</p>
/// <p>To update the currently selected items, use the bitwise OR of <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#SelectionFlag-enum">QItemSelectionModel::Current</a> and any of the other <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#SelectionFlag-enum">SelectionFlags</a>. If you omit the <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#SelectionFlag-enum">QItemSelectionModel::Current</a> command, a new current selection will be created, and the previous one added to the whole selection. All functions operate on both layers; for example, <a href="http://doc.qt.io/qt-5/qtablewidget.html#selectedItems">selecteditems()</a> will return items from both layers.</p>
/// <p><b>Note: </b>Since 5.5, <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#model">model</a>, <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#hasSelection">hasSelection</a>, and <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentIndex">currentIndex</a> are meta-object properties.</p></div>
#[repr(C)]
pub struct QItemSelectionModel {
    _unused: u8,
}
impl QItemSelectionModel {
    /// <p>Sets the model item <i>index</i> to be the current item, and emits <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentChanged">currentChanged</a>(). The current item is used for keyboard navigation and focus indication; it is independent of any selected items, although a selected item can also be the current item.</p>
    ///
    /// Returns a built-in Qt slot `QItemSelectionModel::setCurrentIndex` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#setCurrentIndex">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the model item <i>index</i> to be the current item, and emits <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentChanged">currentChanged</a>(). The current item is used for keyboard navigation and focus indication; it is independent of any selected items, although a selected item can also be the current item.</p>
    /// <p>Depending on the specified <i>command</i>, the <i>index</i> can also become part of the current selection.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentIndex">currentIndex</a>() and <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#select">select</a>().</p></div>
    pub fn slot_set_current_index(
        &self,
    ) -> crate::Receiver<(*const crate::QModelIndex, ::std::os::raw::c_int)> {
        unsafe {
            crate::Receiver::new(::cpp_utils::Ref::from_raw_ref(self), ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1setCurrentIndex(const QModelIndex&,QFlags< QItemSelectionModel::SelectionFlag >)\0"))
        }
    }

    /// <p>Selects the model item <i>index</i> using the specified <i>command</i>, and emits <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectionChanged">selectionChanged</a>().</p>
    ///
    /// Returns a built-in Qt slot `QItemSelectionModel::select` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#select">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Selects the model item <i>index</i> using the specified <i>command</i>, and emits <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectionChanged">selectionChanged</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#SelectionFlag-enum">QItemSelectionModel::SelectionFlags</a>.</p></div>
    pub fn slot_select(
        &self,
    ) -> crate::Receiver<(*const crate::QModelIndex, ::std::os::raw::c_int)> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"1select(const QModelIndex&,QFlags< QItemSelectionModel::SelectionFlag >)\0",
                ),
            )
        }
    }

    /// <p>Selects the item <i>selection</i> using the specified <i>command</i>, and emits <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectionChanged">selectionChanged</a>().</p>
    ///
    /// Returns a built-in Qt slot `QItemSelectionModel::select` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#select-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Selects the item <i>selection</i> using the specified <i>command</i>, and emits <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectionChanged">selectionChanged</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#SelectionFlag-enum">QItemSelectionModel::SelectionFlag</a>.</p></div>
    pub fn slot_select2(
        &self,
    ) -> crate::Receiver<(*const crate::QItemSelection, ::std::os::raw::c_int)> {
        unsafe {
            crate::Receiver::new(::cpp_utils::Ref::from_raw_ref(self), ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1select(const QItemSelection&,QFlags< QItemSelectionModel::SelectionFlag >)\0"))
        }
    }

    /// <p>Clears the selection model. Emits <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectionChanged">selectionChanged</a>() and <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentChanged">currentChanged</a>().</p>
    ///
    /// Returns a built-in Qt slot `QItemSelectionModel::clear` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Clears the selection model. Emits <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectionChanged">selectionChanged</a>() and <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentChanged">currentChanged</a>().</p></div>
    pub fn slot_clear(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1clear()\0"),
            )
        }
    }

    /// <p>Clears the selection model. Does not emit any signals.</p>
    ///
    /// Returns a built-in Qt slot `QItemSelectionModel::reset` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#reset">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Clears the selection model. Does not emit any signals.</p></div>
    pub fn slot_reset(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1reset()\0"),
            )
        }
    }

    /// <p>Clears the selection in the selection model. Emits <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectionChanged">selectionChanged</a>().</p>
    ///
    /// Returns a built-in Qt slot `QItemSelectionModel::clearSelection` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#clearSelection">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Clears the selection in the selection model. Emits <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectionChanged">selectionChanged</a>().</p>
    /// <p>This function was introduced in  Qt 4.2.</p></div>
    pub fn slot_clear_selection(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1clearSelection()\0"),
            )
        }
    }

    /// <p>Clears the current index. Emits <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentChanged">currentChanged</a>().</p>
    ///
    /// Returns a built-in Qt slot `QItemSelectionModel::clearCurrentIndex` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#clearCurrentIndex">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Clears the current index. Emits <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentChanged">currentChanged</a>().</p></div>
    pub fn slot_clear_current_index(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1clearCurrentIndex()\0"),
            )
        }
    }

    /// <p>This signal is emitted whenever the selection changes. The change in the selection is represented as an item selection of <i>deselected</i> items and an item selection of <i>selected</i> items.</p>
    ///
    /// Returns a built-in Qt signal `QItemSelectionModel::selectionChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectionChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted whenever the selection changes. The change in the selection is represented as an item selection of <i>deselected</i> items and an item selection of <i>selected</i> items.</p>
    /// <p>Note the that the current index changes independently from the selection. Also note that this signal will not be emitted when the item model is reset.</p>
    /// <p><b>Note:</b> Notifier signal for property <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectedIndexes-prop">selectedIndexes</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#select">select</a>() and <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentChanged">currentChanged</a>().</p></div>
    pub fn selection_changed(
        &self,
    ) -> crate::Signal<(*const crate::QItemSelection, *const crate::QItemSelection)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2selectionChanged(const QItemSelection&,const QItemSelection&)\0",
                ),
            )
        }
    }

    /// <p>This signal is emitted whenever the current item changes. The <i>previous</i> model item index is replaced by the <i>current</i> index as the selection's current item.</p>
    ///
    /// Returns a built-in Qt signal `QItemSelectionModel::currentChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted whenever the current item changes. The <i>previous</i> model item index is replaced by the <i>current</i> index as the selection's current item.</p>
    /// <p>Note that this signal will not be emitted when the item model is reset.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentIndex">currentIndex</a>(), <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#setCurrentIndex">setCurrentIndex</a>(), and <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectionChanged">selectionChanged</a>().</p></div>
    pub fn current_changed(
        &self,
    ) -> crate::Signal<(*const crate::QModelIndex, *const crate::QModelIndex)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2currentChanged(const QModelIndex&,const QModelIndex&)\0",
                ),
            )
        }
    }

    /// <p>This signal is emitted if the <i>current</i> item changes and its row is different to the row of the <i>previous</i> current item.</p>
    ///
    /// Returns a built-in Qt signal `QItemSelectionModel::currentRowChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentRowChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted if the <i>current</i> item changes and its row is different to the row of the <i>previous</i> current item.</p>
    /// <p>Note that this signal will not be emitted when the item model is reset.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentChanged">currentChanged</a>(), <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentColumnChanged">currentColumnChanged</a>(), <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentIndex">currentIndex</a>(), and <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#setCurrentIndex">setCurrentIndex</a>().</p></div>
    pub fn current_row_changed(
        &self,
    ) -> crate::Signal<(*const crate::QModelIndex, *const crate::QModelIndex)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2currentRowChanged(const QModelIndex&,const QModelIndex&)\0",
                ),
            )
        }
    }

    /// <p>This signal is emitted if the <i>current</i> item changes and its column is different to the column of the <i>previous</i> current item.</p>
    ///
    /// Returns a built-in Qt signal `QItemSelectionModel::currentColumnChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentColumnChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted if the <i>current</i> item changes and its column is different to the column of the <i>previous</i> current item.</p>
    /// <p>Note that this signal will not be emitted when the item model is reset.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentChanged">currentChanged</a>(), <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentRowChanged">currentRowChanged</a>(), <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentIndex">currentIndex</a>(), and <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#setCurrentIndex">setCurrentIndex</a>().</p></div>
    pub fn current_column_changed(
        &self,
    ) -> crate::Signal<(*const crate::QModelIndex, *const crate::QModelIndex)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2currentColumnChanged(const QModelIndex&,const QModelIndex&)\0",
                ),
            )
        }
    }

    /// <p>This signal is emitted when the <i>model</i> is successfully set with <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#setModel">setModel</a>().</p>
    ///
    /// Returns a built-in Qt signal `QItemSelectionModel::modelChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#modelChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the <i>model</i> is successfully set with <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#setModel">setModel</a>().</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#model">model</a>() and <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#setModel">setModel</a>().</p></div>
    pub fn model_changed(&self) -> crate::Signal<(*mut crate::QAbstractItemModel,)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2modelChanged(QAbstractItemModel*)\0",
                ),
            )
        }
    }

    /// <p>Clears the selection model. Emits <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectionChanged">selectionChanged</a>() and <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentChanged">currentChanged</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [slot] void QItemSelectionModel::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Clears the selection model. Emits <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectionChanged">selectionChanged</a>() and <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentChanged">currentChanged</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_clear(
            self as *mut crate::QItemSelectionModel,
        )
    }

    /// <p>Clears the current index. Emits <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentChanged">currentChanged</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [slot] void QItemSelectionModel::clearCurrentIndex()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#clearCurrentIndex">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Clears the current index. Emits <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentChanged">currentChanged</a>().</p></div>
    pub unsafe fn clear_current_index(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_clearCurrentIndex(
            self as *mut crate::QItemSelectionModel,
        )
    }

    /// <p>Clears the selection in the selection model. Emits <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectionChanged">selectionChanged</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QItemSelectionModel::clearSelection()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#clearSelection">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Clears the selection in the selection model. Emits <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectionChanged">selectionChanged</a>().</p>
    /// <p>This function was introduced in  Qt 4.2.</p></div>
    pub unsafe fn clear_selection(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_clearSelection(
            self as *mut crate::QItemSelectionModel,
        )
    }

    /// <p>Returns <code>true</code> if there are any items selected in the <i>column</i> with the given <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QItemSelectionModel::columnIntersectsSelection(int column, const QModelIndex& parent) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#columnIntersectsSelection">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if there are any items selected in the <i>column</i> with the given <i>parent</i>.</p></div>
    pub unsafe fn column_intersects_selection(
        &self,
        column: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_columnIntersectsSelection(
            self as *const crate::QItemSelectionModel,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the model item index for the current item, or an invalid index if there is no current item.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QModelIndex QItemSelectionModel::currentIndex() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentIndex">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the model item index for the current item, or an invalid index if there is no current item.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#setCurrentIndex">setCurrentIndex</a>().</p></div>
    pub unsafe fn current_index(&self) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_currentIndex(
            self as *const crate::QItemSelectionModel,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if the selection model contains any selection ranges; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QItemSelectionModel::hasSelection() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#hasSelection">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the selection model contains any selection ranges; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.2.</p></div>
    pub unsafe fn has_selection(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_hasSelection(
            self as *const crate::QItemSelectionModel,
        )
    }

    /// <p>Returns <code>true</code> if all items are selected in the <i>column</i> with the given <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QItemSelectionModel::isColumnSelected(int column, const QModelIndex& parent) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#isColumnSelected">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if all items are selected in the <i>column</i> with the given <i>parent</i>.</p>
    /// <p>Note that this function is usually faster than calling <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#isSelected">isSelected</a>() on all items in the same column and that unselectable items are ignored.</p></div>
    pub unsafe fn is_column_selected(
        &self,
        column: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_isColumnSelected(
            self as *const crate::QItemSelectionModel,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if all items are selected in the <i>row</i> with the given <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QItemSelectionModel::isRowSelected(int row, const QModelIndex& parent) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#isRowSelected">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if all items are selected in the <i>row</i> with the given <i>parent</i>.</p>
    /// <p>Note that this function is usually faster than calling <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#isSelected">isSelected</a>() on all items in the same row and that unselectable items are ignored.</p></div>
    pub unsafe fn is_row_selected(
        &self,
        row: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_isRowSelected(
            self as *const crate::QItemSelectionModel,
            row,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if the given model item <i>index</i> is selected.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QItemSelectionModel::isSelected(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#isSelected">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the given model item <i>index</i> is selected.</p></div>
    pub unsafe fn is_selected(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_isSelected(
            self as *const crate::QItemSelectionModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QItemSelectionModel::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_metaObject(
            self as *const crate::QItemSelectionModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns the item model operated on by the selection model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QAbstractItemModel* QItemSelectionModel::model() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#model">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item model operated on by the selection model.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#setModel">setModel</a>().</p></div>
    pub unsafe fn model(&self) -> ::cpp_utils::Ptr<crate::QAbstractItemModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_model(
            self as *const crate::QItemSelectionModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns the item model operated on by the selection model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* QItemSelectionModel::model()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#model-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item model operated on by the selection model.</p>
    /// <p>This function was introduced in  Qt 5.5.</p></div>
    pub unsafe fn model_mut(&mut self) -> ::cpp_utils::MutPtr<crate::QAbstractItemModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_model1(
            self as *mut crate::QItemSelectionModel,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Constructs a selection model that operates on the specified item <i>model</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QItemSelectionModel::QItemSelectionModel(QAbstractItemModel* model = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#QItemSelectionModel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a selection model that operates on the specified item <i>model</i>.</p></div>
    pub unsafe fn new_1a(
        model: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractItemModel>>,
    ) -> ::cpp_utils::CppBox<crate::QItemSelectionModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_QItemSelectionModel(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractItemModel>>::cast_into(
                model,
            )
            .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a selection model that operates on the specified item <i>model</i> with <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QItemSelectionModel::QItemSelectionModel(QAbstractItemModel* model, QObject* parent)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#QItemSelectionModel-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a selection model that operates on the specified item <i>model</i> with <i>parent</i>.</p></div>
    pub unsafe fn new_2a(
        model: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractItemModel>>,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QItemSelectionModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_QItemSelectionModel1(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractItemModel>>::cast_into(
                model,
            )
            .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html">QItemSelectionModel</a> class keeps track of a view's selected items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QItemSelectionModel::QItemSelectionModel()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html">QItemSelectionModel</a> class keeps track of a view's selected items.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html">QItemSelectionModel</a> keeps track of the selected items in a view, or in several views onto the same model. It also keeps track of the currently selected item in a view.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html">QItemSelectionModel</a> class is one of the <a href="http://doc.qt.io/qt-5/model-view-programming.html#model-view-classes">Model/View Classes</a> and is part of Qt's <a href="http://doc.qt.io/qt-5/model-view-programming.html">model/view framework</a>.</p>
    /// <p>The selected items are stored using ranges. Whenever you want to modify the selected items use <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#select">select</a>() and provide either a <a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a>, or a <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> and a <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#SelectionFlag-enum">QItemSelectionModel::SelectionFlag</a>.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html">QItemSelectionModel</a> takes a two layer approach to selection management, dealing with both selected items that have been committed and items that are part of the current selection. The current selected items are part of the current interactive selection (for example with rubber-band selection or keyboard-shift selections).</p>
    /// <p>To update the currently selected items, use the bitwise OR of <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#SelectionFlag-enum">QItemSelectionModel::Current</a> and any of the other <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#SelectionFlag-enum">SelectionFlags</a>. If you omit the <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#SelectionFlag-enum">QItemSelectionModel::Current</a> command, a new current selection will be created, and the previous one added to the whole selection. All functions operate on both layers; for example, <a href="http://doc.qt.io/qt-5/qtablewidget.html#selectedItems">selecteditems()</a> will return items from both layers.</p>
    /// <p><b>Note: </b>Since 5.5, <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#model">model</a>, <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#hasSelection">hasSelection</a>, and <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentIndex">currentIndex</a> are meta-object properties.</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QItemSelectionModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_QItemSelectionModel2();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QItemSelectionModel::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_qt_metacall(
            self as *mut crate::QItemSelectionModel,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QItemSelectionModel::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_qt_metacast(
            self as *mut crate::QItemSelectionModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Clears the selection model. Does not emit any signals.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [slot] void QItemSelectionModel::reset()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#reset">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Clears the selection model. Does not emit any signals.</p></div>
    pub unsafe fn reset(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_reset(
            self as *mut crate::QItemSelectionModel,
        )
    }

    /// <p>Returns <code>true</code> if there are any items selected in the <i>row</i> with the given <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QItemSelectionModel::rowIntersectsSelection(int row, const QModelIndex& parent) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#rowIntersectsSelection">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if there are any items selected in the <i>row</i> with the given <i>parent</i>.</p></div>
    pub unsafe fn row_intersects_selection(
        &self,
        row: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_rowIntersectsSelection(
            self as *const crate::QItemSelectionModel,
            row,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Selects the model item <i>index</i> using the specified <i>command</i>, and emits <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectionChanged">selectionChanged</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [slot] void QItemSelectionModel::select(const QModelIndex& index, QFlags<QItemSelectionModel::SelectionFlag> command)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#select">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Selects the model item <i>index</i> using the specified <i>command</i>, and emits <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectionChanged">selectionChanged</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#SelectionFlag-enum">QItemSelectionModel::SelectionFlags</a>.</p></div>
    pub unsafe fn select_q_model_index_q_flags_selection_flag(
        &mut self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        command: crate::QFlags<crate::q_item_selection_model::SelectionFlag>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_select(
            self as *mut crate::QItemSelectionModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
            command.to_int(),
        )
    }

    /// <p>Selects the item <i>selection</i> using the specified <i>command</i>, and emits <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectionChanged">selectionChanged</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [slot] void QItemSelectionModel::select(const QItemSelection& selection, QFlags<QItemSelectionModel::SelectionFlag> command)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#select-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Selects the item <i>selection</i> using the specified <i>command</i>, and emits <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectionChanged">selectionChanged</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#SelectionFlag-enum">QItemSelectionModel::SelectionFlag</a>.</p></div>
    pub unsafe fn select_q_item_selection_q_flags_selection_flag(
        &mut self,
        selection: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelection>>,
        command: crate::QFlags<crate::q_item_selection_model::SelectionFlag>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_select1(
            self as *mut crate::QItemSelectionModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelection>>::cast_into(selection)
                .as_raw_ptr(),
            command.to_int(),
        )
    }

    /// <p>Returns the indexes in the given <i>row</i> for columns where all rows are selected.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex> QItemSelectionModel::selectedColumns(int row = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectedColumns">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the indexes in the given <i>row</i> for columns where all rows are selected.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectedIndexes">selectedIndexes</a>() and <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectedRows">selectedRows</a>().</p></div>
    pub unsafe fn selected_columns_1a(
        &self,
        row: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_selectedColumns(
            self as *const crate::QItemSelectionModel,
            row,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the indexes in the given <i>row</i> for columns where all rows are selected.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex> QItemSelectionModel::selectedColumns() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectedColumns">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the indexes in the given <i>row</i> for columns where all rows are selected.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectedIndexes">selectedIndexes</a>() and <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectedRows">selectedRows</a>().</p></div>
    pub unsafe fn selected_columns_0a(&self) -> ::cpp_utils::CppBox<crate::QListOfQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_selectedColumns1(
            self as *const crate::QItemSelectionModel,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This property was introduced in  Qt 5.5.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex> QItemSelectionModel::selectedIndexes() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectedIndexes-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property was introduced in  Qt 5.5.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QModelIndexList </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectedIndexes">selectedIndexes</a></b></span>() const</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectionChanged">selectionChanged</a></b></span>(const QItemSelection &amp;<i>selected</i>, const QItemSelection &amp;<i>deselected</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn selected_indexes(&self) -> ::cpp_utils::CppBox<crate::QListOfQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_selectedIndexes(
            self as *const crate::QItemSelectionModel,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the indexes in the given <i>column</i> for the rows where all columns are selected.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex> QItemSelectionModel::selectedRows(int column = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectedRows">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the indexes in the given <i>column</i> for the rows where all columns are selected.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectedIndexes">selectedIndexes</a>() and <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectedColumns">selectedColumns</a>().</p></div>
    pub unsafe fn selected_rows_1a(
        &self,
        column: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_selectedRows(
            self as *const crate::QItemSelectionModel,
            column,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the indexes in the given <i>column</i> for the rows where all columns are selected.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex> QItemSelectionModel::selectedRows() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectedRows">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the indexes in the given <i>column</i> for the rows where all columns are selected.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectedIndexes">selectedIndexes</a>() and <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectedColumns">selectedColumns</a>().</p></div>
    pub unsafe fn selected_rows_0a(&self) -> ::cpp_utils::CppBox<crate::QListOfQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_selectedRows1(
            self as *const crate::QItemSelectionModel,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the selection ranges stored in the selection model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QItemSelection QItemSelectionModel::selection() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selection">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the selection ranges stored in the selection model.</p></div>
    pub unsafe fn selection(&self) -> ::cpp_utils::CppBox<crate::QItemSelection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_selection(
            self as *const crate::QItemSelectionModel,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Sets the model item <i>index</i> to be the current item, and emits <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentChanged">currentChanged</a>(). The current item is used for keyboard navigation and focus indication; it is independent of any selected items, although a selected item can also be the current item.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [slot] void QItemSelectionModel::setCurrentIndex(const QModelIndex& index, QFlags<QItemSelectionModel::SelectionFlag> command)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#setCurrentIndex">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the model item <i>index</i> to be the current item, and emits <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentChanged">currentChanged</a>(). The current item is used for keyboard navigation and focus indication; it is independent of any selected items, although a selected item can also be the current item.</p>
    /// <p>Depending on the specified <i>command</i>, the <i>index</i> can also become part of the current selection.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#currentIndex">currentIndex</a>() and <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#select">select</a>().</p></div>
    pub unsafe fn set_current_index(
        &mut self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        command: crate::QFlags<crate::q_item_selection_model::SelectionFlag>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_setCurrentIndex(
            self as *mut crate::QItemSelectionModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
            command.to_int(),
        )
    }

    /// <p>Sets the model to <i>model</i>. The <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#modelChanged">modelChanged</a>() signal will be emitted.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QItemSelectionModel::setModel(QAbstractItemModel* model)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#setModel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the model to <i>model</i>. The <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#modelChanged">modelChanged</a>() signal will be emitted.</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#model">model</a>() and <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#modelChanged">modelChanged</a>().</p></div>
    pub unsafe fn set_model(
        &mut self,
        model: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractItemModel>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_setModel(
            self as *mut crate::QItemSelectionModel,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractItemModel>>::cast_into(
                model,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QItemSelectionModel::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QItemSelectionModel::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a> class manages information about selected items in a model.</p>
///
/// C++ class: <span style='color: green;'>```QItemSelection```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qitemselection.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a> class manages information about selected items in a model.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a> describes the items in a model that have been selected by the user. A <a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a> is basically a list of selection ranges, see <a href="http://doc.qt.io/qt-5/qitemselectionrange.html">QItemSelectionRange</a>. It provides functions for creating and manipulating selections, and selecting a range of items from a model.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a> class is one of the <a href="http://doc.qt.io/qt-5/model-view-programming.html#model-view-classes">Model/View Classes</a> and is part of Qt's <a href="http://doc.qt.io/qt-5/model-view-programming.html">model/view framework</a>.</p>
/// <p>An item selection can be constructed and initialized to contain a range of items from an existing model. The following example constructs a selection that contains a range of items from the given <code>model</code>, beginning at the <code>topLeft</code>, and ending at the <code>bottomRight</code>.</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qitemselection.html#QItemSelection">QItemSelection</a></span> <span class="operator">*</span>selection <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qitemselection.html#QItemSelection">QItemSelection</a></span>(topLeft<span class="operator">,</span> bottomRight);
///
/// </pre>
/// <p>An empty item selection can be constructed, and later populated as required. So, if the model is going to be unavailable when we construct the item selection, we can rewrite the above code in the following way:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qitemselection.html#QItemSelection">QItemSelection</a></span> <span class="operator">*</span>selection <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qitemselection.html#QItemSelection">QItemSelection</a></span>();
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   selection<span class="operator">-</span><span class="operator">&gt;</span>select(topLeft<span class="operator">,</span> bottomRight);
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a> saves memory, and avoids unnecessary work, by working with selection ranges rather than recording the model item index for each item in the selection. Generally, an instance of this class will contain a list of non-overlapping selection ranges.</p>
/// <p>Use <a href="http://doc.qt.io/qt-5/qitemselection.html#merge">merge</a>() to merge one item selection into another without making overlapping ranges. Use <a href="http://doc.qt.io/qt-5/qitemselection.html#split">split</a>() to split one selection range into smaller ranges based on a another selection range.</p></div>
#[repr(C)]
pub struct QItemSelection {
    _unused: u8,
}
impl QItemSelection {
    /// <p>Returns <code>true</code> if the selection contains the given <i>index</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QItemSelection::contains(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselection.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the selection contains the given <i>index</i>; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn contains(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QItemSelection_contains(
            self as *const crate::QItemSelection,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        )
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a> class manages information about selected items in a model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QItemSelection& QItemSelection::operator=(const QItemSelection& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselection.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a> class manages information about selected items in a model.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a> describes the items in a model that have been selected by the user. A <a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a> is basically a list of selection ranges, see <a href="http://doc.qt.io/qt-5/qitemselectionrange.html">QItemSelectionRange</a>. It provides functions for creating and manipulating selections, and selecting a range of items from a model.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a> class is one of the <a href="http://doc.qt.io/qt-5/model-view-programming.html#model-view-classes">Model/View Classes</a> and is part of Qt's <a href="http://doc.qt.io/qt-5/model-view-programming.html">model/view framework</a>.</p>
    /// <p>An item selection can be constructed and initialized to contain a range of items from an existing model. The following example constructs a selection that contains a range of items from the given <code>model</code>, beginning at the <code>topLeft</code>, and ending at the <code>bottomRight</code>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qitemselection.html#QItemSelection">QItemSelection</a></span> <span class="operator">*</span>selection <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qitemselection.html#QItemSelection">QItemSelection</a></span>(topLeft<span class="operator">,</span> bottomRight);
    ///
    /// </pre>
    /// <p>An empty item selection can be constructed, and later populated as required. So, if the model is going to be unavailable when we construct the item selection, we can rewrite the above code in the following way:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qitemselection.html#QItemSelection">QItemSelection</a></span> <span class="operator">*</span>selection <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qitemselection.html#QItemSelection">QItemSelection</a></span>();
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   selection<span class="operator">-</span><span class="operator">&gt;</span>select(topLeft<span class="operator">,</span> bottomRight);
    ///
    /// </pre>
    /// <p><a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a> saves memory, and avoids unnecessary work, by working with selection ranges rather than recording the model item index for each item in the selection. Generally, an instance of this class will contain a list of non-overlapping selection ranges.</p>
    /// <p>Use <a href="http://doc.qt.io/qt-5/qitemselection.html#merge">merge</a>() to merge one item selection into another without making overlapping ranges. Use <a href="http://doc.qt.io/qt-5/qitemselection.html#split">split</a>() to split one selection range into smaller ranges based on a another selection range.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelection>>,
    ) -> ::cpp_utils::MutRef<crate::QItemSelection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelection_operator_(
            self as *mut crate::QItemSelection,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelection>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a list of model indexes that correspond to the selected items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex> QItemSelection::indexes() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselection.html#indexes">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of model indexes that correspond to the selected items.</p></div>
    pub unsafe fn indexes(&self) -> ::cpp_utils::CppBox<crate::QListOfQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelection_indexes(
            self as *const crate::QItemSelection,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Merges the <i>other</i> selection with this <a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a> using the <i>command</i> given. This method guarantees that no ranges are overlapping.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QItemSelection::merge(const QItemSelection& other, QFlags<QItemSelectionModel::SelectionFlag> command)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselection.html#merge">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Merges the <i>other</i> selection with this <a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a> using the <i>command</i> given. This method guarantees that no ranges are overlapping.</p>
    /// <p>Note that only <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#SelectionFlag-enum">QItemSelectionModel::Select</a>, <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#SelectionFlag-enum">QItemSelectionModel::Deselect</a>, and <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#SelectionFlag-enum">QItemSelectionModel::Toggle</a> are supported.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qitemselection.html#split">split</a>().</p></div>
    pub unsafe fn merge(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelection>>,
        command: crate::QFlags<crate::q_item_selection_model::SelectionFlag>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QItemSelection_merge(
            self as *mut crate::QItemSelection,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelection>>::cast_into(other)
                .as_raw_ptr(),
            command.to_int(),
        )
    }

    /// <p>Constructs an empty selection.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QItemSelection::QItemSelection()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselection.html#QItemSelection">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty selection.</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QItemSelection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelection_QItemSelection();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs an item selection that extends from the top-left model item, specified by the <i>topLeft</i> index, to the bottom-right item, specified by <i>bottomRight</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QItemSelection::QItemSelection(const QModelIndex& topLeft, const QModelIndex& bottomRight)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselection.html#QItemSelection-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an item selection that extends from the top-left model item, specified by the <i>topLeft</i> index, to the bottom-right item, specified by <i>bottomRight</i>.</p></div>
    pub unsafe fn new_2a(
        top_left: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        bottom_right: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QItemSelection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelection_QItemSelection1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(top_left)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(bottom_right)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a> class manages information about selected items in a model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QItemSelection::QItemSelection(const QItemSelection& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselection.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a> class manages information about selected items in a model.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a> describes the items in a model that have been selected by the user. A <a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a> is basically a list of selection ranges, see <a href="http://doc.qt.io/qt-5/qitemselectionrange.html">QItemSelectionRange</a>. It provides functions for creating and manipulating selections, and selecting a range of items from a model.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a> class is one of the <a href="http://doc.qt.io/qt-5/model-view-programming.html#model-view-classes">Model/View Classes</a> and is part of Qt's <a href="http://doc.qt.io/qt-5/model-view-programming.html">model/view framework</a>.</p>
    /// <p>An item selection can be constructed and initialized to contain a range of items from an existing model. The following example constructs a selection that contains a range of items from the given <code>model</code>, beginning at the <code>topLeft</code>, and ending at the <code>bottomRight</code>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qitemselection.html#QItemSelection">QItemSelection</a></span> <span class="operator">*</span>selection <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qitemselection.html#QItemSelection">QItemSelection</a></span>(topLeft<span class="operator">,</span> bottomRight);
    ///
    /// </pre>
    /// <p>An empty item selection can be constructed, and later populated as required. So, if the model is going to be unavailable when we construct the item selection, we can rewrite the above code in the following way:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qitemselection.html#QItemSelection">QItemSelection</a></span> <span class="operator">*</span>selection <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qitemselection.html#QItemSelection">QItemSelection</a></span>();
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   selection<span class="operator">-</span><span class="operator">&gt;</span>select(topLeft<span class="operator">,</span> bottomRight);
    ///
    /// </pre>
    /// <p><a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a> saves memory, and avoids unnecessary work, by working with selection ranges rather than recording the model item index for each item in the selection. Generally, an instance of this class will contain a list of non-overlapping selection ranges.</p>
    /// <p>Use <a href="http://doc.qt.io/qt-5/qitemselection.html#merge">merge</a>() to merge one item selection into another without making overlapping ranges. Use <a href="http://doc.qt.io/qt-5/qitemselection.html#split">split</a>() to split one selection range into smaller ranges based on a another selection range.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelection>>,
    ) -> ::cpp_utils::CppBox<crate::QItemSelection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QItemSelection_QItemSelection2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelection>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Adds the items in the range that extends from the top-left model item, specified by the <i>topLeft</i> index, to the bottom-right item, specified by <i>bottomRight</i> to the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QItemSelection::select(const QModelIndex& topLeft, const QModelIndex& bottomRight)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselection.html#select">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds the items in the range that extends from the top-left model item, specified by the <i>topLeft</i> index, to the bottom-right item, specified by <i>bottomRight</i> to the list.</p>
    /// <p><b>Note: </b><i>topLeft</i> and <i>bottomRight</i> must have the same parent.</p></div>
    pub unsafe fn select(
        &mut self,
        top_left: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        bottom_right: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QItemSelection_select(
            self as *mut crate::QItemSelection,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(top_left)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(bottom_right)
                .as_raw_ptr(),
        )
    }

    /// <p>Splits the selection <i>range</i> using the selection <i>other</i> range. Removes all items in <i>other</i> from <i>range</i> and puts the result in <i>result</i>. This can be compared with the semantics of the <i>subtract</i> operation of a set.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QItemSelection::split(const QItemSelectionRange& range, const QItemSelectionRange& other, QItemSelection* result)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselection.html#split">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Splits the selection <i>range</i> using the selection <i>other</i> range. Removes all items in <i>other</i> from <i>range</i> and puts the result in <i>result</i>. This can be compared with the semantics of the <i>subtract</i> operation of a set.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qitemselection.html#merge">merge</a>().</p></div>
    pub unsafe fn split(
        range: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelectionRange>>,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelectionRange>>,
        result: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QItemSelection>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QItemSelection_split(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelectionRange>>::cast_into(range)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelectionRange>>::cast_into(other)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QItemSelection>>::cast_into(result)
                .as_mut_raw_ptr(),
        )
    }
}

pub mod q_json_value {
    //! C++ type: <span style='color: green;'>```QJsonValue```</span>

    /// <p>This enum describes the type of the JSON value.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QJsonValue::Type```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the type of the JSON value.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Type(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Type {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Type(value)
        }
    }

    impl From<Type> for ::std::os::raw::c_int {
        fn from(value: Type) -> Self {
            value.0
        }
    }

    impl Type {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Type {
        /// A Null value (C++ enum variant: <span style='color: green;'>```Null = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const Null: crate::q_json_value::Type = crate::q_json_value::Type(0);
        /// A boolean value. Use <a href="http://doc.qt.io/qt-5/qjsonvalue.html#toBool">toBool</a>() to convert to a bool. (C++ enum variant: <span style='color: green;'>```Bool = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Bool: crate::q_json_value::Type = crate::q_json_value::Type(1);
        /// A double. Use <a href="http://doc.qt.io/qt-5/qjsonvalue.html#toDouble">toDouble</a>() to convert to a double. (C++ enum variant: <span style='color: green;'>```Double = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const Double: crate::q_json_value::Type = crate::q_json_value::Type(2);
        /// A string. Use <a href="http://doc.qt.io/qt-5/qjsonvalue.html#toString">toString</a>() to convert to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. (C++ enum variant: <span style='color: green;'>```String = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const String: crate::q_json_value::Type = crate::q_json_value::Type(3);
        /// An array. Use <a href="http://doc.qt.io/qt-5/qjsonvalue.html#toArray-1">toArray</a>() to convert to a <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>. (C++ enum variant: <span style='color: green;'>```Array = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const Array: crate::q_json_value::Type = crate::q_json_value::Type(4);
        /// An object. Use <a href="http://doc.qt.io/qt-5/qjsonvalue.html#toObject-1">toObject</a>() to convert to a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. (C++ enum variant: <span style='color: green;'>```Object = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const Object: crate::q_json_value::Type = crate::q_json_value::Type(5);
        /// The value is undefined. This is usually returned as an error condition, when trying to read an out of bounds value in an array or a non existent key in an object. (C++ enum variant: <span style='color: green;'>```Undefined = 128```</span>)
        #[allow(non_upper_case_globals)]
        pub const Undefined: crate::q_json_value::Type = crate::q_json_value::Type(128);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> class encapsulates a value in JSON.</p>
///
/// C++ class: <span style='color: green;'>```QJsonValue```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qjsonvalue.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> class encapsulates a value in JSON.</p>
/// <p>A value in JSON can be one of 6 basic types:</p>
/// <p>JSON is a format to store structured data. It has 6 basic data types:</p>
/// <ul>
/// <li>bool <a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">QJsonValue::Bool</a></li>
/// <li>double <a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">QJsonValue::Double</a></li>
/// <li>string <a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">QJsonValue::String</a></li>
/// <li>array <a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">QJsonValue::Array</a></li>
/// <li>object <a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">QJsonValue::Object</a></li>
/// <li>null <a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">QJsonValue::Null</a></li>
/// </ul>
/// <p>A value can represent any of the above data types. In addition, <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> has one special flag to represent undefined values. This can be queried with <a href="http://doc.qt.io/qt-5/qjsonvalue.html#isUndefined">isUndefined</a>().</p>
/// <p>The type of the value can be queried with <a href="http://doc.qt.io/qt-5/qjsonvalue.html#type">type</a>() or accessors like <a href="http://doc.qt.io/qt-5/qjsonvalue.html#isBool">isBool</a>(), <a href="http://doc.qt.io/qt-5/qjsonvalue.html#isString">isString</a>(), and so on. Likewise, the value can be converted to the type stored in it using the <a href="http://doc.qt.io/qt-5/qjsonvalue.html#toBool">toBool</a>(), <a href="http://doc.qt.io/qt-5/qjsonvalue.html#toString">toString</a>() and so on.</p>
/// <p>Values are strictly typed internally and contrary to <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> will not attempt to do any implicit type conversions. This implies that converting to a type that is not stored in the value will return a default constructed return value.</p>
/// <a name="qjsonvalueref"></a></div>
#[repr(C)]
pub struct QJsonValue {
    _unused: u8,
}
impl QJsonValue {
    /// <p>Assigns the value stored in <i>other</i> to this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonValue& QJsonValue::operator=(const QJsonValue& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns the value stored in <i>other</i> to this object.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonValue>>,
    ) -> ::cpp_utils::MutRef<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValue_operator_(
            self as *mut crate::QJsonValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonValue>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Converts <i>variant</i> to a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QJsonValue QJsonValue::fromVariant(const QVariant& variant)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#fromVariant">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts <i>variant</i> to a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> and returns it.</p>
    /// <p>The conversion will convert <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> types as follows:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Source type</th><th>Destination type</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td><ul>
    /// <li>QMetaType::Nullptr</li>
    /// </ul>
    /// </td><td><a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">QJsonValue::Null</a></td></tr>
    /// <tr valign="top" class="even"><td><ul>
    /// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Bool</a></li>
    /// </ul>
    /// </td><td><a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">QJsonValue::Bool</a></td></tr>
    /// <tr valign="top" class="odd"><td><ul>
    /// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Int</a></li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UInt</a></li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::LongLong</a></li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::ULongLong</a></li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Float</a></li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::Double</a></li>
    /// </ul>
    /// </td><td><a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">QJsonValue::Double</a></td></tr>
    /// <tr valign="top" class="even"><td><ul>
    /// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QString</a></li>
    /// </ul>
    /// </td><td><a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">QJsonValue::String</a></td></tr>
    /// <tr valign="top" class="odd"><td><ul>
    /// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QStringList</a></li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QVariantList</a></li>
    /// </ul>
    /// </td><td><a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">QJsonValue::Array</a></td></tr>
    /// <tr valign="top" class="even"><td><ul>
    /// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QVariantMap</a></li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::QVariantHash</a></li>
    /// </ul>
    /// </td><td><a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">QJsonValue::Object</a></td></tr>
    /// </tbody></table></div>
    /// <p>For all other <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> types a conversion to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> will be attempted. If the returned string is empty, a Null <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> will be stored, otherwise a String value using the returned <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonvalue.html#toVariant">toVariant</a>().</p></div>
    pub unsafe fn from_variant(
        variant: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValue_fromVariant(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(variant)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> representing the value for the key <i>key</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonValue QJsonValue::operator[](const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> representing the value for the key <i>key</i>.</p>
    /// <p>Equivalent to calling <a href="http://doc.qt.io/qt-5/qjsonvalue.html#toObject-1">toObject</a>().value(key).</p>
    /// <p>The returned <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> is <a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">QJsonValue::Undefined</a> if the key does not exist, or if <a href="http://doc.qt.io/qt-5/qjsonvalue.html#isObject">isObject</a>() is false.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>, <a href="http://doc.qt.io/qt-5/qjsonvalue.html#isUndefined">QJsonValue::isUndefined</a>(), and <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn index_q_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValue_operator__4(
            self as *const crate::QJsonValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonValue QJsonValue::operator[](QLatin1String key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn index_q_latin1_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValue_operator__5(
            self as *const crate::QJsonValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> representing the value for index <i>i</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonValue QJsonValue::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#operator-5b-5d-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> representing the value for index <i>i</i>.</p>
    /// <p>Equivalent to calling <a href="http://doc.qt.io/qt-5/qjsonvalue.html#toArray-1">toArray</a>().at(i).</p>
    /// <p>The returned <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> is <a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">QJsonValue::Undefined</a>, if <i>i</i> is out of bounds, or if <a href="http://doc.qt.io/qt-5/qjsonvalue.html#isArray">isArray</a>() is false.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>, <a href="http://doc.qt.io/qt-5/qjsonvalue.html#isUndefined">QJsonValue::isUndefined</a>(), and <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn index_int(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValue_operator__6(
            self as *const crate::QJsonValue,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if the value contains an array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QJsonValue::isArray() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#isArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the value contains an array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonvalue.html#toArray-1">toArray</a>().</p></div>
    pub unsafe fn is_array(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonValue_isArray(self as *const crate::QJsonValue)
    }

    /// <p>Returns <code>true</code> if the value contains a boolean.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QJsonValue::isBool() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#isBool">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the value contains a boolean.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonvalue.html#toBool">toBool</a>().</p></div>
    pub unsafe fn is_bool(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonValue_isBool(self as *const crate::QJsonValue)
    }

    /// <p>Returns <code>true</code> if the value contains a double.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QJsonValue::isDouble() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#isDouble">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the value contains a double.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonvalue.html#toDouble">toDouble</a>().</p></div>
    pub unsafe fn is_double(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonValue_isDouble(self as *const crate::QJsonValue)
    }

    /// <p>Returns <code>true</code> if the value is null.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QJsonValue::isNull() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#isNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the value is null.</p></div>
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonValue_isNull(self as *const crate::QJsonValue)
    }

    /// <p>Returns <code>true</code> if the value contains an object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QJsonValue::isObject() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#isObject">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the value contains an object.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonvalue.html#toObject-1">toObject</a>().</p></div>
    pub unsafe fn is_object(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonValue_isObject(self as *const crate::QJsonValue)
    }

    /// <p>Returns <code>true</code> if the value contains a string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QJsonValue::isString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#isString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the value contains a string.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonvalue.html#toString">toString</a>().</p></div>
    pub unsafe fn is_string(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonValue_isString(self as *const crate::QJsonValue)
    }

    /// <p>Returns <code>true</code> if the value is undefined. This can happen in certain error cases as e.g. accessing a non existing key in a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QJsonValue::isUndefined() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#isUndefined">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the value is undefined. This can happen in certain error cases as e.g. accessing a non existing key in a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p></div>
    pub unsafe fn is_undefined(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonValue_isUndefined(self as *const crate::QJsonValue)
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> of type <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonValue::QJsonValue(QJsonValue::Type arg1 = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#QJsonValue">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> of type <i>type</i>.</p>
    /// <p>The default is to create a Null value.</p></div>
    pub unsafe fn from_type(
        arg1: crate::q_json_value::Type,
    ) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValue_QJsonValue(arg1);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a value of type Bool, with value <i>b</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonValue::QJsonValue(bool b)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#QJsonValue-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a value of type Bool, with value <i>b</i>.</p></div>
    pub unsafe fn from_bool(b: bool) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValue_QJsonValue1(b);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a value of type Double, with value <i>n</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonValue::QJsonValue(double n)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#QJsonValue-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a value of type Double, with value <i>n</i>.</p></div>
    pub unsafe fn from_double(
        n: ::std::os::raw::c_double,
    ) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValue_QJsonValue2(n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonValue::QJsonValue(int n)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#QJsonValue-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Creates a value of type Double, with value <i>n</i>.</p></div>
    pub unsafe fn from_int(n: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValue_QJsonValue3(n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonValue::QJsonValue(qint64 n)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#QJsonValue-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Creates a value of type Double, with value <i>n</i>. NOTE: the integer limits for IEEE 754 double precision data is 2^53 (-9007199254740992 to +9007199254740992). If you pass in values outside this range expect a loss of precision to occur.</p></div>
    pub unsafe fn from_i64(n: i64) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValue_QJsonValue4(n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a value of type String, with value <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonValue::QJsonValue(const QString& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#QJsonValue-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a value of type String, with value <i>s</i>.</p></div>
    pub unsafe fn from_q_string(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValue_QJsonValue5(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a value of type String, with value <i>s</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonValue::QJsonValue(QLatin1String s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#QJsonValue-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a value of type String, with value <i>s</i>.</p></div>
    pub unsafe fn from_q_latin1_string(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValue_QJsonValue6(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a value of type String with value <i>s</i>, assuming UTF-8 encoding of the input.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonValue::QJsonValue(const char* s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#QJsonValue-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a value of type String with value <i>s</i>, assuming UTF-8 encoding of the input.</p>
    /// <p>You can disable this constructor by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications.</p>
    /// <p>This function was introduced in  Qt 5.3.</p></div>
    pub unsafe fn from_char(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValue_QJsonValue7(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a value of type Array, with value <i>a</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonValue::QJsonValue(const QJsonArray& a)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#QJsonValue-8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a value of type Array, with value <i>a</i>.</p></div>
    pub unsafe fn from_q_json_array(
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonArray>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValue_QJsonValue8(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonArray>>::cast_into(a).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a value of type Object, with value <i>o</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonValue::QJsonValue(const QJsonObject& o)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#QJsonValue-9">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a value of type Object, with value <i>o</i>.</p></div>
    pub unsafe fn from_q_json_object(
        o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonObject>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValue_QJsonValue9(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonObject>>::cast_into(o)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> class encapsulates a value in JSON.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonValue::QJsonValue()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> class encapsulates a value in JSON.</p>
    /// <p>A value in JSON can be one of 6 basic types:</p>
    /// <p>JSON is a format to store structured data. It has 6 basic data types:</p>
    /// <ul>
    /// <li>bool <a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">QJsonValue::Bool</a></li>
    /// <li>double <a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">QJsonValue::Double</a></li>
    /// <li>string <a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">QJsonValue::String</a></li>
    /// <li>array <a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">QJsonValue::Array</a></li>
    /// <li>object <a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">QJsonValue::Object</a></li>
    /// <li>null <a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">QJsonValue::Null</a></li>
    /// </ul>
    /// <p>A value can represent any of the above data types. In addition, <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> has one special flag to represent undefined values. This can be queried with <a href="http://doc.qt.io/qt-5/qjsonvalue.html#isUndefined">isUndefined</a>().</p>
    /// <p>The type of the value can be queried with <a href="http://doc.qt.io/qt-5/qjsonvalue.html#type">type</a>() or accessors like <a href="http://doc.qt.io/qt-5/qjsonvalue.html#isBool">isBool</a>(), <a href="http://doc.qt.io/qt-5/qjsonvalue.html#isString">isString</a>(), and so on. Likewise, the value can be converted to the type stored in it using the <a href="http://doc.qt.io/qt-5/qjsonvalue.html#toBool">toBool</a>(), <a href="http://doc.qt.io/qt-5/qjsonvalue.html#toString">toString</a>() and so on.</p>
    /// <p>Values are strictly typed internally and contrary to <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> will not attempt to do any implicit type conversions. This implies that converting to a type that is not stored in the value will return a default constructed return value.</p>
    /// <a name="qjsonvalueref"></a></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValue_QJsonValue11();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonValue::QJsonValue(const QJsonValue& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#QJsonValue-10">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a copy of <i>other</i>.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonValue>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValue_QJsonValue10(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonValue>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Swaps the value <i>other</i> with this. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QJsonValue::swap(QJsonValue& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps the value <i>other</i> with this. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QJsonValue>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QJsonValue_swap(
            self as *mut crate::QJsonValue,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QJsonValue>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonArray QJsonValue::toArray() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#toArray-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Converts the value to an array and returns it.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qjsonvalue.html#type">type</a>() is not Array, a <a href="http://doc.qt.io/qt-5/qjsonarray.html#QJsonArray">QJsonArray()</a> will be returned.</p></div>
    pub unsafe fn to_array_0a(&self) -> ::cpp_utils::CppBox<crate::QJsonArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QJsonValue_toArray(self as *const crate::QJsonValue);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts the value to an array and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonArray QJsonValue::toArray(const QJsonArray& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#toArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the value to an array and returns it.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qjsonvalue.html#type">type</a>() is not Array, the <i>defaultValue</i> will be returned.</p></div>
    pub unsafe fn to_array_1a(
        &self,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonArray>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValue_toArray1(
            self as *const crate::QJsonValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonArray>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts the value to a bool and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QJsonValue::toBool(bool defaultValue = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#toBool">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the value to a bool and returns it.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qjsonvalue.html#type">type</a>() is not bool, the <i>defaultValue</i> will be returned.</p></div>
    pub unsafe fn to_bool_1a(&self, default_value: bool) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonValue_toBool(
            self as *const crate::QJsonValue,
            default_value,
        )
    }

    /// <p>Converts the value to a bool and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QJsonValue::toBool() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#toBool">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the value to a bool and returns it.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qjsonvalue.html#type">type</a>() is not bool, the <i>defaultValue</i> will be returned.</p></div>
    pub unsafe fn to_bool_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonValue_toBool1(self as *const crate::QJsonValue)
    }

    /// <p>Converts the value to a double and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QJsonValue::toDouble(double defaultValue = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#toDouble">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the value to a double and returns it.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qjsonvalue.html#type">type</a>() is not Double, the <i>defaultValue</i> will be returned.</p></div>
    pub unsafe fn to_double_1a(
        &self,
        default_value: ::std::os::raw::c_double,
    ) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QJsonValue_toDouble(
            self as *const crate::QJsonValue,
            default_value,
        )
    }

    /// <p>Converts the value to a double and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QJsonValue::toDouble() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#toDouble">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the value to a double and returns it.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qjsonvalue.html#type">type</a>() is not Double, the <i>defaultValue</i> will be returned.</p></div>
    pub unsafe fn to_double_0a(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QJsonValue_toDouble1(self as *const crate::QJsonValue)
    }

    /// <p>Converts the value to an int and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QJsonValue::toInt(int defaultValue = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#toInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the value to an int and returns it.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qjsonvalue.html#type">type</a>() is not Double or the value is not a whole number, the <i>defaultValue</i> will be returned.</p></div>
    pub unsafe fn to_int_1a(&self, default_value: ::std::os::raw::c_int) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QJsonValue_toInt(
            self as *const crate::QJsonValue,
            default_value,
        )
    }

    /// <p>Converts the value to an int and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QJsonValue::toInt() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#toInt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the value to an int and returns it.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qjsonvalue.html#type">type</a>() is not Double or the value is not a whole number, the <i>defaultValue</i> will be returned.</p></div>
    pub unsafe fn to_int_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QJsonValue_toInt1(self as *const crate::QJsonValue)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonObject QJsonValue::toObject() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#toObject-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Converts the value to an object and returns it.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qjsonvalue.html#type">type</a>() is not Object, the <a href="http://doc.qt.io/qt-5/qjsonobject.html#QJsonObject">QJsonObject()</a> will be returned.</p></div>
    pub unsafe fn to_object_0a(&self) -> ::cpp_utils::CppBox<crate::QJsonObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QJsonValue_toObject(self as *const crate::QJsonValue);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts the value to an object and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonObject QJsonValue::toObject(const QJsonObject& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#toObject">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the value to an object and returns it.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qjsonvalue.html#type">type</a>() is not Object, the <i>defaultValue</i> will be returned.</p></div>
    pub unsafe fn to_object_1a(
        &self,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonObject>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValue_toObject1(
            self as *const crate::QJsonValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonObject>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts the value to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QJsonValue::toString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#toString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the value to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and returns it.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qjsonvalue.html#type">type</a>() is not String, a null <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> will be returned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#isNull">QString::isNull</a>().</p></div>
    pub unsafe fn to_string_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QJsonValue_toString(self as *const crate::QJsonValue);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts the value to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QJsonValue::toString(const QString& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#toString-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the value to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and returns it.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qjsonvalue.html#type">type</a>() is not String, the <i>defaultValue</i> will be returned.</p></div>
    pub unsafe fn to_string_1a(
        &self,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValue_toString1(
            self as *const crate::QJsonValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts the value to a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant">QVariant()</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QJsonValue::toVariant() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#toVariant">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the value to a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariant">QVariant()</a>.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> types will be converted as follows:</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonvalue.html#fromVariant">fromVariant</a>().</p></div>
    pub unsafe fn to_variant(&self) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QJsonValue_toVariant(self as *const crate::QJsonValue);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the type of the value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonValue::Type QJsonValue::type() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#type">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the type of the value.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">QJsonValue::Type</a>.</p></div>
    pub unsafe fn type_(&self) -> crate::q_json_value::Type {
        crate::__ffi::ctr_qt_core_ffi_QJsonValue_type(self as *const crate::QJsonValue)
    }
}

/// C++ class: <span style='color: green;'>```QJsonValueRef```</span>.
#[repr(C)]
pub struct QJsonValueRef {
    _unused: u8,
}
impl QJsonValueRef {
    /// Calls C++ function: <span style='color: green;'>```QJsonValueRef& QJsonValueRef::operator=(const QJsonValue& val)```</span>.
    pub unsafe fn copy_from_q_json_value(
        &mut self,
        val: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonValue>>,
    ) -> ::cpp_utils::MutRef<crate::QJsonValueRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValueRef_operator_(
            self as *mut crate::QJsonValueRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonValue>>::cast_into(val)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```QJsonValueRef& QJsonValueRef::operator=(const QJsonValueRef& val)```</span>.
    pub unsafe fn copy_from_q_json_value_ref(
        &mut self,
        val: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonValueRef>>,
    ) -> ::cpp_utils::MutRef<crate::QJsonValueRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValueRef_operator_1(
            self as *mut crate::QJsonValueRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonValueRef>>::cast_into(val)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QJsonValueRef::isArray() const```</span>.
    pub unsafe fn is_array(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonValueRef_isArray(self as *const crate::QJsonValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QJsonValueRef::isBool() const```</span>.
    pub unsafe fn is_bool(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonValueRef_isBool(self as *const crate::QJsonValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QJsonValueRef::isDouble() const```</span>.
    pub unsafe fn is_double(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonValueRef_isDouble(self as *const crate::QJsonValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QJsonValueRef::isNull() const```</span>.
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonValueRef_isNull(self as *const crate::QJsonValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QJsonValueRef::isObject() const```</span>.
    pub unsafe fn is_object(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonValueRef_isObject(self as *const crate::QJsonValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QJsonValueRef::isString() const```</span>.
    pub unsafe fn is_string(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonValueRef_isString(self as *const crate::QJsonValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QJsonValueRef::isUndefined() const```</span>.
    pub unsafe fn is_undefined(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonValueRef_isUndefined(self as *const crate::QJsonValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonValueRef::QJsonValueRef(QJsonArray* array, int idx)```</span>.
    pub unsafe fn from_q_json_array_int(
        array: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QJsonArray>>,
        idx: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QJsonValueRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValueRef_QJsonValueRef(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QJsonArray>>::cast_into(array)
                .as_mut_raw_ptr(),
            idx,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonValueRef::QJsonValueRef(QJsonObject* object, int idx)```</span>.
    pub unsafe fn from_q_json_object_int(
        object: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QJsonObject>>,
        idx: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QJsonValueRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValueRef_QJsonValueRef1(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QJsonObject>>::cast_into(object)
                .as_mut_raw_ptr(),
            idx,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonValueRef::QJsonValueRef(const QJsonValueRef& other)```</span>.
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonValueRef>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonValueRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValueRef_QJsonValueRef3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonValueRef>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QJsonArray QJsonValueRef::toArray() const```</span>.
    pub unsafe fn to_array(&self) -> ::cpp_utils::CppBox<crate::QJsonArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValueRef_toArray(
            self as *const crate::QJsonValueRef,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QJsonValueRef::toBool() const```</span>.
    pub unsafe fn to_bool_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonValueRef_toBool(self as *const crate::QJsonValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QJsonValueRef::toBool(bool defaultValue) const```</span>.
    pub unsafe fn to_bool_1a(&self, default_value: bool) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonValueRef_toBool1(
            self as *const crate::QJsonValueRef,
            default_value,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```double QJsonValueRef::toDouble() const```</span>.
    pub unsafe fn to_double_0a(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QJsonValueRef_toDouble(self as *const crate::QJsonValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```double QJsonValueRef::toDouble(double defaultValue) const```</span>.
    pub unsafe fn to_double_1a(
        &self,
        default_value: ::std::os::raw::c_double,
    ) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QJsonValueRef_toDouble1(
            self as *const crate::QJsonValueRef,
            default_value,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```int QJsonValueRef::toInt() const```</span>.
    pub unsafe fn to_int_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QJsonValueRef_toInt(self as *const crate::QJsonValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```int QJsonValueRef::toInt(int defaultValue) const```</span>.
    pub unsafe fn to_int_1a(&self, default_value: ::std::os::raw::c_int) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QJsonValueRef_toInt1(
            self as *const crate::QJsonValueRef,
            default_value,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```QJsonObject QJsonValueRef::toObject() const```</span>.
    pub unsafe fn to_object(&self) -> ::cpp_utils::CppBox<crate::QJsonObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValueRef_toObject(
            self as *const crate::QJsonValueRef,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QJsonValue QJsonValueRef::operator QJsonValue() const```</span>.
    pub unsafe fn to_q_json_value(&self) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValueRef_operator_QJsonValue(
            self as *const crate::QJsonValueRef,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QString QJsonValueRef::toString() const```</span>.
    pub unsafe fn to_string_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValueRef_toString(
            self as *const crate::QJsonValueRef,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QString QJsonValueRef::toString(const QString& defaultValue) const```</span>.
    pub unsafe fn to_string_1a(
        &self,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValueRef_toString1(
            self as *const crate::QJsonValueRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QVariant QJsonValueRef::toVariant() const```</span>.
    pub unsafe fn to_variant(&self) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonValueRef_toVariant(
            self as *const crate::QJsonValueRef,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QJsonValue::Type QJsonValueRef::type() const```</span>.
    pub unsafe fn type_(&self) -> crate::q_json_value::Type {
        crate::__ffi::ctr_qt_core_ffi_QJsonValueRef_type(self as *const crate::QJsonValueRef)
    }
}

pub mod q_json_array {
    //! C++ type: <span style='color: green;'>```QJsonArray```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html">QJsonArray::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QJsonArray::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html">QJsonArray::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html">QJsonArray::iterator</a> allows you to iterate over a <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and to modify the array item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>, use <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html">QJsonArray::const_iterator</a> instead. It is generally a good practice to use <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html">QJsonArray::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> through the iterator. Const iterators are slightly faster and improves code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html">QJsonArray::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> function like <a href="http://doc.qt.io/qt-5/qjsonarray.html#begin">QJsonArray::begin</a>(), <a href="http://doc.qt.io/qt-5/qjsonarray.html#end">QJsonArray::end</a>(), or <a href="http://doc.qt.io/qt-5/qjsonarray.html#insert">QJsonArray::insert</a>() before you can start iterating.</p>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>Multiple iterators can be used on the same array. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> will render all existing iterators undefined.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// Returns the value of the <span style='color: green;'>```a```</span> field.
        pub unsafe fn a(&self) -> ::cpp_utils::MutPtr<crate::QJsonArray> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_a(
                self as *const crate::q_json_array::Iterator,
            );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }

        /// <p>The <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html">QJsonArray::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonArray::iterator& QJsonArray::iterator::operator=(const QJsonArray::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html">QJsonArray::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html">QJsonArray::iterator</a> allows you to iterate over a <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and to modify the array item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>, use <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html">QJsonArray::const_iterator</a> instead. It is generally a good practice to use <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html">QJsonArray::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> through the iterator. Const iterators are slightly faster and improves code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html">QJsonArray::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> function like <a href="http://doc.qt.io/qt-5/qjsonarray.html#begin">QJsonArray::begin</a>(), <a href="http://doc.qt.io/qt-5/qjsonarray.html#end">QJsonArray::end</a>(), or <a href="http://doc.qt.io/qt-5/qjsonarray.html#insert">QJsonArray::insert</a>() before you can start iterating.</p>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>Multiple iterators can be used on the same array. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> will render all existing iterators undefined.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_json_array::Iterator>>,
        ) -> ::cpp_utils::MutRef<crate::q_json_array::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_operator_8(self as *mut crate::q_json_array::Iterator, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_json_array::Iterator>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonArray::iterator QJsonArray::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_json_array::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_operator__12(
                self as *mut crate::q_json_array::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Returns the value of the <span style='color: green;'>```i```</span> field.
        pub unsafe fn i(&self) -> ::std::os::raw::c_int {
            crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_i(
                self as *const crate::q_json_array::Iterator,
            )
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonArray::iterator QJsonArray::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_json_array::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_operator__10(
                self as *mut crate::q_json_array::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonValueRef QJsonArray::iterator::operator[](int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>. If you assign to it, the assignment will apply to the character in the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> of <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> from which you got the reference.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::QJsonValueRef> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_operator__(
                self as *const crate::q_json_array::Iterator,
                j,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonArray::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#begin">QJsonArray::begin</a>() and <a href="http://doc.qt.io/qt-5/qjsonarray.html#end">QJsonArray::end</a>().</p></div>
        pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::q_json_array::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonArray::iterator::iterator(QJsonArray* array, int index)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#iterator">C++ documentation</a> for <span style='color: green;'>```iterator::iterator()```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#begin">QJsonArray::begin</a>() and <a href="http://doc.qt.io/qt-5/qjsonarray.html#end">QJsonArray::end</a>().</p></div>
        pub unsafe fn new_2a(
            array: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QJsonArray>>,
            index: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_json_array::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_iterator1(
                ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QJsonArray>>::cast_into(array)
                    .as_mut_raw_ptr(),
                index,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>The <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html">QJsonArray::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonArray::iterator::iterator(const QJsonArray::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html">QJsonArray::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html">QJsonArray::iterator</a> allows you to iterate over a <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and to modify the array item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>, use <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html">QJsonArray::const_iterator</a> instead. It is generally a good practice to use <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html">QJsonArray::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> through the iterator. Const iterators are slightly faster and improves code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html">QJsonArray::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> function like <a href="http://doc.qt.io/qt-5/qjsonarray.html#begin">QJsonArray::begin</a>(), <a href="http://doc.qt.io/qt-5/qjsonarray.html#end">QJsonArray::end</a>(), or <a href="http://doc.qt.io/qt-5/qjsonarray.html#insert">QJsonArray::insert</a>() before you can start iterating.</p>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>Multiple iterators can be used on the same array. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> will render all existing iterators undefined.</p></div>
        pub unsafe fn new_copy(
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_json_array::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_json_array::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_iterator2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_json_array::Iterator>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Sets the value of the <span style='color: green;'>```a```</span> field.
        pub unsafe fn set_a(
            &mut self,
            value: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QJsonArray>>,
        ) {
            crate::__ffi::ctr_qt_core_ffi_set_QJsonArray_iterator_a(
                self as *mut crate::q_json_array::Iterator,
                ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QJsonArray>>::cast_into(value)
                    .as_mut_raw_ptr(),
            )
        }

        /// Sets the value of the <span style='color: green;'>```i```</span> field.
        pub unsafe fn set_i(&mut self, value: ::std::os::raw::c_int) {
            crate::__ffi::ctr_qt_core_ffi_set_QJsonArray_iterator_i(
                self as *mut crate::q_json_array::Iterator,
                value,
            )
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html">QJsonArray::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QJsonArray::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html">QJsonArray::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html">QJsonArray::const_iterator</a> allows you to iterate over a <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>. If you want to modify the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html">QJsonArray::iterator</a> instead. It is generally a good practice to use <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html">QJsonArray::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> through the iterator. Const iterators are slightly faster and improves code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html">QJsonArray::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> function like <a href="http://doc.qt.io/qt-5/qjsonarray.html#constBegin">QJsonArray::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qjsonarray.html#constEnd">QJsonArray::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qjsonarray.html#insert">QJsonArray::insert</a>() before you can start iterating.</p>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>Multiple iterators can be used on the same array. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> will render all existing iterators undefined.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// Returns the value of the <span style='color: green;'>```a```</span> field.
        pub unsafe fn a(&self) -> ::cpp_utils::Ptr<crate::QJsonArray> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonArray_const_iterator_a(
                self as *const crate::q_json_array::ConstIterator,
            );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }

        /// <p>The <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html">QJsonArray::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonArray::const_iterator& QJsonArray::const_iterator::operator=(const QJsonArray::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html">QJsonArray::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html">QJsonArray::const_iterator</a> allows you to iterate over a <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>. If you want to modify the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html">QJsonArray::iterator</a> instead. It is generally a good practice to use <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html">QJsonArray::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> through the iterator. Const iterators are slightly faster and improves code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html">QJsonArray::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> function like <a href="http://doc.qt.io/qt-5/qjsonarray.html#constBegin">QJsonArray::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qjsonarray.html#constEnd">QJsonArray::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qjsonarray.html#insert">QJsonArray::insert</a>() before you can start iterating.</p>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>Multiple iterators can be used on the same array. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> will render all existing iterators undefined.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_json_array::ConstIterator>>,
        ) -> ::cpp_utils::MutRef<crate::q_json_array::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonArray_const_iterator_operator_6(self as *mut crate::q_json_array::ConstIterator, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_json_array::ConstIterator>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonArray::const_iterator QJsonArray::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_json_array::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonArray_const_iterator_operator__8(
                self as *mut crate::q_json_array::ConstIterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Returns the value of the <span style='color: green;'>```i```</span> field.
        pub unsafe fn i(&self) -> ::std::os::raw::c_int {
            crate::__ffi::ctr_qt_core_ffi_QJsonArray_const_iterator_i(
                self as *const crate::q_json_array::ConstIterator,
            )
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonArray::const_iterator QJsonArray::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_json_array::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonArray_const_iterator_operator__6(
                self as *mut crate::q_json_array::ConstIterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonValue QJsonArray::const_iterator::operator[](int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::QJsonValue> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonArray_const_iterator_operator__(
                self as *const crate::q_json_array::ConstIterator,
                j,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonArray::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#constBegin">QJsonArray::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qjsonarray.html#constEnd">QJsonArray::constEnd</a>().</p></div>
        pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::q_json_array::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_const_iterator_const_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonArray::const_iterator::const_iterator(const QJsonArray* array, int index)```</span>.
        pub unsafe fn new_2a(
            array: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QJsonArray>>,
            index: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_json_array::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_const_iterator_const_iterator1(
                    ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QJsonArray>>::cast_into(array)
                        .as_raw_ptr(),
                    index,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs a copy of <i>other</i>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonArray::const_iterator::const_iterator(const QJsonArray::iterator& o)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#const_iterator-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p></div>
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_json_array::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_json_array::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonArray_const_iterator_const_iterator3(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_json_array::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs a copy of <i>other</i>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonArray::const_iterator::const_iterator(const QJsonArray::const_iterator& o)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#const_iterator-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p></div>
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_json_array::ConstIterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_json_array::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonArray_const_iterator_const_iterator2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_json_array::ConstIterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Sets the value of the <span style='color: green;'>```a```</span> field.
        pub unsafe fn set_a(
            &mut self,
            value: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QJsonArray>>,
        ) {
            crate::__ffi::ctr_qt_core_ffi_set_QJsonArray_const_iterator_a(
                self as *mut crate::q_json_array::ConstIterator,
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QJsonArray>>::cast_into(value)
                    .as_raw_ptr(),
            )
        }

        /// Sets the value of the <span style='color: green;'>```i```</span> field.
        pub unsafe fn set_i(&mut self, value: ::std::os::raw::c_int) {
            crate::__ffi::ctr_qt_core_ffi_set_QJsonArray_const_iterator_i(
                self as *mut crate::q_json_array::ConstIterator,
                value,
            )
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_json_array::Iterator {
        type Output = ::cpp_utils::CppBox<crate::QJsonValueRef>;
        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonValueRef QJsonArray::iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>().</p></div>
        fn indirection(self) -> ::cpp_utils::CppBox<crate::QJsonValueRef> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_operator_(
                    self as *const crate::q_json_array::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_json_array::Iterator>>
        for crate::q_json_array::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QJsonArray::iterator::operator==(const QJsonArray::iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_json_array::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_operator__1(
                    self as *const crate::q_json_array::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_json_array::Iterator>>
        for crate::q_json_array::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QJsonArray::iterator::operator<(const QJsonArray::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_json_array::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_operator_1(
                    self as *const crate::q_json_array::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_json_array::Iterator>>
        for crate::q_json_array::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QJsonArray::iterator::operator<=(const QJsonArray::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_json_array::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_operator__3(
                    self as *const crate::q_json_array::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_json_array::Iterator>>
        for crate::q_json_array::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QJsonArray::iterator::operator>(const QJsonArray::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_json_array::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_operator_2(
                    self as *const crate::q_json_array::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_json_array::Iterator>>
        for crate::q_json_array::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QJsonArray::iterator::operator>=(const QJsonArray::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_json_array::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_operator__4(
                    self as *const crate::q_json_array::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_json_array::ConstIterator>>
        for crate::q_json_array::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QJsonArray::iterator::operator==(const QJsonArray::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_json_array::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_operator__5(
                    self as *const crate::q_json_array::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_json_array::ConstIterator>>
        for crate::q_json_array::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QJsonArray::iterator::operator<(const QJsonArray::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_json_array::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_operator_3(
                    self as *const crate::q_json_array::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_json_array::ConstIterator>>
        for crate::q_json_array::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QJsonArray::iterator::operator<=(const QJsonArray::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_json_array::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_operator__7(
                    self as *const crate::q_json_array::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_json_array::ConstIterator>>
        for crate::q_json_array::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QJsonArray::iterator::operator>(const QJsonArray::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_json_array::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_operator_4(
                    self as *const crate::q_json_array::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_json_array::ConstIterator>>
        for crate::q_json_array::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QJsonArray::iterator::operator>=(const QJsonArray::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_json_array::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_operator__8(
                    self as *const crate::q_json_array::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_json_array::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_json_array::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonArray::iterator& QJsonArray::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_json_array::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_operator__9(
                    self as *mut crate::q_json_array::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_json_array::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_json_array::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonArray::iterator& QJsonArray::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_json_array::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_operator__11(
                    self as *mut crate::q_json_array::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_int> for crate::q_json_array::Iterator {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonArray::iterator& QJsonArray::iterator::operator+=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_operator__13(
                    self as *mut crate::q_json_array::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_int> for crate::q_json_array::Iterator {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonArray::iterator& QJsonArray::iterator::operator-=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_operator__14(
                    self as *mut crate::q_json_array::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_int> for &crate::q_json_array::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_json_array::Iterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions forward from this iterator. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonArray::iterator QJsonArray::iterator::operator+(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions forward from this iterator. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p>This operation can be slow for large <i>j</i> values.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn add(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_json_array::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_operator_5(
                    self as *const crate::q_json_array::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_int> for &crate::q_json_array::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_json_array::Iterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions backward from this iterator. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonArray::iterator QJsonArray::iterator::operator-(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions backward from this iterator. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p>This operation can be slow for large <i>j</i> values.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn sub(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_json_array::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_operator_6(
                    self as *const crate::q_json_array::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_json_array::Iterator>>
        for &crate::q_json_array::Iterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QJsonArray::iterator::operator-(QJsonArray::iterator j) const```</span>.
        fn sub(self, j: ::cpp_utils::Ref<crate::q_json_array::Iterator>) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_operator_7(
                    self as *const crate::q_json_array::Iterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_json_array::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::QJsonValue>;
        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonValue QJsonArray::const_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        fn indirection(self) -> ::cpp_utils::CppBox<crate::QJsonValue> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_const_iterator_operator_(
                    self as *const crate::q_json_array::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_json_array::ConstIterator>>
        for crate::q_json_array::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QJsonArray::const_iterator::operator==(const QJsonArray::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_json_array::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_const_iterator_operator__1(
                    self as *const crate::q_json_array::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_json_array::ConstIterator>>
        for crate::q_json_array::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QJsonArray::const_iterator::operator<(const QJsonArray::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_json_array::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_const_iterator_operator_1(
                    self as *const crate::q_json_array::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_json_array::ConstIterator>>
        for crate::q_json_array::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QJsonArray::const_iterator::operator<=(const QJsonArray::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_json_array::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_const_iterator_operator__3(
                    self as *const crate::q_json_array::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_json_array::ConstIterator>>
        for crate::q_json_array::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QJsonArray::const_iterator::operator>(const QJsonArray::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_json_array::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_const_iterator_operator_2(
                    self as *const crate::q_json_array::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_json_array::ConstIterator>>
        for crate::q_json_array::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QJsonArray::const_iterator::operator>=(const QJsonArray::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_json_array::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_const_iterator_operator__4(
                    self as *const crate::q_json_array::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_json_array::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_json_array::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonArray::const_iterator& QJsonArray::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_json_array::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_const_iterator_operator__5(
                    self as *mut crate::q_json_array::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_json_array::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_json_array::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonArray::const_iterator& QJsonArray::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_json_array::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_const_iterator_operator__7(
                    self as *mut crate::q_json_array::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_int> for crate::q_json_array::ConstIterator {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonArray::const_iterator& QJsonArray::const_iterator::operator+=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_const_iterator_operator__9(
                    self as *mut crate::q_json_array::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_int> for crate::q_json_array::ConstIterator {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonArray::const_iterator& QJsonArray::const_iterator::operator-=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_const_iterator_operator__10(
                    self as *mut crate::q_json_array::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_int> for &crate::q_json_array::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_json_array::ConstIterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions forward from this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonArray::const_iterator QJsonArray::const_iterator::operator+(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions forward from this iterator.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>().</p></div>
        fn add(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_json_array::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_const_iterator_operator_3(
                    self as *const crate::q_json_array::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_int> for &crate::q_json_array::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_json_array::ConstIterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions backward from this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonArray::const_iterator QJsonArray::const_iterator::operator-(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions backward from this iterator.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>().</p></div>
        fn sub(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_json_array::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_const_iterator_operator_4(
                    self as *const crate::q_json_array::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_json_array::ConstIterator>>
        for &crate::q_json_array::ConstIterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QJsonArray::const_iterator::operator-(QJsonArray::const_iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_json_array::ConstIterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonArray_const_iterator_operator_5(
                    self as *const crate::q_json_array::ConstIterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_json_array::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html">QJsonArray::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QJsonArray::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html">QJsonArray::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html">QJsonArray::iterator</a> allows you to iterate over a <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and to modify the array item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>, use <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html">QJsonArray::const_iterator</a> instead. It is generally a good practice to use <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html">QJsonArray::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> through the iterator. Const iterators are slightly faster and improves code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html">QJsonArray::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> function like <a href="http://doc.qt.io/qt-5/qjsonarray.html#begin">QJsonArray::begin</a>(), <a href="http://doc.qt.io/qt-5/qjsonarray.html#end">QJsonArray::end</a>(), or <a href="http://doc.qt.io/qt-5/qjsonarray.html#insert">QJsonArray::insert</a>() before you can start iterating.</p>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>Multiple iterators can be used on the same array. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> will render all existing iterators undefined.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QJsonArray_iterator_diterator(
                self as *mut crate::q_json_array::Iterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_json_array::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html">QJsonArray::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QJsonArray::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html">QJsonArray::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html">QJsonArray::const_iterator</a> allows you to iterate over a <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>. If you want to modify the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html">QJsonArray::iterator</a> instead. It is generally a good practice to use <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html">QJsonArray::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> through the iterator. Const iterators are slightly faster and improves code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html">QJsonArray::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> function like <a href="http://doc.qt.io/qt-5/qjsonarray.html#constBegin">QJsonArray::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qjsonarray.html#constEnd">QJsonArray::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qjsonarray.html#insert">QJsonArray::insert</a>() before you can start iterating.</p>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>Multiple iterators can be used on the same array. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> will render all existing iterators undefined.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QJsonArray_const_iterator_dconst_iterator(
                self as *mut crate::q_json_array::ConstIterator,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> class encapsulates a JSON array.</p>
///
/// C++ class: <span style='color: green;'>```QJsonArray```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qjsonarray.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> class encapsulates a JSON array.</p>
/// <p>A JSON array is a list of values. The list can be manipulated by inserting and removing <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>'s from the array.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> can be converted to and from a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a>. You can query the number of entries with <a href="http://doc.qt.io/qt-5/qjsonarray.html#size">size</a>(), <a href="http://doc.qt.io/qt-5/qjsonarray.html#insert">insert</a>(), and <a href="http://doc.qt.io/qt-5/qjsonarray.html#removeAt">removeAt</a>() entries from it and iterate over its content using the standard C++ iterator pattern.</p>
/// <p><a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> is an implicitly shared class and shares the data with the document it has been created from as long as it is not being modified.</p>
/// <p>You can convert the array to and from text based JSON through <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a>.</p></div>
#[repr(C)]
pub struct QJsonArray {
    _unused: u8,
}
impl QJsonArray {
    /// <p>Inserts <i>value</i> at the end of the array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QJsonArray::append(const QJsonValue& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#prepend">prepend</a>() and <a href="http://doc.qt.io/qt-5/qjsonarray.html#insert">insert</a>().</p></div>
    pub unsafe fn append(
        &mut self,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonValue>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QJsonArray_append(
            self as *mut crate::QJsonArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonValue>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> representing the value for index <i>i</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonValue QJsonArray::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> representing the value for index <i>i</i>.</p>
    /// <p>The returned <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> is <code>Undefined</code>, if <i>i</i> is out of bounds.</p></div>
    pub unsafe fn at(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QJsonArray_at(self as *const crate::QJsonArray, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonArray::const_iterator QJsonArray::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qjsonarray.html#cend">cend</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn cbegin(&self) -> ::cpp_utils::CppBox<crate::q_json_array::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QJsonArray_cbegin(self as *const crate::QJsonArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonArray::const_iterator QJsonArray::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qjsonarray.html#end">end</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn cend(&self) -> ::cpp_utils::CppBox<crate::q_json_array::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QJsonArray_cend(self as *const crate::QJsonArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonArray::const_iterator QJsonArray::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qjsonarray.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(&self) -> ::cpp_utils::CppBox<crate::q_json_array::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QJsonArray_constBegin(self as *const crate::QJsonArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonArray::const_iterator QJsonArray::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qjsonarray.html#end">end</a>().</p></div>
    pub unsafe fn const_end(&self) -> ::cpp_utils::CppBox<crate::q_json_array::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QJsonArray_constEnd(self as *const crate::QJsonArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if the array contains an occurrence of <i>value</i>, otherwise <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QJsonArray::contains(const QJsonValue& element) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the array contains an occurrence of <i>value</i>, otherwise <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#count">count</a>().</p></div>
    pub unsafe fn contains(
        &self,
        element: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonValue>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonArray_contains(
            self as *const crate::QJsonArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonValue>>::cast_into(element)
                .as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonArray& QJsonArray::operator=(const QJsonArray& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this array.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonArray>>,
    ) -> ::cpp_utils::MutRef<crate::QJsonArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonArray_operator_(
            self as *mut crate::QJsonArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonArray>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Same as <a href="http://doc.qt.io/qt-5/qjsonarray.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QJsonArray::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as <a href="http://doc.qt.io/qt-5/qjsonarray.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#size">size</a>().</p></div>
    pub unsafe fn count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QJsonArray_count(self as *const crate::QJsonArray)
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qjsonarray.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the array is empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QJsonArray::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qjsonarray.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the array is empty.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonArray_empty(self as *const crate::QJsonArray)
    }

    /// <p>Removes the item pointed to by <i>it</i>, and returns an iterator pointing to the next item.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonArray::iterator QJsonArray::erase(QJsonArray::iterator it)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item pointed to by <i>it</i>, and returns an iterator pointing to the next item.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn erase(
        &mut self,
        it: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_json_array::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_json_array::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonArray_erase(
            self as *mut crate::QJsonArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_json_array::Iterator>>::cast_into(it)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the first value stored in the array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonValue QJsonArray::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the first value stored in the array.</p>
    /// <p>Same as <code>at(0)</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#at">at</a>().</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QJsonArray_first(self as *const crate::QJsonArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts the string list <i>list</i> to a <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QJsonArray QJsonArray::fromStringList(const QStringList& list)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#fromStringList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the string list <i>list</i> to a <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>.</p>
    /// <p>The values in <i>list</i> will be converted to JSON values.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#toVariantList">toVariantList</a>() and <a href="http://doc.qt.io/qt-5/qjsonvalue.html#fromVariant">QJsonValue::fromVariant</a>().</p></div>
    pub unsafe fn from_string_list(
        list: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonArray_fromStringList(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(list)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts the variant list <i>list</i> to a <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QJsonArray QJsonArray::fromVariantList(const QList<QVariant>& list)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#fromVariantList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the variant list <i>list</i> to a <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> values in <i>list</i> will be converted to JSON values.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#toVariantList">toVariantList</a>() and <a href="http://doc.qt.io/qt-5/qjsonvalue.html#fromVariant">QJsonValue::fromVariant</a>().</p></div>
    pub unsafe fn from_variant_list(
        list: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonArray_fromVariantList(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQVariant>>::cast_into(list)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the array (i.e., <code>0 &lt;= i &lt; size()</code>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonValueRef QJsonArray::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the array (i.e., <code>0 &lt;= i &lt; size()</code>).</p>
    /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>. If you assign to it, the assignment will apply to the character in the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> of <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> from which you got the reference.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#at">at</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QJsonValueRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QJsonArray_operator__(self as *mut crate::QJsonArray, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonValue QJsonArray::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qjsonarray.html#at">at</a>().</p></div>
    pub unsafe fn index(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonArray_operator__1(
            self as *const crate::QJsonArray,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the array. If <i>i</i> is <code>0</code>, the value is prepended to the array. If <i>i</i> is <a href="http://doc.qt.io/qt-5/qjsonarray.html#size">size</a>(), the value is appended to the array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QJsonArray::insert(int i, const QJsonValue& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the array. If <i>i</i> is <code>0</code>, the value is prepended to the array. If <i>i</i> is <a href="http://doc.qt.io/qt-5/qjsonarray.html#size">size</a>(), the value is appended to the array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qjsonarray.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qjsonarray.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qjsonarray.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn insert_int_q_json_value(
        &mut self,
        i: ::std::os::raw::c_int,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonValue>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QJsonArray_insert(
            self as *mut crate::QJsonArray,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonValue>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>Inserts <i>value</i> before the position pointed to by <i>before</i>, and returns an iterator pointing to the newly inserted item.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonArray::iterator QJsonArray::insert(QJsonArray::iterator before, const QJsonValue& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> before the position pointed to by <i>before</i>, and returns an iterator pointing to the newly inserted item.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#erase">erase</a>() and <a href="http://doc.qt.io/qt-5/qjsonarray.html#insert">insert</a>().</p></div>
    pub unsafe fn insert_iterator_q_json_value(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_json_array::Iterator>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonValue>>,
    ) -> ::cpp_utils::CppBox<crate::q_json_array::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonArray_insert1(
            self as *mut crate::QJsonArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_json_array::Iterator>>::cast_into(
                before,
            )
            .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonValue>>::cast_into(value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if the object is empty. This is the same as <a href="http://doc.qt.io/qt-5/qjsonarray.html#size">size</a>() == 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QJsonArray::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the object is empty. This is the same as <a href="http://doc.qt.io/qt-5/qjsonarray.html#size">size</a>() == 0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonArray_isEmpty(self as *const crate::QJsonArray)
    }

    /// <p>Returns the last value stored in the array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonValue QJsonArray::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the last value stored in the array.</p>
    /// <p>Same as <code>at(size() - 1)</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#at">at</a>().</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QJsonArray_last(self as *const crate::QJsonArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates an empty array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonArray::QJsonArray()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#QJsonArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates an empty array.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QJsonArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonArray_QJsonArray();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonArray::QJsonArray(const QJsonArray& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#QJsonArray-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a copy of <i>other</i>.</p>
    /// <p>Since <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> is implicitly shared, the copy is shallow as long as the object doesn't get modified.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonArray>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonArray_QJsonArray2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonArray>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qjsonarray.html#removeLast">removeLast</a>(). The array must not be empty. If the array can be empty, call <a href="http://doc.qt.io/qt-5/qjsonarray.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QJsonArray::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qjsonarray.html#removeLast">removeLast</a>(). The array must not be empty. If the array can be empty, call <a href="http://doc.qt.io/qt-5/qjsonarray.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QJsonArray_pop_back(self as *mut crate::QJsonArray)
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qjsonarray.html#removeFirst">removeFirst</a>(). The array must not be empty. If the array can be empty, call <a href="http://doc.qt.io/qt-5/qjsonarray.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QJsonArray::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qjsonarray.html#removeFirst">removeFirst</a>(). The array must not be empty. If the array can be empty, call <a href="http://doc.qt.io/qt-5/qjsonarray.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QJsonArray_pop_front(self as *mut crate::QJsonArray)
    }

    /// <p>Inserts <i>value</i> at the beginning of the array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QJsonArray::prepend(const QJsonValue& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the array.</p>
    /// <p>This is the same as <code>insert(0, value)</code> and will prepend <i>value</i> to the array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qjsonarray.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonValue>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QJsonArray_prepend(
            self as *mut crate::QJsonArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonValue>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qjsonarray.html#append">append</a>(value) and will append <i>value</i> to the array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QJsonArray::push_back(const QJsonValue& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qjsonarray.html#append">append</a>(value) and will append <i>value</i> to the array.</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonValue>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QJsonArray_push_back(
            self as *mut crate::QJsonArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonValue>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qjsonarray.html#prepend">prepend</a>(value) and will prepend <i>value</i> to the array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QJsonArray::push_front(const QJsonValue& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qjsonarray.html#prepend">prepend</a>(value) and will prepend <i>value</i> to the array.</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonValue>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QJsonArray_push_front(
            self as *mut crate::QJsonArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonValue>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes the value at index position <i>i</i>. <i>i</i> must be a valid index position in the array (i.e., <code>0 &lt;= i &lt; size()</code>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QJsonArray::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the value at index position <i>i</i>. <i>i</i> must be a valid index position in the array (i.e., <code>0 &lt;= i &lt; size()</code>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qjsonarray.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QJsonArray_removeAt(self as *mut crate::QJsonArray, i)
    }

    /// <p>Removes the first item in the array. Calling this function is equivalent to calling <code>removeAt(0)</code>. The array must not be empty. If the array can be empty, call <a href="http://doc.qt.io/qt-5/qjsonarray.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QJsonArray::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the array. Calling this function is equivalent to calling <code>removeAt(0)</code>. The array must not be empty. If the array can be empty, call <a href="http://doc.qt.io/qt-5/qjsonarray.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qjsonarray.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QJsonArray_removeFirst(self as *mut crate::QJsonArray)
    }

    /// <p>Removes the last item in the array. Calling this function is equivalent to calling <code>removeAt(size() - 1)</code>. The array must not be empty. If the array can be empty, call <a href="http://doc.qt.io/qt-5/qjsonarray.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QJsonArray::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the array. Calling this function is equivalent to calling <code>removeAt(size() - 1)</code>. The array must not be empty. If the array can be empty, call <a href="http://doc.qt.io/qt-5/qjsonarray.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qjsonarray.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QJsonArray_removeLast(self as *mut crate::QJsonArray)
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the array (i.e., <code>0 &lt;= i &lt; size()</code>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QJsonArray::replace(int i, const QJsonValue& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the array (i.e., <code>0 &lt;= i &lt; size()</code>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qjsonarray.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonValue>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QJsonArray_replace(
            self as *mut crate::QJsonArray,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonValue>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the number of values stored in the array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QJsonArray::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of values stored in the array.</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QJsonArray_size(self as *const crate::QJsonArray)
    }

    /// <p>Swaps the array <i>other</i> with this. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QJsonArray::swap(QJsonArray& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps the array <i>other</i> with this. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QJsonArray>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QJsonArray_swap(
            self as *mut crate::QJsonArray,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QJsonArray>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the array (i.e., <code>0 &lt;= i &lt; size()</code>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonValue QJsonArray::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the array (i.e., <code>0 &lt;= i &lt; size()</code>).</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qjsonarray.html#removeAt">removeAt</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QJsonArray_takeAt(self as *mut crate::QJsonArray, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts this object to a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant> QJsonArray::toVariantList() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#toVariantList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts this object to a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a>.</p>
    /// <p>Returns the created map.</p></div>
    pub unsafe fn to_variant_list(&self) -> ::cpp_utils::CppBox<crate::QListOfQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonArray_toVariantList(
            self as *const crate::QJsonArray,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_json_parse_error {
    //! C++ type: <span style='color: green;'>```QJsonParseError```</span>

    /// <p>This enum describes the type of error that occurred during the parsing of a JSON document.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QJsonParseError::ParseError```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonparseerror.html#ParseError-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the type of error that occurred during the parsing of a JSON document.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct ParseError(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for ParseError {
        fn from(value: ::std::os::raw::c_int) -> Self {
            ParseError(value)
        }
    }

    impl From<ParseError> for ::std::os::raw::c_int {
        fn from(value: ParseError) -> Self {
            value.0
        }
    }

    impl ParseError {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl ParseError {
        /// No error occurred (C++ enum variant: <span style='color: green;'>```NoError = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoError: crate::q_json_parse_error::ParseError =
            crate::q_json_parse_error::ParseError(0);
        /// An object is not correctly terminated with a closing curly bracket (C++ enum variant: <span style='color: green;'>```UnterminatedObject = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const UnterminatedObject: crate::q_json_parse_error::ParseError =
            crate::q_json_parse_error::ParseError(1);
        /// A comma separating different items is missing (C++ enum variant: <span style='color: green;'>```MissingNameSeparator = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const MissingNameSeparator: crate::q_json_parse_error::ParseError =
            crate::q_json_parse_error::ParseError(2);
        /// The array is not correctly terminated with a closing square bracket (C++ enum variant: <span style='color: green;'>```UnterminatedArray = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const UnterminatedArray: crate::q_json_parse_error::ParseError =
            crate::q_json_parse_error::ParseError(3);
        /// A colon separating keys from values inside objects is missing (C++ enum variant: <span style='color: green;'>```MissingValueSeparator = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const MissingValueSeparator: crate::q_json_parse_error::ParseError =
            crate::q_json_parse_error::ParseError(4);
        /// The value is illegal (C++ enum variant: <span style='color: green;'>```IllegalValue = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const IllegalValue: crate::q_json_parse_error::ParseError =
            crate::q_json_parse_error::ParseError(5);
        /// The input stream ended while parsing a number (C++ enum variant: <span style='color: green;'>```TerminationByNumber = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const TerminationByNumber: crate::q_json_parse_error::ParseError =
            crate::q_json_parse_error::ParseError(6);
        /// The number is not well formed (C++ enum variant: <span style='color: green;'>```IllegalNumber = 7```</span>)
        #[allow(non_upper_case_globals)]
        pub const IllegalNumber: crate::q_json_parse_error::ParseError =
            crate::q_json_parse_error::ParseError(7);
        /// An illegal escape sequence occurred in the input (C++ enum variant: <span style='color: green;'>```IllegalEscapeSequence = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const IllegalEscapeSequence: crate::q_json_parse_error::ParseError =
            crate::q_json_parse_error::ParseError(8);
        /// An illegal UTF8 sequence occurred in the input (C++ enum variant: <span style='color: green;'>```IllegalUTF8String = 9```</span>)
        #[allow(non_upper_case_globals)]
        pub const IllegalUTF8String: crate::q_json_parse_error::ParseError =
            crate::q_json_parse_error::ParseError(9);
        /// A string wasn't terminated with a quote (C++ enum variant: <span style='color: green;'>```UnterminatedString = 10```</span>)
        #[allow(non_upper_case_globals)]
        pub const UnterminatedString: crate::q_json_parse_error::ParseError =
            crate::q_json_parse_error::ParseError(10);
        /// An object was expected but couldn't be found (C++ enum variant: <span style='color: green;'>```MissingObject = 11```</span>)
        #[allow(non_upper_case_globals)]
        pub const MissingObject: crate::q_json_parse_error::ParseError =
            crate::q_json_parse_error::ParseError(11);
        /// The JSON document is too deeply nested for the parser to parse it (C++ enum variant: <span style='color: green;'>```DeepNesting = 12```</span>)
        #[allow(non_upper_case_globals)]
        pub const DeepNesting: crate::q_json_parse_error::ParseError =
            crate::q_json_parse_error::ParseError(12);
        /// The JSON document is too large for the parser to parse it (C++ enum variant: <span style='color: green;'>```DocumentTooLarge = 13```</span>)
        #[allow(non_upper_case_globals)]
        pub const DocumentTooLarge: crate::q_json_parse_error::ParseError =
            crate::q_json_parse_error::ParseError(13);
        /// The parsed document contains additional garbage characters at the end (C++ enum variant: <span style='color: green;'>```GarbageAtEnd = 14```</span>)
        #[allow(non_upper_case_globals)]
        pub const GarbageAtEnd: crate::q_json_parse_error::ParseError =
            crate::q_json_parse_error::ParseError(14);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qjsonparseerror.html">QJsonParseError</a> class is used to report errors during JSON parsing.</p>
///
/// C++ class: <span style='color: green;'>```QJsonParseError```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qjsonparseerror.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qjsonparseerror.html">QJsonParseError</a> class is used to report errors during JSON parsing.</p></div>
#[repr(C)]
pub struct QJsonParseError {
    _unused: u8,
}
impl QJsonParseError {
    /// <p>The <a href="http://doc.qt.io/qt-5/qjsonparseerror.html">QJsonParseError</a> class is used to report errors during JSON parsing.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonParseError& QJsonParseError::operator=(const QJsonParseError& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonparseerror.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qjsonparseerror.html">QJsonParseError</a> class is used to report errors during JSON parsing.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonParseError>>,
    ) -> ::cpp_utils::MutRef<crate::QJsonParseError> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonParseError_operator_(
            self as *mut crate::QJsonParseError,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonParseError>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Returns the value of the <span style='color: green;'>```error```</span> field.
    pub unsafe fn error(&self) -> crate::q_json_parse_error::ParseError {
        crate::__ffi::ctr_qt_core_ffi_QJsonParseError_error(self as *const crate::QJsonParseError)
    }

    /// <p>Returns the human-readable message appropriate to the reported JSON parsing error.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QJsonParseError::errorString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonparseerror.html#errorString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the human-readable message appropriate to the reported JSON parsing error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonparseerror.html#error-var">error</a>.</p></div>
    pub unsafe fn error_string(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonParseError_errorString(
            self as *const crate::QJsonParseError,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qjsonparseerror.html">QJsonParseError</a> class is used to report errors during JSON parsing.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonParseError::QJsonParseError()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonparseerror.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qjsonparseerror.html">QJsonParseError</a> class is used to report errors during JSON parsing.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QJsonParseError> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonParseError_QJsonParseError();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qjsonparseerror.html">QJsonParseError</a> class is used to report errors during JSON parsing.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonParseError::QJsonParseError(const QJsonParseError& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonparseerror.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qjsonparseerror.html">QJsonParseError</a> class is used to report errors during JSON parsing.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonParseError>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonParseError> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonParseError_QJsonParseError1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonParseError>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Returns the value of the <span style='color: green;'>```offset```</span> field.
    pub unsafe fn offset(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QJsonParseError_offset(self as *const crate::QJsonParseError)
    }

    /// Sets the value of the <span style='color: green;'>```error```</span> field.
    pub unsafe fn set_error(&mut self, value: crate::q_json_parse_error::ParseError) {
        crate::__ffi::ctr_qt_core_ffi_set_QJsonParseError_error(
            self as *mut crate::QJsonParseError,
            value,
        )
    }

    /// Sets the value of the <span style='color: green;'>```offset```</span> field.
    pub unsafe fn set_offset(&mut self, value: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_set_QJsonParseError_offset(
            self as *mut crate::QJsonParseError,
            value,
        )
    }
}

pub mod q_json_document {
    //! C++ type: <span style='color: green;'>```QJsonDocument```</span>

    /// <p>This value is used to tell <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> whether to validate the binary data when converting to a <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> using <a href="http://doc.qt.io/qt-5/qjsondocument.html#fromBinaryData">fromBinaryData</a>() or <a href="http://doc.qt.io/qt-5/qjsondocument.html#fromRawData">fromRawData</a>().</p>
    ///
    /// C++ enum: <span style='color: green;'>```QJsonDocument::DataValidation```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#DataValidation-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This value is used to tell <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> whether to validate the binary data when converting to a <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> using <a href="http://doc.qt.io/qt-5/qjsondocument.html#fromBinaryData">fromBinaryData</a>() or <a href="http://doc.qt.io/qt-5/qjsondocument.html#fromRawData">fromRawData</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct DataValidation(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for DataValidation {
        fn from(value: ::std::os::raw::c_int) -> Self {
            DataValidation(value)
        }
    }

    impl From<DataValidation> for ::std::os::raw::c_int {
        fn from(value: DataValidation) -> Self {
            value.0
        }
    }

    impl DataValidation {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl DataValidation {
        /// Validate the data before using it. This is the default. (C++ enum variant: <span style='color: green;'>```Validate = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const Validate: crate::q_json_document::DataValidation =
            crate::q_json_document::DataValidation(0);
        /// Bypasses data validation. Only use if you received the data from a trusted place and know it's valid, as using of invalid data can crash the application. (C++ enum variant: <span style='color: green;'>```BypassValidation = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const BypassValidation: crate::q_json_document::DataValidation =
            crate::q_json_document::DataValidation(1);
    }

    /// <p>This value defines the format of the JSON byte array produced when converting to a <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> using <a href="http://doc.qt.io/qt-5/qjsondocument.html#toJson">toJson</a>().</p>
    ///
    /// C++ enum: <span style='color: green;'>```QJsonDocument::JsonFormat```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#JsonFormat-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This value defines the format of the JSON byte array produced when converting to a <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> using <a href="http://doc.qt.io/qt-5/qjsondocument.html#toJson">toJson</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct JsonFormat(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for JsonFormat {
        fn from(value: ::std::os::raw::c_int) -> Self {
            JsonFormat(value)
        }
    }

    impl From<JsonFormat> for ::std::os::raw::c_int {
        fn from(value: JsonFormat) -> Self {
            value.0
        }
    }

    impl JsonFormat {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl JsonFormat {
        /// Defines human readable output as follows:<pre class="cpp">
        ///
        ///   {
        /// &#32;     <span class="string">"Array"</span>: <span class="operator">[</span>
        /// &#32;   &#32;     <span class="keyword">true</span><span class="operator">,</span>
        /// &#32;   &#32;     <span class="number">999</span><span class="operator">,</span>
        /// &#32;   &#32;     <span class="string">"string"</span>
        /// &#32;     <span class="operator">]</span><span class="operator">,</span>
        /// &#32;     <span class="string">"Key"</span>: <span class="string">"Value"</span><span class="operator">,</span>
        /// &#32;     <span class="string">"null"</span>: null
        ///   }
        ///
        /// </pre> (C++ enum variant: <span style='color: green;'>```Indented = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const Indented: crate::q_json_document::JsonFormat =
            crate::q_json_document::JsonFormat(0);
        /// Defines a compact output as follows:<pre class="cpp">
        ///
        ///   {<span class="string">"Array"</span>:<span class="operator">[</span><span class="keyword">true</span><span class="operator">,</span><span class="number">999</span><span class="operator">,</span><span class="string">"string"</span><span class="operator">]</span><span class="operator">,</span><span class="string">"Key"</span>:<span class="string">"Value"</span><span class="operator">,</span><span class="string">"null"</span>:null}
        ///
        /// </pre> (C++ enum variant: <span style='color: green;'>```Compact = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Compact: crate::q_json_document::JsonFormat =
            crate::q_json_document::JsonFormat(1);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> class provides a way to read and write JSON documents.</p>
///
/// C++ class: <span style='color: green;'>```QJsonDocument```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qjsondocument.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> class provides a way to read and write JSON documents.</p>
/// <p><a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> is a class that wraps a complete JSON document and can read and write this document both from a UTF-8 encoded text based representation as well as Qt's own binary format.</p>
/// <p>A JSON document can be converted from its text-based representation to a <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> using <a href="http://doc.qt.io/qt-5/qjsondocument.html#fromJson">QJsonDocument::fromJson</a>(). <a href="http://doc.qt.io/qt-5/qjsondocument.html#toJson">toJson</a>() converts it back to text. The parser is very fast and efficient and converts the JSON to the binary representation used by Qt.</p>
/// <p>Validity of the parsed document can be queried with !<a href="http://doc.qt.io/qt-5/qjsondocument.html#isNull">isNull</a>()</p>
/// <p>A document can be queried as to whether it contains an array or an object using <a href="http://doc.qt.io/qt-5/qjsondocument.html#isArray">isArray</a>() and <a href="http://doc.qt.io/qt-5/qjsondocument.html#isObject">isObject</a>(). The array or object contained in the document can be retrieved using <a href="http://doc.qt.io/qt-5/qjsondocument.html#array">array</a>() or <a href="http://doc.qt.io/qt-5/qjsondocument.html#object">object</a>() and then read or manipulated.</p>
/// <p>A document can also be created from a stored binary representation using <a href="http://doc.qt.io/qt-5/qjsondocument.html#fromBinaryData">fromBinaryData</a>() or <a href="http://doc.qt.io/qt-5/qjsondocument.html#fromRawData">fromRawData</a>().</p></div>
#[repr(C)]
pub struct QJsonDocument {
    _unused: u8,
}
impl QJsonDocument {
    /// <p>Returns the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> contained in the document.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonArray QJsonDocument::array() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#array">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> contained in the document.</p>
    /// <p>Returns an empty array if the document contains an object.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsondocument.html#isArray">isArray</a>(), <a href="http://doc.qt.io/qt-5/qjsondocument.html#object">object</a>(), and <a href="http://doc.qt.io/qt-5/qjsondocument.html#setArray">setArray</a>().</p></div>
    pub unsafe fn array(&self) -> ::cpp_utils::CppBox<crate::QJsonArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QJsonDocument_array(self as *const crate::QJsonDocument);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Returns the value of the <span style='color: green;'>```BinaryFormatTag```</span> field.
    pub unsafe fn binary_format_tag() -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QJsonDocument_BinaryFormatTag()
    }

    /// <p>Assigns the <i>other</i> document to this <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a>. Returns a reference to this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonDocument& QJsonDocument::operator=(const QJsonDocument& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns the <i>other</i> document to this <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a>. Returns a reference to this object.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonDocument>>,
    ) -> ::cpp_utils::MutRef<crate::QJsonDocument> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonDocument_operator_(
            self as *mut crate::QJsonDocument,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonDocument>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> from <i>data</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QJsonDocument QJsonDocument::fromBinaryData(const QByteArray& data, QJsonDocument::DataValidation validation = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#fromBinaryData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> from <i>data</i>.</p>
    /// <p><i>validation</i> decides whether the data is checked for validity before being used. By default the data is validated. If the <i>data</i> is not valid, the method returns a null document.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsondocument.html#toBinaryData">toBinaryData</a>(), <a href="http://doc.qt.io/qt-5/qjsondocument.html#fromRawData">fromRawData</a>(), <a href="http://doc.qt.io/qt-5/qjsondocument.html#isNull">isNull</a>(), and <a href="http://doc.qt.io/qt-5/qjsondocument.html#DataValidation-enum">DataValidation</a>.</p></div>
    pub unsafe fn from_binary_data_2a(
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        validation: crate::q_json_document::DataValidation,
    ) -> ::cpp_utils::CppBox<crate::QJsonDocument> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonDocument_fromBinaryData(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(data)
                .as_raw_ptr(),
            validation,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> from <i>data</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QJsonDocument QJsonDocument::fromBinaryData(const QByteArray& data)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#fromBinaryData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> from <i>data</i>.</p>
    /// <p><i>validation</i> decides whether the data is checked for validity before being used. By default the data is validated. If the <i>data</i> is not valid, the method returns a null document.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsondocument.html#toBinaryData">toBinaryData</a>(), <a href="http://doc.qt.io/qt-5/qjsondocument.html#fromRawData">fromRawData</a>(), <a href="http://doc.qt.io/qt-5/qjsondocument.html#isNull">isNull</a>(), and <a href="http://doc.qt.io/qt-5/qjsondocument.html#DataValidation-enum">DataValidation</a>.</p></div>
    pub unsafe fn from_binary_data_1a(
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonDocument> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonDocument_fromBinaryData1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(data)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Parses <i>json</i> as a UTF-8 encoded JSON document, and creates a <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> from it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QJsonDocument QJsonDocument::fromJson(const QByteArray& json, QJsonParseError* error = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#fromJson">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Parses <i>json</i> as a UTF-8 encoded JSON document, and creates a <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> from it.</p>
    /// <p>Returns a valid (non-null) <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> if the parsing succeeds. If it fails, the returned document will be null, and the optional <i>error</i> variable will contain further details about the error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsondocument.html#toJson">toJson</a>(), <a href="http://doc.qt.io/qt-5/qjsonparseerror.html">QJsonParseError</a>, and <a href="http://doc.qt.io/qt-5/qjsondocument.html#isNull">isNull</a>().</p></div>
    pub unsafe fn from_json_2a(
        json: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        error: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QJsonParseError>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonDocument> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonDocument_fromJson(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(json)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QJsonParseError>>::cast_into(error)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Parses <i>json</i> as a UTF-8 encoded JSON document, and creates a <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> from it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QJsonDocument QJsonDocument::fromJson(const QByteArray& json)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#fromJson">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Parses <i>json</i> as a UTF-8 encoded JSON document, and creates a <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> from it.</p>
    /// <p>Returns a valid (non-null) <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> if the parsing succeeds. If it fails, the returned document will be null, and the optional <i>error</i> variable will contain further details about the error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsondocument.html#toJson">toJson</a>(), <a href="http://doc.qt.io/qt-5/qjsonparseerror.html">QJsonParseError</a>, and <a href="http://doc.qt.io/qt-5/qjsondocument.html#isNull">isNull</a>().</p></div>
    pub unsafe fn from_json_1a(
        json: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonDocument> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonDocument_fromJson1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(json)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> that uses the first <i>size</i> bytes from <i>data</i>. It assumes <i>data</i> contains a binary encoded JSON document. The created document does not take ownership of <i>data</i> and the caller has to guarantee that <i>data</i> will not be deleted or modified as long as any <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a>, <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> or <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> still references the data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QJsonDocument QJsonDocument::fromRawData(const char* data, int size, QJsonDocument::DataValidation validation = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#fromRawData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> that uses the first <i>size</i> bytes from <i>data</i>. It assumes <i>data</i> contains a binary encoded JSON document. The created document does not take ownership of <i>data</i> and the caller has to guarantee that <i>data</i> will not be deleted or modified as long as any <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a>, <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> or <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> still references the data.</p>
    /// <p><i>data</i> has to be aligned to a 4 byte boundary.</p>
    /// <p><i>validation</i> decides whether the data is checked for validity before being used. By default the data is validated. If the <i>data</i> is not valid, the method returns a null document.</p>
    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> representing the data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsondocument.html#rawData">rawData</a>(), <a href="http://doc.qt.io/qt-5/qjsondocument.html#fromBinaryData">fromBinaryData</a>(), <a href="http://doc.qt.io/qt-5/qjsondocument.html#isNull">isNull</a>(), and <a href="http://doc.qt.io/qt-5/qjsondocument.html#DataValidation-enum">DataValidation</a>.</p></div>
    pub unsafe fn from_raw_data_3a(
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        size: ::std::os::raw::c_int,
        validation: crate::q_json_document::DataValidation,
    ) -> ::cpp_utils::CppBox<crate::QJsonDocument> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonDocument_fromRawData(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(data)
                .as_raw_ptr(),
            size,
            validation,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> that uses the first <i>size</i> bytes from <i>data</i>. It assumes <i>data</i> contains a binary encoded JSON document. The created document does not take ownership of <i>data</i> and the caller has to guarantee that <i>data</i> will not be deleted or modified as long as any <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a>, <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> or <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> still references the data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QJsonDocument QJsonDocument::fromRawData(const char* data, int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#fromRawData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> that uses the first <i>size</i> bytes from <i>data</i>. It assumes <i>data</i> contains a binary encoded JSON document. The created document does not take ownership of <i>data</i> and the caller has to guarantee that <i>data</i> will not be deleted or modified as long as any <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a>, <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> or <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> still references the data.</p>
    /// <p><i>data</i> has to be aligned to a 4 byte boundary.</p>
    /// <p><i>validation</i> decides whether the data is checked for validity before being used. By default the data is validated. If the <i>data</i> is not valid, the method returns a null document.</p>
    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> representing the data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsondocument.html#rawData">rawData</a>(), <a href="http://doc.qt.io/qt-5/qjsondocument.html#fromBinaryData">fromBinaryData</a>(), <a href="http://doc.qt.io/qt-5/qjsondocument.html#isNull">isNull</a>(), and <a href="http://doc.qt.io/qt-5/qjsondocument.html#DataValidation-enum">DataValidation</a>.</p></div>
    pub unsafe fn from_raw_data_2a(
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QJsonDocument> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonDocument_fromRawData1(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(data)
                .as_raw_ptr(),
            size,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> from the <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> <i>variant</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QJsonDocument QJsonDocument::fromVariant(const QVariant& variant)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#fromVariant">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> from the <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> <i>variant</i>.</p>
    /// <p>If the <i>variant</i> contains any other type than a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantMap-typedef">QVariantMap</a>, <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantHash-typedef">QVariantHash</a>, <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a> or <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a>, the returned document is invalid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsondocument.html#toVariant">toVariant</a>().</p></div>
    pub unsafe fn from_variant(
        variant: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonDocument> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonDocument_fromVariant(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(variant)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> representing the value for the key <i>key</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonValue QJsonDocument::operator[](const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> representing the value for the key <i>key</i>.</p>
    /// <p>Equivalent to calling <a href="http://doc.qt.io/qt-5/qjsondocument.html#object">object</a>().value(key).</p>
    /// <p>The returned <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> is <a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">QJsonValue::Undefined</a> if the key does not exist, or if <a href="http://doc.qt.io/qt-5/qjsondocument.html#isObject">isObject</a>() is false.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>, <a href="http://doc.qt.io/qt-5/qjsonvalue.html#isUndefined">QJsonValue::isUndefined</a>(), and <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn index_q_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonDocument_operator__4(
            self as *const crate::QJsonDocument,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonValue QJsonDocument::operator[](QLatin1String key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn index_q_latin1_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonDocument_operator__5(
            self as *const crate::QJsonDocument,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> representing the value for index <i>i</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonValue QJsonDocument::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#operator-5b-5d-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> representing the value for index <i>i</i>.</p>
    /// <p>Equivalent to calling <a href="http://doc.qt.io/qt-5/qjsondocument.html#array">array</a>().at(i).</p>
    /// <p>The returned <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> is <a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">QJsonValue::Undefined</a>, if <i>i</i> is out of bounds, or if <a href="http://doc.qt.io/qt-5/qjsondocument.html#isArray">isArray</a>() is false.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>, <a href="http://doc.qt.io/qt-5/qjsonvalue.html#isUndefined">QJsonValue::isUndefined</a>(), and <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn index_int(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonDocument_operator__6(
            self as *const crate::QJsonDocument,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if the document contains an array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QJsonDocument::isArray() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#isArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the document contains an array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsondocument.html#array">array</a>() and <a href="http://doc.qt.io/qt-5/qjsondocument.html#isObject">isObject</a>().</p></div>
    pub unsafe fn is_array(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonDocument_isArray(self as *const crate::QJsonDocument)
    }

    /// <p>Returns <code>true</code> if the document doesn't contain any data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QJsonDocument::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the document doesn't contain any data.</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonDocument_isEmpty(self as *const crate::QJsonDocument)
    }

    /// <p>returns <code>true</code> if this document is null.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QJsonDocument::isNull() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#isNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>returns <code>true</code> if this document is null.</p>
    /// <p>Null documents are documents created through the default constructor.</p>
    /// <p>Documents created from UTF-8 encoded text or the binary format are validated during parsing. If validation fails, the returned document will also be null.</p></div>
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonDocument_isNull(self as *const crate::QJsonDocument)
    }

    /// <p>Returns <code>true</code> if the document contains an object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QJsonDocument::isObject() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#isObject">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the document contains an object.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsondocument.html#object">object</a>() and <a href="http://doc.qt.io/qt-5/qjsondocument.html#isArray">isArray</a>().</p></div>
    pub unsafe fn is_object(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonDocument_isObject(self as *const crate::QJsonDocument)
    }

    /// <p>Constructs an empty and invalid document.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonDocument::QJsonDocument()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#QJsonDocument">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty and invalid document.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QJsonDocument> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonDocument_QJsonDocument();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> from <i>object</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonDocument::QJsonDocument(const QJsonObject& object)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#QJsonDocument-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> from <i>object</i>.</p></div>
    pub unsafe fn from_q_json_object(
        object: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonObject>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonDocument> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonDocument_QJsonDocument1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonObject>>::cast_into(object)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> from <i>array</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonDocument::QJsonDocument(const QJsonArray& array)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#QJsonDocument-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> from <i>array</i>.</p></div>
    pub unsafe fn from_q_json_array(
        array: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonArray>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonDocument> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonDocument_QJsonDocument2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonArray>>::cast_into(array)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a copy of the <i>other</i> document.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonDocument::QJsonDocument(const QJsonDocument& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#QJsonDocument-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a copy of the <i>other</i> document.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonDocument>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonDocument> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonDocument_QJsonDocument3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonDocument>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> contained in the document.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonObject QJsonDocument::object() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#object">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> contained in the document.</p>
    /// <p>Returns an empty object if the document contains an array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsondocument.html#isObject">isObject</a>(), <a href="http://doc.qt.io/qt-5/qjsondocument.html#array">array</a>(), and <a href="http://doc.qt.io/qt-5/qjsondocument.html#setObject">setObject</a>().</p></div>
    pub unsafe fn object(&self) -> ::cpp_utils::CppBox<crate::QJsonObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QJsonDocument_object(self as *const crate::QJsonDocument);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the raw binary representation of the data <i>size</i> will contain the size of the returned data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QJsonDocument::rawData(int* size) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#rawData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the raw binary representation of the data <i>size</i> will contain the size of the returned data.</p>
    /// <p>This method is useful to e.g. stream the JSON document in it's binary form to a file.</p></div>
    pub unsafe fn raw_data(
        &self,
        size: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonDocument_rawData(
            self as *const crate::QJsonDocument,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_int>>::cast_into(size)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Sets <i>array</i> as the main object of this document.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QJsonDocument::setArray(const QJsonArray& array)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#setArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets <i>array</i> as the main object of this document.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsondocument.html#setObject">setObject</a>() and <a href="http://doc.qt.io/qt-5/qjsondocument.html#array">array</a>().</p></div>
    pub unsafe fn set_array(
        &mut self,
        array: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonArray>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QJsonDocument_setArray(
            self as *mut crate::QJsonDocument,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonArray>>::cast_into(array)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets <i>object</i> as the main object of this document.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QJsonDocument::setObject(const QJsonObject& object)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#setObject">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets <i>object</i> as the main object of this document.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsondocument.html#setArray">setArray</a>() and <a href="http://doc.qt.io/qt-5/qjsondocument.html#object">object</a>().</p></div>
    pub unsafe fn set_object(
        &mut self,
        object: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonObject>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QJsonDocument_setObject(
            self as *mut crate::QJsonDocument,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonObject>>::cast_into(object)
                .as_raw_ptr(),
        )
    }

    /// <p>Swaps the document <i>other</i> with this. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QJsonDocument::swap(QJsonDocument& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps the document <i>other</i> with this. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QJsonDocument>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QJsonDocument_swap(
            self as *mut crate::QJsonDocument,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QJsonDocument>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns a binary representation of the document.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QJsonDocument::toBinaryData() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#toBinaryData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a binary representation of the document.</p>
    /// <p>The binary representation is also the native format used internally in Qt, and is very efficient and fast to convert to and from.</p>
    /// <p>The binary format can be stored on disk and interchanged with other applications or computers. <a href="http://doc.qt.io/qt-5/qjsondocument.html#fromBinaryData">fromBinaryData</a>() can be used to convert it back into a JSON document.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsondocument.html#fromBinaryData">fromBinaryData</a>().</p></div>
    pub unsafe fn to_binary_data(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonDocument_toBinaryData(
            self as *const crate::QJsonDocument,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts the <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> to a UTF-8 encoded JSON document in the provided <i>format</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QJsonDocument::toJson() const```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsondocument.html#toJson">C++ documentation</a> for <span style='color: green;'>```QByteArray QJsonDocument::toJson(JsonFormat format = Indented) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> to a UTF-8 encoded JSON document in the provided <i>format</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsondocument.html#fromJson">fromJson</a>() and <a href="http://doc.qt.io/qt-5/qjsondocument.html#JsonFormat-enum">JsonFormat</a>.</p></div>
    pub unsafe fn to_json_0a(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QJsonDocument_toJson(self as *const crate::QJsonDocument);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts the <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> to a UTF-8 encoded JSON document in the provided <i>format</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QJsonDocument::toJson(QJsonDocument::JsonFormat format) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#toJson">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a> to a UTF-8 encoded JSON document in the provided <i>format</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsondocument.html#fromJson">fromJson</a>() and <a href="http://doc.qt.io/qt-5/qjsondocument.html#JsonFormat-enum">JsonFormat</a>.</p></div>
    pub unsafe fn to_json_1a(
        &self,
        format: crate::q_json_document::JsonFormat,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonDocument_toJson1(
            self as *const crate::QJsonDocument,
            format,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> representing the Json document.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QJsonDocument::toVariant() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#toVariant">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> representing the Json document.</p>
    /// <p>The returned variant will be a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a> if the document is a <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantMap-typedef">QVariantMap</a> if the document is a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsondocument.html#fromVariant">fromVariant</a>() and <a href="http://doc.qt.io/qt-5/qjsonvalue.html#toVariant">QJsonValue::toVariant</a>().</p></div>
    pub unsafe fn to_variant(&self) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonDocument_toVariant(
            self as *const crate::QJsonDocument,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_json_object {
    //! C++ type: <span style='color: green;'>```QJsonObject```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html">QJsonObject::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QJsonObject::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html">QJsonObject::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html">QJsonObject::iterator</a> allows you to iterate over a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> and to modify the value (but not the key) stored under a particular key. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>, you should use <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a>. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> through the iterator. Const iterators are slightly faster, and improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html">QJsonObject::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> function like <a href="http://doc.qt.io/qt-5/qjsonobject.html#begin">QJsonObject::begin</a>(), <a href="http://doc.qt.io/qt-5/qjsonobject.html#end">QJsonObject::end</a>(), or <a href="http://doc.qt.io/qt-5/qjsonobject.html#find">QJsonObject::find</a>() before you can start iterating.</p>
    /// <p>Multiple iterators can be used on the same object. Existing iterators will however become dangling once the object gets modified.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html">QJsonObject::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonObject::iterator& QJsonObject::iterator::operator=(const QJsonObject::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html">QJsonObject::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html">QJsonObject::iterator</a> allows you to iterate over a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> and to modify the value (but not the key) stored under a particular key. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>, you should use <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a>. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> through the iterator. Const iterators are slightly faster, and improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html">QJsonObject::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> function like <a href="http://doc.qt.io/qt-5/qjsonobject.html#begin">QJsonObject::begin</a>(), <a href="http://doc.qt.io/qt-5/qjsonobject.html#end">QJsonObject::end</a>(), or <a href="http://doc.qt.io/qt-5/qjsonobject.html#find">QJsonObject::find</a>() before you can start iterating.</p>
        /// <p>Multiple iterators can be used on the same object. Existing iterators will however become dangling once the object gets modified.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_json_object::Iterator>>,
        ) -> ::cpp_utils::MutRef<crate::q_json_object::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_iterator_operator_3(self as *mut crate::q_json_object::Iterator, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_json_object::Iterator>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonObject::iterator QJsonObject::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_json_object::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_iterator_operator__5(
                self as *mut crate::q_json_object::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonObject::iterator QJsonObject::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_json_object::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_iterator_operator__3(
                self as *mut crate::q_json_object::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the current item's key as a const reference.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QString QJsonObject::iterator::key() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current item's key as a const reference.</p>
        /// <p>There is no direct way of changing an item's key through an iterator, although it can be done by calling <a href="http://doc.qt.io/qt-5/qhash.html#erase">QHash::erase</a>() followed by <a href="http://doc.qt.io/qt-5/qhash.html#insert">QHash::insert</a>() or <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">QHash::insertMulti</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn key(&self) -> ::cpp_utils::CppBox<crate::QString> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_iterator_key(
                self as *const crate::q_json_object::Iterator,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonObject::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like <a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html#key">key</a>(), <a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html#value">value</a>(), and operator++() must not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonobject.html#begin">QJsonObject::begin</a>() and <a href="http://doc.qt.io/qt-5/qjsonobject.html#end">QJsonObject::end</a>().</p></div>
        pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::q_json_object::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonObject::iterator::iterator(QJsonObject* obj, int index)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html#iterator">C++ documentation</a> for <span style='color: green;'>```iterator::iterator()```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like <a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html#key">key</a>(), <a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html#value">value</a>(), and operator++() must not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonobject.html#begin">QJsonObject::begin</a>() and <a href="http://doc.qt.io/qt-5/qjsonobject.html#end">QJsonObject::end</a>().</p></div>
        pub unsafe fn new_2a(
            obj: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QJsonObject>>,
            index: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_json_object::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_iterator_iterator1(
                ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QJsonObject>>::cast_into(obj)
                    .as_mut_raw_ptr(),
                index,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>The <a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html">QJsonObject::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonObject::iterator::iterator(const QJsonObject::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html">QJsonObject::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html">QJsonObject::iterator</a> allows you to iterate over a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> and to modify the value (but not the key) stored under a particular key. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>, you should use <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a>. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> through the iterator. Const iterators are slightly faster, and improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html">QJsonObject::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> function like <a href="http://doc.qt.io/qt-5/qjsonobject.html#begin">QJsonObject::begin</a>(), <a href="http://doc.qt.io/qt-5/qjsonobject.html#end">QJsonObject::end</a>(), or <a href="http://doc.qt.io/qt-5/qjsonobject.html#find">QJsonObject::find</a>() before you can start iterating.</p>
        /// <p>Multiple iterators can be used on the same object. Existing iterators will however become dangling once the object gets modified.</p></div>
        pub unsafe fn new_copy(
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_json_object::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_json_object::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_iterator_iterator2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_json_object::Iterator>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonValueRef QJsonObject::iterator::value() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>You can change the value of an item by using value() on the left side of an assignment, for example:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="keyword">if</span> (i<span class="operator">.</span>key() <span class="operator">=</span><span class="operator">=</span> <span class="string">"Hello"</span>)
        /// &#32;     i<span class="operator">.</span>value() <span class="operator">=</span> <span class="string">"Bonjour"</span>;
        ///
        /// </pre>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">operator*</a>().</p></div>
        pub unsafe fn value(&self) -> ::cpp_utils::CppBox<crate::QJsonValueRef> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_iterator_value(
                self as *const crate::q_json_object::Iterator,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QJsonObject::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a> allows you to iterate over a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. If you want to modify the <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> as you iterate over it, you must use <a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html">QJsonObject::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> through the iterator. Const iterators are slightly faster and improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> function like <a href="http://doc.qt.io/qt-5/qjsonobject.html#constBegin">QJsonObject::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qjsonobject.html#constEnd">QJsonObject::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qjsonobject.html#find">QJsonObject::find</a>() before you can start iterating.</p>
    /// <p>Multiple iterators can be used on the same object. Existing iterators will however become dangling if the object gets modified.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonObject::const_iterator& QJsonObject::const_iterator::operator=(const QJsonObject::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a> allows you to iterate over a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. If you want to modify the <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> as you iterate over it, you must use <a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html">QJsonObject::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> through the iterator. Const iterators are slightly faster and improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> function like <a href="http://doc.qt.io/qt-5/qjsonobject.html#constBegin">QJsonObject::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qjsonobject.html#constEnd">QJsonObject::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qjsonobject.html#find">QJsonObject::find</a>() before you can start iterating.</p>
        /// <p>Multiple iterators can be used on the same object. Existing iterators will however become dangling if the object gets modified.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_json_object::ConstIterator>>,
        ) -> ::cpp_utils::MutRef<crate::q_json_object::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_const_iterator_operator_3(self as *mut crate::q_json_object::ConstIterator, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_json_object::ConstIterator>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonObject::const_iterator QJsonObject::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_json_object::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_const_iterator_operator__5(
                self as *mut crate::q_json_object::ConstIterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonObject::const_iterator QJsonObject::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_json_object::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_const_iterator_operator__3(
                self as *mut crate::q_json_object::ConstIterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the current key, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QString QJsonObject::const_iterator::key() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#key">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current key, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        pub unsafe fn key(&self) -> ::cpp_utils::CppBox<crate::QString> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_const_iterator_key(
                self as *const crate::q_json_object::ConstIterator,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonObject::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html#key">key</a>(), <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html#value">value</a>(), and operator++() must not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonobject.html#constBegin">QJsonObject::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qjsonobject.html#constEnd">QJsonObject::constEnd</a>().</p></div>
        pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::q_json_object::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QJsonObject_const_iterator_const_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonObject::const_iterator::const_iterator(const QJsonObject* obj, int index)```</span>.
        pub unsafe fn new_2a(
            obj: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QJsonObject>>,
            index: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_json_object::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QJsonObject_const_iterator_const_iterator1(
                    ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QJsonObject>>::cast_into(obj)
                        .as_raw_ptr(),
                    index,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs a copy of <i>other</i>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonObject::const_iterator::const_iterator(const QJsonObject::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html#const_iterator-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p></div>
        pub unsafe fn new_1a(
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_json_object::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_json_object::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_const_iterator_const_iterator2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_json_object::Iterator>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>The <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonObject::const_iterator::const_iterator(const QJsonObject::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a> allows you to iterate over a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. If you want to modify the <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> as you iterate over it, you must use <a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html">QJsonObject::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> through the iterator. Const iterators are slightly faster and improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> function like <a href="http://doc.qt.io/qt-5/qjsonobject.html#constBegin">QJsonObject::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qjsonobject.html#constEnd">QJsonObject::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qjsonobject.html#find">QJsonObject::find</a>() before you can start iterating.</p>
        /// <p>Multiple iterators can be used on the same object. Existing iterators will however become dangling if the object gets modified.</p></div>
        pub unsafe fn new_copy(
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_json_object::ConstIterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_json_object::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_const_iterator_const_iterator3(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_json_object::ConstIterator>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonValue QJsonObject::const_iterator::value() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        pub unsafe fn value(&self) -> ::cpp_utils::CppBox<crate::QJsonValue> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_const_iterator_value(
                self as *const crate::q_json_object::ConstIterator,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_json_object::Iterator {
        type Output = ::cpp_utils::CppBox<crate::QJsonValueRef>;
        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonValueRef QJsonObject::iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>().</p></div>
        fn indirection(self) -> ::cpp_utils::CppBox<crate::QJsonValueRef> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonObject_iterator_operator_(
                    self as *const crate::q_json_object::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_json_object::Iterator>>
        for crate::q_json_object::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QJsonObject::iterator::operator==(const QJsonObject::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, other: &::cpp_utils::Ref<crate::q_json_object::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonObject_iterator_operator__(
                    self as *const crate::q_json_object::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_json_object::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_json_object::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonObject::iterator& QJsonObject::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_json_object::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonObject_iterator_operator__2(
                    self as *mut crate::q_json_object::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_json_object::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_json_object::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonObject::iterator& QJsonObject::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_json_object::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonObject_iterator_operator__4(
                    self as *mut crate::q_json_object::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_int> for &crate::q_json_object::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_json_object::Iterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions forward from this iterator. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonObject::iterator QJsonObject::iterator::operator+(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions forward from this iterator. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p>This operation can be slow for large <i>j</i> values.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn add(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_json_object::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonObject_iterator_operator_1(
                    self as *const crate::q_json_object::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_int> for &crate::q_json_object::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_json_object::Iterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions backward from this iterator. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonObject::iterator QJsonObject::iterator::operator-(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions backward from this iterator. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p>This operation can be slow for large <i>j</i> values.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn sub(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_json_object::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonObject_iterator_operator_2(
                    self as *const crate::q_json_object::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_int> for crate::q_json_object::Iterator {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonObject::iterator& QJsonObject::iterator::operator+=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonObject_iterator_operator__6(
                    self as *mut crate::q_json_object::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_int> for crate::q_json_object::Iterator {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonObject::iterator& QJsonObject::iterator::operator-=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonObject_iterator_operator__7(
                    self as *mut crate::q_json_object::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_json_object::ConstIterator>>
        for crate::q_json_object::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QJsonObject::iterator::operator==(const QJsonObject::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, other: &::cpp_utils::Ref<crate::q_json_object::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonObject_iterator_operator__8(
                    self as *const crate::q_json_object::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_json_object::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::QJsonValue>;
        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonValue QJsonObject::const_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        fn indirection(self) -> ::cpp_utils::CppBox<crate::QJsonValue> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonObject_const_iterator_operator_(
                    self as *const crate::q_json_object::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_json_object::ConstIterator>>
        for crate::q_json_object::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QJsonObject::const_iterator::operator==(const QJsonObject::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, other: &::cpp_utils::Ref<crate::q_json_object::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonObject_const_iterator_operator__(
                    self as *const crate::q_json_object::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_json_object::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_json_object::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonObject::const_iterator& QJsonObject::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_json_object::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonObject_const_iterator_operator__2(
                    self as *mut crate::q_json_object::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_json_object::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_json_object::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonObject::const_iterator& QJsonObject::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_json_object::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonObject_const_iterator_operator__4(
                    self as *mut crate::q_json_object::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_int> for &crate::q_json_object::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_json_object::ConstIterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions forward from this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonObject::const_iterator QJsonObject::const_iterator::operator+(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions forward from this iterator.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>().</p></div>
        fn add(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_json_object::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonObject_const_iterator_operator_1(
                    self as *const crate::q_json_object::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_int> for &crate::q_json_object::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_json_object::ConstIterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions backward from this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonObject::const_iterator QJsonObject::const_iterator::operator-(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions backward from this iterator.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>().</p></div>
        fn sub(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_json_object::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonObject_const_iterator_operator_2(
                    self as *const crate::q_json_object::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_int> for crate::q_json_object::ConstIterator {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonObject::const_iterator& QJsonObject::const_iterator::operator+=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonObject_const_iterator_operator__6(
                    self as *mut crate::q_json_object::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_int> for crate::q_json_object::ConstIterator {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QJsonObject::const_iterator& QJsonObject::const_iterator::operator-=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonObject_const_iterator_operator__7(
                    self as *mut crate::q_json_object::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_json_object::Iterator>>
        for crate::q_json_object::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QJsonObject::const_iterator::operator==(const QJsonObject::iterator& other) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool const_iterator::operator==(const const_iterator &other) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, other: &::cpp_utils::Ref<crate::q_json_object::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QJsonObject_const_iterator_operator__8(
                    self as *const crate::q_json_object::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_json_object::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html">QJsonObject::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QJsonObject::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html">QJsonObject::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html">QJsonObject::iterator</a> allows you to iterate over a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> and to modify the value (but not the key) stored under a particular key. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>, you should use <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a>. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> through the iterator. Const iterators are slightly faster, and improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html">QJsonObject::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> function like <a href="http://doc.qt.io/qt-5/qjsonobject.html#begin">QJsonObject::begin</a>(), <a href="http://doc.qt.io/qt-5/qjsonobject.html#end">QJsonObject::end</a>(), or <a href="http://doc.qt.io/qt-5/qjsonobject.html#find">QJsonObject::find</a>() before you can start iterating.</p>
        /// <p>Multiple iterators can be used on the same object. Existing iterators will however become dangling once the object gets modified.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QJsonObject_iterator_diterator(
                self as *mut crate::q_json_object::Iterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_json_object::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QJsonObject::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a> allows you to iterate over a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. If you want to modify the <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> as you iterate over it, you must use <a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html">QJsonObject::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> through the iterator. Const iterators are slightly faster and improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qjsonobject-const-iterator.html">QJsonObject::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> function like <a href="http://doc.qt.io/qt-5/qjsonobject.html#constBegin">QJsonObject::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qjsonobject.html#constEnd">QJsonObject::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qjsonobject.html#find">QJsonObject::find</a>() before you can start iterating.</p>
        /// <p>Multiple iterators can be used on the same object. Existing iterators will however become dangling if the object gets modified.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QJsonObject_const_iterator_dconst_iterator(
                self as *mut crate::q_json_object::ConstIterator,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> class encapsulates a JSON object.</p>
///
/// C++ class: <span style='color: green;'>```QJsonObject```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qjsonobject.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> class encapsulates a JSON object.</p>
/// <p>A JSON object is a list of key value pairs, where the keys are unique strings and the values are represented by a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> can be converted to and from a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantMap-typedef">QVariantMap</a>. You can query the number of (key, value) pairs with <a href="http://doc.qt.io/qt-5/qjsonobject.html#size">size</a>(), <a href="http://doc.qt.io/qt-5/qjsonobject.html#insert">insert</a>(), and <a href="http://doc.qt.io/qt-5/qjsonobject.html#remove">remove</a>() entries from it and iterate over its content using the standard C++ iterator pattern.</p>
/// <p><a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> is an implicitly shared class, and shares the data with the document it has been created from as long as it is not being modified.</p>
/// <p>You can convert the object to and from text based JSON through <a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a>.</p></div>
#[repr(C)]
pub struct QJsonObject {
    _unused: u8,
}
impl QJsonObject {
    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonObject::const_iterator QJsonObject::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the object.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonobject.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qjsonobject.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(&self) -> ::cpp_utils::CppBox<crate::q_json_object::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QJsonObject_constBegin(self as *const crate::QJsonObject);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonObject::const_iterator QJsonObject::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the object.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonobject.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qjsonobject.html#end">end</a>().</p></div>
    pub unsafe fn const_end(&self) -> ::cpp_utils::CppBox<crate::q_json_object::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QJsonObject_constEnd(self as *const crate::QJsonObject);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const iterator pointing to the item with key <i>key</i> in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonObject::const_iterator QJsonObject::constFind(const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#constFind">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const iterator pointing to the item with key <i>key</i> in the map.</p>
    /// <p>If the map contains no item with key <i>key</i>, the function returns <a href="http://doc.qt.io/qt-5/qjsonobject.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_find_q_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::q_json_object::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_constFind(
            self as *const crate::QJsonObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonObject::const_iterator QJsonObject::constFind(QLatin1String key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#constFind-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 5.7.</p></div>
    pub unsafe fn const_find_q_latin1_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::q_json_object::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_constFind1(
            self as *const crate::QJsonObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if the object contains key <i>key</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QJsonObject::contains(const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the object contains key <i>key</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonobject.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qjsonobject.html#remove">remove</a>(), and <a href="http://doc.qt.io/qt-5/qjsonobject.html#take">take</a>().</p></div>
    pub unsafe fn contains_q_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonObject_contains(
            self as *const crate::QJsonObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QJsonObject::contains(QLatin1String key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#contains-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 5.7.</p></div>
    pub unsafe fn contains_q_latin1_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonObject_contains1(
            self as *const crate::QJsonObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(key)
                .as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonObject& QJsonObject::operator=(const QJsonObject& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this object.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonObject>>,
    ) -> ::cpp_utils::MutRef<crate::QJsonObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_operator_(
            self as *mut crate::QJsonObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonObject>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QJsonObject::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qjsonobject.html#size">size</a>().</p></div>
    pub unsafe fn count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QJsonObject_count(self as *const crate::QJsonObject)
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qjsonobject.html#isEmpty">isEmpty</a>(), returning <code>true</code> if the object is empty; otherwise returning <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QJsonObject::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qjsonobject.html#isEmpty">isEmpty</a>(), returning <code>true</code> if the object is empty; otherwise returning <code>false</code>.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonObject_empty(self as *const crate::QJsonObject)
    }

    /// <p>Removes the (key, value) pair pointed to by the iterator <i>it</i> from the map, and returns an iterator to the next item in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonObject::iterator QJsonObject::erase(QJsonObject::iterator it)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the (key, value) pair pointed to by the iterator <i>it</i> from the map, and returns an iterator to the next item in the map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonobject.html#remove">remove</a>().</p></div>
    pub unsafe fn erase(
        &mut self,
        it: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_json_object::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_json_object::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_erase(
            self as *mut crate::QJsonObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_json_object::Iterator>>::cast_into(
                it,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns an iterator pointing to the item with key <i>key</i> in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonObject::iterator QJsonObject::find(const QString& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#find">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator pointing to the item with key <i>key</i> in the map.</p>
    /// <p>If the map contains no item with key <i>key</i>, the function returns <a href="http://doc.qt.io/qt-5/qjsonobject.html#end">end</a>().</p></div>
    pub unsafe fn find_q_string_mut(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::q_json_object::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_find(
            self as *mut crate::QJsonObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonObject::iterator QJsonObject::find(QLatin1String key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#find-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 5.7.</p></div>
    pub unsafe fn find_q_latin1_string_mut(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::q_json_object::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_find1(
            self as *mut crate::QJsonObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonObject::const_iterator QJsonObject::find(const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#find-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn find_q_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::q_json_object::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_find2(
            self as *const crate::QJsonObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonObject::const_iterator QJsonObject::find(QLatin1String key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#find-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 5.7.</p></div>
    pub unsafe fn find_q_latin1_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::q_json_object::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_find3(
            self as *const crate::QJsonObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts the variant hash <i>hash</i> to a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QJsonObject QJsonObject::fromVariantHash(const QHash<QString, QVariant>& map)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#fromVariantHash">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the variant hash <i>hash</i> to a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p>
    /// <p>The keys in <i>hash</i> will be used as the keys in the JSON object, and the <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> values will be converted to JSON values.</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonobject.html#fromVariantMap">fromVariantMap</a>(), <a href="http://doc.qt.io/qt-5/qjsonobject.html#toVariantHash">toVariantHash</a>(), and <a href="http://doc.qt.io/qt-5/qjsonvalue.html#fromVariant">QJsonValue::fromVariant</a>().</p></div>
    pub unsafe fn from_variant_hash(
        map: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QHashOfQStringQVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_fromVariantHash(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QHashOfQStringQVariant>>::cast_into(
                map,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts the variant map <i>map</i> to a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QJsonObject QJsonObject::fromVariantMap(const QMap<QString, QVariant>& map)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#fromVariantMap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the variant map <i>map</i> to a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p>
    /// <p>The keys in <i>map</i> will be used as the keys in the JSON object, and the <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> values will be converted to JSON values.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonobject.html#fromVariantHash">fromVariantHash</a>(), <a href="http://doc.qt.io/qt-5/qjsonobject.html#toVariantMap">toVariantMap</a>(), and <a href="http://doc.qt.io/qt-5/qjsonvalue.html#fromVariant">QJsonValue::fromVariant</a>().</p></div>
    pub unsafe fn from_variant_map(
        map: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMapOfQStringQVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_fromVariantMap(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMapOfQStringQVariant>>::cast_into(map)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> representing the value for the key <i>key</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonValue QJsonObject::operator[](const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> representing the value for the key <i>key</i>.</p>
    /// <p>This does the same as <a href="http://doc.qt.io/qt-5/qjsonobject.html#value">value</a>().</p>
    /// <p>The returned <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> is <a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">QJsonValue::Undefined</a> if the key does not exist.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonobject.html#value">value</a>(), <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>, and <a href="http://doc.qt.io/qt-5/qjsonvalue.html#isUndefined">QJsonValue::isUndefined</a>().</p></div>
    pub unsafe fn index_q_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_operator__(
            self as *const crate::QJsonObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonValue QJsonObject::operator[](QLatin1String key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 5.7.</p></div>
    pub unsafe fn index_q_latin1_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_operator__1(
            self as *const crate::QJsonObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the value for <i>key</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonValueRef QJsonObject::operator[](const QString& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#operator-5b-5d-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the value for <i>key</i>.</p>
    /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>. If you assign to it, the assignment will apply to the element in the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> or <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> from which you got the reference.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonobject.html#value">value</a>().</p></div>
    pub unsafe fn index_q_string_mut(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonValueRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_operator__2(
            self as *mut crate::QJsonObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonValueRef QJsonObject::operator[](QLatin1String key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#operator-5b-5d-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 5.7.</p></div>
    pub unsafe fn index_q_latin1_string_mut(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonValueRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_operator__3(
            self as *mut crate::QJsonObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Inserts a new item with the key <i>key</i> and a value of <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonObject::iterator QJsonObject::insert(const QString& key, const QJsonValue& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts a new item with the key <i>key</i> and a value of <i>value</i>.</p>
    /// <p>If there is already an item with the key <i>key</i>, then that item's value is replaced with <i>value</i>.</p>
    /// <p>Returns an iterator pointing to the inserted item.</p>
    /// <p>If the value is <a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">QJsonValue::Undefined</a>, it will cause the key to get removed from the object. The returned iterator will then point to <a href="http://doc.qt.io/qt-5/qjsonobject.html#end">end</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonobject.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qjsonobject.html#take">take</a>(), <a href="http://doc.qt.io/qt-5/qjsonobject-iterator.html">QJsonObject::iterator</a>, and <a href="http://doc.qt.io/qt-5/qjsonobject.html#end">end</a>().</p></div>
    pub unsafe fn insert(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonValue>>,
    ) -> ::cpp_utils::CppBox<crate::q_json_object::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_insert(
            self as *mut crate::QJsonObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonValue>>::cast_into(value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if the object is empty. This is the same as <a href="http://doc.qt.io/qt-5/qjsonobject.html#size">size</a>() == 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QJsonObject::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the object is empty. This is the same as <a href="http://doc.qt.io/qt-5/qjsonobject.html#size">size</a>() == 0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonobject.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QJsonObject_isEmpty(self as *const crate::QJsonObject)
    }

    /// <p>Returns a list of all keys in this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QJsonObject::keys() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#keys">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of all keys in this object.</p>
    /// <p>The list is sorted lexographically.</p></div>
    pub unsafe fn keys(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QJsonObject_keys(self as *const crate::QJsonObject);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QJsonObject::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qjsonobject.html#size">size</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QJsonObject_length(self as *const crate::QJsonObject)
    }

    /// <p>Constructs an empty JSON object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonObject::QJsonObject()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#QJsonObject">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty JSON object.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonobject.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QJsonObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_QJsonObject();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QJsonObject::QJsonObject(const QJsonObject& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#QJsonObject-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a copy of <i>other</i>.</p>
    /// <p>Since <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> is implicitly shared, the copy is shallow as long as the object does not get modified.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonObject>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_QJsonObject2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonObject>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes <i>key</i> from the object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QJsonObject::remove(const QString& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#remove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes <i>key</i> from the object.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonobject.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qjsonobject.html#take">take</a>().</p></div>
    pub unsafe fn remove(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QJsonObject_remove(
            self as *mut crate::QJsonObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        )
    }

    /// <p>Returns the number of (key, value) pairs stored in the object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QJsonObject::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of (key, value) pairs stored in the object.</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QJsonObject_size(self as *const crate::QJsonObject)
    }

    /// <p>Swaps the object <i>other</i> with this. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QJsonObject::swap(QJsonObject& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps the object <i>other</i> with this. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QJsonObject>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QJsonObject_swap(
            self as *mut crate::QJsonObject,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QJsonObject>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Removes <i>key</i> from the object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonValue QJsonObject::take(const QString& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#take">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes <i>key</i> from the object.</p>
    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> containing the value referenced by <i>key</i>. If <i>key</i> was not contained in the object, the returned <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> is <a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">QJsonValue::Undefined</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonobject.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qjsonobject.html#remove">remove</a>(), and <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>.</p></div>
    pub unsafe fn take(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_take(
            self as *mut crate::QJsonObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts this object to a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantHash-typedef">QVariantHash</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant> QJsonObject::toVariantHash() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#toVariantHash">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts this object to a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantHash-typedef">QVariantHash</a>.</p>
    /// <p>Returns the created hash.</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonobject.html#toVariantMap">toVariantMap</a>().</p></div>
    pub unsafe fn to_variant_hash(&self) -> ::cpp_utils::CppBox<crate::QHashOfQStringQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_toVariantHash(
            self as *const crate::QJsonObject,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts this object to a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantMap-typedef">QVariantMap</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant> QJsonObject::toVariantMap() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#toVariantMap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts this object to a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantMap-typedef">QVariantMap</a>.</p>
    /// <p>Returns the created map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonobject.html#toVariantHash">toVariantHash</a>().</p></div>
    pub unsafe fn to_variant_map(&self) -> ::cpp_utils::CppBox<crate::QMapOfQStringQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_toVariantMap(
            self as *const crate::QJsonObject,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> representing the value for the key <i>key</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonValue QJsonObject::value(const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> representing the value for the key <i>key</i>.</p>
    /// <p>The returned <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> is <a href="http://doc.qt.io/qt-5/qjsonvalue.html#Type-enum">QJsonValue::Undefined</a> if the key does not exist.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> and <a href="http://doc.qt.io/qt-5/qjsonvalue.html#isUndefined">QJsonValue::isUndefined</a>().</p></div>
    pub unsafe fn value_q_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_value(
            self as *const crate::QJsonObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonValue QJsonObject::value(QLatin1String key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 5.7.</p></div>
    pub unsafe fn value_q_latin1_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QJsonObject_value1(
            self as *const crate::QJsonObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_library {
    //! C++ type: <span style='color: green;'>```QLibrary```</span>

    /// <p>This enum describes the possible hints that can be used to change the way libraries are handled when they are loaded. These values indicate how symbols are resolved when libraries are loaded, and are specified using the <a href="http://doc.qt.io/qt-5/qlibrary.html#loadHints-prop">setLoadHints</a>() function.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QLibrary::LoadHint```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibrary.html#LoadHint-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the possible hints that can be used to change the way libraries are handled when they are loaded. These values indicate how symbols are resolved when libraries are loaded, and are specified using the <a href="http://doc.qt.io/qt-5/qlibrary.html#loadHints-prop">setLoadHints</a>() function.</p>
    ///
    /// <p>The LoadHints type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;LoadHint&gt;. It stores an OR combination of LoadHint values.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlibrary.html#loadHints-prop">loadHints</a>.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct LoadHint(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for LoadHint {
        fn from(value: ::std::os::raw::c_int) -> Self {
            LoadHint(value)
        }
    }

    impl From<LoadHint> for ::std::os::raw::c_int {
        fn from(value: LoadHint) -> Self {
            value.0
        }
    }

    impl LoadHint {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl LoadHint {
        /// Causes all symbols in a library to be resolved when it is loaded, not simply when <a href="http://doc.qt.io/qt-5/qlibrary.html#resolve">resolve</a>() is called. (C++ enum variant: <span style='color: green;'>```ResolveAllSymbolsHint = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const ResolveAllSymbolsHint: crate::q_library::LoadHint = crate::q_library::LoadHint(1);
        /// Exports unresolved and external symbols in the library so that they can be resolved in other dynamically-loaded libraries loaded later. (C++ enum variant: <span style='color: green;'>```ExportExternalSymbolsHint = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const ExportExternalSymbolsHint: crate::q_library::LoadHint =
            crate::q_library::LoadHint(2);
        /// Allows the file name of the library to specify a particular object file within an archive file. If this hint is given, the filename of the library consists of a path, which is a reference to an archive file, followed by a reference to the archive member. (C++ enum variant: <span style='color: green;'>```LoadArchiveMemberHint = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const LoadArchiveMemberHint: crate::q_library::LoadHint = crate::q_library::LoadHint(4);
        /// Prevents the library from being unloaded from the address space if close() is called. The library's static variables are not reinitialized if open() is called at a later time. (C++ enum variant: <span style='color: green;'>```PreventUnloadHint = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const PreventUnloadHint: crate::q_library::LoadHint = crate::q_library::LoadHint(8);
        /// Instructs the linker to prefer definitions in the loaded library over exported definitions in the loading application when resolving external symbols in the loaded library. This option is only supported on Linux. (C++ enum variant: <span style='color: green;'>```DeepBindHint = 16```</span>)
        #[allow(non_upper_case_globals)]
        pub const DeepBindHint: crate::q_library::LoadHint = crate::q_library::LoadHint(16);
    }

    impl From<crate::q_library::LoadHint> for crate::QFlags<crate::q_library::LoadHint> {
        fn from(value: crate::q_library::LoadHint) -> Self {
            Self::from(value.to_int())
        }
    }

    impl<T: Into<crate::QFlags<crate::q_library::LoadHint>>> std::ops::BitOr<T>
        for crate::q_library::LoadHint
    {
        type Output = crate::QFlags<crate::q_library::LoadHint>;
        fn bitor(self, rhs: T) -> crate::QFlags<crate::q_library::LoadHint> {
            Into::<crate::QFlags<crate::q_library::LoadHint>>::into(self) | rhs
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> class loads shared libraries at runtime.</p>
///
/// C++ class: <span style='color: green;'>```QLibrary```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlibrary.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> class loads shared libraries at runtime.</p>
/// <p>An instance of a <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> object operates on a single shared object file (which we call a "library", but is also known as a "DLL"). A <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> provides access to the functionality in the library in a platform independent way. You can either pass a file name in the constructor, or set it explicitly with <a href="http://doc.qt.io/qt-5/qlibrary.html#fileName-prop">setFileName</a>(). When loading the library, <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> searches in all the system-specific library locations (e.g. <code>LD_LIBRARY_PATH</code> on Unix), unless the file name has an absolute path.</p>
/// <p>If the file name is an absolute path then an attempt is made to load this path first. If the file cannot be found, <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> tries the name with different platform-specific file prefixes, like "lib" on Unix and Mac, and suffixes, like ".so" on Unix, ".dylib" on the Mac, or ".dll" on Windows.</p>
/// <p>If the file path is not absolute then <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> modifies the search order to try the system-specific prefixes and suffixes first, followed by the file path specified.</p>
/// <p>This makes it possible to specify shared libraries that are only identified by their basename (i.e. without their suffix), so the same code will work on different operating systems yet still minimise the number of attempts to find the library.</p>
/// <p>The most important functions are <a href="http://doc.qt.io/qt-5/qlibrary.html#load">load</a>() to dynamically load the library file, <a href="http://doc.qt.io/qt-5/qlibrary.html#isLoaded">isLoaded</a>() to check whether loading was successful, and <a href="http://doc.qt.io/qt-5/qlibrary.html#resolve">resolve</a>() to resolve a symbol in the library. The <a href="http://doc.qt.io/qt-5/qlibrary.html#resolve">resolve</a>() function implicitly tries to load the library if it has not been loaded yet. Multiple instances of <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> can be used to access the same physical library. Once loaded, libraries remain in memory until the application terminates. You can attempt to unload a library using <a href="http://doc.qt.io/qt-5/qlibrary.html#unload">unload</a>(), but if other instances of <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> are using the same library, the call will fail, and unloading will only happen when every instance has called <a href="http://doc.qt.io/qt-5/qlibrary.html#unload">unload</a>().</p>
/// <p>A typical use of <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> is to resolve an exported symbol in a library, and to call the C function that this symbol represents. This is called "explicit linking" in contrast to "implicit linking", which is done by the link step in the build process when linking an executable against a library.</p>
/// <p>The following code snippet loads a library, resolves the symbol "mysymbol", and calls the function if everything succeeded. If something goes wrong, e.g. the library file does not exist or the symbol is not defined, the function pointer will be 0 and won't be called.</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlibrary.html#QLibrary">QLibrary</a></span> myLib(<span class="string">"mylib"</span>);
///   <span class="keyword">typedef</span> <span class="type">void</span> (<span class="operator">*</span>MyPrototype)();
///   MyPrototype myFunction <span class="operator">=</span> (MyPrototype) myLib<span class="operator">.</span>resolve(<span class="string">"mysymbol"</span>);
///   <span class="keyword">if</span> (myFunction)
/// &#32;     myFunction();
///
/// </pre>
/// <p>The symbol must be exported as a C function from the library for <a href="http://doc.qt.io/qt-5/qlibrary.html#resolve">resolve</a>() to work. This means that the function must be wrapped in an <code>extern "C"</code> block if the library is compiled with a C++ compiler. On Windows, this also requires the use of a <code>dllexport</code> macro; see <a href="http://doc.qt.io/qt-5/qlibrary.html#resolve">resolve</a>() for the details of how this is done. For convenience, there is a static <a href="http://doc.qt.io/qt-5/qlibrary.html#resolve">resolve</a>() function which you can use if you just want to call a function in a library without explicitly loading the library first:</p>
/// <pre class="cpp">
///
///   <span class="keyword">typedef</span> <span class="type">void</span> (<span class="operator">*</span>MyPrototype)();
///   MyPrototype myFunction <span class="operator">=</span>
/// &#32;   &#32;     (MyPrototype) <span class="type"><a href="http://doc.qt.io/qt-5/qlibrary.html#QLibrary">QLibrary</a></span><span class="operator">::</span>resolve(<span class="string">"mylib"</span><span class="operator">,</span> <span class="string">"mysymbol"</span>);
///   <span class="keyword">if</span> (myFunction)
/// &#32;     myFunction();
///
/// </pre></div>
#[repr(C)]
pub struct QLibrary {
    _unused: u8,
}
impl QLibrary {
    /// <p>Returns a text string with the description of the last error that occurred. Currently, errorString will only be set if <a href="http://doc.qt.io/qt-5/qlibrary.html#load">load</a>(), <a href="http://doc.qt.io/qt-5/qlibrary.html#unload">unload</a>() or <a href="http://doc.qt.io/qt-5/qlibrary.html#resolve">resolve</a>() for some reason fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLibrary::errorString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibrary.html#errorString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a text string with the description of the last error that occurred. Currently, errorString will only be set if <a href="http://doc.qt.io/qt-5/qlibrary.html#load">load</a>(), <a href="http://doc.qt.io/qt-5/qlibrary.html#unload">unload</a>() or <a href="http://doc.qt.io/qt-5/qlibrary.html#resolve">resolve</a>() for some reason fails.</p>
    /// <p>This function was introduced in  Qt 4.2.</p></div>
    pub unsafe fn error_string(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLibrary_errorString(self as *const crate::QLibrary);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This property holds the file name of the library</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QLibrary::fileName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibrary.html#fileName-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the file name of the library</p>
    /// <p>We recommend omitting the file's suffix in the file name, since <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> will automatically look for the file with the appropriate suffix (see <a href="http://doc.qt.io/qt-5/qlibrary.html#isLibrary">isLibrary</a>()).</p>
    /// <p>When loading the library, <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> searches in all system-specific library locations (for example, <code>LD_LIBRARY_PATH</code> on Unix), unless the file name has an absolute path. After loading the library successfully, fileName() returns the fully-qualified file name of the library, including the full path to the library if one was given in the constructor or passed to setFileName().</p>
    /// <p>For example, after successfully loading the "GL" library on Unix platforms, fileName() will return "libGL.so". If the file name was originally passed as "/usr/lib/libGL", fileName() will return "/usr/lib/libGL.so".</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QString </td><td class="memItemRight bottomAlign"><span class="name"><b>fileName</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setFileName</b></span>(const QString &amp;<i>fileName</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn file_name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLibrary_fileName(self as *const crate::QLibrary);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if <i>fileName</i> has a valid suffix for a loadable library; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QLibrary::isLibrary(const QString& fileName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibrary.html#isLibrary">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>fileName</i> has a valid suffix for a loadable library; otherwise returns <code>false</code>.</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Platform</th><th>Valid suffixes</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>Windows</td><td><code>.dll</code>, <code>.DLL</code></td></tr>
    /// <tr valign="top" class="even"><td>Unix/Linux</td><td><code>.so</code></td></tr>
    /// <tr valign="top" class="odd"><td>AIX</td><td><code>.a</code></td></tr>
    /// <tr valign="top" class="even"><td>HP-UX</td><td><code>.sl</code>, <code>.so</code> (HP-UXi)</td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS</td><td><code>.dylib</code>, <code>.bundle</code>, <code>.so</code></td></tr>
    /// </tbody></table></div>
    /// <p>Trailing versioning numbers on Unix are ignored.</p></div>
    pub unsafe fn is_library(
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLibrary_isLibrary(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if the library is loaded; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLibrary::isLoaded() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibrary.html#isLoaded">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the library is loaded; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlibrary.html#load">load</a>().</p></div>
    pub unsafe fn is_loaded(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLibrary_isLoaded(self as *const crate::QLibrary)
    }

    /// <p>Loads the library and returns <code>true</code> if the library was loaded successfully; otherwise returns <code>false</code>. Since <a href="http://doc.qt.io/qt-5/qlibrary.html#resolve">resolve</a>() always calls this function before resolving any symbols it is not necessary to call it explicitly. In some situations you might want the library loaded in advance, in which case you would use this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLibrary::load()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibrary.html#load">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Loads the library and returns <code>true</code> if the library was loaded successfully; otherwise returns <code>false</code>. Since <a href="http://doc.qt.io/qt-5/qlibrary.html#resolve">resolve</a>() always calls this function before resolving any symbols it is not necessary to call it explicitly. In some situations you might want the library loaded in advance, in which case you would use this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlibrary.html#unload">unload</a>().</p></div>
    pub unsafe fn load(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLibrary_load(self as *mut crate::QLibrary)
    }

    /// <p>Give the <a href="http://doc.qt.io/qt-5/qlibrary.html#load">load</a>() function some hints on how it should behave.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFlags<QLibrary::LoadHint> QLibrary::loadHints() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibrary.html#loadHints-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Give the <a href="http://doc.qt.io/qt-5/qlibrary.html#load">load</a>() function some hints on how it should behave.</p>
    /// <p>You can give some hints on how the symbols are resolved. Usually, the symbols are not resolved at load time, but resolved lazily, (that is, when <a href="http://doc.qt.io/qt-5/qlibrary.html#resolve">resolve</a>() is called). If you set the loadHints to <a href="http://doc.qt.io/qt-5/qlibrary.html#LoadHint-enum">ResolveAllSymbolsHint</a>, then all symbols will be resolved at load time if the platform supports it.</p>
    /// <p>Setting <a href="http://doc.qt.io/qt-5/qlibrary.html#LoadHint-enum">ExportExternalSymbolsHint</a> will make the external symbols in the library available for resolution in subsequent loaded libraries.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qlibrary.html#LoadHint-enum">LoadArchiveMemberHint</a> is set, the file name is composed of two components: A path which is a reference to an archive file followed by the second component which is the reference to the archive member. For instance, the <a href="http://doc.qt.io/qt-5/qlibrary.html#fileName-prop">fileName</a> <code>libGL.a(shr_64.o)</code> will refer to the library <code>shr_64.o</code> in the archive file named <code>libGL.a</code>. This is only supported on the AIX platform.</p>
    /// <p>The interpretation of the load hints is platform dependent, and if you use it you are probably making some assumptions on which platform you are compiling for, so use them only if you understand the consequences of them.</p>
    /// <p>By default, none of these flags are set, so libraries will be loaded with lazy symbol resolution, and will not export external symbols for resolution in other dynamically-loaded libraries.</p>
    /// <p><b>Note: </b>Setting this property after the library has been loaded has no effect and loadHints() will not reflect those changes.</p><p><b>Note: </b>This property is shared among all <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> instances that refer to the same library.</p><p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> LoadHints </td><td class="memItemRight bottomAlign"><span class="name"><b>loadHints</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setLoadHints</b></span>(LoadHints <i>hints</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn load_hints(&self) -> crate::QFlags<crate::q_library::LoadHint> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLibrary_loadHints(self as *const crate::QLibrary);
        crate::QFlags::from(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QLibrary::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLibrary_metaObject(self as *const crate::QLibrary);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs a library with the given <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLibrary::QLibrary(QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibrary.html#QLibrary">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a library with the given <i>parent</i>.</p></div>
    pub unsafe fn from_q_object(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QLibrary> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLibrary_QLibrary(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a library object with the given <i>parent</i> that will load the library specified by <i>fileName</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLibrary::QLibrary(const QString& fileName, QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibrary.html#QLibrary-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a library object with the given <i>parent</i> that will load the library specified by <i>fileName</i>.</p>
    /// <p>We recommend omitting the file's suffix in <i>fileName</i>, since <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> will automatically look for the file with the appropriate suffix in accordance with the platform, e.g. ".so" on Unix, ".dylib" on <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS, and ".dll" on Windows. (See <a href="http://doc.qt.io/qt-5/qlibrary.html#fileName-prop">fileName</a>.)</p></div>
    pub unsafe fn from_q_string_q_object(
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QLibrary> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLibrary_QLibrary1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a library object with the given <i>parent</i> that will load the library specified by <i>fileName</i> and major version number <i>verNum</i>. Currently, the version number is ignored on Windows.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLibrary::QLibrary(const QString& fileName, int verNum, QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibrary.html#QLibrary-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a library object with the given <i>parent</i> that will load the library specified by <i>fileName</i> and major version number <i>verNum</i>. Currently, the version number is ignored on Windows.</p>
    /// <p>We recommend omitting the file's suffix in <i>fileName</i>, since <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> will automatically look for the file with the appropriate suffix in accordance with the platform, e.g. ".so" on Unix, ".dylib" on <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS, and ".dll" on Windows. (See <a href="http://doc.qt.io/qt-5/qlibrary.html#fileName-prop">fileName</a>.)</p></div>
    pub unsafe fn from_q_string_int_q_object(
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        ver_num: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QLibrary> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLibrary_QLibrary2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
            ver_num,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a library object with the given <i>parent</i> that will load the library specified by <i>fileName</i> and full version number <i>version</i>. Currently, the version number is ignored on Windows.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLibrary::QLibrary(const QString& fileName, const QString& version, QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibrary.html#QLibrary-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a library object with the given <i>parent</i> that will load the library specified by <i>fileName</i> and full version number <i>version</i>. Currently, the version number is ignored on Windows.</p>
    /// <p>We recommend omitting the file's suffix in <i>fileName</i>, since <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> will automatically look for the file with the appropriate suffix in accordance with the platform, e.g. ".so" on Unix, ".dylib" on <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS, and ".dll" on Windows. (See <a href="http://doc.qt.io/qt-5/qlibrary.html#fileName-prop">fileName</a>.)</p></div>
    pub unsafe fn from_2_q_string_q_object(
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        version: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QLibrary> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLibrary_QLibrary3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(version)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> class loads shared libraries at runtime.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLibrary::QLibrary()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibrary.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> class loads shared libraries at runtime.</p>
    /// <p>An instance of a <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> object operates on a single shared object file (which we call a "library", but is also known as a "DLL"). A <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> provides access to the functionality in the library in a platform independent way. You can either pass a file name in the constructor, or set it explicitly with <a href="http://doc.qt.io/qt-5/qlibrary.html#fileName-prop">setFileName</a>(). When loading the library, <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> searches in all the system-specific library locations (e.g. <code>LD_LIBRARY_PATH</code> on Unix), unless the file name has an absolute path.</p>
    /// <p>If the file name is an absolute path then an attempt is made to load this path first. If the file cannot be found, <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> tries the name with different platform-specific file prefixes, like "lib" on Unix and Mac, and suffixes, like ".so" on Unix, ".dylib" on the Mac, or ".dll" on Windows.</p>
    /// <p>If the file path is not absolute then <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> modifies the search order to try the system-specific prefixes and suffixes first, followed by the file path specified.</p>
    /// <p>This makes it possible to specify shared libraries that are only identified by their basename (i.e. without their suffix), so the same code will work on different operating systems yet still minimise the number of attempts to find the library.</p>
    /// <p>The most important functions are <a href="http://doc.qt.io/qt-5/qlibrary.html#load">load</a>() to dynamically load the library file, <a href="http://doc.qt.io/qt-5/qlibrary.html#isLoaded">isLoaded</a>() to check whether loading was successful, and <a href="http://doc.qt.io/qt-5/qlibrary.html#resolve">resolve</a>() to resolve a symbol in the library. The <a href="http://doc.qt.io/qt-5/qlibrary.html#resolve">resolve</a>() function implicitly tries to load the library if it has not been loaded yet. Multiple instances of <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> can be used to access the same physical library. Once loaded, libraries remain in memory until the application terminates. You can attempt to unload a library using <a href="http://doc.qt.io/qt-5/qlibrary.html#unload">unload</a>(), but if other instances of <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> are using the same library, the call will fail, and unloading will only happen when every instance has called <a href="http://doc.qt.io/qt-5/qlibrary.html#unload">unload</a>().</p>
    /// <p>A typical use of <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> is to resolve an exported symbol in a library, and to call the C function that this symbol represents. This is called "explicit linking" in contrast to "implicit linking", which is done by the link step in the build process when linking an executable against a library.</p>
    /// <p>The following code snippet loads a library, resolves the symbol "mysymbol", and calls the function if everything succeeded. If something goes wrong, e.g. the library file does not exist or the symbol is not defined, the function pointer will be 0 and won't be called.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlibrary.html#QLibrary">QLibrary</a></span> myLib(<span class="string">"mylib"</span>);
    ///   <span class="keyword">typedef</span> <span class="type">void</span> (<span class="operator">*</span>MyPrototype)();
    ///   MyPrototype myFunction <span class="operator">=</span> (MyPrototype) myLib<span class="operator">.</span>resolve(<span class="string">"mysymbol"</span>);
    ///   <span class="keyword">if</span> (myFunction)
    /// &#32;     myFunction();
    ///
    /// </pre>
    /// <p>The symbol must be exported as a C function from the library for <a href="http://doc.qt.io/qt-5/qlibrary.html#resolve">resolve</a>() to work. This means that the function must be wrapped in an <code>extern "C"</code> block if the library is compiled with a C++ compiler. On Windows, this also requires the use of a <code>dllexport</code> macro; see <a href="http://doc.qt.io/qt-5/qlibrary.html#resolve">resolve</a>() for the details of how this is done. For convenience, there is a static <a href="http://doc.qt.io/qt-5/qlibrary.html#resolve">resolve</a>() function which you can use if you just want to call a function in a library without explicitly loading the library first:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">typedef</span> <span class="type">void</span> (<span class="operator">*</span>MyPrototype)();
    ///   MyPrototype myFunction <span class="operator">=</span>
    /// &#32;   &#32;     (MyPrototype) <span class="type"><a href="http://doc.qt.io/qt-5/qlibrary.html#QLibrary">QLibrary</a></span><span class="operator">::</span>resolve(<span class="string">"mylib"</span><span class="operator">,</span> <span class="string">"mysymbol"</span>);
    ///   <span class="keyword">if</span> (myFunction)
    /// &#32;     myFunction();
    ///
    /// </pre></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QLibrary> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLibrary_QLibrary4();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a library object with the given <i>parent</i> that will load the library specified by <i>fileName</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLibrary::QLibrary(const QString& fileName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibrary.html#QLibrary-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a library object with the given <i>parent</i> that will load the library specified by <i>fileName</i>.</p>
    /// <p>We recommend omitting the file's suffix in <i>fileName</i>, since <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> will automatically look for the file with the appropriate suffix in accordance with the platform, e.g. ".so" on Unix, ".dylib" on <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS, and ".dll" on Windows. (See <a href="http://doc.qt.io/qt-5/qlibrary.html#fileName-prop">fileName</a>.)</p></div>
    pub unsafe fn from_q_string(
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QLibrary> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLibrary_QLibrary5(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a library object with the given <i>parent</i> that will load the library specified by <i>fileName</i> and major version number <i>verNum</i>. Currently, the version number is ignored on Windows.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLibrary::QLibrary(const QString& fileName, int verNum)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibrary.html#QLibrary-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a library object with the given <i>parent</i> that will load the library specified by <i>fileName</i> and major version number <i>verNum</i>. Currently, the version number is ignored on Windows.</p>
    /// <p>We recommend omitting the file's suffix in <i>fileName</i>, since <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> will automatically look for the file with the appropriate suffix in accordance with the platform, e.g. ".so" on Unix, ".dylib" on <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS, and ".dll" on Windows. (See <a href="http://doc.qt.io/qt-5/qlibrary.html#fileName-prop">fileName</a>.)</p></div>
    pub unsafe fn from_q_string_int(
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        ver_num: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QLibrary> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLibrary_QLibrary6(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
            ver_num,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a library object with the given <i>parent</i> that will load the library specified by <i>fileName</i> and full version number <i>version</i>. Currently, the version number is ignored on Windows.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLibrary::QLibrary(const QString& fileName, const QString& version)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibrary.html#QLibrary-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a library object with the given <i>parent</i> that will load the library specified by <i>fileName</i> and full version number <i>version</i>. Currently, the version number is ignored on Windows.</p>
    /// <p>We recommend omitting the file's suffix in <i>fileName</i>, since <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> will automatically look for the file with the appropriate suffix in accordance with the platform, e.g. ".so" on Unix, ".dylib" on <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS, and ".dll" on Windows. (See <a href="http://doc.qt.io/qt-5/qlibrary.html#fileName-prop">fileName</a>.)</p></div>
    pub unsafe fn from_2_q_string(
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        version: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QLibrary> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLibrary_QLibrary7(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(version)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QLibrary::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QLibrary_qt_metacall(
            self as *mut crate::QLibrary,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QLibrary::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLibrary_qt_metacast(
            self as *mut crate::QLibrary,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns the address of the exported symbol <i>symbol</i>. The library is loaded if necessary. The function returns 0 if the symbol could not be resolved or if the library could not be loaded.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void (*FN_PTR)() QLibrary::resolve(const char* symbol)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibrary.html#resolve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the address of the exported symbol <i>symbol</i>. The library is loaded if necessary. The function returns 0 if the symbol could not be resolved or if the library could not be loaded.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">typedef</span> <span class="type">int</span> (<span class="operator">*</span>AvgFunction)(<span class="type">int</span><span class="operator">,</span> <span class="type">int</span>);
    ///
    ///   AvgFunction avg <span class="operator">=</span> (AvgFunction) library<span class="operator">-</span><span class="operator">&gt;</span>resolve(<span class="string">"avg"</span>);
    ///   <span class="keyword">if</span> (avg)
    /// &#32;     <span class="keyword">return</span> avg(<span class="number">5</span><span class="operator">,</span> <span class="number">8</span>);
    ///   <span class="keyword">else</span>
    /// &#32;     <span class="keyword">return</span> <span class="operator">-</span><span class="number">1</span>;
    ///
    /// </pre>
    /// <p>The symbol must be exported as a C function from the library. This means that the function must be wrapped in an <code>extern "C"</code> if the library is compiled with a C++ compiler. On Windows you must also explicitly export the function from the DLL using the <code>__declspec(dllexport)</code> compiler directive, for example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">extern</span> <span class="string">"C"</span> MY_EXPORT <span class="type">int</span> avg(<span class="type">int</span> a<span class="operator">,</span> <span class="type">int</span> b)
    ///   {
    /// &#32;     <span class="keyword">return</span> (a <span class="operator">+</span> b) <span class="operator">/</span> <span class="number">2</span>;
    ///   }
    ///
    /// </pre>
    /// <p>with <code>MY_EXPORT</code> defined as</p>
    /// <pre class="cpp">
    ///
    ///   <span class="preprocessor">#ifdef Q_OS_WIN</span>
    ///   <span class="preprocessor">#define MY_EXPORT __declspec(dllexport)</span>
    ///   <span class="preprocessor">#else</span>
    ///   <span class="preprocessor">#define MY_EXPORT</span>
    ///   <span class="preprocessor">#endif</span>
    ///
    /// </pre></div>
    pub unsafe fn resolve_char(
        &mut self,
        symbol: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::std::option::Option<extern "C" fn()> {
        crate::__ffi::ctr_qt_core_ffi_QLibrary_resolve(
            self as *mut crate::QLibrary,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(symbol)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void (*FN_PTR)() QLibrary::resolve(const QString& fileName, const char* symbol)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibrary.html#resolve-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Loads the library <i>fileName</i> and returns the address of the exported symbol <i>symbol</i>. Note that <i>fileName</i> should not include the platform-specific file suffix; (see <a href="http://doc.qt.io/qt-5/qlibrary.html#fileName-prop">fileName</a>). The library remains loaded until the application exits.</p>
    /// <p>The function returns 0 if the symbol could not be resolved or if the library could not be loaded.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlibrary.html#resolve">resolve</a>().</p></div>
    pub unsafe fn resolve_q_string_char(
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        symbol: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::std::option::Option<extern "C" fn()> {
        crate::__ffi::ctr_qt_core_ffi_QLibrary_resolve1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(symbol)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void (*FN_PTR)() QLibrary::resolve(const QString& fileName, int verNum, const char* symbol)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibrary.html#resolve-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Loads the library <i>fileName</i> with major version number <i>verNum</i> and returns the address of the exported symbol <i>symbol</i>. Note that <i>fileName</i> should not include the platform-specific file suffix; (see <a href="http://doc.qt.io/qt-5/qlibrary.html#fileName-prop">fileName</a>). The library remains loaded until the application exits. <i>verNum</i> is ignored on Windows.</p>
    /// <p>The function returns 0 if the symbol could not be resolved or if the library could not be loaded.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlibrary.html#resolve">resolve</a>().</p></div>
    pub unsafe fn resolve_q_string_int_char(
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        ver_num: ::std::os::raw::c_int,
        symbol: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::std::option::Option<extern "C" fn()> {
        crate::__ffi::ctr_qt_core_ffi_QLibrary_resolve2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
            ver_num,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(symbol)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void (*FN_PTR)() QLibrary::resolve(const QString& fileName, const QString& version, const char* symbol)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibrary.html#resolve-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Loads the library <i>fileName</i> with full version number <i>version</i> and returns the address of the exported symbol <i>symbol</i>. Note that <i>fileName</i> should not include the platform-specific file suffix; (see <a href="http://doc.qt.io/qt-5/qlibrary.html#fileName-prop">fileName</a>). The library remains loaded until the application exits. <i>version</i> is ignored on Windows.</p>
    /// <p>The function returns 0 if the symbol could not be resolved or if the library could not be loaded.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlibrary.html#resolve">resolve</a>().</p></div>
    pub unsafe fn resolve_2_q_string_char(
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        version: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        symbol: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::std::option::Option<extern "C" fn()> {
        crate::__ffi::ctr_qt_core_ffi_QLibrary_resolve3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(version)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(symbol)
                .as_raw_ptr(),
        )
    }

    /// <p>This property holds the file name of the library</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QLibrary::setFileName(const QString& fileName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibrary.html#fileName-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the file name of the library</p>
    /// <p>We recommend omitting the file's suffix in the file name, since <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> will automatically look for the file with the appropriate suffix (see <a href="http://doc.qt.io/qt-5/qlibrary.html#isLibrary">isLibrary</a>()).</p>
    /// <p>When loading the library, <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> searches in all system-specific library locations (for example, <code>LD_LIBRARY_PATH</code> on Unix), unless the file name has an absolute path. After loading the library successfully, fileName() returns the fully-qualified file name of the library, including the full path to the library if one was given in the constructor or passed to setFileName().</p>
    /// <p>For example, after successfully loading the "GL" library on Unix platforms, fileName() will return "libGL.so". If the file name was originally passed as "/usr/lib/libGL", fileName() will return "/usr/lib/libGL.so".</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QString </td><td class="memItemRight bottomAlign"><span class="name"><b>fileName</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setFileName</b></span>(const QString &amp;<i>fileName</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn set_file_name(
        &mut self,
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QLibrary_setFileName(
            self as *mut crate::QLibrary,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the <a href="http://doc.qt.io/qt-5/qlibrary.html#fileName-prop">fileName</a> property and major version number to <i>fileName</i> and <i>versionNumber</i> respectively. The <i>versionNumber</i> is ignored on Windows.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QLibrary::setFileNameAndVersion(const QString& fileName, int verNum)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibrary.html#setFileNameAndVersion">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the <a href="http://doc.qt.io/qt-5/qlibrary.html#fileName-prop">fileName</a> property and major version number to <i>fileName</i> and <i>versionNumber</i> respectively. The <i>versionNumber</i> is ignored on Windows.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlibrary.html#fileName-prop">setFileName</a>().</p></div>
    pub unsafe fn set_file_name_and_version_q_string_int(
        &mut self,
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        ver_num: ::std::os::raw::c_int,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QLibrary_setFileNameAndVersion(
            self as *mut crate::QLibrary,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
            ver_num,
        )
    }

    /// <p>Sets the <a href="http://doc.qt.io/qt-5/qlibrary.html#fileName-prop">fileName</a> property and full version number to <i>fileName</i> and <i>version</i> respectively. The <i>version</i> parameter is ignored on Windows.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QLibrary::setFileNameAndVersion(const QString& fileName, const QString& version)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibrary.html#setFileNameAndVersion-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the <a href="http://doc.qt.io/qt-5/qlibrary.html#fileName-prop">fileName</a> property and full version number to <i>fileName</i> and <i>version</i> respectively. The <i>version</i> parameter is ignored on Windows.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlibrary.html#fileName-prop">setFileName</a>().</p></div>
    pub unsafe fn set_file_name_and_version_2_q_string(
        &mut self,
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        version: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QLibrary_setFileNameAndVersion1(
            self as *mut crate::QLibrary,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(version)
                .as_raw_ptr(),
        )
    }

    /// <p>Give the <a href="http://doc.qt.io/qt-5/qlibrary.html#load">load</a>() function some hints on how it should behave.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QLibrary::setLoadHints(QFlags<QLibrary::LoadHint> hints)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibrary.html#loadHints-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Give the <a href="http://doc.qt.io/qt-5/qlibrary.html#load">load</a>() function some hints on how it should behave.</p>
    /// <p>You can give some hints on how the symbols are resolved. Usually, the symbols are not resolved at load time, but resolved lazily, (that is, when <a href="http://doc.qt.io/qt-5/qlibrary.html#resolve">resolve</a>() is called). If you set the loadHints to <a href="http://doc.qt.io/qt-5/qlibrary.html#LoadHint-enum">ResolveAllSymbolsHint</a>, then all symbols will be resolved at load time if the platform supports it.</p>
    /// <p>Setting <a href="http://doc.qt.io/qt-5/qlibrary.html#LoadHint-enum">ExportExternalSymbolsHint</a> will make the external symbols in the library available for resolution in subsequent loaded libraries.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qlibrary.html#LoadHint-enum">LoadArchiveMemberHint</a> is set, the file name is composed of two components: A path which is a reference to an archive file followed by the second component which is the reference to the archive member. For instance, the <a href="http://doc.qt.io/qt-5/qlibrary.html#fileName-prop">fileName</a> <code>libGL.a(shr_64.o)</code> will refer to the library <code>shr_64.o</code> in the archive file named <code>libGL.a</code>. This is only supported on the AIX platform.</p>
    /// <p>The interpretation of the load hints is platform dependent, and if you use it you are probably making some assumptions on which platform you are compiling for, so use them only if you understand the consequences of them.</p>
    /// <p>By default, none of these flags are set, so libraries will be loaded with lazy symbol resolution, and will not export external symbols for resolution in other dynamically-loaded libraries.</p>
    /// <p><b>Note: </b>Setting this property after the library has been loaded has no effect and loadHints() will not reflect those changes.</p><p><b>Note: </b>This property is shared among all <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> instances that refer to the same library.</p><p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> LoadHints </td><td class="memItemRight bottomAlign"><span class="name"><b>loadHints</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setLoadHints</b></span>(LoadHints <i>hints</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn set_load_hints(&mut self, hints: crate::QFlags<crate::q_library::LoadHint>) {
        crate::__ffi::ctr_qt_core_ffi_QLibrary_setLoadHints(
            self as *mut crate::QLibrary,
            hints.to_int(),
        )
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLibrary_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QLibrary::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLibrary_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QLibrary::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLibrary_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Unloads the library and returns <code>true</code> if the library could be unloaded; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLibrary::unload()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibrary.html#unload">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Unloads the library and returns <code>true</code> if the library could be unloaded; otherwise returns <code>false</code>.</p>
    /// <p>This happens automatically on application termination, so you shouldn't normally need to call this function.</p>
    /// <p>If other instances of <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> are using the same library, the call will fail, and unloading will only happen when every instance has called unload().</p>
    /// <p>Note that on Mac OS X 10.3 (Panther), dynamic libraries cannot be unloaded.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlibrary.html#resolve">resolve</a>() and <a href="http://doc.qt.io/qt-5/qlibrary.html#load">load</a>().</p></div>
    pub unsafe fn unload(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLibrary_unload(self as *mut crate::QLibrary)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> class contains a version number with an arbitrary number of segments.</p>
///
/// C++ class: <span style='color: green;'>```QVersionNumber```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qversionnumber.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> class contains a version number with an arbitrary number of segments.</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qversionnumber.html#QVersionNumber">QVersionNumber</a></span> version(<span class="number">1</span><span class="operator">,</span> <span class="number">2</span><span class="operator">,</span> <span class="number">3</span>);  <span class="comment">// 1.2.3</span>
///
/// </pre></div>
#[repr(C)]
pub struct QVersionNumber {
    _unused: u8,
}
impl QVersionNumber {
    /// <p><a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> QVersionNumber::commonPrefix(const <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> &amp;v1, const <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> &amp;v2)</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QVersionNumber QVersionNumber::commonPrefix(const QVersionNumber& v1, const QVersionNumber& v2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qversionnumber.html#commonPrefix">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p><a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> QVersionNumber::commonPrefix(const <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> &amp;v1, const <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> &amp;v2)</p>
    /// <p>Returns a version number that is a parent version of both <i>v1</i> and <i>v2</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qversionnumber.html#isPrefixOf">isPrefixOf</a>().</p></div>
    pub unsafe fn common_prefix(
        v1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVersionNumber>>,
        v2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVersionNumber>>,
    ) -> ::cpp_utils::CppBox<crate::QVersionNumber> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVersionNumber_commonPrefix(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVersionNumber>>::cast_into(v1)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVersionNumber>>::cast_into(v2)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Compares <i>v1</i> with <i>v2</i> and returns an integer less than, equal to, or greater than zero, depending on whether <i>v1</i> is less than, equal to, or greater than <i>v2</i>, respectively.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QVersionNumber::compare(const QVersionNumber& v1, const QVersionNumber& v2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qversionnumber.html#compare">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares <i>v1</i> with <i>v2</i> and returns an integer less than, equal to, or greater than zero, depending on whether <i>v1</i> is less than, equal to, or greater than <i>v2</i>, respectively.</p>
    /// <p>Comparisons are performed by comparing the segments of <i>v1</i> and <i>v2</i> starting at index 0 and working towards the end of the longer list.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qversionnumber.html#QVersionNumber">QVersionNumber</a></span> v1(<span class="number">1</span><span class="operator">,</span> <span class="number">2</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qversionnumber.html#QVersionNumber">QVersionNumber</a></span> v2(<span class="number">1</span><span class="operator">,</span> <span class="number">2</span><span class="operator">,</span> <span class="number">0</span>);
    ///   <span class="type">int</span> compare <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qversionnumber.html#QVersionNumber">QVersionNumber</a></span><span class="operator">::</span>compare(v1<span class="operator">,</span> v2); <span class="comment">// compare == -1</span>
    ///
    /// </pre></div>
    pub unsafe fn compare(
        v1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVersionNumber>>,
        v2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVersionNumber>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVersionNumber_compare(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVersionNumber>>::cast_into(v1)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVersionNumber>>::cast_into(v2)
                .as_raw_ptr(),
        )
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> class contains a version number with an arbitrary number of segments.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVersionNumber& QVersionNumber::operator=(const QVersionNumber& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qversionnumber.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> class contains a version number with an arbitrary number of segments.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qversionnumber.html#QVersionNumber">QVersionNumber</a></span> version(<span class="number">1</span><span class="operator">,</span> <span class="number">2</span><span class="operator">,</span> <span class="number">3</span>);  <span class="comment">// 1.2.3</span>
    ///
    /// </pre></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVersionNumber>>,
    ) -> ::cpp_utils::MutRef<crate::QVersionNumber> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVersionNumber_operator_(
            self as *mut crate::QVersionNumber,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVersionNumber>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> from a specially formatted <i>string</i> of non-negative decimal numbers delimited by a period (<code>.</code>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QVersionNumber QVersionNumber::fromString(const QString& string, int* suffixIndex = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qversionnumber.html#fromString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> from a specially formatted <i>string</i> of non-negative decimal numbers delimited by a period (<code>.</code>).</p>
    /// <p>Once the numerical segments have been parsed, the remainder of the string is considered to be the suffix string. The start index of that string will be stored in <i>suffixIndex</i> if it is not null.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> string(<span class="string">"5.4.0-alpha"</span>);
    ///   <span class="type">int</span> suffixIndex;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qversionnumber.html#QVersionNumber">QVersionNumber</a></span> version <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qversionnumber.html#QVersionNumber">QVersionNumber</a></span><span class="operator">::</span>fromString(string<span class="operator">,</span> <span class="operator">&amp;</span>suffixIndex);
    ///   <span class="comment">// version is 5.4.0</span>
    ///   <span class="comment">// suffixIndex is 5</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qversionnumber.html#isNull">isNull</a>().</p></div>
    pub unsafe fn from_string_q_string_int(
        string: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        suffix_index: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::CppBox<crate::QVersionNumber> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVersionNumber_fromString(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(string)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_int>>::cast_into(
                suffix_index,
            )
            .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> from a specially formatted <i>string</i> of non-negative decimal numbers delimited by a period (<code>.</code>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QVersionNumber QVersionNumber::fromString(const QString& string)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qversionnumber.html#fromString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> from a specially formatted <i>string</i> of non-negative decimal numbers delimited by a period (<code>.</code>).</p>
    /// <p>Once the numerical segments have been parsed, the remainder of the string is considered to be the suffix string. The start index of that string will be stored in <i>suffixIndex</i> if it is not null.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> string(<span class="string">"5.4.0-alpha"</span>);
    ///   <span class="type">int</span> suffixIndex;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qversionnumber.html#QVersionNumber">QVersionNumber</a></span> version <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qversionnumber.html#QVersionNumber">QVersionNumber</a></span><span class="operator">::</span>fromString(string<span class="operator">,</span> <span class="operator">&amp;</span>suffixIndex);
    ///   <span class="comment">// version is 5.4.0</span>
    ///   <span class="comment">// suffixIndex is 5</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qversionnumber.html#isNull">isNull</a>().</p></div>
    pub unsafe fn from_string_q_string(
        string: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QVersionNumber> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVersionNumber_fromString1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(string)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QVersionNumber QVersionNumber::fromString(QLatin1String string, int* suffixIndex = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qversionnumber.html#fromString-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> from a specially formatted <i>string</i> of non-negative decimal numbers delimited by '.'.</p>
    /// <p>Once the numerical segments have been parsed, the remainder of the string is considered to be the suffix string. The start index of that string will be stored in <i>suffixIndex</i> if it is not null.</p>
    /// <pre class="cpp">
    ///
    ///   QLatin1String string(<span class="string">"5.4.0-alpha"</span>);
    ///   <span class="type">int</span> suffixIndex;
    ///   <span class="keyword">auto</span> version <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qversionnumber.html#QVersionNumber">QVersionNumber</a></span><span class="operator">::</span>fromString(string<span class="operator">,</span> <span class="operator">&amp;</span>suffixIndex);
    ///   <span class="comment">// version is 5.4.0</span>
    ///   <span class="comment">// suffixIndex is 5</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qversionnumber.html#isNull">isNull</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn from_string_q_latin1_string_int(
        string: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        suffix_index: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::CppBox<crate::QVersionNumber> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVersionNumber_fromString4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(string)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_int>>::cast_into(
                suffix_index,
            )
            .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QVersionNumber QVersionNumber::fromString(QStringView string, int* suffixIndex = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qversionnumber.html#fromString-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> from a specially formatted <i>string</i> of non-negative decimal numbers delimited by '.'.</p>
    /// <p>Once the numerical segments have been parsed, the remainder of the string is considered to be the suffix string. The start index of that string will be stored in <i>suffixIndex</i> if it is not null.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> string(<span class="string">"5.4.0-alpha"</span>);
    ///   <span class="type">int</span> suffixIndex;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qversionnumber.html#QVersionNumber">QVersionNumber</a></span> version <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qversionnumber.html#QVersionNumber">QVersionNumber</a></span><span class="operator">::</span>fromString(string<span class="operator">,</span> <span class="operator">&amp;</span>suffixIndex);
    ///   <span class="comment">// version is 5.4.0</span>
    ///   <span class="comment">// suffixIndex is 5</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qversionnumber.html#isNull">isNull</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn from_string_q_string_view_int(
        string: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
        suffix_index: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::CppBox<crate::QVersionNumber> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVersionNumber_fromString5(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(string)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_int>>::cast_into(
                suffix_index,
            )
            .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QVersionNumber QVersionNumber::fromString(QLatin1String string)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qversionnumber.html#fromString-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> from a specially formatted <i>string</i> of non-negative decimal numbers delimited by '.'.</p>
    /// <p>Once the numerical segments have been parsed, the remainder of the string is considered to be the suffix string. The start index of that string will be stored in <i>suffixIndex</i> if it is not null.</p>
    /// <pre class="cpp">
    ///
    ///   QLatin1String string(<span class="string">"5.4.0-alpha"</span>);
    ///   <span class="type">int</span> suffixIndex;
    ///   <span class="keyword">auto</span> version <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qversionnumber.html#QVersionNumber">QVersionNumber</a></span><span class="operator">::</span>fromString(string<span class="operator">,</span> <span class="operator">&amp;</span>suffixIndex);
    ///   <span class="comment">// version is 5.4.0</span>
    ///   <span class="comment">// suffixIndex is 5</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qversionnumber.html#isNull">isNull</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn from_string_q_latin1_string(
        string: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::QVersionNumber> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVersionNumber_fromString6(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(string)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QVersionNumber QVersionNumber::fromString(QStringView string)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qversionnumber.html#fromString-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> from a specially formatted <i>string</i> of non-negative decimal numbers delimited by '.'.</p>
    /// <p>Once the numerical segments have been parsed, the remainder of the string is considered to be the suffix string. The start index of that string will be stored in <i>suffixIndex</i> if it is not null.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> string(<span class="string">"5.4.0-alpha"</span>);
    ///   <span class="type">int</span> suffixIndex;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qversionnumber.html#QVersionNumber">QVersionNumber</a></span> version <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qversionnumber.html#QVersionNumber">QVersionNumber</a></span><span class="operator">::</span>fromString(string<span class="operator">,</span> <span class="operator">&amp;</span>suffixIndex);
    ///   <span class="comment">// version is 5.4.0</span>
    ///   <span class="comment">// suffixIndex is 5</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qversionnumber.html#isNull">isNull</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn from_string_q_string_view(
        string: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::cpp_utils::CppBox<crate::QVersionNumber> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVersionNumber_fromString7(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(string)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if the version number does not contain any trailing zeros, otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVersionNumber::isNormalized() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qversionnumber.html#isNormalized">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the version number does not contain any trailing zeros, otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qversionnumber.html#normalized">normalized</a>().</p></div>
    pub unsafe fn is_normalized(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVersionNumber_isNormalized(
            self as *const crate::QVersionNumber,
        )
    }

    /// <p>Returns <code>true</code> if there are zero numerical segments, otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVersionNumber::isNull() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qversionnumber.html#isNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if there are zero numerical segments, otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qversionnumber.html#segments">segments</a>().</p></div>
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVersionNumber_isNull(self as *const crate::QVersionNumber)
    }

    /// <p>Returns <code>true</code> if the current version number is contained in the <i>other</i> version number, otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVersionNumber::isPrefixOf(const QVersionNumber& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qversionnumber.html#isPrefixOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the current version number is contained in the <i>other</i> version number, otherwise returns <code>false</code>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qversionnumber.html#QVersionNumber">QVersionNumber</a></span> v1(<span class="number">5</span><span class="operator">,</span> <span class="number">3</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qversionnumber.html#QVersionNumber">QVersionNumber</a></span> v2(<span class="number">5</span><span class="operator">,</span> <span class="number">3</span><span class="operator">,</span> <span class="number">1</span>);
    ///   bool value <span class="operator">=</span> v1<span class="operator">.</span>isPrefixOf(v2); <span class="comment">// true</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qversionnumber.html#commonPrefix">commonPrefix</a>().</p></div>
    pub unsafe fn is_prefix_of(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVersionNumber>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVersionNumber_isPrefixOf(
            self as *const crate::QVersionNumber,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVersionNumber>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the major version number, that is, the first segment. This function is equivalent to <a href="http://doc.qt.io/qt-5/qversionnumber.html#segmentAt">segmentAt</a>(0). If this <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> object is null, this function returns 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVersionNumber::majorVersion() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qversionnumber.html#majorVersion">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the major version number, that is, the first segment. This function is equivalent to <a href="http://doc.qt.io/qt-5/qversionnumber.html#segmentAt">segmentAt</a>(0). If this <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> object is null, this function returns 0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qversionnumber.html#isNull">isNull</a>() and <a href="http://doc.qt.io/qt-5/qversionnumber.html#segmentAt">segmentAt</a>().</p></div>
    pub unsafe fn major_version(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVersionNumber_majorVersion(
            self as *const crate::QVersionNumber,
        )
    }

    /// <p>Returns the micro version number, that is, the third segment. This function is equivalent to <a href="http://doc.qt.io/qt-5/qversionnumber.html#segmentAt">segmentAt</a>(2). If this <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> object does not contain a micro number, this function returns 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVersionNumber::microVersion() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qversionnumber.html#microVersion">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the micro version number, that is, the third segment. This function is equivalent to <a href="http://doc.qt.io/qt-5/qversionnumber.html#segmentAt">segmentAt</a>(2). If this <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> object does not contain a micro number, this function returns 0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qversionnumber.html#isNull">isNull</a>() and <a href="http://doc.qt.io/qt-5/qversionnumber.html#segmentAt">segmentAt</a>().</p></div>
    pub unsafe fn micro_version(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVersionNumber_microVersion(
            self as *const crate::QVersionNumber,
        )
    }

    /// <p>Returns the minor version number, that is, the second segment. This function is equivalent to <a href="http://doc.qt.io/qt-5/qversionnumber.html#segmentAt">segmentAt</a>(1). If this <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> object does not contain a minor number, this function returns 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVersionNumber::minorVersion() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qversionnumber.html#minorVersion">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the minor version number, that is, the second segment. This function is equivalent to <a href="http://doc.qt.io/qt-5/qversionnumber.html#segmentAt">segmentAt</a>(1). If this <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> object does not contain a minor number, this function returns 0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qversionnumber.html#isNull">isNull</a>() and <a href="http://doc.qt.io/qt-5/qversionnumber.html#segmentAt">segmentAt</a>().</p></div>
    pub unsafe fn minor_version(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVersionNumber_minorVersion(
            self as *const crate::QVersionNumber,
        )
    }

    /// <p>Produces a null version.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVersionNumber::QVersionNumber()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qversionnumber.html#QVersionNumber">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Produces a null version.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qversionnumber.html#isNull">isNull</a>().</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QVersionNumber> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVersionNumber_QVersionNumber();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a version number from the list of numbers contained in <i>seg</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVersionNumber::QVersionNumber(const QVector<int>& seg)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qversionnumber.html#QVersionNumber-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a version number from the list of numbers contained in <i>seg</i>.</p></div>
    pub unsafe fn from_q_vector_of_int(
        seg: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfInt>>,
    ) -> ::cpp_utils::CppBox<crate::QVersionNumber> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVersionNumber_QVersionNumber1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfInt>>::cast_into(seg)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> consisting of just the major version number <i>maj</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVersionNumber::QVersionNumber(int maj)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qversionnumber.html#QVersionNumber-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> consisting of just the major version number <i>maj</i>.</p></div>
    pub unsafe fn from_int(
        maj: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVersionNumber> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVersionNumber_QVersionNumber4(maj);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> consisting of the major and minor version numbers <i>maj</i> and <i>min</i>, respectively.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVersionNumber::QVersionNumber(int maj, int min)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qversionnumber.html#QVersionNumber-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> consisting of the major and minor version numbers <i>maj</i> and <i>min</i>, respectively.</p></div>
    pub unsafe fn from_2_int(
        maj: ::std::os::raw::c_int,
        min: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVersionNumber> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVersionNumber_QVersionNumber5(maj, min);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> consisting of the major, minor, and micro version numbers <i>maj</i>, <i>min</i> and <i>mic</i>, respectively.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVersionNumber::QVersionNumber(int maj, int min, int mic)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qversionnumber.html#QVersionNumber-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> consisting of the major, minor, and micro version numbers <i>maj</i>, <i>min</i> and <i>mic</i>, respectively.</p></div>
    pub unsafe fn from_3_int(
        maj: ::std::os::raw::c_int,
        min: ::std::os::raw::c_int,
        mic: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVersionNumber> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVersionNumber_QVersionNumber6(maj, min, mic);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> class contains a version number with an arbitrary number of segments.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVersionNumber::QVersionNumber(const QVersionNumber& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qversionnumber.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> class contains a version number with an arbitrary number of segments.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qversionnumber.html#QVersionNumber">QVersionNumber</a></span> version(<span class="number">1</span><span class="operator">,</span> <span class="number">2</span><span class="operator">,</span> <span class="number">3</span>);  <span class="comment">// 1.2.3</span>
    ///
    /// </pre></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVersionNumber>>,
    ) -> ::cpp_utils::CppBox<crate::QVersionNumber> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVersionNumber_QVersionNumber7(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVersionNumber>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns an equivalent version number but with all trailing zeros removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVersionNumber QVersionNumber::normalized() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qversionnumber.html#normalized">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an equivalent version number but with all trailing zeros removed.</p>
    /// <p>To check if two numbers are equivalent, use normalized() on both version numbers before performing the compare.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qversionnumber.html#QVersionNumber">QVersionNumber</a></span> v1(<span class="number">5</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qversionnumber.html#QVersionNumber">QVersionNumber</a></span> v2(<span class="number">5</span><span class="operator">,</span> <span class="number">4</span><span class="operator">,</span> <span class="number">0</span>);
    ///   bool equivalent <span class="operator">=</span> v1<span class="operator">.</span>normalized() <span class="operator">=</span><span class="operator">=</span> v2<span class="operator">.</span>normalized();
    ///   bool equal <span class="operator">=</span> v1 <span class="operator">=</span><span class="operator">=</span> v2;
    ///   <span class="comment">// equivalent is true</span>
    ///   <span class="comment">// equal is false</span>
    ///
    /// </pre></div>
    pub unsafe fn normalized(&self) -> ::cpp_utils::CppBox<crate::QVersionNumber> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVersionNumber_normalized(
            self as *const crate::QVersionNumber,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the segement value at <i>index</i>. If the index does not exist, returns 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVersionNumber::segmentAt(int index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qversionnumber.html#segmentAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the segement value at <i>index</i>. If the index does not exist, returns 0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qversionnumber.html#segments">segments</a>() and <a href="http://doc.qt.io/qt-5/qversionnumber.html#segmentCount">segmentCount</a>().</p></div>
    pub unsafe fn segment_at(&self, index: ::std::os::raw::c_int) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVersionNumber_segmentAt(
            self as *const crate::QVersionNumber,
            index,
        )
    }

    /// <p>Returns the number of integers stored in <a href="http://doc.qt.io/qt-5/qversionnumber.html#segments">segments</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVersionNumber::segmentCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qversionnumber.html#segmentCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of integers stored in <a href="http://doc.qt.io/qt-5/qversionnumber.html#segments">segments</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qversionnumber.html#segments">segments</a>().</p></div>
    pub unsafe fn segment_count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVersionNumber_segmentCount(
            self as *const crate::QVersionNumber,
        )
    }

    /// <p>Returns all of the numerical segments.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<int> QVersionNumber::segments() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qversionnumber.html#segments">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns all of the numerical segments.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qversionnumber.html#majorVersion">majorVersion</a>(), <a href="http://doc.qt.io/qt-5/qversionnumber.html#minorVersion">minorVersion</a>(), and <a href="http://doc.qt.io/qt-5/qversionnumber.html#microVersion">microVersion</a>().</p></div>
    pub unsafe fn segments(&self) -> ::cpp_utils::CppBox<crate::QVectorOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVersionNumber_segments(
            self as *const crate::QVersionNumber,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a string with all of the segments delimited by a period (<code>.</code>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QVersionNumber::toString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qversionnumber.html#toString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string with all of the segments delimited by a period (<code>.</code>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qversionnumber.html#majorVersion">majorVersion</a>(), <a href="http://doc.qt.io/qt-5/qversionnumber.html#minorVersion">minorVersion</a>(), <a href="http://doc.qt.io/qt-5/qversionnumber.html#microVersion">microVersion</a>(), and <a href="http://doc.qt.io/qt-5/qversionnumber.html#segments">segments</a>().</p></div>
    pub unsafe fn to_string(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVersionNumber_toString(
            self as *const crate::QVersionNumber,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_library_info {
    //! C++ type: <span style='color: green;'>```QLibraryInfo```</span>

    /// <p>This enum type is used to specify a specific location specifier:</p>
    ///
    /// C++ enum: <span style='color: green;'>```QLibraryInfo::LibraryLocation```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibraryinfo.html#LibraryLocation-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type is used to specify a specific location specifier:</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlibraryinfo.html#location">location</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct LibraryLocation(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for LibraryLocation {
        fn from(value: ::std::os::raw::c_int) -> Self {
            LibraryLocation(value)
        }
    }

    impl From<LibraryLocation> for ::std::os::raw::c_int {
        fn from(value: LibraryLocation) -> Self {
            value.0
        }
    }

    impl LibraryLocation {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl LibraryLocation {
        /// The default prefix for all paths. (C++ enum variant: <span style='color: green;'>```PrefixPath = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const PrefixPath: crate::q_library_info::LibraryLocation =
            crate::q_library_info::LibraryLocation(0);
        /// The location for documentation upon install. (C++ enum variant: <span style='color: green;'>```DocumentationPath = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const DocumentationPath: crate::q_library_info::LibraryLocation =
            crate::q_library_info::LibraryLocation(1);
        /// The location for all headers. (C++ enum variant: <span style='color: green;'>```HeadersPath = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const HeadersPath: crate::q_library_info::LibraryLocation =
            crate::q_library_info::LibraryLocation(2);
        /// The location of installed libraries. (C++ enum variant: <span style='color: green;'>```LibrariesPath = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const LibrariesPath: crate::q_library_info::LibraryLocation =
            crate::q_library_info::LibraryLocation(3);
        /// The location of installed executables required by libraries at runtime. (C++ enum variant: <span style='color: green;'>```LibraryExecutablesPath = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const LibraryExecutablesPath: crate::q_library_info::LibraryLocation =
            crate::q_library_info::LibraryLocation(4);
        /// The location of installed Qt binaries (tools and applications). (C++ enum variant: <span style='color: green;'>```BinariesPath = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const BinariesPath: crate::q_library_info::LibraryLocation =
            crate::q_library_info::LibraryLocation(5);
        /// The location of installed Qt plugins. (C++ enum variant: <span style='color: green;'>```PluginsPath = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const PluginsPath: crate::q_library_info::LibraryLocation =
            crate::q_library_info::LibraryLocation(6);
        /// The location of installed QML extensions to import (QML 1.x). (C++ enum variant: <span style='color: green;'>```ImportsPath = 7```</span>)
        #[allow(non_upper_case_globals)]
        pub const ImportsPath: crate::q_library_info::LibraryLocation =
            crate::q_library_info::LibraryLocation(7);
        /// The location of installed QML extensions to import (QML 2.x). (C++ enum variant: <span style='color: green;'>```Qml2ImportsPath = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const Qml2ImportsPath: crate::q_library_info::LibraryLocation =
            crate::q_library_info::LibraryLocation(8);
        /// The location of general architecture-dependent Qt data. (C++ enum variant: <span style='color: green;'>```ArchDataPath = 9```</span>)
        #[allow(non_upper_case_globals)]
        pub const ArchDataPath: crate::q_library_info::LibraryLocation =
            crate::q_library_info::LibraryLocation(9);
        /// The location of general architecture-independent Qt data. (C++ enum variant: <span style='color: green;'>```DataPath = 10```</span>)
        #[allow(non_upper_case_globals)]
        pub const DataPath: crate::q_library_info::LibraryLocation =
            crate::q_library_info::LibraryLocation(10);
        /// The location of translation information for Qt strings. (C++ enum variant: <span style='color: green;'>```TranslationsPath = 11```</span>)
        #[allow(non_upper_case_globals)]
        pub const TranslationsPath: crate::q_library_info::LibraryLocation =
            crate::q_library_info::LibraryLocation(11);
        /// The location for examples upon install. (C++ enum variant: <span style='color: green;'>```ExamplesPath = 12```</span>)
        #[allow(non_upper_case_globals)]
        pub const ExamplesPath: crate::q_library_info::LibraryLocation =
            crate::q_library_info::LibraryLocation(12);
        /// The location of installed Qt testcases. (C++ enum variant: <span style='color: green;'>```TestsPath = 13```</span>)
        #[allow(non_upper_case_globals)]
        pub const TestsPath: crate::q_library_info::LibraryLocation =
            crate::q_library_info::LibraryLocation(13);
        /// The location for Qt settings. Not applicable on Windows. (C++ enum variant: <span style='color: green;'>```SettingsPath = 100```</span>)
        #[allow(non_upper_case_globals)]
        pub const SettingsPath: crate::q_library_info::LibraryLocation =
            crate::q_library_info::LibraryLocation(100);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qlibraryinfo.html">QLibraryInfo</a> class provides information about the Qt library.</p>
///
/// C++ class: <span style='color: green;'>```QLibraryInfo```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlibraryinfo.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlibraryinfo.html">QLibraryInfo</a> class provides information about the Qt library.</p>
/// <p>Many pieces of information are established when Qt is configured and built. This class provides an abstraction for accessing that information. By using the static functions of this class, an application can obtain information about the instance of the Qt library which the application is using at run-time.</p>
/// <p>You can also use a <code>qt.conf</code> file to override the hard-coded paths that are compiled into the Qt library. For more information, see the <a href="http://doc.qt.io/qt-5/qt-conf.html">Using qt.conf</a> documentation.</p></div>
#[repr(C)]
pub struct QLibraryInfo {
    _unused: u8,
}
impl QLibraryInfo {
    /// Calls C++ function: <span style='color: green;'>```static const char* QLibraryInfo::build()```</span>.
    pub unsafe fn build() -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLibraryInfo_build();
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>This function used to return the installation date for this build of Qt, but now returns a constant date.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QDate QLibraryInfo::buildDate()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibraryinfo-obsolete.html#buildDate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function used to return the installation date for this build of Qt, but now returns a constant date.</p>
    /// <p>This function was introduced in  Qt 4.6.</p></div>
    pub unsafe fn build_date() -> ::cpp_utils::CppBox<crate::QDate> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLibraryInfo_buildDate();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlibraryinfo.html">QLibraryInfo</a> class provides information about the Qt library.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLibraryInfo& QLibraryInfo::operator=(const QLibraryInfo& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibraryinfo.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlibraryinfo.html">QLibraryInfo</a> class provides information about the Qt library.</p>
    /// <p>Many pieces of information are established when Qt is configured and built. This class provides an abstraction for accessing that information. By using the static functions of this class, an application can obtain information about the instance of the Qt library which the application is using at run-time.</p>
    /// <p>You can also use a <code>qt.conf</code> file to override the hard-coded paths that are compiled into the Qt library. For more information, see the <a href="http://doc.qt.io/qt-5/qt-conf.html">Using qt.conf</a> documentation.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLibraryInfo>>,
    ) -> ::cpp_utils::MutRef<crate::QLibraryInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLibraryInfo_operator_(
            self as *mut crate::QLibraryInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLibraryInfo>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if this build of Qt was built with debugging enabled, or false if it was built in release mode.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QLibraryInfo::isDebugBuild()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibraryinfo.html#isDebugBuild">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this build of Qt was built with debugging enabled, or false if it was built in release mode.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn is_debug_build() -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLibraryInfo_isDebugBuild()
    }

    /// <p>This function used to return the products that the license for this build of Qt has access to, now returns an empty string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QLibraryInfo::licensedProducts()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibraryinfo-obsolete.html#licensedProducts">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function used to return the products that the license for this build of Qt has access to, now returns an empty string.</p></div>
    pub unsafe fn licensed_products() -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLibraryInfo_licensedProducts();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function used to return the person to whom this build of Qt is licensed, now returns an empty string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QLibraryInfo::licensee()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibraryinfo-obsolete.html#licensee">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function used to return the person to whom this build of Qt is licensed, now returns an empty string.</p></div>
    pub unsafe fn licensee() -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLibraryInfo_licensee();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the location specified by <i>loc</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QLibraryInfo::location(QLibraryInfo::LibraryLocation arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibraryinfo.html#location">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the location specified by <i>loc</i>.</p></div>
    pub unsafe fn location(
        arg1: crate::q_library_info::LibraryLocation,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLibraryInfo_location(arg1);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlibraryinfo.html">QLibraryInfo</a> class provides information about the Qt library.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLibraryInfo::QLibraryInfo(const QLibraryInfo& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibraryinfo.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlibraryinfo.html">QLibraryInfo</a> class provides information about the Qt library.</p>
    /// <p>Many pieces of information are established when Qt is configured and built. This class provides an abstraction for accessing that information. By using the static functions of this class, an application can obtain information about the instance of the Qt library which the application is using at run-time.</p>
    /// <p>You can also use a <code>qt.conf</code> file to override the hard-coded paths that are compiled into the Qt library. For more information, see the <a href="http://doc.qt.io/qt-5/qt-conf.html">Using qt.conf</a> documentation.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLibraryInfo>>,
    ) -> ::cpp_utils::CppBox<crate::QLibraryInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLibraryInfo_QLibraryInfo(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLibraryInfo>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QStringList QLibraryInfo::platformPluginArguments(const QString& platformName)```</span>.
    pub unsafe fn platform_plugin_arguments(
        platform_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLibraryInfo_platformPluginArguments(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(platform_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the version of the Qt library.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QVersionNumber QLibraryInfo::version()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibraryinfo.html#version">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the version of the Qt library.</p>
    /// <p>This function was introduced in  Qt 5.8.</p>
    /// <p><b>See also </b>qVersion().</p></div>
    pub unsafe fn version() -> ::cpp_utils::CppBox<crate::QVersionNumber> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLibraryInfo_version();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> class defines a point in the plane using integer precision.</p>
///
/// C++ class: <span style='color: green;'>```QPoint```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qpoint.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> class defines a point in the plane using integer precision.</p>
/// <p>A point is specified by a x coordinate and an y coordinate which can be accessed using the <a href="http://doc.qt.io/qt-5/qpoint.html#x">x</a>() and <a href="http://doc.qt.io/qt-5/qpoint.html#y">y</a>() functions. The <a href="http://doc.qt.io/qt-5/qpoint.html#isNull">isNull</a>() function returns <code>true</code> if both x and y are set to 0. The coordinates can be set (or altered) using the <a href="http://doc.qt.io/qt-5/qpoint.html#setX">setX</a>() and <a href="http://doc.qt.io/qt-5/qpoint.html#setY">setY</a>() functions, or alternatively the <a href="http://doc.qt.io/qt-5/qpoint.html#rx">rx</a>() and <a href="http://doc.qt.io/qt-5/qpoint.html#ry">ry</a>() functions which return references to the coordinates (allowing direct manipulation).</p>
/// <p>Given a point <i>p</i>, the following statements are all equivalent:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html#QPoint">QPoint</a></span> p;
///
///   p<span class="operator">.</span>setX(p<span class="operator">.</span>x() <span class="operator">+</span> <span class="number">1</span>);
///   p <span class="operator">+</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html#QPoint">QPoint</a></span>(<span class="number">1</span><span class="operator">,</span> <span class="number">0</span>);
///   p<span class="operator">.</span>rx()<span class="operator">+</span><span class="operator">+</span>;
///
/// </pre>
/// <p>A <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> object can also be used as a vector: Addition and subtraction are defined as for vectors (each component is added separately). A <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> object can also be divided or multiplied by an <code>int</code> or a <code>qreal</code>.</p>
/// <p>In addition, the <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> class provides the <a href="http://doc.qt.io/qt-5/qpoint.html#manhattanLength">manhattanLength</a>() function which gives an inexpensive approximation of the length of the <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> object interpreted as a vector. Finally, <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> objects can be streamed as well as compared.</p></div>
#[repr(C)]
pub struct QPoint {
    _unused: u8,
}
impl QPoint {
    /// <p>The <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> class defines a point in the plane using integer precision.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPoint& QPoint::operator=(const QPoint& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpoint.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> class defines a point in the plane using integer precision.</p>
    /// <p>A point is specified by a x coordinate and an y coordinate which can be accessed using the <a href="http://doc.qt.io/qt-5/qpoint.html#x">x</a>() and <a href="http://doc.qt.io/qt-5/qpoint.html#y">y</a>() functions. The <a href="http://doc.qt.io/qt-5/qpoint.html#isNull">isNull</a>() function returns <code>true</code> if both x and y are set to 0. The coordinates can be set (or altered) using the <a href="http://doc.qt.io/qt-5/qpoint.html#setX">setX</a>() and <a href="http://doc.qt.io/qt-5/qpoint.html#setY">setY</a>() functions, or alternatively the <a href="http://doc.qt.io/qt-5/qpoint.html#rx">rx</a>() and <a href="http://doc.qt.io/qt-5/qpoint.html#ry">ry</a>() functions which return references to the coordinates (allowing direct manipulation).</p>
    /// <p>Given a point <i>p</i>, the following statements are all equivalent:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html#QPoint">QPoint</a></span> p;
    ///
    ///   p<span class="operator">.</span>setX(p<span class="operator">.</span>x() <span class="operator">+</span> <span class="number">1</span>);
    ///   p <span class="operator">+</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html#QPoint">QPoint</a></span>(<span class="number">1</span><span class="operator">,</span> <span class="number">0</span>);
    ///   p<span class="operator">.</span>rx()<span class="operator">+</span><span class="operator">+</span>;
    ///
    /// </pre>
    /// <p>A <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> object can also be used as a vector: Addition and subtraction are defined as for vectors (each component is added separately). A <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> object can also be divided or multiplied by an <code>int</code> or a <code>qreal</code>.</p>
    /// <p>In addition, the <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> class provides the <a href="http://doc.qt.io/qt-5/qpoint.html#manhattanLength">manhattanLength</a>() function which gives an inexpensive approximation of the length of the <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> object interpreted as a vector. Finally, <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> objects can be streamed as well as compared.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
    ) -> ::cpp_utils::MutRef<crate::QPoint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPoint_operator_(
            self as *mut crate::QPoint,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html#QPoint">QPoint</a></span> p( <span class="number">3</span><span class="operator">,</span> <span class="number">7</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html#QPoint">QPoint</a></span> q(<span class="operator">-</span><span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="type">int</span> lengthSquared <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html#QPoint">QPoint</a></span><span class="operator">::</span>dotProduct(p<span class="operator">,</span> q);   <span class="comment">// lengthSquared becomes 25</span>
    ///
    /// </pre>
    /// <p>Returns the dot product of <i>p1</i> and <i>p2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QPoint::dotProduct(const QPoint& p1, const QPoint& p2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpoint.html#dotProduct">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html#QPoint">QPoint</a></span> p( <span class="number">3</span><span class="operator">,</span> <span class="number">7</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html#QPoint">QPoint</a></span> q(<span class="operator">-</span><span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="type">int</span> lengthSquared <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html#QPoint">QPoint</a></span><span class="operator">::</span>dotProduct(p<span class="operator">,</span> q);   <span class="comment">// lengthSquared becomes 25</span>
    ///
    /// </pre>
    /// <p>Returns the dot product of <i>p1</i> and <i>p2</i>.</p>
    /// <p>This function was introduced in  Qt 5.1.</p></div>
    pub unsafe fn dot_product(
        p1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
        p2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QPoint_dotProduct(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(p1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(p2).as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if both the x and y coordinates are set to 0, otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QPoint::isNull() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpoint.html#isNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if both the x and y coordinates are set to 0, otherwise returns <code>false</code>.</p></div>
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QPoint_isNull(self as *const crate::QPoint)
    }

    /// <p>Returns the sum of the absolute values of <a href="http://doc.qt.io/qt-5/qpoint.html#x">x</a>() and <a href="http://doc.qt.io/qt-5/qpoint.html#y">y</a>(), traditionally known as the "Manhattan length" of the vector from the origin to the point. For example:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QPoint::manhattanLength() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpoint.html#manhattanLength">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the sum of the absolute values of <a href="http://doc.qt.io/qt-5/qpoint.html#x">x</a>() and <a href="http://doc.qt.io/qt-5/qpoint.html#y">y</a>(), traditionally known as the "Manhattan length" of the vector from the origin to the point. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html#QPoint">QPoint</a></span> oldPosition;
    ///
    ///   MyWidget<span class="operator">::</span>mouseMoveEvent(<span class="type"><a href="http://doc.qt.io/qt-5/qmouseevent.html">QMouseEvent</a></span> <span class="operator">*</span>event)
    ///   {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html#QPoint">QPoint</a></span> point <span class="operator">=</span> event<span class="operator">-</span><span class="operator">&gt;</span>pos() <span class="operator">-</span> oldPosition;
    /// &#32;     <span class="keyword">if</span> (point<span class="operator">.</span>manhattanLength() <span class="operator">&gt;</span> <span class="number">3</span>)
    /// &#32;   &#32;     <span class="comment">// the mouse has moved more than 3 pixels since the oldPosition</span>
    ///   }
    ///
    /// </pre>
    /// <p>This is a useful, and quick to calculate, approximation to the true length:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">double</span> trueLength <span class="operator">=</span> std<span class="operator">::</span>sqrt(std<span class="operator">::</span>pow(x()<span class="operator">,</span> <span class="number">2</span>) <span class="operator">+</span> std<span class="operator">::</span>pow(y()<span class="operator">,</span> <span class="number">2</span>));
    ///
    /// </pre>
    /// <p>The tradition of "Manhattan length" arises because such distances apply to travelers who can only travel on a rectangular grid, like the streets of Manhattan.</p></div>
    pub unsafe fn manhattan_length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QPoint_manhattanLength(self as *const crate::QPoint)
    }

    /// Calls C++ function: <span style='color: green;'>```QPoint operator*(const QPoint& p, double factor)```</span>.
    pub unsafe fn mul(
        &self,
        factor: ::std::os::raw::c_double,
    ) -> ::cpp_utils::CppBox<crate::QPoint> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_operator_73(self as *const crate::QPoint, factor);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Multiplies this point's coordinates by the given <i>factor</i>, and returns a reference to this point. For example:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPoint& QPoint::operator*=(double factor)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpoint.html#operator-2a-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Multiplies this point's coordinates by the given <i>factor</i>, and returns a reference to this point. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html#QPoint">QPoint</a></span> p(<span class="operator">-</span><span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   p <span class="operator">*</span><span class="operator">=</span> <span class="number">2.5</span>;    <span class="comment">// p becomes (-3, 10)</span>
    ///
    /// </pre>
    /// <p>Note that the result is rounded to the nearest integer as points are held as integers. Use <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> for floating point accuracy.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpoint.html#operator-2f-eq">operator/=</a>().</p></div>
    pub unsafe fn mul_assign(
        &mut self,
        factor: ::std::os::raw::c_double,
    ) -> ::cpp_utils::MutRef<crate::QPoint> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QPoint_operator__3(self as *mut crate::QPoint, factor);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Constructs a null point, i.e. with coordinates (0, 0)</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPoint::QPoint()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpoint.html#QPoint">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a null point, i.e. with coordinates (0, 0)</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpoint.html#isNull">isNull</a>().</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QPoint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPoint_QPoint();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a point with the given coordinates (<i>xpos</i>, <i>ypos</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPoint::QPoint(int xpos, int ypos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpoint.html#QPoint-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a point with the given coordinates (<i>xpos</i>, <i>ypos</i>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpoint.html#setX">setX</a>() and <a href="http://doc.qt.io/qt-5/qpoint.html#setY">setY</a>().</p></div>
    pub unsafe fn new_2a(
        xpos: ::std::os::raw::c_int,
        ypos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QPoint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPoint_QPoint1(xpos, ypos);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> class defines a point in the plane using integer precision.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPoint::QPoint(const QPoint& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpoint.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> class defines a point in the plane using integer precision.</p>
    /// <p>A point is specified by a x coordinate and an y coordinate which can be accessed using the <a href="http://doc.qt.io/qt-5/qpoint.html#x">x</a>() and <a href="http://doc.qt.io/qt-5/qpoint.html#y">y</a>() functions. The <a href="http://doc.qt.io/qt-5/qpoint.html#isNull">isNull</a>() function returns <code>true</code> if both x and y are set to 0. The coordinates can be set (or altered) using the <a href="http://doc.qt.io/qt-5/qpoint.html#setX">setX</a>() and <a href="http://doc.qt.io/qt-5/qpoint.html#setY">setY</a>() functions, or alternatively the <a href="http://doc.qt.io/qt-5/qpoint.html#rx">rx</a>() and <a href="http://doc.qt.io/qt-5/qpoint.html#ry">ry</a>() functions which return references to the coordinates (allowing direct manipulation).</p>
    /// <p>Given a point <i>p</i>, the following statements are all equivalent:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html#QPoint">QPoint</a></span> p;
    ///
    ///   p<span class="operator">.</span>setX(p<span class="operator">.</span>x() <span class="operator">+</span> <span class="number">1</span>);
    ///   p <span class="operator">+</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html#QPoint">QPoint</a></span>(<span class="number">1</span><span class="operator">,</span> <span class="number">0</span>);
    ///   p<span class="operator">.</span>rx()<span class="operator">+</span><span class="operator">+</span>;
    ///
    /// </pre>
    /// <p>A <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> object can also be used as a vector: Addition and subtraction are defined as for vectors (each component is added separately). A <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> object can also be divided or multiplied by an <code>int</code> or a <code>qreal</code>.</p>
    /// <p>In addition, the <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> class provides the <a href="http://doc.qt.io/qt-5/qpoint.html#manhattanLength">manhattanLength</a>() function which gives an inexpensive approximation of the length of the <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> object interpreted as a vector. Finally, <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> objects can be streamed as well as compared.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
    ) -> ::cpp_utils::CppBox<crate::QPoint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPoint_QPoint2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the x coordinate of this point.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int& QPoint::rx()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpoint.html#rx">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the x coordinate of this point.</p>
    /// <p>Using a reference makes it possible to directly manipulate x. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html#QPoint">QPoint</a></span> p(<span class="number">1</span><span class="operator">,</span> <span class="number">2</span>);
    ///   p<span class="operator">.</span>rx()<span class="operator">-</span><span class="operator">-</span>;   <span class="comment">// p becomes (0, 2)</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpoint.html#x">x</a>() and <a href="http://doc.qt.io/qt-5/qpoint.html#setX">setX</a>().</p></div>
    pub unsafe fn rx(&mut self) -> ::cpp_utils::MutRef<::std::os::raw::c_int> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPoint_rx(self as *mut crate::QPoint);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the y coordinate of this point.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int& QPoint::ry()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpoint.html#ry">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the y coordinate of this point.</p>
    /// <p>Using a reference makes it possible to directly manipulate y. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html#QPoint">QPoint</a></span> p(<span class="number">1</span><span class="operator">,</span> <span class="number">2</span>);
    ///   p<span class="operator">.</span>ry()<span class="operator">+</span><span class="operator">+</span>;   <span class="comment">// p becomes (1, 3)</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpoint.html#y">y</a>() and <a href="http://doc.qt.io/qt-5/qpoint.html#setY">setY</a>().</p></div>
    pub unsafe fn ry(&mut self) -> ::cpp_utils::MutRef<::std::os::raw::c_int> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPoint_ry(self as *mut crate::QPoint);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Sets the x coordinate of this point to the given <i>x</i> coordinate.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QPoint::setX(int x)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpoint.html#setX">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the x coordinate of this point to the given <i>x</i> coordinate.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpoint.html#x">x</a>() and <a href="http://doc.qt.io/qt-5/qpoint.html#setY">setY</a>().</p></div>
    pub unsafe fn set_x(&mut self, x: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QPoint_setX(self as *mut crate::QPoint, x)
    }

    /// <p>Sets the y coordinate of this point to the given <i>y</i> coordinate.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QPoint::setY(int y)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpoint.html#setY">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the y coordinate of this point to the given <i>y</i> coordinate.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpoint.html#y">y</a>() and <a href="http://doc.qt.io/qt-5/qpoint.html#setX">setX</a>().</p></div>
    pub unsafe fn set_y(&mut self, y: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QPoint_setY(self as *mut crate::QPoint, y)
    }

    /// Calls C++ function: <span style='color: green;'>```QPoint operator+(const QPoint& p)```</span>.
    pub unsafe fn unary_plus(&self) -> ::cpp_utils::CppBox<crate::QPoint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_operator_78(self as *const crate::QPoint);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the x coordinate of this point.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QPoint::x() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpoint.html#x">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the x coordinate of this point.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpoint.html#setX">setX</a>() and <a href="http://doc.qt.io/qt-5/qpoint.html#rx">rx</a>().</p></div>
    pub unsafe fn x(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QPoint_x(self as *const crate::QPoint)
    }

    /// <p>Returns the y coordinate of this point.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QPoint::y() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpoint.html#y">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the y coordinate of this point.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpoint.html#setY">setY</a>() and <a href="http://doc.qt.io/qt-5/qpoint.html#ry">ry</a>().</p></div>
    pub unsafe fn y(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QPoint_y(self as *const crate::QPoint)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> class defines a point in the plane using floating point precision.</p>
///
/// C++ class: <span style='color: green;'>```QPointF```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qpointf.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> class defines a point in the plane using floating point precision.</p>
/// <p>A point is specified by a x coordinate and an y coordinate which can be accessed using the <a href="http://doc.qt.io/qt-5/qpointf.html#x">x</a>() and <a href="http://doc.qt.io/qt-5/qpointf.html#y">y</a>() functions. The coordinates of the point are specified using floating point numbers for accuracy. The <a href="http://doc.qt.io/qt-5/qpointf.html#isNull">isNull</a>() function returns <code>true</code> if both x and y are set to 0.0. The coordinates can be set (or altered) using the <a href="http://doc.qt.io/qt-5/qpointf.html#setX">setX</a>() and <a href="http://doc.qt.io/qt-5/qpointf.html#setY">setY</a>() functions, or alternatively the <a href="http://doc.qt.io/qt-5/qpointf.html#rx">rx</a>() and <a href="http://doc.qt.io/qt-5/qpointf.html#ry">ry</a>() functions which return references to the coordinates (allowing direct manipulation).</p>
/// <p>Given a point <i>p</i>, the following statements are all equivalent:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qpointf.html#QPointF">QPointF</a></span> p;
///
///   p<span class="operator">.</span>setX(p<span class="operator">.</span>x() <span class="operator">+</span> <span class="number">1.0</span>);
///   p <span class="operator">+</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qpointf.html#QPointF">QPointF</a></span>(<span class="number">1.0</span><span class="operator">,</span> <span class="number">0.0</span>);
///   p<span class="operator">.</span>rx()<span class="operator">+</span><span class="operator">+</span>;
///
/// </pre>
/// <p>A <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> object can also be used as a vector: Addition and subtraction are defined as for vectors (each component is added separately). A <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> object can also be divided or multiplied by an <code>int</code> or a <code>qreal</code>.</p>
/// <p>In addition, the <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> class provides a constructor converting a <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> object into a <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> object, and a corresponding <a href="http://doc.qt.io/qt-5/qpointf.html#toPoint">toPoint</a>() function which returns a <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> copy of <i>this</i> point. Finally, <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> objects can be streamed as well as compared.</p></div>
#[repr(C)]
pub struct QPointF {
    _unused: u8,
}
impl QPointF {
    /// <p>The <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> class defines a point in the plane using floating point precision.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPointF& QPointF::operator=(const QPointF& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpointf.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> class defines a point in the plane using floating point precision.</p>
    /// <p>A point is specified by a x coordinate and an y coordinate which can be accessed using the <a href="http://doc.qt.io/qt-5/qpointf.html#x">x</a>() and <a href="http://doc.qt.io/qt-5/qpointf.html#y">y</a>() functions. The coordinates of the point are specified using floating point numbers for accuracy. The <a href="http://doc.qt.io/qt-5/qpointf.html#isNull">isNull</a>() function returns <code>true</code> if both x and y are set to 0.0. The coordinates can be set (or altered) using the <a href="http://doc.qt.io/qt-5/qpointf.html#setX">setX</a>() and <a href="http://doc.qt.io/qt-5/qpointf.html#setY">setY</a>() functions, or alternatively the <a href="http://doc.qt.io/qt-5/qpointf.html#rx">rx</a>() and <a href="http://doc.qt.io/qt-5/qpointf.html#ry">ry</a>() functions which return references to the coordinates (allowing direct manipulation).</p>
    /// <p>Given a point <i>p</i>, the following statements are all equivalent:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpointf.html#QPointF">QPointF</a></span> p;
    ///
    ///   p<span class="operator">.</span>setX(p<span class="operator">.</span>x() <span class="operator">+</span> <span class="number">1.0</span>);
    ///   p <span class="operator">+</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qpointf.html#QPointF">QPointF</a></span>(<span class="number">1.0</span><span class="operator">,</span> <span class="number">0.0</span>);
    ///   p<span class="operator">.</span>rx()<span class="operator">+</span><span class="operator">+</span>;
    ///
    /// </pre>
    /// <p>A <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> object can also be used as a vector: Addition and subtraction are defined as for vectors (each component is added separately). A <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> object can also be divided or multiplied by an <code>int</code> or a <code>qreal</code>.</p>
    /// <p>In addition, the <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> class provides a constructor converting a <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> object into a <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> object, and a corresponding <a href="http://doc.qt.io/qt-5/qpointf.html#toPoint">toPoint</a>() function which returns a <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> copy of <i>this</i> point. Finally, <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> objects can be streamed as well as compared.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) -> ::cpp_utils::MutRef<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPointF_operator_(
            self as *mut crate::QPointF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpointf.html#QPointF">QPointF</a></span> p( <span class="number">3.1</span><span class="operator">,</span> <span class="number">7.1</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpointf.html#QPointF">QPointF</a></span> q(<span class="operator">-</span><span class="number">1.0</span><span class="operator">,</span> <span class="number">4.1</span>);
    ///   <span class="type">int</span> lengthSquared <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qpointf.html#QPointF">QPointF</a></span><span class="operator">::</span>dotProduct(p<span class="operator">,</span> q);   <span class="comment">// lengthSquared becomes 26.01</span>
    ///
    /// </pre>
    /// <p>Returns the dot product of <i>p1</i> and <i>p2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static double QPointF::dotProduct(const QPointF& p1, const QPointF& p2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpointf.html#dotProduct">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpointf.html#QPointF">QPointF</a></span> p( <span class="number">3.1</span><span class="operator">,</span> <span class="number">7.1</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpointf.html#QPointF">QPointF</a></span> q(<span class="operator">-</span><span class="number">1.0</span><span class="operator">,</span> <span class="number">4.1</span>);
    ///   <span class="type">int</span> lengthSquared <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qpointf.html#QPointF">QPointF</a></span><span class="operator">::</span>dotProduct(p<span class="operator">,</span> q);   <span class="comment">// lengthSquared becomes 26.01</span>
    ///
    /// </pre>
    /// <p>Returns the dot product of <i>p1</i> and <i>p2</i>.</p>
    /// <p>This function was introduced in  Qt 5.1.</p></div>
    pub unsafe fn dot_product(
        p1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
        p2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QPointF_dotProduct(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(p1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(p2).as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if both the x and y coordinates are set to 0.0 (ignoring the sign); otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QPointF::isNull() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpointf.html#isNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if both the x and y coordinates are set to 0.0 (ignoring the sign); otherwise returns <code>false</code>.</p></div>
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QPointF_isNull(self as *const crate::QPointF)
    }

    /// <p>Returns the sum of the absolute values of <a href="http://doc.qt.io/qt-5/qpointf.html#x">x</a>() and <a href="http://doc.qt.io/qt-5/qpointf.html#y">y</a>(), traditionally known as the "Manhattan length" of the vector from the origin to the point.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QPointF::manhattanLength() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpointf.html#manhattanLength">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the sum of the absolute values of <a href="http://doc.qt.io/qt-5/qpointf.html#x">x</a>() and <a href="http://doc.qt.io/qt-5/qpointf.html#y">y</a>(), traditionally known as the "Manhattan length" of the vector from the origin to the point.</p>
    /// <p>This function was introduced in  Qt 4.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpoint.html#manhattanLength">QPoint::manhattanLength</a>().</p></div>
    pub unsafe fn manhattan_length(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QPointF_manhattanLength(self as *const crate::QPointF)
    }

    /// <p>Constructs a null point, i.e. with coordinates (0.0, 0.0)</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPointF::QPointF()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpointf.html#QPointF">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a null point, i.e. with coordinates (0.0, 0.0)</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpointf.html#isNull">isNull</a>().</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPointF_QPointF();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of the given <i>point</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPointF::QPointF(const QPoint& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpointf.html#QPointF-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of the given <i>point</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpointf.html#toPoint">toPoint</a>().</p></div>
    pub unsafe fn new_1a(
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
    ) -> ::cpp_utils::CppBox<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPointF_QPointF1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(p).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a point with the given coordinates (<i>xpos</i>, <i>ypos</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPointF::QPointF(double xpos, double ypos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpointf.html#QPointF-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a point with the given coordinates (<i>xpos</i>, <i>ypos</i>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpointf.html#setX">setX</a>() and <a href="http://doc.qt.io/qt-5/qpointf.html#setY">setY</a>().</p></div>
    pub unsafe fn new_2a(
        xpos: ::std::os::raw::c_double,
        ypos: ::std::os::raw::c_double,
    ) -> ::cpp_utils::CppBox<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPointF_QPointF2(xpos, ypos);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> class defines a point in the plane using floating point precision.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPointF::QPointF(const QPointF& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpointf.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> class defines a point in the plane using floating point precision.</p>
    /// <p>A point is specified by a x coordinate and an y coordinate which can be accessed using the <a href="http://doc.qt.io/qt-5/qpointf.html#x">x</a>() and <a href="http://doc.qt.io/qt-5/qpointf.html#y">y</a>() functions. The coordinates of the point are specified using floating point numbers for accuracy. The <a href="http://doc.qt.io/qt-5/qpointf.html#isNull">isNull</a>() function returns <code>true</code> if both x and y are set to 0.0. The coordinates can be set (or altered) using the <a href="http://doc.qt.io/qt-5/qpointf.html#setX">setX</a>() and <a href="http://doc.qt.io/qt-5/qpointf.html#setY">setY</a>() functions, or alternatively the <a href="http://doc.qt.io/qt-5/qpointf.html#rx">rx</a>() and <a href="http://doc.qt.io/qt-5/qpointf.html#ry">ry</a>() functions which return references to the coordinates (allowing direct manipulation).</p>
    /// <p>Given a point <i>p</i>, the following statements are all equivalent:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpointf.html#QPointF">QPointF</a></span> p;
    ///
    ///   p<span class="operator">.</span>setX(p<span class="operator">.</span>x() <span class="operator">+</span> <span class="number">1.0</span>);
    ///   p <span class="operator">+</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qpointf.html#QPointF">QPointF</a></span>(<span class="number">1.0</span><span class="operator">,</span> <span class="number">0.0</span>);
    ///   p<span class="operator">.</span>rx()<span class="operator">+</span><span class="operator">+</span>;
    ///
    /// </pre>
    /// <p>A <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> object can also be used as a vector: Addition and subtraction are defined as for vectors (each component is added separately). A <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> object can also be divided or multiplied by an <code>int</code> or a <code>qreal</code>.</p>
    /// <p>In addition, the <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> class provides a constructor converting a <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> object into a <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> object, and a corresponding <a href="http://doc.qt.io/qt-5/qpointf.html#toPoint">toPoint</a>() function which returns a <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> copy of <i>this</i> point. Finally, <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> objects can be streamed as well as compared.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) -> ::cpp_utils::CppBox<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPointF_QPointF3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the x coordinate of this point.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double& QPointF::rx()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpointf.html#rx">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the x coordinate of this point.</p>
    /// <p>Using a reference makes it possible to directly manipulate x. For example:</p>
    /// <pre class="cpp">
    ///
    ///    <span class="type"><a href="http://doc.qt.io/qt-5/qpointf.html#QPointF">QPointF</a></span> p(<span class="number">1.1</span><span class="operator">,</span> <span class="number">2.5</span>);
    ///    p<span class="operator">.</span>rx()<span class="operator">-</span><span class="operator">-</span>;   <span class="comment">// p becomes (0.1, 2.5)</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpointf.html#x">x</a>() and <a href="http://doc.qt.io/qt-5/qpointf.html#setX">setX</a>().</p></div>
    pub unsafe fn rx(&mut self) -> ::cpp_utils::MutRef<::std::os::raw::c_double> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPointF_rx(self as *mut crate::QPointF);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the y coordinate of this point.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double& QPointF::ry()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpointf.html#ry">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the y coordinate of this point.</p>
    /// <p>Using a reference makes it possible to directly manipulate y. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpointf.html#QPointF">QPointF</a></span> p(<span class="number">1.1</span><span class="operator">,</span> <span class="number">2.5</span>);
    ///   p<span class="operator">.</span>ry()<span class="operator">+</span><span class="operator">+</span>;   <span class="comment">// p becomes (1.1, 3.5)</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpointf.html#y">y</a>() and <a href="http://doc.qt.io/qt-5/qpointf.html#setY">setY</a>().</p></div>
    pub unsafe fn ry(&mut self) -> ::cpp_utils::MutRef<::std::os::raw::c_double> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPointF_ry(self as *mut crate::QPointF);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Sets the x coordinate of this point to the given <i>x</i> coordinate.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QPointF::setX(double x)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpointf.html#setX">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the x coordinate of this point to the given <i>x</i> coordinate.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpointf.html#x">x</a>() and <a href="http://doc.qt.io/qt-5/qpointf.html#setY">setY</a>().</p></div>
    pub unsafe fn set_x(&mut self, x: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_QPointF_setX(self as *mut crate::QPointF, x)
    }

    /// <p>Sets the y coordinate of this point to the given <i>y</i> coordinate.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QPointF::setY(double y)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpointf.html#setY">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the y coordinate of this point to the given <i>y</i> coordinate.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpointf.html#y">y</a>() and <a href="http://doc.qt.io/qt-5/qpointf.html#setX">setX</a>().</p></div>
    pub unsafe fn set_y(&mut self, y: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_QPointF_setY(self as *mut crate::QPointF, y)
    }

    /// <p>Rounds the coordinates of this point to the nearest integer, and returns a <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> object with the rounded coordinates.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPoint QPointF::toPoint() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpointf.html#toPoint">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Rounds the coordinates of this point to the nearest integer, and returns a <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> object with the rounded coordinates.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpointf.html#QPointF">QPointF</a>().</p></div>
    pub unsafe fn to_point(&self) -> ::cpp_utils::CppBox<crate::QPoint> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QPointF_toPoint(self as *const crate::QPointF);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QPointF operator+(const QPointF& p)```</span>.
    pub unsafe fn unary_plus(&self) -> ::cpp_utils::CppBox<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_operator_85(self as *const crate::QPointF);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the x coordinate of this point.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QPointF::x() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpointf.html#x">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the x coordinate of this point.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpointf.html#setX">setX</a>() and <a href="http://doc.qt.io/qt-5/qpointf.html#rx">rx</a>().</p></div>
    pub unsafe fn x(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QPointF_x(self as *const crate::QPointF)
    }

    /// <p>Returns the y coordinate of this point.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QPointF::y() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpointf.html#y">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the y coordinate of this point.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpointf.html#setY">setY</a>() and <a href="http://doc.qt.io/qt-5/qpointf.html#ry">ry</a>().</p></div>
    pub unsafe fn y(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QPointF_y(self as *const crate::QPointF)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qline.html">QLine</a> class provides a two-dimensional vector using integer precision.</p>
///
/// C++ class: <span style='color: green;'>```QLine```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qline.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qline.html">QLine</a> class provides a two-dimensional vector using integer precision.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qline.html">QLine</a> describes a finite length line (or a line segment) on a two-dimensional surface. The start and end points of the line are specified using integer point accuracy for coordinates. Use the <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> constructor to retrieve a floating point copy.</p>
/// <div class="table"><table class="generic">
///  <tbody><tr valign="top" class="odd"><td><img src="http://doc.qt.io/qt-5/images/qline-point.png" alt=""></td><td><img src="http://doc.qt.io/qt-5/images/qline-coordinates.png" alt=""></td></tr>
/// </tbody></table></div>
/// <p>The positions of the line's start and end points can be retrieved using the <a href="http://doc.qt.io/qt-5/qline.html#p1">p1</a>(), <a href="http://doc.qt.io/qt-5/qline.html#x1">x1</a>(), <a href="http://doc.qt.io/qt-5/qline.html#y1">y1</a>(), <a href="http://doc.qt.io/qt-5/qline.html#p2">p2</a>(), <a href="http://doc.qt.io/qt-5/qline.html#x2">x2</a>(), and <a href="http://doc.qt.io/qt-5/qline.html#y2">y2</a>() functions. The <a href="http://doc.qt.io/qt-5/qline.html#dx">dx</a>() and <a href="http://doc.qt.io/qt-5/qline.html#dy">dy</a>() functions return the horizontal and vertical components of the line. Use <a href="http://doc.qt.io/qt-5/qline.html#isNull">isNull</a>() to determine whether the <a href="http://doc.qt.io/qt-5/qline.html">QLine</a> represents a valid line or a null line.</p>
/// <p>Finally, the line can be translated a given offset using the <a href="http://doc.qt.io/qt-5/qline.html#translate">translate</a>() function.</p></div>
#[repr(C)]
pub struct QLine {
    _unused: u8,
}
impl QLine {
    /// <p>Returns the center point of this line. This is equivalent to (<a href="http://doc.qt.io/qt-5/qline.html#p1">p1</a>() + <a href="http://doc.qt.io/qt-5/qline.html#p2">p2</a>()) / 2, except it will never overflow.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPoint QLine::center() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qline.html#center">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the center point of this line. This is equivalent to (<a href="http://doc.qt.io/qt-5/qline.html#p1">p1</a>() + <a href="http://doc.qt.io/qt-5/qline.html#p2">p2</a>()) / 2, except it will never overflow.</p>
    /// <p>This function was introduced in  Qt 5.8.</p></div>
    pub unsafe fn center(&self) -> ::cpp_utils::CppBox<crate::QPoint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLine_center(self as *const crate::QLine);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qline.html">QLine</a> class provides a two-dimensional vector using integer precision.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLine& QLine::operator=(const QLine& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qline.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qline.html">QLine</a> class provides a two-dimensional vector using integer precision.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qline.html">QLine</a> describes a finite length line (or a line segment) on a two-dimensional surface. The start and end points of the line are specified using integer point accuracy for coordinates. Use the <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> constructor to retrieve a floating point copy.</p>
    /// <div class="table"><table class="generic">
    ///  <tbody><tr valign="top" class="odd"><td><img src="http://doc.qt.io/qt-5/images/qline-point.png" alt=""></td><td><img src="http://doc.qt.io/qt-5/images/qline-coordinates.png" alt=""></td></tr>
    /// </tbody></table></div>
    /// <p>The positions of the line's start and end points can be retrieved using the <a href="http://doc.qt.io/qt-5/qline.html#p1">p1</a>(), <a href="http://doc.qt.io/qt-5/qline.html#x1">x1</a>(), <a href="http://doc.qt.io/qt-5/qline.html#y1">y1</a>(), <a href="http://doc.qt.io/qt-5/qline.html#p2">p2</a>(), <a href="http://doc.qt.io/qt-5/qline.html#x2">x2</a>(), and <a href="http://doc.qt.io/qt-5/qline.html#y2">y2</a>() functions. The <a href="http://doc.qt.io/qt-5/qline.html#dx">dx</a>() and <a href="http://doc.qt.io/qt-5/qline.html#dy">dy</a>() functions return the horizontal and vertical components of the line. Use <a href="http://doc.qt.io/qt-5/qline.html#isNull">isNull</a>() to determine whether the <a href="http://doc.qt.io/qt-5/qline.html">QLine</a> represents a valid line or a null line.</p>
    /// <p>Finally, the line can be translated a given offset using the <a href="http://doc.qt.io/qt-5/qline.html#translate">translate</a>() function.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLine>>,
    ) -> ::cpp_utils::MutRef<crate::QLine> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLine_operator_(
            self as *mut crate::QLine,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLine>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the horizontal component of the line's vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QLine::dx() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qline.html#dx">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the horizontal component of the line's vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qline.html#dy">dy</a>().</p></div>
    pub unsafe fn dx(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QLine_dx(self as *const crate::QLine)
    }

    /// <p>Returns the vertical component of the line's vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QLine::dy() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qline.html#dy">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the vertical component of the line's vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qline.html#dx">dx</a>().</p></div>
    pub unsafe fn dy(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QLine_dy(self as *const crate::QLine)
    }

    /// <p>Returns <code>true</code> if the line is not set up with valid start and end point; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLine::isNull() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qline.html#isNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the line is not set up with valid start and end point; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLine_isNull(self as *const crate::QLine)
    }

    /// <p>Constructs a null line.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLine::QLine()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qline.html#QLine">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a null line.</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QLine> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLine_QLine();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a line object that represents the line between <i>p1</i> and <i>p2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLine::QLine(const QPoint& pt1, const QPoint& pt2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qline.html#QLine-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a line object that represents the line between <i>p1</i> and <i>p2</i>.</p></div>
    pub unsafe fn new_2a(
        pt1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
        pt2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
    ) -> ::cpp_utils::CppBox<crate::QLine> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLine_QLine1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(pt1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(pt2).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a line object that represents the line between (<i>x1</i>, <i>y1</i>) and (<i>x2</i>, <i>y2</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLine::QLine(int x1, int y1, int x2, int y2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qline.html#QLine-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a line object that represents the line between (<i>x1</i>, <i>y1</i>) and (<i>x2</i>, <i>y2</i>).</p></div>
    pub unsafe fn new_4a(
        x1: ::std::os::raw::c_int,
        y1: ::std::os::raw::c_int,
        x2: ::std::os::raw::c_int,
        y2: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QLine> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLine_QLine2(x1, y1, x2, y2);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qline.html">QLine</a> class provides a two-dimensional vector using integer precision.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLine::QLine(const QLine& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qline.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qline.html">QLine</a> class provides a two-dimensional vector using integer precision.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qline.html">QLine</a> describes a finite length line (or a line segment) on a two-dimensional surface. The start and end points of the line are specified using integer point accuracy for coordinates. Use the <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> constructor to retrieve a floating point copy.</p>
    /// <div class="table"><table class="generic">
    ///  <tbody><tr valign="top" class="odd"><td><img src="http://doc.qt.io/qt-5/images/qline-point.png" alt=""></td><td><img src="http://doc.qt.io/qt-5/images/qline-coordinates.png" alt=""></td></tr>
    /// </tbody></table></div>
    /// <p>The positions of the line's start and end points can be retrieved using the <a href="http://doc.qt.io/qt-5/qline.html#p1">p1</a>(), <a href="http://doc.qt.io/qt-5/qline.html#x1">x1</a>(), <a href="http://doc.qt.io/qt-5/qline.html#y1">y1</a>(), <a href="http://doc.qt.io/qt-5/qline.html#p2">p2</a>(), <a href="http://doc.qt.io/qt-5/qline.html#x2">x2</a>(), and <a href="http://doc.qt.io/qt-5/qline.html#y2">y2</a>() functions. The <a href="http://doc.qt.io/qt-5/qline.html#dx">dx</a>() and <a href="http://doc.qt.io/qt-5/qline.html#dy">dy</a>() functions return the horizontal and vertical components of the line. Use <a href="http://doc.qt.io/qt-5/qline.html#isNull">isNull</a>() to determine whether the <a href="http://doc.qt.io/qt-5/qline.html">QLine</a> represents a valid line or a null line.</p>
    /// <p>Finally, the line can be translated a given offset using the <a href="http://doc.qt.io/qt-5/qline.html#translate">translate</a>() function.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLine>>,
    ) -> ::cpp_utils::CppBox<crate::QLine> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLine_QLine3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLine>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the line's start point.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPoint QLine::p1() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qline.html#p1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the line's start point.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qline.html#setP1">setP1</a>(), <a href="http://doc.qt.io/qt-5/qline.html#x1">x1</a>(), <a href="http://doc.qt.io/qt-5/qline.html#y1">y1</a>(), and <a href="http://doc.qt.io/qt-5/qline.html#p2">p2</a>().</p></div>
    pub unsafe fn p1(&self) -> ::cpp_utils::CppBox<crate::QPoint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLine_p1(self as *const crate::QLine);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the line's end point.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPoint QLine::p2() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qline.html#p2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the line's end point.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qline.html#setP2">setP2</a>(), <a href="http://doc.qt.io/qt-5/qline.html#x2">x2</a>(), <a href="http://doc.qt.io/qt-5/qline.html#y2">y2</a>(), and <a href="http://doc.qt.io/qt-5/qline.html#p1">p1</a>().</p></div>
    pub unsafe fn p2(&self) -> ::cpp_utils::CppBox<crate::QPoint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLine_p2(self as *const crate::QLine);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Sets this line to the start in <i>x1</i>, <i>y1</i> and end in <i>x2</i>, <i>y2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QLine::setLine(int x1, int y1, int x2, int y2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qline.html#setLine">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets this line to the start in <i>x1</i>, <i>y1</i> and end in <i>x2</i>, <i>y2</i>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qline.html#setP1">setP1</a>(), <a href="http://doc.qt.io/qt-5/qline.html#setP2">setP2</a>(), <a href="http://doc.qt.io/qt-5/qline.html#p1">p1</a>(), and <a href="http://doc.qt.io/qt-5/qline.html#p2">p2</a>().</p></div>
    pub unsafe fn set_line(
        &mut self,
        x1: ::std::os::raw::c_int,
        y1: ::std::os::raw::c_int,
        x2: ::std::os::raw::c_int,
        y2: ::std::os::raw::c_int,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QLine_setLine(self as *mut crate::QLine, x1, y1, x2, y2)
    }

    /// <p>Sets the starting point of this line to <i>p1</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QLine::setP1(const QPoint& p1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qline.html#setP1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the starting point of this line to <i>p1</i>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qline.html#setP2">setP2</a>() and <a href="http://doc.qt.io/qt-5/qline.html#p1">p1</a>().</p></div>
    pub unsafe fn set_p1(
        &mut self,
        p1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QLine_setP1(
            self as *mut crate::QLine,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(p1).as_raw_ptr(),
        )
    }

    /// <p>Sets the end point of this line to <i>p2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QLine::setP2(const QPoint& p2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qline.html#setP2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the end point of this line to <i>p2</i>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qline.html#setP1">setP1</a>() and <a href="http://doc.qt.io/qt-5/qline.html#p2">p2</a>().</p></div>
    pub unsafe fn set_p2(
        &mut self,
        p2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QLine_setP2(
            self as *mut crate::QLine,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(p2).as_raw_ptr(),
        )
    }

    /// <p>Sets the start point of this line to <i>p1</i> and the end point of this line to <i>p2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QLine::setPoints(const QPoint& p1, const QPoint& p2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qline.html#setPoints">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the start point of this line to <i>p1</i> and the end point of this line to <i>p2</i>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qline.html#setP1">setP1</a>(), <a href="http://doc.qt.io/qt-5/qline.html#setP2">setP2</a>(), <a href="http://doc.qt.io/qt-5/qline.html#p1">p1</a>(), and <a href="http://doc.qt.io/qt-5/qline.html#p2">p2</a>().</p></div>
    pub unsafe fn set_points(
        &mut self,
        p1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
        p2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QLine_setPoints(
            self as *mut crate::QLine,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(p1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(p2).as_raw_ptr(),
        )
    }

    /// <p>Translates this line by the given <i>offset</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QLine::translate(const QPoint& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qline.html#translate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Translates this line by the given <i>offset</i>.</p></div>
    pub unsafe fn translate_1a(
        &mut self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QLine_translate(
            self as *mut crate::QLine,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(p).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QLine::translate(int dx, int dy)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qline.html#translate-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Translates this line the distance specified by <i>dx</i> and <i>dy</i>.</p></div>
    pub unsafe fn translate_2a(&mut self, dx: ::std::os::raw::c_int, dy: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QLine_translate1(self as *mut crate::QLine, dx, dy)
    }

    /// <p>Returns this line translated by the given <i>offset</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLine QLine::translated(const QPoint& p) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qline.html#translated">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns this line translated by the given <i>offset</i>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p></div>
    pub unsafe fn translated_1a(
        &self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
    ) -> ::cpp_utils::CppBox<crate::QLine> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLine_translated(
            self as *const crate::QLine,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(p).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLine QLine::translated(int dx, int dy) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qline.html#translated-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns this line translated the distance specified by <i>dx</i> and <i>dy</i>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p></div>
    pub unsafe fn translated_2a(
        &self,
        dx: ::std::os::raw::c_int,
        dy: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QLine> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLine_translated1(self as *const crate::QLine, dx, dy);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the x-coordinate of the line's start point.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QLine::x1() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qline.html#x1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the x-coordinate of the line's start point.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qline.html#p1">p1</a>().</p></div>
    pub unsafe fn x1(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QLine_x1(self as *const crate::QLine)
    }

    /// <p>Returns the x-coordinate of the line's end point.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QLine::x2() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qline.html#x2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the x-coordinate of the line's end point.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qline.html#p2">p2</a>().</p></div>
    pub unsafe fn x2(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QLine_x2(self as *const crate::QLine)
    }

    /// <p>Returns the y-coordinate of the line's start point.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QLine::y1() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qline.html#y1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the y-coordinate of the line's start point.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qline.html#p1">p1</a>().</p></div>
    pub unsafe fn y1(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QLine_y1(self as *const crate::QLine)
    }

    /// <p>Returns the y-coordinate of the line's end point.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QLine::y2() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qline.html#y2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the y-coordinate of the line's end point.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qline.html#p2">p2</a>().</p></div>
    pub unsafe fn y2(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QLine_y2(self as *const crate::QLine)
    }
}

pub mod q_line_f {
    //! C++ type: <span style='color: green;'>```QLineF```</span>

    /// <p>Describes the intersection between two lines.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QLineF::IntersectType```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#IntersectType-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Describes the intersection between two lines.</p>
    /// <div class="table"><table class="generic">
    ///  <tbody><tr valign="top" class="odd"><td><img src="http://doc.qt.io/qt-5/images/qlinef-unbounded.png" alt=""></td><td><img src="http://doc.qt.io/qt-5/images/qlinef-bounded.png" alt=""></td></tr>
    /// <tr valign="top" class="even"><td>QLineF::UnboundedIntersection</td><td>QLineF::BoundedIntersection</td></tr>
    /// </tbody></table></div>
    ///
    /// <p><a href="http://doc.qt.io/qt-5/qlinef.html#intersect">intersect</a>() will also return this value if the intersect point is within the start and end point of only one of the lines.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlinef.html#intersect">intersect</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct IntersectType(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for IntersectType {
        fn from(value: ::std::os::raw::c_int) -> Self {
            IntersectType(value)
        }
    }

    impl From<IntersectType> for ::std::os::raw::c_int {
        fn from(value: IntersectType) -> Self {
            value.0
        }
    }

    impl IntersectType {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl IntersectType {
        /// Indicates that the lines do not intersect; i.e. they are parallel. (C++ enum variant: <span style='color: green;'>```NoIntersection = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoIntersection: crate::q_line_f::IntersectType =
            crate::q_line_f::IntersectType(0);
        /// The two lines intersect with each other within the start and end points of each line. (C++ enum variant: <span style='color: green;'>```BoundedIntersection = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const BoundedIntersection: crate::q_line_f::IntersectType =
            crate::q_line_f::IntersectType(1);
        /// The two lines intersect, but not within the range defined by their lengths. This will be the case if the lines are not parallel. (C++ enum variant: <span style='color: green;'>```UnboundedIntersection = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const UnboundedIntersection: crate::q_line_f::IntersectType =
            crate::q_line_f::IntersectType(2);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> class provides a two-dimensional vector using floating point precision.</p>
///
/// C++ class: <span style='color: green;'>```QLineF```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlinef.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> class provides a two-dimensional vector using floating point precision.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> describes a finite length line (or line segment) on a two-dimensional surface. <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> defines the start and end points of the line using floating point accuracy for coordinates. Use the <a href="http://doc.qt.io/qt-5/qlinef.html#toLine">toLine</a>() function to retrieve an integer based copy of this line.</p>
/// <div class="table"><table class="generic">
///  <tbody><tr valign="top" class="odd"><td><img src="http://doc.qt.io/qt-5/images/qline-point.png" alt=""></td><td><img src="http://doc.qt.io/qt-5/images/qline-coordinates.png" alt=""></td></tr>
/// </tbody></table></div>
/// <p>The positions of the line's start and end points can be retrieved using the <a href="http://doc.qt.io/qt-5/qlinef.html#p1">p1</a>(), <a href="http://doc.qt.io/qt-5/qlinef.html#x1">x1</a>(), <a href="http://doc.qt.io/qt-5/qlinef.html#y1">y1</a>(), <a href="http://doc.qt.io/qt-5/qlinef.html#p2">p2</a>(), <a href="http://doc.qt.io/qt-5/qlinef.html#x2">x2</a>(), and <a href="http://doc.qt.io/qt-5/qlinef.html#y2">y2</a>() functions. The <a href="http://doc.qt.io/qt-5/qlinef.html#dx">dx</a>() and <a href="http://doc.qt.io/qt-5/qlinef.html#dy">dy</a>() functions return the horizontal and vertical components of the line, respectively.</p>
/// <p>The line's length can be retrieved using the <a href="http://doc.qt.io/qt-5/qlinef.html#length">length</a>() function, and altered using the <a href="http://doc.qt.io/qt-5/qlinef.html#setLength">setLength</a>() function. Similarly, <a href="http://doc.qt.io/qt-5/qlinef.html#angle">angle</a>() and <a href="http://doc.qt.io/qt-5/qlinef.html#setAngle">setAngle</a>() are respectively used for retrieving and altering the angle of the line. Use the <a href="http://doc.qt.io/qt-5/qlinef.html#isNull">isNull</a>() function to determine whether the <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> represents a valid line or a null line.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qlinef.html#intersect">intersect</a>() function determines the <a href="http://doc.qt.io/qt-5/qlinef.html#IntersectType-enum">IntersectType</a> for this line and a given line, while the <a href="http://doc.qt.io/qt-5/qlinef.html#angleTo">angleTo</a>() function returns the angle between the lines. In addition, the <a href="http://doc.qt.io/qt-5/qlinef.html#unitVector">unitVector</a>() function returns a line that has the same starting point as this line, but with a length of only 1, while the <a href="http://doc.qt.io/qt-5/qlinef.html#normalVector">normalVector</a>() function returns a line that is perpendicular to this line with the same starting point and length.</p>
/// <p>Finally, the line can be translated a given offset using the <a href="http://doc.qt.io/qt-5/qlinef.html#translate">translate</a>() function, and can be traversed using the <a href="http://doc.qt.io/qt-5/qlinef.html#pointAt">pointAt</a>() function.</p>
/// <a name="constraints"></a></div>
#[repr(C)]
pub struct QLineF {
    _unused: u8,
}
impl QLineF {
    /// <p>Returns the angle (in degrees) between this line and the given <i>line</i>, taking the direction of the lines into account. If the lines do not intersect within their range, it is the intersection point of the extended lines that serves as origin (see <a href="http://doc.qt.io/qt-5/qlinef.html#IntersectType-enum">QLineF::UnboundedIntersection</a>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QLineF::angle() const```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qlinef-obsolete.html#angle-1">C++ documentation</a> for <span style='color: green;'>```qreal QLineF::angle(const QLineF &line) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the angle (in degrees) between this line and the given <i>line</i>, taking the direction of the lines into account. If the lines do not intersect within their range, it is the intersection point of the extended lines that serves as origin (see <a href="http://doc.qt.io/qt-5/qlinef.html#IntersectType-enum">QLineF::UnboundedIntersection</a>).</p>
    /// <div class="table"><table class="generic">
    ///  <tbody><tr valign="top" class="odd"><td><img src="http://doc.qt.io/qt-5/images/qlinef-angle-identicaldirection.png" alt=""></td><td><img src="http://doc.qt.io/qt-5/images/qlinef-angle-oppositedirection.png" alt=""></td></tr>
    /// </tbody></table></div>
    /// <p>When the lines are parallel, this function returns 0 if they have the same direction; otherwise it returns 180.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlinef.html#intersect">intersect</a>().</p></div>
    pub unsafe fn angle_0a(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QLineF_angle(self as *const crate::QLineF)
    }

    /// <p>Returns the angle (in degrees) between this line and the given <i>line</i>, taking the direction of the lines into account. If the lines do not intersect within their range, it is the intersection point of the extended lines that serves as origin (see <a href="http://doc.qt.io/qt-5/qlinef.html#IntersectType-enum">QLineF::UnboundedIntersection</a>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QLineF::angle(const QLineF& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef-obsolete.html#angle-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the angle (in degrees) between this line and the given <i>line</i>, taking the direction of the lines into account. If the lines do not intersect within their range, it is the intersection point of the extended lines that serves as origin (see <a href="http://doc.qt.io/qt-5/qlinef.html#IntersectType-enum">QLineF::UnboundedIntersection</a>).</p>
    /// <div class="table"><table class="generic">
    ///  <tbody><tr valign="top" class="odd"><td><img src="http://doc.qt.io/qt-5/images/qlinef-angle-identicaldirection.png" alt=""></td><td><img src="http://doc.qt.io/qt-5/images/qlinef-angle-oppositedirection.png" alt=""></td></tr>
    /// </tbody></table></div>
    /// <p>When the lines are parallel, this function returns 0 if they have the same direction; otherwise it returns 180.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlinef.html#intersect">intersect</a>().</p></div>
    pub unsafe fn angle_1a(
        &self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLineF>>,
    ) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QLineF_angle1(
            self as *const crate::QLineF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLineF>>::cast_into(l).as_raw_ptr(),
        )
    }

    /// <p>Returns the angle (in degrees) from this line to the given <i>line</i>, taking the direction of the lines into account. If the lines do not intersect within their range, it is the intersection point of the extended lines that serves as origin (see <a href="http://doc.qt.io/qt-5/qlinef.html#IntersectType-enum">QLineF::UnboundedIntersection</a>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QLineF::angleTo(const QLineF& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#angleTo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the angle (in degrees) from this line to the given <i>line</i>, taking the direction of the lines into account. If the lines do not intersect within their range, it is the intersection point of the extended lines that serves as origin (see <a href="http://doc.qt.io/qt-5/qlinef.html#IntersectType-enum">QLineF::UnboundedIntersection</a>).</p>
    /// <p>The returned value represents the number of degrees you need to add to this line to make it have the same angle as the given <i>line</i>, going counter-clockwise.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlinef.html#intersect">intersect</a>().</p></div>
    pub unsafe fn angle_to(
        &self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLineF>>,
    ) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QLineF_angleTo(
            self as *const crate::QLineF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLineF>>::cast_into(l).as_raw_ptr(),
        )
    }

    /// <p>Returns the center point of this line. This is equivalent to 0.5 * <a href="http://doc.qt.io/qt-5/qlinef.html#p1">p1</a>() + 0.5 * <a href="http://doc.qt.io/qt-5/qlinef.html#p2">p2</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPointF QLineF::center() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#center">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the center point of this line. This is equivalent to 0.5 * <a href="http://doc.qt.io/qt-5/qlinef.html#p1">p1</a>() + 0.5 * <a href="http://doc.qt.io/qt-5/qlinef.html#p2">p2</a>().</p>
    /// <p>This function was introduced in  Qt 5.8.</p></div>
    pub unsafe fn center(&self) -> ::cpp_utils::CppBox<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLineF_center(self as *const crate::QLineF);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> class provides a two-dimensional vector using floating point precision.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLineF& QLineF::operator=(const QLineF& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> class provides a two-dimensional vector using floating point precision.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> describes a finite length line (or line segment) on a two-dimensional surface. <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> defines the start and end points of the line using floating point accuracy for coordinates. Use the <a href="http://doc.qt.io/qt-5/qlinef.html#toLine">toLine</a>() function to retrieve an integer based copy of this line.</p>
    /// <div class="table"><table class="generic">
    ///  <tbody><tr valign="top" class="odd"><td><img src="http://doc.qt.io/qt-5/images/qline-point.png" alt=""></td><td><img src="http://doc.qt.io/qt-5/images/qline-coordinates.png" alt=""></td></tr>
    /// </tbody></table></div>
    /// <p>The positions of the line's start and end points can be retrieved using the <a href="http://doc.qt.io/qt-5/qlinef.html#p1">p1</a>(), <a href="http://doc.qt.io/qt-5/qlinef.html#x1">x1</a>(), <a href="http://doc.qt.io/qt-5/qlinef.html#y1">y1</a>(), <a href="http://doc.qt.io/qt-5/qlinef.html#p2">p2</a>(), <a href="http://doc.qt.io/qt-5/qlinef.html#x2">x2</a>(), and <a href="http://doc.qt.io/qt-5/qlinef.html#y2">y2</a>() functions. The <a href="http://doc.qt.io/qt-5/qlinef.html#dx">dx</a>() and <a href="http://doc.qt.io/qt-5/qlinef.html#dy">dy</a>() functions return the horizontal and vertical components of the line, respectively.</p>
    /// <p>The line's length can be retrieved using the <a href="http://doc.qt.io/qt-5/qlinef.html#length">length</a>() function, and altered using the <a href="http://doc.qt.io/qt-5/qlinef.html#setLength">setLength</a>() function. Similarly, <a href="http://doc.qt.io/qt-5/qlinef.html#angle">angle</a>() and <a href="http://doc.qt.io/qt-5/qlinef.html#setAngle">setAngle</a>() are respectively used for retrieving and altering the angle of the line. Use the <a href="http://doc.qt.io/qt-5/qlinef.html#isNull">isNull</a>() function to determine whether the <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> represents a valid line or a null line.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qlinef.html#intersect">intersect</a>() function determines the <a href="http://doc.qt.io/qt-5/qlinef.html#IntersectType-enum">IntersectType</a> for this line and a given line, while the <a href="http://doc.qt.io/qt-5/qlinef.html#angleTo">angleTo</a>() function returns the angle between the lines. In addition, the <a href="http://doc.qt.io/qt-5/qlinef.html#unitVector">unitVector</a>() function returns a line that has the same starting point as this line, but with a length of only 1, while the <a href="http://doc.qt.io/qt-5/qlinef.html#normalVector">normalVector</a>() function returns a line that is perpendicular to this line with the same starting point and length.</p>
    /// <p>Finally, the line can be translated a given offset using the <a href="http://doc.qt.io/qt-5/qlinef.html#translate">translate</a>() function, and can be traversed using the <a href="http://doc.qt.io/qt-5/qlinef.html#pointAt">pointAt</a>() function.</p>
    /// <a name="constraints"></a></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLineF>>,
    ) -> ::cpp_utils::MutRef<crate::QLineF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLineF_operator_(
            self as *mut crate::QLineF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLineF>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the horizontal component of the line's vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QLineF::dx() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#dx">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the horizontal component of the line's vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlinef.html#dy">dy</a>() and <a href="http://doc.qt.io/qt-5/qlinef.html#pointAt">pointAt</a>().</p></div>
    pub unsafe fn dx(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QLineF_dx(self as *const crate::QLineF)
    }

    /// <p>Returns the vertical component of the line's vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QLineF::dy() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#dy">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the vertical component of the line's vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlinef.html#dx">dx</a>() and <a href="http://doc.qt.io/qt-5/qlinef.html#pointAt">pointAt</a>().</p></div>
    pub unsafe fn dy(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QLineF_dy(self as *const crate::QLineF)
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> with the given <i>length</i> and <i>angle</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QLineF QLineF::fromPolar(double length, double angle)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#fromPolar">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> with the given <i>length</i> and <i>angle</i>.</p>
    /// <p>The first point of the line will be on the origin.</p>
    /// <p>Positive values for the angles mean counter-clockwise while negative values mean the clockwise direction. Zero degrees is at the 3 o'clock position.</p>
    /// <p>This function was introduced in  Qt 4.4.</p></div>
    pub unsafe fn from_polar(
        length: ::std::os::raw::c_double,
        angle: ::std::os::raw::c_double,
    ) -> ::cpp_utils::CppBox<crate::QLineF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLineF_fromPolar(length, angle);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a value indicating whether or not <i>this</i> line intersects with the given <i>line</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLineF::IntersectType QLineF::intersect(const QLineF& l, QPointF* intersectionPoint) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#intersect">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a value indicating whether or not <i>this</i> line intersects with the given <i>line</i>.</p>
    /// <p>The actual intersection point is extracted to <i>intersectionPoint</i> (if the pointer is valid). If the lines are parallel, the intersection point is undefined.</p></div>
    pub unsafe fn intersect(
        &self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLineF>>,
        intersection_point: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QPointF>>,
    ) -> crate::q_line_f::IntersectType {
        crate::__ffi::ctr_qt_core_ffi_QLineF_intersect(
            self as *const crate::QLineF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLineF>>::cast_into(l).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QPointF>>::cast_into(
                intersection_point,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if the line is not set up with valid start and end point; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLineF::isNull() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#isNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the line is not set up with valid start and end point; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLineF_isNull(self as *const crate::QLineF)
    }

    /// <p>Returns the length of the line.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QLineF::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the length of the line.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlinef.html#setLength">setLength</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QLineF_length(self as *const crate::QLineF)
    }

    /// <p>Constructs a null line.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLineF::QLineF()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#QLineF">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a null line.</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QLineF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLineF_QLineF();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a line object that represents the line between <i>p1</i> and <i>p2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLineF::QLineF(const QPointF& pt1, const QPointF& pt2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#QLineF-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a line object that represents the line between <i>p1</i> and <i>p2</i>.</p></div>
    pub unsafe fn new_2a(
        pt1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
        pt2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) -> ::cpp_utils::CppBox<crate::QLineF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLineF_QLineF1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(pt1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(pt2).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a line object that represents the line between (<i>x1</i>, <i>y1</i>) and (<i>x2</i>, <i>y2</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLineF::QLineF(double x1, double y1, double x2, double y2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#QLineF-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a line object that represents the line between (<i>x1</i>, <i>y1</i>) and (<i>x2</i>, <i>y2</i>).</p></div>
    pub unsafe fn new_4a(
        x1: ::std::os::raw::c_double,
        y1: ::std::os::raw::c_double,
        x2: ::std::os::raw::c_double,
        y2: ::std::os::raw::c_double,
    ) -> ::cpp_utils::CppBox<crate::QLineF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLineF_QLineF2(x1, y1, x2, y2);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Construct a <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> object from the given integer-based <i>line</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLineF::QLineF(const QLine& line)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#QLineF-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Construct a <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> object from the given integer-based <i>line</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlinef.html#toLine">toLine</a>().</p></div>
    pub unsafe fn new_1a(
        line: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLine>>,
    ) -> ::cpp_utils::CppBox<crate::QLineF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLineF_QLineF3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLine>>::cast_into(line).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> class provides a two-dimensional vector using floating point precision.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLineF::QLineF(const QLineF& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> class provides a two-dimensional vector using floating point precision.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> describes a finite length line (or line segment) on a two-dimensional surface. <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> defines the start and end points of the line using floating point accuracy for coordinates. Use the <a href="http://doc.qt.io/qt-5/qlinef.html#toLine">toLine</a>() function to retrieve an integer based copy of this line.</p>
    /// <div class="table"><table class="generic">
    ///  <tbody><tr valign="top" class="odd"><td><img src="http://doc.qt.io/qt-5/images/qline-point.png" alt=""></td><td><img src="http://doc.qt.io/qt-5/images/qline-coordinates.png" alt=""></td></tr>
    /// </tbody></table></div>
    /// <p>The positions of the line's start and end points can be retrieved using the <a href="http://doc.qt.io/qt-5/qlinef.html#p1">p1</a>(), <a href="http://doc.qt.io/qt-5/qlinef.html#x1">x1</a>(), <a href="http://doc.qt.io/qt-5/qlinef.html#y1">y1</a>(), <a href="http://doc.qt.io/qt-5/qlinef.html#p2">p2</a>(), <a href="http://doc.qt.io/qt-5/qlinef.html#x2">x2</a>(), and <a href="http://doc.qt.io/qt-5/qlinef.html#y2">y2</a>() functions. The <a href="http://doc.qt.io/qt-5/qlinef.html#dx">dx</a>() and <a href="http://doc.qt.io/qt-5/qlinef.html#dy">dy</a>() functions return the horizontal and vertical components of the line, respectively.</p>
    /// <p>The line's length can be retrieved using the <a href="http://doc.qt.io/qt-5/qlinef.html#length">length</a>() function, and altered using the <a href="http://doc.qt.io/qt-5/qlinef.html#setLength">setLength</a>() function. Similarly, <a href="http://doc.qt.io/qt-5/qlinef.html#angle">angle</a>() and <a href="http://doc.qt.io/qt-5/qlinef.html#setAngle">setAngle</a>() are respectively used for retrieving and altering the angle of the line. Use the <a href="http://doc.qt.io/qt-5/qlinef.html#isNull">isNull</a>() function to determine whether the <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> represents a valid line or a null line.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qlinef.html#intersect">intersect</a>() function determines the <a href="http://doc.qt.io/qt-5/qlinef.html#IntersectType-enum">IntersectType</a> for this line and a given line, while the <a href="http://doc.qt.io/qt-5/qlinef.html#angleTo">angleTo</a>() function returns the angle between the lines. In addition, the <a href="http://doc.qt.io/qt-5/qlinef.html#unitVector">unitVector</a>() function returns a line that has the same starting point as this line, but with a length of only 1, while the <a href="http://doc.qt.io/qt-5/qlinef.html#normalVector">normalVector</a>() function returns a line that is perpendicular to this line with the same starting point and length.</p>
    /// <p>Finally, the line can be translated a given offset using the <a href="http://doc.qt.io/qt-5/qlinef.html#translate">translate</a>() function, and can be traversed using the <a href="http://doc.qt.io/qt-5/qlinef.html#pointAt">pointAt</a>() function.</p>
    /// <a name="constraints"></a></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLineF>>,
    ) -> ::cpp_utils::CppBox<crate::QLineF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLineF_QLineF4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLineF>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a line that is perpendicular to this line with the same starting point and length.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLineF QLineF::normalVector() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#normalVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a line that is perpendicular to this line with the same starting point and length.</p>
    /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qlinef-normalvector.png" alt=""></p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qlinef.html#unitVector">unitVector</a>().</p></div>
    pub unsafe fn normal_vector(&self) -> ::cpp_utils::CppBox<crate::QLineF> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLineF_normalVector(self as *const crate::QLineF);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the line's start point.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPointF QLineF::p1() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#p1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the line's start point.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlinef.html#setP1">setP1</a>(), <a href="http://doc.qt.io/qt-5/qlinef.html#x1">x1</a>(), <a href="http://doc.qt.io/qt-5/qlinef.html#y1">y1</a>(), and <a href="http://doc.qt.io/qt-5/qlinef.html#p2">p2</a>().</p></div>
    pub unsafe fn p1(&self) -> ::cpp_utils::CppBox<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLineF_p1(self as *const crate::QLineF);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the line's end point.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPointF QLineF::p2() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#p2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the line's end point.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlinef.html#setP2">setP2</a>(), <a href="http://doc.qt.io/qt-5/qlinef.html#x2">x2</a>(), <a href="http://doc.qt.io/qt-5/qlinef.html#y2">y2</a>(), and <a href="http://doc.qt.io/qt-5/qlinef.html#p1">p1</a>().</p></div>
    pub unsafe fn p2(&self) -> ::cpp_utils::CppBox<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLineF_p2(self as *const crate::QLineF);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the point at the parameterized position specified by <i>t</i>. The function returns the line's start point if t = 0, and its end point if t = 1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPointF QLineF::pointAt(double t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#pointAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the point at the parameterized position specified by <i>t</i>. The function returns the line's start point if t = 0, and its end point if t = 1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlinef.html#dx">dx</a>() and <a href="http://doc.qt.io/qt-5/qlinef.html#dy">dy</a>().</p></div>
    pub unsafe fn point_at(
        &self,
        t: ::std::os::raw::c_double,
    ) -> ::cpp_utils::CppBox<crate::QPointF> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLineF_pointAt(self as *const crate::QLineF, t);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Sets the angle of the line to the given <i>angle</i> (in degrees). This will change the position of the second point of the line such that the line has the given angle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QLineF::setAngle(double angle)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#setAngle">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the angle of the line to the given <i>angle</i> (in degrees). This will change the position of the second point of the line such that the line has the given angle.</p>
    /// <p>Positive values for the angles mean counter-clockwise while negative values mean the clockwise direction. Zero degrees is at the 3 o'clock position.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlinef.html#angle">angle</a>().</p></div>
    pub unsafe fn set_angle(&mut self, angle: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_QLineF_setAngle(self as *mut crate::QLineF, angle)
    }

    /// <p>Sets the length of the line to the given <i>length</i>. <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> will move the end point - <a href="http://doc.qt.io/qt-5/qlinef.html#p2">p2</a>() - of the line to give the line its new length.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QLineF::setLength(double len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#setLength">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the length of the line to the given <i>length</i>. <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> will move the end point - <a href="http://doc.qt.io/qt-5/qlinef.html#p2">p2</a>() - of the line to give the line its new length.</p>
    /// <p>If the line is a null line, the length will remain zero regardless of the length specified.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlinef.html#length">length</a>() and <a href="http://doc.qt.io/qt-5/qlinef.html#isNull">isNull</a>().</p></div>
    pub unsafe fn set_length(&mut self, len: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_QLineF_setLength(self as *mut crate::QLineF, len)
    }

    /// <p>Sets this line to the start in <i>x1</i>, <i>y1</i> and end in <i>x2</i>, <i>y2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QLineF::setLine(double x1, double y1, double x2, double y2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#setLine">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets this line to the start in <i>x1</i>, <i>y1</i> and end in <i>x2</i>, <i>y2</i>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlinef.html#setP1">setP1</a>(), <a href="http://doc.qt.io/qt-5/qlinef.html#setP2">setP2</a>(), <a href="http://doc.qt.io/qt-5/qlinef.html#p1">p1</a>(), and <a href="http://doc.qt.io/qt-5/qlinef.html#p2">p2</a>().</p></div>
    pub unsafe fn set_line(
        &mut self,
        x1: ::std::os::raw::c_double,
        y1: ::std::os::raw::c_double,
        x2: ::std::os::raw::c_double,
        y2: ::std::os::raw::c_double,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QLineF_setLine(self as *mut crate::QLineF, x1, y1, x2, y2)
    }

    /// <p>Sets the starting point of this line to <i>p1</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QLineF::setP1(const QPointF& p1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#setP1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the starting point of this line to <i>p1</i>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlinef.html#setP2">setP2</a>() and <a href="http://doc.qt.io/qt-5/qlinef.html#p1">p1</a>().</p></div>
    pub unsafe fn set_p1(
        &mut self,
        p1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QLineF_setP1(
            self as *mut crate::QLineF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(p1).as_raw_ptr(),
        )
    }

    /// <p>Sets the end point of this line to <i>p2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QLineF::setP2(const QPointF& p2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#setP2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the end point of this line to <i>p2</i>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlinef.html#setP1">setP1</a>() and <a href="http://doc.qt.io/qt-5/qlinef.html#p2">p2</a>().</p></div>
    pub unsafe fn set_p2(
        &mut self,
        p2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QLineF_setP2(
            self as *mut crate::QLineF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(p2).as_raw_ptr(),
        )
    }

    /// <p>Sets the start point of this line to <i>p1</i> and the end point of this line to <i>p2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QLineF::setPoints(const QPointF& p1, const QPointF& p2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#setPoints">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the start point of this line to <i>p1</i> and the end point of this line to <i>p2</i>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlinef.html#setP1">setP1</a>(), <a href="http://doc.qt.io/qt-5/qlinef.html#setP2">setP2</a>(), <a href="http://doc.qt.io/qt-5/qlinef.html#p1">p1</a>(), and <a href="http://doc.qt.io/qt-5/qlinef.html#p2">p2</a>().</p></div>
    pub unsafe fn set_points(
        &mut self,
        p1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
        p2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QLineF_setPoints(
            self as *mut crate::QLineF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(p1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(p2).as_raw_ptr(),
        )
    }

    /// <p>Returns an integer based copy of this line.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLine QLineF::toLine() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#toLine">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an integer based copy of this line.</p>
    /// <p>Note that the returned line's start and end points are rounded to the nearest integer.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlinef.html#QLineF">QLineF</a>().</p></div>
    pub unsafe fn to_line(&self) -> ::cpp_utils::CppBox<crate::QLine> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLineF_toLine(self as *const crate::QLineF);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Translates this line by the given <i>offset</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QLineF::translate(const QPointF& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#translate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Translates this line by the given <i>offset</i>.</p></div>
    pub unsafe fn translate_1a(
        &mut self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QLineF_translate(
            self as *mut crate::QLineF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(p).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QLineF::translate(double dx, double dy)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#translate-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Translates this line the distance specified by <i>dx</i> and <i>dy</i>.</p></div>
    pub unsafe fn translate_2a(
        &mut self,
        dx: ::std::os::raw::c_double,
        dy: ::std::os::raw::c_double,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QLineF_translate1(self as *mut crate::QLineF, dx, dy)
    }

    /// <p>Returns this line translated by the given <i>offset</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLineF QLineF::translated(const QPointF& p) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#translated">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns this line translated by the given <i>offset</i>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p></div>
    pub unsafe fn translated_1a(
        &self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) -> ::cpp_utils::CppBox<crate::QLineF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLineF_translated(
            self as *const crate::QLineF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(p).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLineF QLineF::translated(double dx, double dy) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#translated-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns this line translated the distance specified by <i>dx</i> and <i>dy</i>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p></div>
    pub unsafe fn translated_2a(
        &self,
        dx: ::std::os::raw::c_double,
        dy: ::std::os::raw::c_double,
    ) -> ::cpp_utils::CppBox<crate::QLineF> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLineF_translated1(self as *const crate::QLineF, dx, dy);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the unit vector for this line, i.e a line starting at the same point as <i>this</i> line with a length of 1.0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLineF QLineF::unitVector() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#unitVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the unit vector for this line, i.e a line starting at the same point as <i>this</i> line with a length of 1.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlinef.html#normalVector">normalVector</a>().</p></div>
    pub unsafe fn unit_vector(&self) -> ::cpp_utils::CppBox<crate::QLineF> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QLineF_unitVector(self as *const crate::QLineF);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the x-coordinate of the line's start point.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QLineF::x1() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#x1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the x-coordinate of the line's start point.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlinef.html#p1">p1</a>().</p></div>
    pub unsafe fn x1(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QLineF_x1(self as *const crate::QLineF)
    }

    /// <p>Returns the x-coordinate of the line's end point.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QLineF::x2() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#x2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the x-coordinate of the line's end point.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlinef.html#p2">p2</a>().</p></div>
    pub unsafe fn x2(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QLineF_x2(self as *const crate::QLineF)
    }

    /// <p>Returns the y-coordinate of the line's start point.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QLineF::y1() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#y1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the y-coordinate of the line's start point.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlinef.html#p1">p1</a>().</p></div>
    pub unsafe fn y1(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QLineF_y1(self as *const crate::QLineF)
    }

    /// <p>Returns the y-coordinate of the line's end point.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QLineF::y2() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#y2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the y-coordinate of the line's end point.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlinef.html#p2">p2</a>().</p></div>
    pub unsafe fn y2(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QLineF_y2(self as *const crate::QLineF)
    }
}

pub mod q_lock_file {
    //! C++ type: <span style='color: green;'>```QLockFile```</span>

    /// <p>This enum describes the result of the last call to <a href="http://doc.qt.io/qt-5/qlockfile.html#lock">lock</a>() or <a href="http://doc.qt.io/qt-5/qlockfile.html#tryLock">tryLock</a>().</p>
    ///
    /// C++ enum: <span style='color: green;'>```QLockFile::LockError```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlockfile.html#LockError-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the result of the last call to <a href="http://doc.qt.io/qt-5/qlockfile.html#lock">lock</a>() or <a href="http://doc.qt.io/qt-5/qlockfile.html#tryLock">tryLock</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct LockError(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for LockError {
        fn from(value: ::std::os::raw::c_int) -> Self {
            LockError(value)
        }
    }

    impl From<LockError> for ::std::os::raw::c_int {
        fn from(value: LockError) -> Self {
            value.0
        }
    }

    impl LockError {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl LockError {
        /// The lock was acquired successfully. (C++ enum variant: <span style='color: green;'>```NoError = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoError: crate::q_lock_file::LockError = crate::q_lock_file::LockError(0);
        /// The lock could not be acquired because another process holds it. (C++ enum variant: <span style='color: green;'>```LockFailedError = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const LockFailedError: crate::q_lock_file::LockError = crate::q_lock_file::LockError(1);
        /// The lock file could not be created, for lack of permissions in the parent directory. (C++ enum variant: <span style='color: green;'>```PermissionError = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const PermissionError: crate::q_lock_file::LockError = crate::q_lock_file::LockError(2);
        /// Another error happened, for instance a full partition prevented writing out the lock file. (C++ enum variant: <span style='color: green;'>```UnknownError = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const UnknownError: crate::q_lock_file::LockError = crate::q_lock_file::LockError(3);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qlockfile.html">QLockFile</a> class provides locking between processes using a file.</p>
///
/// C++ class: <span style='color: green;'>```QLockFile```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlockfile.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlockfile.html">QLockFile</a> class provides locking between processes using a file.</p>
/// <p>A lock file can be used to prevent multiple processes from accessing concurrently the same resource. For instance, a configuration file on disk, or a socket, a port, a region of shared memory...</p>
/// <p>Serialization is only guaranteed if all processes that access the shared resource use <a href="http://doc.qt.io/qt-5/qlockfile.html">QLockFile</a>, with the same file path.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlockfile.html">QLockFile</a> supports two use cases: to protect a resource for a short-term operation (e.g. verifying if a configuration file has changed before saving new settings), and for long-lived protection of a resource (e.g. a document opened by a user in an editor) for an indefinite amount of time.</p>
/// <p>When protecting for a short-term operation, it is acceptable to call <a href="http://doc.qt.io/qt-5/qlockfile.html#lock">lock</a>() and wait until any running operation finishes. When protecting a resource over a long time, however, the application should always call <a href="http://doc.qt.io/qt-5/qlockfile.html#setStaleLockTime">setStaleLockTime</a>(0) and then <a href="http://doc.qt.io/qt-5/qlockfile.html#tryLock">tryLock</a>() with a short timeout, in order to warn the user that the resource is locked.</p>
/// <p>If the process holding the lock crashes, the lock file stays on disk and can prevent any other process from accessing the shared resource, ever. For this reason, <a href="http://doc.qt.io/qt-5/qlockfile.html">QLockFile</a> tries to detect such a "stale" lock file, based on the process ID written into the file. To cover the situation that the process ID got reused meanwhile, the current process name is compared to the name of the process that corresponds to the process ID from the lock file. If the process names differ, the lock file is considered stale. Additionally, the last modification time of the lock file (30s by default, for the use case of a short-lived operation) is taken into account. If the lock file is found to be stale, it will be deleted.</p>
/// <p>For the use case of protecting a resource over a long time, you should therefore call <a href="http://doc.qt.io/qt-5/qlockfile.html#setStaleLockTime">setStaleLockTime</a>(0), and when <a href="http://doc.qt.io/qt-5/qlockfile.html#tryLock">tryLock</a>() returns <a href="http://doc.qt.io/qt-5/qlockfile.html#LockError-enum">LockFailedError</a>, inform the user that the document is locked, possibly using <a href="http://doc.qt.io/qt-5/qlockfile.html#getLockInfo">getLockInfo</a>() for more details.</p>
/// <p><b>Note: </b>On Windows, this class has problems detecting a stale lock if the machine's hostname contains characters outside the US-ASCII character set.</p></div>
#[repr(C)]
pub struct QLockFile {
    _unused: u8,
}
impl QLockFile {
    /// <p>Returns the lock file error status.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLockFile::LockError QLockFile::error() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlockfile.html#error">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the lock file error status.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qlockfile.html#tryLock">tryLock</a>() returns <code>false</code>, this function can be called to find out the reason why the locking failed.</p></div>
    pub unsafe fn error(&self) -> crate::q_lock_file::LockError {
        crate::__ffi::ctr_qt_core_ffi_QLockFile_error(self as *const crate::QLockFile)
    }

    /// <p>Retrieves information about the current owner of the lock file.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLockFile::getLockInfo(qint64* pid, QString* hostname, QString* appname) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlockfile.html#getLockInfo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Retrieves information about the current owner of the lock file.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qlockfile.html#tryLock">tryLock</a>() returns <code>false</code>, and <a href="http://doc.qt.io/qt-5/qlockfile.html#error">error</a>() returns <a href="http://doc.qt.io/qt-5/qlockfile.html#LockError-enum">LockFailedError</a>, this function can be called to find out more information about the existing lock file:</p>
    /// <ul>
    /// <li>the PID of the application (returned in <i>pid</i>)</li>
    /// <li>the <i>hostname</i> it's running on (useful in case of networked filesystems),</li>
    /// <li>the name of the application which created it (returned in <i>appname</i>),</li>
    /// </ul>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlockfile.html#tryLock">tryLock</a>() automatically deleted the file if there is no running application with this PID, so <a href="http://doc.qt.io/qt-5/qlockfile.html#LockError-enum">LockFailedError</a> can only happen if there is an application with this PID (it could be unrelated though).</p>
    /// <p>This can be used to inform users about the existing lock file and give them the choice to delete it. After removing the file using <a href="http://doc.qt.io/qt-5/qlockfile.html#removeStaleLockFile">removeStaleLockFile</a>(), the application can call <a href="http://doc.qt.io/qt-5/qlockfile.html#tryLock">tryLock</a>() again.</p>
    /// <p>This function returns <code>true</code> if the information could be successfully retrieved, false if the lock file doesn't exist or doesn't contain the expected data. This can happen if the lock file was deleted between the time where <a href="http://doc.qt.io/qt-5/qlockfile.html#tryLock">tryLock</a>() failed and the call to this function. Simply call <a href="http://doc.qt.io/qt-5/qlockfile.html#tryLock">tryLock</a>() again if this happens.</p></div>
    pub unsafe fn get_lock_info(
        &self,
        pid: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<i64>>,
        hostname: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QString>>,
        appname: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLockFile_getLockInfo(
            self as *const crate::QLockFile,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<i64>>::cast_into(pid).as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QString>>::cast_into(hostname)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QString>>::cast_into(appname)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if the lock was acquired by this <a href="http://doc.qt.io/qt-5/qlockfile.html">QLockFile</a> instance, otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLockFile::isLocked() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlockfile.html#isLocked">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the lock was acquired by this <a href="http://doc.qt.io/qt-5/qlockfile.html">QLockFile</a> instance, otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlockfile.html#lock">lock</a>(), <a href="http://doc.qt.io/qt-5/qlockfile.html#unlock">unlock</a>(), and <a href="http://doc.qt.io/qt-5/qlockfile.html#tryLock">tryLock</a>().</p></div>
    pub unsafe fn is_locked(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLockFile_isLocked(self as *const crate::QLockFile)
    }

    /// <p>Creates the lock file.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLockFile::lock()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlockfile.html#lock">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates the lock file.</p>
    /// <p>If another process (or another thread) has created the lock file already, this function will block until that process (or thread) releases it.</p>
    /// <p>Calling this function multiple times on the same lock from the same thread without unlocking first is not allowed. This function will <i>dead-lock</i> when the file is locked recursively.</p>
    /// <p>Returns <code>true</code> if the lock was acquired, false if it could not be acquired due to an unrecoverable error, such as no permissions in the parent directory.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlockfile.html#unlock">unlock</a>() and <a href="http://doc.qt.io/qt-5/qlockfile.html#tryLock">tryLock</a>().</p></div>
    pub unsafe fn lock(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLockFile_lock(self as *mut crate::QLockFile)
    }

    /// <p>Constructs a new lock file object. The object is created in an unlocked state. When calling <a href="http://doc.qt.io/qt-5/qlockfile.html#lock">lock</a>() or <a href="http://doc.qt.io/qt-5/qlockfile.html#tryLock">tryLock</a>(), a lock file named <i>fileName</i> will be created, if it doesn't already exist.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLockFile::QLockFile(const QString& fileName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlockfile.html#QLockFile">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new lock file object. The object is created in an unlocked state. When calling <a href="http://doc.qt.io/qt-5/qlockfile.html#lock">lock</a>() or <a href="http://doc.qt.io/qt-5/qlockfile.html#tryLock">tryLock</a>(), a lock file named <i>fileName</i> will be created, if it doesn't already exist.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlockfile.html#lock">lock</a>() and <a href="http://doc.qt.io/qt-5/qlockfile.html#unlock">unlock</a>().</p></div>
    pub unsafe fn new(
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QLockFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLockFile_QLockFile(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Attempts to forcefully remove an existing lock file.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLockFile::removeStaleLockFile()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlockfile.html#removeStaleLockFile">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to forcefully remove an existing lock file.</p>
    /// <p>Calling this is not recommended when protecting a short-lived operation: <a href="http://doc.qt.io/qt-5/qlockfile.html">QLockFile</a> already takes care of removing lock files after they are older than <a href="http://doc.qt.io/qt-5/qlockfile.html#staleLockTime">staleLockTime</a>().</p>
    /// <p>This method should only be called when protecting a resource for a long time, i.e. with <a href="http://doc.qt.io/qt-5/qlockfile.html#staleLockTime">staleLockTime</a>(0), and after <a href="http://doc.qt.io/qt-5/qlockfile.html#tryLock">tryLock</a>() returned <a href="http://doc.qt.io/qt-5/qlockfile.html#LockError-enum">LockFailedError</a>, and the user agreed on removing the lock file.</p>
    /// <p>Returns <code>true</code> on success, false if the lock file couldn't be removed. This happens on Windows, when the application owning the lock is still running.</p></div>
    pub unsafe fn remove_stale_lock_file(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLockFile_removeStaleLockFile(self as *mut crate::QLockFile)
    }

    /// <p>Sets <i>staleLockTime</i> to be the time in milliseconds after which a lock file is considered stale. The default value is 30000, i.e. 30 seconds. If your application typically keeps the file locked for more than 30 seconds (for instance while saving megabytes of data for 2 minutes), you should set a bigger value using setStaleLockTime().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QLockFile::setStaleLockTime(int arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlockfile.html#setStaleLockTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets <i>staleLockTime</i> to be the time in milliseconds after which a lock file is considered stale. The default value is 30000, i.e. 30 seconds. If your application typically keeps the file locked for more than 30 seconds (for instance while saving megabytes of data for 2 minutes), you should set a bigger value using setStaleLockTime().</p>
    /// <p>The value of <i>staleLockTime</i> is used by <a href="http://doc.qt.io/qt-5/qlockfile.html#lock">lock</a>() and <a href="http://doc.qt.io/qt-5/qlockfile.html#tryLock">tryLock</a>() in order to determine when an existing lock file is considered stale, i.e. left over by a crashed process. This is useful for the case where the PID got reused meanwhile, so one way to detect a stale lock file is by the fact that it has been around for a long time.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlockfile.html#staleLockTime">staleLockTime</a>().</p></div>
    pub unsafe fn set_stale_lock_time(&mut self, arg1: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QLockFile_setStaleLockTime(
            self as *mut crate::QLockFile,
            arg1,
        )
    }

    /// <p>Returns the time in milliseconds after which a lock file is considered stale.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QLockFile::staleLockTime() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlockfile.html#staleLockTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the time in milliseconds after which a lock file is considered stale.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlockfile.html#setStaleLockTime">setStaleLockTime</a>().</p></div>
    pub unsafe fn stale_lock_time(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QLockFile_staleLockTime(self as *const crate::QLockFile)
    }

    /// <p>Attempts to create the lock file. This function returns <code>true</code> if the lock was obtained; otherwise it returns <code>false</code>. If another process (or another thread) has created the lock file already, this function will wait for at most <i>timeout</i> milliseconds for the lock file to become available.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLockFile::tryLock(int timeout = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlockfile.html#tryLock">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to create the lock file. This function returns <code>true</code> if the lock was obtained; otherwise it returns <code>false</code>. If another process (or another thread) has created the lock file already, this function will wait for at most <i>timeout</i> milliseconds for the lock file to become available.</p>
    /// <p>Note: Passing a negative number as the <i>timeout</i> is equivalent to calling <a href="http://doc.qt.io/qt-5/qlockfile.html#lock">lock</a>(), i.e. this function will wait forever until the lock file can be locked if <i>timeout</i> is negative.</p>
    /// <p>If the lock was obtained, it must be released with <a href="http://doc.qt.io/qt-5/qlockfile.html#unlock">unlock</a>() before another process (or thread) can successfully lock it.</p>
    /// <p>Calling this function multiple times on the same lock from the same thread without unlocking first is not allowed, this function will <i>always</i> return false when attempting to lock the file recursively.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlockfile.html#lock">lock</a>() and <a href="http://doc.qt.io/qt-5/qlockfile.html#unlock">unlock</a>().</p></div>
    pub unsafe fn try_lock_1a(&mut self, timeout: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLockFile_tryLock(self as *mut crate::QLockFile, timeout)
    }

    /// <p>Attempts to create the lock file. This function returns <code>true</code> if the lock was obtained; otherwise it returns <code>false</code>. If another process (or another thread) has created the lock file already, this function will wait for at most <i>timeout</i> milliseconds for the lock file to become available.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLockFile::tryLock()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlockfile.html#tryLock">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to create the lock file. This function returns <code>true</code> if the lock was obtained; otherwise it returns <code>false</code>. If another process (or another thread) has created the lock file already, this function will wait for at most <i>timeout</i> milliseconds for the lock file to become available.</p>
    /// <p>Note: Passing a negative number as the <i>timeout</i> is equivalent to calling <a href="http://doc.qt.io/qt-5/qlockfile.html#lock">lock</a>(), i.e. this function will wait forever until the lock file can be locked if <i>timeout</i> is negative.</p>
    /// <p>If the lock was obtained, it must be released with <a href="http://doc.qt.io/qt-5/qlockfile.html#unlock">unlock</a>() before another process (or thread) can successfully lock it.</p>
    /// <p>Calling this function multiple times on the same lock from the same thread without unlocking first is not allowed, this function will <i>always</i> return false when attempting to lock the file recursively.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlockfile.html#lock">lock</a>() and <a href="http://doc.qt.io/qt-5/qlockfile.html#unlock">unlock</a>().</p></div>
    pub unsafe fn try_lock_0a(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLockFile_tryLock1(self as *mut crate::QLockFile)
    }

    /// <p>Releases the lock, by deleting the lock file.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QLockFile::unlock()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlockfile.html#unlock">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Releases the lock, by deleting the lock file.</p>
    /// <p>Calling unlock() without locking the file first, does nothing.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlockfile.html#lock">lock</a>() and <a href="http://doc.qt.io/qt-5/qlockfile.html#tryLock">tryLock</a>().</p></div>
    pub unsafe fn unlock(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QLockFile_unlock(self as *mut crate::QLockFile)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qloggingcategory.html">QLoggingCategory</a> class represents a category, or 'area' in the logging infrastructure.</p>
///
/// C++ class: <span style='color: green;'>```QLoggingCategory```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qloggingcategory.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qloggingcategory.html">QLoggingCategory</a> class represents a category, or 'area' in the logging infrastructure.</p>
/// <p><a href="http://doc.qt.io/qt-5/qloggingcategory.html">QLoggingCategory</a> represents a certain logging category - identified by a string - at runtime. A category can be configured to enable or disable logging of messages per message type. Whether a message type is enabled or not can be checked with the <a href="http://doc.qt.io/qt-5/qloggingcategory.html#isDebugEnabled">isDebugEnabled</a>(), <a href="http://doc.qt.io/qt-5/qloggingcategory.html#isInfoEnabled">isInfoEnabled</a>(), <a href="http://doc.qt.io/qt-5/qloggingcategory.html#isWarningEnabled">isWarningEnabled</a>(), and <a href="http://doc.qt.io/qt-5/qloggingcategory.html#isCriticalEnabled">isCriticalEnabled</a>() methods.</p>
/// <p>All objects are meant to be configured by a common registry (see also <a href="http://doc.qt.io/qt-5/qloggingcategory.html#configuring-categories">Configuring Categories</a>). Different objects can also represent the same category. It is therefore not recommended to export objects across module boundaries, nor to manipulate the objects directly, nor to inherit from <a href="http://doc.qt.io/qt-5/qloggingcategory.html">QLoggingCategory</a>.</p>
/// <a name="creating-category-objects"></a></div>
#[repr(C)]
pub struct QLoggingCategory {
    _unused: u8,
}
impl QLoggingCategory {
    /// <p>Returns the object itself. This allows both a <a href="http://doc.qt.io/qt-5/qloggingcategory.html">QLoggingCategory</a> variable, and a factory method returning a <a href="http://doc.qt.io/qt-5/qloggingcategory.html">QLoggingCategory</a>, to be used in <a href="http://doc.qt.io/qt-5/qloggingcategory.html#qCDebug">qCDebug</a>(), <a href="http://doc.qt.io/qt-5/qloggingcategory.html#qCWarning">qCWarning</a>(), <a href="http://doc.qt.io/qt-5/qloggingcategory.html#qCCritical">qCCritical</a>() macros.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLoggingCategory& QLoggingCategory::operator()()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qloggingcategory.html#operator-28-29">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the object itself. This allows both a <a href="http://doc.qt.io/qt-5/qloggingcategory.html">QLoggingCategory</a> variable, and a factory method returning a <a href="http://doc.qt.io/qt-5/qloggingcategory.html">QLoggingCategory</a>, to be used in <a href="http://doc.qt.io/qt-5/qloggingcategory.html#qCDebug">qCDebug</a>(), <a href="http://doc.qt.io/qt-5/qloggingcategory.html#qCWarning">qCWarning</a>(), <a href="http://doc.qt.io/qt-5/qloggingcategory.html#qCCritical">qCCritical</a>() macros.</p></div>
    pub unsafe fn call_mut(&mut self) -> ::cpp_utils::MutRef<crate::QLoggingCategory> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLoggingCategory_operator__(
            self as *mut crate::QLoggingCategory,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the object itself. This allows both a <a href="http://doc.qt.io/qt-5/qloggingcategory.html">QLoggingCategory</a> variable, and a factory method returning a <a href="http://doc.qt.io/qt-5/qloggingcategory.html">QLoggingCategory</a>, to be used in <a href="http://doc.qt.io/qt-5/qloggingcategory.html#qCDebug">qCDebug</a>(), <a href="http://doc.qt.io/qt-5/qloggingcategory.html#qCWarning">qCWarning</a>(), <a href="http://doc.qt.io/qt-5/qloggingcategory.html#qCCritical">qCCritical</a>() macros.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QLoggingCategory& QLoggingCategory::operator()() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qloggingcategory.html#operator-28-29-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the object itself. This allows both a <a href="http://doc.qt.io/qt-5/qloggingcategory.html">QLoggingCategory</a> variable, and a factory method returning a <a href="http://doc.qt.io/qt-5/qloggingcategory.html">QLoggingCategory</a>, to be used in <a href="http://doc.qt.io/qt-5/qloggingcategory.html#qCDebug">qCDebug</a>(), <a href="http://doc.qt.io/qt-5/qloggingcategory.html#qCWarning">qCWarning</a>(), <a href="http://doc.qt.io/qt-5/qloggingcategory.html#qCCritical">qCCritical</a>() macros.</p></div>
    pub unsafe fn call(&self) -> ::cpp_utils::Ref<crate::QLoggingCategory> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLoggingCategory_operator__1(
            self as *const crate::QLoggingCategory,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the name of the category.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QLoggingCategory::categoryName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qloggingcategory.html#categoryName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the name of the category.</p></div>
    pub unsafe fn category_name(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLoggingCategory_categoryName(
            self as *const crate::QLoggingCategory,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns a pointer to the global category <code>"default"</code> that is used e.g. by <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>(), <a href="http://doc.qt.io/qt-5/qtglobal.html#qInfo">qInfo</a>(), <a href="http://doc.qt.io/qt-5/qtglobal.html#qWarning">qWarning</a>(), <a href="http://doc.qt.io/qt-5/qtglobal.html#qCritical">qCritical</a>(), <a href="http://doc.qt.io/qt-5/qtglobal.html#qFatal">qFatal</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QLoggingCategory* QLoggingCategory::defaultCategory()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qloggingcategory.html#defaultCategory">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the global category <code>"default"</code> that is used e.g. by <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>(), <a href="http://doc.qt.io/qt-5/qtglobal.html#qInfo">qInfo</a>(), <a href="http://doc.qt.io/qt-5/qtglobal.html#qWarning">qWarning</a>(), <a href="http://doc.qt.io/qt-5/qtglobal.html#qCritical">qCritical</a>(), <a href="http://doc.qt.io/qt-5/qtglobal.html#qFatal">qFatal</a>().</p>
    /// <p><b>Note: </b>The returned pointer may be null during destruction of static objects.</p><p><b>Note: </b>Ownership of the category is not transferred, do not <code>delete</code> the returned pointer.</p></div>
    pub unsafe fn default_category() -> ::cpp_utils::MutPtr<crate::QLoggingCategory> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLoggingCategory_defaultCategory();
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Installs a function <i>filter</i> that is used to determine which categories and message types should be enabled. Returns a pointer to the previous installed filter.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void (*FN_PTR)(QLoggingCategory*) QLoggingCategory::installFilter(void (*FN_PTR)(QLoggingCategory*) arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qloggingcategory.html#installFilter">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Installs a function <i>filter</i> that is used to determine which categories and message types should be enabled. Returns a pointer to the previous installed filter.</p>
    /// <p>Every <a href="http://doc.qt.io/qt-5/qloggingcategory.html">QLoggingCategory</a> object created is passed to the filter, and the filter is free to change the respective category configuration with <a href="http://doc.qt.io/qt-5/qloggingcategory.html#setEnabled">setEnabled</a>().</p>
    /// <p>The filter might be called from different threads, but never concurrently. The filter shall not call any static functions of <a href="http://doc.qt.io/qt-5/qloggingcategory.html">QLoggingCategory</a>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qloggingcategory.html#QLoggingCategory">QLoggingCategory</a></span><span class="operator">::</span>CategoryFilter oldCategoryFilter;
    ///
    ///   <span class="type">void</span> myCategoryFilter(<span class="type"><a href="http://doc.qt.io/qt-5/qloggingcategory.html#QLoggingCategory">QLoggingCategory</a></span> <span class="operator">*</span>category)
    ///   {
    /// &#32;     <span class="comment">// configure driver.usb category here, otherwise forward to to default filter.</span>
    /// &#32;     <span class="keyword">if</span> (qstrcmp(category<span class="operator">-</span><span class="operator">&gt;</span>categoryName()<span class="operator">,</span> <span class="string">"driver.usb"</span>) <span class="operator">=</span><span class="operator">=</span> <span class="number">0</span>)
    /// &#32;   &#32;     category<span class="operator">-</span><span class="operator">&gt;</span>setEnabled(<span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#QtMsgType-enum">QtDebugMsg</a></span><span class="operator">,</span> <span class="keyword">true</span>);
    /// &#32;     <span class="keyword">else</span>
    /// &#32;   &#32;     oldCategoryFilter(category);
    ///   }
    ///
    /// </pre>
    /// <p>An alternative way of configuring the default filter is via <a href="http://doc.qt.io/qt-5/qloggingcategory.html#setFilterRules">setFilterRules</a>().</p></div>
    pub unsafe fn install_filter(
        arg1: ::std::option::Option<extern "C" fn(*mut crate::QLoggingCategory)>,
    ) -> ::std::option::Option<extern "C" fn(*mut crate::QLoggingCategory)> {
        crate::__ffi::ctr_qt_core_ffi_QLoggingCategory_installFilter(arg1)
    }

    /// <p>Returns <code>true</code> if critical messages should be shown for this category. Returns <code>false</code> otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLoggingCategory::isCriticalEnabled() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qloggingcategory.html#isCriticalEnabled">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if critical messages should be shown for this category. Returns <code>false</code> otherwise.</p>
    /// <p><b>Note: </b>The <a href="http://doc.qt.io/qt-5/qloggingcategory.html#qCCritical">qCCritical</a>() macro already does this check before executing any code. However, calling this method may be useful to avoid expensive generation of data that is only used for debug output.</p></div>
    pub unsafe fn is_critical_enabled(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLoggingCategory_isCriticalEnabled(
            self as *const crate::QLoggingCategory,
        )
    }

    /// <p>Returns <code>true</code> if debug messages should be shown for this category. Returns <code>false</code> otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLoggingCategory::isDebugEnabled() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qloggingcategory.html#isDebugEnabled">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if debug messages should be shown for this category. Returns <code>false</code> otherwise.</p>
    /// <p><b>Note: </b>The <a href="http://doc.qt.io/qt-5/qloggingcategory.html#qCDebug">qCDebug</a>() macro already does this check before executing any code. However, calling this method may be useful to avoid expensive generation of data that is only used for debug output.</p></div>
    pub unsafe fn is_debug_enabled(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLoggingCategory_isDebugEnabled(
            self as *const crate::QLoggingCategory,
        )
    }

    /// <p>Returns <code>true</code> if a message of type <i>msgtype</i> for the category should be shown. Returns <code>false</code> otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLoggingCategory::isEnabled(QtMsgType type) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qloggingcategory.html#isEnabled">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if a message of type <i>msgtype</i> for the category should be shown. Returns <code>false</code> otherwise.</p></div>
    pub unsafe fn is_enabled(&self, type_: crate::QtMsgType) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLoggingCategory_isEnabled(
            self as *const crate::QLoggingCategory,
            type_,
        )
    }

    /// <p>Returns <code>true</code> if informational messages should be shown for this category. Returns <code>false</code> otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLoggingCategory::isInfoEnabled() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qloggingcategory.html#isInfoEnabled">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if informational messages should be shown for this category. Returns <code>false</code> otherwise.</p>
    /// <p><b>Note: </b>The <a href="http://doc.qt.io/qt-5/qloggingcategory.html#qCInfo">qCInfo</a>() macro already does this check before executing any code. However, calling this method may be useful to avoid expensive generation of data that is only used for debug output.</p><p>This function was introduced in  Qt 5.5.</p></div>
    pub unsafe fn is_info_enabled(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLoggingCategory_isInfoEnabled(
            self as *const crate::QLoggingCategory,
        )
    }

    /// <p>Returns <code>true</code> if warning messages should be shown for this category. Returns <code>false</code> otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLoggingCategory::isWarningEnabled() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qloggingcategory.html#isWarningEnabled">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if warning messages should be shown for this category. Returns <code>false</code> otherwise.</p>
    /// <p><b>Note: </b>The <a href="http://doc.qt.io/qt-5/qloggingcategory.html#qCWarning">qCWarning</a>() macro already does this check before executing any code. However, calling this method may be useful to avoid expensive generation of data that is only used for debug output.</p></div>
    pub unsafe fn is_warning_enabled(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QLoggingCategory_isWarningEnabled(
            self as *const crate::QLoggingCategory,
        )
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qloggingcategory.html">QLoggingCategory</a> object with the provided <i>category</i> name. All message types for this category are enabled by default.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLoggingCategory::QLoggingCategory(const char* category)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qloggingcategory.html#QLoggingCategory">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qloggingcategory.html">QLoggingCategory</a> object with the provided <i>category</i> name. All message types for this category are enabled by default.</p>
    /// <p>If <i>category</i> is <code>0</code>, the category name is changed to <code>"default"</code>.</p></div>
    pub unsafe fn new_1a(
        category: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QLoggingCategory> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLoggingCategory_QLoggingCategory(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(category)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qloggingcategory.html">QLoggingCategory</a> object with the provided <i>category</i> name, and enables all messages with types more severe or equal than <i>enableForLevel</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QLoggingCategory::QLoggingCategory(const char* category, QtMsgType severityLevel)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qloggingcategory.html#QLoggingCategory-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qloggingcategory.html">QLoggingCategory</a> object with the provided <i>category</i> name, and enables all messages with types more severe or equal than <i>enableForLevel</i>.</p>
    /// <p>If <i>category</i> is <code>0</code>, the category name is changed to <code>"default"</code>.</p>
    /// <p>This function was introduced in  Qt 5.4.</p></div>
    pub unsafe fn new_2a(
        category: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        severity_level: crate::QtMsgType,
    ) -> ::cpp_utils::CppBox<crate::QLoggingCategory> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QLoggingCategory_QLoggingCategory1(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(category)
                .as_raw_ptr(),
            severity_level,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Changes the message type <i>type</i> for the category to <i>enable</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QLoggingCategory::setEnabled(QtMsgType type, bool enable)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qloggingcategory.html#setEnabled">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Changes the message type <i>type</i> for the category to <i>enable</i>.</p>
    /// <p>This method is meant to be used only from inside a filter installed by <a href="http://doc.qt.io/qt-5/qloggingcategory.html#installFilter">installFilter</a>(). See <a href="http://doc.qt.io/qt-5/qloggingcategory.html#configuring-categories">Configuring Categories</a> for an overview on how to configure categories globally.</p>
    /// <p><b>Note: </b><code>QtFatalMsg</code> cannot be changed. It will always remain <code>true</code>.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qloggingcategory.html#isEnabled">isEnabled</a>().</p></div>
    pub unsafe fn set_enabled(&mut self, type_: crate::QtMsgType, enable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QLoggingCategory_setEnabled(
            self as *mut crate::QLoggingCategory,
            type_,
            enable,
        )
    }

    /// <p>Configures which categories and message types should be enabled through a a set of <i>rules</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QLoggingCategory::setFilterRules(const QString& rules)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qloggingcategory.html#setFilterRules">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Configures which categories and message types should be enabled through a a set of <i>rules</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qloggingcategory.html#QLoggingCategory">QLoggingCategory</a></span><span class="operator">::</span>setFilterRules(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QStringLiteral">QStringLiteral</a></span>(<span class="string">"driver.usb.debug=true"</span>));
    ///
    /// </pre>
    /// <p><b>Note: </b>The rules might be ignored if a custom category filter is installed with <a href="http://doc.qt.io/qt-5/qloggingcategory.html#installFilter">installFilter</a>(), or if the user defined <code>QT_LOGGING_CONF</code> or <code>QT_LOGGING_RULES</code> environment variable.</p></div>
    pub unsafe fn set_filter_rules(
        rules: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QLoggingCategory_setFilterRules(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(rules)
                .as_raw_ptr(),
        )
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qmargins.html">QMargins</a> class defines the four margins of a rectangle.</p>
///
/// C++ class: <span style='color: green;'>```QMargins```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qmargins.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmargins.html">QMargins</a> class defines the four margins of a rectangle.</p>
/// <p>QMargin defines a set of four margins; left, top, right and bottom, that describe the size of the borders surrounding a rectangle.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qmargins.html#isNull">isNull</a>() function returns <code>true</code> only if all margins are set to zero.</p>
/// <p>QMargin objects can be streamed as well as compared.</p></div>
#[repr(C)]
pub struct QMargins {
    _unused: u8,
}
impl QMargins {
    /// <p>Returns the bottom margin.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMargins::bottom() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmargins.html#bottom">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the bottom margin.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmargins.html#setBottom">setBottom</a>().</p></div>
    pub unsafe fn bottom(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMargins_bottom(self as *const crate::QMargins)
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qmargins.html">QMargins</a> class defines the four margins of a rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMargins& QMargins::operator=(const QMargins& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmargins.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmargins.html">QMargins</a> class defines the four margins of a rectangle.</p>
    /// <p>QMargin defines a set of four margins; left, top, right and bottom, that describe the size of the borders surrounding a rectangle.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qmargins.html#isNull">isNull</a>() function returns <code>true</code> only if all margins are set to zero.</p>
    /// <p>QMargin objects can be streamed as well as compared.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMargins>>,
    ) -> ::cpp_utils::MutRef<crate::QMargins> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMargins_operator_(
            self as *mut crate::QMargins,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMargins>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if all margins are is 0; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMargins::isNull() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmargins.html#isNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if all margins are is 0; otherwise returns false.</p></div>
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMargins_isNull(self as *const crate::QMargins)
    }

    /// <p>Returns the left margin.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMargins::left() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmargins.html#left">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the left margin.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmargins.html#setLeft">setLeft</a>().</p></div>
    pub unsafe fn left(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMargins_left(self as *const crate::QMargins)
    }

    /// <p>Constructs a margins object with all margins set to 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMargins::QMargins()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmargins.html#QMargins">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a margins object with all margins set to 0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmargins.html#isNull">isNull</a>().</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QMargins> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMargins_QMargins();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs margins with the given <i>left</i>, <i>top</i>, <i>right</i>, <i>bottom</i></p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMargins::QMargins(int left, int top, int right, int bottom)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmargins.html#QMargins-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs margins with the given <i>left</i>, <i>top</i>, <i>right</i>, <i>bottom</i></p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmargins.html#setLeft">setLeft</a>(), <a href="http://doc.qt.io/qt-5/qmargins.html#setRight">setRight</a>(), <a href="http://doc.qt.io/qt-5/qmargins.html#setTop">setTop</a>(), and <a href="http://doc.qt.io/qt-5/qmargins.html#setBottom">setBottom</a>().</p></div>
    pub unsafe fn new_4a(
        left: ::std::os::raw::c_int,
        top: ::std::os::raw::c_int,
        right: ::std::os::raw::c_int,
        bottom: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QMargins> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMargins_QMargins1(left, top, right, bottom);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qmargins.html">QMargins</a> class defines the four margins of a rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMargins::QMargins(const QMargins& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmargins.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmargins.html">QMargins</a> class defines the four margins of a rectangle.</p>
    /// <p>QMargin defines a set of four margins; left, top, right and bottom, that describe the size of the borders surrounding a rectangle.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qmargins.html#isNull">isNull</a>() function returns <code>true</code> only if all margins are set to zero.</p>
    /// <p>QMargin objects can be streamed as well as compared.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMargins>>,
    ) -> ::cpp_utils::CppBox<crate::QMargins> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMargins_QMargins2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMargins>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the right margin.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMargins::right() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmargins.html#right">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the right margin.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmargins.html#setRight">setRight</a>().</p></div>
    pub unsafe fn right(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMargins_right(self as *const crate::QMargins)
    }

    /// <p>Sets the bottom margin to <i>bottom</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMargins::setBottom(int bottom)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmargins.html#setBottom">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the bottom margin to <i>bottom</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmargins.html#bottom">bottom</a>().</p></div>
    pub unsafe fn set_bottom(&mut self, bottom: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QMargins_setBottom(self as *mut crate::QMargins, bottom)
    }

    /// <p>Sets the left margin to <i>left</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMargins::setLeft(int left)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmargins.html#setLeft">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the left margin to <i>left</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmargins.html#left">left</a>().</p></div>
    pub unsafe fn set_left(&mut self, left: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QMargins_setLeft(self as *mut crate::QMargins, left)
    }

    /// <p>Sets the right margin to <i>right</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMargins::setRight(int right)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmargins.html#setRight">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the right margin to <i>right</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmargins.html#right">right</a>().</p></div>
    pub unsafe fn set_right(&mut self, right: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QMargins_setRight(self as *mut crate::QMargins, right)
    }

    /// <p>Sets the Top margin to <i>Top</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMargins::setTop(int top)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmargins.html#setTop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the Top margin to <i>Top</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmargins.html#top">top</a>().</p></div>
    pub unsafe fn set_top(&mut self, top: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QMargins_setTop(self as *mut crate::QMargins, top)
    }

    /// <p>Returns the top margin.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMargins::top() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmargins.html#top">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the top margin.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmargins.html#setTop">setTop</a>().</p></div>
    pub unsafe fn top(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMargins_top(self as *const crate::QMargins)
    }

    /// Calls C++ function: <span style='color: green;'>```QMargins operator+(const QMargins& margins)```</span>.
    pub unsafe fn unary_plus(&self) -> ::cpp_utils::CppBox<crate::QMargins> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_operator_99(self as *const crate::QMargins);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qmarginsf.html">QMarginsF</a> class defines the four margins of a rectangle.</p>
///
/// C++ class: <span style='color: green;'>```QMarginsF```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qmarginsf.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmarginsf.html">QMarginsF</a> class defines the four margins of a rectangle.</p>
/// <p><a href="http://doc.qt.io/qt-5/qmarginsf.html">QMarginsF</a> defines a set of four margins; left, top, right and bottom, that describe the size of the borders surrounding a rectangle.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qmarginsf.html#isNull">isNull</a>() function returns <code>true</code> only if all margins are set to zero.</p>
/// <p><a href="http://doc.qt.io/qt-5/qmarginsf.html">QMarginsF</a> objects can be streamed as well as compared.</p></div>
#[repr(C)]
pub struct QMarginsF {
    _unused: u8,
}
impl QMarginsF {
    /// <p>Returns the bottom margin.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QMarginsF::bottom() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmarginsf.html#bottom">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the bottom margin.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmarginsf.html#setBottom">setBottom</a>().</p></div>
    pub unsafe fn bottom(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QMarginsF_bottom(self as *const crate::QMarginsF)
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qmarginsf.html">QMarginsF</a> class defines the four margins of a rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMarginsF& QMarginsF::operator=(const QMarginsF& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmarginsf.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmarginsf.html">QMarginsF</a> class defines the four margins of a rectangle.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qmarginsf.html">QMarginsF</a> defines a set of four margins; left, top, right and bottom, that describe the size of the borders surrounding a rectangle.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qmarginsf.html#isNull">isNull</a>() function returns <code>true</code> only if all margins are set to zero.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qmarginsf.html">QMarginsF</a> objects can be streamed as well as compared.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMarginsF>>,
    ) -> ::cpp_utils::MutRef<crate::QMarginsF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMarginsF_operator_(
            self as *mut crate::QMarginsF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMarginsF>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if all margins are 0; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMarginsF::isNull() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmarginsf.html#isNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if all margins are 0; otherwise returns false.</p></div>
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMarginsF_isNull(self as *const crate::QMarginsF)
    }

    /// <p>Returns the left margin.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QMarginsF::left() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmarginsf.html#left">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the left margin.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmarginsf.html#setLeft">setLeft</a>().</p></div>
    pub unsafe fn left(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QMarginsF_left(self as *const crate::QMarginsF)
    }

    /// <p>Constructs a margins object with all margins set to 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMarginsF::QMarginsF()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmarginsf.html#QMarginsF">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a margins object with all margins set to 0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmarginsf.html#isNull">isNull</a>().</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QMarginsF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMarginsF_QMarginsF();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs margins with the given <i>left</i>, <i>top</i>, <i>right</i>, <i>bottom</i></p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMarginsF::QMarginsF(double left, double top, double right, double bottom)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmarginsf.html#QMarginsF-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs margins with the given <i>left</i>, <i>top</i>, <i>right</i>, <i>bottom</i></p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmarginsf.html#setLeft">setLeft</a>(), <a href="http://doc.qt.io/qt-5/qmarginsf.html#setRight">setRight</a>(), <a href="http://doc.qt.io/qt-5/qmarginsf.html#setTop">setTop</a>(), and <a href="http://doc.qt.io/qt-5/qmarginsf.html#setBottom">setBottom</a>().</p></div>
    pub unsafe fn new_4a(
        left: ::std::os::raw::c_double,
        top: ::std::os::raw::c_double,
        right: ::std::os::raw::c_double,
        bottom: ::std::os::raw::c_double,
    ) -> ::cpp_utils::CppBox<crate::QMarginsF> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMarginsF_QMarginsF1(left, top, right, bottom);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs margins copied from the given <i>margins</i></p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMarginsF::QMarginsF(const QMargins& margins)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmarginsf.html#QMarginsF-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs margins copied from the given <i>margins</i></p></div>
    pub unsafe fn new_1a(
        margins: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMargins>>,
    ) -> ::cpp_utils::CppBox<crate::QMarginsF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMarginsF_QMarginsF2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMargins>>::cast_into(margins)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qmarginsf.html">QMarginsF</a> class defines the four margins of a rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMarginsF::QMarginsF(const QMarginsF& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmarginsf.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmarginsf.html">QMarginsF</a> class defines the four margins of a rectangle.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qmarginsf.html">QMarginsF</a> defines a set of four margins; left, top, right and bottom, that describe the size of the borders surrounding a rectangle.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qmarginsf.html#isNull">isNull</a>() function returns <code>true</code> only if all margins are set to zero.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qmarginsf.html">QMarginsF</a> objects can be streamed as well as compared.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMarginsF>>,
    ) -> ::cpp_utils::CppBox<crate::QMarginsF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMarginsF_QMarginsF3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMarginsF>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the right margin.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QMarginsF::right() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmarginsf.html#right">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the right margin.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmarginsf.html#setRight">setRight</a>().</p></div>
    pub unsafe fn right(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QMarginsF_right(self as *const crate::QMarginsF)
    }

    /// <p>Sets the bottom margin to <i>bottom</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMarginsF::setBottom(double bottom)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmarginsf.html#setBottom">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the bottom margin to <i>bottom</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmarginsf.html#bottom">bottom</a>().</p></div>
    pub unsafe fn set_bottom(&mut self, bottom: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_QMarginsF_setBottom(self as *mut crate::QMarginsF, bottom)
    }

    /// <p>Sets the left margin to <i>left</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMarginsF::setLeft(double left)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmarginsf.html#setLeft">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the left margin to <i>left</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmarginsf.html#left">left</a>().</p></div>
    pub unsafe fn set_left(&mut self, left: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_QMarginsF_setLeft(self as *mut crate::QMarginsF, left)
    }

    /// <p>Sets the right margin to <i>right</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMarginsF::setRight(double right)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmarginsf.html#setRight">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the right margin to <i>right</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmarginsf.html#right">right</a>().</p></div>
    pub unsafe fn set_right(&mut self, right: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_QMarginsF_setRight(self as *mut crate::QMarginsF, right)
    }

    /// <p>Sets the Top margin to <i>Top</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMarginsF::setTop(double top)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmarginsf.html#setTop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the Top margin to <i>Top</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmarginsf.html#top">top</a>().</p></div>
    pub unsafe fn set_top(&mut self, top: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_QMarginsF_setTop(self as *mut crate::QMarginsF, top)
    }

    /// <p>Returns an integer based copy of this margins object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMargins QMarginsF::toMargins() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmarginsf.html#toMargins">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an integer based copy of this margins object.</p>
    /// <p>Note that the components in the returned margins will be rounded to the nearest integer.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmarginsf.html#QMarginsF">QMarginsF</a>().</p></div>
    pub unsafe fn to_margins(&self) -> ::cpp_utils::CppBox<crate::QMargins> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMarginsF_toMargins(self as *const crate::QMarginsF);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the top margin.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QMarginsF::top() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmarginsf.html#top">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the top margin.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmarginsf.html#setTop">setTop</a>().</p></div>
    pub unsafe fn top(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QMarginsF_top(self as *const crate::QMarginsF)
    }

    /// Calls C++ function: <span style='color: green;'>```QMarginsF operator+(const QMarginsF& margins)```</span>.
    pub unsafe fn unary_plus(&self) -> ::cpp_utils::CppBox<crate::QMarginsF> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_operator_109(self as *const crate::QMarginsF);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qmessageauthenticationcode.html">QMessageAuthenticationCode</a> class provides a way to generate hash-based message authentication codes.</p>
///
/// C++ class: <span style='color: green;'>```QMessageAuthenticationCode```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qmessageauthenticationcode.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmessageauthenticationcode.html">QMessageAuthenticationCode</a> class provides a way to generate hash-based message authentication codes.</p>
/// <p><a href="http://doc.qt.io/qt-5/qmessageauthenticationcode.html">QMessageAuthenticationCode</a> supports all cryptographic hashes which are supported by <a href="http://doc.qt.io/qt-5/qcryptographichash.html">QCryptographicHash</a>.</p>
/// <p>To generate message authentication code, pass hash algorithm <a href="http://doc.qt.io/qt-5/qcryptographichash.html#Algorithm-enum">QCryptographicHash::Algorithm</a> to constructor, then set key and message by <a href="http://doc.qt.io/qt-5/qmessageauthenticationcode.html#setKey">setKey</a>() and <a href="http://doc.qt.io/qt-5/qmessageauthenticationcode.html#addData">addData</a>() functions. Result can be acquired by <a href="http://doc.qt.io/qt-5/qmessageauthenticationcode.html#result">result</a>() function.</p>
/// <pre class="cpp">
///
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span> key <span class="operator">=</span> <span class="string">"key"</span>;
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span> message <span class="operator">=</span> <span class="string">"The quick brown fox jumps over the lazy dog"</span>;
/// &#32;     ...
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qmessageauthenticationcode.html#QMessageAuthenticationCode">QMessageAuthenticationCode</a></span> code(<span class="type"><a href="http://doc.qt.io/qt-5/qcryptographichash.html">QCryptographicHash</a></span><span class="operator">::</span>Sha1);
/// &#32;     code<span class="operator">.</span>setKey(key);
/// &#32;     code<span class="operator">.</span>addData(message);
/// &#32;     code<span class="operator">.</span>result()<span class="operator">.</span>toHex();&#32;     <span class="comment">// returns "de7c9b85b8b78aa6bc8a7a36f70a90701c9db4d9"</span>
///
/// </pre>
/// <p>Alternatively, this effect can be achieved by providing message, key and method to <a href="http://doc.qt.io/qt-5/qmessageauthenticationcode.html#hash">hash</a>() method.</p>
/// <pre class="cpp">
///
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qmessageauthenticationcode.html#QMessageAuthenticationCode">QMessageAuthenticationCode</a></span><span class="operator">::</span>hash(message<span class="operator">,</span> key<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcryptographichash.html">QCryptographicHash</a></span><span class="operator">::</span>Sha1)<span class="operator">.</span>toHex();
///
/// </pre></div>
#[repr(C)]
pub struct QMessageAuthenticationCode {
    _unused: u8,
}
impl QMessageAuthenticationCode {
    /// <p>Adds the first <i>length</i> chars of <i>data</i> to the message.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMessageAuthenticationCode::addData(const char* data, int length)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmessageauthenticationcode.html#addData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds the first <i>length</i> chars of <i>data</i> to the message.</p></div>
    pub unsafe fn add_data_char_int(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        length: ::std::os::raw::c_int,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QMessageAuthenticationCode_addData(
            self as *mut crate::QMessageAuthenticationCode,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(data)
                .as_raw_ptr(),
            length,
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qmessageauthenticationcode.html#addData">addData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMessageAuthenticationCode::addData(const QByteArray& data)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmessageauthenticationcode.html#addData-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qmessageauthenticationcode.html#addData">addData</a>().</p></div>
    pub unsafe fn add_data_q_byte_array(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QMessageAuthenticationCode_addData1(
            self as *mut crate::QMessageAuthenticationCode,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(data)
                .as_raw_ptr(),
        )
    }

    /// <p>Reads the data from the open <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> <i>device</i> until it ends and adds it to message. Returns <code>true</code> if reading was successful.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMessageAuthenticationCode::addData(QIODevice* device)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmessageauthenticationcode.html#addData-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads the data from the open <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> <i>device</i> until it ends and adds it to message. Returns <code>true</code> if reading was successful.</p>
    /// <p><b>Note: </b><i>device</i> must be already opened.</p></div>
    pub unsafe fn add_data_q_io_device(
        &mut self,
        device: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QIODevice>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMessageAuthenticationCode_addData2(
            self as *mut crate::QMessageAuthenticationCode,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QIODevice>>::cast_into(device)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the authentication code for the message <i>message</i> using the key <i>key</i> and the method <i>method</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QMessageAuthenticationCode::hash(const QByteArray& message, const QByteArray& key, QCryptographicHash::Algorithm method)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmessageauthenticationcode.html#hash">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the authentication code for the message <i>message</i> using the key <i>key</i> and the method <i>method</i>.</p></div>
    pub unsafe fn hash(
        message: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        method: crate::q_cryptographic_hash::Algorithm,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMessageAuthenticationCode_hash(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(message)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(key)
                .as_raw_ptr(),
            method,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs an object that can be used to create a cryptographic hash from data using method <i>method</i> and key <i>key</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMessageAuthenticationCode::QMessageAuthenticationCode(QCryptographicHash::Algorithm method, const QByteArray& key = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmessageauthenticationcode.html#QMessageAuthenticationCode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an object that can be used to create a cryptographic hash from data using method <i>method</i> and key <i>key</i>.</p></div>
    pub unsafe fn new_2a(
        method: crate::q_cryptographic_hash::Algorithm,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QMessageAuthenticationCode> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMessageAuthenticationCode_QMessageAuthenticationCode(
                method,
                ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(key)
                    .as_raw_ptr(),
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs an object that can be used to create a cryptographic hash from data using method <i>method</i> and key <i>key</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMessageAuthenticationCode::QMessageAuthenticationCode(QCryptographicHash::Algorithm method)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmessageauthenticationcode.html#QMessageAuthenticationCode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an object that can be used to create a cryptographic hash from data using method <i>method</i> and key <i>key</i>.</p></div>
    pub unsafe fn new_1a(
        method: crate::q_cryptographic_hash::Algorithm,
    ) -> ::cpp_utils::CppBox<crate::QMessageAuthenticationCode> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMessageAuthenticationCode_QMessageAuthenticationCode2(
                method,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Resets message data. Calling this method doesn't affect the key.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMessageAuthenticationCode::reset()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmessageauthenticationcode.html#reset">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Resets message data. Calling this method doesn't affect the key.</p></div>
    pub unsafe fn reset(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMessageAuthenticationCode_reset(
            self as *mut crate::QMessageAuthenticationCode,
        )
    }

    /// <p>Returns the final authentication code.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QMessageAuthenticationCode::result() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmessageauthenticationcode.html#result">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the final authentication code.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#toHex">QByteArray::toHex</a>().</p></div>
    pub unsafe fn result(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMessageAuthenticationCode_result(
            self as *const crate::QMessageAuthenticationCode,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Sets secret <i>key</i>. Calling this method automatically resets the object state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMessageAuthenticationCode::setKey(const QByteArray& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmessageauthenticationcode.html#setKey">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets secret <i>key</i>. Calling this method automatically resets the object state.</p></div>
    pub unsafe fn set_key(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QMessageAuthenticationCode_setKey(
            self as *mut crate::QMessageAuthenticationCode,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(key)
                .as_raw_ptr(),
        )
    }
}

pub mod q_meta_method {
    //! C++ type: <span style='color: green;'>```QMetaMethod```</span>

    /// <p>This enum describes the access level of a method, following the conventions used in C++.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QMetaMethod::Access```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetamethod.html#Access-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the access level of a method, following the conventions used in C++.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Access(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Access {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Access(value)
        }
    }

    impl From<Access> for ::std::os::raw::c_int {
        fn from(value: Access) -> Self {
            value.0
        }
    }

    impl Access {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Access {
        /// C++ enum variant: <span style='color: green;'>```Private = 0```</span>
        #[allow(non_upper_case_globals)]
        pub const Private: crate::q_meta_method::Access = crate::q_meta_method::Access(0);
        /// C++ enum variant: <span style='color: green;'>```Protected = 1```</span>
        #[allow(non_upper_case_globals)]
        pub const Protected: crate::q_meta_method::Access = crate::q_meta_method::Access(1);
        /// C++ enum variant: <span style='color: green;'>```Public = 2```</span>
        #[allow(non_upper_case_globals)]
        pub const Public: crate::q_meta_method::Access = crate::q_meta_method::Access(2);
    }

    /// C++ enum: <span style='color: green;'>```QMetaMethod::MethodType```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetamethod.html#MethodType-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct MethodType(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for MethodType {
        fn from(value: ::std::os::raw::c_int) -> Self {
            MethodType(value)
        }
    }

    impl From<MethodType> for ::std::os::raw::c_int {
        fn from(value: MethodType) -> Self {
            value.0
        }
    }

    impl MethodType {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl MethodType {
        /// The function is a plain member function. (C++ enum variant: <span style='color: green;'>```Method = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const Method: crate::q_meta_method::MethodType = crate::q_meta_method::MethodType(0);
        /// The function is a signal. (C++ enum variant: <span style='color: green;'>```Signal = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Signal: crate::q_meta_method::MethodType = crate::q_meta_method::MethodType(1);
        /// The function is a slot. (C++ enum variant: <span style='color: green;'>```Slot = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const Slot: crate::q_meta_method::MethodType = crate::q_meta_method::MethodType(2);
        /// The function is a constructor. (C++ enum variant: <span style='color: green;'>```Constructor = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const Constructor: crate::q_meta_method::MethodType =
            crate::q_meta_method::MethodType(3);
    }

    /// C++ enum: <span style='color: green;'>```QMetaMethod::Attributes```</span>.
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Attributes(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Attributes {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Attributes(value)
        }
    }

    impl From<Attributes> for ::std::os::raw::c_int {
        fn from(value: Attributes) -> Self {
            value.0
        }
    }

    impl Attributes {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Attributes {
        /// C++ enum variant: <span style='color: green;'>```Compatibility = 1```</span>
        #[allow(non_upper_case_globals)]
        pub const Compatibility: crate::q_meta_method::Attributes =
            crate::q_meta_method::Attributes(1);
        /// C++ enum variant: <span style='color: green;'>```Cloned = 2```</span>
        #[allow(non_upper_case_globals)]
        pub const Cloned: crate::q_meta_method::Attributes = crate::q_meta_method::Attributes(2);
        /// C++ enum variant: <span style='color: green;'>```Scriptable = 4```</span>
        #[allow(non_upper_case_globals)]
        pub const Scriptable: crate::q_meta_method::Attributes =
            crate::q_meta_method::Attributes(4);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qmetamethod.html">QMetaMethod</a> class provides meta-data about a member function.</p>
///
/// C++ class: <span style='color: green;'>```QMetaMethod```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qmetamethod.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmetamethod.html">QMetaMethod</a> class provides meta-data about a member function.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qmetamethod.html">QMetaMethod</a> has a <a href="http://doc.qt.io/qt-5/qmetamethod.html#methodType">methodType</a>(), a <a href="http://doc.qt.io/qt-5/qmetamethod.html#methodSignature">methodSignature</a>(), a list of <a href="http://doc.qt.io/qt-5/qmetamethod.html#parameterTypes">parameterTypes</a>() and <a href="http://doc.qt.io/qt-5/qmetamethod.html#parameterNames">parameterNames</a>(), a return <a href="http://doc.qt.io/qt-5/qmetamethod.html#typeName">typeName</a>(), a <a href="http://doc.qt.io/qt-5/qmetamethod.html#tag">tag</a>(), and an <a href="http://doc.qt.io/qt-5/qmetamethod.html#access">access</a>() specifier. You can use <a href="http://doc.qt.io/qt-5/qmetamethod.html#invoke">invoke</a>() to invoke the method on an arbitrary <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>.</p></div>
#[repr(C)]
pub struct QMetaMethod {
    _unused: u8,
}
impl QMetaMethod {
    /// <p>Returns the access specification of this method (private, protected, or public).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMetaMethod::Access QMetaMethod::access() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetamethod.html#access">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the access specification of this method (private, protected, or public).</p>
    /// <p><b>Note: </b>Signals are always public, but you should regard that as an implementation detail. It is almost always a bad idea to emit a signal from outside its class.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetamethod.html#methodType">methodType</a>().</p></div>
    pub unsafe fn access(&self) -> crate::q_meta_method::Access {
        crate::__ffi::ctr_qt_core_ffi_QMetaMethod_access(self as *const crate::QMetaMethod)
    }

    /// Calls C++ function: <span style='color: green;'>```int QMetaMethod::attributes() const```</span>.
    pub unsafe fn attributes(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaMethod_attributes(self as *const crate::QMetaMethod)
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qmetamethod.html">QMetaMethod</a> class provides meta-data about a member function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMetaMethod& QMetaMethod::operator=(const QMetaMethod& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetamethod.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmetamethod.html">QMetaMethod</a> class provides meta-data about a member function.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qmetamethod.html">QMetaMethod</a> has a <a href="http://doc.qt.io/qt-5/qmetamethod.html#methodType">methodType</a>(), a <a href="http://doc.qt.io/qt-5/qmetamethod.html#methodSignature">methodSignature</a>(), a list of <a href="http://doc.qt.io/qt-5/qmetamethod.html#parameterTypes">parameterTypes</a>() and <a href="http://doc.qt.io/qt-5/qmetamethod.html#parameterNames">parameterNames</a>(), a return <a href="http://doc.qt.io/qt-5/qmetamethod.html#typeName">typeName</a>(), a <a href="http://doc.qt.io/qt-5/qmetamethod.html#tag">tag</a>(), and an <a href="http://doc.qt.io/qt-5/qmetamethod.html#access">access</a>() specifier. You can use <a href="http://doc.qt.io/qt-5/qmetamethod.html#invoke">invoke</a>() to invoke the method on an arbitrary <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMetaMethod>>,
    ) -> ::cpp_utils::MutRef<crate::QMetaMethod> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaMethod_operator_(
            self as *mut crate::QMetaMethod,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMetaMethod>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```const QMetaObject* QMetaMethod::enclosingMetaObject() const```</span>.
    pub unsafe fn enclosing_meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaMethod_enclosingMetaObject(
            self as *const crate::QMetaMethod,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```void QMetaMethod::getParameterTypes(int* types) const```</span>.
    pub unsafe fn get_parameter_types(
        &self,
        types: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_int>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QMetaMethod_getParameterTypes(
            self as *const crate::QMetaMethod,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_int>>::cast_into(types)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if this method is valid (can be introspected and invoked), otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaMethod::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetamethod.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this method is valid (can be introspected and invoked), otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn is_valid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaMethod_isValid(self as *const crate::QMetaMethod)
    }

    /// <p>Returns this method's index.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaMethod::methodIndex() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetamethod.html#methodIndex">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns this method's index.</p>
    /// <p>This function was introduced in  Qt 4.6.</p></div>
    pub unsafe fn method_index(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaMethod_methodIndex(self as *const crate::QMetaMethod)
    }

    /// <p>Returns the signature of this method (e.g., <code>setValue(double)</code>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QMetaMethod::methodSignature() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetamethod.html#methodSignature">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the signature of this method (e.g., <code>setValue(double)</code>).</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetamethod.html#parameterTypes">parameterTypes</a>() and <a href="http://doc.qt.io/qt-5/qmetamethod.html#parameterNames">parameterNames</a>().</p></div>
    pub unsafe fn method_signature(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaMethod_methodSignature(
            self as *const crate::QMetaMethod,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the type of this method (signal, slot, or method).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMetaMethod::MethodType QMetaMethod::methodType() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetamethod.html#methodType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the type of this method (signal, slot, or method).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetamethod.html#access">access</a>().</p></div>
    pub unsafe fn method_type(&self) -> crate::q_meta_method::MethodType {
        crate::__ffi::ctr_qt_core_ffi_QMetaMethod_methodType(self as *const crate::QMetaMethod)
    }

    /// <p>Returns the name of this method.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QMetaMethod::name() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetamethod.html#name">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the name of this method.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetamethod.html#methodSignature">methodSignature</a>() and <a href="http://doc.qt.io/qt-5/qmetamethod.html#parameterCount">parameterCount</a>().</p></div>
    pub unsafe fn name(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMetaMethod_name(self as *const crate::QMetaMethod);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMetaMethod::QMetaMethod()```</span>.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QMetaMethod> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaMethod_QMetaMethod();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qmetamethod.html">QMetaMethod</a> class provides meta-data about a member function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMetaMethod::QMetaMethod(const QMetaMethod& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetamethod.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmetamethod.html">QMetaMethod</a> class provides meta-data about a member function.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qmetamethod.html">QMetaMethod</a> has a <a href="http://doc.qt.io/qt-5/qmetamethod.html#methodType">methodType</a>(), a <a href="http://doc.qt.io/qt-5/qmetamethod.html#methodSignature">methodSignature</a>(), a list of <a href="http://doc.qt.io/qt-5/qmetamethod.html#parameterTypes">parameterTypes</a>() and <a href="http://doc.qt.io/qt-5/qmetamethod.html#parameterNames">parameterNames</a>(), a return <a href="http://doc.qt.io/qt-5/qmetamethod.html#typeName">typeName</a>(), a <a href="http://doc.qt.io/qt-5/qmetamethod.html#tag">tag</a>(), and an <a href="http://doc.qt.io/qt-5/qmetamethod.html#access">access</a>() specifier. You can use <a href="http://doc.qt.io/qt-5/qmetamethod.html#invoke">invoke</a>() to invoke the method on an arbitrary <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMetaMethod>>,
    ) -> ::cpp_utils::CppBox<crate::QMetaMethod> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaMethod_QMetaMethod1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMetaMethod>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the number of parameters of this method.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaMethod::parameterCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetamethod.html#parameterCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of parameters of this method.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetamethod.html#parameterType">parameterType</a>() and <a href="http://doc.qt.io/qt-5/qmetamethod.html#parameterNames">parameterNames</a>().</p></div>
    pub unsafe fn parameter_count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaMethod_parameterCount(self as *const crate::QMetaMethod)
    }

    /// <p>Returns a list of parameter names.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QByteArray> QMetaMethod::parameterNames() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetamethod.html#parameterNames">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of parameter names.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetamethod.html#parameterTypes">parameterTypes</a>() and <a href="http://doc.qt.io/qt-5/qmetamethod.html#methodSignature">methodSignature</a>().</p></div>
    pub unsafe fn parameter_names(&self) -> ::cpp_utils::CppBox<crate::QListOfQByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaMethod_parameterNames(
            self as *const crate::QMetaMethod,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the type of the parameter at the given <i>index</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaMethod::parameterType(int index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetamethod.html#parameterType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the type of the parameter at the given <i>index</i>.</p>
    /// <p>The return value is one of the types that are registered with <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UnknownType</a> if the type is not registered.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetamethod.html#parameterCount">parameterCount</a>(), <a href="http://doc.qt.io/qt-5/qmetamethod.html#returnType">returnType</a>(), and <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a>.</p></div>
    pub unsafe fn parameter_type(&self, index: ::std::os::raw::c_int) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaMethod_parameterType(
            self as *const crate::QMetaMethod,
            index,
        )
    }

    /// <p>Returns a list of parameter types.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QByteArray> QMetaMethod::parameterTypes() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetamethod.html#parameterTypes">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of parameter types.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetamethod.html#parameterNames">parameterNames</a>() and <a href="http://doc.qt.io/qt-5/qmetamethod.html#methodSignature">methodSignature</a>().</p></div>
    pub unsafe fn parameter_types(&self) -> ::cpp_utils::CppBox<crate::QListOfQByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaMethod_parameterTypes(
            self as *const crate::QMetaMethod,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the return type of this method.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaMethod::returnType() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetamethod.html#returnType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the return type of this method.</p>
    /// <p>The return value is one of the types that are registered with <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UnknownType</a> if the type is not registered.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetamethod.html#parameterType">parameterType</a>(), <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a>, and <a href="http://doc.qt.io/qt-5/qmetamethod.html#typeName">typeName</a>().</p></div>
    pub unsafe fn return_type(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaMethod_returnType(self as *const crate::QMetaMethod)
    }

    /// <p>Returns the method revision if one was specified by <a href="http://doc.qt.io/qt-5/qobject.html#Q_REVISION">Q_REVISION</a>, otherwise returns 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaMethod::revision() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetamethod.html#revision">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the method revision if one was specified by <a href="http://doc.qt.io/qt-5/qobject.html#Q_REVISION">Q_REVISION</a>, otherwise returns 0.</p>
    /// <p>This function was introduced in  Qt 5.1.</p></div>
    pub unsafe fn revision(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaMethod_revision(self as *const crate::QMetaMethod)
    }

    /// <p>Returns the tag associated with this method.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QMetaMethod::tag() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetamethod.html#tag">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the tag associated with this method.</p>
    /// <p>Tags are special macros recognized by <code>moc</code> that make it possible to add extra information about a method.</p>
    /// <p>Tag information can be added in the following way in the function declaration:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="comment">// In the class MainWindow declaration</span>
    ///   <span class="preprocessor">#ifndef Q_MOC_RUN</span>
    ///   <span class="comment">// define the tag text as empty, so the compiler doesn't see it</span>
    ///   <span class="preprocessor">#  define MY_CUSTOM_TAG</span>
    ///   <span class="preprocessor">#endif</span>
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <span class="keyword">private</span> <span class="keyword">slots</span>:
    /// &#32;     MY_CUSTOM_TAG <span class="type">void</span> testFunc();
    ///
    /// </pre>
    /// <p>and the information can be accessed by using:</p>
    /// <pre class="cpp">
    ///
    ///   MainWindow win;
    ///   win<span class="operator">.</span>show();
    ///
    ///   <span class="type">int</span> functionIndex <span class="operator">=</span> win<span class="operator">.</span>metaObject()<span class="operator">-</span><span class="operator">&gt;</span>indexOfSlot(<span class="string">"testFunc()"</span>);
    ///   <span class="type">QMetaMethod</span> mm <span class="operator">=</span> win<span class="operator">.</span>metaObject()<span class="operator">-</span><span class="operator">&gt;</span>method(functionIndex);
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> mm<span class="operator">.</span>tag(); <span class="comment">// prints MY_CUSTOM_TAG</span>
    ///
    /// </pre>
    /// <p>For the moment, <code>moc</code> will extract and record all tags, but it will not handle any of them specially. You can use the tags to annotate your methods differently, and treat them according to the specific needs of your application.</p>
    /// <p><b>Note: </b>Since Qt 5.0, <code>moc</code> expands preprocessor macros, so it is necessary to surround the definition with <code>#ifndef</code> <code>Q_MOC_RUN</code>, as shown in the example above. This was not required in Qt 4. The code as shown above works with Qt 4 too.</p></div>
    pub unsafe fn tag(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMetaMethod_tag(self as *const crate::QMetaMethod);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns the return type name of this method.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QMetaMethod::typeName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetamethod.html#typeName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the return type name of this method.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetamethod.html#returnType">returnType</a>() and <a href="http://doc.qt.io/qt-5/qmetatype.html#type">QMetaType::type</a>().</p></div>
    pub unsafe fn type_name(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMetaMethod_typeName(self as *const crate::QMetaMethod);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qmetaenum.html">QMetaEnum</a> class provides meta-data about an enumerator.</p>
///
/// C++ class: <span style='color: green;'>```QMetaEnum```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qmetaenum.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmetaenum.html">QMetaEnum</a> class provides meta-data about an enumerator.</p>
/// <p>Use <a href="http://doc.qt.io/qt-5/qmetaenum.html#name">name</a>() for the enumerator's name. The enumerator's keys (names of each enumerated item) are returned by <a href="http://doc.qt.io/qt-5/qmetaenum.html#key">key</a>(); use <a href="http://doc.qt.io/qt-5/qmetaenum.html#keyCount">keyCount</a>() to find the number of keys. <a href="http://doc.qt.io/qt-5/qmetaenum.html#isFlag">isFlag</a>() returns whether the enumerator is meant to be used as a flag, meaning that its values can be combined using the OR operator.</p>
/// <p>The conversion functions <a href="http://doc.qt.io/qt-5/qmetaenum.html#keyToValue">keyToValue</a>(), <a href="http://doc.qt.io/qt-5/qmetaenum.html#valueToKey">valueToKey</a>(), <a href="http://doc.qt.io/qt-5/qmetaenum.html#keysToValue">keysToValue</a>(), and <a href="http://doc.qt.io/qt-5/qmetaenum.html#valueToKeys">valueToKeys</a>() allow conversion between the integer representation of an enumeration or set value and its literal representation. The <a href="http://doc.qt.io/qt-5/qmetaenum.html#scope">scope</a>() function returns the class scope this enumerator was declared in.</p></div>
#[repr(C)]
pub struct QMetaEnum {
    _unused: u8,
}
impl QMetaEnum {
    /// <p>The <a href="http://doc.qt.io/qt-5/qmetaenum.html">QMetaEnum</a> class provides meta-data about an enumerator.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMetaEnum& QMetaEnum::operator=(const QMetaEnum& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaenum.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmetaenum.html">QMetaEnum</a> class provides meta-data about an enumerator.</p>
    /// <p>Use <a href="http://doc.qt.io/qt-5/qmetaenum.html#name">name</a>() for the enumerator's name. The enumerator's keys (names of each enumerated item) are returned by <a href="http://doc.qt.io/qt-5/qmetaenum.html#key">key</a>(); use <a href="http://doc.qt.io/qt-5/qmetaenum.html#keyCount">keyCount</a>() to find the number of keys. <a href="http://doc.qt.io/qt-5/qmetaenum.html#isFlag">isFlag</a>() returns whether the enumerator is meant to be used as a flag, meaning that its values can be combined using the OR operator.</p>
    /// <p>The conversion functions <a href="http://doc.qt.io/qt-5/qmetaenum.html#keyToValue">keyToValue</a>(), <a href="http://doc.qt.io/qt-5/qmetaenum.html#valueToKey">valueToKey</a>(), <a href="http://doc.qt.io/qt-5/qmetaenum.html#keysToValue">keysToValue</a>(), and <a href="http://doc.qt.io/qt-5/qmetaenum.html#valueToKeys">valueToKeys</a>() allow conversion between the integer representation of an enumeration or set value and its literal representation. The <a href="http://doc.qt.io/qt-5/qmetaenum.html#scope">scope</a>() function returns the class scope this enumerator was declared in.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMetaEnum>>,
    ) -> ::cpp_utils::MutRef<crate::QMetaEnum> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaEnum_operator_(
            self as *mut crate::QMetaEnum,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMetaEnum>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```const QMetaObject* QMetaEnum::enclosingMetaObject() const```</span>.
    pub unsafe fn enclosing_meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaEnum_enclosingMetaObject(
            self as *const crate::QMetaEnum,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns the enum name of the flag (without the scope).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QMetaEnum::enumName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaenum.html#enumName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the enum name of the flag (without the scope).</p>
    /// <p>For example, the <a href="http://doc.qt.io/qt-5/qt.html#AlignmentFlag-enum">Qt::AlignmentFlag</a> flag has <code>AlignmentFlag</code> as the enum name, but <code>Alignment</code> as as the type name. Non flag enums has the same type and enum names.</p>
    /// <p>Enum names have the same scope as the type name.</p>
    /// <p>This function was introduced in  Qt 5.12.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaenum.html#isValid">isValid</a>() and <a href="http://doc.qt.io/qt-5/qmetaenum.html#name">name</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn enum_name(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMetaEnum_enumName(self as *const crate::QMetaEnum);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns <code>true</code> if this enumerator is used as a flag; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaEnum::isFlag() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaenum.html#isFlag">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this enumerator is used as a flag; otherwise returns false.</p>
    /// <p>When used as flags, enumerators can be combined using the OR operator.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaenum.html#keysToValue">keysToValue</a>() and <a href="http://doc.qt.io/qt-5/qmetaenum.html#valueToKeys">valueToKeys</a>().</p></div>
    pub unsafe fn is_flag(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaEnum_isFlag(self as *const crate::QMetaEnum)
    }

    /// <p>Returns <code>true</code> if this enumerator is declared as a C++11 enum class; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaEnum::isScoped() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaenum.html#isScoped">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this enumerator is declared as a C++11 enum class; otherwise returns false.</p>
    /// <p>This function was introduced in  Qt 5.8.</p></div>
    pub unsafe fn is_scoped(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaEnum_isScoped(self as *const crate::QMetaEnum)
    }

    /// <p>Returns <code>true</code> if this enum is valid (has a name); otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaEnum::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaenum.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this enum is valid (has a name); otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaenum.html#name">name</a>().</p></div>
    pub unsafe fn is_valid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaEnum_isValid(self as *const crate::QMetaEnum)
    }

    /// <p>Returns the key with the given <i>index</i>, or 0 if no such key exists.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QMetaEnum::key(int index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaenum.html#key">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the key with the given <i>index</i>, or 0 if no such key exists.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaenum.html#keyCount">keyCount</a>(), <a href="http://doc.qt.io/qt-5/qmetaenum.html#value">value</a>(), and <a href="http://doc.qt.io/qt-5/qmetaenum.html#valueToKey">valueToKey</a>().</p></div>
    pub unsafe fn key(
        &self,
        index: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMetaEnum_key(self as *const crate::QMetaEnum, index);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns the number of keys.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaEnum::keyCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaenum.html#keyCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of keys.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaenum.html#key">key</a>().</p></div>
    pub unsafe fn key_count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaEnum_keyCount(self as *const crate::QMetaEnum)
    }

    /// <p>Returns the integer value of the given enumeration <i>key</i>, or -1 if <i>key</i> is not defined.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaEnum::keyToValue(const char* key, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaenum.html#keyToValue">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the integer value of the given enumeration <i>key</i>, or -1 if <i>key</i> is not defined.</p>
    /// <p>If <i>key</i> is not defined, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <p>For flag types, use <a href="http://doc.qt.io/qt-5/qmetaenum.html#keysToValue">keysToValue</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaenum.html#valueToKey">valueToKey</a>(), <a href="http://doc.qt.io/qt-5/qmetaenum.html#isFlag">isFlag</a>(), and <a href="http://doc.qt.io/qt-5/qmetaenum.html#keysToValue">keysToValue</a>().</p></div>
    pub unsafe fn key_to_value_2a(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaEnum_keyToValue(
            self as *const crate::QMetaEnum,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(key)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the integer value of the given enumeration <i>key</i>, or -1 if <i>key</i> is not defined.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaEnum::keyToValue(const char* key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaenum.html#keyToValue">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the integer value of the given enumeration <i>key</i>, or -1 if <i>key</i> is not defined.</p>
    /// <p>If <i>key</i> is not defined, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <p>For flag types, use <a href="http://doc.qt.io/qt-5/qmetaenum.html#keysToValue">keysToValue</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaenum.html#valueToKey">valueToKey</a>(), <a href="http://doc.qt.io/qt-5/qmetaenum.html#isFlag">isFlag</a>(), and <a href="http://doc.qt.io/qt-5/qmetaenum.html#keysToValue">keysToValue</a>().</p></div>
    pub unsafe fn key_to_value_1a(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaEnum_keyToValue1(
            self as *const crate::QMetaEnum,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(key)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the value derived from combining together the values of the <i>keys</i> using the OR operator, or -1 if <i>keys</i> is not defined. Note that the strings in <i>keys</i> must be '|'-separated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaEnum::keysToValue(const char* keys, bool* ok = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaenum.html#keysToValue">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value derived from combining together the values of the <i>keys</i> using the OR operator, or -1 if <i>keys</i> is not defined. Note that the strings in <i>keys</i> must be '|'-separated.</p>
    /// <p>If <i>keys</i> is not defined, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaenum.html#isFlag">isFlag</a>(), <a href="http://doc.qt.io/qt-5/qmetaenum.html#valueToKey">valueToKey</a>(), and <a href="http://doc.qt.io/qt-5/qmetaenum.html#valueToKeys">valueToKeys</a>().</p></div>
    pub unsafe fn keys_to_value_2a(
        &self,
        keys: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaEnum_keysToValue(
            self as *const crate::QMetaEnum,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(keys)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the value derived from combining together the values of the <i>keys</i> using the OR operator, or -1 if <i>keys</i> is not defined. Note that the strings in <i>keys</i> must be '|'-separated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaEnum::keysToValue(const char* keys) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaenum.html#keysToValue">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value derived from combining together the values of the <i>keys</i> using the OR operator, or -1 if <i>keys</i> is not defined. Note that the strings in <i>keys</i> must be '|'-separated.</p>
    /// <p>If <i>keys</i> is not defined, *<i>ok</i> is set to false; otherwise *<i>ok</i> is set to true.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaenum.html#isFlag">isFlag</a>(), <a href="http://doc.qt.io/qt-5/qmetaenum.html#valueToKey">valueToKey</a>(), and <a href="http://doc.qt.io/qt-5/qmetaenum.html#valueToKeys">valueToKeys</a>().</p></div>
    pub unsafe fn keys_to_value_1a(
        &self,
        keys: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaEnum_keysToValue1(
            self as *const crate::QMetaEnum,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(keys)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the name of the enumerator (without the scope).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QMetaEnum::name() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaenum.html#name">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the name of the enumerator (without the scope).</p>
    /// <p>For example, the <a href="http://doc.qt.io/qt-5/qt.html#AlignmentFlag-enum">Qt::AlignmentFlag</a> enumeration has <code>AlignmentFlag</code> as the name and <a href="http://doc.qt.io/qt-5/qt.html">Qt</a> as the scope.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaenum.html#isValid">isValid</a>() and <a href="http://doc.qt.io/qt-5/qmetaenum.html#scope">scope</a>().</p></div>
    pub unsafe fn name(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMetaEnum_name(self as *const crate::QMetaEnum);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMetaEnum::QMetaEnum()```</span>.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QMetaEnum> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaEnum_QMetaEnum();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qmetaenum.html">QMetaEnum</a> class provides meta-data about an enumerator.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMetaEnum::QMetaEnum(const QMetaEnum& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaenum.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmetaenum.html">QMetaEnum</a> class provides meta-data about an enumerator.</p>
    /// <p>Use <a href="http://doc.qt.io/qt-5/qmetaenum.html#name">name</a>() for the enumerator's name. The enumerator's keys (names of each enumerated item) are returned by <a href="http://doc.qt.io/qt-5/qmetaenum.html#key">key</a>(); use <a href="http://doc.qt.io/qt-5/qmetaenum.html#keyCount">keyCount</a>() to find the number of keys. <a href="http://doc.qt.io/qt-5/qmetaenum.html#isFlag">isFlag</a>() returns whether the enumerator is meant to be used as a flag, meaning that its values can be combined using the OR operator.</p>
    /// <p>The conversion functions <a href="http://doc.qt.io/qt-5/qmetaenum.html#keyToValue">keyToValue</a>(), <a href="http://doc.qt.io/qt-5/qmetaenum.html#valueToKey">valueToKey</a>(), <a href="http://doc.qt.io/qt-5/qmetaenum.html#keysToValue">keysToValue</a>(), and <a href="http://doc.qt.io/qt-5/qmetaenum.html#valueToKeys">valueToKeys</a>() allow conversion between the integer representation of an enumeration or set value and its literal representation. The <a href="http://doc.qt.io/qt-5/qmetaenum.html#scope">scope</a>() function returns the class scope this enumerator was declared in.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMetaEnum>>,
    ) -> ::cpp_utils::CppBox<crate::QMetaEnum> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaEnum_QMetaEnum1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMetaEnum>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the scope this enumerator was declared in.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QMetaEnum::scope() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaenum.html#scope">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the scope this enumerator was declared in.</p>
    /// <p>For example, the <a href="http://doc.qt.io/qt-5/qt.html#AlignmentFlag-enum">Qt::AlignmentFlag</a> enumeration has <code>Qt</code> as the scope and <code>AlignmentFlag</code> as the name.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaenum.html#name">name</a>().</p></div>
    pub unsafe fn scope(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMetaEnum_scope(self as *const crate::QMetaEnum);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns the value with the given <i>index</i>; or returns -1 if there is no such value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaEnum::value(int index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaenum.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value with the given <i>index</i>; or returns -1 if there is no such value.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaenum.html#keyCount">keyCount</a>(), <a href="http://doc.qt.io/qt-5/qmetaenum.html#key">key</a>(), and <a href="http://doc.qt.io/qt-5/qmetaenum.html#keyToValue">keyToValue</a>().</p></div>
    pub unsafe fn value(&self, index: ::std::os::raw::c_int) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaEnum_value(self as *const crate::QMetaEnum, index)
    }

    /// <p>Returns the string that is used as the name of the given enumeration <i>value</i>, or 0 if <i>value</i> is not defined.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QMetaEnum::valueToKey(int value) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaenum.html#valueToKey">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string that is used as the name of the given enumeration <i>value</i>, or 0 if <i>value</i> is not defined.</p>
    /// <p>For flag types, use <a href="http://doc.qt.io/qt-5/qmetaenum.html#valueToKeys">valueToKeys</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaenum.html#isFlag">isFlag</a>() and <a href="http://doc.qt.io/qt-5/qmetaenum.html#valueToKeys">valueToKeys</a>().</p></div>
    pub unsafe fn value_to_key(
        &self,
        value: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaEnum_valueToKey(
            self as *const crate::QMetaEnum,
            value,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns a byte array of '|'-separated keys that represents the given <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QMetaEnum::valueToKeys(int value) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaenum.html#valueToKeys">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a byte array of '|'-separated keys that represents the given <i>value</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaenum.html#isFlag">isFlag</a>(), <a href="http://doc.qt.io/qt-5/qmetaenum.html#valueToKey">valueToKey</a>(), and <a href="http://doc.qt.io/qt-5/qmetaenum.html#keysToValue">keysToValue</a>().</p></div>
    pub unsafe fn value_to_keys(
        &self,
        value: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaEnum_valueToKeys(
            self as *const crate::QMetaEnum,
            value,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qmetaproperty.html">QMetaProperty</a> class provides meta-data about a property.</p>
///
/// C++ class: <span style='color: green;'>```QMetaProperty```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qmetaproperty.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmetaproperty.html">QMetaProperty</a> class provides meta-data about a property.</p>
/// <p>Property meta-data is obtained from an object's meta-object. See <a href="http://doc.qt.io/qt-5/qmetaobject.html#property">QMetaObject::property</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#propertyCount">QMetaObject::propertyCount</a>() for details.</p>
/// <a name="property-meta-data"></a></div>
#[repr(C)]
pub struct QMetaProperty {
    _unused: u8,
}
impl QMetaProperty {
    /// <p>The <a href="http://doc.qt.io/qt-5/qmetaproperty.html">QMetaProperty</a> class provides meta-data about a property.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMetaProperty& QMetaProperty::operator=(const QMetaProperty& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmetaproperty.html">QMetaProperty</a> class provides meta-data about a property.</p>
    /// <p>Property meta-data is obtained from an object's meta-object. See <a href="http://doc.qt.io/qt-5/qmetaobject.html#property">QMetaObject::property</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#propertyCount">QMetaObject::propertyCount</a>() for details.</p>
    /// <a name="property-meta-data"></a></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMetaProperty>>,
    ) -> ::cpp_utils::MutRef<crate::QMetaProperty> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaProperty_operator_(
            self as *mut crate::QMetaProperty,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMetaProperty>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```const QMetaObject* QMetaProperty::enclosingMetaObject() const```</span>.
    pub unsafe fn enclosing_meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaProperty_enclosingMetaObject(
            self as *const crate::QMetaProperty,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns the enumerator if this property's type is an enumerator type; otherwise the returned value is undefined.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMetaEnum QMetaProperty::enumerator() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#enumerator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the enumerator if this property's type is an enumerator type; otherwise the returned value is undefined.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaproperty.html#isEnumType">isEnumType</a>() and <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isFlagType">isFlagType</a>().</p></div>
    pub unsafe fn enumerator(&self) -> ::cpp_utils::CppBox<crate::QMetaEnum> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaProperty_enumerator(
            self as *const crate::QMetaProperty,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if this property has a corresponding change notify signal; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaProperty::hasNotifySignal() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#hasNotifySignal">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this property has a corresponding change notify signal; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaproperty.html#notifySignal">notifySignal</a>().</p></div>
    pub unsafe fn has_notify_signal(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_hasNotifySignal(
            self as *const crate::QMetaProperty,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QMetaProperty::hasStdCppSet() const```</span>.
    pub unsafe fn has_std_cpp_set(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_hasStdCppSet(
            self as *const crate::QMetaProperty,
        )
    }

    /// <p>Returns <code>true</code> if the property is constant; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaProperty::isConstant() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isConstant">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the property is constant; otherwise returns <code>false</code>.</p>
    /// <p>A property is constant if the <code>Q_PROPERTY()</code>'s <code>CONSTANT</code> attribute is set.</p>
    /// <p>This function was introduced in  Qt 4.6.</p></div>
    pub unsafe fn is_constant(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_isConstant(self as *const crate::QMetaProperty)
    }

    /// <p>Returns <code>true</code> if this property is designable for the given <i>object</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaProperty::isDesignable(const QObject* obj = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isDesignable">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this property is designable for the given <i>object</i>; otherwise returns <code>false</code>.</p>
    /// <p>If no <i>object</i> is given, the function returns <code>false</code> if the <code>Q_PROPERTY()</code>'s <code>DESIGNABLE</code> attribute is false; otherwise returns <code>true</code> (if the attribute is true or is a function or expression).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaproperty.html#isScriptable">isScriptable</a>() and <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isStored">isStored</a>().</p></div>
    pub unsafe fn is_designable_1a(
        &self,
        obj: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_isDesignable(
            self as *const crate::QMetaProperty,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(obj).as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if this property is designable for the given <i>object</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaProperty::isDesignable() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isDesignable">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this property is designable for the given <i>object</i>; otherwise returns <code>false</code>.</p>
    /// <p>If no <i>object</i> is given, the function returns <code>false</code> if the <code>Q_PROPERTY()</code>'s <code>DESIGNABLE</code> attribute is false; otherwise returns <code>true</code> (if the attribute is true or is a function or expression).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaproperty.html#isScriptable">isScriptable</a>() and <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isStored">isStored</a>().</p></div>
    pub unsafe fn is_designable_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_isDesignable1(
            self as *const crate::QMetaProperty,
        )
    }

    /// <p>Returns <code>true</code> if the property is editable for the given <i>object</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaProperty::isEditable(const QObject* obj = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty-obsolete.html#isEditable">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the property is editable for the given <i>object</i>; otherwise returns <code>false</code>.</p>
    /// <p>If no <i>object</i> is given, the function returns <code>false</code> if the <code>Q_PROPERTY()</code>'s <code>EDITABLE</code> attribute is false; otherwise returns true (if the attribute is true or is a function or expression).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaproperty.html#isDesignable">isDesignable</a>(), <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isScriptable">isScriptable</a>(), and <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isStored">isStored</a>().</p></div>
    pub unsafe fn is_editable_1a(
        &self,
        obj: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_isEditable(
            self as *const crate::QMetaProperty,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(obj).as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if the property is editable for the given <i>object</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaProperty::isEditable() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty-obsolete.html#isEditable">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the property is editable for the given <i>object</i>; otherwise returns <code>false</code>.</p>
    /// <p>If no <i>object</i> is given, the function returns <code>false</code> if the <code>Q_PROPERTY()</code>'s <code>EDITABLE</code> attribute is false; otherwise returns true (if the attribute is true or is a function or expression).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaproperty.html#isDesignable">isDesignable</a>(), <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isScriptable">isScriptable</a>(), and <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isStored">isStored</a>().</p></div>
    pub unsafe fn is_editable_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_isEditable1(self as *const crate::QMetaProperty)
    }

    /// <p>Returns <code>true</code> if the property's type is an enumeration value; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaProperty::isEnumType() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isEnumType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the property's type is an enumeration value; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaproperty.html#enumerator">enumerator</a>() and <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isFlagType">isFlagType</a>().</p></div>
    pub unsafe fn is_enum_type(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_isEnumType(self as *const crate::QMetaProperty)
    }

    /// <p>Returns <code>true</code> if the property is final; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaProperty::isFinal() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isFinal">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the property is final; otherwise returns <code>false</code>.</p>
    /// <p>A property is final if the <code>Q_PROPERTY()</code>'s <code>FINAL</code> attribute is set.</p>
    /// <p>This function was introduced in  Qt 4.6.</p></div>
    pub unsafe fn is_final(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_isFinal(self as *const crate::QMetaProperty)
    }

    /// <p>Returns <code>true</code> if the property's type is an enumeration value that is used as a flag; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaProperty::isFlagType() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isFlagType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the property's type is an enumeration value that is used as a flag; otherwise returns <code>false</code>.</p>
    /// <p>Flags can be combined using the OR operator. A flag type is implicitly also an enum type.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaproperty.html#isEnumType">isEnumType</a>(), <a href="http://doc.qt.io/qt-5/qmetaproperty.html#enumerator">enumerator</a>(), and <a href="http://doc.qt.io/qt-5/qmetaenum.html#isFlag">QMetaEnum::isFlag</a>().</p></div>
    pub unsafe fn is_flag_type(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_isFlagType(self as *const crate::QMetaProperty)
    }

    /// <p>Returns <code>true</code> if this property is readable; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaProperty::isReadable() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isReadable">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this property is readable; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaproperty.html#isWritable">isWritable</a>(), <a href="http://doc.qt.io/qt-5/qmetaproperty.html#read">read</a>(), and <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isValid">isValid</a>().</p></div>
    pub unsafe fn is_readable(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_isReadable(self as *const crate::QMetaProperty)
    }

    /// <p>Returns <code>true</code> if this property can be reset to a default value; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaProperty::isResettable() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isResettable">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this property can be reset to a default value; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaproperty.html#reset">reset</a>().</p></div>
    pub unsafe fn is_resettable(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_isResettable(
            self as *const crate::QMetaProperty,
        )
    }

    /// <p>Returns <code>true</code> if the property is scriptable for the given <i>object</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaProperty::isScriptable(const QObject* obj = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isScriptable">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the property is scriptable for the given <i>object</i>; otherwise returns <code>false</code>.</p>
    /// <p>If no <i>object</i> is given, the function returns <code>false</code> if the <code>Q_PROPERTY()</code>'s <code>SCRIPTABLE</code> attribute is false; otherwise returns true (if the attribute is true or is a function or expression).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaproperty.html#isDesignable">isDesignable</a>() and <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isStored">isStored</a>().</p></div>
    pub unsafe fn is_scriptable_1a(
        &self,
        obj: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_isScriptable(
            self as *const crate::QMetaProperty,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(obj).as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if the property is scriptable for the given <i>object</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaProperty::isScriptable() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isScriptable">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the property is scriptable for the given <i>object</i>; otherwise returns <code>false</code>.</p>
    /// <p>If no <i>object</i> is given, the function returns <code>false</code> if the <code>Q_PROPERTY()</code>'s <code>SCRIPTABLE</code> attribute is false; otherwise returns true (if the attribute is true or is a function or expression).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaproperty.html#isDesignable">isDesignable</a>() and <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isStored">isStored</a>().</p></div>
    pub unsafe fn is_scriptable_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_isScriptable1(
            self as *const crate::QMetaProperty,
        )
    }

    /// <p>Returns <code>true</code> if the property is stored for <i>object</i>; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaProperty::isStored(const QObject* obj = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isStored">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the property is stored for <i>object</i>; otherwise returns false.</p>
    /// <p>If no <i>object</i> is given, the function returns <code>false</code> if the <code>Q_PROPERTY()</code>'s <code>STORED</code> attribute is false; otherwise returns true (if the attribute is true or is a function or expression).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaproperty.html#isDesignable">isDesignable</a>() and <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isScriptable">isScriptable</a>().</p></div>
    pub unsafe fn is_stored_1a(
        &self,
        obj: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_isStored(
            self as *const crate::QMetaProperty,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(obj).as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if the property is stored for <i>object</i>; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaProperty::isStored() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isStored">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the property is stored for <i>object</i>; otherwise returns false.</p>
    /// <p>If no <i>object</i> is given, the function returns <code>false</code> if the <code>Q_PROPERTY()</code>'s <code>STORED</code> attribute is false; otherwise returns true (if the attribute is true or is a function or expression).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaproperty.html#isDesignable">isDesignable</a>() and <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isScriptable">isScriptable</a>().</p></div>
    pub unsafe fn is_stored_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_isStored1(self as *const crate::QMetaProperty)
    }

    /// <p>Returns <code>true</code> if this property is designated as the <code>USER</code> property, i.e., the one that the user can edit for <i>object</i> or that is significant in some other way. Otherwise it returns false. e.g., the <code>text</code> property is the <code>USER</code> editable property of a <a href="http://doc.qt.io/qt-5/qlineedit.html">QLineEdit</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaProperty::isUser(const QObject* obj = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isUser">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this property is designated as the <code>USER</code> property, i.e., the one that the user can edit for <i>object</i> or that is significant in some other way. Otherwise it returns false. e.g., the <code>text</code> property is the <code>USER</code> editable property of a <a href="http://doc.qt.io/qt-5/qlineedit.html">QLineEdit</a>.</p>
    /// <p>If <i>object</i> is null, the function returns <code>false</code> if the <code>Q_PROPERTY()</code>'s <code>USER</code> attribute is false. Otherwise it returns true.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#userProperty">QMetaObject::userProperty</a>(), <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isDesignable">isDesignable</a>(), and <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isScriptable">isScriptable</a>().</p></div>
    pub unsafe fn is_user_1a(
        &self,
        obj: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_isUser(
            self as *const crate::QMetaProperty,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(obj).as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if this property is designated as the <code>USER</code> property, i.e., the one that the user can edit for <i>object</i> or that is significant in some other way. Otherwise it returns false. e.g., the <code>text</code> property is the <code>USER</code> editable property of a <a href="http://doc.qt.io/qt-5/qlineedit.html">QLineEdit</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaProperty::isUser() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isUser">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this property is designated as the <code>USER</code> property, i.e., the one that the user can edit for <i>object</i> or that is significant in some other way. Otherwise it returns false. e.g., the <code>text</code> property is the <code>USER</code> editable property of a <a href="http://doc.qt.io/qt-5/qlineedit.html">QLineEdit</a>.</p>
    /// <p>If <i>object</i> is null, the function returns <code>false</code> if the <code>Q_PROPERTY()</code>'s <code>USER</code> attribute is false. Otherwise it returns true.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#userProperty">QMetaObject::userProperty</a>(), <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isDesignable">isDesignable</a>(), and <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isScriptable">isScriptable</a>().</p></div>
    pub unsafe fn is_user_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_isUser1(self as *const crate::QMetaProperty)
    }

    /// <p>Returns <code>true</code> if this property is valid (readable); otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaProperty::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this property is valid (readable); otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaproperty.html#isReadable">isReadable</a>().</p></div>
    pub unsafe fn is_valid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_isValid(self as *const crate::QMetaProperty)
    }

    /// <p>Returns <code>true</code> if this property is writable; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaProperty::isWritable() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isWritable">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this property is writable; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaproperty.html#isReadable">isReadable</a>() and <a href="http://doc.qt.io/qt-5/qmetaproperty.html#write">write</a>().</p></div>
    pub unsafe fn is_writable(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_isWritable(self as *const crate::QMetaProperty)
    }

    /// <p>Returns this property's name.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QMetaProperty::name() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#name">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns this property's name.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaproperty.html#type">type</a>() and <a href="http://doc.qt.io/qt-5/qmetaproperty.html#typeName">typeName</a>().</p></div>
    pub unsafe fn name(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMetaProperty_name(self as *const crate::QMetaProperty);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMetaProperty::QMetaProperty()```</span>.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QMetaProperty> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaProperty_QMetaProperty();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qmetaproperty.html">QMetaProperty</a> class provides meta-data about a property.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMetaProperty::QMetaProperty(const QMetaProperty& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmetaproperty.html">QMetaProperty</a> class provides meta-data about a property.</p>
    /// <p>Property meta-data is obtained from an object's meta-object. See <a href="http://doc.qt.io/qt-5/qmetaobject.html#property">QMetaObject::property</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#propertyCount">QMetaObject::propertyCount</a>() for details.</p>
    /// <a name="property-meta-data"></a></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMetaProperty>>,
    ) -> ::cpp_utils::CppBox<crate::QMetaProperty> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaProperty_QMetaProperty1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMetaProperty>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the <a href="http://doc.qt.io/qt-5/qmetamethod.html">QMetaMethod</a> instance of the property change notifying signal if one was specified, otherwise returns an invalid <a href="http://doc.qt.io/qt-5/qmetamethod.html">QMetaMethod</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMetaMethod QMetaProperty::notifySignal() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#notifySignal">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the <a href="http://doc.qt.io/qt-5/qmetamethod.html">QMetaMethod</a> instance of the property change notifying signal if one was specified, otherwise returns an invalid <a href="http://doc.qt.io/qt-5/qmetamethod.html">QMetaMethod</a>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaproperty.html#hasNotifySignal">hasNotifySignal</a>().</p></div>
    pub unsafe fn notify_signal(&self) -> ::cpp_utils::CppBox<crate::QMetaMethod> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaProperty_notifySignal(
            self as *const crate::QMetaProperty,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the index of the property change notifying signal if one was specified, otherwise returns -1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaProperty::notifySignalIndex() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#notifySignalIndex">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index of the property change notifying signal if one was specified, otherwise returns -1.</p>
    /// <p>This function was introduced in  Qt 4.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaproperty.html#hasNotifySignal">hasNotifySignal</a>().</p></div>
    pub unsafe fn notify_signal_index(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_notifySignalIndex(
            self as *const crate::QMetaProperty,
        )
    }

    /// <p>Returns this property's index.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaProperty::propertyIndex() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#propertyIndex">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns this property's index.</p>
    /// <p>This function was introduced in  Qt 4.6.</p></div>
    pub unsafe fn property_index(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_propertyIndex(
            self as *const crate::QMetaProperty,
        )
    }

    /// <p>Reads the property's value from the given <i>object</i>. Returns the value if it was able to read it; otherwise returns an invalid variant.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QMetaProperty::read(const QObject* obj) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#read">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads the property's value from the given <i>object</i>. Returns the value if it was able to read it; otherwise returns an invalid variant.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaproperty.html#write">write</a>(), <a href="http://doc.qt.io/qt-5/qmetaproperty.html#reset">reset</a>(), and <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isReadable">isReadable</a>().</p></div>
    pub unsafe fn read(
        &self,
        obj: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaProperty_read(
            self as *const crate::QMetaProperty,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(obj).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reads the property's value from the given <i>gadget</i>. Returns the value if it was able to read it; otherwise returns an invalid variant.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QMetaProperty::readOnGadget(const void* gadget) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#readOnGadget">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads the property's value from the given <i>gadget</i>. Returns the value if it was able to read it; otherwise returns an invalid variant.</p>
    /// <p>This function should only be used if this is a property of a <a href="http://doc.qt.io/qt-5/qobject.html#Q_GADGET">Q_GADGET</a></p>
    /// <p>This function was introduced in  Qt 5.5.</p></div>
    pub unsafe fn read_on_gadget(
        &self,
        gadget: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::ffi::c_void>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaProperty_readOnGadget(
            self as *const crate::QMetaProperty,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::ffi::c_void>>::cast_into(gadget)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Resets the property for the given <i>object</i> with a reset method. Returns <code>true</code> if the reset worked; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaProperty::reset(QObject* obj) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#reset">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Resets the property for the given <i>object</i> with a reset method. Returns <code>true</code> if the reset worked; otherwise returns <code>false</code>.</p>
    /// <p>Reset methods are optional; only a few properties support them.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaproperty.html#read">read</a>() and <a href="http://doc.qt.io/qt-5/qmetaproperty.html#write">write</a>().</p></div>
    pub unsafe fn reset(
        &self,
        obj: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_reset(
            self as *const crate::QMetaProperty,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(obj)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Resets the property for the given <i>gadget</i> with a reset method. Returns <code>true</code> if the reset worked; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaProperty::resetOnGadget(void* gadget) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#resetOnGadget">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Resets the property for the given <i>gadget</i> with a reset method. Returns <code>true</code> if the reset worked; otherwise returns <code>false</code>.</p>
    /// <p>Reset methods are optional; only a few properties support them.</p>
    /// <p>This function should only be used if this is a property of a <a href="http://doc.qt.io/qt-5/qobject.html#Q_GADGET">Q_GADGET</a></p>
    /// <p>This function was introduced in  Qt 5.5.</p></div>
    pub unsafe fn reset_on_gadget(
        &self,
        gadget: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_resetOnGadget(
            self as *const crate::QMetaProperty,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(gadget)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the property revision if one was specified by REVISION, otherwise returns 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaProperty::revision() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#revision">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the property revision if one was specified by REVISION, otherwise returns 0.</p>
    /// <p>This function was introduced in  Qt 5.1.</p></div>
    pub unsafe fn revision(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_revision(self as *const crate::QMetaProperty)
    }

    /// <p>Returns this property's type. The return value is one of the values of the QVariant::Type enumeration.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant::Type QMetaProperty::type() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#type">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns this property's type. The return value is one of the values of the QVariant::Type enumeration.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaproperty.html#userType">userType</a>(), <a href="http://doc.qt.io/qt-5/qmetaproperty.html#typeName">typeName</a>(), and <a href="http://doc.qt.io/qt-5/qmetaproperty.html#name">name</a>().</p></div>
    pub unsafe fn type_(&self) -> crate::q_variant::Type {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_type(self as *const crate::QMetaProperty)
    }

    /// <p>Returns the name of this property's type.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QMetaProperty::typeName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#typeName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the name of this property's type.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaproperty.html#type">type</a>() and <a href="http://doc.qt.io/qt-5/qmetaproperty.html#name">name</a>().</p></div>
    pub unsafe fn type_name(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaProperty_typeName(
            self as *const crate::QMetaProperty,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns this property's user type. The return value is one of the values that are registered with <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UnknownType</a> if the type is not registered.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMetaProperty::userType() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#userType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns this property's user type. The return value is one of the values that are registered with <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a>, or <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QMetaType::UnknownType</a> if the type is not registered.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaproperty.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a>, and <a href="http://doc.qt.io/qt-5/qmetaproperty.html#typeName">typeName</a>().</p></div>
    pub unsafe fn user_type(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_userType(self as *const crate::QMetaProperty)
    }

    /// <p>Writes <i>value</i> as the property's value to the given <i>object</i>. Returns true if the write succeeded; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaProperty::write(QObject* obj, const QVariant& value) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#write">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes <i>value</i> as the property's value to the given <i>object</i>. Returns true if the write succeeded; otherwise returns <code>false</code>.</p>
    /// <p>If <i>value</i> is not of the same type type as the property, a conversion is attempted. An empty QVariant() is equivalent to a call to <a href="http://doc.qt.io/qt-5/qmetaproperty.html#reset">reset</a>() if this property is resetable, or setting a default-constructed object otherwise.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaproperty.html#read">read</a>(), <a href="http://doc.qt.io/qt-5/qmetaproperty.html#reset">reset</a>(), and <a href="http://doc.qt.io/qt-5/qmetaproperty.html#isWritable">isWritable</a>().</p></div>
    pub unsafe fn write(
        &self,
        obj: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_write(
            self as *const crate::QMetaProperty,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(obj)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>Writes <i>value</i> as the property's value to the given <i>gadget</i>. Returns true if the write succeeded; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMetaProperty::writeOnGadget(void* gadget, const QVariant& value) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html#writeOnGadget">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes <i>value</i> as the property's value to the given <i>gadget</i>. Returns true if the write succeeded; otherwise returns <code>false</code>.</p>
    /// <p>This function should only be used if this is a property of a <a href="http://doc.qt.io/qt-5/qobject.html#Q_GADGET">Q_GADGET</a></p>
    /// <p>This function was introduced in  Qt 5.5.</p></div>
    pub unsafe fn write_on_gadget(
        &self,
        gadget: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_writeOnGadget(
            self as *const crate::QMetaProperty,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(gadget)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        )
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qmetaclassinfo.html">QMetaClassInfo</a> class provides additional information about a class.</p>
///
/// C++ class: <span style='color: green;'>```QMetaClassInfo```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qmetaclassinfo.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmetaclassinfo.html">QMetaClassInfo</a> class provides additional information about a class.</p>
/// <p>Class information items are simple <i>name</i>--<i>value</i> pairs that are specified using <a href="http://doc.qt.io/qt-5/qobject.html#Q_CLASSINFO">Q_CLASSINFO</a>() in the source code. The information can be retrieved using <a href="http://doc.qt.io/qt-5/qmetaclassinfo.html#name">name</a>() and <a href="http://doc.qt.io/qt-5/qmetaclassinfo.html#value">value</a>(). For example:</p>
/// <pre class="cpp">
///
///   <span class="keyword">class</span> MyClass
///   {
/// &#32;     Q_OBJECT
/// &#32;     Q_CLASSINFO(<span class="string">"author"</span><span class="operator">,</span> <span class="string">"Sabrina Schweinsteiger"</span>)
/// &#32;     Q_CLASSINFO(<span class="string">"url"</span><span class="operator">,</span> <span class="string">"http://doc.moosesoft.co.uk/1.0/"</span>)
///
///   <span class="keyword">public</span>:
/// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   };
///
/// </pre>
/// <p>This mechanism is free for you to use in your Qt applications. Qt doesn't use it for any of its classes.</p></div>
#[repr(C)]
pub struct QMetaClassInfo {
    _unused: u8,
}
impl QMetaClassInfo {
    /// <p>The <a href="http://doc.qt.io/qt-5/qmetaclassinfo.html">QMetaClassInfo</a> class provides additional information about a class.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMetaClassInfo& QMetaClassInfo::operator=(const QMetaClassInfo& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaclassinfo.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmetaclassinfo.html">QMetaClassInfo</a> class provides additional information about a class.</p>
    /// <p>Class information items are simple <i>name</i>--<i>value</i> pairs that are specified using <a href="http://doc.qt.io/qt-5/qobject.html#Q_CLASSINFO">Q_CLASSINFO</a>() in the source code. The information can be retrieved using <a href="http://doc.qt.io/qt-5/qmetaclassinfo.html#name">name</a>() and <a href="http://doc.qt.io/qt-5/qmetaclassinfo.html#value">value</a>(). For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">class</span> MyClass
    ///   {
    /// &#32;     Q_OBJECT
    /// &#32;     Q_CLASSINFO(<span class="string">"author"</span><span class="operator">,</span> <span class="string">"Sabrina Schweinsteiger"</span>)
    /// &#32;     Q_CLASSINFO(<span class="string">"url"</span><span class="operator">,</span> <span class="string">"http://doc.moosesoft.co.uk/1.0/"</span>)
    ///
    ///   <span class="keyword">public</span>:
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   };
    ///
    /// </pre>
    /// <p>This mechanism is free for you to use in your Qt applications. Qt doesn't use it for any of its classes.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMetaClassInfo>>,
    ) -> ::cpp_utils::MutRef<crate::QMetaClassInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaClassInfo_operator_(
            self as *mut crate::QMetaClassInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMetaClassInfo>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```const QMetaObject* QMetaClassInfo::enclosingMetaObject() const```</span>.
    pub unsafe fn enclosing_meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaClassInfo_enclosingMetaObject(
            self as *const crate::QMetaClassInfo,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns the name of this item.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QMetaClassInfo::name() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaclassinfo.html#name">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the name of this item.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaclassinfo.html#value">value</a>().</p></div>
    pub unsafe fn name(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMetaClassInfo_name(self as *const crate::QMetaClassInfo);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMetaClassInfo::QMetaClassInfo()```</span>.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QMetaClassInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaClassInfo_QMetaClassInfo();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qmetaclassinfo.html">QMetaClassInfo</a> class provides additional information about a class.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMetaClassInfo::QMetaClassInfo(const QMetaClassInfo& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaclassinfo.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmetaclassinfo.html">QMetaClassInfo</a> class provides additional information about a class.</p>
    /// <p>Class information items are simple <i>name</i>--<i>value</i> pairs that are specified using <a href="http://doc.qt.io/qt-5/qobject.html#Q_CLASSINFO">Q_CLASSINFO</a>() in the source code. The information can be retrieved using <a href="http://doc.qt.io/qt-5/qmetaclassinfo.html#name">name</a>() and <a href="http://doc.qt.io/qt-5/qmetaclassinfo.html#value">value</a>(). For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">class</span> MyClass
    ///   {
    /// &#32;     Q_OBJECT
    /// &#32;     Q_CLASSINFO(<span class="string">"author"</span><span class="operator">,</span> <span class="string">"Sabrina Schweinsteiger"</span>)
    /// &#32;     Q_CLASSINFO(<span class="string">"url"</span><span class="operator">,</span> <span class="string">"http://doc.moosesoft.co.uk/1.0/"</span>)
    ///
    ///   <span class="keyword">public</span>:
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   };
    ///
    /// </pre>
    /// <p>This mechanism is free for you to use in your Qt applications. Qt doesn't use it for any of its classes.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMetaClassInfo>>,
    ) -> ::cpp_utils::CppBox<crate::QMetaClassInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaClassInfo_QMetaClassInfo1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMetaClassInfo>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value of this item.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QMetaClassInfo::value() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaclassinfo.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value of this item.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaclassinfo.html#name">name</a>().</p></div>
    pub unsafe fn value(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMetaClassInfo_value(
            self as *const crate::QMetaClassInfo,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qmimedata.html">QMimeData</a> class provides a container for data that records information about its MIME type.</p>
///
/// C++ class: <span style='color: green;'>```QMimeData```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qmimedata.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmimedata.html">QMimeData</a> class provides a container for data that records information about its MIME type.</p>
/// <p><a href="http://doc.qt.io/qt-5/qmimedata.html">QMimeData</a> is used to describe information that can be stored in the <a href="http://doc.qt.io/qt-5/qclipboard.html">clipboard</a>, and transferred via the <a href="http://doc.qt.io/qt-5/qtgui-index.html#drag-and-drop">drag and drop</a> mechanism. <a href="http://doc.qt.io/qt-5/qmimedata.html">QMimeData</a> objects associate the data that they hold with the corresponding MIME types to ensure that information can be safely transferred between applications, and copied around within the same application.</p>
/// <p><a href="http://doc.qt.io/qt-5/qmimedata.html">QMimeData</a> objects are usually created using <code>new</code> and supplied to <a href="http://doc.qt.io/qt-5/qdrag.html">QDrag</a> or <a href="http://doc.qt.io/qt-5/qclipboard.html">QClipboard</a> objects. This is to enable Qt to manage the memory that they use.</p>
/// <p>A single <a href="http://doc.qt.io/qt-5/qmimedata.html">QMimeData</a> object can store the same data using several different formats at the same time. The <a href="http://doc.qt.io/qt-5/qmimedata.html#formats">formats</a>() function returns a list of the available formats in order of preference. The <a href="http://doc.qt.io/qt-5/qmimedata.html#data">data</a>() function returns the raw data associated with a MIME type, and <a href="http://doc.qt.io/qt-5/qmimedata.html#setData">setData</a>() allows you to set the data for a MIME type.</p>
/// <p>For the most common MIME types, <a href="http://doc.qt.io/qt-5/qmimedata.html">QMimeData</a> provides convenience functions to access the data:</p>
/// <div class="table"><table class="generic">
///  <thead><tr class="qt-style"><th>Tester</th><th>Getter</th><th>Setter</th><th>MIME Types</th></tr></thead>
/// <tbody><tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qmimedata.html#hasText">hasText</a>()</td><td><a href="http://doc.qt.io/qt-5/qmimedata.html#text">text</a>()</td><td><a href="http://doc.qt.io/qt-5/qmimedata.html#setText">setText</a>()</td><td><code>text/plain</code></td></tr>
/// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qmimedata.html#hasHtml">hasHtml</a>()</td><td><a href="http://doc.qt.io/qt-5/qmimedata.html#html">html</a>()</td><td><a href="http://doc.qt.io/qt-5/qmimedata.html#setHtml">setHtml</a>()</td><td><code>text/html</code></td></tr>
/// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qmimedata.html#hasUrls">hasUrls</a>()</td><td><a href="http://doc.qt.io/qt-5/qmimedata.html#urls">urls</a>()</td><td><a href="http://doc.qt.io/qt-5/qmimedata.html#setUrls">setUrls</a>()</td><td><code>text/uri-list</code></td></tr>
/// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qmimedata.html#hasImage">hasImage</a>()</td><td><a href="http://doc.qt.io/qt-5/qmimedata.html#imageData">imageData</a>()</td><td><a href="http://doc.qt.io/qt-5/qmimedata.html#setImageData">setImageData</a>()</td><td><code>image/</code> *</td></tr>
/// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qmimedata.html#hasColor">hasColor</a>()</td><td><a href="http://doc.qt.io/qt-5/qmimedata.html#colorData">colorData</a>()</td><td><a href="http://doc.qt.io/qt-5/qmimedata.html#setColorData">setColorData</a>()</td><td><code>application/x-color</code></td></tr>
/// </tbody></table></div>
/// <p>For example, if your write a widget that accepts URL drags, you would end up writing code like this:</p>
/// <pre class="cpp">
///
///   <span class="type">void</span> MyWidget<span class="operator">::</span>dragEnterEvent(<span class="type"><a href="http://doc.qt.io/qt-5/qdragenterevent.html">QDragEnterEvent</a></span> <span class="operator">*</span>event)
///   {
/// &#32;     <span class="keyword">if</span> (event<span class="operator">-</span><span class="operator">&gt;</span>mimeData()<span class="operator">-</span><span class="operator">&gt;</span>hasUrls())
/// &#32;   &#32;     event<span class="operator">-</span><span class="operator">&gt;</span>acceptProposedAction();
///   }
///
///   <span class="type">void</span> MyWidget<span class="operator">::</span>dropEvent(<span class="type"><a href="http://doc.qt.io/qt-5/qdropevent.html">QDropEvent</a></span> <span class="operator">*</span>event)
///   {
/// &#32;     <span class="keyword">if</span> (event<span class="operator">-</span><span class="operator">&gt;</span>mimeData()<span class="operator">-</span><span class="operator">&gt;</span>hasUrls()) {
/// &#32;   &#32;     foreach (<span class="type"><a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a></span> url<span class="operator">,</span> event<span class="operator">-</span><span class="operator">&gt;</span>mimeData()<span class="operator">-</span><span class="operator">&gt;</span>urls()) {
/// &#32;   &#32;   &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
/// &#32;   &#32;     }
/// &#32;     }
///   }
///
/// </pre>
/// <p>There are three approaches for storing custom data in a <a href="http://doc.qt.io/qt-5/qmimedata.html">QMimeData</a> object:</p>
/// <ol class="1" type="1"><li>Custom data can be stored directly in a <a href="http://doc.qt.io/qt-5/qmimedata.html">QMimeData</a> object as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> using <a href="http://doc.qt.io/qt-5/qmimedata.html#setData">setData</a>(). For example:<pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span> csvData <span class="operator">=</span> <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>;
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qmimedata.html#QMimeData">QMimeData</a></span> <span class="operator">*</span>mimeData <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qmimedata.html#QMimeData">QMimeData</a></span>;
///   mimeData<span class="operator">-</span><span class="operator">&gt;</span>setData(<span class="string">"text/csv"</span><span class="operator">,</span> csvData);
///
/// </pre>
/// </li>
/// <li>We can subclass <a href="http://doc.qt.io/qt-5/qmimedata.html">QMimeData</a> and reimplement <a href="http://doc.qt.io/qt-5/qmimedata.html#hasFormat">hasFormat</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#formats">formats</a>(), and <a href="http://doc.qt.io/qt-5/qmimedata.html#retrieveData">retrieveData</a>().</li>
/// <li>If the drag and drop operation occurs within a single application, we can subclass <a href="http://doc.qt.io/qt-5/qmimedata.html">QMimeData</a> and add extra data in it, and use a <a href="http://doc.qt.io/qt-5/qobject.html#qobject_cast">qobject_cast</a>() in the receiver's drop event handler. For example:<pre class="cpp">
///
///   <span class="type">void</span> MyWidget<span class="operator">::</span>dropEvent(<span class="type"><a href="http://doc.qt.io/qt-5/qdropevent.html">QDropEvent</a></span> <span class="operator">*</span>event)
///   {
/// &#32;     <span class="keyword">const</span> MyMimeData <span class="operator">*</span>myData <span class="operator">=</span>
/// &#32;   &#32;   &#32;     qobject_cast<span class="operator">&lt;</span><span class="keyword">const</span> MyMimeData <span class="operator">*</span><span class="operator">&gt;</span>(event<span class="operator">-</span><span class="operator">&gt;</span>mimeData());
/// &#32;     <span class="keyword">if</span> (myData) {
/// &#32;   &#32;     <span class="comment">// access myData's data directly (not through QMimeData's API)</span>
/// &#32;     }
///   }
///
/// </pre>
/// </li>
/// </ol>
/// <a name="platform-specific-mime-types"></a></div>
#[repr(C)]
pub struct QMimeData {
    _unused: u8,
}
impl QMimeData {
    /// <p>Removes all the MIME type and data entries in the object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMimeData::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedata.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all the MIME type and data entries in the object.</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMimeData_clear(self as *mut crate::QMimeData)
    }

    /// <p>Returns a color if the data stored in the object represents a color (MIME type <code>application/x-color</code>); otherwise returns a null variant.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QMimeData::colorData() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedata.html#colorData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a color if the data stored in the object represents a color (MIME type <code>application/x-color</code>); otherwise returns a null variant.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> is used because <a href="http://doc.qt.io/qt-5/qmimedata.html">QMimeData</a> belongs to the Qt Core module, whereas <a href="http://doc.qt.io/qt-5/qcolor.html">QColor</a> belongs to Qt GUI. To convert the <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> to a <a href="http://doc.qt.io/qt-5/qcolor.html">QColor</a>, simply use <a href="http://doc.qt.io/qt-5/qvariant.html#qvariant_cast">qvariant_cast</a>(). For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">if</span> (event<span class="operator">-</span><span class="operator">&gt;</span>mimeData()<span class="operator">-</span><span class="operator">&gt;</span>hasColor()) {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcolor.html">QColor</a></span> color <span class="operator">=</span> qvariant_cast<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qcolor.html">QColor</a></span><span class="operator">&gt;</span>(event<span class="operator">-</span><span class="operator">&gt;</span>mimeData()<span class="operator">-</span><span class="operator">&gt;</span>colorData());
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   }
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmimedata.html#hasColor">hasColor</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#setColorData">setColorData</a>(), and <a href="http://doc.qt.io/qt-5/qmimedata.html#data">data</a>().</p></div>
    pub unsafe fn color_data(&self) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMimeData_colorData(self as *const crate::QMimeData);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the data stored in the object in the format described by the MIME type specified by <i>mimeType</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QMimeData::data(const QString& mimetype) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedata.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the data stored in the object in the format described by the MIME type specified by <i>mimeType</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmimedata.html#setData">setData</a>().</p></div>
    pub unsafe fn data(
        &self,
        mimetype: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMimeData_data(
            self as *const crate::QMimeData,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(mimetype)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list of formats supported by the object. This is a list of MIME types for which the object can return suitable data. The formats in the list are in a priority order.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QStringList QMimeData::formats() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedata.html#formats">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of formats supported by the object. This is a list of MIME types for which the object can return suitable data. The formats in the list are in a priority order.</p>
    /// <p>For the most common types of data, you can call the higher-level functions <a href="http://doc.qt.io/qt-5/qmimedata.html#hasText">hasText</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#hasHtml">hasHtml</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#hasUrls">hasUrls</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#hasImage">hasImage</a>(), and <a href="http://doc.qt.io/qt-5/qmimedata.html#hasColor">hasColor</a>() instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmimedata.html#hasFormat">hasFormat</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#setData">setData</a>(), and <a href="http://doc.qt.io/qt-5/qmimedata.html#data">data</a>().</p></div>
    pub unsafe fn formats(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMimeData_formats(self as *const crate::QMimeData);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if the object can return a color (MIME type <code>application/x-color</code>); otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMimeData::hasColor() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedata.html#hasColor">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the object can return a color (MIME type <code>application/x-color</code>); otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmimedata.html#setColorData">setColorData</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#colorData">colorData</a>(), and <a href="http://doc.qt.io/qt-5/qmimedata.html#hasFormat">hasFormat</a>().</p></div>
    pub unsafe fn has_color(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMimeData_hasColor(self as *const crate::QMimeData)
    }

    /// <p>Returns <code>true</code> if the object can return data for the MIME type specified by <i>mimeType</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QMimeData::hasFormat(const QString& mimetype) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedata.html#hasFormat">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the object can return data for the MIME type specified by <i>mimeType</i>; otherwise returns <code>false</code>.</p>
    /// <p>For the most common types of data, you can call the higher-level functions <a href="http://doc.qt.io/qt-5/qmimedata.html#hasText">hasText</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#hasHtml">hasHtml</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#hasUrls">hasUrls</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#hasImage">hasImage</a>(), and <a href="http://doc.qt.io/qt-5/qmimedata.html#hasColor">hasColor</a>() instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmimedata.html#formats">formats</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#setData">setData</a>(), and <a href="http://doc.qt.io/qt-5/qmimedata.html#data">data</a>().</p></div>
    pub unsafe fn has_format(
        &self,
        mimetype: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMimeData_hasFormat(
            self as *const crate::QMimeData,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(mimetype)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if the object can return HTML (MIME type <code>text/html</code>); otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMimeData::hasHtml() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedata.html#hasHtml">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the object can return HTML (MIME type <code>text/html</code>); otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmimedata.html#setHtml">setHtml</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#html">html</a>(), and <a href="http://doc.qt.io/qt-5/qmimedata.html#hasFormat">hasFormat</a>().</p></div>
    pub unsafe fn has_html(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMimeData_hasHtml(self as *const crate::QMimeData)
    }

    /// <p>Returns <code>true</code> if the object can return an image; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMimeData::hasImage() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedata.html#hasImage">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the object can return an image; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmimedata.html#setImageData">setImageData</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#imageData">imageData</a>(), and <a href="http://doc.qt.io/qt-5/qmimedata.html#hasFormat">hasFormat</a>().</p></div>
    pub unsafe fn has_image(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMimeData_hasImage(self as *const crate::QMimeData)
    }

    /// <p>Returns <code>true</code> if the object can return plain text (MIME type <code>text/plain</code>); otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMimeData::hasText() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedata.html#hasText">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the object can return plain text (MIME type <code>text/plain</code>); otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmimedata.html#setText">setText</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#text">text</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#hasHtml">hasHtml</a>(), and <a href="http://doc.qt.io/qt-5/qmimedata.html#hasFormat">hasFormat</a>().</p></div>
    pub unsafe fn has_text(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMimeData_hasText(self as *const crate::QMimeData)
    }

    /// <p>Returns <code>true</code> if the object can return a list of urls; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMimeData::hasUrls() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedata.html#hasUrls">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the object can return a list of urls; otherwise returns <code>false</code>.</p>
    /// <p>URLs correspond to the MIME type <code>text/uri-list</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmimedata.html#setUrls">setUrls</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#urls">urls</a>(), and <a href="http://doc.qt.io/qt-5/qmimedata.html#hasFormat">hasFormat</a>().</p></div>
    pub unsafe fn has_urls(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMimeData_hasUrls(self as *const crate::QMimeData)
    }

    /// <p>Returns a string if the data stored in the object is HTML (MIME type <code>text/html</code>); otherwise returns an empty string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QMimeData::html() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedata.html#html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string if the data stored in the object is HTML (MIME type <code>text/html</code>); otherwise returns an empty string.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmimedata.html#setHtml">setHtml</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#hasHtml">hasHtml</a>(), and <a href="http://doc.qt.io/qt-5/qmimedata.html#setData">setData</a>().</p></div>
    pub unsafe fn html(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMimeData_html(self as *const crate::QMimeData);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> storing a <a href="http://doc.qt.io/qt-5/qimage.html">QImage</a> if the object can return an image; otherwise returns a null variant.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QMimeData::imageData() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedata.html#imageData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> storing a <a href="http://doc.qt.io/qt-5/qimage.html">QImage</a> if the object can return an image; otherwise returns a null variant.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> is used because <a href="http://doc.qt.io/qt-5/qmimedata.html">QMimeData</a> belongs to the Qt Core module, whereas <a href="http://doc.qt.io/qt-5/qimage.html">QImage</a> belongs to Qt GUI. To convert the <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> to a <a href="http://doc.qt.io/qt-5/qimage.html">QImage</a>, simply use <a href="http://doc.qt.io/qt-5/qvariant.html#qvariant_cast">qvariant_cast</a>(). For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">if</span> (event<span class="operator">-</span><span class="operator">&gt;</span>mimeData()<span class="operator">-</span><span class="operator">&gt;</span>hasImage()) {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qimage.html">QImage</a></span> image <span class="operator">=</span> qvariant_cast<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qimage.html">QImage</a></span><span class="operator">&gt;</span>(event<span class="operator">-</span><span class="operator">&gt;</span>mimeData()<span class="operator">-</span><span class="operator">&gt;</span>imageData());
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   }
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmimedata.html#setImageData">setImageData</a>() and <a href="http://doc.qt.io/qt-5/qmimedata.html#hasImage">hasImage</a>().</p></div>
    pub unsafe fn image_data(&self) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMimeData_imageData(self as *const crate::QMimeData);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QMimeData::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMimeData_metaObject(self as *const crate::QMimeData);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs a new MIME data object with no data in it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMimeData::QMimeData()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedata.html#QMimeData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new MIME data object with no data in it.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QMimeData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMimeData_QMimeData();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QMimeData::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMimeData_qt_metacall(
            self as *mut crate::QMimeData,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QMimeData::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMimeData_qt_metacast(
            self as *mut crate::QMimeData,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Removes the data entry for <i>mimeType</i> in the object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMimeData::removeFormat(const QString& mimetype)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedata.html#removeFormat">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the data entry for <i>mimeType</i> in the object.</p>
    /// <p>This function was introduced in  Qt 4.4.</p></div>
    pub unsafe fn remove_format(
        &mut self,
        mimetype: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QMimeData_removeFormat(
            self as *mut crate::QMimeData,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(mimetype)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the color data in the object to the given <i>color</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMimeData::setColorData(const QVariant& color)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedata.html#setColorData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the color data in the object to the given <i>color</i>.</p>
    /// <p>Colors correspond to the MIME type <code>application/x-color</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmimedata.html#colorData">colorData</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#hasColor">hasColor</a>(), and <a href="http://doc.qt.io/qt-5/qmimedata.html#setData">setData</a>().</p></div>
    pub unsafe fn set_color_data(
        &mut self,
        color: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QMimeData_setColorData(
            self as *mut crate::QMimeData,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(color)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the data associated with the MIME type given by <i>mimeType</i> to the specified <i>data</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMimeData::setData(const QString& mimetype, const QByteArray& data)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedata.html#setData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the data associated with the MIME type given by <i>mimeType</i> to the specified <i>data</i>.</p>
    /// <p>For the most common types of data, you can call the higher-level functions <a href="http://doc.qt.io/qt-5/qmimedata.html#setText">setText</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#setHtml">setHtml</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#setUrls">setUrls</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#setImageData">setImageData</a>(), and <a href="http://doc.qt.io/qt-5/qmimedata.html#setColorData">setColorData</a>() instead.</p>
    /// <p>Note that if you want to use a custom data type in an item view drag and drop operation, you must register it as a Qt <a href="http://doc.qt.io/qt-5/qmetatype.html">meta type</a>, using the <a href="http://doc.qt.io/qt-5/qmetatype.html#Q_DECLARE_METATYPE">Q_DECLARE_METATYPE</a>() macro, and implement stream operators for it. The stream operators must then be registered with the <a href="http://doc.qt.io/qt-5/qmetatype.html#qRegisterMetaTypeStreamOperators">qRegisterMetaTypeStreamOperators</a>() function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmimedata.html#data">data</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#hasFormat">hasFormat</a>(), <a href="http://doc.qt.io/qt-5/qmetatype.html">QMetaType</a>, and <a href="http://doc.qt.io/qt-5/qmetatype.html#qRegisterMetaTypeStreamOperators">qRegisterMetaTypeStreamOperators</a>().</p></div>
    pub unsafe fn set_data(
        &mut self,
        mimetype: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QMimeData_setData(
            self as *mut crate::QMimeData,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(mimetype)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(data)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets <i>html</i> as the HTML (MIME type <code>text/html</code>) used to represent the data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMimeData::setHtml(const QString& html)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedata.html#setHtml">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets <i>html</i> as the HTML (MIME type <code>text/html</code>) used to represent the data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmimedata.html#html">html</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#hasHtml">hasHtml</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#setText">setText</a>(), and <a href="http://doc.qt.io/qt-5/qmimedata.html#setData">setData</a>().</p></div>
    pub unsafe fn set_html(
        &mut self,
        html: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QMimeData_setHtml(
            self as *mut crate::QMimeData,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(html).as_raw_ptr(),
        )
    }

    /// <p>Sets the data in the object to the given <i>image</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMimeData::setImageData(const QVariant& image)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedata.html#setImageData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the data in the object to the given <i>image</i>.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> is used because <a href="http://doc.qt.io/qt-5/qmimedata.html">QMimeData</a> belongs to the Qt Core module, whereas <a href="http://doc.qt.io/qt-5/qimage.html">QImage</a> belongs to Qt GUI. The conversion from <a href="http://doc.qt.io/qt-5/qimage.html">QImage</a> to <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> is implicit. For example:</p>
    /// <pre class="cpp">
    ///
    ///   mimeData<span class="operator">-</span><span class="operator">&gt;</span>setImageData(<span class="type"><a href="http://doc.qt.io/qt-5/qimage.html">QImage</a></span>(<span class="string">"beautifulfjord.png"</span>));
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmimedata.html#imageData">imageData</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#hasImage">hasImage</a>(), and <a href="http://doc.qt.io/qt-5/qmimedata.html#setData">setData</a>().</p></div>
    pub unsafe fn set_image_data(
        &mut self,
        image: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QMimeData_setImageData(
            self as *mut crate::QMimeData,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(image)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets <i>text</i> as the plain text (MIME type <code>text/plain</code>) used to represent the data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMimeData::setText(const QString& text)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedata.html#setText">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets <i>text</i> as the plain text (MIME type <code>text/plain</code>) used to represent the data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmimedata.html#text">text</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#hasText">hasText</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#setHtml">setHtml</a>(), and <a href="http://doc.qt.io/qt-5/qmimedata.html#setData">setData</a>().</p></div>
    pub unsafe fn set_text(
        &mut self,
        text: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QMimeData_setText(
            self as *mut crate::QMimeData,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(text).as_raw_ptr(),
        )
    }

    /// <p>Sets the URLs stored in the MIME data object to those specified by <i>urls</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMimeData::setUrls(const QList<QUrl>& urls)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedata.html#setUrls">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the URLs stored in the MIME data object to those specified by <i>urls</i>.</p>
    /// <p>URLs correspond to the MIME type <code>text/uri-list</code>.</p>
    /// <p>Since Qt 5.0, setUrls also exports the urls as plain text, if <a href="http://doc.qt.io/qt-5/qmimedata.html#setText">setText</a> was not called before, to make it possible to drop them into any lineedit and text editor.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmimedata.html#urls">urls</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#hasUrls">hasUrls</a>(), and <a href="http://doc.qt.io/qt-5/qmimedata.html#setData">setData</a>().</p></div>
    pub unsafe fn set_urls(
        &mut self,
        urls: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQUrl>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QMimeData_setUrls(
            self as *mut crate::QMimeData,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQUrl>>::cast_into(urls)
                .as_raw_ptr(),
        )
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMimeData_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a plain text (MIME type <code>text/plain</code>) representation of the data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QMimeData::text() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedata.html#text">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a plain text (MIME type <code>text/plain</code>) representation of the data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmimedata.html#setText">setText</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#hasText">hasText</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#html">html</a>(), and <a href="http://doc.qt.io/qt-5/qmimedata.html#data">data</a>().</p></div>
    pub unsafe fn text(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMimeData_text(self as *const crate::QMimeData);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QMimeData::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMimeData_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QMimeData::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMimeData_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list of URLs contained within the MIME data object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QUrl> QMimeData::urls() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedata.html#urls">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of URLs contained within the MIME data object.</p>
    /// <p>URLs correspond to the MIME type <code>text/uri-list</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmimedata.html#setUrls">setUrls</a>(), <a href="http://doc.qt.io/qt-5/qmimedata.html#hasUrls">hasUrls</a>(), and <a href="http://doc.qt.io/qt-5/qmimedata.html#data">data</a>().</p></div>
    pub unsafe fn urls(&self) -> ::cpp_utils::CppBox<crate::QListOfQUrl> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMimeData_urls(self as *const crate::QMimeData);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qmimetype.html">QMimeType</a> class describes types of file or data, represented by a MIME type string.</p>
///
/// C++ class: <span style='color: green;'>```QMimeType```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qmimetype.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmimetype.html">QMimeType</a> class describes types of file or data, represented by a MIME type string.</p>
/// <p>For instance a file named "readme.txt" has the MIME type "text/plain". The MIME type can be determined from the file name, or from the file contents, or from both. MIME type determination can also be done on buffers of data not coming from files.</p>
/// <p>Determining the MIME type of a file can be useful to make sure your application supports it. It is also useful in file-manager-like applications or widgets, in order to display an appropriate <a href="http://doc.qt.io/qt-5/qmimetype.html#iconName">icon</a> for the file, or even the descriptive <a href="http://doc.qt.io/qt-5/qmimetype.html#comment">comment</a> in detailed views.</p>
/// <p>To check if a file has the expected MIME type, you should use <a href="http://doc.qt.io/qt-5/qmimetype.html#inherits">inherits</a>() rather than a simple string comparison based on the <a href="http://doc.qt.io/qt-5/qmimetype.html#name">name</a>(). This is because MIME types can inherit from each other: for instance a C source file is a specific type of plain text file, so text/x-csrc inherits text/plain.</p></div>
#[repr(C)]
pub struct QMimeType {
    _unused: u8,
}
impl QMimeType {
    /// <p>Return the list of aliases of this mimetype.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QMimeType::aliases() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimetype.html#aliases">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Return the list of aliases of this mimetype.</p>
    /// <p>For instance, for text/csv, the returned list would be: text/x-csv, text/x-comma-separated-values.</p>
    /// <p>Note that all <a href="http://doc.qt.io/qt-5/qmimetype.html">QMimeType</a> instances refer to proper mimetypes, never to aliases directly.</p>
    /// <p>The order of the aliases in the list is undefined.</p></div>
    pub unsafe fn aliases(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMimeType_aliases(self as *const crate::QMimeType);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Return all the parent mimetypes of this mimetype, direct and indirect. This includes the parent(s) of its parent(s), etc.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QMimeType::allAncestors() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimetype.html#allAncestors">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Return all the parent mimetypes of this mimetype, direct and indirect. This includes the parent(s) of its parent(s), etc.</p>
    /// <p>For instance, for image/svg+xml the list would be: application/xml, text/plain, application/octet-stream.</p>
    /// <p>Note that application/octet-stream is the ultimate parent for all types of files (but not directories).</p></div>
    pub unsafe fn all_ancestors(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMimeType_allAncestors(self as *const crate::QMimeType);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the description of the MIME type to be displayed on user interfaces.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QMimeType::comment() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimetype.html#comment">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the description of the MIME type to be displayed on user interfaces.</p>
    /// <p>The default language (QLocale().<a href="http://doc.qt.io/qt-5/qmimetype.html#name">name</a>()) is used to select the appropriate translation.</p></div>
    pub unsafe fn comment(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMimeType_comment(self as *const crate::QMimeType);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Assigns the data of <i>other</i> to this <a href="http://doc.qt.io/qt-5/qmimetype.html">QMimeType</a> object, and returns a reference to this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMimeType& QMimeType::operator=(const QMimeType& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimetype.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns the data of <i>other</i> to this <a href="http://doc.qt.io/qt-5/qmimetype.html">QMimeType</a> object, and returns a reference to this object.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMimeType>>,
    ) -> ::cpp_utils::MutRef<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMimeType_operator_(
            self as *mut crate::QMimeType,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMimeType>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a filter string usable for a file dialog.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QMimeType::filterString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimetype.html#filterString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a filter string usable for a file dialog.</p></div>
    pub unsafe fn filter_string(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMimeType_filterString(self as *const crate::QMimeType);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the file name of a generic icon that represents the MIME type.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QMimeType::genericIconName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimetype.html#genericIconName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the file name of a generic icon that represents the MIME type.</p>
    /// <p>This should be used if the icon returned by <a href="http://doc.qt.io/qt-5/qmimetype.html#iconName">iconName</a>() cannot be found on the system. It is used for categories of similar types (like spreadsheets or archives) that can use a common icon. The freedesktop.org Icon Naming Specification lists a set of such icon names.</p>
    /// <p>The icon name can be given to <a href="http://doc.qt.io/qt-5/qicon.html#fromTheme">QIcon::fromTheme</a>() in order to load the icon.</p></div>
    pub unsafe fn generic_icon_name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMimeType_genericIconName(
            self as *const crate::QMimeType,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the list of glob matching patterns.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QMimeType::globPatterns() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimetype.html#globPatterns">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the list of glob matching patterns.</p></div>
    pub unsafe fn glob_patterns(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMimeType_globPatterns(self as *const crate::QMimeType);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the file name of an icon image that represents the MIME type.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QMimeType::iconName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimetype.html#iconName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the file name of an icon image that represents the MIME type.</p>
    /// <p>The icon name can be given to <a href="http://doc.qt.io/qt-5/qicon.html#fromTheme">QIcon::fromTheme</a>() in order to load the icon.</p></div>
    pub unsafe fn icon_name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMimeType_iconName(self as *const crate::QMimeType);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if this mimetype is <i>mimeTypeName</i>, or inherits <i>mimeTypeName</i> (see <a href="http://doc.qt.io/qt-5/qmimetype.html#parentMimeTypes">parentMimeTypes</a>()), or <i>mimeTypeName</i> is an alias for this mimetype.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMimeType::inherits(const QString& mimeTypeName) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimetype.html#inherits">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this mimetype is <i>mimeTypeName</i>, or inherits <i>mimeTypeName</i> (see <a href="http://doc.qt.io/qt-5/qmimetype.html#parentMimeTypes">parentMimeTypes</a>()), or <i>mimeTypeName</i> is an alias for this mimetype.</p></div>
    pub unsafe fn inherits(
        &self,
        mime_type_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMimeType_inherits(
            self as *const crate::QMimeType,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(mime_type_name)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if this MIME type is the default MIME type which applies to all files: application/octet-stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMimeType::isDefault() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimetype.html#isDefault">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this MIME type is the default MIME type which applies to all files: application/octet-stream.</p></div>
    pub unsafe fn is_default(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMimeType_isDefault(self as *const crate::QMimeType)
    }

    /// <p>Returns <code>true</code> if the <a href="http://doc.qt.io/qt-5/qmimetype.html">QMimeType</a> object contains valid data, otherwise returns <code>false</code>. A valid MIME type has a non-empty <a href="http://doc.qt.io/qt-5/qmimetype.html#name">name</a>(). The invalid MIME type is the default-constructed <a href="http://doc.qt.io/qt-5/qmimetype.html">QMimeType</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMimeType::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimetype.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the <a href="http://doc.qt.io/qt-5/qmimetype.html">QMimeType</a> object contains valid data, otherwise returns <code>false</code>. A valid MIME type has a non-empty <a href="http://doc.qt.io/qt-5/qmimetype.html#name">name</a>(). The invalid MIME type is the default-constructed <a href="http://doc.qt.io/qt-5/qmimetype.html">QMimeType</a>.</p></div>
    pub unsafe fn is_valid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMimeType_isValid(self as *const crate::QMimeType)
    }

    /// <p>Returns the name of the MIME type.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QMimeType::name() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimetype.html#name">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the name of the MIME type.</p></div>
    pub unsafe fn name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMimeType_name(self as *const crate::QMimeType);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs this <a href="http://doc.qt.io/qt-5/qmimetype.html">QMimeType</a> object initialized with default property values that indicate an invalid MIME type.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMimeType::QMimeType()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimetype.html#QMimeType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs this <a href="http://doc.qt.io/qt-5/qmimetype.html">QMimeType</a> object initialized with default property values that indicate an invalid MIME type.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMimeType_QMimeType();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs this <a href="http://doc.qt.io/qt-5/qmimetype.html">QMimeType</a> object as a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMimeType::QMimeType(const QMimeType& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimetype.html#QMimeType-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs this <a href="http://doc.qt.io/qt-5/qmimetype.html">QMimeType</a> object as a copy of <i>other</i>.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMimeType>>,
    ) -> ::cpp_utils::CppBox<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMimeType_QMimeType1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMimeType>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>A type is a subclass of another type if any instance of the first type is also an instance of the second. For example, all image/svg+xml files are also text/xml, text/plain and application/octet-stream files. Subclassing is about the format, rather than the category of the data (for example, there is no 'generic spreadsheet' class that all spreadsheets inherit from). Conversely, the parent mimetype of image/svg+xml is text/xml.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QMimeType::parentMimeTypes() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimetype.html#parentMimeTypes">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>A type is a subclass of another type if any instance of the first type is also an instance of the second. For example, all image/svg+xml files are also text/xml, text/plain and application/octet-stream files. Subclassing is about the format, rather than the category of the data (for example, there is no 'generic spreadsheet' class that all spreadsheets inherit from). Conversely, the parent mimetype of image/svg+xml is text/xml.</p>
    /// <p>A mimetype can have multiple parents. For instance application/x-perl has two parents: application/x-executable and text/plain. This makes it possible to both execute perl scripts, and to open them in text editors.</p></div>
    pub unsafe fn parent_mime_types(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMimeType_parentMimeTypes(
            self as *const crate::QMimeType,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the preferred suffix for the MIME type. No leading dot is included, so for instance this would return "pdf" for application/pdf. The return value can be empty, for mime types which do not have any suffixes associated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QMimeType::preferredSuffix() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimetype.html#preferredSuffix">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the preferred suffix for the MIME type. No leading dot is included, so for instance this would return "pdf" for application/pdf. The return value can be empty, for mime types which do not have any suffixes associated.</p></div>
    pub unsafe fn preferred_suffix(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMimeType_preferredSuffix(
            self as *const crate::QMimeType,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMimeType_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the known suffixes for the MIME type. No leading dot is included, so for instance this would return "jpg", "jpeg" for image/jpeg.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QMimeType::suffixes() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimetype.html#suffixes">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the known suffixes for the MIME type. No leading dot is included, so for instance this would return "jpg", "jpeg" for image/jpeg.</p></div>
    pub unsafe fn suffixes(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMimeType_suffixes(self as *const crate::QMimeType);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Swaps <a href="http://doc.qt.io/qt-5/qmimetype.html">QMimeType</a> <i>other</i> with this <a href="http://doc.qt.io/qt-5/qmimetype.html">QMimeType</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMimeType::swap(QMimeType& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimetype.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps <a href="http://doc.qt.io/qt-5/qmimetype.html">QMimeType</a> <i>other</i> with this <a href="http://doc.qt.io/qt-5/qmimetype.html">QMimeType</a> object.</p>
    /// <p>This operation is very fast and never fails.</p>
    /// <p>The swap() method helps with the implementation of assignment operators in an exception-safe way. For more information consult <a href="http://en.wikibooks.org/wiki/More_C++_Idioms/Copy-and-swap">More C++ Idioms - Copy-and-swap</a>.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QMimeType>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QMimeType_swap(
            self as *mut crate::QMimeType,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QMimeType>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }
}

pub mod q_mime_database {
    //! C++ type: <span style='color: green;'>```QMimeDatabase```</span>

    /// <p>This enum specifies how matching a file to a MIME type is performed.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QMimeDatabase::MatchMode```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedatabase.html#MatchMode-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum specifies how matching a file to a MIME type is performed.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct MatchMode(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for MatchMode {
        fn from(value: ::std::os::raw::c_int) -> Self {
            MatchMode(value)
        }
    }

    impl From<MatchMode> for ::std::os::raw::c_int {
        fn from(value: MatchMode) -> Self {
            value.0
        }
    }

    impl MatchMode {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl MatchMode {
        /// Both the file name and content are used to look for a match (C++ enum variant: <span style='color: green;'>```MatchDefault = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const MatchDefault: crate::q_mime_database::MatchMode =
            crate::q_mime_database::MatchMode(0);
        /// Only the file name is used to look for a match (C++ enum variant: <span style='color: green;'>```MatchExtension = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const MatchExtension: crate::q_mime_database::MatchMode =
            crate::q_mime_database::MatchMode(1);
        /// The file content is used to look for a match (C++ enum variant: <span style='color: green;'>```MatchContent = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const MatchContent: crate::q_mime_database::MatchMode =
            crate::q_mime_database::MatchMode(2);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qmimedatabase.html">QMimeDatabase</a> class maintains a database of MIME types.</p>
///
/// C++ class: <span style='color: green;'>```QMimeDatabase```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qmimedatabase.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmimedatabase.html">QMimeDatabase</a> class maintains a database of MIME types.</p>
/// <p>The MIME type database is provided by the freedesktop.org shared-mime-info project. If the MIME type database cannot be found on the system, as is the case on most Windows, <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, and iOS systems, Qt will use its own copy of it.</p>
/// <p>Applications which want to define custom MIME types need to install an XML file into the locations searched for MIME definitions. These locations can be queried with</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qstandardpaths.html">QStandardPaths</a></span><span class="operator">::</span>locateAll(<span class="type"><a href="http://doc.qt.io/qt-5/qstandardpaths.html">QStandardPaths</a></span><span class="operator">::</span>GenericDataLocation<span class="operator">,</span> QLatin1String(<span class="string">"mime/packages"</span>)<span class="operator">,</span>
/// &#32;   &#32;   &#32;   &#32;   &#32;   &#32;   &#32;   <span class="type"><a href="http://doc.qt.io/qt-5/qstandardpaths.html">QStandardPaths</a></span><span class="operator">::</span>LocateDirectory);
///
/// </pre>
/// <p>On a typical Unix system, this will be /usr/share/mime/packages/, but it is also possible to extend the list of directories by setting the environment variable <code>XDG_DATA_DIRS</code>. For instance adding /opt/myapp/share to <code>XDG_DATA_DIRS</code> will result in /opt/myapp/share/mime/packages/ being searched for MIME definitions.</p>
/// <p>Here is an example of MIME XML:</p>
/// <pre class="cpp">
///
///   <span class="operator">&lt;</span><span class="operator">?</span>xml version<span class="operator">=</span><span class="string">"1.0"</span> encoding<span class="operator">=</span><span class="string">"UTF-8"</span><span class="operator">?</span><span class="operator">&gt;</span>
///   <span class="operator">&lt;</span>mime<span class="operator">-</span>info xmlns<span class="operator">=</span><span class="string">"http://www.freedesktop.org/standards/shared-mime-info"</span><span class="operator">&gt;</span>
/// &#32;   <span class="operator">&lt;</span>mime<span class="operator">-</span>type type<span class="operator">=</span><span class="string">"application/vnd.qt.qmakeprofile"</span><span class="operator">&gt;</span>
/// &#32;     <span class="operator">&lt;</span>comment xml:lang<span class="operator">=</span><span class="string">"en"</span><span class="operator">&gt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span> qmake Profile<span class="operator">&lt;</span><span class="operator">/</span>comment<span class="operator">&gt;</span>
/// &#32;     <span class="operator">&lt;</span>glob pattern<span class="operator">=</span><span class="string">"*.pro"</span> weight<span class="operator">=</span><span class="string">"50"</span><span class="operator">/</span><span class="operator">&gt;</span>
/// &#32;   <span class="operator">&lt;</span><span class="operator">/</span>mime<span class="operator">-</span>type<span class="operator">&gt;</span>
///   <span class="operator">&lt;</span><span class="operator">/</span>mime<span class="operator">-</span>info<span class="operator">&gt;</span>
///
/// </pre>
/// <p>For more details about the syntax of XML MIME definitions, including defining "magic" in order to detect MIME types based on data as well, read the Shared Mime Info specification at http://standards.freedesktop.org/shared-mime-info-spec/shared-mime-info-spec-latest.html</p>
/// <p>On Unix systems, a binary cache is used for more performance. This cache is generated by the command "update-mime-database path", where path would be /opt/myapp/share/mime in the above example. Make sure to run this command when installing the MIME type definition file.</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qmimedatabase.html#QMimeDatabase">QMimeDatabase</a></span> db;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qmimetype.html">QMimeType</a></span> mime <span class="operator">=</span> db<span class="operator">.</span>mimeTypeForFile(fileName);
///   <span class="keyword">if</span> (mime<span class="operator">.</span>inherits(<span class="string">"text/plain"</span>)) {
/// &#32;     <span class="comment">// The file is plain text, we can display it in a QTextEdit</span>
///   }
///
/// </pre></div>
#[repr(C)]
pub struct QMimeDatabase {
    _unused: u8,
}
impl QMimeDatabase {
    /// <p>Returns the list of all available MIME types.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType> QMimeDatabase::allMimeTypes() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedatabase.html#allMimeTypes">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the list of all available MIME types.</p>
    /// <p>This can be useful for showing all MIME types to the user, for instance in a MIME type editor. Do not use unless really necessary in other cases though, prefer using the <a href="http://doc.qt.io/qt-5/qmimedatabase.html#mimeTypeForData">mimeTypeForXxx()</a> methods for performance reasons.</p></div>
    pub unsafe fn all_mime_types(&self) -> ::cpp_utils::CppBox<crate::QListOfQMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMimeDatabase_allMimeTypes(
            self as *const crate::QMimeDatabase,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a MIME type for <i>data</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMimeType QMimeDatabase::mimeTypeForData(const QByteArray& data) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedatabase.html#mimeTypeForData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a MIME type for <i>data</i>.</p>
    /// <p>A valid MIME type is always returned. If <i>data</i> doesn't match any known MIME type data, the default MIME type (application/octet-stream) is returned.</p></div>
    pub unsafe fn mime_type_for_data_q_byte_array(
        &self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMimeDatabase_mimeTypeForData(
            self as *const crate::QMimeDatabase,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(data)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a MIME type for the data in <i>device</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMimeType QMimeDatabase::mimeTypeForData(QIODevice* device) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedatabase.html#mimeTypeForData-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a MIME type for the data in <i>device</i>.</p>
    /// <p>A valid MIME type is always returned. If the data in <i>device</i> doesn't match any known MIME type data, the default MIME type (application/octet-stream) is returned.</p></div>
    pub unsafe fn mime_type_for_data_q_io_device(
        &self,
        device: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QIODevice>>,
    ) -> ::cpp_utils::CppBox<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMimeDatabase_mimeTypeForData1(
            self as *const crate::QMimeDatabase,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QIODevice>>::cast_into(device)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a MIME type for the file named <i>fileName</i> using <i>mode</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMimeType QMimeDatabase::mimeTypeForFile(const QString& fileName, QMimeDatabase::MatchMode mode = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedatabase.html#mimeTypeForFile-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a MIME type for the file named <i>fileName</i> using <i>mode</i>.</p>
    /// <p>This is an overloaded function.</p></div>
    pub unsafe fn mime_type_for_file_q_string_match_mode(
        &self,
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        mode: crate::q_mime_database::MatchMode,
    ) -> ::cpp_utils::CppBox<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMimeDatabase_mimeTypeForFile(
            self as *const crate::QMimeDatabase,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
            mode,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a MIME type for <i>fileInfo</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMimeType QMimeDatabase::mimeTypeForFile(const QFileInfo& fileInfo, QMimeDatabase::MatchMode mode = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedatabase.html#mimeTypeForFile">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a MIME type for <i>fileInfo</i>.</p>
    /// <p>A valid MIME type is always returned.</p>
    /// <p>The default matching algorithm looks at both the file name and the file contents, if necessary. The file extension has priority over the contents, but the contents will be used if the file extension is unknown, or matches multiple MIME types. If <i>fileInfo</i> is a Unix symbolic link, the file that it refers to will be used instead. If the file doesn't match any known pattern or data, the default MIME type (application/octet-stream) is returned.</p>
    /// <p>When <i>mode</i> is set to <a href="http://doc.qt.io/qt-5/qmimedatabase.html#MatchMode-enum">MatchExtension</a>, only the file name is used, not the file contents. The file doesn't even have to exist. If the file name doesn't match any known pattern, the default MIME type (application/octet-stream) is returned. If multiple MIME types match this file, the first one (alphabetically) is returned.</p>
    /// <p>When <i>mode</i> is set to <a href="http://doc.qt.io/qt-5/qmimedatabase.html#MatchMode-enum">MatchContent</a>, and the file is readable, only the file contents are used to determine the MIME type. This is equivalent to calling <a href="http://doc.qt.io/qt-5/qmimedatabase.html#mimeTypeForData">mimeTypeForData</a> with a <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> as input device.</p>
    /// <p><i>fileInfo</i> may refer to an absolute or relative path.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmimetype.html#isDefault">QMimeType::isDefault</a>() and <a href="http://doc.qt.io/qt-5/qmimedatabase.html#mimeTypeForData">mimeTypeForData</a>().</p></div>
    pub unsafe fn mime_type_for_file_q_file_info_match_mode(
        &self,
        file_info: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QFileInfo>>,
        mode: crate::q_mime_database::MatchMode,
    ) -> ::cpp_utils::CppBox<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMimeDatabase_mimeTypeForFile1(
            self as *const crate::QMimeDatabase,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QFileInfo>>::cast_into(file_info)
                .as_raw_ptr(),
            mode,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a MIME type for the file named <i>fileName</i> using <i>mode</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMimeType QMimeDatabase::mimeTypeForFile(const QString& fileName) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedatabase.html#mimeTypeForFile-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a MIME type for the file named <i>fileName</i> using <i>mode</i>.</p>
    /// <p>This is an overloaded function.</p></div>
    pub unsafe fn mime_type_for_file_q_string(
        &self,
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMimeDatabase_mimeTypeForFile2(
            self as *const crate::QMimeDatabase,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a MIME type for <i>fileInfo</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMimeType QMimeDatabase::mimeTypeForFile(const QFileInfo& fileInfo) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedatabase.html#mimeTypeForFile">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a MIME type for <i>fileInfo</i>.</p>
    /// <p>A valid MIME type is always returned.</p>
    /// <p>The default matching algorithm looks at both the file name and the file contents, if necessary. The file extension has priority over the contents, but the contents will be used if the file extension is unknown, or matches multiple MIME types. If <i>fileInfo</i> is a Unix symbolic link, the file that it refers to will be used instead. If the file doesn't match any known pattern or data, the default MIME type (application/octet-stream) is returned.</p>
    /// <p>When <i>mode</i> is set to <a href="http://doc.qt.io/qt-5/qmimedatabase.html#MatchMode-enum">MatchExtension</a>, only the file name is used, not the file contents. The file doesn't even have to exist. If the file name doesn't match any known pattern, the default MIME type (application/octet-stream) is returned. If multiple MIME types match this file, the first one (alphabetically) is returned.</p>
    /// <p>When <i>mode</i> is set to <a href="http://doc.qt.io/qt-5/qmimedatabase.html#MatchMode-enum">MatchContent</a>, and the file is readable, only the file contents are used to determine the MIME type. This is equivalent to calling <a href="http://doc.qt.io/qt-5/qmimedatabase.html#mimeTypeForData">mimeTypeForData</a> with a <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> as input device.</p>
    /// <p><i>fileInfo</i> may refer to an absolute or relative path.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmimetype.html#isDefault">QMimeType::isDefault</a>() and <a href="http://doc.qt.io/qt-5/qmimedatabase.html#mimeTypeForData">mimeTypeForData</a>().</p></div>
    pub unsafe fn mime_type_for_file_q_file_info(
        &self,
        file_info: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QFileInfo>>,
    ) -> ::cpp_utils::CppBox<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMimeDatabase_mimeTypeForFile3(
            self as *const crate::QMimeDatabase,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QFileInfo>>::cast_into(file_info)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a MIME type for the given <i>fileName</i> and <i>device</i> data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMimeType QMimeDatabase::mimeTypeForFileNameAndData(const QString& fileName, QIODevice* device) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedatabase.html#mimeTypeForFileNameAndData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a MIME type for the given <i>fileName</i> and <i>device</i> data.</p>
    /// <p>This overload can be useful when the file is remote, and we started to download some of its data in a device. This allows to do full MIME type matching for remote files as well.</p>
    /// <p>If the device is not open, it will be opened by this function, and closed after the MIME type detection is completed.</p>
    /// <p>A valid MIME type is always returned. If <i>device</i> data doesn't match any known MIME type data, the default MIME type (application/octet-stream) is returned.</p>
    /// <p>This method looks at both the file name and the file contents, if necessary. The file extension has priority over the contents, but the contents will be used if the file extension is unknown, or matches multiple MIME types.</p></div>
    pub unsafe fn mime_type_for_file_name_and_data_q_string_q_io_device(
        &self,
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        device: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QIODevice>>,
    ) -> ::cpp_utils::CppBox<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMimeDatabase_mimeTypeForFileNameAndData(
            self as *const crate::QMimeDatabase,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QIODevice>>::cast_into(device)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a MIME type for the given <i>fileName</i> and device <i>data</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMimeType QMimeDatabase::mimeTypeForFileNameAndData(const QString& fileName, const QByteArray& data) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedatabase.html#mimeTypeForFileNameAndData-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a MIME type for the given <i>fileName</i> and device <i>data</i>.</p>
    /// <p>This overload can be useful when the file is remote, and we started to download some of its data. This allows to do full MIME type matching for remote files as well.</p>
    /// <p>A valid MIME type is always returned. If <i>data</i> doesn't match any known MIME type data, the default MIME type (application/octet-stream) is returned.</p>
    /// <p>This method looks at both the file name and the file contents, if necessary. The file extension has priority over the contents, but the contents will be used if the file extension is unknown, or matches multiple MIME types.</p></div>
    pub unsafe fn mime_type_for_file_name_and_data_q_string_q_byte_array(
        &self,
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMimeDatabase_mimeTypeForFileNameAndData1(
            self as *const crate::QMimeDatabase,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(data)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a MIME type for <i>nameOrAlias</i> or an invalid one if none found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMimeType QMimeDatabase::mimeTypeForName(const QString& nameOrAlias) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedatabase.html#mimeTypeForName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a MIME type for <i>nameOrAlias</i> or an invalid one if none found.</p></div>
    pub unsafe fn mime_type_for_name(
        &self,
        name_or_alias: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMimeDatabase_mimeTypeForName(
            self as *const crate::QMimeDatabase,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name_or_alias)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a MIME type for <i>url</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMimeType QMimeDatabase::mimeTypeForUrl(const QUrl& url) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedatabase.html#mimeTypeForUrl">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a MIME type for <i>url</i>.</p>
    /// <p>If the URL is a local file, this calls <a href="http://doc.qt.io/qt-5/qmimedatabase.html#mimeTypeForFile">mimeTypeForFile</a>.</p>
    /// <p>Otherwise the matching is done based on the file name only, except for schemes where file names don't mean much, like HTTP. This method always returns the default mimetype for HTTP URLs, use <a href="http://doc.qt.io/qt-5/qnetworkaccessmanager.html">QNetworkAccessManager</a> to handle HTTP URLs properly.</p>
    /// <p>A valid MIME type is always returned. If <i>url</i> doesn't match any known MIME type data, the default MIME type (application/octet-stream) is returned.</p></div>
    pub unsafe fn mime_type_for_url(
        &self,
        url: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
    ) -> ::cpp_utils::CppBox<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMimeDatabase_mimeTypeForUrl(
            self as *const crate::QMimeDatabase,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(url).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the MIME types for the file name <i>fileName</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType> QMimeDatabase::mimeTypesForFileName(const QString& fileName) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedatabase.html#mimeTypesForFileName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the MIME types for the file name <i>fileName</i>.</p>
    /// <p>If the file name doesn't match any known pattern, an empty list is returned. If multiple MIME types match this file, they are all returned.</p>
    /// <p>This function does not try to open the file. To also use the content when determining the MIME type, use <a href="http://doc.qt.io/qt-5/qmimedatabase.html#mimeTypeForFile">mimeTypeForFile</a>() or <a href="http://doc.qt.io/qt-5/qmimedatabase.html#mimeTypeForFileNameAndData">mimeTypeForFileNameAndData</a>() instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmimedatabase.html#mimeTypeForFile">mimeTypeForFile</a>().</p></div>
    pub unsafe fn mime_types_for_file_name(
        &self,
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMimeDatabase_mimeTypesForFileName(
            self as *const crate::QMimeDatabase,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qmimedatabase.html">QMimeDatabase</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMimeDatabase::QMimeDatabase()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedatabase.html#QMimeDatabase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qmimedatabase.html">QMimeDatabase</a> object.</p>
    /// <p>It is perfectly OK to create an instance of <a href="http://doc.qt.io/qt-5/qmimedatabase.html">QMimeDatabase</a> every time you need to perform a lookup. The parsing of mimetypes is done on demand (when shared-mime-info is installed) or when the very first instance is constructed (when parsing XML files directly).</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QMimeDatabase> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMimeDatabase_QMimeDatabase();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the suffix for the file <i>fileName</i>, as known by the MIME database.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QMimeDatabase::suffixForFileName(const QString& fileName) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedatabase.html#suffixForFileName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the suffix for the file <i>fileName</i>, as known by the MIME database.</p>
    /// <p>This allows to pre-select "tar.bz2" for foo.tar.bz2, but still only "txt" for my.file.with.dots.txt.</p></div>
    pub unsafe fn suffix_for_file_name(
        &self,
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMimeDatabase_suffixForFileName(
            self as *const crate::QMimeDatabase,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qobjectcleanuphandler.html">QObjectCleanupHandler</a> class watches the lifetime of multiple QObjects.</p>
///
/// C++ class: <span style='color: green;'>```QObjectCleanupHandler```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qobjectcleanuphandler.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qobjectcleanuphandler.html">QObjectCleanupHandler</a> class watches the lifetime of multiple QObjects.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qobjectcleanuphandler.html">QObjectCleanupHandler</a> is useful whenever you need to know when a number of <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>s that are owned by someone else have been deleted. This is important, for example, when referencing memory in an application that has been allocated in a shared library.</p>
/// <p>To keep track of some <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>s, create a <a href="http://doc.qt.io/qt-5/qobjectcleanuphandler.html">QObjectCleanupHandler</a>, and <a href="http://doc.qt.io/qt-5/qobjectcleanuphandler.html#add">add</a>() the objects you are interested in. If you are no longer interested in tracking a particular object, use <a href="http://doc.qt.io/qt-5/qobjectcleanuphandler.html#remove">remove</a>() to remove it from the cleanup handler. If an object being tracked by the cleanup handler gets deleted by someone else it will automatically be removed from the cleanup handler. You can delete all the objects in the cleanup handler with <a href="http://doc.qt.io/qt-5/qobjectcleanuphandler.html#clear">clear</a>(), or by destroying the cleanup handler. <a href="http://doc.qt.io/qt-5/qobjectcleanuphandler.html#isEmpty">isEmpty</a>() returns <code>true</code> if the <a href="http://doc.qt.io/qt-5/qobjectcleanuphandler.html">QObjectCleanupHandler</a> has no objects to keep track of.</p></div>
#[repr(C)]
pub struct QObjectCleanupHandler {
    _unused: u8,
}
impl QObjectCleanupHandler {
    /// Returns a built-in Qt slot `QObjectCleanupHandler::objectDestroyed` that can be passed to `qt_core::Signal::connect`.
    pub fn slot_object_destroyed(&self) -> crate::Receiver<(*mut crate::QObject,)> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1objectDestroyed(QObject*)\0"),
            )
        }
    }

    /// <p>Adds <i>object</i> to this cleanup handler and returns the pointer to the object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QObject* QObjectCleanupHandler::add(QObject* object)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobjectcleanuphandler.html#add">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds <i>object</i> to this cleanup handler and returns the pointer to the object.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobjectcleanuphandler.html#remove">remove</a>().</p></div>
    pub unsafe fn add(
        &mut self,
        object: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObjectCleanupHandler_add(
            self as *mut crate::QObjectCleanupHandler,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(object)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Deletes all objects in this cleanup handler. The cleanup handler becomes empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QObjectCleanupHandler::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobjectcleanuphandler.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Deletes all objects in this cleanup handler. The cleanup handler becomes empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobjectcleanuphandler.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QObjectCleanupHandler_clear(
            self as *mut crate::QObjectCleanupHandler,
        )
    }

    /// <p>Returns <code>true</code> if this cleanup handler is empty or if all objects in this cleanup handler have been destroyed; otherwise return false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QObjectCleanupHandler::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobjectcleanuphandler.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this cleanup handler is empty or if all objects in this cleanup handler have been destroyed; otherwise return false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobjectcleanuphandler.html#add">add</a>(), <a href="http://doc.qt.io/qt-5/qobjectcleanuphandler.html#remove">remove</a>(), and <a href="http://doc.qt.io/qt-5/qobjectcleanuphandler.html#clear">clear</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QObjectCleanupHandler_isEmpty(
            self as *const crate::QObjectCleanupHandler,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QObjectCleanupHandler::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObjectCleanupHandler_metaObject(
            self as *const crate::QObjectCleanupHandler,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs an empty <a href="http://doc.qt.io/qt-5/qobjectcleanuphandler.html">QObjectCleanupHandler</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QObjectCleanupHandler::QObjectCleanupHandler()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobjectcleanuphandler.html#QObjectCleanupHandler">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty <a href="http://doc.qt.io/qt-5/qobjectcleanuphandler.html">QObjectCleanupHandler</a>.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QObjectCleanupHandler> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QObjectCleanupHandler_QObjectCleanupHandler();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QObjectCleanupHandler::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QObjectCleanupHandler_qt_metacall(
            self as *mut crate::QObjectCleanupHandler,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QObjectCleanupHandler::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObjectCleanupHandler_qt_metacast(
            self as *mut crate::QObjectCleanupHandler,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Removes the <i>object</i> from this cleanup handler. The object will not be destroyed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QObjectCleanupHandler::remove(QObject* object)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobjectcleanuphandler.html#remove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the <i>object</i> from this cleanup handler. The object will not be destroyed.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobjectcleanuphandler.html#add">add</a>().</p></div>
    pub unsafe fn remove(
        &mut self,
        object: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QObjectCleanupHandler_remove(
            self as *mut crate::QObjectCleanupHandler,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(object)
                .as_mut_raw_ptr(),
        )
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObjectCleanupHandler_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QObjectCleanupHandler::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObjectCleanupHandler_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QObjectCleanupHandler::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QObjectCleanupHandler_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_operating_system_version {
    //! C++ type: <span style='color: green;'>```QOperatingSystemVersion```</span>

    /// <p>This enum provides symbolic names for the various operating system families supported by <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a>.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QOperatingSystemVersion::OSType```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#OSType-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum provides symbolic names for the various operating system families supported by <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a>.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct OSType(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for OSType {
        fn from(value: ::std::os::raw::c_int) -> Self {
            OSType(value)
        }
    }

    impl From<OSType> for ::std::os::raw::c_int {
        fn from(value: OSType) -> Self {
            value.0
        }
    }

    impl OSType {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl OSType {
        /// An unknown or unsupported operating system. (C++ enum variant: <span style='color: green;'>```Unknown = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const Unknown: crate::q_operating_system_version::OSType =
            crate::q_operating_system_version::OSType(0);
        /// The Microsoft Windows operating system. (C++ enum variant: <span style='color: green;'>```Windows = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Windows: crate::q_operating_system_version::OSType =
            crate::q_operating_system_version::OSType(1);
        /// The Apple <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> operating system. (C++ enum variant: <span style='color: green;'>```MacOS = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const MacOS: crate::q_operating_system_version::OSType =
            crate::q_operating_system_version::OSType(2);
        /// The Apple iOS operating system. (C++ enum variant: <span style='color: green;'>```IOS = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const IOS: crate::q_operating_system_version::OSType =
            crate::q_operating_system_version::OSType(3);
        /// The Apple tvOS operating system. (C++ enum variant: <span style='color: green;'>```TvOS = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const TvOS: crate::q_operating_system_version::OSType =
            crate::q_operating_system_version::OSType(4);
        /// The Apple watchOS operating system. (C++ enum variant: <span style='color: green;'>```WatchOS = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const WatchOS: crate::q_operating_system_version::OSType =
            crate::q_operating_system_version::OSType(5);
        /// The Google Android operating system. (C++ enum variant: <span style='color: green;'>```Android = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const Android: crate::q_operating_system_version::OSType =
            crate::q_operating_system_version::OSType(6);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> class provides information about the operating system version.</p>
///
/// C++ class: <span style='color: green;'>```QOperatingSystemVersion```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> class provides information about the operating system version.</p>
/// <p>Unlike other version functions in <a href="http://doc.qt.io/qt-5/qsysinfo.html">QSysInfo</a>, <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> provides access to the full version number that <i>developers</i> typically use to vary behavior or determine whether to enable APIs or features based on the operating system version (as opposed to the kernel version number or marketing version).</p>
/// <p>This class is also a complete replacement for QSysInfo::macVersion and QSysInfo::windowsVersion, additionally providing access to the third (micro) version number component.</p>
/// <p>Presently, Android, Apple Platforms (iOS, <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, tvOS, and watchOS), and Windows are supported.</p>
/// <p>The <i>majorVersion()</i>, <i>minorVersion()</i>, and <i>microVersion()</i> functions return the parts of the operating system version number based on:</p>
/// <div class="table"><table class="generic">
///  <thead><tr class="qt-style"><th>Platforms</th><th>Value</th></tr></thead>
/// <tbody><tr valign="top" class="odd"><td>Android</td><td>result of parsing <a href="https://developer.android.com/reference/android/os/Build.VERSION.html#RELEASE">android.os.Build.VERSION.RELEASE</a> using <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a>, with a fallback to <a href="https://developer.android.com/reference/android/os/Build.VERSION.html#SDK_INT">android.os.Build.VERSION.SDK_INT</a> to determine the major and minor version component if the former fails</td></tr>
/// <tr valign="top" class="even"><td>Apple Platforms</td><td><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#majorVersion">majorVersion</a>, <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#minorVersion">minorVersion</a>, and patchVersion from <a href="https://developer.apple.com/reference/foundation/nsprocessinfo/1410906-operatingsystemversion?language=objc">NSProcessInfo.operatingSystemVersion</a></td></tr>
/// <tr valign="top" class="odd"><td>Windows</td><td>dwMajorVersion, dwMinorVersion, and dwBuildNumber from <a href="https://msdn.microsoft.com/en-us/library/mt723418.aspx">RtlGetVersion</a> - note that this function ALWAYS return the version number of the underlying operating system, as opposed to the shim underneath GetVersionEx that hides the real version number if the application is not manifested for that version of the OS</td></tr>
/// </tbody></table></div>
/// <p>Because <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> stores both a version number and an OS type, the OS type can be taken into account when performing comparisons. For example, on a <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> system running <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> Sierra (v10.12), the following expression will return <code>false</code> even though the major version number component of the object on the left hand side of the expression (10) is greater than that of the object on the right (9):</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span><span class="operator">::</span>current() <span class="operator">&gt;</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span>(<span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span><span class="operator">::</span>IOS<span class="operator">,</span> <span class="number">9</span>)
///
/// </pre>
/// <p>This allows expressions for multiple operating systems to be joined with a logical OR operator and still work as expected. For example:</p>
/// <pre class="cpp">
///
///   <span class="keyword">auto</span> current <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span><span class="operator">::</span>current();
///   <span class="keyword">if</span> (current <span class="operator">&gt;</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span><span class="operator">::</span>OSXYosemite <span class="operator">|</span><span class="operator">|</span>
/// &#32;     current <span class="operator">&gt;</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span>(<span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span><span class="operator">::</span>IOS<span class="operator">,</span> <span class="number">8</span>)) {
/// &#32;     <span class="comment">// returns true on macOS &gt;= 10.10 and iOS &gt;= 8.0, but false on macOS &lt; 10.10 and iOS &lt; 8.0</span>
///   }
///
/// </pre>
/// <p>A more naive comparison algorithm might incorrectly return true on all versions of <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, including Mac OS 9. This behavior is achieved by overloading the comparison operators to return <code>false</code> whenever the OS types of the <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> instances being compared do not match. Be aware that due to this it can be the case <code>x</code> &gt;= y and <code>x</code> &lt; y are BOTH <code>false</code> for the same instances of <code>x</code> and <code>y</code>.</p></div>
#[repr(C)]
pub struct QOperatingSystemVersion {
    _unused: u8,
}
impl QOperatingSystemVersion {
    /// Returns a reference to the <span style='color: green;'>```AndroidJellyBean```</span> field.
    pub unsafe fn android_jelly_bean() -> ::cpp_utils::Ref<crate::QOperatingSystemVersion> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_AndroidJellyBean();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Returns a reference to the <span style='color: green;'>```AndroidJellyBean_MR1```</span> field.
    pub unsafe fn android_jelly_bean_m_r1() -> ::cpp_utils::Ref<crate::QOperatingSystemVersion> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_AndroidJellyBean_MR1();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Returns a reference to the <span style='color: green;'>```AndroidJellyBean_MR2```</span> field.
    pub unsafe fn android_jelly_bean_m_r2() -> ::cpp_utils::Ref<crate::QOperatingSystemVersion> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_AndroidJellyBean_MR2();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Returns a reference to the <span style='color: green;'>```AndroidKitKat```</span> field.
    pub unsafe fn android_kit_kat() -> ::cpp_utils::Ref<crate::QOperatingSystemVersion> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_AndroidKitKat();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Returns a reference to the <span style='color: green;'>```AndroidLollipop```</span> field.
    pub unsafe fn android_lollipop() -> ::cpp_utils::Ref<crate::QOperatingSystemVersion> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_AndroidLollipop();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Returns a reference to the <span style='color: green;'>```AndroidLollipop_MR1```</span> field.
    pub unsafe fn android_lollipop_m_r1() -> ::cpp_utils::Ref<crate::QOperatingSystemVersion> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_AndroidLollipop_MR1();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Returns a reference to the <span style='color: green;'>```AndroidMarshmallow```</span> field.
    pub unsafe fn android_marshmallow() -> ::cpp_utils::Ref<crate::QOperatingSystemVersion> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_AndroidMarshmallow();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Returns a reference to the <span style='color: green;'>```AndroidNougat```</span> field.
    pub unsafe fn android_nougat() -> ::cpp_utils::Ref<crate::QOperatingSystemVersion> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_AndroidNougat();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Returns a reference to the <span style='color: green;'>```AndroidNougat_MR1```</span> field.
    pub unsafe fn android_nougat_m_r1() -> ::cpp_utils::Ref<crate::QOperatingSystemVersion> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_AndroidNougat_MR1();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Returns a reference to the <span style='color: green;'>```AndroidOreo```</span> field.
    pub unsafe fn android_oreo() -> ::cpp_utils::Ref<crate::QOperatingSystemVersion> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_AndroidOreo();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> class provides information about the operating system version.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QOperatingSystemVersion& QOperatingSystemVersion::operator=(const QOperatingSystemVersion& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> class provides information about the operating system version.</p>
    /// <p>Unlike other version functions in <a href="http://doc.qt.io/qt-5/qsysinfo.html">QSysInfo</a>, <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> provides access to the full version number that <i>developers</i> typically use to vary behavior or determine whether to enable APIs or features based on the operating system version (as opposed to the kernel version number or marketing version).</p>
    /// <p>This class is also a complete replacement for QSysInfo::macVersion and QSysInfo::windowsVersion, additionally providing access to the third (micro) version number component.</p>
    /// <p>Presently, Android, Apple Platforms (iOS, <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, tvOS, and watchOS), and Windows are supported.</p>
    /// <p>The <i>majorVersion()</i>, <i>minorVersion()</i>, and <i>microVersion()</i> functions return the parts of the operating system version number based on:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Platforms</th><th>Value</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>Android</td><td>result of parsing <a href="https://developer.android.com/reference/android/os/Build.VERSION.html#RELEASE">android.os.Build.VERSION.RELEASE</a> using <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a>, with a fallback to <a href="https://developer.android.com/reference/android/os/Build.VERSION.html#SDK_INT">android.os.Build.VERSION.SDK_INT</a> to determine the major and minor version component if the former fails</td></tr>
    /// <tr valign="top" class="even"><td>Apple Platforms</td><td><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#majorVersion">majorVersion</a>, <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#minorVersion">minorVersion</a>, and patchVersion from <a href="https://developer.apple.com/reference/foundation/nsprocessinfo/1410906-operatingsystemversion?language=objc">NSProcessInfo.operatingSystemVersion</a></td></tr>
    /// <tr valign="top" class="odd"><td>Windows</td><td>dwMajorVersion, dwMinorVersion, and dwBuildNumber from <a href="https://msdn.microsoft.com/en-us/library/mt723418.aspx">RtlGetVersion</a> - note that this function ALWAYS return the version number of the underlying operating system, as opposed to the shim underneath GetVersionEx that hides the real version number if the application is not manifested for that version of the OS</td></tr>
    /// </tbody></table></div>
    /// <p>Because <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> stores both a version number and an OS type, the OS type can be taken into account when performing comparisons. For example, on a <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> system running <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> Sierra (v10.12), the following expression will return <code>false</code> even though the major version number component of the object on the left hand side of the expression (10) is greater than that of the object on the right (9):</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span><span class="operator">::</span>current() <span class="operator">&gt;</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span>(<span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span><span class="operator">::</span>IOS<span class="operator">,</span> <span class="number">9</span>)
    ///
    /// </pre>
    /// <p>This allows expressions for multiple operating systems to be joined with a logical OR operator and still work as expected. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">auto</span> current <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span><span class="operator">::</span>current();
    ///   <span class="keyword">if</span> (current <span class="operator">&gt;</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span><span class="operator">::</span>OSXYosemite <span class="operator">|</span><span class="operator">|</span>
    /// &#32;     current <span class="operator">&gt;</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span>(<span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span><span class="operator">::</span>IOS<span class="operator">,</span> <span class="number">8</span>)) {
    /// &#32;     <span class="comment">// returns true on macOS &gt;= 10.10 and iOS &gt;= 8.0, but false on macOS &lt; 10.10 and iOS &lt; 8.0</span>
    ///   }
    ///
    /// </pre>
    /// <p>A more naive comparison algorithm might incorrectly return true on all versions of <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, including Mac OS 9. This behavior is achieved by overloading the comparison operators to return <code>false</code> whenever the OS types of the <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> instances being compared do not match. Be aware that due to this it can be the case <code>x</code> &gt;= y and <code>x</code> &lt; y are BOTH <code>false</code> for the same instances of <code>x</code> and <code>y</code>.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QOperatingSystemVersion>>,
    ) -> ::cpp_utils::MutRef<crate::QOperatingSystemVersion> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_operator_(
            self as *mut crate::QOperatingSystemVersion,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QOperatingSystemVersion>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> indicating the current OS and its version number.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QOperatingSystemVersion QOperatingSystemVersion::current()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#current">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> indicating the current OS and its version number.</p></div>
    pub unsafe fn current() -> ::cpp_utils::CppBox<crate::QOperatingSystemVersion> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_current();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the current OS type without constructing a <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> instance.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QOperatingSystemVersion::OSType QOperatingSystemVersion::currentType()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#currentType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current OS type without constructing a <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> instance.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#current">current</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn current_type() -> crate::q_operating_system_version::OSType {
        crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_currentType()
    }

    /// Returns a reference to the <span style='color: green;'>```MacOSHighSierra```</span> field.
    pub unsafe fn mac_o_s_high_sierra() -> ::cpp_utils::Ref<crate::QOperatingSystemVersion> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_MacOSHighSierra();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Returns a reference to the <span style='color: green;'>```MacOSMojave```</span> field.
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn mac_o_s_mojave() -> ::cpp_utils::Ref<crate::QOperatingSystemVersion> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_MacOSMojave();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Returns a reference to the <span style='color: green;'>```MacOSSierra```</span> field.
    pub unsafe fn mac_o_s_sierra() -> ::cpp_utils::Ref<crate::QOperatingSystemVersion> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_MacOSSierra();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the major version number, that is, the first segment of the operating system's version number.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QOperatingSystemVersion::majorVersion() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#majorVersion">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the major version number, that is, the first segment of the operating system's version number.</p>
    /// <p>See the main class documentation for what the major version number is on a given operating system.</p>
    /// <p>-1 indicates an unknown or absent version number component.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#minorVersion">minorVersion</a>() and <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#microVersion">microVersion</a>().</p></div>
    pub unsafe fn major_version(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_majorVersion(
            self as *const crate::QOperatingSystemVersion,
        )
    }

    /// <p>Returns the micro version number, that is, the third segment of the operating system's version number.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QOperatingSystemVersion::microVersion() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#microVersion">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the micro version number, that is, the third segment of the operating system's version number.</p>
    /// <p>See the main class documentation for what the micro version number is on a given operating system.</p>
    /// <p>-1 indicates an unknown or absent version number component.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#majorVersion">majorVersion</a>() and <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#minorVersion">minorVersion</a>().</p></div>
    pub unsafe fn micro_version(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_microVersion(
            self as *const crate::QOperatingSystemVersion,
        )
    }

    /// <p>Returns the minor version number, that is, the second segment of the operating system's version number.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QOperatingSystemVersion::minorVersion() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#minorVersion">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the minor version number, that is, the second segment of the operating system's version number.</p>
    /// <p>See the main class documentation for what the minor version number is on a given operating system.</p>
    /// <p>-1 indicates an unknown or absent version number component.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#majorVersion">majorVersion</a>() and <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#microVersion">microVersion</a>().</p></div>
    pub unsafe fn minor_version(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_minorVersion(
            self as *const crate::QOperatingSystemVersion,
        )
    }

    /// <p>Returns a string representation of the OS type identified by the <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QOperatingSystemVersion::name() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#name">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string representation of the OS type identified by the <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#type">type</a>().</p></div>
    pub unsafe fn name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_name(
            self as *const crate::QOperatingSystemVersion,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> consisting of the OS type <i>osType</i>, and major, minor, and micro version numbers <i>vmajor</i>, <i>vminor</i> and <i>vmicro</i>, respectively.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QOperatingSystemVersion::QOperatingSystemVersion(QOperatingSystemVersion::OSType osType, int vmajor, int vminor = …, int vmicro = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> consisting of the OS type <i>osType</i>, and major, minor, and micro version numbers <i>vmajor</i>, <i>vminor</i> and <i>vmicro</i>, respectively.</p></div>
    pub unsafe fn new_4a(
        os_type: crate::q_operating_system_version::OSType,
        vmajor: ::std::os::raw::c_int,
        vminor: ::std::os::raw::c_int,
        vmicro: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QOperatingSystemVersion> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_QOperatingSystemVersion(
                os_type, vmajor, vminor, vmicro,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> consisting of the OS type <i>osType</i>, and major, minor, and micro version numbers <i>vmajor</i>, <i>vminor</i> and <i>vmicro</i>, respectively.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QOperatingSystemVersion::QOperatingSystemVersion(QOperatingSystemVersion::OSType osType, int vmajor, int vminor = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> consisting of the OS type <i>osType</i>, and major, minor, and micro version numbers <i>vmajor</i>, <i>vminor</i> and <i>vmicro</i>, respectively.</p></div>
    pub unsafe fn new_3a(
        os_type: crate::q_operating_system_version::OSType,
        vmajor: ::std::os::raw::c_int,
        vminor: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QOperatingSystemVersion> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_QOperatingSystemVersion2(
                os_type, vmajor, vminor,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> consisting of the OS type <i>osType</i>, and major, minor, and micro version numbers <i>vmajor</i>, <i>vminor</i> and <i>vmicro</i>, respectively.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QOperatingSystemVersion::QOperatingSystemVersion(QOperatingSystemVersion::OSType osType, int vmajor)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> consisting of the OS type <i>osType</i>, and major, minor, and micro version numbers <i>vmajor</i>, <i>vminor</i> and <i>vmicro</i>, respectively.</p></div>
    pub unsafe fn new_2a(
        os_type: crate::q_operating_system_version::OSType,
        vmajor: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QOperatingSystemVersion> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_QOperatingSystemVersion3(
                os_type, vmajor,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> class provides information about the operating system version.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QOperatingSystemVersion::QOperatingSystemVersion(const QOperatingSystemVersion& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> class provides information about the operating system version.</p>
    /// <p>Unlike other version functions in <a href="http://doc.qt.io/qt-5/qsysinfo.html">QSysInfo</a>, <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> provides access to the full version number that <i>developers</i> typically use to vary behavior or determine whether to enable APIs or features based on the operating system version (as opposed to the kernel version number or marketing version).</p>
    /// <p>This class is also a complete replacement for QSysInfo::macVersion and QSysInfo::windowsVersion, additionally providing access to the third (micro) version number component.</p>
    /// <p>Presently, Android, Apple Platforms (iOS, <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, tvOS, and watchOS), and Windows are supported.</p>
    /// <p>The <i>majorVersion()</i>, <i>minorVersion()</i>, and <i>microVersion()</i> functions return the parts of the operating system version number based on:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Platforms</th><th>Value</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>Android</td><td>result of parsing <a href="https://developer.android.com/reference/android/os/Build.VERSION.html#RELEASE">android.os.Build.VERSION.RELEASE</a> using <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a>, with a fallback to <a href="https://developer.android.com/reference/android/os/Build.VERSION.html#SDK_INT">android.os.Build.VERSION.SDK_INT</a> to determine the major and minor version component if the former fails</td></tr>
    /// <tr valign="top" class="even"><td>Apple Platforms</td><td><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#majorVersion">majorVersion</a>, <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#minorVersion">minorVersion</a>, and patchVersion from <a href="https://developer.apple.com/reference/foundation/nsprocessinfo/1410906-operatingsystemversion?language=objc">NSProcessInfo.operatingSystemVersion</a></td></tr>
    /// <tr valign="top" class="odd"><td>Windows</td><td>dwMajorVersion, dwMinorVersion, and dwBuildNumber from <a href="https://msdn.microsoft.com/en-us/library/mt723418.aspx">RtlGetVersion</a> - note that this function ALWAYS return the version number of the underlying operating system, as opposed to the shim underneath GetVersionEx that hides the real version number if the application is not manifested for that version of the OS</td></tr>
    /// </tbody></table></div>
    /// <p>Because <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> stores both a version number and an OS type, the OS type can be taken into account when performing comparisons. For example, on a <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> system running <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> Sierra (v10.12), the following expression will return <code>false</code> even though the major version number component of the object on the left hand side of the expression (10) is greater than that of the object on the right (9):</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span><span class="operator">::</span>current() <span class="operator">&gt;</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span>(<span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span><span class="operator">::</span>IOS<span class="operator">,</span> <span class="number">9</span>)
    ///
    /// </pre>
    /// <p>This allows expressions for multiple operating systems to be joined with a logical OR operator and still work as expected. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">auto</span> current <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span><span class="operator">::</span>current();
    ///   <span class="keyword">if</span> (current <span class="operator">&gt;</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span><span class="operator">::</span>OSXYosemite <span class="operator">|</span><span class="operator">|</span>
    /// &#32;     current <span class="operator">&gt;</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span>(<span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span><span class="operator">::</span>IOS<span class="operator">,</span> <span class="number">8</span>)) {
    /// &#32;     <span class="comment">// returns true on macOS &gt;= 10.10 and iOS &gt;= 8.0, but false on macOS &lt; 10.10 and iOS &lt; 8.0</span>
    ///   }
    ///
    /// </pre>
    /// <p>A more naive comparison algorithm might incorrectly return true on all versions of <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, including Mac OS 9. This behavior is achieved by overloading the comparison operators to return <code>false</code> whenever the OS types of the <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> instances being compared do not match. Be aware that due to this it can be the case <code>x</code> &gt;= y and <code>x</code> &lt; y are BOTH <code>false</code> for the same instances of <code>x</code> and <code>y</code>.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QOperatingSystemVersion>>,
    ) -> ::cpp_utils::CppBox<crate::QOperatingSystemVersion> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_QOperatingSystemVersion1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QOperatingSystemVersion>>::cast_into(other).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Returns a reference to the <span style='color: green;'>```OSXElCapitan```</span> field.
    pub unsafe fn o_s_x_el_capitan() -> ::cpp_utils::Ref<crate::QOperatingSystemVersion> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_OSXElCapitan();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Returns a reference to the <span style='color: green;'>```OSXMavericks```</span> field.
    pub unsafe fn o_s_x_mavericks() -> ::cpp_utils::Ref<crate::QOperatingSystemVersion> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_OSXMavericks();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Returns a reference to the <span style='color: green;'>```OSXYosemite```</span> field.
    pub unsafe fn o_s_x_yosemite() -> ::cpp_utils::Ref<crate::QOperatingSystemVersion> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_OSXYosemite();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of integers stored in the version number.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QOperatingSystemVersion::segmentCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#segmentCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of integers stored in the version number.</p></div>
    pub unsafe fn segment_count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_segmentCount(
            self as *const crate::QOperatingSystemVersion,
        )
    }

    /// <p>Returns the OS type identified by the <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QOperatingSystemVersion::OSType QOperatingSystemVersion::type() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#type">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the OS type identified by the <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#name">name</a>().</p></div>
    pub unsafe fn type_(&self) -> crate::q_operating_system_version::OSType {
        crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_type(
            self as *const crate::QOperatingSystemVersion,
        )
    }

    /// Returns a reference to the <span style='color: green;'>```Windows10```</span> field.
    pub unsafe fn windows10() -> ::cpp_utils::Ref<crate::QOperatingSystemVersion> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_Windows10();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Returns a reference to the <span style='color: green;'>```Windows7```</span> field.
    pub unsafe fn windows7() -> ::cpp_utils::Ref<crate::QOperatingSystemVersion> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_Windows7();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Returns a reference to the <span style='color: green;'>```Windows8```</span> field.
    pub unsafe fn windows8() -> ::cpp_utils::Ref<crate::QOperatingSystemVersion> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_Windows8();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Returns a reference to the <span style='color: green;'>```Windows8_1```</span> field.
    pub unsafe fn windows8_1() -> ::cpp_utils::Ref<crate::QOperatingSystemVersion> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_Windows8_1();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qparallelanimationgroup.html">QParallelAnimationGroup</a> class provides a parallel group of animations.</p>
///
/// C++ class: <span style='color: green;'>```QParallelAnimationGroup```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qparallelanimationgroup.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qparallelanimationgroup.html">QParallelAnimationGroup</a> class provides a parallel group of animations.</p>
/// <p><a href="http://doc.qt.io/qt-5/qparallelanimationgroup.html">QParallelAnimationGroup</a>--a <a href="http://doc.qt.io/qt-5/qanimationgroup.html">container for animations</a>--starts all its animations when it is <a href="http://doc.qt.io/qt-5/qabstractanimation.html#start">started</a> itself, i.e., runs all animations in parallel. The animation group finishes when the longest lasting animation has finished.</p>
/// <p>You can treat <a href="http://doc.qt.io/qt-5/qparallelanimationgroup.html">QParallelAnimationGroup</a> as any other <a href="http://doc.qt.io/qt-5/qabstractanimation.html">QAbstractAnimation</a>, e.g., pause, resume, or add it to other animation groups.</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qparallelanimationgroup.html#QParallelAnimationGroup">QParallelAnimationGroup</a></span> <span class="operator">*</span>group <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qparallelanimationgroup.html#QParallelAnimationGroup">QParallelAnimationGroup</a></span>;
///   group<span class="operator">-</span><span class="operator">&gt;</span>addAnimation(anim1);
///   group<span class="operator">-</span><span class="operator">&gt;</span>addAnimation(anim2);
///
///   group<span class="operator">-</span><span class="operator">&gt;</span>start();
///
/// </pre>
/// <p>In this example, <code>anim1</code> and <code>anim2</code> are two <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a>s that have already been set up.</p></div>
#[repr(C)]
pub struct QParallelAnimationGroup {
    _unused: u8,
}
impl QParallelAnimationGroup {
    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractanimation.html#duration">QAbstractAnimation::duration</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual int QParallelAnimationGroup::duration() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qparallelanimationgroup.html#duration">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractanimation.html#duration">QAbstractAnimation::duration</a>().</p></div>
    pub unsafe fn duration(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QParallelAnimationGroup_duration(
            self as *const crate::QParallelAnimationGroup,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QParallelAnimationGroup::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QParallelAnimationGroup_metaObject(
            self as *const crate::QParallelAnimationGroup,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qparallelanimationgroup.html">QParallelAnimationGroup</a>. <i>parent</i> is passed to <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>'s constructor.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QParallelAnimationGroup::QParallelAnimationGroup(QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qparallelanimationgroup.html#QParallelAnimationGroup">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qparallelanimationgroup.html">QParallelAnimationGroup</a>. <i>parent</i> is passed to <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>'s constructor.</p></div>
    pub unsafe fn new_1a(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QParallelAnimationGroup> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QParallelAnimationGroup_QParallelAnimationGroup(
                ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                    .as_mut_raw_ptr(),
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qparallelanimationgroup.html">QParallelAnimationGroup</a> class provides a parallel group of animations.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QParallelAnimationGroup::QParallelAnimationGroup()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qparallelanimationgroup.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qparallelanimationgroup.html">QParallelAnimationGroup</a> class provides a parallel group of animations.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qparallelanimationgroup.html">QParallelAnimationGroup</a>--a <a href="http://doc.qt.io/qt-5/qanimationgroup.html">container for animations</a>--starts all its animations when it is <a href="http://doc.qt.io/qt-5/qabstractanimation.html#start">started</a> itself, i.e., runs all animations in parallel. The animation group finishes when the longest lasting animation has finished.</p>
    /// <p>You can treat <a href="http://doc.qt.io/qt-5/qparallelanimationgroup.html">QParallelAnimationGroup</a> as any other <a href="http://doc.qt.io/qt-5/qabstractanimation.html">QAbstractAnimation</a>, e.g., pause, resume, or add it to other animation groups.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qparallelanimationgroup.html#QParallelAnimationGroup">QParallelAnimationGroup</a></span> <span class="operator">*</span>group <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qparallelanimationgroup.html#QParallelAnimationGroup">QParallelAnimationGroup</a></span>;
    ///   group<span class="operator">-</span><span class="operator">&gt;</span>addAnimation(anim1);
    ///   group<span class="operator">-</span><span class="operator">&gt;</span>addAnimation(anim2);
    ///
    ///   group<span class="operator">-</span><span class="operator">&gt;</span>start();
    ///
    /// </pre>
    /// <p>In this example, <code>anim1</code> and <code>anim2</code> are two <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a>s that have already been set up.</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QParallelAnimationGroup> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QParallelAnimationGroup_QParallelAnimationGroup1();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QParallelAnimationGroup::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QParallelAnimationGroup_qt_metacall(
            self as *mut crate::QParallelAnimationGroup,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QParallelAnimationGroup::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QParallelAnimationGroup_qt_metacast(
            self as *mut crate::QParallelAnimationGroup,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QParallelAnimationGroup_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QParallelAnimationGroup::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QParallelAnimationGroup_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QParallelAnimationGroup::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QParallelAnimationGroup_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qpauseanimation.html">QPauseAnimation</a> class provides a pause for <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html">QSequentialAnimationGroup</a>.</p>
///
/// C++ class: <span style='color: green;'>```QPauseAnimation```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qpauseanimation.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpauseanimation.html">QPauseAnimation</a> class provides a pause for <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html">QSequentialAnimationGroup</a>.</p>
/// <p>If you wish to introduce a delay between animations in a <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html">QSequentialAnimationGroup</a>, you can insert a <a href="http://doc.qt.io/qt-5/qpauseanimation.html">QPauseAnimation</a>. This class does not animate anything, but does not <a href="http://doc.qt.io/qt-5/qabstractanimation.html#finished">finish</a> before a specified number of milliseconds have elapsed from when it was started. You specify the duration of the pause in the constructor. It can also be set directly with <a href="http://doc.qt.io/qt-5/qpauseanimation.html#duration-prop">setDuration</a>().</p>
/// <p>It is not necessary to construct a <a href="http://doc.qt.io/qt-5/qpauseanimation.html">QPauseAnimation</a> yourself. <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html">QSequentialAnimationGroup</a> provides the convenience functions <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#addPause">addPause()</a> and <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#insertPause">insertPause()</a>. These functions simply take the number of milliseconds the pause should last.</p></div>
#[repr(C)]
pub struct QPauseAnimation {
    _unused: u8,
}
impl QPauseAnimation {
    /// <p>This property holds the duration of the pause.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual int QPauseAnimation::duration() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpauseanimation.html#duration-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the duration of the pause.</p>
    /// <p>The duration of the pause. The duration should not be negative. The default duration is 250 milliseconds.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> virtual int </td><td class="memItemRight bottomAlign"><span class="name"><b>duration</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setDuration</b></span>(int <i>msecs</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn duration(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QPauseAnimation_duration(
            self as *const crate::QPauseAnimation,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QPauseAnimation::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPauseAnimation_metaObject(
            self as *const crate::QPauseAnimation,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qpauseanimation.html">QPauseAnimation</a>. <i>parent</i> is passed to <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>'s constructor. The default duration is 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPauseAnimation::QPauseAnimation(QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpauseanimation.html#QPauseAnimation">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qpauseanimation.html">QPauseAnimation</a>. <i>parent</i> is passed to <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>'s constructor. The default duration is 0.</p></div>
    pub unsafe fn from_q_object(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QPauseAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPauseAnimation_QPauseAnimation(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qpauseanimation.html">QPauseAnimation</a>. <i>msecs</i> is the duration of the pause. <i>parent</i> is passed to <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>'s constructor.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPauseAnimation::QPauseAnimation(int msecs, QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpauseanimation.html#QPauseAnimation-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qpauseanimation.html">QPauseAnimation</a>. <i>msecs</i> is the duration of the pause. <i>parent</i> is passed to <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>'s constructor.</p></div>
    pub unsafe fn from_int_q_object(
        msecs: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QPauseAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPauseAnimation_QPauseAnimation1(
            msecs,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qpauseanimation.html">QPauseAnimation</a> class provides a pause for <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html">QSequentialAnimationGroup</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPauseAnimation::QPauseAnimation()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpauseanimation.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpauseanimation.html">QPauseAnimation</a> class provides a pause for <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html">QSequentialAnimationGroup</a>.</p>
    /// <p>If you wish to introduce a delay between animations in a <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html">QSequentialAnimationGroup</a>, you can insert a <a href="http://doc.qt.io/qt-5/qpauseanimation.html">QPauseAnimation</a>. This class does not animate anything, but does not <a href="http://doc.qt.io/qt-5/qabstractanimation.html#finished">finish</a> before a specified number of milliseconds have elapsed from when it was started. You specify the duration of the pause in the constructor. It can also be set directly with <a href="http://doc.qt.io/qt-5/qpauseanimation.html#duration-prop">setDuration</a>().</p>
    /// <p>It is not necessary to construct a <a href="http://doc.qt.io/qt-5/qpauseanimation.html">QPauseAnimation</a> yourself. <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html">QSequentialAnimationGroup</a> provides the convenience functions <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#addPause">addPause()</a> and <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#insertPause">insertPause()</a>. These functions simply take the number of milliseconds the pause should last.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QPauseAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPauseAnimation_QPauseAnimation2();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qpauseanimation.html">QPauseAnimation</a>. <i>msecs</i> is the duration of the pause. <i>parent</i> is passed to <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>'s constructor.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPauseAnimation::QPauseAnimation(int msecs)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpauseanimation.html#QPauseAnimation-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qpauseanimation.html">QPauseAnimation</a>. <i>msecs</i> is the duration of the pause. <i>parent</i> is passed to <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>'s constructor.</p></div>
    pub unsafe fn from_int(
        msecs: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QPauseAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPauseAnimation_QPauseAnimation3(msecs);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QPauseAnimation::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QPauseAnimation_qt_metacall(
            self as *mut crate::QPauseAnimation,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QPauseAnimation::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPauseAnimation_qt_metacast(
            self as *mut crate::QPauseAnimation,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This property holds the duration of the pause.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QPauseAnimation::setDuration(int msecs)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpauseanimation.html#duration-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the duration of the pause.</p>
    /// <p>The duration of the pause. The duration should not be negative. The default duration is 250 milliseconds.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> virtual int </td><td class="memItemRight bottomAlign"><span class="name"><b>duration</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setDuration</b></span>(int <i>msecs</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn set_duration(&mut self, msecs: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QPauseAnimation_setDuration(
            self as *mut crate::QPauseAnimation,
            msecs,
        )
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPauseAnimation_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QPauseAnimation::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPauseAnimation_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QPauseAnimation::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPauseAnimation_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p><a href="http://doc.qt.io/qt-5/qstaticplugin.html">QStaticPlugin</a> is a struct containing a reference to a static plugin instance together with its meta data.</p>
///
/// C++ class: <span style='color: green;'>```QStaticPlugin```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qstaticplugin.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p><a href="http://doc.qt.io/qt-5/qstaticplugin.html">QStaticPlugin</a> is a struct containing a reference to a static plugin instance together with its meta data.</p></div>
#[repr(C)]
pub struct QStaticPlugin {
    _unused: u8,
}
impl QStaticPlugin {
    /// <p><a href="http://doc.qt.io/qt-5/qstaticplugin.html">QStaticPlugin</a> is a struct containing a reference to a static plugin instance together with its meta data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStaticPlugin& QStaticPlugin::operator=(const QStaticPlugin& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstaticplugin.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p><a href="http://doc.qt.io/qt-5/qstaticplugin.html">QStaticPlugin</a> is a struct containing a reference to a static plugin instance together with its meta data.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStaticPlugin>>,
    ) -> ::cpp_utils::MutRef<crate::QStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStaticPlugin_operator_(
            self as *mut crate::QStaticPlugin,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStaticPlugin>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Returns the value of the <span style='color: green;'>```instance```</span> field.
    pub unsafe fn instance(&self) -> ::std::option::Option<extern "C" fn() -> *mut crate::QObject> {
        crate::__ffi::ctr_qt_core_ffi_QStaticPlugin_instance(self as *const crate::QStaticPlugin)
    }

    /// <p>Returns a the meta data for the plugin as a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonObject QStaticPlugin::metaData() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstaticplugin.html#metaData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a the meta data for the plugin as a <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstaticplugin.html#rawMetaData">rawMetaData</a>().</p></div>
    pub unsafe fn meta_data(&self) -> ::cpp_utils::CppBox<crate::QJsonObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStaticPlugin_metaData(
            self as *const crate::QStaticPlugin,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p><a href="http://doc.qt.io/qt-5/qstaticplugin.html">QStaticPlugin</a> is a struct containing a reference to a static plugin instance together with its meta data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStaticPlugin::QStaticPlugin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstaticplugin.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p><a href="http://doc.qt.io/qt-5/qstaticplugin.html">QStaticPlugin</a> is a struct containing a reference to a static plugin instance together with its meta data.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStaticPlugin_QStaticPlugin();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p><a href="http://doc.qt.io/qt-5/qstaticplugin.html">QStaticPlugin</a> is a struct containing a reference to a static plugin instance together with its meta data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStaticPlugin::QStaticPlugin(const QStaticPlugin& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstaticplugin.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p><a href="http://doc.qt.io/qt-5/qstaticplugin.html">QStaticPlugin</a> is a struct containing a reference to a static plugin instance together with its meta data.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStaticPlugin>>,
    ) -> ::cpp_utils::CppBox<crate::QStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStaticPlugin_QStaticPlugin1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStaticPlugin>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Returns the value of the <span style='color: green;'>```rawMetaData```</span> field.
    pub unsafe fn raw_meta_data(
        &self,
    ) -> ::std::option::Option<extern "C" fn() -> *const ::std::os::raw::c_char> {
        crate::__ffi::ctr_qt_core_ffi_QStaticPlugin_rawMetaData(self as *const crate::QStaticPlugin)
    }

    /// Sets the value of the <span style='color: green;'>```instance```</span> field.
    pub unsafe fn set_instance(
        &mut self,
        value: ::std::option::Option<extern "C" fn() -> *mut crate::QObject>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_set_QStaticPlugin_instance(
            self as *mut crate::QStaticPlugin,
            value,
        )
    }

    /// Sets the value of the <span style='color: green;'>```rawMetaData```</span> field.
    pub unsafe fn set_raw_meta_data(
        &mut self,
        value: ::std::option::Option<extern "C" fn() -> *const ::std::os::raw::c_char>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_set_QStaticPlugin_rawMetaData(
            self as *mut crate::QStaticPlugin,
            value,
        )
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> class loads a plugin at run-time.</p>
///
/// C++ class: <span style='color: green;'>```QPluginLoader```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qpluginloader.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> class loads a plugin at run-time.</p>
/// <p><a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> provides access to a <a href="http://doc.qt.io/qt-5/plugins-howto.html">Qt plugin</a>. A Qt plugin is stored in a shared library (a DLL) and offers these benefits over shared libraries accessed using <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a>:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> checks that a plugin is linked against the same version of Qt as the application.</li>
/// <li><a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> provides direct access to a root component object (<a href="http://doc.qt.io/qt-5/qpluginloader.html#instance">instance</a>()), instead of forcing you to resolve a C function manually.</li>
/// </ul>
/// <p>An instance of a <a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> object operates on a single shared library file, which we call a plugin. It provides access to the functionality in the plugin in a platform-independent way. To specify which plugin to load, either pass a file name in the constructor or set it with <a href="http://doc.qt.io/qt-5/qpluginloader.html#fileName-prop">setFileName</a>().</p>
/// <p>The most important functions are <a href="http://doc.qt.io/qt-5/qpluginloader.html#load">load</a>() to dynamically load the plugin file, <a href="http://doc.qt.io/qt-5/qpluginloader.html#isLoaded">isLoaded</a>() to check whether loading was successful, and <a href="http://doc.qt.io/qt-5/qpluginloader.html#instance">instance</a>() to access the root component in the plugin. The <a href="http://doc.qt.io/qt-5/qpluginloader.html#instance">instance</a>() function implicitly tries to load the plugin if it has not been loaded yet. Multiple instances of <a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> can be used to access the same physical plugin.</p>
/// <p>Once loaded, plugins remain in memory until all instances of <a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> has been unloaded, or until the application terminates. You can attempt to unload a plugin using <a href="http://doc.qt.io/qt-5/qpluginloader.html#unload">unload</a>(), but if other instances of <a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> are using the same library, the call will fail, and unloading will only happen when every instance has called <a href="http://doc.qt.io/qt-5/qpluginloader.html#unload">unload</a>(). Right before the unloading happen, the root component will also be deleted.</p>
/// <p>See <a href="http://doc.qt.io/qt-5/plugins-howto.html">How to Create Qt Plugins</a> for more information about how to make your application extensible through plugins.</p>
/// <p>Note that the <a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> cannot be used if your application is statically linked against Qt. In this case, you will also have to link to plugins statically. You can use <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> if you need to load dynamic libraries in a statically linked application.</p></div>
#[repr(C)]
pub struct QPluginLoader {
    _unused: u8,
}
impl QPluginLoader {
    /// <p>Returns a text string with the description of the last error that occurred.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QPluginLoader::errorString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpluginloader.html#errorString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a text string with the description of the last error that occurred.</p>
    /// <p>This function was introduced in  Qt 4.2.</p></div>
    pub unsafe fn error_string(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPluginLoader_errorString(
            self as *const crate::QPluginLoader,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This property holds the file name of the plugin</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QPluginLoader::fileName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpluginloader.html#fileName-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the file name of the plugin</p>
    /// <p>We recommend omitting the file's suffix in the file name, since <a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> will automatically look for the file with the appropriate suffix (see <a href="http://doc.qt.io/qt-5/qlibrary.html#isLibrary">QLibrary::isLibrary</a>()).</p>
    /// <p>When loading the plugin, <a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> searches in the current directory and in all plugin locations specified by <a href="http://doc.qt.io/qt-5/qcoreapplication.html#libraryPaths">QCoreApplication::libraryPaths</a>(), unless the file name has an absolute path. After loading the plugin successfully, fileName() returns the fully-qualified file name of the plugin, including the full path to the plugin if one was given in the constructor or passed to setFileName().</p>
    /// <p>If the file name does not exist, it will not be set. This property will then contain an empty string.</p>
    /// <p>By default, this property contains an empty string.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QString </td><td class="memItemRight bottomAlign"><span class="name"><b>fileName</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setFileName</b></span>(const QString &amp;<i>fileName</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpluginloader.html#load">load</a>().</p></div>
    pub unsafe fn file_name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPluginLoader_fileName(
            self as *const crate::QPluginLoader,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the root component object of the plugin. The plugin is loaded if necessary. The function returns 0 if the plugin could not be loaded or if the root component object could not be instantiated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QObject* QPluginLoader::instance()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpluginloader.html#instance">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the root component object of the plugin. The plugin is loaded if necessary. The function returns 0 if the plugin could not be loaded or if the root component object could not be instantiated.</p>
    /// <p>If the root component object was destroyed, calling this function creates a new instance.</p>
    /// <p>The root component, returned by this function, is not deleted when the <a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> is destroyed. If you want to ensure that the root component is deleted, you should call <a href="http://doc.qt.io/qt-5/qpluginloader.html#unload">unload</a>() as soon you don't need to access the core component anymore. When the library is finally unloaded, the root component will automatically be deleted.</p>
    /// <p>The component object is a <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>. Use <a href="http://doc.qt.io/qt-5/qobject.html#qobject_cast">qobject_cast</a>() to access interfaces you are interested in.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpluginloader.html#load">load</a>().</p></div>
    pub unsafe fn instance(&mut self) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QPluginLoader_instance(self as *mut crate::QPluginLoader);
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns <code>true</code> if the plugin is loaded; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QPluginLoader::isLoaded() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpluginloader.html#isLoaded">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the plugin is loaded; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpluginloader.html#load">load</a>().</p></div>
    pub unsafe fn is_loaded(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QPluginLoader_isLoaded(self as *const crate::QPluginLoader)
    }

    /// <p>Loads the plugin and returns <code>true</code> if the plugin was loaded successfully; otherwise returns <code>false</code>. Since <a href="http://doc.qt.io/qt-5/qpluginloader.html#instance">instance</a>() always calls this function before resolving any symbols it is not necessary to call it explicitly. In some situations you might want the plugin loaded in advance, in which case you would use this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QPluginLoader::load()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpluginloader.html#load">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Loads the plugin and returns <code>true</code> if the plugin was loaded successfully; otherwise returns <code>false</code>. Since <a href="http://doc.qt.io/qt-5/qpluginloader.html#instance">instance</a>() always calls this function before resolving any symbols it is not necessary to call it explicitly. In some situations you might want the plugin loaded in advance, in which case you would use this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpluginloader.html#unload">unload</a>().</p></div>
    pub unsafe fn load(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QPluginLoader_load(self as *mut crate::QPluginLoader)
    }

    /// <p>Give the <a href="http://doc.qt.io/qt-5/qpluginloader.html#load">load</a>() function some hints on how it should behave.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFlags<QLibrary::LoadHint> QPluginLoader::loadHints() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpluginloader.html#loadHints-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Give the <a href="http://doc.qt.io/qt-5/qpluginloader.html#load">load</a>() function some hints on how it should behave.</p>
    /// <p>You can give hints on how the symbols in the plugin are resolved. By default since Qt 5.7, <a href="http://doc.qt.io/qt-5/qlibrary.html#LoadHint-enum">QLibrary::PreventUnloadHint</a> is set.</p>
    /// <p>See the documentation of <a href="http://doc.qt.io/qt-5/qlibrary.html#loadHints-prop">QLibrary::loadHints</a> for a complete description of how this property works.</p>
    /// <p>This property was introduced in  Qt 4.4.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QLibrary::LoadHints </td><td class="memItemRight bottomAlign"><span class="name"><b>loadHints</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setLoadHints</b></span>(QLibrary::LoadHints <i>loadHints</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlibrary.html#loadHints-prop">QLibrary::loadHints</a>.</p></div>
    pub unsafe fn load_hints(&self) -> crate::QFlags<crate::q_library::LoadHint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPluginLoader_loadHints(
            self as *const crate::QPluginLoader,
        );
        crate::QFlags::from(ffi_result)
    }

    /// <p>Returns the meta data for this plugin. The meta data is data specified in a json format using the <a href="http://doc.qt.io/qt-5/qtplugin.html#Q_PLUGIN_METADATA">Q_PLUGIN_METADATA</a>() macro when compiling the plugin.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonObject QPluginLoader::metaData() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpluginloader.html#metaData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the meta data for this plugin. The meta data is data specified in a json format using the <a href="http://doc.qt.io/qt-5/qtplugin.html#Q_PLUGIN_METADATA">Q_PLUGIN_METADATA</a>() macro when compiling the plugin.</p>
    /// <p>The meta data can be queried in a fast and inexpensive way without actually loading the plugin. This makes it possible to e.g. store capabilities of the plugin in there, and make the decision whether to load the plugin dependent on this meta data.</p></div>
    pub unsafe fn meta_data(&self) -> ::cpp_utils::CppBox<crate::QJsonObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPluginLoader_metaData(
            self as *const crate::QPluginLoader,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QPluginLoader::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPluginLoader_metaObject(
            self as *const crate::QPluginLoader,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs a plugin loader with the given <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPluginLoader::QPluginLoader(QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpluginloader.html#QPluginLoader">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a plugin loader with the given <i>parent</i>.</p></div>
    pub unsafe fn from_q_object(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QPluginLoader> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPluginLoader_QPluginLoader(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a plugin loader with the given <i>parent</i> that will load the plugin specified by <i>fileName</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPluginLoader::QPluginLoader(const QString& fileName, QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpluginloader.html#QPluginLoader-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a plugin loader with the given <i>parent</i> that will load the plugin specified by <i>fileName</i>.</p>
    /// <p>To be loadable, the file's suffix must be a valid suffix for a loadable library in accordance with the platform, e.g. <code>.so</code> on Unix, - <code>.dylib</code> on <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS, and <code>.dll</code> on Windows. The suffix can be verified with <a href="http://doc.qt.io/qt-5/qlibrary.html#isLibrary">QLibrary::isLibrary</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpluginloader.html#fileName-prop">setFileName</a>().</p></div>
    pub unsafe fn from_q_string_q_object(
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QPluginLoader> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPluginLoader_QPluginLoader1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> class loads a plugin at run-time.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPluginLoader::QPluginLoader()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpluginloader.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> class loads a plugin at run-time.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> provides access to a <a href="http://doc.qt.io/qt-5/plugins-howto.html">Qt plugin</a>. A Qt plugin is stored in a shared library (a DLL) and offers these benefits over shared libraries accessed using <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a>:</p>
    /// <ul>
    /// <li><a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> checks that a plugin is linked against the same version of Qt as the application.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> provides direct access to a root component object (<a href="http://doc.qt.io/qt-5/qpluginloader.html#instance">instance</a>()), instead of forcing you to resolve a C function manually.</li>
    /// </ul>
    /// <p>An instance of a <a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> object operates on a single shared library file, which we call a plugin. It provides access to the functionality in the plugin in a platform-independent way. To specify which plugin to load, either pass a file name in the constructor or set it with <a href="http://doc.qt.io/qt-5/qpluginloader.html#fileName-prop">setFileName</a>().</p>
    /// <p>The most important functions are <a href="http://doc.qt.io/qt-5/qpluginloader.html#load">load</a>() to dynamically load the plugin file, <a href="http://doc.qt.io/qt-5/qpluginloader.html#isLoaded">isLoaded</a>() to check whether loading was successful, and <a href="http://doc.qt.io/qt-5/qpluginloader.html#instance">instance</a>() to access the root component in the plugin. The <a href="http://doc.qt.io/qt-5/qpluginloader.html#instance">instance</a>() function implicitly tries to load the plugin if it has not been loaded yet. Multiple instances of <a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> can be used to access the same physical plugin.</p>
    /// <p>Once loaded, plugins remain in memory until all instances of <a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> has been unloaded, or until the application terminates. You can attempt to unload a plugin using <a href="http://doc.qt.io/qt-5/qpluginloader.html#unload">unload</a>(), but if other instances of <a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> are using the same library, the call will fail, and unloading will only happen when every instance has called <a href="http://doc.qt.io/qt-5/qpluginloader.html#unload">unload</a>(). Right before the unloading happen, the root component will also be deleted.</p>
    /// <p>See <a href="http://doc.qt.io/qt-5/plugins-howto.html">How to Create Qt Plugins</a> for more information about how to make your application extensible through plugins.</p>
    /// <p>Note that the <a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> cannot be used if your application is statically linked against Qt. In this case, you will also have to link to plugins statically. You can use <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> if you need to load dynamic libraries in a statically linked application.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QPluginLoader> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPluginLoader_QPluginLoader2();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a plugin loader with the given <i>parent</i> that will load the plugin specified by <i>fileName</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPluginLoader::QPluginLoader(const QString& fileName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpluginloader.html#QPluginLoader-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a plugin loader with the given <i>parent</i> that will load the plugin specified by <i>fileName</i>.</p>
    /// <p>To be loadable, the file's suffix must be a valid suffix for a loadable library in accordance with the platform, e.g. <code>.so</code> on Unix, - <code>.dylib</code> on <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS, and <code>.dll</code> on Windows. The suffix can be verified with <a href="http://doc.qt.io/qt-5/qlibrary.html#isLibrary">QLibrary::isLibrary</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpluginloader.html#fileName-prop">setFileName</a>().</p></div>
    pub unsafe fn from_q_string(
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QPluginLoader> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPluginLoader_QPluginLoader3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QPluginLoader::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QPluginLoader_qt_metacall(
            self as *mut crate::QPluginLoader,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QPluginLoader::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPluginLoader_qt_metacast(
            self as *mut crate::QPluginLoader,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This property holds the file name of the plugin</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QPluginLoader::setFileName(const QString& fileName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpluginloader.html#fileName-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the file name of the plugin</p>
    /// <p>We recommend omitting the file's suffix in the file name, since <a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> will automatically look for the file with the appropriate suffix (see <a href="http://doc.qt.io/qt-5/qlibrary.html#isLibrary">QLibrary::isLibrary</a>()).</p>
    /// <p>When loading the plugin, <a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> searches in the current directory and in all plugin locations specified by <a href="http://doc.qt.io/qt-5/qcoreapplication.html#libraryPaths">QCoreApplication::libraryPaths</a>(), unless the file name has an absolute path. After loading the plugin successfully, fileName() returns the fully-qualified file name of the plugin, including the full path to the plugin if one was given in the constructor or passed to setFileName().</p>
    /// <p>If the file name does not exist, it will not be set. This property will then contain an empty string.</p>
    /// <p>By default, this property contains an empty string.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QString </td><td class="memItemRight bottomAlign"><span class="name"><b>fileName</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setFileName</b></span>(const QString &amp;<i>fileName</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpluginloader.html#load">load</a>().</p></div>
    pub unsafe fn set_file_name(
        &mut self,
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QPluginLoader_setFileName(
            self as *mut crate::QPluginLoader,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        )
    }

    /// <p>Give the <a href="http://doc.qt.io/qt-5/qpluginloader.html#load">load</a>() function some hints on how it should behave.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QPluginLoader::setLoadHints(QFlags<QLibrary::LoadHint> loadHints)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpluginloader.html#loadHints-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Give the <a href="http://doc.qt.io/qt-5/qpluginloader.html#load">load</a>() function some hints on how it should behave.</p>
    /// <p>You can give hints on how the symbols in the plugin are resolved. By default since Qt 5.7, <a href="http://doc.qt.io/qt-5/qlibrary.html#LoadHint-enum">QLibrary::PreventUnloadHint</a> is set.</p>
    /// <p>See the documentation of <a href="http://doc.qt.io/qt-5/qlibrary.html#loadHints-prop">QLibrary::loadHints</a> for a complete description of how this property works.</p>
    /// <p>This property was introduced in  Qt 4.4.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QLibrary::LoadHints </td><td class="memItemRight bottomAlign"><span class="name"><b>loadHints</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setLoadHints</b></span>(QLibrary::LoadHints <i>loadHints</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlibrary.html#loadHints-prop">QLibrary::loadHints</a>.</p></div>
    pub unsafe fn set_load_hints(&mut self, load_hints: crate::QFlags<crate::q_library::LoadHint>) {
        crate::__ffi::ctr_qt_core_ffi_QPluginLoader_setLoadHints(
            self as *mut crate::QPluginLoader,
            load_hints.to_int(),
        )
    }

    /// <p>Returns a list of static plugin instances (root components) held by the plugin loader.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QList<QObject*> QPluginLoader::staticInstances()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpluginloader.html#staticInstances">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of static plugin instances (root components) held by the plugin loader.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpluginloader.html#staticPlugins">staticPlugins</a>().</p></div>
    pub unsafe fn static_instances() -> ::cpp_utils::CppBox<crate::QListOfQObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPluginLoader_staticInstances();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPluginLoader_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a list of QStaticPlugins held by the plugin loader. The function is similar to <a href="http://doc.qt.io/qt-5/qpluginloader.html#staticInstances">staticInstances</a>() with the addition that a <a href="http://doc.qt.io/qt-5/qstaticplugin.html">QStaticPlugin</a> also contains meta data information.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QVector<QStaticPlugin> QPluginLoader::staticPlugins()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpluginloader.html#staticPlugins">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of QStaticPlugins held by the plugin loader. The function is similar to <a href="http://doc.qt.io/qt-5/qpluginloader.html#staticInstances">staticInstances</a>() with the addition that a <a href="http://doc.qt.io/qt-5/qstaticplugin.html">QStaticPlugin</a> also contains meta data information.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpluginloader.html#staticInstances">staticInstances</a>().</p></div>
    pub unsafe fn static_plugins() -> ::cpp_utils::CppBox<crate::QVectorOfQStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPluginLoader_staticPlugins();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QPluginLoader::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPluginLoader_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QPluginLoader::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPluginLoader_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Unloads the plugin and returns <code>true</code> if the plugin could be unloaded; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QPluginLoader::unload()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpluginloader.html#unload">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Unloads the plugin and returns <code>true</code> if the plugin could be unloaded; otherwise returns <code>false</code>.</p>
    /// <p>This happens automatically on application termination, so you shouldn't normally need to call this function.</p>
    /// <p>If other instances of <a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> are using the same plugin, the call will fail, and unloading will only happen when every instance has called unload().</p>
    /// <p>Don't try to delete the root component. Instead rely on that unload() will automatically delete it when needed.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpluginloader.html#instance">instance</a>() and <a href="http://doc.qt.io/qt-5/qpluginloader.html#load">load</a>().</p></div>
    pub unsafe fn unload(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QPluginLoader_unload(self as *mut crate::QPluginLoader)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> class holds the environment variables that can be passed to a program.</p>
///
/// C++ class: <span style='color: green;'>```QProcessEnvironment```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> class holds the environment variables that can be passed to a program.</p>
/// <p>A process's environment is composed of a set of key=value pairs known as environment variables. The <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> class wraps that concept and allows easy manipulation of those variables. It's meant to be used along with <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>, to set the environment for child processes. It cannot be used to change the current process's environment.</p>
/// <p>The environment of the calling process can be obtained using <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#systemEnvironment">QProcessEnvironment::systemEnvironment</a>().</p>
/// <p>On Unix systems, the variable names are case-sensitive. Note that the Unix environment allows both variable names and contents to contain arbitrary binary data (except for the NUL character). <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> will preserve such variables, but does not support manipulating variables whose names or values cannot be encoded by the current locale settings (see <a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForLocale">QTextCodec::codecForLocale</a>).</p>
/// <p>On Windows, the variable names are case-insensitive, but case-preserving. <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> behaves accordingly.</p></div>
#[repr(C)]
pub struct QProcessEnvironment {
    _unused: u8,
}
impl QProcessEnvironment {
    /// <p>Removes all key=value pairs from this <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object, making it empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcessEnvironment::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all key=value pairs from this <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object, making it empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocessenvironment.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#systemEnvironment">systemEnvironment</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QProcessEnvironment_clear(
            self as *mut crate::QProcessEnvironment,
        )
    }

    /// <p>Returns <code>true</code> if the environment variable of name <i>name</i> is found in this <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QProcessEnvironment::contains(const QString& name) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the environment variable of name <i>name</i> is found in this <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocessenvironment.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#value">value</a>().</p></div>
    pub unsafe fn contains(
        &self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QProcessEnvironment_contains(
            self as *const crate::QProcessEnvironment,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
        )
    }

    /// <p>Copies the contents of the <i>other</i> <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object into this one.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QProcessEnvironment& QProcessEnvironment::operator=(const QProcessEnvironment& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#operator-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Copies the contents of the <i>other</i> <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object into this one.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QProcessEnvironment>>,
    ) -> ::cpp_utils::MutRef<crate::QProcessEnvironment> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QProcessEnvironment_operator_1(
            self as *mut crate::QProcessEnvironment,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QProcessEnvironment>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Inserts the environment variable of name <i>name</i> and contents <i>value</i> into this <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object. If that variable already existed, it is replaced by the new value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcessEnvironment::insert(const QString& name, const QString& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts the environment variable of name <i>name</i> and contents <i>value</i> into this <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object. If that variable already existed, it is replaced by the new value.</p>
    /// <p>On most systems, inserting a variable with no contents will have the same effect for applications as if the variable had not been set at all. However, to guarantee that there are no incompatibilities, to remove a variable, please use the <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#remove">remove</a>() function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocessenvironment.html#contains">contains</a>(), <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#remove">remove</a>(), and <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#value">value</a>().</p></div>
    pub unsafe fn insert_2a(
        &mut self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QProcessEnvironment_insert(
            self as *mut crate::QProcessEnvironment,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcessEnvironment::insert(const QProcessEnvironment& e)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts the contents of <i>e</i> in this <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object. Variables in this object that also exist in <i>e</i> will be overwritten.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn insert_1a(
        &mut self,
        e: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QProcessEnvironment>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QProcessEnvironment_insert1(
            self as *mut crate::QProcessEnvironment,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QProcessEnvironment>>::cast_into(e)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if this <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object is empty: that is there are no key=value pairs set.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QProcessEnvironment::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object is empty: that is there are no key=value pairs set.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocessenvironment.html#clear">clear</a>(), <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#systemEnvironment">systemEnvironment</a>(), and <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#insert">insert</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QProcessEnvironment_isEmpty(
            self as *const crate::QProcessEnvironment,
        )
    }

    /// <p>Returns a list containing all the variable names in this <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QProcessEnvironment::keys() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#keys">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list containing all the variable names in this <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn keys(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QProcessEnvironment_keys(
            self as *const crate::QProcessEnvironment,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a new <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object. This constructor creates an empty environment. If set on a <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>, this will cause the current environment variables to be removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QProcessEnvironment::QProcessEnvironment()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#QProcessEnvironment">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a new <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object. This constructor creates an empty environment. If set on a <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>, this will cause the current environment variables to be removed.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QProcessEnvironment> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QProcessEnvironment_QProcessEnvironment();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object that is a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QProcessEnvironment::QProcessEnvironment(const QProcessEnvironment& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#QProcessEnvironment-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object that is a copy of <i>other</i>.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QProcessEnvironment>>,
    ) -> ::cpp_utils::CppBox<crate::QProcessEnvironment> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QProcessEnvironment_QProcessEnvironment1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QProcessEnvironment>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the environment variable identified by <i>name</i> from this <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object. If that variable did not exist before, nothing happens.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcessEnvironment::remove(const QString& name)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#remove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the environment variable identified by <i>name</i> from this <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object. If that variable did not exist before, nothing happens.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocessenvironment.html#contains">contains</a>(), <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#insert">insert</a>(), and <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#value">value</a>().</p></div>
    pub unsafe fn remove(
        &mut self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QProcessEnvironment_remove(
            self as *mut crate::QProcessEnvironment,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
        )
    }

    /// <p>Swaps this process environment instance with <i>other</i>. This function is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcessEnvironment::swap(QProcessEnvironment& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps this process environment instance with <i>other</i>. This function is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QProcessEnvironment>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QProcessEnvironment_swap(
            self as *mut crate::QProcessEnvironment,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QProcessEnvironment>>::cast_into(
                other,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>The systemEnvironment function returns the environment of the calling process.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QProcessEnvironment QProcessEnvironment::systemEnvironment()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#systemEnvironment">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The systemEnvironment function returns the environment of the calling process.</p>
    /// <p>It is returned as a <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a>. This function does not cache the system environment. Therefore, it's possible to obtain an updated version of the environment if low-level C library functions like <code>setenv</code> or <code>putenv</code> have been called.</p>
    /// <p>However, note that repeated calls to this function will recreate the <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object, which is a non-trivial operation.</p>
    /// <p>This function was introduced in  Qt 4.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#systemEnvironment">QProcess::systemEnvironment</a>().</p></div>
    pub unsafe fn system_environment() -> ::cpp_utils::CppBox<crate::QProcessEnvironment> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QProcessEnvironment_systemEnvironment();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts this <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object into a list of strings, one for each environment variable that is set. The environment variable's name and its value are separated by an equal character ('=').</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QProcessEnvironment::toStringList() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#toStringList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts this <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object into a list of strings, one for each environment variable that is set. The environment variable's name and its value are separated by an equal character ('=').</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> contents returned by this function are suitable for presentation. Use with the QProcess::setEnvironment function is not recommended due to potential encoding problems under Unix, and worse performance.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocessenvironment.html#systemEnvironment">systemEnvironment</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#systemEnvironment">QProcess::systemEnvironment</a>(), and <a href="http://doc.qt.io/qt-5/qprocess.html#setProcessEnvironment">QProcess::setProcessEnvironment</a>().</p></div>
    pub unsafe fn to_string_list(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QProcessEnvironment_toStringList(
            self as *const crate::QProcessEnvironment,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Searches this <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object for a variable identified by <i>name</i> and returns its value. If the variable is not found in this object, then <i>defaultValue</i> is returned instead.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QProcessEnvironment::value(const QString& name, const QString& defaultValue = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Searches this <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object for a variable identified by <i>name</i> and returns its value. If the variable is not found in this object, then <i>defaultValue</i> is returned instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocessenvironment.html#contains">contains</a>(), <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#insert">insert</a>(), and <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#remove">remove</a>().</p></div>
    pub unsafe fn value_2a(
        &self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QProcessEnvironment_value(
            self as *const crate::QProcessEnvironment,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Searches this <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object for a variable identified by <i>name</i> and returns its value. If the variable is not found in this object, then <i>defaultValue</i> is returned instead.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QProcessEnvironment::value(const QString& name) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Searches this <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object for a variable identified by <i>name</i> and returns its value. If the variable is not found in this object, then <i>defaultValue</i> is returned instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocessenvironment.html#contains">contains</a>(), <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#insert">insert</a>(), and <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#remove">remove</a>().</p></div>
    pub unsafe fn value_1a(
        &self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QProcessEnvironment_value1(
            self as *const crate::QProcessEnvironment,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_process {
    //! C++ type: <span style='color: green;'>```QProcess```</span>

    /// <p>This enum describes the different types of errors that are reported by <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QProcess::ProcessError```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#ProcessError-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the different types of errors that are reported by <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#error">error</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct ProcessError(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for ProcessError {
        fn from(value: ::std::os::raw::c_int) -> Self {
            ProcessError(value)
        }
    }

    impl From<ProcessError> for ::std::os::raw::c_int {
        fn from(value: ProcessError) -> Self {
            value.0
        }
    }

    impl ProcessError {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl ProcessError {
        /// The process failed to start. Either the invoked program is missing, or you may have insufficient permissions to invoke the program. (C++ enum variant: <span style='color: green;'>```FailedToStart = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const FailedToStart: crate::q_process::ProcessError = crate::q_process::ProcessError(0);
        /// The process crashed some time after starting successfully. (C++ enum variant: <span style='color: green;'>```Crashed = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Crashed: crate::q_process::ProcessError = crate::q_process::ProcessError(1);
        /// The last waitFor...() function timed out. The state of <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> is unchanged, and you can try calling waitFor...() again. (C++ enum variant: <span style='color: green;'>```Timedout = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const Timedout: crate::q_process::ProcessError = crate::q_process::ProcessError(2);
        /// An error occurred when attempting to read from the process. For example, the process may not be running. (C++ enum variant: <span style='color: green;'>```ReadError = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const ReadError: crate::q_process::ProcessError = crate::q_process::ProcessError(3);
        /// An error occurred when attempting to write to the process. For example, the process may not be running, or it may have closed its input channel. (C++ enum variant: <span style='color: green;'>```WriteError = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const WriteError: crate::q_process::ProcessError = crate::q_process::ProcessError(4);
        /// An unknown error occurred. This is the default return value of <a href="http://doc.qt.io/qt-5/qprocess.html#error">error</a>(). (C++ enum variant: <span style='color: green;'>```UnknownError = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const UnknownError: crate::q_process::ProcessError = crate::q_process::ProcessError(5);
    }

    /// <p>This enum describes the different states of <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QProcess::ProcessState```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#ProcessState-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the different states of <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#state">state</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct ProcessState(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for ProcessState {
        fn from(value: ::std::os::raw::c_int) -> Self {
            ProcessState(value)
        }
    }

    impl From<ProcessState> for ::std::os::raw::c_int {
        fn from(value: ProcessState) -> Self {
            value.0
        }
    }

    impl ProcessState {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl ProcessState {
        /// The process is not running. (C++ enum variant: <span style='color: green;'>```NotRunning = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NotRunning: crate::q_process::ProcessState = crate::q_process::ProcessState(0);
        /// The process is starting, but the program has not yet been invoked. (C++ enum variant: <span style='color: green;'>```Starting = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Starting: crate::q_process::ProcessState = crate::q_process::ProcessState(1);
        /// The process is running and is ready for reading and writing. (C++ enum variant: <span style='color: green;'>```Running = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const Running: crate::q_process::ProcessState = crate::q_process::ProcessState(2);
    }

    /// <p>This enum describes the process channels used by the running process. Pass one of these values to <a href="http://doc.qt.io/qt-5/qprocess.html#setReadChannel">setReadChannel</a>() to set the current read channel of <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QProcess::ProcessChannel```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#ProcessChannel-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the process channels used by the running process. Pass one of these values to <a href="http://doc.qt.io/qt-5/qprocess.html#setReadChannel">setReadChannel</a>() to set the current read channel of <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#setReadChannel">setReadChannel</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct ProcessChannel(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for ProcessChannel {
        fn from(value: ::std::os::raw::c_int) -> Self {
            ProcessChannel(value)
        }
    }

    impl From<ProcessChannel> for ::std::os::raw::c_int {
        fn from(value: ProcessChannel) -> Self {
            value.0
        }
    }

    impl ProcessChannel {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl ProcessChannel {
        /// The standard output (stdout) of the running process. (C++ enum variant: <span style='color: green;'>```StandardOutput = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const StandardOutput: crate::q_process::ProcessChannel =
            crate::q_process::ProcessChannel(0);
        /// The standard error (stderr) of the running process. (C++ enum variant: <span style='color: green;'>```StandardError = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const StandardError: crate::q_process::ProcessChannel =
            crate::q_process::ProcessChannel(1);
    }

    /// <p>This enum describes the process output channel modes of <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>. Pass one of these values to <a href="http://doc.qt.io/qt-5/qprocess.html#setProcessChannelMode">setProcessChannelMode</a>() to set the current read channel mode.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QProcess::ProcessChannelMode```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#ProcessChannelMode-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the process output channel modes of <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>. Pass one of these values to <a href="http://doc.qt.io/qt-5/qprocess.html#setProcessChannelMode">setProcessChannelMode</a>() to set the current read channel mode.</p>
    ///
    /// <p><b>Note: </b>Windows intentionally suppresses output from GUI-only applications to inherited consoles. This does <i>not</i> apply to output redirected to files or pipes. To forward the output of GUI-only applications on the console nonetheless, you must use SeparateChannels and do the forwarding yourself by reading the output and writing it to the appropriate output channels.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#setProcessChannelMode">setProcessChannelMode</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct ProcessChannelMode(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for ProcessChannelMode {
        fn from(value: ::std::os::raw::c_int) -> Self {
            ProcessChannelMode(value)
        }
    }

    impl From<ProcessChannelMode> for ::std::os::raw::c_int {
        fn from(value: ProcessChannelMode) -> Self {
            value.0
        }
    }

    impl ProcessChannelMode {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl ProcessChannelMode {
        /// <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> manages the output of the running process, keeping standard output and standard error data in separate internal buffers. You can select the <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>'s current read channel by calling <a href="http://doc.qt.io/qt-5/qprocess.html#setReadChannel">setReadChannel</a>(). This is the default channel mode of <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>. (C++ enum variant: <span style='color: green;'>```SeparateChannels = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const SeparateChannels: crate::q_process::ProcessChannelMode =
            crate::q_process::ProcessChannelMode(0);
        /// <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> merges the output of the running process into the standard output channel (<code>stdout</code>). The standard error channel (<code>stderr</code>) will not receive any data. The standard output and standard error data of the running process are interleaved. (C++ enum variant: <span style='color: green;'>```MergedChannels = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const MergedChannels: crate::q_process::ProcessChannelMode =
            crate::q_process::ProcessChannelMode(1);
        /// <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> forwards the output of the running process onto the main process. Anything the child process writes to its standard output and standard error will be written to the standard output and standard error of the main process. (C++ enum variant: <span style='color: green;'>```ForwardedChannels = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const ForwardedChannels: crate::q_process::ProcessChannelMode =
            crate::q_process::ProcessChannelMode(2);
        /// Complementary to ForwardedErrorChannel. (This value was introduced in Qt 5.2.) (C++ enum variant: <span style='color: green;'>```ForwardedOutputChannel = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const ForwardedOutputChannel: crate::q_process::ProcessChannelMode =
            crate::q_process::ProcessChannelMode(3);
        /// <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> manages the standard output of the running process, but forwards its standard error onto the main process. This reflects the typical use of command line tools as filters, where the standard output is redirected to another process or a file, while standard error is printed to the console for diagnostic purposes. (This value was introduced in Qt 5.2.) (C++ enum variant: <span style='color: green;'>```ForwardedErrorChannel = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const ForwardedErrorChannel: crate::q_process::ProcessChannelMode =
            crate::q_process::ProcessChannelMode(4);
    }

    /// <p>This enum describes the process input channel modes of <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>. Pass one of these values to <a href="http://doc.qt.io/qt-5/qprocess.html#setInputChannelMode">setInputChannelMode</a>() to set the current write channel mode.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QProcess::InputChannelMode```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#InputChannelMode-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the process input channel modes of <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>. Pass one of these values to <a href="http://doc.qt.io/qt-5/qprocess.html#setInputChannelMode">setInputChannelMode</a>() to set the current write channel mode.</p>
    ///
    /// <p>This enum was introduced or modified in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#setInputChannelMode">setInputChannelMode</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct InputChannelMode(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for InputChannelMode {
        fn from(value: ::std::os::raw::c_int) -> Self {
            InputChannelMode(value)
        }
    }

    impl From<InputChannelMode> for ::std::os::raw::c_int {
        fn from(value: InputChannelMode) -> Self {
            value.0
        }
    }

    impl InputChannelMode {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl InputChannelMode {
        /// <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> manages the input of the running process. This is the default input channel mode of <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>. (C++ enum variant: <span style='color: green;'>```ManagedInputChannel = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const ManagedInputChannel: crate::q_process::InputChannelMode =
            crate::q_process::InputChannelMode(0);
        /// <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> forwards the input of the main process onto the running process. The child process reads its standard input from the same source as the main process. Note that the main process must not try to read its standard input while the child process is running. (C++ enum variant: <span style='color: green;'>```ForwardedInputChannel = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const ForwardedInputChannel: crate::q_process::InputChannelMode =
            crate::q_process::InputChannelMode(1);
    }

    /// <p>This enum describes the different exit statuses of <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QProcess::ExitStatus```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#ExitStatus-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the different exit statuses of <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#exitStatus">exitStatus</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct ExitStatus(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for ExitStatus {
        fn from(value: ::std::os::raw::c_int) -> Self {
            ExitStatus(value)
        }
    }

    impl From<ExitStatus> for ::std::os::raw::c_int {
        fn from(value: ExitStatus) -> Self {
            value.0
        }
    }

    impl ExitStatus {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl ExitStatus {
        /// The process exited normally. (C++ enum variant: <span style='color: green;'>```NormalExit = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NormalExit: crate::q_process::ExitStatus = crate::q_process::ExitStatus(0);
        /// The process crashed. (C++ enum variant: <span style='color: green;'>```CrashExit = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const CrashExit: crate::q_process::ExitStatus = crate::q_process::ExitStatus(1);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> class is used to start external programs and to communicate with them.</p>
///
/// C++ class: <span style='color: green;'>```QProcess```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qprocess.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> class is used to start external programs and to communicate with them.</p>
/// <a name="running-a-process"></a></div>
#[repr(C)]
pub struct QProcess {
    _unused: u8,
}
impl QProcess {
    /// <p>Attempts to terminate the process.</p>
    ///
    /// Returns a built-in Qt slot `QProcess::terminate` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#terminate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to terminate the process.</p>
    /// <p>The process may not exit as a result of calling this function (it is given the chance to prompt the user for any unsaved files, etc).</p>
    /// <p>On Windows, terminate() posts a WM_CLOSE message to all top-level windows of the process and then to the main thread of the process itself. On Unix and <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> the <code>SIGTERM</code> signal is sent.</p>
    /// <p>Console applications on Windows that do not run an event loop, or whose event loop does not handle the WM_CLOSE message, can only be terminated by calling <a href="http://doc.qt.io/qt-5/qprocess.html#kill">kill</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#kill">kill</a>().</p></div>
    pub fn slot_terminate(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1terminate()\0"),
            )
        }
    }

    /// <p>Kills the current process, causing it to exit immediately.</p>
    ///
    /// Returns a built-in Qt slot `QProcess::kill` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#kill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Kills the current process, causing it to exit immediately.</p>
    /// <p>On Windows, kill() uses TerminateProcess, and on Unix and <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, the SIGKILL signal is sent to the process.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#terminate">terminate</a>().</p></div>
    pub fn slot_kill(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1kill()\0"),
            )
        }
    }

    /// <p>This signal is emitted by <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> when the process has started, and <a href="http://doc.qt.io/qt-5/qprocess.html#state">state</a>() returns <a href="http://doc.qt.io/qt-5/qprocess.html#ProcessState-enum">Running</a>.</p>
    ///
    /// Returns a built-in Qt signal `QProcess::started` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#started">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted by <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> when the process has started, and <a href="http://doc.qt.io/qt-5/qprocess.html#state">state</a>() returns <a href="http://doc.qt.io/qt-5/qprocess.html#ProcessState-enum">Running</a>.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p></div>
    pub fn started(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2started()\0"),
            )
        }
    }

    /// <p>This signal is emitted when the process finishes. <i>exitCode</i> is the exit code of the process (only valid for normal exits), and <i>exitStatus</i> is the exit status. After the process has finished, the buffers in <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> are still intact. You can still read any data that the process may have written before it finished.</p>
    ///
    /// Returns a built-in Qt signal `QProcess::finished` that can be passed to `qt_core::Signal::connect`.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qprocess.html#finished">C++ documentation</a> for <span style='color: green;'>``` void QProcess::finished(int exitCode, QProcess::ExitStatus exitStatus)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the process finishes. <i>exitCode</i> is the exit code of the process (only valid for normal exits), and <i>exitStatus</i> is the exit status. After the process has finished, the buffers in <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> are still intact. You can still read any data that the process may have written before it finished.</p>
    /// <p><b>Note:</b> Signal <i>finished</i> is overloaded in this class. To connect to this one using the function pointer syntax, you must specify the signal type in a static cast, as shown in this example:</p><pre class="cpp">
    ///
    ///   connect(process<span class="operator">,</span> <span class="keyword">static_cast</span><span class="operator">&lt;</span><span class="type">void</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span><span class="operator">::</span><span class="operator">*</span>)(<span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span><span class="operator">::</span>ExitStatus)<span class="operator">&gt;</span>(<span class="operator">&amp;</span><span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span><span class="operator">::</span>finished)<span class="operator">,</span>
    /// &#32;     <span class="operator">[</span><span class="operator">=</span><span class="operator">]</span>(<span class="type">int</span> exitCode<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span><span class="operator">::</span>ExitStatus exitStatus){ <span class="comment">/* ... */</span> });
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#exitStatus">exitStatus</a>().</p></div>
    pub fn finished(&self) -> crate::Signal<(::std::os::raw::c_int,)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2finished(int)\0"),
            )
        }
    }

    /// <p>This signal is emitted when the process finishes. <i>exitCode</i> is the exit code of the process (only valid for normal exits), and <i>exitStatus</i> is the exit status. After the process has finished, the buffers in <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> are still intact. You can still read any data that the process may have written before it finished.</p>
    ///
    /// Returns a built-in Qt signal `QProcess::finished` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#finished">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the process finishes. <i>exitCode</i> is the exit code of the process (only valid for normal exits), and <i>exitStatus</i> is the exit status. After the process has finished, the buffers in <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> are still intact. You can still read any data that the process may have written before it finished.</p>
    /// <p><b>Note:</b> Signal <i>finished</i> is overloaded in this class. To connect to this one using the function pointer syntax, you must specify the signal type in a static cast, as shown in this example:</p><pre class="cpp">
    ///
    ///   connect(process<span class="operator">,</span> <span class="keyword">static_cast</span><span class="operator">&lt;</span><span class="type">void</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span><span class="operator">::</span><span class="operator">*</span>)(<span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span><span class="operator">::</span>ExitStatus)<span class="operator">&gt;</span>(<span class="operator">&amp;</span><span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span><span class="operator">::</span>finished)<span class="operator">,</span>
    /// &#32;     <span class="operator">[</span><span class="operator">=</span><span class="operator">]</span>(<span class="type">int</span> exitCode<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span><span class="operator">::</span>ExitStatus exitStatus){ <span class="comment">/* ... */</span> });
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#exitStatus">exitStatus</a>().</p></div>
    pub fn finished2(
        &self,
    ) -> crate::Signal<(::std::os::raw::c_int, crate::q_process::ExitStatus)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2finished(int,QProcess::ExitStatus)\0",
                ),
            )
        }
    }

    /// <p>Use <a href="http://doc.qt.io/qt-5/qprocess.html#errorOccurred">errorOccurred</a>() instead.</p>
    ///
    /// Returns a built-in Qt signal `QProcess::error` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess-obsolete.html#error-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Use <a href="http://doc.qt.io/qt-5/qprocess.html#errorOccurred">errorOccurred</a>() instead.</p>
    /// <p><b>Note:</b> Signal <i>error</i> is overloaded in this class. To connect to this one using the function pointer syntax, you must specify the signal type in a static cast, as shown in this example:</p><pre class="cpp">
    ///
    ///   connect(process<span class="operator">,</span> <span class="keyword">static_cast</span><span class="operator">&lt;</span><span class="type">void</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span><span class="operator">::</span><span class="operator">*</span>)(<span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span><span class="operator">::</span>ProcessError)<span class="operator">&gt;</span>(<span class="operator">&amp;</span><span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span><span class="operator">::</span>error)<span class="operator">,</span>
    /// &#32;     <span class="operator">[</span><span class="operator">=</span><span class="operator">]</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span><span class="operator">::</span>ProcessError error){ <span class="comment">/* ... */</span> });
    ///
    /// </pre></div>
    pub fn error(&self) -> crate::Signal<(crate::q_process::ProcessError,)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2error(QProcess::ProcessError)\0",
                ),
            )
        }
    }

    /// <p>This signal is emitted when an error occurs with the process. The specified <i>error</i> describes the type of error that occurred.</p>
    ///
    /// Returns a built-in Qt signal `QProcess::errorOccurred` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#errorOccurred">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when an error occurs with the process. The specified <i>error</i> describes the type of error that occurred.</p>
    /// <p>This function was introduced in  Qt 5.6.</p></div>
    pub fn error_occurred(&self) -> crate::Signal<(crate::q_process::ProcessError,)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2errorOccurred(QProcess::ProcessError)\0",
                ),
            )
        }
    }

    /// <p>This signal is emitted whenever the state of <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> changes. The <i>newState</i> argument is the state <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> changed to.</p>
    ///
    /// Returns a built-in Qt signal `QProcess::stateChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#stateChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted whenever the state of <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> changes. The <i>newState</i> argument is the state <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> changed to.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p></div>
    pub fn state_changed(&self) -> crate::Signal<(crate::q_process::ProcessState,)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2stateChanged(QProcess::ProcessState)\0",
                ),
            )
        }
    }

    /// <p>This signal is emitted when the process has made new data available through its standard output channel (<code>stdout</code>). It is emitted regardless of the current <a href="http://doc.qt.io/qt-5/qprocess.html#readChannel">read channel</a>.</p>
    ///
    /// Returns a built-in Qt signal `QProcess::readyReadStandardOutput` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#readyReadStandardOutput">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the process has made new data available through its standard output channel (<code>stdout</code>). It is emitted regardless of the current <a href="http://doc.qt.io/qt-5/qprocess.html#readChannel">read channel</a>.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#readAllStandardOutput">readAllStandardOutput</a>() and <a href="http://doc.qt.io/qt-5/qprocess.html#readChannel">readChannel</a>().</p></div>
    pub fn ready_read_standard_output(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2readyReadStandardOutput()\0"),
            )
        }
    }

    /// <p>This signal is emitted when the process has made new data available through its standard error channel (<code>stderr</code>). It is emitted regardless of the current <a href="http://doc.qt.io/qt-5/qprocess.html#readChannel">read channel</a>.</p>
    ///
    /// Returns a built-in Qt signal `QProcess::readyReadStandardError` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#readyReadStandardError">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the process has made new data available through its standard error channel (<code>stderr</code>). It is emitted regardless of the current <a href="http://doc.qt.io/qt-5/qprocess.html#readChannel">read channel</a>.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#readAllStandardError">readAllStandardError</a>() and <a href="http://doc.qt.io/qt-5/qprocess.html#readChannel">readChannel</a>().</p></div>
    pub fn ready_read_standard_error(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2readyReadStandardError()\0"),
            )
        }
    }

    /// <p>Returns the command line arguments the process was last started with.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QProcess::arguments() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#arguments">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the command line arguments the process was last started with.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#setArguments">setArguments</a>() and <a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>().</p></div>
    pub unsafe fn arguments(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QProcess_arguments(self as *const crate::QProcess);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#atEnd">QIODevice::atEnd</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QProcess::atEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#atEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#atEnd">QIODevice::atEnd</a>().</p>
    /// <p>Returns <code>true</code> if the process is not running, and no more data is available for reading; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn at_end(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QProcess_atEnd(self as *const crate::QProcess)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#bytesAvailable">QIODevice::bytesAvailable</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual qint64 QProcess::bytesAvailable() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#bytesAvailable">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#bytesAvailable">QIODevice::bytesAvailable</a>().</p></div>
    pub unsafe fn bytes_available(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QProcess_bytesAvailable(self as *const crate::QProcess)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#bytesToWrite">QIODevice::bytesToWrite</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual qint64 QProcess::bytesToWrite() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#bytesToWrite">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#bytesToWrite">QIODevice::bytesToWrite</a>().</p></div>
    pub unsafe fn bytes_to_write(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QProcess_bytesToWrite(self as *const crate::QProcess)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#canReadLine">QIODevice::canReadLine</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QProcess::canReadLine() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#canReadLine">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#canReadLine">QIODevice::canReadLine</a>().</p>
    /// <p>This function operates on the current read channel.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#readChannel">readChannel</a>() and <a href="http://doc.qt.io/qt-5/qprocess.html#setReadChannel">setReadChannel</a>().</p></div>
    pub unsafe fn can_read_line(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QProcess_canReadLine(self as *const crate::QProcess)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#close">QIODevice::close</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual void QProcess::close()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#close">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#close">QIODevice::close</a>().</p>
    /// <p>Closes all communication with the process and kills it. After calling this function, <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> will no longer emit <a href="http://doc.qt.io/qt-5/qiodevice.html#readyRead">readyRead</a>(), and data can no longer be read or written.</p></div>
    pub unsafe fn close(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QProcess_close(self as *mut crate::QProcess)
    }

    /// <p>Closes the read channel <i>channel</i>. After calling this function, <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> will no longer receive data on the channel. Any data that has already been received is still available for reading.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcess::closeReadChannel(QProcess::ProcessChannel channel)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#closeReadChannel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Closes the read channel <i>channel</i>. After calling this function, <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> will no longer receive data on the channel. Any data that has already been received is still available for reading.</p>
    /// <p>Call this function to save memory, if you are not interested in the output of the process.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#closeWriteChannel">closeWriteChannel</a>() and <a href="http://doc.qt.io/qt-5/qprocess.html#setReadChannel">setReadChannel</a>().</p></div>
    pub unsafe fn close_read_channel(&mut self, channel: crate::q_process::ProcessChannel) {
        crate::__ffi::ctr_qt_core_ffi_QProcess_closeReadChannel(
            self as *mut crate::QProcess,
            channel,
        )
    }

    /// <p>Schedules the write channel of <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> to be closed. The channel will close once all data has been written to the process. After calling this function, any attempts to write to the process will fail.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcess::closeWriteChannel()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#closeWriteChannel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Schedules the write channel of <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> to be closed. The channel will close once all data has been written to the process. After calling this function, any attempts to write to the process will fail.</p>
    /// <p>Closing the write channel is necessary for programs that read input data until the channel has been closed. For example, the program "more" is used to display text data in a console on both Unix and Windows. But it will not display the text data until <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>'s write channel has been closed. Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span> more;
    ///   more<span class="operator">.</span>start(<span class="string">"more"</span>);
    ///   more<span class="operator">.</span>write(<span class="string">"Text to display"</span>);
    ///   more<span class="operator">.</span>closeWriteChannel();
    ///   <span class="comment">// QProcess will emit readyRead() once "more" starts printing</span>
    ///
    /// </pre>
    /// <p>The write channel is implicitly opened when <a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>() is called.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#closeReadChannel">closeReadChannel</a>().</p></div>
    pub unsafe fn close_write_channel(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QProcess_closeWriteChannel(self as *mut crate::QProcess)
    }

    /// <p>Returns the environment that <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> will pass to its child process, or an empty <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> if no environment has been set using <a href="http://doc.qt.io/qt-5/qprocess-obsolete.html#setEnvironment">setEnvironment</a>(). If no environment has been set, the environment of the calling process will be used.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QProcess::environment() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess-obsolete.html#environment">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the environment that <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> will pass to its child process, or an empty <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> if no environment has been set using <a href="http://doc.qt.io/qt-5/qprocess-obsolete.html#setEnvironment">setEnvironment</a>(). If no environment has been set, the environment of the calling process will be used.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#processEnvironment">processEnvironment</a>(), <a href="http://doc.qt.io/qt-5/qprocess-obsolete.html#setEnvironment">setEnvironment</a>(), and <a href="http://doc.qt.io/qt-5/qprocess.html#systemEnvironment">systemEnvironment</a>().</p></div>
    pub unsafe fn environment(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QProcess_environment(self as *const crate::QProcess);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Use <a href="http://doc.qt.io/qt-5/qprocess.html#errorOccurred">errorOccurred</a>() instead.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QProcess::ProcessError QProcess::error() const```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qprocess-obsolete.html#error-1">C++ documentation</a> for <span style='color: green;'>``` void QProcess::error(QProcess::ProcessError error)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Use <a href="http://doc.qt.io/qt-5/qprocess.html#errorOccurred">errorOccurred</a>() instead.</p>
    /// <p><b>Note:</b> Signal <i>error</i> is overloaded in this class. To connect to this one using the function pointer syntax, you must specify the signal type in a static cast, as shown in this example:</p><pre class="cpp">
    ///
    ///   connect(process<span class="operator">,</span> <span class="keyword">static_cast</span><span class="operator">&lt;</span><span class="type">void</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span><span class="operator">::</span><span class="operator">*</span>)(<span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span><span class="operator">::</span>ProcessError)<span class="operator">&gt;</span>(<span class="operator">&amp;</span><span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span><span class="operator">::</span>error)<span class="operator">,</span>
    /// &#32;     <span class="operator">[</span><span class="operator">=</span><span class="operator">]</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span><span class="operator">::</span>ProcessError error){ <span class="comment">/* ... */</span> });
    ///
    /// </pre></div>
    pub unsafe fn error2(&self) -> crate::q_process::ProcessError {
        crate::__ffi::ctr_qt_core_ffi_QProcess_error(self as *const crate::QProcess)
    }

    /// <p>Starts the program <i>program</i> with the arguments <i>arguments</i> in a new process, waits for it to finish, and then returns the exit code of the process. Any data the new process writes to the console is forwarded to the calling process.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QProcess::execute(const QString& program, const QStringList& arguments)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#execute">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Starts the program <i>program</i> with the arguments <i>arguments</i> in a new process, waits for it to finish, and then returns the exit code of the process. Any data the new process writes to the console is forwarded to the calling process.</p>
    /// <p>The environment and working directory are inherited from the calling process.</p>
    /// <p>Argument handling is identical to the respective <a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>() overload.</p>
    /// <p>If the process cannot be started, -2 is returned. If the process crashes, -1 is returned. Otherwise, the process' exit code is returned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>().</p></div>
    pub unsafe fn execute_2a(
        program: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        arguments: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QProcess_execute(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(program)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(arguments)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QProcess::execute(const QString& command)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#execute-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Starts the program <i>command</i> in a new process, waits for it to finish, and then returns the exit code.</p>
    /// <p>Argument handling is identical to the respective <a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>() overload.</p>
    /// <p>After the <i>command</i> string has been split and unquoted, this function behaves like the overload which takes the arguments as a string list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>().</p></div>
    pub unsafe fn execute_1a(
        command: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QProcess_execute1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(command)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the exit code of the last process that finished.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QProcess::exitCode() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#exitCode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the exit code of the last process that finished.</p>
    /// <p>This value is not valid unless <a href="http://doc.qt.io/qt-5/qprocess.html#exitStatus">exitStatus</a>() returns <a href="http://doc.qt.io/qt-5/qprocess.html#ExitStatus-enum">NormalExit</a>.</p></div>
    pub unsafe fn exit_code(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QProcess_exitCode(self as *const crate::QProcess)
    }

    /// <p>Returns the exit status of the last process that finished.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QProcess::ExitStatus QProcess::exitStatus() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#exitStatus">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the exit status of the last process that finished.</p>
    /// <p>On Windows, if the process was terminated with TerminateProcess() from another application, this function will still return <a href="http://doc.qt.io/qt-5/qprocess.html#ExitStatus-enum">NormalExit</a> unless the exit code is less than 0.</p>
    /// <p>This function was introduced in  Qt 4.1.</p></div>
    pub unsafe fn exit_status(&self) -> crate::q_process::ExitStatus {
        crate::__ffi::ctr_qt_core_ffi_QProcess_exitStatus(self as *const crate::QProcess)
    }

    /// <p>Returns the channel mode of the <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> standard input channel.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QProcess::InputChannelMode QProcess::inputChannelMode() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#inputChannelMode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the channel mode of the <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> standard input channel.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#setInputChannelMode">setInputChannelMode</a>() and <a href="http://doc.qt.io/qt-5/qprocess.html#InputChannelMode-enum">InputChannelMode</a>.</p></div>
    pub unsafe fn input_channel_mode(&self) -> crate::q_process::InputChannelMode {
        crate::__ffi::ctr_qt_core_ffi_QProcess_inputChannelMode(self as *const crate::QProcess)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#isSequential">QIODevice::isSequential</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QProcess::isSequential() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#isSequential">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#isSequential">QIODevice::isSequential</a>().</p></div>
    pub unsafe fn is_sequential(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QProcess_isSequential(self as *const crate::QProcess)
    }

    /// <p>Kills the current process, causing it to exit immediately.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QProcess::kill()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#kill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Kills the current process, causing it to exit immediately.</p>
    /// <p>On Windows, kill() uses TerminateProcess, and on Unix and <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, the SIGKILL signal is sent to the process.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#terminate">terminate</a>().</p></div>
    pub unsafe fn kill(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QProcess_kill(self as *mut crate::QProcess)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QProcess::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QProcess_metaObject(self as *const crate::QProcess);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> object with the given <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QProcess::QProcess(QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> object with the given <i>parent</i>.</p></div>
    pub unsafe fn new_1a(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QProcess> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QProcess_QProcess(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> class is used to start external programs and to communicate with them.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QProcess::QProcess()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> class is used to start external programs and to communicate with them.</p>
    /// <a name="running-a-process"></a></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QProcess> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QProcess_QProcess1();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The null device of the operating system.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QProcess::nullDevice()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#nullDevice">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The null device of the operating system.</p>
    /// <p>The returned file path uses native directory separators.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#setStandardInputFile">QProcess::setStandardInputFile</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#setStandardOutputFile">QProcess::setStandardOutputFile</a>(), and <a href="http://doc.qt.io/qt-5/qprocess.html#setStandardErrorFile">QProcess::setStandardErrorFile</a>().</p></div>
    pub unsafe fn null_device() -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QProcess_nullDevice();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#open">QIODevice::open</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QProcess::open(QFlags<QIODevice::OpenModeFlag> mode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#open">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#open">QIODevice::open</a>().</p>
    /// <p>Starts the program set by <a href="http://doc.qt.io/qt-5/qprocess.html#setProgram">setProgram</a>() with arguments set by <a href="http://doc.qt.io/qt-5/qprocess.html#setArguments">setArguments</a>(). The <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">OpenMode</a> is set to <i>mode</i>.</p>
    /// <p>This method is an alias for <a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>(), and exists only to fully implement the interface defined by <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#setProgram">setProgram</a>(), and <a href="http://doc.qt.io/qt-5/qprocess.html#setArguments">setArguments</a>().</p></div>
    pub unsafe fn open_1a(
        &mut self,
        mode: crate::QFlags<crate::q_io_device::OpenModeFlag>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QProcess_open(self as *mut crate::QProcess, mode.to_int())
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#open">QIODevice::open</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QProcess::open()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#open">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#open">QIODevice::open</a>().</p>
    /// <p>Starts the program set by <a href="http://doc.qt.io/qt-5/qprocess.html#setProgram">setProgram</a>() with arguments set by <a href="http://doc.qt.io/qt-5/qprocess.html#setArguments">setArguments</a>(). The <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">OpenMode</a> is set to <i>mode</i>.</p>
    /// <p>This method is an alias for <a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>(), and exists only to fully implement the interface defined by <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#setProgram">setProgram</a>(), and <a href="http://doc.qt.io/qt-5/qprocess.html#setArguments">setArguments</a>().</p></div>
    pub unsafe fn open_0a(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QProcess_open1(self as *mut crate::QProcess)
    }

    /// <p>Returns the channel mode of the <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> standard output and standard error channels.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QProcess::ProcessChannelMode QProcess::processChannelMode() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#processChannelMode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the channel mode of the <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> standard output and standard error channels.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#setProcessChannelMode">setProcessChannelMode</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#ProcessChannelMode-enum">ProcessChannelMode</a>, and <a href="http://doc.qt.io/qt-5/qprocess.html#setReadChannel">setReadChannel</a>().</p></div>
    pub unsafe fn process_channel_mode(&self) -> crate::q_process::ProcessChannelMode {
        crate::__ffi::ctr_qt_core_ffi_QProcess_processChannelMode(self as *const crate::QProcess)
    }

    /// <p>Returns the environment that <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> will pass to its child process, or an empty object if no environment has been set using setEnvironment() or <a href="http://doc.qt.io/qt-5/qprocess.html#setProcessEnvironment">setProcessEnvironment</a>(). If no environment has been set, the environment of the calling process will be used.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QProcessEnvironment QProcess::processEnvironment() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#processEnvironment">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the environment that <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> will pass to its child process, or an empty object if no environment has been set using setEnvironment() or <a href="http://doc.qt.io/qt-5/qprocess.html#setProcessEnvironment">setProcessEnvironment</a>(). If no environment has been set, the environment of the calling process will be used.</p>
    /// <p>This function was introduced in  Qt 4.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#setProcessEnvironment">setProcessEnvironment</a>(), <a href="http://doc.qt.io/qt-5/qprocess-obsolete.html#setEnvironment">setEnvironment</a>(), and <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#isEmpty">QProcessEnvironment::isEmpty</a>().</p></div>
    pub unsafe fn process_environment(&self) -> ::cpp_utils::CppBox<crate::QProcessEnvironment> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QProcess_processEnvironment(
            self as *const crate::QProcess,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the native process identifier for the running process, if available. If no process is currently running, <code>0</code> is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QProcess::processId() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#processId">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the native process identifier for the running process, if available. If no process is currently running, <code>0</code> is returned.</p>
    /// <p>This function was introduced in  Qt 5.3.</p></div>
    pub unsafe fn process_id(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QProcess_processId(self as *const crate::QProcess)
    }

    /// <p>Returns the program the process was last started with.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QProcess::program() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#program">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the program the process was last started with.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#setProgram">setProgram</a>() and <a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>().</p></div>
    pub unsafe fn program(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QProcess_program(self as *const crate::QProcess);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QProcess::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QProcess_qt_metacall(
            self as *mut crate::QProcess,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QProcess::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QProcess_qt_metacast(
            self as *mut crate::QProcess,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Regardless of the current read channel, this function returns all data available from the standard error of the process as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QProcess::readAllStandardError()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#readAllStandardError">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Regardless of the current read channel, this function returns all data available from the standard error of the process as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#readyReadStandardError">readyReadStandardError</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#readAllStandardOutput">readAllStandardOutput</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#readChannel">readChannel</a>(), and <a href="http://doc.qt.io/qt-5/qprocess.html#setReadChannel">setReadChannel</a>().</p></div>
    pub unsafe fn read_all_standard_error(&mut self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QProcess_readAllStandardError(
            self as *mut crate::QProcess,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Regardless of the current read channel, this function returns all data available from the standard output of the process as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QProcess::readAllStandardOutput()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#readAllStandardOutput">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Regardless of the current read channel, this function returns all data available from the standard output of the process as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#readyReadStandardOutput">readyReadStandardOutput</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#readAllStandardError">readAllStandardError</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#readChannel">readChannel</a>(), and <a href="http://doc.qt.io/qt-5/qprocess.html#setReadChannel">setReadChannel</a>().</p></div>
    pub unsafe fn read_all_standard_output(&mut self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QProcess_readAllStandardOutput(
            self as *mut crate::QProcess,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the current read channel of the <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QProcess::ProcessChannel QProcess::readChannel() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#readChannel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current read channel of the <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#setReadChannel">setReadChannel</a>().</p></div>
    pub unsafe fn read_channel(&self) -> crate::q_process::ProcessChannel {
        crate::__ffi::ctr_qt_core_ffi_QProcess_readChannel(self as *const crate::QProcess)
    }

    /// <p>Returns the read channel mode of the <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>. This function is equivalent to <a href="http://doc.qt.io/qt-5/qprocess.html#processChannelMode">processChannelMode</a>()</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QProcess::ProcessChannelMode QProcess::readChannelMode() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess-obsolete.html#readChannelMode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the read channel mode of the <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>. This function is equivalent to <a href="http://doc.qt.io/qt-5/qprocess.html#processChannelMode">processChannelMode</a>()</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess-obsolete.html#setReadChannelMode">setReadChannelMode</a>() and <a href="http://doc.qt.io/qt-5/qprocess.html#processChannelMode">processChannelMode</a>().</p></div>
    pub unsafe fn read_channel_mode(&self) -> crate::q_process::ProcessChannelMode {
        crate::__ffi::ctr_qt_core_ffi_QProcess_readChannelMode(self as *const crate::QProcess)
    }

    /// <p>Set the <i>arguments</i> to pass to the called program when starting the process. This function must be called before <a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcess::setArguments(const QStringList& arguments)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#setArguments">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Set the <i>arguments</i> to pass to the called program when starting the process. This function must be called before <a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>().</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#setProgram">setProgram</a>(), and <a href="http://doc.qt.io/qt-5/qprocess.html#arguments">arguments</a>().</p></div>
    pub unsafe fn set_arguments(
        &mut self,
        arguments: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QProcess_setArguments(
            self as *mut crate::QProcess,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(arguments)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the environment that <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> will pass to the child process. The parameter <i>environment</i> is a list of key=value pairs.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcess::setEnvironment(const QStringList& environment)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess-obsolete.html#setEnvironment">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the environment that <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> will pass to the child process. The parameter <i>environment</i> is a list of key=value pairs.</p>
    /// <p>For example, the following code adds the environment variable <code>TMPDIR</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span> process;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> env <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span><span class="operator">::</span>systemEnvironment();
    ///   env <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"TMPDIR=C:\\MyApp\\temp"</span>; <span class="comment">// Add an environment variable</span>
    ///   process<span class="operator">.</span>setEnvironment(env);
    ///   process<span class="operator">.</span>start(<span class="string">"myapp"</span>);
    ///
    /// </pre>
    /// <p><b>Note: </b>This function is less efficient than the <a href="http://doc.qt.io/qt-5/qprocess.html#setProcessEnvironment">setProcessEnvironment</a>() function.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess-obsolete.html#environment">environment</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#setProcessEnvironment">setProcessEnvironment</a>(), and <a href="http://doc.qt.io/qt-5/qprocess.html#systemEnvironment">systemEnvironment</a>().</p></div>
    pub unsafe fn set_environment(
        &mut self,
        environment: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QProcess_setEnvironment(
            self as *mut crate::QProcess,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(environment)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the channel mode of the <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> standard input channel to the <i>mode</i> specified. This mode will be used the next time <a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>() is called.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcess::setInputChannelMode(QProcess::InputChannelMode mode)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#setInputChannelMode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the channel mode of the <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> standard input channel to the <i>mode</i> specified. This mode will be used the next time <a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>() is called.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#inputChannelMode">inputChannelMode</a>() and <a href="http://doc.qt.io/qt-5/qprocess.html#InputChannelMode-enum">InputChannelMode</a>.</p></div>
    pub unsafe fn set_input_channel_mode(&mut self, mode: crate::q_process::InputChannelMode) {
        crate::__ffi::ctr_qt_core_ffi_QProcess_setInputChannelMode(
            self as *mut crate::QProcess,
            mode,
        )
    }

    /// <p>Sets the channel mode of the <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> standard output and standard error channels to the <i>mode</i> specified. This mode will be used the next time <a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>() is called. For example:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcess::setProcessChannelMode(QProcess::ProcessChannelMode mode)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#setProcessChannelMode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the channel mode of the <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> standard output and standard error channels to the <i>mode</i> specified. This mode will be used the next time <a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>() is called. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span> builder;
    ///   builder<span class="operator">.</span>setProcessChannelMode(<span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span><span class="operator">::</span>MergedChannels);
    ///   builder<span class="operator">.</span>start(<span class="string">"make"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"-j2"</span>);
    ///
    ///   <span class="keyword">if</span> (<span class="operator">!</span>builder<span class="operator">.</span>waitForFinished())
    /// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Make failed:"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> builder<span class="operator">.</span>errorString();
    ///   <span class="keyword">else</span>
    /// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Make output:"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> builder<span class="operator">.</span>readAll();
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#processChannelMode">processChannelMode</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#ProcessChannelMode-enum">ProcessChannelMode</a>, and <a href="http://doc.qt.io/qt-5/qprocess.html#setReadChannel">setReadChannel</a>().</p></div>
    pub unsafe fn set_process_channel_mode(&mut self, mode: crate::q_process::ProcessChannelMode) {
        crate::__ffi::ctr_qt_core_ffi_QProcess_setProcessChannelMode(
            self as *mut crate::QProcess,
            mode,
        )
    }

    /// <p>Sets the <i>environment</i> that <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> will pass to the child process.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcess::setProcessEnvironment(const QProcessEnvironment& environment)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#setProcessEnvironment">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the <i>environment</i> that <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> will pass to the child process.</p>
    /// <p>For example, the following code adds the environment variable <code>TMPDIR</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span> process;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a></span> env <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a></span><span class="operator">::</span>systemEnvironment();
    ///   env<span class="operator">.</span>insert(<span class="string">"TMPDIR"</span><span class="operator">,</span> <span class="string">"C:\\MyApp\\temp"</span>); <span class="comment">// Add an environment variable</span>
    ///   process<span class="operator">.</span>setProcessEnvironment(env);
    ///   process<span class="operator">.</span>start(<span class="string">"myapp"</span>);
    ///
    /// </pre>
    /// <p>Note how, on Windows, environment variable names are case-insensitive.</p>
    /// <p>This function was introduced in  Qt 4.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#processEnvironment">processEnvironment</a>(), <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#systemEnvironment">QProcessEnvironment::systemEnvironment</a>(), and <a href="http://doc.qt.io/qt-5/qprocess-obsolete.html#setEnvironment">setEnvironment</a>().</p></div>
    pub unsafe fn set_process_environment(
        &mut self,
        environment: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QProcessEnvironment>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QProcess_setProcessEnvironment(
            self as *mut crate::QProcess,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QProcessEnvironment>>::cast_into(
                environment,
            )
            .as_raw_ptr(),
        )
    }

    /// <p>Set the <i>program</i> to use when starting the process. This function must be called before <a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcess::setProgram(const QString& program)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#setProgram">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Set the <i>program</i> to use when starting the process. This function must be called before <a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>().</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#setArguments">setArguments</a>(), and <a href="http://doc.qt.io/qt-5/qprocess.html#program">program</a>().</p></div>
    pub unsafe fn set_program(
        &mut self,
        program: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QProcess_setProgram(
            self as *mut crate::QProcess,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(program)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the current read channel of the <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> to the given <i>channel</i>. The current input channel is used by the functions <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#readAll">readAll</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#readLine">readLine</a>(), and <a href="http://doc.qt.io/qt-5/qiodevice.html#getChar">getChar</a>(). It also determines which channel triggers <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> to emit <a href="http://doc.qt.io/qt-5/qiodevice.html#readyRead">readyRead</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcess::setReadChannel(QProcess::ProcessChannel channel)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#setReadChannel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the current read channel of the <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> to the given <i>channel</i>. The current input channel is used by the functions <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#readAll">readAll</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#readLine">readLine</a>(), and <a href="http://doc.qt.io/qt-5/qiodevice.html#getChar">getChar</a>(). It also determines which channel triggers <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> to emit <a href="http://doc.qt.io/qt-5/qiodevice.html#readyRead">readyRead</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#readChannel">readChannel</a>().</p></div>
    pub unsafe fn set_read_channel(&mut self, channel: crate::q_process::ProcessChannel) {
        crate::__ffi::ctr_qt_core_ffi_QProcess_setReadChannel(self as *mut crate::QProcess, channel)
    }

    /// <p>Use <a href="http://doc.qt.io/qt-5/qprocess.html#setProcessChannelMode">setProcessChannelMode</a>(<i>mode</i>) instead.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcess::setReadChannelMode(QProcess::ProcessChannelMode mode)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess-obsolete.html#setReadChannelMode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Use <a href="http://doc.qt.io/qt-5/qprocess.html#setProcessChannelMode">setProcessChannelMode</a>(<i>mode</i>) instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess-obsolete.html#readChannelMode">readChannelMode</a>() and <a href="http://doc.qt.io/qt-5/qprocess.html#setProcessChannelMode">setProcessChannelMode</a>().</p></div>
    pub unsafe fn set_read_channel_mode(&mut self, mode: crate::q_process::ProcessChannelMode) {
        crate::__ffi::ctr_qt_core_ffi_QProcess_setReadChannelMode(
            self as *mut crate::QProcess,
            mode,
        )
    }

    /// <p>Redirects the process' standard error to the file <i>fileName</i>. When the redirection is in place, the standard error read channel is closed: reading from it using <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>() will always fail, as will <a href="http://doc.qt.io/qt-5/qprocess.html#readAllStandardError">readAllStandardError</a>(). The file will be appended to if <i>mode</i> is Append, otherwise, it will be truncated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcess::setStandardErrorFile(const QString& fileName, QFlags<QIODevice::OpenModeFlag> mode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#setStandardErrorFile">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Redirects the process' standard error to the file <i>fileName</i>. When the redirection is in place, the standard error read channel is closed: reading from it using <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>() will always fail, as will <a href="http://doc.qt.io/qt-5/qprocess.html#readAllStandardError">readAllStandardError</a>(). The file will be appended to if <i>mode</i> is Append, otherwise, it will be truncated.</p>
    /// <p>See <a href="http://doc.qt.io/qt-5/qprocess.html#setStandardOutputFile">setStandardOutputFile</a>() for more information on how the file is opened.</p>
    /// <p>Note: if <a href="http://doc.qt.io/qt-5/qprocess.html#setProcessChannelMode">setProcessChannelMode</a>() was called with an argument of <a href="http://doc.qt.io/qt-5/qprocess.html#ProcessChannelMode-enum">QProcess::MergedChannels</a>, this function has no effect.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#setStandardInputFile">setStandardInputFile</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#setStandardOutputFile">setStandardOutputFile</a>(), and <a href="http://doc.qt.io/qt-5/qprocess.html#setStandardOutputProcess">setStandardOutputProcess</a>().</p></div>
    pub unsafe fn set_standard_error_file_2a(
        &mut self,
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        mode: crate::QFlags<crate::q_io_device::OpenModeFlag>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QProcess_setStandardErrorFile(
            self as *mut crate::QProcess,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
            mode.to_int(),
        )
    }

    /// <p>Redirects the process' standard error to the file <i>fileName</i>. When the redirection is in place, the standard error read channel is closed: reading from it using <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>() will always fail, as will <a href="http://doc.qt.io/qt-5/qprocess.html#readAllStandardError">readAllStandardError</a>(). The file will be appended to if <i>mode</i> is Append, otherwise, it will be truncated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcess::setStandardErrorFile(const QString& fileName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#setStandardErrorFile">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Redirects the process' standard error to the file <i>fileName</i>. When the redirection is in place, the standard error read channel is closed: reading from it using <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>() will always fail, as will <a href="http://doc.qt.io/qt-5/qprocess.html#readAllStandardError">readAllStandardError</a>(). The file will be appended to if <i>mode</i> is Append, otherwise, it will be truncated.</p>
    /// <p>See <a href="http://doc.qt.io/qt-5/qprocess.html#setStandardOutputFile">setStandardOutputFile</a>() for more information on how the file is opened.</p>
    /// <p>Note: if <a href="http://doc.qt.io/qt-5/qprocess.html#setProcessChannelMode">setProcessChannelMode</a>() was called with an argument of <a href="http://doc.qt.io/qt-5/qprocess.html#ProcessChannelMode-enum">QProcess::MergedChannels</a>, this function has no effect.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#setStandardInputFile">setStandardInputFile</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#setStandardOutputFile">setStandardOutputFile</a>(), and <a href="http://doc.qt.io/qt-5/qprocess.html#setStandardOutputProcess">setStandardOutputProcess</a>().</p></div>
    pub unsafe fn set_standard_error_file_1a(
        &mut self,
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QProcess_setStandardErrorFile1(
            self as *mut crate::QProcess,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        )
    }

    /// <p>Redirects the process' standard input to the file indicated by <i>fileName</i>. When an input redirection is in place, the <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> object will be in read-only mode (calling <a href="http://doc.qt.io/qt-5/qiodevice.html#write">write</a>() will result in error).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcess::setStandardInputFile(const QString& fileName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#setStandardInputFile">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Redirects the process' standard input to the file indicated by <i>fileName</i>. When an input redirection is in place, the <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> object will be in read-only mode (calling <a href="http://doc.qt.io/qt-5/qiodevice.html#write">write</a>() will result in error).</p>
    /// <p>To make the process read EOF right away, pass <a href="http://doc.qt.io/qt-5/qprocess.html#nullDevice">nullDevice</a>() here. This is cleaner than using <a href="http://doc.qt.io/qt-5/qprocess.html#closeWriteChannel">closeWriteChannel</a>() before writing any data, because it can be set up prior to starting the process.</p>
    /// <p>If the file <i>fileName</i> does not exist at the moment <a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>() is called or is not readable, starting the process will fail.</p>
    /// <p>Calling setStandardInputFile() after the process has started has no effect.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#setStandardOutputFile">setStandardOutputFile</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#setStandardErrorFile">setStandardErrorFile</a>(), and <a href="http://doc.qt.io/qt-5/qprocess.html#setStandardOutputProcess">setStandardOutputProcess</a>().</p></div>
    pub unsafe fn set_standard_input_file(
        &mut self,
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QProcess_setStandardInputFile(
            self as *mut crate::QProcess,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        )
    }

    /// <p>Redirects the process' standard output to the file <i>fileName</i>. When the redirection is in place, the standard output read channel is closed: reading from it using <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>() will always fail, as will <a href="http://doc.qt.io/qt-5/qprocess.html#readAllStandardOutput">readAllStandardOutput</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcess::setStandardOutputFile(const QString& fileName, QFlags<QIODevice::OpenModeFlag> mode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#setStandardOutputFile">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Redirects the process' standard output to the file <i>fileName</i>. When the redirection is in place, the standard output read channel is closed: reading from it using <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>() will always fail, as will <a href="http://doc.qt.io/qt-5/qprocess.html#readAllStandardOutput">readAllStandardOutput</a>().</p>
    /// <p>To discard all standard output from the process, pass <a href="http://doc.qt.io/qt-5/qprocess.html#nullDevice">nullDevice</a>() here. This is more efficient than simply never reading the standard output, as no <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> buffers are filled.</p>
    /// <p>If the file <i>fileName</i> doesn't exist at the moment <a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>() is called, it will be created. If it cannot be created, the starting will fail.</p>
    /// <p>If the file exists and <i>mode</i> is <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">QIODevice::Truncate</a>, the file will be truncated. Otherwise (if <i>mode</i> is <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">QIODevice::Append</a>), the file will be appended to.</p>
    /// <p>Calling setStandardOutputFile() after the process has started has no effect.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#setStandardInputFile">setStandardInputFile</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#setStandardErrorFile">setStandardErrorFile</a>(), and <a href="http://doc.qt.io/qt-5/qprocess.html#setStandardOutputProcess">setStandardOutputProcess</a>().</p></div>
    pub unsafe fn set_standard_output_file_2a(
        &mut self,
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        mode: crate::QFlags<crate::q_io_device::OpenModeFlag>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QProcess_setStandardOutputFile(
            self as *mut crate::QProcess,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
            mode.to_int(),
        )
    }

    /// <p>Redirects the process' standard output to the file <i>fileName</i>. When the redirection is in place, the standard output read channel is closed: reading from it using <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>() will always fail, as will <a href="http://doc.qt.io/qt-5/qprocess.html#readAllStandardOutput">readAllStandardOutput</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcess::setStandardOutputFile(const QString& fileName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#setStandardOutputFile">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Redirects the process' standard output to the file <i>fileName</i>. When the redirection is in place, the standard output read channel is closed: reading from it using <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>() will always fail, as will <a href="http://doc.qt.io/qt-5/qprocess.html#readAllStandardOutput">readAllStandardOutput</a>().</p>
    /// <p>To discard all standard output from the process, pass <a href="http://doc.qt.io/qt-5/qprocess.html#nullDevice">nullDevice</a>() here. This is more efficient than simply never reading the standard output, as no <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> buffers are filled.</p>
    /// <p>If the file <i>fileName</i> doesn't exist at the moment <a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>() is called, it will be created. If it cannot be created, the starting will fail.</p>
    /// <p>If the file exists and <i>mode</i> is <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">QIODevice::Truncate</a>, the file will be truncated. Otherwise (if <i>mode</i> is <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">QIODevice::Append</a>), the file will be appended to.</p>
    /// <p>Calling setStandardOutputFile() after the process has started has no effect.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#setStandardInputFile">setStandardInputFile</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#setStandardErrorFile">setStandardErrorFile</a>(), and <a href="http://doc.qt.io/qt-5/qprocess.html#setStandardOutputProcess">setStandardOutputProcess</a>().</p></div>
    pub unsafe fn set_standard_output_file_1a(
        &mut self,
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QProcess_setStandardOutputFile1(
            self as *mut crate::QProcess,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        )
    }

    /// <p>Pipes the standard output stream of this process to the <i>destination</i> process' standard input.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcess::setStandardOutputProcess(QProcess* destination)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#setStandardOutputProcess">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Pipes the standard output stream of this process to the <i>destination</i> process' standard input.</p>
    /// <p>The following shell command:</p>
    /// <pre class="cpp">
    ///
    ///   command1 <span class="operator">|</span> command2
    ///
    /// </pre>
    /// <p>Can be accomplished with <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> with the following code:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span> process1;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span> process2;
    ///
    ///   process1<span class="operator">.</span>setStandardOutputProcess(<span class="operator">&amp;</span>process2);
    ///
    ///   process1<span class="operator">.</span>start(<span class="string">"command1"</span>);
    ///   process2<span class="operator">.</span>start(<span class="string">"command2"</span>);
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 4.2.</p></div>
    pub unsafe fn set_standard_output_process(
        &mut self,
        destination: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QProcess>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QProcess_setStandardOutputProcess(
            self as *mut crate::QProcess,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QProcess>>::cast_into(destination)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Sets the working directory to <i>dir</i>. <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> will start the process in this directory. The default behavior is to start the process in the working directory of the calling process.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcess::setWorkingDirectory(const QString& dir)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#setWorkingDirectory">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the working directory to <i>dir</i>. <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> will start the process in this directory. The default behavior is to start the process in the working directory of the calling process.</p>
    /// <p><b>Note: </b>On QNX, this may cause all application threads to temporarily freeze.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#workingDirectory">workingDirectory</a>() and <a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>().</p></div>
    pub unsafe fn set_working_directory(
        &mut self,
        dir: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QProcess_setWorkingDirectory(
            self as *mut crate::QProcess,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(dir).as_raw_ptr(),
        )
    }

    /// <p>Starts the given <i>program</i> in a new process, passing the command line arguments in <i>arguments</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcess::start(const QString& program, const QStringList& arguments, QFlags<QIODevice::OpenModeFlag> mode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#start">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Starts the given <i>program</i> in a new process, passing the command line arguments in <i>arguments</i>.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> object will immediately enter the Starting state. If the process starts successfully, <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> will emit <a href="http://doc.qt.io/qt-5/qprocess.html#started">started</a>(); otherwise, <a href="http://doc.qt.io/qt-5/qprocess.html#errorOccurred">errorOccurred</a>() will be emitted.</p>
    /// <p><b>Note: </b>Processes are started asynchronously, which means the <a href="http://doc.qt.io/qt-5/qprocess.html#started">started</a>() and <a href="http://doc.qt.io/qt-5/qprocess.html#errorOccurred">errorOccurred</a>() signals may be delayed. Call <a href="http://doc.qt.io/qt-5/qprocess.html#waitForStarted">waitForStarted</a>() to make sure the process has started (or has failed to start) and those signals have been emitted.</p><p><b>Note: </b>No further splitting of the arguments is performed.</p><p><b>Windows:</b> The arguments are quoted and joined into a command line that is compatible with the <code>CommandLineToArgvW()</code> Windows function. For programs that have different command line quoting requirements, you need to use <a href="http://doc.qt.io/qt-5/qprocess.html#setNativeArguments">setNativeArguments</a>(). One notable program that does not follow the <code>CommandLineToArgvW()</code> rules is cmd.exe and, by consequence, all batch scripts.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">OpenMode</a> is set to <i>mode</i>.</p>
    /// <p>If the <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> object is already running a process, a warning may be printed at the console, and the existing process will continue running unaffected.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#processId">processId</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#started">started</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#waitForStarted">waitForStarted</a>(), and <a href="http://doc.qt.io/qt-5/qprocess.html#setNativeArguments">setNativeArguments</a>().</p></div>
    pub unsafe fn start_q_string_q_string_list_q_flags_open_mode_flag(
        &mut self,
        program: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        arguments: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
        mode: crate::QFlags<crate::q_io_device::OpenModeFlag>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QProcess_start(
            self as *mut crate::QProcess,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(program)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(arguments)
                .as_raw_ptr(),
            mode.to_int(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcess::start(const QString& command, QFlags<QIODevice::OpenModeFlag> mode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#start-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Starts the command <i>command</i> in a new process. The <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">OpenMode</a> is set to <i>mode</i>.</p>
    /// <p><i>command</i> is a single string of text containing both the program name and its arguments. The arguments are separated by one or more spaces. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span> process;
    ///   process<span class="operator">.</span>start(<span class="string">"del /s *.txt"</span>);
    ///   <span class="comment">// same as process.start("del", QStringList() &lt;&lt; "/s" &lt;&lt; "*.txt");</span>
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///
    /// </pre>
    /// <p>Arguments containing spaces must be quoted to be correctly supplied to the new process. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span> process;
    ///   process<span class="operator">.</span>start(<span class="string">"dir \"My Documents\""</span>);
    ///
    /// </pre>
    /// <p>Literal quotes in the <i>command</i> string are represented by triple quotes. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span> process;
    ///   process<span class="operator">.</span>start(<span class="string">"dir \"Epic 12\"\"\" Singles\""</span>);
    ///
    /// </pre>
    /// <p>After the <i>command</i> string has been split and unquoted, this function behaves like the overload which takes the arguments as a string list.</p>
    /// <p>You can disable this overload by defining <code>QT_NO_PROCESS_COMBINED_ARGUMENT_START</code> when you compile your applications. This can be useful if you want to ensure that you are not splitting arguments unintentionally, for example. In virtually all cases, using the other overload is the preferred method.</p>
    /// <p>On operating systems where the system API for passing command line arguments to a subprocess natively uses a single string (Windows), one can conceive command lines which cannot be passed via <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>'s portable list-based API. In these rare cases you need to use <a href="http://doc.qt.io/qt-5/qprocess.html#setProgram">setProgram</a>() and <a href="http://doc.qt.io/qt-5/qprocess.html#setNativeArguments">setNativeArguments</a>() instead of this function.</p></div>
    pub unsafe fn start_q_string_q_flags_open_mode_flag(
        &mut self,
        command: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        mode: crate::QFlags<crate::q_io_device::OpenModeFlag>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QProcess_start1(
            self as *mut crate::QProcess,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(command)
                .as_raw_ptr(),
            mode.to_int(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcess::start(QFlags<QIODevice::OpenModeFlag> mode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#start-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Starts the program set by <a href="http://doc.qt.io/qt-5/qprocess.html#setProgram">setProgram</a>() with arguments set by <a href="http://doc.qt.io/qt-5/qprocess.html#setArguments">setArguments</a>(). The <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">OpenMode</a> is set to <i>mode</i>.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#open">open</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#setProgram">setProgram</a>(), and <a href="http://doc.qt.io/qt-5/qprocess.html#setArguments">setArguments</a>().</p></div>
    pub unsafe fn start_q_flags_open_mode_flag(
        &mut self,
        mode: crate::QFlags<crate::q_io_device::OpenModeFlag>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QProcess_start2(self as *mut crate::QProcess, mode.to_int())
    }

    /// <p>Starts the given <i>program</i> in a new process, passing the command line arguments in <i>arguments</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcess::start(const QString& program, const QStringList& arguments)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#start">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Starts the given <i>program</i> in a new process, passing the command line arguments in <i>arguments</i>.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> object will immediately enter the Starting state. If the process starts successfully, <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> will emit <a href="http://doc.qt.io/qt-5/qprocess.html#started">started</a>(); otherwise, <a href="http://doc.qt.io/qt-5/qprocess.html#errorOccurred">errorOccurred</a>() will be emitted.</p>
    /// <p><b>Note: </b>Processes are started asynchronously, which means the <a href="http://doc.qt.io/qt-5/qprocess.html#started">started</a>() and <a href="http://doc.qt.io/qt-5/qprocess.html#errorOccurred">errorOccurred</a>() signals may be delayed. Call <a href="http://doc.qt.io/qt-5/qprocess.html#waitForStarted">waitForStarted</a>() to make sure the process has started (or has failed to start) and those signals have been emitted.</p><p><b>Note: </b>No further splitting of the arguments is performed.</p><p><b>Windows:</b> The arguments are quoted and joined into a command line that is compatible with the <code>CommandLineToArgvW()</code> Windows function. For programs that have different command line quoting requirements, you need to use <a href="http://doc.qt.io/qt-5/qprocess.html#setNativeArguments">setNativeArguments</a>(). One notable program that does not follow the <code>CommandLineToArgvW()</code> rules is cmd.exe and, by consequence, all batch scripts.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">OpenMode</a> is set to <i>mode</i>.</p>
    /// <p>If the <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> object is already running a process, a warning may be printed at the console, and the existing process will continue running unaffected.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#processId">processId</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#started">started</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#waitForStarted">waitForStarted</a>(), and <a href="http://doc.qt.io/qt-5/qprocess.html#setNativeArguments">setNativeArguments</a>().</p></div>
    pub unsafe fn start_q_string_q_string_list(
        &mut self,
        program: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        arguments: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QProcess_start3(
            self as *mut crate::QProcess,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(program)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(arguments)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcess::start(const QString& command)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#start-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Starts the command <i>command</i> in a new process. The <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">OpenMode</a> is set to <i>mode</i>.</p>
    /// <p><i>command</i> is a single string of text containing both the program name and its arguments. The arguments are separated by one or more spaces. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span> process;
    ///   process<span class="operator">.</span>start(<span class="string">"del /s *.txt"</span>);
    ///   <span class="comment">// same as process.start("del", QStringList() &lt;&lt; "/s" &lt;&lt; "*.txt");</span>
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///
    /// </pre>
    /// <p>Arguments containing spaces must be quoted to be correctly supplied to the new process. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span> process;
    ///   process<span class="operator">.</span>start(<span class="string">"dir \"My Documents\""</span>);
    ///
    /// </pre>
    /// <p>Literal quotes in the <i>command</i> string are represented by triple quotes. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span> process;
    ///   process<span class="operator">.</span>start(<span class="string">"dir \"Epic 12\"\"\" Singles\""</span>);
    ///
    /// </pre>
    /// <p>After the <i>command</i> string has been split and unquoted, this function behaves like the overload which takes the arguments as a string list.</p>
    /// <p>You can disable this overload by defining <code>QT_NO_PROCESS_COMBINED_ARGUMENT_START</code> when you compile your applications. This can be useful if you want to ensure that you are not splitting arguments unintentionally, for example. In virtually all cases, using the other overload is the preferred method.</p>
    /// <p>On operating systems where the system API for passing command line arguments to a subprocess natively uses a single string (Windows), one can conceive command lines which cannot be passed via <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a>'s portable list-based API. In these rare cases you need to use <a href="http://doc.qt.io/qt-5/qprocess.html#setProgram">setProgram</a>() and <a href="http://doc.qt.io/qt-5/qprocess.html#setNativeArguments">setNativeArguments</a>() instead of this function.</p></div>
    pub unsafe fn start_q_string(
        &mut self,
        command: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QProcess_start4(
            self as *mut crate::QProcess,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(command)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QProcess::start()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#start-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Starts the program set by <a href="http://doc.qt.io/qt-5/qprocess.html#setProgram">setProgram</a>() with arguments set by <a href="http://doc.qt.io/qt-5/qprocess.html#setArguments">setArguments</a>(). The <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">OpenMode</a> is set to <i>mode</i>.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#open">open</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#setProgram">setProgram</a>(), and <a href="http://doc.qt.io/qt-5/qprocess.html#setArguments">setArguments</a>().</p></div>
    pub unsafe fn start(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QProcess_start5(self as *mut crate::QProcess)
    }

    /// <p>Starts the program <i>program</i> with the arguments <i>arguments</i> in a new process, and detaches from it. Returns <code>true</code> on success; otherwise returns <code>false</code>. If the calling process exits, the detached process will continue to run unaffected.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QProcess::startDetached(const QString& program, const QStringList& arguments, const QString& workingDirectory, qint64* pid = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#startDetached">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Starts the program <i>program</i> with the arguments <i>arguments</i> in a new process, and detaches from it. Returns <code>true</code> on success; otherwise returns <code>false</code>. If the calling process exits, the detached process will continue to run unaffected.</p>
    /// <p>Argument handling is identical to the respective <a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>() overload.</p>
    /// <p><b>Unix:</b> The started process will run in its own session and act like a daemon.</p>
    /// <p>The process will be started in the directory <i>workingDirectory</i>. If <i>workingDirectory</i> is empty, the working directory is inherited from the calling process.</p>
    /// <p><b>Note: </b>On QNX, this may cause all application threads to temporarily freeze.</p><p>If the function is successful then *<i>pid</i> is set to the process identifier of the started process.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>().</p></div>
    pub unsafe fn start_detached_4a(
        program: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        arguments: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
        working_directory: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        pid: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<i64>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QProcess_startDetached(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(program)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(arguments)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(working_directory)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<i64>>::cast_into(pid).as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```static bool QProcess::startDetached(const QString& program, const QStringList& arguments)```</span>.
    pub unsafe fn start_detached_2a(
        program: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        arguments: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QProcess_startDetached1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(program)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(arguments)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QProcess::startDetached(const QString& command)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#startDetached-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Starts the command <i>command</i> in a new process, and detaches from it. Returns <code>true</code> on success; otherwise returns <code>false</code>.</p>
    /// <p>Argument handling is identical to the respective <a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>() overload.</p>
    /// <p>After the <i>command</i> string has been split and unquoted, this function behaves like the overload which takes the arguments as a string list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#start-1">start</a>(const QString &amp;command, OpenMode mode).</p></div>
    pub unsafe fn start_detached_1a(
        command: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QProcess_startDetached2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(command)
                .as_raw_ptr(),
        )
    }

    /// <p>Starts the program <i>program</i> with the arguments <i>arguments</i> in a new process, and detaches from it. Returns <code>true</code> on success; otherwise returns <code>false</code>. If the calling process exits, the detached process will continue to run unaffected.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QProcess::startDetached(const QString& program, const QStringList& arguments, const QString& workingDirectory)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#startDetached">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Starts the program <i>program</i> with the arguments <i>arguments</i> in a new process, and detaches from it. Returns <code>true</code> on success; otherwise returns <code>false</code>. If the calling process exits, the detached process will continue to run unaffected.</p>
    /// <p>Argument handling is identical to the respective <a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>() overload.</p>
    /// <p><b>Unix:</b> The started process will run in its own session and act like a daemon.</p>
    /// <p>The process will be started in the directory <i>workingDirectory</i>. If <i>workingDirectory</i> is empty, the working directory is inherited from the calling process.</p>
    /// <p><b>Note: </b>On QNX, this may cause all application threads to temporarily freeze.</p><p>If the function is successful then *<i>pid</i> is set to the process identifier of the started process.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>().</p></div>
    pub unsafe fn start_detached_3a(
        program: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        arguments: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
        working_directory: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QProcess_startDetached3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(program)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(arguments)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(working_directory)
                .as_raw_ptr(),
        )
    }

    /// <p>Starts the program set by <a href="http://doc.qt.io/qt-5/qprocess.html#setProgram">setProgram</a>() with arguments set by <a href="http://doc.qt.io/qt-5/qprocess.html#setArguments">setArguments</a>() in a new process, and detaches from it. Returns <code>true</code> on success; otherwise returns <code>false</code>. If the calling process exits, the detached process will continue to run unaffected.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QProcess::startDetached(qint64* pid = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#startDetached">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Starts the program set by <a href="http://doc.qt.io/qt-5/qprocess.html#setProgram">setProgram</a>() with arguments set by <a href="http://doc.qt.io/qt-5/qprocess.html#setArguments">setArguments</a>() in a new process, and detaches from it. Returns <code>true</code> on success; otherwise returns <code>false</code>. If the calling process exits, the detached process will continue to run unaffected.</p>
    /// <p><b>Unix:</b> The started process will run in its own session and act like a daemon.</p>
    /// <p>The process will be started in the directory set by <a href="http://doc.qt.io/qt-5/qprocess.html#setWorkingDirectory">setWorkingDirectory</a>(). If <a href="http://doc.qt.io/qt-5/qprocess.html#workingDirectory">workingDirectory</a>() is empty, the working directory is inherited from the calling process.</p>
    /// <p><b>Note: </b>On QNX, this may cause all application threads to temporarily freeze.</p><p>If the function is successful then *<i>pid</i> is set to the process identifier of the started process. Note that the child process may exit and the PID may become invalid without notice. Furthermore, after the child process exits, the same PID may be recycled and used by a completely different process. User code should be careful when using this variable, especially if one intends to forcibly terminate the process by operating system means.</p>
    /// <p>Only the following property setters are supported by startDetached():</p>
    /// <ul>
    /// <li><a href="http://doc.qt.io/qt-5/qprocess.html#setArguments">setArguments</a>()</li>
    /// <li><a href="http://doc.qt.io/qt-5/qprocess.html#setCreateProcessArgumentsModifier">setCreateProcessArgumentsModifier</a>()</li>
    /// <li><a href="http://doc.qt.io/qt-5/qprocess.html#setNativeArguments">setNativeArguments</a>()</li>
    /// <li><a href="http://doc.qt.io/qt-5/qprocess.html#setProcessEnvironment">setProcessEnvironment</a>()</li>
    /// <li><a href="http://doc.qt.io/qt-5/qprocess.html#setProgram">setProgram</a>()</li>
    /// <li><a href="http://doc.qt.io/qt-5/qprocess.html#setStandardErrorFile">setStandardErrorFile</a>()</li>
    /// <li><a href="http://doc.qt.io/qt-5/qprocess.html#setStandardInputFile">setStandardInputFile</a>()</li>
    /// <li><a href="http://doc.qt.io/qt-5/qprocess.html#setStandardOutputFile">setStandardOutputFile</a>()</li>
    /// <li><a href="http://doc.qt.io/qt-5/qprocess.html#setWorkingDirectory">setWorkingDirectory</a>()</li>
    /// </ul>
    /// <p>All other properties of the <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> object are ignored.</p>
    /// <p><b>Note: </b>The called process inherits the console window of the calling process. To suppress console output, redirect standard/error output to <a href="http://doc.qt.io/qt-5/qprocess.html#nullDevice">QProcess::nullDevice</a>().</p><p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#startDetached-1">startDetached</a>(const QString &amp;program, const QStringList &amp;arguments, const QString &amp;workingDirectory, qint64 *pid), and <a href="http://doc.qt.io/qt-5/qprocess.html#startDetached-2">startDetached</a>(const QString &amp;command).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn start_detached_1a_mut(
        &mut self,
        pid: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<i64>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QProcess_startDetached8(
            self as *mut crate::QProcess,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<i64>>::cast_into(pid).as_mut_raw_ptr(),
        )
    }

    /// <p>Starts the program set by <a href="http://doc.qt.io/qt-5/qprocess.html#setProgram">setProgram</a>() with arguments set by <a href="http://doc.qt.io/qt-5/qprocess.html#setArguments">setArguments</a>() in a new process, and detaches from it. Returns <code>true</code> on success; otherwise returns <code>false</code>. If the calling process exits, the detached process will continue to run unaffected.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QProcess::startDetached()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#startDetached">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Starts the program set by <a href="http://doc.qt.io/qt-5/qprocess.html#setProgram">setProgram</a>() with arguments set by <a href="http://doc.qt.io/qt-5/qprocess.html#setArguments">setArguments</a>() in a new process, and detaches from it. Returns <code>true</code> on success; otherwise returns <code>false</code>. If the calling process exits, the detached process will continue to run unaffected.</p>
    /// <p><b>Unix:</b> The started process will run in its own session and act like a daemon.</p>
    /// <p>The process will be started in the directory set by <a href="http://doc.qt.io/qt-5/qprocess.html#setWorkingDirectory">setWorkingDirectory</a>(). If <a href="http://doc.qt.io/qt-5/qprocess.html#workingDirectory">workingDirectory</a>() is empty, the working directory is inherited from the calling process.</p>
    /// <p><b>Note: </b>On QNX, this may cause all application threads to temporarily freeze.</p><p>If the function is successful then *<i>pid</i> is set to the process identifier of the started process. Note that the child process may exit and the PID may become invalid without notice. Furthermore, after the child process exits, the same PID may be recycled and used by a completely different process. User code should be careful when using this variable, especially if one intends to forcibly terminate the process by operating system means.</p>
    /// <p>Only the following property setters are supported by startDetached():</p>
    /// <ul>
    /// <li><a href="http://doc.qt.io/qt-5/qprocess.html#setArguments">setArguments</a>()</li>
    /// <li><a href="http://doc.qt.io/qt-5/qprocess.html#setCreateProcessArgumentsModifier">setCreateProcessArgumentsModifier</a>()</li>
    /// <li><a href="http://doc.qt.io/qt-5/qprocess.html#setNativeArguments">setNativeArguments</a>()</li>
    /// <li><a href="http://doc.qt.io/qt-5/qprocess.html#setProcessEnvironment">setProcessEnvironment</a>()</li>
    /// <li><a href="http://doc.qt.io/qt-5/qprocess.html#setProgram">setProgram</a>()</li>
    /// <li><a href="http://doc.qt.io/qt-5/qprocess.html#setStandardErrorFile">setStandardErrorFile</a>()</li>
    /// <li><a href="http://doc.qt.io/qt-5/qprocess.html#setStandardInputFile">setStandardInputFile</a>()</li>
    /// <li><a href="http://doc.qt.io/qt-5/qprocess.html#setStandardOutputFile">setStandardOutputFile</a>()</li>
    /// <li><a href="http://doc.qt.io/qt-5/qprocess.html#setWorkingDirectory">setWorkingDirectory</a>()</li>
    /// </ul>
    /// <p>All other properties of the <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> object are ignored.</p>
    /// <p><b>Note: </b>The called process inherits the console window of the calling process. To suppress console output, redirect standard/error output to <a href="http://doc.qt.io/qt-5/qprocess.html#nullDevice">QProcess::nullDevice</a>().</p><p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#start">start</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#startDetached-1">startDetached</a>(const QString &amp;program, const QStringList &amp;arguments, const QString &amp;workingDirectory, qint64 *pid), and <a href="http://doc.qt.io/qt-5/qprocess.html#startDetached-2">startDetached</a>(const QString &amp;command).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn start_detached_0a_mut(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QProcess_startDetached9(self as *mut crate::QProcess)
    }

    /// <p>Returns the current state of the process.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QProcess::ProcessState QProcess::state() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#state">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current state of the process.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#stateChanged">stateChanged</a>() and <a href="http://doc.qt.io/qt-5/qprocess.html#error">error</a>().</p></div>
    pub unsafe fn state(&self) -> crate::q_process::ProcessState {
        crate::__ffi::ctr_qt_core_ffi_QProcess_state(self as *const crate::QProcess)
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QProcess_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the environment of the calling process as a list of key=value pairs. Example:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QStringList QProcess::systemEnvironment()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#systemEnvironment">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the environment of the calling process as a list of key=value pairs. Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> environment <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qprocess.html#QProcess">QProcess</a></span><span class="operator">::</span>systemEnvironment();
    ///   <span class="comment">// environment = {"PATH=/usr/bin:/usr/local/bin",</span>
    ///   <span class="comment">//                "USER=greg", "HOME=/home/greg"}</span>
    ///
    /// </pre>
    /// <p>This function does not cache the system environment. Therefore, it's possible to obtain an updated version of the environment if low-level C library functions like <code>setenv</code> or <code>putenv</code> have been called.</p>
    /// <p>However, note that repeated calls to this function will recreate the list of environment variables, which is a non-trivial operation.</p>
    /// <p><b>Note: </b>For new code, it is recommended to use <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#systemEnvironment">QProcessEnvironment::systemEnvironment</a>()</p><p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocessenvironment.html#systemEnvironment">QProcessEnvironment::systemEnvironment</a>() and <a href="http://doc.qt.io/qt-5/qprocess.html#setProcessEnvironment">setProcessEnvironment</a>().</p></div>
    pub unsafe fn system_environment() -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QProcess_systemEnvironment();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Attempts to terminate the process.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QProcess::terminate()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#terminate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to terminate the process.</p>
    /// <p>The process may not exit as a result of calling this function (it is given the chance to prompt the user for any unsaved files, etc).</p>
    /// <p>On Windows, terminate() posts a WM_CLOSE message to all top-level windows of the process and then to the main thread of the process itself. On Unix and <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> the <code>SIGTERM</code> signal is sent.</p>
    /// <p>Console applications on Windows that do not run an event loop, or whose event loop does not handle the WM_CLOSE message, can only be terminated by calling <a href="http://doc.qt.io/qt-5/qprocess.html#kill">kill</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#kill">kill</a>().</p></div>
    pub unsafe fn terminate(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QProcess_terminate(self as *mut crate::QProcess)
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QProcess::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QProcess_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QProcess::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QProcess_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#waitForBytesWritten">QIODevice::waitForBytesWritten</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QProcess::waitForBytesWritten(int msecs = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#waitForBytesWritten">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#waitForBytesWritten">QIODevice::waitForBytesWritten</a>().</p></div>
    pub unsafe fn wait_for_bytes_written_1a(&mut self, msecs: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QProcess_waitForBytesWritten(
            self as *mut crate::QProcess,
            msecs,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#waitForBytesWritten">QIODevice::waitForBytesWritten</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QProcess::waitForBytesWritten()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#waitForBytesWritten">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#waitForBytesWritten">QIODevice::waitForBytesWritten</a>().</p></div>
    pub unsafe fn wait_for_bytes_written_0a(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QProcess_waitForBytesWritten1(self as *mut crate::QProcess)
    }

    /// <p>Blocks until the process has finished and the <a href="http://doc.qt.io/qt-5/qprocess.html#finished">finished</a>() signal has been emitted, or until <i>msecs</i> milliseconds have passed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QProcess::waitForFinished(int msecs = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#waitForFinished">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Blocks until the process has finished and the <a href="http://doc.qt.io/qt-5/qprocess.html#finished">finished</a>() signal has been emitted, or until <i>msecs</i> milliseconds have passed.</p>
    /// <p>Returns <code>true</code> if the process finished; otherwise returns <code>false</code> (if the operation timed out, if an error occurred, or if this <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> is already finished).</p>
    /// <p>This function can operate without an event loop. It is useful when writing non-GUI applications and when performing I/O operations in a non-GUI thread.</p>
    /// <p><b>Warning:</b> Calling this function from the main (GUI) thread might cause your user interface to freeze.</p>
    /// <p>If msecs is -1, this function will not time out.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#finished">finished</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#waitForStarted">waitForStarted</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#waitForReadyRead">waitForReadyRead</a>(), and <a href="http://doc.qt.io/qt-5/qprocess.html#waitForBytesWritten">waitForBytesWritten</a>().</p></div>
    pub unsafe fn wait_for_finished_1a(&mut self, msecs: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QProcess_waitForFinished(self as *mut crate::QProcess, msecs)
    }

    /// <p>Blocks until the process has finished and the <a href="http://doc.qt.io/qt-5/qprocess.html#finished">finished</a>() signal has been emitted, or until <i>msecs</i> milliseconds have passed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QProcess::waitForFinished()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#waitForFinished">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Blocks until the process has finished and the <a href="http://doc.qt.io/qt-5/qprocess.html#finished">finished</a>() signal has been emitted, or until <i>msecs</i> milliseconds have passed.</p>
    /// <p>Returns <code>true</code> if the process finished; otherwise returns <code>false</code> (if the operation timed out, if an error occurred, or if this <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> is already finished).</p>
    /// <p>This function can operate without an event loop. It is useful when writing non-GUI applications and when performing I/O operations in a non-GUI thread.</p>
    /// <p><b>Warning:</b> Calling this function from the main (GUI) thread might cause your user interface to freeze.</p>
    /// <p>If msecs is -1, this function will not time out.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#finished">finished</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#waitForStarted">waitForStarted</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#waitForReadyRead">waitForReadyRead</a>(), and <a href="http://doc.qt.io/qt-5/qprocess.html#waitForBytesWritten">waitForBytesWritten</a>().</p></div>
    pub unsafe fn wait_for_finished_0a(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QProcess_waitForFinished1(self as *mut crate::QProcess)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#waitForReadyRead">QIODevice::waitForReadyRead</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QProcess::waitForReadyRead(int msecs = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#waitForReadyRead">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#waitForReadyRead">QIODevice::waitForReadyRead</a>().</p></div>
    pub unsafe fn wait_for_ready_read_1a(&mut self, msecs: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QProcess_waitForReadyRead(self as *mut crate::QProcess, msecs)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#waitForReadyRead">QIODevice::waitForReadyRead</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QProcess::waitForReadyRead()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#waitForReadyRead">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#waitForReadyRead">QIODevice::waitForReadyRead</a>().</p></div>
    pub unsafe fn wait_for_ready_read_0a(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QProcess_waitForReadyRead1(self as *mut crate::QProcess)
    }

    /// <p>Blocks until the process has started and the <a href="http://doc.qt.io/qt-5/qprocess.html#started">started</a>() signal has been emitted, or until <i>msecs</i> milliseconds have passed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QProcess::waitForStarted(int msecs = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#waitForStarted">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Blocks until the process has started and the <a href="http://doc.qt.io/qt-5/qprocess.html#started">started</a>() signal has been emitted, or until <i>msecs</i> milliseconds have passed.</p>
    /// <p>Returns <code>true</code> if the process was started successfully; otherwise returns <code>false</code> (if the operation timed out or if an error occurred).</p>
    /// <p>This function can operate without an event loop. It is useful when writing non-GUI applications and when performing I/O operations in a non-GUI thread.</p>
    /// <p><b>Warning:</b> Calling this function from the main (GUI) thread might cause your user interface to freeze.</p>
    /// <p>If msecs is -1, this function will not time out.</p>
    /// <p><b>Note: </b>On some UNIX operating systems, this function may return true but the process may later report a <a href="http://doc.qt.io/qt-5/qprocess.html#ProcessError-enum">QProcess::FailedToStart</a> error.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#started">started</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#waitForReadyRead">waitForReadyRead</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#waitForBytesWritten">waitForBytesWritten</a>(), and <a href="http://doc.qt.io/qt-5/qprocess.html#waitForFinished">waitForFinished</a>().</p></div>
    pub unsafe fn wait_for_started_1a(&mut self, msecs: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QProcess_waitForStarted(self as *mut crate::QProcess, msecs)
    }

    /// <p>Blocks until the process has started and the <a href="http://doc.qt.io/qt-5/qprocess.html#started">started</a>() signal has been emitted, or until <i>msecs</i> milliseconds have passed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QProcess::waitForStarted()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#waitForStarted">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Blocks until the process has started and the <a href="http://doc.qt.io/qt-5/qprocess.html#started">started</a>() signal has been emitted, or until <i>msecs</i> milliseconds have passed.</p>
    /// <p>Returns <code>true</code> if the process was started successfully; otherwise returns <code>false</code> (if the operation timed out or if an error occurred).</p>
    /// <p>This function can operate without an event loop. It is useful when writing non-GUI applications and when performing I/O operations in a non-GUI thread.</p>
    /// <p><b>Warning:</b> Calling this function from the main (GUI) thread might cause your user interface to freeze.</p>
    /// <p>If msecs is -1, this function will not time out.</p>
    /// <p><b>Note: </b>On some UNIX operating systems, this function may return true but the process may later report a <a href="http://doc.qt.io/qt-5/qprocess.html#ProcessError-enum">QProcess::FailedToStart</a> error.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#started">started</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#waitForReadyRead">waitForReadyRead</a>(), <a href="http://doc.qt.io/qt-5/qprocess.html#waitForBytesWritten">waitForBytesWritten</a>(), and <a href="http://doc.qt.io/qt-5/qprocess.html#waitForFinished">waitForFinished</a>().</p></div>
    pub unsafe fn wait_for_started_0a(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QProcess_waitForStarted1(self as *mut crate::QProcess)
    }

    /// <p>If <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> has been assigned a working directory, this function returns the working directory that the <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> will enter before the program has started. Otherwise, (i.e., no directory has been assigned,) an empty string is returned, and <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> will use the application's current working directory instead.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QProcess::workingDirectory() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#workingDirectory">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> has been assigned a working directory, this function returns the working directory that the <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> will enter before the program has started. Otherwise, (i.e., no directory has been assigned,) an empty string is returned, and <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> will use the application's current working directory instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocess.html#setWorkingDirectory">setWorkingDirectory</a>().</p></div>
    pub unsafe fn working_directory(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QProcess_workingDirectory(self as *const crate::QProcess);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qvariantanimation.html">QVariantAnimation</a> class provides a base class for animations.</p>
///
/// C++ class: <span style='color: green;'>```QVariantAnimation```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qvariantanimation.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qvariantanimation.html">QVariantAnimation</a> class provides a base class for animations.</p>
/// <p>This class is part of <a href="http://doc.qt.io/qt-5/animation-overview.html">The Animation Framework</a>. It serves as a base class for property and item animations, with functions for shared functionality.</p>
/// <p>The class performs interpolation over <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>s, but leaves using the interpolated values to its subclasses. Currently, Qt provides <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a>, which animates Qt <a href="http://doc.qt.io/qt-5/properties.html">properties</a>. See the <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a> class description if you wish to animate such properties.</p>
/// <p>You can then set start and end values for the property by calling <a href="http://doc.qt.io/qt-5/qvariantanimation.html#startValue-prop">setStartValue</a>() and <a href="http://doc.qt.io/qt-5/qvariantanimation.html#endValue-prop">setEndValue</a>(), and finally call <a href="http://doc.qt.io/qt-5/qabstractanimation.html#start">start</a>() to start the animation. <a href="http://doc.qt.io/qt-5/qvariantanimation.html">QVariantAnimation</a> will interpolate the property of the target object and emit <a href="http://doc.qt.io/qt-5/qvariantanimation.html#valueChanged">valueChanged</a>(). To react to a change in the current value you have to reimplement the <a href="http://doc.qt.io/qt-5/qvariantanimation.html#updateCurrentValue">updateCurrentValue</a>() virtual function or connect to said signal.</p>
/// <p>It is also possible to set values at specified steps situated between the start and end value. The interpolation will then touch these points at the specified steps. Note that the start and end values are defined as the key values at 0.0 and 1.0.</p>
/// <p>There are two ways to affect how <a href="http://doc.qt.io/qt-5/qvariantanimation.html">QVariantAnimation</a> interpolates the values. You can set an easing curve by calling <a href="http://doc.qt.io/qt-5/qvariantanimation.html#easingCurve-prop">setEasingCurve</a>(), and configure the duration by calling <a href="http://doc.qt.io/qt-5/qvariantanimation.html#duration-prop">setDuration</a>(). You can change how the <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>s are interpolated by creating a subclass of <a href="http://doc.qt.io/qt-5/qvariantanimation.html">QVariantAnimation</a>, and reimplementing the virtual <a href="http://doc.qt.io/qt-5/qvariantanimation.html#interpolated">interpolated</a>() function.</p>
/// <p>Subclassing <a href="http://doc.qt.io/qt-5/qvariantanimation.html">QVariantAnimation</a> can be an alternative if you have <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>s that you do not wish to declare as Qt properties. Note, however, that you in most cases will be better off declaring your <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> as a property.</p>
/// <p>Not all <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> types are supported. Below is a list of currently supported <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> types:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">Int</a></li>
/// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">UInt</a></li>
/// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">Double</a></li>
/// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">Float</a></li>
/// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QLine</a></li>
/// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QLineF</a></li>
/// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QPoint</a></li>
/// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QPointF</a></li>
/// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QSize</a></li>
/// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QSizeF</a></li>
/// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QRect</a></li>
/// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QRectF</a></li>
/// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QColor</a></li>
/// </ul>
/// <p>If you need to interpolate other variant types, including custom types, you have to implement interpolation for these yourself. To do this, you can register an interpolator function for a given type. This function takes 3 parameters: the start value, the end value, and the current progress.</p>
/// <p>Example:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> myColorInterpolator(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcolor.html">QColor</a></span> <span class="operator">&amp;</span>start<span class="operator">,</span> <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcolor.html">QColor</a></span> <span class="operator">&amp;</span>end<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#qreal-typedef">qreal</a></span> progress)
///   {
/// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
/// &#32;     <span class="keyword">return</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcolor.html">QColor</a></span>(<span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>);
///   }
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   <a href="http://doc.qt.io/qt-5/qvariantanimation.html#qRegisterAnimationInterpolator">qRegisterAnimationInterpolator</a><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qcolor.html">QColor</a></span><span class="operator">&gt;</span>(myColorInterpolator);
///
/// </pre>
/// <p>Another option is to reimplement <a href="http://doc.qt.io/qt-5/qvariantanimation.html#interpolated">interpolated</a>(), which returns interpolation values for the value being interpolated.</p></div>
#[repr(C)]
pub struct QVariantAnimation {
    _unused: u8,
}
impl QVariantAnimation {
    /// <p><a href="http://doc.qt.io/qt-5/qvariantanimation.html">QVariantAnimation</a> emits this signal whenever the current <i>value</i> changes.</p>
    ///
    /// Returns a built-in Qt signal `QVariantAnimation::valueChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariantanimation.html#valueChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p><a href="http://doc.qt.io/qt-5/qvariantanimation.html">QVariantAnimation</a> emits this signal whenever the current <i>value</i> changes.</p>
    /// <p><b>Note:</b> Notifier signal for property <a href="http://doc.qt.io/qt-5/qvariantanimation.html#currentValue-prop">currentValue</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariantanimation.html#currentValue-prop">currentValue</a>, <a href="http://doc.qt.io/qt-5/qvariantanimation.html#startValue-prop">startValue</a>, and <a href="http://doc.qt.io/qt-5/qvariantanimation.html#endValue-prop">endValue</a>.</p></div>
    pub fn value_changed(&self) -> crate::Signal<(*const crate::QVariant,)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2valueChanged(const QVariant&)\0",
                ),
            )
        }
    }

    /// <p>This property holds the current value of the animation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QVariantAnimation::currentValue() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariantanimation.html#currentValue-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the current value of the animation.</p>
    /// <p>This property describes the current value; an interpolated value between the <a href="http://doc.qt.io/qt-5/qvariantanimation.html#startValue-prop">start value</a> and the <a href="http://doc.qt.io/qt-5/qvariantanimation.html#endValue-prop">end value</a>, using the current time for progress. The value itself is obtained from <a href="http://doc.qt.io/qt-5/qvariantanimation.html#interpolated">interpolated</a>(), which is called repeatedly as the animation is running.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qvariantanimation.html">QVariantAnimation</a> calls the virtual <a href="http://doc.qt.io/qt-5/qvariantanimation.html#updateCurrentValue">updateCurrentValue</a>() function when the current value changes. This is particularly useful for subclasses that need to track updates. For example, <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a> uses this function to animate Qt <a href="http://doc.qt.io/qt-5/properties.html">properties</a>.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QVariant </td><td class="memItemRight bottomAlign"><span class="name"><b>currentValue</b></span>() const</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qvariantanimation.html#valueChanged">valueChanged</a></b></span>(const QVariant &amp;<i>value</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariantanimation.html#startValue-prop">startValue</a> and <a href="http://doc.qt.io/qt-5/qvariantanimation.html#endValue-prop">endValue</a>.</p></div>
    pub unsafe fn current_value(&self) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariantAnimation_currentValue(
            self as *const crate::QVariantAnimation,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This property holds the duration of the animation</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual int QVariantAnimation::duration() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariantanimation.html#duration-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the duration of the animation</p>
    /// <p>This property describes the duration in milliseconds of the animation. The default duration is 250 milliseconds.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> virtual int </td><td class="memItemRight bottomAlign"><span class="name"><b>duration</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setDuration</b></span>(int <i>msecs</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#duration">QAbstractAnimation::duration</a>().</p></div>
    pub unsafe fn duration(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVariantAnimation_duration(
            self as *const crate::QVariantAnimation,
        )
    }

    /// <p>This property holds the easing curve of the animation</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QEasingCurve QVariantAnimation::easingCurve() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariantanimation.html#easingCurve-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the easing curve of the animation</p>
    /// <p>This property defines the easing curve of the animation. By default, a linear easing curve is used, resulting in linear interpolation. Other curves are provided, for instance, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::InCirc</a>, which provides a circular entry curve. Another example is <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::InOutElastic</a>, which provides an elastic effect on the values of the interpolated variant.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qvariantanimation.html">QVariantAnimation</a> will use the <a href="http://doc.qt.io/qt-5/qeasingcurve.html#valueForProgress">QEasingCurve::valueForProgress</a>() to transform the "normalized progress" (<a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentTime-prop">currentTime</a> / <a href="http://doc.qt.io/qt-5/qabstractanimation.html#totalDuration">totalDuration</a>) of the animation into the effective progress actually used by the animation. It is this effective progress that will be the progress when <a href="http://doc.qt.io/qt-5/qvariantanimation.html#interpolated">interpolated</a>() is called. Also, the steps in the <a href="http://doc.qt.io/qt-5/qvariantanimation.html#keyValues">keyValues</a> are referring to this effective progress.</p>
    /// <p>The easing curve is used with the interpolator, the <a href="http://doc.qt.io/qt-5/qvariantanimation.html#interpolated">interpolated</a>() virtual function, and the animation's duration to control how the current value changes as the animation progresses.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QEasingCurve </td><td class="memItemRight bottomAlign"><span class="name"><b>easingCurve</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setEasingCurve</b></span>(const QEasingCurve &amp;<i>easing</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn easing_curve(&self) -> ::cpp_utils::CppBox<crate::QEasingCurve> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariantAnimation_easingCurve(
            self as *const crate::QVariantAnimation,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This property holds the end value of the animation</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QVariantAnimation::endValue() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariantanimation.html#endValue-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the end value of the animation</p>
    /// <p>This property describes the end value of the animation.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QVariant </td><td class="memItemRight bottomAlign"><span class="name"><b>endValue</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setEndValue</b></span>(const QVariant &amp;<i>value</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariantanimation.html#startValue-prop">startValue</a>.</p></div>
    pub unsafe fn end_value(&self) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariantAnimation_endValue(
            self as *const crate::QVariantAnimation,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the key frame value for the given <i>step</i>. The given <i>step</i> must be in the range 0 to 1. If there is no <a href="http://doc.qt.io/qt-5/qvariantanimation.html#KeyValue-typedef">KeyValue</a> for <i>step</i>, it returns an invalid <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QVariantAnimation::keyValueAt(double step) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariantanimation.html#keyValueAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the key frame value for the given <i>step</i>. The given <i>step</i> must be in the range 0 to 1. If there is no <a href="http://doc.qt.io/qt-5/qvariantanimation.html#KeyValue-typedef">KeyValue</a> for <i>step</i>, it returns an invalid <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariantanimation.html#keyValues">keyValues</a>() and <a href="http://doc.qt.io/qt-5/qvariantanimation.html#setKeyValueAt">setKeyValueAt</a>().</p></div>
    pub unsafe fn key_value_at(
        &self,
        step: ::std::os::raw::c_double,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariantAnimation_keyValueAt(
            self as *const crate::QVariantAnimation,
            step,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the key frames of this animation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QPair<double, QVariant>> QVariantAnimation::keyValues() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariantanimation.html#keyValues">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the key frames of this animation.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariantanimation.html#keyValueAt">keyValueAt</a>() and <a href="http://doc.qt.io/qt-5/qvariantanimation.html#setKeyValues">setKeyValues</a>().</p></div>
    pub unsafe fn key_values(&self) -> ::cpp_utils::CppBox<crate::QVectorOfQPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariantAnimation_keyValues(
            self as *const crate::QVariantAnimation,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QVariantAnimation::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariantAnimation_metaObject(
            self as *const crate::QVariantAnimation,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Construct a <a href="http://doc.qt.io/qt-5/qvariantanimation.html">QVariantAnimation</a> object. <i>parent</i> is passed to <a href="http://doc.qt.io/qt-5/qabstractanimation.html">QAbstractAnimation</a>'s constructor.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariantAnimation::QVariantAnimation(QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariantanimation.html#QVariantAnimation">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Construct a <a href="http://doc.qt.io/qt-5/qvariantanimation.html">QVariantAnimation</a> object. <i>parent</i> is passed to <a href="http://doc.qt.io/qt-5/qabstractanimation.html">QAbstractAnimation</a>'s constructor.</p></div>
    pub unsafe fn new_1a(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QVariantAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariantAnimation_QVariantAnimation(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qvariantanimation.html">QVariantAnimation</a> class provides a base class for animations.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVariantAnimation::QVariantAnimation()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariantanimation.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qvariantanimation.html">QVariantAnimation</a> class provides a base class for animations.</p>
    /// <p>This class is part of <a href="http://doc.qt.io/qt-5/animation-overview.html">The Animation Framework</a>. It serves as a base class for property and item animations, with functions for shared functionality.</p>
    /// <p>The class performs interpolation over <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>s, but leaves using the interpolated values to its subclasses. Currently, Qt provides <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a>, which animates Qt <a href="http://doc.qt.io/qt-5/properties.html">properties</a>. See the <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a> class description if you wish to animate such properties.</p>
    /// <p>You can then set start and end values for the property by calling <a href="http://doc.qt.io/qt-5/qvariantanimation.html#startValue-prop">setStartValue</a>() and <a href="http://doc.qt.io/qt-5/qvariantanimation.html#endValue-prop">setEndValue</a>(), and finally call <a href="http://doc.qt.io/qt-5/qabstractanimation.html#start">start</a>() to start the animation. <a href="http://doc.qt.io/qt-5/qvariantanimation.html">QVariantAnimation</a> will interpolate the property of the target object and emit <a href="http://doc.qt.io/qt-5/qvariantanimation.html#valueChanged">valueChanged</a>(). To react to a change in the current value you have to reimplement the <a href="http://doc.qt.io/qt-5/qvariantanimation.html#updateCurrentValue">updateCurrentValue</a>() virtual function or connect to said signal.</p>
    /// <p>It is also possible to set values at specified steps situated between the start and end value. The interpolation will then touch these points at the specified steps. Note that the start and end values are defined as the key values at 0.0 and 1.0.</p>
    /// <p>There are two ways to affect how <a href="http://doc.qt.io/qt-5/qvariantanimation.html">QVariantAnimation</a> interpolates the values. You can set an easing curve by calling <a href="http://doc.qt.io/qt-5/qvariantanimation.html#easingCurve-prop">setEasingCurve</a>(), and configure the duration by calling <a href="http://doc.qt.io/qt-5/qvariantanimation.html#duration-prop">setDuration</a>(). You can change how the <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>s are interpolated by creating a subclass of <a href="http://doc.qt.io/qt-5/qvariantanimation.html">QVariantAnimation</a>, and reimplementing the virtual <a href="http://doc.qt.io/qt-5/qvariantanimation.html#interpolated">interpolated</a>() function.</p>
    /// <p>Subclassing <a href="http://doc.qt.io/qt-5/qvariantanimation.html">QVariantAnimation</a> can be an alternative if you have <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>s that you do not wish to declare as Qt properties. Note, however, that you in most cases will be better off declaring your <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> as a property.</p>
    /// <p>Not all <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> types are supported. Below is a list of currently supported <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> types:</p>
    /// <ul>
    /// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">Int</a></li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">UInt</a></li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">Double</a></li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">Float</a></li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QLine</a></li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QLineF</a></li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QPoint</a></li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QPointF</a></li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QSize</a></li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QSizeF</a></li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QRect</a></li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QRectF</a></li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">QColor</a></li>
    /// </ul>
    /// <p>If you need to interpolate other variant types, including custom types, you have to implement interpolation for these yourself. To do this, you can register an interpolator function for a given type. This function takes 3 parameters: the start value, the end value, and the current progress.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> myColorInterpolator(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcolor.html">QColor</a></span> <span class="operator">&amp;</span>start<span class="operator">,</span> <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcolor.html">QColor</a></span> <span class="operator">&amp;</span>end<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#qreal-typedef">qreal</a></span> progress)
    ///   {
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    /// &#32;     <span class="keyword">return</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcolor.html">QColor</a></span>(<span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>);
    ///   }
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qvariantanimation.html#qRegisterAnimationInterpolator">qRegisterAnimationInterpolator</a><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qcolor.html">QColor</a></span><span class="operator">&gt;</span>(myColorInterpolator);
    ///
    /// </pre>
    /// <p>Another option is to reimplement <a href="http://doc.qt.io/qt-5/qvariantanimation.html#interpolated">interpolated</a>(), which returns interpolation values for the value being interpolated.</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QVariantAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariantAnimation_QVariantAnimation1();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QVariantAnimation::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVariantAnimation_qt_metacall(
            self as *mut crate::QVariantAnimation,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QVariantAnimation::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariantAnimation_qt_metacast(
            self as *mut crate::QVariantAnimation,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This property holds the duration of the animation</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVariantAnimation::setDuration(int msecs)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariantanimation.html#duration-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the duration of the animation</p>
    /// <p>This property describes the duration in milliseconds of the animation. The default duration is 250 milliseconds.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> virtual int </td><td class="memItemRight bottomAlign"><span class="name"><b>duration</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setDuration</b></span>(int <i>msecs</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractanimation.html#duration">QAbstractAnimation::duration</a>().</p></div>
    pub unsafe fn set_duration(&mut self, msecs: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVariantAnimation_setDuration(
            self as *mut crate::QVariantAnimation,
            msecs,
        )
    }

    /// <p>This property holds the easing curve of the animation</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVariantAnimation::setEasingCurve(const QEasingCurve& easing)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariantanimation.html#easingCurve-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the easing curve of the animation</p>
    /// <p>This property defines the easing curve of the animation. By default, a linear easing curve is used, resulting in linear interpolation. Other curves are provided, for instance, <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::InCirc</a>, which provides a circular entry curve. Another example is <a href="http://doc.qt.io/qt-5/qeasingcurve.html#Type-enum">QEasingCurve::InOutElastic</a>, which provides an elastic effect on the values of the interpolated variant.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qvariantanimation.html">QVariantAnimation</a> will use the <a href="http://doc.qt.io/qt-5/qeasingcurve.html#valueForProgress">QEasingCurve::valueForProgress</a>() to transform the "normalized progress" (<a href="http://doc.qt.io/qt-5/qabstractanimation.html#currentTime-prop">currentTime</a> / <a href="http://doc.qt.io/qt-5/qabstractanimation.html#totalDuration">totalDuration</a>) of the animation into the effective progress actually used by the animation. It is this effective progress that will be the progress when <a href="http://doc.qt.io/qt-5/qvariantanimation.html#interpolated">interpolated</a>() is called. Also, the steps in the <a href="http://doc.qt.io/qt-5/qvariantanimation.html#keyValues">keyValues</a> are referring to this effective progress.</p>
    /// <p>The easing curve is used with the interpolator, the <a href="http://doc.qt.io/qt-5/qvariantanimation.html#interpolated">interpolated</a>() virtual function, and the animation's duration to control how the current value changes as the animation progresses.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QEasingCurve </td><td class="memItemRight bottomAlign"><span class="name"><b>easingCurve</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setEasingCurve</b></span>(const QEasingCurve &amp;<i>easing</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn set_easing_curve(
        &mut self,
        easing: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QEasingCurve>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVariantAnimation_setEasingCurve(
            self as *mut crate::QVariantAnimation,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QEasingCurve>>::cast_into(easing)
                .as_raw_ptr(),
        )
    }

    /// <p>This property holds the end value of the animation</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVariantAnimation::setEndValue(const QVariant& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariantanimation.html#endValue-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the end value of the animation</p>
    /// <p>This property describes the end value of the animation.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QVariant </td><td class="memItemRight bottomAlign"><span class="name"><b>endValue</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setEndValue</b></span>(const QVariant &amp;<i>value</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariantanimation.html#startValue-prop">startValue</a>.</p></div>
    pub unsafe fn set_end_value(
        &mut self,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVariantAnimation_setEndValue(
            self as *mut crate::QVariantAnimation,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>Creates a key frame at the given <i>step</i> with the given <i>value</i>. The given <i>step</i> must be in the range 0 to 1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVariantAnimation::setKeyValueAt(double step, const QVariant& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariantanimation.html#setKeyValueAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a key frame at the given <i>step</i> with the given <i>value</i>. The given <i>step</i> must be in the range 0 to 1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariantanimation.html#setKeyValues">setKeyValues</a>() and <a href="http://doc.qt.io/qt-5/qvariantanimation.html#keyValueAt">keyValueAt</a>().</p></div>
    pub unsafe fn set_key_value_at(
        &mut self,
        step: ::std::os::raw::c_double,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVariantAnimation_setKeyValueAt(
            self as *mut crate::QVariantAnimation,
            step,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>Replaces the current set of key frames with the given <i>keyValues</i>. the step of the key frames must be in the range 0 to 1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVariantAnimation::setKeyValues(const QVector<QPair<double, QVariant>>& values)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariantanimation.html#setKeyValues">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the current set of key frames with the given <i>keyValues</i>. the step of the key frames must be in the range 0 to 1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariantanimation.html#keyValues">keyValues</a>() and <a href="http://doc.qt.io/qt-5/qvariantanimation.html#keyValueAt">keyValueAt</a>().</p></div>
    pub unsafe fn set_key_values(
        &mut self,
        values: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVariantAnimation_setKeyValues(self as *mut crate::QVariantAnimation, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>>::cast_into(values).as_raw_ptr())
    }

    /// <p>This property holds the optional start value of the animation</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVariantAnimation::setStartValue(const QVariant& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariantanimation.html#startValue-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the optional start value of the animation</p>
    /// <p>This property describes the optional start value of the animation. If omitted, or if a null <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> is assigned as the start value, the animation will use the current position of the end when the animation is started.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QVariant </td><td class="memItemRight bottomAlign"><span class="name"><b>startValue</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setStartValue</b></span>(const QVariant &amp;<i>value</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariantanimation.html#endValue-prop">endValue</a>.</p></div>
    pub unsafe fn set_start_value(
        &mut self,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVariantAnimation_setStartValue(
            self as *mut crate::QVariantAnimation,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>This property holds the optional start value of the animation</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QVariantAnimation::startValue() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariantanimation.html#startValue-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the optional start value of the animation</p>
    /// <p>This property describes the optional start value of the animation. If omitted, or if a null <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> is assigned as the start value, the animation will use the current position of the end when the animation is started.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QVariant </td><td class="memItemRight bottomAlign"><span class="name"><b>startValue</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setStartValue</b></span>(const QVariant &amp;<i>value</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariantanimation.html#endValue-prop">endValue</a>.</p></div>
    pub unsafe fn start_value(&self) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariantAnimation_startValue(
            self as *const crate::QVariantAnimation,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariantAnimation_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QVariantAnimation::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariantAnimation_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QVariantAnimation::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVariantAnimation_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a> class animates Qt properties</p>
///
/// C++ class: <span style='color: green;'>```QPropertyAnimation```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a> class animates Qt properties</p>
/// <p><a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a> interpolates over <a href="http://doc.qt.io/qt-5/properties.html">Qt properties</a>. As property values are stored in <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>s, the class inherits <a href="http://doc.qt.io/qt-5/qvariantanimation.html">QVariantAnimation</a>, and supports animation of the same <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">meta types</a> as its super class.</p>
/// <p>A class declaring properties must be a <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>. To make it possible to animate a property, it must provide a setter (so that <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a> can set the property's value). Note that this makes it possible to animate many of Qt's widgets. Let's look at an example:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qpropertyanimation.html#QPropertyAnimation">QPropertyAnimation</a></span> <span class="operator">*</span>animation <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qpropertyanimation.html#QPropertyAnimation">QPropertyAnimation</a></span>(myWidget<span class="operator">,</span> <span class="string">"geometry"</span>);
///   animation<span class="operator">-</span><span class="operator">&gt;</span>setDuration(<span class="number">10000</span>);
///   animation<span class="operator">-</span><span class="operator">&gt;</span>setStartValue(<span class="type"><a href="http://doc.qt.io/qt-5/qrect.html">QRect</a></span>(<span class="number">0</span><span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="number">100</span><span class="operator">,</span> <span class="number">30</span>));
///   animation<span class="operator">-</span><span class="operator">&gt;</span>setEndValue(<span class="type"><a href="http://doc.qt.io/qt-5/qrect.html">QRect</a></span>(<span class="number">250</span><span class="operator">,</span> <span class="number">250</span><span class="operator">,</span> <span class="number">100</span><span class="operator">,</span> <span class="number">30</span>));
///
///   animation<span class="operator">-</span><span class="operator">&gt;</span>start();
///
/// </pre>
/// <p>The property name and the <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> instance of which property should be animated are passed to the constructor. You can then specify the start and end value of the property. The procedure is equal for properties in classes you have implemented yourself--just check with <a href="http://doc.qt.io/qt-5/qvariantanimation.html">QVariantAnimation</a> that your <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> type is supported.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qvariantanimation.html">QVariantAnimation</a> class description explains how to set up the animation in detail. Note, however, that if a start value is not set, the property will start at the value it had when the <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a> instance was created.</p>
/// <p><a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a> works like a charm on its own. For complex animations that, for instance, contain several objects, <a href="http://doc.qt.io/qt-5/qanimationgroup.html">QAnimationGroup</a> is provided. An animation group is an animation that can contain other animations, and that can manage when its animations are played. Look at <a href="http://doc.qt.io/qt-5/qparallelanimationgroup.html">QParallelAnimationGroup</a> for an example.</p></div>
#[repr(C)]
pub struct QPropertyAnimation {
    _unused: u8,
}
impl QPropertyAnimation {
    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QPropertyAnimation::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPropertyAnimation_metaObject(
            self as *const crate::QPropertyAnimation,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Construct a <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a> object. <i>parent</i> is passed to <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>'s constructor.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPropertyAnimation::QPropertyAnimation(QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpropertyanimation.html#QPropertyAnimation">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Construct a <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a> object. <i>parent</i> is passed to <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>'s constructor.</p></div>
    pub unsafe fn new_1a(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QPropertyAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPropertyAnimation_QPropertyAnimation(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Construct a <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a> object. <i>parent</i> is passed to <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>'s constructor. The animation changes the property <i>propertyName</i> on <i>target</i>. The default duration is 250ms.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPropertyAnimation::QPropertyAnimation(QObject* target, const QByteArray& propertyName, QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpropertyanimation.html#QPropertyAnimation-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Construct a <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a> object. <i>parent</i> is passed to <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>'s constructor. The animation changes the property <i>propertyName</i> on <i>target</i>. The default duration is 250ms.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpropertyanimation.html#targetObject-prop">targetObject</a> and <a href="http://doc.qt.io/qt-5/qpropertyanimation.html#propertyName-prop">propertyName</a>.</p></div>
    pub unsafe fn new_3a(
        target: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
        property_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QPropertyAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPropertyAnimation_QPropertyAnimation1(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(target)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(property_name)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a> class animates Qt properties</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPropertyAnimation::QPropertyAnimation()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a> class animates Qt properties</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a> interpolates over <a href="http://doc.qt.io/qt-5/properties.html">Qt properties</a>. As property values are stored in <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>s, the class inherits <a href="http://doc.qt.io/qt-5/qvariantanimation.html">QVariantAnimation</a>, and supports animation of the same <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">meta types</a> as its super class.</p>
    /// <p>A class declaring properties must be a <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>. To make it possible to animate a property, it must provide a setter (so that <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a> can set the property's value). Note that this makes it possible to animate many of Qt's widgets. Let's look at an example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpropertyanimation.html#QPropertyAnimation">QPropertyAnimation</a></span> <span class="operator">*</span>animation <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qpropertyanimation.html#QPropertyAnimation">QPropertyAnimation</a></span>(myWidget<span class="operator">,</span> <span class="string">"geometry"</span>);
    ///   animation<span class="operator">-</span><span class="operator">&gt;</span>setDuration(<span class="number">10000</span>);
    ///   animation<span class="operator">-</span><span class="operator">&gt;</span>setStartValue(<span class="type"><a href="http://doc.qt.io/qt-5/qrect.html">QRect</a></span>(<span class="number">0</span><span class="operator">,</span> <span class="number">0</span><span class="operator">,</span> <span class="number">100</span><span class="operator">,</span> <span class="number">30</span>));
    ///   animation<span class="operator">-</span><span class="operator">&gt;</span>setEndValue(<span class="type"><a href="http://doc.qt.io/qt-5/qrect.html">QRect</a></span>(<span class="number">250</span><span class="operator">,</span> <span class="number">250</span><span class="operator">,</span> <span class="number">100</span><span class="operator">,</span> <span class="number">30</span>));
    ///
    ///   animation<span class="operator">-</span><span class="operator">&gt;</span>start();
    ///
    /// </pre>
    /// <p>The property name and the <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> instance of which property should be animated are passed to the constructor. You can then specify the start and end value of the property. The procedure is equal for properties in classes you have implemented yourself--just check with <a href="http://doc.qt.io/qt-5/qvariantanimation.html">QVariantAnimation</a> that your <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> type is supported.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qvariantanimation.html">QVariantAnimation</a> class description explains how to set up the animation in detail. Note, however, that if a start value is not set, the property will start at the value it had when the <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a> instance was created.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a> works like a charm on its own. For complex animations that, for instance, contain several objects, <a href="http://doc.qt.io/qt-5/qanimationgroup.html">QAnimationGroup</a> is provided. An animation group is an animation that can contain other animations, and that can manage when its animations are played. Look at <a href="http://doc.qt.io/qt-5/qparallelanimationgroup.html">QParallelAnimationGroup</a> for an example.</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QPropertyAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPropertyAnimation_QPropertyAnimation2();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Construct a <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a> object. <i>parent</i> is passed to <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>'s constructor. The animation changes the property <i>propertyName</i> on <i>target</i>. The default duration is 250ms.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPropertyAnimation::QPropertyAnimation(QObject* target, const QByteArray& propertyName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpropertyanimation.html#QPropertyAnimation-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Construct a <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a> object. <i>parent</i> is passed to <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>'s constructor. The animation changes the property <i>propertyName</i> on <i>target</i>. The default duration is 250ms.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpropertyanimation.html#targetObject-prop">targetObject</a> and <a href="http://doc.qt.io/qt-5/qpropertyanimation.html#propertyName-prop">propertyName</a>.</p></div>
    pub unsafe fn new_2a(
        target: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
        property_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QPropertyAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPropertyAnimation_QPropertyAnimation3(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(target)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(property_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This property holds the target property name for this animation</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QPropertyAnimation::propertyName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpropertyanimation.html#propertyName-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the target property name for this animation</p>
    /// <p>This property defines the target property name for this animation. The property name is required for the animation to operate.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QByteArray </td><td class="memItemRight bottomAlign"><span class="name"><b>propertyName</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setPropertyName</b></span>(const QByteArray &amp;<i>propertyName</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn property_name(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPropertyAnimation_propertyName(
            self as *const crate::QPropertyAnimation,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QPropertyAnimation::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QPropertyAnimation_qt_metacall(
            self as *mut crate::QPropertyAnimation,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QPropertyAnimation::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPropertyAnimation_qt_metacast(
            self as *mut crate::QPropertyAnimation,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This property holds the target property name for this animation</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QPropertyAnimation::setPropertyName(const QByteArray& propertyName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpropertyanimation.html#propertyName-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the target property name for this animation</p>
    /// <p>This property defines the target property name for this animation. The property name is required for the animation to operate.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QByteArray </td><td class="memItemRight bottomAlign"><span class="name"><b>propertyName</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setPropertyName</b></span>(const QByteArray &amp;<i>propertyName</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn set_property_name(
        &mut self,
        property_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QPropertyAnimation_setPropertyName(
            self as *mut crate::QPropertyAnimation,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(property_name)
                .as_raw_ptr(),
        )
    }

    /// <p>This property holds the target <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> for this animation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QPropertyAnimation::setTargetObject(QObject* target)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpropertyanimation.html#targetObject-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the target <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> for this animation.</p>
    /// <p>This property defines the target <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> for this animation.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QObject *</td><td class="memItemRight bottomAlign"><span class="name"><b>targetObject</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setTargetObject</b></span>(QObject *<i>target</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn set_target_object(
        &mut self,
        target: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QPropertyAnimation_setTargetObject(
            self as *mut crate::QPropertyAnimation,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(target)
                .as_mut_raw_ptr(),
        )
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPropertyAnimation_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This property holds the target <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> for this animation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QObject* QPropertyAnimation::targetObject() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpropertyanimation.html#targetObject-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the target <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> for this animation.</p>
    /// <p>This property defines the target <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> for this animation.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QObject *</td><td class="memItemRight bottomAlign"><span class="name"><b>targetObject</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setTargetObject</b></span>(QObject *<i>target</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn target_object(&self) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPropertyAnimation_targetObject(
            self as *const crate::QPropertyAnimation,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QPropertyAnimation::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPropertyAnimation_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QPropertyAnimation::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPropertyAnimation_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_read_write_lock {
    //! C++ type: <span style='color: green;'>```QReadWriteLock```</span>

    /// <p>This enum was introduced or modified in  Qt 4.4.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QReadWriteLock::RecursionMode```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qreadwritelock.html#RecursionMode-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum was introduced or modified in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qreadwritelock.html#QReadWriteLock">QReadWriteLock</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct RecursionMode(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for RecursionMode {
        fn from(value: ::std::os::raw::c_int) -> Self {
            RecursionMode(value)
        }
    }

    impl From<RecursionMode> for ::std::os::raw::c_int {
        fn from(value: RecursionMode) -> Self {
            value.0
        }
    }

    impl RecursionMode {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl RecursionMode {
        /// In this mode, a thread may only lock a <a href="http://doc.qt.io/qt-5/qreadwritelock.html">QReadWriteLock</a> once. (C++ enum variant: <span style='color: green;'>```NonRecursive = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NonRecursive: crate::q_read_write_lock::RecursionMode =
            crate::q_read_write_lock::RecursionMode(0);
        /// In this mode, a thread can lock the same <a href="http://doc.qt.io/qt-5/qreadwritelock.html">QReadWriteLock</a> multiple times. The <a href="http://doc.qt.io/qt-5/qreadwritelock.html">QReadWriteLock</a> won't be unlocked until a corresponding number of <a href="http://doc.qt.io/qt-5/qreadwritelock.html#unlock">unlock</a>() calls have been made. (C++ enum variant: <span style='color: green;'>```Recursive = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Recursive: crate::q_read_write_lock::RecursionMode =
            crate::q_read_write_lock::RecursionMode(1);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qreadwritelock.html">QReadWriteLock</a> class provides read-write locking.</p>
///
/// C++ class: <span style='color: green;'>```QReadWriteLock```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qreadwritelock.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qreadwritelock.html">QReadWriteLock</a> class provides read-write locking.</p>
/// <p>A read-write lock is a synchronization tool for protecting resources that can be accessed for reading and writing. This type of lock is useful if you want to allow multiple threads to have simultaneous read-only access, but as soon as one thread wants to write to the resource, all other threads must be blocked until the writing is complete.</p>
/// <p>In many cases, <a href="http://doc.qt.io/qt-5/qreadwritelock.html">QReadWriteLock</a> is a direct competitor to <a href="http://doc.qt.io/qt-5/qmutex.html">QMutex</a>. <a href="http://doc.qt.io/qt-5/qreadwritelock.html">QReadWriteLock</a> is a good choice if there are many concurrent reads and writing occurs infrequently.</p>
/// <p>Example:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qreadwritelock.html#QReadWriteLock">QReadWriteLock</a></span> lock;
///
///   <span class="type">void</span> ReaderThread<span class="operator">::</span>run()
///   {
/// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
/// &#32;     lock<span class="operator">.</span>lockForRead();
/// &#32;     read_file();
/// &#32;     lock<span class="operator">.</span>unlock();
/// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   }
///
///   <span class="type">void</span> WriterThread<span class="operator">::</span>run()
///   {
/// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
/// &#32;     lock<span class="operator">.</span>lockForWrite();
/// &#32;     write_file();
/// &#32;     lock<span class="operator">.</span>unlock();
/// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   }
///
/// </pre>
/// <p>To ensure that writers aren't blocked forever by readers, readers attempting to obtain a lock will not succeed if there is a blocked writer waiting for access, even if the lock is currently only accessed by other readers. Also, if the lock is accessed by a writer and another writer comes in, that writer will have priority over any readers that might also be waiting.</p>
/// <p>Like <a href="http://doc.qt.io/qt-5/qmutex.html">QMutex</a>, a <a href="http://doc.qt.io/qt-5/qreadwritelock.html">QReadWriteLock</a> can be recursively locked by the same thread when constructed with <a href="http://doc.qt.io/qt-5/qreadwritelock.html#RecursionMode-enum">QReadWriteLock::Recursive</a> as <a href="http://doc.qt.io/qt-5/qreadwritelock.html#RecursionMode-enum">QReadWriteLock::RecursionMode</a>. In such cases, <a href="http://doc.qt.io/qt-5/qreadwritelock.html#unlock">unlock</a>() must be called the same number of times <a href="http://doc.qt.io/qt-5/qreadwritelock.html#lockForWrite">lockForWrite</a>() or <a href="http://doc.qt.io/qt-5/qreadwritelock.html#lockForRead">lockForRead</a>() was called. Note that the lock type cannot be changed when trying to lock recursively, i.e. it is not possible to lock for reading in a thread that already has locked for writing (and vice versa).</p></div>
#[repr(C)]
pub struct QReadWriteLock {
    _unused: u8,
}
impl QReadWriteLock {
    /// <p>Locks the lock for reading. This function will block the current thread if another thread has locked for writing.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QReadWriteLock::lockForRead()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qreadwritelock.html#lockForRead">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Locks the lock for reading. This function will block the current thread if another thread has locked for writing.</p>
    /// <p>It is not possible to lock for read if the thread already has locked for write.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qreadwritelock.html#unlock">unlock</a>(), <a href="http://doc.qt.io/qt-5/qreadwritelock.html#lockForWrite">lockForWrite</a>(), and <a href="http://doc.qt.io/qt-5/qreadwritelock.html#tryLockForRead">tryLockForRead</a>().</p></div>
    pub unsafe fn lock_for_read(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QReadWriteLock_lockForRead(self as *mut crate::QReadWriteLock)
    }

    /// <p>Locks the lock for writing. This function will block the current thread if another thread (including the current) has locked for reading or writing (unless the lock has been created using the <a href="http://doc.qt.io/qt-5/qreadwritelock.html#RecursionMode-enum">QReadWriteLock::Recursive</a> mode).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QReadWriteLock::lockForWrite()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qreadwritelock.html#lockForWrite">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Locks the lock for writing. This function will block the current thread if another thread (including the current) has locked for reading or writing (unless the lock has been created using the <a href="http://doc.qt.io/qt-5/qreadwritelock.html#RecursionMode-enum">QReadWriteLock::Recursive</a> mode).</p>
    /// <p>It is not possible to lock for write if the thread already has locked for read.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qreadwritelock.html#unlock">unlock</a>(), <a href="http://doc.qt.io/qt-5/qreadwritelock.html#lockForRead">lockForRead</a>(), and <a href="http://doc.qt.io/qt-5/qreadwritelock.html#tryLockForWrite">tryLockForWrite</a>().</p></div>
    pub unsafe fn lock_for_write(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QReadWriteLock_lockForWrite(
            self as *mut crate::QReadWriteLock,
        )
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qreadwritelock.html">QReadWriteLock</a> object in the given <i>recursionMode</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QReadWriteLock::QReadWriteLock(QReadWriteLock::RecursionMode recursionMode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qreadwritelock.html#QReadWriteLock">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qreadwritelock.html">QReadWriteLock</a> object in the given <i>recursionMode</i>.</p>
    /// <p>The default recursion mode is <a href="http://doc.qt.io/qt-5/qreadwritelock.html#RecursionMode-enum">NonRecursive</a>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qreadwritelock.html#lockForRead">lockForRead</a>(), <a href="http://doc.qt.io/qt-5/qreadwritelock.html#lockForWrite">lockForWrite</a>(), and <a href="http://doc.qt.io/qt-5/qreadwritelock.html#RecursionMode-enum">RecursionMode</a>.</p></div>
    pub unsafe fn new_1a(
        recursion_mode: crate::q_read_write_lock::RecursionMode,
    ) -> ::cpp_utils::CppBox<crate::QReadWriteLock> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QReadWriteLock_QReadWriteLock(recursion_mode);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qreadwritelock.html">QReadWriteLock</a> class provides read-write locking.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QReadWriteLock::QReadWriteLock()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qreadwritelock.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qreadwritelock.html">QReadWriteLock</a> class provides read-write locking.</p>
    /// <p>A read-write lock is a synchronization tool for protecting resources that can be accessed for reading and writing. This type of lock is useful if you want to allow multiple threads to have simultaneous read-only access, but as soon as one thread wants to write to the resource, all other threads must be blocked until the writing is complete.</p>
    /// <p>In many cases, <a href="http://doc.qt.io/qt-5/qreadwritelock.html">QReadWriteLock</a> is a direct competitor to <a href="http://doc.qt.io/qt-5/qmutex.html">QMutex</a>. <a href="http://doc.qt.io/qt-5/qreadwritelock.html">QReadWriteLock</a> is a good choice if there are many concurrent reads and writing occurs infrequently.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qreadwritelock.html#QReadWriteLock">QReadWriteLock</a></span> lock;
    ///
    ///   <span class="type">void</span> ReaderThread<span class="operator">::</span>run()
    ///   {
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    /// &#32;     lock<span class="operator">.</span>lockForRead();
    /// &#32;     read_file();
    /// &#32;     lock<span class="operator">.</span>unlock();
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   }
    ///
    ///   <span class="type">void</span> WriterThread<span class="operator">::</span>run()
    ///   {
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    /// &#32;     lock<span class="operator">.</span>lockForWrite();
    /// &#32;     write_file();
    /// &#32;     lock<span class="operator">.</span>unlock();
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   }
    ///
    /// </pre>
    /// <p>To ensure that writers aren't blocked forever by readers, readers attempting to obtain a lock will not succeed if there is a blocked writer waiting for access, even if the lock is currently only accessed by other readers. Also, if the lock is accessed by a writer and another writer comes in, that writer will have priority over any readers that might also be waiting.</p>
    /// <p>Like <a href="http://doc.qt.io/qt-5/qmutex.html">QMutex</a>, a <a href="http://doc.qt.io/qt-5/qreadwritelock.html">QReadWriteLock</a> can be recursively locked by the same thread when constructed with <a href="http://doc.qt.io/qt-5/qreadwritelock.html#RecursionMode-enum">QReadWriteLock::Recursive</a> as <a href="http://doc.qt.io/qt-5/qreadwritelock.html#RecursionMode-enum">QReadWriteLock::RecursionMode</a>. In such cases, <a href="http://doc.qt.io/qt-5/qreadwritelock.html#unlock">unlock</a>() must be called the same number of times <a href="http://doc.qt.io/qt-5/qreadwritelock.html#lockForWrite">lockForWrite</a>() or <a href="http://doc.qt.io/qt-5/qreadwritelock.html#lockForRead">lockForRead</a>() was called. Note that the lock type cannot be changed when trying to lock recursively, i.e. it is not possible to lock for reading in a thread that already has locked for writing (and vice versa).</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QReadWriteLock> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QReadWriteLock_QReadWriteLock1();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Attempts to lock for reading. If the lock was obtained, this function returns <code>true</code>, otherwise it returns <code>false</code> instead of waiting for the lock to become available, i.e. it does not block.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QReadWriteLock::tryLockForRead()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qreadwritelock.html#tryLockForRead">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to lock for reading. If the lock was obtained, this function returns <code>true</code>, otherwise it returns <code>false</code> instead of waiting for the lock to become available, i.e. it does not block.</p>
    /// <p>The lock attempt will fail if another thread has locked for writing.</p>
    /// <p>If the lock was obtained, the lock must be unlocked with <a href="http://doc.qt.io/qt-5/qreadwritelock.html#unlock">unlock</a>() before another thread can successfully lock it for writing.</p>
    /// <p>It is not possible to lock for read if the thread already has locked for write.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qreadwritelock.html#unlock">unlock</a>() and <a href="http://doc.qt.io/qt-5/qreadwritelock.html#lockForRead">lockForRead</a>().</p></div>
    pub unsafe fn try_lock_for_read_0a(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QReadWriteLock_tryLockForRead(
            self as *mut crate::QReadWriteLock,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QReadWriteLock::tryLockForRead(int timeout)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qreadwritelock.html#tryLockForRead-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Attempts to lock for reading. This function returns <code>true</code> if the lock was obtained; otherwise it returns <code>false</code>. If another thread has locked for writing, this function will wait for at most <i>timeout</i> milliseconds for the lock to become available.</p>
    /// <p>Note: Passing a negative number as the <i>timeout</i> is equivalent to calling <a href="http://doc.qt.io/qt-5/qreadwritelock.html#lockForRead">lockForRead</a>(), i.e. this function will wait forever until lock can be locked for reading when <i>timeout</i> is negative.</p>
    /// <p>If the lock was obtained, the lock must be unlocked with <a href="http://doc.qt.io/qt-5/qreadwritelock.html#unlock">unlock</a>() before another thread can successfully lock it for writing.</p>
    /// <p>It is not possible to lock for read if the thread already has locked for write.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qreadwritelock.html#unlock">unlock</a>() and <a href="http://doc.qt.io/qt-5/qreadwritelock.html#lockForRead">lockForRead</a>().</p></div>
    pub unsafe fn try_lock_for_read_1a(&mut self, timeout: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QReadWriteLock_tryLockForRead1(
            self as *mut crate::QReadWriteLock,
            timeout,
        )
    }

    /// <p>Attempts to lock for writing. If the lock was obtained, this function returns <code>true</code>; otherwise, it returns <code>false</code> immediately.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QReadWriteLock::tryLockForWrite()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qreadwritelock.html#tryLockForWrite">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to lock for writing. If the lock was obtained, this function returns <code>true</code>; otherwise, it returns <code>false</code> immediately.</p>
    /// <p>The lock attempt will fail if another thread has locked for reading or writing.</p>
    /// <p>If the lock was obtained, the lock must be unlocked with <a href="http://doc.qt.io/qt-5/qreadwritelock.html#unlock">unlock</a>() before another thread can successfully lock it.</p>
    /// <p>It is not possible to lock for write if the thread already has locked for read.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qreadwritelock.html#unlock">unlock</a>() and <a href="http://doc.qt.io/qt-5/qreadwritelock.html#lockForWrite">lockForWrite</a>().</p></div>
    pub unsafe fn try_lock_for_write_0a(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QReadWriteLock_tryLockForWrite(
            self as *mut crate::QReadWriteLock,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QReadWriteLock::tryLockForWrite(int timeout)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qreadwritelock.html#tryLockForWrite-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Attempts to lock for writing. This function returns <code>true</code> if the lock was obtained; otherwise it returns <code>false</code>. If another thread has locked for reading or writing, this function will wait for at most <i>timeout</i> milliseconds for the lock to become available.</p>
    /// <p>Note: Passing a negative number as the <i>timeout</i> is equivalent to calling <a href="http://doc.qt.io/qt-5/qreadwritelock.html#lockForWrite">lockForWrite</a>(), i.e. this function will wait forever until lock can be locked for writing when <i>timeout</i> is negative.</p>
    /// <p>If the lock was obtained, the lock must be unlocked with <a href="http://doc.qt.io/qt-5/qreadwritelock.html#unlock">unlock</a>() before another thread can successfully lock it.</p>
    /// <p>It is not possible to lock for write if the thread already has locked for read.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qreadwritelock.html#unlock">unlock</a>() and <a href="http://doc.qt.io/qt-5/qreadwritelock.html#lockForWrite">lockForWrite</a>().</p></div>
    pub unsafe fn try_lock_for_write_1a(&mut self, timeout: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QReadWriteLock_tryLockForWrite1(
            self as *mut crate::QReadWriteLock,
            timeout,
        )
    }

    /// <p>Unlocks the lock.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QReadWriteLock::unlock()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qreadwritelock.html#unlock">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Unlocks the lock.</p>
    /// <p>Attempting to unlock a lock that is not locked is an error, and will result in program termination.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qreadwritelock.html#lockForRead">lockForRead</a>(), <a href="http://doc.qt.io/qt-5/qreadwritelock.html#lockForWrite">lockForWrite</a>(), <a href="http://doc.qt.io/qt-5/qreadwritelock.html#tryLockForRead">tryLockForRead</a>(), and <a href="http://doc.qt.io/qt-5/qreadwritelock.html#tryLockForWrite">tryLockForWrite</a>().</p></div>
    pub unsafe fn unlock(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QReadWriteLock_unlock(self as *mut crate::QReadWriteLock)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qreadlocker.html">QReadLocker</a> class is a convenience class that simplifies locking and unlocking read-write locks for read access.</p>
///
/// C++ class: <span style='color: green;'>```QReadLocker```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qreadlocker.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qreadlocker.html">QReadLocker</a> class is a convenience class that simplifies locking and unlocking read-write locks for read access.</p>
/// <p>The purpose of <a href="http://doc.qt.io/qt-5/qreadlocker.html">QReadLocker</a> (and <a href="http://doc.qt.io/qt-5/qwritelocker.html">QWriteLocker</a>) is to simplify <a href="http://doc.qt.io/qt-5/qreadwritelock.html">QReadWriteLock</a> locking and unlocking. Locking and unlocking statements or in exception handling code is error-prone and difficult to debug. <a href="http://doc.qt.io/qt-5/qreadlocker.html">QReadLocker</a> can be used in such situations to ensure that the state of the lock is always well-defined.</p>
/// <p>Here's an example that uses <a href="http://doc.qt.io/qt-5/qreadlocker.html">QReadLocker</a> to lock and unlock a read-write lock for reading:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qreadwritelock.html">QReadWriteLock</a></span> lock;
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span> readData()
///   {
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qreadlocker.html#QReadLocker">QReadLocker</a></span> locker(<span class="operator">&amp;</span>lock);
/// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
/// &#32;     <span class="keyword">return</span> data;
///   }
///
/// </pre>
/// <p>It is equivalent to the following code:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qreadwritelock.html">QReadWriteLock</a></span> lock;
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span> readData()
///   {
/// &#32;     lock<span class="operator">.</span>lockForRead();
/// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
/// &#32;     lock<span class="operator">.</span>unlock();
/// &#32;     <span class="keyword">return</span> data;
///   }
///
/// </pre>
/// <p>The <a href="http://doc.qt.io/qt-5/qmutexlocker.html">QMutexLocker</a> documentation shows examples where the use of a locker object greatly simplifies programming.</p></div>
#[repr(C)]
pub struct QReadLocker {
    _unused: u8,
}
impl QReadLocker {
    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qreadlocker.html">QReadLocker</a> and locks <i>lock</i> for reading. The lock will be unlocked when the <a href="http://doc.qt.io/qt-5/qreadlocker.html">QReadLocker</a> is destroyed. If <code>lock</code> is zero, <a href="http://doc.qt.io/qt-5/qreadlocker.html">QReadLocker</a> does nothing.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QReadLocker::QReadLocker(QReadWriteLock* readWriteLock)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qreadlocker.html#QReadLocker">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qreadlocker.html">QReadLocker</a> and locks <i>lock</i> for reading. The lock will be unlocked when the <a href="http://doc.qt.io/qt-5/qreadlocker.html">QReadLocker</a> is destroyed. If <code>lock</code> is zero, <a href="http://doc.qt.io/qt-5/qreadlocker.html">QReadLocker</a> does nothing.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qreadwritelock.html#lockForRead">QReadWriteLock::lockForRead</a>().</p></div>
    pub unsafe fn new(
        read_write_lock: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QReadWriteLock>>,
    ) -> ::cpp_utils::CppBox<crate::QReadLocker> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QReadLocker_QReadLocker(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QReadWriteLock>>::cast_into(
                read_write_lock,
            )
            .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a pointer to the read-write lock that was passed to the constructor.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QReadWriteLock* QReadLocker::readWriteLock() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qreadlocker.html#readWriteLock">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the read-write lock that was passed to the constructor.</p></div>
    pub unsafe fn read_write_lock(&self) -> ::cpp_utils::MutPtr<crate::QReadWriteLock> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QReadLocker_readWriteLock(
            self as *const crate::QReadLocker,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Relocks an unlocked lock.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QReadLocker::relock()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qreadlocker.html#relock">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Relocks an unlocked lock.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qreadlocker.html#unlock">unlock</a>().</p></div>
    pub unsafe fn relock(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QReadLocker_relock(self as *mut crate::QReadLocker)
    }

    /// <p>Unlocks the lock associated with this locker.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QReadLocker::unlock()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qreadlocker.html#unlock">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Unlocks the lock associated with this locker.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qreadwritelock.html#unlock">QReadWriteLock::unlock</a>().</p></div>
    pub unsafe fn unlock(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QReadLocker_unlock(self as *mut crate::QReadLocker)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qwritelocker.html">QWriteLocker</a> class is a convenience class that simplifies locking and unlocking read-write locks for write access.</p>
///
/// C++ class: <span style='color: green;'>```QWriteLocker```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qwritelocker.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qwritelocker.html">QWriteLocker</a> class is a convenience class that simplifies locking and unlocking read-write locks for write access.</p>
/// <p>The purpose of <a href="http://doc.qt.io/qt-5/qwritelocker.html">QWriteLocker</a> (and <a href="http://doc.qt.io/qt-5/qreadlocker.html">QReadLocker</a>) is to simplify <a href="http://doc.qt.io/qt-5/qreadwritelock.html">QReadWriteLock</a> locking and unlocking. Locking and unlocking statements or in exception handling code is error-prone and difficult to debug. <a href="http://doc.qt.io/qt-5/qwritelocker.html">QWriteLocker</a> can be used in such situations to ensure that the state of the lock is always well-defined.</p>
/// <p>Here's an example that uses <a href="http://doc.qt.io/qt-5/qwritelocker.html">QWriteLocker</a> to lock and unlock a read-write lock for writing:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qreadwritelock.html">QReadWriteLock</a></span> lock;
///
///   <span class="type">void</span> writeData(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span> <span class="operator">&amp;</span>data)
///   {
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qwritelocker.html#QWriteLocker">QWriteLocker</a></span> locker(<span class="operator">&amp;</span>lock);
/// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   }
///
/// </pre>
/// <p>It is equivalent to the following code:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qreadwritelock.html">QReadWriteLock</a></span> lock;
///
///   <span class="type">void</span> writeData(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span> <span class="operator">&amp;</span>data)
///   {
/// &#32;     lock<span class="operator">.</span>lockForWrite();
/// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
/// &#32;     lock<span class="operator">.</span>unlock();
///   }
///
/// </pre>
/// <p>The <a href="http://doc.qt.io/qt-5/qmutexlocker.html">QMutexLocker</a> documentation shows examples where the use of a locker object greatly simplifies programming.</p></div>
#[repr(C)]
pub struct QWriteLocker {
    _unused: u8,
}
impl QWriteLocker {
    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qwritelocker.html">QWriteLocker</a> and locks <i>lock</i> for writing. The lock will be unlocked when the <a href="http://doc.qt.io/qt-5/qwritelocker.html">QWriteLocker</a> is destroyed. If <code>lock</code> is zero, <a href="http://doc.qt.io/qt-5/qwritelocker.html">QWriteLocker</a> does nothing.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QWriteLocker::QWriteLocker(QReadWriteLock* readWriteLock)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qwritelocker.html#QWriteLocker">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qwritelocker.html">QWriteLocker</a> and locks <i>lock</i> for writing. The lock will be unlocked when the <a href="http://doc.qt.io/qt-5/qwritelocker.html">QWriteLocker</a> is destroyed. If <code>lock</code> is zero, <a href="http://doc.qt.io/qt-5/qwritelocker.html">QWriteLocker</a> does nothing.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qreadwritelock.html#lockForWrite">QReadWriteLock::lockForWrite</a>().</p></div>
    pub unsafe fn new(
        read_write_lock: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QReadWriteLock>>,
    ) -> ::cpp_utils::CppBox<crate::QWriteLocker> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QWriteLocker_QWriteLocker(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QReadWriteLock>>::cast_into(
                read_write_lock,
            )
            .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a pointer to the read-write lock that was passed to the constructor.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QReadWriteLock* QWriteLocker::readWriteLock() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qwritelocker.html#readWriteLock">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the read-write lock that was passed to the constructor.</p></div>
    pub unsafe fn read_write_lock(&self) -> ::cpp_utils::MutPtr<crate::QReadWriteLock> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QWriteLocker_readWriteLock(
            self as *const crate::QWriteLocker,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Relocks an unlocked lock.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QWriteLocker::relock()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qwritelocker.html#relock">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Relocks an unlocked lock.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qwritelocker.html#unlock">unlock</a>().</p></div>
    pub unsafe fn relock(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QWriteLocker_relock(self as *mut crate::QWriteLocker)
    }

    /// <p>Unlocks the lock associated with this locker.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QWriteLocker::unlock()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qwritelocker.html#unlock">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Unlocks the lock associated with this locker.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qreadwritelock.html#unlock">QReadWriteLock::unlock</a>().</p></div>
    pub unsafe fn unlock(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QWriteLocker_unlock(self as *mut crate::QWriteLocker)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a> class defines the size of a two-dimensional object using integer point precision.</p>
///
/// C++ class: <span style='color: green;'>```QSize```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qsize.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a> class defines the size of a two-dimensional object using integer point precision.</p>
/// <p>A size is specified by a <a href="http://doc.qt.io/qt-5/qsize.html#width">width</a>() and a <a href="http://doc.qt.io/qt-5/qsize.html#height">height</a>(). It can be set in the constructor and changed using the <a href="http://doc.qt.io/qt-5/qsize.html#setWidth">setWidth</a>(), <a href="http://doc.qt.io/qt-5/qsize.html#setHeight">setHeight</a>(), or <a href="http://doc.qt.io/qt-5/qsize.html#scale">scale</a>() functions, or using arithmetic operators. A size can also be manipulated directly by retrieving references to the width and height using the <a href="http://doc.qt.io/qt-5/qsize.html#rwidth">rwidth</a>() and <a href="http://doc.qt.io/qt-5/qsize.html#rheight">rheight</a>() functions. Finally, the width and height can be swapped using the <a href="http://doc.qt.io/qt-5/qsize.html#transpose">transpose</a>() function.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qsize.html#isValid">isValid</a>() function determines if a size is valid (a valid size has both width and height greater than or equal to zero). The <a href="http://doc.qt.io/qt-5/qsize.html#isEmpty">isEmpty</a>() function returns <code>true</code> if either of the width and height is less than, or equal to, zero, while the <a href="http://doc.qt.io/qt-5/qsize.html#isNull">isNull</a>() function returns <code>true</code> only if both the width and the height is zero.</p>
/// <p>Use the <a href="http://doc.qt.io/qt-5/qsize.html#expandedTo">expandedTo</a>() function to retrieve a size which holds the maximum height and width of <i>this</i> size and a given size. Similarly, the <a href="http://doc.qt.io/qt-5/qsize.html#boundedTo">boundedTo</a>() function returns a size which holds the minimum height and width of <i>this</i> size and a given size.</p>
/// <p><a href="http://doc.qt.io/qt-5/qsize.html">QSize</a> objects can be streamed as well as compared.</p></div>
#[repr(C)]
pub struct QSize {
    _unused: u8,
}
impl QSize {
    /// <p>Returns a size holding the minimum width and height of this size and the given <i>otherSize</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSize QSize::boundedTo(const QSize& arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsize.html#boundedTo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a size holding the minimum width and height of this size and the given <i>otherSize</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsize.html#expandedTo">expandedTo</a>() and <a href="http://doc.qt.io/qt-5/qsize.html#scale">scale</a>().</p></div>
    pub unsafe fn bounded_to(
        &self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSize>>,
    ) -> ::cpp_utils::CppBox<crate::QSize> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSize_boundedTo(
            self as *const crate::QSize,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSize>>::cast_into(arg1).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a> class defines the size of a two-dimensional object using integer point precision.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSize& QSize::operator=(const QSize& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsize.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a> class defines the size of a two-dimensional object using integer point precision.</p>
    /// <p>A size is specified by a <a href="http://doc.qt.io/qt-5/qsize.html#width">width</a>() and a <a href="http://doc.qt.io/qt-5/qsize.html#height">height</a>(). It can be set in the constructor and changed using the <a href="http://doc.qt.io/qt-5/qsize.html#setWidth">setWidth</a>(), <a href="http://doc.qt.io/qt-5/qsize.html#setHeight">setHeight</a>(), or <a href="http://doc.qt.io/qt-5/qsize.html#scale">scale</a>() functions, or using arithmetic operators. A size can also be manipulated directly by retrieving references to the width and height using the <a href="http://doc.qt.io/qt-5/qsize.html#rwidth">rwidth</a>() and <a href="http://doc.qt.io/qt-5/qsize.html#rheight">rheight</a>() functions. Finally, the width and height can be swapped using the <a href="http://doc.qt.io/qt-5/qsize.html#transpose">transpose</a>() function.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qsize.html#isValid">isValid</a>() function determines if a size is valid (a valid size has both width and height greater than or equal to zero). The <a href="http://doc.qt.io/qt-5/qsize.html#isEmpty">isEmpty</a>() function returns <code>true</code> if either of the width and height is less than, or equal to, zero, while the <a href="http://doc.qt.io/qt-5/qsize.html#isNull">isNull</a>() function returns <code>true</code> only if both the width and the height is zero.</p>
    /// <p>Use the <a href="http://doc.qt.io/qt-5/qsize.html#expandedTo">expandedTo</a>() function to retrieve a size which holds the maximum height and width of <i>this</i> size and a given size. Similarly, the <a href="http://doc.qt.io/qt-5/qsize.html#boundedTo">boundedTo</a>() function returns a size which holds the minimum height and width of <i>this</i> size and a given size.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qsize.html">QSize</a> objects can be streamed as well as compared.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSize>>,
    ) -> ::cpp_utils::MutRef<crate::QSize> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSize_operator_(
            self as *mut crate::QSize,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSize>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a size holding the maximum width and height of this size and the given <i>otherSize</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSize QSize::expandedTo(const QSize& arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsize.html#expandedTo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a size holding the maximum width and height of this size and the given <i>otherSize</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsize.html#boundedTo">boundedTo</a>() and <a href="http://doc.qt.io/qt-5/qsize.html#scale">scale</a>().</p></div>
    pub unsafe fn expanded_to(
        &self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSize>>,
    ) -> ::cpp_utils::CppBox<crate::QSize> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSize_expandedTo(
            self as *const crate::QSize,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSize>>::cast_into(arg1).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the height.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QSize::height() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsize.html#height">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the height.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsize.html#width">width</a>() and <a href="http://doc.qt.io/qt-5/qsize.html#setHeight">setHeight</a>().</p></div>
    pub unsafe fn height(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QSize_height(self as *const crate::QSize)
    }

    /// <p>Returns <code>true</code> if either of the width and height is less than or equal to 0; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSize::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsize.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if either of the width and height is less than or equal to 0; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsize.html#isNull">isNull</a>() and <a href="http://doc.qt.io/qt-5/qsize.html#isValid">isValid</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSize_isEmpty(self as *const crate::QSize)
    }

    /// <p>Returns <code>true</code> if both the width and height is 0; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSize::isNull() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsize.html#isNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if both the width and height is 0; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsize.html#isValid">isValid</a>() and <a href="http://doc.qt.io/qt-5/qsize.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSize_isNull(self as *const crate::QSize)
    }

    /// <p>Returns <code>true</code> if both the width and height is equal to or greater than 0; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSize::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsize.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if both the width and height is equal to or greater than 0; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsize.html#isNull">isNull</a>() and <a href="http://doc.qt.io/qt-5/qsize.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn is_valid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSize_isValid(self as *const crate::QSize)
    }

    /// <p>Constructs a size with an invalid width and height (i.e., <a href="http://doc.qt.io/qt-5/qsize.html#isValid">isValid</a>() returns <code>false</code>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSize::QSize()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsize.html#QSize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a size with an invalid width and height (i.e., <a href="http://doc.qt.io/qt-5/qsize.html#isValid">isValid</a>() returns <code>false</code>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsize.html#isValid">isValid</a>().</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QSize> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSize_QSize();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a size with the given <i>width</i> and <i>height</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSize::QSize(int w, int h)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsize.html#QSize-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a size with the given <i>width</i> and <i>height</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsize.html#setWidth">setWidth</a>() and <a href="http://doc.qt.io/qt-5/qsize.html#setHeight">setHeight</a>().</p></div>
    pub unsafe fn new_2a(
        w: ::std::os::raw::c_int,
        h: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QSize> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSize_QSize1(w, h);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a> class defines the size of a two-dimensional object using integer point precision.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSize::QSize(const QSize& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsize.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a> class defines the size of a two-dimensional object using integer point precision.</p>
    /// <p>A size is specified by a <a href="http://doc.qt.io/qt-5/qsize.html#width">width</a>() and a <a href="http://doc.qt.io/qt-5/qsize.html#height">height</a>(). It can be set in the constructor and changed using the <a href="http://doc.qt.io/qt-5/qsize.html#setWidth">setWidth</a>(), <a href="http://doc.qt.io/qt-5/qsize.html#setHeight">setHeight</a>(), or <a href="http://doc.qt.io/qt-5/qsize.html#scale">scale</a>() functions, or using arithmetic operators. A size can also be manipulated directly by retrieving references to the width and height using the <a href="http://doc.qt.io/qt-5/qsize.html#rwidth">rwidth</a>() and <a href="http://doc.qt.io/qt-5/qsize.html#rheight">rheight</a>() functions. Finally, the width and height can be swapped using the <a href="http://doc.qt.io/qt-5/qsize.html#transpose">transpose</a>() function.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qsize.html#isValid">isValid</a>() function determines if a size is valid (a valid size has both width and height greater than or equal to zero). The <a href="http://doc.qt.io/qt-5/qsize.html#isEmpty">isEmpty</a>() function returns <code>true</code> if either of the width and height is less than, or equal to, zero, while the <a href="http://doc.qt.io/qt-5/qsize.html#isNull">isNull</a>() function returns <code>true</code> only if both the width and the height is zero.</p>
    /// <p>Use the <a href="http://doc.qt.io/qt-5/qsize.html#expandedTo">expandedTo</a>() function to retrieve a size which holds the maximum height and width of <i>this</i> size and a given size. Similarly, the <a href="http://doc.qt.io/qt-5/qsize.html#boundedTo">boundedTo</a>() function returns a size which holds the minimum height and width of <i>this</i> size and a given size.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qsize.html">QSize</a> objects can be streamed as well as compared.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSize>>,
    ) -> ::cpp_utils::CppBox<crate::QSize> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSize_QSize2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSize>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the height.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int& QSize::rheight()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsize.html#rheight">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the height.</p>
    /// <p>Using a reference makes it possible to manipulate the height directly. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsize.html#QSize">QSize</a></span> size(<span class="number">100</span><span class="operator">,</span> <span class="number">10</span>);
    ///   size<span class="operator">.</span>rheight() <span class="operator">+</span><span class="operator">=</span> <span class="number">5</span>;
    ///
    ///   <span class="comment">// size becomes (100,15)</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsize.html#rwidth">rwidth</a>() and <a href="http://doc.qt.io/qt-5/qsize.html#setHeight">setHeight</a>().</p></div>
    pub unsafe fn rheight(&mut self) -> ::cpp_utils::MutRef<::std::os::raw::c_int> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSize_rheight(self as *mut crate::QSize);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the width.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int& QSize::rwidth()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsize.html#rwidth">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the width.</p>
    /// <p>Using a reference makes it possible to manipulate the width directly. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsize.html#QSize">QSize</a></span> size(<span class="number">100</span><span class="operator">,</span> <span class="number">10</span>);
    ///   size<span class="operator">.</span>rwidth() <span class="operator">+</span><span class="operator">=</span> <span class="number">20</span>;
    ///
    ///   <span class="comment">// size becomes (120,10)</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsize.html#rheight">rheight</a>() and <a href="http://doc.qt.io/qt-5/qsize.html#setWidth">setWidth</a>().</p></div>
    pub unsafe fn rwidth(&mut self) -> ::cpp_utils::MutRef<::std::os::raw::c_int> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSize_rwidth(self as *mut crate::QSize);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Scales the size to a rectangle with the given <i>width</i> and <i>height</i>, according to the specified <i>mode</i>:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSize::scale(int w, int h, Qt::AspectRatioMode mode)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsize.html#scale">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Scales the size to a rectangle with the given <i>width</i> and <i>height</i>, according to the specified <i>mode</i>:</p>
    /// <ul>
    /// <li>If <i>mode</i> is <a href="http://doc.qt.io/qt-5/qt.html#AspectRatioMode-enum">Qt::IgnoreAspectRatio</a>, the size is set to (<i>width</i>, <i>height</i>).</li>
    /// <li>If <i>mode</i> is <a href="http://doc.qt.io/qt-5/qt.html#AspectRatioMode-enum">Qt::KeepAspectRatio</a>, the current size is scaled to a rectangle as large as possible inside (<i>width</i>, <i>height</i>), preserving the aspect ratio.</li>
    /// <li>If <i>mode</i> is <a href="http://doc.qt.io/qt-5/qt.html#AspectRatioMode-enum">Qt::KeepAspectRatioByExpanding</a>, the current size is scaled to a rectangle as small as possible outside (<i>width</i>, <i>height</i>), preserving the aspect ratio.</li>
    /// </ul>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsize.html#QSize">QSize</a></span> t1(<span class="number">10</span><span class="operator">,</span> <span class="number">12</span>);
    ///   t1<span class="operator">.</span>scale(<span class="number">60</span><span class="operator">,</span> <span class="number">60</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>IgnoreAspectRatio);
    ///   <span class="comment">// t1 is (60, 60)</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsize.html#QSize">QSize</a></span> t2(<span class="number">10</span><span class="operator">,</span> <span class="number">12</span>);
    ///   t2<span class="operator">.</span>scale(<span class="number">60</span><span class="operator">,</span> <span class="number">60</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>KeepAspectRatio);
    ///   <span class="comment">// t2 is (50, 60)</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsize.html#QSize">QSize</a></span> t3(<span class="number">10</span><span class="operator">,</span> <span class="number">12</span>);
    ///   t3<span class="operator">.</span>scale(<span class="number">60</span><span class="operator">,</span> <span class="number">60</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>KeepAspectRatioByExpanding);
    ///   <span class="comment">// t3 is (60, 72)</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsize.html#setWidth">setWidth</a>(), <a href="http://doc.qt.io/qt-5/qsize.html#setHeight">setHeight</a>(), and <a href="http://doc.qt.io/qt-5/qsize.html#scaled">scaled</a>().</p></div>
    pub unsafe fn scale_3a(
        &mut self,
        w: ::std::os::raw::c_int,
        h: ::std::os::raw::c_int,
        mode: crate::AspectRatioMode,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSize_scale(self as *mut crate::QSize, w, h, mode)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSize::scale(const QSize& s, Qt::AspectRatioMode mode)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsize.html#scale-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Scales the size to a rectangle with the given <i>size</i>, according to the specified <i>mode</i>.</p></div>
    pub unsafe fn scale_2a(
        &mut self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSize>>,
        mode: crate::AspectRatioMode,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSize_scale1(
            self as *mut crate::QSize,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSize>>::cast_into(s).as_raw_ptr(),
            mode,
        )
    }

    /// <p>Return a size scaled to a rectangle with the given <i>width</i> and <i>height</i>, according to the specified <i>mode</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSize QSize::scaled(int w, int h, Qt::AspectRatioMode mode) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsize.html#scaled">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Return a size scaled to a rectangle with the given <i>width</i> and <i>height</i>, according to the specified <i>mode</i>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsize.html#scale">scale</a>().</p></div>
    pub unsafe fn scaled_3a(
        &self,
        w: ::std::os::raw::c_int,
        h: ::std::os::raw::c_int,
        mode: crate::AspectRatioMode,
    ) -> ::cpp_utils::CppBox<crate::QSize> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QSize_scaled(self as *const crate::QSize, w, h, mode);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSize QSize::scaled(const QSize& s, Qt::AspectRatioMode mode) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsize.html#scaled-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Return a size scaled to a rectangle with the given size <i>s</i>, according to the specified <i>mode</i>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn scaled_2a(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSize>>,
        mode: crate::AspectRatioMode,
    ) -> ::cpp_utils::CppBox<crate::QSize> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSize_scaled1(
            self as *const crate::QSize,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSize>>::cast_into(s).as_raw_ptr(),
            mode,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Sets the height to the given <i>height</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSize::setHeight(int h)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsize.html#setHeight">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the height to the given <i>height</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsize.html#rheight">rheight</a>(), <a href="http://doc.qt.io/qt-5/qsize.html#height">height</a>(), and <a href="http://doc.qt.io/qt-5/qsize.html#setWidth">setWidth</a>().</p></div>
    pub unsafe fn set_height(&mut self, h: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QSize_setHeight(self as *mut crate::QSize, h)
    }

    /// <p>Sets the width to the given <i>width</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSize::setWidth(int w)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsize.html#setWidth">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the width to the given <i>width</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsize.html#rwidth">rwidth</a>(), <a href="http://doc.qt.io/qt-5/qsize.html#width">width</a>(), and <a href="http://doc.qt.io/qt-5/qsize.html#setHeight">setHeight</a>().</p></div>
    pub unsafe fn set_width(&mut self, w: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QSize_setWidth(self as *mut crate::QSize, w)
    }

    /// <p>Swaps the width and height values.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSize::transpose()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsize.html#transpose">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps the width and height values.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsize.html#setWidth">setWidth</a>(), <a href="http://doc.qt.io/qt-5/qsize.html#setHeight">setHeight</a>(), and <a href="http://doc.qt.io/qt-5/qsize.html#transposed">transposed</a>().</p></div>
    pub unsafe fn transpose(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSize_transpose(self as *mut crate::QSize)
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a> with width and height swapped.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSize QSize::transposed() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsize.html#transposed">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a> with width and height swapped.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsize.html#transpose">transpose</a>().</p></div>
    pub unsafe fn transposed(&self) -> ::cpp_utils::CppBox<crate::QSize> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QSize_transposed(self as *const crate::QSize);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the width.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QSize::width() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsize.html#width">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the width.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsize.html#height">height</a>() and <a href="http://doc.qt.io/qt-5/qsize.html#setWidth">setWidth</a>().</p></div>
    pub unsafe fn width(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QSize_width(self as *const crate::QSize)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a> class defines the size of a two-dimensional object using floating point precision.</p>
///
/// C++ class: <span style='color: green;'>```QSizeF```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qsizef.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a> class defines the size of a two-dimensional object using floating point precision.</p>
/// <p>A size is specified by a <a href="http://doc.qt.io/qt-5/qsizef.html#width">width</a>() and a <a href="http://doc.qt.io/qt-5/qsizef.html#height">height</a>(). It can be set in the constructor and changed using the <a href="http://doc.qt.io/qt-5/qsizef.html#setWidth">setWidth</a>(), <a href="http://doc.qt.io/qt-5/qsizef.html#setHeight">setHeight</a>(), or <a href="http://doc.qt.io/qt-5/qsizef.html#scale">scale</a>() functions, or using arithmetic operators. A size can also be manipulated directly by retrieving references to the width and height using the <a href="http://doc.qt.io/qt-5/qsizef.html#rwidth">rwidth</a>() and <a href="http://doc.qt.io/qt-5/qsizef.html#rheight">rheight</a>() functions. Finally, the width and height can be swapped using the <a href="http://doc.qt.io/qt-5/qsizef.html#transpose">transpose</a>() function.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qsizef.html#isValid">isValid</a>() function determines if a size is valid. A valid size has both width and height greater than or equal to zero. The <a href="http://doc.qt.io/qt-5/qsizef.html#isEmpty">isEmpty</a>() function returns <code>true</code> if either of the width and height is <i>less</i> than (or equal to) zero, while the <a href="http://doc.qt.io/qt-5/qsizef.html#isNull">isNull</a>() function returns <code>true</code> only if both the width and the height is zero.</p>
/// <p>Use the <a href="http://doc.qt.io/qt-5/qsizef.html#expandedTo">expandedTo</a>() function to retrieve a size which holds the maximum height and width of this size and a given size. Similarly, the <a href="http://doc.qt.io/qt-5/qsizef.html#boundedTo">boundedTo</a>() function returns a size which holds the minimum height and width of this size and a given size.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a> class also provides the <a href="http://doc.qt.io/qt-5/qsizef.html#toSize">toSize</a>() function returning a <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a> copy of this size, constructed by rounding the width and height to the nearest integers.</p>
/// <p><a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a> objects can be streamed as well as compared.</p></div>
#[repr(C)]
pub struct QSizeF {
    _unused: u8,
}
impl QSizeF {
    /// <p>Returns a size holding the minimum width and height of this size and the given <i>otherSize</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSizeF QSizeF::boundedTo(const QSizeF& arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html#boundedTo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a size holding the minimum width and height of this size and the given <i>otherSize</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsizef.html#expandedTo">expandedTo</a>() and <a href="http://doc.qt.io/qt-5/qsizef.html#scale">scale</a>().</p></div>
    pub unsafe fn bounded_to(
        &self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSizeF>>,
    ) -> ::cpp_utils::CppBox<crate::QSizeF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSizeF_boundedTo(
            self as *const crate::QSizeF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSizeF>>::cast_into(arg1).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a> class defines the size of a two-dimensional object using floating point precision.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSizeF& QSizeF::operator=(const QSizeF& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a> class defines the size of a two-dimensional object using floating point precision.</p>
    /// <p>A size is specified by a <a href="http://doc.qt.io/qt-5/qsizef.html#width">width</a>() and a <a href="http://doc.qt.io/qt-5/qsizef.html#height">height</a>(). It can be set in the constructor and changed using the <a href="http://doc.qt.io/qt-5/qsizef.html#setWidth">setWidth</a>(), <a href="http://doc.qt.io/qt-5/qsizef.html#setHeight">setHeight</a>(), or <a href="http://doc.qt.io/qt-5/qsizef.html#scale">scale</a>() functions, or using arithmetic operators. A size can also be manipulated directly by retrieving references to the width and height using the <a href="http://doc.qt.io/qt-5/qsizef.html#rwidth">rwidth</a>() and <a href="http://doc.qt.io/qt-5/qsizef.html#rheight">rheight</a>() functions. Finally, the width and height can be swapped using the <a href="http://doc.qt.io/qt-5/qsizef.html#transpose">transpose</a>() function.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qsizef.html#isValid">isValid</a>() function determines if a size is valid. A valid size has both width and height greater than or equal to zero. The <a href="http://doc.qt.io/qt-5/qsizef.html#isEmpty">isEmpty</a>() function returns <code>true</code> if either of the width and height is <i>less</i> than (or equal to) zero, while the <a href="http://doc.qt.io/qt-5/qsizef.html#isNull">isNull</a>() function returns <code>true</code> only if both the width and the height is zero.</p>
    /// <p>Use the <a href="http://doc.qt.io/qt-5/qsizef.html#expandedTo">expandedTo</a>() function to retrieve a size which holds the maximum height and width of this size and a given size. Similarly, the <a href="http://doc.qt.io/qt-5/qsizef.html#boundedTo">boundedTo</a>() function returns a size which holds the minimum height and width of this size and a given size.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a> class also provides the <a href="http://doc.qt.io/qt-5/qsizef.html#toSize">toSize</a>() function returning a <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a> copy of this size, constructed by rounding the width and height to the nearest integers.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a> objects can be streamed as well as compared.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSizeF>>,
    ) -> ::cpp_utils::MutRef<crate::QSizeF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSizeF_operator_(
            self as *mut crate::QSizeF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSizeF>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a size holding the maximum width and height of this size and the given <i>otherSize</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSizeF QSizeF::expandedTo(const QSizeF& arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html#expandedTo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a size holding the maximum width and height of this size and the given <i>otherSize</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsizef.html#boundedTo">boundedTo</a>() and <a href="http://doc.qt.io/qt-5/qsizef.html#scale">scale</a>().</p></div>
    pub unsafe fn expanded_to(
        &self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSizeF>>,
    ) -> ::cpp_utils::CppBox<crate::QSizeF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSizeF_expandedTo(
            self as *const crate::QSizeF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSizeF>>::cast_into(arg1).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the height.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QSizeF::height() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html#height">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the height.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsizef.html#width">width</a>() and <a href="http://doc.qt.io/qt-5/qsizef.html#setHeight">setHeight</a>().</p></div>
    pub unsafe fn height(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QSizeF_height(self as *const crate::QSizeF)
    }

    /// <p>Returns <code>true</code> if either of the width and height is less than or equal to 0; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSizeF::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if either of the width and height is less than or equal to 0; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsizef.html#isNull">isNull</a>() and <a href="http://doc.qt.io/qt-5/qsizef.html#isValid">isValid</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSizeF_isEmpty(self as *const crate::QSizeF)
    }

    /// <p>Returns <code>true</code> if both the width and height are 0.0 (ignoring the sign); otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSizeF::isNull() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html#isNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if both the width and height are 0.0 (ignoring the sign); otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsizef.html#isValid">isValid</a>() and <a href="http://doc.qt.io/qt-5/qsizef.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSizeF_isNull(self as *const crate::QSizeF)
    }

    /// <p>Returns <code>true</code> if both the width and height is equal to or greater than 0; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSizeF::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if both the width and height is equal to or greater than 0; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsizef.html#isNull">isNull</a>() and <a href="http://doc.qt.io/qt-5/qsizef.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn is_valid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSizeF_isValid(self as *const crate::QSizeF)
    }

    /// <p>Constructs an invalid size.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSizeF::QSizeF()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html#QSizeF">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an invalid size.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsizef.html#isValid">isValid</a>().</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QSizeF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSizeF_QSizeF();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a size with floating point accuracy from the given <i>size</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSizeF::QSizeF(const QSize& sz)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html#QSizeF-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a size with floating point accuracy from the given <i>size</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsizef.html#toSize">toSize</a>().</p></div>
    pub unsafe fn new_1a(
        sz: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSize>>,
    ) -> ::cpp_utils::CppBox<crate::QSizeF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSizeF_QSizeF1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSize>>::cast_into(sz).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a size with the given <i>width</i> and <i>height</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSizeF::QSizeF(double w, double h)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html#QSizeF-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a size with the given <i>width</i> and <i>height</i>.</p></div>
    pub unsafe fn new_2a(
        w: ::std::os::raw::c_double,
        h: ::std::os::raw::c_double,
    ) -> ::cpp_utils::CppBox<crate::QSizeF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSizeF_QSizeF2(w, h);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a> class defines the size of a two-dimensional object using floating point precision.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSizeF::QSizeF(const QSizeF& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a> class defines the size of a two-dimensional object using floating point precision.</p>
    /// <p>A size is specified by a <a href="http://doc.qt.io/qt-5/qsizef.html#width">width</a>() and a <a href="http://doc.qt.io/qt-5/qsizef.html#height">height</a>(). It can be set in the constructor and changed using the <a href="http://doc.qt.io/qt-5/qsizef.html#setWidth">setWidth</a>(), <a href="http://doc.qt.io/qt-5/qsizef.html#setHeight">setHeight</a>(), or <a href="http://doc.qt.io/qt-5/qsizef.html#scale">scale</a>() functions, or using arithmetic operators. A size can also be manipulated directly by retrieving references to the width and height using the <a href="http://doc.qt.io/qt-5/qsizef.html#rwidth">rwidth</a>() and <a href="http://doc.qt.io/qt-5/qsizef.html#rheight">rheight</a>() functions. Finally, the width and height can be swapped using the <a href="http://doc.qt.io/qt-5/qsizef.html#transpose">transpose</a>() function.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qsizef.html#isValid">isValid</a>() function determines if a size is valid. A valid size has both width and height greater than or equal to zero. The <a href="http://doc.qt.io/qt-5/qsizef.html#isEmpty">isEmpty</a>() function returns <code>true</code> if either of the width and height is <i>less</i> than (or equal to) zero, while the <a href="http://doc.qt.io/qt-5/qsizef.html#isNull">isNull</a>() function returns <code>true</code> only if both the width and the height is zero.</p>
    /// <p>Use the <a href="http://doc.qt.io/qt-5/qsizef.html#expandedTo">expandedTo</a>() function to retrieve a size which holds the maximum height and width of this size and a given size. Similarly, the <a href="http://doc.qt.io/qt-5/qsizef.html#boundedTo">boundedTo</a>() function returns a size which holds the minimum height and width of this size and a given size.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a> class also provides the <a href="http://doc.qt.io/qt-5/qsizef.html#toSize">toSize</a>() function returning a <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a> copy of this size, constructed by rounding the width and height to the nearest integers.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a> objects can be streamed as well as compared.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSizeF>>,
    ) -> ::cpp_utils::CppBox<crate::QSizeF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSizeF_QSizeF3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSizeF>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the height.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double& QSizeF::rheight()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html#rheight">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the height.</p>
    /// <p>Using a reference makes it possible to manipulate the height directly. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsizef.html#QSizeF">QSizeF</a></span> size(<span class="number">100</span><span class="operator">,</span> <span class="number">10.2</span>);
    ///   size<span class="operator">.</span>rheight() <span class="operator">+</span><span class="operator">=</span> <span class="number">5.5</span>;
    ///
    ///   <span class="comment">// size becomes (100,15.7)</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsizef.html#rwidth">rwidth</a>() and <a href="http://doc.qt.io/qt-5/qsizef.html#setHeight">setHeight</a>().</p></div>
    pub unsafe fn rheight(&mut self) -> ::cpp_utils::MutRef<::std::os::raw::c_double> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSizeF_rheight(self as *mut crate::QSizeF);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the width.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double& QSizeF::rwidth()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html#rwidth">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the width.</p>
    /// <p>Using a reference makes it possible to manipulate the width directly. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsizef.html#QSizeF">QSizeF</a></span> size(<span class="number">100.3</span><span class="operator">,</span> <span class="number">10</span>);
    ///   size<span class="operator">.</span>rwidth() <span class="operator">+</span><span class="operator">=</span> <span class="number">20.5</span>;
    ///
    ///    <span class="comment">// size becomes (120.8,10)</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsizef.html#rheight">rheight</a>() and <a href="http://doc.qt.io/qt-5/qsizef.html#setWidth">setWidth</a>().</p></div>
    pub unsafe fn rwidth(&mut self) -> ::cpp_utils::MutRef<::std::os::raw::c_double> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSizeF_rwidth(self as *mut crate::QSizeF);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Scales the size to a rectangle with the given <i>width</i> and <i>height</i>, according to the specified <i>mode</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSizeF::scale(double w, double h, Qt::AspectRatioMode mode)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html#scale">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Scales the size to a rectangle with the given <i>width</i> and <i>height</i>, according to the specified <i>mode</i>.</p>
    /// <ul>
    /// <li>If <i>mode</i> is <a href="http://doc.qt.io/qt-5/qt.html#AspectRatioMode-enum">Qt::IgnoreAspectRatio</a>, the size is set to (<i>width</i>, <i>height</i>).</li>
    /// <li>If <i>mode</i> is <a href="http://doc.qt.io/qt-5/qt.html#AspectRatioMode-enum">Qt::KeepAspectRatio</a>, the current size is scaled to a rectangle as large as possible inside (<i>width</i>, <i>height</i>), preserving the aspect ratio.</li>
    /// <li>If <i>mode</i> is <a href="http://doc.qt.io/qt-5/qt.html#AspectRatioMode-enum">Qt::KeepAspectRatioByExpanding</a>, the current size is scaled to a rectangle as small as possible outside (<i>width</i>, <i>height</i>), preserving the aspect ratio.</li>
    /// </ul>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsizef.html#QSizeF">QSizeF</a></span> t1(<span class="number">10</span><span class="operator">,</span> <span class="number">12</span>);
    ///   t1<span class="operator">.</span>scale(<span class="number">60</span><span class="operator">,</span> <span class="number">60</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>IgnoreAspectRatio);
    ///   <span class="comment">// t1 is (60, 60)</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsizef.html#QSizeF">QSizeF</a></span> t2(<span class="number">10</span><span class="operator">,</span> <span class="number">12</span>);
    ///   t2<span class="operator">.</span>scale(<span class="number">60</span><span class="operator">,</span> <span class="number">60</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>KeepAspectRatio);
    ///   <span class="comment">// t2 is (50, 60)</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsizef.html#QSizeF">QSizeF</a></span> t3(<span class="number">10</span><span class="operator">,</span> <span class="number">12</span>);
    ///   t3<span class="operator">.</span>scale(<span class="number">60</span><span class="operator">,</span> <span class="number">60</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>KeepAspectRatioByExpanding);
    ///   <span class="comment">// t3 is (60, 72)</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsizef.html#setWidth">setWidth</a>(), <a href="http://doc.qt.io/qt-5/qsizef.html#setHeight">setHeight</a>(), and <a href="http://doc.qt.io/qt-5/qsizef.html#scaled">scaled</a>().</p></div>
    pub unsafe fn scale_3a(
        &mut self,
        w: ::std::os::raw::c_double,
        h: ::std::os::raw::c_double,
        mode: crate::AspectRatioMode,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSizeF_scale(self as *mut crate::QSizeF, w, h, mode)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSizeF::scale(const QSizeF& s, Qt::AspectRatioMode mode)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html#scale-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Scales the size to a rectangle with the given <i>size</i>, according to the specified <i>mode</i>.</p></div>
    pub unsafe fn scale_2a(
        &mut self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSizeF>>,
        mode: crate::AspectRatioMode,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSizeF_scale1(
            self as *mut crate::QSizeF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSizeF>>::cast_into(s).as_raw_ptr(),
            mode,
        )
    }

    /// <p>Returns a size scaled to a rectangle with the given <i>width</i> and <i>height</i>, according to the specified <i>mode</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSizeF QSizeF::scaled(double w, double h, Qt::AspectRatioMode mode) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html#scaled">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a size scaled to a rectangle with the given <i>width</i> and <i>height</i>, according to the specified <i>mode</i>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsizef.html#scale">scale</a>().</p></div>
    pub unsafe fn scaled_3a(
        &self,
        w: ::std::os::raw::c_double,
        h: ::std::os::raw::c_double,
        mode: crate::AspectRatioMode,
    ) -> ::cpp_utils::CppBox<crate::QSizeF> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QSizeF_scaled(self as *const crate::QSizeF, w, h, mode);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSizeF QSizeF::scaled(const QSizeF& s, Qt::AspectRatioMode mode) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html#scaled-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a size scaled to a rectangle with the given size <i>s</i>, according to the specified <i>mode</i>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn scaled_2a(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSizeF>>,
        mode: crate::AspectRatioMode,
    ) -> ::cpp_utils::CppBox<crate::QSizeF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSizeF_scaled1(
            self as *const crate::QSizeF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSizeF>>::cast_into(s).as_raw_ptr(),
            mode,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Sets the height to the given <i>height</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSizeF::setHeight(double h)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html#setHeight">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the height to the given <i>height</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsizef.html#height">height</a>(), <a href="http://doc.qt.io/qt-5/qsizef.html#rheight">rheight</a>(), and <a href="http://doc.qt.io/qt-5/qsizef.html#setWidth">setWidth</a>().</p></div>
    pub unsafe fn set_height(&mut self, h: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_QSizeF_setHeight(self as *mut crate::QSizeF, h)
    }

    /// <p>Sets the width to the given <i>width</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSizeF::setWidth(double w)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html#setWidth">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the width to the given <i>width</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsizef.html#width">width</a>(), <a href="http://doc.qt.io/qt-5/qsizef.html#rwidth">rwidth</a>(), and <a href="http://doc.qt.io/qt-5/qsizef.html#setHeight">setHeight</a>().</p></div>
    pub unsafe fn set_width(&mut self, w: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_QSizeF_setWidth(self as *mut crate::QSizeF, w)
    }

    /// <p>Returns an integer based copy of this size.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSize QSizeF::toSize() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html#toSize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an integer based copy of this size.</p>
    /// <p>Note that the coordinates in the returned size will be rounded to the nearest integer.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsizef.html#QSizeF">QSizeF</a>().</p></div>
    pub unsafe fn to_size(&self) -> ::cpp_utils::CppBox<crate::QSize> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSizeF_toSize(self as *const crate::QSizeF);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Swaps the width and height values.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSizeF::transpose()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html#transpose">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps the width and height values.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsizef.html#setWidth">setWidth</a>(), <a href="http://doc.qt.io/qt-5/qsizef.html#setHeight">setHeight</a>(), and <a href="http://doc.qt.io/qt-5/qsizef.html#transposed">transposed</a>().</p></div>
    pub unsafe fn transpose(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSizeF_transpose(self as *mut crate::QSizeF)
    }

    /// <p>Returns the size with width and height values swapped.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSizeF QSizeF::transposed() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html#transposed">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the size with width and height values swapped.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsizef.html#transpose">transpose</a>().</p></div>
    pub unsafe fn transposed(&self) -> ::cpp_utils::CppBox<crate::QSizeF> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QSizeF_transposed(self as *const crate::QSizeF);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the width.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QSizeF::width() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html#width">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the width.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsizef.html#height">height</a>() and <a href="http://doc.qt.io/qt-5/qsizef.html#setWidth">setWidth</a>().</p></div>
    pub unsafe fn width(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QSizeF_width(self as *const crate::QSizeF)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> class defines a rectangle in the plane using integer precision.</p>
///
/// C++ class: <span style='color: green;'>```QRect```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qrect.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> class defines a rectangle in the plane using integer precision.</p>
/// <p>A rectangle is normally expressed as an upper-left corner and a size. The size (width and height) of a <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> is always equivalent to the mathematical rectangle that forms the basis for its rendering.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> can be constructed with a set of left, top, width and height integers, or from a <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> and a <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a>. The following code creates two identical rectangles.</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qrect.html#QRect">QRect</a></span> r1(<span class="number">100</span><span class="operator">,</span> <span class="number">200</span><span class="operator">,</span> <span class="number">11</span><span class="operator">,</span> <span class="number">16</span>);
///   <span class="type"><a href="http://doc.qt.io/qt-5/qrect.html#QRect">QRect</a></span> r2(<span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a></span>(<span class="number">100</span><span class="operator">,</span> <span class="number">200</span>)<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsize.html">QSize</a></span>(<span class="number">11</span><span class="operator">,</span> <span class="number">16</span>));
///
/// </pre>
/// <p>There is a third constructor that creates a <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> using the top-left and bottom-right coordinates, but we recommend that you avoid using it. The rationale is that for historical reasons the values returned by the <a href="http://doc.qt.io/qt-5/qrect.html#bottom">bottom</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#right">right</a>() functions deviate from the true bottom-right corner of the rectangle.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> class provides a collection of functions that return the various rectangle coordinates, and enable manipulation of these. <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> also provide functions to move the rectangle relative to the various coordinates. In addition there is a <a href="http://doc.qt.io/qt-5/qrect.html#moveTo">moveTo</a>() function that moves the rectangle, leaving its top left corner at the given coordinates. Alternatively, the <a href="http://doc.qt.io/qt-5/qrect.html#translate">translate</a>() function moves the rectangle the given offset relative to the current position, and the <a href="http://doc.qt.io/qt-5/qrect.html#translated">translated</a>() function returns a translated copy of this rectangle.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qrect.html#size">size</a>() function returns the rectange's dimensions as a <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a>. The dimensions can also be retrieved separately using the <a href="http://doc.qt.io/qt-5/qrect.html#width">width</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#height">height</a>() functions. To manipulate the dimensions use the <a href="http://doc.qt.io/qt-5/qrect.html#setSize">setSize</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#setWidth">setWidth</a>() or <a href="http://doc.qt.io/qt-5/qrect.html#setHeight">setHeight</a>() functions. Alternatively, the size can be changed by applying either of the functions setting the rectangle coordinates, for example, <a href="http://doc.qt.io/qt-5/qrect.html#setBottom">setBottom</a>() or <a href="http://doc.qt.io/qt-5/qrect.html#setRight">setRight</a>().</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qrect.html#contains">contains</a>() function tells whether a given point is inside the rectangle or not, and the <a href="http://doc.qt.io/qt-5/qrect.html#intersects">intersects</a>() function returns <code>true</code> if this rectangle intersects with a given rectangle. The <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> class also provides the <a href="http://doc.qt.io/qt-5/qrect.html#intersected">intersected</a>() function which returns the intersection rectangle, and the <a href="http://doc.qt.io/qt-5/qrect.html#united">united</a>() function which returns the rectangle that encloses the given rectangle and this:</p>
/// <div class="table"><table class="generic">
///  <tbody><tr valign="top" class="odd"><td><img src="http://doc.qt.io/qt-5/images/qrect-intersect.png" alt=""></td><td><img src="http://doc.qt.io/qt-5/images/qrect-unite.png" alt=""></td></tr>
/// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qrect.html#intersected">intersected</a>()</td><td><a href="http://doc.qt.io/qt-5/qrect.html#united">united</a>()</td></tr>
/// </tbody></table></div>
/// <p>The <a href="http://doc.qt.io/qt-5/qrect.html#isEmpty">isEmpty</a>() function returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qrect.html#left">left</a>() &gt; <a href="http://doc.qt.io/qt-5/qrect.html#right">right</a>() or <a href="http://doc.qt.io/qt-5/qrect.html#top">top</a>() &gt; <a href="http://doc.qt.io/qt-5/qrect.html#bottom">bottom</a>(). Note that an empty rectangle is not valid: The <a href="http://doc.qt.io/qt-5/qrect.html#isValid">isValid</a>() function returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qrect.html#left">left</a>() &lt;= <a href="http://doc.qt.io/qt-5/qrect.html#right">right</a>() <i>and</i> <a href="http://doc.qt.io/qt-5/qrect.html#top">top</a>() &lt;= <a href="http://doc.qt.io/qt-5/qrect.html#bottom">bottom</a>(). A null rectangle (<a href="http://doc.qt.io/qt-5/qrect.html#isNull">isNull</a>() == true) on the other hand, has both width and height set to 0.</p>
/// <p>Note that due to the way <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> and <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> are defined, an empty <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> is defined in essentially the same way as <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a>.</p>
/// <p>Finally, <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> objects can be streamed as well as compared.</p>
/// <a name="rendering"></a></div>
#[repr(C)]
pub struct QRect {
    _unused: u8,
}
impl QRect {
    /// <p>Adds <i>dx1</i>, <i>dy1</i>, <i>dx2</i> and <i>dy2</i> respectively to the existing coordinates of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::adjust(int x1, int y1, int x2, int y2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#adjust">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds <i>dx1</i>, <i>dy1</i>, <i>dx2</i> and <i>dy2</i> respectively to the existing coordinates of the rectangle.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#adjusted">adjusted</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#setRect">setRect</a>().</p></div>
    pub unsafe fn adjust(
        &mut self,
        x1: ::std::os::raw::c_int,
        y1: ::std::os::raw::c_int,
        x2: ::std::os::raw::c_int,
        y2: ::std::os::raw::c_int,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRect_adjust(self as *mut crate::QRect, x1, y1, x2, y2)
    }

    /// <p>Returns a new rectangle with <i>dx1</i>, <i>dy1</i>, <i>dx2</i> and <i>dy2</i> added respectively to the existing coordinates of this rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRect QRect::adjusted(int x1, int y1, int x2, int y2) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#adjusted">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a new rectangle with <i>dx1</i>, <i>dy1</i>, <i>dx2</i> and <i>dy2</i> added respectively to the existing coordinates of this rectangle.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#adjust">adjust</a>().</p></div>
    pub unsafe fn adjusted(
        &self,
        x1: ::std::os::raw::c_int,
        y1: ::std::os::raw::c_int,
        x2: ::std::os::raw::c_int,
        y2: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QRect> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRect_adjusted(
            self as *const crate::QRect,
            x1,
            y1,
            x2,
            y2,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the y-coordinate of the rectangle's bottom edge.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRect::bottom() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#bottom">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the y-coordinate of the rectangle's bottom edge.</p>
    /// <p>Note that for historical reasons this function returns <a href="http://doc.qt.io/qt-5/qrect.html#top">top</a>() + <a href="http://doc.qt.io/qt-5/qrect.html#height">height</a>() - 1; use <a href="http://doc.qt.io/qt-5/qrect.html#y">y</a>() + <a href="http://doc.qt.io/qt-5/qrect.html#height">height</a>() to retrieve the true y-coordinate.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#setBottom">setBottom</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#bottomLeft">bottomLeft</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#bottomRight">bottomRight</a>().</p></div>
    pub unsafe fn bottom(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRect_bottom(self as *const crate::QRect)
    }

    /// <p>Returns the position of the rectangle's bottom-left corner. Note that for historical reasons this function returns <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a>(<a href="http://doc.qt.io/qt-5/qrect.html#left">left</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#top">top</a>() + <a href="http://doc.qt.io/qt-5/qrect.html#height">height</a>() - 1).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPoint QRect::bottomLeft() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#bottomLeft">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the position of the rectangle's bottom-left corner. Note that for historical reasons this function returns <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a>(<a href="http://doc.qt.io/qt-5/qrect.html#left">left</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#top">top</a>() + <a href="http://doc.qt.io/qt-5/qrect.html#height">height</a>() - 1).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#setBottomLeft">setBottomLeft</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#bottom">bottom</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#left">left</a>().</p></div>
    pub unsafe fn bottom_left(&self) -> ::cpp_utils::CppBox<crate::QPoint> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QRect_bottomLeft(self as *const crate::QRect);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the position of the rectangle's bottom-right corner.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPoint QRect::bottomRight() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#bottomRight">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the position of the rectangle's bottom-right corner.</p>
    /// <p>Note that for historical reasons this function returns <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a>(<a href="http://doc.qt.io/qt-5/qrect.html#left">left</a>() + <a href="http://doc.qt.io/qt-5/qrect.html#width">width</a>() -1, <a href="http://doc.qt.io/qt-5/qrect.html#top">top</a>() + <a href="http://doc.qt.io/qt-5/qrect.html#height">height</a>() - 1).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#setBottomRight">setBottomRight</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#bottom">bottom</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#right">right</a>().</p></div>
    pub unsafe fn bottom_right(&self) -> ::cpp_utils::CppBox<crate::QPoint> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QRect_bottomRight(self as *const crate::QRect);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the center point of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPoint QRect::center() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#center">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the center point of the rectangle.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#moveCenter">moveCenter</a>().</p></div>
    pub unsafe fn center(&self) -> ::cpp_utils::CppBox<crate::QPoint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRect_center(self as *const crate::QRect);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRect::contains(const QRect& r, bool proper = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#contains-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the given <i>rectangle</i> is inside this rectangle. otherwise returns <code>false</code>. If <i>proper</i> is true, this function only returns <code>true</code> if the <i>rectangle</i> is entirely inside this rectangle (not on the edge).</p></div>
    pub unsafe fn contains_q_rect_bool(
        &self,
        r: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRect>>,
        proper: bool,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRect_contains(
            self as *const crate::QRect,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRect>>::cast_into(r).as_raw_ptr(),
            proper,
        )
    }

    /// <p>Returns <code>true</code> if the given <i>point</i> is inside or on the edge of the rectangle, otherwise returns <code>false</code>. If <i>proper</i> is true, this function only returns <code>true</code> if the given <i>point</i> is <i>inside</i> the rectangle (i.e., not on the edge).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRect::contains(const QPoint& p, bool proper = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the given <i>point</i> is inside or on the edge of the rectangle, otherwise returns <code>false</code>. If <i>proper</i> is true, this function only returns <code>true</code> if the given <i>point</i> is <i>inside</i> the rectangle (i.e., not on the edge).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#intersects">intersects</a>().</p></div>
    pub unsafe fn contains_q_point_bool(
        &self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
        proper: bool,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRect_contains1(
            self as *const crate::QRect,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(p).as_raw_ptr(),
            proper,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRect::contains(int x, int y) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#contains-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the point (<i>x</i>, <i>y</i>) is inside this rectangle, otherwise returns <code>false</code>.</p></div>
    pub unsafe fn contains_2_int(
        &self,
        x: ::std::os::raw::c_int,
        y: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRect_contains2(self as *const crate::QRect, x, y)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRect::contains(int x, int y, bool proper) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#contains-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the point (<i>x</i>, <i>y</i>) is inside or on the edge of the rectangle, otherwise returns <code>false</code>. If <i>proper</i> is true, this function only returns <code>true</code> if the point is entirely inside the rectangle(not on the edge).</p></div>
    pub unsafe fn contains_2_int_bool(
        &self,
        x: ::std::os::raw::c_int,
        y: ::std::os::raw::c_int,
        proper: bool,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRect_contains3(self as *const crate::QRect, x, y, proper)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRect::contains(const QRect& r) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#contains-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the given <i>rectangle</i> is inside this rectangle. otherwise returns <code>false</code>. If <i>proper</i> is true, this function only returns <code>true</code> if the <i>rectangle</i> is entirely inside this rectangle (not on the edge).</p></div>
    pub unsafe fn contains_q_rect(
        &self,
        r: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRect>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRect_contains4(
            self as *const crate::QRect,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRect>>::cast_into(r).as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if the given <i>point</i> is inside or on the edge of the rectangle, otherwise returns <code>false</code>. If <i>proper</i> is true, this function only returns <code>true</code> if the given <i>point</i> is <i>inside</i> the rectangle (i.e., not on the edge).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRect::contains(const QPoint& p) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the given <i>point</i> is inside or on the edge of the rectangle, otherwise returns <code>false</code>. If <i>proper</i> is true, this function only returns <code>true</code> if the given <i>point</i> is <i>inside</i> the rectangle (i.e., not on the edge).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#intersects">intersects</a>().</p></div>
    pub unsafe fn contains_q_point(
        &self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRect_contains5(
            self as *const crate::QRect,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(p).as_raw_ptr(),
        )
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> class defines a rectangle in the plane using integer precision.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRect& QRect::operator=(const QRect& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> class defines a rectangle in the plane using integer precision.</p>
    /// <p>A rectangle is normally expressed as an upper-left corner and a size. The size (width and height) of a <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> is always equivalent to the mathematical rectangle that forms the basis for its rendering.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> can be constructed with a set of left, top, width and height integers, or from a <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> and a <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a>. The following code creates two identical rectangles.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qrect.html#QRect">QRect</a></span> r1(<span class="number">100</span><span class="operator">,</span> <span class="number">200</span><span class="operator">,</span> <span class="number">11</span><span class="operator">,</span> <span class="number">16</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qrect.html#QRect">QRect</a></span> r2(<span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a></span>(<span class="number">100</span><span class="operator">,</span> <span class="number">200</span>)<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsize.html">QSize</a></span>(<span class="number">11</span><span class="operator">,</span> <span class="number">16</span>));
    ///
    /// </pre>
    /// <p>There is a third constructor that creates a <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> using the top-left and bottom-right coordinates, but we recommend that you avoid using it. The rationale is that for historical reasons the values returned by the <a href="http://doc.qt.io/qt-5/qrect.html#bottom">bottom</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#right">right</a>() functions deviate from the true bottom-right corner of the rectangle.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> class provides a collection of functions that return the various rectangle coordinates, and enable manipulation of these. <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> also provide functions to move the rectangle relative to the various coordinates. In addition there is a <a href="http://doc.qt.io/qt-5/qrect.html#moveTo">moveTo</a>() function that moves the rectangle, leaving its top left corner at the given coordinates. Alternatively, the <a href="http://doc.qt.io/qt-5/qrect.html#translate">translate</a>() function moves the rectangle the given offset relative to the current position, and the <a href="http://doc.qt.io/qt-5/qrect.html#translated">translated</a>() function returns a translated copy of this rectangle.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qrect.html#size">size</a>() function returns the rectange's dimensions as a <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a>. The dimensions can also be retrieved separately using the <a href="http://doc.qt.io/qt-5/qrect.html#width">width</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#height">height</a>() functions. To manipulate the dimensions use the <a href="http://doc.qt.io/qt-5/qrect.html#setSize">setSize</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#setWidth">setWidth</a>() or <a href="http://doc.qt.io/qt-5/qrect.html#setHeight">setHeight</a>() functions. Alternatively, the size can be changed by applying either of the functions setting the rectangle coordinates, for example, <a href="http://doc.qt.io/qt-5/qrect.html#setBottom">setBottom</a>() or <a href="http://doc.qt.io/qt-5/qrect.html#setRight">setRight</a>().</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qrect.html#contains">contains</a>() function tells whether a given point is inside the rectangle or not, and the <a href="http://doc.qt.io/qt-5/qrect.html#intersects">intersects</a>() function returns <code>true</code> if this rectangle intersects with a given rectangle. The <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> class also provides the <a href="http://doc.qt.io/qt-5/qrect.html#intersected">intersected</a>() function which returns the intersection rectangle, and the <a href="http://doc.qt.io/qt-5/qrect.html#united">united</a>() function which returns the rectangle that encloses the given rectangle and this:</p>
    /// <div class="table"><table class="generic">
    ///  <tbody><tr valign="top" class="odd"><td><img src="http://doc.qt.io/qt-5/images/qrect-intersect.png" alt=""></td><td><img src="http://doc.qt.io/qt-5/images/qrect-unite.png" alt=""></td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qrect.html#intersected">intersected</a>()</td><td><a href="http://doc.qt.io/qt-5/qrect.html#united">united</a>()</td></tr>
    /// </tbody></table></div>
    /// <p>The <a href="http://doc.qt.io/qt-5/qrect.html#isEmpty">isEmpty</a>() function returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qrect.html#left">left</a>() &gt; <a href="http://doc.qt.io/qt-5/qrect.html#right">right</a>() or <a href="http://doc.qt.io/qt-5/qrect.html#top">top</a>() &gt; <a href="http://doc.qt.io/qt-5/qrect.html#bottom">bottom</a>(). Note that an empty rectangle is not valid: The <a href="http://doc.qt.io/qt-5/qrect.html#isValid">isValid</a>() function returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qrect.html#left">left</a>() &lt;= <a href="http://doc.qt.io/qt-5/qrect.html#right">right</a>() <i>and</i> <a href="http://doc.qt.io/qt-5/qrect.html#top">top</a>() &lt;= <a href="http://doc.qt.io/qt-5/qrect.html#bottom">bottom</a>(). A null rectangle (<a href="http://doc.qt.io/qt-5/qrect.html#isNull">isNull</a>() == true) on the other hand, has both width and height set to 0.</p>
    /// <p>Note that due to the way <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> and <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> are defined, an empty <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> is defined in essentially the same way as <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a>.</p>
    /// <p>Finally, <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> objects can be streamed as well as compared.</p>
    /// <a name="rendering"></a></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRect>>,
    ) -> ::cpp_utils::MutRef<crate::QRect> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRect_operator_2(
            self as *mut crate::QRect,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRect>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Extracts the position of the rectangle's top-left corner to *<i>x1</i> and *<i>y1</i>, and the position of the bottom-right corner to *<i>x2</i> and *<i>y2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::getCoords(int* x1, int* y1, int* x2, int* y2) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#getCoords">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Extracts the position of the rectangle's top-left corner to *<i>x1</i> and *<i>y1</i>, and the position of the bottom-right corner to *<i>x2</i> and *<i>y2</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#setCoords">setCoords</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#getRect">getRect</a>().</p></div>
    pub unsafe fn get_coords(
        &self,
        x1: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_int>>,
        y1: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_int>>,
        x2: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_int>>,
        y2: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_int>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRect_getCoords(
            self as *const crate::QRect,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_int>>::cast_into(x1)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_int>>::cast_into(y1)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_int>>::cast_into(x2)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_int>>::cast_into(y2)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Extracts the position of the rectangle's top-left corner to *<i>x</i> and *<i>y</i>, and its dimensions to *<i>width</i> and *<i>height</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::getRect(int* x, int* y, int* w, int* h) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#getRect">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Extracts the position of the rectangle's top-left corner to *<i>x</i> and *<i>y</i>, and its dimensions to *<i>width</i> and *<i>height</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#setRect">setRect</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#getCoords">getCoords</a>().</p></div>
    pub unsafe fn get_rect(
        &self,
        x: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_int>>,
        y: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_int>>,
        w: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_int>>,
        h: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_int>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRect_getRect(
            self as *const crate::QRect,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_int>>::cast_into(x)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_int>>::cast_into(y)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_int>>::cast_into(w)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_int>>::cast_into(h)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the height of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRect::height() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#height">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the height of the rectangle.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#setHeight">setHeight</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#width">width</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#size">size</a>().</p></div>
    pub unsafe fn height(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRect_height(self as *const crate::QRect)
    }

    /// <p>Returns the intersection of this rectangle and the given <i>rectangle</i>. Note that <code>r.intersected(s)</code> is equivalent to <code>r &amp; s</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRect QRect::intersected(const QRect& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#intersected">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the intersection of this rectangle and the given <i>rectangle</i>. Note that <code>r.intersected(s)</code> is equivalent to <code>r &amp; s</code>.</p>
    /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qrect-intersect.png" alt=""></p><p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#intersects">intersects</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#united">united</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#operator-and-eq">operator&amp;=</a>().</p></div>
    pub unsafe fn intersected(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRect>>,
    ) -> ::cpp_utils::CppBox<crate::QRect> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRect_intersected(
            self as *const crate::QRect,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRect>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if this rectangle intersects with the given <i>rectangle</i> (i.e., there is at least one pixel that is within both rectangles), otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRect::intersects(const QRect& r) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#intersects">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this rectangle intersects with the given <i>rectangle</i> (i.e., there is at least one pixel that is within both rectangles), otherwise returns <code>false</code>.</p>
    /// <p>The intersection rectangle can be retrieved using the <a href="http://doc.qt.io/qt-5/qrect.html#intersected">intersected</a>() function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#contains">contains</a>().</p></div>
    pub unsafe fn intersects(
        &self,
        r: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRect>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRect_intersects(
            self as *const crate::QRect,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRect>>::cast_into(r).as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if the rectangle is empty, otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRect::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the rectangle is empty, otherwise returns <code>false</code>.</p>
    /// <p>An empty rectangle has a <a href="http://doc.qt.io/qt-5/qrect.html#left">left</a>() &gt; <a href="http://doc.qt.io/qt-5/qrect.html#right">right</a>() or <a href="http://doc.qt.io/qt-5/qrect.html#top">top</a>() &gt; <a href="http://doc.qt.io/qt-5/qrect.html#bottom">bottom</a>(). An empty rectangle is not valid (i.e., isEmpty() == !<a href="http://doc.qt.io/qt-5/qrect.html#isValid">isValid</a>()).</p>
    /// <p>Use the <a href="http://doc.qt.io/qt-5/qrect.html#normalized">normalized</a>() function to retrieve a rectangle where the corners are swapped.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#isNull">isNull</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#isValid">isValid</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#normalized">normalized</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRect_isEmpty(self as *const crate::QRect)
    }

    /// <p>Returns <code>true</code> if the rectangle is a null rectangle, otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRect::isNull() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#isNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the rectangle is a null rectangle, otherwise returns <code>false</code>.</p>
    /// <p>A null rectangle has both the width and the height set to 0 (i.e., <a href="http://doc.qt.io/qt-5/qrect.html#right">right</a>() == <a href="http://doc.qt.io/qt-5/qrect.html#left">left</a>() - 1 and <a href="http://doc.qt.io/qt-5/qrect.html#bottom">bottom</a>() == <a href="http://doc.qt.io/qt-5/qrect.html#top">top</a>() - 1). A null rectangle is also empty, and hence is not valid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#isValid">isValid</a>().</p></div>
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRect_isNull(self as *const crate::QRect)
    }

    /// <p>Returns <code>true</code> if the rectangle is valid, otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRect::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the rectangle is valid, otherwise returns <code>false</code>.</p>
    /// <p>A valid rectangle has a <a href="http://doc.qt.io/qt-5/qrect.html#left">left</a>() &lt;= <a href="http://doc.qt.io/qt-5/qrect.html#right">right</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#top">top</a>() &lt;= <a href="http://doc.qt.io/qt-5/qrect.html#bottom">bottom</a>(). Note that non-trivial operations like intersections are not defined for invalid rectangles. A valid rectangle is not empty (i.e., isValid() == !<a href="http://doc.qt.io/qt-5/qrect.html#isEmpty">isEmpty</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#isNull">isNull</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#normalized">normalized</a>().</p></div>
    pub unsafe fn is_valid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRect_isValid(self as *const crate::QRect)
    }

    /// <p>Returns the x-coordinate of the rectangle's left edge. Equivalent to <a href="http://doc.qt.io/qt-5/qrect.html#x">x</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRect::left() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#left">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the x-coordinate of the rectangle's left edge. Equivalent to <a href="http://doc.qt.io/qt-5/qrect.html#x">x</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#setLeft">setLeft</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#topLeft">topLeft</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#bottomLeft">bottomLeft</a>().</p></div>
    pub unsafe fn left(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRect_left(self as *const crate::QRect)
    }

    /// <p>Returns a rectangle grown by the <i>margins</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRect QRect::marginsAdded(const QMargins& margins) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#marginsAdded">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a rectangle grown by the <i>margins</i>.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#operator-2b-eq">operator+=</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#marginsRemoved">marginsRemoved</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#operator--eq">operator-=</a>().</p></div>
    pub unsafe fn margins_added(
        &self,
        margins: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMargins>>,
    ) -> ::cpp_utils::CppBox<crate::QRect> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRect_marginsAdded(
            self as *const crate::QRect,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMargins>>::cast_into(margins)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the <i>margins</i> from the rectangle, shrinking it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRect QRect::marginsRemoved(const QMargins& margins) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#marginsRemoved">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the <i>margins</i> from the rectangle, shrinking it.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#marginsAdded">marginsAdded</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#operator-2b-eq">operator+=</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#operator--eq">operator-=</a>().</p></div>
    pub unsafe fn margins_removed(
        &self,
        margins: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMargins>>,
    ) -> ::cpp_utils::CppBox<crate::QRect> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRect_marginsRemoved(
            self as *const crate::QRect,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMargins>>::cast_into(margins)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the rectangle vertically, leaving the rectangle's bottom edge at the given <i>y</i> coordinate. The rectangle's size is unchanged.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::moveBottom(int pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#moveBottom">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the rectangle vertically, leaving the rectangle's bottom edge at the given <i>y</i> coordinate. The rectangle's size is unchanged.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#bottom">bottom</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#setBottom">setBottom</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#moveTop">moveTop</a>().</p></div>
    pub unsafe fn move_bottom(&mut self, pos: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QRect_moveBottom(self as *mut crate::QRect, pos)
    }

    /// <p>Moves the rectangle, leaving the bottom-left corner at the given <i>position</i>. The rectangle's size is unchanged.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::moveBottomLeft(const QPoint& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#moveBottomLeft">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the rectangle, leaving the bottom-left corner at the given <i>position</i>. The rectangle's size is unchanged.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#setBottomLeft">setBottomLeft</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#moveBottom">moveBottom</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#moveLeft">moveLeft</a>().</p></div>
    pub unsafe fn move_bottom_left(
        &mut self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRect_moveBottomLeft(
            self as *mut crate::QRect,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(p).as_raw_ptr(),
        )
    }

    /// <p>Moves the rectangle, leaving the bottom-right corner at the given <i>position</i>. The rectangle's size is unchanged.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::moveBottomRight(const QPoint& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#moveBottomRight">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the rectangle, leaving the bottom-right corner at the given <i>position</i>. The rectangle's size is unchanged.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#setBottomRight">setBottomRight</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#moveRight">moveRight</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#moveBottom">moveBottom</a>().</p></div>
    pub unsafe fn move_bottom_right(
        &mut self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRect_moveBottomRight(
            self as *mut crate::QRect,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(p).as_raw_ptr(),
        )
    }

    /// <p>Moves the rectangle, leaving the center point at the given <i>position</i>. The rectangle's size is unchanged.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::moveCenter(const QPoint& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#moveCenter">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the rectangle, leaving the center point at the given <i>position</i>. The rectangle's size is unchanged.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#center">center</a>().</p></div>
    pub unsafe fn move_center(
        &mut self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRect_moveCenter(
            self as *mut crate::QRect,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(p).as_raw_ptr(),
        )
    }

    /// <p>Moves the rectangle horizontally, leaving the rectangle's left edge at the given <i>x</i> coordinate. The rectangle's size is unchanged.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::moveLeft(int pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#moveLeft">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the rectangle horizontally, leaving the rectangle's left edge at the given <i>x</i> coordinate. The rectangle's size is unchanged.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#left">left</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#setLeft">setLeft</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#moveRight">moveRight</a>().</p></div>
    pub unsafe fn move_left(&mut self, pos: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QRect_moveLeft(self as *mut crate::QRect, pos)
    }

    /// <p>Moves the rectangle horizontally, leaving the rectangle's right edge at the given <i>x</i> coordinate. The rectangle's size is unchanged.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::moveRight(int pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#moveRight">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the rectangle horizontally, leaving the rectangle's right edge at the given <i>x</i> coordinate. The rectangle's size is unchanged.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#right">right</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#setRight">setRight</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#moveLeft">moveLeft</a>().</p></div>
    pub unsafe fn move_right(&mut self, pos: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QRect_moveRight(self as *mut crate::QRect, pos)
    }

    /// <p>Moves the rectangle, leaving the top-left corner at the given position (<i>x</i>, <i>y</i>). The rectangle's size is unchanged.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::moveTo(int x, int t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#moveTo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the rectangle, leaving the top-left corner at the given position (<i>x</i>, <i>y</i>). The rectangle's size is unchanged.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#translate">translate</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#moveTopLeft">moveTopLeft</a>().</p></div>
    pub unsafe fn move_to_2a(&mut self, x: ::std::os::raw::c_int, t: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QRect_moveTo(self as *mut crate::QRect, x, t)
    }

    /// <p>Moves the rectangle, leaving the top-left corner at the given <i>position</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::moveTo(const QPoint& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#moveTo-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the rectangle, leaving the top-left corner at the given <i>position</i>.</p></div>
    pub unsafe fn move_to_1a(
        &mut self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRect_moveTo1(
            self as *mut crate::QRect,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(p).as_raw_ptr(),
        )
    }

    /// <p>Moves the rectangle vertically, leaving the rectangle's top edge at the given <i>y</i> coordinate. The rectangle's size is unchanged.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::moveTop(int pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#moveTop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the rectangle vertically, leaving the rectangle's top edge at the given <i>y</i> coordinate. The rectangle's size is unchanged.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#top">top</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#setTop">setTop</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#moveBottom">moveBottom</a>().</p></div>
    pub unsafe fn move_top(&mut self, pos: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QRect_moveTop(self as *mut crate::QRect, pos)
    }

    /// <p>Moves the rectangle, leaving the top-left corner at the given <i>position</i>. The rectangle's size is unchanged.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::moveTopLeft(const QPoint& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#moveTopLeft">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the rectangle, leaving the top-left corner at the given <i>position</i>. The rectangle's size is unchanged.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#setTopLeft">setTopLeft</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#moveTop">moveTop</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#moveLeft">moveLeft</a>().</p></div>
    pub unsafe fn move_top_left(
        &mut self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRect_moveTopLeft(
            self as *mut crate::QRect,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(p).as_raw_ptr(),
        )
    }

    /// <p>Moves the rectangle, leaving the top-right corner at the given <i>position</i>. The rectangle's size is unchanged.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::moveTopRight(const QPoint& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#moveTopRight">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the rectangle, leaving the top-right corner at the given <i>position</i>. The rectangle's size is unchanged.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#setTopRight">setTopRight</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#moveTop">moveTop</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#moveRight">moveRight</a>().</p></div>
    pub unsafe fn move_top_right(
        &mut self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRect_moveTopRight(
            self as *mut crate::QRect,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(p).as_raw_ptr(),
        )
    }

    /// <p>Constructs a null rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRect::QRect()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#QRect">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a null rectangle.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#isNull">isNull</a>().</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QRect> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRect_QRect();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a rectangle with the given <i>topLeft</i> and <i>bottomRight</i> corners.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRect::QRect(const QPoint& topleft, const QPoint& bottomright)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#QRect-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a rectangle with the given <i>topLeft</i> and <i>bottomRight</i> corners.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#setTopLeft">setTopLeft</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#setBottomRight">setBottomRight</a>().</p></div>
    pub unsafe fn from_2_q_point(
        topleft: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
        bottomright: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
    ) -> ::cpp_utils::CppBox<crate::QRect> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRect_QRect1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(topleft)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(bottomright)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a rectangle with the given <i>topLeft</i> corner and the given <i>size</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRect::QRect(const QPoint& topleft, const QSize& size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#QRect-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a rectangle with the given <i>topLeft</i> corner and the given <i>size</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#setTopLeft">setTopLeft</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#setSize">setSize</a>().</p></div>
    pub unsafe fn from_q_point_q_size(
        topleft: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
        size: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSize>>,
    ) -> ::cpp_utils::CppBox<crate::QRect> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRect_QRect2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(topleft)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSize>>::cast_into(size).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a rectangle with (<i>x</i>, <i>y</i>) as its top-left corner and the given <i>width</i> and <i>height</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRect::QRect(int left, int top, int width, int height)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#QRect-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a rectangle with (<i>x</i>, <i>y</i>) as its top-left corner and the given <i>width</i> and <i>height</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#setRect">setRect</a>().</p></div>
    pub unsafe fn from_4_int(
        left: ::std::os::raw::c_int,
        top: ::std::os::raw::c_int,
        width: ::std::os::raw::c_int,
        height: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QRect> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRect_QRect3(left, top, width, height);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> class defines a rectangle in the plane using integer precision.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRect::QRect(const QRect& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> class defines a rectangle in the plane using integer precision.</p>
    /// <p>A rectangle is normally expressed as an upper-left corner and a size. The size (width and height) of a <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> is always equivalent to the mathematical rectangle that forms the basis for its rendering.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> can be constructed with a set of left, top, width and height integers, or from a <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> and a <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a>. The following code creates two identical rectangles.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qrect.html#QRect">QRect</a></span> r1(<span class="number">100</span><span class="operator">,</span> <span class="number">200</span><span class="operator">,</span> <span class="number">11</span><span class="operator">,</span> <span class="number">16</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qrect.html#QRect">QRect</a></span> r2(<span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a></span>(<span class="number">100</span><span class="operator">,</span> <span class="number">200</span>)<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsize.html">QSize</a></span>(<span class="number">11</span><span class="operator">,</span> <span class="number">16</span>));
    ///
    /// </pre>
    /// <p>There is a third constructor that creates a <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> using the top-left and bottom-right coordinates, but we recommend that you avoid using it. The rationale is that for historical reasons the values returned by the <a href="http://doc.qt.io/qt-5/qrect.html#bottom">bottom</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#right">right</a>() functions deviate from the true bottom-right corner of the rectangle.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> class provides a collection of functions that return the various rectangle coordinates, and enable manipulation of these. <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> also provide functions to move the rectangle relative to the various coordinates. In addition there is a <a href="http://doc.qt.io/qt-5/qrect.html#moveTo">moveTo</a>() function that moves the rectangle, leaving its top left corner at the given coordinates. Alternatively, the <a href="http://doc.qt.io/qt-5/qrect.html#translate">translate</a>() function moves the rectangle the given offset relative to the current position, and the <a href="http://doc.qt.io/qt-5/qrect.html#translated">translated</a>() function returns a translated copy of this rectangle.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qrect.html#size">size</a>() function returns the rectange's dimensions as a <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a>. The dimensions can also be retrieved separately using the <a href="http://doc.qt.io/qt-5/qrect.html#width">width</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#height">height</a>() functions. To manipulate the dimensions use the <a href="http://doc.qt.io/qt-5/qrect.html#setSize">setSize</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#setWidth">setWidth</a>() or <a href="http://doc.qt.io/qt-5/qrect.html#setHeight">setHeight</a>() functions. Alternatively, the size can be changed by applying either of the functions setting the rectangle coordinates, for example, <a href="http://doc.qt.io/qt-5/qrect.html#setBottom">setBottom</a>() or <a href="http://doc.qt.io/qt-5/qrect.html#setRight">setRight</a>().</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qrect.html#contains">contains</a>() function tells whether a given point is inside the rectangle or not, and the <a href="http://doc.qt.io/qt-5/qrect.html#intersects">intersects</a>() function returns <code>true</code> if this rectangle intersects with a given rectangle. The <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> class also provides the <a href="http://doc.qt.io/qt-5/qrect.html#intersected">intersected</a>() function which returns the intersection rectangle, and the <a href="http://doc.qt.io/qt-5/qrect.html#united">united</a>() function which returns the rectangle that encloses the given rectangle and this:</p>
    /// <div class="table"><table class="generic">
    ///  <tbody><tr valign="top" class="odd"><td><img src="http://doc.qt.io/qt-5/images/qrect-intersect.png" alt=""></td><td><img src="http://doc.qt.io/qt-5/images/qrect-unite.png" alt=""></td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qrect.html#intersected">intersected</a>()</td><td><a href="http://doc.qt.io/qt-5/qrect.html#united">united</a>()</td></tr>
    /// </tbody></table></div>
    /// <p>The <a href="http://doc.qt.io/qt-5/qrect.html#isEmpty">isEmpty</a>() function returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qrect.html#left">left</a>() &gt; <a href="http://doc.qt.io/qt-5/qrect.html#right">right</a>() or <a href="http://doc.qt.io/qt-5/qrect.html#top">top</a>() &gt; <a href="http://doc.qt.io/qt-5/qrect.html#bottom">bottom</a>(). Note that an empty rectangle is not valid: The <a href="http://doc.qt.io/qt-5/qrect.html#isValid">isValid</a>() function returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qrect.html#left">left</a>() &lt;= <a href="http://doc.qt.io/qt-5/qrect.html#right">right</a>() <i>and</i> <a href="http://doc.qt.io/qt-5/qrect.html#top">top</a>() &lt;= <a href="http://doc.qt.io/qt-5/qrect.html#bottom">bottom</a>(). A null rectangle (<a href="http://doc.qt.io/qt-5/qrect.html#isNull">isNull</a>() == true) on the other hand, has both width and height set to 0.</p>
    /// <p>Note that due to the way <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> and <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> are defined, an empty <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> is defined in essentially the same way as <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a>.</p>
    /// <p>Finally, <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> objects can be streamed as well as compared.</p>
    /// <a name="rendering"></a></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRect>>,
    ) -> ::cpp_utils::CppBox<crate::QRect> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRect_QRect4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRect>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a normalized rectangle; i.e., a rectangle that has a non-negative width and height.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRect QRect::normalized() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#normalized">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a normalized rectangle; i.e., a rectangle that has a non-negative width and height.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qrect.html#width">width</a>() &lt; 0 the function swaps the left and right corners, and it swaps the top and bottom corners if <a href="http://doc.qt.io/qt-5/qrect.html#height">height</a>() &lt; 0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#isValid">isValid</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn normalized(&self) -> ::cpp_utils::CppBox<crate::QRect> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QRect_normalized(self as *const crate::QRect);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the x-coordinate of the rectangle's right edge.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRect::right() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#right">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the x-coordinate of the rectangle's right edge.</p>
    /// <p>Note that for historical reasons this function returns <a href="http://doc.qt.io/qt-5/qrect.html#left">left</a>() + <a href="http://doc.qt.io/qt-5/qrect.html#width">width</a>() - 1; use <a href="http://doc.qt.io/qt-5/qrect.html#x">x</a>() + <a href="http://doc.qt.io/qt-5/qrect.html#width">width</a>() to retrieve the true x-coordinate.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#setRight">setRight</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#topRight">topRight</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#bottomRight">bottomRight</a>().</p></div>
    pub unsafe fn right(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRect_right(self as *const crate::QRect)
    }

    /// <p>Sets the bottom edge of the rectangle to the given <i>y</i> coordinate. May change the height, but will never change the top edge of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::setBottom(int pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#setBottom">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the bottom edge of the rectangle to the given <i>y</i> coordinate. May change the height, but will never change the top edge of the rectangle.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#bottom">bottom</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#moveBottom">moveBottom</a>().</p></div>
    pub unsafe fn set_bottom(&mut self, pos: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QRect_setBottom(self as *mut crate::QRect, pos)
    }

    /// <p>Set the bottom-left corner of the rectangle to the given <i>position</i>. May change the size, but will never change the top-right corner of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::setBottomLeft(const QPoint& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#setBottomLeft">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Set the bottom-left corner of the rectangle to the given <i>position</i>. May change the size, but will never change the top-right corner of the rectangle.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#bottomLeft">bottomLeft</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#moveBottomLeft">moveBottomLeft</a>().</p></div>
    pub unsafe fn set_bottom_left(
        &mut self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRect_setBottomLeft(
            self as *mut crate::QRect,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(p).as_raw_ptr(),
        )
    }

    /// <p>Set the bottom-right corner of the rectangle to the given <i>position</i>. May change the size, but will never change the top-left corner of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::setBottomRight(const QPoint& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#setBottomRight">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Set the bottom-right corner of the rectangle to the given <i>position</i>. May change the size, but will never change the top-left corner of the rectangle.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#bottomRight">bottomRight</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#moveBottomRight">moveBottomRight</a>().</p></div>
    pub unsafe fn set_bottom_right(
        &mut self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRect_setBottomRight(
            self as *mut crate::QRect,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(p).as_raw_ptr(),
        )
    }

    /// <p>Sets the coordinates of the rectangle's top-left corner to (<i>x1</i>, <i>y1</i>), and the coordinates of its bottom-right corner to (<i>x2</i>, <i>y2</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::setCoords(int x1, int y1, int x2, int y2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#setCoords">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the coordinates of the rectangle's top-left corner to (<i>x1</i>, <i>y1</i>), and the coordinates of its bottom-right corner to (<i>x2</i>, <i>y2</i>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#getCoords">getCoords</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#setRect">setRect</a>().</p></div>
    pub unsafe fn set_coords(
        &mut self,
        x1: ::std::os::raw::c_int,
        y1: ::std::os::raw::c_int,
        x2: ::std::os::raw::c_int,
        y2: ::std::os::raw::c_int,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRect_setCoords(self as *mut crate::QRect, x1, y1, x2, y2)
    }

    /// <p>Sets the height of the rectangle to the given <i>height</i>. The bottom edge is changed, but not the top one.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::setHeight(int h)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#setHeight">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the height of the rectangle to the given <i>height</i>. The bottom edge is changed, but not the top one.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#height">height</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#setSize">setSize</a>().</p></div>
    pub unsafe fn set_height(&mut self, h: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QRect_setHeight(self as *mut crate::QRect, h)
    }

    /// <p>Sets the left edge of the rectangle to the given <i>x</i> coordinate. May change the width, but will never change the right edge of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::setLeft(int pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#setLeft">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the left edge of the rectangle to the given <i>x</i> coordinate. May change the width, but will never change the right edge of the rectangle.</p>
    /// <p>Equivalent to <a href="http://doc.qt.io/qt-5/qrect.html#setX">setX</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#left">left</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#moveLeft">moveLeft</a>().</p></div>
    pub unsafe fn set_left(&mut self, pos: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QRect_setLeft(self as *mut crate::QRect, pos)
    }

    /// <p>Sets the coordinates of the rectangle's top-left corner to (<i>x</i>, <i>y</i>), and its size to the given <i>width</i> and <i>height</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::setRect(int x, int y, int w, int h)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#setRect">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the coordinates of the rectangle's top-left corner to (<i>x</i>, <i>y</i>), and its size to the given <i>width</i> and <i>height</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#getRect">getRect</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#setCoords">setCoords</a>().</p></div>
    pub unsafe fn set_rect(
        &mut self,
        x: ::std::os::raw::c_int,
        y: ::std::os::raw::c_int,
        w: ::std::os::raw::c_int,
        h: ::std::os::raw::c_int,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRect_setRect(self as *mut crate::QRect, x, y, w, h)
    }

    /// <p>Sets the right edge of the rectangle to the given <i>x</i> coordinate. May change the width, but will never change the left edge of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::setRight(int pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#setRight">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the right edge of the rectangle to the given <i>x</i> coordinate. May change the width, but will never change the left edge of the rectangle.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#right">right</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#moveRight">moveRight</a>().</p></div>
    pub unsafe fn set_right(&mut self, pos: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QRect_setRight(self as *mut crate::QRect, pos)
    }

    /// <p>Sets the size of the rectangle to the given <i>size</i>. The top-left corner is not moved.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::setSize(const QSize& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#setSize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the size of the rectangle to the given <i>size</i>. The top-left corner is not moved.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#size">size</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#setWidth">setWidth</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#setHeight">setHeight</a>().</p></div>
    pub unsafe fn set_size(
        &mut self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSize>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRect_setSize(
            self as *mut crate::QRect,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSize>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Sets the top edge of the rectangle to the given <i>y</i> coordinate. May change the height, but will never change the bottom edge of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::setTop(int pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#setTop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the top edge of the rectangle to the given <i>y</i> coordinate. May change the height, but will never change the bottom edge of the rectangle.</p>
    /// <p>Equivalent to <a href="http://doc.qt.io/qt-5/qrect.html#setY">setY</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#top">top</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#moveTop">moveTop</a>().</p></div>
    pub unsafe fn set_top(&mut self, pos: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QRect_setTop(self as *mut crate::QRect, pos)
    }

    /// <p>Set the top-left corner of the rectangle to the given <i>position</i>. May change the size, but will never change the bottom-right corner of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::setTopLeft(const QPoint& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#setTopLeft">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Set the top-left corner of the rectangle to the given <i>position</i>. May change the size, but will never change the bottom-right corner of the rectangle.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#topLeft">topLeft</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#moveTopLeft">moveTopLeft</a>().</p></div>
    pub unsafe fn set_top_left(
        &mut self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRect_setTopLeft(
            self as *mut crate::QRect,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(p).as_raw_ptr(),
        )
    }

    /// <p>Set the top-right corner of the rectangle to the given <i>position</i>. May change the size, but will never change the bottom-left corner of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::setTopRight(const QPoint& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#setTopRight">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Set the top-right corner of the rectangle to the given <i>position</i>. May change the size, but will never change the bottom-left corner of the rectangle.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#topRight">topRight</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#moveTopRight">moveTopRight</a>().</p></div>
    pub unsafe fn set_top_right(
        &mut self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRect_setTopRight(
            self as *mut crate::QRect,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(p).as_raw_ptr(),
        )
    }

    /// <p>Sets the width of the rectangle to the given <i>width</i>. The right edge is changed, but not the left one.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::setWidth(int w)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#setWidth">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the width of the rectangle to the given <i>width</i>. The right edge is changed, but not the left one.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#width">width</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#setSize">setSize</a>().</p></div>
    pub unsafe fn set_width(&mut self, w: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QRect_setWidth(self as *mut crate::QRect, w)
    }

    /// <p>Sets the left edge of the rectangle to the given <i>x</i> coordinate. May change the width, but will never change the right edge of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::setX(int x)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#setX">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the left edge of the rectangle to the given <i>x</i> coordinate. May change the width, but will never change the right edge of the rectangle.</p>
    /// <p>Equivalent to <a href="http://doc.qt.io/qt-5/qrect.html#setLeft">setLeft</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#x">x</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#setY">setY</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#setTopLeft">setTopLeft</a>().</p></div>
    pub unsafe fn set_x(&mut self, x: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QRect_setX(self as *mut crate::QRect, x)
    }

    /// <p>Sets the top edge of the rectangle to the given <i>y</i> coordinate. May change the height, but will never change the bottom edge of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::setY(int y)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#setY">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the top edge of the rectangle to the given <i>y</i> coordinate. May change the height, but will never change the bottom edge of the rectangle.</p>
    /// <p>Equivalent to <a href="http://doc.qt.io/qt-5/qrect.html#setTop">setTop</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#y">y</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#setX">setX</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#setTopLeft">setTopLeft</a>().</p></div>
    pub unsafe fn set_y(&mut self, y: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QRect_setY(self as *mut crate::QRect, y)
    }

    /// <p>Returns the size of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSize QRect::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the size of the rectangle.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#setSize">setSize</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#width">width</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#height">height</a>().</p></div>
    pub unsafe fn size(&self) -> ::cpp_utils::CppBox<crate::QSize> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRect_size(self as *const crate::QRect);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the y-coordinate of the rectangle's top edge. Equivalent to <a href="http://doc.qt.io/qt-5/qrect.html#y">y</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRect::top() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#top">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the y-coordinate of the rectangle's top edge. Equivalent to <a href="http://doc.qt.io/qt-5/qrect.html#y">y</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#setTop">setTop</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#topLeft">topLeft</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#topRight">topRight</a>().</p></div>
    pub unsafe fn top(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRect_top(self as *const crate::QRect)
    }

    /// <p>Returns the position of the rectangle's top-left corner.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPoint QRect::topLeft() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#topLeft">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the position of the rectangle's top-left corner.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#setTopLeft">setTopLeft</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#top">top</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#left">left</a>().</p></div>
    pub unsafe fn top_left(&self) -> ::cpp_utils::CppBox<crate::QPoint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRect_topLeft(self as *const crate::QRect);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the position of the rectangle's top-right corner.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPoint QRect::topRight() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#topRight">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the position of the rectangle's top-right corner.</p>
    /// <p>Note that for historical reasons this function returns <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a>(<a href="http://doc.qt.io/qt-5/qrect.html#left">left</a>() + <a href="http://doc.qt.io/qt-5/qrect.html#width">width</a>() -1, <a href="http://doc.qt.io/qt-5/qrect.html#top">top</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#setTopRight">setTopRight</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#top">top</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#right">right</a>().</p></div>
    pub unsafe fn top_right(&self) -> ::cpp_utils::CppBox<crate::QPoint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRect_topRight(self as *const crate::QRect);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the rectangle <i>dx</i> along the x axis and <i>dy</i> along the y axis, relative to the current position. Positive values move the rectangle to the right and down.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::translate(int dx, int dy)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#translate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the rectangle <i>dx</i> along the x axis and <i>dy</i> along the y axis, relative to the current position. Positive values move the rectangle to the right and down.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#moveTopLeft">moveTopLeft</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#moveTo">moveTo</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#translated">translated</a>().</p></div>
    pub unsafe fn translate_2a(&mut self, dx: ::std::os::raw::c_int, dy: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QRect_translate(self as *mut crate::QRect, dx, dy)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRect::translate(const QPoint& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#translate-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Moves the rectangle <i>offset</i>.<a href="http://doc.qt.io/qt-5/qpoint.html#x">x()</a> along the x axis and <i>offset</i>.<a href="http://doc.qt.io/qt-5/qpoint.html#y">y()</a> along the y axis, relative to the current position.</p></div>
    pub unsafe fn translate_1a(
        &mut self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRect_translate1(
            self as *mut crate::QRect,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(p).as_raw_ptr(),
        )
    }

    /// <p>Returns a copy of the rectangle that is translated <i>dx</i> along the x axis and <i>dy</i> along the y axis, relative to the current position. Positive values move the rectangle to the right and down.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRect QRect::translated(int dx, int dy) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#translated">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a copy of the rectangle that is translated <i>dx</i> along the x axis and <i>dy</i> along the y axis, relative to the current position. Positive values move the rectangle to the right and down.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#translate">translate</a>().</p></div>
    pub unsafe fn translated_2a(
        &self,
        dx: ::std::os::raw::c_int,
        dy: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QRect> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QRect_translated(self as *const crate::QRect, dx, dy);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRect QRect::translated(const QPoint& p) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#translated-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a copy of the rectangle that is translated <i>offset</i>.<a href="http://doc.qt.io/qt-5/qpoint.html#x">x()</a> along the x axis and <i>offset</i>.<a href="http://doc.qt.io/qt-5/qpoint.html#y">y()</a> along the y axis, relative to the current position.</p></div>
    pub unsafe fn translated_1a(
        &self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPoint>>,
    ) -> ::cpp_utils::CppBox<crate::QRect> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRect_translated1(
            self as *const crate::QRect,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPoint>>::cast_into(p).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a copy of the rectangle that has its width and height exchanged:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRect QRect::transposed() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#transposed">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a copy of the rectangle that has its width and height exchanged:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qrect.html#QRect">QRect</a></span> r <span class="operator">=</span> {<span class="number">15</span><span class="operator">,</span> <span class="number">51</span><span class="operator">,</span> <span class="number">42</span><span class="operator">,</span> <span class="number">24</span>};
    ///   r <span class="operator">=</span> r<span class="operator">.</span>transposed(); <span class="comment">// r == {15, 51, 24, 42}</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsize.html#transposed">QSize::transposed</a>().</p></div>
    pub unsafe fn transposed(&self) -> ::cpp_utils::CppBox<crate::QRect> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QRect_transposed(self as *const crate::QRect);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the bounding rectangle of this rectangle and the given <i>rectangle</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRect QRect::united(const QRect& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#united">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the bounding rectangle of this rectangle and the given <i>rectangle</i>.</p>
    /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qrect-unite.png" alt=""></p><p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#intersected">intersected</a>().</p></div>
    pub unsafe fn united(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRect>>,
    ) -> ::cpp_utils::CppBox<crate::QRect> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRect_united(
            self as *const crate::QRect,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRect>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the width of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRect::width() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#width">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the width of the rectangle.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#setWidth">setWidth</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#height">height</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#size">size</a>().</p></div>
    pub unsafe fn width(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRect_width(self as *const crate::QRect)
    }

    /// <p>Returns the x-coordinate of the rectangle's left edge. Equivalent to <a href="http://doc.qt.io/qt-5/qrect.html#left">left</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRect::x() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#x">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the x-coordinate of the rectangle's left edge. Equivalent to <a href="http://doc.qt.io/qt-5/qrect.html#left">left</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#setX">setX</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#y">y</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#topLeft">topLeft</a>().</p></div>
    pub unsafe fn x(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRect_x(self as *const crate::QRect)
    }

    /// <p>Returns the y-coordinate of the rectangle's top edge. Equivalent to <a href="http://doc.qt.io/qt-5/qrect.html#top">top</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRect::y() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#y">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the y-coordinate of the rectangle's top edge. Equivalent to <a href="http://doc.qt.io/qt-5/qrect.html#top">top</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#setY">setY</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#x">x</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#topLeft">topLeft</a>().</p></div>
    pub unsafe fn y(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRect_y(self as *const crate::QRect)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> class defines a rectangle in the plane using floating point precision.</p>
///
/// C++ class: <span style='color: green;'>```QRectF```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qrectf.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> class defines a rectangle in the plane using floating point precision.</p>
/// <p>A rectangle is normally expressed as an upper-left corner and a size. The size (width and height) of a <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> is always equivalent to the mathematical rectangle that forms the basis for its rendering.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> can be constructed with a set of left, top, width and height coordinates, or from a <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> and a <a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a>. The following code creates two identical rectangles.</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qrectf.html#QRectF">QRectF</a></span> r1(<span class="number">100.0</span><span class="operator">,</span> <span class="number">200.1</span><span class="operator">,</span> <span class="number">11.2</span><span class="operator">,</span> <span class="number">16.3</span>);
///   <span class="type"><a href="http://doc.qt.io/qt-5/qrectf.html#QRectF">QRectF</a></span> r2(<span class="type"><a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a></span>(<span class="number">100.0</span><span class="operator">,</span> <span class="number">200.1</span>)<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a></span>(<span class="number">11.2</span><span class="operator">,</span> <span class="number">16.3</span>));
///
/// </pre>
/// <p>There is also a third constructor creating a <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> from a <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a>, and a corresponding <a href="http://doc.qt.io/qt-5/qrectf.html#toRect">toRect</a>() function that returns a <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> object based on the values of this rectangle (note that the coordinates in the returned rectangle are rounded to the nearest integer).</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> class provides a collection of functions that return the various rectangle coordinates, and enable manipulation of these. <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> also provide functions to move the rectangle relative to the various coordinates. In addition there is a <a href="http://doc.qt.io/qt-5/qrectf.html#moveTo">moveTo</a>() function that moves the rectangle, leaving its top left corner at the given coordinates. Alternatively, the <a href="http://doc.qt.io/qt-5/qrectf.html#translate">translate</a>() function moves the rectangle the given offset relative to the current position, and the <a href="http://doc.qt.io/qt-5/qrectf.html#translated">translated</a>() function returns a translated copy of this rectangle.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qrectf.html#size">size</a>() function returns the rectange's dimensions as a <a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a>. The dimensions can also be retrieved separately using the <a href="http://doc.qt.io/qt-5/qrectf.html#width">width</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#height">height</a>() functions. To manipulate the dimensions use the <a href="http://doc.qt.io/qt-5/qrectf.html#setSize">setSize</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#setWidth">setWidth</a>() or <a href="http://doc.qt.io/qt-5/qrectf.html#setHeight">setHeight</a>() functions. Alternatively, the size can be changed by applying either of the functions setting the rectangle coordinates, for example, <a href="http://doc.qt.io/qt-5/qrectf.html#setBottom">setBottom</a>() or <a href="http://doc.qt.io/qt-5/qrectf.html#setRight">setRight</a>().</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qrectf.html#contains">contains</a>() function tells whether a given point is inside the rectangle or not, and the <a href="http://doc.qt.io/qt-5/qrectf.html#intersects">intersects</a>() function returns <code>true</code> if this rectangle intersects with a given rectangle (otherwise false). The <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> class also provides the <a href="http://doc.qt.io/qt-5/qrectf.html#intersected">intersected</a>() function which returns the intersection rectangle, and the <a href="http://doc.qt.io/qt-5/qrectf.html#united">united</a>() function which returns the rectangle that encloses the given rectangle and this:</p>
/// <div class="table"><table class="generic">
///  <tbody><tr valign="top" class="odd"><td><img src="http://doc.qt.io/qt-5/images/qrect-intersect.png" alt=""></td><td><img src="http://doc.qt.io/qt-5/images/qrect-unite.png" alt=""></td></tr>
/// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qrectf.html#intersected">intersected</a>()</td><td><a href="http://doc.qt.io/qt-5/qrectf.html#united">united</a>()</td></tr>
/// </tbody></table></div>
/// <p>The <a href="http://doc.qt.io/qt-5/qrectf.html#isEmpty">isEmpty</a>() function returns <code>true</code> if the rectangle's width or height is less than, or equal to, 0. Note that an empty rectangle is not valid: The <a href="http://doc.qt.io/qt-5/qrectf.html#isValid">isValid</a>() function returns <code>true</code> if both width and height is larger than 0. A null rectangle (<a href="http://doc.qt.io/qt-5/qrectf.html#isNull">isNull</a>() == true) on the other hand, has both width and height set to 0.</p>
/// <p>Note that due to the way <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> and <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> are defined, an empty <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> is defined in essentially the same way as <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a>.</p>
/// <p>Finally, <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> objects can be streamed as well as compared.</p>
/// <a name="rendering"></a></div>
#[repr(C)]
pub struct QRectF {
    _unused: u8,
}
impl QRectF {
    /// <p>Adds <i>dx1</i>, <i>dy1</i>, <i>dx2</i> and <i>dy2</i> respectively to the existing coordinates of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::adjust(double x1, double y1, double x2, double y2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#adjust">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds <i>dx1</i>, <i>dy1</i>, <i>dx2</i> and <i>dy2</i> respectively to the existing coordinates of the rectangle.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#adjusted">adjusted</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#setRect">setRect</a>().</p></div>
    pub unsafe fn adjust(
        &mut self,
        x1: ::std::os::raw::c_double,
        y1: ::std::os::raw::c_double,
        x2: ::std::os::raw::c_double,
        y2: ::std::os::raw::c_double,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_adjust(self as *mut crate::QRectF, x1, y1, x2, y2)
    }

    /// <p>Returns a new rectangle with <i>dx1</i>, <i>dy1</i>, <i>dx2</i> and <i>dy2</i> added respectively to the existing coordinates of this rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRectF QRectF::adjusted(double x1, double y1, double x2, double y2) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#adjusted">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a new rectangle with <i>dx1</i>, <i>dy1</i>, <i>dx2</i> and <i>dy2</i> added respectively to the existing coordinates of this rectangle.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#adjust">adjust</a>().</p></div>
    pub unsafe fn adjusted(
        &self,
        x1: ::std::os::raw::c_double,
        y1: ::std::os::raw::c_double,
        x2: ::std::os::raw::c_double,
        y2: ::std::os::raw::c_double,
    ) -> ::cpp_utils::CppBox<crate::QRectF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRectF_adjusted(
            self as *const crate::QRectF,
            x1,
            y1,
            x2,
            y2,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the y-coordinate of the rectangle's bottom edge.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QRectF::bottom() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#bottom">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the y-coordinate of the rectangle's bottom edge.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#setBottom">setBottom</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#bottomLeft">bottomLeft</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#bottomRight">bottomRight</a>().</p></div>
    pub unsafe fn bottom(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QRectF_bottom(self as *const crate::QRectF)
    }

    /// <p>Returns the position of the rectangle's bottom-left corner.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPointF QRectF::bottomLeft() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#bottomLeft">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the position of the rectangle's bottom-left corner.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#setBottomLeft">setBottomLeft</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#bottom">bottom</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#left">left</a>().</p></div>
    pub unsafe fn bottom_left(&self) -> ::cpp_utils::CppBox<crate::QPointF> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QRectF_bottomLeft(self as *const crate::QRectF);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the position of the rectangle's bottom-right corner.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPointF QRectF::bottomRight() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#bottomRight">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the position of the rectangle's bottom-right corner.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#setBottomRight">setBottomRight</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#bottom">bottom</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#right">right</a>().</p></div>
    pub unsafe fn bottom_right(&self) -> ::cpp_utils::CppBox<crate::QPointF> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QRectF_bottomRight(self as *const crate::QRectF);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the center point of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPointF QRectF::center() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#center">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the center point of the rectangle.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#moveCenter">moveCenter</a>().</p></div>
    pub unsafe fn center(&self) -> ::cpp_utils::CppBox<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRectF_center(self as *const crate::QRectF);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRectF::contains(const QRectF& r) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#contains-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the given <i>rectangle</i> is inside this rectangle; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn contains_q_rect_f(
        &self,
        r: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRectF>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRectF_contains(
            self as *const crate::QRectF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRectF>>::cast_into(r).as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if the given <i>point</i> is inside or on the edge of the rectangle; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRectF::contains(const QPointF& p) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the given <i>point</i> is inside or on the edge of the rectangle; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#intersects">intersects</a>().</p></div>
    pub unsafe fn contains_q_point_f(
        &self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRectF_contains1(
            self as *const crate::QRectF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(p).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRectF::contains(double x, double y) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#contains-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the point (<i>x</i>, <i>y</i>) is inside or on the edge of the rectangle; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn contains_2_double(
        &self,
        x: ::std::os::raw::c_double,
        y: ::std::os::raw::c_double,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRectF_contains2(self as *const crate::QRectF, x, y)
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> class defines a rectangle in the plane using floating point precision.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRectF& QRectF::operator=(const QRectF& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> class defines a rectangle in the plane using floating point precision.</p>
    /// <p>A rectangle is normally expressed as an upper-left corner and a size. The size (width and height) of a <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> is always equivalent to the mathematical rectangle that forms the basis for its rendering.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> can be constructed with a set of left, top, width and height coordinates, or from a <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> and a <a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a>. The following code creates two identical rectangles.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qrectf.html#QRectF">QRectF</a></span> r1(<span class="number">100.0</span><span class="operator">,</span> <span class="number">200.1</span><span class="operator">,</span> <span class="number">11.2</span><span class="operator">,</span> <span class="number">16.3</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qrectf.html#QRectF">QRectF</a></span> r2(<span class="type"><a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a></span>(<span class="number">100.0</span><span class="operator">,</span> <span class="number">200.1</span>)<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a></span>(<span class="number">11.2</span><span class="operator">,</span> <span class="number">16.3</span>));
    ///
    /// </pre>
    /// <p>There is also a third constructor creating a <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> from a <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a>, and a corresponding <a href="http://doc.qt.io/qt-5/qrectf.html#toRect">toRect</a>() function that returns a <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> object based on the values of this rectangle (note that the coordinates in the returned rectangle are rounded to the nearest integer).</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> class provides a collection of functions that return the various rectangle coordinates, and enable manipulation of these. <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> also provide functions to move the rectangle relative to the various coordinates. In addition there is a <a href="http://doc.qt.io/qt-5/qrectf.html#moveTo">moveTo</a>() function that moves the rectangle, leaving its top left corner at the given coordinates. Alternatively, the <a href="http://doc.qt.io/qt-5/qrectf.html#translate">translate</a>() function moves the rectangle the given offset relative to the current position, and the <a href="http://doc.qt.io/qt-5/qrectf.html#translated">translated</a>() function returns a translated copy of this rectangle.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qrectf.html#size">size</a>() function returns the rectange's dimensions as a <a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a>. The dimensions can also be retrieved separately using the <a href="http://doc.qt.io/qt-5/qrectf.html#width">width</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#height">height</a>() functions. To manipulate the dimensions use the <a href="http://doc.qt.io/qt-5/qrectf.html#setSize">setSize</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#setWidth">setWidth</a>() or <a href="http://doc.qt.io/qt-5/qrectf.html#setHeight">setHeight</a>() functions. Alternatively, the size can be changed by applying either of the functions setting the rectangle coordinates, for example, <a href="http://doc.qt.io/qt-5/qrectf.html#setBottom">setBottom</a>() or <a href="http://doc.qt.io/qt-5/qrectf.html#setRight">setRight</a>().</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qrectf.html#contains">contains</a>() function tells whether a given point is inside the rectangle or not, and the <a href="http://doc.qt.io/qt-5/qrectf.html#intersects">intersects</a>() function returns <code>true</code> if this rectangle intersects with a given rectangle (otherwise false). The <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> class also provides the <a href="http://doc.qt.io/qt-5/qrectf.html#intersected">intersected</a>() function which returns the intersection rectangle, and the <a href="http://doc.qt.io/qt-5/qrectf.html#united">united</a>() function which returns the rectangle that encloses the given rectangle and this:</p>
    /// <div class="table"><table class="generic">
    ///  <tbody><tr valign="top" class="odd"><td><img src="http://doc.qt.io/qt-5/images/qrect-intersect.png" alt=""></td><td><img src="http://doc.qt.io/qt-5/images/qrect-unite.png" alt=""></td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qrectf.html#intersected">intersected</a>()</td><td><a href="http://doc.qt.io/qt-5/qrectf.html#united">united</a>()</td></tr>
    /// </tbody></table></div>
    /// <p>The <a href="http://doc.qt.io/qt-5/qrectf.html#isEmpty">isEmpty</a>() function returns <code>true</code> if the rectangle's width or height is less than, or equal to, 0. Note that an empty rectangle is not valid: The <a href="http://doc.qt.io/qt-5/qrectf.html#isValid">isValid</a>() function returns <code>true</code> if both width and height is larger than 0. A null rectangle (<a href="http://doc.qt.io/qt-5/qrectf.html#isNull">isNull</a>() == true) on the other hand, has both width and height set to 0.</p>
    /// <p>Note that due to the way <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> and <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> are defined, an empty <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> is defined in essentially the same way as <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a>.</p>
    /// <p>Finally, <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> objects can be streamed as well as compared.</p>
    /// <a name="rendering"></a></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRectF>>,
    ) -> ::cpp_utils::MutRef<crate::QRectF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRectF_operator_2(
            self as *mut crate::QRectF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRectF>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Extracts the position of the rectangle's top-left corner to *<i>x1</i> and *<i>y1</i>, and the position of the bottom-right corner to *<i>x2</i> and *<i>y2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::getCoords(double* x1, double* y1, double* x2, double* y2) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#getCoords">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Extracts the position of the rectangle's top-left corner to *<i>x1</i> and *<i>y1</i>, and the position of the bottom-right corner to *<i>x2</i> and *<i>y2</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#setCoords">setCoords</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#getRect">getRect</a>().</p></div>
    pub unsafe fn get_coords(
        &self,
        x1: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_double>>,
        y1: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_double>>,
        x2: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_double>>,
        y2: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_double>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_getCoords(
            self as *const crate::QRectF,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_double>>::cast_into(x1)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_double>>::cast_into(y1)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_double>>::cast_into(x2)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_double>>::cast_into(y2)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Extracts the position of the rectangle's top-left corner to *<i>x</i> and *<i>y</i>, and its dimensions to *<i>width</i> and *<i>height</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::getRect(double* x, double* y, double* w, double* h) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#getRect">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Extracts the position of the rectangle's top-left corner to *<i>x</i> and *<i>y</i>, and its dimensions to *<i>width</i> and *<i>height</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#setRect">setRect</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#getCoords">getCoords</a>().</p></div>
    pub unsafe fn get_rect(
        &self,
        x: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_double>>,
        y: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_double>>,
        w: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_double>>,
        h: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_double>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_getRect(
            self as *const crate::QRectF,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_double>>::cast_into(x)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_double>>::cast_into(y)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_double>>::cast_into(w)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_double>>::cast_into(h)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the height of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QRectF::height() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#height">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the height of the rectangle.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#setHeight">setHeight</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#width">width</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#size">size</a>().</p></div>
    pub unsafe fn height(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QRectF_height(self as *const crate::QRectF)
    }

    /// <p>Returns the intersection of this rectangle and the given <i>rectangle</i>. Note that <code>r.intersected(s)</code> is equivalent to <code>r &amp; s</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRectF QRectF::intersected(const QRectF& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#intersected">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the intersection of this rectangle and the given <i>rectangle</i>. Note that <code>r.intersected(s)</code> is equivalent to <code>r &amp; s</code>.</p>
    /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qrect-intersect.png" alt=""></p><p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#intersects">intersects</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#united">united</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#operator-and-eq">operator&amp;=</a>().</p></div>
    pub unsafe fn intersected(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRectF>>,
    ) -> ::cpp_utils::CppBox<crate::QRectF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRectF_intersected(
            self as *const crate::QRectF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRectF>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if this rectangle intersects with the given <i>rectangle</i> (i.e. there is a non-empty area of overlap between them), otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRectF::intersects(const QRectF& r) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#intersects">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this rectangle intersects with the given <i>rectangle</i> (i.e. there is a non-empty area of overlap between them), otherwise returns <code>false</code>.</p>
    /// <p>The intersection rectangle can be retrieved using the <a href="http://doc.qt.io/qt-5/qrectf.html#intersected">intersected</a>() function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#contains">contains</a>().</p></div>
    pub unsafe fn intersects(
        &self,
        r: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRectF>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRectF_intersects(
            self as *const crate::QRectF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRectF>>::cast_into(r).as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if the rectangle is empty, otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRectF::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the rectangle is empty, otherwise returns <code>false</code>.</p>
    /// <p>An empty rectangle has <a href="http://doc.qt.io/qt-5/qrectf.html#width">width</a>() &lt;= 0 or <a href="http://doc.qt.io/qt-5/qrectf.html#height">height</a>() &lt;= 0. An empty rectangle is not valid (i.e., isEmpty() == !<a href="http://doc.qt.io/qt-5/qrectf.html#isValid">isValid</a>()).</p>
    /// <p>Use the <a href="http://doc.qt.io/qt-5/qrectf.html#normalized">normalized</a>() function to retrieve a rectangle where the corners are swapped.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#isNull">isNull</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#isValid">isValid</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#normalized">normalized</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRectF_isEmpty(self as *const crate::QRectF)
    }

    /// <p>Returns <code>true</code> if the rectangle is a null rectangle, otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRectF::isNull() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#isNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the rectangle is a null rectangle, otherwise returns <code>false</code>.</p>
    /// <p>A null rectangle has both the width and the height set to 0. A null rectangle is also empty, and hence not valid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#isValid">isValid</a>().</p></div>
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRectF_isNull(self as *const crate::QRectF)
    }

    /// <p>Returns <code>true</code> if the rectangle is valid, otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRectF::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the rectangle is valid, otherwise returns <code>false</code>.</p>
    /// <p>A valid rectangle has a <a href="http://doc.qt.io/qt-5/qrectf.html#width">width</a>() &gt; 0 and <a href="http://doc.qt.io/qt-5/qrectf.html#height">height</a>() &gt; 0. Note that non-trivial operations like intersections are not defined for invalid rectangles. A valid rectangle is not empty (i.e., isValid() == !<a href="http://doc.qt.io/qt-5/qrectf.html#isEmpty">isEmpty</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#isNull">isNull</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#normalized">normalized</a>().</p></div>
    pub unsafe fn is_valid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRectF_isValid(self as *const crate::QRectF)
    }

    /// <p>Returns the x-coordinate of the rectangle's left edge. Equivalent to <a href="http://doc.qt.io/qt-5/qrectf.html#x">x</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QRectF::left() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#left">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the x-coordinate of the rectangle's left edge. Equivalent to <a href="http://doc.qt.io/qt-5/qrectf.html#x">x</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#setLeft">setLeft</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#topLeft">topLeft</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#bottomLeft">bottomLeft</a>().</p></div>
    pub unsafe fn left(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QRectF_left(self as *const crate::QRectF)
    }

    /// <p>Returns a rectangle grown by the <i>margins</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRectF QRectF::marginsAdded(const QMarginsF& margins) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#marginsAdded">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a rectangle grown by the <i>margins</i>.</p>
    /// <p>This function was introduced in  Qt 5.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#operator-2b-eq">operator+=</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#marginsRemoved">marginsRemoved</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#operator--eq">operator-=</a>().</p></div>
    pub unsafe fn margins_added(
        &self,
        margins: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMarginsF>>,
    ) -> ::cpp_utils::CppBox<crate::QRectF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRectF_marginsAdded(
            self as *const crate::QRectF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMarginsF>>::cast_into(margins)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the <i>margins</i> from the rectangle, shrinking it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRectF QRectF::marginsRemoved(const QMarginsF& margins) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#marginsRemoved">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the <i>margins</i> from the rectangle, shrinking it.</p>
    /// <p>This function was introduced in  Qt 5.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#marginsAdded">marginsAdded</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#operator-2b-eq">operator+=</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#operator--eq">operator-=</a>().</p></div>
    pub unsafe fn margins_removed(
        &self,
        margins: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMarginsF>>,
    ) -> ::cpp_utils::CppBox<crate::QRectF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRectF_marginsRemoved(
            self as *const crate::QRectF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMarginsF>>::cast_into(margins)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the rectangle vertically, leaving the rectangle's bottom edge at the given <i>y</i> coordinate. The rectangle's size is unchanged.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::moveBottom(double pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#moveBottom">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the rectangle vertically, leaving the rectangle's bottom edge at the given <i>y</i> coordinate. The rectangle's size is unchanged.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#bottom">bottom</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#setBottom">setBottom</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#moveTop">moveTop</a>().</p></div>
    pub unsafe fn move_bottom(&mut self, pos: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_moveBottom(self as *mut crate::QRectF, pos)
    }

    /// <p>Moves the rectangle, leaving the bottom-left corner at the given <i>position</i>. The rectangle's size is unchanged.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::moveBottomLeft(const QPointF& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#moveBottomLeft">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the rectangle, leaving the bottom-left corner at the given <i>position</i>. The rectangle's size is unchanged.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#setBottomLeft">setBottomLeft</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#moveBottom">moveBottom</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#moveLeft">moveLeft</a>().</p></div>
    pub unsafe fn move_bottom_left(
        &mut self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_moveBottomLeft(
            self as *mut crate::QRectF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(p).as_raw_ptr(),
        )
    }

    /// <p>Moves the rectangle, leaving the bottom-right corner at the given <i>position</i>. The rectangle's size is unchanged.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::moveBottomRight(const QPointF& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#moveBottomRight">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the rectangle, leaving the bottom-right corner at the given <i>position</i>. The rectangle's size is unchanged.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#setBottomRight">setBottomRight</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#moveBottom">moveBottom</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#moveRight">moveRight</a>().</p></div>
    pub unsafe fn move_bottom_right(
        &mut self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_moveBottomRight(
            self as *mut crate::QRectF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(p).as_raw_ptr(),
        )
    }

    /// <p>Moves the rectangle, leaving the center point at the given <i>position</i>. The rectangle's size is unchanged.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::moveCenter(const QPointF& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#moveCenter">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the rectangle, leaving the center point at the given <i>position</i>. The rectangle's size is unchanged.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#center">center</a>().</p></div>
    pub unsafe fn move_center(
        &mut self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_moveCenter(
            self as *mut crate::QRectF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(p).as_raw_ptr(),
        )
    }

    /// <p>Moves the rectangle horizontally, leaving the rectangle's left edge at the given <i>x</i> coordinate. The rectangle's size is unchanged.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::moveLeft(double pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#moveLeft">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the rectangle horizontally, leaving the rectangle's left edge at the given <i>x</i> coordinate. The rectangle's size is unchanged.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#left">left</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#setLeft">setLeft</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#moveRight">moveRight</a>().</p></div>
    pub unsafe fn move_left(&mut self, pos: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_moveLeft(self as *mut crate::QRectF, pos)
    }

    /// <p>Moves the rectangle horizontally, leaving the rectangle's right edge at the given <i>x</i> coordinate. The rectangle's size is unchanged.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::moveRight(double pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#moveRight">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the rectangle horizontally, leaving the rectangle's right edge at the given <i>x</i> coordinate. The rectangle's size is unchanged.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#right">right</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#setRight">setRight</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#moveLeft">moveLeft</a>().</p></div>
    pub unsafe fn move_right(&mut self, pos: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_moveRight(self as *mut crate::QRectF, pos)
    }

    /// <p>Moves the rectangle, leaving the top-left corner at the given position (<i>x</i>, <i>y</i>). The rectangle's size is unchanged.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::moveTo(double x, double y)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#moveTo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the rectangle, leaving the top-left corner at the given position (<i>x</i>, <i>y</i>). The rectangle's size is unchanged.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#translate">translate</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#moveTopLeft">moveTopLeft</a>().</p></div>
    pub unsafe fn move_to_2a(&mut self, x: ::std::os::raw::c_double, y: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_moveTo(self as *mut crate::QRectF, x, y)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::moveTo(const QPointF& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#moveTo-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Moves the rectangle, leaving the top-left corner at the given <i>position</i>.</p></div>
    pub unsafe fn move_to_1a(
        &mut self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_moveTo1(
            self as *mut crate::QRectF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(p).as_raw_ptr(),
        )
    }

    /// <p>Moves the rectangle vertically, leaving the rectangle's top line at the given <i>y</i> coordinate. The rectangle's size is unchanged.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::moveTop(double pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#moveTop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the rectangle vertically, leaving the rectangle's top line at the given <i>y</i> coordinate. The rectangle's size is unchanged.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#top">top</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#setTop">setTop</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#moveBottom">moveBottom</a>().</p></div>
    pub unsafe fn move_top(&mut self, pos: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_moveTop(self as *mut crate::QRectF, pos)
    }

    /// <p>Moves the rectangle, leaving the top-left corner at the given <i>position</i>. The rectangle's size is unchanged.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::moveTopLeft(const QPointF& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#moveTopLeft">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the rectangle, leaving the top-left corner at the given <i>position</i>. The rectangle's size is unchanged.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#setTopLeft">setTopLeft</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#moveTop">moveTop</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#moveLeft">moveLeft</a>().</p></div>
    pub unsafe fn move_top_left(
        &mut self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_moveTopLeft(
            self as *mut crate::QRectF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(p).as_raw_ptr(),
        )
    }

    /// <p>Moves the rectangle, leaving the top-right corner at the given <i>position</i>. The rectangle's size is unchanged.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::moveTopRight(const QPointF& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#moveTopRight">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the rectangle, leaving the top-right corner at the given <i>position</i>. The rectangle's size is unchanged.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#setTopRight">setTopRight</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#moveTop">moveTop</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#moveRight">moveRight</a>().</p></div>
    pub unsafe fn move_top_right(
        &mut self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_moveTopRight(
            self as *mut crate::QRectF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(p).as_raw_ptr(),
        )
    }

    /// <p>Constructs a null rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRectF::QRectF()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#QRectF">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a null rectangle.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#isNull">isNull</a>().</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QRectF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRectF_QRectF();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a rectangle with the given <i>topLeft</i> corner and the given <i>size</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRectF::QRectF(const QPointF& topleft, const QSizeF& size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#QRectF-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a rectangle with the given <i>topLeft</i> corner and the given <i>size</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#setTopLeft">setTopLeft</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#setSize">setSize</a>().</p></div>
    pub unsafe fn from_q_point_f_q_size_f(
        topleft: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
        size: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSizeF>>,
    ) -> ::cpp_utils::CppBox<crate::QRectF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRectF_QRectF1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(topleft)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSizeF>>::cast_into(size).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a rectangle with the given <i>topLeft</i> and <i>bottomRight</i> corners.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRectF::QRectF(const QPointF& topleft, const QPointF& bottomRight)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#QRectF-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a rectangle with the given <i>topLeft</i> and <i>bottomRight</i> corners.</p>
    /// <p>This function was introduced in  Qt 4.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#setTopLeft">setTopLeft</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#setBottomRight">setBottomRight</a>().</p></div>
    pub unsafe fn from_2_q_point_f(
        topleft: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
        bottom_right: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) -> ::cpp_utils::CppBox<crate::QRectF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRectF_QRectF2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(topleft)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(bottom_right)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a rectangle with (<i>x</i>, <i>y</i>) as its top-left corner and the given <i>width</i> and <i>height</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRectF::QRectF(double left, double top, double width, double height)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#QRectF-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a rectangle with (<i>x</i>, <i>y</i>) as its top-left corner and the given <i>width</i> and <i>height</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#setRect">setRect</a>().</p></div>
    pub unsafe fn from_4_double(
        left: ::std::os::raw::c_double,
        top: ::std::os::raw::c_double,
        width: ::std::os::raw::c_double,
        height: ::std::os::raw::c_double,
    ) -> ::cpp_utils::CppBox<crate::QRectF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRectF_QRectF3(left, top, width, height);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> rectangle from the given <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> <i>rectangle</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRectF::QRectF(const QRect& rect)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#QRectF-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> rectangle from the given <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> <i>rectangle</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#toRect">toRect</a>().</p></div>
    pub unsafe fn from_q_rect(
        rect: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRect>>,
    ) -> ::cpp_utils::CppBox<crate::QRectF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRectF_QRectF4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRect>>::cast_into(rect).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> class defines a rectangle in the plane using floating point precision.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRectF::QRectF(const QRectF& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> class defines a rectangle in the plane using floating point precision.</p>
    /// <p>A rectangle is normally expressed as an upper-left corner and a size. The size (width and height) of a <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> is always equivalent to the mathematical rectangle that forms the basis for its rendering.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> can be constructed with a set of left, top, width and height coordinates, or from a <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> and a <a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a>. The following code creates two identical rectangles.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qrectf.html#QRectF">QRectF</a></span> r1(<span class="number">100.0</span><span class="operator">,</span> <span class="number">200.1</span><span class="operator">,</span> <span class="number">11.2</span><span class="operator">,</span> <span class="number">16.3</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qrectf.html#QRectF">QRectF</a></span> r2(<span class="type"><a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a></span>(<span class="number">100.0</span><span class="operator">,</span> <span class="number">200.1</span>)<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a></span>(<span class="number">11.2</span><span class="operator">,</span> <span class="number">16.3</span>));
    ///
    /// </pre>
    /// <p>There is also a third constructor creating a <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> from a <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a>, and a corresponding <a href="http://doc.qt.io/qt-5/qrectf.html#toRect">toRect</a>() function that returns a <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> object based on the values of this rectangle (note that the coordinates in the returned rectangle are rounded to the nearest integer).</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> class provides a collection of functions that return the various rectangle coordinates, and enable manipulation of these. <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> also provide functions to move the rectangle relative to the various coordinates. In addition there is a <a href="http://doc.qt.io/qt-5/qrectf.html#moveTo">moveTo</a>() function that moves the rectangle, leaving its top left corner at the given coordinates. Alternatively, the <a href="http://doc.qt.io/qt-5/qrectf.html#translate">translate</a>() function moves the rectangle the given offset relative to the current position, and the <a href="http://doc.qt.io/qt-5/qrectf.html#translated">translated</a>() function returns a translated copy of this rectangle.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qrectf.html#size">size</a>() function returns the rectange's dimensions as a <a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a>. The dimensions can also be retrieved separately using the <a href="http://doc.qt.io/qt-5/qrectf.html#width">width</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#height">height</a>() functions. To manipulate the dimensions use the <a href="http://doc.qt.io/qt-5/qrectf.html#setSize">setSize</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#setWidth">setWidth</a>() or <a href="http://doc.qt.io/qt-5/qrectf.html#setHeight">setHeight</a>() functions. Alternatively, the size can be changed by applying either of the functions setting the rectangle coordinates, for example, <a href="http://doc.qt.io/qt-5/qrectf.html#setBottom">setBottom</a>() or <a href="http://doc.qt.io/qt-5/qrectf.html#setRight">setRight</a>().</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qrectf.html#contains">contains</a>() function tells whether a given point is inside the rectangle or not, and the <a href="http://doc.qt.io/qt-5/qrectf.html#intersects">intersects</a>() function returns <code>true</code> if this rectangle intersects with a given rectangle (otherwise false). The <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> class also provides the <a href="http://doc.qt.io/qt-5/qrectf.html#intersected">intersected</a>() function which returns the intersection rectangle, and the <a href="http://doc.qt.io/qt-5/qrectf.html#united">united</a>() function which returns the rectangle that encloses the given rectangle and this:</p>
    /// <div class="table"><table class="generic">
    ///  <tbody><tr valign="top" class="odd"><td><img src="http://doc.qt.io/qt-5/images/qrect-intersect.png" alt=""></td><td><img src="http://doc.qt.io/qt-5/images/qrect-unite.png" alt=""></td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qrectf.html#intersected">intersected</a>()</td><td><a href="http://doc.qt.io/qt-5/qrectf.html#united">united</a>()</td></tr>
    /// </tbody></table></div>
    /// <p>The <a href="http://doc.qt.io/qt-5/qrectf.html#isEmpty">isEmpty</a>() function returns <code>true</code> if the rectangle's width or height is less than, or equal to, 0. Note that an empty rectangle is not valid: The <a href="http://doc.qt.io/qt-5/qrectf.html#isValid">isValid</a>() function returns <code>true</code> if both width and height is larger than 0. A null rectangle (<a href="http://doc.qt.io/qt-5/qrectf.html#isNull">isNull</a>() == true) on the other hand, has both width and height set to 0.</p>
    /// <p>Note that due to the way <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> and <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> are defined, an empty <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> is defined in essentially the same way as <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a>.</p>
    /// <p>Finally, <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> objects can be streamed as well as compared.</p>
    /// <a name="rendering"></a></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRectF>>,
    ) -> ::cpp_utils::CppBox<crate::QRectF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRectF_QRectF5(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRectF>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a normalized rectangle; i.e., a rectangle that has a non-negative width and height.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRectF QRectF::normalized() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#normalized">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a normalized rectangle; i.e., a rectangle that has a non-negative width and height.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qrectf.html#width">width</a>() &lt; 0 the function swaps the left and right corners, and it swaps the top and bottom corners if <a href="http://doc.qt.io/qt-5/qrectf.html#height">height</a>() &lt; 0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#isValid">isValid</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn normalized(&self) -> ::cpp_utils::CppBox<crate::QRectF> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QRectF_normalized(self as *const crate::QRectF);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the x-coordinate of the rectangle's right edge.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QRectF::right() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#right">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the x-coordinate of the rectangle's right edge.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#setRight">setRight</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#topRight">topRight</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#bottomRight">bottomRight</a>().</p></div>
    pub unsafe fn right(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QRectF_right(self as *const crate::QRectF)
    }

    /// <p>Sets the bottom edge of the rectangle to the given <i>y</i> coordinate. May change the height, but will never change the top edge of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::setBottom(double pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#setBottom">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the bottom edge of the rectangle to the given <i>y</i> coordinate. May change the height, but will never change the top edge of the rectangle.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#bottom">bottom</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#moveBottom">moveBottom</a>().</p></div>
    pub unsafe fn set_bottom(&mut self, pos: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_setBottom(self as *mut crate::QRectF, pos)
    }

    /// <p>Set the bottom-left corner of the rectangle to the given <i>position</i>. May change the size, but will never change the top-right corner of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::setBottomLeft(const QPointF& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#setBottomLeft">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Set the bottom-left corner of the rectangle to the given <i>position</i>. May change the size, but will never change the top-right corner of the rectangle.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#bottomLeft">bottomLeft</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#moveBottomLeft">moveBottomLeft</a>().</p></div>
    pub unsafe fn set_bottom_left(
        &mut self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_setBottomLeft(
            self as *mut crate::QRectF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(p).as_raw_ptr(),
        )
    }

    /// <p>Set the bottom-right corner of the rectangle to the given <i>position</i>. May change the size, but will never change the top-left corner of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::setBottomRight(const QPointF& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#setBottomRight">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Set the bottom-right corner of the rectangle to the given <i>position</i>. May change the size, but will never change the top-left corner of the rectangle.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#bottomRight">bottomRight</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#moveBottomRight">moveBottomRight</a>().</p></div>
    pub unsafe fn set_bottom_right(
        &mut self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_setBottomRight(
            self as *mut crate::QRectF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(p).as_raw_ptr(),
        )
    }

    /// <p>Sets the coordinates of the rectangle's top-left corner to (<i>x1</i>, <i>y1</i>), and the coordinates of its bottom-right corner to (<i>x2</i>, <i>y2</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::setCoords(double x1, double y1, double x2, double y2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#setCoords">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the coordinates of the rectangle's top-left corner to (<i>x1</i>, <i>y1</i>), and the coordinates of its bottom-right corner to (<i>x2</i>, <i>y2</i>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#getCoords">getCoords</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#setRect">setRect</a>().</p></div>
    pub unsafe fn set_coords(
        &mut self,
        x1: ::std::os::raw::c_double,
        y1: ::std::os::raw::c_double,
        x2: ::std::os::raw::c_double,
        y2: ::std::os::raw::c_double,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_setCoords(self as *mut crate::QRectF, x1, y1, x2, y2)
    }

    /// <p>Sets the height of the rectangle to the given <i>height</i>. The bottom edge is changed, but not the top one.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::setHeight(double h)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#setHeight">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the height of the rectangle to the given <i>height</i>. The bottom edge is changed, but not the top one.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#height">height</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#setSize">setSize</a>().</p></div>
    pub unsafe fn set_height(&mut self, h: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_setHeight(self as *mut crate::QRectF, h)
    }

    /// <p>Sets the left edge of the rectangle to the given <i>x</i> coordinate. May change the width, but will never change the right edge of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::setLeft(double pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#setLeft">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the left edge of the rectangle to the given <i>x</i> coordinate. May change the width, but will never change the right edge of the rectangle.</p>
    /// <p>Equivalent to <a href="http://doc.qt.io/qt-5/qrectf.html#setX">setX</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#left">left</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#moveLeft">moveLeft</a>().</p></div>
    pub unsafe fn set_left(&mut self, pos: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_setLeft(self as *mut crate::QRectF, pos)
    }

    /// <p>Sets the coordinates of the rectangle's top-left corner to (<i>x</i>, <i>y</i>), and its size to the given <i>width</i> and <i>height</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::setRect(double x, double y, double w, double h)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#setRect">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the coordinates of the rectangle's top-left corner to (<i>x</i>, <i>y</i>), and its size to the given <i>width</i> and <i>height</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#getRect">getRect</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#setCoords">setCoords</a>().</p></div>
    pub unsafe fn set_rect(
        &mut self,
        x: ::std::os::raw::c_double,
        y: ::std::os::raw::c_double,
        w: ::std::os::raw::c_double,
        h: ::std::os::raw::c_double,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_setRect(self as *mut crate::QRectF, x, y, w, h)
    }

    /// <p>Sets the right edge of the rectangle to the given <i>x</i> coordinate. May change the width, but will never change the left edge of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::setRight(double pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#setRight">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the right edge of the rectangle to the given <i>x</i> coordinate. May change the width, but will never change the left edge of the rectangle.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#right">right</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#moveRight">moveRight</a>().</p></div>
    pub unsafe fn set_right(&mut self, pos: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_setRight(self as *mut crate::QRectF, pos)
    }

    /// <p>Sets the size of the rectangle to the given <i>size</i>. The top-left corner is not moved.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::setSize(const QSizeF& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#setSize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the size of the rectangle to the given <i>size</i>. The top-left corner is not moved.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#size">size</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#setWidth">setWidth</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#setHeight">setHeight</a>().</p></div>
    pub unsafe fn set_size(
        &mut self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSizeF>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_setSize(
            self as *mut crate::QRectF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSizeF>>::cast_into(s).as_raw_ptr(),
        )
    }

    /// <p>Sets the top edge of the rectangle to the given <i>y</i> coordinate. May change the height, but will never change the bottom edge of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::setTop(double pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#setTop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the top edge of the rectangle to the given <i>y</i> coordinate. May change the height, but will never change the bottom edge of the rectangle.</p>
    /// <p>Equivalent to <a href="http://doc.qt.io/qt-5/qrectf.html#setY">setY</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#top">top</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#moveTop">moveTop</a>().</p></div>
    pub unsafe fn set_top(&mut self, pos: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_setTop(self as *mut crate::QRectF, pos)
    }

    /// <p>Set the top-left corner of the rectangle to the given <i>position</i>. May change the size, but will never change the bottom-right corner of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::setTopLeft(const QPointF& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#setTopLeft">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Set the top-left corner of the rectangle to the given <i>position</i>. May change the size, but will never change the bottom-right corner of the rectangle.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#topLeft">topLeft</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#moveTopLeft">moveTopLeft</a>().</p></div>
    pub unsafe fn set_top_left(
        &mut self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_setTopLeft(
            self as *mut crate::QRectF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(p).as_raw_ptr(),
        )
    }

    /// <p>Set the top-right corner of the rectangle to the given <i>position</i>. May change the size, but will never change the bottom-left corner of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::setTopRight(const QPointF& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#setTopRight">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Set the top-right corner of the rectangle to the given <i>position</i>. May change the size, but will never change the bottom-left corner of the rectangle.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#topRight">topRight</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#moveTopRight">moveTopRight</a>().</p></div>
    pub unsafe fn set_top_right(
        &mut self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_setTopRight(
            self as *mut crate::QRectF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(p).as_raw_ptr(),
        )
    }

    /// <p>Sets the width of the rectangle to the given <i>width</i>. The right edge is changed, but not the left one.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::setWidth(double w)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#setWidth">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the width of the rectangle to the given <i>width</i>. The right edge is changed, but not the left one.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#width">width</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#setSize">setSize</a>().</p></div>
    pub unsafe fn set_width(&mut self, w: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_setWidth(self as *mut crate::QRectF, w)
    }

    /// <p>Sets the left edge of the rectangle to the given <i>x</i> coordinate. May change the width, but will never change the right edge of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::setX(double pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#setX">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the left edge of the rectangle to the given <i>x</i> coordinate. May change the width, but will never change the right edge of the rectangle.</p>
    /// <p>Equivalent to <a href="http://doc.qt.io/qt-5/qrectf.html#setLeft">setLeft</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#x">x</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#setY">setY</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#setTopLeft">setTopLeft</a>().</p></div>
    pub unsafe fn set_x(&mut self, pos: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_setX(self as *mut crate::QRectF, pos)
    }

    /// <p>Sets the top edge of the rectangle to the given <i>y</i> coordinate. May change the height, but will never change the bottom edge of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::setY(double pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#setY">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the top edge of the rectangle to the given <i>y</i> coordinate. May change the height, but will never change the bottom edge of the rectangle.</p>
    /// <p>Equivalent to <a href="http://doc.qt.io/qt-5/qrectf.html#setTop">setTop</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#y">y</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#setX">setX</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#setTopLeft">setTopLeft</a>().</p></div>
    pub unsafe fn set_y(&mut self, pos: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_setY(self as *mut crate::QRectF, pos)
    }

    /// <p>Returns the size of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSizeF QRectF::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the size of the rectangle.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#setSize">setSize</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#width">width</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#height">height</a>().</p></div>
    pub unsafe fn size(&self) -> ::cpp_utils::CppBox<crate::QSizeF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRectF_size(self as *const crate::QRectF);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> based on the values of this rectangle that is the smallest possible integer rectangle that completely contains this rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRect QRectF::toAlignedRect() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#toAlignedRect">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> based on the values of this rectangle that is the smallest possible integer rectangle that completely contains this rectangle.</p>
    /// <p>This function was introduced in  Qt 4.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#toRect">toRect</a>().</p></div>
    pub unsafe fn to_aligned_rect(&self) -> ::cpp_utils::CppBox<crate::QRect> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QRectF_toAlignedRect(self as *const crate::QRectF);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> based on the values of this rectangle. Note that the coordinates in the returned rectangle are rounded to the nearest integer.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRect QRectF::toRect() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#toRect">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> based on the values of this rectangle. Note that the coordinates in the returned rectangle are rounded to the nearest integer.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#QRectF">QRectF</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#toAlignedRect">toAlignedRect</a>().</p></div>
    pub unsafe fn to_rect(&self) -> ::cpp_utils::CppBox<crate::QRect> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRectF_toRect(self as *const crate::QRectF);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the y-coordinate of the rectangle's top edge. Equivalent to <a href="http://doc.qt.io/qt-5/qrectf.html#y">y</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QRectF::top() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#top">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the y-coordinate of the rectangle's top edge. Equivalent to <a href="http://doc.qt.io/qt-5/qrectf.html#y">y</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#setTop">setTop</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#topLeft">topLeft</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#topRight">topRight</a>().</p></div>
    pub unsafe fn top(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QRectF_top(self as *const crate::QRectF)
    }

    /// <p>Returns the position of the rectangle's top-left corner.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPointF QRectF::topLeft() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#topLeft">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the position of the rectangle's top-left corner.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#setTopLeft">setTopLeft</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#top">top</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#left">left</a>().</p></div>
    pub unsafe fn top_left(&self) -> ::cpp_utils::CppBox<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRectF_topLeft(self as *const crate::QRectF);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the position of the rectangle's top-right corner.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPointF QRectF::topRight() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#topRight">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the position of the rectangle's top-right corner.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#setTopRight">setTopRight</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#top">top</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#right">right</a>().</p></div>
    pub unsafe fn top_right(&self) -> ::cpp_utils::CppBox<crate::QPointF> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QRectF_topRight(self as *const crate::QRectF);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the rectangle <i>dx</i> along the x-axis and <i>dy</i> along the y-axis, relative to the current position. Positive values move the rectangle to the right and downwards.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::translate(double dx, double dy)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#translate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the rectangle <i>dx</i> along the x-axis and <i>dy</i> along the y-axis, relative to the current position. Positive values move the rectangle to the right and downwards.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#moveTopLeft">moveTopLeft</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#moveTo">moveTo</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#translated">translated</a>().</p></div>
    pub unsafe fn translate_2a(
        &mut self,
        dx: ::std::os::raw::c_double,
        dy: ::std::os::raw::c_double,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_translate(self as *mut crate::QRectF, dx, dy)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRectF::translate(const QPointF& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#translate-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Moves the rectangle <i>offset</i>.<a href="http://doc.qt.io/qt-5/qpointf.html#x">x()</a> along the x axis and <i>offset</i>.<a href="http://doc.qt.io/qt-5/qpointf.html#y">y()</a> along the y axis, relative to the current position.</p></div>
    pub unsafe fn translate_1a(
        &mut self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_translate1(
            self as *mut crate::QRectF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(p).as_raw_ptr(),
        )
    }

    /// <p>Returns a copy of the rectangle that is translated <i>dx</i> along the x axis and <i>dy</i> along the y axis, relative to the current position. Positive values move the rectangle to the right and down.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRectF QRectF::translated(double dx, double dy) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#translated">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a copy of the rectangle that is translated <i>dx</i> along the x axis and <i>dy</i> along the y axis, relative to the current position. Positive values move the rectangle to the right and down.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#translate">translate</a>().</p></div>
    pub unsafe fn translated_2a(
        &self,
        dx: ::std::os::raw::c_double,
        dy: ::std::os::raw::c_double,
    ) -> ::cpp_utils::CppBox<crate::QRectF> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QRectF_translated(self as *const crate::QRectF, dx, dy);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRectF QRectF::translated(const QPointF& p) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#translated-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a copy of the rectangle that is translated <i>offset</i>.<a href="http://doc.qt.io/qt-5/qpointf.html#x">x()</a> along the x axis and <i>offset</i>.<a href="http://doc.qt.io/qt-5/qpointf.html#y">y()</a> along the y axis, relative to the current position.</p></div>
    pub unsafe fn translated_1a(
        &self,
        p: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) -> ::cpp_utils::CppBox<crate::QRectF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRectF_translated1(
            self as *const crate::QRectF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(p).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a copy of the rectangle that has its width and height exchanged:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRectF QRectF::transposed() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#transposed">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a copy of the rectangle that has its width and height exchanged:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qrectf.html#QRectF">QRectF</a></span> r <span class="operator">=</span> {<span class="number">1.5</span><span class="operator">,</span> <span class="number">5.1</span><span class="operator">,</span> <span class="number">4.2</span><span class="operator">,</span> <span class="number">2.4</span>};
    ///   r <span class="operator">=</span> r<span class="operator">.</span>transposed(); <span class="comment">// r == {1.5, 5.1, 2.4, 4.2}</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsizef.html#transposed">QSizeF::transposed</a>().</p></div>
    pub unsafe fn transposed(&self) -> ::cpp_utils::CppBox<crate::QRectF> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QRectF_transposed(self as *const crate::QRectF);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the bounding rectangle of this rectangle and the given <i>rectangle</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRectF QRectF::united(const QRectF& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#united">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the bounding rectangle of this rectangle and the given <i>rectangle</i>.</p>
    /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qrect-unite.png" alt=""></p><p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#intersected">intersected</a>().</p></div>
    pub unsafe fn united(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRectF>>,
    ) -> ::cpp_utils::CppBox<crate::QRectF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRectF_united(
            self as *const crate::QRectF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRectF>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the width of the rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QRectF::width() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#width">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the width of the rectangle.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#setWidth">setWidth</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#height">height</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#size">size</a>().</p></div>
    pub unsafe fn width(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QRectF_width(self as *const crate::QRectF)
    }

    /// <p>Returns the x-coordinate of the rectangle's left edge. Equivalent to <a href="http://doc.qt.io/qt-5/qrectf.html#left">left</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QRectF::x() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#x">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the x-coordinate of the rectangle's left edge. Equivalent to <a href="http://doc.qt.io/qt-5/qrectf.html#left">left</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#setX">setX</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#y">y</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#topLeft">topLeft</a>().</p></div>
    pub unsafe fn x(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QRectF_x(self as *const crate::QRectF)
    }

    /// <p>Returns the y-coordinate of the rectangle's top edge. Equivalent to <a href="http://doc.qt.io/qt-5/qrectf.html#top">top</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QRectF::y() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#y">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the y-coordinate of the rectangle's top edge. Equivalent to <a href="http://doc.qt.io/qt-5/qrectf.html#top">top</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#setY">setY</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#x">x</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#topLeft">topLeft</a>().</p></div>
    pub unsafe fn y(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QRectF_y(self as *const crate::QRectF)
    }
}

pub mod q_regular_expression {
    //! C++ type: <span style='color: green;'>```QRegularExpression```</span>

    /// <p>The PatternOption enum defines modifiers to the way the pattern string should be interpreted, and therefore the way the pattern matches against a subject string.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QRegularExpression::PatternOption```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#PatternOption-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The PatternOption enum defines modifiers to the way the pattern string should be interpreted, and therefore the way the pattern matches against a subject string.</p>
    ///
    /// <p>The PatternOptions type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;PatternOption&gt;. It stores an OR combination of PatternOption values.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct PatternOption(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for PatternOption {
        fn from(value: ::std::os::raw::c_int) -> Self {
            PatternOption(value)
        }
    }

    impl From<PatternOption> for ::std::os::raw::c_int {
        fn from(value: PatternOption) -> Self {
            value.0
        }
    }

    impl PatternOption {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl PatternOption {
        /// No pattern options are set. (C++ enum variant: <span style='color: green;'>```NoPatternOption = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoPatternOption: crate::q_regular_expression::PatternOption =
            crate::q_regular_expression::PatternOption(0);
        /// The pattern should match against the subject string in a case insensitive way. This option corresponds to the /i modifier in Perl regular expressions. (C++ enum variant: <span style='color: green;'>```CaseInsensitiveOption = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const CaseInsensitiveOption: crate::q_regular_expression::PatternOption =
            crate::q_regular_expression::PatternOption(1);
        /// The dot metacharacter (<code>.</code>) in the pattern string is allowed to match any character in the subject string, including newlines (normally, the dot does not match newlines). This option corresponds to the <code>/s</code> modifier in Perl regular expressions. (C++ enum variant: <span style='color: green;'>```DotMatchesEverythingOption = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const DotMatchesEverythingOption: crate::q_regular_expression::PatternOption =
            crate::q_regular_expression::PatternOption(2);
        /// The caret (<code>^</code>) and the dollar (<code>$</code>) metacharacters in the pattern string are allowed to match, respectively, immediately after and immediately before any newline in the subject string, as well as at the very beginning and at the very end of the subject string. This option corresponds to the <code>/m</code> modifier in Perl regular expressions. (C++ enum variant: <span style='color: green;'>```MultilineOption = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const MultilineOption: crate::q_regular_expression::PatternOption =
            crate::q_regular_expression::PatternOption(4);
        /// Any whitespace in the pattern string which is not escaped and outside a character class is ignored. Moreover, an unescaped sharp (<b>#</b>) outside a character class causes all the following characters, until the first newline (included), to be ignored. This can be used to increase the readability of a pattern string as well as put comments inside regular expressions; this is particulary useful if the pattern string is loaded from a file or written by the user, because in C++ code it is always possible to use the rules for string literals to put comments outside the pattern string. This option corresponds to the <code>/x</code> modifier in Perl regular expressions. (C++ enum variant: <span style='color: green;'>```ExtendedPatternSyntaxOption = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const ExtendedPatternSyntaxOption: crate::q_regular_expression::PatternOption =
            crate::q_regular_expression::PatternOption(8);
        /// The greediness of the quantifiers is inverted: <code>*</code>, <code>+</code>, <code>?</code>, <code>{m,n}</code>, etc. become lazy, while their lazy versions (<code>*?</code>, <code>+?</code>, <code>??</code>, <code>{m,n}?</code>, etc.) become greedy. There is no equivalent for this option in Perl regular expressions. (C++ enum variant: <span style='color: green;'>```InvertedGreedinessOption = 16```</span>)
        #[allow(non_upper_case_globals)]
        pub const InvertedGreedinessOption: crate::q_regular_expression::PatternOption =
            crate::q_regular_expression::PatternOption(16);
        /// The non-named capturing groups do not capture substrings; named capturing groups still work as intended, as well as the implicit capturing group number 0 corresponding to the entire match. There is no equivalent for this option in Perl regular expressions. (C++ enum variant: <span style='color: green;'>```DontCaptureOption = 32```</span>)
        #[allow(non_upper_case_globals)]
        pub const DontCaptureOption: crate::q_regular_expression::PatternOption =
            crate::q_regular_expression::PatternOption(32);
        /// The meaning of the <code>\w</code>, <code>\d</code>, etc., character classes, as well as the meaning of their counterparts (<code>\W</code>, <code>\D</code>, etc.), is changed from matching ASCII characters only to matching any character with the corresponding Unicode property. For instance, <code>\d</code> is changed to match any character with the Unicode Nd (decimal digit) property; <code>\w</code> to match any character with either the Unicode L (letter) or N (digit) property, plus underscore, and so on. This option corresponds to the <code>/u</code> modifier in Perl regular expressions. (C++ enum variant: <span style='color: green;'>```UseUnicodePropertiesOption = 64```</span>)
        #[allow(non_upper_case_globals)]
        pub const UseUnicodePropertiesOption: crate::q_regular_expression::PatternOption =
            crate::q_regular_expression::PatternOption(64);
        /// The regular expression will be optimized (and possibly JIT-compiled) on its first usage, instead of after a certain (undefined) number of usages. See also <a href="http://doc.qt.io/qt-5/qregularexpression.html#optimize">optimize()</a>. This enum value has been introduced in Qt 5.4. (C++ enum variant: <span style='color: green;'>```OptimizeOnFirstUsageOption = 128```</span>)
        #[allow(non_upper_case_globals)]
        pub const OptimizeOnFirstUsageOption: crate::q_regular_expression::PatternOption =
            crate::q_regular_expression::PatternOption(128);
        /// Regular expressions are automatically optimized after a certain number of usages; setting this option prevents such optimizations, therefore avoiding possible unpredictable spikes in CPU and memory usage. If both this option and the <code>OptimizeOnFirstUsageOption</code> option are set, then this option takes precedence. Note: this option will still let the regular expression to be optimized by manually calling <a href="http://doc.qt.io/qt-5/qregularexpression.html#optimize">optimize()</a>. This enum value has been introduced in Qt 5.4. (C++ enum variant: <span style='color: green;'>```DontAutomaticallyOptimizeOption = 256```</span>)
        #[allow(non_upper_case_globals)]
        pub const DontAutomaticallyOptimizeOption: crate::q_regular_expression::PatternOption =
            crate::q_regular_expression::PatternOption(256);
    }

    /// <p>The MatchType enum defines the type of the match that should be attempted against the subject string.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QRegularExpression::MatchType```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#MatchType-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The MatchType enum defines the type of the match that should be attempted against the subject string.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct MatchType(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for MatchType {
        fn from(value: ::std::os::raw::c_int) -> Self {
            MatchType(value)
        }
    }

    impl From<MatchType> for ::std::os::raw::c_int {
        fn from(value: MatchType) -> Self {
            value.0
        }
    }

    impl MatchType {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl MatchType {
        /// A normal match is done. (C++ enum variant: <span style='color: green;'>```NormalMatch = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NormalMatch: crate::q_regular_expression::MatchType =
            crate::q_regular_expression::MatchType(0);
        /// The pattern string is matched partially against the subject string. If a partial match is found, then it is recorded, and other matching alternatives are tried as usual. If a complete match is then found, then it's preferred to the partial match; in this case only the complete match is reported. If instead no complete match is found (but only the partial one), then the partial one is reported. (C++ enum variant: <span style='color: green;'>```PartialPreferCompleteMatch = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const PartialPreferCompleteMatch: crate::q_regular_expression::MatchType =
            crate::q_regular_expression::MatchType(1);
        /// The pattern string is matched partially against the subject string. If a partial match is found, then matching stops and the partial match is reported. In this case, other matching alternatives (potentially leading to a complete match) are not tried. Moreover, this match type assumes that the subject string only a substring of a larger text, and that (in this text) there are other characters beyond the end of the subject string. This can lead to surprising results; see the discussion in the <a href="http://doc.qt.io/qt-5/qregularexpression.html#partial-matching">partial matching</a> section for more details. (C++ enum variant: <span style='color: green;'>```PartialPreferFirstMatch = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const PartialPreferFirstMatch: crate::q_regular_expression::MatchType =
            crate::q_regular_expression::MatchType(2);
        /// No matching is done. This value is returned as the match type by a default constructed <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> or <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a>. Using this match type is not very useful for the user, as no matching ever happens. This enum value has been introduced in Qt 5.1. (C++ enum variant: <span style='color: green;'>```NoMatch = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoMatch: crate::q_regular_expression::MatchType =
            crate::q_regular_expression::MatchType(3);
    }

    /// <p>The MatchOptions type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;MatchOption&gt;. It stores an OR combination of MatchOption values.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QRegularExpression::MatchOption```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#MatchOption-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The MatchOptions type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;MatchOption&gt;. It stores an OR combination of MatchOption values.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct MatchOption(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for MatchOption {
        fn from(value: ::std::os::raw::c_int) -> Self {
            MatchOption(value)
        }
    }

    impl From<MatchOption> for ::std::os::raw::c_int {
        fn from(value: MatchOption) -> Self {
            value.0
        }
    }

    impl MatchOption {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl MatchOption {
        /// No match options are set. (C++ enum variant: <span style='color: green;'>```NoMatchOption = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoMatchOption: crate::q_regular_expression::MatchOption =
            crate::q_regular_expression::MatchOption(0);
        /// The match is constrained to start exactly at the offset passed to <a href="http://doc.qt.io/qt-5/qregularexpression.html#match">match</a>() in order to be successful, even if the pattern string does not contain any metacharacter that anchors the match at that point. (C++ enum variant: <span style='color: green;'>```AnchoredMatchOption = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const AnchoredMatchOption: crate::q_regular_expression::MatchOption =
            crate::q_regular_expression::MatchOption(1);
        /// The subject string is not checked for UTF-16 validity before attempting a match. Use this option with extreme caution, as attempting to match an invalid string may crash the program and/or constitute a security issue. This enum value has been introduced in Qt 5.4. (C++ enum variant: <span style='color: green;'>```DontCheckSubjectStringMatchOption = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const DontCheckSubjectStringMatchOption: crate::q_regular_expression::MatchOption =
            crate::q_regular_expression::MatchOption(2);
    }

    impl From<crate::q_regular_expression::PatternOption>
        for crate::QFlags<crate::q_regular_expression::PatternOption>
    {
        fn from(value: crate::q_regular_expression::PatternOption) -> Self {
            Self::from(value.to_int())
        }
    }

    impl<T: Into<crate::QFlags<crate::q_regular_expression::PatternOption>>> std::ops::BitOr<T>
        for crate::q_regular_expression::PatternOption
    {
        type Output = crate::QFlags<crate::q_regular_expression::PatternOption>;
        fn bitor(self, rhs: T) -> crate::QFlags<crate::q_regular_expression::PatternOption> {
            Into::<crate::QFlags<crate::q_regular_expression::PatternOption>>::into(self) | rhs
        }
    }

    impl From<crate::q_regular_expression::MatchOption>
        for crate::QFlags<crate::q_regular_expression::MatchOption>
    {
        fn from(value: crate::q_regular_expression::MatchOption) -> Self {
            Self::from(value.to_int())
        }
    }

    impl<T: Into<crate::QFlags<crate::q_regular_expression::MatchOption>>> std::ops::BitOr<T>
        for crate::q_regular_expression::MatchOption
    {
        type Output = crate::QFlags<crate::q_regular_expression::MatchOption>;
        fn bitor(self, rhs: T) -> crate::QFlags<crate::q_regular_expression::MatchOption> {
            Into::<crate::QFlags<crate::q_regular_expression::MatchOption>>::into(self) | rhs
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> class provides pattern matching using regular expressions.</p>
///
/// C++ class: <span style='color: green;'>```QRegularExpression```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qregularexpression.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> class provides pattern matching using regular expressions.</p>
/// <p>Regular expressions, or <i>regexps</i>, are a very powerful tool to handle strings and texts. This is useful in many contexts, e.g.,</p>
/// <div class="table"><table class="generic">
///  <tbody><tr valign="top" class="odd"><td>Validation</td><td>A regexp can test whether a substring meets some criteria, e.g. is an integer or contains no whitespace.</td></tr>
/// <tr valign="top" class="even"><td>Searching</td><td>A regexp provides more powerful pattern matching than simple substring matching, e.g., match one of the words <i>mail</i>, <i>letter</i> or <i>correspondence</i>, but none of the words <i>email</i>, <i>mailman</i>, <i>mailer</i>, <i>letterbox</i>, etc.</td></tr>
/// <tr valign="top" class="odd"><td>Search and Replace</td><td>A regexp can replace all occurrences of a substring with a different substring, e.g., replace all occurrences of <i>&amp;</i> with <i>&amp;amp;</i> except where the <i>&amp;</i> is already followed by an <i>amp;</i>.</td></tr>
/// <tr valign="top" class="even"><td>String Splitting</td><td>A regexp can be used to identify where a string should be split apart, e.g. splitting tab-delimited strings.</td></tr>
/// </tbody></table></div>
/// <p>This document is by no means a complete reference to pattern matching using regular expressions, and the following parts will require the reader to have some basic knowledge about Perl-like regular expressions and their pattern syntax.</p>
/// <p>Good references about regular expressions include:</p>
/// <ul>
/// <li><i>Mastering Regular Expressions</i> (Third Edition) by Jeffrey E. F. Friedl, ISBN 0-596-52812-4;</li>
/// <li>the <a href="http://pcre.org/pcre.txt">pcrepattern</a>(3) man page, describing the pattern syntax supported by PCRE (the reference implementation of Perl-compatible regular expressions);</li>
/// <li>the <a href="http://perldoc.perl.org/perlre.html">Perl's regular expression documentation</a> and the <a href="http://perldoc.perl.org/perlretut.html">Perl's regular expression tutorial</a>.</li>
/// </ul>
/// <a name="introduction"></a></div>
#[repr(C)]
pub struct QRegularExpression {
    _unused: u8,
}
impl QRegularExpression {
    /// <p>Returns the <i>expression</i> wrapped between the <code>\A</code> and <code>\z</code> anchors to be used for exact matching.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QRegularExpression::anchoredPattern(const QString& expression)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#anchoredPattern">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the <i>expression</i> wrapped between the <code>\A</code> and <code>\z</code> anchors to be used for exact matching.</p>
    /// <p>This function was introduced in  Qt 5.12.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#porting-from-qregexp-s-exact-matching">Porting from QRegExp's Exact Matching</a>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn anchored_pattern(
        expression: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpression_anchoredPattern(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(expression)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the number of capturing groups inside the pattern string, or -1 if the regular expression is not valid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRegularExpression::captureCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#captureCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of capturing groups inside the pattern string, or -1 if the regular expression is not valid.</p>
    /// <p><b>Note: </b>The implicit capturing group 0 is <i>not</i> included in the returned number.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#isValid">isValid</a>().</p></div>
    pub unsafe fn capture_count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpression_captureCount(
            self as *const crate::QRegularExpression,
        )
    }

    /// <p>Assigns the regular expression <i>re</i> to this object, and returns a reference to the copy. Both the pattern and the pattern options are copied.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpression& QRegularExpression::operator=(const QRegularExpression& re)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns the regular expression <i>re</i> to this object, and returns a reference to the copy. Both the pattern and the pattern options are copied.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
    ) -> ::cpp_utils::MutRef<crate::QRegularExpression> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpression_operator_(
            self as *mut crate::QRegularExpression,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a textual description of the error found when checking the validity of the regular expression, or "no error" if no error was found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QRegularExpression::errorString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#errorString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a textual description of the error found when checking the validity of the regular expression, or "no error" if no error was found.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#isValid">isValid</a>() and <a href="http://doc.qt.io/qt-5/qregularexpression.html#patternErrorOffset">patternErrorOffset</a>().</p></div>
    pub unsafe fn error_string(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpression_errorString(
            self as *const crate::QRegularExpression,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Escapes all characters of <i>str</i> so that they no longer have any special meaning when used as a regular expression pattern string, and returns the escaped string. For instance:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QRegularExpression::escape(const QString& str)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#escape">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Escapes all characters of <i>str</i> so that they no longer have any special meaning when used as a regular expression pattern string, and returns the escaped string. For instance:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> escaped <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html#QRegularExpression">QRegularExpression</a></span><span class="operator">::</span>escape(<span class="string">"a(x) = f(x) + g(x)"</span>);
    ///   <span class="comment">// escaped == "a\\(x\\)\\ \\=\\ f\\(x\\)\\ \\+\\ g\\(x\\)"</span>
    ///
    /// </pre>
    /// <p>This is very convenient in order to build patterns from arbitrary strings:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> pattern <span class="operator">=</span> <span class="string">"("</span> <span class="operator">+</span> <span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html#QRegularExpression">QRegularExpression</a></span><span class="operator">::</span>escape(name) <span class="operator">+</span>
    /// &#32;   &#32;   &#32;   &#32;   &#32;   <span class="string">"|"</span> <span class="operator">+</span> <span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html#QRegularExpression">QRegularExpression</a></span><span class="operator">::</span>escape(nickname) <span class="operator">+</span> <span class="string">")"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html#QRegularExpression">QRegularExpression</a></span> re(pattern);
    ///
    /// </pre>
    /// <p><b>Note: </b>This function implements Perl's quotemeta algorithm and escapes with a backslash all characters in <i>str</i>, except for the characters in the <code>[A-Z]</code>, <code>[a-z]</code> and <code>[0-9]</code> ranges, as well as the underscore (<code>_</code>) character. The only difference with Perl is that a literal NUL inside <i>str</i> is escaped with the sequence <code>"\\0"</code> (backslash + <code>'0'</code>), instead of <code>"\\\0"</code> (backslash + <code>NUL</code>).</p></div>
    pub unsafe fn escape(
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpression_escape(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(str).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Attempts to perform a global match of the regular expression against the given <i>subject</i> string, starting at the position <i>offset</i> inside the subject, using a match of type <i>matchType</i> and honoring the given <i>matchOptions</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpressionMatchIterator QRegularExpression::globalMatch(const QString& subject, int offset = …, QRegularExpression::MatchType matchType = …, QFlags<QRegularExpression::MatchOption> matchOptions = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#globalMatch">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to perform a global match of the regular expression against the given <i>subject</i> string, starting at the position <i>offset</i> inside the subject, using a match of type <i>matchType</i> and honoring the given <i>matchOptions</i>.</p>
    /// <p>The returned <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> is positioned before the first match result (if any).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> and <a href="http://doc.qt.io/qt-5/qregularexpression.html#global-matching">global matching</a>.</p></div>
    pub unsafe fn global_match_q_string_int_match_type_q_flags_match_option(
        &self,
        subject: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        offset: ::std::os::raw::c_int,
        match_type: crate::q_regular_expression::MatchType,
        match_options: crate::QFlags<crate::q_regular_expression::MatchOption>,
    ) -> ::cpp_utils::CppBox<crate::QRegularExpressionMatchIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpression_globalMatch(
            self as *const crate::QRegularExpression,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(subject)
                .as_raw_ptr(),
            offset,
            match_type,
            match_options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpressionMatchIterator QRegularExpression::globalMatch(const QStringRef& subjectRef, int offset = …, QRegularExpression::MatchType matchType = …, QFlags<QRegularExpression::MatchOption> matchOptions = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#globalMatch-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Attempts to perform a global match of the regular expression against the given <i>subjectRef</i> string reference, starting at the position <i>offset</i> inside the subject, using a match of type <i>matchType</i> and honoring the given <i>matchOptions</i>.</p>
    /// <p>The returned <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> is positioned before the first match result (if any).</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> and <a href="http://doc.qt.io/qt-5/qregularexpression.html#global-matching">global matching</a>.</p></div>
    pub unsafe fn global_match_q_string_ref_int_match_type_q_flags_match_option(
        &self,
        subject_ref: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        offset: ::std::os::raw::c_int,
        match_type: crate::q_regular_expression::MatchType,
        match_options: crate::QFlags<crate::q_regular_expression::MatchOption>,
    ) -> ::cpp_utils::CppBox<crate::QRegularExpressionMatchIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpression_globalMatch1(
            self as *const crate::QRegularExpression,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(subject_ref)
                .as_raw_ptr(),
            offset,
            match_type,
            match_options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Attempts to perform a global match of the regular expression against the given <i>subject</i> string, starting at the position <i>offset</i> inside the subject, using a match of type <i>matchType</i> and honoring the given <i>matchOptions</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpressionMatchIterator QRegularExpression::globalMatch(const QString& subject, int offset = …, QRegularExpression::MatchType matchType = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#globalMatch">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to perform a global match of the regular expression against the given <i>subject</i> string, starting at the position <i>offset</i> inside the subject, using a match of type <i>matchType</i> and honoring the given <i>matchOptions</i>.</p>
    /// <p>The returned <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> is positioned before the first match result (if any).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> and <a href="http://doc.qt.io/qt-5/qregularexpression.html#global-matching">global matching</a>.</p></div>
    pub unsafe fn global_match_q_string_int_match_type(
        &self,
        subject: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        offset: ::std::os::raw::c_int,
        match_type: crate::q_regular_expression::MatchType,
    ) -> ::cpp_utils::CppBox<crate::QRegularExpressionMatchIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpression_globalMatch2(
            self as *const crate::QRegularExpression,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(subject)
                .as_raw_ptr(),
            offset,
            match_type,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Attempts to perform a global match of the regular expression against the given <i>subject</i> string, starting at the position <i>offset</i> inside the subject, using a match of type <i>matchType</i> and honoring the given <i>matchOptions</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpressionMatchIterator QRegularExpression::globalMatch(const QString& subject, int offset = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#globalMatch">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to perform a global match of the regular expression against the given <i>subject</i> string, starting at the position <i>offset</i> inside the subject, using a match of type <i>matchType</i> and honoring the given <i>matchOptions</i>.</p>
    /// <p>The returned <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> is positioned before the first match result (if any).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> and <a href="http://doc.qt.io/qt-5/qregularexpression.html#global-matching">global matching</a>.</p></div>
    pub unsafe fn global_match_q_string_int(
        &self,
        subject: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        offset: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QRegularExpressionMatchIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpression_globalMatch3(
            self as *const crate::QRegularExpression,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(subject)
                .as_raw_ptr(),
            offset,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Attempts to perform a global match of the regular expression against the given <i>subject</i> string, starting at the position <i>offset</i> inside the subject, using a match of type <i>matchType</i> and honoring the given <i>matchOptions</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpressionMatchIterator QRegularExpression::globalMatch(const QString& subject) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#globalMatch">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to perform a global match of the regular expression against the given <i>subject</i> string, starting at the position <i>offset</i> inside the subject, using a match of type <i>matchType</i> and honoring the given <i>matchOptions</i>.</p>
    /// <p>The returned <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> is positioned before the first match result (if any).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> and <a href="http://doc.qt.io/qt-5/qregularexpression.html#global-matching">global matching</a>.</p></div>
    pub unsafe fn global_match_q_string(
        &self,
        subject: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QRegularExpressionMatchIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpression_globalMatch4(
            self as *const crate::QRegularExpression,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(subject)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpressionMatchIterator QRegularExpression::globalMatch(const QStringRef& subjectRef, int offset = …, QRegularExpression::MatchType matchType = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#globalMatch-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Attempts to perform a global match of the regular expression against the given <i>subjectRef</i> string reference, starting at the position <i>offset</i> inside the subject, using a match of type <i>matchType</i> and honoring the given <i>matchOptions</i>.</p>
    /// <p>The returned <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> is positioned before the first match result (if any).</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> and <a href="http://doc.qt.io/qt-5/qregularexpression.html#global-matching">global matching</a>.</p></div>
    pub unsafe fn global_match_q_string_ref_int_match_type(
        &self,
        subject_ref: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        offset: ::std::os::raw::c_int,
        match_type: crate::q_regular_expression::MatchType,
    ) -> ::cpp_utils::CppBox<crate::QRegularExpressionMatchIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpression_globalMatch5(
            self as *const crate::QRegularExpression,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(subject_ref)
                .as_raw_ptr(),
            offset,
            match_type,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpressionMatchIterator QRegularExpression::globalMatch(const QStringRef& subjectRef, int offset = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#globalMatch-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Attempts to perform a global match of the regular expression against the given <i>subjectRef</i> string reference, starting at the position <i>offset</i> inside the subject, using a match of type <i>matchType</i> and honoring the given <i>matchOptions</i>.</p>
    /// <p>The returned <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> is positioned before the first match result (if any).</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> and <a href="http://doc.qt.io/qt-5/qregularexpression.html#global-matching">global matching</a>.</p></div>
    pub unsafe fn global_match_q_string_ref_int(
        &self,
        subject_ref: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        offset: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QRegularExpressionMatchIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpression_globalMatch6(
            self as *const crate::QRegularExpression,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(subject_ref)
                .as_raw_ptr(),
            offset,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpressionMatchIterator QRegularExpression::globalMatch(const QStringRef& subjectRef) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#globalMatch-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Attempts to perform a global match of the regular expression against the given <i>subjectRef</i> string reference, starting at the position <i>offset</i> inside the subject, using a match of type <i>matchType</i> and honoring the given <i>matchOptions</i>.</p>
    /// <p>The returned <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> is positioned before the first match result (if any).</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> and <a href="http://doc.qt.io/qt-5/qregularexpression.html#global-matching">global matching</a>.</p></div>
    pub unsafe fn global_match_q_string_ref(
        &self,
        subject_ref: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::cpp_utils::CppBox<crate::QRegularExpressionMatchIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpression_globalMatch7(
            self as *const crate::QRegularExpression,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(subject_ref)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if the regular expression is a valid regular expression (that is, it contains no syntax errors, etc.), or false otherwise. Use <a href="http://doc.qt.io/qt-5/qregularexpression.html#errorString">errorString</a>() to obtain a textual description of the error.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRegularExpression::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the regular expression is a valid regular expression (that is, it contains no syntax errors, etc.), or false otherwise. Use <a href="http://doc.qt.io/qt-5/qregularexpression.html#errorString">errorString</a>() to obtain a textual description of the error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#errorString">errorString</a>() and <a href="http://doc.qt.io/qt-5/qregularexpression.html#patternErrorOffset">patternErrorOffset</a>().</p></div>
    pub unsafe fn is_valid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpression_isValid(
            self as *const crate::QRegularExpression,
        )
    }

    /// <p>Attempts to match the regular expression against the given <i>subject</i> string, starting at the position <i>offset</i> inside the subject, using a match of type <i>matchType</i> and honoring the given <i>matchOptions</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpressionMatch QRegularExpression::match(const QString& subject, int offset = …, QRegularExpression::MatchType matchType = …, QFlags<QRegularExpression::MatchOption> matchOptions = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#match">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to match the regular expression against the given <i>subject</i> string, starting at the position <i>offset</i> inside the subject, using a match of type <i>matchType</i> and honoring the given <i>matchOptions</i>.</p>
    /// <p>The returned <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> object contains the results of the match.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> and <a href="http://doc.qt.io/qt-5/qregularexpression.html#normal-matching">normal matching</a>.</p></div>
    pub unsafe fn match_q_string_int_match_type_q_flags_match_option(
        &self,
        subject: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        offset: ::std::os::raw::c_int,
        match_type: crate::q_regular_expression::MatchType,
        match_options: crate::QFlags<crate::q_regular_expression::MatchOption>,
    ) -> ::cpp_utils::CppBox<crate::QRegularExpressionMatch> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpression_match(
            self as *const crate::QRegularExpression,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(subject)
                .as_raw_ptr(),
            offset,
            match_type,
            match_options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpressionMatch QRegularExpression::match(const QStringRef& subjectRef, int offset = …, QRegularExpression::MatchType matchType = …, QFlags<QRegularExpression::MatchOption> matchOptions = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#match-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Attempts to match the regular expression against the given <i>subjectRef</i> string reference, starting at the position <i>offset</i> inside the subject, using a match of type <i>matchType</i> and honoring the given <i>matchOptions</i>.</p>
    /// <p>The returned <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> object contains the results of the match.</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> and <a href="http://doc.qt.io/qt-5/qregularexpression.html#normal-matching">normal matching</a>.</p></div>
    pub unsafe fn match_q_string_ref_int_match_type_q_flags_match_option(
        &self,
        subject_ref: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        offset: ::std::os::raw::c_int,
        match_type: crate::q_regular_expression::MatchType,
        match_options: crate::QFlags<crate::q_regular_expression::MatchOption>,
    ) -> ::cpp_utils::CppBox<crate::QRegularExpressionMatch> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpression_match1(
            self as *const crate::QRegularExpression,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(subject_ref)
                .as_raw_ptr(),
            offset,
            match_type,
            match_options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Attempts to match the regular expression against the given <i>subject</i> string, starting at the position <i>offset</i> inside the subject, using a match of type <i>matchType</i> and honoring the given <i>matchOptions</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpressionMatch QRegularExpression::match(const QString& subject, int offset = …, QRegularExpression::MatchType matchType = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#match">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to match the regular expression against the given <i>subject</i> string, starting at the position <i>offset</i> inside the subject, using a match of type <i>matchType</i> and honoring the given <i>matchOptions</i>.</p>
    /// <p>The returned <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> object contains the results of the match.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> and <a href="http://doc.qt.io/qt-5/qregularexpression.html#normal-matching">normal matching</a>.</p></div>
    pub unsafe fn match_q_string_int_match_type(
        &self,
        subject: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        offset: ::std::os::raw::c_int,
        match_type: crate::q_regular_expression::MatchType,
    ) -> ::cpp_utils::CppBox<crate::QRegularExpressionMatch> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpression_match2(
            self as *const crate::QRegularExpression,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(subject)
                .as_raw_ptr(),
            offset,
            match_type,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Attempts to match the regular expression against the given <i>subject</i> string, starting at the position <i>offset</i> inside the subject, using a match of type <i>matchType</i> and honoring the given <i>matchOptions</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpressionMatch QRegularExpression::match(const QString& subject, int offset = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#match">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to match the regular expression against the given <i>subject</i> string, starting at the position <i>offset</i> inside the subject, using a match of type <i>matchType</i> and honoring the given <i>matchOptions</i>.</p>
    /// <p>The returned <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> object contains the results of the match.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> and <a href="http://doc.qt.io/qt-5/qregularexpression.html#normal-matching">normal matching</a>.</p></div>
    pub unsafe fn match_q_string_int(
        &self,
        subject: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        offset: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QRegularExpressionMatch> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpression_match3(
            self as *const crate::QRegularExpression,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(subject)
                .as_raw_ptr(),
            offset,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Attempts to match the regular expression against the given <i>subject</i> string, starting at the position <i>offset</i> inside the subject, using a match of type <i>matchType</i> and honoring the given <i>matchOptions</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpressionMatch QRegularExpression::match(const QString& subject) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#match">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to match the regular expression against the given <i>subject</i> string, starting at the position <i>offset</i> inside the subject, using a match of type <i>matchType</i> and honoring the given <i>matchOptions</i>.</p>
    /// <p>The returned <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> object contains the results of the match.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> and <a href="http://doc.qt.io/qt-5/qregularexpression.html#normal-matching">normal matching</a>.</p></div>
    pub unsafe fn match_q_string(
        &self,
        subject: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QRegularExpressionMatch> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpression_match4(
            self as *const crate::QRegularExpression,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(subject)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpressionMatch QRegularExpression::match(const QStringRef& subjectRef, int offset = …, QRegularExpression::MatchType matchType = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#match-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Attempts to match the regular expression against the given <i>subjectRef</i> string reference, starting at the position <i>offset</i> inside the subject, using a match of type <i>matchType</i> and honoring the given <i>matchOptions</i>.</p>
    /// <p>The returned <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> object contains the results of the match.</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> and <a href="http://doc.qt.io/qt-5/qregularexpression.html#normal-matching">normal matching</a>.</p></div>
    pub unsafe fn match_q_string_ref_int_match_type(
        &self,
        subject_ref: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        offset: ::std::os::raw::c_int,
        match_type: crate::q_regular_expression::MatchType,
    ) -> ::cpp_utils::CppBox<crate::QRegularExpressionMatch> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpression_match5(
            self as *const crate::QRegularExpression,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(subject_ref)
                .as_raw_ptr(),
            offset,
            match_type,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpressionMatch QRegularExpression::match(const QStringRef& subjectRef, int offset = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#match-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Attempts to match the regular expression against the given <i>subjectRef</i> string reference, starting at the position <i>offset</i> inside the subject, using a match of type <i>matchType</i> and honoring the given <i>matchOptions</i>.</p>
    /// <p>The returned <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> object contains the results of the match.</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> and <a href="http://doc.qt.io/qt-5/qregularexpression.html#normal-matching">normal matching</a>.</p></div>
    pub unsafe fn match_q_string_ref_int(
        &self,
        subject_ref: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        offset: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QRegularExpressionMatch> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpression_match6(
            self as *const crate::QRegularExpression,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(subject_ref)
                .as_raw_ptr(),
            offset,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpressionMatch QRegularExpression::match(const QStringRef& subjectRef) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#match-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Attempts to match the regular expression against the given <i>subjectRef</i> string reference, starting at the position <i>offset</i> inside the subject, using a match of type <i>matchType</i> and honoring the given <i>matchOptions</i>.</p>
    /// <p>The returned <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> object contains the results of the match.</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> and <a href="http://doc.qt.io/qt-5/qregularexpression.html#normal-matching">normal matching</a>.</p></div>
    pub unsafe fn match_q_string_ref(
        &self,
        subject_ref: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::cpp_utils::CppBox<crate::QRegularExpressionMatch> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpression_match7(
            self as *const crate::QRegularExpression,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(subject_ref)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list of <a href="http://doc.qt.io/qt-5/qregularexpression.html#captureCount">captureCount</a>() + 1 elements, containing the names of the named capturing groups in the pattern string. The list is sorted such that the element of the list at position <code>i</code> is the name of the <code>i</code>-th capturing group, if it has a name, or an empty string if that capturing group is unnamed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QRegularExpression::namedCaptureGroups() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#namedCaptureGroups">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of <a href="http://doc.qt.io/qt-5/qregularexpression.html#captureCount">captureCount</a>() + 1 elements, containing the names of the named capturing groups in the pattern string. The list is sorted such that the element of the list at position <code>i</code> is the name of the <code>i</code>-th capturing group, if it has a name, or an empty string if that capturing group is unnamed.</p>
    /// <p>For instance, given the regular expression</p>
    /// <pre class="cpp">
    ///
    ///   (<span class="operator">?</span><span class="operator">&lt;</span>day<span class="operator">&gt;</span>\d\d)<span class="operator">-</span>(<span class="operator">?</span><span class="operator">&lt;</span>month<span class="operator">&gt;</span>\d\d)<span class="operator">-</span>(<span class="operator">?</span><span class="operator">&lt;</span>year<span class="operator">&gt;</span>\d\d\d\d) (\w<span class="operator">+</span>) (<span class="operator">?</span><span class="operator">&lt;</span>name<span class="operator">&gt;</span>\w<span class="operator">+</span>)
    ///
    /// </pre>
    /// <p>namedCaptureGroups() will return the following list:</p>
    /// <pre class="cpp">
    ///
    ///   (<span class="string">""</span><span class="operator">,</span> <span class="string">"day"</span><span class="operator">,</span> <span class="string">"month"</span><span class="operator">,</span> <span class="string">"year"</span><span class="operator">,</span> <span class="string">""</span><span class="operator">,</span> <span class="string">"name"</span>)
    ///
    /// </pre>
    /// <p>which corresponds to the fact that the capturing group #0 (corresponding to the whole match) has no name, the capturing group #1 has name "day", the capturing group #2 has name "month", etc.</p>
    /// <p>If the regular expression is not valid, returns an empty list.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#isValid">isValid</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#captured">QRegularExpressionMatch::captured</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#isEmpty">QString::isEmpty</a>().</p></div>
    pub unsafe fn named_capture_groups(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpression_namedCaptureGroups(
            self as *const crate::QRegularExpression,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> object with an empty pattern and no pattern options.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRegularExpression::QRegularExpression()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#QRegularExpression">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> object with an empty pattern and no pattern options.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#setPattern">setPattern</a>() and <a href="http://doc.qt.io/qt-5/qregularexpression.html#setPatternOptions">setPatternOptions</a>().</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QRegularExpression> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpression_QRegularExpression();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> object using the given <i>pattern</i> as pattern and the <i>options</i> as the pattern options.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRegularExpression::QRegularExpression(const QString& pattern, QFlags<QRegularExpression::PatternOption> options = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#QRegularExpression-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> object using the given <i>pattern</i> as pattern and the <i>options</i> as the pattern options.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#setPattern">setPattern</a>() and <a href="http://doc.qt.io/qt-5/qregularexpression.html#setPatternOptions">setPatternOptions</a>().</p></div>
    pub unsafe fn new_2a(
        pattern: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        options: crate::QFlags<crate::q_regular_expression::PatternOption>,
    ) -> ::cpp_utils::CppBox<crate::QRegularExpression> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpression_QRegularExpression1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(pattern)
                .as_raw_ptr(),
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> object using the given <i>pattern</i> as pattern and the <i>options</i> as the pattern options.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRegularExpression::QRegularExpression(const QString& pattern)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#QRegularExpression-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> object using the given <i>pattern</i> as pattern and the <i>options</i> as the pattern options.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#setPattern">setPattern</a>() and <a href="http://doc.qt.io/qt-5/qregularexpression.html#setPatternOptions">setPatternOptions</a>().</p></div>
    pub unsafe fn new_1a(
        pattern: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QRegularExpression> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpression_QRegularExpression3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(pattern)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> object as a copy of <i>re</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRegularExpression::QRegularExpression(const QRegularExpression& re)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#QRegularExpression-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> object as a copy of <i>re</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        re: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
    ) -> ::cpp_utils::CppBox<crate::QRegularExpression> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpression_QRegularExpression2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(re)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Forces an immediate optimization of the pattern, including JIT-compiling it (if the JIT compiler is enabled).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRegularExpression::optimize() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#optimize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Forces an immediate optimization of the pattern, including JIT-compiling it (if the JIT compiler is enabled).</p>
    /// <p>Patterns are normally optimized only after a certain number of usages. If you can predict that this <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> object is going to be used for several matches, it may be convenient to optimize it in advance by calling this function.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#PatternOption-enum">QRegularExpression::OptimizeOnFirstUsageOption</a>.</p></div>
    pub unsafe fn optimize(&self) {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpression_optimize(
            self as *const crate::QRegularExpression,
        )
    }

    /// <p>Returns the pattern string of the regular expression.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QRegularExpression::pattern() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#pattern">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the pattern string of the regular expression.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#setPattern">setPattern</a>() and <a href="http://doc.qt.io/qt-5/qregularexpression.html#patternOptions">patternOptions</a>().</p></div>
    pub unsafe fn pattern(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpression_pattern(
            self as *const crate::QRegularExpression,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the offset, inside the pattern string, at which an error was found when checking the validity of the regular expression. If no error was found, then -1 is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRegularExpression::patternErrorOffset() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#patternErrorOffset">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the offset, inside the pattern string, at which an error was found when checking the validity of the regular expression. If no error was found, then -1 is returned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#pattern">pattern</a>(), <a href="http://doc.qt.io/qt-5/qregularexpression.html#isValid">isValid</a>(), and <a href="http://doc.qt.io/qt-5/qregularexpression.html#errorString">errorString</a>().</p></div>
    pub unsafe fn pattern_error_offset(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpression_patternErrorOffset(
            self as *const crate::QRegularExpression,
        )
    }

    /// <p>Returns the pattern options for the regular expression.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFlags<QRegularExpression::PatternOption> QRegularExpression::patternOptions() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#patternOptions">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the pattern options for the regular expression.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#setPatternOptions">setPatternOptions</a>() and <a href="http://doc.qt.io/qt-5/qregularexpression.html#pattern">pattern</a>().</p></div>
    pub unsafe fn pattern_options(
        &self,
    ) -> crate::QFlags<crate::q_regular_expression::PatternOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpression_patternOptions(
            self as *const crate::QRegularExpression,
        );
        crate::QFlags::from(ffi_result)
    }

    /// <p>Sets the pattern string of the regular expression to <i>pattern</i>. The pattern options are left unchanged.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRegularExpression::setPattern(const QString& pattern)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#setPattern">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the pattern string of the regular expression to <i>pattern</i>. The pattern options are left unchanged.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#pattern">pattern</a>() and <a href="http://doc.qt.io/qt-5/qregularexpression.html#setPatternOptions">setPatternOptions</a>().</p></div>
    pub unsafe fn set_pattern(
        &mut self,
        pattern: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpression_setPattern(
            self as *mut crate::QRegularExpression,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(pattern)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the given <i>options</i> as the pattern options of the regular expression. The pattern string is left unchanged.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRegularExpression::setPatternOptions(QFlags<QRegularExpression::PatternOption> options)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#setPatternOptions">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the given <i>options</i> as the pattern options of the regular expression. The pattern string is left unchanged.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#patternOptions">patternOptions</a>() and <a href="http://doc.qt.io/qt-5/qregularexpression.html#setPattern">setPattern</a>().</p></div>
    pub unsafe fn set_pattern_options(
        &mut self,
        options: crate::QFlags<crate::q_regular_expression::PatternOption>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpression_setPatternOptions(
            self as *mut crate::QRegularExpression,
            options.to_int(),
        )
    }

    /// <p>Swaps the regular expression <i>other</i> with this regular expression. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRegularExpression::swap(QRegularExpression& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps the regular expression <i>other</i> with this regular expression. This operation is very fast and never fails.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QRegularExpression>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpression_swap(
            self as *mut crate::QRegularExpression,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QRegularExpression>>::cast_into(
                other,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns a regular expression representation of the given glob <i>pattern</i>. The transformation is targeting file path globbing, which means in particular that path separators receive special treatment. This implies that it is not just a basic translation from "*" to ".*".</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QRegularExpression::wildcardToRegularExpression(const QString& str)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#wildcardToRegularExpression">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a regular expression representation of the given glob <i>pattern</i>. The transformation is targeting file path globbing, which means in particular that path separators receive special treatment. This implies that it is not just a basic translation from "*" to ".*".</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> wildcard <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html#QRegularExpression">QRegularExpression</a></span><span class="operator">::</span>wildcardToRegularExpression(<span class="string">"*.jpeg"</span>);
    ///   <span class="comment">// Will match files with names like:</span>
    ///   <span class="comment">//    foo.jpeg</span>
    ///   <span class="comment">//    f_o_o.jpeg</span>
    ///   <span class="comment">//    föö.jpeg</span>
    ///
    /// </pre>
    /// <p><b>Warning:</b> Unlike <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a>, this implementation follows closely the definition of wildcard for glob patterns:</p>
    /// <div class="table"><table class="generic">
    ///  <tbody><tr valign="top" class="odd"><td><b>c</b></td><td>Any character represents itself apart from those mentioned below. Thus <b>c</b> matches the character <i>c</i>.</td></tr>
    /// <tr valign="top" class="even"><td><b>?</b></td><td>Matches any single character. It is the same as <b>.</b> in full regexps.</td></tr>
    /// <tr valign="top" class="odd"><td><b>*</b></td><td>Matches zero or more of any characters. It is the same as <b>.*</b> in full regexps.</td></tr>
    /// <tr valign="top" class="even"><td><b>[abc]</b></td><td>Matches one character given in the bracket.</td></tr>
    /// <tr valign="top" class="odd"><td><b>[a-c]</b></td><td>Matches one character from the range given in the bracket.</td></tr>
    /// <tr valign="top" class="even"><td><b>[!abc]</b></td><td>Matches one character that is not given in the bracket. It is the same as <b>[^abc]</b> in full regexp.</td></tr>
    /// <tr valign="top" class="odd"><td><b>[!a-c]</b></td><td>Matches one character that is not from the range given in the bracket. It is the same as <b>[^a-c]</b> in full regexp.</td></tr>
    /// </tbody></table></div>
    /// <p><b>Note: </b>The backslash (\) character is <i>not</i> an escape char in this context. In order to match one of the special characters, place it in square brackets (for example, "[?]").</p><p>More information about the implementation can be found in:</p>
    /// <ul>
    /// <li><a href="https://en.wikipedia.org/wiki/Glob_(programming)">The Wikipedia Glob article</a></li>
    /// <li><code>man</code> 7 glob</li>
    /// </ul>
    /// <p>This function was introduced in  Qt 5.12.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#escape">escape</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn wildcard_to_regular_expression(
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QRegularExpression_wildcardToRegularExpression(
                ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(str)
                    .as_raw_ptr(),
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> class provides the results of a matching a <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> against a string.</p>
///
/// C++ class: <span style='color: green;'>```QRegularExpressionMatch```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> class provides the results of a matching a <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> against a string.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> object can be obtained by calling the <a href="http://doc.qt.io/qt-5/qregularexpression.html#match">QRegularExpression::match</a>() function, or as a single result of a global match from a <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a>.</p>
/// <p>The success or the failure of a match attempt can be inspected by calling the <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#hasMatch">hasMatch</a>() function. <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> also reports a successful partial match through the <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#hasPartialMatch">hasPartialMatch</a>() function.</p>
/// <p>In addition, <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> returns the substrings captured by the capturing groups in the pattern string. The implicit capturing group with index 0 captures the result of the whole match. The <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#captured">captured</a>() function returns each substring captured, either by the capturing group's index or by its name:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a></span> re(<span class="string">"(\\d\\d) (?&lt;name&gt;\\w+)"</span>);
///   <span class="type"><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#QRegularExpressionMatch">QRegularExpressionMatch</a></span> match <span class="operator">=</span> re<span class="operator">.</span>match(<span class="string">"23 Jordan"</span>);
///   <span class="keyword">if</span> (match<span class="operator">.</span>hasMatch()) {
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> number <span class="operator">=</span> match<span class="operator">.</span>captured(<span class="number">1</span>); <span class="comment">// first == "23"</span>
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> name <span class="operator">=</span> match<span class="operator">.</span>captured(<span class="string">"name"</span>); <span class="comment">// name == "Jordan"</span>
///   }
///
/// </pre>
/// <p>For each captured substring it is possible to query its starting and ending offsets in the subject string by calling the <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedStart">capturedStart</a>() and the <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedEnd">capturedEnd</a>() function, respectively. The length of each captured substring is available using the <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedLength">capturedLength</a>() function.</p>
/// <p>The convenience function <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedTexts">capturedTexts</a>() will return <i>all</i> the captured substrings at once (including the substring matched by the entire pattern) in the order they have been captured by captring groups; that is, <code>captured(i) == capturedTexts().at(i)</code>.</p>
/// <p>You can retrieve the <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> object the subject string was matched against by calling the <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#regularExpression">regularExpression</a>() function; the match type and the match options are available as well by calling the <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#matchType">matchType</a>() and the <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#matchOptions">matchOptions</a>() respectively.</p>
/// <p>Please refer to the <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> documentation for more information about the Qt regular expression classes.</p></div>
#[repr(C)]
pub struct QRegularExpressionMatch {
    _unused: u8,
}
impl QRegularExpressionMatch {
    /// <p>Returns the substring captured by the <i>nth</i> capturing group. If the <i>nth</i> capturing group did not capture a string or doesn't exist, returns a null <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QRegularExpressionMatch::captured(int nth = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#captured">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the substring captured by the <i>nth</i> capturing group. If the <i>nth</i> capturing group did not capture a string or doesn't exist, returns a null <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedRef">capturedRef</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#lastCapturedIndex">lastCapturedIndex</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedStart">capturedStart</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedEnd">capturedEnd</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedLength">capturedLength</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#isNull">QString::isNull</a>().</p></div>
    pub unsafe fn captured_int(
        &self,
        nth: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_captured(
            self as *const crate::QRegularExpressionMatch,
            nth,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the substring captured by the capturing group named <i>name</i>. If the capturing group named <i>name</i> did not capture a string or doesn't exist, returns a null <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QRegularExpressionMatch::captured(const QString& name) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#captured-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the substring captured by the capturing group named <i>name</i>. If the capturing group named <i>name</i> did not capture a string or doesn't exist, returns a null <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedRef">capturedRef</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedStart">capturedStart</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedEnd">capturedEnd</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedLength">capturedLength</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#isNull">QString::isNull</a>().</p></div>
    pub unsafe fn captured_q_string(
        &self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_captured1(
            self as *const crate::QRegularExpressionMatch,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the substring captured by the <i>nth</i> capturing group. If the <i>nth</i> capturing group did not capture a string or doesn't exist, returns a null <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QRegularExpressionMatch::captured() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#captured">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the substring captured by the <i>nth</i> capturing group. If the <i>nth</i> capturing group did not capture a string or doesn't exist, returns a null <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedRef">capturedRef</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#lastCapturedIndex">lastCapturedIndex</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedStart">capturedStart</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedEnd">capturedEnd</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedLength">capturedLength</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#isNull">QString::isNull</a>().</p></div>
    pub unsafe fn captured(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_captured2(
            self as *const crate::QRegularExpressionMatch,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the substring captured by the capturing group named <i>name</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QRegularExpressionMatch::captured(QStringView name) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#captured-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the substring captured by the capturing group named <i>name</i>.</p>
    /// <p>If the named capturing group <i>name</i> did not capture a string, or if there is no capturing group named <i>name</i>, returns a null <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedRef">capturedRef</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedView">capturedView</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedStart">capturedStart</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedEnd">capturedEnd</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedLength">capturedLength</a>(), and <a href="http://doc.qt.io/qt-5/qstring.html#isNull">QString::isNull</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn captured_q_string_view(
        &self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_captured6(
            self as *const crate::QRegularExpressionMatch,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the offset inside the subject string immediately after the ending position of the substring captured by the <i>nth</i> capturing group. If the <i>nth</i> capturing group did not capture a string or doesn't exist, returns -1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRegularExpressionMatch::capturedEnd(int nth = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the offset inside the subject string immediately after the ending position of the substring captured by the <i>nth</i> capturing group. If the <i>nth</i> capturing group did not capture a string or doesn't exist, returns -1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedStart">capturedStart</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedLength">capturedLength</a>(), and <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#captured">captured</a>().</p></div>
    pub unsafe fn captured_end_int(&self, nth: ::std::os::raw::c_int) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_capturedEnd(
            self as *const crate::QRegularExpressionMatch,
            nth,
        )
    }

    /// <p>Returns the offset inside the subject string immediately after the ending position of the substring captured by the capturing group named <i>name</i>. If the capturing group named <i>name</i> did not capture a string or doesn't exist, returns -1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRegularExpressionMatch::capturedEnd(const QString& name) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedEnd-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the offset inside the subject string immediately after the ending position of the substring captured by the capturing group named <i>name</i>. If the capturing group named <i>name</i> did not capture a string or doesn't exist, returns -1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedStart">capturedStart</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedLength">capturedLength</a>(), and <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#captured">captured</a>().</p></div>
    pub unsafe fn captured_end_q_string(
        &self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_capturedEnd1(
            self as *const crate::QRegularExpressionMatch,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
        )
    }

    /// <p>Returns the offset inside the subject string immediately after the ending position of the substring captured by the <i>nth</i> capturing group. If the <i>nth</i> capturing group did not capture a string or doesn't exist, returns -1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRegularExpressionMatch::capturedEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the offset inside the subject string immediately after the ending position of the substring captured by the <i>nth</i> capturing group. If the <i>nth</i> capturing group did not capture a string or doesn't exist, returns -1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedStart">capturedStart</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedLength">capturedLength</a>(), and <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#captured">captured</a>().</p></div>
    pub unsafe fn captured_end(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_capturedEnd2(
            self as *const crate::QRegularExpressionMatch,
        )
    }

    /// <p>Returns the offset inside the subject string immediately after the ending position of the substring captured by the capturing group named <i>name</i>. If the capturing group named <i>name</i> did not capture a string or doesn't exist, returns -1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRegularExpressionMatch::capturedEnd(QStringView name) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedEnd-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the offset inside the subject string immediately after the ending position of the substring captured by the capturing group named <i>name</i>. If the capturing group named <i>name</i> did not capture a string or doesn't exist, returns -1.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedStart">capturedStart</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedLength">capturedLength</a>(), and <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#captured">captured</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn captured_end_q_string_view(
        &self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_capturedEnd6(
            self as *const crate::QRegularExpressionMatch,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(name)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the length of the substring captured by the <i>nth</i> capturing group.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRegularExpressionMatch::capturedLength(int nth = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedLength">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the length of the substring captured by the <i>nth</i> capturing group.</p>
    /// <p><b>Note: </b>This function returns 0 if the <i>nth</i> capturing group did not capture a string or doesn't exist.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedStart">capturedStart</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedEnd">capturedEnd</a>(), and <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#captured">captured</a>().</p></div>
    pub unsafe fn captured_length_int(&self, nth: ::std::os::raw::c_int) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_capturedLength(
            self as *const crate::QRegularExpressionMatch,
            nth,
        )
    }

    /// <p>Returns the offset inside the subject string corresponding to the starting position of the substring captured by the capturing group named <i>name</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRegularExpressionMatch::capturedLength(const QString& name) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedLength-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the offset inside the subject string corresponding to the starting position of the substring captured by the capturing group named <i>name</i>.</p>
    /// <p><b>Note: </b>This function returns 0 if the capturing group named <i>name</i> did not capture a string or doesn't exist.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedStart">capturedStart</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedEnd">capturedEnd</a>(), and <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#captured">captured</a>().</p></div>
    pub unsafe fn captured_length_q_string(
        &self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_capturedLength1(
            self as *const crate::QRegularExpressionMatch,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
        )
    }

    /// <p>Returns the length of the substring captured by the <i>nth</i> capturing group.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRegularExpressionMatch::capturedLength() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedLength">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the length of the substring captured by the <i>nth</i> capturing group.</p>
    /// <p><b>Note: </b>This function returns 0 if the <i>nth</i> capturing group did not capture a string or doesn't exist.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedStart">capturedStart</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedEnd">capturedEnd</a>(), and <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#captured">captured</a>().</p></div>
    pub unsafe fn captured_length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_capturedLength2(
            self as *const crate::QRegularExpressionMatch,
        )
    }

    /// <p>Returns the length of the substring captured by the capturing group named <i>name</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRegularExpressionMatch::capturedLength(QStringView name) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedLength-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the length of the substring captured by the capturing group named <i>name</i>.</p>
    /// <p><b>Note: </b>This function returns 0 if the capturing group named <i>name</i> did not capture a string or doesn't exist.</p><p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedStart">capturedStart</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedEnd">capturedEnd</a>(), and <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#captured">captured</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn captured_length_q_string_view(
        &self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_capturedLength6(
            self as *const crate::QRegularExpressionMatch,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(name)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a reference to the substring captured by the <i>nth</i> capturing group. If the <i>nth</i> capturing group did not capture a string or doesn't exist, returns a null <a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QRegularExpressionMatch::capturedRef(int nth = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedRef">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the substring captured by the <i>nth</i> capturing group. If the <i>nth</i> capturing group did not capture a string or doesn't exist, returns a null <a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#captured">captured</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#lastCapturedIndex">lastCapturedIndex</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedStart">capturedStart</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedEnd">capturedEnd</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedLength">capturedLength</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#isNull">QStringRef::isNull</a>().</p></div>
    pub unsafe fn captured_ref_int(
        &self,
        nth: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_capturedRef(
            self as *const crate::QRegularExpressionMatch,
            nth,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the string captured by the capturing group named <i>name</i>. If the capturing group named <i>name</i> did not capture a string or doesn't exist, returns a null <a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QRegularExpressionMatch::capturedRef(const QString& name) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedRef-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the string captured by the capturing group named <i>name</i>. If the capturing group named <i>name</i> did not capture a string or doesn't exist, returns a null <a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#captured">captured</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedStart">capturedStart</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedEnd">capturedEnd</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedLength">capturedLength</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#isNull">QStringRef::isNull</a>().</p></div>
    pub unsafe fn captured_ref_q_string(
        &self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_capturedRef1(
            self as *const crate::QRegularExpressionMatch,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the substring captured by the <i>nth</i> capturing group. If the <i>nth</i> capturing group did not capture a string or doesn't exist, returns a null <a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QRegularExpressionMatch::capturedRef() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedRef">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the substring captured by the <i>nth</i> capturing group. If the <i>nth</i> capturing group did not capture a string or doesn't exist, returns a null <a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#captured">captured</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#lastCapturedIndex">lastCapturedIndex</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedStart">capturedStart</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedEnd">capturedEnd</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedLength">capturedLength</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#isNull">QStringRef::isNull</a>().</p></div>
    pub unsafe fn captured_ref(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_capturedRef2(
            self as *const crate::QRegularExpressionMatch,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the string captured by the capturing group named <i>name</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QRegularExpressionMatch::capturedRef(QStringView name) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedRef-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the string captured by the capturing group named <i>name</i>.</p>
    /// <p>If the named capturing group <i>name</i> did not capture a string, or if there is no capturing group named <i>name</i>, returns a null <a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a>.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#captured">captured</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedView">capturedView</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedStart">capturedStart</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedEnd">capturedEnd</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedLength">capturedLength</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#isNull">QStringRef::isNull</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn captured_ref_q_string_view(
        &self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_capturedRef6(
            self as *const crate::QRegularExpressionMatch,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the offset inside the subject string corresponding to the starting position of the substring captured by the <i>nth</i> capturing group. If the <i>nth</i> capturing group did not capture a string or doesn't exist, returns -1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRegularExpressionMatch::capturedStart(int nth = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedStart">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the offset inside the subject string corresponding to the starting position of the substring captured by the <i>nth</i> capturing group. If the <i>nth</i> capturing group did not capture a string or doesn't exist, returns -1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedEnd">capturedEnd</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedLength">capturedLength</a>(), and <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#captured">captured</a>().</p></div>
    pub unsafe fn captured_start_int(&self, nth: ::std::os::raw::c_int) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_capturedStart(
            self as *const crate::QRegularExpressionMatch,
            nth,
        )
    }

    /// <p>Returns the offset inside the subject string corresponding to the starting position of the substring captured by the capturing group named <i>name</i>. If the capturing group named <i>name</i> did not capture a string or doesn't exist, returns -1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRegularExpressionMatch::capturedStart(const QString& name) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedStart-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the offset inside the subject string corresponding to the starting position of the substring captured by the capturing group named <i>name</i>. If the capturing group named <i>name</i> did not capture a string or doesn't exist, returns -1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedEnd">capturedEnd</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedLength">capturedLength</a>(), and <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#captured">captured</a>().</p></div>
    pub unsafe fn captured_start_q_string(
        &self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_capturedStart1(
            self as *const crate::QRegularExpressionMatch,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
        )
    }

    /// <p>Returns the offset inside the subject string corresponding to the starting position of the substring captured by the <i>nth</i> capturing group. If the <i>nth</i> capturing group did not capture a string or doesn't exist, returns -1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRegularExpressionMatch::capturedStart() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedStart">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the offset inside the subject string corresponding to the starting position of the substring captured by the <i>nth</i> capturing group. If the <i>nth</i> capturing group did not capture a string or doesn't exist, returns -1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedEnd">capturedEnd</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedLength">capturedLength</a>(), and <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#captured">captured</a>().</p></div>
    pub unsafe fn captured_start(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_capturedStart2(
            self as *const crate::QRegularExpressionMatch,
        )
    }

    /// <p>Returns the offset inside the subject string corresponding to the starting position of the substring captured by the capturing group named <i>name</i>. If the capturing group named <i>name</i> did not capture a string or doesn't exist, returns -1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRegularExpressionMatch::capturedStart(QStringView name) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedStart-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the offset inside the subject string corresponding to the starting position of the substring captured by the capturing group named <i>name</i>. If the capturing group named <i>name</i> did not capture a string or doesn't exist, returns -1.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedEnd">capturedEnd</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedLength">capturedLength</a>(), and <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#captured">captured</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn captured_start_q_string_view(
        &self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_capturedStart6(
            self as *const crate::QRegularExpressionMatch,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(name)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a list of all strings captured by capturing groups, in the order the groups themselves appear in the pattern string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QRegularExpressionMatch::capturedTexts() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedTexts">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of all strings captured by capturing groups, in the order the groups themselves appear in the pattern string.</p></div>
    pub unsafe fn captured_texts(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_capturedTexts(
            self as *const crate::QRegularExpressionMatch,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a view of the substring captured by the <i>nth</i> capturing group.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringView QRegularExpressionMatch::capturedView(int nth = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedView">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a view of the substring captured by the <i>nth</i> capturing group.</p>
    /// <p>If the <i>nth</i> capturing group did not capture a string, or if there is no such capturing group, returns a null <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a>.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#captured">captured</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedRef">capturedRef</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#lastCapturedIndex">lastCapturedIndex</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedStart">capturedStart</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedEnd">capturedEnd</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedLength">capturedLength</a>(), and <a href="http://doc.qt.io/qt-5/qstringview.html#isNull">QStringView::isNull</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn captured_view_int(
        &self,
        nth: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QStringView> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_capturedView(
            self as *const crate::QRegularExpressionMatch,
            nth,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a view of the string captured by the capturing group named <i>name</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringView QRegularExpressionMatch::capturedView(QStringView name) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedView-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a view of the string captured by the capturing group named <i>name</i>.</p>
    /// <p>If the named capturing group <i>name</i> did not capture a string, or if there is no capturing group named <i>name</i>, returns a null <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a>.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#captured">captured</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedRef">capturedRef</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedStart">capturedStart</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedEnd">capturedEnd</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedLength">capturedLength</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#isNull">QStringRef::isNull</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn captured_view_q_string_view(
        &self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::cpp_utils::CppBox<crate::QStringView> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_capturedView1(
            self as *const crate::QRegularExpressionMatch,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a view of the substring captured by the <i>nth</i> capturing group.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringView QRegularExpressionMatch::capturedView() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedView">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a view of the substring captured by the <i>nth</i> capturing group.</p>
    /// <p>If the <i>nth</i> capturing group did not capture a string, or if there is no such capturing group, returns a null <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a>.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#captured">captured</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedRef">capturedRef</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#lastCapturedIndex">lastCapturedIndex</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedStart">capturedStart</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedEnd">capturedEnd</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedLength">capturedLength</a>(), and <a href="http://doc.qt.io/qt-5/qstringview.html#isNull">QStringView::isNull</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn captured_view(&self) -> ::cpp_utils::CppBox<crate::QStringView> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_capturedView2(
            self as *const crate::QRegularExpressionMatch,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Assigns the match result <i>match</i> to this object, and returns a reference to the copy.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpressionMatch& QRegularExpressionMatch::operator=(const QRegularExpressionMatch& match)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns the match result <i>match</i> to this object, and returns a reference to the copy.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        match_: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpressionMatch>>,
    ) -> ::cpp_utils::MutRef<crate::QRegularExpressionMatch> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_operator_(
            self as *mut crate::QRegularExpressionMatch,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpressionMatch>>::cast_into(
                match_,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the regular expression matched against the subject string, or false otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRegularExpressionMatch::hasMatch() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#hasMatch">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the regular expression matched against the subject string, or false otherwise.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#match">QRegularExpression::match</a>() and <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#hasPartialMatch">hasPartialMatch</a>().</p></div>
    pub unsafe fn has_match(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_hasMatch(
            self as *const crate::QRegularExpressionMatch,
        )
    }

    /// <p>Returns <code>true</code> if the regular expression partially matched against the subject string, or false otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRegularExpressionMatch::hasPartialMatch() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#hasPartialMatch">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the regular expression partially matched against the subject string, or false otherwise.</p>
    /// <p><b>Note: </b>Only a match that explicitly used the one of the partial match types can yield a partial match. Still, if such a match succeeds totally, this function will return false, while <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#hasMatch">hasMatch</a>() will return true.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#match">QRegularExpression::match</a>(), <a href="http://doc.qt.io/qt-5/qregularexpression.html#MatchType-enum">QRegularExpression::MatchType</a>, and <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#hasMatch">hasMatch</a>().</p></div>
    pub unsafe fn has_partial_match(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_hasPartialMatch(
            self as *const crate::QRegularExpressionMatch,
        )
    }

    /// <p>Returns <code>true</code> if the match object was obtained as a result from the <a href="http://doc.qt.io/qt-5/qregularexpression.html#match">QRegularExpression::match</a>() function invoked on a valid <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> object; returns <code>false</code> if the <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> was invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRegularExpressionMatch::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the match object was obtained as a result from the <a href="http://doc.qt.io/qt-5/qregularexpression.html#match">QRegularExpression::match</a>() function invoked on a valid <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> object; returns <code>false</code> if the <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> was invalid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#match">QRegularExpression::match</a>() and <a href="http://doc.qt.io/qt-5/qregularexpression.html#isValid">QRegularExpression::isValid</a>().</p></div>
    pub unsafe fn is_valid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_isValid(
            self as *const crate::QRegularExpressionMatch,
        )
    }

    /// <p>Returns the index of the last capturing group that captured something, including the implicit capturing group 0. This can be used to extract all the substrings that were captured:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRegularExpressionMatch::lastCapturedIndex() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#lastCapturedIndex">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index of the last capturing group that captured something, including the implicit capturing group 0. This can be used to extract all the substrings that were captured:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#QRegularExpressionMatch">QRegularExpressionMatch</a></span> match <span class="operator">=</span> re<span class="operator">.</span>match(string);
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span><span class="operator">=</span> match<span class="operator">.</span>lastCapturedIndex(); <span class="operator">+</span><span class="operator">+</span>i) {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> captured <span class="operator">=</span> match<span class="operator">.</span>captured(i);
    /// &#32;     <span class="comment">// ...</span>
    ///   }
    ///
    /// </pre>
    /// <p>Note that some of the capturing groups with an index less than lastCapturedIndex() could have not matched, and therefore captured nothing.</p>
    /// <p>If the regular expression did not match, this function returns -1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#captured">captured</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedStart">capturedStart</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedEnd">capturedEnd</a>(), and <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#capturedLength">capturedLength</a>().</p></div>
    pub unsafe fn last_captured_index(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_lastCapturedIndex(
            self as *const crate::QRegularExpressionMatch,
        )
    }

    /// <p>Returns the match options that were used to get this <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> object, that is, the match options that were passed to <a href="http://doc.qt.io/qt-5/qregularexpression.html#match">QRegularExpression::match</a>() or <a href="http://doc.qt.io/qt-5/qregularexpression.html#globalMatch">QRegularExpression::globalMatch</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFlags<QRegularExpression::MatchOption> QRegularExpressionMatch::matchOptions() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#matchOptions">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the match options that were used to get this <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> object, that is, the match options that were passed to <a href="http://doc.qt.io/qt-5/qregularexpression.html#match">QRegularExpression::match</a>() or <a href="http://doc.qt.io/qt-5/qregularexpression.html#globalMatch">QRegularExpression::globalMatch</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#match">QRegularExpression::match</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#regularExpression">regularExpression</a>(), and <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#matchType">matchType</a>().</p></div>
    pub unsafe fn match_options(&self) -> crate::QFlags<crate::q_regular_expression::MatchOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_matchOptions(
            self as *const crate::QRegularExpressionMatch,
        );
        crate::QFlags::from(ffi_result)
    }

    /// <p>Returns the match type that was used to get this <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> object, that is, the match type that was passed to <a href="http://doc.qt.io/qt-5/qregularexpression.html#match">QRegularExpression::match</a>() or <a href="http://doc.qt.io/qt-5/qregularexpression.html#globalMatch">QRegularExpression::globalMatch</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpression::MatchType QRegularExpressionMatch::matchType() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#matchType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the match type that was used to get this <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> object, that is, the match type that was passed to <a href="http://doc.qt.io/qt-5/qregularexpression.html#match">QRegularExpression::match</a>() or <a href="http://doc.qt.io/qt-5/qregularexpression.html#globalMatch">QRegularExpression::globalMatch</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#match">QRegularExpression::match</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#regularExpression">regularExpression</a>(), and <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#matchOptions">matchOptions</a>().</p></div>
    pub unsafe fn match_type(&self) -> crate::q_regular_expression::MatchType {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_matchType(
            self as *const crate::QRegularExpressionMatch,
        )
    }

    /// <p>Constructs a valid, empty <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> object. The regular expression is set to a default-constructed one; the match type to <a href="http://doc.qt.io/qt-5/qregularexpression.html#MatchType-enum">QRegularExpression::NoMatch</a> and the match options to <a href="http://doc.qt.io/qt-5/qregularexpression.html#MatchOption-enum">QRegularExpression::NoMatchOption</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRegularExpressionMatch::QRegularExpressionMatch()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#QRegularExpressionMatch">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a valid, empty <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> object. The regular expression is set to a default-constructed one; the match type to <a href="http://doc.qt.io/qt-5/qregularexpression.html#MatchType-enum">QRegularExpression::NoMatch</a> and the match options to <a href="http://doc.qt.io/qt-5/qregularexpression.html#MatchOption-enum">QRegularExpression::NoMatchOption</a>.</p>
    /// <p>The object will report no match through the <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#hasMatch">hasMatch</a>() and the <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#hasPartialMatch">hasPartialMatch</a>() member functions.</p>
    /// <p>This function was introduced in  Qt 5.1.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QRegularExpressionMatch> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_QRegularExpressionMatch();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a match result by copying the result of the given <i>match</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRegularExpressionMatch::QRegularExpressionMatch(const QRegularExpressionMatch& match)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#QRegularExpressionMatch-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a match result by copying the result of the given <i>match</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        match_: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpressionMatch>>,
    ) -> ::cpp_utils::CppBox<crate::QRegularExpressionMatch> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_QRegularExpressionMatch1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpressionMatch>>::cast_into(match_).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> object whose match() function returned this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpression QRegularExpressionMatch::regularExpression() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#regularExpression">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> object whose match() function returned this object.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#match">QRegularExpression::match</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#matchType">matchType</a>(), and <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#matchOptions">matchOptions</a>().</p></div>
    pub unsafe fn regular_expression(&self) -> ::cpp_utils::CppBox<crate::QRegularExpression> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_regularExpression(
            self as *const crate::QRegularExpressionMatch,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Swaps the match result <i>other</i> with this match result. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRegularExpressionMatch::swap(QRegularExpressionMatch& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps the match result <i>other</i> with this match result. This operation is very fast and never fails.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QRegularExpressionMatch>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_swap(self as *mut crate::QRegularExpressionMatch, ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QRegularExpressionMatch>>::cast_into(other).as_mut_raw_ptr())
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> class provides an iterator on the results of a global match of a <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> object against a string.</p>
///
/// C++ class: <span style='color: green;'>```QRegularExpressionMatchIterator```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> class provides an iterator on the results of a global match of a <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> object against a string.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> object is a forward only Java-like iterator; it can be obtained by calling the <a href="http://doc.qt.io/qt-5/qregularexpression.html#globalMatch">QRegularExpression::globalMatch</a>() function. A new <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> will be positioned before the first result. You can then call the <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#hasNext">hasNext</a>() function to check if there are more results available; if so, the <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#next">next</a>() function will return the next result and advance the iterator.</p>
/// <p>Each result is a <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a> object holding all the information for that result (including captured substrings).</p>
/// <p>For instance:</p>
/// <pre class="cpp">
///
///   <span class="comment">// extracts the words</span>
///   <span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a></span> re(<span class="string">"(\\w+)"</span>);
///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> subject(<span class="string">"the quick fox"</span>);
///   <span class="type"><a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#QRegularExpressionMatchIterator">QRegularExpressionMatchIterator</a></span> i <span class="operator">=</span> re<span class="operator">.</span>globalMatch(subject);
///   <span class="keyword">while</span> (i<span class="operator">.</span>hasNext()) {
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html">QRegularExpressionMatch</a></span> match <span class="operator">=</span> i<span class="operator">.</span>next();
/// &#32;     <span class="comment">// ...</span>
///   }
///
/// </pre>
/// <p>Moreover, <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> offers a <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#peekNext">peekNext</a>() function to get the next result <i>without</i> advancing the iterator.</p>
/// <p>You can retrieve the <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> object the subject string was matched against by calling the <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#regularExpression">regularExpression</a>() function; the match type and the match options are available as well by calling the <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#matchType">matchType</a>() and the <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#matchOptions">matchOptions</a>() respectively.</p>
/// <p>Please refer to the <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> documentation for more information about the Qt regular expression classes.</p></div>
#[repr(C)]
pub struct QRegularExpressionMatchIterator {
    _unused: u8,
}
impl QRegularExpressionMatchIterator {
    /// <p>Assigns the iterator <i>iterator</i> to this object, and returns a reference to the copy.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpressionMatchIterator& QRegularExpressionMatchIterator::operator=(const QRegularExpressionMatchIterator& iterator)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns the iterator <i>iterator</i> to this object, and returns a reference to the copy.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        iterator: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpressionMatchIterator>>,
    ) -> ::cpp_utils::MutRef<crate::QRegularExpressionMatchIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatchIterator_operator_(self as *mut crate::QRegularExpressionMatchIterator, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpressionMatchIterator>>::cast_into(iterator).as_raw_ptr());
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if there is at least one match result ahead of the iterator; otherwise it returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRegularExpressionMatchIterator::hasNext() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#hasNext">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if there is at least one match result ahead of the iterator; otherwise it returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#next">next</a>().</p></div>
    pub unsafe fn has_next(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatchIterator_hasNext(
            self as *const crate::QRegularExpressionMatchIterator,
        )
    }

    /// <p>Returns <code>true</code> if the iterator object was obtained as a result from the <a href="http://doc.qt.io/qt-5/qregularexpression.html#globalMatch">QRegularExpression::globalMatch</a>() function invoked on a valid <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> object; returns <code>false</code> if the <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> was invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRegularExpressionMatchIterator::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the iterator object was obtained as a result from the <a href="http://doc.qt.io/qt-5/qregularexpression.html#globalMatch">QRegularExpression::globalMatch</a>() function invoked on a valid <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> object; returns <code>false</code> if the <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> was invalid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#globalMatch">QRegularExpression::globalMatch</a>() and <a href="http://doc.qt.io/qt-5/qregularexpression.html#isValid">QRegularExpression::isValid</a>().</p></div>
    pub unsafe fn is_valid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatchIterator_isValid(
            self as *const crate::QRegularExpressionMatchIterator,
        )
    }

    /// <p>Returns the match options that were used to get this <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> object, that is, the match options that were passed to <a href="http://doc.qt.io/qt-5/qregularexpression.html#globalMatch">QRegularExpression::globalMatch</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFlags<QRegularExpression::MatchOption> QRegularExpressionMatchIterator::matchOptions() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#matchOptions">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the match options that were used to get this <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> object, that is, the match options that were passed to <a href="http://doc.qt.io/qt-5/qregularexpression.html#globalMatch">QRegularExpression::globalMatch</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#globalMatch">QRegularExpression::globalMatch</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#regularExpression">regularExpression</a>(), and <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#matchType">matchType</a>().</p></div>
    pub unsafe fn match_options(&self) -> crate::QFlags<crate::q_regular_expression::MatchOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatchIterator_matchOptions(
            self as *const crate::QRegularExpressionMatchIterator,
        );
        crate::QFlags::from(ffi_result)
    }

    /// <p>Returns the match type that was used to get this <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> object, that is, the match type that was passed to <a href="http://doc.qt.io/qt-5/qregularexpression.html#globalMatch">QRegularExpression::globalMatch</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpression::MatchType QRegularExpressionMatchIterator::matchType() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#matchType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the match type that was used to get this <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> object, that is, the match type that was passed to <a href="http://doc.qt.io/qt-5/qregularexpression.html#globalMatch">QRegularExpression::globalMatch</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#globalMatch">QRegularExpression::globalMatch</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#regularExpression">regularExpression</a>(), and <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#matchOptions">matchOptions</a>().</p></div>
    pub unsafe fn match_type(&self) -> crate::q_regular_expression::MatchType {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatchIterator_matchType(
            self as *const crate::QRegularExpressionMatchIterator,
        )
    }

    /// <p>Constructs an empty, valid <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> object. The regular expression is set to a default-constructed one; the match type to <a href="http://doc.qt.io/qt-5/qregularexpression.html#MatchType-enum">QRegularExpression::NoMatch</a> and the match options to <a href="http://doc.qt.io/qt-5/qregularexpression.html#MatchOption-enum">QRegularExpression::NoMatchOption</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRegularExpressionMatchIterator::QRegularExpressionMatchIterator()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#QRegularExpressionMatchIterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty, valid <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> object. The regular expression is set to a default-constructed one; the match type to <a href="http://doc.qt.io/qt-5/qregularexpression.html#MatchType-enum">QRegularExpression::NoMatch</a> and the match options to <a href="http://doc.qt.io/qt-5/qregularexpression.html#MatchOption-enum">QRegularExpression::NoMatchOption</a>.</p>
    /// <p>Invoking the <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#hasNext">hasNext</a>() member function on the constructed object will return false, as the iterator is not iterating on a valid sequence of matches.</p>
    /// <p>This function was introduced in  Qt 5.1.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QRegularExpressionMatchIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatchIterator_QRegularExpressionMatchIterator();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> object as a copy of <i>iterator</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRegularExpressionMatchIterator::QRegularExpressionMatchIterator(const QRegularExpressionMatchIterator& iterator)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#QRegularExpressionMatchIterator-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> object as a copy of <i>iterator</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        iterator: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpressionMatchIterator>>,
    ) -> ::cpp_utils::CppBox<crate::QRegularExpressionMatchIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatchIterator_QRegularExpressionMatchIterator1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpressionMatchIterator>>::cast_into(iterator).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the next match result and advances the iterator by one position.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpressionMatch QRegularExpressionMatchIterator::next()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#next">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the next match result and advances the iterator by one position.</p>
    /// <p><b>Note: </b>Calling this function when the iterator is at the end of the result set leads to undefined results.</p></div>
    pub unsafe fn next(&mut self) -> ::cpp_utils::CppBox<crate::QRegularExpressionMatch> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatchIterator_next(
            self as *mut crate::QRegularExpressionMatchIterator,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the next match result without moving the iterator.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpressionMatch QRegularExpressionMatchIterator::peekNext() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#peekNext">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the next match result without moving the iterator.</p>
    /// <p><b>Note: </b>Calling this function when the iterator is at the end of the result set leads to undefined results.</p></div>
    pub unsafe fn peek_next(&self) -> ::cpp_utils::CppBox<crate::QRegularExpressionMatch> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatchIterator_peekNext(
            self as *const crate::QRegularExpressionMatchIterator,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> object whose globalMatch() function returned this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpression QRegularExpressionMatchIterator::regularExpression() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#regularExpression">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> object whose globalMatch() function returned this object.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#globalMatch">QRegularExpression::globalMatch</a>(), <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#matchType">matchType</a>(), and <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#matchOptions">matchOptions</a>().</p></div>
    pub unsafe fn regular_expression(&self) -> ::cpp_utils::CppBox<crate::QRegularExpression> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatchIterator_regularExpression(
                self as *const crate::QRegularExpressionMatchIterator,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Swaps the iterator <i>other</i> with this iterator object. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRegularExpressionMatchIterator::swap(QRegularExpressionMatchIterator& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps the iterator <i>other</i> with this iterator object. This operation is very fast and never fails.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QRegularExpressionMatchIterator>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatchIterator_swap(self as *mut crate::QRegularExpressionMatchIterator, ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QRegularExpressionMatchIterator>>::cast_into(other).as_mut_raw_ptr())
    }
}

pub mod q_resource {
    //! C++ type: <span style='color: green;'>```QResource```</span>

    /// <p>This enum is used by <a href="http://doc.qt.io/qt-5/qresource.html#compressionAlgorithm">compressionAlgorithm</a>() to indicate which algorithm the RCC tool used to compress the payload.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QResource::Compression```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qresource.html#Compression-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum is used by <a href="http://doc.qt.io/qt-5/qresource.html#compressionAlgorithm">compressionAlgorithm</a>() to indicate which algorithm the RCC tool used to compress the payload.</p>
    ///
    /// <p>This enum was introduced or modified in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qresource.html#compressionAlgorithm">compressionAlgorithm</a>() and <a href="http://doc.qt.io/qt-5/qresource.html#isCompressed">isCompressed</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Compression(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Compression {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Compression(value)
        }
    }

    impl From<Compression> for ::std::os::raw::c_int {
        fn from(value: Compression) -> Self {
            value.0
        }
    }

    impl Compression {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Compression {
        /// Contents are not compressed (<a href="http://doc.qt.io/qt-5/qresource.html#isCompressed">isCompressed</a>() is false). (C++ enum variant: <span style='color: green;'>```NoCompression = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoCompression: crate::q_resource::Compression = crate::q_resource::Compression(0);
        /// Contents are compressed using <a href="http://doc.qt.io/qt-5/qtcore-attribution-zlib.html#zlib">https://zlib.net</a> and can be decompressed using the <a href="http://doc.qt.io/qt-5/qbytearray.html#qUncompress">qUncompress</a>() function. (C++ enum variant: <span style='color: green;'>```ZlibCompression = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const ZlibCompression: crate::q_resource::Compression =
            crate::q_resource::Compression(1);
        /// Contents are compressed using https://zstd.net. To decompress, use the <code>ZSTD_decompress</code> function from the zstd library. (C++ enum variant: <span style='color: green;'>```ZstdCompression = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const ZstdCompression: crate::q_resource::Compression =
            crate::q_resource::Compression(2);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> class provides an interface for reading directly from resources.</p>
///
/// C++ class: <span style='color: green;'>```QResource```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qresource.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> class provides an interface for reading directly from resources.</p>
/// <p><a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> is an object that represents a set of data (and possibly children) relating to a single resource entity. <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> gives direct access to the bytes in their raw format. In this way direct access allows reading data without buffer copying or indirection. Indirection is often useful when interacting with the resource entity as if it is a file, this can be achieved with <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a>. The data and children behind a <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> are normally compiled into an application/library, but it is also possible to load a resource at runtime. When loaded at run time the resource file will be loaded as one big set of data and then given out in pieces via references into the resource tree.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> can either be loaded with an absolute path, either treated as a file system rooted with a <code>/</code> character, or in resource notation rooted with a <code>:</code> character. A relative resource can also be opened which will be found in the list of paths returned by <a href="http://doc.qt.io/qt-5/qdir.html#searchPaths">QDir::searchPaths</a>().</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> that is representing a file will have data backing it, this data can possibly be compressed, in which case <a href="http://doc.qt.io/qt-5/qbytearray.html#qUncompress">qUncompress</a>() must be used to access the real data; this happens implicitly when accessed through a <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a>. A <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> that is representing a directory will have only children and no data.</p>
/// <a name="dynamic-resource-loading"></a></div>
#[repr(C)]
pub struct QResource {
    _unused: u8,
}
impl QResource {
    /// <p>Returns the real path that this <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> represents, if the resource was found via the <a href="http://doc.qt.io/qt-5/qdir.html#searchPaths">QDir::searchPaths</a>() it will be indicated in the path.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QResource::absoluteFilePath() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qresource.html#absoluteFilePath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the real path that this <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> represents, if the resource was found via the <a href="http://doc.qt.io/qt-5/qdir.html#searchPaths">QDir::searchPaths</a>() it will be indicated in the path.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qresource.html#fileName">fileName</a>().</p></div>
    pub unsafe fn absolute_file_path(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QResource_absoluteFilePath(
            self as *const crate::QResource,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Use <a href="http://doc.qt.io/qt-5/qdir.html#addSearchPath">QDir::addSearchPath</a>() with a prefix instead.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QResource::addSearchPath(const QString& path)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qresource-obsolete.html#addSearchPath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Use <a href="http://doc.qt.io/qt-5/qdir.html#addSearchPath">QDir::addSearchPath</a>() with a prefix instead.</p>
    /// <p>Adds <i>path</i> to the search paths searched in to find resources that are not specified with an absolute path. The <i>path</i> must be an absolute path (start with <code>/</code>).</p>
    /// <p>The default search path is to search only in the root (<code>:/</code>). The last path added will be consulted first upon next <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> creation.</p></div>
    pub unsafe fn add_search_path(
        path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QResource_addSearchPath(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(path).as_raw_ptr(),
        )
    }

    /// <p>Returns the compression type that this resource is compressed with, if any. If it is not compressed, this function returns <a href="http://doc.qt.io/qt-5/qresource.html#Compression-enum">QResource::NoCompression</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QResource::Compression QResource::compressionAlgorithm() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qresource.html#compressionAlgorithm">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the compression type that this resource is compressed with, if any. If it is not compressed, this function returns <a href="http://doc.qt.io/qt-5/qresource.html#Compression-enum">QResource::NoCompression</a>.</p>
    /// <p>If this function returns <a href="http://doc.qt.io/qt-5/qresource.html#Compression-enum">QResource::ZlibCompression</a>, you may decompress the data using the <a href="http://doc.qt.io/qt-5/qbytearray.html#qUncompress">qUncompress</a>() function. Up until Qt 5.13, this was the only possible compression algorithm.</p>
    /// <p>If this function returns <a href="http://doc.qt.io/qt-5/qresource.html#Compression-enum">QResource::ZstdCompression</a>, you need to use the Zstandard library functios (<code>&lt;zstd.h&gt; header). Qt does not provide a wrapper. See \l{http://facebook.github.io/zstd/zstd_manual.html}{Zstandard manual}. \sa isCompressed(), data(), isFile()</code></p>
    /// <p>This function was introduced in  Qt 5.13.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn compression_algorithm(&self) -> crate::q_resource::Compression {
        crate::__ffi::ctr_qt_core_ffi_QResource_compressionAlgorithm(
            self as *const crate::QResource,
        )
    }

    /// <p>Returns direct access to a read only segment of data that this resource represents. If the resource is compressed the data returns is compressed and <a href="http://doc.qt.io/qt-5/qbytearray.html#qUncompress">qUncompress</a>() must be used to access the data. If the resource is a directory 0 is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const unsigned char* QResource::data() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qresource.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns direct access to a read only segment of data that this resource represents. If the resource is compressed the data returns is compressed and <a href="http://doc.qt.io/qt-5/qbytearray.html#qUncompress">qUncompress</a>() must be used to access the data. If the resource is a directory 0 is returned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qresource.html#size">size</a>(), <a href="http://doc.qt.io/qt-5/qresource.html#isCompressed">isCompressed</a>(), and <a href="http://doc.qt.io/qt-5/qresource.html#isFile">isFile</a>().</p></div>
    pub unsafe fn data(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_uchar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QResource_data(self as *const crate::QResource);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns the full path to the file that this <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> represents as it was passed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QResource::fileName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qresource.html#fileName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the full path to the file that this <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> represents as it was passed.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qresource.html#setFileName">setFileName</a>() and <a href="http://doc.qt.io/qt-5/qresource.html#absoluteFilePath">absoluteFilePath</a>().</p></div>
    pub unsafe fn file_name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QResource_fileName(self as *const crate::QResource);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if the resource represents a file and the data backing it is in a compressed format, false otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QResource::isCompressed() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qresource.html#isCompressed">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the resource represents a file and the data backing it is in a compressed format, false otherwise.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qresource.html#data">data</a>() and <a href="http://doc.qt.io/qt-5/qresource.html#isFile">isFile</a>().</p></div>
    pub unsafe fn is_compressed(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QResource_isCompressed(self as *const crate::QResource)
    }

    /// <p>Returns <code>true</code> if the resource really exists in the resource hierarchy, false otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QResource::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qresource.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the resource really exists in the resource hierarchy, false otherwise.</p></div>
    pub unsafe fn is_valid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QResource_isValid(self as *const crate::QResource)
    }

    /// <p>Returns the date and time when the file was last modified before packaging into a resource.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDateTime QResource::lastModified() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qresource.html#lastModified">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the date and time when the file was last modified before packaging into a resource.</p></div>
    pub unsafe fn last_modified(&self) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QResource_lastModified(self as *const crate::QResource);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the locale used to locate the data for the <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale QResource::locale() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qresource.html#locale">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the locale used to locate the data for the <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qresource.html#setLocale">setLocale</a>().</p></div>
    pub unsafe fn locale(&self) -> ::cpp_utils::CppBox<crate::QLocale> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QResource_locale(self as *const crate::QResource);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> pointing to <i>file</i>. <i>locale</i> is used to load a specific localization of a resource data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QResource::QResource(const QString& file = …, const QLocale& locale = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qresource.html#QResource">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> pointing to <i>file</i>. <i>locale</i> is used to load a specific localization of a resource data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a>, <a href="http://doc.qt.io/qt-5/qdir.html#searchPaths">QDir::searchPaths</a>(), <a href="http://doc.qt.io/qt-5/qresource.html#setFileName">setFileName</a>(), and <a href="http://doc.qt.io/qt-5/qresource.html#setLocale">setLocale</a>().</p></div>
    pub unsafe fn new_2a(
        file: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        locale: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    ) -> ::cpp_utils::CppBox<crate::QResource> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QResource_QResource(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(locale)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> class provides an interface for reading directly from resources.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QResource::QResource()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qresource.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> class provides an interface for reading directly from resources.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> is an object that represents a set of data (and possibly children) relating to a single resource entity. <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> gives direct access to the bytes in their raw format. In this way direct access allows reading data without buffer copying or indirection. Indirection is often useful when interacting with the resource entity as if it is a file, this can be achieved with <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a>. The data and children behind a <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> are normally compiled into an application/library, but it is also possible to load a resource at runtime. When loaded at run time the resource file will be loaded as one big set of data and then given out in pieces via references into the resource tree.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> can either be loaded with an absolute path, either treated as a file system rooted with a <code>/</code> character, or in resource notation rooted with a <code>:</code> character. A relative resource can also be opened which will be found in the list of paths returned by <a href="http://doc.qt.io/qt-5/qdir.html#searchPaths">QDir::searchPaths</a>().</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> that is representing a file will have data backing it, this data can possibly be compressed, in which case <a href="http://doc.qt.io/qt-5/qbytearray.html#qUncompress">qUncompress</a>() must be used to access the real data; this happens implicitly when accessed through a <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a>. A <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> that is representing a directory will have only children and no data.</p>
    /// <a name="dynamic-resource-loading"></a></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QResource> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QResource_QResource1();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> pointing to <i>file</i>. <i>locale</i> is used to load a specific localization of a resource data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QResource::QResource(const QString& file = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qresource.html#QResource">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> pointing to <i>file</i>. <i>locale</i> is used to load a specific localization of a resource data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a>, <a href="http://doc.qt.io/qt-5/qdir.html#searchPaths">QDir::searchPaths</a>(), <a href="http://doc.qt.io/qt-5/qresource.html#setFileName">setFileName</a>(), and <a href="http://doc.qt.io/qt-5/qresource.html#setLocale">setLocale</a>().</p></div>
    pub unsafe fn new_1a(
        file: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QResource> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QResource_QResource3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Registers the resource with the given <i>rccFileName</i> at the location in the resource tree specified by <i>mapRoot</i>, and returns <code>true</code> if the file is successfully opened; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QResource::registerResource(const QString& rccFilename, const QString& resourceRoot = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qresource.html#registerResource">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Registers the resource with the given <i>rccFileName</i> at the location in the resource tree specified by <i>mapRoot</i>, and returns <code>true</code> if the file is successfully opened; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qresource.html#unregisterResource">unregisterResource</a>().</p></div>
    pub unsafe fn register_resource_2_q_string(
        rcc_filename: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        resource_root: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QResource_registerResource(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(rcc_filename)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(resource_root)
                .as_raw_ptr(),
        )
    }

    /// <p>Registers the resource with the given <i>rccData</i> at the location in the resource tree specified by <i>mapRoot</i>, and returns <code>true</code> if the file is successfully opened; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QResource::registerResource(const unsigned char* rccData, const QString& resourceRoot = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qresource.html#registerResource-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Registers the resource with the given <i>rccData</i> at the location in the resource tree specified by <i>mapRoot</i>, and returns <code>true</code> if the file is successfully opened; otherwise returns <code>false</code>.</p>
    /// <p><b>Warning:</b> The data must remain valid throughout the life of any <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> that may reference the resource data.</p>
    /// <p>This function was introduced in  Qt 4.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qresource.html#unregisterResource">unregisterResource</a>().</p></div>
    pub unsafe fn register_resource_uchar_q_string(
        rcc_data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_uchar>>,
        resource_root: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QResource_registerResource1(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_uchar>>::cast_into(rcc_data)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(resource_root)
                .as_raw_ptr(),
        )
    }

    /// <p>Registers the resource with the given <i>rccFileName</i> at the location in the resource tree specified by <i>mapRoot</i>, and returns <code>true</code> if the file is successfully opened; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QResource::registerResource(const QString& rccFilename)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qresource.html#registerResource">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Registers the resource with the given <i>rccFileName</i> at the location in the resource tree specified by <i>mapRoot</i>, and returns <code>true</code> if the file is successfully opened; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qresource.html#unregisterResource">unregisterResource</a>().</p></div>
    pub unsafe fn register_resource_q_string(
        rcc_filename: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QResource_registerResource2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(rcc_filename)
                .as_raw_ptr(),
        )
    }

    /// <p>Registers the resource with the given <i>rccData</i> at the location in the resource tree specified by <i>mapRoot</i>, and returns <code>true</code> if the file is successfully opened; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QResource::registerResource(const unsigned char* rccData)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qresource.html#registerResource-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Registers the resource with the given <i>rccData</i> at the location in the resource tree specified by <i>mapRoot</i>, and returns <code>true</code> if the file is successfully opened; otherwise returns <code>false</code>.</p>
    /// <p><b>Warning:</b> The data must remain valid throughout the life of any <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> that may reference the resource data.</p>
    /// <p>This function was introduced in  Qt 4.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qresource.html#unregisterResource">unregisterResource</a>().</p></div>
    pub unsafe fn register_resource_uchar(
        rcc_data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_uchar>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QResource_registerResource3(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_uchar>>::cast_into(rcc_data)
                .as_raw_ptr(),
        )
    }

    /// <p>Use <a href="http://doc.qt.io/qt-5/qdir.html#searchPaths">QDir::searchPaths</a>() instead.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QStringList QResource::searchPaths()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qresource-obsolete.html#searchPaths">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Use <a href="http://doc.qt.io/qt-5/qdir.html#searchPaths">QDir::searchPaths</a>() instead.</p>
    /// <p>Returns the current search path list. This list is consulted when creating a relative resource.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#addSearchPath">QDir::addSearchPath</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#setSearchPaths">QDir::setSearchPaths</a>().</p></div>
    pub unsafe fn search_paths() -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QResource_searchPaths();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Sets a <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> to point to <i>file</i>. <i>file</i> can either be absolute, in which case it is opened directly, if relative then the file will be tried to be found in <a href="http://doc.qt.io/qt-5/qdir.html#searchPaths">QDir::searchPaths</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QResource::setFileName(const QString& file)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qresource.html#setFileName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets a <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> to point to <i>file</i>. <i>file</i> can either be absolute, in which case it is opened directly, if relative then the file will be tried to be found in <a href="http://doc.qt.io/qt-5/qdir.html#searchPaths">QDir::searchPaths</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qresource.html#fileName">fileName</a>() and <a href="http://doc.qt.io/qt-5/qresource.html#absoluteFilePath">absoluteFilePath</a>().</p></div>
    pub unsafe fn set_file_name(
        &mut self,
        file: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QResource_setFileName(
            self as *mut crate::QResource,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file).as_raw_ptr(),
        )
    }

    /// <p>Sets a <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> to only load the localization of resource to for <i>locale</i>. If a resource for the specific locale is not found then the C locale is used.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QResource::setLocale(const QLocale& locale)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qresource.html#setLocale">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets a <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> to only load the localization of resource to for <i>locale</i>. If a resource for the specific locale is not found then the C locale is used.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qresource.html#locale">locale</a>() and <a href="http://doc.qt.io/qt-5/qresource.html#setFileName">setFileName</a>().</p></div>
    pub unsafe fn set_locale(
        &mut self,
        locale: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QResource_setLocale(
            self as *mut crate::QResource,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(locale)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the size of the data backing the resource.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QResource::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qresource.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the size of the data backing the resource.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qresource.html#data">data</a>() and <a href="http://doc.qt.io/qt-5/qresource.html#isFile">isFile</a>().</p></div>
    pub unsafe fn size(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QResource_size(self as *const crate::QResource)
    }

    /// <p>Unregisters the resource with the given <i>rccFileName</i> at the location in the resource tree specified by <i>mapRoot</i>, and returns <code>true</code> if the resource is successfully unloaded and no references exist for the resource; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QResource::unregisterResource(const QString& rccFilename, const QString& resourceRoot = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qresource.html#unregisterResource">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Unregisters the resource with the given <i>rccFileName</i> at the location in the resource tree specified by <i>mapRoot</i>, and returns <code>true</code> if the resource is successfully unloaded and no references exist for the resource; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qresource.html#registerResource">registerResource</a>().</p></div>
    pub unsafe fn unregister_resource_2_q_string(
        rcc_filename: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        resource_root: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QResource_unregisterResource(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(rcc_filename)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(resource_root)
                .as_raw_ptr(),
        )
    }

    /// <p>Unregisters the resource with the given <i>rccData</i> at the location in the resource tree specified by <i>mapRoot</i>, and returns <code>true</code> if the resource is successfully unloaded and no references exist into the resource; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QResource::unregisterResource(const unsigned char* rccData, const QString& resourceRoot = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qresource.html#unregisterResource-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Unregisters the resource with the given <i>rccData</i> at the location in the resource tree specified by <i>mapRoot</i>, and returns <code>true</code> if the resource is successfully unloaded and no references exist into the resource; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qresource.html#registerResource">registerResource</a>().</p></div>
    pub unsafe fn unregister_resource_uchar_q_string(
        rcc_data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_uchar>>,
        resource_root: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QResource_unregisterResource1(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_uchar>>::cast_into(rcc_data)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(resource_root)
                .as_raw_ptr(),
        )
    }

    /// <p>Unregisters the resource with the given <i>rccFileName</i> at the location in the resource tree specified by <i>mapRoot</i>, and returns <code>true</code> if the resource is successfully unloaded and no references exist for the resource; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QResource::unregisterResource(const QString& rccFilename)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qresource.html#unregisterResource">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Unregisters the resource with the given <i>rccFileName</i> at the location in the resource tree specified by <i>mapRoot</i>, and returns <code>true</code> if the resource is successfully unloaded and no references exist for the resource; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qresource.html#registerResource">registerResource</a>().</p></div>
    pub unsafe fn unregister_resource_q_string(
        rcc_filename: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QResource_unregisterResource2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(rcc_filename)
                .as_raw_ptr(),
        )
    }

    /// <p>Unregisters the resource with the given <i>rccData</i> at the location in the resource tree specified by <i>mapRoot</i>, and returns <code>true</code> if the resource is successfully unloaded and no references exist into the resource; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QResource::unregisterResource(const unsigned char* rccData)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qresource.html#unregisterResource-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Unregisters the resource with the given <i>rccData</i> at the location in the resource tree specified by <i>mapRoot</i>, and returns <code>true</code> if the resource is successfully unloaded and no references exist into the resource; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qresource.html#registerResource">registerResource</a>().</p></div>
    pub unsafe fn unregister_resource_uchar(
        rcc_data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_uchar>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QResource_unregisterResource3(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_uchar>>::cast_into(rcc_data)
                .as_raw_ptr(),
        )
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qsavefile.html">QSaveFile</a> class provides an interface for safely writing to files.</p>
///
/// C++ class: <span style='color: green;'>```QSaveFile```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qsavefile.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsavefile.html">QSaveFile</a> class provides an interface for safely writing to files.</p>
/// <p><a href="http://doc.qt.io/qt-5/qsavefile.html">QSaveFile</a> is an I/O device for writing text and binary files, without losing existing data if the writing operation fails.</p>
/// <p>While writing, the contents will be written to a temporary file, and if no error happened, <a href="http://doc.qt.io/qt-5/qsavefile.html#commit">commit</a>() will move it to the final file. This ensures that no data at the final file is lost in case an error happens while writing, and no partially-written file is ever present at the final location. Always use <a href="http://doc.qt.io/qt-5/qsavefile.html">QSaveFile</a> when saving entire documents to disk.</p>
/// <p><a href="http://doc.qt.io/qt-5/qsavefile.html">QSaveFile</a> automatically detects errors while writing, such as the full partition situation, where <a href="http://doc.qt.io/qt-5/qiodevice.html#write">write</a>() cannot write all the bytes. It will remember that an error happened, and will discard the temporary file in <a href="http://doc.qt.io/qt-5/qsavefile.html#commit">commit</a>().</p>
/// <p>Much like with <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a>, the file is opened with <a href="http://doc.qt.io/qt-5/qsavefile.html#open">open</a>(). Data is usually read and written using <a href="http://doc.qt.io/qt-5/qdatastream.html">QDataStream</a> or <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>, but you can also call the <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>-inherited functions <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#readLine">readLine</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#readAll">readAll</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#write">write</a>().</p>
/// <p>Unlike <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a>, calling <a href="http://doc.qt.io/qt-5/qiodevice.html#close">close</a>() is not allowed. <a href="http://doc.qt.io/qt-5/qsavefile.html#commit">commit</a>() replaces it. If <a href="http://doc.qt.io/qt-5/qsavefile.html#commit">commit</a>() was not called and the <a href="http://doc.qt.io/qt-5/qsavefile.html">QSaveFile</a> instance is destroyed, the temporary file is discarded.</p>
/// <p>To abort saving due to an application error, call <a href="http://doc.qt.io/qt-5/qsavefile.html#cancelWriting">cancelWriting</a>(), so that even a call to <a href="http://doc.qt.io/qt-5/qsavefile.html#commit">commit</a>() later on will not save.</p></div>
#[repr(C)]
pub struct QSaveFile {
    _unused: u8,
}
impl QSaveFile {
    /// <p>Cancels writing the new file.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSaveFile::cancelWriting()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsavefile.html#cancelWriting">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Cancels writing the new file.</p>
    /// <p>If the application changes its mind while saving, it can call cancelWriting(), which sets an error code so that <a href="http://doc.qt.io/qt-5/qsavefile.html#commit">commit</a>() will discard the temporary file.</p>
    /// <p>Alternatively, it can simply make sure not to call <a href="http://doc.qt.io/qt-5/qsavefile.html#commit">commit</a>().</p>
    /// <p>Further write operations are possible after calling this method, but none of it will have any effect, the written file will be discarded.</p>
    /// <p>This method has no effect when direct write fallback is used. This is the case when saving over an existing file in a readonly directory: no temporary file can be created, so the existing file is overwritten no matter what, and cancelWriting() cannot do anything about that, the contents of the existing file will be lost.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsavefile.html#commit">commit</a>().</p></div>
    pub unsafe fn cancel_writing(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSaveFile_cancelWriting(self as *mut crate::QSaveFile)
    }

    /// <p>Commits the changes to disk, if all previous writes were successful.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSaveFile::commit()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsavefile.html#commit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Commits the changes to disk, if all previous writes were successful.</p>
    /// <p>It is mandatory to call this at the end of the saving operation, otherwise the file will be discarded.</p>
    /// <p>If an error happened during writing, deletes the temporary file and returns <code>false</code>. Otherwise, renames it to the final <a href="http://doc.qt.io/qt-5/qsavefile.html#fileName">fileName</a> and returns <code>true</code> on success. Finally, closes the device.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsavefile.html#cancelWriting">cancelWriting</a>().</p></div>
    pub unsafe fn commit(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSaveFile_commit(self as *mut crate::QSaveFile)
    }

    /// <p>Returns <code>true</code> if the fallback solution for saving files in read-only directories is enabled.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSaveFile::directWriteFallback() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsavefile.html#directWriteFallback">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the fallback solution for saving files in read-only directories is enabled.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsavefile.html#setDirectWriteFallback">setDirectWriteFallback</a>().</p></div>
    pub unsafe fn direct_write_fallback(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSaveFile_directWriteFallback(self as *const crate::QSaveFile)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qfiledevice.html#fileName">QFileDevice::fileName</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QString QSaveFile::fileName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsavefile.html#fileName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qfiledevice.html#fileName">QFileDevice::fileName</a>().</p>
    /// <p>Returns the name set by <a href="http://doc.qt.io/qt-5/qsavefile.html#setFileName">setFileName</a>() or to the <a href="http://doc.qt.io/qt-5/qsavefile.html">QSaveFile</a> constructor.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsavefile.html#setFileName">setFileName</a>().</p></div>
    pub unsafe fn file_name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QSaveFile_fileName(self as *const crate::QSaveFile);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QSaveFile::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QSaveFile_metaObject(self as *const crate::QSaveFile);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs a new file object to represent the file with the given <i>name</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSaveFile::QSaveFile(const QString& name)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsavefile.html#QSaveFile">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new file object to represent the file with the given <i>name</i>.</p></div>
    pub unsafe fn from_q_string(
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QSaveFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSaveFile_QSaveFile(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new file object with the given <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSaveFile::QSaveFile(QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsavefile.html#QSaveFile-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new file object with the given <i>parent</i>.</p></div>
    pub unsafe fn from_q_object(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QSaveFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSaveFile_QSaveFile1(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new file object with the given <i>parent</i> to represent the file with the specified <i>name</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSaveFile::QSaveFile(const QString& name, QObject* parent)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsavefile.html#QSaveFile-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new file object with the given <i>parent</i> to represent the file with the specified <i>name</i>.</p></div>
    pub unsafe fn from_q_string_q_object(
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QSaveFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSaveFile_QSaveFile2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qsavefile.html">QSaveFile</a> class provides an interface for safely writing to files.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSaveFile::QSaveFile()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsavefile.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsavefile.html">QSaveFile</a> class provides an interface for safely writing to files.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qsavefile.html">QSaveFile</a> is an I/O device for writing text and binary files, without losing existing data if the writing operation fails.</p>
    /// <p>While writing, the contents will be written to a temporary file, and if no error happened, <a href="http://doc.qt.io/qt-5/qsavefile.html#commit">commit</a>() will move it to the final file. This ensures that no data at the final file is lost in case an error happens while writing, and no partially-written file is ever present at the final location. Always use <a href="http://doc.qt.io/qt-5/qsavefile.html">QSaveFile</a> when saving entire documents to disk.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qsavefile.html">QSaveFile</a> automatically detects errors while writing, such as the full partition situation, where <a href="http://doc.qt.io/qt-5/qiodevice.html#write">write</a>() cannot write all the bytes. It will remember that an error happened, and will discard the temporary file in <a href="http://doc.qt.io/qt-5/qsavefile.html#commit">commit</a>().</p>
    /// <p>Much like with <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a>, the file is opened with <a href="http://doc.qt.io/qt-5/qsavefile.html#open">open</a>(). Data is usually read and written using <a href="http://doc.qt.io/qt-5/qdatastream.html">QDataStream</a> or <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>, but you can also call the <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>-inherited functions <a href="http://doc.qt.io/qt-5/qiodevice.html#read">read</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#readLine">readLine</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#readAll">readAll</a>(), <a href="http://doc.qt.io/qt-5/qiodevice.html#write">write</a>().</p>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a>, calling <a href="http://doc.qt.io/qt-5/qiodevice.html#close">close</a>() is not allowed. <a href="http://doc.qt.io/qt-5/qsavefile.html#commit">commit</a>() replaces it. If <a href="http://doc.qt.io/qt-5/qsavefile.html#commit">commit</a>() was not called and the <a href="http://doc.qt.io/qt-5/qsavefile.html">QSaveFile</a> instance is destroyed, the temporary file is discarded.</p>
    /// <p>To abort saving due to an application error, call <a href="http://doc.qt.io/qt-5/qsavefile.html#cancelWriting">cancelWriting</a>(), so that even a call to <a href="http://doc.qt.io/qt-5/qsavefile.html#commit">commit</a>() later on will not save.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QSaveFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSaveFile_QSaveFile3();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#open">QIODevice::open</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QSaveFile::open(QFlags<QIODevice::OpenModeFlag> flags)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsavefile.html#open">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qiodevice.html#open">QIODevice::open</a>().</p>
    /// <p>Opens the file using <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">OpenMode</a> <i>mode</i>, returning true if successful; otherwise false.</p>
    /// <p>Important: the <i>mode</i> must include <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">QIODevice::WriteOnly</a>. It may also have additional flags, such as <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">QIODevice::Text</a> and <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">QIODevice::Unbuffered</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">QIODevice::ReadWrite</a> and <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">QIODevice::Append</a> are not supported at the moment.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">QIODevice::OpenMode</a> and <a href="http://doc.qt.io/qt-5/qsavefile.html#setFileName">setFileName</a>().</p></div>
    pub unsafe fn open(&mut self, flags: crate::QFlags<crate::q_io_device::OpenModeFlag>) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSaveFile_open(self as *mut crate::QSaveFile, flags.to_int())
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QSaveFile::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QSaveFile_qt_metacall(
            self as *mut crate::QSaveFile,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QSaveFile::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSaveFile_qt_metacast(
            self as *mut crate::QSaveFile,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Allows writing over the existing file if necessary.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSaveFile::setDirectWriteFallback(bool enabled)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsavefile.html#setDirectWriteFallback">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Allows writing over the existing file if necessary.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qsavefile.html">QSaveFile</a> creates a temporary file in the same directory as the final file and atomically renames it. However this is not possible if the directory permissions do not allow creating new files. In order to preserve atomicity guarantees, <a href="http://doc.qt.io/qt-5/qsavefile.html#open">open</a>() fails when it cannot create the temporary file.</p>
    /// <p>In order to allow users to edit files with write permissions in a directory with restricted permissions, call setDirectWriteFallback() with <i>enabled</i> set to true, and the following calls to <a href="http://doc.qt.io/qt-5/qsavefile.html#open">open</a>() will fallback to opening the existing file directly and writing into it, without the use of a temporary file. This does not have atomicity guarantees, i.e. an application crash or for instance a power failure could lead to a partially-written file on disk. It also means <a href="http://doc.qt.io/qt-5/qsavefile.html#cancelWriting">cancelWriting</a>() has no effect, in such a case.</p>
    /// <p>Typically, to save documents edited by the user, call setDirectWriteFallback(true), and to save application internal files (configuration files, data files, ...), keep the default setting which ensures atomicity.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsavefile.html#directWriteFallback">directWriteFallback</a>().</p></div>
    pub unsafe fn set_direct_write_fallback(&mut self, enabled: bool) {
        crate::__ffi::ctr_qt_core_ffi_QSaveFile_setDirectWriteFallback(
            self as *mut crate::QSaveFile,
            enabled,
        )
    }

    /// <p>Sets the <i>name</i> of the file. The name can have no path, a relative path, or an absolute path.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSaveFile::setFileName(const QString& name)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsavefile.html#setFileName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the <i>name</i> of the file. The name can have no path, a relative path, or an absolute path.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfile.html#setFileName">QFile::setFileName</a>() and <a href="http://doc.qt.io/qt-5/qsavefile.html#fileName">fileName</a>().</p></div>
    pub unsafe fn set_file_name(
        &mut self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSaveFile_setFileName(
            self as *mut crate::QSaveFile,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
        )
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSaveFile_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QSaveFile::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSaveFile_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QSaveFile::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSaveFile_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qsemaphore.html">QSemaphore</a> class provides a general counting semaphore.</p>
///
/// C++ class: <span style='color: green;'>```QSemaphore```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qsemaphore.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsemaphore.html">QSemaphore</a> class provides a general counting semaphore.</p>
/// <p>A semaphore is a generalization of a mutex. While a mutex can only be locked once, it's possible to acquire a semaphore multiple times. Semaphores are typically used to protect a certain number of identical resources.</p>
/// <p>Semaphores support two fundamental operations, <a href="http://doc.qt.io/qt-5/qsemaphore.html#acquire">acquire</a>() and <a href="http://doc.qt.io/qt-5/qsemaphore.html#release">release</a>():</p>
/// <ul>
/// <li>acquire(<i>n</i>) tries to acquire <i>n</i> resources. If there aren't that many resources available, the call will block until this is the case.</li>
/// <li>release(<i>n</i>) releases <i>n</i> resources.</li>
/// </ul>
/// <p>There's also a <a href="http://doc.qt.io/qt-5/qsemaphore.html#tryAcquire">tryAcquire</a>() function that returns immediately if it cannot acquire the resources, and an <a href="http://doc.qt.io/qt-5/qsemaphore.html#available">available</a>() function that returns the number of available resources at any time.</p>
/// <p>Example:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qsemaphore.html#QSemaphore">QSemaphore</a></span> sem(<span class="number">5</span>);      <span class="comment">// sem.available() == 5</span>
///
///   sem<span class="operator">.</span>acquire(<span class="number">3</span>);         <span class="comment">// sem.available() == 2</span>
///   sem<span class="operator">.</span>acquire(<span class="number">2</span>);         <span class="comment">// sem.available() == 0</span>
///   sem<span class="operator">.</span>release(<span class="number">5</span>);         <span class="comment">// sem.available() == 5</span>
///   sem<span class="operator">.</span>release(<span class="number">5</span>);         <span class="comment">// sem.available() == 10</span>
///
///   sem<span class="operator">.</span>tryAcquire(<span class="number">1</span>);      <span class="comment">// sem.available() == 9, returns true</span>
///   sem<span class="operator">.</span>tryAcquire(<span class="number">250</span>);    <span class="comment">// sem.available() == 9, returns false</span>
///
/// </pre>
/// <p>A typical application of semaphores is for controlling access to a circular buffer shared by a producer thread and a consumer thread. The <a href="http://doc.qt.io/qt-5/qtcore-threads-semaphores-example.html">Semaphores Example</a> shows how to use <a href="http://doc.qt.io/qt-5/qsemaphore.html">QSemaphore</a> to solve that problem.</p>
/// <p>A non-computing example of a semaphore would be dining at a restaurant. A semaphore is initialized with the number of chairs in the restaurant. As people arrive, they want a seat. As seats are filled, <a href="http://doc.qt.io/qt-5/qsemaphore.html#available">available</a>() is decremented. As people leave, the <a href="http://doc.qt.io/qt-5/qsemaphore.html#available">available</a>() is incremented, allowing more people to enter. If a party of 10 people want to be seated, but there are only 9 seats, those 10 people will wait, but a party of 4 people would be seated (taking the available seats to 5, making the party of 10 people wait longer).</p></div>
#[repr(C)]
pub struct QSemaphore {
    _unused: u8,
}
impl QSemaphore {
    /// <p>Tries to acquire <code>n</code> resources guarded by the semaphore. If <i>n</i> &gt; <a href="http://doc.qt.io/qt-5/qsemaphore.html#available">available</a>(), this call will block until enough resources are available.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSemaphore::acquire(int n = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsemaphore.html#acquire">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Tries to acquire <code>n</code> resources guarded by the semaphore. If <i>n</i> &gt; <a href="http://doc.qt.io/qt-5/qsemaphore.html#available">available</a>(), this call will block until enough resources are available.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsemaphore.html#release">release</a>(), <a href="http://doc.qt.io/qt-5/qsemaphore.html#available">available</a>(), and <a href="http://doc.qt.io/qt-5/qsemaphore.html#tryAcquire">tryAcquire</a>().</p></div>
    pub unsafe fn acquire_1a(&mut self, n: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QSemaphore_acquire(self as *mut crate::QSemaphore, n)
    }

    /// <p>Tries to acquire <code>n</code> resources guarded by the semaphore. If <i>n</i> &gt; <a href="http://doc.qt.io/qt-5/qsemaphore.html#available">available</a>(), this call will block until enough resources are available.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSemaphore::acquire()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsemaphore.html#acquire">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Tries to acquire <code>n</code> resources guarded by the semaphore. If <i>n</i> &gt; <a href="http://doc.qt.io/qt-5/qsemaphore.html#available">available</a>(), this call will block until enough resources are available.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsemaphore.html#release">release</a>(), <a href="http://doc.qt.io/qt-5/qsemaphore.html#available">available</a>(), and <a href="http://doc.qt.io/qt-5/qsemaphore.html#tryAcquire">tryAcquire</a>().</p></div>
    pub unsafe fn acquire_0a(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSemaphore_acquire1(self as *mut crate::QSemaphore)
    }

    /// <p>Returns the number of resources currently available to the semaphore. This number can never be negative.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QSemaphore::available() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsemaphore.html#available">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of resources currently available to the semaphore. This number can never be negative.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsemaphore.html#acquire">acquire</a>() and <a href="http://doc.qt.io/qt-5/qsemaphore.html#release">release</a>().</p></div>
    pub unsafe fn available(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QSemaphore_available(self as *const crate::QSemaphore)
    }

    /// <p>Creates a new semaphore and initializes the number of resources it guards to <i>n</i> (by default, 0).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSemaphore::QSemaphore(int n = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsemaphore.html#QSemaphore">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a new semaphore and initializes the number of resources it guards to <i>n</i> (by default, 0).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsemaphore.html#release">release</a>() and <a href="http://doc.qt.io/qt-5/qsemaphore.html#available">available</a>().</p></div>
    pub unsafe fn new_1a(n: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QSemaphore> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSemaphore_QSemaphore(n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qsemaphore.html">QSemaphore</a> class provides a general counting semaphore.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSemaphore::QSemaphore()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsemaphore.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsemaphore.html">QSemaphore</a> class provides a general counting semaphore.</p>
    /// <p>A semaphore is a generalization of a mutex. While a mutex can only be locked once, it's possible to acquire a semaphore multiple times. Semaphores are typically used to protect a certain number of identical resources.</p>
    /// <p>Semaphores support two fundamental operations, <a href="http://doc.qt.io/qt-5/qsemaphore.html#acquire">acquire</a>() and <a href="http://doc.qt.io/qt-5/qsemaphore.html#release">release</a>():</p>
    /// <ul>
    /// <li>acquire(<i>n</i>) tries to acquire <i>n</i> resources. If there aren't that many resources available, the call will block until this is the case.</li>
    /// <li>release(<i>n</i>) releases <i>n</i> resources.</li>
    /// </ul>
    /// <p>There's also a <a href="http://doc.qt.io/qt-5/qsemaphore.html#tryAcquire">tryAcquire</a>() function that returns immediately if it cannot acquire the resources, and an <a href="http://doc.qt.io/qt-5/qsemaphore.html#available">available</a>() function that returns the number of available resources at any time.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsemaphore.html#QSemaphore">QSemaphore</a></span> sem(<span class="number">5</span>);      <span class="comment">// sem.available() == 5</span>
    ///
    ///   sem<span class="operator">.</span>acquire(<span class="number">3</span>);         <span class="comment">// sem.available() == 2</span>
    ///   sem<span class="operator">.</span>acquire(<span class="number">2</span>);         <span class="comment">// sem.available() == 0</span>
    ///   sem<span class="operator">.</span>release(<span class="number">5</span>);         <span class="comment">// sem.available() == 5</span>
    ///   sem<span class="operator">.</span>release(<span class="number">5</span>);         <span class="comment">// sem.available() == 10</span>
    ///
    ///   sem<span class="operator">.</span>tryAcquire(<span class="number">1</span>);      <span class="comment">// sem.available() == 9, returns true</span>
    ///   sem<span class="operator">.</span>tryAcquire(<span class="number">250</span>);    <span class="comment">// sem.available() == 9, returns false</span>
    ///
    /// </pre>
    /// <p>A typical application of semaphores is for controlling access to a circular buffer shared by a producer thread and a consumer thread. The <a href="http://doc.qt.io/qt-5/qtcore-threads-semaphores-example.html">Semaphores Example</a> shows how to use <a href="http://doc.qt.io/qt-5/qsemaphore.html">QSemaphore</a> to solve that problem.</p>
    /// <p>A non-computing example of a semaphore would be dining at a restaurant. A semaphore is initialized with the number of chairs in the restaurant. As people arrive, they want a seat. As seats are filled, <a href="http://doc.qt.io/qt-5/qsemaphore.html#available">available</a>() is decremented. As people leave, the <a href="http://doc.qt.io/qt-5/qsemaphore.html#available">available</a>() is incremented, allowing more people to enter. If a party of 10 people want to be seated, but there are only 9 seats, those 10 people will wait, but a party of 4 people would be seated (taking the available seats to 5, making the party of 10 people wait longer).</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QSemaphore> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSemaphore_QSemaphore1();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Releases <i>n</i> resources guarded by the semaphore.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSemaphore::release(int n = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsemaphore.html#release">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Releases <i>n</i> resources guarded by the semaphore.</p>
    /// <p>This function can be used to "create" resources as well. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsemaphore.html#QSemaphore">QSemaphore</a></span> sem(<span class="number">5</span>);      <span class="comment">// a semaphore that guards 5 resources</span>
    ///   sem<span class="operator">.</span>acquire(<span class="number">5</span>);         <span class="comment">// acquire all 5 resources</span>
    ///   sem<span class="operator">.</span>release(<span class="number">5</span>);         <span class="comment">// release the 5 resources</span>
    ///   sem<span class="operator">.</span>release(<span class="number">10</span>);        <span class="comment">// "create" 10 new resources</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsemaphore.html#acquire">acquire</a>() and <a href="http://doc.qt.io/qt-5/qsemaphore.html#available">available</a>().</p></div>
    pub unsafe fn release_1a(&mut self, n: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QSemaphore_release(self as *mut crate::QSemaphore, n)
    }

    /// <p>Releases <i>n</i> resources guarded by the semaphore.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSemaphore::release()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsemaphore.html#release">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Releases <i>n</i> resources guarded by the semaphore.</p>
    /// <p>This function can be used to "create" resources as well. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsemaphore.html#QSemaphore">QSemaphore</a></span> sem(<span class="number">5</span>);      <span class="comment">// a semaphore that guards 5 resources</span>
    ///   sem<span class="operator">.</span>acquire(<span class="number">5</span>);         <span class="comment">// acquire all 5 resources</span>
    ///   sem<span class="operator">.</span>release(<span class="number">5</span>);         <span class="comment">// release the 5 resources</span>
    ///   sem<span class="operator">.</span>release(<span class="number">10</span>);        <span class="comment">// "create" 10 new resources</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsemaphore.html#acquire">acquire</a>() and <a href="http://doc.qt.io/qt-5/qsemaphore.html#available">available</a>().</p></div>
    pub unsafe fn release_0a(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSemaphore_release1(self as *mut crate::QSemaphore)
    }

    /// <p>Tries to acquire <code>n</code> resources guarded by the semaphore and returns <code>true</code> on success. If <a href="http://doc.qt.io/qt-5/qsemaphore.html#available">available</a>() &lt; <i>n</i>, this call immediately returns <code>false</code> without acquiring any resources.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSemaphore::tryAcquire(int n = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsemaphore.html#tryAcquire">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Tries to acquire <code>n</code> resources guarded by the semaphore and returns <code>true</code> on success. If <a href="http://doc.qt.io/qt-5/qsemaphore.html#available">available</a>() &lt; <i>n</i>, this call immediately returns <code>false</code> without acquiring any resources.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsemaphore.html#QSemaphore">QSemaphore</a></span> sem(<span class="number">5</span>);      <span class="comment">// sem.available() == 5</span>
    ///   sem<span class="operator">.</span>tryAcquire(<span class="number">250</span>);    <span class="comment">// sem.available() == 5, returns false</span>
    ///   sem<span class="operator">.</span>tryAcquire(<span class="number">3</span>);      <span class="comment">// sem.available() == 2, returns true</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsemaphore.html#acquire">acquire</a>().</p></div>
    pub unsafe fn try_acquire_1a(&mut self, n: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSemaphore_tryAcquire(self as *mut crate::QSemaphore, n)
    }

    /// <p>Tries to acquire <code>n</code> resources guarded by the semaphore and returns <code>true</code> on success. If <a href="http://doc.qt.io/qt-5/qsemaphore.html#available">available</a>() &lt; <i>n</i>, this call will wait for at most <i>timeout</i> milliseconds for resources to become available.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSemaphore::tryAcquire(int n, int timeout)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsemaphore.html#tryAcquire-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Tries to acquire <code>n</code> resources guarded by the semaphore and returns <code>true</code> on success. If <a href="http://doc.qt.io/qt-5/qsemaphore.html#available">available</a>() &lt; <i>n</i>, this call will wait for at most <i>timeout</i> milliseconds for resources to become available.</p>
    /// <p>Note: Passing a negative number as the <i>timeout</i> is equivalent to calling <a href="http://doc.qt.io/qt-5/qsemaphore.html#acquire">acquire</a>(), i.e. this function will wait forever for resources to become available if <i>timeout</i> is negative.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsemaphore.html#QSemaphore">QSemaphore</a></span> sem(<span class="number">5</span>);            <span class="comment">// sem.available() == 5</span>
    ///   sem<span class="operator">.</span>tryAcquire(<span class="number">250</span><span class="operator">,</span> <span class="number">1000</span>);    <span class="comment">// sem.available() == 5, waits 1000 milliseconds and returns false</span>
    ///   sem<span class="operator">.</span>tryAcquire(<span class="number">3</span><span class="operator">,</span> <span class="number">30000</span>);     <span class="comment">// sem.available() == 2, returns true without waiting</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsemaphore.html#acquire">acquire</a>().</p></div>
    pub unsafe fn try_acquire_2a(
        &mut self,
        n: ::std::os::raw::c_int,
        timeout: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSemaphore_tryAcquire1(
            self as *mut crate::QSemaphore,
            n,
            timeout,
        )
    }

    /// <p>Tries to acquire <code>n</code> resources guarded by the semaphore and returns <code>true</code> on success. If <a href="http://doc.qt.io/qt-5/qsemaphore.html#available">available</a>() &lt; <i>n</i>, this call immediately returns <code>false</code> without acquiring any resources.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSemaphore::tryAcquire()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsemaphore.html#tryAcquire">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Tries to acquire <code>n</code> resources guarded by the semaphore and returns <code>true</code> on success. If <a href="http://doc.qt.io/qt-5/qsemaphore.html#available">available</a>() &lt; <i>n</i>, this call immediately returns <code>false</code> without acquiring any resources.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsemaphore.html#QSemaphore">QSemaphore</a></span> sem(<span class="number">5</span>);      <span class="comment">// sem.available() == 5</span>
    ///   sem<span class="operator">.</span>tryAcquire(<span class="number">250</span>);    <span class="comment">// sem.available() == 5, returns false</span>
    ///   sem<span class="operator">.</span>tryAcquire(<span class="number">3</span>);      <span class="comment">// sem.available() == 2, returns true</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsemaphore.html#acquire">acquire</a>().</p></div>
    pub unsafe fn try_acquire_0a(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSemaphore_tryAcquire2(self as *mut crate::QSemaphore)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html">QSequentialAnimationGroup</a> class provides a sequential group of animations.</p>
///
/// C++ class: <span style='color: green;'>```QSequentialAnimationGroup```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html">QSequentialAnimationGroup</a> class provides a sequential group of animations.</p>
/// <p><a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html">QSequentialAnimationGroup</a> is a <a href="http://doc.qt.io/qt-5/qanimationgroup.html">QAnimationGroup</a> that runs its animations in sequence, i.e., it starts one animation after another has finished playing. The animations are played in the order they are added to the group (using <a href="http://doc.qt.io/qt-5/qanimationgroup.html#addAnimation">addAnimation()</a> or <a href="http://doc.qt.io/qt-5/qanimationgroup.html#insertAnimation">insertAnimation()</a>). The animation group finishes when its last animation has finished.</p>
/// <p>At each moment there is at most one animation that is active in the group; it is returned by <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#currentAnimation-prop">currentAnimation</a>(). An empty group has no current animation.</p>
/// <p>A sequential animation group can be treated as any other animation, i.e., it can be started, stopped, and added to other groups. You can also call <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#addPause">addPause</a>() or <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#insertPause">insertPause</a>() to add a pause to a sequential animation group.</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#QSequentialAnimationGroup">QSequentialAnimationGroup</a></span> <span class="operator">*</span>group <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#QSequentialAnimationGroup">QSequentialAnimationGroup</a></span>;
///
///   group<span class="operator">-</span><span class="operator">&gt;</span>addAnimation(anim1);
///   group<span class="operator">-</span><span class="operator">&gt;</span>addAnimation(anim2);
///
///   group<span class="operator">-</span><span class="operator">&gt;</span>start();
///
/// </pre>
/// <p>In this example, <code>anim1</code> and <code>anim2</code> are two already set up <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a>s.</p></div>
#[repr(C)]
pub struct QSequentialAnimationGroup {
    _unused: u8,
}
impl QSequentialAnimationGroup {
    /// <p><a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html">QSequentialAnimationGroup</a> emits this signal when <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#currentAnimation-prop">currentAnimation</a> has been changed. <i>current</i> is the current animation.</p>
    ///
    /// Returns a built-in Qt signal `QSequentialAnimationGroup::currentAnimationChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#currentAnimationChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p><a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html">QSequentialAnimationGroup</a> emits this signal when <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#currentAnimation-prop">currentAnimation</a> has been changed. <i>current</i> is the current animation.</p>
    /// <p><b>Note:</b> Notifier signal for property <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#currentAnimation-prop">currentAnimation</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#currentAnimation-prop">currentAnimation</a>().</p></div>
    pub fn current_animation_changed(&self) -> crate::Signal<(*mut crate::QAbstractAnimation,)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2currentAnimationChanged(QAbstractAnimation*)\0",
                ),
            )
        }
    }

    /// <p>Adds a pause of <i>msecs</i> to this animation group. The pause is considered as a special type of animation, thus <a href="http://doc.qt.io/qt-5/qanimationgroup.html#animationCount">animationCount</a> will be increased by one.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPauseAnimation* QSequentialAnimationGroup::addPause(int msecs)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#addPause">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds a pause of <i>msecs</i> to this animation group. The pause is considered as a special type of animation, thus <a href="http://doc.qt.io/qt-5/qanimationgroup.html#animationCount">animationCount</a> will be increased by one.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#insertPause">insertPause</a>() and <a href="http://doc.qt.io/qt-5/qanimationgroup.html#addAnimation">QAnimationGroup::addAnimation</a>().</p></div>
    pub unsafe fn add_pause(
        &mut self,
        msecs: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutPtr<crate::QPauseAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSequentialAnimationGroup_addPause(
            self as *mut crate::QSequentialAnimationGroup,
            msecs,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns the animation in the current time.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* QSequentialAnimationGroup::currentAnimation() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#currentAnimation-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the animation in the current time.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QAbstractAnimation *</td><td class="memItemRight bottomAlign"><span class="name"><b>currentAnimation</b></span>() const</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#currentAnimationChanged">currentAnimationChanged</a></b></span>(QAbstractAnimation *<i>current</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn current_animation(&self) -> ::cpp_utils::MutPtr<crate::QAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSequentialAnimationGroup_currentAnimation(
            self as *const crate::QSequentialAnimationGroup,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractanimation.html#duration">QAbstractAnimation::duration</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual int QSequentialAnimationGroup::duration() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#duration">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractanimation.html#duration">QAbstractAnimation::duration</a>().</p></div>
    pub unsafe fn duration(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QSequentialAnimationGroup_duration(
            self as *const crate::QSequentialAnimationGroup,
        )
    }

    /// <p>Inserts a pause of <i>msecs</i> milliseconds at <i>index</i> in this animation group.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPauseAnimation* QSequentialAnimationGroup::insertPause(int index, int msecs)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#insertPause">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts a pause of <i>msecs</i> milliseconds at <i>index</i> in this animation group.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#addPause">addPause</a>() and <a href="http://doc.qt.io/qt-5/qanimationgroup.html#insertAnimation">QAnimationGroup::insertAnimation</a>().</p></div>
    pub unsafe fn insert_pause(
        &mut self,
        index: ::std::os::raw::c_int,
        msecs: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutPtr<crate::QPauseAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSequentialAnimationGroup_insertPause(
            self as *mut crate::QSequentialAnimationGroup,
            index,
            msecs,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QSequentialAnimationGroup::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSequentialAnimationGroup_metaObject(
            self as *const crate::QSequentialAnimationGroup,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html">QSequentialAnimationGroup</a>. <i>parent</i> is passed to <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>'s constructor.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSequentialAnimationGroup::QSequentialAnimationGroup(QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#QSequentialAnimationGroup">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html">QSequentialAnimationGroup</a>. <i>parent</i> is passed to <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>'s constructor.</p></div>
    pub unsafe fn new_1a(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QSequentialAnimationGroup> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QSequentialAnimationGroup_QSequentialAnimationGroup(
                ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                    .as_mut_raw_ptr(),
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html">QSequentialAnimationGroup</a> class provides a sequential group of animations.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSequentialAnimationGroup::QSequentialAnimationGroup()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html">QSequentialAnimationGroup</a> class provides a sequential group of animations.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html">QSequentialAnimationGroup</a> is a <a href="http://doc.qt.io/qt-5/qanimationgroup.html">QAnimationGroup</a> that runs its animations in sequence, i.e., it starts one animation after another has finished playing. The animations are played in the order they are added to the group (using <a href="http://doc.qt.io/qt-5/qanimationgroup.html#addAnimation">addAnimation()</a> or <a href="http://doc.qt.io/qt-5/qanimationgroup.html#insertAnimation">insertAnimation()</a>). The animation group finishes when its last animation has finished.</p>
    /// <p>At each moment there is at most one animation that is active in the group; it is returned by <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#currentAnimation-prop">currentAnimation</a>(). An empty group has no current animation.</p>
    /// <p>A sequential animation group can be treated as any other animation, i.e., it can be started, stopped, and added to other groups. You can also call <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#addPause">addPause</a>() or <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#insertPause">insertPause</a>() to add a pause to a sequential animation group.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#QSequentialAnimationGroup">QSequentialAnimationGroup</a></span> <span class="operator">*</span>group <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#QSequentialAnimationGroup">QSequentialAnimationGroup</a></span>;
    ///
    ///   group<span class="operator">-</span><span class="operator">&gt;</span>addAnimation(anim1);
    ///   group<span class="operator">-</span><span class="operator">&gt;</span>addAnimation(anim2);
    ///
    ///   group<span class="operator">-</span><span class="operator">&gt;</span>start();
    ///
    /// </pre>
    /// <p>In this example, <code>anim1</code> and <code>anim2</code> are two already set up <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a>s.</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QSequentialAnimationGroup> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QSequentialAnimationGroup_QSequentialAnimationGroup1();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QSequentialAnimationGroup::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QSequentialAnimationGroup_qt_metacall(
            self as *mut crate::QSequentialAnimationGroup,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QSequentialAnimationGroup::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSequentialAnimationGroup_qt_metacast(
            self as *mut crate::QSequentialAnimationGroup,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSequentialAnimationGroup_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QSequentialAnimationGroup::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSequentialAnimationGroup_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QSequentialAnimationGroup::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSequentialAnimationGroup_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_settings {
    //! C++ type: <span style='color: green;'>```QSettings```</span>

    /// <p>The following status values are possible:</p>
    ///
    /// C++ enum: <span style='color: green;'>```QSettings::Status```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#Status-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The following status values are possible:</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#status">status</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Status(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Status {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Status(value)
        }
    }

    impl From<Status> for ::std::os::raw::c_int {
        fn from(value: Status) -> Self {
            value.0
        }
    }

    impl Status {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Status {
        /// No error occurred. (C++ enum variant: <span style='color: green;'>```NoError = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoError: crate::q_settings::Status = crate::q_settings::Status(0);
        /// An access error occurred (e.g. trying to write to a read-only file). (C++ enum variant: <span style='color: green;'>```AccessError = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const AccessError: crate::q_settings::Status = crate::q_settings::Status(1);
        /// A format error occurred (e.g. loading a malformed INI file). (C++ enum variant: <span style='color: green;'>```FormatError = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const FormatError: crate::q_settings::Status = crate::q_settings::Status(2);
    }

    /// <p>This enum type specifies the storage format used by <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a>.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QSettings::Format```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type specifies the storage format used by <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a>.</p>
    ///
    /// <p>On Unix, NativeFormat and IniFormat mean the same thing, except that the file extension is different (<code>.conf</code> for NativeFormat, <code>.ini</code> for IniFormat).</p>
    /// <p>The INI file format is a Windows file format that Qt supports on all platforms. In the absence of an INI standard, we try to follow what Microsoft does, with the following exceptions:</p>
    /// <ul>
    /// <li>If you store types that <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> can't convert to <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> (e.g., <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a>, <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a>, and <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a>), Qt uses an <code>@</code>-based syntax to encode the type. For example:<pre class="cpp">
    ///
    ///   pos <span class="operator">=</span> @Point(<span class="number">100</span> <span class="number">100</span>)
    ///
    /// </pre>
    /// <p>To minimize compatibility issues, any <code>@</code> that doesn't appear at the first position in the value or that isn't followed by a Qt type (<code>Point</code>, <code>Rect</code>, <code>Size</code>, etc.) is treated as a normal character.</p>
    /// </li>
    /// <li>Although backslash is a special character in INI files, most Windows applications don't escape backslashes (<code>\</code>) in file paths:<pre class="cpp">
    ///
    ///   windir <span class="operator">=</span> C:\Windows
    ///
    /// </pre>
    /// <p><a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> always treats backslash as a special character and provides no API for reading or writing such entries.</p>
    /// </li>
    /// <li>The INI file format has severe restrictions on the syntax of a key. Qt works around this by using <code>%</code> as an escape character in keys. In addition, if you save a top-level setting (a key with no slashes in it, e.g., "someKey"), it will appear in the INI file's "General" section. To avoid overwriting other keys, if you save something using a key such as "General/someKey", the key will be located in the "%General" section, <i>not</i> in the "General" section.</li>
    /// <li>Following the philosophy that we should be liberal in what we accept and conservative in what we generate, <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> will accept Latin-1 encoded INI files, but generate pure ASCII files, where non-ASCII values are encoded using standard INI escape sequences. To make the INI files more readable (but potentially less compatible), call <a href="http://doc.qt.io/qt-5/qsettings.html#setIniCodec">setIniCodec</a>().</li>
    /// </ul>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#registerFormat">registerFormat</a>() and <a href="http://doc.qt.io/qt-5/qsettings.html#setPath">setPath</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Format(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Format {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Format(value)
        }
    }

    impl From<Format> for ::std::os::raw::c_int {
        fn from(value: Format) -> Self {
            value.0
        }
    }

    impl Format {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Format {
        /// Store the settings using the most appropriate storage format for the platform. On Windows, this means the system registry; on <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS, this means the CFPreferences API; on Unix, this means textual configuration files in INI format. (C++ enum variant: <span style='color: green;'>```NativeFormat = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NativeFormat: crate::q_settings::Format = crate::q_settings::Format(0);
        /// Store the settings in INI files. (C++ enum variant: <span style='color: green;'>```IniFormat = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const IniFormat: crate::q_settings::Format = crate::q_settings::Format(1);
        /// Special value returned by <a href="http://doc.qt.io/qt-5/qsettings.html#registerFormat">registerFormat</a>(). (C++ enum variant: <span style='color: green;'>```InvalidFormat = 16```</span>)
        #[allow(non_upper_case_globals)]
        pub const InvalidFormat: crate::q_settings::Format = crate::q_settings::Format(16);
        /// C++ enum variant: <span style='color: green;'>```CustomFormat1 = 17```</span>
        #[allow(non_upper_case_globals)]
        pub const CustomFormat1: crate::q_settings::Format = crate::q_settings::Format(17);
        /// C++ enum variant: <span style='color: green;'>```CustomFormat2 = 18```</span>
        #[allow(non_upper_case_globals)]
        pub const CustomFormat2: crate::q_settings::Format = crate::q_settings::Format(18);
        /// C++ enum variant: <span style='color: green;'>```CustomFormat3 = 19```</span>
        #[allow(non_upper_case_globals)]
        pub const CustomFormat3: crate::q_settings::Format = crate::q_settings::Format(19);
        /// C++ enum variant: <span style='color: green;'>```CustomFormat4 = 20```</span>
        #[allow(non_upper_case_globals)]
        pub const CustomFormat4: crate::q_settings::Format = crate::q_settings::Format(20);
        /// C++ enum variant: <span style='color: green;'>```CustomFormat5 = 21```</span>
        #[allow(non_upper_case_globals)]
        pub const CustomFormat5: crate::q_settings::Format = crate::q_settings::Format(21);
        /// C++ enum variant: <span style='color: green;'>```CustomFormat6 = 22```</span>
        #[allow(non_upper_case_globals)]
        pub const CustomFormat6: crate::q_settings::Format = crate::q_settings::Format(22);
        /// C++ enum variant: <span style='color: green;'>```CustomFormat7 = 23```</span>
        #[allow(non_upper_case_globals)]
        pub const CustomFormat7: crate::q_settings::Format = crate::q_settings::Format(23);
        /// C++ enum variant: <span style='color: green;'>```CustomFormat8 = 24```</span>
        #[allow(non_upper_case_globals)]
        pub const CustomFormat8: crate::q_settings::Format = crate::q_settings::Format(24);
        /// C++ enum variant: <span style='color: green;'>```CustomFormat9 = 25```</span>
        #[allow(non_upper_case_globals)]
        pub const CustomFormat9: crate::q_settings::Format = crate::q_settings::Format(25);
        /// C++ enum variant: <span style='color: green;'>```CustomFormat10 = 26```</span>
        #[allow(non_upper_case_globals)]
        pub const CustomFormat10: crate::q_settings::Format = crate::q_settings::Format(26);
        /// C++ enum variant: <span style='color: green;'>```CustomFormat11 = 27```</span>
        #[allow(non_upper_case_globals)]
        pub const CustomFormat11: crate::q_settings::Format = crate::q_settings::Format(27);
        /// C++ enum variant: <span style='color: green;'>```CustomFormat12 = 28```</span>
        #[allow(non_upper_case_globals)]
        pub const CustomFormat12: crate::q_settings::Format = crate::q_settings::Format(28);
        /// C++ enum variant: <span style='color: green;'>```CustomFormat13 = 29```</span>
        #[allow(non_upper_case_globals)]
        pub const CustomFormat13: crate::q_settings::Format = crate::q_settings::Format(29);
        /// C++ enum variant: <span style='color: green;'>```CustomFormat14 = 30```</span>
        #[allow(non_upper_case_globals)]
        pub const CustomFormat14: crate::q_settings::Format = crate::q_settings::Format(30);
        /// C++ enum variant: <span style='color: green;'>```CustomFormat15 = 31```</span>
        #[allow(non_upper_case_globals)]
        pub const CustomFormat15: crate::q_settings::Format = crate::q_settings::Format(31);
        /// C++ enum variant: <span style='color: green;'>```CustomFormat16 = 32```</span>
        #[allow(non_upper_case_globals)]
        pub const CustomFormat16: crate::q_settings::Format = crate::q_settings::Format(32);
    }

    /// <p>This enum specifies whether settings are user-specific or shared by all users of the same system.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QSettings::Scope```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#Scope-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum specifies whether settings are user-specific or shared by all users of the same system.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#setPath">setPath</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Scope(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Scope {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Scope(value)
        }
    }

    impl From<Scope> for ::std::os::raw::c_int {
        fn from(value: Scope) -> Self {
            value.0
        }
    }

    impl Scope {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Scope {
        /// Store settings in a location specific to the current user (e.g., in the user's home directory). (C++ enum variant: <span style='color: green;'>```UserScope = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const UserScope: crate::q_settings::Scope = crate::q_settings::Scope(0);
        /// Store settings in a global location, so that all users on the same machine access the same set of settings. (C++ enum variant: <span style='color: green;'>```SystemScope = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const SystemScope: crate::q_settings::Scope = crate::q_settings::Scope(1);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> class provides persistent platform-independent application settings.</p>
///
/// C++ class: <span style='color: green;'>```QSettings```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qsettings.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> class provides persistent platform-independent application settings.</p>
/// <p>Users normally expect an application to remember its settings (window sizes and positions, options, etc.) across sessions. This information is often stored in the system registry on Windows, and in property list files on <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS. On Unix systems, in the absence of a standard, many applications (including the KDE applications) use INI text files.</p>
/// <p><a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> is an abstraction around these technologies, enabling you to save and restore application settings in a portable manner. It also supports <a href="http://doc.qt.io/qt-5/qsettings.html#registerFormat">custom storage formats</a>.</p>
/// <p><a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a>'s API is based on <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>, allowing you to save most value-based types, such as <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a>, and <a href="http://doc.qt.io/qt-5/qimage.html">QImage</a>, with the minimum of effort.</p>
/// <p>If all you need is a non-persistent memory-based structure, consider using <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>&gt; instead.</p>
/// <a name="basic-usage"></a></div>
#[repr(C)]
pub struct QSettings {
    _unused: u8,
}
impl QSettings {
    /// <p>Returns a list of all keys, including subkeys, that can be read using the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QSettings::allKeys() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#allKeys">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of all keys, including subkeys, that can be read using the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsettings.html#QSettings">QSettings</a></span> settings;
    ///   settings<span class="operator">.</span>setValue(<span class="string">"fridge/color"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcolor.html">QColor</a></span>(<span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>white));
    ///   settings<span class="operator">.</span>setValue(<span class="string">"fridge/size"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsize.html">QSize</a></span>(<span class="number">32</span><span class="operator">,</span> <span class="number">96</span>));
    ///   settings<span class="operator">.</span>setValue(<span class="string">"sofa"</span><span class="operator">,</span> <span class="keyword">true</span>);
    ///   settings<span class="operator">.</span>setValue(<span class="string">"tv"</span><span class="operator">,</span> <span class="keyword">false</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> keys <span class="operator">=</span> settings<span class="operator">.</span>allKeys();
    ///   <span class="comment">// keys: ["fridge/color", "fridge/size", "sofa", "tv"]</span>
    ///
    /// </pre>
    /// <p>If a group is set using <a href="http://doc.qt.io/qt-5/qsettings.html#beginGroup">beginGroup</a>(), only the keys in the group are returned, without the group prefix:</p>
    /// <pre class="cpp">
    ///
    ///   settings<span class="operator">.</span>beginGroup(<span class="string">"fridge"</span>);
    ///   keys <span class="operator">=</span> settings<span class="operator">.</span>allKeys();
    ///   <span class="comment">// keys: ["color", "size"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#childGroups">childGroups</a>() and <a href="http://doc.qt.io/qt-5/qsettings.html#childKeys">childKeys</a>().</p></div>
    pub unsafe fn all_keys(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QSettings_allKeys(self as *const crate::QSettings);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the application name used for storing the settings.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QSettings::applicationName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#applicationName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the application name used for storing the settings.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationName-prop">QCoreApplication::applicationName</a>(), <a href="http://doc.qt.io/qt-5/qsettings.html#format">format</a>(), <a href="http://doc.qt.io/qt-5/qsettings.html#scope">scope</a>(), and <a href="http://doc.qt.io/qt-5/qsettings.html#organizationName">organizationName</a>().</p></div>
    pub unsafe fn application_name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSettings_applicationName(
            self as *const crate::QSettings,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Appends <i>prefix</i> to the current group.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSettings::beginGroup(const QString& prefix)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#beginGroup">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>prefix</i> to the current group.</p>
    /// <p>The current group is automatically prepended to all keys specified to <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a>. In addition, query functions such as <a href="http://doc.qt.io/qt-5/qsettings.html#childGroups">childGroups</a>(), <a href="http://doc.qt.io/qt-5/qsettings.html#childKeys">childKeys</a>(), and <a href="http://doc.qt.io/qt-5/qsettings.html#allKeys">allKeys</a>() are based on the group. By default, no group is set.</p>
    /// <p>Groups are useful to avoid typing in the same setting paths over and over. For example:</p>
    /// <pre class="cpp">
    ///
    ///   settings<span class="operator">.</span>beginGroup(<span class="string">"mainwindow"</span>);
    ///   settings<span class="operator">.</span>setValue(<span class="string">"size"</span><span class="operator">,</span> win<span class="operator">-</span><span class="operator">&gt;</span>size());
    ///   settings<span class="operator">.</span>setValue(<span class="string">"fullScreen"</span><span class="operator">,</span> win<span class="operator">-</span><span class="operator">&gt;</span>isFullScreen());
    ///   settings<span class="operator">.</span>endGroup();
    ///
    ///   settings<span class="operator">.</span>beginGroup(<span class="string">"outputpanel"</span>);
    ///   settings<span class="operator">.</span>setValue(<span class="string">"visible"</span><span class="operator">,</span> panel<span class="operator">-</span><span class="operator">&gt;</span>isVisible());
    ///   settings<span class="operator">.</span>endGroup();
    ///
    /// </pre>
    /// <p>This will set the value of three settings:</p>
    /// <ul>
    /// <li><code>mainwindow/size</code></li>
    /// <li><code>mainwindow/fullScreen</code></li>
    /// <li><code>outputpanel/visible</code></li>
    /// </ul>
    /// <p>Call <a href="http://doc.qt.io/qt-5/qsettings.html#endGroup">endGroup</a>() to reset the current group to what it was before the corresponding beginGroup() call. Groups can be nested.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#endGroup">endGroup</a>() and <a href="http://doc.qt.io/qt-5/qsettings.html#group">group</a>().</p></div>
    pub unsafe fn begin_group(
        &mut self,
        prefix: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSettings_beginGroup(
            self as *mut crate::QSettings,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(prefix)
                .as_raw_ptr(),
        )
    }

    /// <p>Adds <i>prefix</i> to the current group and starts reading from an array. Returns the size of the array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QSettings::beginReadArray(const QString& prefix)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#beginReadArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds <i>prefix</i> to the current group and starts reading from an array. Returns the size of the array.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">struct</span> Login {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> userName;
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> password;
    ///   };
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span>Login<span class="operator">&gt;</span> logins;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsettings.html#QSettings">QSettings</a></span> settings;
    ///   <span class="type">int</span> size <span class="operator">=</span> settings<span class="operator">.</span>beginReadArray(<span class="string">"logins"</span>);
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> size; <span class="operator">+</span><span class="operator">+</span>i) {
    /// &#32;     settings<span class="operator">.</span>setArrayIndex(i);
    /// &#32;     Login login;
    /// &#32;     login<span class="operator">.</span>userName <span class="operator">=</span> settings<span class="operator">.</span>value(<span class="string">"userName"</span>)<span class="operator">.</span>toString();
    /// &#32;     login<span class="operator">.</span>password <span class="operator">=</span> settings<span class="operator">.</span>value(<span class="string">"password"</span>)<span class="operator">.</span>toString();
    /// &#32;     logins<span class="operator">.</span>append(login);
    ///   }
    ///   settings<span class="operator">.</span>endArray();
    ///
    /// </pre>
    /// <p>Use <a href="http://doc.qt.io/qt-5/qsettings.html#beginWriteArray">beginWriteArray</a>() to write the array in the first place.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#beginWriteArray">beginWriteArray</a>(), <a href="http://doc.qt.io/qt-5/qsettings.html#endArray">endArray</a>(), and <a href="http://doc.qt.io/qt-5/qsettings.html#setArrayIndex">setArrayIndex</a>().</p></div>
    pub unsafe fn begin_read_array(
        &mut self,
        prefix: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QSettings_beginReadArray(
            self as *mut crate::QSettings,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(prefix)
                .as_raw_ptr(),
        )
    }

    /// <p>Adds <i>prefix</i> to the current group and starts writing an array of size <i>size</i>. If <i>size</i> is -1 (the default), it is automatically determined based on the indexes of the entries written.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSettings::beginWriteArray(const QString& prefix, int size = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#beginWriteArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds <i>prefix</i> to the current group and starts writing an array of size <i>size</i>. If <i>size</i> is -1 (the default), it is automatically determined based on the indexes of the entries written.</p>
    /// <p>If you have many occurrences of a certain set of keys, you can use arrays to make your life easier. For example, let's suppose that you want to save a variable-length list of user names and passwords. You could then write:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">struct</span> Login {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> userName;
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> password;
    ///   };
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span>Login<span class="operator">&gt;</span> logins;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsettings.html#QSettings">QSettings</a></span> settings;
    ///   settings<span class="operator">.</span>beginWriteArray(<span class="string">"logins"</span>);
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> logins<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
    /// &#32;     settings<span class="operator">.</span>setArrayIndex(i);
    /// &#32;     settings<span class="operator">.</span>setValue(<span class="string">"userName"</span><span class="operator">,</span> list<span class="operator">.</span>at(i)<span class="operator">.</span>userName);
    /// &#32;     settings<span class="operator">.</span>setValue(<span class="string">"password"</span><span class="operator">,</span> list<span class="operator">.</span>at(i)<span class="operator">.</span>password);
    ///   }
    ///   settings<span class="operator">.</span>endArray();
    ///
    /// </pre>
    /// <p>The generated keys will have the form</p>
    /// <ul>
    /// <li><code>logins/size</code></li>
    /// <li><code>logins/1/userName</code></li>
    /// <li><code>logins/1/password</code></li>
    /// <li><code>logins/2/userName</code></li>
    /// <li><code>logins/2/password</code></li>
    /// <li><code>logins/3/userName</code></li>
    /// <li><code>logins/3/password</code></li>
    /// <li>...</li>
    /// </ul>
    /// <p>To read back an array, use <a href="http://doc.qt.io/qt-5/qsettings.html#beginReadArray">beginReadArray</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#beginReadArray">beginReadArray</a>(), <a href="http://doc.qt.io/qt-5/qsettings.html#endArray">endArray</a>(), and <a href="http://doc.qt.io/qt-5/qsettings.html#setArrayIndex">setArrayIndex</a>().</p></div>
    pub unsafe fn begin_write_array_2a(
        &mut self,
        prefix: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        size: ::std::os::raw::c_int,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSettings_beginWriteArray(
            self as *mut crate::QSettings,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(prefix)
                .as_raw_ptr(),
            size,
        )
    }

    /// <p>Adds <i>prefix</i> to the current group and starts writing an array of size <i>size</i>. If <i>size</i> is -1 (the default), it is automatically determined based on the indexes of the entries written.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSettings::beginWriteArray(const QString& prefix)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#beginWriteArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds <i>prefix</i> to the current group and starts writing an array of size <i>size</i>. If <i>size</i> is -1 (the default), it is automatically determined based on the indexes of the entries written.</p>
    /// <p>If you have many occurrences of a certain set of keys, you can use arrays to make your life easier. For example, let's suppose that you want to save a variable-length list of user names and passwords. You could then write:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">struct</span> Login {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> userName;
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> password;
    ///   };
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span>Login<span class="operator">&gt;</span> logins;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsettings.html#QSettings">QSettings</a></span> settings;
    ///   settings<span class="operator">.</span>beginWriteArray(<span class="string">"logins"</span>);
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> logins<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
    /// &#32;     settings<span class="operator">.</span>setArrayIndex(i);
    /// &#32;     settings<span class="operator">.</span>setValue(<span class="string">"userName"</span><span class="operator">,</span> list<span class="operator">.</span>at(i)<span class="operator">.</span>userName);
    /// &#32;     settings<span class="operator">.</span>setValue(<span class="string">"password"</span><span class="operator">,</span> list<span class="operator">.</span>at(i)<span class="operator">.</span>password);
    ///   }
    ///   settings<span class="operator">.</span>endArray();
    ///
    /// </pre>
    /// <p>The generated keys will have the form</p>
    /// <ul>
    /// <li><code>logins/size</code></li>
    /// <li><code>logins/1/userName</code></li>
    /// <li><code>logins/1/password</code></li>
    /// <li><code>logins/2/userName</code></li>
    /// <li><code>logins/2/password</code></li>
    /// <li><code>logins/3/userName</code></li>
    /// <li><code>logins/3/password</code></li>
    /// <li>...</li>
    /// </ul>
    /// <p>To read back an array, use <a href="http://doc.qt.io/qt-5/qsettings.html#beginReadArray">beginReadArray</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#beginReadArray">beginReadArray</a>(), <a href="http://doc.qt.io/qt-5/qsettings.html#endArray">endArray</a>(), and <a href="http://doc.qt.io/qt-5/qsettings.html#setArrayIndex">setArrayIndex</a>().</p></div>
    pub unsafe fn begin_write_array_1a(
        &mut self,
        prefix: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSettings_beginWriteArray1(
            self as *mut crate::QSettings,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(prefix)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a list of all key top-level groups that contain keys that can be read using the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QSettings::childGroups() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#childGroups">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of all key top-level groups that contain keys that can be read using the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsettings.html#QSettings">QSettings</a></span> settings;
    ///   settings<span class="operator">.</span>setValue(<span class="string">"fridge/color"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcolor.html">QColor</a></span>(<span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>white));
    ///   settings<span class="operator">.</span>setValue(<span class="string">"fridge/size"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsize.html">QSize</a></span>(<span class="number">32</span><span class="operator">,</span> <span class="number">96</span>));
    ///   settings<span class="operator">.</span>setValue(<span class="string">"sofa"</span><span class="operator">,</span> <span class="keyword">true</span>);
    ///   settings<span class="operator">.</span>setValue(<span class="string">"tv"</span><span class="operator">,</span> <span class="keyword">false</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> groups <span class="operator">=</span> settings<span class="operator">.</span>childGroups();
    ///   <span class="comment">// groups: ["fridge"]</span>
    ///
    /// </pre>
    /// <p>If a group is set using <a href="http://doc.qt.io/qt-5/qsettings.html#beginGroup">beginGroup</a>(), the first-level keys in that group are returned, without the group prefix.</p>
    /// <pre class="cpp">
    ///
    ///   settings<span class="operator">.</span>beginGroup(<span class="string">"fridge"</span>);
    ///   groups <span class="operator">=</span> settings<span class="operator">.</span>childGroups();
    ///   <span class="comment">// groups: []</span>
    ///
    /// </pre>
    /// <p>You can navigate through the entire setting hierarchy using <a href="http://doc.qt.io/qt-5/qsettings.html#childKeys">childKeys</a>() and childGroups() recursively.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#childKeys">childKeys</a>() and <a href="http://doc.qt.io/qt-5/qsettings.html#allKeys">allKeys</a>().</p></div>
    pub unsafe fn child_groups(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QSettings_childGroups(self as *const crate::QSettings);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list of all top-level keys that can be read using the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QSettings::childKeys() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#childKeys">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of all top-level keys that can be read using the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsettings.html#QSettings">QSettings</a></span> settings;
    ///   settings<span class="operator">.</span>setValue(<span class="string">"fridge/color"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcolor.html">QColor</a></span>(<span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>white));
    ///   settings<span class="operator">.</span>setValue(<span class="string">"fridge/size"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsize.html">QSize</a></span>(<span class="number">32</span><span class="operator">,</span> <span class="number">96</span>));
    ///   settings<span class="operator">.</span>setValue(<span class="string">"sofa"</span><span class="operator">,</span> <span class="keyword">true</span>);
    ///   settings<span class="operator">.</span>setValue(<span class="string">"tv"</span><span class="operator">,</span> <span class="keyword">false</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> keys <span class="operator">=</span> settings<span class="operator">.</span>childKeys();
    ///   <span class="comment">// keys: ["sofa", "tv"]</span>
    ///
    /// </pre>
    /// <p>If a group is set using <a href="http://doc.qt.io/qt-5/qsettings.html#beginGroup">beginGroup</a>(), the top-level keys in that group are returned, without the group prefix:</p>
    /// <pre class="cpp">
    ///
    ///   settings<span class="operator">.</span>beginGroup(<span class="string">"fridge"</span>);
    ///   keys <span class="operator">=</span> settings<span class="operator">.</span>childKeys();
    ///   <span class="comment">// keys: ["color", "size"]</span>
    ///
    /// </pre>
    /// <p>You can navigate through the entire setting hierarchy using childKeys() and <a href="http://doc.qt.io/qt-5/qsettings.html#childGroups">childGroups</a>() recursively.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#childGroups">childGroups</a>() and <a href="http://doc.qt.io/qt-5/qsettings.html#allKeys">allKeys</a>().</p></div>
    pub unsafe fn child_keys(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QSettings_childKeys(self as *const crate::QSettings);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all entries in the primary location associated to this <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSettings::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all entries in the primary location associated to this <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object.</p>
    /// <p>Entries in fallback locations are not removed.</p>
    /// <p>If you only want to remove the entries in the current <a href="http://doc.qt.io/qt-5/qsettings.html#group">group</a>(), use remove("") instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#remove">remove</a>() and <a href="http://doc.qt.io/qt-5/qsettings.html#setFallbacksEnabled">setFallbacksEnabled</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSettings_clear(self as *mut crate::QSettings)
    }

    /// <p>Returns <code>true</code> if there exists a setting called <i>key</i>; returns false otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSettings::contains(const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if there exists a setting called <i>key</i>; returns false otherwise.</p>
    /// <p>If a group is set using <a href="http://doc.qt.io/qt-5/qsettings.html#beginGroup">beginGroup</a>(), <i>key</i> is taken to be relative to that group.</p>
    /// <p>Note that the Windows registry and INI files use case-insensitive keys, whereas the CFPreferences API on <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS uses case-sensitive keys. To avoid portability problems, see the <a href="http://doc.qt.io/qt-5/qsettings.html#section-and-key-syntax">Section and Key Syntax</a> rules.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qsettings.html#setValue">setValue</a>().</p></div>
    pub unsafe fn contains(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSettings_contains(
            self as *const crate::QSettings,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        )
    }

    /// <p>Returns default file format used for storing settings for the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a>(<a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> *) constructor. If no default format is set, <a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">QSettings::NativeFormat</a> is used.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QSettings::Format QSettings::defaultFormat()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#defaultFormat">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns default file format used for storing settings for the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a>(<a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> *) constructor. If no default format is set, <a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">QSettings::NativeFormat</a> is used.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#setDefaultFormat">setDefaultFormat</a>() and <a href="http://doc.qt.io/qt-5/qsettings.html#format">format</a>().</p></div>
    pub unsafe fn default_format() -> crate::q_settings::Format {
        crate::__ffi::ctr_qt_core_ffi_QSettings_defaultFormat()
    }

    /// <p>Closes the array that was started using <a href="http://doc.qt.io/qt-5/qsettings.html#beginReadArray">beginReadArray</a>() or <a href="http://doc.qt.io/qt-5/qsettings.html#beginWriteArray">beginWriteArray</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSettings::endArray()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#endArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Closes the array that was started using <a href="http://doc.qt.io/qt-5/qsettings.html#beginReadArray">beginReadArray</a>() or <a href="http://doc.qt.io/qt-5/qsettings.html#beginWriteArray">beginWriteArray</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#beginReadArray">beginReadArray</a>() and <a href="http://doc.qt.io/qt-5/qsettings.html#beginWriteArray">beginWriteArray</a>().</p></div>
    pub unsafe fn end_array(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSettings_endArray(self as *mut crate::QSettings)
    }

    /// <p>Resets the group to what it was before the corresponding <a href="http://doc.qt.io/qt-5/qsettings.html#beginGroup">beginGroup</a>() call.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSettings::endGroup()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#endGroup">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Resets the group to what it was before the corresponding <a href="http://doc.qt.io/qt-5/qsettings.html#beginGroup">beginGroup</a>() call.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   settings<span class="operator">.</span>beginGroup(<span class="string">"alpha"</span>);
    ///   <span class="comment">// settings.group() == "alpha"</span>
    ///
    ///   settings<span class="operator">.</span>beginGroup(<span class="string">"beta"</span>);
    ///   <span class="comment">// settings.group() == "alpha/beta"</span>
    ///
    ///   settings<span class="operator">.</span>endGroup();
    ///   <span class="comment">// settings.group() == "alpha"</span>
    ///
    ///   settings<span class="operator">.</span>endGroup();
    ///   <span class="comment">// settings.group() == ""</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#beginGroup">beginGroup</a>() and <a href="http://doc.qt.io/qt-5/qsettings.html#group">group</a>().</p></div>
    pub unsafe fn end_group(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSettings_endGroup(self as *mut crate::QSettings)
    }

    /// <p>Returns <code>true</code> if fallbacks are enabled; returns <code>false</code> otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSettings::fallbacksEnabled() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#fallbacksEnabled">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if fallbacks are enabled; returns <code>false</code> otherwise.</p>
    /// <p>By default, fallbacks are enabled.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#setFallbacksEnabled">setFallbacksEnabled</a>().</p></div>
    pub unsafe fn fallbacks_enabled(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSettings_fallbacksEnabled(self as *const crate::QSettings)
    }

    /// <p>Returns the path where settings written using this <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object are stored.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QSettings::fileName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#fileName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the path where settings written using this <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object are stored.</p>
    /// <p>On Windows, if the format is <a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">QSettings::NativeFormat</a>, the return value is a system registry path, not a file path.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#isWritable">isWritable</a>() and <a href="http://doc.qt.io/qt-5/qsettings.html#format">format</a>().</p></div>
    pub unsafe fn file_name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QSettings_fileName(self as *const crate::QSettings);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the format used for storing the settings.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSettings::Format QSettings::format() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#format">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the format used for storing the settings.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#defaultFormat">defaultFormat</a>(), <a href="http://doc.qt.io/qt-5/qsettings.html#fileName">fileName</a>(), <a href="http://doc.qt.io/qt-5/qsettings.html#scope">scope</a>(), <a href="http://doc.qt.io/qt-5/qsettings.html#organizationName">organizationName</a>(), and <a href="http://doc.qt.io/qt-5/qsettings.html#applicationName">applicationName</a>().</p></div>
    pub unsafe fn format(&self) -> crate::q_settings::Format {
        crate::__ffi::ctr_qt_core_ffi_QSettings_format(self as *const crate::QSettings)
    }

    /// <p>Returns the current group.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QSettings::group() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#group">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current group.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#beginGroup">beginGroup</a>() and <a href="http://doc.qt.io/qt-5/qsettings.html#endGroup">endGroup</a>().</p></div>
    pub unsafe fn group(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QSettings_group(self as *const crate::QSettings);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the codec that is used for accessing INI files. By default, no codec is used, so a null pointer is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextCodec* QSettings::iniCodec() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#iniCodec">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the codec that is used for accessing INI files. By default, no codec is used, so a null pointer is returned.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#setIniCodec">setIniCodec</a>().</p></div>
    pub unsafe fn ini_codec(&self) -> ::cpp_utils::MutPtr<crate::QTextCodec> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QSettings_iniCodec(self as *const crate::QSettings);
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> is only allowed to perform atomic saving and reloading (synchronization) of the settings. Returns <code>false</code> if it is allowed to save the settings contents directly to the configuration file.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSettings::isAtomicSyncRequired() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#isAtomicSyncRequired">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> is only allowed to perform atomic saving and reloading (synchronization) of the settings. Returns <code>false</code> if it is allowed to save the settings contents directly to the configuration file.</p>
    /// <p>The default is <code>true</code>.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#setAtomicSyncRequired">setAtomicSyncRequired</a>() and <a href="http://doc.qt.io/qt-5/qsavefile.html">QSaveFile</a>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn is_atomic_sync_required(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSettings_isAtomicSyncRequired(
            self as *const crate::QSettings,
        )
    }

    /// <p>Returns <code>true</code> if settings can be written using this <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object; returns <code>false</code> otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSettings::isWritable() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#isWritable">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if settings can be written using this <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object; returns <code>false</code> otherwise.</p>
    /// <p>One reason why isWritable() might return false is if <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> operates on a read-only file.</p>
    /// <p><b>Warning:</b> This function is not perfectly reliable, because the file permissions can change at any time.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#fileName">fileName</a>(), <a href="http://doc.qt.io/qt-5/qsettings.html#status">status</a>(), and <a href="http://doc.qt.io/qt-5/qsettings.html#sync">sync</a>().</p></div>
    pub unsafe fn is_writable(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSettings_isWritable(self as *const crate::QSettings)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QSettings::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QSettings_metaObject(self as *const crate::QSettings);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object for accessing settings of the application called <i>application</i> from the organization called <i>organization</i>, and with parent <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSettings::QSettings(const QString& organization, const QString& application = …, QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#QSettings">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object for accessing settings of the application called <i>application</i> from the organization called <i>organization</i>, and with parent <i>parent</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">QSettings</span> settings(<span class="string">"Moose Tech"</span><span class="operator">,</span> <span class="string">"Facturo-Pro"</span>);
    ///
    /// </pre>
    /// <p>The scope is set to <a href="http://doc.qt.io/qt-5/qsettings.html#Scope-enum">QSettings::UserScope</a>, and the format is set to <a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">QSettings::NativeFormat</a> (i.e. calling <a href="http://doc.qt.io/qt-5/qsettings.html#setDefaultFormat">setDefaultFormat</a>() before calling this constructor has no effect).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#setDefaultFormat">setDefaultFormat</a>() and <a href="http://doc.qt.io/qt-5/qsettings.html#fallback-mechanism">Fallback Mechanism</a>.</p></div>
    pub unsafe fn from_2_q_string_q_object(
        organization: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        application: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QSettings> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSettings_QSettings(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(organization)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(application)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object for accessing settings of the application called <i>application</i> from the organization called <i>organization</i>, and with parent <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSettings::QSettings(QSettings::Scope scope, const QString& organization, const QString& application = …, QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#QSettings-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object for accessing settings of the application called <i>application</i> from the organization called <i>organization</i>, and with parent <i>parent</i>.</p>
    /// <p>If <i>scope</i> is <a href="http://doc.qt.io/qt-5/qsettings.html#Scope-enum">QSettings::UserScope</a>, the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object searches user-specific settings first, before it searches system-wide settings as a fallback. If <i>scope</i> is <a href="http://doc.qt.io/qt-5/qsettings.html#Scope-enum">QSettings::SystemScope</a>, the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object ignores user-specific settings and provides access to system-wide settings.</p>
    /// <p>The storage format is set to <a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">QSettings::NativeFormat</a> (i.e. calling <a href="http://doc.qt.io/qt-5/qsettings.html#setDefaultFormat">setDefaultFormat</a>() before calling this constructor has no effect).</p>
    /// <p>If no application name is given, the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object will only access the organization-wide <a href="http://doc.qt.io/qt-5/qsettings.html#fallback-mechanism">locations</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#setDefaultFormat">setDefaultFormat</a>().</p></div>
    pub unsafe fn from_scope2_q_string_q_object(
        scope: crate::q_settings::Scope,
        organization: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        application: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QSettings> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSettings_QSettings1(
            scope,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(organization)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(application)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object for accessing settings of the application called <i>application</i> from the organization called <i>organization</i>, and with parent <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSettings::QSettings(QSettings::Format format, QSettings::Scope scope, const QString& organization, const QString& application = …, QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#QSettings-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object for accessing settings of the application called <i>application</i> from the organization called <i>organization</i>, and with parent <i>parent</i>.</p>
    /// <p>If <i>scope</i> is <a href="http://doc.qt.io/qt-5/qsettings.html#Scope-enum">QSettings::UserScope</a>, the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object searches user-specific settings first, before it searches system-wide settings as a fallback. If <i>scope</i> is <a href="http://doc.qt.io/qt-5/qsettings.html#Scope-enum">QSettings::SystemScope</a>, the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object ignores user-specific settings and provides access to system-wide settings.</p>
    /// <p>If <i>format</i> is <a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">QSettings::NativeFormat</a>, the native API is used for storing settings. If <i>format</i> is <a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">QSettings::IniFormat</a>, the INI format is used.</p>
    /// <p>If no application name is given, the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object will only access the organization-wide <a href="http://doc.qt.io/qt-5/qsettings.html#fallback-mechanism">locations</a>.</p></div>
    pub unsafe fn from_format_scope2_q_string_q_object(
        format: crate::q_settings::Format,
        scope: crate::q_settings::Scope,
        organization: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        application: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QSettings> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSettings_QSettings2(
            format,
            scope,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(organization)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(application)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object for accessing the settings stored in the file called <i>fileName</i>, with parent <i>parent</i>. If the file doesn't already exist, it is created.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSettings::QSettings(const QString& fileName, QSettings::Format format, QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#QSettings-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object for accessing the settings stored in the file called <i>fileName</i>, with parent <i>parent</i>. If the file doesn't already exist, it is created.</p>
    /// <p>If <i>format</i> is <a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">QSettings::NativeFormat</a>, the meaning of <i>fileName</i> depends on the platform. On Unix, <i>fileName</i> is the name of an INI file. On <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS, <i>fileName</i> is the name of a <code>.plist</code> file. On Windows, <i>fileName</i> is a path in the system registry.</p>
    /// <p>If <i>format</i> is <a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">QSettings::IniFormat</a>, <i>fileName</i> is the name of an INI file.</p>
    /// <p><b>Warning:</b> This function is provided for convenience. It works well for accessing INI or <code>.plist</code> files generated by Qt, but might fail on some syntaxes found in such files originated by other programs. In particular, be aware of the following limitations:</p>
    /// <ul>
    /// <li><a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> provides no way of reading INI "path" entries, i.e., entries with unescaped slash characters. (This is because these entries are ambiguous and cannot be resolved automatically.)</li>
    /// <li>In INI files, <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> uses the <code>@</code> character as a metacharacter in some contexts, to encode Qt-specific data types (e.g., <code>@Rect</code>), and might therefore misinterpret it when it occurs in pure INI files.</li>
    /// </ul>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#fileName">fileName</a>().</p></div>
    pub unsafe fn from_q_string_format_q_object(
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        format: crate::q_settings::Format,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QSettings> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSettings_QSettings3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
            format,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object for accessing settings of the application and organization set previously with a call to <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationName-prop">QCoreApplication::setOrganizationName</a>(), <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationDomain-prop">QCoreApplication::setOrganizationDomain</a>(), and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationName-prop">QCoreApplication::setApplicationName</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSettings::QSettings(QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#QSettings-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object for accessing settings of the application and organization set previously with a call to <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationName-prop">QCoreApplication::setOrganizationName</a>(), <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationDomain-prop">QCoreApplication::setOrganizationDomain</a>(), and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationName-prop">QCoreApplication::setApplicationName</a>().</p>
    /// <p>The scope is <a href="http://doc.qt.io/qt-5/qsettings.html#Scope-enum">QSettings::UserScope</a> and the format is <a href="http://doc.qt.io/qt-5/qsettings.html#defaultFormat">defaultFormat</a>() (<a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">QSettings::NativeFormat</a> by default). Use <a href="http://doc.qt.io/qt-5/qsettings.html#setDefaultFormat">setDefaultFormat</a>() before calling this constructor to change the default format used by this constructor.</p>
    /// <p>The code</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsettings.html#QSettings">QSettings</a></span> settings(<span class="string">"Moose Soft"</span><span class="operator">,</span> <span class="string">"Facturo-Pro"</span>);
    ///
    /// </pre>
    /// <p>is equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>setOrganizationName(<span class="string">"Moose Soft"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>setApplicationName(<span class="string">"Facturo-Pro"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsettings.html#QSettings">QSettings</a></span> settings;
    ///
    /// </pre>
    /// <p>If <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationName-prop">QCoreApplication::setOrganizationName</a>() and <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationName-prop">QCoreApplication::setApplicationName</a>() has not been previously called, the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object will not be able to read or write any settings, and <a href="http://doc.qt.io/qt-5/qsettings.html#status">status</a>() will return <a href="http://doc.qt.io/qt-5/qsettings.html#Status-enum">AccessError</a>.</p>
    /// <p>On <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS, if both a name and an Internet domain are specified for the organization, the domain is preferred over the name. On other platforms, the name is preferred over the domain.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationName-prop">QCoreApplication::setOrganizationName</a>(), <a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationDomain-prop">QCoreApplication::setOrganizationDomain</a>(), <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationName-prop">QCoreApplication::setApplicationName</a>(), and <a href="http://doc.qt.io/qt-5/qsettings.html#setDefaultFormat">setDefaultFormat</a>().</p></div>
    pub unsafe fn from_q_object(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QSettings> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSettings_QSettings4(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> class provides persistent platform-independent application settings.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSettings::QSettings()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> class provides persistent platform-independent application settings.</p>
    /// <p>Users normally expect an application to remember its settings (window sizes and positions, options, etc.) across sessions. This information is often stored in the system registry on Windows, and in property list files on <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS. On Unix systems, in the absence of a standard, many applications (including the KDE applications) use INI text files.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> is an abstraction around these technologies, enabling you to save and restore application settings in a portable manner. It also supports <a href="http://doc.qt.io/qt-5/qsettings.html#registerFormat">custom storage formats</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a>'s API is based on <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>, allowing you to save most value-based types, such as <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a>, and <a href="http://doc.qt.io/qt-5/qimage.html">QImage</a>, with the minimum of effort.</p>
    /// <p>If all you need is a non-persistent memory-based structure, consider using <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>&gt; instead.</p>
    /// <a name="basic-usage"></a></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QSettings> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSettings_QSettings5();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object for accessing settings of the application called <i>application</i> from the organization called <i>organization</i>, and with parent <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSettings::QSettings(const QString& organization, const QString& application = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#QSettings">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object for accessing settings of the application called <i>application</i> from the organization called <i>organization</i>, and with parent <i>parent</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">QSettings</span> settings(<span class="string">"Moose Tech"</span><span class="operator">,</span> <span class="string">"Facturo-Pro"</span>);
    ///
    /// </pre>
    /// <p>The scope is set to <a href="http://doc.qt.io/qt-5/qsettings.html#Scope-enum">QSettings::UserScope</a>, and the format is set to <a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">QSettings::NativeFormat</a> (i.e. calling <a href="http://doc.qt.io/qt-5/qsettings.html#setDefaultFormat">setDefaultFormat</a>() before calling this constructor has no effect).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#setDefaultFormat">setDefaultFormat</a>() and <a href="http://doc.qt.io/qt-5/qsettings.html#fallback-mechanism">Fallback Mechanism</a>.</p></div>
    pub unsafe fn from_2_q_string(
        organization: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        application: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QSettings> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSettings_QSettings6(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(organization)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(application)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object for accessing settings of the application called <i>application</i> from the organization called <i>organization</i>, and with parent <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSettings::QSettings(const QString& organization)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#QSettings">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object for accessing settings of the application called <i>application</i> from the organization called <i>organization</i>, and with parent <i>parent</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">QSettings</span> settings(<span class="string">"Moose Tech"</span><span class="operator">,</span> <span class="string">"Facturo-Pro"</span>);
    ///
    /// </pre>
    /// <p>The scope is set to <a href="http://doc.qt.io/qt-5/qsettings.html#Scope-enum">QSettings::UserScope</a>, and the format is set to <a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">QSettings::NativeFormat</a> (i.e. calling <a href="http://doc.qt.io/qt-5/qsettings.html#setDefaultFormat">setDefaultFormat</a>() before calling this constructor has no effect).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#setDefaultFormat">setDefaultFormat</a>() and <a href="http://doc.qt.io/qt-5/qsettings.html#fallback-mechanism">Fallback Mechanism</a>.</p></div>
    pub unsafe fn from_q_string(
        organization: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QSettings> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSettings_QSettings7(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(organization)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object for accessing settings of the application called <i>application</i> from the organization called <i>organization</i>, and with parent <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSettings::QSettings(QSettings::Scope scope, const QString& organization, const QString& application = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#QSettings-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object for accessing settings of the application called <i>application</i> from the organization called <i>organization</i>, and with parent <i>parent</i>.</p>
    /// <p>If <i>scope</i> is <a href="http://doc.qt.io/qt-5/qsettings.html#Scope-enum">QSettings::UserScope</a>, the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object searches user-specific settings first, before it searches system-wide settings as a fallback. If <i>scope</i> is <a href="http://doc.qt.io/qt-5/qsettings.html#Scope-enum">QSettings::SystemScope</a>, the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object ignores user-specific settings and provides access to system-wide settings.</p>
    /// <p>The storage format is set to <a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">QSettings::NativeFormat</a> (i.e. calling <a href="http://doc.qt.io/qt-5/qsettings.html#setDefaultFormat">setDefaultFormat</a>() before calling this constructor has no effect).</p>
    /// <p>If no application name is given, the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object will only access the organization-wide <a href="http://doc.qt.io/qt-5/qsettings.html#fallback-mechanism">locations</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#setDefaultFormat">setDefaultFormat</a>().</p></div>
    pub unsafe fn from_scope2_q_string(
        scope: crate::q_settings::Scope,
        organization: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        application: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QSettings> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSettings_QSettings8(
            scope,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(organization)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(application)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object for accessing settings of the application called <i>application</i> from the organization called <i>organization</i>, and with parent <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSettings::QSettings(QSettings::Scope scope, const QString& organization)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#QSettings-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object for accessing settings of the application called <i>application</i> from the organization called <i>organization</i>, and with parent <i>parent</i>.</p>
    /// <p>If <i>scope</i> is <a href="http://doc.qt.io/qt-5/qsettings.html#Scope-enum">QSettings::UserScope</a>, the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object searches user-specific settings first, before it searches system-wide settings as a fallback. If <i>scope</i> is <a href="http://doc.qt.io/qt-5/qsettings.html#Scope-enum">QSettings::SystemScope</a>, the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object ignores user-specific settings and provides access to system-wide settings.</p>
    /// <p>The storage format is set to <a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">QSettings::NativeFormat</a> (i.e. calling <a href="http://doc.qt.io/qt-5/qsettings.html#setDefaultFormat">setDefaultFormat</a>() before calling this constructor has no effect).</p>
    /// <p>If no application name is given, the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object will only access the organization-wide <a href="http://doc.qt.io/qt-5/qsettings.html#fallback-mechanism">locations</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#setDefaultFormat">setDefaultFormat</a>().</p></div>
    pub unsafe fn from_scope_q_string(
        scope: crate::q_settings::Scope,
        organization: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QSettings> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSettings_QSettings9(
            scope,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(organization)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object for accessing settings of the application called <i>application</i> from the organization called <i>organization</i>, and with parent <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSettings::QSettings(QSettings::Format format, QSettings::Scope scope, const QString& organization, const QString& application = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#QSettings-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object for accessing settings of the application called <i>application</i> from the organization called <i>organization</i>, and with parent <i>parent</i>.</p>
    /// <p>If <i>scope</i> is <a href="http://doc.qt.io/qt-5/qsettings.html#Scope-enum">QSettings::UserScope</a>, the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object searches user-specific settings first, before it searches system-wide settings as a fallback. If <i>scope</i> is <a href="http://doc.qt.io/qt-5/qsettings.html#Scope-enum">QSettings::SystemScope</a>, the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object ignores user-specific settings and provides access to system-wide settings.</p>
    /// <p>If <i>format</i> is <a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">QSettings::NativeFormat</a>, the native API is used for storing settings. If <i>format</i> is <a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">QSettings::IniFormat</a>, the INI format is used.</p>
    /// <p>If no application name is given, the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object will only access the organization-wide <a href="http://doc.qt.io/qt-5/qsettings.html#fallback-mechanism">locations</a>.</p></div>
    pub unsafe fn from_format_scope2_q_string(
        format: crate::q_settings::Format,
        scope: crate::q_settings::Scope,
        organization: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        application: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QSettings> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSettings_QSettings10(
            format,
            scope,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(organization)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(application)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object for accessing settings of the application called <i>application</i> from the organization called <i>organization</i>, and with parent <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSettings::QSettings(QSettings::Format format, QSettings::Scope scope, const QString& organization)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#QSettings-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object for accessing settings of the application called <i>application</i> from the organization called <i>organization</i>, and with parent <i>parent</i>.</p>
    /// <p>If <i>scope</i> is <a href="http://doc.qt.io/qt-5/qsettings.html#Scope-enum">QSettings::UserScope</a>, the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object searches user-specific settings first, before it searches system-wide settings as a fallback. If <i>scope</i> is <a href="http://doc.qt.io/qt-5/qsettings.html#Scope-enum">QSettings::SystemScope</a>, the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object ignores user-specific settings and provides access to system-wide settings.</p>
    /// <p>If <i>format</i> is <a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">QSettings::NativeFormat</a>, the native API is used for storing settings. If <i>format</i> is <a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">QSettings::IniFormat</a>, the INI format is used.</p>
    /// <p>If no application name is given, the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object will only access the organization-wide <a href="http://doc.qt.io/qt-5/qsettings.html#fallback-mechanism">locations</a>.</p></div>
    pub unsafe fn from_format_scope_q_string(
        format: crate::q_settings::Format,
        scope: crate::q_settings::Scope,
        organization: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QSettings> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSettings_QSettings11(
            format,
            scope,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(organization)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object for accessing the settings stored in the file called <i>fileName</i>, with parent <i>parent</i>. If the file doesn't already exist, it is created.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSettings::QSettings(const QString& fileName, QSettings::Format format)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#QSettings-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object for accessing the settings stored in the file called <i>fileName</i>, with parent <i>parent</i>. If the file doesn't already exist, it is created.</p>
    /// <p>If <i>format</i> is <a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">QSettings::NativeFormat</a>, the meaning of <i>fileName</i> depends on the platform. On Unix, <i>fileName</i> is the name of an INI file. On <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS, <i>fileName</i> is the name of a <code>.plist</code> file. On Windows, <i>fileName</i> is a path in the system registry.</p>
    /// <p>If <i>format</i> is <a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">QSettings::IniFormat</a>, <i>fileName</i> is the name of an INI file.</p>
    /// <p><b>Warning:</b> This function is provided for convenience. It works well for accessing INI or <code>.plist</code> files generated by Qt, but might fail on some syntaxes found in such files originated by other programs. In particular, be aware of the following limitations:</p>
    /// <ul>
    /// <li><a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> provides no way of reading INI "path" entries, i.e., entries with unescaped slash characters. (This is because these entries are ambiguous and cannot be resolved automatically.)</li>
    /// <li>In INI files, <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> uses the <code>@</code> character as a metacharacter in some contexts, to encode Qt-specific data types (e.g., <code>@Rect</code>), and might therefore misinterpret it when it occurs in pure INI files.</li>
    /// </ul>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#fileName">fileName</a>().</p></div>
    pub unsafe fn from_q_string_format(
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        format: crate::q_settings::Format,
    ) -> ::cpp_utils::CppBox<crate::QSettings> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSettings_QSettings12(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
            format,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object in the same way as <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a>(<a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> *parent) but with the given <i>scope</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSettings::QSettings(QSettings::Scope scope, QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#QSettings-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object in the same way as <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a>(<a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> *parent) but with the given <i>scope</i>.</p>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#QSettings-4">QSettings</a>(QObject *parent).</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn from_scope_q_object(
        scope: crate::q_settings::Scope,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QSettings> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSettings_QSettings26(
            scope,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object in the same way as <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a>(<a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> *parent) but with the given <i>scope</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSettings::QSettings(QSettings::Scope scope)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#QSettings-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object in the same way as <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a>(<a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> *parent) but with the given <i>scope</i>.</p>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#QSettings-4">QSettings</a>(QObject *parent).</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn from_scope(
        scope: crate::q_settings::Scope,
    ) -> ::cpp_utils::CppBox<crate::QSettings> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSettings_QSettings27(scope);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the organization name used for storing the settings.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QSettings::organizationName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#organizationName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the organization name used for storing the settings.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#organizationName-prop">QCoreApplication::organizationName</a>(), <a href="http://doc.qt.io/qt-5/qsettings.html#format">format</a>(), <a href="http://doc.qt.io/qt-5/qsettings.html#scope">scope</a>(), and <a href="http://doc.qt.io/qt-5/qsettings.html#applicationName">applicationName</a>().</p></div>
    pub unsafe fn organization_name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSettings_organizationName(
            self as *const crate::QSettings,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QSettings::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QSettings_qt_metacall(
            self as *mut crate::QSettings,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QSettings::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSettings_qt_metacast(
            self as *mut crate::QSettings,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Removes the setting <i>key</i> and any sub-settings of <i>key</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSettings::remove(const QString& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#remove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the setting <i>key</i> and any sub-settings of <i>key</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsettings.html#QSettings">QSettings</a></span> settings;
    ///   settings<span class="operator">.</span>setValue(<span class="string">"ape"</span>);
    ///   settings<span class="operator">.</span>setValue(<span class="string">"monkey"</span><span class="operator">,</span> <span class="number">1</span>);
    ///   settings<span class="operator">.</span>setValue(<span class="string">"monkey/sea"</span><span class="operator">,</span> <span class="number">2</span>);
    ///   settings<span class="operator">.</span>setValue(<span class="string">"monkey/doe"</span><span class="operator">,</span> <span class="number">4</span>);
    ///
    ///   settings<span class="operator">.</span>remove(<span class="string">"monkey"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> keys <span class="operator">=</span> settings<span class="operator">.</span>allKeys();
    ///   <span class="comment">// keys: ["ape"]</span>
    ///
    /// </pre>
    /// <p>Be aware that if one of the fallback locations contains a setting with the same key, that setting will be visible after calling remove().</p>
    /// <p>If <i>key</i> is an empty string, all keys in the current <a href="http://doc.qt.io/qt-5/qsettings.html#group">group</a>() are removed. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsettings.html#QSettings">QSettings</a></span> settings;
    ///   settings<span class="operator">.</span>setValue(<span class="string">"ape"</span>);
    ///   settings<span class="operator">.</span>setValue(<span class="string">"monkey"</span><span class="operator">,</span> <span class="number">1</span>);
    ///   settings<span class="operator">.</span>setValue(<span class="string">"monkey/sea"</span><span class="operator">,</span> <span class="number">2</span>);
    ///   settings<span class="operator">.</span>setValue(<span class="string">"monkey/doe"</span><span class="operator">,</span> <span class="number">4</span>);
    ///
    ///   settings<span class="operator">.</span>beginGroup(<span class="string">"monkey"</span>);
    ///   settings<span class="operator">.</span>remove(<span class="string">""</span>);
    ///   settings<span class="operator">.</span>endGroup();
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> keys <span class="operator">=</span> settings<span class="operator">.</span>allKeys();
    ///   <span class="comment">// keys: ["ape"]</span>
    ///
    /// </pre>
    /// <p>Note that the Windows registry and INI files use case-insensitive keys, whereas the CFPreferences API on <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS uses case-sensitive keys. To avoid portability problems, see the <a href="http://doc.qt.io/qt-5/qsettings.html#section-and-key-syntax">Section and Key Syntax</a> rules.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#setValue">setValue</a>(), <a href="http://doc.qt.io/qt-5/qsettings.html#value">value</a>(), and <a href="http://doc.qt.io/qt-5/qsettings.html#contains">contains</a>().</p></div>
    pub unsafe fn remove(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSettings_remove(
            self as *mut crate::QSettings,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        )
    }

    /// <p>Returns the scope used for storing the settings.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSettings::Scope QSettings::scope() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#scope">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the scope used for storing the settings.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#format">format</a>(), <a href="http://doc.qt.io/qt-5/qsettings.html#organizationName">organizationName</a>(), and <a href="http://doc.qt.io/qt-5/qsettings.html#applicationName">applicationName</a>().</p></div>
    pub unsafe fn scope(&self) -> crate::q_settings::Scope {
        crate::__ffi::ctr_qt_core_ffi_QSettings_scope(self as *const crate::QSettings)
    }

    /// <p>Sets the current array index to <i>i</i>. Calls to functions such as <a href="http://doc.qt.io/qt-5/qsettings.html#setValue">setValue</a>(), <a href="http://doc.qt.io/qt-5/qsettings.html#value">value</a>(), <a href="http://doc.qt.io/qt-5/qsettings.html#remove">remove</a>(), and <a href="http://doc.qt.io/qt-5/qsettings.html#contains">contains</a>() will operate on the array entry at that index.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSettings::setArrayIndex(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#setArrayIndex">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the current array index to <i>i</i>. Calls to functions such as <a href="http://doc.qt.io/qt-5/qsettings.html#setValue">setValue</a>(), <a href="http://doc.qt.io/qt-5/qsettings.html#value">value</a>(), <a href="http://doc.qt.io/qt-5/qsettings.html#remove">remove</a>(), and <a href="http://doc.qt.io/qt-5/qsettings.html#contains">contains</a>() will operate on the array entry at that index.</p>
    /// <p>You must call <a href="http://doc.qt.io/qt-5/qsettings.html#beginReadArray">beginReadArray</a>() or <a href="http://doc.qt.io/qt-5/qsettings.html#beginWriteArray">beginWriteArray</a>() before you can call this function.</p></div>
    pub unsafe fn set_array_index(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QSettings_setArrayIndex(self as *mut crate::QSettings, i)
    }

    /// <p>Configures whether <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> is required to perform atomic saving and reloading (synchronization) of the settings. If the <i>enable</i> argument is <code>true</code> (the default), <a href="http://doc.qt.io/qt-5/qsettings.html#sync">sync</a>() will only perform synchronization operations that are atomic. If this is not possible, <a href="http://doc.qt.io/qt-5/qsettings.html#sync">sync</a>() will fail and <a href="http://doc.qt.io/qt-5/qsettings.html#status">status</a>() will be an error condition.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSettings::setAtomicSyncRequired(bool enable)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#setAtomicSyncRequired">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Configures whether <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> is required to perform atomic saving and reloading (synchronization) of the settings. If the <i>enable</i> argument is <code>true</code> (the default), <a href="http://doc.qt.io/qt-5/qsettings.html#sync">sync</a>() will only perform synchronization operations that are atomic. If this is not possible, <a href="http://doc.qt.io/qt-5/qsettings.html#sync">sync</a>() will fail and <a href="http://doc.qt.io/qt-5/qsettings.html#status">status</a>() will be an error condition.</p>
    /// <p>Setting this property to <code>false</code> will allow <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> to write directly to the configuration file and ignore any errors trying to lock it against other processes trying to write at the same time. Because of the potential for corruption, this option should be used with care, but is required in certain conditions, like a <a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">QSettings::IniFormat</a> configuration file that exists in an otherwise non-writeable directory or NTFS Alternate Data Streams.</p>
    /// <p>See <a href="http://doc.qt.io/qt-5/qsavefile.html">QSaveFile</a> for more information on the feature.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#isAtomicSyncRequired">isAtomicSyncRequired</a>() and <a href="http://doc.qt.io/qt-5/qsavefile.html">QSaveFile</a>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn set_atomic_sync_required(&mut self, enable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QSettings_setAtomicSyncRequired(
            self as *mut crate::QSettings,
            enable,
        )
    }

    /// <p>Sets the default file format to the given <i>format</i>, which is used for storing settings for the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a>(<a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> *) constructor.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QSettings::setDefaultFormat(QSettings::Format format)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#setDefaultFormat">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the default file format to the given <i>format</i>, which is used for storing settings for the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a>(<a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> *) constructor.</p>
    /// <p>If no default format is set, <a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">QSettings::NativeFormat</a> is used. See the documentation for the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> constructor you are using to see if that constructor will ignore this function.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#defaultFormat">defaultFormat</a>() and <a href="http://doc.qt.io/qt-5/qsettings.html#format">format</a>().</p></div>
    pub unsafe fn set_default_format(format: crate::q_settings::Format) {
        crate::__ffi::ctr_qt_core_ffi_QSettings_setDefaultFormat(format)
    }

    /// <p>Sets whether fallbacks are enabled to <i>b</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSettings::setFallbacksEnabled(bool b)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#setFallbacksEnabled">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets whether fallbacks are enabled to <i>b</i>.</p>
    /// <p>By default, fallbacks are enabled.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#fallbacksEnabled">fallbacksEnabled</a>().</p></div>
    pub unsafe fn set_fallbacks_enabled(&mut self, b: bool) {
        crate::__ffi::ctr_qt_core_ffi_QSettings_setFallbacksEnabled(
            self as *mut crate::QSettings,
            b,
        )
    }

    /// <p>Sets the codec for accessing INI files (including <code>.conf</code> files on Unix) to <i>codec</i>. The codec is used for decoding any data that is read from the INI file, and for encoding any data that is written to the file. By default, no codec is used, and non-ASCII characters are encoded using standard INI escape sequences.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSettings::setIniCodec(QTextCodec* codec)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#setIniCodec">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the codec for accessing INI files (including <code>.conf</code> files on Unix) to <i>codec</i>. The codec is used for decoding any data that is read from the INI file, and for encoding any data that is written to the file. By default, no codec is used, and non-ASCII characters are encoded using standard INI escape sequences.</p>
    /// <p><b>Warning:</b> The codec must be set immediately after creating the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object, before accessing any data.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#iniCodec">iniCodec</a>().</p></div>
    pub unsafe fn set_ini_codec_q_text_codec(
        &mut self,
        codec: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QTextCodec>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSettings_setIniCodec(
            self as *mut crate::QSettings,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QTextCodec>>::cast_into(codec)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSettings::setIniCodec(const char* codecName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#setIniCodec-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Sets the codec for accessing INI files (including <code>.conf</code> files on Unix) to the <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> for the encoding specified by <i>codecName</i>. Common values for <code>codecName</code> include "ISO 8859-1", "UTF-8", and "UTF-16". If the encoding isn't recognized, nothing happens.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForName">QTextCodec::codecForName</a>().</p></div>
    pub unsafe fn set_ini_codec_char(
        &mut self,
        codec_name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSettings_setIniCodec1(
            self as *mut crate::QSettings,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(
                codec_name,
            )
            .as_raw_ptr(),
        )
    }

    /// <p>Sets the path used for storing settings for the given <i>format</i> and <i>scope</i>, to <i>path</i>. The <i>format</i> can be a custom format.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QSettings::setPath(QSettings::Format format, QSettings::Scope scope, const QString& path)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#setPath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the path used for storing settings for the given <i>format</i> and <i>scope</i>, to <i>path</i>. The <i>format</i> can be a custom format.</p>
    /// <p>The table below summarizes the default values:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Platform</th><th>Format</th><th>Scope</th><th>Path</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td rowspan="2">Windows</td><td rowspan="2"><a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">IniFormat</a></td><td><a href="http://doc.qt.io/qt-5/qsettings.html#Scope-enum">UserScope</a></td><td><code>FOLDERID_RoamingAppData</code></td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qsettings.html#Scope-enum">SystemScope</a></td><td><code>FOLDERID_ProgramData</code></td></tr>
    /// <tr valign="top" class="odd"><td rowspan="2">Unix</td><td rowspan="2"><a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">NativeFormat</a>, <a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">IniFormat</a></td><td><a href="http://doc.qt.io/qt-5/qsettings.html#Scope-enum">UserScope</a></td><td><code>$HOME/.config</code></td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qsettings.html#Scope-enum">SystemScope</a></td><td><code>/etc/xdg</code></td></tr>
    /// <tr valign="top" class="odd"><td rowspan="2">Qt for Embedded Linux</td><td rowspan="2"><a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">NativeFormat</a>, <a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">IniFormat</a></td><td><a href="http://doc.qt.io/qt-5/qsettings.html#Scope-enum">UserScope</a></td><td><code>$HOME/Settings</code></td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qsettings.html#Scope-enum">SystemScope</a></td><td><code>/etc/xdg</code></td></tr>
    /// <tr valign="top" class="odd"><td rowspan="2"><a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS</td><td rowspan="2"><a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">IniFormat</a></td><td><a href="http://doc.qt.io/qt-5/qsettings.html#Scope-enum">UserScope</a></td><td><code>$HOME/.config</code></td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qsettings.html#Scope-enum">SystemScope</a></td><td><code>/etc/xdg</code></td></tr>
    /// </tbody></table></div>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qsettings.html#Scope-enum">UserScope</a> paths on Unix, <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, and iOS (<code>$HOME/.config</code> or $HOME/Settings) can be overridden by the user by setting the <code>XDG_CONFIG_HOME</code> environment variable. The default <a href="http://doc.qt.io/qt-5/qsettings.html#Scope-enum">SystemScope</a> paths on Unix, <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, and iOS (<code>/etc/xdg</code>) can be overridden when building the Qt library using the <code>configure</code> script's <code>-sysconfdir</code> flag (see <a href="http://doc.qt.io/qt-5/qlibraryinfo.html">QLibraryInfo</a> for details).</p>
    /// <p>Setting the <a href="http://doc.qt.io/qt-5/qsettings.html#Format-enum">NativeFormat</a> paths on Windows, <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, and iOS has no effect.</p>
    /// <p><b>Warning:</b> This function doesn't affect existing <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> objects.</p>
    /// <p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#registerFormat">registerFormat</a>().</p></div>
    pub unsafe fn set_path(
        format: crate::q_settings::Format,
        scope: crate::q_settings::Scope,
        path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSettings_setPath(
            format,
            scope,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(path).as_raw_ptr(),
        )
    }

    /// <p>Use <a href="http://doc.qt.io/qt-5/qsettings.html#setPath">setPath</a>() instead.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QSettings::setSystemIniPath(const QString& dir)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings-obsolete.html#setSystemIniPath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Use <a href="http://doc.qt.io/qt-5/qsettings.html#setPath">setPath</a>() instead.</p>
    /// <p>For example, if you have code like</p>
    /// <pre class="cpp plain">
    ///
    ///   setSystemIniPath(path);
    ///
    /// </pre>
    /// <p>you can rewrite it as</p>
    /// <pre class="cpp">
    ///
    ///   setPath(<span class="type"><a href="http://doc.qt.io/qt-5/qsettings.html#QSettings">QSettings</a></span><span class="operator">::</span>NativeFormat<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsettings.html#QSettings">QSettings</a></span><span class="operator">::</span>SystemScope<span class="operator">,</span> path);
    ///   setPath(<span class="type"><a href="http://doc.qt.io/qt-5/qsettings.html#QSettings">QSettings</a></span><span class="operator">::</span>IniFormat<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsettings.html#QSettings">QSettings</a></span><span class="operator">::</span>SystemScope<span class="operator">,</span> path);
    ///
    /// </pre></div>
    pub unsafe fn set_system_ini_path(
        dir: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSettings_setSystemIniPath(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(dir).as_raw_ptr(),
        )
    }

    /// <p>Use <a href="http://doc.qt.io/qt-5/qsettings.html#setPath">setPath</a>() instead.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QSettings::setUserIniPath(const QString& dir)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings-obsolete.html#setUserIniPath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Use <a href="http://doc.qt.io/qt-5/qsettings.html#setPath">setPath</a>() instead.</p></div>
    pub unsafe fn set_user_ini_path(
        dir: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSettings_setUserIniPath(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(dir).as_raw_ptr(),
        )
    }

    /// <p>Sets the value of setting <i>key</i> to <i>value</i>. If the <i>key</i> already exists, the previous value is overwritten.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSettings::setValue(const QString& key, const QVariant& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#setValue">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the value of setting <i>key</i> to <i>value</i>. If the <i>key</i> already exists, the previous value is overwritten.</p>
    /// <p>Note that the Windows registry and INI files use case-insensitive keys, whereas the CFPreferences API on <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS uses case-sensitive keys. To avoid portability problems, see the <a href="http://doc.qt.io/qt-5/qsettings.html#section-and-key-syntax">Section and Key Syntax</a> rules.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsettings.html#QSettings">QSettings</a></span> settings;
    ///   settings<span class="operator">.</span>setValue(<span class="string">"interval"</span><span class="operator">,</span> <span class="number">30</span>);
    ///   settings<span class="operator">.</span>value(<span class="string">"interval"</span>)<span class="operator">.</span>toInt();     <span class="comment">// returns 30</span>
    ///
    ///   settings<span class="operator">.</span>setValue(<span class="string">"interval"</span><span class="operator">,</span> <span class="number">6.55</span>);
    ///   settings<span class="operator">.</span>value(<span class="string">"interval"</span>)<span class="operator">.</span>toDouble();  <span class="comment">// returns 6.55</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#value">value</a>(), <a href="http://doc.qt.io/qt-5/qsettings.html#remove">remove</a>(), and <a href="http://doc.qt.io/qt-5/qsettings.html#contains">contains</a>().</p></div>
    pub unsafe fn set_value(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSettings_setValue(
            self as *mut crate::QSettings,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSettings_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a status code indicating the first error that was met by <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a>, or <a href="http://doc.qt.io/qt-5/qsettings.html#Status-enum">QSettings::NoError</a> if no error occurred.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSettings::Status QSettings::status() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#status">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a status code indicating the first error that was met by <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a>, or <a href="http://doc.qt.io/qt-5/qsettings.html#Status-enum">QSettings::NoError</a> if no error occurred.</p>
    /// <p>Be aware that <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> delays performing some operations. For this reason, you might want to call <a href="http://doc.qt.io/qt-5/qsettings.html#sync">sync</a>() to ensure that the data stored in <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> is written to disk before calling status().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#sync">sync</a>().</p></div>
    pub unsafe fn status(&self) -> crate::q_settings::Status {
        crate::__ffi::ctr_qt_core_ffi_QSettings_status(self as *const crate::QSettings)
    }

    /// <p>Writes any unsaved changes to permanent storage, and reloads any settings that have been changed in the meantime by another application.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSettings::sync()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#sync">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes any unsaved changes to permanent storage, and reloads any settings that have been changed in the meantime by another application.</p>
    /// <p>This function is called automatically from <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a>'s destructor and by the event loop at regular intervals, so you normally don't need to call it yourself.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#status">status</a>().</p></div>
    pub unsafe fn sync(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSettings_sync(self as *mut crate::QSettings)
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QSettings::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSettings_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QSettings::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSettings_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value for setting <i>key</i>. If the setting doesn't exist, returns <i>defaultValue</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QSettings::value(const QString& key, const QVariant& defaultValue = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value for setting <i>key</i>. If the setting doesn't exist, returns <i>defaultValue</i>.</p>
    /// <p>If no default value is specified, a default <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> is returned.</p>
    /// <p>Note that the Windows registry and INI files use case-insensitive keys, whereas the CFPreferences API on <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS uses case-sensitive keys. To avoid portability problems, see the <a href="http://doc.qt.io/qt-5/qsettings.html#section-and-key-syntax">Section and Key Syntax</a> rules.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsettings.html#QSettings">QSettings</a></span> settings;
    ///   settings<span class="operator">.</span>setValue(<span class="string">"animal/snake"</span><span class="operator">,</span> <span class="number">58</span>);
    ///   settings<span class="operator">.</span>value(<span class="string">"animal/snake"</span><span class="operator">,</span> <span class="number">1024</span>)<span class="operator">.</span>toInt();   <span class="comment">// returns 58</span>
    ///   settings<span class="operator">.</span>value(<span class="string">"animal/zebra"</span><span class="operator">,</span> <span class="number">1024</span>)<span class="operator">.</span>toInt();   <span class="comment">// returns 1024</span>
    ///   settings<span class="operator">.</span>value(<span class="string">"animal/zebra"</span>)<span class="operator">.</span>toInt();         <span class="comment">// returns 0</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#setValue">setValue</a>(), <a href="http://doc.qt.io/qt-5/qsettings.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qsettings.html#remove">remove</a>().</p></div>
    pub unsafe fn value_2a(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSettings_value(
            self as *const crate::QSettings,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value for setting <i>key</i>. If the setting doesn't exist, returns <i>defaultValue</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QSettings::value(const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value for setting <i>key</i>. If the setting doesn't exist, returns <i>defaultValue</i>.</p>
    /// <p>If no default value is specified, a default <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> is returned.</p>
    /// <p>Note that the Windows registry and INI files use case-insensitive keys, whereas the CFPreferences API on <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> and iOS uses case-sensitive keys. To avoid portability problems, see the <a href="http://doc.qt.io/qt-5/qsettings.html#section-and-key-syntax">Section and Key Syntax</a> rules.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsettings.html#QSettings">QSettings</a></span> settings;
    ///   settings<span class="operator">.</span>setValue(<span class="string">"animal/snake"</span><span class="operator">,</span> <span class="number">58</span>);
    ///   settings<span class="operator">.</span>value(<span class="string">"animal/snake"</span><span class="operator">,</span> <span class="number">1024</span>)<span class="operator">.</span>toInt();   <span class="comment">// returns 58</span>
    ///   settings<span class="operator">.</span>value(<span class="string">"animal/zebra"</span><span class="operator">,</span> <span class="number">1024</span>)<span class="operator">.</span>toInt();   <span class="comment">// returns 1024</span>
    ///   settings<span class="operator">.</span>value(<span class="string">"animal/zebra"</span>)<span class="operator">.</span>toInt();         <span class="comment">// returns 0</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#setValue">setValue</a>(), <a href="http://doc.qt.io/qt-5/qsettings.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qsettings.html#remove">remove</a>().</p></div>
    pub unsafe fn value_1a(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSettings_value1(
            self as *const crate::QSettings,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_shared_memory {
    //! C++ type: <span style='color: green;'>```QSharedMemory```</span>

    /// C++ enum: <span style='color: green;'>```QSharedMemory::AccessMode```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsharedmemory.html#AccessMode-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct AccessMode(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for AccessMode {
        fn from(value: ::std::os::raw::c_int) -> Self {
            AccessMode(value)
        }
    }

    impl From<AccessMode> for ::std::os::raw::c_int {
        fn from(value: AccessMode) -> Self {
            value.0
        }
    }

    impl AccessMode {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl AccessMode {
        /// The shared memory segment is read-only. Writing to the shared memory segment is not allowed. An attempt to write to a shared memory segment created with ReadOnly causes the program to abort. (C++ enum variant: <span style='color: green;'>```ReadOnly = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const ReadOnly: crate::q_shared_memory::AccessMode =
            crate::q_shared_memory::AccessMode(0);
        /// Reading and writing the shared memory segment are both allowed. (C++ enum variant: <span style='color: green;'>```ReadWrite = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const ReadWrite: crate::q_shared_memory::AccessMode =
            crate::q_shared_memory::AccessMode(1);
    }

    /// C++ enum: <span style='color: green;'>```QSharedMemory::SharedMemoryError```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsharedmemory.html#SharedMemoryError-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct SharedMemoryError(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for SharedMemoryError {
        fn from(value: ::std::os::raw::c_int) -> Self {
            SharedMemoryError(value)
        }
    }

    impl From<SharedMemoryError> for ::std::os::raw::c_int {
        fn from(value: SharedMemoryError) -> Self {
            value.0
        }
    }

    impl SharedMemoryError {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl SharedMemoryError {
        /// No error occurred. (C++ enum variant: <span style='color: green;'>```NoError = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoError: crate::q_shared_memory::SharedMemoryError =
            crate::q_shared_memory::SharedMemoryError(0);
        /// The operation failed because the caller didn't have the required permissions. (C++ enum variant: <span style='color: green;'>```PermissionDenied = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const PermissionDenied: crate::q_shared_memory::SharedMemoryError =
            crate::q_shared_memory::SharedMemoryError(1);
        /// A create operation failed because the requested size was invalid. (C++ enum variant: <span style='color: green;'>```InvalidSize = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const InvalidSize: crate::q_shared_memory::SharedMemoryError =
            crate::q_shared_memory::SharedMemoryError(2);
        /// The operation failed because of an invalid key. (C++ enum variant: <span style='color: green;'>```KeyError = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const KeyError: crate::q_shared_memory::SharedMemoryError =
            crate::q_shared_memory::SharedMemoryError(3);
        /// A <a href="http://doc.qt.io/qt-5/qsharedmemory.html#create">create</a>() operation failed because a shared memory segment with the specified key already existed. (C++ enum variant: <span style='color: green;'>```AlreadyExists = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const AlreadyExists: crate::q_shared_memory::SharedMemoryError =
            crate::q_shared_memory::SharedMemoryError(4);
        /// An <a href="http://doc.qt.io/qt-5/qsharedmemory.html#attach">attach</a>() failed because a shared memory segment with the specified key could not be found. (C++ enum variant: <span style='color: green;'>```NotFound = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const NotFound: crate::q_shared_memory::SharedMemoryError =
            crate::q_shared_memory::SharedMemoryError(5);
        /// The attempt to <a href="http://doc.qt.io/qt-5/qsharedmemory.html#lock">lock</a>() the shared memory segment failed because <a href="http://doc.qt.io/qt-5/qsharedmemory.html#create">create</a>() or <a href="http://doc.qt.io/qt-5/qsharedmemory.html#attach">attach</a>() failed and returned false, or because a system error occurred in <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#acquire">QSystemSemaphore::acquire</a>(). (C++ enum variant: <span style='color: green;'>```LockError = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const LockError: crate::q_shared_memory::SharedMemoryError =
            crate::q_shared_memory::SharedMemoryError(6);
        /// A <a href="http://doc.qt.io/qt-5/qsharedmemory.html#create">create</a>() operation failed because there was not enough memory available to fill the request. (C++ enum variant: <span style='color: green;'>```OutOfResources = 7```</span>)
        #[allow(non_upper_case_globals)]
        pub const OutOfResources: crate::q_shared_memory::SharedMemoryError =
            crate::q_shared_memory::SharedMemoryError(7);
        /// Something else happened and it was bad. (C++ enum variant: <span style='color: green;'>```UnknownError = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const UnknownError: crate::q_shared_memory::SharedMemoryError =
            crate::q_shared_memory::SharedMemoryError(8);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a> class provides access to a shared memory segment.</p>
///
/// C++ class: <span style='color: green;'>```QSharedMemory```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qsharedmemory.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a> class provides access to a shared memory segment.</p>
/// <p><a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a> provides access to a shared memory segment by multiple threads and processes. It also provides a way for a single thread or process to lock the memory for exclusive access.</p>
/// <p>When using this class, be aware of the following platform differences:</p>
/// <ul>
/// <li>Windows: <a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a> does not "own" the shared memory segment. When all threads or processes that have an instance of <a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a> attached to a particular shared memory segment have either destroyed their instance of <a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a> or exited, the Windows kernel releases the shared memory segment automatically.</li>
/// <li>Unix: <a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a> "owns" the shared memory segment. When the last thread or process that has an instance of <a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a> attached to a particular shared memory segment detaches from the segment by destroying its instance of <a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a>, the Unix kernel release the shared memory segment. But if that last thread or process crashes without running the <a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a> destructor, the shared memory segment survives the crash.</li>
/// <li>HP-UX: Only one attach to a shared memory segment is allowed per process. This means that <a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a> should not be used across multiple threads in the same process in HP-UX.</li>
/// </ul>
/// <p>Remember to lock the shared memory with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#lock">lock</a>() before reading from or writing to the shared memory, and remember to release the lock with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#unlock">unlock</a>() after you are done.</p>
/// <p><a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a> automatically destroys the shared memory segment when the last instance of <a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a> is detached from the segment, and no references to the segment remain.</p>
/// <p><b>Warning:</b> <a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a> changes the key in a Qt-specific way, unless otherwise specified. Interoperation with non-Qt applications is achieved by first creating a default shared memory with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#QSharedMemory">QSharedMemory</a>() and then setting a native key with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setNativeKey">setNativeKey</a>(). When using native keys, shared memory is not protected against multiple accesses on it (for example, unable to <a href="http://doc.qt.io/qt-5/qsharedmemory.html#lock">lock</a>()) and a user-defined mechanism should be used to achieve such protection.</p></div>
#[repr(C)]
pub struct QSharedMemory {
    _unused: u8,
}
impl QSharedMemory {
    /// <p>Attempts to attach the process to the shared memory segment identified by the key that was passed to the constructor or to a call to <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setKey">setKey</a>() or <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setNativeKey">setNativeKey</a>(). The access <i>mode</i> is <a href="http://doc.qt.io/qt-5/qsharedmemory.html#AccessMode-enum">ReadWrite</a> by default. It can also be <a href="http://doc.qt.io/qt-5/qsharedmemory.html#AccessMode-enum">ReadOnly</a>. Returns <code>true</code> if the attach operation is successful. If false is returned, call <a href="http://doc.qt.io/qt-5/qsharedmemory.html#error">error</a>() to determine which error occurred. After attaching the shared memory segment, a pointer to the shared memory can be obtained by calling <a href="http://doc.qt.io/qt-5/qsharedmemory.html#data">data</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSharedMemory::attach(QSharedMemory::AccessMode mode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsharedmemory.html#attach">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to attach the process to the shared memory segment identified by the key that was passed to the constructor or to a call to <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setKey">setKey</a>() or <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setNativeKey">setNativeKey</a>(). The access <i>mode</i> is <a href="http://doc.qt.io/qt-5/qsharedmemory.html#AccessMode-enum">ReadWrite</a> by default. It can also be <a href="http://doc.qt.io/qt-5/qsharedmemory.html#AccessMode-enum">ReadOnly</a>. Returns <code>true</code> if the attach operation is successful. If false is returned, call <a href="http://doc.qt.io/qt-5/qsharedmemory.html#error">error</a>() to determine which error occurred. After attaching the shared memory segment, a pointer to the shared memory can be obtained by calling <a href="http://doc.qt.io/qt-5/qsharedmemory.html#data">data</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsharedmemory.html#isAttached">isAttached</a>(), <a href="http://doc.qt.io/qt-5/qsharedmemory.html#detach">detach</a>(), and <a href="http://doc.qt.io/qt-5/qsharedmemory.html#create">create</a>().</p></div>
    pub unsafe fn attach_1a(&mut self, mode: crate::q_shared_memory::AccessMode) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSharedMemory_attach(self as *mut crate::QSharedMemory, mode)
    }

    /// <p>Attempts to attach the process to the shared memory segment identified by the key that was passed to the constructor or to a call to <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setKey">setKey</a>() or <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setNativeKey">setNativeKey</a>(). The access <i>mode</i> is <a href="http://doc.qt.io/qt-5/qsharedmemory.html#AccessMode-enum">ReadWrite</a> by default. It can also be <a href="http://doc.qt.io/qt-5/qsharedmemory.html#AccessMode-enum">ReadOnly</a>. Returns <code>true</code> if the attach operation is successful. If false is returned, call <a href="http://doc.qt.io/qt-5/qsharedmemory.html#error">error</a>() to determine which error occurred. After attaching the shared memory segment, a pointer to the shared memory can be obtained by calling <a href="http://doc.qt.io/qt-5/qsharedmemory.html#data">data</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSharedMemory::attach()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsharedmemory.html#attach">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to attach the process to the shared memory segment identified by the key that was passed to the constructor or to a call to <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setKey">setKey</a>() or <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setNativeKey">setNativeKey</a>(). The access <i>mode</i> is <a href="http://doc.qt.io/qt-5/qsharedmemory.html#AccessMode-enum">ReadWrite</a> by default. It can also be <a href="http://doc.qt.io/qt-5/qsharedmemory.html#AccessMode-enum">ReadOnly</a>. Returns <code>true</code> if the attach operation is successful. If false is returned, call <a href="http://doc.qt.io/qt-5/qsharedmemory.html#error">error</a>() to determine which error occurred. After attaching the shared memory segment, a pointer to the shared memory can be obtained by calling <a href="http://doc.qt.io/qt-5/qsharedmemory.html#data">data</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsharedmemory.html#isAttached">isAttached</a>(), <a href="http://doc.qt.io/qt-5/qsharedmemory.html#detach">detach</a>(), and <a href="http://doc.qt.io/qt-5/qsharedmemory.html#create">create</a>().</p></div>
    pub unsafe fn attach_0a(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSharedMemory_attach1(self as *mut crate::QSharedMemory)
    }

    /// <p>Returns a const pointer to the contents of the shared memory segment, if one is attached. Otherwise it returns null. Remember to lock the shared memory with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#lock">lock</a>() before reading from or writing to the shared memory, and remember to release the lock with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#unlock">unlock</a>() after you are done.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const void* QSharedMemory::constData() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsharedmemory.html#constData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const pointer to the contents of the shared memory segment, if one is attached. Otherwise it returns null. Remember to lock the shared memory with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#lock">lock</a>() before reading from or writing to the shared memory, and remember to release the lock with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#unlock">unlock</a>() after you are done.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsharedmemory.html#attach">attach</a>() and <a href="http://doc.qt.io/qt-5/qsharedmemory.html#create">create</a>().</p></div>
    pub unsafe fn const_data(&self) -> ::cpp_utils::Ptr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSharedMemory_constData(
            self as *const crate::QSharedMemory,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Creates a shared memory segment of <i>size</i> bytes with the key passed to the constructor, set with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setKey">setKey</a>() or set with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setNativeKey">setNativeKey</a>(), then attaches to the new shared memory segment with the given access <i>mode</i> and returns <code>true</code>. If a shared memory segment identified by the key already exists, the attach operation is not performed and <code>false</code> is returned. When the return value is <code>false</code>, call <a href="http://doc.qt.io/qt-5/qsharedmemory.html#error">error</a>() to determine which error occurred.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSharedMemory::create(int size, QSharedMemory::AccessMode mode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsharedmemory.html#create">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a shared memory segment of <i>size</i> bytes with the key passed to the constructor, set with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setKey">setKey</a>() or set with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setNativeKey">setNativeKey</a>(), then attaches to the new shared memory segment with the given access <i>mode</i> and returns <code>true</code>. If a shared memory segment identified by the key already exists, the attach operation is not performed and <code>false</code> is returned. When the return value is <code>false</code>, call <a href="http://doc.qt.io/qt-5/qsharedmemory.html#error">error</a>() to determine which error occurred.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsharedmemory.html#error">error</a>().</p></div>
    pub unsafe fn create_2a(
        &mut self,
        size: ::std::os::raw::c_int,
        mode: crate::q_shared_memory::AccessMode,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSharedMemory_create(
            self as *mut crate::QSharedMemory,
            size,
            mode,
        )
    }

    /// <p>Creates a shared memory segment of <i>size</i> bytes with the key passed to the constructor, set with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setKey">setKey</a>() or set with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setNativeKey">setNativeKey</a>(), then attaches to the new shared memory segment with the given access <i>mode</i> and returns <code>true</code>. If a shared memory segment identified by the key already exists, the attach operation is not performed and <code>false</code> is returned. When the return value is <code>false</code>, call <a href="http://doc.qt.io/qt-5/qsharedmemory.html#error">error</a>() to determine which error occurred.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSharedMemory::create(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsharedmemory.html#create">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a shared memory segment of <i>size</i> bytes with the key passed to the constructor, set with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setKey">setKey</a>() or set with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setNativeKey">setNativeKey</a>(), then attaches to the new shared memory segment with the given access <i>mode</i> and returns <code>true</code>. If a shared memory segment identified by the key already exists, the attach operation is not performed and <code>false</code> is returned. When the return value is <code>false</code>, call <a href="http://doc.qt.io/qt-5/qsharedmemory.html#error">error</a>() to determine which error occurred.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsharedmemory.html#error">error</a>().</p></div>
    pub unsafe fn create_1a(&mut self, size: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSharedMemory_create1(self as *mut crate::QSharedMemory, size)
    }

    /// <p>Returns a pointer to the contents of the shared memory segment, if one is attached. Otherwise it returns null. Remember to lock the shared memory with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#lock">lock</a>() before reading from or writing to the shared memory, and remember to release the lock with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#unlock">unlock</a>() after you are done.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void* QSharedMemory::data()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsharedmemory.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the contents of the shared memory segment, if one is attached. Otherwise it returns null. Remember to lock the shared memory with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#lock">lock</a>() before reading from or writing to the shared memory, and remember to release the lock with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#unlock">unlock</a>() after you are done.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsharedmemory.html#attach">attach</a>().</p></div>
    pub unsafe fn data_mut(&mut self) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QSharedMemory_data(self as *mut crate::QSharedMemory);
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qsharedmemory.html#data">data</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const void* QSharedMemory::data() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsharedmemory.html#data-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qsharedmemory.html#data">data</a>().</p></div>
    pub unsafe fn data(&self) -> ::cpp_utils::Ptr<::std::ffi::c_void> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QSharedMemory_data1(self as *const crate::QSharedMemory);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Detaches the process from the shared memory segment. If this was the last process attached to the shared memory segment, then the shared memory segment is released by the system, i.e., the contents are destroyed. The function returns <code>true</code> if it detaches the shared memory segment. If it returns <code>false</code>, it usually means the segment either isn't attached, or it is locked by another process.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSharedMemory::detach()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsharedmemory.html#detach">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Detaches the process from the shared memory segment. If this was the last process attached to the shared memory segment, then the shared memory segment is released by the system, i.e., the contents are destroyed. The function returns <code>true</code> if it detaches the shared memory segment. If it returns <code>false</code>, it usually means the segment either isn't attached, or it is locked by another process.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsharedmemory.html#attach">attach</a>() and <a href="http://doc.qt.io/qt-5/qsharedmemory.html#isAttached">isAttached</a>().</p></div>
    pub unsafe fn detach(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSharedMemory_detach(self as *mut crate::QSharedMemory)
    }

    /// <p>Returns a value indicating whether an error occurred, and, if so, which error it was.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSharedMemory::SharedMemoryError QSharedMemory::error() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsharedmemory.html#error">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a value indicating whether an error occurred, and, if so, which error it was.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsharedmemory.html#errorString">errorString</a>().</p></div>
    pub unsafe fn error(&self) -> crate::q_shared_memory::SharedMemoryError {
        crate::__ffi::ctr_qt_core_ffi_QSharedMemory_error(self as *const crate::QSharedMemory)
    }

    /// <p>Returns a text description of the last error that occurred. If <a href="http://doc.qt.io/qt-5/qsharedmemory.html#error">error</a>() returns an <a href="http://doc.qt.io/qt-5/qsharedmemory.html#SharedMemoryError-enum">error value</a>, call this function to get a text string that describes the error.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QSharedMemory::errorString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsharedmemory.html#errorString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a text description of the last error that occurred. If <a href="http://doc.qt.io/qt-5/qsharedmemory.html#error">error</a>() returns an <a href="http://doc.qt.io/qt-5/qsharedmemory.html#SharedMemoryError-enum">error value</a>, call this function to get a text string that describes the error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsharedmemory.html#error">error</a>().</p></div>
    pub unsafe fn error_string(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSharedMemory_errorString(
            self as *const crate::QSharedMemory,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if this process is attached to the shared memory segment.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSharedMemory::isAttached() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsharedmemory.html#isAttached">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this process is attached to the shared memory segment.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsharedmemory.html#attach">attach</a>() and <a href="http://doc.qt.io/qt-5/qsharedmemory.html#detach">detach</a>().</p></div>
    pub unsafe fn is_attached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSharedMemory_isAttached(self as *const crate::QSharedMemory)
    }

    /// <p>Returns the key assigned with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setKey">setKey</a>() to this shared memory, or a null key if no key has been assigned, or if the segment is using a <a href="http://doc.qt.io/qt-5/qsharedmemory.html#nativeKey">nativeKey</a>(). The key is the identifier used by Qt applications to identify the shared memory segment.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QSharedMemory::key() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsharedmemory.html#key">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the key assigned with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setKey">setKey</a>() to this shared memory, or a null key if no key has been assigned, or if the segment is using a <a href="http://doc.qt.io/qt-5/qsharedmemory.html#nativeKey">nativeKey</a>(). The key is the identifier used by Qt applications to identify the shared memory segment.</p>
    /// <p>You can find the native, platform specific, key used by the operating system by calling <a href="http://doc.qt.io/qt-5/qsharedmemory.html#nativeKey">nativeKey</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsharedmemory.html#setKey">setKey</a>() and <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setNativeKey">setNativeKey</a>().</p></div>
    pub unsafe fn key(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QSharedMemory_key(self as *const crate::QSharedMemory);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is a semaphore that locks the shared memory segment for access by this process and returns <code>true</code>. If another process has locked the segment, this function blocks until the lock is released. Then it acquires the lock and returns <code>true</code>. If this function returns <code>false</code>, it means that you have ignored a false return from <a href="http://doc.qt.io/qt-5/qsharedmemory.html#create">create</a>() or <a href="http://doc.qt.io/qt-5/qsharedmemory.html#attach">attach</a>(), that you have set the key with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setNativeKey">setNativeKey</a>() or that <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#acquire">QSystemSemaphore::acquire</a>() failed due to an unknown system error.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSharedMemory::lock()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsharedmemory.html#lock">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is a semaphore that locks the shared memory segment for access by this process and returns <code>true</code>. If another process has locked the segment, this function blocks until the lock is released. Then it acquires the lock and returns <code>true</code>. If this function returns <code>false</code>, it means that you have ignored a false return from <a href="http://doc.qt.io/qt-5/qsharedmemory.html#create">create</a>() or <a href="http://doc.qt.io/qt-5/qsharedmemory.html#attach">attach</a>(), that you have set the key with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setNativeKey">setNativeKey</a>() or that <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#acquire">QSystemSemaphore::acquire</a>() failed due to an unknown system error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsharedmemory.html#unlock">unlock</a>(), <a href="http://doc.qt.io/qt-5/qsharedmemory.html#data">data</a>(), and <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#acquire">QSystemSemaphore::acquire</a>().</p></div>
    pub unsafe fn lock(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSharedMemory_lock(self as *mut crate::QSharedMemory)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QSharedMemory::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSharedMemory_metaObject(
            self as *const crate::QSharedMemory,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns the native, platform specific, key for this shared memory object. The native key is the identifier used by the operating system to identify the shared memory segment.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QSharedMemory::nativeKey() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsharedmemory.html#nativeKey">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the native, platform specific, key for this shared memory object. The native key is the identifier used by the operating system to identify the shared memory segment.</p>
    /// <p>You can use the native key to access shared memory segments that have not been created by Qt, or to grant shared memory access to non-Qt applications.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsharedmemory.html#setKey">setKey</a>() and <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setNativeKey">setNativeKey</a>().</p></div>
    pub unsafe fn native_key(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSharedMemory_nativeKey(
            self as *const crate::QSharedMemory,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qsharedmemory.html#QSharedMemory">QSharedMemory</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSharedMemory::QSharedMemory(QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsharedmemory.html#QSharedMemory-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qsharedmemory.html#QSharedMemory">QSharedMemory</a>().</p>
    /// <p>Constructs a shared memory object with the given <i>parent</i>. The shared memory object's key is not set by the constructor, so the shared memory object does not have an underlying shared memory segment attached. The key must be set with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setKey">setKey</a>() or <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setNativeKey">setNativeKey</a>() before <a href="http://doc.qt.io/qt-5/qsharedmemory.html#create">create</a>() or <a href="http://doc.qt.io/qt-5/qsharedmemory.html#attach">attach</a>() can be used.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsharedmemory.html#setKey">setKey</a>().</p></div>
    pub unsafe fn from_q_object(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QSharedMemory> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSharedMemory_QSharedMemory(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a shared memory object with the given <i>parent</i> and with its key set to <i>key</i>. Because its key is set, its <a href="http://doc.qt.io/qt-5/qsharedmemory.html#create">create</a>() and <a href="http://doc.qt.io/qt-5/qsharedmemory.html#attach">attach</a>() functions can be called.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSharedMemory::QSharedMemory(const QString& key, QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsharedmemory.html#QSharedMemory">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a shared memory object with the given <i>parent</i> and with its key set to <i>key</i>. Because its key is set, its <a href="http://doc.qt.io/qt-5/qsharedmemory.html#create">create</a>() and <a href="http://doc.qt.io/qt-5/qsharedmemory.html#attach">attach</a>() functions can be called.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsharedmemory.html#setKey">setKey</a>(), <a href="http://doc.qt.io/qt-5/qsharedmemory.html#create">create</a>(), and <a href="http://doc.qt.io/qt-5/qsharedmemory.html#attach">attach</a>().</p></div>
    pub unsafe fn from_q_string_q_object(
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QSharedMemory> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSharedMemory_QSharedMemory1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a> class provides access to a shared memory segment.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSharedMemory::QSharedMemory()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsharedmemory.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a> class provides access to a shared memory segment.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a> provides access to a shared memory segment by multiple threads and processes. It also provides a way for a single thread or process to lock the memory for exclusive access.</p>
    /// <p>When using this class, be aware of the following platform differences:</p>
    /// <ul>
    /// <li>Windows: <a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a> does not "own" the shared memory segment. When all threads or processes that have an instance of <a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a> attached to a particular shared memory segment have either destroyed their instance of <a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a> or exited, the Windows kernel releases the shared memory segment automatically.</li>
    /// <li>Unix: <a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a> "owns" the shared memory segment. When the last thread or process that has an instance of <a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a> attached to a particular shared memory segment detaches from the segment by destroying its instance of <a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a>, the Unix kernel release the shared memory segment. But if that last thread or process crashes without running the <a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a> destructor, the shared memory segment survives the crash.</li>
    /// <li>HP-UX: Only one attach to a shared memory segment is allowed per process. This means that <a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a> should not be used across multiple threads in the same process in HP-UX.</li>
    /// </ul>
    /// <p>Remember to lock the shared memory with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#lock">lock</a>() before reading from or writing to the shared memory, and remember to release the lock with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#unlock">unlock</a>() after you are done.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a> automatically destroys the shared memory segment when the last instance of <a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a> is detached from the segment, and no references to the segment remain.</p>
    /// <p><b>Warning:</b> <a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a> changes the key in a Qt-specific way, unless otherwise specified. Interoperation with non-Qt applications is achieved by first creating a default shared memory with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#QSharedMemory">QSharedMemory</a>() and then setting a native key with <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setNativeKey">setNativeKey</a>(). When using native keys, shared memory is not protected against multiple accesses on it (for example, unable to <a href="http://doc.qt.io/qt-5/qsharedmemory.html#lock">lock</a>()) and a user-defined mechanism should be used to achieve such protection.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QSharedMemory> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSharedMemory_QSharedMemory2();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a shared memory object with the given <i>parent</i> and with its key set to <i>key</i>. Because its key is set, its <a href="http://doc.qt.io/qt-5/qsharedmemory.html#create">create</a>() and <a href="http://doc.qt.io/qt-5/qsharedmemory.html#attach">attach</a>() functions can be called.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSharedMemory::QSharedMemory(const QString& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsharedmemory.html#QSharedMemory">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a shared memory object with the given <i>parent</i> and with its key set to <i>key</i>. Because its key is set, its <a href="http://doc.qt.io/qt-5/qsharedmemory.html#create">create</a>() and <a href="http://doc.qt.io/qt-5/qsharedmemory.html#attach">attach</a>() functions can be called.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsharedmemory.html#setKey">setKey</a>(), <a href="http://doc.qt.io/qt-5/qsharedmemory.html#create">create</a>(), and <a href="http://doc.qt.io/qt-5/qsharedmemory.html#attach">attach</a>().</p></div>
    pub unsafe fn from_q_string(
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QSharedMemory> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSharedMemory_QSharedMemory3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QSharedMemory::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QSharedMemory_qt_metacall(
            self as *mut crate::QSharedMemory,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QSharedMemory::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSharedMemory_qt_metacast(
            self as *mut crate::QSharedMemory,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Sets the platform independent <i>key</i> for this shared memory object. If <i>key</i> is the same as the current key, the function returns without doing anything.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSharedMemory::setKey(const QString& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setKey">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the platform independent <i>key</i> for this shared memory object. If <i>key</i> is the same as the current key, the function returns without doing anything.</p>
    /// <p>You can call <a href="http://doc.qt.io/qt-5/qsharedmemory.html#key">key</a>() to retrieve the platform independent key. Internally, <a href="http://doc.qt.io/qt-5/qsharedmemory.html">QSharedMemory</a> converts this key into a platform specific key. If you instead call <a href="http://doc.qt.io/qt-5/qsharedmemory.html#nativeKey">nativeKey</a>(), you will get the platform specific, converted key.</p>
    /// <p>If the shared memory object is attached to an underlying shared memory segment, it will <a href="http://doc.qt.io/qt-5/qsharedmemory.html#detach">detach</a> from it before setting the new key. This function does not do an <a href="http://doc.qt.io/qt-5/qsharedmemory.html#attach">attach</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsharedmemory.html#key">key</a>(), <a href="http://doc.qt.io/qt-5/qsharedmemory.html#nativeKey">nativeKey</a>(), and <a href="http://doc.qt.io/qt-5/qsharedmemory.html#isAttached">isAttached</a>().</p></div>
    pub unsafe fn set_key(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSharedMemory_setKey(
            self as *mut crate::QSharedMemory,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        )
    }

    /// <p>Sets the native, platform specific, <i>key</i> for this shared memory object. If <i>key</i> is the same as the current native key, the function returns without doing anything. If all you want is to assign a key to a segment, you should call <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setKey">setKey</a>() instead.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSharedMemory::setNativeKey(const QString& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setNativeKey">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the native, platform specific, <i>key</i> for this shared memory object. If <i>key</i> is the same as the current native key, the function returns without doing anything. If all you want is to assign a key to a segment, you should call <a href="http://doc.qt.io/qt-5/qsharedmemory.html#setKey">setKey</a>() instead.</p>
    /// <p>You can call <a href="http://doc.qt.io/qt-5/qsharedmemory.html#nativeKey">nativeKey</a>() to retrieve the native key. If a native key has been assigned, calling <a href="http://doc.qt.io/qt-5/qsharedmemory.html#key">key</a>() will return a null string.</p>
    /// <p>If the shared memory object is attached to an underlying shared memory segment, it will <a href="http://doc.qt.io/qt-5/qsharedmemory.html#detach">detach</a> from it before setting the new key. This function does not do an <a href="http://doc.qt.io/qt-5/qsharedmemory.html#attach">attach</a>().</p>
    /// <p>The application will not be portable if you set a native key.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsharedmemory.html#nativeKey">nativeKey</a>(), <a href="http://doc.qt.io/qt-5/qsharedmemory.html#key">key</a>(), and <a href="http://doc.qt.io/qt-5/qsharedmemory.html#isAttached">isAttached</a>().</p></div>
    pub unsafe fn set_native_key(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSharedMemory_setNativeKey(
            self as *mut crate::QSharedMemory,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        )
    }

    /// <p>Returns the size of the attached shared memory segment. If no shared memory segment is attached, 0 is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QSharedMemory::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsharedmemory.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the size of the attached shared memory segment. If no shared memory segment is attached, 0 is returned.</p>
    /// <p><b>Note: </b>The size of the segment may be larger than the requested size that was passed to <a href="http://doc.qt.io/qt-5/qsharedmemory.html#create">create</a>().</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qsharedmemory.html#create">create</a>() and <a href="http://doc.qt.io/qt-5/qsharedmemory.html#attach">attach</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QSharedMemory_size(self as *const crate::QSharedMemory)
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSharedMemory_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QSharedMemory::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSharedMemory_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QSharedMemory::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSharedMemory_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Releases the lock on the shared memory segment and returns <code>true</code>, if the lock is currently held by this process. If the segment is not locked, or if the lock is held by another process, nothing happens and false is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSharedMemory::unlock()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsharedmemory.html#unlock">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Releases the lock on the shared memory segment and returns <code>true</code>, if the lock is currently held by this process. If the segment is not locked, or if the lock is held by another process, nothing happens and false is returned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsharedmemory.html#lock">lock</a>().</p></div>
    pub unsafe fn unlock(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSharedMemory_unlock(self as *mut crate::QSharedMemory)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qsignalmapper.html">QSignalMapper</a> class bundles signals from identifiable senders.</p>
///
/// C++ class: <span style='color: green;'>```QSignalMapper```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qsignalmapper.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsignalmapper.html">QSignalMapper</a> class bundles signals from identifiable senders.</p>
/// <p>This class collects a set of parameterless signals, and re-emits them with integer, string or widget parameters corresponding to the object that sent the signal.</p>
/// <p>The class supports the mapping of particular strings or integers with particular objects using <a href="http://doc.qt.io/qt-5/qsignalmapper.html#setMapping">setMapping</a>(). The objects' signals can then be connected to the <a href="http://doc.qt.io/qt-5/qsignalmapper.html#map">map</a>() slot which will emit the <a href="http://doc.qt.io/qt-5/qsignalmapper.html#mapped">mapped</a>() signal with the string or integer associated with the original signalling object. Mappings can be removed later using <a href="http://doc.qt.io/qt-5/qsignalmapper.html#removeMappings">removeMappings</a>().</p>
/// <p>Example: Suppose we want to create a custom widget that contains a group of buttons (like a tool palette). One approach is to connect each button's <code>clicked()</code> signal to its own custom slot; but in this example we want to connect all the buttons to a single slot and parameterize the slot by the button that was clicked.</p>
/// <p>Here's the definition of a simple custom widget that has a single signal, <code>clicked()</code>, which is emitted with the text of the button that was clicked:</p>
/// <pre class="cpp">
///
///   <span class="keyword">class</span> ButtonWidget : <span class="keyword">public</span> <span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span>
///   {
/// &#32;     Q_OBJECT
///
///   <span class="keyword">public</span>:
/// &#32;     ButtonWidget(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> <span class="operator">&amp;</span>texts<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span>parent <span class="operator">=</span> <span class="number">0</span>);
///
///   <span class="keyword">signals</span>:
/// &#32;     <span class="type">void</span> clicked(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>text);
///
///   <span class="keyword">private</span>:
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qsignalmapper.html#QSignalMapper">QSignalMapper</a></span> <span class="operator">*</span>signalMapper;
///   };
///
/// </pre>
/// <p>The only function that we need to implement is the constructor:</p>
/// <pre class="cpp">
///
///   ButtonWidget<span class="operator">::</span>ButtonWidget(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> <span class="operator">&amp;</span>texts<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span>parent)
/// &#32;     : <span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span>(parent)
///   {
/// &#32;     signalMapper <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsignalmapper.html#QSignalMapper">QSignalMapper</a></span>(<span class="keyword">this</span>);
///
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qgridlayout.html">QGridLayout</a></span> <span class="operator">*</span>gridLayout <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qgridlayout.html">QGridLayout</a></span>;
/// &#32;     <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> texts<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span>button <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span>(texts<span class="operator">[</span>i<span class="operator">]</span>);
/// &#32;   &#32;     connect(button<span class="operator">,</span> SIGNAL(clicked())<span class="operator">,</span> signalMapper<span class="operator">,</span> SLOT(map()));
/// &#32;   &#32;     signalMapper<span class="operator">-</span><span class="operator">&gt;</span>setMapping(button<span class="operator">,</span> texts<span class="operator">[</span>i<span class="operator">]</span>);
/// &#32;   &#32;     gridLayout<span class="operator">-</span><span class="operator">&gt;</span>addWidget(button<span class="operator">,</span> i <span class="operator">/</span> <span class="number">3</span><span class="operator">,</span> i <span class="operator">%</span> <span class="number">3</span>);
/// &#32;     }
///
/// &#32;     connect(signalMapper<span class="operator">,</span> SIGNAL(mapped(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>))<span class="operator">,</span>
/// &#32;   &#32;   &#32;     <span class="keyword">this</span><span class="operator">,</span> SIGNAL(clicked(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>)));
///
/// &#32;     setLayout(gridLayout);
///   }
///
/// </pre>
/// <p>A list of texts is passed to the constructor. A signal mapper is constructed and for each text in the list a <a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a> is created. We connect each button's <code>clicked()</code> signal to the signal mapper's <a href="http://doc.qt.io/qt-5/qsignalmapper.html#map">map</a>() slot, and create a mapping in the signal mapper from each button to the button's text. Finally we connect the signal mapper's <a href="http://doc.qt.io/qt-5/qsignalmapper.html#mapped">mapped</a>() signal to the custom widget's <code>clicked()</code> signal. When the user clicks a button, the custom widget will emit a single <code>clicked()</code> signal whose argument is the text of the button the user clicked.</p></div>
#[repr(C)]
pub struct QSignalMapper {
    _unused: u8,
}
impl QSignalMapper {
    /// <p>This signal is emitted when <a href="http://doc.qt.io/qt-5/qsignalmapper.html#map">map</a>() is signalled from an object that has an integer mapping set. The object's mapped integer is passed in <i>i</i>.</p>
    ///
    /// Returns a built-in Qt signal `QSignalMapper::mapped` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignalmapper.html#mapped">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when <a href="http://doc.qt.io/qt-5/qsignalmapper.html#map">map</a>() is signalled from an object that has an integer mapping set. The object's mapped integer is passed in <i>i</i>.</p>
    /// <p><b>Note:</b> Signal <i>mapped</i> is overloaded in this class. To connect to this one using the function pointer syntax, you must specify the signal type in a static cast, as shown in this example:</p><pre class="cpp">
    ///
    ///   connect(signalMapper<span class="operator">,</span> <span class="keyword">static_cast</span><span class="operator">&lt;</span><span class="type">void</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qsignalmapper.html#QSignalMapper">QSignalMapper</a></span><span class="operator">::</span><span class="operator">*</span>)(<span class="type">int</span>)<span class="operator">&gt;</span>(<span class="operator">&amp;</span><span class="type"><a href="http://doc.qt.io/qt-5/qsignalmapper.html#QSignalMapper">QSignalMapper</a></span><span class="operator">::</span>mapped)<span class="operator">,</span>
    /// &#32;     <span class="operator">[</span><span class="operator">=</span><span class="operator">]</span>(<span class="type">int</span> i){ <span class="comment">/* ... */</span> });
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsignalmapper.html#setMapping">setMapping</a>().</p></div>
    pub fn mapped(&self) -> crate::Signal<(::std::os::raw::c_int,)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2mapped(int)\0"),
            )
        }
    }

    /// <p>This signal is emitted when <a href="http://doc.qt.io/qt-5/qsignalmapper.html#map">map</a>() is signalled from an object that has a string mapping set. The object's mapped string is passed in <i>text</i>.</p>
    ///
    /// Returns a built-in Qt signal `QSignalMapper::mapped` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignalmapper.html#mapped-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when <a href="http://doc.qt.io/qt-5/qsignalmapper.html#map">map</a>() is signalled from an object that has a string mapping set. The object's mapped string is passed in <i>text</i>.</p>
    /// <p><b>Note:</b> Signal <i>mapped</i> is overloaded in this class. To connect to this one using the function pointer syntax, you must specify the signal type in a static cast, as shown in this example:</p><pre class="cpp">
    ///
    ///   connect(signalMapper<span class="operator">,</span> <span class="keyword">static_cast</span><span class="operator">&lt;</span><span class="type">void</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qsignalmapper.html#QSignalMapper">QSignalMapper</a></span><span class="operator">::</span><span class="operator">*</span>)(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>)<span class="operator">&gt;</span>(<span class="operator">&amp;</span><span class="type"><a href="http://doc.qt.io/qt-5/qsignalmapper.html#QSignalMapper">QSignalMapper</a></span><span class="operator">::</span>mapped)<span class="operator">,</span>
    /// &#32;     <span class="operator">[</span><span class="operator">=</span><span class="operator">]</span>(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>text){ <span class="comment">/* ... */</span> });
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsignalmapper.html#setMapping">setMapping</a>().</p></div>
    pub fn mapped2(&self) -> crate::Signal<(*const crate::QString,)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2mapped(const QString&)\0"),
            )
        }
    }

    /// <p>This signal is emitted when <a href="http://doc.qt.io/qt-5/qsignalmapper.html#map">map</a>() is signalled from an object that has an object mapping set. The object provided by the map is passed in <i>object</i>.</p>
    ///
    /// Returns a built-in Qt signal `QSignalMapper::mapped` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignalmapper.html#mapped-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when <a href="http://doc.qt.io/qt-5/qsignalmapper.html#map">map</a>() is signalled from an object that has an object mapping set. The object provided by the map is passed in <i>object</i>.</p>
    /// <p><b>Note:</b> Signal <i>mapped</i> is overloaded in this class. To connect to this one using the function pointer syntax, you must specify the signal type in a static cast, as shown in this example:</p><pre class="cpp">
    ///
    ///   connect(signalMapper<span class="operator">,</span> <span class="keyword">static_cast</span><span class="operator">&lt;</span><span class="type">void</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qsignalmapper.html#QSignalMapper">QSignalMapper</a></span><span class="operator">::</span><span class="operator">*</span>)(<span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span> <span class="operator">*</span>)<span class="operator">&gt;</span>(<span class="operator">&amp;</span><span class="type"><a href="http://doc.qt.io/qt-5/qsignalmapper.html#QSignalMapper">QSignalMapper</a></span><span class="operator">::</span>mapped)<span class="operator">,</span>
    /// &#32;     <span class="operator">[</span><span class="operator">=</span><span class="operator">]</span>(<span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span> <span class="operator">*</span>object){ <span class="comment">/* ... */</span> });
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsignalmapper.html#setMapping">setMapping</a>().</p></div>
    pub fn mapped3(&self) -> crate::Signal<(*mut crate::QObject,)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2mapped(QObject*)\0"),
            )
        }
    }

    /// <p>This slot emits signals based on which object sends signals to it.</p>
    ///
    /// Returns a built-in Qt slot `QSignalMapper::map` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignalmapper.html#map">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This slot emits signals based on which object sends signals to it.</p></div>
    pub fn slot_map(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1map()\0"),
            )
        }
    }

    /// <p>This slot emits signals based on the <i>sender</i> object.</p>
    ///
    /// Returns a built-in Qt slot `QSignalMapper::map` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignalmapper.html#map-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This slot emits signals based on the <i>sender</i> object.</p></div>
    pub fn slot_map2(&self) -> crate::Receiver<(*mut crate::QObject,)> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1map(QObject*)\0"),
            )
        }
    }

    /// <p>This slot emits signals based on which object sends signals to it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QSignalMapper::map()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignalmapper.html#map">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This slot emits signals based on which object sends signals to it.</p></div>
    pub unsafe fn map_0a(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSignalMapper_map(self as *mut crate::QSignalMapper)
    }

    /// <p>This slot emits signals based on the <i>sender</i> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QSignalMapper::map(QObject* sender)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignalmapper.html#map-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This slot emits signals based on the <i>sender</i> object.</p></div>
    pub unsafe fn map_1a(
        &mut self,
        sender: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSignalMapper_map1(
            self as *mut crate::QSignalMapper,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(sender)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the sender <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> that is associated with the <i>id</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QObject* QSignalMapper::mapping(int id) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignalmapper.html#mapping">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the sender <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> that is associated with the <i>id</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsignalmapper.html#setMapping">setMapping</a>().</p></div>
    pub unsafe fn mapping_int(
        &self,
        id: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSignalMapper_mapping(
            self as *const crate::QSignalMapper,
            id,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qsignalmapper.html#mapping">mapping</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QObject* QSignalMapper::mapping(const QString& text) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignalmapper.html#mapping-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qsignalmapper.html#mapping">mapping</a>().</p></div>
    pub unsafe fn mapping_q_string(
        &self,
        text: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSignalMapper_mapping1(
            self as *const crate::QSignalMapper,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(text).as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qsignalmapper.html#mapping">mapping</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QObject* QSignalMapper::mapping(QObject* object) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignalmapper.html#mapping-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qsignalmapper.html#mapping">mapping</a>().</p>
    /// <p>Returns the sender <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> that is associated with the <i>object</i>.</p></div>
    pub unsafe fn mapping_q_object(
        &self,
        object: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSignalMapper_mapping3(
            self as *const crate::QSignalMapper,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(object)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QSignalMapper::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSignalMapper_metaObject(
            self as *const crate::QSignalMapper,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qsignalmapper.html">QSignalMapper</a> with parent <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSignalMapper::QSignalMapper(QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignalmapper.html#QSignalMapper">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qsignalmapper.html">QSignalMapper</a> with parent <i>parent</i>.</p></div>
    pub unsafe fn new_1a(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QSignalMapper> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSignalMapper_QSignalMapper(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qsignalmapper.html">QSignalMapper</a> class bundles signals from identifiable senders.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSignalMapper::QSignalMapper()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignalmapper.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsignalmapper.html">QSignalMapper</a> class bundles signals from identifiable senders.</p>
    /// <p>This class collects a set of parameterless signals, and re-emits them with integer, string or widget parameters corresponding to the object that sent the signal.</p>
    /// <p>The class supports the mapping of particular strings or integers with particular objects using <a href="http://doc.qt.io/qt-5/qsignalmapper.html#setMapping">setMapping</a>(). The objects' signals can then be connected to the <a href="http://doc.qt.io/qt-5/qsignalmapper.html#map">map</a>() slot which will emit the <a href="http://doc.qt.io/qt-5/qsignalmapper.html#mapped">mapped</a>() signal with the string or integer associated with the original signalling object. Mappings can be removed later using <a href="http://doc.qt.io/qt-5/qsignalmapper.html#removeMappings">removeMappings</a>().</p>
    /// <p>Example: Suppose we want to create a custom widget that contains a group of buttons (like a tool palette). One approach is to connect each button's <code>clicked()</code> signal to its own custom slot; but in this example we want to connect all the buttons to a single slot and parameterize the slot by the button that was clicked.</p>
    /// <p>Here's the definition of a simple custom widget that has a single signal, <code>clicked()</code>, which is emitted with the text of the button that was clicked:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">class</span> ButtonWidget : <span class="keyword">public</span> <span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span>
    ///   {
    /// &#32;     Q_OBJECT
    ///
    ///   <span class="keyword">public</span>:
    /// &#32;     ButtonWidget(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> <span class="operator">&amp;</span>texts<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span>parent <span class="operator">=</span> <span class="number">0</span>);
    ///
    ///   <span class="keyword">signals</span>:
    /// &#32;     <span class="type">void</span> clicked(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>text);
    ///
    ///   <span class="keyword">private</span>:
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qsignalmapper.html#QSignalMapper">QSignalMapper</a></span> <span class="operator">*</span>signalMapper;
    ///   };
    ///
    /// </pre>
    /// <p>The only function that we need to implement is the constructor:</p>
    /// <pre class="cpp">
    ///
    ///   ButtonWidget<span class="operator">::</span>ButtonWidget(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> <span class="operator">&amp;</span>texts<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span>parent)
    /// &#32;     : <span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span>(parent)
    ///   {
    /// &#32;     signalMapper <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsignalmapper.html#QSignalMapper">QSignalMapper</a></span>(<span class="keyword">this</span>);
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qgridlayout.html">QGridLayout</a></span> <span class="operator">*</span>gridLayout <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qgridlayout.html">QGridLayout</a></span>;
    /// &#32;     <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> texts<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
    /// &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> <span class="operator">*</span>button <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span>(texts<span class="operator">[</span>i<span class="operator">]</span>);
    /// &#32;   &#32;     connect(button<span class="operator">,</span> SIGNAL(clicked())<span class="operator">,</span> signalMapper<span class="operator">,</span> SLOT(map()));
    /// &#32;   &#32;     signalMapper<span class="operator">-</span><span class="operator">&gt;</span>setMapping(button<span class="operator">,</span> texts<span class="operator">[</span>i<span class="operator">]</span>);
    /// &#32;   &#32;     gridLayout<span class="operator">-</span><span class="operator">&gt;</span>addWidget(button<span class="operator">,</span> i <span class="operator">/</span> <span class="number">3</span><span class="operator">,</span> i <span class="operator">%</span> <span class="number">3</span>);
    /// &#32;     }
    ///
    /// &#32;     connect(signalMapper<span class="operator">,</span> SIGNAL(mapped(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>))<span class="operator">,</span>
    /// &#32;   &#32;   &#32;     <span class="keyword">this</span><span class="operator">,</span> SIGNAL(clicked(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>)));
    ///
    /// &#32;     setLayout(gridLayout);
    ///   }
    ///
    /// </pre>
    /// <p>A list of texts is passed to the constructor. A signal mapper is constructed and for each text in the list a <a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a> is created. We connect each button's <code>clicked()</code> signal to the signal mapper's <a href="http://doc.qt.io/qt-5/qsignalmapper.html#map">map</a>() slot, and create a mapping in the signal mapper from each button to the button's text. Finally we connect the signal mapper's <a href="http://doc.qt.io/qt-5/qsignalmapper.html#mapped">mapped</a>() signal to the custom widget's <code>clicked()</code> signal. When the user clicks a button, the custom widget will emit a single <code>clicked()</code> signal whose argument is the text of the button the user clicked.</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QSignalMapper> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSignalMapper_QSignalMapper1();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QSignalMapper::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QSignalMapper_qt_metacall(
            self as *mut crate::QSignalMapper,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QSignalMapper::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSignalMapper_qt_metacast(
            self as *mut crate::QSignalMapper,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Removes all mappings for <i>sender</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSignalMapper::removeMappings(QObject* sender)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignalmapper.html#removeMappings">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all mappings for <i>sender</i>.</p>
    /// <p>This is done automatically when mapped objects are destroyed.</p>
    /// <p><b>Note: </b>This does not disconnect any signals. If <i>sender</i> is not destroyed then this will need to be done explicitly if required.</p></div>
    pub unsafe fn remove_mappings(
        &mut self,
        sender: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSignalMapper_removeMappings(
            self as *mut crate::QSignalMapper,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(sender)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Adds a mapping so that when <a href="http://doc.qt.io/qt-5/qsignalmapper.html#map">map</a>() is signalled from the given <i>sender</i>, the signal mapped(<i>id</i>) is emitted.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSignalMapper::setMapping(QObject* sender, int id)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignalmapper.html#setMapping">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds a mapping so that when <a href="http://doc.qt.io/qt-5/qsignalmapper.html#map">map</a>() is signalled from the given <i>sender</i>, the signal mapped(<i>id</i>) is emitted.</p>
    /// <p>There may be at most one integer ID for each sender.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsignalmapper.html#mapping">mapping</a>().</p></div>
    pub unsafe fn set_mapping_q_object_int(
        &mut self,
        sender: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
        id: ::std::os::raw::c_int,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSignalMapper_setMapping(
            self as *mut crate::QSignalMapper,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(sender)
                .as_mut_raw_ptr(),
            id,
        )
    }

    /// <p>Adds a mapping so that when <a href="http://doc.qt.io/qt-5/qsignalmapper.html#map">map</a>() is signalled from the <i>sender</i>, the signal mapped(<i>text</i> ) is emitted.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSignalMapper::setMapping(QObject* sender, const QString& text)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignalmapper.html#setMapping-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds a mapping so that when <a href="http://doc.qt.io/qt-5/qsignalmapper.html#map">map</a>() is signalled from the <i>sender</i>, the signal mapped(<i>text</i> ) is emitted.</p>
    /// <p>There may be at most one text for each sender.</p></div>
    pub unsafe fn set_mapping_q_object_q_string(
        &mut self,
        sender: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
        text: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSignalMapper_setMapping1(
            self as *mut crate::QSignalMapper,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(sender)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(text).as_raw_ptr(),
        )
    }

    /// <p>Adds a mapping so that when <a href="http://doc.qt.io/qt-5/qsignalmapper.html#map">map</a>() is signalled from the <i>sender</i>, the signal mapped(<i>object</i> ) is emitted.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSignalMapper::setMapping(QObject* sender, QObject* object)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignalmapper.html#setMapping-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds a mapping so that when <a href="http://doc.qt.io/qt-5/qsignalmapper.html#map">map</a>() is signalled from the <i>sender</i>, the signal mapped(<i>object</i> ) is emitted.</p>
    /// <p>There may be at most one object for each sender.</p></div>
    pub unsafe fn set_mapping_2_q_object(
        &mut self,
        sender: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
        object: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSignalMapper_setMapping3(
            self as *mut crate::QSignalMapper,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(sender)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(object)
                .as_mut_raw_ptr(),
        )
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSignalMapper_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QSignalMapper::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSignalMapper_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QSignalMapper::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSignalMapper_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qsignaltransition.html">QSignalTransition</a> class provides a transition based on a Qt signal.</p>
///
/// C++ class: <span style='color: green;'>```QSignalTransition```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qsignaltransition.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsignaltransition.html">QSignalTransition</a> class provides a transition based on a Qt signal.</p>
/// <p>Typically you would use the overload of <a href="http://doc.qt.io/qt-5/qstate.html#addTransition">QState::addTransition</a>() that takes a sender and signal as arguments, rather than creating <a href="http://doc.qt.io/qt-5/qsignaltransition.html">QSignalTransition</a> objects directly. <a href="http://doc.qt.io/qt-5/qsignaltransition.html">QSignalTransition</a> is part of <a href="http://doc.qt.io/qt-5/statemachine-api.html">The State Machine Framework</a>.</p>
/// <p>You can subclass <a href="http://doc.qt.io/qt-5/qsignaltransition.html">QSignalTransition</a> and reimplement <a href="http://doc.qt.io/qt-5/qsignaltransition.html#eventTest">eventTest</a>() to make a signal transition conditional; the event object passed to <a href="http://doc.qt.io/qt-5/qsignaltransition.html#eventTest">eventTest</a>() will be a <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html">QStateMachine::SignalEvent</a> object. Example:</p>
/// <pre class="cpp">
///
///   <span class="keyword">class</span> CheckedTransition : <span class="keyword">public</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsignaltransition.html#QSignalTransition">QSignalTransition</a></span>
///   {
///   <span class="keyword">public</span>:
/// &#32;     CheckedTransition(<span class="type"><a href="http://doc.qt.io/qt-5/qcheckbox.html">QCheckBox</a></span> <span class="operator">*</span>check)
/// &#32;   &#32;     : <span class="type"><a href="http://doc.qt.io/qt-5/qsignaltransition.html#QSignalTransition">QSignalTransition</a></span>(check<span class="operator">,</span> SIGNAL(stateChanged(<span class="type">int</span>))) {}
///   <span class="keyword">protected</span>:
/// &#32;     bool eventTest(<span class="type"><a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a></span> <span class="operator">*</span>e) {
/// &#32;   &#32;     <span class="keyword">if</span> (<span class="operator">!</span><span class="type"><a href="http://doc.qt.io/qt-5/qsignaltransition.html#QSignalTransition">QSignalTransition</a></span><span class="operator">::</span>eventTest(e))
/// &#32;   &#32;   &#32;     <span class="keyword">return</span> <span class="keyword">false</span>;
/// &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a></span><span class="operator">::</span>SignalEvent <span class="operator">*</span>se <span class="operator">=</span> <span class="keyword">static_cast</span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a></span><span class="operator">::</span>SignalEvent<span class="operator">*</span><span class="operator">&gt;</span>(e);
/// &#32;   &#32;     <span class="keyword">return</span> (se<span class="operator">-</span><span class="operator">&gt;</span>arguments()<span class="operator">.</span>at(<span class="number">0</span>)<span class="operator">.</span>toInt() <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>Checked);
/// &#32;     }
///   };
///
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qcheckbox.html">QCheckBox</a></span> <span class="operator">*</span>check <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcheckbox.html">QCheckBox</a></span>();
///   check<span class="operator">-</span><span class="operator">&gt;</span>setTristate(<span class="keyword">true</span>);
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span> <span class="operator">*</span>s1 <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span>();
///   <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span> <span class="operator">*</span>s2 <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span>();
///   CheckedTransition <span class="operator">*</span>t1 <span class="operator">=</span> <span class="keyword">new</span> CheckedTransition(check);
///   t1<span class="operator">-</span><span class="operator">&gt;</span>setTargetState(s2);
///   s1<span class="operator">-</span><span class="operator">&gt;</span>addTransition(t1);
///
/// </pre></div>
#[repr(C)]
pub struct QSignalTransition {
    _unused: u8,
}
impl QSignalTransition {
    /// <p>This signal is emitted when the <a href="http://doc.qt.io/qt-5/qsignaltransition.html#senderObject-prop">senderObject</a> property is changed.</p>
    ///
    /// Returns a built-in Qt signal `QSignalTransition::senderObjectChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignaltransition.html#senderObjectChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the <a href="http://doc.qt.io/qt-5/qsignaltransition.html#senderObject-prop">senderObject</a> property is changed.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>Note:</b> Notifier signal for property <a href="http://doc.qt.io/qt-5/qsignaltransition.html#senderObject-prop">senderObject</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qsignaltransition.html#senderObject-prop">QSignalTransition::senderObject</a>.</p></div>
    pub fn sender_object_changed(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2senderObjectChanged()\0"),
            )
        }
    }

    /// <p>This signal is emitted when the signal property is changed.</p>
    ///
    /// Returns a built-in Qt signal `QSignalTransition::signalChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignaltransition.html#signalChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the signal property is changed.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>Note:</b> Notifier signal for property <a href="http://doc.qt.io/qt-5/qsignaltransition.html#signal-prop">signal</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qsignaltransition.html#signal-prop">QSignalTransition::signal</a>.</p></div>
    pub fn signal_changed(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2signalChanged()\0"),
            )
        }
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QSignalTransition::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSignalTransition_metaObject(
            self as *const crate::QSignalTransition,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs a new signal transition with the given <i>sourceState</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSignalTransition::QSignalTransition(QState* sourceState = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignaltransition.html#QSignalTransition">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new signal transition with the given <i>sourceState</i>.</p></div>
    pub unsafe fn new_1a(
        source_state: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QState>>,
    ) -> ::cpp_utils::CppBox<crate::QSignalTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSignalTransition_QSignalTransition(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QState>>::cast_into(source_state)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new signal transition associated with the given <i>signal</i> of the given <i>sender</i>, and with the given <i>sourceState</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSignalTransition::QSignalTransition(const QObject* sender, const char* signal, QState* sourceState = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignaltransition.html#QSignalTransition-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new signal transition associated with the given <i>signal</i> of the given <i>sender</i>, and with the given <i>sourceState</i>.</p></div>
    pub unsafe fn new_3a(
        sender: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        signal: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        source_state: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QState>>,
    ) -> ::cpp_utils::CppBox<crate::QSignalTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSignalTransition_QSignalTransition1(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(sender)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(signal)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QState>>::cast_into(source_state)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qsignaltransition.html">QSignalTransition</a> class provides a transition based on a Qt signal.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSignalTransition::QSignalTransition()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignaltransition.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsignaltransition.html">QSignalTransition</a> class provides a transition based on a Qt signal.</p>
    /// <p>Typically you would use the overload of <a href="http://doc.qt.io/qt-5/qstate.html#addTransition">QState::addTransition</a>() that takes a sender and signal as arguments, rather than creating <a href="http://doc.qt.io/qt-5/qsignaltransition.html">QSignalTransition</a> objects directly. <a href="http://doc.qt.io/qt-5/qsignaltransition.html">QSignalTransition</a> is part of <a href="http://doc.qt.io/qt-5/statemachine-api.html">The State Machine Framework</a>.</p>
    /// <p>You can subclass <a href="http://doc.qt.io/qt-5/qsignaltransition.html">QSignalTransition</a> and reimplement <a href="http://doc.qt.io/qt-5/qsignaltransition.html#eventTest">eventTest</a>() to make a signal transition conditional; the event object passed to <a href="http://doc.qt.io/qt-5/qsignaltransition.html#eventTest">eventTest</a>() will be a <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html">QStateMachine::SignalEvent</a> object. Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">class</span> CheckedTransition : <span class="keyword">public</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsignaltransition.html#QSignalTransition">QSignalTransition</a></span>
    ///   {
    ///   <span class="keyword">public</span>:
    /// &#32;     CheckedTransition(<span class="type"><a href="http://doc.qt.io/qt-5/qcheckbox.html">QCheckBox</a></span> <span class="operator">*</span>check)
    /// &#32;   &#32;     : <span class="type"><a href="http://doc.qt.io/qt-5/qsignaltransition.html#QSignalTransition">QSignalTransition</a></span>(check<span class="operator">,</span> SIGNAL(stateChanged(<span class="type">int</span>))) {}
    ///   <span class="keyword">protected</span>:
    /// &#32;     bool eventTest(<span class="type"><a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a></span> <span class="operator">*</span>e) {
    /// &#32;   &#32;     <span class="keyword">if</span> (<span class="operator">!</span><span class="type"><a href="http://doc.qt.io/qt-5/qsignaltransition.html#QSignalTransition">QSignalTransition</a></span><span class="operator">::</span>eventTest(e))
    /// &#32;   &#32;   &#32;     <span class="keyword">return</span> <span class="keyword">false</span>;
    /// &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a></span><span class="operator">::</span>SignalEvent <span class="operator">*</span>se <span class="operator">=</span> <span class="keyword">static_cast</span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a></span><span class="operator">::</span>SignalEvent<span class="operator">*</span><span class="operator">&gt;</span>(e);
    /// &#32;   &#32;     <span class="keyword">return</span> (se<span class="operator">-</span><span class="operator">&gt;</span>arguments()<span class="operator">.</span>at(<span class="number">0</span>)<span class="operator">.</span>toInt() <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qt.html">Qt</a></span><span class="operator">::</span>Checked);
    /// &#32;     }
    ///   };
    ///
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qcheckbox.html">QCheckBox</a></span> <span class="operator">*</span>check <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcheckbox.html">QCheckBox</a></span>();
    ///   check<span class="operator">-</span><span class="operator">&gt;</span>setTristate(<span class="keyword">true</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span> <span class="operator">*</span>s1 <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span>();
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span> <span class="operator">*</span>s2 <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html">QState</a></span>();
    ///   CheckedTransition <span class="operator">*</span>t1 <span class="operator">=</span> <span class="keyword">new</span> CheckedTransition(check);
    ///   t1<span class="operator">-</span><span class="operator">&gt;</span>setTargetState(s2);
    ///   s1<span class="operator">-</span><span class="operator">&gt;</span>addTransition(t1);
    ///
    /// </pre></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QSignalTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSignalTransition_QSignalTransition2();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new signal transition associated with the given <i>signal</i> of the given <i>sender</i>, and with the given <i>sourceState</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSignalTransition::QSignalTransition(const QObject* sender, const char* signal)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignaltransition.html#QSignalTransition-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new signal transition associated with the given <i>signal</i> of the given <i>sender</i>, and with the given <i>sourceState</i>.</p></div>
    pub unsafe fn new_2a(
        sender: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        signal: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QSignalTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSignalTransition_QSignalTransition3(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(sender)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(signal)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QSignalTransition::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QSignalTransition_qt_metacall(
            self as *mut crate::QSignalTransition,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QSignalTransition::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSignalTransition_qt_metacast(
            self as *mut crate::QSignalTransition,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This property holds the sender object that this signal transition is associated with</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QObject* QSignalTransition::senderObject() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignaltransition.html#senderObject-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the sender object that this signal transition is associated with</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QObject *</td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qsignaltransition.html#senderObject">senderObject</a></b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qsignaltransition.html#setSenderObject">setSenderObject</a></b></span>(const QObject *<i>sender</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qsignaltransition.html#senderObjectChanged">senderObjectChanged</a></b></span>()</td><td class="memItemRight bottomAlign">[see note below]</td></tr>
    /// </tbody></table></div>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p></div>
    pub unsafe fn sender_object(&self) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSignalTransition_senderObject(
            self as *const crate::QSignalTransition,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Sets the <i>sender</i> object associated with this signal transition.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSignalTransition::setSenderObject(const QObject* sender)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignaltransition.html#setSenderObject">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the <i>sender</i> object associated with this signal transition.</p>
    /// <p><b>Note:</b> Setter function for property <a href="http://doc.qt.io/qt-5/qsignaltransition.html#senderObject-prop">senderObject</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qsignaltransition.html#senderObject">senderObject</a>().</p></div>
    pub unsafe fn set_sender_object(
        &mut self,
        sender: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSignalTransition_setSenderObject(
            self as *mut crate::QSignalTransition,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(sender)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the <i>signal</i> associated with this signal transition.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSignalTransition::setSignal(const QByteArray& signal)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignaltransition.html#setSignal">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the <i>signal</i> associated with this signal transition.</p>
    /// <p><b>Note:</b> Setter function for property <a href="http://doc.qt.io/qt-5/qsignaltransition.html#signal-prop">signal</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qsignaltransition.html#signal">signal</a>().</p></div>
    pub unsafe fn set_signal(
        &mut self,
        signal: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSignalTransition_setSignal(
            self as *mut crate::QSignalTransition,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(signal)
                .as_raw_ptr(),
        )
    }

    /// <p>This property holds the signal that this signal transition is associated with</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QSignalTransition::signal() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignaltransition.html#signal-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the signal that this signal transition is associated with</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QByteArray </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qsignaltransition.html#signal">signal</a></b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qsignaltransition.html#setSignal">setSignal</a></b></span>(const QByteArray &amp;<i>signal</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qsignaltransition.html#signalChanged">signalChanged</a></b></span>()</td><td class="memItemRight bottomAlign">[see note below]</td></tr>
    /// </tbody></table></div>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p></div>
    pub unsafe fn signal(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSignalTransition_signal(
            self as *const crate::QSignalTransition,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSignalTransition_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QSignalTransition::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSignalTransition_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QSignalTransition::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSignalTransition_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_socket_notifier {
    //! C++ type: <span style='color: green;'>```QSocketNotifier```</span>

    /// <p>This enum describes the various types of events that a socket notifier can recognize. The type must be specified when constructing the socket notifier.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QSocketNotifier::Type```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#Type-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the various types of events that a socket notifier can recognize. The type must be specified when constructing the socket notifier.</p>
    /// <p>Note that if you need to monitor both reads and writes for the same file descriptor, you must create two socket notifiers. Note also that it is not possible to install two socket notifiers of the same type (Read, Write, Exception) on the same socket.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsocketnotifier.html#QSocketNotifier">QSocketNotifier</a>() and <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#type">type</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Type(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Type {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Type(value)
        }
    }

    impl From<Type> for ::std::os::raw::c_int {
        fn from(value: Type) -> Self {
            value.0
        }
    }

    impl Type {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Type {
        /// There is data to be read. (C++ enum variant: <span style='color: green;'>```Read = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const Read: crate::q_socket_notifier::Type = crate::q_socket_notifier::Type(0);
        /// Data can be written. (C++ enum variant: <span style='color: green;'>```Write = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Write: crate::q_socket_notifier::Type = crate::q_socket_notifier::Type(1);
        /// An exception has occurred. We recommend against using this. (C++ enum variant: <span style='color: green;'>```Exception = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const Exception: crate::q_socket_notifier::Type = crate::q_socket_notifier::Type(2);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qsocketnotifier.html">QSocketNotifier</a> class provides support for monitoring activity on a file descriptor.</p>
///
/// C++ class: <span style='color: green;'>```QSocketNotifier```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qsocketnotifier.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsocketnotifier.html">QSocketNotifier</a> class provides support for monitoring activity on a file descriptor.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qsocketnotifier.html">QSocketNotifier</a> makes it possible to integrate Qt's event loop with other event loops based on file descriptors. File descriptor action is detected in Qt's main event loop (<a href="http://doc.qt.io/qt-5/qcoreapplication.html#exec">QCoreApplication::exec</a>()).</p>
/// <a name="write-notifiers"></a><p>Once you have opened a device using a low-level (usually platform-specific) API, you can create a socket notifier to monitor the file descriptor. The socket notifier is enabled by default, i.e. it emits the <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#activated">activated</a>() signal whenever a socket event corresponding to its type occurs. Connect the <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#activated">activated</a>() signal to the slot you want to be called when an event corresponding to your socket notifier's type occurs.</p>
/// <p>There are three types of socket notifiers: read, write, and exception. The type is described by the <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#Type-enum">Type</a> enum, and must be specified when constructing the socket notifier. After construction it can be determined using the <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#type">type</a>() function. Note that if you need to monitor both reads and writes for the same file descriptor, you must create two socket notifiers. Note also that it is not possible to install two socket notifiers of the same type (<a href="http://doc.qt.io/qt-5/qsocketnotifier.html#Type-enum">Read</a>, <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#Type-enum">Write</a>, <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#Type-enum">Exception</a>) on the same socket.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#setEnabled">setEnabled</a>() function allows you to disable as well as enable the socket notifier. It is generally advisable to explicitly enable or disable the socket notifier, especially for write notifiers. A disabled notifier ignores socket events (the same effect as not creating the socket notifier). Use the <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#isEnabled">isEnabled</a>() function to determine the notifier's current status.</p>
/// <p>Finally, you can use the <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#socket">socket</a>() function to retrieve the socket identifier. Although the class is called <a href="http://doc.qt.io/qt-5/qsocketnotifier.html">QSocketNotifier</a>, it is normally used for other types of devices than sockets. <a href="http://doc.qt.io/qt-5/qtcpsocket.html">QTcpSocket</a> and <a href="http://doc.qt.io/qt-5/qudpsocket.html">QUdpSocket</a> provide notification through signals, so there is normally no need to use a <a href="http://doc.qt.io/qt-5/qsocketnotifier.html">QSocketNotifier</a> on them.</p></div>
#[repr(C)]
pub struct QSocketNotifier {
    _unused: u8,
}
impl QSocketNotifier {
    /// <p>If <i>enable</i> is true, the notifier is enabled; otherwise the notifier is disabled.</p>
    ///
    /// Returns a built-in Qt slot `QSocketNotifier::setEnabled` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#setEnabled">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If <i>enable</i> is true, the notifier is enabled; otherwise the notifier is disabled.</p>
    /// <p>The notifier is enabled by default, i.e. it emits the <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#activated">activated</a>() signal whenever a socket event corresponding to its <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#type">type</a> occurs. If it is disabled, it ignores socket events (the same effect as not creating the socket notifier).</p>
    /// <p>Write notifiers should normally be disabled immediately after the <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#activated">activated</a>() signal has been emitted</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsocketnotifier.html#isEnabled">isEnabled</a>() and <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#activated">activated</a>().</p></div>
    pub fn slot_set_enabled(&self) -> crate::Receiver<(bool,)> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1setEnabled(bool)\0"),
            )
        }
    }

    /// <p>This signal is emitted whenever the socket notifier is enabled and a socket event corresponding to its <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#Type-enum">type</a> occurs.</p>
    ///
    /// Returns a built-in Qt signal `QSocketNotifier::activated` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#activated">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted whenever the socket notifier is enabled and a socket event corresponding to its <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#Type-enum">type</a> occurs.</p>
    /// <p>The socket identifier is passed in the <i>socket</i> parameter.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsocketnotifier.html#type">type</a>() and <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#socket">socket</a>().</p></div>
    pub fn activated(&self) -> crate::Signal<(::std::os::raw::c_int,)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2activated(int)\0"),
            )
        }
    }

    /// <p>Returns <code>true</code> if the notifier is enabled; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSocketNotifier::isEnabled() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#isEnabled">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the notifier is enabled; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsocketnotifier.html#setEnabled">setEnabled</a>().</p></div>
    pub unsafe fn is_enabled(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSocketNotifier_isEnabled(
            self as *const crate::QSocketNotifier,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QSocketNotifier::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSocketNotifier_metaObject(
            self as *const crate::QSocketNotifier,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs a socket notifier with the given <i>parent</i>. It enables the <i>socket</i>, and watches for events of the given <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSocketNotifier::QSocketNotifier(qintptr socket, QSocketNotifier::Type arg2, QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#QSocketNotifier">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a socket notifier with the given <i>parent</i>. It enables the <i>socket</i>, and watches for events of the given <i>type</i>.</p>
    /// <p>It is generally advisable to explicitly enable or disable the socket notifier, especially for write notifiers.</p>
    /// <p><b>Note for Windows users:</b> The socket passed to <a href="http://doc.qt.io/qt-5/qsocketnotifier.html">QSocketNotifier</a> will become non-blocking, even if it was created as a blocking socket.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsocketnotifier.html#setEnabled">setEnabled</a>() and <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#isEnabled">isEnabled</a>().</p></div>
    pub unsafe fn new_3a(
        socket: isize,
        arg2: crate::q_socket_notifier::Type,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QSocketNotifier> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSocketNotifier_QSocketNotifier(
            socket,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a socket notifier with the given <i>parent</i>. It enables the <i>socket</i>, and watches for events of the given <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSocketNotifier::QSocketNotifier(qintptr socket, QSocketNotifier::Type arg2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#QSocketNotifier">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a socket notifier with the given <i>parent</i>. It enables the <i>socket</i>, and watches for events of the given <i>type</i>.</p>
    /// <p>It is generally advisable to explicitly enable or disable the socket notifier, especially for write notifiers.</p>
    /// <p><b>Note for Windows users:</b> The socket passed to <a href="http://doc.qt.io/qt-5/qsocketnotifier.html">QSocketNotifier</a> will become non-blocking, even if it was created as a blocking socket.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsocketnotifier.html#setEnabled">setEnabled</a>() and <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#isEnabled">isEnabled</a>().</p></div>
    pub unsafe fn new_2a(
        socket: isize,
        arg2: crate::q_socket_notifier::Type,
    ) -> ::cpp_utils::CppBox<crate::QSocketNotifier> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QSocketNotifier_QSocketNotifier2(socket, arg2);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QSocketNotifier::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QSocketNotifier_qt_metacall(
            self as *mut crate::QSocketNotifier,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QSocketNotifier::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSocketNotifier_qt_metacast(
            self as *mut crate::QSocketNotifier,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>If <i>enable</i> is true, the notifier is enabled; otherwise the notifier is disabled.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QSocketNotifier::setEnabled(bool arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#setEnabled">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If <i>enable</i> is true, the notifier is enabled; otherwise the notifier is disabled.</p>
    /// <p>The notifier is enabled by default, i.e. it emits the <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#activated">activated</a>() signal whenever a socket event corresponding to its <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#type">type</a> occurs. If it is disabled, it ignores socket events (the same effect as not creating the socket notifier).</p>
    /// <p>Write notifiers should normally be disabled immediately after the <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#activated">activated</a>() signal has been emitted</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsocketnotifier.html#isEnabled">isEnabled</a>() and <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#activated">activated</a>().</p></div>
    pub unsafe fn set_enabled(&mut self, arg1: bool) {
        crate::__ffi::ctr_qt_core_ffi_QSocketNotifier_setEnabled(
            self as *mut crate::QSocketNotifier,
            arg1,
        )
    }

    /// <p>Returns the socket identifier specified to the constructor.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qintptr QSocketNotifier::socket() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#socket">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the socket identifier specified to the constructor.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsocketnotifier.html#type">type</a>().</p></div>
    pub unsafe fn socket(&self) -> isize {
        crate::__ffi::ctr_qt_core_ffi_QSocketNotifier_socket(self as *const crate::QSocketNotifier)
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSocketNotifier_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QSocketNotifier::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSocketNotifier_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QSocketNotifier::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSocketNotifier_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the socket event type specified to the constructor.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSocketNotifier::Type QSocketNotifier::type() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#type">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the socket event type specified to the constructor.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsocketnotifier.html#socket">socket</a>().</p></div>
    pub unsafe fn type_(&self) -> crate::q_socket_notifier::Type {
        crate::__ffi::ctr_qt_core_ffi_QSocketNotifier_type(self as *const crate::QSocketNotifier)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html">QSortFilterProxyModel</a> class provides support for sorting and filtering data passed between another model and a view.</p>
///
/// C++ class: <span style='color: green;'>```QSortFilterProxyModel```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html">QSortFilterProxyModel</a> class provides support for sorting and filtering data passed between another model and a view.</p>
/// <p><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html">QSortFilterProxyModel</a> can be used for sorting items, filtering out items, or both. The model transforms the structure of a source model by mapping the model indexes it supplies to new indexes, corresponding to different locations, for views to use. This approach allows a given source model to be restructured as far as views are concerned without requiring any transformations on the underlying data, and without duplicating the data in memory.</p>
/// <p>Let's assume that we want to sort and filter the items provided by a custom model. The code to set up the model and the view, <i>without</i> sorting and filtering, would look like this:</p>
/// <pre class="cpp">
///
/// &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qtreeview.html">QTreeView</a></span> <span class="operator">*</span>treeView <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtreeview.html">QTreeView</a></span>;
/// &#32;   &#32;     MyItemModel <span class="operator">*</span>model <span class="operator">=</span> <span class="keyword">new</span> MyItemModel(<span class="keyword">this</span>);
///
/// &#32;   &#32;     treeView<span class="operator">-</span><span class="operator">&gt;</span>setModel(model);
///
/// </pre>
/// <p>To add sorting and filtering support to <code>MyItemModel</code>, we need to create a <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html">QSortFilterProxyModel</a>, call <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setSourceModel">setSourceModel</a>() with the <code>MyItemModel</code> as argument, and install the <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html">QSortFilterProxyModel</a> on the view:</p>
/// <pre class="cpp">
///
/// &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qtreeview.html">QTreeView</a></span> <span class="operator">*</span>treeView <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtreeview.html">QTreeView</a></span>;
/// &#32;   &#32;     MyItemModel <span class="operator">*</span>sourceModel <span class="operator">=</span> <span class="keyword">new</span> MyItemModel(<span class="keyword">this</span>);
/// &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#QSortFilterProxyModel">QSortFilterProxyModel</a></span> <span class="operator">*</span>proxyModel <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#QSortFilterProxyModel">QSortFilterProxyModel</a></span>(<span class="keyword">this</span>);
///
/// &#32;   &#32;     proxyModel<span class="operator">-</span><span class="operator">&gt;</span>setSourceModel(sourceModel);
/// &#32;   &#32;     treeView<span class="operator">-</span><span class="operator">&gt;</span>setModel(proxyModel);
///
/// </pre>
/// <p>At this point, neither sorting nor filtering is enabled; the original data is displayed in the view. Any changes made through the <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html">QSortFilterProxyModel</a> are applied to the original model.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html">QSortFilterProxyModel</a> acts as a wrapper for the original model. If you need to convert source <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a>es to sorted/filtered model indexes or vice versa, use <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#mapToSource">mapToSource</a>(), <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#mapFromSource">mapFromSource</a>(), <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#mapSelectionToSource">mapSelectionToSource</a>(), and <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#mapSelectionFromSource">mapSelectionFromSource</a>().</p>
/// <p><b>Note: </b>By default, the model dynamically re-sorts and re-filters data whenever the original model changes. This behavior can be changed by setting the <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#dynamicSortFilter-prop">dynamicSortFilter</a> property.</p><p>The <a href="http://doc.qt.io/qt-5/qtwidgets-itemviews-basicsortfiltermodel-example.html">Basic Sort/Filter Model</a> and <a href="http://doc.qt.io/qt-5/qtwidgets-itemviews-customsortfiltermodel-example.html">Custom Sort/Filter Model</a> examples illustrate how to use <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html">QSortFilterProxyModel</a> to perform basic sorting and filtering and how to subclass it to implement custom behavior.</p>
/// <a name="sorting"></a></div>
#[repr(C)]
pub struct QSortFilterProxyModel {
    _unused: u8,
}
impl QSortFilterProxyModel {
    /// <p>This property holds the <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> used to filter the contents of the source model</p>
    ///
    /// Returns a built-in Qt slot `QSortFilterProxyModel::setFilterRegExp` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterRegExp-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> used to filter the contents of the source model</p>
    /// <p>Setting this property overwrites the current <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterCaseSensitivity-prop">filterCaseSensitivity</a>. By default, the <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> is an empty string matching all contents.</p>
    /// <p>If no <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> or an empty string is set, everything in the source model will be accepted.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QRegExp </td><td class="memItemRight bottomAlign"><span class="name"><b>filterRegExp</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setFilterRegExp</b></span>(const QRegExp &amp;<i>regExp</i>)</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterRegExp-1">setFilterRegExp</a></b></span>(const QString &amp;<i>pattern</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterCaseSensitivity-prop">filterCaseSensitivity</a>, <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterWildcard">setFilterWildcard</a>(), and <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterFixedString">setFilterFixedString</a>().</p></div>
    pub fn slot_set_filter_reg_exp(&self) -> crate::Receiver<(*const crate::QString,)> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"1setFilterRegExp(const QString&)\0",
                ),
            )
        }
    }

    /// <p>Sets the wildcard expression used to filter the contents of the source model to the given <i>pattern</i>.</p>
    ///
    /// Returns a built-in Qt slot `QSortFilterProxyModel::setFilterWildcard` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterWildcard">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the wildcard expression used to filter the contents of the source model to the given <i>pattern</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterCaseSensitivity-prop">setFilterCaseSensitivity</a>(), <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterRegExp-prop">setFilterRegExp</a>(), <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterFixedString">setFilterFixedString</a>(), and <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterRegExp-prop">filterRegExp</a>().</p></div>
    pub fn slot_set_filter_wildcard(&self) -> crate::Receiver<(*const crate::QString,)> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"1setFilterWildcard(const QString&)\0",
                ),
            )
        }
    }

    /// <p>Sets the fixed string used to filter the contents of the source model to the given <i>pattern</i>.</p>
    ///
    /// Returns a built-in Qt slot `QSortFilterProxyModel::setFilterFixedString` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterFixedString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the fixed string used to filter the contents of the source model to the given <i>pattern</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterCaseSensitivity-prop">setFilterCaseSensitivity</a>(), <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterRegExp-prop">setFilterRegExp</a>(), <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterWildcard">setFilterWildcard</a>(), and <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterRegExp-prop">filterRegExp</a>().</p></div>
    pub fn slot_set_filter_fixed_string(&self) -> crate::Receiver<(*const crate::QString,)> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"1setFilterFixedString(const QString&)\0",
                ),
            )
        }
    }

    /// <p>This function is obsolete. Use <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#invalidate">invalidate</a>() instead.</p>
    ///
    /// Returns a built-in Qt slot `QSortFilterProxyModel::clear` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel-obsolete.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is obsolete. Use <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#invalidate">invalidate</a>() instead.</p></div>
    pub fn slot_clear(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1clear()\0"),
            )
        }
    }

    /// <p>Invalidates the current sorting and filtering.</p>
    ///
    /// Returns a built-in Qt slot `QSortFilterProxyModel::invalidate` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#invalidate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Invalidates the current sorting and filtering.</p>
    /// <p>This function was introduced in  Qt 4.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#invalidateFilter">invalidateFilter</a>().</p></div>
    pub fn slot_invalidate(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1invalidate()\0"),
            )
        }
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#buddy">QAbstractItemModel::buddy</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QSortFilterProxyModel::buddy(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#buddy">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#buddy">QAbstractItemModel::buddy</a>().</p></div>
    pub unsafe fn buddy(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_buddy(
            self as *const crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#canFetchMore">QAbstractItemModel::canFetchMore</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QSortFilterProxyModel::canFetchMore(const QModelIndex& parent) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#canFetchMore">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#canFetchMore">QAbstractItemModel::canFetchMore</a>().</p></div>
    pub unsafe fn can_fetch_more(
        &self,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_canFetchMore(
            self as *const crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is obsolete. Use <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#invalidate">invalidate</a>() instead.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QSortFilterProxyModel::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel-obsolete.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is obsolete. Use <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#invalidate">invalidate</a>() instead.</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_clear(
            self as *mut crate::QSortFilterProxyModel,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnCount">QAbstractItemModel::columnCount</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual int QSortFilterProxyModel::columnCount(const QModelIndex& parent = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#columnCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnCount">QAbstractItemModel::columnCount</a>().</p></div>
    pub unsafe fn column_count_1a(
        &self,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_columnCount(
            self as *const crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnCount">QAbstractItemModel::columnCount</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual int QSortFilterProxyModel::columnCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#columnCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnCount">QAbstractItemModel::columnCount</a>().</p></div>
    pub unsafe fn column_count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_columnCount1(
            self as *const crate::QSortFilterProxyModel,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">QAbstractItemModel::data</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QVariant QSortFilterProxyModel::data(const QModelIndex& index, int role = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">QAbstractItemModel::data</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setData">setData</a>().</p></div>
    pub unsafe fn data_2a(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        role: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_data(
            self as *const crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
            role,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">QAbstractItemModel::data</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QVariant QSortFilterProxyModel::data(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">QAbstractItemModel::data</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setData">setData</a>().</p></div>
    pub unsafe fn data_1a(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_data1(
            self as *const crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dropMimeData">QAbstractItemModel::dropMimeData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QSortFilterProxyModel::dropMimeData(const QMimeData* data, Qt::DropAction action, int row, int column, const QModelIndex& parent)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#dropMimeData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dropMimeData">QAbstractItemModel::dropMimeData</a>().</p></div>
    pub unsafe fn drop_mime_data(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QMimeData>>,
        action: crate::DropAction,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_dropMimeData(
            self as *mut crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QMimeData>>::cast_into(data)
                .as_raw_ptr(),
            action,
            row,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>This property holds whether the proxy model is dynamically sorted and filtered whenever the contents of the source model change</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSortFilterProxyModel::dynamicSortFilter() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#dynamicSortFilter-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds whether the proxy model is dynamically sorted and filtered whenever the contents of the source model change</p>
    /// <p>Note that you should not update the source model through the proxy model when dynamicSortFilter is true. For instance, if you set the proxy model on a <a href="http://doc.qt.io/qt-5/qcombobox.html">QComboBox</a>, then using functions that update the model, e.g., <a href="http://doc.qt.io/qt-5/qcombobox.html#addItem-1">addItem()</a>, will not work as expected. An alternative is to set dynamicSortFilter to false and call <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#sort">sort()</a> after adding items to the <a href="http://doc.qt.io/qt-5/qcombobox.html">QComboBox</a>.</p>
    /// <p>The default value is true.</p>
    /// <p>This property was introduced in  Qt 4.2.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> bool </td><td class="memItemRight bottomAlign"><span class="name"><b>dynamicSortFilter</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setDynamicSortFilter</b></span>(bool <i>enable</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn dynamic_sort_filter(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_dynamicSortFilter(
            self as *const crate::QSortFilterProxyModel,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#fetchMore">QAbstractItemModel::fetchMore</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual void QSortFilterProxyModel::fetchMore(const QModelIndex& parent)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#fetchMore">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#fetchMore">QAbstractItemModel::fetchMore</a>().</p></div>
    pub unsafe fn fetch_more(
        &mut self,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_fetchMore(
            self as *mut crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>This property holds the case sensitivity of the <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> pattern used to filter the contents of the source model</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```Qt::CaseSensitivity QSortFilterProxyModel::filterCaseSensitivity() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterCaseSensitivity-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the case sensitivity of the <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> pattern used to filter the contents of the source model</p>
    /// <p>By default, the filter is case sensitive.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> Qt::CaseSensitivity </td><td class="memItemRight bottomAlign"><span class="name"><b>filterCaseSensitivity</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setFilterCaseSensitivity</b></span>(Qt::CaseSensitivity <i>cs</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterRegExp-prop">filterRegExp</a> and <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#sortCaseSensitivity-prop">sortCaseSensitivity</a>.</p></div>
    pub unsafe fn filter_case_sensitivity(&self) -> crate::CaseSensitivity {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_filterCaseSensitivity(
            self as *const crate::QSortFilterProxyModel,
        )
    }

    /// <p>This property holds the column where the key used to filter the contents of the source model is read from.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QSortFilterProxyModel::filterKeyColumn() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterKeyColumn-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the column where the key used to filter the contents of the source model is read from.</p>
    /// <p>The default value is 0. If the value is -1, the keys will be read from all columns.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>filterKeyColumn</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setFilterKeyColumn</b></span>(int <i>column</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn filter_key_column(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_filterKeyColumn(
            self as *const crate::QSortFilterProxyModel,
        )
    }

    /// <p>This property holds the <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> used to filter the contents of the source model</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegExp QSortFilterProxyModel::filterRegExp() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterRegExp-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> used to filter the contents of the source model</p>
    /// <p>Setting this property overwrites the current <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterCaseSensitivity-prop">filterCaseSensitivity</a>. By default, the <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> is an empty string matching all contents.</p>
    /// <p>If no <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> or an empty string is set, everything in the source model will be accepted.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QRegExp </td><td class="memItemRight bottomAlign"><span class="name"><b>filterRegExp</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setFilterRegExp</b></span>(const QRegExp &amp;<i>regExp</i>)</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterRegExp-1">setFilterRegExp</a></b></span>(const QString &amp;<i>pattern</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterCaseSensitivity-prop">filterCaseSensitivity</a>, <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterWildcard">setFilterWildcard</a>(), and <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterFixedString">setFilterFixedString</a>().</p></div>
    pub unsafe fn filter_reg_exp(&self) -> ::cpp_utils::CppBox<crate::QRegExp> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_filterRegExp(
            self as *const crate::QSortFilterProxyModel,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This property holds the <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> used to filter the contents of the source model</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpression QSortFilterProxyModel::filterRegularExpression() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterRegularExpression-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> used to filter the contents of the source model</p>
    /// <p>Setting this property overwrites the current <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterCaseSensitivity-prop">filterCaseSensitivity</a>. By default, the <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> is an empty string matching all contents.</p>
    /// <p>If no <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> or an empty string is set, everything in the source model will be accepted.</p>
    /// <p>This property was introduced in  Qt 5.12.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QRegularExpression </td><td class="memItemRight bottomAlign"><span class="name"><b>filterRegularExpression</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterRegularExpression">setFilterRegularExpression</a></b></span>(const QString &amp;<i>pattern</i>)</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setFilterRegularExpression</b></span>(const QRegularExpression &amp;<i>regularExpression</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterCaseSensitivity-prop">filterCaseSensitivity</a>, <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterWildcard">setFilterWildcard</a>(), and <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterFixedString">setFilterFixedString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn filter_regular_expression(
        &self,
    ) -> ::cpp_utils::CppBox<crate::QRegularExpression> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_filterRegularExpression(
                self as *const crate::QSortFilterProxyModel,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This property holds the item role that is used to query the source model's data when filtering items</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QSortFilterProxyModel::filterRole() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterRole-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the item role that is used to query the source model's data when filtering items</p>
    /// <p>The default value is <a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::DisplayRole</a>.</p>
    /// <p>This property was introduced in  Qt 4.2.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>filterRole</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setFilterRole</b></span>(int <i>role</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterAcceptsRow">filterAcceptsRow</a>().</p></div>
    pub unsafe fn filter_role(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_filterRole(
            self as *const crate::QSortFilterProxyModel,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#flags">QAbstractItemModel::flags</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QFlags<Qt::ItemFlag> QSortFilterProxyModel::flags(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#flags">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#flags">QAbstractItemModel::flags</a>().</p></div>
    pub unsafe fn flags(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> crate::QFlags<crate::ItemFlag> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_flags(
            self as *const crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        );
        crate::QFlags::from(ffi_result)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#hasChildren">QAbstractItemModel::hasChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QSortFilterProxyModel::hasChildren(const QModelIndex& parent = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#hasChildren">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#hasChildren">QAbstractItemModel::hasChildren</a>().</p></div>
    pub unsafe fn has_children_1a(
        &self,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_hasChildren(
            self as *const crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#hasChildren">QAbstractItemModel::hasChildren</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QSortFilterProxyModel::hasChildren() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#hasChildren">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#hasChildren">QAbstractItemModel::hasChildren</a>().</p></div>
    pub unsafe fn has_children_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_hasChildren1(
            self as *const crate::QSortFilterProxyModel,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerData">QAbstractItemModel::headerData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QVariant QSortFilterProxyModel::headerData(int section, Qt::Orientation orientation, int role = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#headerData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerData">QAbstractItemModel::headerData</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setHeaderData">setHeaderData</a>().</p></div>
    pub unsafe fn header_data_3a(
        &self,
        section: ::std::os::raw::c_int,
        orientation: crate::Orientation,
        role: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_headerData(
            self as *const crate::QSortFilterProxyModel,
            section,
            orientation,
            role,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerData">QAbstractItemModel::headerData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QVariant QSortFilterProxyModel::headerData(int section, Qt::Orientation orientation) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#headerData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerData">QAbstractItemModel::headerData</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setHeaderData">setHeaderData</a>().</p></div>
    pub unsafe fn header_data_2a(
        &self,
        section: ::std::os::raw::c_int,
        orientation: crate::Orientation,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_headerData1(
            self as *const crate::QSortFilterProxyModel,
            section,
            orientation,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QSortFilterProxyModel::index(int row, int column, const QModelIndex& parent = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#index">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>().</p></div>
    pub unsafe fn index_3a(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_index(
            self as *const crate::QSortFilterProxyModel,
            row,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QSortFilterProxyModel::index(int row, int column) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#index">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>().</p></div>
    pub unsafe fn index_2a(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_index1(
            self as *const crate::QSortFilterProxyModel,
            row,
            column,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumns">QAbstractItemModel::insertColumns</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QSortFilterProxyModel::insertColumns(int column, int count, const QModelIndex& parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#insertColumns">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumns">QAbstractItemModel::insertColumns</a>().</p></div>
    pub unsafe fn insert_columns_3a(
        &mut self,
        column: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_insertColumns(
            self as *mut crate::QSortFilterProxyModel,
            column,
            count,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumns">QAbstractItemModel::insertColumns</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QSortFilterProxyModel::insertColumns(int column, int count)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#insertColumns">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumns">QAbstractItemModel::insertColumns</a>().</p></div>
    pub unsafe fn insert_columns_2a(
        &mut self,
        column: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_insertColumns1(
            self as *mut crate::QSortFilterProxyModel,
            column,
            count,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">QAbstractItemModel::insertRows</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QSortFilterProxyModel::insertRows(int row, int count, const QModelIndex& parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#insertRows">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">QAbstractItemModel::insertRows</a>().</p></div>
    pub unsafe fn insert_rows_3a(
        &mut self,
        row: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_insertRows(
            self as *mut crate::QSortFilterProxyModel,
            row,
            count,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">QAbstractItemModel::insertRows</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QSortFilterProxyModel::insertRows(int row, int count)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#insertRows">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">QAbstractItemModel::insertRows</a>().</p></div>
    pub unsafe fn insert_rows_2a(
        &mut self,
        row: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_insertRows1(
            self as *mut crate::QSortFilterProxyModel,
            row,
            count,
        )
    }

    /// <p>Invalidates the current sorting and filtering.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QSortFilterProxyModel::invalidate()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#invalidate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Invalidates the current sorting and filtering.</p>
    /// <p>This function was introduced in  Qt 4.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#invalidateFilter">invalidateFilter</a>().</p></div>
    pub unsafe fn invalidate(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_invalidate(
            self as *mut crate::QSortFilterProxyModel,
        )
    }

    /// <p>This property holds whether the filter to be applied recursively on children, and for any matching child, its parents will be visible as well.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSortFilterProxyModel::isRecursiveFilteringEnabled() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#recursiveFilteringEnabled-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds whether the filter to be applied recursively on children, and for any matching child, its parents will be visible as well.</p>
    /// <p>The default value is false.</p>
    /// <p>This property was introduced in  Qt 5.10.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> bool </td><td class="memItemRight bottomAlign"><span class="name"><b>isRecursiveFilteringEnabled</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setRecursiveFilteringEnabled</b></span>(bool <i>recursive</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterAcceptsRow">filterAcceptsRow</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn is_recursive_filtering_enabled(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_isRecursiveFilteringEnabled(
            self as *const crate::QSortFilterProxyModel,
        )
    }

    /// <p>This property holds the local aware setting used for comparing strings when sorting</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSortFilterProxyModel::isSortLocaleAware() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#isSortLocaleAware-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the local aware setting used for comparing strings when sorting</p>
    /// <p>By default, sorting is not local aware.</p>
    /// <p>This property was introduced in  Qt 4.3.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> bool </td><td class="memItemRight bottomAlign"><span class="name"><b>isSortLocaleAware</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setSortLocaleAware</b></span>(bool <i>on</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#sortCaseSensitivity-prop">sortCaseSensitivity</a> and <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#lessThan">lessThan</a>().</p></div>
    pub unsafe fn is_sort_locale_aware(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_isSortLocaleAware(
            self as *const crate::QSortFilterProxyModel,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapFromSource">QAbstractProxyModel::mapFromSource</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QSortFilterProxyModel::mapFromSource(const QModelIndex& sourceIndex) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#mapFromSource">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapFromSource">QAbstractProxyModel::mapFromSource</a>().</p>
    /// <p>Returns the model index in the <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html">QSortFilterProxyModel</a> given the <i>sourceIndex</i> from the source model.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#mapToSource">mapToSource</a>().</p></div>
    pub unsafe fn map_from_source(
        &self,
        source_index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_mapFromSource(
            self as *const crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(source_index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapSelectionFromSource">QAbstractProxyModel::mapSelectionFromSource</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QItemSelection QSortFilterProxyModel::mapSelectionFromSource(const QItemSelection& sourceSelection) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#mapSelectionFromSource">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapSelectionFromSource">QAbstractProxyModel::mapSelectionFromSource</a>().</p></div>
    pub unsafe fn map_selection_from_source(
        &self,
        source_selection: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelection>>,
    ) -> ::cpp_utils::CppBox<crate::QItemSelection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_mapSelectionFromSource(
            self as *const crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelection>>::cast_into(
                source_selection,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapSelectionToSource">QAbstractProxyModel::mapSelectionToSource</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QItemSelection QSortFilterProxyModel::mapSelectionToSource(const QItemSelection& proxySelection) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#mapSelectionToSource">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapSelectionToSource">QAbstractProxyModel::mapSelectionToSource</a>().</p></div>
    pub unsafe fn map_selection_to_source(
        &self,
        proxy_selection: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelection>>,
    ) -> ::cpp_utils::CppBox<crate::QItemSelection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_mapSelectionToSource(
            self as *const crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelection>>::cast_into(
                proxy_selection,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapToSource">QAbstractProxyModel::mapToSource</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QSortFilterProxyModel::mapToSource(const QModelIndex& proxyIndex) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#mapToSource">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapToSource">QAbstractProxyModel::mapToSource</a>().</p>
    /// <p>Returns the source model index corresponding to the given <i>proxyIndex</i> from the sorting filter model.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#mapFromSource">mapFromSource</a>().</p></div>
    pub unsafe fn map_to_source(
        &self,
        proxy_index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_mapToSource(
            self as *const crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(proxy_index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#match">QAbstractItemModel::match</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QList<QModelIndex> QSortFilterProxyModel::match(const QModelIndex& start, int role, const QVariant& value, int hits = …, QFlags<Qt::MatchFlag> flags = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#match">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#match">QAbstractItemModel::match</a>().</p></div>
    pub unsafe fn match_5a(
        &self,
        start: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        role: ::std::os::raw::c_int,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
        hits: ::std::os::raw::c_int,
        flags: crate::QFlags<crate::MatchFlag>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_match(
            self as *const crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(start)
                .as_raw_ptr(),
            role,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
            hits,
            flags.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#match">QAbstractItemModel::match</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QList<QModelIndex> QSortFilterProxyModel::match(const QModelIndex& start, int role, const QVariant& value, int hits = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#match">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#match">QAbstractItemModel::match</a>().</p></div>
    pub unsafe fn match_4a(
        &self,
        start: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        role: ::std::os::raw::c_int,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
        hits: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_match1(
            self as *const crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(start)
                .as_raw_ptr(),
            role,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
            hits,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#match">QAbstractItemModel::match</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QList<QModelIndex> QSortFilterProxyModel::match(const QModelIndex& start, int role, const QVariant& value) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#match">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#match">QAbstractItemModel::match</a>().</p></div>
    pub unsafe fn match_3a(
        &self,
        start: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        role: ::std::os::raw::c_int,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_match2(
            self as *const crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(start)
                .as_raw_ptr(),
            role,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QSortFilterProxyModel::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_metaObject(
            self as *const crate::QSortFilterProxyModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeData">QAbstractItemModel::mimeData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QMimeData* QSortFilterProxyModel::mimeData(const QList<QModelIndex>& indexes) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#mimeData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeData">QAbstractItemModel::mimeData</a>().</p></div>
    pub unsafe fn mime_data(
        &self,
        indexes: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQModelIndex>>,
    ) -> ::cpp_utils::MutPtr<crate::QMimeData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_mimeData(
            self as *const crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQModelIndex>>::cast_into(
                indexes,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeTypes">QAbstractItemModel::mimeTypes</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QStringList QSortFilterProxyModel::mimeTypes() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#mimeTypes">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeTypes">QAbstractItemModel::mimeTypes</a>().</p></div>
    pub unsafe fn mime_types(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_mimeTypes(
            self as *const crate::QSortFilterProxyModel,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a sorting filter model with the given <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSortFilterProxyModel::QSortFilterProxyModel(QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#QSortFilterProxyModel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a sorting filter model with the given <i>parent</i>.</p></div>
    pub unsafe fn new_1a(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QSortFilterProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_QSortFilterProxyModel(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html">QSortFilterProxyModel</a> class provides support for sorting and filtering data passed between another model and a view.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSortFilterProxyModel::QSortFilterProxyModel()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html">QSortFilterProxyModel</a> class provides support for sorting and filtering data passed between another model and a view.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html">QSortFilterProxyModel</a> can be used for sorting items, filtering out items, or both. The model transforms the structure of a source model by mapping the model indexes it supplies to new indexes, corresponding to different locations, for views to use. This approach allows a given source model to be restructured as far as views are concerned without requiring any transformations on the underlying data, and without duplicating the data in memory.</p>
    /// <p>Let's assume that we want to sort and filter the items provided by a custom model. The code to set up the model and the view, <i>without</i> sorting and filtering, would look like this:</p>
    /// <pre class="cpp">
    ///
    /// &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qtreeview.html">QTreeView</a></span> <span class="operator">*</span>treeView <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtreeview.html">QTreeView</a></span>;
    /// &#32;   &#32;     MyItemModel <span class="operator">*</span>model <span class="operator">=</span> <span class="keyword">new</span> MyItemModel(<span class="keyword">this</span>);
    ///
    /// &#32;   &#32;     treeView<span class="operator">-</span><span class="operator">&gt;</span>setModel(model);
    ///
    /// </pre>
    /// <p>To add sorting and filtering support to <code>MyItemModel</code>, we need to create a <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html">QSortFilterProxyModel</a>, call <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setSourceModel">setSourceModel</a>() with the <code>MyItemModel</code> as argument, and install the <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html">QSortFilterProxyModel</a> on the view:</p>
    /// <pre class="cpp">
    ///
    /// &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qtreeview.html">QTreeView</a></span> <span class="operator">*</span>treeView <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtreeview.html">QTreeView</a></span>;
    /// &#32;   &#32;     MyItemModel <span class="operator">*</span>sourceModel <span class="operator">=</span> <span class="keyword">new</span> MyItemModel(<span class="keyword">this</span>);
    /// &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#QSortFilterProxyModel">QSortFilterProxyModel</a></span> <span class="operator">*</span>proxyModel <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#QSortFilterProxyModel">QSortFilterProxyModel</a></span>(<span class="keyword">this</span>);
    ///
    /// &#32;   &#32;     proxyModel<span class="operator">-</span><span class="operator">&gt;</span>setSourceModel(sourceModel);
    /// &#32;   &#32;     treeView<span class="operator">-</span><span class="operator">&gt;</span>setModel(proxyModel);
    ///
    /// </pre>
    /// <p>At this point, neither sorting nor filtering is enabled; the original data is displayed in the view. Any changes made through the <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html">QSortFilterProxyModel</a> are applied to the original model.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html">QSortFilterProxyModel</a> acts as a wrapper for the original model. If you need to convert source <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a>es to sorted/filtered model indexes or vice versa, use <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#mapToSource">mapToSource</a>(), <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#mapFromSource">mapFromSource</a>(), <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#mapSelectionToSource">mapSelectionToSource</a>(), and <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#mapSelectionFromSource">mapSelectionFromSource</a>().</p>
    /// <p><b>Note: </b>By default, the model dynamically re-sorts and re-filters data whenever the original model changes. This behavior can be changed by setting the <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#dynamicSortFilter-prop">dynamicSortFilter</a> property.</p><p>The <a href="http://doc.qt.io/qt-5/qtwidgets-itemviews-basicsortfiltermodel-example.html">Basic Sort/Filter Model</a> and <a href="http://doc.qt.io/qt-5/qtwidgets-itemviews-customsortfiltermodel-example.html">Custom Sort/Filter Model</a> examples illustrate how to use <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html">QSortFilterProxyModel</a> to perform basic sorting and filtering and how to subclass it to implement custom behavior.</p>
    /// <a name="sorting"></a></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QSortFilterProxyModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_QSortFilterProxyModel1();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#parent">QAbstractItemModel::parent</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QSortFilterProxyModel::parent(const QModelIndex& child) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#parent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#parent">QAbstractItemModel::parent</a>().</p></div>
    pub unsafe fn parent(
        &self,
        child: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_parent(
            self as *const crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(child)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QSortFilterProxyModel::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_qt_metacall(
            self as *mut crate::QSortFilterProxyModel,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QSortFilterProxyModel::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_qt_metacast(
            self as *mut crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumns">QAbstractItemModel::removeColumns</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QSortFilterProxyModel::removeColumns(int column, int count, const QModelIndex& parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#removeColumns">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumns">QAbstractItemModel::removeColumns</a>().</p></div>
    pub unsafe fn remove_columns_3a(
        &mut self,
        column: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_removeColumns(
            self as *mut crate::QSortFilterProxyModel,
            column,
            count,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumns">QAbstractItemModel::removeColumns</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QSortFilterProxyModel::removeColumns(int column, int count)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#removeColumns">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumns">QAbstractItemModel::removeColumns</a>().</p></div>
    pub unsafe fn remove_columns_2a(
        &mut self,
        column: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_removeColumns1(
            self as *mut crate::QSortFilterProxyModel,
            column,
            count,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">QAbstractItemModel::removeRows</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QSortFilterProxyModel::removeRows(int row, int count, const QModelIndex& parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#removeRows">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">QAbstractItemModel::removeRows</a>().</p></div>
    pub unsafe fn remove_rows_3a(
        &mut self,
        row: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_removeRows(
            self as *mut crate::QSortFilterProxyModel,
            row,
            count,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">QAbstractItemModel::removeRows</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QSortFilterProxyModel::removeRows(int row, int count)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#removeRows">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">QAbstractItemModel::removeRows</a>().</p></div>
    pub unsafe fn remove_rows_2a(
        &mut self,
        row: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_removeRows1(
            self as *mut crate::QSortFilterProxyModel,
            row,
            count,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">QAbstractItemModel::rowCount</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual int QSortFilterProxyModel::rowCount(const QModelIndex& parent = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#rowCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">QAbstractItemModel::rowCount</a>().</p></div>
    pub unsafe fn row_count_1a(
        &self,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_rowCount(
            self as *const crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">QAbstractItemModel::rowCount</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual int QSortFilterProxyModel::rowCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#rowCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">QAbstractItemModel::rowCount</a>().</p></div>
    pub unsafe fn row_count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_rowCount1(
            self as *const crate::QSortFilterProxyModel,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">QAbstractItemModel::setData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QSortFilterProxyModel::setData(const QModelIndex& index, const QVariant& value, int role = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">QAbstractItemModel::setData</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#data">data</a>().</p></div>
    pub unsafe fn set_data_3a(
        &mut self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
        role: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_setData(
            self as *mut crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
            role,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">QAbstractItemModel::setData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QSortFilterProxyModel::setData(const QModelIndex& index, const QVariant& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">QAbstractItemModel::setData</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#data">data</a>().</p></div>
    pub unsafe fn set_data_2a(
        &mut self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_setData1(
            self as *mut crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>This property holds whether the proxy model is dynamically sorted and filtered whenever the contents of the source model change</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSortFilterProxyModel::setDynamicSortFilter(bool enable)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#dynamicSortFilter-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds whether the proxy model is dynamically sorted and filtered whenever the contents of the source model change</p>
    /// <p>Note that you should not update the source model through the proxy model when dynamicSortFilter is true. For instance, if you set the proxy model on a <a href="http://doc.qt.io/qt-5/qcombobox.html">QComboBox</a>, then using functions that update the model, e.g., <a href="http://doc.qt.io/qt-5/qcombobox.html#addItem-1">addItem()</a>, will not work as expected. An alternative is to set dynamicSortFilter to false and call <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#sort">sort()</a> after adding items to the <a href="http://doc.qt.io/qt-5/qcombobox.html">QComboBox</a>.</p>
    /// <p>The default value is true.</p>
    /// <p>This property was introduced in  Qt 4.2.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> bool </td><td class="memItemRight bottomAlign"><span class="name"><b>dynamicSortFilter</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setDynamicSortFilter</b></span>(bool <i>enable</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn set_dynamic_sort_filter(&mut self, enable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_setDynamicSortFilter(
            self as *mut crate::QSortFilterProxyModel,
            enable,
        )
    }

    /// <p>This property holds the case sensitivity of the <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> pattern used to filter the contents of the source model</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSortFilterProxyModel::setFilterCaseSensitivity(Qt::CaseSensitivity cs)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterCaseSensitivity-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the case sensitivity of the <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> pattern used to filter the contents of the source model</p>
    /// <p>By default, the filter is case sensitive.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> Qt::CaseSensitivity </td><td class="memItemRight bottomAlign"><span class="name"><b>filterCaseSensitivity</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setFilterCaseSensitivity</b></span>(Qt::CaseSensitivity <i>cs</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterRegExp-prop">filterRegExp</a> and <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#sortCaseSensitivity-prop">sortCaseSensitivity</a>.</p></div>
    pub unsafe fn set_filter_case_sensitivity(&mut self, cs: crate::CaseSensitivity) {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_setFilterCaseSensitivity(
            self as *mut crate::QSortFilterProxyModel,
            cs,
        )
    }

    /// <p>Sets the fixed string used to filter the contents of the source model to the given <i>pattern</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QSortFilterProxyModel::setFilterFixedString(const QString& pattern)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterFixedString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the fixed string used to filter the contents of the source model to the given <i>pattern</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterCaseSensitivity-prop">setFilterCaseSensitivity</a>(), <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterRegExp-prop">setFilterRegExp</a>(), <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterWildcard">setFilterWildcard</a>(), and <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterRegExp-prop">filterRegExp</a>().</p></div>
    pub unsafe fn set_filter_fixed_string(
        &mut self,
        pattern: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_setFilterFixedString(
            self as *mut crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(pattern)
                .as_raw_ptr(),
        )
    }

    /// <p>This property holds the column where the key used to filter the contents of the source model is read from.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSortFilterProxyModel::setFilterKeyColumn(int column)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterKeyColumn-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the column where the key used to filter the contents of the source model is read from.</p>
    /// <p>The default value is 0. If the value is -1, the keys will be read from all columns.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>filterKeyColumn</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setFilterKeyColumn</b></span>(int <i>column</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn set_filter_key_column(&mut self, column: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_setFilterKeyColumn(
            self as *mut crate::QSortFilterProxyModel,
            column,
        )
    }

    /// <p>This property holds the <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> used to filter the contents of the source model</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSortFilterProxyModel::setFilterRegExp(const QRegExp& regExp)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterRegExp-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> used to filter the contents of the source model</p>
    /// <p>Setting this property overwrites the current <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterCaseSensitivity-prop">filterCaseSensitivity</a>. By default, the <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> is an empty string matching all contents.</p>
    /// <p>If no <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> or an empty string is set, everything in the source model will be accepted.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QRegExp </td><td class="memItemRight bottomAlign"><span class="name"><b>filterRegExp</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setFilterRegExp</b></span>(const QRegExp &amp;<i>regExp</i>)</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterRegExp-1">setFilterRegExp</a></b></span>(const QString &amp;<i>pattern</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterCaseSensitivity-prop">filterCaseSensitivity</a>, <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterWildcard">setFilterWildcard</a>(), and <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterFixedString">setFilterFixedString</a>().</p></div>
    pub unsafe fn set_filter_reg_exp_q_reg_exp(
        &mut self,
        reg_exp: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_setFilterRegExp(
            self as *mut crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(reg_exp)
                .as_raw_ptr(),
        )
    }

    /// <p>This property holds the <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> used to filter the contents of the source model</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QSortFilterProxyModel::setFilterRegExp(const QString& pattern)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterRegExp-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> used to filter the contents of the source model</p>
    /// <p>Setting this property overwrites the current <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterCaseSensitivity-prop">filterCaseSensitivity</a>. By default, the <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> is an empty string matching all contents.</p>
    /// <p>If no <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> or an empty string is set, everything in the source model will be accepted.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QRegExp </td><td class="memItemRight bottomAlign"><span class="name"><b>filterRegExp</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setFilterRegExp</b></span>(const QRegExp &amp;<i>regExp</i>)</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterRegExp-1">setFilterRegExp</a></b></span>(const QString &amp;<i>pattern</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterCaseSensitivity-prop">filterCaseSensitivity</a>, <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterWildcard">setFilterWildcard</a>(), and <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterFixedString">setFilterFixedString</a>().</p></div>
    pub unsafe fn set_filter_reg_exp_q_string(
        &mut self,
        pattern: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_setFilterRegExp1(
            self as *mut crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(pattern)
                .as_raw_ptr(),
        )
    }

    /// <p>This property holds the <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> used to filter the contents of the source model</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSortFilterProxyModel::setFilterRegularExpression(const QString& pattern)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterRegularExpression-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> used to filter the contents of the source model</p>
    /// <p>Setting this property overwrites the current <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterCaseSensitivity-prop">filterCaseSensitivity</a>. By default, the <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> is an empty string matching all contents.</p>
    /// <p>If no <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> or an empty string is set, everything in the source model will be accepted.</p>
    /// <p>This property was introduced in  Qt 5.12.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QRegularExpression </td><td class="memItemRight bottomAlign"><span class="name"><b>filterRegularExpression</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterRegularExpression">setFilterRegularExpression</a></b></span>(const QString &amp;<i>pattern</i>)</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setFilterRegularExpression</b></span>(const QRegularExpression &amp;<i>regularExpression</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterCaseSensitivity-prop">filterCaseSensitivity</a>, <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterWildcard">setFilterWildcard</a>(), and <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterFixedString">setFilterFixedString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn set_filter_regular_expression_q_string(
        &mut self,
        pattern: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_setFilterRegularExpression(
            self as *mut crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(pattern)
                .as_raw_ptr(),
        )
    }

    /// <p>This property holds the <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> used to filter the contents of the source model</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSortFilterProxyModel::setFilterRegularExpression(const QRegularExpression& regularExpression)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterRegularExpression-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> used to filter the contents of the source model</p>
    /// <p>Setting this property overwrites the current <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterCaseSensitivity-prop">filterCaseSensitivity</a>. By default, the <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> is an empty string matching all contents.</p>
    /// <p>If no <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> or an empty string is set, everything in the source model will be accepted.</p>
    /// <p>This property was introduced in  Qt 5.12.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QRegularExpression </td><td class="memItemRight bottomAlign"><span class="name"><b>filterRegularExpression</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterRegularExpression">setFilterRegularExpression</a></b></span>(const QString &amp;<i>pattern</i>)</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setFilterRegularExpression</b></span>(const QRegularExpression &amp;<i>regularExpression</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterCaseSensitivity-prop">filterCaseSensitivity</a>, <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterWildcard">setFilterWildcard</a>(), and <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterFixedString">setFilterFixedString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn set_filter_regular_expression_q_regular_expression(
        &mut self,
        regular_expression: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_setFilterRegularExpression1(
            self as *mut crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(
                regular_expression,
            )
            .as_raw_ptr(),
        )
    }

    /// <p>This property holds the item role that is used to query the source model's data when filtering items</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSortFilterProxyModel::setFilterRole(int role)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterRole-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the item role that is used to query the source model's data when filtering items</p>
    /// <p>The default value is <a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::DisplayRole</a>.</p>
    /// <p>This property was introduced in  Qt 4.2.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>filterRole</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setFilterRole</b></span>(int <i>role</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterAcceptsRow">filterAcceptsRow</a>().</p></div>
    pub unsafe fn set_filter_role(&mut self, role: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_setFilterRole(
            self as *mut crate::QSortFilterProxyModel,
            role,
        )
    }

    /// <p>Sets the wildcard expression used to filter the contents of the source model to the given <i>pattern</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QSortFilterProxyModel::setFilterWildcard(const QString& pattern)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterWildcard">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the wildcard expression used to filter the contents of the source model to the given <i>pattern</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterCaseSensitivity-prop">setFilterCaseSensitivity</a>(), <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterRegExp-prop">setFilterRegExp</a>(), <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setFilterFixedString">setFilterFixedString</a>(), and <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterRegExp-prop">filterRegExp</a>().</p></div>
    pub unsafe fn set_filter_wildcard(
        &mut self,
        pattern: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_setFilterWildcard(
            self as *mut crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(pattern)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setHeaderData">QAbstractItemModel::setHeaderData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QSortFilterProxyModel::setHeaderData(int section, Qt::Orientation orientation, const QVariant& value, int role = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setHeaderData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setHeaderData">QAbstractItemModel::setHeaderData</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#headerData">headerData</a>().</p></div>
    pub unsafe fn set_header_data_4a(
        &mut self,
        section: ::std::os::raw::c_int,
        orientation: crate::Orientation,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
        role: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_setHeaderData(
            self as *mut crate::QSortFilterProxyModel,
            section,
            orientation,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
            role,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setHeaderData">QAbstractItemModel::setHeaderData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QSortFilterProxyModel::setHeaderData(int section, Qt::Orientation orientation, const QVariant& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setHeaderData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setHeaderData">QAbstractItemModel::setHeaderData</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#headerData">headerData</a>().</p></div>
    pub unsafe fn set_header_data_3a(
        &mut self,
        section: ::std::os::raw::c_int,
        orientation: crate::Orientation,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_setHeaderData1(
            self as *mut crate::QSortFilterProxyModel,
            section,
            orientation,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>This property holds whether the filter to be applied recursively on children, and for any matching child, its parents will be visible as well.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSortFilterProxyModel::setRecursiveFilteringEnabled(bool recursive)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#recursiveFilteringEnabled-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds whether the filter to be applied recursively on children, and for any matching child, its parents will be visible as well.</p>
    /// <p>The default value is false.</p>
    /// <p>This property was introduced in  Qt 5.10.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> bool </td><td class="memItemRight bottomAlign"><span class="name"><b>isRecursiveFilteringEnabled</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setRecursiveFilteringEnabled</b></span>(bool <i>recursive</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterAcceptsRow">filterAcceptsRow</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn set_recursive_filtering_enabled(&mut self, recursive: bool) {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_setRecursiveFilteringEnabled(
            self as *mut crate::QSortFilterProxyModel,
            recursive,
        )
    }

    /// <p>This property holds the case sensitivity setting used for comparing strings when sorting</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSortFilterProxyModel::setSortCaseSensitivity(Qt::CaseSensitivity cs)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#sortCaseSensitivity-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the case sensitivity setting used for comparing strings when sorting</p>
    /// <p>By default, sorting is case sensitive.</p>
    /// <p>This property was introduced in  Qt 4.2.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> Qt::CaseSensitivity </td><td class="memItemRight bottomAlign"><span class="name"><b>sortCaseSensitivity</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setSortCaseSensitivity</b></span>(Qt::CaseSensitivity <i>cs</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterCaseSensitivity-prop">filterCaseSensitivity</a> and <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#lessThan">lessThan</a>().</p></div>
    pub unsafe fn set_sort_case_sensitivity(&mut self, cs: crate::CaseSensitivity) {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_setSortCaseSensitivity(
            self as *mut crate::QSortFilterProxyModel,
            cs,
        )
    }

    /// <p>This property holds the local aware setting used for comparing strings when sorting</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSortFilterProxyModel::setSortLocaleAware(bool on)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#isSortLocaleAware-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the local aware setting used for comparing strings when sorting</p>
    /// <p>By default, sorting is not local aware.</p>
    /// <p>This property was introduced in  Qt 4.3.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> bool </td><td class="memItemRight bottomAlign"><span class="name"><b>isSortLocaleAware</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setSortLocaleAware</b></span>(bool <i>on</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#sortCaseSensitivity-prop">sortCaseSensitivity</a> and <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#lessThan">lessThan</a>().</p></div>
    pub unsafe fn set_sort_locale_aware(&mut self, on: bool) {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_setSortLocaleAware(
            self as *mut crate::QSortFilterProxyModel,
            on,
        )
    }

    /// <p>This property holds the item role that is used to query the source model's data when sorting items</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSortFilterProxyModel::setSortRole(int role)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#sortRole-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the item role that is used to query the source model's data when sorting items</p>
    /// <p>The default value is <a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::DisplayRole</a>.</p>
    /// <p>This property was introduced in  Qt 4.2.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>sortRole</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setSortRole</b></span>(int <i>role</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#lessThan">lessThan</a>().</p></div>
    pub unsafe fn set_sort_role(&mut self, role: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_setSortRole(
            self as *mut crate::QSortFilterProxyModel,
            role,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#setSourceModel">QAbstractProxyModel::setSourceModel</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual void QSortFilterProxyModel::setSourceModel(QAbstractItemModel* sourceModel)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#setSourceModel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#setSourceModel">QAbstractProxyModel::setSourceModel</a>().</p></div>
    pub unsafe fn set_source_model(
        &mut self,
        source_model: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractItemModel>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_setSourceModel(
            self as *mut crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractItemModel>>::cast_into(
                source_model,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sibling">QAbstractItemModel::sibling</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QSortFilterProxyModel::sibling(int row, int column, const QModelIndex& idx) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#sibling">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sibling">QAbstractItemModel::sibling</a>().</p></div>
    pub unsafe fn sibling(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
        idx: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_sibling(
            self as *const crate::QSortFilterProxyModel,
            row,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(idx)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sort">QAbstractItemModel::sort</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual void QSortFilterProxyModel::sort(int column, Qt::SortOrder order = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#sort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sort">QAbstractItemModel::sort</a>().</p></div>
    pub unsafe fn sort_2a(&mut self, column: ::std::os::raw::c_int, order: crate::SortOrder) {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_sort(
            self as *mut crate::QSortFilterProxyModel,
            column,
            order,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sort">QAbstractItemModel::sort</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual void QSortFilterProxyModel::sort(int column)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#sort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sort">QAbstractItemModel::sort</a>().</p></div>
    pub unsafe fn sort_1a(&mut self, column: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_sort1(
            self as *mut crate::QSortFilterProxyModel,
            column,
        )
    }

    /// <p>This property holds the case sensitivity setting used for comparing strings when sorting</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```Qt::CaseSensitivity QSortFilterProxyModel::sortCaseSensitivity() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#sortCaseSensitivity-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the case sensitivity setting used for comparing strings when sorting</p>
    /// <p>By default, sorting is case sensitive.</p>
    /// <p>This property was introduced in  Qt 4.2.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> Qt::CaseSensitivity </td><td class="memItemRight bottomAlign"><span class="name"><b>sortCaseSensitivity</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setSortCaseSensitivity</b></span>(Qt::CaseSensitivity <i>cs</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#filterCaseSensitivity-prop">filterCaseSensitivity</a> and <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#lessThan">lessThan</a>().</p></div>
    pub unsafe fn sort_case_sensitivity(&self) -> crate::CaseSensitivity {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_sortCaseSensitivity(
            self as *const crate::QSortFilterProxyModel,
        )
    }

    /// <p>the column currently used for sorting</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QSortFilterProxyModel::sortColumn() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#sortColumn">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>the column currently used for sorting</p>
    /// <p>This returns the most recently used sort column.</p>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn sort_column(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_sortColumn(
            self as *const crate::QSortFilterProxyModel,
        )
    }

    /// <p>the order currently used for sorting</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```Qt::SortOrder QSortFilterProxyModel::sortOrder() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#sortOrder">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>the order currently used for sorting</p>
    /// <p>This returns the most recently used sort order.</p>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn sort_order(&self) -> crate::SortOrder {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_sortOrder(
            self as *const crate::QSortFilterProxyModel,
        )
    }

    /// <p>This property holds the item role that is used to query the source model's data when sorting items</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QSortFilterProxyModel::sortRole() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#sortRole-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the item role that is used to query the source model's data when sorting items</p>
    /// <p>The default value is <a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::DisplayRole</a>.</p>
    /// <p>This property was introduced in  Qt 4.2.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>sortRole</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setSortRole</b></span>(int <i>role</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#lessThan">lessThan</a>().</p></div>
    pub unsafe fn sort_role(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_sortRole(
            self as *const crate::QSortFilterProxyModel,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#span">QAbstractItemModel::span</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QSize QSortFilterProxyModel::span(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#span">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#span">QAbstractItemModel::span</a>().</p></div>
    pub unsafe fn span(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QSize> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_span(
            self as *const crate::QSortFilterProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#supportedDropActions">QAbstractItemModel::supportedDropActions</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QFlags<Qt::DropAction> QSortFilterProxyModel::supportedDropActions() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#supportedDropActions">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#supportedDropActions">QAbstractItemModel::supportedDropActions</a>().</p></div>
    pub unsafe fn supported_drop_actions(&self) -> crate::QFlags<crate::DropAction> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_supportedDropActions(
            self as *const crate::QSortFilterProxyModel,
        );
        crate::QFlags::from(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QSortFilterProxyModel::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QSortFilterProxyModel::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_standard_paths {
    //! C++ type: <span style='color: green;'>```QStandardPaths```</span>

    /// <p>This enum describes the different locations that can be queried using methods such as <a href="http://doc.qt.io/qt-5/qstandardpaths.html#writableLocation">QStandardPaths::writableLocation</a>, <a href="http://doc.qt.io/qt-5/qstandardpaths.html#standardLocations">QStandardPaths::standardLocations</a>, and <a href="http://doc.qt.io/qt-5/qstandardpaths.html#displayName">QStandardPaths::displayName</a>.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QStandardPaths::StandardLocation```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstandardpaths.html#StandardLocation-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the different locations that can be queried using methods such as <a href="http://doc.qt.io/qt-5/qstandardpaths.html#writableLocation">QStandardPaths::writableLocation</a>, <a href="http://doc.qt.io/qt-5/qstandardpaths.html#standardLocations">QStandardPaths::standardLocations</a>, and <a href="http://doc.qt.io/qt-5/qstandardpaths.html#displayName">QStandardPaths::displayName</a>.</p>
    /// <p>Some of the values in this enum represent a user configuration. Such enum values will return the same paths in different applications, so they could be used to share data with other applications. Other values are specific to this application. Each enum value in the table below describes whether it's application-specific or generic.</p>
    /// <p>Application-specific directories should be assumed to be unreachable by other applications. Therefore, files placed there might not be readable by other applications, even if run by the same user. On the other hand, generic directories should be assumed to be accessible by all applications run by this user, but should still be assumed to be unreachable by applications by other users.</p>
    /// <p>Data interchange with other users is out of the scope of <a href="http://doc.qt.io/qt-5/qstandardpaths.html">QStandardPaths</a>.</p>
    ///
    /// <p>The following table gives examples of paths on different operating systems. The first path is the writable path (unless noted). Other, additional paths, if any, represent non-writable locations.</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Path type</th><th><a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a></th><th>Windows</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>DesktopLocation</td><td>"~/Desktop"</td><td>"C:/Users/&lt;USER&gt;/Desktop"</td></tr>
    /// <tr valign="top" class="even"><td>DocumentsLocation</td><td>"~/Documents"</td><td>"C:/Users/&lt;USER&gt;/Documents"</td></tr>
    /// <tr valign="top" class="odd"><td>FontsLocation</td><td>"/System/Library/Fonts" (not writable)</td><td>"C:/Windows/Fonts" (not writable)</td></tr>
    /// <tr valign="top" class="even"><td>ApplicationsLocation</td><td>"/Applications" (not writable)</td><td>"C:/Users/&lt;USER&gt;/AppData/Roaming/Microsoft/Windows/Start Menu/Programs"</td></tr>
    /// <tr valign="top" class="odd"><td>MusicLocation</td><td>"~/Music"</td><td>"C:/Users/&lt;USER&gt;/Music"</td></tr>
    /// <tr valign="top" class="even"><td>MoviesLocation</td><td>"~/Movies"</td><td>"C:/Users/&lt;USER&gt;/Videos"</td></tr>
    /// <tr valign="top" class="odd"><td>PicturesLocation</td><td>"~/Pictures"</td><td>"C:/Users/&lt;USER&gt;/Pictures"</td></tr>
    /// <tr valign="top" class="even"><td>TempLocation</td><td>randomly generated by the OS</td><td>"C:/Users/&lt;USER&gt;/AppData/Local/Temp"</td></tr>
    /// <tr valign="top" class="odd"><td>HomeLocation</td><td>"~"</td><td>"C:/Users/&lt;USER&gt;"</td></tr>
    /// <tr valign="top" class="even"><td>DataLocation</td><td>"~/Library/Application Support/&lt;APPNAME&gt;", "/Library/Application Support/&lt;APPNAME&gt;". "&lt;APPDIR&gt;/../Resources"</td><td>"C:/Users/&lt;USER&gt;/AppData/Local/&lt;APPNAME&gt;", "C:/ProgramData/&lt;APPNAME&gt;", "&lt;APPDIR&gt;", "&lt;APPDIR&gt;/data"</td></tr>
    /// <tr valign="top" class="odd"><td>CacheLocation</td><td>"~/Library/Caches/&lt;APPNAME&gt;", "/Library/Caches/&lt;APPNAME&gt;"</td><td>"C:/Users/&lt;USER&gt;/AppData/Local/&lt;APPNAME&gt;/cache"</td></tr>
    /// <tr valign="top" class="even"><td>GenericDataLocation</td><td>"~/Library/Application Support", "/Library/Application Support"</td><td>"C:/Users/&lt;USER&gt;/AppData/Local", "C:/ProgramData", "&lt;APPDIR&gt;", "&lt;APPDIR&gt;/data"</td></tr>
    /// <tr valign="top" class="odd"><td>RuntimeLocation</td><td>"~/Library/Application Support"</td><td>"C:/Users/&lt;USER&gt;"</td></tr>
    /// <tr valign="top" class="even"><td>ConfigLocation</td><td>"~/Library/Preferences"</td><td>"C:/Users/&lt;USER&gt;/AppData/Local/&lt;APPNAME&gt;", "C:/ProgramData/&lt;APPNAME&gt;"</td></tr>
    /// <tr valign="top" class="odd"><td>GenericConfigLocation</td><td>"~/Library/Preferences"</td><td>"C:/Users/&lt;USER&gt;/AppData/Local", "C:/ProgramData"</td></tr>
    /// <tr valign="top" class="even"><td>DownloadLocation</td><td>"~/Downloads"</td><td>"C:/Users/&lt;USER&gt;/Documents"</td></tr>
    /// <tr valign="top" class="odd"><td>GenericCacheLocation</td><td>"~/Library/Caches", "/Library/Caches"</td><td>"C:/Users/&lt;USER&gt;/AppData/Local/cache"</td></tr>
    /// <tr valign="top" class="even"><td>AppDataLocation</td><td>"~/Library/Application Support/&lt;APPNAME&gt;", "/Library/Application Support/&lt;APPNAME&gt;". "&lt;APPDIR&gt;/../Resources"</td><td>"C:/Users/&lt;USER&gt;/AppData/Roaming/&lt;APPNAME&gt;", "C:/ProgramData/&lt;APPNAME&gt;", "&lt;APPDIR&gt;", "&lt;APPDIR&gt;/data"</td></tr>
    /// <tr valign="top" class="odd"><td>AppLocalDataLocation</td><td>"~/Library/Application Support/&lt;APPNAME&gt;", "/Library/Application Support/&lt;APPNAME&gt;". "&lt;APPDIR&gt;/../Resources"</td><td>"C:/Users/&lt;USER&gt;/AppData/Local/&lt;APPNAME&gt;", "C:/ProgramData/&lt;APPNAME&gt;", "&lt;APPDIR&gt;", "&lt;APPDIR&gt;/data"</td></tr>
    /// <tr valign="top" class="even"><td>AppConfigLocation</td><td>"~/Library/Preferences/&lt;APPNAME&gt;"</td><td>"C:/Users/&lt;USER&gt;/AppData/Local/&lt;APPNAME&gt;", "C:/ProgramData/&lt;APPNAME&gt;"</td></tr>
    /// </tbody></table></div>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Path type</th><th>Linux</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>DesktopLocation</td><td>"~/Desktop"</td></tr>
    /// <tr valign="top" class="even"><td>DocumentsLocation</td><td>"~/Documents"</td></tr>
    /// <tr valign="top" class="odd"><td>FontsLocation</td><td>"~/.fonts"</td></tr>
    /// <tr valign="top" class="even"><td>ApplicationsLocation</td><td>"~/.local/share/applications", "/usr/local/share/applications", "/usr/share/applications"</td></tr>
    /// <tr valign="top" class="odd"><td>MusicLocation</td><td>"~/Music"</td></tr>
    /// <tr valign="top" class="even"><td>MoviesLocation</td><td>"~/Videos"</td></tr>
    /// <tr valign="top" class="odd"><td>PicturesLocation</td><td>"~/Pictures"</td></tr>
    /// <tr valign="top" class="even"><td>TempLocation</td><td>"/tmp"</td></tr>
    /// <tr valign="top" class="odd"><td>HomeLocation</td><td>"~"</td></tr>
    /// <tr valign="top" class="even"><td>DataLocation</td><td>"~/.local/share/&lt;APPNAME&gt;", "/usr/local/share/&lt;APPNAME&gt;", "/usr/share/&lt;APPNAME&gt;"</td></tr>
    /// <tr valign="top" class="odd"><td>CacheLocation</td><td>"~/.cache/&lt;APPNAME&gt;"</td></tr>
    /// <tr valign="top" class="even"><td>GenericDataLocation</td><td>"~/.local/share", "/usr/local/share", "/usr/share"</td></tr>
    /// <tr valign="top" class="odd"><td>RuntimeLocation</td><td>"/run/user/&lt;USER&gt;"</td></tr>
    /// <tr valign="top" class="even"><td>ConfigLocation</td><td>"~/.config", "/etc/xdg"</td></tr>
    /// <tr valign="top" class="odd"><td>GenericConfigLocation</td><td>"~/.config", "/etc/xdg"</td></tr>
    /// <tr valign="top" class="even"><td>DownloadLocation</td><td>"~/Downloads"</td></tr>
    /// <tr valign="top" class="odd"><td>GenericCacheLocation</td><td>"~/.cache"</td></tr>
    /// <tr valign="top" class="even"><td>AppDataLocation</td><td>"~/.local/share/&lt;APPNAME&gt;", "/usr/local/share/&lt;APPNAME&gt;", "/usr/share/&lt;APPNAME&gt;"</td></tr>
    /// <tr valign="top" class="odd"><td>AppLocalDataLocation</td><td>"~/.local/share/&lt;APPNAME&gt;", "/usr/local/share/&lt;APPNAME&gt;", "/usr/share/&lt;APPNAME&gt;"</td></tr>
    /// <tr valign="top" class="even"><td>AppConfigLocation</td><td>"~/.config/&lt;APPNAME&gt;", "/etc/xdg/&lt;APPNAME&gt;"</td></tr>
    /// </tbody></table></div>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Path type</th><th>Android</th><th>iOS</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>DesktopLocation</td><td>"&lt;APPROOT&gt;/files"</td><td>"&lt;APPROOT&gt;/Documents/Desktop"</td></tr>
    /// <tr valign="top" class="even"><td>DocumentsLocation</td><td>"&lt;USER&gt;/Documents", "&lt;USER&gt;/&lt;APPNAME&gt;/Documents"</td><td>"&lt;APPROOT&gt;/Documents"</td></tr>
    /// <tr valign="top" class="odd"><td>FontsLocation</td><td>"/system/fonts" (not writable)</td><td>"&lt;APPROOT&gt;/Library/Fonts"</td></tr>
    /// <tr valign="top" class="even"><td>ApplicationsLocation</td><td>not supported (directory not readable)</td><td>not supported</td></tr>
    /// <tr valign="top" class="odd"><td>MusicLocation</td><td>"&lt;USER&gt;/Music", "&lt;USER&gt;/&lt;APPNAME&gt;/Music"</td><td>"&lt;APPROOT&gt;/Documents/Music"</td></tr>
    /// <tr valign="top" class="even"><td>MoviesLocation</td><td>"&lt;USER&gt;/Movies", "&lt;USER&gt;/&lt;APPNAME&gt;/Movies"</td><td>"&lt;APPROOT&gt;/Documents/Movies"</td></tr>
    /// <tr valign="top" class="odd"><td>PicturesLocation</td><td>"&lt;USER&gt;/Pictures", "&lt;USER&gt;/&lt;APPNAME&gt;/Pictures"</td><td>"&lt;APPROOT&gt;/Documents/Pictures", "assets-library://"</td></tr>
    /// <tr valign="top" class="even"><td>TempLocation</td><td>"&lt;APPROOT&gt;/cache"</td><td>"&lt;APPROOT&gt;/tmp"</td></tr>
    /// <tr valign="top" class="odd"><td>HomeLocation</td><td>"&lt;APPROOT&gt;/files"</td><td>"&lt;APPROOT&gt;" (not writable)</td></tr>
    /// <tr valign="top" class="even"><td>DataLocation</td><td>"&lt;APPROOT&gt;/files", "&lt;USER&gt;/&lt;APPNAME&gt;/files"</td><td>"&lt;APPROOT&gt;/Library/Application Support"</td></tr>
    /// <tr valign="top" class="odd"><td>CacheLocation</td><td>"&lt;APPROOT&gt;/cache", "&lt;USER&gt;/&lt;APPNAME&gt;/cache"</td><td>"&lt;APPROOT&gt;/Library/Caches"</td></tr>
    /// <tr valign="top" class="even"><td>GenericDataLocation</td><td>"&lt;USER&gt;"</td><td>"&lt;APPROOT&gt;/Documents"</td></tr>
    /// <tr valign="top" class="odd"><td>RuntimeLocation</td><td>"&lt;APPROOT&gt;/cache"</td><td>not supported</td></tr>
    /// <tr valign="top" class="even"><td>ConfigLocation</td><td>"&lt;APPROOT&gt;/files/settings"</td><td>"&lt;APPROOT&gt;/Library/Preferences"</td></tr>
    /// <tr valign="top" class="odd"><td>GenericConfigLocation</td><td>"&lt;APPROOT&gt;/files/settings" (there is no shared settings)</td><td>"&lt;APPROOT&gt;/Library/Preferences"</td></tr>
    /// <tr valign="top" class="even"><td>DownloadLocation</td><td>"&lt;USER&gt;/Downloads", "&lt;USER&gt;/&lt;APPNAME&gt;/Downloads"</td><td>"&lt;APPROOT&gt;/Documents/Downloads"</td></tr>
    /// <tr valign="top" class="odd"><td>GenericCacheLocation</td><td>"&lt;APPROOT&gt;/cache" (there is no shared cache)</td><td>"&lt;APPROOT&gt;/Library/Caches"</td></tr>
    /// <tr valign="top" class="even"><td>AppDataLocation</td><td>"&lt;APPROOT&gt;/files", "&lt;USER&gt;/&lt;APPNAME&gt;/files"</td><td>"&lt;APPROOT&gt;/Library/Application Support"</td></tr>
    /// <tr valign="top" class="odd"><td>AppConfigLocation</td><td>"&lt;APPROOT&gt;/files/settings"</td><td>"&lt;APPROOT&gt;/Library/Preferences/&lt;APPNAME&gt;"</td></tr>
    /// <tr valign="top" class="even"><td>AppLocalDataLocation</td><td>"&lt;APPROOT&gt;/files", "&lt;USER&gt;/&lt;APPNAME&gt;/files"</td><td>"&lt;APPROOT&gt;/Library/Application Support"</td></tr>
    /// </tbody></table></div>
    /// <p>In the table above, <code>&lt;APPNAME&gt;</code> is usually the organization name, the application name, or both, or a unique name generated at packaging. Similarly, &lt;APPROOT&gt; is the location where this application is installed (often a sandbox). &lt;APPDIR&gt; is the directory containing the application executable.</p>
    /// <p>The paths above should not be relied upon, as they may change according to OS configuration, locale, or they may change in future Qt versions.</p>
    /// <p><b>Note: </b>On Android, applications with open files on the external storage (&lt;USER&gt; locations), will be killed if the external storage is unmounted.</p><p><b>Note: </b>On iOS, if you do pass <code>QStandardPaths::standardLocations(QStandardPaths::PicturesLocation).last()</code> as argument to <a href="http://doc.qt.io/qt-5/qfiledialog.html#setDirectory-1">QFileDialog::setDirectory</a>(), a native image picker dialog will be used for accessing the user's photo album. The filename returned can be loaded using <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> and related APIs. This feature was added in Qt 5.5.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qstandardpaths.html#writableLocation">writableLocation</a>(), <a href="http://doc.qt.io/qt-5/qstandardpaths.html#standardLocations">standardLocations</a>(), <a href="http://doc.qt.io/qt-5/qstandardpaths.html#displayName">displayName</a>(), <a href="http://doc.qt.io/qt-5/qstandardpaths.html#locate">locate</a>(), and <a href="http://doc.qt.io/qt-5/qstandardpaths.html#locateAll">locateAll</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct StandardLocation(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for StandardLocation {
        fn from(value: ::std::os::raw::c_int) -> Self {
            StandardLocation(value)
        }
    }

    impl From<StandardLocation> for ::std::os::raw::c_int {
        fn from(value: StandardLocation) -> Self {
            value.0
        }
    }

    impl StandardLocation {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl StandardLocation {
        /// Returns the user's desktop directory. This is a generic value. On systems with no concept of a desktop. (C++ enum variant: <span style='color: green;'>```DesktopLocation = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const DesktopLocation: crate::q_standard_paths::StandardLocation =
            crate::q_standard_paths::StandardLocation(0);
        /// Returns the directory containing user document files. This is a generic value. The returned path is never empty. (C++ enum variant: <span style='color: green;'>```DocumentsLocation = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const DocumentsLocation: crate::q_standard_paths::StandardLocation =
            crate::q_standard_paths::StandardLocation(1);
        /// Returns the directory containing user's fonts. This is a generic value. Note that installing fonts may require additional, platform-specific operations. (C++ enum variant: <span style='color: green;'>```FontsLocation = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const FontsLocation: crate::q_standard_paths::StandardLocation =
            crate::q_standard_paths::StandardLocation(2);
        /// Returns the directory containing the user applications (either executables, application bundles, or shortcuts to them). This is a generic value. Note that installing applications may require additional, platform-specific operations. Files, folders or shortcuts in this directory are platform-specific. (C++ enum variant: <span style='color: green;'>```ApplicationsLocation = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const ApplicationsLocation: crate::q_standard_paths::StandardLocation =
            crate::q_standard_paths::StandardLocation(3);
        /// Returns the directory containing the user's music or other audio files. This is a generic value. If no directory specific for music files exists, a sensible fallback for storing user documents is returned. (C++ enum variant: <span style='color: green;'>```MusicLocation = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const MusicLocation: crate::q_standard_paths::StandardLocation =
            crate::q_standard_paths::StandardLocation(4);
        /// Returns the directory containing the user's movies and videos. This is a generic value. If no directory specific for movie files exists, a sensible fallback for storing user documents is returned. (C++ enum variant: <span style='color: green;'>```MoviesLocation = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const MoviesLocation: crate::q_standard_paths::StandardLocation =
            crate::q_standard_paths::StandardLocation(5);
        /// Returns the directory containing the user's pictures or photos. This is a generic value. If no directory specific for picture files exists, a sensible fallback for storing user documents is returned. (C++ enum variant: <span style='color: green;'>```PicturesLocation = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const PicturesLocation: crate::q_standard_paths::StandardLocation =
            crate::q_standard_paths::StandardLocation(6);
        /// Returns a directory where temporary files can be stored. The returned value might be application-specific, shared among other applications for this user, or even system-wide. The returned path is never empty. (C++ enum variant: <span style='color: green;'>```TempLocation = 7```</span>)
        #[allow(non_upper_case_globals)]
        pub const TempLocation: crate::q_standard_paths::StandardLocation =
            crate::q_standard_paths::StandardLocation(7);
        /// Returns the user's home directory (the same as <a href="http://doc.qt.io/qt-5/qdir.html#homePath">QDir::homePath</a>()). On Unix systems, this is equal to the HOME environment variable. This value might be generic or application-specific, but the returned path is never empty. (C++ enum variant: <span style='color: green;'>```HomeLocation = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const HomeLocation: crate::q_standard_paths::StandardLocation =
            crate::q_standard_paths::StandardLocation(8);
        /// Returns the same value as AppLocalDataLocation. This enumeration value is deprecated. Using AppDataLocation is preferable since on Windows, the roaming path is recommended. (C++ enum variant: <span style='color: green;'>```DataLocation = 9```</span>)
        #[allow(non_upper_case_globals)]
        pub const DataLocation: crate::q_standard_paths::StandardLocation =
            crate::q_standard_paths::StandardLocation(9);
        /// Returns a directory location where user-specific non-essential (cached) data should be written. This is an application-specific directory. The returned path is never empty. (C++ enum variant: <span style='color: green;'>```CacheLocation = 10```</span>)
        #[allow(non_upper_case_globals)]
        pub const CacheLocation: crate::q_standard_paths::StandardLocation =
            crate::q_standard_paths::StandardLocation(10);
        /// Returns a directory location where persistent data shared across applications can be stored. This is a generic value. The returned path is never empty. (C++ enum variant: <span style='color: green;'>```GenericDataLocation = 11```</span>)
        #[allow(non_upper_case_globals)]
        pub const GenericDataLocation: crate::q_standard_paths::StandardLocation =
            crate::q_standard_paths::StandardLocation(11);
        /// Returns a directory location where runtime communication files should be written, like Unix local sockets. This is a generic value. The returned path may be empty on some systems. (C++ enum variant: <span style='color: green;'>```RuntimeLocation = 12```</span>)
        #[allow(non_upper_case_globals)]
        pub const RuntimeLocation: crate::q_standard_paths::StandardLocation =
            crate::q_standard_paths::StandardLocation(12);
        /// Returns a directory location where user-specific configuration files should be written. This may be either a generic value or application-specific, and the returned path is never empty. (C++ enum variant: <span style='color: green;'>```ConfigLocation = 13```</span>)
        #[allow(non_upper_case_globals)]
        pub const ConfigLocation: crate::q_standard_paths::StandardLocation =
            crate::q_standard_paths::StandardLocation(13);
        /// Returns a directory for user's downloaded files. This is a generic value. If no directory specific for downloads exists, a sensible fallback for storing user documents is returned. (C++ enum variant: <span style='color: green;'>```DownloadLocation = 14```</span>)
        #[allow(non_upper_case_globals)]
        pub const DownloadLocation: crate::q_standard_paths::StandardLocation =
            crate::q_standard_paths::StandardLocation(14);
        /// Returns a directory location where user-specific non-essential (cached) data, shared across applications, should be written. This is a generic value. Note that the returned path may be empty if the system has no concept of shared cache. (C++ enum variant: <span style='color: green;'>```GenericCacheLocation = 15```</span>)
        #[allow(non_upper_case_globals)]
        pub const GenericCacheLocation: crate::q_standard_paths::StandardLocation =
            crate::q_standard_paths::StandardLocation(15);
        /// Returns a directory location where user-specific configuration files shared between multiple applications should be written. This is a generic value and the returned path is never empty. (C++ enum variant: <span style='color: green;'>```GenericConfigLocation = 16```</span>)
        #[allow(non_upper_case_globals)]
        pub const GenericConfigLocation: crate::q_standard_paths::StandardLocation =
            crate::q_standard_paths::StandardLocation(16);
        /// Returns a directory location where persistent application data can be stored. This is an application-specific directory. To obtain a path to store data to be shared with other applications, use QStandardPaths::GenericDataLocation. The returned path is never empty. On the Windows operating system, this returns the roaming path. This enum value was added in Qt 5.4. (C++ enum variant: <span style='color: green;'>```AppDataLocation = 17```</span>)
        #[allow(non_upper_case_globals)]
        pub const AppDataLocation: crate::q_standard_paths::StandardLocation =
            crate::q_standard_paths::StandardLocation(17);
        /// Returns a directory location where user-specific configuration files should be written. This is an application-specific directory, and the returned path is never empty. This enum value was added in Qt 5.5. (C++ enum variant: <span style='color: green;'>```AppConfigLocation = 18```</span>)
        #[allow(non_upper_case_globals)]
        pub const AppConfigLocation: crate::q_standard_paths::StandardLocation =
            crate::q_standard_paths::StandardLocation(18);
        /// Returns the local settings path on the Windows operating system. On all other platforms, it returns the same value as AppDataLocation. This enum value was added in Qt 5.4. (C++ enum variant: <span style='color: green;'>```AppLocalDataLocation = 9```</span>)
        #[allow(non_upper_case_globals)]
        pub const AppLocalDataLocation: crate::q_standard_paths::StandardLocation =
            crate::q_standard_paths::StandardLocation(9);
    }

    /// <p>This enum describes the different flags that can be used for controlling the behavior of <a href="http://doc.qt.io/qt-5/qstandardpaths.html#locate">QStandardPaths::locate</a> and <a href="http://doc.qt.io/qt-5/qstandardpaths.html#locateAll">QStandardPaths::locateAll</a>.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QStandardPaths::LocateOption```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstandardpaths.html#LocateOption-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the different flags that can be used for controlling the behavior of <a href="http://doc.qt.io/qt-5/qstandardpaths.html#locate">QStandardPaths::locate</a> and <a href="http://doc.qt.io/qt-5/qstandardpaths.html#locateAll">QStandardPaths::locateAll</a>.</p>
    ///
    /// <p>The LocateOptions type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;LocateOption&gt;. It stores an OR combination of LocateOption values.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct LocateOption(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for LocateOption {
        fn from(value: ::std::os::raw::c_int) -> Self {
            LocateOption(value)
        }
    }

    impl From<LocateOption> for ::std::os::raw::c_int {
        fn from(value: LocateOption) -> Self {
            value.0
        }
    }

    impl LocateOption {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl LocateOption {
        /// return only files (C++ enum variant: <span style='color: green;'>```LocateFile = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const LocateFile: crate::q_standard_paths::LocateOption =
            crate::q_standard_paths::LocateOption(0);
        /// return only directories (C++ enum variant: <span style='color: green;'>```LocateDirectory = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const LocateDirectory: crate::q_standard_paths::LocateOption =
            crate::q_standard_paths::LocateOption(1);
    }

    impl From<crate::q_standard_paths::LocateOption>
        for crate::QFlags<crate::q_standard_paths::LocateOption>
    {
        fn from(value: crate::q_standard_paths::LocateOption) -> Self {
            Self::from(value.to_int())
        }
    }

    impl<T: Into<crate::QFlags<crate::q_standard_paths::LocateOption>>> std::ops::BitOr<T>
        for crate::q_standard_paths::LocateOption
    {
        type Output = crate::QFlags<crate::q_standard_paths::LocateOption>;
        fn bitor(self, rhs: T) -> crate::QFlags<crate::q_standard_paths::LocateOption> {
            Into::<crate::QFlags<crate::q_standard_paths::LocateOption>>::into(self) | rhs
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qstandardpaths.html">QStandardPaths</a> class provides methods for accessing standard paths.</p>
///
/// C++ class: <span style='color: green;'>```QStandardPaths```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qstandardpaths.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qstandardpaths.html">QStandardPaths</a> class provides methods for accessing standard paths.</p>
/// <p>This class contains functions to query standard locations on the local filesystem, for common tasks such as user-specific directories or system-wide configuration directories.</p></div>
#[repr(C)]
pub struct QStandardPaths {
    _unused: u8,
}
impl QStandardPaths {
    /// <p>The <a href="http://doc.qt.io/qt-5/qstandardpaths.html">QStandardPaths</a> class provides methods for accessing standard paths.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStandardPaths& QStandardPaths::operator=(const QStandardPaths& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstandardpaths.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qstandardpaths.html">QStandardPaths</a> class provides methods for accessing standard paths.</p>
    /// <p>This class contains functions to query standard locations on the local filesystem, for common tasks such as user-specific directories or system-wide configuration directories.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStandardPaths>>,
    ) -> ::cpp_utils::MutRef<crate::QStandardPaths> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStandardPaths_operator_(
            self as *mut crate::QStandardPaths,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStandardPaths>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a localized display name for the given location <i>type</i> or an empty <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> if no relevant location can be found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QStandardPaths::displayName(QStandardPaths::StandardLocation type)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstandardpaths.html#displayName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a localized display name for the given location <i>type</i> or an empty <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> if no relevant location can be found.</p></div>
    pub unsafe fn display_name(
        type_: crate::q_standard_paths::StandardLocation,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStandardPaths_displayName(type_);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is obsolete. It is provided to keep old source code working. We strongly advise against using it in new code.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QStandardPaths::enableTestMode(bool testMode)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstandardpaths-obsolete.html#enableTestMode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is obsolete. It is provided to keep old source code working. We strongly advise against using it in new code.</p></div>
    pub unsafe fn enable_test_mode(test_mode: bool) {
        crate::__ffi::ctr_qt_core_ffi_QStandardPaths_enableTestMode(test_mode)
    }

    /// <p>Finds the executable named <i>executableName</i> in the paths specified by <i>paths</i>, or the system paths if <i>paths</i> is empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QStandardPaths::findExecutable(const QString& executableName, const QStringList& paths = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstandardpaths.html#findExecutable">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Finds the executable named <i>executableName</i> in the paths specified by <i>paths</i>, or the system paths if <i>paths</i> is empty.</p>
    /// <p>On most operating systems the system path is determined by the PATH environment variable.</p>
    /// <p>The directories where to search for the executable can be set in the <i>paths</i> argument. To search in both your own paths and the system paths, call findExecutable twice, once with <i>paths</i> set and once with <i>paths</i> empty.</p>
    /// <p>Symlinks are not resolved, in order to preserve behavior for the case of executables whose behavior depends on the name they are invoked with.</p>
    /// <p><b>Note: </b>On Windows, the usual executable extensions (from the PATHEXT environment variable) are automatically appended, so that for instance findExecutable("foo") will find foo.exe or foo.bat if present.</p><p>Returns the absolute file path to the executable, or an empty string if not found.</p></div>
    pub unsafe fn find_executable_2a(
        executable_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        paths: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStandardPaths_findExecutable(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(executable_name)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(paths)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Finds the executable named <i>executableName</i> in the paths specified by <i>paths</i>, or the system paths if <i>paths</i> is empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QStandardPaths::findExecutable(const QString& executableName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstandardpaths.html#findExecutable">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Finds the executable named <i>executableName</i> in the paths specified by <i>paths</i>, or the system paths if <i>paths</i> is empty.</p>
    /// <p>On most operating systems the system path is determined by the PATH environment variable.</p>
    /// <p>The directories where to search for the executable can be set in the <i>paths</i> argument. To search in both your own paths and the system paths, call findExecutable twice, once with <i>paths</i> set and once with <i>paths</i> empty.</p>
    /// <p>Symlinks are not resolved, in order to preserve behavior for the case of executables whose behavior depends on the name they are invoked with.</p>
    /// <p><b>Note: </b>On Windows, the usual executable extensions (from the PATHEXT environment variable) are automatically appended, so that for instance findExecutable("foo") will find foo.exe or foo.bat if present.</p><p>Returns the absolute file path to the executable, or an empty string if not found.</p></div>
    pub unsafe fn find_executable_1a(
        executable_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStandardPaths_findExecutable1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(executable_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static bool QStandardPaths::isTestModeEnabled()```</span>.
    pub unsafe fn is_test_mode_enabled() -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStandardPaths_isTestModeEnabled()
    }

    /// <p>Tries to find a file or directory called <i>fileName</i> in the standard locations for <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QStandardPaths::locate(QStandardPaths::StandardLocation type, const QString& fileName, QFlags<QStandardPaths::LocateOption> options = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstandardpaths.html#locate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Tries to find a file or directory called <i>fileName</i> in the standard locations for <i>type</i>.</p>
    /// <p>The full path to the first file or directory (depending on <i>options</i>) found is returned. If no such file or directory can be found, an empty string is returned.</p></div>
    pub unsafe fn locate_3a(
        type_: crate::q_standard_paths::StandardLocation,
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        options: crate::QFlags<crate::q_standard_paths::LocateOption>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStandardPaths_locate(
            type_,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Tries to find a file or directory called <i>fileName</i> in the standard locations for <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QStandardPaths::locate(QStandardPaths::StandardLocation type, const QString& fileName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstandardpaths.html#locate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Tries to find a file or directory called <i>fileName</i> in the standard locations for <i>type</i>.</p>
    /// <p>The full path to the first file or directory (depending on <i>options</i>) found is returned. If no such file or directory can be found, an empty string is returned.</p></div>
    pub unsafe fn locate_2a(
        type_: crate::q_standard_paths::StandardLocation,
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStandardPaths_locate1(
            type_,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Tries to find all files or directories called <i>fileName</i> in the standard locations for <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QStringList QStandardPaths::locateAll(QStandardPaths::StandardLocation type, const QString& fileName, QFlags<QStandardPaths::LocateOption> options = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstandardpaths.html#locateAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Tries to find all files or directories called <i>fileName</i> in the standard locations for <i>type</i>.</p>
    /// <p>The <i>options</i> flag allows to specify whether to look for files or directories.</p>
    /// <p>Returns the list of all the files that were found.</p></div>
    pub unsafe fn locate_all_3a(
        type_: crate::q_standard_paths::StandardLocation,
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        options: crate::QFlags<crate::q_standard_paths::LocateOption>,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStandardPaths_locateAll(
            type_,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Tries to find all files or directories called <i>fileName</i> in the standard locations for <i>type</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QStringList QStandardPaths::locateAll(QStandardPaths::StandardLocation type, const QString& fileName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstandardpaths.html#locateAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Tries to find all files or directories called <i>fileName</i> in the standard locations for <i>type</i>.</p>
    /// <p>The <i>options</i> flag allows to specify whether to look for files or directories.</p>
    /// <p>Returns the list of all the files that were found.</p></div>
    pub unsafe fn locate_all_2a(
        type_: crate::q_standard_paths::StandardLocation,
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStandardPaths_locateAll1(
            type_,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qstandardpaths.html">QStandardPaths</a> class provides methods for accessing standard paths.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStandardPaths::QStandardPaths(const QStandardPaths& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstandardpaths.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qstandardpaths.html">QStandardPaths</a> class provides methods for accessing standard paths.</p>
    /// <p>This class contains functions to query standard locations on the local filesystem, for common tasks such as user-specific directories or system-wide configuration directories.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStandardPaths>>,
    ) -> ::cpp_utils::MutRef<crate::QStandardPaths> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStandardPaths_QStandardPaths(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStandardPaths>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>If <i>testMode</i> is true, this enables a special "test mode" in <a href="http://doc.qt.io/qt-5/qstandardpaths.html">QStandardPaths</a>, which changes writable locations to point to test directories, in order to prevent auto tests from reading from or writing to the current user's configuration.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QStandardPaths::setTestModeEnabled(bool testMode)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstandardpaths.html#setTestModeEnabled">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If <i>testMode</i> is true, this enables a special "test mode" in <a href="http://doc.qt.io/qt-5/qstandardpaths.html">QStandardPaths</a>, which changes writable locations to point to test directories, in order to prevent auto tests from reading from or writing to the current user's configuration.</p>
    /// <p>This affects the locations into which test programs might write files: <a href="http://doc.qt.io/qt-5/qstandardpaths.html#StandardLocation-enum">GenericDataLocation</a>, <a href="http://doc.qt.io/qt-5/qstandardpaths.html#StandardLocation-enum">DataLocation</a>, <a href="http://doc.qt.io/qt-5/qstandardpaths.html#StandardLocation-enum">ConfigLocation</a>, <a href="http://doc.qt.io/qt-5/qstandardpaths.html#StandardLocation-enum">GenericConfigLocation</a>, <a href="http://doc.qt.io/qt-5/qstandardpaths.html#StandardLocation-enum">AppConfigLocation</a>, <a href="http://doc.qt.io/qt-5/qstandardpaths.html#StandardLocation-enum">GenericCacheLocation</a>, <a href="http://doc.qt.io/qt-5/qstandardpaths.html#StandardLocation-enum">CacheLocation</a>. Other locations are not affected.</p>
    /// <p>On Unix, <code>XDG_DATA_HOME</code> is set to <i>~/.qttest/share</i>, <code>XDG_CONFIG_HOME</code> is set to <i>~/.qttest/config</i>, and <code>XDG_CACHE_HOME</code> is set to <i>~/.qttest/cache</i>.</p>
    /// <p>On <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, data goes to <i>~/.qttest/Application</i> Support, cache goes to <i>~/.qttest/Cache</i>, and config goes to <i>~/.qttest/Preferences</i>.</p>
    /// <p>On Windows, everything goes to a "qttest" directory under Application Data.</p></div>
    pub unsafe fn set_test_mode_enabled(test_mode: bool) {
        crate::__ffi::ctr_qt_core_ffi_QStandardPaths_setTestModeEnabled(test_mode)
    }

    /// <p>Returns all the directories where files of <i>type</i> belong.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QStringList QStandardPaths::standardLocations(QStandardPaths::StandardLocation type)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstandardpaths.html#standardLocations">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns all the directories where files of <i>type</i> belong.</p>
    /// <p>The list of directories is sorted from high to low priority, starting with <a href="http://doc.qt.io/qt-5/qstandardpaths.html#writableLocation">writableLocation</a>() if it can be determined. This list is empty if no locations for <i>type</i> are defined.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstandardpaths.html#writableLocation">writableLocation</a>().</p></div>
    pub unsafe fn standard_locations(
        type_: crate::q_standard_paths::StandardLocation,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStandardPaths_standardLocations(type_);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStandardPaths_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the directory where files of <i>type</i> should be written to, or an empty string if the location cannot be determined.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QStandardPaths::writableLocation(QStandardPaths::StandardLocation type)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstandardpaths.html#writableLocation">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the directory where files of <i>type</i> should be written to, or an empty string if the location cannot be determined.</p>
    /// <p><b>Note: </b>The storage location returned can be a directory that does not exist; i.e., it may need to be created by the system or the user.</p></div>
    pub unsafe fn writable_location(
        type_: crate::q_standard_paths::StandardLocation,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStandardPaths_writableLocation(type_);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_state {
    //! C++ type: <span style='color: green;'>```QState```</span>

    /// <p>This enum specifies how a state's child states are treated.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QState::ChildMode```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstate.html#ChildMode-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum specifies how a state's child states are treated.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct ChildMode(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for ChildMode {
        fn from(value: ::std::os::raw::c_int) -> Self {
            ChildMode(value)
        }
    }

    impl From<ChildMode> for ::std::os::raw::c_int {
        fn from(value: ChildMode) -> Self {
            value.0
        }
    }

    impl ChildMode {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl ChildMode {
        /// The child states are mutually exclusive and an initial state must be set by calling <a href="http://doc.qt.io/qt-5/qstate.html#setInitialState">QState::setInitialState</a>(). (C++ enum variant: <span style='color: green;'>```ExclusiveStates = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const ExclusiveStates: crate::q_state::ChildMode = crate::q_state::ChildMode(0);
        /// The child states are parallel. When the parent state is entered, all its child states are entered in parallel. (C++ enum variant: <span style='color: green;'>```ParallelStates = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const ParallelStates: crate::q_state::ChildMode = crate::q_state::ChildMode(1);
    }

    /// <p>This enum specifies the restore policy type. The restore policy takes effect when the machine enters a state which sets one or more properties. If the restore policy is set to RestoreProperties, the state machine will save the original value of the property before the new value is set.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QState::RestorePolicy```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstate.html#RestorePolicy-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum specifies the restore policy type. The restore policy takes effect when the machine enters a state which sets one or more properties. If the restore policy is set to RestoreProperties, the state machine will save the original value of the property before the new value is set.</p>
    /// <p>Later, when the machine either enters a state which does not set a value for the given property, the property will automatically be restored to its initial value.</p>
    /// <p>Only one initial value will be saved for any given property. If a value for a property has already been saved by the state machine, it will not be overwritten until the property has been successfully restored.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstatemachine.html#globalRestorePolicy-prop">QStateMachine::globalRestorePolicy</a> and <a href="http://doc.qt.io/qt-5/qstate.html#assignProperty">QState::assignProperty</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct RestorePolicy(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for RestorePolicy {
        fn from(value: ::std::os::raw::c_int) -> Self {
            RestorePolicy(value)
        }
    }

    impl From<RestorePolicy> for ::std::os::raw::c_int {
        fn from(value: RestorePolicy) -> Self {
            value.0
        }
    }

    impl RestorePolicy {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl RestorePolicy {
        /// The state machine should not save the initial values of properties and restore them later. (C++ enum variant: <span style='color: green;'>```DontRestoreProperties = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const DontRestoreProperties: crate::q_state::RestorePolicy =
            crate::q_state::RestorePolicy(0);
        /// The state machine should save the initial values of properties and restore them later. (C++ enum variant: <span style='color: green;'>```RestoreProperties = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const RestoreProperties: crate::q_state::RestorePolicy =
            crate::q_state::RestorePolicy(1);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qstate.html">QState</a> class provides a general-purpose state for <a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a>.</p>
///
/// C++ class: <span style='color: green;'>```QState```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qstate.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qstate.html">QState</a> class provides a general-purpose state for <a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a>.</p>
/// <p><a href="http://doc.qt.io/qt-5/qstate.html">QState</a> objects can have child states, and can have transitions to other states. <a href="http://doc.qt.io/qt-5/qstate.html">QState</a> is part of <a href="http://doc.qt.io/qt-5/statemachine-api.html">The State Machine Framework</a>.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qstate.html#addTransition">addTransition</a>() function adds a transition. The <a href="http://doc.qt.io/qt-5/qstate.html#removeTransition">removeTransition</a>() function removes a transition. The <a href="http://doc.qt.io/qt-5/qstate.html#transitions">transitions</a>() function returns the state's outgoing transitions.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qstate.html#assignProperty">assignProperty</a>() function is used for defining property assignments that should be performed when a state is entered.</p>
/// <p>Top-level states must be passed a <a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a> object as their parent state, or added to a state machine using <a href="http://doc.qt.io/qt-5/qstatemachine.html#addState">QStateMachine::addState</a>().</p>
/// <a name="states-with-child-states"></a></div>
#[repr(C)]
pub struct QState {
    _unused: u8,
}
impl QState {
    /// <p>This signal is emitted when a final child state of this state is entered.</p>
    ///
    /// Returns a built-in Qt signal `QState::finished` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstate.html#finished">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when a final child state of this state is entered.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfinalstate.html">QFinalState</a>.</p></div>
    pub fn finished(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2finished()\0"),
            )
        }
    }

    /// <p>This signal is emitted when all properties have been assigned their final value. If the state assigns a value to one or more properties for which an animation exists (either set on the transition or as a default animation on the state machine), then the signal will not be emitted until all such animations have finished playing.</p>
    ///
    /// Returns a built-in Qt signal `QState::propertiesAssigned` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstate.html#propertiesAssigned">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when all properties have been assigned their final value. If the state assigns a value to one or more properties for which an animation exists (either set on the transition or as a default animation on the state machine), then the signal will not be emitted until all such animations have finished playing.</p>
    /// <p>If there are no relevant animations, or no property assignments defined for the state, then the signal will be emitted immediately before the state is entered.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstate.html#assignProperty">QState::assignProperty</a>() and <a href="http://doc.qt.io/qt-5/qabstracttransition.html#addAnimation">QAbstractTransition::addAnimation</a>().</p></div>
    pub fn properties_assigned(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2propertiesAssigned()\0"),
            )
        }
    }

    /// <p>This signal is emitted when the <a href="http://doc.qt.io/qt-5/qstate.html#childMode-prop">childMode</a> property is changed.</p>
    ///
    /// Returns a built-in Qt signal `QState::childModeChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstate.html#childModeChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the <a href="http://doc.qt.io/qt-5/qstate.html#childMode-prop">childMode</a> property is changed.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>Note:</b> Notifier signal for property <a href="http://doc.qt.io/qt-5/qstate.html#childMode-prop">childMode</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qstate.html#childMode-prop">QState::childMode</a>.</p></div>
    pub fn child_mode_changed(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2childModeChanged()\0"),
            )
        }
    }

    /// <p>This signal is emitted when the <a href="http://doc.qt.io/qt-5/qstate.html#initialState-prop">initialState</a> property is changed.</p>
    ///
    /// Returns a built-in Qt signal `QState::initialStateChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstate.html#initialStateChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the <a href="http://doc.qt.io/qt-5/qstate.html#initialState-prop">initialState</a> property is changed.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>Note:</b> Notifier signal for property <a href="http://doc.qt.io/qt-5/qstate.html#initialState-prop">initialState</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qstate.html#initialState-prop">QState::initialState</a>.</p></div>
    pub fn initial_state_changed(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2initialStateChanged()\0"),
            )
        }
    }

    /// <p>This signal is emitted when the <a href="http://doc.qt.io/qt-5/qstate.html#errorState-prop">errorState</a> property is changed.</p>
    ///
    /// Returns a built-in Qt signal `QState::errorStateChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstate.html#errorStateChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the <a href="http://doc.qt.io/qt-5/qstate.html#errorState-prop">errorState</a> property is changed.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>Note:</b> Notifier signal for property <a href="http://doc.qt.io/qt-5/qstate.html#errorState-prop">errorState</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qstate.html#errorState-prop">QState::errorState</a>.</p></div>
    pub fn error_state_changed(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2errorStateChanged()\0"),
            )
        }
    }

    /// <p>Adds the given <i>transition</i>. The transition has this state as the source. This state takes ownership of the transition.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QState::addTransition(QAbstractTransition* transition)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstate.html#addTransition">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds the given <i>transition</i>. The transition has this state as the source. This state takes ownership of the transition.</p></div>
    pub unsafe fn add_transition_q_abstract_transition(
        &mut self,
        transition: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractTransition>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QState_addTransition(
            self as *mut crate::QState,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractTransition>>::cast_into(
                transition,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Adds a transition associated with the given <i>signal</i> of the given <i>sender</i> object, and returns the new <a href="http://doc.qt.io/qt-5/qsignaltransition.html">QSignalTransition</a> object. The transition has this state as the source, and the given <i>target</i> as the target state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSignalTransition* QState::addTransition(const QObject* sender, const char* signal, QAbstractState* target)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstate.html#addTransition-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds a transition associated with the given <i>signal</i> of the given <i>sender</i> object, and returns the new <a href="http://doc.qt.io/qt-5/qsignaltransition.html">QSignalTransition</a> object. The transition has this state as the source, and the given <i>target</i> as the target state.</p></div>
    pub unsafe fn add_transition_q_object_char_q_abstract_state(
        &mut self,
        sender: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        signal: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        target: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractState>>,
    ) -> ::cpp_utils::MutPtr<crate::QSignalTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QState_addTransition1(
            self as *mut crate::QState,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(sender)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(signal)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractState>>::cast_into(target)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Adds an unconditional transition from this state to the given <i>target</i> state, and returns then new transition object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractTransition* QState::addTransition(QAbstractState* target)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstate.html#addTransition-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds an unconditional transition from this state to the given <i>target</i> state, and returns then new transition object.</p></div>
    pub unsafe fn add_transition_q_abstract_state(
        &mut self,
        target: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractState>>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QState_addTransition2(
            self as *mut crate::QState,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractState>>::cast_into(target)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Instructs this state to set the property with the given <i>name</i> of the given <i>object</i> to the given <i>value</i> when the state is entered.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QState::assignProperty(QObject* object, const char* name, const QVariant& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstate.html#assignProperty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Instructs this state to set the property with the given <i>name</i> of the given <i>object</i> to the given <i>value</i> when the state is entered.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstate.html#propertiesAssigned">propertiesAssigned</a>().</p></div>
    pub unsafe fn assign_property(
        &mut self,
        object: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QState_assignProperty(
            self as *mut crate::QState,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(object)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(name)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>This property holds the child mode of this state</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QState::ChildMode QState::childMode() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstate.html#childMode-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the child mode of this state</p>
    /// <p>The default value of this property is <a href="http://doc.qt.io/qt-5/qstate.html#ChildMode-enum">QState::ExclusiveStates</a>.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> ChildMode </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qstate.html#childMode">childMode</a></b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qstate.html#setChildMode">setChildMode</a></b></span>(ChildMode <i>mode</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qstate.html#childModeChanged">childModeChanged</a></b></span>()</td><td class="memItemRight bottomAlign">[see note below]</td></tr>
    /// </tbody></table></div>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p></div>
    pub unsafe fn child_mode(&self) -> crate::q_state::ChildMode {
        crate::__ffi::ctr_qt_core_ffi_QState_childMode(self as *const crate::QState)
    }

    /// <p>This property holds the error state of this state</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractState* QState::errorState() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstate.html#errorState-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the error state of this state</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QAbstractState *</td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qstate.html#errorState">errorState</a></b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qstate.html#setErrorState">setErrorState</a></b></span>(QAbstractState *<i>state</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qstate.html#errorStateChanged">errorStateChanged</a></b></span>()</td><td class="memItemRight bottomAlign">[see note below]</td></tr>
    /// </tbody></table></div>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p></div>
    pub unsafe fn error_state(&self) -> ::cpp_utils::MutPtr<crate::QAbstractState> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QState_errorState(self as *const crate::QState);
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This property holds the initial state of this state (one of its child states)</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractState* QState::initialState() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstate.html#initialState-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the initial state of this state (one of its child states)</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QAbstractState *</td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qstate.html#initialState">initialState</a></b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qstate.html#setInitialState">setInitialState</a></b></span>(QAbstractState *<i>state</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qstate.html#initialStateChanged">initialStateChanged</a></b></span>()</td><td class="memItemRight bottomAlign">[see note below]</td></tr>
    /// </tbody></table></div>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p></div>
    pub unsafe fn initial_state(&self) -> ::cpp_utils::MutPtr<crate::QAbstractState> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QState_initialState(self as *const crate::QState);
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QState::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QState_metaObject(self as *const crate::QState);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs a new state with the given <i>parent</i> state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QState::QState(QState* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstate.html#QState">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new state with the given <i>parent</i> state.</p></div>
    pub unsafe fn from_q_state(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QState>>,
    ) -> ::cpp_utils::CppBox<crate::QState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QState_QState(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QState>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new state with the given <i>childMode</i> and the given <i>parent</i> state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QState::QState(QState::ChildMode childMode, QState* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstate.html#QState-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new state with the given <i>childMode</i> and the given <i>parent</i> state.</p></div>
    pub unsafe fn from_child_mode_q_state(
        child_mode: crate::q_state::ChildMode,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QState>>,
    ) -> ::cpp_utils::CppBox<crate::QState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QState_QState1(
            child_mode,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QState>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qstate.html">QState</a> class provides a general-purpose state for <a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QState::QState()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstate.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qstate.html">QState</a> class provides a general-purpose state for <a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qstate.html">QState</a> objects can have child states, and can have transitions to other states. <a href="http://doc.qt.io/qt-5/qstate.html">QState</a> is part of <a href="http://doc.qt.io/qt-5/statemachine-api.html">The State Machine Framework</a>.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qstate.html#addTransition">addTransition</a>() function adds a transition. The <a href="http://doc.qt.io/qt-5/qstate.html#removeTransition">removeTransition</a>() function removes a transition. The <a href="http://doc.qt.io/qt-5/qstate.html#transitions">transitions</a>() function returns the state's outgoing transitions.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qstate.html#assignProperty">assignProperty</a>() function is used for defining property assignments that should be performed when a state is entered.</p>
    /// <p>Top-level states must be passed a <a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a> object as their parent state, or added to a state machine using <a href="http://doc.qt.io/qt-5/qstatemachine.html#addState">QStateMachine::addState</a>().</p>
    /// <a name="states-with-child-states"></a></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QState_QState2();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new state with the given <i>childMode</i> and the given <i>parent</i> state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QState::QState(QState::ChildMode childMode)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstate.html#QState-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new state with the given <i>childMode</i> and the given <i>parent</i> state.</p></div>
    pub unsafe fn from_child_mode(
        child_mode: crate::q_state::ChildMode,
    ) -> ::cpp_utils::CppBox<crate::QState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QState_QState3(child_mode);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QState::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QState_qt_metacall(
            self as *mut crate::QState,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QState::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QState_qt_metacast(
            self as *mut crate::QState,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Removes the given <i>transition</i> from this state. The state releases ownership of the transition.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QState::removeTransition(QAbstractTransition* transition)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstate.html#removeTransition">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the given <i>transition</i> from this state. The state releases ownership of the transition.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstate.html#addTransition">addTransition</a>().</p></div>
    pub unsafe fn remove_transition(
        &mut self,
        transition: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractTransition>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QState_removeTransition(
            self as *mut crate::QState,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractTransition>>::cast_into(
                transition,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Sets the child <i>mode</i> of this state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QState::setChildMode(QState::ChildMode mode)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstate.html#setChildMode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the child <i>mode</i> of this state.</p>
    /// <p><b>Note:</b> Setter function for property <a href="http://doc.qt.io/qt-5/qstate.html#childMode-prop">childMode</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qstate.html#childMode">childMode</a>().</p></div>
    pub unsafe fn set_child_mode(&mut self, mode: crate::q_state::ChildMode) {
        crate::__ffi::ctr_qt_core_ffi_QState_setChildMode(self as *mut crate::QState, mode)
    }

    /// <p>Sets this state's error state to be the given <i>state</i>. If the error state is not set, or if it is set to 0, the state will inherit its parent's error state recursively. If no error state is set for the state itself or any of its ancestors, an error will cause the machine to stop executing and an error will be printed to the console.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QState::setErrorState(QAbstractState* state)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstate.html#setErrorState">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets this state's error state to be the given <i>state</i>. If the error state is not set, or if it is set to 0, the state will inherit its parent's error state recursively. If no error state is set for the state itself or any of its ancestors, an error will cause the machine to stop executing and an error will be printed to the console.</p>
    /// <p><b>Note:</b> Setter function for property <a href="http://doc.qt.io/qt-5/qstate.html#errorState-prop">errorState</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qstate.html#errorState">errorState</a>().</p></div>
    pub unsafe fn set_error_state(
        &mut self,
        state: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractState>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QState_setErrorState(
            self as *mut crate::QState,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractState>>::cast_into(state)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Sets this state's initial state to be the given <i>state</i>. <i>state</i> has to be a child of this state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QState::setInitialState(QAbstractState* state)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstate.html#setInitialState">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets this state's initial state to be the given <i>state</i>. <i>state</i> has to be a child of this state.</p>
    /// <p><b>Note:</b> Setter function for property <a href="http://doc.qt.io/qt-5/qstate.html#initialState-prop">initialState</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qstate.html#initialState">initialState</a>().</p></div>
    pub unsafe fn set_initial_state(
        &mut self,
        state: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractState>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QState_setInitialState(
            self as *mut crate::QState,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractState>>::cast_into(state)
                .as_mut_raw_ptr(),
        )
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QState_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QState::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QState_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QState::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QState_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns this state's outgoing transitions (i.e. transitions where this state is the <a href="http://doc.qt.io/qt-5/qabstracttransition.html#sourceState">source state</a>), or an empty list if this state has no outgoing transitions.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*> QState::transitions() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstate.html#transitions">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns this state's outgoing transitions (i.e. transitions where this state is the <a href="http://doc.qt.io/qt-5/qabstracttransition.html#sourceState">source state</a>), or an empty list if this state has no outgoing transitions.</p>
    /// <p>This function was introduced in  Qt 4.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstate.html#addTransition">addTransition</a>().</p></div>
    pub unsafe fn transitions(&self) -> ::cpp_utils::CppBox<crate::QListOfQAbstractTransition> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QState_transitions(self as *const crate::QState);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_state_machine {
    //! C++ type: <span style='color: green;'>```QStateMachine```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html">SignalEvent</a> class represents a Qt signal event.</p>
    ///
    /// C++ class: <span style='color: green;'>```QStateMachine::SignalEvent```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html">SignalEvent</a> class represents a Qt signal event.</p>
    /// <p>A signal event is generated by a <a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a> in response to a Qt signal. The <a href="http://doc.qt.io/qt-5/qsignaltransition.html">QSignalTransition</a> class provides a transition associated with a signal event. <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html">QStateMachine::SignalEvent</a> is part of <a href="http://doc.qt.io/qt-5/statemachine-api.html">The State Machine Framework</a>.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html#sender">sender</a>() function returns the object that generated the signal. The <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html#signalIndex">signalIndex</a>() function returns the index of the signal. The <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html#arguments">arguments</a>() function returns the arguments of the signal.</p></div>
    #[repr(C)]
    pub struct SignalEvent {
        _unused: u8,
    }
    impl SignalEvent {
        /// <p>Returns the arguments of the signal.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QVariant> QStateMachine::SignalEvent::arguments() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html#arguments">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the arguments of the signal.</p></div>
        pub unsafe fn arguments(&self) -> ::cpp_utils::CppBox<crate::QListOfQVariant> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStateMachine_SignalEvent_arguments(
                self as *const crate::q_state_machine::SignalEvent,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>The <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html">SignalEvent</a> class represents a Qt signal event.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QStateMachine::SignalEvent& QStateMachine::SignalEvent::operator=(const QStateMachine::SignalEvent& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html">SignalEvent</a> class represents a Qt signal event.</p>
        /// <p>A signal event is generated by a <a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a> in response to a Qt signal. The <a href="http://doc.qt.io/qt-5/qsignaltransition.html">QSignalTransition</a> class provides a transition associated with a signal event. <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html">QStateMachine::SignalEvent</a> is part of <a href="http://doc.qt.io/qt-5/statemachine-api.html">The State Machine Framework</a>.</p>
        /// <p>The <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html#sender">sender</a>() function returns the object that generated the signal. The <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html#signalIndex">signalIndex</a>() function returns the index of the signal. The <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html#arguments">arguments</a>() function returns the arguments of the signal.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_state_machine::SignalEvent>>,
        ) -> ::cpp_utils::MutRef<crate::q_state_machine::SignalEvent> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStateMachine_SignalEvent_operator_(self as *mut crate::q_state_machine::SignalEvent, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_state_machine::SignalEvent>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QStateMachine::SignalEvent::SignalEvent(QObject* sender, int signalIndex, const QList<QVariant>& arguments)```</span>.
        pub unsafe fn new(
            sender: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
            signal_index: ::std::os::raw::c_int,
            arguments: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQVariant>>,
        ) -> ::cpp_utils::CppBox<crate::q_state_machine::SignalEvent> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStateMachine_SignalEvent_SignalEvent(
                ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(sender)
                    .as_mut_raw_ptr(),
                signal_index,
                ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQVariant>>::cast_into(
                    arguments,
                )
                .as_raw_ptr(),
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>The <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html">SignalEvent</a> class represents a Qt signal event.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QStateMachine::SignalEvent::SignalEvent(const QStateMachine::SignalEvent& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html">SignalEvent</a> class represents a Qt signal event.</p>
        /// <p>A signal event is generated by a <a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a> in response to a Qt signal. The <a href="http://doc.qt.io/qt-5/qsignaltransition.html">QSignalTransition</a> class provides a transition associated with a signal event. <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html">QStateMachine::SignalEvent</a> is part of <a href="http://doc.qt.io/qt-5/statemachine-api.html">The State Machine Framework</a>.</p>
        /// <p>The <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html#sender">sender</a>() function returns the object that generated the signal. The <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html#signalIndex">signalIndex</a>() function returns the index of the signal. The <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html#arguments">arguments</a>() function returns the arguments of the signal.</p></div>
        pub unsafe fn new_copy(
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_state_machine::SignalEvent>>,
        ) -> ::cpp_utils::CppBox<crate::q_state_machine::SignalEvent> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStateMachine_SignalEvent_SignalEvent2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_state_machine::SignalEvent>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the object that emitted the signal.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QObject* QStateMachine::SignalEvent::sender() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html#sender">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the object that emitted the signal.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#sender">QObject::sender</a>().</p></div>
        pub unsafe fn sender(&self) -> ::cpp_utils::MutPtr<crate::QObject> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStateMachine_SignalEvent_sender(
                self as *const crate::q_state_machine::SignalEvent,
            );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }

        /// <p>Returns the index of the signal.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```int QStateMachine::SignalEvent::signalIndex() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html#signalIndex">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index of the signal.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfSignal">QMetaObject::indexOfSignal</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#method">QMetaObject::method</a>().</p></div>
        pub unsafe fn signal_index(&self) -> ::std::os::raw::c_int {
            crate::__ffi::ctr_qt_core_ffi_QStateMachine_SignalEvent_signalIndex(
                self as *const crate::q_state_machine::SignalEvent,
            )
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qstatemachine-wrappedevent.html">WrappedEvent</a> class inherits <a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a> and holds a clone of an event associated with a <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QStateMachine::WrappedEvent```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine-wrappedevent.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qstatemachine-wrappedevent.html">WrappedEvent</a> class inherits <a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a> and holds a clone of an event associated with a <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>.</p>
    /// <p>A wrapped event is generated by a <a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a> in response to a Qt event. The <a href="http://doc.qt.io/qt-5/qeventtransition.html">QEventTransition</a> class provides a transition associated with a such an event. <a href="http://doc.qt.io/qt-5/qstatemachine-wrappedevent.html">QStateMachine::WrappedEvent</a> is part of <a href="http://doc.qt.io/qt-5/statemachine-api.html">The State Machine Framework</a>.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qstatemachine-wrappedevent.html#object">object</a>() function returns the object that generated the event. The <a href="http://doc.qt.io/qt-5/qstatemachine-wrappedevent.html#event">event</a>() function returns a clone of the original event.</p></div>
    #[repr(C)]
    pub struct WrappedEvent {
        _unused: u8,
    }
    impl WrappedEvent {
        /// <p>The <a href="http://doc.qt.io/qt-5/qstatemachine-wrappedevent.html">WrappedEvent</a> class inherits <a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a> and holds a clone of an event associated with a <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QStateMachine::WrappedEvent& QStateMachine::WrappedEvent::operator=(const QStateMachine::WrappedEvent& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qstatemachine-wrappedevent.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qstatemachine-wrappedevent.html">WrappedEvent</a> class inherits <a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a> and holds a clone of an event associated with a <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>.</p>
        /// <p>A wrapped event is generated by a <a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a> in response to a Qt event. The <a href="http://doc.qt.io/qt-5/qeventtransition.html">QEventTransition</a> class provides a transition associated with a such an event. <a href="http://doc.qt.io/qt-5/qstatemachine-wrappedevent.html">QStateMachine::WrappedEvent</a> is part of <a href="http://doc.qt.io/qt-5/statemachine-api.html">The State Machine Framework</a>.</p>
        /// <p>The <a href="http://doc.qt.io/qt-5/qstatemachine-wrappedevent.html#object">object</a>() function returns the object that generated the event. The <a href="http://doc.qt.io/qt-5/qstatemachine-wrappedevent.html#event">event</a>() function returns a clone of the original event.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_state_machine::WrappedEvent>>,
        ) -> ::cpp_utils::MutRef<crate::q_state_machine::WrappedEvent> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStateMachine_WrappedEvent_operator_(self as *mut crate::q_state_machine::WrappedEvent, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_state_machine::WrappedEvent>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Returns a clone of the original event.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QEvent* QStateMachine::WrappedEvent::event() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qstatemachine-wrappedevent.html#event">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a clone of the original event.</p></div>
        pub unsafe fn event(&self) -> ::cpp_utils::MutPtr<crate::QEvent> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStateMachine_WrappedEvent_event(
                self as *const crate::q_state_machine::WrappedEvent,
            );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QStateMachine::WrappedEvent::WrappedEvent(QObject* object, QEvent* event)```</span>.
        pub unsafe fn new(
            object: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
            event: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QEvent>>,
        ) -> ::cpp_utils::CppBox<crate::q_state_machine::WrappedEvent> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStateMachine_WrappedEvent_WrappedEvent(
                ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(object)
                    .as_mut_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QEvent>>::cast_into(event)
                    .as_mut_raw_ptr(),
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>The <a href="http://doc.qt.io/qt-5/qstatemachine-wrappedevent.html">WrappedEvent</a> class inherits <a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a> and holds a clone of an event associated with a <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QStateMachine::WrappedEvent::WrappedEvent(const QStateMachine::WrappedEvent& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qstatemachine-wrappedevent.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qstatemachine-wrappedevent.html">WrappedEvent</a> class inherits <a href="http://doc.qt.io/qt-5/qevent.html">QEvent</a> and holds a clone of an event associated with a <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>.</p>
        /// <p>A wrapped event is generated by a <a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a> in response to a Qt event. The <a href="http://doc.qt.io/qt-5/qeventtransition.html">QEventTransition</a> class provides a transition associated with a such an event. <a href="http://doc.qt.io/qt-5/qstatemachine-wrappedevent.html">QStateMachine::WrappedEvent</a> is part of <a href="http://doc.qt.io/qt-5/statemachine-api.html">The State Machine Framework</a>.</p>
        /// <p>The <a href="http://doc.qt.io/qt-5/qstatemachine-wrappedevent.html#object">object</a>() function returns the object that generated the event. The <a href="http://doc.qt.io/qt-5/qstatemachine-wrappedevent.html#event">event</a>() function returns a clone of the original event.</p></div>
        pub unsafe fn new_copy(
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_state_machine::WrappedEvent>>,
        ) -> ::cpp_utils::CppBox<crate::q_state_machine::WrappedEvent> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStateMachine_WrappedEvent_WrappedEvent2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_state_machine::WrappedEvent>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the object that the event is associated with.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QObject* QStateMachine::WrappedEvent::object() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qstatemachine-wrappedevent.html#object">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the object that the event is associated with.</p></div>
        pub unsafe fn object(&self) -> ::cpp_utils::MutPtr<crate::QObject> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStateMachine_WrappedEvent_object(
                self as *const crate::q_state_machine::WrappedEvent,
            );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    /// <p>This enum type specifies the priority of an event posted to the state machine using <a href="http://doc.qt.io/qt-5/qstatemachine.html#postEvent">postEvent</a>().</p>
    ///
    /// C++ enum: <span style='color: green;'>```QStateMachine::EventPriority```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#EventPriority-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type specifies the priority of an event posted to the state machine using <a href="http://doc.qt.io/qt-5/qstatemachine.html#postEvent">postEvent</a>().</p>
    /// <p>Events of high priority are processed before events of normal priority.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct EventPriority(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for EventPriority {
        fn from(value: ::std::os::raw::c_int) -> Self {
            EventPriority(value)
        }
    }

    impl From<EventPriority> for ::std::os::raw::c_int {
        fn from(value: EventPriority) -> Self {
            value.0
        }
    }

    impl EventPriority {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl EventPriority {
        /// The event has normal priority. (C++ enum variant: <span style='color: green;'>```NormalPriority = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NormalPriority: crate::q_state_machine::EventPriority =
            crate::q_state_machine::EventPriority(0);
        /// The event has high priority. (C++ enum variant: <span style='color: green;'>```HighPriority = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const HighPriority: crate::q_state_machine::EventPriority =
            crate::q_state_machine::EventPriority(1);
    }

    /// <p>This enum type defines errors that can occur in the state machine at run time. When the state machine encounters an unrecoverable error at run time, it will set the error code returned by <a href="http://doc.qt.io/qt-5/qstatemachine.html#error">error</a>(), the error message returned by <a href="http://doc.qt.io/qt-5/qstatemachine.html#errorString">errorString</a>(), and enter an error state based on the context of the error.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QStateMachine::Error```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#Error-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type defines errors that can occur in the state machine at run time. When the state machine encounters an unrecoverable error at run time, it will set the error code returned by <a href="http://doc.qt.io/qt-5/qstatemachine.html#error">error</a>(), the error message returned by <a href="http://doc.qt.io/qt-5/qstatemachine.html#errorString">errorString</a>(), and enter an error state based on the context of the error.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstate.html#setErrorState">setErrorState</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Error(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Error {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Error(value)
        }
    }

    impl From<Error> for ::std::os::raw::c_int {
        fn from(value: Error) -> Self {
            value.0
        }
    }

    impl Error {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Error {
        /// No error has occurred. (C++ enum variant: <span style='color: green;'>```NoError = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoError: crate::q_state_machine::Error = crate::q_state_machine::Error(0);
        /// The machine has entered a <a href="http://doc.qt.io/qt-5/qstate.html">QState</a> with children which does not have an initial state set. The context of this error is the state which is missing an initial state. (C++ enum variant: <span style='color: green;'>```NoInitialStateError = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoInitialStateError: crate::q_state_machine::Error =
            crate::q_state_machine::Error(1);
        /// The machine has entered a <a href="http://doc.qt.io/qt-5/qhistorystate.html">QHistoryState</a> which does not have a default state set. The context of this error is the <a href="http://doc.qt.io/qt-5/qhistorystate.html">QHistoryState</a> which is missing a default state. (C++ enum variant: <span style='color: green;'>```NoDefaultStateInHistoryStateError = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoDefaultStateInHistoryStateError: crate::q_state_machine::Error =
            crate::q_state_machine::Error(2);
        /// The machine has selected a transition whose source and targets are not part of the same tree of states, and thus are not part of the same state machine. Commonly, this could mean that one of the states has not been given any parent or added to any machine. The context of this error is the source state of the transition. (C++ enum variant: <span style='color: green;'>```NoCommonAncestorForTransitionError = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoCommonAncestorForTransitionError: crate::q_state_machine::Error =
            crate::q_state_machine::Error(3);
    }

    impl ::cpp_utils::CppDeletable for crate::q_state_machine::SignalEvent {
        /// <p>Destroys this <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html">SignalEvent</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QStateMachine::SignalEvent::~SignalEvent()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html#dtor.SignalEvent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys this <a href="http://doc.qt.io/qt-5/qstatemachine-signalevent.html">SignalEvent</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QStateMachine_SignalEvent_dSignalEvent(
                self as *mut crate::q_state_machine::SignalEvent,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_state_machine::WrappedEvent {
        /// <p>Destroys this <a href="http://doc.qt.io/qt-5/qstatemachine-wrappedevent.html">WrappedEvent</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QStateMachine::WrappedEvent::~WrappedEvent()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qstatemachine-wrappedevent.html#dtor.WrappedEvent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys this <a href="http://doc.qt.io/qt-5/qstatemachine-wrappedevent.html">WrappedEvent</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QStateMachine_WrappedEvent_dWrappedEvent(
                self as *mut crate::q_state_machine::WrappedEvent,
            )
        }
    }

    impl ::cpp_utils::StaticDowncast<crate::q_state_machine::SignalEvent> for crate::QEvent {
        /// Calls C++ function: <span style='color: green;'>```QStateMachine::SignalEvent* static_cast<QStateMachine::SignalEvent*>(QEvent* ptr)```</span>.
        unsafe fn static_downcast(
            ptr: ::cpp_utils::Ptr<crate::QEvent>,
        ) -> ::cpp_utils::Ptr<crate::q_state_machine::SignalEvent> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_static_cast_QStateMachine_SignalEvent_ptr(
                    ptr.as_raw_ptr() as *mut crate::QEvent,
                );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }

        /// Calls C++ function: <span style='color: green;'>```QStateMachine::SignalEvent* static_cast<QStateMachine::SignalEvent*>(QEvent* ptr)```</span>.
        unsafe fn static_downcast_mut(
            ptr: ::cpp_utils::MutPtr<crate::QEvent>,
        ) -> ::cpp_utils::MutPtr<crate::q_state_machine::SignalEvent> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_static_cast_QStateMachine_SignalEvent_ptr(
                    ptr.as_mut_raw_ptr(),
                );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    impl ::cpp_utils::StaticUpcast<crate::QEvent> for crate::q_state_machine::SignalEvent {
        /// Calls C++ function: <span style='color: green;'>```QEvent* static_cast<QEvent*>(QStateMachine::SignalEvent* ptr)```</span>.
        unsafe fn static_upcast(
            ptr: ::cpp_utils::Ptr<crate::q_state_machine::SignalEvent>,
        ) -> ::cpp_utils::Ptr<crate::QEvent> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QEvent_ptr4(
                ptr.as_raw_ptr() as *mut crate::q_state_machine::SignalEvent,
            );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }

        /// Calls C++ function: <span style='color: green;'>```QEvent* static_cast<QEvent*>(QStateMachine::SignalEvent* ptr)```</span>.
        unsafe fn static_upcast_mut(
            ptr: ::cpp_utils::MutPtr<crate::q_state_machine::SignalEvent>,
        ) -> ::cpp_utils::MutPtr<crate::QEvent> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_static_cast_QEvent_ptr4(ptr.as_mut_raw_ptr());
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    impl ::std::ops::Deref for crate::q_state_machine::SignalEvent {
        type Target = crate::QEvent;
        /// Calls C++ function: <span style='color: green;'>```QEvent* static_cast<QEvent*>(QStateMachine::SignalEvent* ptr)```</span>.
        fn deref(&self) -> &crate::QEvent {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_static_cast_QEvent_ptr4(
                    self as *const crate::q_state_machine::SignalEvent
                        as *mut crate::q_state_machine::SignalEvent,
                )
            };
            unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
        }
    }

    impl ::std::ops::DerefMut for crate::q_state_machine::SignalEvent {
        /// Calls C++ function: <span style='color: green;'>```QEvent* static_cast<QEvent*>(QStateMachine::SignalEvent* ptr)```</span>.
        fn deref_mut(&mut self) -> &mut crate::QEvent {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_static_cast_QEvent_ptr4(
                    self as *mut crate::q_state_machine::SignalEvent,
                )
            };
            unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
        }
    }

    impl ::cpp_utils::DynamicCast<crate::q_state_machine::SignalEvent> for crate::QEvent {
        /// Calls C++ function: <span style='color: green;'>```QStateMachine::SignalEvent* dynamic_cast<QStateMachine::SignalEvent*>(QEvent* ptr)```</span>.
        unsafe fn dynamic_cast(
            ptr: ::cpp_utils::Ptr<crate::QEvent>,
        ) -> ::cpp_utils::Ptr<crate::q_state_machine::SignalEvent> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QStateMachine_SignalEvent_ptr(
                    ptr.as_raw_ptr() as *mut crate::QEvent,
                );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }

        /// Calls C++ function: <span style='color: green;'>```QStateMachine::SignalEvent* dynamic_cast<QStateMachine::SignalEvent*>(QEvent* ptr)```</span>.
        unsafe fn dynamic_cast_mut(
            ptr: ::cpp_utils::MutPtr<crate::QEvent>,
        ) -> ::cpp_utils::MutPtr<crate::q_state_machine::SignalEvent> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QStateMachine_SignalEvent_ptr(
                    ptr.as_mut_raw_ptr(),
                );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    impl ::cpp_utils::StaticDowncast<crate::q_state_machine::WrappedEvent> for crate::QEvent {
        /// Calls C++ function: <span style='color: green;'>```QStateMachine::WrappedEvent* static_cast<QStateMachine::WrappedEvent*>(QEvent* ptr)```</span>.
        unsafe fn static_downcast(
            ptr: ::cpp_utils::Ptr<crate::QEvent>,
        ) -> ::cpp_utils::Ptr<crate::q_state_machine::WrappedEvent> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_static_cast_QStateMachine_WrappedEvent_ptr(
                    ptr.as_raw_ptr() as *mut crate::QEvent,
                );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }

        /// Calls C++ function: <span style='color: green;'>```QStateMachine::WrappedEvent* static_cast<QStateMachine::WrappedEvent*>(QEvent* ptr)```</span>.
        unsafe fn static_downcast_mut(
            ptr: ::cpp_utils::MutPtr<crate::QEvent>,
        ) -> ::cpp_utils::MutPtr<crate::q_state_machine::WrappedEvent> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_static_cast_QStateMachine_WrappedEvent_ptr(
                    ptr.as_mut_raw_ptr(),
                );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    impl ::cpp_utils::StaticUpcast<crate::QEvent> for crate::q_state_machine::WrappedEvent {
        /// Calls C++ function: <span style='color: green;'>```QEvent* static_cast<QEvent*>(QStateMachine::WrappedEvent* ptr)```</span>.
        unsafe fn static_upcast(
            ptr: ::cpp_utils::Ptr<crate::q_state_machine::WrappedEvent>,
        ) -> ::cpp_utils::Ptr<crate::QEvent> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QEvent_ptr5(
                ptr.as_raw_ptr() as *mut crate::q_state_machine::WrappedEvent,
            );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }

        /// Calls C++ function: <span style='color: green;'>```QEvent* static_cast<QEvent*>(QStateMachine::WrappedEvent* ptr)```</span>.
        unsafe fn static_upcast_mut(
            ptr: ::cpp_utils::MutPtr<crate::q_state_machine::WrappedEvent>,
        ) -> ::cpp_utils::MutPtr<crate::QEvent> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_static_cast_QEvent_ptr5(ptr.as_mut_raw_ptr());
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    impl ::std::ops::Deref for crate::q_state_machine::WrappedEvent {
        type Target = crate::QEvent;
        /// Calls C++ function: <span style='color: green;'>```QEvent* static_cast<QEvent*>(QStateMachine::WrappedEvent* ptr)```</span>.
        fn deref(&self) -> &crate::QEvent {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_static_cast_QEvent_ptr5(
                    self as *const crate::q_state_machine::WrappedEvent
                        as *mut crate::q_state_machine::WrappedEvent,
                )
            };
            unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
        }
    }

    impl ::std::ops::DerefMut for crate::q_state_machine::WrappedEvent {
        /// Calls C++ function: <span style='color: green;'>```QEvent* static_cast<QEvent*>(QStateMachine::WrappedEvent* ptr)```</span>.
        fn deref_mut(&mut self) -> &mut crate::QEvent {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_static_cast_QEvent_ptr5(
                    self as *mut crate::q_state_machine::WrappedEvent,
                )
            };
            unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
        }
    }

    impl ::cpp_utils::DynamicCast<crate::q_state_machine::WrappedEvent> for crate::QEvent {
        /// Calls C++ function: <span style='color: green;'>```QStateMachine::WrappedEvent* dynamic_cast<QStateMachine::WrappedEvent*>(QEvent* ptr)```</span>.
        unsafe fn dynamic_cast(
            ptr: ::cpp_utils::Ptr<crate::QEvent>,
        ) -> ::cpp_utils::Ptr<crate::q_state_machine::WrappedEvent> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QStateMachine_WrappedEvent_ptr(
                    ptr.as_raw_ptr() as *mut crate::QEvent,
                );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }

        /// Calls C++ function: <span style='color: green;'>```QStateMachine::WrappedEvent* dynamic_cast<QStateMachine::WrappedEvent*>(QEvent* ptr)```</span>.
        unsafe fn dynamic_cast_mut(
            ptr: ::cpp_utils::MutPtr<crate::QEvent>,
        ) -> ::cpp_utils::MutPtr<crate::q_state_machine::WrappedEvent> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QStateMachine_WrappedEvent_ptr(
                    ptr.as_mut_raw_ptr(),
                );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a> class provides a hierarchical finite state machine.</p>
///
/// C++ class: <span style='color: green;'>```QStateMachine```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qstatemachine.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a> class provides a hierarchical finite state machine.</p>
/// <p><a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a> is based on the concepts and notation of <a href="http://www.wisdom.weizmann.ac.il/~dharel/SCANNED.PAPERS/Statecharts.pdf">Statecharts</a>. <a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a> is part of <a href="http://doc.qt.io/qt-5/statemachine-api.html">The State Machine Framework</a>.</p>
/// <p>A state machine manages a set of states (classes that inherit from <a href="http://doc.qt.io/qt-5/qabstractstate.html">QAbstractState</a>) and transitions (descendants of <a href="http://doc.qt.io/qt-5/qabstracttransition.html">QAbstractTransition</a>) between those states; these states and transitions define a state graph. Once a state graph has been built, the state machine can execute it. <a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a>'s execution algorithm is based on the <a href="http://www.w3.org/TR/scxml/">State Chart XML (SCXML)</a> algorithm. The framework's <a href="http://doc.qt.io/qt-5/statemachine-api.html">overview</a> gives several state graphs and the code to build them.</p>
/// <p>Use the <a href="http://doc.qt.io/qt-5/qstatemachine.html#addState">addState</a>() function to add a top-level state to the state machine. States are removed with the <a href="http://doc.qt.io/qt-5/qstatemachine.html#removeState">removeState</a>() function. Removing states while the machine is running is discouraged.</p>
/// <p>Before the machine can be started, the <a href="http://doc.qt.io/qt-5/qstate.html#initialState-prop">initial state</a> must be set. The initial state is the state that the machine enters when started. You can then <a href="http://doc.qt.io/qt-5/qstatemachine.html#start">start</a>() the state machine. The <a href="http://doc.qt.io/qt-5/qstatemachine.html#started">started</a>() signal is emitted when the initial state is entered.</p>
/// <p>The machine is event driven and keeps its own event loop. Events are posted to the machine through <a href="http://doc.qt.io/qt-5/qstatemachine.html#postEvent">postEvent</a>(). Note that this means that it executes asynchronously, and that it will not progress without a running event loop. You will normally not have to post events to the machine directly as Qt's transitions, e.g., <a href="http://doc.qt.io/qt-5/qeventtransition.html">QEventTransition</a> and its subclasses, handle this. But for custom transitions triggered by events, <a href="http://doc.qt.io/qt-5/qstatemachine.html#postEvent">postEvent</a>() is useful.</p>
/// <p>The state machine processes events and takes transitions until a top-level final state is entered; the state machine then emits the <a href="http://doc.qt.io/qt-5/qstate.html#finished">finished</a>() signal. You can also <a href="http://doc.qt.io/qt-5/qstatemachine.html#stop">stop</a>() the state machine explicitly. The <a href="http://doc.qt.io/qt-5/qstatemachine.html#stopped">stopped</a>() signal is emitted in this case.</p>
/// <p>The following snippet shows a state machine that will finish when a button is clicked:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> button;
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qstatemachine.html#QStateMachine">QStateMachine</a></span> machine;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html#QState">QState</a></span> <span class="operator">*</span>s1 <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html#QState">QState</a></span>();
///   s1<span class="operator">-</span><span class="operator">&gt;</span>assignProperty(<span class="operator">&amp;</span>button<span class="operator">,</span> <span class="string">"text"</span><span class="operator">,</span> <span class="string">"Click me"</span>);
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qfinalstate.html">QFinalState</a></span> <span class="operator">*</span>s2 <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qfinalstate.html">QFinalState</a></span>();
///   s1<span class="operator">-</span><span class="operator">&gt;</span>addTransition(<span class="operator">&amp;</span>button<span class="operator">,</span> SIGNAL(clicked())<span class="operator">,</span> s2);
///
///   machine<span class="operator">.</span>addState(s1);
///   machine<span class="operator">.</span>addState(s2);
///   machine<span class="operator">.</span>setInitialState(s1);
///   machine<span class="operator">.</span>start();
///
/// </pre>
/// <p>This code example uses <a href="http://doc.qt.io/qt-5/qstate.html">QState</a>, which inherits <a href="http://doc.qt.io/qt-5/qabstractstate.html">QAbstractState</a>. The <a href="http://doc.qt.io/qt-5/qstate.html">QState</a> class provides a state that you can use to set properties and invoke methods on <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>s when the state is entered or exited. It also contains convenience functions for adding transitions, e.g., <a href="http://doc.qt.io/qt-5/qsignaltransition.html">QSignalTransition</a>s as in this example. See the <a href="http://doc.qt.io/qt-5/qstate.html">QState</a> class description for further details.</p>
/// <p>If an error is encountered, the machine will look for an <a href="http://doc.qt.io/qt-5/qstate.html#errorState-prop">error state</a>, and if one is available, it will enter this state. The types of errors possible are described by the <a href="http://doc.qt.io/qt-5/qstatemachine.html#Error-enum">Error</a> enum. After the error state is entered, the type of the error can be retrieved with <a href="http://doc.qt.io/qt-5/qstatemachine.html#error">error</a>(). The execution of the state graph will not stop when the error state is entered. If no error state applies to the erroneous state, the machine will stop executing and an error message will be printed to the console.</p></div>
#[repr(C)]
pub struct QStateMachine {
    _unused: u8,
}
impl QStateMachine {
    /// <p>Starts this state machine. The machine will reset its configuration and transition to the initial state. When a final top-level state (<a href="http://doc.qt.io/qt-5/qfinalstate.html">QFinalState</a>) is entered, the machine will emit the <a href="http://doc.qt.io/qt-5/qstate.html#finished">finished</a>() signal.</p>
    ///
    /// Returns a built-in Qt slot `QStateMachine::start` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#start">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Starts this state machine. The machine will reset its configuration and transition to the initial state. When a final top-level state (<a href="http://doc.qt.io/qt-5/qfinalstate.html">QFinalState</a>) is entered, the machine will emit the <a href="http://doc.qt.io/qt-5/qstate.html#finished">finished</a>() signal.</p>
    /// <p><b>Note: </b>A state machine will not run without a running event loop, such as the main application event loop started with <a href="http://doc.qt.io/qt-5/qcoreapplication.html#exec">QCoreApplication::exec</a>() or <a href="http://doc.qt.io/qt-5/qapplication.html#exec">QApplication::exec</a>().</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qstatemachine.html#started">started</a>(), <a href="http://doc.qt.io/qt-5/qstate.html#finished">finished</a>(), <a href="http://doc.qt.io/qt-5/qstatemachine.html#stop">stop</a>(), <a href="http://doc.qt.io/qt-5/qstate.html#initialState">initialState</a>(), and <a href="http://doc.qt.io/qt-5/qstatemachine.html#running-prop">setRunning</a>().</p></div>
    pub fn slot_start(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1start()\0"),
            )
        }
    }

    /// <p>Stops this state machine. The state machine will stop processing events and then emit the <a href="http://doc.qt.io/qt-5/qstatemachine.html#stopped">stopped</a>() signal.</p>
    ///
    /// Returns a built-in Qt slot `QStateMachine::stop` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#stop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Stops this state machine. The state machine will stop processing events and then emit the <a href="http://doc.qt.io/qt-5/qstatemachine.html#stopped">stopped</a>() signal.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstatemachine.html#stopped">stopped</a>(), <a href="http://doc.qt.io/qt-5/qstatemachine.html#start">start</a>(), and <a href="http://doc.qt.io/qt-5/qstatemachine.html#running-prop">setRunning</a>().</p></div>
    pub fn slot_stop(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1stop()\0"),
            )
        }
    }

    /// <p>This property holds the running state of this state machine</p>
    ///
    /// Returns a built-in Qt slot `QStateMachine::setRunning` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#running-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the running state of this state machine</p>
    /// <p>This property was introduced in  Qt 5.4.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> bool </td><td class="memItemRight bottomAlign"><span class="name"><b>isRunning</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setRunning</b></span>(bool <i>running</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qstatemachine.html#runningChanged">runningChanged</a></b></span>(bool <i>running</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstatemachine.html#start">start</a>(), <a href="http://doc.qt.io/qt-5/qstatemachine.html#stop">stop</a>(), <a href="http://doc.qt.io/qt-5/qstatemachine.html#started">started</a>(), <a href="http://doc.qt.io/qt-5/qstatemachine.html#stopped">stopped</a>(), and <a href="http://doc.qt.io/qt-5/qstatemachine.html#runningChanged">runningChanged</a>().</p></div>
    pub fn slot_set_running(&self) -> crate::Receiver<(bool,)> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1setRunning(bool)\0"),
            )
        }
    }

    /// <p>This signal is emitted when the state machine has entered its initial state (<a href="http://doc.qt.io/qt-5/qstate.html#initialState-prop">QStateMachine::initialState</a>).</p>
    ///
    /// Returns a built-in Qt signal `QStateMachine::started` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#started">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the state machine has entered its initial state (<a href="http://doc.qt.io/qt-5/qstate.html#initialState-prop">QStateMachine::initialState</a>).</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstate.html#finished">QStateMachine::finished</a>() and <a href="http://doc.qt.io/qt-5/qstatemachine.html#start">QStateMachine::start</a>().</p></div>
    pub fn started(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2started()\0"),
            )
        }
    }

    /// <p>This signal is emitted when the state machine has stopped.</p>
    ///
    /// Returns a built-in Qt signal `QStateMachine::stopped` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#stopped">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the state machine has stopped.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstatemachine.html#stop">QStateMachine::stop</a>() and <a href="http://doc.qt.io/qt-5/qstate.html#finished">QStateMachine::finished</a>().</p></div>
    pub fn stopped(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2stopped()\0"),
            )
        }
    }

    /// <p>This signal is emitted when the running property is changed with <i>running</i> as argument.</p>
    ///
    /// Returns a built-in Qt signal `QStateMachine::runningChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#runningChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the running property is changed with <i>running</i> as argument.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>Note:</b> Notifier signal for property <a href="http://doc.qt.io/qt-5/qstatemachine.html#running-prop">running</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qstatemachine.html#running-prop">QStateMachine::running</a>.</p></div>
    pub fn running_changed(&self) -> crate::Signal<(bool,)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2runningChanged(bool)\0"),
            )
        }
    }

    /// <p>Adds a default <i>animation</i> to be considered for any transition.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QStateMachine::addDefaultAnimation(QAbstractAnimation* animation)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#addDefaultAnimation">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds a default <i>animation</i> to be considered for any transition.</p></div>
    pub unsafe fn add_default_animation(
        &mut self,
        animation: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractAnimation>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QStateMachine_addDefaultAnimation(
            self as *mut crate::QStateMachine,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractAnimation>>::cast_into(
                animation,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Adds the given <i>state</i> to this state machine. The state becomes a top-level state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QStateMachine::addState(QAbstractState* state)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#addState">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds the given <i>state</i> to this state machine. The state becomes a top-level state.</p>
    /// <p>If the state is already in a different machine, it will first be removed from its old machine, and then added to this machine.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstatemachine.html#removeState">removeState</a>() and <a href="http://doc.qt.io/qt-5/qstate.html#setInitialState">setInitialState</a>().</p></div>
    pub unsafe fn add_state(
        &mut self,
        state: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractState>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QStateMachine_addState(
            self as *mut crate::QStateMachine,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractState>>::cast_into(state)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Cancels the delayed event identified by the given <i>id</i>. The id should be a value returned by a call to <a href="http://doc.qt.io/qt-5/qstatemachine.html#postDelayedEvent">postDelayedEvent</a>(). Returns <code>true</code> if the event was successfully cancelled, otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStateMachine::cancelDelayedEvent(int id)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#cancelDelayedEvent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Cancels the delayed event identified by the given <i>id</i>. The id should be a value returned by a call to <a href="http://doc.qt.io/qt-5/qstatemachine.html#postDelayedEvent">postDelayedEvent</a>(). Returns <code>true</code> if the event was successfully cancelled, otherwise returns <code>false</code>.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstatemachine.html#postDelayedEvent">postDelayedEvent</a>().</p></div>
    pub unsafe fn cancel_delayed_event(&mut self, id: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStateMachine_cancelDelayedEvent(
            self as *mut crate::QStateMachine,
            id,
        )
    }

    /// <p>Clears the error string and error code of the state machine.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QStateMachine::clearError()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#clearError">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Clears the error string and error code of the state machine.</p></div>
    pub unsafe fn clear_error(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QStateMachine_clearError(self as *mut crate::QStateMachine)
    }

    /// <p>Returns the maximal consistent set of states (including parallel and final states) that this state machine is currently in. If a state <code>s</code> is in the configuration, it is always the case that the parent of <code>s</code> is also in c. Note, however, that the machine itself is not an explicit member of the configuration.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*> QStateMachine::configuration() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#configuration">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the maximal consistent set of states (including parallel and final states) that this state machine is currently in. If a state <code>s</code> is in the configuration, it is always the case that the parent of <code>s</code> is also in c. Note, however, that the machine itself is not an explicit member of the configuration.</p></div>
    pub unsafe fn configuration(&self) -> ::cpp_utils::CppBox<crate::QSetOfQAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStateMachine_configuration(
            self as *const crate::QStateMachine,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the list of default animations that will be considered for any transition.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*> QStateMachine::defaultAnimations() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#defaultAnimations">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the list of default animations that will be considered for any transition.</p></div>
    pub unsafe fn default_animations(
        &self,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStateMachine_defaultAnimations(
            self as *const crate::QStateMachine,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the error code of the last error that occurred in the state machine.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStateMachine::Error QStateMachine::error() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#error">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the error code of the last error that occurred in the state machine.</p></div>
    pub unsafe fn error(&self) -> crate::q_state_machine::Error {
        crate::__ffi::ctr_qt_core_ffi_QStateMachine_error(self as *const crate::QStateMachine)
    }

    /// <p>This property holds the error string of this state machine</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QStateMachine::errorString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#errorString-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the error string of this state machine</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QString </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qstatemachine.html#errorString">errorString</a></b></span>() const</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn error_string(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStateMachine_errorString(
            self as *const crate::QStateMachine,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qobject.html#eventFilter">QObject::eventFilter</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QStateMachine::eventFilter(QObject* watched, QEvent* event)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#eventFilter">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qobject.html#eventFilter">QObject::eventFilter</a>().</p></div>
    pub unsafe fn event_filter(
        &mut self,
        watched: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
        event: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QEvent>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStateMachine_eventFilter(
            self as *mut crate::QStateMachine,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(watched)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QEvent>>::cast_into(event)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>This property holds the restore policy for states of this state machine.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QState::RestorePolicy QStateMachine::globalRestorePolicy() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#globalRestorePolicy-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the restore policy for states of this state machine.</p>
    /// <p>The default value of this property is <a href="http://doc.qt.io/qt-5/qstate.html#RestorePolicy-enum">QState::DontRestoreProperties</a>.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QState::RestorePolicy </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qstatemachine.html#globalRestorePolicy">globalRestorePolicy</a></b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qstatemachine.html#setGlobalRestorePolicy">setGlobalRestorePolicy</a></b></span>(QState::RestorePolicy <i>restorePolicy</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn global_restore_policy(&self) -> crate::q_state::RestorePolicy {
        crate::__ffi::ctr_qt_core_ffi_QStateMachine_globalRestorePolicy(
            self as *const crate::QStateMachine,
        )
    }

    /// <p>Returns whether animations are enabled for this state machine.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStateMachine::isAnimated() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#isAnimated">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns whether animations are enabled for this state machine.</p>
    /// <p><b>Note:</b> Getter function for property <a href="http://doc.qt.io/qt-5/qstatemachine.html#animated-prop">animated</a>. </p></div>
    pub unsafe fn is_animated(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStateMachine_isAnimated(self as *const crate::QStateMachine)
    }

    /// <p>This property holds the running state of this state machine</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStateMachine::isRunning() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#running-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the running state of this state machine</p>
    /// <p>This property was introduced in  Qt 5.4.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> bool </td><td class="memItemRight bottomAlign"><span class="name"><b>isRunning</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setRunning</b></span>(bool <i>running</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qstatemachine.html#runningChanged">runningChanged</a></b></span>(bool <i>running</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstatemachine.html#start">start</a>(), <a href="http://doc.qt.io/qt-5/qstatemachine.html#stop">stop</a>(), <a href="http://doc.qt.io/qt-5/qstatemachine.html#started">started</a>(), <a href="http://doc.qt.io/qt-5/qstatemachine.html#stopped">stopped</a>(), and <a href="http://doc.qt.io/qt-5/qstatemachine.html#runningChanged">runningChanged</a>().</p></div>
    pub unsafe fn is_running(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStateMachine_isRunning(self as *const crate::QStateMachine)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QStateMachine::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStateMachine_metaObject(
            self as *const crate::QStateMachine,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs a new state machine with the given <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStateMachine::QStateMachine(QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#QStateMachine">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new state machine with the given <i>parent</i>.</p></div>
    pub unsafe fn from_q_object(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QStateMachine> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStateMachine_QStateMachine(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new state machine with the given <i>childMode</i> and <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStateMachine::QStateMachine(QState::ChildMode childMode, QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#QStateMachine-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new state machine with the given <i>childMode</i> and <i>parent</i>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn from_child_mode_q_object(
        child_mode: crate::q_state::ChildMode,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QStateMachine> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStateMachine_QStateMachine1(
            child_mode,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a> class provides a hierarchical finite state machine.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStateMachine::QStateMachine()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a> class provides a hierarchical finite state machine.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a> is based on the concepts and notation of <a href="http://www.wisdom.weizmann.ac.il/~dharel/SCANNED.PAPERS/Statecharts.pdf">Statecharts</a>. <a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a> is part of <a href="http://doc.qt.io/qt-5/statemachine-api.html">The State Machine Framework</a>.</p>
    /// <p>A state machine manages a set of states (classes that inherit from <a href="http://doc.qt.io/qt-5/qabstractstate.html">QAbstractState</a>) and transitions (descendants of <a href="http://doc.qt.io/qt-5/qabstracttransition.html">QAbstractTransition</a>) between those states; these states and transitions define a state graph. Once a state graph has been built, the state machine can execute it. <a href="http://doc.qt.io/qt-5/qstatemachine.html">QStateMachine</a>'s execution algorithm is based on the <a href="http://www.w3.org/TR/scxml/">State Chart XML (SCXML)</a> algorithm. The framework's <a href="http://doc.qt.io/qt-5/statemachine-api.html">overview</a> gives several state graphs and the code to build them.</p>
    /// <p>Use the <a href="http://doc.qt.io/qt-5/qstatemachine.html#addState">addState</a>() function to add a top-level state to the state machine. States are removed with the <a href="http://doc.qt.io/qt-5/qstatemachine.html#removeState">removeState</a>() function. Removing states while the machine is running is discouraged.</p>
    /// <p>Before the machine can be started, the <a href="http://doc.qt.io/qt-5/qstate.html#initialState-prop">initial state</a> must be set. The initial state is the state that the machine enters when started. You can then <a href="http://doc.qt.io/qt-5/qstatemachine.html#start">start</a>() the state machine. The <a href="http://doc.qt.io/qt-5/qstatemachine.html#started">started</a>() signal is emitted when the initial state is entered.</p>
    /// <p>The machine is event driven and keeps its own event loop. Events are posted to the machine through <a href="http://doc.qt.io/qt-5/qstatemachine.html#postEvent">postEvent</a>(). Note that this means that it executes asynchronously, and that it will not progress without a running event loop. You will normally not have to post events to the machine directly as Qt's transitions, e.g., <a href="http://doc.qt.io/qt-5/qeventtransition.html">QEventTransition</a> and its subclasses, handle this. But for custom transitions triggered by events, <a href="http://doc.qt.io/qt-5/qstatemachine.html#postEvent">postEvent</a>() is useful.</p>
    /// <p>The state machine processes events and takes transitions until a top-level final state is entered; the state machine then emits the <a href="http://doc.qt.io/qt-5/qstate.html#finished">finished</a>() signal. You can also <a href="http://doc.qt.io/qt-5/qstatemachine.html#stop">stop</a>() the state machine explicitly. The <a href="http://doc.qt.io/qt-5/qstatemachine.html#stopped">stopped</a>() signal is emitted in this case.</p>
    /// <p>The following snippet shows a state machine that will finish when a button is clicked:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> button;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstatemachine.html#QStateMachine">QStateMachine</a></span> machine;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html#QState">QState</a></span> <span class="operator">*</span>s1 <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstate.html#QState">QState</a></span>();
    ///   s1<span class="operator">-</span><span class="operator">&gt;</span>assignProperty(<span class="operator">&amp;</span>button<span class="operator">,</span> <span class="string">"text"</span><span class="operator">,</span> <span class="string">"Click me"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qfinalstate.html">QFinalState</a></span> <span class="operator">*</span>s2 <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qfinalstate.html">QFinalState</a></span>();
    ///   s1<span class="operator">-</span><span class="operator">&gt;</span>addTransition(<span class="operator">&amp;</span>button<span class="operator">,</span> SIGNAL(clicked())<span class="operator">,</span> s2);
    ///
    ///   machine<span class="operator">.</span>addState(s1);
    ///   machine<span class="operator">.</span>addState(s2);
    ///   machine<span class="operator">.</span>setInitialState(s1);
    ///   machine<span class="operator">.</span>start();
    ///
    /// </pre>
    /// <p>This code example uses <a href="http://doc.qt.io/qt-5/qstate.html">QState</a>, which inherits <a href="http://doc.qt.io/qt-5/qabstractstate.html">QAbstractState</a>. The <a href="http://doc.qt.io/qt-5/qstate.html">QState</a> class provides a state that you can use to set properties and invoke methods on <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>s when the state is entered or exited. It also contains convenience functions for adding transitions, e.g., <a href="http://doc.qt.io/qt-5/qsignaltransition.html">QSignalTransition</a>s as in this example. See the <a href="http://doc.qt.io/qt-5/qstate.html">QState</a> class description for further details.</p>
    /// <p>If an error is encountered, the machine will look for an <a href="http://doc.qt.io/qt-5/qstate.html#errorState-prop">error state</a>, and if one is available, it will enter this state. The types of errors possible are described by the <a href="http://doc.qt.io/qt-5/qstatemachine.html#Error-enum">Error</a> enum. After the error state is entered, the type of the error can be retrieved with <a href="http://doc.qt.io/qt-5/qstatemachine.html#error">error</a>(). The execution of the state graph will not stop when the error state is entered. If no error state applies to the erroneous state, the machine will stop executing and an error message will be printed to the console.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QStateMachine> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStateMachine_QStateMachine2();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new state machine with the given <i>childMode</i> and <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStateMachine::QStateMachine(QState::ChildMode childMode)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#QStateMachine-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new state machine with the given <i>childMode</i> and <i>parent</i>.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    pub unsafe fn from_child_mode(
        child_mode: crate::q_state::ChildMode,
    ) -> ::cpp_utils::CppBox<crate::QStateMachine> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStateMachine_QStateMachine3(child_mode);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Posts the given <i>event</i> for processing by this state machine, with the given <i>delay</i> in milliseconds. Returns an identifier associated with the delayed event, or -1 if the event could not be posted.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStateMachine::postDelayedEvent(QEvent* event, int delay)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#postDelayedEvent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Posts the given <i>event</i> for processing by this state machine, with the given <i>delay</i> in milliseconds. Returns an identifier associated with the delayed event, or -1 if the event could not be posted.</p>
    /// <p>This function returns immediately. When the delay has expired, the event will be added to the state machine's event queue for processing. The state machine takes ownership of the event and deletes it once it has been processed.</p>
    /// <p>You can only post events when the state machine is running.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstatemachine.html#cancelDelayedEvent">cancelDelayedEvent</a>() and <a href="http://doc.qt.io/qt-5/qstatemachine.html#postEvent">postEvent</a>().</p></div>
    pub unsafe fn post_delayed_event(
        &mut self,
        event: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QEvent>>,
        delay: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStateMachine_postDelayedEvent(
            self as *mut crate::QStateMachine,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QEvent>>::cast_into(event)
                .as_mut_raw_ptr(),
            delay,
        )
    }

    /// <p>Posts the given <i>event</i> of the given <i>priority</i> for processing by this state machine.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QStateMachine::postEvent(QEvent* event, QStateMachine::EventPriority priority = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#postEvent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Posts the given <i>event</i> of the given <i>priority</i> for processing by this state machine.</p>
    /// <p>This function returns immediately. The event is added to the state machine's event queue. Events are processed in the order posted. The state machine takes ownership of the event and deletes it once it has been processed.</p>
    /// <p>You can only post events when the state machine is running or when it is starting up.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstatemachine.html#postDelayedEvent">postDelayedEvent</a>().</p></div>
    pub unsafe fn post_event_2a(
        &mut self,
        event: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QEvent>>,
        priority: crate::q_state_machine::EventPriority,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QStateMachine_postEvent(
            self as *mut crate::QStateMachine,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QEvent>>::cast_into(event)
                .as_mut_raw_ptr(),
            priority,
        )
    }

    /// <p>Posts the given <i>event</i> of the given <i>priority</i> for processing by this state machine.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QStateMachine::postEvent(QEvent* event)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#postEvent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Posts the given <i>event</i> of the given <i>priority</i> for processing by this state machine.</p>
    /// <p>This function returns immediately. The event is added to the state machine's event queue. Events are processed in the order posted. The state machine takes ownership of the event and deletes it once it has been processed.</p>
    /// <p>You can only post events when the state machine is running or when it is starting up.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstatemachine.html#postDelayedEvent">postDelayedEvent</a>().</p></div>
    pub unsafe fn post_event_1a(
        &mut self,
        event: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QEvent>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QStateMachine_postEvent1(
            self as *mut crate::QStateMachine,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QEvent>>::cast_into(event)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QStateMachine::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStateMachine_qt_metacall(
            self as *mut crate::QStateMachine,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QStateMachine::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStateMachine_qt_metacast(
            self as *mut crate::QStateMachine,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Removes <i>animation</i> from the list of default animations.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QStateMachine::removeDefaultAnimation(QAbstractAnimation* animation)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#removeDefaultAnimation">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes <i>animation</i> from the list of default animations.</p></div>
    pub unsafe fn remove_default_animation(
        &mut self,
        animation: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractAnimation>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QStateMachine_removeDefaultAnimation(
            self as *mut crate::QStateMachine,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractAnimation>>::cast_into(
                animation,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Removes the given <i>state</i> from this state machine. The state machine releases ownership of the state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QStateMachine::removeState(QAbstractState* state)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#removeState">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the given <i>state</i> from this state machine. The state machine releases ownership of the state.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstatemachine.html#addState">addState</a>().</p></div>
    pub unsafe fn remove_state(
        &mut self,
        state: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractState>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QStateMachine_removeState(
            self as *mut crate::QStateMachine,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractState>>::cast_into(state)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Sets whether animations are <i>enabled</i> for this state machine.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QStateMachine::setAnimated(bool enabled)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#setAnimated">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets whether animations are <i>enabled</i> for this state machine.</p>
    /// <p><b>Note:</b> Setter function for property <a href="http://doc.qt.io/qt-5/qstatemachine.html#animated-prop">animated</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qstatemachine.html#isAnimated">isAnimated</a>().</p></div>
    pub unsafe fn set_animated(&mut self, enabled: bool) {
        crate::__ffi::ctr_qt_core_ffi_QStateMachine_setAnimated(
            self as *mut crate::QStateMachine,
            enabled,
        )
    }

    /// <p>Sets the restore policy of the state machine to <i>restorePolicy</i>. The default restore policy is <a href="http://doc.qt.io/qt-5/qstate.html#RestorePolicy-enum">QState::DontRestoreProperties</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QStateMachine::setGlobalRestorePolicy(QState::RestorePolicy restorePolicy)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#setGlobalRestorePolicy">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the restore policy of the state machine to <i>restorePolicy</i>. The default restore policy is <a href="http://doc.qt.io/qt-5/qstate.html#RestorePolicy-enum">QState::DontRestoreProperties</a>.</p>
    /// <p><b>Note:</b> Setter function for property <a href="http://doc.qt.io/qt-5/qstatemachine.html#globalRestorePolicy-prop">globalRestorePolicy</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qstatemachine.html#globalRestorePolicy">globalRestorePolicy</a>().</p></div>
    pub unsafe fn set_global_restore_policy(
        &mut self,
        restore_policy: crate::q_state::RestorePolicy,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QStateMachine_setGlobalRestorePolicy(
            self as *mut crate::QStateMachine,
            restore_policy,
        )
    }

    /// <p>This property holds the running state of this state machine</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QStateMachine::setRunning(bool running)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#running-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the running state of this state machine</p>
    /// <p>This property was introduced in  Qt 5.4.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> bool </td><td class="memItemRight bottomAlign"><span class="name"><b>isRunning</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setRunning</b></span>(bool <i>running</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>Notifier signal:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qstatemachine.html#runningChanged">runningChanged</a></b></span>(bool <i>running</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstatemachine.html#start">start</a>(), <a href="http://doc.qt.io/qt-5/qstatemachine.html#stop">stop</a>(), <a href="http://doc.qt.io/qt-5/qstatemachine.html#started">started</a>(), <a href="http://doc.qt.io/qt-5/qstatemachine.html#stopped">stopped</a>(), and <a href="http://doc.qt.io/qt-5/qstatemachine.html#runningChanged">runningChanged</a>().</p></div>
    pub unsafe fn set_running(&mut self, running: bool) {
        crate::__ffi::ctr_qt_core_ffi_QStateMachine_setRunning(
            self as *mut crate::QStateMachine,
            running,
        )
    }

    /// <p>Starts this state machine. The machine will reset its configuration and transition to the initial state. When a final top-level state (<a href="http://doc.qt.io/qt-5/qfinalstate.html">QFinalState</a>) is entered, the machine will emit the <a href="http://doc.qt.io/qt-5/qstate.html#finished">finished</a>() signal.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QStateMachine::start()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#start">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Starts this state machine. The machine will reset its configuration and transition to the initial state. When a final top-level state (<a href="http://doc.qt.io/qt-5/qfinalstate.html">QFinalState</a>) is entered, the machine will emit the <a href="http://doc.qt.io/qt-5/qstate.html#finished">finished</a>() signal.</p>
    /// <p><b>Note: </b>A state machine will not run without a running event loop, such as the main application event loop started with <a href="http://doc.qt.io/qt-5/qcoreapplication.html#exec">QCoreApplication::exec</a>() or <a href="http://doc.qt.io/qt-5/qapplication.html#exec">QApplication::exec</a>().</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qstatemachine.html#started">started</a>(), <a href="http://doc.qt.io/qt-5/qstate.html#finished">finished</a>(), <a href="http://doc.qt.io/qt-5/qstatemachine.html#stop">stop</a>(), <a href="http://doc.qt.io/qt-5/qstate.html#initialState">initialState</a>(), and <a href="http://doc.qt.io/qt-5/qstatemachine.html#running-prop">setRunning</a>().</p></div>
    pub unsafe fn start(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QStateMachine_start(self as *mut crate::QStateMachine)
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStateMachine_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Stops this state machine. The state machine will stop processing events and then emit the <a href="http://doc.qt.io/qt-5/qstatemachine.html#stopped">stopped</a>() signal.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QStateMachine::stop()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#stop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Stops this state machine. The state machine will stop processing events and then emit the <a href="http://doc.qt.io/qt-5/qstatemachine.html#stopped">stopped</a>() signal.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstatemachine.html#stopped">stopped</a>(), <a href="http://doc.qt.io/qt-5/qstatemachine.html#start">start</a>(), and <a href="http://doc.qt.io/qt-5/qstatemachine.html#running-prop">setRunning</a>().</p></div>
    pub unsafe fn stop(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QStateMachine_stop(self as *mut crate::QStateMachine)
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QStateMachine::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStateMachine_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QStateMachine::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStateMachine_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>Provides information about currently mounted storage and drives.</p>
///
/// C++ class: <span style='color: green;'>```QStorageInfo```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qstorageinfo.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Provides information about currently mounted storage and drives.</p>
/// <p>Allows retrieving information about the volume's space, its mount point, label, and filesystem name.</p>
/// <p>You can create an instance of <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> by passing the path to the volume's mount point as a constructor parameter, or you can set it using the <a href="http://doc.qt.io/qt-5/qstorageinfo.html#setPath">setPath</a>() method. The static <a href="http://doc.qt.io/qt-5/qstorageinfo.html#mountedVolumes">mountedVolumes</a>() method can be used to get the list of all mounted filesystems.</p>
/// <p><a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> always caches the retrieved information, but you can call <a href="http://doc.qt.io/qt-5/qstorageinfo.html#refresh">refresh</a>() to invalidate the cache.</p>
/// <p>The following example retrieves the most common information about the root volume of the system, and prints information about it.</p>
/// <pre class="cpp">
///
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstorageinfo.html#QStorageInfo">QStorageInfo</a></span> storage <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstorageinfo.html#QStorageInfo">QStorageInfo</a></span><span class="operator">::</span>root();
///
/// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> storage<span class="operator">.</span>rootPath();
/// &#32;     <span class="keyword">if</span> (storage<span class="operator">.</span>isReadOnly())
/// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"isReadOnly:"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> storage<span class="operator">.</span>isReadOnly();
///
/// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"name:"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> storage<span class="operator">.</span>name();
/// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"fileSystemType:"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> storage<span class="operator">.</span>fileSystemType();
/// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"size:"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> storage<span class="operator">.</span>bytesTotal()<span class="operator">/</span><span class="number">1000</span><span class="operator">/</span><span class="number">1000</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"MB"</span>;
/// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"availableSize:"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> storage<span class="operator">.</span>bytesAvailable()<span class="operator">/</span><span class="number">1000</span><span class="operator">/</span><span class="number">1000</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"MB"</span>;
///
/// </pre></div>
#[repr(C)]
pub struct QStorageInfo {
    _unused: u8,
}
impl QStorageInfo {
    /// <p>Returns the optimal transfer block size for this filesystem.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStorageInfo::blockSize() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstorageinfo.html#blockSize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the optimal transfer block size for this filesystem.</p>
    /// <p>Returns -1 if <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> could not determine the size or if the <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> object is not valid.</p>
    /// <p>This function was introduced in  Qt 5.6.</p></div>
    pub unsafe fn block_size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStorageInfo_blockSize(self as *const crate::QStorageInfo)
    }

    /// <p>Returns the size (in bytes) available for the current user. It returns the total size available if the user is the root user or a system administrator.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QStorageInfo::bytesAvailable() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstorageinfo.html#bytesAvailable">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the size (in bytes) available for the current user. It returns the total size available if the user is the root user or a system administrator.</p>
    /// <p>This size can be less than or equal to the free size returned by <a href="http://doc.qt.io/qt-5/qstorageinfo.html#bytesFree">bytesFree</a>() function.</p>
    /// <p>Returns -1 if <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> object is not valid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstorageinfo.html#bytesTotal">bytesTotal</a>() and <a href="http://doc.qt.io/qt-5/qstorageinfo.html#bytesFree">bytesFree</a>().</p></div>
    pub unsafe fn bytes_available(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QStorageInfo_bytesAvailable(
            self as *const crate::QStorageInfo,
        )
    }

    /// <p>Returns the number of free bytes in a volume. Note that if there are quotas on the filesystem, this value can be larger than the value returned by <a href="http://doc.qt.io/qt-5/qstorageinfo.html#bytesAvailable">bytesAvailable</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QStorageInfo::bytesFree() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstorageinfo.html#bytesFree">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of free bytes in a volume. Note that if there are quotas on the filesystem, this value can be larger than the value returned by <a href="http://doc.qt.io/qt-5/qstorageinfo.html#bytesAvailable">bytesAvailable</a>().</p>
    /// <p>Returns -1 if <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> object is not valid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstorageinfo.html#bytesTotal">bytesTotal</a>() and <a href="http://doc.qt.io/qt-5/qstorageinfo.html#bytesAvailable">bytesAvailable</a>().</p></div>
    pub unsafe fn bytes_free(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QStorageInfo_bytesFree(self as *const crate::QStorageInfo)
    }

    /// <p>Returns the total volume size in bytes.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QStorageInfo::bytesTotal() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstorageinfo.html#bytesTotal">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the total volume size in bytes.</p>
    /// <p>Returns -1 if <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> object is not valid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstorageinfo.html#bytesFree">bytesFree</a>() and <a href="http://doc.qt.io/qt-5/qstorageinfo.html#bytesAvailable">bytesAvailable</a>().</p></div>
    pub unsafe fn bytes_total(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QStorageInfo_bytesTotal(self as *const crate::QStorageInfo)
    }

    /// <p>Makes a copy of the <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> object <i>other</i> and assigns it to this <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStorageInfo& QStorageInfo::operator=(const QStorageInfo& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstorageinfo.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes a copy of the <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> object <i>other</i> and assigns it to this <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> object.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStorageInfo>>,
    ) -> ::cpp_utils::MutRef<crate::QStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStorageInfo_operator_(
            self as *mut crate::QStorageInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStorageInfo>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the device for this volume.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QStorageInfo::device() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstorageinfo.html#device">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the device for this volume.</p>
    /// <p>For example, on Unix filesystems (including <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>), this returns the devpath like <code>/dev/sda0</code> for local storages. On Windows, it returns the UNC path starting with <code>\\\\?\\</code> for local storages (in other words, the volume GUID).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstorageinfo.html#rootPath">rootPath</a>() and <a href="http://doc.qt.io/qt-5/qstorageinfo.html#subvolume">subvolume</a>().</p></div>
    pub unsafe fn device(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStorageInfo_device(self as *const crate::QStorageInfo);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the volume's name, if available, or the root path if not.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QStorageInfo::displayName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstorageinfo.html#displayName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the volume's name, if available, or the root path if not.</p></div>
    pub unsafe fn display_name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStorageInfo_displayName(
            self as *const crate::QStorageInfo,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the type name of the filesystem.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QStorageInfo::fileSystemType() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstorageinfo.html#fileSystemType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the type name of the filesystem.</p>
    /// <p>This is a platform-dependent function, and filesystem names can vary between different operating systems. For example, on Windows filesystems they can be named <code>NTFS</code>, and on Linux they can be named <code>ntfs-3g</code> or <code>fuseblk</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstorageinfo.html#name">name</a>().</p></div>
    pub unsafe fn file_system_type(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStorageInfo_fileSystemType(
            self as *const crate::QStorageInfo,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns true if the current filesystem is protected from writing; false otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStorageInfo::isReadOnly() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstorageinfo.html#isReadOnly">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the current filesystem is protected from writing; false otherwise.</p></div>
    pub unsafe fn is_read_only(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStorageInfo_isReadOnly(self as *const crate::QStorageInfo)
    }

    /// <p>Returns true if the current filesystem is ready to work; false otherwise. For example, false is returned if the CD volume is not inserted.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStorageInfo::isReady() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstorageinfo.html#isReady">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the current filesystem is ready to work; false otherwise. For example, false is returned if the CD volume is not inserted.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qstorageinfo.html#fileSystemType">fileSystemType</a>(), <a href="http://doc.qt.io/qt-5/qstorageinfo.html#name">name</a>(), <a href="http://doc.qt.io/qt-5/qstorageinfo.html#bytesTotal">bytesTotal</a>(), <a href="http://doc.qt.io/qt-5/qstorageinfo.html#bytesFree">bytesFree</a>(), and <a href="http://doc.qt.io/qt-5/qstorageinfo.html#bytesAvailable">bytesAvailable</a>() will return invalid data until the volume is ready.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstorageinfo.html#isValid">isValid</a>().</p></div>
    pub unsafe fn is_ready(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStorageInfo_isReady(self as *const crate::QStorageInfo)
    }

    /// <p>Returns true if this <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> represents the system root volume; false otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStorageInfo::isRoot() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstorageinfo.html#isRoot">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if this <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> represents the system root volume; false otherwise.</p>
    /// <p>On Unix filesystems, the root volume is a volume mounted on <code>/</code>. On Windows, the root volume is the volume where the OS is installed.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstorageinfo.html#root">root</a>().</p></div>
    pub unsafe fn is_root(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStorageInfo_isRoot(self as *const crate::QStorageInfo)
    }

    /// <p>Returns true if the <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> specified by <a href="http://doc.qt.io/qt-5/qstorageinfo.html#rootPath">rootPath</a> exists and is mounted correctly.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStorageInfo::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstorageinfo.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> specified by <a href="http://doc.qt.io/qt-5/qstorageinfo.html#rootPath">rootPath</a> exists and is mounted correctly.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstorageinfo.html#isReady">isReady</a>().</p></div>
    pub unsafe fn is_valid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStorageInfo_isValid(self as *const crate::QStorageInfo)
    }

    /// <p>Returns the list of <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> objects that corresponds to the list of currently mounted filesystems.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QList<QStorageInfo> QStorageInfo::mountedVolumes()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstorageinfo.html#mountedVolumes">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the list of <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> objects that corresponds to the list of currently mounted filesystems.</p>
    /// <p>On Windows, this returns the drives visible in the <b>My Computer</b> folder. On Unix operating systems, it returns the list of all mounted filesystems (except for pseudo filesystems).</p>
    /// <p>Returns all currently mounted filesystems by default.</p>
    /// <p>The example shows how to retrieve all available filesystems, skipping read-only ones.</p>
    /// <pre class="cpp">
    ///
    /// &#32;     foreach (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstorageinfo.html#QStorageInfo">QStorageInfo</a></span> <span class="operator">&amp;</span>storage<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstorageinfo.html#QStorageInfo">QStorageInfo</a></span><span class="operator">::</span>mountedVolumes()) {
    /// &#32;   &#32;     <span class="keyword">if</span> (storage<span class="operator">.</span>isValid() <span class="operator">&amp;</span><span class="operator">&amp;</span> storage<span class="operator">.</span>isReady()) {
    /// &#32;   &#32;   &#32;     <span class="keyword">if</span> (<span class="operator">!</span>storage<span class="operator">.</span>isReadOnly()) {
    /// &#32;   &#32;   &#32;   &#32;     <span class="comment">// ...</span>
    /// &#32;   &#32;   &#32;     }
    /// &#32;   &#32;     }
    /// &#32;     }
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstorageinfo.html#root">root</a>().</p></div>
    pub unsafe fn mounted_volumes() -> ::cpp_utils::CppBox<crate::QListOfQStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStorageInfo_mountedVolumes();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the human-readable name of a filesystem, usually called <code>label</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QStorageInfo::name() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstorageinfo.html#name">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the human-readable name of a filesystem, usually called <code>label</code>.</p>
    /// <p>Not all filesystems support this feature. In this case, the value returned by this method could be empty. An empty string is returned if the file system does not support labels, or if no label is set.</p>
    /// <p>On Linux, retrieving the volume's label requires <code>udev</code> to be present in the system.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstorageinfo.html#fileSystemType">fileSystemType</a>().</p></div>
    pub unsafe fn name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStorageInfo_name(self as *const crate::QStorageInfo);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs an empty <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStorageInfo::QStorageInfo()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstorageinfo.html#QStorageInfo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> object.</p>
    /// <p>Objects created with the default constructor will be invalid and therefore not ready for use.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstorageinfo.html#setPath">setPath</a>(), <a href="http://doc.qt.io/qt-5/qstorageinfo.html#isReady">isReady</a>(), and <a href="http://doc.qt.io/qt-5/qstorageinfo.html#isValid">isValid</a>().</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStorageInfo_QStorageInfo();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> object that gives information about the volume mounted at <i>path</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStorageInfo::QStorageInfo(const QString& path)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstorageinfo.html#QStorageInfo-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> object that gives information about the volume mounted at <i>path</i>.</p>
    /// <p>If you pass a directory or file, the <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> object will refer to the volume where this directory or file is located. You can check if the created object is correct using the <a href="http://doc.qt.io/qt-5/qstorageinfo.html#isValid">isValid</a>() method.</p>
    /// <p>The following example shows how to get the volume on which the application is located. It is recommended to always check that the volume is ready and valid.</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstorageinfo.html#QStorageInfo">QStorageInfo</a></span> storage(qApp<span class="operator">-</span><span class="operator">&gt;</span>applicationDirPath());
    /// &#32;     <span class="keyword">if</span> (storage<span class="operator">.</span>isValid() <span class="operator">&amp;</span><span class="operator">&amp;</span> storage<span class="operator">.</span>isReady()) {
    /// &#32;   &#32;     <span class="comment">// ...</span>
    /// &#32;     }
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstorageinfo.html#setPath">setPath</a>().</p></div>
    pub unsafe fn from_q_string(
        path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStorageInfo_QStorageInfo1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(path).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> object that gives information about the volume containing the <i>dir</i> folder.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStorageInfo::QStorageInfo(const QDir& dir)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstorageinfo.html#QStorageInfo-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> object that gives information about the volume containing the <i>dir</i> folder.</p></div>
    pub unsafe fn from_q_dir(
        dir: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDir>>,
    ) -> ::cpp_utils::CppBox<crate::QStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStorageInfo_QStorageInfo2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDir>>::cast_into(dir).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a new <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> object that is a copy of the <i>other</i> <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStorageInfo::QStorageInfo(const QStorageInfo& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstorageinfo.html#QStorageInfo-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> object that is a copy of the <i>other</i> <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> object.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStorageInfo>>,
    ) -> ::cpp_utils::CppBox<crate::QStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStorageInfo_QStorageInfo3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStorageInfo>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Resets <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a>'s internal cache.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QStorageInfo::refresh()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstorageinfo.html#refresh">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Resets <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a>'s internal cache.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> caches information about storage to speed up performance. <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> retrieves information during object construction and/or when calling the <a href="http://doc.qt.io/qt-5/qstorageinfo.html#setPath">setPath</a>() method. You have to manually reset the cache by calling this function to update storage information.</p></div>
    pub unsafe fn refresh(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QStorageInfo_refresh(self as *mut crate::QStorageInfo)
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> object that represents the system root volume.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QStorageInfo QStorageInfo::root()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstorageinfo.html#root">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> object that represents the system root volume.</p>
    /// <p>On Unix systems this call returns the root ('/') volume; in Windows the volume where the operating system is installed.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstorageinfo.html#isRoot">isRoot</a>().</p></div>
    pub unsafe fn root() -> ::cpp_utils::CppBox<crate::QStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStorageInfo_root();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the mount point of the filesystem this <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> object represents.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QStorageInfo::rootPath() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstorageinfo.html#rootPath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the mount point of the filesystem this <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> object represents.</p>
    /// <p>On Windows, it returns the volume letter in case the volume is not mounted to a directory.</p>
    /// <p>Note that the value returned by rootPath() is the real mount point of a volume, and may not be equal to the value passed to the constructor or <a href="http://doc.qt.io/qt-5/qstorageinfo.html#setPath">setPath</a>() method. For example, if you have only the root volume in the system, and pass '/directory' to <a href="http://doc.qt.io/qt-5/qstorageinfo.html#setPath">setPath</a>(), then this method will return '/'.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstorageinfo.html#setPath">setPath</a>() and <a href="http://doc.qt.io/qt-5/qstorageinfo.html#device">device</a>().</p></div>
    pub unsafe fn root_path(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStorageInfo_rootPath(self as *const crate::QStorageInfo);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Sets this <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> object to the filesystem mounted where <i>path</i> is located.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QStorageInfo::setPath(const QString& path)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstorageinfo.html#setPath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets this <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> object to the filesystem mounted where <i>path</i> is located.</p>
    /// <p><i>path</i> can either be a root path of the filesystem, a directory, or a file within that filesystem.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstorageinfo.html#rootPath">rootPath</a>().</p></div>
    pub unsafe fn set_path(
        &mut self,
        path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QStorageInfo_setPath(
            self as *mut crate::QStorageInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(path).as_raw_ptr(),
        )
    }

    /// <p>Returns the subvolume name for this volume.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QStorageInfo::subvolume() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstorageinfo.html#subvolume">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the subvolume name for this volume.</p>
    /// <p>Some filesystem types allow multiple subvolumes inside one device, which may be mounted in different paths. If the subvolume could be detected, it is returned here. The format of the subvolume name is specific to each filesystem type.</p>
    /// <p>If this volume was not mounted from a subvolume of a larger filesystem or if the subvolume could not be detected, this function returns an empty byte array.</p>
    /// <p>This function was introduced in  Qt 5.9.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstorageinfo.html#device">device</a>().</p></div>
    pub unsafe fn subvolume(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStorageInfo_subvolume(
            self as *const crate::QStorageInfo,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Swaps this volume info with <i>other</i>. This function is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QStorageInfo::swap(QStorageInfo& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstorageinfo.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps this volume info with <i>other</i>. This function is very fast and never fails.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QStorageInfo>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QStorageInfo_swap(
            self as *mut crate::QStorageInfo,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QStorageInfo>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qstringlistmodel.html">QStringListModel</a> class provides a model that supplies strings to views.</p>
///
/// C++ class: <span style='color: green;'>```QStringListModel```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qstringlistmodel.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qstringlistmodel.html">QStringListModel</a> class provides a model that supplies strings to views.</p>
/// <p><a href="http://doc.qt.io/qt-5/qstringlistmodel.html">QStringListModel</a> is an editable model that can be used for simple cases where you need to display a number of strings in a view widget, such as a <a href="http://doc.qt.io/qt-5/qlistview.html">QListView</a> or a <a href="http://doc.qt.io/qt-5/qcombobox.html">QComboBox</a>.</p>
/// <p>The model provides all the standard functions of an editable model, representing the data in the string list as a model with one column and a number of rows equal to the number of items in the list.</p>
/// <p>Model indexes corresponding to items are obtained with the <a href="http://doc.qt.io/qt-5/qabstractlistmodel.html#index">index()</a> function, and item flags are obtained with <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#flags">flags</a>(). Item data is read with the <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#data">data</a>() function and written with <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#setData">setData</a>(). The number of rows (and number of items in the string list) can be found with the <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#rowCount">rowCount</a>() function.</p>
/// <p>The model can be constructed with an existing string list, or strings can be set later with the <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#setStringList">setStringList</a>() convenience function. Strings can also be inserted in the usual way with the <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#insertRows">insertRows</a>() function, and removed with <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#removeRows">removeRows</a>(). The contents of the string list can be retrieved with the <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#stringList">stringList</a>() convenience function.</p>
/// <p>An example usage of <a href="http://doc.qt.io/qt-5/qstringlistmodel.html">QStringListModel</a>:</p>
/// <pre class="cpp">
///
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstringlistmodel.html#QStringListModel">QStringListModel</a></span> <span class="operator">*</span>model <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstringlistmodel.html#QStringListModel">QStringListModel</a></span>();
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> list;
/// &#32;     list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"a"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"b"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"c"</span>;
/// &#32;     model<span class="operator">-</span><span class="operator">&gt;</span>setStringList(list);
///
/// </pre></div>
#[repr(C)]
pub struct QStringListModel {
    _unused: u8,
}
impl QStringListModel {
    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">QAbstractItemModel::data</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QVariant QStringListModel::data(const QModelIndex& index, int role = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">QAbstractItemModel::data</a>().</p>
    /// <p>Returns data for the specified <i>role</i>, from the item with the given <i>index</i>.</p>
    /// <p>If the view requests an invalid index, an invalid variant is returned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlistmodel.html#setData">setData</a>().</p></div>
    pub unsafe fn data_2a(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        role: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringListModel_data(
            self as *const crate::QStringListModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
            role,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">QAbstractItemModel::data</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QVariant QStringListModel::data(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">QAbstractItemModel::data</a>().</p>
    /// <p>Returns data for the specified <i>role</i>, from the item with the given <i>index</i>.</p>
    /// <p>If the view requests an invalid index, an invalid variant is returned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlistmodel.html#setData">setData</a>().</p></div>
    pub unsafe fn data_1a(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringListModel_data1(
            self as *const crate::QStringListModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#flags">QAbstractItemModel::flags</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QFlags<Qt::ItemFlag> QStringListModel::flags(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#flags">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#flags">QAbstractItemModel::flags</a>().</p>
    /// <p>Returns the flags for the item with the given <i>index</i>.</p>
    /// <p>Valid items are enabled, selectable, editable, drag enabled and drop enabled.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#flags">QAbstractItemModel::flags</a>().</p></div>
    pub unsafe fn flags(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> crate::QFlags<crate::ItemFlag> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringListModel_flags(
            self as *const crate::QStringListModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        );
        crate::QFlags::from(ffi_result)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">QAbstractItemModel::insertRows</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QStringListModel::insertRows(int row, int count, const QModelIndex& parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#insertRows">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">QAbstractItemModel::insertRows</a>().</p>
    /// <p>Inserts <i>count</i> rows into the model, beginning at the given <i>row</i>.</p>
    /// <p>The <i>parent</i> index of the rows is optional and is only used for consistency with <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a>. By default, a null index is specified, indicating that the rows are inserted in the top level of the model.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">QAbstractItemModel::insertRows</a>().</p></div>
    pub unsafe fn insert_rows_3a(
        &mut self,
        row: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringListModel_insertRows(
            self as *mut crate::QStringListModel,
            row,
            count,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">QAbstractItemModel::insertRows</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QStringListModel::insertRows(int row, int count)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#insertRows">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">QAbstractItemModel::insertRows</a>().</p>
    /// <p>Inserts <i>count</i> rows into the model, beginning at the given <i>row</i>.</p>
    /// <p>The <i>parent</i> index of the rows is optional and is only used for consistency with <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a>. By default, a null index is specified, indicating that the rows are inserted in the top level of the model.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">QAbstractItemModel::insertRows</a>().</p></div>
    pub unsafe fn insert_rows_2a(
        &mut self,
        row: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringListModel_insertRows1(
            self as *mut crate::QStringListModel,
            row,
            count,
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#itemData">QAbstractItemModel::itemData</a>(const QModelIndex &amp;index) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QMap<int, QVariant> QStringListModel::itemData(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#itemData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#itemData">QAbstractItemModel::itemData</a>(const QModelIndex &amp;index) const.</p>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlistmodel.html#setItemData">setItemData</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn item_data(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QMapOfIntQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringListModel_itemData(
            self as *const crate::QStringListModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QStringListModel::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringListModel_metaObject(
            self as *const crate::QStringListModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#moveRows">QAbstractItemModel::moveRows</a>(const QModelIndex &amp;sourceParent, int sourceRow, int count, const QModelIndex &amp;destinationParent, int destinationChild).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QStringListModel::moveRows(const QModelIndex& sourceParent, int sourceRow, int count, const QModelIndex& destinationParent, int destinationChild)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#moveRows">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#moveRows">QAbstractItemModel::moveRows</a>(const QModelIndex &amp;sourceParent, int sourceRow, int count, const QModelIndex &amp;destinationParent, int destinationChild).</p>
    /// <p>This function was introduced in  Qt 5.13.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn move_rows(
        &mut self,
        source_parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        source_row: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
        destination_parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        destination_child: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringListModel_moveRows(
            self as *mut crate::QStringListModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(source_parent)
                .as_raw_ptr(),
            source_row,
            count,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(
                destination_parent,
            )
            .as_raw_ptr(),
            destination_child,
        )
    }

    /// <p>Constructs a string list model with the given <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStringListModel::QStringListModel(QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#QStringListModel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a string list model with the given <i>parent</i>.</p></div>
    pub unsafe fn from_q_object(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QStringListModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringListModel_QStringListModel(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a string list model containing the specified <i>strings</i> with the given <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStringListModel::QStringListModel(const QStringList& strings, QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#QStringListModel-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a string list model containing the specified <i>strings</i> with the given <i>parent</i>.</p></div>
    pub unsafe fn from_q_string_list_q_object(
        strings: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QStringListModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringListModel_QStringListModel1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(strings)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qstringlistmodel.html">QStringListModel</a> class provides a model that supplies strings to views.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStringListModel::QStringListModel()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlistmodel.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qstringlistmodel.html">QStringListModel</a> class provides a model that supplies strings to views.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qstringlistmodel.html">QStringListModel</a> is an editable model that can be used for simple cases where you need to display a number of strings in a view widget, such as a <a href="http://doc.qt.io/qt-5/qlistview.html">QListView</a> or a <a href="http://doc.qt.io/qt-5/qcombobox.html">QComboBox</a>.</p>
    /// <p>The model provides all the standard functions of an editable model, representing the data in the string list as a model with one column and a number of rows equal to the number of items in the list.</p>
    /// <p>Model indexes corresponding to items are obtained with the <a href="http://doc.qt.io/qt-5/qabstractlistmodel.html#index">index()</a> function, and item flags are obtained with <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#flags">flags</a>(). Item data is read with the <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#data">data</a>() function and written with <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#setData">setData</a>(). The number of rows (and number of items in the string list) can be found with the <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#rowCount">rowCount</a>() function.</p>
    /// <p>The model can be constructed with an existing string list, or strings can be set later with the <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#setStringList">setStringList</a>() convenience function. Strings can also be inserted in the usual way with the <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#insertRows">insertRows</a>() function, and removed with <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#removeRows">removeRows</a>(). The contents of the string list can be retrieved with the <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#stringList">stringList</a>() convenience function.</p>
    /// <p>An example usage of <a href="http://doc.qt.io/qt-5/qstringlistmodel.html">QStringListModel</a>:</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstringlistmodel.html#QStringListModel">QStringListModel</a></span> <span class="operator">*</span>model <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstringlistmodel.html#QStringListModel">QStringListModel</a></span>();
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> list;
    /// &#32;     list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"a"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"b"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"c"</span>;
    /// &#32;     model<span class="operator">-</span><span class="operator">&gt;</span>setStringList(list);
    ///
    /// </pre></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QStringListModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringListModel_QStringListModel2();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a string list model containing the specified <i>strings</i> with the given <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStringListModel::QStringListModel(const QStringList& strings)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#QStringListModel-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a string list model containing the specified <i>strings</i> with the given <i>parent</i>.</p></div>
    pub unsafe fn from_q_string_list(
        strings: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) -> ::cpp_utils::CppBox<crate::QStringListModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringListModel_QStringListModel3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(strings)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QStringListModel::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringListModel_qt_metacall(
            self as *mut crate::QStringListModel,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QStringListModel::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringListModel_qt_metacast(
            self as *mut crate::QStringListModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">QAbstractItemModel::removeRows</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QStringListModel::removeRows(int row, int count, const QModelIndex& parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#removeRows">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">QAbstractItemModel::removeRows</a>().</p>
    /// <p>Removes <i>count</i> rows from the model, beginning at the given <i>row</i>.</p>
    /// <p>The <i>parent</i> index of the rows is optional and is only used for consistency with <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a>. By default, a null index is specified, indicating that the rows are removed in the top level of the model.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">QAbstractItemModel::removeRows</a>().</p></div>
    pub unsafe fn remove_rows_3a(
        &mut self,
        row: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringListModel_removeRows(
            self as *mut crate::QStringListModel,
            row,
            count,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">QAbstractItemModel::removeRows</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QStringListModel::removeRows(int row, int count)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#removeRows">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">QAbstractItemModel::removeRows</a>().</p>
    /// <p>Removes <i>count</i> rows from the model, beginning at the given <i>row</i>.</p>
    /// <p>The <i>parent</i> index of the rows is optional and is only used for consistency with <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a>. By default, a null index is specified, indicating that the rows are removed in the top level of the model.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">QAbstractItemModel::removeRows</a>().</p></div>
    pub unsafe fn remove_rows_2a(
        &mut self,
        row: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringListModel_removeRows1(
            self as *mut crate::QStringListModel,
            row,
            count,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">QAbstractItemModel::rowCount</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual int QStringListModel::rowCount(const QModelIndex& parent = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#rowCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">QAbstractItemModel::rowCount</a>().</p>
    /// <p>Returns the number of rows in the model. This value corresponds to the number of items in the model's internal string list.</p>
    /// <p>The optional <i>parent</i> argument is in most models used to specify the parent of the rows to be counted. Because this is a list if a valid parent is specified, the result will always be 0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlistmodel.html#insertRows">insertRows</a>(), <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#removeRows">removeRows</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">QAbstractItemModel::rowCount</a>().</p></div>
    pub unsafe fn row_count_1a(
        &self,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringListModel_rowCount(
            self as *const crate::QStringListModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">QAbstractItemModel::rowCount</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual int QStringListModel::rowCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#rowCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">QAbstractItemModel::rowCount</a>().</p>
    /// <p>Returns the number of rows in the model. This value corresponds to the number of items in the model's internal string list.</p>
    /// <p>The optional <i>parent</i> argument is in most models used to specify the parent of the rows to be counted. Because this is a list if a valid parent is specified, the result will always be 0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlistmodel.html#insertRows">insertRows</a>(), <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#removeRows">removeRows</a>(), and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">QAbstractItemModel::rowCount</a>().</p></div>
    pub unsafe fn row_count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringListModel_rowCount1(
            self as *const crate::QStringListModel,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">QAbstractItemModel::setData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QStringListModel::setData(const QModelIndex& index, const QVariant& value, int role = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#setData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">QAbstractItemModel::setData</a>().</p>
    /// <p>Sets the data for the specified <i>role</i> in the item with the given <i>index</i> in the model, to the provided <i>value</i>.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dataChanged">dataChanged</a>() signal is emitted if the item is changed.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::ItemDataRole</a> and <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#data">data</a>().</p></div>
    pub unsafe fn set_data_3a(
        &mut self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
        role: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringListModel_setData(
            self as *mut crate::QStringListModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
            role,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">QAbstractItemModel::setData</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QStringListModel::setData(const QModelIndex& index, const QVariant& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#setData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">QAbstractItemModel::setData</a>().</p>
    /// <p>Sets the data for the specified <i>role</i> in the item with the given <i>index</i> in the model, to the provided <i>value</i>.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dataChanged">dataChanged</a>() signal is emitted if the item is changed.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::ItemDataRole</a> and <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#data">data</a>().</p></div>
    pub unsafe fn set_data_2a(
        &mut self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringListModel_setData1(
            self as *mut crate::QStringListModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setItemData">QAbstractItemModel::setItemData</a>(const QModelIndex &amp;index, const QMap&lt;int, QVariant&gt; &amp;roles).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QStringListModel::setItemData(const QModelIndex& index, const QMap<int, QVariant>& roles)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#setItemData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setItemData">QAbstractItemModel::setItemData</a>(const QModelIndex &amp;index, const QMap&lt;int, QVariant&gt; &amp;roles).</p>
    /// <p>If <i>roles</i> contains both <a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::DisplayRole</a> and <a href="http://doc.qt.io/qt-5/qt.html#ItemDataRole-enum">Qt::EditRole</a>, the latter will take precedence</p>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlistmodel.html#itemData">itemData</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn set_item_data(
        &mut self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        roles: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMapOfIntQVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringListModel_setItemData(
            self as *mut crate::QStringListModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMapOfIntQVariant>>::cast_into(roles)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the model's internal string list to <i>strings</i>. The model will notify any attached views that its underlying data has changed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QStringListModel::setStringList(const QStringList& strings)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#setStringList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the model's internal string list to <i>strings</i>. The model will notify any attached views that its underlying data has changed.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlistmodel.html#stringList">stringList</a>() and <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dataChanged">dataChanged</a>().</p></div>
    pub unsafe fn set_string_list(
        &mut self,
        strings: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QStringListModel_setStringList(
            self as *mut crate::QStringListModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(strings)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sibling">QAbstractItemModel::sibling</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QStringListModel::sibling(int row, int column, const QModelIndex& idx) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#sibling">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sibling">QAbstractItemModel::sibling</a>().</p></div>
    pub unsafe fn sibling(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
        idx: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringListModel_sibling(
            self as *const crate::QStringListModel,
            row,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(idx)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sort">QAbstractItemModel::sort</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual void QStringListModel::sort(int column, Qt::SortOrder order = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#sort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sort">QAbstractItemModel::sort</a>().</p></div>
    pub unsafe fn sort_2a(&mut self, column: ::std::os::raw::c_int, order: crate::SortOrder) {
        crate::__ffi::ctr_qt_core_ffi_QStringListModel_sort(
            self as *mut crate::QStringListModel,
            column,
            order,
        )
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sort">QAbstractItemModel::sort</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual void QStringListModel::sort(int column)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#sort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#sort">QAbstractItemModel::sort</a>().</p></div>
    pub unsafe fn sort_1a(&mut self, column: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QStringListModel_sort1(
            self as *mut crate::QStringListModel,
            column,
        )
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringListModel_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the string list used by the model to store data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QStringListModel::stringList() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#stringList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string list used by the model to store data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringlistmodel.html#setStringList">setStringList</a>().</p></div>
    pub unsafe fn string_list(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringListModel_stringList(
            self as *const crate::QStringListModel,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#supportedDropActions">QAbstractItemModel::supportedDropActions</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QFlags<Qt::DropAction> QStringListModel::supportedDropActions() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#supportedDropActions">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#supportedDropActions">QAbstractItemModel::supportedDropActions</a>().</p></div>
    pub unsafe fn supported_drop_actions(&self) -> crate::QFlags<crate::DropAction> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringListModel_supportedDropActions(
            self as *const crate::QStringListModel,
        );
        crate::QFlags::from(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QStringListModel::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringListModel_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QStringListModel::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringListModel_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_system_semaphore {
    //! C++ type: <span style='color: green;'>```QSystemSemaphore```</span>

    /// <p>This enum is used by the constructor and <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#setKey">setKey</a>(). Its purpose is to enable handling the problem in Unix implementations of semaphores that survive a crash. In Unix, when a semaphore survives a crash, we need a way to force it to reset its resource count, when the system reuses the semaphore. In Windows, where semaphores can't survive a crash, this enum has no effect.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QSystemSemaphore::AccessMode```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#AccessMode-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum is used by the constructor and <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#setKey">setKey</a>(). Its purpose is to enable handling the problem in Unix implementations of semaphores that survive a crash. In Unix, when a semaphore survives a crash, we need a way to force it to reset its resource count, when the system reuses the semaphore. In Windows, where semaphores can't survive a crash, this enum has no effect.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct AccessMode(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for AccessMode {
        fn from(value: ::std::os::raw::c_int) -> Self {
            AccessMode(value)
        }
    }

    impl From<AccessMode> for ::std::os::raw::c_int {
        fn from(value: AccessMode) -> Self {
            value.0
        }
    }

    impl AccessMode {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl AccessMode {
        /// If the semaphore already exists, its initial resource count is not reset. If the semaphore does not already exist, it is created and its initial resource count set. (C++ enum variant: <span style='color: green;'>```Open = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const Open: crate::q_system_semaphore::AccessMode =
            crate::q_system_semaphore::AccessMode(0);
        /// <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html">QSystemSemaphore</a> takes ownership of the semaphore and sets its resource count to the requested value, regardless of whether the semaphore already exists by having survived a crash. This value should be passed to the constructor, when the first semaphore for a particular key is constructed and you know that if the semaphore already exists it could only be because of a crash. In Windows, where a semaphore can't survive a crash, Create and Open have the same behavior. (C++ enum variant: <span style='color: green;'>```Create = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Create: crate::q_system_semaphore::AccessMode =
            crate::q_system_semaphore::AccessMode(1);
    }

    /// C++ enum: <span style='color: green;'>```QSystemSemaphore::SystemSemaphoreError```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#SystemSemaphoreError-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct SystemSemaphoreError(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for SystemSemaphoreError {
        fn from(value: ::std::os::raw::c_int) -> Self {
            SystemSemaphoreError(value)
        }
    }

    impl From<SystemSemaphoreError> for ::std::os::raw::c_int {
        fn from(value: SystemSemaphoreError) -> Self {
            value.0
        }
    }

    impl SystemSemaphoreError {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl SystemSemaphoreError {
        /// No error occurred. (C++ enum variant: <span style='color: green;'>```NoError = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoError: crate::q_system_semaphore::SystemSemaphoreError =
            crate::q_system_semaphore::SystemSemaphoreError(0);
        /// The operation failed because the caller didn't have the required permissions. (C++ enum variant: <span style='color: green;'>```PermissionDenied = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const PermissionDenied: crate::q_system_semaphore::SystemSemaphoreError =
            crate::q_system_semaphore::SystemSemaphoreError(1);
        /// The operation failed because of an invalid key. (C++ enum variant: <span style='color: green;'>```KeyError = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const KeyError: crate::q_system_semaphore::SystemSemaphoreError =
            crate::q_system_semaphore::SystemSemaphoreError(2);
        /// The operation failed because a system semaphore with the specified key already existed. (C++ enum variant: <span style='color: green;'>```AlreadyExists = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const AlreadyExists: crate::q_system_semaphore::SystemSemaphoreError =
            crate::q_system_semaphore::SystemSemaphoreError(3);
        /// The operation failed because a system semaphore with the specified key could not be found. (C++ enum variant: <span style='color: green;'>```NotFound = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const NotFound: crate::q_system_semaphore::SystemSemaphoreError =
            crate::q_system_semaphore::SystemSemaphoreError(4);
        /// The operation failed because there was not enough memory available to fill the request. (C++ enum variant: <span style='color: green;'>```OutOfResources = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const OutOfResources: crate::q_system_semaphore::SystemSemaphoreError =
            crate::q_system_semaphore::SystemSemaphoreError(5);
        /// Something else happened and it was bad. (C++ enum variant: <span style='color: green;'>```UnknownError = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const UnknownError: crate::q_system_semaphore::SystemSemaphoreError =
            crate::q_system_semaphore::SystemSemaphoreError(6);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html">QSystemSemaphore</a> class provides a general counting system semaphore.</p>
///
/// C++ class: <span style='color: green;'>```QSystemSemaphore```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html">QSystemSemaphore</a> class provides a general counting system semaphore.</p>
/// <p>A semaphore is a generalization of a mutex. While a mutex can be locked only once, a semaphore can be acquired multiple times. Typically, a semaphore is used to protect a certain number of identical resources.</p>
/// <p>Like its lighter counterpart <a href="http://doc.qt.io/qt-5/qsemaphore.html">QSemaphore</a>, a <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html">QSystemSemaphore</a> can be accessed from multiple <a href="http://doc.qt.io/qt-5/qthread.html">threads</a>. Unlike <a href="http://doc.qt.io/qt-5/qsemaphore.html">QSemaphore</a>, a <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html">QSystemSemaphore</a> can also be accessed from multiple <a href="http://doc.qt.io/qt-5/qprocess.html">processes</a>. This means <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html">QSystemSemaphore</a> is a much heavier class, so if your application doesn't need to access your semaphores across multiple processes, you will probably want to use <a href="http://doc.qt.io/qt-5/qsemaphore.html">QSemaphore</a>.</p>
/// <p>Semaphores support two fundamental operations, <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#acquire">acquire</a>() and <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#release">release</a>():</p>
/// <p><a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#acquire">acquire</a>() tries to acquire one resource. If there isn't a resource available, the call blocks until a resource becomes available. Then the resource is acquired and the call returns.</p>
/// <p><a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#release">release</a>() releases one resource so it can be acquired by another process. The function can also be called with a parameter n &gt; 1, which releases n resources.</p>
/// <p>A system semaphore is created with a string key that other processes can use to use the same semaphore.</p>
/// <p>Example: Create a system semaphore</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#QSystemSemaphore">QSystemSemaphore</a></span> sem(<span class="string">"market"</span><span class="operator">,</span> <span class="number">3</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#QSystemSemaphore">QSystemSemaphore</a></span><span class="operator">::</span>Create);
/// &#32;   &#32;   &#32;   &#32;   &#32;   &#32;   &#32;      <span class="comment">// resources available == 3</span>
///   sem<span class="operator">.</span>acquire();               <span class="comment">// resources available == 2</span>
///   sem<span class="operator">.</span>acquire();               <span class="comment">// resources available == 1</span>
///   sem<span class="operator">.</span>acquire();               <span class="comment">// resources available == 0</span>
///   sem<span class="operator">.</span>release();               <span class="comment">// resources available == 1</span>
///   sem<span class="operator">.</span>release(<span class="number">2</span>);              <span class="comment">// resources available == 3</span>
///
/// </pre>
/// <p>A typical application of system semaphores is for controlling access to a circular buffer shared by a producer process and a consumer processes.</p>
/// <a name="platform-specific-behavior"></a></div>
#[repr(C)]
pub struct QSystemSemaphore {
    _unused: u8,
}
impl QSystemSemaphore {
    /// <p>Acquires one of the resources guarded by this semaphore, if there is one available, and returns <code>true</code>. If all the resources guarded by this semaphore have already been acquired, the call blocks until one of them is released by another process or thread having a semaphore with the same key.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSystemSemaphore::acquire()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#acquire">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Acquires one of the resources guarded by this semaphore, if there is one available, and returns <code>true</code>. If all the resources guarded by this semaphore have already been acquired, the call blocks until one of them is released by another process or thread having a semaphore with the same key.</p>
    /// <p>If false is returned, a system error has occurred. Call <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#error">error</a>() to get a value of <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#SystemSemaphoreError-enum">QSystemSemaphore::SystemSemaphoreError</a> that indicates which error occurred.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#release">release</a>().</p></div>
    pub unsafe fn acquire(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSystemSemaphore_acquire(self as *mut crate::QSystemSemaphore)
    }

    /// <p>Returns a value indicating whether an error occurred, and, if so, which error it was.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSystemSemaphore::SystemSemaphoreError QSystemSemaphore::error() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#error">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a value indicating whether an error occurred, and, if so, which error it was.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#errorString">errorString</a>().</p></div>
    pub unsafe fn error(&self) -> crate::q_system_semaphore::SystemSemaphoreError {
        crate::__ffi::ctr_qt_core_ffi_QSystemSemaphore_error(self as *const crate::QSystemSemaphore)
    }

    /// <p>Returns a text description of the last error that occurred. If <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#error">error</a>() returns an <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#SystemSemaphoreError-enum">error value</a>, call this function to get a text string that describes the error.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QSystemSemaphore::errorString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#errorString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a text description of the last error that occurred. If <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#error">error</a>() returns an <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#SystemSemaphoreError-enum">error value</a>, call this function to get a text string that describes the error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#error">error</a>().</p></div>
    pub unsafe fn error_string(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSystemSemaphore_errorString(
            self as *const crate::QSystemSemaphore,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the key assigned to this system semaphore. The key is the name by which the semaphore can be accessed from other processes.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QSystemSemaphore::key() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#key">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the key assigned to this system semaphore. The key is the name by which the semaphore can be accessed from other processes.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#setKey">setKey</a>().</p></div>
    pub unsafe fn key(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSystemSemaphore_key(
            self as *const crate::QSystemSemaphore,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Requests a system semaphore for the specified <i>key</i>. The parameters <i>initialValue</i> and <i>mode</i> are used according to the following rules, which are system dependent.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSystemSemaphore::QSystemSemaphore(const QString& key, int initialValue = …, QSystemSemaphore::AccessMode mode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#QSystemSemaphore">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Requests a system semaphore for the specified <i>key</i>. The parameters <i>initialValue</i> and <i>mode</i> are used according to the following rules, which are system dependent.</p>
    /// <p>In Unix, if the <i>mode</i> is <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#AccessMode-enum">Open</a> and the system already has a semaphore identified by <i>key</i>, that semaphore is used, and the semaphore's resource count is not changed, i.e., <i>initialValue</i> is ignored. But if the system does not already have a semaphore identified by <i>key</i>, it creates a new semaphore for that key and sets its resource count to <i>initialValue</i>.</p>
    /// <p>In Unix, if the <i>mode</i> is <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#AccessMode-enum">Create</a> and the system already has a semaphore identified by <i>key</i>, that semaphore is used, and its resource count is set to <i>initialValue</i>. If the system does not already have a semaphore identified by <i>key</i>, it creates a new semaphore for that key and sets its resource count to <i>initialValue</i>.</p>
    /// <p>In Windows, <i>mode</i> is ignored, and the system always tries to create a semaphore for the specified <i>key</i>. If the system does not already have a semaphore identified as <i>key</i>, it creates the semaphore and sets its resource count to <i>initialValue</i>. But if the system already has a semaphore identified as <i>key</i> it uses that semaphore and ignores <i>initialValue</i>.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#AccessMode-enum">mode</a> parameter is only used in Unix systems to handle the case where a semaphore survives a process crash. In that case, the next process to allocate a semaphore with the same <i>key</i> will get the semaphore that survived the crash, and unless <i>mode</i> is <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#AccessMode-enum">Create</a>, the resource count will not be reset to <i>initialValue</i> but will retain the initial value it had been given by the crashed process.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#acquire">acquire</a>() and <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#key">key</a>().</p></div>
    pub unsafe fn new_3a(
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        initial_value: ::std::os::raw::c_int,
        mode: crate::q_system_semaphore::AccessMode,
    ) -> ::cpp_utils::CppBox<crate::QSystemSemaphore> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSystemSemaphore_QSystemSemaphore(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
            initial_value,
            mode,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Requests a system semaphore for the specified <i>key</i>. The parameters <i>initialValue</i> and <i>mode</i> are used according to the following rules, which are system dependent.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSystemSemaphore::QSystemSemaphore(const QString& key, int initialValue = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#QSystemSemaphore">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Requests a system semaphore for the specified <i>key</i>. The parameters <i>initialValue</i> and <i>mode</i> are used according to the following rules, which are system dependent.</p>
    /// <p>In Unix, if the <i>mode</i> is <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#AccessMode-enum">Open</a> and the system already has a semaphore identified by <i>key</i>, that semaphore is used, and the semaphore's resource count is not changed, i.e., <i>initialValue</i> is ignored. But if the system does not already have a semaphore identified by <i>key</i>, it creates a new semaphore for that key and sets its resource count to <i>initialValue</i>.</p>
    /// <p>In Unix, if the <i>mode</i> is <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#AccessMode-enum">Create</a> and the system already has a semaphore identified by <i>key</i>, that semaphore is used, and its resource count is set to <i>initialValue</i>. If the system does not already have a semaphore identified by <i>key</i>, it creates a new semaphore for that key and sets its resource count to <i>initialValue</i>.</p>
    /// <p>In Windows, <i>mode</i> is ignored, and the system always tries to create a semaphore for the specified <i>key</i>. If the system does not already have a semaphore identified as <i>key</i>, it creates the semaphore and sets its resource count to <i>initialValue</i>. But if the system already has a semaphore identified as <i>key</i> it uses that semaphore and ignores <i>initialValue</i>.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#AccessMode-enum">mode</a> parameter is only used in Unix systems to handle the case where a semaphore survives a process crash. In that case, the next process to allocate a semaphore with the same <i>key</i> will get the semaphore that survived the crash, and unless <i>mode</i> is <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#AccessMode-enum">Create</a>, the resource count will not be reset to <i>initialValue</i> but will retain the initial value it had been given by the crashed process.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#acquire">acquire</a>() and <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#key">key</a>().</p></div>
    pub unsafe fn new_2a(
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        initial_value: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QSystemSemaphore> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSystemSemaphore_QSystemSemaphore2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
            initial_value,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Requests a system semaphore for the specified <i>key</i>. The parameters <i>initialValue</i> and <i>mode</i> are used according to the following rules, which are system dependent.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSystemSemaphore::QSystemSemaphore(const QString& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#QSystemSemaphore">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Requests a system semaphore for the specified <i>key</i>. The parameters <i>initialValue</i> and <i>mode</i> are used according to the following rules, which are system dependent.</p>
    /// <p>In Unix, if the <i>mode</i> is <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#AccessMode-enum">Open</a> and the system already has a semaphore identified by <i>key</i>, that semaphore is used, and the semaphore's resource count is not changed, i.e., <i>initialValue</i> is ignored. But if the system does not already have a semaphore identified by <i>key</i>, it creates a new semaphore for that key and sets its resource count to <i>initialValue</i>.</p>
    /// <p>In Unix, if the <i>mode</i> is <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#AccessMode-enum">Create</a> and the system already has a semaphore identified by <i>key</i>, that semaphore is used, and its resource count is set to <i>initialValue</i>. If the system does not already have a semaphore identified by <i>key</i>, it creates a new semaphore for that key and sets its resource count to <i>initialValue</i>.</p>
    /// <p>In Windows, <i>mode</i> is ignored, and the system always tries to create a semaphore for the specified <i>key</i>. If the system does not already have a semaphore identified as <i>key</i>, it creates the semaphore and sets its resource count to <i>initialValue</i>. But if the system already has a semaphore identified as <i>key</i> it uses that semaphore and ignores <i>initialValue</i>.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#AccessMode-enum">mode</a> parameter is only used in Unix systems to handle the case where a semaphore survives a process crash. In that case, the next process to allocate a semaphore with the same <i>key</i> will get the semaphore that survived the crash, and unless <i>mode</i> is <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#AccessMode-enum">Create</a>, the resource count will not be reset to <i>initialValue</i> but will retain the initial value it had been given by the crashed process.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#acquire">acquire</a>() and <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#key">key</a>().</p></div>
    pub unsafe fn new_1a(
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QSystemSemaphore> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSystemSemaphore_QSystemSemaphore3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Releases <i>n</i> resources guarded by the semaphore. Returns <code>true</code> unless there is a system error.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSystemSemaphore::release(int n = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#release">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Releases <i>n</i> resources guarded by the semaphore. Returns <code>true</code> unless there is a system error.</p>
    /// <p>Example: Create a system semaphore having five resources; acquire them all and then release them all.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#QSystemSemaphore">QSystemSemaphore</a></span> sem(<span class="string">"market"</span><span class="operator">,</span> <span class="number">5</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#QSystemSemaphore">QSystemSemaphore</a></span><span class="operator">::</span>Create);
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> <span class="number">5</span>; <span class="operator">+</span><span class="operator">+</span>i)  <span class="comment">// acquire all 5 resources</span>
    /// &#32;     sem<span class="operator">.</span>acquire();
    ///   sem<span class="operator">.</span>release(<span class="number">5</span>);              <span class="comment">// release the 5 resources</span>
    ///
    /// </pre>
    /// <p>This function can also "create" resources. For example, immediately following the sequence of statements above, suppose we add the statement:</p>
    /// <pre class="cpp">
    ///
    ///   sem<span class="operator">.</span>release(<span class="number">10</span>);          <span class="comment">// "create" 10 new resources</span>
    ///
    /// </pre>
    /// <p>Ten new resources are now guarded by the semaphore, in addition to the five that already existed. You would not normally use this function to create more resources.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#acquire">acquire</a>().</p></div>
    pub unsafe fn release_1a(&mut self, n: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSystemSemaphore_release(
            self as *mut crate::QSystemSemaphore,
            n,
        )
    }

    /// <p>Releases <i>n</i> resources guarded by the semaphore. Returns <code>true</code> unless there is a system error.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSystemSemaphore::release()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#release">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Releases <i>n</i> resources guarded by the semaphore. Returns <code>true</code> unless there is a system error.</p>
    /// <p>Example: Create a system semaphore having five resources; acquire them all and then release them all.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#QSystemSemaphore">QSystemSemaphore</a></span> sem(<span class="string">"market"</span><span class="operator">,</span> <span class="number">5</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#QSystemSemaphore">QSystemSemaphore</a></span><span class="operator">::</span>Create);
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> <span class="number">5</span>; <span class="operator">+</span><span class="operator">+</span>i)  <span class="comment">// acquire all 5 resources</span>
    /// &#32;     sem<span class="operator">.</span>acquire();
    ///   sem<span class="operator">.</span>release(<span class="number">5</span>);              <span class="comment">// release the 5 resources</span>
    ///
    /// </pre>
    /// <p>This function can also "create" resources. For example, immediately following the sequence of statements above, suppose we add the statement:</p>
    /// <pre class="cpp">
    ///
    ///   sem<span class="operator">.</span>release(<span class="number">10</span>);          <span class="comment">// "create" 10 new resources</span>
    ///
    /// </pre>
    /// <p>Ten new resources are now guarded by the semaphore, in addition to the five that already existed. You would not normally use this function to create more resources.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#acquire">acquire</a>().</p></div>
    pub unsafe fn release_0a(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSystemSemaphore_release1(
            self as *mut crate::QSystemSemaphore,
        )
    }

    /// <p>This function works the same as the constructor. It reconstructs this <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html">QSystemSemaphore</a> object. If the new <i>key</i> is different from the old key, calling this function is like calling the destructor of the semaphore with the old key, then calling the constructor to create a new semaphore with the new <i>key</i>. The <i>initialValue</i> and <i>mode</i> parameters are as defined for the constructor.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSystemSemaphore::setKey(const QString& key, int initialValue = …, QSystemSemaphore::AccessMode mode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#setKey">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function works the same as the constructor. It reconstructs this <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html">QSystemSemaphore</a> object. If the new <i>key</i> is different from the old key, calling this function is like calling the destructor of the semaphore with the old key, then calling the constructor to create a new semaphore with the new <i>key</i>. The <i>initialValue</i> and <i>mode</i> parameters are as defined for the constructor.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#QSystemSemaphore">QSystemSemaphore</a>() and <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#key">key</a>().</p></div>
    pub unsafe fn set_key_3a(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        initial_value: ::std::os::raw::c_int,
        mode: crate::q_system_semaphore::AccessMode,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSystemSemaphore_setKey(
            self as *mut crate::QSystemSemaphore,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
            initial_value,
            mode,
        )
    }

    /// <p>This function works the same as the constructor. It reconstructs this <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html">QSystemSemaphore</a> object. If the new <i>key</i> is different from the old key, calling this function is like calling the destructor of the semaphore with the old key, then calling the constructor to create a new semaphore with the new <i>key</i>. The <i>initialValue</i> and <i>mode</i> parameters are as defined for the constructor.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSystemSemaphore::setKey(const QString& key, int initialValue = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#setKey">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function works the same as the constructor. It reconstructs this <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html">QSystemSemaphore</a> object. If the new <i>key</i> is different from the old key, calling this function is like calling the destructor of the semaphore with the old key, then calling the constructor to create a new semaphore with the new <i>key</i>. The <i>initialValue</i> and <i>mode</i> parameters are as defined for the constructor.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#QSystemSemaphore">QSystemSemaphore</a>() and <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#key">key</a>().</p></div>
    pub unsafe fn set_key_2a(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        initial_value: ::std::os::raw::c_int,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSystemSemaphore_setKey1(
            self as *mut crate::QSystemSemaphore,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
            initial_value,
        )
    }

    /// <p>This function works the same as the constructor. It reconstructs this <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html">QSystemSemaphore</a> object. If the new <i>key</i> is different from the old key, calling this function is like calling the destructor of the semaphore with the old key, then calling the constructor to create a new semaphore with the new <i>key</i>. The <i>initialValue</i> and <i>mode</i> parameters are as defined for the constructor.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSystemSemaphore::setKey(const QString& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#setKey">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function works the same as the constructor. It reconstructs this <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html">QSystemSemaphore</a> object. If the new <i>key</i> is different from the old key, calling this function is like calling the destructor of the semaphore with the old key, then calling the constructor to create a new semaphore with the new <i>key</i>. The <i>initialValue</i> and <i>mode</i> parameters are as defined for the constructor.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#QSystemSemaphore">QSystemSemaphore</a>() and <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#key">key</a>().</p></div>
    pub unsafe fn set_key_1a(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSystemSemaphore_setKey2(
            self as *mut crate::QSystemSemaphore,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        )
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qtemporarydir.html">QTemporaryDir</a> class creates a unique directory for temporary use.</p>
///
/// C++ class: <span style='color: green;'>```QTemporaryDir```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtemporarydir.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qtemporarydir.html">QTemporaryDir</a> class creates a unique directory for temporary use.</p>
/// <p><a href="http://doc.qt.io/qt-5/qtemporarydir.html">QTemporaryDir</a> is used to create unique temporary directories safely. The directory itself is created by the constructor. The name of the temporary directory is guaranteed to be unique (i.e., you are guaranteed to not overwrite an existing directory), and the directory will subsequently be removed upon destruction of the <a href="http://doc.qt.io/qt-5/qtemporarydir.html">QTemporaryDir</a> object. The directory name is either auto-generated, or created based on a template, which is passed to <a href="http://doc.qt.io/qt-5/qtemporarydir.html">QTemporaryDir</a>'s constructor.</p>
/// <p>Example:</p>
/// <pre class="cpp">
///
/// &#32;     <span class="comment">// Within a function/method...</span>
///
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qtemporarydir.html#QTemporaryDir">QTemporaryDir</a></span> dir;
/// &#32;     <span class="keyword">if</span> (dir<span class="operator">.</span>isValid()) {
/// &#32;   &#32;     <span class="comment">// dir.path() returns the unique directory path</span>
/// &#32;     }
///
/// &#32;     <span class="comment">// The QTemporaryDir destructor removes the temporary directory</span>
/// &#32;     <span class="comment">// as it goes out of scope.</span>
///
/// </pre>
/// <p>It is very important to test that the temporary directory could be created, using <a href="http://doc.qt.io/qt-5/qtemporarydir.html#isValid">isValid</a>(). Do not use <a href="http://doc.qt.io/qt-5/qdir.html#exists-1">exists()</a>, since a default-constructed <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> represents the current directory, which exists.</p>
/// <p>The path to the temporary directory can be found by calling <a href="http://doc.qt.io/qt-5/qtemporarydir.html#path">path</a>().</p>
/// <p>A temporary directory will have some static part of the name and some part that is calculated to be unique. The default path will be determined from <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationName-prop">QCoreApplication::applicationName</a>() (otherwise <code>qt_temp</code>) and will be placed into the temporary path as returned by <a href="http://doc.qt.io/qt-5/qdir.html#tempPath">QDir::tempPath</a>(). If you specify your own path, a relative path will not be placed in the temporary directory by default, but be relative to the current working directory. In all cases, a random string will be appended to the path in order to make it unique.</p></div>
#[repr(C)]
pub struct QTemporaryDir {
    _unused: u8,
}
impl QTemporaryDir {
    /// <p>Returns <code>true</code> if the <a href="http://doc.qt.io/qt-5/qtemporarydir.html">QTemporaryDir</a> is in auto remove mode. Auto-remove mode will automatically delete the directory from disk upon destruction. This makes it very easy to create your <a href="http://doc.qt.io/qt-5/qtemporarydir.html">QTemporaryDir</a> object on the stack, fill it with files, do something with the files, and finally on function return it will automatically clean up after itself.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTemporaryDir::autoRemove() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtemporarydir.html#autoRemove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the <a href="http://doc.qt.io/qt-5/qtemporarydir.html">QTemporaryDir</a> is in auto remove mode. Auto-remove mode will automatically delete the directory from disk upon destruction. This makes it very easy to create your <a href="http://doc.qt.io/qt-5/qtemporarydir.html">QTemporaryDir</a> object on the stack, fill it with files, do something with the files, and finally on function return it will automatically clean up after itself.</p>
    /// <p>Auto-remove is on by default.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtemporarydir.html#setAutoRemove">setAutoRemove</a>() and <a href="http://doc.qt.io/qt-5/qtemporarydir.html#remove">remove</a>().</p></div>
    pub unsafe fn auto_remove(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTemporaryDir_autoRemove(self as *const crate::QTemporaryDir)
    }

    /// <p>If <a href="http://doc.qt.io/qt-5/qtemporarydir.html#isValid">isValid</a>() returns <code>false</code>, this function returns the error string that explains why the creation of the temporary directory failed. Otherwise, this function return an empty string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QTemporaryDir::errorString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtemporarydir.html#errorString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If <a href="http://doc.qt.io/qt-5/qtemporarydir.html#isValid">isValid</a>() returns <code>false</code>, this function returns the error string that explains why the creation of the temporary directory failed. Otherwise, this function return an empty string.</p>
    /// <p>This function was introduced in  Qt 5.6.</p></div>
    pub unsafe fn error_string(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTemporaryDir_errorString(
            self as *const crate::QTemporaryDir,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the path name of a file in the temporary directory. Does <i>not</i> check if the file actually exists in the directory. Redundant multiple separators or "." and ".." directories in <i>fileName</i> are not removed (see <a href="http://doc.qt.io/qt-5/qdir.html#cleanPath">QDir::cleanPath</a>()). Absolute paths are not allowed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QTemporaryDir::filePath(const QString& fileName) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtemporarydir.html#filePath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the path name of a file in the temporary directory. Does <i>not</i> check if the file actually exists in the directory. Redundant multiple separators or "." and ".." directories in <i>fileName</i> are not removed (see <a href="http://doc.qt.io/qt-5/qdir.html#cleanPath">QDir::cleanPath</a>()). Absolute paths are not allowed.</p>
    /// <p>This function was introduced in  Qt 5.9.</p></div>
    pub unsafe fn file_path(
        &self,
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTemporaryDir_filePath(
            self as *const crate::QTemporaryDir,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if the <a href="http://doc.qt.io/qt-5/qtemporarydir.html">QTemporaryDir</a> was created successfully.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTemporaryDir::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtemporarydir.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the <a href="http://doc.qt.io/qt-5/qtemporarydir.html">QTemporaryDir</a> was created successfully.</p></div>
    pub unsafe fn is_valid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTemporaryDir_isValid(self as *const crate::QTemporaryDir)
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qtemporarydir.html">QTemporaryDir</a> using as template the application name returned by <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationName-prop">QCoreApplication::applicationName</a>() (otherwise <code>qt_temp</code>). The directory is stored in the system's temporary directory, <a href="http://doc.qt.io/qt-5/qdir.html#tempPath">QDir::tempPath</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTemporaryDir::QTemporaryDir()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtemporarydir.html#QTemporaryDir">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qtemporarydir.html">QTemporaryDir</a> using as template the application name returned by <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationName-prop">QCoreApplication::applicationName</a>() (otherwise <code>qt_temp</code>). The directory is stored in the system's temporary directory, <a href="http://doc.qt.io/qt-5/qdir.html#tempPath">QDir::tempPath</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#tempPath">QDir::tempPath</a>().</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QTemporaryDir> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTemporaryDir_QTemporaryDir();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qtemporarydir.html">QTemporaryDir</a> with a template of <i>templatePath</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTemporaryDir::QTemporaryDir(const QString& templateName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtemporarydir.html#QTemporaryDir-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qtemporarydir.html">QTemporaryDir</a> with a template of <i>templatePath</i>.</p>
    /// <p>If <i>templatePath</i> is a relative path, the path will be relative to the current working directory. You can use <a href="http://doc.qt.io/qt-5/qdir.html#tempPath">QDir::tempPath</a>() to construct <i>templatePath</i> if you want use the system's temporary directory.</p>
    /// <p>If the <i>templatePath</i> ends with XXXXXX it will be used as the dynamic portion of the directory name, otherwise it will be appended. Unlike <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a>, XXXXXX in the middle of the template string is not supported.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdir.html#tempPath">QDir::tempPath</a>().</p></div>
    pub unsafe fn new_1a(
        template_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QTemporaryDir> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTemporaryDir_QTemporaryDir1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(template_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the path to the temporary directory. Empty if the <a href="http://doc.qt.io/qt-5/qtemporarydir.html">QTemporaryDir</a> could not be created.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QTemporaryDir::path() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtemporarydir.html#path">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the path to the temporary directory. Empty if the <a href="http://doc.qt.io/qt-5/qtemporarydir.html">QTemporaryDir</a> could not be created.</p></div>
    pub unsafe fn path(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QTemporaryDir_path(self as *const crate::QTemporaryDir);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the temporary directory, including all its contents.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTemporaryDir::remove()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtemporarydir.html#remove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the temporary directory, including all its contents.</p>
    /// <p>Returns <code>true</code> if removing was successful.</p></div>
    pub unsafe fn remove(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTemporaryDir_remove(self as *mut crate::QTemporaryDir)
    }

    /// <p>Sets the <a href="http://doc.qt.io/qt-5/qtemporarydir.html">QTemporaryDir</a> into auto-remove mode if <i>b</i> is true.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTemporaryDir::setAutoRemove(bool b)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtemporarydir.html#setAutoRemove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the <a href="http://doc.qt.io/qt-5/qtemporarydir.html">QTemporaryDir</a> into auto-remove mode if <i>b</i> is true.</p>
    /// <p>Auto-remove is on by default.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtemporarydir.html#autoRemove">autoRemove</a>() and <a href="http://doc.qt.io/qt-5/qtemporarydir.html#remove">remove</a>().</p></div>
    pub unsafe fn set_auto_remove(&mut self, b: bool) {
        crate::__ffi::ctr_qt_core_ffi_QTemporaryDir_setAutoRemove(
            self as *mut crate::QTemporaryDir,
            b,
        )
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a> class is an I/O device that operates on temporary files.</p>
///
/// C++ class: <span style='color: green;'>```QTemporaryFile```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtemporaryfile.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a> class is an I/O device that operates on temporary files.</p>
/// <p><a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a> is used to create unique temporary files safely. The file itself is created by calling <a href="http://doc.qt.io/qt-5/qtemporaryfile.html#open">open</a>(). The name of the temporary file is guaranteed to be unique (i.e., you are guaranteed to not overwrite an existing file), and the file will subsequently be removed upon destruction of the <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a> object. This is an important technique that avoids data corruption for applications that store data in temporary files. The file name is either auto-generated, or created based on a template, which is passed to <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a>'s constructor.</p>
/// <p>Example:</p>
/// <pre class="cpp">
///
/// &#32;     <span class="comment">// Within a function/method...</span>
///
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qtemporaryfile.html#QTemporaryFile">QTemporaryFile</a></span> file;
/// &#32;     <span class="keyword">if</span> (file<span class="operator">.</span>open()) {
/// &#32;   &#32;     <span class="comment">// file.fileName() returns the unique file name</span>
/// &#32;     }
///
/// &#32;     <span class="comment">// The QTemporaryFile destructor removes the temporary file</span>
/// &#32;     <span class="comment">// as it goes out of scope.</span>
///
/// </pre>
/// <p>Reopening a <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a> after calling <a href="http://doc.qt.io/qt-5/qfiledevice.html#close">close</a>() is safe. For as long as the <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a> object itself is not destroyed, the unique temporary file will exist and be kept open internally by <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a>.</p>
/// <p>The file name of the temporary file can be found by calling <a href="http://doc.qt.io/qt-5/qtemporaryfile.html#fileName">fileName</a>(). Note that this is only defined after the file is first opened; the function returns an empty string before this.</p>
/// <p>A temporary file will have some static part of the name and some part that is calculated to be unique. The default filename will be determined from <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationName-prop">QCoreApplication::applicationName</a>() (otherwise <code>qt_temp</code>) and will be placed into the temporary path as returned by <a href="http://doc.qt.io/qt-5/qdir.html#tempPath">QDir::tempPath</a>(). If you specify your own filename, a relative file path will not be placed in the temporary directory by default, but be relative to the current working directory.</p>
/// <p>Specified filenames can contain the following template <code>XXXXXX</code> (six upper case "X" characters), which will be replaced by the auto-generated portion of the filename. Note that the template is case sensitive. If the template is not present in the filename, <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a> appends the generated part to the filename given.</p></div>
#[repr(C)]
pub struct QTemporaryFile {
    _unused: u8,
}
impl QTemporaryFile {
    /// <p>Returns <code>true</code> if the <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a> is in auto remove mode. Auto-remove mode will automatically delete the filename from disk upon destruction. This makes it very easy to create your <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a> object on the stack, fill it with data, read from it, and finally on function return it will automatically clean up after itself.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTemporaryFile::autoRemove() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtemporaryfile.html#autoRemove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a> is in auto remove mode. Auto-remove mode will automatically delete the filename from disk upon destruction. This makes it very easy to create your <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a> object on the stack, fill it with data, read from it, and finally on function return it will automatically clean up after itself.</p>
    /// <p>Auto-remove is on by default.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtemporaryfile.html#setAutoRemove">setAutoRemove</a>() and <a href="http://doc.qt.io/qt-5/qfile.html#remove">remove</a>().</p></div>
    pub unsafe fn auto_remove(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTemporaryFile_autoRemove(
            self as *const crate::QTemporaryFile,
        )
    }

    /// <p>Use <a href="http://doc.qt.io/qt-5/qtemporaryfile.html#createNativeFile">QTemporaryFile::createNativeFile</a>(<a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> &amp;file) instead.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QTemporaryFile* QTemporaryFile::createLocalFile(const QString& fileName)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qtemporaryfile-obsolete.html#createLocalFile-1">C++ documentation</a> for <span style='color: green;'>```static QTemporaryFile *QTemporaryFile::createLocalFile(QFile &file)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Use <a href="http://doc.qt.io/qt-5/qtemporaryfile.html#createNativeFile">QTemporaryFile::createNativeFile</a>(<a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> &amp;file) instead.</p></div>
    pub unsafe fn create_local_file_q_string(
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::MutPtr<crate::QTemporaryFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTemporaryFile_createLocalFile(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Use <a href="http://doc.qt.io/qt-5/qtemporaryfile.html#createNativeFile">QTemporaryFile::createNativeFile</a>(<a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> &amp;file) instead.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QTemporaryFile* QTemporaryFile::createLocalFile(QFile& file)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtemporaryfile-obsolete.html#createLocalFile-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Use <a href="http://doc.qt.io/qt-5/qtemporaryfile.html#createNativeFile">QTemporaryFile::createNativeFile</a>(<a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> &amp;file) instead.</p></div>
    pub unsafe fn create_local_file_q_file(
        file: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QFile>>,
    ) -> ::cpp_utils::MutPtr<crate::QTemporaryFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTemporaryFile_createLocalFile1(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QFile>>::cast_into(file)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QTemporaryFile* QTemporaryFile::createNativeFile(const QString& fileName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtemporaryfile.html#createNativeFile-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Works on the given <i>fileName</i> rather than an existing <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a> object.</p></div>
    pub unsafe fn create_native_file_q_string(
        file_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::MutPtr<crate::QTemporaryFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTemporaryFile_createNativeFile(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(file_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>If <i>file</i> is not already a native file, then a <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a> is created in <a href="http://doc.qt.io/qt-5/qdir.html#tempPath">QDir::tempPath</a>(), the contents of <i>file</i> is copied into it, and a pointer to the temporary file is returned. Does nothing and returns <code>0</code> if <i>file</i> is already a native file.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QTemporaryFile* QTemporaryFile::createNativeFile(QFile& file)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtemporaryfile.html#createNativeFile">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If <i>file</i> is not already a native file, then a <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a> is created in <a href="http://doc.qt.io/qt-5/qdir.html#tempPath">QDir::tempPath</a>(), the contents of <i>file</i> is copied into it, and a pointer to the temporary file is returned. Does nothing and returns <code>0</code> if <i>file</i> is already a native file.</p>
    /// <p>For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qfile.html#QFile">QFile</a></span> f(<span class="string">":/resources/file.txt"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtemporaryfile.html#QTemporaryFile">QTemporaryFile</a></span><span class="operator">::</span>createNativeFile(f); <span class="comment">// Returns a pointer to a temporary file</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qfile.html#QFile">QFile</a></span> f(<span class="string">"/users/qt/file.txt"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtemporaryfile.html#QTemporaryFile">QTemporaryFile</a></span><span class="operator">::</span>createNativeFile(f); <span class="comment">// Returns 0</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#isNativePath">QFileInfo::isNativePath</a>().</p></div>
    pub unsafe fn create_native_file_q_file(
        file: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QFile>>,
    ) -> ::cpp_utils::MutPtr<crate::QTemporaryFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTemporaryFile_createNativeFile1(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QFile>>::cast_into(file)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qfiledevice.html#fileName">QFileDevice::fileName</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QString QTemporaryFile::fileName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtemporaryfile.html#fileName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qfiledevice.html#fileName">QFileDevice::fileName</a>().</p>
    /// <p>Returns the complete unique filename backing the <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a> object. This string is null before the <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a> is opened, afterwards it will contain the <a href="http://doc.qt.io/qt-5/qtemporaryfile.html#fileTemplate">fileTemplate</a>() plus additional characters to make it unique.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtemporaryfile.html#fileTemplate">fileTemplate</a>().</p></div>
    pub unsafe fn file_name(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTemporaryFile_fileName(
            self as *const crate::QTemporaryFile,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the set file template. The default file template will be called qcoreappname.XXXXXX and be placed in <a href="http://doc.qt.io/qt-5/qdir.html#tempPath">QDir::tempPath</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QTemporaryFile::fileTemplate() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtemporaryfile.html#fileTemplate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the set file template. The default file template will be called qcoreappname.XXXXXX and be placed in <a href="http://doc.qt.io/qt-5/qdir.html#tempPath">QDir::tempPath</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtemporaryfile.html#setFileTemplate">setFileTemplate</a>().</p></div>
    pub unsafe fn file_template(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTemporaryFile_fileTemplate(
            self as *const crate::QTemporaryFile,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QTemporaryFile::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTemporaryFile_metaObject(
            self as *const crate::QTemporaryFile,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a> using as file template the application name returned by <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationName-prop">QCoreApplication::applicationName</a>() (otherwise <code>qt_temp</code>) followed by ".XXXXXX". The file is stored in the system's temporary directory, <a href="http://doc.qt.io/qt-5/qdir.html#tempPath">QDir::tempPath</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTemporaryFile::QTemporaryFile()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtemporaryfile.html#QTemporaryFile">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a> using as file template the application name returned by <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationName-prop">QCoreApplication::applicationName</a>() (otherwise <code>qt_temp</code>) followed by ".XXXXXX". The file is stored in the system's temporary directory, <a href="http://doc.qt.io/qt-5/qdir.html#tempPath">QDir::tempPath</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtemporaryfile.html#setFileTemplate">setFileTemplate</a>() and <a href="http://doc.qt.io/qt-5/qdir.html#tempPath">QDir::tempPath</a>().</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QTemporaryFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTemporaryFile_QTemporaryFile();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a> with a template filename of <i>templateName</i>. Upon opening the temporary file this will be used to create a unique filename.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTemporaryFile::QTemporaryFile(const QString& templateName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtemporaryfile.html#QTemporaryFile-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a> with a template filename of <i>templateName</i>. Upon opening the temporary file this will be used to create a unique filename.</p>
    /// <p>If the <i>templateName</i> does not contain XXXXXX it will automatically be appended and used as the dynamic portion of the filename.</p>
    /// <p>If <i>templateName</i> is a relative path, the path will be relative to the current working directory. You can use <a href="http://doc.qt.io/qt-5/qdir.html#tempPath">QDir::tempPath</a>() to construct <i>templateName</i> if you want use the system's temporary directory.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtemporaryfile.html#open">open</a>() and <a href="http://doc.qt.io/qt-5/qtemporaryfile.html#fileTemplate">fileTemplate</a>().</p></div>
    pub unsafe fn from_q_string(
        template_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QTemporaryFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTemporaryFile_QTemporaryFile1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(template_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a> (with the given <i>parent</i>) using as file template the application name returned by <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationName-prop">QCoreApplication::applicationName</a>() (otherwise <code>qt_temp</code>) followed by ".XXXXXX". The file is stored in the system's temporary directory, <a href="http://doc.qt.io/qt-5/qdir.html#tempPath">QDir::tempPath</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTemporaryFile::QTemporaryFile(QObject* parent)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtemporaryfile.html#QTemporaryFile-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a> (with the given <i>parent</i>) using as file template the application name returned by <a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationName-prop">QCoreApplication::applicationName</a>() (otherwise <code>qt_temp</code>) followed by ".XXXXXX". The file is stored in the system's temporary directory, <a href="http://doc.qt.io/qt-5/qdir.html#tempPath">QDir::tempPath</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtemporaryfile.html#setFileTemplate">setFileTemplate</a>().</p></div>
    pub unsafe fn from_q_object(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QTemporaryFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTemporaryFile_QTemporaryFile2(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a> with a template filename of <i>templateName</i> and the specified <i>parent</i>. Upon opening the temporary file this will be used to create a unique filename.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTemporaryFile::QTemporaryFile(const QString& templateName, QObject* parent)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtemporaryfile.html#QTemporaryFile-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a> with a template filename of <i>templateName</i> and the specified <i>parent</i>. Upon opening the temporary file this will be used to create a unique filename.</p>
    /// <p>If the <i>templateName</i> does not contain XXXXXX it will automatically be appended and used as the dynamic portion of the filename.</p>
    /// <p>If <i>templateName</i> is a relative path, the path will be relative to the current working directory. You can use <a href="http://doc.qt.io/qt-5/qdir.html#tempPath">QDir::tempPath</a>() to construct <i>templateName</i> if you want use the system's temporary directory.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtemporaryfile.html#open">open</a>() and <a href="http://doc.qt.io/qt-5/qtemporaryfile.html#fileTemplate">fileTemplate</a>().</p></div>
    pub unsafe fn from_q_string_q_object(
        template_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QTemporaryFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTemporaryFile_QTemporaryFile3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(template_name)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>A <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a> will always be opened in <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">QIODevice::ReadWrite</a> mode, this allows easy access to the data in the file. This function will return true upon success and will set the <a href="http://doc.qt.io/qt-5/qtemporaryfile.html#fileName">fileName</a>() to the unique filename used.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTemporaryFile::open()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtemporaryfile.html#open">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>A <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a> will always be opened in <a href="http://doc.qt.io/qt-5/qiodevice.html#OpenModeFlag-enum">QIODevice::ReadWrite</a> mode, this allows easy access to the data in the file. This function will return true upon success and will set the <a href="http://doc.qt.io/qt-5/qtemporaryfile.html#fileName">fileName</a>() to the unique filename used.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtemporaryfile.html#fileName">fileName</a>().</p></div>
    pub unsafe fn open(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTemporaryFile_open(self as *mut crate::QTemporaryFile)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QTemporaryFile::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTemporaryFile_qt_metacall(
            self as *mut crate::QTemporaryFile,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QTemporaryFile::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTemporaryFile_qt_metacast(
            self as *mut crate::QTemporaryFile,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QTemporaryFile::rename(const QString& newName)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn rename(
        &mut self,
        new_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTemporaryFile_rename(
            self as *mut crate::QTemporaryFile,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(new_name)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a> into auto-remove mode if <i>b</i> is true.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTemporaryFile::setAutoRemove(bool b)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtemporaryfile.html#setAutoRemove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the <a href="http://doc.qt.io/qt-5/qtcore-attribution-qtemporaryfile.html#qtemporaryfile">QTemporaryFile</a> into auto-remove mode if <i>b</i> is true.</p>
    /// <p>Auto-remove is on by default.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtemporaryfile.html#autoRemove">autoRemove</a>() and <a href="http://doc.qt.io/qt-5/qfile.html#remove">remove</a>().</p></div>
    pub unsafe fn set_auto_remove(&mut self, b: bool) {
        crate::__ffi::ctr_qt_core_ffi_QTemporaryFile_setAutoRemove(
            self as *mut crate::QTemporaryFile,
            b,
        )
    }

    /// <p>Sets the static portion of the file name to <i>name</i>. If the file template contains XXXXXX that will automatically be replaced with the unique part of the filename, otherwise a filename will be determined automatically based on the static portion specified.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTemporaryFile::setFileTemplate(const QString& name)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtemporaryfile.html#setFileTemplate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the static portion of the file name to <i>name</i>. If the file template contains XXXXXX that will automatically be replaced with the unique part of the filename, otherwise a filename will be determined automatically based on the static portion specified.</p>
    /// <p>If <i>name</i> contains a relative file path, the path will be relative to the current working directory. You can use <a href="http://doc.qt.io/qt-5/qdir.html#tempPath">QDir::tempPath</a>() to construct <i>name</i> if you want use the system's temporary directory.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtemporaryfile.html#fileTemplate">fileTemplate</a>().</p></div>
    pub unsafe fn set_file_template(
        &mut self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QTemporaryFile_setFileTemplate(
            self as *mut crate::QTemporaryFile,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
        )
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTemporaryFile_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QTemporaryFile::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTemporaryFile_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QTemporaryFile::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTemporaryFile_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_text_boundary_finder {
    //! C++ type: <span style='color: green;'>```QTextBoundaryFinder```</span>

    /// C++ enum: <span style='color: green;'>```QTextBoundaryFinder::BoundaryType```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#BoundaryType-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct BoundaryType(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for BoundaryType {
        fn from(value: ::std::os::raw::c_int) -> Self {
            BoundaryType(value)
        }
    }

    impl From<BoundaryType> for ::std::os::raw::c_int {
        fn from(value: BoundaryType) -> Self {
            value.0
        }
    }

    impl BoundaryType {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl BoundaryType {
        /// Finds a grapheme which is the smallest boundary. It including letters, punctuation marks, numerals and more. (C++ enum variant: <span style='color: green;'>```Grapheme = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const Grapheme: crate::q_text_boundary_finder::BoundaryType =
            crate::q_text_boundary_finder::BoundaryType(0);
        /// Finds a word. (C++ enum variant: <span style='color: green;'>```Word = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Word: crate::q_text_boundary_finder::BoundaryType =
            crate::q_text_boundary_finder::BoundaryType(1);
        /// Finds sentence boundaries. These include periods, question marks etc. (C++ enum variant: <span style='color: green;'>```Sentence = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const Sentence: crate::q_text_boundary_finder::BoundaryType =
            crate::q_text_boundary_finder::BoundaryType(2);
        /// Finds possible positions for breaking the text into multiple lines. (C++ enum variant: <span style='color: green;'>```Line = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const Line: crate::q_text_boundary_finder::BoundaryType =
            crate::q_text_boundary_finder::BoundaryType(3);
    }

    /// <p>The BoundaryReasons type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;BoundaryReason&gt;. It stores an OR combination of BoundaryReason values.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QTextBoundaryFinder::BoundaryReason```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#BoundaryReason-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The BoundaryReasons type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;BoundaryReason&gt;. It stores an OR combination of BoundaryReason values.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct BoundaryReason(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for BoundaryReason {
        fn from(value: ::std::os::raw::c_int) -> Self {
            BoundaryReason(value)
        }
    }

    impl From<BoundaryReason> for ::std::os::raw::c_int {
        fn from(value: BoundaryReason) -> Self {
            value.0
        }
    }

    impl BoundaryReason {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl BoundaryReason {
        /// The boundary finder is not at a boundary position. (C++ enum variant: <span style='color: green;'>```NotAtBoundary = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NotAtBoundary: crate::q_text_boundary_finder::BoundaryReason =
            crate::q_text_boundary_finder::BoundaryReason(0);
        /// The boundary finder is at a break opportunity position. Such a break opportunity might also be an item boundary (either StartOfItem, EndOfItem, or combination of both), a mandatory line break, or a soft hyphen. (C++ enum variant: <span style='color: green;'>```BreakOpportunity = 31```</span>)
        #[allow(non_upper_case_globals)]
        pub const BreakOpportunity: crate::q_text_boundary_finder::BoundaryReason =
            crate::q_text_boundary_finder::BoundaryReason(31);
        /// Since 5.0. The boundary finder is at the start of a grapheme, a word, a sentence, or a line. (C++ enum variant: <span style='color: green;'>```StartOfItem = 32```</span>)
        #[allow(non_upper_case_globals)]
        pub const StartOfItem: crate::q_text_boundary_finder::BoundaryReason =
            crate::q_text_boundary_finder::BoundaryReason(32);
        /// Since 5.0. The boundary finder is at the end of a grapheme, a word, a sentence, or a line. (C++ enum variant: <span style='color: green;'>```EndOfItem = 64```</span>)
        #[allow(non_upper_case_globals)]
        pub const EndOfItem: crate::q_text_boundary_finder::BoundaryReason =
            crate::q_text_boundary_finder::BoundaryReason(64);
        /// Since 5.0. The boundary finder is at the end of line (can occur for a Line boundary type only). (C++ enum variant: <span style='color: green;'>```MandatoryBreak = 128```</span>)
        #[allow(non_upper_case_globals)]
        pub const MandatoryBreak: crate::q_text_boundary_finder::BoundaryReason =
            crate::q_text_boundary_finder::BoundaryReason(128);
        /// The boundary finder is at the soft hyphen (can occur for a Line boundary type only). (C++ enum variant: <span style='color: green;'>```SoftHyphen = 256```</span>)
        #[allow(non_upper_case_globals)]
        pub const SoftHyphen: crate::q_text_boundary_finder::BoundaryReason =
            crate::q_text_boundary_finder::BoundaryReason(256);
    }

    impl From<crate::q_text_boundary_finder::BoundaryReason>
        for crate::QFlags<crate::q_text_boundary_finder::BoundaryReason>
    {
        fn from(value: crate::q_text_boundary_finder::BoundaryReason) -> Self {
            Self::from(value.to_int())
        }
    }

    impl<T: Into<crate::QFlags<crate::q_text_boundary_finder::BoundaryReason>>> std::ops::BitOr<T>
        for crate::q_text_boundary_finder::BoundaryReason
    {
        type Output = crate::QFlags<crate::q_text_boundary_finder::BoundaryReason>;
        fn bitor(self, rhs: T) -> crate::QFlags<crate::q_text_boundary_finder::BoundaryReason> {
            Into::<crate::QFlags<crate::q_text_boundary_finder::BoundaryReason>>::into(self) | rhs
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> class provides a way of finding Unicode text boundaries in a string.</p>
///
/// C++ class: <span style='color: green;'>```QTextBoundaryFinder```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> class provides a way of finding Unicode text boundaries in a string.</p>
/// <p><a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> allows to find Unicode text boundaries in a string, accordingly to the Unicode text boundary specification (see <a href="http://www.unicode.org/reports/tr14/">Unicode Standard Annex #14</a> and <a href="http://www.unicode.org/reports/tr29/">Unicode Standard Annex #29</a>).</p>
/// <p><a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> can operate on a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> in four possible modes depending on the value of <i>BoundaryType</i>.</p>
/// <p>Units of Unicode characters that make up what the user thinks of as a character or basic unit of the language are here called Grapheme clusters. The two unicode characters 'A' + diaeresis do for example form one grapheme cluster as the user thinks of them as one character, yet it is in this case represented by two unicode code points (see <a href="http://www.unicode.org/reports/tr29/#Grapheme_Cluster_Boundaries">http://www.unicode.org/reports/tr29/#Grapheme_Cluster_Boundaries</a>).</p>
/// <p>Word boundaries are there to locate the start and end of what a language considers to be a word (see <a href="http://www.unicode.org/reports/tr29/#Word_Boundaries">http://www.unicode.org/reports/tr29/#Word_Boundaries</a>).</p>
/// <p>Line break boundaries give possible places where a line break might happen and sentence boundaries will show the beginning and end of whole sentences (see <a href="http://www.unicode.org/reports/tr29/#Sentence_Boundaries">http://www.unicode.org/reports/tr29/#Sentence_Boundaries</a> and <a href="http://www.unicode.org/reports/tr14/">http://www.unicode.org/reports/tr14/</a>).</p>
/// <p>The first position in a string is always a valid boundary and refers to the position before the first character. The last position at the length of the string is also valid and refers to the position after the last character.</p></div>
#[repr(C)]
pub struct QTextBoundaryFinder {
    _unused: u8,
}
impl QTextBoundaryFinder {
    /// <p>Returns the reasons for the boundary finder to have chosen the current position as a boundary.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFlags<QTextBoundaryFinder::BoundaryReason> QTextBoundaryFinder::boundaryReasons() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#boundaryReasons">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the reasons for the boundary finder to have chosen the current position as a boundary.</p></div>
    pub unsafe fn boundary_reasons(
        &self,
    ) -> crate::QFlags<crate::q_text_boundary_finder::BoundaryReason> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextBoundaryFinder_boundaryReasons(
            self as *const crate::QTextBoundaryFinder,
        );
        crate::QFlags::from(ffi_result)
    }

    /// <p>Assigns the object, <i>other</i>, to another <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextBoundaryFinder& QTextBoundaryFinder::operator=(const QTextBoundaryFinder& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns the object, <i>other</i>, to another <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> object.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTextBoundaryFinder>>,
    ) -> ::cpp_utils::MutRef<crate::QTextBoundaryFinder> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextBoundaryFinder_operator_(
            self as *mut crate::QTextBoundaryFinder,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTextBoundaryFinder>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the object's <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#position">position</a>() is currently at a valid text boundary.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTextBoundaryFinder::isAtBoundary() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#isAtBoundary">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the object's <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#position">position</a>() is currently at a valid text boundary.</p></div>
    pub unsafe fn is_at_boundary(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTextBoundaryFinder_isAtBoundary(
            self as *const crate::QTextBoundaryFinder,
        )
    }

    /// <p>Returns <code>true</code> if the text boundary finder is valid; otherwise returns <code>false</code>. A default <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> is invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTextBoundaryFinder::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the text boundary finder is valid; otherwise returns <code>false</code>. A default <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> is invalid.</p></div>
    pub unsafe fn is_valid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTextBoundaryFinder_isValid(
            self as *const crate::QTextBoundaryFinder,
        )
    }

    /// <p>Constructs an invalid <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTextBoundaryFinder::QTextBoundaryFinder()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#QTextBoundaryFinder">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an invalid <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> object.</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QTextBoundaryFinder> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextBoundaryFinder_QTextBoundaryFinder();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> object of <i>type</i> operating on <i>string</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTextBoundaryFinder::QTextBoundaryFinder(QTextBoundaryFinder::BoundaryType type, const QString& string)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#QTextBoundaryFinder-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> object of <i>type</i> operating on <i>string</i>.</p></div>
    pub unsafe fn new_2a(
        type_: crate::q_text_boundary_finder::BoundaryType,
        string: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QTextBoundaryFinder> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextBoundaryFinder_QTextBoundaryFinder2(
            type_,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(string)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> object of <i>type</i> operating on <i>chars</i> with <i>length</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTextBoundaryFinder::QTextBoundaryFinder(QTextBoundaryFinder::BoundaryType type, const QChar* chars, int length, unsigned char* buffer = …, int bufferSize = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#QTextBoundaryFinder-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> object of <i>type</i> operating on <i>chars</i> with <i>length</i>.</p>
    /// <p><i>buffer</i> is an optional working buffer of size <i>bufferSize</i> you can pass to the <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a>. If the buffer is large enough to hold the working data required (bufferSize &gt;= length + 1), it will use this instead of allocating its own buffer.</p>
    /// <p><b>Warning:</b> <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> does not create a copy of <i>chars</i>. It is the application programmer's responsibility to ensure the array is allocated for as long as the <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> object stays alive. The same applies to <i>buffer</i>.</p></div>
    pub unsafe fn new_5a(
        type_: crate::q_text_boundary_finder::BoundaryType,
        chars: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QChar>>,
        length: ::std::os::raw::c_int,
        buffer: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_uchar>>,
        buffer_size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QTextBoundaryFinder> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextBoundaryFinder_QTextBoundaryFinder3(
            type_,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QChar>>::cast_into(chars).as_raw_ptr(),
            length,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_uchar>>::cast_into(
                buffer,
            )
            .as_mut_raw_ptr(),
            buffer_size,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> object of <i>type</i> operating on <i>chars</i> with <i>length</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTextBoundaryFinder::QTextBoundaryFinder(QTextBoundaryFinder::BoundaryType type, const QChar* chars, int length, unsigned char* buffer = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#QTextBoundaryFinder-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> object of <i>type</i> operating on <i>chars</i> with <i>length</i>.</p>
    /// <p><i>buffer</i> is an optional working buffer of size <i>bufferSize</i> you can pass to the <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a>. If the buffer is large enough to hold the working data required (bufferSize &gt;= length + 1), it will use this instead of allocating its own buffer.</p>
    /// <p><b>Warning:</b> <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> does not create a copy of <i>chars</i>. It is the application programmer's responsibility to ensure the array is allocated for as long as the <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> object stays alive. The same applies to <i>buffer</i>.</p></div>
    pub unsafe fn new_4a(
        type_: crate::q_text_boundary_finder::BoundaryType,
        chars: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QChar>>,
        length: ::std::os::raw::c_int,
        buffer: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_uchar>>,
    ) -> ::cpp_utils::CppBox<crate::QTextBoundaryFinder> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextBoundaryFinder_QTextBoundaryFinder4(
            type_,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QChar>>::cast_into(chars).as_raw_ptr(),
            length,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_uchar>>::cast_into(
                buffer,
            )
            .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> object of <i>type</i> operating on <i>chars</i> with <i>length</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTextBoundaryFinder::QTextBoundaryFinder(QTextBoundaryFinder::BoundaryType type, const QChar* chars, int length)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#QTextBoundaryFinder-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> object of <i>type</i> operating on <i>chars</i> with <i>length</i>.</p>
    /// <p><i>buffer</i> is an optional working buffer of size <i>bufferSize</i> you can pass to the <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a>. If the buffer is large enough to hold the working data required (bufferSize &gt;= length + 1), it will use this instead of allocating its own buffer.</p>
    /// <p><b>Warning:</b> <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> does not create a copy of <i>chars</i>. It is the application programmer's responsibility to ensure the array is allocated for as long as the <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> object stays alive. The same applies to <i>buffer</i>.</p></div>
    pub unsafe fn new_3a(
        type_: crate::q_text_boundary_finder::BoundaryType,
        chars: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QChar>>,
        length: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QTextBoundaryFinder> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextBoundaryFinder_QTextBoundaryFinder5(
            type_,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QChar>>::cast_into(chars).as_raw_ptr(),
            length,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Copies the <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> object, <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTextBoundaryFinder::QTextBoundaryFinder(const QTextBoundaryFinder& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#QTextBoundaryFinder-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Copies the <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> object, <i>other</i>.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTextBoundaryFinder>>,
    ) -> ::cpp_utils::CppBox<crate::QTextBoundaryFinder> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextBoundaryFinder_QTextBoundaryFinder1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTextBoundaryFinder>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the current position of the <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTextBoundaryFinder::position() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#position">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current position of the <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a>.</p>
    /// <p>The range is from 0 (the beginning of the string) to the length of the string inclusive.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#setPosition">setPosition</a>().</p></div>
    pub unsafe fn position(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTextBoundaryFinder_position(
            self as *const crate::QTextBoundaryFinder,
        )
    }

    /// <p>Sets the current position of the <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> to <i>position</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTextBoundaryFinder::setPosition(int position)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#setPosition">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the current position of the <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> to <i>position</i>.</p>
    /// <p>If <i>position</i> is out of bounds, it will be bound to only valid positions. In this case, valid positions are from 0 to the length of the string inclusive.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#position">position</a>().</p></div>
    pub unsafe fn set_position(&mut self, position: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QTextBoundaryFinder_setPosition(
            self as *mut crate::QTextBoundaryFinder,
            position,
        )
    }

    /// <p>Returns the string the <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> object operates on.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QTextBoundaryFinder::string() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#string">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string the <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> object operates on.</p></div>
    pub unsafe fn string(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextBoundaryFinder_string(
            self as *const crate::QTextBoundaryFinder,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the finder to the end of the string. This is equivalent to <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#setPosition">setPosition</a>(string.length()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTextBoundaryFinder::toEnd()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#toEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the finder to the end of the string. This is equivalent to <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#setPosition">setPosition</a>(string.length()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#setPosition">setPosition</a>() and <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#position">position</a>().</p></div>
    pub unsafe fn to_end(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QTextBoundaryFinder_toEnd(
            self as *mut crate::QTextBoundaryFinder,
        )
    }

    /// <p>Moves the <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> to the next boundary position and returns that position.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTextBoundaryFinder::toNextBoundary()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#toNextBoundary">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> to the next boundary position and returns that position.</p>
    /// <p>Returns -1 if there is no next boundary.</p></div>
    pub unsafe fn to_next_boundary(&mut self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTextBoundaryFinder_toNextBoundary(
            self as *mut crate::QTextBoundaryFinder,
        )
    }

    /// <p>Moves the <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> to the previous boundary position and returns that position.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTextBoundaryFinder::toPreviousBoundary()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#toPreviousBoundary">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> to the previous boundary position and returns that position.</p>
    /// <p>Returns -1 if there is no previous boundary.</p></div>
    pub unsafe fn to_previous_boundary(&mut self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTextBoundaryFinder_toPreviousBoundary(
            self as *mut crate::QTextBoundaryFinder,
        )
    }

    /// <p>Moves the finder to the start of the string. This is equivalent to <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#setPosition">setPosition</a>(0).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTextBoundaryFinder::toStart()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#toStart">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the finder to the start of the string. This is equivalent to <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#setPosition">setPosition</a>(0).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#setPosition">setPosition</a>() and <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#position">position</a>().</p></div>
    pub unsafe fn to_start(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QTextBoundaryFinder_toStart(
            self as *mut crate::QTextBoundaryFinder,
        )
    }

    /// <p>Returns the type of the <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextBoundaryFinder::BoundaryType QTextBoundaryFinder::type() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#type">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the type of the <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a>.</p></div>
    pub unsafe fn type_(&self) -> crate::q_text_boundary_finder::BoundaryType {
        crate::__ffi::ctr_qt_core_ffi_QTextBoundaryFinder_type(
            self as *const crate::QTextBoundaryFinder,
        )
    }
}

pub mod q_text_codec {
    //! C++ type: <span style='color: green;'>```QTextCodec```</span>

    /// <p>The ConversionFlags type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;ConversionFlag&gt;. It stores an OR combination of ConversionFlag values.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QTextCodec::ConversionFlag```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextcodec.html#ConversionFlag-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The ConversionFlags type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;ConversionFlag&gt;. It stores an OR combination of ConversionFlag values.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct ConversionFlag(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for ConversionFlag {
        fn from(value: ::std::os::raw::c_int) -> Self {
            ConversionFlag(value)
        }
    }

    impl From<ConversionFlag> for ::std::os::raw::c_int {
        fn from(value: ConversionFlag) -> Self {
            value.0
        }
    }

    impl ConversionFlag {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl ConversionFlag {
        /// No flag is set. (C++ enum variant: <span style='color: green;'>```DefaultConversion = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const DefaultConversion: crate::q_text_codec::ConversionFlag =
            crate::q_text_codec::ConversionFlag(0);
        /// If this flag is set, each invalid input character is output as a null character. (C++ enum variant: <span style='color: green;'>```ConvertInvalidToNull = -2147483648```</span>)
        #[allow(non_upper_case_globals)]
        pub const ConvertInvalidToNull: crate::q_text_codec::ConversionFlag =
            crate::q_text_codec::ConversionFlag(-2147483648);
        /// Ignore any Unicode byte-order mark and don't generate any. (C++ enum variant: <span style='color: green;'>```IgnoreHeader = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const IgnoreHeader: crate::q_text_codec::ConversionFlag =
            crate::q_text_codec::ConversionFlag(1);
        /// C++ enum variant: <span style='color: green;'>```FreeFunction = 2```</span>
        #[allow(non_upper_case_globals)]
        pub const FreeFunction: crate::q_text_codec::ConversionFlag =
            crate::q_text_codec::ConversionFlag(2);
    }

    impl From<crate::q_text_codec::ConversionFlag>
        for crate::QFlags<crate::q_text_codec::ConversionFlag>
    {
        fn from(value: crate::q_text_codec::ConversionFlag) -> Self {
            Self::from(value.to_int())
        }
    }

    impl<T: Into<crate::QFlags<crate::q_text_codec::ConversionFlag>>> std::ops::BitOr<T>
        for crate::q_text_codec::ConversionFlag
    {
        type Output = crate::QFlags<crate::q_text_codec::ConversionFlag>;
        fn bitor(self, rhs: T) -> crate::QFlags<crate::q_text_codec::ConversionFlag> {
            Into::<crate::QFlags<crate::q_text_codec::ConversionFlag>>::into(self) | rhs
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> class provides conversions between text encodings.</p>
///
/// C++ class: <span style='color: green;'>```QTextCodec```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtextcodec.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> class provides conversions between text encodings.</p>
/// <p>Qt uses Unicode to store, draw and manipulate strings. In many situations you may wish to deal with data that uses a different encoding. For example, most Japanese documents are still stored in Shift-JIS or ISO 2022-JP, while Russian users often have their documents in KOI8-R or Windows-1251.</p>
/// <p>Qt provides a set of <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> classes to help with converting non-Unicode formats to and from Unicode. You can also create your own codec classes.</p>
/// <p>The supported encodings are:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/codec-big5.html">Big5</a></li>
/// <li><a href="http://doc.qt.io/qt-5/codec-big5hkscs.html">Big5-HKSCS</a></li>
/// <li>CP949</li>
/// <li><a href="http://doc.qt.io/qt-5/codec-eucjp.html">EUC-JP</a></li>
/// <li><a href="http://doc.qt.io/qt-5/codec-euckr.html">EUC-KR</a></li>
/// <li><a href="http://doc.qt.io/qt-5/codec-gbk.html">GB18030</a></li>
/// <li>HP-ROMAN8</li>
/// <li>IBM 850</li>
/// <li>IBM 866</li>
/// <li>IBM 874</li>
/// <li><a href="http://doc.qt.io/qt-5/codecs-jis.html">ISO 2022-JP</a></li>
/// <li>ISO 8859-1 to 10</li>
/// <li>ISO 8859-13 to 16</li>
/// <li>Iscii-Bng, Dev, Gjr, Knd, Mlm, Ori, Pnj, Tlg, and Tml</li>
/// <li>KOI8-R</li>
/// <li>KOI8-U</li>
/// <li>Macintosh</li>
/// <li><a href="http://doc.qt.io/qt-5/codec-sjis.html">Shift-JIS</a></li>
/// <li>TIS-620</li>
/// <li><a href="http://doc.qt.io/qt-5/codec-tscii.html">TSCII</a></li>
/// <li>UTF-8</li>
/// <li>UTF-16</li>
/// <li>UTF-16BE</li>
/// <li>UTF-16LE</li>
/// <li>UTF-32</li>
/// <li>UTF-32BE</li>
/// <li>UTF-32LE</li>
/// <li>Windows-1250 to 1258</li>
/// </ul>
/// <p>If Qt is compiled with ICU support enabled, most codecs supported by ICU will also be available to the application.</p>
/// <p><a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a>s can be used as follows to convert some locally encoded string to Unicode. Suppose you have some string encoded in Russian KOI8-R encoding, and want to convert it to Unicode. The simple way to do it is like this:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span> encodedString <span class="operator">=</span> <span class="string">"..."</span>;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qtextcodec.html#QTextCodec">QTextCodec</a></span> <span class="operator">*</span>codec <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtextcodec.html#QTextCodec">QTextCodec</a></span><span class="operator">::</span>codecForName(<span class="string">"KOI8-R"</span>);
///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> string <span class="operator">=</span> codec<span class="operator">-</span><span class="operator">&gt;</span>toUnicode(encodedString);
///
/// </pre>
/// <p>After this, <code>string</code> holds the text converted to Unicode. Converting a string from Unicode to the local encoding is just as easy:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> string <span class="operator">=</span> <span class="string">"..."</span>;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qtextcodec.html#QTextCodec">QTextCodec</a></span> <span class="operator">*</span>codec <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtextcodec.html#QTextCodec">QTextCodec</a></span><span class="operator">::</span>codecForName(<span class="string">"KOI8-R"</span>);
///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span> encodedString <span class="operator">=</span> codec<span class="operator">-</span><span class="operator">&gt;</span>fromUnicode(string);
///
/// </pre>
/// <p>To read or write files in various encodings, use <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> and its <a href="http://doc.qt.io/qt-5/qtextstream.html#setCodec">setCodec()</a> function. See the <a href="http://doc.qt.io/qt-5/qtwidgets-tools-codecs-example.html">Codecs</a> example for an application of <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> to file I/O.</p>
/// <p>Some care must be taken when trying to convert the data in chunks, for example, when receiving it over a network. In such cases it is possible that a multi-byte character will be split over two chunks. At best this might result in the loss of a character and at worst cause the entire conversion to fail.</p>
/// <p>The approach to use in these situations is to create a <a href="http://doc.qt.io/qt-5/qtextdecoder.html">QTextDecoder</a> object for the codec and use this <a href="http://doc.qt.io/qt-5/qtextdecoder.html">QTextDecoder</a> for the whole decoding process, as shown below:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qtextcodec.html#QTextCodec">QTextCodec</a></span> <span class="operator">*</span>codec <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtextcodec.html#QTextCodec">QTextCodec</a></span><span class="operator">::</span>codecForName(<span class="string">"Shift-JIS"</span>);
///   <span class="type"><a href="http://doc.qt.io/qt-5/qtextdecoder.html">QTextDecoder</a></span> <span class="operator">*</span>decoder <span class="operator">=</span> codec<span class="operator">-</span><span class="operator">&gt;</span>makeDecoder();
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> string;
///   <span class="keyword">while</span> (new_data_available()) {
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span> chunk <span class="operator">=</span> get_new_data();
/// &#32;     string <span class="operator">+</span><span class="operator">=</span> decoder<span class="operator">-</span><span class="operator">&gt;</span>toUnicode(chunk);
///   }
///   <span class="keyword">delete</span> decoder;
///
/// </pre>
/// <p>The <a href="http://doc.qt.io/qt-5/qtextdecoder.html">QTextDecoder</a> object maintains state between chunks and therefore works correctly even if a multi-byte character is split between chunks.</p>
/// <a name="creating-your-own-codec-class"></a></div>
#[repr(C)]
pub struct QTextCodec {
    _unused: u8,
}
impl QTextCodec {
    /// <p>Subclasses can return a number of aliases for the codec in question.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QList<QByteArray> QTextCodec::aliases() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextcodec.html#aliases">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Subclasses can return a number of aliases for the codec in question.</p>
    /// <p>Standard aliases for codecs can be found in the <a href="http://www.iana.org/assignments/character-sets/character-sets.xml">IANA character-sets encoding file</a>.</p></div>
    pub unsafe fn aliases(&self) -> ::cpp_utils::CppBox<crate::QListOfQByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QTextCodec_aliases(self as *const crate::QTextCodec);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the list of all available codecs, by name. Call <a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForName">QTextCodec::codecForName</a>() to obtain the <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> for the name.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QList<QByteArray> QTextCodec::availableCodecs()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextcodec.html#availableCodecs">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the list of all available codecs, by name. Call <a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForName">QTextCodec::codecForName</a>() to obtain the <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> for the name.</p>
    /// <p>The list may contain many mentions of the same codec if the codec has aliases.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextcodec.html#availableMibs">availableMibs</a>(), <a href="http://doc.qt.io/qt-5/qtextcodec.html#name">name</a>(), and <a href="http://doc.qt.io/qt-5/qtextcodec.html#aliases">aliases</a>().</p></div>
    pub unsafe fn available_codecs() -> ::cpp_utils::CppBox<crate::QListOfQByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextCodec_availableCodecs();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the list of MIBs for all available codecs. Call <a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForMib">QTextCodec::codecForMib</a>() to obtain the <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> for the MIB.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QList<int> QTextCodec::availableMibs()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextcodec.html#availableMibs">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the list of MIBs for all available codecs. Call <a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForMib">QTextCodec::codecForMib</a>() to obtain the <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> for the MIB.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextcodec.html#availableCodecs">availableCodecs</a>() and <a href="http://doc.qt.io/qt-5/qtextcodec.html#mibEnum">mibEnum</a>().</p></div>
    pub unsafe fn available_mibs() -> ::cpp_utils::CppBox<crate::QListOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextCodec_availableMibs();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if the Unicode character <i>ch</i> can be fully encoded with this codec; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTextCodec::canEncode(QChar arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextcodec.html#canEncode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the Unicode character <i>ch</i> can be fully encoded with this codec; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn can_encode_q_char(
        &self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTextCodec_canEncode(
            self as *const crate::QTextCodec,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(arg1).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTextCodec::canEncode(const QString& arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextcodec.html#canEncode-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><i>s</i> contains the string being tested for encode-ability.</p></div>
    pub unsafe fn can_encode_q_string(
        &self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTextCodec_canEncode1(
            self as *const crate::QTextCodec,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(arg1).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTextCodec::canEncode(QStringView arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextcodec.html#canEncode-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if the Unicode string <i>s</i> can be fully encoded with this codec; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn can_encode_q_string_view(
        &self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTextCodec_canEncode4(
            self as *const crate::QTextCodec,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(arg1)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QTextCodec* QTextCodec::codecForHtml(const QByteArray& ba)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForHtml-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Tries to detect the encoding of the provided snippet of HTML in the given byte array, <i>ba</i>, by checking the BOM (Byte Order Mark) and the content-type meta header and returns a <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> instance that is capable of decoding the html to unicode. If the codec cannot be detected, this overload returns a Latin-1 <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a>.</p></div>
    pub unsafe fn codec_for_html_1a(
        ba: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::MutPtr<crate::QTextCodec> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextCodec_codecForHtml(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(ba)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Tries to detect the encoding of the provided snippet of HTML in the given byte array, <i>ba</i>, by checking the BOM (Byte Order Mark) and the content-type meta header and returns a <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> instance that is capable of decoding the html to unicode. If the codec cannot be detected from the content provided, <i>defaultCodec</i> is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QTextCodec* QTextCodec::codecForHtml(const QByteArray& ba, QTextCodec* defaultCodec)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForHtml">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Tries to detect the encoding of the provided snippet of HTML in the given byte array, <i>ba</i>, by checking the BOM (Byte Order Mark) and the content-type meta header and returns a <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> instance that is capable of decoding the html to unicode. If the codec cannot be detected from the content provided, <i>defaultCodec</i> is returned.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForUtfText">codecForUtfText</a>().</p></div>
    pub unsafe fn codec_for_html_2a(
        ba: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        default_codec: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QTextCodec>>,
    ) -> ::cpp_utils::MutPtr<crate::QTextCodec> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextCodec_codecForHtml1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(ba)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QTextCodec>>::cast_into(
                default_codec,
            )
            .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns a pointer to the codec most suitable for this locale.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QTextCodec* QTextCodec::codecForLocale()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForLocale">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the codec most suitable for this locale.</p>
    /// <p>On Windows, the codec will be based on a system locale. On Unix systems, the codec will might fall back to using the <i>iconv</i> library if no builtin codec for the locale can be found.</p>
    /// <p>Note that in these cases the codec's name will be "System".</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextcodec.html#setCodecForLocale">setCodecForLocale</a>().</p></div>
    pub unsafe fn codec_for_locale() -> ::cpp_utils::MutPtr<crate::QTextCodec> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextCodec_codecForLocale();
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns the <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> which matches the <a href="http://doc.qt.io/qt-5/qtextcodec.html#mibEnum">MIBenum</a> <i>mib</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QTextCodec* QTextCodec::codecForMib(int mib)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForMib">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> which matches the <a href="http://doc.qt.io/qt-5/qtextcodec.html#mibEnum">MIBenum</a> <i>mib</i>.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p></div>
    pub unsafe fn codec_for_mib(
        mib: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutPtr<crate::QTextCodec> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextCodec_codecForMib(mib);
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Searches all installed <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> objects and returns the one which best matches <i>name</i>; the match is case-insensitive. Returns 0 if no codec matching the name <i>name</i> could be found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QTextCodec* QTextCodec::codecForName(const QByteArray& name)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Searches all installed <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> objects and returns the one which best matches <i>name</i>; the match is case-insensitive. Returns 0 if no codec matching the name <i>name</i> could be found.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p></div>
    pub unsafe fn codec_for_name_q_byte_array(
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::MutPtr<crate::QTextCodec> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextCodec_codecForName(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(name)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Searches all installed <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> objects and returns the one which best matches <i>name</i>; the match is case-insensitive. Returns 0 if no codec matching the name <i>name</i> could be found.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QTextCodec* QTextCodec::codecForName(const char* name)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForName-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Searches all installed <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> objects and returns the one which best matches <i>name</i>; the match is case-insensitive. Returns 0 if no codec matching the name <i>name</i> could be found.</p></div>
    pub unsafe fn codec_for_name_char(
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<crate::QTextCodec> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextCodec_codecForName1(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(name)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QTextCodec* QTextCodec::codecForUtfText(const QByteArray& ba)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForUtfText-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Tries to detect the encoding of the provided snippet <i>ba</i> by using the BOM (Byte Order Mark) and returns a <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> instance that is capable of decoding the text to unicode. If the codec cannot be detected, this overload returns a Latin-1 <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForHtml">codecForHtml</a>().</p></div>
    pub unsafe fn codec_for_utf_text_1a(
        ba: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::MutPtr<crate::QTextCodec> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextCodec_codecForUtfText(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(ba)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Tries to detect the encoding of the provided snippet <i>ba</i> by using the BOM (Byte Order Mark) and returns a <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> instance that is capable of decoding the text to unicode. If the codec cannot be detected from the content provided, <i>defaultCodec</i> is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QTextCodec* QTextCodec::codecForUtfText(const QByteArray& ba, QTextCodec* defaultCodec)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForUtfText">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Tries to detect the encoding of the provided snippet <i>ba</i> by using the BOM (Byte Order Mark) and returns a <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> instance that is capable of decoding the text to unicode. If the codec cannot be detected from the content provided, <i>defaultCodec</i> is returned.</p>
    /// <p>This function was introduced in  Qt 4.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForHtml">codecForHtml</a>().</p></div>
    pub unsafe fn codec_for_utf_text_2a(
        ba: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        default_codec: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QTextCodec>>,
    ) -> ::cpp_utils::MutPtr<crate::QTextCodec> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextCodec_codecForUtfText1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(ba)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QTextCodec>>::cast_into(
                default_codec,
            )
            .as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Converts <i>str</i> from Unicode to the encoding of this codec, and returns the result in a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QTextCodec::fromUnicode(const QString& uc) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextcodec.html#fromUnicode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts <i>str</i> from Unicode to the encoding of this codec, and returns the result in a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p></div>
    pub unsafe fn from_unicode_q_string(
        &self,
        uc: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextCodec_fromUnicode(
            self as *const crate::QTextCodec,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(uc).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QTextCodec::fromUnicode(QStringView uc) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextcodec.html#fromUnicode-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Converts <i>str</i> from Unicode to the encoding of this codec, and returns the result in a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn from_unicode_q_string_view(
        &self,
        uc: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextCodec_fromUnicode2(
            self as *const crate::QTextCodec,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(uc)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qtextdecoder.html">QTextDecoder</a> with a specified <i>flags</i> to decode chunks of <code>char *</code> data to create chunks of Unicode data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextDecoder* QTextCodec::makeDecoder(QFlags<QTextCodec::ConversionFlag> flags = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextcodec.html#makeDecoder">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qtextdecoder.html">QTextDecoder</a> with a specified <i>flags</i> to decode chunks of <code>char *</code> data to create chunks of Unicode data.</p>
    /// <p>The caller is responsible for deleting the returned object.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn make_decoder_1a(
        &self,
        flags: crate::QFlags<crate::q_text_codec::ConversionFlag>,
    ) -> ::cpp_utils::MutPtr<crate::QTextDecoder> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextCodec_makeDecoder(
            self as *const crate::QTextCodec,
            flags.to_int(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qtextdecoder.html">QTextDecoder</a> with a specified <i>flags</i> to decode chunks of <code>char *</code> data to create chunks of Unicode data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextDecoder* QTextCodec::makeDecoder() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextcodec.html#makeDecoder">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qtextdecoder.html">QTextDecoder</a> with a specified <i>flags</i> to decode chunks of <code>char *</code> data to create chunks of Unicode data.</p>
    /// <p>The caller is responsible for deleting the returned object.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn make_decoder_0a(&self) -> ::cpp_utils::MutPtr<crate::QTextDecoder> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QTextCodec_makeDecoder1(self as *const crate::QTextCodec);
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qtextencoder.html">QTextEncoder</a> with a specified <i>flags</i> to encode chunks of Unicode data as <code>char *</code> data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextEncoder* QTextCodec::makeEncoder(QFlags<QTextCodec::ConversionFlag> flags = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextcodec.html#makeEncoder">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qtextencoder.html">QTextEncoder</a> with a specified <i>flags</i> to encode chunks of Unicode data as <code>char *</code> data.</p>
    /// <p>The caller is responsible for deleting the returned object.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn make_encoder_1a(
        &self,
        flags: crate::QFlags<crate::q_text_codec::ConversionFlag>,
    ) -> ::cpp_utils::MutPtr<crate::QTextEncoder> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextCodec_makeEncoder(
            self as *const crate::QTextCodec,
            flags.to_int(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qtextencoder.html">QTextEncoder</a> with a specified <i>flags</i> to encode chunks of Unicode data as <code>char *</code> data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextEncoder* QTextCodec::makeEncoder() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextcodec.html#makeEncoder">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qtextencoder.html">QTextEncoder</a> with a specified <i>flags</i> to encode chunks of Unicode data as <code>char *</code> data.</p>
    /// <p>The caller is responsible for deleting the returned object.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn make_encoder_0a(&self) -> ::cpp_utils::MutPtr<crate::QTextEncoder> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QTextCodec_makeEncoder1(self as *const crate::QTextCodec);
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Subclasses of <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> must reimplement this function. It returns the MIBenum (see <a href="http://www.iana.org/assignments/character-sets/character-sets.xml">IANA character-sets encoding file</a> for more information). It is important that each <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> subclass returns the correct unique value for this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual int QTextCodec::mibEnum() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextcodec.html#mibEnum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Subclasses of <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> must reimplement this function. It returns the MIBenum (see <a href="http://www.iana.org/assignments/character-sets/character-sets.xml">IANA character-sets encoding file</a> for more information). It is important that each <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> subclass returns the correct unique value for this function.</p></div>
    pub unsafe fn mib_enum(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTextCodec_mibEnum(self as *const crate::QTextCodec)
    }

    /// <p><a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> subclasses must reimplement this function. It returns the name of the encoding supported by the subclass.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```pure virtual QByteArray QTextCodec::name() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextcodec.html#name">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p><a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> subclasses must reimplement this function. It returns the name of the encoding supported by the subclass.</p>
    /// <p>If the codec is registered as a character set in the <a href="http://www.iana.org/assignments/character-sets/character-sets.xml">IANA character-sets encoding file</a> this method should return the preferred mime name for the codec if defined, otherwise its name.</p></div>
    pub unsafe fn name(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QTextCodec_name(self as *const crate::QTextCodec);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Set the codec to <i>c</i>; this will be returned by <a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForLocale">codecForLocale</a>(). If <i>c</i> is a null pointer, the codec is reset to the default.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QTextCodec::setCodecForLocale(QTextCodec* c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextcodec.html#setCodecForLocale">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Set the codec to <i>c</i>; this will be returned by <a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForLocale">codecForLocale</a>(). If <i>c</i> is a null pointer, the codec is reset to the default.</p>
    /// <p>This might be needed for some applications that want to use their own mechanism for setting the locale.</p>
    /// <p><b>Warning:</b> This function is not <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">reentrant</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForLocale">codecForLocale</a>().</p></div>
    pub unsafe fn set_codec_for_locale(
        c: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QTextCodec>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QTextCodec_setCodecForLocale(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QTextCodec>>::cast_into(c)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Converts <i>a</i> from the encoding of this codec to Unicode, and returns the result in a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QTextCodec::toUnicode(const QByteArray& arg1) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextcodec.html#toUnicode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts <i>a</i> from the encoding of this codec to Unicode, and returns the result in a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p></div>
    pub unsafe fn to_unicode_q_byte_array(
        &self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextCodec_toUnicode(
            self as *const crate::QTextCodec,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QTextCodec::toUnicode(const char* chars) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextcodec.html#toUnicode-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p><i>chars</i> contains the source characters.</p></div>
    pub unsafe fn to_unicode_char(
        &self,
        chars: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextCodec_toUnicode1(
            self as *const crate::QTextCodec,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(chars)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qtextencoder.html">QTextEncoder</a> class provides a state-based encoder.</p>
///
/// C++ class: <span style='color: green;'>```QTextEncoder```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtextencoder.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qtextencoder.html">QTextEncoder</a> class provides a state-based encoder.</p>
/// <p>A text encoder converts text from Unicode into an encoded text format using a specific codec.</p>
/// <p>The encoder converts Unicode into another format, remembering any state that is required between calls.</p></div>
#[repr(C)]
pub struct QTextEncoder {
    _unused: u8,
}
impl QTextEncoder {
    /// <p>Converts the Unicode string <i>str</i> into an encoded <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QTextEncoder::fromUnicode(const QString& str)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextencoder.html#fromUnicode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the Unicode string <i>str</i> into an encoded <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p></div>
    pub unsafe fn from_unicode_q_string(
        &mut self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextEncoder_fromUnicode(
            self as *mut crate::QTextEncoder,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(str).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QTextEncoder::fromUnicode(const QChar* uc, int len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextencoder.html#fromUnicode-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Converts <i>len</i> characters (not bytes) from <i>uc</i>, and returns the result in a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p></div>
    pub unsafe fn from_unicode_q_char_int(
        &mut self,
        uc: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QChar>>,
        len: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextEncoder_fromUnicode1(
            self as *mut crate::QTextEncoder,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QChar>>::cast_into(uc).as_raw_ptr(),
            len,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QTextEncoder::fromUnicode(QStringView str)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextencoder.html#fromUnicode-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Converts the Unicode string <i>str</i> into an encoded <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn from_unicode_q_string_view(
        &mut self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextEncoder_fromUnicode4(
            self as *mut crate::QTextEncoder,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(str)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QTextEncoder::hasFailure() const```</span>.
    pub unsafe fn has_failure(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTextEncoder_hasFailure(self as *const crate::QTextEncoder)
    }

    /// <p>Constructs a text encoder for the given <i>codec</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTextEncoder::QTextEncoder(const QTextCodec* codec)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextencoder.html#QTextEncoder">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a text encoder for the given <i>codec</i>.</p></div>
    pub unsafe fn new_1a(
        codec: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QTextCodec>>,
    ) -> ::cpp_utils::CppBox<crate::QTextEncoder> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextEncoder_QTextEncoder(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QTextCodec>>::cast_into(codec)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a text encoder for the given <i>codec</i> and conversion <i>flags</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTextEncoder::QTextEncoder(const QTextCodec* codec, QFlags<QTextCodec::ConversionFlag> flags)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextencoder.html#QTextEncoder-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a text encoder for the given <i>codec</i> and conversion <i>flags</i>.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn new_2a(
        codec: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QTextCodec>>,
        flags: crate::QFlags<crate::q_text_codec::ConversionFlag>,
    ) -> ::cpp_utils::CppBox<crate::QTextEncoder> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextEncoder_QTextEncoder1(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QTextCodec>>::cast_into(codec)
                .as_raw_ptr(),
            flags.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qtextdecoder.html">QTextDecoder</a> class provides a state-based decoder.</p>
///
/// C++ class: <span style='color: green;'>```QTextDecoder```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtextdecoder.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qtextdecoder.html">QTextDecoder</a> class provides a state-based decoder.</p>
/// <p>A text decoder converts text from an encoded text format into Unicode using a specific codec.</p>
/// <p>The decoder converts text in this format into Unicode, remembering any state that is required between calls.</p></div>
#[repr(C)]
pub struct QTextDecoder {
    _unused: u8,
}
impl QTextDecoder {
    /// Calls C++ function: <span style='color: green;'>```bool QTextDecoder::hasFailure() const```</span>.
    pub unsafe fn has_failure(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTextDecoder_hasFailure(self as *const crate::QTextDecoder)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QTextDecoder::needsMoreData() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn needs_more_data(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTextDecoder_needsMoreData(self as *const crate::QTextDecoder)
    }

    /// <p>Constructs a text decoder for the given <i>codec</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTextDecoder::QTextDecoder(const QTextCodec* codec)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextdecoder.html#QTextDecoder">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a text decoder for the given <i>codec</i>.</p></div>
    pub unsafe fn new_1a(
        codec: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QTextCodec>>,
    ) -> ::cpp_utils::CppBox<crate::QTextDecoder> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextDecoder_QTextDecoder(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QTextCodec>>::cast_into(codec)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a text decoder for the given <i>codec</i> and conversion <i>flags</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTextDecoder::QTextDecoder(const QTextCodec* codec, QFlags<QTextCodec::ConversionFlag> flags)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextdecoder.html#QTextDecoder-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a text decoder for the given <i>codec</i> and conversion <i>flags</i>.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn new_2a(
        codec: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QTextCodec>>,
        flags: crate::QFlags<crate::q_text_codec::ConversionFlag>,
    ) -> ::cpp_utils::CppBox<crate::QTextDecoder> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextDecoder_QTextDecoder1(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QTextCodec>>::cast_into(codec)
                .as_raw_ptr(),
            flags.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts the first <i>len</i> bytes in <i>chars</i> to Unicode, returning the result.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QTextDecoder::toUnicode(const char* chars, int len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextdecoder.html#toUnicode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the first <i>len</i> bytes in <i>chars</i> to Unicode, returning the result.</p>
    /// <p>If not all characters are used (e.g. if only part of a multi-byte encoding is at the end of the characters), the decoder remembers enough state to continue with the next call to this function.</p></div>
    pub unsafe fn to_unicode_2a(
        &mut self,
        chars: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        len: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextDecoder_toUnicode(
            self as *mut crate::QTextDecoder,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(chars)
                .as_raw_ptr(),
            len,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QTextDecoder::toUnicode(const QByteArray& ba)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextdecoder.html#toUnicode-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Converts the bytes in the byte array specified by <i>ba</i> to Unicode and returns the result.</p></div>
    pub unsafe fn to_unicode_1a(
        &mut self,
        ba: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTextDecoder_toUnicode1(
            self as *mut crate::QTextDecoder,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(ba)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTextDecoder::toUnicode(QString* target, const char* chars, int len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextdecoder.html#toUnicode-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>The converted string is returned in <i>target</i>.</p></div>
    pub unsafe fn to_unicode_3a(
        &mut self,
        target: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QString>>,
        chars: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        len: ::std::os::raw::c_int,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QTextDecoder_toUnicode2(
            self as *mut crate::QTextDecoder,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QString>>::cast_into(target)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(chars)
                .as_raw_ptr(),
            len,
        )
    }
}

pub mod q_thread {
    //! C++ type: <span style='color: green;'>```QThread```</span>

    /// <p>This enum type indicates how the operating system should schedule newly created threads.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QThread::Priority```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#Priority-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum type indicates how the operating system should schedule newly created threads.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Priority(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Priority {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Priority(value)
        }
    }

    impl From<Priority> for ::std::os::raw::c_int {
        fn from(value: Priority) -> Self {
            value.0
        }
    }

    impl Priority {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Priority {
        /// scheduled only when no other threads are running. (C++ enum variant: <span style='color: green;'>```IdlePriority = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const IdlePriority: crate::q_thread::Priority = crate::q_thread::Priority(0);
        /// scheduled less often than LowPriority. (C++ enum variant: <span style='color: green;'>```LowestPriority = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const LowestPriority: crate::q_thread::Priority = crate::q_thread::Priority(1);
        /// scheduled less often than NormalPriority. (C++ enum variant: <span style='color: green;'>```LowPriority = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const LowPriority: crate::q_thread::Priority = crate::q_thread::Priority(2);
        /// the default priority of the operating system. (C++ enum variant: <span style='color: green;'>```NormalPriority = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const NormalPriority: crate::q_thread::Priority = crate::q_thread::Priority(3);
        /// scheduled more often than NormalPriority. (C++ enum variant: <span style='color: green;'>```HighPriority = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const HighPriority: crate::q_thread::Priority = crate::q_thread::Priority(4);
        /// scheduled more often than HighPriority. (C++ enum variant: <span style='color: green;'>```HighestPriority = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const HighestPriority: crate::q_thread::Priority = crate::q_thread::Priority(5);
        /// scheduled as often as possible. (C++ enum variant: <span style='color: green;'>```TimeCriticalPriority = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const TimeCriticalPriority: crate::q_thread::Priority = crate::q_thread::Priority(6);
        /// use the same priority as the creating thread. This is the default. (C++ enum variant: <span style='color: green;'>```InheritPriority = 7```</span>)
        #[allow(non_upper_case_globals)]
        pub const InheritPriority: crate::q_thread::Priority = crate::q_thread::Priority(7);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a> class provides a platform-independent way to manage threads.</p>
///
/// C++ class: <span style='color: green;'>```QThread```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qthread.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a> class provides a platform-independent way to manage threads.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a> object manages one thread of control within the program. QThreads begin executing in <a href="http://doc.qt.io/qt-5/qthread.html#run">run</a>(). By default, <a href="http://doc.qt.io/qt-5/qthread.html#run">run</a>() starts the event loop by calling <a href="http://doc.qt.io/qt-5/qthread.html#exec">exec</a>() and runs a Qt event loop inside the thread.</p>
/// <p>You can use worker objects by moving them to the thread using <a href="http://doc.qt.io/qt-5/qobject.html#moveToThread">QObject::moveToThread</a>().</p>
/// <pre class="cpp">
///
///   <span class="keyword">class</span> Worker : <span class="keyword">public</span> <span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span>
///   {
/// &#32;     Q_OBJECT
///
///   <span class="keyword">public</span> <span class="keyword">slots</span>:
/// &#32;     <span class="type">void</span> doWork(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>parameter) {
/// &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> result;
/// &#32;   &#32;     <span class="comment">/* ... here is the expensive or blocking operation ... */</span>
/// &#32;   &#32;     <span class="keyword">emit</span> resultReady(result);
/// &#32;     }
///
///   <span class="keyword">signals</span>:
/// &#32;     <span class="type">void</span> resultReady(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>result);
///   };
///
///   <span class="keyword">class</span> Controller : <span class="keyword">public</span> <span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span>
///   {
/// &#32;     Q_OBJECT
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qthread.html#QThread">QThread</a></span> workerThread;
///   <span class="keyword">public</span>:
/// &#32;     Controller() {
/// &#32;   &#32;     Worker <span class="operator">*</span>worker <span class="operator">=</span> <span class="keyword">new</span> Worker;
/// &#32;   &#32;     worker<span class="operator">-</span><span class="operator">&gt;</span>moveToThread(<span class="operator">&amp;</span>workerThread);
/// &#32;   &#32;     connect(<span class="operator">&amp;</span>workerThread<span class="operator">,</span> <span class="operator">&amp;</span><span class="type"><a href="http://doc.qt.io/qt-5/qthread.html#QThread">QThread</a></span><span class="operator">::</span>finished<span class="operator">,</span> worker<span class="operator">,</span> <span class="operator">&amp;</span><span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span><span class="operator">::</span>deleteLater);
/// &#32;   &#32;     connect(<span class="keyword">this</span><span class="operator">,</span> <span class="operator">&amp;</span>Controller<span class="operator">::</span>operate<span class="operator">,</span> worker<span class="operator">,</span> <span class="operator">&amp;</span>Worker<span class="operator">::</span>doWork);
/// &#32;   &#32;     connect(worker<span class="operator">,</span> <span class="operator">&amp;</span>Worker<span class="operator">::</span>resultReady<span class="operator">,</span> <span class="keyword">this</span><span class="operator">,</span> <span class="operator">&amp;</span>Controller<span class="operator">::</span>handleResults);
/// &#32;   &#32;     workerThread<span class="operator">.</span>start();
/// &#32;     }
/// &#32;     <span class="operator">~</span>Controller() {
/// &#32;   &#32;     workerThread<span class="operator">.</span>quit();
/// &#32;   &#32;     workerThread<span class="operator">.</span>wait();
/// &#32;     }
///   <span class="keyword">public</span> <span class="keyword">slots</span>:
/// &#32;     <span class="type">void</span> handleResults(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>);
///   <span class="keyword">signals</span>:
/// &#32;     <span class="type">void</span> operate(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>);
///   };
///
/// </pre>
/// <p>The code inside the Worker's slot would then execute in a separate thread. However, you are free to connect the Worker's slots to any signal, from any object, in any thread. It is safe to connect signals and slots across different threads, thanks to a mechanism called <a href="http://doc.qt.io/qt-5/qt.html#ConnectionType-enum">queued connections</a>.</p>
/// <p>Another way to make code run in a separate thread, is to subclass <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a> and reimplement <a href="http://doc.qt.io/qt-5/qthread.html#run">run</a>(). For example:</p>
/// <pre class="cpp">
///
///   <span class="keyword">class</span> WorkerThread : <span class="keyword">public</span> <span class="type"><a href="http://doc.qt.io/qt-5/qthread.html#QThread">QThread</a></span>
///   {
/// &#32;     Q_OBJECT
/// &#32;     <span class="type">void</span> run() override {
/// &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> result;
/// &#32;   &#32;     <span class="comment">/* ... here is the expensive or blocking operation ... */</span>
/// &#32;   &#32;     <span class="keyword">emit</span> resultReady(result);
/// &#32;     }
///   <span class="keyword">signals</span>:
/// &#32;     <span class="type">void</span> resultReady(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>s);
///   };
///
///   <span class="type">void</span> MyObject<span class="operator">::</span>startWorkInAThread()
///   {
/// &#32;     WorkerThread <span class="operator">*</span>workerThread <span class="operator">=</span> <span class="keyword">new</span> WorkerThread(<span class="keyword">this</span>);
/// &#32;     connect(workerThread<span class="operator">,</span> <span class="operator">&amp;</span>WorkerThread<span class="operator">::</span>resultReady<span class="operator">,</span> <span class="keyword">this</span><span class="operator">,</span> <span class="operator">&amp;</span>MyObject<span class="operator">::</span>handleResults);
/// &#32;     connect(workerThread<span class="operator">,</span> <span class="operator">&amp;</span>WorkerThread<span class="operator">::</span>finished<span class="operator">,</span> workerThread<span class="operator">,</span> <span class="operator">&amp;</span><span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span><span class="operator">::</span>deleteLater);
/// &#32;     workerThread<span class="operator">-</span><span class="operator">&gt;</span>start();
///   }
///
/// </pre>
/// <p>In that example, the thread will exit after the run function has returned. There will not be any event loop running in the thread unless you call <a href="http://doc.qt.io/qt-5/qthread.html#exec">exec</a>().</p>
/// <p>It is important to remember that a <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a> instance <a href="http://doc.qt.io/qt-5/qobject.html#thread-affinity">lives in</a> the old thread that instantiated it, not in the new thread that calls <a href="http://doc.qt.io/qt-5/qthread.html#run">run</a>(). This means that all of <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a>'s queued slots will execute in the old thread. Thus, a developer who wishes to invoke slots in the new thread must use the worker-object approach; new slots should not be implemented directly into a subclassed <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a>.</p>
/// <p>When subclassing <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a>, keep in mind that the constructor executes in the old thread while <a href="http://doc.qt.io/qt-5/qthread.html#run">run</a>() executes in the new thread. If a member variable is accessed from both functions, then the variable is accessed from two different threads. Check that it is safe to do so.</p>
/// <p><b>Note: </b>Care must be taken when interacting with objects across different threads. See <a href="http://doc.qt.io/qt-5/threads-synchronizing.html">Synchronizing Threads</a> for details.</p><a name="managing-threads"></a></div>
#[repr(C)]
pub struct QThread {
    _unused: u8,
}
impl QThread {
    /// <p>Begins execution of the thread by calling <a href="http://doc.qt.io/qt-5/qthread.html#run">run</a>(). The operating system will schedule the thread according to the <i>priority</i> parameter. If the thread is already running, this function does nothing.</p>
    ///
    /// Returns a built-in Qt slot `QThread::start` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#start">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Begins execution of the thread by calling <a href="http://doc.qt.io/qt-5/qthread.html#run">run</a>(). The operating system will schedule the thread according to the <i>priority</i> parameter. If the thread is already running, this function does nothing.</p>
    /// <p>The effect of the <i>priority</i> parameter is dependent on the operating system's scheduling policy. In particular, the <i>priority</i> will be ignored on systems that do not support thread priorities (such as on Linux, see the <a href="http://linux.die.net/man/2/sched_setscheduler">sched_setscheduler</a> documentation for more details).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#run">run</a>() and <a href="http://doc.qt.io/qt-5/qthread.html#terminate">terminate</a>().</p></div>
    pub fn slot_start(&self) -> crate::Receiver<(crate::q_thread::Priority,)> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1start(QThread::Priority)\0"),
            )
        }
    }

    /// <p>Terminates the execution of the thread. The thread may or may not be terminated immediately, depending on the operating system's scheduling policies. Use <a href="http://doc.qt.io/qt-5/qthread.html#wait">QThread::wait</a>() after terminate(), to be sure.</p>
    ///
    /// Returns a built-in Qt slot `QThread::terminate` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#terminate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Terminates the execution of the thread. The thread may or may not be terminated immediately, depending on the operating system's scheduling policies. Use <a href="http://doc.qt.io/qt-5/qthread.html#wait">QThread::wait</a>() after terminate(), to be sure.</p>
    /// <p>When the thread is terminated, all threads waiting for the thread to finish will be woken up.</p>
    /// <p><b>Warning:</b> This function is dangerous and its use is discouraged. The thread can be terminated at any point in its code path. Threads can be terminated while modifying data. There is no chance for the thread to clean up after itself, unlock any held mutexes, etc. In short, use this function only if absolutely necessary.</p>
    /// <p>Termination can be explicitly enabled or disabled by calling <a href="http://doc.qt.io/qt-5/qthread.html#setTerminationEnabled">QThread::setTerminationEnabled</a>(). Calling this function while termination is disabled results in the termination being deferred, until termination is re-enabled. See the documentation of <a href="http://doc.qt.io/qt-5/qthread.html#setTerminationEnabled">QThread::setTerminationEnabled</a>() for more information.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#setTerminationEnabled">setTerminationEnabled</a>().</p></div>
    pub fn slot_terminate(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1terminate()\0"),
            )
        }
    }

    /// <p>Tells the thread's event loop to exit with return code 0 (success). Equivalent to calling <a href="http://doc.qt.io/qt-5/qthread.html#exit">QThread::exit</a>(0).</p>
    ///
    /// Returns a built-in Qt slot `QThread::quit` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#quit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Tells the thread's event loop to exit with return code 0 (success). Equivalent to calling <a href="http://doc.qt.io/qt-5/qthread.html#exit">QThread::exit</a>(0).</p>
    /// <p>This function does nothing if the thread does not have an event loop.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#exit">exit</a>() and <a href="http://doc.qt.io/qt-5/qeventloop.html">QEventLoop</a>.</p></div>
    pub fn slot_quit(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1quit()\0"),
            )
        }
    }

    /// <p>This signal is emitted from the associated thread when it starts executing, before the <a href="http://doc.qt.io/qt-5/qthread.html#run">run</a>() function is called.</p>
    ///
    /// Returns a built-in Qt signal `QThread::started` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#started">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted from the associated thread when it starts executing, before the <a href="http://doc.qt.io/qt-5/qthread.html#run">run</a>() function is called.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#finished">finished</a>().</p></div>
    pub fn started(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2started()\0"),
            )
        }
    }

    /// <p>This signal is emitted from the associated thread right before it finishes executing.</p>
    ///
    /// Returns a built-in Qt signal `QThread::finished` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#finished">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted from the associated thread right before it finishes executing.</p>
    /// <p>When this signal is emitted, the event loop has already stopped running. No more events will be processed in the thread, except for deferred deletion events. This signal can be connected to <a href="http://doc.qt.io/qt-5/qobject.html#deleteLater">QObject::deleteLater</a>(), to free objects in that thread.</p>
    /// <p><b>Note: </b>If the associated thread was terminated using <a href="http://doc.qt.io/qt-5/qthread.html#terminate">terminate</a>(), it is undefined from which thread this signal is emitted.</p><p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#started">started</a>().</p></div>
    pub fn finished(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2finished()\0"),
            )
        }
    }

    /// <p>Returns a pointer to a <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a> which manages the currently executing thread.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QThread* QThread::currentThread()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#currentThread">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to a <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a> which manages the currently executing thread.</p></div>
    pub unsafe fn current_thread() -> ::cpp_utils::MutPtr<crate::QThread> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QThread_currentThread();
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns the thread handle of the currently executing thread.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void* QThread::currentThreadId()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#currentThreadId">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the thread handle of the currently executing thread.</p>
    /// <p><b>Warning:</b> The handle returned by this function is used for internal purposes and should not be used in any application code.</p>
    /// <p><b>Warning:</b> On Windows, the returned value is a pseudo-handle for the current thread. It can't be used for numerical comparison. i.e., this function returns the DWORD (Windows-Thread ID) returned by the Win32 function getCurrentThreadId(), not the HANDLE (Windows-Thread HANDLE) returned by the Win32 function getCurrentThread().</p></div>
    pub unsafe fn current_thread_id() -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QThread_currentThreadId();
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Reimplemented from <a href="http://doc.qt.io/qt-5/qobject.html#event">QObject::event</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QThread::event(QEvent* event)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#event">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplemented from <a href="http://doc.qt.io/qt-5/qobject.html#event">QObject::event</a>().</p></div>
    pub unsafe fn event(
        &mut self,
        event: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QEvent>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QThread_event(
            self as *mut crate::QThread,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QEvent>>::cast_into(event)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns a pointer to the event dispatcher object for the thread. If no event dispatcher exists for the thread, this function returns 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractEventDispatcher* QThread::eventDispatcher() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#eventDispatcher">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the event dispatcher object for the thread. If no event dispatcher exists for the thread, this function returns 0.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#setEventDispatcher">setEventDispatcher</a>().</p></div>
    pub unsafe fn event_dispatcher(&self) -> ::cpp_utils::MutPtr<crate::QAbstractEventDispatcher> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QThread_eventDispatcher(self as *const crate::QThread);
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Tells the thread's event loop to exit with a return code.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QThread::exit(int retcode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#exit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Tells the thread's event loop to exit with a return code.</p>
    /// <p>After calling this function, the thread leaves the event loop and returns from the call to <a href="http://doc.qt.io/qt-5/qeventloop.html#exec">QEventLoop::exec</a>(). The <a href="http://doc.qt.io/qt-5/qeventloop.html#exec">QEventLoop::exec</a>() function returns <i>returnCode</i>.</p>
    /// <p>By convention, a <i>returnCode</i> of 0 means success, any non-zero value indicates an error.</p>
    /// <p>Note that unlike the C library function of the same name, this function <i>does</i> return to the caller -- it is event processing that stops.</p>
    /// <p>No QEventLoops will be started anymore in this thread until <a href="http://doc.qt.io/qt-5/qthread.html#exec">QThread::exec</a>() has been called again. If the eventloop in <a href="http://doc.qt.io/qt-5/qthread.html#exec">QThread::exec</a>() is not running then the next call to <a href="http://doc.qt.io/qt-5/qthread.html#exec">QThread::exec</a>() will also return immediately.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#quit">quit</a>() and <a href="http://doc.qt.io/qt-5/qeventloop.html">QEventLoop</a>.</p></div>
    pub unsafe fn exit_1a(&mut self, retcode: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QThread_exit(self as *mut crate::QThread, retcode)
    }

    /// <p>Tells the thread's event loop to exit with a return code.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QThread::exit()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#exit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Tells the thread's event loop to exit with a return code.</p>
    /// <p>After calling this function, the thread leaves the event loop and returns from the call to <a href="http://doc.qt.io/qt-5/qeventloop.html#exec">QEventLoop::exec</a>(). The <a href="http://doc.qt.io/qt-5/qeventloop.html#exec">QEventLoop::exec</a>() function returns <i>returnCode</i>.</p>
    /// <p>By convention, a <i>returnCode</i> of 0 means success, any non-zero value indicates an error.</p>
    /// <p>Note that unlike the C library function of the same name, this function <i>does</i> return to the caller -- it is event processing that stops.</p>
    /// <p>No QEventLoops will be started anymore in this thread until <a href="http://doc.qt.io/qt-5/qthread.html#exec">QThread::exec</a>() has been called again. If the eventloop in <a href="http://doc.qt.io/qt-5/qthread.html#exec">QThread::exec</a>() is not running then the next call to <a href="http://doc.qt.io/qt-5/qthread.html#exec">QThread::exec</a>() will also return immediately.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#quit">quit</a>() and <a href="http://doc.qt.io/qt-5/qeventloop.html">QEventLoop</a>.</p></div>
    pub unsafe fn exit_0a(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QThread_exit1(self as *mut crate::QThread)
    }

    /// <p>Returns the ideal number of threads that can be run on the system. This is done querying the number of processor cores, both real and logical, in the system. This function returns -1 if the number of processor cores could not be detected.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static int QThread::idealThreadCount()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#idealThreadCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the ideal number of threads that can be run on the system. This is done querying the number of processor cores, both real and logical, in the system. This function returns -1 if the number of processor cores could not be detected.</p></div>
    pub unsafe fn ideal_thread_count() -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QThread_idealThreadCount()
    }

    /// <p>Returns <code>true</code> if the thread is finished; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QThread::isFinished() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#isFinished">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the thread is finished; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#isRunning">isRunning</a>().</p></div>
    pub unsafe fn is_finished(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QThread_isFinished(self as *const crate::QThread)
    }

    /// <p>Return true if the task running on this thread should be stopped. An interruption can be requested by <a href="http://doc.qt.io/qt-5/qthread.html#requestInterruption">requestInterruption</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QThread::isInterruptionRequested() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#isInterruptionRequested">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Return true if the task running on this thread should be stopped. An interruption can be requested by <a href="http://doc.qt.io/qt-5/qthread.html#requestInterruption">requestInterruption</a>().</p>
    /// <p>This function can be used to make long running tasks cleanly interruptible. Never checking or acting on the value returned by this function is safe, however it is advisable do so regularly in long running functions. Take care not to call it too often, to keep the overhead low.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">void</span> long_task() {
    /// &#32;      forever {
    /// &#32;   &#32;     <span class="keyword">if</span> ( <span class="type"><a href="http://doc.qt.io/qt-5/qthread.html#QThread">QThread</a></span><span class="operator">::</span>currentThread()<span class="operator">-</span><span class="operator">&gt;</span>isInterruptionRequested() ) {
    /// &#32;   &#32;   &#32;     <span class="keyword">return</span>;
    /// &#32;   &#32;     }
    /// &#32;     }
    ///   }
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#currentThread">currentThread</a>() and <a href="http://doc.qt.io/qt-5/qthread.html#requestInterruption">requestInterruption</a>().</p></div>
    pub unsafe fn is_interruption_requested(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QThread_isInterruptionRequested(self as *const crate::QThread)
    }

    /// <p>Returns <code>true</code> if the thread is running; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QThread::isRunning() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#isRunning">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the thread is running; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#isFinished">isFinished</a>().</p></div>
    pub unsafe fn is_running(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QThread_isRunning(self as *const crate::QThread)
    }

    /// <p>Returns the current event loop level for the thread.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QThread::loopLevel() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#loopLevel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current event loop level for the thread.</p>
    /// <p><b>Note: </b>This can only be called within the thread itself, i.e. when it is the current thread.</p><p>This function was introduced in  Qt 5.5.</p></div>
    pub unsafe fn loop_level(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QThread_loopLevel(self as *const crate::QThread)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QThread::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QThread_metaObject(self as *const crate::QThread);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Forces the current thread to sleep for <i>msecs</i> milliseconds.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QThread::msleep(unsigned long arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#msleep">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Forces the current thread to sleep for <i>msecs</i> milliseconds.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#sleep">sleep</a>() and <a href="http://doc.qt.io/qt-5/qthread.html#usleep">usleep</a>().</p></div>
    pub unsafe fn msleep(arg1: ::std::os::raw::c_ulong) {
        crate::__ffi::ctr_qt_core_ffi_QThread_msleep(arg1)
    }

    /// <p>Constructs a new <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a> to manage a new thread. The <i>parent</i> takes ownership of the <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a>. The thread does not begin executing until <a href="http://doc.qt.io/qt-5/qthread.html#start">start</a>() is called.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QThread::QThread(QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#QThread">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a> to manage a new thread. The <i>parent</i> takes ownership of the <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a>. The thread does not begin executing until <a href="http://doc.qt.io/qt-5/qthread.html#start">start</a>() is called.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#start">start</a>().</p></div>
    pub unsafe fn new_1a(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QThread> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QThread_QThread(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a> class provides a platform-independent way to manage threads.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QThread::QThread()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a> class provides a platform-independent way to manage threads.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a> object manages one thread of control within the program. QThreads begin executing in <a href="http://doc.qt.io/qt-5/qthread.html#run">run</a>(). By default, <a href="http://doc.qt.io/qt-5/qthread.html#run">run</a>() starts the event loop by calling <a href="http://doc.qt.io/qt-5/qthread.html#exec">exec</a>() and runs a Qt event loop inside the thread.</p>
    /// <p>You can use worker objects by moving them to the thread using <a href="http://doc.qt.io/qt-5/qobject.html#moveToThread">QObject::moveToThread</a>().</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">class</span> Worker : <span class="keyword">public</span> <span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span>
    ///   {
    /// &#32;     Q_OBJECT
    ///
    ///   <span class="keyword">public</span> <span class="keyword">slots</span>:
    /// &#32;     <span class="type">void</span> doWork(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>parameter) {
    /// &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> result;
    /// &#32;   &#32;     <span class="comment">/* ... here is the expensive or blocking operation ... */</span>
    /// &#32;   &#32;     <span class="keyword">emit</span> resultReady(result);
    /// &#32;     }
    ///
    ///   <span class="keyword">signals</span>:
    /// &#32;     <span class="type">void</span> resultReady(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>result);
    ///   };
    ///
    ///   <span class="keyword">class</span> Controller : <span class="keyword">public</span> <span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span>
    ///   {
    /// &#32;     Q_OBJECT
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qthread.html#QThread">QThread</a></span> workerThread;
    ///   <span class="keyword">public</span>:
    /// &#32;     Controller() {
    /// &#32;   &#32;     Worker <span class="operator">*</span>worker <span class="operator">=</span> <span class="keyword">new</span> Worker;
    /// &#32;   &#32;     worker<span class="operator">-</span><span class="operator">&gt;</span>moveToThread(<span class="operator">&amp;</span>workerThread);
    /// &#32;   &#32;     connect(<span class="operator">&amp;</span>workerThread<span class="operator">,</span> <span class="operator">&amp;</span><span class="type"><a href="http://doc.qt.io/qt-5/qthread.html#QThread">QThread</a></span><span class="operator">::</span>finished<span class="operator">,</span> worker<span class="operator">,</span> <span class="operator">&amp;</span><span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span><span class="operator">::</span>deleteLater);
    /// &#32;   &#32;     connect(<span class="keyword">this</span><span class="operator">,</span> <span class="operator">&amp;</span>Controller<span class="operator">::</span>operate<span class="operator">,</span> worker<span class="operator">,</span> <span class="operator">&amp;</span>Worker<span class="operator">::</span>doWork);
    /// &#32;   &#32;     connect(worker<span class="operator">,</span> <span class="operator">&amp;</span>Worker<span class="operator">::</span>resultReady<span class="operator">,</span> <span class="keyword">this</span><span class="operator">,</span> <span class="operator">&amp;</span>Controller<span class="operator">::</span>handleResults);
    /// &#32;   &#32;     workerThread<span class="operator">.</span>start();
    /// &#32;     }
    /// &#32;     <span class="operator">~</span>Controller() {
    /// &#32;   &#32;     workerThread<span class="operator">.</span>quit();
    /// &#32;   &#32;     workerThread<span class="operator">.</span>wait();
    /// &#32;     }
    ///   <span class="keyword">public</span> <span class="keyword">slots</span>:
    /// &#32;     <span class="type">void</span> handleResults(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>);
    ///   <span class="keyword">signals</span>:
    /// &#32;     <span class="type">void</span> operate(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>);
    ///   };
    ///
    /// </pre>
    /// <p>The code inside the Worker's slot would then execute in a separate thread. However, you are free to connect the Worker's slots to any signal, from any object, in any thread. It is safe to connect signals and slots across different threads, thanks to a mechanism called <a href="http://doc.qt.io/qt-5/qt.html#ConnectionType-enum">queued connections</a>.</p>
    /// <p>Another way to make code run in a separate thread, is to subclass <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a> and reimplement <a href="http://doc.qt.io/qt-5/qthread.html#run">run</a>(). For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">class</span> WorkerThread : <span class="keyword">public</span> <span class="type"><a href="http://doc.qt.io/qt-5/qthread.html#QThread">QThread</a></span>
    ///   {
    /// &#32;     Q_OBJECT
    /// &#32;     <span class="type">void</span> run() override {
    /// &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> result;
    /// &#32;   &#32;     <span class="comment">/* ... here is the expensive or blocking operation ... */</span>
    /// &#32;   &#32;     <span class="keyword">emit</span> resultReady(result);
    /// &#32;     }
    ///   <span class="keyword">signals</span>:
    /// &#32;     <span class="type">void</span> resultReady(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>s);
    ///   };
    ///
    ///   <span class="type">void</span> MyObject<span class="operator">::</span>startWorkInAThread()
    ///   {
    /// &#32;     WorkerThread <span class="operator">*</span>workerThread <span class="operator">=</span> <span class="keyword">new</span> WorkerThread(<span class="keyword">this</span>);
    /// &#32;     connect(workerThread<span class="operator">,</span> <span class="operator">&amp;</span>WorkerThread<span class="operator">::</span>resultReady<span class="operator">,</span> <span class="keyword">this</span><span class="operator">,</span> <span class="operator">&amp;</span>MyObject<span class="operator">::</span>handleResults);
    /// &#32;     connect(workerThread<span class="operator">,</span> <span class="operator">&amp;</span>WorkerThread<span class="operator">::</span>finished<span class="operator">,</span> workerThread<span class="operator">,</span> <span class="operator">&amp;</span><span class="type"><a href="http://doc.qt.io/qt-5/qobject.html#QObject">QObject</a></span><span class="operator">::</span>deleteLater);
    /// &#32;     workerThread<span class="operator">-</span><span class="operator">&gt;</span>start();
    ///   }
    ///
    /// </pre>
    /// <p>In that example, the thread will exit after the run function has returned. There will not be any event loop running in the thread unless you call <a href="http://doc.qt.io/qt-5/qthread.html#exec">exec</a>().</p>
    /// <p>It is important to remember that a <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a> instance <a href="http://doc.qt.io/qt-5/qobject.html#thread-affinity">lives in</a> the old thread that instantiated it, not in the new thread that calls <a href="http://doc.qt.io/qt-5/qthread.html#run">run</a>(). This means that all of <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a>'s queued slots will execute in the old thread. Thus, a developer who wishes to invoke slots in the new thread must use the worker-object approach; new slots should not be implemented directly into a subclassed <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a>.</p>
    /// <p>When subclassing <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a>, keep in mind that the constructor executes in the old thread while <a href="http://doc.qt.io/qt-5/qthread.html#run">run</a>() executes in the new thread. If a member variable is accessed from both functions, then the variable is accessed from two different threads. Check that it is safe to do so.</p>
    /// <p><b>Note: </b>Care must be taken when interacting with objects across different threads. See <a href="http://doc.qt.io/qt-5/threads-synchronizing.html">Synchronizing Threads</a> for details.</p><a name="managing-threads"></a></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QThread> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QThread_QThread1();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the priority for a running thread. If the thread is not running, this function returns <code>InheritPriority</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QThread::Priority QThread::priority() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#priority">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the priority for a running thread. If the thread is not running, this function returns <code>InheritPriority</code>.</p>
    /// <p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#Priority-enum">Priority</a>, <a href="http://doc.qt.io/qt-5/qthread.html#setPriority">setPriority</a>(), and <a href="http://doc.qt.io/qt-5/qthread.html#start">start</a>().</p></div>
    pub unsafe fn priority(&self) -> crate::q_thread::Priority {
        crate::__ffi::ctr_qt_core_ffi_QThread_priority(self as *const crate::QThread)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QThread::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QThread_qt_metacall(
            self as *mut crate::QThread,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QThread::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QThread_qt_metacast(
            self as *mut crate::QThread,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Tells the thread's event loop to exit with return code 0 (success). Equivalent to calling <a href="http://doc.qt.io/qt-5/qthread.html#exit">QThread::exit</a>(0).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QThread::quit()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#quit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Tells the thread's event loop to exit with return code 0 (success). Equivalent to calling <a href="http://doc.qt.io/qt-5/qthread.html#exit">QThread::exit</a>(0).</p>
    /// <p>This function does nothing if the thread does not have an event loop.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#exit">exit</a>() and <a href="http://doc.qt.io/qt-5/qeventloop.html">QEventLoop</a>.</p></div>
    pub unsafe fn quit(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QThread_quit(self as *mut crate::QThread)
    }

    /// <p>Request the interruption of the thread. That request is advisory and it is up to code running on the thread to decide if and how it should act upon such request. This function does not stop any event loop running on the thread and does not terminate it in any way.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QThread::requestInterruption()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#requestInterruption">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Request the interruption of the thread. That request is advisory and it is up to code running on the thread to decide if and how it should act upon such request. This function does not stop any event loop running on the thread and does not terminate it in any way.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#isInterruptionRequested">isInterruptionRequested</a>().</p></div>
    pub unsafe fn request_interruption(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QThread_requestInterruption(self as *mut crate::QThread)
    }

    /// <p>Sets the event dispatcher for the thread to <i>eventDispatcher</i>. This is only possible as long as there is no event dispatcher installed for the thread yet. That is, before the thread has been started with <a href="http://doc.qt.io/qt-5/qthread.html#start">start</a>() or, in case of the main thread, before <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> has been instantiated. This method takes ownership of the object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QThread::setEventDispatcher(QAbstractEventDispatcher* eventDispatcher)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#setEventDispatcher">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the event dispatcher for the thread to <i>eventDispatcher</i>. This is only possible as long as there is no event dispatcher installed for the thread yet. That is, before the thread has been started with <a href="http://doc.qt.io/qt-5/qthread.html#start">start</a>() or, in case of the main thread, before <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> has been instantiated. This method takes ownership of the object.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#eventDispatcher">eventDispatcher</a>().</p></div>
    pub unsafe fn set_event_dispatcher(
        &mut self,
        event_dispatcher: impl ::cpp_utils::CastInto<
            ::cpp_utils::MutPtr<crate::QAbstractEventDispatcher>,
        >,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QThread_setEventDispatcher(self as *mut crate::QThread, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractEventDispatcher>>::cast_into(event_dispatcher).as_mut_raw_ptr())
    }

    /// <p>This function sets the <i>priority</i> for a running thread. If the thread is not running, this function does nothing and returns immediately. Use <a href="http://doc.qt.io/qt-5/qthread.html#start">start</a>() to start a thread with a specific priority.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QThread::setPriority(QThread::Priority priority)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#setPriority">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function sets the <i>priority</i> for a running thread. If the thread is not running, this function does nothing and returns immediately. Use <a href="http://doc.qt.io/qt-5/qthread.html#start">start</a>() to start a thread with a specific priority.</p>
    /// <p>The <i>priority</i> argument can be any value in the <code>QThread::Priority</code> enum except for <code>InheritPriorty</code>.</p>
    /// <p>The effect of the <i>priority</i> parameter is dependent on the operating system's scheduling policy. In particular, the <i>priority</i> will be ignored on systems that do not support thread priorities (such as on Linux, see http://linux.die.net/man/2/sched_setscheduler for more details).</p>
    /// <p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#Priority-enum">Priority</a>, <a href="http://doc.qt.io/qt-5/qthread.html#priority">priority</a>(), and <a href="http://doc.qt.io/qt-5/qthread.html#start">start</a>().</p></div>
    pub unsafe fn set_priority(&mut self, priority: crate::q_thread::Priority) {
        crate::__ffi::ctr_qt_core_ffi_QThread_setPriority(self as *mut crate::QThread, priority)
    }

    /// <p>Sets the maximum stack size for the thread to <i>stackSize</i>. If <i>stackSize</i> is greater than zero, the maximum stack size is set to <i>stackSize</i> bytes, otherwise the maximum stack size is automatically determined by the operating system.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QThread::setStackSize(unsigned int stackSize)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#setStackSize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the maximum stack size for the thread to <i>stackSize</i>. If <i>stackSize</i> is greater than zero, the maximum stack size is set to <i>stackSize</i> bytes, otherwise the maximum stack size is automatically determined by the operating system.</p>
    /// <p><b>Warning:</b> Most operating systems place minimum and maximum limits on thread stack sizes. The thread will fail to start if the stack size is outside these limits.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#stackSize">stackSize</a>().</p></div>
    pub unsafe fn set_stack_size(&mut self, stack_size: ::std::os::raw::c_uint) {
        crate::__ffi::ctr_qt_core_ffi_QThread_setStackSize(self as *mut crate::QThread, stack_size)
    }

    /// <p>Forces the current thread to sleep for <i>secs</i> seconds.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QThread::sleep(unsigned long arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#sleep">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Forces the current thread to sleep for <i>secs</i> seconds.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#msleep">msleep</a>() and <a href="http://doc.qt.io/qt-5/qthread.html#usleep">usleep</a>().</p></div>
    pub unsafe fn sleep(arg1: ::std::os::raw::c_ulong) {
        crate::__ffi::ctr_qt_core_ffi_QThread_sleep(arg1)
    }

    /// <p>Returns the maximum stack size for the thread (if set with <a href="http://doc.qt.io/qt-5/qthread.html#setStackSize">setStackSize</a>()); otherwise returns zero.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QThread::stackSize() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#stackSize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the maximum stack size for the thread (if set with <a href="http://doc.qt.io/qt-5/qthread.html#setStackSize">setStackSize</a>()); otherwise returns zero.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#setStackSize">setStackSize</a>().</p></div>
    pub unsafe fn stack_size(&self) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QThread_stackSize(self as *const crate::QThread)
    }

    /// <p>Begins execution of the thread by calling <a href="http://doc.qt.io/qt-5/qthread.html#run">run</a>(). The operating system will schedule the thread according to the <i>priority</i> parameter. If the thread is already running, this function does nothing.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QThread::start(QThread::Priority arg1 = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#start">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Begins execution of the thread by calling <a href="http://doc.qt.io/qt-5/qthread.html#run">run</a>(). The operating system will schedule the thread according to the <i>priority</i> parameter. If the thread is already running, this function does nothing.</p>
    /// <p>The effect of the <i>priority</i> parameter is dependent on the operating system's scheduling policy. In particular, the <i>priority</i> will be ignored on systems that do not support thread priorities (such as on Linux, see the <a href="http://linux.die.net/man/2/sched_setscheduler">sched_setscheduler</a> documentation for more details).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#run">run</a>() and <a href="http://doc.qt.io/qt-5/qthread.html#terminate">terminate</a>().</p></div>
    pub unsafe fn start_1a(&mut self, arg1: crate::q_thread::Priority) {
        crate::__ffi::ctr_qt_core_ffi_QThread_start(self as *mut crate::QThread, arg1)
    }

    /// <p>Begins execution of the thread by calling <a href="http://doc.qt.io/qt-5/qthread.html#run">run</a>(). The operating system will schedule the thread according to the <i>priority</i> parameter. If the thread is already running, this function does nothing.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QThread::start()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#start">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Begins execution of the thread by calling <a href="http://doc.qt.io/qt-5/qthread.html#run">run</a>(). The operating system will schedule the thread according to the <i>priority</i> parameter. If the thread is already running, this function does nothing.</p>
    /// <p>The effect of the <i>priority</i> parameter is dependent on the operating system's scheduling policy. In particular, the <i>priority</i> will be ignored on systems that do not support thread priorities (such as on Linux, see the <a href="http://linux.die.net/man/2/sched_setscheduler">sched_setscheduler</a> documentation for more details).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#run">run</a>() and <a href="http://doc.qt.io/qt-5/qthread.html#terminate">terminate</a>().</p></div>
    pub unsafe fn start_0a(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QThread_start1(self as *mut crate::QThread)
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QThread_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Terminates the execution of the thread. The thread may or may not be terminated immediately, depending on the operating system's scheduling policies. Use <a href="http://doc.qt.io/qt-5/qthread.html#wait">QThread::wait</a>() after terminate(), to be sure.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QThread::terminate()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#terminate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Terminates the execution of the thread. The thread may or may not be terminated immediately, depending on the operating system's scheduling policies. Use <a href="http://doc.qt.io/qt-5/qthread.html#wait">QThread::wait</a>() after terminate(), to be sure.</p>
    /// <p>When the thread is terminated, all threads waiting for the thread to finish will be woken up.</p>
    /// <p><b>Warning:</b> This function is dangerous and its use is discouraged. The thread can be terminated at any point in its code path. Threads can be terminated while modifying data. There is no chance for the thread to clean up after itself, unlock any held mutexes, etc. In short, use this function only if absolutely necessary.</p>
    /// <p>Termination can be explicitly enabled or disabled by calling <a href="http://doc.qt.io/qt-5/qthread.html#setTerminationEnabled">QThread::setTerminationEnabled</a>(). Calling this function while termination is disabled results in the termination being deferred, until termination is re-enabled. See the documentation of <a href="http://doc.qt.io/qt-5/qthread.html#setTerminationEnabled">QThread::setTerminationEnabled</a>() for more information.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#setTerminationEnabled">setTerminationEnabled</a>().</p></div>
    pub unsafe fn terminate(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QThread_terminate(self as *mut crate::QThread)
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QThread::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QThread_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QThread::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QThread_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Forces the current thread to sleep for <i>usecs</i> microseconds.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QThread::usleep(unsigned long arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#usleep">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Forces the current thread to sleep for <i>usecs</i> microseconds.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#sleep">sleep</a>() and <a href="http://doc.qt.io/qt-5/qthread.html#msleep">msleep</a>().</p></div>
    pub unsafe fn usleep(arg1: ::std::os::raw::c_ulong) {
        crate::__ffi::ctr_qt_core_ffi_QThread_usleep(arg1)
    }

    /// <p>Blocks the thread until either of these conditions is met:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QThread::wait(unsigned long time = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#wait">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Blocks the thread until either of these conditions is met:</p>
    /// <ul>
    /// <li>The thread associated with this <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a> object has finished execution (i.e. when it returns from <a href="http://doc.qt.io/qt-5/qthread.html#run">run</a>()). This function will return true if the thread has finished. It also returns true if the thread has not been started yet.</li>
    /// <li><i>time</i> milliseconds has elapsed. If <i>time</i> is ULONG_MAX (the default), then the wait will never timeout (the thread must return from <a href="http://doc.qt.io/qt-5/qthread.html#run">run</a>()). This function will return false if the wait timed out.</li>
    /// </ul>
    /// <p>This provides similar functionality to the POSIX <code>pthread_join()</code> function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#sleep">sleep</a>() and <a href="http://doc.qt.io/qt-5/qthread.html#terminate">terminate</a>().</p></div>
    pub unsafe fn wait_1a(&mut self, time: ::std::os::raw::c_ulong) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QThread_wait(self as *mut crate::QThread, time)
    }

    /// <p>Blocks the thread until either of these conditions is met:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QThread::wait()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#wait">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Blocks the thread until either of these conditions is met:</p>
    /// <ul>
    /// <li>The thread associated with this <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a> object has finished execution (i.e. when it returns from <a href="http://doc.qt.io/qt-5/qthread.html#run">run</a>()). This function will return true if the thread has finished. It also returns true if the thread has not been started yet.</li>
    /// <li><i>time</i> milliseconds has elapsed. If <i>time</i> is ULONG_MAX (the default), then the wait will never timeout (the thread must return from <a href="http://doc.qt.io/qt-5/qthread.html#run">run</a>()). This function will return false if the wait timed out.</li>
    /// </ul>
    /// <p>This provides similar functionality to the POSIX <code>pthread_join()</code> function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthread.html#sleep">sleep</a>() and <a href="http://doc.qt.io/qt-5/qthread.html#terminate">terminate</a>().</p></div>
    pub unsafe fn wait_0a(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QThread_wait1(self as *mut crate::QThread)
    }

    /// <p>Yields execution of the current thread to another runnable thread, if any. Note that the operating system decides to which thread to switch.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QThread::yieldCurrentThread()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#yieldCurrentThread">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Yields execution of the current thread to another runnable thread, if any. Note that the operating system decides to which thread to switch.</p></div>
    pub unsafe fn yield_current_thread() {
        crate::__ffi::ctr_qt_core_ffi_QThread_yieldCurrentThread()
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a> class manages a collection of QThreads.</p>
///
/// C++ class: <span style='color: green;'>```QThreadPool```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qthreadpool.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a> class manages a collection of QThreads.</p>
/// <p><a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a> manages and recyles individual <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a> objects to help reduce thread creation costs in programs that use threads. Each Qt application has one global <a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a> object, which can be accessed by calling <a href="http://doc.qt.io/qt-5/qthreadpool.html#globalInstance">globalInstance</a>().</p>
/// <p>To use one of the <a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a> threads, subclass <a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a> and implement the run() virtual function. Then create an object of that class and pass it to <a href="http://doc.qt.io/qt-5/qthreadpool.html#start">QThreadPool::start</a>().</p>
/// <pre class="cpp">
///
///   <span class="keyword">class</span> HelloWorldTask : <span class="keyword">public</span> <span class="type"><a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a></span>
///   {
/// &#32;     <span class="type">void</span> run()
/// &#32;     {
/// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Hello world from thread"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qthread.html">QThread</a></span><span class="operator">::</span>currentThread();
/// &#32;     }
///   };
///
///   HelloWorldTask <span class="operator">*</span>hello <span class="operator">=</span> <span class="keyword">new</span> HelloWorldTask();
///   <span class="comment">// QThreadPool takes ownership and deletes 'hello' automatically</span>
///   <span class="type"><a href="http://doc.qt.io/qt-5/qthreadpool.html#QThreadPool">QThreadPool</a></span><span class="operator">::</span>globalInstance()<span class="operator">-</span><span class="operator">&gt;</span>start(hello);
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a> deletes the <a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a> automatically by default. Use <a href="http://doc.qt.io/qt-5/qrunnable.html#setAutoDelete">QRunnable::setAutoDelete</a>() to change the auto-deletion flag.</p>
/// <p><a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a> supports executing the same <a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a> more than once by calling <a href="http://doc.qt.io/qt-5/qthreadpool.html#tryStart">tryStart</a>(this) from within <a href="http://doc.qt.io/qt-5/qrunnable.html#run">QRunnable::run</a>(). If autoDelete is enabled the <a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a> will be deleted when the last thread exits the run function. Calling <a href="http://doc.qt.io/qt-5/qthreadpool.html#start">start</a>() multiple times with the same <a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a> when autoDelete is enabled creates a race condition and is not recommended.</p>
/// <p>Threads that are unused for a certain amount of time will expire. The default expiry timeout is 30000 milliseconds (30 seconds). This can be changed using <a href="http://doc.qt.io/qt-5/qthreadpool.html#expiryTimeout-prop">setExpiryTimeout</a>(). Setting a negative expiry timeout disables the expiry mechanism.</p>
/// <p>Call <a href="http://doc.qt.io/qt-5/qthreadpool.html#maxThreadCount-prop">maxThreadCount</a>() to query the maximum number of threads to be used. If needed, you can change the limit with <a href="http://doc.qt.io/qt-5/qthreadpool.html#maxThreadCount-prop">setMaxThreadCount</a>(). The default <a href="http://doc.qt.io/qt-5/qthreadpool.html#maxThreadCount-prop">maxThreadCount</a>() is <a href="http://doc.qt.io/qt-5/qthread.html#idealThreadCount">QThread::idealThreadCount</a>(). The <a href="http://doc.qt.io/qt-5/qthreadpool.html#activeThreadCount-prop">activeThreadCount</a>() function returns the number of threads currently doing work.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qthreadpool.html#reserveThread">reserveThread</a>() function reserves a thread for external use. Use <a href="http://doc.qt.io/qt-5/qthreadpool.html#releaseThread">releaseThread</a>() when your are done with the thread, so that it may be reused. Essentially, these functions temporarily increase or reduce the active thread count and are useful when implementing time-consuming operations that are not visible to the <a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a>.</p>
/// <p>Note that <a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a> is a low-level class for managing threads, see the Qt Concurrent module for higher level alternatives.</p></div>
#[repr(C)]
pub struct QThreadPool {
    _unused: u8,
}
impl QThreadPool {
    /// <p>This property represents the number of active threads in the thread pool.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QThreadPool::activeThreadCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthreadpool.html#activeThreadCount-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property represents the number of active threads in the thread pool.</p>
    /// <p><b>Note: </b>It is possible for this function to return a value that is greater than <a href="http://doc.qt.io/qt-5/qthreadpool.html#maxThreadCount-prop">maxThreadCount</a>(). See <a href="http://doc.qt.io/qt-5/qthreadpool.html#reserveThread">reserveThread</a>() for more details.</p><p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>activeThreadCount</b></span>() const</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthreadpool.html#reserveThread">reserveThread</a>() and <a href="http://doc.qt.io/qt-5/qthreadpool.html#releaseThread">releaseThread</a>().</p></div>
    pub unsafe fn active_thread_count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QThreadPool_activeThreadCount(
            self as *const crate::QThreadPool,
        )
    }

    /// <p>Removes the specified <i>runnable</i> from the queue if it is not yet started. The runnables for which <a href="http://doc.qt.io/qt-5/qrunnable.html#autoDelete">runnable-&gt;autoDelete()</a> returns <code>true</code> are deleted.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QThreadPool::cancel(QRunnable* runnable)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthreadpool-obsolete.html#cancel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the specified <i>runnable</i> from the queue if it is not yet started. The runnables for which <a href="http://doc.qt.io/qt-5/qrunnable.html#autoDelete">runnable-&gt;autoDelete()</a> returns <code>true</code> are deleted.</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthreadpool.html#start">start</a>() and <a href="http://doc.qt.io/qt-5/qthreadpool.html#tryTake">tryTake</a>().</p></div>
    pub unsafe fn cancel(
        &mut self,
        runnable: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QRunnable>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QThreadPool_cancel(
            self as *mut crate::QThreadPool,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QRunnable>>::cast_into(runnable)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Removes the runnables that are not yet started from the queue. The runnables for which <a href="http://doc.qt.io/qt-5/qrunnable.html#autoDelete">runnable-&gt;autoDelete()</a> returns <code>true</code> are deleted.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QThreadPool::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthreadpool.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the runnables that are not yet started from the queue. The runnables for which <a href="http://doc.qt.io/qt-5/qrunnable.html#autoDelete">runnable-&gt;autoDelete()</a> returns <code>true</code> are deleted.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthreadpool.html#start">start</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QThreadPool_clear(self as *mut crate::QThreadPool)
    }

    /// <p>Threads that are unused for <i>expiryTimeout</i> milliseconds are considered to have expired and will exit. Such threads will be restarted as needed. The default <i>expiryTimeout</i> is 30000 milliseconds (30 seconds). If <i>expiryTimeout</i> is negative, newly created threads will not expire, e.g., they will not exit until the thread pool is destroyed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QThreadPool::expiryTimeout() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthreadpool.html#expiryTimeout-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Threads that are unused for <i>expiryTimeout</i> milliseconds are considered to have expired and will exit. Such threads will be restarted as needed. The default <i>expiryTimeout</i> is 30000 milliseconds (30 seconds). If <i>expiryTimeout</i> is negative, newly created threads will not expire, e.g., they will not exit until the thread pool is destroyed.</p>
    /// <p>Note that setting <i>expiryTimeout</i> has no effect on already running threads. Only newly created threads will use the new <i>expiryTimeout</i>. We recommend setting the <i>expiryTimeout</i> immediately after creating the thread pool, but before calling <a href="http://doc.qt.io/qt-5/qthreadpool.html#start">start</a>().</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>expiryTimeout</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setExpiryTimeout</b></span>(int <i>expiryTimeout</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn expiry_timeout(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QThreadPool_expiryTimeout(self as *const crate::QThreadPool)
    }

    /// <p>Returns the global <a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a> instance.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QThreadPool* QThreadPool::globalInstance()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthreadpool.html#globalInstance">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the global <a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a> instance.</p></div>
    pub unsafe fn global_instance() -> ::cpp_utils::MutPtr<crate::QThreadPool> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QThreadPool_globalInstance();
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This property represents the maximum number of threads used by the thread pool.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QThreadPool::maxThreadCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthreadpool.html#maxThreadCount-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property represents the maximum number of threads used by the thread pool.</p>
    /// <p><b>Note: </b>The thread pool will always use at least 1 thread, even if <i>maxThreadCount</i> limit is zero or negative.</p><p>The default <i>maxThreadCount</i> is <a href="http://doc.qt.io/qt-5/qthread.html#idealThreadCount">QThread::idealThreadCount</a>().</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>maxThreadCount</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setMaxThreadCount</b></span>(int <i>maxThreadCount</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn max_thread_count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QThreadPool_maxThreadCount(self as *const crate::QThreadPool)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QThreadPool::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QThreadPool_metaObject(self as *const crate::QThreadPool);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs a thread pool with the given <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QThreadPool::QThreadPool(QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthreadpool.html#QThreadPool">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a thread pool with the given <i>parent</i>.</p></div>
    pub unsafe fn new_1a(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QThreadPool> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QThreadPool_QThreadPool(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a> class manages a collection of QThreads.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QThreadPool::QThreadPool()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthreadpool.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a> class manages a collection of QThreads.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a> manages and recyles individual <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a> objects to help reduce thread creation costs in programs that use threads. Each Qt application has one global <a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a> object, which can be accessed by calling <a href="http://doc.qt.io/qt-5/qthreadpool.html#globalInstance">globalInstance</a>().</p>
    /// <p>To use one of the <a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a> threads, subclass <a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a> and implement the run() virtual function. Then create an object of that class and pass it to <a href="http://doc.qt.io/qt-5/qthreadpool.html#start">QThreadPool::start</a>().</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">class</span> HelloWorldTask : <span class="keyword">public</span> <span class="type"><a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a></span>
    ///   {
    /// &#32;     <span class="type">void</span> run()
    /// &#32;     {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Hello world from thread"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qthread.html">QThread</a></span><span class="operator">::</span>currentThread();
    /// &#32;     }
    ///   };
    ///
    ///   HelloWorldTask <span class="operator">*</span>hello <span class="operator">=</span> <span class="keyword">new</span> HelloWorldTask();
    ///   <span class="comment">// QThreadPool takes ownership and deletes 'hello' automatically</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qthreadpool.html#QThreadPool">QThreadPool</a></span><span class="operator">::</span>globalInstance()<span class="operator">-</span><span class="operator">&gt;</span>start(hello);
    ///
    /// </pre>
    /// <p><a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a> deletes the <a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a> automatically by default. Use <a href="http://doc.qt.io/qt-5/qrunnable.html#setAutoDelete">QRunnable::setAutoDelete</a>() to change the auto-deletion flag.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a> supports executing the same <a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a> more than once by calling <a href="http://doc.qt.io/qt-5/qthreadpool.html#tryStart">tryStart</a>(this) from within <a href="http://doc.qt.io/qt-5/qrunnable.html#run">QRunnable::run</a>(). If autoDelete is enabled the <a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a> will be deleted when the last thread exits the run function. Calling <a href="http://doc.qt.io/qt-5/qthreadpool.html#start">start</a>() multiple times with the same <a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a> when autoDelete is enabled creates a race condition and is not recommended.</p>
    /// <p>Threads that are unused for a certain amount of time will expire. The default expiry timeout is 30000 milliseconds (30 seconds). This can be changed using <a href="http://doc.qt.io/qt-5/qthreadpool.html#expiryTimeout-prop">setExpiryTimeout</a>(). Setting a negative expiry timeout disables the expiry mechanism.</p>
    /// <p>Call <a href="http://doc.qt.io/qt-5/qthreadpool.html#maxThreadCount-prop">maxThreadCount</a>() to query the maximum number of threads to be used. If needed, you can change the limit with <a href="http://doc.qt.io/qt-5/qthreadpool.html#maxThreadCount-prop">setMaxThreadCount</a>(). The default <a href="http://doc.qt.io/qt-5/qthreadpool.html#maxThreadCount-prop">maxThreadCount</a>() is <a href="http://doc.qt.io/qt-5/qthread.html#idealThreadCount">QThread::idealThreadCount</a>(). The <a href="http://doc.qt.io/qt-5/qthreadpool.html#activeThreadCount-prop">activeThreadCount</a>() function returns the number of threads currently doing work.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qthreadpool.html#reserveThread">reserveThread</a>() function reserves a thread for external use. Use <a href="http://doc.qt.io/qt-5/qthreadpool.html#releaseThread">releaseThread</a>() when your are done with the thread, so that it may be reused. Essentially, these functions temporarily increase or reduce the active thread count and are useful when implementing time-consuming operations that are not visible to the <a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a> is a low-level class for managing threads, see the Qt Concurrent module for higher level alternatives.</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QThreadPool> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QThreadPool_QThreadPool1();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QThreadPool::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QThreadPool_qt_metacall(
            self as *mut crate::QThreadPool,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QThreadPool::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QThreadPool_qt_metacast(
            self as *mut crate::QThreadPool,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Releases a thread previously reserved by a call to <a href="http://doc.qt.io/qt-5/qthreadpool.html#reserveThread">reserveThread</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QThreadPool::releaseThread()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthreadpool.html#releaseThread">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Releases a thread previously reserved by a call to <a href="http://doc.qt.io/qt-5/qthreadpool.html#reserveThread">reserveThread</a>().</p>
    /// <p><b>Note: </b>Calling this function without previously reserving a thread temporarily increases <a href="http://doc.qt.io/qt-5/qthreadpool.html#maxThreadCount-prop">maxThreadCount</a>(). This is useful when a thread goes to sleep waiting for more work, allowing other threads to continue. Be sure to call <a href="http://doc.qt.io/qt-5/qthreadpool.html#reserveThread">reserveThread</a>() when done waiting, so that the thread pool can correctly maintain the <a href="http://doc.qt.io/qt-5/qthreadpool.html#activeThreadCount-prop">activeThreadCount</a>().</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qthreadpool.html#reserveThread">reserveThread</a>().</p></div>
    pub unsafe fn release_thread(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QThreadPool_releaseThread(self as *mut crate::QThreadPool)
    }

    /// <p>Reserves one thread, disregarding <a href="http://doc.qt.io/qt-5/qthreadpool.html#activeThreadCount-prop">activeThreadCount</a>() and <a href="http://doc.qt.io/qt-5/qthreadpool.html#maxThreadCount-prop">maxThreadCount</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QThreadPool::reserveThread()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthreadpool.html#reserveThread">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reserves one thread, disregarding <a href="http://doc.qt.io/qt-5/qthreadpool.html#activeThreadCount-prop">activeThreadCount</a>() and <a href="http://doc.qt.io/qt-5/qthreadpool.html#maxThreadCount-prop">maxThreadCount</a>().</p>
    /// <p>Once you are done with the thread, call <a href="http://doc.qt.io/qt-5/qthreadpool.html#releaseThread">releaseThread</a>() to allow it to be reused.</p>
    /// <p><b>Note: </b>This function will always increase the number of active threads. This means that by using this function, it is possible for <a href="http://doc.qt.io/qt-5/qthreadpool.html#activeThreadCount-prop">activeThreadCount</a>() to return a value greater than <a href="http://doc.qt.io/qt-5/qthreadpool.html#maxThreadCount-prop">maxThreadCount</a>() .</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qthreadpool.html#releaseThread">releaseThread</a>().</p></div>
    pub unsafe fn reserve_thread(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QThreadPool_reserveThread(self as *mut crate::QThreadPool)
    }

    /// <p>Threads that are unused for <i>expiryTimeout</i> milliseconds are considered to have expired and will exit. Such threads will be restarted as needed. The default <i>expiryTimeout</i> is 30000 milliseconds (30 seconds). If <i>expiryTimeout</i> is negative, newly created threads will not expire, e.g., they will not exit until the thread pool is destroyed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QThreadPool::setExpiryTimeout(int expiryTimeout)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthreadpool.html#expiryTimeout-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Threads that are unused for <i>expiryTimeout</i> milliseconds are considered to have expired and will exit. Such threads will be restarted as needed. The default <i>expiryTimeout</i> is 30000 milliseconds (30 seconds). If <i>expiryTimeout</i> is negative, newly created threads will not expire, e.g., they will not exit until the thread pool is destroyed.</p>
    /// <p>Note that setting <i>expiryTimeout</i> has no effect on already running threads. Only newly created threads will use the new <i>expiryTimeout</i>. We recommend setting the <i>expiryTimeout</i> immediately after creating the thread pool, but before calling <a href="http://doc.qt.io/qt-5/qthreadpool.html#start">start</a>().</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>expiryTimeout</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setExpiryTimeout</b></span>(int <i>expiryTimeout</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn set_expiry_timeout(&mut self, expiry_timeout: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QThreadPool_setExpiryTimeout(
            self as *mut crate::QThreadPool,
            expiry_timeout,
        )
    }

    /// <p>This property represents the maximum number of threads used by the thread pool.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QThreadPool::setMaxThreadCount(int maxThreadCount)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthreadpool.html#maxThreadCount-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property represents the maximum number of threads used by the thread pool.</p>
    /// <p><b>Note: </b>The thread pool will always use at least 1 thread, even if <i>maxThreadCount</i> limit is zero or negative.</p><p>The default <i>maxThreadCount</i> is <a href="http://doc.qt.io/qt-5/qthread.html#idealThreadCount">QThread::idealThreadCount</a>().</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>maxThreadCount</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setMaxThreadCount</b></span>(int <i>maxThreadCount</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn set_max_thread_count(&mut self, max_thread_count: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QThreadPool_setMaxThreadCount(
            self as *mut crate::QThreadPool,
            max_thread_count,
        )
    }

    /// <p>This property contains the stack size for the thread pool worker threads.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QThreadPool::setStackSize(unsigned int stackSize)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthreadpool.html#stackSize-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property contains the stack size for the thread pool worker threads.</p>
    /// <p>The value of the property is only used when the thread pool creates new threads. Changing it has no effect for already created or running threads.</p>
    /// <p>The default value is 0, which makes <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a> use the operating system default stack size.</p>
    /// <p>This property was introduced in  Qt 5.10.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> uint </td><td class="memItemRight bottomAlign"><span class="name"><b>stackSize</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setStackSize</b></span>(uint <i>stackSize</i>)</td></tr>
    /// </tbody></table></div></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn set_stack_size(&mut self, stack_size: ::std::os::raw::c_uint) {
        crate::__ffi::ctr_qt_core_ffi_QThreadPool_setStackSize(
            self as *mut crate::QThreadPool,
            stack_size,
        )
    }

    /// <p>This property contains the stack size for the thread pool worker threads.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QThreadPool::stackSize() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthreadpool.html#stackSize-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property contains the stack size for the thread pool worker threads.</p>
    /// <p>The value of the property is only used when the thread pool creates new threads. Changing it has no effect for already created or running threads.</p>
    /// <p>The default value is 0, which makes <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a> use the operating system default stack size.</p>
    /// <p>This property was introduced in  Qt 5.10.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> uint </td><td class="memItemRight bottomAlign"><span class="name"><b>stackSize</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setStackSize</b></span>(uint <i>stackSize</i>)</td></tr>
    /// </tbody></table></div></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn stack_size(&self) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QThreadPool_stackSize(self as *const crate::QThreadPool)
    }

    /// <p>Reserves a thread and uses it to run <i>runnable</i>, unless this thread will make the current thread count exceed <a href="http://doc.qt.io/qt-5/qthreadpool.html#maxThreadCount-prop">maxThreadCount</a>(). In that case, <i>runnable</i> is added to a run queue instead. The <i>priority</i> argument can be used to control the run queue's order of execution.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QThreadPool::start(QRunnable* runnable, int priority = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthreadpool.html#start">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reserves a thread and uses it to run <i>runnable</i>, unless this thread will make the current thread count exceed <a href="http://doc.qt.io/qt-5/qthreadpool.html#maxThreadCount-prop">maxThreadCount</a>(). In that case, <i>runnable</i> is added to a run queue instead. The <i>priority</i> argument can be used to control the run queue's order of execution.</p>
    /// <p>Note that the thread pool takes ownership of the <i>runnable</i> if <a href="http://doc.qt.io/qt-5/qrunnable.html#autoDelete">runnable-&gt;autoDelete()</a> returns <code>true</code>, and the <i>runnable</i> will be deleted automatically by the thread pool after the <a href="http://doc.qt.io/qt-5/qrunnable.html#run">runnable-&gt;run()</a> returns. If <a href="http://doc.qt.io/qt-5/qrunnable.html#autoDelete">runnable-&gt;autoDelete()</a> returns <code>false</code>, ownership of <i>runnable</i> remains with the caller. Note that changing the auto-deletion on <i>runnable</i> after calling this functions results in undefined behavior.</p></div>
    pub unsafe fn start_2a(
        &mut self,
        runnable: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QRunnable>>,
        priority: ::std::os::raw::c_int,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QThreadPool_start(
            self as *mut crate::QThreadPool,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QRunnable>>::cast_into(runnable)
                .as_mut_raw_ptr(),
            priority,
        )
    }

    /// <p>Reserves a thread and uses it to run <i>runnable</i>, unless this thread will make the current thread count exceed <a href="http://doc.qt.io/qt-5/qthreadpool.html#maxThreadCount-prop">maxThreadCount</a>(). In that case, <i>runnable</i> is added to a run queue instead. The <i>priority</i> argument can be used to control the run queue's order of execution.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QThreadPool::start(QRunnable* runnable)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthreadpool.html#start">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reserves a thread and uses it to run <i>runnable</i>, unless this thread will make the current thread count exceed <a href="http://doc.qt.io/qt-5/qthreadpool.html#maxThreadCount-prop">maxThreadCount</a>(). In that case, <i>runnable</i> is added to a run queue instead. The <i>priority</i> argument can be used to control the run queue's order of execution.</p>
    /// <p>Note that the thread pool takes ownership of the <i>runnable</i> if <a href="http://doc.qt.io/qt-5/qrunnable.html#autoDelete">runnable-&gt;autoDelete()</a> returns <code>true</code>, and the <i>runnable</i> will be deleted automatically by the thread pool after the <a href="http://doc.qt.io/qt-5/qrunnable.html#run">runnable-&gt;run()</a> returns. If <a href="http://doc.qt.io/qt-5/qrunnable.html#autoDelete">runnable-&gt;autoDelete()</a> returns <code>false</code>, ownership of <i>runnable</i> remains with the caller. Note that changing the auto-deletion on <i>runnable</i> after calling this functions results in undefined behavior.</p></div>
    pub unsafe fn start_1a(
        &mut self,
        runnable: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QRunnable>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QThreadPool_start1(
            self as *mut crate::QThreadPool,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QRunnable>>::cast_into(runnable)
                .as_mut_raw_ptr(),
        )
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QThreadPool_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QThreadPool::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QThreadPool_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QThreadPool::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QThreadPool_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Attempts to reserve a thread to run <i>runnable</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QThreadPool::tryStart(QRunnable* runnable)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthreadpool.html#tryStart">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to reserve a thread to run <i>runnable</i>.</p>
    /// <p>If no threads are available at the time of calling, then this function does nothing and returns <code>false</code>. Otherwise, <i>runnable</i> is run immediately using one available thread and this function returns <code>true</code>.</p>
    /// <p>Note that the thread pool takes ownership of the <i>runnable</i> if <a href="http://doc.qt.io/qt-5/qrunnable.html#autoDelete">runnable-&gt;autoDelete()</a> returns <code>true</code>, and the <i>runnable</i> will be deleted automatically by the thread pool after the <a href="http://doc.qt.io/qt-5/qrunnable.html#run">runnable-&gt;run()</a> returns. If <a href="http://doc.qt.io/qt-5/qrunnable.html#autoDelete">runnable-&gt;autoDelete()</a> returns <code>false</code>, ownership of <i>runnable</i> remains with the caller. Note that changing the auto-deletion on <i>runnable</i> after calling this function results in undefined behavior.</p></div>
    pub unsafe fn try_start(
        &mut self,
        runnable: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QRunnable>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QThreadPool_tryStart(
            self as *mut crate::QThreadPool,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QRunnable>>::cast_into(runnable)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Attempts to remove the specified <i>runnable</i> from the queue if it is not yet started. If the runnable had not been started, returns <code>true</code>, and ownership of <i>runnable</i> is transferred to the caller (even when <code>runnable-&gt;autoDelete() == true</code>). Otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QThreadPool::tryTake(QRunnable* runnable)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthreadpool.html#tryTake">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to remove the specified <i>runnable</i> from the queue if it is not yet started. If the runnable had not been started, returns <code>true</code>, and ownership of <i>runnable</i> is transferred to the caller (even when <code>runnable-&gt;autoDelete() == true</code>). Otherwise returns <code>false</code>.</p>
    /// <p><b>Note: </b>If <code>runnable-&gt;autoDelete() == true</code>, this function may remove the wrong runnable. This is known as the <a href="https://en.wikipedia.org/wiki/ABA_problem">ABA problem</a>: the original <i>runnable</i> may already have executed and has since been deleted. The memory is re-used for another runnable, which then gets removed instead of the intended one. For this reason, we recommend calling this function only for runnables that are not auto-deleting.</p><p>This function was introduced in  Qt 5.9.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qthreadpool.html#start">start</a>() and <a href="http://doc.qt.io/qt-5/qrunnable.html#autoDelete">QRunnable::autoDelete</a>().</p></div>
    pub unsafe fn try_take(
        &mut self,
        runnable: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QRunnable>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QThreadPool_tryTake(
            self as *mut crate::QThreadPool,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QRunnable>>::cast_into(runnable)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Waits up to <i>msecs</i> milliseconds for all threads to exit and removes all threads from the thread pool. Returns <code>true</code> if all threads were removed; otherwise it returns <code>false</code>. If <i>msecs</i> is -1 (the default), the timeout is ignored (waits for the last thread to exit).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QThreadPool::waitForDone(int msecs = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthreadpool.html#waitForDone">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Waits up to <i>msecs</i> milliseconds for all threads to exit and removes all threads from the thread pool. Returns <code>true</code> if all threads were removed; otherwise it returns <code>false</code>. If <i>msecs</i> is -1 (the default), the timeout is ignored (waits for the last thread to exit).</p></div>
    pub unsafe fn wait_for_done_1a(&mut self, msecs: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QThreadPool_waitForDone(
            self as *mut crate::QThreadPool,
            msecs,
        )
    }

    /// <p>Waits up to <i>msecs</i> milliseconds for all threads to exit and removes all threads from the thread pool. Returns <code>true</code> if all threads were removed; otherwise it returns <code>false</code>. If <i>msecs</i> is -1 (the default), the timeout is ignored (waits for the last thread to exit).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QThreadPool::waitForDone()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthreadpool.html#waitForDone">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Waits up to <i>msecs</i> milliseconds for all threads to exit and removes all threads from the thread pool. Returns <code>true</code> if all threads were removed; otherwise it returns <code>false</code>. If <i>msecs</i> is -1 (the default), the timeout is ignored (waits for the last thread to exit).</p></div>
    pub unsafe fn wait_for_done_0a(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QThreadPool_waitForDone1(self as *mut crate::QThreadPool)
    }
}

pub mod q_time_line {
    //! C++ type: <span style='color: green;'>```QTimeLine```</span>

    /// <p>This enum describes the state of the timeline.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QTimeLine::State```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#State-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the state of the timeline.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimeline.html#state">state</a>() and <a href="http://doc.qt.io/qt-5/qtimeline.html#stateChanged">stateChanged</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct State(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for State {
        fn from(value: ::std::os::raw::c_int) -> Self {
            State(value)
        }
    }

    impl From<State> for ::std::os::raw::c_int {
        fn from(value: State) -> Self {
            value.0
        }
    }

    impl State {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl State {
        /// The timeline is not running. This is the initial state of <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a>, and the state <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> reenters when finished. The current time, frame and value remain unchanged until either <a href="http://doc.qt.io/qt-5/qtimeline.html#currentTime-prop">setCurrentTime</a>() is called, or the timeline is started by calling <a href="http://doc.qt.io/qt-5/qtimeline.html#start">start</a>(). (C++ enum variant: <span style='color: green;'>```NotRunning = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NotRunning: crate::q_time_line::State = crate::q_time_line::State(0);
        /// The timeline is paused (i.e., temporarily suspended). Calling <a href="http://doc.qt.io/qt-5/qtimeline.html#setPaused">setPaused</a>(false) will resume timeline activity. (C++ enum variant: <span style='color: green;'>```Paused = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Paused: crate::q_time_line::State = crate::q_time_line::State(1);
        /// The timeline is running. While control is in the event loop, <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> will update its current time at regular intervals, emitting <a href="http://doc.qt.io/qt-5/qtimeline.html#valueChanged">valueChanged</a>() and <a href="http://doc.qt.io/qt-5/qtimeline.html#frameChanged">frameChanged</a>() when appropriate. (C++ enum variant: <span style='color: green;'>```Running = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const Running: crate::q_time_line::State = crate::q_time_line::State(2);
    }

    /// <p>This enum describes the direction of the timeline when in <a href="http://doc.qt.io/qt-5/qtimeline.html#State-enum">Running</a> state.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QTimeLine::Direction```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#Direction-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the direction of the timeline when in <a href="http://doc.qt.io/qt-5/qtimeline.html#State-enum">Running</a> state.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimeline.html#direction-prop">setDirection</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Direction(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Direction {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Direction(value)
        }
    }

    impl From<Direction> for ::std::os::raw::c_int {
        fn from(value: Direction) -> Self {
            value.0
        }
    }

    impl Direction {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Direction {
        /// The current time of the timeline increases with time (i.e., moves from 0 and towards the end / duration). (C++ enum variant: <span style='color: green;'>```Forward = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const Forward: crate::q_time_line::Direction = crate::q_time_line::Direction(0);
        /// The current time of the timeline decreases with time (i.e., moves from the end / duration and towards 0). (C++ enum variant: <span style='color: green;'>```Backward = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Backward: crate::q_time_line::Direction = crate::q_time_line::Direction(1);
    }

    /// <p>This enum describes the default shape of <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a>'s value curve. The default, shape is EaseInOutCurve. The curve defines the relation between the value and the timeline.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QTimeLine::CurveShape```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#CurveShape-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum describes the default shape of <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a>'s value curve. The default, shape is EaseInOutCurve. The curve defines the relation between the value and the timeline.</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimeline.html#curveShape-prop">setCurveShape</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct CurveShape(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for CurveShape {
        fn from(value: ::std::os::raw::c_int) -> Self {
            CurveShape(value)
        }
    }

    impl From<CurveShape> for ::std::os::raw::c_int {
        fn from(value: CurveShape) -> Self {
            value.0
        }
    }

    impl CurveShape {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl CurveShape {
        /// The value starts growing slowly, then increases in speed. (C++ enum variant: <span style='color: green;'>```EaseInCurve = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const EaseInCurve: crate::q_time_line::CurveShape = crate::q_time_line::CurveShape(0);
        /// The value starts growing steadily, then ends slowly. (C++ enum variant: <span style='color: green;'>```EaseOutCurve = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const EaseOutCurve: crate::q_time_line::CurveShape = crate::q_time_line::CurveShape(1);
        /// The value starts growing slowly, then runs steadily, then grows slowly again. (C++ enum variant: <span style='color: green;'>```EaseInOutCurve = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const EaseInOutCurve: crate::q_time_line::CurveShape =
            crate::q_time_line::CurveShape(2);
        /// The value grows linearly (e.g., if the duration is 1000 ms, the value at time 500 ms is 0.5). (C++ enum variant: <span style='color: green;'>```LinearCurve = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const LinearCurve: crate::q_time_line::CurveShape = crate::q_time_line::CurveShape(3);
        /// The value grows sinusoidally. (C++ enum variant: <span style='color: green;'>```SineCurve = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const SineCurve: crate::q_time_line::CurveShape = crate::q_time_line::CurveShape(4);
        /// The value grows cosinusoidally. (C++ enum variant: <span style='color: green;'>```CosineCurve = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const CosineCurve: crate::q_time_line::CurveShape = crate::q_time_line::CurveShape(5);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> class provides a timeline for controlling animations.</p>
///
/// C++ class: <span style='color: green;'>```QTimeLine```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtimeline.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> class provides a timeline for controlling animations.</p>
/// <p>It's most commonly used to animate a GUI control by calling a slot periodically. You can construct a timeline by passing its duration in milliseconds to <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a>'s constructor. The timeline's duration describes for how long the animation will run. Then you set a suitable frame range by calling <a href="http://doc.qt.io/qt-5/qtimeline.html#setFrameRange">setFrameRange</a>(). Finally connect the <a href="http://doc.qt.io/qt-5/qtimeline.html#frameChanged">frameChanged</a>() signal to a suitable slot in the widget you wish to animate (for example, <a href="http://doc.qt.io/qt-5/qprogressbar.html#value-prop">setValue()</a> in <a href="http://doc.qt.io/qt-5/qprogressbar.html">QProgressBar</a>). When you proceed to calling <a href="http://doc.qt.io/qt-5/qtimeline.html#start">start</a>(), <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> will enter Running state, and start emitting <a href="http://doc.qt.io/qt-5/qtimeline.html#frameChanged">frameChanged</a>() at regular intervals, causing your widget's connected property's value to grow from the lower end to the upper and of your frame range, at a steady rate. You can specify the update interval by calling <a href="http://doc.qt.io/qt-5/qtimeline.html#updateInterval-prop">setUpdateInterval</a>(). When done, <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> enters <a href="http://doc.qt.io/qt-5/qtimeline.html#State-enum">NotRunning</a> state, and emits <a href="http://doc.qt.io/qt-5/qtimeline.html#finished">finished</a>().</p>
/// <p>Example:</p>
/// <pre class="cpp">
///
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   progressBar <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qprogressbar.html">QProgressBar</a></span>(<span class="keyword">this</span>);
///   progressBar<span class="operator">-</span><span class="operator">&gt;</span>setRange(<span class="number">0</span><span class="operator">,</span> <span class="number">100</span>);
///
///   <span class="comment">// Construct a 1-second timeline with a frame range of 0 - 100</span>
///   <span class="type"><a href="http://doc.qt.io/qt-5/qtimeline.html#QTimeLine">QTimeLine</a></span> <span class="operator">*</span>timeLine <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtimeline.html#QTimeLine">QTimeLine</a></span>(<span class="number">1000</span><span class="operator">,</span> <span class="keyword">this</span>);
///   timeLine<span class="operator">-</span><span class="operator">&gt;</span>setFrameRange(<span class="number">0</span><span class="operator">,</span> <span class="number">100</span>);
///   connect(timeLine<span class="operator">,</span> SIGNAL(frameChanged(<span class="type">int</span>))<span class="operator">,</span> progressBar<span class="operator">,</span> SLOT(setValue(<span class="type">int</span>)));
///
///   <span class="comment">// Clicking the push button will start the progress bar animation</span>
///   pushButton <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span>(tr(<span class="string">"Start animation"</span>)<span class="operator">,</span> <span class="keyword">this</span>);
///   connect(pushButton<span class="operator">,</span> SIGNAL(clicked())<span class="operator">,</span> timeLine<span class="operator">,</span> SLOT(start()));
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///
/// </pre>
/// <p>By default the timeline runs once, from the beginning and towards the end, upon which you must call <a href="http://doc.qt.io/qt-5/qtimeline.html#start">start</a>() again to restart from the beginning. To make the timeline loop, you can call <a href="http://doc.qt.io/qt-5/qtimeline.html#loopCount-prop">setLoopCount</a>(), passing the number of times the timeline should run before finishing. The direction can also be changed, causing the timeline to run backward, by calling <a href="http://doc.qt.io/qt-5/qtimeline.html#direction-prop">setDirection</a>(). You can also pause and unpause the timeline while it's running by calling <a href="http://doc.qt.io/qt-5/qtimeline.html#setPaused">setPaused</a>(). For interactive control, the <a href="http://doc.qt.io/qt-5/qtimeline.html#currentTime-prop">setCurrentTime</a>() function is provided, which sets the time position of the time line directly. Although most useful in <a href="http://doc.qt.io/qt-5/qtimeline.html#State-enum">NotRunning</a> state, (e.g., connected to a <a href="http://doc.qt.io/qt-5/qtimeline.html#valueChanged">valueChanged</a>() signal in a <a href="http://doc.qt.io/qt-5/qslider.html">QSlider</a>,) this function can be called at any time.</p>
/// <p>The frame interface is useful for standard widgets, but <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> can be used to control any type of animation. The heart of <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> lies in the <a href="http://doc.qt.io/qt-5/qtimeline.html#valueForTime">valueForTime</a>() function, which generates a <i>value</i> between 0 and 1 for a given time. This value is typically used to describe the steps of an animation, where 0 is the first step of an animation, and 1 is the last step. When running, <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> generates values between 0 and 1 by calling <a href="http://doc.qt.io/qt-5/qtimeline.html#valueForTime">valueForTime</a>() and emitting <a href="http://doc.qt.io/qt-5/qtimeline.html#valueChanged">valueChanged</a>(). By default, <a href="http://doc.qt.io/qt-5/qtimeline.html#valueForTime">valueForTime</a>() applies an interpolation algorithm to generate these value. You can choose from a set of predefined timeline algorithms by calling <a href="http://doc.qt.io/qt-5/qtimeline.html#curveShape-prop">setCurveShape</a>().</p>
/// <p>Note that by default, <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> uses the EaseInOut curve shape, which provides a value that grows slowly, then grows steadily, and finally grows slowly. For a custom timeline, you can reimplement <a href="http://doc.qt.io/qt-5/qtimeline.html#valueForTime">valueForTime</a>(), in which case <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a>'s <a href="http://doc.qt.io/qt-5/qtimeline.html#curveShape-prop">curveShape</a> property is ignored.</p></div>
#[repr(C)]
pub struct QTimeLine {
    _unused: u8,
}
impl QTimeLine {
    /// <p>Starts the timeline. <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> will enter Running state, and once it enters the event loop, it will update its current time, frame and value at regular intervals. The default interval is 40 ms (i.e., 25 times per second). You can change the update interval by calling <a href="http://doc.qt.io/qt-5/qtimeline.html#updateInterval-prop">setUpdateInterval</a>().</p>
    ///
    /// Returns a built-in Qt slot `QTimeLine::start` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#start">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Starts the timeline. <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> will enter Running state, and once it enters the event loop, it will update its current time, frame and value at regular intervals. The default interval is 40 ms (i.e., 25 times per second). You can change the update interval by calling <a href="http://doc.qt.io/qt-5/qtimeline.html#updateInterval-prop">setUpdateInterval</a>().</p>
    /// <p>The timeline will start from position 0, or the end if going backward. If you want to resume a stopped timeline without restarting, you can call <a href="http://doc.qt.io/qt-5/qtimeline.html#resume">resume</a>() instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimeline.html#resume">resume</a>(), <a href="http://doc.qt.io/qt-5/qtimeline.html#updateInterval-prop">updateInterval</a>(), <a href="http://doc.qt.io/qt-5/qtimeline.html#frameChanged">frameChanged</a>(), and <a href="http://doc.qt.io/qt-5/qtimeline.html#valueChanged">valueChanged</a>().</p></div>
    pub fn slot_start(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1start()\0"),
            )
        }
    }

    /// <p>Resumes the timeline from the current time. <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> will reenter Running state, and once it enters the event loop, it will update its current time, frame and value at regular intervals.</p>
    ///
    /// Returns a built-in Qt slot `QTimeLine::resume` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#resume">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Resumes the timeline from the current time. <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> will reenter Running state, and once it enters the event loop, it will update its current time, frame and value at regular intervals.</p>
    /// <p>In contrast to <a href="http://doc.qt.io/qt-5/qtimeline.html#start">start</a>(), this function does not restart the timeline before it resumes.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimeline.html#start">start</a>(), <a href="http://doc.qt.io/qt-5/qtimeline.html#updateInterval-prop">updateInterval</a>(), <a href="http://doc.qt.io/qt-5/qtimeline.html#frameChanged">frameChanged</a>(), and <a href="http://doc.qt.io/qt-5/qtimeline.html#valueChanged">valueChanged</a>().</p></div>
    pub fn slot_resume(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1resume()\0"),
            )
        }
    }

    /// <p>Stops the timeline, causing <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> to enter <a href="http://doc.qt.io/qt-5/qtimeline.html#State-enum">NotRunning</a> state.</p>
    ///
    /// Returns a built-in Qt slot `QTimeLine::stop` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#stop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Stops the timeline, causing <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> to enter <a href="http://doc.qt.io/qt-5/qtimeline.html#State-enum">NotRunning</a> state.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimeline.html#start">start</a>().</p></div>
    pub fn slot_stop(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1stop()\0"),
            )
        }
    }

    /// <p>If <i>paused</i> is true, the timeline is paused, causing <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> to enter Paused state. No updates will be signaled until either <a href="http://doc.qt.io/qt-5/qtimeline.html#start">start</a>() or setPaused(false) is called. If <i>paused</i> is false, the timeline is resumed and continues where it left.</p>
    ///
    /// Returns a built-in Qt slot `QTimeLine::setPaused` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#setPaused">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If <i>paused</i> is true, the timeline is paused, causing <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> to enter Paused state. No updates will be signaled until either <a href="http://doc.qt.io/qt-5/qtimeline.html#start">start</a>() or setPaused(false) is called. If <i>paused</i> is false, the timeline is resumed and continues where it left.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimeline.html#state">state</a>() and <a href="http://doc.qt.io/qt-5/qtimeline.html#start">start</a>().</p></div>
    pub fn slot_set_paused(&self) -> crate::Receiver<(bool,)> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1setPaused(bool)\0"),
            )
        }
    }

    /// <p>This property holds the current time of the time line.</p>
    ///
    /// Returns a built-in Qt slot `QTimeLine::setCurrentTime` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#currentTime-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the current time of the time line.</p>
    /// <p>When <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> is in Running state, this value is updated continuously as a function of the duration and direction of the timeline. Otherwise, it is value that was current when <a href="http://doc.qt.io/qt-5/qtimeline.html#stop">stop</a>() was called last, or the value set by setCurrentTime().</p>
    /// <p>By default, this property contains a value of 0.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>currentTime</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setCurrentTime</b></span>(int <i>msec</i>)</td></tr>
    /// </tbody></table></div></div>
    pub fn slot_set_current_time(&self) -> crate::Receiver<(::std::os::raw::c_int,)> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1setCurrentTime(int)\0"),
            )
        }
    }

    /// <p>Toggles the direction of the timeline. If the direction was Forward, it becomes Backward, and vice verca.</p>
    ///
    /// Returns a built-in Qt slot `QTimeLine::toggleDirection` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#toggleDirection">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Toggles the direction of the timeline. If the direction was Forward, it becomes Backward, and vice verca.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimeline.html#direction-prop">setDirection</a>().</p></div>
    pub fn slot_toggle_direction(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1toggleDirection()\0"),
            )
        }
    }

    /// <p><a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> emits this signal at regular intervals when in <a href="http://doc.qt.io/qt-5/qtimeline.html#State-enum">Running</a> state, but only if the current value changes. <i>value</i> is the current value. <i>value</i> is a number between 0.0 and 1.0</p>
    ///
    /// Returns a built-in Qt signal `QTimeLine::valueChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qtimeline.html#valueChanged">C++ documentation</a> for <span style='color: green;'>``` void QTimeLine::valueChanged(qreal value)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p><a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> emits this signal at regular intervals when in <a href="http://doc.qt.io/qt-5/qtimeline.html#State-enum">Running</a> state, but only if the current value changes. <i>value</i> is the current value. <i>value</i> is a number between 0.0 and 1.0</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimeline.html#duration-prop">QTimeLine::setDuration</a>(), <a href="http://doc.qt.io/qt-5/qtimeline.html#valueForTime">QTimeLine::valueForTime</a>(), and <a href="http://doc.qt.io/qt-5/qtimeline.html#updateInterval-prop">QTimeLine::updateInterval</a>.</p></div>
    pub fn value_changed(&self) -> crate::Signal<(::std::os::raw::c_double,)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2valueChanged(double)\0"),
            )
        }
    }

    /// <p><a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> emits this signal at regular intervals when in <a href="http://doc.qt.io/qt-5/qtimeline.html#State-enum">Running</a> state, but only if the current frame changes. <i>frame</i> is the current frame number.</p>
    ///
    /// Returns a built-in Qt signal `QTimeLine::frameChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#frameChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p><a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> emits this signal at regular intervals when in <a href="http://doc.qt.io/qt-5/qtimeline.html#State-enum">Running</a> state, but only if the current frame changes. <i>frame</i> is the current frame number.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimeline.html#setFrameRange">QTimeLine::setFrameRange</a>() and <a href="http://doc.qt.io/qt-5/qtimeline.html#updateInterval-prop">QTimeLine::updateInterval</a>.</p></div>
    pub fn frame_changed(&self) -> crate::Signal<(::std::os::raw::c_int,)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2frameChanged(int)\0"),
            )
        }
    }

    /// <p>This signal is emitted whenever <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a>'s state changes. The new state is <i>newState</i>.</p>
    ///
    /// Returns a built-in Qt signal `QTimeLine::stateChanged` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#stateChanged">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted whenever <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a>'s state changes. The new state is <i>newState</i>.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p></div>
    pub fn state_changed(&self) -> crate::Signal<(crate::q_time_line::State,)> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"2stateChanged(QTimeLine::State)\0",
                ),
            )
        }
    }

    /// <p>This signal is emitted when <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> finishes (i.e., reaches the end of its time line), and does not loop.</p>
    ///
    /// Returns a built-in Qt signal `QTimeLine::finished` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#finished">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> finishes (i.e., reaches the end of its time line), and does not loop.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p></div>
    pub fn finished(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2finished()\0"),
            )
        }
    }

    /// <p>Returns the frame corresponding to the current time.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTimeLine::currentFrame() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#currentFrame">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the frame corresponding to the current time.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimeline.html#currentTime-prop">currentTime</a>(), <a href="http://doc.qt.io/qt-5/qtimeline.html#frameForTime">frameForTime</a>(), and <a href="http://doc.qt.io/qt-5/qtimeline.html#setFrameRange">setFrameRange</a>().</p></div>
    pub unsafe fn current_frame(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_currentFrame(self as *const crate::QTimeLine)
    }

    /// <p>This property holds the current time of the time line.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTimeLine::currentTime() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#currentTime-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the current time of the time line.</p>
    /// <p>When <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> is in Running state, this value is updated continuously as a function of the duration and direction of the timeline. Otherwise, it is value that was current when <a href="http://doc.qt.io/qt-5/qtimeline.html#stop">stop</a>() was called last, or the value set by setCurrentTime().</p>
    /// <p>By default, this property contains a value of 0.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>currentTime</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setCurrentTime</b></span>(int <i>msec</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn current_time(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_currentTime(self as *const crate::QTimeLine)
    }

    /// <p>Returns the value corresponding to the current time.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QTimeLine::currentValue() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#currentValue">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value corresponding to the current time.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimeline.html#valueForTime">valueForTime</a>() and <a href="http://doc.qt.io/qt-5/qtimeline.html#currentFrame">currentFrame</a>().</p></div>
    pub unsafe fn current_value(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_currentValue(self as *const crate::QTimeLine)
    }

    /// <p>This property holds the shape of the timeline curve.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTimeLine::CurveShape QTimeLine::curveShape() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#curveShape-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the shape of the timeline curve.</p>
    /// <p>The curve shape describes the relation between the time and value for the base implementation of <a href="http://doc.qt.io/qt-5/qtimeline.html#valueForTime">valueForTime</a>().</p>
    /// <p>If you have reimplemented <a href="http://doc.qt.io/qt-5/qtimeline.html#valueForTime">valueForTime</a>(), this value is ignored.</p>
    /// <p>By default, this property is set to <a href="http://doc.qt.io/qt-5/qtimeline.html#CurveShape-enum">EaseInOutCurve</a>.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> CurveShape </td><td class="memItemRight bottomAlign"><span class="name"><b>curveShape</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setCurveShape</b></span>(CurveShape <i>shape</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimeline.html#valueForTime">valueForTime</a>().</p></div>
    pub unsafe fn curve_shape(&self) -> crate::q_time_line::CurveShape {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_curveShape(self as *const crate::QTimeLine)
    }

    /// <p>This property holds the direction of the timeline when <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> is in <a href="http://doc.qt.io/qt-5/qtimeline.html#State-enum">Running</a> state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTimeLine::Direction QTimeLine::direction() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#direction-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the direction of the timeline when <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> is in <a href="http://doc.qt.io/qt-5/qtimeline.html#State-enum">Running</a> state.</p>
    /// <p>This direction indicates whether the time moves from 0 towards the timeline duration, or from the value of the duration and towards 0 after <a href="http://doc.qt.io/qt-5/qtimeline.html#start">start</a>() has been called.</p>
    /// <p>By default, this property is set to <a href="http://doc.qt.io/qt-5/qtimeline.html#Direction-enum">Forward</a>.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> Direction </td><td class="memItemRight bottomAlign"><span class="name"><b>direction</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setDirection</b></span>(Direction <i>direction</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn direction(&self) -> crate::q_time_line::Direction {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_direction(self as *const crate::QTimeLine)
    }

    /// <p>This property holds the total duration of the timeline in milliseconds.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTimeLine::duration() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#duration-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the total duration of the timeline in milliseconds.</p>
    /// <p>By default, this value is 1000 (i.e., 1 second), but you can change this by either passing a duration to <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a>'s constructor, or by calling setDuration(). The duration must be larger than 0.</p>
    /// <p><b>Note: </b>Changing the duration does not cause the current time to be reset to zero or the new duration. You also need to call <a href="http://doc.qt.io/qt-5/qtimeline.html#currentTime-prop">setCurrentTime</a>() with the desired value.</p><p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>duration</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setDuration</b></span>(int <i>duration</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn duration(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_duration(self as *const crate::QTimeLine)
    }

    /// <p>Specifies the easing curve that the timeline will use. If both easing curve and <a href="http://doc.qt.io/qt-5/qtimeline.html#curveShape-prop">curveShape</a> are set, the last set property will override the previous one. (If <a href="http://doc.qt.io/qt-5/qtimeline.html#valueForTime">valueForTime</a>() is reimplemented it will override both)</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QEasingCurve QTimeLine::easingCurve() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#easingCurve-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Specifies the easing curve that the timeline will use. If both easing curve and <a href="http://doc.qt.io/qt-5/qtimeline.html#curveShape-prop">curveShape</a> are set, the last set property will override the previous one. (If <a href="http://doc.qt.io/qt-5/qtimeline.html#valueForTime">valueForTime</a>() is reimplemented it will override both)</p>
    /// <p>This property was introduced in  Qt 4.6.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QEasingCurve </td><td class="memItemRight bottomAlign"><span class="name"><b>easingCurve</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setEasingCurve</b></span>(const QEasingCurve &amp;<i>curve</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn easing_curve(&self) -> ::cpp_utils::CppBox<crate::QEasingCurve> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QTimeLine_easingCurve(self as *const crate::QTimeLine);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the end frame, which is the frame corresponding to the end of the timeline (i.e., the frame for which the current value is 1).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTimeLine::endFrame() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#endFrame">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the end frame, which is the frame corresponding to the end of the timeline (i.e., the frame for which the current value is 1).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimeline.html#setEndFrame">setEndFrame</a>() and <a href="http://doc.qt.io/qt-5/qtimeline.html#setFrameRange">setFrameRange</a>().</p></div>
    pub unsafe fn end_frame(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_endFrame(self as *const crate::QTimeLine)
    }

    /// <p>Returns the frame corresponding to the time <i>msec</i>. This value is calculated using a linear interpolation of the start and end frame, based on the value returned by <a href="http://doc.qt.io/qt-5/qtimeline.html#valueForTime">valueForTime</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTimeLine::frameForTime(int msec) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#frameForTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the frame corresponding to the time <i>msec</i>. This value is calculated using a linear interpolation of the start and end frame, based on the value returned by <a href="http://doc.qt.io/qt-5/qtimeline.html#valueForTime">valueForTime</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimeline.html#valueForTime">valueForTime</a>() and <a href="http://doc.qt.io/qt-5/qtimeline.html#setFrameRange">setFrameRange</a>().</p></div>
    pub unsafe fn frame_for_time(&self, msec: ::std::os::raw::c_int) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_frameForTime(self as *const crate::QTimeLine, msec)
    }

    /// <p>This property holds the number of times the timeline should loop before it's finished.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTimeLine::loopCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#loopCount-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the number of times the timeline should loop before it's finished.</p>
    /// <p>A loop count of of 0 means that the timeline will loop forever.</p>
    /// <p>By default, this property contains a value of 1.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>loopCount</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setLoopCount</b></span>(int <i>count</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn loop_count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_loopCount(self as *const crate::QTimeLine)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QTimeLine::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QTimeLine_metaObject(self as *const crate::QTimeLine);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs a timeline with a duration of <i>duration</i> milliseconds. <i>parent</i> is passed to <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>'s constructor. The default duration is 1000 milliseconds.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTimeLine::QTimeLine(int duration = …, QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#QTimeLine">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a timeline with a duration of <i>duration</i> milliseconds. <i>parent</i> is passed to <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>'s constructor. The default duration is 1000 milliseconds.</p></div>
    pub unsafe fn new_2a(
        duration: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QTimeLine> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeLine_QTimeLine(
            duration,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> class provides a timeline for controlling animations.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTimeLine::QTimeLine()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> class provides a timeline for controlling animations.</p>
    /// <p>It's most commonly used to animate a GUI control by calling a slot periodically. You can construct a timeline by passing its duration in milliseconds to <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a>'s constructor. The timeline's duration describes for how long the animation will run. Then you set a suitable frame range by calling <a href="http://doc.qt.io/qt-5/qtimeline.html#setFrameRange">setFrameRange</a>(). Finally connect the <a href="http://doc.qt.io/qt-5/qtimeline.html#frameChanged">frameChanged</a>() signal to a suitable slot in the widget you wish to animate (for example, <a href="http://doc.qt.io/qt-5/qprogressbar.html#value-prop">setValue()</a> in <a href="http://doc.qt.io/qt-5/qprogressbar.html">QProgressBar</a>). When you proceed to calling <a href="http://doc.qt.io/qt-5/qtimeline.html#start">start</a>(), <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> will enter Running state, and start emitting <a href="http://doc.qt.io/qt-5/qtimeline.html#frameChanged">frameChanged</a>() at regular intervals, causing your widget's connected property's value to grow from the lower end to the upper and of your frame range, at a steady rate. You can specify the update interval by calling <a href="http://doc.qt.io/qt-5/qtimeline.html#updateInterval-prop">setUpdateInterval</a>(). When done, <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> enters <a href="http://doc.qt.io/qt-5/qtimeline.html#State-enum">NotRunning</a> state, and emits <a href="http://doc.qt.io/qt-5/qtimeline.html#finished">finished</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   progressBar <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qprogressbar.html">QProgressBar</a></span>(<span class="keyword">this</span>);
    ///   progressBar<span class="operator">-</span><span class="operator">&gt;</span>setRange(<span class="number">0</span><span class="operator">,</span> <span class="number">100</span>);
    ///
    ///   <span class="comment">// Construct a 1-second timeline with a frame range of 0 - 100</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtimeline.html#QTimeLine">QTimeLine</a></span> <span class="operator">*</span>timeLine <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtimeline.html#QTimeLine">QTimeLine</a></span>(<span class="number">1000</span><span class="operator">,</span> <span class="keyword">this</span>);
    ///   timeLine<span class="operator">-</span><span class="operator">&gt;</span>setFrameRange(<span class="number">0</span><span class="operator">,</span> <span class="number">100</span>);
    ///   connect(timeLine<span class="operator">,</span> SIGNAL(frameChanged(<span class="type">int</span>))<span class="operator">,</span> progressBar<span class="operator">,</span> SLOT(setValue(<span class="type">int</span>)));
    ///
    ///   <span class="comment">// Clicking the push button will start the progress bar animation</span>
    ///   pushButton <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span>(tr(<span class="string">"Start animation"</span>)<span class="operator">,</span> <span class="keyword">this</span>);
    ///   connect(pushButton<span class="operator">,</span> SIGNAL(clicked())<span class="operator">,</span> timeLine<span class="operator">,</span> SLOT(start()));
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///
    /// </pre>
    /// <p>By default the timeline runs once, from the beginning and towards the end, upon which you must call <a href="http://doc.qt.io/qt-5/qtimeline.html#start">start</a>() again to restart from the beginning. To make the timeline loop, you can call <a href="http://doc.qt.io/qt-5/qtimeline.html#loopCount-prop">setLoopCount</a>(), passing the number of times the timeline should run before finishing. The direction can also be changed, causing the timeline to run backward, by calling <a href="http://doc.qt.io/qt-5/qtimeline.html#direction-prop">setDirection</a>(). You can also pause and unpause the timeline while it's running by calling <a href="http://doc.qt.io/qt-5/qtimeline.html#setPaused">setPaused</a>(). For interactive control, the <a href="http://doc.qt.io/qt-5/qtimeline.html#currentTime-prop">setCurrentTime</a>() function is provided, which sets the time position of the time line directly. Although most useful in <a href="http://doc.qt.io/qt-5/qtimeline.html#State-enum">NotRunning</a> state, (e.g., connected to a <a href="http://doc.qt.io/qt-5/qtimeline.html#valueChanged">valueChanged</a>() signal in a <a href="http://doc.qt.io/qt-5/qslider.html">QSlider</a>,) this function can be called at any time.</p>
    /// <p>The frame interface is useful for standard widgets, but <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> can be used to control any type of animation. The heart of <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> lies in the <a href="http://doc.qt.io/qt-5/qtimeline.html#valueForTime">valueForTime</a>() function, which generates a <i>value</i> between 0 and 1 for a given time. This value is typically used to describe the steps of an animation, where 0 is the first step of an animation, and 1 is the last step. When running, <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> generates values between 0 and 1 by calling <a href="http://doc.qt.io/qt-5/qtimeline.html#valueForTime">valueForTime</a>() and emitting <a href="http://doc.qt.io/qt-5/qtimeline.html#valueChanged">valueChanged</a>(). By default, <a href="http://doc.qt.io/qt-5/qtimeline.html#valueForTime">valueForTime</a>() applies an interpolation algorithm to generate these value. You can choose from a set of predefined timeline algorithms by calling <a href="http://doc.qt.io/qt-5/qtimeline.html#curveShape-prop">setCurveShape</a>().</p>
    /// <p>Note that by default, <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> uses the EaseInOut curve shape, which provides a value that grows slowly, then grows steadily, and finally grows slowly. For a custom timeline, you can reimplement <a href="http://doc.qt.io/qt-5/qtimeline.html#valueForTime">valueForTime</a>(), in which case <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a>'s <a href="http://doc.qt.io/qt-5/qtimeline.html#curveShape-prop">curveShape</a> property is ignored.</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QTimeLine> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeLine_QTimeLine1();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a timeline with a duration of <i>duration</i> milliseconds. <i>parent</i> is passed to <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>'s constructor. The default duration is 1000 milliseconds.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTimeLine::QTimeLine(int duration = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#QTimeLine">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a timeline with a duration of <i>duration</i> milliseconds. <i>parent</i> is passed to <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>'s constructor. The default duration is 1000 milliseconds.</p></div>
    pub unsafe fn new_1a(duration: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QTimeLine> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeLine_QTimeLine2(duration);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QTimeLine::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_qt_metacall(
            self as *mut crate::QTimeLine,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QTimeLine::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeLine_qt_metacast(
            self as *mut crate::QTimeLine,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Resumes the timeline from the current time. <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> will reenter Running state, and once it enters the event loop, it will update its current time, frame and value at regular intervals.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QTimeLine::resume()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#resume">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Resumes the timeline from the current time. <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> will reenter Running state, and once it enters the event loop, it will update its current time, frame and value at regular intervals.</p>
    /// <p>In contrast to <a href="http://doc.qt.io/qt-5/qtimeline.html#start">start</a>(), this function does not restart the timeline before it resumes.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimeline.html#start">start</a>(), <a href="http://doc.qt.io/qt-5/qtimeline.html#updateInterval-prop">updateInterval</a>(), <a href="http://doc.qt.io/qt-5/qtimeline.html#frameChanged">frameChanged</a>(), and <a href="http://doc.qt.io/qt-5/qtimeline.html#valueChanged">valueChanged</a>().</p></div>
    pub unsafe fn resume(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_resume(self as *mut crate::QTimeLine)
    }

    /// <p>This property holds the current time of the time line.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QTimeLine::setCurrentTime(int msec)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#currentTime-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the current time of the time line.</p>
    /// <p>When <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> is in Running state, this value is updated continuously as a function of the duration and direction of the timeline. Otherwise, it is value that was current when <a href="http://doc.qt.io/qt-5/qtimeline.html#stop">stop</a>() was called last, or the value set by setCurrentTime().</p>
    /// <p>By default, this property contains a value of 0.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>currentTime</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setCurrentTime</b></span>(int <i>msec</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn set_current_time(&mut self, msec: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_setCurrentTime(self as *mut crate::QTimeLine, msec)
    }

    /// <p>This property holds the shape of the timeline curve.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTimeLine::setCurveShape(QTimeLine::CurveShape shape)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#curveShape-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the shape of the timeline curve.</p>
    /// <p>The curve shape describes the relation between the time and value for the base implementation of <a href="http://doc.qt.io/qt-5/qtimeline.html#valueForTime">valueForTime</a>().</p>
    /// <p>If you have reimplemented <a href="http://doc.qt.io/qt-5/qtimeline.html#valueForTime">valueForTime</a>(), this value is ignored.</p>
    /// <p>By default, this property is set to <a href="http://doc.qt.io/qt-5/qtimeline.html#CurveShape-enum">EaseInOutCurve</a>.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> CurveShape </td><td class="memItemRight bottomAlign"><span class="name"><b>curveShape</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setCurveShape</b></span>(CurveShape <i>shape</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimeline.html#valueForTime">valueForTime</a>().</p></div>
    pub unsafe fn set_curve_shape(&mut self, shape: crate::q_time_line::CurveShape) {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_setCurveShape(self as *mut crate::QTimeLine, shape)
    }

    /// <p>This property holds the direction of the timeline when <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> is in <a href="http://doc.qt.io/qt-5/qtimeline.html#State-enum">Running</a> state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTimeLine::setDirection(QTimeLine::Direction direction)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#direction-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the direction of the timeline when <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> is in <a href="http://doc.qt.io/qt-5/qtimeline.html#State-enum">Running</a> state.</p>
    /// <p>This direction indicates whether the time moves from 0 towards the timeline duration, or from the value of the duration and towards 0 after <a href="http://doc.qt.io/qt-5/qtimeline.html#start">start</a>() has been called.</p>
    /// <p>By default, this property is set to <a href="http://doc.qt.io/qt-5/qtimeline.html#Direction-enum">Forward</a>.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> Direction </td><td class="memItemRight bottomAlign"><span class="name"><b>direction</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setDirection</b></span>(Direction <i>direction</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn set_direction(&mut self, direction: crate::q_time_line::Direction) {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_setDirection(
            self as *mut crate::QTimeLine,
            direction,
        )
    }

    /// <p>This property holds the total duration of the timeline in milliseconds.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTimeLine::setDuration(int duration)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#duration-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the total duration of the timeline in milliseconds.</p>
    /// <p>By default, this value is 1000 (i.e., 1 second), but you can change this by either passing a duration to <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a>'s constructor, or by calling setDuration(). The duration must be larger than 0.</p>
    /// <p><b>Note: </b>Changing the duration does not cause the current time to be reset to zero or the new duration. You also need to call <a href="http://doc.qt.io/qt-5/qtimeline.html#currentTime-prop">setCurrentTime</a>() with the desired value.</p><p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>duration</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setDuration</b></span>(int <i>duration</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn set_duration(&mut self, duration: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_setDuration(self as *mut crate::QTimeLine, duration)
    }

    /// <p>Specifies the easing curve that the timeline will use. If both easing curve and <a href="http://doc.qt.io/qt-5/qtimeline.html#curveShape-prop">curveShape</a> are set, the last set property will override the previous one. (If <a href="http://doc.qt.io/qt-5/qtimeline.html#valueForTime">valueForTime</a>() is reimplemented it will override both)</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTimeLine::setEasingCurve(const QEasingCurve& curve)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#easingCurve-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Specifies the easing curve that the timeline will use. If both easing curve and <a href="http://doc.qt.io/qt-5/qtimeline.html#curveShape-prop">curveShape</a> are set, the last set property will override the previous one. (If <a href="http://doc.qt.io/qt-5/qtimeline.html#valueForTime">valueForTime</a>() is reimplemented it will override both)</p>
    /// <p>This property was introduced in  Qt 4.6.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> QEasingCurve </td><td class="memItemRight bottomAlign"><span class="name"><b>easingCurve</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setEasingCurve</b></span>(const QEasingCurve &amp;<i>curve</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn set_easing_curve(
        &mut self,
        curve: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QEasingCurve>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_setEasingCurve(
            self as *mut crate::QTimeLine,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QEasingCurve>>::cast_into(curve)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the end frame, which is the frame corresponding to the end of the timeline (i.e., the frame for which the current value is 1), to <i>frame</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTimeLine::setEndFrame(int frame)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#setEndFrame">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the end frame, which is the frame corresponding to the end of the timeline (i.e., the frame for which the current value is 1), to <i>frame</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimeline.html#endFrame">endFrame</a>(), <a href="http://doc.qt.io/qt-5/qtimeline.html#startFrame">startFrame</a>(), and <a href="http://doc.qt.io/qt-5/qtimeline.html#setFrameRange">setFrameRange</a>().</p></div>
    pub unsafe fn set_end_frame(&mut self, frame: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_setEndFrame(self as *mut crate::QTimeLine, frame)
    }

    /// <p>Sets the timeline's frame counter to start at <i>startFrame</i>, and end and <i>endFrame</i>. For each time value, <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> will find the corresponding frame when you call <a href="http://doc.qt.io/qt-5/qtimeline.html#currentFrame">currentFrame</a>() or <a href="http://doc.qt.io/qt-5/qtimeline.html#frameForTime">frameForTime</a>() by interpolating, using the return value of <a href="http://doc.qt.io/qt-5/qtimeline.html#valueForTime">valueForTime</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTimeLine::setFrameRange(int startFrame, int endFrame)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#setFrameRange">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the timeline's frame counter to start at <i>startFrame</i>, and end and <i>endFrame</i>. For each time value, <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> will find the corresponding frame when you call <a href="http://doc.qt.io/qt-5/qtimeline.html#currentFrame">currentFrame</a>() or <a href="http://doc.qt.io/qt-5/qtimeline.html#frameForTime">frameForTime</a>() by interpolating, using the return value of <a href="http://doc.qt.io/qt-5/qtimeline.html#valueForTime">valueForTime</a>().</p>
    /// <p>When in Running state, <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> also emits the <a href="http://doc.qt.io/qt-5/qtimeline.html#frameChanged">frameChanged</a>() signal when the frame changes.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimeline.html#startFrame">startFrame</a>(), <a href="http://doc.qt.io/qt-5/qtimeline.html#endFrame">endFrame</a>(), <a href="http://doc.qt.io/qt-5/qtimeline.html#start">start</a>(), and <a href="http://doc.qt.io/qt-5/qtimeline.html#currentFrame">currentFrame</a>().</p></div>
    pub unsafe fn set_frame_range(
        &mut self,
        start_frame: ::std::os::raw::c_int,
        end_frame: ::std::os::raw::c_int,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_setFrameRange(
            self as *mut crate::QTimeLine,
            start_frame,
            end_frame,
        )
    }

    /// <p>This property holds the number of times the timeline should loop before it's finished.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTimeLine::setLoopCount(int count)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#loopCount-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the number of times the timeline should loop before it's finished.</p>
    /// <p>A loop count of of 0 means that the timeline will loop forever.</p>
    /// <p>By default, this property contains a value of 1.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>loopCount</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setLoopCount</b></span>(int <i>count</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn set_loop_count(&mut self, count: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_setLoopCount(self as *mut crate::QTimeLine, count)
    }

    /// <p>If <i>paused</i> is true, the timeline is paused, causing <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> to enter Paused state. No updates will be signaled until either <a href="http://doc.qt.io/qt-5/qtimeline.html#start">start</a>() or setPaused(false) is called. If <i>paused</i> is false, the timeline is resumed and continues where it left.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QTimeLine::setPaused(bool paused)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#setPaused">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If <i>paused</i> is true, the timeline is paused, causing <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> to enter Paused state. No updates will be signaled until either <a href="http://doc.qt.io/qt-5/qtimeline.html#start">start</a>() or setPaused(false) is called. If <i>paused</i> is false, the timeline is resumed and continues where it left.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimeline.html#state">state</a>() and <a href="http://doc.qt.io/qt-5/qtimeline.html#start">start</a>().</p></div>
    pub unsafe fn set_paused(&mut self, paused: bool) {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_setPaused(self as *mut crate::QTimeLine, paused)
    }

    /// <p>Sets the start frame, which is the frame corresponding to the start of the timeline (i.e., the frame for which the current value is 0), to <i>frame</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTimeLine::setStartFrame(int frame)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#setStartFrame">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the start frame, which is the frame corresponding to the start of the timeline (i.e., the frame for which the current value is 0), to <i>frame</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimeline.html#startFrame">startFrame</a>(), <a href="http://doc.qt.io/qt-5/qtimeline.html#endFrame">endFrame</a>(), and <a href="http://doc.qt.io/qt-5/qtimeline.html#setFrameRange">setFrameRange</a>().</p></div>
    pub unsafe fn set_start_frame(&mut self, frame: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_setStartFrame(self as *mut crate::QTimeLine, frame)
    }

    /// <p>This property holds the time in milliseconds between each time <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> updates its current time.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTimeLine::setUpdateInterval(int interval)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#updateInterval-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the time in milliseconds between each time <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> updates its current time.</p>
    /// <p>When updating the current time, <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> will emit <a href="http://doc.qt.io/qt-5/qtimeline.html#valueChanged">valueChanged</a>() if the current value changed, and <a href="http://doc.qt.io/qt-5/qtimeline.html#frameChanged">frameChanged</a>() if the frame changed.</p>
    /// <p>By default, the interval is 40 ms, which corresponds to a rate of 25 updates per second.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>updateInterval</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setUpdateInterval</b></span>(int <i>interval</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn set_update_interval(&mut self, interval: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_setUpdateInterval(
            self as *mut crate::QTimeLine,
            interval,
        )
    }

    /// <p>Starts the timeline. <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> will enter Running state, and once it enters the event loop, it will update its current time, frame and value at regular intervals. The default interval is 40 ms (i.e., 25 times per second). You can change the update interval by calling <a href="http://doc.qt.io/qt-5/qtimeline.html#updateInterval-prop">setUpdateInterval</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QTimeLine::start()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#start">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Starts the timeline. <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> will enter Running state, and once it enters the event loop, it will update its current time, frame and value at regular intervals. The default interval is 40 ms (i.e., 25 times per second). You can change the update interval by calling <a href="http://doc.qt.io/qt-5/qtimeline.html#updateInterval-prop">setUpdateInterval</a>().</p>
    /// <p>The timeline will start from position 0, or the end if going backward. If you want to resume a stopped timeline without restarting, you can call <a href="http://doc.qt.io/qt-5/qtimeline.html#resume">resume</a>() instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimeline.html#resume">resume</a>(), <a href="http://doc.qt.io/qt-5/qtimeline.html#updateInterval-prop">updateInterval</a>(), <a href="http://doc.qt.io/qt-5/qtimeline.html#frameChanged">frameChanged</a>(), and <a href="http://doc.qt.io/qt-5/qtimeline.html#valueChanged">valueChanged</a>().</p></div>
    pub unsafe fn start(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_start(self as *mut crate::QTimeLine)
    }

    /// <p>Returns the start frame, which is the frame corresponding to the start of the timeline (i.e., the frame for which the current value is 0).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTimeLine::startFrame() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#startFrame">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the start frame, which is the frame corresponding to the start of the timeline (i.e., the frame for which the current value is 0).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimeline.html#setStartFrame">setStartFrame</a>() and <a href="http://doc.qt.io/qt-5/qtimeline.html#setFrameRange">setFrameRange</a>().</p></div>
    pub unsafe fn start_frame(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_startFrame(self as *const crate::QTimeLine)
    }

    /// <p>Returns the state of the timeline.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTimeLine::State QTimeLine::state() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#state">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the state of the timeline.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimeline.html#start">start</a>(), <a href="http://doc.qt.io/qt-5/qtimeline.html#setPaused">setPaused</a>(), and <a href="http://doc.qt.io/qt-5/qtimeline.html#stop">stop</a>().</p></div>
    pub unsafe fn state(&self) -> crate::q_time_line::State {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_state(self as *const crate::QTimeLine)
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeLine_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Stops the timeline, causing <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> to enter <a href="http://doc.qt.io/qt-5/qtimeline.html#State-enum">NotRunning</a> state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QTimeLine::stop()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#stop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Stops the timeline, causing <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> to enter <a href="http://doc.qt.io/qt-5/qtimeline.html#State-enum">NotRunning</a> state.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimeline.html#start">start</a>().</p></div>
    pub unsafe fn stop(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_stop(self as *mut crate::QTimeLine)
    }

    /// <p>Toggles the direction of the timeline. If the direction was Forward, it becomes Backward, and vice verca.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QTimeLine::toggleDirection()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#toggleDirection">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Toggles the direction of the timeline. If the direction was Forward, it becomes Backward, and vice verca.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimeline.html#direction-prop">setDirection</a>().</p></div>
    pub unsafe fn toggle_direction(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_toggleDirection(self as *mut crate::QTimeLine)
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QTimeLine::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeLine_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QTimeLine::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeLine_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This property holds the time in milliseconds between each time <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> updates its current time.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTimeLine::updateInterval() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#updateInterval-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the time in milliseconds between each time <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> updates its current time.</p>
    /// <p>When updating the current time, <a href="http://doc.qt.io/qt-5/qtimeline.html">QTimeLine</a> will emit <a href="http://doc.qt.io/qt-5/qtimeline.html#valueChanged">valueChanged</a>() if the current value changed, and <a href="http://doc.qt.io/qt-5/qtimeline.html#frameChanged">frameChanged</a>() if the frame changed.</p>
    /// <p>By default, the interval is 40 ms, which corresponds to a rate of 25 updates per second.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>updateInterval</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setUpdateInterval</b></span>(int <i>interval</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn update_interval(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_updateInterval(self as *const crate::QTimeLine)
    }

    /// <p>Returns the timeline value for the time <i>msec</i>. The returned value, which varies depending on the curve shape, is always between 0 and 1. If <i>msec</i> is 0, the default implementation always returns 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual double QTimeLine::valueForTime(int msec) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#valueForTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the timeline value for the time <i>msec</i>. The returned value, which varies depending on the curve shape, is always between 0 and 1. If <i>msec</i> is 0, the default implementation always returns 0.</p>
    /// <p>Reimplement this function to provide a custom curve shape for your timeline.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimeline.html#CurveShape-enum">CurveShape</a> and <a href="http://doc.qt.io/qt-5/qtimeline.html#frameForTime">frameForTime</a>().</p></div>
    pub unsafe fn value_for_time(&self, msec: ::std::os::raw::c_int) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_valueForTime(self as *const crate::QTimeLine, msec)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> class provides repetitive and single-shot timers.</p>
///
/// C++ class: <span style='color: green;'>```QTimer```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtimer.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> class provides repetitive and single-shot timers.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> class provides a high-level programming interface for timers. To use it, create a <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a>, connect its <a href="http://doc.qt.io/qt-5/qtimer.html#timeout">timeout</a>() signal to the appropriate slots, and call <a href="http://doc.qt.io/qt-5/qtimer.html#start-1">start</a>(). From then on, it will emit the <a href="http://doc.qt.io/qt-5/qtimer.html#timeout">timeout</a>() signal at constant intervals.</p>
/// <p>Example for a one second (1000 millisecond) timer (from the <a href="http://doc.qt.io/qt-5/qtwidgets-widgets-analogclock-example.html">Analog Clock</a> example):</p>
/// <pre class="cpp">
///
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qtimer.html#QTimer">QTimer</a></span> <span class="operator">*</span>timer <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtimer.html#QTimer">QTimer</a></span>(<span class="keyword">this</span>);
/// &#32;     connect(timer<span class="operator">,</span> SIGNAL(timeout())<span class="operator">,</span> <span class="keyword">this</span><span class="operator">,</span> SLOT(update()));
/// &#32;     timer<span class="operator">-</span><span class="operator">&gt;</span>start(<span class="number">1000</span>);
///
/// </pre>
/// <p>From then on, the <code>update()</code> slot is called every second.</p>
/// <p>You can set a timer to time out only once by calling <a href="http://doc.qt.io/qt-5/qtimer.html#singleShot-prop">setSingleShot</a>(true). You can also use the static <a href="http://doc.qt.io/qt-5/qtimer.html#singleShot">QTimer::singleShot</a>() function to call a slot after a specified interval:</p>
/// <pre class="cpp">
///
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qtimer.html#QTimer">QTimer</a></span><span class="operator">::</span>singleShot(<span class="number">200</span><span class="operator">,</span> <span class="keyword">this</span><span class="operator">,</span> SLOT(updateCaption()));
///
/// </pre>
/// <p>In multithreaded applications, you can use <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> in any thread that has an event loop. To start an event loop from a non-GUI thread, use <a href="http://doc.qt.io/qt-5/qthread.html#exec">QThread::exec</a>(). Qt uses the timer's <a href="http://doc.qt.io/qt-5/qobject.html#thread">thread affinity</a> to determine which thread will emit the <a href="http://doc.qt.io/qt-5/qtimer.html#timeout">timeout()</a> signal. Because of this, you must start and stop the timer in its thread; it is not possible to start a timer from another thread.</p>
/// <p>As a special case, a <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> with a timeout of 0 will time out as soon as all the events in the window system's event queue have been processed. This can be used to do heavy work while providing a snappy user interface:</p>
/// <pre class="cpp">
///
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qtimer.html#QTimer">QTimer</a></span> <span class="operator">*</span>timer <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtimer.html#QTimer">QTimer</a></span>(<span class="keyword">this</span>);
/// &#32;     connect(timer<span class="operator">,</span> SIGNAL(timeout())<span class="operator">,</span> <span class="keyword">this</span><span class="operator">,</span> SLOT(processOneThing()));
/// &#32;     timer<span class="operator">-</span><span class="operator">&gt;</span>start();
///
/// </pre>
/// <p>From then on, <code>processOneThing()</code> will be called repeatedly. It should be written in such a way that it always returns quickly (typically after processing one data item) so that Qt can deliver events to the user interface and stop the timer as soon as it has done all its work. This is the traditional way of implementing heavy work in GUI applications, but as multithreading is nowadays becoming available on more and more platforms, we expect that zero-millisecond <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> objects will gradually be replaced by <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a>s.</p>
/// <a name="accuracy-and-timer-resolution"></a></div>
#[repr(C)]
pub struct QTimer {
    _unused: u8,
}
impl QTimer {
    /// <p>Starts or restarts the timer with a timeout interval of <i>msec</i> milliseconds.</p>
    ///
    /// Returns a built-in Qt slot `QTimer::start` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimer.html#start">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Starts or restarts the timer with a timeout interval of <i>msec</i> milliseconds.</p>
    /// <p>If the timer is already running, it will be <a href="http://doc.qt.io/qt-5/qtimer.html#stop">stopped</a> and restarted.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qtimer.html#singleShot-prop">singleShot</a> is true, the timer will be activated only once.</p></div>
    pub fn slot_start(&self) -> crate::Receiver<(::std::os::raw::c_int,)> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1start(int)\0"),
            )
        }
    }

    /// <p>This function overloads start().</p>
    ///
    /// Returns a built-in Qt slot `QTimer::start` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimer.html#start-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads start().</p>
    /// <p>Starts or restarts the timer with the timeout specified in <a href="http://doc.qt.io/qt-5/qtimer.html#interval-prop">interval</a>.</p>
    /// <p>If the timer is already running, it will be <a href="http://doc.qt.io/qt-5/qtimer.html#stop">stopped</a> and restarted.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qtimer.html#singleShot-prop">singleShot</a> is true, the timer will be activated only once.</p></div>
    pub fn slot_start2(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1start()\0"),
            )
        }
    }

    /// <p>Stops the timer.</p>
    ///
    /// Returns a built-in Qt slot `QTimer::stop` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimer.html#stop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Stops the timer.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimer.html#start-1">start</a>().</p></div>
    pub fn slot_stop(&self) -> crate::Receiver<()> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1stop()\0"),
            )
        }
    }

    /// <p>This signal is emitted when the timer times out.</p>
    ///
    /// Returns a built-in Qt signal `QTimer::timeout` that can be passed to `qt_core::Signal::connect`.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimer.html#timeout">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This signal is emitted when the timer times out.</p>
    /// <p><b>Note: </b>This is a private signal. It can be used in signal connections but cannot be emitted by the user.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimer.html#interval-prop">interval</a>, <a href="http://doc.qt.io/qt-5/qtimer.html#start-1">start</a>(), and <a href="http://doc.qt.io/qt-5/qtimer.html#stop">stop</a>().</p></div>
    pub fn timeout(&self) -> crate::Signal<()> {
        unsafe {
            crate::Signal::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"2timeout()\0"),
            )
        }
    }

    /// <p>This property holds the timeout interval in milliseconds</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTimer::interval() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimer.html#interval-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the timeout interval in milliseconds</p>
    /// <p>The default value for this property is 0. A <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> with a timeout interval of 0 will time out as soon as all the events in the window system's event queue have been processed.</p>
    /// <p>Setting the interval of an active timer changes its <a href="http://doc.qt.io/qt-5/qtimer.html#timerId">timerId</a>().</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>interval</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setInterval</b></span>(int <i>msec</i>)</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setInterval</b></span>(std::chrono::milliseconds <i>value</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimer.html#singleShot-prop">singleShot</a>.</p></div>
    pub unsafe fn interval(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTimer_interval(self as *const crate::QTimer)
    }

    /// <p>Returns <code>true</code> if the timer is running (pending); otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTimer::isActive() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimer.html#isActive">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the timer is running (pending); otherwise returns false.</p>
    /// <p><b>Note:</b> Getter function for property <a href="http://doc.qt.io/qt-5/qtimer.html#active-prop">active</a>. </p></div>
    pub unsafe fn is_active(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTimer_isActive(self as *const crate::QTimer)
    }

    /// <p>This property holds whether the timer is a single-shot timer</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTimer::isSingleShot() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimer.html#singleShot-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds whether the timer is a single-shot timer</p>
    /// <p>A single-shot timer fires only once, non-single-shot timers fire every <a href="http://doc.qt.io/qt-5/qtimer.html#interval-prop">interval</a> milliseconds.</p>
    /// <p>The default value for this property is <code>false</code>.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> bool </td><td class="memItemRight bottomAlign"><span class="name"><b>isSingleShot</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setSingleShot</b></span>(bool <i>singleShot</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimer.html#interval-prop">interval</a> and <a href="http://doc.qt.io/qt-5/qtimer.html#singleShot">singleShot</a>().</p></div>
    pub unsafe fn is_single_shot(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTimer_isSingleShot(self as *const crate::QTimer)
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QTimer::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QTimer_metaObject(self as *const crate::QTimer);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs a timer with the given <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTimer::QTimer(QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimer.html#QTimer">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a timer with the given <i>parent</i>.</p></div>
    pub unsafe fn new_1a(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QTimer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimer_QTimer(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> class provides repetitive and single-shot timers.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTimer::QTimer()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimer.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> class provides repetitive and single-shot timers.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> class provides a high-level programming interface for timers. To use it, create a <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a>, connect its <a href="http://doc.qt.io/qt-5/qtimer.html#timeout">timeout</a>() signal to the appropriate slots, and call <a href="http://doc.qt.io/qt-5/qtimer.html#start-1">start</a>(). From then on, it will emit the <a href="http://doc.qt.io/qt-5/qtimer.html#timeout">timeout</a>() signal at constant intervals.</p>
    /// <p>Example for a one second (1000 millisecond) timer (from the <a href="http://doc.qt.io/qt-5/qtwidgets-widgets-analogclock-example.html">Analog Clock</a> example):</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qtimer.html#QTimer">QTimer</a></span> <span class="operator">*</span>timer <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtimer.html#QTimer">QTimer</a></span>(<span class="keyword">this</span>);
    /// &#32;     connect(timer<span class="operator">,</span> SIGNAL(timeout())<span class="operator">,</span> <span class="keyword">this</span><span class="operator">,</span> SLOT(update()));
    /// &#32;     timer<span class="operator">-</span><span class="operator">&gt;</span>start(<span class="number">1000</span>);
    ///
    /// </pre>
    /// <p>From then on, the <code>update()</code> slot is called every second.</p>
    /// <p>You can set a timer to time out only once by calling <a href="http://doc.qt.io/qt-5/qtimer.html#singleShot-prop">setSingleShot</a>(true). You can also use the static <a href="http://doc.qt.io/qt-5/qtimer.html#singleShot">QTimer::singleShot</a>() function to call a slot after a specified interval:</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qtimer.html#QTimer">QTimer</a></span><span class="operator">::</span>singleShot(<span class="number">200</span><span class="operator">,</span> <span class="keyword">this</span><span class="operator">,</span> SLOT(updateCaption()));
    ///
    /// </pre>
    /// <p>In multithreaded applications, you can use <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> in any thread that has an event loop. To start an event loop from a non-GUI thread, use <a href="http://doc.qt.io/qt-5/qthread.html#exec">QThread::exec</a>(). Qt uses the timer's <a href="http://doc.qt.io/qt-5/qobject.html#thread">thread affinity</a> to determine which thread will emit the <a href="http://doc.qt.io/qt-5/qtimer.html#timeout">timeout()</a> signal. Because of this, you must start and stop the timer in its thread; it is not possible to start a timer from another thread.</p>
    /// <p>As a special case, a <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> with a timeout of 0 will time out as soon as all the events in the window system's event queue have been processed. This can be used to do heavy work while providing a snappy user interface:</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qtimer.html#QTimer">QTimer</a></span> <span class="operator">*</span>timer <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtimer.html#QTimer">QTimer</a></span>(<span class="keyword">this</span>);
    /// &#32;     connect(timer<span class="operator">,</span> SIGNAL(timeout())<span class="operator">,</span> <span class="keyword">this</span><span class="operator">,</span> SLOT(processOneThing()));
    /// &#32;     timer<span class="operator">-</span><span class="operator">&gt;</span>start();
    ///
    /// </pre>
    /// <p>From then on, <code>processOneThing()</code> will be called repeatedly. It should be written in such a way that it always returns quickly (typically after processing one data item) so that Qt can deliver events to the user interface and stop the timer as soon as it has done all its work. This is the traditional way of implementing heavy work in GUI applications, but as multithreading is nowadays becoming available on more and more platforms, we expect that zero-millisecond <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> objects will gradually be replaced by <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a>s.</p>
    /// <a name="accuracy-and-timer-resolution"></a></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QTimer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimer_QTimer1();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QTimer::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTimer_qt_metacall(
            self as *mut crate::QTimer,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QTimer::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimer_qt_metacast(
            self as *mut crate::QTimer,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This property holds the remaining time in milliseconds</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTimer::remainingTime() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimer.html#remainingTime-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the remaining time in milliseconds</p>
    /// <p>Returns the timer's remaining value in milliseconds left until the timeout. If the timer is inactive, the returned value will be -1. If the timer is overdue, the returned value will be 0.</p>
    /// <p>This property was introduced in  Qt 5.0.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>remainingTime</b></span>() const</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimer.html#interval-prop">interval</a>.</p></div>
    pub unsafe fn remaining_time(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTimer_remainingTime(self as *const crate::QTimer)
    }

    /// <p>This property holds the timeout interval in milliseconds</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTimer::setInterval(int msec)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimer.html#interval-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the timeout interval in milliseconds</p>
    /// <p>The default value for this property is 0. A <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> with a timeout interval of 0 will time out as soon as all the events in the window system's event queue have been processed.</p>
    /// <p>Setting the interval of an active timer changes its <a href="http://doc.qt.io/qt-5/qtimer.html#timerId">timerId</a>().</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>interval</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setInterval</b></span>(int <i>msec</i>)</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setInterval</b></span>(std::chrono::milliseconds <i>value</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimer.html#singleShot-prop">singleShot</a>.</p></div>
    pub unsafe fn set_interval(&mut self, msec: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QTimer_setInterval(self as *mut crate::QTimer, msec)
    }

    /// <p>This property holds whether the timer is a single-shot timer</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTimer::setSingleShot(bool singleShot)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimer.html#singleShot-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds whether the timer is a single-shot timer</p>
    /// <p>A single-shot timer fires only once, non-single-shot timers fire every <a href="http://doc.qt.io/qt-5/qtimer.html#interval-prop">interval</a> milliseconds.</p>
    /// <p>The default value for this property is <code>false</code>.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> bool </td><td class="memItemRight bottomAlign"><span class="name"><b>isSingleShot</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setSingleShot</b></span>(bool <i>singleShot</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimer.html#interval-prop">interval</a> and <a href="http://doc.qt.io/qt-5/qtimer.html#singleShot">singleShot</a>().</p></div>
    pub unsafe fn set_single_shot(&mut self, single_shot: bool) {
        crate::__ffi::ctr_qt_core_ffi_QTimer_setSingleShot(self as *mut crate::QTimer, single_shot)
    }

    /// <p>controls the accuracy of the timer</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTimer::setTimerType(Qt::TimerType atype)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimer.html#timerType-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>controls the accuracy of the timer</p>
    /// <p>The default value for this property is <code>Qt::CoarseTimer</code>.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> Qt::TimerType </td><td class="memItemRight bottomAlign"><span class="name"><b>timerType</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setTimerType</b></span>(Qt::TimerType <i>atype</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qt.html#TimerType-enum">Qt::TimerType</a>.</p></div>
    pub unsafe fn set_timer_type(&mut self, atype: crate::TimerType) {
        crate::__ffi::ctr_qt_core_ffi_QTimer_setTimerType(self as *mut crate::QTimer, atype)
    }

    /// <p>This static function calls a slot after a given time interval.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QTimer::singleShot(int msec, const QObject* receiver, const char* member)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimer.html#singleShot">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This static function calls a slot after a given time interval.</p>
    /// <p>It is very convenient to use this function because you do not need to bother with a <a href="http://doc.qt.io/qt-5/qobject.html#timerEvent">timerEvent</a> or create a local <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> object.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="preprocessor">#include &lt;QApplication&gt;</span>
    ///   <span class="preprocessor">#include &lt;QTimer&gt;</span>
    ///
    ///   <span class="type">int</span> main(<span class="type">int</span> argc<span class="operator">,</span> <span class="type">char</span> <span class="operator">*</span>argv<span class="operator">[</span><span class="operator">]</span>)
    ///   {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qapplication.html">QApplication</a></span> app(argc<span class="operator">,</span> argv);
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qtimer.html#QTimer">QTimer</a></span><span class="operator">::</span>singleShot(<span class="number">600000</span><span class="operator">,</span> <span class="operator">&amp;</span>app<span class="operator">,</span> SLOT(quit()));
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    /// &#32;     <span class="keyword">return</span> app<span class="operator">.</span>exec();
    ///   }
    ///
    /// </pre>
    /// <p>This sample program automatically terminates after 10 minutes (600,000 milliseconds).</p>
    /// <p>The <i>receiver</i> is the receiving object and the <i>member</i> is the slot. The time interval is <i>msec</i> milliseconds.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">reentrant</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimer.html#singleShot-prop">setSingleShot</a>() and <a href="http://doc.qt.io/qt-5/qtimer.html#start-1">start</a>().</p></div>
    pub unsafe fn single_shot_3a(
        msec: ::std::os::raw::c_int,
        receiver: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        member: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QTimer_singleShot(
            msec,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(receiver)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(member)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QTimer::singleShot(int msec, Qt::TimerType timerType, const QObject* receiver, const char* member)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimer.html#singleShot-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This static function calls a slot after a given time interval.</p>
    /// <p>It is very convenient to use this function because you do not need to bother with a <a href="http://doc.qt.io/qt-5/qobject.html#timerEvent">timerEvent</a> or create a local <a href="http://doc.qt.io/qt-5/qtimer.html">QTimer</a> object.</p>
    /// <p>The <i>receiver</i> is the receiving object and the <i>member</i> is the slot. The time interval is <i>msec</i> milliseconds. The <i>timerType</i> affects the accuracy of the timer.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">reentrant</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimer.html#start-1">start</a>().</p></div>
    pub unsafe fn single_shot_4a(
        msec: ::std::os::raw::c_int,
        timer_type: crate::TimerType,
        receiver: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QObject>>,
        member: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QTimer_singleShot1(
            msec,
            timer_type,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QObject>>::cast_into(receiver)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(member)
                .as_raw_ptr(),
        )
    }

    /// <p>Starts or restarts the timer with a timeout interval of <i>msec</i> milliseconds.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QTimer::start(int msec)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimer.html#start">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Starts or restarts the timer with a timeout interval of <i>msec</i> milliseconds.</p>
    /// <p>If the timer is already running, it will be <a href="http://doc.qt.io/qt-5/qtimer.html#stop">stopped</a> and restarted.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qtimer.html#singleShot-prop">singleShot</a> is true, the timer will be activated only once.</p></div>
    pub unsafe fn start_1a(&mut self, msec: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QTimer_start(self as *mut crate::QTimer, msec)
    }

    /// <p>This function overloads start().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QTimer::start()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimer.html#start-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads start().</p>
    /// <p>Starts or restarts the timer with the timeout specified in <a href="http://doc.qt.io/qt-5/qtimer.html#interval-prop">interval</a>.</p>
    /// <p>If the timer is already running, it will be <a href="http://doc.qt.io/qt-5/qtimer.html#stop">stopped</a> and restarted.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qtimer.html#singleShot-prop">singleShot</a> is true, the timer will be activated only once.</p></div>
    pub unsafe fn start_0a(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QTimer_start1(self as *mut crate::QTimer)
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimer_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Stops the timer.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[slot] void QTimer::stop()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimer.html#stop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Stops the timer.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimer.html#start-1">start</a>().</p></div>
    pub unsafe fn stop(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QTimer_stop(self as *mut crate::QTimer)
    }

    /// <p>Returns the ID of the timer if the timer is running; otherwise returns -1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTimer::timerId() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimer.html#timerId">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the ID of the timer if the timer is running; otherwise returns -1.</p></div>
    pub unsafe fn timer_id(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTimer_timerId(self as *const crate::QTimer)
    }

    /// <p>controls the accuracy of the timer</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```Qt::TimerType QTimer::timerType() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimer.html#timerType-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>controls the accuracy of the timer</p>
    /// <p>The default value for this property is <code>Qt::CoarseTimer</code>.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> Qt::TimerType </td><td class="memItemRight bottomAlign"><span class="name"><b>timerType</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setTimerType</b></span>(Qt::TimerType <i>atype</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qt.html#TimerType-enum">Qt::TimerType</a>.</p></div>
    pub unsafe fn timer_type(&self) -> crate::TimerType {
        crate::__ffi::ctr_qt_core_ffi_QTimer_timerType(self as *const crate::QTimer)
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QTimer::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimer_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QTimer::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimer_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_time_zone {
    //! C++ type: <span style='color: green;'>```QTimeZone```</span>

    /// <p>The type of time zone time, for example when requesting the name. In time zones that do not apply DST, all three values may return the same result.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QTimeZone::TimeType```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#TimeType-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The type of time zone time, for example when requesting the name. In time zones that do not apply DST, all three values may return the same result.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct TimeType(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for TimeType {
        fn from(value: ::std::os::raw::c_int) -> Self {
            TimeType(value)
        }
    }

    impl From<TimeType> for ::std::os::raw::c_int {
        fn from(value: TimeType) -> Self {
            value.0
        }
    }

    impl TimeType {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl TimeType {
        /// The standard time in a time zone, i.e. when Daylight-Saving is not in effect. For example when formatting a display name this will show something like "Pacific Standard Time". (C++ enum variant: <span style='color: green;'>```StandardTime = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const StandardTime: crate::q_time_zone::TimeType = crate::q_time_zone::TimeType(0);
        /// A time when Daylight-Saving is in effect. For example when formatting a display name this will show something like "Pacific daylight-saving time". (C++ enum variant: <span style='color: green;'>```DaylightTime = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const DaylightTime: crate::q_time_zone::TimeType = crate::q_time_zone::TimeType(1);
        /// A time which is not specifically Standard or Daylight-Saving time, either an unknown time or a neutral form. For example when formatting a display name this will show something like "Pacific Time". (C++ enum variant: <span style='color: green;'>```GenericTime = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const GenericTime: crate::q_time_zone::TimeType = crate::q_time_zone::TimeType(2);
    }

    /// <p>The type of time zone name.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QTimeZone::NameType```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#NameType-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The type of time zone name.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct NameType(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for NameType {
        fn from(value: ::std::os::raw::c_int) -> Self {
            NameType(value)
        }
    }

    impl From<NameType> for ::std::os::raw::c_int {
        fn from(value: NameType) -> Self {
            value.0
        }
    }

    impl NameType {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl NameType {
        /// The default form of the time zone name, e.g. LongName, ShortName or OffsetName (C++ enum variant: <span style='color: green;'>```DefaultName = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const DefaultName: crate::q_time_zone::NameType = crate::q_time_zone::NameType(0);
        /// The long form of the time zone name, e.g. "Central European Time" (C++ enum variant: <span style='color: green;'>```LongName = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const LongName: crate::q_time_zone::NameType = crate::q_time_zone::NameType(1);
        /// The short form of the time zone name, usually an abbreviation, e.g. "CET" (C++ enum variant: <span style='color: green;'>```ShortName = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const ShortName: crate::q_time_zone::NameType = crate::q_time_zone::NameType(2);
        /// The standard ISO offset form of the time zone name, e.g. "UTC+01:00" (C++ enum variant: <span style='color: green;'>```OffsetName = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const OffsetName: crate::q_time_zone::NameType = crate::q_time_zone::NameType(3);
    }

    /// <p>The time zone offset data for a given moment in time, i.e. the time zone offsets and abbreviation to use at that moment in time.</p>
    ///
    /// C++ class: <span style='color: green;'>```QTimeZone::OffsetData```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The time zone offset data for a given moment in time, i.e. the time zone offsets and abbreviation to use at that moment in time.</p>
    /// <ul>
    /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#atUtc-var">OffsetData::atUtc</a> The datetime of the offset data in UTC time.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#offsetFromUtc-var">OffsetData::offsetFromUtc</a> The total offset from UTC in effect at the datetime.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#standardTimeOffset-var">OffsetData::standardTimeOffset</a> The standard time offset component of the total offset.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#daylightTimeOffset-var">OffsetData::daylightTimeOffset</a> The DST offset component of the total offset.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#abbreviation-var">OffsetData::abbreviation</a> The abbreviation in effect at the datetime.</li>
    /// </ul>
    /// <p>For example, for time zone "Europe/Berlin" the OffsetDate in standard and DST might be:</p>
    /// <ul>
    /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#atUtc-var">atUtc</a> = <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>(<a href="http://doc.qt.io/qt-5/qdate.html">QDate</a>(2013, 1, 1), <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a>(0, 0, 0), <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>)</li>
    /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#offsetFromUtc-var">offsetFromUtc</a> = 3600</li>
    /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#standardTimeOffset-var">standardTimeOffset</a> = 3600</li>
    /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#daylightTimeOffset-var">daylightTimeOffset</a> = 0</li>
    /// <li>abbreviation = "CET"</li>
    /// </ul>
    /// <ul>
    /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#atUtc-var">atUtc</a> = <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>(<a href="http://doc.qt.io/qt-5/qdate.html">QDate</a>(2013, 6, 1), <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a>(0, 0, 0), <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>)</li>
    /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#offsetFromUtc-var">offsetFromUtc</a> = 7200</li>
    /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#standardTimeOffset-var">standardTimeOffset</a> = 3600</li>
    /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#daylightTimeOffset-var">daylightTimeOffset</a> = 3600</li>
    /// <li>abbreviation = "CEST"</li>
    /// </ul></div>
    #[repr(C)]
    pub struct OffsetData {
        _unused: u8,
    }
    impl OffsetData {
        /// Returns a reference to the <span style='color: green;'>```abbreviation```</span> field.
        pub unsafe fn abbreviation(&self) -> ::cpp_utils::Ref<crate::QString> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_OffsetData_abbreviation(
                self as *const crate::q_time_zone::OffsetData,
            );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// Returns a mutable reference to the <span style='color: green;'>```abbreviation```</span> field.
        pub unsafe fn abbreviation_mut(&mut self) -> ::cpp_utils::MutRef<crate::QString> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_OffsetData_abbreviation_mut(
                self as *mut crate::q_time_zone::OffsetData,
            );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// Returns a reference to the <span style='color: green;'>```atUtc```</span> field.
        pub unsafe fn at_utc(&self) -> ::cpp_utils::Ref<crate::QDateTime> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_OffsetData_atUtc(
                self as *const crate::q_time_zone::OffsetData,
            );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// Returns a mutable reference to the <span style='color: green;'>```atUtc```</span> field.
        pub unsafe fn at_utc_mut(&mut self) -> ::cpp_utils::MutRef<crate::QDateTime> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_OffsetData_atUtc_mut(
                self as *mut crate::q_time_zone::OffsetData,
            );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>The time zone offset data for a given moment in time, i.e. the time zone offsets and abbreviation to use at that moment in time.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QTimeZone::OffsetData& QTimeZone::OffsetData::operator=(const QTimeZone::OffsetData& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The time zone offset data for a given moment in time, i.e. the time zone offsets and abbreviation to use at that moment in time.</p>
        /// <ul>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#atUtc-var">OffsetData::atUtc</a> The datetime of the offset data in UTC time.</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#offsetFromUtc-var">OffsetData::offsetFromUtc</a> The total offset from UTC in effect at the datetime.</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#standardTimeOffset-var">OffsetData::standardTimeOffset</a> The standard time offset component of the total offset.</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#daylightTimeOffset-var">OffsetData::daylightTimeOffset</a> The DST offset component of the total offset.</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#abbreviation-var">OffsetData::abbreviation</a> The abbreviation in effect at the datetime.</li>
        /// </ul>
        /// <p>For example, for time zone "Europe/Berlin" the OffsetDate in standard and DST might be:</p>
        /// <ul>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#atUtc-var">atUtc</a> = <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>(<a href="http://doc.qt.io/qt-5/qdate.html">QDate</a>(2013, 1, 1), <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a>(0, 0, 0), <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>)</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#offsetFromUtc-var">offsetFromUtc</a> = 3600</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#standardTimeOffset-var">standardTimeOffset</a> = 3600</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#daylightTimeOffset-var">daylightTimeOffset</a> = 0</li>
        /// <li>abbreviation = "CET"</li>
        /// </ul>
        /// <ul>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#atUtc-var">atUtc</a> = <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>(<a href="http://doc.qt.io/qt-5/qdate.html">QDate</a>(2013, 6, 1), <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a>(0, 0, 0), <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>)</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#offsetFromUtc-var">offsetFromUtc</a> = 7200</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#standardTimeOffset-var">standardTimeOffset</a> = 3600</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#daylightTimeOffset-var">daylightTimeOffset</a> = 3600</li>
        /// <li>abbreviation = "CEST"</li>
        /// </ul></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>,
        ) -> ::cpp_utils::MutRef<crate::q_time_zone::OffsetData> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_OffsetData_operator_(self as *mut crate::q_time_zone::OffsetData, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// Returns the value of the <span style='color: green;'>```daylightTimeOffset```</span> field.
        pub unsafe fn daylight_time_offset(&self) -> ::std::os::raw::c_int {
            crate::__ffi::ctr_qt_core_ffi_QTimeZone_OffsetData_daylightTimeOffset(
                self as *const crate::q_time_zone::OffsetData,
            )
        }

        /// <p>The time zone offset data for a given moment in time, i.e. the time zone offsets and abbreviation to use at that moment in time.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QTimeZone::OffsetData::OffsetData()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The time zone offset data for a given moment in time, i.e. the time zone offsets and abbreviation to use at that moment in time.</p>
        /// <ul>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#atUtc-var">OffsetData::atUtc</a> The datetime of the offset data in UTC time.</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#offsetFromUtc-var">OffsetData::offsetFromUtc</a> The total offset from UTC in effect at the datetime.</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#standardTimeOffset-var">OffsetData::standardTimeOffset</a> The standard time offset component of the total offset.</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#daylightTimeOffset-var">OffsetData::daylightTimeOffset</a> The DST offset component of the total offset.</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#abbreviation-var">OffsetData::abbreviation</a> The abbreviation in effect at the datetime.</li>
        /// </ul>
        /// <p>For example, for time zone "Europe/Berlin" the OffsetDate in standard and DST might be:</p>
        /// <ul>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#atUtc-var">atUtc</a> = <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>(<a href="http://doc.qt.io/qt-5/qdate.html">QDate</a>(2013, 1, 1), <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a>(0, 0, 0), <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>)</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#offsetFromUtc-var">offsetFromUtc</a> = 3600</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#standardTimeOffset-var">standardTimeOffset</a> = 3600</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#daylightTimeOffset-var">daylightTimeOffset</a> = 0</li>
        /// <li>abbreviation = "CET"</li>
        /// </ul>
        /// <ul>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#atUtc-var">atUtc</a> = <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>(<a href="http://doc.qt.io/qt-5/qdate.html">QDate</a>(2013, 6, 1), <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a>(0, 0, 0), <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>)</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#offsetFromUtc-var">offsetFromUtc</a> = 7200</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#standardTimeOffset-var">standardTimeOffset</a> = 3600</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#daylightTimeOffset-var">daylightTimeOffset</a> = 3600</li>
        /// <li>abbreviation = "CEST"</li>
        /// </ul></div>
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_time_zone::OffsetData> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_OffsetData_OffsetData();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>The time zone offset data for a given moment in time, i.e. the time zone offsets and abbreviation to use at that moment in time.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QTimeZone::OffsetData::OffsetData(const QTimeZone::OffsetData& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The time zone offset data for a given moment in time, i.e. the time zone offsets and abbreviation to use at that moment in time.</p>
        /// <ul>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#atUtc-var">OffsetData::atUtc</a> The datetime of the offset data in UTC time.</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#offsetFromUtc-var">OffsetData::offsetFromUtc</a> The total offset from UTC in effect at the datetime.</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#standardTimeOffset-var">OffsetData::standardTimeOffset</a> The standard time offset component of the total offset.</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#daylightTimeOffset-var">OffsetData::daylightTimeOffset</a> The DST offset component of the total offset.</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#abbreviation-var">OffsetData::abbreviation</a> The abbreviation in effect at the datetime.</li>
        /// </ul>
        /// <p>For example, for time zone "Europe/Berlin" the OffsetDate in standard and DST might be:</p>
        /// <ul>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#atUtc-var">atUtc</a> = <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>(<a href="http://doc.qt.io/qt-5/qdate.html">QDate</a>(2013, 1, 1), <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a>(0, 0, 0), <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>)</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#offsetFromUtc-var">offsetFromUtc</a> = 3600</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#standardTimeOffset-var">standardTimeOffset</a> = 3600</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#daylightTimeOffset-var">daylightTimeOffset</a> = 0</li>
        /// <li>abbreviation = "CET"</li>
        /// </ul>
        /// <ul>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#atUtc-var">atUtc</a> = <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>(<a href="http://doc.qt.io/qt-5/qdate.html">QDate</a>(2013, 6, 1), <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a>(0, 0, 0), <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>)</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#offsetFromUtc-var">offsetFromUtc</a> = 7200</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#standardTimeOffset-var">standardTimeOffset</a> = 3600</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#daylightTimeOffset-var">daylightTimeOffset</a> = 3600</li>
        /// <li>abbreviation = "CEST"</li>
        /// </ul></div>
        pub unsafe fn new_copy(
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>,
        ) -> ::cpp_utils::CppBox<crate::q_time_zone::OffsetData> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_OffsetData_OffsetData1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Returns the value of the <span style='color: green;'>```offsetFromUtc```</span> field.
        pub unsafe fn offset_from_utc(&self) -> ::std::os::raw::c_int {
            crate::__ffi::ctr_qt_core_ffi_QTimeZone_OffsetData_offsetFromUtc(
                self as *const crate::q_time_zone::OffsetData,
            )
        }

        /// Sets the value of the <span style='color: green;'>```abbreviation```</span> field.
        pub unsafe fn set_abbreviation(
            &mut self,
            value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        ) {
            crate::__ffi::ctr_qt_core_ffi_set_QTimeZone_OffsetData_abbreviation(
                self as *mut crate::q_time_zone::OffsetData,
                ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(value)
                    .as_raw_ptr(),
            )
        }

        /// Sets the value of the <span style='color: green;'>```atUtc```</span> field.
        pub unsafe fn set_at_utc(
            &mut self,
            value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
        ) {
            crate::__ffi::ctr_qt_core_ffi_set_QTimeZone_OffsetData_atUtc(
                self as *mut crate::q_time_zone::OffsetData,
                ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(value)
                    .as_raw_ptr(),
            )
        }

        /// Sets the value of the <span style='color: green;'>```daylightTimeOffset```</span> field.
        pub unsafe fn set_daylight_time_offset(&mut self, value: ::std::os::raw::c_int) {
            crate::__ffi::ctr_qt_core_ffi_set_QTimeZone_OffsetData_daylightTimeOffset(
                self as *mut crate::q_time_zone::OffsetData,
                value,
            )
        }

        /// Sets the value of the <span style='color: green;'>```offsetFromUtc```</span> field.
        pub unsafe fn set_offset_from_utc(&mut self, value: ::std::os::raw::c_int) {
            crate::__ffi::ctr_qt_core_ffi_set_QTimeZone_OffsetData_offsetFromUtc(
                self as *mut crate::q_time_zone::OffsetData,
                value,
            )
        }

        /// Sets the value of the <span style='color: green;'>```standardTimeOffset```</span> field.
        pub unsafe fn set_standard_time_offset(&mut self, value: ::std::os::raw::c_int) {
            crate::__ffi::ctr_qt_core_ffi_set_QTimeZone_OffsetData_standardTimeOffset(
                self as *mut crate::q_time_zone::OffsetData,
                value,
            )
        }

        /// Returns the value of the <span style='color: green;'>```standardTimeOffset```</span> field.
        pub unsafe fn standard_time_offset(&self) -> ::std::os::raw::c_int {
            crate::__ffi::ctr_qt_core_ffi_QTimeZone_OffsetData_standardTimeOffset(
                self as *const crate::q_time_zone::OffsetData,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_time_zone::OffsetData {
        /// <p>The time zone offset data for a given moment in time, i.e. the time zone offsets and abbreviation to use at that moment in time.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QTimeZone::OffsetData::~OffsetData()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The time zone offset data for a given moment in time, i.e. the time zone offsets and abbreviation to use at that moment in time.</p>
        /// <ul>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#atUtc-var">OffsetData::atUtc</a> The datetime of the offset data in UTC time.</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#offsetFromUtc-var">OffsetData::offsetFromUtc</a> The total offset from UTC in effect at the datetime.</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#standardTimeOffset-var">OffsetData::standardTimeOffset</a> The standard time offset component of the total offset.</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#daylightTimeOffset-var">OffsetData::daylightTimeOffset</a> The DST offset component of the total offset.</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#abbreviation-var">OffsetData::abbreviation</a> The abbreviation in effect at the datetime.</li>
        /// </ul>
        /// <p>For example, for time zone "Europe/Berlin" the OffsetDate in standard and DST might be:</p>
        /// <ul>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#atUtc-var">atUtc</a> = <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>(<a href="http://doc.qt.io/qt-5/qdate.html">QDate</a>(2013, 1, 1), <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a>(0, 0, 0), <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>)</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#offsetFromUtc-var">offsetFromUtc</a> = 3600</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#standardTimeOffset-var">standardTimeOffset</a> = 3600</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#daylightTimeOffset-var">daylightTimeOffset</a> = 0</li>
        /// <li>abbreviation = "CET"</li>
        /// </ul>
        /// <ul>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#atUtc-var">atUtc</a> = <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>(<a href="http://doc.qt.io/qt-5/qdate.html">QDate</a>(2013, 6, 1), <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a>(0, 0, 0), <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::UTC</a>)</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#offsetFromUtc-var">offsetFromUtc</a> = 7200</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#standardTimeOffset-var">standardTimeOffset</a> = 3600</li>
        /// <li><a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html#daylightTimeOffset-var">daylightTimeOffset</a> = 3600</li>
        /// <li>abbreviation = "CEST"</li>
        /// </ul></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QTimeZone_OffsetData_dOffsetData(
                self as *mut crate::q_time_zone::OffsetData,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qtimezone.html">QTimeZone</a> class converts between UTC and local time in a specific time zone.</p>
///
/// C++ class: <span style='color: green;'>```QTimeZone```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtimezone.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qtimezone.html">QTimeZone</a> class converts between UTC and local time in a specific time zone.</p>
/// <p>This class provides a stateless calculator for time zone conversions between UTC and the local time in a specific time zone. By default it uses the host system time zone data to perform these conversions.</p>
/// <p>This class is primarily designed for use in <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>; most applications will not need to access this class directly and should instead use <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> with a <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeSpec</a> of <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeZone</a>.</p>
/// <p><b>Note: </b>For consistency with <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>, <a href="http://doc.qt.io/qt-5/qtimezone.html">QTimeZone</a> does not account for leap seconds.</p><a name="iana-time-zone-ids"></a>
/// <a name="iana-time-zone-ids"></a>
/// <h4>IANA Time Zone IDs</h4>
/// <p><a href="http://doc.qt.io/qt-5/qtimezone.html">QTimeZone</a> uses the IANA time zone IDs as defined in the IANA Time Zone Database (http://www.iana.org/time-zones). This is to ensure a standard ID across all supported platforms. Most platforms support the IANA IDs and the IANA Database natively, but for Windows a mapping is required to the native IDs. See below for more details.</p>
/// <p>The IANA IDs can and do change on a regular basis, and can vary depending on how recently the host system data was updated. As such you cannot rely on any given ID existing on any host system. You must use <a href="http://doc.qt.io/qt-5/qtimezone.html#availableTimeZoneIds">availableTimeZoneIds</a>() to determine what IANA IDs are available.</p>
/// <p>The IANA IDs and database are also know as the Olson IDs and database, named after their creator.</p>
/// <a name="utc-offset-time-zones"></a>
/// <h4>UTC Offset Time Zones</h4>
/// <p>A default UTC time zone backend is provided which is always guaranteed to be available. This provides a set of generic Offset From UTC time zones in the range UTC-14:00 to UTC+14:00. These time zones can be created using either the standard ISO format names "UTC+00:00" as listed by <a href="http://doc.qt.io/qt-5/qtimezone.html#availableTimeZoneIds">availableTimeZoneIds</a>(), or using the number of offset seconds.</p>
/// <a name="windows-time-zones"></a>
/// <h4>Windows Time Zones</h4>
/// <p>Windows native time zone support is severely limited compared to the standard IANA TZ Database. Windows time zones cover larger geographic areas and are thus less accurate in their conversions. They also do not support as much historic conversion data and so may only be accurate for the current year.</p>
/// <p><a href="http://doc.qt.io/qt-5/qtimezone.html">QTimeZone</a> uses a conversion table derived form the Unicode CLDR data to map between IANA IDs and Windows IDs. Depending on your version of Windows and Qt, this table may not be able to provide a valid conversion, in which "UTC" will be returned.</p>
/// <p><a href="http://doc.qt.io/qt-5/qtimezone.html">QTimeZone</a> provides a public API to use this conversion table. The Windows ID used is the Windows Registry Key for the time zone which is also the MS Exchange EWS ID as well, but is different to the Time Zone Name (TZID) and COD code used by MS Exchange in versions before 2007.</p>
/// <a name="system-time-zone"></a>
/// <h4>System Time Zone</h4>
/// <p><a href="http://doc.qt.io/qt-5/qtimezone.html">QTimeZone</a> does not support any concept of a system or default time zone. If you require a <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> that uses the current system time zone at any given moment then you should use a <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::TimeSpec</a> of <a href="http://doc.qt.io/qt-5/qt.html#TimeSpec-enum">Qt::LocalTime</a>.</p>
/// <p>The method <a href="http://doc.qt.io/qt-5/qtimezone.html#systemTimeZoneId">systemTimeZoneId</a>() returns the current system IANA time zone ID which on Unix-like systems will always be correct. On Windows this ID is translated from the Windows system ID using an internal translation table and the user's selected country. As a consequence there is a small chance any Windows install may have IDs not known by Qt, in which case "UTC" will be returned.</p>
/// <p>Creating a new <a href="http://doc.qt.io/qt-5/qtimezone.html">QTimeZone</a> instance using the system time zone ID will only produce a fixed named copy of the time zone, it will not change if the system time zone changes.</p>
/// <a name="time-zone-offsets"></a>
/// <h4>Time Zone Offsets</h4>
/// <p>The difference between UTC and the local time in a time zone is expressed as an offset in seconds from UTC, i.e. the number of seconds to add to UTC to obtain the local time. The total offset is comprised of two component parts, the standard time offset and the daylight-saving time offset. The standard time offset is the number of seconds to add to UTC to obtain standard time in the time zone. The daylight-saving time offset is the number of seconds to add to the standard time offset to obtain daylight-saving time (abbreviated DST and sometimes called "daylight time" or "summer time") in the time zone.</p>
/// <p>Note that the standard and DST offsets for a time zone may change over time as countries have changed DST laws or even their standard time offset.</p>
/// <a name="license"></a>
/// <h4>License</h4>
/// <p>This class includes data obtained from the CLDR data files under the terms of the Unicode Data Files and Software License. See Unicode CLDR (Unicode Common Locale Data Repository) for the details.</p></div>
#[repr(C)]
pub struct QTimeZone {
    _unused: u8,
}
impl QTimeZone {
    /// <p>Returns the time zone abbreviation at the given <i>atDateTime</i>. The abbreviation may change depending on DST or even historical events.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QTimeZone::abbreviation(const QDateTime& atDateTime) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#abbreviation">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the time zone abbreviation at the given <i>atDateTime</i>. The abbreviation may change depending on DST or even historical events.</p>
    /// <p>Note that the abbreviation is not guaranteed to be unique to this time zone and should not be used in place of the ID or display name.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#displayName">displayName</a>().</p></div>
    pub unsafe fn abbreviation(
        &self,
        at_date_time: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_abbreviation(
            self as *const crate::QTimeZone,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(at_date_time)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list of all available IANA time zone IDs on this system.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QList<QByteArray> QTimeZone::availableTimeZoneIds()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#availableTimeZoneIds">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of all available IANA time zone IDs on this system.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#isTimeZoneIdAvailable">isTimeZoneIdAvailable</a>().</p></div>
    pub unsafe fn available_time_zone_ids() -> ::cpp_utils::CppBox<crate::QListOfQByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_availableTimeZoneIds();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list of all available IANA time zone IDs for a given <i>country</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QList<QByteArray> QTimeZone::availableTimeZoneIds(QLocale::Country country)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#availableTimeZoneIds-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of all available IANA time zone IDs for a given <i>country</i>.</p>
    /// <p>As a special case, a <i>country</i> of Qt::AnyCountry returns those time zones that do not have any country related to them, such as UTC. If you require a list of all time zone IDs for all countries then use the standard <a href="http://doc.qt.io/qt-5/qtimezone.html#availableTimeZoneIds">availableTimeZoneIds</a>() method.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#isTimeZoneIdAvailable">isTimeZoneIdAvailable</a>().</p></div>
    pub unsafe fn available_time_zone_ids_country(
        country: crate::q_locale::Country,
    ) -> ::cpp_utils::CppBox<crate::QListOfQByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_availableTimeZoneIds1(country);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list of all available IANA time zone IDs with a given standard time offset of <i>offsetSeconds</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QList<QByteArray> QTimeZone::availableTimeZoneIds(int offsetSeconds)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#availableTimeZoneIds-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of all available IANA time zone IDs with a given standard time offset of <i>offsetSeconds</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#isTimeZoneIdAvailable">isTimeZoneIdAvailable</a>().</p></div>
    pub unsafe fn available_time_zone_ids_int(
        offset_seconds: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QTimeZone_availableTimeZoneIds2(offset_seconds);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns any comment for the time zone.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QTimeZone::comment() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#comment">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns any comment for the time zone.</p>
    /// <p>A comment may be provided by the host platform to assist users in choosing the correct time zone. Depending on the platform this may not be localized.</p></div>
    pub unsafe fn comment(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QTimeZone_comment(self as *const crate::QTimeZone);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Assignment operator, assign <i>other</i> to this.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTimeZone& QTimeZone::operator=(const QTimeZone& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assignment operator, assign <i>other</i> to this.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTimeZone>>,
    ) -> ::cpp_utils::MutRef<crate::QTimeZone> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_operator_(
            self as *mut crate::QTimeZone,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTimeZone>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the country for the time zone.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale::Country QTimeZone::country() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#country">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the country for the time zone.</p></div>
    pub unsafe fn country(&self) -> crate::q_locale::Country {
        crate::__ffi::ctr_qt_core_ffi_QTimeZone_country(self as *const crate::QTimeZone)
    }

    /// <p>Returns the daylight-saving time offset at the given <i>atDateTime</i>, i.e. the number of seconds to add to the standard time offset to obtain the local daylight-saving time.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTimeZone::daylightTimeOffset(const QDateTime& atDateTime) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#daylightTimeOffset">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the daylight-saving time offset at the given <i>atDateTime</i>, i.e. the number of seconds to add to the standard time offset to obtain the local daylight-saving time.</p>
    /// <p>For example, for the time zone "Europe/Berlin" the DST offset is +3600 seconds. During standard time daylightTimeOffset() will return 0, and when daylight-saving is in effect it will return +3600.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#offsetFromUtc">offsetFromUtc</a>() and <a href="http://doc.qt.io/qt-5/qtimezone.html#standardTimeOffset">standardTimeOffset</a>().</p></div>
    pub unsafe fn daylight_time_offset(
        &self,
        at_date_time: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTimeZone_daylightTimeOffset(
            self as *const crate::QTimeZone,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(at_date_time)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the localized time zone display name at the given <i>atDateTime</i> for the given <i>nameType</i> in the given <i>locale</i>. The <i>nameType</i> and <i>locale</i> requested may not be supported on all platforms, in which case the best available option will be returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QTimeZone::displayName(const QDateTime& atDateTime, QTimeZone::NameType nameType = …, const QLocale& locale = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#displayName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the localized time zone display name at the given <i>atDateTime</i> for the given <i>nameType</i> in the given <i>locale</i>. The <i>nameType</i> and <i>locale</i> requested may not be supported on all platforms, in which case the best available option will be returned.</p>
    /// <p>If the <i>locale</i> is not provided then the application default locale will be used.</p>
    /// <p>The display name may change depending on DST or historical events.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#abbreviation">abbreviation</a>().</p></div>
    pub unsafe fn display_name_q_date_time_name_type_q_locale(
        &self,
        at_date_time: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
        name_type: crate::q_time_zone::NameType,
        locale: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_displayName(
            self as *const crate::QTimeZone,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(at_date_time)
                .as_raw_ptr(),
            name_type,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(locale)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the localized time zone display name for the given <i>timeType</i> and <i>nameType</i> in the given <i>locale</i>. The <i>nameType</i> and <i>locale</i> requested may not be supported on all platforms, in which case the best available option will be returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QTimeZone::displayName(QTimeZone::TimeType timeType, QTimeZone::NameType nameType = …, const QLocale& locale = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#displayName-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the localized time zone display name for the given <i>timeType</i> and <i>nameType</i> in the given <i>locale</i>. The <i>nameType</i> and <i>locale</i> requested may not be supported on all platforms, in which case the best available option will be returned.</p>
    /// <p>If the <i>locale</i> is not provided then the application default locale will be used.</p>
    /// <p>Where the time zone display names have changed over time then the most recent names will be used.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#abbreviation">abbreviation</a>().</p></div>
    pub unsafe fn display_name_time_type_name_type_q_locale(
        &self,
        time_type: crate::q_time_zone::TimeType,
        name_type: crate::q_time_zone::NameType,
        locale: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_displayName1(
            self as *const crate::QTimeZone,
            time_type,
            name_type,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(locale)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the localized time zone display name at the given <i>atDateTime</i> for the given <i>nameType</i> in the given <i>locale</i>. The <i>nameType</i> and <i>locale</i> requested may not be supported on all platforms, in which case the best available option will be returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QTimeZone::displayName(const QDateTime& atDateTime, QTimeZone::NameType nameType = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#displayName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the localized time zone display name at the given <i>atDateTime</i> for the given <i>nameType</i> in the given <i>locale</i>. The <i>nameType</i> and <i>locale</i> requested may not be supported on all platforms, in which case the best available option will be returned.</p>
    /// <p>If the <i>locale</i> is not provided then the application default locale will be used.</p>
    /// <p>The display name may change depending on DST or historical events.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#abbreviation">abbreviation</a>().</p></div>
    pub unsafe fn display_name_q_date_time_name_type(
        &self,
        at_date_time: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
        name_type: crate::q_time_zone::NameType,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_displayName2(
            self as *const crate::QTimeZone,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(at_date_time)
                .as_raw_ptr(),
            name_type,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the localized time zone display name at the given <i>atDateTime</i> for the given <i>nameType</i> in the given <i>locale</i>. The <i>nameType</i> and <i>locale</i> requested may not be supported on all platforms, in which case the best available option will be returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QTimeZone::displayName(const QDateTime& atDateTime) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#displayName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the localized time zone display name at the given <i>atDateTime</i> for the given <i>nameType</i> in the given <i>locale</i>. The <i>nameType</i> and <i>locale</i> requested may not be supported on all platforms, in which case the best available option will be returned.</p>
    /// <p>If the <i>locale</i> is not provided then the application default locale will be used.</p>
    /// <p>The display name may change depending on DST or historical events.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#abbreviation">abbreviation</a>().</p></div>
    pub unsafe fn display_name_q_date_time(
        &self,
        at_date_time: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_displayName3(
            self as *const crate::QTimeZone,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(at_date_time)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the localized time zone display name for the given <i>timeType</i> and <i>nameType</i> in the given <i>locale</i>. The <i>nameType</i> and <i>locale</i> requested may not be supported on all platforms, in which case the best available option will be returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QTimeZone::displayName(QTimeZone::TimeType timeType, QTimeZone::NameType nameType = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#displayName-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the localized time zone display name for the given <i>timeType</i> and <i>nameType</i> in the given <i>locale</i>. The <i>nameType</i> and <i>locale</i> requested may not be supported on all platforms, in which case the best available option will be returned.</p>
    /// <p>If the <i>locale</i> is not provided then the application default locale will be used.</p>
    /// <p>Where the time zone display names have changed over time then the most recent names will be used.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#abbreviation">abbreviation</a>().</p></div>
    pub unsafe fn display_name_time_type_name_type(
        &self,
        time_type: crate::q_time_zone::TimeType,
        name_type: crate::q_time_zone::NameType,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_displayName4(
            self as *const crate::QTimeZone,
            time_type,
            name_type,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the localized time zone display name for the given <i>timeType</i> and <i>nameType</i> in the given <i>locale</i>. The <i>nameType</i> and <i>locale</i> requested may not be supported on all platforms, in which case the best available option will be returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QTimeZone::displayName(QTimeZone::TimeType timeType) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#displayName-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the localized time zone display name for the given <i>timeType</i> and <i>nameType</i> in the given <i>locale</i>. The <i>nameType</i> and <i>locale</i> requested may not be supported on all platforms, in which case the best available option will be returned.</p>
    /// <p>If the <i>locale</i> is not provided then the application default locale will be used.</p>
    /// <p>Where the time zone display names have changed over time then the most recent names will be used.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#abbreviation">abbreviation</a>().</p></div>
    pub unsafe fn display_name_time_type(
        &self,
        time_type: crate::q_time_zone::TimeType,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_displayName5(
            self as *const crate::QTimeZone,
            time_type,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if the time zone has practiced daylight-saving at any time.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTimeZone::hasDaylightTime() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#hasDaylightTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the time zone has practiced daylight-saving at any time.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#isDaylightTime">isDaylightTime</a>() and <a href="http://doc.qt.io/qt-5/qtimezone.html#daylightTimeOffset">daylightTimeOffset</a>().</p></div>
    pub unsafe fn has_daylight_time(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTimeZone_hasDaylightTime(self as *const crate::QTimeZone)
    }

    /// <p>Returns <code>true</code> if the system backend supports obtaining transitions.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTimeZone::hasTransitions() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#hasTransitions">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the system backend supports obtaining transitions.</p>
    /// <p>Transitions are changes in the time-zone: these happen when DST turns on or off and when authorities alter the offsets for the time-zone.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#nextTransition">nextTransition</a>(), <a href="http://doc.qt.io/qt-5/qtimezone.html#previousTransition">previousTransition</a>(), and <a href="http://doc.qt.io/qt-5/qtimezone.html#transitions">transitions</a>().</p></div>
    pub unsafe fn has_transitions(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTimeZone_hasTransitions(self as *const crate::QTimeZone)
    }

    /// <p>Returns the Windows ID equivalent to the given <i>ianaId</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QTimeZone::ianaIdToWindowsId(const QByteArray& ianaId)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#ianaIdToWindowsId">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the Windows ID equivalent to the given <i>ianaId</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#windowsIdToDefaultIanaId">windowsIdToDefaultIanaId</a>() and <a href="http://doc.qt.io/qt-5/qtimezone.html#windowsIdToIanaIds">windowsIdToIanaIds</a>().</p></div>
    pub unsafe fn iana_id_to_windows_id(
        iana_id: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_ianaIdToWindowsId(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(iana_id)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the IANA ID for the time zone.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QTimeZone::id() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#id">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the IANA ID for the time zone.</p>
    /// <p>IANA IDs are used on all platforms. On Windows these are translated from the Windows ID into the closest IANA ID for the time zone and country.</p></div>
    pub unsafe fn id(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QTimeZone_id(self as *const crate::QTimeZone);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if daylight-saving was in effect at the given <i>atDateTime</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTimeZone::isDaylightTime(const QDateTime& atDateTime) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#isDaylightTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if daylight-saving was in effect at the given <i>atDateTime</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#hasDaylightTime">hasDaylightTime</a>() and <a href="http://doc.qt.io/qt-5/qtimezone.html#daylightTimeOffset">daylightTimeOffset</a>().</p></div>
    pub unsafe fn is_daylight_time(
        &self,
        at_date_time: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTimeZone_isDaylightTime(
            self as *const crate::QTimeZone,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(at_date_time)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if a given time zone <i>ianaId</i> is available on this system.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static bool QTimeZone::isTimeZoneIdAvailable(const QByteArray& ianaId)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#isTimeZoneIdAvailable">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if a given time zone <i>ianaId</i> is available on this system.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#availableTimeZoneIds">availableTimeZoneIds</a>().</p></div>
    pub unsafe fn is_time_zone_id_available(
        iana_id: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTimeZone_isTimeZoneIdAvailable(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(iana_id)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if this time zone is valid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTimeZone::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this time zone is valid.</p></div>
    pub unsafe fn is_valid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTimeZone_isValid(self as *const crate::QTimeZone)
    }

    /// <p>Create a null/invalid time zone instance.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTimeZone::QTimeZone()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#QTimeZone">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Create a null/invalid time zone instance.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QTimeZone> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_QTimeZone();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates an instance of the requested time zone <i>ianaId</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTimeZone::QTimeZone(const QByteArray& ianaId)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#QTimeZone-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates an instance of the requested time zone <i>ianaId</i>.</p>
    /// <p>The ID must be one of the available system IDs otherwise an invalid time zone will be returned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#availableTimeZoneIds">availableTimeZoneIds</a>().</p></div>
    pub unsafe fn from_q_byte_array(
        iana_id: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QTimeZone> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_QTimeZone1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(iana_id)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates an instance of a time zone with the requested Offset from UTC of <i>offsetSeconds</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTimeZone::QTimeZone(int offsetSeconds)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#QTimeZone-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates an instance of a time zone with the requested Offset from UTC of <i>offsetSeconds</i>.</p>
    /// <p>The <i>offsetSeconds</i> from UTC must be in the range -14 hours to +14 hours otherwise an invalid time zone will be returned.</p></div>
    pub unsafe fn from_int(
        offset_seconds: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QTimeZone> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_QTimeZone2(offset_seconds);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a custom time zone with an ID of <i>ianaId</i> and an offset from UTC of <i>offsetSeconds</i>. The <i>name</i> will be the name used by <a href="http://doc.qt.io/qt-5/qtimezone.html#displayName">displayName</a>() for the <a href="http://doc.qt.io/qt-5/qtimezone.html#NameType-enum">LongName</a>, the <i>abbreviation</i> will be used by <a href="http://doc.qt.io/qt-5/qtimezone.html#displayName">displayName</a>() for the <a href="http://doc.qt.io/qt-5/qtimezone.html#NameType-enum">ShortName</a> and by <a href="http://doc.qt.io/qt-5/qtimezone.html#abbreviation">abbreviation</a>(), and the optional <i>country</i> will be used by <a href="http://doc.qt.io/qt-5/qtimezone.html#country">country</a>(). The <i>comment</i> is an optional note that may be displayed in a GUI to assist users in selecting a time zone.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTimeZone::QTimeZone(const QByteArray& zoneId, int offsetSeconds, const QString& name, const QString& abbreviation, QLocale::Country country = …, const QString& comment = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#QTimeZone-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a custom time zone with an ID of <i>ianaId</i> and an offset from UTC of <i>offsetSeconds</i>. The <i>name</i> will be the name used by <a href="http://doc.qt.io/qt-5/qtimezone.html#displayName">displayName</a>() for the <a href="http://doc.qt.io/qt-5/qtimezone.html#NameType-enum">LongName</a>, the <i>abbreviation</i> will be used by <a href="http://doc.qt.io/qt-5/qtimezone.html#displayName">displayName</a>() for the <a href="http://doc.qt.io/qt-5/qtimezone.html#NameType-enum">ShortName</a> and by <a href="http://doc.qt.io/qt-5/qtimezone.html#abbreviation">abbreviation</a>(), and the optional <i>country</i> will be used by <a href="http://doc.qt.io/qt-5/qtimezone.html#country">country</a>(). The <i>comment</i> is an optional note that may be displayed in a GUI to assist users in selecting a time zone.</p>
    /// <p>The <i>ianaId</i> must not be one of the available system IDs returned by <a href="http://doc.qt.io/qt-5/qtimezone.html#availableTimeZoneIds">availableTimeZoneIds</a>(). The <i>offsetSeconds</i> from UTC must be in the range -14 hours to +14 hours.</p>
    /// <p>If the custom time zone does not have a specific country then set it to the default value of <a href="http://doc.qt.io/qt-5/qlocale.html#Country-enum">QLocale::AnyCountry</a>.</p></div>
    pub unsafe fn from_q_byte_array_int2_q_string_country_q_string(
        zone_id: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        offset_seconds: ::std::os::raw::c_int,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        abbreviation: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        country: crate::q_locale::Country,
        comment: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QTimeZone> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_QTimeZone3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(zone_id)
                .as_raw_ptr(),
            offset_seconds,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(abbreviation)
                .as_raw_ptr(),
            country,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(comment)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a custom time zone with an ID of <i>ianaId</i> and an offset from UTC of <i>offsetSeconds</i>. The <i>name</i> will be the name used by <a href="http://doc.qt.io/qt-5/qtimezone.html#displayName">displayName</a>() for the <a href="http://doc.qt.io/qt-5/qtimezone.html#NameType-enum">LongName</a>, the <i>abbreviation</i> will be used by <a href="http://doc.qt.io/qt-5/qtimezone.html#displayName">displayName</a>() for the <a href="http://doc.qt.io/qt-5/qtimezone.html#NameType-enum">ShortName</a> and by <a href="http://doc.qt.io/qt-5/qtimezone.html#abbreviation">abbreviation</a>(), and the optional <i>country</i> will be used by <a href="http://doc.qt.io/qt-5/qtimezone.html#country">country</a>(). The <i>comment</i> is an optional note that may be displayed in a GUI to assist users in selecting a time zone.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTimeZone::QTimeZone(const QByteArray& zoneId, int offsetSeconds, const QString& name, const QString& abbreviation, QLocale::Country country = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#QTimeZone-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a custom time zone with an ID of <i>ianaId</i> and an offset from UTC of <i>offsetSeconds</i>. The <i>name</i> will be the name used by <a href="http://doc.qt.io/qt-5/qtimezone.html#displayName">displayName</a>() for the <a href="http://doc.qt.io/qt-5/qtimezone.html#NameType-enum">LongName</a>, the <i>abbreviation</i> will be used by <a href="http://doc.qt.io/qt-5/qtimezone.html#displayName">displayName</a>() for the <a href="http://doc.qt.io/qt-5/qtimezone.html#NameType-enum">ShortName</a> and by <a href="http://doc.qt.io/qt-5/qtimezone.html#abbreviation">abbreviation</a>(), and the optional <i>country</i> will be used by <a href="http://doc.qt.io/qt-5/qtimezone.html#country">country</a>(). The <i>comment</i> is an optional note that may be displayed in a GUI to assist users in selecting a time zone.</p>
    /// <p>The <i>ianaId</i> must not be one of the available system IDs returned by <a href="http://doc.qt.io/qt-5/qtimezone.html#availableTimeZoneIds">availableTimeZoneIds</a>(). The <i>offsetSeconds</i> from UTC must be in the range -14 hours to +14 hours.</p>
    /// <p>If the custom time zone does not have a specific country then set it to the default value of <a href="http://doc.qt.io/qt-5/qlocale.html#Country-enum">QLocale::AnyCountry</a>.</p></div>
    pub unsafe fn from_q_byte_array_int2_q_string_country(
        zone_id: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        offset_seconds: ::std::os::raw::c_int,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        abbreviation: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        country: crate::q_locale::Country,
    ) -> ::cpp_utils::CppBox<crate::QTimeZone> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_QTimeZone5(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(zone_id)
                .as_raw_ptr(),
            offset_seconds,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(abbreviation)
                .as_raw_ptr(),
            country,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a custom time zone with an ID of <i>ianaId</i> and an offset from UTC of <i>offsetSeconds</i>. The <i>name</i> will be the name used by <a href="http://doc.qt.io/qt-5/qtimezone.html#displayName">displayName</a>() for the <a href="http://doc.qt.io/qt-5/qtimezone.html#NameType-enum">LongName</a>, the <i>abbreviation</i> will be used by <a href="http://doc.qt.io/qt-5/qtimezone.html#displayName">displayName</a>() for the <a href="http://doc.qt.io/qt-5/qtimezone.html#NameType-enum">ShortName</a> and by <a href="http://doc.qt.io/qt-5/qtimezone.html#abbreviation">abbreviation</a>(), and the optional <i>country</i> will be used by <a href="http://doc.qt.io/qt-5/qtimezone.html#country">country</a>(). The <i>comment</i> is an optional note that may be displayed in a GUI to assist users in selecting a time zone.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTimeZone::QTimeZone(const QByteArray& zoneId, int offsetSeconds, const QString& name, const QString& abbreviation)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#QTimeZone-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a custom time zone with an ID of <i>ianaId</i> and an offset from UTC of <i>offsetSeconds</i>. The <i>name</i> will be the name used by <a href="http://doc.qt.io/qt-5/qtimezone.html#displayName">displayName</a>() for the <a href="http://doc.qt.io/qt-5/qtimezone.html#NameType-enum">LongName</a>, the <i>abbreviation</i> will be used by <a href="http://doc.qt.io/qt-5/qtimezone.html#displayName">displayName</a>() for the <a href="http://doc.qt.io/qt-5/qtimezone.html#NameType-enum">ShortName</a> and by <a href="http://doc.qt.io/qt-5/qtimezone.html#abbreviation">abbreviation</a>(), and the optional <i>country</i> will be used by <a href="http://doc.qt.io/qt-5/qtimezone.html#country">country</a>(). The <i>comment</i> is an optional note that may be displayed in a GUI to assist users in selecting a time zone.</p>
    /// <p>The <i>ianaId</i> must not be one of the available system IDs returned by <a href="http://doc.qt.io/qt-5/qtimezone.html#availableTimeZoneIds">availableTimeZoneIds</a>(). The <i>offsetSeconds</i> from UTC must be in the range -14 hours to +14 hours.</p>
    /// <p>If the custom time zone does not have a specific country then set it to the default value of <a href="http://doc.qt.io/qt-5/qlocale.html#Country-enum">QLocale::AnyCountry</a>.</p></div>
    pub unsafe fn from_q_byte_array_int2_q_string(
        zone_id: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        offset_seconds: ::std::os::raw::c_int,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        abbreviation: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QTimeZone> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_QTimeZone6(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(zone_id)
                .as_raw_ptr(),
            offset_seconds,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(abbreviation)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Copy constructor, copy <i>other</i> to this.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTimeZone::QTimeZone(const QTimeZone& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#QTimeZone-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Copy constructor, copy <i>other</i> to this.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTimeZone>>,
    ) -> ::cpp_utils::CppBox<crate::QTimeZone> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_QTimeZone4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTimeZone>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the first time zone Transition after the given <i>afterDateTime</i>. This is most useful when you have a Transition time and wish to find the Transition after it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTimeZone::OffsetData QTimeZone::nextTransition(const QDateTime& afterDateTime) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#nextTransition">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the first time zone Transition after the given <i>afterDateTime</i>. This is most useful when you have a Transition time and wish to find the Transition after it.</p>
    /// <p>If there is no transition after the given <i>afterDateTime</i> then an invalid <a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html">OffsetData</a> will be returned with an invalid <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>.</p>
    /// <p>The given <i>afterDateTime</i> is exclusive.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#hasTransitions">hasTransitions</a>(), <a href="http://doc.qt.io/qt-5/qtimezone.html#previousTransition">previousTransition</a>(), and <a href="http://doc.qt.io/qt-5/qtimezone.html#transitions">transitions</a>().</p></div>
    pub unsafe fn next_transition(
        &self,
        after_date_time: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
    ) -> ::cpp_utils::CppBox<crate::q_time_zone::OffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_nextTransition(
            self as *const crate::QTimeZone,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(after_date_time)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the effective offset details at the given <i>forDateTime</i>. This is the equivalent of calling <a href="http://doc.qt.io/qt-5/qtimezone.html#offsetFromUtc">offsetFromUtc</a>(), <a href="http://doc.qt.io/qt-5/qtimezone.html#abbreviation">abbreviation</a>(), etc individually but is more efficient.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTimeZone::OffsetData QTimeZone::offsetData(const QDateTime& forDateTime) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#offsetData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the effective offset details at the given <i>forDateTime</i>. This is the equivalent of calling <a href="http://doc.qt.io/qt-5/qtimezone.html#offsetFromUtc">offsetFromUtc</a>(), <a href="http://doc.qt.io/qt-5/qtimezone.html#abbreviation">abbreviation</a>(), etc individually but is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#offsetFromUtc">offsetFromUtc</a>(), <a href="http://doc.qt.io/qt-5/qtimezone.html#standardTimeOffset">standardTimeOffset</a>(), <a href="http://doc.qt.io/qt-5/qtimezone.html#daylightTimeOffset">daylightTimeOffset</a>(), and <a href="http://doc.qt.io/qt-5/qtimezone.html#abbreviation">abbreviation</a>().</p></div>
    pub unsafe fn offset_data(
        &self,
        for_date_time: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
    ) -> ::cpp_utils::CppBox<crate::q_time_zone::OffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_offsetData(
            self as *const crate::QTimeZone,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(for_date_time)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the total effective offset at the given <i>atDateTime</i>, i.e. the number of seconds to add to UTC to obtain the local time. This includes any DST offset that may be in effect, i.e. it is the sum of <a href="http://doc.qt.io/qt-5/qtimezone.html#standardTimeOffset">standardTimeOffset</a>() and <a href="http://doc.qt.io/qt-5/qtimezone.html#daylightTimeOffset">daylightTimeOffset</a>() for the given datetime.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTimeZone::offsetFromUtc(const QDateTime& atDateTime) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#offsetFromUtc">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the total effective offset at the given <i>atDateTime</i>, i.e. the number of seconds to add to UTC to obtain the local time. This includes any DST offset that may be in effect, i.e. it is the sum of <a href="http://doc.qt.io/qt-5/qtimezone.html#standardTimeOffset">standardTimeOffset</a>() and <a href="http://doc.qt.io/qt-5/qtimezone.html#daylightTimeOffset">daylightTimeOffset</a>() for the given datetime.</p>
    /// <p>For example, for the time zone "Europe/Berlin" the standard time offset is +3600 seconds and the DST offset is +3600 seconds. During standard time offsetFromUtc() will return +3600 (UTC+01:00), and during DST it will return +7200 (UTC+02:00).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#standardTimeOffset">standardTimeOffset</a>() and <a href="http://doc.qt.io/qt-5/qtimezone.html#daylightTimeOffset">daylightTimeOffset</a>().</p></div>
    pub unsafe fn offset_from_utc(
        &self,
        at_date_time: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTimeZone_offsetFromUtc(
            self as *const crate::QTimeZone,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(at_date_time)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the first time zone Transition before the given <i>beforeDateTime</i>. This is most useful when you have a Transition time and wish to find the Transition before it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTimeZone::OffsetData QTimeZone::previousTransition(const QDateTime& beforeDateTime) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#previousTransition">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the first time zone Transition before the given <i>beforeDateTime</i>. This is most useful when you have a Transition time and wish to find the Transition before it.</p>
    /// <p>If there is no transition before the given <i>beforeDateTime</i> then an invalid <a href="http://doc.qt.io/qt-5/qtimezone-offsetdata.html">OffsetData</a> will be returned with an invalid <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>.</p>
    /// <p>The given <i>beforeDateTime</i> is exclusive.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#hasTransitions">hasTransitions</a>(), <a href="http://doc.qt.io/qt-5/qtimezone.html#nextTransition">nextTransition</a>(), and <a href="http://doc.qt.io/qt-5/qtimezone.html#transitions">transitions</a>().</p></div>
    pub unsafe fn previous_transition(
        &self,
        before_date_time: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
    ) -> ::cpp_utils::CppBox<crate::q_time_zone::OffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_previousTransition(
            self as *const crate::QTimeZone,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(
                before_date_time,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the standard time offset at the given <i>atDateTime</i>, i.e. the number of seconds to add to UTC to obtain the local Standard Time. This excludes any DST offset that may be in effect.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QTimeZone::standardTimeOffset(const QDateTime& atDateTime) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#standardTimeOffset">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the standard time offset at the given <i>atDateTime</i>, i.e. the number of seconds to add to UTC to obtain the local Standard Time. This excludes any DST offset that may be in effect.</p>
    /// <p>For example, for the time zone "Europe/Berlin" the standard time offset is +3600 seconds. During both standard and DST <a href="http://doc.qt.io/qt-5/qtimezone.html#offsetFromUtc">offsetFromUtc</a>() will return +3600 (UTC+01:00).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#offsetFromUtc">offsetFromUtc</a>() and <a href="http://doc.qt.io/qt-5/qtimezone.html#daylightTimeOffset">daylightTimeOffset</a>().</p></div>
    pub unsafe fn standard_time_offset(
        &self,
        at_date_time: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTimeZone_standardTimeOffset(
            self as *const crate::QTimeZone,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(at_date_time)
                .as_raw_ptr(),
        )
    }

    /// <p>Swaps this time zone instance with <i>other</i>. This function is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QTimeZone::swap(QTimeZone& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps this time zone instance with <i>other</i>. This function is very fast and never fails.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QTimeZone>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QTimeZone_swap(
            self as *mut crate::QTimeZone,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QTimeZone>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qtimezone.html">QTimeZone</a> object that refers to the local system time, as specified by <a href="http://doc.qt.io/qt-5/qtimezone.html#systemTimeZoneId">systemTimeZoneId</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QTimeZone QTimeZone::systemTimeZone()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#systemTimeZone">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qtimezone.html">QTimeZone</a> object that refers to the local system time, as specified by <a href="http://doc.qt.io/qt-5/qtimezone.html#systemTimeZoneId">systemTimeZoneId</a>().</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#utc">utc</a>().</p></div>
    pub unsafe fn system_time_zone() -> ::cpp_utils::CppBox<crate::QTimeZone> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_systemTimeZone();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the current system time zone IANA ID.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QTimeZone::systemTimeZoneId()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#systemTimeZoneId">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current system time zone IANA ID.</p>
    /// <p>On Windows this ID is translated from the Windows ID using an internal translation table and the user's selected country. As a consequence there is a small chance any Windows install may have IDs not known by Qt, in which case "UTC" will be returned.</p></div>
    pub unsafe fn system_time_zone_id() -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_systemTimeZoneId();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list of all time zone transitions between the given datetimes.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QTimeZone::OffsetData> QTimeZone::transitions(const QDateTime& fromDateTime, const QDateTime& toDateTime) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#transitions">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of all time zone transitions between the given datetimes.</p>
    /// <p>The given <i>fromDateTime</i> and <i>toDateTime</i> are inclusive.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#hasTransitions">hasTransitions</a>(), <a href="http://doc.qt.io/qt-5/qtimezone.html#nextTransition">nextTransition</a>(), and <a href="http://doc.qt.io/qt-5/qtimezone.html#previousTransition">previousTransition</a>().</p></div>
    pub unsafe fn transitions(
        &self,
        from_date_time: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
        to_date_time: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfOffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_transitions(
            self as *const crate::QTimeZone,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(from_date_time)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(to_date_time)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qtimezone.html">QTimeZone</a> object that refers to UTC (Universal Time Coordinated).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QTimeZone QTimeZone::utc()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#utc">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qtimezone.html">QTimeZone</a> object that refers to UTC (Universal Time Coordinated).</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#systemTimeZone">systemTimeZone</a>().</p></div>
    pub unsafe fn utc() -> ::cpp_utils::CppBox<crate::QTimeZone> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_utc();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the default IANA ID for a given <i>windowsId</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QTimeZone::windowsIdToDefaultIanaId(const QByteArray& windowsId)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#windowsIdToDefaultIanaId">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the default IANA ID for a given <i>windowsId</i>.</p>
    /// <p>Because a Windows ID can cover several IANA IDs in several different countries, this function returns the most frequently used IANA ID with no regard for the country and should thus be used with care. It is usually best to request the default for a specific country.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#ianaIdToWindowsId">ianaIdToWindowsId</a>() and <a href="http://doc.qt.io/qt-5/qtimezone.html#windowsIdToIanaIds">windowsIdToIanaIds</a>().</p></div>
    pub unsafe fn windows_id_to_default_iana_id_1a(
        windows_id: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_windowsIdToDefaultIanaId(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(windows_id)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the default IANA ID for a given <i>windowsId</i> and <i>country</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QTimeZone::windowsIdToDefaultIanaId(const QByteArray& windowsId, QLocale::Country country)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#windowsIdToDefaultIanaId-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the default IANA ID for a given <i>windowsId</i> and <i>country</i>.</p>
    /// <p>Because a Windows ID can cover several IANA IDs within a given country, the most frequently used IANA ID in that country is returned.</p>
    /// <p>As a special case, <a href="http://doc.qt.io/qt-5/qlocale.html#Country-enum">QLocale::AnyCountry</a> returns the default of those IANA IDs that do not have any specific country.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#ianaIdToWindowsId">ianaIdToWindowsId</a>() and <a href="http://doc.qt.io/qt-5/qtimezone.html#windowsIdToIanaIds">windowsIdToIanaIds</a>().</p></div>
    pub unsafe fn windows_id_to_default_iana_id_2a(
        windows_id: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        country: crate::q_locale::Country,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_windowsIdToDefaultIanaId1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(windows_id)
                .as_raw_ptr(),
            country,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns all the IANA IDs for a given <i>windowsId</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QList<QByteArray> QTimeZone::windowsIdToIanaIds(const QByteArray& windowsId)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#windowsIdToIanaIds">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns all the IANA IDs for a given <i>windowsId</i>.</p>
    /// <p>The returned list is sorted alphabetically.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#ianaIdToWindowsId">ianaIdToWindowsId</a>() and <a href="http://doc.qt.io/qt-5/qtimezone.html#windowsIdToDefaultIanaId">windowsIdToDefaultIanaId</a>().</p></div>
    pub unsafe fn windows_id_to_iana_ids_1a(
        windows_id: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_windowsIdToIanaIds(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(windows_id)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns all the IANA IDs for a given <i>windowsId</i> and <i>country</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QList<QByteArray> QTimeZone::windowsIdToIanaIds(const QByteArray& windowsId, QLocale::Country country)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#windowsIdToIanaIds-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns all the IANA IDs for a given <i>windowsId</i> and <i>country</i>.</p>
    /// <p>As a special case <a href="http://doc.qt.io/qt-5/qlocale.html#Country-enum">QLocale::AnyCountry</a> returns those IANA IDs that do not have any specific country.</p>
    /// <p>The returned list is in order of frequency of usage, i.e. larger zones within a country are listed first.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtimezone.html#ianaIdToWindowsId">ianaIdToWindowsId</a>() and <a href="http://doc.qt.io/qt-5/qtimezone.html#windowsIdToDefaultIanaId">windowsIdToDefaultIanaId</a>().</p></div>
    pub unsafe fn windows_id_to_iana_ids_2a(
        windows_id: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        country: crate::q_locale::Country,
    ) -> ::cpp_utils::CppBox<crate::QListOfQByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTimeZone_windowsIdToIanaIds1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(windows_id)
                .as_raw_ptr(),
            country,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qtranslator.html">QTranslator</a> class provides internationalization support for text output.</p>
///
/// C++ class: <span style='color: green;'>```QTranslator```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtranslator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qtranslator.html">QTranslator</a> class provides internationalization support for text output.</p>
/// <p>An object of this class contains a set of translations from a source language to a target language. <a href="http://doc.qt.io/qt-5/qtranslator.html">QTranslator</a> provides functions to look up translations in a translation file. Translation files are created using <a href="http://doc.qt.io/qt-5/qtlinguist-index.html">Qt Linguist</a>.</p>
/// <p>The most common use of <a href="http://doc.qt.io/qt-5/qtranslator.html">QTranslator</a> is to: load a translation file, install it using <a href="http://doc.qt.io/qt-5/qcoreapplication.html#installTranslator">QCoreApplication::installTranslator</a>(), and use it via <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(). Here's an example <code>main()</code> function using the <a href="http://doc.qt.io/qt-5/qtranslator.html">QTranslator</a>:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> main(<span class="type">int</span> argc<span class="operator">,</span> <span class="type">char</span> <span class="operator">*</span>argv<span class="operator">[</span><span class="operator">]</span>)
///   {
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qapplication.html">QApplication</a></span> app(argc<span class="operator">,</span> argv);
///
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qtranslator.html#QTranslator">QTranslator</a></span> translator;
/// &#32;     <span class="comment">// look up e.g. :/translations/myapp_de.qm</span>
/// &#32;     <span class="keyword">if</span> (translator<span class="operator">.</span>load(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span>()<span class="operator">,</span> QLatin1String(<span class="string">"myapp"</span>)<span class="operator">,</span> QLatin1String(<span class="string">"_"</span>)<span class="operator">,</span> QLatin1String(<span class="string">":/translations"</span>)))
/// &#32;   &#32;     app<span class="operator">.</span>installTranslator(<span class="operator">&amp;</span>translator);
///
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> hello(<span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"main"</span><span class="operator">,</span> <span class="string">"Hello world!"</span>));
/// &#32;     hello<span class="operator">.</span>resize(<span class="number">100</span><span class="operator">,</span> <span class="number">30</span>);
///
/// &#32;     hello<span class="operator">.</span>show();
/// &#32;     <span class="keyword">return</span> app<span class="operator">.</span>exec();
///   }
///
/// </pre>
/// <p>Note that the translator must be created <i>before</i> the application's widgets.</p>
/// <p>Most applications will never need to do anything else with this class. The other functions provided by this class are useful for applications that work on translator files.</p>
/// <a name="looking-up-translations"></a></div>
#[repr(C)]
pub struct QTranslator {
    _unused: u8,
}
impl QTranslator {
    /// <p>Returns <code>true</code> if this translator is empty, otherwise returns <code>false</code>. This function works with stripped and unstripped translation files.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QTranslator::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtranslator.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this translator is empty, otherwise returns <code>false</code>. This function works with stripped and unstripped translation files.</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTranslator_isEmpty(self as *const crate::QTranslator)
    }

    /// <p>Loads <i>filename</i> + <i>suffix</i> (".qm" if the <i>suffix</i> is not specified), which may be an absolute file name or relative to <i>directory</i>. Returns <code>true</code> if the translation is successfully loaded; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTranslator::load(const QString& filename, const QString& directory = …, const QString& search_delimiters = …, const QString& suffix = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtranslator.html#load">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Loads <i>filename</i> + <i>suffix</i> (".qm" if the <i>suffix</i> is not specified), which may be an absolute file name or relative to <i>directory</i>. Returns <code>true</code> if the translation is successfully loaded; otherwise returns <code>false</code>.</p>
    /// <p>If <i>directory</i> is not specified, the current directory is used (i.e., as <a href="http://doc.qt.io/qt-5/qdir.html#currentPath">currentPath()</a>).</p>
    /// <p>The previous contents of this translator object are discarded.</p>
    /// <p>If the file name does not exist, other file names are tried in the following order:</p>
    /// <ol class="1" type="1"><li>File name without <i>suffix</i> appended.</li>
    /// <li>File name with text after a character in <i>search_delimiters</i> stripped ("_." is the default for <i>search_delimiters</i> if it is an empty string) and <i>suffix</i>.</li>
    /// <li>File name stripped without <i>suffix</i> appended.</li>
    /// <li>File name stripped further, etc.</li>
    /// </ol>
    /// <p>For example, an application running in the fr_CA locale (French-speaking Canada) might call load("foo.fr_ca", "/opt/foolib"). load() would then try to open the first existing readable file from this list:</p>
    /// <ol class="1" type="1"><li><code>/opt/foolib/foo.fr_ca.qm</code></li>
    /// <li><code>/opt/foolib/foo.fr_ca</code></li>
    /// <li><code>/opt/foolib/foo.fr.qm</code></li>
    /// <li><code>/opt/foolib/foo.fr</code></li>
    /// <li><code>/opt/foolib/foo.qm</code></li>
    /// <li><code>/opt/foolib/foo</code></li>
    /// </ol>
    /// <p>Usually, it is better to use the QTranslator::load(const <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> &amp;, const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;, const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;, const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;, const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;) function instead, because it uses <a href="http://doc.qt.io/qt-5/qlocale.html#uiLanguages">QLocale::uiLanguages</a>() and not simply the locale name, which refers to the formatting of dates and numbers and not necessarily the UI language.</p></div>
    pub unsafe fn load_4_q_string(
        &mut self,
        filename: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        directory: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        search_delimiters: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        suffix: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTranslator_load(
            self as *mut crate::QTranslator,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(filename)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(directory)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(search_delimiters)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(suffix)
                .as_raw_ptr(),
        )
    }

    /// <p>Loads <i>filename</i> + <i>prefix</i> + <a href="http://doc.qt.io/qt-5/qlocale.html#uiLanguages">ui language name</a> + <i>suffix</i> (".qm" if the <i>suffix</i> is not specified), which may be an absolute file name or relative to <i>directory</i>. Returns <code>true</code> if the translation is successfully loaded; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTranslator::load(const QLocale& locale, const QString& filename, const QString& prefix = …, const QString& directory = …, const QString& suffix = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtranslator.html#load-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Loads <i>filename</i> + <i>prefix</i> + <a href="http://doc.qt.io/qt-5/qlocale.html#uiLanguages">ui language name</a> + <i>suffix</i> (".qm" if the <i>suffix</i> is not specified), which may be an absolute file name or relative to <i>directory</i>. Returns <code>true</code> if the translation is successfully loaded; otherwise returns <code>false</code>.</p>
    /// <p>The previous contents of this translator object are discarded.</p>
    /// <p>If the file name does not exist, other file names are tried in the following order:</p>
    /// <ol class="1" type="1"><li>File name without <i>suffix</i> appended.</li>
    /// <li>File name with ui language part after a "_" character stripped and <i>suffix</i>.</li>
    /// <li>File name with ui language part stripped without <i>suffix</i> appended.</li>
    /// <li>File name with ui language part stripped further, etc.</li>
    /// </ol>
    /// <p>For example, an application running in the <i>locale</i> with the following <a href="http://doc.qt.io/qt-5/qlocale.html#uiLanguages">ui languages</a> - "es", "fr-CA", "de" might call load(QLocale(), "foo", ".", "/opt/foolib", ".qm"). <a href="http://doc.qt.io/qt-5/qtranslator.html#load">load</a>() would replace '-' (dash) with '_' (underscore) in the ui language and then try to open the first existing readable file from this list:</p>
    /// <ol class="1" type="1"><li><code>/opt/foolib/foo.es.qm</code></li>
    /// <li><code>/opt/foolib/foo.es</code></li>
    /// <li><code>/opt/foolib/foo.fr_CA.qm</code></li>
    /// <li><code>/opt/foolib/foo.fr_CA</code></li>
    /// <li><code>/opt/foolib/foo.de.qm</code></li>
    /// <li><code>/opt/foolib/foo.de</code></li>
    /// <li><code>/opt/foolib/foo.fr.qm</code></li>
    /// <li><code>/opt/foolib/foo.fr</code></li>
    /// <li><code>/opt/foolib/foo.qm</code></li>
    /// <li><code>/opt/foolib/foo</code>.</li>
    /// <li><code>/opt/foolib/foo</code></li>
    /// </ol>
    /// <p>On operating systems where file system is case sensitive, <a href="http://doc.qt.io/qt-5/qtranslator.html">QTranslator</a> also tries to load a lower-cased version of the locale name.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn load_q_locale4_q_string(
        &mut self,
        locale: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
        filename: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        prefix: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        directory: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        suffix: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTranslator_load1(
            self as *mut crate::QTranslator,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(locale)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(filename)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(prefix)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(directory)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(suffix)
                .as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qtranslator.html#load">load</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTranslator::load(const unsigned char* data, int len, const QString& directory = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtranslator.html#load-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qtranslator.html#load">load</a>().</p>
    /// <p>Loads the QM file data <i>data</i> of length <i>len</i> into the translator.</p>
    /// <p>The data is not copied. The caller must be able to guarantee that <i>data</i> will not be deleted or modified.</p>
    /// <p><i>directory</i> is only used to specify the base directory when loading the dependencies of a QM file. If the file does not have dependencies, this argument is ignored.</p></div>
    pub unsafe fn load_uchar_int_q_string(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_uchar>>,
        len: ::std::os::raw::c_int,
        directory: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTranslator_load2(
            self as *mut crate::QTranslator,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_uchar>>::cast_into(data)
                .as_raw_ptr(),
            len,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(directory)
                .as_raw_ptr(),
        )
    }

    /// <p>Loads <i>filename</i> + <i>suffix</i> (".qm" if the <i>suffix</i> is not specified), which may be an absolute file name or relative to <i>directory</i>. Returns <code>true</code> if the translation is successfully loaded; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTranslator::load(const QString& filename, const QString& directory = …, const QString& search_delimiters = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtranslator.html#load">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Loads <i>filename</i> + <i>suffix</i> (".qm" if the <i>suffix</i> is not specified), which may be an absolute file name or relative to <i>directory</i>. Returns <code>true</code> if the translation is successfully loaded; otherwise returns <code>false</code>.</p>
    /// <p>If <i>directory</i> is not specified, the current directory is used (i.e., as <a href="http://doc.qt.io/qt-5/qdir.html#currentPath">currentPath()</a>).</p>
    /// <p>The previous contents of this translator object are discarded.</p>
    /// <p>If the file name does not exist, other file names are tried in the following order:</p>
    /// <ol class="1" type="1"><li>File name without <i>suffix</i> appended.</li>
    /// <li>File name with text after a character in <i>search_delimiters</i> stripped ("_." is the default for <i>search_delimiters</i> if it is an empty string) and <i>suffix</i>.</li>
    /// <li>File name stripped without <i>suffix</i> appended.</li>
    /// <li>File name stripped further, etc.</li>
    /// </ol>
    /// <p>For example, an application running in the fr_CA locale (French-speaking Canada) might call load("foo.fr_ca", "/opt/foolib"). load() would then try to open the first existing readable file from this list:</p>
    /// <ol class="1" type="1"><li><code>/opt/foolib/foo.fr_ca.qm</code></li>
    /// <li><code>/opt/foolib/foo.fr_ca</code></li>
    /// <li><code>/opt/foolib/foo.fr.qm</code></li>
    /// <li><code>/opt/foolib/foo.fr</code></li>
    /// <li><code>/opt/foolib/foo.qm</code></li>
    /// <li><code>/opt/foolib/foo</code></li>
    /// </ol>
    /// <p>Usually, it is better to use the QTranslator::load(const <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> &amp;, const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;, const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;, const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;, const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;) function instead, because it uses <a href="http://doc.qt.io/qt-5/qlocale.html#uiLanguages">QLocale::uiLanguages</a>() and not simply the locale name, which refers to the formatting of dates and numbers and not necessarily the UI language.</p></div>
    pub unsafe fn load_3_q_string(
        &mut self,
        filename: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        directory: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        search_delimiters: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTranslator_load3(
            self as *mut crate::QTranslator,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(filename)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(directory)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(search_delimiters)
                .as_raw_ptr(),
        )
    }

    /// <p>Loads <i>filename</i> + <i>suffix</i> (".qm" if the <i>suffix</i> is not specified), which may be an absolute file name or relative to <i>directory</i>. Returns <code>true</code> if the translation is successfully loaded; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTranslator::load(const QString& filename, const QString& directory = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtranslator.html#load">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Loads <i>filename</i> + <i>suffix</i> (".qm" if the <i>suffix</i> is not specified), which may be an absolute file name or relative to <i>directory</i>. Returns <code>true</code> if the translation is successfully loaded; otherwise returns <code>false</code>.</p>
    /// <p>If <i>directory</i> is not specified, the current directory is used (i.e., as <a href="http://doc.qt.io/qt-5/qdir.html#currentPath">currentPath()</a>).</p>
    /// <p>The previous contents of this translator object are discarded.</p>
    /// <p>If the file name does not exist, other file names are tried in the following order:</p>
    /// <ol class="1" type="1"><li>File name without <i>suffix</i> appended.</li>
    /// <li>File name with text after a character in <i>search_delimiters</i> stripped ("_." is the default for <i>search_delimiters</i> if it is an empty string) and <i>suffix</i>.</li>
    /// <li>File name stripped without <i>suffix</i> appended.</li>
    /// <li>File name stripped further, etc.</li>
    /// </ol>
    /// <p>For example, an application running in the fr_CA locale (French-speaking Canada) might call load("foo.fr_ca", "/opt/foolib"). load() would then try to open the first existing readable file from this list:</p>
    /// <ol class="1" type="1"><li><code>/opt/foolib/foo.fr_ca.qm</code></li>
    /// <li><code>/opt/foolib/foo.fr_ca</code></li>
    /// <li><code>/opt/foolib/foo.fr.qm</code></li>
    /// <li><code>/opt/foolib/foo.fr</code></li>
    /// <li><code>/opt/foolib/foo.qm</code></li>
    /// <li><code>/opt/foolib/foo</code></li>
    /// </ol>
    /// <p>Usually, it is better to use the QTranslator::load(const <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> &amp;, const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;, const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;, const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;, const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;) function instead, because it uses <a href="http://doc.qt.io/qt-5/qlocale.html#uiLanguages">QLocale::uiLanguages</a>() and not simply the locale name, which refers to the formatting of dates and numbers and not necessarily the UI language.</p></div>
    pub unsafe fn load_2_q_string(
        &mut self,
        filename: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        directory: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTranslator_load4(
            self as *mut crate::QTranslator,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(filename)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(directory)
                .as_raw_ptr(),
        )
    }

    /// <p>Loads <i>filename</i> + <i>suffix</i> (".qm" if the <i>suffix</i> is not specified), which may be an absolute file name or relative to <i>directory</i>. Returns <code>true</code> if the translation is successfully loaded; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTranslator::load(const QString& filename)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtranslator.html#load">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Loads <i>filename</i> + <i>suffix</i> (".qm" if the <i>suffix</i> is not specified), which may be an absolute file name or relative to <i>directory</i>. Returns <code>true</code> if the translation is successfully loaded; otherwise returns <code>false</code>.</p>
    /// <p>If <i>directory</i> is not specified, the current directory is used (i.e., as <a href="http://doc.qt.io/qt-5/qdir.html#currentPath">currentPath()</a>).</p>
    /// <p>The previous contents of this translator object are discarded.</p>
    /// <p>If the file name does not exist, other file names are tried in the following order:</p>
    /// <ol class="1" type="1"><li>File name without <i>suffix</i> appended.</li>
    /// <li>File name with text after a character in <i>search_delimiters</i> stripped ("_." is the default for <i>search_delimiters</i> if it is an empty string) and <i>suffix</i>.</li>
    /// <li>File name stripped without <i>suffix</i> appended.</li>
    /// <li>File name stripped further, etc.</li>
    /// </ol>
    /// <p>For example, an application running in the fr_CA locale (French-speaking Canada) might call load("foo.fr_ca", "/opt/foolib"). load() would then try to open the first existing readable file from this list:</p>
    /// <ol class="1" type="1"><li><code>/opt/foolib/foo.fr_ca.qm</code></li>
    /// <li><code>/opt/foolib/foo.fr_ca</code></li>
    /// <li><code>/opt/foolib/foo.fr.qm</code></li>
    /// <li><code>/opt/foolib/foo.fr</code></li>
    /// <li><code>/opt/foolib/foo.qm</code></li>
    /// <li><code>/opt/foolib/foo</code></li>
    /// </ol>
    /// <p>Usually, it is better to use the QTranslator::load(const <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> &amp;, const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;, const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;, const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;, const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;) function instead, because it uses <a href="http://doc.qt.io/qt-5/qlocale.html#uiLanguages">QLocale::uiLanguages</a>() and not simply the locale name, which refers to the formatting of dates and numbers and not necessarily the UI language.</p></div>
    pub unsafe fn load_q_string(
        &mut self,
        filename: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTranslator_load5(
            self as *mut crate::QTranslator,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(filename)
                .as_raw_ptr(),
        )
    }

    /// <p>Loads <i>filename</i> + <i>prefix</i> + <a href="http://doc.qt.io/qt-5/qlocale.html#uiLanguages">ui language name</a> + <i>suffix</i> (".qm" if the <i>suffix</i> is not specified), which may be an absolute file name or relative to <i>directory</i>. Returns <code>true</code> if the translation is successfully loaded; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTranslator::load(const QLocale& locale, const QString& filename, const QString& prefix = …, const QString& directory = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtranslator.html#load-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Loads <i>filename</i> + <i>prefix</i> + <a href="http://doc.qt.io/qt-5/qlocale.html#uiLanguages">ui language name</a> + <i>suffix</i> (".qm" if the <i>suffix</i> is not specified), which may be an absolute file name or relative to <i>directory</i>. Returns <code>true</code> if the translation is successfully loaded; otherwise returns <code>false</code>.</p>
    /// <p>The previous contents of this translator object are discarded.</p>
    /// <p>If the file name does not exist, other file names are tried in the following order:</p>
    /// <ol class="1" type="1"><li>File name without <i>suffix</i> appended.</li>
    /// <li>File name with ui language part after a "_" character stripped and <i>suffix</i>.</li>
    /// <li>File name with ui language part stripped without <i>suffix</i> appended.</li>
    /// <li>File name with ui language part stripped further, etc.</li>
    /// </ol>
    /// <p>For example, an application running in the <i>locale</i> with the following <a href="http://doc.qt.io/qt-5/qlocale.html#uiLanguages">ui languages</a> - "es", "fr-CA", "de" might call load(QLocale(), "foo", ".", "/opt/foolib", ".qm"). <a href="http://doc.qt.io/qt-5/qtranslator.html#load">load</a>() would replace '-' (dash) with '_' (underscore) in the ui language and then try to open the first existing readable file from this list:</p>
    /// <ol class="1" type="1"><li><code>/opt/foolib/foo.es.qm</code></li>
    /// <li><code>/opt/foolib/foo.es</code></li>
    /// <li><code>/opt/foolib/foo.fr_CA.qm</code></li>
    /// <li><code>/opt/foolib/foo.fr_CA</code></li>
    /// <li><code>/opt/foolib/foo.de.qm</code></li>
    /// <li><code>/opt/foolib/foo.de</code></li>
    /// <li><code>/opt/foolib/foo.fr.qm</code></li>
    /// <li><code>/opt/foolib/foo.fr</code></li>
    /// <li><code>/opt/foolib/foo.qm</code></li>
    /// <li><code>/opt/foolib/foo</code>.</li>
    /// <li><code>/opt/foolib/foo</code></li>
    /// </ol>
    /// <p>On operating systems where file system is case sensitive, <a href="http://doc.qt.io/qt-5/qtranslator.html">QTranslator</a> also tries to load a lower-cased version of the locale name.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn load_q_locale3_q_string(
        &mut self,
        locale: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
        filename: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        prefix: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        directory: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTranslator_load6(
            self as *mut crate::QTranslator,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(locale)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(filename)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(prefix)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(directory)
                .as_raw_ptr(),
        )
    }

    /// <p>Loads <i>filename</i> + <i>prefix</i> + <a href="http://doc.qt.io/qt-5/qlocale.html#uiLanguages">ui language name</a> + <i>suffix</i> (".qm" if the <i>suffix</i> is not specified), which may be an absolute file name or relative to <i>directory</i>. Returns <code>true</code> if the translation is successfully loaded; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTranslator::load(const QLocale& locale, const QString& filename, const QString& prefix = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtranslator.html#load-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Loads <i>filename</i> + <i>prefix</i> + <a href="http://doc.qt.io/qt-5/qlocale.html#uiLanguages">ui language name</a> + <i>suffix</i> (".qm" if the <i>suffix</i> is not specified), which may be an absolute file name or relative to <i>directory</i>. Returns <code>true</code> if the translation is successfully loaded; otherwise returns <code>false</code>.</p>
    /// <p>The previous contents of this translator object are discarded.</p>
    /// <p>If the file name does not exist, other file names are tried in the following order:</p>
    /// <ol class="1" type="1"><li>File name without <i>suffix</i> appended.</li>
    /// <li>File name with ui language part after a "_" character stripped and <i>suffix</i>.</li>
    /// <li>File name with ui language part stripped without <i>suffix</i> appended.</li>
    /// <li>File name with ui language part stripped further, etc.</li>
    /// </ol>
    /// <p>For example, an application running in the <i>locale</i> with the following <a href="http://doc.qt.io/qt-5/qlocale.html#uiLanguages">ui languages</a> - "es", "fr-CA", "de" might call load(QLocale(), "foo", ".", "/opt/foolib", ".qm"). <a href="http://doc.qt.io/qt-5/qtranslator.html#load">load</a>() would replace '-' (dash) with '_' (underscore) in the ui language and then try to open the first existing readable file from this list:</p>
    /// <ol class="1" type="1"><li><code>/opt/foolib/foo.es.qm</code></li>
    /// <li><code>/opt/foolib/foo.es</code></li>
    /// <li><code>/opt/foolib/foo.fr_CA.qm</code></li>
    /// <li><code>/opt/foolib/foo.fr_CA</code></li>
    /// <li><code>/opt/foolib/foo.de.qm</code></li>
    /// <li><code>/opt/foolib/foo.de</code></li>
    /// <li><code>/opt/foolib/foo.fr.qm</code></li>
    /// <li><code>/opt/foolib/foo.fr</code></li>
    /// <li><code>/opt/foolib/foo.qm</code></li>
    /// <li><code>/opt/foolib/foo</code>.</li>
    /// <li><code>/opt/foolib/foo</code></li>
    /// </ol>
    /// <p>On operating systems where file system is case sensitive, <a href="http://doc.qt.io/qt-5/qtranslator.html">QTranslator</a> also tries to load a lower-cased version of the locale name.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn load_q_locale2_q_string(
        &mut self,
        locale: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
        filename: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        prefix: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTranslator_load7(
            self as *mut crate::QTranslator,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(locale)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(filename)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(prefix)
                .as_raw_ptr(),
        )
    }

    /// <p>Loads <i>filename</i> + <i>prefix</i> + <a href="http://doc.qt.io/qt-5/qlocale.html#uiLanguages">ui language name</a> + <i>suffix</i> (".qm" if the <i>suffix</i> is not specified), which may be an absolute file name or relative to <i>directory</i>. Returns <code>true</code> if the translation is successfully loaded; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTranslator::load(const QLocale& locale, const QString& filename)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtranslator.html#load-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Loads <i>filename</i> + <i>prefix</i> + <a href="http://doc.qt.io/qt-5/qlocale.html#uiLanguages">ui language name</a> + <i>suffix</i> (".qm" if the <i>suffix</i> is not specified), which may be an absolute file name or relative to <i>directory</i>. Returns <code>true</code> if the translation is successfully loaded; otherwise returns <code>false</code>.</p>
    /// <p>The previous contents of this translator object are discarded.</p>
    /// <p>If the file name does not exist, other file names are tried in the following order:</p>
    /// <ol class="1" type="1"><li>File name without <i>suffix</i> appended.</li>
    /// <li>File name with ui language part after a "_" character stripped and <i>suffix</i>.</li>
    /// <li>File name with ui language part stripped without <i>suffix</i> appended.</li>
    /// <li>File name with ui language part stripped further, etc.</li>
    /// </ol>
    /// <p>For example, an application running in the <i>locale</i> with the following <a href="http://doc.qt.io/qt-5/qlocale.html#uiLanguages">ui languages</a> - "es", "fr-CA", "de" might call load(QLocale(), "foo", ".", "/opt/foolib", ".qm"). <a href="http://doc.qt.io/qt-5/qtranslator.html#load">load</a>() would replace '-' (dash) with '_' (underscore) in the ui language and then try to open the first existing readable file from this list:</p>
    /// <ol class="1" type="1"><li><code>/opt/foolib/foo.es.qm</code></li>
    /// <li><code>/opt/foolib/foo.es</code></li>
    /// <li><code>/opt/foolib/foo.fr_CA.qm</code></li>
    /// <li><code>/opt/foolib/foo.fr_CA</code></li>
    /// <li><code>/opt/foolib/foo.de.qm</code></li>
    /// <li><code>/opt/foolib/foo.de</code></li>
    /// <li><code>/opt/foolib/foo.fr.qm</code></li>
    /// <li><code>/opt/foolib/foo.fr</code></li>
    /// <li><code>/opt/foolib/foo.qm</code></li>
    /// <li><code>/opt/foolib/foo</code>.</li>
    /// <li><code>/opt/foolib/foo</code></li>
    /// </ol>
    /// <p>On operating systems where file system is case sensitive, <a href="http://doc.qt.io/qt-5/qtranslator.html">QTranslator</a> also tries to load a lower-cased version of the locale name.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn load_q_locale_q_string(
        &mut self,
        locale: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
        filename: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTranslator_load8(
            self as *mut crate::QTranslator,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(locale)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(filename)
                .as_raw_ptr(),
        )
    }

    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qtranslator.html#load">load</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTranslator::load(const unsigned char* data, int len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtranslator.html#load-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qtranslator.html#load">load</a>().</p>
    /// <p>Loads the QM file data <i>data</i> of length <i>len</i> into the translator.</p>
    /// <p>The data is not copied. The caller must be able to guarantee that <i>data</i> will not be deleted or modified.</p>
    /// <p><i>directory</i> is only used to specify the base directory when loading the dependencies of a QM file. If the file does not have dependencies, this argument is ignored.</p></div>
    pub unsafe fn load_uchar_int(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_uchar>>,
        len: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTranslator_load9(
            self as *mut crate::QTranslator,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_uchar>>::cast_into(data)
                .as_raw_ptr(),
            len,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QTranslator::metaObject() const```</span>.
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QTranslator_metaObject(self as *const crate::QTranslator);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Constructs an empty message file object with parent <i>parent</i> that is not connected to any file.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTranslator::QTranslator(QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtranslator.html#QTranslator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty message file object with parent <i>parent</i> that is not connected to any file.</p></div>
    pub unsafe fn new_1a(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QTranslator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTranslator_QTranslator(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qtranslator.html">QTranslator</a> class provides internationalization support for text output.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTranslator::QTranslator()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtranslator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qtranslator.html">QTranslator</a> class provides internationalization support for text output.</p>
    /// <p>An object of this class contains a set of translations from a source language to a target language. <a href="http://doc.qt.io/qt-5/qtranslator.html">QTranslator</a> provides functions to look up translations in a translation file. Translation files are created using <a href="http://doc.qt.io/qt-5/qtlinguist-index.html">Qt Linguist</a>.</p>
    /// <p>The most common use of <a href="http://doc.qt.io/qt-5/qtranslator.html">QTranslator</a> is to: load a translation file, install it using <a href="http://doc.qt.io/qt-5/qcoreapplication.html#installTranslator">QCoreApplication::installTranslator</a>(), and use it via <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(). Here's an example <code>main()</code> function using the <a href="http://doc.qt.io/qt-5/qtranslator.html">QTranslator</a>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">int</span> main(<span class="type">int</span> argc<span class="operator">,</span> <span class="type">char</span> <span class="operator">*</span>argv<span class="operator">[</span><span class="operator">]</span>)
    ///   {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qapplication.html">QApplication</a></span> app(argc<span class="operator">,</span> argv);
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qtranslator.html#QTranslator">QTranslator</a></span> translator;
    /// &#32;     <span class="comment">// look up e.g. :/translations/myapp_de.qm</span>
    /// &#32;     <span class="keyword">if</span> (translator<span class="operator">.</span>load(<span class="type"><a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a></span>()<span class="operator">,</span> QLatin1String(<span class="string">"myapp"</span>)<span class="operator">,</span> QLatin1String(<span class="string">"_"</span>)<span class="operator">,</span> QLatin1String(<span class="string">":/translations"</span>)))
    /// &#32;   &#32;     app<span class="operator">.</span>installTranslator(<span class="operator">&amp;</span>translator);
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qpushbutton.html">QPushButton</a></span> hello(<span class="type"><a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a></span><span class="operator">::</span>translate(<span class="string">"main"</span><span class="operator">,</span> <span class="string">"Hello world!"</span>));
    /// &#32;     hello<span class="operator">.</span>resize(<span class="number">100</span><span class="operator">,</span> <span class="number">30</span>);
    ///
    /// &#32;     hello<span class="operator">.</span>show();
    /// &#32;     <span class="keyword">return</span> app<span class="operator">.</span>exec();
    ///   }
    ///
    /// </pre>
    /// <p>Note that the translator must be created <i>before</i> the application's widgets.</p>
    /// <p>Most applications will never need to do anything else with this class. The other functions provided by this class are useful for applications that work on translator files.</p>
    /// <a name="looking-up-translations"></a></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QTranslator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTranslator_QTranslator1();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QTranslator::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTranslator_qt_metacall(
            self as *mut crate::QTranslator,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QTranslator::qt_metacast(const char* arg1)```</span>.
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTranslator_qt_metacast(
            self as *mut crate::QTranslator,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTranslator_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QTranslator::tr(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTranslator_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QTranslator::trUtf8(const char* s, const char* c, int n)```</span>.
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTranslator_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the translation for the key (<i>context</i>, <i>sourceText</i>, <i>disambiguation</i>). If none is found, also tries (<i>context</i>, <i>sourceText</i>, ""). If that still fails, returns a null string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QString QTranslator::translate(const char* context, const char* sourceText, const char* disambiguation = …, int n = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtranslator.html#translate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the translation for the key (<i>context</i>, <i>sourceText</i>, <i>disambiguation</i>). If none is found, also tries (<i>context</i>, <i>sourceText</i>, ""). If that still fails, returns a null string.</p>
    /// <p><b>Note: </b>Incomplete translations may result in unexpected behavior: If no translation for (<i>context</i>, <i>sourceText</i>, "") is provided, the method might in this case actually return a translation for a different <i>disambiguation</i>.</p><p>If <i>n</i> is not -1, it is used to choose an appropriate form for the translation (e.g. "%n file found" vs. "%n files found").</p>
    /// <p>If you need to programatically insert translations into a <a href="http://doc.qt.io/qt-5/qtranslator.html">QTranslator</a>, this function can be reimplemented.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtranslator.html#load">load</a>().</p></div>
    pub unsafe fn translate_4a(
        &self,
        context: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        source_text: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        disambiguation: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTranslator_translate(
            self as *const crate::QTranslator,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(context)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(
                source_text,
            )
            .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(
                disambiguation,
            )
            .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the translation for the key (<i>context</i>, <i>sourceText</i>, <i>disambiguation</i>). If none is found, also tries (<i>context</i>, <i>sourceText</i>, ""). If that still fails, returns a null string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QString QTranslator::translate(const char* context, const char* sourceText, const char* disambiguation = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtranslator.html#translate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the translation for the key (<i>context</i>, <i>sourceText</i>, <i>disambiguation</i>). If none is found, also tries (<i>context</i>, <i>sourceText</i>, ""). If that still fails, returns a null string.</p>
    /// <p><b>Note: </b>Incomplete translations may result in unexpected behavior: If no translation for (<i>context</i>, <i>sourceText</i>, "") is provided, the method might in this case actually return a translation for a different <i>disambiguation</i>.</p><p>If <i>n</i> is not -1, it is used to choose an appropriate form for the translation (e.g. "%n file found" vs. "%n files found").</p>
    /// <p>If you need to programatically insert translations into a <a href="http://doc.qt.io/qt-5/qtranslator.html">QTranslator</a>, this function can be reimplemented.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtranslator.html#load">load</a>().</p></div>
    pub unsafe fn translate_3a(
        &self,
        context: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        source_text: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        disambiguation: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTranslator_translate1(
            self as *const crate::QTranslator,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(context)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(
                source_text,
            )
            .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(
                disambiguation,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the translation for the key (<i>context</i>, <i>sourceText</i>, <i>disambiguation</i>). If none is found, also tries (<i>context</i>, <i>sourceText</i>, ""). If that still fails, returns a null string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QString QTranslator::translate(const char* context, const char* sourceText) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtranslator.html#translate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the translation for the key (<i>context</i>, <i>sourceText</i>, <i>disambiguation</i>). If none is found, also tries (<i>context</i>, <i>sourceText</i>, ""). If that still fails, returns a null string.</p>
    /// <p><b>Note: </b>Incomplete translations may result in unexpected behavior: If no translation for (<i>context</i>, <i>sourceText</i>, "") is provided, the method might in this case actually return a translation for a different <i>disambiguation</i>.</p><p>If <i>n</i> is not -1, it is used to choose an appropriate form for the translation (e.g. "%n file found" vs. "%n files found").</p>
    /// <p>If you need to programatically insert translations into a <a href="http://doc.qt.io/qt-5/qtranslator.html">QTranslator</a>, this function can be reimplemented.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtranslator.html#load">load</a>().</p></div>
    pub unsafe fn translate_2a(
        &self,
        context: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        source_text: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTranslator_translate2(
            self as *const crate::QTranslator,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(context)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(
                source_text,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_url {
    //! C++ type: <span style='color: green;'>```QUrl```</span>

    /// <p>The parsing mode controls the way <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> parses strings.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QUrl::ParsingMode```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The parsing mode controls the way <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> parses strings.</p>
    ///
    /// <p>In TolerantMode, the parser has the following behaviour:</p>
    /// <ul>
    /// <li>Spaces and "%20": unencoded space characters will be accepted and will be treated as equivalent to "%20".</li>
    /// <li>Single "%" characters: Any occurrences of a percent character "%" not followed by exactly two hexadecimal characters (e.g., "13% coverage.html") will be replaced by "%25". Note that one lone "%" character will trigger the correction mode for all percent characters.</li>
    /// <li>Reserved and unreserved characters: An encoded URL should only contain a few characters as literals; all other characters should be percent-encoded. In TolerantMode, these characters will be accepted if they are found in the URL: space / double-quote / "&lt;" / "&gt;" / "" / "^" / "`" / "{" / "|" / "}" Those same characters can be decoded again by passing <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::DecodeReserved</a> to <a href="http://doc.qt.io/qt-5/qurl.html#toString">toString</a>() or <a href="http://doc.qt.io/qt-5/qurl.html#toEncoded">toEncoded</a>(). In the getters of individual components, those characters are often returned in decoded form.</li>
    /// </ul>
    /// <p>When in StrictMode, if a parsing error is found, <a href="http://doc.qt.io/qt-5/qurl.html#isValid">isValid</a>() will return <code>false</code> and <a href="http://doc.qt.io/qt-5/qurl.html#errorString">errorString</a>() will return a message describing the error. If more than one error is detected, it is undefined which error gets reported.</p>
    /// <p>Note that TolerantMode is not usually enough for parsing user input, which often contains more errors and expectations than the parser can deal with. When dealing with data coming directly from the user -- as opposed to data coming from data-transfer sources, such as other programs -- it is recommended to use <a href="http://doc.qt.io/qt-5/qurl.html#fromUserInput">fromUserInput</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#fromUserInput">fromUserInput</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#setUrl">setUrl</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#toString">toString</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#toEncoded">toEncoded</a>(), and <a href="http://doc.qt.io/qt-5/qurl.html#UrlFormattingOption-enum">QUrl::FormattingOptions</a>.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct ParsingMode(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for ParsingMode {
        fn from(value: ::std::os::raw::c_int) -> Self {
            ParsingMode(value)
        }
    }

    impl From<ParsingMode> for ::std::os::raw::c_int {
        fn from(value: ParsingMode) -> Self {
            value.0
        }
    }

    impl ParsingMode {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl ParsingMode {
        /// <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> will try to correct some common errors in URLs. This mode is useful for parsing URLs coming from sources not known to be strictly standards-conforming. (C++ enum variant: <span style='color: green;'>```TolerantMode = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const TolerantMode: crate::q_url::ParsingMode = crate::q_url::ParsingMode(0);
        /// Only valid URLs are accepted. This mode is useful for general URL validation. (C++ enum variant: <span style='color: green;'>```StrictMode = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const StrictMode: crate::q_url::ParsingMode = crate::q_url::ParsingMode(1);
        /// <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> will interpret the URL component in the fully-decoded form, where percent characters stand for themselves, not as the beginning of a percent-encoded sequence. This mode is only valid for the setters setting components of a URL; it is not permitted in the <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> constructor, in <a href="http://doc.qt.io/qt-5/qurl.html#fromEncoded">fromEncoded</a>() or in <a href="http://doc.qt.io/qt-5/qurl.html#setUrl">setUrl</a>(). For more information on this mode, see the documentation for <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a>. (C++ enum variant: <span style='color: green;'>```DecodedMode = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const DecodedMode: crate::q_url::ParsingMode = crate::q_url::ParsingMode(2);
    }

    /// <p>The formatting options define how the URL is formatted when written out as text.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QUrl::UrlFormattingOption```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#UrlFormattingOption-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The formatting options define how the URL is formatted when written out as text.</p>
    ///
    /// <p>Note that the case folding rules in <a href="http://www.rfc-editor.org/rfc/rfc3491.txt">Nameprep</a>, which <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> conforms to, require host names to always be converted to lower case, regardless of the Qt::FormattingOptions used.</p>
    /// <p>The options from <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::ComponentFormattingOptions</a> are also possible.</p>
    /// <p>The FormattingOptions type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;UrlFormattingOption&gt;. It stores an OR combination of UrlFormattingOption values.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::ComponentFormattingOptions</a>.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct UrlFormattingOption(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for UrlFormattingOption {
        fn from(value: ::std::os::raw::c_int) -> Self {
            UrlFormattingOption(value)
        }
    }

    impl From<UrlFormattingOption> for ::std::os::raw::c_int {
        fn from(value: UrlFormattingOption) -> Self {
            value.0
        }
    }

    impl UrlFormattingOption {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl UrlFormattingOption {
        /// The format of the URL is unchanged. (C++ enum variant: <span style='color: green;'>```None = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const None: crate::q_url::UrlFormattingOption = crate::q_url::UrlFormattingOption(0);
        /// The scheme is removed from the URL. (C++ enum variant: <span style='color: green;'>```RemoveScheme = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const RemoveScheme: crate::q_url::UrlFormattingOption =
            crate::q_url::UrlFormattingOption(1);
        /// Any password in the URL is removed. (C++ enum variant: <span style='color: green;'>```RemovePassword = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const RemovePassword: crate::q_url::UrlFormattingOption =
            crate::q_url::UrlFormattingOption(2);
        /// Any user information in the URL is removed. (C++ enum variant: <span style='color: green;'>```RemoveUserInfo = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const RemoveUserInfo: crate::q_url::UrlFormattingOption =
            crate::q_url::UrlFormattingOption(6);
        /// Any specified port is removed from the URL. (C++ enum variant: <span style='color: green;'>```RemovePort = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const RemovePort: crate::q_url::UrlFormattingOption =
            crate::q_url::UrlFormattingOption(8);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```RemoveAuthority = 30```</span>)
        #[allow(non_upper_case_globals)]
        pub const RemoveAuthority: crate::q_url::UrlFormattingOption =
            crate::q_url::UrlFormattingOption(30);
        /// The URL's path is removed, leaving only the scheme, host address, and port (if present). (C++ enum variant: <span style='color: green;'>```RemovePath = 32```</span>)
        #[allow(non_upper_case_globals)]
        pub const RemovePath: crate::q_url::UrlFormattingOption =
            crate::q_url::UrlFormattingOption(32);
        /// The query part of the URL (following a '?' character) is removed. (C++ enum variant: <span style='color: green;'>```RemoveQuery = 64```</span>)
        #[allow(non_upper_case_globals)]
        pub const RemoveQuery: crate::q_url::UrlFormattingOption =
            crate::q_url::UrlFormattingOption(64);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```RemoveFragment = 128```</span>)
        #[allow(non_upper_case_globals)]
        pub const RemoveFragment: crate::q_url::UrlFormattingOption =
            crate::q_url::UrlFormattingOption(128);
        /// If the URL is a local file according to <a href="http://doc.qt.io/qt-5/qurl.html#isLocalFile">isLocalFile</a>() and contains no query or fragment, a local file path is returned. (C++ enum variant: <span style='color: green;'>```PreferLocalFile = 512```</span>)
        #[allow(non_upper_case_globals)]
        pub const PreferLocalFile: crate::q_url::UrlFormattingOption =
            crate::q_url::UrlFormattingOption(512);
        /// The trailing slash is removed from the path, if one is present. (C++ enum variant: <span style='color: green;'>```StripTrailingSlash = 1024```</span>)
        #[allow(non_upper_case_globals)]
        pub const StripTrailingSlash: crate::q_url::UrlFormattingOption =
            crate::q_url::UrlFormattingOption(1024);
        /// The filename (i.e. everything after the last '/' in the path) is removed. The trailing '/' is kept, unless StripTrailingSlash is set. Only valid if RemovePath is not set. (C++ enum variant: <span style='color: green;'>```RemoveFilename = 2048```</span>)
        #[allow(non_upper_case_globals)]
        pub const RemoveFilename: crate::q_url::UrlFormattingOption =
            crate::q_url::UrlFormattingOption(2048);
        /// Modifies the path to remove redundant directory separators, and to resolve "."s and ".."s (as far as possible). (C++ enum variant: <span style='color: green;'>```NormalizePathSegments = 4096```</span>)
        #[allow(non_upper_case_globals)]
        pub const NormalizePathSegments: crate::q_url::UrlFormattingOption =
            crate::q_url::UrlFormattingOption(4096);
    }

    /// <p>The component formatting options define how the components of an URL will be formatted when written out as text. They can be combined with the options from <a href="http://doc.qt.io/qt-5/qurl.html#UrlFormattingOption-enum">QUrl::FormattingOptions</a> when used in <a href="http://doc.qt.io/qt-5/qurl.html#toString">toString</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#toEncoded">toEncoded</a>().</p>
    ///
    /// C++ enum: <span style='color: green;'>```QUrl::ComponentFormattingOption```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The component formatting options define how the components of an URL will be formatted when written out as text. They can be combined with the options from <a href="http://doc.qt.io/qt-5/qurl.html#UrlFormattingOption-enum">QUrl::FormattingOptions</a> when used in <a href="http://doc.qt.io/qt-5/qurl.html#toString">toString</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#toEncoded">toEncoded</a>().</p>
    ///
    /// <p>The values of EncodeReserved and DecodeReserved should not be used together in one call. The behavior is undefined if that happens. They are provided as separate values because the behavior of the "pretty mode" with regards to reserved characters is different on certain components and specially on the full URL.</p>
    /// <a name="full-decoding"></a>
    /// <h5>Full decoding</h5>
    /// <p>The FullyDecoded mode is similar to the behavior of the functions returning <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> in Qt 4.x, in that every character represents itself and never has any special meaning. This is true even for the percent character ('%'), which should be interpreted to mean a literal percent, not the beginning of a percent-encoded sequence. The same actual character, in all other decoding modes, is represented by the sequence "%25".</p>
    /// <p>Whenever re-applying data obtained with QUrl::FullyDecoded into a <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a>, care must be taken to use the <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">QUrl::DecodedMode</a> parameter to the setters (like <a href="http://doc.qt.io/qt-5/qurl.html#setPath">setPath</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#setUserName">setUserName</a>()). Failure to do so may cause re-interpretation of the percent character ('%') as the beginning of a percent-encoded sequence.</p>
    /// <p>This mode is quite useful when portions of a URL are used in a non-URL context. For example, to extract the username, password or file paths in an FTP client application, the FullyDecoded mode should be used.</p>
    /// <p>This mode should be used with care, since there are two conditions that cannot be reliably represented in the returned <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. They are:</p>
    /// <ul>
    /// <li><b>Non-UTF-8 sequences:</b> URLs may contain sequences of percent-encoded characters that do not form valid UTF-8 sequences. Since URLs need to be decoded using UTF-8, any decoder failure will result in the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> containing one or more replacement characters where the sequence existed.</li>
    /// <li><b>Encoded delimiters:</b> URLs are also allowed to make a distinction between a delimiter found in its literal form and its equivalent in percent-encoded form. This is most commonly found in the query, but is permitted in most parts of the URL.</li>
    /// </ul>
    /// <p>The following example illustrates the problem:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span> original(<span class="string">"http://example.com/?q=a%2B%3Db%26c"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span> copy(original);
    ///   copy<span class="operator">.</span>setQuery(copy<span class="operator">.</span>query(<span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span><span class="operator">::</span>FullyDecoded)<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span><span class="operator">::</span>DecodedMode);
    ///
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> original<span class="operator">.</span>toString();   <span class="comment">// prints: http://example.com/?q=a%2B%3Db%26c</span>
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> copy<span class="operator">.</span>toString();       <span class="comment">// prints: http://example.com/?q=a+=b&amp;c</span>
    ///
    /// </pre>
    /// <p>If the two URLs were used via HTTP GET, the interpretation by the web server would probably be different. In the first case, it would interpret as one parameter, with a key of "q" and value "a+=b&amp;c". In the second case, it would probably interpret as two parameters, one with a key of "q" and value "a =b", and the second with a key "c" and no value.</p>
    /// <p>This enum was introduced or modified in  Qt 5.0.</p>
    /// <p>The ComponentFormattingOptions type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;ComponentFormattingOption&gt;. It stores an OR combination of ComponentFormattingOption values.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#UrlFormattingOption-enum">QUrl::FormattingOptions</a>.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct ComponentFormattingOption(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for ComponentFormattingOption {
        fn from(value: ::std::os::raw::c_int) -> Self {
            ComponentFormattingOption(value)
        }
    }

    impl From<ComponentFormattingOption> for ::std::os::raw::c_int {
        fn from(value: ComponentFormattingOption) -> Self {
            value.0
        }
    }

    impl ComponentFormattingOption {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl ComponentFormattingOption {
        /// The component is returned in a "pretty form", with most percent-encoded characters decoded. The exact behavior of PrettyDecoded varies from component to component and may also change from Qt release to Qt release. This is the default. (C++ enum variant: <span style='color: green;'>```PrettyDecoded = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const PrettyDecoded: crate::q_url::ComponentFormattingOption =
            crate::q_url::ComponentFormattingOption(0);
        /// Leave space characters in their encoded form ("%20"). (C++ enum variant: <span style='color: green;'>```EncodeSpaces = 1048576```</span>)
        #[allow(non_upper_case_globals)]
        pub const EncodeSpaces: crate::q_url::ComponentFormattingOption =
            crate::q_url::ComponentFormattingOption(1048576);
        /// Leave non-US-ASCII characters encoded in their UTF-8 percent-encoded form (e.g., "%C3%A9" for the U+00E9 codepoint, LATIN SMALL LETTER E WITH ACUTE). (C++ enum variant: <span style='color: green;'>```EncodeUnicode = 2097152```</span>)
        #[allow(non_upper_case_globals)]
        pub const EncodeUnicode: crate::q_url::ComponentFormattingOption =
            crate::q_url::ComponentFormattingOption(2097152);
        /// Leave certain delimiters in their encoded form, as would appear in the URL when the full URL is represented as text. The delimiters are affected by this option change from component to component. This flag has no effect in <a href="http://doc.qt.io/qt-5/qurl.html#toString">toString</a>() or <a href="http://doc.qt.io/qt-5/qurl.html#toEncoded">toEncoded</a>(). (C++ enum variant: <span style='color: green;'>```EncodeDelimiters = 12582912```</span>)
        #[allow(non_upper_case_globals)]
        pub const EncodeDelimiters: crate::q_url::ComponentFormattingOption =
            crate::q_url::ComponentFormattingOption(12582912);
        /// Leave US-ASCII characters not permitted in the URL by the specification in their encoded form. This is the default on <a href="http://doc.qt.io/qt-5/qurl.html#toString">toString</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#toEncoded">toEncoded</a>(). (C++ enum variant: <span style='color: green;'>```EncodeReserved = 16777216```</span>)
        #[allow(non_upper_case_globals)]
        pub const EncodeReserved: crate::q_url::ComponentFormattingOption =
            crate::q_url::ComponentFormattingOption(16777216);
        /// Decode the US-ASCII characters that the URL specification does not allow to appear in the URL. This is the default on the getters of individual components. (C++ enum variant: <span style='color: green;'>```DecodeReserved = 33554432```</span>)
        #[allow(non_upper_case_globals)]
        pub const DecodeReserved: crate::q_url::ComponentFormattingOption =
            crate::q_url::ComponentFormattingOption(33554432);
        /// Leave all characters in their properly-encoded form, as this component would appear as part of a URL. When used with <a href="http://doc.qt.io/qt-5/qurl.html#toString">toString</a>(), this produces a fully-compliant URL in <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> form, exactly equal to the result of <a href="http://doc.qt.io/qt-5/qurl.html#toEncoded">toEncoded</a>() (C++ enum variant: <span style='color: green;'>```FullyEncoded = 32505856```</span>)
        #[allow(non_upper_case_globals)]
        pub const FullyEncoded: crate::q_url::ComponentFormattingOption =
            crate::q_url::ComponentFormattingOption(32505856);
        /// Attempt to decode as much as possible. For individual components of the URL, this decodes every percent encoding sequence, including control characters (U+0000 to U+001F) and UTF-8 sequences found in percent-encoded form. Use of this mode may cause data loss, see below for more information. (C++ enum variant: <span style='color: green;'>```FullyDecoded = 133169152```</span>)
        #[allow(non_upper_case_globals)]
        pub const FullyDecoded: crate::q_url::ComponentFormattingOption =
            crate::q_url::ComponentFormattingOption(133169152);
    }

    /// <p>The user input resolution options define how <a href="http://doc.qt.io/qt-5/qurl.html#fromUserInput">fromUserInput</a>() should interpret strings that could either be a relative path or the short form of a HTTP URL. For instance <code>file.pl</code> can be either a local file or the URL <code>http://file.pl</code>.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QUrl::UserInputResolutionOption```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#UserInputResolutionOption-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The user input resolution options define how <a href="http://doc.qt.io/qt-5/qurl.html#fromUserInput">fromUserInput</a>() should interpret strings that could either be a relative path or the short form of a HTTP URL. For instance <code>file.pl</code> can be either a local file or the URL <code>http://file.pl</code>.</p>
    ///
    /// <p>This enum was introduced or modified in  Qt 5.4.</p>
    /// <p>The UserInputResolutionOptions type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;UserInputResolutionOption&gt;. It stores an OR combination of UserInputResolutionOption values.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#fromUserInput">fromUserInput</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct UserInputResolutionOption(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for UserInputResolutionOption {
        fn from(value: ::std::os::raw::c_int) -> Self {
            UserInputResolutionOption(value)
        }
    }

    impl From<UserInputResolutionOption> for ::std::os::raw::c_int {
        fn from(value: UserInputResolutionOption) -> Self {
            value.0
        }
    }

    impl UserInputResolutionOption {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl UserInputResolutionOption {
        /// The default resolution mechanism is to check whether a local file exists, in the working directory given to <a href="http://doc.qt.io/qt-5/qurl.html#fromUserInput">fromUserInput</a>, and only return a local path in that case. Otherwise a URL is assumed. (C++ enum variant: <span style='color: green;'>```DefaultResolution = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const DefaultResolution: crate::q_url::UserInputResolutionOption =
            crate::q_url::UserInputResolutionOption(0);
        /// This option makes <a href="http://doc.qt.io/qt-5/qurl.html#fromUserInput">fromUserInput</a>() always return a local path unless the input contains a scheme, such as <code>http://file.pl</code>. This is useful for applications such as text editors, which are able to create the file if it doesn't exist. (C++ enum variant: <span style='color: green;'>```AssumeLocalFile = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const AssumeLocalFile: crate::q_url::UserInputResolutionOption =
            crate::q_url::UserInputResolutionOption(1);
    }

    impl From<crate::q_url::UserInputResolutionOption>
        for crate::QFlags<crate::q_url::UserInputResolutionOption>
    {
        fn from(value: crate::q_url::UserInputResolutionOption) -> Self {
            Self::from(value.to_int())
        }
    }

    impl<T: Into<crate::QFlags<crate::q_url::UserInputResolutionOption>>> std::ops::BitOr<T>
        for crate::q_url::UserInputResolutionOption
    {
        type Output = crate::QFlags<crate::q_url::UserInputResolutionOption>;
        fn bitor(self, rhs: T) -> crate::QFlags<crate::q_url::UserInputResolutionOption> {
            Into::<crate::QFlags<crate::q_url::UserInputResolutionOption>>::into(self) | rhs
        }
    }

    impl From<crate::q_url::ComponentFormattingOption>
        for crate::QFlags<crate::q_url::ComponentFormattingOption>
    {
        fn from(value: crate::q_url::ComponentFormattingOption) -> Self {
            Self::from(value.to_int())
        }
    }

    impl<T: Into<crate::QFlags<crate::q_url::ComponentFormattingOption>>> std::ops::BitOr<T>
        for crate::q_url::ComponentFormattingOption
    {
        type Output = crate::QFlags<crate::q_url::ComponentFormattingOption>;
        fn bitor(self, rhs: T) -> crate::QFlags<crate::q_url::ComponentFormattingOption> {
            Into::<crate::QFlags<crate::q_url::ComponentFormattingOption>>::into(self) | rhs
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> class provides a convenient interface for working with URLs.</p>
///
/// C++ class: <span style='color: green;'>```QUrl```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qurl.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> class provides a convenient interface for working with URLs.</p>
/// <p>It can parse and construct URLs in both encoded and unencoded form. <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> also has support for internationalized domain names (IDNs).</p>
/// <p>The most common way to use <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> is to initialize it via the constructor by passing a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. Otherwise, <a href="http://doc.qt.io/qt-5/qurl.html#setUrl">setUrl</a>() can also be used.</p>
/// <p>URLs can be represented in two forms: encoded or unencoded. The unencoded representation is suitable for showing to users, but the encoded representation is typically what you would send to a web server. For example, the unencoded URL "http://bühler.example.com/List of applicants.xml" would be sent to the server as "http://xn--bhler-kva.example.com/List%20of%20applicants.xml".</p>
/// <p>A URL can also be constructed piece by piece by calling <a href="http://doc.qt.io/qt-5/qurl.html#setScheme">setScheme</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#setUserName">setUserName</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#setPassword">setPassword</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#setHost">setHost</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#setPort">setPort</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#setPath">setPath</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#setQuery">setQuery</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#setFragment">setFragment</a>(). Some convenience functions are also available: <a href="http://doc.qt.io/qt-5/qurl.html#setAuthority">setAuthority</a>() sets the user name, password, host and port. <a href="http://doc.qt.io/qt-5/qurl.html#setUserInfo">setUserInfo</a>() sets the user name and password at once.</p>
/// <p>Call <a href="http://doc.qt.io/qt-5/qurl.html#isValid">isValid</a>() to check if the URL is valid. This can be done at any point during the constructing of a URL. If <a href="http://doc.qt.io/qt-5/qurl.html#isValid">isValid</a>() returns <code>false</code>, you should <a href="http://doc.qt.io/qt-5/qurl.html#clear">clear</a>() the URL before proceeding, or start over by parsing a new URL with <a href="http://doc.qt.io/qt-5/qurl.html#setUrl">setUrl</a>().</p>
/// <p>Constructing a query is particularly convenient through the use of the <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> class and its methods <a href="http://doc.qt.io/qt-5/qurlquery.html#setQueryItems">QUrlQuery::setQueryItems</a>(), <a href="http://doc.qt.io/qt-5/qurlquery.html#addQueryItem">QUrlQuery::addQueryItem</a>() and <a href="http://doc.qt.io/qt-5/qurlquery.html#removeQueryItem">QUrlQuery::removeQueryItem</a>(). Use <a href="http://doc.qt.io/qt-5/qurlquery.html#setQueryDelimiters">QUrlQuery::setQueryDelimiters</a>() to customize the delimiters used for generating the query string.</p>
/// <p>For the convenience of generating encoded URL strings or query strings, there are two static functions called <a href="http://doc.qt.io/qt-5/qurl.html#fromPercentEncoding">fromPercentEncoding</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#toPercentEncoding">toPercentEncoding</a>() which deal with percent encoding and decoding of <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> objects.</p>
/// <p><a href="http://doc.qt.io/qt-5/qurl.html#fromLocalFile">fromLocalFile</a>() constructs a <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> by parsing a local file path. <a href="http://doc.qt.io/qt-5/qurl.html#toLocalFile">toLocalFile</a>() converts a URL to a local file path.</p>
/// <p>The human readable representation of the URL is fetched with <a href="http://doc.qt.io/qt-5/qurl.html#toString">toString</a>(). This representation is appropriate for displaying a URL to a user in unencoded form. The encoded form however, as returned by <a href="http://doc.qt.io/qt-5/qurl.html#toEncoded">toEncoded</a>(), is for internal use, passing to web servers, mail clients and so on. Both forms are technically correct and represent the same URL unambiguously -- in fact, passing either form to <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a>'s constructor or to <a href="http://doc.qt.io/qt-5/qurl.html#setUrl">setUrl</a>() will yield the same <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> object.</p>
/// <p><a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> conforms to the URI specification from <a href="http://www.rfc-editor.org/rfc/rfc3986.txt">RFC 3986</a> (Uniform Resource Identifier: Generic Syntax), and includes scheme extensions from <a href="http://www.rfc-editor.org/rfc/rfc1738.txt">RFC 1738</a> (Uniform Resource Locators). Case folding rules in <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> conform to <a href="http://www.rfc-editor.org/rfc/rfc3491.txt">RFC 3491</a> (Nameprep: A Stringprep Profile for Internationalized Domain Names (IDN)). It is also compatible with the <a href="http://freedesktop.org/wiki/Specifications/file-uri-spec/">file URI specification</a> from freedesktop.org, provided that the locale encodes file names using UTF-8 (required by IDN).</p>
/// <a name="relative-urls-vs-relative-paths"></a>
/// <h4>Relative URLs vs Relative Paths</h4>
/// <p>Calling <a href="http://doc.qt.io/qt-5/qurl.html#isRelative">isRelative</a>() will return whether or not the URL is relative. A relative URL has no <a href="http://doc.qt.io/qt-5/qurl.html#scheme">scheme</a>. For example:</p>
/// <pre class="cpp">
///
///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span>(<span class="string">"main.qml"</span>)<span class="operator">.</span>isRelative();          <span class="comment">// true: no scheme</span>
///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span>(<span class="string">"qml/main.qml"</span>)<span class="operator">.</span>isRelative();      <span class="comment">// true: no scheme</span>
///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span>(<span class="string">"file:main.qml"</span>)<span class="operator">.</span>isRelative();     <span class="comment">// false: has "file" scheme</span>
///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span>(<span class="string">"file:qml/main.qml"</span>)<span class="operator">.</span>isRelative(); <span class="comment">// false: has "file" scheme</span>
///
/// </pre>
/// <p>Notice that a URL can be absolute while containing a relative path, and vice versa:</p>
/// <pre class="cpp">
///
///   <span class="comment">// Absolute URL, relative path</span>
///   <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span> url(<span class="string">"file:file.txt"</span>);
///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> url<span class="operator">.</span>isRelative();                 <span class="comment">// false: has "file" scheme</span>
///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdir.html">QDir</a></span><span class="operator">::</span>isAbsolutePath(url<span class="operator">.</span>path()); <span class="comment">// false: relative path</span>
///
///   <span class="comment">// Relative URL, absolute path</span>
///   url <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span>(<span class="string">"/home/user/file.txt"</span>);
///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> url<span class="operator">.</span>isRelative();                 <span class="comment">// true: has no scheme</span>
///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdir.html">QDir</a></span><span class="operator">::</span>isAbsolutePath(url<span class="operator">.</span>path()); <span class="comment">// true: absolute path</span>
///
/// </pre>
/// <p>A relative URL can be resolved by passing it as an argument to <a href="http://doc.qt.io/qt-5/qurl.html#resolved">resolved</a>(), which returns an absolute URL. <a href="http://doc.qt.io/qt-5/qurl.html#isParentOf">isParentOf</a>() is used for determining whether one URL is a parent of another.</p>
/// <a name="error-checking"></a>
/// <h4>Error checking</h4>
/// <p><a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> is capable of detecting many errors in URLs while parsing it or when components of the URL are set with individual setter methods (like <a href="http://doc.qt.io/qt-5/qurl.html#setScheme">setScheme</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#setHost">setHost</a>() or <a href="http://doc.qt.io/qt-5/qurl.html#setPath">setPath</a>()). If the parsing or setter function is successful, any previously recorded error conditions will be discarded.</p>
/// <p>By default, <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> setter methods operate in <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">QUrl::TolerantMode</a>, which means they accept some common mistakes and mis-representation of data. An alternate method of parsing is <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">QUrl::StrictMode</a>, which applies further checks. See <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">QUrl::ParsingMode</a> for a description of the difference of the parsing modes.</p>
/// <p><a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> only checks for conformance with the URL specification. It does not try to verify that high-level protocol URLs are in the format they are expected to be by handlers elsewhere. For example, the following URIs are all considered valid by <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a>, even if they do not make sense when used:</p>
/// <ul>
/// <li>"http:/filename.html"</li>
/// <li>"mailto://example.com"</li>
/// </ul>
/// <p>When the parser encounters an error, it signals the event by making <a href="http://doc.qt.io/qt-5/qurl.html#isValid">isValid</a>() return false and <a href="http://doc.qt.io/qt-5/qurl.html#toString">toString</a>() / <a href="http://doc.qt.io/qt-5/qurl.html#toEncoded">toEncoded</a>() return an empty string. If it is necessary to show the user the reason why the URL failed to parse, the error condition can be obtained from <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> by calling <a href="http://doc.qt.io/qt-5/qurl.html#errorString">errorString</a>(). Note that this message is highly technical and may not make sense to end-users.</p>
/// <p><a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> is capable of recording only one error condition. If more than one error is found, it is undefined which error is reported.</p>
/// <a name="character-conversions"></a>
/// <h4>Character Conversions</h4>
/// <p>Follow these rules to avoid erroneous character conversion when dealing with URLs and strings:</p>
/// <ul>
/// <li>When creating a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> to contain a URL from a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> or a char*, always use <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">QString::fromUtf8</a>().</li>
/// </ul></div>
#[repr(C)]
pub struct QUrl {
    _unused: u8,
}
impl QUrl {
    /// <p>Returns an adjusted version of the URL. The output can be customized by passing flags with <i>options</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QUrl QUrl::adjusted(QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption> options) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#adjusted">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an adjusted version of the URL. The output can be customized by passing flags with <i>options</i>.</p>
    /// <p>The encoding options from <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::ComponentFormattingOption</a> don't make much sense for this method, nor does <a href="http://doc.qt.io/qt-5/qurl.html#UrlFormattingOption-enum">QUrl::PreferLocalFile</a>.</p>
    /// <p>This is always equivalent to <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a>(url.<a href="http://doc.qt.io/qt-5/qurl.html#toString">toString</a>(options)).</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#UrlFormattingOption-enum">FormattingOptions</a>, <a href="http://doc.qt.io/qt-5/qurl.html#toEncoded">toEncoded</a>(), and <a href="http://doc.qt.io/qt-5/qurl.html#toString">toString</a>().</p></div>
    pub unsafe fn adjusted(
        &self,
        options: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>,
        >,
    ) -> ::cpp_utils::CppBox<crate::QUrl> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_adjusted(
            self as *const crate::QUrl,
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>,
            >::cast_into(options)
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the authority of the URL if it is defined; otherwise an empty string is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::authority(QFlags<QUrl::ComponentFormattingOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#authority">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the authority of the URL if it is defined; otherwise an empty string is returned.</p>
    /// <p>This function returns an unambiguous value, which may contain that characters still percent-encoded, plus some control sequences not representable in decoded form in <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p>The <i>options</i> argument controls how to format the user info component. The value of <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> is not permitted in this function. If you need to obtain fully decoded data, call <a href="http://doc.qt.io/qt-5/qurl.html#userName">userName</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#password">password</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#host">host</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#port">port</a>() individually.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#setAuthority">setAuthority</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#userInfo">userInfo</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#userName">userName</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#password">password</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#host">host</a>(), and <a href="http://doc.qt.io/qt-5/qurl.html#port">port</a>().</p></div>
    pub unsafe fn authority_1a(
        &self,
        options: crate::QFlags<crate::q_url::ComponentFormattingOption>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_authority(
            self as *const crate::QUrl,
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the authority of the URL if it is defined; otherwise an empty string is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::authority() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#authority">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the authority of the URL if it is defined; otherwise an empty string is returned.</p>
    /// <p>This function returns an unambiguous value, which may contain that characters still percent-encoded, plus some control sequences not representable in decoded form in <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p>The <i>options</i> argument controls how to format the user info component. The value of <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> is not permitted in this function. If you need to obtain fully decoded data, call <a href="http://doc.qt.io/qt-5/qurl.html#userName">userName</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#password">password</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#host">host</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#port">port</a>() individually.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#setAuthority">setAuthority</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#userInfo">userInfo</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#userName">userName</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#password">password</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#host">host</a>(), and <a href="http://doc.qt.io/qt-5/qurl.html#port">port</a>().</p></div>
    pub unsafe fn authority_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_authority1(self as *const crate::QUrl);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Resets the content of the <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a>. After calling this function, the <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> is equal to one that has been constructed with the default empty constructor.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrl::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Resets the content of the <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a>. After calling this function, the <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> is equal to one that has been constructed with the default empty constructor.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QUrl_clear(self as *mut crate::QUrl)
    }

    /// <p>Assigns the specified <i>url</i> to this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QUrl& QUrl::operator=(const QUrl& copy)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns the specified <i>url</i> to this object.</p></div>
    pub unsafe fn copy_from_q_url(
        &mut self,
        copy: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
    ) -> ::cpp_utils::MutRef<crate::QUrl> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_operator_(
            self as *mut crate::QUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(copy).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Assigns the specified <i>url</i> to this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QUrl& QUrl::operator=(const QString& url)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#operator-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns the specified <i>url</i> to this object.</p></div>
    pub unsafe fn copy_from_q_string(
        &mut self,
        url: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::MutRef<crate::QUrl> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_operator_1(
            self as *mut crate::QUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(url).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```void QUrl::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QUrl_detach(self as *mut crate::QUrl)
    }

    /// <p>Returns an error message if the last operation that modified this <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> object ran into a parsing error. If no error was detected, this function returns an empty string and <a href="http://doc.qt.io/qt-5/qurl.html#isValid">isValid</a>() returns <code>true</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::errorString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#errorString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an error message if the last operation that modified this <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> object ran into a parsing error. If no error was detected, this function returns an empty string and <a href="http://doc.qt.io/qt-5/qurl.html#isValid">isValid</a>() returns <code>true</code>.</p>
    /// <p>The error message returned by this function is technical in nature and may not be understood by end users. It is mostly useful to developers trying to understand why <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> will not accept some input.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">QUrl::ParsingMode</a>.</p></div>
    pub unsafe fn error_string(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_errorString(self as *const crate::QUrl);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the name of the file, excluding the directory path.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::fileName(QFlags<QUrl::ComponentFormattingOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#fileName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the name of the file, excluding the directory path.</p>
    /// <p>Note that, if this <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> object is given a path ending in a slash, the name of the file is considered empty.</p>
    /// <p>If the path doesn't contain any slash, it is fully returned as the fileName.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span> url(<span class="string">"http://qt-project.org/support/file.html"</span>);
    ///   <span class="comment">// url.adjusted(RemoveFilename) == "http://qt-project.org/support/"</span>
    ///   <span class="comment">// url.fileName() == "file.html"</span>
    ///
    /// </pre>
    /// <p>The <i>options</i> argument controls how to format the file name component. All values produce an unambiguous result. With <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a>, all percent-encoded sequences are decoded; otherwise, the returned value may contain some percent-encoded sequences for some control sequences not representable in decoded form in <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#path">path</a>().</p></div>
    pub unsafe fn file_name_1a(
        &self,
        options: crate::QFlags<crate::q_url::ComponentFormattingOption>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_fileName(
            self as *const crate::QUrl,
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the name of the file, excluding the directory path.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::fileName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#fileName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the name of the file, excluding the directory path.</p>
    /// <p>Note that, if this <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> object is given a path ending in a slash, the name of the file is considered empty.</p>
    /// <p>If the path doesn't contain any slash, it is fully returned as the fileName.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span> url(<span class="string">"http://qt-project.org/support/file.html"</span>);
    ///   <span class="comment">// url.adjusted(RemoveFilename) == "http://qt-project.org/support/"</span>
    ///   <span class="comment">// url.fileName() == "file.html"</span>
    ///
    /// </pre>
    /// <p>The <i>options</i> argument controls how to format the file name component. All values produce an unambiguous result. With <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a>, all percent-encoded sequences are decoded; otherwise, the returned value may contain some percent-encoded sequences for some control sequences not representable in decoded form in <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#path">path</a>().</p></div>
    pub unsafe fn file_name_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_fileName1(self as *const crate::QUrl);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the fragment of the URL. To determine if the parsed URL contained a fragment, use <a href="http://doc.qt.io/qt-5/qurl.html#hasFragment">hasFragment</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::fragment(QFlags<QUrl::ComponentFormattingOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#fragment">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the fragment of the URL. To determine if the parsed URL contained a fragment, use <a href="http://doc.qt.io/qt-5/qurl.html#hasFragment">hasFragment</a>().</p>
    /// <p>The <i>options</i> argument controls how to format the fragment component. All values produce an unambiguous result. With <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a>, all percent-encoded sequences are decoded; otherwise, the returned value may contain some percent-encoded sequences for some control sequences not representable in decoded form in <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> may cause data loss if those non-representable sequences are present. It is recommended to use that value when the result will be used in a non-URL context.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#setFragment">setFragment</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#hasFragment">hasFragment</a>().</p></div>
    pub unsafe fn fragment_1a(
        &self,
        options: crate::QFlags<crate::q_url::ComponentFormattingOption>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_fragment(
            self as *const crate::QUrl,
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the fragment of the URL. To determine if the parsed URL contained a fragment, use <a href="http://doc.qt.io/qt-5/qurl.html#hasFragment">hasFragment</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::fragment() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#fragment">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the fragment of the URL. To determine if the parsed URL contained a fragment, use <a href="http://doc.qt.io/qt-5/qurl.html#hasFragment">hasFragment</a>().</p>
    /// <p>The <i>options</i> argument controls how to format the fragment component. All values produce an unambiguous result. With <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a>, all percent-encoded sequences are decoded; otherwise, the returned value may contain some percent-encoded sequences for some control sequences not representable in decoded form in <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> may cause data loss if those non-representable sequences are present. It is recommended to use that value when the result will be used in a non-URL context.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#setFragment">setFragment</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#hasFragment">hasFragment</a>().</p></div>
    pub unsafe fn fragment_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_fragment1(self as *const crate::QUrl);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the Unicode form of the given domain name <i>domain</i>, which is encoded in the ASCII Compatible Encoding (ACE). The result of this function is considered equivalent to <i>domain</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QUrl::fromAce(const QByteArray& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#fromAce">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the Unicode form of the given domain name <i>domain</i>, which is encoded in the ASCII Compatible Encoding (ACE). The result of this function is considered equivalent to <i>domain</i>.</p>
    /// <p>If the value in <i>domain</i> cannot be encoded, it will be converted to <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and returned.</p>
    /// <p>The ASCII Compatible Encoding (ACE) is defined by RFC 3490, RFC 3491 and RFC 3492. It is part of the Internationalizing Domain Names in Applications (IDNA) specification, which allows for domain names (like <code>"example.com"</code>) to be written using international characters.</p>
    /// <p>This function was introduced in  Qt 4.2.</p></div>
    pub unsafe fn from_ace(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_fromAce(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Parses <i>input</i> and returns the corresponding <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a>. <i>input</i> is assumed to be in encoded form, containing only ASCII characters.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QUrl QUrl::fromEncoded(const QByteArray& url, QUrl::ParsingMode mode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#fromEncoded">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Parses <i>input</i> and returns the corresponding <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a>. <i>input</i> is assumed to be in encoded form, containing only ASCII characters.</p>
    /// <p>Parses the URL using <i>parsingMode</i>. See <a href="http://doc.qt.io/qt-5/qurl.html#setUrl">setUrl</a>() for more information on this parameter. <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">QUrl::DecodedMode</a> is not permitted in this context.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#toEncoded">toEncoded</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#setUrl">setUrl</a>().</p></div>
    pub unsafe fn from_encoded_2a(
        url: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        mode: crate::q_url::ParsingMode,
    ) -> ::cpp_utils::CppBox<crate::QUrl> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_fromEncoded(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(url)
                .as_raw_ptr(),
            mode,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Parses <i>input</i> and returns the corresponding <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a>. <i>input</i> is assumed to be in encoded form, containing only ASCII characters.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QUrl QUrl::fromEncoded(const QByteArray& url)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#fromEncoded">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Parses <i>input</i> and returns the corresponding <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a>. <i>input</i> is assumed to be in encoded form, containing only ASCII characters.</p>
    /// <p>Parses the URL using <i>parsingMode</i>. See <a href="http://doc.qt.io/qt-5/qurl.html#setUrl">setUrl</a>() for more information on this parameter. <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">QUrl::DecodedMode</a> is not permitted in this context.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#toEncoded">toEncoded</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#setUrl">setUrl</a>().</p></div>
    pub unsafe fn from_encoded_1a(
        url: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QUrl> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_fromEncoded1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(url)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> representation of <i>localFile</i>, interpreted as a local file. This function accepts paths separated by slashes as well as the native separator for this platform.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QUrl QUrl::fromLocalFile(const QString& localfile)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#fromLocalFile">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> representation of <i>localFile</i>, interpreted as a local file. This function accepts paths separated by slashes as well as the native separator for this platform.</p>
    /// <p>This function also accepts paths with a doubled leading slash (or backslash) to indicate a remote file, as in "//servername/path/to/file.txt". Note that only certain platforms can actually open this file using <a href="http://doc.qt.io/qt-5/qfile.html#open">QFile::open</a>().</p>
    /// <p>An empty <i>localFile</i> leads to an empty URL (since Qt 5.4).</p>
    /// <pre class="cpp">
    ///
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span><span class="operator">::</span>fromLocalFile(<span class="string">"file.txt"</span>);            <span class="comment">// QUrl("file:file.txt")</span>
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span><span class="operator">::</span>fromLocalFile(<span class="string">"/home/user/file.txt"</span>); <span class="comment">// QUrl("file:///home/user/file.txt")</span>
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span><span class="operator">::</span>fromLocalFile(<span class="string">"file:file.txt"</span>);       <span class="comment">// doesn't make sense; expects path, not url with scheme</span>
    ///
    /// </pre>
    /// <p>In the first line in snippet above, a file URL is constructed from a local, relative path. A file URL with a relative path only makes sense if there is a base URL to resolve it against. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span> url <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span><span class="operator">::</span>fromLocalFile(<span class="string">"file.txt"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span> baseUrl <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span>(<span class="string">"file:/home/user/"</span>);
    ///   <span class="comment">// wrong: prints QUrl("file:file.txt"), as url already has a scheme</span>
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> baseUrl<span class="operator">.</span>resolved(url);
    ///
    /// </pre>
    /// <p>To resolve such a URL, it's necessary to remove the scheme beforehand:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="comment">// correct: prints QUrl("file:///home/user/file.txt")</span>
    ///   url<span class="operator">.</span>setScheme(<span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>());
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> baseUrl<span class="operator">.</span>resolved(url);
    ///
    /// </pre>
    /// <p>For this reason, it is better to use a relative URL (that is, no scheme) for relative file paths:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span> url <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span>(<span class="string">"file.txt"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span> baseUrl <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span>(<span class="string">"file:/home/user/"</span>);
    ///   <span class="comment">// prints QUrl("file:///home/user/file.txt")</span>
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> baseUrl<span class="operator">.</span>resolved(url);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#toLocalFile">toLocalFile</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#isLocalFile">isLocalFile</a>(), and <a href="http://doc.qt.io/qt-5/qdir.html#toNativeSeparators">QDir::toNativeSeparators</a>().</p></div>
    pub unsafe fn from_local_file(
        localfile: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QUrl> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_fromLocalFile(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(localfile)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a decoded copy of <i>input</i>. <i>input</i> is first decoded from percent encoding, then converted from UTF-8 to unicode.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QString QUrl::fromPercentEncoding(const QByteArray& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#fromPercentEncoding">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a decoded copy of <i>input</i>. <i>input</i> is first decoded from percent encoding, then converted from UTF-8 to unicode.</p>
    /// <p><b>Note: </b>Given invalid input (such as a string containing the sequence "%G5", which is not a valid hexadecimal number) the output will be invalid as well. As an example: the sequence "%G5" could be decoded to 'W'.</p></div>
    pub unsafe fn from_percent_encoding(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_fromPercentEncoding(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts a list of strings representing <i>urls</i> into a list of urls, using <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a>(str, <i>mode</i>). Note that this means all strings must be urls, not for instance local paths.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QList<QUrl> QUrl::fromStringList(const QStringList& uris, QUrl::ParsingMode mode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#fromStringList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts a list of strings representing <i>urls</i> into a list of urls, using <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a>(str, <i>mode</i>). Note that this means all strings must be urls, not for instance local paths.</p>
    /// <p>This function was introduced in  Qt 5.1.</p></div>
    pub unsafe fn from_string_list_2a(
        uris: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
        mode: crate::q_url::ParsingMode,
    ) -> ::cpp_utils::CppBox<crate::QListOfQUrl> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_fromStringList(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(uris)
                .as_raw_ptr(),
            mode,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts a list of strings representing <i>urls</i> into a list of urls, using <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a>(str, <i>mode</i>). Note that this means all strings must be urls, not for instance local paths.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QList<QUrl> QUrl::fromStringList(const QStringList& uris)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#fromStringList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts a list of strings representing <i>urls</i> into a list of urls, using <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a>(str, <i>mode</i>). Note that this means all strings must be urls, not for instance local paths.</p>
    /// <p>This function was introduced in  Qt 5.1.</p></div>
    pub unsafe fn from_string_list_1a(
        uris: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQUrl> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_fromStringList1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(uris)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a valid URL from a user supplied <i>userInput</i> string if one can be deducted. In the case that is not possible, an invalid <a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a>() is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QUrl QUrl::fromUserInput(const QString& userInput)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#fromUserInput">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a valid URL from a user supplied <i>userInput</i> string if one can be deducted. In the case that is not possible, an invalid <a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a>() is returned.</p>
    /// <p>Most applications that can browse the web, allow the user to input a URL in the form of a plain string. This string can be manually typed into a location bar, obtained from the clipboard, or passed in via command line arguments.</p>
    /// <p>When the string is not already a valid URL, a best guess is performed, making various web related assumptions.</p>
    /// <p>In the case the string corresponds to a valid file path on the system, a file:// URL is constructed, using <a href="http://doc.qt.io/qt-5/qurl.html#fromLocalFile">QUrl::fromLocalFile</a>().</p>
    /// <p>If that is not the case, an attempt is made to turn the string into a http:// or ftp:// URL. The latter in the case the string starts with 'ftp'. The result is then passed through <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a>'s tolerant parser, and in the case or success, a valid <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> is returned, or else a <a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a>().</p>
    /// <a name="examples"></a>
    /// <h4>Examples:</h4>
    /// <ul>
    /// <li>qt-project.org becomes http://qt-project.org</li>
    /// <li>ftp.qt-project.org becomes ftp://ftp.qt-project.org</li>
    /// <li>hostname becomes http://hostname</li>
    /// <li>/home/user/test.html becomes file:///home/user/test.html</li>
    /// </ul>
    /// <p>This function was introduced in  Qt 4.6.</p></div>
    pub unsafe fn from_user_input_1a(
        user_input: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QUrl> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_fromUserInput(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(user_input)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a valid URL from a user supplied <i>userInput</i> string if one can be deducted. In the case that is not possible, an invalid <a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a>() is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QUrl QUrl::fromUserInput(const QString& userInput, const QString& workingDirectory, QFlags<QUrl::UserInputResolutionOption> options = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#fromUserInput-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a valid URL from a user supplied <i>userInput</i> string if one can be deducted. In the case that is not possible, an invalid <a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a>() is returned.</p>
    /// <p>This overload takes a <i>workingDirectory</i> path, in order to be able to handle relative paths. This is especially useful when handling command line arguments. If <i>workingDirectory</i> is empty, no handling of relative paths will be done, so this method will behave like its one argument overload.</p>
    /// <p>By default, an input string that looks like a relative path will only be treated as such if the file actually exists in the given working directory.</p>
    /// <p>If the application can handle files that don't exist yet, it should pass the flag <a href="http://doc.qt.io/qt-5/qurl.html#UserInputResolutionOption-enum">AssumeLocalFile</a> in <i>options</i>.</p>
    /// <p>This function was introduced in  Qt 5.4.</p></div>
    pub unsafe fn from_user_input_3a(
        user_input: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        working_directory: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        options: crate::QFlags<crate::q_url::UserInputResolutionOption>,
    ) -> ::cpp_utils::CppBox<crate::QUrl> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_fromUserInput1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(user_input)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(working_directory)
                .as_raw_ptr(),
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a valid URL from a user supplied <i>userInput</i> string if one can be deducted. In the case that is not possible, an invalid <a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a>() is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QUrl QUrl::fromUserInput(const QString& userInput, const QString& workingDirectory)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#fromUserInput-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a valid URL from a user supplied <i>userInput</i> string if one can be deducted. In the case that is not possible, an invalid <a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a>() is returned.</p>
    /// <p>This overload takes a <i>workingDirectory</i> path, in order to be able to handle relative paths. This is especially useful when handling command line arguments. If <i>workingDirectory</i> is empty, no handling of relative paths will be done, so this method will behave like its one argument overload.</p>
    /// <p>By default, an input string that looks like a relative path will only be treated as such if the file actually exists in the given working directory.</p>
    /// <p>If the application can handle files that don't exist yet, it should pass the flag <a href="http://doc.qt.io/qt-5/qurl.html#UserInputResolutionOption-enum">AssumeLocalFile</a> in <i>options</i>.</p>
    /// <p>This function was introduced in  Qt 5.4.</p></div>
    pub unsafe fn from_user_input_2a(
        user_input: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        working_directory: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QUrl> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_fromUserInput2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(user_input)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(working_directory)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if this URL contains a fragment (i.e., if # was seen on it).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QUrl::hasFragment() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#hasFragment">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this URL contains a fragment (i.e., if # was seen on it).</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#fragment">fragment</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#setFragment">setFragment</a>().</p></div>
    pub unsafe fn has_fragment(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QUrl_hasFragment(self as *const crate::QUrl)
    }

    /// <p>Returns <code>true</code> if this URL contains a Query (i.e., if ? was seen on it).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QUrl::hasQuery() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#hasQuery">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this URL contains a Query (i.e., if ? was seen on it).</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#setQuery">setQuery</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#query">query</a>(), and <a href="http://doc.qt.io/qt-5/qurl.html#hasFragment">hasFragment</a>().</p></div>
    pub unsafe fn has_query(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QUrl_hasQuery(self as *const crate::QUrl)
    }

    /// <p>Returns the host of the URL if it is defined; otherwise an empty string is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::host(QFlags<QUrl::ComponentFormattingOption> arg1 = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#host">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the host of the URL if it is defined; otherwise an empty string is returned.</p>
    /// <p>The <i>options</i> argument controls how the hostname will be formatted. The <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::EncodeUnicode</a> option will cause this function to return the hostname in the ASCII-Compatible Encoding (ACE) form, which is suitable for use in channels that are not 8-bit clean or that require the legacy hostname (such as DNS requests or in HTTP request headers). If that flag is not present, this function returns the International Domain Name (IDN) in Unicode form, according to the list of permissible top-level domains (see <a href="http://doc.qt.io/qt-5/qurl.html#idnWhitelist">idnWhitelist</a>()).</p>
    /// <p>All other flags are ignored. Host names cannot contain control or percent characters, so the returned value can be considered fully decoded.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#setHost">setHost</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#idnWhitelist">idnWhitelist</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#setIdnWhitelist">setIdnWhitelist</a>(), and <a href="http://doc.qt.io/qt-5/qurl.html#authority">authority</a>().</p></div>
    pub unsafe fn host_1a(
        &self,
        arg1: crate::QFlags<crate::q_url::ComponentFormattingOption>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QUrl_host(self as *const crate::QUrl, arg1.to_int());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the host of the URL if it is defined; otherwise an empty string is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::host() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#host">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the host of the URL if it is defined; otherwise an empty string is returned.</p>
    /// <p>The <i>options</i> argument controls how the hostname will be formatted. The <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::EncodeUnicode</a> option will cause this function to return the hostname in the ASCII-Compatible Encoding (ACE) form, which is suitable for use in channels that are not 8-bit clean or that require the legacy hostname (such as DNS requests or in HTTP request headers). If that flag is not present, this function returns the International Domain Name (IDN) in Unicode form, according to the list of permissible top-level domains (see <a href="http://doc.qt.io/qt-5/qurl.html#idnWhitelist">idnWhitelist</a>()).</p>
    /// <p>All other flags are ignored. Host names cannot contain control or percent characters, so the returned value can be considered fully decoded.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#setHost">setHost</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#idnWhitelist">idnWhitelist</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#setIdnWhitelist">setIdnWhitelist</a>(), and <a href="http://doc.qt.io/qt-5/qurl.html#authority">authority</a>().</p></div>
    pub unsafe fn host_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_host1(self as *const crate::QUrl);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the current whitelist of top-level domains that are allowed to have non-ASCII characters in their compositions.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QStringList QUrl::idnWhitelist()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#idnWhitelist">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current whitelist of top-level domains that are allowed to have non-ASCII characters in their compositions.</p>
    /// <p>See <a href="http://doc.qt.io/qt-5/qurl.html#setIdnWhitelist">setIdnWhitelist</a>() for the rationale of this list.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#setIdnWhitelist">setIdnWhitelist</a>().</p></div>
    pub unsafe fn idn_whitelist() -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_idnWhitelist();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QUrl::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QUrl_isDetached(self as *const crate::QUrl)
    }

    /// <p>Returns <code>true</code> if the URL has no data; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QUrl::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the URL has no data; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#clear">clear</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QUrl_isEmpty(self as *const crate::QUrl)
    }

    /// <p>Returns <code>true</code> if this URL is pointing to a local file path. A URL is a local file path if the scheme is "file".</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QUrl::isLocalFile() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#isLocalFile">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this URL is pointing to a local file path. A URL is a local file path if the scheme is "file".</p>
    /// <p>Note that this function considers URLs with hostnames to be local file paths, even if the eventual file path cannot be opened with <a href="http://doc.qt.io/qt-5/qfile.html#open">QFile::open</a>().</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#fromLocalFile">fromLocalFile</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#toLocalFile">toLocalFile</a>().</p></div>
    pub unsafe fn is_local_file(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QUrl_isLocalFile(self as *const crate::QUrl)
    }

    /// <p>Returns <code>true</code> if this URL is a parent of <i>childUrl</i>. <i>childUrl</i> is a child of this URL if the two URLs share the same scheme and authority, and this URL's path is a parent of the path of <i>childUrl</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QUrl::isParentOf(const QUrl& url) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#isParentOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this URL is a parent of <i>childUrl</i>. <i>childUrl</i> is a child of this URL if the two URLs share the same scheme and authority, and this URL's path is a parent of the path of <i>childUrl</i>.</p></div>
    pub unsafe fn is_parent_of(
        &self,
        url: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QUrl_isParentOf(
            self as *const crate::QUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(url).as_raw_ptr(),
        )
    }

    /// <p>Returns <code>true</code> if the URL is relative; otherwise returns <code>false</code>. A URL is relative reference if its scheme is undefined; this function is therefore equivalent to calling <a href="http://doc.qt.io/qt-5/qurl.html#scheme">scheme</a>().<a href="http://doc.qt.io/qt-5/qurl.html#isEmpty">isEmpty</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QUrl::isRelative() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#isRelative">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the URL is relative; otherwise returns <code>false</code>. A URL is relative reference if its scheme is undefined; this function is therefore equivalent to calling <a href="http://doc.qt.io/qt-5/qurl.html#scheme">scheme</a>().<a href="http://doc.qt.io/qt-5/qurl.html#isEmpty">isEmpty</a>().</p>
    /// <p>Relative references are defined in RFC 3986 section 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#relative-urls-vs-relative-paths">Relative URLs vs Relative Paths</a>.</p></div>
    pub unsafe fn is_relative(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QUrl_isRelative(self as *const crate::QUrl)
    }

    /// <p>Returns <code>true</code> if the URL is non-empty and valid; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QUrl::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the URL is non-empty and valid; otherwise returns <code>false</code>.</p>
    /// <p>The URL is run through a conformance test. Every part of the URL must conform to the standard encoding rules of the URI standard for the URL to be reported as valid.</p>
    /// <pre class="cpp">
    ///
    ///   bool checkUrl(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span> <span class="operator">&amp;</span>url) {
    /// &#32;     <span class="keyword">if</span> (<span class="operator">!</span>url<span class="operator">.</span>isValid()) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>(<span class="string">"Invalid URL: %s"</span><span class="operator">,</span> qUtf8Printable(url<span class="operator">.</span>toString()));
    /// &#32;   &#32;     <span class="keyword">return</span> <span class="keyword">false</span>;
    /// &#32;     }
    ///
    /// &#32;     <span class="keyword">return</span> <span class="keyword">true</span>;
    ///   }
    ///
    /// </pre></div>
    pub unsafe fn is_valid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QUrl_isValid(self as *const crate::QUrl)
    }

    /// <p>Returns <code>true</code> if this URL and the given <i>url</i> are equal after applying <i>options</i> to both; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QUrl::matches(const QUrl& url, QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption> options) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#matches">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this URL and the given <i>url</i> are equal after applying <i>options</i> to both; otherwise returns <code>false</code>.</p>
    /// <p>This is equivalent to calling adjusted(options) on both URLs and comparing the resulting urls, but faster.</p>
    /// <p>This function was introduced in  Qt 5.2.</p></div>
    pub unsafe fn matches(
        &self,
        url: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
        options: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>,
        >,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QUrl_matches(
            self as *const crate::QUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(url).as_raw_ptr(),
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>,
            >::cast_into(options)
            .as_raw_ptr(),
        )
    }

    /// <p>Constructs an empty <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QUrl::QUrl()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#QUrl">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> object.</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QUrl> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_QUrl();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a URL by parsing <i>url</i>. <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> will automatically percent encode all characters that are not allowed in a URL and decode the percent-encoded sequences that represent an unreserved character (letters, digits, hyphens, undercores, dots and tildes). All other characters are left in their original forms.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QUrl::QUrl(const QString& url, QUrl::ParsingMode mode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#QUrl-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a URL by parsing <i>url</i>. <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> will automatically percent encode all characters that are not allowed in a URL and decode the percent-encoded sequences that represent an unreserved character (letters, digits, hyphens, undercores, dots and tildes). All other characters are left in their original forms.</p>
    /// <p>Parses the <i>url</i> using the parser mode <i>parsingMode</i>. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">TolerantMode</a> (the default), <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> will correct certain mistakes, notably the presence of a percent character ('%') not followed by two hexadecimal digits, and it will accept any character in any position. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">StrictMode</a>, encoding mistakes will not be tolerated and <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> will also check that certain forbidden characters are not present in unencoded form. If an error is detected in <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">StrictMode</a>, <a href="http://doc.qt.io/qt-5/qurl.html#isValid">isValid</a>() will return false. The parsing mode <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">DecodedMode</a> is not permitted in this context.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span> url(<span class="string">"http://www.example.com/List of holidays.xml"</span>);
    ///   <span class="comment">// url.toEncoded() == "http://www.example.com/List%20of%20holidays.xml"</span>
    ///
    /// </pre>
    /// <p>To construct a URL from an encoded string, you can also use <a href="http://doc.qt.io/qt-5/qurl.html#fromEncoded">fromEncoded</a>():</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span> url <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span><span class="operator">::</span>fromEncoded(<span class="string">"http://qt-project.org/List%20of%20holidays.xml"</span>);
    ///
    /// </pre>
    /// <p>Both functions are equivalent and, in Qt 5, both functions accept encoded data. Usually, the choice of the <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> constructor or <a href="http://doc.qt.io/qt-5/qurl.html#setUrl">setUrl</a>() versus <a href="http://doc.qt.io/qt-5/qurl.html#fromEncoded">fromEncoded</a>() will depend on the source data: the constructor and <a href="http://doc.qt.io/qt-5/qurl.html#setUrl">setUrl</a>() take a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, whereas <a href="http://doc.qt.io/qt-5/qurl.html#fromEncoded">fromEncoded</a> takes a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#setUrl">setUrl</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#fromEncoded">fromEncoded</a>(), and <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">TolerantMode</a>.</p></div>
    pub unsafe fn new_2a(
        url: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        mode: crate::q_url::ParsingMode,
    ) -> ::cpp_utils::CppBox<crate::QUrl> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_QUrl2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(url).as_raw_ptr(),
            mode,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a URL by parsing <i>url</i>. <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> will automatically percent encode all characters that are not allowed in a URL and decode the percent-encoded sequences that represent an unreserved character (letters, digits, hyphens, undercores, dots and tildes). All other characters are left in their original forms.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QUrl::QUrl(const QString& url)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#QUrl-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a URL by parsing <i>url</i>. <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> will automatically percent encode all characters that are not allowed in a URL and decode the percent-encoded sequences that represent an unreserved character (letters, digits, hyphens, undercores, dots and tildes). All other characters are left in their original forms.</p>
    /// <p>Parses the <i>url</i> using the parser mode <i>parsingMode</i>. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">TolerantMode</a> (the default), <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> will correct certain mistakes, notably the presence of a percent character ('%') not followed by two hexadecimal digits, and it will accept any character in any position. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">StrictMode</a>, encoding mistakes will not be tolerated and <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> will also check that certain forbidden characters are not present in unencoded form. If an error is detected in <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">StrictMode</a>, <a href="http://doc.qt.io/qt-5/qurl.html#isValid">isValid</a>() will return false. The parsing mode <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">DecodedMode</a> is not permitted in this context.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span> url(<span class="string">"http://www.example.com/List of holidays.xml"</span>);
    ///   <span class="comment">// url.toEncoded() == "http://www.example.com/List%20of%20holidays.xml"</span>
    ///
    /// </pre>
    /// <p>To construct a URL from an encoded string, you can also use <a href="http://doc.qt.io/qt-5/qurl.html#fromEncoded">fromEncoded</a>():</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span> url <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span><span class="operator">::</span>fromEncoded(<span class="string">"http://qt-project.org/List%20of%20holidays.xml"</span>);
    ///
    /// </pre>
    /// <p>Both functions are equivalent and, in Qt 5, both functions accept encoded data. Usually, the choice of the <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> constructor or <a href="http://doc.qt.io/qt-5/qurl.html#setUrl">setUrl</a>() versus <a href="http://doc.qt.io/qt-5/qurl.html#fromEncoded">fromEncoded</a>() will depend on the source data: the constructor and <a href="http://doc.qt.io/qt-5/qurl.html#setUrl">setUrl</a>() take a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, whereas <a href="http://doc.qt.io/qt-5/qurl.html#fromEncoded">fromEncoded</a> takes a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#setUrl">setUrl</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#fromEncoded">fromEncoded</a>(), and <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">TolerantMode</a>.</p></div>
    pub unsafe fn new_1a(
        url: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QUrl> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_QUrl4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(url).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QUrl::QUrl(const QUrl& copy)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#QUrl-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p></div>
    pub unsafe fn new_copy(
        copy: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
    ) -> ::cpp_utils::CppBox<crate::QUrl> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_QUrl1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(copy).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the password of the URL if it is defined; otherwise an empty string is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::password(QFlags<QUrl::ComponentFormattingOption> arg1 = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#password">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the password of the URL if it is defined; otherwise an empty string is returned.</p>
    /// <p>The <i>options</i> argument controls how to format the user name component. All values produce an unambiguous result. With <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a>, all percent-encoded sequences are decoded; otherwise, the returned value may contain some percent-encoded sequences for some control sequences not representable in decoded form in <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> may cause data loss if those non-representable sequences are present. It is recommended to use that value when the result will be used in a non-URL context, such as setting in <a href="http://doc.qt.io/qt-5/qauthenticator.html">QAuthenticator</a> or negotiating a login.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#setPassword">setPassword</a>().</p></div>
    pub unsafe fn password_1a(
        &self,
        arg1: crate::QFlags<crate::q_url::ComponentFormattingOption>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QUrl_password(self as *const crate::QUrl, arg1.to_int());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the password of the URL if it is defined; otherwise an empty string is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::password() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#password">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the password of the URL if it is defined; otherwise an empty string is returned.</p>
    /// <p>The <i>options</i> argument controls how to format the user name component. All values produce an unambiguous result. With <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a>, all percent-encoded sequences are decoded; otherwise, the returned value may contain some percent-encoded sequences for some control sequences not representable in decoded form in <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> may cause data loss if those non-representable sequences are present. It is recommended to use that value when the result will be used in a non-URL context, such as setting in <a href="http://doc.qt.io/qt-5/qauthenticator.html">QAuthenticator</a> or negotiating a login.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#setPassword">setPassword</a>().</p></div>
    pub unsafe fn password_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_password1(self as *const crate::QUrl);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the path of the URL.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::path(QFlags<QUrl::ComponentFormattingOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#path">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the path of the URL.</p>
    /// <pre class="cpp">
    ///
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span>(<span class="string">"file:file.txt"</span>)<span class="operator">.</span>path();                   <span class="comment">// "file.txt"</span>
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span>(<span class="string">"/home/user/file.txt"</span>)<span class="operator">.</span>path();             <span class="comment">// "/home/user/file.txt"</span>
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span>(<span class="string">"http://www.example.com/test/123"</span>)<span class="operator">.</span>path(); <span class="comment">// "/test/123"</span>
    ///
    /// </pre>
    /// <p>The <i>options</i> argument controls how to format the path component. All values produce an unambiguous result. With <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a>, all percent-encoded sequences are decoded; otherwise, the returned value may contain some percent-encoded sequences for some control sequences not representable in decoded form in <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> may cause data loss if those non-representable sequences are present. It is recommended to use that value when the result will be used in a non-URL context, such as sending to an FTP server.</p>
    /// <p>An example of data loss is when you have non-Unicode percent-encoded sequences and use <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">FullyDecoded</a> (the default):</p>
    /// <pre class="cpp">
    ///
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span>(<span class="string">"/foo%FFbar"</span>)<span class="operator">.</span>path();
    ///
    /// </pre>
    /// <p>In this example, there will be some level of data loss because the <code>%FF</code> cannot be converted.</p>
    /// <p>Data loss can also occur when the path contains sub-delimiters (such as <code>+</code>):</p>
    /// <pre class="cpp">
    ///
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span>(<span class="string">"/foo+bar%2B"</span>)<span class="operator">.</span>path(); <span class="comment">// "/foo+bar+"</span>
    ///
    /// </pre>
    /// <p>Other decoding examples:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span> url(<span class="string">"/tmp/Mambo %235%3F.mp3"</span>);
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> url<span class="operator">.</span>path(<span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span><span class="operator">::</span>FullyDecoded);  <span class="comment">// "/tmp/Mambo #5?.mp3"</span>
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> url<span class="operator">.</span>path(<span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span><span class="operator">::</span>PrettyDecoded); <span class="comment">// "/tmp/Mambo #5?.mp3"</span>
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> url<span class="operator">.</span>path(<span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span><span class="operator">::</span>FullyEncoded);  <span class="comment">// "/tmp/Mambo%20%235%3F.mp3"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#setPath">setPath</a>().</p></div>
    pub unsafe fn path_1a(
        &self,
        options: crate::QFlags<crate::q_url::ComponentFormattingOption>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QUrl_path(self as *const crate::QUrl, options.to_int());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the path of the URL.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::path() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#path">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the path of the URL.</p>
    /// <pre class="cpp">
    ///
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span>(<span class="string">"file:file.txt"</span>)<span class="operator">.</span>path();                   <span class="comment">// "file.txt"</span>
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span>(<span class="string">"/home/user/file.txt"</span>)<span class="operator">.</span>path();             <span class="comment">// "/home/user/file.txt"</span>
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span>(<span class="string">"http://www.example.com/test/123"</span>)<span class="operator">.</span>path(); <span class="comment">// "/test/123"</span>
    ///
    /// </pre>
    /// <p>The <i>options</i> argument controls how to format the path component. All values produce an unambiguous result. With <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a>, all percent-encoded sequences are decoded; otherwise, the returned value may contain some percent-encoded sequences for some control sequences not representable in decoded form in <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> may cause data loss if those non-representable sequences are present. It is recommended to use that value when the result will be used in a non-URL context, such as sending to an FTP server.</p>
    /// <p>An example of data loss is when you have non-Unicode percent-encoded sequences and use <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">FullyDecoded</a> (the default):</p>
    /// <pre class="cpp">
    ///
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span>(<span class="string">"/foo%FFbar"</span>)<span class="operator">.</span>path();
    ///
    /// </pre>
    /// <p>In this example, there will be some level of data loss because the <code>%FF</code> cannot be converted.</p>
    /// <p>Data loss can also occur when the path contains sub-delimiters (such as <code>+</code>):</p>
    /// <pre class="cpp">
    ///
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span>(<span class="string">"/foo+bar%2B"</span>)<span class="operator">.</span>path(); <span class="comment">// "/foo+bar+"</span>
    ///
    /// </pre>
    /// <p>Other decoding examples:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span> url(<span class="string">"/tmp/Mambo %235%3F.mp3"</span>);
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> url<span class="operator">.</span>path(<span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span><span class="operator">::</span>FullyDecoded);  <span class="comment">// "/tmp/Mambo #5?.mp3"</span>
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> url<span class="operator">.</span>path(<span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span><span class="operator">::</span>PrettyDecoded); <span class="comment">// "/tmp/Mambo #5?.mp3"</span>
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> url<span class="operator">.</span>path(<span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span><span class="operator">::</span>FullyEncoded);  <span class="comment">// "/tmp/Mambo%20%235%3F.mp3"</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#setPath">setPath</a>().</p></div>
    pub unsafe fn path_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_path1(self as *const crate::QUrl);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the port of the URL, or <i>defaultPort</i> if the port is unspecified.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QUrl::port(int defaultPort = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#port">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the port of the URL, or <i>defaultPort</i> if the port is unspecified.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtcpsocket.html">QTcpSocket</a></span> sock;
    ///   sock<span class="operator">.</span>connectToHost(url<span class="operator">.</span>host()<span class="operator">,</span> url<span class="operator">.</span>port(<span class="number">80</span>));
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#setPort">setPort</a>().</p></div>
    pub unsafe fn port_1a(&self, default_port: ::std::os::raw::c_int) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QUrl_port(self as *const crate::QUrl, default_port)
    }

    /// <p>Returns the port of the URL, or <i>defaultPort</i> if the port is unspecified.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QUrl::port() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#port">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the port of the URL, or <i>defaultPort</i> if the port is unspecified.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtcpsocket.html">QTcpSocket</a></span> sock;
    ///   sock<span class="operator">.</span>connectToHost(url<span class="operator">.</span>host()<span class="operator">,</span> url<span class="operator">.</span>port(<span class="number">80</span>));
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#setPort">setPort</a>().</p></div>
    pub unsafe fn port_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QUrl_port1(self as *const crate::QUrl)
    }

    /// <p>Returns the query string of the URL if there's a query string, or an empty result if not. To determine if the parsed URL contained a query string, use <a href="http://doc.qt.io/qt-5/qurl.html#hasQuery">hasQuery</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::query(QFlags<QUrl::ComponentFormattingOption> arg1 = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#query">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the query string of the URL if there's a query string, or an empty result if not. To determine if the parsed URL contained a query string, use <a href="http://doc.qt.io/qt-5/qurl.html#hasQuery">hasQuery</a>().</p>
    /// <p>The <i>options</i> argument controls how to format the query component. All values produce an unambiguous result. With <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a>, all percent-encoded sequences are decoded; otherwise, the returned value may contain some percent-encoded sequences for some control sequences not representable in decoded form in <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p>Note that use of <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> in queries is discouraged, as queries often contain data that is supposed to remain percent-encoded, including the use of the "%2B" sequence to represent a plus character ('+').</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#setQuery">setQuery</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#hasQuery">hasQuery</a>().</p></div>
    pub unsafe fn query_1a(
        &self,
        arg1: crate::QFlags<crate::q_url::ComponentFormattingOption>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QUrl_query(self as *const crate::QUrl, arg1.to_int());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the query string of the URL if there's a query string, or an empty result if not. To determine if the parsed URL contained a query string, use <a href="http://doc.qt.io/qt-5/qurl.html#hasQuery">hasQuery</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::query() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#query">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the query string of the URL if there's a query string, or an empty result if not. To determine if the parsed URL contained a query string, use <a href="http://doc.qt.io/qt-5/qurl.html#hasQuery">hasQuery</a>().</p>
    /// <p>The <i>options</i> argument controls how to format the query component. All values produce an unambiguous result. With <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a>, all percent-encoded sequences are decoded; otherwise, the returned value may contain some percent-encoded sequences for some control sequences not representable in decoded form in <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p>Note that use of <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> in queries is discouraged, as queries often contain data that is supposed to remain percent-encoded, including the use of the "%2B" sequence to represent a plus character ('+').</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#setQuery">setQuery</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#hasQuery">hasQuery</a>().</p></div>
    pub unsafe fn query_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_query1(self as *const crate::QUrl);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the result of the merge of this URL with <i>relative</i>. This URL is used as a base to convert <i>relative</i> to an absolute URL.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QUrl QUrl::resolved(const QUrl& relative) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#resolved">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the result of the merge of this URL with <i>relative</i>. This URL is used as a base to convert <i>relative</i> to an absolute URL.</p>
    /// <p>If <i>relative</i> is not a relative URL, this function will return <i>relative</i> directly. Otherwise, the paths of the two URLs are merged, and the new URL returned has the scheme and authority of the base URL, but with the merged path, as in the following example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span> baseUrl(<span class="string">"http://qt.digia.com/Support/"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span> relativeUrl(<span class="string">"../Product/Library/"</span>);
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>(baseUrl<span class="operator">.</span>resolved(relativeUrl)<span class="operator">.</span>toString());
    ///   <span class="comment">// prints "http://qt.digia.com/Product/Library/"</span>
    ///
    /// </pre>
    /// <p>Calling resolved() with ".." returns a <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> whose directory is one level higher than the original. Similarly, calling resolved() with "../.." removes two levels from the path. If <i>relative</i> is "/", the path becomes "/".</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#isRelative">isRelative</a>().</p></div>
    pub unsafe fn resolved(
        &self,
        relative: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
    ) -> ::cpp_utils::CppBox<crate::QUrl> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_resolved(
            self as *const crate::QUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(relative)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the scheme of the URL. If an empty string is returned, this means the scheme is undefined and the URL is then relative.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::scheme() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#scheme">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the scheme of the URL. If an empty string is returned, this means the scheme is undefined and the URL is then relative.</p>
    /// <p>The scheme can only contain US-ASCII letters or digits, which means it cannot contain any character that would otherwise require encoding. Additionally, schemes are always returned in lowercase form.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#setScheme">setScheme</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#isRelative">isRelative</a>().</p></div>
    pub unsafe fn scheme(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_scheme(self as *const crate::QUrl);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Sets the authority of the URL to <i>authority</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrl::setAuthority(const QString& authority, QUrl::ParsingMode mode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#setAuthority">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the authority of the URL to <i>authority</i>.</p>
    /// <p>The authority of a URL is the combination of user info, a host name and a port. All of these elements are optional; an empty authority is therefore valid.</p>
    /// <p>The user info and host are separated by a '@', and the host and port are separated by a ':'. If the user info is empty, the '@' must be omitted; although a stray ':' is permitted if the port is empty.</p>
    /// <p>The following example shows a valid authority string:</p>
    /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qurl-authority.png" alt=""></p><p>The <i>authority</i> data is interpreted according to <i>mode</i>: in <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">StrictMode</a>, any '%' characters must be followed by exactly two hexadecimal characters and some characters (including space) are not allowed in undecoded form. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">TolerantMode</a> (the default), all characters are accepted in undecoded form and the tolerant parser will correct stray '%' not followed by two hex characters.</p>
    /// <p>This function does not allow <i>mode</i> to be <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">QUrl::DecodedMode</a>. To set fully decoded data, call <a href="http://doc.qt.io/qt-5/qurl.html#setUserName">setUserName</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#setPassword">setPassword</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#setHost">setHost</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#setPort">setPort</a>() individually.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#authority">authority</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#setUserInfo">setUserInfo</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#setHost">setHost</a>(), and <a href="http://doc.qt.io/qt-5/qurl.html#setPort">setPort</a>().</p></div>
    pub unsafe fn set_authority_2a(
        &mut self,
        authority: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        mode: crate::q_url::ParsingMode,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrl_setAuthority(
            self as *mut crate::QUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(authority)
                .as_raw_ptr(),
            mode,
        )
    }

    /// <p>Sets the authority of the URL to <i>authority</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrl::setAuthority(const QString& authority)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#setAuthority">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the authority of the URL to <i>authority</i>.</p>
    /// <p>The authority of a URL is the combination of user info, a host name and a port. All of these elements are optional; an empty authority is therefore valid.</p>
    /// <p>The user info and host are separated by a '@', and the host and port are separated by a ':'. If the user info is empty, the '@' must be omitted; although a stray ':' is permitted if the port is empty.</p>
    /// <p>The following example shows a valid authority string:</p>
    /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qurl-authority.png" alt=""></p><p>The <i>authority</i> data is interpreted according to <i>mode</i>: in <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">StrictMode</a>, any '%' characters must be followed by exactly two hexadecimal characters and some characters (including space) are not allowed in undecoded form. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">TolerantMode</a> (the default), all characters are accepted in undecoded form and the tolerant parser will correct stray '%' not followed by two hex characters.</p>
    /// <p>This function does not allow <i>mode</i> to be <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">QUrl::DecodedMode</a>. To set fully decoded data, call <a href="http://doc.qt.io/qt-5/qurl.html#setUserName">setUserName</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#setPassword">setPassword</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#setHost">setHost</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#setPort">setPort</a>() individually.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#authority">authority</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#setUserInfo">setUserInfo</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#setHost">setHost</a>(), and <a href="http://doc.qt.io/qt-5/qurl.html#setPort">setPort</a>().</p></div>
    pub unsafe fn set_authority_1a(
        &mut self,
        authority: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrl_setAuthority1(
            self as *mut crate::QUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(authority)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the fragment of the URL to <i>fragment</i>. The fragment is the last part of the URL, represented by a '#' followed by a string of characters. It is typically used in HTTP for referring to a certain link or point on a page:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrl::setFragment(const QString& fragment, QUrl::ParsingMode mode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#setFragment">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the fragment of the URL to <i>fragment</i>. The fragment is the last part of the URL, represented by a '#' followed by a string of characters. It is typically used in HTTP for referring to a certain link or point on a page:</p>
    /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qurl-fragment.png" alt=""></p><p>The fragment is sometimes also referred to as the URL "reference".</p>
    /// <p>Passing an argument of QString() (a null <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>) will unset the fragment. Passing an argument of <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>("") (an empty but not null <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>) will set the fragment to an empty string (as if the original URL had a lone "#").</p>
    /// <p>The <i>fragment</i> data is interpreted according to <i>mode</i>: in <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">StrictMode</a>, any '%' characters must be followed by exactly two hexadecimal characters and some characters (including space) are not allowed in undecoded form. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">TolerantMode</a>, all characters are accepted in undecoded form and the tolerant parser will correct stray '%' not followed by two hex characters. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">DecodedMode</a>, '%' stand for themselves and encoded characters are not possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">QUrl::DecodedMode</a> should be used when setting the fragment from a data source which is not a URL or with a fragment obtained by calling <a href="http://doc.qt.io/qt-5/qurl.html#fragment">fragment</a>() with the <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> formatting option.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#fragment">fragment</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#hasFragment">hasFragment</a>().</p></div>
    pub unsafe fn set_fragment_2a(
        &mut self,
        fragment: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        mode: crate::q_url::ParsingMode,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrl_setFragment(
            self as *mut crate::QUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(fragment)
                .as_raw_ptr(),
            mode,
        )
    }

    /// <p>Sets the fragment of the URL to <i>fragment</i>. The fragment is the last part of the URL, represented by a '#' followed by a string of characters. It is typically used in HTTP for referring to a certain link or point on a page:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrl::setFragment(const QString& fragment)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#setFragment">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the fragment of the URL to <i>fragment</i>. The fragment is the last part of the URL, represented by a '#' followed by a string of characters. It is typically used in HTTP for referring to a certain link or point on a page:</p>
    /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qurl-fragment.png" alt=""></p><p>The fragment is sometimes also referred to as the URL "reference".</p>
    /// <p>Passing an argument of QString() (a null <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>) will unset the fragment. Passing an argument of <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>("") (an empty but not null <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>) will set the fragment to an empty string (as if the original URL had a lone "#").</p>
    /// <p>The <i>fragment</i> data is interpreted according to <i>mode</i>: in <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">StrictMode</a>, any '%' characters must be followed by exactly two hexadecimal characters and some characters (including space) are not allowed in undecoded form. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">TolerantMode</a>, all characters are accepted in undecoded form and the tolerant parser will correct stray '%' not followed by two hex characters. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">DecodedMode</a>, '%' stand for themselves and encoded characters are not possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">QUrl::DecodedMode</a> should be used when setting the fragment from a data source which is not a URL or with a fragment obtained by calling <a href="http://doc.qt.io/qt-5/qurl.html#fragment">fragment</a>() with the <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> formatting option.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#fragment">fragment</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#hasFragment">hasFragment</a>().</p></div>
    pub unsafe fn set_fragment_1a(
        &mut self,
        fragment: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrl_setFragment1(
            self as *mut crate::QUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(fragment)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the host of the URL to <i>host</i>. The host is part of the authority.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrl::setHost(const QString& host, QUrl::ParsingMode mode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#setHost">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the host of the URL to <i>host</i>. The host is part of the authority.</p>
    /// <p>The <i>host</i> data is interpreted according to <i>mode</i>: in <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">StrictMode</a>, any '%' characters must be followed by exactly two hexadecimal characters and some characters (including space) are not allowed in undecoded form. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">TolerantMode</a>, all characters are accepted in undecoded form and the tolerant parser will correct stray '%' not followed by two hex characters. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">DecodedMode</a>, '%' stand for themselves and encoded characters are not possible.</p>
    /// <p>Note that, in all cases, the result of the parsing must be a valid hostname according to STD 3 rules, as modified by the Internationalized Resource Identifiers specification (RFC 3987). Invalid hostnames are not permitted and will cause <a href="http://doc.qt.io/qt-5/qurl.html#isValid">isValid</a>() to become false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#host">host</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#setAuthority">setAuthority</a>().</p></div>
    pub unsafe fn set_host_2a(
        &mut self,
        host: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        mode: crate::q_url::ParsingMode,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrl_setHost(
            self as *mut crate::QUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(host).as_raw_ptr(),
            mode,
        )
    }

    /// <p>Sets the host of the URL to <i>host</i>. The host is part of the authority.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrl::setHost(const QString& host)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#setHost">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the host of the URL to <i>host</i>. The host is part of the authority.</p>
    /// <p>The <i>host</i> data is interpreted according to <i>mode</i>: in <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">StrictMode</a>, any '%' characters must be followed by exactly two hexadecimal characters and some characters (including space) are not allowed in undecoded form. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">TolerantMode</a>, all characters are accepted in undecoded form and the tolerant parser will correct stray '%' not followed by two hex characters. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">DecodedMode</a>, '%' stand for themselves and encoded characters are not possible.</p>
    /// <p>Note that, in all cases, the result of the parsing must be a valid hostname according to STD 3 rules, as modified by the Internationalized Resource Identifiers specification (RFC 3987). Invalid hostnames are not permitted and will cause <a href="http://doc.qt.io/qt-5/qurl.html#isValid">isValid</a>() to become false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#host">host</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#setAuthority">setAuthority</a>().</p></div>
    pub unsafe fn set_host_1a(
        &mut self,
        host: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrl_setHost1(
            self as *mut crate::QUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(host).as_raw_ptr(),
        )
    }

    /// <p>Sets the whitelist of Top-Level Domains (TLDs) that are allowed to have non-ASCII characters in domains to the value of <i>list</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static void QUrl::setIdnWhitelist(const QStringList& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#setIdnWhitelist">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the whitelist of Top-Level Domains (TLDs) that are allowed to have non-ASCII characters in domains to the value of <i>list</i>.</p>
    /// <p>Note that if you call this function, you need to do so <i>before</i> you start any threads that might access <a href="http://doc.qt.io/qt-5/qurl.html#idnWhitelist">idnWhitelist</a>().</p>
    /// <p>Qt comes with a default list that contains the Internet top-level domains that have published support for Internationalized Domain Names (IDNs) and rules to guarantee that no deception can happen between similarly-looking characters (such as the Latin lowercase letter <code>'a'</code> and the Cyrillic equivalent, which in most fonts are visually identical).</p>
    /// <p>This list is periodically maintained, as registrars publish new rules.</p>
    /// <p>This function is provided for those who need to manipulate the list, in order to add or remove a TLD. It is not recommended to change its value for purposes other than testing, as it may expose users to security risks.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#idnWhitelist">idnWhitelist</a>().</p></div>
    pub unsafe fn set_idn_whitelist(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrl_setIdnWhitelist(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(arg1)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the URL's password to <i>password</i>. The <i>password</i> is part of the user info element in the authority of the URL, as described in <a href="http://doc.qt.io/qt-5/qurl.html#setUserInfo">setUserInfo</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrl::setPassword(const QString& password, QUrl::ParsingMode mode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#setPassword">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the URL's password to <i>password</i>. The <i>password</i> is part of the user info element in the authority of the URL, as described in <a href="http://doc.qt.io/qt-5/qurl.html#setUserInfo">setUserInfo</a>().</p>
    /// <p>The <i>password</i> data is interpreted according to <i>mode</i>: in <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">StrictMode</a>, any '%' characters must be followed by exactly two hexadecimal characters and some characters (including space) are not allowed in undecoded form. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">TolerantMode</a>, all characters are accepted in undecoded form and the tolerant parser will correct stray '%' not followed by two hex characters. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">DecodedMode</a>, '%' stand for themselves and encoded characters are not possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">QUrl::DecodedMode</a> should be used when setting the password from a data source which is not a URL, such as a password dialog shown to the user or with a password obtained by calling <a href="http://doc.qt.io/qt-5/qurl.html#password">password</a>() with the <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> formatting option.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#password">password</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#setUserInfo">setUserInfo</a>().</p></div>
    pub unsafe fn set_password_2a(
        &mut self,
        password: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        mode: crate::q_url::ParsingMode,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrl_setPassword(
            self as *mut crate::QUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(password)
                .as_raw_ptr(),
            mode,
        )
    }

    /// <p>Sets the URL's password to <i>password</i>. The <i>password</i> is part of the user info element in the authority of the URL, as described in <a href="http://doc.qt.io/qt-5/qurl.html#setUserInfo">setUserInfo</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrl::setPassword(const QString& password)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#setPassword">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the URL's password to <i>password</i>. The <i>password</i> is part of the user info element in the authority of the URL, as described in <a href="http://doc.qt.io/qt-5/qurl.html#setUserInfo">setUserInfo</a>().</p>
    /// <p>The <i>password</i> data is interpreted according to <i>mode</i>: in <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">StrictMode</a>, any '%' characters must be followed by exactly two hexadecimal characters and some characters (including space) are not allowed in undecoded form. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">TolerantMode</a>, all characters are accepted in undecoded form and the tolerant parser will correct stray '%' not followed by two hex characters. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">DecodedMode</a>, '%' stand for themselves and encoded characters are not possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">QUrl::DecodedMode</a> should be used when setting the password from a data source which is not a URL, such as a password dialog shown to the user or with a password obtained by calling <a href="http://doc.qt.io/qt-5/qurl.html#password">password</a>() with the <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> formatting option.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#password">password</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#setUserInfo">setUserInfo</a>().</p></div>
    pub unsafe fn set_password_1a(
        &mut self,
        password: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrl_setPassword1(
            self as *mut crate::QUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(password)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the path of the URL to <i>path</i>. The path is the part of the URL that comes after the authority but before the query string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrl::setPath(const QString& path, QUrl::ParsingMode mode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#setPath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the path of the URL to <i>path</i>. The path is the part of the URL that comes after the authority but before the query string.</p>
    /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qurl-ftppath.png" alt=""></p><p>For non-hierarchical schemes, the path will be everything following the scheme declaration, as in the following example:</p>
    /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qurl-mailtopath.png" alt=""></p><p>The <i>path</i> data is interpreted according to <i>mode</i>: in <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">StrictMode</a>, any '%' characters must be followed by exactly two hexadecimal characters and some characters (including space) are not allowed in undecoded form. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">TolerantMode</a> (the default), all characters are accepted in undecoded form and the tolerant parser will correct stray '%' not followed by two hex characters. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">DecodedMode</a>, '%' stand for themselves and encoded characters are not possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">QUrl::DecodedMode</a> should be used when setting the path from a data source which is not a URL, such as a dialog shown to the user or with a path obtained by calling <a href="http://doc.qt.io/qt-5/qurl.html#path">path</a>() with the <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> formatting option.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#path">path</a>().</p></div>
    pub unsafe fn set_path_2a(
        &mut self,
        path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        mode: crate::q_url::ParsingMode,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrl_setPath(
            self as *mut crate::QUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(path).as_raw_ptr(),
            mode,
        )
    }

    /// <p>Sets the path of the URL to <i>path</i>. The path is the part of the URL that comes after the authority but before the query string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrl::setPath(const QString& path)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#setPath">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the path of the URL to <i>path</i>. The path is the part of the URL that comes after the authority but before the query string.</p>
    /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qurl-ftppath.png" alt=""></p><p>For non-hierarchical schemes, the path will be everything following the scheme declaration, as in the following example:</p>
    /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qurl-mailtopath.png" alt=""></p><p>The <i>path</i> data is interpreted according to <i>mode</i>: in <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">StrictMode</a>, any '%' characters must be followed by exactly two hexadecimal characters and some characters (including space) are not allowed in undecoded form. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">TolerantMode</a> (the default), all characters are accepted in undecoded form and the tolerant parser will correct stray '%' not followed by two hex characters. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">DecodedMode</a>, '%' stand for themselves and encoded characters are not possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">QUrl::DecodedMode</a> should be used when setting the path from a data source which is not a URL, such as a dialog shown to the user or with a path obtained by calling <a href="http://doc.qt.io/qt-5/qurl.html#path">path</a>() with the <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> formatting option.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#path">path</a>().</p></div>
    pub unsafe fn set_path_1a(
        &mut self,
        path: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrl_setPath1(
            self as *mut crate::QUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(path).as_raw_ptr(),
        )
    }

    /// <p>Sets the port of the URL to <i>port</i>. The port is part of the authority of the URL, as described in <a href="http://doc.qt.io/qt-5/qurl.html#setAuthority">setAuthority</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrl::setPort(int port)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#setPort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the port of the URL to <i>port</i>. The port is part of the authority of the URL, as described in <a href="http://doc.qt.io/qt-5/qurl.html#setAuthority">setAuthority</a>().</p>
    /// <p><i>port</i> must be between 0 and 65535 inclusive. Setting the port to -1 indicates that the port is unspecified.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#port">port</a>().</p></div>
    pub unsafe fn set_port(&mut self, port: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QUrl_setPort(self as *mut crate::QUrl, port)
    }

    /// <p>Sets the query string of the URL to <i>query</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrl::setQuery(const QString& query, QUrl::ParsingMode mode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#setQuery">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the query string of the URL to <i>query</i>.</p>
    /// <p>This function is useful if you need to pass a query string that does not fit into the key-value pattern, or that uses a different scheme for encoding special characters than what is suggested by <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a>.</p>
    /// <p>Passing a value of QString() to <i>query</i> (a null <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>) unsets the query completely. However, passing a value of <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>("") will set the query to an empty value, as if the original URL had a lone "?".</p>
    /// <p>The <i>query</i> data is interpreted according to <i>mode</i>: in <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">StrictMode</a>, any '%' characters must be followed by exactly two hexadecimal characters and some characters (including space) are not allowed in undecoded form. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">TolerantMode</a>, all characters are accepted in undecoded form and the tolerant parser will correct stray '%' not followed by two hex characters. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">DecodedMode</a>, '%' stand for themselves and encoded characters are not possible.</p>
    /// <p>Query strings often contain percent-encoded sequences, so use of <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">DecodedMode</a> is discouraged. One special sequence to be aware of is that of the plus character ('+'). <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> does not convert spaces to plus characters, even though HTML forms posted by web browsers do. In order to represent an actual plus character in a query, the sequence "%2B" is usually used. This function will leave "%2B" sequences untouched in <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">TolerantMode</a> or <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">StrictMode</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#query">query</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#hasQuery">hasQuery</a>().</p></div>
    pub unsafe fn set_query_q_string_parsing_mode(
        &mut self,
        query: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        mode: crate::q_url::ParsingMode,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrl_setQuery(
            self as *mut crate::QUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(query)
                .as_raw_ptr(),
            mode,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrl::setQuery(const QUrlQuery& query)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#setQuery-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Sets the query string of the URL to <i>query</i>.</p>
    /// <p>This function reconstructs the query string from the <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> object and sets on this <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> object. This function does not have parsing parameters because the <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> contains data that is already parsed.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#query">query</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#hasQuery">hasQuery</a>().</p></div>
    pub unsafe fn set_query_q_url_query(
        &mut self,
        query: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrlQuery>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrl_setQuery1(
            self as *mut crate::QUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrlQuery>>::cast_into(query)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the query string of the URL to <i>query</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrl::setQuery(const QString& query)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#setQuery">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the query string of the URL to <i>query</i>.</p>
    /// <p>This function is useful if you need to pass a query string that does not fit into the key-value pattern, or that uses a different scheme for encoding special characters than what is suggested by <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a>.</p>
    /// <p>Passing a value of QString() to <i>query</i> (a null <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>) unsets the query completely. However, passing a value of <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>("") will set the query to an empty value, as if the original URL had a lone "?".</p>
    /// <p>The <i>query</i> data is interpreted according to <i>mode</i>: in <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">StrictMode</a>, any '%' characters must be followed by exactly two hexadecimal characters and some characters (including space) are not allowed in undecoded form. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">TolerantMode</a>, all characters are accepted in undecoded form and the tolerant parser will correct stray '%' not followed by two hex characters. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">DecodedMode</a>, '%' stand for themselves and encoded characters are not possible.</p>
    /// <p>Query strings often contain percent-encoded sequences, so use of <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">DecodedMode</a> is discouraged. One special sequence to be aware of is that of the plus character ('+'). <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> does not convert spaces to plus characters, even though HTML forms posted by web browsers do. In order to represent an actual plus character in a query, the sequence "%2B" is usually used. This function will leave "%2B" sequences untouched in <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">TolerantMode</a> or <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">StrictMode</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#query">query</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#hasQuery">hasQuery</a>().</p></div>
    pub unsafe fn set_query_q_string(
        &mut self,
        query: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrl_setQuery2(
            self as *mut crate::QUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(query)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the scheme of the URL to <i>scheme</i>. As a scheme can only contain ASCII characters, no conversion or decoding is done on the input. It must also start with an ASCII letter.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrl::setScheme(const QString& scheme)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#setScheme">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the scheme of the URL to <i>scheme</i>. As a scheme can only contain ASCII characters, no conversion or decoding is done on the input. It must also start with an ASCII letter.</p>
    /// <p>The scheme describes the type (or protocol) of the URL. It's represented by one or more ASCII characters at the start the URL.</p>
    /// <p>A scheme is strictly <a href="http://www.ietf.org/rfc/rfc3986.txt">RFC 3986</a>-compliant: <code>scheme = ALPHA *( ALPHA / DIGIT / "+" / "-" / "." )</code></p>
    /// <p>The following example shows a URL where the scheme is "ftp":</p>
    /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qurl-authority2.png" alt=""></p><p>To set the scheme, the following call is used:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span> url;
    ///   url<span class="operator">.</span>setScheme(<span class="string">"ftp"</span>);
    ///
    /// </pre>
    /// <p>The scheme can also be empty, in which case the URL is interpreted as relative.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#scheme">scheme</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#isRelative">isRelative</a>().</p></div>
    pub unsafe fn set_scheme(
        &mut self,
        scheme: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrl_setScheme(
            self as *mut crate::QUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(scheme)
                .as_raw_ptr(),
        )
    }

    /// <p>Parses <i>url</i> and sets this object to that value. <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> will automatically percent encode all characters that are not allowed in a URL and decode the percent-encoded sequences that represent an unreserved character (letters, digits, hyphens, undercores, dots and tildes). All other characters are left in their original forms.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrl::setUrl(const QString& url, QUrl::ParsingMode mode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#setUrl">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Parses <i>url</i> and sets this object to that value. <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> will automatically percent encode all characters that are not allowed in a URL and decode the percent-encoded sequences that represent an unreserved character (letters, digits, hyphens, undercores, dots and tildes). All other characters are left in their original forms.</p>
    /// <p>Parses the <i>url</i> using the parser mode <i>parsingMode</i>. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">TolerantMode</a> (the default), <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> will correct certain mistakes, notably the presence of a percent character ('%') not followed by two hexadecimal digits, and it will accept any character in any position. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">StrictMode</a>, encoding mistakes will not be tolerated and <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> will also check that certain forbidden characters are not present in unencoded form. If an error is detected in <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">StrictMode</a>, <a href="http://doc.qt.io/qt-5/qurl.html#isValid">isValid</a>() will return false. The parsing mode <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">DecodedMode</a> is not permitted in this context and will produce a run-time warning.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#url">url</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#toString">toString</a>().</p></div>
    pub unsafe fn set_url_2a(
        &mut self,
        url: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        mode: crate::q_url::ParsingMode,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrl_setUrl(
            self as *mut crate::QUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(url).as_raw_ptr(),
            mode,
        )
    }

    /// <p>Parses <i>url</i> and sets this object to that value. <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> will automatically percent encode all characters that are not allowed in a URL and decode the percent-encoded sequences that represent an unreserved character (letters, digits, hyphens, undercores, dots and tildes). All other characters are left in their original forms.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrl::setUrl(const QString& url)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#setUrl">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Parses <i>url</i> and sets this object to that value. <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> will automatically percent encode all characters that are not allowed in a URL and decode the percent-encoded sequences that represent an unreserved character (letters, digits, hyphens, undercores, dots and tildes). All other characters are left in their original forms.</p>
    /// <p>Parses the <i>url</i> using the parser mode <i>parsingMode</i>. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">TolerantMode</a> (the default), <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> will correct certain mistakes, notably the presence of a percent character ('%') not followed by two hexadecimal digits, and it will accept any character in any position. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">StrictMode</a>, encoding mistakes will not be tolerated and <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> will also check that certain forbidden characters are not present in unencoded form. If an error is detected in <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">StrictMode</a>, <a href="http://doc.qt.io/qt-5/qurl.html#isValid">isValid</a>() will return false. The parsing mode <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">DecodedMode</a> is not permitted in this context and will produce a run-time warning.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#url">url</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#toString">toString</a>().</p></div>
    pub unsafe fn set_url_1a(
        &mut self,
        url: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrl_setUrl1(
            self as *mut crate::QUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(url).as_raw_ptr(),
        )
    }

    /// <p>Sets the user info of the URL to <i>userInfo</i>. The user info is an optional part of the authority of the URL, as described in <a href="http://doc.qt.io/qt-5/qurl.html#setAuthority">setAuthority</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrl::setUserInfo(const QString& userInfo, QUrl::ParsingMode mode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#setUserInfo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the user info of the URL to <i>userInfo</i>. The user info is an optional part of the authority of the URL, as described in <a href="http://doc.qt.io/qt-5/qurl.html#setAuthority">setAuthority</a>().</p>
    /// <p>The user info consists of a user name and optionally a password, separated by a ':'. If the password is empty, the colon must be omitted. The following example shows a valid user info string:</p>
    /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qurl-authority3.png" alt=""></p><p>The <i>userInfo</i> data is interpreted according to <i>mode</i>: in <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">StrictMode</a>, any '%' characters must be followed by exactly two hexadecimal characters and some characters (including space) are not allowed in undecoded form. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">TolerantMode</a> (the default), all characters are accepted in undecoded form and the tolerant parser will correct stray '%' not followed by two hex characters.</p>
    /// <p>This function does not allow <i>mode</i> to be <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">QUrl::DecodedMode</a>. To set fully decoded data, call <a href="http://doc.qt.io/qt-5/qurl.html#setUserName">setUserName</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#setPassword">setPassword</a>() individually.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#userInfo">userInfo</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#setUserName">setUserName</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#setPassword">setPassword</a>(), and <a href="http://doc.qt.io/qt-5/qurl.html#setAuthority">setAuthority</a>().</p></div>
    pub unsafe fn set_user_info_2a(
        &mut self,
        user_info: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        mode: crate::q_url::ParsingMode,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrl_setUserInfo(
            self as *mut crate::QUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(user_info)
                .as_raw_ptr(),
            mode,
        )
    }

    /// <p>Sets the user info of the URL to <i>userInfo</i>. The user info is an optional part of the authority of the URL, as described in <a href="http://doc.qt.io/qt-5/qurl.html#setAuthority">setAuthority</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrl::setUserInfo(const QString& userInfo)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#setUserInfo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the user info of the URL to <i>userInfo</i>. The user info is an optional part of the authority of the URL, as described in <a href="http://doc.qt.io/qt-5/qurl.html#setAuthority">setAuthority</a>().</p>
    /// <p>The user info consists of a user name and optionally a password, separated by a ':'. If the password is empty, the colon must be omitted. The following example shows a valid user info string:</p>
    /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qurl-authority3.png" alt=""></p><p>The <i>userInfo</i> data is interpreted according to <i>mode</i>: in <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">StrictMode</a>, any '%' characters must be followed by exactly two hexadecimal characters and some characters (including space) are not allowed in undecoded form. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">TolerantMode</a> (the default), all characters are accepted in undecoded form and the tolerant parser will correct stray '%' not followed by two hex characters.</p>
    /// <p>This function does not allow <i>mode</i> to be <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">QUrl::DecodedMode</a>. To set fully decoded data, call <a href="http://doc.qt.io/qt-5/qurl.html#setUserName">setUserName</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#setPassword">setPassword</a>() individually.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#userInfo">userInfo</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#setUserName">setUserName</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#setPassword">setPassword</a>(), and <a href="http://doc.qt.io/qt-5/qurl.html#setAuthority">setAuthority</a>().</p></div>
    pub unsafe fn set_user_info_1a(
        &mut self,
        user_info: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrl_setUserInfo1(
            self as *mut crate::QUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(user_info)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the URL's user name to <i>userName</i>. The <i>userName</i> is part of the user info element in the authority of the URL, as described in <a href="http://doc.qt.io/qt-5/qurl.html#setUserInfo">setUserInfo</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrl::setUserName(const QString& userName, QUrl::ParsingMode mode = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#setUserName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the URL's user name to <i>userName</i>. The <i>userName</i> is part of the user info element in the authority of the URL, as described in <a href="http://doc.qt.io/qt-5/qurl.html#setUserInfo">setUserInfo</a>().</p>
    /// <p>The <i>userName</i> data is interpreted according to <i>mode</i>: in <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">StrictMode</a>, any '%' characters must be followed by exactly two hexadecimal characters and some characters (including space) are not allowed in undecoded form. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">TolerantMode</a> (the default), all characters are accepted in undecoded form and the tolerant parser will correct stray '%' not followed by two hex characters. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">DecodedMode</a>, '%' stand for themselves and encoded characters are not possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">QUrl::DecodedMode</a> should be used when setting the user name from a data source which is not a URL, such as a password dialog shown to the user or with a user name obtained by calling <a href="http://doc.qt.io/qt-5/qurl.html#userName">userName</a>() with the <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> formatting option.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#userName">userName</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#setUserInfo">setUserInfo</a>().</p></div>
    pub unsafe fn set_user_name_2a(
        &mut self,
        user_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        mode: crate::q_url::ParsingMode,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrl_setUserName(
            self as *mut crate::QUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(user_name)
                .as_raw_ptr(),
            mode,
        )
    }

    /// <p>Sets the URL's user name to <i>userName</i>. The <i>userName</i> is part of the user info element in the authority of the URL, as described in <a href="http://doc.qt.io/qt-5/qurl.html#setUserInfo">setUserInfo</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrl::setUserName(const QString& userName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#setUserName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the URL's user name to <i>userName</i>. The <i>userName</i> is part of the user info element in the authority of the URL, as described in <a href="http://doc.qt.io/qt-5/qurl.html#setUserInfo">setUserInfo</a>().</p>
    /// <p>The <i>userName</i> data is interpreted according to <i>mode</i>: in <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">StrictMode</a>, any '%' characters must be followed by exactly two hexadecimal characters and some characters (including space) are not allowed in undecoded form. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">TolerantMode</a> (the default), all characters are accepted in undecoded form and the tolerant parser will correct stray '%' not followed by two hex characters. In <a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">DecodedMode</a>, '%' stand for themselves and encoded characters are not possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qurl.html#ParsingMode-enum">QUrl::DecodedMode</a> should be used when setting the user name from a data source which is not a URL, such as a password dialog shown to the user or with a user name obtained by calling <a href="http://doc.qt.io/qt-5/qurl.html#userName">userName</a>() with the <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> formatting option.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#userName">userName</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#setUserInfo">setUserInfo</a>().</p></div>
    pub unsafe fn set_user_name_1a(
        &mut self,
        user_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrl_setUserName1(
            self as *mut crate::QUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(user_name)
                .as_raw_ptr(),
        )
    }

    /// <p>Swaps URL <i>other</i> with this URL. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrl::swap(QUrl& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps URL <i>other</i> with this URL. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QUrl>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrl_swap(
            self as *mut crate::QUrl,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QUrl>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the ASCII Compatible Encoding of the given domain name <i>domain</i>. The result of this function is considered equivalent to <i>domain</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QUrl::toAce(const QString& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#toAce">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the ASCII Compatible Encoding of the given domain name <i>domain</i>. The result of this function is considered equivalent to <i>domain</i>.</p>
    /// <p>The ASCII-Compatible Encoding (ACE) is defined by RFC 3490, RFC 3491 and RFC 3492. It is part of the Internationalizing Domain Names in Applications (IDNA) specification, which allows for domain names (like <code>"example.com"</code>) to be written using international characters.</p>
    /// <p>This function returns an empty <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> if <i>domain</i> is not a valid hostname. Note, in particular, that IPv6 literals are not valid domain names.</p>
    /// <p>This function was introduced in  Qt 4.2.</p></div>
    pub unsafe fn to_ace(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_toAce(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(arg1).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a human-displayable string representation of the URL. The output can be customized by passing flags with <i>options</i>. The option <a href="http://doc.qt.io/qt-5/qurl.html#UrlFormattingOption-enum">RemovePassword</a> is always enabled, since passwords should never be shown back to users.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::toDisplayString(QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#toDisplayString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a human-displayable string representation of the URL. The output can be customized by passing flags with <i>options</i>. The option <a href="http://doc.qt.io/qt-5/qurl.html#UrlFormattingOption-enum">RemovePassword</a> is always enabled, since passwords should never be shown back to users.</p>
    /// <p>With the default options, the resulting <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> can be passed back to a <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> later on, but any password that was present initially will be lost.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#UrlFormattingOption-enum">FormattingOptions</a>, <a href="http://doc.qt.io/qt-5/qurl.html#toEncoded">toEncoded</a>(), and <a href="http://doc.qt.io/qt-5/qurl.html#toString">toString</a>().</p></div>
    pub unsafe fn to_display_string_1a(
        &self,
        options: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>,
        >,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_toDisplayString(
            self as *const crate::QUrl,
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>,
            >::cast_into(options)
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a human-displayable string representation of the URL. The output can be customized by passing flags with <i>options</i>. The option <a href="http://doc.qt.io/qt-5/qurl.html#UrlFormattingOption-enum">RemovePassword</a> is always enabled, since passwords should never be shown back to users.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::toDisplayString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#toDisplayString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a human-displayable string representation of the URL. The output can be customized by passing flags with <i>options</i>. The option <a href="http://doc.qt.io/qt-5/qurl.html#UrlFormattingOption-enum">RemovePassword</a> is always enabled, since passwords should never be shown back to users.</p>
    /// <p>With the default options, the resulting <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> can be passed back to a <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> later on, but any password that was present initially will be lost.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#UrlFormattingOption-enum">FormattingOptions</a>, <a href="http://doc.qt.io/qt-5/qurl.html#toEncoded">toEncoded</a>(), and <a href="http://doc.qt.io/qt-5/qurl.html#toString">toString</a>().</p></div>
    pub unsafe fn to_display_string_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QUrl_toDisplayString1(self as *const crate::QUrl);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the encoded representation of the URL if it's valid; otherwise an empty <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> is returned. The output can be customized by passing flags with <i>options</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QUrl::toEncoded(QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#toEncoded">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the encoded representation of the URL if it's valid; otherwise an empty <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> is returned. The output can be customized by passing flags with <i>options</i>.</p>
    /// <p>The user info, path and fragment are all converted to UTF-8, and all non-ASCII characters are then percent encoded. The host name is encoded using Punycode.</p></div>
    pub unsafe fn to_encoded_1a(
        &self,
        options: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>,
        >,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_toEncoded(
            self as *const crate::QUrl,
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>,
            >::cast_into(options)
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the encoded representation of the URL if it's valid; otherwise an empty <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> is returned. The output can be customized by passing flags with <i>options</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QUrl::toEncoded() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#toEncoded">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the encoded representation of the URL if it's valid; otherwise an empty <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> is returned. The output can be customized by passing flags with <i>options</i>.</p>
    /// <p>The user info, path and fragment are all converted to UTF-8, and all non-ASCII characters are then percent encoded. The host name is encoded using Punycode.</p></div>
    pub unsafe fn to_encoded_0a(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_toEncoded1(self as *const crate::QUrl);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the path of this URL formatted as a local file path. The path returned will use forward slashes, even if it was originally created from one with backslashes.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::toLocalFile() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#toLocalFile">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the path of this URL formatted as a local file path. The path returned will use forward slashes, even if it was originally created from one with backslashes.</p>
    /// <p>If this URL contains a non-empty hostname, it will be encoded in the returned value in the form found on SMB networks (for example, "//servername/path/to/file.txt").</p>
    /// <pre class="cpp">
    ///
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span>(<span class="string">"file:file.txt"</span>)<span class="operator">.</span>toLocalFile();            <span class="comment">// "file:file.txt"</span>
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span>(<span class="string">"file:/home/user/file.txt"</span>)<span class="operator">.</span>toLocalFile(); <span class="comment">// "file:///home/user/file.txt"</span>
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span>(<span class="string">"file.txt"</span>)<span class="operator">.</span>toLocalFile();                 <span class="comment">// ""; wasn't a local file as it had no scheme</span>
    ///
    /// </pre>
    /// <p>Note: if the path component of this URL contains a non-UTF-8 binary sequence (such as %80), the behaviour of this function is undefined.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#fromLocalFile">fromLocalFile</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#isLocalFile">isLocalFile</a>().</p></div>
    pub unsafe fn to_local_file(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_toLocalFile(self as *const crate::QUrl);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns an encoded copy of <i>input</i>. <i>input</i> is first converted to UTF-8, and all ASCII-characters that are not in the unreserved group are percent encoded. To prevent characters from being percent encoded pass them to <i>exclude</i>. To force characters to be percent encoded pass them to <i>include</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QUrl::toPercentEncoding(const QString& arg1, const QByteArray& exclude = …, const QByteArray& include = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#toPercentEncoding">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an encoded copy of <i>input</i>. <i>input</i> is first converted to UTF-8, and all ASCII-characters that are not in the unreserved group are percent encoded. To prevent characters from being percent encoded pass them to <i>exclude</i>. To force characters to be percent encoded pass them to <i>include</i>.</p>
    /// <p>Unreserved is defined as: <code>ALPHA / DIGIT / "-" / "." / "_" / "~"</code></p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span> ba <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span><span class="operator">::</span>toPercentEncoding(<span class="string">"{a fishy string?}"</span><span class="operator">,</span> <span class="string">"{}"</span><span class="operator">,</span> <span class="string">"s"</span>);
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>(ba<span class="operator">.</span>constData());
    ///   <span class="comment">// prints "{a fi%73hy %73tring%3F}"</span>
    ///
    /// </pre></div>
    pub unsafe fn to_percent_encoding_3a(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        exclude: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        include: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_toPercentEncoding(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(arg1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(exclude)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(include)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns an encoded copy of <i>input</i>. <i>input</i> is first converted to UTF-8, and all ASCII-characters that are not in the unreserved group are percent encoded. To prevent characters from being percent encoded pass them to <i>exclude</i>. To force characters to be percent encoded pass them to <i>include</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QUrl::toPercentEncoding(const QString& arg1, const QByteArray& exclude = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#toPercentEncoding">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an encoded copy of <i>input</i>. <i>input</i> is first converted to UTF-8, and all ASCII-characters that are not in the unreserved group are percent encoded. To prevent characters from being percent encoded pass them to <i>exclude</i>. To force characters to be percent encoded pass them to <i>include</i>.</p>
    /// <p>Unreserved is defined as: <code>ALPHA / DIGIT / "-" / "." / "_" / "~"</code></p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span> ba <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span><span class="operator">::</span>toPercentEncoding(<span class="string">"{a fishy string?}"</span><span class="operator">,</span> <span class="string">"{}"</span><span class="operator">,</span> <span class="string">"s"</span>);
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>(ba<span class="operator">.</span>constData());
    ///   <span class="comment">// prints "{a fi%73hy %73tring%3F}"</span>
    ///
    /// </pre></div>
    pub unsafe fn to_percent_encoding_2a(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        exclude: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_toPercentEncoding1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(arg1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(exclude)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns an encoded copy of <i>input</i>. <i>input</i> is first converted to UTF-8, and all ASCII-characters that are not in the unreserved group are percent encoded. To prevent characters from being percent encoded pass them to <i>exclude</i>. To force characters to be percent encoded pass them to <i>include</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QByteArray QUrl::toPercentEncoding(const QString& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#toPercentEncoding">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an encoded copy of <i>input</i>. <i>input</i> is first converted to UTF-8, and all ASCII-characters that are not in the unreserved group are percent encoded. To prevent characters from being percent encoded pass them to <i>exclude</i>. To force characters to be percent encoded pass them to <i>include</i>.</p>
    /// <p>Unreserved is defined as: <code>ALPHA / DIGIT / "-" / "." / "_" / "~"</code></p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span> ba <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qurl.html#QUrl">QUrl</a></span><span class="operator">::</span>toPercentEncoding(<span class="string">"{a fishy string?}"</span><span class="operator">,</span> <span class="string">"{}"</span><span class="operator">,</span> <span class="string">"s"</span>);
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>(ba<span class="operator">.</span>constData());
    ///   <span class="comment">// prints "{a fi%73hy %73tring%3F}"</span>
    ///
    /// </pre></div>
    pub unsafe fn to_percent_encoding_1a(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_toPercentEncoding2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(arg1).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a string representation of the URL. The output can be customized by passing flags with <i>options</i>. The option <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> is not permitted in this function since it would generate ambiguous data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::toString(QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#toString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string representation of the URL. The output can be customized by passing flags with <i>options</i>. The option <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> is not permitted in this function since it would generate ambiguous data.</p>
    /// <p>The default formatting option is <a href="http://doc.qt.io/qt-5/qurl.html#UrlFormattingOption-enum">PrettyDecoded</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#UrlFormattingOption-enum">FormattingOptions</a>, <a href="http://doc.qt.io/qt-5/qurl.html#url">url</a>(), and <a href="http://doc.qt.io/qt-5/qurl.html#setUrl">setUrl</a>().</p></div>
    pub unsafe fn to_string_1a(
        &self,
        options: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>,
        >,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_toString(
            self as *const crate::QUrl,
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>,
            >::cast_into(options)
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a string representation of the URL. The output can be customized by passing flags with <i>options</i>. The option <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> is not permitted in this function since it would generate ambiguous data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::toString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#toString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string representation of the URL. The output can be customized by passing flags with <i>options</i>. The option <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> is not permitted in this function since it would generate ambiguous data.</p>
    /// <p>The default formatting option is <a href="http://doc.qt.io/qt-5/qurl.html#UrlFormattingOption-enum">PrettyDecoded</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#UrlFormattingOption-enum">FormattingOptions</a>, <a href="http://doc.qt.io/qt-5/qurl.html#url">url</a>(), and <a href="http://doc.qt.io/qt-5/qurl.html#setUrl">setUrl</a>().</p></div>
    pub unsafe fn to_string_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_toString1(self as *const crate::QUrl);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts a list of <i>urls</i> into a list of <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> objects, using <a href="http://doc.qt.io/qt-5/qurl.html#toString">toString</a>(<i>options</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QStringList QUrl::toStringList(const QList<QUrl>& uris, QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption> options = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#toStringList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts a list of <i>urls</i> into a list of <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> objects, using <a href="http://doc.qt.io/qt-5/qurl.html#toString">toString</a>(<i>options</i>).</p>
    /// <p>This function was introduced in  Qt 5.1.</p></div>
    pub unsafe fn to_string_list_2a(
        uris: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQUrl>>,
        options: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>,
        >,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_toStringList(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQUrl>>::cast_into(uris)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>,
            >::cast_into(options)
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts a list of <i>urls</i> into a list of <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> objects, using <a href="http://doc.qt.io/qt-5/qurl.html#toString">toString</a>(<i>options</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QStringList QUrl::toStringList(const QList<QUrl>& uris)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#toStringList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts a list of <i>urls</i> into a list of <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> objects, using <a href="http://doc.qt.io/qt-5/qurl.html#toString">toString</a>(<i>options</i>).</p>
    /// <p>This function was introduced in  Qt 5.1.</p></div>
    pub unsafe fn to_string_list_1a(
        uris: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQUrl>>,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_toStringList1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQUrl>>::cast_into(uris)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the TLD (Top-Level Domain) of the URL, (e.g. .co.uk, .net). Note that the return value is prefixed with a '.' unless the URL does not contain a valid TLD, in which case the function returns an empty string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::topLevelDomain(QFlags<QUrl::ComponentFormattingOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#topLevelDomain">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the TLD (Top-Level Domain) of the URL, (e.g. .co.uk, .net). Note that the return value is prefixed with a '.' unless the URL does not contain a valid TLD, in which case the function returns an empty string.</p>
    /// <p>Note that this function considers a TLD to be any domain that allows users to register subdomains under, including many home, dynamic DNS websites and blogging providers. This is useful for determining whether two websites belong to the same infrastructure and communication should be allowed, such as browser cookies: two domains should be considered part of the same website if they share at least one label in addition to the value returned by this function.</p>
    /// <ul>
    /// <li><code>foo.co.uk</code> and <code>foo.com</code> do not share a top-level domain</li>
    /// <li><code>foo.co.uk</code> and <code>bar.co.uk</code> share the <code>.co.uk</code> domain, but the next label is different</li>
    /// <li><code>www.foo.co.uk</code> and <code>ftp.foo.co.uk</code> share the same top-level domain and one more label, so they are considered part of the same site</li>
    /// </ul>
    /// <p>If <i>options</i> includes <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">EncodeUnicode</a>, the returned string will be in ASCII Compatible Encoding.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn top_level_domain_1a(
        &self,
        options: crate::QFlags<crate::q_url::ComponentFormattingOption>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_topLevelDomain(
            self as *const crate::QUrl,
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the TLD (Top-Level Domain) of the URL, (e.g. .co.uk, .net). Note that the return value is prefixed with a '.' unless the URL does not contain a valid TLD, in which case the function returns an empty string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::topLevelDomain() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#topLevelDomain">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the TLD (Top-Level Domain) of the URL, (e.g. .co.uk, .net). Note that the return value is prefixed with a '.' unless the URL does not contain a valid TLD, in which case the function returns an empty string.</p>
    /// <p>Note that this function considers a TLD to be any domain that allows users to register subdomains under, including many home, dynamic DNS websites and blogging providers. This is useful for determining whether two websites belong to the same infrastructure and communication should be allowed, such as browser cookies: two domains should be considered part of the same website if they share at least one label in addition to the value returned by this function.</p>
    /// <ul>
    /// <li><code>foo.co.uk</code> and <code>foo.com</code> do not share a top-level domain</li>
    /// <li><code>foo.co.uk</code> and <code>bar.co.uk</code> share the <code>.co.uk</code> domain, but the next label is different</li>
    /// <li><code>www.foo.co.uk</code> and <code>ftp.foo.co.uk</code> share the same top-level domain and one more label, so they are considered part of the same site</li>
    /// </ul>
    /// <p>If <i>options</i> includes <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">EncodeUnicode</a>, the returned string will be in ASCII Compatible Encoding.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn top_level_domain_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QUrl_topLevelDomain1(self as *const crate::QUrl);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a string representation of the URL. The output can be customized by passing flags with <i>options</i>. The option <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> is not permitted in this function since it would generate ambiguous data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::url(QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#url">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string representation of the URL. The output can be customized by passing flags with <i>options</i>. The option <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> is not permitted in this function since it would generate ambiguous data.</p>
    /// <p>The resulting <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> can be passed back to a <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> later on.</p>
    /// <p>Synonym for <a href="http://doc.qt.io/qt-5/qurl.html#toString">toString</a>(options).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#setUrl">setUrl</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#UrlFormattingOption-enum">FormattingOptions</a>, <a href="http://doc.qt.io/qt-5/qurl.html#toEncoded">toEncoded</a>(), and <a href="http://doc.qt.io/qt-5/qurl.html#toString">toString</a>().</p></div>
    pub unsafe fn url_1a(
        &self,
        options: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>,
        >,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_url(
            self as *const crate::QUrl,
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>,
            >::cast_into(options)
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a string representation of the URL. The output can be customized by passing flags with <i>options</i>. The option <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> is not permitted in this function since it would generate ambiguous data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::url() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#url">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string representation of the URL. The output can be customized by passing flags with <i>options</i>. The option <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> is not permitted in this function since it would generate ambiguous data.</p>
    /// <p>The resulting <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> can be passed back to a <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> later on.</p>
    /// <p>Synonym for <a href="http://doc.qt.io/qt-5/qurl.html#toString">toString</a>(options).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#setUrl">setUrl</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#UrlFormattingOption-enum">FormattingOptions</a>, <a href="http://doc.qt.io/qt-5/qurl.html#toEncoded">toEncoded</a>(), and <a href="http://doc.qt.io/qt-5/qurl.html#toString">toString</a>().</p></div>
    pub unsafe fn url_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_url1(self as *const crate::QUrl);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the user info of the URL, or an empty string if the user info is undefined.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::userInfo(QFlags<QUrl::ComponentFormattingOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#userInfo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the user info of the URL, or an empty string if the user info is undefined.</p>
    /// <p>This function returns an unambiguous value, which may contain that characters still percent-encoded, plus some control sequences not representable in decoded form in <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p>The <i>options</i> argument controls how to format the user info component. The value of <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> is not permitted in this function. If you need to obtain fully decoded data, call <a href="http://doc.qt.io/qt-5/qurl.html#userName">userName</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#password">password</a>() individually.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#setUserInfo">setUserInfo</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#userName">userName</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#password">password</a>(), and <a href="http://doc.qt.io/qt-5/qurl.html#authority">authority</a>().</p></div>
    pub unsafe fn user_info_1a(
        &self,
        options: crate::QFlags<crate::q_url::ComponentFormattingOption>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_userInfo(
            self as *const crate::QUrl,
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the user info of the URL, or an empty string if the user info is undefined.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::userInfo() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#userInfo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the user info of the URL, or an empty string if the user info is undefined.</p>
    /// <p>This function returns an unambiguous value, which may contain that characters still percent-encoded, plus some control sequences not representable in decoded form in <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p>The <i>options</i> argument controls how to format the user info component. The value of <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> is not permitted in this function. If you need to obtain fully decoded data, call <a href="http://doc.qt.io/qt-5/qurl.html#userName">userName</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#password">password</a>() individually.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#setUserInfo">setUserInfo</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#userName">userName</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#password">password</a>(), and <a href="http://doc.qt.io/qt-5/qurl.html#authority">authority</a>().</p></div>
    pub unsafe fn user_info_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_userInfo1(self as *const crate::QUrl);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the user name of the URL if it is defined; otherwise an empty string is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::userName(QFlags<QUrl::ComponentFormattingOption> options = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#userName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the user name of the URL if it is defined; otherwise an empty string is returned.</p>
    /// <p>The <i>options</i> argument controls how to format the user name component. All values produce an unambiguous result. With <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a>, all percent-encoded sequences are decoded; otherwise, the returned value may contain some percent-encoded sequences for some control sequences not representable in decoded form in <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> may cause data loss if those non-representable sequences are present. It is recommended to use that value when the result will be used in a non-URL context, such as setting in <a href="http://doc.qt.io/qt-5/qauthenticator.html">QAuthenticator</a> or negotiating a login.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#setUserName">setUserName</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#userInfo">userInfo</a>().</p></div>
    pub unsafe fn user_name_1a(
        &self,
        options: crate::QFlags<crate::q_url::ComponentFormattingOption>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_userName(
            self as *const crate::QUrl,
            options.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the user name of the URL if it is defined; otherwise an empty string is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrl::userName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#userName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the user name of the URL if it is defined; otherwise an empty string is returned.</p>
    /// <p>The <i>options</i> argument controls how to format the user name component. All values produce an unambiguous result. With <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a>, all percent-encoded sequences are decoded; otherwise, the returned value may contain some percent-encoded sequences for some control sequences not representable in decoded form in <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qurl.html#ComponentFormattingOption-enum">QUrl::FullyDecoded</a> may cause data loss if those non-representable sequences are present. It is recommended to use that value when the result will be used in a non-URL context, such as setting in <a href="http://doc.qt.io/qt-5/qauthenticator.html">QAuthenticator</a> or negotiating a login.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#setUserName">setUserName</a>() and <a href="http://doc.qt.io/qt-5/qurl.html#userInfo">userInfo</a>().</p></div>
    pub unsafe fn user_name_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrl_userName1(self as *const crate::QUrl);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> class provides a way to manipulate a key-value pairs in a URL's query.</p>
///
/// C++ class: <span style='color: green;'>```QUrlQuery```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qurlquery.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> class provides a way to manipulate a key-value pairs in a URL's query.</p>
/// <p>It is used to parse the query strings found in URLs like the following:</p>
/// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qurl-querystring.png" alt=""></p><p>Query strings like the above are used to transmit options in the URL and are usually decoded into multiple key-value pairs. The one above would contain two entries in its list, with keys "type" and "color". <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> can also be used to create a query string suitable for use in <a href="http://doc.qt.io/qt-5/qurl.html#setQuery">QUrl::setQuery</a>() from the individual components of the query.</p>
/// <p>The most common way of parsing a query string is to initialize it in the constructor by passing it the query string. Otherwise, the <a href="http://doc.qt.io/qt-5/qurlquery.html#setQuery">setQuery</a>() method can be used to set the query to be parsed. That method can also be used to parse a query with non-standard delimiters, after having set them using the <a href="http://doc.qt.io/qt-5/qurlquery.html#setQueryDelimiters">setQueryDelimiters</a>() function.</p>
/// <p>The encoded query string can be obtained again using <a href="http://doc.qt.io/qt-5/qurlquery.html#query">query</a>(). This will take all the internally-stored items and encode the string using the delimiters.</p>
/// <a name="encoding"></a></div>
#[repr(C)]
pub struct QUrlQuery {
    _unused: u8,
}
impl QUrlQuery {
    /// <p>Appends the pair <i>key</i> = <i>value</i> to the end of the query string of the URL. This method does not overwrite existing items that might exist with the same key.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrlQuery::addQueryItem(const QString& key, const QString& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#addQueryItem">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the pair <i>key</i> = <i>value</i> to the end of the query string of the URL. This method does not overwrite existing items that might exist with the same key.</p>
    /// <p><b>Note: </b>This method does not treat spaces (ASCII 0x20) and plus ("+") signs as the same, like HTML forms do. If you need spaces to be represented as plus signs, use actual plus signs.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qurlquery.html#hasQueryItem">hasQueryItem</a>() and <a href="http://doc.qt.io/qt-5/qurlquery.html#queryItemValue">queryItemValue</a>().</p></div>
    pub unsafe fn add_query_item(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrlQuery_addQueryItem(
            self as *mut crate::QUrlQuery,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the a list of query string values whose key is equal to <i>key</i> from the URL, using the options specified in <i>encoding</i> to encode the return value. If the key <i>key</i> is not found, this function returns an empty list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QUrlQuery::allQueryItemValues(const QString& key, QFlags<QUrl::ComponentFormattingOption> encoding = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#allQueryItemValues">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the a list of query string values whose key is equal to <i>key</i> from the URL, using the options specified in <i>encoding</i> to encode the return value. If the key <i>key</i> is not found, this function returns an empty list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurlquery.html#queryItemValue">queryItemValue</a>() and <a href="http://doc.qt.io/qt-5/qurlquery.html#addQueryItem">addQueryItem</a>().</p></div>
    pub unsafe fn all_query_item_values_2a(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        encoding: crate::QFlags<crate::q_url::ComponentFormattingOption>,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrlQuery_allQueryItemValues(
            self as *const crate::QUrlQuery,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
            encoding.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the a list of query string values whose key is equal to <i>key</i> from the URL, using the options specified in <i>encoding</i> to encode the return value. If the key <i>key</i> is not found, this function returns an empty list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QUrlQuery::allQueryItemValues(const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#allQueryItemValues">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the a list of query string values whose key is equal to <i>key</i> from the URL, using the options specified in <i>encoding</i> to encode the return value. If the key <i>key</i> is not found, this function returns an empty list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurlquery.html#queryItemValue">queryItemValue</a>() and <a href="http://doc.qt.io/qt-5/qurlquery.html#addQueryItem">addQueryItem</a>().</p></div>
    pub unsafe fn all_query_item_values_1a(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrlQuery_allQueryItemValues1(
            self as *const crate::QUrlQuery,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Clears this <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> object by removing all of the key-value pairs currently stored. If the query delimiters have been changed, this function will leave them with their changed values.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrlQuery::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Clears this <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> object by removing all of the key-value pairs currently stored. If the query delimiters have been changed, this function will leave them with their changed values.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurlquery.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qurlquery.html#setQueryDelimiters">setQueryDelimiters</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QUrlQuery_clear(self as *mut crate::QUrlQuery)
    }

    /// <p>Copies the contents of the <i>other</i> <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> object, including the query delimiters.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QUrlQuery& QUrlQuery::operator=(const QUrlQuery& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Copies the contents of the <i>other</i> <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> object, including the query delimiters.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrlQuery>>,
    ) -> ::cpp_utils::MutRef<crate::QUrlQuery> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrlQuery_operator_(
            self as *mut crate::QUrlQuery,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrlQuery>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the default character for separating keys-value pairs from each other, an ampersand ("&amp;").</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QChar QUrlQuery::defaultQueryPairDelimiter()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#defaultQueryPairDelimiter">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the default character for separating keys-value pairs from each other, an ampersand ("&amp;").</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurlquery.html#setQueryDelimiters">setQueryDelimiters</a>(), <a href="http://doc.qt.io/qt-5/qurlquery.html#queryPairDelimiter">queryPairDelimiter</a>(), and <a href="http://doc.qt.io/qt-5/qurlquery.html#defaultQueryValueDelimiter">defaultQueryValueDelimiter</a>().</p></div>
    pub unsafe fn default_query_pair_delimiter() -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrlQuery_defaultQueryPairDelimiter();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the default character for separating keys from values in the query, an equal sign ("=").</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QChar QUrlQuery::defaultQueryValueDelimiter()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#defaultQueryValueDelimiter">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the default character for separating keys from values in the query, an equal sign ("=").</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurlquery.html#setQueryDelimiters">setQueryDelimiters</a>(), <a href="http://doc.qt.io/qt-5/qurlquery.html#queryValueDelimiter">queryValueDelimiter</a>(), and <a href="http://doc.qt.io/qt-5/qurlquery.html#defaultQueryPairDelimiter">defaultQueryPairDelimiter</a>().</p></div>
    pub unsafe fn default_query_value_delimiter() -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrlQuery_defaultQueryValueDelimiter();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if there is a query string pair whose key is equal to <i>key</i> from the URL.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QUrlQuery::hasQueryItem(const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#hasQueryItem">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if there is a query string pair whose key is equal to <i>key</i> from the URL.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurlquery.html#addQueryItem">addQueryItem</a>() and <a href="http://doc.qt.io/qt-5/qurlquery.html#queryItemValue">queryItemValue</a>().</p></div>
    pub unsafe fn has_query_item(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QUrlQuery_hasQueryItem(
            self as *const crate::QUrlQuery,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QUrlQuery::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QUrlQuery_isDetached(self as *const crate::QUrlQuery)
    }

    /// <p>Returns <code>true</code> if this <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> object contains no key-value pairs, such as after being default-constructed or after parsing an empty query string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QUrlQuery::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> object contains no key-value pairs, such as after being default-constructed or after parsing an empty query string.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurlquery.html#setQuery">setQuery</a>() and <a href="http://doc.qt.io/qt-5/qurlquery.html#clear">clear</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QUrlQuery_isEmpty(self as *const crate::QUrlQuery)
    }

    /// <p>Constructs an empty <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> object. A query can be set afterwards by calling <a href="http://doc.qt.io/qt-5/qurlquery.html#setQuery">setQuery</a>() or items can be added by using <a href="http://doc.qt.io/qt-5/qurlquery.html#addQueryItem">addQueryItem</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QUrlQuery::QUrlQuery()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#QUrlQuery">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> object. A query can be set afterwards by calling <a href="http://doc.qt.io/qt-5/qurlquery.html#setQuery">setQuery</a>() or items can be added by using <a href="http://doc.qt.io/qt-5/qurlquery.html#addQueryItem">addQueryItem</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurlquery.html#setQuery">setQuery</a>() and <a href="http://doc.qt.io/qt-5/qurlquery.html#addQueryItem">addQueryItem</a>().</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QUrlQuery> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrlQuery_QUrlQuery();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> object and parses the query string found in the <i>url</i> URL, using the default query delimiters. To parse a query string using other delimiters, you should first set them using <a href="http://doc.qt.io/qt-5/qurlquery.html#setQueryDelimiters">setQueryDelimiters</a>() and then set the query with <a href="http://doc.qt.io/qt-5/qurlquery.html#setQuery">setQuery</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QUrlQuery::QUrlQuery(const QUrl& url)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#QUrlQuery-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> object and parses the query string found in the <i>url</i> URL, using the default query delimiters. To parse a query string using other delimiters, you should first set them using <a href="http://doc.qt.io/qt-5/qurlquery.html#setQueryDelimiters">setQueryDelimiters</a>() and then set the query with <a href="http://doc.qt.io/qt-5/qurlquery.html#setQuery">setQuery</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurl.html#query">QUrl::query</a>().</p></div>
    pub unsafe fn from_q_url(
        url: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
    ) -> ::cpp_utils::CppBox<crate::QUrlQuery> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrlQuery_QUrlQuery1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(url).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> object and parses the <i>queryString</i> query string, using the default query delimiters. To parse a query string using other delimiters, you should first set them using <a href="http://doc.qt.io/qt-5/qurlquery.html#setQueryDelimiters">setQueryDelimiters</a>() and then set the query with <a href="http://doc.qt.io/qt-5/qurlquery.html#setQuery">setQuery</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QUrlQuery::QUrlQuery(const QString& queryString)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#QUrlQuery-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> object and parses the <i>queryString</i> query string, using the default query delimiters. To parse a query string using other delimiters, you should first set them using <a href="http://doc.qt.io/qt-5/qurlquery.html#setQueryDelimiters">setQueryDelimiters</a>() and then set the query with <a href="http://doc.qt.io/qt-5/qurlquery.html#setQuery">setQuery</a>().</p></div>
    pub unsafe fn from_q_string(
        query_string: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QUrlQuery> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrlQuery_QUrlQuery2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(query_string)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Copies the contents of the <i>other</i> <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> object, including the query delimiters.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QUrlQuery::QUrlQuery(const QUrlQuery& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#QUrlQuery-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Copies the contents of the <i>other</i> <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> object, including the query delimiters.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrlQuery>>,
    ) -> ::cpp_utils::CppBox<crate::QUrlQuery> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrlQuery_QUrlQuery3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrlQuery>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the reconstructed query string, formed from the key-value pairs currently stored in this <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> object and separated by the query delimiters chosen for this object. The keys and values are encoded using the options given by the <i>encoding</i> parameter.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrlQuery::query(QFlags<QUrl::ComponentFormattingOption> encoding = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#query">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the reconstructed query string, formed from the key-value pairs currently stored in this <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> object and separated by the query delimiters chosen for this object. The keys and values are encoded using the options given by the <i>encoding</i> parameter.</p>
    /// <p>For this function, the only ambiguous delimiter is the hash ("#"), as in URLs it is used to separate the query string from the fragment that may follow.</p>
    /// <p>The order of the key-value pairs in the returned string is exactly the same as in the original query.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurlquery.html#setQuery">setQuery</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#setQuery">QUrl::setQuery</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#fragment">QUrl::fragment</a>(), and <a href="http://doc.qt.io/qt-5/qurlquery.html#encoding">Encoding</a>.</p></div>
    pub unsafe fn query_1a(
        &self,
        encoding: crate::QFlags<crate::q_url::ComponentFormattingOption>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrlQuery_query(
            self as *const crate::QUrlQuery,
            encoding.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the reconstructed query string, formed from the key-value pairs currently stored in this <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> object and separated by the query delimiters chosen for this object. The keys and values are encoded using the options given by the <i>encoding</i> parameter.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrlQuery::query() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#query">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the reconstructed query string, formed from the key-value pairs currently stored in this <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> object and separated by the query delimiters chosen for this object. The keys and values are encoded using the options given by the <i>encoding</i> parameter.</p>
    /// <p>For this function, the only ambiguous delimiter is the hash ("#"), as in URLs it is used to separate the query string from the fragment that may follow.</p>
    /// <p>The order of the key-value pairs in the returned string is exactly the same as in the original query.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurlquery.html#setQuery">setQuery</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#setQuery">QUrl::setQuery</a>(), <a href="http://doc.qt.io/qt-5/qurl.html#fragment">QUrl::fragment</a>(), and <a href="http://doc.qt.io/qt-5/qurlquery.html#encoding">Encoding</a>.</p></div>
    pub unsafe fn query_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QUrlQuery_query1(self as *const crate::QUrlQuery);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the query value associated with key <i>key</i> from the URL, using the options specified in <i>encoding</i> to encode the return value. If the key <i>key</i> is not found, this function returns an empty string. If you need to distinguish between an empty value and a non-existent key, you should check for the key's presence first using <a href="http://doc.qt.io/qt-5/qurlquery.html#hasQueryItem">hasQueryItem</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrlQuery::queryItemValue(const QString& key, QFlags<QUrl::ComponentFormattingOption> encoding = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#queryItemValue">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the query value associated with key <i>key</i> from the URL, using the options specified in <i>encoding</i> to encode the return value. If the key <i>key</i> is not found, this function returns an empty string. If you need to distinguish between an empty value and a non-existent key, you should check for the key's presence first using <a href="http://doc.qt.io/qt-5/qurlquery.html#hasQueryItem">hasQueryItem</a>().</p>
    /// <p>If the key <i>key</i> is multiply defined, this function will return the first one found, in the order they were present in the query string or added using <a href="http://doc.qt.io/qt-5/qurlquery.html#addQueryItem">addQueryItem</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurlquery.html#addQueryItem">addQueryItem</a>(), <a href="http://doc.qt.io/qt-5/qurlquery.html#allQueryItemValues">allQueryItemValues</a>(), and <a href="http://doc.qt.io/qt-5/qurlquery.html#encoding">Encoding</a>.</p></div>
    pub unsafe fn query_item_value_2a(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        encoding: crate::QFlags<crate::q_url::ComponentFormattingOption>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrlQuery_queryItemValue(
            self as *const crate::QUrlQuery,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
            encoding.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the query value associated with key <i>key</i> from the URL, using the options specified in <i>encoding</i> to encode the return value. If the key <i>key</i> is not found, this function returns an empty string. If you need to distinguish between an empty value and a non-existent key, you should check for the key's presence first using <a href="http://doc.qt.io/qt-5/qurlquery.html#hasQueryItem">hasQueryItem</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrlQuery::queryItemValue(const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#queryItemValue">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the query value associated with key <i>key</i> from the URL, using the options specified in <i>encoding</i> to encode the return value. If the key <i>key</i> is not found, this function returns an empty string. If you need to distinguish between an empty value and a non-existent key, you should check for the key's presence first using <a href="http://doc.qt.io/qt-5/qurlquery.html#hasQueryItem">hasQueryItem</a>().</p>
    /// <p>If the key <i>key</i> is multiply defined, this function will return the first one found, in the order they were present in the query string or added using <a href="http://doc.qt.io/qt-5/qurlquery.html#addQueryItem">addQueryItem</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurlquery.html#addQueryItem">addQueryItem</a>(), <a href="http://doc.qt.io/qt-5/qurlquery.html#allQueryItemValues">allQueryItemValues</a>(), and <a href="http://doc.qt.io/qt-5/qurlquery.html#encoding">Encoding</a>.</p></div>
    pub unsafe fn query_item_value_1a(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrlQuery_queryItemValue1(
            self as *const crate::QUrlQuery,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the query string of the URL, as a map of keys and values, using the options specified in <i>encoding</i> to encode the items. The order of the elements is the same as the one found in the query string or set with <a href="http://doc.qt.io/qt-5/qurlquery.html#setQueryItems">setQueryItems</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>> QUrlQuery::queryItems(QFlags<QUrl::ComponentFormattingOption> encoding = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#queryItems">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the query string of the URL, as a map of keys and values, using the options specified in <i>encoding</i> to encode the items. The order of the elements is the same as the one found in the query string or set with <a href="http://doc.qt.io/qt-5/qurlquery.html#setQueryItems">setQueryItems</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurlquery.html#setQueryItems">setQueryItems</a>() and <a href="http://doc.qt.io/qt-5/qurlquery.html#encoding">Encoding</a>.</p></div>
    pub unsafe fn query_items_1a(
        &self,
        encoding: crate::QFlags<crate::q_url::ComponentFormattingOption>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrlQuery_queryItems(
            self as *const crate::QUrlQuery,
            encoding.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the query string of the URL, as a map of keys and values, using the options specified in <i>encoding</i> to encode the items. The order of the elements is the same as the one found in the query string or set with <a href="http://doc.qt.io/qt-5/qurlquery.html#setQueryItems">setQueryItems</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>> QUrlQuery::queryItems() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#queryItems">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the query string of the URL, as a map of keys and values, using the options specified in <i>encoding</i> to encode the items. The order of the elements is the same as the one found in the query string or set with <a href="http://doc.qt.io/qt-5/qurlquery.html#setQueryItems">setQueryItems</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurlquery.html#setQueryItems">setQueryItems</a>() and <a href="http://doc.qt.io/qt-5/qurlquery.html#encoding">Encoding</a>.</p></div>
    pub unsafe fn query_items_0a(
        &self,
    ) -> ::cpp_utils::CppBox<crate::QListOfQPairOfQStringQString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QUrlQuery_queryItems1(self as *const crate::QUrlQuery);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the character used to delimit between keys-value pairs when reconstructing the query string in <a href="http://doc.qt.io/qt-5/qurlquery.html#query">query</a>() or when parsing in <a href="http://doc.qt.io/qt-5/qurlquery.html#setQuery">setQuery</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QUrlQuery::queryPairDelimiter() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#queryPairDelimiter">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the character used to delimit between keys-value pairs when reconstructing the query string in <a href="http://doc.qt.io/qt-5/qurlquery.html#query">query</a>() or when parsing in <a href="http://doc.qt.io/qt-5/qurlquery.html#setQuery">setQuery</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurlquery.html#setQueryDelimiters">setQueryDelimiters</a>() and <a href="http://doc.qt.io/qt-5/qurlquery.html#queryValueDelimiter">queryValueDelimiter</a>().</p></div>
    pub unsafe fn query_pair_delimiter(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrlQuery_queryPairDelimiter(
            self as *const crate::QUrlQuery,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the character used to delimit between keys and values when reconstructing the query string in <a href="http://doc.qt.io/qt-5/qurlquery.html#query">query</a>() or when parsing in <a href="http://doc.qt.io/qt-5/qurlquery.html#setQuery">setQuery</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QUrlQuery::queryValueDelimiter() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#queryValueDelimiter">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the character used to delimit between keys and values when reconstructing the query string in <a href="http://doc.qt.io/qt-5/qurlquery.html#query">query</a>() or when parsing in <a href="http://doc.qt.io/qt-5/qurlquery.html#setQuery">setQuery</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurlquery.html#setQueryDelimiters">setQueryDelimiters</a>() and <a href="http://doc.qt.io/qt-5/qurlquery.html#queryPairDelimiter">queryPairDelimiter</a>().</p></div>
    pub unsafe fn query_value_delimiter(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrlQuery_queryValueDelimiter(
            self as *const crate::QUrlQuery,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all the query string pairs whose key is equal to <i>key</i> from the URL.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrlQuery::removeAllQueryItems(const QString& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#removeAllQueryItems">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all the query string pairs whose key is equal to <i>key</i> from the URL.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurlquery.html#removeQueryItem">removeQueryItem</a>().</p></div>
    pub unsafe fn remove_all_query_items(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrlQuery_removeAllQueryItems(
            self as *mut crate::QUrlQuery,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        )
    }

    /// <p>Removes the query string pair whose key is equal to <i>key</i> from the URL. If there are multiple items with a key equal to <i>key</i>, it removes the first item in the order they were present in the query string or added with <a href="http://doc.qt.io/qt-5/qurlquery.html#addQueryItem">addQueryItem</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrlQuery::removeQueryItem(const QString& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#removeQueryItem">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the query string pair whose key is equal to <i>key</i> from the URL. If there are multiple items with a key equal to <i>key</i>, it removes the first item in the order they were present in the query string or added with <a href="http://doc.qt.io/qt-5/qurlquery.html#addQueryItem">addQueryItem</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurlquery.html#removeAllQueryItems">removeAllQueryItems</a>().</p></div>
    pub unsafe fn remove_query_item(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrlQuery_removeQueryItem(
            self as *mut crate::QUrlQuery,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        )
    }

    /// <p>Parses the query string in <i>queryString</i> and sets the internal items to the values found there. If any delimiters have been specified with <a href="http://doc.qt.io/qt-5/qurlquery.html#setQueryDelimiters">setQueryDelimiters</a>(), this function will use them instead of the default delimiters to parse the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrlQuery::setQuery(const QString& queryString)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#setQuery">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Parses the query string in <i>queryString</i> and sets the internal items to the values found there. If any delimiters have been specified with <a href="http://doc.qt.io/qt-5/qurlquery.html#setQueryDelimiters">setQueryDelimiters</a>(), this function will use them instead of the default delimiters to parse the string.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurlquery.html#query">query</a>().</p></div>
    pub unsafe fn set_query(
        &mut self,
        query_string: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrlQuery_setQuery(
            self as *mut crate::QUrlQuery,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(query_string)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the characters used for delimiting between keys and values, and between key-value pairs in the URL's query string. The default value delimiter is '=' and the default pair delimiter is '&amp;'.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrlQuery::setQueryDelimiters(QChar valueDelimiter, QChar pairDelimiter)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#setQueryDelimiters">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the characters used for delimiting between keys and values, and between key-value pairs in the URL's query string. The default value delimiter is '=' and the default pair delimiter is '&amp;'.</p>
    /// <p class="centerAlign"><img src="http://doc.qt.io/qt-5/images/qurl-querystring.png" alt=""></p><p><i>valueDelimiter</i> will be used for separating keys from values, and <i>pairDelimiter</i> will be used to separate key-value pairs. Any occurrences of these delimiting characters in the encoded representation of the keys and values of the query string are percent encoded when returned in <a href="http://doc.qt.io/qt-5/qurlquery.html#query">query</a>().</p>
    /// <p>If <i>valueDelimiter</i> is set to '(' and <i>pairDelimiter</i> is ')', the above query string would instead be represented like this:</p>
    /// <pre class="cpp">
    ///
    ///   http:<span class="comment">//www.example.com/cgi-bin/drawgraph.cgi?type(pie)color(green)</span>
    ///
    /// </pre>
    /// <p><b>Note: </b>Non-standard delimiters should be chosen from among what RFC 3986 calls "sub-delimiters". They are:</p><pre class="cpp">
    ///
    ///   sub<span class="operator">-</span>delims    <span class="operator">=</span> <span class="string">"!"</span> <span class="operator">/</span> <span class="string">"$"</span> <span class="operator">/</span> <span class="string">"&amp;"</span> <span class="operator">/</span> <span class="string">"'"</span> <span class="operator">/</span> <span class="string">"("</span> <span class="operator">/</span> <span class="string">")"</span>
    /// &#32;   &#32;   &#32;   &#32;   <span class="operator">/</span> <span class="string">"*"</span> <span class="operator">/</span> <span class="string">"+"</span> <span class="operator">/</span> <span class="string">","</span> <span class="operator">/</span> <span class="string">";"</span> <span class="operator">/</span> <span class="string">"="</span>
    ///
    /// </pre>
    /// <p>Use of other characters is not supported and may result in unexpected behaviour. This method does not verify that you passed a valid delimiter.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qurlquery.html#queryValueDelimiter">queryValueDelimiter</a>() and <a href="http://doc.qt.io/qt-5/qurlquery.html#queryPairDelimiter">queryPairDelimiter</a>().</p></div>
    pub unsafe fn set_query_delimiters(
        &mut self,
        value_delimiter: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        pair_delimiter: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrlQuery_setQueryDelimiters(
            self as *mut crate::QUrlQuery,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(value_delimiter)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(pair_delimiter)
                .as_raw_ptr(),
        )
    }

    /// <p>Sets the items in this <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> object to <i>query</i>. The order of the elements in <i>query</i> is preserved.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrlQuery::setQueryItems(const QList<QPair<QString, QString>>& query)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#setQueryItems">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the items in this <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> object to <i>query</i>. The order of the elements in <i>query</i> is preserved.</p>
    /// <p><b>Note: </b>This method does not treat spaces (ASCII 0x20) and plus ("+") signs as the same, like HTML forms do. If you need spaces to be represented as plus signs, use actual plus signs.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qurlquery.html#queryItems">queryItems</a>() and <a href="http://doc.qt.io/qt-5/qurlquery.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn set_query_items(
        &mut self,
        query: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrlQuery_setQueryItems(self as *mut crate::QUrlQuery, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>>::cast_into(query).as_raw_ptr())
    }

    /// <p>Swaps this URL query instance with <i>other</i>. This function is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QUrlQuery::swap(QUrlQuery& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps this URL query instance with <i>other</i>. This function is very fast and never fails.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QUrlQuery>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QUrlQuery_swap(
            self as *mut crate::QUrlQuery,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QUrlQuery>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns this <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> as a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. <i>encoding</i> can be used to specify the URL string encoding of the return value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrlQuery::toString(QFlags<QUrl::ComponentFormattingOption> encoding = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#toString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns this <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> as a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. <i>encoding</i> can be used to specify the URL string encoding of the return value.</p></div>
    pub unsafe fn to_string_1a(
        &self,
        encoding: crate::QFlags<crate::q_url::ComponentFormattingOption>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrlQuery_toString(
            self as *const crate::QUrlQuery,
            encoding.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns this <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> as a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. <i>encoding</i> can be used to specify the URL string encoding of the return value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUrlQuery::toString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#toString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns this <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> as a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. <i>encoding</i> can be used to specify the URL string encoding of the return value.</p></div>
    pub unsafe fn to_string_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QUrlQuery_toString1(self as *const crate::QUrlQuery);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_uuid {
    //! C++ type: <span style='color: green;'>```QUuid```</span>

    /// <p>This enum defines the values used in the <a href="http://doc.qt.io/qt-5/quuid.html#variant-field">variant field</a> of the UUID. The value in the variant field determines the layout of the 128-bit value.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QUuid::Variant```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum defines the values used in the <a href="http://doc.qt.io/qt-5/quuid.html#variant-field">variant field</a> of the UUID. The value in the variant field determines the layout of the 128-bit value.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Variant(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Variant {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Variant(value)
        }
    }

    impl From<Variant> for ::std::os::raw::c_int {
        fn from(value: Variant) -> Self {
            value.0
        }
    }

    impl Variant {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Variant {
        /// Variant is unknown (C++ enum variant: <span style='color: green;'>```VarUnknown = -1```</span>)
        #[allow(non_upper_case_globals)]
        pub const VarUnknown: crate::q_uuid::Variant = crate::q_uuid::Variant(-1);
        /// Reserved for NCS (Network Computing System) backward compatibility (C++ enum variant: <span style='color: green;'>```NCS = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NCS: crate::q_uuid::Variant = crate::q_uuid::Variant(0);
        /// Distributed Computing Environment, the scheme used by <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> (C++ enum variant: <span style='color: green;'>```DCE = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const DCE: crate::q_uuid::Variant = crate::q_uuid::Variant(2);
        /// Reserved for Microsoft backward compatibility (GUID) (C++ enum variant: <span style='color: green;'>```Microsoft = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const Microsoft: crate::q_uuid::Variant = crate::q_uuid::Variant(6);
        /// Reserved for future definition (C++ enum variant: <span style='color: green;'>```Reserved = 7```</span>)
        #[allow(non_upper_case_globals)]
        pub const Reserved: crate::q_uuid::Variant = crate::q_uuid::Variant(7);
    }

    /// <p>This enum defines the values used in the <a href="http://doc.qt.io/qt-5/quuid.html#version-field">version field</a> of the UUID. The version field is meaningful only if the value in the <a href="http://doc.qt.io/qt-5/quuid.html#variant-field">variant field</a> is <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::DCE</a>.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QUuid::Version```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html#Version-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum defines the values used in the <a href="http://doc.qt.io/qt-5/quuid.html#version-field">version field</a> of the UUID. The version field is meaningful only if the value in the <a href="http://doc.qt.io/qt-5/quuid.html#variant-field">variant field</a> is <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::DCE</a>.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Version(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Version {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Version(value)
        }
    }

    impl From<Version> for ::std::os::raw::c_int {
        fn from(value: Version) -> Self {
            value.0
        }
    }

    impl Version {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Version {
        /// Version is unknown (C++ enum variant: <span style='color: green;'>```VerUnknown = -1```</span>)
        #[allow(non_upper_case_globals)]
        pub const VerUnknown: crate::q_uuid::Version = crate::q_uuid::Version(-1);
        /// Time-based, by using timestamp, clock sequence, and MAC network card address (if available) for the node sections (C++ enum variant: <span style='color: green;'>```Time = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Time: crate::q_uuid::Version = crate::q_uuid::Version(1);
        /// DCE Security version, with embedded POSIX UUIDs (C++ enum variant: <span style='color: green;'>```EmbeddedPOSIX = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const EmbeddedPOSIX: crate::q_uuid::Version = crate::q_uuid::Version(2);
        /// Alias for Name (C++ enum variant: <span style='color: green;'>```Md5 = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const Md5: crate::q_uuid::Version = crate::q_uuid::Version(3);
        /// Name-based, by using values from a name for all sections (C++ enum variant: <span style='color: green;'>```Name = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const Name: crate::q_uuid::Version = crate::q_uuid::Version(3);
        /// Random-based, by using random numbers for all sections (C++ enum variant: <span style='color: green;'>```Random = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const Random: crate::q_uuid::Version = crate::q_uuid::Version(4);
        /// &nbsp; (C++ enum variant: <span style='color: green;'>```Sha1 = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const Sha1: crate::q_uuid::Version = crate::q_uuid::Version(5);
    }

    /// <p>This enum is used by <a href="http://doc.qt.io/qt-5/quuid.html#toString">toString</a>(StringFormat) to control the formatting of the string representation. The possible values are:</p>
    ///
    /// C++ enum: <span style='color: green;'>```QUuid::StringFormat```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html#StringFormat-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum is used by <a href="http://doc.qt.io/qt-5/quuid.html#toString">toString</a>(StringFormat) to control the formatting of the string representation. The possible values are:</p>
    ///
    /// <p>This enum was introduced or modified in  Qt 5.11.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct StringFormat(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for StringFormat {
        fn from(value: ::std::os::raw::c_int) -> Self {
            StringFormat(value)
        }
    }

    impl From<StringFormat> for ::std::os::raw::c_int {
        fn from(value: StringFormat) -> Self {
            value.0
        }
    }

    impl StringFormat {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl StringFormat {
        /// The default, <a href="http://doc.qt.io/qt-5/quuid.html#toString">toString</a>() will return five hex fields, separated by dashes and surrounded by braces. Example: {00000000-0000-0000-0000-000000000000}. (C++ enum variant: <span style='color: green;'>```WithBraces = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const WithBraces: crate::q_uuid::StringFormat = crate::q_uuid::StringFormat(0);
        /// Only the five dash-separated fields, without the braces. Example: 00000000-0000-0000-0000-000000000000. (C++ enum variant: <span style='color: green;'>```WithoutBraces = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const WithoutBraces: crate::q_uuid::StringFormat = crate::q_uuid::StringFormat(1);
        /// Only the hex digits, without braces or dashes. Note that <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> cannot parse this back again as input. (C++ enum variant: <span style='color: green;'>```Id128 = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const Id128: crate::q_uuid::StringFormat = crate::q_uuid::StringFormat(3);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> class stores a Universally Unique Identifier (UUID).</p>
///
/// C++ class: <span style='color: green;'>```QUuid```</span>.
///
/// <a href="http://doc.qt.io/qt-5/quuid.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> class stores a Universally Unique Identifier (UUID).</p>
/// <p>Using <i>U</i>niversally <i>U</i>nique <i>ID</i>entifiers (UUID) is a standard way to uniquely identify entities in a distributed computing environment. A UUID is a 16-byte (128-bit) number generated by some algorithm that is meant to guarantee that the UUID will be unique in the distributed computing environment where it is used. The acronym GUID is often used instead, <i>G</i>lobally <i>U</i>nique <i>ID</i>entifiers, but it refers to the same thing.</p>
/// <a name="variant-field"></a><p>Actually, the GUID is one <i>variant</i> of UUID. Multiple variants are in use. Each UUID contains a bit field that specifies which type (variant) of UUID it is. Call <a href="http://doc.qt.io/qt-5/quuid.html#variant">variant</a>() to discover which type of UUID an instance of <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> contains. It extracts the three most significant bits of byte 8 of the 16 bytes. In <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>, byte 8 is <code>QUuid::data4[0]</code>. If you create instances of <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> using the constructor that accepts all the numeric values as parameters, use the following table to set the three most significant bits of parameter <code>b1</code>, which becomes <code>QUuid::data4[0]</code> and contains the variant field in its three most significant bits. In the table, 'x' means <i>don't care</i>.</p>
/// <div class="table"><table class="generic">
///  <thead><tr class="qt-style"><th>msb0</th><th>msb1</th><th>msb2</th><th>Variant</th></tr></thead>
/// <tbody><tr valign="top" class="odd"><td>0</td><td>x</td><td>x</td><td>NCS (Network Computing System)</td></tr>
/// <tr valign="top" class="even"><td>1</td><td>0</td><td>x</td><td>DCE (Distributed Computing Environment)</td></tr>
/// <tr valign="top" class="odd"><td>1</td><td>1</td><td>0</td><td>Microsoft (GUID)</td></tr>
/// <tr valign="top" class="even"><td>1</td><td>1</td><td>1</td><td>Reserved for future expansion</td></tr>
/// </tbody></table></div>
/// <a name="version-field"></a><p>If <a href="http://doc.qt.io/qt-5/quuid.html#variant">variant</a>() returns <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::DCE</a>, the UUID also contains a <i>version</i> field in the four most significant bits of <code>QUuid::data3</code>, and you can call <a href="http://doc.qt.io/qt-5/quuid.html#version">version</a>() to discover which version your <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> contains. If you create instances of <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> using the constructor that accepts all the numeric values as parameters, use the following table to set the four most significant bits of parameter <code>w2</code>, which becomes <code>QUuid::data3</code> and contains the version field in its four most significant bits.</p>
/// <div class="table"><table class="generic">
///  <thead><tr class="qt-style"><th>msb0</th><th>msb1</th><th>msb2</th><th>msb3</th><th>Version</th></tr></thead>
/// <tbody><tr valign="top" class="odd"><td>0</td><td>0</td><td>0</td><td>1</td><td>Time</td></tr>
/// <tr valign="top" class="even"><td>0</td><td>0</td><td>1</td><td>0</td><td>Embedded POSIX</td></tr>
/// <tr valign="top" class="odd"><td>0</td><td>0</td><td>1</td><td>1</td><td>Md5(Name)</td></tr>
/// <tr valign="top" class="even"><td>0</td><td>1</td><td>0</td><td>0</td><td>Random</td></tr>
/// <tr valign="top" class="odd"><td>0</td><td>1</td><td>0</td><td>1</td><td>Sha1</td></tr>
/// </tbody></table></div>
/// <p>The field layouts for the DCE versions listed in the table above are specified in the <a href="http://www.ietf.org/rfc/rfc4122.txt">Network Working Group UUID Specification</a>.</p>
/// <p>Most platforms provide a tool for generating new UUIDs, e.g. <code>uuidgen</code> and <code>guidgen</code>. You can also use <a href="http://doc.qt.io/qt-5/quuid.html#createUuid">createUuid</a>(). UUIDs generated by <a href="http://doc.qt.io/qt-5/quuid.html#createUuid">createUuid</a>() are of the random type. Their <a href="http://doc.qt.io/qt-5/quuid.html#Version-enum">QUuid::Version</a> bits are set to <a href="http://doc.qt.io/qt-5/quuid.html#Version-enum">QUuid::Random</a>, and their <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::Variant</a> bits are set to <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::DCE</a>. The rest of the UUID is composed of random numbers. Theoretically, this means there is a small chance that a UUID generated by <a href="http://doc.qt.io/qt-5/quuid.html#createUuid">createUuid</a>() will not be unique. But it is <a href="http://en.wikipedia.org/wiki/Universally_Unique_Identifier#Random_UUID_probability_of_duplicates">a <i>very</i> small chance</a>.</p>
/// <p>UUIDs can be constructed from numeric values or from strings, or using the static <a href="http://doc.qt.io/qt-5/quuid.html#createUuid">createUuid</a>() function. They can be converted to a string with <a href="http://doc.qt.io/qt-5/quuid.html#toString">toString</a>(). UUIDs have a <a href="http://doc.qt.io/qt-5/quuid.html#variant">variant</a>() and a <a href="http://doc.qt.io/qt-5/quuid.html#version">version</a>(), and null UUIDs return true from <a href="http://doc.qt.io/qt-5/quuid.html#isNull">isNull</a>().</p></div>
#[repr(C)]
pub struct QUuid {
    _unused: u8,
}
impl QUuid {
    /// <p>The <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> class stores a Universally Unique Identifier (UUID).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QUuid& QUuid::operator=(const QUuid& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> class stores a Universally Unique Identifier (UUID).</p>
    /// <p>Using <i>U</i>niversally <i>U</i>nique <i>ID</i>entifiers (UUID) is a standard way to uniquely identify entities in a distributed computing environment. A UUID is a 16-byte (128-bit) number generated by some algorithm that is meant to guarantee that the UUID will be unique in the distributed computing environment where it is used. The acronym GUID is often used instead, <i>G</i>lobally <i>U</i>nique <i>ID</i>entifiers, but it refers to the same thing.</p>
    /// <a name="variant-field"></a><p>Actually, the GUID is one <i>variant</i> of UUID. Multiple variants are in use. Each UUID contains a bit field that specifies which type (variant) of UUID it is. Call <a href="http://doc.qt.io/qt-5/quuid.html#variant">variant</a>() to discover which type of UUID an instance of <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> contains. It extracts the three most significant bits of byte 8 of the 16 bytes. In <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>, byte 8 is <code>QUuid::data4[0]</code>. If you create instances of <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> using the constructor that accepts all the numeric values as parameters, use the following table to set the three most significant bits of parameter <code>b1</code>, which becomes <code>QUuid::data4[0]</code> and contains the variant field in its three most significant bits. In the table, 'x' means <i>don't care</i>.</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>msb0</th><th>msb1</th><th>msb2</th><th>Variant</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>0</td><td>x</td><td>x</td><td>NCS (Network Computing System)</td></tr>
    /// <tr valign="top" class="even"><td>1</td><td>0</td><td>x</td><td>DCE (Distributed Computing Environment)</td></tr>
    /// <tr valign="top" class="odd"><td>1</td><td>1</td><td>0</td><td>Microsoft (GUID)</td></tr>
    /// <tr valign="top" class="even"><td>1</td><td>1</td><td>1</td><td>Reserved for future expansion</td></tr>
    /// </tbody></table></div>
    /// <a name="version-field"></a><p>If <a href="http://doc.qt.io/qt-5/quuid.html#variant">variant</a>() returns <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::DCE</a>, the UUID also contains a <i>version</i> field in the four most significant bits of <code>QUuid::data3</code>, and you can call <a href="http://doc.qt.io/qt-5/quuid.html#version">version</a>() to discover which version your <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> contains. If you create instances of <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> using the constructor that accepts all the numeric values as parameters, use the following table to set the four most significant bits of parameter <code>w2</code>, which becomes <code>QUuid::data3</code> and contains the version field in its four most significant bits.</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>msb0</th><th>msb1</th><th>msb2</th><th>msb3</th><th>Version</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>0</td><td>0</td><td>0</td><td>1</td><td>Time</td></tr>
    /// <tr valign="top" class="even"><td>0</td><td>0</td><td>1</td><td>0</td><td>Embedded POSIX</td></tr>
    /// <tr valign="top" class="odd"><td>0</td><td>0</td><td>1</td><td>1</td><td>Md5(Name)</td></tr>
    /// <tr valign="top" class="even"><td>0</td><td>1</td><td>0</td><td>0</td><td>Random</td></tr>
    /// <tr valign="top" class="odd"><td>0</td><td>1</td><td>0</td><td>1</td><td>Sha1</td></tr>
    /// </tbody></table></div>
    /// <p>The field layouts for the DCE versions listed in the table above are specified in the <a href="http://www.ietf.org/rfc/rfc4122.txt">Network Working Group UUID Specification</a>.</p>
    /// <p>Most platforms provide a tool for generating new UUIDs, e.g. <code>uuidgen</code> and <code>guidgen</code>. You can also use <a href="http://doc.qt.io/qt-5/quuid.html#createUuid">createUuid</a>(). UUIDs generated by <a href="http://doc.qt.io/qt-5/quuid.html#createUuid">createUuid</a>() are of the random type. Their <a href="http://doc.qt.io/qt-5/quuid.html#Version-enum">QUuid::Version</a> bits are set to <a href="http://doc.qt.io/qt-5/quuid.html#Version-enum">QUuid::Random</a>, and their <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::Variant</a> bits are set to <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::DCE</a>. The rest of the UUID is composed of random numbers. Theoretically, this means there is a small chance that a UUID generated by <a href="http://doc.qt.io/qt-5/quuid.html#createUuid">createUuid</a>() will not be unique. But it is <a href="http://en.wikipedia.org/wiki/Universally_Unique_Identifier#Random_UUID_probability_of_duplicates">a <i>very</i> small chance</a>.</p>
    /// <p>UUIDs can be constructed from numeric values or from strings, or using the static <a href="http://doc.qt.io/qt-5/quuid.html#createUuid">createUuid</a>() function. They can be converted to a string with <a href="http://doc.qt.io/qt-5/quuid.html#toString">toString</a>(). UUIDs have a <a href="http://doc.qt.io/qt-5/quuid.html#variant">variant</a>() and a <a href="http://doc.qt.io/qt-5/quuid.html#version">version</a>(), and null UUIDs return true from <a href="http://doc.qt.io/qt-5/quuid.html#isNull">isNull</a>().</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUuid>>,
    ) -> ::cpp_utils::MutRef<crate::QUuid> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUuid_operator_2(
            self as *mut crate::QUuid,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUuid>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>On any platform other than Windows, this function returns a new UUID with variant <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::DCE</a> and version <a href="http://doc.qt.io/qt-5/quuid.html#Version-enum">QUuid::Random</a>. If the /dev/urandom device exists, then the numbers used to construct the UUID will be of cryptographic quality, which will make the UUID unique. Otherwise, the numbers of the UUID will be obtained from the local pseudo-random number generator (<a href="http://doc.qt.io/qt-5/qtglobal.html#qrand">qrand</a>(), which is seeded by <a href="http://doc.qt.io/qt-5/qtglobal.html#qsrand">qsrand</a>()) which is usually not of cryptograhic quality, which means that the UUID can't be guaranteed to be unique.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QUuid QUuid::createUuid()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html#createUuid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>On any platform other than Windows, this function returns a new UUID with variant <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::DCE</a> and version <a href="http://doc.qt.io/qt-5/quuid.html#Version-enum">QUuid::Random</a>. If the /dev/urandom device exists, then the numbers used to construct the UUID will be of cryptographic quality, which will make the UUID unique. Otherwise, the numbers of the UUID will be obtained from the local pseudo-random number generator (<a href="http://doc.qt.io/qt-5/qtglobal.html#qrand">qrand</a>(), which is seeded by <a href="http://doc.qt.io/qt-5/qtglobal.html#qsrand">qsrand</a>()) which is usually not of cryptograhic quality, which means that the UUID can't be guaranteed to be unique.</p>
    /// <p>On a Windows platform, a GUID is generated, which almost certainly <i>will</i> be unique, on this or any other system, networked or not.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/quuid.html#variant">variant</a>() and <a href="http://doc.qt.io/qt-5/quuid.html#version">version</a>().</p></div>
    pub unsafe fn create_uuid() -> ::cpp_utils::CppBox<crate::QUuid> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUuid_createUuid();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function returns a new UUID with variant <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::DCE</a> and version <a href="http://doc.qt.io/qt-5/quuid.html#Version-enum">QUuid::Md5</a>. <i>ns</i> is the namespace and <i>baseData</i> is the basic data as described by RFC 4122.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QUuid QUuid::createUuidV3(const QUuid& ns, const QByteArray& baseData)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html#createUuidV3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function returns a new UUID with variant <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::DCE</a> and version <a href="http://doc.qt.io/qt-5/quuid.html#Version-enum">QUuid::Md5</a>. <i>ns</i> is the namespace and <i>baseData</i> is the basic data as described by RFC 4122.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/quuid.html#variant">variant</a>(), <a href="http://doc.qt.io/qt-5/quuid.html#version">version</a>(), and <a href="http://doc.qt.io/qt-5/quuid.html#createUuidV5">createUuidV5</a>().</p></div>
    pub unsafe fn create_uuid_v3_q_uuid_q_byte_array(
        ns: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUuid>>,
        base_data: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QUuid> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUuid_createUuidV3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUuid>>::cast_into(ns).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(base_data)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function returns a new UUID with variant <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::DCE</a> and version <a href="http://doc.qt.io/qt-5/quuid.html#Version-enum">QUuid::Md5</a>. <i>ns</i> is the namespace and <i>baseData</i> is the basic data as described by RFC 4122.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QUuid QUuid::createUuidV3(const QUuid& ns, const QString& baseData)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html#createUuidV3-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function returns a new UUID with variant <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::DCE</a> and version <a href="http://doc.qt.io/qt-5/quuid.html#Version-enum">QUuid::Md5</a>. <i>ns</i> is the namespace and <i>baseData</i> is the basic data as described by RFC 4122.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/quuid.html#variant">variant</a>(), <a href="http://doc.qt.io/qt-5/quuid.html#version">version</a>(), and <a href="http://doc.qt.io/qt-5/quuid.html#createUuidV5">createUuidV5</a>().</p></div>
    pub unsafe fn create_uuid_v3_q_uuid_q_string(
        ns: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUuid>>,
        base_data: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QUuid> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUuid_createUuidV31(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUuid>>::cast_into(ns).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(base_data)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function returns a new UUID with variant <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::DCE</a> and version <a href="http://doc.qt.io/qt-5/quuid.html#Version-enum">QUuid::Sha1</a>. <i>ns</i> is the namespace and <i>baseData</i> is the basic data as described by RFC 4122.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QUuid QUuid::createUuidV5(const QUuid& ns, const QByteArray& baseData)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html#createUuidV5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function returns a new UUID with variant <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::DCE</a> and version <a href="http://doc.qt.io/qt-5/quuid.html#Version-enum">QUuid::Sha1</a>. <i>ns</i> is the namespace and <i>baseData</i> is the basic data as described by RFC 4122.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/quuid.html#variant">variant</a>(), <a href="http://doc.qt.io/qt-5/quuid.html#version">version</a>(), and <a href="http://doc.qt.io/qt-5/quuid.html#createUuidV3">createUuidV3</a>().</p></div>
    pub unsafe fn create_uuid_v5_q_uuid_q_byte_array(
        ns: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUuid>>,
        base_data: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QUuid> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUuid_createUuidV5(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUuid>>::cast_into(ns).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(base_data)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function returns a new UUID with variant <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::DCE</a> and version <a href="http://doc.qt.io/qt-5/quuid.html#Version-enum">QUuid::Sha1</a>. <i>ns</i> is the namespace and <i>baseData</i> is the basic data as described by RFC 4122.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QUuid QUuid::createUuidV5(const QUuid& ns, const QString& baseData)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html#createUuidV5-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function returns a new UUID with variant <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::DCE</a> and version <a href="http://doc.qt.io/qt-5/quuid.html#Version-enum">QUuid::Sha1</a>. <i>ns</i> is the namespace and <i>baseData</i> is the basic data as described by RFC 4122.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/quuid.html#variant">variant</a>(), <a href="http://doc.qt.io/qt-5/quuid.html#version">version</a>(), and <a href="http://doc.qt.io/qt-5/quuid.html#createUuidV3">createUuidV3</a>().</p></div>
    pub unsafe fn create_uuid_v5_q_uuid_q_string(
        ns: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUuid>>,
        base_data: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QUuid> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUuid_createUuidV51(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUuid>>::cast_into(ns).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(base_data)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Returns the value of the <span style='color: green;'>```data1```</span> field.
    pub unsafe fn data1(&self) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QUuid_data1(self as *const crate::QUuid)
    }

    /// Returns the value of the <span style='color: green;'>```data2```</span> field.
    pub unsafe fn data2(&self) -> ::std::os::raw::c_ushort {
        crate::__ffi::ctr_qt_core_ffi_QUuid_data2(self as *const crate::QUuid)
    }

    /// Returns the value of the <span style='color: green;'>```data3```</span> field.
    pub unsafe fn data3(&self) -> ::std::os::raw::c_ushort {
        crate::__ffi::ctr_qt_core_ffi_QUuid_data3(self as *const crate::QUuid)
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> object from the binary representation of the UUID, as specified by RFC 4122 section 4.1.2. See <a href="http://doc.qt.io/qt-5/quuid.html#toRfc4122">toRfc4122</a>() for a further explanation of the order of <i>bytes</i> required.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QUuid QUuid::fromRfc4122(const QByteArray& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html#fromRfc4122">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> object from the binary representation of the UUID, as specified by RFC 4122 section 4.1.2. See <a href="http://doc.qt.io/qt-5/quuid.html#toRfc4122">toRfc4122</a>() for a further explanation of the order of <i>bytes</i> required.</p>
    /// <p>The byte array accepted is NOT a human readable format.</p>
    /// <p>If the conversion fails, a null UUID is created.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/quuid.html#toRfc4122">toRfc4122</a>() and <a href="http://doc.qt.io/qt-5/quuid.html#QUuid">QUuid</a>().</p></div>
    pub unsafe fn from_rfc4122(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QUuid> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUuid_fromRfc4122(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> object from the string <i>text</i>, which must be formatted as five hex fields separated by '-', e.g., "{xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx}" where each 'x' is a hex digit. The curly braces shown here are optional, but it is normal to include them. If the conversion fails, a null UUID is returned. See <a href="http://doc.qt.io/qt-5/quuid.html#toString">toString</a>() for an explanation of how the five hex fields map to the public data members in <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QUuid QUuid::fromString(QStringView string)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html#fromString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> object from the string <i>text</i>, which must be formatted as five hex fields separated by '-', e.g., "{xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx}" where each 'x' is a hex digit. The curly braces shown here are optional, but it is normal to include them. If the conversion fails, a null UUID is returned. See <a href="http://doc.qt.io/qt-5/quuid.html#toString">toString</a>() for an explanation of how the five hex fields map to the public data members in <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/quuid.html#toString">toString</a>() and <a href="http://doc.qt.io/qt-5/quuid.html#QUuid">QUuid</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn from_string_q_string_view(
        string: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::cpp_utils::CppBox<crate::QUuid> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUuid_fromString(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(string)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QUuid QUuid::fromString(QLatin1String string)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html#fromString-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Creates a <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> object from the string <i>text</i>, which must be formatted as five hex fields separated by '-', e.g., "{xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx}" where each 'x' is a hex digit. The curly braces shown here are optional, but it is normal to include them. If the conversion fails, a null UUID is returned. See <a href="http://doc.qt.io/qt-5/quuid.html#toString">toString</a>() for an explanation of how the five hex fields map to the public data members in <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/quuid.html#toString">toString</a>() and <a href="http://doc.qt.io/qt-5/quuid.html#QUuid">QUuid</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn from_string_q_latin1_string(
        string: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::QUuid> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUuid_fromString1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(string)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if this is the null UUID {00000000-0000-0000-0000-000000000000}; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QUuid::isNull() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html#isNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this is the null UUID {00000000-0000-0000-0000-000000000000}; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QUuid_isNull(self as *const crate::QUuid)
    }

    /// <p>Creates the null UUID. <a href="http://doc.qt.io/qt-5/quuid.html#toString">toString</a>() will output the null UUID as "{00000000-0000-0000-0000-000000000000}".</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QUuid::QUuid()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html#QUuid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates the null UUID. <a href="http://doc.qt.io/qt-5/quuid.html#toString">toString</a>() will output the null UUID as "{00000000-0000-0000-0000-000000000000}".</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QUuid> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUuid_QUuid();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QUuid::QUuid(unsigned int l, unsigned short w1, unsigned short w2, unsigned char b1, unsigned char b2, unsigned char b3, unsigned char b4, unsigned char b5, unsigned char b6, unsigned char b7, unsigned char b8)```</span>.
    pub unsafe fn from_uint2_ushort8_uchar(
        l: ::std::os::raw::c_uint,
        w1: ::std::os::raw::c_ushort,
        w2: ::std::os::raw::c_ushort,
        b1: ::std::os::raw::c_uchar,
        b2: ::std::os::raw::c_uchar,
        b3: ::std::os::raw::c_uchar,
        b4: ::std::os::raw::c_uchar,
        b5: ::std::os::raw::c_uchar,
        b6: ::std::os::raw::c_uchar,
        b7: ::std::os::raw::c_uchar,
        b8: ::std::os::raw::c_uchar,
    ) -> ::cpp_utils::CppBox<crate::QUuid> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QUuid_QUuid1(l, w1, w2, b1, b2, b3, b4, b5, b6, b7, b8);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> object from the string <i>text</i>, which must be formatted as five hex fields separated by '-', e.g., "{xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx}" where 'x' is a hex digit. The curly braces shown here are optional, but it is normal to include them. If the conversion fails, a null UUID is created. See <a href="http://doc.qt.io/qt-5/quuid.html#toString">toString</a>() for an explanation of how the five hex fields map to the public data members in <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QUuid::QUuid(const QString& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html#QUuid-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> object from the string <i>text</i>, which must be formatted as five hex fields separated by '-', e.g., "{xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx}" where 'x' is a hex digit. The curly braces shown here are optional, but it is normal to include them. If the conversion fails, a null UUID is created. See <a href="http://doc.qt.io/qt-5/quuid.html#toString">toString</a>() for an explanation of how the five hex fields map to the public data members in <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/quuid.html#toString">toString</a>() and <a href="http://doc.qt.io/qt-5/quuid.html#QUuid">QUuid</a>().</p></div>
    pub unsafe fn from_q_string(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QUuid> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUuid_QUuid2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(arg1).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QUuid::QUuid(const char* arg1)```</span>.
    pub unsafe fn from_char(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QUuid> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUuid_QUuid3(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> object from the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> <i>text</i>, which must be formatted as five hex fields separated by '-', e.g., "{xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx}" where 'x' is a hex digit. The curly braces shown here are optional, but it is normal to include them. If the conversion fails, a null UUID is created. See <a href="http://doc.qt.io/qt-5/quuid.html#toByteArray">toByteArray</a>() for an explanation of how the five hex fields map to the public data members in <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QUuid::QUuid(const QByteArray& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html#QUuid-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> object from the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> <i>text</i>, which must be formatted as five hex fields separated by '-', e.g., "{xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx}" where 'x' is a hex digit. The curly braces shown here are optional, but it is normal to include them. If the conversion fails, a null UUID is created. See <a href="http://doc.qt.io/qt-5/quuid.html#toByteArray">toByteArray</a>() for an explanation of how the five hex fields map to the public data members in <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>.</p>
    /// <p>This function was introduced in  Qt 4.8.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/quuid.html#toByteArray">toByteArray</a>() and <a href="http://doc.qt.io/qt-5/quuid.html#QUuid">QUuid</a>().</p></div>
    pub unsafe fn from_q_byte_array(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QUuid> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUuid_QUuid4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> class stores a Universally Unique Identifier (UUID).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QUuid::QUuid(const QUuid& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> class stores a Universally Unique Identifier (UUID).</p>
    /// <p>Using <i>U</i>niversally <i>U</i>nique <i>ID</i>entifiers (UUID) is a standard way to uniquely identify entities in a distributed computing environment. A UUID is a 16-byte (128-bit) number generated by some algorithm that is meant to guarantee that the UUID will be unique in the distributed computing environment where it is used. The acronym GUID is often used instead, <i>G</i>lobally <i>U</i>nique <i>ID</i>entifiers, but it refers to the same thing.</p>
    /// <a name="variant-field"></a><p>Actually, the GUID is one <i>variant</i> of UUID. Multiple variants are in use. Each UUID contains a bit field that specifies which type (variant) of UUID it is. Call <a href="http://doc.qt.io/qt-5/quuid.html#variant">variant</a>() to discover which type of UUID an instance of <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> contains. It extracts the three most significant bits of byte 8 of the 16 bytes. In <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>, byte 8 is <code>QUuid::data4[0]</code>. If you create instances of <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> using the constructor that accepts all the numeric values as parameters, use the following table to set the three most significant bits of parameter <code>b1</code>, which becomes <code>QUuid::data4[0]</code> and contains the variant field in its three most significant bits. In the table, 'x' means <i>don't care</i>.</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>msb0</th><th>msb1</th><th>msb2</th><th>Variant</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>0</td><td>x</td><td>x</td><td>NCS (Network Computing System)</td></tr>
    /// <tr valign="top" class="even"><td>1</td><td>0</td><td>x</td><td>DCE (Distributed Computing Environment)</td></tr>
    /// <tr valign="top" class="odd"><td>1</td><td>1</td><td>0</td><td>Microsoft (GUID)</td></tr>
    /// <tr valign="top" class="even"><td>1</td><td>1</td><td>1</td><td>Reserved for future expansion</td></tr>
    /// </tbody></table></div>
    /// <a name="version-field"></a><p>If <a href="http://doc.qt.io/qt-5/quuid.html#variant">variant</a>() returns <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::DCE</a>, the UUID also contains a <i>version</i> field in the four most significant bits of <code>QUuid::data3</code>, and you can call <a href="http://doc.qt.io/qt-5/quuid.html#version">version</a>() to discover which version your <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> contains. If you create instances of <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> using the constructor that accepts all the numeric values as parameters, use the following table to set the four most significant bits of parameter <code>w2</code>, which becomes <code>QUuid::data3</code> and contains the version field in its four most significant bits.</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>msb0</th><th>msb1</th><th>msb2</th><th>msb3</th><th>Version</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>0</td><td>0</td><td>0</td><td>1</td><td>Time</td></tr>
    /// <tr valign="top" class="even"><td>0</td><td>0</td><td>1</td><td>0</td><td>Embedded POSIX</td></tr>
    /// <tr valign="top" class="odd"><td>0</td><td>0</td><td>1</td><td>1</td><td>Md5(Name)</td></tr>
    /// <tr valign="top" class="even"><td>0</td><td>1</td><td>0</td><td>0</td><td>Random</td></tr>
    /// <tr valign="top" class="odd"><td>0</td><td>1</td><td>0</td><td>1</td><td>Sha1</td></tr>
    /// </tbody></table></div>
    /// <p>The field layouts for the DCE versions listed in the table above are specified in the <a href="http://www.ietf.org/rfc/rfc4122.txt">Network Working Group UUID Specification</a>.</p>
    /// <p>Most platforms provide a tool for generating new UUIDs, e.g. <code>uuidgen</code> and <code>guidgen</code>. You can also use <a href="http://doc.qt.io/qt-5/quuid.html#createUuid">createUuid</a>(). UUIDs generated by <a href="http://doc.qt.io/qt-5/quuid.html#createUuid">createUuid</a>() are of the random type. Their <a href="http://doc.qt.io/qt-5/quuid.html#Version-enum">QUuid::Version</a> bits are set to <a href="http://doc.qt.io/qt-5/quuid.html#Version-enum">QUuid::Random</a>, and their <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::Variant</a> bits are set to <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::DCE</a>. The rest of the UUID is composed of random numbers. Theoretically, this means there is a small chance that a UUID generated by <a href="http://doc.qt.io/qt-5/quuid.html#createUuid">createUuid</a>() will not be unique. But it is <a href="http://en.wikipedia.org/wiki/Universally_Unique_Identifier#Random_UUID_probability_of_duplicates">a <i>very</i> small chance</a>.</p>
    /// <p>UUIDs can be constructed from numeric values or from strings, or using the static <a href="http://doc.qt.io/qt-5/quuid.html#createUuid">createUuid</a>() function. They can be converted to a string with <a href="http://doc.qt.io/qt-5/quuid.html#toString">toString</a>(). UUIDs have a <a href="http://doc.qt.io/qt-5/quuid.html#variant">variant</a>() and a <a href="http://doc.qt.io/qt-5/quuid.html#version">version</a>(), and null UUIDs return true from <a href="http://doc.qt.io/qt-5/quuid.html#isNull">isNull</a>().</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUuid>>,
    ) -> ::cpp_utils::CppBox<crate::QUuid> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUuid_QUuid5(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUuid>>::cast_into(other).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Sets the value of the <span style='color: green;'>```data1```</span> field.
    pub unsafe fn set_data1(&mut self, value: ::std::os::raw::c_uint) {
        crate::__ffi::ctr_qt_core_ffi_set_QUuid_data1(self as *mut crate::QUuid, value)
    }

    /// Sets the value of the <span style='color: green;'>```data2```</span> field.
    pub unsafe fn set_data2(&mut self, value: ::std::os::raw::c_ushort) {
        crate::__ffi::ctr_qt_core_ffi_set_QUuid_data2(self as *mut crate::QUuid, value)
    }

    /// Sets the value of the <span style='color: green;'>```data3```</span> field.
    pub unsafe fn set_data3(&mut self, value: ::std::os::raw::c_ushort) {
        crate::__ffi::ctr_qt_core_ffi_set_QUuid_data3(self as *mut crate::QUuid, value)
    }

    /// <p>Returns the binary representation of this <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>. The byte array is formatted as five hex fields separated by '-' and enclosed in curly braces, i.e., "{xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx}" where 'x' is a hex digit. From left to right, the five hex fields are obtained from the four public data members in <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> as follows:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QUuid::toByteArray() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html#toByteArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the binary representation of this <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>. The byte array is formatted as five hex fields separated by '-' and enclosed in curly braces, i.e., "{xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx}" where 'x' is a hex digit. From left to right, the five hex fields are obtained from the four public data members in <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> as follows:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Field #</th><th>Source</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>1</td><td>data1</td></tr>
    /// <tr valign="top" class="even"><td>2</td><td>data2</td></tr>
    /// <tr valign="top" class="odd"><td>3</td><td>data3</td></tr>
    /// <tr valign="top" class="even"><td>4</td><td>data4[0] .. data4[1]</td></tr>
    /// <tr valign="top" class="odd"><td>5</td><td>data4[2] .. data4[7]</td></tr>
    /// </tbody></table></div>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn to_byte_array_0a(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QUuid_toByteArray(self as *const crate::QUuid);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the string representation of this <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>, with the formattiong controlled by the <i>mode</i> parameter. From left to right, the five hex fields are obtained from the four public data members in <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> as follows:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QUuid::toByteArray(QUuid::StringFormat mode) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html#toByteArray-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string representation of this <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>, with the formattiong controlled by the <i>mode</i> parameter. From left to right, the five hex fields are obtained from the four public data members in <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> as follows:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Field #</th><th>Source</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>1</td><td>data1</td></tr>
    /// <tr valign="top" class="even"><td>2</td><td>data2</td></tr>
    /// <tr valign="top" class="odd"><td>3</td><td>data3</td></tr>
    /// <tr valign="top" class="even"><td>4</td><td>data4[0] .. data4[1]</td></tr>
    /// <tr valign="top" class="odd"><td>5</td><td>data4[2] .. data4[7]</td></tr>
    /// </tbody></table></div>
    /// <p>This function was introduced in  Qt 5.11.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_byte_array_1a(
        &self,
        mode: crate::q_uuid::StringFormat,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QUuid_toByteArray2(self as *const crate::QUuid, mode);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the binary representation of this <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>. The byte array is in big endian format, and formatted according to RFC 4122, section 4.1.2 - "Layout and byte order".</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QUuid::toRfc4122() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html#toRfc4122">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the binary representation of this <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>. The byte array is in big endian format, and formatted according to RFC 4122, section 4.1.2 - "Layout and byte order".</p>
    /// <p>The order is as follows:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Field #</th><th>Source</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>1</td><td>data1</td></tr>
    /// <tr valign="top" class="even"><td>2</td><td>data2</td></tr>
    /// <tr valign="top" class="odd"><td>3</td><td>data3</td></tr>
    /// <tr valign="top" class="even"><td>4</td><td>data4[0] .. data4[7]</td></tr>
    /// </tbody></table></div>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn to_rfc4122(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUuid_toRfc4122(self as *const crate::QUuid);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the string representation of this <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>. The string is formatted as five hex fields separated by '-' and enclosed in curly braces, i.e., "{xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx}" where 'x' is a hex digit. From left to right, the five hex fields are obtained from the four public data members in <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> as follows:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUuid::toString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html#toString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string representation of this <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>. The string is formatted as five hex fields separated by '-' and enclosed in curly braces, i.e., "{xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx}" where 'x' is a hex digit. From left to right, the five hex fields are obtained from the four public data members in <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> as follows:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Field #</th><th>Source</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>1</td><td>data1</td></tr>
    /// <tr valign="top" class="even"><td>2</td><td>data2</td></tr>
    /// <tr valign="top" class="odd"><td>3</td><td>data3</td></tr>
    /// <tr valign="top" class="even"><td>4</td><td>data4[0] .. data4[1]</td></tr>
    /// <tr valign="top" class="odd"><td>5</td><td>data4[2] .. data4[7]</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn to_string_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUuid_toString(self as *const crate::QUuid);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the string representation of this <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>, with the formattiong controlled by the <i>mode</i> parameter. From left to right, the five hex fields are obtained from the four public data members in <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> as follows:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QUuid::toString(QUuid::StringFormat mode) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html#toString-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string representation of this <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>, with the formattiong controlled by the <i>mode</i> parameter. From left to right, the five hex fields are obtained from the four public data members in <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> as follows:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Field #</th><th>Source</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>1</td><td>data1</td></tr>
    /// <tr valign="top" class="even"><td>2</td><td>data2</td></tr>
    /// <tr valign="top" class="odd"><td>3</td><td>data3</td></tr>
    /// <tr valign="top" class="even"><td>4</td><td>data4[0] .. data4[1]</td></tr>
    /// <tr valign="top" class="odd"><td>5</td><td>data4[2] .. data4[7]</td></tr>
    /// </tbody></table></div>
    /// <p>This function was introduced in  Qt 5.11.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_string_1a(
        &self,
        mode: crate::q_uuid::StringFormat,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QUuid_toString2(self as *const crate::QUuid, mode);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value in the <a href="http://doc.qt.io/qt-5/quuid.html#variant-field">variant field</a> of the UUID. If the return value is <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::DCE</a>, call <a href="http://doc.qt.io/qt-5/quuid.html#version">version</a>() to see which layout it uses. The null UUID is considered to be of an unknown variant.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QUuid::Variant QUuid::variant() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html#variant">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value in the <a href="http://doc.qt.io/qt-5/quuid.html#variant-field">variant field</a> of the UUID. If the return value is <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::DCE</a>, call <a href="http://doc.qt.io/qt-5/quuid.html#version">version</a>() to see which layout it uses. The null UUID is considered to be of an unknown variant.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/quuid.html#version">version</a>().</p></div>
    pub unsafe fn variant(&self) -> crate::q_uuid::Variant {
        crate::__ffi::ctr_qt_core_ffi_QUuid_variant(self as *const crate::QUuid)
    }

    /// <p>Returns the <a href="http://doc.qt.io/qt-5/quuid.html#version-field">version field</a> of the UUID, if the UUID's <a href="http://doc.qt.io/qt-5/quuid.html#variant-field">variant field</a> is <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::DCE</a>. Otherwise it returns <a href="http://doc.qt.io/qt-5/quuid.html#Version-enum">QUuid::VerUnknown</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QUuid::Version QUuid::version() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html#version">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the <a href="http://doc.qt.io/qt-5/quuid.html#version-field">version field</a> of the UUID, if the UUID's <a href="http://doc.qt.io/qt-5/quuid.html#variant-field">variant field</a> is <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::DCE</a>. Otherwise it returns <a href="http://doc.qt.io/qt-5/quuid.html#Version-enum">QUuid::VerUnknown</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/quuid.html#variant">variant</a>().</p></div>
    pub unsafe fn version(&self) -> crate::q_uuid::Version {
        crate::__ffi::ctr_qt_core_ffi_QUuid_version(self as *const crate::QUuid)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qwaitcondition.html">QWaitCondition</a> class provides a condition variable for synchronizing threads.</p>
///
/// C++ class: <span style='color: green;'>```QWaitCondition```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qwaitcondition.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qwaitcondition.html">QWaitCondition</a> class provides a condition variable for synchronizing threads.</p>
/// <p><a href="http://doc.qt.io/qt-5/qwaitcondition.html">QWaitCondition</a> allows a thread to tell other threads that some sort of condition has been met. One or many threads can block waiting for a <a href="http://doc.qt.io/qt-5/qwaitcondition.html">QWaitCondition</a> to set a condition with <a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeOne">wakeOne</a>() or <a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeAll">wakeAll</a>(). Use <a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeOne">wakeOne</a>() to wake one randomly selected thread or <a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeAll">wakeAll</a>() to wake them all.</p>
/// <p>For example, let's suppose that we have three tasks that should be performed whenever the user presses a key. Each task could be split into a thread, each of which would have a <a href="http://doc.qt.io/qt-5/qthread.html#run">run()</a> body like this:</p>
/// <pre class="cpp">
///
///   forever {
/// &#32;     mutex<span class="operator">.</span>lock();
/// &#32;     keyPressed<span class="operator">.</span>wait(<span class="operator">&amp;</span>mutex);
/// &#32;     do_something();
/// &#32;     mutex<span class="operator">.</span>unlock();
///   }
///
/// </pre>
/// <p>Here, the <code>keyPressed</code> variable is a global variable of type <a href="http://doc.qt.io/qt-5/qwaitcondition.html">QWaitCondition</a>.</p>
/// <p>A fourth thread would read key presses and wake the other three threads up every time it receives one, like this:</p>
/// <pre class="cpp">
///
///   forever {
/// &#32;     getchar();
/// &#32;     keyPressed<span class="operator">.</span>wakeAll();
///   }
///
/// </pre>
/// <p>The order in which the three threads are woken up is undefined. Also, if some of the threads are still in <code>do_something()</code> when the key is pressed, they won't be woken up (since they're not waiting on the condition variable) and so the task will not be performed for that key press. This issue can be solved using a counter and a <a href="http://doc.qt.io/qt-5/qmutex.html">QMutex</a> to guard it. For example, here's the new code for the worker threads:</p>
/// <pre class="cpp">
///
///   forever {
/// &#32;     mutex<span class="operator">.</span>lock();
/// &#32;     keyPressed<span class="operator">.</span>wait(<span class="operator">&amp;</span>mutex);
/// &#32;     <span class="operator">+</span><span class="operator">+</span>count;
/// &#32;     mutex<span class="operator">.</span>unlock();
///
/// &#32;     do_something();
///
/// &#32;     mutex<span class="operator">.</span>lock();
/// &#32;     <span class="operator">-</span><span class="operator">-</span>count;
/// &#32;     mutex<span class="operator">.</span>unlock();
///   }
///
/// </pre>
/// <p>Here's the code for the fourth thread:</p>
/// <pre class="cpp">
///
///   forever {
/// &#32;     getchar();
///
/// &#32;     mutex<span class="operator">.</span>lock();
/// &#32;     <span class="comment">// Sleep until there are no busy worker threads</span>
/// &#32;     <span class="keyword">while</span> (count <span class="operator">&gt;</span> <span class="number">0</span>) {
/// &#32;   &#32;     mutex<span class="operator">.</span>unlock();
/// &#32;   &#32;     sleep(<span class="number">1</span>);
/// &#32;   &#32;     mutex<span class="operator">.</span>lock();
/// &#32;     }
/// &#32;     keyPressed<span class="operator">.</span>wakeAll();
/// &#32;     mutex<span class="operator">.</span>unlock();
///   }
///
/// </pre>
/// <p>The mutex is necessary because the results of two threads attempting to change the value of the same variable simultaneously are unpredictable.</p>
/// <p>Wait conditions are a powerful thread synchronization primitive. The <a href="http://doc.qt.io/qt-5/qtcore-threads-waitconditions-example.html">Wait Conditions Example</a> example shows how to use <a href="http://doc.qt.io/qt-5/qwaitcondition.html">QWaitCondition</a> as an alternative to <a href="http://doc.qt.io/qt-5/qsemaphore.html">QSemaphore</a> for controlling access to a circular buffer shared by a producer thread and a consumer thread.</p></div>
#[repr(C)]
pub struct QWaitCondition {
    _unused: u8,
}
impl QWaitCondition {
    /// <p>Constructs a new wait condition object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QWaitCondition::QWaitCondition()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qwaitcondition.html#QWaitCondition">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new wait condition object.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QWaitCondition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QWaitCondition_QWaitCondition();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeAll">wakeAll</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QWaitCondition::notify_all()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qwaitcondition.html#notify_all">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeAll">wakeAll</a>().</p>
    /// <p>This function was introduced in  Qt 5.8.</p></div>
    pub unsafe fn notify_all(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QWaitCondition_notify_all(self as *mut crate::QWaitCondition)
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeOne">wakeOne</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QWaitCondition::notify_one()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qwaitcondition.html#notify_one">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeOne">wakeOne</a>().</p>
    /// <p>This function was introduced in  Qt 5.8.</p></div>
    pub unsafe fn notify_one(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QWaitCondition_notify_one(self as *mut crate::QWaitCondition)
    }

    /// <p>Releases the <i>lockedMutex</i> and waits on the wait condition. The <i>lockedMutex</i> must be initially locked by the calling thread. If <i>lockedMutex</i> is not in a locked state, the behavior is undefined. If <i>lockedMutex</i> is a recursive mutex, this function returns immediately. The <i>lockedMutex</i> will be unlocked, and the calling thread will block until either of these conditions is met:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QWaitCondition::wait(QMutex* lockedMutex, unsigned long time = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qwaitcondition.html#wait">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Releases the <i>lockedMutex</i> and waits on the wait condition. The <i>lockedMutex</i> must be initially locked by the calling thread. If <i>lockedMutex</i> is not in a locked state, the behavior is undefined. If <i>lockedMutex</i> is a recursive mutex, this function returns immediately. The <i>lockedMutex</i> will be unlocked, and the calling thread will block until either of these conditions is met:</p>
    /// <ul>
    /// <li>Another thread signals it using <a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeOne">wakeOne</a>() or <a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeAll">wakeAll</a>(). This function will return true in this case.</li>
    /// <li><i>time</i> milliseconds has elapsed. If <i>time</i> is <code>ULONG_MAX</code> (the default), then the wait will never timeout (the event must be signalled). This function will return false if the wait timed out.</li>
    /// </ul>
    /// <p>The <i>lockedMutex</i> will be returned to the same locked state. This function is provided to allow the atomic transition from the locked state to the wait state.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeOne">wakeOne</a>() and <a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeAll">wakeAll</a>().</p></div>
    pub unsafe fn wait_q_mutex_ulong(
        &mut self,
        locked_mutex: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QMutex>>,
        time: ::std::os::raw::c_ulong,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QWaitCondition_wait(
            self as *mut crate::QWaitCondition,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QMutex>>::cast_into(locked_mutex)
                .as_mut_raw_ptr(),
            time,
        )
    }

    /// <p>Releases the <i>lockedReadWriteLock</i> and waits on the wait condition. The <i>lockedReadWriteLock</i> must be initially locked by the calling thread. If <i>lockedReadWriteLock</i> is not in a locked state, this function returns immediately. The <i>lockedReadWriteLock</i> must not be locked recursively, otherwise this function will not release the lock properly. The <i>lockedReadWriteLock</i> will be unlocked, and the calling thread will block until either of these conditions is met:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QWaitCondition::wait(QReadWriteLock* lockedReadWriteLock, unsigned long time = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qwaitcondition.html#wait-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Releases the <i>lockedReadWriteLock</i> and waits on the wait condition. The <i>lockedReadWriteLock</i> must be initially locked by the calling thread. If <i>lockedReadWriteLock</i> is not in a locked state, this function returns immediately. The <i>lockedReadWriteLock</i> must not be locked recursively, otherwise this function will not release the lock properly. The <i>lockedReadWriteLock</i> will be unlocked, and the calling thread will block until either of these conditions is met:</p>
    /// <ul>
    /// <li>Another thread signals it using <a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeOne">wakeOne</a>() or <a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeAll">wakeAll</a>(). This function will return true in this case.</li>
    /// <li><i>time</i> milliseconds has elapsed. If <i>time</i> is <code>ULONG_MAX</code> (the default), then the wait will never timeout (the event must be signalled). This function will return false if the wait timed out.</li>
    /// </ul>
    /// <p>The <i>lockedReadWriteLock</i> will be returned to the same locked state. This function is provided to allow the atomic transition from the locked state to the wait state.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeOne">wakeOne</a>() and <a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeAll">wakeAll</a>().</p></div>
    pub unsafe fn wait_q_read_write_lock_ulong(
        &mut self,
        locked_read_write_lock: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QReadWriteLock>>,
        time: ::std::os::raw::c_ulong,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QWaitCondition_wait1(
            self as *mut crate::QWaitCondition,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QReadWriteLock>>::cast_into(
                locked_read_write_lock,
            )
            .as_mut_raw_ptr(),
            time,
        )
    }

    /// <p>Releases the <i>lockedMutex</i> and waits on the wait condition. The <i>lockedMutex</i> must be initially locked by the calling thread. If <i>lockedMutex</i> is not in a locked state, the behavior is undefined. If <i>lockedMutex</i> is a recursive mutex, this function returns immediately. The <i>lockedMutex</i> will be unlocked, and the calling thread will block until either of these conditions is met:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QWaitCondition::wait(QMutex* lockedMutex)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qwaitcondition.html#wait">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Releases the <i>lockedMutex</i> and waits on the wait condition. The <i>lockedMutex</i> must be initially locked by the calling thread. If <i>lockedMutex</i> is not in a locked state, the behavior is undefined. If <i>lockedMutex</i> is a recursive mutex, this function returns immediately. The <i>lockedMutex</i> will be unlocked, and the calling thread will block until either of these conditions is met:</p>
    /// <ul>
    /// <li>Another thread signals it using <a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeOne">wakeOne</a>() or <a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeAll">wakeAll</a>(). This function will return true in this case.</li>
    /// <li><i>time</i> milliseconds has elapsed. If <i>time</i> is <code>ULONG_MAX</code> (the default), then the wait will never timeout (the event must be signalled). This function will return false if the wait timed out.</li>
    /// </ul>
    /// <p>The <i>lockedMutex</i> will be returned to the same locked state. This function is provided to allow the atomic transition from the locked state to the wait state.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeOne">wakeOne</a>() and <a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeAll">wakeAll</a>().</p></div>
    pub unsafe fn wait_q_mutex(
        &mut self,
        locked_mutex: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QMutex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QWaitCondition_wait2(
            self as *mut crate::QWaitCondition,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QMutex>>::cast_into(locked_mutex)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Releases the <i>lockedReadWriteLock</i> and waits on the wait condition. The <i>lockedReadWriteLock</i> must be initially locked by the calling thread. If <i>lockedReadWriteLock</i> is not in a locked state, this function returns immediately. The <i>lockedReadWriteLock</i> must not be locked recursively, otherwise this function will not release the lock properly. The <i>lockedReadWriteLock</i> will be unlocked, and the calling thread will block until either of these conditions is met:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QWaitCondition::wait(QReadWriteLock* lockedReadWriteLock)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qwaitcondition.html#wait-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Releases the <i>lockedReadWriteLock</i> and waits on the wait condition. The <i>lockedReadWriteLock</i> must be initially locked by the calling thread. If <i>lockedReadWriteLock</i> is not in a locked state, this function returns immediately. The <i>lockedReadWriteLock</i> must not be locked recursively, otherwise this function will not release the lock properly. The <i>lockedReadWriteLock</i> will be unlocked, and the calling thread will block until either of these conditions is met:</p>
    /// <ul>
    /// <li>Another thread signals it using <a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeOne">wakeOne</a>() or <a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeAll">wakeAll</a>(). This function will return true in this case.</li>
    /// <li><i>time</i> milliseconds has elapsed. If <i>time</i> is <code>ULONG_MAX</code> (the default), then the wait will never timeout (the event must be signalled). This function will return false if the wait timed out.</li>
    /// </ul>
    /// <p>The <i>lockedReadWriteLock</i> will be returned to the same locked state. This function is provided to allow the atomic transition from the locked state to the wait state.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeOne">wakeOne</a>() and <a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeAll">wakeAll</a>().</p></div>
    pub unsafe fn wait_q_read_write_lock(
        &mut self,
        locked_read_write_lock: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QReadWriteLock>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QWaitCondition_wait3(
            self as *mut crate::QWaitCondition,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QReadWriteLock>>::cast_into(
                locked_read_write_lock,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QWaitCondition::wait(QMutex* lockedMutex, QDeadlineTimer deadline)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn wait_q_mutex_q_deadline_timer(
        &mut self,
        locked_mutex: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QMutex>>,
        deadline: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDeadlineTimer>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QWaitCondition_wait8(
            self as *mut crate::QWaitCondition,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QMutex>>::cast_into(locked_mutex)
                .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDeadlineTimer>>::cast_into(deadline)
                .as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QWaitCondition::wait(QReadWriteLock* lockedReadWriteLock, QDeadlineTimer deadline)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn wait_q_read_write_lock_q_deadline_timer(
        &mut self,
        locked_read_write_lock: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QReadWriteLock>>,
        deadline: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDeadlineTimer>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QWaitCondition_wait9(
            self as *mut crate::QWaitCondition,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QReadWriteLock>>::cast_into(
                locked_read_write_lock,
            )
            .as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDeadlineTimer>>::cast_into(deadline)
                .as_raw_ptr(),
        )
    }

    /// <p>Wakes all threads waiting on the wait condition. The order in which the threads are woken up depends on the operating system's scheduling policies and cannot be controlled or predicted.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QWaitCondition::wakeAll()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Wakes all threads waiting on the wait condition. The order in which the threads are woken up depends on the operating system's scheduling policies and cannot be controlled or predicted.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeOne">wakeOne</a>().</p></div>
    pub unsafe fn wake_all(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QWaitCondition_wakeAll(self as *mut crate::QWaitCondition)
    }

    /// <p>Wakes one thread waiting on the wait condition. The thread that is woken up depends on the operating system's scheduling policies, and cannot be controlled or predicted.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QWaitCondition::wakeOne()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeOne">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Wakes one thread waiting on the wait condition. The thread that is woken up depends on the operating system's scheduling policies, and cannot be controlled or predicted.</p>
    /// <p>If you want to wake up a specific thread, the solution is typically to use different wait conditions and have different threads wait on different conditions.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qwaitcondition.html#wakeAll">wakeAll</a>().</p></div>
    pub unsafe fn wake_one(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QWaitCondition_wakeOne(self as *mut crate::QWaitCondition)
    }
}

/// C++ class: <span style='color: green;'>```QXmlStreamStringRef```</span>.
#[repr(C)]
pub struct QXmlStreamStringRef {
    _unused: u8,
}
impl QXmlStreamStringRef {
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamStringRef::clear()```</span>.
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamStringRef_clear(
            self as *mut crate::QXmlStreamStringRef,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```QXmlStreamStringRef& QXmlStreamStringRef::operator=(const QXmlStreamStringRef& other)```</span>.
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamStringRef>>,
    ) -> ::cpp_utils::MutRef<crate::QXmlStreamStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamStringRef_operator_1(
            self as *mut crate::QXmlStreamStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamStringRef>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamStringRef::QXmlStreamStringRef()```</span>.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QXmlStreamStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamStringRef_QXmlStreamStringRef();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamStringRef::QXmlStreamStringRef(const QStringRef& aString)```</span>.
    pub unsafe fn from_q_string_ref(
        a_string: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamStringRef_QXmlStreamStringRef1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(a_string)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamStringRef::QXmlStreamStringRef(const QString& aString)```</span>.
    pub unsafe fn from_q_string(
        a_string: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamStringRef_QXmlStreamStringRef2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(a_string)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamStringRef::QXmlStreamStringRef(const QXmlStreamStringRef& other)```</span>.
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamStringRef>>,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamStringRef_QXmlStreamStringRef4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamStringRef>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```int QXmlStreamStringRef::position() const```</span>.
    pub unsafe fn position(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamStringRef_position(
            self as *const crate::QXmlStreamStringRef,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```int QXmlStreamStringRef::size() const```</span>.
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamStringRef_size(
            self as *const crate::QXmlStreamStringRef,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```const QString* QXmlStreamStringRef::string() const```</span>.
    pub unsafe fn string(&self) -> ::cpp_utils::Ptr<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamStringRef_string(
            self as *const crate::QXmlStreamStringRef,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamStringRef::swap(QXmlStreamStringRef& other)```</span>.
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QXmlStreamStringRef>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamStringRef_swap(
            self as *mut crate::QXmlStreamStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QXmlStreamStringRef>>::cast_into(
                other,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamStringRef::operator QStringRef() const```</span>.
    pub unsafe fn to_q_string_ref(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamStringRef_operator_QStringRef(
            self as *const crate::QXmlStreamStringRef,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html">QXmlStreamAttribute</a> class represents a single XML attribute</p>
///
/// C++ class: <span style='color: green;'>```QXmlStreamAttribute```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html">QXmlStreamAttribute</a> class represents a single XML attribute</p>
/// <p>An attribute consists of an optionally empty <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html#namespaceUri">namespaceUri</a>(), a <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html#name">name</a>(), a <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html#value">value</a>(), and an <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html#isDefault">isDefault</a>() attribute.</p>
/// <p>The raw XML attribute name is returned as <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html#qualifiedName">qualifiedName</a>().</p></div>
#[repr(C)]
pub struct QXmlStreamAttribute {
    _unused: u8,
}
impl QXmlStreamAttribute {
    /// <p>Assigns <i>other</i> to this attribute.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamAttribute& QXmlStreamAttribute::operator=(const QXmlStreamAttribute& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html#operator-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this attribute.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamAttribute>>,
    ) -> ::cpp_utils::MutRef<crate::QXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamAttribute_operator_1(
            self as *mut crate::QXmlStreamAttribute,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamAttribute>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the parser added this attribute with a default value following an ATTLIST declaration in the DTD; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QXmlStreamAttribute::isDefault() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html#isDefault">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the parser added this attribute with a default value following an ATTLIST declaration in the DTD; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn is_default(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamAttribute_isDefault(
            self as *const crate::QXmlStreamAttribute,
        )
    }

    /// <p>Returns the attribute's local name.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamAttribute::name() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html#name">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the attribute's local name.</p></div>
    pub unsafe fn name(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamAttribute_name(
            self as *const crate::QXmlStreamAttribute,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the attribute's resolved namespaceUri, or an empty string reference if the attribute does not have a defined namespace.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamAttribute::namespaceUri() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html#namespaceUri">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the attribute's resolved namespaceUri, or an empty string reference if the attribute does not have a defined namespace.</p></div>
    pub unsafe fn namespace_uri(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamAttribute_namespaceUri(
            self as *const crate::QXmlStreamAttribute,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates an empty attribute.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamAttribute::QXmlStreamAttribute()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html#QXmlStreamAttribute">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates an empty attribute.</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamAttribute_QXmlStreamAttribute();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs an attribute with qualified name <i>qualifiedName</i> and value <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamAttribute::QXmlStreamAttribute(const QString& qualifiedName, const QString& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html#QXmlStreamAttribute-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an attribute with qualified name <i>qualifiedName</i> and value <i>value</i>.</p></div>
    pub unsafe fn new_2a(
        qualified_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamAttribute_QXmlStreamAttribute1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(qualified_name)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs an attribute in the namespace described with <i>namespaceUri</i> with <i>name</i> and value <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamAttribute::QXmlStreamAttribute(const QString& namespaceUri, const QString& name, const QString& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html#QXmlStreamAttribute-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an attribute in the namespace described with <i>namespaceUri</i> with <i>name</i> and value <i>value</i>.</p></div>
    pub unsafe fn new_3a(
        namespace_uri: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamAttribute_QXmlStreamAttribute2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(namespace_uri)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamAttribute::QXmlStreamAttribute(const QXmlStreamAttribute& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html#QXmlStreamAttribute-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a copy of <i>other</i>.</p></div>
    pub unsafe fn new_copy(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamAttribute>>,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamAttribute_QXmlStreamAttribute3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamAttribute>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the attribute's namespace prefix.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamAttribute::prefix() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html#prefix">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the attribute's namespace prefix.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html#name">name</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html#qualifiedName">qualifiedName</a>().</p></div>
    pub unsafe fn prefix(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamAttribute_prefix(
            self as *const crate::QXmlStreamAttribute,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the attribute's qualified name.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamAttribute::qualifiedName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html#qualifiedName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the attribute's qualified name.</p>
    /// <p>A qualified name is the raw name of an attribute in the XML data. It consists of the namespace <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html#prefix">prefix</a>(), followed by colon, followed by the attribute's local <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html#name">name</a>(). Since the namespace prefix is not unique (the same prefix can point to different namespaces and different prefixes can point to the same namespace), you shouldn't use qualifiedName(), but the resolved <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html#namespaceUri">namespaceUri</a>() and the attribute's local <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html#name">name</a>().</p></div>
    pub unsafe fn qualified_name(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamAttribute_qualifiedName(
            self as *const crate::QXmlStreamAttribute,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the attribute's value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamAttribute::value() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the attribute's value.</p></div>
    pub unsafe fn value(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamAttribute_value(
            self as *const crate::QXmlStreamAttribute,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html">QXmlStreamAttributes</a> class represents a vector of <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html">QXmlStreamAttribute</a>.</p>
///
/// C++ class: <span style='color: green;'>```QXmlStreamAttributes```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html">QXmlStreamAttributes</a> class represents a vector of <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html">QXmlStreamAttribute</a>.</p>
/// <p>Attributes are returned by a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html">QXmlStreamReader</a> in <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#attributes">attributes()</a> when the reader reports a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">start element</a>. The class can also be used with a <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a> as an argument to <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeAttributes">writeAttributes()</a>.</p>
/// <p>The convenience function <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html#value">value</a>() loops over the vector and returns an attribute value for a given namespaceUri and an attribute's name.</p>
/// <p>New attributes can be added with <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html#append">append</a>().</p></div>
#[repr(C)]
pub struct QXmlStreamAttributes {
    _unused: u8,
}
impl QXmlStreamAttributes {
    /// <p>Appends a new attribute with <i>name</i> in the namespace described with <i>namespaceUri</i>, and value <i>value</i>. The <i>namespaceUri</i> can be empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamAttributes::append(const QString& namespaceUri, const QString& name, const QString& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends a new attribute with <i>name</i> in the namespace described with <i>namespaceUri</i>, and value <i>value</i>. The <i>namespaceUri</i> can be empty.</p></div>
    pub unsafe fn append_3a(
        &mut self,
        namespace_uri: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamAttributes_append(
            self as *mut crate::QXmlStreamAttributes,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(namespace_uri)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamAttributes::append(const QString& qualifiedName, const QString& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html#append-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends a new attribute with qualified name <i>qualifiedName</i> and value <i>value</i>.</p></div>
    pub unsafe fn append_2a(
        &mut self,
        qualified_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamAttributes_append1(
            self as *mut crate::QXmlStreamAttributes,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(qualified_name)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html">QXmlStreamAttributes</a> class represents a vector of <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html">QXmlStreamAttribute</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamAttributes& QXmlStreamAttributes::operator=(const QXmlStreamAttributes& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html">QXmlStreamAttributes</a> class represents a vector of <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html">QXmlStreamAttribute</a>.</p>
    /// <p>Attributes are returned by a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html">QXmlStreamReader</a> in <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#attributes">attributes()</a> when the reader reports a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">start element</a>. The class can also be used with a <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a> as an argument to <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeAttributes">writeAttributes()</a>.</p>
    /// <p>The convenience function <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html#value">value</a>() loops over the vector and returns an attribute value for a given namespaceUri and an attribute's name.</p>
    /// <p>New attributes can be added with <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html#append">append</a>().</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamAttributes>>,
    ) -> ::cpp_utils::MutRef<crate::QXmlStreamAttributes> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamAttributes_operator_(
            self as *mut crate::QXmlStreamAttributes,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamAttributes>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if this <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html">QXmlStreamAttributes</a> has an attribute whose qualified name is <i>qualifiedName</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QXmlStreamAttributes::hasAttribute(const QString& qualifiedName) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html#hasAttribute">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html">QXmlStreamAttributes</a> has an attribute whose qualified name is <i>qualifiedName</i>; otherwise returns <code>false</code>.</p>
    /// <p>Note that this is not namespace aware. For instance, if this <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html">QXmlStreamAttributes</a> contains an attribute whose lexical name is "xlink:href" this doesn't tell that an attribute named <code>href</code> in the XLink namespace is present, since the <code>xlink</code> prefix can be bound to any namespace. Use the overload that takes a namespace URI and a local name as parameter, for namespace aware code.</p>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn has_attribute_q_string(
        &self,
        qualified_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamAttributes_hasAttribute(
            self as *const crate::QXmlStreamAttributes,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(qualified_name)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QXmlStreamAttributes::hasAttribute(QLatin1String qualifiedName) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html#hasAttribute-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn has_attribute_q_latin1_string(
        &self,
        qualified_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamAttributes_hasAttribute1(
            self as *const crate::QXmlStreamAttributes,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(
                qualified_name,
            )
            .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QXmlStreamAttributes::hasAttribute(const QString& namespaceUri, const QString& name) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html#hasAttribute-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns <code>true</code> if this <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html">QXmlStreamAttributes</a> has an attribute whose namespace URI and name correspond to <i>namespaceUri</i> and <i>name</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p></div>
    pub unsafe fn has_attribute_2_q_string(
        &self,
        namespace_uri: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamAttributes_hasAttribute2(
            self as *const crate::QXmlStreamAttributes,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(namespace_uri)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
        )
    }

    /// <p>A constructor for <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html">QXmlStreamAttributes</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamAttributes::QXmlStreamAttributes()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html#QXmlStreamAttributes">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>A constructor for <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html">QXmlStreamAttributes</a>.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QXmlStreamAttributes> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamAttributes_QXmlStreamAttributes();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html">QXmlStreamAttributes</a> class represents a vector of <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html">QXmlStreamAttribute</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamAttributes::QXmlStreamAttributes(const QXmlStreamAttributes& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html">QXmlStreamAttributes</a> class represents a vector of <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html">QXmlStreamAttribute</a>.</p>
    /// <p>Attributes are returned by a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html">QXmlStreamReader</a> in <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#attributes">attributes()</a> when the reader reports a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">start element</a>. The class can also be used with a <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a> as an argument to <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeAttributes">writeAttributes()</a>.</p>
    /// <p>The convenience function <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html#value">value</a>() loops over the vector and returns an attribute value for a given namespaceUri and an attribute's name.</p>
    /// <p>New attributes can be added with <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html#append">append</a>().</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamAttributes>>,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamAttributes> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamAttributes_QXmlStreamAttributes1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamAttributes>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value of the attribute <i>name</i> in the namespace described with <i>namespaceUri</i>, or an empty string reference if the attribute is not defined. The <i>namespaceUri</i> can be empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamAttributes::value(const QString& namespaceUri, const QString& name) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value of the attribute <i>name</i> in the namespace described with <i>namespaceUri</i>, or an empty string reference if the attribute is not defined. The <i>namespaceUri</i> can be empty.</p></div>
    pub unsafe fn value_2_q_string(
        &self,
        namespace_uri: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamAttributes_value(
            self as *const crate::QXmlStreamAttributes,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(namespace_uri)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamAttributes::value(const QString& namespaceUri, QLatin1String name) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the value of the attribute <i>name</i> in the namespace described with <i>namespaceUri</i>, or an empty string reference if the attribute is not defined. The <i>namespaceUri</i> can be empty.</p></div>
    pub unsafe fn value_q_string_q_latin1_string(
        &self,
        namespace_uri: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamAttributes_value1(
            self as *const crate::QXmlStreamAttributes,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(namespace_uri)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamAttributes::value(QLatin1String namespaceUri, QLatin1String name) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html#value-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the value of the attribute <i>name</i> in the namespace described with <i>namespaceUri</i>, or an empty string reference if the attribute is not defined. The <i>namespaceUri</i> can be empty.</p></div>
    pub unsafe fn value_2_q_latin1_string(
        &self,
        namespace_uri: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamAttributes_value2(
            self as *const crate::QXmlStreamAttributes,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(
                namespace_uri,
            )
            .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamAttributes::value(const QString& qualifiedName) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html#value-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the value of the attribute with qualified name <i>qualifiedName</i> , or an empty string reference if the attribute is not defined. A qualified name is the raw name of an attribute in the XML data. It consists of the namespace prefix, followed by colon, followed by the attribute's local name. Since the namespace prefix is not unique (the same prefix can point to different namespaces and different prefixes can point to the same namespace), you shouldn't use qualified names, but a resolved namespaceUri and the attribute's local name.</p></div>
    pub unsafe fn value_q_string(
        &self,
        qualified_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamAttributes_value3(
            self as *const crate::QXmlStreamAttributes,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(qualified_name)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamAttributes::value(QLatin1String qualifiedName) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html#value-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the value of the attribute with qualified name <i>qualifiedName</i> , or an empty string reference if the attribute is not defined. A qualified name is the raw name of an attribute in the XML data. It consists of the namespace prefix, followed by colon, followed by the attribute's local name. Since the namespace prefix is not unique (the same prefix can point to different namespaces and different prefixes can point to the same namespace), you shouldn't use qualified names, but a resolved namespaceUri and the attribute's local name.</p></div>
    pub unsafe fn value_q_latin1_string(
        &self,
        qualified_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamAttributes_value4(
            self as *const crate::QXmlStreamAttributes,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(
                qualified_name,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qxmlstreamnamespacedeclaration.html">QXmlStreamNamespaceDeclaration</a> class represents a namespace declaration.</p>
///
/// C++ class: <span style='color: green;'>```QXmlStreamNamespaceDeclaration```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qxmlstreamnamespacedeclaration.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qxmlstreamnamespacedeclaration.html">QXmlStreamNamespaceDeclaration</a> class represents a namespace declaration.</p>
/// <p>An namespace declaration consists of a <a href="http://doc.qt.io/qt-5/qxmlstreamnamespacedeclaration.html#prefix">prefix</a>() and a <a href="http://doc.qt.io/qt-5/qxmlstreamnamespacedeclaration.html#namespaceUri">namespaceUri</a>().</p></div>
#[repr(C)]
pub struct QXmlStreamNamespaceDeclaration {
    _unused: u8,
}
impl QXmlStreamNamespaceDeclaration {
    /// <p>Assigns <i>other</i> to this namespace declaration.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamNamespaceDeclaration& QXmlStreamNamespaceDeclaration::operator=(const QXmlStreamNamespaceDeclaration& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamnamespacedeclaration.html#operator-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this namespace declaration.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>,
    ) -> ::cpp_utils::MutRef<crate::QXmlStreamNamespaceDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamNamespaceDeclaration_operator_1(self as *mut crate::QXmlStreamNamespaceDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the namespaceUri.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamNamespaceDeclaration::namespaceUri() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamnamespacedeclaration.html#namespaceUri">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the namespaceUri.</p></div>
    pub unsafe fn namespace_uri(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamNamespaceDeclaration_namespaceUri(
            self as *const crate::QXmlStreamNamespaceDeclaration,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates an empty namespace declaration.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamNamespaceDeclaration::QXmlStreamNamespaceDeclaration()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamnamespacedeclaration.html#QXmlStreamNamespaceDeclaration">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates an empty namespace declaration.</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QXmlStreamNamespaceDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamNamespaceDeclaration_QXmlStreamNamespaceDeclaration();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a namespace declaration with <i>prefix</i> and <i>namespaceUri</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamNamespaceDeclaration::QXmlStreamNamespaceDeclaration(const QString& prefix, const QString& namespaceUri)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamnamespacedeclaration.html#QXmlStreamNamespaceDeclaration-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a namespace declaration with <i>prefix</i> and <i>namespaceUri</i>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p></div>
    pub unsafe fn new_2a(
        prefix: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        namespace_uri: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamNamespaceDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamNamespaceDeclaration_QXmlStreamNamespaceDeclaration3(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(prefix).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(namespace_uri).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamNamespaceDeclaration::QXmlStreamNamespaceDeclaration(const QXmlStreamNamespaceDeclaration& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamnamespacedeclaration.html#QXmlStreamNamespaceDeclaration-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a copy of <i>other</i>.</p></div>
    pub unsafe fn new_copy(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamNamespaceDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamNamespaceDeclaration_QXmlStreamNamespaceDeclaration1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the prefix.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamNamespaceDeclaration::prefix() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamnamespacedeclaration.html#prefix">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the prefix.</p></div>
    pub unsafe fn prefix(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamNamespaceDeclaration_prefix(
            self as *const crate::QXmlStreamNamespaceDeclaration,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qxmlstreamnotationdeclaration.html">QXmlStreamNotationDeclaration</a> class represents a DTD notation declaration.</p>
///
/// C++ class: <span style='color: green;'>```QXmlStreamNotationDeclaration```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qxmlstreamnotationdeclaration.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qxmlstreamnotationdeclaration.html">QXmlStreamNotationDeclaration</a> class represents a DTD notation declaration.</p>
/// <p>An notation declaration consists of a <a href="http://doc.qt.io/qt-5/qxmlstreamnotationdeclaration.html#name">name</a>(), a <a href="http://doc.qt.io/qt-5/qxmlstreamnotationdeclaration.html#systemId">systemId</a>(), and a <a href="http://doc.qt.io/qt-5/qxmlstreamnotationdeclaration.html#publicId">publicId</a>().</p></div>
#[repr(C)]
pub struct QXmlStreamNotationDeclaration {
    _unused: u8,
}
impl QXmlStreamNotationDeclaration {
    /// <p>Assigns <i>other</i> to this notation declaration.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamNotationDeclaration& QXmlStreamNotationDeclaration::operator=(const QXmlStreamNotationDeclaration& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamnotationdeclaration.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this notation declaration.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>,
    ) -> ::cpp_utils::MutRef<crate::QXmlStreamNotationDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamNotationDeclaration_operator_(self as *mut crate::QXmlStreamNotationDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the notation name.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamNotationDeclaration::name() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamnotationdeclaration.html#name">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the notation name.</p></div>
    pub unsafe fn name(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamNotationDeclaration_name(
            self as *const crate::QXmlStreamNotationDeclaration,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates an empty notation declaration.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamNotationDeclaration::QXmlStreamNotationDeclaration()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamnotationdeclaration.html#QXmlStreamNotationDeclaration">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates an empty notation declaration.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QXmlStreamNotationDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamNotationDeclaration_QXmlStreamNotationDeclaration();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamNotationDeclaration::QXmlStreamNotationDeclaration(const QXmlStreamNotationDeclaration& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamnotationdeclaration.html#QXmlStreamNotationDeclaration-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a copy of <i>other</i>.</p></div>
    pub unsafe fn new_copy(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamNotationDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamNotationDeclaration_QXmlStreamNotationDeclaration1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the public identifier.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamNotationDeclaration::publicId() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamnotationdeclaration.html#publicId">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the public identifier.</p></div>
    pub unsafe fn public_id(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamNotationDeclaration_publicId(
            self as *const crate::QXmlStreamNotationDeclaration,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the system identifier.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamNotationDeclaration::systemId() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamnotationdeclaration.html#systemId">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the system identifier.</p></div>
    pub unsafe fn system_id(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamNotationDeclaration_systemId(
            self as *const crate::QXmlStreamNotationDeclaration,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qxmlstreamentitydeclaration.html">QXmlStreamEntityDeclaration</a> class represents a DTD entity declaration.</p>
///
/// C++ class: <span style='color: green;'>```QXmlStreamEntityDeclaration```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qxmlstreamentitydeclaration.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qxmlstreamentitydeclaration.html">QXmlStreamEntityDeclaration</a> class represents a DTD entity declaration.</p>
/// <p>An entity declaration consists of a <a href="http://doc.qt.io/qt-5/qxmlstreamentitydeclaration.html#name">name</a>(), a <a href="http://doc.qt.io/qt-5/qxmlstreamentitydeclaration.html#notationName">notationName</a>(), a <a href="http://doc.qt.io/qt-5/qxmlstreamentitydeclaration.html#systemId">systemId</a>(), a <a href="http://doc.qt.io/qt-5/qxmlstreamentitydeclaration.html#publicId">publicId</a>(), and a <a href="http://doc.qt.io/qt-5/qxmlstreamentitydeclaration.html#value">value</a>().</p></div>
#[repr(C)]
pub struct QXmlStreamEntityDeclaration {
    _unused: u8,
}
impl QXmlStreamEntityDeclaration {
    /// <p>Assigns <i>other</i> to this entity declaration.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamEntityDeclaration& QXmlStreamEntityDeclaration::operator=(const QXmlStreamEntityDeclaration& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamentitydeclaration.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this entity declaration.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>,
    ) -> ::cpp_utils::MutRef<crate::QXmlStreamEntityDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamEntityDeclaration_operator_(self as *mut crate::QXmlStreamEntityDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the entity name.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamEntityDeclaration::name() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamentitydeclaration.html#name">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the entity name.</p></div>
    pub unsafe fn name(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamEntityDeclaration_name(
            self as *const crate::QXmlStreamEntityDeclaration,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates an empty entity declaration.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamEntityDeclaration::QXmlStreamEntityDeclaration()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamentitydeclaration.html#QXmlStreamEntityDeclaration">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates an empty entity declaration.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QXmlStreamEntityDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QXmlStreamEntityDeclaration_QXmlStreamEntityDeclaration();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamEntityDeclaration::QXmlStreamEntityDeclaration(const QXmlStreamEntityDeclaration& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamentitydeclaration.html#QXmlStreamEntityDeclaration-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a copy of <i>other</i>.</p></div>
    pub unsafe fn new_copy(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamEntityDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamEntityDeclaration_QXmlStreamEntityDeclaration1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the notation name.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamEntityDeclaration::notationName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamentitydeclaration.html#notationName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the notation name.</p></div>
    pub unsafe fn notation_name(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamEntityDeclaration_notationName(
            self as *const crate::QXmlStreamEntityDeclaration,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the public identifier.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamEntityDeclaration::publicId() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamentitydeclaration.html#publicId">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the public identifier.</p></div>
    pub unsafe fn public_id(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamEntityDeclaration_publicId(
            self as *const crate::QXmlStreamEntityDeclaration,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the system identifier.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamEntityDeclaration::systemId() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamentitydeclaration.html#systemId">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the system identifier.</p></div>
    pub unsafe fn system_id(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamEntityDeclaration_systemId(
            self as *const crate::QXmlStreamEntityDeclaration,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the entity's value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamEntityDeclaration::value() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamentitydeclaration.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the entity's value.</p></div>
    pub unsafe fn value(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamEntityDeclaration_value(
            self as *const crate::QXmlStreamEntityDeclaration,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qxmlstreamentityresolver.html">QXmlStreamEntityResolver</a> class provides an entity resolver for a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html">QXmlStreamReader</a>.</p>
///
/// C++ class: <span style='color: green;'>```QXmlStreamEntityResolver```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qxmlstreamentityresolver.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qxmlstreamentityresolver.html">QXmlStreamEntityResolver</a> class provides an entity resolver for a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html">QXmlStreamReader</a>.</p></div>
#[repr(C)]
pub struct QXmlStreamEntityResolver {
    _unused: u8,
}
impl QXmlStreamEntityResolver {
    /// <p>The <a href="http://doc.qt.io/qt-5/qxmlstreamentityresolver.html">QXmlStreamEntityResolver</a> class provides an entity resolver for a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html">QXmlStreamReader</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamEntityResolver& QXmlStreamEntityResolver::operator=(const QXmlStreamEntityResolver& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamentityresolver.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qxmlstreamentityresolver.html">QXmlStreamEntityResolver</a> class provides an entity resolver for a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html">QXmlStreamReader</a>.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamEntityResolver>>,
    ) -> ::cpp_utils::MutRef<crate::QXmlStreamEntityResolver> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamEntityResolver_operator_(
            self as *mut crate::QXmlStreamEntityResolver,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamEntityResolver>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qxmlstreamentityresolver.html">QXmlStreamEntityResolver</a> class provides an entity resolver for a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html">QXmlStreamReader</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamEntityResolver::QXmlStreamEntityResolver()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamentityresolver.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qxmlstreamentityresolver.html">QXmlStreamEntityResolver</a> class provides an entity resolver for a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html">QXmlStreamReader</a>.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QXmlStreamEntityResolver> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QXmlStreamEntityResolver_QXmlStreamEntityResolver();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qxmlstreamentityresolver.html">QXmlStreamEntityResolver</a> class provides an entity resolver for a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html">QXmlStreamReader</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamEntityResolver::QXmlStreamEntityResolver(const QXmlStreamEntityResolver& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamentityresolver.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qxmlstreamentityresolver.html">QXmlStreamEntityResolver</a> class provides an entity resolver for a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html">QXmlStreamReader</a>.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamEntityResolver>>,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamEntityResolver> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamEntityResolver_QXmlStreamEntityResolver1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamEntityResolver>>::cast_into(other).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual QString QXmlStreamEntityResolver::resolveEntity(const QString& publicId, const QString& systemId)```</span>.
    pub unsafe fn resolve_entity(
        &mut self,
        public_id: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        system_id: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamEntityResolver_resolveEntity(
            self as *mut crate::QXmlStreamEntityResolver,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(public_id)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(system_id)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Resolves the undeclared entity <i>name</i> and returns its replacement text. If the entity is also unknown to the entity resolver, it returns an empty string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QString QXmlStreamEntityResolver::resolveUndeclaredEntity(const QString& name)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamentityresolver.html#resolveUndeclaredEntity">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Resolves the undeclared entity <i>name</i> and returns its replacement text. If the entity is also unknown to the entity resolver, it returns an empty string.</p>
    /// <p>The default implementation always returns an empty string.</p></div>
    pub unsafe fn resolve_undeclared_entity(
        &mut self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QXmlStreamEntityResolver_resolveUndeclaredEntity(
                self as *mut crate::QXmlStreamEntityResolver,
                ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name)
                    .as_raw_ptr(),
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_xml_stream_reader {
    //! C++ type: <span style='color: green;'>```QXmlStreamReader```</span>

    /// <p>This enum specifies the type of token the reader just read.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QXmlStreamReader::TokenType```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum specifies the type of token the reader just read.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct TokenType(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for TokenType {
        fn from(value: ::std::os::raw::c_int) -> Self {
            TokenType(value)
        }
    }

    impl From<TokenType> for ::std::os::raw::c_int {
        fn from(value: TokenType) -> Self {
            value.0
        }
    }

    impl TokenType {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl TokenType {
        /// The reader has not yet read anything. (C++ enum variant: <span style='color: green;'>```NoToken = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoToken: crate::q_xml_stream_reader::TokenType =
            crate::q_xml_stream_reader::TokenType(0);
        /// An error has occurred, reported in <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#error">error</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#errorString">errorString</a>(). (C++ enum variant: <span style='color: green;'>```Invalid = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Invalid: crate::q_xml_stream_reader::TokenType =
            crate::q_xml_stream_reader::TokenType(1);
        /// The reader reports the XML version number in <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#documentVersion">documentVersion</a>(), and the encoding as specified in the XML document in <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#documentEncoding">documentEncoding</a>(). If the document is declared standalone, <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isStandaloneDocument">isStandaloneDocument</a>() returns <code>true</code>; otherwise it returns <code>false</code>. (C++ enum variant: <span style='color: green;'>```StartDocument = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const StartDocument: crate::q_xml_stream_reader::TokenType =
            crate::q_xml_stream_reader::TokenType(2);
        /// The reader reports the end of the document. (C++ enum variant: <span style='color: green;'>```EndDocument = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const EndDocument: crate::q_xml_stream_reader::TokenType =
            crate::q_xml_stream_reader::TokenType(3);
        /// The reader reports the start of an element with <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#namespaceUri">namespaceUri</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#name">name</a>(). Empty elements are also reported as StartElement, followed directly by EndElement. The convenience function <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#readElementText">readElementText</a>() can be called to concatenate all content until the corresponding EndElement. Attributes are reported in <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#attributes">attributes</a>(), namespace declarations in <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#namespaceDeclarations">namespaceDeclarations</a>(). (C++ enum variant: <span style='color: green;'>```StartElement = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const StartElement: crate::q_xml_stream_reader::TokenType =
            crate::q_xml_stream_reader::TokenType(4);
        /// The reader reports the end of an element with <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#namespaceUri">namespaceUri</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#name">name</a>(). (C++ enum variant: <span style='color: green;'>```EndElement = 5```</span>)
        #[allow(non_upper_case_globals)]
        pub const EndElement: crate::q_xml_stream_reader::TokenType =
            crate::q_xml_stream_reader::TokenType(5);
        /// The reader reports characters in <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#text">text</a>(). If the characters are all white-space, <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isWhitespace">isWhitespace</a>() returns <code>true</code>. If the characters stem from a CDATA section, <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isCDATA">isCDATA</a>() returns <code>true</code>. (C++ enum variant: <span style='color: green;'>```Characters = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const Characters: crate::q_xml_stream_reader::TokenType =
            crate::q_xml_stream_reader::TokenType(6);
        /// The reader reports a comment in <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#text">text</a>(). (C++ enum variant: <span style='color: green;'>```Comment = 7```</span>)
        #[allow(non_upper_case_globals)]
        pub const Comment: crate::q_xml_stream_reader::TokenType =
            crate::q_xml_stream_reader::TokenType(7);
        /// The reader reports a DTD in <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#text">text</a>(), notation declarations in <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#notationDeclarations">notationDeclarations</a>(), and entity declarations in <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#entityDeclarations">entityDeclarations</a>(). Details of the DTD declaration are reported in in <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#dtdName">dtdName</a>(), <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#dtdPublicId">dtdPublicId</a>(), and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#dtdSystemId">dtdSystemId</a>(). (C++ enum variant: <span style='color: green;'>```DTD = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const DTD: crate::q_xml_stream_reader::TokenType =
            crate::q_xml_stream_reader::TokenType(8);
        /// The reader reports an entity reference that could not be resolved. The name of the reference is reported in <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#name">name</a>(), the replacement text in <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#text">text</a>(). (C++ enum variant: <span style='color: green;'>```EntityReference = 9```</span>)
        #[allow(non_upper_case_globals)]
        pub const EntityReference: crate::q_xml_stream_reader::TokenType =
            crate::q_xml_stream_reader::TokenType(9);
        /// The reader reports a processing instruction in <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#processingInstructionTarget">processingInstructionTarget</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#processingInstructionData">processingInstructionData</a>(). (C++ enum variant: <span style='color: green;'>```ProcessingInstruction = 10```</span>)
        #[allow(non_upper_case_globals)]
        pub const ProcessingInstruction: crate::q_xml_stream_reader::TokenType =
            crate::q_xml_stream_reader::TokenType(10);
    }

    /// <p>This enum specifies the different behaviours of <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#readElementText">readElementText</a>().</p>
    ///
    /// C++ enum: <span style='color: green;'>```QXmlStreamReader::ReadElementTextBehaviour```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#ReadElementTextBehaviour-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum specifies the different behaviours of <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#readElementText">readElementText</a>().</p>
    ///
    /// <p>This enum was introduced or modified in  Qt 4.6.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct ReadElementTextBehaviour(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for ReadElementTextBehaviour {
        fn from(value: ::std::os::raw::c_int) -> Self {
            ReadElementTextBehaviour(value)
        }
    }

    impl From<ReadElementTextBehaviour> for ::std::os::raw::c_int {
        fn from(value: ReadElementTextBehaviour) -> Self {
            value.0
        }
    }

    impl ReadElementTextBehaviour {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl ReadElementTextBehaviour {
        /// Raise an <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#Error-enum">UnexpectedElementError</a> and return what was read so far when a child element is encountered. (C++ enum variant: <span style='color: green;'>```ErrorOnUnexpectedElement = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const ErrorOnUnexpectedElement: crate::q_xml_stream_reader::ReadElementTextBehaviour =
            crate::q_xml_stream_reader::ReadElementTextBehaviour(0);
        /// Recursively include the text from child elements. (C++ enum variant: <span style='color: green;'>```IncludeChildElements = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const IncludeChildElements: crate::q_xml_stream_reader::ReadElementTextBehaviour =
            crate::q_xml_stream_reader::ReadElementTextBehaviour(1);
        /// Skip child elements. (C++ enum variant: <span style='color: green;'>```SkipChildElements = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const SkipChildElements: crate::q_xml_stream_reader::ReadElementTextBehaviour =
            crate::q_xml_stream_reader::ReadElementTextBehaviour(2);
    }

    /// <p>This enum specifies different error cases</p>
    ///
    /// C++ enum: <span style='color: green;'>```QXmlStreamReader::Error```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#Error-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum specifies different error cases</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Error(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Error {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Error(value)
        }
    }

    impl From<Error> for ::std::os::raw::c_int {
        fn from(value: Error) -> Self {
            value.0
        }
    }

    impl Error {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Error {
        /// No error has occurred. (C++ enum variant: <span style='color: green;'>```NoError = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoError: crate::q_xml_stream_reader::Error = crate::q_xml_stream_reader::Error(0);
        /// The parser encountered an element that was different to those it expected. (C++ enum variant: <span style='color: green;'>```UnexpectedElementError = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const UnexpectedElementError: crate::q_xml_stream_reader::Error =
            crate::q_xml_stream_reader::Error(1);
        /// A custom error has been raised with <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#raiseError">raiseError</a>() (C++ enum variant: <span style='color: green;'>```CustomError = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const CustomError: crate::q_xml_stream_reader::Error =
            crate::q_xml_stream_reader::Error(2);
        /// The parser internally raised an error due to the read XML not being well-formed. (C++ enum variant: <span style='color: green;'>```NotWellFormedError = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const NotWellFormedError: crate::q_xml_stream_reader::Error =
            crate::q_xml_stream_reader::Error(3);
        /// The input stream ended before a well-formed XML document was parsed. Recovery from this error is possible if more XML arrives in the stream, either by calling <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#addData">addData</a>() or by waiting for it to arrive on the <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#device">device</a>(). (C++ enum variant: <span style='color: green;'>```PrematureEndOfDocumentError = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const PrematureEndOfDocumentError: crate::q_xml_stream_reader::Error =
            crate::q_xml_stream_reader::Error(4);
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html">QXmlStreamReader</a> class provides a fast parser for reading well-formed XML via a simple streaming API.</p>
///
/// C++ class: <span style='color: green;'>```QXmlStreamReader```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html">QXmlStreamReader</a> class provides a fast parser for reading well-formed XML via a simple streaming API.</p>
/// <p><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html">QXmlStreamReader</a> is a faster and more convenient replacement for Qt's own SAX parser (see <a href="http://doc.qt.io/qt-5/qxmlsimplereader.html">QXmlSimpleReader</a>). In some cases it might also be a faster and more convenient alternative for use in applications that would otherwise use a DOM tree (see <a href="http://doc.qt.io/qt-5/qdomdocument.html">QDomDocument</a>). <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html">QXmlStreamReader</a> reads data either from a <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> (see <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#setDevice">setDevice</a>()), or from a raw <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> (see <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#addData">addData</a>()).</p>
/// <p>Qt provides <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a> for writing XML.</p>
/// <p>The basic concept of a stream reader is to report an XML document as a stream of tokens, similar to SAX. The main difference between <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html">QXmlStreamReader</a> and SAX is <i>how</i> these XML tokens are reported. With SAX, the application must provide handlers (callback functions) that receive so-called XML <i>events</i> from the parser at the parser's convenience. With <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html">QXmlStreamReader</a>, the application code itself drives the loop and pulls <i>tokens</i> from the reader, one after another, as it needs them. This is done by calling <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#readNext">readNext</a>(), where the reader reads from the input stream until it completes the next token, at which point it returns the <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>(). A set of convenient functions including <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isStartElement">isStartElement</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#text">text</a>() can then be used to examine the token to obtain information about what has been read. The big advantage of this <i>pulling</i> approach is the possibility to build recursive descent parsers with it, meaning you can split your XML parsing code easily into different methods or classes. This makes it easy to keep track of the application's own state when parsing XML.</p>
/// <p>A typical loop with <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html">QXmlStreamReader</a> looks like this:</p>
/// <pre class="cpp">
///
/// &#32;   <span class="type"><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#QXmlStreamReader">QXmlStreamReader</a></span> xml;
/// &#32;   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
/// &#32;   <span class="keyword">while</span> (<span class="operator">!</span>xml<span class="operator">.</span>atEnd()) {
/// &#32;   &#32;     xml<span class="operator">.</span>readNext();
/// &#32;   &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span> <span class="comment">// do processing</span>
/// &#32;   }
/// &#32;   <span class="keyword">if</span> (xml<span class="operator">.</span>hasError()) {
/// &#32;   &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span> <span class="comment">// do error handling</span>
/// &#32;   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html">QXmlStreamReader</a> is a well-formed XML 1.0 parser that does <i>not</i> include external parsed entities. As long as no error occurs, the application code can thus be assured that the data provided by the stream reader satisfies the W3C's criteria for well-formed XML. For example, you can be certain that all tags are indeed nested and closed properly, that references to internal entities have been replaced with the correct replacement text, and that attributes have been normalized or added according to the internal subset of the DTD.</p>
/// <p>If an error occurs while parsing, <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#atEnd">atEnd</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#hasError">hasError</a>() return true, and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#error">error</a>() returns the error that occurred. The functions <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#errorString">errorString</a>(), <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#lineNumber">lineNumber</a>(), <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#columnNumber">columnNumber</a>(), and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#characterOffset">characterOffset</a>() are for constructing an appropriate error or warning message. To simplify application code, <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html">QXmlStreamReader</a> contains a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#raiseError">raiseError</a>() mechanism that lets you raise custom errors that trigger the same error handling described.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qtxml-streambookmarks-example.html">QXmlStream Bookmarks Example</a> illustrates how to use the recursive descent technique to read an XML bookmark file (XBEL) with a stream reader.</p>
/// <a name="namespaces"></a></div>
#[repr(C)]
pub struct QXmlStreamReader {
    _unused: u8,
}
impl QXmlStreamReader {
    /// <p>Adds more <i>data</i> for the reader to read. This function does nothing if the reader has a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#device">device</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamReader::addData(const QByteArray& data)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#addData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds more <i>data</i> for the reader to read. This function does nothing if the reader has a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#device">device</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#readNext">readNext</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#clear">clear</a>().</p></div>
    pub unsafe fn add_data_q_byte_array(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_addData(
            self as *mut crate::QXmlStreamReader,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(data)
                .as_raw_ptr(),
        )
    }

    /// <p>Adds more <i>data</i> for the reader to read. This function does nothing if the reader has a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#device">device</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamReader::addData(const QString& data)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#addData-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds more <i>data</i> for the reader to read. This function does nothing if the reader has a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#device">device</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#readNext">readNext</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#clear">clear</a>().</p></div>
    pub unsafe fn add_data_q_string(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_addData1(
            self as *mut crate::QXmlStreamReader,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(data).as_raw_ptr(),
        )
    }

    /// <p>Adds more <i>data</i> for the reader to read. This function does nothing if the reader has a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#device">device</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamReader::addData(const char* data)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#addData-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds more <i>data</i> for the reader to read. This function does nothing if the reader has a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#device">device</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#readNext">readNext</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#clear">clear</a>().</p></div>
    pub unsafe fn add_data_char(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_addData2(
            self as *mut crate::QXmlStreamReader,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(data)
                .as_raw_ptr(),
        )
    }

    /// <p>Adds an <i>extraNamespaceDeclaration</i>. The declaration will be valid for children of the current element, or - should the function be called before any elements are read - for the entire XML document.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamReader::addExtraNamespaceDeclaration(const QXmlStreamNamespaceDeclaration& extraNamespaceDeclaraction)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#addExtraNamespaceDeclaration">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds an <i>extraNamespaceDeclaration</i>. The declaration will be valid for children of the current element, or - should the function be called before any elements are read - for the entire XML document.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#namespaceDeclarations">namespaceDeclarations</a>(), <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#addExtraNamespaceDeclarations">addExtraNamespaceDeclarations</a>(), and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#namespaceProcessing-prop">setNamespaceProcessing</a>().</p></div>
    pub unsafe fn add_extra_namespace_declaration(
        &mut self,
        extra_namespace_declaraction: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>,
        >,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_addExtraNamespaceDeclaration(self as *mut crate::QXmlStreamReader, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>::cast_into(extra_namespace_declaraction).as_raw_ptr())
    }

    /// <p>Adds a vector of declarations specified by <i>extraNamespaceDeclarations</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamReader::addExtraNamespaceDeclarations(const QVector<QXmlStreamNamespaceDeclaration>& extraNamespaceDeclaractions)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#addExtraNamespaceDeclarations">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds a vector of declarations specified by <i>extraNamespaceDeclarations</i>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#namespaceDeclarations">namespaceDeclarations</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#addExtraNamespaceDeclaration">addExtraNamespaceDeclaration</a>().</p></div>
    pub unsafe fn add_extra_namespace_declarations(
        &mut self,
        extra_namespace_declaractions: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::QVectorOfQXmlStreamNamespaceDeclaration>,
        >,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_addExtraNamespaceDeclarations(self as *mut crate::QXmlStreamReader, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQXmlStreamNamespaceDeclaration>>::cast_into(extra_namespace_declaractions).as_raw_ptr())
    }

    /// <p>Returns <code>true</code> if the reader has read until the end of the XML document, or if an <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#error">error</a>() has occurred and reading has been aborted. Otherwise, it returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QXmlStreamReader::atEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#atEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the reader has read until the end of the XML document, or if an <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#error">error</a>() has occurred and reading has been aborted. Otherwise, it returns <code>false</code>.</p>
    /// <p>When atEnd() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#hasError">hasError</a>() return true and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#error">error</a>() returns <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#Error-enum">PrematureEndOfDocumentError</a>, it means the XML has been well-formed so far, but a complete XML document has not been parsed. The next chunk of XML can be added with <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#addData">addData</a>(), if the XML is being read from a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>, or by waiting for more data to arrive if the XML is being read from a <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>. Either way, atEnd() will return false once more data is available.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#hasError">hasError</a>(), <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#error">error</a>(), <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#device">device</a>(), and <a href="http://doc.qt.io/qt-5/qiodevice.html#atEnd">QIODevice::atEnd</a>().</p></div>
    pub unsafe fn at_end(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_atEnd(self as *const crate::QXmlStreamReader)
    }

    /// <p>Returns the attributes of a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">StartElement</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamAttributes QXmlStreamReader::attributes() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#attributes">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the attributes of a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">StartElement</a>.</p></div>
    pub unsafe fn attributes(&self) -> ::cpp_utils::CppBox<crate::QXmlStreamAttributes> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_attributes(
            self as *const crate::QXmlStreamReader,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the current character offset, starting with 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QXmlStreamReader::characterOffset() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#characterOffset">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current character offset, starting with 0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#lineNumber">lineNumber</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#columnNumber">columnNumber</a>().</p></div>
    pub unsafe fn character_offset(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_characterOffset(
            self as *const crate::QXmlStreamReader,
        )
    }

    /// <p>Removes any <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#device">device</a>() or data from the reader and resets its internal state to the initial state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamReader::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes any <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#device">device</a>() or data from the reader and resets its internal state to the initial state.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#addData">addData</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_clear(self as *mut crate::QXmlStreamReader)
    }

    /// <p>Returns the current column number, starting with 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QXmlStreamReader::columnNumber() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#columnNumber">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current column number, starting with 0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#lineNumber">lineNumber</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#characterOffset">characterOffset</a>().</p></div>
    pub unsafe fn column_number(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_columnNumber(
            self as *const crate::QXmlStreamReader,
        )
    }

    /// <p>Returns the current device associated with the <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html">QXmlStreamReader</a>, or 0 if no device has been assigned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QIODevice* QXmlStreamReader::device() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#device">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current device associated with the <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html">QXmlStreamReader</a>, or 0 if no device has been assigned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#setDevice">setDevice</a>().</p></div>
    pub unsafe fn device(&self) -> ::cpp_utils::MutPtr<crate::QIODevice> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_device(
            self as *const crate::QXmlStreamReader,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>If the <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() is <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">StartDocument</a>, this function returns the encoding string as specified in the XML declaration. Otherwise an empty string is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamReader::documentEncoding() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#documentEncoding">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If the <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() is <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">StartDocument</a>, this function returns the encoding string as specified in the XML declaration. Otherwise an empty string is returned.</p>
    /// <p>This function was introduced in  Qt 4.4.</p></div>
    pub unsafe fn document_encoding(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_documentEncoding(
            self as *const crate::QXmlStreamReader,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>If the <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() is <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">StartDocument</a>, this function returns the version string as specified in the XML declaration. Otherwise an empty string is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamReader::documentVersion() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#documentVersion">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If the <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() is <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">StartDocument</a>, this function returns the version string as specified in the XML declaration. Otherwise an empty string is returned.</p>
    /// <p>This function was introduced in  Qt 4.4.</p></div>
    pub unsafe fn document_version(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_documentVersion(
            self as *const crate::QXmlStreamReader,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>If the <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() is <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">DTD</a>, this function returns the DTD's name. Otherwise an empty string is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamReader::dtdName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#dtdName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If the <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() is <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">DTD</a>, this function returns the DTD's name. Otherwise an empty string is returned.</p>
    /// <p>This function was introduced in  Qt 4.4.</p></div>
    pub unsafe fn dtd_name(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_dtdName(
            self as *const crate::QXmlStreamReader,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>If the <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() is <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">DTD</a>, this function returns the DTD's public identifier. Otherwise an empty string is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamReader::dtdPublicId() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#dtdPublicId">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If the <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() is <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">DTD</a>, this function returns the DTD's public identifier. Otherwise an empty string is returned.</p>
    /// <p>This function was introduced in  Qt 4.4.</p></div>
    pub unsafe fn dtd_public_id(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_dtdPublicId(
            self as *const crate::QXmlStreamReader,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>If the <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() is <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">DTD</a>, this function returns the DTD's system identifier. Otherwise an empty string is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamReader::dtdSystemId() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#dtdSystemId">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If the <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() is <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">DTD</a>, this function returns the DTD's system identifier. Otherwise an empty string is returned.</p>
    /// <p>This function was introduced in  Qt 4.4.</p></div>
    pub unsafe fn dtd_system_id(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_dtdSystemId(
            self as *const crate::QXmlStreamReader,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>If the <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() is <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">DTD</a>, this function returns the DTD's unparsed (external) entity declarations. Otherwise an empty vector is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamEntityDeclaration> QXmlStreamReader::entityDeclarations() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#entityDeclarations">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If the <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() is <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">DTD</a>, this function returns the DTD's unparsed (external) entity declarations. Otherwise an empty vector is returned.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qxmlstreamentitydeclaration.html#QXmlStreamEntityDeclarations-typedef">QXmlStreamEntityDeclarations</a> class is defined to be a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> of <a href="http://doc.qt.io/qt-5/qxmlstreamentitydeclaration.html">QXmlStreamEntityDeclaration</a>.</p></div>
    pub unsafe fn entity_declarations(
        &self,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamEntityDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_entityDeclarations(
            self as *const crate::QXmlStreamReader,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the entity resolver, or 0 if there is no entity resolver.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamEntityResolver* QXmlStreamReader::entityResolver() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#entityResolver">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the entity resolver, or 0 if there is no entity resolver.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#setEntityResolver">setEntityResolver</a>().</p></div>
    pub unsafe fn entity_resolver(&self) -> ::cpp_utils::MutPtr<crate::QXmlStreamEntityResolver> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_entityResolver(
            self as *const crate::QXmlStreamReader,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns the type of the current error, or <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#Error-enum">NoError</a> if no error occurred.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamReader::Error QXmlStreamReader::error() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#error">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the type of the current error, or <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#Error-enum">NoError</a> if no error occurred.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#errorString">errorString</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#raiseError">raiseError</a>().</p></div>
    pub unsafe fn error(&self) -> crate::q_xml_stream_reader::Error {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_error(self as *const crate::QXmlStreamReader)
    }

    /// <p>Returns the error message that was set with <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#raiseError">raiseError</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QXmlStreamReader::errorString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#errorString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the error message that was set with <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#raiseError">raiseError</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#error">error</a>(), <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#lineNumber">lineNumber</a>(), <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#columnNumber">columnNumber</a>(), and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#characterOffset">characterOffset</a>().</p></div>
    pub unsafe fn error_string(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_errorString(
            self as *const crate::QXmlStreamReader,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if an error has occurred, otherwise <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QXmlStreamReader::hasError() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#hasError">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if an error has occurred, otherwise <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#errorString">errorString</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#error">error</a>().</p></div>
    pub unsafe fn has_error(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_hasError(
            self as *const crate::QXmlStreamReader,
        )
    }

    /// <p>Returns <code>true</code> if the reader reports characters that stem from a CDATA section; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QXmlStreamReader::isCDATA() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isCDATA">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the reader reports characters that stem from a CDATA section; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isCharacters">isCharacters</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#text">text</a>().</p></div>
    pub unsafe fn is_c_d_a_t_a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_isCDATA(
            self as *const crate::QXmlStreamReader,
        )
    }

    /// <p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() equals <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">Characters</a>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QXmlStreamReader::isCharacters() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isCharacters">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() equals <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">Characters</a>; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isWhitespace">isWhitespace</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isCDATA">isCDATA</a>().</p></div>
    pub unsafe fn is_characters(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_isCharacters(
            self as *const crate::QXmlStreamReader,
        )
    }

    /// <p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() equals <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">Comment</a>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QXmlStreamReader::isComment() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isComment">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() equals <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">Comment</a>; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn is_comment(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_isComment(
            self as *const crate::QXmlStreamReader,
        )
    }

    /// <p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() equals <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">DTD</a>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QXmlStreamReader::isDTD() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isDTD">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() equals <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">DTD</a>; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn is_d_t_d(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_isDTD(self as *const crate::QXmlStreamReader)
    }

    /// <p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() equals <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EndDocument</a>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QXmlStreamReader::isEndDocument() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isEndDocument">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() equals <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EndDocument</a>; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn is_end_document(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_isEndDocument(
            self as *const crate::QXmlStreamReader,
        )
    }

    /// <p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() equals <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EndElement</a>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QXmlStreamReader::isEndElement() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isEndElement">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() equals <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EndElement</a>; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn is_end_element(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_isEndElement(
            self as *const crate::QXmlStreamReader,
        )
    }

    /// <p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() equals <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EntityReference</a>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QXmlStreamReader::isEntityReference() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isEntityReference">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() equals <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EntityReference</a>; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn is_entity_reference(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_isEntityReference(
            self as *const crate::QXmlStreamReader,
        )
    }

    /// <p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() equals <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">ProcessingInstruction</a>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QXmlStreamReader::isProcessingInstruction() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isProcessingInstruction">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() equals <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">ProcessingInstruction</a>; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn is_processing_instruction(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_isProcessingInstruction(
            self as *const crate::QXmlStreamReader,
        )
    }

    /// <p>Returns <code>true</code> if this document has been declared standalone in the XML declaration; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QXmlStreamReader::isStandaloneDocument() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isStandaloneDocument">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this document has been declared standalone in the XML declaration; otherwise returns <code>false</code>.</p>
    /// <p>If no XML declaration has been parsed, this function returns <code>false</code>.</p></div>
    pub unsafe fn is_standalone_document(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_isStandaloneDocument(
            self as *const crate::QXmlStreamReader,
        )
    }

    /// <p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() equals <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">StartDocument</a>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QXmlStreamReader::isStartDocument() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isStartDocument">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() equals <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">StartDocument</a>; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn is_start_document(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_isStartDocument(
            self as *const crate::QXmlStreamReader,
        )
    }

    /// <p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() equals <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">StartElement</a>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QXmlStreamReader::isStartElement() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isStartElement">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() equals <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">StartElement</a>; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn is_start_element(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_isStartElement(
            self as *const crate::QXmlStreamReader,
        )
    }

    /// <p>Returns <code>true</code> if the reader reports characters that only consist of white-space; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QXmlStreamReader::isWhitespace() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isWhitespace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the reader reports characters that only consist of white-space; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isCharacters">isCharacters</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#text">text</a>().</p></div>
    pub unsafe fn is_whitespace(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_isWhitespace(
            self as *const crate::QXmlStreamReader,
        )
    }

    /// <p>Returns the current line number, starting with 1.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QXmlStreamReader::lineNumber() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#lineNumber">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current line number, starting with 1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#columnNumber">columnNumber</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#characterOffset">characterOffset</a>().</p></div>
    pub unsafe fn line_number(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_lineNumber(
            self as *const crate::QXmlStreamReader,
        )
    }

    /// <p>Returns the local name of a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">StartElement</a>, <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EndElement</a>, or an <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EntityReference</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamReader::name() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#name">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the local name of a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">StartElement</a>, <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EndElement</a>, or an <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EntityReference</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#namespaceUri">namespaceUri</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#qualifiedName">qualifiedName</a>().</p></div>
    pub unsafe fn name(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_name(
            self as *const crate::QXmlStreamReader,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>If the <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() is <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">StartElement</a>, this function returns the element's namespace declarations. Otherwise an empty vector is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamNamespaceDeclaration> QXmlStreamReader::namespaceDeclarations() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#namespaceDeclarations">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If the <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() is <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">StartElement</a>, this function returns the element's namespace declarations. Otherwise an empty vector is returned.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qxmlstreamnamespacedeclaration.html#QXmlStreamNamespaceDeclarations-typedef">QXmlStreamNamespaceDeclarations</a> class is defined to be a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> of <a href="http://doc.qt.io/qt-5/qxmlstreamnamespacedeclaration.html">QXmlStreamNamespaceDeclaration</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#addExtraNamespaceDeclaration">addExtraNamespaceDeclaration</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#addExtraNamespaceDeclarations">addExtraNamespaceDeclarations</a>().</p></div>
    pub unsafe fn namespace_declarations(
        &self,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamNamespaceDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_namespaceDeclarations(
            self as *const crate::QXmlStreamReader,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The namespace-processing flag of the stream reader</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QXmlStreamReader::namespaceProcessing() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#namespaceProcessing-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The namespace-processing flag of the stream reader</p>
    /// <p>This property controls whether or not the stream reader processes namespaces. If enabled, the reader processes namespaces, otherwise it does not.</p>
    /// <p>By default, namespace-processing is enabled.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> bool </td><td class="memItemRight bottomAlign"><span class="name"><b>namespaceProcessing</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setNamespaceProcessing</b></span>(<i>bool</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn namespace_processing(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_namespaceProcessing(
            self as *const crate::QXmlStreamReader,
        )
    }

    /// <p>Returns the namespaceUri of a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">StartElement</a> or <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EndElement</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamReader::namespaceUri() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#namespaceUri">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the namespaceUri of a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">StartElement</a> or <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EndElement</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#name">name</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#qualifiedName">qualifiedName</a>().</p></div>
    pub unsafe fn namespace_uri(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_namespaceUri(
            self as *const crate::QXmlStreamReader,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a stream reader.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamReader::QXmlStreamReader()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#QXmlStreamReader">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a stream reader.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#setDevice">setDevice</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#addData">addData</a>().</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QXmlStreamReader> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_QXmlStreamReader();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a new stream reader that reads from <i>device</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamReader::QXmlStreamReader(QIODevice* device)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#QXmlStreamReader-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a new stream reader that reads from <i>device</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#setDevice">setDevice</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#clear">clear</a>().</p></div>
    pub unsafe fn from_q_io_device(
        device: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QIODevice>>,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamReader> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_QXmlStreamReader1(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QIODevice>>::cast_into(device)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a new stream reader that reads from <i>data</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamReader::QXmlStreamReader(const QByteArray& data)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#QXmlStreamReader-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a new stream reader that reads from <i>data</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#addData">addData</a>(), <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#clear">clear</a>(), and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#setDevice">setDevice</a>().</p></div>
    pub unsafe fn from_q_byte_array(
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamReader> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_QXmlStreamReader2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(data)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a new stream reader that reads from <i>data</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamReader::QXmlStreamReader(const QString& data)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#QXmlStreamReader-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a new stream reader that reads from <i>data</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#addData">addData</a>(), <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#clear">clear</a>(), and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#setDevice">setDevice</a>().</p></div>
    pub unsafe fn from_q_string(
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamReader> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_QXmlStreamReader3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(data).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a new stream reader that reads from <i>data</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamReader::QXmlStreamReader(const char* data)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#QXmlStreamReader-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a new stream reader that reads from <i>data</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#addData">addData</a>(), <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#clear">clear</a>(), and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#setDevice">setDevice</a>().</p></div>
    pub unsafe fn from_char(
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamReader> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_QXmlStreamReader4(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(data)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>If the <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() is <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">DTD</a>, this function returns the DTD's notation declarations. Otherwise an empty vector is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamNotationDeclaration> QXmlStreamReader::notationDeclarations() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#notationDeclarations">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If the <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() is <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">DTD</a>, this function returns the DTD's notation declarations. Otherwise an empty vector is returned.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qxmlstreamnotationdeclaration.html#QXmlStreamNotationDeclarations-typedef">QXmlStreamNotationDeclarations</a> class is defined to be a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> of <a href="http://doc.qt.io/qt-5/qxmlstreamnotationdeclaration.html">QXmlStreamNotationDeclaration</a>.</p></div>
    pub unsafe fn notation_declarations(
        &self,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamNotationDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_notationDeclarations(
            self as *const crate::QXmlStreamReader,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the prefix of a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">StartElement</a> or <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EndElement</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamReader::prefix() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#prefix">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the prefix of a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">StartElement</a> or <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EndElement</a>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#name">name</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#qualifiedName">qualifiedName</a>().</p></div>
    pub unsafe fn prefix(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_prefix(
            self as *const crate::QXmlStreamReader,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the data of a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">ProcessingInstruction</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamReader::processingInstructionData() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#processingInstructionData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the data of a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">ProcessingInstruction</a>.</p></div>
    pub unsafe fn processing_instruction_data(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_processingInstructionData(
            self as *const crate::QXmlStreamReader,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the target of a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">ProcessingInstruction</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamReader::processingInstructionTarget() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#processingInstructionTarget">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the target of a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">ProcessingInstruction</a>.</p></div>
    pub unsafe fn processing_instruction_target(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_processingInstructionTarget(
            self as *const crate::QXmlStreamReader,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the qualified name of a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">StartElement</a> or <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EndElement</a>;</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamReader::qualifiedName() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#qualifiedName">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the qualified name of a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">StartElement</a> or <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EndElement</a>;</p>
    /// <p>A qualified name is the raw name of an element in the XML data. It consists of the namespace prefix, followed by colon, followed by the element's local name. Since the namespace prefix is not unique (the same prefix can point to different namespaces and different prefixes can point to the same namespace), you shouldn't use qualifiedName(), but the resolved <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#namespaceUri">namespaceUri</a>() and the attribute's local <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#name">name</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#name">name</a>(), <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#prefix">prefix</a>(), and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#namespaceUri">namespaceUri</a>().</p></div>
    pub unsafe fn qualified_name(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_qualifiedName(
            self as *const crate::QXmlStreamReader,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Raises a custom error with an optional error <i>message</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamReader::raiseError(const QString& message = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#raiseError">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Raises a custom error with an optional error <i>message</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#error">error</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#errorString">errorString</a>().</p></div>
    pub unsafe fn raise_error_1a(
        &mut self,
        message: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_raiseError(
            self as *mut crate::QXmlStreamReader,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(message)
                .as_raw_ptr(),
        )
    }

    /// <p>Raises a custom error with an optional error <i>message</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamReader::raiseError()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#raiseError">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Raises a custom error with an optional error <i>message</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#error">error</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#errorString">errorString</a>().</p></div>
    pub unsafe fn raise_error_0a(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_raiseError1(
            self as *mut crate::QXmlStreamReader,
        )
    }

    /// <p>Convenience function to be called in case a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">StartElement</a> was read. Reads until the corresponding <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EndElement</a> and returns all text in-between. In case of no error, the current token (see <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>()) after having called this function is <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EndElement</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QXmlStreamReader::readElementText(QXmlStreamReader::ReadElementTextBehaviour behaviour = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#readElementText">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Convenience function to be called in case a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">StartElement</a> was read. Reads until the corresponding <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EndElement</a> and returns all text in-between. In case of no error, the current token (see <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>()) after having called this function is <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EndElement</a>.</p>
    /// <p>The function concatenates <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#text">text</a>() when it reads either <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">Characters</a> or <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EntityReference</a> tokens, but skips <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">ProcessingInstruction</a> and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">Comment</a>. If the current token is not <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">StartElement</a>, an empty string is returned.</p>
    /// <p>The <i>behaviour</i> defines what happens in case anything else is read before reaching <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EndElement</a>. The function can include the text from child elements (useful for example for HTML), ignore child elements, or raise an <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#Error-enum">UnexpectedElementError</a> and return what was read so far (default).</p>
    /// <p>This function was introduced in  Qt 4.6.</p></div>
    pub unsafe fn read_element_text_1a(
        &mut self,
        behaviour: crate::q_xml_stream_reader::ReadElementTextBehaviour,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_readElementText(
            self as *mut crate::QXmlStreamReader,
            behaviour,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Convenience function to be called in case a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">StartElement</a> was read. Reads until the corresponding <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EndElement</a> and returns all text in-between. In case of no error, the current token (see <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>()) after having called this function is <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EndElement</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QXmlStreamReader::readElementText()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#readElementText">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Convenience function to be called in case a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">StartElement</a> was read. Reads until the corresponding <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EndElement</a> and returns all text in-between. In case of no error, the current token (see <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>()) after having called this function is <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EndElement</a>.</p>
    /// <p>The function concatenates <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#text">text</a>() when it reads either <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">Characters</a> or <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EntityReference</a> tokens, but skips <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">ProcessingInstruction</a> and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">Comment</a>. If the current token is not <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">StartElement</a>, an empty string is returned.</p>
    /// <p>The <i>behaviour</i> defines what happens in case anything else is read before reaching <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EndElement</a>. The function can include the text from child elements (useful for example for HTML), ignore child elements, or raise an <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#Error-enum">UnexpectedElementError</a> and return what was read so far (default).</p>
    /// <p>This function was introduced in  Qt 4.6.</p></div>
    pub unsafe fn read_element_text_0a(&mut self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_readElementText1(
            self as *mut crate::QXmlStreamReader,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reads the next token and returns its type.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamReader::TokenType QXmlStreamReader::readNext()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#readNext">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads the next token and returns its type.</p>
    /// <p>With one exception, once an <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#error">error</a>() is reported by readNext(), further reading of the XML stream is not possible. Then <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#atEnd">atEnd</a>() returns <code>true</code>, <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#hasError">hasError</a>() returns <code>true</code>, and this function returns <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">QXmlStreamReader::Invalid</a>.</p>
    /// <p>The exception is when <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#error">error</a>() returns <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#Error-enum">PrematureEndOfDocumentError</a>. This error is reported when the end of an otherwise well-formed chunk of XML is reached, but the chunk doesn't represent a complete XML document. In that case, parsing <i>can</i> be resumed by calling <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#addData">addData</a>() to add the next chunk of XML, when the stream is being read from a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>, or by waiting for more data to arrive when the stream is being read from a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#device">device</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenString">tokenString</a>().</p></div>
    pub unsafe fn read_next(&mut self) -> crate::q_xml_stream_reader::TokenType {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_readNext(
            self as *mut crate::QXmlStreamReader,
        )
    }

    /// <p>Reads until the next start element within the current element. Returns <code>true</code> when a start element was reached. When the end element was reached, or when an error occurred, false is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QXmlStreamReader::readNextStartElement()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#readNextStartElement">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads until the next start element within the current element. Returns <code>true</code> when a start element was reached. When the end element was reached, or when an error occurred, false is returned.</p>
    /// <p>The current element is the element matching the most recently parsed start element of which a matching end element has not yet been reached. When the parser has reached the end element, the current element becomes the parent element.</p>
    /// <p>This is a convenience function for when you're only concerned with parsing XML elements. The <a href="http://doc.qt.io/qt-5/qtxml-streambookmarks-example.html">QXmlStream Bookmarks Example</a> makes extensive use of this function.</p>
    /// <p>This function was introduced in  Qt 4.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#readNext">readNext</a>().</p></div>
    pub unsafe fn read_next_start_element(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_readNextStartElement(
            self as *mut crate::QXmlStreamReader,
        )
    }

    /// <p>Sets the current device to <i>device</i>. Setting the device resets the stream to its initial state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamReader::setDevice(QIODevice* device)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#setDevice">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the current device to <i>device</i>. Setting the device resets the stream to its initial state.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#device">device</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#clear">clear</a>().</p></div>
    pub unsafe fn set_device(
        &mut self,
        device: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QIODevice>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_setDevice(
            self as *mut crate::QXmlStreamReader,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QIODevice>>::cast_into(device)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Makes <i>resolver</i> the new <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#entityResolver">entityResolver</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamReader::setEntityResolver(QXmlStreamEntityResolver* resolver)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#setEntityResolver">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes <i>resolver</i> the new <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#entityResolver">entityResolver</a>().</p>
    /// <p>The stream reader does <i>not</i> take ownership of the resolver. It's the callers responsibility to ensure that the resolver is valid during the entire life-time of the stream reader object, or until another resolver or 0 is set.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#entityResolver">entityResolver</a>().</p></div>
    pub unsafe fn set_entity_resolver(
        &mut self,
        resolver: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QXmlStreamEntityResolver>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_setEntityResolver(self as *mut crate::QXmlStreamReader, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QXmlStreamEntityResolver>>::cast_into(resolver).as_mut_raw_ptr())
    }

    /// <p>The namespace-processing flag of the stream reader</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamReader::setNamespaceProcessing(bool arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#namespaceProcessing-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The namespace-processing flag of the stream reader</p>
    /// <p>This property controls whether or not the stream reader processes namespaces. If enabled, the reader processes namespaces, otherwise it does not.</p>
    /// <p>By default, namespace-processing is enabled.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> bool </td><td class="memItemRight bottomAlign"><span class="name"><b>namespaceProcessing</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setNamespaceProcessing</b></span>(<i>bool</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn set_namespace_processing(&mut self, arg1: bool) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_setNamespaceProcessing(
            self as *mut crate::QXmlStreamReader,
            arg1,
        )
    }

    /// <p>Reads until the end of the current element, skipping any child nodes. This function is useful for skipping unknown elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamReader::skipCurrentElement()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#skipCurrentElement">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads until the end of the current element, skipping any child nodes. This function is useful for skipping unknown elements.</p>
    /// <p>The current element is the element matching the most recently parsed start element of which a matching end element has not yet been reached. When the parser has reached the end element, the current element becomes the parent element.</p>
    /// <p>This function was introduced in  Qt 4.6.</p></div>
    pub unsafe fn skip_current_element(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_skipCurrentElement(
            self as *mut crate::QXmlStreamReader,
        )
    }

    /// <p>Returns the text of <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">Characters</a>, <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">Comment</a>, <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">DTD</a>, or <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EntityReference</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QXmlStreamReader::text() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#text">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the text of <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">Characters</a>, <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">Comment</a>, <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">DTD</a>, or <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">EntityReference</a>.</p></div>
    pub unsafe fn text(&self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_text(
            self as *const crate::QXmlStreamReader,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the reader's current token as string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QXmlStreamReader::tokenString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the reader's current token as string.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">tokenType</a>().</p></div>
    pub unsafe fn token_string(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_tokenString(
            self as *const crate::QXmlStreamReader,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the type of the current token.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamReader::TokenType QXmlStreamReader::tokenType() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the type of the current token.</p>
    /// <p>The current token can also be queried with the convenience functions <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isStartDocument">isStartDocument</a>(), <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isEndDocument">isEndDocument</a>(), <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isStartElement">isStartElement</a>(), <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isEndElement">isEndElement</a>(), <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isCharacters">isCharacters</a>(), <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isComment">isComment</a>(), <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isDTD">isDTD</a>(), <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isEntityReference">isEntityReference</a>(), and <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#isProcessingInstruction">isProcessingInstruction</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenString">tokenString</a>().</p></div>
    pub unsafe fn token_type(&self) -> crate::q_xml_stream_reader::TokenType {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_tokenType(
            self as *const crate::QXmlStreamReader,
        )
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a> class provides an XML writer with a simple streaming API.</p>
///
/// C++ class: <span style='color: green;'>```QXmlStreamWriter```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a> class provides an XML writer with a simple streaming API.</p>
/// <p><a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a> is the counterpart to <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html">QXmlStreamReader</a> for writing XML. Like its related class, it operates on a <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> specified with <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#setDevice">setDevice</a>(). The API is simple and straightforward: for every XML token or event you want to write, the writer provides a specialized function.</p>
/// <p>You start a document with <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeStartDocument-1">writeStartDocument</a>() and end it with <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeEndDocument">writeEndDocument</a>(). This will implicitly close all remaining open tags.</p>
/// <p>Element tags are opened with <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeStartElement">writeStartElement</a>() followed by <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeAttribute">writeAttribute</a>() or <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeAttributes">writeAttributes</a>(), element content, and then <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeEndElement">writeEndElement</a>(). A shorter form <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeEmptyElement">writeEmptyElement</a>() can be used to write empty elements, followed by <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeAttributes">writeAttributes</a>().</p>
/// <p>Element content consists of either characters, entity references or nested elements. It is written with <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeCharacters">writeCharacters</a>(), which also takes care of escaping all forbidden characters and character sequences, <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeEntityReference">writeEntityReference</a>(), or subsequent calls to <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeStartElement">writeStartElement</a>(). A convenience method <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeTextElement">writeTextElement</a>() can be used for writing terminal elements that contain nothing but text.</p>
/// <p>The following abridged code snippet shows the basic use of the class to write formatted XML with indentation:</p>
/// <pre class="cpp">
///
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#QXmlStreamWriter">QXmlStreamWriter</a></span> stream(<span class="operator">&amp;</span>output);
/// &#32;     stream<span class="operator">.</span>setAutoFormatting(<span class="keyword">true</span>);
/// &#32;     stream<span class="operator">.</span>writeStartDocument();
/// &#32;     ...
/// &#32;     stream<span class="operator">.</span>writeStartElement(<span class="string">"bookmark"</span>);
/// &#32;     stream<span class="operator">.</span>writeAttribute(<span class="string">"href"</span><span class="operator">,</span> <span class="string">"http://qt-project.org/"</span>);
/// &#32;     stream<span class="operator">.</span>writeTextElement(<span class="string">"title"</span><span class="operator">,</span> <span class="string">"Qt Project"</span>);
/// &#32;     stream<span class="operator">.</span>writeEndElement(); <span class="comment">// bookmark</span>
/// &#32;     ...
/// &#32;     stream<span class="operator">.</span>writeEndDocument();
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a> takes care of prefixing namespaces, all you have to do is specify the <code>namespaceUri</code> when writing elements or attributes. If you must conform to certain prefixes, you can force the writer to use them by declaring the namespaces manually with either <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeNamespace">writeNamespace</a>() or <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeDefaultNamespace">writeDefaultNamespace</a>(). Alternatively, you can bypass the stream writer's namespace support and use overloaded methods that take a qualified name instead. The namespace <i>http://www.w3.org/XML/1998/namespace</i> is implicit and mapped to the prefix <i>xml</i>.</p>
/// <p>The stream writer can automatically format the generated XML data by adding line-breaks and indentation to empty sections between elements, making the XML data more readable for humans and easier to work with for most source code management systems. The feature can be turned on with the <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#autoFormatting-prop">autoFormatting</a> property, and customized with the <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#autoFormattingIndent-prop">autoFormattingIndent</a> property.</p>
/// <p>Other functions are <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeCDATA">writeCDATA</a>(), <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeComment">writeComment</a>(), <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeProcessingInstruction">writeProcessingInstruction</a>(), and <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeDTD">writeDTD</a>(). Chaining of XML streams is supported with <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeCurrentToken">writeCurrentToken</a>().</p>
/// <p>By default, <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a> encodes XML in UTF-8. Different encodings can be enforced using <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#setCodec">setCodec</a>().</p>
/// <p>If an error occurs while writing to the underlying device, <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#hasError">hasError</a>() starts returning true and subsequent writes are ignored.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qtxml-streambookmarks-example.html">QXmlStream Bookmarks Example</a> illustrates how to use a stream writer to write an XML bookmark file (XBEL) that was previously read in by a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html">QXmlStreamReader</a>.</p></div>
#[repr(C)]
pub struct QXmlStreamWriter {
    _unused: u8,
}
impl QXmlStreamWriter {
    /// <p>The auto-formatting flag of the stream writer</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QXmlStreamWriter::autoFormatting() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#autoFormatting-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The auto-formatting flag of the stream writer</p>
    /// <p>This property controls whether or not the stream writer automatically formats the generated XML data. If enabled, the writer automatically adds line-breaks and indentation to empty sections between elements (ignorable whitespace). The main purpose of auto-formatting is to split the data into several lines, and to increase readability for a human reader. The indentation depth can be controlled through the <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#autoFormattingIndent-prop">autoFormattingIndent</a> property.</p>
    /// <p>By default, auto-formatting is disabled.</p>
    /// <p>This property was introduced in  Qt 4.4.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> bool </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#autoFormatting">autoFormatting</a></b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b><a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#setAutoFormatting">setAutoFormatting</a></b></span>(bool <i>enable</i>)</td></tr>
    /// </tbody></table></div></div>
    pub unsafe fn auto_formatting(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_autoFormatting(
            self as *const crate::QXmlStreamWriter,
        )
    }

    /// <p>This property holds the number of spaces or tabs used for indentation when auto-formatting is enabled. Positive numbers indicate spaces, negative numbers tabs.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QXmlStreamWriter::autoFormattingIndent() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#autoFormattingIndent-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the number of spaces or tabs used for indentation when auto-formatting is enabled. Positive numbers indicate spaces, negative numbers tabs.</p>
    /// <p>The default indentation is 4.</p>
    /// <p>This property was introduced in  Qt 4.4.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>autoFormattingIndent</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setAutoFormattingIndent</b></span>(int <i>spacesOrTabs</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#autoFormatting-prop">autoFormatting</a>.</p></div>
    pub unsafe fn auto_formatting_indent(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_autoFormattingIndent(
            self as *const crate::QXmlStreamWriter,
        )
    }

    /// <p>Returns the codec that is currently assigned to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextCodec* QXmlStreamWriter::codec() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#codec">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the codec that is currently assigned to the stream.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#setCodec">setCodec</a>().</p></div>
    pub unsafe fn codec(&self) -> ::cpp_utils::MutPtr<crate::QTextCodec> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_codec(
            self as *const crate::QXmlStreamWriter,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns the current device associated with the <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a>, or 0 if no device has been assigned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QIODevice* QXmlStreamWriter::device() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#device">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current device associated with the <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a>, or 0 if no device has been assigned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#setDevice">setDevice</a>().</p></div>
    pub unsafe fn device(&self) -> ::cpp_utils::MutPtr<crate::QIODevice> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_device(
            self as *const crate::QXmlStreamWriter,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns <code>true</code> if the stream failed to write to the underlying device.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QXmlStreamWriter::hasError() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#hasError">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the stream failed to write to the underlying device.</p>
    /// <p>The error status is never reset. Writes happening after the error occurred are ignored, even if the error condition is cleared.</p></div>
    pub unsafe fn has_error(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_hasError(
            self as *const crate::QXmlStreamWriter,
        )
    }

    /// <p>Constructs a stream writer.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamWriter::QXmlStreamWriter()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#QXmlStreamWriter">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a stream writer.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#setDevice">setDevice</a>().</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QXmlStreamWriter> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_QXmlStreamWriter();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a stream writer that writes into <i>device</i>;</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamWriter::QXmlStreamWriter(QIODevice* device)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#QXmlStreamWriter-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a stream writer that writes into <i>device</i>;</p></div>
    pub unsafe fn from_q_io_device(
        device: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QIODevice>>,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamWriter> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_QXmlStreamWriter1(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QIODevice>>::cast_into(device)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a stream writer that writes into <i>array</i>. This is the same as creating an xml writer that operates on a <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> device which in turn operates on <i>array</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamWriter::QXmlStreamWriter(QByteArray* array)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#QXmlStreamWriter-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a stream writer that writes into <i>array</i>. This is the same as creating an xml writer that operates on a <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> device which in turn operates on <i>array</i>.</p></div>
    pub unsafe fn from_q_byte_array(
        array: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamWriter> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_QXmlStreamWriter2(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QByteArray>>::cast_into(array)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a stream writer that writes into <i>string</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QXmlStreamWriter::QXmlStreamWriter(QString* string)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#QXmlStreamWriter-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a stream writer that writes into <i>string</i>.</p></div>
    pub unsafe fn from_q_string(
        string: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamWriter> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_QXmlStreamWriter3(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QString>>::cast_into(string)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Enables auto formatting if <i>enable</i> is <code>true</code>, otherwise disables it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::setAutoFormatting(bool arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#setAutoFormatting">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Enables auto formatting if <i>enable</i> is <code>true</code>, otherwise disables it.</p>
    /// <p>The default value is <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>Note:</b> Setter function for property <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#autoFormatting-prop">autoFormatting</a>. </p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#autoFormatting">autoFormatting</a>().</p></div>
    pub unsafe fn set_auto_formatting(&mut self, arg1: bool) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_setAutoFormatting(
            self as *mut crate::QXmlStreamWriter,
            arg1,
        )
    }

    /// <p>This property holds the number of spaces or tabs used for indentation when auto-formatting is enabled. Positive numbers indicate spaces, negative numbers tabs.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::setAutoFormattingIndent(int spacesOrTabs)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#autoFormattingIndent-prop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This property holds the number of spaces or tabs used for indentation when auto-formatting is enabled. Positive numbers indicate spaces, negative numbers tabs.</p>
    /// <p>The default indentation is 4.</p>
    /// <p>This property was introduced in  Qt 4.4.</p>
    /// <p><b>Access functions:</b></p>
    /// <div class="table"><table class="alignedsummary">
    /// <tbody><tr><td class="memItemLeft topAlign rightAlign"> int </td><td class="memItemRight bottomAlign"><span class="name"><b>autoFormattingIndent</b></span>() const</td></tr>
    /// <tr><td class="memItemLeft topAlign rightAlign"> void </td><td class="memItemRight bottomAlign"><span class="name"><b>setAutoFormattingIndent</b></span>(int <i>spacesOrTabs</i>)</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#autoFormatting-prop">autoFormatting</a>.</p></div>
    pub unsafe fn set_auto_formatting_indent(&mut self, spaces_or_tabs: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_setAutoFormattingIndent(
            self as *mut crate::QXmlStreamWriter,
            spaces_or_tabs,
        )
    }

    /// <p>Sets the codec for this stream to <i>codec</i>. The codec is used for encoding any data that is written. By default, <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a> uses UTF-8.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::setCodec(QTextCodec* codec)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#setCodec">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the codec for this stream to <i>codec</i>. The codec is used for encoding any data that is written. By default, <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a> uses UTF-8.</p>
    /// <p>The encoding information is stored in the initial xml tag which gets written when you call <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeStartDocument-1">writeStartDocument</a>(). Call this function before calling <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeStartDocument-1">writeStartDocument</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#codec">codec</a>().</p></div>
    pub unsafe fn set_codec_q_text_codec(
        &mut self,
        codec: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QTextCodec>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_setCodec(
            self as *mut crate::QXmlStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QTextCodec>>::cast_into(codec)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Sets the codec for this stream to the <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> for the encoding specified by <i>codecName</i>. Common values for <code>codecName</code> include "ISO 8859-1", "UTF-8", and "UTF-16". If the encoding isn't recognized, nothing happens.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::setCodec(const char* codecName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#setCodec-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the codec for this stream to the <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a> for the encoding specified by <i>codecName</i>. Common values for <code>codecName</code> include "ISO 8859-1", "UTF-8", and "UTF-16". If the encoding isn't recognized, nothing happens.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForName">QTextCodec::codecForName</a>().</p></div>
    pub unsafe fn set_codec_char(
        &mut self,
        codec_name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_setCodec1(
            self as *mut crate::QXmlStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(
                codec_name,
            )
            .as_raw_ptr(),
        )
    }

    /// <p>Sets the current device to <i>device</i>. If you want the stream to write into a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>, you can create a <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> device.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::setDevice(QIODevice* device)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#setDevice">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the current device to <i>device</i>. If you want the stream to write into a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>, you can create a <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a> device.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#device">device</a>().</p></div>
    pub unsafe fn set_device(
        &mut self,
        device: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QIODevice>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_setDevice(
            self as *mut crate::QXmlStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QIODevice>>::cast_into(device)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::writeAttribute(const QString& qualifiedName, const QString& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeAttribute-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes an attribute with <i>qualifiedName</i> and <i>value</i>.</p>
    /// <p>This function can only be called after <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeStartElement">writeStartElement</a>() before any content is written, or after <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeEmptyElement">writeEmptyElement</a>().</p></div>
    pub unsafe fn write_attribute_2a(
        &mut self,
        qualified_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_writeAttribute(
            self as *mut crate::QXmlStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(qualified_name)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>Writes an attribute with <i>name</i> and <i>value</i>, prefixed for the specified <i>namespaceUri</i>. If the namespace has not been declared yet, <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a> will generate a namespace declaration for it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::writeAttribute(const QString& namespaceUri, const QString& name, const QString& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeAttribute">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes an attribute with <i>name</i> and <i>value</i>, prefixed for the specified <i>namespaceUri</i>. If the namespace has not been declared yet, <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a> will generate a namespace declaration for it.</p>
    /// <p>This function can only be called after <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeStartElement">writeStartElement</a>() before any content is written, or after <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeEmptyElement">writeEmptyElement</a>().</p></div>
    pub unsafe fn write_attribute_3a(
        &mut self,
        namespace_uri: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_writeAttribute1(
            self as *mut crate::QXmlStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(namespace_uri)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::writeAttribute(const QXmlStreamAttribute& attribute)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeAttribute-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes the <i>attribute</i>.</p>
    /// <p>This function can only be called after <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeStartElement">writeStartElement</a>() before any content is written, or after <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeEmptyElement">writeEmptyElement</a>().</p></div>
    pub unsafe fn write_attribute_1a(
        &mut self,
        attribute: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamAttribute>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_writeAttribute2(
            self as *mut crate::QXmlStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamAttribute>>::cast_into(
                attribute,
            )
            .as_raw_ptr(),
        )
    }

    /// <p>Writes the attribute vector <i>attributes</i>. If a namespace referenced in an attribute not been declared yet, <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a> will generate a namespace declaration for it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::writeAttributes(const QXmlStreamAttributes& attributes)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeAttributes">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes the attribute vector <i>attributes</i>. If a namespace referenced in an attribute not been declared yet, <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a> will generate a namespace declaration for it.</p>
    /// <p>This function can only be called after <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeStartElement">writeStartElement</a>() before any content is written, or after <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeEmptyElement">writeEmptyElement</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeAttribute">writeAttribute</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeNamespace">writeNamespace</a>().</p></div>
    pub unsafe fn write_attributes(
        &mut self,
        attributes: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamAttributes>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_writeAttributes(
            self as *mut crate::QXmlStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamAttributes>>::cast_into(
                attributes,
            )
            .as_raw_ptr(),
        )
    }

    /// <p>Writes <i>text</i> as CDATA section. If <i>text</i> contains the forbidden character sequence "]]&gt;", it is split into different CDATA sections.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::writeCDATA(const QString& text)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeCDATA">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes <i>text</i> as CDATA section. If <i>text</i> contains the forbidden character sequence "]]&gt;", it is split into different CDATA sections.</p>
    /// <p>This function mainly exists for completeness. Normally you should not need use it, because <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeCharacters">writeCharacters</a>() automatically escapes all non-content characters.</p></div>
    pub unsafe fn write_c_d_a_t_a(
        &mut self,
        text: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_writeCDATA(
            self as *mut crate::QXmlStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(text).as_raw_ptr(),
        )
    }

    /// <p>Writes <i>text</i>. The characters "&lt;", "&amp;", and """ are escaped as entity references "&amp;lt;", "&amp;amp;, and "&amp;quot;". To avoid the forbidden sequence "]]&gt;", "&gt;" is also escaped as "&amp;gt;".</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::writeCharacters(const QString& text)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeCharacters">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes <i>text</i>. The characters "&lt;", "&amp;", and """ are escaped as entity references "&amp;lt;", "&amp;amp;, and "&amp;quot;". To avoid the forbidden sequence "]]&gt;", "&gt;" is also escaped as "&amp;gt;".</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeEntityReference">writeEntityReference</a>().</p></div>
    pub unsafe fn write_characters(
        &mut self,
        text: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_writeCharacters(
            self as *mut crate::QXmlStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(text).as_raw_ptr(),
        )
    }

    /// <p>Writes <i>text</i> as XML comment, where <i>text</i> must not contain the forbidden sequence "--" or end with "-". Note that XML does not provide any way to escape "-" in a comment.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::writeComment(const QString& text)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeComment">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes <i>text</i> as XML comment, where <i>text</i> must not contain the forbidden sequence "--" or end with "-". Note that XML does not provide any way to escape "-" in a comment.</p></div>
    pub unsafe fn write_comment(
        &mut self,
        text: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_writeComment(
            self as *mut crate::QXmlStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(text).as_raw_ptr(),
        )
    }

    /// <p>Writes the current state of the <i>reader</i>. All possible valid states are supported.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::writeCurrentToken(const QXmlStreamReader& reader)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeCurrentToken">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes the current state of the <i>reader</i>. All possible valid states are supported.</p>
    /// <p>The purpose of this function is to support chained processing of XML data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#tokenType">QXmlStreamReader::tokenType</a>().</p></div>
    pub unsafe fn write_current_token(
        &mut self,
        reader: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamReader>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_writeCurrentToken(
            self as *mut crate::QXmlStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamReader>>::cast_into(reader)
                .as_raw_ptr(),
        )
    }

    /// <p>Writes a DTD section. The <i>dtd</i> represents the entire doctypedecl production from the XML 1.0 specification.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::writeDTD(const QString& dtd)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeDTD">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes a DTD section. The <i>dtd</i> represents the entire doctypedecl production from the XML 1.0 specification.</p></div>
    pub unsafe fn write_d_t_d(
        &mut self,
        dtd: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_writeDTD(
            self as *mut crate::QXmlStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(dtd).as_raw_ptr(),
        )
    }

    /// <p>Writes a default namespace declaration for <i>namespaceUri</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::writeDefaultNamespace(const QString& namespaceUri)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeDefaultNamespace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes a default namespace declaration for <i>namespaceUri</i>.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeStartElement">writeStartElement</a>() or <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeEmptyElement">writeEmptyElement</a>() was called, the declaration applies to the current element; otherwise it applies to the next child element.</p>
    /// <p>Note that the namespaces <i>http://www.w3.org/XML/1998/namespace</i> (bound to <i>xmlns</i>) and <i>http://www.w3.org/2000/xmlns/</i> (bound to <i>xml</i>) by definition cannot be declared as default.</p></div>
    pub unsafe fn write_default_namespace(
        &mut self,
        namespace_uri: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_writeDefaultNamespace(
            self as *mut crate::QXmlStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(namespace_uri)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::writeEmptyElement(const QString& qualifiedName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeEmptyElement-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes an empty element with qualified name <i>qualifiedName</i>. Subsequent calls to <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeAttribute">writeAttribute</a>() will add attributes to this element.</p></div>
    pub unsafe fn write_empty_element_1a(
        &mut self,
        qualified_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_writeEmptyElement(
            self as *mut crate::QXmlStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(qualified_name)
                .as_raw_ptr(),
        )
    }

    /// <p>Writes an empty element with <i>name</i>, prefixed for the specified <i>namespaceUri</i>. If the namespace has not been declared, <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a> will generate a namespace declaration for it. Subsequent calls to <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeAttribute">writeAttribute</a>() will add attributes to this element.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::writeEmptyElement(const QString& namespaceUri, const QString& name)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeEmptyElement">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes an empty element with <i>name</i>, prefixed for the specified <i>namespaceUri</i>. If the namespace has not been declared, <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a> will generate a namespace declaration for it. Subsequent calls to <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeAttribute">writeAttribute</a>() will add attributes to this element.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeNamespace">writeNamespace</a>().</p></div>
    pub unsafe fn write_empty_element_2a(
        &mut self,
        namespace_uri: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_writeEmptyElement1(
            self as *mut crate::QXmlStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(namespace_uri)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
        )
    }

    /// <p>Closes all remaining open start elements and writes a newline.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::writeEndDocument()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeEndDocument">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Closes all remaining open start elements and writes a newline.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeStartDocument-1">writeStartDocument</a>().</p></div>
    pub unsafe fn write_end_document(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_writeEndDocument(
            self as *mut crate::QXmlStreamWriter,
        )
    }

    /// <p>Closes the previous start element.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::writeEndElement()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeEndElement">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Closes the previous start element.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeStartElement">writeStartElement</a>().</p></div>
    pub unsafe fn write_end_element(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_writeEndElement(
            self as *mut crate::QXmlStreamWriter,
        )
    }

    /// <p>Writes the entity reference <i>name</i> to the stream, as "&amp;<i>name</i>;".</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::writeEntityReference(const QString& name)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeEntityReference">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes the entity reference <i>name</i> to the stream, as "&amp;<i>name</i>;".</p></div>
    pub unsafe fn write_entity_reference(
        &mut self,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_writeEntityReference(
            self as *mut crate::QXmlStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
        )
    }

    /// <p>Writes a namespace declaration for <i>namespaceUri</i> with <i>prefix</i>. If <i>prefix</i> is empty, <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a> assigns a unique prefix consisting of the letter 'n' followed by a number.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::writeNamespace(const QString& namespaceUri, const QString& prefix = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeNamespace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes a namespace declaration for <i>namespaceUri</i> with <i>prefix</i>. If <i>prefix</i> is empty, <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a> assigns a unique prefix consisting of the letter 'n' followed by a number.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeStartElement">writeStartElement</a>() or <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeEmptyElement">writeEmptyElement</a>() was called, the declaration applies to the current element; otherwise it applies to the next child element.</p>
    /// <p>Note that the prefix <i>xml</i> is both predefined and reserved for <i>http://www.w3.org/XML/1998/namespace</i>, which in turn cannot be bound to any other prefix. The prefix <i>xmlns</i> and its URI <i>http://www.w3.org/2000/xmlns/</i> are used for the namespace mechanism itself and thus completely forbidden in declarations.</p></div>
    pub unsafe fn write_namespace_2a(
        &mut self,
        namespace_uri: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        prefix: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_writeNamespace(
            self as *mut crate::QXmlStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(namespace_uri)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(prefix)
                .as_raw_ptr(),
        )
    }

    /// <p>Writes a namespace declaration for <i>namespaceUri</i> with <i>prefix</i>. If <i>prefix</i> is empty, <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a> assigns a unique prefix consisting of the letter 'n' followed by a number.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::writeNamespace(const QString& namespaceUri)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeNamespace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes a namespace declaration for <i>namespaceUri</i> with <i>prefix</i>. If <i>prefix</i> is empty, <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a> assigns a unique prefix consisting of the letter 'n' followed by a number.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeStartElement">writeStartElement</a>() or <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeEmptyElement">writeEmptyElement</a>() was called, the declaration applies to the current element; otherwise it applies to the next child element.</p>
    /// <p>Note that the prefix <i>xml</i> is both predefined and reserved for <i>http://www.w3.org/XML/1998/namespace</i>, which in turn cannot be bound to any other prefix. The prefix <i>xmlns</i> and its URI <i>http://www.w3.org/2000/xmlns/</i> are used for the namespace mechanism itself and thus completely forbidden in declarations.</p></div>
    pub unsafe fn write_namespace_1a(
        &mut self,
        namespace_uri: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_writeNamespace1(
            self as *mut crate::QXmlStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(namespace_uri)
                .as_raw_ptr(),
        )
    }

    /// <p>Writes an XML processing instruction with <i>target</i> and <i>data</i>, where <i>data</i> must not contain the sequence "?&gt;".</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::writeProcessingInstruction(const QString& target, const QString& data = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeProcessingInstruction">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes an XML processing instruction with <i>target</i> and <i>data</i>, where <i>data</i> must not contain the sequence "?&gt;".</p></div>
    pub unsafe fn write_processing_instruction_2a(
        &mut self,
        target: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_writeProcessingInstruction(
            self as *mut crate::QXmlStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(target)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(data).as_raw_ptr(),
        )
    }

    /// <p>Writes an XML processing instruction with <i>target</i> and <i>data</i>, where <i>data</i> must not contain the sequence "?&gt;".</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::writeProcessingInstruction(const QString& target)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeProcessingInstruction">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes an XML processing instruction with <i>target</i> and <i>data</i>, where <i>data</i> must not contain the sequence "?&gt;".</p></div>
    pub unsafe fn write_processing_instruction_1a(
        &mut self,
        target: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_writeProcessingInstruction1(
            self as *mut crate::QXmlStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(target)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::writeStartDocument()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeStartDocument-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes a document start with XML version number "1.0". This also writes the encoding information.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeEndDocument">writeEndDocument</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#setCodec">setCodec</a>().</p></div>
    pub unsafe fn write_start_document_0a(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_writeStartDocument(
            self as *mut crate::QXmlStreamWriter,
        )
    }

    /// <p>Writes a document start with the XML version number <i>version</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::writeStartDocument(const QString& version)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeStartDocument">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes a document start with the XML version number <i>version</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeEndDocument">writeEndDocument</a>().</p></div>
    pub unsafe fn write_start_document_1a(
        &mut self,
        version: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_writeStartDocument1(
            self as *mut crate::QXmlStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(version)
                .as_raw_ptr(),
        )
    }

    /// <p>Writes a document start with the XML version number <i>version</i> and a standalone attribute <i>standalone</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::writeStartDocument(const QString& version, bool standalone)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeStartDocument-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes a document start with the XML version number <i>version</i> and a standalone attribute <i>standalone</i>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeEndDocument">writeEndDocument</a>().</p></div>
    pub unsafe fn write_start_document_2a(
        &mut self,
        version: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        standalone: bool,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_writeStartDocument2(
            self as *mut crate::QXmlStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(version)
                .as_raw_ptr(),
            standalone,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::writeStartElement(const QString& qualifiedName)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeStartElement-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes a start element with <i>qualifiedName</i>. Subsequent calls to <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeAttribute">writeAttribute</a>() will add attributes to this element.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeEndElement">writeEndElement</a>() and <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeEmptyElement">writeEmptyElement</a>().</p></div>
    pub unsafe fn write_start_element_1a(
        &mut self,
        qualified_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_writeStartElement(
            self as *mut crate::QXmlStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(qualified_name)
                .as_raw_ptr(),
        )
    }

    /// <p>Writes a start element with <i>name</i>, prefixed for the specified <i>namespaceUri</i>. If the namespace has not been declared yet, <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a> will generate a namespace declaration for it. Subsequent calls to <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeAttribute">writeAttribute</a>() will add attributes to this element.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::writeStartElement(const QString& namespaceUri, const QString& name)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeStartElement">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes a start element with <i>name</i>, prefixed for the specified <i>namespaceUri</i>. If the namespace has not been declared yet, <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a> will generate a namespace declaration for it. Subsequent calls to <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeAttribute">writeAttribute</a>() will add attributes to this element.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeNamespace">writeNamespace</a>(), <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeEndElement">writeEndElement</a>(), and <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeEmptyElement">writeEmptyElement</a>().</p></div>
    pub unsafe fn write_start_element_2a(
        &mut self,
        namespace_uri: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_writeStartElement1(
            self as *mut crate::QXmlStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(namespace_uri)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::writeTextElement(const QString& qualifiedName, const QString& text)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeTextElement-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes a text element with <i>qualifiedName</i> and <i>text</i>.</p>
    /// <p>This is a convenience function equivalent to:</p>
    /// <pre class="cpp">
    ///
    /// &#32;   &#32;     writeStartElement(qualifiedName);
    /// &#32;   &#32;     writeCharacters(text);
    /// &#32;   &#32;     writeEndElement();
    ///
    /// </pre></div>
    pub unsafe fn write_text_element_2a(
        &mut self,
        qualified_name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        text: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_writeTextElement(
            self as *mut crate::QXmlStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(qualified_name)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(text).as_raw_ptr(),
        )
    }

    /// <p>Writes a text element with <i>name</i>, prefixed for the specified <i>namespaceUri</i>, and <i>text</i>. If the namespace has not been declared, <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a> will generate a namespace declaration for it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QXmlStreamWriter::writeTextElement(const QString& namespaceUri, const QString& name, const QString& text)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeTextElement">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes a text element with <i>name</i>, prefixed for the specified <i>namespaceUri</i>, and <i>text</i>. If the namespace has not been declared, <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a> will generate a namespace declaration for it.</p>
    /// <p>This is a convenience function equivalent to:</p>
    /// <pre class="cpp">
    ///
    /// &#32;   &#32;     writeStartElement(namespaceUri<span class="operator">,</span> name);
    /// &#32;   &#32;     writeCharacters(text);
    /// &#32;   &#32;     writeEndElement();
    ///
    /// </pre></div>
    pub unsafe fn write_text_element_3a(
        &mut self,
        namespace_uri: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        name: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        text: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_writeTextElement1(
            self as *mut crate::QXmlStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(namespace_uri)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(name).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(text).as_raw_ptr(),
        )
    }
}

pub mod q_list_of_q_string {
    //! C++ type: <span style='color: green;'>```QList<QString>```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QString>::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QString>::iterator& QList<QString>::iterator::operator=(const QList<QString>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_string::Iterator>>,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_string::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_iterator_operator_8(self as *mut crate::q_list_of_q_string::Iterator, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_string::Iterator>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QString>::iterator QList<QString>::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_string::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_iterator_operator__13(
                self as *mut crate::q_list_of_q_string::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QString>::iterator QList<QString>::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_string::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_iterator_operator__11(
                self as *mut crate::q_list_of_q_string::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QString& QList<QString>::iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValueRef iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>. If you assign to it, the assignment will apply to the character in the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> of <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> from which you got the reference.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::MutRef<crate::QString> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_iterator_operator__1(
                self as *const crate::q_list_of_q_string::Iterator,
                j,
            );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QString>::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>().</p></div>
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_list_of_q_string::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QString>::iterator::iterator(const QList<QString>::iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_string::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_string::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_iterator_iterator1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_string::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QString* QList<QString>::iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's value.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::MutPtr<crate::QString> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_iterator_operator__(
                self as *const crate::q_list_of_q_string::Iterator,
            );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QString>::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QString>::const_iterator& QList<QString>::const_iterator::operator=(const QList<QString>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_string::ConstIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_string::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QString_const_iterator_operator_6(
                    self as *mut crate::q_list_of_q_string::ConstIterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_string::ConstIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QString>::const_iterator QList<QString>::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_string::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_const_iterator_operator__9(
                self as *mut crate::q_list_of_q_string::ConstIterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QString>::const_iterator QList<QString>::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_string::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_const_iterator_operator__7(
                self as *mut crate::q_list_of_q_string::ConstIterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QString& QList<QString>::const_iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValue const_iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::Ref<crate::QString> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_const_iterator_operator__1(
                self as *const crate::q_list_of_q_string::ConstIterator,
                j,
            );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QString>::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>().</p></div>
        pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::q_list_of_q_string::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QString_const_iterator_const_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QString>::const_iterator::const_iterator(const QList<QString>::iterator& o)```</span>.
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_string::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_string::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_const_iterator_const_iterator2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_string::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QString>::const_iterator::const_iterator(const QList<QString>::const_iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_string::ConstIterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_string::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QString_const_iterator_const_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_string::ConstIterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current result.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QString* QList<QString>::const_iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current result.</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::Ptr<crate::QString> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_const_iterator_operator__(
                self as *const crate::q_list_of_q_string::ConstIterator,
            );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_string::Iterator {
        type Output = ::cpp_utils::MutRef<crate::QString>;
        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QString& QList<QString>::iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>().</p></div>
        fn indirection(self) -> ::cpp_utils::MutRef<crate::QString> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_iterator_operator_(
                    self as *const crate::q_list_of_q_string::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_string::Iterator>>
        for crate::q_list_of_q_string::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QString>::iterator::operator==(const QList<QString>::iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_string::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_iterator_operator__2(
                    self as *const crate::q_list_of_q_string::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_string::Iterator>>
        for crate::q_list_of_q_string::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QString>::iterator::operator<(const QList<QString>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_string::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_iterator_operator_1(
                    self as *const crate::q_list_of_q_string::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_string::Iterator>>
        for crate::q_list_of_q_string::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QString>::iterator::operator<=(const QList<QString>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_string::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_iterator_operator__4(
                    self as *const crate::q_list_of_q_string::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_string::Iterator>>
        for crate::q_list_of_q_string::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QString>::iterator::operator>(const QList<QString>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_string::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_iterator_operator_2(
                    self as *const crate::q_list_of_q_string::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_string::Iterator>>
        for crate::q_list_of_q_string::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QString>::iterator::operator>=(const QList<QString>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_string::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_iterator_operator__5(
                    self as *const crate::q_list_of_q_string::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_string::ConstIterator>>
        for crate::q_list_of_q_string::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QString>::iterator::operator==(const QList<QString>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_string::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_iterator_operator__6(
                    self as *const crate::q_list_of_q_string::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_string::ConstIterator>>
        for crate::q_list_of_q_string::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QString>::iterator::operator<(const QList<QString>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_string::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_iterator_operator_3(
                    self as *const crate::q_list_of_q_string::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_string::ConstIterator>>
        for crate::q_list_of_q_string::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QString>::iterator::operator<=(const QList<QString>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_string::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_iterator_operator__8(
                    self as *const crate::q_list_of_q_string::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_string::ConstIterator>>
        for crate::q_list_of_q_string::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QString>::iterator::operator>(const QList<QString>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_string::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_iterator_operator_4(
                    self as *const crate::q_list_of_q_string::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_string::ConstIterator>>
        for crate::q_list_of_q_string::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QString>::iterator::operator>=(const QList<QString>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_string::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_iterator_operator__9(
                    self as *const crate::q_list_of_q_string::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_string::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_string::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QString>::iterator& QList<QString>::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_string::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_iterator_operator__10(
                    self as *mut crate::q_list_of_q_string::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_string::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_string::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QString>::iterator& QList<QString>::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_string::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_iterator_operator__12(
                    self as *mut crate::q_list_of_q_string::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong> for crate::q_list_of_q_string::Iterator {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QString>::iterator& QList<QString>::iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_iterator_operator__14(
                    self as *mut crate::q_list_of_q_string::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong> for crate::q_list_of_q_string::Iterator {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QString>::iterator& QList<QString>::iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_iterator_operator__15(
                    self as *mut crate::q_list_of_q_string::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_list_of_q_string::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_string::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QString>::iterator QList<QString>::iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_string::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_iterator_operator_5(
                    self as *const crate::q_list_of_q_string::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_list_of_q_string::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_string::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QString>::iterator QList<QString>::iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_string::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_iterator_operator_6(
                    self as *const crate::q_list_of_q_string::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_string::Iterator>>
        for &crate::q_list_of_q_string::Iterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QString>::iterator::operator-(QList<QString>::iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_string::Iterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_iterator_operator_7(
                    self as *const crate::q_list_of_q_string::Iterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_string::ConstIterator {
        type Output = ::cpp_utils::Ref<crate::QString>;
        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QString& QList<QString>::const_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        fn indirection(self) -> ::cpp_utils::Ref<crate::QString> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_const_iterator_operator_(
                    self as *const crate::q_list_of_q_string::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_string::ConstIterator>>
        for crate::q_list_of_q_string::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QString>::const_iterator::operator==(const QList<QString>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_string::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_const_iterator_operator__2(
                    self as *const crate::q_list_of_q_string::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_string::ConstIterator>>
        for crate::q_list_of_q_string::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QString>::const_iterator::operator<(const QList<QString>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_string::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_const_iterator_operator_1(
                    self as *const crate::q_list_of_q_string::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_string::ConstIterator>>
        for crate::q_list_of_q_string::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QString>::const_iterator::operator<=(const QList<QString>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_string::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_const_iterator_operator__4(
                    self as *const crate::q_list_of_q_string::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_string::ConstIterator>>
        for crate::q_list_of_q_string::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QString>::const_iterator::operator>(const QList<QString>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_string::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_const_iterator_operator_2(
                    self as *const crate::q_list_of_q_string::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_string::ConstIterator>>
        for crate::q_list_of_q_string::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QString>::const_iterator::operator>=(const QList<QString>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_string::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_const_iterator_operator__5(
                    self as *const crate::q_list_of_q_string::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_string::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_string::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QString>::const_iterator& QList<QString>::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_string::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_const_iterator_operator__6(
                    self as *mut crate::q_list_of_q_string::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_string::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_string::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QString>::const_iterator& QList<QString>::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_string::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_const_iterator_operator__8(
                    self as *mut crate::q_list_of_q_string::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_string::ConstIterator
    {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QString>::const_iterator& QList<QString>::const_iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_const_iterator_operator__10(
                    self as *mut crate::q_list_of_q_string::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_string::ConstIterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QString>::const_iterator& QList<QString>::const_iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_const_iterator_operator__11(
                    self as *mut crate::q_list_of_q_string::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_list_of_q_string::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_string::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QString>::const_iterator QList<QString>::const_iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_string::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_const_iterator_operator_3(
                    self as *const crate::q_list_of_q_string::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_list_of_q_string::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_string::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QString>::const_iterator QList<QString>::const_iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_string::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_const_iterator_operator_4(
                    self as *const crate::q_list_of_q_string::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_string::ConstIterator>>
        for &crate::q_list_of_q_string::ConstIterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QString>::const_iterator::operator-(QList<QString>::const_iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_string::ConstIterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QString_const_iterator_operator_5(
                    self as *const crate::q_list_of_q_string::ConstIterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_string::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QString>::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QString_iterator_diterator(
                self as *mut crate::q_list_of_q_string::Iterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_string::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QString>::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QString_const_iterator_dconst_iterator(
                self as *mut crate::q_list_of_q_string::ConstIterator,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
///
/// C++ class: <span style='color: green;'>```QList<QString>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlist.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores items in a list that provides fast index-based access and index-based insertions and removals.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Internally, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T if <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. Otherwise, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T* and the items are allocated on the heap.</p>
/// <p>The array representation allows very fast insertions and index-based access. The <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() operations are also very fast because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates memory at both ends of its internal array. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.</p>
/// <p>Note, however, that when the conditions specified above are not met, each append or insert of a new item requires allocating the new item on the heap, and this per item allocation will make <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> a better choice for use cases that do a lot of appending or inserting, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> can allocate memory for many items in a single heap allocation.</p>
/// <p>Note that the internal array only ever gets bigger over the life of the list. It never shrinks. The internal array is deallocated by the destructor and by the assignment operator, when one list is assigned to another.</p>
/// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerList;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span><span class="operator">&gt;</span> dateList;
///
/// </pre>
/// <p>Qt includes a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class that inherits <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>&gt; and adds a few convenience functions, such as <a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#filter">QStringList::filter</a>(). <a href="http://doc.qt.io/qt-5/qstring.html#split">QString::split</a>() creates QStringLists from strings.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> stores a list of items. The default constructor creates an empty list. To insert items into the list, you can use operator&lt;&lt;():</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"one"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"two"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"three"</span>;
///   <span class="comment">// list: ["one", "two", "three"]</span>
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(). In addition, it provides the following convenience functions: <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const lists, operator[]() returns a reference to the item and can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Bob"</span>)
/// &#32;     list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Robert"</span>;
///
/// </pre>
/// <p>Because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented as an array of pointers for types that are larger than a pointer or are not movable, this operation requires (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>). For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> list<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (list<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Jane"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Jane at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>A common requirement is to remove an item from a list and do something with it. For this, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(). Here's a loop that removes the items from a list one at a time and calls <code>delete</code> on them:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   <span class="keyword">while</span> (<span class="operator">!</span>list<span class="operator">.</span>isEmpty())
/// &#32;     <span class="keyword">delete</span> list<span class="operator">.</span>takeFirst();
///
/// </pre>
/// <p>Inserting and removing items at either end of the list is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> in most cases), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
/// <p>If you want to find all occurrences of a particular value in a list, use <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of a matching item if they find it; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> list<span class="operator">.</span>indexOf(<span class="string">"Jane"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Jane is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a list contains a particular value, use <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the list, use <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>(). If you want to replace all occurrences of a particular value with another, use <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlistiterator.html">QListIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablelistiterator.html">QMutableListIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented in such a way that direct index-based access is just as fast as using iterators.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <p>To make <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as efficient as possible, its member functions don't validate their input before using it. Except for <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), member functions always assume the list is <i>not</i> empty. Member functions that take index values as parameters always assume their index value parameters are in the valid range. This means <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> member functions can fail. If you define QT_NO_DEBUG when you compile, failures will not be detected. If you <i>don't</i> define QT_NO_DEBUG, failures will be detected using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT">Q_ASSERT</a>() or <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT_X">Q_ASSERT_X</a>() with an appropriate message.</p>
/// <p>To avoid failures when your list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling other member functions. If you must pass an index value that might not be in the valid range, check that it is less than the value returned by <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() but <i>not</i> less than 0.</p>
/// <a name="more-members"></a></div>
#[repr(C)]
pub struct QListOfQString {
    _unused: u8,
}
impl QListOfQString {
    /// <p>Inserts <i>value</i> at the end of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QString>::append(const QString& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["one", "two", "three"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn append_q_string(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_append(
            self as *mut crate::QListOfQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QString>::append(const QList<QString>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> list to this list.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_list_of_q_string(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_append1(
            self as *mut crate::QListOfQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQString>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QString& QList<QString>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>This function is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QString_at(self as *const crate::QListOfQString, i);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QList<QString>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QString_back(self as *mut crate::QListOfQString);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QString& QList<QString>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QString_back1(self as *const crate::QListOfQString);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QString>::const_iterator QList<QString>::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(&self) -> ::cpp_utils::CppBox<crate::q_list_of_q_string::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_cbegin(
            self as *const crate::QListOfQString,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QString>::const_iterator QList<QString>::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn cend(&self) -> ::cpp_utils::CppBox<crate::q_list_of_q_string::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QString_cend(self as *const crate::QListOfQString);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all items from the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QString>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all items from the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_clear(self as *mut crate::QListOfQString)
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QString>::const_iterator QList<QString>::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_string::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_constBegin(
            self as *const crate::QListOfQString,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QString>::const_iterator QList<QString>::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn const_end(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_string::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_constEnd(
            self as *const crate::QListOfQString,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QString& QList<QString>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>().</p></div>
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_constFirst(
            self as *const crate::QListOfQString,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QString& QList<QString>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>().</p></div>
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_constLast(
            self as *const crate::QListOfQString,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QString>::contains(const QString& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn contains(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_contains(
            self as *const crate::QListOfQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QString>& QList<QString>::operator=(const QList<QString>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this list and returns a reference to this list.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQString>>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_operator_(
            self as *mut crate::QListOfQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQString>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of occurrences of <i>value</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QString>::count(const QString& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of occurrences of <i>value</i> in the list.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_count(
            self as *const crate::QListOfQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QString>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_count1(self as *const crate::QListOfQString)
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QString>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_detach(self as *mut crate::QListOfQString)
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QString>::detachShared()```</span>.
    pub unsafe fn detach_shared(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_detachShared(self as *mut crate::QListOfQString)
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QString>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_empty(self as *const crate::QListOfQString)
    }

    /// <p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QString>::endsWith(const QString& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn ends_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_endsWith(
            self as *const crate::QListOfQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QString>::iterator QList<QString>::erase(QList<QString>::iterator pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn erase_1a(
        &mut self,
        pos: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_string::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_string::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_erase(self as *mut crate::QListOfQString, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_string::Iterator>>::cast_into(pos).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QString>::iterator QList<QString>::erase(QList<QString>::iterator first, QList<QString>::iterator last)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes all the items from <i>begin</i> up to (but not including) <i>end</i>. Returns an iterator to the same item that <i>end</i> referred to before the call.</p></div>
    pub unsafe fn erase_2a(
        &mut self,
        first: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_string::Iterator>>,
        last: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_string::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_string::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_erase1(self as *mut crate::QListOfQString, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_string::Iterator>>::cast_into(first).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_string::Iterator>>::cast_into(last).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QList<QString>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QString_first(self as *mut crate::QListOfQString);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QString& QList<QString>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_first1(
            self as *const crate::QListOfQString,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QList<QString>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QString_front(self as *mut crate::QListOfQString);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QString& QList<QString>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_front1(
            self as *const crate::QListOfQString,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QString& QList<QString>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>(). This function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p></div>
    pub unsafe fn index(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_operator__2(
            self as *const crate::QListOfQString,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QList<QString>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If this function is called on a list that is currently being shared, it will trigger a copy of all elements. Otherwise, this function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>. If you do not want to modify the list you should use <a href="http://doc.qt.io/qt-5/qlist.html#at">QList::at</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_operator__3(
            self as *mut crate::QListOfQString,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QString>::indexOf(const QString& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_indexOf(
            self as *const crate::QListOfQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(t).as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QString>::indexOf(const QString& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_indexOf1(
            self as *const crate::QListOfQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QString>::insert(int i, const QString& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   list<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// list: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn insert_int_q_string(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_insert(
            self as *mut crate::QListOfQString,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QString>::iterator QList<QString>::insert(QList<QString>::iterator before, const QString& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>value</i> in front of the item pointed to by the iterator <i>before</i>. Returns an iterator pointing at the inserted item. Note that the iterator passed to the function will be invalid after the call; the returned iterator should be used instead.</p></div>
    pub unsafe fn insert_iterator_q_string(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_string::Iterator>>,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_string::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_insert1(self as *mut crate::QListOfQString, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_string::Iterator>>::cast_into(before).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(t).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QString>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_isDetached(self as *const crate::QListOfQString)
    }

    /// <p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QString>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_isEmpty(self as *const crate::QListOfQString)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QString>::isSharedWith(const QList<QString>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_isSharedWith(
            self as *const crate::QListOfQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQString>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QList<QString>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QString_last(self as *mut crate::QListOfQString);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QString& QList<QString>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QString_last1(self as *const crate::QListOfQString);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QString>::lastIndexOf(const QString& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_lastIndexOf(
            self as *const crate::QListOfQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(t).as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QString>::lastIndexOf(const QString& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_lastIndexOf1(
            self as *const crate::QListOfQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QString>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_length(self as *const crate::QListOfQString)
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QString> QList<QString>::mid(int pos, int length = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        length: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_mid(
            self as *const crate::QListOfQString,
            pos,
            length,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QString> QList<QString>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_mid1(
            self as *const crate::QListOfQString,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QString>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>move(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "C", "D", "E", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This is the same as insert(<i>to</i>, <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<i>from</i>)).This function assumes that both <i>from</i> and <i>to</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>from</i> and <i>to</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_move(
            self as *mut crate::QListOfQString,
            from,
            to,
        )
    }

    /// <p>Constructs an empty list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QString>::QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty list.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QListOfQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_QList();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QString>::QList(const QList<QString>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQString>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_QList1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQString>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QString>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_pop_back(self as *mut crate::QListOfQString)
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QString>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_pop_front(self as *mut crate::QListOfQString)
    }

    /// <p>Inserts <i>value</i> at the beginning of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QString>::prepend(const QString& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(0, <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_prepend(
            self as *mut crate::QListOfQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QString>::push_back(const QString& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_push_back(
            self as *mut crate::QListOfQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QString>::push_front(const QString& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_push_front(
            self as *mut crate::QListOfQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QString>::removeAll(const QString& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeAll(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_all(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_removeAll(
            self as *mut crate::QListOfQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QString>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_removeAt(self as *mut crate::QListOfQString, i)
    }

    /// <p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QString>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_removeFirst(self as *mut crate::QListOfQString)
    }

    /// <p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QString>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_removeLast(self as *mut crate::QListOfQString)
    }

    /// <p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QString>::removeOne(const QString& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeOne(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", ,"sun", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_one(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_removeOne(
            self as *mut crate::QListOfQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QString>::replace(int i, const QString& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_replace(
            self as *mut crate::QListOfQString,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Reserve space for <i>alloc</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QString>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reserve space for <i>alloc</i> elements.</p>
    /// <p>If <i>alloc</i> is smaller than the current size of the list, nothing will happen.</p>
    /// <p>Use this function to avoid repetetive reallocation of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s internal data if you can predict how many elements will be appended. Note that the reservation applies only to the internal pointer array.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_reserve(
            self as *mut crate::QListOfQString,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QString>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_setSharable(
            self as *mut crate::QListOfQString,
            sharable,
        )
    }

    /// <p>Returns the number of items in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QString>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_size(self as *const crate::QListOfQString)
    }

    /// <p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QString>::startsWith(const QString& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn starts_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_startsWith(
            self as *const crate::QListOfQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QString>::swap(QList<QString>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap_1a(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QListOfQString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_swap(
            self as *mut crate::QListOfQString,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QListOfQString>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QString>::swap(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swap(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    pub unsafe fn swap_2a(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_swap1(self as *mut crate::QListOfQString, i, j)
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QString>::swapItemsAt(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swapItemsAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swapItemsAt(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn swap_items_at(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_swapItemsAt(
            self as *mut crate::QListOfQString,
            i,
            j,
        )
    }

    /// <p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QList<QString>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_takeAt(
            self as *mut crate::QListOfQString,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QList<QString>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_takeFirst(
            self as *mut crate::QListOfQString,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QList<QString>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_takeLast(
            self as *mut crate::QListOfQString,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value at index position <i>i</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QList<QString>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the list.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that the index is going to be within bounds, you can use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_value(
            self as *const crate::QListOfQString,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QList<QString>::value(int i, const QString& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QString_value1(
            self as *const crate::QListOfQString,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_list_of_q_item_selection_range {
    //! C++ type: <span style='color: green;'>```QList<QItemSelectionRange>```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QItemSelectionRange>::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::iterator& QList<QItemSelectionRange>::iterator::operator=(const QList<QItemSelectionRange>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::Iterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_item_selection_range::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_iterator_operator_8(
                    self as *mut crate::q_list_of_q_item_selection_range::Iterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::Iterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::iterator QList<QItemSelectionRange>::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_iterator_operator__13(
                    self as *mut crate::q_list_of_q_item_selection_range::Iterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::iterator QList<QItemSelectionRange>::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_iterator_operator__11(
                    self as *mut crate::q_list_of_q_item_selection_range::Iterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QItemSelectionRange& QList<QItemSelectionRange>::iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValueRef iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>. If you assign to it, the assignment will apply to the character in the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> of <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> from which you got the reference.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::MutRef<crate::QItemSelectionRange> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_iterator_operator__1(
                    self as *const crate::q_list_of_q_item_selection_range::Iterator,
                    j,
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QItemSelectionRange>::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>().</p></div>
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::Iterator>
        {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QItemSelectionRange>::iterator::iterator(const QList<QItemSelectionRange>::iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::Iterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_iterator_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::Iterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QItemSelectionRange* QList<QItemSelectionRange>::iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's value.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::MutPtr<crate::QItemSelectionRange> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_iterator_operator__(
                    self as *const crate::q_list_of_q_item_selection_range::Iterator,
                );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QItemSelectionRange>::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::const_iterator& QList<QItemSelectionRange>::const_iterator::operator=(const QList<QItemSelectionRange>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::ConstIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_item_selection_range::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_const_iterator_operator_6(
                    self as *mut crate::q_list_of_q_item_selection_range::ConstIterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::ConstIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::const_iterator QList<QItemSelectionRange>::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_const_iterator_operator__9(
                    self as *mut crate::q_list_of_q_item_selection_range::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::const_iterator QList<QItemSelectionRange>::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_const_iterator_operator__7(
                    self as *mut crate::q_list_of_q_item_selection_range::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QItemSelectionRange& QList<QItemSelectionRange>::const_iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValue const_iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::Ref<crate::QItemSelectionRange> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_const_iterator_operator__1(
                    self as *const crate::q_list_of_q_item_selection_range::ConstIterator,
                    j,
                );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QItemSelectionRange>::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>().</p></div>
        pub unsafe fn new_0a(
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_const_iterator_const_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QItemSelectionRange>::const_iterator::const_iterator(const QList<QItemSelectionRange>::iterator& o)```</span>.
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::Iterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_const_iterator_const_iterator2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QItemSelectionRange>::const_iterator::const_iterator(const QList<QItemSelectionRange>::const_iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::ConstIterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_const_iterator_const_iterator1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::ConstIterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current result.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QItemSelectionRange* QList<QItemSelectionRange>::const_iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current result.</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::Ptr<crate::QItemSelectionRange> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_const_iterator_operator__(
                    self as *const crate::q_list_of_q_item_selection_range::ConstIterator,
                );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_item_selection_range::Iterator {
        type Output = ::cpp_utils::MutRef<crate::QItemSelectionRange>;
        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QItemSelectionRange& QList<QItemSelectionRange>::iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>().</p></div>
        fn indirection(self) -> ::cpp_utils::MutRef<crate::QItemSelectionRange> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_iterator_operator_(
                    self as *const crate::q_list_of_q_item_selection_range::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::Iterator>>
        for crate::q_list_of_q_item_selection_range::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QItemSelectionRange>::iterator::operator==(const QList<QItemSelectionRange>::iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_iterator_operator__2(
                    self as *const crate::q_list_of_q_item_selection_range::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::Iterator>>
        for crate::q_list_of_q_item_selection_range::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QItemSelectionRange>::iterator::operator<(const QList<QItemSelectionRange>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_iterator_operator_1(
                    self as *const crate::q_list_of_q_item_selection_range::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::Iterator>>
        for crate::q_list_of_q_item_selection_range::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QItemSelectionRange>::iterator::operator<=(const QList<QItemSelectionRange>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_iterator_operator__4(
                    self as *const crate::q_list_of_q_item_selection_range::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::Iterator>>
        for crate::q_list_of_q_item_selection_range::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QItemSelectionRange>::iterator::operator>(const QList<QItemSelectionRange>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_iterator_operator_2(
                    self as *const crate::q_list_of_q_item_selection_range::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::Iterator>>
        for crate::q_list_of_q_item_selection_range::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QItemSelectionRange>::iterator::operator>=(const QList<QItemSelectionRange>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_iterator_operator__5(
                    self as *const crate::q_list_of_q_item_selection_range::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::std::cmp::PartialEq<
            ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::ConstIterator>,
        > for crate::q_list_of_q_item_selection_range::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QItemSelectionRange>::iterator::operator==(const QList<QItemSelectionRange>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_iterator_operator__6(
                    self as *const crate::q_list_of_q_item_selection_range::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Lt<
            ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::ConstIterator>,
        > for crate::q_list_of_q_item_selection_range::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QItemSelectionRange>::iterator::operator<(const QList<QItemSelectionRange>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_iterator_operator_3(
                    self as *const crate::q_list_of_q_item_selection_range::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Le<
            ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::ConstIterator>,
        > for crate::q_list_of_q_item_selection_range::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QItemSelectionRange>::iterator::operator<=(const QList<QItemSelectionRange>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_iterator_operator__8(
                    self as *const crate::q_list_of_q_item_selection_range::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Gt<
            ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::ConstIterator>,
        > for crate::q_list_of_q_item_selection_range::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QItemSelectionRange>::iterator::operator>(const QList<QItemSelectionRange>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_iterator_operator_4(
                    self as *const crate::q_list_of_q_item_selection_range::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Ge<
            ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::ConstIterator>,
        > for crate::q_list_of_q_item_selection_range::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QItemSelectionRange>::iterator::operator>=(const QList<QItemSelectionRange>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_iterator_operator__9(
                    self as *const crate::q_list_of_q_item_selection_range::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_item_selection_range::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_item_selection_range::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::iterator& QList<QItemSelectionRange>::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_item_selection_range::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_iterator_operator__10(
                    self as *mut crate::q_list_of_q_item_selection_range::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_item_selection_range::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_item_selection_range::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::iterator& QList<QItemSelectionRange>::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_item_selection_range::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_iterator_operator__12(
                    self as *mut crate::q_list_of_q_item_selection_range::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_item_selection_range::Iterator
    {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::iterator& QList<QItemSelectionRange>::iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_iterator_operator__14(
                    self as *mut crate::q_list_of_q_item_selection_range::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_item_selection_range::Iterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::iterator& QList<QItemSelectionRange>::iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_iterator_operator__15(
                    self as *mut crate::q_list_of_q_item_selection_range::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_item_selection_range::Iterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::iterator QList<QItemSelectionRange>::iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_iterator_operator_5(
                    self as *const crate::q_list_of_q_item_selection_range::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_item_selection_range::Iterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::iterator QList<QItemSelectionRange>::iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_iterator_operator_6(
                    self as *const crate::q_list_of_q_item_selection_range::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::Iterator>>
        for &crate::q_list_of_q_item_selection_range::Iterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QItemSelectionRange>::iterator::operator-(QList<QItemSelectionRange>::iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::Iterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_iterator_operator_7(
                    self as *const crate::q_list_of_q_item_selection_range::Iterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_item_selection_range::ConstIterator {
        type Output = ::cpp_utils::Ref<crate::QItemSelectionRange>;
        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QItemSelectionRange& QList<QItemSelectionRange>::const_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        fn indirection(self) -> ::cpp_utils::Ref<crate::QItemSelectionRange> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_const_iterator_operator_(
                    self as *const crate::q_list_of_q_item_selection_range::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
            }
        }
    }

    impl
        ::std::cmp::PartialEq<
            ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::ConstIterator>,
        > for crate::q_list_of_q_item_selection_range::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QItemSelectionRange>::const_iterator::operator==(const QList<QItemSelectionRange>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_const_iterator_operator__2(
                    self as *const crate::q_list_of_q_item_selection_range::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Lt<
            ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::ConstIterator>,
        > for crate::q_list_of_q_item_selection_range::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QItemSelectionRange>::const_iterator::operator<(const QList<QItemSelectionRange>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_const_iterator_operator_1(
                    self as *const crate::q_list_of_q_item_selection_range::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Le<
            ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::ConstIterator>,
        > for crate::q_list_of_q_item_selection_range::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QItemSelectionRange>::const_iterator::operator<=(const QList<QItemSelectionRange>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_const_iterator_operator__4(
                    self as *const crate::q_list_of_q_item_selection_range::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Gt<
            ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::ConstIterator>,
        > for crate::q_list_of_q_item_selection_range::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QItemSelectionRange>::const_iterator::operator>(const QList<QItemSelectionRange>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_const_iterator_operator_2(
                    self as *const crate::q_list_of_q_item_selection_range::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Ge<
            ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::ConstIterator>,
        > for crate::q_list_of_q_item_selection_range::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QItemSelectionRange>::const_iterator::operator>=(const QList<QItemSelectionRange>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_const_iterator_operator__5(
                    self as *const crate::q_list_of_q_item_selection_range::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_item_selection_range::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_item_selection_range::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::const_iterator& QList<QItemSelectionRange>::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(
            self,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_item_selection_range::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_const_iterator_operator__6(
                    self as *mut crate::q_list_of_q_item_selection_range::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_item_selection_range::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_item_selection_range::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::const_iterator& QList<QItemSelectionRange>::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(
            self,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_item_selection_range::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_const_iterator_operator__8(
                    self as *mut crate::q_list_of_q_item_selection_range::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_item_selection_range::ConstIterator
    {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::const_iterator& QList<QItemSelectionRange>::const_iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_const_iterator_operator__10(
                    self as *mut crate::q_list_of_q_item_selection_range::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_item_selection_range::ConstIterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::const_iterator& QList<QItemSelectionRange>::const_iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_const_iterator_operator__11(
                    self as *mut crate::q_list_of_q_item_selection_range::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_item_selection_range::ConstIterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::const_iterator QList<QItemSelectionRange>::const_iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_const_iterator_operator_3(
                    self as *const crate::q_list_of_q_item_selection_range::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_item_selection_range::ConstIterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::const_iterator QList<QItemSelectionRange>::const_iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_const_iterator_operator_4(
                    self as *const crate::q_list_of_q_item_selection_range::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::ConstIterator>>
        for &crate::q_list_of_q_item_selection_range::ConstIterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QItemSelectionRange>::const_iterator::operator-(QList<QItemSelectionRange>::const_iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::ConstIterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_const_iterator_operator_5(
                    self as *const crate::q_list_of_q_item_selection_range::ConstIterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_item_selection_range::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QItemSelectionRange>::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_iterator_diterator(
                self as *mut crate::q_list_of_q_item_selection_range::Iterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_item_selection_range::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QItemSelectionRange>::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_const_iterator_dconst_iterator(
                self as *mut crate::q_list_of_q_item_selection_range::ConstIterator,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
///
/// C++ class: <span style='color: green;'>```QList<QItemSelectionRange>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlist.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores items in a list that provides fast index-based access and index-based insertions and removals.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Internally, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T if <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. Otherwise, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T* and the items are allocated on the heap.</p>
/// <p>The array representation allows very fast insertions and index-based access. The <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() operations are also very fast because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates memory at both ends of its internal array. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.</p>
/// <p>Note, however, that when the conditions specified above are not met, each append or insert of a new item requires allocating the new item on the heap, and this per item allocation will make <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> a better choice for use cases that do a lot of appending or inserting, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> can allocate memory for many items in a single heap allocation.</p>
/// <p>Note that the internal array only ever gets bigger over the life of the list. It never shrinks. The internal array is deallocated by the destructor and by the assignment operator, when one list is assigned to another.</p>
/// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerList;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span><span class="operator">&gt;</span> dateList;
///
/// </pre>
/// <p>Qt includes a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class that inherits <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>&gt; and adds a few convenience functions, such as <a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#filter">QStringList::filter</a>(). <a href="http://doc.qt.io/qt-5/qstring.html#split">QString::split</a>() creates QStringLists from strings.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> stores a list of items. The default constructor creates an empty list. To insert items into the list, you can use operator&lt;&lt;():</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"one"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"two"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"three"</span>;
///   <span class="comment">// list: ["one", "two", "three"]</span>
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(). In addition, it provides the following convenience functions: <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const lists, operator[]() returns a reference to the item and can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Bob"</span>)
/// &#32;     list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Robert"</span>;
///
/// </pre>
/// <p>Because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented as an array of pointers for types that are larger than a pointer or are not movable, this operation requires (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>). For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> list<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (list<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Jane"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Jane at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>A common requirement is to remove an item from a list and do something with it. For this, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(). Here's a loop that removes the items from a list one at a time and calls <code>delete</code> on them:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   <span class="keyword">while</span> (<span class="operator">!</span>list<span class="operator">.</span>isEmpty())
/// &#32;     <span class="keyword">delete</span> list<span class="operator">.</span>takeFirst();
///
/// </pre>
/// <p>Inserting and removing items at either end of the list is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> in most cases), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
/// <p>If you want to find all occurrences of a particular value in a list, use <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of a matching item if they find it; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> list<span class="operator">.</span>indexOf(<span class="string">"Jane"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Jane is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a list contains a particular value, use <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the list, use <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>(). If you want to replace all occurrences of a particular value with another, use <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlistiterator.html">QListIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablelistiterator.html">QMutableListIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented in such a way that direct index-based access is just as fast as using iterators.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <p>To make <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as efficient as possible, its member functions don't validate their input before using it. Except for <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), member functions always assume the list is <i>not</i> empty. Member functions that take index values as parameters always assume their index value parameters are in the valid range. This means <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> member functions can fail. If you define QT_NO_DEBUG when you compile, failures will not be detected. If you <i>don't</i> define QT_NO_DEBUG, failures will be detected using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT">Q_ASSERT</a>() or <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT_X">Q_ASSERT_X</a>() with an appropriate message.</p>
/// <p>To avoid failures when your list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling other member functions. If you must pass an index value that might not be in the valid range, check that it is less than the value returned by <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() but <i>not</i> less than 0.</p>
/// <a name="more-members"></a></div>
#[repr(C)]
pub struct QListOfQItemSelectionRange {
    _unused: u8,
}
impl QListOfQItemSelectionRange {
    /// <p>Inserts <i>value</i> at the end of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QItemSelectionRange>::append(const QItemSelectionRange& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["one", "two", "three"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn append_q_item_selection_range(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelectionRange>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_append(
            self as *mut crate::QListOfQItemSelectionRange,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelectionRange>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QItemSelectionRange>::append(const QList<QItemSelectionRange>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> list to this list.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_list_of_q_item_selection_range(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQItemSelectionRange>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_append1(self as *mut crate::QListOfQItemSelectionRange, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQItemSelectionRange>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QItemSelectionRange& QList<QItemSelectionRange>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>This function is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ref<crate::QItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_at(
            self as *const crate::QListOfQItemSelectionRange,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QItemSelectionRange& QList<QItemSelectionRange>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::MutRef<crate::QItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_back(
            self as *mut crate::QListOfQItemSelectionRange,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QItemSelectionRange& QList<QItemSelectionRange>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<crate::QItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_back1(
            self as *const crate::QListOfQItemSelectionRange,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::const_iterator QList<QItemSelectionRange>::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_cbegin(
            self as *const crate::QListOfQItemSelectionRange,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::const_iterator QList<QItemSelectionRange>::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn cend(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_cend(
            self as *const crate::QListOfQItemSelectionRange,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all items from the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QItemSelectionRange>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all items from the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_clear(
            self as *mut crate::QListOfQItemSelectionRange,
        )
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::const_iterator QList<QItemSelectionRange>::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_constBegin(
            self as *const crate::QListOfQItemSelectionRange,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::const_iterator QList<QItemSelectionRange>::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn const_end(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_constEnd(
            self as *const crate::QListOfQItemSelectionRange,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QItemSelectionRange& QList<QItemSelectionRange>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>().</p></div>
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<crate::QItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_constFirst(
            self as *const crate::QListOfQItemSelectionRange,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QItemSelectionRange& QList<QItemSelectionRange>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>().</p></div>
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<crate::QItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_constLast(
            self as *const crate::QListOfQItemSelectionRange,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QItemSelectionRange>::contains(const QItemSelectionRange& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn contains(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelectionRange>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_contains(
            self as *const crate::QListOfQItemSelectionRange,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelectionRange>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>& QList<QItemSelectionRange>::operator=(const QList<QItemSelectionRange>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this list and returns a reference to this list.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQItemSelectionRange>>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_operator_(self as *mut crate::QListOfQItemSelectionRange, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQItemSelectionRange>>::cast_into(l).as_raw_ptr());
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of occurrences of <i>value</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QItemSelectionRange>::count(const QItemSelectionRange& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of occurrences of <i>value</i> in the list.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelectionRange>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_count(
            self as *const crate::QListOfQItemSelectionRange,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelectionRange>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QItemSelectionRange>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_count1(
            self as *const crate::QListOfQItemSelectionRange,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QItemSelectionRange>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_detach(
            self as *mut crate::QListOfQItemSelectionRange,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QItemSelectionRange>::detachShared()```</span>.
    pub unsafe fn detach_shared(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_detachShared(
            self as *mut crate::QListOfQItemSelectionRange,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QItemSelectionRange>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_empty(
            self as *const crate::QListOfQItemSelectionRange,
        )
    }

    /// <p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QItemSelectionRange>::endsWith(const QItemSelectionRange& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn ends_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelectionRange>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_endsWith(
            self as *const crate::QListOfQItemSelectionRange,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelectionRange>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::iterator QList<QItemSelectionRange>::erase(QList<QItemSelectionRange>::iterator pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn erase_1a(
        &mut self,
        pos: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::Iterator>,
        >,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_erase(
            self as *mut crate::QListOfQItemSelectionRange,
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::Iterator>,
            >::cast_into(pos)
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::iterator QList<QItemSelectionRange>::erase(QList<QItemSelectionRange>::iterator first, QList<QItemSelectionRange>::iterator last)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes all the items from <i>begin</i> up to (but not including) <i>end</i>. Returns an iterator to the same item that <i>end</i> referred to before the call.</p></div>
    pub unsafe fn erase_2a(
        &mut self,
        first: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::Iterator>,
        >,
        last: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::Iterator>,
        >,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_erase1(
            self as *mut crate::QListOfQItemSelectionRange,
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::Iterator>,
            >::cast_into(first)
            .as_raw_ptr(),
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::Iterator>,
            >::cast_into(last)
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QItemSelectionRange& QList<QItemSelectionRange>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<crate::QItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_first(
            self as *mut crate::QListOfQItemSelectionRange,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QItemSelectionRange& QList<QItemSelectionRange>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::QItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_first1(
            self as *const crate::QListOfQItemSelectionRange,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QItemSelectionRange& QList<QItemSelectionRange>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::MutRef<crate::QItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_front(
            self as *mut crate::QListOfQItemSelectionRange,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QItemSelectionRange& QList<QItemSelectionRange>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<crate::QItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_front1(
            self as *const crate::QListOfQItemSelectionRange,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QItemSelectionRange& QList<QItemSelectionRange>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>(). This function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p></div>
    pub unsafe fn index(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ref<crate::QItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_operator__2(
            self as *const crate::QListOfQItemSelectionRange,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QItemSelectionRange& QList<QItemSelectionRange>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If this function is called on a list that is currently being shared, it will trigger a copy of all elements. Otherwise, this function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>. If you do not want to modify the list you should use <a href="http://doc.qt.io/qt-5/qlist.html#at">QList::at</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_operator__3(
            self as *mut crate::QListOfQItemSelectionRange,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QItemSelectionRange>::indexOf(const QItemSelectionRange& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelectionRange>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_indexOf(
            self as *const crate::QListOfQItemSelectionRange,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelectionRange>>::cast_into(t)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QItemSelectionRange>::indexOf(const QItemSelectionRange& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelectionRange>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_indexOf1(
            self as *const crate::QListOfQItemSelectionRange,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelectionRange>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QItemSelectionRange>::insert(int i, const QItemSelectionRange& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   list<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// list: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn insert_int_q_item_selection_range(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelectionRange>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_insert(
            self as *mut crate::QListOfQItemSelectionRange,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelectionRange>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::iterator QList<QItemSelectionRange>::insert(QList<QItemSelectionRange>::iterator before, const QItemSelectionRange& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>value</i> in front of the item pointed to by the iterator <i>before</i>. Returns an iterator pointing at the inserted item. Note that the iterator passed to the function will be invalid after the call; the returned iterator should be used instead.</p></div>
    pub unsafe fn insert_iterator_q_item_selection_range(
        &mut self,
        before: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::Iterator>,
        >,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelectionRange>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_insert1(
            self as *mut crate::QListOfQItemSelectionRange,
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::q_list_of_q_item_selection_range::Iterator>,
            >::cast_into(before)
            .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelectionRange>>::cast_into(t)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QItemSelectionRange>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_isDetached(
            self as *const crate::QListOfQItemSelectionRange,
        )
    }

    /// <p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QItemSelectionRange>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_isEmpty(
            self as *const crate::QListOfQItemSelectionRange,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QItemSelectionRange>::isSharedWith(const QList<QItemSelectionRange>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQItemSelectionRange>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_isSharedWith(self as *const crate::QListOfQItemSelectionRange, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQItemSelectionRange>>::cast_into(other).as_raw_ptr())
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QItemSelectionRange& QList<QItemSelectionRange>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<crate::QItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_last(
            self as *mut crate::QListOfQItemSelectionRange,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QItemSelectionRange& QList<QItemSelectionRange>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::QItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_last1(
            self as *const crate::QListOfQItemSelectionRange,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QItemSelectionRange>::lastIndexOf(const QItemSelectionRange& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelectionRange>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_lastIndexOf(
            self as *const crate::QListOfQItemSelectionRange,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelectionRange>>::cast_into(t)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QItemSelectionRange>::lastIndexOf(const QItemSelectionRange& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelectionRange>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_lastIndexOf1(
            self as *const crate::QListOfQItemSelectionRange,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelectionRange>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QItemSelectionRange>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_length(
            self as *const crate::QListOfQItemSelectionRange,
        )
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange> QList<QItemSelectionRange>::mid(int pos, int length = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        length: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_mid(
            self as *const crate::QListOfQItemSelectionRange,
            pos,
            length,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange> QList<QItemSelectionRange>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_mid1(
            self as *const crate::QListOfQItemSelectionRange,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QItemSelectionRange>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>move(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "C", "D", "E", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This is the same as insert(<i>to</i>, <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<i>from</i>)).This function assumes that both <i>from</i> and <i>to</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>from</i> and <i>to</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_move(
            self as *mut crate::QListOfQItemSelectionRange,
            from,
            to,
        )
    }

    /// <p>Constructs an empty list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QItemSelectionRange>::QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty list.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QListOfQItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_QList();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QItemSelectionRange>::QList(const QList<QItemSelectionRange>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQItemSelectionRange>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_QList1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQItemSelectionRange>>::cast_into(l).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QItemSelectionRange>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_pop_back(
            self as *mut crate::QListOfQItemSelectionRange,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QItemSelectionRange>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_pop_front(
            self as *mut crate::QListOfQItemSelectionRange,
        )
    }

    /// <p>Inserts <i>value</i> at the beginning of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QItemSelectionRange>::prepend(const QItemSelectionRange& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(0, <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelectionRange>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_prepend(
            self as *mut crate::QListOfQItemSelectionRange,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelectionRange>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QItemSelectionRange>::push_back(const QItemSelectionRange& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelectionRange>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_push_back(
            self as *mut crate::QListOfQItemSelectionRange,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelectionRange>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QItemSelectionRange>::push_front(const QItemSelectionRange& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelectionRange>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_push_front(
            self as *mut crate::QListOfQItemSelectionRange,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelectionRange>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QItemSelectionRange>::removeAll(const QItemSelectionRange& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeAll(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_all(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelectionRange>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_removeAll(
            self as *mut crate::QListOfQItemSelectionRange,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelectionRange>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QItemSelectionRange>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_removeAt(
            self as *mut crate::QListOfQItemSelectionRange,
            i,
        )
    }

    /// <p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QItemSelectionRange>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_removeFirst(
            self as *mut crate::QListOfQItemSelectionRange,
        )
    }

    /// <p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QItemSelectionRange>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_removeLast(
            self as *mut crate::QListOfQItemSelectionRange,
        )
    }

    /// <p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QItemSelectionRange>::removeOne(const QItemSelectionRange& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeOne(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", ,"sun", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_one(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelectionRange>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_removeOne(
            self as *mut crate::QListOfQItemSelectionRange,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelectionRange>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QItemSelectionRange>::replace(int i, const QItemSelectionRange& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelectionRange>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_replace(
            self as *mut crate::QListOfQItemSelectionRange,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelectionRange>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Reserve space for <i>alloc</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QItemSelectionRange>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reserve space for <i>alloc</i> elements.</p>
    /// <p>If <i>alloc</i> is smaller than the current size of the list, nothing will happen.</p>
    /// <p>Use this function to avoid repetetive reallocation of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s internal data if you can predict how many elements will be appended. Note that the reservation applies only to the internal pointer array.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_reserve(
            self as *mut crate::QListOfQItemSelectionRange,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QItemSelectionRange>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_setSharable(
            self as *mut crate::QListOfQItemSelectionRange,
            sharable,
        )
    }

    /// <p>Returns the number of items in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QItemSelectionRange>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_size(
            self as *const crate::QListOfQItemSelectionRange,
        )
    }

    /// <p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QItemSelectionRange>::startsWith(const QItemSelectionRange& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn starts_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelectionRange>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_startsWith(
            self as *const crate::QListOfQItemSelectionRange,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelectionRange>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QItemSelectionRange>::swap(QList<QItemSelectionRange>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap_1a(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QListOfQItemSelectionRange>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_swap(self as *mut crate::QListOfQItemSelectionRange, ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QListOfQItemSelectionRange>>::cast_into(other).as_mut_raw_ptr())
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QItemSelectionRange>::swap(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swap(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    pub unsafe fn swap_2a(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_swap1(
            self as *mut crate::QListOfQItemSelectionRange,
            i,
            j,
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QItemSelectionRange>::swapItemsAt(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swapItemsAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swapItemsAt(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn swap_items_at(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_swapItemsAt(
            self as *mut crate::QListOfQItemSelectionRange,
            i,
            j,
        )
    }

    /// <p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QItemSelectionRange QList<QItemSelectionRange>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_takeAt(
            self as *mut crate::QListOfQItemSelectionRange,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QItemSelectionRange QList<QItemSelectionRange>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> ::cpp_utils::CppBox<crate::QItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_takeFirst(
            self as *mut crate::QListOfQItemSelectionRange,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QItemSelectionRange QList<QItemSelectionRange>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> ::cpp_utils::CppBox<crate::QItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_takeLast(
            self as *mut crate::QListOfQItemSelectionRange,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value at index position <i>i</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QItemSelectionRange QList<QItemSelectionRange>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the list.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that the index is going to be within bounds, you can use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_value(
            self as *const crate::QListOfQItemSelectionRange,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QItemSelectionRange QList<QItemSelectionRange>::value(int i, const QItemSelectionRange& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelectionRange>>,
    ) -> ::cpp_utils::CppBox<crate::QItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_value1(
            self as *const crate::QListOfQItemSelectionRange,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelectionRange>>::cast_into(
                default_value,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_list_of_q_object {
    //! C++ type: <span style='color: green;'>```QList<QObject*>```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QObject*>::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::iterator& QList<QObject*>::iterator::operator=(const QList<QObject*>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_object::Iterator>>,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_object::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_iterator_operator_8(self as *mut crate::q_list_of_q_object::Iterator, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_object::Iterator>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::iterator QList<QObject*>::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_object::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_iterator_operator__13(
                self as *mut crate::q_list_of_q_object::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::iterator QList<QObject*>::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_object::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_iterator_operator__11(
                self as *mut crate::q_list_of_q_object::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QObject*& QList<QObject*>::iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValueRef iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>. If you assign to it, the assignment will apply to the character in the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> of <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> from which you got the reference.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::MutRef<*mut crate::QObject> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_iterator_operator__1(
                self as *const crate::q_list_of_q_object::Iterator,
                j,
            );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QObject*>::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>().</p></div>
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_list_of_q_object::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QObject*>::iterator::iterator(const QList<QObject*>::iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_object::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_object::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_iterator_iterator1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_object::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QObject** QList<QObject*>::iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's value.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::MutPtr<*mut crate::QObject> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_iterator_operator__(
                self as *const crate::q_list_of_q_object::Iterator,
            );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QObject*>::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::const_iterator& QList<QObject*>::const_iterator::operator=(const QList<QObject*>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_object::ConstIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_object::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_const_iterator_operator_6(
                    self as *mut crate::q_list_of_q_object::ConstIterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_object::ConstIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::const_iterator QList<QObject*>::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_object::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_const_iterator_operator__9(
                    self as *mut crate::q_list_of_q_object::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::const_iterator QList<QObject*>::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_object::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_const_iterator_operator__7(
                    self as *mut crate::q_list_of_q_object::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QObject*>::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>().</p></div>
        pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::q_list_of_q_object::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_const_iterator_const_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QObject*>::const_iterator::const_iterator(const QList<QObject*>::iterator& o)```</span>.
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_object::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_object::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_const_iterator_const_iterator2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_object::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QObject*>::const_iterator::const_iterator(const QList<QObject*>::const_iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_object::ConstIterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_object::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_const_iterator_const_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_object::ConstIterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_object::Iterator {
        type Output = ::cpp_utils::MutRef<*mut crate::QObject>;
        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QObject*& QList<QObject*>::iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>().</p></div>
        fn indirection(self) -> ::cpp_utils::MutRef<*mut crate::QObject> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_iterator_operator_(
                    self as *const crate::q_list_of_q_object::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_object::Iterator>>
        for crate::q_list_of_q_object::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QObject*>::iterator::operator==(const QList<QObject*>::iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_object::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_iterator_operator__2(
                    self as *const crate::q_list_of_q_object::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_object::Iterator>>
        for crate::q_list_of_q_object::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QObject*>::iterator::operator<(const QList<QObject*>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_object::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_iterator_operator_1(
                    self as *const crate::q_list_of_q_object::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_object::Iterator>>
        for crate::q_list_of_q_object::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QObject*>::iterator::operator<=(const QList<QObject*>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_object::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_iterator_operator__4(
                    self as *const crate::q_list_of_q_object::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_object::Iterator>>
        for crate::q_list_of_q_object::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QObject*>::iterator::operator>(const QList<QObject*>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_object::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_iterator_operator_2(
                    self as *const crate::q_list_of_q_object::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_object::Iterator>>
        for crate::q_list_of_q_object::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QObject*>::iterator::operator>=(const QList<QObject*>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_object::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_iterator_operator__5(
                    self as *const crate::q_list_of_q_object::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_object::ConstIterator>>
        for crate::q_list_of_q_object::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QObject*>::iterator::operator==(const QList<QObject*>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_object::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_iterator_operator__6(
                    self as *const crate::q_list_of_q_object::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_object::ConstIterator>>
        for crate::q_list_of_q_object::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QObject*>::iterator::operator<(const QList<QObject*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_object::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_iterator_operator_3(
                    self as *const crate::q_list_of_q_object::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_object::ConstIterator>>
        for crate::q_list_of_q_object::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QObject*>::iterator::operator<=(const QList<QObject*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_object::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_iterator_operator__8(
                    self as *const crate::q_list_of_q_object::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_object::ConstIterator>>
        for crate::q_list_of_q_object::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QObject*>::iterator::operator>(const QList<QObject*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_object::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_iterator_operator_4(
                    self as *const crate::q_list_of_q_object::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_object::ConstIterator>>
        for crate::q_list_of_q_object::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QObject*>::iterator::operator>=(const QList<QObject*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_object::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_iterator_operator__9(
                    self as *const crate::q_list_of_q_object::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_object::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_object::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::iterator& QList<QObject*>::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_object::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_iterator_operator__10(
                    self as *mut crate::q_list_of_q_object::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_object::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_object::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::iterator& QList<QObject*>::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_object::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_iterator_operator__12(
                    self as *mut crate::q_list_of_q_object::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong> for crate::q_list_of_q_object::Iterator {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::iterator& QList<QObject*>::iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_iterator_operator__14(
                    self as *mut crate::q_list_of_q_object::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong> for crate::q_list_of_q_object::Iterator {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::iterator& QList<QObject*>::iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_iterator_operator__15(
                    self as *mut crate::q_list_of_q_object::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_list_of_q_object::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_object::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::iterator QList<QObject*>::iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_object::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_iterator_operator_5(
                    self as *const crate::q_list_of_q_object::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_list_of_q_object::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_object::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::iterator QList<QObject*>::iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_object::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_iterator_operator_6(
                    self as *const crate::q_list_of_q_object::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_object::Iterator>>
        for &crate::q_list_of_q_object::Iterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QObject*>::iterator::operator-(QList<QObject*>::iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_object::Iterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_iterator_operator_7(
                    self as *const crate::q_list_of_q_object::Iterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_object::ConstIterator>>
        for crate::q_list_of_q_object::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QObject*>::const_iterator::operator==(const QList<QObject*>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_object::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_const_iterator_operator__2(
                    self as *const crate::q_list_of_q_object::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_object::ConstIterator>>
        for crate::q_list_of_q_object::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QObject*>::const_iterator::operator<(const QList<QObject*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_object::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_const_iterator_operator_1(
                    self as *const crate::q_list_of_q_object::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_object::ConstIterator>>
        for crate::q_list_of_q_object::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QObject*>::const_iterator::operator<=(const QList<QObject*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_object::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_const_iterator_operator__4(
                    self as *const crate::q_list_of_q_object::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_object::ConstIterator>>
        for crate::q_list_of_q_object::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QObject*>::const_iterator::operator>(const QList<QObject*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_object::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_const_iterator_operator_2(
                    self as *const crate::q_list_of_q_object::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_object::ConstIterator>>
        for crate::q_list_of_q_object::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QObject*>::const_iterator::operator>=(const QList<QObject*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_object::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_const_iterator_operator__5(
                    self as *const crate::q_list_of_q_object::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_object::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_object::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::const_iterator& QList<QObject*>::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_object::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_const_iterator_operator__6(
                    self as *mut crate::q_list_of_q_object::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_object::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_object::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::const_iterator& QList<QObject*>::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_object::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_const_iterator_operator__8(
                    self as *mut crate::q_list_of_q_object::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_object::ConstIterator
    {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::const_iterator& QList<QObject*>::const_iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_const_iterator_operator__10(
                    self as *mut crate::q_list_of_q_object::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_object::ConstIterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::const_iterator& QList<QObject*>::const_iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_const_iterator_operator__11(
                    self as *mut crate::q_list_of_q_object::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_list_of_q_object::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_object::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::const_iterator QList<QObject*>::const_iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_object::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_const_iterator_operator_3(
                    self as *const crate::q_list_of_q_object::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_list_of_q_object::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_object::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::const_iterator QList<QObject*>::const_iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_object::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_const_iterator_operator_4(
                    self as *const crate::q_list_of_q_object::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_object::ConstIterator>>
        for &crate::q_list_of_q_object::ConstIterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QObject*>::const_iterator::operator-(QList<QObject*>::const_iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_object::ConstIterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_const_iterator_operator_5(
                    self as *const crate::q_list_of_q_object::ConstIterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_object::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QObject*>::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_iterator_diterator(
                self as *mut crate::q_list_of_q_object::Iterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_object::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QObject*>::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_const_iterator_dconst_iterator(
                self as *mut crate::q_list_of_q_object::ConstIterator,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
///
/// C++ class: <span style='color: green;'>```QList<QObject*>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlist.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores items in a list that provides fast index-based access and index-based insertions and removals.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Internally, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T if <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. Otherwise, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T* and the items are allocated on the heap.</p>
/// <p>The array representation allows very fast insertions and index-based access. The <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() operations are also very fast because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates memory at both ends of its internal array. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.</p>
/// <p>Note, however, that when the conditions specified above are not met, each append or insert of a new item requires allocating the new item on the heap, and this per item allocation will make <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> a better choice for use cases that do a lot of appending or inserting, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> can allocate memory for many items in a single heap allocation.</p>
/// <p>Note that the internal array only ever gets bigger over the life of the list. It never shrinks. The internal array is deallocated by the destructor and by the assignment operator, when one list is assigned to another.</p>
/// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerList;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span><span class="operator">&gt;</span> dateList;
///
/// </pre>
/// <p>Qt includes a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class that inherits <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>&gt; and adds a few convenience functions, such as <a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#filter">QStringList::filter</a>(). <a href="http://doc.qt.io/qt-5/qstring.html#split">QString::split</a>() creates QStringLists from strings.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> stores a list of items. The default constructor creates an empty list. To insert items into the list, you can use operator&lt;&lt;():</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"one"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"two"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"three"</span>;
///   <span class="comment">// list: ["one", "two", "three"]</span>
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(). In addition, it provides the following convenience functions: <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const lists, operator[]() returns a reference to the item and can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Bob"</span>)
/// &#32;     list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Robert"</span>;
///
/// </pre>
/// <p>Because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented as an array of pointers for types that are larger than a pointer or are not movable, this operation requires (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>). For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> list<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (list<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Jane"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Jane at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>A common requirement is to remove an item from a list and do something with it. For this, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(). Here's a loop that removes the items from a list one at a time and calls <code>delete</code> on them:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   <span class="keyword">while</span> (<span class="operator">!</span>list<span class="operator">.</span>isEmpty())
/// &#32;     <span class="keyword">delete</span> list<span class="operator">.</span>takeFirst();
///
/// </pre>
/// <p>Inserting and removing items at either end of the list is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> in most cases), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
/// <p>If you want to find all occurrences of a particular value in a list, use <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of a matching item if they find it; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> list<span class="operator">.</span>indexOf(<span class="string">"Jane"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Jane is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a list contains a particular value, use <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the list, use <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>(). If you want to replace all occurrences of a particular value with another, use <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlistiterator.html">QListIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablelistiterator.html">QMutableListIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented in such a way that direct index-based access is just as fast as using iterators.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <p>To make <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as efficient as possible, its member functions don't validate their input before using it. Except for <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), member functions always assume the list is <i>not</i> empty. Member functions that take index values as parameters always assume their index value parameters are in the valid range. This means <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> member functions can fail. If you define QT_NO_DEBUG when you compile, failures will not be detected. If you <i>don't</i> define QT_NO_DEBUG, failures will be detected using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT">Q_ASSERT</a>() or <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT_X">Q_ASSERT_X</a>() with an appropriate message.</p>
/// <p>To avoid failures when your list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling other member functions. If you must pass an index value that might not be in the valid range, check that it is less than the value returned by <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() but <i>not</i> less than 0.</p>
/// <a name="more-members"></a></div>
#[repr(C)]
pub struct QListOfQObject {
    _unused: u8,
}
impl QListOfQObject {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QObject*>::append(const QList<QObject*>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> list to this list.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQObject>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_append1(
            self as *mut crate::QListOfQObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQObject>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QObject*& QList<QObject*>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn back(&mut self) -> ::cpp_utils::MutRef<*mut crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_back(
            self as *mut crate::QListOfQObject,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::const_iterator QList<QObject*>::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(&self) -> ::cpp_utils::CppBox<crate::q_list_of_q_object::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_cbegin(
            self as *const crate::QListOfQObject,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::const_iterator QList<QObject*>::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn cend(&self) -> ::cpp_utils::CppBox<crate::q_list_of_q_object::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_cend(
            self as *const crate::QListOfQObject,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all items from the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QObject*>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all items from the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_clear(self as *mut crate::QListOfQObject)
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::const_iterator QList<QObject*>::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_object::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_constBegin(
            self as *const crate::QListOfQObject,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::const_iterator QList<QObject*>::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn const_end(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_object::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_constEnd(
            self as *const crate::QListOfQObject,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Assigns <i>other</i> to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QObject*>& QList<QObject*>::operator=(const QList<QObject*>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this list and returns a reference to this list.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQObject>>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_operator_(
            self as *mut crate::QListOfQObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQObject>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QObject*>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_count1(self as *const crate::QListOfQObject)
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QObject*>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_detach(self as *mut crate::QListOfQObject)
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QObject*>::detachShared()```</span>.
    pub unsafe fn detach_shared(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_detachShared(
            self as *mut crate::QListOfQObject,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QObject*>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_empty(self as *const crate::QListOfQObject)
    }

    /// <p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::iterator QList<QObject*>::erase(QList<QObject*>::iterator pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn erase_1a(
        &mut self,
        pos: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_object::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_object::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_erase(self as *mut crate::QListOfQObject, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_object::Iterator>>::cast_into(pos).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::iterator QList<QObject*>::erase(QList<QObject*>::iterator first, QList<QObject*>::iterator last)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes all the items from <i>begin</i> up to (but not including) <i>end</i>. Returns an iterator to the same item that <i>end</i> referred to before the call.</p></div>
    pub unsafe fn erase_2a(
        &mut self,
        first: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_object::Iterator>>,
        last: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_object::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_object::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_erase1(self as *mut crate::QListOfQObject, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_object::Iterator>>::cast_into(first).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_object::Iterator>>::cast_into(last).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QObject*& QList<QObject*>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn first(&mut self) -> ::cpp_utils::MutRef<*mut crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_first(
            self as *mut crate::QListOfQObject,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QObject*& QList<QObject*>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn front(&mut self) -> ::cpp_utils::MutRef<*mut crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_front(
            self as *mut crate::QListOfQObject,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QObject*& QList<QObject*>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If this function is called on a list that is currently being shared, it will trigger a copy of all elements. Otherwise, this function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>. If you do not want to modify the list you should use <a href="http://doc.qt.io/qt-5/qlist.html#at">QList::at</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>().</p></div>
    pub unsafe fn index(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<*mut crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_operator__3(
            self as *mut crate::QListOfQObject,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QObject*>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_isDetached(
            self as *const crate::QListOfQObject,
        )
    }

    /// <p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QObject*>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_isEmpty(
            self as *const crate::QListOfQObject,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QObject*>::isSharedWith(const QList<QObject*>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQObject>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_isSharedWith(
            self as *const crate::QListOfQObject,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQObject>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QObject*& QList<QObject*>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn last(&mut self) -> ::cpp_utils::MutRef<*mut crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_last(
            self as *mut crate::QListOfQObject,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QObject*>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_length(self as *const crate::QListOfQObject)
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QObject*> QList<QObject*>::mid(int pos, int length = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        length: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_mid(
            self as *const crate::QListOfQObject,
            pos,
            length,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QObject*> QList<QObject*>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_mid1(
            self as *const crate::QListOfQObject,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QObject*>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>move(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "C", "D", "E", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This is the same as insert(<i>to</i>, <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<i>from</i>)).This function assumes that both <i>from</i> and <i>to</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>from</i> and <i>to</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_move(
            self as *mut crate::QListOfQObject,
            from,
            to,
        )
    }

    /// <p>Constructs an empty list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QObject*>::QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty list.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QListOfQObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_QList();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QObject*>::QList(const QList<QObject*>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQObject>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_QList1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQObject>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QObject*>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_pop_back(self as *mut crate::QListOfQObject)
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QObject*>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_pop_front(
            self as *mut crate::QListOfQObject,
        )
    }

    /// <p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QObject*>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_removeAt(
            self as *mut crate::QListOfQObject,
            i,
        )
    }

    /// <p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QObject*>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_removeFirst(
            self as *mut crate::QListOfQObject,
        )
    }

    /// <p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QObject*>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_removeLast(
            self as *mut crate::QListOfQObject,
        )
    }

    /// <p>Reserve space for <i>alloc</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QObject*>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reserve space for <i>alloc</i> elements.</p>
    /// <p>If <i>alloc</i> is smaller than the current size of the list, nothing will happen.</p>
    /// <p>Use this function to avoid repetetive reallocation of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s internal data if you can predict how many elements will be appended. Note that the reservation applies only to the internal pointer array.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_reserve(
            self as *mut crate::QListOfQObject,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QObject*>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_setSharable(
            self as *mut crate::QListOfQObject,
            sharable,
        )
    }

    /// <p>Returns the number of items in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QObject*>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_size(self as *const crate::QListOfQObject)
    }

    /// <p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QObject*>::swap(QList<QObject*>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap_1a(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QListOfQObject>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_swap(
            self as *mut crate::QListOfQObject,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QListOfQObject>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QObject*>::swap(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swap(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    pub unsafe fn swap_2a(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_swap1(
            self as *mut crate::QListOfQObject,
            i,
            j,
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QObject*>::swapItemsAt(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swapItemsAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swapItemsAt(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn swap_items_at(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_swapItemsAt(
            self as *mut crate::QListOfQObject,
            i,
            j,
        )
    }

    /// <p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QObject* QList<QObject*>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_takeAt(
            self as *mut crate::QListOfQObject,
            i,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QObject* QList<QObject*>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_takeFirst(
            self as *mut crate::QListOfQObject,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QObject* QList<QObject*>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_takeLast(
            self as *mut crate::QListOfQObject,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns the value at index position <i>i</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QObject* QList<QObject*>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the list.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that the index is going to be within bounds, you can use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_value(
            self as *const crate::QListOfQObject,
            i,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

pub mod q_list_of_q_variant {
    //! C++ type: <span style='color: green;'>```QList<QVariant>```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QVariant>::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::iterator& QList<QVariant>::iterator::operator=(const QList<QVariant>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_variant::Iterator>>,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_variant::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_iterator_operator_8(self as *mut crate::q_list_of_q_variant::Iterator, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_variant::Iterator>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::iterator QList<QVariant>::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_variant::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_iterator_operator__13(
                self as *mut crate::q_list_of_q_variant::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::iterator QList<QVariant>::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_variant::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_iterator_operator__11(
                self as *mut crate::q_list_of_q_variant::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QVariant& QList<QVariant>::iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValueRef iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>. If you assign to it, the assignment will apply to the character in the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> of <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> from which you got the reference.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::MutRef<crate::QVariant> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_iterator_operator__1(
                self as *const crate::q_list_of_q_variant::Iterator,
                j,
            );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QVariant>::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>().</p></div>
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_list_of_q_variant::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QVariant>::iterator::iterator(const QList<QVariant>::iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_variant::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_variant::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_iterator_iterator1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_variant::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QVariant* QList<QVariant>::iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's value.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::MutPtr<crate::QVariant> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_iterator_operator__(
                self as *const crate::q_list_of_q_variant::Iterator,
            );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QVariant>::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::const_iterator& QList<QVariant>::const_iterator::operator=(const QList<QVariant>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_variant::ConstIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_variant::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_const_iterator_operator_6(
                    self as *mut crate::q_list_of_q_variant::ConstIterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_variant::ConstIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::const_iterator QList<QVariant>::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_variant::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_const_iterator_operator__9(
                    self as *mut crate::q_list_of_q_variant::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::const_iterator QList<QVariant>::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_variant::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_const_iterator_operator__7(
                    self as *mut crate::q_list_of_q_variant::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QVariant& QList<QVariant>::const_iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValue const_iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::Ref<crate::QVariant> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_const_iterator_operator__1(
                    self as *const crate::q_list_of_q_variant::ConstIterator,
                    j,
                );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QVariant>::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>().</p></div>
        pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::q_list_of_q_variant::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_const_iterator_const_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QVariant>::const_iterator::const_iterator(const QList<QVariant>::iterator& o)```</span>.
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_variant::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_variant::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_const_iterator_const_iterator2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_variant::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QVariant>::const_iterator::const_iterator(const QList<QVariant>::const_iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_variant::ConstIterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_variant::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_const_iterator_const_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_variant::ConstIterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current result.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QVariant* QList<QVariant>::const_iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current result.</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::Ptr<crate::QVariant> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_const_iterator_operator__(
                self as *const crate::q_list_of_q_variant::ConstIterator,
            );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_variant::Iterator {
        type Output = ::cpp_utils::MutRef<crate::QVariant>;
        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QVariant& QList<QVariant>::iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>().</p></div>
        fn indirection(self) -> ::cpp_utils::MutRef<crate::QVariant> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_iterator_operator_(
                    self as *const crate::q_list_of_q_variant::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_variant::Iterator>>
        for crate::q_list_of_q_variant::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QVariant>::iterator::operator==(const QList<QVariant>::iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_variant::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_iterator_operator__2(
                    self as *const crate::q_list_of_q_variant::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_variant::Iterator>>
        for crate::q_list_of_q_variant::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QVariant>::iterator::operator<(const QList<QVariant>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_variant::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_iterator_operator_1(
                    self as *const crate::q_list_of_q_variant::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_variant::Iterator>>
        for crate::q_list_of_q_variant::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QVariant>::iterator::operator<=(const QList<QVariant>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_variant::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_iterator_operator__4(
                    self as *const crate::q_list_of_q_variant::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_variant::Iterator>>
        for crate::q_list_of_q_variant::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QVariant>::iterator::operator>(const QList<QVariant>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_variant::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_iterator_operator_2(
                    self as *const crate::q_list_of_q_variant::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_variant::Iterator>>
        for crate::q_list_of_q_variant::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QVariant>::iterator::operator>=(const QList<QVariant>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_variant::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_iterator_operator__5(
                    self as *const crate::q_list_of_q_variant::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_variant::ConstIterator>>
        for crate::q_list_of_q_variant::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QVariant>::iterator::operator==(const QList<QVariant>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_variant::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_iterator_operator__6(
                    self as *const crate::q_list_of_q_variant::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_variant::ConstIterator>>
        for crate::q_list_of_q_variant::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QVariant>::iterator::operator<(const QList<QVariant>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_variant::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_iterator_operator_3(
                    self as *const crate::q_list_of_q_variant::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_variant::ConstIterator>>
        for crate::q_list_of_q_variant::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QVariant>::iterator::operator<=(const QList<QVariant>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_variant::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_iterator_operator__8(
                    self as *const crate::q_list_of_q_variant::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_variant::ConstIterator>>
        for crate::q_list_of_q_variant::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QVariant>::iterator::operator>(const QList<QVariant>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_variant::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_iterator_operator_4(
                    self as *const crate::q_list_of_q_variant::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_variant::ConstIterator>>
        for crate::q_list_of_q_variant::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QVariant>::iterator::operator>=(const QList<QVariant>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_variant::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_iterator_operator__9(
                    self as *const crate::q_list_of_q_variant::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_variant::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_variant::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::iterator& QList<QVariant>::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_variant::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_iterator_operator__10(
                    self as *mut crate::q_list_of_q_variant::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_variant::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_variant::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::iterator& QList<QVariant>::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_variant::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_iterator_operator__12(
                    self as *mut crate::q_list_of_q_variant::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong> for crate::q_list_of_q_variant::Iterator {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::iterator& QList<QVariant>::iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_iterator_operator__14(
                    self as *mut crate::q_list_of_q_variant::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong> for crate::q_list_of_q_variant::Iterator {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::iterator& QList<QVariant>::iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_iterator_operator__15(
                    self as *mut crate::q_list_of_q_variant::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_list_of_q_variant::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_variant::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::iterator QList<QVariant>::iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_variant::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_iterator_operator_5(
                    self as *const crate::q_list_of_q_variant::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_list_of_q_variant::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_variant::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::iterator QList<QVariant>::iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_variant::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_iterator_operator_6(
                    self as *const crate::q_list_of_q_variant::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_variant::Iterator>>
        for &crate::q_list_of_q_variant::Iterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QVariant>::iterator::operator-(QList<QVariant>::iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_variant::Iterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_iterator_operator_7(
                    self as *const crate::q_list_of_q_variant::Iterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_variant::ConstIterator {
        type Output = ::cpp_utils::Ref<crate::QVariant>;
        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QVariant& QList<QVariant>::const_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        fn indirection(self) -> ::cpp_utils::Ref<crate::QVariant> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_const_iterator_operator_(
                    self as *const crate::q_list_of_q_variant::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_variant::ConstIterator>>
        for crate::q_list_of_q_variant::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QVariant>::const_iterator::operator==(const QList<QVariant>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_variant::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_const_iterator_operator__2(
                    self as *const crate::q_list_of_q_variant::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_variant::ConstIterator>>
        for crate::q_list_of_q_variant::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QVariant>::const_iterator::operator<(const QList<QVariant>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_variant::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_const_iterator_operator_1(
                    self as *const crate::q_list_of_q_variant::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_variant::ConstIterator>>
        for crate::q_list_of_q_variant::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QVariant>::const_iterator::operator<=(const QList<QVariant>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_variant::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_const_iterator_operator__4(
                    self as *const crate::q_list_of_q_variant::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_variant::ConstIterator>>
        for crate::q_list_of_q_variant::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QVariant>::const_iterator::operator>(const QList<QVariant>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_variant::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_const_iterator_operator_2(
                    self as *const crate::q_list_of_q_variant::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_variant::ConstIterator>>
        for crate::q_list_of_q_variant::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QVariant>::const_iterator::operator>=(const QList<QVariant>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_variant::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_const_iterator_operator__5(
                    self as *const crate::q_list_of_q_variant::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_variant::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_variant::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::const_iterator& QList<QVariant>::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_variant::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_const_iterator_operator__6(
                    self as *mut crate::q_list_of_q_variant::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_variant::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_variant::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::const_iterator& QList<QVariant>::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_variant::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_const_iterator_operator__8(
                    self as *mut crate::q_list_of_q_variant::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_variant::ConstIterator
    {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::const_iterator& QList<QVariant>::const_iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_const_iterator_operator__10(
                    self as *mut crate::q_list_of_q_variant::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_variant::ConstIterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::const_iterator& QList<QVariant>::const_iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_const_iterator_operator__11(
                    self as *mut crate::q_list_of_q_variant::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_list_of_q_variant::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_variant::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::const_iterator QList<QVariant>::const_iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_variant::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_const_iterator_operator_3(
                    self as *const crate::q_list_of_q_variant::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_list_of_q_variant::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_variant::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::const_iterator QList<QVariant>::const_iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_variant::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_const_iterator_operator_4(
                    self as *const crate::q_list_of_q_variant::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_variant::ConstIterator>>
        for &crate::q_list_of_q_variant::ConstIterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QVariant>::const_iterator::operator-(QList<QVariant>::const_iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_variant::ConstIterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QVariant_const_iterator_operator_5(
                    self as *const crate::q_list_of_q_variant::ConstIterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_variant::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QVariant>::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QVariant_iterator_diterator(
                self as *mut crate::q_list_of_q_variant::Iterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_variant::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QVariant>::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QVariant_const_iterator_dconst_iterator(
                self as *mut crate::q_list_of_q_variant::ConstIterator,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
///
/// C++ class: <span style='color: green;'>```QList<QVariant>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlist.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores items in a list that provides fast index-based access and index-based insertions and removals.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Internally, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T if <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. Otherwise, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T* and the items are allocated on the heap.</p>
/// <p>The array representation allows very fast insertions and index-based access. The <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() operations are also very fast because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates memory at both ends of its internal array. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.</p>
/// <p>Note, however, that when the conditions specified above are not met, each append or insert of a new item requires allocating the new item on the heap, and this per item allocation will make <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> a better choice for use cases that do a lot of appending or inserting, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> can allocate memory for many items in a single heap allocation.</p>
/// <p>Note that the internal array only ever gets bigger over the life of the list. It never shrinks. The internal array is deallocated by the destructor and by the assignment operator, when one list is assigned to another.</p>
/// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerList;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span><span class="operator">&gt;</span> dateList;
///
/// </pre>
/// <p>Qt includes a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class that inherits <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>&gt; and adds a few convenience functions, such as <a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#filter">QStringList::filter</a>(). <a href="http://doc.qt.io/qt-5/qstring.html#split">QString::split</a>() creates QStringLists from strings.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> stores a list of items. The default constructor creates an empty list. To insert items into the list, you can use operator&lt;&lt;():</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"one"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"two"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"three"</span>;
///   <span class="comment">// list: ["one", "two", "three"]</span>
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(). In addition, it provides the following convenience functions: <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const lists, operator[]() returns a reference to the item and can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Bob"</span>)
/// &#32;     list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Robert"</span>;
///
/// </pre>
/// <p>Because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented as an array of pointers for types that are larger than a pointer or are not movable, this operation requires (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>). For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> list<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (list<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Jane"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Jane at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>A common requirement is to remove an item from a list and do something with it. For this, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(). Here's a loop that removes the items from a list one at a time and calls <code>delete</code> on them:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   <span class="keyword">while</span> (<span class="operator">!</span>list<span class="operator">.</span>isEmpty())
/// &#32;     <span class="keyword">delete</span> list<span class="operator">.</span>takeFirst();
///
/// </pre>
/// <p>Inserting and removing items at either end of the list is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> in most cases), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
/// <p>If you want to find all occurrences of a particular value in a list, use <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of a matching item if they find it; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> list<span class="operator">.</span>indexOf(<span class="string">"Jane"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Jane is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a list contains a particular value, use <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the list, use <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>(). If you want to replace all occurrences of a particular value with another, use <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlistiterator.html">QListIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablelistiterator.html">QMutableListIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented in such a way that direct index-based access is just as fast as using iterators.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <p>To make <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as efficient as possible, its member functions don't validate their input before using it. Except for <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), member functions always assume the list is <i>not</i> empty. Member functions that take index values as parameters always assume their index value parameters are in the valid range. This means <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> member functions can fail. If you define QT_NO_DEBUG when you compile, failures will not be detected. If you <i>don't</i> define QT_NO_DEBUG, failures will be detected using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT">Q_ASSERT</a>() or <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT_X">Q_ASSERT_X</a>() with an appropriate message.</p>
/// <p>To avoid failures when your list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling other member functions. If you must pass an index value that might not be in the valid range, check that it is less than the value returned by <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() but <i>not</i> less than 0.</p>
/// <a name="more-members"></a></div>
#[repr(C)]
pub struct QListOfQVariant {
    _unused: u8,
}
impl QListOfQVariant {
    /// <p>Inserts <i>value</i> at the end of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QVariant>::append(const QVariant& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["one", "two", "three"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn append_q_variant(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_append(
            self as *mut crate::QListOfQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QVariant>::append(const QList<QVariant>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> list to this list.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_list_of_q_variant(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_append1(
            self as *mut crate::QListOfQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQVariant>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QVariant& QList<QVariant>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>This function is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_at(
            self as *const crate::QListOfQVariant,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant& QList<QVariant>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::MutRef<crate::QVariant> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QVariant_back(self as *mut crate::QListOfQVariant);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QVariant& QList<QVariant>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_back1(
            self as *const crate::QListOfQVariant,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::const_iterator QList<QVariant>::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(&self) -> ::cpp_utils::CppBox<crate::q_list_of_q_variant::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_cbegin(
            self as *const crate::QListOfQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::const_iterator QList<QVariant>::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn cend(&self) -> ::cpp_utils::CppBox<crate::q_list_of_q_variant::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_cend(
            self as *const crate::QListOfQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all items from the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QVariant>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all items from the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_clear(self as *mut crate::QListOfQVariant)
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::const_iterator QList<QVariant>::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_variant::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_constBegin(
            self as *const crate::QListOfQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::const_iterator QList<QVariant>::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn const_end(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_variant::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_constEnd(
            self as *const crate::QListOfQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QVariant& QList<QVariant>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>().</p></div>
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_constFirst(
            self as *const crate::QListOfQVariant,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QVariant& QList<QVariant>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>().</p></div>
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_constLast(
            self as *const crate::QListOfQVariant,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QVariant>::contains(const QVariant& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn contains(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_contains(
            self as *const crate::QListOfQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant>& QList<QVariant>::operator=(const QList<QVariant>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this list and returns a reference to this list.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQVariant>>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_operator_(
            self as *mut crate::QListOfQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQVariant>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of occurrences of <i>value</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QVariant>::count(const QVariant& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of occurrences of <i>value</i> in the list.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_count(
            self as *const crate::QListOfQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QVariant>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_count1(self as *const crate::QListOfQVariant)
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QVariant>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_detach(self as *mut crate::QListOfQVariant)
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QVariant>::detachShared()```</span>.
    pub unsafe fn detach_shared(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_detachShared(
            self as *mut crate::QListOfQVariant,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QVariant>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_empty(self as *const crate::QListOfQVariant)
    }

    /// <p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QVariant>::endsWith(const QVariant& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn ends_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_endsWith(
            self as *const crate::QListOfQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::iterator QList<QVariant>::erase(QList<QVariant>::iterator pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn erase_1a(
        &mut self,
        pos: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_variant::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_variant::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_erase(self as *mut crate::QListOfQVariant, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_variant::Iterator>>::cast_into(pos).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::iterator QList<QVariant>::erase(QList<QVariant>::iterator first, QList<QVariant>::iterator last)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes all the items from <i>begin</i> up to (but not including) <i>end</i>. Returns an iterator to the same item that <i>end</i> referred to before the call.</p></div>
    pub unsafe fn erase_2a(
        &mut self,
        first: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_variant::Iterator>>,
        last: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_variant::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_variant::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_erase1(self as *mut crate::QListOfQVariant, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_variant::Iterator>>::cast_into(first).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_variant::Iterator>>::cast_into(last).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant& QList<QVariant>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<crate::QVariant> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QVariant_first(self as *mut crate::QListOfQVariant);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QVariant& QList<QVariant>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_first1(
            self as *const crate::QListOfQVariant,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant& QList<QVariant>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::MutRef<crate::QVariant> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QVariant_front(self as *mut crate::QListOfQVariant);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QVariant& QList<QVariant>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_front1(
            self as *const crate::QListOfQVariant,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QVariant& QList<QVariant>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>(). This function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p></div>
    pub unsafe fn index(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_operator__2(
            self as *const crate::QListOfQVariant,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant& QList<QVariant>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If this function is called on a list that is currently being shared, it will trigger a copy of all elements. Otherwise, this function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>. If you do not want to modify the list you should use <a href="http://doc.qt.io/qt-5/qlist.html#at">QList::at</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_operator__3(
            self as *mut crate::QListOfQVariant,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QVariant>::indexOf(const QVariant& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_indexOf(
            self as *const crate::QListOfQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(t).as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QVariant>::indexOf(const QVariant& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_indexOf1(
            self as *const crate::QListOfQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QVariant>::insert(int i, const QVariant& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   list<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// list: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn insert_int_q_variant(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_insert(
            self as *mut crate::QListOfQVariant,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::iterator QList<QVariant>::insert(QList<QVariant>::iterator before, const QVariant& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>value</i> in front of the item pointed to by the iterator <i>before</i>. Returns an iterator pointing at the inserted item. Note that the iterator passed to the function will be invalid after the call; the returned iterator should be used instead.</p></div>
    pub unsafe fn insert_iterator_q_variant(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_variant::Iterator>>,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_variant::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_insert1(self as *mut crate::QListOfQVariant, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_variant::Iterator>>::cast_into(before).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(t).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QVariant>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_isDetached(
            self as *const crate::QListOfQVariant,
        )
    }

    /// <p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QVariant>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_isEmpty(self as *const crate::QListOfQVariant)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QVariant>::isSharedWith(const QList<QVariant>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_isSharedWith(
            self as *const crate::QListOfQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQVariant>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant& QList<QVariant>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<crate::QVariant> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QVariant_last(self as *mut crate::QListOfQVariant);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QVariant& QList<QVariant>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_last1(
            self as *const crate::QListOfQVariant,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QVariant>::lastIndexOf(const QVariant& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_lastIndexOf(
            self as *const crate::QListOfQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(t).as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QVariant>::lastIndexOf(const QVariant& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_lastIndexOf1(
            self as *const crate::QListOfQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QVariant>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_length(self as *const crate::QListOfQVariant)
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant> QList<QVariant>::mid(int pos, int length = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        length: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_mid(
            self as *const crate::QListOfQVariant,
            pos,
            length,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant> QList<QVariant>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_mid1(
            self as *const crate::QListOfQVariant,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QVariant>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>move(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "C", "D", "E", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This is the same as insert(<i>to</i>, <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<i>from</i>)).This function assumes that both <i>from</i> and <i>to</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>from</i> and <i>to</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_move(
            self as *mut crate::QListOfQVariant,
            from,
            to,
        )
    }

    /// <p>Constructs an empty list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QVariant>::QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty list.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QListOfQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_QList();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QVariant>::QList(const QList<QVariant>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_QList1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQVariant>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QVariant>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_pop_back(self as *mut crate::QListOfQVariant)
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QVariant>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_pop_front(self as *mut crate::QListOfQVariant)
    }

    /// <p>Inserts <i>value</i> at the beginning of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QVariant>::prepend(const QVariant& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(0, <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_prepend(
            self as *mut crate::QListOfQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QVariant>::push_back(const QVariant& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_push_back(
            self as *mut crate::QListOfQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QVariant>::push_front(const QVariant& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_push_front(
            self as *mut crate::QListOfQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QVariant>::removeAll(const QVariant& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeAll(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_all(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_removeAll(
            self as *mut crate::QListOfQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QVariant>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_removeAt(
            self as *mut crate::QListOfQVariant,
            i,
        )
    }

    /// <p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QVariant>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_removeFirst(
            self as *mut crate::QListOfQVariant,
        )
    }

    /// <p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QVariant>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_removeLast(self as *mut crate::QListOfQVariant)
    }

    /// <p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QVariant>::removeOne(const QVariant& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeOne(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", ,"sun", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_one(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_removeOne(
            self as *mut crate::QListOfQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QVariant>::replace(int i, const QVariant& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_replace(
            self as *mut crate::QListOfQVariant,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Reserve space for <i>alloc</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QVariant>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reserve space for <i>alloc</i> elements.</p>
    /// <p>If <i>alloc</i> is smaller than the current size of the list, nothing will happen.</p>
    /// <p>Use this function to avoid repetetive reallocation of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s internal data if you can predict how many elements will be appended. Note that the reservation applies only to the internal pointer array.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_reserve(
            self as *mut crate::QListOfQVariant,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QVariant>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_setSharable(
            self as *mut crate::QListOfQVariant,
            sharable,
        )
    }

    /// <p>Returns the number of items in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QVariant>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_size(self as *const crate::QListOfQVariant)
    }

    /// <p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QVariant>::startsWith(const QVariant& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn starts_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_startsWith(
            self as *const crate::QListOfQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QVariant>::swap(QList<QVariant>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap_1a(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QListOfQVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_swap(
            self as *mut crate::QListOfQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QListOfQVariant>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QVariant>::swap(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swap(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    pub unsafe fn swap_2a(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_swap1(
            self as *mut crate::QListOfQVariant,
            i,
            j,
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QVariant>::swapItemsAt(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swapItemsAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swapItemsAt(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn swap_items_at(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_swapItemsAt(
            self as *mut crate::QListOfQVariant,
            i,
            j,
        )
    }

    /// <p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QList<QVariant>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_takeAt(
            self as *mut crate::QListOfQVariant,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QList<QVariant>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_takeFirst(
            self as *mut crate::QListOfQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QList<QVariant>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_takeLast(
            self as *mut crate::QListOfQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value at index position <i>i</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QList<QVariant>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the list.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that the index is going to be within bounds, you can use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_value(
            self as *const crate::QListOfQVariant,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QList<QVariant>::value(int i, const QVariant& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QVariant_value1(
            self as *const crate::QListOfQVariant,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> class is a template class that provides a dynamic array.</p>
///
/// C++ class: <span style='color: green;'>```QVector<QXmlStreamAttribute>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qvector.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> class is a template class that provides a dynamic array.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores its items in adjacent memory locations and provides fast index-based access.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Here's an example of a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> that stores <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerVector;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> stringVector;
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> stores its items in a vector (array). Typically, vectors are created with an initial size. For example, the following code constructs a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> with 200 elements:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">200</span>);
///
/// </pre>
/// <p>The elements are automatically initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you want to initialize the vector with a different value, pass that value as the second argument to the constructor:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">200</span><span class="operator">,</span> <span class="string">"Pass"</span>);
///
/// </pre>
/// <p>You can also call <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>() at any time to fill the vector with a value.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const vectors, operator[]() returns a reference to the item that can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (vector<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Liz"</span>)
/// &#32;     vector<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Elizabeth"</span>;
///
/// </pre>
/// <p>For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> vector<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (vector<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Alfonso"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Alfonso at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>Another way to access the data stored in a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> is to call <a href="http://doc.qt.io/qt-5/qvector.html#data">data</a>(). The function returns a pointer to the first item in the vector. You can use the pointer to directly access and modify the elements stored in the vector. The pointer is also useful if you need to pass a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to a function that accepts a plain C++ array.</p>
/// <p>If you want to find all occurrences of a particular value in a vector, use <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of the matching item if they found one; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> vector<span class="operator">.</span>indexOf(<span class="string">"Harumi"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Harumi is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a vector contains a particular value, use <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the vector, use <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(). With the exception of <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), these functions can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>) for large vectors, because they require moving many items in the vector by one position in memory. If you want a container class that provides fast insertion/removal in the middle, use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> or <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
/// <p>Unlike plain C++ arrays, QVectors can be resized at any time by calling <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>(). If the new size is larger than the old size, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> might need to reallocate the whole vector. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> tries to reduce the number of reallocations by preallocating up to twice as much memory as the actual data needs.</p>
/// <p>If you know in advance approximately how many items the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> will contain, you can call <a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>(), asking <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to preallocate a certain amount of memory. You can also call <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>() to find out how much memory <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> actually allocated.</p>
/// <p>Note that using non-const operators and functions can cause <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to do a deep copy of the data. This is due to <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicit sharing</a>.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qvectoriterator.html">QVectorIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablevectoriterator.html">QMutableVectorIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qvector.html#const_iterator-typedef">QVector::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qvector.html#iterator-typedefx">QVector::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>.</p>
/// <p>In addition to <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>, Qt also provides <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a>, a very low-level class with little functionality that is optimized for speed.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <a name="more-information-on-using-qt-containers"></a>
/// <h4>More Information on Using Qt Containers</h4>
/// <p>For a detailed discussion comparing Qt containers with each other and with STL containers, see <a href="http://marcmutz.wordpress.com/effective-qt/containers/">Understand the Qt Containers</a>.</p></div>
#[repr(C)]
pub struct QVectorOfQXmlStreamAttribute {
    _unused: u8,
}
impl QVectorOfQXmlStreamAttribute {
    /// <p>Inserts <i>value</i> at the end of the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamAttribute>::append(const QXmlStreamAttribute& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   vector<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> three <span class="operator">=</span> <span class="string">"three"</span>;
    ///   vector<span class="operator">.</span>append(three);
    ///   <span class="comment">// vector: ["one", "two", "three"]</span>
    ///   <span class="comment">// three: "three"</span>
    ///
    /// </pre>
    /// <p>This is the same as calling resize(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() + 1) and assigning <i>value</i> to the new last element in the vector.</p>
    /// <p>This operation is relatively fast, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> typically allocates more memory than necessary, so it can grow without reallocating the entire vector each time.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>().</p></div>
    pub unsafe fn append_q_xml_stream_attribute(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamAttribute>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_append(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamAttribute>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamAttribute>::append(const QVector<QXmlStreamAttribute>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#append-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> vector to this vector.</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_vector_of_q_xml_stream_attribute(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQXmlStreamAttribute>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_append2(self as *mut crate::QVectorOfQXmlStreamAttribute, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQXmlStreamAttribute>>::cast_into(l).as_raw_ptr())
    }

    /// <p>Returns the item at index position <i>i</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamAttribute& QVector<QXmlStreamAttribute>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the vector.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ref<crate::QXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_at(
            self as *const crate::QVectorOfQXmlStreamAttribute,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamAttribute& QVector<QXmlStreamAttribute>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::MutRef<crate::QXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_back(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamAttribute& QVector<QXmlStreamAttribute>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<crate::QXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_back1(
            self as *const crate::QVectorOfQXmlStreamAttribute,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the maximum number of items that can be stored in the vector without forcing a reallocation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamAttribute>::capacity() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#capacity">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the maximum number of items that can be stored in the vector without forcing a reallocation.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s memory usage. In general, you will rarely ever need to call this function. If you want to know how many items are in the vector, call <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p></div>
    pub unsafe fn capacity(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_capacity(
            self as *const crate::QVectorOfQXmlStreamAttribute,
        )
    }

    /// <p>Removes all the elements from the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamAttribute>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all the elements from the vector.</p>
    /// <p><b>Note: </b>Until Qt 5.6, this also released the memory used by the vector. From Qt 5.7, the capacity is preserved. To shed all capacity, swap with a default-constructed vector:</p><pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span>T<span class="operator">&gt;</span> v <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span>T<span class="operator">&gt;</span>()<span class="operator">.</span>swap(v);
    ///   Q_ASSERT(v<span class="operator">.</span>capacity() <span class="operator">=</span><span class="operator">=</span> <span class="number">0</span>);
    ///
    /// </pre>
    /// <p>or call <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_clear(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
        )
    }

    /// <p>Returns a const pointer to the data stored in the vector. The pointer can be used to access the items in the vector. The pointer remains valid as long as the vector isn't reallocated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamAttribute* QVector<QXmlStreamAttribute>::constData() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const pointer to the data stored in the vector. The pointer can be used to access the items in the vector. The pointer remains valid as long as the vector isn't reallocated.</p>
    /// <p>This function is mostly useful to pass a vector to a function that accepts a plain C++ array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#data">data</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn const_data(&self) -> ::cpp_utils::Ptr<crate::QXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_constData(
            self as *const crate::QVectorOfQXmlStreamAttribute,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns a const reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamAttribute& QVector<QXmlStreamAttribute>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<crate::QXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_constFirst(
            self as *const crate::QVectorOfQXmlStreamAttribute,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamAttribute& QVector<QXmlStreamAttribute>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<crate::QXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_constLast(
            self as *const crate::QVectorOfQXmlStreamAttribute,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the vector contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamAttribute>::contains(const QXmlStreamAttribute& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the vector contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p></div>
    pub unsafe fn contains(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamAttribute>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_contains(
            self as *const crate::QVectorOfQXmlStreamAttribute,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamAttribute>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamAttribute>& QVector<QXmlStreamAttribute>::operator=(const QVector<QXmlStreamAttribute>& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this vector and returns a reference to this vector.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        v: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQXmlStreamAttribute>>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_operator_(self as *mut crate::QVectorOfQXmlStreamAttribute, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQXmlStreamAttribute>>::cast_into(v).as_raw_ptr());
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of occurrences of <i>value</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamAttribute>::count(const QXmlStreamAttribute& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of occurrences of <i>value</i> in the vector.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamAttribute>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_count(
            self as *const crate::QVectorOfQXmlStreamAttribute,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamAttribute>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamAttribute>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_count1(
            self as *const crate::QVectorOfQXmlStreamAttribute,
        )
    }

    /// <p>Returns a pointer to the data stored in the vector. The pointer can be used to access and modify the items in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamAttribute* QVector<QXmlStreamAttribute>::data()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the data stored in the vector. The pointer can be used to access and modify the items in the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> vector(<span class="number">10</span>);
    ///   <span class="type">int</span> <span class="operator">*</span>data <span class="operator">=</span> vector<span class="operator">.</span>data();
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> <span class="number">10</span>; <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     data<span class="operator">[</span>i<span class="operator">]</span> <span class="operator">=</span> <span class="number">2</span> <span class="operator">*</span> i;
    ///
    /// </pre>
    /// <p>The pointer remains valid as long as the vector isn't reallocated.</p>
    /// <p>This function is mostly useful to pass a vector to a function that accepts a plain C++ array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constData">constData</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn data_mut(&mut self) -> ::cpp_utils::MutPtr<crate::QXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_data(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamAttribute* QVector<QXmlStreamAttribute>::data() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#data-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn data(&self) -> ::cpp_utils::Ptr<crate::QXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_data1(
            self as *const crate::QVectorOfQXmlStreamAttribute,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamAttribute>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_detach(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), returning <code>true</code> if the vector is empty; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamAttribute>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), returning <code>true</code> if the vector is empty; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_empty(
            self as *const crate::QVectorOfQXmlStreamAttribute,
        )
    }

    /// <p>Returns <code>true</code> if this vector is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamAttribute>::endsWith(const QXmlStreamAttribute& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this vector is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn ends_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamAttribute>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_endsWith(
            self as *const crate::QVectorOfQXmlStreamAttribute,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamAttribute>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamAttribute>& QVector<QXmlStreamAttribute>::fill(const QXmlStreamAttribute& t, int size = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">3</span>);
    ///   vector<span class="operator">.</span>fill(<span class="string">"Yes"</span>);
    ///   <span class="comment">// vector: ["Yes", "Yes", "Yes"]</span>
    ///
    ///   vector<span class="operator">.</span>fill(<span class="string">"oh"</span><span class="operator">,</span> <span class="number">5</span>);
    ///   <span class="comment">// vector: ["oh", "oh", "oh", "oh", "oh"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_2a(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamAttribute>>,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_fill(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamAttribute>>::cast_into(t)
                .as_raw_ptr(),
            size,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamAttribute>& QVector<QXmlStreamAttribute>::fill(const QXmlStreamAttribute& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">3</span>);
    ///   vector<span class="operator">.</span>fill(<span class="string">"Yes"</span>);
    ///   <span class="comment">// vector: ["Yes", "Yes", "Yes"]</span>
    ///
    ///   vector<span class="operator">.</span>fill(<span class="string">"oh"</span><span class="operator">,</span> <span class="number">5</span>);
    ///   <span class="comment">// vector: ["oh", "oh", "oh", "oh", "oh"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_1a(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamAttribute>>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_fill1(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamAttribute>>::cast_into(t)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamAttribute& QVector<QXmlStreamAttribute>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#constFirst">constFirst</a>().</p></div>
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<crate::QXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_first(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamAttribute& QVector<QXmlStreamAttribute>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::QXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_first1(
            self as *const crate::QVectorOfQXmlStreamAttribute,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamAttribute& QVector<QXmlStreamAttribute>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::MutRef<crate::QXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_front(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamAttribute& QVector<QXmlStreamAttribute>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<crate::QXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_front1(
            self as *const crate::QVectorOfQXmlStreamAttribute,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamAttribute& QVector<QXmlStreamAttribute>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p>Note that using non-const operators can cause <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to do a deep copy.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_operator__2(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamAttribute& QVector<QXmlStreamAttribute>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as at(<i>i</i>).</p></div>
    pub unsafe fn index(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ref<crate::QXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_operator__3(
            self as *const crate::QVectorOfQXmlStreamAttribute,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamAttribute>::indexOf(const QXmlStreamAttribute& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span>);            <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);         <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);         <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"X"</span>);            <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamAttribute>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_indexOf(
            self as *const crate::QVectorOfQXmlStreamAttribute,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamAttribute>>::cast_into(t)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamAttribute>::indexOf(const QXmlStreamAttribute& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span>);            <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);         <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);         <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"X"</span>);            <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamAttribute>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_indexOf1(
            self as *const crate::QVectorOfQXmlStreamAttribute,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamAttribute>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the vector. If <i>i</i> is 0, the value is prepended to the vector. If <i>i</i> is <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), the value is appended to the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamAttribute>::insert(int i, const QXmlStreamAttribute& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the vector. If <i>i</i> is 0, the value is prepended to the vector. If <i>i</i> is <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), the value is appended to the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   vector<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// vector: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p>For large vectors, this operation can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because it requires moving all the items at indexes <i>i</i> and above by one position further in memory. If you want a container class that provides a fast insert() function, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>().</p></div>
    pub unsafe fn insert_2a(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamAttribute>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_insert(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamAttribute>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamAttribute>::insert(int i, int n, const QXmlStreamAttribute& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>count</i> copies of <i>value</i> at index position <i>i</i> in the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">double</span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">2.718</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">1.442</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">0.4342</span>;
    ///   vector<span class="operator">.</span>insert(<span class="number">1</span><span class="operator">,</span> <span class="number">3</span><span class="operator">,</span> <span class="number">9.9</span>);
    ///   <span class="comment">// vector: [2.718, 9.9, 9.9, 9.9, 1.442, 0.4342]</span>
    ///
    /// </pre></div>
    pub unsafe fn insert_3a(
        &mut self,
        i: ::std::os::raw::c_int,
        n: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamAttribute>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_insert1(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
            i,
            n,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamAttribute>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamAttribute>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_isDetached(
            self as *const crate::QVectorOfQXmlStreamAttribute,
        )
    }

    /// <p>Returns <code>true</code> if the vector has size 0; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamAttribute>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the vector has size 0; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_isEmpty(
            self as *const crate::QVectorOfQXmlStreamAttribute,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamAttribute>::isSharedWith(const QVector<QXmlStreamAttribute>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQXmlStreamAttribute>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_isSharedWith(self as *const crate::QVectorOfQXmlStreamAttribute, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQXmlStreamAttribute>>::cast_into(other).as_raw_ptr())
    }

    /// <p>Returns a reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamAttribute& QVector<QXmlStreamAttribute>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#constLast">constLast</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<crate::QXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_last(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamAttribute& QVector<QXmlStreamAttribute>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::QXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_last1(
            self as *const crate::QVectorOfQXmlStreamAttribute,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamAttribute>::lastIndexOf(const QXmlStreamAttribute& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);        <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);     <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);     <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamAttribute>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_lastIndexOf(
            self as *const crate::QVectorOfQXmlStreamAttribute,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamAttribute>>::cast_into(t)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamAttribute>::lastIndexOf(const QXmlStreamAttribute& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);        <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);     <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);     <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamAttribute>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_lastIndexOf1(
            self as *const crate::QVectorOfQXmlStreamAttribute,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamAttribute>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamAttribute>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#length">QList::length</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_length(
            self as *const crate::QVectorOfQXmlStreamAttribute,
        )
    }

    /// <p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamAttribute> QVector<QXmlStreamAttribute>::mid(int pos, int len = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        len: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_mid(
            self as *const crate::QVectorOfQXmlStreamAttribute,
            pos,
            len,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamAttribute> QVector<QXmlStreamAttribute>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_mid1(
            self as *const crate::QVectorOfQXmlStreamAttribute,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamAttribute>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">QList::move</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_move(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
            from,
            to,
        )
    }

    /// <p>Constructs an empty vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QXmlStreamAttribute>::QVector()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_QVector();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a vector with an initial size of <i>size</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QXmlStreamAttribute>::QVector(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a vector with an initial size of <i>size</i> elements.</p>
    /// <p>The elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn new_1a(
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_QVector1(size);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a vector with an initial size of <i>size</i> elements. Each element is initialized with <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QXmlStreamAttribute>::QVector(int size, const QXmlStreamAttribute& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a vector with an initial size of <i>size</i> elements. Each element is initialized with <i>value</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn new_2a(
        size: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamAttribute>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_QVector2(
            size,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamAttribute>>::cast_into(t)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QXmlStreamAttribute>::QVector(const QVector<QXmlStreamAttribute>& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        v: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQXmlStreamAttribute>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_QVector3(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQXmlStreamAttribute>>::cast_into(v).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamAttribute>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_pop_back(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamAttribute>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_pop_front(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
        )
    }

    /// <p>Inserts <i>value</i> at the beginning of the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamAttribute>::prepend(const QXmlStreamAttribute& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   vector<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   vector<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// vector: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as vector.insert(0, <i>value</i>).</p>
    /// <p>For large vectors, this operation can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because it requires moving all the items in the vector by one position further in memory. If you want a container class that provides a fast prepend() function, use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> or <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamAttribute>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_prepend(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamAttribute>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to append(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamAttribute>::push_back(const QXmlStreamAttribute& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to append(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamAttribute>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_push_back(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamAttribute>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to prepend(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamAttribute>::push_front(const QXmlStreamAttribute& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to prepend(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamAttribute>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_push_front(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamAttribute>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamAttribute>::remove(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#remove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes the element at index position <i>i</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn remove_1a(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_remove(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
            i,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamAttribute>::remove(int i, int n)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#remove-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes <i>count</i> elements from the middle of the vector, starting at index position <i>i</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn remove_2a(&mut self, i: ::std::os::raw::c_int, n: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_remove1(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
            i,
            n,
        )
    }

    /// <p>Removes all elements that compare equal to <i>t</i> from the vector. Returns the number of elements removed, if any.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamAttribute>::removeAll(const QXmlStreamAttribute& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all elements that compare equal to <i>t</i> from the vector. Returns the number of elements removed, if any.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#removeOne">removeOne</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">QList::removeAll</a>().</p></div>
    pub unsafe fn remove_all(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamAttribute>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_removeAll(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamAttribute>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes the element at index position <i>i</i>. Equivalent to</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamAttribute>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the element at index position <i>i</i>. Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   remove(i);
    ///
    /// </pre>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">QList::removeAt</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_removeAt(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
            i,
        )
    }

    /// <p>Removes the first item in the vector. Calling this function is equivalent to calling remove(0). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamAttribute>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the vector. Calling this function is equivalent to calling remove(0). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_removeFirst(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
        )
    }

    /// <p>Removes the last item in the vector. Calling this function is equivalent to calling remove(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() - 1). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamAttribute>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the vector. Calling this function is equivalent to calling remove(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() - 1). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_removeLast(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
        )
    }

    /// <p>Removes the first element that compares equal to <i>t</i> from the vector. Returns whether an element was, in fact, removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamAttribute>::removeOne(const QXmlStreamAttribute& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeOne">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first element that compares equal to <i>t</i> from the vector. Returns whether an element was, in fact, removed.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#removeAll">removeAll</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">QList::removeOne</a>().</p></div>
    pub unsafe fn remove_one(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamAttribute>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_removeOne(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamAttribute>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamAttribute>::replace(int i, const QXmlStreamAttribute& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamAttribute>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_replace(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamAttribute>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Attempts to allocate memory for at least <i>size</i> elements. If you know in advance how large the vector will be, you should call this function to prevent reallocations and memory fragmentation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamAttribute>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to allocate memory for at least <i>size</i> elements. If you know in advance how large the vector will be, you should call this function to prevent reallocations and memory fragmentation.</p>
    /// <p>If <i>size</i> is an underestimate, the worst that will happen is that the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> will be a bit slower. If <i>size</i> is an overestimate, you may have used more memory than the normal <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> growth strategy would have allocated—or you may have used less.</p>
    /// <p>An alternative to reserve() is calling <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>(). Whether or not that is faster than reserve() depends on the element type, because <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() default-constructs all elements, and requires assignment to existing entries rather than calling <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(), which copy- or move-constructs. For simple types, like <code>int</code> or <code>double</code>, <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() is typically faster, but for anything more complex, you should prefer reserve().</p>
    /// <p><b>Warning:</b> If the size passed to <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() was underestimated, you run out of allocated space and into undefined behavior. This problem does not exist with reserve(), because it treats the size as just a hint.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>().</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_reserve(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
            size,
        )
    }

    /// <p>Sets the size of the vector to <i>size</i>. If <i>size</i> is greater than the current size, elements are added to the end; the new elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If <i>size</i> is less than the current size, elements are removed from the end.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamAttribute>::resize(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#resize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the size of the vector to <i>size</i>. If <i>size</i> is greater than the current size, elements are added to the end; the new elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If <i>size</i> is less than the current size, elements are removed from the end.</p>
    /// <p>Since Qt 5.6, resize() doesn't shrink the capacity anymore. To shed excess capacity, use <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p></div>
    pub unsafe fn resize(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_resize(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamAttribute>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_setSharable(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
            sharable,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamAttribute>::shrink_to_fit()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#shrink_to_fit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn shrink_to_fit(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_shrink_to_fit(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
        )
    }

    /// <p>Returns the number of items in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamAttribute>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_size(
            self as *const crate::QVectorOfQXmlStreamAttribute,
        )
    }

    /// <p>Releases any memory not required to store the items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamAttribute>::squeeze()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Releases any memory not required to store the items.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s memory usage. In general, you will rarely ever need to call this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>().</p></div>
    pub unsafe fn squeeze(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_squeeze(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
        )
    }

    /// <p>Returns <code>true</code> if this vector is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamAttribute>::startsWith(const QXmlStreamAttribute& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this vector is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn starts_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamAttribute>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_startsWith(
            self as *const crate::QVectorOfQXmlStreamAttribute,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamAttribute>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Swaps vector <i>other</i> with this vector. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamAttribute>::swap(QVector<QXmlStreamAttribute>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps vector <i>other</i> with this vector. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QVectorOfQXmlStreamAttribute>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_swap(self as *mut crate::QVectorOfQXmlStreamAttribute, ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QVectorOfQXmlStreamAttribute>>::cast_into(other).as_mut_raw_ptr())
    }

    /// <p>Removes the element at index position <i>i</i> and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamAttribute QVector<QXmlStreamAttribute>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the element at index position <i>i</i> and returns it.</p>
    /// <p>Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   T t <span class="operator">=</span> at(i);
    ///   remove(i);
    ///   <span class="keyword">return</span> t;
    ///
    /// </pre>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">QList::takeAt</a>().</p></div>
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_takeAt(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the first item in the vector and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamAttribute QVector<QXmlStreamAttribute>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the vector and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> ::cpp_utils::CppBox<crate::QXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_takeFirst(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the last item in the list and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamAttribute QVector<QXmlStreamAttribute>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> ::cpp_utils::CppBox<crate::QXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_takeLast(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value at index position <i>i</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamAttribute QVector<QXmlStreamAttribute>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the vector.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that <i>i</i> is within bounds, you can use <a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_value(
            self as *const crate::QVectorOfQXmlStreamAttribute,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamAttribute QVector<QXmlStreamAttribute>::value(int i, const QXmlStreamAttribute& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamAttribute>>,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_value1(
            self as *const crate::QVectorOfQXmlStreamAttribute,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamAttribute>>::cast_into(
                default_value,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl From<crate::MouseButton> for crate::QFlags<crate::MouseButton> {
    fn from(value: crate::MouseButton) -> Self {
        Self::from(value.to_int())
    }
}

impl<T: Into<crate::QFlags<crate::MouseButton>>> std::ops::BitOr<T> for crate::MouseButton {
    type Output = crate::QFlags<crate::MouseButton>;
    fn bitor(self, rhs: T) -> crate::QFlags<crate::MouseButton> {
        Into::<crate::QFlags<crate::MouseButton>>::into(self) | rhs
    }
}

impl From<crate::Orientation> for crate::QFlags<crate::Orientation> {
    fn from(value: crate::Orientation) -> Self {
        Self::from(value.to_int())
    }
}

impl<T: Into<crate::QFlags<crate::Orientation>>> std::ops::BitOr<T> for crate::Orientation {
    type Output = crate::QFlags<crate::Orientation>;
    fn bitor(self, rhs: T) -> crate::QFlags<crate::Orientation> {
        Into::<crate::QFlags<crate::Orientation>>::into(self) | rhs
    }
}

impl From<crate::KeyboardModifier> for crate::QFlags<crate::KeyboardModifier> {
    fn from(value: crate::KeyboardModifier) -> Self {
        Self::from(value.to_int())
    }
}

impl<T: Into<crate::QFlags<crate::KeyboardModifier>>> std::ops::BitOr<T>
    for crate::KeyboardModifier
{
    type Output = crate::QFlags<crate::KeyboardModifier>;
    fn bitor(self, rhs: T) -> crate::QFlags<crate::KeyboardModifier> {
        Into::<crate::QFlags<crate::KeyboardModifier>>::into(self) | rhs
    }
}

impl From<crate::WindowType> for crate::QFlags<crate::WindowType> {
    fn from(value: crate::WindowType) -> Self {
        Self::from(value.to_int())
    }
}

impl<T: Into<crate::QFlags<crate::WindowType>>> std::ops::BitOr<T> for crate::WindowType {
    type Output = crate::QFlags<crate::WindowType>;
    fn bitor(self, rhs: T) -> crate::QFlags<crate::WindowType> {
        Into::<crate::QFlags<crate::WindowType>>::into(self) | rhs
    }
}

impl From<crate::AlignmentFlag> for crate::QFlags<crate::AlignmentFlag> {
    fn from(value: crate::AlignmentFlag) -> Self {
        Self::from(value.to_int())
    }
}

impl<T: Into<crate::QFlags<crate::AlignmentFlag>>> std::ops::BitOr<T> for crate::AlignmentFlag {
    type Output = crate::QFlags<crate::AlignmentFlag>;
    fn bitor(self, rhs: T) -> crate::QFlags<crate::AlignmentFlag> {
        Into::<crate::QFlags<crate::AlignmentFlag>>::into(self) | rhs
    }
}

impl From<crate::Edge> for crate::QFlags<crate::Edge> {
    fn from(value: crate::Edge) -> Self {
        Self::from(value.to_int())
    }
}

impl<T: Into<crate::QFlags<crate::Edge>>> std::ops::BitOr<T> for crate::Edge {
    type Output = crate::QFlags<crate::Edge>;
    fn bitor(self, rhs: T) -> crate::QFlags<crate::Edge> {
        Into::<crate::QFlags<crate::Edge>>::into(self) | rhs
    }
}

impl From<crate::ImageConversionFlag> for crate::QFlags<crate::ImageConversionFlag> {
    fn from(value: crate::ImageConversionFlag) -> Self {
        Self::from(value.to_int())
    }
}

impl<T: Into<crate::QFlags<crate::ImageConversionFlag>>> std::ops::BitOr<T>
    for crate::ImageConversionFlag
{
    type Output = crate::QFlags<crate::ImageConversionFlag>;
    fn bitor(self, rhs: T) -> crate::QFlags<crate::ImageConversionFlag> {
        Into::<crate::QFlags<crate::ImageConversionFlag>>::into(self) | rhs
    }
}

impl From<crate::DockWidgetArea> for crate::QFlags<crate::DockWidgetArea> {
    fn from(value: crate::DockWidgetArea) -> Self {
        Self::from(value.to_int())
    }
}

impl<T: Into<crate::QFlags<crate::DockWidgetArea>>> std::ops::BitOr<T> for crate::DockWidgetArea {
    type Output = crate::QFlags<crate::DockWidgetArea>;
    fn bitor(self, rhs: T) -> crate::QFlags<crate::DockWidgetArea> {
        Into::<crate::QFlags<crate::DockWidgetArea>>::into(self) | rhs
    }
}

impl From<crate::ToolBarArea> for crate::QFlags<crate::ToolBarArea> {
    fn from(value: crate::ToolBarArea) -> Self {
        Self::from(value.to_int())
    }
}

impl<T: Into<crate::QFlags<crate::ToolBarArea>>> std::ops::BitOr<T> for crate::ToolBarArea {
    type Output = crate::QFlags<crate::ToolBarArea>;
    fn bitor(self, rhs: T) -> crate::QFlags<crate::ToolBarArea> {
        Into::<crate::QFlags<crate::ToolBarArea>>::into(self) | rhs
    }
}

impl From<crate::WindowState> for crate::QFlags<crate::WindowState> {
    fn from(value: crate::WindowState) -> Self {
        Self::from(value.to_int())
    }
}

impl<T: Into<crate::QFlags<crate::WindowState>>> std::ops::BitOr<T> for crate::WindowState {
    type Output = crate::QFlags<crate::WindowState>;
    fn bitor(self, rhs: T) -> crate::QFlags<crate::WindowState> {
        Into::<crate::QFlags<crate::WindowState>>::into(self) | rhs
    }
}

impl From<crate::ScreenOrientation> for crate::QFlags<crate::ScreenOrientation> {
    fn from(value: crate::ScreenOrientation) -> Self {
        Self::from(value.to_int())
    }
}

impl<T: Into<crate::QFlags<crate::ScreenOrientation>>> std::ops::BitOr<T>
    for crate::ScreenOrientation
{
    type Output = crate::QFlags<crate::ScreenOrientation>;
    fn bitor(self, rhs: T) -> crate::QFlags<crate::ScreenOrientation> {
        Into::<crate::QFlags<crate::ScreenOrientation>>::into(self) | rhs
    }
}

impl From<crate::DropAction> for crate::QFlags<crate::DropAction> {
    fn from(value: crate::DropAction) -> Self {
        Self::from(value.to_int())
    }
}

impl<T: Into<crate::QFlags<crate::DropAction>>> std::ops::BitOr<T> for crate::DropAction {
    type Output = crate::QFlags<crate::DropAction>;
    fn bitor(self, rhs: T) -> crate::QFlags<crate::DropAction> {
        Into::<crate::QFlags<crate::DropAction>>::into(self) | rhs
    }
}

impl From<crate::ItemFlag> for crate::QFlags<crate::ItemFlag> {
    fn from(value: crate::ItemFlag) -> Self {
        Self::from(value.to_int())
    }
}

impl<T: Into<crate::QFlags<crate::ItemFlag>>> std::ops::BitOr<T> for crate::ItemFlag {
    type Output = crate::QFlags<crate::ItemFlag>;
    fn bitor(self, rhs: T) -> crate::QFlags<crate::ItemFlag> {
        Into::<crate::QFlags<crate::ItemFlag>>::into(self) | rhs
    }
}

impl From<crate::MatchFlag> for crate::QFlags<crate::MatchFlag> {
    fn from(value: crate::MatchFlag) -> Self {
        Self::from(value.to_int())
    }
}

impl<T: Into<crate::QFlags<crate::MatchFlag>>> std::ops::BitOr<T> for crate::MatchFlag {
    type Output = crate::QFlags<crate::MatchFlag>;
    fn bitor(self, rhs: T) -> crate::QFlags<crate::MatchFlag> {
        Into::<crate::QFlags<crate::MatchFlag>>::into(self) | rhs
    }
}

impl From<crate::TextInteractionFlag> for crate::QFlags<crate::TextInteractionFlag> {
    fn from(value: crate::TextInteractionFlag) -> Self {
        Self::from(value.to_int())
    }
}

impl<T: Into<crate::QFlags<crate::TextInteractionFlag>>> std::ops::BitOr<T>
    for crate::TextInteractionFlag
{
    type Output = crate::QFlags<crate::TextInteractionFlag>;
    fn bitor(self, rhs: T) -> crate::QFlags<crate::TextInteractionFlag> {
        Into::<crate::QFlags<crate::TextInteractionFlag>>::into(self) | rhs
    }
}

impl From<crate::InputMethodQuery> for crate::QFlags<crate::InputMethodQuery> {
    fn from(value: crate::InputMethodQuery) -> Self {
        Self::from(value.to_int())
    }
}

impl<T: Into<crate::QFlags<crate::InputMethodQuery>>> std::ops::BitOr<T>
    for crate::InputMethodQuery
{
    type Output = crate::QFlags<crate::InputMethodQuery>;
    fn bitor(self, rhs: T) -> crate::QFlags<crate::InputMethodQuery> {
        Into::<crate::QFlags<crate::InputMethodQuery>>::into(self) | rhs
    }
}

impl From<crate::InputMethodHint> for crate::QFlags<crate::InputMethodHint> {
    fn from(value: crate::InputMethodHint) -> Self {
        Self::from(value.to_int())
    }
}

impl<T: Into<crate::QFlags<crate::InputMethodHint>>> std::ops::BitOr<T> for crate::InputMethodHint {
    type Output = crate::QFlags<crate::InputMethodHint>;
    fn bitor(self, rhs: T) -> crate::QFlags<crate::InputMethodHint> {
        Into::<crate::QFlags<crate::InputMethodHint>>::into(self) | rhs
    }
}

impl From<crate::TouchPointState> for crate::QFlags<crate::TouchPointState> {
    fn from(value: crate::TouchPointState) -> Self {
        Self::from(value.to_int())
    }
}

impl<T: Into<crate::QFlags<crate::TouchPointState>>> std::ops::BitOr<T> for crate::TouchPointState {
    type Output = crate::QFlags<crate::TouchPointState>;
    fn bitor(self, rhs: T) -> crate::QFlags<crate::TouchPointState> {
        Into::<crate::QFlags<crate::TouchPointState>>::into(self) | rhs
    }
}

impl From<crate::MouseEventFlag> for crate::QFlags<crate::MouseEventFlag> {
    fn from(value: crate::MouseEventFlag) -> Self {
        Self::from(value.to_int())
    }
}

impl<T: Into<crate::QFlags<crate::MouseEventFlag>>> std::ops::BitOr<T> for crate::MouseEventFlag {
    type Output = crate::QFlags<crate::MouseEventFlag>;
    fn bitor(self, rhs: T) -> crate::QFlags<crate::MouseEventFlag> {
        Into::<crate::QFlags<crate::MouseEventFlag>>::into(self) | rhs
    }
}

impl From<crate::GestureFlag> for crate::QFlags<crate::GestureFlag> {
    fn from(value: crate::GestureFlag) -> Self {
        Self::from(value.to_int())
    }
}

impl<T: Into<crate::QFlags<crate::GestureFlag>>> std::ops::BitOr<T> for crate::GestureFlag {
    type Output = crate::QFlags<crate::GestureFlag>;
    fn bitor(self, rhs: T) -> crate::QFlags<crate::GestureFlag> {
        Into::<crate::QFlags<crate::GestureFlag>>::into(self) | rhs
    }
}

pub mod q_list_of_q_byte_array {
    //! C++ type: <span style='color: green;'>```QList<QByteArray>```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QByteArray>::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::iterator& QList<QByteArray>::iterator::operator=(const QList<QByteArray>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_byte_array::Iterator>>,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_byte_array::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_iterator_operator_8(
                    self as *mut crate::q_list_of_q_byte_array::Iterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_byte_array::Iterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::iterator QList<QByteArray>::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_iterator_operator__13(
                self as *mut crate::q_list_of_q_byte_array::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::iterator QList<QByteArray>::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_iterator_operator__11(
                self as *mut crate::q_list_of_q_byte_array::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QByteArray& QList<QByteArray>::iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValueRef iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>. If you assign to it, the assignment will apply to the character in the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> of <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> from which you got the reference.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::MutRef<crate::QByteArray> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_iterator_operator__1(
                self as *const crate::q_list_of_q_byte_array::Iterator,
                j,
            );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QByteArray>::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>().</p></div>
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QByteArray>::iterator::iterator(const QList<QByteArray>::iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_byte_array::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_iterator_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_byte_array::Iterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QByteArray* QList<QByteArray>::iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's value.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::MutPtr<crate::QByteArray> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_iterator_operator__(
                self as *const crate::q_list_of_q_byte_array::Iterator,
            );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QByteArray>::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::const_iterator& QList<QByteArray>::const_iterator::operator=(const QList<QByteArray>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_byte_array::ConstIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_byte_array::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_const_iterator_operator_6(
                    self as *mut crate::q_list_of_q_byte_array::ConstIterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_byte_array::ConstIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::const_iterator QList<QByteArray>::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_const_iterator_operator__9(
                    self as *mut crate::q_list_of_q_byte_array::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::const_iterator QList<QByteArray>::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_const_iterator_operator__7(
                    self as *mut crate::q_list_of_q_byte_array::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QByteArray& QList<QByteArray>::const_iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValue const_iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::Ref<crate::QByteArray> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_const_iterator_operator__1(
                    self as *const crate::q_list_of_q_byte_array::ConstIterator,
                    j,
                );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QByteArray>::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>().</p></div>
        pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::ConstIterator>
        {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_const_iterator_const_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QByteArray>::const_iterator::const_iterator(const QList<QByteArray>::iterator& o)```</span>.
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_byte_array::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_const_iterator_const_iterator2(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_byte_array::Iterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QByteArray>::const_iterator::const_iterator(const QList<QByteArray>::const_iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_byte_array::ConstIterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_const_iterator_const_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_byte_array::ConstIterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current result.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QByteArray* QList<QByteArray>::const_iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current result.</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::Ptr<crate::QByteArray> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_const_iterator_operator__(
                    self as *const crate::q_list_of_q_byte_array::ConstIterator,
                );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_byte_array::Iterator {
        type Output = ::cpp_utils::MutRef<crate::QByteArray>;
        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QByteArray& QList<QByteArray>::iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>().</p></div>
        fn indirection(self) -> ::cpp_utils::MutRef<crate::QByteArray> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_iterator_operator_(
                    self as *const crate::q_list_of_q_byte_array::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_byte_array::Iterator>>
        for crate::q_list_of_q_byte_array::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QByteArray>::iterator::operator==(const QList<QByteArray>::iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_byte_array::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_iterator_operator__2(
                    self as *const crate::q_list_of_q_byte_array::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_byte_array::Iterator>>
        for crate::q_list_of_q_byte_array::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QByteArray>::iterator::operator<(const QList<QByteArray>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_byte_array::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_iterator_operator_1(
                    self as *const crate::q_list_of_q_byte_array::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_byte_array::Iterator>>
        for crate::q_list_of_q_byte_array::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QByteArray>::iterator::operator<=(const QList<QByteArray>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_byte_array::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_iterator_operator__4(
                    self as *const crate::q_list_of_q_byte_array::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_byte_array::Iterator>>
        for crate::q_list_of_q_byte_array::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QByteArray>::iterator::operator>(const QList<QByteArray>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_byte_array::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_iterator_operator_2(
                    self as *const crate::q_list_of_q_byte_array::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_byte_array::Iterator>>
        for crate::q_list_of_q_byte_array::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QByteArray>::iterator::operator>=(const QList<QByteArray>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_byte_array::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_iterator_operator__5(
                    self as *const crate::q_list_of_q_byte_array::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_byte_array::ConstIterator>>
        for crate::q_list_of_q_byte_array::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QByteArray>::iterator::operator==(const QList<QByteArray>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_byte_array::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_iterator_operator__6(
                    self as *const crate::q_list_of_q_byte_array::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_byte_array::ConstIterator>>
        for crate::q_list_of_q_byte_array::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QByteArray>::iterator::operator<(const QList<QByteArray>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_byte_array::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_iterator_operator_3(
                    self as *const crate::q_list_of_q_byte_array::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_byte_array::ConstIterator>>
        for crate::q_list_of_q_byte_array::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QByteArray>::iterator::operator<=(const QList<QByteArray>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_byte_array::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_iterator_operator__8(
                    self as *const crate::q_list_of_q_byte_array::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_byte_array::ConstIterator>>
        for crate::q_list_of_q_byte_array::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QByteArray>::iterator::operator>(const QList<QByteArray>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_byte_array::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_iterator_operator_4(
                    self as *const crate::q_list_of_q_byte_array::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_byte_array::ConstIterator>>
        for crate::q_list_of_q_byte_array::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QByteArray>::iterator::operator>=(const QList<QByteArray>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_byte_array::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_iterator_operator__9(
                    self as *const crate::q_list_of_q_byte_array::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_byte_array::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_byte_array::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::iterator& QList<QByteArray>::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_byte_array::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_iterator_operator__10(
                    self as *mut crate::q_list_of_q_byte_array::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_byte_array::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_byte_array::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::iterator& QList<QByteArray>::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_byte_array::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_iterator_operator__12(
                    self as *mut crate::q_list_of_q_byte_array::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong> for crate::q_list_of_q_byte_array::Iterator {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::iterator& QList<QByteArray>::iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_iterator_operator__14(
                    self as *mut crate::q_list_of_q_byte_array::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong> for crate::q_list_of_q_byte_array::Iterator {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::iterator& QList<QByteArray>::iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_iterator_operator__15(
                    self as *mut crate::q_list_of_q_byte_array::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_list_of_q_byte_array::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::iterator QList<QByteArray>::iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_iterator_operator_5(
                    self as *const crate::q_list_of_q_byte_array::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_list_of_q_byte_array::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::iterator QList<QByteArray>::iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_iterator_operator_6(
                    self as *const crate::q_list_of_q_byte_array::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_byte_array::Iterator>>
        for &crate::q_list_of_q_byte_array::Iterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QByteArray>::iterator::operator-(QList<QByteArray>::iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_byte_array::Iterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_iterator_operator_7(
                    self as *const crate::q_list_of_q_byte_array::Iterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_byte_array::ConstIterator {
        type Output = ::cpp_utils::Ref<crate::QByteArray>;
        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QByteArray& QList<QByteArray>::const_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        fn indirection(self) -> ::cpp_utils::Ref<crate::QByteArray> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_const_iterator_operator_(
                    self as *const crate::q_list_of_q_byte_array::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_byte_array::ConstIterator>>
        for crate::q_list_of_q_byte_array::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QByteArray>::const_iterator::operator==(const QList<QByteArray>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_byte_array::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_const_iterator_operator__2(
                    self as *const crate::q_list_of_q_byte_array::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_byte_array::ConstIterator>>
        for crate::q_list_of_q_byte_array::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QByteArray>::const_iterator::operator<(const QList<QByteArray>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_byte_array::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_const_iterator_operator_1(
                    self as *const crate::q_list_of_q_byte_array::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_byte_array::ConstIterator>>
        for crate::q_list_of_q_byte_array::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QByteArray>::const_iterator::operator<=(const QList<QByteArray>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_byte_array::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_const_iterator_operator__4(
                    self as *const crate::q_list_of_q_byte_array::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_byte_array::ConstIterator>>
        for crate::q_list_of_q_byte_array::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QByteArray>::const_iterator::operator>(const QList<QByteArray>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_byte_array::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_const_iterator_operator_2(
                    self as *const crate::q_list_of_q_byte_array::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_byte_array::ConstIterator>>
        for crate::q_list_of_q_byte_array::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QByteArray>::const_iterator::operator>=(const QList<QByteArray>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_byte_array::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_const_iterator_operator__5(
                    self as *const crate::q_list_of_q_byte_array::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_byte_array::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_byte_array::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::const_iterator& QList<QByteArray>::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_byte_array::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_const_iterator_operator__6(
                    self as *mut crate::q_list_of_q_byte_array::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_byte_array::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_byte_array::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::const_iterator& QList<QByteArray>::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_byte_array::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_const_iterator_operator__8(
                    self as *mut crate::q_list_of_q_byte_array::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_byte_array::ConstIterator
    {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::const_iterator& QList<QByteArray>::const_iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_const_iterator_operator__10(
                    self as *mut crate::q_list_of_q_byte_array::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_byte_array::ConstIterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::const_iterator& QList<QByteArray>::const_iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_const_iterator_operator__11(
                    self as *mut crate::q_list_of_q_byte_array::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_list_of_q_byte_array::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::const_iterator QList<QByteArray>::const_iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_const_iterator_operator_3(
                    self as *const crate::q_list_of_q_byte_array::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_list_of_q_byte_array::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::const_iterator QList<QByteArray>::const_iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_const_iterator_operator_4(
                    self as *const crate::q_list_of_q_byte_array::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_byte_array::ConstIterator>>
        for &crate::q_list_of_q_byte_array::ConstIterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QByteArray>::const_iterator::operator-(QList<QByteArray>::const_iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_byte_array::ConstIterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_const_iterator_operator_5(
                    self as *const crate::q_list_of_q_byte_array::ConstIterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_byte_array::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QByteArray>::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_iterator_diterator(
                self as *mut crate::q_list_of_q_byte_array::Iterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_byte_array::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QByteArray>::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_const_iterator_dconst_iterator(
                self as *mut crate::q_list_of_q_byte_array::ConstIterator,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
///
/// C++ class: <span style='color: green;'>```QList<QByteArray>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlist.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores items in a list that provides fast index-based access and index-based insertions and removals.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Internally, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T if <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. Otherwise, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T* and the items are allocated on the heap.</p>
/// <p>The array representation allows very fast insertions and index-based access. The <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() operations are also very fast because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates memory at both ends of its internal array. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.</p>
/// <p>Note, however, that when the conditions specified above are not met, each append or insert of a new item requires allocating the new item on the heap, and this per item allocation will make <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> a better choice for use cases that do a lot of appending or inserting, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> can allocate memory for many items in a single heap allocation.</p>
/// <p>Note that the internal array only ever gets bigger over the life of the list. It never shrinks. The internal array is deallocated by the destructor and by the assignment operator, when one list is assigned to another.</p>
/// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerList;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span><span class="operator">&gt;</span> dateList;
///
/// </pre>
/// <p>Qt includes a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class that inherits <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>&gt; and adds a few convenience functions, such as <a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#filter">QStringList::filter</a>(). <a href="http://doc.qt.io/qt-5/qstring.html#split">QString::split</a>() creates QStringLists from strings.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> stores a list of items. The default constructor creates an empty list. To insert items into the list, you can use operator&lt;&lt;():</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"one"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"two"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"three"</span>;
///   <span class="comment">// list: ["one", "two", "three"]</span>
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(). In addition, it provides the following convenience functions: <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const lists, operator[]() returns a reference to the item and can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Bob"</span>)
/// &#32;     list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Robert"</span>;
///
/// </pre>
/// <p>Because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented as an array of pointers for types that are larger than a pointer or are not movable, this operation requires (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>). For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> list<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (list<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Jane"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Jane at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>A common requirement is to remove an item from a list and do something with it. For this, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(). Here's a loop that removes the items from a list one at a time and calls <code>delete</code> on them:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   <span class="keyword">while</span> (<span class="operator">!</span>list<span class="operator">.</span>isEmpty())
/// &#32;     <span class="keyword">delete</span> list<span class="operator">.</span>takeFirst();
///
/// </pre>
/// <p>Inserting and removing items at either end of the list is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> in most cases), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
/// <p>If you want to find all occurrences of a particular value in a list, use <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of a matching item if they find it; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> list<span class="operator">.</span>indexOf(<span class="string">"Jane"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Jane is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a list contains a particular value, use <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the list, use <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>(). If you want to replace all occurrences of a particular value with another, use <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlistiterator.html">QListIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablelistiterator.html">QMutableListIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented in such a way that direct index-based access is just as fast as using iterators.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <p>To make <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as efficient as possible, its member functions don't validate their input before using it. Except for <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), member functions always assume the list is <i>not</i> empty. Member functions that take index values as parameters always assume their index value parameters are in the valid range. This means <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> member functions can fail. If you define QT_NO_DEBUG when you compile, failures will not be detected. If you <i>don't</i> define QT_NO_DEBUG, failures will be detected using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT">Q_ASSERT</a>() or <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT_X">Q_ASSERT_X</a>() with an appropriate message.</p>
/// <p>To avoid failures when your list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling other member functions. If you must pass an index value that might not be in the valid range, check that it is less than the value returned by <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() but <i>not</i> less than 0.</p>
/// <a name="more-members"></a></div>
#[repr(C)]
pub struct QListOfQByteArray {
    _unused: u8,
}
impl QListOfQByteArray {
    /// <p>Inserts <i>value</i> at the end of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QByteArray>::append(const QByteArray& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["one", "two", "three"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn append_q_byte_array(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_append(
            self as *mut crate::QListOfQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QByteArray>::append(const QList<QByteArray>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> list to this list.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_list_of_q_byte_array(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQByteArray>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_append1(
            self as *mut crate::QListOfQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQByteArray>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QByteArray& QList<QByteArray>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>This function is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_at(
            self as *const crate::QListOfQByteArray,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QList<QByteArray>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_back(
            self as *mut crate::QListOfQByteArray,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QByteArray& QList<QByteArray>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_back1(
            self as *const crate::QListOfQByteArray,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::const_iterator QList<QByteArray>::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_cbegin(
            self as *const crate::QListOfQByteArray,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::const_iterator QList<QByteArray>::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn cend(&self) -> ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_cend(
            self as *const crate::QListOfQByteArray,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all items from the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QByteArray>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all items from the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_clear(self as *mut crate::QListOfQByteArray)
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::const_iterator QList<QByteArray>::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_constBegin(
            self as *const crate::QListOfQByteArray,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::const_iterator QList<QByteArray>::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn const_end(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_constEnd(
            self as *const crate::QListOfQByteArray,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QByteArray& QList<QByteArray>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>().</p></div>
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_constFirst(
            self as *const crate::QListOfQByteArray,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QByteArray& QList<QByteArray>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>().</p></div>
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_constLast(
            self as *const crate::QListOfQByteArray,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QByteArray>::contains(const QByteArray& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn contains(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_contains(
            self as *const crate::QListOfQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>& QList<QByteArray>::operator=(const QList<QByteArray>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this list and returns a reference to this list.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQByteArray>>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_operator_(
            self as *mut crate::QListOfQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQByteArray>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of occurrences of <i>value</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QByteArray>::count(const QByteArray& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of occurrences of <i>value</i> in the list.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_count(
            self as *const crate::QListOfQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QByteArray>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_count1(
            self as *const crate::QListOfQByteArray,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QByteArray>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_detach(self as *mut crate::QListOfQByteArray)
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QByteArray>::detachShared()```</span>.
    pub unsafe fn detach_shared(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_detachShared(
            self as *mut crate::QListOfQByteArray,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QByteArray>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_empty(
            self as *const crate::QListOfQByteArray,
        )
    }

    /// <p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QByteArray>::endsWith(const QByteArray& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn ends_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_endsWith(
            self as *const crate::QListOfQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::iterator QList<QByteArray>::erase(QList<QByteArray>::iterator pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn erase_1a(
        &mut self,
        pos: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_byte_array::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_erase(self as *mut crate::QListOfQByteArray, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_byte_array::Iterator>>::cast_into(pos).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::iterator QList<QByteArray>::erase(QList<QByteArray>::iterator first, QList<QByteArray>::iterator last)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes all the items from <i>begin</i> up to (but not including) <i>end</i>. Returns an iterator to the same item that <i>end</i> referred to before the call.</p></div>
    pub unsafe fn erase_2a(
        &mut self,
        first: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_byte_array::Iterator>>,
        last: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_byte_array::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_erase1(self as *mut crate::QListOfQByteArray, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_byte_array::Iterator>>::cast_into(first).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_byte_array::Iterator>>::cast_into(last).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QList<QByteArray>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_first(
            self as *mut crate::QListOfQByteArray,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QByteArray& QList<QByteArray>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_first1(
            self as *const crate::QListOfQByteArray,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QList<QByteArray>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_front(
            self as *mut crate::QListOfQByteArray,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QByteArray& QList<QByteArray>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_front1(
            self as *const crate::QListOfQByteArray,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QByteArray& QList<QByteArray>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>(). This function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p></div>
    pub unsafe fn index(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_operator__2(
            self as *const crate::QListOfQByteArray,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QList<QByteArray>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If this function is called on a list that is currently being shared, it will trigger a copy of all elements. Otherwise, this function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>. If you do not want to modify the list you should use <a href="http://doc.qt.io/qt-5/qlist.html#at">QList::at</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_operator__3(
            self as *mut crate::QListOfQByteArray,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QByteArray>::indexOf(const QByteArray& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_indexOf(
            self as *const crate::QListOfQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(t).as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QByteArray>::indexOf(const QByteArray& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_indexOf1(
            self as *const crate::QListOfQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QByteArray>::insert(int i, const QByteArray& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   list<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// list: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn insert_int_q_byte_array(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_insert(
            self as *mut crate::QListOfQByteArray,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::iterator QList<QByteArray>::insert(QList<QByteArray>::iterator before, const QByteArray& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>value</i> in front of the item pointed to by the iterator <i>before</i>. Returns an iterator pointing at the inserted item. Note that the iterator passed to the function will be invalid after the call; the returned iterator should be used instead.</p></div>
    pub unsafe fn insert_iterator_q_byte_array(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_byte_array::Iterator>>,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_insert1(self as *mut crate::QListOfQByteArray, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_byte_array::Iterator>>::cast_into(before).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(t).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QByteArray>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_isDetached(
            self as *const crate::QListOfQByteArray,
        )
    }

    /// <p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QByteArray>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_isEmpty(
            self as *const crate::QListOfQByteArray,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QByteArray>::isSharedWith(const QList<QByteArray>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQByteArray>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_isSharedWith(
            self as *const crate::QListOfQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQByteArray>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QList<QByteArray>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_last(
            self as *mut crate::QListOfQByteArray,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QByteArray& QList<QByteArray>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_last1(
            self as *const crate::QListOfQByteArray,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QByteArray>::lastIndexOf(const QByteArray& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_lastIndexOf(
            self as *const crate::QListOfQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(t).as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QByteArray>::lastIndexOf(const QByteArray& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_lastIndexOf1(
            self as *const crate::QListOfQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QByteArray>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_length(
            self as *const crate::QListOfQByteArray,
        )
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QByteArray> QList<QByteArray>::mid(int pos, int length = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        length: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_mid(
            self as *const crate::QListOfQByteArray,
            pos,
            length,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QByteArray> QList<QByteArray>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_mid1(
            self as *const crate::QListOfQByteArray,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QByteArray>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>move(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "C", "D", "E", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This is the same as insert(<i>to</i>, <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<i>from</i>)).This function assumes that both <i>from</i> and <i>to</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>from</i> and <i>to</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_move(
            self as *mut crate::QListOfQByteArray,
            from,
            to,
        )
    }

    /// <p>Constructs an empty list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QByteArray>::QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty list.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QListOfQByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_QList();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QByteArray>::QList(const QList<QByteArray>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_QList1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQByteArray>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QByteArray>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_pop_back(
            self as *mut crate::QListOfQByteArray,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QByteArray>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_pop_front(
            self as *mut crate::QListOfQByteArray,
        )
    }

    /// <p>Inserts <i>value</i> at the beginning of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QByteArray>::prepend(const QByteArray& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(0, <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_prepend(
            self as *mut crate::QListOfQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QByteArray>::push_back(const QByteArray& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_push_back(
            self as *mut crate::QListOfQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QByteArray>::push_front(const QByteArray& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_push_front(
            self as *mut crate::QListOfQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QByteArray>::removeAll(const QByteArray& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeAll(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_all(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_removeAll(
            self as *mut crate::QListOfQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QByteArray>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_removeAt(
            self as *mut crate::QListOfQByteArray,
            i,
        )
    }

    /// <p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QByteArray>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_removeFirst(
            self as *mut crate::QListOfQByteArray,
        )
    }

    /// <p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QByteArray>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_removeLast(
            self as *mut crate::QListOfQByteArray,
        )
    }

    /// <p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QByteArray>::removeOne(const QByteArray& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeOne(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", ,"sun", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_one(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_removeOne(
            self as *mut crate::QListOfQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QByteArray>::replace(int i, const QByteArray& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_replace(
            self as *mut crate::QListOfQByteArray,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Reserve space for <i>alloc</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QByteArray>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reserve space for <i>alloc</i> elements.</p>
    /// <p>If <i>alloc</i> is smaller than the current size of the list, nothing will happen.</p>
    /// <p>Use this function to avoid repetetive reallocation of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s internal data if you can predict how many elements will be appended. Note that the reservation applies only to the internal pointer array.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_reserve(
            self as *mut crate::QListOfQByteArray,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QByteArray>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_setSharable(
            self as *mut crate::QListOfQByteArray,
            sharable,
        )
    }

    /// <p>Returns the number of items in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QByteArray>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_size(self as *const crate::QListOfQByteArray)
    }

    /// <p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QByteArray>::startsWith(const QByteArray& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn starts_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_startsWith(
            self as *const crate::QListOfQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QByteArray>::swap(QList<QByteArray>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap_1a(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QListOfQByteArray>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_swap(
            self as *mut crate::QListOfQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QListOfQByteArray>>::cast_into(
                other,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QByteArray>::swap(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swap(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    pub unsafe fn swap_2a(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_swap1(
            self as *mut crate::QListOfQByteArray,
            i,
            j,
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QByteArray>::swapItemsAt(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swapItemsAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swapItemsAt(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn swap_items_at(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_swapItemsAt(
            self as *mut crate::QListOfQByteArray,
            i,
            j,
        )
    }

    /// <p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QList<QByteArray>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_takeAt(
            self as *mut crate::QListOfQByteArray,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QList<QByteArray>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_takeFirst(
            self as *mut crate::QListOfQByteArray,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QList<QByteArray>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_takeLast(
            self as *mut crate::QListOfQByteArray,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value at index position <i>i</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QList<QByteArray>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the list.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that the index is going to be within bounds, you can use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_value(
            self as *const crate::QListOfQByteArray,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QList<QByteArray>::value(int i, const QByteArray& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_value1(
            self as *const crate::QListOfQByteArray,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> class is a template class that provides a dynamic array.</p>
///
/// C++ class: <span style='color: green;'>```QVector<QStringRef>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qvector.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> class is a template class that provides a dynamic array.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores its items in adjacent memory locations and provides fast index-based access.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Here's an example of a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> that stores <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerVector;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> stringVector;
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> stores its items in a vector (array). Typically, vectors are created with an initial size. For example, the following code constructs a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> with 200 elements:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">200</span>);
///
/// </pre>
/// <p>The elements are automatically initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you want to initialize the vector with a different value, pass that value as the second argument to the constructor:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">200</span><span class="operator">,</span> <span class="string">"Pass"</span>);
///
/// </pre>
/// <p>You can also call <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>() at any time to fill the vector with a value.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const vectors, operator[]() returns a reference to the item that can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (vector<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Liz"</span>)
/// &#32;     vector<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Elizabeth"</span>;
///
/// </pre>
/// <p>For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> vector<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (vector<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Alfonso"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Alfonso at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>Another way to access the data stored in a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> is to call <a href="http://doc.qt.io/qt-5/qvector.html#data">data</a>(). The function returns a pointer to the first item in the vector. You can use the pointer to directly access and modify the elements stored in the vector. The pointer is also useful if you need to pass a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to a function that accepts a plain C++ array.</p>
/// <p>If you want to find all occurrences of a particular value in a vector, use <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of the matching item if they found one; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> vector<span class="operator">.</span>indexOf(<span class="string">"Harumi"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Harumi is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a vector contains a particular value, use <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the vector, use <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(). With the exception of <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), these functions can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>) for large vectors, because they require moving many items in the vector by one position in memory. If you want a container class that provides fast insertion/removal in the middle, use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> or <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
/// <p>Unlike plain C++ arrays, QVectors can be resized at any time by calling <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>(). If the new size is larger than the old size, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> might need to reallocate the whole vector. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> tries to reduce the number of reallocations by preallocating up to twice as much memory as the actual data needs.</p>
/// <p>If you know in advance approximately how many items the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> will contain, you can call <a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>(), asking <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to preallocate a certain amount of memory. You can also call <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>() to find out how much memory <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> actually allocated.</p>
/// <p>Note that using non-const operators and functions can cause <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to do a deep copy of the data. This is due to <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicit sharing</a>.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qvectoriterator.html">QVectorIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablevectoriterator.html">QMutableVectorIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qvector.html#const_iterator-typedef">QVector::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qvector.html#iterator-typedefx">QVector::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>.</p>
/// <p>In addition to <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>, Qt also provides <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a>, a very low-level class with little functionality that is optimized for speed.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <a name="more-information-on-using-qt-containers"></a>
/// <h4>More Information on Using Qt Containers</h4>
/// <p>For a detailed discussion comparing Qt containers with each other and with STL containers, see <a href="http://marcmutz.wordpress.com/effective-qt/containers/">Understand the Qt Containers</a>.</p></div>
#[repr(C)]
pub struct QVectorOfQStringRef {
    _unused: u8,
}
impl QVectorOfQStringRef {
    /// <p>Inserts <i>value</i> at the end of the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStringRef>::append(const QStringRef& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   vector<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> three <span class="operator">=</span> <span class="string">"three"</span>;
    ///   vector<span class="operator">.</span>append(three);
    ///   <span class="comment">// vector: ["one", "two", "three"]</span>
    ///   <span class="comment">// three: "three"</span>
    ///
    /// </pre>
    /// <p>This is the same as calling resize(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() + 1) and assigning <i>value</i> to the new last element in the vector.</p>
    /// <p>This operation is relatively fast, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> typically allocates more memory than necessary, so it can grow without reallocating the entire vector each time.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>().</p></div>
    pub unsafe fn append_q_string_ref(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_append(
            self as *mut crate::QVectorOfQStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStringRef>::append(const QVector<QStringRef>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#append-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> vector to this vector.</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_vector_of_q_string_ref(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQStringRef>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_append2(
            self as *mut crate::QVectorOfQStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQStringRef>>::cast_into(l)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the item at index position <i>i</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStringRef& QVector<QStringRef>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the vector.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_at(
            self as *const crate::QVectorOfQStringRef,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef& QVector<QStringRef>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::MutRef<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_back(
            self as *mut crate::QVectorOfQStringRef,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStringRef& QVector<QStringRef>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_back1(
            self as *const crate::QVectorOfQStringRef,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the maximum number of items that can be stored in the vector without forcing a reallocation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QStringRef>::capacity() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#capacity">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the maximum number of items that can be stored in the vector without forcing a reallocation.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s memory usage. In general, you will rarely ever need to call this function. If you want to know how many items are in the vector, call <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p></div>
    pub unsafe fn capacity(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_capacity(
            self as *const crate::QVectorOfQStringRef,
        )
    }

    /// <p>Removes all the elements from the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStringRef>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all the elements from the vector.</p>
    /// <p><b>Note: </b>Until Qt 5.6, this also released the memory used by the vector. From Qt 5.7, the capacity is preserved. To shed all capacity, swap with a default-constructed vector:</p><pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span>T<span class="operator">&gt;</span> v <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span>T<span class="operator">&gt;</span>()<span class="operator">.</span>swap(v);
    ///   Q_ASSERT(v<span class="operator">.</span>capacity() <span class="operator">=</span><span class="operator">=</span> <span class="number">0</span>);
    ///
    /// </pre>
    /// <p>or call <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_clear(
            self as *mut crate::QVectorOfQStringRef,
        )
    }

    /// <p>Returns a const pointer to the data stored in the vector. The pointer can be used to access the items in the vector. The pointer remains valid as long as the vector isn't reallocated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStringRef* QVector<QStringRef>::constData() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const pointer to the data stored in the vector. The pointer can be used to access the items in the vector. The pointer remains valid as long as the vector isn't reallocated.</p>
    /// <p>This function is mostly useful to pass a vector to a function that accepts a plain C++ array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#data">data</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn const_data(&self) -> ::cpp_utils::Ptr<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_constData(
            self as *const crate::QVectorOfQStringRef,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns a const reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStringRef& QVector<QStringRef>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_constFirst(
            self as *const crate::QVectorOfQStringRef,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStringRef& QVector<QStringRef>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_constLast(
            self as *const crate::QVectorOfQStringRef,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the vector contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QStringRef>::contains(const QStringRef& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the vector contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p></div>
    pub unsafe fn contains(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_contains(
            self as *const crate::QVectorOfQStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStringRef>& QVector<QStringRef>::operator=(const QVector<QStringRef>& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this vector and returns a reference to this vector.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        v: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQStringRef>>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_operator_(
            self as *mut crate::QVectorOfQStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQStringRef>>::cast_into(v)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of occurrences of <i>value</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QStringRef>::count(const QStringRef& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of occurrences of <i>value</i> in the vector.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_count(
            self as *const crate::QVectorOfQStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QStringRef>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_count1(
            self as *const crate::QVectorOfQStringRef,
        )
    }

    /// <p>Returns a pointer to the data stored in the vector. The pointer can be used to access and modify the items in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef* QVector<QStringRef>::data()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the data stored in the vector. The pointer can be used to access and modify the items in the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> vector(<span class="number">10</span>);
    ///   <span class="type">int</span> <span class="operator">*</span>data <span class="operator">=</span> vector<span class="operator">.</span>data();
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> <span class="number">10</span>; <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     data<span class="operator">[</span>i<span class="operator">]</span> <span class="operator">=</span> <span class="number">2</span> <span class="operator">*</span> i;
    ///
    /// </pre>
    /// <p>The pointer remains valid as long as the vector isn't reallocated.</p>
    /// <p>This function is mostly useful to pass a vector to a function that accepts a plain C++ array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constData">constData</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn data_mut(&mut self) -> ::cpp_utils::MutPtr<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_data(
            self as *mut crate::QVectorOfQStringRef,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStringRef* QVector<QStringRef>::data() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#data-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn data(&self) -> ::cpp_utils::Ptr<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_data1(
            self as *const crate::QVectorOfQStringRef,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```void QVector<QStringRef>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_detach(
            self as *mut crate::QVectorOfQStringRef,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), returning <code>true</code> if the vector is empty; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QStringRef>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), returning <code>true</code> if the vector is empty; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_empty(
            self as *const crate::QVectorOfQStringRef,
        )
    }

    /// <p>Returns <code>true</code> if this vector is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QStringRef>::endsWith(const QStringRef& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this vector is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn ends_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_endsWith(
            self as *const crate::QVectorOfQStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStringRef>& QVector<QStringRef>::fill(const QStringRef& t, int size = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">3</span>);
    ///   vector<span class="operator">.</span>fill(<span class="string">"Yes"</span>);
    ///   <span class="comment">// vector: ["Yes", "Yes", "Yes"]</span>
    ///
    ///   vector<span class="operator">.</span>fill(<span class="string">"oh"</span><span class="operator">,</span> <span class="number">5</span>);
    ///   <span class="comment">// vector: ["oh", "oh", "oh", "oh", "oh"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_2a(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_fill(
            self as *mut crate::QVectorOfQStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(t).as_raw_ptr(),
            size,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStringRef>& QVector<QStringRef>::fill(const QStringRef& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">3</span>);
    ///   vector<span class="operator">.</span>fill(<span class="string">"Yes"</span>);
    ///   <span class="comment">// vector: ["Yes", "Yes", "Yes"]</span>
    ///
    ///   vector<span class="operator">.</span>fill(<span class="string">"oh"</span><span class="operator">,</span> <span class="number">5</span>);
    ///   <span class="comment">// vector: ["oh", "oh", "oh", "oh", "oh"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_1a(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_fill1(
            self as *mut crate::QVectorOfQStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(t).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef& QVector<QStringRef>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#constFirst">constFirst</a>().</p></div>
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_first(
            self as *mut crate::QVectorOfQStringRef,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStringRef& QVector<QStringRef>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_first1(
            self as *const crate::QVectorOfQStringRef,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef& QVector<QStringRef>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::MutRef<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_front(
            self as *mut crate::QVectorOfQStringRef,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStringRef& QVector<QStringRef>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_front1(
            self as *const crate::QVectorOfQStringRef,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef& QVector<QStringRef>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p>Note that using non-const operators can cause <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to do a deep copy.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_operator__2(
            self as *mut crate::QVectorOfQStringRef,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStringRef& QVector<QStringRef>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as at(<i>i</i>).</p></div>
    pub unsafe fn index(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_operator__3(
            self as *const crate::QVectorOfQStringRef,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QStringRef>::indexOf(const QStringRef& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span>);            <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);         <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);         <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"X"</span>);            <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_indexOf(
            self as *const crate::QVectorOfQStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(t).as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QStringRef>::indexOf(const QStringRef& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span>);            <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);         <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);         <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"X"</span>);            <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_indexOf1(
            self as *const crate::QVectorOfQStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the vector. If <i>i</i> is 0, the value is prepended to the vector. If <i>i</i> is <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), the value is appended to the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStringRef>::insert(int i, const QStringRef& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the vector. If <i>i</i> is 0, the value is prepended to the vector. If <i>i</i> is <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), the value is appended to the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   vector<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// vector: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p>For large vectors, this operation can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because it requires moving all the items at indexes <i>i</i> and above by one position further in memory. If you want a container class that provides a fast insert() function, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>().</p></div>
    pub unsafe fn insert_2a(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_insert(
            self as *mut crate::QVectorOfQStringRef,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStringRef>::insert(int i, int n, const QStringRef& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>count</i> copies of <i>value</i> at index position <i>i</i> in the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">double</span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">2.718</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">1.442</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">0.4342</span>;
    ///   vector<span class="operator">.</span>insert(<span class="number">1</span><span class="operator">,</span> <span class="number">3</span><span class="operator">,</span> <span class="number">9.9</span>);
    ///   <span class="comment">// vector: [2.718, 9.9, 9.9, 9.9, 1.442, 0.4342]</span>
    ///
    /// </pre></div>
    pub unsafe fn insert_3a(
        &mut self,
        i: ::std::os::raw::c_int,
        n: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_insert1(
            self as *mut crate::QVectorOfQStringRef,
            i,
            n,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QVector<QStringRef>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_isDetached(
            self as *const crate::QVectorOfQStringRef,
        )
    }

    /// <p>Returns <code>true</code> if the vector has size 0; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QStringRef>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the vector has size 0; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_isEmpty(
            self as *const crate::QVectorOfQStringRef,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QVector<QStringRef>::isSharedWith(const QVector<QStringRef>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQStringRef>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_isSharedWith(
            self as *const crate::QVectorOfQStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQStringRef>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef& QVector<QStringRef>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#constLast">constLast</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_last(
            self as *mut crate::QVectorOfQStringRef,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStringRef& QVector<QStringRef>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_last1(
            self as *const crate::QVectorOfQStringRef,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QStringRef>::lastIndexOf(const QStringRef& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);        <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);     <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);     <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_lastIndexOf(
            self as *const crate::QVectorOfQStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(t).as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QStringRef>::lastIndexOf(const QStringRef& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);        <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);     <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);     <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_lastIndexOf1(
            self as *const crate::QVectorOfQStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QStringRef>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#length">QList::length</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_length(
            self as *const crate::QVectorOfQStringRef,
        )
    }

    /// <p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStringRef> QVector<QStringRef>::mid(int pos, int len = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        len: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_mid(
            self as *const crate::QVectorOfQStringRef,
            pos,
            len,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStringRef> QVector<QStringRef>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_mid1(
            self as *const crate::QVectorOfQStringRef,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStringRef>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">QList::move</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_move(
            self as *mut crate::QVectorOfQStringRef,
            from,
            to,
        )
    }

    /// <p>Constructs an empty vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QStringRef>::QVector()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QVectorOfQStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_QVector();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a vector with an initial size of <i>size</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QStringRef>::QVector(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a vector with an initial size of <i>size</i> elements.</p>
    /// <p>The elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn new_1a(
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_QVector1(size);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a vector with an initial size of <i>size</i> elements. Each element is initialized with <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QStringRef>::QVector(int size, const QStringRef& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a vector with an initial size of <i>size</i> elements. Each element is initialized with <i>value</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn new_2a(
        size: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_QVector2(
            size,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(t).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QStringRef>::QVector(const QVector<QStringRef>& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        v: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQStringRef>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_QVector3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQStringRef>>::cast_into(v)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStringRef>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_pop_back(
            self as *mut crate::QVectorOfQStringRef,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStringRef>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_pop_front(
            self as *mut crate::QVectorOfQStringRef,
        )
    }

    /// <p>Inserts <i>value</i> at the beginning of the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStringRef>::prepend(const QStringRef& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   vector<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   vector<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// vector: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as vector.insert(0, <i>value</i>).</p>
    /// <p>For large vectors, this operation can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because it requires moving all the items in the vector by one position further in memory. If you want a container class that provides a fast prepend() function, use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> or <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_prepend(
            self as *mut crate::QVectorOfQStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to append(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStringRef>::push_back(const QStringRef& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to append(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_push_back(
            self as *mut crate::QVectorOfQStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to prepend(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStringRef>::push_front(const QStringRef& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to prepend(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_push_front(
            self as *mut crate::QVectorOfQStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStringRef>::remove(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#remove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes the element at index position <i>i</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn remove_1a(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_remove(
            self as *mut crate::QVectorOfQStringRef,
            i,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStringRef>::remove(int i, int n)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#remove-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes <i>count</i> elements from the middle of the vector, starting at index position <i>i</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn remove_2a(&mut self, i: ::std::os::raw::c_int, n: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_remove1(
            self as *mut crate::QVectorOfQStringRef,
            i,
            n,
        )
    }

    /// <p>Removes all elements that compare equal to <i>t</i> from the vector. Returns the number of elements removed, if any.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QStringRef>::removeAll(const QStringRef& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all elements that compare equal to <i>t</i> from the vector. Returns the number of elements removed, if any.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#removeOne">removeOne</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">QList::removeAll</a>().</p></div>
    pub unsafe fn remove_all(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_removeAll(
            self as *mut crate::QVectorOfQStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes the element at index position <i>i</i>. Equivalent to</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStringRef>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the element at index position <i>i</i>. Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   remove(i);
    ///
    /// </pre>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">QList::removeAt</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_removeAt(
            self as *mut crate::QVectorOfQStringRef,
            i,
        )
    }

    /// <p>Removes the first item in the vector. Calling this function is equivalent to calling remove(0). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStringRef>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the vector. Calling this function is equivalent to calling remove(0). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_removeFirst(
            self as *mut crate::QVectorOfQStringRef,
        )
    }

    /// <p>Removes the last item in the vector. Calling this function is equivalent to calling remove(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() - 1). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStringRef>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the vector. Calling this function is equivalent to calling remove(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() - 1). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_removeLast(
            self as *mut crate::QVectorOfQStringRef,
        )
    }

    /// <p>Removes the first element that compares equal to <i>t</i> from the vector. Returns whether an element was, in fact, removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QStringRef>::removeOne(const QStringRef& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeOne">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first element that compares equal to <i>t</i> from the vector. Returns whether an element was, in fact, removed.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#removeAll">removeAll</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">QList::removeOne</a>().</p></div>
    pub unsafe fn remove_one(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_removeOne(
            self as *mut crate::QVectorOfQStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStringRef>::replace(int i, const QStringRef& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_replace(
            self as *mut crate::QVectorOfQStringRef,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Attempts to allocate memory for at least <i>size</i> elements. If you know in advance how large the vector will be, you should call this function to prevent reallocations and memory fragmentation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStringRef>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to allocate memory for at least <i>size</i> elements. If you know in advance how large the vector will be, you should call this function to prevent reallocations and memory fragmentation.</p>
    /// <p>If <i>size</i> is an underestimate, the worst that will happen is that the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> will be a bit slower. If <i>size</i> is an overestimate, you may have used more memory than the normal <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> growth strategy would have allocated—or you may have used less.</p>
    /// <p>An alternative to reserve() is calling <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>(). Whether or not that is faster than reserve() depends on the element type, because <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() default-constructs all elements, and requires assignment to existing entries rather than calling <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(), which copy- or move-constructs. For simple types, like <code>int</code> or <code>double</code>, <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() is typically faster, but for anything more complex, you should prefer reserve().</p>
    /// <p><b>Warning:</b> If the size passed to <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() was underestimated, you run out of allocated space and into undefined behavior. This problem does not exist with reserve(), because it treats the size as just a hint.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>().</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_reserve(
            self as *mut crate::QVectorOfQStringRef,
            size,
        )
    }

    /// <p>Sets the size of the vector to <i>size</i>. If <i>size</i> is greater than the current size, elements are added to the end; the new elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If <i>size</i> is less than the current size, elements are removed from the end.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStringRef>::resize(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#resize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the size of the vector to <i>size</i>. If <i>size</i> is greater than the current size, elements are added to the end; the new elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If <i>size</i> is less than the current size, elements are removed from the end.</p>
    /// <p>Since Qt 5.6, resize() doesn't shrink the capacity anymore. To shed excess capacity, use <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p></div>
    pub unsafe fn resize(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_resize(
            self as *mut crate::QVectorOfQStringRef,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QVector<QStringRef>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_setSharable(
            self as *mut crate::QVectorOfQStringRef,
            sharable,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStringRef>::shrink_to_fit()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#shrink_to_fit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn shrink_to_fit(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_shrink_to_fit(
            self as *mut crate::QVectorOfQStringRef,
        )
    }

    /// <p>Returns the number of items in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QStringRef>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_size(
            self as *const crate::QVectorOfQStringRef,
        )
    }

    /// <p>Releases any memory not required to store the items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStringRef>::squeeze()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Releases any memory not required to store the items.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s memory usage. In general, you will rarely ever need to call this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>().</p></div>
    pub unsafe fn squeeze(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_squeeze(
            self as *mut crate::QVectorOfQStringRef,
        )
    }

    /// <p>Returns <code>true</code> if this vector is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QStringRef>::startsWith(const QStringRef& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this vector is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn starts_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_startsWith(
            self as *const crate::QVectorOfQStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Swaps vector <i>other</i> with this vector. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStringRef>::swap(QVector<QStringRef>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps vector <i>other</i> with this vector. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QVectorOfQStringRef>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_swap(
            self as *mut crate::QVectorOfQStringRef,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QVectorOfQStringRef>>::cast_into(
                other,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Removes the element at index position <i>i</i> and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QVector<QStringRef>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the element at index position <i>i</i> and returns it.</p>
    /// <p>Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   T t <span class="operator">=</span> at(i);
    ///   remove(i);
    ///   <span class="keyword">return</span> t;
    ///
    /// </pre>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">QList::takeAt</a>().</p></div>
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_takeAt(
            self as *mut crate::QVectorOfQStringRef,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the first item in the vector and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QVector<QStringRef>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the vector and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_takeFirst(
            self as *mut crate::QVectorOfQStringRef,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the last item in the list and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QVector<QStringRef>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_takeLast(
            self as *mut crate::QVectorOfQStringRef,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value at index position <i>i</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QVector<QStringRef>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the vector.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that <i>i</i> is within bounds, you can use <a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_value(
            self as *const crate::QVectorOfQStringRef,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringRef QVector<QStringRef>::value(int i, const QStringRef& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    ) -> ::cpp_utils::CppBox<crate::QStringRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_value1(
            self as *const crate::QVectorOfQStringRef,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> class is a template class that provides a dynamic array.</p>
///
/// C++ class: <span style='color: green;'>```QVector<unsigned int>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qvector.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> class is a template class that provides a dynamic array.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores its items in adjacent memory locations and provides fast index-based access.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Here's an example of a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> that stores <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerVector;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> stringVector;
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> stores its items in a vector (array). Typically, vectors are created with an initial size. For example, the following code constructs a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> with 200 elements:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">200</span>);
///
/// </pre>
/// <p>The elements are automatically initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you want to initialize the vector with a different value, pass that value as the second argument to the constructor:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">200</span><span class="operator">,</span> <span class="string">"Pass"</span>);
///
/// </pre>
/// <p>You can also call <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>() at any time to fill the vector with a value.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const vectors, operator[]() returns a reference to the item that can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (vector<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Liz"</span>)
/// &#32;     vector<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Elizabeth"</span>;
///
/// </pre>
/// <p>For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> vector<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (vector<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Alfonso"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Alfonso at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>Another way to access the data stored in a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> is to call <a href="http://doc.qt.io/qt-5/qvector.html#data">data</a>(). The function returns a pointer to the first item in the vector. You can use the pointer to directly access and modify the elements stored in the vector. The pointer is also useful if you need to pass a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to a function that accepts a plain C++ array.</p>
/// <p>If you want to find all occurrences of a particular value in a vector, use <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of the matching item if they found one; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> vector<span class="operator">.</span>indexOf(<span class="string">"Harumi"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Harumi is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a vector contains a particular value, use <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the vector, use <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(). With the exception of <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), these functions can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>) for large vectors, because they require moving many items in the vector by one position in memory. If you want a container class that provides fast insertion/removal in the middle, use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> or <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
/// <p>Unlike plain C++ arrays, QVectors can be resized at any time by calling <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>(). If the new size is larger than the old size, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> might need to reallocate the whole vector. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> tries to reduce the number of reallocations by preallocating up to twice as much memory as the actual data needs.</p>
/// <p>If you know in advance approximately how many items the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> will contain, you can call <a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>(), asking <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to preallocate a certain amount of memory. You can also call <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>() to find out how much memory <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> actually allocated.</p>
/// <p>Note that using non-const operators and functions can cause <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to do a deep copy of the data. This is due to <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicit sharing</a>.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qvectoriterator.html">QVectorIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablevectoriterator.html">QMutableVectorIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qvector.html#const_iterator-typedef">QVector::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qvector.html#iterator-typedefx">QVector::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>.</p>
/// <p>In addition to <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>, Qt also provides <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a>, a very low-level class with little functionality that is optimized for speed.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <a name="more-information-on-using-qt-containers"></a>
/// <h4>More Information on Using Qt Containers</h4>
/// <p>For a detailed discussion comparing Qt containers with each other and with STL containers, see <a href="http://marcmutz.wordpress.com/effective-qt/containers/">Understand the Qt Containers</a>.</p></div>
#[repr(C)]
pub struct QVectorOfUint {
    _unused: u8,
}
impl QVectorOfUint {
    /// <p>Inserts <i>value</i> at the end of the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<unsigned int>::append(const unsigned int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   vector<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> three <span class="operator">=</span> <span class="string">"three"</span>;
    ///   vector<span class="operator">.</span>append(three);
    ///   <span class="comment">// vector: ["one", "two", "three"]</span>
    ///   <span class="comment">// three: "three"</span>
    ///
    /// </pre>
    /// <p>This is the same as calling resize(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() + 1) and assigning <i>value</i> to the new last element in the vector.</p>
    /// <p>This operation is relatively fast, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> typically allocates more memory than necessary, so it can grow without reallocating the entire vector each time.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>().</p></div>
    pub unsafe fn append_uint(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_uint>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_append(
            self as *mut crate::QVectorOfUint,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_uint>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<unsigned int>::append(const QVector<unsigned int>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#append-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> vector to this vector.</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_vector_of_uint(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfUint>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_append2(
            self as *mut crate::QVectorOfUint,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfUint>>::cast_into(l)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the item at index position <i>i</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const unsigned int& QVector<unsigned int>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the vector.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<::std::os::raw::c_uint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_at(
            self as *const crate::QVectorOfUint,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int& QVector<unsigned int>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::MutRef<::std::os::raw::c_uint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_back(
            self as *mut crate::QVectorOfUint,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const unsigned int& QVector<unsigned int>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<::std::os::raw::c_uint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_back1(
            self as *const crate::QVectorOfUint,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the maximum number of items that can be stored in the vector without forcing a reallocation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<unsigned int>::capacity() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#capacity">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the maximum number of items that can be stored in the vector without forcing a reallocation.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s memory usage. In general, you will rarely ever need to call this function. If you want to know how many items are in the vector, call <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p></div>
    pub unsafe fn capacity(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_capacity(
            self as *const crate::QVectorOfUint,
        )
    }

    /// <p>Removes all the elements from the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<unsigned int>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all the elements from the vector.</p>
    /// <p><b>Note: </b>Until Qt 5.6, this also released the memory used by the vector. From Qt 5.7, the capacity is preserved. To shed all capacity, swap with a default-constructed vector:</p><pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span>T<span class="operator">&gt;</span> v <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span>T<span class="operator">&gt;</span>()<span class="operator">.</span>swap(v);
    ///   Q_ASSERT(v<span class="operator">.</span>capacity() <span class="operator">=</span><span class="operator">=</span> <span class="number">0</span>);
    ///
    /// </pre>
    /// <p>or call <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_clear(self as *mut crate::QVectorOfUint)
    }

    /// <p>Returns a const pointer to the data stored in the vector. The pointer can be used to access the items in the vector. The pointer remains valid as long as the vector isn't reallocated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const unsigned int* QVector<unsigned int>::constData() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const pointer to the data stored in the vector. The pointer can be used to access the items in the vector. The pointer remains valid as long as the vector isn't reallocated.</p>
    /// <p>This function is mostly useful to pass a vector to a function that accepts a plain C++ array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#data">data</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn const_data(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_uint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_constData(
            self as *const crate::QVectorOfUint,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns a const reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const unsigned int& QVector<unsigned int>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<::std::os::raw::c_uint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_constFirst(
            self as *const crate::QVectorOfUint,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const unsigned int& QVector<unsigned int>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<::std::os::raw::c_uint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_constLast(
            self as *const crate::QVectorOfUint,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the vector contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<unsigned int>::contains(const unsigned int& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the vector contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p></div>
    pub unsafe fn contains(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_uint>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_contains(
            self as *const crate::QVectorOfUint,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_uint>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<unsigned int>& QVector<unsigned int>::operator=(const QVector<unsigned int>& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this vector and returns a reference to this vector.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        v: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfUint>>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfUint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_operator_(
            self as *mut crate::QVectorOfUint,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfUint>>::cast_into(v)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of occurrences of <i>value</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<unsigned int>::count(const unsigned int& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of occurrences of <i>value</i> in the vector.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_uint>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_count(
            self as *const crate::QVectorOfUint,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_uint>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<unsigned int>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_count1(
            self as *const crate::QVectorOfUint,
        )
    }

    /// <p>Returns a pointer to the data stored in the vector. The pointer can be used to access and modify the items in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int* QVector<unsigned int>::data()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the data stored in the vector. The pointer can be used to access and modify the items in the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> vector(<span class="number">10</span>);
    ///   <span class="type">int</span> <span class="operator">*</span>data <span class="operator">=</span> vector<span class="operator">.</span>data();
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> <span class="number">10</span>; <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     data<span class="operator">[</span>i<span class="operator">]</span> <span class="operator">=</span> <span class="number">2</span> <span class="operator">*</span> i;
    ///
    /// </pre>
    /// <p>The pointer remains valid as long as the vector isn't reallocated.</p>
    /// <p>This function is mostly useful to pass a vector to a function that accepts a plain C++ array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constData">constData</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn data_mut(&mut self) -> ::cpp_utils::MutPtr<::std::os::raw::c_uint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_data(
            self as *mut crate::QVectorOfUint,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const unsigned int* QVector<unsigned int>::data() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#data-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn data(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_uint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_data1(
            self as *const crate::QVectorOfUint,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```void QVector<unsigned int>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_detach(self as *mut crate::QVectorOfUint)
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), returning <code>true</code> if the vector is empty; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<unsigned int>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), returning <code>true</code> if the vector is empty; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_empty(
            self as *const crate::QVectorOfUint,
        )
    }

    /// <p>Returns <code>true</code> if this vector is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<unsigned int>::endsWith(const unsigned int& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this vector is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn ends_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_uint>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_endsWith(
            self as *const crate::QVectorOfUint,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_uint>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<unsigned int>& QVector<unsigned int>::fill(const unsigned int& t, int size = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">3</span>);
    ///   vector<span class="operator">.</span>fill(<span class="string">"Yes"</span>);
    ///   <span class="comment">// vector: ["Yes", "Yes", "Yes"]</span>
    ///
    ///   vector<span class="operator">.</span>fill(<span class="string">"oh"</span><span class="operator">,</span> <span class="number">5</span>);
    ///   <span class="comment">// vector: ["oh", "oh", "oh", "oh", "oh"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_2a(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_uint>>,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfUint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_fill(
            self as *mut crate::QVectorOfUint,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_uint>>::cast_into(t)
                .as_raw_ptr(),
            size,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<unsigned int>& QVector<unsigned int>::fill(const unsigned int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">3</span>);
    ///   vector<span class="operator">.</span>fill(<span class="string">"Yes"</span>);
    ///   <span class="comment">// vector: ["Yes", "Yes", "Yes"]</span>
    ///
    ///   vector<span class="operator">.</span>fill(<span class="string">"oh"</span><span class="operator">,</span> <span class="number">5</span>);
    ///   <span class="comment">// vector: ["oh", "oh", "oh", "oh", "oh"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_1a(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_uint>>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfUint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_fill1(
            self as *mut crate::QVectorOfUint,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_uint>>::cast_into(t)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int& QVector<unsigned int>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#constFirst">constFirst</a>().</p></div>
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<::std::os::raw::c_uint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_first(
            self as *mut crate::QVectorOfUint,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const unsigned int& QVector<unsigned int>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<::std::os::raw::c_uint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_first1(
            self as *const crate::QVectorOfUint,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int& QVector<unsigned int>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::MutRef<::std::os::raw::c_uint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_front(
            self as *mut crate::QVectorOfUint,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const unsigned int& QVector<unsigned int>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<::std::os::raw::c_uint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_front1(
            self as *const crate::QVectorOfUint,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int& QVector<unsigned int>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p>Note that using non-const operators can cause <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to do a deep copy.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<::std::os::raw::c_uint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_operator__2(
            self as *mut crate::QVectorOfUint,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const unsigned int& QVector<unsigned int>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as at(<i>i</i>).</p></div>
    pub unsafe fn index(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ref<::std::os::raw::c_uint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_operator__3(
            self as *const crate::QVectorOfUint,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<unsigned int>::indexOf(const unsigned int& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span>);            <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);         <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);         <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"X"</span>);            <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_uint>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_indexOf(
            self as *const crate::QVectorOfUint,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_uint>>::cast_into(t)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<unsigned int>::indexOf(const unsigned int& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span>);            <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);         <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);         <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"X"</span>);            <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_uint>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_indexOf1(
            self as *const crate::QVectorOfUint,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_uint>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the vector. If <i>i</i> is 0, the value is prepended to the vector. If <i>i</i> is <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), the value is appended to the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<unsigned int>::insert(int i, const unsigned int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the vector. If <i>i</i> is 0, the value is prepended to the vector. If <i>i</i> is <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), the value is appended to the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   vector<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// vector: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p>For large vectors, this operation can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because it requires moving all the items at indexes <i>i</i> and above by one position further in memory. If you want a container class that provides a fast insert() function, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>().</p></div>
    pub unsafe fn insert_2a(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_uint>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_insert(
            self as *mut crate::QVectorOfUint,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_uint>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<unsigned int>::insert(int i, int n, const unsigned int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>count</i> copies of <i>value</i> at index position <i>i</i> in the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">double</span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">2.718</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">1.442</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">0.4342</span>;
    ///   vector<span class="operator">.</span>insert(<span class="number">1</span><span class="operator">,</span> <span class="number">3</span><span class="operator">,</span> <span class="number">9.9</span>);
    ///   <span class="comment">// vector: [2.718, 9.9, 9.9, 9.9, 1.442, 0.4342]</span>
    ///
    /// </pre></div>
    pub unsafe fn insert_3a(
        &mut self,
        i: ::std::os::raw::c_int,
        n: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_uint>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_insert1(
            self as *mut crate::QVectorOfUint,
            i,
            n,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_uint>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QVector<unsigned int>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_isDetached(
            self as *const crate::QVectorOfUint,
        )
    }

    /// <p>Returns <code>true</code> if the vector has size 0; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<unsigned int>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the vector has size 0; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_isEmpty(
            self as *const crate::QVectorOfUint,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QVector<unsigned int>::isSharedWith(const QVector<unsigned int>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfUint>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_isSharedWith(
            self as *const crate::QVectorOfUint,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfUint>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int& QVector<unsigned int>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#constLast">constLast</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<::std::os::raw::c_uint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_last(
            self as *mut crate::QVectorOfUint,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const unsigned int& QVector<unsigned int>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<::std::os::raw::c_uint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_last1(
            self as *const crate::QVectorOfUint,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<unsigned int>::lastIndexOf(const unsigned int& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);        <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);     <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);     <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_uint>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_lastIndexOf(
            self as *const crate::QVectorOfUint,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_uint>>::cast_into(t)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<unsigned int>::lastIndexOf(const unsigned int& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);        <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);     <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);     <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_uint>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_lastIndexOf1(
            self as *const crate::QVectorOfUint,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_uint>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<unsigned int>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#length">QList::length</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_length(
            self as *const crate::QVectorOfUint,
        )
    }

    /// <p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<unsigned int> QVector<unsigned int>::mid(int pos, int len = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        len: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfUint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_mid(
            self as *const crate::QVectorOfUint,
            pos,
            len,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<unsigned int> QVector<unsigned int>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfUint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_mid1(
            self as *const crate::QVectorOfUint,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<unsigned int>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">QList::move</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_move(
            self as *mut crate::QVectorOfUint,
            from,
            to,
        )
    }

    /// <p>Constructs an empty vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<unsigned int>::QVector()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QVectorOfUint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_QVector();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a vector with an initial size of <i>size</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<unsigned int>::QVector(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a vector with an initial size of <i>size</i> elements.</p>
    /// <p>The elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn new_1a(size: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QVectorOfUint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_QVector1(size);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a vector with an initial size of <i>size</i> elements. Each element is initialized with <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<unsigned int>::QVector(int size, const unsigned int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a vector with an initial size of <i>size</i> elements. Each element is initialized with <i>value</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn new_2a(
        size: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_uint>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfUint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_QVector2(
            size,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_uint>>::cast_into(t)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<unsigned int>::QVector(const QVector<unsigned int>& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        v: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfUint>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfUint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_QVector3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfUint>>::cast_into(v)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<unsigned int>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_pop_back(
            self as *mut crate::QVectorOfUint,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<unsigned int>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_pop_front(
            self as *mut crate::QVectorOfUint,
        )
    }

    /// <p>Inserts <i>value</i> at the beginning of the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<unsigned int>::prepend(const unsigned int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   vector<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   vector<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// vector: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as vector.insert(0, <i>value</i>).</p>
    /// <p>For large vectors, this operation can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because it requires moving all the items in the vector by one position further in memory. If you want a container class that provides a fast prepend() function, use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> or <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_uint>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_prepend(
            self as *mut crate::QVectorOfUint,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_uint>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to append(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<unsigned int>::push_back(const unsigned int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to append(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_uint>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_push_back(
            self as *mut crate::QVectorOfUint,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_uint>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to prepend(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<unsigned int>::push_front(const unsigned int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to prepend(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_uint>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_push_front(
            self as *mut crate::QVectorOfUint,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_uint>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<unsigned int>::remove(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#remove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes the element at index position <i>i</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn remove_1a(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_remove(
            self as *mut crate::QVectorOfUint,
            i,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<unsigned int>::remove(int i, int n)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#remove-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes <i>count</i> elements from the middle of the vector, starting at index position <i>i</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn remove_2a(&mut self, i: ::std::os::raw::c_int, n: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_remove1(
            self as *mut crate::QVectorOfUint,
            i,
            n,
        )
    }

    /// <p>Removes all elements that compare equal to <i>t</i> from the vector. Returns the number of elements removed, if any.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<unsigned int>::removeAll(const unsigned int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all elements that compare equal to <i>t</i> from the vector. Returns the number of elements removed, if any.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#removeOne">removeOne</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">QList::removeAll</a>().</p></div>
    pub unsafe fn remove_all(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_uint>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_removeAll(
            self as *mut crate::QVectorOfUint,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_uint>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes the element at index position <i>i</i>. Equivalent to</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<unsigned int>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the element at index position <i>i</i>. Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   remove(i);
    ///
    /// </pre>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">QList::removeAt</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_removeAt(
            self as *mut crate::QVectorOfUint,
            i,
        )
    }

    /// <p>Removes the first item in the vector. Calling this function is equivalent to calling remove(0). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<unsigned int>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the vector. Calling this function is equivalent to calling remove(0). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_removeFirst(
            self as *mut crate::QVectorOfUint,
        )
    }

    /// <p>Removes the last item in the vector. Calling this function is equivalent to calling remove(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() - 1). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<unsigned int>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the vector. Calling this function is equivalent to calling remove(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() - 1). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_removeLast(
            self as *mut crate::QVectorOfUint,
        )
    }

    /// <p>Removes the first element that compares equal to <i>t</i> from the vector. Returns whether an element was, in fact, removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<unsigned int>::removeOne(const unsigned int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeOne">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first element that compares equal to <i>t</i> from the vector. Returns whether an element was, in fact, removed.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#removeAll">removeAll</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">QList::removeOne</a>().</p></div>
    pub unsafe fn remove_one(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_uint>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_removeOne(
            self as *mut crate::QVectorOfUint,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_uint>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<unsigned int>::replace(int i, const unsigned int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_uint>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_replace(
            self as *mut crate::QVectorOfUint,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_uint>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Attempts to allocate memory for at least <i>size</i> elements. If you know in advance how large the vector will be, you should call this function to prevent reallocations and memory fragmentation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<unsigned int>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to allocate memory for at least <i>size</i> elements. If you know in advance how large the vector will be, you should call this function to prevent reallocations and memory fragmentation.</p>
    /// <p>If <i>size</i> is an underestimate, the worst that will happen is that the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> will be a bit slower. If <i>size</i> is an overestimate, you may have used more memory than the normal <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> growth strategy would have allocated—or you may have used less.</p>
    /// <p>An alternative to reserve() is calling <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>(). Whether or not that is faster than reserve() depends on the element type, because <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() default-constructs all elements, and requires assignment to existing entries rather than calling <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(), which copy- or move-constructs. For simple types, like <code>int</code> or <code>double</code>, <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() is typically faster, but for anything more complex, you should prefer reserve().</p>
    /// <p><b>Warning:</b> If the size passed to <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() was underestimated, you run out of allocated space and into undefined behavior. This problem does not exist with reserve(), because it treats the size as just a hint.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>().</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_reserve(
            self as *mut crate::QVectorOfUint,
            size,
        )
    }

    /// <p>Sets the size of the vector to <i>size</i>. If <i>size</i> is greater than the current size, elements are added to the end; the new elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If <i>size</i> is less than the current size, elements are removed from the end.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<unsigned int>::resize(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#resize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the size of the vector to <i>size</i>. If <i>size</i> is greater than the current size, elements are added to the end; the new elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If <i>size</i> is less than the current size, elements are removed from the end.</p>
    /// <p>Since Qt 5.6, resize() doesn't shrink the capacity anymore. To shed excess capacity, use <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p></div>
    pub unsafe fn resize(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_resize(
            self as *mut crate::QVectorOfUint,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QVector<unsigned int>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_setSharable(
            self as *mut crate::QVectorOfUint,
            sharable,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<unsigned int>::shrink_to_fit()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#shrink_to_fit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn shrink_to_fit(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_shrink_to_fit(
            self as *mut crate::QVectorOfUint,
        )
    }

    /// <p>Returns the number of items in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<unsigned int>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_size(self as *const crate::QVectorOfUint)
    }

    /// <p>Releases any memory not required to store the items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<unsigned int>::squeeze()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Releases any memory not required to store the items.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s memory usage. In general, you will rarely ever need to call this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>().</p></div>
    pub unsafe fn squeeze(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_squeeze(
            self as *mut crate::QVectorOfUint,
        )
    }

    /// <p>Returns <code>true</code> if this vector is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<unsigned int>::startsWith(const unsigned int& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this vector is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn starts_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_uint>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_startsWith(
            self as *const crate::QVectorOfUint,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_uint>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Swaps vector <i>other</i> with this vector. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<unsigned int>::swap(QVector<unsigned int>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps vector <i>other</i> with this vector. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QVectorOfUint>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_swap(
            self as *mut crate::QVectorOfUint,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QVectorOfUint>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Removes the element at index position <i>i</i> and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QVector<unsigned int>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the element at index position <i>i</i> and returns it.</p>
    /// <p>Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   T t <span class="operator">=</span> at(i);
    ///   remove(i);
    ///   <span class="keyword">return</span> t;
    ///
    /// </pre>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">QList::takeAt</a>().</p></div>
    pub unsafe fn take_at(&mut self, i: ::std::os::raw::c_int) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_takeAt(
            self as *mut crate::QVectorOfUint,
            i,
        )
    }

    /// <p>Removes the first item in the vector and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QVector<unsigned int>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the vector and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_takeFirst(
            self as *mut crate::QVectorOfUint,
        )
    }

    /// <p>Removes the last item in the list and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QVector<unsigned int>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_takeLast(
            self as *mut crate::QVectorOfUint,
        )
    }

    /// <p>Returns the value at index position <i>i</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QVector<unsigned int>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the vector.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that <i>i</i> is within bounds, you can use <a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(&self, i: ::std::os::raw::c_int) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_value(
            self as *const crate::QVectorOfUint,
            i,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QVector<unsigned int>::value(int i, const unsigned int& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_uint>>,
    ) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_value1(
            self as *const crate::QVectorOfUint,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_uint>>::cast_into(
                default_value,
            )
            .as_raw_ptr(),
        )
    }
}

impl From<crate::FindChildOption> for crate::QFlags<crate::FindChildOption> {
    fn from(value: crate::FindChildOption) -> Self {
        Self::from(value.to_int())
    }
}

impl<T: Into<crate::QFlags<crate::FindChildOption>>> std::ops::BitOr<T> for crate::FindChildOption {
    type Output = crate::QFlags<crate::FindChildOption>;
    fn bitor(self, rhs: T) -> crate::QFlags<crate::FindChildOption> {
        Into::<crate::QFlags<crate::FindChildOption>>::into(self) | rhs
    }
}

pub mod q_list_of_timer_info {
    //! C++ type: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::iterator& QList<QAbstractEventDispatcher::TimerInfo>::iterator::operator=(const QList<QAbstractEventDispatcher::TimerInfo>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_timer_info::Iterator>>,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_timer_info::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_iterator_operator_8(self as *mut crate::q_list_of_timer_info::Iterator, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_timer_info::Iterator>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::iterator QList<QAbstractEventDispatcher::TimerInfo>::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_timer_info::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_iterator_operator__13(self as *mut crate::q_list_of_timer_info::Iterator, arg1);
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::iterator QList<QAbstractEventDispatcher::TimerInfo>::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_timer_info::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_iterator_operator__11(self as *mut crate::q_list_of_timer_info::Iterator, arg1);
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QAbstractEventDispatcher::TimerInfo& QList<QAbstractEventDispatcher::TimerInfo>::iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValueRef iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>. If you assign to it, the assignment will apply to the character in the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> of <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> from which you got the reference.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::MutRef<crate::q_abstract_event_dispatcher::TimerInfo> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_iterator_operator__1(self as *const crate::q_list_of_timer_info::Iterator, j);
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractEventDispatcher::TimerInfo>::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>().</p></div>
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_list_of_timer_info::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractEventDispatcher::TimerInfo>::iterator::iterator(const QList<QAbstractEventDispatcher::TimerInfo>::iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_timer_info::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_timer_info::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_iterator_iterator1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_timer_info::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QAbstractEventDispatcher::TimerInfo* QList<QAbstractEventDispatcher::TimerInfo>::iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's value.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn struct_deref(
            &self,
        ) -> ::cpp_utils::MutPtr<crate::q_abstract_event_dispatcher::TimerInfo> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_iterator_operator__(self as *const crate::q_list_of_timer_info::Iterator);
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::const_iterator& QList<QAbstractEventDispatcher::TimerInfo>::const_iterator::operator=(const QList<QAbstractEventDispatcher::TimerInfo>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_timer_info::ConstIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_timer_info::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_const_iterator_operator_6(self as *mut crate::q_list_of_timer_info::ConstIterator, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_timer_info::ConstIterator>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::const_iterator QList<QAbstractEventDispatcher::TimerInfo>::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_timer_info::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_const_iterator_operator__9(self as *mut crate::q_list_of_timer_info::ConstIterator, arg1);
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::const_iterator QList<QAbstractEventDispatcher::TimerInfo>::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_timer_info::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_const_iterator_operator__7(self as *mut crate::q_list_of_timer_info::ConstIterator, arg1);
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QAbstractEventDispatcher::TimerInfo& QList<QAbstractEventDispatcher::TimerInfo>::const_iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValue const_iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_const_iterator_operator__1(self as *const crate::q_list_of_timer_info::ConstIterator, j);
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractEventDispatcher::TimerInfo>::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>().</p></div>
        pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::q_list_of_timer_info::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_const_iterator_const_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractEventDispatcher::TimerInfo>::const_iterator::const_iterator(const QList<QAbstractEventDispatcher::TimerInfo>::iterator& o)```</span>.
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_timer_info::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_timer_info::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_const_iterator_const_iterator2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_timer_info::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractEventDispatcher::TimerInfo>::const_iterator::const_iterator(const QList<QAbstractEventDispatcher::TimerInfo>::const_iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_timer_info::ConstIterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_timer_info::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_const_iterator_const_iterator1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_timer_info::ConstIterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current result.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QAbstractEventDispatcher::TimerInfo* QList<QAbstractEventDispatcher::TimerInfo>::const_iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current result.</p></div>
        pub unsafe fn struct_deref(
            &self,
        ) -> ::cpp_utils::Ptr<crate::q_abstract_event_dispatcher::TimerInfo> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_const_iterator_operator__(self as *const crate::q_list_of_timer_info::ConstIterator);
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_timer_info::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_abstract_event_dispatcher::TimerInfo>;
        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QAbstractEventDispatcher::TimerInfo& QList<QAbstractEventDispatcher::TimerInfo>::iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>().</p></div>
        fn indirection(self) -> ::cpp_utils::MutRef<crate::q_abstract_event_dispatcher::TimerInfo> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_iterator_operator_(self as *const crate::q_list_of_timer_info::Iterator)
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_timer_info::Iterator>>
        for crate::q_list_of_timer_info::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractEventDispatcher::TimerInfo>::iterator::operator==(const QList<QAbstractEventDispatcher::TimerInfo>::iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_timer_info::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_iterator_operator__2(self as *const crate::q_list_of_timer_info::Iterator, o.as_raw_ptr())
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_timer_info::Iterator>>
        for crate::q_list_of_timer_info::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractEventDispatcher::TimerInfo>::iterator::operator<(const QList<QAbstractEventDispatcher::TimerInfo>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_timer_info::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_iterator_operator_1(self as *const crate::q_list_of_timer_info::Iterator, other.as_raw_ptr())
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_timer_info::Iterator>>
        for crate::q_list_of_timer_info::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractEventDispatcher::TimerInfo>::iterator::operator<=(const QList<QAbstractEventDispatcher::TimerInfo>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_timer_info::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_iterator_operator__4(self as *const crate::q_list_of_timer_info::Iterator, other.as_raw_ptr())
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_timer_info::Iterator>>
        for crate::q_list_of_timer_info::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractEventDispatcher::TimerInfo>::iterator::operator>(const QList<QAbstractEventDispatcher::TimerInfo>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_timer_info::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_iterator_operator_2(self as *const crate::q_list_of_timer_info::Iterator, other.as_raw_ptr())
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_timer_info::Iterator>>
        for crate::q_list_of_timer_info::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractEventDispatcher::TimerInfo>::iterator::operator>=(const QList<QAbstractEventDispatcher::TimerInfo>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_timer_info::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_iterator_operator__5(self as *const crate::q_list_of_timer_info::Iterator, other.as_raw_ptr())
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_timer_info::ConstIterator>>
        for crate::q_list_of_timer_info::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractEventDispatcher::TimerInfo>::iterator::operator==(const QList<QAbstractEventDispatcher::TimerInfo>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_timer_info::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_iterator_operator__6(self as *const crate::q_list_of_timer_info::Iterator, o.as_raw_ptr())
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_timer_info::ConstIterator>>
        for crate::q_list_of_timer_info::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractEventDispatcher::TimerInfo>::iterator::operator<(const QList<QAbstractEventDispatcher::TimerInfo>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_timer_info::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_iterator_operator_3(self as *const crate::q_list_of_timer_info::Iterator, other.as_raw_ptr())
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_timer_info::ConstIterator>>
        for crate::q_list_of_timer_info::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractEventDispatcher::TimerInfo>::iterator::operator<=(const QList<QAbstractEventDispatcher::TimerInfo>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_timer_info::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_iterator_operator__8(self as *const crate::q_list_of_timer_info::Iterator, other.as_raw_ptr())
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_timer_info::ConstIterator>>
        for crate::q_list_of_timer_info::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractEventDispatcher::TimerInfo>::iterator::operator>(const QList<QAbstractEventDispatcher::TimerInfo>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_timer_info::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_iterator_operator_4(self as *const crate::q_list_of_timer_info::Iterator, other.as_raw_ptr())
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_timer_info::ConstIterator>>
        for crate::q_list_of_timer_info::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractEventDispatcher::TimerInfo>::iterator::operator>=(const QList<QAbstractEventDispatcher::TimerInfo>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_timer_info::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_iterator_operator__9(self as *const crate::q_list_of_timer_info::Iterator, other.as_raw_ptr())
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_timer_info::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_timer_info::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::iterator& QList<QAbstractEventDispatcher::TimerInfo>::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_timer_info::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_iterator_operator__10(self as *mut crate::q_list_of_timer_info::Iterator)
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_timer_info::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_timer_info::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::iterator& QList<QAbstractEventDispatcher::TimerInfo>::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_timer_info::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_iterator_operator__12(self as *mut crate::q_list_of_timer_info::Iterator)
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong> for crate::q_list_of_timer_info::Iterator {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::iterator& QList<QAbstractEventDispatcher::TimerInfo>::iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_iterator_operator__14(self as *mut crate::q_list_of_timer_info::Iterator, j)
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong> for crate::q_list_of_timer_info::Iterator {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::iterator& QList<QAbstractEventDispatcher::TimerInfo>::iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_iterator_operator__15(self as *mut crate::q_list_of_timer_info::Iterator, j)
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_list_of_timer_info::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_timer_info::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::iterator QList<QAbstractEventDispatcher::TimerInfo>::iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_timer_info::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_iterator_operator_5(self as *const crate::q_list_of_timer_info::Iterator, j)
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_list_of_timer_info::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_timer_info::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::iterator QList<QAbstractEventDispatcher::TimerInfo>::iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_timer_info::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_iterator_operator_6(self as *const crate::q_list_of_timer_info::Iterator, j)
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_timer_info::Iterator>>
        for &crate::q_list_of_timer_info::Iterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QAbstractEventDispatcher::TimerInfo>::iterator::operator-(QList<QAbstractEventDispatcher::TimerInfo>::iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_timer_info::Iterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_iterator_operator_7(self as *const crate::q_list_of_timer_info::Iterator, j.as_raw_ptr())
            }
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_timer_info::ConstIterator {
        type Output = ::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo>;
        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QAbstractEventDispatcher::TimerInfo& QList<QAbstractEventDispatcher::TimerInfo>::const_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        fn indirection(self) -> ::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_const_iterator_operator_(self as *const crate::q_list_of_timer_info::ConstIterator)
            };
            unsafe {
                ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_timer_info::ConstIterator>>
        for crate::q_list_of_timer_info::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractEventDispatcher::TimerInfo>::const_iterator::operator==(const QList<QAbstractEventDispatcher::TimerInfo>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_timer_info::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_const_iterator_operator__2(self as *const crate::q_list_of_timer_info::ConstIterator, o.as_raw_ptr())
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_timer_info::ConstIterator>>
        for crate::q_list_of_timer_info::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractEventDispatcher::TimerInfo>::const_iterator::operator<(const QList<QAbstractEventDispatcher::TimerInfo>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_timer_info::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_const_iterator_operator_1(self as *const crate::q_list_of_timer_info::ConstIterator, other.as_raw_ptr())
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_timer_info::ConstIterator>>
        for crate::q_list_of_timer_info::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractEventDispatcher::TimerInfo>::const_iterator::operator<=(const QList<QAbstractEventDispatcher::TimerInfo>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_timer_info::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_const_iterator_operator__4(self as *const crate::q_list_of_timer_info::ConstIterator, other.as_raw_ptr())
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_timer_info::ConstIterator>>
        for crate::q_list_of_timer_info::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractEventDispatcher::TimerInfo>::const_iterator::operator>(const QList<QAbstractEventDispatcher::TimerInfo>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_timer_info::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_const_iterator_operator_2(self as *const crate::q_list_of_timer_info::ConstIterator, other.as_raw_ptr())
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_timer_info::ConstIterator>>
        for crate::q_list_of_timer_info::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractEventDispatcher::TimerInfo>::const_iterator::operator>=(const QList<QAbstractEventDispatcher::TimerInfo>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_timer_info::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_const_iterator_operator__5(self as *const crate::q_list_of_timer_info::ConstIterator, other.as_raw_ptr())
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_timer_info::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_timer_info::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::const_iterator& QList<QAbstractEventDispatcher::TimerInfo>::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_timer_info::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_const_iterator_operator__6(self as *mut crate::q_list_of_timer_info::ConstIterator)
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_timer_info::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_timer_info::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::const_iterator& QList<QAbstractEventDispatcher::TimerInfo>::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_timer_info::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_const_iterator_operator__8(self as *mut crate::q_list_of_timer_info::ConstIterator)
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_timer_info::ConstIterator
    {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::const_iterator& QList<QAbstractEventDispatcher::TimerInfo>::const_iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_const_iterator_operator__10(self as *mut crate::q_list_of_timer_info::ConstIterator, j)
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_timer_info::ConstIterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::const_iterator& QList<QAbstractEventDispatcher::TimerInfo>::const_iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_const_iterator_operator__11(self as *mut crate::q_list_of_timer_info::ConstIterator, j)
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_list_of_timer_info::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_timer_info::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::const_iterator QList<QAbstractEventDispatcher::TimerInfo>::const_iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_timer_info::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_const_iterator_operator_3(self as *const crate::q_list_of_timer_info::ConstIterator, j)
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_list_of_timer_info::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_timer_info::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::const_iterator QList<QAbstractEventDispatcher::TimerInfo>::const_iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_timer_info::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_const_iterator_operator_4(self as *const crate::q_list_of_timer_info::ConstIterator, j)
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_timer_info::ConstIterator>>
        for &crate::q_list_of_timer_info::ConstIterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QAbstractEventDispatcher::TimerInfo>::const_iterator::operator-(QList<QAbstractEventDispatcher::TimerInfo>::const_iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_timer_info::ConstIterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_const_iterator_operator_5(self as *const crate::q_list_of_timer_info::ConstIterator, j.as_raw_ptr())
            }
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_timer_info::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QAbstractEventDispatcher::TimerInfo>::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_iterator_diterator(self as *mut crate::q_list_of_timer_info::Iterator)
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_timer_info::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QAbstractEventDispatcher::TimerInfo>::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_const_iterator_dconst_iterator(self as *mut crate::q_list_of_timer_info::ConstIterator)
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
///
/// C++ class: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlist.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores items in a list that provides fast index-based access and index-based insertions and removals.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Internally, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T if <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. Otherwise, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T* and the items are allocated on the heap.</p>
/// <p>The array representation allows very fast insertions and index-based access. The <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() operations are also very fast because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates memory at both ends of its internal array. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.</p>
/// <p>Note, however, that when the conditions specified above are not met, each append or insert of a new item requires allocating the new item on the heap, and this per item allocation will make <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> a better choice for use cases that do a lot of appending or inserting, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> can allocate memory for many items in a single heap allocation.</p>
/// <p>Note that the internal array only ever gets bigger over the life of the list. It never shrinks. The internal array is deallocated by the destructor and by the assignment operator, when one list is assigned to another.</p>
/// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerList;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span><span class="operator">&gt;</span> dateList;
///
/// </pre>
/// <p>Qt includes a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class that inherits <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>&gt; and adds a few convenience functions, such as <a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#filter">QStringList::filter</a>(). <a href="http://doc.qt.io/qt-5/qstring.html#split">QString::split</a>() creates QStringLists from strings.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> stores a list of items. The default constructor creates an empty list. To insert items into the list, you can use operator&lt;&lt;():</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"one"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"two"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"three"</span>;
///   <span class="comment">// list: ["one", "two", "three"]</span>
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(). In addition, it provides the following convenience functions: <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const lists, operator[]() returns a reference to the item and can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Bob"</span>)
/// &#32;     list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Robert"</span>;
///
/// </pre>
/// <p>Because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented as an array of pointers for types that are larger than a pointer or are not movable, this operation requires (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>). For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> list<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (list<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Jane"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Jane at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>A common requirement is to remove an item from a list and do something with it. For this, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(). Here's a loop that removes the items from a list one at a time and calls <code>delete</code> on them:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   <span class="keyword">while</span> (<span class="operator">!</span>list<span class="operator">.</span>isEmpty())
/// &#32;     <span class="keyword">delete</span> list<span class="operator">.</span>takeFirst();
///
/// </pre>
/// <p>Inserting and removing items at either end of the list is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> in most cases), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
/// <p>If you want to find all occurrences of a particular value in a list, use <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of a matching item if they find it; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> list<span class="operator">.</span>indexOf(<span class="string">"Jane"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Jane is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a list contains a particular value, use <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the list, use <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>(). If you want to replace all occurrences of a particular value with another, use <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlistiterator.html">QListIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablelistiterator.html">QMutableListIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented in such a way that direct index-based access is just as fast as using iterators.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <p>To make <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as efficient as possible, its member functions don't validate their input before using it. Except for <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), member functions always assume the list is <i>not</i> empty. Member functions that take index values as parameters always assume their index value parameters are in the valid range. This means <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> member functions can fail. If you define QT_NO_DEBUG when you compile, failures will not be detected. If you <i>don't</i> define QT_NO_DEBUG, failures will be detected using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT">Q_ASSERT</a>() or <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT_X">Q_ASSERT_X</a>() with an appropriate message.</p>
/// <p>To avoid failures when your list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling other member functions. If you must pass an index value that might not be in the valid range, check that it is less than the value returned by <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() but <i>not</i> less than 0.</p>
/// <a name="more-members"></a></div>
#[repr(C)]
pub struct QListOfTimerInfo {
    _unused: u8,
}
impl QListOfTimerInfo {
    /// <p>Inserts <i>value</i> at the end of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractEventDispatcher::TimerInfo>::append(const QAbstractEventDispatcher::TimerInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["one", "two", "three"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn append_timer_info(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_append(self as *mut crate::QListOfTimerInfo, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo>>::cast_into(t).as_raw_ptr())
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractEventDispatcher::TimerInfo>::append(const QList<QAbstractEventDispatcher::TimerInfo>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> list to this list.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_list_of_timer_info(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfTimerInfo>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_append1(
            self as *mut crate::QListOfTimerInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfTimerInfo>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QAbstractEventDispatcher::TimerInfo& QList<QAbstractEventDispatcher::TimerInfo>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>This function is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_at(
            self as *const crate::QListOfTimerInfo,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractEventDispatcher::TimerInfo& QList<QAbstractEventDispatcher::TimerInfo>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn back_mut(
        &mut self,
    ) -> ::cpp_utils::MutRef<crate::q_abstract_event_dispatcher::TimerInfo> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_back(
                self as *mut crate::QListOfTimerInfo,
            );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QAbstractEventDispatcher::TimerInfo& QList<QAbstractEventDispatcher::TimerInfo>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_back1(
                self as *const crate::QListOfTimerInfo,
            );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::const_iterator QList<QAbstractEventDispatcher::TimerInfo>::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(&self) -> ::cpp_utils::CppBox<crate::q_list_of_timer_info::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_cbegin(
                self as *const crate::QListOfTimerInfo,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::const_iterator QList<QAbstractEventDispatcher::TimerInfo>::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn cend(&self) -> ::cpp_utils::CppBox<crate::q_list_of_timer_info::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_cend(
                self as *const crate::QListOfTimerInfo,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all items from the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractEventDispatcher::TimerInfo>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all items from the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_clear(
            self as *mut crate::QListOfTimerInfo,
        )
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::const_iterator QList<QAbstractEventDispatcher::TimerInfo>::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_timer_info::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_constBegin(
                self as *const crate::QListOfTimerInfo,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::const_iterator QList<QAbstractEventDispatcher::TimerInfo>::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn const_end(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_timer_info::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_constEnd(
                self as *const crate::QListOfTimerInfo,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QAbstractEventDispatcher::TimerInfo& QList<QAbstractEventDispatcher::TimerInfo>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>().</p></div>
    pub unsafe fn const_first(
        &self,
    ) -> ::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_constFirst(
                self as *const crate::QListOfTimerInfo,
            );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QAbstractEventDispatcher::TimerInfo& QList<QAbstractEventDispatcher::TimerInfo>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>().</p></div>
    pub unsafe fn const_last(
        &self,
    ) -> ::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_constLast(
                self as *const crate::QListOfTimerInfo,
            );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Assigns <i>other</i> to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>& QList<QAbstractEventDispatcher::TimerInfo>::operator=(const QList<QAbstractEventDispatcher::TimerInfo>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this list and returns a reference to this list.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfTimerInfo>>,
    ) -> ::cpp_utils::MutRef<crate::QListOfTimerInfo> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_operator_(
                self as *mut crate::QListOfTimerInfo,
                ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfTimerInfo>>::cast_into(l)
                    .as_raw_ptr(),
            );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QAbstractEventDispatcher::TimerInfo>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_count1(
            self as *const crate::QListOfTimerInfo,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractEventDispatcher::TimerInfo>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_detach(
            self as *mut crate::QListOfTimerInfo,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractEventDispatcher::TimerInfo>::detachShared()```</span>.
    pub unsafe fn detach_shared(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_detachShared(
            self as *mut crate::QListOfTimerInfo,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractEventDispatcher::TimerInfo>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_empty(
            self as *const crate::QListOfTimerInfo,
        )
    }

    /// <p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::iterator QList<QAbstractEventDispatcher::TimerInfo>::erase(QList<QAbstractEventDispatcher::TimerInfo>::iterator pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn erase_1a(
        &mut self,
        pos: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_timer_info::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_timer_info::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_erase(self as *mut crate::QListOfTimerInfo, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_timer_info::Iterator>>::cast_into(pos).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::iterator QList<QAbstractEventDispatcher::TimerInfo>::erase(QList<QAbstractEventDispatcher::TimerInfo>::iterator first, QList<QAbstractEventDispatcher::TimerInfo>::iterator last)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes all the items from <i>begin</i> up to (but not including) <i>end</i>. Returns an iterator to the same item that <i>end</i> referred to before the call.</p></div>
    pub unsafe fn erase_2a(
        &mut self,
        first: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_timer_info::Iterator>>,
        last: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_timer_info::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_timer_info::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_erase1(self as *mut crate::QListOfTimerInfo, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_timer_info::Iterator>>::cast_into(first).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_timer_info::Iterator>>::cast_into(last).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractEventDispatcher::TimerInfo& QList<QAbstractEventDispatcher::TimerInfo>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn first_mut(
        &mut self,
    ) -> ::cpp_utils::MutRef<crate::q_abstract_event_dispatcher::TimerInfo> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_first(
                self as *mut crate::QListOfTimerInfo,
            );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QAbstractEventDispatcher::TimerInfo& QList<QAbstractEventDispatcher::TimerInfo>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_first1(
                self as *const crate::QListOfTimerInfo,
            );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractEventDispatcher::TimerInfo& QList<QAbstractEventDispatcher::TimerInfo>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn front_mut(
        &mut self,
    ) -> ::cpp_utils::MutRef<crate::q_abstract_event_dispatcher::TimerInfo> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_front(
                self as *mut crate::QListOfTimerInfo,
            );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QAbstractEventDispatcher::TimerInfo& QList<QAbstractEventDispatcher::TimerInfo>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_front1(
                self as *const crate::QListOfTimerInfo,
            );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QAbstractEventDispatcher::TimerInfo& QList<QAbstractEventDispatcher::TimerInfo>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>(). This function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p></div>
    pub unsafe fn index(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_operator__2(
                self as *const crate::QListOfTimerInfo,
                i,
            );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractEventDispatcher::TimerInfo& QList<QAbstractEventDispatcher::TimerInfo>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If this function is called on a list that is currently being shared, it will trigger a copy of all elements. Otherwise, this function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>. If you do not want to modify the list you should use <a href="http://doc.qt.io/qt-5/qlist.html#at">QList::at</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::q_abstract_event_dispatcher::TimerInfo> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_operator__3(
                self as *mut crate::QListOfTimerInfo,
                i,
            );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractEventDispatcher::TimerInfo>::insert(int i, const QAbstractEventDispatcher::TimerInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   list<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// list: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn insert_int_timer_info(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_insert(self as *mut crate::QListOfTimerInfo, i, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo>>::cast_into(t).as_raw_ptr())
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::iterator QList<QAbstractEventDispatcher::TimerInfo>::insert(QList<QAbstractEventDispatcher::TimerInfo>::iterator before, const QAbstractEventDispatcher::TimerInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>value</i> in front of the item pointed to by the iterator <i>before</i>. Returns an iterator pointing at the inserted item. Note that the iterator passed to the function will be invalid after the call; the returned iterator should be used instead.</p></div>
    pub unsafe fn insert_iterator_timer_info(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_timer_info::Iterator>>,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_timer_info::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_insert1(self as *mut crate::QListOfTimerInfo, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_timer_info::Iterator>>::cast_into(before).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo>>::cast_into(t).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractEventDispatcher::TimerInfo>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_isDetached(
            self as *const crate::QListOfTimerInfo,
        )
    }

    /// <p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractEventDispatcher::TimerInfo>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_isEmpty(
            self as *const crate::QListOfTimerInfo,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractEventDispatcher::TimerInfo>::isSharedWith(const QList<QAbstractEventDispatcher::TimerInfo>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfTimerInfo>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_isSharedWith(
            self as *const crate::QListOfTimerInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfTimerInfo>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractEventDispatcher::TimerInfo& QList<QAbstractEventDispatcher::TimerInfo>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn last_mut(
        &mut self,
    ) -> ::cpp_utils::MutRef<crate::q_abstract_event_dispatcher::TimerInfo> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_last(
                self as *mut crate::QListOfTimerInfo,
            );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QAbstractEventDispatcher::TimerInfo& QList<QAbstractEventDispatcher::TimerInfo>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_last1(
                self as *const crate::QListOfTimerInfo,
            );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QAbstractEventDispatcher::TimerInfo>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_length(
            self as *const crate::QListOfTimerInfo,
        )
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo> QList<QAbstractEventDispatcher::TimerInfo>::mid(int pos, int length = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        length: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfTimerInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_mid(
            self as *const crate::QListOfTimerInfo,
            pos,
            length,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo> QList<QAbstractEventDispatcher::TimerInfo>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfTimerInfo> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_mid1(
                self as *const crate::QListOfTimerInfo,
                pos,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractEventDispatcher::TimerInfo>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>move(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "C", "D", "E", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This is the same as insert(<i>to</i>, <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<i>from</i>)).This function assumes that both <i>from</i> and <i>to</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>from</i> and <i>to</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_move(
            self as *mut crate::QListOfTimerInfo,
            from,
            to,
        )
    }

    /// <p>Constructs an empty list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractEventDispatcher::TimerInfo>::QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty list.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QListOfTimerInfo> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_QList();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractEventDispatcher::TimerInfo>::QList(const QList<QAbstractEventDispatcher::TimerInfo>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfTimerInfo>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfTimerInfo> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_QList1(
                ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfTimerInfo>>::cast_into(l)
                    .as_raw_ptr(),
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractEventDispatcher::TimerInfo>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_pop_back(
            self as *mut crate::QListOfTimerInfo,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractEventDispatcher::TimerInfo>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_pop_front(
            self as *mut crate::QListOfTimerInfo,
        )
    }

    /// <p>Inserts <i>value</i> at the beginning of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractEventDispatcher::TimerInfo>::prepend(const QAbstractEventDispatcher::TimerInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(0, <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_prepend(self as *mut crate::QListOfTimerInfo, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo>>::cast_into(t).as_raw_ptr())
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractEventDispatcher::TimerInfo>::push_back(const QAbstractEventDispatcher::TimerInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_push_back(self as *mut crate::QListOfTimerInfo, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo>>::cast_into(t).as_raw_ptr())
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractEventDispatcher::TimerInfo>::push_front(const QAbstractEventDispatcher::TimerInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_push_front(self as *mut crate::QListOfTimerInfo, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractEventDispatcher::TimerInfo>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_removeAt(
            self as *mut crate::QListOfTimerInfo,
            i,
        )
    }

    /// <p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractEventDispatcher::TimerInfo>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_removeFirst(
            self as *mut crate::QListOfTimerInfo,
        )
    }

    /// <p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractEventDispatcher::TimerInfo>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_removeLast(
            self as *mut crate::QListOfTimerInfo,
        )
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractEventDispatcher::TimerInfo>::replace(int i, const QAbstractEventDispatcher::TimerInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_replace(self as *mut crate::QListOfTimerInfo, i, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Reserve space for <i>alloc</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractEventDispatcher::TimerInfo>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reserve space for <i>alloc</i> elements.</p>
    /// <p>If <i>alloc</i> is smaller than the current size of the list, nothing will happen.</p>
    /// <p>Use this function to avoid repetetive reallocation of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s internal data if you can predict how many elements will be appended. Note that the reservation applies only to the internal pointer array.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_reserve(
            self as *mut crate::QListOfTimerInfo,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractEventDispatcher::TimerInfo>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_setSharable(
            self as *mut crate::QListOfTimerInfo,
            sharable,
        )
    }

    /// <p>Returns the number of items in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QAbstractEventDispatcher::TimerInfo>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_size(
            self as *const crate::QListOfTimerInfo,
        )
    }

    /// <p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractEventDispatcher::TimerInfo>::swap(QList<QAbstractEventDispatcher::TimerInfo>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap_1a(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QListOfTimerInfo>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_swap(
            self as *mut crate::QListOfTimerInfo,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QListOfTimerInfo>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractEventDispatcher::TimerInfo>::swap(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swap(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    pub unsafe fn swap_2a(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_swap1(
            self as *mut crate::QListOfTimerInfo,
            i,
            j,
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractEventDispatcher::TimerInfo>::swapItemsAt(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swapItemsAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swapItemsAt(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn swap_items_at(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_swapItemsAt(
            self as *mut crate::QListOfTimerInfo,
            i,
            j,
        )
    }

    /// <p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractEventDispatcher::TimerInfo QList<QAbstractEventDispatcher::TimerInfo>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::q_abstract_event_dispatcher::TimerInfo> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_takeAt(
                self as *mut crate::QListOfTimerInfo,
                i,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractEventDispatcher::TimerInfo QList<QAbstractEventDispatcher::TimerInfo>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(
        &mut self,
    ) -> ::cpp_utils::CppBox<crate::q_abstract_event_dispatcher::TimerInfo> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_takeFirst(
                self as *mut crate::QListOfTimerInfo,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractEventDispatcher::TimerInfo QList<QAbstractEventDispatcher::TimerInfo>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(
        &mut self,
    ) -> ::cpp_utils::CppBox<crate::q_abstract_event_dispatcher::TimerInfo> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_takeLast(
                self as *mut crate::QListOfTimerInfo,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractEventDispatcher::TimerInfo QList<QAbstractEventDispatcher::TimerInfo>::value(int i, const QAbstractEventDispatcher::TimerInfo& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo>,
        >,
    ) -> ::cpp_utils::CppBox<crate::q_abstract_event_dispatcher::TimerInfo> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_value1(
                self as *const crate::QListOfTimerInfo,
                i,
                ::cpp_utils::CastInto::<
                    ::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo>,
                >::cast_into(default_value)
                .as_raw_ptr(),
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_map_of_q_string_q_variant {
    //! C++ type: <span style='color: green;'>```QMap<QString, QVariant>```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QMap<QString, QVariant>::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>&lt;Key, T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> (or <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>) and to modify the value (but not the key) stored under a particular key. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, you should use <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a>. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> function like <a href="http://doc.qt.io/qt-5/qmap.html#begin">QMap::begin</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#end">QMap::end</a>(), or <a href="http://doc.qt.io/qt-5/qmap.html#find">QMap::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a map:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> map;
    ///   map<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
    ///   map<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   map<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> map<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, which stores its items in an arbitrary order, <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> stores its items ordered by key. Items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">QMap::insertMulti</a>(), or due to a <a href="http://doc.qt.io/qt-5/qmap.html#unite">unite</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
    /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a>. Here's an example that increments every value stored in the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> by 2:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> map<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     i<span class="operator">.</span>value() <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
    ///
    /// </pre>
    /// <p>Here's an example that removes all the items whose key is a string that starts with an underscore character:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> map<span class="operator">.</span>begin();
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end()) {
    /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
    /// &#32;   &#32;     i <span class="operator">=</span> map<span class="operator">.</span>erase(i);
    /// &#32;     <span class="keyword">else</span>
    /// &#32;   &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    ///   }
    ///
    /// </pre>
    /// <p>The call to <a href="http://doc.qt.io/qt-5/qmap.html#erase">QMap::erase</a>() removes the item pointed to by the iterator from the map, and returns an iterator to the next item. Here's another way of removing an item while iterating:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> map<span class="operator">.</span>begin();
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end()) {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator prev <span class="operator">=</span> i;
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    /// &#32;     <span class="keyword">if</span> (prev<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
    /// &#32;   &#32;     map<span class="operator">.</span>erase(prev);
    ///   }
    ///
    /// </pre>
    /// <p>It might be tempting to write code like this:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="comment">// WRONG</span>
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end()) {
    /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
    /// &#32;   &#32;     map<span class="operator">.</span>erase(i);
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    ///   }
    ///
    /// </pre>
    /// <p>However, this will potentially crash in <code>++i</code>, because <code>i</code> is a dangling iterator after the call to <a href="http://doc.qt.io/qt-5/qmap.html#erase">erase</a>().</p>
    /// <p>Multiple iterators can be used on the same map. If you add items to the map, existing iterators will remain valid. If you remove items from the map, iterators that point to the removed items will become dangling iterators.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::iterator& QMap<QString, QVariant>::iterator::operator=(const QMap<QString, QVariant>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmap-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>&lt;Key, T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> (or <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>) and to modify the value (but not the key) stored under a particular key. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, you should use <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a>. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> function like <a href="http://doc.qt.io/qt-5/qmap.html#begin">QMap::begin</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#end">QMap::end</a>(), or <a href="http://doc.qt.io/qt-5/qmap.html#find">QMap::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a map:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> map;
        ///   map<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
        ///   map<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   map<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> map<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Unlike <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, which stores its items in an arbitrary order, <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> stores its items ordered by key. Items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">QMap::insertMulti</a>(), or due to a <a href="http://doc.qt.io/qt-5/qmap.html#unite">unite</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a>. Here's an example that increments every value stored in the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> map<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     i<span class="operator">.</span>value() <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Here's an example that removes all the items whose key is a string that starts with an underscore character:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> map<span class="operator">.</span>begin();
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end()) {
        /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     i <span class="operator">=</span> map<span class="operator">.</span>erase(i);
        /// &#32;     <span class="keyword">else</span>
        /// &#32;   &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        ///   }
        ///
        /// </pre>
        /// <p>The call to <a href="http://doc.qt.io/qt-5/qmap.html#erase">QMap::erase</a>() removes the item pointed to by the iterator from the map, and returns an iterator to the next item. Here's another way of removing an item while iterating:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> map<span class="operator">.</span>begin();
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end()) {
        /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator prev <span class="operator">=</span> i;
        /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        /// &#32;     <span class="keyword">if</span> (prev<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     map<span class="operator">.</span>erase(prev);
        ///   }
        ///
        /// </pre>
        /// <p>It might be tempting to write code like this:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="comment">// WRONG</span>
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end()) {
        /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     map<span class="operator">.</span>erase(i);
        /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        ///   }
        ///
        /// </pre>
        /// <p>However, this will potentially crash in <code>++i</code>, because <code>i</code> is a dangling iterator after the call to <a href="http://doc.qt.io/qt-5/qmap.html#erase">erase</a>().</p>
        /// <p>Multiple iterators can be used on the same map. If you add items to the map, existing iterators will remain valid. If you remove items from the map, iterators that point to the removed items will become dangling iterators.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::Iterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_map_of_q_string_q_variant::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_iterator_operator_3(
                    self as *mut crate::q_map_of_q_string_q_variant::Iterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::Iterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::iterator QMap<QString, QVariant>::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_iterator_operator__6(
                    self as *mut crate::q_map_of_q_string_q_variant::Iterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::iterator QMap<QString, QVariant>::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_iterator_operator__4(
                    self as *mut crate::q_map_of_q_string_q_variant::Iterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the current item's key as a const reference.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QString& QMap<QString, QVariant>::iterator::key() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current item's key as a const reference.</p>
        /// <p>There is no direct way of changing an item's key through an iterator, although it can be done by calling <a href="http://doc.qt.io/qt-5/qhash.html#erase">QHash::erase</a>() followed by <a href="http://doc.qt.io/qt-5/qhash.html#insert">QHash::insert</a>() or <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">QHash::insertMulti</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn key(&self) -> ::cpp_utils::Ref<crate::QString> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_iterator_key(
                self as *const crate::q_map_of_q_string_q_variant::Iterator,
            );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QMap<QString, QVariant>::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmap-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like <a href="http://doc.qt.io/qt-5/qmap-iterator.html#key">key</a>(), <a href="http://doc.qt.io/qt-5/qmap-iterator.html#value">value</a>(), and operator++() must not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#begin">QMap::begin</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#end">QMap::end</a>().</p></div>
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>The <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QMap<QString, QVariant>::iterator::iterator(const QMap<QString, QVariant>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmap-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>&lt;Key, T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> (or <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>) and to modify the value (but not the key) stored under a particular key. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, you should use <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a>. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> function like <a href="http://doc.qt.io/qt-5/qmap.html#begin">QMap::begin</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#end">QMap::end</a>(), or <a href="http://doc.qt.io/qt-5/qmap.html#find">QMap::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a map:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> map;
        ///   map<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
        ///   map<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   map<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> map<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Unlike <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, which stores its items in an arbitrary order, <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> stores its items ordered by key. Items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">QMap::insertMulti</a>(), or due to a <a href="http://doc.qt.io/qt-5/qmap.html#unite">unite</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a>. Here's an example that increments every value stored in the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> map<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     i<span class="operator">.</span>value() <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Here's an example that removes all the items whose key is a string that starts with an underscore character:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> map<span class="operator">.</span>begin();
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end()) {
        /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     i <span class="operator">=</span> map<span class="operator">.</span>erase(i);
        /// &#32;     <span class="keyword">else</span>
        /// &#32;   &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        ///   }
        ///
        /// </pre>
        /// <p>The call to <a href="http://doc.qt.io/qt-5/qmap.html#erase">QMap::erase</a>() removes the item pointed to by the iterator from the map, and returns an iterator to the next item. Here's another way of removing an item while iterating:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> map<span class="operator">.</span>begin();
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end()) {
        /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator prev <span class="operator">=</span> i;
        /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        /// &#32;     <span class="keyword">if</span> (prev<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     map<span class="operator">.</span>erase(prev);
        ///   }
        ///
        /// </pre>
        /// <p>It might be tempting to write code like this:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="comment">// WRONG</span>
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end()) {
        /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     map<span class="operator">.</span>erase(i);
        /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        ///   }
        ///
        /// </pre>
        /// <p>However, this will potentially crash in <code>++i</code>, because <code>i</code> is a dangling iterator after the call to <a href="http://doc.qt.io/qt-5/qmap.html#erase">erase</a>().</p>
        /// <p>Multiple iterators can be used on the same map. If you add items to the map, existing iterators will remain valid. If you remove items from the map, iterators that point to the removed items will become dangling iterators.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn new_copy(
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::Iterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_iterator_iterator1(
                ::cpp_utils::CastInto::<
                    ::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::Iterator>,
                >::cast_into(other)
                .as_raw_ptr(),
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QVariant* QMap<QString, QVariant>::iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's value.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::MutPtr<crate::QVariant> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_iterator_operator__(
                    self as *const crate::q_map_of_q_string_q_variant::Iterator,
                );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }

        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QVariant& QMap<QString, QVariant>::iterator::value() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>You can change the value of an item by using value() on the left side of an assignment, for example:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="keyword">if</span> (i<span class="operator">.</span>key() <span class="operator">=</span><span class="operator">=</span> <span class="string">"Hello"</span>)
        /// &#32;     i<span class="operator">.</span>value() <span class="operator">=</span> <span class="string">"Bonjour"</span>;
        ///
        /// </pre>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">operator*</a>().</p></div>
        pub unsafe fn value(&self) -> ::cpp_utils::MutRef<crate::QVariant> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_iterator_value(
                self as *const crate::q_map_of_q_string_q_variant::Iterator,
            );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QMap<QString, QVariant>::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>&lt;Key, T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> (or a <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>). If you want to modify the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> as you iterate over it, you must use <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> function like <a href="http://doc.qt.io/qt-5/qmap.html#constBegin">QMap::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#constEnd">QMap::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qmap.html#find">QMap::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a map:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> map;
    ///   map<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
    ///   map<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   map<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> map<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, which stores its items in an arbitrary order, <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> stores its items ordered by key. Items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">QMap::insertMulti</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
    /// <p>Multiple iterators can be used on the same map. If you add items to the map, existing iterators will remain valid. If you remove items from the map, iterators that point to the removed items will become dangling iterators.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::const_iterator& QMap<QString, QVariant>::const_iterator::operator=(const QMap<QString, QVariant>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>&lt;Key, T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> (or a <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>). If you want to modify the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> as you iterate over it, you must use <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> function like <a href="http://doc.qt.io/qt-5/qmap.html#constBegin">QMap::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#constEnd">QMap::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qmap.html#find">QMap::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a map:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> map;
        ///   map<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
        ///   map<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   map<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> map<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Unlike <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, which stores its items in an arbitrary order, <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> stores its items ordered by key. Items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">QMap::insertMulti</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
        /// <p>Multiple iterators can be used on the same map. If you add items to the map, existing iterators will remain valid. If you remove items from the map, iterators that point to the removed items will become dangling iterators.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::ConstIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_map_of_q_string_q_variant::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_const_iterator_operator_3(
                    self as *mut crate::q_map_of_q_string_q_variant::ConstIterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::ConstIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::const_iterator QMap<QString, QVariant>::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_const_iterator_operator__6(
                    self as *mut crate::q_map_of_q_string_q_variant::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::const_iterator QMap<QString, QVariant>::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_const_iterator_operator__4(
                    self as *mut crate::q_map_of_q_string_q_variant::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the current key, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QString& QMap<QString, QVariant>::const_iterator::key() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#key">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current key, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        pub unsafe fn key(&self) -> ::cpp_utils::Ref<crate::QString> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_const_iterator_key(
                self as *const crate::q_map_of_q_string_q_variant::ConstIterator,
            );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QMap<QString, QVariant>::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html#key">key</a>(), <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html#value">value</a>(), and operator++() must not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#constBegin">QMap::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#constEnd">QMap::constEnd</a>().</p></div>
        pub unsafe fn new_0a(
        ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_const_iterator_const_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs a copy of <i>other</i>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QMap<QString, QVariant>::const_iterator::const_iterator(const QMap<QString, QVariant>::iterator& o)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html#const_iterator-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p></div>
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::Iterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_const_iterator_const_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::Iterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>The <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QMap<QString, QVariant>::const_iterator::const_iterator(const QMap<QString, QVariant>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>&lt;Key, T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> (or a <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>). If you want to modify the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> as you iterate over it, you must use <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> function like <a href="http://doc.qt.io/qt-5/qmap.html#constBegin">QMap::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#constEnd">QMap::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qmap.html#find">QMap::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a map:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> map;
        ///   map<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
        ///   map<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   map<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> map<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Unlike <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, which stores its items in an arbitrary order, <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> stores its items ordered by key. Items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">QMap::insertMulti</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
        /// <p>Multiple iterators can be used on the same map. If you add items to the map, existing iterators will remain valid. If you remove items from the map, iterators that point to the removed items will become dangling iterators.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn new_copy(
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::ConstIterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_const_iterator_const_iterator2(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::ConstIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current result.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QVariant* QMap<QString, QVariant>::const_iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current result.</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::Ptr<crate::QVariant> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_const_iterator_operator__(
                    self as *const crate::q_map_of_q_string_q_variant::ConstIterator,
                );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }

        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QVariant& QMap<QString, QVariant>::const_iterator::value() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        pub unsafe fn value(&self) -> ::cpp_utils::Ref<crate::QVariant> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_const_iterator_value(
                    self as *const crate::q_map_of_q_string_q_variant::ConstIterator,
                );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a> keys.</p>
    ///
    /// C++ class: <span style='color: green;'>```QMap<QString, QVariant>::key_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a> keys.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> is essentially the same as <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> with the difference that operator*() and operator-&gt;() return a key instead of a value.</p>
    /// <p>For most uses <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> and <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> should be used, you can easily access the key by calling <a href="http://doc.qt.io/qt-5/qmap-iterator.html#key">QMap::iterator::key</a>():</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">for</span> (<span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator it <span class="operator">=</span> map<span class="operator">.</span>cbegin()<span class="operator">,</span> end <span class="operator">=</span> map<span class="operator">.</span>cend(); it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The key: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Also the value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> (<span class="operator">*</span>it) <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///   }
    ///
    /// </pre>
    /// <p>However, to have interoperability between <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>'s keys and STL-style algorithms we need an iterator that dereferences to a key instead of a value. With <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> we can apply an algorithm to a range of keys without having to call <a href="http://doc.qt.io/qt-5/qmap.html#keys">QMap::keys</a>(), which is inefficient as it costs one <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> iteration and memory allocation to create a temporary <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="comment">// Inefficient, keys() is expensive</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> keys <span class="operator">=</span> map<span class="operator">.</span>keys();
    ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(map<span class="operator">.</span>cbegin()<span class="operator">,</span> map<span class="operator">.</span>cend()<span class="operator">,</span> isPrimeNumber);
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(map2<span class="operator">.</span>keys());
    ///
    ///   <span class="comment">// Efficient, no memory allocation needed</span>
    ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(map<span class="operator">.</span>keyBegin()<span class="operator">,</span> map<span class="operator">.</span>keyEnd()<span class="operator">,</span> isPrimeNumber);
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(map2<span class="operator">.</span>keyBegin()<span class="operator">,</span> map2<span class="operator">.</span>keyEnd());
    ///
    /// </pre>
    /// <p><a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> is const, it's not possible to modify the key.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> function like <a href="http://doc.qt.io/qt-5/qmap.html#keyBegin">QMap::keyBegin</a>() or <a href="http://doc.qt.io/qt-5/qmap.html#keyEnd">QMap::keyEnd</a>().</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct KeyIterator {
        _unused: u8,
    }
    impl KeyIterator {
        /// <p>Returns the underlying <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">const_iterator</a> this <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">key_iterator</a> is based on.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::const_iterator QMap<QString, QVariant>::key_iterator::base() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#base">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the underlying <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">const_iterator</a> this <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">key_iterator</a> is based on.</p></div>
        pub unsafe fn base(
            &self,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_key_iterator_base(
                self as *const crate::q_map_of_q_string_q_variant::KeyIterator,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>The <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a> keys.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::key_iterator& QMap<QString, QVariant>::key_iterator::operator=(const QMap<QString, QVariant>::key_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a> keys.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> is essentially the same as <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> with the difference that operator*() and operator-&gt;() return a key instead of a value.</p>
        /// <p>For most uses <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> and <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> should be used, you can easily access the key by calling <a href="http://doc.qt.io/qt-5/qmap-iterator.html#key">QMap::iterator::key</a>():</p>
        /// <pre class="cpp">
        ///
        ///   <span class="keyword">for</span> (<span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator it <span class="operator">=</span> map<span class="operator">.</span>cbegin()<span class="operator">,</span> end <span class="operator">=</span> map<span class="operator">.</span>cend(); it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The key: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Also the value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> (<span class="operator">*</span>it) <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///   }
        ///
        /// </pre>
        /// <p>However, to have interoperability between <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>'s keys and STL-style algorithms we need an iterator that dereferences to a key instead of a value. With <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> we can apply an algorithm to a range of keys without having to call <a href="http://doc.qt.io/qt-5/qmap.html#keys">QMap::keys</a>(), which is inefficient as it costs one <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> iteration and memory allocation to create a temporary <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <pre class="cpp">
        ///
        ///   <span class="comment">// Inefficient, keys() is expensive</span>
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> keys <span class="operator">=</span> map<span class="operator">.</span>keys();
        ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(map<span class="operator">.</span>cbegin()<span class="operator">,</span> map<span class="operator">.</span>cend()<span class="operator">,</span> isPrimeNumber);
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(map2<span class="operator">.</span>keys());
        ///
        ///   <span class="comment">// Efficient, no memory allocation needed</span>
        ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(map<span class="operator">.</span>keyBegin()<span class="operator">,</span> map<span class="operator">.</span>keyEnd()<span class="operator">,</span> isPrimeNumber);
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(map2<span class="operator">.</span>keyBegin()<span class="operator">,</span> map2<span class="operator">.</span>keyEnd());
        ///
        /// </pre>
        /// <p><a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> is const, it's not possible to modify the key.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> function like <a href="http://doc.qt.io/qt-5/qmap.html#keyBegin">QMap::keyBegin</a>() or <a href="http://doc.qt.io/qt-5/qmap.html#keyEnd">QMap::keyEnd</a>().</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::KeyIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_map_of_q_string_q_variant::KeyIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_key_iterator_operator_1(
                    self as *mut crate::q_map_of_q_string_q_variant::KeyIterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::KeyIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::key_iterator QMap<QString, QVariant>::key_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previous item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::KeyIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_key_iterator_operator__6(
                    self as *mut crate::q_map_of_q_string_q_variant::KeyIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::key_iterator QMap<QString, QVariant>::key_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previous item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::KeyIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_key_iterator_operator__4(
                    self as *mut crate::q_map_of_q_string_q_variant::KeyIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Default constructs an instance of key_iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QMap<QString, QVariant>::key_iterator::key_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html#key_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Default constructs an instance of key_iterator.</p></div>
        pub unsafe fn new_0a(
        ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::KeyIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_key_iterator_key_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QMap<QString, QVariant>::key_iterator::key_iterator(QMap<QString, QVariant>::const_iterator o)```</span>.
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::ConstIterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::KeyIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_key_iterator_key_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::ConstIterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>The <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a> keys.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QMap<QString, QVariant>::key_iterator::key_iterator(const QMap<QString, QVariant>::key_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a> keys.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> is essentially the same as <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> with the difference that operator*() and operator-&gt;() return a key instead of a value.</p>
        /// <p>For most uses <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> and <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> should be used, you can easily access the key by calling <a href="http://doc.qt.io/qt-5/qmap-iterator.html#key">QMap::iterator::key</a>():</p>
        /// <pre class="cpp">
        ///
        ///   <span class="keyword">for</span> (<span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator it <span class="operator">=</span> map<span class="operator">.</span>cbegin()<span class="operator">,</span> end <span class="operator">=</span> map<span class="operator">.</span>cend(); it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The key: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Also the value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> (<span class="operator">*</span>it) <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///   }
        ///
        /// </pre>
        /// <p>However, to have interoperability between <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>'s keys and STL-style algorithms we need an iterator that dereferences to a key instead of a value. With <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> we can apply an algorithm to a range of keys without having to call <a href="http://doc.qt.io/qt-5/qmap.html#keys">QMap::keys</a>(), which is inefficient as it costs one <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> iteration and memory allocation to create a temporary <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <pre class="cpp">
        ///
        ///   <span class="comment">// Inefficient, keys() is expensive</span>
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> keys <span class="operator">=</span> map<span class="operator">.</span>keys();
        ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(map<span class="operator">.</span>cbegin()<span class="operator">,</span> map<span class="operator">.</span>cend()<span class="operator">,</span> isPrimeNumber);
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(map2<span class="operator">.</span>keys());
        ///
        ///   <span class="comment">// Efficient, no memory allocation needed</span>
        ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(map<span class="operator">.</span>keyBegin()<span class="operator">,</span> map<span class="operator">.</span>keyEnd()<span class="operator">,</span> isPrimeNumber);
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(map2<span class="operator">.</span>keyBegin()<span class="operator">,</span> map2<span class="operator">.</span>keyEnd());
        ///
        /// </pre>
        /// <p><a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> is const, it's not possible to modify the key.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> function like <a href="http://doc.qt.io/qt-5/qmap.html#keyBegin">QMap::keyBegin</a>() or <a href="http://doc.qt.io/qt-5/qmap.html#keyEnd">QMap::keyEnd</a>().</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn new_copy(
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::KeyIterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::KeyIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_key_iterator_key_iterator2(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::KeyIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's key.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QString* QMap<QString, QVariant>::key_iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's key.</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::Ptr<crate::QString> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_key_iterator_operator__(
                    self as *const crate::q_map_of_q_string_q_variant::KeyIterator,
                );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_map_of_q_string_q_variant::Iterator {
        type Output = ::cpp_utils::MutRef<crate::QVariant>;
        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QVariant& QMap<QString, QVariant>::iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>().</p></div>
        fn indirection(self) -> ::cpp_utils::MutRef<crate::QVariant> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_iterator_operator_(
                    self as *const crate::q_map_of_q_string_q_variant::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::Iterator>>
        for crate::q_map_of_q_string_q_variant::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QMap<QString, QVariant>::iterator::operator==(const QMap<QString, QVariant>::iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_iterator_operator__1(
                    self as *const crate::q_map_of_q_string_q_variant::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_map_of_q_string_q_variant::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_map_of_q_string_q_variant::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::iterator& QMap<QString, QVariant>::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_map_of_q_string_q_variant::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_iterator_operator__3(
                    self as *mut crate::q_map_of_q_string_q_variant::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_map_of_q_string_q_variant::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_map_of_q_string_q_variant::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::iterator& QMap<QString, QVariant>::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_map_of_q_string_q_variant::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_iterator_operator__5(
                    self as *mut crate::q_map_of_q_string_q_variant::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_int> for &crate::q_map_of_q_string_q_variant::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::Iterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions forward from this iterator. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::iterator QMap<QString, QVariant>::iterator::operator+(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions forward from this iterator. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p>This operation can be slow for large <i>j</i> values.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn add(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_iterator_operator_1(
                    self as *const crate::q_map_of_q_string_q_variant::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_int> for &crate::q_map_of_q_string_q_variant::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::Iterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions backward from this iterator. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::iterator QMap<QString, QVariant>::iterator::operator-(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions backward from this iterator. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p>This operation can be slow for large <i>j</i> values.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn sub(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_iterator_operator_2(
                    self as *const crate::q_map_of_q_string_q_variant::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_int> for crate::q_map_of_q_string_q_variant::Iterator {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::iterator& QMap<QString, QVariant>::iterator::operator+=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_iterator_operator__7(
                    self as *mut crate::q_map_of_q_string_q_variant::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_int> for crate::q_map_of_q_string_q_variant::Iterator {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::iterator& QMap<QString, QVariant>::iterator::operator-=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_iterator_operator__8(
                    self as *mut crate::q_map_of_q_string_q_variant::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::ConstIterator>>
        for crate::q_map_of_q_string_q_variant::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QMap<QString, QVariant>::iterator::operator==(const QMap<QString, QVariant>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_iterator_operator__9(
                    self as *const crate::q_map_of_q_string_q_variant::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_map_of_q_string_q_variant::ConstIterator {
        type Output = ::cpp_utils::Ref<crate::QVariant>;
        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QVariant& QMap<QString, QVariant>::const_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        fn indirection(self) -> ::cpp_utils::Ref<crate::QVariant> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_const_iterator_operator_(
                    self as *const crate::q_map_of_q_string_q_variant::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::ConstIterator>>
        for crate::q_map_of_q_string_q_variant::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QMap<QString, QVariant>::const_iterator::operator==(const QMap<QString, QVariant>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_const_iterator_operator__1(
                    self as *const crate::q_map_of_q_string_q_variant::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_map_of_q_string_q_variant::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_map_of_q_string_q_variant::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::const_iterator& QMap<QString, QVariant>::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_map_of_q_string_q_variant::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_const_iterator_operator__3(
                    self as *mut crate::q_map_of_q_string_q_variant::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_map_of_q_string_q_variant::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_map_of_q_string_q_variant::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::const_iterator& QMap<QString, QVariant>::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_map_of_q_string_q_variant::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_const_iterator_operator__5(
                    self as *mut crate::q_map_of_q_string_q_variant::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_int> for &crate::q_map_of_q_string_q_variant::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::ConstIterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions forward from this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::const_iterator QMap<QString, QVariant>::const_iterator::operator+(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions forward from this iterator.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>().</p></div>
        fn add(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_const_iterator_operator_1(
                    self as *const crate::q_map_of_q_string_q_variant::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_int> for &crate::q_map_of_q_string_q_variant::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::ConstIterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions backward from this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::const_iterator QMap<QString, QVariant>::const_iterator::operator-(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions backward from this iterator.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>().</p></div>
        fn sub(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_const_iterator_operator_2(
                    self as *const crate::q_map_of_q_string_q_variant::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_int>
        for crate::q_map_of_q_string_q_variant::ConstIterator
    {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::const_iterator& QMap<QString, QVariant>::const_iterator::operator+=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_const_iterator_operator__7(
                    self as *mut crate::q_map_of_q_string_q_variant::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_int>
        for crate::q_map_of_q_string_q_variant::ConstIterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::const_iterator& QMap<QString, QVariant>::const_iterator::operator-=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_const_iterator_operator__8(
                    self as *mut crate::q_map_of_q_string_q_variant::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_map_of_q_string_q_variant::KeyIterator {
        type Output = ::cpp_utils::Ref<crate::QString>;
        /// <p>Returns the current item's key.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QString& QMap<QString, QVariant>::key_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current item's key.</p></div>
        fn indirection(self) -> ::cpp_utils::Ref<crate::QString> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_key_iterator_operator_(
                    self as *const crate::q_map_of_q_string_q_variant::KeyIterator,
                )
            };
            unsafe {
                ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::KeyIterator>>
        for crate::q_map_of_q_string_q_variant::KeyIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QMap<QString, QVariant>::key_iterator::operator==(QMap<QString, QVariant>::key_iterator o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::KeyIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_key_iterator_operator__1(
                    self as *const crate::q_map_of_q_string_q_variant::KeyIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_map_of_q_string_q_variant::KeyIterator {
        type Output = ::cpp_utils::MutRef<crate::q_map_of_q_string_q_variant::KeyIterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::key_iterator& QMap<QString, QVariant>::key_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#keyEnd">QHash::keyEnd</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_map_of_q_string_q_variant::KeyIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_key_iterator_operator__3(
                    self as *mut crate::q_map_of_q_string_q_variant::KeyIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_map_of_q_string_q_variant::KeyIterator {
        type Output = ::cpp_utils::MutRef<crate::q_map_of_q_string_q_variant::KeyIterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::key_iterator& QMap<QString, QVariant>::key_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#keyBegin">QHash::keyBegin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_map_of_q_string_q_variant::KeyIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_key_iterator_operator__5(
                    self as *mut crate::q_map_of_q_string_q_variant::KeyIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_map_of_q_string_q_variant::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QMap<QString, QVariant>::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmap-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>&lt;Key, T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> (or <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>) and to modify the value (but not the key) stored under a particular key. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, you should use <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a>. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> function like <a href="http://doc.qt.io/qt-5/qmap.html#begin">QMap::begin</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#end">QMap::end</a>(), or <a href="http://doc.qt.io/qt-5/qmap.html#find">QMap::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a map:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> map;
        ///   map<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
        ///   map<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   map<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> map<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Unlike <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, which stores its items in an arbitrary order, <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> stores its items ordered by key. Items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">QMap::insertMulti</a>(), or due to a <a href="http://doc.qt.io/qt-5/qmap.html#unite">unite</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a>. Here's an example that increments every value stored in the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> map<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     i<span class="operator">.</span>value() <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Here's an example that removes all the items whose key is a string that starts with an underscore character:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> map<span class="operator">.</span>begin();
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end()) {
        /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     i <span class="operator">=</span> map<span class="operator">.</span>erase(i);
        /// &#32;     <span class="keyword">else</span>
        /// &#32;   &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        ///   }
        ///
        /// </pre>
        /// <p>The call to <a href="http://doc.qt.io/qt-5/qmap.html#erase">QMap::erase</a>() removes the item pointed to by the iterator from the map, and returns an iterator to the next item. Here's another way of removing an item while iterating:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> map<span class="operator">.</span>begin();
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end()) {
        /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator prev <span class="operator">=</span> i;
        /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        /// &#32;     <span class="keyword">if</span> (prev<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     map<span class="operator">.</span>erase(prev);
        ///   }
        ///
        /// </pre>
        /// <p>It might be tempting to write code like this:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="comment">// WRONG</span>
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end()) {
        /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     map<span class="operator">.</span>erase(i);
        /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        ///   }
        ///
        /// </pre>
        /// <p>However, this will potentially crash in <code>++i</code>, because <code>i</code> is a dangling iterator after the call to <a href="http://doc.qt.io/qt-5/qmap.html#erase">erase</a>().</p>
        /// <p>Multiple iterators can be used on the same map. If you add items to the map, existing iterators will remain valid. If you remove items from the map, iterators that point to the removed items will become dangling iterators.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_iterator_diterator(
                self as *mut crate::q_map_of_q_string_q_variant::Iterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_map_of_q_string_q_variant::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QMap<QString, QVariant>::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>&lt;Key, T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> (or a <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>). If you want to modify the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> as you iterate over it, you must use <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> function like <a href="http://doc.qt.io/qt-5/qmap.html#constBegin">QMap::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#constEnd">QMap::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qmap.html#find">QMap::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a map:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> map;
        ///   map<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
        ///   map<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   map<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> map<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Unlike <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, which stores its items in an arbitrary order, <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> stores its items ordered by key. Items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">QMap::insertMulti</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
        /// <p>Multiple iterators can be used on the same map. If you add items to the map, existing iterators will remain valid. If you remove items from the map, iterators that point to the removed items will become dangling iterators.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_const_iterator_dconst_iterator(
                self as *mut crate::q_map_of_q_string_q_variant::ConstIterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_map_of_q_string_q_variant::KeyIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a> keys.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QMap<QString, QVariant>::key_iterator::~key_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a> keys.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> is essentially the same as <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> with the difference that operator*() and operator-&gt;() return a key instead of a value.</p>
        /// <p>For most uses <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> and <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> should be used, you can easily access the key by calling <a href="http://doc.qt.io/qt-5/qmap-iterator.html#key">QMap::iterator::key</a>():</p>
        /// <pre class="cpp">
        ///
        ///   <span class="keyword">for</span> (<span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator it <span class="operator">=</span> map<span class="operator">.</span>cbegin()<span class="operator">,</span> end <span class="operator">=</span> map<span class="operator">.</span>cend(); it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The key: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Also the value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> (<span class="operator">*</span>it) <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///   }
        ///
        /// </pre>
        /// <p>However, to have interoperability between <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>'s keys and STL-style algorithms we need an iterator that dereferences to a key instead of a value. With <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> we can apply an algorithm to a range of keys without having to call <a href="http://doc.qt.io/qt-5/qmap.html#keys">QMap::keys</a>(), which is inefficient as it costs one <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> iteration and memory allocation to create a temporary <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <pre class="cpp">
        ///
        ///   <span class="comment">// Inefficient, keys() is expensive</span>
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> keys <span class="operator">=</span> map<span class="operator">.</span>keys();
        ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(map<span class="operator">.</span>cbegin()<span class="operator">,</span> map<span class="operator">.</span>cend()<span class="operator">,</span> isPrimeNumber);
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(map2<span class="operator">.</span>keys());
        ///
        ///   <span class="comment">// Efficient, no memory allocation needed</span>
        ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(map<span class="operator">.</span>keyBegin()<span class="operator">,</span> map<span class="operator">.</span>keyEnd()<span class="operator">,</span> isPrimeNumber);
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(map2<span class="operator">.</span>keyBegin()<span class="operator">,</span> map2<span class="operator">.</span>keyEnd());
        ///
        /// </pre>
        /// <p><a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> is const, it's not possible to modify the key.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> function like <a href="http://doc.qt.io/qt-5/qmap.html#keyBegin">QMap::keyBegin</a>() or <a href="http://doc.qt.io/qt-5/qmap.html#keyEnd">QMap::keyEnd</a>().</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_key_iterator_dkey_iterator(
                self as *mut crate::q_map_of_q_string_q_variant::KeyIterator,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> class is a template class that provides a red-black-tree-based dictionary.</p>
///
/// C++ class: <span style='color: green;'>```QMap<QString, QVariant>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qmap.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> class is a template class that provides a red-black-tree-based dictionary.</p>
/// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>&lt;Key, T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores (key, value) pairs and provides fast lookup of the value associated with a key.</p>
/// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> provide very similar functionality. The differences are:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> provides average faster lookups than <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.)</li>
/// <li>When iterating over a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, the items are arbitrarily ordered. With <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, the items are always sorted by key.</li>
/// <li>The key type of a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> must provide operator==() and a global <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a>(Key) function. The key type of a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> must provide operator&lt;() specifying a total order. Since Qt 5.8.1 it is also safe to use a pointer type as key, even if the underlying operator&lt;() does not provide a total order.</li>
/// </ul>
/// <p>Here's an example <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> with <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> keys and <code>int</code> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> map;
///
/// </pre>
/// <p>To insert a (key, value) pair into the map, you can use operator[]():</p>
/// <pre class="cpp">
///
///   map<span class="operator">[</span><span class="string">"one"</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>;
///   map<span class="operator">[</span><span class="string">"three"</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">3</span>;
///   map<span class="operator">[</span><span class="string">"seven"</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">7</span>;
///
/// </pre>
/// <p>This inserts the following three (key, value) pairs into the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>: ("one", 1), ("three", 3), and ("seven", 7). Another way to insert items into the map is to use <a href="http://doc.qt.io/qt-5/qmap.html#insert">insert</a>():</p>
/// <pre class="cpp">
///
///   map<span class="operator">.</span>insert(<span class="string">"twelve"</span><span class="operator">,</span> <span class="number">12</span>);
///
/// </pre>
/// <p>To look up a value, use operator[]() or <a href="http://doc.qt.io/qt-5/qmap.html#value">value</a>():</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> num1 <span class="operator">=</span> map<span class="operator">[</span><span class="string">"thirteen"</span><span class="operator">]</span>;
///   <span class="type">int</span> num2 <span class="operator">=</span> map<span class="operator">.</span>value(<span class="string">"thirteen"</span>);
///
/// </pre>
/// <p>If there is no item with the specified key in the map, these functions return a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>.</p>
/// <p>If you want to check whether the map contains a certain key, use <a href="http://doc.qt.io/qt-5/qmap.html#contains">contains</a>():</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> timeout <span class="operator">=</span> <span class="number">30</span>;
///   <span class="keyword">if</span> (map<span class="operator">.</span>contains(<span class="string">"TIMEOUT"</span>))
/// &#32;     timeout <span class="operator">=</span> map<span class="operator">.</span>value(<span class="string">"TIMEOUT"</span>);
///
/// </pre>
/// <p>There is also a <a href="http://doc.qt.io/qt-5/qmap.html#value">value</a>() overload that uses its second argument as a default value if there is no item with the specified key:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> timeout <span class="operator">=</span> map<span class="operator">.</span>value(<span class="string">"TIMEOUT"</span><span class="operator">,</span> <span class="number">30</span>);
///
/// </pre>
/// <p>In general, we recommend that you use <a href="http://doc.qt.io/qt-5/qmap.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#value">value</a>() rather than operator[]() for looking up a key in a map. The reason is that operator[]() silently inserts an item into the map if no item exists with the same key (unless the map is const). For example, the following code snippet will create 1000 items in memory:</p>
/// <pre class="cpp">
///
///   <span class="comment">// WRONG</span>
///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> map;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> <span class="number">1000</span>; <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (map<span class="operator">[</span>i<span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> okButton)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found button at index "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p>To avoid this problem, replace <code>map[i]</code> with <code>map.value(i)</code> in the code above.</p>
/// <p>If you want to navigate through all the (key, value) pairs stored in a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, you can use an iterator. <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qmapiterator.html">QMapIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablemapiterator.html">QMutableMapIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a>). Here's how to iterate over a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, int&gt; using a Java-style iterator:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qmapiterator.html">QMapIterator</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> i(map);
///   <span class="keyword">while</span> (i<span class="operator">.</span>hasNext()) {
/// &#32;     i<span class="operator">.</span>next();
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p>Here's the same code, but using an STL-style iterator this time:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i <span class="operator">=</span> map<span class="operator">.</span>constBegin();
///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>constEnd()) {
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
/// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
///   }
///
/// </pre>
/// <p>The items are traversed in ascending key order.</p>
/// <p>Normally, a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> allows only one value per key. If you call <a href="http://doc.qt.io/qt-5/qmap.html#insert">insert</a>() with a key that already exists in the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, the previous value will be erased. For example:</p>
/// <pre class="cpp">
///
///   map<span class="operator">.</span>insert(<span class="string">"plenty"</span><span class="operator">,</span> <span class="number">100</span>);
///   map<span class="operator">.</span>insert(<span class="string">"plenty"</span><span class="operator">,</span> <span class="number">2000</span>);
///   <span class="comment">// map.value("plenty") == 2000</span>
///
/// </pre>
/// <p>However, you can store multiple values per key by using <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">insertMulti</a>() instead of <a href="http://doc.qt.io/qt-5/qmap.html#insert">insert</a>() (or using the convenience subclass <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>). If you want to retrieve all the values for a single key, you can use values(const Key &amp;key), which returns a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> values <span class="operator">=</span> map<span class="operator">.</span>values(<span class="string">"plenty"</span>);
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> values<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> values<span class="operator">.</span>at(i) <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>The items that share the same key are available from most recently to least recently inserted. Another approach is to call <a href="http://doc.qt.io/qt-5/qmap.html#find">find</a>() to get the STL-style iterator for the first item with a key and iterate from there:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> map<span class="operator">.</span>find(<span class="string">"plenty"</span>);
///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end() <span class="operator">&amp;</span><span class="operator">&amp;</span> i<span class="operator">.</span>key() <span class="operator">=</span><span class="operator">=</span> <span class="string">"plenty"</span>) {
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
/// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
///   }
///
/// </pre>
/// <p>If you only need to extract the values from a map (not the keys), you can also use <a href="http://doc.qt.io/qt-5/containers.html#foreach">foreach</a>:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> map;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   foreach (<span class="type">int</span> value<span class="operator">,</span> map)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> value <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>Items can be removed from the map in several ways. One way is to call <a href="http://doc.qt.io/qt-5/qmap.html#remove">remove</a>(); this will remove any item with the given key. Another way is to use <a href="http://doc.qt.io/qt-5/qmutablemapiterator.html#remove">QMutableMapIterator::remove</a>(). In addition, you can clear the entire map using <a href="http://doc.qt.io/qt-5/qmap.html#clear">clear</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>'s key and value data types must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. This covers most data types you are likely to encounter, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. In addition, <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>'s key type must provide operator&lt;(). <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> uses it to keep its items sorted, and assumes that two keys <code>x</code> and <code>y</code> are equal if neither <code>x &lt; y</code> nor <code>y &lt; x</code> is true.</p>
/// <p>Example:</p>
/// <pre class="cpp">
///
///   <span class="preprocessor">#ifndef EMPLOYEE_H</span>
///   <span class="preprocessor">#define EMPLOYEE_H</span>
///
///   <span class="keyword">class</span> Employee
///   {
///   <span class="keyword">public</span>:
/// &#32;     Employee() {}
/// &#32;     Employee(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>name<span class="operator">,</span> <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span> <span class="operator">&amp;</span>dateOfBirth);
/// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///
///   <span class="keyword">private</span>:
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> myName;
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span> myDateOfBirth;
///   };
///
///   <span class="keyword">inline</span> bool <span class="keyword">operator</span><span class="operator">&lt;</span>(<span class="keyword">const</span> Employee <span class="operator">&amp;</span>e1<span class="operator">,</span> <span class="keyword">const</span> Employee <span class="operator">&amp;</span>e2)
///   {
/// &#32;     <span class="keyword">if</span> (e1<span class="operator">.</span>name() <span class="operator">!</span><span class="operator">=</span> e2<span class="operator">.</span>name())
/// &#32;   &#32;     <span class="keyword">return</span> e1<span class="operator">.</span>name() <span class="operator">&lt;</span> e2<span class="operator">.</span>name();
/// &#32;     <span class="keyword">return</span> e1<span class="operator">.</span>dateOfBirth() <span class="operator">&lt;</span> e2<span class="operator">.</span>dateOfBirth();
///   }
///
///   <span class="preprocessor">#endif // EMPLOYEE_H</span>
///
/// </pre>
/// <p>In the example, we start by comparing the employees' names. If they're equal, we compare their dates of birth to break the tie.</p></div>
#[repr(C)]
pub struct QMapOfQStringQVariant {
    _unused: u8,
}
impl QMapOfQStringQVariant {
    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::const_iterator QMap<QString, QVariant>::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the map.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_cbegin(
            self as *const crate::QMapOfQStringQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::const_iterator QMap<QString, QVariant>::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the map.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#end">end</a>().</p></div>
    pub unsafe fn cend(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_cend(
            self as *const crate::QMapOfQStringQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all items from the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMap<QString, QVariant>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all items from the map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#remove">remove</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_clear(
            self as *mut crate::QMapOfQStringQVariant,
        )
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::const_iterator QMap<QString, QVariant>::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_constBegin(
            self as *const crate::QMapOfQStringQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::const_iterator QMap<QString, QVariant>::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#end">end</a>().</p></div>
    pub unsafe fn const_end(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_constEnd(
            self as *const crate::QMapOfQStringQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns an const iterator pointing to the item with key <i>key</i> in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::const_iterator QMap<QString, QVariant>::constFind(const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#constFind">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an const iterator pointing to the item with key <i>key</i> in the map.</p>
    /// <p>If the map contains no item with key <i>key</i>, the function returns <a href="http://doc.qt.io/qt-5/qmap.html#constEnd">constEnd</a>().</p>
    /// <p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#find">find</a>() and <a href="http://doc.qt.io/qt-5/qmultimap.html#constFind">QMultiMap::constFind</a>().</p></div>
    pub unsafe fn const_find(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_constFind(
            self as *const crate::QMapOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if the map contains an item with key <i>key</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMap<QString, QVariant>::contains(const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the map contains an item with key <i>key</i>; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#count-1">count</a>() and <a href="http://doc.qt.io/qt-5/qmultimap.html#contains">QMultiMap::contains</a>().</p></div>
    pub unsafe fn contains(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_contains(
            self as *const crate::QMapOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this map and returns a reference to this map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>& QMap<QString, QVariant>::operator=(const QMap<QString, QVariant>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this map and returns a reference to this map.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMapOfQStringQVariant>>,
    ) -> ::cpp_utils::MutRef<crate::QMapOfQStringQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_operator_(
            self as *mut crate::QMapOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMapOfQStringQVariant>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of items associated with key <i>key</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMap<QString, QVariant>::count(const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items associated with key <i>key</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#contains">contains</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">insertMulti</a>(), and <a href="http://doc.qt.io/qt-5/qmultimap.html#count">QMultiMap::count</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_count(
            self as *const crate::QMapOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMap<QString, QVariant>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qmap.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_count1(
            self as *const crate::QMapOfQStringQVariant,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QMap<QString, QVariant>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_detach(
            self as *mut crate::QMapOfQStringQVariant,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qmap.html#isEmpty">isEmpty</a>(), returning true if the map is empty; otherwise returning false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMap<QString, QVariant>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qmap.html#isEmpty">isEmpty</a>(), returning true if the map is empty; otherwise returning false.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_empty(
            self as *const crate::QMapOfQStringQVariant,
        )
    }

    /// <p>Removes the (key, value) pair pointed to by the iterator <i>pos</i> from the map, and returns an iterator to the next item in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::iterator QMap<QString, QVariant>::erase(QMap<QString, QVariant>::iterator it)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the (key, value) pair pointed to by the iterator <i>pos</i> from the map, and returns an iterator to the next item in the map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#remove">remove</a>().</p></div>
    pub unsafe fn erase(
        &mut self,
        it: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::Iterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_erase(
                self as *mut crate::QMapOfQStringQVariant,
                ::cpp_utils::CastInto::<
                    ::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::Iterator>,
                >::cast_into(it)
                .as_raw_ptr(),
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns an iterator pointing to the item with key <i>key</i> in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::iterator QMap<QString, QVariant>::find(const QString& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#find">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator pointing to the item with key <i>key</i> in the map.</p>
    /// <p>If the map contains no item with key <i>key</i>, the function returns <a href="http://doc.qt.io/qt-5/qmap.html#end">end</a>().</p>
    /// <p>If the map contains multiple items with key <i>key</i>, this function returns an iterator that points to the most recently inserted value. The other values are accessible by incrementing the iterator. For example, here's some code that iterates over all the items with the same key:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> map;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i <span class="operator">=</span> map<span class="operator">.</span>find(<span class="string">"HDR"</span>);
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end() <span class="operator">&amp;</span><span class="operator">&amp;</span> i<span class="operator">.</span>key() <span class="operator">=</span><span class="operator">=</span> <span class="string">"HDR"</span>) {
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    ///   }
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#constFind">constFind</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#value">value</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#values">values</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#lowerBound">lowerBound</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#upperBound">upperBound</a>(), and <a href="http://doc.qt.io/qt-5/qmultimap.html#find">QMultiMap::find</a>().</p></div>
    pub unsafe fn find_mut(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_find(
            self as *mut crate::QMapOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::const_iterator QMap<QString, QVariant>::find(const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#find-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn find(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_find1(
            self as *const crate::QMapOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the first value in the map, that is the value mapped to the smallest key. This function assumes that the map is not empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant& QMap<QString, QVariant>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first value in the map, that is the value mapped to the smallest key. This function assumes that the map is not empty.</p>
    /// <p>When unshared (or const version is called), this executes in <a href="http://doc.qt.io/qt-5/containers.html#constant-time">constant time</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#last">last</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#firstKey">firstKey</a>(), and <a href="http://doc.qt.io/qt-5/qmap.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_first(
            self as *mut crate::QMapOfQStringQVariant,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QVariant& QMap<QString, QVariant>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 5.2.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_first1(
            self as *const crate::QMapOfQStringQVariant,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the smallest key in the map. This function assumes that the map is not empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QString& QMap<QString, QVariant>::firstKey() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#firstKey">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the smallest key in the map. This function assumes that the map is not empty.</p>
    /// <p>This executes in <a href="http://doc.qt.io/qt-5/containers.html#constant-time">constant time</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#lastKey">lastKey</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#first">first</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#keyBegin">keyBegin</a>(), and <a href="http://doc.qt.io/qt-5/qmap.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn first_key(&self) -> ::cpp_utils::Ref<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_firstKey(
            self as *const crate::QMapOfQStringQVariant,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the value associated with the key <i>key</i> as a modifiable reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant& QMap<QString, QVariant>::operator[](const QString& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value associated with the key <i>key</i> as a modifiable reference.</p>
    /// <p>If the map contains no item with key <i>key</i>, the function inserts a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a> into the map with key <i>key</i>, and returns a reference to it. If the map contains multiple items with key <i>key</i>, this function returns a reference to the most recently inserted value.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::MutRef<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_operator__2(
            self as *mut crate::QMapOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QMap<QString, QVariant>::operator[](const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qmap.html#value">value</a>().</p></div>
    pub unsafe fn index(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_operator__3(
            self as *const crate::QMapOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Inserts a new item with the key <i>key</i> and a value of <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::iterator QMap<QString, QVariant>::insert(const QString& key, const QVariant& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts a new item with the key <i>key</i> and a value of <i>value</i>.</p>
    /// <p>If there is already an item with the key <i>key</i>, that item's value is replaced with <i>value</i>.</p>
    /// <p>If there are multiple items with the key <i>key</i>, the most recently inserted item's value is replaced with <i>value</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">insertMulti</a>().</p></div>
    pub unsafe fn insert_2a(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_insert(
            self as *mut crate::QMapOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::iterator QMap<QString, QVariant>::insert(QMap<QString, QVariant>::const_iterator pos, const QString& key, const QVariant& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts a new item with the key <i>key</i> and value <i>value</i> and with hint <i>pos</i> suggesting where to do the insert.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qmap.html#constBegin">constBegin</a>() is used as hint it indicates that the <i>key</i> is less than any key in the map while <a href="http://doc.qt.io/qt-5/qmap.html#constEnd">constEnd</a>() suggests that the <i>key</i> is (strictly) larger than any key in the map. Otherwise the hint should meet the condition (<i>pos</i> - 1).<a href="http://doc.qt.io/qt-5/qmap.html#key">key</a>() &lt; <i>key</i> &lt;= pos.<a href="http://doc.qt.io/qt-5/qmap.html#key">key</a>(). If the hint <i>pos</i> is wrong it is ignored and a regular insert is done.</p>
    /// <p>If there is already an item with the key <i>key</i>, that item's value is replaced with <i>value</i>.</p>
    /// <p>If there are multiple items with the key <i>key</i>, then exactly one of them is replaced with <i>value</i>.</p>
    /// <p>If the hint is correct and the map is unshared, the insert executes in amortized <a href="http://doc.qt.io/qt-5/containers.html#constant-time">constant time</a>.</p>
    /// <p>When creating a map from sorted data inserting the largest key first with <a href="http://doc.qt.io/qt-5/qmap.html#constBegin">constBegin</a>() is faster than inserting in sorted order with <a href="http://doc.qt.io/qt-5/qmap.html#constEnd">constEnd</a>(), since <a href="http://doc.qt.io/qt-5/qmap.html#constEnd">constEnd</a>() - 1 (which is needed to check if the hint is valid) needs <a href="http://doc.qt.io/qt-5/containers.html#logarithmic-time">logarithmic time</a>.</p>
    /// <p><b>Note:</b> Be careful with the hint. Providing an iterator from an older shared instance might crash but there is also a risk that it will silently corrupt both the map and the <i>pos</i> map.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">insertMulti</a>().</p></div>
    pub unsafe fn insert_3a(
        &mut self,
        pos: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::ConstIterator>,
        >,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_insert1(
            self as *mut crate::QMapOfQStringQVariant,
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::ConstIterator>,
            >::cast_into(pos)
            .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Inserts a new item with the key <i>key</i> and a value of <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::iterator QMap<QString, QVariant>::insertMulti(const QString& key, const QVariant& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts a new item with the key <i>key</i> and a value of <i>value</i>.</p>
    /// <p>If there is already an item with the same key in the map, this function will simply create a new one. (This behavior is different from <a href="http://doc.qt.io/qt-5/qmap.html#insert">insert</a>(), which overwrites the value of an existing item.)</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#values">values</a>().</p></div>
    pub unsafe fn insert_multi_2a(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_insertMulti(
            self as *mut crate::QMapOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::iterator QMap<QString, QVariant>::insertMulti(QMap<QString, QVariant>::const_iterator pos, const QString& akey, const QVariant& avalue)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts a new item with the key <i>key</i> and value <i>value</i> and with hint <i>pos</i> suggesting where to do the insert.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qmap.html#constBegin">constBegin</a>() is used as hint it indicates that the <i>key</i> is less than any key in the map while <a href="http://doc.qt.io/qt-5/qmap.html#constEnd">constEnd</a>() suggests that the <i>key</i> is larger than any key in the map. Otherwise the hint should meet the condition (<i>pos</i> - 1).<a href="http://doc.qt.io/qt-5/qmap.html#key">key</a>() &lt; <i>key</i> &lt;= pos.<a href="http://doc.qt.io/qt-5/qmap.html#key">key</a>(). If the hint <i>pos</i> is wrong it is ignored and a regular <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">insertMulti</a> is done.</p>
    /// <p>If there is already an item with the same key in the map, this function will simply create a new one.</p>
    /// <p><b>Note:</b> Be careful with the hint. Providing an iterator from an older shared instance might crash but there is also a risk that it will silently corrupt both the map and the <i>pos</i> map.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#insert">insert</a>().</p></div>
    pub unsafe fn insert_multi_3a(
        &mut self,
        pos: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::ConstIterator>,
        >,
        akey: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        avalue: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_insertMulti1(
            self as *mut crate::QMapOfQStringQVariant,
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::q_map_of_q_string_q_variant::ConstIterator>,
            >::cast_into(pos)
            .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(akey).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(avalue)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QMap<QString, QVariant>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_isDetached(
            self as *const crate::QMapOfQStringQVariant,
        )
    }

    /// <p>Returns <code>true</code> if the map contains no items; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMap<QString, QVariant>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the map contains no items; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_isEmpty(
            self as *const crate::QMapOfQStringQVariant,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QMap<QString, QVariant>::isSharedWith(const QMap<QString, QVariant>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMapOfQStringQVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_isSharedWith(
            self as *const crate::QMapOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMapOfQStringQVariant>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QMap<QString, QVariant>::key(const QVariant& value, const QString& defaultKey = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#key">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the first key with value <i>value</i>, or <i>defaultKey</i> if the map contains no item with value <i>value</i>. If no <i>defaultKey</i> is provided the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed key</a>.</p>
    /// <p>This function can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>'s internal data structure is optimized for fast lookup by key, not by value.</p>
    /// <p>This function was introduced in  Qt 4.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#keys">keys</a>().</p></div>
    pub unsafe fn key_2a(
        &self,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
        default_key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_key(
            self as *const crate::QMapOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(default_key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QMap<QString, QVariant>::key(const QVariant& value) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#key">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the first key with value <i>value</i>, or <i>defaultKey</i> if the map contains no item with value <i>value</i>. If no <i>defaultKey</i> is provided the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed key</a>.</p>
    /// <p>This function can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>'s internal data structure is optimized for fast lookup by key, not by value.</p>
    /// <p>This function was introduced in  Qt 4.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#keys">keys</a>().</p></div>
    pub unsafe fn key_1a(
        &self,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_key1(
            self as *const crate::QMapOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first key in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::key_iterator QMap<QString, QVariant>::keyBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#keyBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first key in the map.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#keyEnd">keyEnd</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#firstKey">firstKey</a>().</p></div>
    pub unsafe fn key_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::KeyIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_keyBegin(
            self as *const crate::QMapOfQStringQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last key in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::key_iterator QMap<QString, QVariant>::keyEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#keyEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last key in the map.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#keyBegin">keyBegin</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#lastKey">lastKey</a>().</p></div>
    pub unsafe fn key_end(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::KeyIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_keyEnd(
            self as *const crate::QMapOfQStringQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list containing all the keys in the map in ascending order. Keys that occur multiple times in the map (because items were inserted with <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">insertMulti</a>(), or <a href="http://doc.qt.io/qt-5/qmap.html#unite">unite</a>() was used) also occur multiple times in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QString> QMap<QString, QVariant>::keys() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#keys">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list containing all the keys in the map in ascending order. Keys that occur multiple times in the map (because items were inserted with <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">insertMulti</a>(), or <a href="http://doc.qt.io/qt-5/qmap.html#unite">unite</a>() was used) also occur multiple times in the list.</p>
    /// <p>To obtain a list of unique keys, where each key from the map only occurs once, use <a href="http://doc.qt.io/qt-5/qmap.html#uniqueKeys">uniqueKeys</a>().</p>
    /// <p>The order is guaranteed to be the same as that used by <a href="http://doc.qt.io/qt-5/qmap.html#values">values</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#uniqueKeys">uniqueKeys</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#values">values</a>(), and <a href="http://doc.qt.io/qt-5/qmap.html#key">key</a>().</p></div>
    pub unsafe fn keys_0a(&self) -> ::cpp_utils::CppBox<crate::QListOfQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_keys(
            self as *const crate::QMapOfQStringQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QString> QMap<QString, QVariant>::keys(const QVariant& value) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#keys-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a list containing all the keys associated with value <i>value</i> in ascending order.</p>
    /// <p>This function can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>'s internal data structure is optimized for fast lookup by key, not by value.</p></div>
    pub unsafe fn keys_1a(
        &self,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_keys1(
            self as *const crate::QMapOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the last value in the map, that is the value mapped to the largest key. This function assumes that the map is not empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant& QMap<QString, QVariant>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last value in the map, that is the value mapped to the largest key. This function assumes that the map is not empty.</p>
    /// <p>When unshared (or const version is called), this executes in <a href="http://doc.qt.io/qt-5/containers.html#logarithmic-time">logarithmic time</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#first">first</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#lastKey">lastKey</a>(), and <a href="http://doc.qt.io/qt-5/qmap.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_last(
            self as *mut crate::QMapOfQStringQVariant,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QVariant& QMap<QString, QVariant>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 5.2.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_last1(
            self as *const crate::QMapOfQStringQVariant,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the largest key in the map. This function assumes that the map is not empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QString& QMap<QString, QVariant>::lastKey() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#lastKey">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the largest key in the map. This function assumes that the map is not empty.</p>
    /// <p>This executes in <a href="http://doc.qt.io/qt-5/containers.html#logarithmic-time">logarithmic time</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#firstKey">firstKey</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#last">last</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#keyEnd">keyEnd</a>(), and <a href="http://doc.qt.io/qt-5/qmap.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn last_key(&self) -> ::cpp_utils::Ref<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_lastKey(
            self as *const crate::QMapOfQStringQVariant,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns an iterator pointing to the first item with key <i>key</i> in the map. If the map contains no item with key <i>key</i>, the function returns an iterator to the nearest item with a greater key.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::iterator QMap<QString, QVariant>::lowerBound(const QString& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#lowerBound">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator pointing to the first item with key <i>key</i> in the map. If the map contains no item with key <i>key</i>, the function returns an iterator to the nearest item with a greater key.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> map;
    ///   map<span class="operator">.</span>insert(<span class="number">1</span><span class="operator">,</span> <span class="string">"one"</span>);
    ///   map<span class="operator">.</span>insert(<span class="number">5</span><span class="operator">,</span> <span class="string">"five"</span>);
    ///   map<span class="operator">.</span>insert(<span class="number">10</span><span class="operator">,</span> <span class="string">"ten"</span>);
    ///
    ///   map<span class="operator">.</span>lowerBound(<span class="number">0</span>);      <span class="comment">// returns iterator to (1, "one")</span>
    ///   map<span class="operator">.</span>lowerBound(<span class="number">1</span>);      <span class="comment">// returns iterator to (1, "one")</span>
    ///   map<span class="operator">.</span>lowerBound(<span class="number">2</span>);      <span class="comment">// returns iterator to (5, "five")</span>
    ///   map<span class="operator">.</span>lowerBound(<span class="number">10</span>);     <span class="comment">// returns iterator to (10, "ten")</span>
    ///   map<span class="operator">.</span>lowerBound(<span class="number">999</span>);    <span class="comment">// returns end()</span>
    ///
    /// </pre>
    /// <p>If the map contains multiple items with key <i>key</i>, this function returns an iterator that points to the most recently inserted value. The other values are accessible by incrementing the iterator. For example, here's some code that iterates over all the items with the same key:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> map;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i <span class="operator">=</span> map<span class="operator">.</span>lowerBound(<span class="string">"HDR"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator upperBound <span class="operator">=</span> map<span class="operator">.</span>upperBound(<span class="string">"HDR"</span>);
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> upperBound) {
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    ///   }
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#upperBound">upperBound</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#find">find</a>().</p></div>
    pub unsafe fn lower_bound_mut(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_lowerBound(
            self as *mut crate::QMapOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::const_iterator QMap<QString, QVariant>::lowerBound(const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#lowerBound-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn lower_bound(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_lowerBound1(
            self as *const crate::QMapOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs an empty map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMap<QString, QVariant>::QMap()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#QMap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#clear">clear</a>().</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QMapOfQStringQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_QMap();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMap<QString, QVariant>::QMap(const QMap<QString, QVariant>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#QMap-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation occurs in <a href="http://doc.qt.io/qt-5/containers.html#constant-time">constant time</a>, because <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and this takes <a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMapOfQStringQVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QMapOfQStringQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_QMap1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMapOfQStringQVariant>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all the items that have the key <i>key</i> from the map. Returns the number of items removed which is usually 1 but will be 0 if the key isn't in the map, or &gt; 1 if <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">insertMulti</a>() has been used with the <i>key</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMap<QString, QVariant>::remove(const QString& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#remove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all the items that have the key <i>key</i> from the map. Returns the number of items removed which is usually 1 but will be 0 if the key isn't in the map, or &gt; 1 if <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">insertMulti</a>() has been used with the <i>key</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#clear">clear</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#take">take</a>(), and <a href="http://doc.qt.io/qt-5/qmultimap.html#remove">QMultiMap::remove</a>().</p></div>
    pub unsafe fn remove(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_remove(
            self as *mut crate::QMapOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QMap<QString, QVariant>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_setSharable(
            self as *mut crate::QMapOfQStringQVariant,
            sharable,
        )
    }

    /// <p>Returns the number of (key, value) pairs in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMap<QString, QVariant>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of (key, value) pairs in the map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#count-1">count</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_size(
            self as *const crate::QMapOfQStringQVariant,
        )
    }

    /// <p>Swaps map <i>other</i> with this map. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMap<QString, QVariant>::swap(QMap<QString, QVariant>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps map <i>other</i> with this map. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QMapOfQStringQVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_swap(
            self as *mut crate::QMapOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QMapOfQStringQVariant>>::cast_into(
                other,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Removes the item with the key <i>key</i> from the map and returns the value associated with it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QMap<QString, QVariant>::take(const QString& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#take">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item with the key <i>key</i> from the map and returns the value associated with it.</p>
    /// <p>If the item does not exist in the map, the function simply returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If there are multiple items for <i>key</i> in the map, only the most recently inserted one is removed and returned.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qmap.html#remove">remove</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#remove">remove</a>().</p></div>
    pub unsafe fn take(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_take(
            self as *mut crate::QMapOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list containing all the keys in the map in ascending order. Keys that occur multiple times in the map (because items were inserted with <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">insertMulti</a>(), or <a href="http://doc.qt.io/qt-5/qmap.html#unite">unite</a>() was used) occur only once in the returned list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QString> QMap<QString, QVariant>::uniqueKeys() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#uniqueKeys">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list containing all the keys in the map in ascending order. Keys that occur multiple times in the map (because items were inserted with <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">insertMulti</a>(), or <a href="http://doc.qt.io/qt-5/qmap.html#unite">unite</a>() was used) occur only once in the returned list.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#keys">keys</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#values">values</a>().</p></div>
    pub unsafe fn unique_keys(&self) -> ::cpp_utils::CppBox<crate::QListOfQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_uniqueKeys(
            self as *const crate::QMapOfQStringQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Inserts all the items in the <i>other</i> map into this map. If a key is common to both maps, the resulting map will contain the key multiple times.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>& QMap<QString, QVariant>::unite(const QMap<QString, QVariant>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#unite">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts all the items in the <i>other</i> map into this map. If a key is common to both maps, the resulting map will contain the key multiple times.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">insertMulti</a>().</p></div>
    pub unsafe fn unite(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMapOfQStringQVariant>>,
    ) -> ::cpp_utils::MutRef<crate::QMapOfQStringQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_unite(
            self as *mut crate::QMapOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMapOfQStringQVariant>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns an iterator pointing to the item that immediately follows the last item with key <i>key</i> in the map. If the map contains no item with key <i>key</i>, the function returns an iterator to the nearest item with a greater key.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::iterator QMap<QString, QVariant>::upperBound(const QString& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#upperBound">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator pointing to the item that immediately follows the last item with key <i>key</i> in the map. If the map contains no item with key <i>key</i>, the function returns an iterator to the nearest item with a greater key.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> map;
    ///   map<span class="operator">.</span>insert(<span class="number">1</span><span class="operator">,</span> <span class="string">"one"</span>);
    ///   map<span class="operator">.</span>insert(<span class="number">5</span><span class="operator">,</span> <span class="string">"five"</span>);
    ///   map<span class="operator">.</span>insert(<span class="number">10</span><span class="operator">,</span> <span class="string">"ten"</span>);
    ///
    ///   map<span class="operator">.</span>upperBound(<span class="number">0</span>);      <span class="comment">// returns iterator to (1, "one")</span>
    ///   map<span class="operator">.</span>upperBound(<span class="number">1</span>);      <span class="comment">// returns iterator to (5, "five")</span>
    ///   map<span class="operator">.</span>upperBound(<span class="number">2</span>);      <span class="comment">// returns iterator to (5, "five")</span>
    ///   map<span class="operator">.</span>upperBound(<span class="number">10</span>);     <span class="comment">// returns end()</span>
    ///   map<span class="operator">.</span>upperBound(<span class="number">999</span>);    <span class="comment">// returns end()</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#lowerBound">lowerBound</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#find">find</a>().</p></div>
    pub unsafe fn upper_bound_mut(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_upperBound(
            self as *mut crate::QMapOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::const_iterator QMap<QString, QVariant>::upperBound(const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#upperBound-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn upper_bound(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_upperBound1(
            self as *const crate::QMapOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value associated with the key <i>key</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QMap<QString, QVariant>::value(const QString& key, const QVariant& defaultValue = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value associated with the key <i>key</i>.</p>
    /// <p>If the map contains no item with key <i>key</i>, the function returns <i>defaultValue</i>. If no <i>defaultValue</i> is specified, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If there are multiple items for <i>key</i> in the map, the value of the most recently inserted one is returned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#key">key</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#values">values</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qmap.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_2a(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_value(
            self as *const crate::QMapOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value associated with the key <i>key</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QMap<QString, QVariant>::value(const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value associated with the key <i>key</i>.</p>
    /// <p>If the map contains no item with key <i>key</i>, the function returns <i>defaultValue</i>. If no <i>defaultValue</i> is specified, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If there are multiple items for <i>key</i> in the map, the value of the most recently inserted one is returned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#key">key</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#values">values</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qmap.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_value1(
            self as *const crate::QMapOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list containing all the values in the map, in ascending order of their keys. If a key is associated with multiple values, all of its values will be in the list, and not just the most recently inserted one.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant> QMap<QString, QVariant>::values() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#values">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list containing all the values in the map, in ascending order of their keys. If a key is associated with multiple values, all of its values will be in the list, and not just the most recently inserted one.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#keys">keys</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#value">value</a>().</p></div>
    pub unsafe fn values_0a(&self) -> ::cpp_utils::CppBox<crate::QListOfQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_values(
            self as *const crate::QMapOfQStringQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant> QMap<QString, QVariant>::values(const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#values-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a list containing all the values associated with key <i>key</i>, from the most recently inserted to the least recently inserted one.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#count-1">count</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">insertMulti</a>().</p></div>
    pub unsafe fn values_1a(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_values1(
            self as *const crate::QMapOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_hash_of_q_string_q_variant {
    //! C++ type: <span style='color: green;'>```QHash<QString, QVariant>```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QHash<QString, QVariant>::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;Key, T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> (or <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>) and to modify the value (but not the key) associated with a particular key. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, you should use <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a>. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> function like <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>(), or <a href="http://doc.qt.io/qt-5/qhash.html#find">QHash::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a hash:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
    ///   hash<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
    ///   hash<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   hash<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> hash<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, which orders its items by key, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> stores its items in an arbitrary order. The only guarantee is that items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">QHash::insertMulti</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
    /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a>. Here's an example that increments every value stored in the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> by 2:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> hash<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     i<span class="operator">.</span>value() <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
    ///
    /// </pre>
    /// <p>Here's an example that removes all the items whose key is a string that starts with an underscore character:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> hash<span class="operator">.</span>begin();
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end()) {
    /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
    /// &#32;   &#32;     i <span class="operator">=</span> hash<span class="operator">.</span>erase(i);
    /// &#32;     <span class="keyword">else</span>
    /// &#32;   &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    ///   }
    ///
    /// </pre>
    /// <p>The call to <a href="http://doc.qt.io/qt-5/qhash.html#erase">QHash::erase</a>() removes the item pointed to by the iterator from the hash, and returns an iterator to the next item. Here's another way of removing an item while iterating:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> hash<span class="operator">.</span>begin();
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end()) {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator prev <span class="operator">=</span> i;
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    /// &#32;     <span class="keyword">if</span> (prev<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
    /// &#32;   &#32;     hash<span class="operator">.</span>erase(prev);
    ///   }
    ///
    /// </pre>
    /// <p>It might be tempting to write code like this:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="comment">// WRONG</span>
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end()) {
    /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
    /// &#32;   &#32;     hash<span class="operator">.</span>erase(i);
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    ///   }
    ///
    /// </pre>
    /// <p>However, this will potentially crash in <code>++i</code>, because <code>i</code> is a dangling iterator after the call to <a href="http://doc.qt.io/qt-5/qhash.html#erase">erase</a>().</p>
    /// <p>Multiple iterators can be used on the same hash. However, be aware that any modification performed directly on the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> has the potential of dramatically changing the order in which the items are stored in the hash, as they might cause <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> to rehash its internal data structure. There is one notable exception: <a href="http://doc.qt.io/qt-5/qhash.html#erase">QHash::erase</a>(). This function can safely be called while iterating, and won't affect the order of items in the hash. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> rather than <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::iterator& QHash<QString, QVariant>::iterator::operator=(const QHash<QString, QVariant>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;Key, T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> (or <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>) and to modify the value (but not the key) associated with a particular key. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, you should use <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a>. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> function like <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>(), or <a href="http://doc.qt.io/qt-5/qhash.html#find">QHash::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a hash:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
        ///   hash<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
        ///   hash<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   hash<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> hash<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Unlike <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, which orders its items by key, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> stores its items in an arbitrary order. The only guarantee is that items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">QHash::insertMulti</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a>. Here's an example that increments every value stored in the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> hash<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     i<span class="operator">.</span>value() <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Here's an example that removes all the items whose key is a string that starts with an underscore character:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> hash<span class="operator">.</span>begin();
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end()) {
        /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     i <span class="operator">=</span> hash<span class="operator">.</span>erase(i);
        /// &#32;     <span class="keyword">else</span>
        /// &#32;   &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        ///   }
        ///
        /// </pre>
        /// <p>The call to <a href="http://doc.qt.io/qt-5/qhash.html#erase">QHash::erase</a>() removes the item pointed to by the iterator from the hash, and returns an iterator to the next item. Here's another way of removing an item while iterating:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> hash<span class="operator">.</span>begin();
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end()) {
        /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator prev <span class="operator">=</span> i;
        /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        /// &#32;     <span class="keyword">if</span> (prev<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     hash<span class="operator">.</span>erase(prev);
        ///   }
        ///
        /// </pre>
        /// <p>It might be tempting to write code like this:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="comment">// WRONG</span>
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end()) {
        /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     hash<span class="operator">.</span>erase(i);
        /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        ///   }
        ///
        /// </pre>
        /// <p>However, this will potentially crash in <code>++i</code>, because <code>i</code> is a dangling iterator after the call to <a href="http://doc.qt.io/qt-5/qhash.html#erase">erase</a>().</p>
        /// <p>Multiple iterators can be used on the same hash. However, be aware that any modification performed directly on the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> has the potential of dramatically changing the order in which the items are stored in the hash, as they might cause <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> to rehash its internal data structure. There is one notable exception: <a href="http://doc.qt.io/qt-5/qhash.html#erase">QHash::erase</a>(). This function can safely be called while iterating, and won't affect the order of items in the hash. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> rather than <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::Iterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_hash_of_q_string_q_variant::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_iterator_operator_3(
                    self as *mut crate::q_hash_of_q_string_q_variant::Iterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::Iterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::iterator QHash<QString, QVariant>::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_iterator_operator__6(
                    self as *mut crate::q_hash_of_q_string_q_variant::Iterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::iterator QHash<QString, QVariant>::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_iterator_operator__4(
                    self as *mut crate::q_hash_of_q_string_q_variant::Iterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the current item's key as a const reference.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QString& QHash<QString, QVariant>::iterator::key() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current item's key as a const reference.</p>
        /// <p>There is no direct way of changing an item's key through an iterator, although it can be done by calling <a href="http://doc.qt.io/qt-5/qhash.html#erase">QHash::erase</a>() followed by <a href="http://doc.qt.io/qt-5/qhash.html#insert">QHash::insert</a>() or <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">QHash::insertMulti</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn key(&self) -> ::cpp_utils::Ref<crate::QString> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_iterator_key(
                self as *const crate::q_hash_of_q_string_q_variant::Iterator,
            );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QHash<QString, QVariant>::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like <a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>(), <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>(), and operator++() must not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>().</p></div>
        pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::Iterator>
        {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QHash<QString, QVariant>::iterator::iterator(void* node)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#iterator">C++ documentation</a> for <span style='color: green;'>```iterator::iterator()```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like <a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>(), <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>(), and operator++() must not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>().</p></div>
        pub unsafe fn new_1a(
            node: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_iterator_iterator1(
                    ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(
                        node,
                    )
                    .as_mut_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>The <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QHash<QString, QVariant>::iterator::iterator(const QHash<QString, QVariant>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;Key, T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> (or <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>) and to modify the value (but not the key) associated with a particular key. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, you should use <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a>. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> function like <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>(), or <a href="http://doc.qt.io/qt-5/qhash.html#find">QHash::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a hash:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
        ///   hash<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
        ///   hash<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   hash<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> hash<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Unlike <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, which orders its items by key, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> stores its items in an arbitrary order. The only guarantee is that items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">QHash::insertMulti</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a>. Here's an example that increments every value stored in the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> hash<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     i<span class="operator">.</span>value() <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Here's an example that removes all the items whose key is a string that starts with an underscore character:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> hash<span class="operator">.</span>begin();
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end()) {
        /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     i <span class="operator">=</span> hash<span class="operator">.</span>erase(i);
        /// &#32;     <span class="keyword">else</span>
        /// &#32;   &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        ///   }
        ///
        /// </pre>
        /// <p>The call to <a href="http://doc.qt.io/qt-5/qhash.html#erase">QHash::erase</a>() removes the item pointed to by the iterator from the hash, and returns an iterator to the next item. Here's another way of removing an item while iterating:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> hash<span class="operator">.</span>begin();
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end()) {
        /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator prev <span class="operator">=</span> i;
        /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        /// &#32;     <span class="keyword">if</span> (prev<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     hash<span class="operator">.</span>erase(prev);
        ///   }
        ///
        /// </pre>
        /// <p>It might be tempting to write code like this:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="comment">// WRONG</span>
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end()) {
        /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     hash<span class="operator">.</span>erase(i);
        /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        ///   }
        ///
        /// </pre>
        /// <p>However, this will potentially crash in <code>++i</code>, because <code>i</code> is a dangling iterator after the call to <a href="http://doc.qt.io/qt-5/qhash.html#erase">erase</a>().</p>
        /// <p>Multiple iterators can be used on the same hash. However, be aware that any modification performed directly on the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> has the potential of dramatically changing the order in which the items are stored in the hash, as they might cause <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> to rehash its internal data structure. There is one notable exception: <a href="http://doc.qt.io/qt-5/qhash.html#erase">QHash::erase</a>(). This function can safely be called while iterating, and won't affect the order of items in the hash. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> rather than <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn new_copy(
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::Iterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_iterator_iterator2(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::Iterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QVariant* QHash<QString, QVariant>::iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's value.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::MutPtr<crate::QVariant> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_iterator_operator__(
                    self as *const crate::q_hash_of_q_string_q_variant::Iterator,
                );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }

        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QVariant& QHash<QString, QVariant>::iterator::value() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>You can change the value of an item by using value() on the left side of an assignment, for example:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="keyword">if</span> (i<span class="operator">.</span>key() <span class="operator">=</span><span class="operator">=</span> <span class="string">"Hello"</span>)
        /// &#32;     i<span class="operator">.</span>value() <span class="operator">=</span> <span class="string">"Bonjour"</span>;
        ///
        /// </pre>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">operator*</a>().</p></div>
        pub unsafe fn value(&self) -> ::cpp_utils::MutRef<crate::QVariant> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_iterator_value(
                self as *const crate::q_hash_of_q_string_q_variant::Iterator,
            );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QHash<QString, QVariant>::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;Key, T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> (or a <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>). If you want to modify the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as you iterate over it, you must use <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> function like <a href="http://doc.qt.io/qt-5/qhash.html#constBegin">QHash::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#constEnd">QHash::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qhash.html#find">QHash::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a hash:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
    ///   hash<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
    ///   hash<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   hash<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> hash<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, which orders its items by key, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> stores its items in an arbitrary order. The only guarantee is that items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">QHash::insertMulti</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
    /// <p>Multiple iterators can be used on the same hash. However, be aware that any modification performed directly on the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> has the potential of dramatically changing the order in which the items are stored in the hash, as they might cause <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> to rehash its internal data structure. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> rather than <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::const_iterator& QHash<QString, QVariant>::const_iterator::operator=(const QHash<QString, QVariant>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;Key, T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> (or a <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>). If you want to modify the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as you iterate over it, you must use <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> function like <a href="http://doc.qt.io/qt-5/qhash.html#constBegin">QHash::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#constEnd">QHash::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qhash.html#find">QHash::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a hash:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
        ///   hash<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
        ///   hash<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   hash<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> hash<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Unlike <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, which orders its items by key, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> stores its items in an arbitrary order. The only guarantee is that items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">QHash::insertMulti</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
        /// <p>Multiple iterators can be used on the same hash. However, be aware that any modification performed directly on the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> has the potential of dramatically changing the order in which the items are stored in the hash, as they might cause <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> to rehash its internal data structure. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> rather than <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::ConstIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_hash_of_q_string_q_variant::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_const_iterator_operator_3(
                    self as *mut crate::q_hash_of_q_string_q_variant::ConstIterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::ConstIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::const_iterator QHash<QString, QVariant>::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_const_iterator_operator__6(
                    self as *mut crate::q_hash_of_q_string_q_variant::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::const_iterator QHash<QString, QVariant>::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_const_iterator_operator__4(
                    self as *mut crate::q_hash_of_q_string_q_variant::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the current key, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QString& QHash<QString, QVariant>::const_iterator::key() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#key">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current key, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        pub unsafe fn key(&self) -> ::cpp_utils::Ref<crate::QString> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_const_iterator_key(
                    self as *const crate::q_hash_of_q_string_q_variant::ConstIterator,
                );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QHash<QString, QVariant>::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html#key">key</a>(), <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html#value">value</a>(), and operator++() must not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#constBegin">QHash::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#constEnd">QHash::constEnd</a>().</p></div>
        pub unsafe fn new(
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_const_iterator_const_iterator(
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QHash<QString, QVariant>::const_iterator::const_iterator(void* node)```</span>.
        pub unsafe fn from_void(
            node: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_const_iterator_const_iterator1(
                    ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(
                        node,
                    )
                    .as_mut_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs a copy of <i>other</i>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QHash<QString, QVariant>::const_iterator::const_iterator(const QHash<QString, QVariant>::iterator& o)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html#const_iterator-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p></div>
        pub unsafe fn from_iterator(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::Iterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_const_iterator_const_iterator2(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::Iterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>The <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QHash<QString, QVariant>::const_iterator::const_iterator(const QHash<QString, QVariant>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;Key, T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> (or a <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>). If you want to modify the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as you iterate over it, you must use <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> function like <a href="http://doc.qt.io/qt-5/qhash.html#constBegin">QHash::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#constEnd">QHash::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qhash.html#find">QHash::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a hash:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
        ///   hash<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
        ///   hash<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   hash<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> hash<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Unlike <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, which orders its items by key, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> stores its items in an arbitrary order. The only guarantee is that items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">QHash::insertMulti</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
        /// <p>Multiple iterators can be used on the same hash. However, be aware that any modification performed directly on the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> has the potential of dramatically changing the order in which the items are stored in the hash, as they might cause <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> to rehash its internal data structure. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> rather than <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn new_copy(
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::ConstIterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_const_iterator_const_iterator3(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::ConstIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current result.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QVariant* QHash<QString, QVariant>::const_iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current result.</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::Ptr<crate::QVariant> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_const_iterator_operator__(
                    self as *const crate::q_hash_of_q_string_q_variant::ConstIterator,
                );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }

        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QVariant& QHash<QString, QVariant>::const_iterator::value() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        pub unsafe fn value(&self) -> ::cpp_utils::Ref<crate::QVariant> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_const_iterator_value(
                    self as *const crate::q_hash_of_q_string_q_variant::ConstIterator,
                );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a> keys.</p>
    ///
    /// C++ class: <span style='color: green;'>```QHash<QString, QVariant>::key_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a> keys.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> is essentially the same as <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> with the difference that operator*() and operator-&gt;() return a key instead of a value.</p>
    /// <p>For most uses <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> and <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> should be used, you can easily access the key by calling <a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">QHash::iterator::key</a>():</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">for</span> (<span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator it <span class="operator">=</span> hash<span class="operator">.</span>cbegin()<span class="operator">,</span> end <span class="operator">=</span> hash<span class="operator">.</span>cend(); it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The key: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Also the value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> (<span class="operator">*</span>it) <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///   }
    ///
    /// </pre>
    /// <p>However, to have interoperability between <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s keys and STL-style algorithms we need an iterator that dereferences to a key instead of a value. With <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> we can apply an algorithm to a range of keys without having to call <a href="http://doc.qt.io/qt-5/qhash.html#keys">QHash::keys</a>(), which is inefficient as it costs one <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> iteration and memory allocation to create a temporary <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="comment">// Inefficient, keys() is expensive</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> keys <span class="operator">=</span> hash<span class="operator">.</span>keys();
    ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(keys<span class="operator">.</span>cbegin()<span class="operator">,</span> keys<span class="operator">.</span>cend()<span class="operator">,</span> isPrimeNumber);
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(hash2<span class="operator">.</span>keys());
    ///
    ///   <span class="comment">// Efficient, no memory allocation needed</span>
    ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(hash<span class="operator">.</span>keyBegin()<span class="operator">,</span> hash<span class="operator">.</span>keyEnd()<span class="operator">,</span> isPrimeNumber);
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(hash2<span class="operator">.</span>keyBegin()<span class="operator">,</span> hash2<span class="operator">.</span>keyEnd());
    ///
    /// </pre>
    /// <p><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> is const, it's not possible to modify the key.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> function like <a href="http://doc.qt.io/qt-5/qhash.html#keyBegin">QHash::keyBegin</a>() or <a href="http://doc.qt.io/qt-5/qhash.html#keyEnd">QHash::keyEnd</a>().</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct KeyIterator {
        _unused: u8,
    }
    impl KeyIterator {
        /// <p>Returns the underlying <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">const_iterator</a> this <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">key_iterator</a> is based on.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::const_iterator QHash<QString, QVariant>::key_iterator::base() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#base">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the underlying <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">const_iterator</a> this <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">key_iterator</a> is based on.</p></div>
        pub unsafe fn base(
            &self,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_key_iterator_base(
                self as *const crate::q_hash_of_q_string_q_variant::KeyIterator,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>The <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a> keys.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::key_iterator& QHash<QString, QVariant>::key_iterator::operator=(const QHash<QString, QVariant>::key_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a> keys.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> is essentially the same as <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> with the difference that operator*() and operator-&gt;() return a key instead of a value.</p>
        /// <p>For most uses <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> and <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> should be used, you can easily access the key by calling <a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">QHash::iterator::key</a>():</p>
        /// <pre class="cpp">
        ///
        ///   <span class="keyword">for</span> (<span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator it <span class="operator">=</span> hash<span class="operator">.</span>cbegin()<span class="operator">,</span> end <span class="operator">=</span> hash<span class="operator">.</span>cend(); it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The key: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Also the value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> (<span class="operator">*</span>it) <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///   }
        ///
        /// </pre>
        /// <p>However, to have interoperability between <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s keys and STL-style algorithms we need an iterator that dereferences to a key instead of a value. With <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> we can apply an algorithm to a range of keys without having to call <a href="http://doc.qt.io/qt-5/qhash.html#keys">QHash::keys</a>(), which is inefficient as it costs one <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> iteration and memory allocation to create a temporary <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <pre class="cpp">
        ///
        ///   <span class="comment">// Inefficient, keys() is expensive</span>
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> keys <span class="operator">=</span> hash<span class="operator">.</span>keys();
        ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(keys<span class="operator">.</span>cbegin()<span class="operator">,</span> keys<span class="operator">.</span>cend()<span class="operator">,</span> isPrimeNumber);
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(hash2<span class="operator">.</span>keys());
        ///
        ///   <span class="comment">// Efficient, no memory allocation needed</span>
        ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(hash<span class="operator">.</span>keyBegin()<span class="operator">,</span> hash<span class="operator">.</span>keyEnd()<span class="operator">,</span> isPrimeNumber);
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(hash2<span class="operator">.</span>keyBegin()<span class="operator">,</span> hash2<span class="operator">.</span>keyEnd());
        ///
        /// </pre>
        /// <p><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> is const, it's not possible to modify the key.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> function like <a href="http://doc.qt.io/qt-5/qhash.html#keyBegin">QHash::keyBegin</a>() or <a href="http://doc.qt.io/qt-5/qhash.html#keyEnd">QHash::keyEnd</a>().</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::KeyIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_hash_of_q_string_q_variant::KeyIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_key_iterator_operator_1(
                    self as *mut crate::q_hash_of_q_string_q_variant::KeyIterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::KeyIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::key_iterator QHash<QString, QVariant>::key_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previous item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::KeyIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_key_iterator_operator__6(
                    self as *mut crate::q_hash_of_q_string_q_variant::KeyIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::key_iterator QHash<QString, QVariant>::key_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previous item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::KeyIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_key_iterator_operator__4(
                    self as *mut crate::q_hash_of_q_string_q_variant::KeyIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Default constructs an instance of key_iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QHash<QString, QVariant>::key_iterator::key_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#key_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Default constructs an instance of key_iterator.</p></div>
        pub unsafe fn new_0a(
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::KeyIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_key_iterator_key_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QHash<QString, QVariant>::key_iterator::key_iterator(QHash<QString, QVariant>::const_iterator o)```</span>.
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::ConstIterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::KeyIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_key_iterator_key_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::ConstIterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>The <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a> keys.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QHash<QString, QVariant>::key_iterator::key_iterator(const QHash<QString, QVariant>::key_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a> keys.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> is essentially the same as <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> with the difference that operator*() and operator-&gt;() return a key instead of a value.</p>
        /// <p>For most uses <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> and <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> should be used, you can easily access the key by calling <a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">QHash::iterator::key</a>():</p>
        /// <pre class="cpp">
        ///
        ///   <span class="keyword">for</span> (<span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator it <span class="operator">=</span> hash<span class="operator">.</span>cbegin()<span class="operator">,</span> end <span class="operator">=</span> hash<span class="operator">.</span>cend(); it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The key: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Also the value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> (<span class="operator">*</span>it) <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///   }
        ///
        /// </pre>
        /// <p>However, to have interoperability between <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s keys and STL-style algorithms we need an iterator that dereferences to a key instead of a value. With <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> we can apply an algorithm to a range of keys without having to call <a href="http://doc.qt.io/qt-5/qhash.html#keys">QHash::keys</a>(), which is inefficient as it costs one <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> iteration and memory allocation to create a temporary <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <pre class="cpp">
        ///
        ///   <span class="comment">// Inefficient, keys() is expensive</span>
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> keys <span class="operator">=</span> hash<span class="operator">.</span>keys();
        ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(keys<span class="operator">.</span>cbegin()<span class="operator">,</span> keys<span class="operator">.</span>cend()<span class="operator">,</span> isPrimeNumber);
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(hash2<span class="operator">.</span>keys());
        ///
        ///   <span class="comment">// Efficient, no memory allocation needed</span>
        ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(hash<span class="operator">.</span>keyBegin()<span class="operator">,</span> hash<span class="operator">.</span>keyEnd()<span class="operator">,</span> isPrimeNumber);
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(hash2<span class="operator">.</span>keyBegin()<span class="operator">,</span> hash2<span class="operator">.</span>keyEnd());
        ///
        /// </pre>
        /// <p><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> is const, it's not possible to modify the key.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> function like <a href="http://doc.qt.io/qt-5/qhash.html#keyBegin">QHash::keyBegin</a>() or <a href="http://doc.qt.io/qt-5/qhash.html#keyEnd">QHash::keyEnd</a>().</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn new_copy(
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::KeyIterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::KeyIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_key_iterator_key_iterator2(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::KeyIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's key.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QString* QHash<QString, QVariant>::key_iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's key.</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::Ptr<crate::QString> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_key_iterator_operator__(
                    self as *const crate::q_hash_of_q_string_q_variant::KeyIterator,
                );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_hash_of_q_string_q_variant::Iterator {
        type Output = ::cpp_utils::MutRef<crate::QVariant>;
        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QVariant& QHash<QString, QVariant>::iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>().</p></div>
        fn indirection(self) -> ::cpp_utils::MutRef<crate::QVariant> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_iterator_operator_(
                    self as *const crate::q_hash_of_q_string_q_variant::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::Iterator>>
        for crate::q_hash_of_q_string_q_variant::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QHash<QString, QVariant>::iterator::operator==(const QHash<QString, QVariant>::iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_iterator_operator__1(
                    self as *const crate::q_hash_of_q_string_q_variant::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_hash_of_q_string_q_variant::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_hash_of_q_string_q_variant::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::iterator& QHash<QString, QVariant>::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_hash_of_q_string_q_variant::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_iterator_operator__3(
                    self as *mut crate::q_hash_of_q_string_q_variant::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_hash_of_q_string_q_variant::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_hash_of_q_string_q_variant::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::iterator& QHash<QString, QVariant>::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_hash_of_q_string_q_variant::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_iterator_operator__5(
                    self as *mut crate::q_hash_of_q_string_q_variant::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_int> for &crate::q_hash_of_q_string_q_variant::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::Iterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions forward from this iterator. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::iterator QHash<QString, QVariant>::iterator::operator+(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions forward from this iterator. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p>This operation can be slow for large <i>j</i> values.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn add(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_iterator_operator_1(
                    self as *const crate::q_hash_of_q_string_q_variant::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_int> for &crate::q_hash_of_q_string_q_variant::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::Iterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions backward from this iterator. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::iterator QHash<QString, QVariant>::iterator::operator-(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions backward from this iterator. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p>This operation can be slow for large <i>j</i> values.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn sub(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_iterator_operator_2(
                    self as *const crate::q_hash_of_q_string_q_variant::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_int>
        for crate::q_hash_of_q_string_q_variant::Iterator
    {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::iterator& QHash<QString, QVariant>::iterator::operator+=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_iterator_operator__7(
                    self as *mut crate::q_hash_of_q_string_q_variant::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_int>
        for crate::q_hash_of_q_string_q_variant::Iterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::iterator& QHash<QString, QVariant>::iterator::operator-=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_iterator_operator__8(
                    self as *mut crate::q_hash_of_q_string_q_variant::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::ConstIterator>>
        for crate::q_hash_of_q_string_q_variant::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QHash<QString, QVariant>::iterator::operator==(const QHash<QString, QVariant>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_iterator_operator__9(
                    self as *const crate::q_hash_of_q_string_q_variant::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_hash_of_q_string_q_variant::ConstIterator {
        type Output = ::cpp_utils::Ref<crate::QVariant>;
        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QVariant& QHash<QString, QVariant>::const_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        fn indirection(self) -> ::cpp_utils::Ref<crate::QVariant> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_const_iterator_operator_(
                    self as *const crate::q_hash_of_q_string_q_variant::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::ConstIterator>>
        for crate::q_hash_of_q_string_q_variant::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QHash<QString, QVariant>::const_iterator::operator==(const QHash<QString, QVariant>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_const_iterator_operator__1(
                    self as *const crate::q_hash_of_q_string_q_variant::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_hash_of_q_string_q_variant::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_hash_of_q_string_q_variant::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::const_iterator& QHash<QString, QVariant>::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_hash_of_q_string_q_variant::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_const_iterator_operator__3(
                    self as *mut crate::q_hash_of_q_string_q_variant::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_hash_of_q_string_q_variant::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_hash_of_q_string_q_variant::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::const_iterator& QHash<QString, QVariant>::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_hash_of_q_string_q_variant::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_const_iterator_operator__5(
                    self as *mut crate::q_hash_of_q_string_q_variant::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_int>
        for &crate::q_hash_of_q_string_q_variant::ConstIterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::ConstIterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions forward from this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::const_iterator QHash<QString, QVariant>::const_iterator::operator+(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions forward from this iterator.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>().</p></div>
        fn add(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_const_iterator_operator_1(
                    self as *const crate::q_hash_of_q_string_q_variant::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_int>
        for &crate::q_hash_of_q_string_q_variant::ConstIterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::ConstIterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions backward from this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::const_iterator QHash<QString, QVariant>::const_iterator::operator-(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions backward from this iterator.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>().</p></div>
        fn sub(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_const_iterator_operator_2(
                    self as *const crate::q_hash_of_q_string_q_variant::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_int>
        for crate::q_hash_of_q_string_q_variant::ConstIterator
    {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::const_iterator& QHash<QString, QVariant>::const_iterator::operator+=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_const_iterator_operator__7(
                    self as *mut crate::q_hash_of_q_string_q_variant::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_int>
        for crate::q_hash_of_q_string_q_variant::ConstIterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::const_iterator& QHash<QString, QVariant>::const_iterator::operator-=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_const_iterator_operator__8(
                    self as *mut crate::q_hash_of_q_string_q_variant::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_hash_of_q_string_q_variant::KeyIterator {
        type Output = ::cpp_utils::Ref<crate::QString>;
        /// <p>Returns the current item's key.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QString& QHash<QString, QVariant>::key_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current item's key.</p></div>
        fn indirection(self) -> ::cpp_utils::Ref<crate::QString> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_key_iterator_operator_(
                    self as *const crate::q_hash_of_q_string_q_variant::KeyIterator,
                )
            };
            unsafe {
                ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::KeyIterator>>
        for crate::q_hash_of_q_string_q_variant::KeyIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QHash<QString, QVariant>::key_iterator::operator==(QHash<QString, QVariant>::key_iterator o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::KeyIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_key_iterator_operator__1(
                    self as *const crate::q_hash_of_q_string_q_variant::KeyIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_hash_of_q_string_q_variant::KeyIterator {
        type Output = ::cpp_utils::MutRef<crate::q_hash_of_q_string_q_variant::KeyIterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::key_iterator& QHash<QString, QVariant>::key_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#keyEnd">QHash::keyEnd</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_hash_of_q_string_q_variant::KeyIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_key_iterator_operator__3(
                    self as *mut crate::q_hash_of_q_string_q_variant::KeyIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_hash_of_q_string_q_variant::KeyIterator {
        type Output = ::cpp_utils::MutRef<crate::q_hash_of_q_string_q_variant::KeyIterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::key_iterator& QHash<QString, QVariant>::key_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#keyBegin">QHash::keyBegin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_hash_of_q_string_q_variant::KeyIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_key_iterator_operator__5(
                    self as *mut crate::q_hash_of_q_string_q_variant::KeyIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_hash_of_q_string_q_variant::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QHash<QString, QVariant>::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;Key, T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> (or <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>) and to modify the value (but not the key) associated with a particular key. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, you should use <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a>. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> function like <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>(), or <a href="http://doc.qt.io/qt-5/qhash.html#find">QHash::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a hash:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
        ///   hash<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
        ///   hash<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   hash<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> hash<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Unlike <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, which orders its items by key, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> stores its items in an arbitrary order. The only guarantee is that items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">QHash::insertMulti</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a>. Here's an example that increments every value stored in the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> hash<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     i<span class="operator">.</span>value() <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Here's an example that removes all the items whose key is a string that starts with an underscore character:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> hash<span class="operator">.</span>begin();
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end()) {
        /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     i <span class="operator">=</span> hash<span class="operator">.</span>erase(i);
        /// &#32;     <span class="keyword">else</span>
        /// &#32;   &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        ///   }
        ///
        /// </pre>
        /// <p>The call to <a href="http://doc.qt.io/qt-5/qhash.html#erase">QHash::erase</a>() removes the item pointed to by the iterator from the hash, and returns an iterator to the next item. Here's another way of removing an item while iterating:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> hash<span class="operator">.</span>begin();
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end()) {
        /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator prev <span class="operator">=</span> i;
        /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        /// &#32;     <span class="keyword">if</span> (prev<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     hash<span class="operator">.</span>erase(prev);
        ///   }
        ///
        /// </pre>
        /// <p>It might be tempting to write code like this:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="comment">// WRONG</span>
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end()) {
        /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     hash<span class="operator">.</span>erase(i);
        /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        ///   }
        ///
        /// </pre>
        /// <p>However, this will potentially crash in <code>++i</code>, because <code>i</code> is a dangling iterator after the call to <a href="http://doc.qt.io/qt-5/qhash.html#erase">erase</a>().</p>
        /// <p>Multiple iterators can be used on the same hash. However, be aware that any modification performed directly on the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> has the potential of dramatically changing the order in which the items are stored in the hash, as they might cause <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> to rehash its internal data structure. There is one notable exception: <a href="http://doc.qt.io/qt-5/qhash.html#erase">QHash::erase</a>(). This function can safely be called while iterating, and won't affect the order of items in the hash. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> rather than <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_iterator_diterator(
                self as *mut crate::q_hash_of_q_string_q_variant::Iterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_hash_of_q_string_q_variant::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QHash<QString, QVariant>::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;Key, T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> (or a <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>). If you want to modify the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as you iterate over it, you must use <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> function like <a href="http://doc.qt.io/qt-5/qhash.html#constBegin">QHash::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#constEnd">QHash::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qhash.html#find">QHash::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a hash:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
        ///   hash<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
        ///   hash<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   hash<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> hash<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Unlike <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, which orders its items by key, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> stores its items in an arbitrary order. The only guarantee is that items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">QHash::insertMulti</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
        /// <p>Multiple iterators can be used on the same hash. However, be aware that any modification performed directly on the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> has the potential of dramatically changing the order in which the items are stored in the hash, as they might cause <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> to rehash its internal data structure. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> rather than <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_const_iterator_dconst_iterator(
                self as *mut crate::q_hash_of_q_string_q_variant::ConstIterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_hash_of_q_string_q_variant::KeyIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a> keys.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QHash<QString, QVariant>::key_iterator::~key_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a> keys.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> is essentially the same as <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> with the difference that operator*() and operator-&gt;() return a key instead of a value.</p>
        /// <p>For most uses <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> and <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> should be used, you can easily access the key by calling <a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">QHash::iterator::key</a>():</p>
        /// <pre class="cpp">
        ///
        ///   <span class="keyword">for</span> (<span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator it <span class="operator">=</span> hash<span class="operator">.</span>cbegin()<span class="operator">,</span> end <span class="operator">=</span> hash<span class="operator">.</span>cend(); it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The key: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Also the value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> (<span class="operator">*</span>it) <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///   }
        ///
        /// </pre>
        /// <p>However, to have interoperability between <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s keys and STL-style algorithms we need an iterator that dereferences to a key instead of a value. With <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> we can apply an algorithm to a range of keys without having to call <a href="http://doc.qt.io/qt-5/qhash.html#keys">QHash::keys</a>(), which is inefficient as it costs one <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> iteration and memory allocation to create a temporary <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <pre class="cpp">
        ///
        ///   <span class="comment">// Inefficient, keys() is expensive</span>
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> keys <span class="operator">=</span> hash<span class="operator">.</span>keys();
        ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(keys<span class="operator">.</span>cbegin()<span class="operator">,</span> keys<span class="operator">.</span>cend()<span class="operator">,</span> isPrimeNumber);
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(hash2<span class="operator">.</span>keys());
        ///
        ///   <span class="comment">// Efficient, no memory allocation needed</span>
        ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(hash<span class="operator">.</span>keyBegin()<span class="operator">,</span> hash<span class="operator">.</span>keyEnd()<span class="operator">,</span> isPrimeNumber);
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(hash2<span class="operator">.</span>keyBegin()<span class="operator">,</span> hash2<span class="operator">.</span>keyEnd());
        ///
        /// </pre>
        /// <p><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> is const, it's not possible to modify the key.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> function like <a href="http://doc.qt.io/qt-5/qhash.html#keyBegin">QHash::keyBegin</a>() or <a href="http://doc.qt.io/qt-5/qhash.html#keyEnd">QHash::keyEnd</a>().</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_key_iterator_dkey_iterator(
                self as *mut crate::q_hash_of_q_string_q_variant::KeyIterator,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> class is a template class that provides a hash-table-based dictionary.</p>
///
/// C++ class: <span style='color: green;'>```QHash<QString, QVariant>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> class is a template class that provides a hash-table-based dictionary.</p>
/// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;Key, T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores (key, value) pairs and provides very fast lookup of the value associated with a key.</p>
/// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> provides very similar functionality to <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>. The differences are:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> provides faster lookups than <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.)</li>
/// <li>When iterating over a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, the items are always sorted by key. With <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, the items are arbitrarily ordered.</li>
/// <li>The key type of a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> must provide operator&lt;(). The key type of a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> must provide operator==() and a global hash function called <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() (see <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a>).</li>
/// </ul>
/// <p>Here's an example <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> with <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> keys and <code>int</code> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
///
/// </pre>
/// <p>To insert a (key, value) pair into the hash, you can use operator[]():</p>
/// <pre class="cpp">
///
///   hash<span class="operator">[</span><span class="string">"one"</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>;
///   hash<span class="operator">[</span><span class="string">"three"</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">3</span>;
///   hash<span class="operator">[</span><span class="string">"seven"</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">7</span>;
///
/// </pre>
/// <p>This inserts the following three (key, value) pairs into the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>: ("one", 1), ("three", 3), and ("seven", 7). Another way to insert items into the hash is to use <a href="http://doc.qt.io/qt-5/qhash.html#insert">insert</a>():</p>
/// <pre class="cpp">
///
///   hash<span class="operator">.</span>insert(<span class="string">"twelve"</span><span class="operator">,</span> <span class="number">12</span>);
///
/// </pre>
/// <p>To look up a value, use operator[]() or <a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>():</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> num1 <span class="operator">=</span> hash<span class="operator">[</span><span class="string">"thirteen"</span><span class="operator">]</span>;
///   <span class="type">int</span> num2 <span class="operator">=</span> hash<span class="operator">.</span>value(<span class="string">"thirteen"</span>);
///
/// </pre>
/// <p>If there is no item with the specified key in the hash, these functions return a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>.</p>
/// <p>If you want to check whether the hash contains a particular key, use <a href="http://doc.qt.io/qt-5/qhash.html#contains">contains</a>():</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> timeout <span class="operator">=</span> <span class="number">30</span>;
///   <span class="keyword">if</span> (hash<span class="operator">.</span>contains(<span class="string">"TIMEOUT"</span>))
/// &#32;     timeout <span class="operator">=</span> hash<span class="operator">.</span>value(<span class="string">"TIMEOUT"</span>);
///
/// </pre>
/// <p>There is also a <a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>() overload that uses its second argument as a default value if there is no item with the specified key:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> timeout <span class="operator">=</span> hash<span class="operator">.</span>value(<span class="string">"TIMEOUT"</span><span class="operator">,</span> <span class="number">30</span>);
///
/// </pre>
/// <p>In general, we recommend that you use <a href="http://doc.qt.io/qt-5/qhash.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>() rather than operator[]() for looking up a key in a hash. The reason is that operator[]() silently inserts an item into the hash if no item exists with the same key (unless the hash is const). For example, the following code snippet will create 1000 items in memory:</p>
/// <pre class="cpp">
///
///   <span class="comment">// WRONG</span>
///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> hash;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> <span class="number">1000</span>; <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (hash<span class="operator">[</span>i<span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> okButton)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found button at index "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p>To avoid this problem, replace <code>hash[i]</code> with <code>hash.value(i)</code> in the code above.</p>
/// <p>Internally, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> uses a hash table to perform lookups. Unlike Qt 3's <code>QDict</code> class, which needed to be initialized with a prime number, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s hash table automatically grows and shrinks to provide fast lookups without wasting too much memory. You can still control the size of the hash table by calling <a href="http://doc.qt.io/qt-5/qhash.html#reserve">reserve</a>() if you already know approximately how many items the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> will contain, but this isn't necessary to obtain good performance. You can also call <a href="http://doc.qt.io/qt-5/qhash.html#capacity">capacity</a>() to retrieve the hash table's size.</p>
/// <p>If you want to navigate through all the (key, value) pairs stored in a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, you can use an iterator. <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qhashiterator.html">QHashIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablehashiterator.html">QMutableHashIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a>). Here's how to iterate over a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, int&gt; using a Java-style iterator:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qhashiterator.html">QHashIterator</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> i(hash);
///   <span class="keyword">while</span> (i<span class="operator">.</span>hasNext()) {
/// &#32;     i<span class="operator">.</span>next();
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p>Here's the same code, but using an STL-style iterator:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i <span class="operator">=</span> hash<span class="operator">.</span>constBegin();
///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>constEnd()) {
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
/// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> is unordered, so an iterator's sequence cannot be assumed to be predictable. If ordering by key is required, use a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>.</p>
/// <p>Normally, a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> allows only one value per key. If you call <a href="http://doc.qt.io/qt-5/qhash.html#insert">insert</a>() with a key that already exists in the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, the previous value is erased. For example:</p>
/// <pre class="cpp">
///
///   hash<span class="operator">.</span>insert(<span class="string">"plenty"</span><span class="operator">,</span> <span class="number">100</span>);
///   hash<span class="operator">.</span>insert(<span class="string">"plenty"</span><span class="operator">,</span> <span class="number">2000</span>);
///   <span class="comment">// hash.value("plenty") == 2000</span>
///
/// </pre>
/// <p>However, you can store multiple values per key by using <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">insertMulti</a>() instead of <a href="http://doc.qt.io/qt-5/qhash.html#insert">insert</a>() (or using the convenience subclass <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>). If you want to retrieve all the values for a single key, you can use values(const Key &amp;key), which returns a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> values <span class="operator">=</span> hash<span class="operator">.</span>values(<span class="string">"plenty"</span>);
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> values<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> values<span class="operator">.</span>at(i) <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>The items that share the same key are available from most recently to least recently inserted. A more efficient approach is to call <a href="http://doc.qt.io/qt-5/qhash.html#find">find</a>() to get the iterator for the first item with a key and iterate from there:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> hash<span class="operator">.</span>find(<span class="string">"plenty"</span>);
///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end() <span class="operator">&amp;</span><span class="operator">&amp;</span> i<span class="operator">.</span>key() <span class="operator">=</span><span class="operator">=</span> <span class="string">"plenty"</span>) {
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
/// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
///   }
///
/// </pre>
/// <p>If you only need to extract the values from a hash (not the keys), you can also use <a href="http://doc.qt.io/qt-5/containers.html#foreach">foreach</a>:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   foreach (<span class="type">int</span> value<span class="operator">,</span> hash)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> value <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>Items can be removed from the hash in several ways. One way is to call <a href="http://doc.qt.io/qt-5/qhash.html#remove">remove</a>(); this will remove any item with the given key. Another way is to use <a href="http://doc.qt.io/qt-5/qmutablehashiterator.html#remove">QMutableHashIterator::remove</a>(). In addition, you can clear the entire hash using <a href="http://doc.qt.io/qt-5/qhash.html#clear">clear</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s key and value data types must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *.</p>
/// <a name="qhash"></a><a name="the-qhash-hashing-function"></a>
/// <h4>The qHash() hashing function</h4>
/// <p>A <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s key type has additional requirements other than being an assignable data type: it must provide operator==(), and there must also be a <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function in the type's namespace that returns a hash value for an argument of the key's type.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function computes a numeric value based on a key. It can use any algorithm imaginable, as long as it always returns the same value if given the same argument. In other words, if <code>e1 == e2</code>, then <code>qHash(e1) == qHash(e2)</code> must hold as well. However, to obtain good performance, the <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function should attempt to return different hash values for different keys to the largest extent possible.</p>
/// <p>For a key type <code>K</code>, the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a> function must have one of these signatures:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(K key);
///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(<span class="keyword">const</span> K <span class="operator">&amp;</span>key);
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(K key<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> seed);
///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(<span class="keyword">const</span> K <span class="operator">&amp;</span>key<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> seed);
///
/// </pre>
/// <p>The two-arguments overloads take an unsigned integer that should be used to seed the calculation of the hash function. This seed is provided by <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> in order to prevent a family of <a href="http://doc.qt.io/qt-5/qhash.html#algorithmic-complexity-attacks">algorithmic complexity attacks</a>. If both a one-argument and a two-arguments overload are defined for a key type, the latter is used by <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> (note that you can simply define a two-arguments version, and use a default value for the seed parameter).</p>
/// <p>Here's a partial list of the C++ and Qt types that can serve as keys in a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>: any integer type (char, unsigned long, etc.), any pointer type, <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>, <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, and <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>. For all of these, the <code>&lt;QHash&gt;</code> header defines a <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function that computes an adequate hash value. Many other Qt classes also declare a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a> overload for their type; please refer to the documentation of each class.</p>
/// <p>If you want to use other types as the key, make sure that you provide operator==() and a <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() implementation.</p>
/// <p>Example:</p>
/// <pre class="cpp">
///
///   <span class="preprocessor">#ifndef EMPLOYEE_H</span>
///   <span class="preprocessor">#define EMPLOYEE_H</span>
///
///   <span class="keyword">class</span> Employee
///   {
///   <span class="keyword">public</span>:
/// &#32;     Employee() {}
/// &#32;     Employee(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>name<span class="operator">,</span> <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span> <span class="operator">&amp;</span>dateOfBirth);
/// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///
///   <span class="keyword">private</span>:
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> myName;
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span> myDateOfBirth;
///   };
///
///   <span class="keyword">inline</span> bool <span class="keyword">operator</span><span class="operator">=</span><span class="operator">=</span>(<span class="keyword">const</span> Employee <span class="operator">&amp;</span>e1<span class="operator">,</span> <span class="keyword">const</span> Employee <span class="operator">&amp;</span>e2)
///   {
/// &#32;     <span class="keyword">return</span> e1<span class="operator">.</span>name() <span class="operator">=</span><span class="operator">=</span> e2<span class="operator">.</span>name()
/// &#32;   &#32;   &#32;    <span class="operator">&amp;</span><span class="operator">&amp;</span> e1<span class="operator">.</span>dateOfBirth() <span class="operator">=</span><span class="operator">=</span> e2<span class="operator">.</span>dateOfBirth();
///   }
///
///   <span class="keyword">inline</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(<span class="keyword">const</span> Employee <span class="operator">&amp;</span>key<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> seed)
///   {
/// &#32;     <span class="keyword">return</span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(key<span class="operator">.</span>name()<span class="operator">,</span> seed) <span class="operator">^</span> key<span class="operator">.</span>dateOfBirth()<span class="operator">.</span>day();
///   }
///
///   <span class="preprocessor">#endif // EMPLOYEE_H</span>
///
/// </pre>
/// <p>In the example above, we've relied on Qt's global <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a>(const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;, uint) to give us a hash value for the employee's name, and XOR'ed this with the day they were born to help produce unique hashes for people with the same name.</p>
/// <p>Note that the implementation of the <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() overloads offered by Qt may change at any time. You <b>must not</b> rely on the fact that <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() will give the same results (for the same inputs) across different Qt versions.</p>
/// <a name="algorithmic-complexity-attacks"></a>
/// <h4>Algorithmic complexity attacks</h4>
/// <p>All hash tables are vulnerable to a particular class of denial of service attacks, in which the attacker carefully pre-computes a set of different keys that are going to be hashed in the same bucket of a hash table (or even have the very same hash value). The attack aims at getting the worst-case algorithmic behavior (O(n) instead of amortized O(1), see <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for the details) when the data is fed into the table.</p>
/// <p>In order to avoid this worst-case behavior, the calculation of the hash value done by <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() can be salted by a random seed, that nullifies the attack's extent. This seed is automatically generated by <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> once per process, and then passed by <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as the second argument of the two-arguments overload of the <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function.</p>
/// <p>This randomization of <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> is enabled by default. Even though programs should never depend on a particular <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> ordering, there may be situations where you temporarily need deterministic behavior, for example for debugging or regression testing. To disable the randomization, define the environment variable <code>QT_HASH_SEED</code>. The contents of that variable, interpreted as a decimal value, will be used as the seed for <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(). Alternatively, you can call the <a href="http://doc.qt.io/qt-5/qhash.html#qSetGlobalQHashSeed">qSetGlobalQHashSeed</a>() function.</p></div>
#[repr(C)]
pub struct QHashOfQStringQVariant {
    _unused: u8,
}
impl QHashOfQStringQVariant {
    /// <p>Returns the number of buckets in the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s internal hash table.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QHash<QString, QVariant>::capacity() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#capacity">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of buckets in the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s internal hash table.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s memory usage. In general, you will rarely ever need to call this function. If you want to know how many items are in the hash, call <a href="http://doc.qt.io/qt-5/qhash.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#squeeze">squeeze</a>().</p></div>
    pub unsafe fn capacity(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_capacity(
            self as *const crate::QHashOfQStringQVariant,
        )
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::const_iterator QHash<QString, QVariant>::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the hash.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_cbegin(
            self as *const crate::QHashOfQStringQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::const_iterator QHash<QString, QVariant>::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the hash.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#end">end</a>().</p></div>
    pub unsafe fn cend(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_cend(
            self as *const crate::QHashOfQStringQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all items from the hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QHash<QString, QVariant>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all items from the hash.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#remove">remove</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_clear(
            self as *mut crate::QHashOfQStringQVariant,
        )
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::const_iterator QHash<QString, QVariant>::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the hash.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_constBegin(
            self as *const crate::QHashOfQStringQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::const_iterator QHash<QString, QVariant>::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the hash.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#end">end</a>().</p></div>
    pub unsafe fn const_end(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_constEnd(
            self as *const crate::QHashOfQStringQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns an iterator pointing to the item with the <i>key</i> in the hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::const_iterator QHash<QString, QVariant>::constFind(const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#constFind">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator pointing to the item with the <i>key</i> in the hash.</p>
    /// <p>If the hash contains no item with the <i>key</i>, the function returns <a href="http://doc.qt.io/qt-5/qhash.html#constEnd">constEnd</a>().</p>
    /// <p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#find">find</a>() and <a href="http://doc.qt.io/qt-5/qmultihash.html#constFind">QMultiHash::constFind</a>().</p></div>
    pub unsafe fn const_find(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_constFind(
            self as *const crate::QHashOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if the hash contains an item with the <i>key</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QHash<QString, QVariant>::contains(const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the hash contains an item with the <i>key</i>; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#count-1">count</a>() and <a href="http://doc.qt.io/qt-5/qmultihash.html#contains">QMultiHash::contains</a>().</p></div>
    pub unsafe fn contains(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_contains(
            self as *const crate::QHashOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        )
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> class is a template class that provides a hash-table-based dictionary.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>& QHash<QString, QVariant>::operator=(const QHash<QString, QVariant>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> class is a template class that provides a hash-table-based dictionary.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;Key, T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores (key, value) pairs and provides very fast lookup of the value associated with a key.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> provides very similar functionality to <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>. The differences are:</p>
    /// <ul>
    /// <li><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> provides faster lookups than <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.)</li>
    /// <li>When iterating over a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, the items are always sorted by key. With <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, the items are arbitrarily ordered.</li>
    /// <li>The key type of a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> must provide operator&lt;(). The key type of a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> must provide operator==() and a global hash function called <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() (see <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a>).</li>
    /// </ul>
    /// <p>Here's an example <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> with <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> keys and <code>int</code> values:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
    ///
    /// </pre>
    /// <p>To insert a (key, value) pair into the hash, you can use operator[]():</p>
    /// <pre class="cpp">
    ///
    ///   hash<span class="operator">[</span><span class="string">"one"</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>;
    ///   hash<span class="operator">[</span><span class="string">"three"</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">3</span>;
    ///   hash<span class="operator">[</span><span class="string">"seven"</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">7</span>;
    ///
    /// </pre>
    /// <p>This inserts the following three (key, value) pairs into the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>: ("one", 1), ("three", 3), and ("seven", 7). Another way to insert items into the hash is to use <a href="http://doc.qt.io/qt-5/qhash.html#insert">insert</a>():</p>
    /// <pre class="cpp">
    ///
    ///   hash<span class="operator">.</span>insert(<span class="string">"twelve"</span><span class="operator">,</span> <span class="number">12</span>);
    ///
    /// </pre>
    /// <p>To look up a value, use operator[]() or <a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>():</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">int</span> num1 <span class="operator">=</span> hash<span class="operator">[</span><span class="string">"thirteen"</span><span class="operator">]</span>;
    ///   <span class="type">int</span> num2 <span class="operator">=</span> hash<span class="operator">.</span>value(<span class="string">"thirteen"</span>);
    ///
    /// </pre>
    /// <p>If there is no item with the specified key in the hash, these functions return a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>.</p>
    /// <p>If you want to check whether the hash contains a particular key, use <a href="http://doc.qt.io/qt-5/qhash.html#contains">contains</a>():</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">int</span> timeout <span class="operator">=</span> <span class="number">30</span>;
    ///   <span class="keyword">if</span> (hash<span class="operator">.</span>contains(<span class="string">"TIMEOUT"</span>))
    /// &#32;     timeout <span class="operator">=</span> hash<span class="operator">.</span>value(<span class="string">"TIMEOUT"</span>);
    ///
    /// </pre>
    /// <p>There is also a <a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>() overload that uses its second argument as a default value if there is no item with the specified key:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">int</span> timeout <span class="operator">=</span> hash<span class="operator">.</span>value(<span class="string">"TIMEOUT"</span><span class="operator">,</span> <span class="number">30</span>);
    ///
    /// </pre>
    /// <p>In general, we recommend that you use <a href="http://doc.qt.io/qt-5/qhash.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>() rather than operator[]() for looking up a key in a hash. The reason is that operator[]() silently inserts an item into the hash if no item exists with the same key (unless the hash is const). For example, the following code snippet will create 1000 items in memory:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="comment">// WRONG</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> hash;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> <span class="number">1000</span>; <span class="operator">+</span><span class="operator">+</span>i) {
    /// &#32;     <span class="keyword">if</span> (hash<span class="operator">[</span>i<span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> okButton)
    /// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found button at index "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///   }
    ///
    /// </pre>
    /// <p>To avoid this problem, replace <code>hash[i]</code> with <code>hash.value(i)</code> in the code above.</p>
    /// <p>Internally, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> uses a hash table to perform lookups. Unlike Qt 3's <code>QDict</code> class, which needed to be initialized with a prime number, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s hash table automatically grows and shrinks to provide fast lookups without wasting too much memory. You can still control the size of the hash table by calling <a href="http://doc.qt.io/qt-5/qhash.html#reserve">reserve</a>() if you already know approximately how many items the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> will contain, but this isn't necessary to obtain good performance. You can also call <a href="http://doc.qt.io/qt-5/qhash.html#capacity">capacity</a>() to retrieve the hash table's size.</p>
    /// <p>If you want to navigate through all the (key, value) pairs stored in a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, you can use an iterator. <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qhashiterator.html">QHashIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablehashiterator.html">QMutableHashIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a>). Here's how to iterate over a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, int&gt; using a Java-style iterator:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhashiterator.html">QHashIterator</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> i(hash);
    ///   <span class="keyword">while</span> (i<span class="operator">.</span>hasNext()) {
    /// &#32;     i<span class="operator">.</span>next();
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///   }
    ///
    /// </pre>
    /// <p>Here's the same code, but using an STL-style iterator:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i <span class="operator">=</span> hash<span class="operator">.</span>constBegin();
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>constEnd()) {
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    ///   }
    ///
    /// </pre>
    /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> is unordered, so an iterator's sequence cannot be assumed to be predictable. If ordering by key is required, use a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>.</p>
    /// <p>Normally, a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> allows only one value per key. If you call <a href="http://doc.qt.io/qt-5/qhash.html#insert">insert</a>() with a key that already exists in the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, the previous value is erased. For example:</p>
    /// <pre class="cpp">
    ///
    ///   hash<span class="operator">.</span>insert(<span class="string">"plenty"</span><span class="operator">,</span> <span class="number">100</span>);
    ///   hash<span class="operator">.</span>insert(<span class="string">"plenty"</span><span class="operator">,</span> <span class="number">2000</span>);
    ///   <span class="comment">// hash.value("plenty") == 2000</span>
    ///
    /// </pre>
    /// <p>However, you can store multiple values per key by using <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">insertMulti</a>() instead of <a href="http://doc.qt.io/qt-5/qhash.html#insert">insert</a>() (or using the convenience subclass <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>). If you want to retrieve all the values for a single key, you can use values(const Key &amp;key), which returns a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> values <span class="operator">=</span> hash<span class="operator">.</span>values(<span class="string">"plenty"</span>);
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> values<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> values<span class="operator">.</span>at(i) <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>The items that share the same key are available from most recently to least recently inserted. A more efficient approach is to call <a href="http://doc.qt.io/qt-5/qhash.html#find">find</a>() to get the iterator for the first item with a key and iterate from there:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> hash<span class="operator">.</span>find(<span class="string">"plenty"</span>);
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end() <span class="operator">&amp;</span><span class="operator">&amp;</span> i<span class="operator">.</span>key() <span class="operator">=</span><span class="operator">=</span> <span class="string">"plenty"</span>) {
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    ///   }
    ///
    /// </pre>
    /// <p>If you only need to extract the values from a hash (not the keys), you can also use <a href="http://doc.qt.io/qt-5/containers.html#foreach">foreach</a>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   foreach (<span class="type">int</span> value<span class="operator">,</span> hash)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> value <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Items can be removed from the hash in several ways. One way is to call <a href="http://doc.qt.io/qt-5/qhash.html#remove">remove</a>(); this will remove any item with the given key. Another way is to use <a href="http://doc.qt.io/qt-5/qmutablehashiterator.html#remove">QMutableHashIterator::remove</a>(). In addition, you can clear the entire hash using <a href="http://doc.qt.io/qt-5/qhash.html#clear">clear</a>().</p>
    /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s key and value data types must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *.</p>
    /// <a name="qhash"></a><a name="the-qhash-hashing-function"></a>
    /// <h4>The qHash() hashing function</h4>
    /// <p>A <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s key type has additional requirements other than being an assignable data type: it must provide operator==(), and there must also be a <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function in the type's namespace that returns a hash value for an argument of the key's type.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function computes a numeric value based on a key. It can use any algorithm imaginable, as long as it always returns the same value if given the same argument. In other words, if <code>e1 == e2</code>, then <code>qHash(e1) == qHash(e2)</code> must hold as well. However, to obtain good performance, the <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function should attempt to return different hash values for different keys to the largest extent possible.</p>
    /// <p>For a key type <code>K</code>, the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a> function must have one of these signatures:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(K key);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(<span class="keyword">const</span> K <span class="operator">&amp;</span>key);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(K key<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> seed);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(<span class="keyword">const</span> K <span class="operator">&amp;</span>key<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> seed);
    ///
    /// </pre>
    /// <p>The two-arguments overloads take an unsigned integer that should be used to seed the calculation of the hash function. This seed is provided by <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> in order to prevent a family of <a href="http://doc.qt.io/qt-5/qhash.html#algorithmic-complexity-attacks">algorithmic complexity attacks</a>. If both a one-argument and a two-arguments overload are defined for a key type, the latter is used by <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> (note that you can simply define a two-arguments version, and use a default value for the seed parameter).</p>
    /// <p>Here's a partial list of the C++ and Qt types that can serve as keys in a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>: any integer type (char, unsigned long, etc.), any pointer type, <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>, <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, and <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>. For all of these, the <code>&lt;QHash&gt;</code> header defines a <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function that computes an adequate hash value. Many other Qt classes also declare a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a> overload for their type; please refer to the documentation of each class.</p>
    /// <p>If you want to use other types as the key, make sure that you provide operator==() and a <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() implementation.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="preprocessor">#ifndef EMPLOYEE_H</span>
    ///   <span class="preprocessor">#define EMPLOYEE_H</span>
    ///
    ///   <span class="keyword">class</span> Employee
    ///   {
    ///   <span class="keyword">public</span>:
    /// &#32;     Employee() {}
    /// &#32;     Employee(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>name<span class="operator">,</span> <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span> <span class="operator">&amp;</span>dateOfBirth);
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///
    ///   <span class="keyword">private</span>:
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> myName;
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span> myDateOfBirth;
    ///   };
    ///
    ///   <span class="keyword">inline</span> bool <span class="keyword">operator</span><span class="operator">=</span><span class="operator">=</span>(<span class="keyword">const</span> Employee <span class="operator">&amp;</span>e1<span class="operator">,</span> <span class="keyword">const</span> Employee <span class="operator">&amp;</span>e2)
    ///   {
    /// &#32;     <span class="keyword">return</span> e1<span class="operator">.</span>name() <span class="operator">=</span><span class="operator">=</span> e2<span class="operator">.</span>name()
    /// &#32;   &#32;   &#32;    <span class="operator">&amp;</span><span class="operator">&amp;</span> e1<span class="operator">.</span>dateOfBirth() <span class="operator">=</span><span class="operator">=</span> e2<span class="operator">.</span>dateOfBirth();
    ///   }
    ///
    ///   <span class="keyword">inline</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(<span class="keyword">const</span> Employee <span class="operator">&amp;</span>key<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> seed)
    ///   {
    /// &#32;     <span class="keyword">return</span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(key<span class="operator">.</span>name()<span class="operator">,</span> seed) <span class="operator">^</span> key<span class="operator">.</span>dateOfBirth()<span class="operator">.</span>day();
    ///   }
    ///
    ///   <span class="preprocessor">#endif // EMPLOYEE_H</span>
    ///
    /// </pre>
    /// <p>In the example above, we've relied on Qt's global <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a>(const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;, uint) to give us a hash value for the employee's name, and XOR'ed this with the day they were born to help produce unique hashes for people with the same name.</p>
    /// <p>Note that the implementation of the <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() overloads offered by Qt may change at any time. You <b>must not</b> rely on the fact that <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() will give the same results (for the same inputs) across different Qt versions.</p>
    /// <a name="algorithmic-complexity-attacks"></a>
    /// <h4>Algorithmic complexity attacks</h4>
    /// <p>All hash tables are vulnerable to a particular class of denial of service attacks, in which the attacker carefully pre-computes a set of different keys that are going to be hashed in the same bucket of a hash table (or even have the very same hash value). The attack aims at getting the worst-case algorithmic behavior (O(n) instead of amortized O(1), see <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for the details) when the data is fed into the table.</p>
    /// <p>In order to avoid this worst-case behavior, the calculation of the hash value done by <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() can be salted by a random seed, that nullifies the attack's extent. This seed is automatically generated by <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> once per process, and then passed by <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as the second argument of the two-arguments overload of the <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function.</p>
    /// <p>This randomization of <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> is enabled by default. Even though programs should never depend on a particular <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> ordering, there may be situations where you temporarily need deterministic behavior, for example for debugging or regression testing. To disable the randomization, define the environment variable <code>QT_HASH_SEED</code>. The contents of that variable, interpreted as a decimal value, will be used as the seed for <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(). Alternatively, you can call the <a href="http://doc.qt.io/qt-5/qhash.html#qSetGlobalQHashSeed">qSetGlobalQHashSeed</a>() function.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QHashOfQStringQVariant>>,
    ) -> ::cpp_utils::MutRef<crate::QHashOfQStringQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_operator_(
            self as *mut crate::QHashOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QHashOfQStringQVariant>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of items associated with the <i>key</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QHash<QString, QVariant>::count(const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items associated with the <i>key</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">insertMulti</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_count(
            self as *const crate::QHashOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QHash<QString, QVariant>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_count1(
            self as *const crate::QHashOfQStringQVariant,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QHash<QString, QVariant>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_detach(
            self as *mut crate::QHashOfQStringQVariant,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qhash.html#isEmpty">isEmpty</a>(), returning true if the hash is empty; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QHash<QString, QVariant>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qhash.html#isEmpty">isEmpty</a>(), returning true if the hash is empty; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_empty(
            self as *const crate::QHashOfQStringQVariant,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::iterator QHash<QString, QVariant>::erase(QHash<QString, QVariant>::iterator it)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#erase-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn erase_iterator(
        &mut self,
        it: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::Iterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_erase(
                self as *mut crate::QHashOfQStringQVariant,
                ::cpp_utils::CastInto::<
                    ::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::Iterator>,
                >::cast_into(it)
                .as_raw_ptr(),
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the (key, value) pair associated with the iterator <i>pos</i> from the hash, and returns an iterator to the next item in the hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::iterator QHash<QString, QVariant>::erase(QHash<QString, QVariant>::const_iterator it)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the (key, value) pair associated with the iterator <i>pos</i> from the hash, and returns an iterator to the next item in the hash.</p>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qhash.html#remove">remove</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#take">take</a>(), this function never causes <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> to rehash its internal data structure. This means that it can safely be called while iterating, and won't affect the order of items in the hash. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qobject.html">QObject</a></span> <span class="operator">*</span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> objectHash;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qobject.html">QObject</a></span> <span class="operator">*</span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> objectHash<span class="operator">.</span>find(obj);
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> objectHash<span class="operator">.</span>end() <span class="operator">&amp;</span><span class="operator">&amp;</span> i<span class="operator">.</span>key() <span class="operator">=</span><span class="operator">=</span> obj) {
    /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>value() <span class="operator">=</span><span class="operator">=</span> <span class="number">0</span>) {
    /// &#32;   &#32;     i <span class="operator">=</span> objectHash<span class="operator">.</span>erase(i);
    /// &#32;     } <span class="keyword">else</span> {
    /// &#32;   &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    /// &#32;     }
    ///   }
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#take">take</a>(), and <a href="http://doc.qt.io/qt-5/qhash.html#find">find</a>().</p></div>
    pub unsafe fn erase_const_iterator(
        &mut self,
        it: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::ConstIterator>,
        >,
    ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_erase1(
            self as *mut crate::QHashOfQStringQVariant,
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::q_hash_of_q_string_q_variant::ConstIterator>,
            >::cast_into(it)
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns an iterator pointing to the item with the <i>key</i> in the hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::iterator QHash<QString, QVariant>::find(const QString& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#find">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator pointing to the item with the <i>key</i> in the hash.</p>
    /// <p>If the hash contains no item with the <i>key</i>, the function returns <a href="http://doc.qt.io/qt-5/qhash.html#end">end</a>().</p>
    /// <p>If the hash contains multiple items with the <i>key</i>, this function returns an iterator that points to the most recently inserted value. The other values are accessible by incrementing the iterator. For example, here's some code that iterates over all the items with the same key:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i <span class="operator">=</span> hash<span class="operator">.</span>find(<span class="string">"HDR"</span>);
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end() <span class="operator">&amp;</span><span class="operator">&amp;</span> i<span class="operator">.</span>key() <span class="operator">=</span><span class="operator">=</span> <span class="string">"HDR"</span>) {
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    ///   }
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#values">values</a>(), and <a href="http://doc.qt.io/qt-5/qmultihash.html#find">QMultiHash::find</a>().</p></div>
    pub unsafe fn find_mut(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_find(
            self as *mut crate::QHashOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::const_iterator QHash<QString, QVariant>::find(const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#find-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn find(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_find1(
            self as *const crate::QHashOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value associated with the <i>key</i> as a modifiable reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant& QHash<QString, QVariant>::operator[](const QString& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value associated with the <i>key</i> as a modifiable reference.</p>
    /// <p>If the hash contains no item with the <i>key</i>, the function inserts a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a> into the hash with the <i>key</i>, and returns a reference to it. If the hash contains multiple items with the <i>key</i>, this function returns a reference to the most recently inserted value.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::MutRef<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_operator__(
            self as *mut crate::QHashOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QHash<QString, QVariant>::operator[](const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>().</p></div>
    pub unsafe fn index(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_operator__1(
            self as *const crate::QHashOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Inserts a new item with the <i>key</i> and a value of <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::iterator QHash<QString, QVariant>::insert(const QString& key, const QVariant& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts a new item with the <i>key</i> and a value of <i>value</i>.</p>
    /// <p>If there is already an item with the <i>key</i>, that item's value is replaced with <i>value</i>.</p>
    /// <p>If there are multiple items with the <i>key</i>, the most recently inserted item's value is replaced with <i>value</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">insertMulti</a>().</p></div>
    pub unsafe fn insert(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_insert(
            self as *mut crate::QHashOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Inserts a new item with the <i>key</i> and a value of <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::iterator QHash<QString, QVariant>::insertMulti(const QString& key, const QVariant& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts a new item with the <i>key</i> and a value of <i>value</i>.</p>
    /// <p>If there is already an item with the same key in the hash, this function will simply create a new one. (This behavior is different from <a href="http://doc.qt.io/qt-5/qhash.html#insert">insert</a>(), which overwrites the value of an existing item.)</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#values">values</a>().</p></div>
    pub unsafe fn insert_multi(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_insertMulti(
            self as *mut crate::QHashOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QHash<QString, QVariant>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_isDetached(
            self as *const crate::QHashOfQStringQVariant,
        )
    }

    /// <p>Returns <code>true</code> if the hash contains no items; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QHash<QString, QVariant>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the hash contains no items; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_isEmpty(
            self as *const crate::QHashOfQStringQVariant,
        )
    }

    /// <p>Returns the first key mapped to <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QHash<QString, QVariant>::key(const QVariant& value) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#key">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the first key mapped to <i>value</i>.</p>
    /// <p>If the hash contains no item with the <i>value</i>, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed key</a>.</p>
    /// <p>This function can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s internal data structure is optimized for fast lookup by key, not by value.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#keys">keys</a>().</p></div>
    pub unsafe fn key_1a(
        &self,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_key(
            self as *const crate::QHashOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QHash<QString, QVariant>::key(const QVariant& value, const QString& defaultKey) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#key-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the first key mapped to <i>value</i>, or <i>defaultKey</i> if the hash contains no item mapped to <i>value</i>.</p>
    /// <p>This function can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s internal data structure is optimized for fast lookup by key, not by value.</p>
    /// <p>This function was introduced in  Qt 4.3.</p></div>
    pub unsafe fn key_2a(
        &self,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
        default_key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_key1(
            self as *const crate::QHashOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(default_key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first key in the hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::key_iterator QHash<QString, QVariant>::keyBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#keyBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first key in the hash.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#keyEnd">keyEnd</a>().</p></div>
    pub unsafe fn key_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::KeyIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_keyBegin(
            self as *const crate::QHashOfQStringQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last key in the hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::key_iterator QHash<QString, QVariant>::keyEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#keyEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last key in the hash.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#keyBegin">keyBegin</a>().</p></div>
    pub unsafe fn key_end(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::KeyIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_keyEnd(
            self as *const crate::QHashOfQStringQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list containing all the keys in the hash, in an arbitrary order. Keys that occur multiple times in the hash (because items were inserted with <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">insertMulti</a>(), or <a href="http://doc.qt.io/qt-5/qhash.html#unite">unite</a>() was used) also occur multiple times in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QString> QHash<QString, QVariant>::keys() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#keys">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list containing all the keys in the hash, in an arbitrary order. Keys that occur multiple times in the hash (because items were inserted with <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">insertMulti</a>(), or <a href="http://doc.qt.io/qt-5/qhash.html#unite">unite</a>() was used) also occur multiple times in the list.</p>
    /// <p>To obtain a list of unique keys, where each key from the map only occurs once, use <a href="http://doc.qt.io/qt-5/qhash.html#uniqueKeys">uniqueKeys</a>().</p>
    /// <p>The order is guaranteed to be the same as that used by <a href="http://doc.qt.io/qt-5/qhash.html#values">values</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#uniqueKeys">uniqueKeys</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#values">values</a>(), and <a href="http://doc.qt.io/qt-5/qhash.html#key">key</a>().</p></div>
    pub unsafe fn keys_0a(&self) -> ::cpp_utils::CppBox<crate::QListOfQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_keys(
            self as *const crate::QHashOfQStringQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QString> QHash<QString, QVariant>::keys(const QVariant& value) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#keys-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a list containing all the keys associated with value <i>value</i>, in an arbitrary order.</p>
    /// <p>This function can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s internal data structure is optimized for fast lookup by key, not by value.</p></div>
    pub unsafe fn keys_1a(
        &self,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_keys1(
            self as *const crate::QHashOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs an empty hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QHash<QString, QVariant>::QHash()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#QHash">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty hash.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#clear">clear</a>().</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QHashOfQStringQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_QHash();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> class is a template class that provides a hash-table-based dictionary.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QHash<QString, QVariant>::QHash(const QHash<QString, QVariant>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> class is a template class that provides a hash-table-based dictionary.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;Key, T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores (key, value) pairs and provides very fast lookup of the value associated with a key.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> provides very similar functionality to <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>. The differences are:</p>
    /// <ul>
    /// <li><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> provides faster lookups than <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.)</li>
    /// <li>When iterating over a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, the items are always sorted by key. With <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, the items are arbitrarily ordered.</li>
    /// <li>The key type of a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> must provide operator&lt;(). The key type of a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> must provide operator==() and a global hash function called <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() (see <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a>).</li>
    /// </ul>
    /// <p>Here's an example <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> with <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> keys and <code>int</code> values:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
    ///
    /// </pre>
    /// <p>To insert a (key, value) pair into the hash, you can use operator[]():</p>
    /// <pre class="cpp">
    ///
    ///   hash<span class="operator">[</span><span class="string">"one"</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>;
    ///   hash<span class="operator">[</span><span class="string">"three"</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">3</span>;
    ///   hash<span class="operator">[</span><span class="string">"seven"</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">7</span>;
    ///
    /// </pre>
    /// <p>This inserts the following three (key, value) pairs into the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>: ("one", 1), ("three", 3), and ("seven", 7). Another way to insert items into the hash is to use <a href="http://doc.qt.io/qt-5/qhash.html#insert">insert</a>():</p>
    /// <pre class="cpp">
    ///
    ///   hash<span class="operator">.</span>insert(<span class="string">"twelve"</span><span class="operator">,</span> <span class="number">12</span>);
    ///
    /// </pre>
    /// <p>To look up a value, use operator[]() or <a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>():</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">int</span> num1 <span class="operator">=</span> hash<span class="operator">[</span><span class="string">"thirteen"</span><span class="operator">]</span>;
    ///   <span class="type">int</span> num2 <span class="operator">=</span> hash<span class="operator">.</span>value(<span class="string">"thirteen"</span>);
    ///
    /// </pre>
    /// <p>If there is no item with the specified key in the hash, these functions return a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>.</p>
    /// <p>If you want to check whether the hash contains a particular key, use <a href="http://doc.qt.io/qt-5/qhash.html#contains">contains</a>():</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">int</span> timeout <span class="operator">=</span> <span class="number">30</span>;
    ///   <span class="keyword">if</span> (hash<span class="operator">.</span>contains(<span class="string">"TIMEOUT"</span>))
    /// &#32;     timeout <span class="operator">=</span> hash<span class="operator">.</span>value(<span class="string">"TIMEOUT"</span>);
    ///
    /// </pre>
    /// <p>There is also a <a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>() overload that uses its second argument as a default value if there is no item with the specified key:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">int</span> timeout <span class="operator">=</span> hash<span class="operator">.</span>value(<span class="string">"TIMEOUT"</span><span class="operator">,</span> <span class="number">30</span>);
    ///
    /// </pre>
    /// <p>In general, we recommend that you use <a href="http://doc.qt.io/qt-5/qhash.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>() rather than operator[]() for looking up a key in a hash. The reason is that operator[]() silently inserts an item into the hash if no item exists with the same key (unless the hash is const). For example, the following code snippet will create 1000 items in memory:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="comment">// WRONG</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> hash;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> <span class="number">1000</span>; <span class="operator">+</span><span class="operator">+</span>i) {
    /// &#32;     <span class="keyword">if</span> (hash<span class="operator">[</span>i<span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> okButton)
    /// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found button at index "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///   }
    ///
    /// </pre>
    /// <p>To avoid this problem, replace <code>hash[i]</code> with <code>hash.value(i)</code> in the code above.</p>
    /// <p>Internally, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> uses a hash table to perform lookups. Unlike Qt 3's <code>QDict</code> class, which needed to be initialized with a prime number, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s hash table automatically grows and shrinks to provide fast lookups without wasting too much memory. You can still control the size of the hash table by calling <a href="http://doc.qt.io/qt-5/qhash.html#reserve">reserve</a>() if you already know approximately how many items the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> will contain, but this isn't necessary to obtain good performance. You can also call <a href="http://doc.qt.io/qt-5/qhash.html#capacity">capacity</a>() to retrieve the hash table's size.</p>
    /// <p>If you want to navigate through all the (key, value) pairs stored in a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, you can use an iterator. <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qhashiterator.html">QHashIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablehashiterator.html">QMutableHashIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a>). Here's how to iterate over a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, int&gt; using a Java-style iterator:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhashiterator.html">QHashIterator</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> i(hash);
    ///   <span class="keyword">while</span> (i<span class="operator">.</span>hasNext()) {
    /// &#32;     i<span class="operator">.</span>next();
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///   }
    ///
    /// </pre>
    /// <p>Here's the same code, but using an STL-style iterator:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i <span class="operator">=</span> hash<span class="operator">.</span>constBegin();
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>constEnd()) {
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    ///   }
    ///
    /// </pre>
    /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> is unordered, so an iterator's sequence cannot be assumed to be predictable. If ordering by key is required, use a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>.</p>
    /// <p>Normally, a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> allows only one value per key. If you call <a href="http://doc.qt.io/qt-5/qhash.html#insert">insert</a>() with a key that already exists in the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, the previous value is erased. For example:</p>
    /// <pre class="cpp">
    ///
    ///   hash<span class="operator">.</span>insert(<span class="string">"plenty"</span><span class="operator">,</span> <span class="number">100</span>);
    ///   hash<span class="operator">.</span>insert(<span class="string">"plenty"</span><span class="operator">,</span> <span class="number">2000</span>);
    ///   <span class="comment">// hash.value("plenty") == 2000</span>
    ///
    /// </pre>
    /// <p>However, you can store multiple values per key by using <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">insertMulti</a>() instead of <a href="http://doc.qt.io/qt-5/qhash.html#insert">insert</a>() (or using the convenience subclass <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>). If you want to retrieve all the values for a single key, you can use values(const Key &amp;key), which returns a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> values <span class="operator">=</span> hash<span class="operator">.</span>values(<span class="string">"plenty"</span>);
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> values<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> values<span class="operator">.</span>at(i) <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>The items that share the same key are available from most recently to least recently inserted. A more efficient approach is to call <a href="http://doc.qt.io/qt-5/qhash.html#find">find</a>() to get the iterator for the first item with a key and iterate from there:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> hash<span class="operator">.</span>find(<span class="string">"plenty"</span>);
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end() <span class="operator">&amp;</span><span class="operator">&amp;</span> i<span class="operator">.</span>key() <span class="operator">=</span><span class="operator">=</span> <span class="string">"plenty"</span>) {
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    ///   }
    ///
    /// </pre>
    /// <p>If you only need to extract the values from a hash (not the keys), you can also use <a href="http://doc.qt.io/qt-5/containers.html#foreach">foreach</a>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   foreach (<span class="type">int</span> value<span class="operator">,</span> hash)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> value <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Items can be removed from the hash in several ways. One way is to call <a href="http://doc.qt.io/qt-5/qhash.html#remove">remove</a>(); this will remove any item with the given key. Another way is to use <a href="http://doc.qt.io/qt-5/qmutablehashiterator.html#remove">QMutableHashIterator::remove</a>(). In addition, you can clear the entire hash using <a href="http://doc.qt.io/qt-5/qhash.html#clear">clear</a>().</p>
    /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s key and value data types must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *.</p>
    /// <a name="qhash"></a><a name="the-qhash-hashing-function"></a>
    /// <h4>The qHash() hashing function</h4>
    /// <p>A <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s key type has additional requirements other than being an assignable data type: it must provide operator==(), and there must also be a <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function in the type's namespace that returns a hash value for an argument of the key's type.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function computes a numeric value based on a key. It can use any algorithm imaginable, as long as it always returns the same value if given the same argument. In other words, if <code>e1 == e2</code>, then <code>qHash(e1) == qHash(e2)</code> must hold as well. However, to obtain good performance, the <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function should attempt to return different hash values for different keys to the largest extent possible.</p>
    /// <p>For a key type <code>K</code>, the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a> function must have one of these signatures:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(K key);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(<span class="keyword">const</span> K <span class="operator">&amp;</span>key);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(K key<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> seed);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(<span class="keyword">const</span> K <span class="operator">&amp;</span>key<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> seed);
    ///
    /// </pre>
    /// <p>The two-arguments overloads take an unsigned integer that should be used to seed the calculation of the hash function. This seed is provided by <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> in order to prevent a family of <a href="http://doc.qt.io/qt-5/qhash.html#algorithmic-complexity-attacks">algorithmic complexity attacks</a>. If both a one-argument and a two-arguments overload are defined for a key type, the latter is used by <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> (note that you can simply define a two-arguments version, and use a default value for the seed parameter).</p>
    /// <p>Here's a partial list of the C++ and Qt types that can serve as keys in a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>: any integer type (char, unsigned long, etc.), any pointer type, <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>, <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, and <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>. For all of these, the <code>&lt;QHash&gt;</code> header defines a <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function that computes an adequate hash value. Many other Qt classes also declare a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a> overload for their type; please refer to the documentation of each class.</p>
    /// <p>If you want to use other types as the key, make sure that you provide operator==() and a <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() implementation.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="preprocessor">#ifndef EMPLOYEE_H</span>
    ///   <span class="preprocessor">#define EMPLOYEE_H</span>
    ///
    ///   <span class="keyword">class</span> Employee
    ///   {
    ///   <span class="keyword">public</span>:
    /// &#32;     Employee() {}
    /// &#32;     Employee(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>name<span class="operator">,</span> <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span> <span class="operator">&amp;</span>dateOfBirth);
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///
    ///   <span class="keyword">private</span>:
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> myName;
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span> myDateOfBirth;
    ///   };
    ///
    ///   <span class="keyword">inline</span> bool <span class="keyword">operator</span><span class="operator">=</span><span class="operator">=</span>(<span class="keyword">const</span> Employee <span class="operator">&amp;</span>e1<span class="operator">,</span> <span class="keyword">const</span> Employee <span class="operator">&amp;</span>e2)
    ///   {
    /// &#32;     <span class="keyword">return</span> e1<span class="operator">.</span>name() <span class="operator">=</span><span class="operator">=</span> e2<span class="operator">.</span>name()
    /// &#32;   &#32;   &#32;    <span class="operator">&amp;</span><span class="operator">&amp;</span> e1<span class="operator">.</span>dateOfBirth() <span class="operator">=</span><span class="operator">=</span> e2<span class="operator">.</span>dateOfBirth();
    ///   }
    ///
    ///   <span class="keyword">inline</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(<span class="keyword">const</span> Employee <span class="operator">&amp;</span>key<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> seed)
    ///   {
    /// &#32;     <span class="keyword">return</span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(key<span class="operator">.</span>name()<span class="operator">,</span> seed) <span class="operator">^</span> key<span class="operator">.</span>dateOfBirth()<span class="operator">.</span>day();
    ///   }
    ///
    ///   <span class="preprocessor">#endif // EMPLOYEE_H</span>
    ///
    /// </pre>
    /// <p>In the example above, we've relied on Qt's global <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a>(const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;, uint) to give us a hash value for the employee's name, and XOR'ed this with the day they were born to help produce unique hashes for people with the same name.</p>
    /// <p>Note that the implementation of the <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() overloads offered by Qt may change at any time. You <b>must not</b> rely on the fact that <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() will give the same results (for the same inputs) across different Qt versions.</p>
    /// <a name="algorithmic-complexity-attacks"></a>
    /// <h4>Algorithmic complexity attacks</h4>
    /// <p>All hash tables are vulnerable to a particular class of denial of service attacks, in which the attacker carefully pre-computes a set of different keys that are going to be hashed in the same bucket of a hash table (or even have the very same hash value). The attack aims at getting the worst-case algorithmic behavior (O(n) instead of amortized O(1), see <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for the details) when the data is fed into the table.</p>
    /// <p>In order to avoid this worst-case behavior, the calculation of the hash value done by <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() can be salted by a random seed, that nullifies the attack's extent. This seed is automatically generated by <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> once per process, and then passed by <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as the second argument of the two-arguments overload of the <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function.</p>
    /// <p>This randomization of <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> is enabled by default. Even though programs should never depend on a particular <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> ordering, there may be situations where you temporarily need deterministic behavior, for example for debugging or regression testing. To disable the randomization, define the environment variable <code>QT_HASH_SEED</code>. The contents of that variable, interpreted as a decimal value, will be used as the seed for <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(). Alternatively, you can call the <a href="http://doc.qt.io/qt-5/qhash.html#qSetGlobalQHashSeed">qSetGlobalQHashSeed</a>() function.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QHashOfQStringQVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QHashOfQStringQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_QHash1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QHashOfQStringQVariant>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all the items that have the <i>key</i> from the hash. Returns the number of items removed which is usually 1 but will be 0 if the key isn't in the hash, or greater than 1 if <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">insertMulti</a>() has been used with the <i>key</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QHash<QString, QVariant>::remove(const QString& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#remove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all the items that have the <i>key</i> from the hash. Returns the number of items removed which is usually 1 but will be 0 if the key isn't in the hash, or greater than 1 if <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">insertMulti</a>() has been used with the <i>key</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#clear">clear</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#take">take</a>(), and <a href="http://doc.qt.io/qt-5/qmultihash.html#remove">QMultiHash::remove</a>().</p></div>
    pub unsafe fn remove(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_remove(
            self as *mut crate::QHashOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        )
    }

    /// <p>Ensures that the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s internal hash table consists of at least <i>size</i> buckets.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QHash<QString, QVariant>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Ensures that the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s internal hash table consists of at least <i>size</i> buckets.</p>
    /// <p>This function is useful for code that needs to build a huge hash and wants to avoid repeated reallocation. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
    ///   hash<span class="operator">.</span>reserve(<span class="number">20000</span>);
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> <span class="number">20000</span>; <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     hash<span class="operator">.</span>insert(keys<span class="operator">[</span>i<span class="operator">]</span><span class="operator">,</span> values<span class="operator">[</span>i<span class="operator">]</span>);
    ///
    /// </pre>
    /// <p>Ideally, <i>size</i> should be slightly more than the maximum number of items expected in the hash. <i>size</i> doesn't have to be prime, because <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> will use a prime number internally anyway. If <i>size</i> is an underestimate, the worst that will happen is that the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> will be a bit slower.</p>
    /// <p>In general, you will rarely ever need to call this function. <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s internal hash table automatically shrinks or grows to provide good performance without wasting too much memory.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#squeeze">squeeze</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#capacity">capacity</a>().</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_reserve(
            self as *mut crate::QHashOfQStringQVariant,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QHash<QString, QVariant>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_setSharable(
            self as *mut crate::QHashOfQStringQVariant,
            sharable,
        )
    }

    /// <p>Returns the number of items in the hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QHash<QString, QVariant>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the hash.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#count-1">count</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_size(
            self as *const crate::QHashOfQStringQVariant,
        )
    }

    /// <p>Reduces the size of the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s internal hash table to save memory.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QHash<QString, QVariant>::squeeze()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#squeeze">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reduces the size of the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s internal hash table to save memory.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s memory usage. In general, you will rarely ever need to call this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#capacity">capacity</a>().</p></div>
    pub unsafe fn squeeze(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_squeeze(
            self as *mut crate::QHashOfQStringQVariant,
        )
    }

    /// <p>Removes the item with the <i>key</i> from the hash and returns the value associated with it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QHash<QString, QVariant>::take(const QString& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#take">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item with the <i>key</i> from the hash and returns the value associated with it.</p>
    /// <p>If the item does not exist in the hash, the function simply returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If there are multiple items for <i>key</i> in the hash, only the most recently inserted one is removed.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qhash.html#remove">remove</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#remove">remove</a>().</p></div>
    pub unsafe fn take(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_take(
            self as *mut crate::QHashOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list containing all the keys in the map. Keys that occur multiple times in the map (because items were inserted with <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">insertMulti</a>(), or <a href="http://doc.qt.io/qt-5/qhash.html#unite">unite</a>() was used) occur only once in the returned list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QString> QHash<QString, QVariant>::uniqueKeys() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#uniqueKeys">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list containing all the keys in the map. Keys that occur multiple times in the map (because items were inserted with <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">insertMulti</a>(), or <a href="http://doc.qt.io/qt-5/qhash.html#unite">unite</a>() was used) occur only once in the returned list.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#keys">keys</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#values">values</a>().</p></div>
    pub unsafe fn unique_keys(&self) -> ::cpp_utils::CppBox<crate::QListOfQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_uniqueKeys(
            self as *const crate::QHashOfQStringQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value associated with the <i>key</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QHash<QString, QVariant>::value(const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value associated with the <i>key</i>.</p>
    /// <p>If the hash contains no item with the <i>key</i>, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If there are multiple items for the <i>key</i> in the hash, the value of the most recently inserted one is returned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#key">key</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#values">values</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qhash.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_value(
            self as *const crate::QHashOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QHash<QString, QVariant>::value(const QString& key, const QVariant& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the hash contains no item with the given <i>key</i>, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_value1(
            self as *const crate::QHashOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list containing all the values in the hash, in an arbitrary order. If a key is associated with multiple values, all of its values will be in the list, and not just the most recently inserted one.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant> QHash<QString, QVariant>::values() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#values">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list containing all the values in the hash, in an arbitrary order. If a key is associated with multiple values, all of its values will be in the list, and not just the most recently inserted one.</p>
    /// <p>The order is guaranteed to be the same as that used by <a href="http://doc.qt.io/qt-5/qhash.html#keys">keys</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#keys">keys</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>().</p></div>
    pub unsafe fn values_0a(&self) -> ::cpp_utils::CppBox<crate::QListOfQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_values(
            self as *const crate::QHashOfQStringQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant> QHash<QString, QVariant>::values(const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#values-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a list of all the values associated with the <i>key</i>, from the most recently inserted to the least recently inserted.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#count-1">count</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">insertMulti</a>().</p></div>
    pub unsafe fn values_1a(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_values1(
            self as *const crate::QHashOfQStringQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_map_of_int_q_variant {
    //! C++ type: <span style='color: green;'>```QMap<int, QVariant>```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QMap<int, QVariant>::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>&lt;Key, T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> (or <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>) and to modify the value (but not the key) stored under a particular key. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, you should use <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a>. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> function like <a href="http://doc.qt.io/qt-5/qmap.html#begin">QMap::begin</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#end">QMap::end</a>(), or <a href="http://doc.qt.io/qt-5/qmap.html#find">QMap::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a map:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> map;
    ///   map<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
    ///   map<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   map<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> map<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, which stores its items in an arbitrary order, <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> stores its items ordered by key. Items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">QMap::insertMulti</a>(), or due to a <a href="http://doc.qt.io/qt-5/qmap.html#unite">unite</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
    /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a>. Here's an example that increments every value stored in the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> by 2:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> map<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     i<span class="operator">.</span>value() <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
    ///
    /// </pre>
    /// <p>Here's an example that removes all the items whose key is a string that starts with an underscore character:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> map<span class="operator">.</span>begin();
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end()) {
    /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
    /// &#32;   &#32;     i <span class="operator">=</span> map<span class="operator">.</span>erase(i);
    /// &#32;     <span class="keyword">else</span>
    /// &#32;   &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    ///   }
    ///
    /// </pre>
    /// <p>The call to <a href="http://doc.qt.io/qt-5/qmap.html#erase">QMap::erase</a>() removes the item pointed to by the iterator from the map, and returns an iterator to the next item. Here's another way of removing an item while iterating:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> map<span class="operator">.</span>begin();
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end()) {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator prev <span class="operator">=</span> i;
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    /// &#32;     <span class="keyword">if</span> (prev<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
    /// &#32;   &#32;     map<span class="operator">.</span>erase(prev);
    ///   }
    ///
    /// </pre>
    /// <p>It might be tempting to write code like this:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="comment">// WRONG</span>
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end()) {
    /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
    /// &#32;   &#32;     map<span class="operator">.</span>erase(i);
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    ///   }
    ///
    /// </pre>
    /// <p>However, this will potentially crash in <code>++i</code>, because <code>i</code> is a dangling iterator after the call to <a href="http://doc.qt.io/qt-5/qmap.html#erase">erase</a>().</p>
    /// <p>Multiple iterators can be used on the same map. If you add items to the map, existing iterators will remain valid. If you remove items from the map, iterators that point to the removed items will become dangling iterators.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::iterator& QMap<int, QVariant>::iterator::operator=(const QMap<int, QVariant>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmap-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>&lt;Key, T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> (or <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>) and to modify the value (but not the key) stored under a particular key. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, you should use <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a>. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> function like <a href="http://doc.qt.io/qt-5/qmap.html#begin">QMap::begin</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#end">QMap::end</a>(), or <a href="http://doc.qt.io/qt-5/qmap.html#find">QMap::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a map:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> map;
        ///   map<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
        ///   map<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   map<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> map<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Unlike <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, which stores its items in an arbitrary order, <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> stores its items ordered by key. Items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">QMap::insertMulti</a>(), or due to a <a href="http://doc.qt.io/qt-5/qmap.html#unite">unite</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a>. Here's an example that increments every value stored in the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> map<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     i<span class="operator">.</span>value() <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Here's an example that removes all the items whose key is a string that starts with an underscore character:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> map<span class="operator">.</span>begin();
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end()) {
        /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     i <span class="operator">=</span> map<span class="operator">.</span>erase(i);
        /// &#32;     <span class="keyword">else</span>
        /// &#32;   &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        ///   }
        ///
        /// </pre>
        /// <p>The call to <a href="http://doc.qt.io/qt-5/qmap.html#erase">QMap::erase</a>() removes the item pointed to by the iterator from the map, and returns an iterator to the next item. Here's another way of removing an item while iterating:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> map<span class="operator">.</span>begin();
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end()) {
        /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator prev <span class="operator">=</span> i;
        /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        /// &#32;     <span class="keyword">if</span> (prev<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     map<span class="operator">.</span>erase(prev);
        ///   }
        ///
        /// </pre>
        /// <p>It might be tempting to write code like this:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="comment">// WRONG</span>
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end()) {
        /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     map<span class="operator">.</span>erase(i);
        /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        ///   }
        ///
        /// </pre>
        /// <p>However, this will potentially crash in <code>++i</code>, because <code>i</code> is a dangling iterator after the call to <a href="http://doc.qt.io/qt-5/qmap.html#erase">erase</a>().</p>
        /// <p>Multiple iterators can be used on the same map. If you add items to the map, existing iterators will remain valid. If you remove items from the map, iterators that point to the removed items will become dangling iterators.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_map_of_int_q_variant::Iterator>>,
        ) -> ::cpp_utils::MutRef<crate::q_map_of_int_q_variant::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_iterator_operator_3(
                    self as *mut crate::q_map_of_int_q_variant::Iterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_map_of_int_q_variant::Iterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::iterator QMap<int, QVariant>::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_iterator_operator__6(
                self as *mut crate::q_map_of_int_q_variant::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::iterator QMap<int, QVariant>::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_iterator_operator__4(
                self as *mut crate::q_map_of_int_q_variant::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the current item's key as a const reference.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const int& QMap<int, QVariant>::iterator::key() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current item's key as a const reference.</p>
        /// <p>There is no direct way of changing an item's key through an iterator, although it can be done by calling <a href="http://doc.qt.io/qt-5/qhash.html#erase">QHash::erase</a>() followed by <a href="http://doc.qt.io/qt-5/qhash.html#insert">QHash::insert</a>() or <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">QHash::insertMulti</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn key(&self) -> ::cpp_utils::Ref<::std::os::raw::c_int> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_iterator_key(
                self as *const crate::q_map_of_int_q_variant::Iterator,
            );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QMap<int, QVariant>::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmap-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like <a href="http://doc.qt.io/qt-5/qmap-iterator.html#key">key</a>(), <a href="http://doc.qt.io/qt-5/qmap-iterator.html#value">value</a>(), and operator++() must not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#begin">QMap::begin</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#end">QMap::end</a>().</p></div>
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>The <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QMap<int, QVariant>::iterator::iterator(const QMap<int, QVariant>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmap-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>&lt;Key, T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> (or <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>) and to modify the value (but not the key) stored under a particular key. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, you should use <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a>. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> function like <a href="http://doc.qt.io/qt-5/qmap.html#begin">QMap::begin</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#end">QMap::end</a>(), or <a href="http://doc.qt.io/qt-5/qmap.html#find">QMap::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a map:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> map;
        ///   map<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
        ///   map<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   map<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> map<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Unlike <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, which stores its items in an arbitrary order, <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> stores its items ordered by key. Items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">QMap::insertMulti</a>(), or due to a <a href="http://doc.qt.io/qt-5/qmap.html#unite">unite</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a>. Here's an example that increments every value stored in the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> map<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     i<span class="operator">.</span>value() <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Here's an example that removes all the items whose key is a string that starts with an underscore character:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> map<span class="operator">.</span>begin();
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end()) {
        /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     i <span class="operator">=</span> map<span class="operator">.</span>erase(i);
        /// &#32;     <span class="keyword">else</span>
        /// &#32;   &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        ///   }
        ///
        /// </pre>
        /// <p>The call to <a href="http://doc.qt.io/qt-5/qmap.html#erase">QMap::erase</a>() removes the item pointed to by the iterator from the map, and returns an iterator to the next item. Here's another way of removing an item while iterating:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> map<span class="operator">.</span>begin();
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end()) {
        /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator prev <span class="operator">=</span> i;
        /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        /// &#32;     <span class="keyword">if</span> (prev<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     map<span class="operator">.</span>erase(prev);
        ///   }
        ///
        /// </pre>
        /// <p>It might be tempting to write code like this:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="comment">// WRONG</span>
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end()) {
        /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     map<span class="operator">.</span>erase(i);
        /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        ///   }
        ///
        /// </pre>
        /// <p>However, this will potentially crash in <code>++i</code>, because <code>i</code> is a dangling iterator after the call to <a href="http://doc.qt.io/qt-5/qmap.html#erase">erase</a>().</p>
        /// <p>Multiple iterators can be used on the same map. If you add items to the map, existing iterators will remain valid. If you remove items from the map, iterators that point to the removed items will become dangling iterators.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn new_copy(
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_map_of_int_q_variant::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_iterator_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_map_of_int_q_variant::Iterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QVariant* QMap<int, QVariant>::iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's value.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::MutPtr<crate::QVariant> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_iterator_operator__(
                self as *const crate::q_map_of_int_q_variant::Iterator,
            );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }

        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QVariant& QMap<int, QVariant>::iterator::value() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>You can change the value of an item by using value() on the left side of an assignment, for example:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="keyword">if</span> (i<span class="operator">.</span>key() <span class="operator">=</span><span class="operator">=</span> <span class="string">"Hello"</span>)
        /// &#32;     i<span class="operator">.</span>value() <span class="operator">=</span> <span class="string">"Bonjour"</span>;
        ///
        /// </pre>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">operator*</a>().</p></div>
        pub unsafe fn value(&self) -> ::cpp_utils::MutRef<crate::QVariant> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_iterator_value(
                self as *const crate::q_map_of_int_q_variant::Iterator,
            );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QMap<int, QVariant>::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>&lt;Key, T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> (or a <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>). If you want to modify the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> as you iterate over it, you must use <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> function like <a href="http://doc.qt.io/qt-5/qmap.html#constBegin">QMap::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#constEnd">QMap::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qmap.html#find">QMap::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a map:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> map;
    ///   map<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
    ///   map<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   map<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> map<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, which stores its items in an arbitrary order, <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> stores its items ordered by key. Items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">QMap::insertMulti</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
    /// <p>Multiple iterators can be used on the same map. If you add items to the map, existing iterators will remain valid. If you remove items from the map, iterators that point to the removed items will become dangling iterators.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::const_iterator& QMap<int, QVariant>::const_iterator::operator=(const QMap<int, QVariant>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>&lt;Key, T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> (or a <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>). If you want to modify the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> as you iterate over it, you must use <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> function like <a href="http://doc.qt.io/qt-5/qmap.html#constBegin">QMap::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#constEnd">QMap::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qmap.html#find">QMap::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a map:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> map;
        ///   map<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
        ///   map<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   map<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> map<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Unlike <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, which stores its items in an arbitrary order, <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> stores its items ordered by key. Items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">QMap::insertMulti</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
        /// <p>Multiple iterators can be used on the same map. If you add items to the map, existing iterators will remain valid. If you remove items from the map, iterators that point to the removed items will become dangling iterators.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_map_of_int_q_variant::ConstIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_map_of_int_q_variant::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_const_iterator_operator_3(
                    self as *mut crate::q_map_of_int_q_variant::ConstIterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_map_of_int_q_variant::ConstIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::const_iterator QMap<int, QVariant>::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_const_iterator_operator__6(
                    self as *mut crate::q_map_of_int_q_variant::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::const_iterator QMap<int, QVariant>::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_const_iterator_operator__4(
                    self as *mut crate::q_map_of_int_q_variant::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the current key, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const int& QMap<int, QVariant>::const_iterator::key() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#key">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current key, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        pub unsafe fn key(&self) -> ::cpp_utils::Ref<::std::os::raw::c_int> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_const_iterator_key(
                self as *const crate::q_map_of_int_q_variant::ConstIterator,
            );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QMap<int, QVariant>::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html#key">key</a>(), <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html#value">value</a>(), and operator++() must not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#constBegin">QMap::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#constEnd">QMap::constEnd</a>().</p></div>
        pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::ConstIterator>
        {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_const_iterator_const_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs a copy of <i>other</i>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QMap<int, QVariant>::const_iterator::const_iterator(const QMap<int, QVariant>::iterator& o)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html#const_iterator-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p></div>
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_map_of_int_q_variant::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_const_iterator_const_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_map_of_int_q_variant::Iterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>The <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QMap<int, QVariant>::const_iterator::const_iterator(const QMap<int, QVariant>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>&lt;Key, T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> (or a <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>). If you want to modify the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> as you iterate over it, you must use <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> function like <a href="http://doc.qt.io/qt-5/qmap.html#constBegin">QMap::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#constEnd">QMap::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qmap.html#find">QMap::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a map:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> map;
        ///   map<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
        ///   map<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   map<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> map<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Unlike <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, which stores its items in an arbitrary order, <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> stores its items ordered by key. Items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">QMap::insertMulti</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
        /// <p>Multiple iterators can be used on the same map. If you add items to the map, existing iterators will remain valid. If you remove items from the map, iterators that point to the removed items will become dangling iterators.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn new_copy(
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_map_of_int_q_variant::ConstIterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_const_iterator_const_iterator2(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_map_of_int_q_variant::ConstIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current result.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QVariant* QMap<int, QVariant>::const_iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current result.</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::Ptr<crate::QVariant> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_const_iterator_operator__(
                    self as *const crate::q_map_of_int_q_variant::ConstIterator,
                );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }

        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QVariant& QMap<int, QVariant>::const_iterator::value() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        pub unsafe fn value(&self) -> ::cpp_utils::Ref<crate::QVariant> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_const_iterator_value(
                self as *const crate::q_map_of_int_q_variant::ConstIterator,
            );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a> keys.</p>
    ///
    /// C++ class: <span style='color: green;'>```QMap<int, QVariant>::key_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a> keys.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> is essentially the same as <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> with the difference that operator*() and operator-&gt;() return a key instead of a value.</p>
    /// <p>For most uses <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> and <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> should be used, you can easily access the key by calling <a href="http://doc.qt.io/qt-5/qmap-iterator.html#key">QMap::iterator::key</a>():</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">for</span> (<span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator it <span class="operator">=</span> map<span class="operator">.</span>cbegin()<span class="operator">,</span> end <span class="operator">=</span> map<span class="operator">.</span>cend(); it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The key: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Also the value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> (<span class="operator">*</span>it) <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///   }
    ///
    /// </pre>
    /// <p>However, to have interoperability between <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>'s keys and STL-style algorithms we need an iterator that dereferences to a key instead of a value. With <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> we can apply an algorithm to a range of keys without having to call <a href="http://doc.qt.io/qt-5/qmap.html#keys">QMap::keys</a>(), which is inefficient as it costs one <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> iteration and memory allocation to create a temporary <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="comment">// Inefficient, keys() is expensive</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> keys <span class="operator">=</span> map<span class="operator">.</span>keys();
    ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(map<span class="operator">.</span>cbegin()<span class="operator">,</span> map<span class="operator">.</span>cend()<span class="operator">,</span> isPrimeNumber);
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(map2<span class="operator">.</span>keys());
    ///
    ///   <span class="comment">// Efficient, no memory allocation needed</span>
    ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(map<span class="operator">.</span>keyBegin()<span class="operator">,</span> map<span class="operator">.</span>keyEnd()<span class="operator">,</span> isPrimeNumber);
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(map2<span class="operator">.</span>keyBegin()<span class="operator">,</span> map2<span class="operator">.</span>keyEnd());
    ///
    /// </pre>
    /// <p><a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> is const, it's not possible to modify the key.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> function like <a href="http://doc.qt.io/qt-5/qmap.html#keyBegin">QMap::keyBegin</a>() or <a href="http://doc.qt.io/qt-5/qmap.html#keyEnd">QMap::keyEnd</a>().</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct KeyIterator {
        _unused: u8,
    }
    impl KeyIterator {
        /// <p>Returns the underlying <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">const_iterator</a> this <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">key_iterator</a> is based on.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::const_iterator QMap<int, QVariant>::key_iterator::base() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#base">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the underlying <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">const_iterator</a> this <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">key_iterator</a> is based on.</p></div>
        pub unsafe fn base(
            &self,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_key_iterator_base(
                self as *const crate::q_map_of_int_q_variant::KeyIterator,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>The <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a> keys.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::key_iterator& QMap<int, QVariant>::key_iterator::operator=(const QMap<int, QVariant>::key_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a> keys.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> is essentially the same as <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> with the difference that operator*() and operator-&gt;() return a key instead of a value.</p>
        /// <p>For most uses <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> and <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> should be used, you can easily access the key by calling <a href="http://doc.qt.io/qt-5/qmap-iterator.html#key">QMap::iterator::key</a>():</p>
        /// <pre class="cpp">
        ///
        ///   <span class="keyword">for</span> (<span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator it <span class="operator">=</span> map<span class="operator">.</span>cbegin()<span class="operator">,</span> end <span class="operator">=</span> map<span class="operator">.</span>cend(); it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The key: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Also the value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> (<span class="operator">*</span>it) <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///   }
        ///
        /// </pre>
        /// <p>However, to have interoperability between <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>'s keys and STL-style algorithms we need an iterator that dereferences to a key instead of a value. With <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> we can apply an algorithm to a range of keys without having to call <a href="http://doc.qt.io/qt-5/qmap.html#keys">QMap::keys</a>(), which is inefficient as it costs one <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> iteration and memory allocation to create a temporary <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <pre class="cpp">
        ///
        ///   <span class="comment">// Inefficient, keys() is expensive</span>
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> keys <span class="operator">=</span> map<span class="operator">.</span>keys();
        ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(map<span class="operator">.</span>cbegin()<span class="operator">,</span> map<span class="operator">.</span>cend()<span class="operator">,</span> isPrimeNumber);
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(map2<span class="operator">.</span>keys());
        ///
        ///   <span class="comment">// Efficient, no memory allocation needed</span>
        ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(map<span class="operator">.</span>keyBegin()<span class="operator">,</span> map<span class="operator">.</span>keyEnd()<span class="operator">,</span> isPrimeNumber);
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(map2<span class="operator">.</span>keyBegin()<span class="operator">,</span> map2<span class="operator">.</span>keyEnd());
        ///
        /// </pre>
        /// <p><a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> is const, it's not possible to modify the key.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> function like <a href="http://doc.qt.io/qt-5/qmap.html#keyBegin">QMap::keyBegin</a>() or <a href="http://doc.qt.io/qt-5/qmap.html#keyEnd">QMap::keyEnd</a>().</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_map_of_int_q_variant::KeyIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_map_of_int_q_variant::KeyIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_key_iterator_operator_1(
                    self as *mut crate::q_map_of_int_q_variant::KeyIterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_map_of_int_q_variant::KeyIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::key_iterator QMap<int, QVariant>::key_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previous item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::KeyIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_key_iterator_operator__6(
                    self as *mut crate::q_map_of_int_q_variant::KeyIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::key_iterator QMap<int, QVariant>::key_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previous item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::KeyIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_key_iterator_operator__4(
                    self as *mut crate::q_map_of_int_q_variant::KeyIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Default constructs an instance of key_iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QMap<int, QVariant>::key_iterator::key_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html#key_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Default constructs an instance of key_iterator.</p></div>
        pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::KeyIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_key_iterator_key_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QMap<int, QVariant>::key_iterator::key_iterator(QMap<int, QVariant>::const_iterator o)```</span>.
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_map_of_int_q_variant::ConstIterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::KeyIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_key_iterator_key_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_map_of_int_q_variant::ConstIterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>The <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a> keys.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QMap<int, QVariant>::key_iterator::key_iterator(const QMap<int, QVariant>::key_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a> keys.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> is essentially the same as <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> with the difference that operator*() and operator-&gt;() return a key instead of a value.</p>
        /// <p>For most uses <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> and <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> should be used, you can easily access the key by calling <a href="http://doc.qt.io/qt-5/qmap-iterator.html#key">QMap::iterator::key</a>():</p>
        /// <pre class="cpp">
        ///
        ///   <span class="keyword">for</span> (<span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator it <span class="operator">=</span> map<span class="operator">.</span>cbegin()<span class="operator">,</span> end <span class="operator">=</span> map<span class="operator">.</span>cend(); it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The key: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Also the value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> (<span class="operator">*</span>it) <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///   }
        ///
        /// </pre>
        /// <p>However, to have interoperability between <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>'s keys and STL-style algorithms we need an iterator that dereferences to a key instead of a value. With <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> we can apply an algorithm to a range of keys without having to call <a href="http://doc.qt.io/qt-5/qmap.html#keys">QMap::keys</a>(), which is inefficient as it costs one <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> iteration and memory allocation to create a temporary <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <pre class="cpp">
        ///
        ///   <span class="comment">// Inefficient, keys() is expensive</span>
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> keys <span class="operator">=</span> map<span class="operator">.</span>keys();
        ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(map<span class="operator">.</span>cbegin()<span class="operator">,</span> map<span class="operator">.</span>cend()<span class="operator">,</span> isPrimeNumber);
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(map2<span class="operator">.</span>keys());
        ///
        ///   <span class="comment">// Efficient, no memory allocation needed</span>
        ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(map<span class="operator">.</span>keyBegin()<span class="operator">,</span> map<span class="operator">.</span>keyEnd()<span class="operator">,</span> isPrimeNumber);
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(map2<span class="operator">.</span>keyBegin()<span class="operator">,</span> map2<span class="operator">.</span>keyEnd());
        ///
        /// </pre>
        /// <p><a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> is const, it's not possible to modify the key.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> function like <a href="http://doc.qt.io/qt-5/qmap.html#keyBegin">QMap::keyBegin</a>() or <a href="http://doc.qt.io/qt-5/qmap.html#keyEnd">QMap::keyEnd</a>().</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn new_copy(
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_map_of_int_q_variant::KeyIterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::KeyIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_key_iterator_key_iterator2(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_map_of_int_q_variant::KeyIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's key.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const int* QMap<int, QVariant>::key_iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's key.</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_int> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_key_iterator_operator__(
                    self as *const crate::q_map_of_int_q_variant::KeyIterator,
                );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_map_of_int_q_variant::Iterator {
        type Output = ::cpp_utils::MutRef<crate::QVariant>;
        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QVariant& QMap<int, QVariant>::iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>().</p></div>
        fn indirection(self) -> ::cpp_utils::MutRef<crate::QVariant> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_iterator_operator_(
                    self as *const crate::q_map_of_int_q_variant::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_map_of_int_q_variant::Iterator>>
        for crate::q_map_of_int_q_variant::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QMap<int, QVariant>::iterator::operator==(const QMap<int, QVariant>::iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_map_of_int_q_variant::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_iterator_operator__1(
                    self as *const crate::q_map_of_int_q_variant::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_map_of_int_q_variant::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_map_of_int_q_variant::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::iterator& QMap<int, QVariant>::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_map_of_int_q_variant::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_iterator_operator__3(
                    self as *mut crate::q_map_of_int_q_variant::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_map_of_int_q_variant::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_map_of_int_q_variant::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::iterator& QMap<int, QVariant>::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_map_of_int_q_variant::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_iterator_operator__5(
                    self as *mut crate::q_map_of_int_q_variant::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_int> for &crate::q_map_of_int_q_variant::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::Iterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions forward from this iterator. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::iterator QMap<int, QVariant>::iterator::operator+(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions forward from this iterator. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p>This operation can be slow for large <i>j</i> values.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn add(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_iterator_operator_1(
                    self as *const crate::q_map_of_int_q_variant::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_int> for &crate::q_map_of_int_q_variant::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::Iterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions backward from this iterator. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::iterator QMap<int, QVariant>::iterator::operator-(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions backward from this iterator. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p>This operation can be slow for large <i>j</i> values.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn sub(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_iterator_operator_2(
                    self as *const crate::q_map_of_int_q_variant::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_int> for crate::q_map_of_int_q_variant::Iterator {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::iterator& QMap<int, QVariant>::iterator::operator+=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_iterator_operator__7(
                    self as *mut crate::q_map_of_int_q_variant::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_int> for crate::q_map_of_int_q_variant::Iterator {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::iterator& QMap<int, QVariant>::iterator::operator-=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_iterator_operator__8(
                    self as *mut crate::q_map_of_int_q_variant::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_map_of_int_q_variant::ConstIterator>>
        for crate::q_map_of_int_q_variant::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QMap<int, QVariant>::iterator::operator==(const QMap<int, QVariant>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_map_of_int_q_variant::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_iterator_operator__9(
                    self as *const crate::q_map_of_int_q_variant::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_map_of_int_q_variant::ConstIterator {
        type Output = ::cpp_utils::Ref<crate::QVariant>;
        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QVariant& QMap<int, QVariant>::const_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        fn indirection(self) -> ::cpp_utils::Ref<crate::QVariant> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_const_iterator_operator_(
                    self as *const crate::q_map_of_int_q_variant::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_map_of_int_q_variant::ConstIterator>>
        for crate::q_map_of_int_q_variant::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QMap<int, QVariant>::const_iterator::operator==(const QMap<int, QVariant>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_map_of_int_q_variant::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_const_iterator_operator__1(
                    self as *const crate::q_map_of_int_q_variant::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_map_of_int_q_variant::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_map_of_int_q_variant::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::const_iterator& QMap<int, QVariant>::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_map_of_int_q_variant::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_const_iterator_operator__3(
                    self as *mut crate::q_map_of_int_q_variant::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_map_of_int_q_variant::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_map_of_int_q_variant::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::const_iterator& QMap<int, QVariant>::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_map_of_int_q_variant::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_const_iterator_operator__5(
                    self as *mut crate::q_map_of_int_q_variant::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_int> for &crate::q_map_of_int_q_variant::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::ConstIterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions forward from this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::const_iterator QMap<int, QVariant>::const_iterator::operator+(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions forward from this iterator.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>().</p></div>
        fn add(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_const_iterator_operator_1(
                    self as *const crate::q_map_of_int_q_variant::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_int> for &crate::q_map_of_int_q_variant::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::ConstIterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions backward from this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::const_iterator QMap<int, QVariant>::const_iterator::operator-(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions backward from this iterator.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>().</p></div>
        fn sub(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_const_iterator_operator_2(
                    self as *const crate::q_map_of_int_q_variant::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_int> for crate::q_map_of_int_q_variant::ConstIterator {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::const_iterator& QMap<int, QVariant>::const_iterator::operator+=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_const_iterator_operator__7(
                    self as *mut crate::q_map_of_int_q_variant::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_int> for crate::q_map_of_int_q_variant::ConstIterator {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::const_iterator& QMap<int, QVariant>::const_iterator::operator-=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_const_iterator_operator__8(
                    self as *mut crate::q_map_of_int_q_variant::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_map_of_int_q_variant::KeyIterator {
        type Output = ::cpp_utils::Ref<::std::os::raw::c_int>;
        /// <p>Returns the current item's key.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const int& QMap<int, QVariant>::key_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current item's key.</p></div>
        fn indirection(self) -> ::cpp_utils::Ref<::std::os::raw::c_int> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_key_iterator_operator_(
                    self as *const crate::q_map_of_int_q_variant::KeyIterator,
                )
            };
            unsafe {
                ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_map_of_int_q_variant::KeyIterator>>
        for crate::q_map_of_int_q_variant::KeyIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QMap<int, QVariant>::key_iterator::operator==(QMap<int, QVariant>::key_iterator o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_map_of_int_q_variant::KeyIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_key_iterator_operator__1(
                    self as *const crate::q_map_of_int_q_variant::KeyIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_map_of_int_q_variant::KeyIterator {
        type Output = ::cpp_utils::MutRef<crate::q_map_of_int_q_variant::KeyIterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::key_iterator& QMap<int, QVariant>::key_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#keyEnd">QHash::keyEnd</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_map_of_int_q_variant::KeyIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_key_iterator_operator__3(
                    self as *mut crate::q_map_of_int_q_variant::KeyIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_map_of_int_q_variant::KeyIterator {
        type Output = ::cpp_utils::MutRef<crate::q_map_of_int_q_variant::KeyIterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::key_iterator& QMap<int, QVariant>::key_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#keyBegin">QHash::keyBegin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_map_of_int_q_variant::KeyIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_key_iterator_operator__5(
                    self as *mut crate::q_map_of_int_q_variant::KeyIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_map_of_int_q_variant::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QMap<int, QVariant>::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmap-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>&lt;Key, T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> (or <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>) and to modify the value (but not the key) stored under a particular key. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, you should use <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a>. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> function like <a href="http://doc.qt.io/qt-5/qmap.html#begin">QMap::begin</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#end">QMap::end</a>(), or <a href="http://doc.qt.io/qt-5/qmap.html#find">QMap::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a map:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> map;
        ///   map<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
        ///   map<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   map<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> map<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Unlike <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, which stores its items in an arbitrary order, <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> stores its items ordered by key. Items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">QMap::insertMulti</a>(), or due to a <a href="http://doc.qt.io/qt-5/qmap.html#unite">unite</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a>. Here's an example that increments every value stored in the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> map<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     i<span class="operator">.</span>value() <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Here's an example that removes all the items whose key is a string that starts with an underscore character:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> map<span class="operator">.</span>begin();
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end()) {
        /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     i <span class="operator">=</span> map<span class="operator">.</span>erase(i);
        /// &#32;     <span class="keyword">else</span>
        /// &#32;   &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        ///   }
        ///
        /// </pre>
        /// <p>The call to <a href="http://doc.qt.io/qt-5/qmap.html#erase">QMap::erase</a>() removes the item pointed to by the iterator from the map, and returns an iterator to the next item. Here's another way of removing an item while iterating:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> map<span class="operator">.</span>begin();
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end()) {
        /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator prev <span class="operator">=</span> i;
        /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        /// &#32;     <span class="keyword">if</span> (prev<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     map<span class="operator">.</span>erase(prev);
        ///   }
        ///
        /// </pre>
        /// <p>It might be tempting to write code like this:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="comment">// WRONG</span>
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end()) {
        /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     map<span class="operator">.</span>erase(i);
        /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        ///   }
        ///
        /// </pre>
        /// <p>However, this will potentially crash in <code>++i</code>, because <code>i</code> is a dangling iterator after the call to <a href="http://doc.qt.io/qt-5/qmap.html#erase">erase</a>().</p>
        /// <p>Multiple iterators can be used on the same map. If you add items to the map, existing iterators will remain valid. If you remove items from the map, iterators that point to the removed items will become dangling iterators.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_iterator_diterator(
                self as *mut crate::q_map_of_int_q_variant::Iterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_map_of_int_q_variant::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QMap<int, QVariant>::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>&lt;Key, T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> (or a <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>). If you want to modify the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> as you iterate over it, you must use <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> function like <a href="http://doc.qt.io/qt-5/qmap.html#constBegin">QMap::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#constEnd">QMap::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qmap.html#find">QMap::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a map:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> map;
        ///   map<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
        ///   map<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   map<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> map<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Unlike <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, which stores its items in an arbitrary order, <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> stores its items ordered by key. Items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">QMap::insertMulti</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
        /// <p>Multiple iterators can be used on the same map. If you add items to the map, existing iterators will remain valid. If you remove items from the map, iterators that point to the removed items will become dangling iterators.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_const_iterator_dconst_iterator(
                self as *mut crate::q_map_of_int_q_variant::ConstIterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_map_of_int_q_variant::KeyIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a> keys.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QMap<int, QVariant>::key_iterator::~key_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a> keys.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> is essentially the same as <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> with the difference that operator*() and operator-&gt;() return a key instead of a value.</p>
        /// <p>For most uses <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a> and <a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> should be used, you can easily access the key by calling <a href="http://doc.qt.io/qt-5/qmap-iterator.html#key">QMap::iterator::key</a>():</p>
        /// <pre class="cpp">
        ///
        ///   <span class="keyword">for</span> (<span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator it <span class="operator">=</span> map<span class="operator">.</span>cbegin()<span class="operator">,</span> end <span class="operator">=</span> map<span class="operator">.</span>cend(); it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The key: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Also the value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> (<span class="operator">*</span>it) <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///   }
        ///
        /// </pre>
        /// <p>However, to have interoperability between <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>'s keys and STL-style algorithms we need an iterator that dereferences to a key instead of a value. With <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> we can apply an algorithm to a range of keys without having to call <a href="http://doc.qt.io/qt-5/qmap.html#keys">QMap::keys</a>(), which is inefficient as it costs one <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> iteration and memory allocation to create a temporary <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <pre class="cpp">
        ///
        ///   <span class="comment">// Inefficient, keys() is expensive</span>
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> keys <span class="operator">=</span> map<span class="operator">.</span>keys();
        ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(map<span class="operator">.</span>cbegin()<span class="operator">,</span> map<span class="operator">.</span>cend()<span class="operator">,</span> isPrimeNumber);
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(map2<span class="operator">.</span>keys());
        ///
        ///   <span class="comment">// Efficient, no memory allocation needed</span>
        ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(map<span class="operator">.</span>keyBegin()<span class="operator">,</span> map<span class="operator">.</span>keyEnd()<span class="operator">,</span> isPrimeNumber);
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(map2<span class="operator">.</span>keyBegin()<span class="operator">,</span> map2<span class="operator">.</span>keyEnd());
        ///
        /// </pre>
        /// <p><a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> is const, it's not possible to modify the key.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qmap-key-iterator.html">QMap::key_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> function like <a href="http://doc.qt.io/qt-5/qmap.html#keyBegin">QMap::keyBegin</a>() or <a href="http://doc.qt.io/qt-5/qmap.html#keyEnd">QMap::keyEnd</a>().</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_key_iterator_dkey_iterator(
                self as *mut crate::q_map_of_int_q_variant::KeyIterator,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> class is a template class that provides a red-black-tree-based dictionary.</p>
///
/// C++ class: <span style='color: green;'>```QMap<int, QVariant>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qmap.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> class is a template class that provides a red-black-tree-based dictionary.</p>
/// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>&lt;Key, T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores (key, value) pairs and provides fast lookup of the value associated with a key.</p>
/// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> and <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> provide very similar functionality. The differences are:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> provides average faster lookups than <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.)</li>
/// <li>When iterating over a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, the items are arbitrarily ordered. With <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, the items are always sorted by key.</li>
/// <li>The key type of a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> must provide operator==() and a global <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a>(Key) function. The key type of a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> must provide operator&lt;() specifying a total order. Since Qt 5.8.1 it is also safe to use a pointer type as key, even if the underlying operator&lt;() does not provide a total order.</li>
/// </ul>
/// <p>Here's an example <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> with <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> keys and <code>int</code> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> map;
///
/// </pre>
/// <p>To insert a (key, value) pair into the map, you can use operator[]():</p>
/// <pre class="cpp">
///
///   map<span class="operator">[</span><span class="string">"one"</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>;
///   map<span class="operator">[</span><span class="string">"three"</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">3</span>;
///   map<span class="operator">[</span><span class="string">"seven"</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">7</span>;
///
/// </pre>
/// <p>This inserts the following three (key, value) pairs into the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>: ("one", 1), ("three", 3), and ("seven", 7). Another way to insert items into the map is to use <a href="http://doc.qt.io/qt-5/qmap.html#insert">insert</a>():</p>
/// <pre class="cpp">
///
///   map<span class="operator">.</span>insert(<span class="string">"twelve"</span><span class="operator">,</span> <span class="number">12</span>);
///
/// </pre>
/// <p>To look up a value, use operator[]() or <a href="http://doc.qt.io/qt-5/qmap.html#value">value</a>():</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> num1 <span class="operator">=</span> map<span class="operator">[</span><span class="string">"thirteen"</span><span class="operator">]</span>;
///   <span class="type">int</span> num2 <span class="operator">=</span> map<span class="operator">.</span>value(<span class="string">"thirteen"</span>);
///
/// </pre>
/// <p>If there is no item with the specified key in the map, these functions return a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>.</p>
/// <p>If you want to check whether the map contains a certain key, use <a href="http://doc.qt.io/qt-5/qmap.html#contains">contains</a>():</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> timeout <span class="operator">=</span> <span class="number">30</span>;
///   <span class="keyword">if</span> (map<span class="operator">.</span>contains(<span class="string">"TIMEOUT"</span>))
/// &#32;     timeout <span class="operator">=</span> map<span class="operator">.</span>value(<span class="string">"TIMEOUT"</span>);
///
/// </pre>
/// <p>There is also a <a href="http://doc.qt.io/qt-5/qmap.html#value">value</a>() overload that uses its second argument as a default value if there is no item with the specified key:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> timeout <span class="operator">=</span> map<span class="operator">.</span>value(<span class="string">"TIMEOUT"</span><span class="operator">,</span> <span class="number">30</span>);
///
/// </pre>
/// <p>In general, we recommend that you use <a href="http://doc.qt.io/qt-5/qmap.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#value">value</a>() rather than operator[]() for looking up a key in a map. The reason is that operator[]() silently inserts an item into the map if no item exists with the same key (unless the map is const). For example, the following code snippet will create 1000 items in memory:</p>
/// <pre class="cpp">
///
///   <span class="comment">// WRONG</span>
///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> map;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> <span class="number">1000</span>; <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (map<span class="operator">[</span>i<span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> okButton)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found button at index "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p>To avoid this problem, replace <code>map[i]</code> with <code>map.value(i)</code> in the code above.</p>
/// <p>If you want to navigate through all the (key, value) pairs stored in a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, you can use an iterator. <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qmapiterator.html">QMapIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablemapiterator.html">QMutableMapIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qmap-const-iterator.html">QMap::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qmap-iterator.html">QMap::iterator</a>). Here's how to iterate over a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, int&gt; using a Java-style iterator:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qmapiterator.html">QMapIterator</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> i(map);
///   <span class="keyword">while</span> (i<span class="operator">.</span>hasNext()) {
/// &#32;     i<span class="operator">.</span>next();
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p>Here's the same code, but using an STL-style iterator this time:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i <span class="operator">=</span> map<span class="operator">.</span>constBegin();
///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>constEnd()) {
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
/// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
///   }
///
/// </pre>
/// <p>The items are traversed in ascending key order.</p>
/// <p>Normally, a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> allows only one value per key. If you call <a href="http://doc.qt.io/qt-5/qmap.html#insert">insert</a>() with a key that already exists in the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, the previous value will be erased. For example:</p>
/// <pre class="cpp">
///
///   map<span class="operator">.</span>insert(<span class="string">"plenty"</span><span class="operator">,</span> <span class="number">100</span>);
///   map<span class="operator">.</span>insert(<span class="string">"plenty"</span><span class="operator">,</span> <span class="number">2000</span>);
///   <span class="comment">// map.value("plenty") == 2000</span>
///
/// </pre>
/// <p>However, you can store multiple values per key by using <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">insertMulti</a>() instead of <a href="http://doc.qt.io/qt-5/qmap.html#insert">insert</a>() (or using the convenience subclass <a href="http://doc.qt.io/qt-5/qmultimap.html">QMultiMap</a>). If you want to retrieve all the values for a single key, you can use values(const Key &amp;key), which returns a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> values <span class="operator">=</span> map<span class="operator">.</span>values(<span class="string">"plenty"</span>);
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> values<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> values<span class="operator">.</span>at(i) <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>The items that share the same key are available from most recently to least recently inserted. Another approach is to call <a href="http://doc.qt.io/qt-5/qmap.html#find">find</a>() to get the STL-style iterator for the first item with a key and iterate from there:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> map<span class="operator">.</span>find(<span class="string">"plenty"</span>);
///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end() <span class="operator">&amp;</span><span class="operator">&amp;</span> i<span class="operator">.</span>key() <span class="operator">=</span><span class="operator">=</span> <span class="string">"plenty"</span>) {
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
/// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
///   }
///
/// </pre>
/// <p>If you only need to extract the values from a map (not the keys), you can also use <a href="http://doc.qt.io/qt-5/containers.html#foreach">foreach</a>:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> map;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   foreach (<span class="type">int</span> value<span class="operator">,</span> map)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> value <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>Items can be removed from the map in several ways. One way is to call <a href="http://doc.qt.io/qt-5/qmap.html#remove">remove</a>(); this will remove any item with the given key. Another way is to use <a href="http://doc.qt.io/qt-5/qmutablemapiterator.html#remove">QMutableMapIterator::remove</a>(). In addition, you can clear the entire map using <a href="http://doc.qt.io/qt-5/qmap.html#clear">clear</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>'s key and value data types must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. This covers most data types you are likely to encounter, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. In addition, <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>'s key type must provide operator&lt;(). <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> uses it to keep its items sorted, and assumes that two keys <code>x</code> and <code>y</code> are equal if neither <code>x &lt; y</code> nor <code>y &lt; x</code> is true.</p>
/// <p>Example:</p>
/// <pre class="cpp">
///
///   <span class="preprocessor">#ifndef EMPLOYEE_H</span>
///   <span class="preprocessor">#define EMPLOYEE_H</span>
///
///   <span class="keyword">class</span> Employee
///   {
///   <span class="keyword">public</span>:
/// &#32;     Employee() {}
/// &#32;     Employee(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>name<span class="operator">,</span> <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span> <span class="operator">&amp;</span>dateOfBirth);
/// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///
///   <span class="keyword">private</span>:
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> myName;
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span> myDateOfBirth;
///   };
///
///   <span class="keyword">inline</span> bool <span class="keyword">operator</span><span class="operator">&lt;</span>(<span class="keyword">const</span> Employee <span class="operator">&amp;</span>e1<span class="operator">,</span> <span class="keyword">const</span> Employee <span class="operator">&amp;</span>e2)
///   {
/// &#32;     <span class="keyword">if</span> (e1<span class="operator">.</span>name() <span class="operator">!</span><span class="operator">=</span> e2<span class="operator">.</span>name())
/// &#32;   &#32;     <span class="keyword">return</span> e1<span class="operator">.</span>name() <span class="operator">&lt;</span> e2<span class="operator">.</span>name();
/// &#32;     <span class="keyword">return</span> e1<span class="operator">.</span>dateOfBirth() <span class="operator">&lt;</span> e2<span class="operator">.</span>dateOfBirth();
///   }
///
///   <span class="preprocessor">#endif // EMPLOYEE_H</span>
///
/// </pre>
/// <p>In the example, we start by comparing the employees' names. If they're equal, we compare their dates of birth to break the tie.</p></div>
#[repr(C)]
pub struct QMapOfIntQVariant {
    _unused: u8,
}
impl QMapOfIntQVariant {
    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::const_iterator QMap<int, QVariant>::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the map.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_cbegin(
            self as *const crate::QMapOfIntQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::const_iterator QMap<int, QVariant>::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the map.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#end">end</a>().</p></div>
    pub unsafe fn cend(&self) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_cend(
            self as *const crate::QMapOfIntQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all items from the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMap<int, QVariant>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all items from the map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#remove">remove</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_clear(self as *mut crate::QMapOfIntQVariant)
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::const_iterator QMap<int, QVariant>::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_constBegin(
            self as *const crate::QMapOfIntQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::const_iterator QMap<int, QVariant>::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#end">end</a>().</p></div>
    pub unsafe fn const_end(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_constEnd(
            self as *const crate::QMapOfIntQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns an const iterator pointing to the item with key <i>key</i> in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::const_iterator QMap<int, QVariant>::constFind(const int& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#constFind">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an const iterator pointing to the item with key <i>key</i> in the map.</p>
    /// <p>If the map contains no item with key <i>key</i>, the function returns <a href="http://doc.qt.io/qt-5/qmap.html#constEnd">constEnd</a>().</p>
    /// <p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#find">find</a>() and <a href="http://doc.qt.io/qt-5/qmultimap.html#constFind">QMultiMap::constFind</a>().</p></div>
    pub unsafe fn const_find(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_constFind(
            self as *const crate::QMapOfIntQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if the map contains an item with key <i>key</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMap<int, QVariant>::contains(const int& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the map contains an item with key <i>key</i>; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#count-1">count</a>() and <a href="http://doc.qt.io/qt-5/qmultimap.html#contains">QMultiMap::contains</a>().</p></div>
    pub unsafe fn contains(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_contains(
            self as *const crate::QMapOfIntQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this map and returns a reference to this map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>& QMap<int, QVariant>::operator=(const QMap<int, QVariant>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this map and returns a reference to this map.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMapOfIntQVariant>>,
    ) -> ::cpp_utils::MutRef<crate::QMapOfIntQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_operator_(
            self as *mut crate::QMapOfIntQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMapOfIntQVariant>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of items associated with key <i>key</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMap<int, QVariant>::count(const int& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items associated with key <i>key</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#contains">contains</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">insertMulti</a>(), and <a href="http://doc.qt.io/qt-5/qmultimap.html#count">QMultiMap::count</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_count(
            self as *const crate::QMapOfIntQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMap<int, QVariant>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qmap.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_count1(
            self as *const crate::QMapOfIntQVariant,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QMap<int, QVariant>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_detach(
            self as *mut crate::QMapOfIntQVariant,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qmap.html#isEmpty">isEmpty</a>(), returning true if the map is empty; otherwise returning false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMap<int, QVariant>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qmap.html#isEmpty">isEmpty</a>(), returning true if the map is empty; otherwise returning false.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_empty(
            self as *const crate::QMapOfIntQVariant,
        )
    }

    /// <p>Removes the (key, value) pair pointed to by the iterator <i>pos</i> from the map, and returns an iterator to the next item in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::iterator QMap<int, QVariant>::erase(QMap<int, QVariant>::iterator it)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the (key, value) pair pointed to by the iterator <i>pos</i> from the map, and returns an iterator to the next item in the map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#remove">remove</a>().</p></div>
    pub unsafe fn erase(
        &mut self,
        it: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_map_of_int_q_variant::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_erase(self as *mut crate::QMapOfIntQVariant, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_map_of_int_q_variant::Iterator>>::cast_into(it).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns an iterator pointing to the item with key <i>key</i> in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::iterator QMap<int, QVariant>::find(const int& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#find">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator pointing to the item with key <i>key</i> in the map.</p>
    /// <p>If the map contains no item with key <i>key</i>, the function returns <a href="http://doc.qt.io/qt-5/qmap.html#end">end</a>().</p>
    /// <p>If the map contains multiple items with key <i>key</i>, this function returns an iterator that points to the most recently inserted value. The other values are accessible by incrementing the iterator. For example, here's some code that iterates over all the items with the same key:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> map;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i <span class="operator">=</span> map<span class="operator">.</span>find(<span class="string">"HDR"</span>);
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end() <span class="operator">&amp;</span><span class="operator">&amp;</span> i<span class="operator">.</span>key() <span class="operator">=</span><span class="operator">=</span> <span class="string">"HDR"</span>) {
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    ///   }
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#constFind">constFind</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#value">value</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#values">values</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#lowerBound">lowerBound</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#upperBound">upperBound</a>(), and <a href="http://doc.qt.io/qt-5/qmultimap.html#find">QMultiMap::find</a>().</p></div>
    pub unsafe fn find_mut(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_find(
            self as *mut crate::QMapOfIntQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::const_iterator QMap<int, QVariant>::find(const int& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#find-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn find(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_find1(
            self as *const crate::QMapOfIntQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the first value in the map, that is the value mapped to the smallest key. This function assumes that the map is not empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant& QMap<int, QVariant>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first value in the map, that is the value mapped to the smallest key. This function assumes that the map is not empty.</p>
    /// <p>When unshared (or const version is called), this executes in <a href="http://doc.qt.io/qt-5/containers.html#constant-time">constant time</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#last">last</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#firstKey">firstKey</a>(), and <a href="http://doc.qt.io/qt-5/qmap.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_first(
            self as *mut crate::QMapOfIntQVariant,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QVariant& QMap<int, QVariant>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 5.2.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_first1(
            self as *const crate::QMapOfIntQVariant,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the smallest key in the map. This function assumes that the map is not empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const int& QMap<int, QVariant>::firstKey() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#firstKey">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the smallest key in the map. This function assumes that the map is not empty.</p>
    /// <p>This executes in <a href="http://doc.qt.io/qt-5/containers.html#constant-time">constant time</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#lastKey">lastKey</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#first">first</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#keyBegin">keyBegin</a>(), and <a href="http://doc.qt.io/qt-5/qmap.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn first_key(&self) -> ::cpp_utils::Ref<::std::os::raw::c_int> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_firstKey(
            self as *const crate::QMapOfIntQVariant,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the value associated with the key <i>key</i> as a modifiable reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant& QMap<int, QVariant>::operator[](const int& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value associated with the key <i>key</i> as a modifiable reference.</p>
    /// <p>If the map contains no item with key <i>key</i>, the function inserts a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a> into the map with key <i>key</i>, and returns a reference to it. If the map contains multiple items with key <i>key</i>, this function returns a reference to the most recently inserted value.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::MutRef<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_operator__2(
            self as *mut crate::QMapOfIntQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QMap<int, QVariant>::operator[](const int& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qmap.html#value">value</a>().</p></div>
    pub unsafe fn index(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_operator__3(
            self as *const crate::QMapOfIntQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Inserts a new item with the key <i>key</i> and a value of <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::iterator QMap<int, QVariant>::insert(const int& key, const QVariant& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts a new item with the key <i>key</i> and a value of <i>value</i>.</p>
    /// <p>If there is already an item with the key <i>key</i>, that item's value is replaced with <i>value</i>.</p>
    /// <p>If there are multiple items with the key <i>key</i>, the most recently inserted item's value is replaced with <i>value</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">insertMulti</a>().</p></div>
    pub unsafe fn insert_2a(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_insert(
            self as *mut crate::QMapOfIntQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::iterator QMap<int, QVariant>::insert(QMap<int, QVariant>::const_iterator pos, const int& key, const QVariant& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts a new item with the key <i>key</i> and value <i>value</i> and with hint <i>pos</i> suggesting where to do the insert.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qmap.html#constBegin">constBegin</a>() is used as hint it indicates that the <i>key</i> is less than any key in the map while <a href="http://doc.qt.io/qt-5/qmap.html#constEnd">constEnd</a>() suggests that the <i>key</i> is (strictly) larger than any key in the map. Otherwise the hint should meet the condition (<i>pos</i> - 1).<a href="http://doc.qt.io/qt-5/qmap.html#key">key</a>() &lt; <i>key</i> &lt;= pos.<a href="http://doc.qt.io/qt-5/qmap.html#key">key</a>(). If the hint <i>pos</i> is wrong it is ignored and a regular insert is done.</p>
    /// <p>If there is already an item with the key <i>key</i>, that item's value is replaced with <i>value</i>.</p>
    /// <p>If there are multiple items with the key <i>key</i>, then exactly one of them is replaced with <i>value</i>.</p>
    /// <p>If the hint is correct and the map is unshared, the insert executes in amortized <a href="http://doc.qt.io/qt-5/containers.html#constant-time">constant time</a>.</p>
    /// <p>When creating a map from sorted data inserting the largest key first with <a href="http://doc.qt.io/qt-5/qmap.html#constBegin">constBegin</a>() is faster than inserting in sorted order with <a href="http://doc.qt.io/qt-5/qmap.html#constEnd">constEnd</a>(), since <a href="http://doc.qt.io/qt-5/qmap.html#constEnd">constEnd</a>() - 1 (which is needed to check if the hint is valid) needs <a href="http://doc.qt.io/qt-5/containers.html#logarithmic-time">logarithmic time</a>.</p>
    /// <p><b>Note:</b> Be careful with the hint. Providing an iterator from an older shared instance might crash but there is also a risk that it will silently corrupt both the map and the <i>pos</i> map.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">insertMulti</a>().</p></div>
    pub unsafe fn insert_3a(
        &mut self,
        pos: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_map_of_int_q_variant::ConstIterator>>,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::Iterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_insert1(
                self as *mut crate::QMapOfIntQVariant,
                ::cpp_utils::CastInto::<
                    ::cpp_utils::Ref<crate::q_map_of_int_q_variant::ConstIterator>,
                >::cast_into(pos)
                .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                    .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                    .as_raw_ptr(),
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Inserts a new item with the key <i>key</i> and a value of <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::iterator QMap<int, QVariant>::insertMulti(const int& key, const QVariant& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts a new item with the key <i>key</i> and a value of <i>value</i>.</p>
    /// <p>If there is already an item with the same key in the map, this function will simply create a new one. (This behavior is different from <a href="http://doc.qt.io/qt-5/qmap.html#insert">insert</a>(), which overwrites the value of an existing item.)</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#values">values</a>().</p></div>
    pub unsafe fn insert_multi_2a(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_insertMulti(
            self as *mut crate::QMapOfIntQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::iterator QMap<int, QVariant>::insertMulti(QMap<int, QVariant>::const_iterator pos, const int& akey, const QVariant& avalue)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts a new item with the key <i>key</i> and value <i>value</i> and with hint <i>pos</i> suggesting where to do the insert.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qmap.html#constBegin">constBegin</a>() is used as hint it indicates that the <i>key</i> is less than any key in the map while <a href="http://doc.qt.io/qt-5/qmap.html#constEnd">constEnd</a>() suggests that the <i>key</i> is larger than any key in the map. Otherwise the hint should meet the condition (<i>pos</i> - 1).<a href="http://doc.qt.io/qt-5/qmap.html#key">key</a>() &lt; <i>key</i> &lt;= pos.<a href="http://doc.qt.io/qt-5/qmap.html#key">key</a>(). If the hint <i>pos</i> is wrong it is ignored and a regular <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">insertMulti</a> is done.</p>
    /// <p>If there is already an item with the same key in the map, this function will simply create a new one.</p>
    /// <p><b>Note:</b> Be careful with the hint. Providing an iterator from an older shared instance might crash but there is also a risk that it will silently corrupt both the map and the <i>pos</i> map.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#insert">insert</a>().</p></div>
    pub unsafe fn insert_multi_3a(
        &mut self,
        pos: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_map_of_int_q_variant::ConstIterator>>,
        akey: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
        avalue: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::Iterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_insertMulti1(
                self as *mut crate::QMapOfIntQVariant,
                ::cpp_utils::CastInto::<
                    ::cpp_utils::Ref<crate::q_map_of_int_q_variant::ConstIterator>,
                >::cast_into(pos)
                .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(akey)
                    .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(avalue)
                    .as_raw_ptr(),
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QMap<int, QVariant>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_isDetached(
            self as *const crate::QMapOfIntQVariant,
        )
    }

    /// <p>Returns <code>true</code> if the map contains no items; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMap<int, QVariant>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the map contains no items; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_isEmpty(
            self as *const crate::QMapOfIntQVariant,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QMap<int, QVariant>::isSharedWith(const QMap<int, QVariant>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMapOfIntQVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_isSharedWith(
            self as *const crate::QMapOfIntQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMapOfIntQVariant>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMap<int, QVariant>::key(const QVariant& value, const int& defaultKey = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#key">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the first key with value <i>value</i>, or <i>defaultKey</i> if the map contains no item with value <i>value</i>. If no <i>defaultKey</i> is provided the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed key</a>.</p>
    /// <p>This function can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>'s internal data structure is optimized for fast lookup by key, not by value.</p>
    /// <p>This function was introduced in  Qt 4.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#keys">keys</a>().</p></div>
    pub unsafe fn key_2a(
        &self,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
        default_key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_key(
            self as *const crate::QMapOfIntQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(
                default_key,
            )
            .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMap<int, QVariant>::key(const QVariant& value) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#key">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the first key with value <i>value</i>, or <i>defaultKey</i> if the map contains no item with value <i>value</i>. If no <i>defaultKey</i> is provided the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed key</a>.</p>
    /// <p>This function can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>'s internal data structure is optimized for fast lookup by key, not by value.</p>
    /// <p>This function was introduced in  Qt 4.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#keys">keys</a>().</p></div>
    pub unsafe fn key_1a(
        &self,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_key1(
            self as *const crate::QMapOfIntQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first key in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::key_iterator QMap<int, QVariant>::keyBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#keyBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first key in the map.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#keyEnd">keyEnd</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#firstKey">firstKey</a>().</p></div>
    pub unsafe fn key_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::KeyIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_keyBegin(
            self as *const crate::QMapOfIntQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last key in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::key_iterator QMap<int, QVariant>::keyEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#keyEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last key in the map.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#keyBegin">keyBegin</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#lastKey">lastKey</a>().</p></div>
    pub unsafe fn key_end(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::KeyIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_keyEnd(
            self as *const crate::QMapOfIntQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list containing all the keys in the map in ascending order. Keys that occur multiple times in the map (because items were inserted with <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">insertMulti</a>(), or <a href="http://doc.qt.io/qt-5/qmap.html#unite">unite</a>() was used) also occur multiple times in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<int> QMap<int, QVariant>::keys() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#keys">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list containing all the keys in the map in ascending order. Keys that occur multiple times in the map (because items were inserted with <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">insertMulti</a>(), or <a href="http://doc.qt.io/qt-5/qmap.html#unite">unite</a>() was used) also occur multiple times in the list.</p>
    /// <p>To obtain a list of unique keys, where each key from the map only occurs once, use <a href="http://doc.qt.io/qt-5/qmap.html#uniqueKeys">uniqueKeys</a>().</p>
    /// <p>The order is guaranteed to be the same as that used by <a href="http://doc.qt.io/qt-5/qmap.html#values">values</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#uniqueKeys">uniqueKeys</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#values">values</a>(), and <a href="http://doc.qt.io/qt-5/qmap.html#key">key</a>().</p></div>
    pub unsafe fn keys_0a(&self) -> ::cpp_utils::CppBox<crate::QListOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_keys(
            self as *const crate::QMapOfIntQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<int> QMap<int, QVariant>::keys(const QVariant& value) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#keys-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a list containing all the keys associated with value <i>value</i> in ascending order.</p>
    /// <p>This function can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>'s internal data structure is optimized for fast lookup by key, not by value.</p></div>
    pub unsafe fn keys_1a(
        &self,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_keys1(
            self as *const crate::QMapOfIntQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the last value in the map, that is the value mapped to the largest key. This function assumes that the map is not empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant& QMap<int, QVariant>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last value in the map, that is the value mapped to the largest key. This function assumes that the map is not empty.</p>
    /// <p>When unshared (or const version is called), this executes in <a href="http://doc.qt.io/qt-5/containers.html#logarithmic-time">logarithmic time</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#first">first</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#lastKey">lastKey</a>(), and <a href="http://doc.qt.io/qt-5/qmap.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_last(
            self as *mut crate::QMapOfIntQVariant,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QVariant& QMap<int, QVariant>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 5.2.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_last1(
            self as *const crate::QMapOfIntQVariant,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the largest key in the map. This function assumes that the map is not empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const int& QMap<int, QVariant>::lastKey() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#lastKey">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the largest key in the map. This function assumes that the map is not empty.</p>
    /// <p>This executes in <a href="http://doc.qt.io/qt-5/containers.html#logarithmic-time">logarithmic time</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#firstKey">firstKey</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#last">last</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#keyEnd">keyEnd</a>(), and <a href="http://doc.qt.io/qt-5/qmap.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn last_key(&self) -> ::cpp_utils::Ref<::std::os::raw::c_int> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_lastKey(
            self as *const crate::QMapOfIntQVariant,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns an iterator pointing to the first item with key <i>key</i> in the map. If the map contains no item with key <i>key</i>, the function returns an iterator to the nearest item with a greater key.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::iterator QMap<int, QVariant>::lowerBound(const int& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#lowerBound">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator pointing to the first item with key <i>key</i> in the map. If the map contains no item with key <i>key</i>, the function returns an iterator to the nearest item with a greater key.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> map;
    ///   map<span class="operator">.</span>insert(<span class="number">1</span><span class="operator">,</span> <span class="string">"one"</span>);
    ///   map<span class="operator">.</span>insert(<span class="number">5</span><span class="operator">,</span> <span class="string">"five"</span>);
    ///   map<span class="operator">.</span>insert(<span class="number">10</span><span class="operator">,</span> <span class="string">"ten"</span>);
    ///
    ///   map<span class="operator">.</span>lowerBound(<span class="number">0</span>);      <span class="comment">// returns iterator to (1, "one")</span>
    ///   map<span class="operator">.</span>lowerBound(<span class="number">1</span>);      <span class="comment">// returns iterator to (1, "one")</span>
    ///   map<span class="operator">.</span>lowerBound(<span class="number">2</span>);      <span class="comment">// returns iterator to (5, "five")</span>
    ///   map<span class="operator">.</span>lowerBound(<span class="number">10</span>);     <span class="comment">// returns iterator to (10, "ten")</span>
    ///   map<span class="operator">.</span>lowerBound(<span class="number">999</span>);    <span class="comment">// returns end()</span>
    ///
    /// </pre>
    /// <p>If the map contains multiple items with key <i>key</i>, this function returns an iterator that points to the most recently inserted value. The other values are accessible by incrementing the iterator. For example, here's some code that iterates over all the items with the same key:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> map;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i <span class="operator">=</span> map<span class="operator">.</span>lowerBound(<span class="string">"HDR"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator upperBound <span class="operator">=</span> map<span class="operator">.</span>upperBound(<span class="string">"HDR"</span>);
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> upperBound) {
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    ///   }
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#upperBound">upperBound</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#find">find</a>().</p></div>
    pub unsafe fn lower_bound_mut(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_lowerBound(
            self as *mut crate::QMapOfIntQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::const_iterator QMap<int, QVariant>::lowerBound(const int& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#lowerBound-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn lower_bound(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_lowerBound1(
            self as *const crate::QMapOfIntQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs an empty map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMap<int, QVariant>::QMap()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#QMap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#clear">clear</a>().</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QMapOfIntQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_QMap();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QMap<int, QVariant>::QMap(const QMap<int, QVariant>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#QMap-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation occurs in <a href="http://doc.qt.io/qt-5/containers.html#constant-time">constant time</a>, because <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and this takes <a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMapOfIntQVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QMapOfIntQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_QMap1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMapOfIntQVariant>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all the items that have the key <i>key</i> from the map. Returns the number of items removed which is usually 1 but will be 0 if the key isn't in the map, or &gt; 1 if <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">insertMulti</a>() has been used with the <i>key</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMap<int, QVariant>::remove(const int& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#remove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all the items that have the key <i>key</i> from the map. Returns the number of items removed which is usually 1 but will be 0 if the key isn't in the map, or &gt; 1 if <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">insertMulti</a>() has been used with the <i>key</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#clear">clear</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#take">take</a>(), and <a href="http://doc.qt.io/qt-5/qmultimap.html#remove">QMultiMap::remove</a>().</p></div>
    pub unsafe fn remove(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_remove(
            self as *mut crate::QMapOfIntQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QMap<int, QVariant>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_setSharable(
            self as *mut crate::QMapOfIntQVariant,
            sharable,
        )
    }

    /// <p>Returns the number of (key, value) pairs in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QMap<int, QVariant>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of (key, value) pairs in the map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#count-1">count</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_size(
            self as *const crate::QMapOfIntQVariant,
        )
    }

    /// <p>Swaps map <i>other</i> with this map. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QMap<int, QVariant>::swap(QMap<int, QVariant>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps map <i>other</i> with this map. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QMapOfIntQVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_swap(
            self as *mut crate::QMapOfIntQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QMapOfIntQVariant>>::cast_into(
                other,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Removes the item with the key <i>key</i> from the map and returns the value associated with it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QMap<int, QVariant>::take(const int& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#take">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item with the key <i>key</i> from the map and returns the value associated with it.</p>
    /// <p>If the item does not exist in the map, the function simply returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If there are multiple items for <i>key</i> in the map, only the most recently inserted one is removed and returned.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qmap.html#remove">remove</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#remove">remove</a>().</p></div>
    pub unsafe fn take(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_take(
            self as *mut crate::QMapOfIntQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list containing all the keys in the map in ascending order. Keys that occur multiple times in the map (because items were inserted with <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">insertMulti</a>(), or <a href="http://doc.qt.io/qt-5/qmap.html#unite">unite</a>() was used) occur only once in the returned list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<int> QMap<int, QVariant>::uniqueKeys() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#uniqueKeys">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list containing all the keys in the map in ascending order. Keys that occur multiple times in the map (because items were inserted with <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">insertMulti</a>(), or <a href="http://doc.qt.io/qt-5/qmap.html#unite">unite</a>() was used) occur only once in the returned list.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#keys">keys</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#values">values</a>().</p></div>
    pub unsafe fn unique_keys(&self) -> ::cpp_utils::CppBox<crate::QListOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_uniqueKeys(
            self as *const crate::QMapOfIntQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Inserts all the items in the <i>other</i> map into this map. If a key is common to both maps, the resulting map will contain the key multiple times.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>& QMap<int, QVariant>::unite(const QMap<int, QVariant>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#unite">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts all the items in the <i>other</i> map into this map. If a key is common to both maps, the resulting map will contain the key multiple times.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">insertMulti</a>().</p></div>
    pub unsafe fn unite(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMapOfIntQVariant>>,
    ) -> ::cpp_utils::MutRef<crate::QMapOfIntQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_unite(
            self as *mut crate::QMapOfIntQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMapOfIntQVariant>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns an iterator pointing to the item that immediately follows the last item with key <i>key</i> in the map. If the map contains no item with key <i>key</i>, the function returns an iterator to the nearest item with a greater key.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::iterator QMap<int, QVariant>::upperBound(const int& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#upperBound">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator pointing to the item that immediately follows the last item with key <i>key</i> in the map. If the map contains no item with key <i>key</i>, the function returns an iterator to the nearest item with a greater key.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html#QMap">QMap</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> map;
    ///   map<span class="operator">.</span>insert(<span class="number">1</span><span class="operator">,</span> <span class="string">"one"</span>);
    ///   map<span class="operator">.</span>insert(<span class="number">5</span><span class="operator">,</span> <span class="string">"five"</span>);
    ///   map<span class="operator">.</span>insert(<span class="number">10</span><span class="operator">,</span> <span class="string">"ten"</span>);
    ///
    ///   map<span class="operator">.</span>upperBound(<span class="number">0</span>);      <span class="comment">// returns iterator to (1, "one")</span>
    ///   map<span class="operator">.</span>upperBound(<span class="number">1</span>);      <span class="comment">// returns iterator to (5, "five")</span>
    ///   map<span class="operator">.</span>upperBound(<span class="number">2</span>);      <span class="comment">// returns iterator to (5, "five")</span>
    ///   map<span class="operator">.</span>upperBound(<span class="number">10</span>);     <span class="comment">// returns end()</span>
    ///   map<span class="operator">.</span>upperBound(<span class="number">999</span>);    <span class="comment">// returns end()</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#lowerBound">lowerBound</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#find">find</a>().</p></div>
    pub unsafe fn upper_bound_mut(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_upperBound(
            self as *mut crate::QMapOfIntQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::const_iterator QMap<int, QVariant>::upperBound(const int& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#upperBound-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn upper_bound(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_upperBound1(
            self as *const crate::QMapOfIntQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value associated with the key <i>key</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QMap<int, QVariant>::value(const int& key, const QVariant& defaultValue = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value associated with the key <i>key</i>.</p>
    /// <p>If the map contains no item with key <i>key</i>, the function returns <i>defaultValue</i>. If no <i>defaultValue</i> is specified, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If there are multiple items for <i>key</i> in the map, the value of the most recently inserted one is returned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#key">key</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#values">values</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qmap.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_2a(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_value(
            self as *const crate::QMapOfIntQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value associated with the key <i>key</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QMap<int, QVariant>::value(const int& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value associated with the key <i>key</i>.</p>
    /// <p>If the map contains no item with key <i>key</i>, the function returns <i>defaultValue</i>. If no <i>defaultValue</i> is specified, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If there are multiple items for <i>key</i> in the map, the value of the most recently inserted one is returned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#key">key</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#values">values</a>(), <a href="http://doc.qt.io/qt-5/qmap.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qmap.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_value1(
            self as *const crate::QMapOfIntQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list containing all the values in the map, in ascending order of their keys. If a key is associated with multiple values, all of its values will be in the list, and not just the most recently inserted one.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant> QMap<int, QVariant>::values() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#values">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list containing all the values in the map, in ascending order of their keys. If a key is associated with multiple values, all of its values will be in the list, and not just the most recently inserted one.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#keys">keys</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#value">value</a>().</p></div>
    pub unsafe fn values_0a(&self) -> ::cpp_utils::CppBox<crate::QListOfQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_values(
            self as *const crate::QMapOfIntQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant> QMap<int, QVariant>::values(const int& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#values-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a list containing all the values associated with key <i>key</i>, from the most recently inserted to the least recently inserted one.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#count-1">count</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#insertMulti">insertMulti</a>().</p></div>
    pub unsafe fn values_1a(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_values1(
            self as *const crate::QMapOfIntQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_list_of_q_model_index {
    //! C++ type: <span style='color: green;'>```QList<QModelIndex>```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QModelIndex>::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::iterator& QList<QModelIndex>::iterator::operator=(const QList<QModelIndex>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_model_index::Iterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_model_index::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_iterator_operator_8(
                    self as *mut crate::q_list_of_q_model_index::Iterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_model_index::Iterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::iterator QList<QModelIndex>::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_model_index::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_iterator_operator__13(
                self as *mut crate::q_list_of_q_model_index::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::iterator QList<QModelIndex>::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_model_index::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_iterator_operator__11(
                self as *mut crate::q_list_of_q_model_index::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QModelIndex& QList<QModelIndex>::iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValueRef iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>. If you assign to it, the assignment will apply to the character in the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> of <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> from which you got the reference.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::MutRef<crate::QModelIndex> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_iterator_operator__1(
                self as *const crate::q_list_of_q_model_index::Iterator,
                j,
            );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QModelIndex>::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>().</p></div>
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_list_of_q_model_index::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QModelIndex>::iterator::iterator(const QList<QModelIndex>::iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_model_index::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_model_index::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_iterator_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_model_index::Iterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QModelIndex* QList<QModelIndex>::iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's value.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::MutPtr<crate::QModelIndex> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_iterator_operator__(
                self as *const crate::q_list_of_q_model_index::Iterator,
            );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QModelIndex>::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::const_iterator& QList<QModelIndex>::const_iterator::operator=(const QList<QModelIndex>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_model_index::ConstIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_model_index::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_const_iterator_operator_6(
                    self as *mut crate::q_list_of_q_model_index::ConstIterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_model_index::ConstIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::const_iterator QList<QModelIndex>::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_model_index::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_const_iterator_operator__9(
                    self as *mut crate::q_list_of_q_model_index::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::const_iterator QList<QModelIndex>::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_model_index::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_const_iterator_operator__7(
                    self as *mut crate::q_list_of_q_model_index::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QModelIndex& QList<QModelIndex>::const_iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValue const_iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::Ref<crate::QModelIndex> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_const_iterator_operator__1(
                    self as *const crate::q_list_of_q_model_index::ConstIterator,
                    j,
                );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QModelIndex>::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>().</p></div>
        pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::q_list_of_q_model_index::ConstIterator>
        {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_const_iterator_const_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QModelIndex>::const_iterator::const_iterator(const QList<QModelIndex>::iterator& o)```</span>.
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_model_index::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_model_index::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_const_iterator_const_iterator2(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_model_index::Iterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QModelIndex>::const_iterator::const_iterator(const QList<QModelIndex>::const_iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_model_index::ConstIterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_model_index::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_const_iterator_const_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_model_index::ConstIterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current result.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QModelIndex* QList<QModelIndex>::const_iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current result.</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::Ptr<crate::QModelIndex> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_const_iterator_operator__(
                    self as *const crate::q_list_of_q_model_index::ConstIterator,
                );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_model_index::Iterator {
        type Output = ::cpp_utils::MutRef<crate::QModelIndex>;
        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QModelIndex& QList<QModelIndex>::iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>().</p></div>
        fn indirection(self) -> ::cpp_utils::MutRef<crate::QModelIndex> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_iterator_operator_(
                    self as *const crate::q_list_of_q_model_index::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_model_index::Iterator>>
        for crate::q_list_of_q_model_index::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QModelIndex>::iterator::operator==(const QList<QModelIndex>::iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_model_index::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_iterator_operator__2(
                    self as *const crate::q_list_of_q_model_index::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_model_index::Iterator>>
        for crate::q_list_of_q_model_index::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QModelIndex>::iterator::operator<(const QList<QModelIndex>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_model_index::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_iterator_operator_1(
                    self as *const crate::q_list_of_q_model_index::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_model_index::Iterator>>
        for crate::q_list_of_q_model_index::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QModelIndex>::iterator::operator<=(const QList<QModelIndex>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_model_index::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_iterator_operator__4(
                    self as *const crate::q_list_of_q_model_index::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_model_index::Iterator>>
        for crate::q_list_of_q_model_index::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QModelIndex>::iterator::operator>(const QList<QModelIndex>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_model_index::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_iterator_operator_2(
                    self as *const crate::q_list_of_q_model_index::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_model_index::Iterator>>
        for crate::q_list_of_q_model_index::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QModelIndex>::iterator::operator>=(const QList<QModelIndex>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_model_index::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_iterator_operator__5(
                    self as *const crate::q_list_of_q_model_index::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_model_index::ConstIterator>>
        for crate::q_list_of_q_model_index::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QModelIndex>::iterator::operator==(const QList<QModelIndex>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_model_index::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_iterator_operator__6(
                    self as *const crate::q_list_of_q_model_index::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_model_index::ConstIterator>>
        for crate::q_list_of_q_model_index::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QModelIndex>::iterator::operator<(const QList<QModelIndex>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_model_index::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_iterator_operator_3(
                    self as *const crate::q_list_of_q_model_index::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_model_index::ConstIterator>>
        for crate::q_list_of_q_model_index::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QModelIndex>::iterator::operator<=(const QList<QModelIndex>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_model_index::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_iterator_operator__8(
                    self as *const crate::q_list_of_q_model_index::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_model_index::ConstIterator>>
        for crate::q_list_of_q_model_index::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QModelIndex>::iterator::operator>(const QList<QModelIndex>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_model_index::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_iterator_operator_4(
                    self as *const crate::q_list_of_q_model_index::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_model_index::ConstIterator>>
        for crate::q_list_of_q_model_index::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QModelIndex>::iterator::operator>=(const QList<QModelIndex>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_model_index::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_iterator_operator__9(
                    self as *const crate::q_list_of_q_model_index::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_model_index::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_model_index::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::iterator& QList<QModelIndex>::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_model_index::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_iterator_operator__10(
                    self as *mut crate::q_list_of_q_model_index::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_model_index::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_model_index::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::iterator& QList<QModelIndex>::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_model_index::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_iterator_operator__12(
                    self as *mut crate::q_list_of_q_model_index::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_model_index::Iterator
    {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::iterator& QList<QModelIndex>::iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_iterator_operator__14(
                    self as *mut crate::q_list_of_q_model_index::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_model_index::Iterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::iterator& QList<QModelIndex>::iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_iterator_operator__15(
                    self as *mut crate::q_list_of_q_model_index::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_list_of_q_model_index::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_model_index::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::iterator QList<QModelIndex>::iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_model_index::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_iterator_operator_5(
                    self as *const crate::q_list_of_q_model_index::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_list_of_q_model_index::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_model_index::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::iterator QList<QModelIndex>::iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_model_index::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_iterator_operator_6(
                    self as *const crate::q_list_of_q_model_index::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_model_index::Iterator>>
        for &crate::q_list_of_q_model_index::Iterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QModelIndex>::iterator::operator-(QList<QModelIndex>::iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_model_index::Iterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_iterator_operator_7(
                    self as *const crate::q_list_of_q_model_index::Iterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_model_index::ConstIterator {
        type Output = ::cpp_utils::Ref<crate::QModelIndex>;
        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QModelIndex& QList<QModelIndex>::const_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        fn indirection(self) -> ::cpp_utils::Ref<crate::QModelIndex> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_const_iterator_operator_(
                    self as *const crate::q_list_of_q_model_index::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_model_index::ConstIterator>>
        for crate::q_list_of_q_model_index::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QModelIndex>::const_iterator::operator==(const QList<QModelIndex>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_model_index::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_const_iterator_operator__2(
                    self as *const crate::q_list_of_q_model_index::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_model_index::ConstIterator>>
        for crate::q_list_of_q_model_index::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QModelIndex>::const_iterator::operator<(const QList<QModelIndex>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_model_index::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_const_iterator_operator_1(
                    self as *const crate::q_list_of_q_model_index::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_model_index::ConstIterator>>
        for crate::q_list_of_q_model_index::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QModelIndex>::const_iterator::operator<=(const QList<QModelIndex>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_model_index::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_const_iterator_operator__4(
                    self as *const crate::q_list_of_q_model_index::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_model_index::ConstIterator>>
        for crate::q_list_of_q_model_index::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QModelIndex>::const_iterator::operator>(const QList<QModelIndex>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_model_index::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_const_iterator_operator_2(
                    self as *const crate::q_list_of_q_model_index::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_model_index::ConstIterator>>
        for crate::q_list_of_q_model_index::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QModelIndex>::const_iterator::operator>=(const QList<QModelIndex>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_model_index::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_const_iterator_operator__5(
                    self as *const crate::q_list_of_q_model_index::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_model_index::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_model_index::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::const_iterator& QList<QModelIndex>::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_model_index::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_const_iterator_operator__6(
                    self as *mut crate::q_list_of_q_model_index::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_model_index::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_model_index::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::const_iterator& QList<QModelIndex>::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_model_index::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_const_iterator_operator__8(
                    self as *mut crate::q_list_of_q_model_index::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_model_index::ConstIterator
    {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::const_iterator& QList<QModelIndex>::const_iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_const_iterator_operator__10(
                    self as *mut crate::q_list_of_q_model_index::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_model_index::ConstIterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::const_iterator& QList<QModelIndex>::const_iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_const_iterator_operator__11(
                    self as *mut crate::q_list_of_q_model_index::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_model_index::ConstIterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_model_index::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::const_iterator QList<QModelIndex>::const_iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_model_index::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_const_iterator_operator_3(
                    self as *const crate::q_list_of_q_model_index::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_model_index::ConstIterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_model_index::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::const_iterator QList<QModelIndex>::const_iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_model_index::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_const_iterator_operator_4(
                    self as *const crate::q_list_of_q_model_index::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_model_index::ConstIterator>>
        for &crate::q_list_of_q_model_index::ConstIterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QModelIndex>::const_iterator::operator-(QList<QModelIndex>::const_iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_model_index::ConstIterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_const_iterator_operator_5(
                    self as *const crate::q_list_of_q_model_index::ConstIterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_model_index::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QModelIndex>::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_iterator_diterator(
                self as *mut crate::q_list_of_q_model_index::Iterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_model_index::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QModelIndex>::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_const_iterator_dconst_iterator(
                self as *mut crate::q_list_of_q_model_index::ConstIterator,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
///
/// C++ class: <span style='color: green;'>```QList<QModelIndex>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlist.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores items in a list that provides fast index-based access and index-based insertions and removals.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Internally, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T if <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. Otherwise, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T* and the items are allocated on the heap.</p>
/// <p>The array representation allows very fast insertions and index-based access. The <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() operations are also very fast because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates memory at both ends of its internal array. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.</p>
/// <p>Note, however, that when the conditions specified above are not met, each append or insert of a new item requires allocating the new item on the heap, and this per item allocation will make <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> a better choice for use cases that do a lot of appending or inserting, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> can allocate memory for many items in a single heap allocation.</p>
/// <p>Note that the internal array only ever gets bigger over the life of the list. It never shrinks. The internal array is deallocated by the destructor and by the assignment operator, when one list is assigned to another.</p>
/// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerList;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span><span class="operator">&gt;</span> dateList;
///
/// </pre>
/// <p>Qt includes a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class that inherits <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>&gt; and adds a few convenience functions, such as <a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#filter">QStringList::filter</a>(). <a href="http://doc.qt.io/qt-5/qstring.html#split">QString::split</a>() creates QStringLists from strings.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> stores a list of items. The default constructor creates an empty list. To insert items into the list, you can use operator&lt;&lt;():</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"one"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"two"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"three"</span>;
///   <span class="comment">// list: ["one", "two", "three"]</span>
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(). In addition, it provides the following convenience functions: <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const lists, operator[]() returns a reference to the item and can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Bob"</span>)
/// &#32;     list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Robert"</span>;
///
/// </pre>
/// <p>Because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented as an array of pointers for types that are larger than a pointer or are not movable, this operation requires (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>). For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> list<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (list<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Jane"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Jane at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>A common requirement is to remove an item from a list and do something with it. For this, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(). Here's a loop that removes the items from a list one at a time and calls <code>delete</code> on them:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   <span class="keyword">while</span> (<span class="operator">!</span>list<span class="operator">.</span>isEmpty())
/// &#32;     <span class="keyword">delete</span> list<span class="operator">.</span>takeFirst();
///
/// </pre>
/// <p>Inserting and removing items at either end of the list is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> in most cases), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
/// <p>If you want to find all occurrences of a particular value in a list, use <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of a matching item if they find it; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> list<span class="operator">.</span>indexOf(<span class="string">"Jane"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Jane is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a list contains a particular value, use <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the list, use <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>(). If you want to replace all occurrences of a particular value with another, use <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlistiterator.html">QListIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablelistiterator.html">QMutableListIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented in such a way that direct index-based access is just as fast as using iterators.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <p>To make <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as efficient as possible, its member functions don't validate their input before using it. Except for <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), member functions always assume the list is <i>not</i> empty. Member functions that take index values as parameters always assume their index value parameters are in the valid range. This means <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> member functions can fail. If you define QT_NO_DEBUG when you compile, failures will not be detected. If you <i>don't</i> define QT_NO_DEBUG, failures will be detected using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT">Q_ASSERT</a>() or <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT_X">Q_ASSERT_X</a>() with an appropriate message.</p>
/// <p>To avoid failures when your list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling other member functions. If you must pass an index value that might not be in the valid range, check that it is less than the value returned by <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() but <i>not</i> less than 0.</p>
/// <a name="more-members"></a></div>
#[repr(C)]
pub struct QListOfQModelIndex {
    _unused: u8,
}
impl QListOfQModelIndex {
    /// <p>Inserts <i>value</i> at the end of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QModelIndex>::append(const QModelIndex& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["one", "two", "three"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn append_q_model_index(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_append(
            self as *mut crate::QListOfQModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QModelIndex>::append(const QList<QModelIndex>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> list to this list.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_list_of_q_model_index(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQModelIndex>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_append1(
            self as *mut crate::QListOfQModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QModelIndex& QList<QModelIndex>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>This function is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_at(
            self as *const crate::QListOfQModelIndex,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QModelIndex& QList<QModelIndex>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::MutRef<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_back(
            self as *mut crate::QListOfQModelIndex,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QModelIndex& QList<QModelIndex>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_back1(
            self as *const crate::QListOfQModelIndex,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::const_iterator QList<QModelIndex>::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_model_index::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_cbegin(
            self as *const crate::QListOfQModelIndex,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::const_iterator QList<QModelIndex>::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn cend(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_model_index::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_cend(
            self as *const crate::QListOfQModelIndex,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all items from the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QModelIndex>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all items from the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_clear(
            self as *mut crate::QListOfQModelIndex,
        )
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::const_iterator QList<QModelIndex>::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_model_index::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_constBegin(
            self as *const crate::QListOfQModelIndex,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::const_iterator QList<QModelIndex>::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn const_end(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_model_index::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_constEnd(
            self as *const crate::QListOfQModelIndex,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QModelIndex& QList<QModelIndex>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>().</p></div>
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_constFirst(
            self as *const crate::QListOfQModelIndex,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QModelIndex& QList<QModelIndex>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>().</p></div>
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_constLast(
            self as *const crate::QListOfQModelIndex,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QModelIndex>::contains(const QModelIndex& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn contains(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_contains(
            self as *const crate::QListOfQModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>& QList<QModelIndex>::operator=(const QList<QModelIndex>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this list and returns a reference to this list.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQModelIndex>>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_operator_(
            self as *mut crate::QListOfQModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQModelIndex>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of occurrences of <i>value</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QModelIndex>::count(const QModelIndex& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of occurrences of <i>value</i> in the list.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_count(
            self as *const crate::QListOfQModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QModelIndex>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_count1(
            self as *const crate::QListOfQModelIndex,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QModelIndex>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_detach(
            self as *mut crate::QListOfQModelIndex,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QModelIndex>::detachShared()```</span>.
    pub unsafe fn detach_shared(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_detachShared(
            self as *mut crate::QListOfQModelIndex,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QModelIndex>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_empty(
            self as *const crate::QListOfQModelIndex,
        )
    }

    /// <p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QModelIndex>::endsWith(const QModelIndex& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn ends_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_endsWith(
            self as *const crate::QListOfQModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::iterator QList<QModelIndex>::erase(QList<QModelIndex>::iterator pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn erase_1a(
        &mut self,
        pos: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_model_index::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_model_index::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_erase(self as *mut crate::QListOfQModelIndex, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_model_index::Iterator>>::cast_into(pos).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::iterator QList<QModelIndex>::erase(QList<QModelIndex>::iterator first, QList<QModelIndex>::iterator last)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes all the items from <i>begin</i> up to (but not including) <i>end</i>. Returns an iterator to the same item that <i>end</i> referred to before the call.</p></div>
    pub unsafe fn erase_2a(
        &mut self,
        first: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_model_index::Iterator>>,
        last: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_model_index::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_model_index::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_erase1(self as *mut crate::QListOfQModelIndex, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_model_index::Iterator>>::cast_into(first).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_model_index::Iterator>>::cast_into(last).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QModelIndex& QList<QModelIndex>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_first(
            self as *mut crate::QListOfQModelIndex,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QModelIndex& QList<QModelIndex>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_first1(
            self as *const crate::QListOfQModelIndex,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QModelIndex& QList<QModelIndex>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::MutRef<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_front(
            self as *mut crate::QListOfQModelIndex,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QModelIndex& QList<QModelIndex>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_front1(
            self as *const crate::QListOfQModelIndex,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QModelIndex& QList<QModelIndex>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>(). This function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p></div>
    pub unsafe fn index(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_operator__2(
            self as *const crate::QListOfQModelIndex,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QModelIndex& QList<QModelIndex>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If this function is called on a list that is currently being shared, it will trigger a copy of all elements. Otherwise, this function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>. If you do not want to modify the list you should use <a href="http://doc.qt.io/qt-5/qlist.html#at">QList::at</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_operator__3(
            self as *mut crate::QListOfQModelIndex,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QModelIndex>::indexOf(const QModelIndex& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_indexOf(
            self as *const crate::QListOfQModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(t)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QModelIndex>::indexOf(const QModelIndex& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_indexOf1(
            self as *const crate::QListOfQModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QModelIndex>::insert(int i, const QModelIndex& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   list<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// list: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn insert_int_q_model_index(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_insert(
            self as *mut crate::QListOfQModelIndex,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::iterator QList<QModelIndex>::insert(QList<QModelIndex>::iterator before, const QModelIndex& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>value</i> in front of the item pointed to by the iterator <i>before</i>. Returns an iterator pointing at the inserted item. Note that the iterator passed to the function will be invalid after the call; the returned iterator should be used instead.</p></div>
    pub unsafe fn insert_iterator_q_model_index(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_model_index::Iterator>>,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_model_index::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_insert1(self as *mut crate::QListOfQModelIndex, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_model_index::Iterator>>::cast_into(before).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(t).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QModelIndex>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_isDetached(
            self as *const crate::QListOfQModelIndex,
        )
    }

    /// <p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QModelIndex>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_isEmpty(
            self as *const crate::QListOfQModelIndex,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QModelIndex>::isSharedWith(const QList<QModelIndex>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_isSharedWith(
            self as *const crate::QListOfQModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQModelIndex>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QModelIndex& QList<QModelIndex>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_last(
            self as *mut crate::QListOfQModelIndex,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QModelIndex& QList<QModelIndex>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_last1(
            self as *const crate::QListOfQModelIndex,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QModelIndex>::lastIndexOf(const QModelIndex& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_lastIndexOf(
            self as *const crate::QListOfQModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(t)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QModelIndex>::lastIndexOf(const QModelIndex& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_lastIndexOf1(
            self as *const crate::QListOfQModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QModelIndex>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_length(
            self as *const crate::QListOfQModelIndex,
        )
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex> QList<QModelIndex>::mid(int pos, int length = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        length: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_mid(
            self as *const crate::QListOfQModelIndex,
            pos,
            length,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex> QList<QModelIndex>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_mid1(
            self as *const crate::QListOfQModelIndex,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QModelIndex>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>move(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "C", "D", "E", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This is the same as insert(<i>to</i>, <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<i>from</i>)).This function assumes that both <i>from</i> and <i>to</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>from</i> and <i>to</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_move(
            self as *mut crate::QListOfQModelIndex,
            from,
            to,
        )
    }

    /// <p>Constructs an empty list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QModelIndex>::QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty list.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QListOfQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_QList();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QModelIndex>::QList(const QList<QModelIndex>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_QList1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQModelIndex>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QModelIndex>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_pop_back(
            self as *mut crate::QListOfQModelIndex,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QModelIndex>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_pop_front(
            self as *mut crate::QListOfQModelIndex,
        )
    }

    /// <p>Inserts <i>value</i> at the beginning of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QModelIndex>::prepend(const QModelIndex& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(0, <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_prepend(
            self as *mut crate::QListOfQModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QModelIndex>::push_back(const QModelIndex& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_push_back(
            self as *mut crate::QListOfQModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QModelIndex>::push_front(const QModelIndex& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_push_front(
            self as *mut crate::QListOfQModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QModelIndex>::removeAll(const QModelIndex& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeAll(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_all(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_removeAll(
            self as *mut crate::QListOfQModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QModelIndex>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_removeAt(
            self as *mut crate::QListOfQModelIndex,
            i,
        )
    }

    /// <p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QModelIndex>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_removeFirst(
            self as *mut crate::QListOfQModelIndex,
        )
    }

    /// <p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QModelIndex>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_removeLast(
            self as *mut crate::QListOfQModelIndex,
        )
    }

    /// <p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QModelIndex>::removeOne(const QModelIndex& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeOne(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", ,"sun", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_one(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_removeOne(
            self as *mut crate::QListOfQModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QModelIndex>::replace(int i, const QModelIndex& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_replace(
            self as *mut crate::QListOfQModelIndex,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Reserve space for <i>alloc</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QModelIndex>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reserve space for <i>alloc</i> elements.</p>
    /// <p>If <i>alloc</i> is smaller than the current size of the list, nothing will happen.</p>
    /// <p>Use this function to avoid repetetive reallocation of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s internal data if you can predict how many elements will be appended. Note that the reservation applies only to the internal pointer array.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_reserve(
            self as *mut crate::QListOfQModelIndex,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QModelIndex>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_setSharable(
            self as *mut crate::QListOfQModelIndex,
            sharable,
        )
    }

    /// <p>Returns the number of items in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QModelIndex>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_size(
            self as *const crate::QListOfQModelIndex,
        )
    }

    /// <p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QModelIndex>::startsWith(const QModelIndex& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn starts_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_startsWith(
            self as *const crate::QListOfQModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QModelIndex>::swap(QList<QModelIndex>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap_1a(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QListOfQModelIndex>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_swap(
            self as *mut crate::QListOfQModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QListOfQModelIndex>>::cast_into(
                other,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QModelIndex>::swap(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swap(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    pub unsafe fn swap_2a(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_swap1(
            self as *mut crate::QListOfQModelIndex,
            i,
            j,
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QModelIndex>::swapItemsAt(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swapItemsAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swapItemsAt(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn swap_items_at(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_swapItemsAt(
            self as *mut crate::QListOfQModelIndex,
            i,
            j,
        )
    }

    /// <p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QModelIndex QList<QModelIndex>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_takeAt(
            self as *mut crate::QListOfQModelIndex,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QModelIndex QList<QModelIndex>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_takeFirst(
            self as *mut crate::QListOfQModelIndex,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QModelIndex QList<QModelIndex>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_takeLast(
            self as *mut crate::QListOfQModelIndex,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value at index position <i>i</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QModelIndex QList<QModelIndex>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the list.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that the index is going to be within bounds, you can use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_value(
            self as *const crate::QListOfQModelIndex,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QModelIndex QList<QModelIndex>::value(int i, const QModelIndex& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_value1(
            self as *const crate::QListOfQModelIndex,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_hash_of_int_q_byte_array {
    //! C++ type: <span style='color: green;'>```QHash<int, QByteArray>```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QHash<int, QByteArray>::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;Key, T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> (or <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>) and to modify the value (but not the key) associated with a particular key. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, you should use <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a>. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> function like <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>(), or <a href="http://doc.qt.io/qt-5/qhash.html#find">QHash::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a hash:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
    ///   hash<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
    ///   hash<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   hash<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> hash<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, which orders its items by key, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> stores its items in an arbitrary order. The only guarantee is that items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">QHash::insertMulti</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
    /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a>. Here's an example that increments every value stored in the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> by 2:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> hash<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     i<span class="operator">.</span>value() <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
    ///
    /// </pre>
    /// <p>Here's an example that removes all the items whose key is a string that starts with an underscore character:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> hash<span class="operator">.</span>begin();
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end()) {
    /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
    /// &#32;   &#32;     i <span class="operator">=</span> hash<span class="operator">.</span>erase(i);
    /// &#32;     <span class="keyword">else</span>
    /// &#32;   &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    ///   }
    ///
    /// </pre>
    /// <p>The call to <a href="http://doc.qt.io/qt-5/qhash.html#erase">QHash::erase</a>() removes the item pointed to by the iterator from the hash, and returns an iterator to the next item. Here's another way of removing an item while iterating:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> hash<span class="operator">.</span>begin();
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end()) {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator prev <span class="operator">=</span> i;
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    /// &#32;     <span class="keyword">if</span> (prev<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
    /// &#32;   &#32;     hash<span class="operator">.</span>erase(prev);
    ///   }
    ///
    /// </pre>
    /// <p>It might be tempting to write code like this:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="comment">// WRONG</span>
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end()) {
    /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
    /// &#32;   &#32;     hash<span class="operator">.</span>erase(i);
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    ///   }
    ///
    /// </pre>
    /// <p>However, this will potentially crash in <code>++i</code>, because <code>i</code> is a dangling iterator after the call to <a href="http://doc.qt.io/qt-5/qhash.html#erase">erase</a>().</p>
    /// <p>Multiple iterators can be used on the same hash. However, be aware that any modification performed directly on the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> has the potential of dramatically changing the order in which the items are stored in the hash, as they might cause <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> to rehash its internal data structure. There is one notable exception: <a href="http://doc.qt.io/qt-5/qhash.html#erase">QHash::erase</a>(). This function can safely be called while iterating, and won't affect the order of items in the hash. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> rather than <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::iterator& QHash<int, QByteArray>::iterator::operator=(const QHash<int, QByteArray>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;Key, T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> (or <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>) and to modify the value (but not the key) associated with a particular key. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, you should use <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a>. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> function like <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>(), or <a href="http://doc.qt.io/qt-5/qhash.html#find">QHash::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a hash:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
        ///   hash<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
        ///   hash<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   hash<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> hash<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Unlike <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, which orders its items by key, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> stores its items in an arbitrary order. The only guarantee is that items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">QHash::insertMulti</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a>. Here's an example that increments every value stored in the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> hash<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     i<span class="operator">.</span>value() <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Here's an example that removes all the items whose key is a string that starts with an underscore character:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> hash<span class="operator">.</span>begin();
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end()) {
        /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     i <span class="operator">=</span> hash<span class="operator">.</span>erase(i);
        /// &#32;     <span class="keyword">else</span>
        /// &#32;   &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        ///   }
        ///
        /// </pre>
        /// <p>The call to <a href="http://doc.qt.io/qt-5/qhash.html#erase">QHash::erase</a>() removes the item pointed to by the iterator from the hash, and returns an iterator to the next item. Here's another way of removing an item while iterating:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> hash<span class="operator">.</span>begin();
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end()) {
        /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator prev <span class="operator">=</span> i;
        /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        /// &#32;     <span class="keyword">if</span> (prev<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     hash<span class="operator">.</span>erase(prev);
        ///   }
        ///
        /// </pre>
        /// <p>It might be tempting to write code like this:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="comment">// WRONG</span>
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end()) {
        /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     hash<span class="operator">.</span>erase(i);
        /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        ///   }
        ///
        /// </pre>
        /// <p>However, this will potentially crash in <code>++i</code>, because <code>i</code> is a dangling iterator after the call to <a href="http://doc.qt.io/qt-5/qhash.html#erase">erase</a>().</p>
        /// <p>Multiple iterators can be used on the same hash. However, be aware that any modification performed directly on the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> has the potential of dramatically changing the order in which the items are stored in the hash, as they might cause <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> to rehash its internal data structure. There is one notable exception: <a href="http://doc.qt.io/qt-5/qhash.html#erase">QHash::erase</a>(). This function can safely be called while iterating, and won't affect the order of items in the hash. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> rather than <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::Iterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_hash_of_int_q_byte_array::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_iterator_operator_3(
                self as *mut crate::q_hash_of_int_q_byte_array::Iterator,
                ::cpp_utils::CastInto::<
                    ::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::Iterator>,
                >::cast_into(other)
                .as_raw_ptr(),
            );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::iterator QHash<int, QByteArray>::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_iterator_operator__6(
                    self as *mut crate::q_hash_of_int_q_byte_array::Iterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::iterator QHash<int, QByteArray>::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_iterator_operator__4(
                    self as *mut crate::q_hash_of_int_q_byte_array::Iterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the current item's key as a const reference.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const int& QHash<int, QByteArray>::iterator::key() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current item's key as a const reference.</p>
        /// <p>There is no direct way of changing an item's key through an iterator, although it can be done by calling <a href="http://doc.qt.io/qt-5/qhash.html#erase">QHash::erase</a>() followed by <a href="http://doc.qt.io/qt-5/qhash.html#insert">QHash::insert</a>() or <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">QHash::insertMulti</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn key(&self) -> ::cpp_utils::Ref<::std::os::raw::c_int> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_iterator_key(
                self as *const crate::q_hash_of_int_q_byte_array::Iterator,
            );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QHash<int, QByteArray>::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like <a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>(), <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>(), and operator++() must not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>().</p></div>
        pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QHash<int, QByteArray>::iterator::iterator(void* node)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#iterator">C++ documentation</a> for <span style='color: green;'>```iterator::iterator()```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like <a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>(), <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>(), and operator++() must not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>().</p></div>
        pub unsafe fn new_1a(
            node: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_iterator_iterator1(
                ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(node)
                    .as_mut_raw_ptr(),
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>The <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QHash<int, QByteArray>::iterator::iterator(const QHash<int, QByteArray>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;Key, T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> (or <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>) and to modify the value (but not the key) associated with a particular key. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, you should use <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a>. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> function like <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>(), or <a href="http://doc.qt.io/qt-5/qhash.html#find">QHash::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a hash:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
        ///   hash<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
        ///   hash<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   hash<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> hash<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Unlike <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, which orders its items by key, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> stores its items in an arbitrary order. The only guarantee is that items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">QHash::insertMulti</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a>. Here's an example that increments every value stored in the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> hash<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     i<span class="operator">.</span>value() <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Here's an example that removes all the items whose key is a string that starts with an underscore character:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> hash<span class="operator">.</span>begin();
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end()) {
        /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     i <span class="operator">=</span> hash<span class="operator">.</span>erase(i);
        /// &#32;     <span class="keyword">else</span>
        /// &#32;   &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        ///   }
        ///
        /// </pre>
        /// <p>The call to <a href="http://doc.qt.io/qt-5/qhash.html#erase">QHash::erase</a>() removes the item pointed to by the iterator from the hash, and returns an iterator to the next item. Here's another way of removing an item while iterating:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> hash<span class="operator">.</span>begin();
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end()) {
        /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator prev <span class="operator">=</span> i;
        /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        /// &#32;     <span class="keyword">if</span> (prev<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     hash<span class="operator">.</span>erase(prev);
        ///   }
        ///
        /// </pre>
        /// <p>It might be tempting to write code like this:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="comment">// WRONG</span>
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end()) {
        /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     hash<span class="operator">.</span>erase(i);
        /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        ///   }
        ///
        /// </pre>
        /// <p>However, this will potentially crash in <code>++i</code>, because <code>i</code> is a dangling iterator after the call to <a href="http://doc.qt.io/qt-5/qhash.html#erase">erase</a>().</p>
        /// <p>Multiple iterators can be used on the same hash. However, be aware that any modification performed directly on the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> has the potential of dramatically changing the order in which the items are stored in the hash, as they might cause <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> to rehash its internal data structure. There is one notable exception: <a href="http://doc.qt.io/qt-5/qhash.html#erase">QHash::erase</a>(). This function can safely be called while iterating, and won't affect the order of items in the hash. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> rather than <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn new_copy(
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::Iterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_iterator_iterator2(
                ::cpp_utils::CastInto::<
                    ::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::Iterator>,
                >::cast_into(other)
                .as_raw_ptr(),
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QByteArray* QHash<int, QByteArray>::iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's value.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::MutPtr<crate::QByteArray> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_iterator_operator__(
                self as *const crate::q_hash_of_int_q_byte_array::Iterator,
            );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }

        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QByteArray& QHash<int, QByteArray>::iterator::value() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>You can change the value of an item by using value() on the left side of an assignment, for example:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="keyword">if</span> (i<span class="operator">.</span>key() <span class="operator">=</span><span class="operator">=</span> <span class="string">"Hello"</span>)
        /// &#32;     i<span class="operator">.</span>value() <span class="operator">=</span> <span class="string">"Bonjour"</span>;
        ///
        /// </pre>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">operator*</a>().</p></div>
        pub unsafe fn value(&self) -> ::cpp_utils::MutRef<crate::QByteArray> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_iterator_value(
                self as *const crate::q_hash_of_int_q_byte_array::Iterator,
            );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QHash<int, QByteArray>::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;Key, T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> (or a <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>). If you want to modify the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as you iterate over it, you must use <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> function like <a href="http://doc.qt.io/qt-5/qhash.html#constBegin">QHash::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#constEnd">QHash::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qhash.html#find">QHash::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a hash:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
    ///   hash<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
    ///   hash<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   hash<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> hash<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, which orders its items by key, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> stores its items in an arbitrary order. The only guarantee is that items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">QHash::insertMulti</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
    /// <p>Multiple iterators can be used on the same hash. However, be aware that any modification performed directly on the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> has the potential of dramatically changing the order in which the items are stored in the hash, as they might cause <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> to rehash its internal data structure. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> rather than <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::const_iterator& QHash<int, QByteArray>::const_iterator::operator=(const QHash<int, QByteArray>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;Key, T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> (or a <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>). If you want to modify the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as you iterate over it, you must use <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> function like <a href="http://doc.qt.io/qt-5/qhash.html#constBegin">QHash::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#constEnd">QHash::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qhash.html#find">QHash::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a hash:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
        ///   hash<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
        ///   hash<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   hash<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> hash<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Unlike <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, which orders its items by key, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> stores its items in an arbitrary order. The only guarantee is that items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">QHash::insertMulti</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
        /// <p>Multiple iterators can be used on the same hash. However, be aware that any modification performed directly on the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> has the potential of dramatically changing the order in which the items are stored in the hash, as they might cause <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> to rehash its internal data structure. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> rather than <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::ConstIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_hash_of_int_q_byte_array::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_const_iterator_operator_3(
                    self as *mut crate::q_hash_of_int_q_byte_array::ConstIterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::ConstIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::const_iterator QHash<int, QByteArray>::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_const_iterator_operator__6(
                    self as *mut crate::q_hash_of_int_q_byte_array::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::const_iterator QHash<int, QByteArray>::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_const_iterator_operator__4(
                    self as *mut crate::q_hash_of_int_q_byte_array::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the current key, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const int& QHash<int, QByteArray>::const_iterator::key() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#key">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current key, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        pub unsafe fn key(&self) -> ::cpp_utils::Ref<::std::os::raw::c_int> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_const_iterator_key(
                self as *const crate::q_hash_of_int_q_byte_array::ConstIterator,
            );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QHash<int, QByteArray>::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html#key">key</a>(), <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html#value">value</a>(), and operator++() must not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#constBegin">QHash::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#constEnd">QHash::constEnd</a>().</p></div>
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::ConstIterator>
        {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_const_iterator_const_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QHash<int, QByteArray>::const_iterator::const_iterator(void* node)```</span>.
        pub unsafe fn from_void(
            node: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_const_iterator_const_iterator1(
                    ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(
                        node,
                    )
                    .as_mut_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs a copy of <i>other</i>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QHash<int, QByteArray>::const_iterator::const_iterator(const QHash<int, QByteArray>::iterator& o)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html#const_iterator-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p></div>
        pub unsafe fn from_iterator(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_const_iterator_const_iterator2(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::Iterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>The <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QHash<int, QByteArray>::const_iterator::const_iterator(const QHash<int, QByteArray>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;Key, T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> (or a <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>). If you want to modify the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as you iterate over it, you must use <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> function like <a href="http://doc.qt.io/qt-5/qhash.html#constBegin">QHash::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#constEnd">QHash::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qhash.html#find">QHash::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a hash:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
        ///   hash<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
        ///   hash<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   hash<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> hash<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Unlike <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, which orders its items by key, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> stores its items in an arbitrary order. The only guarantee is that items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">QHash::insertMulti</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
        /// <p>Multiple iterators can be used on the same hash. However, be aware that any modification performed directly on the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> has the potential of dramatically changing the order in which the items are stored in the hash, as they might cause <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> to rehash its internal data structure. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> rather than <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn new_copy(
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::ConstIterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_const_iterator_const_iterator3(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::ConstIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current result.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QByteArray* QHash<int, QByteArray>::const_iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current result.</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::Ptr<crate::QByteArray> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_const_iterator_operator__(
                    self as *const crate::q_hash_of_int_q_byte_array::ConstIterator,
                );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }

        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QByteArray& QHash<int, QByteArray>::const_iterator::value() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        pub unsafe fn value(&self) -> ::cpp_utils::Ref<crate::QByteArray> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_const_iterator_value(
                    self as *const crate::q_hash_of_int_q_byte_array::ConstIterator,
                );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a> keys.</p>
    ///
    /// C++ class: <span style='color: green;'>```QHash<int, QByteArray>::key_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a> keys.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> is essentially the same as <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> with the difference that operator*() and operator-&gt;() return a key instead of a value.</p>
    /// <p>For most uses <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> and <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> should be used, you can easily access the key by calling <a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">QHash::iterator::key</a>():</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">for</span> (<span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator it <span class="operator">=</span> hash<span class="operator">.</span>cbegin()<span class="operator">,</span> end <span class="operator">=</span> hash<span class="operator">.</span>cend(); it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The key: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Also the value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> (<span class="operator">*</span>it) <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///   }
    ///
    /// </pre>
    /// <p>However, to have interoperability between <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s keys and STL-style algorithms we need an iterator that dereferences to a key instead of a value. With <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> we can apply an algorithm to a range of keys without having to call <a href="http://doc.qt.io/qt-5/qhash.html#keys">QHash::keys</a>(), which is inefficient as it costs one <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> iteration and memory allocation to create a temporary <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="comment">// Inefficient, keys() is expensive</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> keys <span class="operator">=</span> hash<span class="operator">.</span>keys();
    ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(keys<span class="operator">.</span>cbegin()<span class="operator">,</span> keys<span class="operator">.</span>cend()<span class="operator">,</span> isPrimeNumber);
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(hash2<span class="operator">.</span>keys());
    ///
    ///   <span class="comment">// Efficient, no memory allocation needed</span>
    ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(hash<span class="operator">.</span>keyBegin()<span class="operator">,</span> hash<span class="operator">.</span>keyEnd()<span class="operator">,</span> isPrimeNumber);
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(hash2<span class="operator">.</span>keyBegin()<span class="operator">,</span> hash2<span class="operator">.</span>keyEnd());
    ///
    /// </pre>
    /// <p><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> is const, it's not possible to modify the key.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> function like <a href="http://doc.qt.io/qt-5/qhash.html#keyBegin">QHash::keyBegin</a>() or <a href="http://doc.qt.io/qt-5/qhash.html#keyEnd">QHash::keyEnd</a>().</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct KeyIterator {
        _unused: u8,
    }
    impl KeyIterator {
        /// <p>Returns the underlying <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">const_iterator</a> this <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">key_iterator</a> is based on.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::const_iterator QHash<int, QByteArray>::key_iterator::base() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#base">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the underlying <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">const_iterator</a> this <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">key_iterator</a> is based on.</p></div>
        pub unsafe fn base(
            &self,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_key_iterator_base(
                self as *const crate::q_hash_of_int_q_byte_array::KeyIterator,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>The <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a> keys.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::key_iterator& QHash<int, QByteArray>::key_iterator::operator=(const QHash<int, QByteArray>::key_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a> keys.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> is essentially the same as <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> with the difference that operator*() and operator-&gt;() return a key instead of a value.</p>
        /// <p>For most uses <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> and <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> should be used, you can easily access the key by calling <a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">QHash::iterator::key</a>():</p>
        /// <pre class="cpp">
        ///
        ///   <span class="keyword">for</span> (<span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator it <span class="operator">=</span> hash<span class="operator">.</span>cbegin()<span class="operator">,</span> end <span class="operator">=</span> hash<span class="operator">.</span>cend(); it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The key: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Also the value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> (<span class="operator">*</span>it) <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///   }
        ///
        /// </pre>
        /// <p>However, to have interoperability between <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s keys and STL-style algorithms we need an iterator that dereferences to a key instead of a value. With <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> we can apply an algorithm to a range of keys without having to call <a href="http://doc.qt.io/qt-5/qhash.html#keys">QHash::keys</a>(), which is inefficient as it costs one <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> iteration and memory allocation to create a temporary <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <pre class="cpp">
        ///
        ///   <span class="comment">// Inefficient, keys() is expensive</span>
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> keys <span class="operator">=</span> hash<span class="operator">.</span>keys();
        ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(keys<span class="operator">.</span>cbegin()<span class="operator">,</span> keys<span class="operator">.</span>cend()<span class="operator">,</span> isPrimeNumber);
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(hash2<span class="operator">.</span>keys());
        ///
        ///   <span class="comment">// Efficient, no memory allocation needed</span>
        ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(hash<span class="operator">.</span>keyBegin()<span class="operator">,</span> hash<span class="operator">.</span>keyEnd()<span class="operator">,</span> isPrimeNumber);
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(hash2<span class="operator">.</span>keyBegin()<span class="operator">,</span> hash2<span class="operator">.</span>keyEnd());
        ///
        /// </pre>
        /// <p><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> is const, it's not possible to modify the key.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> function like <a href="http://doc.qt.io/qt-5/qhash.html#keyBegin">QHash::keyBegin</a>() or <a href="http://doc.qt.io/qt-5/qhash.html#keyEnd">QHash::keyEnd</a>().</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::KeyIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_hash_of_int_q_byte_array::KeyIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_key_iterator_operator_1(
                    self as *mut crate::q_hash_of_int_q_byte_array::KeyIterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::KeyIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::key_iterator QHash<int, QByteArray>::key_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previous item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::KeyIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_key_iterator_operator__6(
                    self as *mut crate::q_hash_of_int_q_byte_array::KeyIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::key_iterator QHash<int, QByteArray>::key_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previous item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::KeyIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_key_iterator_operator__4(
                    self as *mut crate::q_hash_of_int_q_byte_array::KeyIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Default constructs an instance of key_iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QHash<int, QByteArray>::key_iterator::key_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#key_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Default constructs an instance of key_iterator.</p></div>
        pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::KeyIterator>
        {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_key_iterator_key_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QHash<int, QByteArray>::key_iterator::key_iterator(QHash<int, QByteArray>::const_iterator o)```</span>.
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::ConstIterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::KeyIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_key_iterator_key_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::ConstIterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>The <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a> keys.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QHash<int, QByteArray>::key_iterator::key_iterator(const QHash<int, QByteArray>::key_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a> keys.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> is essentially the same as <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> with the difference that operator*() and operator-&gt;() return a key instead of a value.</p>
        /// <p>For most uses <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> and <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> should be used, you can easily access the key by calling <a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">QHash::iterator::key</a>():</p>
        /// <pre class="cpp">
        ///
        ///   <span class="keyword">for</span> (<span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator it <span class="operator">=</span> hash<span class="operator">.</span>cbegin()<span class="operator">,</span> end <span class="operator">=</span> hash<span class="operator">.</span>cend(); it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The key: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Also the value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> (<span class="operator">*</span>it) <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///   }
        ///
        /// </pre>
        /// <p>However, to have interoperability between <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s keys and STL-style algorithms we need an iterator that dereferences to a key instead of a value. With <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> we can apply an algorithm to a range of keys without having to call <a href="http://doc.qt.io/qt-5/qhash.html#keys">QHash::keys</a>(), which is inefficient as it costs one <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> iteration and memory allocation to create a temporary <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <pre class="cpp">
        ///
        ///   <span class="comment">// Inefficient, keys() is expensive</span>
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> keys <span class="operator">=</span> hash<span class="operator">.</span>keys();
        ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(keys<span class="operator">.</span>cbegin()<span class="operator">,</span> keys<span class="operator">.</span>cend()<span class="operator">,</span> isPrimeNumber);
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(hash2<span class="operator">.</span>keys());
        ///
        ///   <span class="comment">// Efficient, no memory allocation needed</span>
        ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(hash<span class="operator">.</span>keyBegin()<span class="operator">,</span> hash<span class="operator">.</span>keyEnd()<span class="operator">,</span> isPrimeNumber);
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(hash2<span class="operator">.</span>keyBegin()<span class="operator">,</span> hash2<span class="operator">.</span>keyEnd());
        ///
        /// </pre>
        /// <p><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> is const, it's not possible to modify the key.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> function like <a href="http://doc.qt.io/qt-5/qhash.html#keyBegin">QHash::keyBegin</a>() or <a href="http://doc.qt.io/qt-5/qhash.html#keyEnd">QHash::keyEnd</a>().</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn new_copy(
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::KeyIterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::KeyIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_key_iterator_key_iterator2(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::KeyIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's key.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const int* QHash<int, QByteArray>::key_iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's key.</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_int> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_key_iterator_operator__(
                    self as *const crate::q_hash_of_int_q_byte_array::KeyIterator,
                );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_hash_of_int_q_byte_array::Iterator {
        type Output = ::cpp_utils::MutRef<crate::QByteArray>;
        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QByteArray& QHash<int, QByteArray>::iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>().</p></div>
        fn indirection(self) -> ::cpp_utils::MutRef<crate::QByteArray> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_iterator_operator_(
                    self as *const crate::q_hash_of_int_q_byte_array::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::Iterator>>
        for crate::q_hash_of_int_q_byte_array::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QHash<int, QByteArray>::iterator::operator==(const QHash<int, QByteArray>::iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_iterator_operator__1(
                    self as *const crate::q_hash_of_int_q_byte_array::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_hash_of_int_q_byte_array::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_hash_of_int_q_byte_array::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::iterator& QHash<int, QByteArray>::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_hash_of_int_q_byte_array::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_iterator_operator__3(
                    self as *mut crate::q_hash_of_int_q_byte_array::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_hash_of_int_q_byte_array::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_hash_of_int_q_byte_array::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::iterator& QHash<int, QByteArray>::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_hash_of_int_q_byte_array::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_iterator_operator__5(
                    self as *mut crate::q_hash_of_int_q_byte_array::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_int> for &crate::q_hash_of_int_q_byte_array::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::Iterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions forward from this iterator. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::iterator QHash<int, QByteArray>::iterator::operator+(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions forward from this iterator. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p>This operation can be slow for large <i>j</i> values.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn add(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_iterator_operator_1(
                    self as *const crate::q_hash_of_int_q_byte_array::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_int> for &crate::q_hash_of_int_q_byte_array::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::Iterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions backward from this iterator. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::iterator QHash<int, QByteArray>::iterator::operator-(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions backward from this iterator. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p>This operation can be slow for large <i>j</i> values.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn sub(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_iterator_operator_2(
                    self as *const crate::q_hash_of_int_q_byte_array::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_int> for crate::q_hash_of_int_q_byte_array::Iterator {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::iterator& QHash<int, QByteArray>::iterator::operator+=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_iterator_operator__7(
                    self as *mut crate::q_hash_of_int_q_byte_array::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_int> for crate::q_hash_of_int_q_byte_array::Iterator {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::iterator& QHash<int, QByteArray>::iterator::operator-=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_iterator_operator__8(
                    self as *mut crate::q_hash_of_int_q_byte_array::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::ConstIterator>>
        for crate::q_hash_of_int_q_byte_array::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QHash<int, QByteArray>::iterator::operator==(const QHash<int, QByteArray>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_iterator_operator__9(
                    self as *const crate::q_hash_of_int_q_byte_array::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_hash_of_int_q_byte_array::ConstIterator {
        type Output = ::cpp_utils::Ref<crate::QByteArray>;
        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QByteArray& QHash<int, QByteArray>::const_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        fn indirection(self) -> ::cpp_utils::Ref<crate::QByteArray> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_const_iterator_operator_(
                    self as *const crate::q_hash_of_int_q_byte_array::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::ConstIterator>>
        for crate::q_hash_of_int_q_byte_array::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QHash<int, QByteArray>::const_iterator::operator==(const QHash<int, QByteArray>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_const_iterator_operator__1(
                    self as *const crate::q_hash_of_int_q_byte_array::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_hash_of_int_q_byte_array::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_hash_of_int_q_byte_array::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::const_iterator& QHash<int, QByteArray>::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_hash_of_int_q_byte_array::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_const_iterator_operator__3(
                    self as *mut crate::q_hash_of_int_q_byte_array::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_hash_of_int_q_byte_array::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_hash_of_int_q_byte_array::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::const_iterator& QHash<int, QByteArray>::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_hash_of_int_q_byte_array::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_const_iterator_operator__5(
                    self as *mut crate::q_hash_of_int_q_byte_array::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_int> for &crate::q_hash_of_int_q_byte_array::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::ConstIterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions forward from this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::const_iterator QHash<int, QByteArray>::const_iterator::operator+(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions forward from this iterator.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>().</p></div>
        fn add(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_const_iterator_operator_1(
                    self as *const crate::q_hash_of_int_q_byte_array::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_int> for &crate::q_hash_of_int_q_byte_array::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::ConstIterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions backward from this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::const_iterator QHash<int, QByteArray>::const_iterator::operator-(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions backward from this iterator.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>().</p></div>
        fn sub(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_const_iterator_operator_2(
                    self as *const crate::q_hash_of_int_q_byte_array::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_int>
        for crate::q_hash_of_int_q_byte_array::ConstIterator
    {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::const_iterator& QHash<int, QByteArray>::const_iterator::operator+=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_const_iterator_operator__7(
                    self as *mut crate::q_hash_of_int_q_byte_array::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_int>
        for crate::q_hash_of_int_q_byte_array::ConstIterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::const_iterator& QHash<int, QByteArray>::const_iterator::operator-=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_const_iterator_operator__8(
                    self as *mut crate::q_hash_of_int_q_byte_array::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_hash_of_int_q_byte_array::KeyIterator {
        type Output = ::cpp_utils::Ref<::std::os::raw::c_int>;
        /// <p>Returns the current item's key.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const int& QHash<int, QByteArray>::key_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current item's key.</p></div>
        fn indirection(self) -> ::cpp_utils::Ref<::std::os::raw::c_int> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_key_iterator_operator_(
                    self as *const crate::q_hash_of_int_q_byte_array::KeyIterator,
                )
            };
            unsafe {
                ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::KeyIterator>>
        for crate::q_hash_of_int_q_byte_array::KeyIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QHash<int, QByteArray>::key_iterator::operator==(QHash<int, QByteArray>::key_iterator o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::KeyIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_key_iterator_operator__1(
                    self as *const crate::q_hash_of_int_q_byte_array::KeyIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_hash_of_int_q_byte_array::KeyIterator {
        type Output = ::cpp_utils::MutRef<crate::q_hash_of_int_q_byte_array::KeyIterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::key_iterator& QHash<int, QByteArray>::key_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#keyEnd">QHash::keyEnd</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_hash_of_int_q_byte_array::KeyIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_key_iterator_operator__3(
                    self as *mut crate::q_hash_of_int_q_byte_array::KeyIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_hash_of_int_q_byte_array::KeyIterator {
        type Output = ::cpp_utils::MutRef<crate::q_hash_of_int_q_byte_array::KeyIterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::key_iterator& QHash<int, QByteArray>::key_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#keyBegin">QHash::keyBegin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_hash_of_int_q_byte_array::KeyIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_key_iterator_operator__5(
                    self as *mut crate::q_hash_of_int_q_byte_array::KeyIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_hash_of_int_q_byte_array::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QHash<int, QByteArray>::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;Key, T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> (or <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>) and to modify the value (but not the key) associated with a particular key. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, you should use <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a>. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> function like <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>(), or <a href="http://doc.qt.io/qt-5/qhash.html#find">QHash::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a hash:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
        ///   hash<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
        ///   hash<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   hash<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> hash<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Unlike <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, which orders its items by key, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> stores its items in an arbitrary order. The only guarantee is that items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">QHash::insertMulti</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a>. Here's an example that increments every value stored in the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> hash<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     i<span class="operator">.</span>value() <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Here's an example that removes all the items whose key is a string that starts with an underscore character:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> hash<span class="operator">.</span>begin();
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end()) {
        /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     i <span class="operator">=</span> hash<span class="operator">.</span>erase(i);
        /// &#32;     <span class="keyword">else</span>
        /// &#32;   &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        ///   }
        ///
        /// </pre>
        /// <p>The call to <a href="http://doc.qt.io/qt-5/qhash.html#erase">QHash::erase</a>() removes the item pointed to by the iterator from the hash, and returns an iterator to the next item. Here's another way of removing an item while iterating:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> hash<span class="operator">.</span>begin();
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end()) {
        /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator prev <span class="operator">=</span> i;
        /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        /// &#32;     <span class="keyword">if</span> (prev<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     hash<span class="operator">.</span>erase(prev);
        ///   }
        ///
        /// </pre>
        /// <p>It might be tempting to write code like this:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="comment">// WRONG</span>
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end()) {
        /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>key()<span class="operator">.</span>startsWith(<span class="char">'_'</span>))
        /// &#32;   &#32;     hash<span class="operator">.</span>erase(i);
        /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        ///   }
        ///
        /// </pre>
        /// <p>However, this will potentially crash in <code>++i</code>, because <code>i</code> is a dangling iterator after the call to <a href="http://doc.qt.io/qt-5/qhash.html#erase">erase</a>().</p>
        /// <p>Multiple iterators can be used on the same hash. However, be aware that any modification performed directly on the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> has the potential of dramatically changing the order in which the items are stored in the hash, as they might cause <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> to rehash its internal data structure. There is one notable exception: <a href="http://doc.qt.io/qt-5/qhash.html#erase">QHash::erase</a>(). This function can safely be called while iterating, and won't affect the order of items in the hash. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> rather than <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_iterator_diterator(
                self as *mut crate::q_hash_of_int_q_byte_array::Iterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_hash_of_int_q_byte_array::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QHash<int, QByteArray>::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;Key, T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> (or a <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>). If you want to modify the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as you iterate over it, you must use <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> function like <a href="http://doc.qt.io/qt-5/qhash.html#constBegin">QHash::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#constEnd">QHash::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qhash.html#find">QHash::find</a>() before you can start iterating. Here's a typical loop that prints all the (key, value) pairs stored in a hash:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
        ///   hash<span class="operator">.</span>insert(<span class="string">"January"</span><span class="operator">,</span> <span class="number">1</span>);
        ///   hash<span class="operator">.</span>insert(<span class="string">"February"</span><span class="operator">,</span> <span class="number">2</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   hash<span class="operator">.</span>insert(<span class="string">"December"</span><span class="operator">,</span> <span class="number">12</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> hash<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Unlike <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, which orders its items by key, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> stores its items in an arbitrary order. The only guarantee is that items that share the same key (because they were inserted using <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">QHash::insertMulti</a>()) will appear consecutively, from the most recently to the least recently inserted value.</p>
        /// <p>Multiple iterators can be used on the same hash. However, be aware that any modification performed directly on the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> has the potential of dramatically changing the order in which the items are stored in the hash, as they might cause <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> to rehash its internal data structure. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> rather than <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_const_iterator_dconst_iterator(
                self as *mut crate::q_hash_of_int_q_byte_array::ConstIterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_hash_of_int_q_byte_array::KeyIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a> keys.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QHash<int, QByteArray>::key_iterator::~key_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> and <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a> keys.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> is essentially the same as <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> with the difference that operator*() and operator-&gt;() return a key instead of a value.</p>
        /// <p>For most uses <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a> and <a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> should be used, you can easily access the key by calling <a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">QHash::iterator::key</a>():</p>
        /// <pre class="cpp">
        ///
        ///   <span class="keyword">for</span> (<span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator it <span class="operator">=</span> hash<span class="operator">.</span>cbegin()<span class="operator">,</span> end <span class="operator">=</span> hash<span class="operator">.</span>cend(); it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The key: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Also the value: "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> (<span class="operator">*</span>it) <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///   }
        ///
        /// </pre>
        /// <p>However, to have interoperability between <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s keys and STL-style algorithms we need an iterator that dereferences to a key instead of a value. With <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> we can apply an algorithm to a range of keys without having to call <a href="http://doc.qt.io/qt-5/qhash.html#keys">QHash::keys</a>(), which is inefficient as it costs one <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> iteration and memory allocation to create a temporary <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <pre class="cpp">
        ///
        ///   <span class="comment">// Inefficient, keys() is expensive</span>
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> keys <span class="operator">=</span> hash<span class="operator">.</span>keys();
        ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(keys<span class="operator">.</span>cbegin()<span class="operator">,</span> keys<span class="operator">.</span>cend()<span class="operator">,</span> isPrimeNumber);
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(hash2<span class="operator">.</span>keys());
        ///
        ///   <span class="comment">// Efficient, no memory allocation needed</span>
        ///   <span class="type">int</span> numPrimes <span class="operator">=</span> std<span class="operator">::</span>count_if(hash<span class="operator">.</span>keyBegin()<span class="operator">,</span> hash<span class="operator">.</span>keyEnd()<span class="operator">,</span> isPrimeNumber);
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(hash2<span class="operator">.</span>keyBegin()<span class="operator">,</span> hash2<span class="operator">.</span>keyEnd());
        ///
        /// </pre>
        /// <p><a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> is const, it's not possible to modify the key.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qhash-key-iterator.html">QHash::key_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> function like <a href="http://doc.qt.io/qt-5/qhash.html#keyBegin">QHash::keyBegin</a>() or <a href="http://doc.qt.io/qt-5/qhash.html#keyEnd">QHash::keyEnd</a>().</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_key_iterator_dkey_iterator(
                self as *mut crate::q_hash_of_int_q_byte_array::KeyIterator,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> class is a template class that provides a hash-table-based dictionary.</p>
///
/// C++ class: <span style='color: green;'>```QHash<int, QByteArray>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> class is a template class that provides a hash-table-based dictionary.</p>
/// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;Key, T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores (key, value) pairs and provides very fast lookup of the value associated with a key.</p>
/// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> provides very similar functionality to <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>. The differences are:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> provides faster lookups than <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.)</li>
/// <li>When iterating over a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, the items are always sorted by key. With <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, the items are arbitrarily ordered.</li>
/// <li>The key type of a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> must provide operator&lt;(). The key type of a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> must provide operator==() and a global hash function called <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() (see <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a>).</li>
/// </ul>
/// <p>Here's an example <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> with <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> keys and <code>int</code> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
///
/// </pre>
/// <p>To insert a (key, value) pair into the hash, you can use operator[]():</p>
/// <pre class="cpp">
///
///   hash<span class="operator">[</span><span class="string">"one"</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>;
///   hash<span class="operator">[</span><span class="string">"three"</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">3</span>;
///   hash<span class="operator">[</span><span class="string">"seven"</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">7</span>;
///
/// </pre>
/// <p>This inserts the following three (key, value) pairs into the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>: ("one", 1), ("three", 3), and ("seven", 7). Another way to insert items into the hash is to use <a href="http://doc.qt.io/qt-5/qhash.html#insert">insert</a>():</p>
/// <pre class="cpp">
///
///   hash<span class="operator">.</span>insert(<span class="string">"twelve"</span><span class="operator">,</span> <span class="number">12</span>);
///
/// </pre>
/// <p>To look up a value, use operator[]() or <a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>():</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> num1 <span class="operator">=</span> hash<span class="operator">[</span><span class="string">"thirteen"</span><span class="operator">]</span>;
///   <span class="type">int</span> num2 <span class="operator">=</span> hash<span class="operator">.</span>value(<span class="string">"thirteen"</span>);
///
/// </pre>
/// <p>If there is no item with the specified key in the hash, these functions return a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>.</p>
/// <p>If you want to check whether the hash contains a particular key, use <a href="http://doc.qt.io/qt-5/qhash.html#contains">contains</a>():</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> timeout <span class="operator">=</span> <span class="number">30</span>;
///   <span class="keyword">if</span> (hash<span class="operator">.</span>contains(<span class="string">"TIMEOUT"</span>))
/// &#32;     timeout <span class="operator">=</span> hash<span class="operator">.</span>value(<span class="string">"TIMEOUT"</span>);
///
/// </pre>
/// <p>There is also a <a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>() overload that uses its second argument as a default value if there is no item with the specified key:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> timeout <span class="operator">=</span> hash<span class="operator">.</span>value(<span class="string">"TIMEOUT"</span><span class="operator">,</span> <span class="number">30</span>);
///
/// </pre>
/// <p>In general, we recommend that you use <a href="http://doc.qt.io/qt-5/qhash.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>() rather than operator[]() for looking up a key in a hash. The reason is that operator[]() silently inserts an item into the hash if no item exists with the same key (unless the hash is const). For example, the following code snippet will create 1000 items in memory:</p>
/// <pre class="cpp">
///
///   <span class="comment">// WRONG</span>
///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> hash;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> <span class="number">1000</span>; <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (hash<span class="operator">[</span>i<span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> okButton)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found button at index "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p>To avoid this problem, replace <code>hash[i]</code> with <code>hash.value(i)</code> in the code above.</p>
/// <p>Internally, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> uses a hash table to perform lookups. Unlike Qt 3's <code>QDict</code> class, which needed to be initialized with a prime number, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s hash table automatically grows and shrinks to provide fast lookups without wasting too much memory. You can still control the size of the hash table by calling <a href="http://doc.qt.io/qt-5/qhash.html#reserve">reserve</a>() if you already know approximately how many items the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> will contain, but this isn't necessary to obtain good performance. You can also call <a href="http://doc.qt.io/qt-5/qhash.html#capacity">capacity</a>() to retrieve the hash table's size.</p>
/// <p>If you want to navigate through all the (key, value) pairs stored in a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, you can use an iterator. <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qhashiterator.html">QHashIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablehashiterator.html">QMutableHashIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a>). Here's how to iterate over a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, int&gt; using a Java-style iterator:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qhashiterator.html">QHashIterator</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> i(hash);
///   <span class="keyword">while</span> (i<span class="operator">.</span>hasNext()) {
/// &#32;     i<span class="operator">.</span>next();
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p>Here's the same code, but using an STL-style iterator:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i <span class="operator">=</span> hash<span class="operator">.</span>constBegin();
///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>constEnd()) {
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
/// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> is unordered, so an iterator's sequence cannot be assumed to be predictable. If ordering by key is required, use a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>.</p>
/// <p>Normally, a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> allows only one value per key. If you call <a href="http://doc.qt.io/qt-5/qhash.html#insert">insert</a>() with a key that already exists in the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, the previous value is erased. For example:</p>
/// <pre class="cpp">
///
///   hash<span class="operator">.</span>insert(<span class="string">"plenty"</span><span class="operator">,</span> <span class="number">100</span>);
///   hash<span class="operator">.</span>insert(<span class="string">"plenty"</span><span class="operator">,</span> <span class="number">2000</span>);
///   <span class="comment">// hash.value("plenty") == 2000</span>
///
/// </pre>
/// <p>However, you can store multiple values per key by using <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">insertMulti</a>() instead of <a href="http://doc.qt.io/qt-5/qhash.html#insert">insert</a>() (or using the convenience subclass <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>). If you want to retrieve all the values for a single key, you can use values(const Key &amp;key), which returns a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> values <span class="operator">=</span> hash<span class="operator">.</span>values(<span class="string">"plenty"</span>);
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> values<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> values<span class="operator">.</span>at(i) <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>The items that share the same key are available from most recently to least recently inserted. A more efficient approach is to call <a href="http://doc.qt.io/qt-5/qhash.html#find">find</a>() to get the iterator for the first item with a key and iterate from there:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> hash<span class="operator">.</span>find(<span class="string">"plenty"</span>);
///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end() <span class="operator">&amp;</span><span class="operator">&amp;</span> i<span class="operator">.</span>key() <span class="operator">=</span><span class="operator">=</span> <span class="string">"plenty"</span>) {
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
/// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
///   }
///
/// </pre>
/// <p>If you only need to extract the values from a hash (not the keys), you can also use <a href="http://doc.qt.io/qt-5/containers.html#foreach">foreach</a>:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   foreach (<span class="type">int</span> value<span class="operator">,</span> hash)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> value <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>Items can be removed from the hash in several ways. One way is to call <a href="http://doc.qt.io/qt-5/qhash.html#remove">remove</a>(); this will remove any item with the given key. Another way is to use <a href="http://doc.qt.io/qt-5/qmutablehashiterator.html#remove">QMutableHashIterator::remove</a>(). In addition, you can clear the entire hash using <a href="http://doc.qt.io/qt-5/qhash.html#clear">clear</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s key and value data types must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *.</p>
/// <a name="qhash"></a><a name="the-qhash-hashing-function"></a>
/// <h4>The qHash() hashing function</h4>
/// <p>A <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s key type has additional requirements other than being an assignable data type: it must provide operator==(), and there must also be a <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function in the type's namespace that returns a hash value for an argument of the key's type.</p>
/// <p>The <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function computes a numeric value based on a key. It can use any algorithm imaginable, as long as it always returns the same value if given the same argument. In other words, if <code>e1 == e2</code>, then <code>qHash(e1) == qHash(e2)</code> must hold as well. However, to obtain good performance, the <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function should attempt to return different hash values for different keys to the largest extent possible.</p>
/// <p>For a key type <code>K</code>, the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a> function must have one of these signatures:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(K key);
///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(<span class="keyword">const</span> K <span class="operator">&amp;</span>key);
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(K key<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> seed);
///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(<span class="keyword">const</span> K <span class="operator">&amp;</span>key<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> seed);
///
/// </pre>
/// <p>The two-arguments overloads take an unsigned integer that should be used to seed the calculation of the hash function. This seed is provided by <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> in order to prevent a family of <a href="http://doc.qt.io/qt-5/qhash.html#algorithmic-complexity-attacks">algorithmic complexity attacks</a>. If both a one-argument and a two-arguments overload are defined for a key type, the latter is used by <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> (note that you can simply define a two-arguments version, and use a default value for the seed parameter).</p>
/// <p>Here's a partial list of the C++ and Qt types that can serve as keys in a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>: any integer type (char, unsigned long, etc.), any pointer type, <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>, <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, and <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>. For all of these, the <code>&lt;QHash&gt;</code> header defines a <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function that computes an adequate hash value. Many other Qt classes also declare a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a> overload for their type; please refer to the documentation of each class.</p>
/// <p>If you want to use other types as the key, make sure that you provide operator==() and a <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() implementation.</p>
/// <p>Example:</p>
/// <pre class="cpp">
///
///   <span class="preprocessor">#ifndef EMPLOYEE_H</span>
///   <span class="preprocessor">#define EMPLOYEE_H</span>
///
///   <span class="keyword">class</span> Employee
///   {
///   <span class="keyword">public</span>:
/// &#32;     Employee() {}
/// &#32;     Employee(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>name<span class="operator">,</span> <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span> <span class="operator">&amp;</span>dateOfBirth);
/// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///
///   <span class="keyword">private</span>:
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> myName;
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span> myDateOfBirth;
///   };
///
///   <span class="keyword">inline</span> bool <span class="keyword">operator</span><span class="operator">=</span><span class="operator">=</span>(<span class="keyword">const</span> Employee <span class="operator">&amp;</span>e1<span class="operator">,</span> <span class="keyword">const</span> Employee <span class="operator">&amp;</span>e2)
///   {
/// &#32;     <span class="keyword">return</span> e1<span class="operator">.</span>name() <span class="operator">=</span><span class="operator">=</span> e2<span class="operator">.</span>name()
/// &#32;   &#32;   &#32;    <span class="operator">&amp;</span><span class="operator">&amp;</span> e1<span class="operator">.</span>dateOfBirth() <span class="operator">=</span><span class="operator">=</span> e2<span class="operator">.</span>dateOfBirth();
///   }
///
///   <span class="keyword">inline</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(<span class="keyword">const</span> Employee <span class="operator">&amp;</span>key<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> seed)
///   {
/// &#32;     <span class="keyword">return</span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(key<span class="operator">.</span>name()<span class="operator">,</span> seed) <span class="operator">^</span> key<span class="operator">.</span>dateOfBirth()<span class="operator">.</span>day();
///   }
///
///   <span class="preprocessor">#endif // EMPLOYEE_H</span>
///
/// </pre>
/// <p>In the example above, we've relied on Qt's global <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a>(const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;, uint) to give us a hash value for the employee's name, and XOR'ed this with the day they were born to help produce unique hashes for people with the same name.</p>
/// <p>Note that the implementation of the <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() overloads offered by Qt may change at any time. You <b>must not</b> rely on the fact that <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() will give the same results (for the same inputs) across different Qt versions.</p>
/// <a name="algorithmic-complexity-attacks"></a>
/// <h4>Algorithmic complexity attacks</h4>
/// <p>All hash tables are vulnerable to a particular class of denial of service attacks, in which the attacker carefully pre-computes a set of different keys that are going to be hashed in the same bucket of a hash table (or even have the very same hash value). The attack aims at getting the worst-case algorithmic behavior (O(n) instead of amortized O(1), see <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for the details) when the data is fed into the table.</p>
/// <p>In order to avoid this worst-case behavior, the calculation of the hash value done by <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() can be salted by a random seed, that nullifies the attack's extent. This seed is automatically generated by <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> once per process, and then passed by <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as the second argument of the two-arguments overload of the <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function.</p>
/// <p>This randomization of <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> is enabled by default. Even though programs should never depend on a particular <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> ordering, there may be situations where you temporarily need deterministic behavior, for example for debugging or regression testing. To disable the randomization, define the environment variable <code>QT_HASH_SEED</code>. The contents of that variable, interpreted as a decimal value, will be used as the seed for <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(). Alternatively, you can call the <a href="http://doc.qt.io/qt-5/qhash.html#qSetGlobalQHashSeed">qSetGlobalQHashSeed</a>() function.</p></div>
#[repr(C)]
pub struct QHashOfIntQByteArray {
    _unused: u8,
}
impl QHashOfIntQByteArray {
    /// <p>Returns the number of buckets in the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s internal hash table.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QHash<int, QByteArray>::capacity() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#capacity">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of buckets in the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s internal hash table.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s memory usage. In general, you will rarely ever need to call this function. If you want to know how many items are in the hash, call <a href="http://doc.qt.io/qt-5/qhash.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#squeeze">squeeze</a>().</p></div>
    pub unsafe fn capacity(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_capacity(
            self as *const crate::QHashOfIntQByteArray,
        )
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::const_iterator QHash<int, QByteArray>::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the hash.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_cbegin(
            self as *const crate::QHashOfIntQByteArray,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::const_iterator QHash<int, QByteArray>::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the hash.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#end">end</a>().</p></div>
    pub unsafe fn cend(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_cend(
            self as *const crate::QHashOfIntQByteArray,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all items from the hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QHash<int, QByteArray>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all items from the hash.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#remove">remove</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_clear(
            self as *mut crate::QHashOfIntQByteArray,
        )
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::const_iterator QHash<int, QByteArray>::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the hash.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_constBegin(
            self as *const crate::QHashOfIntQByteArray,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::const_iterator QHash<int, QByteArray>::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the hash.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#end">end</a>().</p></div>
    pub unsafe fn const_end(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_constEnd(
            self as *const crate::QHashOfIntQByteArray,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns an iterator pointing to the item with the <i>key</i> in the hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::const_iterator QHash<int, QByteArray>::constFind(const int& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#constFind">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator pointing to the item with the <i>key</i> in the hash.</p>
    /// <p>If the hash contains no item with the <i>key</i>, the function returns <a href="http://doc.qt.io/qt-5/qhash.html#constEnd">constEnd</a>().</p>
    /// <p>This function was introduced in  Qt 4.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#find">find</a>() and <a href="http://doc.qt.io/qt-5/qmultihash.html#constFind">QMultiHash::constFind</a>().</p></div>
    pub unsafe fn const_find(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_constFind(
            self as *const crate::QHashOfIntQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if the hash contains an item with the <i>key</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QHash<int, QByteArray>::contains(const int& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the hash contains an item with the <i>key</i>; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#count-1">count</a>() and <a href="http://doc.qt.io/qt-5/qmultihash.html#contains">QMultiHash::contains</a>().</p></div>
    pub unsafe fn contains(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_contains(
            self as *const crate::QHashOfIntQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
        )
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> class is a template class that provides a hash-table-based dictionary.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>& QHash<int, QByteArray>::operator=(const QHash<int, QByteArray>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> class is a template class that provides a hash-table-based dictionary.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;Key, T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores (key, value) pairs and provides very fast lookup of the value associated with a key.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> provides very similar functionality to <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>. The differences are:</p>
    /// <ul>
    /// <li><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> provides faster lookups than <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.)</li>
    /// <li>When iterating over a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, the items are always sorted by key. With <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, the items are arbitrarily ordered.</li>
    /// <li>The key type of a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> must provide operator&lt;(). The key type of a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> must provide operator==() and a global hash function called <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() (see <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a>).</li>
    /// </ul>
    /// <p>Here's an example <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> with <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> keys and <code>int</code> values:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
    ///
    /// </pre>
    /// <p>To insert a (key, value) pair into the hash, you can use operator[]():</p>
    /// <pre class="cpp">
    ///
    ///   hash<span class="operator">[</span><span class="string">"one"</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>;
    ///   hash<span class="operator">[</span><span class="string">"three"</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">3</span>;
    ///   hash<span class="operator">[</span><span class="string">"seven"</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">7</span>;
    ///
    /// </pre>
    /// <p>This inserts the following three (key, value) pairs into the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>: ("one", 1), ("three", 3), and ("seven", 7). Another way to insert items into the hash is to use <a href="http://doc.qt.io/qt-5/qhash.html#insert">insert</a>():</p>
    /// <pre class="cpp">
    ///
    ///   hash<span class="operator">.</span>insert(<span class="string">"twelve"</span><span class="operator">,</span> <span class="number">12</span>);
    ///
    /// </pre>
    /// <p>To look up a value, use operator[]() or <a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>():</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">int</span> num1 <span class="operator">=</span> hash<span class="operator">[</span><span class="string">"thirteen"</span><span class="operator">]</span>;
    ///   <span class="type">int</span> num2 <span class="operator">=</span> hash<span class="operator">.</span>value(<span class="string">"thirteen"</span>);
    ///
    /// </pre>
    /// <p>If there is no item with the specified key in the hash, these functions return a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>.</p>
    /// <p>If you want to check whether the hash contains a particular key, use <a href="http://doc.qt.io/qt-5/qhash.html#contains">contains</a>():</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">int</span> timeout <span class="operator">=</span> <span class="number">30</span>;
    ///   <span class="keyword">if</span> (hash<span class="operator">.</span>contains(<span class="string">"TIMEOUT"</span>))
    /// &#32;     timeout <span class="operator">=</span> hash<span class="operator">.</span>value(<span class="string">"TIMEOUT"</span>);
    ///
    /// </pre>
    /// <p>There is also a <a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>() overload that uses its second argument as a default value if there is no item with the specified key:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">int</span> timeout <span class="operator">=</span> hash<span class="operator">.</span>value(<span class="string">"TIMEOUT"</span><span class="operator">,</span> <span class="number">30</span>);
    ///
    /// </pre>
    /// <p>In general, we recommend that you use <a href="http://doc.qt.io/qt-5/qhash.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>() rather than operator[]() for looking up a key in a hash. The reason is that operator[]() silently inserts an item into the hash if no item exists with the same key (unless the hash is const). For example, the following code snippet will create 1000 items in memory:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="comment">// WRONG</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> hash;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> <span class="number">1000</span>; <span class="operator">+</span><span class="operator">+</span>i) {
    /// &#32;     <span class="keyword">if</span> (hash<span class="operator">[</span>i<span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> okButton)
    /// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found button at index "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///   }
    ///
    /// </pre>
    /// <p>To avoid this problem, replace <code>hash[i]</code> with <code>hash.value(i)</code> in the code above.</p>
    /// <p>Internally, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> uses a hash table to perform lookups. Unlike Qt 3's <code>QDict</code> class, which needed to be initialized with a prime number, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s hash table automatically grows and shrinks to provide fast lookups without wasting too much memory. You can still control the size of the hash table by calling <a href="http://doc.qt.io/qt-5/qhash.html#reserve">reserve</a>() if you already know approximately how many items the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> will contain, but this isn't necessary to obtain good performance. You can also call <a href="http://doc.qt.io/qt-5/qhash.html#capacity">capacity</a>() to retrieve the hash table's size.</p>
    /// <p>If you want to navigate through all the (key, value) pairs stored in a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, you can use an iterator. <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qhashiterator.html">QHashIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablehashiterator.html">QMutableHashIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a>). Here's how to iterate over a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, int&gt; using a Java-style iterator:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhashiterator.html">QHashIterator</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> i(hash);
    ///   <span class="keyword">while</span> (i<span class="operator">.</span>hasNext()) {
    /// &#32;     i<span class="operator">.</span>next();
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///   }
    ///
    /// </pre>
    /// <p>Here's the same code, but using an STL-style iterator:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i <span class="operator">=</span> hash<span class="operator">.</span>constBegin();
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>constEnd()) {
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    ///   }
    ///
    /// </pre>
    /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> is unordered, so an iterator's sequence cannot be assumed to be predictable. If ordering by key is required, use a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>.</p>
    /// <p>Normally, a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> allows only one value per key. If you call <a href="http://doc.qt.io/qt-5/qhash.html#insert">insert</a>() with a key that already exists in the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, the previous value is erased. For example:</p>
    /// <pre class="cpp">
    ///
    ///   hash<span class="operator">.</span>insert(<span class="string">"plenty"</span><span class="operator">,</span> <span class="number">100</span>);
    ///   hash<span class="operator">.</span>insert(<span class="string">"plenty"</span><span class="operator">,</span> <span class="number">2000</span>);
    ///   <span class="comment">// hash.value("plenty") == 2000</span>
    ///
    /// </pre>
    /// <p>However, you can store multiple values per key by using <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">insertMulti</a>() instead of <a href="http://doc.qt.io/qt-5/qhash.html#insert">insert</a>() (or using the convenience subclass <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>). If you want to retrieve all the values for a single key, you can use values(const Key &amp;key), which returns a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> values <span class="operator">=</span> hash<span class="operator">.</span>values(<span class="string">"plenty"</span>);
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> values<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> values<span class="operator">.</span>at(i) <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>The items that share the same key are available from most recently to least recently inserted. A more efficient approach is to call <a href="http://doc.qt.io/qt-5/qhash.html#find">find</a>() to get the iterator for the first item with a key and iterate from there:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> hash<span class="operator">.</span>find(<span class="string">"plenty"</span>);
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end() <span class="operator">&amp;</span><span class="operator">&amp;</span> i<span class="operator">.</span>key() <span class="operator">=</span><span class="operator">=</span> <span class="string">"plenty"</span>) {
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    ///   }
    ///
    /// </pre>
    /// <p>If you only need to extract the values from a hash (not the keys), you can also use <a href="http://doc.qt.io/qt-5/containers.html#foreach">foreach</a>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   foreach (<span class="type">int</span> value<span class="operator">,</span> hash)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> value <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Items can be removed from the hash in several ways. One way is to call <a href="http://doc.qt.io/qt-5/qhash.html#remove">remove</a>(); this will remove any item with the given key. Another way is to use <a href="http://doc.qt.io/qt-5/qmutablehashiterator.html#remove">QMutableHashIterator::remove</a>(). In addition, you can clear the entire hash using <a href="http://doc.qt.io/qt-5/qhash.html#clear">clear</a>().</p>
    /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s key and value data types must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *.</p>
    /// <a name="qhash"></a><a name="the-qhash-hashing-function"></a>
    /// <h4>The qHash() hashing function</h4>
    /// <p>A <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s key type has additional requirements other than being an assignable data type: it must provide operator==(), and there must also be a <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function in the type's namespace that returns a hash value for an argument of the key's type.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function computes a numeric value based on a key. It can use any algorithm imaginable, as long as it always returns the same value if given the same argument. In other words, if <code>e1 == e2</code>, then <code>qHash(e1) == qHash(e2)</code> must hold as well. However, to obtain good performance, the <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function should attempt to return different hash values for different keys to the largest extent possible.</p>
    /// <p>For a key type <code>K</code>, the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a> function must have one of these signatures:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(K key);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(<span class="keyword">const</span> K <span class="operator">&amp;</span>key);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(K key<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> seed);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(<span class="keyword">const</span> K <span class="operator">&amp;</span>key<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> seed);
    ///
    /// </pre>
    /// <p>The two-arguments overloads take an unsigned integer that should be used to seed the calculation of the hash function. This seed is provided by <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> in order to prevent a family of <a href="http://doc.qt.io/qt-5/qhash.html#algorithmic-complexity-attacks">algorithmic complexity attacks</a>. If both a one-argument and a two-arguments overload are defined for a key type, the latter is used by <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> (note that you can simply define a two-arguments version, and use a default value for the seed parameter).</p>
    /// <p>Here's a partial list of the C++ and Qt types that can serve as keys in a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>: any integer type (char, unsigned long, etc.), any pointer type, <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>, <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, and <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>. For all of these, the <code>&lt;QHash&gt;</code> header defines a <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function that computes an adequate hash value. Many other Qt classes also declare a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a> overload for their type; please refer to the documentation of each class.</p>
    /// <p>If you want to use other types as the key, make sure that you provide operator==() and a <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() implementation.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="preprocessor">#ifndef EMPLOYEE_H</span>
    ///   <span class="preprocessor">#define EMPLOYEE_H</span>
    ///
    ///   <span class="keyword">class</span> Employee
    ///   {
    ///   <span class="keyword">public</span>:
    /// &#32;     Employee() {}
    /// &#32;     Employee(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>name<span class="operator">,</span> <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span> <span class="operator">&amp;</span>dateOfBirth);
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///
    ///   <span class="keyword">private</span>:
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> myName;
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span> myDateOfBirth;
    ///   };
    ///
    ///   <span class="keyword">inline</span> bool <span class="keyword">operator</span><span class="operator">=</span><span class="operator">=</span>(<span class="keyword">const</span> Employee <span class="operator">&amp;</span>e1<span class="operator">,</span> <span class="keyword">const</span> Employee <span class="operator">&amp;</span>e2)
    ///   {
    /// &#32;     <span class="keyword">return</span> e1<span class="operator">.</span>name() <span class="operator">=</span><span class="operator">=</span> e2<span class="operator">.</span>name()
    /// &#32;   &#32;   &#32;    <span class="operator">&amp;</span><span class="operator">&amp;</span> e1<span class="operator">.</span>dateOfBirth() <span class="operator">=</span><span class="operator">=</span> e2<span class="operator">.</span>dateOfBirth();
    ///   }
    ///
    ///   <span class="keyword">inline</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(<span class="keyword">const</span> Employee <span class="operator">&amp;</span>key<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> seed)
    ///   {
    /// &#32;     <span class="keyword">return</span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(key<span class="operator">.</span>name()<span class="operator">,</span> seed) <span class="operator">^</span> key<span class="operator">.</span>dateOfBirth()<span class="operator">.</span>day();
    ///   }
    ///
    ///   <span class="preprocessor">#endif // EMPLOYEE_H</span>
    ///
    /// </pre>
    /// <p>In the example above, we've relied on Qt's global <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a>(const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;, uint) to give us a hash value for the employee's name, and XOR'ed this with the day they were born to help produce unique hashes for people with the same name.</p>
    /// <p>Note that the implementation of the <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() overloads offered by Qt may change at any time. You <b>must not</b> rely on the fact that <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() will give the same results (for the same inputs) across different Qt versions.</p>
    /// <a name="algorithmic-complexity-attacks"></a>
    /// <h4>Algorithmic complexity attacks</h4>
    /// <p>All hash tables are vulnerable to a particular class of denial of service attacks, in which the attacker carefully pre-computes a set of different keys that are going to be hashed in the same bucket of a hash table (or even have the very same hash value). The attack aims at getting the worst-case algorithmic behavior (O(n) instead of amortized O(1), see <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for the details) when the data is fed into the table.</p>
    /// <p>In order to avoid this worst-case behavior, the calculation of the hash value done by <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() can be salted by a random seed, that nullifies the attack's extent. This seed is automatically generated by <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> once per process, and then passed by <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as the second argument of the two-arguments overload of the <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function.</p>
    /// <p>This randomization of <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> is enabled by default. Even though programs should never depend on a particular <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> ordering, there may be situations where you temporarily need deterministic behavior, for example for debugging or regression testing. To disable the randomization, define the environment variable <code>QT_HASH_SEED</code>. The contents of that variable, interpreted as a decimal value, will be used as the seed for <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(). Alternatively, you can call the <a href="http://doc.qt.io/qt-5/qhash.html#qSetGlobalQHashSeed">qSetGlobalQHashSeed</a>() function.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QHashOfIntQByteArray>>,
    ) -> ::cpp_utils::MutRef<crate::QHashOfIntQByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_operator_(
            self as *mut crate::QHashOfIntQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QHashOfIntQByteArray>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of items associated with the <i>key</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QHash<int, QByteArray>::count(const int& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items associated with the <i>key</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">insertMulti</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_count(
            self as *const crate::QHashOfIntQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QHash<int, QByteArray>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_count1(
            self as *const crate::QHashOfIntQByteArray,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QHash<int, QByteArray>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_detach(
            self as *mut crate::QHashOfIntQByteArray,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qhash.html#isEmpty">isEmpty</a>(), returning true if the hash is empty; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QHash<int, QByteArray>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qhash.html#isEmpty">isEmpty</a>(), returning true if the hash is empty; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_empty(
            self as *const crate::QHashOfIntQByteArray,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::iterator QHash<int, QByteArray>::erase(QHash<int, QByteArray>::iterator it)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#erase-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn erase_iterator(
        &mut self,
        it: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::Iterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_erase(
                self as *mut crate::QHashOfIntQByteArray,
                ::cpp_utils::CastInto::<
                    ::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::Iterator>,
                >::cast_into(it)
                .as_raw_ptr(),
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the (key, value) pair associated with the iterator <i>pos</i> from the hash, and returns an iterator to the next item in the hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::iterator QHash<int, QByteArray>::erase(QHash<int, QByteArray>::const_iterator it)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the (key, value) pair associated with the iterator <i>pos</i> from the hash, and returns an iterator to the next item in the hash.</p>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qhash.html#remove">remove</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#take">take</a>(), this function never causes <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> to rehash its internal data structure. This means that it can safely be called while iterating, and won't affect the order of items in the hash. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qobject.html">QObject</a></span> <span class="operator">*</span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> objectHash;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qobject.html">QObject</a></span> <span class="operator">*</span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> objectHash<span class="operator">.</span>find(obj);
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> objectHash<span class="operator">.</span>end() <span class="operator">&amp;</span><span class="operator">&amp;</span> i<span class="operator">.</span>key() <span class="operator">=</span><span class="operator">=</span> obj) {
    /// &#32;     <span class="keyword">if</span> (i<span class="operator">.</span>value() <span class="operator">=</span><span class="operator">=</span> <span class="number">0</span>) {
    /// &#32;   &#32;     i <span class="operator">=</span> objectHash<span class="operator">.</span>erase(i);
    /// &#32;     } <span class="keyword">else</span> {
    /// &#32;   &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    /// &#32;     }
    ///   }
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#take">take</a>(), and <a href="http://doc.qt.io/qt-5/qhash.html#find">find</a>().</p></div>
    pub unsafe fn erase_const_iterator(
        &mut self,
        it: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::ConstIterator>,
        >,
    ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_erase1(
            self as *mut crate::QHashOfIntQByteArray,
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::q_hash_of_int_q_byte_array::ConstIterator>,
            >::cast_into(it)
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns an iterator pointing to the item with the <i>key</i> in the hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::iterator QHash<int, QByteArray>::find(const int& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#find">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator pointing to the item with the <i>key</i> in the hash.</p>
    /// <p>If the hash contains no item with the <i>key</i>, the function returns <a href="http://doc.qt.io/qt-5/qhash.html#end">end</a>().</p>
    /// <p>If the hash contains multiple items with the <i>key</i>, this function returns an iterator that points to the most recently inserted value. The other values are accessible by incrementing the iterator. For example, here's some code that iterates over all the items with the same key:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i <span class="operator">=</span> hash<span class="operator">.</span>find(<span class="string">"HDR"</span>);
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end() <span class="operator">&amp;</span><span class="operator">&amp;</span> i<span class="operator">.</span>key() <span class="operator">=</span><span class="operator">=</span> <span class="string">"HDR"</span>) {
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    ///   }
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#values">values</a>(), and <a href="http://doc.qt.io/qt-5/qmultihash.html#find">QMultiHash::find</a>().</p></div>
    pub unsafe fn find_mut(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_find(
            self as *mut crate::QHashOfIntQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::const_iterator QHash<int, QByteArray>::find(const int& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#find-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn find(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_find1(
            self as *const crate::QHashOfIntQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value associated with the <i>key</i> as a modifiable reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QHash<int, QByteArray>::operator[](const int& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value associated with the <i>key</i> as a modifiable reference.</p>
    /// <p>If the hash contains no item with the <i>key</i>, the function inserts a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a> into the hash with the <i>key</i>, and returns a reference to it. If the hash contains multiple items with the <i>key</i>, this function returns a reference to the most recently inserted value.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::MutRef<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_operator__(
            self as *mut crate::QHashOfIntQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QHash<int, QByteArray>::operator[](const int& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>().</p></div>
    pub unsafe fn index(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_operator__1(
            self as *const crate::QHashOfIntQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Inserts a new item with the <i>key</i> and a value of <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::iterator QHash<int, QByteArray>::insert(const int& key, const QByteArray& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts a new item with the <i>key</i> and a value of <i>value</i>.</p>
    /// <p>If there is already an item with the <i>key</i>, that item's value is replaced with <i>value</i>.</p>
    /// <p>If there are multiple items with the <i>key</i>, the most recently inserted item's value is replaced with <i>value</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">insertMulti</a>().</p></div>
    pub unsafe fn insert(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_insert(
            self as *mut crate::QHashOfIntQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Inserts a new item with the <i>key</i> and a value of <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::iterator QHash<int, QByteArray>::insertMulti(const int& key, const QByteArray& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts a new item with the <i>key</i> and a value of <i>value</i>.</p>
    /// <p>If there is already an item with the same key in the hash, this function will simply create a new one. (This behavior is different from <a href="http://doc.qt.io/qt-5/qhash.html#insert">insert</a>(), which overwrites the value of an existing item.)</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#values">values</a>().</p></div>
    pub unsafe fn insert_multi(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_insertMulti(
            self as *mut crate::QHashOfIntQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QHash<int, QByteArray>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_isDetached(
            self as *const crate::QHashOfIntQByteArray,
        )
    }

    /// <p>Returns <code>true</code> if the hash contains no items; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QHash<int, QByteArray>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the hash contains no items; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_isEmpty(
            self as *const crate::QHashOfIntQByteArray,
        )
    }

    /// <p>Returns the first key mapped to <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QHash<int, QByteArray>::key(const QByteArray& value) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#key">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the first key mapped to <i>value</i>.</p>
    /// <p>If the hash contains no item with the <i>value</i>, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed key</a>.</p>
    /// <p>This function can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s internal data structure is optimized for fast lookup by key, not by value.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#keys">keys</a>().</p></div>
    pub unsafe fn key_1a(
        &self,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_key(
            self as *const crate::QHashOfIntQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QHash<int, QByteArray>::key(const QByteArray& value, const int& defaultKey) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#key-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the first key mapped to <i>value</i>, or <i>defaultKey</i> if the hash contains no item mapped to <i>value</i>.</p>
    /// <p>This function can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s internal data structure is optimized for fast lookup by key, not by value.</p>
    /// <p>This function was introduced in  Qt 4.3.</p></div>
    pub unsafe fn key_2a(
        &self,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        default_key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_key1(
            self as *const crate::QHashOfIntQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(value)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(
                default_key,
            )
            .as_raw_ptr(),
        )
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first key in the hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::key_iterator QHash<int, QByteArray>::keyBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#keyBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first key in the hash.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#keyEnd">keyEnd</a>().</p></div>
    pub unsafe fn key_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::KeyIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_keyBegin(
            self as *const crate::QHashOfIntQByteArray,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last key in the hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::key_iterator QHash<int, QByteArray>::keyEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#keyEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last key in the hash.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#keyBegin">keyBegin</a>().</p></div>
    pub unsafe fn key_end(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::KeyIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_keyEnd(
            self as *const crate::QHashOfIntQByteArray,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list containing all the keys in the hash, in an arbitrary order. Keys that occur multiple times in the hash (because items were inserted with <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">insertMulti</a>(), or <a href="http://doc.qt.io/qt-5/qhash.html#unite">unite</a>() was used) also occur multiple times in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<int> QHash<int, QByteArray>::keys() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#keys">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list containing all the keys in the hash, in an arbitrary order. Keys that occur multiple times in the hash (because items were inserted with <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">insertMulti</a>(), or <a href="http://doc.qt.io/qt-5/qhash.html#unite">unite</a>() was used) also occur multiple times in the list.</p>
    /// <p>To obtain a list of unique keys, where each key from the map only occurs once, use <a href="http://doc.qt.io/qt-5/qhash.html#uniqueKeys">uniqueKeys</a>().</p>
    /// <p>The order is guaranteed to be the same as that used by <a href="http://doc.qt.io/qt-5/qhash.html#values">values</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#uniqueKeys">uniqueKeys</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#values">values</a>(), and <a href="http://doc.qt.io/qt-5/qhash.html#key">key</a>().</p></div>
    pub unsafe fn keys_0a(&self) -> ::cpp_utils::CppBox<crate::QListOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_keys(
            self as *const crate::QHashOfIntQByteArray,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<int> QHash<int, QByteArray>::keys(const QByteArray& value) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#keys-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a list containing all the keys associated with value <i>value</i>, in an arbitrary order.</p>
    /// <p>This function can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s internal data structure is optimized for fast lookup by key, not by value.</p></div>
    pub unsafe fn keys_1a(
        &self,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_keys1(
            self as *const crate::QHashOfIntQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs an empty hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QHash<int, QByteArray>::QHash()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#QHash">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty hash.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#clear">clear</a>().</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QHashOfIntQByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_QHash();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> class is a template class that provides a hash-table-based dictionary.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QHash<int, QByteArray>::QHash(const QHash<int, QByteArray>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> class is a template class that provides a hash-table-based dictionary.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;Key, T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores (key, value) pairs and provides very fast lookup of the value associated with a key.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> provides very similar functionality to <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>. The differences are:</p>
    /// <ul>
    /// <li><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> provides faster lookups than <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.)</li>
    /// <li>When iterating over a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>, the items are always sorted by key. With <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, the items are arbitrarily ordered.</li>
    /// <li>The key type of a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> must provide operator&lt;(). The key type of a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> must provide operator==() and a global hash function called <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() (see <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a>).</li>
    /// </ul>
    /// <p>Here's an example <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> with <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> keys and <code>int</code> values:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
    ///
    /// </pre>
    /// <p>To insert a (key, value) pair into the hash, you can use operator[]():</p>
    /// <pre class="cpp">
    ///
    ///   hash<span class="operator">[</span><span class="string">"one"</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>;
    ///   hash<span class="operator">[</span><span class="string">"three"</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">3</span>;
    ///   hash<span class="operator">[</span><span class="string">"seven"</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">7</span>;
    ///
    /// </pre>
    /// <p>This inserts the following three (key, value) pairs into the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>: ("one", 1), ("three", 3), and ("seven", 7). Another way to insert items into the hash is to use <a href="http://doc.qt.io/qt-5/qhash.html#insert">insert</a>():</p>
    /// <pre class="cpp">
    ///
    ///   hash<span class="operator">.</span>insert(<span class="string">"twelve"</span><span class="operator">,</span> <span class="number">12</span>);
    ///
    /// </pre>
    /// <p>To look up a value, use operator[]() or <a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>():</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">int</span> num1 <span class="operator">=</span> hash<span class="operator">[</span><span class="string">"thirteen"</span><span class="operator">]</span>;
    ///   <span class="type">int</span> num2 <span class="operator">=</span> hash<span class="operator">.</span>value(<span class="string">"thirteen"</span>);
    ///
    /// </pre>
    /// <p>If there is no item with the specified key in the hash, these functions return a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>.</p>
    /// <p>If you want to check whether the hash contains a particular key, use <a href="http://doc.qt.io/qt-5/qhash.html#contains">contains</a>():</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">int</span> timeout <span class="operator">=</span> <span class="number">30</span>;
    ///   <span class="keyword">if</span> (hash<span class="operator">.</span>contains(<span class="string">"TIMEOUT"</span>))
    /// &#32;     timeout <span class="operator">=</span> hash<span class="operator">.</span>value(<span class="string">"TIMEOUT"</span>);
    ///
    /// </pre>
    /// <p>There is also a <a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>() overload that uses its second argument as a default value if there is no item with the specified key:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">int</span> timeout <span class="operator">=</span> hash<span class="operator">.</span>value(<span class="string">"TIMEOUT"</span><span class="operator">,</span> <span class="number">30</span>);
    ///
    /// </pre>
    /// <p>In general, we recommend that you use <a href="http://doc.qt.io/qt-5/qhash.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>() rather than operator[]() for looking up a key in a hash. The reason is that operator[]() silently inserts an item into the hash if no item exists with the same key (unless the hash is const). For example, the following code snippet will create 1000 items in memory:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="comment">// WRONG</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> hash;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> <span class="number">1000</span>; <span class="operator">+</span><span class="operator">+</span>i) {
    /// &#32;     <span class="keyword">if</span> (hash<span class="operator">[</span>i<span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> okButton)
    /// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found button at index "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///   }
    ///
    /// </pre>
    /// <p>To avoid this problem, replace <code>hash[i]</code> with <code>hash.value(i)</code> in the code above.</p>
    /// <p>Internally, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> uses a hash table to perform lookups. Unlike Qt 3's <code>QDict</code> class, which needed to be initialized with a prime number, <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s hash table automatically grows and shrinks to provide fast lookups without wasting too much memory. You can still control the size of the hash table by calling <a href="http://doc.qt.io/qt-5/qhash.html#reserve">reserve</a>() if you already know approximately how many items the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> will contain, but this isn't necessary to obtain good performance. You can also call <a href="http://doc.qt.io/qt-5/qhash.html#capacity">capacity</a>() to retrieve the hash table's size.</p>
    /// <p>If you want to navigate through all the (key, value) pairs stored in a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, you can use an iterator. <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qhashiterator.html">QHashIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablehashiterator.html">QMutableHashIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qhash-const-iterator.html">QHash::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qhash-iterator.html">QHash::iterator</a>). Here's how to iterate over a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, int&gt; using a Java-style iterator:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhashiterator.html">QHashIterator</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> i(hash);
    ///   <span class="keyword">while</span> (i<span class="operator">.</span>hasNext()) {
    /// &#32;     i<span class="operator">.</span>next();
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///   }
    ///
    /// </pre>
    /// <p>Here's the same code, but using an STL-style iterator:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i <span class="operator">=</span> hash<span class="operator">.</span>constBegin();
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>constEnd()) {
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>key() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">": "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    ///   }
    ///
    /// </pre>
    /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> is unordered, so an iterator's sequence cannot be assumed to be predictable. If ordering by key is required, use a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>.</p>
    /// <p>Normally, a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> allows only one value per key. If you call <a href="http://doc.qt.io/qt-5/qhash.html#insert">insert</a>() with a key that already exists in the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>, the previous value is erased. For example:</p>
    /// <pre class="cpp">
    ///
    ///   hash<span class="operator">.</span>insert(<span class="string">"plenty"</span><span class="operator">,</span> <span class="number">100</span>);
    ///   hash<span class="operator">.</span>insert(<span class="string">"plenty"</span><span class="operator">,</span> <span class="number">2000</span>);
    ///   <span class="comment">// hash.value("plenty") == 2000</span>
    ///
    /// </pre>
    /// <p>However, you can store multiple values per key by using <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">insertMulti</a>() instead of <a href="http://doc.qt.io/qt-5/qhash.html#insert">insert</a>() (or using the convenience subclass <a href="http://doc.qt.io/qt-5/qmultihash.html">QMultiHash</a>). If you want to retrieve all the values for a single key, you can use values(const Key &amp;key), which returns a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> values <span class="operator">=</span> hash<span class="operator">.</span>values(<span class="string">"plenty"</span>);
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> values<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> values<span class="operator">.</span>at(i) <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>The items that share the same key are available from most recently to least recently inserted. A more efficient approach is to call <a href="http://doc.qt.io/qt-5/qhash.html#find">find</a>() to get the iterator for the first item with a key and iterate from there:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> hash<span class="operator">.</span>find(<span class="string">"plenty"</span>);
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> hash<span class="operator">.</span>end() <span class="operator">&amp;</span><span class="operator">&amp;</span> i<span class="operator">.</span>key() <span class="operator">=</span><span class="operator">=</span> <span class="string">"plenty"</span>) {
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>value() <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    ///   }
    ///
    /// </pre>
    /// <p>If you only need to extract the values from a hash (not the keys), you can also use <a href="http://doc.qt.io/qt-5/containers.html#foreach">foreach</a>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   foreach (<span class="type">int</span> value<span class="operator">,</span> hash)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> value <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Items can be removed from the hash in several ways. One way is to call <a href="http://doc.qt.io/qt-5/qhash.html#remove">remove</a>(); this will remove any item with the given key. Another way is to use <a href="http://doc.qt.io/qt-5/qmutablehashiterator.html#remove">QMutableHashIterator::remove</a>(). In addition, you can clear the entire hash using <a href="http://doc.qt.io/qt-5/qhash.html#clear">clear</a>().</p>
    /// <p><a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s key and value data types must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *.</p>
    /// <a name="qhash"></a><a name="the-qhash-hashing-function"></a>
    /// <h4>The qHash() hashing function</h4>
    /// <p>A <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s key type has additional requirements other than being an assignable data type: it must provide operator==(), and there must also be a <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function in the type's namespace that returns a hash value for an argument of the key's type.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function computes a numeric value based on a key. It can use any algorithm imaginable, as long as it always returns the same value if given the same argument. In other words, if <code>e1 == e2</code>, then <code>qHash(e1) == qHash(e2)</code> must hold as well. However, to obtain good performance, the <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function should attempt to return different hash values for different keys to the largest extent possible.</p>
    /// <p>For a key type <code>K</code>, the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a> function must have one of these signatures:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(K key);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(<span class="keyword">const</span> K <span class="operator">&amp;</span>key);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(K key<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> seed);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(<span class="keyword">const</span> K <span class="operator">&amp;</span>key<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> seed);
    ///
    /// </pre>
    /// <p>The two-arguments overloads take an unsigned integer that should be used to seed the calculation of the hash function. This seed is provided by <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> in order to prevent a family of <a href="http://doc.qt.io/qt-5/qhash.html#algorithmic-complexity-attacks">algorithmic complexity attacks</a>. If both a one-argument and a two-arguments overload are defined for a key type, the latter is used by <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> (note that you can simply define a two-arguments version, and use a default value for the seed parameter).</p>
    /// <p>Here's a partial list of the C++ and Qt types that can serve as keys in a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>: any integer type (char, unsigned long, etc.), any pointer type, <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>, <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, and <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>. For all of these, the <code>&lt;QHash&gt;</code> header defines a <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function that computes an adequate hash value. Many other Qt classes also declare a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a> overload for their type; please refer to the documentation of each class.</p>
    /// <p>If you want to use other types as the key, make sure that you provide operator==() and a <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() implementation.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="preprocessor">#ifndef EMPLOYEE_H</span>
    ///   <span class="preprocessor">#define EMPLOYEE_H</span>
    ///
    ///   <span class="keyword">class</span> Employee
    ///   {
    ///   <span class="keyword">public</span>:
    /// &#32;     Employee() {}
    /// &#32;     Employee(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>name<span class="operator">,</span> <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span> <span class="operator">&amp;</span>dateOfBirth);
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///
    ///   <span class="keyword">private</span>:
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> myName;
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span> myDateOfBirth;
    ///   };
    ///
    ///   <span class="keyword">inline</span> bool <span class="keyword">operator</span><span class="operator">=</span><span class="operator">=</span>(<span class="keyword">const</span> Employee <span class="operator">&amp;</span>e1<span class="operator">,</span> <span class="keyword">const</span> Employee <span class="operator">&amp;</span>e2)
    ///   {
    /// &#32;     <span class="keyword">return</span> e1<span class="operator">.</span>name() <span class="operator">=</span><span class="operator">=</span> e2<span class="operator">.</span>name()
    /// &#32;   &#32;   &#32;    <span class="operator">&amp;</span><span class="operator">&amp;</span> e1<span class="operator">.</span>dateOfBirth() <span class="operator">=</span><span class="operator">=</span> e2<span class="operator">.</span>dateOfBirth();
    ///   }
    ///
    ///   <span class="keyword">inline</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(<span class="keyword">const</span> Employee <span class="operator">&amp;</span>key<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> seed)
    ///   {
    /// &#32;     <span class="keyword">return</span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(key<span class="operator">.</span>name()<span class="operator">,</span> seed) <span class="operator">^</span> key<span class="operator">.</span>dateOfBirth()<span class="operator">.</span>day();
    ///   }
    ///
    ///   <span class="preprocessor">#endif // EMPLOYEE_H</span>
    ///
    /// </pre>
    /// <p>In the example above, we've relied on Qt's global <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a>(const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;, uint) to give us a hash value for the employee's name, and XOR'ed this with the day they were born to help produce unique hashes for people with the same name.</p>
    /// <p>Note that the implementation of the <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() overloads offered by Qt may change at any time. You <b>must not</b> rely on the fact that <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() will give the same results (for the same inputs) across different Qt versions.</p>
    /// <a name="algorithmic-complexity-attacks"></a>
    /// <h4>Algorithmic complexity attacks</h4>
    /// <p>All hash tables are vulnerable to a particular class of denial of service attacks, in which the attacker carefully pre-computes a set of different keys that are going to be hashed in the same bucket of a hash table (or even have the very same hash value). The attack aims at getting the worst-case algorithmic behavior (O(n) instead of amortized O(1), see <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for the details) when the data is fed into the table.</p>
    /// <p>In order to avoid this worst-case behavior, the calculation of the hash value done by <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() can be salted by a random seed, that nullifies the attack's extent. This seed is automatically generated by <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> once per process, and then passed by <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> as the second argument of the two-arguments overload of the <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() function.</p>
    /// <p>This randomization of <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> is enabled by default. Even though programs should never depend on a particular <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> ordering, there may be situations where you temporarily need deterministic behavior, for example for debugging or regression testing. To disable the randomization, define the environment variable <code>QT_HASH_SEED</code>. The contents of that variable, interpreted as a decimal value, will be used as the seed for <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(). Alternatively, you can call the <a href="http://doc.qt.io/qt-5/qhash.html#qSetGlobalQHashSeed">qSetGlobalQHashSeed</a>() function.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QHashOfIntQByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QHashOfIntQByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_QHash1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QHashOfIntQByteArray>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all the items that have the <i>key</i> from the hash. Returns the number of items removed which is usually 1 but will be 0 if the key isn't in the hash, or greater than 1 if <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">insertMulti</a>() has been used with the <i>key</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QHash<int, QByteArray>::remove(const int& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#remove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all the items that have the <i>key</i> from the hash. Returns the number of items removed which is usually 1 but will be 0 if the key isn't in the hash, or greater than 1 if <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">insertMulti</a>() has been used with the <i>key</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#clear">clear</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#take">take</a>(), and <a href="http://doc.qt.io/qt-5/qmultihash.html#remove">QMultiHash::remove</a>().</p></div>
    pub unsafe fn remove(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_remove(
            self as *mut crate::QHashOfIntQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
        )
    }

    /// <p>Ensures that the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s internal hash table consists of at least <i>size</i> buckets.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QHash<int, QByteArray>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Ensures that the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s internal hash table consists of at least <i>size</i> buckets.</p>
    /// <p>This function is useful for code that needs to build a huge hash and wants to avoid repeated reallocation. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html#QHash">QHash</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">int</span><span class="operator">&gt;</span> hash;
    ///   hash<span class="operator">.</span>reserve(<span class="number">20000</span>);
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> <span class="number">20000</span>; <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     hash<span class="operator">.</span>insert(keys<span class="operator">[</span>i<span class="operator">]</span><span class="operator">,</span> values<span class="operator">[</span>i<span class="operator">]</span>);
    ///
    /// </pre>
    /// <p>Ideally, <i>size</i> should be slightly more than the maximum number of items expected in the hash. <i>size</i> doesn't have to be prime, because <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> will use a prime number internally anyway. If <i>size</i> is an underestimate, the worst that will happen is that the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> will be a bit slower.</p>
    /// <p>In general, you will rarely ever need to call this function. <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s internal hash table automatically shrinks or grows to provide good performance without wasting too much memory.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#squeeze">squeeze</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#capacity">capacity</a>().</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_reserve(
            self as *mut crate::QHashOfIntQByteArray,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QHash<int, QByteArray>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_setSharable(
            self as *mut crate::QHashOfIntQByteArray,
            sharable,
        )
    }

    /// <p>Returns the number of items in the hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QHash<int, QByteArray>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the hash.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#count-1">count</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_size(
            self as *const crate::QHashOfIntQByteArray,
        )
    }

    /// <p>Reduces the size of the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s internal hash table to save memory.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QHash<int, QByteArray>::squeeze()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#squeeze">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reduces the size of the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s internal hash table to save memory.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>'s memory usage. In general, you will rarely ever need to call this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#capacity">capacity</a>().</p></div>
    pub unsafe fn squeeze(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_squeeze(
            self as *mut crate::QHashOfIntQByteArray,
        )
    }

    /// <p>Removes the item with the <i>key</i> from the hash and returns the value associated with it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QHash<int, QByteArray>::take(const int& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#take">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item with the <i>key</i> from the hash and returns the value associated with it.</p>
    /// <p>If the item does not exist in the hash, the function simply returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If there are multiple items for <i>key</i> in the hash, only the most recently inserted one is removed.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qhash.html#remove">remove</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#remove">remove</a>().</p></div>
    pub unsafe fn take(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_take(
            self as *mut crate::QHashOfIntQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list containing all the keys in the map. Keys that occur multiple times in the map (because items were inserted with <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">insertMulti</a>(), or <a href="http://doc.qt.io/qt-5/qhash.html#unite">unite</a>() was used) occur only once in the returned list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<int> QHash<int, QByteArray>::uniqueKeys() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#uniqueKeys">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list containing all the keys in the map. Keys that occur multiple times in the map (because items were inserted with <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">insertMulti</a>(), or <a href="http://doc.qt.io/qt-5/qhash.html#unite">unite</a>() was used) occur only once in the returned list.</p>
    /// <p>This function was introduced in  Qt 4.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#keys">keys</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#values">values</a>().</p></div>
    pub unsafe fn unique_keys(&self) -> ::cpp_utils::CppBox<crate::QListOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_uniqueKeys(
            self as *const crate::QHashOfIntQByteArray,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value associated with the <i>key</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QHash<int, QByteArray>::value(const int& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value associated with the <i>key</i>.</p>
    /// <p>If the hash contains no item with the <i>key</i>, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If there are multiple items for the <i>key</i> in the hash, the value of the most recently inserted one is returned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#key">key</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#values">values</a>(), <a href="http://doc.qt.io/qt-5/qhash.html#contains">contains</a>(), and <a href="http://doc.qt.io/qt-5/qhash.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_value(
            self as *const crate::QHashOfIntQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QHash<int, QByteArray>::value(const int& key, const QByteArray& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the hash contains no item with the given <i>key</i>, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_value1(
            self as *const crate::QHashOfIntQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a list containing all the values in the hash, in an arbitrary order. If a key is associated with multiple values, all of its values will be in the list, and not just the most recently inserted one.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QByteArray> QHash<int, QByteArray>::values() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#values">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list containing all the values in the hash, in an arbitrary order. If a key is associated with multiple values, all of its values will be in the list, and not just the most recently inserted one.</p>
    /// <p>The order is guaranteed to be the same as that used by <a href="http://doc.qt.io/qt-5/qhash.html#keys">keys</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#keys">keys</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#value">value</a>().</p></div>
    pub unsafe fn values_0a(&self) -> ::cpp_utils::CppBox<crate::QListOfQByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_values(
            self as *const crate::QHashOfIntQByteArray,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QByteArray> QHash<int, QByteArray>::values(const int& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#values-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a list of all the values associated with the <i>key</i>, from the most recently inserted to the least recently inserted.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#count-1">count</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#insertMulti">insertMulti</a>().</p></div>
    pub unsafe fn values_1a(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_values1(
            self as *const crate::QHashOfIntQByteArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> class is a template class that provides a dynamic array.</p>
///
/// C++ class: <span style='color: green;'>```QVector<int>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qvector.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> class is a template class that provides a dynamic array.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores its items in adjacent memory locations and provides fast index-based access.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Here's an example of a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> that stores <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerVector;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> stringVector;
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> stores its items in a vector (array). Typically, vectors are created with an initial size. For example, the following code constructs a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> with 200 elements:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">200</span>);
///
/// </pre>
/// <p>The elements are automatically initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you want to initialize the vector with a different value, pass that value as the second argument to the constructor:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">200</span><span class="operator">,</span> <span class="string">"Pass"</span>);
///
/// </pre>
/// <p>You can also call <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>() at any time to fill the vector with a value.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const vectors, operator[]() returns a reference to the item that can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (vector<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Liz"</span>)
/// &#32;     vector<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Elizabeth"</span>;
///
/// </pre>
/// <p>For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> vector<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (vector<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Alfonso"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Alfonso at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>Another way to access the data stored in a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> is to call <a href="http://doc.qt.io/qt-5/qvector.html#data">data</a>(). The function returns a pointer to the first item in the vector. You can use the pointer to directly access and modify the elements stored in the vector. The pointer is also useful if you need to pass a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to a function that accepts a plain C++ array.</p>
/// <p>If you want to find all occurrences of a particular value in a vector, use <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of the matching item if they found one; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> vector<span class="operator">.</span>indexOf(<span class="string">"Harumi"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Harumi is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a vector contains a particular value, use <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the vector, use <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(). With the exception of <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), these functions can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>) for large vectors, because they require moving many items in the vector by one position in memory. If you want a container class that provides fast insertion/removal in the middle, use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> or <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
/// <p>Unlike plain C++ arrays, QVectors can be resized at any time by calling <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>(). If the new size is larger than the old size, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> might need to reallocate the whole vector. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> tries to reduce the number of reallocations by preallocating up to twice as much memory as the actual data needs.</p>
/// <p>If you know in advance approximately how many items the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> will contain, you can call <a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>(), asking <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to preallocate a certain amount of memory. You can also call <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>() to find out how much memory <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> actually allocated.</p>
/// <p>Note that using non-const operators and functions can cause <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to do a deep copy of the data. This is due to <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicit sharing</a>.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qvectoriterator.html">QVectorIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablevectoriterator.html">QMutableVectorIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qvector.html#const_iterator-typedef">QVector::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qvector.html#iterator-typedefx">QVector::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>.</p>
/// <p>In addition to <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>, Qt also provides <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a>, a very low-level class with little functionality that is optimized for speed.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <a name="more-information-on-using-qt-containers"></a>
/// <h4>More Information on Using Qt Containers</h4>
/// <p>For a detailed discussion comparing Qt containers with each other and with STL containers, see <a href="http://marcmutz.wordpress.com/effective-qt/containers/">Understand the Qt Containers</a>.</p></div>
#[repr(C)]
pub struct QVectorOfInt {
    _unused: u8,
}
impl QVectorOfInt {
    /// <p>Inserts <i>value</i> at the end of the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<int>::append(const int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   vector<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> three <span class="operator">=</span> <span class="string">"three"</span>;
    ///   vector<span class="operator">.</span>append(three);
    ///   <span class="comment">// vector: ["one", "two", "three"]</span>
    ///   <span class="comment">// three: "three"</span>
    ///
    /// </pre>
    /// <p>This is the same as calling resize(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() + 1) and assigning <i>value</i> to the new last element in the vector.</p>
    /// <p>This operation is relatively fast, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> typically allocates more memory than necessary, so it can grow without reallocating the entire vector each time.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>().</p></div>
    pub unsafe fn append_int(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_append(
            self as *mut crate::QVectorOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<int>::append(const QVector<int>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#append-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> vector to this vector.</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_vector_of_int(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfInt>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_append2(
            self as *mut crate::QVectorOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfInt>>::cast_into(l)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the item at index position <i>i</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const int& QVector<int>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the vector.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<::std::os::raw::c_int> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_int_at(self as *const crate::QVectorOfInt, i);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int& QVector<int>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::MutRef<::std::os::raw::c_int> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_int_back(self as *mut crate::QVectorOfInt);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const int& QVector<int>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<::std::os::raw::c_int> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_int_back1(self as *const crate::QVectorOfInt);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the maximum number of items that can be stored in the vector without forcing a reallocation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<int>::capacity() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#capacity">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the maximum number of items that can be stored in the vector without forcing a reallocation.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s memory usage. In general, you will rarely ever need to call this function. If you want to know how many items are in the vector, call <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p></div>
    pub unsafe fn capacity(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_capacity(self as *const crate::QVectorOfInt)
    }

    /// <p>Removes all the elements from the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<int>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all the elements from the vector.</p>
    /// <p><b>Note: </b>Until Qt 5.6, this also released the memory used by the vector. From Qt 5.7, the capacity is preserved. To shed all capacity, swap with a default-constructed vector:</p><pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span>T<span class="operator">&gt;</span> v <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span>T<span class="operator">&gt;</span>()<span class="operator">.</span>swap(v);
    ///   Q_ASSERT(v<span class="operator">.</span>capacity() <span class="operator">=</span><span class="operator">=</span> <span class="number">0</span>);
    ///
    /// </pre>
    /// <p>or call <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_clear(self as *mut crate::QVectorOfInt)
    }

    /// <p>Returns a const pointer to the data stored in the vector. The pointer can be used to access the items in the vector. The pointer remains valid as long as the vector isn't reallocated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const int* QVector<int>::constData() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const pointer to the data stored in the vector. The pointer can be used to access the items in the vector. The pointer remains valid as long as the vector isn't reallocated.</p>
    /// <p>This function is mostly useful to pass a vector to a function that accepts a plain C++ array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#data">data</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn const_data(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_int> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_int_constData(self as *const crate::QVectorOfInt);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns a const reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const int& QVector<int>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<::std::os::raw::c_int> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_int_constFirst(
            self as *const crate::QVectorOfInt,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const int& QVector<int>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<::std::os::raw::c_int> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_int_constLast(self as *const crate::QVectorOfInt);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the vector contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<int>::contains(const int& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the vector contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p></div>
    pub unsafe fn contains(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_contains(
            self as *const crate::QVectorOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<int>& QVector<int>::operator=(const QVector<int>& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this vector and returns a reference to this vector.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        v: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfInt>>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_int_operator_(
            self as *mut crate::QVectorOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfInt>>::cast_into(v)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of occurrences of <i>value</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<int>::count(const int& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of occurrences of <i>value</i> in the vector.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_count(
            self as *const crate::QVectorOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<int>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_count1(self as *const crate::QVectorOfInt)
    }

    /// <p>Returns a pointer to the data stored in the vector. The pointer can be used to access and modify the items in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int* QVector<int>::data()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the data stored in the vector. The pointer can be used to access and modify the items in the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> vector(<span class="number">10</span>);
    ///   <span class="type">int</span> <span class="operator">*</span>data <span class="operator">=</span> vector<span class="operator">.</span>data();
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> <span class="number">10</span>; <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     data<span class="operator">[</span>i<span class="operator">]</span> <span class="operator">=</span> <span class="number">2</span> <span class="operator">*</span> i;
    ///
    /// </pre>
    /// <p>The pointer remains valid as long as the vector isn't reallocated.</p>
    /// <p>This function is mostly useful to pass a vector to a function that accepts a plain C++ array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constData">constData</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn data_mut(&mut self) -> ::cpp_utils::MutPtr<::std::os::raw::c_int> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_int_data(self as *mut crate::QVectorOfInt);
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const int* QVector<int>::data() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#data-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn data(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_int> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_int_data1(self as *const crate::QVectorOfInt);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```void QVector<int>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_detach(self as *mut crate::QVectorOfInt)
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), returning <code>true</code> if the vector is empty; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<int>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), returning <code>true</code> if the vector is empty; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_empty(self as *const crate::QVectorOfInt)
    }

    /// <p>Returns <code>true</code> if this vector is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<int>::endsWith(const int& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this vector is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn ends_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_endsWith(
            self as *const crate::QVectorOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<int>& QVector<int>::fill(const int& t, int size = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">3</span>);
    ///   vector<span class="operator">.</span>fill(<span class="string">"Yes"</span>);
    ///   <span class="comment">// vector: ["Yes", "Yes", "Yes"]</span>
    ///
    ///   vector<span class="operator">.</span>fill(<span class="string">"oh"</span><span class="operator">,</span> <span class="number">5</span>);
    ///   <span class="comment">// vector: ["oh", "oh", "oh", "oh", "oh"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_2a(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_int_fill(
            self as *mut crate::QVectorOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
            size,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<int>& QVector<int>::fill(const int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">3</span>);
    ///   vector<span class="operator">.</span>fill(<span class="string">"Yes"</span>);
    ///   <span class="comment">// vector: ["Yes", "Yes", "Yes"]</span>
    ///
    ///   vector<span class="operator">.</span>fill(<span class="string">"oh"</span><span class="operator">,</span> <span class="number">5</span>);
    ///   <span class="comment">// vector: ["oh", "oh", "oh", "oh", "oh"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_1a(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_int_fill1(
            self as *mut crate::QVectorOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int& QVector<int>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#constFirst">constFirst</a>().</p></div>
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<::std::os::raw::c_int> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_int_first(self as *mut crate::QVectorOfInt);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const int& QVector<int>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<::std::os::raw::c_int> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_int_first1(self as *const crate::QVectorOfInt);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> object with the data contained in <i>list</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QVector<int> QVector<int>::fromList(const QList<int>& list)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#fromList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> object with the data contained in <i>list</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Sven"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Kim"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Ola"</span>;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vect <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>fromList(list);
    ///   <span class="comment">// vect: ["Sven", "Kim", "Ola"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#toList">toList</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#toVector">QList::toVector</a>().</p></div>
    pub unsafe fn from_list(
        list: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfInt>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_int_fromList(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfInt>>::cast_into(list)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int& QVector<int>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::MutRef<::std::os::raw::c_int> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_int_front(self as *mut crate::QVectorOfInt);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const int& QVector<int>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<::std::os::raw::c_int> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_int_front1(self as *const crate::QVectorOfInt);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int& QVector<int>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p>Note that using non-const operators can cause <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to do a deep copy.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<::std::os::raw::c_int> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_int_operator__2(
            self as *mut crate::QVectorOfInt,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const int& QVector<int>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as at(<i>i</i>).</p></div>
    pub unsafe fn index(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ref<::std::os::raw::c_int> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_int_operator__3(
            self as *const crate::QVectorOfInt,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<int>::indexOf(const int& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span>);            <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);         <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);         <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"X"</span>);            <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_indexOf(
            self as *const crate::QVectorOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<int>::indexOf(const int& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span>);            <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);         <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);         <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"X"</span>);            <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_indexOf1(
            self as *const crate::QVectorOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the vector. If <i>i</i> is 0, the value is prepended to the vector. If <i>i</i> is <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), the value is appended to the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<int>::insert(int i, const int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the vector. If <i>i</i> is 0, the value is prepended to the vector. If <i>i</i> is <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), the value is appended to the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   vector<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// vector: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p>For large vectors, this operation can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because it requires moving all the items at indexes <i>i</i> and above by one position further in memory. If you want a container class that provides a fast insert() function, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>().</p></div>
    pub unsafe fn insert_2a(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_insert(
            self as *mut crate::QVectorOfInt,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<int>::insert(int i, int n, const int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>count</i> copies of <i>value</i> at index position <i>i</i> in the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">double</span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">2.718</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">1.442</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">0.4342</span>;
    ///   vector<span class="operator">.</span>insert(<span class="number">1</span><span class="operator">,</span> <span class="number">3</span><span class="operator">,</span> <span class="number">9.9</span>);
    ///   <span class="comment">// vector: [2.718, 9.9, 9.9, 9.9, 1.442, 0.4342]</span>
    ///
    /// </pre></div>
    pub unsafe fn insert_3a(
        &mut self,
        i: ::std::os::raw::c_int,
        n: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_insert1(
            self as *mut crate::QVectorOfInt,
            i,
            n,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QVector<int>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_isDetached(self as *const crate::QVectorOfInt)
    }

    /// <p>Returns <code>true</code> if the vector has size 0; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<int>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the vector has size 0; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_isEmpty(self as *const crate::QVectorOfInt)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QVector<int>::isSharedWith(const QVector<int>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfInt>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_isSharedWith(
            self as *const crate::QVectorOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfInt>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int& QVector<int>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#constLast">constLast</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<::std::os::raw::c_int> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_int_last(self as *mut crate::QVectorOfInt);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const int& QVector<int>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<::std::os::raw::c_int> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_int_last1(self as *const crate::QVectorOfInt);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<int>::lastIndexOf(const int& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);        <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);     <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);     <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_lastIndexOf(
            self as *const crate::QVectorOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<int>::lastIndexOf(const int& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);        <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);     <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);     <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_lastIndexOf1(
            self as *const crate::QVectorOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<int>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#length">QList::length</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_length(self as *const crate::QVectorOfInt)
    }

    /// <p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<int> QVector<int>::mid(int pos, int len = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        len: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_int_mid(
            self as *const crate::QVectorOfInt,
            pos,
            len,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<int> QVector<int>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfInt> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_int_mid1(self as *const crate::QVectorOfInt, pos);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<int>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">QList::move</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_move(self as *mut crate::QVectorOfInt, from, to)
    }

    /// <p>Constructs an empty vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<int>::QVector()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QVectorOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_int_QVector();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a vector with an initial size of <i>size</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<int>::QVector(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a vector with an initial size of <i>size</i> elements.</p>
    /// <p>The elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn new_1a(size: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QVectorOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_int_QVector1(size);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a vector with an initial size of <i>size</i> elements. Each element is initialized with <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<int>::QVector(int size, const int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a vector with an initial size of <i>size</i> elements. Each element is initialized with <i>value</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn new_2a(
        size: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_int_QVector2(
            size,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<int>::QVector(const QVector<int>& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        v: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfInt>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_int_QVector3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfInt>>::cast_into(v)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<int>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_pop_back(self as *mut crate::QVectorOfInt)
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<int>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_pop_front(self as *mut crate::QVectorOfInt)
    }

    /// <p>Inserts <i>value</i> at the beginning of the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<int>::prepend(const int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   vector<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   vector<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// vector: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as vector.insert(0, <i>value</i>).</p>
    /// <p>For large vectors, this operation can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because it requires moving all the items in the vector by one position further in memory. If you want a container class that provides a fast prepend() function, use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> or <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_prepend(
            self as *mut crate::QVectorOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to append(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<int>::push_back(const int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to append(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_push_back(
            self as *mut crate::QVectorOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to prepend(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<int>::push_front(const int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to prepend(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_push_front(
            self as *mut crate::QVectorOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<int>::remove(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#remove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes the element at index position <i>i</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn remove_1a(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_remove(self as *mut crate::QVectorOfInt, i)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<int>::remove(int i, int n)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#remove-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes <i>count</i> elements from the middle of the vector, starting at index position <i>i</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn remove_2a(&mut self, i: ::std::os::raw::c_int, n: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_remove1(self as *mut crate::QVectorOfInt, i, n)
    }

    /// <p>Removes all elements that compare equal to <i>t</i> from the vector. Returns the number of elements removed, if any.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<int>::removeAll(const int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all elements that compare equal to <i>t</i> from the vector. Returns the number of elements removed, if any.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#removeOne">removeOne</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">QList::removeAll</a>().</p></div>
    pub unsafe fn remove_all(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_removeAll(
            self as *mut crate::QVectorOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes the element at index position <i>i</i>. Equivalent to</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<int>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the element at index position <i>i</i>. Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   remove(i);
    ///
    /// </pre>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">QList::removeAt</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_removeAt(self as *mut crate::QVectorOfInt, i)
    }

    /// <p>Removes the first item in the vector. Calling this function is equivalent to calling remove(0). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<int>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the vector. Calling this function is equivalent to calling remove(0). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_removeFirst(self as *mut crate::QVectorOfInt)
    }

    /// <p>Removes the last item in the vector. Calling this function is equivalent to calling remove(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() - 1). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<int>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the vector. Calling this function is equivalent to calling remove(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() - 1). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_removeLast(self as *mut crate::QVectorOfInt)
    }

    /// <p>Removes the first element that compares equal to <i>t</i> from the vector. Returns whether an element was, in fact, removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<int>::removeOne(const int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeOne">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first element that compares equal to <i>t</i> from the vector. Returns whether an element was, in fact, removed.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#removeAll">removeAll</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">QList::removeOne</a>().</p></div>
    pub unsafe fn remove_one(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_removeOne(
            self as *mut crate::QVectorOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<int>::replace(int i, const int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_replace(
            self as *mut crate::QVectorOfInt,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Attempts to allocate memory for at least <i>size</i> elements. If you know in advance how large the vector will be, you should call this function to prevent reallocations and memory fragmentation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<int>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to allocate memory for at least <i>size</i> elements. If you know in advance how large the vector will be, you should call this function to prevent reallocations and memory fragmentation.</p>
    /// <p>If <i>size</i> is an underestimate, the worst that will happen is that the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> will be a bit slower. If <i>size</i> is an overestimate, you may have used more memory than the normal <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> growth strategy would have allocated—or you may have used less.</p>
    /// <p>An alternative to reserve() is calling <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>(). Whether or not that is faster than reserve() depends on the element type, because <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() default-constructs all elements, and requires assignment to existing entries rather than calling <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(), which copy- or move-constructs. For simple types, like <code>int</code> or <code>double</code>, <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() is typically faster, but for anything more complex, you should prefer reserve().</p>
    /// <p><b>Warning:</b> If the size passed to <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() was underestimated, you run out of allocated space and into undefined behavior. This problem does not exist with reserve(), because it treats the size as just a hint.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>().</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_reserve(self as *mut crate::QVectorOfInt, size)
    }

    /// <p>Sets the size of the vector to <i>size</i>. If <i>size</i> is greater than the current size, elements are added to the end; the new elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If <i>size</i> is less than the current size, elements are removed from the end.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<int>::resize(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#resize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the size of the vector to <i>size</i>. If <i>size</i> is greater than the current size, elements are added to the end; the new elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If <i>size</i> is less than the current size, elements are removed from the end.</p>
    /// <p>Since Qt 5.6, resize() doesn't shrink the capacity anymore. To shed excess capacity, use <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p></div>
    pub unsafe fn resize(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_resize(self as *mut crate::QVectorOfInt, size)
    }

    /// Calls C++ function: <span style='color: green;'>```void QVector<int>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_setSharable(
            self as *mut crate::QVectorOfInt,
            sharable,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<int>::shrink_to_fit()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#shrink_to_fit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn shrink_to_fit(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_shrink_to_fit(self as *mut crate::QVectorOfInt)
    }

    /// <p>Returns the number of items in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<int>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_size(self as *const crate::QVectorOfInt)
    }

    /// <p>Releases any memory not required to store the items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<int>::squeeze()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Releases any memory not required to store the items.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s memory usage. In general, you will rarely ever need to call this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>().</p></div>
    pub unsafe fn squeeze(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_squeeze(self as *mut crate::QVectorOfInt)
    }

    /// <p>Returns <code>true</code> if this vector is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<int>::startsWith(const int& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this vector is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn starts_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_startsWith(
            self as *const crate::QVectorOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Swaps vector <i>other</i> with this vector. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<int>::swap(QVector<int>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps vector <i>other</i> with this vector. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QVectorOfInt>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_swap(
            self as *mut crate::QVectorOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QVectorOfInt>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Removes the element at index position <i>i</i> and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<int>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the element at index position <i>i</i> and returns it.</p>
    /// <p>Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   T t <span class="operator">=</span> at(i);
    ///   remove(i);
    ///   <span class="keyword">return</span> t;
    ///
    /// </pre>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">QList::takeAt</a>().</p></div>
    pub unsafe fn take_at(&mut self, i: ::std::os::raw::c_int) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_takeAt(self as *mut crate::QVectorOfInt, i)
    }

    /// <p>Removes the first item in the vector and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<int>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the vector and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_takeFirst(self as *mut crate::QVectorOfInt)
    }

    /// <p>Removes the last item in the list and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<int>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_takeLast(self as *mut crate::QVectorOfInt)
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> object with the data contained in this <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<int> QVector<int>::toList() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#toList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> object with the data contained in this <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">double</span><span class="operator">&gt;</span> vect;
    ///   vect <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"red"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"green"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"blue"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"black"</span>;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">double</span><span class="operator">&gt;</span> list <span class="operator">=</span> vect<span class="operator">.</span>toList();
    ///   <span class="comment">// list: ["red", "green", "blue", "black"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#fromList">fromList</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#fromVector">QList::fromVector</a>().</p></div>
    pub unsafe fn to_list(&self) -> ::cpp_utils::CppBox<crate::QListOfInt> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_int_toList(self as *const crate::QVectorOfInt);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value at index position <i>i</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<int>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the vector.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that <i>i</i> is within bounds, you can use <a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(&self, i: ::std::os::raw::c_int) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_value(self as *const crate::QVectorOfInt, i)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<int>::value(int i, const int& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_value1(
            self as *const crate::QVectorOfInt,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(
                default_value,
            )
            .as_raw_ptr(),
        )
    }
}

pub mod q_list_of_q_persistent_model_index {
    //! C++ type: <span style='color: green;'>```QList<QPersistentModelIndex>```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QPersistentModelIndex>::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::iterator& QList<QPersistentModelIndex>::iterator::operator=(const QList<QPersistentModelIndex>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::Iterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_persistent_model_index::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_iterator_operator_8(
                    self as *mut crate::q_list_of_q_persistent_model_index::Iterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::Iterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::iterator QList<QPersistentModelIndex>::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_iterator_operator__13(
                    self as *mut crate::q_list_of_q_persistent_model_index::Iterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::iterator QList<QPersistentModelIndex>::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_iterator_operator__11(
                    self as *mut crate::q_list_of_q_persistent_model_index::Iterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QPersistentModelIndex& QList<QPersistentModelIndex>::iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValueRef iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>. If you assign to it, the assignment will apply to the character in the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> of <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> from which you got the reference.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::MutRef<crate::QPersistentModelIndex> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_iterator_operator__1(
                    self as *const crate::q_list_of_q_persistent_model_index::Iterator,
                    j,
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QPersistentModelIndex>::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>().</p></div>
        pub unsafe fn new(
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QPersistentModelIndex>::iterator::iterator(const QList<QPersistentModelIndex>::iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::Iterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_iterator_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::Iterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QPersistentModelIndex* QList<QPersistentModelIndex>::iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's value.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::MutPtr<crate::QPersistentModelIndex> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_iterator_operator__(
                    self as *const crate::q_list_of_q_persistent_model_index::Iterator,
                );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QPersistentModelIndex>::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::const_iterator& QList<QPersistentModelIndex>::const_iterator::operator=(const QList<QPersistentModelIndex>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::ConstIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_persistent_model_index::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_const_iterator_operator_6(
                    self as *mut crate::q_list_of_q_persistent_model_index::ConstIterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::ConstIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::const_iterator QList<QPersistentModelIndex>::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_const_iterator_operator__9(self as *mut crate::q_list_of_q_persistent_model_index::ConstIterator, arg1);
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::const_iterator QList<QPersistentModelIndex>::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_const_iterator_operator__7(self as *mut crate::q_list_of_q_persistent_model_index::ConstIterator, arg1);
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QPersistentModelIndex& QList<QPersistentModelIndex>::const_iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValue const_iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::Ref<crate::QPersistentModelIndex> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_const_iterator_operator__1(self as *const crate::q_list_of_q_persistent_model_index::ConstIterator, j);
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QPersistentModelIndex>::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>().</p></div>
        pub unsafe fn new_0a(
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_const_iterator_const_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QPersistentModelIndex>::const_iterator::const_iterator(const QList<QPersistentModelIndex>::iterator& o)```</span>.
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::Iterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_const_iterator_const_iterator2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QPersistentModelIndex>::const_iterator::const_iterator(const QList<QPersistentModelIndex>::const_iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::ConstIterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_const_iterator_const_iterator1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::ConstIterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current result.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QPersistentModelIndex* QList<QPersistentModelIndex>::const_iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current result.</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::Ptr<crate::QPersistentModelIndex> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_const_iterator_operator__(
                    self as *const crate::q_list_of_q_persistent_model_index::ConstIterator,
                );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_persistent_model_index::Iterator {
        type Output = ::cpp_utils::MutRef<crate::QPersistentModelIndex>;
        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QPersistentModelIndex& QList<QPersistentModelIndex>::iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>().</p></div>
        fn indirection(self) -> ::cpp_utils::MutRef<crate::QPersistentModelIndex> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_iterator_operator_(
                    self as *const crate::q_list_of_q_persistent_model_index::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl
        ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::Iterator>>
        for crate::q_list_of_q_persistent_model_index::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPersistentModelIndex>::iterator::operator==(const QList<QPersistentModelIndex>::iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_iterator_operator__2(
                    self as *const crate::q_list_of_q_persistent_model_index::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::Iterator>>
        for crate::q_list_of_q_persistent_model_index::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPersistentModelIndex>::iterator::operator<(const QList<QPersistentModelIndex>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_iterator_operator_1(
                    self as *const crate::q_list_of_q_persistent_model_index::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::Iterator>>
        for crate::q_list_of_q_persistent_model_index::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPersistentModelIndex>::iterator::operator<=(const QList<QPersistentModelIndex>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_iterator_operator__4(
                    self as *const crate::q_list_of_q_persistent_model_index::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::Iterator>>
        for crate::q_list_of_q_persistent_model_index::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPersistentModelIndex>::iterator::operator>(const QList<QPersistentModelIndex>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_iterator_operator_2(
                    self as *const crate::q_list_of_q_persistent_model_index::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::Iterator>>
        for crate::q_list_of_q_persistent_model_index::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPersistentModelIndex>::iterator::operator>=(const QList<QPersistentModelIndex>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_iterator_operator__5(
                    self as *const crate::q_list_of_q_persistent_model_index::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::std::cmp::PartialEq<
            ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::ConstIterator>,
        > for crate::q_list_of_q_persistent_model_index::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPersistentModelIndex>::iterator::operator==(const QList<QPersistentModelIndex>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_iterator_operator__6(
                    self as *const crate::q_list_of_q_persistent_model_index::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Lt<
            ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::ConstIterator>,
        > for crate::q_list_of_q_persistent_model_index::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPersistentModelIndex>::iterator::operator<(const QList<QPersistentModelIndex>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_iterator_operator_3(
                    self as *const crate::q_list_of_q_persistent_model_index::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Le<
            ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::ConstIterator>,
        > for crate::q_list_of_q_persistent_model_index::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPersistentModelIndex>::iterator::operator<=(const QList<QPersistentModelIndex>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_iterator_operator__8(
                    self as *const crate::q_list_of_q_persistent_model_index::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Gt<
            ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::ConstIterator>,
        > for crate::q_list_of_q_persistent_model_index::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPersistentModelIndex>::iterator::operator>(const QList<QPersistentModelIndex>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_iterator_operator_4(
                    self as *const crate::q_list_of_q_persistent_model_index::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Ge<
            ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::ConstIterator>,
        > for crate::q_list_of_q_persistent_model_index::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPersistentModelIndex>::iterator::operator>=(const QList<QPersistentModelIndex>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_iterator_operator__9(
                    self as *const crate::q_list_of_q_persistent_model_index::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_persistent_model_index::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_persistent_model_index::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::iterator& QList<QPersistentModelIndex>::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_persistent_model_index::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_iterator_operator__10(
                    self as *mut crate::q_list_of_q_persistent_model_index::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_persistent_model_index::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_persistent_model_index::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::iterator& QList<QPersistentModelIndex>::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_persistent_model_index::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_iterator_operator__12(
                    self as *mut crate::q_list_of_q_persistent_model_index::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_persistent_model_index::Iterator
    {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::iterator& QList<QPersistentModelIndex>::iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_iterator_operator__14(
                    self as *mut crate::q_list_of_q_persistent_model_index::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_persistent_model_index::Iterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::iterator& QList<QPersistentModelIndex>::iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_iterator_operator__15(
                    self as *mut crate::q_list_of_q_persistent_model_index::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_persistent_model_index::Iterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::iterator QList<QPersistentModelIndex>::iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_iterator_operator_5(
                    self as *const crate::q_list_of_q_persistent_model_index::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_persistent_model_index::Iterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::iterator QList<QPersistentModelIndex>::iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_iterator_operator_6(
                    self as *const crate::q_list_of_q_persistent_model_index::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::Iterator>>
        for &crate::q_list_of_q_persistent_model_index::Iterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QPersistentModelIndex>::iterator::operator-(QList<QPersistentModelIndex>::iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::Iterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_iterator_operator_7(
                    self as *const crate::q_list_of_q_persistent_model_index::Iterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_persistent_model_index::ConstIterator {
        type Output = ::cpp_utils::Ref<crate::QPersistentModelIndex>;
        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QPersistentModelIndex& QList<QPersistentModelIndex>::const_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        fn indirection(self) -> ::cpp_utils::Ref<crate::QPersistentModelIndex> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_const_iterator_operator_(
                    self as *const crate::q_list_of_q_persistent_model_index::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
            }
        }
    }

    impl
        ::std::cmp::PartialEq<
            ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::ConstIterator>,
        > for crate::q_list_of_q_persistent_model_index::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPersistentModelIndex>::const_iterator::operator==(const QList<QPersistentModelIndex>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_const_iterator_operator__2(
                    self as *const crate::q_list_of_q_persistent_model_index::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Lt<
            ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::ConstIterator>,
        > for crate::q_list_of_q_persistent_model_index::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPersistentModelIndex>::const_iterator::operator<(const QList<QPersistentModelIndex>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_const_iterator_operator_1(
                    self as *const crate::q_list_of_q_persistent_model_index::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Le<
            ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::ConstIterator>,
        > for crate::q_list_of_q_persistent_model_index::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPersistentModelIndex>::const_iterator::operator<=(const QList<QPersistentModelIndex>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_const_iterator_operator__4(
                    self as *const crate::q_list_of_q_persistent_model_index::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Gt<
            ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::ConstIterator>,
        > for crate::q_list_of_q_persistent_model_index::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPersistentModelIndex>::const_iterator::operator>(const QList<QPersistentModelIndex>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_const_iterator_operator_2(
                    self as *const crate::q_list_of_q_persistent_model_index::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Ge<
            ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::ConstIterator>,
        > for crate::q_list_of_q_persistent_model_index::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPersistentModelIndex>::const_iterator::operator>=(const QList<QPersistentModelIndex>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_const_iterator_operator__5(
                    self as *const crate::q_list_of_q_persistent_model_index::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_persistent_model_index::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_persistent_model_index::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::const_iterator& QList<QPersistentModelIndex>::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(
            self,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_persistent_model_index::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_const_iterator_operator__6(
                    self as *mut crate::q_list_of_q_persistent_model_index::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_persistent_model_index::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_persistent_model_index::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::const_iterator& QList<QPersistentModelIndex>::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(
            self,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_persistent_model_index::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_const_iterator_operator__8(
                    self as *mut crate::q_list_of_q_persistent_model_index::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_persistent_model_index::ConstIterator
    {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::const_iterator& QList<QPersistentModelIndex>::const_iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_const_iterator_operator__10(self as *mut crate::q_list_of_q_persistent_model_index::ConstIterator, j)
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_persistent_model_index::ConstIterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::const_iterator& QList<QPersistentModelIndex>::const_iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_const_iterator_operator__11(self as *mut crate::q_list_of_q_persistent_model_index::ConstIterator, j)
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_persistent_model_index::ConstIterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::const_iterator QList<QPersistentModelIndex>::const_iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_const_iterator_operator_3(
                    self as *const crate::q_list_of_q_persistent_model_index::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_persistent_model_index::ConstIterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::const_iterator QList<QPersistentModelIndex>::const_iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_const_iterator_operator_4(
                    self as *const crate::q_list_of_q_persistent_model_index::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::ConstIterator>>
        for &crate::q_list_of_q_persistent_model_index::ConstIterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QPersistentModelIndex>::const_iterator::operator-(QList<QPersistentModelIndex>::const_iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::ConstIterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_const_iterator_operator_5(
                    self as *const crate::q_list_of_q_persistent_model_index::ConstIterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_persistent_model_index::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QPersistentModelIndex>::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_iterator_diterator(
                self as *mut crate::q_list_of_q_persistent_model_index::Iterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_persistent_model_index::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QPersistentModelIndex>::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_const_iterator_dconst_iterator(
                self as *mut crate::q_list_of_q_persistent_model_index::ConstIterator,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
///
/// C++ class: <span style='color: green;'>```QList<QPersistentModelIndex>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlist.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores items in a list that provides fast index-based access and index-based insertions and removals.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Internally, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T if <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. Otherwise, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T* and the items are allocated on the heap.</p>
/// <p>The array representation allows very fast insertions and index-based access. The <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() operations are also very fast because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates memory at both ends of its internal array. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.</p>
/// <p>Note, however, that when the conditions specified above are not met, each append or insert of a new item requires allocating the new item on the heap, and this per item allocation will make <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> a better choice for use cases that do a lot of appending or inserting, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> can allocate memory for many items in a single heap allocation.</p>
/// <p>Note that the internal array only ever gets bigger over the life of the list. It never shrinks. The internal array is deallocated by the destructor and by the assignment operator, when one list is assigned to another.</p>
/// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerList;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span><span class="operator">&gt;</span> dateList;
///
/// </pre>
/// <p>Qt includes a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class that inherits <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>&gt; and adds a few convenience functions, such as <a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#filter">QStringList::filter</a>(). <a href="http://doc.qt.io/qt-5/qstring.html#split">QString::split</a>() creates QStringLists from strings.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> stores a list of items. The default constructor creates an empty list. To insert items into the list, you can use operator&lt;&lt;():</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"one"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"two"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"three"</span>;
///   <span class="comment">// list: ["one", "two", "three"]</span>
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(). In addition, it provides the following convenience functions: <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const lists, operator[]() returns a reference to the item and can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Bob"</span>)
/// &#32;     list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Robert"</span>;
///
/// </pre>
/// <p>Because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented as an array of pointers for types that are larger than a pointer or are not movable, this operation requires (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>). For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> list<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (list<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Jane"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Jane at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>A common requirement is to remove an item from a list and do something with it. For this, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(). Here's a loop that removes the items from a list one at a time and calls <code>delete</code> on them:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   <span class="keyword">while</span> (<span class="operator">!</span>list<span class="operator">.</span>isEmpty())
/// &#32;     <span class="keyword">delete</span> list<span class="operator">.</span>takeFirst();
///
/// </pre>
/// <p>Inserting and removing items at either end of the list is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> in most cases), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
/// <p>If you want to find all occurrences of a particular value in a list, use <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of a matching item if they find it; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> list<span class="operator">.</span>indexOf(<span class="string">"Jane"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Jane is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a list contains a particular value, use <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the list, use <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>(). If you want to replace all occurrences of a particular value with another, use <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlistiterator.html">QListIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablelistiterator.html">QMutableListIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented in such a way that direct index-based access is just as fast as using iterators.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <p>To make <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as efficient as possible, its member functions don't validate their input before using it. Except for <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), member functions always assume the list is <i>not</i> empty. Member functions that take index values as parameters always assume their index value parameters are in the valid range. This means <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> member functions can fail. If you define QT_NO_DEBUG when you compile, failures will not be detected. If you <i>don't</i> define QT_NO_DEBUG, failures will be detected using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT">Q_ASSERT</a>() or <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT_X">Q_ASSERT_X</a>() with an appropriate message.</p>
/// <p>To avoid failures when your list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling other member functions. If you must pass an index value that might not be in the valid range, check that it is less than the value returned by <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() but <i>not</i> less than 0.</p>
/// <a name="more-members"></a></div>
#[repr(C)]
pub struct QListOfQPersistentModelIndex {
    _unused: u8,
}
impl QListOfQPersistentModelIndex {
    /// <p>Inserts <i>value</i> at the end of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPersistentModelIndex>::append(const QPersistentModelIndex& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["one", "two", "three"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn append_q_persistent_model_index(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPersistentModelIndex>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_append(
            self as *mut crate::QListOfQPersistentModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPersistentModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPersistentModelIndex>::append(const QList<QPersistentModelIndex>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> list to this list.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_list_of_q_persistent_model_index(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_append1(self as *mut crate::QListOfQPersistentModelIndex, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPersistentModelIndex& QList<QPersistentModelIndex>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>This function is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ref<crate::QPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_at(
            self as *const crate::QListOfQPersistentModelIndex,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPersistentModelIndex& QList<QPersistentModelIndex>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::MutRef<crate::QPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_back(
            self as *mut crate::QListOfQPersistentModelIndex,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPersistentModelIndex& QList<QPersistentModelIndex>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<crate::QPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_back1(
            self as *const crate::QListOfQPersistentModelIndex,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::const_iterator QList<QPersistentModelIndex>::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_cbegin(
            self as *const crate::QListOfQPersistentModelIndex,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::const_iterator QList<QPersistentModelIndex>::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn cend(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_cend(
            self as *const crate::QListOfQPersistentModelIndex,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all items from the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPersistentModelIndex>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all items from the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_clear(
            self as *mut crate::QListOfQPersistentModelIndex,
        )
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::const_iterator QList<QPersistentModelIndex>::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_constBegin(
            self as *const crate::QListOfQPersistentModelIndex,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::const_iterator QList<QPersistentModelIndex>::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn const_end(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_constEnd(
            self as *const crate::QListOfQPersistentModelIndex,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPersistentModelIndex& QList<QPersistentModelIndex>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>().</p></div>
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<crate::QPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_constFirst(
            self as *const crate::QListOfQPersistentModelIndex,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPersistentModelIndex& QList<QPersistentModelIndex>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>().</p></div>
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<crate::QPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_constLast(
            self as *const crate::QListOfQPersistentModelIndex,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QPersistentModelIndex>::contains(const QPersistentModelIndex& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn contains(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPersistentModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_contains(
            self as *const crate::QListOfQPersistentModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPersistentModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>& QList<QPersistentModelIndex>::operator=(const QList<QPersistentModelIndex>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this list and returns a reference to this list.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_operator_(self as *mut crate::QListOfQPersistentModelIndex, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>>::cast_into(l).as_raw_ptr());
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of occurrences of <i>value</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QPersistentModelIndex>::count(const QPersistentModelIndex& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of occurrences of <i>value</i> in the list.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPersistentModelIndex>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_count(
            self as *const crate::QListOfQPersistentModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPersistentModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QPersistentModelIndex>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_count1(
            self as *const crate::QListOfQPersistentModelIndex,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QPersistentModelIndex>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_detach(
            self as *mut crate::QListOfQPersistentModelIndex,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QPersistentModelIndex>::detachShared()```</span>.
    pub unsafe fn detach_shared(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_detachShared(
            self as *mut crate::QListOfQPersistentModelIndex,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QPersistentModelIndex>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_empty(
            self as *const crate::QListOfQPersistentModelIndex,
        )
    }

    /// <p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QPersistentModelIndex>::endsWith(const QPersistentModelIndex& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn ends_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPersistentModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_endsWith(
            self as *const crate::QListOfQPersistentModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPersistentModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::iterator QList<QPersistentModelIndex>::erase(QList<QPersistentModelIndex>::iterator pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn erase_1a(
        &mut self,
        pos: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::Iterator>,
        >,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_erase(
            self as *mut crate::QListOfQPersistentModelIndex,
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::Iterator>,
            >::cast_into(pos)
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::iterator QList<QPersistentModelIndex>::erase(QList<QPersistentModelIndex>::iterator first, QList<QPersistentModelIndex>::iterator last)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes all the items from <i>begin</i> up to (but not including) <i>end</i>. Returns an iterator to the same item that <i>end</i> referred to before the call.</p></div>
    pub unsafe fn erase_2a(
        &mut self,
        first: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::Iterator>,
        >,
        last: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::Iterator>,
        >,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_erase1(
            self as *mut crate::QListOfQPersistentModelIndex,
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::Iterator>,
            >::cast_into(first)
            .as_raw_ptr(),
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::Iterator>,
            >::cast_into(last)
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPersistentModelIndex& QList<QPersistentModelIndex>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<crate::QPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_first(
            self as *mut crate::QListOfQPersistentModelIndex,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPersistentModelIndex& QList<QPersistentModelIndex>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::QPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_first1(
            self as *const crate::QListOfQPersistentModelIndex,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPersistentModelIndex& QList<QPersistentModelIndex>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::MutRef<crate::QPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_front(
            self as *mut crate::QListOfQPersistentModelIndex,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPersistentModelIndex& QList<QPersistentModelIndex>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<crate::QPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_front1(
            self as *const crate::QListOfQPersistentModelIndex,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPersistentModelIndex& QList<QPersistentModelIndex>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>(). This function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p></div>
    pub unsafe fn index(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ref<crate::QPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_operator__2(
            self as *const crate::QListOfQPersistentModelIndex,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPersistentModelIndex& QList<QPersistentModelIndex>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If this function is called on a list that is currently being shared, it will trigger a copy of all elements. Otherwise, this function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>. If you do not want to modify the list you should use <a href="http://doc.qt.io/qt-5/qlist.html#at">QList::at</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_operator__3(
            self as *mut crate::QListOfQPersistentModelIndex,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QPersistentModelIndex>::indexOf(const QPersistentModelIndex& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPersistentModelIndex>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_indexOf(
            self as *const crate::QListOfQPersistentModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPersistentModelIndex>>::cast_into(t)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QPersistentModelIndex>::indexOf(const QPersistentModelIndex& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPersistentModelIndex>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_indexOf1(
            self as *const crate::QListOfQPersistentModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPersistentModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPersistentModelIndex>::insert(int i, const QPersistentModelIndex& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   list<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// list: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn insert_int_q_persistent_model_index(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPersistentModelIndex>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_insert(
            self as *mut crate::QListOfQPersistentModelIndex,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPersistentModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::iterator QList<QPersistentModelIndex>::insert(QList<QPersistentModelIndex>::iterator before, const QPersistentModelIndex& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>value</i> in front of the item pointed to by the iterator <i>before</i>. Returns an iterator pointing at the inserted item. Note that the iterator passed to the function will be invalid after the call; the returned iterator should be used instead.</p></div>
    pub unsafe fn insert_iterator_q_persistent_model_index(
        &mut self,
        before: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::Iterator>,
        >,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPersistentModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_insert1(
            self as *mut crate::QListOfQPersistentModelIndex,
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::q_list_of_q_persistent_model_index::Iterator>,
            >::cast_into(before)
            .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPersistentModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QPersistentModelIndex>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_isDetached(
            self as *const crate::QListOfQPersistentModelIndex,
        )
    }

    /// <p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QPersistentModelIndex>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_isEmpty(
            self as *const crate::QListOfQPersistentModelIndex,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QPersistentModelIndex>::isSharedWith(const QList<QPersistentModelIndex>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_isSharedWith(self as *const crate::QListOfQPersistentModelIndex, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>>::cast_into(other).as_raw_ptr())
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPersistentModelIndex& QList<QPersistentModelIndex>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<crate::QPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_last(
            self as *mut crate::QListOfQPersistentModelIndex,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPersistentModelIndex& QList<QPersistentModelIndex>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::QPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_last1(
            self as *const crate::QListOfQPersistentModelIndex,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QPersistentModelIndex>::lastIndexOf(const QPersistentModelIndex& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPersistentModelIndex>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_lastIndexOf(
            self as *const crate::QListOfQPersistentModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPersistentModelIndex>>::cast_into(t)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QPersistentModelIndex>::lastIndexOf(const QPersistentModelIndex& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPersistentModelIndex>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_lastIndexOf1(
            self as *const crate::QListOfQPersistentModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPersistentModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QPersistentModelIndex>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_length(
            self as *const crate::QListOfQPersistentModelIndex,
        )
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex> QList<QPersistentModelIndex>::mid(int pos, int length = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        length: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_mid(
            self as *const crate::QListOfQPersistentModelIndex,
            pos,
            length,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex> QList<QPersistentModelIndex>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_mid1(
            self as *const crate::QListOfQPersistentModelIndex,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPersistentModelIndex>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>move(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "C", "D", "E", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This is the same as insert(<i>to</i>, <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<i>from</i>)).This function assumes that both <i>from</i> and <i>to</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>from</i> and <i>to</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_move(
            self as *mut crate::QListOfQPersistentModelIndex,
            from,
            to,
        )
    }

    /// <p>Constructs an empty list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QPersistentModelIndex>::QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty list.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QListOfQPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_QList();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QPersistentModelIndex>::QList(const QList<QPersistentModelIndex>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_QList1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>>::cast_into(l).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPersistentModelIndex>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_pop_back(
            self as *mut crate::QListOfQPersistentModelIndex,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPersistentModelIndex>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_pop_front(
            self as *mut crate::QListOfQPersistentModelIndex,
        )
    }

    /// <p>Inserts <i>value</i> at the beginning of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPersistentModelIndex>::prepend(const QPersistentModelIndex& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(0, <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPersistentModelIndex>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_prepend(
            self as *mut crate::QListOfQPersistentModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPersistentModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPersistentModelIndex>::push_back(const QPersistentModelIndex& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPersistentModelIndex>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_push_back(
            self as *mut crate::QListOfQPersistentModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPersistentModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPersistentModelIndex>::push_front(const QPersistentModelIndex& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPersistentModelIndex>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_push_front(
            self as *mut crate::QListOfQPersistentModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPersistentModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QPersistentModelIndex>::removeAll(const QPersistentModelIndex& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeAll(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_all(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPersistentModelIndex>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_removeAll(
            self as *mut crate::QListOfQPersistentModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPersistentModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPersistentModelIndex>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_removeAt(
            self as *mut crate::QListOfQPersistentModelIndex,
            i,
        )
    }

    /// <p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPersistentModelIndex>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_removeFirst(
            self as *mut crate::QListOfQPersistentModelIndex,
        )
    }

    /// <p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPersistentModelIndex>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_removeLast(
            self as *mut crate::QListOfQPersistentModelIndex,
        )
    }

    /// <p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QPersistentModelIndex>::removeOne(const QPersistentModelIndex& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeOne(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", ,"sun", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_one(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPersistentModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_removeOne(
            self as *mut crate::QListOfQPersistentModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPersistentModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPersistentModelIndex>::replace(int i, const QPersistentModelIndex& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPersistentModelIndex>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_replace(
            self as *mut crate::QListOfQPersistentModelIndex,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPersistentModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Reserve space for <i>alloc</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPersistentModelIndex>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reserve space for <i>alloc</i> elements.</p>
    /// <p>If <i>alloc</i> is smaller than the current size of the list, nothing will happen.</p>
    /// <p>Use this function to avoid repetetive reallocation of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s internal data if you can predict how many elements will be appended. Note that the reservation applies only to the internal pointer array.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_reserve(
            self as *mut crate::QListOfQPersistentModelIndex,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QPersistentModelIndex>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_setSharable(
            self as *mut crate::QListOfQPersistentModelIndex,
            sharable,
        )
    }

    /// <p>Returns the number of items in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QPersistentModelIndex>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_size(
            self as *const crate::QListOfQPersistentModelIndex,
        )
    }

    /// <p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QPersistentModelIndex>::startsWith(const QPersistentModelIndex& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn starts_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPersistentModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_startsWith(
            self as *const crate::QListOfQPersistentModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPersistentModelIndex>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPersistentModelIndex>::swap(QList<QPersistentModelIndex>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap_1a(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QListOfQPersistentModelIndex>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_swap(self as *mut crate::QListOfQPersistentModelIndex, ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QListOfQPersistentModelIndex>>::cast_into(other).as_mut_raw_ptr())
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPersistentModelIndex>::swap(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swap(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    pub unsafe fn swap_2a(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_swap1(
            self as *mut crate::QListOfQPersistentModelIndex,
            i,
            j,
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPersistentModelIndex>::swapItemsAt(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swapItemsAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swapItemsAt(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn swap_items_at(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_swapItemsAt(
            self as *mut crate::QListOfQPersistentModelIndex,
            i,
            j,
        )
    }

    /// <p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPersistentModelIndex QList<QPersistentModelIndex>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_takeAt(
            self as *mut crate::QListOfQPersistentModelIndex,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPersistentModelIndex QList<QPersistentModelIndex>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> ::cpp_utils::CppBox<crate::QPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_takeFirst(
            self as *mut crate::QListOfQPersistentModelIndex,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPersistentModelIndex QList<QPersistentModelIndex>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> ::cpp_utils::CppBox<crate::QPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_takeLast(
            self as *mut crate::QListOfQPersistentModelIndex,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value at index position <i>i</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPersistentModelIndex QList<QPersistentModelIndex>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the list.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that the index is going to be within bounds, you can use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_value(
            self as *const crate::QListOfQPersistentModelIndex,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPersistentModelIndex QList<QPersistentModelIndex>::value(int i, const QPersistentModelIndex& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPersistentModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_value1(
            self as *const crate::QListOfQPersistentModelIndex,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPersistentModelIndex>>::cast_into(
                default_value,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_list_of_q_abstract_state {
    //! C++ type: <span style='color: green;'>```QList<QAbstractState*>```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QAbstractState*>::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::iterator& QList<QAbstractState*>::iterator::operator=(const QList<QAbstractState*>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_abstract_state::Iterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_abstract_state::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_iterator_operator_8(
                    self as *mut crate::q_list_of_q_abstract_state::Iterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_abstract_state::Iterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::iterator QList<QAbstractState*>::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_iterator_operator__13(
                    self as *mut crate::q_list_of_q_abstract_state::Iterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::iterator QList<QAbstractState*>::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_iterator_operator__11(
                    self as *mut crate::q_list_of_q_abstract_state::Iterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QAbstractState*& QList<QAbstractState*>::iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValueRef iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>. If you assign to it, the assignment will apply to the character in the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> of <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> from which you got the reference.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::MutRef<*mut crate::QAbstractState> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_iterator_operator__1(
                    self as *const crate::q_list_of_q_abstract_state::Iterator,
                    j,
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractState*>::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>().</p></div>
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractState*>::iterator::iterator(const QList<QAbstractState*>::iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_abstract_state::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_iterator_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_abstract_state::Iterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QAbstractState** QList<QAbstractState*>::iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's value.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::MutPtr<*mut crate::QAbstractState> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_iterator_operator__(
                    self as *const crate::q_list_of_q_abstract_state::Iterator,
                );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QAbstractState*>::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::const_iterator& QList<QAbstractState*>::const_iterator::operator=(const QList<QAbstractState*>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_abstract_state::ConstIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_abstract_state::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_const_iterator_operator_6(
                    self as *mut crate::q_list_of_q_abstract_state::ConstIterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_abstract_state::ConstIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::const_iterator QList<QAbstractState*>::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_const_iterator_operator__9(
                    self as *mut crate::q_list_of_q_abstract_state::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::const_iterator QList<QAbstractState*>::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_const_iterator_operator__7(
                    self as *mut crate::q_list_of_q_abstract_state::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractState*>::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>().</p></div>
        pub unsafe fn new_0a(
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_const_iterator_const_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractState*>::const_iterator::const_iterator(const QList<QAbstractState*>::iterator& o)```</span>.
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_abstract_state::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_const_iterator_const_iterator2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_abstract_state::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractState*>::const_iterator::const_iterator(const QList<QAbstractState*>::const_iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_abstract_state::ConstIterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_const_iterator_const_iterator1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_abstract_state::ConstIterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_abstract_state::Iterator {
        type Output = ::cpp_utils::MutRef<*mut crate::QAbstractState>;
        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QAbstractState*& QList<QAbstractState*>::iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>().</p></div>
        fn indirection(self) -> ::cpp_utils::MutRef<*mut crate::QAbstractState> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_iterator_operator_(
                    self as *const crate::q_list_of_q_abstract_state::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_abstract_state::Iterator>>
        for crate::q_list_of_q_abstract_state::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractState*>::iterator::operator==(const QList<QAbstractState*>::iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_abstract_state::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_iterator_operator__2(
                    self as *const crate::q_list_of_q_abstract_state::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_abstract_state::Iterator>>
        for crate::q_list_of_q_abstract_state::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractState*>::iterator::operator<(const QList<QAbstractState*>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_state::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_iterator_operator_1(
                    self as *const crate::q_list_of_q_abstract_state::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_abstract_state::Iterator>>
        for crate::q_list_of_q_abstract_state::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractState*>::iterator::operator<=(const QList<QAbstractState*>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_state::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_iterator_operator__4(
                    self as *const crate::q_list_of_q_abstract_state::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_abstract_state::Iterator>>
        for crate::q_list_of_q_abstract_state::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractState*>::iterator::operator>(const QList<QAbstractState*>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_state::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_iterator_operator_2(
                    self as *const crate::q_list_of_q_abstract_state::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_abstract_state::Iterator>>
        for crate::q_list_of_q_abstract_state::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractState*>::iterator::operator>=(const QList<QAbstractState*>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_state::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_iterator_operator__5(
                    self as *const crate::q_list_of_q_abstract_state::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_abstract_state::ConstIterator>>
        for crate::q_list_of_q_abstract_state::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractState*>::iterator::operator==(const QList<QAbstractState*>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_list_of_q_abstract_state::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_iterator_operator__6(
                    self as *const crate::q_list_of_q_abstract_state::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_abstract_state::ConstIterator>>
        for crate::q_list_of_q_abstract_state::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractState*>::iterator::operator<(const QList<QAbstractState*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_state::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_iterator_operator_3(
                    self as *const crate::q_list_of_q_abstract_state::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_abstract_state::ConstIterator>>
        for crate::q_list_of_q_abstract_state::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractState*>::iterator::operator<=(const QList<QAbstractState*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_state::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_iterator_operator__8(
                    self as *const crate::q_list_of_q_abstract_state::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_abstract_state::ConstIterator>>
        for crate::q_list_of_q_abstract_state::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractState*>::iterator::operator>(const QList<QAbstractState*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_state::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_iterator_operator_4(
                    self as *const crate::q_list_of_q_abstract_state::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_abstract_state::ConstIterator>>
        for crate::q_list_of_q_abstract_state::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractState*>::iterator::operator>=(const QList<QAbstractState*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_state::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_iterator_operator__9(
                    self as *const crate::q_list_of_q_abstract_state::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_abstract_state::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_abstract_state::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::iterator& QList<QAbstractState*>::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_abstract_state::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_iterator_operator__10(
                    self as *mut crate::q_list_of_q_abstract_state::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_abstract_state::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_abstract_state::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::iterator& QList<QAbstractState*>::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_abstract_state::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_iterator_operator__12(
                    self as *mut crate::q_list_of_q_abstract_state::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_abstract_state::Iterator
    {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::iterator& QList<QAbstractState*>::iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_iterator_operator__14(
                    self as *mut crate::q_list_of_q_abstract_state::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_abstract_state::Iterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::iterator& QList<QAbstractState*>::iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_iterator_operator__15(
                    self as *mut crate::q_list_of_q_abstract_state::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_list_of_q_abstract_state::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::iterator QList<QAbstractState*>::iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_iterator_operator_5(
                    self as *const crate::q_list_of_q_abstract_state::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_list_of_q_abstract_state::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::iterator QList<QAbstractState*>::iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_iterator_operator_6(
                    self as *const crate::q_list_of_q_abstract_state::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_abstract_state::Iterator>>
        for &crate::q_list_of_q_abstract_state::Iterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QAbstractState*>::iterator::operator-(QList<QAbstractState*>::iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_abstract_state::Iterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_iterator_operator_7(
                    self as *const crate::q_list_of_q_abstract_state::Iterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_abstract_state::ConstIterator>>
        for crate::q_list_of_q_abstract_state::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractState*>::const_iterator::operator==(const QList<QAbstractState*>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_list_of_q_abstract_state::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_const_iterator_operator__2(
                    self as *const crate::q_list_of_q_abstract_state::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_abstract_state::ConstIterator>>
        for crate::q_list_of_q_abstract_state::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractState*>::const_iterator::operator<(const QList<QAbstractState*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_state::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_const_iterator_operator_1(
                    self as *const crate::q_list_of_q_abstract_state::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_abstract_state::ConstIterator>>
        for crate::q_list_of_q_abstract_state::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractState*>::const_iterator::operator<=(const QList<QAbstractState*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_state::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_const_iterator_operator__4(
                    self as *const crate::q_list_of_q_abstract_state::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_abstract_state::ConstIterator>>
        for crate::q_list_of_q_abstract_state::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractState*>::const_iterator::operator>(const QList<QAbstractState*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_state::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_const_iterator_operator_2(
                    self as *const crate::q_list_of_q_abstract_state::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_abstract_state::ConstIterator>>
        for crate::q_list_of_q_abstract_state::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractState*>::const_iterator::operator>=(const QList<QAbstractState*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_state::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_const_iterator_operator__5(
                    self as *const crate::q_list_of_q_abstract_state::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_abstract_state::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_abstract_state::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::const_iterator& QList<QAbstractState*>::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_abstract_state::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_const_iterator_operator__6(
                    self as *mut crate::q_list_of_q_abstract_state::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_abstract_state::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_abstract_state::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::const_iterator& QList<QAbstractState*>::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_abstract_state::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_const_iterator_operator__8(
                    self as *mut crate::q_list_of_q_abstract_state::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_abstract_state::ConstIterator
    {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::const_iterator& QList<QAbstractState*>::const_iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_const_iterator_operator__10(
                    self as *mut crate::q_list_of_q_abstract_state::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_abstract_state::ConstIterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::const_iterator& QList<QAbstractState*>::const_iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_const_iterator_operator__11(
                    self as *mut crate::q_list_of_q_abstract_state::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_abstract_state::ConstIterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::const_iterator QList<QAbstractState*>::const_iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_const_iterator_operator_3(
                    self as *const crate::q_list_of_q_abstract_state::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_abstract_state::ConstIterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::const_iterator QList<QAbstractState*>::const_iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_const_iterator_operator_4(
                    self as *const crate::q_list_of_q_abstract_state::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_abstract_state::ConstIterator>>
        for &crate::q_list_of_q_abstract_state::ConstIterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QAbstractState*>::const_iterator::operator-(QList<QAbstractState*>::const_iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_abstract_state::ConstIterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_const_iterator_operator_5(
                    self as *const crate::q_list_of_q_abstract_state::ConstIterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_abstract_state::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QAbstractState*>::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_iterator_diterator(
                self as *mut crate::q_list_of_q_abstract_state::Iterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_abstract_state::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QAbstractState*>::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_const_iterator_dconst_iterator(
                self as *mut crate::q_list_of_q_abstract_state::ConstIterator,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
///
/// C++ class: <span style='color: green;'>```QList<QAbstractState*>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlist.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores items in a list that provides fast index-based access and index-based insertions and removals.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Internally, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T if <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. Otherwise, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T* and the items are allocated on the heap.</p>
/// <p>The array representation allows very fast insertions and index-based access. The <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() operations are also very fast because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates memory at both ends of its internal array. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.</p>
/// <p>Note, however, that when the conditions specified above are not met, each append or insert of a new item requires allocating the new item on the heap, and this per item allocation will make <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> a better choice for use cases that do a lot of appending or inserting, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> can allocate memory for many items in a single heap allocation.</p>
/// <p>Note that the internal array only ever gets bigger over the life of the list. It never shrinks. The internal array is deallocated by the destructor and by the assignment operator, when one list is assigned to another.</p>
/// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerList;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span><span class="operator">&gt;</span> dateList;
///
/// </pre>
/// <p>Qt includes a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class that inherits <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>&gt; and adds a few convenience functions, such as <a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#filter">QStringList::filter</a>(). <a href="http://doc.qt.io/qt-5/qstring.html#split">QString::split</a>() creates QStringLists from strings.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> stores a list of items. The default constructor creates an empty list. To insert items into the list, you can use operator&lt;&lt;():</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"one"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"two"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"three"</span>;
///   <span class="comment">// list: ["one", "two", "three"]</span>
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(). In addition, it provides the following convenience functions: <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const lists, operator[]() returns a reference to the item and can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Bob"</span>)
/// &#32;     list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Robert"</span>;
///
/// </pre>
/// <p>Because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented as an array of pointers for types that are larger than a pointer or are not movable, this operation requires (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>). For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> list<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (list<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Jane"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Jane at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>A common requirement is to remove an item from a list and do something with it. For this, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(). Here's a loop that removes the items from a list one at a time and calls <code>delete</code> on them:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   <span class="keyword">while</span> (<span class="operator">!</span>list<span class="operator">.</span>isEmpty())
/// &#32;     <span class="keyword">delete</span> list<span class="operator">.</span>takeFirst();
///
/// </pre>
/// <p>Inserting and removing items at either end of the list is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> in most cases), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
/// <p>If you want to find all occurrences of a particular value in a list, use <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of a matching item if they find it; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> list<span class="operator">.</span>indexOf(<span class="string">"Jane"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Jane is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a list contains a particular value, use <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the list, use <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>(). If you want to replace all occurrences of a particular value with another, use <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlistiterator.html">QListIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablelistiterator.html">QMutableListIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented in such a way that direct index-based access is just as fast as using iterators.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <p>To make <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as efficient as possible, its member functions don't validate their input before using it. Except for <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), member functions always assume the list is <i>not</i> empty. Member functions that take index values as parameters always assume their index value parameters are in the valid range. This means <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> member functions can fail. If you define QT_NO_DEBUG when you compile, failures will not be detected. If you <i>don't</i> define QT_NO_DEBUG, failures will be detected using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT">Q_ASSERT</a>() or <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT_X">Q_ASSERT_X</a>() with an appropriate message.</p>
/// <p>To avoid failures when your list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling other member functions. If you must pass an index value that might not be in the valid range, check that it is less than the value returned by <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() but <i>not</i> less than 0.</p>
/// <a name="more-members"></a></div>
#[repr(C)]
pub struct QListOfQAbstractState {
    _unused: u8,
}
impl QListOfQAbstractState {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractState*>::append(const QList<QAbstractState*>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> list to this list.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQAbstractState>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_append1(
            self as *mut crate::QListOfQAbstractState,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQAbstractState>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractState*& QList<QAbstractState*>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn back(&mut self) -> ::cpp_utils::MutRef<*mut crate::QAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_back(
            self as *mut crate::QListOfQAbstractState,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::const_iterator QList<QAbstractState*>::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_cbegin(
            self as *const crate::QListOfQAbstractState,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::const_iterator QList<QAbstractState*>::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn cend(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_cend(
            self as *const crate::QListOfQAbstractState,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all items from the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractState*>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all items from the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_clear(
            self as *mut crate::QListOfQAbstractState,
        )
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::const_iterator QList<QAbstractState*>::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_constBegin(
            self as *const crate::QListOfQAbstractState,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::const_iterator QList<QAbstractState*>::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn const_end(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_constEnd(
            self as *const crate::QListOfQAbstractState,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Assigns <i>other</i> to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>& QList<QAbstractState*>::operator=(const QList<QAbstractState*>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this list and returns a reference to this list.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQAbstractState>>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_operator_(
            self as *mut crate::QListOfQAbstractState,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQAbstractState>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QAbstractState*>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_count1(
            self as *const crate::QListOfQAbstractState,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractState*>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_detach(
            self as *mut crate::QListOfQAbstractState,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractState*>::detachShared()```</span>.
    pub unsafe fn detach_shared(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_detachShared(
            self as *mut crate::QListOfQAbstractState,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractState*>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_empty(
            self as *const crate::QListOfQAbstractState,
        )
    }

    /// <p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::iterator QList<QAbstractState*>::erase(QList<QAbstractState*>::iterator pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn erase_1a(
        &mut self,
        pos: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_abstract_state::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::Iterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_erase(
                self as *mut crate::QListOfQAbstractState,
                ::cpp_utils::CastInto::<
                    ::cpp_utils::Ref<crate::q_list_of_q_abstract_state::Iterator>,
                >::cast_into(pos)
                .as_raw_ptr(),
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::iterator QList<QAbstractState*>::erase(QList<QAbstractState*>::iterator first, QList<QAbstractState*>::iterator last)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes all the items from <i>begin</i> up to (but not including) <i>end</i>. Returns an iterator to the same item that <i>end</i> referred to before the call.</p></div>
    pub unsafe fn erase_2a(
        &mut self,
        first: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_abstract_state::Iterator>>,
        last: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_abstract_state::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::Iterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_erase1(
                self as *mut crate::QListOfQAbstractState,
                ::cpp_utils::CastInto::<
                    ::cpp_utils::Ref<crate::q_list_of_q_abstract_state::Iterator>,
                >::cast_into(first)
                .as_raw_ptr(),
                ::cpp_utils::CastInto::<
                    ::cpp_utils::Ref<crate::q_list_of_q_abstract_state::Iterator>,
                >::cast_into(last)
                .as_raw_ptr(),
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractState*& QList<QAbstractState*>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn first(&mut self) -> ::cpp_utils::MutRef<*mut crate::QAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_first(
            self as *mut crate::QListOfQAbstractState,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> object with the data contained in <i>set</i>. The order of the elements in the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is undefined.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QList<QAbstractState*> QList<QAbstractState*>::fromSet(const QSet<QAbstractState*>& set)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#fromSet">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> object with the data contained in <i>set</i>. The order of the elements in the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is undefined.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html">QSet</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> set;
    ///   set <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">20</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">30</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">40</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">70</span>;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> list <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>fromSet(set);
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms-obsolete.html#qSort">qSort</a>(list);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#fromVector">fromVector</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#toSet">toSet</a>(), and <a href="http://doc.qt.io/qt-5/qset.html#toList">QSet::toList</a>().</p></div>
    pub unsafe fn from_set(
        set: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSetOfQAbstractState>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_fromSet(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSetOfQAbstractState>>::cast_into(set)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractState*& QList<QAbstractState*>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn front(&mut self) -> ::cpp_utils::MutRef<*mut crate::QAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_front(
            self as *mut crate::QListOfQAbstractState,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractState*& QList<QAbstractState*>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If this function is called on a list that is currently being shared, it will trigger a copy of all elements. Otherwise, this function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>. If you do not want to modify the list you should use <a href="http://doc.qt.io/qt-5/qlist.html#at">QList::at</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>().</p></div>
    pub unsafe fn index(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<*mut crate::QAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_operator__3(
            self as *mut crate::QListOfQAbstractState,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractState*>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_isDetached(
            self as *const crate::QListOfQAbstractState,
        )
    }

    /// <p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractState*>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_isEmpty(
            self as *const crate::QListOfQAbstractState,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractState*>::isSharedWith(const QList<QAbstractState*>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQAbstractState>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_isSharedWith(
            self as *const crate::QListOfQAbstractState,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQAbstractState>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        )
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractState*& QList<QAbstractState*>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn last(&mut self) -> ::cpp_utils::MutRef<*mut crate::QAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_last(
            self as *mut crate::QListOfQAbstractState,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QAbstractState*>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_length(
            self as *const crate::QListOfQAbstractState,
        )
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*> QList<QAbstractState*>::mid(int pos, int length = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        length: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_mid(
            self as *const crate::QListOfQAbstractState,
            pos,
            length,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*> QList<QAbstractState*>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_mid1(
            self as *const crate::QListOfQAbstractState,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractState*>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>move(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "C", "D", "E", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This is the same as insert(<i>to</i>, <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<i>from</i>)).This function assumes that both <i>from</i> and <i>to</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>from</i> and <i>to</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_move(
            self as *mut crate::QListOfQAbstractState,
            from,
            to,
        )
    }

    /// <p>Constructs an empty list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractState*>::QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty list.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QListOfQAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_QList();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractState*>::QList(const QList<QAbstractState*>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQAbstractState>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_QList1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQAbstractState>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractState*>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_pop_back(
            self as *mut crate::QListOfQAbstractState,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractState*>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_pop_front(
            self as *mut crate::QListOfQAbstractState,
        )
    }

    /// <p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractState*>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_removeAt(
            self as *mut crate::QListOfQAbstractState,
            i,
        )
    }

    /// <p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractState*>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_removeFirst(
            self as *mut crate::QListOfQAbstractState,
        )
    }

    /// <p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractState*>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_removeLast(
            self as *mut crate::QListOfQAbstractState,
        )
    }

    /// <p>Reserve space for <i>alloc</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractState*>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reserve space for <i>alloc</i> elements.</p>
    /// <p>If <i>alloc</i> is smaller than the current size of the list, nothing will happen.</p>
    /// <p>Use this function to avoid repetetive reallocation of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s internal data if you can predict how many elements will be appended. Note that the reservation applies only to the internal pointer array.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_reserve(
            self as *mut crate::QListOfQAbstractState,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractState*>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_setSharable(
            self as *mut crate::QListOfQAbstractState,
            sharable,
        )
    }

    /// <p>Returns the number of items in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QAbstractState*>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_size(
            self as *const crate::QListOfQAbstractState,
        )
    }

    /// <p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractState*>::swap(QList<QAbstractState*>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap_1a(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QListOfQAbstractState>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_swap(
            self as *mut crate::QListOfQAbstractState,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QListOfQAbstractState>>::cast_into(
                other,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractState*>::swap(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swap(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    pub unsafe fn swap_2a(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_swap1(
            self as *mut crate::QListOfQAbstractState,
            i,
            j,
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractState*>::swapItemsAt(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swapItemsAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swapItemsAt(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn swap_items_at(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_swapItemsAt(
            self as *mut crate::QListOfQAbstractState,
            i,
            j,
        )
    }

    /// <p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractState* QList<QAbstractState*>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_takeAt(
            self as *mut crate::QListOfQAbstractState,
            i,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractState* QList<QAbstractState*>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> ::cpp_utils::MutPtr<crate::QAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_takeFirst(
            self as *mut crate::QListOfQAbstractState,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractState* QList<QAbstractState*>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> ::cpp_utils::MutPtr<crate::QAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_takeLast(
            self as *mut crate::QListOfQAbstractState,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> object with the data contained in this <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. Since <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> doesn't allow duplicates, the resulting <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> might be smaller than the original list was.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*> QList<QAbstractState*>::toSet() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#toSet">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> object with the data contained in this <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. Since <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> doesn't allow duplicates, the resulting <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> might be smaller than the original list was.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Julia"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Mike"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Mike"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Julia"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Julia"</span>;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> set <span class="operator">=</span> list<span class="operator">.</span>toSet();
    ///   set<span class="operator">.</span>contains(<span class="string">"Julia"</span>);  <span class="comment">// returns true</span>
    ///   set<span class="operator">.</span>contains(<span class="string">"Mike"</span>);   <span class="comment">// returns true</span>
    ///   set<span class="operator">.</span>size();             <span class="comment">// returns 2</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#toVector">toVector</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#fromSet">fromSet</a>(), and <a href="http://doc.qt.io/qt-5/qset.html#fromList">QSet::fromList</a>().</p></div>
    pub unsafe fn to_set(&self) -> ::cpp_utils::CppBox<crate::QSetOfQAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_toSet(
            self as *const crate::QListOfQAbstractState,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value at index position <i>i</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractState* QList<QAbstractState*>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the list.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that the index is going to be within bounds, you can use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_value(
            self as *const crate::QListOfQAbstractState,
            i,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

pub mod q_list_of_q_abstract_animation {
    //! C++ type: <span style='color: green;'>```QList<QAbstractAnimation*>```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QAbstractAnimation*>::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::iterator& QList<QAbstractAnimation*>::iterator::operator=(const QList<QAbstractAnimation*>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::Iterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_abstract_animation::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_iterator_operator_8(
                    self as *mut crate::q_list_of_q_abstract_animation::Iterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::Iterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::iterator QList<QAbstractAnimation*>::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_iterator_operator__13(
                    self as *mut crate::q_list_of_q_abstract_animation::Iterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::iterator QList<QAbstractAnimation*>::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_iterator_operator__11(
                    self as *mut crate::q_list_of_q_abstract_animation::Iterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation*& QList<QAbstractAnimation*>::iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValueRef iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>. If you assign to it, the assignment will apply to the character in the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> of <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> from which you got the reference.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::MutRef<*mut crate::QAbstractAnimation> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_iterator_operator__1(
                    self as *const crate::q_list_of_q_abstract_animation::Iterator,
                    j,
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractAnimation*>::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>().</p></div>
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::Iterator>
        {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractAnimation*>::iterator::iterator(const QList<QAbstractAnimation*>::iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::Iterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_iterator_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::Iterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation** QList<QAbstractAnimation*>::iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's value.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::MutPtr<*mut crate::QAbstractAnimation> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_iterator_operator__(
                    self as *const crate::q_list_of_q_abstract_animation::Iterator,
                );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QAbstractAnimation*>::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::const_iterator& QList<QAbstractAnimation*>::const_iterator::operator=(const QList<QAbstractAnimation*>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::ConstIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_abstract_animation::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_const_iterator_operator_6(self as *mut crate::q_list_of_q_abstract_animation::ConstIterator, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::ConstIterator>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::const_iterator QList<QAbstractAnimation*>::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_const_iterator_operator__9(self as *mut crate::q_list_of_q_abstract_animation::ConstIterator, arg1);
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::const_iterator QList<QAbstractAnimation*>::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_const_iterator_operator__7(self as *mut crate::q_list_of_q_abstract_animation::ConstIterator, arg1);
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractAnimation*>::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>().</p></div>
        pub unsafe fn new_0a(
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_const_iterator_const_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractAnimation*>::const_iterator::const_iterator(const QList<QAbstractAnimation*>::iterator& o)```</span>.
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::Iterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_const_iterator_const_iterator2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractAnimation*>::const_iterator::const_iterator(const QList<QAbstractAnimation*>::const_iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::ConstIterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_const_iterator_const_iterator1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::ConstIterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_abstract_animation::Iterator {
        type Output = ::cpp_utils::MutRef<*mut crate::QAbstractAnimation>;
        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation*& QList<QAbstractAnimation*>::iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>().</p></div>
        fn indirection(self) -> ::cpp_utils::MutRef<*mut crate::QAbstractAnimation> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_iterator_operator_(
                    self as *const crate::q_list_of_q_abstract_animation::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::Iterator>>
        for crate::q_list_of_q_abstract_animation::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractAnimation*>::iterator::operator==(const QList<QAbstractAnimation*>::iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_iterator_operator__2(
                    self as *const crate::q_list_of_q_abstract_animation::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::Iterator>>
        for crate::q_list_of_q_abstract_animation::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractAnimation*>::iterator::operator<(const QList<QAbstractAnimation*>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_iterator_operator_1(
                    self as *const crate::q_list_of_q_abstract_animation::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::Iterator>>
        for crate::q_list_of_q_abstract_animation::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractAnimation*>::iterator::operator<=(const QList<QAbstractAnimation*>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_iterator_operator__4(
                    self as *const crate::q_list_of_q_abstract_animation::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::Iterator>>
        for crate::q_list_of_q_abstract_animation::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractAnimation*>::iterator::operator>(const QList<QAbstractAnimation*>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_iterator_operator_2(
                    self as *const crate::q_list_of_q_abstract_animation::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::Iterator>>
        for crate::q_list_of_q_abstract_animation::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractAnimation*>::iterator::operator>=(const QList<QAbstractAnimation*>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_iterator_operator__5(
                    self as *const crate::q_list_of_q_abstract_animation::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::std::cmp::PartialEq<
            ::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::ConstIterator>,
        > for crate::q_list_of_q_abstract_animation::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractAnimation*>::iterator::operator==(const QList<QAbstractAnimation*>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_iterator_operator__6(
                    self as *const crate::q_list_of_q_abstract_animation::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::ConstIterator>>
        for crate::q_list_of_q_abstract_animation::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractAnimation*>::iterator::operator<(const QList<QAbstractAnimation*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_iterator_operator_3(
                    self as *const crate::q_list_of_q_abstract_animation::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::ConstIterator>>
        for crate::q_list_of_q_abstract_animation::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractAnimation*>::iterator::operator<=(const QList<QAbstractAnimation*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_iterator_operator__8(
                    self as *const crate::q_list_of_q_abstract_animation::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::ConstIterator>>
        for crate::q_list_of_q_abstract_animation::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractAnimation*>::iterator::operator>(const QList<QAbstractAnimation*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_iterator_operator_4(
                    self as *const crate::q_list_of_q_abstract_animation::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::ConstIterator>>
        for crate::q_list_of_q_abstract_animation::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractAnimation*>::iterator::operator>=(const QList<QAbstractAnimation*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_iterator_operator__9(
                    self as *const crate::q_list_of_q_abstract_animation::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_abstract_animation::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_abstract_animation::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::iterator& QList<QAbstractAnimation*>::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_abstract_animation::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_iterator_operator__10(
                    self as *mut crate::q_list_of_q_abstract_animation::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_abstract_animation::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_abstract_animation::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::iterator& QList<QAbstractAnimation*>::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_abstract_animation::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_iterator_operator__12(
                    self as *mut crate::q_list_of_q_abstract_animation::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_abstract_animation::Iterator
    {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::iterator& QList<QAbstractAnimation*>::iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_iterator_operator__14(
                    self as *mut crate::q_list_of_q_abstract_animation::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_abstract_animation::Iterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::iterator& QList<QAbstractAnimation*>::iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_iterator_operator__15(
                    self as *mut crate::q_list_of_q_abstract_animation::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_abstract_animation::Iterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::iterator QList<QAbstractAnimation*>::iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_iterator_operator_5(
                    self as *const crate::q_list_of_q_abstract_animation::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_abstract_animation::Iterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::iterator QList<QAbstractAnimation*>::iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_iterator_operator_6(
                    self as *const crate::q_list_of_q_abstract_animation::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::Iterator>>
        for &crate::q_list_of_q_abstract_animation::Iterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QAbstractAnimation*>::iterator::operator-(QList<QAbstractAnimation*>::iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::Iterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_iterator_operator_7(
                    self as *const crate::q_list_of_q_abstract_animation::Iterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::std::cmp::PartialEq<
            ::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::ConstIterator>,
        > for crate::q_list_of_q_abstract_animation::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractAnimation*>::const_iterator::operator==(const QList<QAbstractAnimation*>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_const_iterator_operator__2(self as *const crate::q_list_of_q_abstract_animation::ConstIterator, o.as_raw_ptr())
            }
        }
    }

    impl
        ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::ConstIterator>>
        for crate::q_list_of_q_abstract_animation::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractAnimation*>::const_iterator::operator<(const QList<QAbstractAnimation*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_const_iterator_operator_1(
                    self as *const crate::q_list_of_q_abstract_animation::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::ConstIterator>>
        for crate::q_list_of_q_abstract_animation::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractAnimation*>::const_iterator::operator<=(const QList<QAbstractAnimation*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_const_iterator_operator__4(self as *const crate::q_list_of_q_abstract_animation::ConstIterator, other.as_raw_ptr())
            }
        }
    }

    impl
        ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::ConstIterator>>
        for crate::q_list_of_q_abstract_animation::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractAnimation*>::const_iterator::operator>(const QList<QAbstractAnimation*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_const_iterator_operator_2(
                    self as *const crate::q_list_of_q_abstract_animation::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::ConstIterator>>
        for crate::q_list_of_q_abstract_animation::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractAnimation*>::const_iterator::operator>=(const QList<QAbstractAnimation*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_const_iterator_operator__5(self as *const crate::q_list_of_q_abstract_animation::ConstIterator, other.as_raw_ptr())
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_abstract_animation::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_abstract_animation::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::const_iterator& QList<QAbstractAnimation*>::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_abstract_animation::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_const_iterator_operator__6(self as *mut crate::q_list_of_q_abstract_animation::ConstIterator)
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_abstract_animation::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_abstract_animation::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::const_iterator& QList<QAbstractAnimation*>::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_abstract_animation::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_const_iterator_operator__8(self as *mut crate::q_list_of_q_abstract_animation::ConstIterator)
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_abstract_animation::ConstIterator
    {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::const_iterator& QList<QAbstractAnimation*>::const_iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_const_iterator_operator__10(self as *mut crate::q_list_of_q_abstract_animation::ConstIterator, j)
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_abstract_animation::ConstIterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::const_iterator& QList<QAbstractAnimation*>::const_iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_const_iterator_operator__11(self as *mut crate::q_list_of_q_abstract_animation::ConstIterator, j)
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_abstract_animation::ConstIterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::const_iterator QList<QAbstractAnimation*>::const_iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_const_iterator_operator_3(
                    self as *const crate::q_list_of_q_abstract_animation::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_abstract_animation::ConstIterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::const_iterator QList<QAbstractAnimation*>::const_iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_const_iterator_operator_4(
                    self as *const crate::q_list_of_q_abstract_animation::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::ConstIterator>>
        for &crate::q_list_of_q_abstract_animation::ConstIterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QAbstractAnimation*>::const_iterator::operator-(QList<QAbstractAnimation*>::const_iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::ConstIterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_const_iterator_operator_5(
                    self as *const crate::q_list_of_q_abstract_animation::ConstIterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_abstract_animation::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QAbstractAnimation*>::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_iterator_diterator(
                self as *mut crate::q_list_of_q_abstract_animation::Iterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_abstract_animation::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QAbstractAnimation*>::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_const_iterator_dconst_iterator(self as *mut crate::q_list_of_q_abstract_animation::ConstIterator)
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
///
/// C++ class: <span style='color: green;'>```QList<QAbstractAnimation*>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlist.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores items in a list that provides fast index-based access and index-based insertions and removals.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Internally, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T if <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. Otherwise, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T* and the items are allocated on the heap.</p>
/// <p>The array representation allows very fast insertions and index-based access. The <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() operations are also very fast because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates memory at both ends of its internal array. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.</p>
/// <p>Note, however, that when the conditions specified above are not met, each append or insert of a new item requires allocating the new item on the heap, and this per item allocation will make <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> a better choice for use cases that do a lot of appending or inserting, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> can allocate memory for many items in a single heap allocation.</p>
/// <p>Note that the internal array only ever gets bigger over the life of the list. It never shrinks. The internal array is deallocated by the destructor and by the assignment operator, when one list is assigned to another.</p>
/// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerList;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span><span class="operator">&gt;</span> dateList;
///
/// </pre>
/// <p>Qt includes a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class that inherits <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>&gt; and adds a few convenience functions, such as <a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#filter">QStringList::filter</a>(). <a href="http://doc.qt.io/qt-5/qstring.html#split">QString::split</a>() creates QStringLists from strings.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> stores a list of items. The default constructor creates an empty list. To insert items into the list, you can use operator&lt;&lt;():</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"one"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"two"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"three"</span>;
///   <span class="comment">// list: ["one", "two", "three"]</span>
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(). In addition, it provides the following convenience functions: <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const lists, operator[]() returns a reference to the item and can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Bob"</span>)
/// &#32;     list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Robert"</span>;
///
/// </pre>
/// <p>Because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented as an array of pointers for types that are larger than a pointer or are not movable, this operation requires (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>). For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> list<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (list<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Jane"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Jane at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>A common requirement is to remove an item from a list and do something with it. For this, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(). Here's a loop that removes the items from a list one at a time and calls <code>delete</code> on them:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   <span class="keyword">while</span> (<span class="operator">!</span>list<span class="operator">.</span>isEmpty())
/// &#32;     <span class="keyword">delete</span> list<span class="operator">.</span>takeFirst();
///
/// </pre>
/// <p>Inserting and removing items at either end of the list is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> in most cases), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
/// <p>If you want to find all occurrences of a particular value in a list, use <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of a matching item if they find it; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> list<span class="operator">.</span>indexOf(<span class="string">"Jane"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Jane is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a list contains a particular value, use <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the list, use <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>(). If you want to replace all occurrences of a particular value with another, use <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlistiterator.html">QListIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablelistiterator.html">QMutableListIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented in such a way that direct index-based access is just as fast as using iterators.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <p>To make <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as efficient as possible, its member functions don't validate their input before using it. Except for <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), member functions always assume the list is <i>not</i> empty. Member functions that take index values as parameters always assume their index value parameters are in the valid range. This means <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> member functions can fail. If you define QT_NO_DEBUG when you compile, failures will not be detected. If you <i>don't</i> define QT_NO_DEBUG, failures will be detected using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT">Q_ASSERT</a>() or <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT_X">Q_ASSERT_X</a>() with an appropriate message.</p>
/// <p>To avoid failures when your list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling other member functions. If you must pass an index value that might not be in the valid range, check that it is less than the value returned by <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() but <i>not</i> less than 0.</p>
/// <a name="more-members"></a></div>
#[repr(C)]
pub struct QListOfQAbstractAnimation {
    _unused: u8,
}
impl QListOfQAbstractAnimation {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractAnimation*>::append(const QList<QAbstractAnimation*>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> list to this list.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQAbstractAnimation>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_append1(
            self as *mut crate::QListOfQAbstractAnimation,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQAbstractAnimation>>::cast_into(
                t,
            )
            .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation*& QList<QAbstractAnimation*>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn back(&mut self) -> ::cpp_utils::MutRef<*mut crate::QAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_back(
            self as *mut crate::QListOfQAbstractAnimation,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::const_iterator QList<QAbstractAnimation*>::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_cbegin(
            self as *const crate::QListOfQAbstractAnimation,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::const_iterator QList<QAbstractAnimation*>::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn cend(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_cend(
            self as *const crate::QListOfQAbstractAnimation,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all items from the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractAnimation*>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all items from the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_clear(
            self as *mut crate::QListOfQAbstractAnimation,
        )
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::const_iterator QList<QAbstractAnimation*>::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_constBegin(
            self as *const crate::QListOfQAbstractAnimation,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::const_iterator QList<QAbstractAnimation*>::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn const_end(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_constEnd(
            self as *const crate::QListOfQAbstractAnimation,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Assigns <i>other</i> to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>& QList<QAbstractAnimation*>::operator=(const QList<QAbstractAnimation*>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this list and returns a reference to this list.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQAbstractAnimation>>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_operator_(
            self as *mut crate::QListOfQAbstractAnimation,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQAbstractAnimation>>::cast_into(
                l,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QAbstractAnimation*>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_count1(
            self as *const crate::QListOfQAbstractAnimation,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractAnimation*>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_detach(
            self as *mut crate::QListOfQAbstractAnimation,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractAnimation*>::detachShared()```</span>.
    pub unsafe fn detach_shared(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_detachShared(
            self as *mut crate::QListOfQAbstractAnimation,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractAnimation*>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_empty(
            self as *const crate::QListOfQAbstractAnimation,
        )
    }

    /// <p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::iterator QList<QAbstractAnimation*>::erase(QList<QAbstractAnimation*>::iterator pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn erase_1a(
        &mut self,
        pos: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::Iterator>,
        >,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_erase(
            self as *mut crate::QListOfQAbstractAnimation,
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::Iterator>,
            >::cast_into(pos)
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::iterator QList<QAbstractAnimation*>::erase(QList<QAbstractAnimation*>::iterator first, QList<QAbstractAnimation*>::iterator last)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes all the items from <i>begin</i> up to (but not including) <i>end</i>. Returns an iterator to the same item that <i>end</i> referred to before the call.</p></div>
    pub unsafe fn erase_2a(
        &mut self,
        first: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::Iterator>,
        >,
        last: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::Iterator>,
        >,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_erase1(
            self as *mut crate::QListOfQAbstractAnimation,
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::Iterator>,
            >::cast_into(first)
            .as_raw_ptr(),
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::q_list_of_q_abstract_animation::Iterator>,
            >::cast_into(last)
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation*& QList<QAbstractAnimation*>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn first(&mut self) -> ::cpp_utils::MutRef<*mut crate::QAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_first(
            self as *mut crate::QListOfQAbstractAnimation,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation*& QList<QAbstractAnimation*>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn front(&mut self) -> ::cpp_utils::MutRef<*mut crate::QAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_front(
            self as *mut crate::QListOfQAbstractAnimation,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation*& QList<QAbstractAnimation*>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If this function is called on a list that is currently being shared, it will trigger a copy of all elements. Otherwise, this function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>. If you do not want to modify the list you should use <a href="http://doc.qt.io/qt-5/qlist.html#at">QList::at</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>().</p></div>
    pub unsafe fn index(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<*mut crate::QAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_operator__3(
            self as *mut crate::QListOfQAbstractAnimation,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractAnimation*>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_isDetached(
            self as *const crate::QListOfQAbstractAnimation,
        )
    }

    /// <p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractAnimation*>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_isEmpty(
            self as *const crate::QListOfQAbstractAnimation,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractAnimation*>::isSharedWith(const QList<QAbstractAnimation*>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQAbstractAnimation>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_isSharedWith(
            self as *const crate::QListOfQAbstractAnimation,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQAbstractAnimation>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        )
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation*& QList<QAbstractAnimation*>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn last(&mut self) -> ::cpp_utils::MutRef<*mut crate::QAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_last(
            self as *mut crate::QListOfQAbstractAnimation,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QAbstractAnimation*>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_length(
            self as *const crate::QListOfQAbstractAnimation,
        )
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*> QList<QAbstractAnimation*>::mid(int pos, int length = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        length: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_mid(
            self as *const crate::QListOfQAbstractAnimation,
            pos,
            length,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*> QList<QAbstractAnimation*>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_mid1(
            self as *const crate::QListOfQAbstractAnimation,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractAnimation*>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>move(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "C", "D", "E", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This is the same as insert(<i>to</i>, <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<i>from</i>)).This function assumes that both <i>from</i> and <i>to</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>from</i> and <i>to</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_move(
            self as *mut crate::QListOfQAbstractAnimation,
            from,
            to,
        )
    }

    /// <p>Constructs an empty list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractAnimation*>::QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty list.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QListOfQAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_QList();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractAnimation*>::QList(const QList<QAbstractAnimation*>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQAbstractAnimation>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_QList1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQAbstractAnimation>>::cast_into(
                l,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractAnimation*>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_pop_back(
            self as *mut crate::QListOfQAbstractAnimation,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractAnimation*>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_pop_front(
            self as *mut crate::QListOfQAbstractAnimation,
        )
    }

    /// <p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractAnimation*>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_removeAt(
            self as *mut crate::QListOfQAbstractAnimation,
            i,
        )
    }

    /// <p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractAnimation*>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_removeFirst(
            self as *mut crate::QListOfQAbstractAnimation,
        )
    }

    /// <p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractAnimation*>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_removeLast(
            self as *mut crate::QListOfQAbstractAnimation,
        )
    }

    /// <p>Reserve space for <i>alloc</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractAnimation*>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reserve space for <i>alloc</i> elements.</p>
    /// <p>If <i>alloc</i> is smaller than the current size of the list, nothing will happen.</p>
    /// <p>Use this function to avoid repetetive reallocation of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s internal data if you can predict how many elements will be appended. Note that the reservation applies only to the internal pointer array.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_reserve(
            self as *mut crate::QListOfQAbstractAnimation,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractAnimation*>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_setSharable(
            self as *mut crate::QListOfQAbstractAnimation,
            sharable,
        )
    }

    /// <p>Returns the number of items in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QAbstractAnimation*>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_size(
            self as *const crate::QListOfQAbstractAnimation,
        )
    }

    /// <p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractAnimation*>::swap(QList<QAbstractAnimation*>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap_1a(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QListOfQAbstractAnimation>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_swap(self as *mut crate::QListOfQAbstractAnimation, ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QListOfQAbstractAnimation>>::cast_into(other).as_mut_raw_ptr())
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractAnimation*>::swap(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swap(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    pub unsafe fn swap_2a(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_swap1(
            self as *mut crate::QListOfQAbstractAnimation,
            i,
            j,
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractAnimation*>::swapItemsAt(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swapItemsAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swapItemsAt(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn swap_items_at(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_swapItemsAt(
            self as *mut crate::QListOfQAbstractAnimation,
            i,
            j,
        )
    }

    /// <p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* QList<QAbstractAnimation*>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_takeAt(
            self as *mut crate::QListOfQAbstractAnimation,
            i,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* QList<QAbstractAnimation*>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> ::cpp_utils::MutPtr<crate::QAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_takeFirst(
            self as *mut crate::QListOfQAbstractAnimation,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* QList<QAbstractAnimation*>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> ::cpp_utils::MutPtr<crate::QAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_takeLast(
            self as *mut crate::QListOfQAbstractAnimation,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns the value at index position <i>i</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* QList<QAbstractAnimation*>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the list.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that the index is going to be within bounds, you can use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_value(
            self as *const crate::QListOfQAbstractAnimation,
            i,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

pub mod q_list_of_day_of_week {
    //! C++ type: <span style='color: green;'>```QList<Qt::DayOfWeek>```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<Qt::DayOfWeek>::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::iterator& QList<Qt::DayOfWeek>::iterator::operator=(const QList<Qt::DayOfWeek>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_day_of_week::Iterator>>,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_day_of_week::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_iterator_operator_8(self as *mut crate::q_list_of_day_of_week::Iterator, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_day_of_week::Iterator>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::iterator QList<Qt::DayOfWeek>::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_day_of_week::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_iterator_operator__13(
                self as *mut crate::q_list_of_day_of_week::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::iterator QList<Qt::DayOfWeek>::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_day_of_week::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_iterator_operator__11(
                self as *mut crate::q_list_of_day_of_week::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```Qt::DayOfWeek& QList<Qt::DayOfWeek>::iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValueRef iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>. If you assign to it, the assignment will apply to the character in the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> of <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> from which you got the reference.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::MutRef<crate::DayOfWeek> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_iterator_operator__1(
                self as *const crate::q_list_of_day_of_week::Iterator,
                j,
            );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<Qt::DayOfWeek>::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>().</p></div>
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_list_of_day_of_week::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<Qt::DayOfWeek>::iterator::iterator(const QList<Qt::DayOfWeek>::iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_day_of_week::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_day_of_week::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_iterator_iterator1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_day_of_week::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```Qt::DayOfWeek* QList<Qt::DayOfWeek>::iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's value.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::MutPtr<crate::DayOfWeek> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_iterator_operator__(
                self as *const crate::q_list_of_day_of_week::Iterator,
            );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<Qt::DayOfWeek>::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::const_iterator& QList<Qt::DayOfWeek>::const_iterator::operator=(const QList<Qt::DayOfWeek>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_day_of_week::ConstIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_day_of_week::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_const_iterator_operator_6(
                    self as *mut crate::q_list_of_day_of_week::ConstIterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_day_of_week::ConstIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::const_iterator QList<Qt::DayOfWeek>::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_day_of_week::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_const_iterator_operator__9(
                    self as *mut crate::q_list_of_day_of_week::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::const_iterator QList<Qt::DayOfWeek>::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_day_of_week::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_const_iterator_operator__7(
                    self as *mut crate::q_list_of_day_of_week::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const Qt::DayOfWeek& QList<Qt::DayOfWeek>::const_iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValue const_iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::Ref<crate::DayOfWeek> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_const_iterator_operator__1(
                    self as *const crate::q_list_of_day_of_week::ConstIterator,
                    j,
                );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<Qt::DayOfWeek>::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>().</p></div>
        pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::q_list_of_day_of_week::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_const_iterator_const_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<Qt::DayOfWeek>::const_iterator::const_iterator(const QList<Qt::DayOfWeek>::iterator& o)```</span>.
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_day_of_week::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_day_of_week::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_const_iterator_const_iterator2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_day_of_week::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<Qt::DayOfWeek>::const_iterator::const_iterator(const QList<Qt::DayOfWeek>::const_iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_day_of_week::ConstIterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_day_of_week::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_const_iterator_const_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_day_of_week::ConstIterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current result.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const Qt::DayOfWeek* QList<Qt::DayOfWeek>::const_iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current result.</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::Ptr<crate::DayOfWeek> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_const_iterator_operator__(
                    self as *const crate::q_list_of_day_of_week::ConstIterator,
                );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_day_of_week::Iterator {
        type Output = ::cpp_utils::MutRef<crate::DayOfWeek>;
        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```Qt::DayOfWeek& QList<Qt::DayOfWeek>::iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>().</p></div>
        fn indirection(self) -> ::cpp_utils::MutRef<crate::DayOfWeek> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_iterator_operator_(
                    self as *const crate::q_list_of_day_of_week::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_day_of_week::Iterator>>
        for crate::q_list_of_day_of_week::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<Qt::DayOfWeek>::iterator::operator==(const QList<Qt::DayOfWeek>::iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_day_of_week::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_iterator_operator__2(
                    self as *const crate::q_list_of_day_of_week::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_day_of_week::Iterator>>
        for crate::q_list_of_day_of_week::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<Qt::DayOfWeek>::iterator::operator<(const QList<Qt::DayOfWeek>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_day_of_week::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_iterator_operator_1(
                    self as *const crate::q_list_of_day_of_week::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_day_of_week::Iterator>>
        for crate::q_list_of_day_of_week::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<Qt::DayOfWeek>::iterator::operator<=(const QList<Qt::DayOfWeek>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_day_of_week::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_iterator_operator__4(
                    self as *const crate::q_list_of_day_of_week::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_day_of_week::Iterator>>
        for crate::q_list_of_day_of_week::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<Qt::DayOfWeek>::iterator::operator>(const QList<Qt::DayOfWeek>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_day_of_week::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_iterator_operator_2(
                    self as *const crate::q_list_of_day_of_week::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_day_of_week::Iterator>>
        for crate::q_list_of_day_of_week::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<Qt::DayOfWeek>::iterator::operator>=(const QList<Qt::DayOfWeek>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_day_of_week::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_iterator_operator__5(
                    self as *const crate::q_list_of_day_of_week::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_day_of_week::ConstIterator>>
        for crate::q_list_of_day_of_week::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<Qt::DayOfWeek>::iterator::operator==(const QList<Qt::DayOfWeek>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_day_of_week::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_iterator_operator__6(
                    self as *const crate::q_list_of_day_of_week::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_day_of_week::ConstIterator>>
        for crate::q_list_of_day_of_week::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<Qt::DayOfWeek>::iterator::operator<(const QList<Qt::DayOfWeek>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_day_of_week::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_iterator_operator_3(
                    self as *const crate::q_list_of_day_of_week::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_day_of_week::ConstIterator>>
        for crate::q_list_of_day_of_week::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<Qt::DayOfWeek>::iterator::operator<=(const QList<Qt::DayOfWeek>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_day_of_week::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_iterator_operator__8(
                    self as *const crate::q_list_of_day_of_week::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_day_of_week::ConstIterator>>
        for crate::q_list_of_day_of_week::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<Qt::DayOfWeek>::iterator::operator>(const QList<Qt::DayOfWeek>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_day_of_week::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_iterator_operator_4(
                    self as *const crate::q_list_of_day_of_week::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_day_of_week::ConstIterator>>
        for crate::q_list_of_day_of_week::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<Qt::DayOfWeek>::iterator::operator>=(const QList<Qt::DayOfWeek>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_day_of_week::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_iterator_operator__9(
                    self as *const crate::q_list_of_day_of_week::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_day_of_week::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_day_of_week::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::iterator& QList<Qt::DayOfWeek>::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_day_of_week::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_iterator_operator__10(
                    self as *mut crate::q_list_of_day_of_week::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_day_of_week::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_day_of_week::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::iterator& QList<Qt::DayOfWeek>::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_day_of_week::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_iterator_operator__12(
                    self as *mut crate::q_list_of_day_of_week::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong> for crate::q_list_of_day_of_week::Iterator {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::iterator& QList<Qt::DayOfWeek>::iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_iterator_operator__14(
                    self as *mut crate::q_list_of_day_of_week::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong> for crate::q_list_of_day_of_week::Iterator {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::iterator& QList<Qt::DayOfWeek>::iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_iterator_operator__15(
                    self as *mut crate::q_list_of_day_of_week::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_list_of_day_of_week::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_day_of_week::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::iterator QList<Qt::DayOfWeek>::iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_day_of_week::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_iterator_operator_5(
                    self as *const crate::q_list_of_day_of_week::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_list_of_day_of_week::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_day_of_week::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::iterator QList<Qt::DayOfWeek>::iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_day_of_week::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_iterator_operator_6(
                    self as *const crate::q_list_of_day_of_week::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_day_of_week::Iterator>>
        for &crate::q_list_of_day_of_week::Iterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<Qt::DayOfWeek>::iterator::operator-(QList<Qt::DayOfWeek>::iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_day_of_week::Iterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_iterator_operator_7(
                    self as *const crate::q_list_of_day_of_week::Iterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_day_of_week::ConstIterator {
        type Output = ::cpp_utils::Ref<crate::DayOfWeek>;
        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const Qt::DayOfWeek& QList<Qt::DayOfWeek>::const_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        fn indirection(self) -> ::cpp_utils::Ref<crate::DayOfWeek> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_const_iterator_operator_(
                    self as *const crate::q_list_of_day_of_week::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_day_of_week::ConstIterator>>
        for crate::q_list_of_day_of_week::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<Qt::DayOfWeek>::const_iterator::operator==(const QList<Qt::DayOfWeek>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_day_of_week::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_const_iterator_operator__2(
                    self as *const crate::q_list_of_day_of_week::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_day_of_week::ConstIterator>>
        for crate::q_list_of_day_of_week::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<Qt::DayOfWeek>::const_iterator::operator<(const QList<Qt::DayOfWeek>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_day_of_week::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_const_iterator_operator_1(
                    self as *const crate::q_list_of_day_of_week::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_day_of_week::ConstIterator>>
        for crate::q_list_of_day_of_week::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<Qt::DayOfWeek>::const_iterator::operator<=(const QList<Qt::DayOfWeek>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_day_of_week::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_const_iterator_operator__4(
                    self as *const crate::q_list_of_day_of_week::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_day_of_week::ConstIterator>>
        for crate::q_list_of_day_of_week::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<Qt::DayOfWeek>::const_iterator::operator>(const QList<Qt::DayOfWeek>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_day_of_week::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_const_iterator_operator_2(
                    self as *const crate::q_list_of_day_of_week::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_day_of_week::ConstIterator>>
        for crate::q_list_of_day_of_week::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<Qt::DayOfWeek>::const_iterator::operator>=(const QList<Qt::DayOfWeek>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_day_of_week::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_const_iterator_operator__5(
                    self as *const crate::q_list_of_day_of_week::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_day_of_week::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_day_of_week::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::const_iterator& QList<Qt::DayOfWeek>::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_day_of_week::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_const_iterator_operator__6(
                    self as *mut crate::q_list_of_day_of_week::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_day_of_week::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_day_of_week::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::const_iterator& QList<Qt::DayOfWeek>::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_day_of_week::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_const_iterator_operator__8(
                    self as *mut crate::q_list_of_day_of_week::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_day_of_week::ConstIterator
    {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::const_iterator& QList<Qt::DayOfWeek>::const_iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_const_iterator_operator__10(
                    self as *mut crate::q_list_of_day_of_week::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_day_of_week::ConstIterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::const_iterator& QList<Qt::DayOfWeek>::const_iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_const_iterator_operator__11(
                    self as *mut crate::q_list_of_day_of_week::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_list_of_day_of_week::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_day_of_week::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::const_iterator QList<Qt::DayOfWeek>::const_iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_day_of_week::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_const_iterator_operator_3(
                    self as *const crate::q_list_of_day_of_week::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_list_of_day_of_week::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_day_of_week::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::const_iterator QList<Qt::DayOfWeek>::const_iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_day_of_week::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_const_iterator_operator_4(
                    self as *const crate::q_list_of_day_of_week::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_day_of_week::ConstIterator>>
        for &crate::q_list_of_day_of_week::ConstIterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<Qt::DayOfWeek>::const_iterator::operator-(QList<Qt::DayOfWeek>::const_iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_day_of_week::ConstIterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_const_iterator_operator_5(
                    self as *const crate::q_list_of_day_of_week::ConstIterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_day_of_week::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<Qt::DayOfWeek>::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_iterator_diterator(
                self as *mut crate::q_list_of_day_of_week::Iterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_day_of_week::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<Qt::DayOfWeek>::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_const_iterator_dconst_iterator(
                self as *mut crate::q_list_of_day_of_week::ConstIterator,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
///
/// C++ class: <span style='color: green;'>```QList<Qt::DayOfWeek>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlist.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores items in a list that provides fast index-based access and index-based insertions and removals.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Internally, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T if <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. Otherwise, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T* and the items are allocated on the heap.</p>
/// <p>The array representation allows very fast insertions and index-based access. The <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() operations are also very fast because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates memory at both ends of its internal array. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.</p>
/// <p>Note, however, that when the conditions specified above are not met, each append or insert of a new item requires allocating the new item on the heap, and this per item allocation will make <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> a better choice for use cases that do a lot of appending or inserting, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> can allocate memory for many items in a single heap allocation.</p>
/// <p>Note that the internal array only ever gets bigger over the life of the list. It never shrinks. The internal array is deallocated by the destructor and by the assignment operator, when one list is assigned to another.</p>
/// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerList;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span><span class="operator">&gt;</span> dateList;
///
/// </pre>
/// <p>Qt includes a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class that inherits <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>&gt; and adds a few convenience functions, such as <a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#filter">QStringList::filter</a>(). <a href="http://doc.qt.io/qt-5/qstring.html#split">QString::split</a>() creates QStringLists from strings.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> stores a list of items. The default constructor creates an empty list. To insert items into the list, you can use operator&lt;&lt;():</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"one"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"two"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"three"</span>;
///   <span class="comment">// list: ["one", "two", "three"]</span>
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(). In addition, it provides the following convenience functions: <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const lists, operator[]() returns a reference to the item and can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Bob"</span>)
/// &#32;     list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Robert"</span>;
///
/// </pre>
/// <p>Because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented as an array of pointers for types that are larger than a pointer or are not movable, this operation requires (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>). For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> list<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (list<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Jane"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Jane at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>A common requirement is to remove an item from a list and do something with it. For this, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(). Here's a loop that removes the items from a list one at a time and calls <code>delete</code> on them:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   <span class="keyword">while</span> (<span class="operator">!</span>list<span class="operator">.</span>isEmpty())
/// &#32;     <span class="keyword">delete</span> list<span class="operator">.</span>takeFirst();
///
/// </pre>
/// <p>Inserting and removing items at either end of the list is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> in most cases), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
/// <p>If you want to find all occurrences of a particular value in a list, use <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of a matching item if they find it; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> list<span class="operator">.</span>indexOf(<span class="string">"Jane"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Jane is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a list contains a particular value, use <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the list, use <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>(). If you want to replace all occurrences of a particular value with another, use <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlistiterator.html">QListIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablelistiterator.html">QMutableListIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented in such a way that direct index-based access is just as fast as using iterators.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <p>To make <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as efficient as possible, its member functions don't validate their input before using it. Except for <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), member functions always assume the list is <i>not</i> empty. Member functions that take index values as parameters always assume their index value parameters are in the valid range. This means <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> member functions can fail. If you define QT_NO_DEBUG when you compile, failures will not be detected. If you <i>don't</i> define QT_NO_DEBUG, failures will be detected using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT">Q_ASSERT</a>() or <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT_X">Q_ASSERT_X</a>() with an appropriate message.</p>
/// <p>To avoid failures when your list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling other member functions. If you must pass an index value that might not be in the valid range, check that it is less than the value returned by <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() but <i>not</i> less than 0.</p>
/// <a name="more-members"></a></div>
#[repr(C)]
pub struct QListOfDayOfWeek {
    _unused: u8,
}
impl QListOfDayOfWeek {
    /// <p>Inserts <i>value</i> at the end of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<Qt::DayOfWeek>::append(const Qt::DayOfWeek& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["one", "two", "three"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn append_day_of_week(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::DayOfWeek>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_append(
            self as *mut crate::QListOfDayOfWeek,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::DayOfWeek>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<Qt::DayOfWeek>::append(const QList<Qt::DayOfWeek>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> list to this list.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_list_of_day_of_week(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfDayOfWeek>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_append1(
            self as *mut crate::QListOfDayOfWeek,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfDayOfWeek>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const Qt::DayOfWeek& QList<Qt::DayOfWeek>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>This function is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::DayOfWeek> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_at(
            self as *const crate::QListOfDayOfWeek,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```Qt::DayOfWeek& QList<Qt::DayOfWeek>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::MutRef<crate::DayOfWeek> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_back(
            self as *mut crate::QListOfDayOfWeek,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const Qt::DayOfWeek& QList<Qt::DayOfWeek>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<crate::DayOfWeek> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_back1(
            self as *const crate::QListOfDayOfWeek,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::const_iterator QList<Qt::DayOfWeek>::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_day_of_week::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_cbegin(
            self as *const crate::QListOfDayOfWeek,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::const_iterator QList<Qt::DayOfWeek>::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn cend(&self) -> ::cpp_utils::CppBox<crate::q_list_of_day_of_week::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_cend(
            self as *const crate::QListOfDayOfWeek,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all items from the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<Qt::DayOfWeek>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all items from the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_clear(self as *mut crate::QListOfDayOfWeek)
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::const_iterator QList<Qt::DayOfWeek>::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_day_of_week::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_constBegin(
            self as *const crate::QListOfDayOfWeek,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::const_iterator QList<Qt::DayOfWeek>::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn const_end(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_day_of_week::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_constEnd(
            self as *const crate::QListOfDayOfWeek,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const Qt::DayOfWeek& QList<Qt::DayOfWeek>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>().</p></div>
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<crate::DayOfWeek> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_constFirst(
            self as *const crate::QListOfDayOfWeek,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const Qt::DayOfWeek& QList<Qt::DayOfWeek>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>().</p></div>
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<crate::DayOfWeek> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_constLast(
            self as *const crate::QListOfDayOfWeek,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<Qt::DayOfWeek>::contains(const Qt::DayOfWeek& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn contains(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::DayOfWeek>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_contains(
            self as *const crate::QListOfDayOfWeek,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::DayOfWeek>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>& QList<Qt::DayOfWeek>::operator=(const QList<Qt::DayOfWeek>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this list and returns a reference to this list.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfDayOfWeek>>,
    ) -> ::cpp_utils::MutRef<crate::QListOfDayOfWeek> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_operator_(
            self as *mut crate::QListOfDayOfWeek,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfDayOfWeek>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of occurrences of <i>value</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<Qt::DayOfWeek>::count(const Qt::DayOfWeek& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of occurrences of <i>value</i> in the list.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::DayOfWeek>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_count(
            self as *const crate::QListOfDayOfWeek,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::DayOfWeek>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<Qt::DayOfWeek>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_count1(
            self as *const crate::QListOfDayOfWeek,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<Qt::DayOfWeek>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_detach(
            self as *mut crate::QListOfDayOfWeek,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<Qt::DayOfWeek>::detachShared()```</span>.
    pub unsafe fn detach_shared(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_detachShared(
            self as *mut crate::QListOfDayOfWeek,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<Qt::DayOfWeek>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_empty(
            self as *const crate::QListOfDayOfWeek,
        )
    }

    /// <p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<Qt::DayOfWeek>::endsWith(const Qt::DayOfWeek& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn ends_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::DayOfWeek>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_endsWith(
            self as *const crate::QListOfDayOfWeek,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::DayOfWeek>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::iterator QList<Qt::DayOfWeek>::erase(QList<Qt::DayOfWeek>::iterator pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn erase_1a(
        &mut self,
        pos: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_day_of_week::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_day_of_week::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_erase(self as *mut crate::QListOfDayOfWeek, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_day_of_week::Iterator>>::cast_into(pos).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::iterator QList<Qt::DayOfWeek>::erase(QList<Qt::DayOfWeek>::iterator first, QList<Qt::DayOfWeek>::iterator last)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes all the items from <i>begin</i> up to (but not including) <i>end</i>. Returns an iterator to the same item that <i>end</i> referred to before the call.</p></div>
    pub unsafe fn erase_2a(
        &mut self,
        first: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_day_of_week::Iterator>>,
        last: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_day_of_week::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_day_of_week::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_erase1(self as *mut crate::QListOfDayOfWeek, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_day_of_week::Iterator>>::cast_into(first).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_day_of_week::Iterator>>::cast_into(last).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```Qt::DayOfWeek& QList<Qt::DayOfWeek>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<crate::DayOfWeek> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_first(
            self as *mut crate::QListOfDayOfWeek,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const Qt::DayOfWeek& QList<Qt::DayOfWeek>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::DayOfWeek> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_first1(
            self as *const crate::QListOfDayOfWeek,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```Qt::DayOfWeek& QList<Qt::DayOfWeek>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::MutRef<crate::DayOfWeek> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_front(
            self as *mut crate::QListOfDayOfWeek,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const Qt::DayOfWeek& QList<Qt::DayOfWeek>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<crate::DayOfWeek> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_front1(
            self as *const crate::QListOfDayOfWeek,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const Qt::DayOfWeek& QList<Qt::DayOfWeek>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>(). This function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p></div>
    pub unsafe fn index(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::DayOfWeek> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_operator__2(
            self as *const crate::QListOfDayOfWeek,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```Qt::DayOfWeek& QList<Qt::DayOfWeek>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If this function is called on a list that is currently being shared, it will trigger a copy of all elements. Otherwise, this function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>. If you do not want to modify the list you should use <a href="http://doc.qt.io/qt-5/qlist.html#at">QList::at</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::DayOfWeek> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_operator__3(
            self as *mut crate::QListOfDayOfWeek,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<Qt::DayOfWeek>::indexOf(const Qt::DayOfWeek& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::DayOfWeek>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_indexOf(
            self as *const crate::QListOfDayOfWeek,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::DayOfWeek>>::cast_into(t).as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<Qt::DayOfWeek>::indexOf(const Qt::DayOfWeek& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::DayOfWeek>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_indexOf1(
            self as *const crate::QListOfDayOfWeek,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::DayOfWeek>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<Qt::DayOfWeek>::insert(int i, const Qt::DayOfWeek& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   list<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// list: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn insert_int_day_of_week(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::DayOfWeek>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_insert(
            self as *mut crate::QListOfDayOfWeek,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::DayOfWeek>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::iterator QList<Qt::DayOfWeek>::insert(QList<Qt::DayOfWeek>::iterator before, const Qt::DayOfWeek& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>value</i> in front of the item pointed to by the iterator <i>before</i>. Returns an iterator pointing at the inserted item. Note that the iterator passed to the function will be invalid after the call; the returned iterator should be used instead.</p></div>
    pub unsafe fn insert_iterator_day_of_week(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_day_of_week::Iterator>>,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::DayOfWeek>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_day_of_week::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_insert1(self as *mut crate::QListOfDayOfWeek, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_day_of_week::Iterator>>::cast_into(before).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::DayOfWeek>>::cast_into(t).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<Qt::DayOfWeek>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_isDetached(
            self as *const crate::QListOfDayOfWeek,
        )
    }

    /// <p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<Qt::DayOfWeek>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_isEmpty(
            self as *const crate::QListOfDayOfWeek,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<Qt::DayOfWeek>::isSharedWith(const QList<Qt::DayOfWeek>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfDayOfWeek>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_isSharedWith(
            self as *const crate::QListOfDayOfWeek,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfDayOfWeek>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```Qt::DayOfWeek& QList<Qt::DayOfWeek>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<crate::DayOfWeek> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_last(
            self as *mut crate::QListOfDayOfWeek,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const Qt::DayOfWeek& QList<Qt::DayOfWeek>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::DayOfWeek> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_last1(
            self as *const crate::QListOfDayOfWeek,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<Qt::DayOfWeek>::lastIndexOf(const Qt::DayOfWeek& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::DayOfWeek>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_lastIndexOf(
            self as *const crate::QListOfDayOfWeek,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::DayOfWeek>>::cast_into(t).as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<Qt::DayOfWeek>::lastIndexOf(const Qt::DayOfWeek& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::DayOfWeek>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_lastIndexOf1(
            self as *const crate::QListOfDayOfWeek,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::DayOfWeek>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<Qt::DayOfWeek>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_length(
            self as *const crate::QListOfDayOfWeek,
        )
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek> QList<Qt::DayOfWeek>::mid(int pos, int length = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        length: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfDayOfWeek> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_mid(
            self as *const crate::QListOfDayOfWeek,
            pos,
            length,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek> QList<Qt::DayOfWeek>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfDayOfWeek> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_mid1(
            self as *const crate::QListOfDayOfWeek,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<Qt::DayOfWeek>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>move(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "C", "D", "E", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This is the same as insert(<i>to</i>, <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<i>from</i>)).This function assumes that both <i>from</i> and <i>to</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>from</i> and <i>to</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_move(
            self as *mut crate::QListOfDayOfWeek,
            from,
            to,
        )
    }

    /// <p>Constructs an empty list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<Qt::DayOfWeek>::QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty list.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QListOfDayOfWeek> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_QList();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<Qt::DayOfWeek>::QList(const QList<Qt::DayOfWeek>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfDayOfWeek>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfDayOfWeek> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_QList1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfDayOfWeek>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<Qt::DayOfWeek>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_pop_back(
            self as *mut crate::QListOfDayOfWeek,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<Qt::DayOfWeek>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_pop_front(
            self as *mut crate::QListOfDayOfWeek,
        )
    }

    /// <p>Inserts <i>value</i> at the beginning of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<Qt::DayOfWeek>::prepend(const Qt::DayOfWeek& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(0, <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::DayOfWeek>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_prepend(
            self as *mut crate::QListOfDayOfWeek,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::DayOfWeek>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<Qt::DayOfWeek>::push_back(const Qt::DayOfWeek& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::DayOfWeek>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_push_back(
            self as *mut crate::QListOfDayOfWeek,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::DayOfWeek>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<Qt::DayOfWeek>::push_front(const Qt::DayOfWeek& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::DayOfWeek>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_push_front(
            self as *mut crate::QListOfDayOfWeek,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::DayOfWeek>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<Qt::DayOfWeek>::removeAll(const Qt::DayOfWeek& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeAll(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_all(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::DayOfWeek>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_removeAll(
            self as *mut crate::QListOfDayOfWeek,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::DayOfWeek>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<Qt::DayOfWeek>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_removeAt(
            self as *mut crate::QListOfDayOfWeek,
            i,
        )
    }

    /// <p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<Qt::DayOfWeek>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_removeFirst(
            self as *mut crate::QListOfDayOfWeek,
        )
    }

    /// <p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<Qt::DayOfWeek>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_removeLast(
            self as *mut crate::QListOfDayOfWeek,
        )
    }

    /// <p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<Qt::DayOfWeek>::removeOne(const Qt::DayOfWeek& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeOne(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", ,"sun", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_one(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::DayOfWeek>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_removeOne(
            self as *mut crate::QListOfDayOfWeek,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::DayOfWeek>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<Qt::DayOfWeek>::replace(int i, const Qt::DayOfWeek& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::DayOfWeek>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_replace(
            self as *mut crate::QListOfDayOfWeek,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::DayOfWeek>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Reserve space for <i>alloc</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<Qt::DayOfWeek>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reserve space for <i>alloc</i> elements.</p>
    /// <p>If <i>alloc</i> is smaller than the current size of the list, nothing will happen.</p>
    /// <p>Use this function to avoid repetetive reallocation of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s internal data if you can predict how many elements will be appended. Note that the reservation applies only to the internal pointer array.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_reserve(
            self as *mut crate::QListOfDayOfWeek,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<Qt::DayOfWeek>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_setSharable(
            self as *mut crate::QListOfDayOfWeek,
            sharable,
        )
    }

    /// <p>Returns the number of items in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<Qt::DayOfWeek>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_size(
            self as *const crate::QListOfDayOfWeek,
        )
    }

    /// <p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<Qt::DayOfWeek>::startsWith(const Qt::DayOfWeek& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn starts_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::DayOfWeek>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_startsWith(
            self as *const crate::QListOfDayOfWeek,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::DayOfWeek>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<Qt::DayOfWeek>::swap(QList<Qt::DayOfWeek>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap_1a(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QListOfDayOfWeek>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_swap(
            self as *mut crate::QListOfDayOfWeek,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QListOfDayOfWeek>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<Qt::DayOfWeek>::swap(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swap(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    pub unsafe fn swap_2a(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_swap1(
            self as *mut crate::QListOfDayOfWeek,
            i,
            j,
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<Qt::DayOfWeek>::swapItemsAt(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swapItemsAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swapItemsAt(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn swap_items_at(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_swapItemsAt(
            self as *mut crate::QListOfDayOfWeek,
            i,
            j,
        )
    }

    /// <p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```Qt::DayOfWeek QList<Qt::DayOfWeek>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn take_at(&mut self, i: ::std::os::raw::c_int) -> crate::DayOfWeek {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_takeAt(
            self as *mut crate::QListOfDayOfWeek,
            i,
        )
    }

    /// <p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```Qt::DayOfWeek QList<Qt::DayOfWeek>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> crate::DayOfWeek {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_takeFirst(
            self as *mut crate::QListOfDayOfWeek,
        )
    }

    /// <p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```Qt::DayOfWeek QList<Qt::DayOfWeek>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> crate::DayOfWeek {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_takeLast(
            self as *mut crate::QListOfDayOfWeek,
        )
    }

    /// <p>Returns the value at index position <i>i</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```Qt::DayOfWeek QList<Qt::DayOfWeek>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the list.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that the index is going to be within bounds, you can use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(&self, i: ::std::os::raw::c_int) -> crate::DayOfWeek {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_value(
            self as *const crate::QListOfDayOfWeek,
            i,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```Qt::DayOfWeek QList<Qt::DayOfWeek>::value(int i, const Qt::DayOfWeek& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::DayOfWeek>>,
    ) -> crate::DayOfWeek {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_value1(
            self as *const crate::QListOfDayOfWeek,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::DayOfWeek>>::cast_into(default_value)
                .as_raw_ptr(),
        )
    }
}

pub mod q_list_of_q_locale {
    //! C++ type: <span style='color: green;'>```QList<QLocale>```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QLocale>::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::iterator& QList<QLocale>::iterator::operator=(const QList<QLocale>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_locale::Iterator>>,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_locale::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_iterator_operator_8(self as *mut crate::q_list_of_q_locale::Iterator, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_locale::Iterator>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::iterator QList<QLocale>::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_locale::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_iterator_operator__13(
                self as *mut crate::q_list_of_q_locale::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::iterator QList<QLocale>::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_locale::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_iterator_operator__11(
                self as *mut crate::q_list_of_q_locale::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QLocale& QList<QLocale>::iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValueRef iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>. If you assign to it, the assignment will apply to the character in the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> of <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> from which you got the reference.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::MutRef<crate::QLocale> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_iterator_operator__1(
                self as *const crate::q_list_of_q_locale::Iterator,
                j,
            );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QLocale>::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>().</p></div>
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_list_of_q_locale::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QLocale>::iterator::iterator(const QList<QLocale>::iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_locale::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_locale::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_iterator_iterator1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_locale::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QLocale* QList<QLocale>::iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's value.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::MutPtr<crate::QLocale> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_iterator_operator__(
                self as *const crate::q_list_of_q_locale::Iterator,
            );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QLocale>::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::const_iterator& QList<QLocale>::const_iterator::operator=(const QList<QLocale>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_locale::ConstIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_locale::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_const_iterator_operator_6(
                    self as *mut crate::q_list_of_q_locale::ConstIterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_locale::ConstIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::const_iterator QList<QLocale>::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_locale::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_const_iterator_operator__9(
                self as *mut crate::q_list_of_q_locale::ConstIterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::const_iterator QList<QLocale>::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_locale::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_const_iterator_operator__7(
                self as *mut crate::q_list_of_q_locale::ConstIterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QLocale& QList<QLocale>::const_iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValue const_iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::Ref<crate::QLocale> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_const_iterator_operator__1(
                self as *const crate::q_list_of_q_locale::ConstIterator,
                j,
            );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QLocale>::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>().</p></div>
        pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::q_list_of_q_locale::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_const_iterator_const_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QLocale>::const_iterator::const_iterator(const QList<QLocale>::iterator& o)```</span>.
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_locale::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_locale::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_const_iterator_const_iterator2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_locale::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QLocale>::const_iterator::const_iterator(const QList<QLocale>::const_iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_locale::ConstIterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_locale::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_const_iterator_const_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_locale::ConstIterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current result.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QLocale* QList<QLocale>::const_iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current result.</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::Ptr<crate::QLocale> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_const_iterator_operator__(
                self as *const crate::q_list_of_q_locale::ConstIterator,
            );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_locale::Iterator {
        type Output = ::cpp_utils::MutRef<crate::QLocale>;
        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QLocale& QList<QLocale>::iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>().</p></div>
        fn indirection(self) -> ::cpp_utils::MutRef<crate::QLocale> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_iterator_operator_(
                    self as *const crate::q_list_of_q_locale::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_locale::Iterator>>
        for crate::q_list_of_q_locale::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale>::iterator::operator==(const QList<QLocale>::iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_locale::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_iterator_operator__2(
                    self as *const crate::q_list_of_q_locale::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_locale::Iterator>>
        for crate::q_list_of_q_locale::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale>::iterator::operator<(const QList<QLocale>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_locale::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_iterator_operator_1(
                    self as *const crate::q_list_of_q_locale::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_locale::Iterator>>
        for crate::q_list_of_q_locale::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale>::iterator::operator<=(const QList<QLocale>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_locale::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_iterator_operator__4(
                    self as *const crate::q_list_of_q_locale::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_locale::Iterator>>
        for crate::q_list_of_q_locale::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale>::iterator::operator>(const QList<QLocale>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_locale::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_iterator_operator_2(
                    self as *const crate::q_list_of_q_locale::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_locale::Iterator>>
        for crate::q_list_of_q_locale::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale>::iterator::operator>=(const QList<QLocale>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_locale::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_iterator_operator__5(
                    self as *const crate::q_list_of_q_locale::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_locale::ConstIterator>>
        for crate::q_list_of_q_locale::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale>::iterator::operator==(const QList<QLocale>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_locale::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_iterator_operator__6(
                    self as *const crate::q_list_of_q_locale::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_locale::ConstIterator>>
        for crate::q_list_of_q_locale::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale>::iterator::operator<(const QList<QLocale>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_locale::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_iterator_operator_3(
                    self as *const crate::q_list_of_q_locale::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_locale::ConstIterator>>
        for crate::q_list_of_q_locale::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale>::iterator::operator<=(const QList<QLocale>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_locale::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_iterator_operator__8(
                    self as *const crate::q_list_of_q_locale::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_locale::ConstIterator>>
        for crate::q_list_of_q_locale::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale>::iterator::operator>(const QList<QLocale>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_locale::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_iterator_operator_4(
                    self as *const crate::q_list_of_q_locale::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_locale::ConstIterator>>
        for crate::q_list_of_q_locale::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale>::iterator::operator>=(const QList<QLocale>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_locale::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_iterator_operator__9(
                    self as *const crate::q_list_of_q_locale::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_locale::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_locale::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::iterator& QList<QLocale>::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_locale::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_iterator_operator__10(
                    self as *mut crate::q_list_of_q_locale::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_locale::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_locale::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::iterator& QList<QLocale>::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_locale::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_iterator_operator__12(
                    self as *mut crate::q_list_of_q_locale::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong> for crate::q_list_of_q_locale::Iterator {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::iterator& QList<QLocale>::iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_iterator_operator__14(
                    self as *mut crate::q_list_of_q_locale::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong> for crate::q_list_of_q_locale::Iterator {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::iterator& QList<QLocale>::iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_iterator_operator__15(
                    self as *mut crate::q_list_of_q_locale::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_list_of_q_locale::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_locale::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::iterator QList<QLocale>::iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_locale::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_iterator_operator_5(
                    self as *const crate::q_list_of_q_locale::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_list_of_q_locale::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_locale::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::iterator QList<QLocale>::iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_locale::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_iterator_operator_6(
                    self as *const crate::q_list_of_q_locale::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_locale::Iterator>>
        for &crate::q_list_of_q_locale::Iterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QLocale>::iterator::operator-(QList<QLocale>::iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_locale::Iterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_iterator_operator_7(
                    self as *const crate::q_list_of_q_locale::Iterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_locale::ConstIterator {
        type Output = ::cpp_utils::Ref<crate::QLocale>;
        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QLocale& QList<QLocale>::const_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        fn indirection(self) -> ::cpp_utils::Ref<crate::QLocale> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_const_iterator_operator_(
                    self as *const crate::q_list_of_q_locale::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_locale::ConstIterator>>
        for crate::q_list_of_q_locale::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale>::const_iterator::operator==(const QList<QLocale>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_locale::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_const_iterator_operator__2(
                    self as *const crate::q_list_of_q_locale::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_locale::ConstIterator>>
        for crate::q_list_of_q_locale::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale>::const_iterator::operator<(const QList<QLocale>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_locale::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_const_iterator_operator_1(
                    self as *const crate::q_list_of_q_locale::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_locale::ConstIterator>>
        for crate::q_list_of_q_locale::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale>::const_iterator::operator<=(const QList<QLocale>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_locale::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_const_iterator_operator__4(
                    self as *const crate::q_list_of_q_locale::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_locale::ConstIterator>>
        for crate::q_list_of_q_locale::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale>::const_iterator::operator>(const QList<QLocale>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_locale::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_const_iterator_operator_2(
                    self as *const crate::q_list_of_q_locale::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_locale::ConstIterator>>
        for crate::q_list_of_q_locale::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale>::const_iterator::operator>=(const QList<QLocale>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_locale::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_const_iterator_operator__5(
                    self as *const crate::q_list_of_q_locale::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_locale::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_locale::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::const_iterator& QList<QLocale>::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_locale::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_const_iterator_operator__6(
                    self as *mut crate::q_list_of_q_locale::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_locale::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_locale::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::const_iterator& QList<QLocale>::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_locale::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_const_iterator_operator__8(
                    self as *mut crate::q_list_of_q_locale::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_locale::ConstIterator
    {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::const_iterator& QList<QLocale>::const_iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_const_iterator_operator__10(
                    self as *mut crate::q_list_of_q_locale::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_locale::ConstIterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::const_iterator& QList<QLocale>::const_iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_const_iterator_operator__11(
                    self as *mut crate::q_list_of_q_locale::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_list_of_q_locale::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_locale::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::const_iterator QList<QLocale>::const_iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_locale::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_const_iterator_operator_3(
                    self as *const crate::q_list_of_q_locale::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_list_of_q_locale::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_locale::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::const_iterator QList<QLocale>::const_iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_locale::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_const_iterator_operator_4(
                    self as *const crate::q_list_of_q_locale::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_locale::ConstIterator>>
        for &crate::q_list_of_q_locale::ConstIterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QLocale>::const_iterator::operator-(QList<QLocale>::const_iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_locale::ConstIterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_const_iterator_operator_5(
                    self as *const crate::q_list_of_q_locale::ConstIterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_locale::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QLocale>::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_iterator_diterator(
                self as *mut crate::q_list_of_q_locale::Iterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_locale::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QLocale>::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_const_iterator_dconst_iterator(
                self as *mut crate::q_list_of_q_locale::ConstIterator,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
///
/// C++ class: <span style='color: green;'>```QList<QLocale>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlist.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores items in a list that provides fast index-based access and index-based insertions and removals.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Internally, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T if <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. Otherwise, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T* and the items are allocated on the heap.</p>
/// <p>The array representation allows very fast insertions and index-based access. The <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() operations are also very fast because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates memory at both ends of its internal array. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.</p>
/// <p>Note, however, that when the conditions specified above are not met, each append or insert of a new item requires allocating the new item on the heap, and this per item allocation will make <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> a better choice for use cases that do a lot of appending or inserting, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> can allocate memory for many items in a single heap allocation.</p>
/// <p>Note that the internal array only ever gets bigger over the life of the list. It never shrinks. The internal array is deallocated by the destructor and by the assignment operator, when one list is assigned to another.</p>
/// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerList;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span><span class="operator">&gt;</span> dateList;
///
/// </pre>
/// <p>Qt includes a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class that inherits <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>&gt; and adds a few convenience functions, such as <a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#filter">QStringList::filter</a>(). <a href="http://doc.qt.io/qt-5/qstring.html#split">QString::split</a>() creates QStringLists from strings.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> stores a list of items. The default constructor creates an empty list. To insert items into the list, you can use operator&lt;&lt;():</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"one"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"two"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"three"</span>;
///   <span class="comment">// list: ["one", "two", "three"]</span>
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(). In addition, it provides the following convenience functions: <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const lists, operator[]() returns a reference to the item and can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Bob"</span>)
/// &#32;     list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Robert"</span>;
///
/// </pre>
/// <p>Because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented as an array of pointers for types that are larger than a pointer or are not movable, this operation requires (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>). For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> list<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (list<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Jane"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Jane at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>A common requirement is to remove an item from a list and do something with it. For this, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(). Here's a loop that removes the items from a list one at a time and calls <code>delete</code> on them:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   <span class="keyword">while</span> (<span class="operator">!</span>list<span class="operator">.</span>isEmpty())
/// &#32;     <span class="keyword">delete</span> list<span class="operator">.</span>takeFirst();
///
/// </pre>
/// <p>Inserting and removing items at either end of the list is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> in most cases), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
/// <p>If you want to find all occurrences of a particular value in a list, use <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of a matching item if they find it; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> list<span class="operator">.</span>indexOf(<span class="string">"Jane"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Jane is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a list contains a particular value, use <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the list, use <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>(). If you want to replace all occurrences of a particular value with another, use <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlistiterator.html">QListIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablelistiterator.html">QMutableListIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented in such a way that direct index-based access is just as fast as using iterators.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <p>To make <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as efficient as possible, its member functions don't validate their input before using it. Except for <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), member functions always assume the list is <i>not</i> empty. Member functions that take index values as parameters always assume their index value parameters are in the valid range. This means <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> member functions can fail. If you define QT_NO_DEBUG when you compile, failures will not be detected. If you <i>don't</i> define QT_NO_DEBUG, failures will be detected using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT">Q_ASSERT</a>() or <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT_X">Q_ASSERT_X</a>() with an appropriate message.</p>
/// <p>To avoid failures when your list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling other member functions. If you must pass an index value that might not be in the valid range, check that it is less than the value returned by <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() but <i>not</i> less than 0.</p>
/// <a name="more-members"></a></div>
#[repr(C)]
pub struct QListOfQLocale {
    _unused: u8,
}
impl QListOfQLocale {
    /// <p>Inserts <i>value</i> at the end of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale>::append(const QLocale& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["one", "two", "three"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn append_q_locale(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_append(
            self as *mut crate::QListOfQLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale>::append(const QList<QLocale>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> list to this list.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_list_of_q_locale(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQLocale>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_append1(
            self as *mut crate::QListOfQLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQLocale>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QLocale& QList<QLocale>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>This function is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::QLocale> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_at(self as *const crate::QListOfQLocale, i);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale& QList<QLocale>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::MutRef<crate::QLocale> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_back(self as *mut crate::QListOfQLocale);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QLocale& QList<QLocale>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<crate::QLocale> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_back1(self as *const crate::QListOfQLocale);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::const_iterator QList<QLocale>::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(&self) -> ::cpp_utils::CppBox<crate::q_list_of_q_locale::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_cbegin(
            self as *const crate::QListOfQLocale,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::const_iterator QList<QLocale>::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn cend(&self) -> ::cpp_utils::CppBox<crate::q_list_of_q_locale::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_cend(self as *const crate::QListOfQLocale);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all items from the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all items from the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_clear(self as *mut crate::QListOfQLocale)
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::const_iterator QList<QLocale>::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_locale::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_constBegin(
            self as *const crate::QListOfQLocale,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::const_iterator QList<QLocale>::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn const_end(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_locale::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_constEnd(
            self as *const crate::QListOfQLocale,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QLocale& QList<QLocale>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>().</p></div>
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<crate::QLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_constFirst(
            self as *const crate::QListOfQLocale,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QLocale& QList<QLocale>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>().</p></div>
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<crate::QLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_constLast(
            self as *const crate::QListOfQLocale,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale>::contains(const QLocale& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn contains(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_contains(
            self as *const crate::QListOfQLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale>& QList<QLocale>::operator=(const QList<QLocale>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this list and returns a reference to this list.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQLocale>>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_operator_(
            self as *mut crate::QListOfQLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQLocale>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of occurrences of <i>value</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QLocale>::count(const QLocale& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of occurrences of <i>value</i> in the list.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_count(
            self as *const crate::QListOfQLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QLocale>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_count1(self as *const crate::QListOfQLocale)
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_detach(self as *mut crate::QListOfQLocale)
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale>::detachShared()```</span>.
    pub unsafe fn detach_shared(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_detachShared(self as *mut crate::QListOfQLocale)
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_empty(self as *const crate::QListOfQLocale)
    }

    /// <p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale>::endsWith(const QLocale& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn ends_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_endsWith(
            self as *const crate::QListOfQLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::iterator QList<QLocale>::erase(QList<QLocale>::iterator pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn erase_1a(
        &mut self,
        pos: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_locale::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_locale::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_erase(self as *mut crate::QListOfQLocale, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_locale::Iterator>>::cast_into(pos).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::iterator QList<QLocale>::erase(QList<QLocale>::iterator first, QList<QLocale>::iterator last)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes all the items from <i>begin</i> up to (but not including) <i>end</i>. Returns an iterator to the same item that <i>end</i> referred to before the call.</p></div>
    pub unsafe fn erase_2a(
        &mut self,
        first: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_locale::Iterator>>,
        last: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_locale::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_locale::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_erase1(self as *mut crate::QListOfQLocale, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_locale::Iterator>>::cast_into(first).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_locale::Iterator>>::cast_into(last).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale& QList<QLocale>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<crate::QLocale> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_first(self as *mut crate::QListOfQLocale);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QLocale& QList<QLocale>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::QLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_first1(
            self as *const crate::QListOfQLocale,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale& QList<QLocale>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::MutRef<crate::QLocale> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_front(self as *mut crate::QListOfQLocale);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QLocale& QList<QLocale>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<crate::QLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_front1(
            self as *const crate::QListOfQLocale,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QLocale& QList<QLocale>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>(). This function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p></div>
    pub unsafe fn index(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::QLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_operator__2(
            self as *const crate::QListOfQLocale,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale& QList<QLocale>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If this function is called on a list that is currently being shared, it will trigger a copy of all elements. Otherwise, this function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>. If you do not want to modify the list you should use <a href="http://doc.qt.io/qt-5/qlist.html#at">QList::at</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_operator__3(
            self as *mut crate::QListOfQLocale,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QLocale>::indexOf(const QLocale& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_indexOf(
            self as *const crate::QListOfQLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(t).as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QLocale>::indexOf(const QLocale& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_indexOf1(
            self as *const crate::QListOfQLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale>::insert(int i, const QLocale& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   list<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// list: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn insert_int_q_locale(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_insert(
            self as *mut crate::QListOfQLocale,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::iterator QList<QLocale>::insert(QList<QLocale>::iterator before, const QLocale& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>value</i> in front of the item pointed to by the iterator <i>before</i>. Returns an iterator pointing at the inserted item. Note that the iterator passed to the function will be invalid after the call; the returned iterator should be used instead.</p></div>
    pub unsafe fn insert_iterator_q_locale(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_locale::Iterator>>,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_locale::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_insert1(self as *mut crate::QListOfQLocale, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_locale::Iterator>>::cast_into(before).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(t).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_isDetached(self as *const crate::QListOfQLocale)
    }

    /// <p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_isEmpty(self as *const crate::QListOfQLocale)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale>::isSharedWith(const QList<QLocale>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQLocale>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_isSharedWith(
            self as *const crate::QListOfQLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQLocale>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale& QList<QLocale>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<crate::QLocale> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_last(self as *mut crate::QListOfQLocale);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QLocale& QList<QLocale>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::QLocale> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_last1(self as *const crate::QListOfQLocale);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QLocale>::lastIndexOf(const QLocale& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_lastIndexOf(
            self as *const crate::QListOfQLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(t).as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QLocale>::lastIndexOf(const QLocale& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_lastIndexOf1(
            self as *const crate::QListOfQLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QLocale>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_length(self as *const crate::QListOfQLocale)
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale> QList<QLocale>::mid(int pos, int length = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        length: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_mid(
            self as *const crate::QListOfQLocale,
            pos,
            length,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale> QList<QLocale>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_mid1(
            self as *const crate::QListOfQLocale,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>move(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "C", "D", "E", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This is the same as insert(<i>to</i>, <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<i>from</i>)).This function assumes that both <i>from</i> and <i>to</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>from</i> and <i>to</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_move(
            self as *mut crate::QListOfQLocale,
            from,
            to,
        )
    }

    /// <p>Constructs an empty list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QLocale>::QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty list.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QListOfQLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_QList();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QLocale>::QList(const QList<QLocale>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQLocale>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_QList1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQLocale>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_pop_back(self as *mut crate::QListOfQLocale)
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_pop_front(self as *mut crate::QListOfQLocale)
    }

    /// <p>Inserts <i>value</i> at the beginning of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale>::prepend(const QLocale& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(0, <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_prepend(
            self as *mut crate::QListOfQLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale>::push_back(const QLocale& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_push_back(
            self as *mut crate::QListOfQLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale>::push_front(const QLocale& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_push_front(
            self as *mut crate::QListOfQLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QLocale>::removeAll(const QLocale& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeAll(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_all(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_removeAll(
            self as *mut crate::QListOfQLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_removeAt(self as *mut crate::QListOfQLocale, i)
    }

    /// <p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_removeFirst(self as *mut crate::QListOfQLocale)
    }

    /// <p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_removeLast(self as *mut crate::QListOfQLocale)
    }

    /// <p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale>::removeOne(const QLocale& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeOne(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", ,"sun", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_one(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_removeOne(
            self as *mut crate::QListOfQLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale>::replace(int i, const QLocale& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_replace(
            self as *mut crate::QListOfQLocale,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Reserve space for <i>alloc</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reserve space for <i>alloc</i> elements.</p>
    /// <p>If <i>alloc</i> is smaller than the current size of the list, nothing will happen.</p>
    /// <p>Use this function to avoid repetetive reallocation of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s internal data if you can predict how many elements will be appended. Note that the reservation applies only to the internal pointer array.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_reserve(
            self as *mut crate::QListOfQLocale,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_setSharable(
            self as *mut crate::QListOfQLocale,
            sharable,
        )
    }

    /// <p>Returns the number of items in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QLocale>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_size(self as *const crate::QListOfQLocale)
    }

    /// <p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale>::startsWith(const QLocale& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn starts_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_startsWith(
            self as *const crate::QListOfQLocale,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale>::swap(QList<QLocale>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap_1a(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QListOfQLocale>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_swap(
            self as *mut crate::QListOfQLocale,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QListOfQLocale>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale>::swap(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swap(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    pub unsafe fn swap_2a(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_swap1(self as *mut crate::QListOfQLocale, i, j)
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale>::swapItemsAt(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swapItemsAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swapItemsAt(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn swap_items_at(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_swapItemsAt(
            self as *mut crate::QListOfQLocale,
            i,
            j,
        )
    }

    /// <p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale QList<QLocale>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_takeAt(
            self as *mut crate::QListOfQLocale,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale QList<QLocale>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> ::cpp_utils::CppBox<crate::QLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_takeFirst(
            self as *mut crate::QListOfQLocale,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale QList<QLocale>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> ::cpp_utils::CppBox<crate::QLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_takeLast(
            self as *mut crate::QListOfQLocale,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value at index position <i>i</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale QList<QLocale>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the list.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that the index is going to be within bounds, you can use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_value(
            self as *const crate::QListOfQLocale,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale QList<QLocale>::value(int i, const QLocale& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    ) -> ::cpp_utils::CppBox<crate::QLocale> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_value1(
            self as *const crate::QListOfQLocale,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_list_of_country {
    //! C++ type: <span style='color: green;'>```QList<QLocale::Country>```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QLocale::Country>::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::iterator& QList<QLocale::Country>::iterator::operator=(const QList<QLocale::Country>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_country::Iterator>>,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_country::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_iterator_operator_8(self as *mut crate::q_list_of_country::Iterator, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_country::Iterator>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::iterator QList<QLocale::Country>::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_country::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_iterator_operator__13(
                    self as *mut crate::q_list_of_country::Iterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::iterator QList<QLocale::Country>::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_country::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_iterator_operator__11(
                    self as *mut crate::q_list_of_country::Iterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QLocale::Country& QList<QLocale::Country>::iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValueRef iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>. If you assign to it, the assignment will apply to the character in the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> of <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> from which you got the reference.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::MutRef<crate::q_locale::Country> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_iterator_operator__1(
                    self as *const crate::q_list_of_country::Iterator,
                    j,
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QLocale::Country>::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>().</p></div>
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_list_of_country::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QLocale::Country>::iterator::iterator(const QList<QLocale::Country>::iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_country::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_country::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_iterator_iterator1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_country::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QLocale::Country* QList<QLocale::Country>::iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's value.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::MutPtr<crate::q_locale::Country> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_iterator_operator__(
                    self as *const crate::q_list_of_country::Iterator,
                );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QLocale::Country>::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::const_iterator& QList<QLocale::Country>::const_iterator::operator=(const QList<QLocale::Country>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_country::ConstIterator>>,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_country::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_const_iterator_operator_6(
                    self as *mut crate::q_list_of_country::ConstIterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_country::ConstIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::const_iterator QList<QLocale::Country>::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_country::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_const_iterator_operator__9(
                    self as *mut crate::q_list_of_country::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::const_iterator QList<QLocale::Country>::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_country::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_const_iterator_operator__7(
                    self as *mut crate::q_list_of_country::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QLocale::Country& QList<QLocale::Country>::const_iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValue const_iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::Ref<crate::q_locale::Country> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_const_iterator_operator__1(
                    self as *const crate::q_list_of_country::ConstIterator,
                    j,
                );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QLocale::Country>::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>().</p></div>
        pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::q_list_of_country::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_const_iterator_const_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QLocale::Country>::const_iterator::const_iterator(const QList<QLocale::Country>::iterator& o)```</span>.
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_country::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_country::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_const_iterator_const_iterator2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_country::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QLocale::Country>::const_iterator::const_iterator(const QList<QLocale::Country>::const_iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_country::ConstIterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_country::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_const_iterator_const_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_country::ConstIterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current result.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QLocale::Country* QList<QLocale::Country>::const_iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current result.</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::Ptr<crate::q_locale::Country> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_const_iterator_operator__(
                    self as *const crate::q_list_of_country::ConstIterator,
                );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_country::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_locale::Country>;
        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QLocale::Country& QList<QLocale::Country>::iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>().</p></div>
        fn indirection(self) -> ::cpp_utils::MutRef<crate::q_locale::Country> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_iterator_operator_(
                    self as *const crate::q_list_of_country::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_country::Iterator>>
        for crate::q_list_of_country::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale::Country>::iterator::operator==(const QList<QLocale::Country>::iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_country::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_iterator_operator__2(
                    self as *const crate::q_list_of_country::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_country::Iterator>>
        for crate::q_list_of_country::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale::Country>::iterator::operator<(const QList<QLocale::Country>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_country::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_iterator_operator_1(
                    self as *const crate::q_list_of_country::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_country::Iterator>>
        for crate::q_list_of_country::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale::Country>::iterator::operator<=(const QList<QLocale::Country>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_country::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_iterator_operator__4(
                    self as *const crate::q_list_of_country::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_country::Iterator>>
        for crate::q_list_of_country::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale::Country>::iterator::operator>(const QList<QLocale::Country>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_country::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_iterator_operator_2(
                    self as *const crate::q_list_of_country::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_country::Iterator>>
        for crate::q_list_of_country::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale::Country>::iterator::operator>=(const QList<QLocale::Country>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_country::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_iterator_operator__5(
                    self as *const crate::q_list_of_country::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_country::ConstIterator>>
        for crate::q_list_of_country::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale::Country>::iterator::operator==(const QList<QLocale::Country>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_country::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_iterator_operator__6(
                    self as *const crate::q_list_of_country::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_country::ConstIterator>>
        for crate::q_list_of_country::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale::Country>::iterator::operator<(const QList<QLocale::Country>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_country::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_iterator_operator_3(
                    self as *const crate::q_list_of_country::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_country::ConstIterator>>
        for crate::q_list_of_country::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale::Country>::iterator::operator<=(const QList<QLocale::Country>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_country::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_iterator_operator__8(
                    self as *const crate::q_list_of_country::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_country::ConstIterator>>
        for crate::q_list_of_country::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale::Country>::iterator::operator>(const QList<QLocale::Country>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_country::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_iterator_operator_4(
                    self as *const crate::q_list_of_country::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_country::ConstIterator>>
        for crate::q_list_of_country::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale::Country>::iterator::operator>=(const QList<QLocale::Country>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_country::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_iterator_operator__9(
                    self as *const crate::q_list_of_country::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_country::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_country::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::iterator& QList<QLocale::Country>::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_country::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_iterator_operator__10(
                    self as *mut crate::q_list_of_country::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_country::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_country::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::iterator& QList<QLocale::Country>::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_country::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_iterator_operator__12(
                    self as *mut crate::q_list_of_country::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong> for crate::q_list_of_country::Iterator {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::iterator& QList<QLocale::Country>::iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_iterator_operator__14(
                    self as *mut crate::q_list_of_country::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong> for crate::q_list_of_country::Iterator {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::iterator& QList<QLocale::Country>::iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_iterator_operator__15(
                    self as *mut crate::q_list_of_country::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_list_of_country::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_country::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::iterator QList<QLocale::Country>::iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_country::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_iterator_operator_5(
                    self as *const crate::q_list_of_country::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_list_of_country::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_country::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::iterator QList<QLocale::Country>::iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_country::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_iterator_operator_6(
                    self as *const crate::q_list_of_country::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_country::Iterator>>
        for &crate::q_list_of_country::Iterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QLocale::Country>::iterator::operator-(QList<QLocale::Country>::iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_country::Iterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_iterator_operator_7(
                    self as *const crate::q_list_of_country::Iterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_country::ConstIterator {
        type Output = ::cpp_utils::Ref<crate::q_locale::Country>;
        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QLocale::Country& QList<QLocale::Country>::const_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        fn indirection(self) -> ::cpp_utils::Ref<crate::q_locale::Country> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_const_iterator_operator_(
                    self as *const crate::q_list_of_country::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_country::ConstIterator>>
        for crate::q_list_of_country::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale::Country>::const_iterator::operator==(const QList<QLocale::Country>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_country::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_const_iterator_operator__2(
                    self as *const crate::q_list_of_country::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_country::ConstIterator>>
        for crate::q_list_of_country::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale::Country>::const_iterator::operator<(const QList<QLocale::Country>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_country::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_const_iterator_operator_1(
                    self as *const crate::q_list_of_country::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_country::ConstIterator>>
        for crate::q_list_of_country::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale::Country>::const_iterator::operator<=(const QList<QLocale::Country>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_country::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_const_iterator_operator__4(
                    self as *const crate::q_list_of_country::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_country::ConstIterator>>
        for crate::q_list_of_country::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale::Country>::const_iterator::operator>(const QList<QLocale::Country>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_country::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_const_iterator_operator_2(
                    self as *const crate::q_list_of_country::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_country::ConstIterator>>
        for crate::q_list_of_country::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale::Country>::const_iterator::operator>=(const QList<QLocale::Country>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_country::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_const_iterator_operator__5(
                    self as *const crate::q_list_of_country::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_country::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_country::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::const_iterator& QList<QLocale::Country>::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_country::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_const_iterator_operator__6(
                    self as *mut crate::q_list_of_country::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_country::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_country::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::const_iterator& QList<QLocale::Country>::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_country::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_const_iterator_operator__8(
                    self as *mut crate::q_list_of_country::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong> for crate::q_list_of_country::ConstIterator {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::const_iterator& QList<QLocale::Country>::const_iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_const_iterator_operator__10(
                    self as *mut crate::q_list_of_country::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong> for crate::q_list_of_country::ConstIterator {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::const_iterator& QList<QLocale::Country>::const_iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_const_iterator_operator__11(
                    self as *mut crate::q_list_of_country::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_list_of_country::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_country::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::const_iterator QList<QLocale::Country>::const_iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_country::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_const_iterator_operator_3(
                    self as *const crate::q_list_of_country::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_list_of_country::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_country::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::const_iterator QList<QLocale::Country>::const_iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_country::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_const_iterator_operator_4(
                    self as *const crate::q_list_of_country::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_country::ConstIterator>>
        for &crate::q_list_of_country::ConstIterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QLocale::Country>::const_iterator::operator-(QList<QLocale::Country>::const_iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_country::ConstIterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_const_iterator_operator_5(
                    self as *const crate::q_list_of_country::ConstIterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_country::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QLocale::Country>::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_iterator_diterator(
                self as *mut crate::q_list_of_country::Iterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_country::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QLocale::Country>::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_const_iterator_dconst_iterator(
                self as *mut crate::q_list_of_country::ConstIterator,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
///
/// C++ class: <span style='color: green;'>```QList<QLocale::Country>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlist.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores items in a list that provides fast index-based access and index-based insertions and removals.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Internally, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T if <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. Otherwise, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T* and the items are allocated on the heap.</p>
/// <p>The array representation allows very fast insertions and index-based access. The <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() operations are also very fast because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates memory at both ends of its internal array. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.</p>
/// <p>Note, however, that when the conditions specified above are not met, each append or insert of a new item requires allocating the new item on the heap, and this per item allocation will make <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> a better choice for use cases that do a lot of appending or inserting, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> can allocate memory for many items in a single heap allocation.</p>
/// <p>Note that the internal array only ever gets bigger over the life of the list. It never shrinks. The internal array is deallocated by the destructor and by the assignment operator, when one list is assigned to another.</p>
/// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerList;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span><span class="operator">&gt;</span> dateList;
///
/// </pre>
/// <p>Qt includes a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class that inherits <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>&gt; and adds a few convenience functions, such as <a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#filter">QStringList::filter</a>(). <a href="http://doc.qt.io/qt-5/qstring.html#split">QString::split</a>() creates QStringLists from strings.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> stores a list of items. The default constructor creates an empty list. To insert items into the list, you can use operator&lt;&lt;():</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"one"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"two"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"three"</span>;
///   <span class="comment">// list: ["one", "two", "three"]</span>
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(). In addition, it provides the following convenience functions: <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const lists, operator[]() returns a reference to the item and can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Bob"</span>)
/// &#32;     list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Robert"</span>;
///
/// </pre>
/// <p>Because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented as an array of pointers for types that are larger than a pointer or are not movable, this operation requires (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>). For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> list<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (list<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Jane"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Jane at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>A common requirement is to remove an item from a list and do something with it. For this, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(). Here's a loop that removes the items from a list one at a time and calls <code>delete</code> on them:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   <span class="keyword">while</span> (<span class="operator">!</span>list<span class="operator">.</span>isEmpty())
/// &#32;     <span class="keyword">delete</span> list<span class="operator">.</span>takeFirst();
///
/// </pre>
/// <p>Inserting and removing items at either end of the list is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> in most cases), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
/// <p>If you want to find all occurrences of a particular value in a list, use <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of a matching item if they find it; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> list<span class="operator">.</span>indexOf(<span class="string">"Jane"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Jane is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a list contains a particular value, use <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the list, use <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>(). If you want to replace all occurrences of a particular value with another, use <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlistiterator.html">QListIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablelistiterator.html">QMutableListIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented in such a way that direct index-based access is just as fast as using iterators.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <p>To make <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as efficient as possible, its member functions don't validate their input before using it. Except for <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), member functions always assume the list is <i>not</i> empty. Member functions that take index values as parameters always assume their index value parameters are in the valid range. This means <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> member functions can fail. If you define QT_NO_DEBUG when you compile, failures will not be detected. If you <i>don't</i> define QT_NO_DEBUG, failures will be detected using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT">Q_ASSERT</a>() or <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT_X">Q_ASSERT_X</a>() with an appropriate message.</p>
/// <p>To avoid failures when your list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling other member functions. If you must pass an index value that might not be in the valid range, check that it is less than the value returned by <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() but <i>not</i> less than 0.</p>
/// <a name="more-members"></a></div>
#[repr(C)]
pub struct QListOfCountry {
    _unused: u8,
}
impl QListOfCountry {
    /// <p>Inserts <i>value</i> at the end of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale::Country>::append(const QLocale::Country& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["one", "two", "three"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn append_country(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_locale::Country>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_append(
            self as *mut crate::QListOfCountry,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_locale::Country>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale::Country>::append(const QList<QLocale::Country>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> list to this list.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_list_of_country(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfCountry>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_append1(
            self as *mut crate::QListOfCountry,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfCountry>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QLocale::Country& QList<QLocale::Country>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>This function is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ref<crate::q_locale::Country> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_at(
            self as *const crate::QListOfCountry,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale::Country& QList<QLocale::Country>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::MutRef<crate::q_locale::Country> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_back(
            self as *mut crate::QListOfCountry,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QLocale::Country& QList<QLocale::Country>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<crate::q_locale::Country> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_back1(
            self as *const crate::QListOfCountry,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::const_iterator QList<QLocale::Country>::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(&self) -> ::cpp_utils::CppBox<crate::q_list_of_country::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_cbegin(
            self as *const crate::QListOfCountry,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::const_iterator QList<QLocale::Country>::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn cend(&self) -> ::cpp_utils::CppBox<crate::q_list_of_country::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_cend(
            self as *const crate::QListOfCountry,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all items from the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale::Country>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all items from the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_clear(
            self as *mut crate::QListOfCountry,
        )
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::const_iterator QList<QLocale::Country>::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_country::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_constBegin(
            self as *const crate::QListOfCountry,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::const_iterator QList<QLocale::Country>::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn const_end(&self) -> ::cpp_utils::CppBox<crate::q_list_of_country::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_constEnd(
            self as *const crate::QListOfCountry,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QLocale::Country& QList<QLocale::Country>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>().</p></div>
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<crate::q_locale::Country> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_constFirst(
            self as *const crate::QListOfCountry,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QLocale::Country& QList<QLocale::Country>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>().</p></div>
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<crate::q_locale::Country> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_constLast(
            self as *const crate::QListOfCountry,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale::Country>::contains(const QLocale::Country& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn contains(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_locale::Country>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_contains(
            self as *const crate::QListOfCountry,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_locale::Country>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>& QList<QLocale::Country>::operator=(const QList<QLocale::Country>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this list and returns a reference to this list.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfCountry>>,
    ) -> ::cpp_utils::MutRef<crate::QListOfCountry> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_operator_(
            self as *mut crate::QListOfCountry,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfCountry>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of occurrences of <i>value</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QLocale::Country>::count(const QLocale::Country& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of occurrences of <i>value</i> in the list.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_locale::Country>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_count(
            self as *const crate::QListOfCountry,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_locale::Country>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QLocale::Country>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_count1(
            self as *const crate::QListOfCountry,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale::Country>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_detach(
            self as *mut crate::QListOfCountry,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale::Country>::detachShared()```</span>.
    pub unsafe fn detach_shared(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_detachShared(
            self as *mut crate::QListOfCountry,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale::Country>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_empty(
            self as *const crate::QListOfCountry,
        )
    }

    /// <p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale::Country>::endsWith(const QLocale::Country& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn ends_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_locale::Country>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_endsWith(
            self as *const crate::QListOfCountry,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_locale::Country>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::iterator QList<QLocale::Country>::erase(QList<QLocale::Country>::iterator pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn erase_1a(
        &mut self,
        pos: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_country::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_country::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_erase(self as *mut crate::QListOfCountry, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_country::Iterator>>::cast_into(pos).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::iterator QList<QLocale::Country>::erase(QList<QLocale::Country>::iterator first, QList<QLocale::Country>::iterator last)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes all the items from <i>begin</i> up to (but not including) <i>end</i>. Returns an iterator to the same item that <i>end</i> referred to before the call.</p></div>
    pub unsafe fn erase_2a(
        &mut self,
        first: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_country::Iterator>>,
        last: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_country::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_country::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_erase1(self as *mut crate::QListOfCountry, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_country::Iterator>>::cast_into(first).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_country::Iterator>>::cast_into(last).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale::Country& QList<QLocale::Country>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<crate::q_locale::Country> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_first(
            self as *mut crate::QListOfCountry,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QLocale::Country& QList<QLocale::Country>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::q_locale::Country> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_first1(
            self as *const crate::QListOfCountry,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale::Country& QList<QLocale::Country>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::MutRef<crate::q_locale::Country> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_front(
            self as *mut crate::QListOfCountry,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QLocale::Country& QList<QLocale::Country>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<crate::q_locale::Country> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_front1(
            self as *const crate::QListOfCountry,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QLocale::Country& QList<QLocale::Country>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>(). This function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p></div>
    pub unsafe fn index(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ref<crate::q_locale::Country> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_operator__2(
            self as *const crate::QListOfCountry,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale::Country& QList<QLocale::Country>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If this function is called on a list that is currently being shared, it will trigger a copy of all elements. Otherwise, this function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>. If you do not want to modify the list you should use <a href="http://doc.qt.io/qt-5/qlist.html#at">QList::at</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::q_locale::Country> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_operator__3(
            self as *mut crate::QListOfCountry,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QLocale::Country>::indexOf(const QLocale::Country& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_locale::Country>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_indexOf(
            self as *const crate::QListOfCountry,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_locale::Country>>::cast_into(t)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QLocale::Country>::indexOf(const QLocale::Country& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_locale::Country>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_indexOf1(
            self as *const crate::QListOfCountry,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_locale::Country>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale::Country>::insert(int i, const QLocale::Country& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   list<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// list: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn insert_int_country(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_locale::Country>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_insert(
            self as *mut crate::QListOfCountry,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_locale::Country>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::iterator QList<QLocale::Country>::insert(QList<QLocale::Country>::iterator before, const QLocale::Country& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>value</i> in front of the item pointed to by the iterator <i>before</i>. Returns an iterator pointing at the inserted item. Note that the iterator passed to the function will be invalid after the call; the returned iterator should be used instead.</p></div>
    pub unsafe fn insert_iterator_country(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_country::Iterator>>,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_locale::Country>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_country::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_insert1(self as *mut crate::QListOfCountry, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_country::Iterator>>::cast_into(before).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_locale::Country>>::cast_into(t).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale::Country>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_isDetached(
            self as *const crate::QListOfCountry,
        )
    }

    /// <p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale::Country>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_isEmpty(
            self as *const crate::QListOfCountry,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale::Country>::isSharedWith(const QList<QLocale::Country>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfCountry>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_isSharedWith(
            self as *const crate::QListOfCountry,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfCountry>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale::Country& QList<QLocale::Country>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<crate::q_locale::Country> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_last(
            self as *mut crate::QListOfCountry,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QLocale::Country& QList<QLocale::Country>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::q_locale::Country> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_last1(
            self as *const crate::QListOfCountry,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QLocale::Country>::lastIndexOf(const QLocale::Country& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_locale::Country>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_lastIndexOf(
            self as *const crate::QListOfCountry,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_locale::Country>>::cast_into(t)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QLocale::Country>::lastIndexOf(const QLocale::Country& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_locale::Country>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_lastIndexOf1(
            self as *const crate::QListOfCountry,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_locale::Country>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QLocale::Country>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_length(
            self as *const crate::QListOfCountry,
        )
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country> QList<QLocale::Country>::mid(int pos, int length = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        length: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfCountry> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_mid(
            self as *const crate::QListOfCountry,
            pos,
            length,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country> QList<QLocale::Country>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfCountry> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_mid1(
            self as *const crate::QListOfCountry,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale::Country>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>move(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "C", "D", "E", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This is the same as insert(<i>to</i>, <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<i>from</i>)).This function assumes that both <i>from</i> and <i>to</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>from</i> and <i>to</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_move(
            self as *mut crate::QListOfCountry,
            from,
            to,
        )
    }

    /// <p>Constructs an empty list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QLocale::Country>::QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty list.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QListOfCountry> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_QList();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QLocale::Country>::QList(const QList<QLocale::Country>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfCountry>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfCountry> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_QList1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfCountry>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale::Country>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_pop_back(
            self as *mut crate::QListOfCountry,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale::Country>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_pop_front(
            self as *mut crate::QListOfCountry,
        )
    }

    /// <p>Inserts <i>value</i> at the beginning of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale::Country>::prepend(const QLocale::Country& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(0, <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_locale::Country>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_prepend(
            self as *mut crate::QListOfCountry,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_locale::Country>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale::Country>::push_back(const QLocale::Country& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_locale::Country>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_push_back(
            self as *mut crate::QListOfCountry,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_locale::Country>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale::Country>::push_front(const QLocale::Country& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_locale::Country>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_push_front(
            self as *mut crate::QListOfCountry,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_locale::Country>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QLocale::Country>::removeAll(const QLocale::Country& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeAll(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_all(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_locale::Country>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_removeAll(
            self as *mut crate::QListOfCountry,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_locale::Country>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale::Country>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_removeAt(
            self as *mut crate::QListOfCountry,
            i,
        )
    }

    /// <p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale::Country>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_removeFirst(
            self as *mut crate::QListOfCountry,
        )
    }

    /// <p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale::Country>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_removeLast(
            self as *mut crate::QListOfCountry,
        )
    }

    /// <p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale::Country>::removeOne(const QLocale::Country& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeOne(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", ,"sun", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_one(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_locale::Country>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_removeOne(
            self as *mut crate::QListOfCountry,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_locale::Country>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale::Country>::replace(int i, const QLocale::Country& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_locale::Country>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_replace(
            self as *mut crate::QListOfCountry,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_locale::Country>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Reserve space for <i>alloc</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale::Country>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reserve space for <i>alloc</i> elements.</p>
    /// <p>If <i>alloc</i> is smaller than the current size of the list, nothing will happen.</p>
    /// <p>Use this function to avoid repetetive reallocation of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s internal data if you can predict how many elements will be appended. Note that the reservation applies only to the internal pointer array.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_reserve(
            self as *mut crate::QListOfCountry,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale::Country>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_setSharable(
            self as *mut crate::QListOfCountry,
            sharable,
        )
    }

    /// <p>Returns the number of items in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QLocale::Country>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_size(
            self as *const crate::QListOfCountry,
        )
    }

    /// <p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale::Country>::startsWith(const QLocale::Country& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn starts_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_locale::Country>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_startsWith(
            self as *const crate::QListOfCountry,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_locale::Country>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale::Country>::swap(QList<QLocale::Country>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap_1a(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QListOfCountry>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_swap(
            self as *mut crate::QListOfCountry,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QListOfCountry>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale::Country>::swap(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swap(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    pub unsafe fn swap_2a(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_swap1(
            self as *mut crate::QListOfCountry,
            i,
            j,
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QLocale::Country>::swapItemsAt(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swapItemsAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swapItemsAt(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn swap_items_at(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_swapItemsAt(
            self as *mut crate::QListOfCountry,
            i,
            j,
        )
    }

    /// <p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale::Country QList<QLocale::Country>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn take_at(&mut self, i: ::std::os::raw::c_int) -> crate::q_locale::Country {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_takeAt(
            self as *mut crate::QListOfCountry,
            i,
        )
    }

    /// <p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale::Country QList<QLocale::Country>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> crate::q_locale::Country {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_takeFirst(
            self as *mut crate::QListOfCountry,
        )
    }

    /// <p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale::Country QList<QLocale::Country>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> crate::q_locale::Country {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_takeLast(
            self as *mut crate::QListOfCountry,
        )
    }

    /// <p>Returns the value at index position <i>i</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale::Country QList<QLocale::Country>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the list.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that the index is going to be within bounds, you can use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(&self, i: ::std::os::raw::c_int) -> crate::q_locale::Country {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_value(
            self as *const crate::QListOfCountry,
            i,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QLocale::Country QList<QLocale::Country>::value(int i, const QLocale::Country& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_locale::Country>>,
    ) -> crate::q_locale::Country {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_value1(
            self as *const crate::QListOfCountry,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_locale::Country>>::cast_into(
                default_value,
            )
            .as_raw_ptr(),
        )
    }
}

pub mod q_list_of_q_command_line_option {
    //! C++ type: <span style='color: green;'>```QList<QCommandLineOption>```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QCommandLineOption>::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::iterator& QList<QCommandLineOption>::iterator::operator=(const QList<QCommandLineOption>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::Iterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_command_line_option::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_iterator_operator_8(
                    self as *mut crate::q_list_of_q_command_line_option::Iterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::Iterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::iterator QList<QCommandLineOption>::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_iterator_operator__13(
                    self as *mut crate::q_list_of_q_command_line_option::Iterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::iterator QList<QCommandLineOption>::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_iterator_operator__11(
                    self as *mut crate::q_list_of_q_command_line_option::Iterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCommandLineOption& QList<QCommandLineOption>::iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValueRef iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>. If you assign to it, the assignment will apply to the character in the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> of <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> from which you got the reference.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::MutRef<crate::QCommandLineOption> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_iterator_operator__1(
                    self as *const crate::q_list_of_q_command_line_option::Iterator,
                    j,
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QCommandLineOption>::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>().</p></div>
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::Iterator>
        {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QCommandLineOption>::iterator::iterator(const QList<QCommandLineOption>::iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::Iterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_iterator_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::Iterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCommandLineOption* QList<QCommandLineOption>::iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's value.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::MutPtr<crate::QCommandLineOption> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_iterator_operator__(
                    self as *const crate::q_list_of_q_command_line_option::Iterator,
                );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QCommandLineOption>::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::const_iterator& QList<QCommandLineOption>::const_iterator::operator=(const QList<QCommandLineOption>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::ConstIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_command_line_option::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_const_iterator_operator_6(
                    self as *mut crate::q_list_of_q_command_line_option::ConstIterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::ConstIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::const_iterator QList<QCommandLineOption>::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_const_iterator_operator__9(
                    self as *mut crate::q_list_of_q_command_line_option::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::const_iterator QList<QCommandLineOption>::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_const_iterator_operator__7(
                    self as *mut crate::q_list_of_q_command_line_option::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QCommandLineOption& QList<QCommandLineOption>::const_iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValue const_iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::Ref<crate::QCommandLineOption> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_const_iterator_operator__1(
                    self as *const crate::q_list_of_q_command_line_option::ConstIterator,
                    j,
                );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QCommandLineOption>::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>().</p></div>
        pub unsafe fn new_0a(
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_const_iterator_const_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QCommandLineOption>::const_iterator::const_iterator(const QList<QCommandLineOption>::iterator& o)```</span>.
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::Iterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_const_iterator_const_iterator2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_command_line_option::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QCommandLineOption>::const_iterator::const_iterator(const QList<QCommandLineOption>::const_iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::ConstIterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_const_iterator_const_iterator1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_command_line_option::ConstIterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current result.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QCommandLineOption* QList<QCommandLineOption>::const_iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current result.</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::Ptr<crate::QCommandLineOption> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_const_iterator_operator__(
                    self as *const crate::q_list_of_q_command_line_option::ConstIterator,
                );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_command_line_option::Iterator {
        type Output = ::cpp_utils::MutRef<crate::QCommandLineOption>;
        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCommandLineOption& QList<QCommandLineOption>::iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>().</p></div>
        fn indirection(self) -> ::cpp_utils::MutRef<crate::QCommandLineOption> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_iterator_operator_(
                    self as *const crate::q_list_of_q_command_line_option::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_command_line_option::Iterator>>
        for crate::q_list_of_q_command_line_option::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QCommandLineOption>::iterator::operator==(const QList<QCommandLineOption>::iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_list_of_q_command_line_option::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_iterator_operator__2(
                    self as *const crate::q_list_of_q_command_line_option::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_command_line_option::Iterator>>
        for crate::q_list_of_q_command_line_option::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QCommandLineOption>::iterator::operator<(const QList<QCommandLineOption>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_command_line_option::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_iterator_operator_1(
                    self as *const crate::q_list_of_q_command_line_option::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_command_line_option::Iterator>>
        for crate::q_list_of_q_command_line_option::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QCommandLineOption>::iterator::operator<=(const QList<QCommandLineOption>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_command_line_option::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_iterator_operator__4(
                    self as *const crate::q_list_of_q_command_line_option::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_command_line_option::Iterator>>
        for crate::q_list_of_q_command_line_option::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QCommandLineOption>::iterator::operator>(const QList<QCommandLineOption>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_command_line_option::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_iterator_operator_2(
                    self as *const crate::q_list_of_q_command_line_option::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_command_line_option::Iterator>>
        for crate::q_list_of_q_command_line_option::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QCommandLineOption>::iterator::operator>=(const QList<QCommandLineOption>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_command_line_option::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_iterator_operator__5(
                    self as *const crate::q_list_of_q_command_line_option::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::std::cmp::PartialEq<
            ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::ConstIterator>,
        > for crate::q_list_of_q_command_line_option::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QCommandLineOption>::iterator::operator==(const QList<QCommandLineOption>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_list_of_q_command_line_option::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_iterator_operator__6(
                    self as *const crate::q_list_of_q_command_line_option::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Lt<
            ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::ConstIterator>,
        > for crate::q_list_of_q_command_line_option::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QCommandLineOption>::iterator::operator<(const QList<QCommandLineOption>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_command_line_option::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_iterator_operator_3(
                    self as *const crate::q_list_of_q_command_line_option::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Le<
            ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::ConstIterator>,
        > for crate::q_list_of_q_command_line_option::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QCommandLineOption>::iterator::operator<=(const QList<QCommandLineOption>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_command_line_option::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_iterator_operator__8(
                    self as *const crate::q_list_of_q_command_line_option::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Gt<
            ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::ConstIterator>,
        > for crate::q_list_of_q_command_line_option::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QCommandLineOption>::iterator::operator>(const QList<QCommandLineOption>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_command_line_option::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_iterator_operator_4(
                    self as *const crate::q_list_of_q_command_line_option::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Ge<
            ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::ConstIterator>,
        > for crate::q_list_of_q_command_line_option::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QCommandLineOption>::iterator::operator>=(const QList<QCommandLineOption>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_command_line_option::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_iterator_operator__9(
                    self as *const crate::q_list_of_q_command_line_option::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_command_line_option::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_command_line_option::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::iterator& QList<QCommandLineOption>::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_command_line_option::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_iterator_operator__10(
                    self as *mut crate::q_list_of_q_command_line_option::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_command_line_option::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_command_line_option::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::iterator& QList<QCommandLineOption>::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_command_line_option::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_iterator_operator__12(
                    self as *mut crate::q_list_of_q_command_line_option::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_command_line_option::Iterator
    {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::iterator& QList<QCommandLineOption>::iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_iterator_operator__14(
                    self as *mut crate::q_list_of_q_command_line_option::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_command_line_option::Iterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::iterator& QList<QCommandLineOption>::iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_iterator_operator__15(
                    self as *mut crate::q_list_of_q_command_line_option::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_command_line_option::Iterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::iterator QList<QCommandLineOption>::iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_iterator_operator_5(
                    self as *const crate::q_list_of_q_command_line_option::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_command_line_option::Iterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::iterator QList<QCommandLineOption>::iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_iterator_operator_6(
                    self as *const crate::q_list_of_q_command_line_option::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_command_line_option::Iterator>>
        for &crate::q_list_of_q_command_line_option::Iterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QCommandLineOption>::iterator::operator-(QList<QCommandLineOption>::iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::Iterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_iterator_operator_7(
                    self as *const crate::q_list_of_q_command_line_option::Iterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_command_line_option::ConstIterator {
        type Output = ::cpp_utils::Ref<crate::QCommandLineOption>;
        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QCommandLineOption& QList<QCommandLineOption>::const_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        fn indirection(self) -> ::cpp_utils::Ref<crate::QCommandLineOption> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_const_iterator_operator_(
                    self as *const crate::q_list_of_q_command_line_option::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
            }
        }
    }

    impl
        ::std::cmp::PartialEq<
            ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::ConstIterator>,
        > for crate::q_list_of_q_command_line_option::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QCommandLineOption>::const_iterator::operator==(const QList<QCommandLineOption>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_list_of_q_command_line_option::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_const_iterator_operator__2(
                    self as *const crate::q_list_of_q_command_line_option::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Lt<
            ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::ConstIterator>,
        > for crate::q_list_of_q_command_line_option::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QCommandLineOption>::const_iterator::operator<(const QList<QCommandLineOption>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_command_line_option::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_const_iterator_operator_1(
                    self as *const crate::q_list_of_q_command_line_option::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Le<
            ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::ConstIterator>,
        > for crate::q_list_of_q_command_line_option::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QCommandLineOption>::const_iterator::operator<=(const QList<QCommandLineOption>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_command_line_option::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_const_iterator_operator__4(
                    self as *const crate::q_list_of_q_command_line_option::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Gt<
            ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::ConstIterator>,
        > for crate::q_list_of_q_command_line_option::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QCommandLineOption>::const_iterator::operator>(const QList<QCommandLineOption>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_command_line_option::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_const_iterator_operator_2(
                    self as *const crate::q_list_of_q_command_line_option::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Ge<
            ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::ConstIterator>,
        > for crate::q_list_of_q_command_line_option::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QCommandLineOption>::const_iterator::operator>=(const QList<QCommandLineOption>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_command_line_option::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_const_iterator_operator__5(
                    self as *const crate::q_list_of_q_command_line_option::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_command_line_option::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_command_line_option::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::const_iterator& QList<QCommandLineOption>::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_command_line_option::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_const_iterator_operator__6(
                    self as *mut crate::q_list_of_q_command_line_option::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_command_line_option::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_command_line_option::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::const_iterator& QList<QCommandLineOption>::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_command_line_option::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_const_iterator_operator__8(
                    self as *mut crate::q_list_of_q_command_line_option::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_command_line_option::ConstIterator
    {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::const_iterator& QList<QCommandLineOption>::const_iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_const_iterator_operator__10(
                    self as *mut crate::q_list_of_q_command_line_option::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_command_line_option::ConstIterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::const_iterator& QList<QCommandLineOption>::const_iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_const_iterator_operator__11(
                    self as *mut crate::q_list_of_q_command_line_option::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_command_line_option::ConstIterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::const_iterator QList<QCommandLineOption>::const_iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_const_iterator_operator_3(
                    self as *const crate::q_list_of_q_command_line_option::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_command_line_option::ConstIterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::const_iterator QList<QCommandLineOption>::const_iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_const_iterator_operator_4(
                    self as *const crate::q_list_of_q_command_line_option::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_command_line_option::ConstIterator>>
        for &crate::q_list_of_q_command_line_option::ConstIterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QCommandLineOption>::const_iterator::operator-(QList<QCommandLineOption>::const_iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::ConstIterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_const_iterator_operator_5(
                    self as *const crate::q_list_of_q_command_line_option::ConstIterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_command_line_option::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QCommandLineOption>::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_iterator_diterator(
                self as *mut crate::q_list_of_q_command_line_option::Iterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_command_line_option::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QCommandLineOption>::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_const_iterator_dconst_iterator(
                self as *mut crate::q_list_of_q_command_line_option::ConstIterator,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
///
/// C++ class: <span style='color: green;'>```QList<QCommandLineOption>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlist.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores items in a list that provides fast index-based access and index-based insertions and removals.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Internally, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T if <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. Otherwise, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T* and the items are allocated on the heap.</p>
/// <p>The array representation allows very fast insertions and index-based access. The <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() operations are also very fast because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates memory at both ends of its internal array. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.</p>
/// <p>Note, however, that when the conditions specified above are not met, each append or insert of a new item requires allocating the new item on the heap, and this per item allocation will make <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> a better choice for use cases that do a lot of appending or inserting, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> can allocate memory for many items in a single heap allocation.</p>
/// <p>Note that the internal array only ever gets bigger over the life of the list. It never shrinks. The internal array is deallocated by the destructor and by the assignment operator, when one list is assigned to another.</p>
/// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerList;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span><span class="operator">&gt;</span> dateList;
///
/// </pre>
/// <p>Qt includes a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class that inherits <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>&gt; and adds a few convenience functions, such as <a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#filter">QStringList::filter</a>(). <a href="http://doc.qt.io/qt-5/qstring.html#split">QString::split</a>() creates QStringLists from strings.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> stores a list of items. The default constructor creates an empty list. To insert items into the list, you can use operator&lt;&lt;():</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"one"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"two"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"three"</span>;
///   <span class="comment">// list: ["one", "two", "three"]</span>
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(). In addition, it provides the following convenience functions: <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const lists, operator[]() returns a reference to the item and can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Bob"</span>)
/// &#32;     list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Robert"</span>;
///
/// </pre>
/// <p>Because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented as an array of pointers for types that are larger than a pointer or are not movable, this operation requires (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>). For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> list<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (list<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Jane"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Jane at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>A common requirement is to remove an item from a list and do something with it. For this, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(). Here's a loop that removes the items from a list one at a time and calls <code>delete</code> on them:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   <span class="keyword">while</span> (<span class="operator">!</span>list<span class="operator">.</span>isEmpty())
/// &#32;     <span class="keyword">delete</span> list<span class="operator">.</span>takeFirst();
///
/// </pre>
/// <p>Inserting and removing items at either end of the list is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> in most cases), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
/// <p>If you want to find all occurrences of a particular value in a list, use <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of a matching item if they find it; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> list<span class="operator">.</span>indexOf(<span class="string">"Jane"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Jane is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a list contains a particular value, use <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the list, use <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>(). If you want to replace all occurrences of a particular value with another, use <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlistiterator.html">QListIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablelistiterator.html">QMutableListIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented in such a way that direct index-based access is just as fast as using iterators.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <p>To make <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as efficient as possible, its member functions don't validate their input before using it. Except for <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), member functions always assume the list is <i>not</i> empty. Member functions that take index values as parameters always assume their index value parameters are in the valid range. This means <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> member functions can fail. If you define QT_NO_DEBUG when you compile, failures will not be detected. If you <i>don't</i> define QT_NO_DEBUG, failures will be detected using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT">Q_ASSERT</a>() or <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT_X">Q_ASSERT_X</a>() with an appropriate message.</p>
/// <p>To avoid failures when your list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling other member functions. If you must pass an index value that might not be in the valid range, check that it is less than the value returned by <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() but <i>not</i> less than 0.</p>
/// <a name="more-members"></a></div>
#[repr(C)]
pub struct QListOfQCommandLineOption {
    _unused: u8,
}
impl QListOfQCommandLineOption {
    /// <p>Inserts <i>value</i> at the end of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QCommandLineOption>::append(const QCommandLineOption& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["one", "two", "three"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn append_q_command_line_option(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCommandLineOption>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_append(
            self as *mut crate::QListOfQCommandLineOption,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCommandLineOption>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QCommandLineOption>::append(const QList<QCommandLineOption>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> list to this list.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_list_of_q_command_line_option(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQCommandLineOption>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_append1(
            self as *mut crate::QListOfQCommandLineOption,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQCommandLineOption>>::cast_into(
                t,
            )
            .as_raw_ptr(),
        )
    }

    /// <p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QCommandLineOption& QList<QCommandLineOption>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>This function is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ref<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_at(
            self as *const crate::QListOfQCommandLineOption,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCommandLineOption& QList<QCommandLineOption>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::MutRef<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_back(
            self as *mut crate::QListOfQCommandLineOption,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QCommandLineOption& QList<QCommandLineOption>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_back1(
            self as *const crate::QListOfQCommandLineOption,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::const_iterator QList<QCommandLineOption>::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_cbegin(
            self as *const crate::QListOfQCommandLineOption,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::const_iterator QList<QCommandLineOption>::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn cend(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_cend(
            self as *const crate::QListOfQCommandLineOption,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all items from the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QCommandLineOption>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all items from the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_clear(
            self as *mut crate::QListOfQCommandLineOption,
        )
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::const_iterator QList<QCommandLineOption>::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_constBegin(
            self as *const crate::QListOfQCommandLineOption,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::const_iterator QList<QCommandLineOption>::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn const_end(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_constEnd(
            self as *const crate::QListOfQCommandLineOption,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QCommandLineOption& QList<QCommandLineOption>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>().</p></div>
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_constFirst(
            self as *const crate::QListOfQCommandLineOption,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QCommandLineOption& QList<QCommandLineOption>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>().</p></div>
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_constLast(
            self as *const crate::QListOfQCommandLineOption,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Assigns <i>other</i> to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>& QList<QCommandLineOption>::operator=(const QList<QCommandLineOption>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this list and returns a reference to this list.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQCommandLineOption>>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_operator_(
            self as *mut crate::QListOfQCommandLineOption,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQCommandLineOption>>::cast_into(
                l,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QCommandLineOption>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_count1(
            self as *const crate::QListOfQCommandLineOption,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QCommandLineOption>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_detach(
            self as *mut crate::QListOfQCommandLineOption,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QCommandLineOption>::detachShared()```</span>.
    pub unsafe fn detach_shared(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_detachShared(
            self as *mut crate::QListOfQCommandLineOption,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QCommandLineOption>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_empty(
            self as *const crate::QListOfQCommandLineOption,
        )
    }

    /// <p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::iterator QList<QCommandLineOption>::erase(QList<QCommandLineOption>::iterator pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn erase_1a(
        &mut self,
        pos: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::Iterator>,
        >,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_erase(
            self as *mut crate::QListOfQCommandLineOption,
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::Iterator>,
            >::cast_into(pos)
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::iterator QList<QCommandLineOption>::erase(QList<QCommandLineOption>::iterator first, QList<QCommandLineOption>::iterator last)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes all the items from <i>begin</i> up to (but not including) <i>end</i>. Returns an iterator to the same item that <i>end</i> referred to before the call.</p></div>
    pub unsafe fn erase_2a(
        &mut self,
        first: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::Iterator>,
        >,
        last: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::Iterator>,
        >,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_erase1(
            self as *mut crate::QListOfQCommandLineOption,
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::Iterator>,
            >::cast_into(first)
            .as_raw_ptr(),
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::Iterator>,
            >::cast_into(last)
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCommandLineOption& QList<QCommandLineOption>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_first(
            self as *mut crate::QListOfQCommandLineOption,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QCommandLineOption& QList<QCommandLineOption>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_first1(
            self as *const crate::QListOfQCommandLineOption,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCommandLineOption& QList<QCommandLineOption>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::MutRef<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_front(
            self as *mut crate::QListOfQCommandLineOption,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QCommandLineOption& QList<QCommandLineOption>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_front1(
            self as *const crate::QListOfQCommandLineOption,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QCommandLineOption& QList<QCommandLineOption>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>(). This function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p></div>
    pub unsafe fn index(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ref<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_operator__2(
            self as *const crate::QListOfQCommandLineOption,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCommandLineOption& QList<QCommandLineOption>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If this function is called on a list that is currently being shared, it will trigger a copy of all elements. Otherwise, this function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>. If you do not want to modify the list you should use <a href="http://doc.qt.io/qt-5/qlist.html#at">QList::at</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_operator__3(
            self as *mut crate::QListOfQCommandLineOption,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QCommandLineOption>::insert(int i, const QCommandLineOption& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   list<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// list: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn insert_int_q_command_line_option(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCommandLineOption>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_insert(
            self as *mut crate::QListOfQCommandLineOption,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCommandLineOption>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::iterator QList<QCommandLineOption>::insert(QList<QCommandLineOption>::iterator before, const QCommandLineOption& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>value</i> in front of the item pointed to by the iterator <i>before</i>. Returns an iterator pointing at the inserted item. Note that the iterator passed to the function will be invalid after the call; the returned iterator should be used instead.</p></div>
    pub unsafe fn insert_iterator_q_command_line_option(
        &mut self,
        before: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::Iterator>,
        >,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCommandLineOption>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_insert1(
            self as *mut crate::QListOfQCommandLineOption,
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::q_list_of_q_command_line_option::Iterator>,
            >::cast_into(before)
            .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCommandLineOption>>::cast_into(t)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QCommandLineOption>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_isDetached(
            self as *const crate::QListOfQCommandLineOption,
        )
    }

    /// <p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QCommandLineOption>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_isEmpty(
            self as *const crate::QListOfQCommandLineOption,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QCommandLineOption>::isSharedWith(const QList<QCommandLineOption>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQCommandLineOption>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_isSharedWith(
            self as *const crate::QListOfQCommandLineOption,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQCommandLineOption>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        )
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCommandLineOption& QList<QCommandLineOption>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_last(
            self as *mut crate::QListOfQCommandLineOption,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QCommandLineOption& QList<QCommandLineOption>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_last1(
            self as *const crate::QListOfQCommandLineOption,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QCommandLineOption>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_length(
            self as *const crate::QListOfQCommandLineOption,
        )
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption> QList<QCommandLineOption>::mid(int pos, int length = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        length: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_mid(
            self as *const crate::QListOfQCommandLineOption,
            pos,
            length,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption> QList<QCommandLineOption>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_mid1(
            self as *const crate::QListOfQCommandLineOption,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QCommandLineOption>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>move(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "C", "D", "E", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This is the same as insert(<i>to</i>, <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<i>from</i>)).This function assumes that both <i>from</i> and <i>to</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>from</i> and <i>to</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_move(
            self as *mut crate::QListOfQCommandLineOption,
            from,
            to,
        )
    }

    /// <p>Constructs an empty list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QCommandLineOption>::QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty list.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QListOfQCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_QList();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QCommandLineOption>::QList(const QList<QCommandLineOption>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQCommandLineOption>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_QList1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQCommandLineOption>>::cast_into(
                l,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QCommandLineOption>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_pop_back(
            self as *mut crate::QListOfQCommandLineOption,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QCommandLineOption>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_pop_front(
            self as *mut crate::QListOfQCommandLineOption,
        )
    }

    /// <p>Inserts <i>value</i> at the beginning of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QCommandLineOption>::prepend(const QCommandLineOption& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(0, <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCommandLineOption>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_prepend(
            self as *mut crate::QListOfQCommandLineOption,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCommandLineOption>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QCommandLineOption>::push_back(const QCommandLineOption& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCommandLineOption>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_push_back(
            self as *mut crate::QListOfQCommandLineOption,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCommandLineOption>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QCommandLineOption>::push_front(const QCommandLineOption& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCommandLineOption>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_push_front(
            self as *mut crate::QListOfQCommandLineOption,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCommandLineOption>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QCommandLineOption>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_removeAt(
            self as *mut crate::QListOfQCommandLineOption,
            i,
        )
    }

    /// <p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QCommandLineOption>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_removeFirst(
            self as *mut crate::QListOfQCommandLineOption,
        )
    }

    /// <p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QCommandLineOption>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_removeLast(
            self as *mut crate::QListOfQCommandLineOption,
        )
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QCommandLineOption>::replace(int i, const QCommandLineOption& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCommandLineOption>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_replace(
            self as *mut crate::QListOfQCommandLineOption,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCommandLineOption>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Reserve space for <i>alloc</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QCommandLineOption>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reserve space for <i>alloc</i> elements.</p>
    /// <p>If <i>alloc</i> is smaller than the current size of the list, nothing will happen.</p>
    /// <p>Use this function to avoid repetetive reallocation of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s internal data if you can predict how many elements will be appended. Note that the reservation applies only to the internal pointer array.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_reserve(
            self as *mut crate::QListOfQCommandLineOption,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QCommandLineOption>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_setSharable(
            self as *mut crate::QListOfQCommandLineOption,
            sharable,
        )
    }

    /// <p>Returns the number of items in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QCommandLineOption>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_size(
            self as *const crate::QListOfQCommandLineOption,
        )
    }

    /// <p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QCommandLineOption>::swap(QList<QCommandLineOption>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap_1a(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QListOfQCommandLineOption>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_swap(self as *mut crate::QListOfQCommandLineOption, ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QListOfQCommandLineOption>>::cast_into(other).as_mut_raw_ptr())
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QCommandLineOption>::swap(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swap(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    pub unsafe fn swap_2a(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_swap1(
            self as *mut crate::QListOfQCommandLineOption,
            i,
            j,
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QCommandLineOption>::swapItemsAt(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swapItemsAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swapItemsAt(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn swap_items_at(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_swapItemsAt(
            self as *mut crate::QListOfQCommandLineOption,
            i,
            j,
        )
    }

    /// <p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCommandLineOption QList<QCommandLineOption>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_takeAt(
            self as *mut crate::QListOfQCommandLineOption,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCommandLineOption QList<QCommandLineOption>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> ::cpp_utils::CppBox<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_takeFirst(
            self as *mut crate::QListOfQCommandLineOption,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCommandLineOption QList<QCommandLineOption>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> ::cpp_utils::CppBox<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_takeLast(
            self as *mut crate::QListOfQCommandLineOption,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCommandLineOption QList<QCommandLineOption>::value(int i, const QCommandLineOption& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCommandLineOption>>,
    ) -> ::cpp_utils::CppBox<crate::QCommandLineOption> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_value1(
            self as *const crate::QListOfQCommandLineOption,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCommandLineOption>>::cast_into(
                default_value,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_list_of_q_file_info {
    //! C++ type: <span style='color: green;'>```QList<QFileInfo>```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QFileInfo>::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::iterator& QList<QFileInfo>::iterator::operator=(const QList<QFileInfo>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_file_info::Iterator>>,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_file_info::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_iterator_operator_8(self as *mut crate::q_list_of_q_file_info::Iterator, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_file_info::Iterator>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::iterator QList<QFileInfo>::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_file_info::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_iterator_operator__13(
                self as *mut crate::q_list_of_q_file_info::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::iterator QList<QFileInfo>::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_file_info::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_iterator_operator__11(
                self as *mut crate::q_list_of_q_file_info::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QFileInfo& QList<QFileInfo>::iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValueRef iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>. If you assign to it, the assignment will apply to the character in the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> of <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> from which you got the reference.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::MutRef<crate::QFileInfo> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_iterator_operator__1(
                self as *const crate::q_list_of_q_file_info::Iterator,
                j,
            );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QFileInfo>::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>().</p></div>
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_list_of_q_file_info::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QFileInfo>::iterator::iterator(const QList<QFileInfo>::iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_file_info::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_file_info::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_iterator_iterator1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_file_info::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QFileInfo* QList<QFileInfo>::iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's value.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::MutPtr<crate::QFileInfo> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_iterator_operator__(
                self as *const crate::q_list_of_q_file_info::Iterator,
            );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QFileInfo>::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::const_iterator& QList<QFileInfo>::const_iterator::operator=(const QList<QFileInfo>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_file_info::ConstIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_file_info::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_const_iterator_operator_6(
                    self as *mut crate::q_list_of_q_file_info::ConstIterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_file_info::ConstIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::const_iterator QList<QFileInfo>::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_file_info::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_const_iterator_operator__9(
                    self as *mut crate::q_list_of_q_file_info::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::const_iterator QList<QFileInfo>::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_file_info::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_const_iterator_operator__7(
                    self as *mut crate::q_list_of_q_file_info::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QFileInfo& QList<QFileInfo>::const_iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValue const_iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::Ref<crate::QFileInfo> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_const_iterator_operator__1(
                    self as *const crate::q_list_of_q_file_info::ConstIterator,
                    j,
                );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QFileInfo>::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>().</p></div>
        pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::q_list_of_q_file_info::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_const_iterator_const_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QFileInfo>::const_iterator::const_iterator(const QList<QFileInfo>::iterator& o)```</span>.
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_file_info::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_file_info::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_const_iterator_const_iterator2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_file_info::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QFileInfo>::const_iterator::const_iterator(const QList<QFileInfo>::const_iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_file_info::ConstIterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_file_info::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_const_iterator_const_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_file_info::ConstIterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current result.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QFileInfo* QList<QFileInfo>::const_iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current result.</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::Ptr<crate::QFileInfo> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_const_iterator_operator__(
                    self as *const crate::q_list_of_q_file_info::ConstIterator,
                );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_file_info::Iterator {
        type Output = ::cpp_utils::MutRef<crate::QFileInfo>;
        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QFileInfo& QList<QFileInfo>::iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>().</p></div>
        fn indirection(self) -> ::cpp_utils::MutRef<crate::QFileInfo> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_iterator_operator_(
                    self as *const crate::q_list_of_q_file_info::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_file_info::Iterator>>
        for crate::q_list_of_q_file_info::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QFileInfo>::iterator::operator==(const QList<QFileInfo>::iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_file_info::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_iterator_operator__2(
                    self as *const crate::q_list_of_q_file_info::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_file_info::Iterator>>
        for crate::q_list_of_q_file_info::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QFileInfo>::iterator::operator<(const QList<QFileInfo>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_file_info::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_iterator_operator_1(
                    self as *const crate::q_list_of_q_file_info::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_file_info::Iterator>>
        for crate::q_list_of_q_file_info::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QFileInfo>::iterator::operator<=(const QList<QFileInfo>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_file_info::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_iterator_operator__4(
                    self as *const crate::q_list_of_q_file_info::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_file_info::Iterator>>
        for crate::q_list_of_q_file_info::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QFileInfo>::iterator::operator>(const QList<QFileInfo>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_file_info::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_iterator_operator_2(
                    self as *const crate::q_list_of_q_file_info::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_file_info::Iterator>>
        for crate::q_list_of_q_file_info::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QFileInfo>::iterator::operator>=(const QList<QFileInfo>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_file_info::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_iterator_operator__5(
                    self as *const crate::q_list_of_q_file_info::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_file_info::ConstIterator>>
        for crate::q_list_of_q_file_info::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QFileInfo>::iterator::operator==(const QList<QFileInfo>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_file_info::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_iterator_operator__6(
                    self as *const crate::q_list_of_q_file_info::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_file_info::ConstIterator>>
        for crate::q_list_of_q_file_info::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QFileInfo>::iterator::operator<(const QList<QFileInfo>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_file_info::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_iterator_operator_3(
                    self as *const crate::q_list_of_q_file_info::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_file_info::ConstIterator>>
        for crate::q_list_of_q_file_info::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QFileInfo>::iterator::operator<=(const QList<QFileInfo>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_file_info::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_iterator_operator__8(
                    self as *const crate::q_list_of_q_file_info::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_file_info::ConstIterator>>
        for crate::q_list_of_q_file_info::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QFileInfo>::iterator::operator>(const QList<QFileInfo>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_file_info::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_iterator_operator_4(
                    self as *const crate::q_list_of_q_file_info::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_file_info::ConstIterator>>
        for crate::q_list_of_q_file_info::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QFileInfo>::iterator::operator>=(const QList<QFileInfo>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_file_info::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_iterator_operator__9(
                    self as *const crate::q_list_of_q_file_info::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_file_info::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_file_info::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::iterator& QList<QFileInfo>::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_file_info::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_iterator_operator__10(
                    self as *mut crate::q_list_of_q_file_info::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_file_info::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_file_info::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::iterator& QList<QFileInfo>::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_file_info::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_iterator_operator__12(
                    self as *mut crate::q_list_of_q_file_info::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong> for crate::q_list_of_q_file_info::Iterator {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::iterator& QList<QFileInfo>::iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_iterator_operator__14(
                    self as *mut crate::q_list_of_q_file_info::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong> for crate::q_list_of_q_file_info::Iterator {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::iterator& QList<QFileInfo>::iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_iterator_operator__15(
                    self as *mut crate::q_list_of_q_file_info::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_list_of_q_file_info::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_file_info::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::iterator QList<QFileInfo>::iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_file_info::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_iterator_operator_5(
                    self as *const crate::q_list_of_q_file_info::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_list_of_q_file_info::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_file_info::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::iterator QList<QFileInfo>::iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_file_info::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_iterator_operator_6(
                    self as *const crate::q_list_of_q_file_info::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_file_info::Iterator>>
        for &crate::q_list_of_q_file_info::Iterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QFileInfo>::iterator::operator-(QList<QFileInfo>::iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_file_info::Iterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_iterator_operator_7(
                    self as *const crate::q_list_of_q_file_info::Iterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_file_info::ConstIterator {
        type Output = ::cpp_utils::Ref<crate::QFileInfo>;
        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QFileInfo& QList<QFileInfo>::const_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        fn indirection(self) -> ::cpp_utils::Ref<crate::QFileInfo> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_const_iterator_operator_(
                    self as *const crate::q_list_of_q_file_info::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_file_info::ConstIterator>>
        for crate::q_list_of_q_file_info::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QFileInfo>::const_iterator::operator==(const QList<QFileInfo>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_file_info::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_const_iterator_operator__2(
                    self as *const crate::q_list_of_q_file_info::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_file_info::ConstIterator>>
        for crate::q_list_of_q_file_info::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QFileInfo>::const_iterator::operator<(const QList<QFileInfo>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_file_info::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_const_iterator_operator_1(
                    self as *const crate::q_list_of_q_file_info::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_file_info::ConstIterator>>
        for crate::q_list_of_q_file_info::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QFileInfo>::const_iterator::operator<=(const QList<QFileInfo>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_file_info::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_const_iterator_operator__4(
                    self as *const crate::q_list_of_q_file_info::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_file_info::ConstIterator>>
        for crate::q_list_of_q_file_info::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QFileInfo>::const_iterator::operator>(const QList<QFileInfo>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_file_info::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_const_iterator_operator_2(
                    self as *const crate::q_list_of_q_file_info::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_file_info::ConstIterator>>
        for crate::q_list_of_q_file_info::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QFileInfo>::const_iterator::operator>=(const QList<QFileInfo>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_file_info::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_const_iterator_operator__5(
                    self as *const crate::q_list_of_q_file_info::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_file_info::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_file_info::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::const_iterator& QList<QFileInfo>::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_file_info::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_const_iterator_operator__6(
                    self as *mut crate::q_list_of_q_file_info::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_file_info::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_file_info::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::const_iterator& QList<QFileInfo>::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_file_info::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_const_iterator_operator__8(
                    self as *mut crate::q_list_of_q_file_info::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_file_info::ConstIterator
    {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::const_iterator& QList<QFileInfo>::const_iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_const_iterator_operator__10(
                    self as *mut crate::q_list_of_q_file_info::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_file_info::ConstIterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::const_iterator& QList<QFileInfo>::const_iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_const_iterator_operator__11(
                    self as *mut crate::q_list_of_q_file_info::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_list_of_q_file_info::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_file_info::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::const_iterator QList<QFileInfo>::const_iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_file_info::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_const_iterator_operator_3(
                    self as *const crate::q_list_of_q_file_info::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_list_of_q_file_info::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_file_info::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::const_iterator QList<QFileInfo>::const_iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_file_info::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_const_iterator_operator_4(
                    self as *const crate::q_list_of_q_file_info::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_file_info::ConstIterator>>
        for &crate::q_list_of_q_file_info::ConstIterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QFileInfo>::const_iterator::operator-(QList<QFileInfo>::const_iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_file_info::ConstIterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_const_iterator_operator_5(
                    self as *const crate::q_list_of_q_file_info::ConstIterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_file_info::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QFileInfo>::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_iterator_diterator(
                self as *mut crate::q_list_of_q_file_info::Iterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_file_info::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QFileInfo>::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_const_iterator_dconst_iterator(
                self as *mut crate::q_list_of_q_file_info::ConstIterator,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
///
/// C++ class: <span style='color: green;'>```QList<QFileInfo>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlist.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores items in a list that provides fast index-based access and index-based insertions and removals.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Internally, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T if <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. Otherwise, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T* and the items are allocated on the heap.</p>
/// <p>The array representation allows very fast insertions and index-based access. The <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() operations are also very fast because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates memory at both ends of its internal array. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.</p>
/// <p>Note, however, that when the conditions specified above are not met, each append or insert of a new item requires allocating the new item on the heap, and this per item allocation will make <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> a better choice for use cases that do a lot of appending or inserting, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> can allocate memory for many items in a single heap allocation.</p>
/// <p>Note that the internal array only ever gets bigger over the life of the list. It never shrinks. The internal array is deallocated by the destructor and by the assignment operator, when one list is assigned to another.</p>
/// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerList;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span><span class="operator">&gt;</span> dateList;
///
/// </pre>
/// <p>Qt includes a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class that inherits <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>&gt; and adds a few convenience functions, such as <a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#filter">QStringList::filter</a>(). <a href="http://doc.qt.io/qt-5/qstring.html#split">QString::split</a>() creates QStringLists from strings.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> stores a list of items. The default constructor creates an empty list. To insert items into the list, you can use operator&lt;&lt;():</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"one"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"two"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"three"</span>;
///   <span class="comment">// list: ["one", "two", "three"]</span>
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(). In addition, it provides the following convenience functions: <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const lists, operator[]() returns a reference to the item and can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Bob"</span>)
/// &#32;     list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Robert"</span>;
///
/// </pre>
/// <p>Because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented as an array of pointers for types that are larger than a pointer or are not movable, this operation requires (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>). For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> list<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (list<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Jane"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Jane at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>A common requirement is to remove an item from a list and do something with it. For this, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(). Here's a loop that removes the items from a list one at a time and calls <code>delete</code> on them:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   <span class="keyword">while</span> (<span class="operator">!</span>list<span class="operator">.</span>isEmpty())
/// &#32;     <span class="keyword">delete</span> list<span class="operator">.</span>takeFirst();
///
/// </pre>
/// <p>Inserting and removing items at either end of the list is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> in most cases), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
/// <p>If you want to find all occurrences of a particular value in a list, use <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of a matching item if they find it; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> list<span class="operator">.</span>indexOf(<span class="string">"Jane"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Jane is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a list contains a particular value, use <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the list, use <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>(). If you want to replace all occurrences of a particular value with another, use <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlistiterator.html">QListIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablelistiterator.html">QMutableListIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented in such a way that direct index-based access is just as fast as using iterators.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <p>To make <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as efficient as possible, its member functions don't validate their input before using it. Except for <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), member functions always assume the list is <i>not</i> empty. Member functions that take index values as parameters always assume their index value parameters are in the valid range. This means <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> member functions can fail. If you define QT_NO_DEBUG when you compile, failures will not be detected. If you <i>don't</i> define QT_NO_DEBUG, failures will be detected using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT">Q_ASSERT</a>() or <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT_X">Q_ASSERT_X</a>() with an appropriate message.</p>
/// <p>To avoid failures when your list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling other member functions. If you must pass an index value that might not be in the valid range, check that it is less than the value returned by <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() but <i>not</i> less than 0.</p>
/// <a name="more-members"></a></div>
#[repr(C)]
pub struct QListOfQFileInfo {
    _unused: u8,
}
impl QListOfQFileInfo {
    /// <p>Inserts <i>value</i> at the end of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QFileInfo>::append(const QFileInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["one", "two", "three"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn append_q_file_info(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QFileInfo>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_append(
            self as *mut crate::QListOfQFileInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QFileInfo>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QFileInfo>::append(const QList<QFileInfo>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> list to this list.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_list_of_q_file_info(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQFileInfo>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_append1(
            self as *mut crate::QListOfQFileInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQFileInfo>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QFileInfo& QList<QFileInfo>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>This function is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::QFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_at(
            self as *const crate::QListOfQFileInfo,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFileInfo& QList<QFileInfo>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::MutRef<crate::QFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_back(
            self as *mut crate::QListOfQFileInfo,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QFileInfo& QList<QFileInfo>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<crate::QFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_back1(
            self as *const crate::QListOfQFileInfo,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::const_iterator QList<QFileInfo>::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_file_info::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_cbegin(
            self as *const crate::QListOfQFileInfo,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::const_iterator QList<QFileInfo>::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn cend(&self) -> ::cpp_utils::CppBox<crate::q_list_of_q_file_info::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_cend(
            self as *const crate::QListOfQFileInfo,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all items from the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QFileInfo>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all items from the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_clear(self as *mut crate::QListOfQFileInfo)
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::const_iterator QList<QFileInfo>::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_file_info::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_constBegin(
            self as *const crate::QListOfQFileInfo,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::const_iterator QList<QFileInfo>::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn const_end(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_file_info::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_constEnd(
            self as *const crate::QListOfQFileInfo,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QFileInfo& QList<QFileInfo>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>().</p></div>
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<crate::QFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_constFirst(
            self as *const crate::QListOfQFileInfo,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QFileInfo& QList<QFileInfo>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>().</p></div>
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<crate::QFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_constLast(
            self as *const crate::QListOfQFileInfo,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QFileInfo>::contains(const QFileInfo& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn contains(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QFileInfo>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_contains(
            self as *const crate::QListOfQFileInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QFileInfo>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>& QList<QFileInfo>::operator=(const QList<QFileInfo>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this list and returns a reference to this list.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQFileInfo>>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_operator_(
            self as *mut crate::QListOfQFileInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQFileInfo>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of occurrences of <i>value</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QFileInfo>::count(const QFileInfo& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of occurrences of <i>value</i> in the list.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QFileInfo>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_count(
            self as *const crate::QListOfQFileInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QFileInfo>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QFileInfo>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_count1(self as *const crate::QListOfQFileInfo)
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QFileInfo>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_detach(self as *mut crate::QListOfQFileInfo)
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QFileInfo>::detachShared()```</span>.
    pub unsafe fn detach_shared(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_detachShared(
            self as *mut crate::QListOfQFileInfo,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QFileInfo>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_empty(self as *const crate::QListOfQFileInfo)
    }

    /// <p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QFileInfo>::endsWith(const QFileInfo& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn ends_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QFileInfo>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_endsWith(
            self as *const crate::QListOfQFileInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QFileInfo>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::iterator QList<QFileInfo>::erase(QList<QFileInfo>::iterator pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn erase_1a(
        &mut self,
        pos: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_file_info::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_file_info::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_erase(self as *mut crate::QListOfQFileInfo, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_file_info::Iterator>>::cast_into(pos).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::iterator QList<QFileInfo>::erase(QList<QFileInfo>::iterator first, QList<QFileInfo>::iterator last)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes all the items from <i>begin</i> up to (but not including) <i>end</i>. Returns an iterator to the same item that <i>end</i> referred to before the call.</p></div>
    pub unsafe fn erase_2a(
        &mut self,
        first: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_file_info::Iterator>>,
        last: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_file_info::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_file_info::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_erase1(self as *mut crate::QListOfQFileInfo, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_file_info::Iterator>>::cast_into(first).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_file_info::Iterator>>::cast_into(last).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFileInfo& QList<QFileInfo>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<crate::QFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_first(
            self as *mut crate::QListOfQFileInfo,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QFileInfo& QList<QFileInfo>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::QFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_first1(
            self as *const crate::QListOfQFileInfo,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFileInfo& QList<QFileInfo>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::MutRef<crate::QFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_front(
            self as *mut crate::QListOfQFileInfo,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QFileInfo& QList<QFileInfo>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<crate::QFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_front1(
            self as *const crate::QListOfQFileInfo,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QFileInfo& QList<QFileInfo>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>(). This function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p></div>
    pub unsafe fn index(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::QFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_operator__2(
            self as *const crate::QListOfQFileInfo,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFileInfo& QList<QFileInfo>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If this function is called on a list that is currently being shared, it will trigger a copy of all elements. Otherwise, this function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>. If you do not want to modify the list you should use <a href="http://doc.qt.io/qt-5/qlist.html#at">QList::at</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_operator__3(
            self as *mut crate::QListOfQFileInfo,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QFileInfo>::indexOf(const QFileInfo& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QFileInfo>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_indexOf(
            self as *const crate::QListOfQFileInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QFileInfo>>::cast_into(t).as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QFileInfo>::indexOf(const QFileInfo& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QFileInfo>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_indexOf1(
            self as *const crate::QListOfQFileInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QFileInfo>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QFileInfo>::insert(int i, const QFileInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   list<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// list: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn insert_int_q_file_info(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QFileInfo>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_insert(
            self as *mut crate::QListOfQFileInfo,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QFileInfo>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::iterator QList<QFileInfo>::insert(QList<QFileInfo>::iterator before, const QFileInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>value</i> in front of the item pointed to by the iterator <i>before</i>. Returns an iterator pointing at the inserted item. Note that the iterator passed to the function will be invalid after the call; the returned iterator should be used instead.</p></div>
    pub unsafe fn insert_iterator_q_file_info(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_file_info::Iterator>>,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QFileInfo>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_file_info::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_insert1(self as *mut crate::QListOfQFileInfo, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_file_info::Iterator>>::cast_into(before).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QFileInfo>>::cast_into(t).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QFileInfo>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_isDetached(
            self as *const crate::QListOfQFileInfo,
        )
    }

    /// <p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QFileInfo>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_isEmpty(
            self as *const crate::QListOfQFileInfo,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QFileInfo>::isSharedWith(const QList<QFileInfo>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQFileInfo>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_isSharedWith(
            self as *const crate::QListOfQFileInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQFileInfo>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFileInfo& QList<QFileInfo>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<crate::QFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_last(
            self as *mut crate::QListOfQFileInfo,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QFileInfo& QList<QFileInfo>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::QFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_last1(
            self as *const crate::QListOfQFileInfo,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QFileInfo>::lastIndexOf(const QFileInfo& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QFileInfo>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_lastIndexOf(
            self as *const crate::QListOfQFileInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QFileInfo>>::cast_into(t).as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QFileInfo>::lastIndexOf(const QFileInfo& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QFileInfo>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_lastIndexOf1(
            self as *const crate::QListOfQFileInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QFileInfo>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QFileInfo>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_length(self as *const crate::QListOfQFileInfo)
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo> QList<QFileInfo>::mid(int pos, int length = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        length: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_mid(
            self as *const crate::QListOfQFileInfo,
            pos,
            length,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo> QList<QFileInfo>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_mid1(
            self as *const crate::QListOfQFileInfo,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QFileInfo>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>move(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "C", "D", "E", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This is the same as insert(<i>to</i>, <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<i>from</i>)).This function assumes that both <i>from</i> and <i>to</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>from</i> and <i>to</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_move(
            self as *mut crate::QListOfQFileInfo,
            from,
            to,
        )
    }

    /// <p>Constructs an empty list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QFileInfo>::QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty list.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QListOfQFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_QList();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QFileInfo>::QList(const QList<QFileInfo>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQFileInfo>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_QList1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQFileInfo>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QFileInfo>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_pop_back(self as *mut crate::QListOfQFileInfo)
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QFileInfo>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_pop_front(
            self as *mut crate::QListOfQFileInfo,
        )
    }

    /// <p>Inserts <i>value</i> at the beginning of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QFileInfo>::prepend(const QFileInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(0, <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QFileInfo>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_prepend(
            self as *mut crate::QListOfQFileInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QFileInfo>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QFileInfo>::push_back(const QFileInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QFileInfo>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_push_back(
            self as *mut crate::QListOfQFileInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QFileInfo>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QFileInfo>::push_front(const QFileInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QFileInfo>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_push_front(
            self as *mut crate::QListOfQFileInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QFileInfo>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QFileInfo>::removeAll(const QFileInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeAll(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_all(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QFileInfo>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_removeAll(
            self as *mut crate::QListOfQFileInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QFileInfo>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QFileInfo>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_removeAt(
            self as *mut crate::QListOfQFileInfo,
            i,
        )
    }

    /// <p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QFileInfo>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_removeFirst(
            self as *mut crate::QListOfQFileInfo,
        )
    }

    /// <p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QFileInfo>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_removeLast(
            self as *mut crate::QListOfQFileInfo,
        )
    }

    /// <p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QFileInfo>::removeOne(const QFileInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeOne(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", ,"sun", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_one(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QFileInfo>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_removeOne(
            self as *mut crate::QListOfQFileInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QFileInfo>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QFileInfo>::replace(int i, const QFileInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QFileInfo>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_replace(
            self as *mut crate::QListOfQFileInfo,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QFileInfo>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Reserve space for <i>alloc</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QFileInfo>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reserve space for <i>alloc</i> elements.</p>
    /// <p>If <i>alloc</i> is smaller than the current size of the list, nothing will happen.</p>
    /// <p>Use this function to avoid repetetive reallocation of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s internal data if you can predict how many elements will be appended. Note that the reservation applies only to the internal pointer array.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_reserve(
            self as *mut crate::QListOfQFileInfo,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QFileInfo>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_setSharable(
            self as *mut crate::QListOfQFileInfo,
            sharable,
        )
    }

    /// <p>Returns the number of items in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QFileInfo>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_size(self as *const crate::QListOfQFileInfo)
    }

    /// <p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QFileInfo>::startsWith(const QFileInfo& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn starts_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QFileInfo>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_startsWith(
            self as *const crate::QListOfQFileInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QFileInfo>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QFileInfo>::swap(QList<QFileInfo>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap_1a(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QListOfQFileInfo>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_swap(
            self as *mut crate::QListOfQFileInfo,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QListOfQFileInfo>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QFileInfo>::swap(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swap(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    pub unsafe fn swap_2a(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_swap1(
            self as *mut crate::QListOfQFileInfo,
            i,
            j,
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QFileInfo>::swapItemsAt(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swapItemsAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swapItemsAt(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn swap_items_at(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_swapItemsAt(
            self as *mut crate::QListOfQFileInfo,
            i,
            j,
        )
    }

    /// <p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFileInfo QList<QFileInfo>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_takeAt(
            self as *mut crate::QListOfQFileInfo,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFileInfo QList<QFileInfo>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> ::cpp_utils::CppBox<crate::QFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_takeFirst(
            self as *mut crate::QListOfQFileInfo,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFileInfo QList<QFileInfo>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> ::cpp_utils::CppBox<crate::QFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_takeLast(
            self as *mut crate::QListOfQFileInfo,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value at index position <i>i</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFileInfo QList<QFileInfo>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the list.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that the index is going to be within bounds, you can use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_value(
            self as *const crate::QListOfQFileInfo,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFileInfo QList<QFileInfo>::value(int i, const QFileInfo& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QFileInfo>>,
    ) -> ::cpp_utils::CppBox<crate::QFileInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_value1(
            self as *const crate::QListOfQFileInfo,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QFileInfo>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> class is a template class that provides a dynamic array.</p>
///
/// C++ class: <span style='color: green;'>```QVector<QPointF>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qvector.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> class is a template class that provides a dynamic array.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores its items in adjacent memory locations and provides fast index-based access.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Here's an example of a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> that stores <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerVector;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> stringVector;
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> stores its items in a vector (array). Typically, vectors are created with an initial size. For example, the following code constructs a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> with 200 elements:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">200</span>);
///
/// </pre>
/// <p>The elements are automatically initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you want to initialize the vector with a different value, pass that value as the second argument to the constructor:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">200</span><span class="operator">,</span> <span class="string">"Pass"</span>);
///
/// </pre>
/// <p>You can also call <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>() at any time to fill the vector with a value.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const vectors, operator[]() returns a reference to the item that can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (vector<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Liz"</span>)
/// &#32;     vector<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Elizabeth"</span>;
///
/// </pre>
/// <p>For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> vector<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (vector<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Alfonso"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Alfonso at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>Another way to access the data stored in a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> is to call <a href="http://doc.qt.io/qt-5/qvector.html#data">data</a>(). The function returns a pointer to the first item in the vector. You can use the pointer to directly access and modify the elements stored in the vector. The pointer is also useful if you need to pass a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to a function that accepts a plain C++ array.</p>
/// <p>If you want to find all occurrences of a particular value in a vector, use <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of the matching item if they found one; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> vector<span class="operator">.</span>indexOf(<span class="string">"Harumi"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Harumi is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a vector contains a particular value, use <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the vector, use <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(). With the exception of <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), these functions can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>) for large vectors, because they require moving many items in the vector by one position in memory. If you want a container class that provides fast insertion/removal in the middle, use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> or <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
/// <p>Unlike plain C++ arrays, QVectors can be resized at any time by calling <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>(). If the new size is larger than the old size, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> might need to reallocate the whole vector. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> tries to reduce the number of reallocations by preallocating up to twice as much memory as the actual data needs.</p>
/// <p>If you know in advance approximately how many items the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> will contain, you can call <a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>(), asking <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to preallocate a certain amount of memory. You can also call <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>() to find out how much memory <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> actually allocated.</p>
/// <p>Note that using non-const operators and functions can cause <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to do a deep copy of the data. This is due to <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicit sharing</a>.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qvectoriterator.html">QVectorIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablevectoriterator.html">QMutableVectorIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qvector.html#const_iterator-typedef">QVector::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qvector.html#iterator-typedefx">QVector::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>.</p>
/// <p>In addition to <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>, Qt also provides <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a>, a very low-level class with little functionality that is optimized for speed.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <a name="more-information-on-using-qt-containers"></a>
/// <h4>More Information on Using Qt Containers</h4>
/// <p>For a detailed discussion comparing Qt containers with each other and with STL containers, see <a href="http://marcmutz.wordpress.com/effective-qt/containers/">Understand the Qt Containers</a>.</p></div>
#[repr(C)]
pub struct QVectorOfQPointF {
    _unused: u8,
}
impl QVectorOfQPointF {
    /// <p>Inserts <i>value</i> at the end of the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPointF>::append(const QPointF& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   vector<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> three <span class="operator">=</span> <span class="string">"three"</span>;
    ///   vector<span class="operator">.</span>append(three);
    ///   <span class="comment">// vector: ["one", "two", "three"]</span>
    ///   <span class="comment">// three: "three"</span>
    ///
    /// </pre>
    /// <p>This is the same as calling resize(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() + 1) and assigning <i>value</i> to the new last element in the vector.</p>
    /// <p>This operation is relatively fast, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> typically allocates more memory than necessary, so it can grow without reallocating the entire vector each time.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>().</p></div>
    pub unsafe fn append_q_point_f(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_append(
            self as *mut crate::QVectorOfQPointF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPointF>::append(const QVector<QPointF>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#append-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> vector to this vector.</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_vector_of_q_point_f(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQPointF>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_append2(
            self as *mut crate::QVectorOfQPointF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQPointF>>::cast_into(l)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the item at index position <i>i</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPointF& QVector<QPointF>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the vector.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_at(
            self as *const crate::QVectorOfQPointF,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPointF& QVector<QPointF>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::MutRef<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_back(
            self as *mut crate::QVectorOfQPointF,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPointF& QVector<QPointF>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_back1(
            self as *const crate::QVectorOfQPointF,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the maximum number of items that can be stored in the vector without forcing a reallocation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QPointF>::capacity() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#capacity">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the maximum number of items that can be stored in the vector without forcing a reallocation.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s memory usage. In general, you will rarely ever need to call this function. If you want to know how many items are in the vector, call <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p></div>
    pub unsafe fn capacity(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_capacity(
            self as *const crate::QVectorOfQPointF,
        )
    }

    /// <p>Removes all the elements from the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPointF>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all the elements from the vector.</p>
    /// <p><b>Note: </b>Until Qt 5.6, this also released the memory used by the vector. From Qt 5.7, the capacity is preserved. To shed all capacity, swap with a default-constructed vector:</p><pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span>T<span class="operator">&gt;</span> v <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span>T<span class="operator">&gt;</span>()<span class="operator">.</span>swap(v);
    ///   Q_ASSERT(v<span class="operator">.</span>capacity() <span class="operator">=</span><span class="operator">=</span> <span class="number">0</span>);
    ///
    /// </pre>
    /// <p>or call <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_clear(self as *mut crate::QVectorOfQPointF)
    }

    /// <p>Returns a const pointer to the data stored in the vector. The pointer can be used to access the items in the vector. The pointer remains valid as long as the vector isn't reallocated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPointF* QVector<QPointF>::constData() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const pointer to the data stored in the vector. The pointer can be used to access the items in the vector. The pointer remains valid as long as the vector isn't reallocated.</p>
    /// <p>This function is mostly useful to pass a vector to a function that accepts a plain C++ array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#data">data</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn const_data(&self) -> ::cpp_utils::Ptr<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_constData(
            self as *const crate::QVectorOfQPointF,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns a const reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPointF& QVector<QPointF>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_constFirst(
            self as *const crate::QVectorOfQPointF,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPointF& QVector<QPointF>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_constLast(
            self as *const crate::QVectorOfQPointF,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the vector contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QPointF>::contains(const QPointF& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the vector contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p></div>
    pub unsafe fn contains(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_contains(
            self as *const crate::QVectorOfQPointF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QPointF>& QVector<QPointF>::operator=(const QVector<QPointF>& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this vector and returns a reference to this vector.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        v: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQPointF>>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_operator_(
            self as *mut crate::QVectorOfQPointF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQPointF>>::cast_into(v)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of occurrences of <i>value</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QPointF>::count(const QPointF& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of occurrences of <i>value</i> in the vector.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_count(
            self as *const crate::QVectorOfQPointF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QPointF>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_count1(self as *const crate::QVectorOfQPointF)
    }

    /// <p>Returns a pointer to the data stored in the vector. The pointer can be used to access and modify the items in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPointF* QVector<QPointF>::data()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the data stored in the vector. The pointer can be used to access and modify the items in the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> vector(<span class="number">10</span>);
    ///   <span class="type">int</span> <span class="operator">*</span>data <span class="operator">=</span> vector<span class="operator">.</span>data();
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> <span class="number">10</span>; <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     data<span class="operator">[</span>i<span class="operator">]</span> <span class="operator">=</span> <span class="number">2</span> <span class="operator">*</span> i;
    ///
    /// </pre>
    /// <p>The pointer remains valid as long as the vector isn't reallocated.</p>
    /// <p>This function is mostly useful to pass a vector to a function that accepts a plain C++ array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constData">constData</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn data_mut(&mut self) -> ::cpp_utils::MutPtr<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_data(
            self as *mut crate::QVectorOfQPointF,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPointF* QVector<QPointF>::data() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#data-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn data(&self) -> ::cpp_utils::Ptr<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_data1(
            self as *const crate::QVectorOfQPointF,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```void QVector<QPointF>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_detach(self as *mut crate::QVectorOfQPointF)
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), returning <code>true</code> if the vector is empty; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QPointF>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), returning <code>true</code> if the vector is empty; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_empty(self as *const crate::QVectorOfQPointF)
    }

    /// <p>Returns <code>true</code> if this vector is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QPointF>::endsWith(const QPointF& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this vector is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn ends_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_endsWith(
            self as *const crate::QVectorOfQPointF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QPointF>& QVector<QPointF>::fill(const QPointF& t, int size = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">3</span>);
    ///   vector<span class="operator">.</span>fill(<span class="string">"Yes"</span>);
    ///   <span class="comment">// vector: ["Yes", "Yes", "Yes"]</span>
    ///
    ///   vector<span class="operator">.</span>fill(<span class="string">"oh"</span><span class="operator">,</span> <span class="number">5</span>);
    ///   <span class="comment">// vector: ["oh", "oh", "oh", "oh", "oh"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_2a(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_fill(
            self as *mut crate::QVectorOfQPointF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(t).as_raw_ptr(),
            size,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QPointF>& QVector<QPointF>::fill(const QPointF& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">3</span>);
    ///   vector<span class="operator">.</span>fill(<span class="string">"Yes"</span>);
    ///   <span class="comment">// vector: ["Yes", "Yes", "Yes"]</span>
    ///
    ///   vector<span class="operator">.</span>fill(<span class="string">"oh"</span><span class="operator">,</span> <span class="number">5</span>);
    ///   <span class="comment">// vector: ["oh", "oh", "oh", "oh", "oh"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_1a(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_fill1(
            self as *mut crate::QVectorOfQPointF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(t).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPointF& QVector<QPointF>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#constFirst">constFirst</a>().</p></div>
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_first(
            self as *mut crate::QVectorOfQPointF,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPointF& QVector<QPointF>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_first1(
            self as *const crate::QVectorOfQPointF,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPointF& QVector<QPointF>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::MutRef<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_front(
            self as *mut crate::QVectorOfQPointF,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPointF& QVector<QPointF>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_front1(
            self as *const crate::QVectorOfQPointF,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPointF& QVector<QPointF>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p>Note that using non-const operators can cause <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to do a deep copy.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_operator__2(
            self as *mut crate::QVectorOfQPointF,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPointF& QVector<QPointF>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as at(<i>i</i>).</p></div>
    pub unsafe fn index(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_operator__3(
            self as *const crate::QVectorOfQPointF,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QPointF>::indexOf(const QPointF& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span>);            <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);         <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);         <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"X"</span>);            <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_indexOf(
            self as *const crate::QVectorOfQPointF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(t).as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QPointF>::indexOf(const QPointF& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span>);            <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);         <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);         <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"X"</span>);            <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_indexOf1(
            self as *const crate::QVectorOfQPointF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the vector. If <i>i</i> is 0, the value is prepended to the vector. If <i>i</i> is <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), the value is appended to the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPointF>::insert(int i, const QPointF& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the vector. If <i>i</i> is 0, the value is prepended to the vector. If <i>i</i> is <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), the value is appended to the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   vector<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// vector: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p>For large vectors, this operation can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because it requires moving all the items at indexes <i>i</i> and above by one position further in memory. If you want a container class that provides a fast insert() function, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>().</p></div>
    pub unsafe fn insert_2a(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_insert(
            self as *mut crate::QVectorOfQPointF,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPointF>::insert(int i, int n, const QPointF& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>count</i> copies of <i>value</i> at index position <i>i</i> in the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">double</span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">2.718</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">1.442</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">0.4342</span>;
    ///   vector<span class="operator">.</span>insert(<span class="number">1</span><span class="operator">,</span> <span class="number">3</span><span class="operator">,</span> <span class="number">9.9</span>);
    ///   <span class="comment">// vector: [2.718, 9.9, 9.9, 9.9, 1.442, 0.4342]</span>
    ///
    /// </pre></div>
    pub unsafe fn insert_3a(
        &mut self,
        i: ::std::os::raw::c_int,
        n: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_insert1(
            self as *mut crate::QVectorOfQPointF,
            i,
            n,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QVector<QPointF>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_isDetached(
            self as *const crate::QVectorOfQPointF,
        )
    }

    /// <p>Returns <code>true</code> if the vector has size 0; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QPointF>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the vector has size 0; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_isEmpty(
            self as *const crate::QVectorOfQPointF,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QVector<QPointF>::isSharedWith(const QVector<QPointF>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQPointF>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_isSharedWith(
            self as *const crate::QVectorOfQPointF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQPointF>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPointF& QVector<QPointF>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#constLast">constLast</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_last(
            self as *mut crate::QVectorOfQPointF,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPointF& QVector<QPointF>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_last1(
            self as *const crate::QVectorOfQPointF,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QPointF>::lastIndexOf(const QPointF& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);        <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);     <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);     <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_lastIndexOf(
            self as *const crate::QVectorOfQPointF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(t).as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QPointF>::lastIndexOf(const QPointF& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);        <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);     <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);     <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_lastIndexOf1(
            self as *const crate::QVectorOfQPointF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QPointF>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#length">QList::length</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_length(self as *const crate::QVectorOfQPointF)
    }

    /// <p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QPointF> QVector<QPointF>::mid(int pos, int len = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        len: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_mid(
            self as *const crate::QVectorOfQPointF,
            pos,
            len,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QPointF> QVector<QPointF>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_mid1(
            self as *const crate::QVectorOfQPointF,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPointF>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">QList::move</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_move(
            self as *mut crate::QVectorOfQPointF,
            from,
            to,
        )
    }

    /// <p>Constructs an empty vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QPointF>::QVector()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QVectorOfQPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_QVector();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a vector with an initial size of <i>size</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QPointF>::QVector(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a vector with an initial size of <i>size</i> elements.</p>
    /// <p>The elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn new_1a(
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_QVector1(size);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a vector with an initial size of <i>size</i> elements. Each element is initialized with <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QPointF>::QVector(int size, const QPointF& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a vector with an initial size of <i>size</i> elements. Each element is initialized with <i>value</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn new_2a(
        size: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_QVector2(
            size,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(t).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QPointF>::QVector(const QVector<QPointF>& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        v: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQPointF>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_QVector3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQPointF>>::cast_into(v)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPointF>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_pop_back(self as *mut crate::QVectorOfQPointF)
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPointF>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_pop_front(
            self as *mut crate::QVectorOfQPointF,
        )
    }

    /// <p>Inserts <i>value</i> at the beginning of the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPointF>::prepend(const QPointF& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   vector<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   vector<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// vector: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as vector.insert(0, <i>value</i>).</p>
    /// <p>For large vectors, this operation can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because it requires moving all the items in the vector by one position further in memory. If you want a container class that provides a fast prepend() function, use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> or <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_prepend(
            self as *mut crate::QVectorOfQPointF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to append(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPointF>::push_back(const QPointF& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to append(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_push_back(
            self as *mut crate::QVectorOfQPointF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to prepend(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPointF>::push_front(const QPointF& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to prepend(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_push_front(
            self as *mut crate::QVectorOfQPointF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPointF>::remove(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#remove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes the element at index position <i>i</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn remove_1a(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_remove(
            self as *mut crate::QVectorOfQPointF,
            i,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPointF>::remove(int i, int n)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#remove-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes <i>count</i> elements from the middle of the vector, starting at index position <i>i</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn remove_2a(&mut self, i: ::std::os::raw::c_int, n: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_remove1(
            self as *mut crate::QVectorOfQPointF,
            i,
            n,
        )
    }

    /// <p>Removes all elements that compare equal to <i>t</i> from the vector. Returns the number of elements removed, if any.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QPointF>::removeAll(const QPointF& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all elements that compare equal to <i>t</i> from the vector. Returns the number of elements removed, if any.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#removeOne">removeOne</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">QList::removeAll</a>().</p></div>
    pub unsafe fn remove_all(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_removeAll(
            self as *mut crate::QVectorOfQPointF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes the element at index position <i>i</i>. Equivalent to</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPointF>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the element at index position <i>i</i>. Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   remove(i);
    ///
    /// </pre>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">QList::removeAt</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_removeAt(
            self as *mut crate::QVectorOfQPointF,
            i,
        )
    }

    /// <p>Removes the first item in the vector. Calling this function is equivalent to calling remove(0). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPointF>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the vector. Calling this function is equivalent to calling remove(0). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_removeFirst(
            self as *mut crate::QVectorOfQPointF,
        )
    }

    /// <p>Removes the last item in the vector. Calling this function is equivalent to calling remove(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() - 1). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPointF>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the vector. Calling this function is equivalent to calling remove(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() - 1). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_removeLast(
            self as *mut crate::QVectorOfQPointF,
        )
    }

    /// <p>Removes the first element that compares equal to <i>t</i> from the vector. Returns whether an element was, in fact, removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QPointF>::removeOne(const QPointF& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeOne">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first element that compares equal to <i>t</i> from the vector. Returns whether an element was, in fact, removed.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#removeAll">removeAll</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">QList::removeOne</a>().</p></div>
    pub unsafe fn remove_one(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_removeOne(
            self as *mut crate::QVectorOfQPointF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPointF>::replace(int i, const QPointF& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_replace(
            self as *mut crate::QVectorOfQPointF,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Attempts to allocate memory for at least <i>size</i> elements. If you know in advance how large the vector will be, you should call this function to prevent reallocations and memory fragmentation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPointF>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to allocate memory for at least <i>size</i> elements. If you know in advance how large the vector will be, you should call this function to prevent reallocations and memory fragmentation.</p>
    /// <p>If <i>size</i> is an underestimate, the worst that will happen is that the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> will be a bit slower. If <i>size</i> is an overestimate, you may have used more memory than the normal <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> growth strategy would have allocated—or you may have used less.</p>
    /// <p>An alternative to reserve() is calling <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>(). Whether or not that is faster than reserve() depends on the element type, because <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() default-constructs all elements, and requires assignment to existing entries rather than calling <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(), which copy- or move-constructs. For simple types, like <code>int</code> or <code>double</code>, <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() is typically faster, but for anything more complex, you should prefer reserve().</p>
    /// <p><b>Warning:</b> If the size passed to <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() was underestimated, you run out of allocated space and into undefined behavior. This problem does not exist with reserve(), because it treats the size as just a hint.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>().</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_reserve(
            self as *mut crate::QVectorOfQPointF,
            size,
        )
    }

    /// <p>Sets the size of the vector to <i>size</i>. If <i>size</i> is greater than the current size, elements are added to the end; the new elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If <i>size</i> is less than the current size, elements are removed from the end.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPointF>::resize(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#resize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the size of the vector to <i>size</i>. If <i>size</i> is greater than the current size, elements are added to the end; the new elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If <i>size</i> is less than the current size, elements are removed from the end.</p>
    /// <p>Since Qt 5.6, resize() doesn't shrink the capacity anymore. To shed excess capacity, use <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p></div>
    pub unsafe fn resize(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_resize(
            self as *mut crate::QVectorOfQPointF,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QVector<QPointF>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_setSharable(
            self as *mut crate::QVectorOfQPointF,
            sharable,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPointF>::shrink_to_fit()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#shrink_to_fit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn shrink_to_fit(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_shrink_to_fit(
            self as *mut crate::QVectorOfQPointF,
        )
    }

    /// <p>Returns the number of items in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QPointF>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_size(self as *const crate::QVectorOfQPointF)
    }

    /// <p>Releases any memory not required to store the items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPointF>::squeeze()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Releases any memory not required to store the items.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s memory usage. In general, you will rarely ever need to call this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>().</p></div>
    pub unsafe fn squeeze(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_squeeze(self as *mut crate::QVectorOfQPointF)
    }

    /// <p>Returns <code>true</code> if this vector is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QPointF>::startsWith(const QPointF& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this vector is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn starts_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_startsWith(
            self as *const crate::QVectorOfQPointF,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Swaps vector <i>other</i> with this vector. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPointF>::swap(QVector<QPointF>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps vector <i>other</i> with this vector. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QVectorOfQPointF>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_swap(
            self as *mut crate::QVectorOfQPointF,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QVectorOfQPointF>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Removes the element at index position <i>i</i> and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPointF QVector<QPointF>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the element at index position <i>i</i> and returns it.</p>
    /// <p>Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   T t <span class="operator">=</span> at(i);
    ///   remove(i);
    ///   <span class="keyword">return</span> t;
    ///
    /// </pre>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">QList::takeAt</a>().</p></div>
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_takeAt(
            self as *mut crate::QVectorOfQPointF,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the first item in the vector and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPointF QVector<QPointF>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the vector and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> ::cpp_utils::CppBox<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_takeFirst(
            self as *mut crate::QVectorOfQPointF,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the last item in the list and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPointF QVector<QPointF>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> ::cpp_utils::CppBox<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_takeLast(
            self as *mut crate::QVectorOfQPointF,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value at index position <i>i</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPointF QVector<QPointF>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the vector.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that <i>i</i> is within bounds, you can use <a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_value(
            self as *const crate::QVectorOfQPointF,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPointF QVector<QPointF>::value(int i, const QPointF& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPointF>>,
    ) -> ::cpp_utils::CppBox<crate::QPointF> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_value1(
            self as *const crate::QVectorOfQPointF,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPointF>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_future_of {
    //! C++ type: <span style='color: green;'>```QFuture<void>```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html">QFuture::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QFuture<void>::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html">QFuture::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html">QFuture::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> function like <a href="http://doc.qt.io/qt-5/qfuture.html#constBegin">QFuture::constBegin</a>() or <a href="http://doc.qt.io/qt-5/qfuture.html#constEnd">QFuture::constEnd</a>() before you start iterating. Here's a typical loop that prints all the results available in a future:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qfuture.html#QFuture">QFuture</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> future <span class="operator">=</span> <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qfuture.html#QFuture">QFuture</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> future<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> future<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
}
/// <p>The <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> class represents the result of an asynchronous computation.</p>
///
/// C++ class: <span style='color: green;'>```QFuture<void>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qfuture.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> class represents the result of an asynchronous computation.</p>
/// <p>To start a computation, use one of the APIs in the <a href="http://doc.qt.io/qt-5/qtconcurrent-index.html">Qt Concurrent</a> framework.</p>
/// <p><a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> allows threads to be synchronized against one or more results which will be ready at a later point in time. The result can be of any type that has a default constructor and a copy constructor. If a result is not available at the time of calling the <a href="http://doc.qt.io/qt-5/qfuture.html#result">result</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#resultAt">resultAt</a>(), or <a href="http://doc.qt.io/qt-5/qfuture.html#results">results</a>() functions, <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> will wait until the result becomes available. You can use the <a href="http://doc.qt.io/qt-5/qfuture.html#isResultReadyAt">isResultReadyAt</a>() function to determine if a result is ready or not. For <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> objects that report more than one result, the <a href="http://doc.qt.io/qt-5/qfuture.html#resultCount">resultCount</a>() function returns the number of continuous results. This means that it is always safe to iterate through the results from 0 to <a href="http://doc.qt.io/qt-5/qfuture.html#resultCount">resultCount</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> provides a <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterator</a> (<a href="http://doc.qt.io/qt-5/qfutureiterator.html">QFutureIterator</a>) and an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> (<a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html">QFuture::const_iterator</a>). Using these iterators is another way to access results in the future.</p>
/// <p><a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> also offers ways to interact with a runnning computation. For instance, the computation can be canceled with the <a href="http://doc.qt.io/qt-5/qfuture.html#cancel">cancel</a>() function. To pause the computation, use the <a href="http://doc.qt.io/qt-5/qfuture.html#setPaused">setPaused</a>() function or one of the <a href="http://doc.qt.io/qt-5/qfuture.html#pause">pause</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#resume">resume</a>(), or <a href="http://doc.qt.io/qt-5/qfuture.html#togglePaused">togglePaused</a>() convenience functions. Be aware that not all asynchronous computations can be canceled or paused. For example, the future returned by <a href="http://doc.qt.io/qt-5/qtconcurrent.html#run">QtConcurrent::run</a>() cannot be canceled; but the future returned by <a href="http://doc.qt.io/qt-5/qtconcurrent.html#mappedReduced-1">QtConcurrent::mappedReduced</a>() can.</p>
/// <p>Progress information is provided by the <a href="http://doc.qt.io/qt-5/qfuture.html#progressValue">progressValue</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#progressMinimum">progressMinimum</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#progressMaximum">progressMaximum</a>(), and <a href="http://doc.qt.io/qt-5/qfuture.html#progressText">progressText</a>() functions. The <a href="http://doc.qt.io/qt-5/qfuture.html#waitForFinished">waitForFinished</a>() function causes the calling thread to block and wait for the computation to finish, ensuring that all results are available.</p>
/// <p>The state of the computation represented by a <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> can be queried using the <a href="http://doc.qt.io/qt-5/qfuture.html#isCanceled">isCanceled</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#isStarted">isStarted</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#isFinished">isFinished</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#isRunning">isRunning</a>(), or <a href="http://doc.qt.io/qt-5/qfuture.html#isPaused">isPaused</a>() functions.</p>
/// <p><a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> is a lightweight reference counted class that can be passed by value.</p>
/// <p><a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a>&lt;void&gt; is specialized to not contain any of the result fetching functions. Any <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a>&lt;T&gt; can be assigned or copied into a <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a>&lt;void&gt; as well. This is useful if only status or progress information is needed - not the actual result data.</p>
/// <p>To interact with running tasks using signals and slots, use <a href="http://doc.qt.io/qt-5/qfuturewatcher.html">QFutureWatcher</a>.</p></div>
#[repr(C)]
pub struct QFutureOf {
    _unused: u8,
}
impl QFutureOf {
    /// <p>Cancels the asynchronous computation represented by this future. Note that the cancelation is asynchronous. Use <a href="http://doc.qt.io/qt-5/qfuture.html#waitForFinished">waitForFinished</a>() after calling cancel() when you need synchronous cancelation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QFuture<void>::cancel()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfuture.html#cancel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Cancels the asynchronous computation represented by this future. Note that the cancelation is asynchronous. Use <a href="http://doc.qt.io/qt-5/qfuture.html#waitForFinished">waitForFinished</a>() after calling cancel() when you need synchronous cancelation.</p>
    /// <p>Results currently available may still be accessed on a canceled future, but new results will <i>not</i> become available after calling this function. Any <a href="http://doc.qt.io/qt-5/qfuturewatcher.html">QFutureWatcher</a> object that is watching this future will not deliver progress and result ready signals on a canceled future.</p>
    /// <p>Be aware that not all asynchronous computations can be canceled. For example, the future returned by <a href="http://doc.qt.io/qt-5/qtconcurrent.html#run">QtConcurrent::run</a>() cannot be canceled; but the future returned by <a href="http://doc.qt.io/qt-5/qtconcurrent.html#mappedReduced-1">QtConcurrent::mappedReduced</a>() can.</p></div>
    pub unsafe fn cancel(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QFuture_void_cancel(self as *mut crate::QFutureOf)
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> class represents the result of an asynchronous computation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QFuture<void>& QFuture<void>::operator=(const QFuture<void>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfuture.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> class represents the result of an asynchronous computation.</p>
    /// <p>To start a computation, use one of the APIs in the <a href="http://doc.qt.io/qt-5/qtconcurrent-index.html">Qt Concurrent</a> framework.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> allows threads to be synchronized against one or more results which will be ready at a later point in time. The result can be of any type that has a default constructor and a copy constructor. If a result is not available at the time of calling the <a href="http://doc.qt.io/qt-5/qfuture.html#result">result</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#resultAt">resultAt</a>(), or <a href="http://doc.qt.io/qt-5/qfuture.html#results">results</a>() functions, <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> will wait until the result becomes available. You can use the <a href="http://doc.qt.io/qt-5/qfuture.html#isResultReadyAt">isResultReadyAt</a>() function to determine if a result is ready or not. For <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> objects that report more than one result, the <a href="http://doc.qt.io/qt-5/qfuture.html#resultCount">resultCount</a>() function returns the number of continuous results. This means that it is always safe to iterate through the results from 0 to <a href="http://doc.qt.io/qt-5/qfuture.html#resultCount">resultCount</a>().</p>
    /// <p><a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> provides a <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterator</a> (<a href="http://doc.qt.io/qt-5/qfutureiterator.html">QFutureIterator</a>) and an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> (<a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html">QFuture::const_iterator</a>). Using these iterators is another way to access results in the future.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> also offers ways to interact with a runnning computation. For instance, the computation can be canceled with the <a href="http://doc.qt.io/qt-5/qfuture.html#cancel">cancel</a>() function. To pause the computation, use the <a href="http://doc.qt.io/qt-5/qfuture.html#setPaused">setPaused</a>() function or one of the <a href="http://doc.qt.io/qt-5/qfuture.html#pause">pause</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#resume">resume</a>(), or <a href="http://doc.qt.io/qt-5/qfuture.html#togglePaused">togglePaused</a>() convenience functions. Be aware that not all asynchronous computations can be canceled or paused. For example, the future returned by <a href="http://doc.qt.io/qt-5/qtconcurrent.html#run">QtConcurrent::run</a>() cannot be canceled; but the future returned by <a href="http://doc.qt.io/qt-5/qtconcurrent.html#mappedReduced-1">QtConcurrent::mappedReduced</a>() can.</p>
    /// <p>Progress information is provided by the <a href="http://doc.qt.io/qt-5/qfuture.html#progressValue">progressValue</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#progressMinimum">progressMinimum</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#progressMaximum">progressMaximum</a>(), and <a href="http://doc.qt.io/qt-5/qfuture.html#progressText">progressText</a>() functions. The <a href="http://doc.qt.io/qt-5/qfuture.html#waitForFinished">waitForFinished</a>() function causes the calling thread to block and wait for the computation to finish, ensuring that all results are available.</p>
    /// <p>The state of the computation represented by a <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> can be queried using the <a href="http://doc.qt.io/qt-5/qfuture.html#isCanceled">isCanceled</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#isStarted">isStarted</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#isFinished">isFinished</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#isRunning">isRunning</a>(), or <a href="http://doc.qt.io/qt-5/qfuture.html#isPaused">isPaused</a>() functions.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> is a lightweight reference counted class that can be passed by value.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a>&lt;void&gt; is specialized to not contain any of the result fetching functions. Any <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a>&lt;T&gt; can be assigned or copied into a <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a>&lt;void&gt; as well. This is useful if only status or progress information is needed - not the actual result data.</p>
    /// <p>To interact with running tasks using signals and slots, use <a href="http://doc.qt.io/qt-5/qfuturewatcher.html">QFutureWatcher</a>.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QFutureOf>>,
    ) -> ::cpp_utils::MutRef<crate::QFutureOf> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFuture_void_operator_(
            self as *mut crate::QFutureOf,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QFutureOf>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the asynchronous computation has been canceled with the <a href="http://doc.qt.io/qt-5/qfuture.html#cancel">cancel</a>() function; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFuture<void>::isCanceled() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfuture.html#isCanceled">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the asynchronous computation has been canceled with the <a href="http://doc.qt.io/qt-5/qfuture.html#cancel">cancel</a>() function; otherwise returns <code>false</code>.</p>
    /// <p>Be aware that the computation may still be running even though this function returns <code>true</code>. See <a href="http://doc.qt.io/qt-5/qfuture.html#cancel">cancel</a>() for more details.</p></div>
    pub unsafe fn is_canceled(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFuture_void_isCanceled(self as *const crate::QFutureOf)
    }

    /// <p>Returns <code>true</code> if the asynchronous computation represented by this future has finished; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFuture<void>::isFinished() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfuture.html#isFinished">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the asynchronous computation represented by this future has finished; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn is_finished(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFuture_void_isFinished(self as *const crate::QFutureOf)
    }

    /// <p>Returns <code>true</code> if the asynchronous computation has been paused with the <a href="http://doc.qt.io/qt-5/qfuture.html#pause">pause</a>() function; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFuture<void>::isPaused() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfuture.html#isPaused">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the asynchronous computation has been paused with the <a href="http://doc.qt.io/qt-5/qfuture.html#pause">pause</a>() function; otherwise returns <code>false</code>.</p>
    /// <p>Be aware that the computation may still be running even though this function returns <code>true</code>. See <a href="http://doc.qt.io/qt-5/qfuture.html#setPaused">setPaused</a>() for more details.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfuture.html#setPaused">setPaused</a>() and <a href="http://doc.qt.io/qt-5/qfuture.html#togglePaused">togglePaused</a>().</p></div>
    pub unsafe fn is_paused(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFuture_void_isPaused(self as *const crate::QFutureOf)
    }

    /// <p>Returns <code>true</code> if the asynchronous computation represented by this future is currently running; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFuture<void>::isRunning() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfuture.html#isRunning">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the asynchronous computation represented by this future is currently running; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn is_running(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFuture_void_isRunning(self as *const crate::QFutureOf)
    }

    /// <p>Returns <code>true</code> if the asynchronous computation represented by this future has been started; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFuture<void>::isStarted() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfuture.html#isStarted">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the asynchronous computation represented by this future has been started; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn is_started(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QFuture_void_isStarted(self as *const crate::QFutureOf)
    }

    /// <p>Constructs an empty, canceled future.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QFuture<void>::QFuture()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfuture.html#QFuture">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty, canceled future.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QFutureOf> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFuture_void_QFuture();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> class represents the result of an asynchronous computation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QFuture<void>::QFuture(const QFuture<void>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfuture.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> class represents the result of an asynchronous computation.</p>
    /// <p>To start a computation, use one of the APIs in the <a href="http://doc.qt.io/qt-5/qtconcurrent-index.html">Qt Concurrent</a> framework.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> allows threads to be synchronized against one or more results which will be ready at a later point in time. The result can be of any type that has a default constructor and a copy constructor. If a result is not available at the time of calling the <a href="http://doc.qt.io/qt-5/qfuture.html#result">result</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#resultAt">resultAt</a>(), or <a href="http://doc.qt.io/qt-5/qfuture.html#results">results</a>() functions, <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> will wait until the result becomes available. You can use the <a href="http://doc.qt.io/qt-5/qfuture.html#isResultReadyAt">isResultReadyAt</a>() function to determine if a result is ready or not. For <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> objects that report more than one result, the <a href="http://doc.qt.io/qt-5/qfuture.html#resultCount">resultCount</a>() function returns the number of continuous results. This means that it is always safe to iterate through the results from 0 to <a href="http://doc.qt.io/qt-5/qfuture.html#resultCount">resultCount</a>().</p>
    /// <p><a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> provides a <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterator</a> (<a href="http://doc.qt.io/qt-5/qfutureiterator.html">QFutureIterator</a>) and an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> (<a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html">QFuture::const_iterator</a>). Using these iterators is another way to access results in the future.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> also offers ways to interact with a runnning computation. For instance, the computation can be canceled with the <a href="http://doc.qt.io/qt-5/qfuture.html#cancel">cancel</a>() function. To pause the computation, use the <a href="http://doc.qt.io/qt-5/qfuture.html#setPaused">setPaused</a>() function or one of the <a href="http://doc.qt.io/qt-5/qfuture.html#pause">pause</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#resume">resume</a>(), or <a href="http://doc.qt.io/qt-5/qfuture.html#togglePaused">togglePaused</a>() convenience functions. Be aware that not all asynchronous computations can be canceled or paused. For example, the future returned by <a href="http://doc.qt.io/qt-5/qtconcurrent.html#run">QtConcurrent::run</a>() cannot be canceled; but the future returned by <a href="http://doc.qt.io/qt-5/qtconcurrent.html#mappedReduced-1">QtConcurrent::mappedReduced</a>() can.</p>
    /// <p>Progress information is provided by the <a href="http://doc.qt.io/qt-5/qfuture.html#progressValue">progressValue</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#progressMinimum">progressMinimum</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#progressMaximum">progressMaximum</a>(), and <a href="http://doc.qt.io/qt-5/qfuture.html#progressText">progressText</a>() functions. The <a href="http://doc.qt.io/qt-5/qfuture.html#waitForFinished">waitForFinished</a>() function causes the calling thread to block and wait for the computation to finish, ensuring that all results are available.</p>
    /// <p>The state of the computation represented by a <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> can be queried using the <a href="http://doc.qt.io/qt-5/qfuture.html#isCanceled">isCanceled</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#isStarted">isStarted</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#isFinished">isFinished</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#isRunning">isRunning</a>(), or <a href="http://doc.qt.io/qt-5/qfuture.html#isPaused">isPaused</a>() functions.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> is a lightweight reference counted class that can be passed by value.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a>&lt;void&gt; is specialized to not contain any of the result fetching functions. Any <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a>&lt;T&gt; can be assigned or copied into a <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a>&lt;void&gt; as well. This is useful if only status or progress information is needed - not the actual result data.</p>
    /// <p>To interact with running tasks using signals and slots, use <a href="http://doc.qt.io/qt-5/qfuturewatcher.html">QFutureWatcher</a>.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QFutureOf>>,
    ) -> ::cpp_utils::CppBox<crate::QFutureOf> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFuture_void_QFuture1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QFutureOf>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Pauses the asynchronous computation represented by this future. This is a convenience method that simply calls <a href="http://doc.qt.io/qt-5/qfuture.html#setPaused">setPaused</a>(true).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QFuture<void>::pause()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfuture.html#pause">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Pauses the asynchronous computation represented by this future. This is a convenience method that simply calls <a href="http://doc.qt.io/qt-5/qfuture.html#setPaused">setPaused</a>(true).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfuture.html#resume">resume</a>().</p></div>
    pub unsafe fn pause(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QFuture_void_pause(self as *mut crate::QFutureOf)
    }

    /// <p>Returns the maximum <a href="http://doc.qt.io/qt-5/qfuture.html#progressValue">progressValue</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QFuture<void>::progressMaximum() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfuture.html#progressMaximum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the maximum <a href="http://doc.qt.io/qt-5/qfuture.html#progressValue">progressValue</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfuture.html#progressValue">progressValue</a>() and <a href="http://doc.qt.io/qt-5/qfuture.html#progressMinimum">progressMinimum</a>().</p></div>
    pub unsafe fn progress_maximum(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QFuture_void_progressMaximum(self as *const crate::QFutureOf)
    }

    /// <p>Returns the minimum <a href="http://doc.qt.io/qt-5/qfuture.html#progressValue">progressValue</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QFuture<void>::progressMinimum() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfuture.html#progressMinimum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the minimum <a href="http://doc.qt.io/qt-5/qfuture.html#progressValue">progressValue</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfuture.html#progressValue">progressValue</a>() and <a href="http://doc.qt.io/qt-5/qfuture.html#progressMaximum">progressMaximum</a>().</p></div>
    pub unsafe fn progress_minimum(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QFuture_void_progressMinimum(self as *const crate::QFutureOf)
    }

    /// <p>Returns the (optional) textual representation of the progress as reported by the asynchronous computation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QFuture<void>::progressText() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfuture.html#progressText">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the (optional) textual representation of the progress as reported by the asynchronous computation.</p>
    /// <p>Be aware that not all computations provide a textual representation of the progress, and as such, this function may return an empty string.</p></div>
    pub unsafe fn progress_text(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QFuture_void_progressText(
            self as *const crate::QFutureOf,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the current progress value, which is between the <a href="http://doc.qt.io/qt-5/qfuture.html#progressMinimum">progressMinimum</a>() and <a href="http://doc.qt.io/qt-5/qfuture.html#progressMaximum">progressMaximum</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QFuture<void>::progressValue() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfuture.html#progressValue">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current progress value, which is between the <a href="http://doc.qt.io/qt-5/qfuture.html#progressMinimum">progressMinimum</a>() and <a href="http://doc.qt.io/qt-5/qfuture.html#progressMaximum">progressMaximum</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfuture.html#progressMinimum">progressMinimum</a>() and <a href="http://doc.qt.io/qt-5/qfuture.html#progressMaximum">progressMaximum</a>().</p></div>
    pub unsafe fn progress_value(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QFuture_void_progressValue(self as *const crate::QFutureOf)
    }

    /// <p>Returns the number of continuous results available in this future. The real number of results stored might be different from this value, due to gaps in the result set. It is always safe to iterate through the results from 0 to resultCount().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QFuture<void>::resultCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfuture.html#resultCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of continuous results available in this future. The real number of results stored might be different from this value, due to gaps in the result set. It is always safe to iterate through the results from 0 to resultCount().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfuture.html#result">result</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#resultAt">resultAt</a>(), and <a href="http://doc.qt.io/qt-5/qfuture.html#results">results</a>().</p></div>
    pub unsafe fn result_count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QFuture_void_resultCount(self as *const crate::QFutureOf)
    }

    /// <p>Resumes the asynchronous computation represented by this future. This is a convenience method that simply calls <a href="http://doc.qt.io/qt-5/qfuture.html#setPaused">setPaused</a>(false).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QFuture<void>::resume()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfuture.html#resume">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Resumes the asynchronous computation represented by this future. This is a convenience method that simply calls <a href="http://doc.qt.io/qt-5/qfuture.html#setPaused">setPaused</a>(false).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfuture.html#pause">pause</a>().</p></div>
    pub unsafe fn resume(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QFuture_void_resume(self as *mut crate::QFutureOf)
    }

    /// <p>If <i>paused</i> is true, this function pauses the asynchronous computation represented by the future. If the computation is already paused, this function does nothing. Any <a href="http://doc.qt.io/qt-5/qfuturewatcher.html">QFutureWatcher</a> object that is watching this future will stop delivering progress and result ready signals while the future is paused. Signal delivery will continue once the future is resumed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QFuture<void>::setPaused(bool paused)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfuture.html#setPaused">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If <i>paused</i> is true, this function pauses the asynchronous computation represented by the future. If the computation is already paused, this function does nothing. Any <a href="http://doc.qt.io/qt-5/qfuturewatcher.html">QFutureWatcher</a> object that is watching this future will stop delivering progress and result ready signals while the future is paused. Signal delivery will continue once the future is resumed.</p>
    /// <p>If <i>paused</i> is false, this function resumes the asynchronous computation. If the computation was not previously paused, this function does nothing.</p>
    /// <p>Be aware that not all computations can be paused. For example, the future returned by <a href="http://doc.qt.io/qt-5/qtconcurrent.html#run">QtConcurrent::run</a>() cannot be paused; but the future returned by <a href="http://doc.qt.io/qt-5/qtconcurrent.html#mappedReduced-1">QtConcurrent::mappedReduced</a>() can.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfuture.html#isPaused">isPaused</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#pause">pause</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#resume">resume</a>(), and <a href="http://doc.qt.io/qt-5/qfuture.html#togglePaused">togglePaused</a>().</p></div>
    pub unsafe fn set_paused(&mut self, paused: bool) {
        crate::__ffi::ctr_qt_core_ffi_QFuture_void_setPaused(self as *mut crate::QFutureOf, paused)
    }

    /// <p>Toggles the paused state of the asynchronous computation. In other words, if the computation is currently paused, calling this function resumes it; if the computation is running, it is paused. This is a convenience method for calling <a href="http://doc.qt.io/qt-5/qfuture.html#setPaused">setPaused</a>(!<a href="http://doc.qt.io/qt-5/qfuture.html#isPaused">isPaused</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QFuture<void>::togglePaused()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfuture.html#togglePaused">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Toggles the paused state of the asynchronous computation. In other words, if the computation is currently paused, calling this function resumes it; if the computation is running, it is paused. This is a convenience method for calling <a href="http://doc.qt.io/qt-5/qfuture.html#setPaused">setPaused</a>(!<a href="http://doc.qt.io/qt-5/qfuture.html#isPaused">isPaused</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfuture.html#setPaused">setPaused</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#pause">pause</a>(), and <a href="http://doc.qt.io/qt-5/qfuture.html#resume">resume</a>().</p></div>
    pub unsafe fn toggle_paused(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QFuture_void_togglePaused(self as *mut crate::QFutureOf)
    }

    /// <p>Waits for the asynchronous computation to finish (including <a href="http://doc.qt.io/qt-5/qfuture.html#cancel">cancel</a>()ed computations).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QFuture<void>::waitForFinished()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfuture.html#waitForFinished">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Waits for the asynchronous computation to finish (including <a href="http://doc.qt.io/qt-5/qfuture.html#cancel">cancel</a>()ed computations).</p></div>
    pub unsafe fn wait_for_finished(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QFuture_void_waitForFinished(self as *mut crate::QFutureOf)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> class is a template class that provides a dynamic array.</p>
///
/// C++ class: <span style='color: green;'>```QVector<void>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qvector.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> class is a template class that provides a dynamic array.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores its items in adjacent memory locations and provides fast index-based access.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Here's an example of a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> that stores <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerVector;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> stringVector;
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> stores its items in a vector (array). Typically, vectors are created with an initial size. For example, the following code constructs a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> with 200 elements:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">200</span>);
///
/// </pre>
/// <p>The elements are automatically initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you want to initialize the vector with a different value, pass that value as the second argument to the constructor:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">200</span><span class="operator">,</span> <span class="string">"Pass"</span>);
///
/// </pre>
/// <p>You can also call <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>() at any time to fill the vector with a value.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const vectors, operator[]() returns a reference to the item that can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (vector<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Liz"</span>)
/// &#32;     vector<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Elizabeth"</span>;
///
/// </pre>
/// <p>For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> vector<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (vector<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Alfonso"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Alfonso at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>Another way to access the data stored in a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> is to call <a href="http://doc.qt.io/qt-5/qvector.html#data">data</a>(). The function returns a pointer to the first item in the vector. You can use the pointer to directly access and modify the elements stored in the vector. The pointer is also useful if you need to pass a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to a function that accepts a plain C++ array.</p>
/// <p>If you want to find all occurrences of a particular value in a vector, use <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of the matching item if they found one; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> vector<span class="operator">.</span>indexOf(<span class="string">"Harumi"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Harumi is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a vector contains a particular value, use <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the vector, use <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(). With the exception of <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), these functions can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>) for large vectors, because they require moving many items in the vector by one position in memory. If you want a container class that provides fast insertion/removal in the middle, use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> or <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
/// <p>Unlike plain C++ arrays, QVectors can be resized at any time by calling <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>(). If the new size is larger than the old size, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> might need to reallocate the whole vector. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> tries to reduce the number of reallocations by preallocating up to twice as much memory as the actual data needs.</p>
/// <p>If you know in advance approximately how many items the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> will contain, you can call <a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>(), asking <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to preallocate a certain amount of memory. You can also call <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>() to find out how much memory <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> actually allocated.</p>
/// <p>Note that using non-const operators and functions can cause <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to do a deep copy of the data. This is due to <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicit sharing</a>.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qvectoriterator.html">QVectorIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablevectoriterator.html">QMutableVectorIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qvector.html#const_iterator-typedef">QVector::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qvector.html#iterator-typedefx">QVector::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>.</p>
/// <p>In addition to <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>, Qt also provides <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a>, a very low-level class with little functionality that is optimized for speed.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <a name="more-information-on-using-qt-containers"></a>
/// <h4>More Information on Using Qt Containers</h4>
/// <p>For a detailed discussion comparing Qt containers with each other and with STL containers, see <a href="http://marcmutz.wordpress.com/effective-qt/containers/">Understand the Qt Containers</a>.</p></div>
#[repr(C)]
pub struct QVectorOf {
    _unused: u8,
}
/// <p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
///
/// C++ class: <span style='color: green;'>```QPair<QString, QJsonValue>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qpair.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
/// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; can be used in your application if the STL <code>pair</code> type is not available. It stores one value of type T1 and one value of type T2. It can be used as a return value for a function that needs to return two values, or as the value type of a <a href="http://doc.qt.io/qt-5/containers.html">generic container</a>.</p>
/// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> that stores one <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and one <code>double</code> value:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qpair.html#QPair">QPair</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> pair;
///
/// </pre>
/// <p>The components are accessible as public data members called <a href="http://doc.qt.io/qt-5/qpair.html#first-var">first</a> and <a href="http://doc.qt.io/qt-5/qpair.html#second-var">second</a>. For example:</p>
/// <pre class="cpp">
///
///   pair<span class="operator">.</span>first <span class="operator">=</span> <span class="string">"pi"</span>;
///   pair<span class="operator">.</span>second <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
///
/// </pre>
/// <p>Note, however, that it is almost always preferable to define a small struct to hold the result of a function with multiple return values. A struct trivially generalizes to more than two values, and allows more descriptive member names than <code>first</code> and <code>second</code>:</p>
/// <pre class="cpp">
///
///   <span class="keyword">struct</span> Variable {
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> name;
/// &#32;     <span class="type">double</span> value;
///   };
///   Variable v;
///   v<span class="operator">.</span>name <span class="operator">=</span> <span class="string">"pi"</span>;
///   v<span class="operator">.</span>value <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
///
/// </pre>
/// <p>The advent of C++11 automatic variable type deduction (<code>auto</code>) shifts the emphasis from the type name to the name of functions and members. Thus, <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>, like <code>std::pair</code> and <code>std::tuple</code>, is mostly useful in generic (template) code, where defining a dedicated type is not possible.</p>
/// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>'s template data types (T1 and T2) must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; these requirements are documented on a per-function basis.</p></div>
#[repr(C)]
pub struct QPairOfQStringQJsonValue {
    _unused: u8,
}
impl QPairOfQStringQJsonValue {
    /// <p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPair<QString, QJsonValue>& QPair<QString, QJsonValue>::operator=(const QPair<QString, QJsonValue>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; can be used in your application if the STL <code>pair</code> type is not available. It stores one value of type T1 and one value of type T2. It can be used as a return value for a function that needs to return two values, or as the value type of a <a href="http://doc.qt.io/qt-5/containers.html">generic container</a>.</p>
    /// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> that stores one <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and one <code>double</code> value:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpair.html#QPair">QPair</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> pair;
    ///
    /// </pre>
    /// <p>The components are accessible as public data members called <a href="http://doc.qt.io/qt-5/qpair.html#first-var">first</a> and <a href="http://doc.qt.io/qt-5/qpair.html#second-var">second</a>. For example:</p>
    /// <pre class="cpp">
    ///
    ///   pair<span class="operator">.</span>first <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   pair<span class="operator">.</span>second <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>Note, however, that it is almost always preferable to define a small struct to hold the result of a function with multiple return values. A struct trivially generalizes to more than two values, and allows more descriptive member names than <code>first</code> and <code>second</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">struct</span> Variable {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> name;
    /// &#32;     <span class="type">double</span> value;
    ///   };
    ///   Variable v;
    ///   v<span class="operator">.</span>name <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   v<span class="operator">.</span>value <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>The advent of C++11 automatic variable type deduction (<code>auto</code>) shifts the emphasis from the type name to the name of functions and members. Thus, <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>, like <code>std::pair</code> and <code>std::tuple</code>, is mostly useful in generic (template) code, where defining a dedicated type is not possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>'s template data types (T1 and T2) must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; these requirements are documented on a per-function basis.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQStringQJsonValue>>,
    ) -> ::cpp_utils::MutRef<crate::QPairOfQStringQJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPair_QString_QJsonValue_operator_(
            self as *mut crate::QPairOfQStringQJsonValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQStringQJsonValue>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Constructs an empty pair. The <code>first</code> and <code>second</code> elements are initialized with <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>s.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPair<QString, QJsonValue>::QPair()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html#QPair">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty pair. The <code>first</code> and <code>second</code> elements are initialized with <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>s.</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QPairOfQStringQJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPair_QString_QJsonValue_QPair();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a pair and initializes the <code>first</code> element with <i>value1</i> and the <code>second</code> element with <i>value2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPair<QString, QJsonValue>::QPair(const QString& t1, const QJsonValue& t2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html#QPair-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a pair and initializes the <code>first</code> element with <i>value1</i> and the <code>second</code> element with <i>value2</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpair.html#qMakePair">qMakePair</a>().</p></div>
    pub unsafe fn new_2a(
        t1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        t2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonValue>>,
    ) -> ::cpp_utils::CppBox<crate::QPairOfQStringQJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPair_QString_QJsonValue_QPair1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(t1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonValue>>::cast_into(t2)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPair<QString, QJsonValue>::QPair(const QPair<QString, QJsonValue>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; can be used in your application if the STL <code>pair</code> type is not available. It stores one value of type T1 and one value of type T2. It can be used as a return value for a function that needs to return two values, or as the value type of a <a href="http://doc.qt.io/qt-5/containers.html">generic container</a>.</p>
    /// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> that stores one <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and one <code>double</code> value:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpair.html#QPair">QPair</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> pair;
    ///
    /// </pre>
    /// <p>The components are accessible as public data members called <a href="http://doc.qt.io/qt-5/qpair.html#first-var">first</a> and <a href="http://doc.qt.io/qt-5/qpair.html#second-var">second</a>. For example:</p>
    /// <pre class="cpp">
    ///
    ///   pair<span class="operator">.</span>first <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   pair<span class="operator">.</span>second <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>Note, however, that it is almost always preferable to define a small struct to hold the result of a function with multiple return values. A struct trivially generalizes to more than two values, and allows more descriptive member names than <code>first</code> and <code>second</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">struct</span> Variable {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> name;
    /// &#32;     <span class="type">double</span> value;
    ///   };
    ///   Variable v;
    ///   v<span class="operator">.</span>name <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   v<span class="operator">.</span>value <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>The advent of C++11 automatic variable type deduction (<code>auto</code>) shifts the emphasis from the type name to the name of functions and members. Thus, <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>, like <code>std::pair</code> and <code>std::tuple</code>, is mostly useful in generic (template) code, where defining a dedicated type is not possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>'s template data types (T1 and T2) must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; these requirements are documented on a per-function basis.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQStringQJsonValue>>,
    ) -> ::cpp_utils::CppBox<crate::QPairOfQStringQJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPair_QString_QJsonValue_QPair2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQStringQJsonValue>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Swaps this pair with <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QPair<QString, QJsonValue>::swap(QPair<QString, QJsonValue>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps this pair with <i>other</i>.</p>
    /// <p>Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms-obsolete.html#qSwap-1">qSwap</a>(<span class="keyword">this</span><span class="operator">-</span><span class="operator">&gt;</span>first<span class="operator">,</span> other<span class="operator">.</span>first);
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms-obsolete.html#qSwap-1">qSwap</a>(<span class="keyword">this</span><span class="operator">-</span><span class="operator">&gt;</span>second<span class="operator">,</span> other<span class="operator">.</span>second);
    ///
    /// </pre>
    /// <p>Swap overloads are found in namespace <code>std</code> as well as via argument-dependent lookup (ADL) in the namespace of <code>T</code> .</p>
    /// <p>This function was introduced in  Qt 5.5.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QPairOfQStringQJsonValue>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QPair_QString_QJsonValue_swap(self as *mut crate::QPairOfQStringQJsonValue, ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QPairOfQStringQJsonValue>>::cast_into(other).as_mut_raw_ptr())
    }
}

pub mod q_list_of_q_url {
    //! C++ type: <span style='color: green;'>```QList<QUrl>```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QUrl>::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::iterator& QList<QUrl>::iterator::operator=(const QList<QUrl>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_url::Iterator>>,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_url::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QUrl_iterator_operator_8(self as *mut crate::q_list_of_q_url::Iterator, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_url::Iterator>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::iterator QList<QUrl>::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_url::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QUrl_iterator_operator__13(
                self as *mut crate::q_list_of_q_url::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::iterator QList<QUrl>::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_url::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QUrl_iterator_operator__11(
                self as *mut crate::q_list_of_q_url::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QUrl& QList<QUrl>::iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValueRef iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>. If you assign to it, the assignment will apply to the character in the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> of <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> from which you got the reference.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::MutRef<crate::QUrl> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QUrl_iterator_operator__1(
                self as *const crate::q_list_of_q_url::Iterator,
                j,
            );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QUrl>::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>().</p></div>
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_list_of_q_url::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QUrl_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QUrl>::iterator::iterator(const QList<QUrl>::iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_url::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_url::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QUrl_iterator_iterator1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_url::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QUrl* QList<QUrl>::iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's value.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::MutPtr<crate::QUrl> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QUrl_iterator_operator__(
                self as *const crate::q_list_of_q_url::Iterator,
            );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QUrl>::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::const_iterator& QList<QUrl>::const_iterator::operator=(const QList<QUrl>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_url::ConstIterator>>,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_url::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QUrl_const_iterator_operator_6(self as *mut crate::q_list_of_q_url::ConstIterator, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_url::ConstIterator>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::const_iterator QList<QUrl>::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_url::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QUrl_const_iterator_operator__9(
                self as *mut crate::q_list_of_q_url::ConstIterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::const_iterator QList<QUrl>::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_url::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QUrl_const_iterator_operator__7(
                self as *mut crate::q_list_of_q_url::ConstIterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QUrl& QList<QUrl>::const_iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValue const_iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(&self, j: ::std::os::raw::c_longlong) -> ::cpp_utils::Ref<crate::QUrl> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QUrl_const_iterator_operator__1(
                self as *const crate::q_list_of_q_url::ConstIterator,
                j,
            );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QUrl>::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>().</p></div>
        pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::q_list_of_q_url::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_const_iterator_const_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QUrl>::const_iterator::const_iterator(const QList<QUrl>::iterator& o)```</span>.
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_url::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_url::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QUrl_const_iterator_const_iterator2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_url::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QUrl>::const_iterator::const_iterator(const QList<QUrl>::const_iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_url::ConstIterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_url::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QUrl_const_iterator_const_iterator1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_url::ConstIterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current result.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QUrl* QList<QUrl>::const_iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current result.</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::Ptr<crate::QUrl> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QUrl_const_iterator_operator__(
                self as *const crate::q_list_of_q_url::ConstIterator,
            );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_url::Iterator {
        type Output = ::cpp_utils::MutRef<crate::QUrl>;
        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QUrl& QList<QUrl>::iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>().</p></div>
        fn indirection(self) -> ::cpp_utils::MutRef<crate::QUrl> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_iterator_operator_(
                    self as *const crate::q_list_of_q_url::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_url::Iterator>>
        for crate::q_list_of_q_url::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QUrl>::iterator::operator==(const QList<QUrl>::iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_url::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_iterator_operator__2(
                    self as *const crate::q_list_of_q_url::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_url::Iterator>>
        for crate::q_list_of_q_url::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QUrl>::iterator::operator<(const QList<QUrl>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_url::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_iterator_operator_1(
                    self as *const crate::q_list_of_q_url::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_url::Iterator>>
        for crate::q_list_of_q_url::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QUrl>::iterator::operator<=(const QList<QUrl>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_url::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_iterator_operator__4(
                    self as *const crate::q_list_of_q_url::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_url::Iterator>>
        for crate::q_list_of_q_url::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QUrl>::iterator::operator>(const QList<QUrl>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_url::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_iterator_operator_2(
                    self as *const crate::q_list_of_q_url::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_url::Iterator>>
        for crate::q_list_of_q_url::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QUrl>::iterator::operator>=(const QList<QUrl>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_url::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_iterator_operator__5(
                    self as *const crate::q_list_of_q_url::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_url::ConstIterator>>
        for crate::q_list_of_q_url::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QUrl>::iterator::operator==(const QList<QUrl>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_url::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_iterator_operator__6(
                    self as *const crate::q_list_of_q_url::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_url::ConstIterator>>
        for crate::q_list_of_q_url::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QUrl>::iterator::operator<(const QList<QUrl>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_url::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_iterator_operator_3(
                    self as *const crate::q_list_of_q_url::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_url::ConstIterator>>
        for crate::q_list_of_q_url::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QUrl>::iterator::operator<=(const QList<QUrl>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_url::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_iterator_operator__8(
                    self as *const crate::q_list_of_q_url::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_url::ConstIterator>>
        for crate::q_list_of_q_url::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QUrl>::iterator::operator>(const QList<QUrl>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_url::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_iterator_operator_4(
                    self as *const crate::q_list_of_q_url::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_url::ConstIterator>>
        for crate::q_list_of_q_url::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QUrl>::iterator::operator>=(const QList<QUrl>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_url::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_iterator_operator__9(
                    self as *const crate::q_list_of_q_url::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_url::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_url::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::iterator& QList<QUrl>::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_url::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_iterator_operator__10(
                    self as *mut crate::q_list_of_q_url::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_url::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_url::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::iterator& QList<QUrl>::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_url::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_iterator_operator__12(
                    self as *mut crate::q_list_of_q_url::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong> for crate::q_list_of_q_url::Iterator {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::iterator& QList<QUrl>::iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_iterator_operator__14(
                    self as *mut crate::q_list_of_q_url::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong> for crate::q_list_of_q_url::Iterator {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::iterator& QList<QUrl>::iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_iterator_operator__15(
                    self as *mut crate::q_list_of_q_url::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_list_of_q_url::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_url::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::iterator QList<QUrl>::iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_url::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_iterator_operator_5(
                    self as *const crate::q_list_of_q_url::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_list_of_q_url::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_url::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::iterator QList<QUrl>::iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_url::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_iterator_operator_6(
                    self as *const crate::q_list_of_q_url::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_url::Iterator>>
        for &crate::q_list_of_q_url::Iterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QUrl>::iterator::operator-(QList<QUrl>::iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_url::Iterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_iterator_operator_7(
                    self as *const crate::q_list_of_q_url::Iterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_url::ConstIterator {
        type Output = ::cpp_utils::Ref<crate::QUrl>;
        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QUrl& QList<QUrl>::const_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        fn indirection(self) -> ::cpp_utils::Ref<crate::QUrl> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_const_iterator_operator_(
                    self as *const crate::q_list_of_q_url::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_url::ConstIterator>>
        for crate::q_list_of_q_url::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QUrl>::const_iterator::operator==(const QList<QUrl>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_url::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_const_iterator_operator__2(
                    self as *const crate::q_list_of_q_url::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_url::ConstIterator>>
        for crate::q_list_of_q_url::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QUrl>::const_iterator::operator<(const QList<QUrl>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_url::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_const_iterator_operator_1(
                    self as *const crate::q_list_of_q_url::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_url::ConstIterator>>
        for crate::q_list_of_q_url::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QUrl>::const_iterator::operator<=(const QList<QUrl>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_url::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_const_iterator_operator__4(
                    self as *const crate::q_list_of_q_url::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_url::ConstIterator>>
        for crate::q_list_of_q_url::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QUrl>::const_iterator::operator>(const QList<QUrl>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_url::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_const_iterator_operator_2(
                    self as *const crate::q_list_of_q_url::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_url::ConstIterator>>
        for crate::q_list_of_q_url::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QUrl>::const_iterator::operator>=(const QList<QUrl>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_url::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_const_iterator_operator__5(
                    self as *const crate::q_list_of_q_url::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_url::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_url::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::const_iterator& QList<QUrl>::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_url::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_const_iterator_operator__6(
                    self as *mut crate::q_list_of_q_url::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_url::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_url::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::const_iterator& QList<QUrl>::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_url::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_const_iterator_operator__8(
                    self as *mut crate::q_list_of_q_url::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong> for crate::q_list_of_q_url::ConstIterator {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::const_iterator& QList<QUrl>::const_iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_const_iterator_operator__10(
                    self as *mut crate::q_list_of_q_url::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong> for crate::q_list_of_q_url::ConstIterator {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::const_iterator& QList<QUrl>::const_iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_const_iterator_operator__11(
                    self as *mut crate::q_list_of_q_url::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_list_of_q_url::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_url::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::const_iterator QList<QUrl>::const_iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_url::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_const_iterator_operator_3(
                    self as *const crate::q_list_of_q_url::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_list_of_q_url::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_url::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::const_iterator QList<QUrl>::const_iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_url::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_const_iterator_operator_4(
                    self as *const crate::q_list_of_q_url::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_url::ConstIterator>>
        for &crate::q_list_of_q_url::ConstIterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QUrl>::const_iterator::operator-(QList<QUrl>::const_iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_url::ConstIterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QUrl_const_iterator_operator_5(
                    self as *const crate::q_list_of_q_url::ConstIterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_url::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QUrl>::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_iterator_diterator(
                self as *mut crate::q_list_of_q_url::Iterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_url::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QUrl>::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_const_iterator_dconst_iterator(
                self as *mut crate::q_list_of_q_url::ConstIterator,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
///
/// C++ class: <span style='color: green;'>```QList<QUrl>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlist.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores items in a list that provides fast index-based access and index-based insertions and removals.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Internally, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T if <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. Otherwise, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T* and the items are allocated on the heap.</p>
/// <p>The array representation allows very fast insertions and index-based access. The <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() operations are also very fast because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates memory at both ends of its internal array. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.</p>
/// <p>Note, however, that when the conditions specified above are not met, each append or insert of a new item requires allocating the new item on the heap, and this per item allocation will make <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> a better choice for use cases that do a lot of appending or inserting, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> can allocate memory for many items in a single heap allocation.</p>
/// <p>Note that the internal array only ever gets bigger over the life of the list. It never shrinks. The internal array is deallocated by the destructor and by the assignment operator, when one list is assigned to another.</p>
/// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerList;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span><span class="operator">&gt;</span> dateList;
///
/// </pre>
/// <p>Qt includes a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class that inherits <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>&gt; and adds a few convenience functions, such as <a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#filter">QStringList::filter</a>(). <a href="http://doc.qt.io/qt-5/qstring.html#split">QString::split</a>() creates QStringLists from strings.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> stores a list of items. The default constructor creates an empty list. To insert items into the list, you can use operator&lt;&lt;():</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"one"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"two"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"three"</span>;
///   <span class="comment">// list: ["one", "two", "three"]</span>
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(). In addition, it provides the following convenience functions: <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const lists, operator[]() returns a reference to the item and can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Bob"</span>)
/// &#32;     list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Robert"</span>;
///
/// </pre>
/// <p>Because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented as an array of pointers for types that are larger than a pointer or are not movable, this operation requires (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>). For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> list<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (list<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Jane"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Jane at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>A common requirement is to remove an item from a list and do something with it. For this, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(). Here's a loop that removes the items from a list one at a time and calls <code>delete</code> on them:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   <span class="keyword">while</span> (<span class="operator">!</span>list<span class="operator">.</span>isEmpty())
/// &#32;     <span class="keyword">delete</span> list<span class="operator">.</span>takeFirst();
///
/// </pre>
/// <p>Inserting and removing items at either end of the list is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> in most cases), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
/// <p>If you want to find all occurrences of a particular value in a list, use <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of a matching item if they find it; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> list<span class="operator">.</span>indexOf(<span class="string">"Jane"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Jane is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a list contains a particular value, use <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the list, use <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>(). If you want to replace all occurrences of a particular value with another, use <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlistiterator.html">QListIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablelistiterator.html">QMutableListIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented in such a way that direct index-based access is just as fast as using iterators.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <p>To make <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as efficient as possible, its member functions don't validate their input before using it. Except for <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), member functions always assume the list is <i>not</i> empty. Member functions that take index values as parameters always assume their index value parameters are in the valid range. This means <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> member functions can fail. If you define QT_NO_DEBUG when you compile, failures will not be detected. If you <i>don't</i> define QT_NO_DEBUG, failures will be detected using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT">Q_ASSERT</a>() or <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT_X">Q_ASSERT_X</a>() with an appropriate message.</p>
/// <p>To avoid failures when your list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling other member functions. If you must pass an index value that might not be in the valid range, check that it is less than the value returned by <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() but <i>not</i> less than 0.</p>
/// <a name="more-members"></a></div>
#[repr(C)]
pub struct QListOfQUrl {
    _unused: u8,
}
impl QListOfQUrl {
    /// <p>Inserts <i>value</i> at the end of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QUrl>::append(const QUrl& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["one", "two", "three"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn append_q_url(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_append(
            self as *mut crate::QListOfQUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QUrl>::append(const QList<QUrl>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> list to this list.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_list_of_q_url(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQUrl>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_append1(
            self as *mut crate::QListOfQUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQUrl>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QUrl& QList<QUrl>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>This function is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::QUrl> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_at(self as *const crate::QListOfQUrl, i);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QUrl& QList<QUrl>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::MutRef<crate::QUrl> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_back(self as *mut crate::QListOfQUrl);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QUrl& QList<QUrl>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<crate::QUrl> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_back1(self as *const crate::QListOfQUrl);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::const_iterator QList<QUrl>::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(&self) -> ::cpp_utils::CppBox<crate::q_list_of_q_url::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_cbegin(self as *const crate::QListOfQUrl);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::const_iterator QList<QUrl>::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn cend(&self) -> ::cpp_utils::CppBox<crate::q_list_of_q_url::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_cend(self as *const crate::QListOfQUrl);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all items from the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QUrl>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all items from the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_clear(self as *mut crate::QListOfQUrl)
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::const_iterator QList<QUrl>::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(&self) -> ::cpp_utils::CppBox<crate::q_list_of_q_url::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_constBegin(self as *const crate::QListOfQUrl);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::const_iterator QList<QUrl>::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn const_end(&self) -> ::cpp_utils::CppBox<crate::q_list_of_q_url::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_constEnd(self as *const crate::QListOfQUrl);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QUrl& QList<QUrl>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>().</p></div>
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<crate::QUrl> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_constFirst(self as *const crate::QListOfQUrl);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QUrl& QList<QUrl>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>().</p></div>
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<crate::QUrl> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_constLast(self as *const crate::QListOfQUrl);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QUrl>::contains(const QUrl& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn contains(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_contains(
            self as *const crate::QListOfQUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QUrl>& QList<QUrl>::operator=(const QList<QUrl>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this list and returns a reference to this list.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQUrl>>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQUrl> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QUrl_operator_(
            self as *mut crate::QListOfQUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQUrl>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of occurrences of <i>value</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QUrl>::count(const QUrl& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of occurrences of <i>value</i> in the list.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_count(
            self as *const crate::QListOfQUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QUrl>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_count1(self as *const crate::QListOfQUrl)
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QUrl>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_detach(self as *mut crate::QListOfQUrl)
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QUrl>::detachShared()```</span>.
    pub unsafe fn detach_shared(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_detachShared(self as *mut crate::QListOfQUrl)
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QUrl>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_empty(self as *const crate::QListOfQUrl)
    }

    /// <p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QUrl>::endsWith(const QUrl& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn ends_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_endsWith(
            self as *const crate::QListOfQUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::iterator QList<QUrl>::erase(QList<QUrl>::iterator pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn erase_1a(
        &mut self,
        pos: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_url::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_url::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QUrl_erase(
            self as *mut crate::QListOfQUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_url::Iterator>>::cast_into(
                pos,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::iterator QList<QUrl>::erase(QList<QUrl>::iterator first, QList<QUrl>::iterator last)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes all the items from <i>begin</i> up to (but not including) <i>end</i>. Returns an iterator to the same item that <i>end</i> referred to before the call.</p></div>
    pub unsafe fn erase_2a(
        &mut self,
        first: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_url::Iterator>>,
        last: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_url::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_url::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QUrl_erase1(
            self as *mut crate::QListOfQUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_url::Iterator>>::cast_into(
                first,
            )
            .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_url::Iterator>>::cast_into(
                last,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QUrl& QList<QUrl>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<crate::QUrl> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_first(self as *mut crate::QListOfQUrl);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QUrl& QList<QUrl>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::QUrl> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_first1(self as *const crate::QListOfQUrl);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QUrl& QList<QUrl>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::MutRef<crate::QUrl> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_front(self as *mut crate::QListOfQUrl);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QUrl& QList<QUrl>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<crate::QUrl> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_front1(self as *const crate::QListOfQUrl);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QUrl& QList<QUrl>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>(). This function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p></div>
    pub unsafe fn index(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::QUrl> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QUrl_operator__2(
            self as *const crate::QListOfQUrl,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QUrl& QList<QUrl>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If this function is called on a list that is currently being shared, it will trigger a copy of all elements. Otherwise, this function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>. If you do not want to modify the list you should use <a href="http://doc.qt.io/qt-5/qlist.html#at">QList::at</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QUrl> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QUrl_operator__3(
            self as *mut crate::QListOfQUrl,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QUrl>::indexOf(const QUrl& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_indexOf(
            self as *const crate::QListOfQUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(t).as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QUrl>::indexOf(const QUrl& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_indexOf1(
            self as *const crate::QListOfQUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QUrl>::insert(int i, const QUrl& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   list<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// list: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn insert_int_q_url(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_insert(
            self as *mut crate::QListOfQUrl,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::iterator QList<QUrl>::insert(QList<QUrl>::iterator before, const QUrl& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>value</i> in front of the item pointed to by the iterator <i>before</i>. Returns an iterator pointing at the inserted item. Note that the iterator passed to the function will be invalid after the call; the returned iterator should be used instead.</p></div>
    pub unsafe fn insert_iterator_q_url(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_url::Iterator>>,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_url::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QUrl_insert1(
            self as *mut crate::QListOfQUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_url::Iterator>>::cast_into(
                before,
            )
            .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(t).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QUrl>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_isDetached(self as *const crate::QListOfQUrl)
    }

    /// <p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QUrl>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_isEmpty(self as *const crate::QListOfQUrl)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QUrl>::isSharedWith(const QList<QUrl>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQUrl>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_isSharedWith(
            self as *const crate::QListOfQUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQUrl>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QUrl& QList<QUrl>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<crate::QUrl> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_last(self as *mut crate::QListOfQUrl);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QUrl& QList<QUrl>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::QUrl> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_last1(self as *const crate::QListOfQUrl);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QUrl>::lastIndexOf(const QUrl& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_lastIndexOf(
            self as *const crate::QListOfQUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(t).as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QUrl>::lastIndexOf(const QUrl& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_lastIndexOf1(
            self as *const crate::QListOfQUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QUrl>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_length(self as *const crate::QListOfQUrl)
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QUrl> QList<QUrl>::mid(int pos, int length = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        length: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQUrl> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QUrl_mid(
            self as *const crate::QListOfQUrl,
            pos,
            length,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QUrl> QList<QUrl>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQUrl> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_mid1(self as *const crate::QListOfQUrl, pos);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QUrl>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>move(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "C", "D", "E", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This is the same as insert(<i>to</i>, <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<i>from</i>)).This function assumes that both <i>from</i> and <i>to</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>from</i> and <i>to</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_move(self as *mut crate::QListOfQUrl, from, to)
    }

    /// <p>Constructs an empty list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QUrl>::QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty list.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QListOfQUrl> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QUrl_QList();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QUrl>::QList(const QList<QUrl>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQUrl>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQUrl> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QUrl_QList1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQUrl>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QUrl>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_pop_back(self as *mut crate::QListOfQUrl)
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QUrl>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_pop_front(self as *mut crate::QListOfQUrl)
    }

    /// <p>Inserts <i>value</i> at the beginning of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QUrl>::prepend(const QUrl& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(0, <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(&mut self, t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>) {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_prepend(
            self as *mut crate::QListOfQUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QUrl>::push_back(const QUrl& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_push_back(
            self as *mut crate::QListOfQUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QUrl>::push_front(const QUrl& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_push_front(
            self as *mut crate::QListOfQUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QUrl>::removeAll(const QUrl& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeAll(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_all(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_removeAll(
            self as *mut crate::QListOfQUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QUrl>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_removeAt(self as *mut crate::QListOfQUrl, i)
    }

    /// <p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QUrl>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_removeFirst(self as *mut crate::QListOfQUrl)
    }

    /// <p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QUrl>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_removeLast(self as *mut crate::QListOfQUrl)
    }

    /// <p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QUrl>::removeOne(const QUrl& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeOne(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", ,"sun", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_one(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_removeOne(
            self as *mut crate::QListOfQUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QUrl>::replace(int i, const QUrl& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_replace(
            self as *mut crate::QListOfQUrl,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Reserve space for <i>alloc</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QUrl>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reserve space for <i>alloc</i> elements.</p>
    /// <p>If <i>alloc</i> is smaller than the current size of the list, nothing will happen.</p>
    /// <p>Use this function to avoid repetetive reallocation of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s internal data if you can predict how many elements will be appended. Note that the reservation applies only to the internal pointer array.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_reserve(self as *mut crate::QListOfQUrl, size)
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QUrl>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_setSharable(
            self as *mut crate::QListOfQUrl,
            sharable,
        )
    }

    /// <p>Returns the number of items in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QUrl>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_size(self as *const crate::QListOfQUrl)
    }

    /// <p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QUrl>::startsWith(const QUrl& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn starts_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_startsWith(
            self as *const crate::QListOfQUrl,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QUrl>::swap(QList<QUrl>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap_1a(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QListOfQUrl>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_swap(
            self as *mut crate::QListOfQUrl,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QListOfQUrl>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QUrl>::swap(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swap(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    pub unsafe fn swap_2a(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_swap1(self as *mut crate::QListOfQUrl, i, j)
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QUrl>::swapItemsAt(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swapItemsAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swapItemsAt(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn swap_items_at(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_swapItemsAt(self as *mut crate::QListOfQUrl, i, j)
    }

    /// <p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QUrl QList<QUrl>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn take_at(&mut self, i: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QUrl> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_takeAt(self as *mut crate::QListOfQUrl, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QUrl QList<QUrl>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> ::cpp_utils::CppBox<crate::QUrl> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_takeFirst(self as *mut crate::QListOfQUrl);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QUrl QList<QUrl>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> ::cpp_utils::CppBox<crate::QUrl> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_takeLast(self as *mut crate::QListOfQUrl);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value at index position <i>i</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QUrl QList<QUrl>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the list.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that the index is going to be within bounds, you can use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QUrl> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_value(self as *const crate::QListOfQUrl, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QUrl QList<QUrl>::value(int i, const QUrl& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
    ) -> ::cpp_utils::CppBox<crate::QUrl> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QUrl_value1(
            self as *const crate::QListOfQUrl,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_list_of_q_mime_type {
    //! C++ type: <span style='color: green;'>```QList<QMimeType>```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QMimeType>::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::iterator& QList<QMimeType>::iterator::operator=(const QList<QMimeType>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_mime_type::Iterator>>,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_mime_type::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_iterator_operator_8(self as *mut crate::q_list_of_q_mime_type::Iterator, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_mime_type::Iterator>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::iterator QList<QMimeType>::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_iterator_operator__13(
                self as *mut crate::q_list_of_q_mime_type::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::iterator QList<QMimeType>::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_iterator_operator__11(
                self as *mut crate::q_list_of_q_mime_type::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMimeType& QList<QMimeType>::iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValueRef iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>. If you assign to it, the assignment will apply to the character in the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> of <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> from which you got the reference.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::MutRef<crate::QMimeType> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_iterator_operator__1(
                self as *const crate::q_list_of_q_mime_type::Iterator,
                j,
            );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QMimeType>::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>().</p></div>
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QMimeType>::iterator::iterator(const QList<QMimeType>::iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_mime_type::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_iterator_iterator1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_mime_type::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMimeType* QList<QMimeType>::iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's value.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::MutPtr<crate::QMimeType> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_iterator_operator__(
                self as *const crate::q_list_of_q_mime_type::Iterator,
            );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QMimeType>::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::const_iterator& QList<QMimeType>::const_iterator::operator=(const QList<QMimeType>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_mime_type::ConstIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_mime_type::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_const_iterator_operator_6(
                    self as *mut crate::q_list_of_q_mime_type::ConstIterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_mime_type::ConstIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::const_iterator QList<QMimeType>::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_const_iterator_operator__9(
                    self as *mut crate::q_list_of_q_mime_type::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::const_iterator QList<QMimeType>::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_const_iterator_operator__7(
                    self as *mut crate::q_list_of_q_mime_type::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QMimeType& QList<QMimeType>::const_iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValue const_iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::Ref<crate::QMimeType> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_const_iterator_operator__1(
                    self as *const crate::q_list_of_q_mime_type::ConstIterator,
                    j,
                );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QMimeType>::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>().</p></div>
        pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_const_iterator_const_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QMimeType>::const_iterator::const_iterator(const QList<QMimeType>::iterator& o)```</span>.
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_mime_type::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_const_iterator_const_iterator2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_mime_type::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QMimeType>::const_iterator::const_iterator(const QList<QMimeType>::const_iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_mime_type::ConstIterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_const_iterator_const_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_mime_type::ConstIterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current result.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QMimeType* QList<QMimeType>::const_iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current result.</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::Ptr<crate::QMimeType> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_const_iterator_operator__(
                    self as *const crate::q_list_of_q_mime_type::ConstIterator,
                );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_mime_type::Iterator {
        type Output = ::cpp_utils::MutRef<crate::QMimeType>;
        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QMimeType& QList<QMimeType>::iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>().</p></div>
        fn indirection(self) -> ::cpp_utils::MutRef<crate::QMimeType> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_iterator_operator_(
                    self as *const crate::q_list_of_q_mime_type::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_mime_type::Iterator>>
        for crate::q_list_of_q_mime_type::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QMimeType>::iterator::operator==(const QList<QMimeType>::iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_mime_type::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_iterator_operator__2(
                    self as *const crate::q_list_of_q_mime_type::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_mime_type::Iterator>>
        for crate::q_list_of_q_mime_type::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QMimeType>::iterator::operator<(const QList<QMimeType>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_mime_type::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_iterator_operator_1(
                    self as *const crate::q_list_of_q_mime_type::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_mime_type::Iterator>>
        for crate::q_list_of_q_mime_type::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QMimeType>::iterator::operator<=(const QList<QMimeType>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_mime_type::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_iterator_operator__4(
                    self as *const crate::q_list_of_q_mime_type::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_mime_type::Iterator>>
        for crate::q_list_of_q_mime_type::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QMimeType>::iterator::operator>(const QList<QMimeType>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_mime_type::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_iterator_operator_2(
                    self as *const crate::q_list_of_q_mime_type::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_mime_type::Iterator>>
        for crate::q_list_of_q_mime_type::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QMimeType>::iterator::operator>=(const QList<QMimeType>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_mime_type::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_iterator_operator__5(
                    self as *const crate::q_list_of_q_mime_type::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_mime_type::ConstIterator>>
        for crate::q_list_of_q_mime_type::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QMimeType>::iterator::operator==(const QList<QMimeType>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_mime_type::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_iterator_operator__6(
                    self as *const crate::q_list_of_q_mime_type::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_mime_type::ConstIterator>>
        for crate::q_list_of_q_mime_type::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QMimeType>::iterator::operator<(const QList<QMimeType>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_mime_type::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_iterator_operator_3(
                    self as *const crate::q_list_of_q_mime_type::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_mime_type::ConstIterator>>
        for crate::q_list_of_q_mime_type::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QMimeType>::iterator::operator<=(const QList<QMimeType>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_mime_type::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_iterator_operator__8(
                    self as *const crate::q_list_of_q_mime_type::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_mime_type::ConstIterator>>
        for crate::q_list_of_q_mime_type::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QMimeType>::iterator::operator>(const QList<QMimeType>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_mime_type::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_iterator_operator_4(
                    self as *const crate::q_list_of_q_mime_type::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_mime_type::ConstIterator>>
        for crate::q_list_of_q_mime_type::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QMimeType>::iterator::operator>=(const QList<QMimeType>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_mime_type::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_iterator_operator__9(
                    self as *const crate::q_list_of_q_mime_type::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_mime_type::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_mime_type::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::iterator& QList<QMimeType>::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_mime_type::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_iterator_operator__10(
                    self as *mut crate::q_list_of_q_mime_type::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_mime_type::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_mime_type::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::iterator& QList<QMimeType>::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_mime_type::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_iterator_operator__12(
                    self as *mut crate::q_list_of_q_mime_type::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong> for crate::q_list_of_q_mime_type::Iterator {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::iterator& QList<QMimeType>::iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_iterator_operator__14(
                    self as *mut crate::q_list_of_q_mime_type::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong> for crate::q_list_of_q_mime_type::Iterator {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::iterator& QList<QMimeType>::iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_iterator_operator__15(
                    self as *mut crate::q_list_of_q_mime_type::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_list_of_q_mime_type::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::iterator QList<QMimeType>::iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_iterator_operator_5(
                    self as *const crate::q_list_of_q_mime_type::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_list_of_q_mime_type::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::iterator QList<QMimeType>::iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_iterator_operator_6(
                    self as *const crate::q_list_of_q_mime_type::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_mime_type::Iterator>>
        for &crate::q_list_of_q_mime_type::Iterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QMimeType>::iterator::operator-(QList<QMimeType>::iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_mime_type::Iterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_iterator_operator_7(
                    self as *const crate::q_list_of_q_mime_type::Iterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_mime_type::ConstIterator {
        type Output = ::cpp_utils::Ref<crate::QMimeType>;
        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QMimeType& QList<QMimeType>::const_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        fn indirection(self) -> ::cpp_utils::Ref<crate::QMimeType> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_const_iterator_operator_(
                    self as *const crate::q_list_of_q_mime_type::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_mime_type::ConstIterator>>
        for crate::q_list_of_q_mime_type::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QMimeType>::const_iterator::operator==(const QList<QMimeType>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_mime_type::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_const_iterator_operator__2(
                    self as *const crate::q_list_of_q_mime_type::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_mime_type::ConstIterator>>
        for crate::q_list_of_q_mime_type::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QMimeType>::const_iterator::operator<(const QList<QMimeType>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_mime_type::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_const_iterator_operator_1(
                    self as *const crate::q_list_of_q_mime_type::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_mime_type::ConstIterator>>
        for crate::q_list_of_q_mime_type::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QMimeType>::const_iterator::operator<=(const QList<QMimeType>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_mime_type::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_const_iterator_operator__4(
                    self as *const crate::q_list_of_q_mime_type::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_mime_type::ConstIterator>>
        for crate::q_list_of_q_mime_type::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QMimeType>::const_iterator::operator>(const QList<QMimeType>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_mime_type::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_const_iterator_operator_2(
                    self as *const crate::q_list_of_q_mime_type::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_mime_type::ConstIterator>>
        for crate::q_list_of_q_mime_type::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QMimeType>::const_iterator::operator>=(const QList<QMimeType>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_mime_type::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_const_iterator_operator__5(
                    self as *const crate::q_list_of_q_mime_type::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_mime_type::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_mime_type::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::const_iterator& QList<QMimeType>::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_mime_type::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_const_iterator_operator__6(
                    self as *mut crate::q_list_of_q_mime_type::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_mime_type::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_mime_type::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::const_iterator& QList<QMimeType>::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_mime_type::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_const_iterator_operator__8(
                    self as *mut crate::q_list_of_q_mime_type::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_mime_type::ConstIterator
    {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::const_iterator& QList<QMimeType>::const_iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_const_iterator_operator__10(
                    self as *mut crate::q_list_of_q_mime_type::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_mime_type::ConstIterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::const_iterator& QList<QMimeType>::const_iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_const_iterator_operator__11(
                    self as *mut crate::q_list_of_q_mime_type::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_list_of_q_mime_type::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::const_iterator QList<QMimeType>::const_iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_const_iterator_operator_3(
                    self as *const crate::q_list_of_q_mime_type::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_list_of_q_mime_type::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::const_iterator QList<QMimeType>::const_iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_const_iterator_operator_4(
                    self as *const crate::q_list_of_q_mime_type::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_mime_type::ConstIterator>>
        for &crate::q_list_of_q_mime_type::ConstIterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QMimeType>::const_iterator::operator-(QList<QMimeType>::const_iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_mime_type::ConstIterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_const_iterator_operator_5(
                    self as *const crate::q_list_of_q_mime_type::ConstIterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_mime_type::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QMimeType>::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_iterator_diterator(
                self as *mut crate::q_list_of_q_mime_type::Iterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_mime_type::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QMimeType>::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_const_iterator_dconst_iterator(
                self as *mut crate::q_list_of_q_mime_type::ConstIterator,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
///
/// C++ class: <span style='color: green;'>```QList<QMimeType>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlist.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores items in a list that provides fast index-based access and index-based insertions and removals.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Internally, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T if <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. Otherwise, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T* and the items are allocated on the heap.</p>
/// <p>The array representation allows very fast insertions and index-based access. The <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() operations are also very fast because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates memory at both ends of its internal array. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.</p>
/// <p>Note, however, that when the conditions specified above are not met, each append or insert of a new item requires allocating the new item on the heap, and this per item allocation will make <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> a better choice for use cases that do a lot of appending or inserting, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> can allocate memory for many items in a single heap allocation.</p>
/// <p>Note that the internal array only ever gets bigger over the life of the list. It never shrinks. The internal array is deallocated by the destructor and by the assignment operator, when one list is assigned to another.</p>
/// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerList;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span><span class="operator">&gt;</span> dateList;
///
/// </pre>
/// <p>Qt includes a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class that inherits <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>&gt; and adds a few convenience functions, such as <a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#filter">QStringList::filter</a>(). <a href="http://doc.qt.io/qt-5/qstring.html#split">QString::split</a>() creates QStringLists from strings.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> stores a list of items. The default constructor creates an empty list. To insert items into the list, you can use operator&lt;&lt;():</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"one"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"two"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"three"</span>;
///   <span class="comment">// list: ["one", "two", "three"]</span>
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(). In addition, it provides the following convenience functions: <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const lists, operator[]() returns a reference to the item and can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Bob"</span>)
/// &#32;     list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Robert"</span>;
///
/// </pre>
/// <p>Because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented as an array of pointers for types that are larger than a pointer or are not movable, this operation requires (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>). For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> list<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (list<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Jane"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Jane at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>A common requirement is to remove an item from a list and do something with it. For this, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(). Here's a loop that removes the items from a list one at a time and calls <code>delete</code> on them:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   <span class="keyword">while</span> (<span class="operator">!</span>list<span class="operator">.</span>isEmpty())
/// &#32;     <span class="keyword">delete</span> list<span class="operator">.</span>takeFirst();
///
/// </pre>
/// <p>Inserting and removing items at either end of the list is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> in most cases), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
/// <p>If you want to find all occurrences of a particular value in a list, use <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of a matching item if they find it; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> list<span class="operator">.</span>indexOf(<span class="string">"Jane"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Jane is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a list contains a particular value, use <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the list, use <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>(). If you want to replace all occurrences of a particular value with another, use <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlistiterator.html">QListIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablelistiterator.html">QMutableListIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented in such a way that direct index-based access is just as fast as using iterators.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <p>To make <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as efficient as possible, its member functions don't validate their input before using it. Except for <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), member functions always assume the list is <i>not</i> empty. Member functions that take index values as parameters always assume their index value parameters are in the valid range. This means <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> member functions can fail. If you define QT_NO_DEBUG when you compile, failures will not be detected. If you <i>don't</i> define QT_NO_DEBUG, failures will be detected using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT">Q_ASSERT</a>() or <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT_X">Q_ASSERT_X</a>() with an appropriate message.</p>
/// <p>To avoid failures when your list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling other member functions. If you must pass an index value that might not be in the valid range, check that it is less than the value returned by <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() but <i>not</i> less than 0.</p>
/// <a name="more-members"></a></div>
#[repr(C)]
pub struct QListOfQMimeType {
    _unused: u8,
}
impl QListOfQMimeType {
    /// <p>Inserts <i>value</i> at the end of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QMimeType>::append(const QMimeType& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["one", "two", "three"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn append_q_mime_type(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMimeType>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_append(
            self as *mut crate::QListOfQMimeType,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMimeType>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QMimeType>::append(const QList<QMimeType>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> list to this list.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_list_of_q_mime_type(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQMimeType>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_append1(
            self as *mut crate::QListOfQMimeType,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQMimeType>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QMimeType& QList<QMimeType>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>This function is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_at(
            self as *const crate::QListOfQMimeType,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMimeType& QList<QMimeType>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::MutRef<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_back(
            self as *mut crate::QListOfQMimeType,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QMimeType& QList<QMimeType>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_back1(
            self as *const crate::QListOfQMimeType,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::const_iterator QList<QMimeType>::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_cbegin(
            self as *const crate::QListOfQMimeType,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::const_iterator QList<QMimeType>::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn cend(&self) -> ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_cend(
            self as *const crate::QListOfQMimeType,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all items from the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QMimeType>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all items from the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_clear(self as *mut crate::QListOfQMimeType)
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::const_iterator QList<QMimeType>::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_constBegin(
            self as *const crate::QListOfQMimeType,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::const_iterator QList<QMimeType>::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn const_end(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_constEnd(
            self as *const crate::QListOfQMimeType,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QMimeType& QList<QMimeType>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>().</p></div>
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_constFirst(
            self as *const crate::QListOfQMimeType,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QMimeType& QList<QMimeType>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>().</p></div>
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_constLast(
            self as *const crate::QListOfQMimeType,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QMimeType>::contains(const QMimeType& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn contains(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMimeType>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_contains(
            self as *const crate::QListOfQMimeType,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMimeType>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>& QList<QMimeType>::operator=(const QList<QMimeType>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this list and returns a reference to this list.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQMimeType>>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_operator_(
            self as *mut crate::QListOfQMimeType,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQMimeType>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of occurrences of <i>value</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QMimeType>::count(const QMimeType& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of occurrences of <i>value</i> in the list.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMimeType>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_count(
            self as *const crate::QListOfQMimeType,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMimeType>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QMimeType>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_count1(self as *const crate::QListOfQMimeType)
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QMimeType>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_detach(self as *mut crate::QListOfQMimeType)
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QMimeType>::detachShared()```</span>.
    pub unsafe fn detach_shared(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_detachShared(
            self as *mut crate::QListOfQMimeType,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QMimeType>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_empty(self as *const crate::QListOfQMimeType)
    }

    /// <p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QMimeType>::endsWith(const QMimeType& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn ends_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMimeType>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_endsWith(
            self as *const crate::QListOfQMimeType,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMimeType>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::iterator QList<QMimeType>::erase(QList<QMimeType>::iterator pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn erase_1a(
        &mut self,
        pos: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_mime_type::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_erase(self as *mut crate::QListOfQMimeType, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_mime_type::Iterator>>::cast_into(pos).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::iterator QList<QMimeType>::erase(QList<QMimeType>::iterator first, QList<QMimeType>::iterator last)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes all the items from <i>begin</i> up to (but not including) <i>end</i>. Returns an iterator to the same item that <i>end</i> referred to before the call.</p></div>
    pub unsafe fn erase_2a(
        &mut self,
        first: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_mime_type::Iterator>>,
        last: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_mime_type::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_erase1(self as *mut crate::QListOfQMimeType, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_mime_type::Iterator>>::cast_into(first).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_mime_type::Iterator>>::cast_into(last).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMimeType& QList<QMimeType>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_first(
            self as *mut crate::QListOfQMimeType,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QMimeType& QList<QMimeType>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_first1(
            self as *const crate::QListOfQMimeType,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMimeType& QList<QMimeType>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::MutRef<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_front(
            self as *mut crate::QListOfQMimeType,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QMimeType& QList<QMimeType>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_front1(
            self as *const crate::QListOfQMimeType,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QMimeType& QList<QMimeType>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>(). This function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p></div>
    pub unsafe fn index(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_operator__2(
            self as *const crate::QListOfQMimeType,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMimeType& QList<QMimeType>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If this function is called on a list that is currently being shared, it will trigger a copy of all elements. Otherwise, this function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>. If you do not want to modify the list you should use <a href="http://doc.qt.io/qt-5/qlist.html#at">QList::at</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_operator__3(
            self as *mut crate::QListOfQMimeType,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QMimeType>::indexOf(const QMimeType& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMimeType>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_indexOf(
            self as *const crate::QListOfQMimeType,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMimeType>>::cast_into(t).as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QMimeType>::indexOf(const QMimeType& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMimeType>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_indexOf1(
            self as *const crate::QListOfQMimeType,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMimeType>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QMimeType>::insert(int i, const QMimeType& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   list<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// list: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn insert_int_q_mime_type(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMimeType>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_insert(
            self as *mut crate::QListOfQMimeType,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMimeType>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::iterator QList<QMimeType>::insert(QList<QMimeType>::iterator before, const QMimeType& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>value</i> in front of the item pointed to by the iterator <i>before</i>. Returns an iterator pointing at the inserted item. Note that the iterator passed to the function will be invalid after the call; the returned iterator should be used instead.</p></div>
    pub unsafe fn insert_iterator_q_mime_type(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_mime_type::Iterator>>,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMimeType>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_insert1(self as *mut crate::QListOfQMimeType, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_mime_type::Iterator>>::cast_into(before).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMimeType>>::cast_into(t).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QMimeType>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_isDetached(
            self as *const crate::QListOfQMimeType,
        )
    }

    /// <p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QMimeType>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_isEmpty(
            self as *const crate::QListOfQMimeType,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QMimeType>::isSharedWith(const QList<QMimeType>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQMimeType>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_isSharedWith(
            self as *const crate::QListOfQMimeType,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQMimeType>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMimeType& QList<QMimeType>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_last(
            self as *mut crate::QListOfQMimeType,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QMimeType& QList<QMimeType>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_last1(
            self as *const crate::QListOfQMimeType,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QMimeType>::lastIndexOf(const QMimeType& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMimeType>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_lastIndexOf(
            self as *const crate::QListOfQMimeType,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMimeType>>::cast_into(t).as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QMimeType>::lastIndexOf(const QMimeType& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMimeType>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_lastIndexOf1(
            self as *const crate::QListOfQMimeType,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMimeType>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QMimeType>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_length(self as *const crate::QListOfQMimeType)
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType> QList<QMimeType>::mid(int pos, int length = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        length: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_mid(
            self as *const crate::QListOfQMimeType,
            pos,
            length,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType> QList<QMimeType>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_mid1(
            self as *const crate::QListOfQMimeType,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QMimeType>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>move(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "C", "D", "E", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This is the same as insert(<i>to</i>, <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<i>from</i>)).This function assumes that both <i>from</i> and <i>to</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>from</i> and <i>to</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_move(
            self as *mut crate::QListOfQMimeType,
            from,
            to,
        )
    }

    /// <p>Constructs an empty list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QMimeType>::QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty list.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QListOfQMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_QList();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QMimeType>::QList(const QList<QMimeType>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQMimeType>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_QList1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQMimeType>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QMimeType>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_pop_back(self as *mut crate::QListOfQMimeType)
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QMimeType>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_pop_front(
            self as *mut crate::QListOfQMimeType,
        )
    }

    /// <p>Inserts <i>value</i> at the beginning of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QMimeType>::prepend(const QMimeType& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(0, <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMimeType>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_prepend(
            self as *mut crate::QListOfQMimeType,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMimeType>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QMimeType>::push_back(const QMimeType& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMimeType>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_push_back(
            self as *mut crate::QListOfQMimeType,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMimeType>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QMimeType>::push_front(const QMimeType& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMimeType>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_push_front(
            self as *mut crate::QListOfQMimeType,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMimeType>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QMimeType>::removeAll(const QMimeType& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeAll(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_all(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMimeType>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_removeAll(
            self as *mut crate::QListOfQMimeType,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMimeType>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QMimeType>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_removeAt(
            self as *mut crate::QListOfQMimeType,
            i,
        )
    }

    /// <p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QMimeType>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_removeFirst(
            self as *mut crate::QListOfQMimeType,
        )
    }

    /// <p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QMimeType>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_removeLast(
            self as *mut crate::QListOfQMimeType,
        )
    }

    /// <p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QMimeType>::removeOne(const QMimeType& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeOne(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", ,"sun", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_one(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMimeType>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_removeOne(
            self as *mut crate::QListOfQMimeType,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMimeType>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QMimeType>::replace(int i, const QMimeType& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMimeType>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_replace(
            self as *mut crate::QListOfQMimeType,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMimeType>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Reserve space for <i>alloc</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QMimeType>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reserve space for <i>alloc</i> elements.</p>
    /// <p>If <i>alloc</i> is smaller than the current size of the list, nothing will happen.</p>
    /// <p>Use this function to avoid repetetive reallocation of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s internal data if you can predict how many elements will be appended. Note that the reservation applies only to the internal pointer array.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_reserve(
            self as *mut crate::QListOfQMimeType,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QMimeType>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_setSharable(
            self as *mut crate::QListOfQMimeType,
            sharable,
        )
    }

    /// <p>Returns the number of items in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QMimeType>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_size(self as *const crate::QListOfQMimeType)
    }

    /// <p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QMimeType>::startsWith(const QMimeType& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn starts_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMimeType>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_startsWith(
            self as *const crate::QListOfQMimeType,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMimeType>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QMimeType>::swap(QList<QMimeType>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap_1a(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QListOfQMimeType>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_swap(
            self as *mut crate::QListOfQMimeType,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QListOfQMimeType>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QMimeType>::swap(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swap(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    pub unsafe fn swap_2a(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_swap1(
            self as *mut crate::QListOfQMimeType,
            i,
            j,
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QMimeType>::swapItemsAt(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swapItemsAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swapItemsAt(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn swap_items_at(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_swapItemsAt(
            self as *mut crate::QListOfQMimeType,
            i,
            j,
        )
    }

    /// <p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMimeType QList<QMimeType>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_takeAt(
            self as *mut crate::QListOfQMimeType,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMimeType QList<QMimeType>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> ::cpp_utils::CppBox<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_takeFirst(
            self as *mut crate::QListOfQMimeType,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMimeType QList<QMimeType>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> ::cpp_utils::CppBox<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_takeLast(
            self as *mut crate::QListOfQMimeType,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value at index position <i>i</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMimeType QList<QMimeType>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the list.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that the index is going to be within bounds, you can use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_value(
            self as *const crate::QListOfQMimeType,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMimeType QList<QMimeType>::value(int i, const QMimeType& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMimeType>>,
    ) -> ::cpp_utils::CppBox<crate::QMimeType> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_value1(
            self as *const crate::QListOfQMimeType,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMimeType>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> class is a template class that provides a dynamic array.</p>
///
/// C++ class: <span style='color: green;'>```QVector<QStaticPlugin>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qvector.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> class is a template class that provides a dynamic array.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores its items in adjacent memory locations and provides fast index-based access.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Here's an example of a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> that stores <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerVector;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> stringVector;
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> stores its items in a vector (array). Typically, vectors are created with an initial size. For example, the following code constructs a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> with 200 elements:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">200</span>);
///
/// </pre>
/// <p>The elements are automatically initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you want to initialize the vector with a different value, pass that value as the second argument to the constructor:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">200</span><span class="operator">,</span> <span class="string">"Pass"</span>);
///
/// </pre>
/// <p>You can also call <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>() at any time to fill the vector with a value.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const vectors, operator[]() returns a reference to the item that can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (vector<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Liz"</span>)
/// &#32;     vector<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Elizabeth"</span>;
///
/// </pre>
/// <p>For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> vector<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (vector<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Alfonso"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Alfonso at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>Another way to access the data stored in a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> is to call <a href="http://doc.qt.io/qt-5/qvector.html#data">data</a>(). The function returns a pointer to the first item in the vector. You can use the pointer to directly access and modify the elements stored in the vector. The pointer is also useful if you need to pass a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to a function that accepts a plain C++ array.</p>
/// <p>If you want to find all occurrences of a particular value in a vector, use <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of the matching item if they found one; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> vector<span class="operator">.</span>indexOf(<span class="string">"Harumi"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Harumi is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a vector contains a particular value, use <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the vector, use <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(). With the exception of <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), these functions can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>) for large vectors, because they require moving many items in the vector by one position in memory. If you want a container class that provides fast insertion/removal in the middle, use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> or <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
/// <p>Unlike plain C++ arrays, QVectors can be resized at any time by calling <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>(). If the new size is larger than the old size, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> might need to reallocate the whole vector. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> tries to reduce the number of reallocations by preallocating up to twice as much memory as the actual data needs.</p>
/// <p>If you know in advance approximately how many items the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> will contain, you can call <a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>(), asking <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to preallocate a certain amount of memory. You can also call <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>() to find out how much memory <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> actually allocated.</p>
/// <p>Note that using non-const operators and functions can cause <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to do a deep copy of the data. This is due to <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicit sharing</a>.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qvectoriterator.html">QVectorIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablevectoriterator.html">QMutableVectorIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qvector.html#const_iterator-typedef">QVector::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qvector.html#iterator-typedefx">QVector::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>.</p>
/// <p>In addition to <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>, Qt also provides <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a>, a very low-level class with little functionality that is optimized for speed.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <a name="more-information-on-using-qt-containers"></a>
/// <h4>More Information on Using Qt Containers</h4>
/// <p>For a detailed discussion comparing Qt containers with each other and with STL containers, see <a href="http://marcmutz.wordpress.com/effective-qt/containers/">Understand the Qt Containers</a>.</p></div>
#[repr(C)]
pub struct QVectorOfQStaticPlugin {
    _unused: u8,
}
impl QVectorOfQStaticPlugin {
    /// <p>Inserts <i>value</i> at the end of the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStaticPlugin>::append(const QStaticPlugin& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   vector<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> three <span class="operator">=</span> <span class="string">"three"</span>;
    ///   vector<span class="operator">.</span>append(three);
    ///   <span class="comment">// vector: ["one", "two", "three"]</span>
    ///   <span class="comment">// three: "three"</span>
    ///
    /// </pre>
    /// <p>This is the same as calling resize(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() + 1) and assigning <i>value</i> to the new last element in the vector.</p>
    /// <p>This operation is relatively fast, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> typically allocates more memory than necessary, so it can grow without reallocating the entire vector each time.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>().</p></div>
    pub unsafe fn append_q_static_plugin(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStaticPlugin>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_append(
            self as *mut crate::QVectorOfQStaticPlugin,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStaticPlugin>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStaticPlugin>::append(const QVector<QStaticPlugin>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#append-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> vector to this vector.</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_vector_of_q_static_plugin(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQStaticPlugin>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_append2(
            self as *mut crate::QVectorOfQStaticPlugin,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQStaticPlugin>>::cast_into(l)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the item at index position <i>i</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStaticPlugin& QVector<QStaticPlugin>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the vector.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::QStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_at(
            self as *const crate::QVectorOfQStaticPlugin,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStaticPlugin& QVector<QStaticPlugin>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::MutRef<crate::QStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_back(
            self as *mut crate::QVectorOfQStaticPlugin,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStaticPlugin& QVector<QStaticPlugin>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<crate::QStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_back1(
            self as *const crate::QVectorOfQStaticPlugin,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the maximum number of items that can be stored in the vector without forcing a reallocation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QStaticPlugin>::capacity() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#capacity">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the maximum number of items that can be stored in the vector without forcing a reallocation.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s memory usage. In general, you will rarely ever need to call this function. If you want to know how many items are in the vector, call <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p></div>
    pub unsafe fn capacity(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_capacity(
            self as *const crate::QVectorOfQStaticPlugin,
        )
    }

    /// <p>Removes all the elements from the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStaticPlugin>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all the elements from the vector.</p>
    /// <p><b>Note: </b>Until Qt 5.6, this also released the memory used by the vector. From Qt 5.7, the capacity is preserved. To shed all capacity, swap with a default-constructed vector:</p><pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span>T<span class="operator">&gt;</span> v <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span>T<span class="operator">&gt;</span>()<span class="operator">.</span>swap(v);
    ///   Q_ASSERT(v<span class="operator">.</span>capacity() <span class="operator">=</span><span class="operator">=</span> <span class="number">0</span>);
    ///
    /// </pre>
    /// <p>or call <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_clear(
            self as *mut crate::QVectorOfQStaticPlugin,
        )
    }

    /// <p>Returns a const pointer to the data stored in the vector. The pointer can be used to access the items in the vector. The pointer remains valid as long as the vector isn't reallocated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStaticPlugin* QVector<QStaticPlugin>::constData() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const pointer to the data stored in the vector. The pointer can be used to access the items in the vector. The pointer remains valid as long as the vector isn't reallocated.</p>
    /// <p>This function is mostly useful to pass a vector to a function that accepts a plain C++ array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#data">data</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn const_data(&self) -> ::cpp_utils::Ptr<crate::QStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_constData(
            self as *const crate::QVectorOfQStaticPlugin,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns a const reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStaticPlugin& QVector<QStaticPlugin>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<crate::QStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_constFirst(
            self as *const crate::QVectorOfQStaticPlugin,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStaticPlugin& QVector<QStaticPlugin>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<crate::QStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_constLast(
            self as *const crate::QVectorOfQStaticPlugin,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Assigns <i>other</i> to this vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStaticPlugin>& QVector<QStaticPlugin>::operator=(const QVector<QStaticPlugin>& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this vector and returns a reference to this vector.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        v: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQStaticPlugin>>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_operator_(
            self as *mut crate::QVectorOfQStaticPlugin,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQStaticPlugin>>::cast_into(v)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QStaticPlugin>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p></div>
    pub unsafe fn count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_count1(
            self as *const crate::QVectorOfQStaticPlugin,
        )
    }

    /// <p>Returns a pointer to the data stored in the vector. The pointer can be used to access and modify the items in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStaticPlugin* QVector<QStaticPlugin>::data()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the data stored in the vector. The pointer can be used to access and modify the items in the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> vector(<span class="number">10</span>);
    ///   <span class="type">int</span> <span class="operator">*</span>data <span class="operator">=</span> vector<span class="operator">.</span>data();
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> <span class="number">10</span>; <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     data<span class="operator">[</span>i<span class="operator">]</span> <span class="operator">=</span> <span class="number">2</span> <span class="operator">*</span> i;
    ///
    /// </pre>
    /// <p>The pointer remains valid as long as the vector isn't reallocated.</p>
    /// <p>This function is mostly useful to pass a vector to a function that accepts a plain C++ array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constData">constData</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn data_mut(&mut self) -> ::cpp_utils::MutPtr<crate::QStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_data(
            self as *mut crate::QVectorOfQStaticPlugin,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStaticPlugin* QVector<QStaticPlugin>::data() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#data-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn data(&self) -> ::cpp_utils::Ptr<crate::QStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_data1(
            self as *const crate::QVectorOfQStaticPlugin,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```void QVector<QStaticPlugin>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_detach(
            self as *mut crate::QVectorOfQStaticPlugin,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), returning <code>true</code> if the vector is empty; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QStaticPlugin>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), returning <code>true</code> if the vector is empty; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_empty(
            self as *const crate::QVectorOfQStaticPlugin,
        )
    }

    /// <p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStaticPlugin>& QVector<QStaticPlugin>::fill(const QStaticPlugin& t, int size = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">3</span>);
    ///   vector<span class="operator">.</span>fill(<span class="string">"Yes"</span>);
    ///   <span class="comment">// vector: ["Yes", "Yes", "Yes"]</span>
    ///
    ///   vector<span class="operator">.</span>fill(<span class="string">"oh"</span><span class="operator">,</span> <span class="number">5</span>);
    ///   <span class="comment">// vector: ["oh", "oh", "oh", "oh", "oh"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_2a(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStaticPlugin>>,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_fill(
            self as *mut crate::QVectorOfQStaticPlugin,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStaticPlugin>>::cast_into(t)
                .as_raw_ptr(),
            size,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStaticPlugin>& QVector<QStaticPlugin>::fill(const QStaticPlugin& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">3</span>);
    ///   vector<span class="operator">.</span>fill(<span class="string">"Yes"</span>);
    ///   <span class="comment">// vector: ["Yes", "Yes", "Yes"]</span>
    ///
    ///   vector<span class="operator">.</span>fill(<span class="string">"oh"</span><span class="operator">,</span> <span class="number">5</span>);
    ///   <span class="comment">// vector: ["oh", "oh", "oh", "oh", "oh"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_1a(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStaticPlugin>>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_fill1(
            self as *mut crate::QVectorOfQStaticPlugin,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStaticPlugin>>::cast_into(t)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStaticPlugin& QVector<QStaticPlugin>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#constFirst">constFirst</a>().</p></div>
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<crate::QStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_first(
            self as *mut crate::QVectorOfQStaticPlugin,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStaticPlugin& QVector<QStaticPlugin>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::QStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_first1(
            self as *const crate::QVectorOfQStaticPlugin,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStaticPlugin& QVector<QStaticPlugin>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::MutRef<crate::QStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_front(
            self as *mut crate::QVectorOfQStaticPlugin,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStaticPlugin& QVector<QStaticPlugin>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<crate::QStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_front1(
            self as *const crate::QVectorOfQStaticPlugin,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStaticPlugin& QVector<QStaticPlugin>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p>Note that using non-const operators can cause <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to do a deep copy.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_operator__2(
            self as *mut crate::QVectorOfQStaticPlugin,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStaticPlugin& QVector<QStaticPlugin>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as at(<i>i</i>).</p></div>
    pub unsafe fn index(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::QStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_operator__3(
            self as *const crate::QVectorOfQStaticPlugin,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the vector. If <i>i</i> is 0, the value is prepended to the vector. If <i>i</i> is <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), the value is appended to the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStaticPlugin>::insert(int i, const QStaticPlugin& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the vector. If <i>i</i> is 0, the value is prepended to the vector. If <i>i</i> is <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), the value is appended to the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   vector<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// vector: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p>For large vectors, this operation can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because it requires moving all the items at indexes <i>i</i> and above by one position further in memory. If you want a container class that provides a fast insert() function, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>().</p></div>
    pub unsafe fn insert_2a(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStaticPlugin>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_insert(
            self as *mut crate::QVectorOfQStaticPlugin,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStaticPlugin>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStaticPlugin>::insert(int i, int n, const QStaticPlugin& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>count</i> copies of <i>value</i> at index position <i>i</i> in the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">double</span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">2.718</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">1.442</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">0.4342</span>;
    ///   vector<span class="operator">.</span>insert(<span class="number">1</span><span class="operator">,</span> <span class="number">3</span><span class="operator">,</span> <span class="number">9.9</span>);
    ///   <span class="comment">// vector: [2.718, 9.9, 9.9, 9.9, 1.442, 0.4342]</span>
    ///
    /// </pre></div>
    pub unsafe fn insert_3a(
        &mut self,
        i: ::std::os::raw::c_int,
        n: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStaticPlugin>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_insert1(
            self as *mut crate::QVectorOfQStaticPlugin,
            i,
            n,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStaticPlugin>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QVector<QStaticPlugin>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_isDetached(
            self as *const crate::QVectorOfQStaticPlugin,
        )
    }

    /// <p>Returns <code>true</code> if the vector has size 0; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QStaticPlugin>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the vector has size 0; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_isEmpty(
            self as *const crate::QVectorOfQStaticPlugin,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QVector<QStaticPlugin>::isSharedWith(const QVector<QStaticPlugin>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQStaticPlugin>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_isSharedWith(
            self as *const crate::QVectorOfQStaticPlugin,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQStaticPlugin>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        )
    }

    /// <p>Returns a reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStaticPlugin& QVector<QStaticPlugin>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#constLast">constLast</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<crate::QStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_last(
            self as *mut crate::QVectorOfQStaticPlugin,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStaticPlugin& QVector<QStaticPlugin>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::QStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_last1(
            self as *const crate::QVectorOfQStaticPlugin,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QStaticPlugin>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#length">QList::length</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_length(
            self as *const crate::QVectorOfQStaticPlugin,
        )
    }

    /// <p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStaticPlugin> QVector<QStaticPlugin>::mid(int pos, int len = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        len: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_mid(
            self as *const crate::QVectorOfQStaticPlugin,
            pos,
            len,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStaticPlugin> QVector<QStaticPlugin>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_mid1(
            self as *const crate::QVectorOfQStaticPlugin,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStaticPlugin>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">QList::move</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_move(
            self as *mut crate::QVectorOfQStaticPlugin,
            from,
            to,
        )
    }

    /// <p>Constructs an empty vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QStaticPlugin>::QVector()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QVectorOfQStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_QVector();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a vector with an initial size of <i>size</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QStaticPlugin>::QVector(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a vector with an initial size of <i>size</i> elements.</p>
    /// <p>The elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn new_1a(
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_QVector1(size);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a vector with an initial size of <i>size</i> elements. Each element is initialized with <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QStaticPlugin>::QVector(int size, const QStaticPlugin& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a vector with an initial size of <i>size</i> elements. Each element is initialized with <i>value</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn new_2a(
        size: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStaticPlugin>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_QVector2(
            size,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStaticPlugin>>::cast_into(t)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QStaticPlugin>::QVector(const QVector<QStaticPlugin>& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        v: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQStaticPlugin>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_QVector3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQStaticPlugin>>::cast_into(v)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStaticPlugin>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_pop_back(
            self as *mut crate::QVectorOfQStaticPlugin,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStaticPlugin>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_pop_front(
            self as *mut crate::QVectorOfQStaticPlugin,
        )
    }

    /// <p>Inserts <i>value</i> at the beginning of the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStaticPlugin>::prepend(const QStaticPlugin& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   vector<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   vector<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// vector: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as vector.insert(0, <i>value</i>).</p>
    /// <p>For large vectors, this operation can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because it requires moving all the items in the vector by one position further in memory. If you want a container class that provides a fast prepend() function, use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> or <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStaticPlugin>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_prepend(
            self as *mut crate::QVectorOfQStaticPlugin,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStaticPlugin>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to append(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStaticPlugin>::push_back(const QStaticPlugin& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to append(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStaticPlugin>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_push_back(
            self as *mut crate::QVectorOfQStaticPlugin,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStaticPlugin>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to prepend(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStaticPlugin>::push_front(const QStaticPlugin& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to prepend(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStaticPlugin>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_push_front(
            self as *mut crate::QVectorOfQStaticPlugin,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStaticPlugin>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStaticPlugin>::remove(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#remove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes the element at index position <i>i</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn remove_1a(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_remove(
            self as *mut crate::QVectorOfQStaticPlugin,
            i,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStaticPlugin>::remove(int i, int n)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#remove-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes <i>count</i> elements from the middle of the vector, starting at index position <i>i</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn remove_2a(&mut self, i: ::std::os::raw::c_int, n: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_remove1(
            self as *mut crate::QVectorOfQStaticPlugin,
            i,
            n,
        )
    }

    /// <p>Removes the element at index position <i>i</i>. Equivalent to</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStaticPlugin>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the element at index position <i>i</i>. Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   remove(i);
    ///
    /// </pre>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">QList::removeAt</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_removeAt(
            self as *mut crate::QVectorOfQStaticPlugin,
            i,
        )
    }

    /// <p>Removes the first item in the vector. Calling this function is equivalent to calling remove(0). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStaticPlugin>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the vector. Calling this function is equivalent to calling remove(0). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_removeFirst(
            self as *mut crate::QVectorOfQStaticPlugin,
        )
    }

    /// <p>Removes the last item in the vector. Calling this function is equivalent to calling remove(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() - 1). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStaticPlugin>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the vector. Calling this function is equivalent to calling remove(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() - 1). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_removeLast(
            self as *mut crate::QVectorOfQStaticPlugin,
        )
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStaticPlugin>::replace(int i, const QStaticPlugin& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStaticPlugin>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_replace(
            self as *mut crate::QVectorOfQStaticPlugin,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStaticPlugin>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Attempts to allocate memory for at least <i>size</i> elements. If you know in advance how large the vector will be, you should call this function to prevent reallocations and memory fragmentation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStaticPlugin>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to allocate memory for at least <i>size</i> elements. If you know in advance how large the vector will be, you should call this function to prevent reallocations and memory fragmentation.</p>
    /// <p>If <i>size</i> is an underestimate, the worst that will happen is that the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> will be a bit slower. If <i>size</i> is an overestimate, you may have used more memory than the normal <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> growth strategy would have allocated—or you may have used less.</p>
    /// <p>An alternative to reserve() is calling <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>(). Whether or not that is faster than reserve() depends on the element type, because <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() default-constructs all elements, and requires assignment to existing entries rather than calling <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(), which copy- or move-constructs. For simple types, like <code>int</code> or <code>double</code>, <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() is typically faster, but for anything more complex, you should prefer reserve().</p>
    /// <p><b>Warning:</b> If the size passed to <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() was underestimated, you run out of allocated space and into undefined behavior. This problem does not exist with reserve(), because it treats the size as just a hint.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>().</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_reserve(
            self as *mut crate::QVectorOfQStaticPlugin,
            size,
        )
    }

    /// <p>Sets the size of the vector to <i>size</i>. If <i>size</i> is greater than the current size, elements are added to the end; the new elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If <i>size</i> is less than the current size, elements are removed from the end.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStaticPlugin>::resize(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#resize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the size of the vector to <i>size</i>. If <i>size</i> is greater than the current size, elements are added to the end; the new elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If <i>size</i> is less than the current size, elements are removed from the end.</p>
    /// <p>Since Qt 5.6, resize() doesn't shrink the capacity anymore. To shed excess capacity, use <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p></div>
    pub unsafe fn resize(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_resize(
            self as *mut crate::QVectorOfQStaticPlugin,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QVector<QStaticPlugin>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_setSharable(
            self as *mut crate::QVectorOfQStaticPlugin,
            sharable,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStaticPlugin>::shrink_to_fit()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#shrink_to_fit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn shrink_to_fit(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_shrink_to_fit(
            self as *mut crate::QVectorOfQStaticPlugin,
        )
    }

    /// <p>Returns the number of items in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QStaticPlugin>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_size(
            self as *const crate::QVectorOfQStaticPlugin,
        )
    }

    /// <p>Releases any memory not required to store the items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStaticPlugin>::squeeze()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Releases any memory not required to store the items.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s memory usage. In general, you will rarely ever need to call this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>().</p></div>
    pub unsafe fn squeeze(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_squeeze(
            self as *mut crate::QVectorOfQStaticPlugin,
        )
    }

    /// <p>Swaps vector <i>other</i> with this vector. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QStaticPlugin>::swap(QVector<QStaticPlugin>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps vector <i>other</i> with this vector. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QVectorOfQStaticPlugin>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_swap(
            self as *mut crate::QVectorOfQStaticPlugin,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QVectorOfQStaticPlugin>>::cast_into(
                other,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Removes the element at index position <i>i</i> and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStaticPlugin QVector<QStaticPlugin>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the element at index position <i>i</i> and returns it.</p>
    /// <p>Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   T t <span class="operator">=</span> at(i);
    ///   remove(i);
    ///   <span class="keyword">return</span> t;
    ///
    /// </pre>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">QList::takeAt</a>().</p></div>
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_takeAt(
            self as *mut crate::QVectorOfQStaticPlugin,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the first item in the vector and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStaticPlugin QVector<QStaticPlugin>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the vector and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> ::cpp_utils::CppBox<crate::QStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_takeFirst(
            self as *mut crate::QVectorOfQStaticPlugin,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the last item in the list and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStaticPlugin QVector<QStaticPlugin>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> ::cpp_utils::CppBox<crate::QStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_takeLast(
            self as *mut crate::QVectorOfQStaticPlugin,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value at index position <i>i</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStaticPlugin QVector<QStaticPlugin>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the vector.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that <i>i</i> is within bounds, you can use <a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_value(
            self as *const crate::QVectorOfQStaticPlugin,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStaticPlugin QVector<QStaticPlugin>::value(int i, const QStaticPlugin& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStaticPlugin>>,
    ) -> ::cpp_utils::CppBox<crate::QStaticPlugin> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_value1(
            self as *const crate::QVectorOfQStaticPlugin,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStaticPlugin>>::cast_into(
                default_value,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
///
/// C++ class: <span style='color: green;'>```QPair<double, QVariant>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qpair.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
/// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; can be used in your application if the STL <code>pair</code> type is not available. It stores one value of type T1 and one value of type T2. It can be used as a return value for a function that needs to return two values, or as the value type of a <a href="http://doc.qt.io/qt-5/containers.html">generic container</a>.</p>
/// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> that stores one <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and one <code>double</code> value:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qpair.html#QPair">QPair</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> pair;
///
/// </pre>
/// <p>The components are accessible as public data members called <a href="http://doc.qt.io/qt-5/qpair.html#first-var">first</a> and <a href="http://doc.qt.io/qt-5/qpair.html#second-var">second</a>. For example:</p>
/// <pre class="cpp">
///
///   pair<span class="operator">.</span>first <span class="operator">=</span> <span class="string">"pi"</span>;
///   pair<span class="operator">.</span>second <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
///
/// </pre>
/// <p>Note, however, that it is almost always preferable to define a small struct to hold the result of a function with multiple return values. A struct trivially generalizes to more than two values, and allows more descriptive member names than <code>first</code> and <code>second</code>:</p>
/// <pre class="cpp">
///
///   <span class="keyword">struct</span> Variable {
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> name;
/// &#32;     <span class="type">double</span> value;
///   };
///   Variable v;
///   v<span class="operator">.</span>name <span class="operator">=</span> <span class="string">"pi"</span>;
///   v<span class="operator">.</span>value <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
///
/// </pre>
/// <p>The advent of C++11 automatic variable type deduction (<code>auto</code>) shifts the emphasis from the type name to the name of functions and members. Thus, <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>, like <code>std::pair</code> and <code>std::tuple</code>, is mostly useful in generic (template) code, where defining a dedicated type is not possible.</p>
/// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>'s template data types (T1 and T2) must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; these requirements are documented on a per-function basis.</p></div>
#[repr(C)]
pub struct QPairOfDoubleQVariant {
    _unused: u8,
}
impl QPairOfDoubleQVariant {
    /// <p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPair<double, QVariant>& QPair<double, QVariant>::operator=(const QPair<double, QVariant>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; can be used in your application if the STL <code>pair</code> type is not available. It stores one value of type T1 and one value of type T2. It can be used as a return value for a function that needs to return two values, or as the value type of a <a href="http://doc.qt.io/qt-5/containers.html">generic container</a>.</p>
    /// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> that stores one <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and one <code>double</code> value:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpair.html#QPair">QPair</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> pair;
    ///
    /// </pre>
    /// <p>The components are accessible as public data members called <a href="http://doc.qt.io/qt-5/qpair.html#first-var">first</a> and <a href="http://doc.qt.io/qt-5/qpair.html#second-var">second</a>. For example:</p>
    /// <pre class="cpp">
    ///
    ///   pair<span class="operator">.</span>first <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   pair<span class="operator">.</span>second <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>Note, however, that it is almost always preferable to define a small struct to hold the result of a function with multiple return values. A struct trivially generalizes to more than two values, and allows more descriptive member names than <code>first</code> and <code>second</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">struct</span> Variable {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> name;
    /// &#32;     <span class="type">double</span> value;
    ///   };
    ///   Variable v;
    ///   v<span class="operator">.</span>name <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   v<span class="operator">.</span>value <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>The advent of C++11 automatic variable type deduction (<code>auto</code>) shifts the emphasis from the type name to the name of functions and members. Thus, <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>, like <code>std::pair</code> and <code>std::tuple</code>, is mostly useful in generic (template) code, where defining a dedicated type is not possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>'s template data types (T1 and T2) must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; these requirements are documented on a per-function basis.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>,
    ) -> ::cpp_utils::MutRef<crate::QPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPair_double_QVariant_operator_(
            self as *mut crate::QPairOfDoubleQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Constructs an empty pair. The <code>first</code> and <code>second</code> elements are initialized with <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>s.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPair<double, QVariant>::QPair()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html#QPair">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty pair. The <code>first</code> and <code>second</code> elements are initialized with <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>s.</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPair_double_QVariant_QPair();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a pair and initializes the <code>first</code> element with <i>value1</i> and the <code>second</code> element with <i>value2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPair<double, QVariant>::QPair(const double& t1, const QVariant& t2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html#QPair-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a pair and initializes the <code>first</code> element with <i>value1</i> and the <code>second</code> element with <i>value2</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpair.html#qMakePair">qMakePair</a>().</p></div>
    pub unsafe fn new_2a(
        t1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_double>>,
        t2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPair_double_QVariant_QPair1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_double>>::cast_into(t1)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(t2).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPair<double, QVariant>::QPair(const QPair<double, QVariant>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; can be used in your application if the STL <code>pair</code> type is not available. It stores one value of type T1 and one value of type T2. It can be used as a return value for a function that needs to return two values, or as the value type of a <a href="http://doc.qt.io/qt-5/containers.html">generic container</a>.</p>
    /// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> that stores one <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and one <code>double</code> value:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpair.html#QPair">QPair</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> pair;
    ///
    /// </pre>
    /// <p>The components are accessible as public data members called <a href="http://doc.qt.io/qt-5/qpair.html#first-var">first</a> and <a href="http://doc.qt.io/qt-5/qpair.html#second-var">second</a>. For example:</p>
    /// <pre class="cpp">
    ///
    ///   pair<span class="operator">.</span>first <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   pair<span class="operator">.</span>second <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>Note, however, that it is almost always preferable to define a small struct to hold the result of a function with multiple return values. A struct trivially generalizes to more than two values, and allows more descriptive member names than <code>first</code> and <code>second</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">struct</span> Variable {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> name;
    /// &#32;     <span class="type">double</span> value;
    ///   };
    ///   Variable v;
    ///   v<span class="operator">.</span>name <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   v<span class="operator">.</span>value <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>The advent of C++11 automatic variable type deduction (<code>auto</code>) shifts the emphasis from the type name to the name of functions and members. Thus, <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>, like <code>std::pair</code> and <code>std::tuple</code>, is mostly useful in generic (template) code, where defining a dedicated type is not possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>'s template data types (T1 and T2) must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; these requirements are documented on a per-function basis.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPair_double_QVariant_QPair2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Swaps this pair with <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QPair<double, QVariant>::swap(QPair<double, QVariant>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps this pair with <i>other</i>.</p>
    /// <p>Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms-obsolete.html#qSwap-1">qSwap</a>(<span class="keyword">this</span><span class="operator">-</span><span class="operator">&gt;</span>first<span class="operator">,</span> other<span class="operator">.</span>first);
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms-obsolete.html#qSwap-1">qSwap</a>(<span class="keyword">this</span><span class="operator">-</span><span class="operator">&gt;</span>second<span class="operator">,</span> other<span class="operator">.</span>second);
    ///
    /// </pre>
    /// <p>Swap overloads are found in namespace <code>std</code> as well as via argument-dependent lookup (ADL) in the namespace of <code>T</code> .</p>
    /// <p>This function was introduced in  Qt 5.5.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QPairOfDoubleQVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QPair_double_QVariant_swap(
            self as *mut crate::QPairOfDoubleQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QPairOfDoubleQVariant>>::cast_into(
                other,
            )
            .as_mut_raw_ptr(),
        )
    }
}

pub mod q_list_of_q_abstract_transition {
    //! C++ type: <span style='color: green;'>```QList<QAbstractTransition*>```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QAbstractTransition*>::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::iterator& QList<QAbstractTransition*>::iterator::operator=(const QList<QAbstractTransition*>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::Iterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_abstract_transition::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_iterator_operator_8(
                    self as *mut crate::q_list_of_q_abstract_transition::Iterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::Iterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::iterator QList<QAbstractTransition*>::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_iterator_operator__13(
                    self as *mut crate::q_list_of_q_abstract_transition::Iterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::iterator QList<QAbstractTransition*>::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_iterator_operator__11(
                    self as *mut crate::q_list_of_q_abstract_transition::Iterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QAbstractTransition*& QList<QAbstractTransition*>::iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValueRef iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>. If you assign to it, the assignment will apply to the character in the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> of <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> from which you got the reference.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::MutRef<*mut crate::QAbstractTransition> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_iterator_operator__1(
                    self as *const crate::q_list_of_q_abstract_transition::Iterator,
                    j,
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractTransition*>::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>().</p></div>
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::Iterator>
        {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractTransition*>::iterator::iterator(const QList<QAbstractTransition*>::iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::Iterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_iterator_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::Iterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QAbstractTransition** QList<QAbstractTransition*>::iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's value.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::MutPtr<*mut crate::QAbstractTransition> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_iterator_operator__(
                    self as *const crate::q_list_of_q_abstract_transition::Iterator,
                );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QAbstractTransition*>::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::const_iterator& QList<QAbstractTransition*>::const_iterator::operator=(const QList<QAbstractTransition*>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::ConstIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_abstract_transition::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_const_iterator_operator_6(self as *mut crate::q_list_of_q_abstract_transition::ConstIterator, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::ConstIterator>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::const_iterator QList<QAbstractTransition*>::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_const_iterator_operator__9(self as *mut crate::q_list_of_q_abstract_transition::ConstIterator, arg1);
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::const_iterator QList<QAbstractTransition*>::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_const_iterator_operator__7(self as *mut crate::q_list_of_q_abstract_transition::ConstIterator, arg1);
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractTransition*>::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>().</p></div>
        pub unsafe fn new_0a(
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_const_iterator_const_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractTransition*>::const_iterator::const_iterator(const QList<QAbstractTransition*>::iterator& o)```</span>.
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::Iterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_const_iterator_const_iterator2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractTransition*>::const_iterator::const_iterator(const QList<QAbstractTransition*>::const_iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::ConstIterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_const_iterator_const_iterator1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::ConstIterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_abstract_transition::Iterator {
        type Output = ::cpp_utils::MutRef<*mut crate::QAbstractTransition>;
        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QAbstractTransition*& QList<QAbstractTransition*>::iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>().</p></div>
        fn indirection(self) -> ::cpp_utils::MutRef<*mut crate::QAbstractTransition> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_iterator_operator_(
                    self as *const crate::q_list_of_q_abstract_transition::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::Iterator>>
        for crate::q_list_of_q_abstract_transition::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractTransition*>::iterator::operator==(const QList<QAbstractTransition*>::iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_iterator_operator__2(
                    self as *const crate::q_list_of_q_abstract_transition::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::Iterator>>
        for crate::q_list_of_q_abstract_transition::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractTransition*>::iterator::operator<(const QList<QAbstractTransition*>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_iterator_operator_1(
                    self as *const crate::q_list_of_q_abstract_transition::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::Iterator>>
        for crate::q_list_of_q_abstract_transition::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractTransition*>::iterator::operator<=(const QList<QAbstractTransition*>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_iterator_operator__4(
                    self as *const crate::q_list_of_q_abstract_transition::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::Iterator>>
        for crate::q_list_of_q_abstract_transition::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractTransition*>::iterator::operator>(const QList<QAbstractTransition*>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_iterator_operator_2(
                    self as *const crate::q_list_of_q_abstract_transition::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::Iterator>>
        for crate::q_list_of_q_abstract_transition::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractTransition*>::iterator::operator>=(const QList<QAbstractTransition*>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_iterator_operator__5(
                    self as *const crate::q_list_of_q_abstract_transition::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::std::cmp::PartialEq<
            ::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::ConstIterator>,
        > for crate::q_list_of_q_abstract_transition::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractTransition*>::iterator::operator==(const QList<QAbstractTransition*>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_iterator_operator__6(
                    self as *const crate::q_list_of_q_abstract_transition::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Lt<
            ::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::ConstIterator>,
        > for crate::q_list_of_q_abstract_transition::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractTransition*>::iterator::operator<(const QList<QAbstractTransition*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_iterator_operator_3(
                    self as *const crate::q_list_of_q_abstract_transition::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Le<
            ::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::ConstIterator>,
        > for crate::q_list_of_q_abstract_transition::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractTransition*>::iterator::operator<=(const QList<QAbstractTransition*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_iterator_operator__8(
                    self as *const crate::q_list_of_q_abstract_transition::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Gt<
            ::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::ConstIterator>,
        > for crate::q_list_of_q_abstract_transition::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractTransition*>::iterator::operator>(const QList<QAbstractTransition*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_iterator_operator_4(
                    self as *const crate::q_list_of_q_abstract_transition::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Ge<
            ::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::ConstIterator>,
        > for crate::q_list_of_q_abstract_transition::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractTransition*>::iterator::operator>=(const QList<QAbstractTransition*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_iterator_operator__9(
                    self as *const crate::q_list_of_q_abstract_transition::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_abstract_transition::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_abstract_transition::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::iterator& QList<QAbstractTransition*>::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_abstract_transition::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_iterator_operator__10(
                    self as *mut crate::q_list_of_q_abstract_transition::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_abstract_transition::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_abstract_transition::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::iterator& QList<QAbstractTransition*>::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_abstract_transition::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_iterator_operator__12(
                    self as *mut crate::q_list_of_q_abstract_transition::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_abstract_transition::Iterator
    {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::iterator& QList<QAbstractTransition*>::iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_iterator_operator__14(
                    self as *mut crate::q_list_of_q_abstract_transition::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_abstract_transition::Iterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::iterator& QList<QAbstractTransition*>::iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_iterator_operator__15(
                    self as *mut crate::q_list_of_q_abstract_transition::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_abstract_transition::Iterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::iterator QList<QAbstractTransition*>::iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_iterator_operator_5(
                    self as *const crate::q_list_of_q_abstract_transition::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_abstract_transition::Iterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::iterator QList<QAbstractTransition*>::iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_iterator_operator_6(
                    self as *const crate::q_list_of_q_abstract_transition::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::Iterator>>
        for &crate::q_list_of_q_abstract_transition::Iterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QAbstractTransition*>::iterator::operator-(QList<QAbstractTransition*>::iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::Iterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_iterator_operator_7(
                    self as *const crate::q_list_of_q_abstract_transition::Iterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::std::cmp::PartialEq<
            ::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::ConstIterator>,
        > for crate::q_list_of_q_abstract_transition::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractTransition*>::const_iterator::operator==(const QList<QAbstractTransition*>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_const_iterator_operator__2(self as *const crate::q_list_of_q_abstract_transition::ConstIterator, o.as_raw_ptr())
            }
        }
    }

    impl
        ::cpp_utils::cmp::Lt<
            ::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::ConstIterator>,
        > for crate::q_list_of_q_abstract_transition::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractTransition*>::const_iterator::operator<(const QList<QAbstractTransition*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_const_iterator_operator_1(self as *const crate::q_list_of_q_abstract_transition::ConstIterator, other.as_raw_ptr())
            }
        }
    }

    impl
        ::cpp_utils::cmp::Le<
            ::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::ConstIterator>,
        > for crate::q_list_of_q_abstract_transition::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractTransition*>::const_iterator::operator<=(const QList<QAbstractTransition*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_const_iterator_operator__4(self as *const crate::q_list_of_q_abstract_transition::ConstIterator, other.as_raw_ptr())
            }
        }
    }

    impl
        ::cpp_utils::cmp::Gt<
            ::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::ConstIterator>,
        > for crate::q_list_of_q_abstract_transition::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractTransition*>::const_iterator::operator>(const QList<QAbstractTransition*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_const_iterator_operator_2(self as *const crate::q_list_of_q_abstract_transition::ConstIterator, other.as_raw_ptr())
            }
        }
    }

    impl
        ::cpp_utils::cmp::Ge<
            ::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::ConstIterator>,
        > for crate::q_list_of_q_abstract_transition::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractTransition*>::const_iterator::operator>=(const QList<QAbstractTransition*>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_const_iterator_operator__5(self as *const crate::q_list_of_q_abstract_transition::ConstIterator, other.as_raw_ptr())
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_abstract_transition::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_abstract_transition::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::const_iterator& QList<QAbstractTransition*>::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_abstract_transition::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_const_iterator_operator__6(self as *mut crate::q_list_of_q_abstract_transition::ConstIterator)
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_abstract_transition::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_abstract_transition::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::const_iterator& QList<QAbstractTransition*>::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_abstract_transition::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_const_iterator_operator__8(self as *mut crate::q_list_of_q_abstract_transition::ConstIterator)
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_abstract_transition::ConstIterator
    {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::const_iterator& QList<QAbstractTransition*>::const_iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_const_iterator_operator__10(self as *mut crate::q_list_of_q_abstract_transition::ConstIterator, j)
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_abstract_transition::ConstIterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::const_iterator& QList<QAbstractTransition*>::const_iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_const_iterator_operator__11(self as *mut crate::q_list_of_q_abstract_transition::ConstIterator, j)
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_abstract_transition::ConstIterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::const_iterator QList<QAbstractTransition*>::const_iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_const_iterator_operator_3(self as *const crate::q_list_of_q_abstract_transition::ConstIterator, j)
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_abstract_transition::ConstIterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::const_iterator QList<QAbstractTransition*>::const_iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_const_iterator_operator_4(self as *const crate::q_list_of_q_abstract_transition::ConstIterator, j)
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::ConstIterator>>
        for &crate::q_list_of_q_abstract_transition::ConstIterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QAbstractTransition*>::const_iterator::operator-(QList<QAbstractTransition*>::const_iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::ConstIterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_const_iterator_operator_5(self as *const crate::q_list_of_q_abstract_transition::ConstIterator, j.as_raw_ptr())
            }
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_abstract_transition::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QAbstractTransition*>::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_iterator_diterator(
                self as *mut crate::q_list_of_q_abstract_transition::Iterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_abstract_transition::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QAbstractTransition*>::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_const_iterator_dconst_iterator(self as *mut crate::q_list_of_q_abstract_transition::ConstIterator)
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
///
/// C++ class: <span style='color: green;'>```QList<QAbstractTransition*>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlist.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores items in a list that provides fast index-based access and index-based insertions and removals.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Internally, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T if <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. Otherwise, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T* and the items are allocated on the heap.</p>
/// <p>The array representation allows very fast insertions and index-based access. The <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() operations are also very fast because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates memory at both ends of its internal array. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.</p>
/// <p>Note, however, that when the conditions specified above are not met, each append or insert of a new item requires allocating the new item on the heap, and this per item allocation will make <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> a better choice for use cases that do a lot of appending or inserting, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> can allocate memory for many items in a single heap allocation.</p>
/// <p>Note that the internal array only ever gets bigger over the life of the list. It never shrinks. The internal array is deallocated by the destructor and by the assignment operator, when one list is assigned to another.</p>
/// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerList;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span><span class="operator">&gt;</span> dateList;
///
/// </pre>
/// <p>Qt includes a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class that inherits <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>&gt; and adds a few convenience functions, such as <a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#filter">QStringList::filter</a>(). <a href="http://doc.qt.io/qt-5/qstring.html#split">QString::split</a>() creates QStringLists from strings.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> stores a list of items. The default constructor creates an empty list. To insert items into the list, you can use operator&lt;&lt;():</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"one"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"two"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"three"</span>;
///   <span class="comment">// list: ["one", "two", "three"]</span>
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(). In addition, it provides the following convenience functions: <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const lists, operator[]() returns a reference to the item and can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Bob"</span>)
/// &#32;     list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Robert"</span>;
///
/// </pre>
/// <p>Because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented as an array of pointers for types that are larger than a pointer or are not movable, this operation requires (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>). For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> list<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (list<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Jane"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Jane at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>A common requirement is to remove an item from a list and do something with it. For this, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(). Here's a loop that removes the items from a list one at a time and calls <code>delete</code> on them:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   <span class="keyword">while</span> (<span class="operator">!</span>list<span class="operator">.</span>isEmpty())
/// &#32;     <span class="keyword">delete</span> list<span class="operator">.</span>takeFirst();
///
/// </pre>
/// <p>Inserting and removing items at either end of the list is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> in most cases), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
/// <p>If you want to find all occurrences of a particular value in a list, use <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of a matching item if they find it; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> list<span class="operator">.</span>indexOf(<span class="string">"Jane"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Jane is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a list contains a particular value, use <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the list, use <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>(). If you want to replace all occurrences of a particular value with another, use <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlistiterator.html">QListIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablelistiterator.html">QMutableListIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented in such a way that direct index-based access is just as fast as using iterators.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <p>To make <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as efficient as possible, its member functions don't validate their input before using it. Except for <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), member functions always assume the list is <i>not</i> empty. Member functions that take index values as parameters always assume their index value parameters are in the valid range. This means <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> member functions can fail. If you define QT_NO_DEBUG when you compile, failures will not be detected. If you <i>don't</i> define QT_NO_DEBUG, failures will be detected using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT">Q_ASSERT</a>() or <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT_X">Q_ASSERT_X</a>() with an appropriate message.</p>
/// <p>To avoid failures when your list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling other member functions. If you must pass an index value that might not be in the valid range, check that it is less than the value returned by <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() but <i>not</i> less than 0.</p>
/// <a name="more-members"></a></div>
#[repr(C)]
pub struct QListOfQAbstractTransition {
    _unused: u8,
}
impl QListOfQAbstractTransition {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractTransition*>::append(const QList<QAbstractTransition*>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> list to this list.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQAbstractTransition>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_append1(self as *mut crate::QListOfQAbstractTransition, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQAbstractTransition>>::cast_into(t).as_raw_ptr())
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractTransition*& QList<QAbstractTransition*>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn back(&mut self) -> ::cpp_utils::MutRef<*mut crate::QAbstractTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_back(
            self as *mut crate::QListOfQAbstractTransition,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::const_iterator QList<QAbstractTransition*>::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_cbegin(
            self as *const crate::QListOfQAbstractTransition,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::const_iterator QList<QAbstractTransition*>::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn cend(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_cend(
            self as *const crate::QListOfQAbstractTransition,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all items from the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractTransition*>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all items from the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_clear(
            self as *mut crate::QListOfQAbstractTransition,
        )
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::const_iterator QList<QAbstractTransition*>::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_constBegin(
            self as *const crate::QListOfQAbstractTransition,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::const_iterator QList<QAbstractTransition*>::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn const_end(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_constEnd(
            self as *const crate::QListOfQAbstractTransition,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Assigns <i>other</i> to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>& QList<QAbstractTransition*>::operator=(const QList<QAbstractTransition*>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this list and returns a reference to this list.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQAbstractTransition>>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQAbstractTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_operator_(self as *mut crate::QListOfQAbstractTransition, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQAbstractTransition>>::cast_into(l).as_raw_ptr());
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QAbstractTransition*>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_count1(
            self as *const crate::QListOfQAbstractTransition,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractTransition*>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_detach(
            self as *mut crate::QListOfQAbstractTransition,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractTransition*>::detachShared()```</span>.
    pub unsafe fn detach_shared(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_detachShared(
            self as *mut crate::QListOfQAbstractTransition,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractTransition*>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_empty(
            self as *const crate::QListOfQAbstractTransition,
        )
    }

    /// <p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::iterator QList<QAbstractTransition*>::erase(QList<QAbstractTransition*>::iterator pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn erase_1a(
        &mut self,
        pos: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::Iterator>,
        >,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_erase(
            self as *mut crate::QListOfQAbstractTransition,
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::Iterator>,
            >::cast_into(pos)
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::iterator QList<QAbstractTransition*>::erase(QList<QAbstractTransition*>::iterator first, QList<QAbstractTransition*>::iterator last)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes all the items from <i>begin</i> up to (but not including) <i>end</i>. Returns an iterator to the same item that <i>end</i> referred to before the call.</p></div>
    pub unsafe fn erase_2a(
        &mut self,
        first: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::Iterator>,
        >,
        last: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::Iterator>,
        >,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_erase1(
            self as *mut crate::QListOfQAbstractTransition,
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::Iterator>,
            >::cast_into(first)
            .as_raw_ptr(),
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::q_list_of_q_abstract_transition::Iterator>,
            >::cast_into(last)
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractTransition*& QList<QAbstractTransition*>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn first(&mut self) -> ::cpp_utils::MutRef<*mut crate::QAbstractTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_first(
            self as *mut crate::QListOfQAbstractTransition,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractTransition*& QList<QAbstractTransition*>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn front(&mut self) -> ::cpp_utils::MutRef<*mut crate::QAbstractTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_front(
            self as *mut crate::QListOfQAbstractTransition,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractTransition*& QList<QAbstractTransition*>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If this function is called on a list that is currently being shared, it will trigger a copy of all elements. Otherwise, this function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>. If you do not want to modify the list you should use <a href="http://doc.qt.io/qt-5/qlist.html#at">QList::at</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>().</p></div>
    pub unsafe fn index(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<*mut crate::QAbstractTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_operator__3(
            self as *mut crate::QListOfQAbstractTransition,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractTransition*>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_isDetached(
            self as *const crate::QListOfQAbstractTransition,
        )
    }

    /// <p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractTransition*>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_isEmpty(
            self as *const crate::QListOfQAbstractTransition,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractTransition*>::isSharedWith(const QList<QAbstractTransition*>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQAbstractTransition>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_isSharedWith(self as *const crate::QListOfQAbstractTransition, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQAbstractTransition>>::cast_into(other).as_raw_ptr())
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractTransition*& QList<QAbstractTransition*>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn last(&mut self) -> ::cpp_utils::MutRef<*mut crate::QAbstractTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_last(
            self as *mut crate::QListOfQAbstractTransition,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QAbstractTransition*>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_length(
            self as *const crate::QListOfQAbstractTransition,
        )
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*> QList<QAbstractTransition*>::mid(int pos, int length = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        length: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_mid(
            self as *const crate::QListOfQAbstractTransition,
            pos,
            length,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*> QList<QAbstractTransition*>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_mid1(
            self as *const crate::QListOfQAbstractTransition,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractTransition*>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>move(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "C", "D", "E", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This is the same as insert(<i>to</i>, <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<i>from</i>)).This function assumes that both <i>from</i> and <i>to</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>from</i> and <i>to</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_move(
            self as *mut crate::QListOfQAbstractTransition,
            from,
            to,
        )
    }

    /// <p>Constructs an empty list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractTransition*>::QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty list.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QListOfQAbstractTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_QList();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QAbstractTransition*>::QList(const QList<QAbstractTransition*>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQAbstractTransition>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_QList1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQAbstractTransition>>::cast_into(l).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractTransition*>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_pop_back(
            self as *mut crate::QListOfQAbstractTransition,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractTransition*>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_pop_front(
            self as *mut crate::QListOfQAbstractTransition,
        )
    }

    /// <p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractTransition*>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_removeAt(
            self as *mut crate::QListOfQAbstractTransition,
            i,
        )
    }

    /// <p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractTransition*>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_removeFirst(
            self as *mut crate::QListOfQAbstractTransition,
        )
    }

    /// <p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractTransition*>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_removeLast(
            self as *mut crate::QListOfQAbstractTransition,
        )
    }

    /// <p>Reserve space for <i>alloc</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractTransition*>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reserve space for <i>alloc</i> elements.</p>
    /// <p>If <i>alloc</i> is smaller than the current size of the list, nothing will happen.</p>
    /// <p>Use this function to avoid repetetive reallocation of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s internal data if you can predict how many elements will be appended. Note that the reservation applies only to the internal pointer array.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_reserve(
            self as *mut crate::QListOfQAbstractTransition,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractTransition*>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_setSharable(
            self as *mut crate::QListOfQAbstractTransition,
            sharable,
        )
    }

    /// <p>Returns the number of items in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QAbstractTransition*>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_size(
            self as *const crate::QListOfQAbstractTransition,
        )
    }

    /// <p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractTransition*>::swap(QList<QAbstractTransition*>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap_1a(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QListOfQAbstractTransition>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_swap(self as *mut crate::QListOfQAbstractTransition, ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QListOfQAbstractTransition>>::cast_into(other).as_mut_raw_ptr())
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractTransition*>::swap(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swap(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    pub unsafe fn swap_2a(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_swap1(
            self as *mut crate::QListOfQAbstractTransition,
            i,
            j,
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QAbstractTransition*>::swapItemsAt(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swapItemsAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swapItemsAt(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn swap_items_at(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_swapItemsAt(
            self as *mut crate::QListOfQAbstractTransition,
            i,
            j,
        )
    }

    /// <p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractTransition* QList<QAbstractTransition*>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_takeAt(
            self as *mut crate::QListOfQAbstractTransition,
            i,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractTransition* QList<QAbstractTransition*>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> ::cpp_utils::MutPtr<crate::QAbstractTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_takeFirst(
            self as *mut crate::QListOfQAbstractTransition,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractTransition* QList<QAbstractTransition*>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> ::cpp_utils::MutPtr<crate::QAbstractTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_takeLast(
            self as *mut crate::QListOfQAbstractTransition,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns the value at index position <i>i</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAbstractTransition* QList<QAbstractTransition*>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the list.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that the index is going to be within bounds, you can use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_value(
            self as *const crate::QListOfQAbstractTransition,
            i,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

pub mod q_set_of_q_abstract_state {
    //! C++ type: <span style='color: green;'>```QSet<QAbstractState*>```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qset-iterator.html">QSet::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QSet<QAbstractState*>::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qset-iterator.html">QSet::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qset.html">QSet</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qset.html">QSet</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> and to remove items (using <a href="http://doc.qt.io/qt-5/qset.html#erase">QSet::erase</a>()) while you iterate. (<a href="http://doc.qt.io/qt-5/qset.html">QSet</a> doesn't let you <i>modify</i> a value through an iterator, because that would potentially require moving the value in the internal hash table used by <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>.) If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>, you should use <a href="http://doc.qt.io/qt-5/qset-const-iterator.html">QSet::const_iterator</a>. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qset-const-iterator.html">QSet::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qset-iterator.html">QSet::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a function like <a href="http://doc.qt.io/qt-5/qset.html#begin">QSet::begin</a>(), <a href="http://doc.qt.io/qt-5/qset.html#end">QSet::end</a>(), or <a href="http://doc.qt.io/qt-5/qset.html#insert">QSet::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a set:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> set;
    ///   set <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"January"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"February"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"December"</span>;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> set<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> set<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i;
    ///
    /// </pre>
    /// <p>Here's a loop that removes certain items (all those that start with 'J') from a set while iterating:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> set;
    ///   set <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"January"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"February"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"December"</span>;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> set<span class="operator">.</span>begin();
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> set<span class="operator">.</span>end()) {
    /// &#32;     <span class="keyword">if</span> ((<span class="operator">*</span>i)<span class="operator">.</span>startsWith(<span class="char">'J'</span>)) {
    /// &#32;   &#32;     i <span class="operator">=</span> set<span class="operator">.</span>erase(i);
    /// &#32;     } <span class="keyword">else</span> {
    /// &#32;   &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    /// &#32;     }
    ///   }
    ///
    /// </pre>
    /// <p>STL-style iterators can be used as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>. For example, here's how to find an item in the set using the qFind() algorithm:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> set;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator it <span class="operator">=</span> <a href="http://doc.qt.io/qt-5/qtalgorithms-obsolete.html#qFind">qFind</a>(set<span class="operator">.</span>begin()<span class="operator">,</span> set<span class="operator">.</span>end()<span class="operator">,</span> <span class="string">"Jeanette"</span>);
    ///   <span class="keyword">if</span> (it <span class="operator">!</span><span class="operator">=</span> set<span class="operator">.</span>end())
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Jeanette"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same set.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// <p>Assigns <i>other</i> to this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::iterator& QSet<QAbstractState*>::iterator::operator=(const QSet<QAbstractState*>::iterator& o)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlinkedlist-iterator.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this iterator.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_set_of_q_abstract_state::Iterator>>,
        ) -> ::cpp_utils::MutRef<crate::q_set_of_q_abstract_state::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_iterator_operator_(
                    self as *mut crate::q_set_of_q_abstract_state::Iterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_set_of_q_abstract_state::Iterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::iterator QSet<QAbstractState*>::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_iterator_operator__8(
                    self as *mut crate::q_set_of_q_abstract_state::Iterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::iterator QSet<QAbstractState*>::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_iterator_operator__6(
                    self as *mut crate::q_set_of_q_abstract_state::Iterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QSet<QAbstractState*>::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qset-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#begin">QSet::begin</a>() and <a href="http://doc.qt.io/qt-5/qset.html#end">QSet::end</a>().</p></div>
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QSet<QAbstractState*>::iterator::iterator(const QSet<QAbstractState*>::iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_set_of_q_abstract_state::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_iterator_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_set_of_q_abstract_state::Iterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qset-const-iterator.html">QSet::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QSet<QAbstractState*>::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qset-const-iterator.html">QSet::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qset.html">QSet</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qset.html">QSet</a>&lt;Key, T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>. If you want to modify the <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> as you iterate over it, you must use <a href="http://doc.qt.io/qt-5/qset-iterator.html">QSet::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qset-const-iterator.html">QSet::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qset-const-iterator.html">QSet::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a function like <a href="http://doc.qt.io/qt-5/qset.html#begin">QSet::begin</a>(), <a href="http://doc.qt.io/qt-5/qset.html#end">QSet::end</a>(), or <a href="http://doc.qt.io/qt-5/qset.html#insert">QSet::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a set:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> set;
    ///   set <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"January"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"February"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"December"</span>;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> set<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> set<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i;
    ///
    /// </pre>
    /// <p>STL-style iterators can be used as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>. For example, here's how to find an item in the set using the qFind() algorithm:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> set;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator it <span class="operator">=</span> <a href="http://doc.qt.io/qt-5/qtalgorithms-obsolete.html#qFind">qFind</a>(set<span class="operator">.</span>begin()<span class="operator">,</span> set<span class="operator">.</span>end()<span class="operator">,</span> <span class="string">"Jeanette"</span>);
    ///   <span class="keyword">if</span> (it <span class="operator">!</span><span class="operator">=</span> set<span class="operator">.</span>constEnd())
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Jeanette"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>Assigns <i>other</i> to this.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::const_iterator& QSet<QAbstractState*>::const_iterator::operator=(const QSet<QAbstractState*>::const_iterator& o)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_set_of_q_abstract_state::ConstIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_set_of_q_abstract_state::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_const_iterator_operator_(
                    self as *mut crate::q_set_of_q_abstract_state::ConstIterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_set_of_q_abstract_state::ConstIterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::const_iterator QSet<QAbstractState*>::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_const_iterator_operator__6(
                    self as *mut crate::q_set_of_q_abstract_state::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::const_iterator QSet<QAbstractState*>::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_const_iterator_operator__4(
                    self as *mut crate::q_set_of_q_abstract_state::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QSet<QAbstractState*>::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qset-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#begin">QSet::begin</a>() and <a href="http://doc.qt.io/qt-5/qset.html#end">QSet::end</a>().</p></div>
        pub unsafe fn new_0a(
        ) -> ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_const_iterator_const_iterator(
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QSet<QAbstractState*>::const_iterator::const_iterator(const QSet<QAbstractState*>::iterator& o)```</span>.
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_set_of_q_abstract_state::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_const_iterator_const_iterator2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_set_of_q_abstract_state::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QSet<QAbstractState*>::const_iterator::const_iterator(const QSet<QAbstractState*>::const_iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_set_of_q_abstract_state::ConstIterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_const_iterator_const_iterator1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_set_of_q_abstract_state::ConstIterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_set_of_q_abstract_state::Iterator>>
        for crate::q_set_of_q_abstract_state::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QSet<QAbstractState*>::iterator::operator==(const QSet<QAbstractState*>::iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_set_of_q_abstract_state::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_iterator_operator__1(
                    self as *const crate::q_set_of_q_abstract_state::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_set_of_q_abstract_state::ConstIterator>>
        for crate::q_set_of_q_abstract_state::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QSet<QAbstractState*>::iterator::operator==(const QSet<QAbstractState*>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_set_of_q_abstract_state::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_iterator_operator__3(
                    self as *const crate::q_set_of_q_abstract_state::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_set_of_q_abstract_state::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_set_of_q_abstract_state::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::iterator& QSet<QAbstractState*>::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_set_of_q_abstract_state::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_iterator_operator__5(
                    self as *mut crate::q_set_of_q_abstract_state::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_set_of_q_abstract_state::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_set_of_q_abstract_state::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::iterator& QSet<QAbstractState*>::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_set_of_q_abstract_state::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_iterator_operator__7(
                    self as *mut crate::q_set_of_q_abstract_state::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_int> for &crate::q_set_of_q_abstract_state::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::Iterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions forward from this iterator. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::iterator QSet<QAbstractState*>::iterator::operator+(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions forward from this iterator. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p>This operation can be slow for large <i>j</i> values.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn add(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_iterator_operator_2(
                    self as *const crate::q_set_of_q_abstract_state::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_int> for &crate::q_set_of_q_abstract_state::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::Iterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions backward from this iterator. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::iterator QSet<QAbstractState*>::iterator::operator-(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions backward from this iterator. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p>This operation can be slow for large <i>j</i> values.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn sub(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_iterator_operator_3(
                    self as *const crate::q_set_of_q_abstract_state::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_int> for crate::q_set_of_q_abstract_state::Iterator {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::iterator& QSet<QAbstractState*>::iterator::operator+=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_iterator_operator__9(
                    self as *mut crate::q_set_of_q_abstract_state::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_int> for crate::q_set_of_q_abstract_state::Iterator {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::iterator& QSet<QAbstractState*>::iterator::operator-=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_iterator_operator__10(
                    self as *mut crate::q_set_of_q_abstract_state::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_set_of_q_abstract_state::ConstIterator>>
        for crate::q_set_of_q_abstract_state::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QSet<QAbstractState*>::const_iterator::operator==(const QSet<QAbstractState*>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_set_of_q_abstract_state::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_const_iterator_operator__1(
                    self as *const crate::q_set_of_q_abstract_state::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_set_of_q_abstract_state::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_set_of_q_abstract_state::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::const_iterator& QSet<QAbstractState*>::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_set_of_q_abstract_state::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_const_iterator_operator__3(
                    self as *mut crate::q_set_of_q_abstract_state::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_set_of_q_abstract_state::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_set_of_q_abstract_state::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::const_iterator& QSet<QAbstractState*>::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_set_of_q_abstract_state::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_const_iterator_operator__5(
                    self as *mut crate::q_set_of_q_abstract_state::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_int> for &crate::q_set_of_q_abstract_state::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::ConstIterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions forward from this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::const_iterator QSet<QAbstractState*>::const_iterator::operator+(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions forward from this iterator.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>().</p></div>
        fn add(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_const_iterator_operator_2(
                    self as *const crate::q_set_of_q_abstract_state::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_int> for &crate::q_set_of_q_abstract_state::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::ConstIterator>;
        /// <p>Returns an iterator to the item at <i>j</i> positions backward from this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::const_iterator QSet<QAbstractState*>::const_iterator::operator-(int j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at <i>j</i> positions backward from this iterator.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>().</p></div>
        fn sub(
            self,
            j: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_const_iterator_operator_3(
                    self as *const crate::q_set_of_q_abstract_state::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_int>
        for crate::q_set_of_q_abstract_state::ConstIterator
    {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::const_iterator& QSet<QAbstractState*>::const_iterator::operator+=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_const_iterator_operator__7(
                    self as *mut crate::q_set_of_q_abstract_state::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_int>
        for crate::q_set_of_q_abstract_state::ConstIterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::const_iterator& QSet<QAbstractState*>::const_iterator::operator-=(int j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_int) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_const_iterator_operator__8(
                    self as *mut crate::q_set_of_q_abstract_state::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_set_of_q_abstract_state::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qset-iterator.html">QSet::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QSet<QAbstractState*>::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qset-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qset-iterator.html">QSet::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qset.html">QSet</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qset.html">QSet</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> and to remove items (using <a href="http://doc.qt.io/qt-5/qset.html#erase">QSet::erase</a>()) while you iterate. (<a href="http://doc.qt.io/qt-5/qset.html">QSet</a> doesn't let you <i>modify</i> a value through an iterator, because that would potentially require moving the value in the internal hash table used by <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>.) If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>, you should use <a href="http://doc.qt.io/qt-5/qset-const-iterator.html">QSet::const_iterator</a>. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qset-const-iterator.html">QSet::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qset-iterator.html">QSet::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a function like <a href="http://doc.qt.io/qt-5/qset.html#begin">QSet::begin</a>(), <a href="http://doc.qt.io/qt-5/qset.html#end">QSet::end</a>(), or <a href="http://doc.qt.io/qt-5/qset.html#insert">QSet::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a set:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> set;
        ///   set <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"January"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"February"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"December"</span>;
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> set<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> set<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i;
        ///
        /// </pre>
        /// <p>Here's a loop that removes certain items (all those that start with 'J') from a set while iterating:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> set;
        ///   set <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"January"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"February"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"December"</span>;
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i <span class="operator">=</span> set<span class="operator">.</span>begin();
        ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> set<span class="operator">.</span>end()) {
        /// &#32;     <span class="keyword">if</span> ((<span class="operator">*</span>i)<span class="operator">.</span>startsWith(<span class="char">'J'</span>)) {
        /// &#32;   &#32;     i <span class="operator">=</span> set<span class="operator">.</span>erase(i);
        /// &#32;     } <span class="keyword">else</span> {
        /// &#32;   &#32;     <span class="operator">+</span><span class="operator">+</span>i;
        /// &#32;     }
        ///   }
        ///
        /// </pre>
        /// <p>STL-style iterators can be used as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>. For example, here's how to find an item in the set using the qFind() algorithm:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> set;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator it <span class="operator">=</span> <a href="http://doc.qt.io/qt-5/qtalgorithms-obsolete.html#qFind">qFind</a>(set<span class="operator">.</span>begin()<span class="operator">,</span> set<span class="operator">.</span>end()<span class="operator">,</span> <span class="string">"Jeanette"</span>);
        ///   <span class="keyword">if</span> (it <span class="operator">!</span><span class="operator">=</span> set<span class="operator">.</span>end())
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Jeanette"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same set.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_iterator_diterator(
                self as *mut crate::q_set_of_q_abstract_state::Iterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_set_of_q_abstract_state::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qset-const-iterator.html">QSet::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QSet<QAbstractState*>::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qset-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qset-const-iterator.html">QSet::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qset.html">QSet</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qset.html">QSet</a>&lt;Key, T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>. If you want to modify the <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> as you iterate over it, you must use <a href="http://doc.qt.io/qt-5/qset-iterator.html">QSet::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qset-const-iterator.html">QSet::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qset-const-iterator.html">QSet::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a function like <a href="http://doc.qt.io/qt-5/qset.html#begin">QSet::begin</a>(), <a href="http://doc.qt.io/qt-5/qset.html#end">QSet::end</a>(), or <a href="http://doc.qt.io/qt-5/qset.html#insert">QSet::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a set:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> set;
        ///   set <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"January"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"February"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"December"</span>;
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> set<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> set<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i;
        ///
        /// </pre>
        /// <p>STL-style iterators can be used as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>. For example, here's how to find an item in the set using the qFind() algorithm:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> set;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator it <span class="operator">=</span> <a href="http://doc.qt.io/qt-5/qtalgorithms-obsolete.html#qFind">qFind</a>(set<span class="operator">.</span>begin()<span class="operator">,</span> set<span class="operator">.</span>end()<span class="operator">,</span> <span class="string">"Jeanette"</span>);
        ///   <span class="keyword">if</span> (it <span class="operator">!</span><span class="operator">=</span> set<span class="operator">.</span>constEnd())
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Jeanette"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_const_iterator_dconst_iterator(
                self as *mut crate::q_set_of_q_abstract_state::ConstIterator,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> class is a template class that provides a hash-table-based set.</p>
///
/// C++ class: <span style='color: green;'>```QSet<QAbstractState*>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qset.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> class is a template class that provides a hash-table-based set.</p>
/// <p><a href="http://doc.qt.io/qt-5/qset.html">QSet</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores values in an unspecified order and provides very fast lookup of the values. Internally, <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>&lt;T&gt; is implemented as a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>.</p>
/// <p>Here's an example <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> with <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> set;
///
/// </pre>
/// <p>To insert a value into the set, use <a href="http://doc.qt.io/qt-5/qset.html#insert">insert</a>():</p>
/// <pre class="cpp">
///
///   set<span class="operator">.</span>insert(<span class="string">"one"</span>);
///   set<span class="operator">.</span>insert(<span class="string">"three"</span>);
///   set<span class="operator">.</span>insert(<span class="string">"seven"</span>);
///
/// </pre>
/// <p>Another way to insert items into the set is to use operator&lt;&lt;():</p>
/// <pre class="cpp">
///
///   set <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"twelve"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"fifteen"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"nineteen"</span>;
///
/// </pre>
/// <p>To test whether an item belongs to the set or not, use <a href="http://doc.qt.io/qt-5/qset.html#contains">contains</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (<span class="operator">!</span>set<span class="operator">.</span>contains(<span class="string">"ninety-nine"</span>))
/// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///
/// </pre>
/// <p>If you want to navigate through all the values stored in a <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>, you can use an iterator. <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> supports both <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qsetiterator.html">QSetIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablesetiterator.html">QMutableSetIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qset-iterator.html">QSet::iterator</a> and <a href="http://doc.qt.io/qt-5/qset-const-iterator.html">QSet::const_iterator</a>). Here's how to iterate over a <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt; using a Java-style iterator:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qsetiterator.html">QSetIterator</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> i(set);
///   <span class="keyword">while</span> (i<span class="operator">.</span>hasNext())
/// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>next();
///
/// </pre>
/// <p>Here's the same code, but using an STL-style iterator:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i <span class="operator">=</span> set<span class="operator">.</span>constBegin();
///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> set<span class="operator">.</span>constEnd()) {
/// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i;
/// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qset.html">QSet</a> is unordered, so an iterator's sequence cannot be assumed to be predictable. If ordering by key is required, use a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>.</p>
/// <p>To navigate through a <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>, you can also use <a href="http://doc.qt.io/qt-5/containers.html#foreach">foreach</a>:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> set;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   foreach (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>value<span class="operator">,</span> set)
/// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> value;
///
/// </pre>
/// <p>Items can be removed from the set using <a href="http://doc.qt.io/qt-5/qset.html#remove">remove</a>(). There is also a <a href="http://doc.qt.io/qt-5/qset.html#clear">clear</a>() function that removes all items.</p>
/// <p><a href="http://doc.qt.io/qt-5/qset.html">QSet</a>'s value data type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. In addition, the type must provide <code>operator==()</code>, and there must also be a global <a href="http://doc.qt.io/qt-5/qhash.html#qHash">qHash</a>() function that returns a hash value for an argument of the key's type. See the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> documentation for a list of types supported by <a href="http://doc.qt.io/qt-5/qhash.html#qHash">qHash</a>().</p>
/// <p>Internally, <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> uses a hash table to perform lookups. The hash table automatically grows and shrinks to provide fast lookups without wasting memory. You can still control the size of the hash table by calling <a href="http://doc.qt.io/qt-5/qset.html#reserve">reserve</a>(), if you already know approximately how many elements the <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> will contain, but this isn't necessary to obtain good performance. You can also call <a href="http://doc.qt.io/qt-5/qset.html#capacity">capacity</a>() to retrieve the hash table's size.</p></div>
#[repr(C)]
pub struct QSetOfQAbstractState {
    _unused: u8,
}
impl QSetOfQAbstractState {
    /// <p>Returns the number of buckets in the set's internal hash table.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QSet<QAbstractState*>::capacity() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#capacity">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of buckets in the set's internal hash table.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>'s memory usage. In general, you will rarely ever need to call this function. If you want to know how many items are in the set, call <a href="http://doc.qt.io/qt-5/qset.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qset.html#squeeze">squeeze</a>().</p></div>
    pub unsafe fn capacity(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_capacity(
            self as *const crate::QSetOfQAbstractState,
        )
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> positioned at the first item in the set.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::const_iterator QSet<QAbstractState*>::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> positioned at the first item in the set.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qset.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_cbegin(
            self as *const crate::QSetOfQAbstractState,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the set.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::const_iterator QSet<QAbstractState*>::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the set.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qset.html#end">end</a>().</p></div>
    pub unsafe fn cend(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_cend(
            self as *const crate::QSetOfQAbstractState,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all elements from the set.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSet<QAbstractState*>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all elements from the set.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#remove">remove</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_clear(
            self as *mut crate::QSetOfQAbstractState,
        )
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> positioned at the first item in the set.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::const_iterator QSet<QAbstractState*>::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> positioned at the first item in the set.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qset.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_constBegin(
            self as *const crate::QSetOfQAbstractState,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the set.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::const_iterator QSet<QAbstractState*>::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the set.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qset.html#end">end</a>().</p></div>
    pub unsafe fn const_end(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_constEnd(
            self as *const crate::QSetOfQAbstractState,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns <code>true</code> if the set contains all items from the <i>other</i> set; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSet<QAbstractState*>::contains(const QSet<QAbstractState*>& set) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#contains-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the set contains all items from the <i>other</i> set; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qset.html#remove">remove</a>(), and <a href="http://doc.qt.io/qt-5/qset.html#find">find</a>().</p></div>
    pub unsafe fn contains(
        &self,
        set: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSetOfQAbstractState>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_contains1(
            self as *const crate::QSetOfQAbstractState,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSetOfQAbstractState>>::cast_into(set)
                .as_raw_ptr(),
        )
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> class is a template class that provides a hash-table-based set.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>& QSet<QAbstractState*>::operator=(const QSet<QAbstractState*>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> class is a template class that provides a hash-table-based set.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qset.html">QSet</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores values in an unspecified order and provides very fast lookup of the values. Internally, <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>&lt;T&gt; is implemented as a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>.</p>
    /// <p>Here's an example <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> with <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> values:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> set;
    ///
    /// </pre>
    /// <p>To insert a value into the set, use <a href="http://doc.qt.io/qt-5/qset.html#insert">insert</a>():</p>
    /// <pre class="cpp">
    ///
    ///   set<span class="operator">.</span>insert(<span class="string">"one"</span>);
    ///   set<span class="operator">.</span>insert(<span class="string">"three"</span>);
    ///   set<span class="operator">.</span>insert(<span class="string">"seven"</span>);
    ///
    /// </pre>
    /// <p>Another way to insert items into the set is to use operator&lt;&lt;():</p>
    /// <pre class="cpp">
    ///
    ///   set <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"twelve"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"fifteen"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"nineteen"</span>;
    ///
    /// </pre>
    /// <p>To test whether an item belongs to the set or not, use <a href="http://doc.qt.io/qt-5/qset.html#contains">contains</a>():</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">if</span> (<span class="operator">!</span>set<span class="operator">.</span>contains(<span class="string">"ninety-nine"</span>))
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///
    /// </pre>
    /// <p>If you want to navigate through all the values stored in a <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>, you can use an iterator. <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> supports both <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qsetiterator.html">QSetIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablesetiterator.html">QMutableSetIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qset-iterator.html">QSet::iterator</a> and <a href="http://doc.qt.io/qt-5/qset-const-iterator.html">QSet::const_iterator</a>). Here's how to iterate over a <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt; using a Java-style iterator:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsetiterator.html">QSetIterator</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> i(set);
    ///   <span class="keyword">while</span> (i<span class="operator">.</span>hasNext())
    /// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>next();
    ///
    /// </pre>
    /// <p>Here's the same code, but using an STL-style iterator:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i <span class="operator">=</span> set<span class="operator">.</span>constBegin();
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> set<span class="operator">.</span>constEnd()) {
    /// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i;
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    ///   }
    ///
    /// </pre>
    /// <p><a href="http://doc.qt.io/qt-5/qset.html">QSet</a> is unordered, so an iterator's sequence cannot be assumed to be predictable. If ordering by key is required, use a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>.</p>
    /// <p>To navigate through a <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>, you can also use <a href="http://doc.qt.io/qt-5/containers.html#foreach">foreach</a>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> set;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   foreach (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>value<span class="operator">,</span> set)
    /// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> value;
    ///
    /// </pre>
    /// <p>Items can be removed from the set using <a href="http://doc.qt.io/qt-5/qset.html#remove">remove</a>(). There is also a <a href="http://doc.qt.io/qt-5/qset.html#clear">clear</a>() function that removes all items.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qset.html">QSet</a>'s value data type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. In addition, the type must provide <code>operator==()</code>, and there must also be a global <a href="http://doc.qt.io/qt-5/qhash.html#qHash">qHash</a>() function that returns a hash value for an argument of the key's type. See the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> documentation for a list of types supported by <a href="http://doc.qt.io/qt-5/qhash.html#qHash">qHash</a>().</p>
    /// <p>Internally, <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> uses a hash table to perform lookups. The hash table automatically grows and shrinks to provide fast lookups without wasting memory. You can still control the size of the hash table by calling <a href="http://doc.qt.io/qt-5/qset.html#reserve">reserve</a>(), if you already know approximately how many elements the <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> will contain, but this isn't necessary to obtain good performance. You can also call <a href="http://doc.qt.io/qt-5/qset.html#capacity">capacity</a>() to retrieve the hash table's size.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSetOfQAbstractState>>,
    ) -> ::cpp_utils::MutRef<crate::QSetOfQAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_operator_4(
            self as *mut crate::QSetOfQAbstractState,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSetOfQAbstractState>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Same as <a href="http://doc.qt.io/qt-5/qset.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QSet<QAbstractState*>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as <a href="http://doc.qt.io/qt-5/qset.html#size">size</a>().</p></div>
    pub unsafe fn count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_count(
            self as *const crate::QSetOfQAbstractState,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QSet<QAbstractState*>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_detach(
            self as *mut crate::QSetOfQAbstractState,
        )
    }

    /// <p>Returns <code>true</code> if the set is empty. This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qset.html#isEmpty">isEmpty</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSet<QAbstractState*>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the set is empty. This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qset.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_empty(
            self as *const crate::QSetOfQAbstractState,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::iterator QSet<QAbstractState*>::erase(QSet<QAbstractState*>::iterator i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#erase-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 4.2.</p></div>
    pub unsafe fn erase_iterator(
        &mut self,
        i: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_set_of_q_abstract_state::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_erase(self as *mut crate::QSetOfQAbstractState, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_set_of_q_abstract_state::Iterator>>::cast_into(i).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the item at the iterator position <i>pos</i> from the set, and returns an iterator positioned at the next item in the set.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::iterator QSet<QAbstractState*>::erase(QSet<QAbstractState*>::const_iterator i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at the iterator position <i>pos</i> from the set, and returns an iterator positioned at the next item in the set.</p>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qset.html#remove">remove</a>(), this function never causes <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> to rehash its internal data structure. This means that it can safely be called while iterating, and won't affect the order of items in the set.</p>
    /// <p>This function was introduced in  Qt 5.7.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#remove">remove</a>() and <a href="http://doc.qt.io/qt-5/qset.html#find">find</a>().</p></div>
    pub unsafe fn erase_const_iterator(
        &mut self,
        i: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_set_of_q_abstract_state::ConstIterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_erase1(
            self as *mut crate::QSetOfQAbstractState,
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::q_set_of_q_abstract_state::ConstIterator>,
            >::cast_into(i)
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a new <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> object containing the data contained in <i>list</i>. Since <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> doesn't allow duplicates, the resulting <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> might be smaller than the <i>list</i>, because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> can contain duplicates.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QSet<QAbstractState*> QSet<QAbstractState*>::fromList(const QList<QAbstractState*>& list)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#fromList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a new <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> object containing the data contained in <i>list</i>. Since <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> doesn't allow duplicates, the resulting <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> might be smaller than the <i>list</i>, because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> can contain duplicates.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Julia"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Mike"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Mike"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Julia"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Julia"</span>;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> set <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>fromList(list);
    ///   set<span class="operator">.</span>contains(<span class="string">"Julia"</span>);  <span class="comment">// returns true</span>
    ///   set<span class="operator">.</span>contains(<span class="string">"Mike"</span>);   <span class="comment">// returns true</span>
    ///   set<span class="operator">.</span>size();             <span class="comment">// returns 2</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#toList">toList</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#toSet">QList::toSet</a>().</p></div>
    pub unsafe fn from_list(
        list: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQAbstractState>>,
    ) -> ::cpp_utils::CppBox<crate::QSetOfQAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_fromList(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQAbstractState>>::cast_into(
                list,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all items from this set that are not contained in the <i>other</i> set. A reference to this set is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>& QSet<QAbstractState*>::intersect(const QSet<QAbstractState*>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#intersect">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all items from this set that are not contained in the <i>other</i> set. A reference to this set is returned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#intersects">intersects</a>(), <a href="http://doc.qt.io/qt-5/qset.html#operator-and-eq">operator&amp;=</a>(), <a href="http://doc.qt.io/qt-5/qset.html#unite">unite</a>(), and <a href="http://doc.qt.io/qt-5/qset.html#subtract">subtract</a>().</p></div>
    pub unsafe fn intersect(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSetOfQAbstractState>>,
    ) -> ::cpp_utils::MutRef<crate::QSetOfQAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_intersect(
            self as *mut crate::QSetOfQAbstractState,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSetOfQAbstractState>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if this set has at least one item in common with <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSet<QAbstractState*>::intersects(const QSet<QAbstractState*>& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#intersects">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this set has at least one item in common with <i>other</i>.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qset.html#intersect">intersect</a>().</p></div>
    pub unsafe fn intersects(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSetOfQAbstractState>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_intersects(
            self as *const crate::QSetOfQAbstractState,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSetOfQAbstractState>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QSet<QAbstractState*>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_isDetached(
            self as *const crate::QSetOfQAbstractState,
        )
    }

    /// <p>Returns <code>true</code> if the set contains no elements; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSet<QAbstractState*>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the set contains no elements; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_isEmpty(
            self as *const crate::QSetOfQAbstractState,
        )
    }

    /// <p>Constructs an empty set.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSet<QAbstractState*>::QSet()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#QSet">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty set.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#clear">clear</a>().</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QSetOfQAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_QSet();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> class is a template class that provides a hash-table-based set.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSet<QAbstractState*>::QSet(const QSet<QAbstractState*>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> class is a template class that provides a hash-table-based set.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qset.html">QSet</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores values in an unspecified order and provides very fast lookup of the values. Internally, <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>&lt;T&gt; is implemented as a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>.</p>
    /// <p>Here's an example <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> with <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> values:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> set;
    ///
    /// </pre>
    /// <p>To insert a value into the set, use <a href="http://doc.qt.io/qt-5/qset.html#insert">insert</a>():</p>
    /// <pre class="cpp">
    ///
    ///   set<span class="operator">.</span>insert(<span class="string">"one"</span>);
    ///   set<span class="operator">.</span>insert(<span class="string">"three"</span>);
    ///   set<span class="operator">.</span>insert(<span class="string">"seven"</span>);
    ///
    /// </pre>
    /// <p>Another way to insert items into the set is to use operator&lt;&lt;():</p>
    /// <pre class="cpp">
    ///
    ///   set <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"twelve"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"fifteen"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"nineteen"</span>;
    ///
    /// </pre>
    /// <p>To test whether an item belongs to the set or not, use <a href="http://doc.qt.io/qt-5/qset.html#contains">contains</a>():</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">if</span> (<span class="operator">!</span>set<span class="operator">.</span>contains(<span class="string">"ninety-nine"</span>))
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///
    /// </pre>
    /// <p>If you want to navigate through all the values stored in a <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>, you can use an iterator. <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> supports both <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qsetiterator.html">QSetIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablesetiterator.html">QMutableSetIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qset-iterator.html">QSet::iterator</a> and <a href="http://doc.qt.io/qt-5/qset-const-iterator.html">QSet::const_iterator</a>). Here's how to iterate over a <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt; using a Java-style iterator:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsetiterator.html">QSetIterator</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> i(set);
    ///   <span class="keyword">while</span> (i<span class="operator">.</span>hasNext())
    /// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>next();
    ///
    /// </pre>
    /// <p>Here's the same code, but using an STL-style iterator:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i <span class="operator">=</span> set<span class="operator">.</span>constBegin();
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> set<span class="operator">.</span>constEnd()) {
    /// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i;
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    ///   }
    ///
    /// </pre>
    /// <p><a href="http://doc.qt.io/qt-5/qset.html">QSet</a> is unordered, so an iterator's sequence cannot be assumed to be predictable. If ordering by key is required, use a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>.</p>
    /// <p>To navigate through a <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>, you can also use <a href="http://doc.qt.io/qt-5/containers.html#foreach">foreach</a>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> set;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   foreach (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>value<span class="operator">,</span> set)
    /// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> value;
    ///
    /// </pre>
    /// <p>Items can be removed from the set using <a href="http://doc.qt.io/qt-5/qset.html#remove">remove</a>(). There is also a <a href="http://doc.qt.io/qt-5/qset.html#clear">clear</a>() function that removes all items.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qset.html">QSet</a>'s value data type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. In addition, the type must provide <code>operator==()</code>, and there must also be a global <a href="http://doc.qt.io/qt-5/qhash.html#qHash">qHash</a>() function that returns a hash value for an argument of the key's type. See the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> documentation for a list of types supported by <a href="http://doc.qt.io/qt-5/qhash.html#qHash">qHash</a>().</p>
    /// <p>Internally, <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> uses a hash table to perform lookups. The hash table automatically grows and shrinks to provide fast lookups without wasting memory. You can still control the size of the hash table by calling <a href="http://doc.qt.io/qt-5/qset.html#reserve">reserve</a>(), if you already know approximately how many elements the <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> will contain, but this isn't necessary to obtain good performance. You can also call <a href="http://doc.qt.io/qt-5/qset.html#capacity">capacity</a>() to retrieve the hash table's size.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSetOfQAbstractState>>,
    ) -> ::cpp_utils::CppBox<crate::QSetOfQAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_QSet1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSetOfQAbstractState>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Ensures that the set's internal hash table consists of at least <i>size</i> buckets.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSet<QAbstractState*>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Ensures that the set's internal hash table consists of at least <i>size</i> buckets.</p>
    /// <p>This function is useful for code that needs to build a huge set and wants to avoid repeated reallocation. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> set;
    ///   set<span class="operator">.</span>reserve(<span class="number">20000</span>);
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> <span class="number">20000</span>; <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     set<span class="operator">.</span>insert(values<span class="operator">[</span>i<span class="operator">]</span>);
    ///
    /// </pre>
    /// <p>Ideally, <i>size</i> should be slightly more than the maximum number of elements expected in the set. <i>size</i> doesn't have to be prime, because <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> will use a prime number internally anyway. If <i>size</i> is an underestimate, the worst that will happen is that the <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> will be a bit slower.</p>
    /// <p>In general, you will rarely ever need to call this function. <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>'s internal hash table automatically shrinks or grows to provide good performance without wasting too much memory.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#squeeze">squeeze</a>() and <a href="http://doc.qt.io/qt-5/qset.html#capacity">capacity</a>().</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_reserve(
            self as *mut crate::QSetOfQAbstractState,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QSet<QAbstractState*>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_setSharable(
            self as *mut crate::QSetOfQAbstractState,
            sharable,
        )
    }

    /// <p>Returns the number of items in the set.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QSet<QAbstractState*>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the set.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qset.html#count">count</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_size(
            self as *const crate::QSetOfQAbstractState,
        )
    }

    /// <p>Reduces the size of the set's internal hash table to save memory.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSet<QAbstractState*>::squeeze()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#squeeze">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reduces the size of the set's internal hash table to save memory.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>'s memory usage. In general, you will rarely ever need to call this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qset.html#capacity">capacity</a>().</p></div>
    pub unsafe fn squeeze(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_squeeze(
            self as *mut crate::QSetOfQAbstractState,
        )
    }

    /// <p>Removes all items from this set that are contained in the <i>other</i> set. Returns a reference to this set.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>& QSet<QAbstractState*>::subtract(const QSet<QAbstractState*>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#subtract">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all items from this set that are contained in the <i>other</i> set. Returns a reference to this set.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#operator--eq">operator-=</a>(), <a href="http://doc.qt.io/qt-5/qset.html#unite">unite</a>(), and <a href="http://doc.qt.io/qt-5/qset.html#intersect">intersect</a>().</p></div>
    pub unsafe fn subtract(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSetOfQAbstractState>>,
    ) -> ::cpp_utils::MutRef<crate::QSetOfQAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_subtract(
            self as *mut crate::QSetOfQAbstractState,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSetOfQAbstractState>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Swaps set <i>other</i> with this set. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSet<QAbstractState*>::swap(QSet<QAbstractState*>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps set <i>other</i> with this set. This operation is very fast and never fails.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QSetOfQAbstractState>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_swap(
            self as *mut crate::QSetOfQAbstractState,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QSetOfQAbstractState>>::cast_into(
                other,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns a new <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> containing the elements in the set. The order of the elements in the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is undefined.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*> QSet<QAbstractState*>::toList() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#toList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a new <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> containing the elements in the set. The order of the elements in the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is undefined.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> set;
    ///   set <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"red"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"green"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"blue"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"black"</span>;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list <span class="operator">=</span> set<span class="operator">.</span>toList();
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms-obsolete.html#qSort">qSort</a>(list);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#fromList">fromList</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#fromSet">QList::fromSet</a>().</p></div>
    pub unsafe fn to_list(&self) -> ::cpp_utils::CppBox<crate::QListOfQAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_toList(
            self as *const crate::QSetOfQAbstractState,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Each item in the <i>other</i> set that isn't already in this set is inserted into this set. A reference to this set is returned.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>& QSet<QAbstractState*>::unite(const QSet<QAbstractState*>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#unite">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Each item in the <i>other</i> set that isn't already in this set is inserted into this set. A reference to this set is returned.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#operator-7c-eq">operator|=</a>(), <a href="http://doc.qt.io/qt-5/qset.html#intersect">intersect</a>(), and <a href="http://doc.qt.io/qt-5/qset.html#subtract">subtract</a>().</p></div>
    pub unsafe fn unite(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSetOfQAbstractState>>,
    ) -> ::cpp_utils::MutRef<crate::QSetOfQAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_unite(
            self as *mut crate::QSetOfQAbstractState,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSetOfQAbstractState>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a new <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> containing the elements in the set. The order of the elements in the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is undefined.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*> QSet<QAbstractState*>::values() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#values">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a new <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> containing the elements in the set. The order of the elements in the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is undefined.</p>
    /// <p>This is the same as <a href="http://doc.qt.io/qt-5/qset.html#toList">toList</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#fromList">fromList</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#fromSet">QList::fromSet</a>().</p></div>
    pub unsafe fn values(&self) -> ::cpp_utils::CppBox<crate::QListOfQAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_values(
            self as *const crate::QSetOfQAbstractState,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_list_of_q_storage_info {
    //! C++ type: <span style='color: green;'>```QList<QStorageInfo>```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QStorageInfo>::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::iterator& QList<QStorageInfo>::iterator::operator=(const QList<QStorageInfo>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_storage_info::Iterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_storage_info::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_iterator_operator_8(
                    self as *mut crate::q_list_of_q_storage_info::Iterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_storage_info::Iterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::iterator QList<QStorageInfo>::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_iterator_operator__13(
                self as *mut crate::q_list_of_q_storage_info::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::iterator QList<QStorageInfo>::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_iterator_operator__11(
                self as *mut crate::q_list_of_q_storage_info::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QStorageInfo& QList<QStorageInfo>::iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValueRef iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>. If you assign to it, the assignment will apply to the character in the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> of <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> from which you got the reference.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::MutRef<crate::QStorageInfo> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_iterator_operator__1(
                self as *const crate::q_list_of_q_storage_info::Iterator,
                j,
            );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QStorageInfo>::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>().</p></div>
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QStorageInfo>::iterator::iterator(const QList<QStorageInfo>::iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_storage_info::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_iterator_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_storage_info::Iterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QStorageInfo* QList<QStorageInfo>::iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's value.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::MutPtr<crate::QStorageInfo> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_iterator_operator__(
                self as *const crate::q_list_of_q_storage_info::Iterator,
            );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QStorageInfo>::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::const_iterator& QList<QStorageInfo>::const_iterator::operator=(const QList<QStorageInfo>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_storage_info::ConstIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_storage_info::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_const_iterator_operator_6(
                    self as *mut crate::q_list_of_q_storage_info::ConstIterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_storage_info::ConstIterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::const_iterator QList<QStorageInfo>::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_const_iterator_operator__9(
                    self as *mut crate::q_list_of_q_storage_info::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::const_iterator QList<QStorageInfo>::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_const_iterator_operator__7(
                    self as *mut crate::q_list_of_q_storage_info::ConstIterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QStorageInfo& QList<QStorageInfo>::const_iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValue const_iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::Ref<crate::QStorageInfo> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_const_iterator_operator__1(
                    self as *const crate::q_list_of_q_storage_info::ConstIterator,
                    j,
                );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QStorageInfo>::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>().</p></div>
        pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::ConstIterator>
        {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_const_iterator_const_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QStorageInfo>::const_iterator::const_iterator(const QList<QStorageInfo>::iterator& o)```</span>.
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_storage_info::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_const_iterator_const_iterator2(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_storage_info::Iterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QStorageInfo>::const_iterator::const_iterator(const QList<QStorageInfo>::const_iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_storage_info::ConstIterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_const_iterator_const_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_storage_info::ConstIterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current result.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QStorageInfo* QList<QStorageInfo>::const_iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current result.</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::Ptr<crate::QStorageInfo> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_const_iterator_operator__(
                    self as *const crate::q_list_of_q_storage_info::ConstIterator,
                );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_storage_info::Iterator {
        type Output = ::cpp_utils::MutRef<crate::QStorageInfo>;
        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QStorageInfo& QList<QStorageInfo>::iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>().</p></div>
        fn indirection(self) -> ::cpp_utils::MutRef<crate::QStorageInfo> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_iterator_operator_(
                    self as *const crate::q_list_of_q_storage_info::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_storage_info::Iterator>>
        for crate::q_list_of_q_storage_info::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QStorageInfo>::iterator::operator==(const QList<QStorageInfo>::iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_storage_info::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_iterator_operator__2(
                    self as *const crate::q_list_of_q_storage_info::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_storage_info::Iterator>>
        for crate::q_list_of_q_storage_info::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QStorageInfo>::iterator::operator<(const QList<QStorageInfo>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_storage_info::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_iterator_operator_1(
                    self as *const crate::q_list_of_q_storage_info::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_storage_info::Iterator>>
        for crate::q_list_of_q_storage_info::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QStorageInfo>::iterator::operator<=(const QList<QStorageInfo>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_storage_info::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_iterator_operator__4(
                    self as *const crate::q_list_of_q_storage_info::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_storage_info::Iterator>>
        for crate::q_list_of_q_storage_info::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QStorageInfo>::iterator::operator>(const QList<QStorageInfo>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_storage_info::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_iterator_operator_2(
                    self as *const crate::q_list_of_q_storage_info::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_storage_info::Iterator>>
        for crate::q_list_of_q_storage_info::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QStorageInfo>::iterator::operator>=(const QList<QStorageInfo>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_q_storage_info::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_iterator_operator__5(
                    self as *const crate::q_list_of_q_storage_info::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_storage_info::ConstIterator>>
        for crate::q_list_of_q_storage_info::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QStorageInfo>::iterator::operator==(const QList<QStorageInfo>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_storage_info::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_iterator_operator__6(
                    self as *const crate::q_list_of_q_storage_info::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_storage_info::ConstIterator>>
        for crate::q_list_of_q_storage_info::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QStorageInfo>::iterator::operator<(const QList<QStorageInfo>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_storage_info::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_iterator_operator_3(
                    self as *const crate::q_list_of_q_storage_info::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_storage_info::ConstIterator>>
        for crate::q_list_of_q_storage_info::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QStorageInfo>::iterator::operator<=(const QList<QStorageInfo>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_storage_info::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_iterator_operator__8(
                    self as *const crate::q_list_of_q_storage_info::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_storage_info::ConstIterator>>
        for crate::q_list_of_q_storage_info::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QStorageInfo>::iterator::operator>(const QList<QStorageInfo>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_storage_info::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_iterator_operator_4(
                    self as *const crate::q_list_of_q_storage_info::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_storage_info::ConstIterator>>
        for crate::q_list_of_q_storage_info::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QStorageInfo>::iterator::operator>=(const QList<QStorageInfo>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_storage_info::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_iterator_operator__9(
                    self as *const crate::q_list_of_q_storage_info::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_storage_info::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_storage_info::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::iterator& QList<QStorageInfo>::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_storage_info::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_iterator_operator__10(
                    self as *mut crate::q_list_of_q_storage_info::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_storage_info::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_storage_info::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::iterator& QList<QStorageInfo>::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_storage_info::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_iterator_operator__12(
                    self as *mut crate::q_list_of_q_storage_info::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_storage_info::Iterator
    {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::iterator& QList<QStorageInfo>::iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_iterator_operator__14(
                    self as *mut crate::q_list_of_q_storage_info::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_storage_info::Iterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::iterator& QList<QStorageInfo>::iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_iterator_operator__15(
                    self as *mut crate::q_list_of_q_storage_info::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_list_of_q_storage_info::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::iterator QList<QStorageInfo>::iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_iterator_operator_5(
                    self as *const crate::q_list_of_q_storage_info::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_list_of_q_storage_info::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::iterator QList<QStorageInfo>::iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_iterator_operator_6(
                    self as *const crate::q_list_of_q_storage_info::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_storage_info::Iterator>>
        for &crate::q_list_of_q_storage_info::Iterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QStorageInfo>::iterator::operator-(QList<QStorageInfo>::iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_storage_info::Iterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_iterator_operator_7(
                    self as *const crate::q_list_of_q_storage_info::Iterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_storage_info::ConstIterator {
        type Output = ::cpp_utils::Ref<crate::QStorageInfo>;
        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QStorageInfo& QList<QStorageInfo>::const_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        fn indirection(self) -> ::cpp_utils::Ref<crate::QStorageInfo> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_const_iterator_operator_(
                    self as *const crate::q_list_of_q_storage_info::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_q_storage_info::ConstIterator>>
        for crate::q_list_of_q_storage_info::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QStorageInfo>::const_iterator::operator==(const QList<QStorageInfo>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_q_storage_info::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_const_iterator_operator__2(
                    self as *const crate::q_list_of_q_storage_info::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_q_storage_info::ConstIterator>>
        for crate::q_list_of_q_storage_info::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QStorageInfo>::const_iterator::operator<(const QList<QStorageInfo>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_storage_info::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_const_iterator_operator_1(
                    self as *const crate::q_list_of_q_storage_info::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_q_storage_info::ConstIterator>>
        for crate::q_list_of_q_storage_info::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QStorageInfo>::const_iterator::operator<=(const QList<QStorageInfo>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_storage_info::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_const_iterator_operator__4(
                    self as *const crate::q_list_of_q_storage_info::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_q_storage_info::ConstIterator>>
        for crate::q_list_of_q_storage_info::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QStorageInfo>::const_iterator::operator>(const QList<QStorageInfo>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_storage_info::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_const_iterator_operator_2(
                    self as *const crate::q_list_of_q_storage_info::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_q_storage_info::ConstIterator>>
        for crate::q_list_of_q_storage_info::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QStorageInfo>::const_iterator::operator>=(const QList<QStorageInfo>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_storage_info::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_const_iterator_operator__5(
                    self as *const crate::q_list_of_q_storage_info::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_storage_info::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_storage_info::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::const_iterator& QList<QStorageInfo>::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_storage_info::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_const_iterator_operator__6(
                    self as *mut crate::q_list_of_q_storage_info::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_storage_info::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_storage_info::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::const_iterator& QList<QStorageInfo>::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_q_storage_info::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_const_iterator_operator__8(
                    self as *mut crate::q_list_of_q_storage_info::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_storage_info::ConstIterator
    {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::const_iterator& QList<QStorageInfo>::const_iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_const_iterator_operator__10(
                    self as *mut crate::q_list_of_q_storage_info::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_storage_info::ConstIterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::const_iterator& QList<QStorageInfo>::const_iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_const_iterator_operator__11(
                    self as *mut crate::q_list_of_q_storage_info::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_storage_info::ConstIterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::const_iterator QList<QStorageInfo>::const_iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_const_iterator_operator_3(
                    self as *const crate::q_list_of_q_storage_info::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_storage_info::ConstIterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::const_iterator QList<QStorageInfo>::const_iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_const_iterator_operator_4(
                    self as *const crate::q_list_of_q_storage_info::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_storage_info::ConstIterator>>
        for &crate::q_list_of_q_storage_info::ConstIterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QStorageInfo>::const_iterator::operator-(QList<QStorageInfo>::const_iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_storage_info::ConstIterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_const_iterator_operator_5(
                    self as *const crate::q_list_of_q_storage_info::ConstIterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_storage_info::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QStorageInfo>::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_iterator_diterator(
                self as *mut crate::q_list_of_q_storage_info::Iterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_storage_info::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QStorageInfo>::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_const_iterator_dconst_iterator(
                self as *mut crate::q_list_of_q_storage_info::ConstIterator,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
///
/// C++ class: <span style='color: green;'>```QList<QStorageInfo>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlist.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores items in a list that provides fast index-based access and index-based insertions and removals.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Internally, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T if <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. Otherwise, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T* and the items are allocated on the heap.</p>
/// <p>The array representation allows very fast insertions and index-based access. The <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() operations are also very fast because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates memory at both ends of its internal array. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.</p>
/// <p>Note, however, that when the conditions specified above are not met, each append or insert of a new item requires allocating the new item on the heap, and this per item allocation will make <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> a better choice for use cases that do a lot of appending or inserting, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> can allocate memory for many items in a single heap allocation.</p>
/// <p>Note that the internal array only ever gets bigger over the life of the list. It never shrinks. The internal array is deallocated by the destructor and by the assignment operator, when one list is assigned to another.</p>
/// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerList;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span><span class="operator">&gt;</span> dateList;
///
/// </pre>
/// <p>Qt includes a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class that inherits <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>&gt; and adds a few convenience functions, such as <a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#filter">QStringList::filter</a>(). <a href="http://doc.qt.io/qt-5/qstring.html#split">QString::split</a>() creates QStringLists from strings.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> stores a list of items. The default constructor creates an empty list. To insert items into the list, you can use operator&lt;&lt;():</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"one"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"two"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"three"</span>;
///   <span class="comment">// list: ["one", "two", "three"]</span>
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(). In addition, it provides the following convenience functions: <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const lists, operator[]() returns a reference to the item and can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Bob"</span>)
/// &#32;     list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Robert"</span>;
///
/// </pre>
/// <p>Because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented as an array of pointers for types that are larger than a pointer or are not movable, this operation requires (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>). For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> list<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (list<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Jane"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Jane at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>A common requirement is to remove an item from a list and do something with it. For this, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(). Here's a loop that removes the items from a list one at a time and calls <code>delete</code> on them:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   <span class="keyword">while</span> (<span class="operator">!</span>list<span class="operator">.</span>isEmpty())
/// &#32;     <span class="keyword">delete</span> list<span class="operator">.</span>takeFirst();
///
/// </pre>
/// <p>Inserting and removing items at either end of the list is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> in most cases), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
/// <p>If you want to find all occurrences of a particular value in a list, use <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of a matching item if they find it; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> list<span class="operator">.</span>indexOf(<span class="string">"Jane"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Jane is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a list contains a particular value, use <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the list, use <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>(). If you want to replace all occurrences of a particular value with another, use <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlistiterator.html">QListIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablelistiterator.html">QMutableListIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented in such a way that direct index-based access is just as fast as using iterators.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <p>To make <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as efficient as possible, its member functions don't validate their input before using it. Except for <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), member functions always assume the list is <i>not</i> empty. Member functions that take index values as parameters always assume their index value parameters are in the valid range. This means <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> member functions can fail. If you define QT_NO_DEBUG when you compile, failures will not be detected. If you <i>don't</i> define QT_NO_DEBUG, failures will be detected using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT">Q_ASSERT</a>() or <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT_X">Q_ASSERT_X</a>() with an appropriate message.</p>
/// <p>To avoid failures when your list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling other member functions. If you must pass an index value that might not be in the valid range, check that it is less than the value returned by <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() but <i>not</i> less than 0.</p>
/// <a name="more-members"></a></div>
#[repr(C)]
pub struct QListOfQStorageInfo {
    _unused: u8,
}
impl QListOfQStorageInfo {
    /// <p>Inserts <i>value</i> at the end of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QStorageInfo>::append(const QStorageInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["one", "two", "three"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn append_q_storage_info(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStorageInfo>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_append(
            self as *mut crate::QListOfQStorageInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStorageInfo>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QStorageInfo>::append(const QList<QStorageInfo>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> list to this list.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_list_of_q_storage_info(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQStorageInfo>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_append1(
            self as *mut crate::QListOfQStorageInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQStorageInfo>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStorageInfo& QList<QStorageInfo>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>This function is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::QStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_at(
            self as *const crate::QListOfQStorageInfo,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStorageInfo& QList<QStorageInfo>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::MutRef<crate::QStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_back(
            self as *mut crate::QListOfQStorageInfo,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStorageInfo& QList<QStorageInfo>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<crate::QStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_back1(
            self as *const crate::QListOfQStorageInfo,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::const_iterator QList<QStorageInfo>::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_cbegin(
            self as *const crate::QListOfQStorageInfo,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::const_iterator QList<QStorageInfo>::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn cend(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_cend(
            self as *const crate::QListOfQStorageInfo,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all items from the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QStorageInfo>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all items from the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_clear(
            self as *mut crate::QListOfQStorageInfo,
        )
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::const_iterator QList<QStorageInfo>::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_constBegin(
            self as *const crate::QListOfQStorageInfo,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::const_iterator QList<QStorageInfo>::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn const_end(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_constEnd(
            self as *const crate::QListOfQStorageInfo,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStorageInfo& QList<QStorageInfo>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>().</p></div>
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<crate::QStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_constFirst(
            self as *const crate::QListOfQStorageInfo,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStorageInfo& QList<QStorageInfo>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>().</p></div>
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<crate::QStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_constLast(
            self as *const crate::QListOfQStorageInfo,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QStorageInfo>::contains(const QStorageInfo& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn contains(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStorageInfo>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_contains(
            self as *const crate::QListOfQStorageInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStorageInfo>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>& QList<QStorageInfo>::operator=(const QList<QStorageInfo>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this list and returns a reference to this list.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQStorageInfo>>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_operator_(
            self as *mut crate::QListOfQStorageInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQStorageInfo>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of occurrences of <i>value</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QStorageInfo>::count(const QStorageInfo& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of occurrences of <i>value</i> in the list.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStorageInfo>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_count(
            self as *const crate::QListOfQStorageInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStorageInfo>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QStorageInfo>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_count1(
            self as *const crate::QListOfQStorageInfo,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QStorageInfo>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_detach(
            self as *mut crate::QListOfQStorageInfo,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QStorageInfo>::detachShared()```</span>.
    pub unsafe fn detach_shared(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_detachShared(
            self as *mut crate::QListOfQStorageInfo,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QStorageInfo>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_empty(
            self as *const crate::QListOfQStorageInfo,
        )
    }

    /// <p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QStorageInfo>::endsWith(const QStorageInfo& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn ends_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStorageInfo>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_endsWith(
            self as *const crate::QListOfQStorageInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStorageInfo>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::iterator QList<QStorageInfo>::erase(QList<QStorageInfo>::iterator pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn erase_1a(
        &mut self,
        pos: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_storage_info::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_erase(self as *mut crate::QListOfQStorageInfo, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_storage_info::Iterator>>::cast_into(pos).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::iterator QList<QStorageInfo>::erase(QList<QStorageInfo>::iterator first, QList<QStorageInfo>::iterator last)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes all the items from <i>begin</i> up to (but not including) <i>end</i>. Returns an iterator to the same item that <i>end</i> referred to before the call.</p></div>
    pub unsafe fn erase_2a(
        &mut self,
        first: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_storage_info::Iterator>>,
        last: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_storage_info::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_erase1(self as *mut crate::QListOfQStorageInfo, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_storage_info::Iterator>>::cast_into(first).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_storage_info::Iterator>>::cast_into(last).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStorageInfo& QList<QStorageInfo>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<crate::QStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_first(
            self as *mut crate::QListOfQStorageInfo,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStorageInfo& QList<QStorageInfo>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::QStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_first1(
            self as *const crate::QListOfQStorageInfo,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStorageInfo& QList<QStorageInfo>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::MutRef<crate::QStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_front(
            self as *mut crate::QListOfQStorageInfo,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStorageInfo& QList<QStorageInfo>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<crate::QStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_front1(
            self as *const crate::QListOfQStorageInfo,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStorageInfo& QList<QStorageInfo>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>(). This function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p></div>
    pub unsafe fn index(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::QStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_operator__2(
            self as *const crate::QListOfQStorageInfo,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStorageInfo& QList<QStorageInfo>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If this function is called on a list that is currently being shared, it will trigger a copy of all elements. Otherwise, this function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>. If you do not want to modify the list you should use <a href="http://doc.qt.io/qt-5/qlist.html#at">QList::at</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_operator__3(
            self as *mut crate::QListOfQStorageInfo,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QStorageInfo>::indexOf(const QStorageInfo& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStorageInfo>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_indexOf(
            self as *const crate::QListOfQStorageInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStorageInfo>>::cast_into(t)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QStorageInfo>::indexOf(const QStorageInfo& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStorageInfo>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_indexOf1(
            self as *const crate::QListOfQStorageInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStorageInfo>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QStorageInfo>::insert(int i, const QStorageInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   list<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// list: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn insert_int_q_storage_info(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStorageInfo>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_insert(
            self as *mut crate::QListOfQStorageInfo,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStorageInfo>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::iterator QList<QStorageInfo>::insert(QList<QStorageInfo>::iterator before, const QStorageInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>value</i> in front of the item pointed to by the iterator <i>before</i>. Returns an iterator pointing at the inserted item. Note that the iterator passed to the function will be invalid after the call; the returned iterator should be used instead.</p></div>
    pub unsafe fn insert_iterator_q_storage_info(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_q_storage_info::Iterator>>,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStorageInfo>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_insert1(self as *mut crate::QListOfQStorageInfo, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_storage_info::Iterator>>::cast_into(before).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStorageInfo>>::cast_into(t).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QStorageInfo>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_isDetached(
            self as *const crate::QListOfQStorageInfo,
        )
    }

    /// <p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QStorageInfo>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_isEmpty(
            self as *const crate::QListOfQStorageInfo,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QStorageInfo>::isSharedWith(const QList<QStorageInfo>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQStorageInfo>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_isSharedWith(
            self as *const crate::QListOfQStorageInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQStorageInfo>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStorageInfo& QList<QStorageInfo>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<crate::QStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_last(
            self as *mut crate::QListOfQStorageInfo,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QStorageInfo& QList<QStorageInfo>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::QStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_last1(
            self as *const crate::QListOfQStorageInfo,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QStorageInfo>::lastIndexOf(const QStorageInfo& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStorageInfo>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_lastIndexOf(
            self as *const crate::QListOfQStorageInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStorageInfo>>::cast_into(t)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QStorageInfo>::lastIndexOf(const QStorageInfo& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStorageInfo>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_lastIndexOf1(
            self as *const crate::QListOfQStorageInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStorageInfo>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QStorageInfo>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_length(
            self as *const crate::QListOfQStorageInfo,
        )
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo> QList<QStorageInfo>::mid(int pos, int length = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        length: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_mid(
            self as *const crate::QListOfQStorageInfo,
            pos,
            length,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo> QList<QStorageInfo>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_mid1(
            self as *const crate::QListOfQStorageInfo,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QStorageInfo>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>move(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "C", "D", "E", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This is the same as insert(<i>to</i>, <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<i>from</i>)).This function assumes that both <i>from</i> and <i>to</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>from</i> and <i>to</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_move(
            self as *mut crate::QListOfQStorageInfo,
            from,
            to,
        )
    }

    /// <p>Constructs an empty list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QStorageInfo>::QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty list.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QListOfQStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_QList();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QStorageInfo>::QList(const QList<QStorageInfo>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQStorageInfo>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_QList1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQStorageInfo>>::cast_into(l)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QStorageInfo>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_pop_back(
            self as *mut crate::QListOfQStorageInfo,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QStorageInfo>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_pop_front(
            self as *mut crate::QListOfQStorageInfo,
        )
    }

    /// <p>Inserts <i>value</i> at the beginning of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QStorageInfo>::prepend(const QStorageInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(0, <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStorageInfo>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_prepend(
            self as *mut crate::QListOfQStorageInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStorageInfo>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QStorageInfo>::push_back(const QStorageInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStorageInfo>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_push_back(
            self as *mut crate::QListOfQStorageInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStorageInfo>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QStorageInfo>::push_front(const QStorageInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStorageInfo>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_push_front(
            self as *mut crate::QListOfQStorageInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStorageInfo>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QStorageInfo>::removeAll(const QStorageInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeAll(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_all(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStorageInfo>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_removeAll(
            self as *mut crate::QListOfQStorageInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStorageInfo>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QStorageInfo>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_removeAt(
            self as *mut crate::QListOfQStorageInfo,
            i,
        )
    }

    /// <p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QStorageInfo>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_removeFirst(
            self as *mut crate::QListOfQStorageInfo,
        )
    }

    /// <p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QStorageInfo>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_removeLast(
            self as *mut crate::QListOfQStorageInfo,
        )
    }

    /// <p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QStorageInfo>::removeOne(const QStorageInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeOne(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", ,"sun", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_one(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStorageInfo>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_removeOne(
            self as *mut crate::QListOfQStorageInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStorageInfo>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QStorageInfo>::replace(int i, const QStorageInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStorageInfo>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_replace(
            self as *mut crate::QListOfQStorageInfo,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStorageInfo>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Reserve space for <i>alloc</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QStorageInfo>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reserve space for <i>alloc</i> elements.</p>
    /// <p>If <i>alloc</i> is smaller than the current size of the list, nothing will happen.</p>
    /// <p>Use this function to avoid repetetive reallocation of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s internal data if you can predict how many elements will be appended. Note that the reservation applies only to the internal pointer array.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_reserve(
            self as *mut crate::QListOfQStorageInfo,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QStorageInfo>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_setSharable(
            self as *mut crate::QListOfQStorageInfo,
            sharable,
        )
    }

    /// <p>Returns the number of items in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QStorageInfo>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_size(
            self as *const crate::QListOfQStorageInfo,
        )
    }

    /// <p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QStorageInfo>::startsWith(const QStorageInfo& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn starts_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStorageInfo>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_startsWith(
            self as *const crate::QListOfQStorageInfo,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStorageInfo>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QStorageInfo>::swap(QList<QStorageInfo>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap_1a(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QListOfQStorageInfo>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_swap(
            self as *mut crate::QListOfQStorageInfo,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QListOfQStorageInfo>>::cast_into(
                other,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QStorageInfo>::swap(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swap(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    pub unsafe fn swap_2a(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_swap1(
            self as *mut crate::QListOfQStorageInfo,
            i,
            j,
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QStorageInfo>::swapItemsAt(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swapItemsAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swapItemsAt(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn swap_items_at(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_swapItemsAt(
            self as *mut crate::QListOfQStorageInfo,
            i,
            j,
        )
    }

    /// <p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStorageInfo QList<QStorageInfo>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_takeAt(
            self as *mut crate::QListOfQStorageInfo,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStorageInfo QList<QStorageInfo>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> ::cpp_utils::CppBox<crate::QStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_takeFirst(
            self as *mut crate::QListOfQStorageInfo,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStorageInfo QList<QStorageInfo>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> ::cpp_utils::CppBox<crate::QStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_takeLast(
            self as *mut crate::QListOfQStorageInfo,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value at index position <i>i</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStorageInfo QList<QStorageInfo>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the list.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that the index is going to be within bounds, you can use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_value(
            self as *const crate::QListOfQStorageInfo,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStorageInfo QList<QStorageInfo>::value(int i, const QStorageInfo& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStorageInfo>>,
    ) -> ::cpp_utils::CppBox<crate::QStorageInfo> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_value1(
            self as *const crate::QListOfQStorageInfo,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStorageInfo>>::cast_into(
                default_value,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_list_of_int {
    //! C++ type: <span style='color: green;'>```QList<int>```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<int>::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<int>::iterator& QList<int>::iterator::operator=(const QList<int>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_int::Iterator>>,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_int::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_int_iterator_operator_8(self as *mut crate::q_list_of_int::Iterator, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_int::Iterator>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<int>::iterator QList<int>::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_int::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_int_iterator_operator__13(
                self as *mut crate::q_list_of_int::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<int>::iterator QList<int>::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_int::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_int_iterator_operator__11(
                self as *mut crate::q_list_of_int::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```int& QList<int>::iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValueRef iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>. If you assign to it, the assignment will apply to the character in the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> of <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> from which you got the reference.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::MutRef<::std::os::raw::c_int> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_int_iterator_operator__1(
                self as *const crate::q_list_of_int::Iterator,
                j,
            );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<int>::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>().</p></div>
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_list_of_int::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_int_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<int>::iterator::iterator(const QList<int>::iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_int::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_int::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_int_iterator_iterator1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_int::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```int* QList<int>::iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's value.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::MutPtr<::std::os::raw::c_int> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_int_iterator_operator__(
                self as *const crate::q_list_of_int::Iterator,
            );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<int>::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<int>::const_iterator& QList<int>::const_iterator::operator=(const QList<int>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_int::ConstIterator>>,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_int::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_int_const_iterator_operator_6(self as *mut crate::q_list_of_int::ConstIterator, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_int::ConstIterator>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<int>::const_iterator QList<int>::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_int::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_int_const_iterator_operator__9(
                self as *mut crate::q_list_of_int::ConstIterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<int>::const_iterator QList<int>::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_int::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_int_const_iterator_operator__7(
                self as *mut crate::q_list_of_int::ConstIterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const int& QList<int>::const_iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValue const_iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::Ref<::std::os::raw::c_int> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_int_const_iterator_operator__1(
                self as *const crate::q_list_of_int::ConstIterator,
                j,
            );
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<int>::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>().</p></div>
        pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::q_list_of_int::ConstIterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_int_const_iterator_const_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<int>::const_iterator::const_iterator(const QList<int>::iterator& o)```</span>.
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_int::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_int::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_int_const_iterator_const_iterator2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_int::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<int>::const_iterator::const_iterator(const QList<int>::const_iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_int::ConstIterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_int::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_int_const_iterator_const_iterator1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_int::ConstIterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current result.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const int* QList<int>::const_iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current result.</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::Ptr<::std::os::raw::c_int> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_int_const_iterator_operator__(
                self as *const crate::q_list_of_int::ConstIterator,
            );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_int::Iterator {
        type Output = ::cpp_utils::MutRef<::std::os::raw::c_int>;
        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```int& QList<int>::iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>().</p></div>
        fn indirection(self) -> ::cpp_utils::MutRef<::std::os::raw::c_int> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_iterator_operator_(
                    self as *const crate::q_list_of_int::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_int::Iterator>>
        for crate::q_list_of_int::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<int>::iterator::operator==(const QList<int>::iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_int::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_iterator_operator__2(
                    self as *const crate::q_list_of_int::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_int::Iterator>>
        for crate::q_list_of_int::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<int>::iterator::operator<(const QList<int>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_int::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_iterator_operator_1(
                    self as *const crate::q_list_of_int::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_int::Iterator>>
        for crate::q_list_of_int::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<int>::iterator::operator<=(const QList<int>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_int::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_iterator_operator__4(
                    self as *const crate::q_list_of_int::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_int::Iterator>>
        for crate::q_list_of_int::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<int>::iterator::operator>(const QList<int>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_int::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_iterator_operator_2(
                    self as *const crate::q_list_of_int::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_int::Iterator>>
        for crate::q_list_of_int::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<int>::iterator::operator>=(const QList<int>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_int::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_iterator_operator__5(
                    self as *const crate::q_list_of_int::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_int::ConstIterator>>
        for crate::q_list_of_int::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<int>::iterator::operator==(const QList<int>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_int::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_iterator_operator__6(
                    self as *const crate::q_list_of_int::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_int::ConstIterator>>
        for crate::q_list_of_int::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<int>::iterator::operator<(const QList<int>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_int::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_iterator_operator_3(
                    self as *const crate::q_list_of_int::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_int::ConstIterator>>
        for crate::q_list_of_int::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<int>::iterator::operator<=(const QList<int>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_int::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_iterator_operator__8(
                    self as *const crate::q_list_of_int::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_int::ConstIterator>>
        for crate::q_list_of_int::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<int>::iterator::operator>(const QList<int>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_int::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_iterator_operator_4(
                    self as *const crate::q_list_of_int::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_int::ConstIterator>>
        for crate::q_list_of_int::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<int>::iterator::operator>=(const QList<int>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_int::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_iterator_operator__9(
                    self as *const crate::q_list_of_int::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_int::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_int::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<int>::iterator& QList<int>::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_int::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_iterator_operator__10(
                    self as *mut crate::q_list_of_int::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_int::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_int::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<int>::iterator& QList<int>::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_int::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_iterator_operator__12(
                    self as *mut crate::q_list_of_int::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong> for crate::q_list_of_int::Iterator {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<int>::iterator& QList<int>::iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_iterator_operator__14(
                    self as *mut crate::q_list_of_int::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong> for crate::q_list_of_int::Iterator {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<int>::iterator& QList<int>::iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_iterator_operator__15(
                    self as *mut crate::q_list_of_int::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_list_of_int::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_int::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<int>::iterator QList<int>::iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_int::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_iterator_operator_5(
                    self as *const crate::q_list_of_int::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_list_of_int::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_int::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<int>::iterator QList<int>::iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_int::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_iterator_operator_6(
                    self as *const crate::q_list_of_int::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_int::Iterator>>
        for &crate::q_list_of_int::Iterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<int>::iterator::operator-(QList<int>::iterator j) const```</span>.
        fn sub(self, j: ::cpp_utils::Ref<crate::q_list_of_int::Iterator>) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_iterator_operator_7(
                    self as *const crate::q_list_of_int::Iterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_int::ConstIterator {
        type Output = ::cpp_utils::Ref<::std::os::raw::c_int>;
        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const int& QList<int>::const_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        fn indirection(self) -> ::cpp_utils::Ref<::std::os::raw::c_int> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_const_iterator_operator_(
                    self as *const crate::q_list_of_int::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_list_of_int::ConstIterator>>
        for crate::q_list_of_int::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<int>::const_iterator::operator==(const QList<int>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_list_of_int::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_const_iterator_operator__2(
                    self as *const crate::q_list_of_int::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_list_of_int::ConstIterator>>
        for crate::q_list_of_int::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<int>::const_iterator::operator<(const QList<int>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_list_of_int::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_const_iterator_operator_1(
                    self as *const crate::q_list_of_int::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_list_of_int::ConstIterator>>
        for crate::q_list_of_int::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<int>::const_iterator::operator<=(const QList<int>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(&self, other: &::cpp_utils::Ref<crate::q_list_of_int::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_const_iterator_operator__4(
                    self as *const crate::q_list_of_int::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_list_of_int::ConstIterator>>
        for crate::q_list_of_int::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<int>::const_iterator::operator>(const QList<int>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_list_of_int::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_const_iterator_operator_2(
                    self as *const crate::q_list_of_int::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_list_of_int::ConstIterator>>
        for crate::q_list_of_int::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<int>::const_iterator::operator>=(const QList<int>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_list_of_int::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_const_iterator_operator__5(
                    self as *const crate::q_list_of_int::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_int::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_int::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<int>::const_iterator& QList<int>::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_list_of_int::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_const_iterator_operator__6(
                    self as *mut crate::q_list_of_int::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_int::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_int::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<int>::const_iterator& QList<int>::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_list_of_int::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_const_iterator_operator__8(
                    self as *mut crate::q_list_of_int::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong> for crate::q_list_of_int::ConstIterator {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<int>::const_iterator& QList<int>::const_iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_const_iterator_operator__10(
                    self as *mut crate::q_list_of_int::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong> for crate::q_list_of_int::ConstIterator {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<int>::const_iterator& QList<int>::const_iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_const_iterator_operator__11(
                    self as *mut crate::q_list_of_int::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_list_of_int::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_int::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<int>::const_iterator QList<int>::const_iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_int::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_const_iterator_operator_3(
                    self as *const crate::q_list_of_int::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_list_of_int::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_int::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<int>::const_iterator QList<int>::const_iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_int::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_const_iterator_operator_4(
                    self as *const crate::q_list_of_int::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_int::ConstIterator>>
        for &crate::q_list_of_int::ConstIterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<int>::const_iterator::operator-(QList<int>::const_iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_int::ConstIterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_int_const_iterator_operator_5(
                    self as *const crate::q_list_of_int::ConstIterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_int::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<int>::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_int_iterator_diterator(
                self as *mut crate::q_list_of_int::Iterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_int::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<int>::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_int_const_iterator_dconst_iterator(
                self as *mut crate::q_list_of_int::ConstIterator,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
///
/// C++ class: <span style='color: green;'>```QList<int>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlist.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores items in a list that provides fast index-based access and index-based insertions and removals.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Internally, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T if <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. Otherwise, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T* and the items are allocated on the heap.</p>
/// <p>The array representation allows very fast insertions and index-based access. The <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() operations are also very fast because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates memory at both ends of its internal array. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.</p>
/// <p>Note, however, that when the conditions specified above are not met, each append or insert of a new item requires allocating the new item on the heap, and this per item allocation will make <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> a better choice for use cases that do a lot of appending or inserting, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> can allocate memory for many items in a single heap allocation.</p>
/// <p>Note that the internal array only ever gets bigger over the life of the list. It never shrinks. The internal array is deallocated by the destructor and by the assignment operator, when one list is assigned to another.</p>
/// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerList;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span><span class="operator">&gt;</span> dateList;
///
/// </pre>
/// <p>Qt includes a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class that inherits <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>&gt; and adds a few convenience functions, such as <a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#filter">QStringList::filter</a>(). <a href="http://doc.qt.io/qt-5/qstring.html#split">QString::split</a>() creates QStringLists from strings.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> stores a list of items. The default constructor creates an empty list. To insert items into the list, you can use operator&lt;&lt;():</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"one"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"two"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"three"</span>;
///   <span class="comment">// list: ["one", "two", "three"]</span>
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(). In addition, it provides the following convenience functions: <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const lists, operator[]() returns a reference to the item and can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Bob"</span>)
/// &#32;     list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Robert"</span>;
///
/// </pre>
/// <p>Because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented as an array of pointers for types that are larger than a pointer or are not movable, this operation requires (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>). For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> list<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (list<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Jane"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Jane at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>A common requirement is to remove an item from a list and do something with it. For this, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(). Here's a loop that removes the items from a list one at a time and calls <code>delete</code> on them:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   <span class="keyword">while</span> (<span class="operator">!</span>list<span class="operator">.</span>isEmpty())
/// &#32;     <span class="keyword">delete</span> list<span class="operator">.</span>takeFirst();
///
/// </pre>
/// <p>Inserting and removing items at either end of the list is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> in most cases), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
/// <p>If you want to find all occurrences of a particular value in a list, use <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of a matching item if they find it; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> list<span class="operator">.</span>indexOf(<span class="string">"Jane"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Jane is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a list contains a particular value, use <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the list, use <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>(). If you want to replace all occurrences of a particular value with another, use <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlistiterator.html">QListIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablelistiterator.html">QMutableListIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented in such a way that direct index-based access is just as fast as using iterators.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <p>To make <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as efficient as possible, its member functions don't validate their input before using it. Except for <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), member functions always assume the list is <i>not</i> empty. Member functions that take index values as parameters always assume their index value parameters are in the valid range. This means <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> member functions can fail. If you define QT_NO_DEBUG when you compile, failures will not be detected. If you <i>don't</i> define QT_NO_DEBUG, failures will be detected using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT">Q_ASSERT</a>() or <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT_X">Q_ASSERT_X</a>() with an appropriate message.</p>
/// <p>To avoid failures when your list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling other member functions. If you must pass an index value that might not be in the valid range, check that it is less than the value returned by <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() but <i>not</i> less than 0.</p>
/// <a name="more-members"></a></div>
#[repr(C)]
pub struct QListOfInt {
    _unused: u8,
}
impl QListOfInt {
    /// <p>Inserts <i>value</i> at the end of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<int>::append(const int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["one", "two", "three"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn append_int(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_int_append(
            self as *mut crate::QListOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<int>::append(const QList<int>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> list to this list.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_list_of_int(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfInt>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_int_append1(
            self as *mut crate::QListOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfInt>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const int& QList<int>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>This function is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<::std::os::raw::c_int> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_int_at(self as *const crate::QListOfInt, i);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int& QList<int>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::MutRef<::std::os::raw::c_int> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_int_back(self as *mut crate::QListOfInt);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const int& QList<int>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<::std::os::raw::c_int> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_int_back1(self as *const crate::QListOfInt);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<int>::const_iterator QList<int>::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(&self) -> ::cpp_utils::CppBox<crate::q_list_of_int::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_int_cbegin(self as *const crate::QListOfInt);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<int>::const_iterator QList<int>::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn cend(&self) -> ::cpp_utils::CppBox<crate::q_list_of_int::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_int_cend(self as *const crate::QListOfInt);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all items from the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<int>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all items from the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_int_clear(self as *mut crate::QListOfInt)
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<int>::const_iterator QList<int>::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(&self) -> ::cpp_utils::CppBox<crate::q_list_of_int::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_int_constBegin(self as *const crate::QListOfInt);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<int>::const_iterator QList<int>::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn const_end(&self) -> ::cpp_utils::CppBox<crate::q_list_of_int::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_int_constEnd(self as *const crate::QListOfInt);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const int& QList<int>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>().</p></div>
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<::std::os::raw::c_int> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_int_constFirst(self as *const crate::QListOfInt);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const int& QList<int>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>().</p></div>
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<::std::os::raw::c_int> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_int_constLast(self as *const crate::QListOfInt);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<int>::contains(const int& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn contains(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_int_contains(
            self as *const crate::QListOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<int>& QList<int>::operator=(const QList<int>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this list and returns a reference to this list.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfInt>>,
    ) -> ::cpp_utils::MutRef<crate::QListOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_int_operator_(
            self as *mut crate::QListOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfInt>>::cast_into(l).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of occurrences of <i>value</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<int>::count(const int& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of occurrences of <i>value</i> in the list.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_int_count(
            self as *const crate::QListOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<int>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_int_count1(self as *const crate::QListOfInt)
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<int>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_int_detach(self as *mut crate::QListOfInt)
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<int>::detachShared()```</span>.
    pub unsafe fn detach_shared(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_int_detachShared(self as *mut crate::QListOfInt)
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<int>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_int_empty(self as *const crate::QListOfInt)
    }

    /// <p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<int>::endsWith(const int& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn ends_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_int_endsWith(
            self as *const crate::QListOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<int>::iterator QList<int>::erase(QList<int>::iterator pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn erase_1a(
        &mut self,
        pos: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_int::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_int::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_int_erase(
            self as *mut crate::QListOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_int::Iterator>>::cast_into(
                pos,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<int>::iterator QList<int>::erase(QList<int>::iterator first, QList<int>::iterator last)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes all the items from <i>begin</i> up to (but not including) <i>end</i>. Returns an iterator to the same item that <i>end</i> referred to before the call.</p></div>
    pub unsafe fn erase_2a(
        &mut self,
        first: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_int::Iterator>>,
        last: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_int::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_int::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_int_erase1(
            self as *mut crate::QListOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_int::Iterator>>::cast_into(
                first,
            )
            .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_int::Iterator>>::cast_into(
                last,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int& QList<int>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<::std::os::raw::c_int> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_int_first(self as *mut crate::QListOfInt);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const int& QList<int>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<::std::os::raw::c_int> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_int_first1(self as *const crate::QListOfInt);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> object with the data contained in <i>vector</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QList<int> QList<int>::fromVector(const QVector<int>& vector)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#fromVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> object with the data contained in <i>vector</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a></span><span class="operator">&lt;</span><span class="type">double</span><span class="operator">&gt;</span> vect;
    ///   vect <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">20.0</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">30.0</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">40.0</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">50.0</span>;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">double</span><span class="operator">&gt;</span> list <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a></span><span class="operator">&lt;</span>T<span class="operator">&gt;</span><span class="operator">::</span>fromVector(vect);
    ///   <span class="comment">// list: [20.0, 30.0, 40.0, 50.0]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#fromSet">fromSet</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#toVector">toVector</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#toList">QVector::toList</a>().</p></div>
    pub unsafe fn from_vector(
        vector: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfInt>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_int_fromVector(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfInt>>::cast_into(vector)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int& QList<int>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::MutRef<::std::os::raw::c_int> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_int_front(self as *mut crate::QListOfInt);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const int& QList<int>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<::std::os::raw::c_int> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_int_front1(self as *const crate::QListOfInt);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const int& QList<int>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>(). This function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p></div>
    pub unsafe fn index(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ref<::std::os::raw::c_int> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_int_operator__2(
            self as *const crate::QListOfInt,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int& QList<int>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If this function is called on a list that is currently being shared, it will trigger a copy of all elements. Otherwise, this function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>. If you do not want to modify the list you should use <a href="http://doc.qt.io/qt-5/qlist.html#at">QList::at</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<::std::os::raw::c_int> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_int_operator__3(self as *mut crate::QListOfInt, i);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<int>::indexOf(const int& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_int_indexOf(
            self as *const crate::QListOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<int>::indexOf(const int& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_int_indexOf1(
            self as *const crate::QListOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<int>::insert(int i, const int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   list<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// list: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn insert_int_int(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_int_insert(
            self as *mut crate::QListOfInt,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<int>::iterator QList<int>::insert(QList<int>::iterator before, const int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>value</i> in front of the item pointed to by the iterator <i>before</i>. Returns an iterator pointing at the inserted item. Note that the iterator passed to the function will be invalid after the call; the returned iterator should be used instead.</p></div>
    pub unsafe fn insert_iterator_int(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_list_of_int::Iterator>>,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_int::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_int_insert1(
            self as *mut crate::QListOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_int::Iterator>>::cast_into(
                before,
            )
            .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<int>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_int_isDetached(self as *const crate::QListOfInt)
    }

    /// <p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<int>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_int_isEmpty(self as *const crate::QListOfInt)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<int>::isSharedWith(const QList<int>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfInt>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_int_isSharedWith(
            self as *const crate::QListOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfInt>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int& QList<int>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<::std::os::raw::c_int> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_int_last(self as *mut crate::QListOfInt);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const int& QList<int>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<::std::os::raw::c_int> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_int_last1(self as *const crate::QListOfInt);
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<int>::lastIndexOf(const int& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_int_lastIndexOf(
            self as *const crate::QListOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<int>::lastIndexOf(const int& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_int_lastIndexOf1(
            self as *const crate::QListOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<int>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_int_length(self as *const crate::QListOfInt)
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<int> QList<int>::mid(int pos, int length = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        length: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_int_mid(
            self as *const crate::QListOfInt,
            pos,
            length,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<int> QList<int>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfInt> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_int_mid1(self as *const crate::QListOfInt, pos);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<int>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>move(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "C", "D", "E", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This is the same as insert(<i>to</i>, <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<i>from</i>)).This function assumes that both <i>from</i> and <i>to</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>from</i> and <i>to</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_int_move(self as *mut crate::QListOfInt, from, to)
    }

    /// <p>Constructs an empty list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<int>::QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty list.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QListOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_int_QList();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<int>::QList(const QList<int>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfInt>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_int_QList1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfInt>>::cast_into(l).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<int>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_int_pop_back(self as *mut crate::QListOfInt)
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<int>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_int_pop_front(self as *mut crate::QListOfInt)
    }

    /// <p>Inserts <i>value</i> at the beginning of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<int>::prepend(const int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(0, <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_int_prepend(
            self as *mut crate::QListOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<int>::push_back(const int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_int_push_back(
            self as *mut crate::QListOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<int>::push_front(const int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_int_push_front(
            self as *mut crate::QListOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<int>::removeAll(const int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeAll(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_all(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_int_removeAll(
            self as *mut crate::QListOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<int>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_int_removeAt(self as *mut crate::QListOfInt, i)
    }

    /// <p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<int>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_int_removeFirst(self as *mut crate::QListOfInt)
    }

    /// <p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<int>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_int_removeLast(self as *mut crate::QListOfInt)
    }

    /// <p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<int>::removeOne(const int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeOne(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", ,"sun", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_one(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_int_removeOne(
            self as *mut crate::QListOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<int>::replace(int i, const int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_int_replace(
            self as *mut crate::QListOfInt,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Reserve space for <i>alloc</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<int>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reserve space for <i>alloc</i> elements.</p>
    /// <p>If <i>alloc</i> is smaller than the current size of the list, nothing will happen.</p>
    /// <p>Use this function to avoid repetetive reallocation of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s internal data if you can predict how many elements will be appended. Note that the reservation applies only to the internal pointer array.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_int_reserve(self as *mut crate::QListOfInt, size)
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<int>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QList_int_setSharable(
            self as *mut crate::QListOfInt,
            sharable,
        )
    }

    /// <p>Returns the number of items in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<int>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_int_size(self as *const crate::QListOfInt)
    }

    /// <p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<int>::startsWith(const int& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn starts_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_int_startsWith(
            self as *const crate::QListOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<int>::swap(QList<int>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap_1a(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QListOfInt>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_int_swap(
            self as *mut crate::QListOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QListOfInt>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<int>::swap(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swap(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    pub unsafe fn swap_2a(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_int_swap1(self as *mut crate::QListOfInt, i, j)
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<int>::swapItemsAt(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swapItemsAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swapItemsAt(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn swap_items_at(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_int_swapItemsAt(self as *mut crate::QListOfInt, i, j)
    }

    /// <p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<int>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn take_at(&mut self, i: ::std::os::raw::c_int) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_int_takeAt(self as *mut crate::QListOfInt, i)
    }

    /// <p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<int>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_int_takeFirst(self as *mut crate::QListOfInt)
    }

    /// <p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<int>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_int_takeLast(self as *mut crate::QListOfInt)
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> object with the data contained in this <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<int> QList<int>::toVector() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#toVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> object with the data contained in this <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Sven"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Kim"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Ola"</span>;
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vect <span class="operator">=</span> list<span class="operator">.</span>toVector();
    ///   <span class="comment">// vect: ["Sven", "Kim", "Ola"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#toSet">toSet</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#fromVector">fromVector</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#fromList">QVector::fromList</a>().</p></div>
    pub unsafe fn to_vector(&self) -> ::cpp_utils::CppBox<crate::QVectorOfInt> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QList_int_toVector(self as *const crate::QListOfInt);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value at index position <i>i</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<int>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the list.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that the index is going to be within bounds, you can use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(&self, i: ::std::os::raw::c_int) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_int_value(self as *const crate::QListOfInt, i)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<int>::value(int i, const int& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_int>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_int_value1(
            self as *const crate::QListOfInt,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_int>>::cast_into(
                default_value,
            )
            .as_raw_ptr(),
        )
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> class is a template class that provides a dynamic array.</p>
///
/// C++ class: <span style='color: green;'>```QVector<QTimeZone::OffsetData>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qvector.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> class is a template class that provides a dynamic array.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores its items in adjacent memory locations and provides fast index-based access.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Here's an example of a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> that stores <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerVector;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> stringVector;
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> stores its items in a vector (array). Typically, vectors are created with an initial size. For example, the following code constructs a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> with 200 elements:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">200</span>);
///
/// </pre>
/// <p>The elements are automatically initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you want to initialize the vector with a different value, pass that value as the second argument to the constructor:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">200</span><span class="operator">,</span> <span class="string">"Pass"</span>);
///
/// </pre>
/// <p>You can also call <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>() at any time to fill the vector with a value.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const vectors, operator[]() returns a reference to the item that can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (vector<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Liz"</span>)
/// &#32;     vector<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Elizabeth"</span>;
///
/// </pre>
/// <p>For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> vector<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (vector<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Alfonso"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Alfonso at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>Another way to access the data stored in a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> is to call <a href="http://doc.qt.io/qt-5/qvector.html#data">data</a>(). The function returns a pointer to the first item in the vector. You can use the pointer to directly access and modify the elements stored in the vector. The pointer is also useful if you need to pass a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to a function that accepts a plain C++ array.</p>
/// <p>If you want to find all occurrences of a particular value in a vector, use <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of the matching item if they found one; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> vector<span class="operator">.</span>indexOf(<span class="string">"Harumi"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Harumi is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a vector contains a particular value, use <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the vector, use <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(). With the exception of <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), these functions can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>) for large vectors, because they require moving many items in the vector by one position in memory. If you want a container class that provides fast insertion/removal in the middle, use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> or <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
/// <p>Unlike plain C++ arrays, QVectors can be resized at any time by calling <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>(). If the new size is larger than the old size, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> might need to reallocate the whole vector. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> tries to reduce the number of reallocations by preallocating up to twice as much memory as the actual data needs.</p>
/// <p>If you know in advance approximately how many items the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> will contain, you can call <a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>(), asking <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to preallocate a certain amount of memory. You can also call <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>() to find out how much memory <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> actually allocated.</p>
/// <p>Note that using non-const operators and functions can cause <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to do a deep copy of the data. This is due to <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicit sharing</a>.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qvectoriterator.html">QVectorIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablevectoriterator.html">QMutableVectorIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qvector.html#const_iterator-typedef">QVector::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qvector.html#iterator-typedefx">QVector::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>.</p>
/// <p>In addition to <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>, Qt also provides <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a>, a very low-level class with little functionality that is optimized for speed.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <a name="more-information-on-using-qt-containers"></a>
/// <h4>More Information on Using Qt Containers</h4>
/// <p>For a detailed discussion comparing Qt containers with each other and with STL containers, see <a href="http://marcmutz.wordpress.com/effective-qt/containers/">Understand the Qt Containers</a>.</p></div>
#[repr(C)]
pub struct QVectorOfOffsetData {
    _unused: u8,
}
impl QVectorOfOffsetData {
    /// <p>Inserts <i>value</i> at the end of the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QTimeZone::OffsetData>::append(const QTimeZone::OffsetData& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   vector<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> three <span class="operator">=</span> <span class="string">"three"</span>;
    ///   vector<span class="operator">.</span>append(three);
    ///   <span class="comment">// vector: ["one", "two", "three"]</span>
    ///   <span class="comment">// three: "three"</span>
    ///
    /// </pre>
    /// <p>This is the same as calling resize(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() + 1) and assigning <i>value</i> to the new last element in the vector.</p>
    /// <p>This operation is relatively fast, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> typically allocates more memory than necessary, so it can grow without reallocating the entire vector each time.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>().</p></div>
    pub unsafe fn append_offset_data(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_append(
            self as *mut crate::QVectorOfOffsetData,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QTimeZone::OffsetData>::append(const QVector<QTimeZone::OffsetData>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#append-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> vector to this vector.</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_vector_of_offset_data(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfOffsetData>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_append2(
            self as *mut crate::QVectorOfOffsetData,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfOffsetData>>::cast_into(l)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the item at index position <i>i</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QTimeZone::OffsetData& QVector<QTimeZone::OffsetData>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the vector.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ref<crate::q_time_zone::OffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_at(
            self as *const crate::QVectorOfOffsetData,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTimeZone::OffsetData& QVector<QTimeZone::OffsetData>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::MutRef<crate::q_time_zone::OffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_back(
            self as *mut crate::QVectorOfOffsetData,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QTimeZone::OffsetData& QVector<QTimeZone::OffsetData>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<crate::q_time_zone::OffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_back1(
            self as *const crate::QVectorOfOffsetData,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the maximum number of items that can be stored in the vector without forcing a reallocation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QTimeZone::OffsetData>::capacity() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#capacity">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the maximum number of items that can be stored in the vector without forcing a reallocation.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s memory usage. In general, you will rarely ever need to call this function. If you want to know how many items are in the vector, call <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p></div>
    pub unsafe fn capacity(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_capacity(
            self as *const crate::QVectorOfOffsetData,
        )
    }

    /// <p>Removes all the elements from the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QTimeZone::OffsetData>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all the elements from the vector.</p>
    /// <p><b>Note: </b>Until Qt 5.6, this also released the memory used by the vector. From Qt 5.7, the capacity is preserved. To shed all capacity, swap with a default-constructed vector:</p><pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span>T<span class="operator">&gt;</span> v <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span>T<span class="operator">&gt;</span>()<span class="operator">.</span>swap(v);
    ///   Q_ASSERT(v<span class="operator">.</span>capacity() <span class="operator">=</span><span class="operator">=</span> <span class="number">0</span>);
    ///
    /// </pre>
    /// <p>or call <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_clear(
            self as *mut crate::QVectorOfOffsetData,
        )
    }

    /// <p>Returns a const pointer to the data stored in the vector. The pointer can be used to access the items in the vector. The pointer remains valid as long as the vector isn't reallocated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QTimeZone::OffsetData* QVector<QTimeZone::OffsetData>::constData() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const pointer to the data stored in the vector. The pointer can be used to access the items in the vector. The pointer remains valid as long as the vector isn't reallocated.</p>
    /// <p>This function is mostly useful to pass a vector to a function that accepts a plain C++ array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#data">data</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn const_data(&self) -> ::cpp_utils::Ptr<crate::q_time_zone::OffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_constData(
            self as *const crate::QVectorOfOffsetData,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns a const reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QTimeZone::OffsetData& QVector<QTimeZone::OffsetData>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<crate::q_time_zone::OffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_constFirst(
            self as *const crate::QVectorOfOffsetData,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QTimeZone::OffsetData& QVector<QTimeZone::OffsetData>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<crate::q_time_zone::OffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_constLast(
            self as *const crate::QVectorOfOffsetData,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Assigns <i>other</i> to this vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QTimeZone::OffsetData>& QVector<QTimeZone::OffsetData>::operator=(const QVector<QTimeZone::OffsetData>& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this vector and returns a reference to this vector.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        v: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfOffsetData>>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfOffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_operator_(
            self as *mut crate::QVectorOfOffsetData,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfOffsetData>>::cast_into(v)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QTimeZone::OffsetData>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p></div>
    pub unsafe fn count(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_count1(
            self as *const crate::QVectorOfOffsetData,
        )
    }

    /// <p>Returns a pointer to the data stored in the vector. The pointer can be used to access and modify the items in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTimeZone::OffsetData* QVector<QTimeZone::OffsetData>::data()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the data stored in the vector. The pointer can be used to access and modify the items in the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> vector(<span class="number">10</span>);
    ///   <span class="type">int</span> <span class="operator">*</span>data <span class="operator">=</span> vector<span class="operator">.</span>data();
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> <span class="number">10</span>; <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     data<span class="operator">[</span>i<span class="operator">]</span> <span class="operator">=</span> <span class="number">2</span> <span class="operator">*</span> i;
    ///
    /// </pre>
    /// <p>The pointer remains valid as long as the vector isn't reallocated.</p>
    /// <p>This function is mostly useful to pass a vector to a function that accepts a plain C++ array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constData">constData</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn data_mut(&mut self) -> ::cpp_utils::MutPtr<crate::q_time_zone::OffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_data(
            self as *mut crate::QVectorOfOffsetData,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QTimeZone::OffsetData* QVector<QTimeZone::OffsetData>::data() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#data-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn data(&self) -> ::cpp_utils::Ptr<crate::q_time_zone::OffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_data1(
            self as *const crate::QVectorOfOffsetData,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```void QVector<QTimeZone::OffsetData>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_detach(
            self as *mut crate::QVectorOfOffsetData,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), returning <code>true</code> if the vector is empty; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QTimeZone::OffsetData>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), returning <code>true</code> if the vector is empty; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_empty(
            self as *const crate::QVectorOfOffsetData,
        )
    }

    /// <p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QTimeZone::OffsetData>& QVector<QTimeZone::OffsetData>::fill(const QTimeZone::OffsetData& t, int size = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">3</span>);
    ///   vector<span class="operator">.</span>fill(<span class="string">"Yes"</span>);
    ///   <span class="comment">// vector: ["Yes", "Yes", "Yes"]</span>
    ///
    ///   vector<span class="operator">.</span>fill(<span class="string">"oh"</span><span class="operator">,</span> <span class="number">5</span>);
    ///   <span class="comment">// vector: ["oh", "oh", "oh", "oh", "oh"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_2a(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfOffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_fill(
            self as *mut crate::QVectorOfOffsetData,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>::cast_into(t)
                .as_raw_ptr(),
            size,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QTimeZone::OffsetData>& QVector<QTimeZone::OffsetData>::fill(const QTimeZone::OffsetData& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">3</span>);
    ///   vector<span class="operator">.</span>fill(<span class="string">"Yes"</span>);
    ///   <span class="comment">// vector: ["Yes", "Yes", "Yes"]</span>
    ///
    ///   vector<span class="operator">.</span>fill(<span class="string">"oh"</span><span class="operator">,</span> <span class="number">5</span>);
    ///   <span class="comment">// vector: ["oh", "oh", "oh", "oh", "oh"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_1a(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfOffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_fill1(
            self as *mut crate::QVectorOfOffsetData,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>::cast_into(t)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTimeZone::OffsetData& QVector<QTimeZone::OffsetData>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#constFirst">constFirst</a>().</p></div>
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<crate::q_time_zone::OffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_first(
            self as *mut crate::QVectorOfOffsetData,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QTimeZone::OffsetData& QVector<QTimeZone::OffsetData>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::q_time_zone::OffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_first1(
            self as *const crate::QVectorOfOffsetData,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTimeZone::OffsetData& QVector<QTimeZone::OffsetData>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::MutRef<crate::q_time_zone::OffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_front(
            self as *mut crate::QVectorOfOffsetData,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QTimeZone::OffsetData& QVector<QTimeZone::OffsetData>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<crate::q_time_zone::OffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_front1(
            self as *const crate::QVectorOfOffsetData,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTimeZone::OffsetData& QVector<QTimeZone::OffsetData>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p>Note that using non-const operators can cause <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to do a deep copy.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::q_time_zone::OffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_operator__2(
            self as *mut crate::QVectorOfOffsetData,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QTimeZone::OffsetData& QVector<QTimeZone::OffsetData>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as at(<i>i</i>).</p></div>
    pub unsafe fn index(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ref<crate::q_time_zone::OffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_operator__3(
            self as *const crate::QVectorOfOffsetData,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the vector. If <i>i</i> is 0, the value is prepended to the vector. If <i>i</i> is <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), the value is appended to the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QTimeZone::OffsetData>::insert(int i, const QTimeZone::OffsetData& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the vector. If <i>i</i> is 0, the value is prepended to the vector. If <i>i</i> is <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), the value is appended to the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   vector<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// vector: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p>For large vectors, this operation can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because it requires moving all the items at indexes <i>i</i> and above by one position further in memory. If you want a container class that provides a fast insert() function, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>().</p></div>
    pub unsafe fn insert_2a(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_insert(
            self as *mut crate::QVectorOfOffsetData,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QTimeZone::OffsetData>::insert(int i, int n, const QTimeZone::OffsetData& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>count</i> copies of <i>value</i> at index position <i>i</i> in the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">double</span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">2.718</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">1.442</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">0.4342</span>;
    ///   vector<span class="operator">.</span>insert(<span class="number">1</span><span class="operator">,</span> <span class="number">3</span><span class="operator">,</span> <span class="number">9.9</span>);
    ///   <span class="comment">// vector: [2.718, 9.9, 9.9, 9.9, 1.442, 0.4342]</span>
    ///
    /// </pre></div>
    pub unsafe fn insert_3a(
        &mut self,
        i: ::std::os::raw::c_int,
        n: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_insert1(
            self as *mut crate::QVectorOfOffsetData,
            i,
            n,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QVector<QTimeZone::OffsetData>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_isDetached(
            self as *const crate::QVectorOfOffsetData,
        )
    }

    /// <p>Returns <code>true</code> if the vector has size 0; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QTimeZone::OffsetData>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the vector has size 0; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_isEmpty(
            self as *const crate::QVectorOfOffsetData,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QVector<QTimeZone::OffsetData>::isSharedWith(const QVector<QTimeZone::OffsetData>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfOffsetData>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_isSharedWith(
            self as *const crate::QVectorOfOffsetData,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfOffsetData>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTimeZone::OffsetData& QVector<QTimeZone::OffsetData>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#constLast">constLast</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<crate::q_time_zone::OffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_last(
            self as *mut crate::QVectorOfOffsetData,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QTimeZone::OffsetData& QVector<QTimeZone::OffsetData>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::q_time_zone::OffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_last1(
            self as *const crate::QVectorOfOffsetData,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QTimeZone::OffsetData>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#length">QList::length</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_length(
            self as *const crate::QVectorOfOffsetData,
        )
    }

    /// <p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QTimeZone::OffsetData> QVector<QTimeZone::OffsetData>::mid(int pos, int len = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        len: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfOffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_mid(
            self as *const crate::QVectorOfOffsetData,
            pos,
            len,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QTimeZone::OffsetData> QVector<QTimeZone::OffsetData>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfOffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_mid1(
            self as *const crate::QVectorOfOffsetData,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QTimeZone::OffsetData>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">QList::move</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_move(
            self as *mut crate::QVectorOfOffsetData,
            from,
            to,
        )
    }

    /// <p>Constructs an empty vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QTimeZone::OffsetData>::QVector()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QVectorOfOffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_QVector();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a vector with an initial size of <i>size</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QTimeZone::OffsetData>::QVector(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a vector with an initial size of <i>size</i> elements.</p>
    /// <p>The elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn new_1a(
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfOffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_QVector1(size);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a vector with an initial size of <i>size</i> elements. Each element is initialized with <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QTimeZone::OffsetData>::QVector(int size, const QTimeZone::OffsetData& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a vector with an initial size of <i>size</i> elements. Each element is initialized with <i>value</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn new_2a(
        size: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfOffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_QVector2(
            size,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>::cast_into(t)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QTimeZone::OffsetData>::QVector(const QVector<QTimeZone::OffsetData>& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        v: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfOffsetData>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfOffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_QVector3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfOffsetData>>::cast_into(v)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QTimeZone::OffsetData>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_pop_back(
            self as *mut crate::QVectorOfOffsetData,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QTimeZone::OffsetData>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_pop_front(
            self as *mut crate::QVectorOfOffsetData,
        )
    }

    /// <p>Inserts <i>value</i> at the beginning of the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QTimeZone::OffsetData>::prepend(const QTimeZone::OffsetData& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   vector<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   vector<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// vector: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as vector.insert(0, <i>value</i>).</p>
    /// <p>For large vectors, this operation can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because it requires moving all the items in the vector by one position further in memory. If you want a container class that provides a fast prepend() function, use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> or <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_prepend(
            self as *mut crate::QVectorOfOffsetData,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to append(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QTimeZone::OffsetData>::push_back(const QTimeZone::OffsetData& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to append(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_push_back(
            self as *mut crate::QVectorOfOffsetData,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to prepend(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QTimeZone::OffsetData>::push_front(const QTimeZone::OffsetData& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to prepend(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_push_front(
            self as *mut crate::QVectorOfOffsetData,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QTimeZone::OffsetData>::remove(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#remove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes the element at index position <i>i</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn remove_1a(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_remove(
            self as *mut crate::QVectorOfOffsetData,
            i,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QTimeZone::OffsetData>::remove(int i, int n)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#remove-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes <i>count</i> elements from the middle of the vector, starting at index position <i>i</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn remove_2a(&mut self, i: ::std::os::raw::c_int, n: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_remove1(
            self as *mut crate::QVectorOfOffsetData,
            i,
            n,
        )
    }

    /// <p>Removes the element at index position <i>i</i>. Equivalent to</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QTimeZone::OffsetData>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the element at index position <i>i</i>. Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   remove(i);
    ///
    /// </pre>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">QList::removeAt</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_removeAt(
            self as *mut crate::QVectorOfOffsetData,
            i,
        )
    }

    /// <p>Removes the first item in the vector. Calling this function is equivalent to calling remove(0). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QTimeZone::OffsetData>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the vector. Calling this function is equivalent to calling remove(0). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_removeFirst(
            self as *mut crate::QVectorOfOffsetData,
        )
    }

    /// <p>Removes the last item in the vector. Calling this function is equivalent to calling remove(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() - 1). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QTimeZone::OffsetData>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the vector. Calling this function is equivalent to calling remove(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() - 1). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_removeLast(
            self as *mut crate::QVectorOfOffsetData,
        )
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QTimeZone::OffsetData>::replace(int i, const QTimeZone::OffsetData& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_replace(
            self as *mut crate::QVectorOfOffsetData,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Attempts to allocate memory for at least <i>size</i> elements. If you know in advance how large the vector will be, you should call this function to prevent reallocations and memory fragmentation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QTimeZone::OffsetData>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to allocate memory for at least <i>size</i> elements. If you know in advance how large the vector will be, you should call this function to prevent reallocations and memory fragmentation.</p>
    /// <p>If <i>size</i> is an underestimate, the worst that will happen is that the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> will be a bit slower. If <i>size</i> is an overestimate, you may have used more memory than the normal <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> growth strategy would have allocated—or you may have used less.</p>
    /// <p>An alternative to reserve() is calling <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>(). Whether or not that is faster than reserve() depends on the element type, because <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() default-constructs all elements, and requires assignment to existing entries rather than calling <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(), which copy- or move-constructs. For simple types, like <code>int</code> or <code>double</code>, <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() is typically faster, but for anything more complex, you should prefer reserve().</p>
    /// <p><b>Warning:</b> If the size passed to <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() was underestimated, you run out of allocated space and into undefined behavior. This problem does not exist with reserve(), because it treats the size as just a hint.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>().</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_reserve(
            self as *mut crate::QVectorOfOffsetData,
            size,
        )
    }

    /// <p>Sets the size of the vector to <i>size</i>. If <i>size</i> is greater than the current size, elements are added to the end; the new elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If <i>size</i> is less than the current size, elements are removed from the end.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QTimeZone::OffsetData>::resize(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#resize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the size of the vector to <i>size</i>. If <i>size</i> is greater than the current size, elements are added to the end; the new elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If <i>size</i> is less than the current size, elements are removed from the end.</p>
    /// <p>Since Qt 5.6, resize() doesn't shrink the capacity anymore. To shed excess capacity, use <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p></div>
    pub unsafe fn resize(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_resize(
            self as *mut crate::QVectorOfOffsetData,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QVector<QTimeZone::OffsetData>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_setSharable(
            self as *mut crate::QVectorOfOffsetData,
            sharable,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QTimeZone::OffsetData>::shrink_to_fit()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#shrink_to_fit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn shrink_to_fit(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_shrink_to_fit(
            self as *mut crate::QVectorOfOffsetData,
        )
    }

    /// <p>Returns the number of items in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QTimeZone::OffsetData>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_size(
            self as *const crate::QVectorOfOffsetData,
        )
    }

    /// <p>Releases any memory not required to store the items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QTimeZone::OffsetData>::squeeze()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Releases any memory not required to store the items.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s memory usage. In general, you will rarely ever need to call this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>().</p></div>
    pub unsafe fn squeeze(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_squeeze(
            self as *mut crate::QVectorOfOffsetData,
        )
    }

    /// <p>Swaps vector <i>other</i> with this vector. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QTimeZone::OffsetData>::swap(QVector<QTimeZone::OffsetData>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps vector <i>other</i> with this vector. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QVectorOfOffsetData>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_swap(
            self as *mut crate::QVectorOfOffsetData,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QVectorOfOffsetData>>::cast_into(
                other,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Removes the element at index position <i>i</i> and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTimeZone::OffsetData QVector<QTimeZone::OffsetData>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the element at index position <i>i</i> and returns it.</p>
    /// <p>Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   T t <span class="operator">=</span> at(i);
    ///   remove(i);
    ///   <span class="keyword">return</span> t;
    ///
    /// </pre>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">QList::takeAt</a>().</p></div>
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::q_time_zone::OffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_takeAt(
            self as *mut crate::QVectorOfOffsetData,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the first item in the vector and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTimeZone::OffsetData QVector<QTimeZone::OffsetData>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the vector and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> ::cpp_utils::CppBox<crate::q_time_zone::OffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_takeFirst(
            self as *mut crate::QVectorOfOffsetData,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the last item in the list and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTimeZone::OffsetData QVector<QTimeZone::OffsetData>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> ::cpp_utils::CppBox<crate::q_time_zone::OffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_takeLast(
            self as *mut crate::QVectorOfOffsetData,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value at index position <i>i</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTimeZone::OffsetData QVector<QTimeZone::OffsetData>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the vector.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that <i>i</i> is within bounds, you can use <a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::q_time_zone::OffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_value(
            self as *const crate::QVectorOfOffsetData,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTimeZone::OffsetData QVector<QTimeZone::OffsetData>::value(int i, const QTimeZone::OffsetData& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>,
    ) -> ::cpp_utils::CppBox<crate::q_time_zone::OffsetData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_value1(
            self as *const crate::QVectorOfOffsetData,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>::cast_into(
                default_value,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// C++ class: <span style='color: green;'>```QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>```</span>.
#[repr(C)]
pub struct QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption {
    _unused: u8,
}
impl QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption {
    /// Calls C++ function: <span style='color: green;'>```QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption> QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::operator~() const```</span>.
    pub unsafe fn bit_not(
        &self,
    ) -> ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>
    {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_operatord(self as *const crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>& QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::operator=(const QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>& other)```</span>.
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>,
        >,
    ) -> ::cpp_utils::MutRef<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>
    {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_operator_11(self as *mut crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>>::cast_into(other).as_raw_ptr());
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::QUrlTwoFlags(QUrl::UrlFormattingOption f)```</span>.
    pub unsafe fn from_url_formatting_option(
        f: crate::q_url::UrlFormattingOption,
    ) -> ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>
    {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_QUrlTwoFlags(f);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::QUrlTwoFlags(QUrl::ComponentFormattingOption f)```</span>.
    pub unsafe fn from_component_formatting_option(
        f: crate::q_url::ComponentFormattingOption,
    ) -> ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>
    {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_QUrlTwoFlags1(f);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::QUrlTwoFlags(QFlags<QUrl::ComponentFormattingOption> f)```</span>.
    pub unsafe fn from_q_flags_component_formatting_option(
        f: crate::QFlags<crate::q_url::ComponentFormattingOption>,
    ) -> ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>
    {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_QUrlTwoFlags2(f.to_int());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::QUrlTwoFlags()```</span>.
    pub unsafe fn new(
    ) -> ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>
    {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_QUrlTwoFlags3();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::QUrlTwoFlags(const QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>& other)```</span>.
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>,
        >,
    ) -> ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>
    {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_QUrlTwoFlags4(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>>::cast_into(other).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::testFlag(QUrl::UrlFormattingOption f) const```</span>.
    pub unsafe fn test_flag_url_formatting_option(
        &self,
        f: crate::q_url::UrlFormattingOption,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_testFlag(self as *const crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption, f)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::testFlag(QUrl::ComponentFormattingOption f) const```</span>.
    pub unsafe fn test_flag_component_formatting_option(
        &self,
        f: crate::q_url::ComponentFormattingOption,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_testFlag1(self as *const crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption, f)
    }

    /// Calls C++ function: <span style='color: green;'>```int QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::operator int() const```</span>.
    pub unsafe fn to_int(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_operator_int(self as *const crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
///
/// C++ class: <span style='color: green;'>```QPair<QString, QString>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qpair.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
/// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; can be used in your application if the STL <code>pair</code> type is not available. It stores one value of type T1 and one value of type T2. It can be used as a return value for a function that needs to return two values, or as the value type of a <a href="http://doc.qt.io/qt-5/containers.html">generic container</a>.</p>
/// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> that stores one <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and one <code>double</code> value:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qpair.html#QPair">QPair</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> pair;
///
/// </pre>
/// <p>The components are accessible as public data members called <a href="http://doc.qt.io/qt-5/qpair.html#first-var">first</a> and <a href="http://doc.qt.io/qt-5/qpair.html#second-var">second</a>. For example:</p>
/// <pre class="cpp">
///
///   pair<span class="operator">.</span>first <span class="operator">=</span> <span class="string">"pi"</span>;
///   pair<span class="operator">.</span>second <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
///
/// </pre>
/// <p>Note, however, that it is almost always preferable to define a small struct to hold the result of a function with multiple return values. A struct trivially generalizes to more than two values, and allows more descriptive member names than <code>first</code> and <code>second</code>:</p>
/// <pre class="cpp">
///
///   <span class="keyword">struct</span> Variable {
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> name;
/// &#32;     <span class="type">double</span> value;
///   };
///   Variable v;
///   v<span class="operator">.</span>name <span class="operator">=</span> <span class="string">"pi"</span>;
///   v<span class="operator">.</span>value <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
///
/// </pre>
/// <p>The advent of C++11 automatic variable type deduction (<code>auto</code>) shifts the emphasis from the type name to the name of functions and members. Thus, <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>, like <code>std::pair</code> and <code>std::tuple</code>, is mostly useful in generic (template) code, where defining a dedicated type is not possible.</p>
/// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>'s template data types (T1 and T2) must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; these requirements are documented on a per-function basis.</p></div>
#[repr(C)]
pub struct QPairOfQStringQString {
    _unused: u8,
}
impl QPairOfQStringQString {
    /// <p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPair<QString, QString>& QPair<QString, QString>::operator=(const QPair<QString, QString>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; can be used in your application if the STL <code>pair</code> type is not available. It stores one value of type T1 and one value of type T2. It can be used as a return value for a function that needs to return two values, or as the value type of a <a href="http://doc.qt.io/qt-5/containers.html">generic container</a>.</p>
    /// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> that stores one <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and one <code>double</code> value:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpair.html#QPair">QPair</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> pair;
    ///
    /// </pre>
    /// <p>The components are accessible as public data members called <a href="http://doc.qt.io/qt-5/qpair.html#first-var">first</a> and <a href="http://doc.qt.io/qt-5/qpair.html#second-var">second</a>. For example:</p>
    /// <pre class="cpp">
    ///
    ///   pair<span class="operator">.</span>first <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   pair<span class="operator">.</span>second <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>Note, however, that it is almost always preferable to define a small struct to hold the result of a function with multiple return values. A struct trivially generalizes to more than two values, and allows more descriptive member names than <code>first</code> and <code>second</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">struct</span> Variable {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> name;
    /// &#32;     <span class="type">double</span> value;
    ///   };
    ///   Variable v;
    ///   v<span class="operator">.</span>name <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   v<span class="operator">.</span>value <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>The advent of C++11 automatic variable type deduction (<code>auto</code>) shifts the emphasis from the type name to the name of functions and members. Thus, <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>, like <code>std::pair</code> and <code>std::tuple</code>, is mostly useful in generic (template) code, where defining a dedicated type is not possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>'s template data types (T1 and T2) must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; these requirements are documented on a per-function basis.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQStringQString>>,
    ) -> ::cpp_utils::MutRef<crate::QPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPair_QString_QString_operator_(
            self as *mut crate::QPairOfQStringQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQStringQString>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Constructs an empty pair. The <code>first</code> and <code>second</code> elements are initialized with <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>s.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPair<QString, QString>::QPair()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html#QPair">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty pair. The <code>first</code> and <code>second</code> elements are initialized with <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>s.</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPair_QString_QString_QPair();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a pair and initializes the <code>first</code> element with <i>value1</i> and the <code>second</code> element with <i>value2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPair<QString, QString>::QPair(const QString& t1, const QString& t2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html#QPair-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a pair and initializes the <code>first</code> element with <i>value1</i> and the <code>second</code> element with <i>value2</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpair.html#qMakePair">qMakePair</a>().</p></div>
    pub unsafe fn new_2a(
        t1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        t2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPair_QString_QString_QPair1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(t1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(t2).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPair<QString, QString>::QPair(const QPair<QString, QString>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; can be used in your application if the STL <code>pair</code> type is not available. It stores one value of type T1 and one value of type T2. It can be used as a return value for a function that needs to return two values, or as the value type of a <a href="http://doc.qt.io/qt-5/containers.html">generic container</a>.</p>
    /// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> that stores one <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and one <code>double</code> value:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpair.html#QPair">QPair</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> pair;
    ///
    /// </pre>
    /// <p>The components are accessible as public data members called <a href="http://doc.qt.io/qt-5/qpair.html#first-var">first</a> and <a href="http://doc.qt.io/qt-5/qpair.html#second-var">second</a>. For example:</p>
    /// <pre class="cpp">
    ///
    ///   pair<span class="operator">.</span>first <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   pair<span class="operator">.</span>second <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>Note, however, that it is almost always preferable to define a small struct to hold the result of a function with multiple return values. A struct trivially generalizes to more than two values, and allows more descriptive member names than <code>first</code> and <code>second</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">struct</span> Variable {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> name;
    /// &#32;     <span class="type">double</span> value;
    ///   };
    ///   Variable v;
    ///   v<span class="operator">.</span>name <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   v<span class="operator">.</span>value <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>The advent of C++11 automatic variable type deduction (<code>auto</code>) shifts the emphasis from the type name to the name of functions and members. Thus, <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>, like <code>std::pair</code> and <code>std::tuple</code>, is mostly useful in generic (template) code, where defining a dedicated type is not possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>'s template data types (T1 and T2) must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; these requirements are documented on a per-function basis.</p></div>
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQStringQString>>,
    ) -> ::cpp_utils::CppBox<crate::QPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPair_QString_QString_QPair2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQStringQString>>::cast_into(
                other,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Swaps this pair with <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QPair<QString, QString>::swap(QPair<QString, QString>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps this pair with <i>other</i>.</p>
    /// <p>Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms-obsolete.html#qSwap-1">qSwap</a>(<span class="keyword">this</span><span class="operator">-</span><span class="operator">&gt;</span>first<span class="operator">,</span> other<span class="operator">.</span>first);
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms-obsolete.html#qSwap-1">qSwap</a>(<span class="keyword">this</span><span class="operator">-</span><span class="operator">&gt;</span>second<span class="operator">,</span> other<span class="operator">.</span>second);
    ///
    /// </pre>
    /// <p>Swap overloads are found in namespace <code>std</code> as well as via argument-dependent lookup (ADL) in the namespace of <code>T</code> .</p>
    /// <p>This function was introduced in  Qt 5.5.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QPairOfQStringQString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QPair_QString_QString_swap(
            self as *mut crate::QPairOfQStringQString,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QPairOfQStringQString>>::cast_into(
                other,
            )
            .as_mut_raw_ptr(),
        )
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> class is a template class that provides a dynamic array.</p>
///
/// C++ class: <span style='color: green;'>```QVector<QXmlStreamNamespaceDeclaration>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qvector.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> class is a template class that provides a dynamic array.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores its items in adjacent memory locations and provides fast index-based access.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Here's an example of a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> that stores <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerVector;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> stringVector;
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> stores its items in a vector (array). Typically, vectors are created with an initial size. For example, the following code constructs a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> with 200 elements:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">200</span>);
///
/// </pre>
/// <p>The elements are automatically initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you want to initialize the vector with a different value, pass that value as the second argument to the constructor:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">200</span><span class="operator">,</span> <span class="string">"Pass"</span>);
///
/// </pre>
/// <p>You can also call <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>() at any time to fill the vector with a value.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const vectors, operator[]() returns a reference to the item that can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (vector<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Liz"</span>)
/// &#32;     vector<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Elizabeth"</span>;
///
/// </pre>
/// <p>For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> vector<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (vector<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Alfonso"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Alfonso at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>Another way to access the data stored in a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> is to call <a href="http://doc.qt.io/qt-5/qvector.html#data">data</a>(). The function returns a pointer to the first item in the vector. You can use the pointer to directly access and modify the elements stored in the vector. The pointer is also useful if you need to pass a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to a function that accepts a plain C++ array.</p>
/// <p>If you want to find all occurrences of a particular value in a vector, use <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of the matching item if they found one; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> vector<span class="operator">.</span>indexOf(<span class="string">"Harumi"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Harumi is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a vector contains a particular value, use <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the vector, use <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(). With the exception of <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), these functions can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>) for large vectors, because they require moving many items in the vector by one position in memory. If you want a container class that provides fast insertion/removal in the middle, use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> or <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
/// <p>Unlike plain C++ arrays, QVectors can be resized at any time by calling <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>(). If the new size is larger than the old size, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> might need to reallocate the whole vector. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> tries to reduce the number of reallocations by preallocating up to twice as much memory as the actual data needs.</p>
/// <p>If you know in advance approximately how many items the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> will contain, you can call <a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>(), asking <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to preallocate a certain amount of memory. You can also call <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>() to find out how much memory <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> actually allocated.</p>
/// <p>Note that using non-const operators and functions can cause <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to do a deep copy of the data. This is due to <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicit sharing</a>.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qvectoriterator.html">QVectorIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablevectoriterator.html">QMutableVectorIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qvector.html#const_iterator-typedef">QVector::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qvector.html#iterator-typedefx">QVector::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>.</p>
/// <p>In addition to <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>, Qt also provides <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a>, a very low-level class with little functionality that is optimized for speed.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <a name="more-information-on-using-qt-containers"></a>
/// <h4>More Information on Using Qt Containers</h4>
/// <p>For a detailed discussion comparing Qt containers with each other and with STL containers, see <a href="http://marcmutz.wordpress.com/effective-qt/containers/">Understand the Qt Containers</a>.</p></div>
#[repr(C)]
pub struct QVectorOfQXmlStreamNamespaceDeclaration {
    _unused: u8,
}
impl QVectorOfQXmlStreamNamespaceDeclaration {
    /// <p>Inserts <i>value</i> at the end of the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNamespaceDeclaration>::append(const QXmlStreamNamespaceDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   vector<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> three <span class="operator">=</span> <span class="string">"three"</span>;
    ///   vector<span class="operator">.</span>append(three);
    ///   <span class="comment">// vector: ["one", "two", "three"]</span>
    ///   <span class="comment">// three: "three"</span>
    ///
    /// </pre>
    /// <p>This is the same as calling resize(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() + 1) and assigning <i>value</i> to the new last element in the vector.</p>
    /// <p>This operation is relatively fast, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> typically allocates more memory than necessary, so it can grow without reallocating the entire vector each time.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>().</p></div>
    pub unsafe fn append_q_xml_stream_namespace_declaration(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_append(self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNamespaceDeclaration>::append(const QVector<QXmlStreamNamespaceDeclaration>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#append-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> vector to this vector.</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_vector_of_q_xml_stream_namespace_declaration(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQXmlStreamNamespaceDeclaration>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_append2(self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQXmlStreamNamespaceDeclaration>>::cast_into(l).as_raw_ptr())
    }

    /// <p>Returns the item at index position <i>i</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamNamespaceDeclaration& QVector<QXmlStreamNamespaceDeclaration>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the vector.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_at(
            self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamNamespaceDeclaration& QVector<QXmlStreamNamespaceDeclaration>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn back_mut(
        &mut self,
    ) -> ::cpp_utils::MutRef<crate::QXmlStreamNamespaceDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_back(
            self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamNamespaceDeclaration& QVector<QXmlStreamNamespaceDeclaration>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_back1(
            self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the maximum number of items that can be stored in the vector without forcing a reallocation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamNamespaceDeclaration>::capacity() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#capacity">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the maximum number of items that can be stored in the vector without forcing a reallocation.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s memory usage. In general, you will rarely ever need to call this function. If you want to know how many items are in the vector, call <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p></div>
    pub unsafe fn capacity(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_capacity(
            self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration,
        )
    }

    /// <p>Removes all the elements from the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNamespaceDeclaration>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all the elements from the vector.</p>
    /// <p><b>Note: </b>Until Qt 5.6, this also released the memory used by the vector. From Qt 5.7, the capacity is preserved. To shed all capacity, swap with a default-constructed vector:</p><pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span>T<span class="operator">&gt;</span> v <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span>T<span class="operator">&gt;</span>()<span class="operator">.</span>swap(v);
    ///   Q_ASSERT(v<span class="operator">.</span>capacity() <span class="operator">=</span><span class="operator">=</span> <span class="number">0</span>);
    ///
    /// </pre>
    /// <p>or call <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_clear(
            self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
        )
    }

    /// <p>Returns a const pointer to the data stored in the vector. The pointer can be used to access the items in the vector. The pointer remains valid as long as the vector isn't reallocated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamNamespaceDeclaration* QVector<QXmlStreamNamespaceDeclaration>::constData() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const pointer to the data stored in the vector. The pointer can be used to access the items in the vector. The pointer remains valid as long as the vector isn't reallocated.</p>
    /// <p>This function is mostly useful to pass a vector to a function that accepts a plain C++ array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#data">data</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn const_data(&self) -> ::cpp_utils::Ptr<crate::QXmlStreamNamespaceDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_constData(
                self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns a const reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamNamespaceDeclaration& QVector<QXmlStreamNamespaceDeclaration>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_constFirst(
                self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration,
            );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamNamespaceDeclaration& QVector<QXmlStreamNamespaceDeclaration>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_constLast(
                self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration,
            );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the vector contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamNamespaceDeclaration>::contains(const QXmlStreamNamespaceDeclaration& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the vector contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p></div>
    pub unsafe fn contains(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_contains(self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Assigns <i>other</i> to this vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamNamespaceDeclaration>& QVector<QXmlStreamNamespaceDeclaration>::operator=(const QVector<QXmlStreamNamespaceDeclaration>& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this vector and returns a reference to this vector.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        v: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQXmlStreamNamespaceDeclaration>>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamNamespaceDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_operator_(
                self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
                ::cpp_utils::CastInto::<
                    ::cpp_utils::Ref<crate::QVectorOfQXmlStreamNamespaceDeclaration>,
                >::cast_into(v)
                .as_raw_ptr(),
            );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of occurrences of <i>value</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamNamespaceDeclaration>::count(const QXmlStreamNamespaceDeclaration& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of occurrences of <i>value</i> in the vector.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_count(self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamNamespaceDeclaration>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_count1(
            self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration,
        )
    }

    /// <p>Returns a pointer to the data stored in the vector. The pointer can be used to access and modify the items in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamNamespaceDeclaration* QVector<QXmlStreamNamespaceDeclaration>::data()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the data stored in the vector. The pointer can be used to access and modify the items in the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> vector(<span class="number">10</span>);
    ///   <span class="type">int</span> <span class="operator">*</span>data <span class="operator">=</span> vector<span class="operator">.</span>data();
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> <span class="number">10</span>; <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     data<span class="operator">[</span>i<span class="operator">]</span> <span class="operator">=</span> <span class="number">2</span> <span class="operator">*</span> i;
    ///
    /// </pre>
    /// <p>The pointer remains valid as long as the vector isn't reallocated.</p>
    /// <p>This function is mostly useful to pass a vector to a function that accepts a plain C++ array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constData">constData</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn data_mut(
        &mut self,
    ) -> ::cpp_utils::MutPtr<crate::QXmlStreamNamespaceDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_data(
            self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamNamespaceDeclaration* QVector<QXmlStreamNamespaceDeclaration>::data() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#data-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn data(&self) -> ::cpp_utils::Ptr<crate::QXmlStreamNamespaceDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_data1(
            self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNamespaceDeclaration>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_detach(
            self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), returning <code>true</code> if the vector is empty; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamNamespaceDeclaration>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), returning <code>true</code> if the vector is empty; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_empty(
            self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration,
        )
    }

    /// <p>Returns <code>true</code> if this vector is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamNamespaceDeclaration>::endsWith(const QXmlStreamNamespaceDeclaration& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this vector is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn ends_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_endsWith(self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamNamespaceDeclaration>& QVector<QXmlStreamNamespaceDeclaration>::fill(const QXmlStreamNamespaceDeclaration& t, int size = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">3</span>);
    ///   vector<span class="operator">.</span>fill(<span class="string">"Yes"</span>);
    ///   <span class="comment">// vector: ["Yes", "Yes", "Yes"]</span>
    ///
    ///   vector<span class="operator">.</span>fill(<span class="string">"oh"</span><span class="operator">,</span> <span class="number">5</span>);
    ///   <span class="comment">// vector: ["oh", "oh", "oh", "oh", "oh"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_2a(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamNamespaceDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_fill(self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>::cast_into(t).as_raw_ptr(), size);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamNamespaceDeclaration>& QVector<QXmlStreamNamespaceDeclaration>::fill(const QXmlStreamNamespaceDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">3</span>);
    ///   vector<span class="operator">.</span>fill(<span class="string">"Yes"</span>);
    ///   <span class="comment">// vector: ["Yes", "Yes", "Yes"]</span>
    ///
    ///   vector<span class="operator">.</span>fill(<span class="string">"oh"</span><span class="operator">,</span> <span class="number">5</span>);
    ///   <span class="comment">// vector: ["oh", "oh", "oh", "oh", "oh"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_1a(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamNamespaceDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_fill1(self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>::cast_into(t).as_raw_ptr());
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamNamespaceDeclaration& QVector<QXmlStreamNamespaceDeclaration>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#constFirst">constFirst</a>().</p></div>
    pub unsafe fn first_mut(
        &mut self,
    ) -> ::cpp_utils::MutRef<crate::QXmlStreamNamespaceDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_first(
            self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamNamespaceDeclaration& QVector<QXmlStreamNamespaceDeclaration>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_first1(
                self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration,
            );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamNamespaceDeclaration& QVector<QXmlStreamNamespaceDeclaration>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn front_mut(
        &mut self,
    ) -> ::cpp_utils::MutRef<crate::QXmlStreamNamespaceDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_front(
            self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamNamespaceDeclaration& QVector<QXmlStreamNamespaceDeclaration>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_front1(
                self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration,
            );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamNamespaceDeclaration& QVector<QXmlStreamNamespaceDeclaration>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p>Note that using non-const operators can cause <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to do a deep copy.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QXmlStreamNamespaceDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_operator__2(
                self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
                i,
            );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamNamespaceDeclaration& QVector<QXmlStreamNamespaceDeclaration>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as at(<i>i</i>).</p></div>
    pub unsafe fn index(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_operator__3(
                self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration,
                i,
            );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamNamespaceDeclaration>::indexOf(const QXmlStreamNamespaceDeclaration& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span>);            <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);         <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);         <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"X"</span>);            <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_indexOf(self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>::cast_into(t).as_raw_ptr(), from)
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamNamespaceDeclaration>::indexOf(const QXmlStreamNamespaceDeclaration& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span>);            <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);         <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);         <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"X"</span>);            <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_indexOf1(self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the vector. If <i>i</i> is 0, the value is prepended to the vector. If <i>i</i> is <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), the value is appended to the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNamespaceDeclaration>::insert(int i, const QXmlStreamNamespaceDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the vector. If <i>i</i> is 0, the value is prepended to the vector. If <i>i</i> is <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), the value is appended to the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   vector<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// vector: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p>For large vectors, this operation can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because it requires moving all the items at indexes <i>i</i> and above by one position further in memory. If you want a container class that provides a fast insert() function, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>().</p></div>
    pub unsafe fn insert_2a(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_insert(self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration, i, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNamespaceDeclaration>::insert(int i, int n, const QXmlStreamNamespaceDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>count</i> copies of <i>value</i> at index position <i>i</i> in the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">double</span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">2.718</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">1.442</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">0.4342</span>;
    ///   vector<span class="operator">.</span>insert(<span class="number">1</span><span class="operator">,</span> <span class="number">3</span><span class="operator">,</span> <span class="number">9.9</span>);
    ///   <span class="comment">// vector: [2.718, 9.9, 9.9, 9.9, 1.442, 0.4342]</span>
    ///
    /// </pre></div>
    pub unsafe fn insert_3a(
        &mut self,
        i: ::std::os::raw::c_int,
        n: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_insert1(self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration, i, n, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamNamespaceDeclaration>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_isDetached(
            self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration,
        )
    }

    /// <p>Returns <code>true</code> if the vector has size 0; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamNamespaceDeclaration>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the vector has size 0; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_isEmpty(
            self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamNamespaceDeclaration>::isSharedWith(const QVector<QXmlStreamNamespaceDeclaration>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::QVectorOfQXmlStreamNamespaceDeclaration>,
        >,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_isSharedWith(self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQXmlStreamNamespaceDeclaration>>::cast_into(other).as_raw_ptr())
    }

    /// <p>Returns a reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamNamespaceDeclaration& QVector<QXmlStreamNamespaceDeclaration>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#constLast">constLast</a>().</p></div>
    pub unsafe fn last_mut(
        &mut self,
    ) -> ::cpp_utils::MutRef<crate::QXmlStreamNamespaceDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_last(
            self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamNamespaceDeclaration& QVector<QXmlStreamNamespaceDeclaration>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_last1(
            self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamNamespaceDeclaration>::lastIndexOf(const QXmlStreamNamespaceDeclaration& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);        <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);     <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);     <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_lastIndexOf(self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>::cast_into(t).as_raw_ptr(), from)
    }

    /// <p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamNamespaceDeclaration>::lastIndexOf(const QXmlStreamNamespaceDeclaration& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);        <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);     <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);     <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_lastIndexOf1(self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamNamespaceDeclaration>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#length">QList::length</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_length(
            self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration,
        )
    }

    /// <p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamNamespaceDeclaration> QVector<QXmlStreamNamespaceDeclaration>::mid(int pos, int len = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        len: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamNamespaceDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_mid(
            self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration,
            pos,
            len,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamNamespaceDeclaration> QVector<QXmlStreamNamespaceDeclaration>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamNamespaceDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_mid1(
            self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNamespaceDeclaration>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">QList::move</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_move(
            self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
            from,
            to,
        )
    }

    /// <p>Constructs an empty vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QXmlStreamNamespaceDeclaration>::QVector()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamNamespaceDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_QVector();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a vector with an initial size of <i>size</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QXmlStreamNamespaceDeclaration>::QVector(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a vector with an initial size of <i>size</i> elements.</p>
    /// <p>The elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn new_1a(
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamNamespaceDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_QVector1(size);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a vector with an initial size of <i>size</i> elements. Each element is initialized with <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QXmlStreamNamespaceDeclaration>::QVector(int size, const QXmlStreamNamespaceDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a vector with an initial size of <i>size</i> elements. Each element is initialized with <i>value</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn new_2a(
        size: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamNamespaceDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_QVector2(size, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>::cast_into(t).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QXmlStreamNamespaceDeclaration>::QVector(const QVector<QXmlStreamNamespaceDeclaration>& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        v: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQXmlStreamNamespaceDeclaration>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamNamespaceDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_QVector3(
                ::cpp_utils::CastInto::<
                    ::cpp_utils::Ref<crate::QVectorOfQXmlStreamNamespaceDeclaration>,
                >::cast_into(v)
                .as_raw_ptr(),
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNamespaceDeclaration>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_pop_back(
            self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNamespaceDeclaration>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_pop_front(
            self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
        )
    }

    /// <p>Inserts <i>value</i> at the beginning of the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNamespaceDeclaration>::prepend(const QXmlStreamNamespaceDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   vector<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   vector<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// vector: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as vector.insert(0, <i>value</i>).</p>
    /// <p>For large vectors, this operation can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because it requires moving all the items in the vector by one position further in memory. If you want a container class that provides a fast prepend() function, use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> or <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_prepend(self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to append(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNamespaceDeclaration>::push_back(const QXmlStreamNamespaceDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to append(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_push_back(self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to prepend(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNamespaceDeclaration>::push_front(const QXmlStreamNamespaceDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to prepend(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_push_front(self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNamespaceDeclaration>::remove(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#remove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes the element at index position <i>i</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn remove_1a(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_remove(
            self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
            i,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNamespaceDeclaration>::remove(int i, int n)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#remove-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes <i>count</i> elements from the middle of the vector, starting at index position <i>i</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn remove_2a(&mut self, i: ::std::os::raw::c_int, n: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_remove1(
            self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
            i,
            n,
        )
    }

    /// <p>Removes all elements that compare equal to <i>t</i> from the vector. Returns the number of elements removed, if any.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamNamespaceDeclaration>::removeAll(const QXmlStreamNamespaceDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all elements that compare equal to <i>t</i> from the vector. Returns the number of elements removed, if any.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#removeOne">removeOne</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">QList::removeAll</a>().</p></div>
    pub unsafe fn remove_all(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_removeAll(self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Removes the element at index position <i>i</i>. Equivalent to</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNamespaceDeclaration>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the element at index position <i>i</i>. Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   remove(i);
    ///
    /// </pre>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">QList::removeAt</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_removeAt(
            self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
            i,
        )
    }

    /// <p>Removes the first item in the vector. Calling this function is equivalent to calling remove(0). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNamespaceDeclaration>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the vector. Calling this function is equivalent to calling remove(0). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_removeFirst(
            self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
        )
    }

    /// <p>Removes the last item in the vector. Calling this function is equivalent to calling remove(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() - 1). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNamespaceDeclaration>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the vector. Calling this function is equivalent to calling remove(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() - 1). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_removeLast(
            self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
        )
    }

    /// <p>Removes the first element that compares equal to <i>t</i> from the vector. Returns whether an element was, in fact, removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamNamespaceDeclaration>::removeOne(const QXmlStreamNamespaceDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeOne">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first element that compares equal to <i>t</i> from the vector. Returns whether an element was, in fact, removed.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#removeAll">removeAll</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">QList::removeOne</a>().</p></div>
    pub unsafe fn remove_one(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_removeOne(self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNamespaceDeclaration>::replace(int i, const QXmlStreamNamespaceDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_replace(self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration, i, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Attempts to allocate memory for at least <i>size</i> elements. If you know in advance how large the vector will be, you should call this function to prevent reallocations and memory fragmentation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNamespaceDeclaration>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to allocate memory for at least <i>size</i> elements. If you know in advance how large the vector will be, you should call this function to prevent reallocations and memory fragmentation.</p>
    /// <p>If <i>size</i> is an underestimate, the worst that will happen is that the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> will be a bit slower. If <i>size</i> is an overestimate, you may have used more memory than the normal <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> growth strategy would have allocated—or you may have used less.</p>
    /// <p>An alternative to reserve() is calling <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>(). Whether or not that is faster than reserve() depends on the element type, because <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() default-constructs all elements, and requires assignment to existing entries rather than calling <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(), which copy- or move-constructs. For simple types, like <code>int</code> or <code>double</code>, <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() is typically faster, but for anything more complex, you should prefer reserve().</p>
    /// <p><b>Warning:</b> If the size passed to <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() was underestimated, you run out of allocated space and into undefined behavior. This problem does not exist with reserve(), because it treats the size as just a hint.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>().</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_reserve(
            self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
            size,
        )
    }

    /// <p>Sets the size of the vector to <i>size</i>. If <i>size</i> is greater than the current size, elements are added to the end; the new elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If <i>size</i> is less than the current size, elements are removed from the end.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNamespaceDeclaration>::resize(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#resize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the size of the vector to <i>size</i>. If <i>size</i> is greater than the current size, elements are added to the end; the new elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If <i>size</i> is less than the current size, elements are removed from the end.</p>
    /// <p>Since Qt 5.6, resize() doesn't shrink the capacity anymore. To shed excess capacity, use <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p></div>
    pub unsafe fn resize(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_resize(
            self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNamespaceDeclaration>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_setSharable(
            self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
            sharable,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNamespaceDeclaration>::shrink_to_fit()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#shrink_to_fit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn shrink_to_fit(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_shrink_to_fit(
            self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
        )
    }

    /// <p>Returns the number of items in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamNamespaceDeclaration>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_size(
            self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration,
        )
    }

    /// <p>Releases any memory not required to store the items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNamespaceDeclaration>::squeeze()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Releases any memory not required to store the items.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s memory usage. In general, you will rarely ever need to call this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>().</p></div>
    pub unsafe fn squeeze(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_squeeze(
            self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
        )
    }

    /// <p>Returns <code>true</code> if this vector is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamNamespaceDeclaration>::startsWith(const QXmlStreamNamespaceDeclaration& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this vector is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn starts_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_startsWith(self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Swaps vector <i>other</i> with this vector. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNamespaceDeclaration>::swap(QVector<QXmlStreamNamespaceDeclaration>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps vector <i>other</i> with this vector. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<
            ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamNamespaceDeclaration>,
        >,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_swap(
            self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
            ::cpp_utils::CastInto::<
                ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamNamespaceDeclaration>,
            >::cast_into(other)
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Removes the element at index position <i>i</i> and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamNamespaceDeclaration QVector<QXmlStreamNamespaceDeclaration>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the element at index position <i>i</i> and returns it.</p>
    /// <p>Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   T t <span class="operator">=</span> at(i);
    ///   remove(i);
    ///   <span class="keyword">return</span> t;
    ///
    /// </pre>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">QList::takeAt</a>().</p></div>
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamNamespaceDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_takeAt(
                self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
                i,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the first item in the vector and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamNamespaceDeclaration QVector<QXmlStreamNamespaceDeclaration>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the vector and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(
        &mut self,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamNamespaceDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_takeFirst(
                self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the last item in the list and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamNamespaceDeclaration QVector<QXmlStreamNamespaceDeclaration>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(
        &mut self,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamNamespaceDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_takeLast(
                self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value at index position <i>i</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamNamespaceDeclaration QVector<QXmlStreamNamespaceDeclaration>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the vector.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that <i>i</i> is within bounds, you can use <a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamNamespaceDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_value(
            self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamNamespaceDeclaration QVector<QXmlStreamNamespaceDeclaration>::value(int i, const QXmlStreamNamespaceDeclaration& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>,
        >,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamNamespaceDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_value1(self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration, i, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>::cast_into(default_value).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> class is a template class that provides a dynamic array.</p>
///
/// C++ class: <span style='color: green;'>```QVector<QXmlStreamNotationDeclaration>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qvector.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> class is a template class that provides a dynamic array.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores its items in adjacent memory locations and provides fast index-based access.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Here's an example of a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> that stores <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerVector;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> stringVector;
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> stores its items in a vector (array). Typically, vectors are created with an initial size. For example, the following code constructs a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> with 200 elements:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">200</span>);
///
/// </pre>
/// <p>The elements are automatically initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you want to initialize the vector with a different value, pass that value as the second argument to the constructor:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">200</span><span class="operator">,</span> <span class="string">"Pass"</span>);
///
/// </pre>
/// <p>You can also call <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>() at any time to fill the vector with a value.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const vectors, operator[]() returns a reference to the item that can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (vector<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Liz"</span>)
/// &#32;     vector<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Elizabeth"</span>;
///
/// </pre>
/// <p>For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> vector<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (vector<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Alfonso"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Alfonso at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>Another way to access the data stored in a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> is to call <a href="http://doc.qt.io/qt-5/qvector.html#data">data</a>(). The function returns a pointer to the first item in the vector. You can use the pointer to directly access and modify the elements stored in the vector. The pointer is also useful if you need to pass a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to a function that accepts a plain C++ array.</p>
/// <p>If you want to find all occurrences of a particular value in a vector, use <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of the matching item if they found one; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> vector<span class="operator">.</span>indexOf(<span class="string">"Harumi"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Harumi is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a vector contains a particular value, use <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the vector, use <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(). With the exception of <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), these functions can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>) for large vectors, because they require moving many items in the vector by one position in memory. If you want a container class that provides fast insertion/removal in the middle, use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> or <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
/// <p>Unlike plain C++ arrays, QVectors can be resized at any time by calling <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>(). If the new size is larger than the old size, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> might need to reallocate the whole vector. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> tries to reduce the number of reallocations by preallocating up to twice as much memory as the actual data needs.</p>
/// <p>If you know in advance approximately how many items the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> will contain, you can call <a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>(), asking <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to preallocate a certain amount of memory. You can also call <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>() to find out how much memory <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> actually allocated.</p>
/// <p>Note that using non-const operators and functions can cause <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to do a deep copy of the data. This is due to <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicit sharing</a>.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qvectoriterator.html">QVectorIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablevectoriterator.html">QMutableVectorIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qvector.html#const_iterator-typedef">QVector::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qvector.html#iterator-typedefx">QVector::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>.</p>
/// <p>In addition to <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>, Qt also provides <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a>, a very low-level class with little functionality that is optimized for speed.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <a name="more-information-on-using-qt-containers"></a>
/// <h4>More Information on Using Qt Containers</h4>
/// <p>For a detailed discussion comparing Qt containers with each other and with STL containers, see <a href="http://marcmutz.wordpress.com/effective-qt/containers/">Understand the Qt Containers</a>.</p></div>
#[repr(C)]
pub struct QVectorOfQXmlStreamNotationDeclaration {
    _unused: u8,
}
impl QVectorOfQXmlStreamNotationDeclaration {
    /// <p>Inserts <i>value</i> at the end of the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNotationDeclaration>::append(const QXmlStreamNotationDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   vector<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> three <span class="operator">=</span> <span class="string">"three"</span>;
    ///   vector<span class="operator">.</span>append(three);
    ///   <span class="comment">// vector: ["one", "two", "three"]</span>
    ///   <span class="comment">// three: "three"</span>
    ///
    /// </pre>
    /// <p>This is the same as calling resize(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() + 1) and assigning <i>value</i> to the new last element in the vector.</p>
    /// <p>This operation is relatively fast, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> typically allocates more memory than necessary, so it can grow without reallocating the entire vector each time.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>().</p></div>
    pub unsafe fn append_q_xml_stream_notation_declaration(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_append(self as *mut crate::QVectorOfQXmlStreamNotationDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNotationDeclaration>::append(const QVector<QXmlStreamNotationDeclaration>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#append-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> vector to this vector.</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_vector_of_q_xml_stream_notation_declaration(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQXmlStreamNotationDeclaration>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_append2(self as *mut crate::QVectorOfQXmlStreamNotationDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQXmlStreamNotationDeclaration>>::cast_into(l).as_raw_ptr())
    }

    /// <p>Returns the item at index position <i>i</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamNotationDeclaration& QVector<QXmlStreamNotationDeclaration>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the vector.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_at(
            self as *const crate::QVectorOfQXmlStreamNotationDeclaration,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamNotationDeclaration& QVector<QXmlStreamNotationDeclaration>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::MutRef<crate::QXmlStreamNotationDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_back(
            self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamNotationDeclaration& QVector<QXmlStreamNotationDeclaration>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_back1(
            self as *const crate::QVectorOfQXmlStreamNotationDeclaration,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the maximum number of items that can be stored in the vector without forcing a reallocation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamNotationDeclaration>::capacity() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#capacity">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the maximum number of items that can be stored in the vector without forcing a reallocation.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s memory usage. In general, you will rarely ever need to call this function. If you want to know how many items are in the vector, call <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p></div>
    pub unsafe fn capacity(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_capacity(
            self as *const crate::QVectorOfQXmlStreamNotationDeclaration,
        )
    }

    /// <p>Removes all the elements from the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNotationDeclaration>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all the elements from the vector.</p>
    /// <p><b>Note: </b>Until Qt 5.6, this also released the memory used by the vector. From Qt 5.7, the capacity is preserved. To shed all capacity, swap with a default-constructed vector:</p><pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span>T<span class="operator">&gt;</span> v <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span>T<span class="operator">&gt;</span>()<span class="operator">.</span>swap(v);
    ///   Q_ASSERT(v<span class="operator">.</span>capacity() <span class="operator">=</span><span class="operator">=</span> <span class="number">0</span>);
    ///
    /// </pre>
    /// <p>or call <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_clear(
            self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
        )
    }

    /// <p>Returns a const pointer to the data stored in the vector. The pointer can be used to access the items in the vector. The pointer remains valid as long as the vector isn't reallocated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamNotationDeclaration* QVector<QXmlStreamNotationDeclaration>::constData() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const pointer to the data stored in the vector. The pointer can be used to access the items in the vector. The pointer remains valid as long as the vector isn't reallocated.</p>
    /// <p>This function is mostly useful to pass a vector to a function that accepts a plain C++ array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#data">data</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn const_data(&self) -> ::cpp_utils::Ptr<crate::QXmlStreamNotationDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_constData(
                self as *const crate::QVectorOfQXmlStreamNotationDeclaration,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns a const reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamNotationDeclaration& QVector<QXmlStreamNotationDeclaration>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_constFirst(
                self as *const crate::QVectorOfQXmlStreamNotationDeclaration,
            );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamNotationDeclaration& QVector<QXmlStreamNotationDeclaration>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_constLast(
                self as *const crate::QVectorOfQXmlStreamNotationDeclaration,
            );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the vector contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamNotationDeclaration>::contains(const QXmlStreamNotationDeclaration& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the vector contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p></div>
    pub unsafe fn contains(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_contains(self as *const crate::QVectorOfQXmlStreamNotationDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Assigns <i>other</i> to this vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamNotationDeclaration>& QVector<QXmlStreamNotationDeclaration>::operator=(const QVector<QXmlStreamNotationDeclaration>& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this vector and returns a reference to this vector.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        v: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQXmlStreamNotationDeclaration>>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamNotationDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_operator_(
                self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
                ::cpp_utils::CastInto::<
                    ::cpp_utils::Ref<crate::QVectorOfQXmlStreamNotationDeclaration>,
                >::cast_into(v)
                .as_raw_ptr(),
            );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of occurrences of <i>value</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamNotationDeclaration>::count(const QXmlStreamNotationDeclaration& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of occurrences of <i>value</i> in the vector.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_count(self as *const crate::QVectorOfQXmlStreamNotationDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamNotationDeclaration>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_count1(
            self as *const crate::QVectorOfQXmlStreamNotationDeclaration,
        )
    }

    /// <p>Returns a pointer to the data stored in the vector. The pointer can be used to access and modify the items in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamNotationDeclaration* QVector<QXmlStreamNotationDeclaration>::data()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the data stored in the vector. The pointer can be used to access and modify the items in the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> vector(<span class="number">10</span>);
    ///   <span class="type">int</span> <span class="operator">*</span>data <span class="operator">=</span> vector<span class="operator">.</span>data();
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> <span class="number">10</span>; <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     data<span class="operator">[</span>i<span class="operator">]</span> <span class="operator">=</span> <span class="number">2</span> <span class="operator">*</span> i;
    ///
    /// </pre>
    /// <p>The pointer remains valid as long as the vector isn't reallocated.</p>
    /// <p>This function is mostly useful to pass a vector to a function that accepts a plain C++ array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constData">constData</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn data_mut(&mut self) -> ::cpp_utils::MutPtr<crate::QXmlStreamNotationDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_data(
            self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamNotationDeclaration* QVector<QXmlStreamNotationDeclaration>::data() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#data-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn data(&self) -> ::cpp_utils::Ptr<crate::QXmlStreamNotationDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_data1(
            self as *const crate::QVectorOfQXmlStreamNotationDeclaration,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNotationDeclaration>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_detach(
            self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), returning <code>true</code> if the vector is empty; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamNotationDeclaration>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), returning <code>true</code> if the vector is empty; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_empty(
            self as *const crate::QVectorOfQXmlStreamNotationDeclaration,
        )
    }

    /// <p>Returns <code>true</code> if this vector is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamNotationDeclaration>::endsWith(const QXmlStreamNotationDeclaration& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this vector is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn ends_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_endsWith(self as *const crate::QVectorOfQXmlStreamNotationDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamNotationDeclaration>& QVector<QXmlStreamNotationDeclaration>::fill(const QXmlStreamNotationDeclaration& t, int size = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">3</span>);
    ///   vector<span class="operator">.</span>fill(<span class="string">"Yes"</span>);
    ///   <span class="comment">// vector: ["Yes", "Yes", "Yes"]</span>
    ///
    ///   vector<span class="operator">.</span>fill(<span class="string">"oh"</span><span class="operator">,</span> <span class="number">5</span>);
    ///   <span class="comment">// vector: ["oh", "oh", "oh", "oh", "oh"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_2a(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamNotationDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_fill(self as *mut crate::QVectorOfQXmlStreamNotationDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>::cast_into(t).as_raw_ptr(), size);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamNotationDeclaration>& QVector<QXmlStreamNotationDeclaration>::fill(const QXmlStreamNotationDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">3</span>);
    ///   vector<span class="operator">.</span>fill(<span class="string">"Yes"</span>);
    ///   <span class="comment">// vector: ["Yes", "Yes", "Yes"]</span>
    ///
    ///   vector<span class="operator">.</span>fill(<span class="string">"oh"</span><span class="operator">,</span> <span class="number">5</span>);
    ///   <span class="comment">// vector: ["oh", "oh", "oh", "oh", "oh"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_1a(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamNotationDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_fill1(self as *mut crate::QVectorOfQXmlStreamNotationDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>::cast_into(t).as_raw_ptr());
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamNotationDeclaration& QVector<QXmlStreamNotationDeclaration>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#constFirst">constFirst</a>().</p></div>
    pub unsafe fn first_mut(
        &mut self,
    ) -> ::cpp_utils::MutRef<crate::QXmlStreamNotationDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_first(
            self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamNotationDeclaration& QVector<QXmlStreamNotationDeclaration>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_first1(
            self as *const crate::QVectorOfQXmlStreamNotationDeclaration,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamNotationDeclaration& QVector<QXmlStreamNotationDeclaration>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn front_mut(
        &mut self,
    ) -> ::cpp_utils::MutRef<crate::QXmlStreamNotationDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_front(
            self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamNotationDeclaration& QVector<QXmlStreamNotationDeclaration>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_front1(
            self as *const crate::QVectorOfQXmlStreamNotationDeclaration,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamNotationDeclaration& QVector<QXmlStreamNotationDeclaration>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p>Note that using non-const operators can cause <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to do a deep copy.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QXmlStreamNotationDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_operator__2(
                self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
                i,
            );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamNotationDeclaration& QVector<QXmlStreamNotationDeclaration>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as at(<i>i</i>).</p></div>
    pub unsafe fn index(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_operator__3(
                self as *const crate::QVectorOfQXmlStreamNotationDeclaration,
                i,
            );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamNotationDeclaration>::indexOf(const QXmlStreamNotationDeclaration& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span>);            <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);         <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);         <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"X"</span>);            <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_indexOf(self as *const crate::QVectorOfQXmlStreamNotationDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>::cast_into(t).as_raw_ptr(), from)
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamNotationDeclaration>::indexOf(const QXmlStreamNotationDeclaration& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span>);            <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);         <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);         <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"X"</span>);            <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_indexOf1(self as *const crate::QVectorOfQXmlStreamNotationDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the vector. If <i>i</i> is 0, the value is prepended to the vector. If <i>i</i> is <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), the value is appended to the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNotationDeclaration>::insert(int i, const QXmlStreamNotationDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the vector. If <i>i</i> is 0, the value is prepended to the vector. If <i>i</i> is <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), the value is appended to the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   vector<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// vector: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p>For large vectors, this operation can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because it requires moving all the items at indexes <i>i</i> and above by one position further in memory. If you want a container class that provides a fast insert() function, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>().</p></div>
    pub unsafe fn insert_2a(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_insert(self as *mut crate::QVectorOfQXmlStreamNotationDeclaration, i, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNotationDeclaration>::insert(int i, int n, const QXmlStreamNotationDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>count</i> copies of <i>value</i> at index position <i>i</i> in the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">double</span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">2.718</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">1.442</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">0.4342</span>;
    ///   vector<span class="operator">.</span>insert(<span class="number">1</span><span class="operator">,</span> <span class="number">3</span><span class="operator">,</span> <span class="number">9.9</span>);
    ///   <span class="comment">// vector: [2.718, 9.9, 9.9, 9.9, 1.442, 0.4342]</span>
    ///
    /// </pre></div>
    pub unsafe fn insert_3a(
        &mut self,
        i: ::std::os::raw::c_int,
        n: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_insert1(self as *mut crate::QVectorOfQXmlStreamNotationDeclaration, i, n, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamNotationDeclaration>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_isDetached(
            self as *const crate::QVectorOfQXmlStreamNotationDeclaration,
        )
    }

    /// <p>Returns <code>true</code> if the vector has size 0; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamNotationDeclaration>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the vector has size 0; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_isEmpty(
            self as *const crate::QVectorOfQXmlStreamNotationDeclaration,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamNotationDeclaration>::isSharedWith(const QVector<QXmlStreamNotationDeclaration>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::QVectorOfQXmlStreamNotationDeclaration>,
        >,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_isSharedWith(self as *const crate::QVectorOfQXmlStreamNotationDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQXmlStreamNotationDeclaration>>::cast_into(other).as_raw_ptr())
    }

    /// <p>Returns a reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamNotationDeclaration& QVector<QXmlStreamNotationDeclaration>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#constLast">constLast</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<crate::QXmlStreamNotationDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_last(
            self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamNotationDeclaration& QVector<QXmlStreamNotationDeclaration>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_last1(
            self as *const crate::QVectorOfQXmlStreamNotationDeclaration,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamNotationDeclaration>::lastIndexOf(const QXmlStreamNotationDeclaration& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);        <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);     <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);     <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_lastIndexOf(self as *const crate::QVectorOfQXmlStreamNotationDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>::cast_into(t).as_raw_ptr(), from)
    }

    /// <p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamNotationDeclaration>::lastIndexOf(const QXmlStreamNotationDeclaration& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);        <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);     <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);     <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_lastIndexOf1(self as *const crate::QVectorOfQXmlStreamNotationDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamNotationDeclaration>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#length">QList::length</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_length(
            self as *const crate::QVectorOfQXmlStreamNotationDeclaration,
        )
    }

    /// <p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamNotationDeclaration> QVector<QXmlStreamNotationDeclaration>::mid(int pos, int len = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        len: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamNotationDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_mid(
            self as *const crate::QVectorOfQXmlStreamNotationDeclaration,
            pos,
            len,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamNotationDeclaration> QVector<QXmlStreamNotationDeclaration>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamNotationDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_mid1(
            self as *const crate::QVectorOfQXmlStreamNotationDeclaration,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNotationDeclaration>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">QList::move</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_move(
            self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
            from,
            to,
        )
    }

    /// <p>Constructs an empty vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QXmlStreamNotationDeclaration>::QVector()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamNotationDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_QVector();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a vector with an initial size of <i>size</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QXmlStreamNotationDeclaration>::QVector(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a vector with an initial size of <i>size</i> elements.</p>
    /// <p>The elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn new_1a(
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamNotationDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_QVector1(size);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a vector with an initial size of <i>size</i> elements. Each element is initialized with <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QXmlStreamNotationDeclaration>::QVector(int size, const QXmlStreamNotationDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a vector with an initial size of <i>size</i> elements. Each element is initialized with <i>value</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn new_2a(
        size: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamNotationDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_QVector2(size, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>::cast_into(t).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QXmlStreamNotationDeclaration>::QVector(const QVector<QXmlStreamNotationDeclaration>& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        v: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQXmlStreamNotationDeclaration>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamNotationDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_QVector3(
                ::cpp_utils::CastInto::<
                    ::cpp_utils::Ref<crate::QVectorOfQXmlStreamNotationDeclaration>,
                >::cast_into(v)
                .as_raw_ptr(),
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNotationDeclaration>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_pop_back(
            self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNotationDeclaration>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_pop_front(
            self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
        )
    }

    /// <p>Inserts <i>value</i> at the beginning of the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNotationDeclaration>::prepend(const QXmlStreamNotationDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   vector<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   vector<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// vector: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as vector.insert(0, <i>value</i>).</p>
    /// <p>For large vectors, this operation can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because it requires moving all the items in the vector by one position further in memory. If you want a container class that provides a fast prepend() function, use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> or <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_prepend(self as *mut crate::QVectorOfQXmlStreamNotationDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to append(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNotationDeclaration>::push_back(const QXmlStreamNotationDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to append(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_push_back(self as *mut crate::QVectorOfQXmlStreamNotationDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to prepend(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNotationDeclaration>::push_front(const QXmlStreamNotationDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to prepend(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_push_front(self as *mut crate::QVectorOfQXmlStreamNotationDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNotationDeclaration>::remove(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#remove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes the element at index position <i>i</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn remove_1a(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_remove(
            self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
            i,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNotationDeclaration>::remove(int i, int n)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#remove-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes <i>count</i> elements from the middle of the vector, starting at index position <i>i</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn remove_2a(&mut self, i: ::std::os::raw::c_int, n: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_remove1(
            self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
            i,
            n,
        )
    }

    /// <p>Removes all elements that compare equal to <i>t</i> from the vector. Returns the number of elements removed, if any.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamNotationDeclaration>::removeAll(const QXmlStreamNotationDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all elements that compare equal to <i>t</i> from the vector. Returns the number of elements removed, if any.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#removeOne">removeOne</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">QList::removeAll</a>().</p></div>
    pub unsafe fn remove_all(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_removeAll(self as *mut crate::QVectorOfQXmlStreamNotationDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Removes the element at index position <i>i</i>. Equivalent to</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNotationDeclaration>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the element at index position <i>i</i>. Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   remove(i);
    ///
    /// </pre>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">QList::removeAt</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_removeAt(
            self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
            i,
        )
    }

    /// <p>Removes the first item in the vector. Calling this function is equivalent to calling remove(0). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNotationDeclaration>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the vector. Calling this function is equivalent to calling remove(0). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_removeFirst(
            self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
        )
    }

    /// <p>Removes the last item in the vector. Calling this function is equivalent to calling remove(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() - 1). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNotationDeclaration>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the vector. Calling this function is equivalent to calling remove(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() - 1). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_removeLast(
            self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
        )
    }

    /// <p>Removes the first element that compares equal to <i>t</i> from the vector. Returns whether an element was, in fact, removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamNotationDeclaration>::removeOne(const QXmlStreamNotationDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeOne">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first element that compares equal to <i>t</i> from the vector. Returns whether an element was, in fact, removed.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#removeAll">removeAll</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">QList::removeOne</a>().</p></div>
    pub unsafe fn remove_one(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_removeOne(self as *mut crate::QVectorOfQXmlStreamNotationDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNotationDeclaration>::replace(int i, const QXmlStreamNotationDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_replace(self as *mut crate::QVectorOfQXmlStreamNotationDeclaration, i, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Attempts to allocate memory for at least <i>size</i> elements. If you know in advance how large the vector will be, you should call this function to prevent reallocations and memory fragmentation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNotationDeclaration>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to allocate memory for at least <i>size</i> elements. If you know in advance how large the vector will be, you should call this function to prevent reallocations and memory fragmentation.</p>
    /// <p>If <i>size</i> is an underestimate, the worst that will happen is that the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> will be a bit slower. If <i>size</i> is an overestimate, you may have used more memory than the normal <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> growth strategy would have allocated—or you may have used less.</p>
    /// <p>An alternative to reserve() is calling <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>(). Whether or not that is faster than reserve() depends on the element type, because <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() default-constructs all elements, and requires assignment to existing entries rather than calling <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(), which copy- or move-constructs. For simple types, like <code>int</code> or <code>double</code>, <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() is typically faster, but for anything more complex, you should prefer reserve().</p>
    /// <p><b>Warning:</b> If the size passed to <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() was underestimated, you run out of allocated space and into undefined behavior. This problem does not exist with reserve(), because it treats the size as just a hint.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>().</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_reserve(
            self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
            size,
        )
    }

    /// <p>Sets the size of the vector to <i>size</i>. If <i>size</i> is greater than the current size, elements are added to the end; the new elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If <i>size</i> is less than the current size, elements are removed from the end.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNotationDeclaration>::resize(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#resize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the size of the vector to <i>size</i>. If <i>size</i> is greater than the current size, elements are added to the end; the new elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If <i>size</i> is less than the current size, elements are removed from the end.</p>
    /// <p>Since Qt 5.6, resize() doesn't shrink the capacity anymore. To shed excess capacity, use <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p></div>
    pub unsafe fn resize(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_resize(
            self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNotationDeclaration>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_setSharable(
            self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
            sharable,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNotationDeclaration>::shrink_to_fit()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#shrink_to_fit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn shrink_to_fit(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_shrink_to_fit(
            self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
        )
    }

    /// <p>Returns the number of items in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamNotationDeclaration>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_size(
            self as *const crate::QVectorOfQXmlStreamNotationDeclaration,
        )
    }

    /// <p>Releases any memory not required to store the items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNotationDeclaration>::squeeze()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Releases any memory not required to store the items.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s memory usage. In general, you will rarely ever need to call this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>().</p></div>
    pub unsafe fn squeeze(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_squeeze(
            self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
        )
    }

    /// <p>Returns <code>true</code> if this vector is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamNotationDeclaration>::startsWith(const QXmlStreamNotationDeclaration& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this vector is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn starts_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_startsWith(self as *const crate::QVectorOfQXmlStreamNotationDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Swaps vector <i>other</i> with this vector. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamNotationDeclaration>::swap(QVector<QXmlStreamNotationDeclaration>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps vector <i>other</i> with this vector. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<
            ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamNotationDeclaration>,
        >,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_swap(
            self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
            ::cpp_utils::CastInto::<
                ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamNotationDeclaration>,
            >::cast_into(other)
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Removes the element at index position <i>i</i> and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamNotationDeclaration QVector<QXmlStreamNotationDeclaration>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the element at index position <i>i</i> and returns it.</p>
    /// <p>Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   T t <span class="operator">=</span> at(i);
    ///   remove(i);
    ///   <span class="keyword">return</span> t;
    ///
    /// </pre>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">QList::takeAt</a>().</p></div>
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamNotationDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_takeAt(
            self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the first item in the vector and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamNotationDeclaration QVector<QXmlStreamNotationDeclaration>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the vector and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(
        &mut self,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamNotationDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_takeFirst(
                self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the last item in the list and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamNotationDeclaration QVector<QXmlStreamNotationDeclaration>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(
        &mut self,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamNotationDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_takeLast(
                self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value at index position <i>i</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamNotationDeclaration QVector<QXmlStreamNotationDeclaration>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the vector.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that <i>i</i> is within bounds, you can use <a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamNotationDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_value(
            self as *const crate::QVectorOfQXmlStreamNotationDeclaration,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamNotationDeclaration QVector<QXmlStreamNotationDeclaration>::value(int i, const QXmlStreamNotationDeclaration& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>,
        >,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamNotationDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_value1(self as *const crate::QVectorOfQXmlStreamNotationDeclaration, i, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>::cast_into(default_value).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> class is a template class that provides a dynamic array.</p>
///
/// C++ class: <span style='color: green;'>```QVector<QXmlStreamEntityDeclaration>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qvector.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> class is a template class that provides a dynamic array.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores its items in adjacent memory locations and provides fast index-based access.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Here's an example of a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> that stores <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerVector;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> stringVector;
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> stores its items in a vector (array). Typically, vectors are created with an initial size. For example, the following code constructs a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> with 200 elements:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">200</span>);
///
/// </pre>
/// <p>The elements are automatically initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you want to initialize the vector with a different value, pass that value as the second argument to the constructor:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">200</span><span class="operator">,</span> <span class="string">"Pass"</span>);
///
/// </pre>
/// <p>You can also call <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>() at any time to fill the vector with a value.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const vectors, operator[]() returns a reference to the item that can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (vector<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Liz"</span>)
/// &#32;     vector<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Elizabeth"</span>;
///
/// </pre>
/// <p>For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> vector<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (vector<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Alfonso"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Alfonso at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>Another way to access the data stored in a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> is to call <a href="http://doc.qt.io/qt-5/qvector.html#data">data</a>(). The function returns a pointer to the first item in the vector. You can use the pointer to directly access and modify the elements stored in the vector. The pointer is also useful if you need to pass a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to a function that accepts a plain C++ array.</p>
/// <p>If you want to find all occurrences of a particular value in a vector, use <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of the matching item if they found one; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> vector<span class="operator">.</span>indexOf(<span class="string">"Harumi"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Harumi is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a vector contains a particular value, use <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the vector, use <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(). With the exception of <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), these functions can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>) for large vectors, because they require moving many items in the vector by one position in memory. If you want a container class that provides fast insertion/removal in the middle, use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> or <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
/// <p>Unlike plain C++ arrays, QVectors can be resized at any time by calling <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>(). If the new size is larger than the old size, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> might need to reallocate the whole vector. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> tries to reduce the number of reallocations by preallocating up to twice as much memory as the actual data needs.</p>
/// <p>If you know in advance approximately how many items the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> will contain, you can call <a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>(), asking <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to preallocate a certain amount of memory. You can also call <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>() to find out how much memory <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> actually allocated.</p>
/// <p>Note that using non-const operators and functions can cause <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to do a deep copy of the data. This is due to <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicit sharing</a>.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qvectoriterator.html">QVectorIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablevectoriterator.html">QMutableVectorIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qvector.html#const_iterator-typedef">QVector::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qvector.html#iterator-typedefx">QVector::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>.</p>
/// <p>In addition to <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>, Qt also provides <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a>, a very low-level class with little functionality that is optimized for speed.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <a name="more-information-on-using-qt-containers"></a>
/// <h4>More Information on Using Qt Containers</h4>
/// <p>For a detailed discussion comparing Qt containers with each other and with STL containers, see <a href="http://marcmutz.wordpress.com/effective-qt/containers/">Understand the Qt Containers</a>.</p></div>
#[repr(C)]
pub struct QVectorOfQXmlStreamEntityDeclaration {
    _unused: u8,
}
impl QVectorOfQXmlStreamEntityDeclaration {
    /// <p>Inserts <i>value</i> at the end of the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamEntityDeclaration>::append(const QXmlStreamEntityDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   vector<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> three <span class="operator">=</span> <span class="string">"three"</span>;
    ///   vector<span class="operator">.</span>append(three);
    ///   <span class="comment">// vector: ["one", "two", "three"]</span>
    ///   <span class="comment">// three: "three"</span>
    ///
    /// </pre>
    /// <p>This is the same as calling resize(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() + 1) and assigning <i>value</i> to the new last element in the vector.</p>
    /// <p>This operation is relatively fast, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> typically allocates more memory than necessary, so it can grow without reallocating the entire vector each time.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>().</p></div>
    pub unsafe fn append_q_xml_stream_entity_declaration(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_append(self as *mut crate::QVectorOfQXmlStreamEntityDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamEntityDeclaration>::append(const QVector<QXmlStreamEntityDeclaration>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#append-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> vector to this vector.</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_vector_of_q_xml_stream_entity_declaration(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQXmlStreamEntityDeclaration>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_append2(self as *mut crate::QVectorOfQXmlStreamEntityDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQXmlStreamEntityDeclaration>>::cast_into(l).as_raw_ptr())
    }

    /// <p>Returns the item at index position <i>i</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamEntityDeclaration& QVector<QXmlStreamEntityDeclaration>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the vector.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_at(
            self as *const crate::QVectorOfQXmlStreamEntityDeclaration,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamEntityDeclaration& QVector<QXmlStreamEntityDeclaration>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::MutRef<crate::QXmlStreamEntityDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_back(
            self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamEntityDeclaration& QVector<QXmlStreamEntityDeclaration>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_back1(
            self as *const crate::QVectorOfQXmlStreamEntityDeclaration,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the maximum number of items that can be stored in the vector without forcing a reallocation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamEntityDeclaration>::capacity() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#capacity">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the maximum number of items that can be stored in the vector without forcing a reallocation.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s memory usage. In general, you will rarely ever need to call this function. If you want to know how many items are in the vector, call <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p></div>
    pub unsafe fn capacity(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_capacity(
            self as *const crate::QVectorOfQXmlStreamEntityDeclaration,
        )
    }

    /// <p>Removes all the elements from the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamEntityDeclaration>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all the elements from the vector.</p>
    /// <p><b>Note: </b>Until Qt 5.6, this also released the memory used by the vector. From Qt 5.7, the capacity is preserved. To shed all capacity, swap with a default-constructed vector:</p><pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span>T<span class="operator">&gt;</span> v <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span>T<span class="operator">&gt;</span>()<span class="operator">.</span>swap(v);
    ///   Q_ASSERT(v<span class="operator">.</span>capacity() <span class="operator">=</span><span class="operator">=</span> <span class="number">0</span>);
    ///
    /// </pre>
    /// <p>or call <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_clear(
            self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
        )
    }

    /// <p>Returns a const pointer to the data stored in the vector. The pointer can be used to access the items in the vector. The pointer remains valid as long as the vector isn't reallocated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamEntityDeclaration* QVector<QXmlStreamEntityDeclaration>::constData() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const pointer to the data stored in the vector. The pointer can be used to access the items in the vector. The pointer remains valid as long as the vector isn't reallocated.</p>
    /// <p>This function is mostly useful to pass a vector to a function that accepts a plain C++ array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#data">data</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn const_data(&self) -> ::cpp_utils::Ptr<crate::QXmlStreamEntityDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_constData(
                self as *const crate::QVectorOfQXmlStreamEntityDeclaration,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns a const reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamEntityDeclaration& QVector<QXmlStreamEntityDeclaration>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_constFirst(
                self as *const crate::QVectorOfQXmlStreamEntityDeclaration,
            );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamEntityDeclaration& QVector<QXmlStreamEntityDeclaration>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_constLast(
                self as *const crate::QVectorOfQXmlStreamEntityDeclaration,
            );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the vector contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamEntityDeclaration>::contains(const QXmlStreamEntityDeclaration& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the vector contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p></div>
    pub unsafe fn contains(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_contains(self as *const crate::QVectorOfQXmlStreamEntityDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Assigns <i>other</i> to this vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamEntityDeclaration>& QVector<QXmlStreamEntityDeclaration>::operator=(const QVector<QXmlStreamEntityDeclaration>& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this vector and returns a reference to this vector.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        v: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQXmlStreamEntityDeclaration>>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamEntityDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_operator_(
                self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
                ::cpp_utils::CastInto::<
                    ::cpp_utils::Ref<crate::QVectorOfQXmlStreamEntityDeclaration>,
                >::cast_into(v)
                .as_raw_ptr(),
            );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of occurrences of <i>value</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamEntityDeclaration>::count(const QXmlStreamEntityDeclaration& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of occurrences of <i>value</i> in the vector.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_count(self as *const crate::QVectorOfQXmlStreamEntityDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamEntityDeclaration>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_count1(
            self as *const crate::QVectorOfQXmlStreamEntityDeclaration,
        )
    }

    /// <p>Returns a pointer to the data stored in the vector. The pointer can be used to access and modify the items in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamEntityDeclaration* QVector<QXmlStreamEntityDeclaration>::data()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the data stored in the vector. The pointer can be used to access and modify the items in the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> vector(<span class="number">10</span>);
    ///   <span class="type">int</span> <span class="operator">*</span>data <span class="operator">=</span> vector<span class="operator">.</span>data();
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> <span class="number">10</span>; <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     data<span class="operator">[</span>i<span class="operator">]</span> <span class="operator">=</span> <span class="number">2</span> <span class="operator">*</span> i;
    ///
    /// </pre>
    /// <p>The pointer remains valid as long as the vector isn't reallocated.</p>
    /// <p>This function is mostly useful to pass a vector to a function that accepts a plain C++ array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constData">constData</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn data_mut(&mut self) -> ::cpp_utils::MutPtr<crate::QXmlStreamEntityDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_data(
            self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamEntityDeclaration* QVector<QXmlStreamEntityDeclaration>::data() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#data-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn data(&self) -> ::cpp_utils::Ptr<crate::QXmlStreamEntityDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_data1(
            self as *const crate::QVectorOfQXmlStreamEntityDeclaration,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamEntityDeclaration>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_detach(
            self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), returning <code>true</code> if the vector is empty; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamEntityDeclaration>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), returning <code>true</code> if the vector is empty; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_empty(
            self as *const crate::QVectorOfQXmlStreamEntityDeclaration,
        )
    }

    /// <p>Returns <code>true</code> if this vector is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamEntityDeclaration>::endsWith(const QXmlStreamEntityDeclaration& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this vector is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn ends_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_endsWith(self as *const crate::QVectorOfQXmlStreamEntityDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamEntityDeclaration>& QVector<QXmlStreamEntityDeclaration>::fill(const QXmlStreamEntityDeclaration& t, int size = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">3</span>);
    ///   vector<span class="operator">.</span>fill(<span class="string">"Yes"</span>);
    ///   <span class="comment">// vector: ["Yes", "Yes", "Yes"]</span>
    ///
    ///   vector<span class="operator">.</span>fill(<span class="string">"oh"</span><span class="operator">,</span> <span class="number">5</span>);
    ///   <span class="comment">// vector: ["oh", "oh", "oh", "oh", "oh"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_2a(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamEntityDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_fill(self as *mut crate::QVectorOfQXmlStreamEntityDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>::cast_into(t).as_raw_ptr(), size);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamEntityDeclaration>& QVector<QXmlStreamEntityDeclaration>::fill(const QXmlStreamEntityDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">3</span>);
    ///   vector<span class="operator">.</span>fill(<span class="string">"Yes"</span>);
    ///   <span class="comment">// vector: ["Yes", "Yes", "Yes"]</span>
    ///
    ///   vector<span class="operator">.</span>fill(<span class="string">"oh"</span><span class="operator">,</span> <span class="number">5</span>);
    ///   <span class="comment">// vector: ["oh", "oh", "oh", "oh", "oh"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_1a(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamEntityDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_fill1(self as *mut crate::QVectorOfQXmlStreamEntityDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>::cast_into(t).as_raw_ptr());
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamEntityDeclaration& QVector<QXmlStreamEntityDeclaration>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#constFirst">constFirst</a>().</p></div>
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<crate::QXmlStreamEntityDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_first(
            self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamEntityDeclaration& QVector<QXmlStreamEntityDeclaration>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_first1(
            self as *const crate::QVectorOfQXmlStreamEntityDeclaration,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamEntityDeclaration& QVector<QXmlStreamEntityDeclaration>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::MutRef<crate::QXmlStreamEntityDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_front(
            self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamEntityDeclaration& QVector<QXmlStreamEntityDeclaration>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_front1(
            self as *const crate::QVectorOfQXmlStreamEntityDeclaration,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamEntityDeclaration& QVector<QXmlStreamEntityDeclaration>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p>Note that using non-const operators can cause <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to do a deep copy.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QXmlStreamEntityDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_operator__2(
                self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
                i,
            );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamEntityDeclaration& QVector<QXmlStreamEntityDeclaration>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as at(<i>i</i>).</p></div>
    pub unsafe fn index(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_operator__3(
                self as *const crate::QVectorOfQXmlStreamEntityDeclaration,
                i,
            );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamEntityDeclaration>::indexOf(const QXmlStreamEntityDeclaration& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span>);            <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);         <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);         <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"X"</span>);            <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_indexOf(self as *const crate::QVectorOfQXmlStreamEntityDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>::cast_into(t).as_raw_ptr(), from)
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamEntityDeclaration>::indexOf(const QXmlStreamEntityDeclaration& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span>);            <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);         <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);         <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"X"</span>);            <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_indexOf1(self as *const crate::QVectorOfQXmlStreamEntityDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the vector. If <i>i</i> is 0, the value is prepended to the vector. If <i>i</i> is <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), the value is appended to the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamEntityDeclaration>::insert(int i, const QXmlStreamEntityDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the vector. If <i>i</i> is 0, the value is prepended to the vector. If <i>i</i> is <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), the value is appended to the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   vector<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// vector: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p>For large vectors, this operation can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because it requires moving all the items at indexes <i>i</i> and above by one position further in memory. If you want a container class that provides a fast insert() function, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>().</p></div>
    pub unsafe fn insert_2a(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_insert(self as *mut crate::QVectorOfQXmlStreamEntityDeclaration, i, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamEntityDeclaration>::insert(int i, int n, const QXmlStreamEntityDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>count</i> copies of <i>value</i> at index position <i>i</i> in the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">double</span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">2.718</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">1.442</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">0.4342</span>;
    ///   vector<span class="operator">.</span>insert(<span class="number">1</span><span class="operator">,</span> <span class="number">3</span><span class="operator">,</span> <span class="number">9.9</span>);
    ///   <span class="comment">// vector: [2.718, 9.9, 9.9, 9.9, 1.442, 0.4342]</span>
    ///
    /// </pre></div>
    pub unsafe fn insert_3a(
        &mut self,
        i: ::std::os::raw::c_int,
        n: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_insert1(self as *mut crate::QVectorOfQXmlStreamEntityDeclaration, i, n, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamEntityDeclaration>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_isDetached(
            self as *const crate::QVectorOfQXmlStreamEntityDeclaration,
        )
    }

    /// <p>Returns <code>true</code> if the vector has size 0; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamEntityDeclaration>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the vector has size 0; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_isEmpty(
            self as *const crate::QVectorOfQXmlStreamEntityDeclaration,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamEntityDeclaration>::isSharedWith(const QVector<QXmlStreamEntityDeclaration>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQXmlStreamEntityDeclaration>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_isSharedWith(self as *const crate::QVectorOfQXmlStreamEntityDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQXmlStreamEntityDeclaration>>::cast_into(other).as_raw_ptr())
    }

    /// <p>Returns a reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamEntityDeclaration& QVector<QXmlStreamEntityDeclaration>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#constLast">constLast</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<crate::QXmlStreamEntityDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_last(
            self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QXmlStreamEntityDeclaration& QVector<QXmlStreamEntityDeclaration>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_last1(
            self as *const crate::QVectorOfQXmlStreamEntityDeclaration,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamEntityDeclaration>::lastIndexOf(const QXmlStreamEntityDeclaration& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);        <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);     <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);     <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_lastIndexOf(self as *const crate::QVectorOfQXmlStreamEntityDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>::cast_into(t).as_raw_ptr(), from)
    }

    /// <p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamEntityDeclaration>::lastIndexOf(const QXmlStreamEntityDeclaration& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);        <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);     <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);     <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_lastIndexOf1(self as *const crate::QVectorOfQXmlStreamEntityDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamEntityDeclaration>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#length">QList::length</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_length(
            self as *const crate::QVectorOfQXmlStreamEntityDeclaration,
        )
    }

    /// <p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamEntityDeclaration> QVector<QXmlStreamEntityDeclaration>::mid(int pos, int len = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        len: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamEntityDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_mid(
            self as *const crate::QVectorOfQXmlStreamEntityDeclaration,
            pos,
            len,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamEntityDeclaration> QVector<QXmlStreamEntityDeclaration>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamEntityDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_mid1(
            self as *const crate::QVectorOfQXmlStreamEntityDeclaration,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamEntityDeclaration>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">QList::move</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_move(
            self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
            from,
            to,
        )
    }

    /// <p>Constructs an empty vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QXmlStreamEntityDeclaration>::QVector()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamEntityDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_QVector();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a vector with an initial size of <i>size</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QXmlStreamEntityDeclaration>::QVector(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a vector with an initial size of <i>size</i> elements.</p>
    /// <p>The elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn new_1a(
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamEntityDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_QVector1(size);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a vector with an initial size of <i>size</i> elements. Each element is initialized with <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QXmlStreamEntityDeclaration>::QVector(int size, const QXmlStreamEntityDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a vector with an initial size of <i>size</i> elements. Each element is initialized with <i>value</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn new_2a(
        size: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamEntityDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_QVector2(size, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>::cast_into(t).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QXmlStreamEntityDeclaration>::QVector(const QVector<QXmlStreamEntityDeclaration>& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        v: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQXmlStreamEntityDeclaration>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamEntityDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_QVector3(
                ::cpp_utils::CastInto::<
                    ::cpp_utils::Ref<crate::QVectorOfQXmlStreamEntityDeclaration>,
                >::cast_into(v)
                .as_raw_ptr(),
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamEntityDeclaration>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_pop_back(
            self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamEntityDeclaration>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_pop_front(
            self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
        )
    }

    /// <p>Inserts <i>value</i> at the beginning of the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamEntityDeclaration>::prepend(const QXmlStreamEntityDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   vector<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   vector<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// vector: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as vector.insert(0, <i>value</i>).</p>
    /// <p>For large vectors, this operation can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because it requires moving all the items in the vector by one position further in memory. If you want a container class that provides a fast prepend() function, use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> or <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_prepend(self as *mut crate::QVectorOfQXmlStreamEntityDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to append(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamEntityDeclaration>::push_back(const QXmlStreamEntityDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to append(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_push_back(self as *mut crate::QVectorOfQXmlStreamEntityDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to prepend(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamEntityDeclaration>::push_front(const QXmlStreamEntityDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to prepend(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_push_front(self as *mut crate::QVectorOfQXmlStreamEntityDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamEntityDeclaration>::remove(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#remove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes the element at index position <i>i</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn remove_1a(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_remove(
            self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
            i,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamEntityDeclaration>::remove(int i, int n)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#remove-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes <i>count</i> elements from the middle of the vector, starting at index position <i>i</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn remove_2a(&mut self, i: ::std::os::raw::c_int, n: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_remove1(
            self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
            i,
            n,
        )
    }

    /// <p>Removes all elements that compare equal to <i>t</i> from the vector. Returns the number of elements removed, if any.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamEntityDeclaration>::removeAll(const QXmlStreamEntityDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all elements that compare equal to <i>t</i> from the vector. Returns the number of elements removed, if any.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#removeOne">removeOne</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">QList::removeAll</a>().</p></div>
    pub unsafe fn remove_all(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_removeAll(self as *mut crate::QVectorOfQXmlStreamEntityDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Removes the element at index position <i>i</i>. Equivalent to</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamEntityDeclaration>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the element at index position <i>i</i>. Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   remove(i);
    ///
    /// </pre>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">QList::removeAt</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_removeAt(
            self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
            i,
        )
    }

    /// <p>Removes the first item in the vector. Calling this function is equivalent to calling remove(0). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamEntityDeclaration>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the vector. Calling this function is equivalent to calling remove(0). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_removeFirst(
            self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
        )
    }

    /// <p>Removes the last item in the vector. Calling this function is equivalent to calling remove(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() - 1). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamEntityDeclaration>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the vector. Calling this function is equivalent to calling remove(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() - 1). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_removeLast(
            self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
        )
    }

    /// <p>Removes the first element that compares equal to <i>t</i> from the vector. Returns whether an element was, in fact, removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamEntityDeclaration>::removeOne(const QXmlStreamEntityDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeOne">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first element that compares equal to <i>t</i> from the vector. Returns whether an element was, in fact, removed.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#removeAll">removeAll</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">QList::removeOne</a>().</p></div>
    pub unsafe fn remove_one(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_removeOne(self as *mut crate::QVectorOfQXmlStreamEntityDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamEntityDeclaration>::replace(int i, const QXmlStreamEntityDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_replace(self as *mut crate::QVectorOfQXmlStreamEntityDeclaration, i, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Attempts to allocate memory for at least <i>size</i> elements. If you know in advance how large the vector will be, you should call this function to prevent reallocations and memory fragmentation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamEntityDeclaration>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to allocate memory for at least <i>size</i> elements. If you know in advance how large the vector will be, you should call this function to prevent reallocations and memory fragmentation.</p>
    /// <p>If <i>size</i> is an underestimate, the worst that will happen is that the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> will be a bit slower. If <i>size</i> is an overestimate, you may have used more memory than the normal <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> growth strategy would have allocated—or you may have used less.</p>
    /// <p>An alternative to reserve() is calling <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>(). Whether or not that is faster than reserve() depends on the element type, because <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() default-constructs all elements, and requires assignment to existing entries rather than calling <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(), which copy- or move-constructs. For simple types, like <code>int</code> or <code>double</code>, <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() is typically faster, but for anything more complex, you should prefer reserve().</p>
    /// <p><b>Warning:</b> If the size passed to <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() was underestimated, you run out of allocated space and into undefined behavior. This problem does not exist with reserve(), because it treats the size as just a hint.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>().</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_reserve(
            self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
            size,
        )
    }

    /// <p>Sets the size of the vector to <i>size</i>. If <i>size</i> is greater than the current size, elements are added to the end; the new elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If <i>size</i> is less than the current size, elements are removed from the end.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamEntityDeclaration>::resize(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#resize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the size of the vector to <i>size</i>. If <i>size</i> is greater than the current size, elements are added to the end; the new elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If <i>size</i> is less than the current size, elements are removed from the end.</p>
    /// <p>Since Qt 5.6, resize() doesn't shrink the capacity anymore. To shed excess capacity, use <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p></div>
    pub unsafe fn resize(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_resize(
            self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamEntityDeclaration>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_setSharable(
            self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
            sharable,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamEntityDeclaration>::shrink_to_fit()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#shrink_to_fit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn shrink_to_fit(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_shrink_to_fit(
            self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
        )
    }

    /// <p>Returns the number of items in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QXmlStreamEntityDeclaration>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_size(
            self as *const crate::QVectorOfQXmlStreamEntityDeclaration,
        )
    }

    /// <p>Releases any memory not required to store the items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamEntityDeclaration>::squeeze()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Releases any memory not required to store the items.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s memory usage. In general, you will rarely ever need to call this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>().</p></div>
    pub unsafe fn squeeze(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_squeeze(
            self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
        )
    }

    /// <p>Returns <code>true</code> if this vector is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamEntityDeclaration>::startsWith(const QXmlStreamEntityDeclaration& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this vector is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn starts_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_startsWith(self as *const crate::QVectorOfQXmlStreamEntityDeclaration, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Swaps vector <i>other</i> with this vector. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QXmlStreamEntityDeclaration>::swap(QVector<QXmlStreamEntityDeclaration>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps vector <i>other</i> with this vector. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<
            ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamEntityDeclaration>,
        >,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_swap(self as *mut crate::QVectorOfQXmlStreamEntityDeclaration, ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QVectorOfQXmlStreamEntityDeclaration>>::cast_into(other).as_mut_raw_ptr())
    }

    /// <p>Removes the element at index position <i>i</i> and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamEntityDeclaration QVector<QXmlStreamEntityDeclaration>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the element at index position <i>i</i> and returns it.</p>
    /// <p>Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   T t <span class="operator">=</span> at(i);
    ///   remove(i);
    ///   <span class="keyword">return</span> t;
    ///
    /// </pre>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">QList::takeAt</a>().</p></div>
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamEntityDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_takeAt(
            self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the first item in the vector and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamEntityDeclaration QVector<QXmlStreamEntityDeclaration>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the vector and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> ::cpp_utils::CppBox<crate::QXmlStreamEntityDeclaration> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_takeFirst(
                self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the last item in the list and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamEntityDeclaration QVector<QXmlStreamEntityDeclaration>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> ::cpp_utils::CppBox<crate::QXmlStreamEntityDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_takeLast(
            self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value at index position <i>i</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamEntityDeclaration QVector<QXmlStreamEntityDeclaration>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the vector.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that <i>i</i> is within bounds, you can use <a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamEntityDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_value(
            self as *const crate::QVectorOfQXmlStreamEntityDeclaration,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamEntityDeclaration QVector<QXmlStreamEntityDeclaration>::value(int i, const QXmlStreamEntityDeclaration& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>,
    ) -> ::cpp_utils::CppBox<crate::QXmlStreamEntityDeclaration> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_value1(self as *const crate::QVectorOfQXmlStreamEntityDeclaration, i, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>::cast_into(default_value).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfOrientationIntInt {
    type Arguments = (
        crate::Orientation,
        ::std::os::raw::c_int,
        ::std::os::raw::c_int,
    );
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"1custom_slot(Qt::Orientation,int,int)\0",
                ),
            )
        }
    }
}

/// Binds a Qt signal with arguments `crate::Orientation,::std::os::raw::c_int,::std::os::raw::c_int` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```Qt::Orientation, int, int```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfOrientationIntInt<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfOrientationIntInt>,
    func: ::std::option::Option<
        Box<Box<dyn FnMut(crate::Orientation, ::std::os::raw::c_int, ::std::os::raw::c_int) + 'a>>,
    >,
}

impl<'a> SlotOfOrientationIntInt<'a> {
    /// Constructs a new object.
    pub fn new<F: FnMut(crate::Orientation, ::std::os::raw::c_int, ::std::os::raw::c_int) + 'a>(
        f: F,
    ) -> SlotOfOrientationIntInt<'a> {
        let mut obj = SlotOfOrientationIntInt::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<F: FnMut(crate::Orientation, ::std::os::raw::c_int, ::std::os::raw::c_int) + 'a>(
        &mut self,
        f: F,
    ) {
        extern "C" fn callback(
            data: *mut ::std::ffi::c_void,
            arg0: crate::Orientation,
            arg1: ::std::os::raw::c_int,
            arg2: ::std::os::raw::c_int,
        ) {
            let func = unsafe {
                &mut *(data as *mut Box<
                    dyn FnMut(crate::Orientation, ::std::os::raw::c_int, ::std::os::raw::c_int),
                >)
            };
            func(arg0, arg1, arg2);
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<
            Box<FnMut(crate::Orientation, ::std::os::raw::c_int, ::std::os::raw::c_int) + 'a>,
        > = Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(&mut self) -> ::cpp_utils::MutRef<crate::RawSlotOfOrientationIntInt> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfOrientationIntInt<'a> {
    fn default() -> Self {
        SlotOfOrientationIntInt {
            raw_slot: unsafe { crate::RawSlotOfOrientationIntInt::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfOrientationIntInt<'a> {
    type Arguments = <crate::RawSlotOfOrientationIntInt as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `crate::Orientation,::std::os::raw::c_int,::std::os::raw::c_int` to a Rust extern function.
///
/// It's recommended to use `SlotOfOrientationIntInt` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```Qt::Orientation, int, int```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfOrientationIntInt {
    _unused: u8,
}
impl RawSlotOfOrientationIntInt {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(
        &mut self,
        arg0: crate::Orientation,
        arg1: ::std::os::raw::c_int,
        arg2: ::std::os::raw::c_int,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int_custom_slot(self as *mut crate::RawSlotOfOrientationIntInt, arg0, arg1, arg2)
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int_metaObject(self as *const crate::RawSlotOfOrientationIntInt);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfOrientationIntInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int_qt_metacall(self as *mut crate::RawSlotOfOrientationIntInt, arg1, arg2, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3).as_mut_raw_ptr())
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int_qt_metacast(self as *mut crate::RawSlotOfOrientationIntInt, ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<
            extern "C" fn(
                *mut ::std::ffi::c_void,
                crate::Orientation,
                ::std::os::raw::c_int,
                ::std::os::raw::c_int,
            ),
        >,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int_set(
            self as *mut crate::RawSlotOfOrientationIntInt,
            func,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int_tr(
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                    .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                    .as_raw_ptr(),
                n,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int_trUtf8(::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c).as_raw_ptr(), n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfIntExitStatus {
    type Arguments = (::std::os::raw::c_int, crate::q_process::ExitStatus);
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"1custom_slot(int,QProcess::ExitStatus)\0",
                ),
            )
        }
    }
}

/// Binds a Qt signal with arguments `::std::os::raw::c_int,crate::q_process::ExitStatus` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```int, QProcess::ExitStatus```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfIntExitStatus<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfIntExitStatus>,
    func: ::std::option::Option<
        Box<Box<dyn FnMut(::std::os::raw::c_int, crate::q_process::ExitStatus) + 'a>>,
    >,
}

impl<'a> SlotOfIntExitStatus<'a> {
    /// Constructs a new object.
    pub fn new<F: FnMut(::std::os::raw::c_int, crate::q_process::ExitStatus) + 'a>(
        f: F,
    ) -> SlotOfIntExitStatus<'a> {
        let mut obj = SlotOfIntExitStatus::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<F: FnMut(::std::os::raw::c_int, crate::q_process::ExitStatus) + 'a>(
        &mut self,
        f: F,
    ) {
        extern "C" fn callback(
            data: *mut ::std::ffi::c_void,
            arg0: ::std::os::raw::c_int,
            arg1: crate::q_process::ExitStatus,
        ) {
            let func = unsafe {
                &mut *(data as *mut Box<
                    dyn FnMut(::std::os::raw::c_int, crate::q_process::ExitStatus),
                >)
            };
            func(arg0, arg1);
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<
            Box<FnMut(::std::os::raw::c_int, crate::q_process::ExitStatus) + 'a>,
        > = Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(&mut self) -> ::cpp_utils::MutRef<crate::RawSlotOfIntExitStatus> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfIntExitStatus<'a> {
    fn default() -> Self {
        SlotOfIntExitStatus {
            raw_slot: unsafe { crate::RawSlotOfIntExitStatus::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfIntExitStatus<'a> {
    type Arguments = <crate::RawSlotOfIntExitStatus as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `::std::os::raw::c_int,crate::q_process::ExitStatus` to a Rust extern function.
///
/// It's recommended to use `SlotOfIntExitStatus` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```int, QProcess::ExitStatus```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfIntExitStatus {
    _unused: u8,
}
impl RawSlotOfIntExitStatus {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(
        &mut self,
        arg0: ::std::os::raw::c_int,
        arg1: crate::q_process::ExitStatus,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus_custom_slot(self as *mut crate::RawSlotOfIntExitStatus, arg0, arg1)
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus_metaObject(self as *const crate::RawSlotOfIntExitStatus);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfIntExitStatus> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus_ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus_qt_metacall(self as *mut crate::RawSlotOfIntExitStatus, arg1, arg2, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3).as_mut_raw_ptr())
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus_qt_metacast(self as *mut crate::RawSlotOfIntExitStatus, ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<
            extern "C" fn(
                *mut ::std::ffi::c_void,
                ::std::os::raw::c_int,
                crate::q_process::ExitStatus,
            ),
        >,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus_set(
            self as *mut crate::RawSlotOfIntExitStatus,
            func,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus_tr(
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                    .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                    .as_raw_ptr(),
                n,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus_trUtf8(::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c).as_raw_ptr(), n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfQVariant {
    type Arguments = (*const crate::QVariant,);
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1custom_slot(const QVariant&)\0"),
            )
        }
    }
}

/// Binds a Qt signal with arguments `::cpp_utils::Ref<crate::QVariant>` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```const QVariant&```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfQVariant<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfQVariant>,
    func: ::std::option::Option<Box<Box<dyn FnMut(::cpp_utils::Ref<crate::QVariant>) + 'a>>>,
}

impl<'a> SlotOfQVariant<'a> {
    /// Constructs a new object.
    pub fn new<F: FnMut(::cpp_utils::Ref<crate::QVariant>) + 'a>(f: F) -> SlotOfQVariant<'a> {
        let mut obj = SlotOfQVariant::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<F: FnMut(::cpp_utils::Ref<crate::QVariant>) + 'a>(&mut self, f: F) {
        extern "C" fn callback(data: *mut ::std::ffi::c_void, arg0: *const crate::QVariant) {
            let func =
                unsafe { &mut *(data as *mut Box<dyn FnMut(::cpp_utils::Ref<crate::QVariant>)>) };
            func(unsafe {
                ::cpp_utils::Ref::from_raw(arg0).expect("attempted to construct a null Ref")
            });
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<Box<FnMut(::cpp_utils::Ref<crate::QVariant>) + 'a>> =
            Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(&mut self) -> ::cpp_utils::MutRef<crate::RawSlotOfQVariant> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfQVariant<'a> {
    fn default() -> Self {
        SlotOfQVariant {
            raw_slot: unsafe { crate::RawSlotOfQVariant::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfQVariant<'a> {
    type Arguments = <crate::RawSlotOfQVariant as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `*const crate::QVariant` to a Rust extern function.
///
/// It's recommended to use `SlotOfQVariant` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```const QVariant&```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfQVariant {
    _unused: u8,
}
impl RawSlotOfQVariant {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(
        &mut self,
        arg0: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref_custom_slot(
            self as *mut crate::RawSlotOfQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(arg0)
                .as_raw_ptr(),
        )
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref_metaObject(self as *const crate::RawSlotOfQVariant);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref_ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref_qt_metacall(
            self as *mut crate::RawSlotOfQVariant,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref_qt_metacast(self as *mut crate::RawSlotOfQVariant, ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<extern "C" fn(*mut ::std::ffi::c_void, *const crate::QVariant)>,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref_set(
            self as *mut crate::RawSlotOfQVariant,
            func,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref_tr(
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                    .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                    .as_raw_ptr(),
                n,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref_trUtf8(
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                    .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                    .as_raw_ptr(),
                n,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfState {
    type Arguments = (crate::q_abstract_animation::State,);
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"1custom_slot(QAbstractAnimation::State)\0",
                ),
            )
        }
    }
}

/// Binds a Qt signal with arguments `crate::q_abstract_animation::State` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```QAbstractAnimation::State```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfState<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfState>,
    func: ::std::option::Option<Box<Box<dyn FnMut(crate::q_abstract_animation::State) + 'a>>>,
}

impl<'a> SlotOfState<'a> {
    /// Constructs a new object.
    pub fn new<F: FnMut(crate::q_abstract_animation::State) + 'a>(f: F) -> SlotOfState<'a> {
        let mut obj = SlotOfState::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<F: FnMut(crate::q_abstract_animation::State) + 'a>(&mut self, f: F) {
        extern "C" fn callback(
            data: *mut ::std::ffi::c_void,
            arg0: crate::q_abstract_animation::State,
        ) {
            let func =
                unsafe { &mut *(data as *mut Box<dyn FnMut(crate::q_abstract_animation::State)>) };
            func(arg0);
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<Box<FnMut(crate::q_abstract_animation::State) + 'a>> =
            Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(&mut self) -> ::cpp_utils::MutRef<crate::RawSlotOfState> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfState<'a> {
    fn default() -> Self {
        SlotOfState {
            raw_slot: unsafe { crate::RawSlotOfState::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfState<'a> {
    type Arguments = <crate::RawSlotOfState as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `crate::q_abstract_animation::State` to a Rust extern function.
///
/// It's recommended to use `SlotOfState` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```QAbstractAnimation::State```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfState {
    _unused: u8,
}
impl RawSlotOfState {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(&mut self, arg0: crate::q_abstract_animation::State) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_custom_slot(self as *mut crate::RawSlotOfState, arg0)
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_metaObject(self as *const crate::RawSlotOfState);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_qt_metacall(self as *mut crate::RawSlotOfState, arg1, arg2, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3).as_mut_raw_ptr())
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_qt_metacast(self as *mut crate::RawSlotOfState, ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<
            extern "C" fn(*mut ::std::ffi::c_void, crate::q_abstract_animation::State),
        >,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_set(
            self as *mut crate::RawSlotOfState,
            func,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_tr(
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                    .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                    .as_raw_ptr(),
                n,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_trUtf8(::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c).as_raw_ptr(), n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfIntI64 {
    type Arguments = (::std::os::raw::c_int, i64);
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1custom_slot(int,qint64)\0"),
            )
        }
    }
}

/// Binds a Qt signal with arguments `::std::os::raw::c_int,i64` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```int, qint64```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfIntI64<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfIntI64>,
    func: ::std::option::Option<Box<Box<dyn FnMut(::std::os::raw::c_int, i64) + 'a>>>,
}

impl<'a> SlotOfIntI64<'a> {
    /// Constructs a new object.
    pub fn new<F: FnMut(::std::os::raw::c_int, i64) + 'a>(f: F) -> SlotOfIntI64<'a> {
        let mut obj = SlotOfIntI64::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<F: FnMut(::std::os::raw::c_int, i64) + 'a>(&mut self, f: F) {
        extern "C" fn callback(
            data: *mut ::std::ffi::c_void,
            arg0: ::std::os::raw::c_int,
            arg1: i64,
        ) {
            let func = unsafe { &mut *(data as *mut Box<dyn FnMut(::std::os::raw::c_int, i64)>) };
            func(arg0, arg1);
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<Box<FnMut(::std::os::raw::c_int, i64) + 'a>> = Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(&mut self) -> ::cpp_utils::MutRef<crate::RawSlotOfIntI64> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfIntI64<'a> {
    fn default() -> Self {
        SlotOfIntI64 {
            raw_slot: unsafe { crate::RawSlotOfIntI64::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfIntI64<'a> {
    type Arguments = <crate::RawSlotOfIntI64 as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `::std::os::raw::c_int,i64` to a Rust extern function.
///
/// It's recommended to use `SlotOfIntI64` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```int, qint64```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfIntI64 {
    _unused: u8,
}
impl RawSlotOfIntI64 {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(&mut self, arg0: ::std::os::raw::c_int, arg1: i64) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_qint64_custom_slot(
            self as *mut crate::RawSlotOfIntI64,
            arg0,
            arg1,
        )
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_qint64_metaObject(
                self as *const crate::RawSlotOfIntI64,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfIntI64> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_qint64_ctr_qt_core_ffi_slot_wrapper_int_qint64();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_qint64_qt_metacall(
            self as *mut crate::RawSlotOfIntI64,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_qint64_qt_metacast(
                self as *mut crate::RawSlotOfIntI64,
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                    .as_raw_ptr(),
            );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<
            extern "C" fn(
                *mut ::std::ffi::c_void,
                ::std::os::raw::c_int,
                ::std::os::raw::c_longlong,
            ),
        >,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_qint64_set(
            self as *mut crate::RawSlotOfIntI64,
            func,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_qint64_staticMetaObject(
            );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_qint64_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_qint64_trUtf8(
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                    .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                    .as_raw_ptr(),
                n,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt {
    type Arguments = (
        *const crate::QModelIndex,
        *const crate::QModelIndex,
        *const crate::QVectorOfInt,
    );
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"1custom_slot(const QModelIndex&,const QModelIndex&,const QVector< int >&)\0",
                ),
            )
        }
    }
}

/// Binds a Qt signal with arguments `::cpp_utils::Ref<crate::QModelIndex>,::cpp_utils::Ref<crate::QModelIndex>,::cpp_utils::Ref<crate::QVectorOfInt>` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```const QModelIndex&, const QModelIndex&, const QVector<int>&```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfQModelIndexQModelIndexQVectorOfInt<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt>,
    func: ::std::option::Option<
        Box<
            Box<
                dyn FnMut(
                        ::cpp_utils::Ref<crate::QModelIndex>,
                        ::cpp_utils::Ref<crate::QModelIndex>,
                        ::cpp_utils::Ref<crate::QVectorOfInt>,
                    ) + 'a,
            >,
        >,
    >,
}

impl<'a> SlotOfQModelIndexQModelIndexQVectorOfInt<'a> {
    /// Constructs a new object.
    pub fn new<
        F: FnMut(
                ::cpp_utils::Ref<crate::QModelIndex>,
                ::cpp_utils::Ref<crate::QModelIndex>,
                ::cpp_utils::Ref<crate::QVectorOfInt>,
            ) + 'a,
    >(
        f: F,
    ) -> SlotOfQModelIndexQModelIndexQVectorOfInt<'a> {
        let mut obj = SlotOfQModelIndexQModelIndexQVectorOfInt::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<
        F: FnMut(
                ::cpp_utils::Ref<crate::QModelIndex>,
                ::cpp_utils::Ref<crate::QModelIndex>,
                ::cpp_utils::Ref<crate::QVectorOfInt>,
            ) + 'a,
    >(
        &mut self,
        f: F,
    ) {
        extern "C" fn callback(
            data: *mut ::std::ffi::c_void,
            arg0: *const crate::QModelIndex,
            arg1: *const crate::QModelIndex,
            arg2: *const crate::QVectorOfInt,
        ) {
            let func = unsafe {
                &mut *(data as *mut Box<
                    dyn FnMut(
                        ::cpp_utils::Ref<crate::QModelIndex>,
                        ::cpp_utils::Ref<crate::QModelIndex>,
                        ::cpp_utils::Ref<crate::QVectorOfInt>,
                    ),
                >)
            };
            func(
                unsafe {
                    ::cpp_utils::Ref::from_raw(arg0).expect("attempted to construct a null Ref")
                },
                unsafe {
                    ::cpp_utils::Ref::from_raw(arg1).expect("attempted to construct a null Ref")
                },
                unsafe {
                    ::cpp_utils::Ref::from_raw(arg2).expect("attempted to construct a null Ref")
                },
            );
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<
            Box<
                FnMut(
                        ::cpp_utils::Ref<crate::QModelIndex>,
                        ::cpp_utils::Ref<crate::QModelIndex>,
                        ::cpp_utils::Ref<crate::QVectorOfInt>,
                    ) + 'a,
            >,
        > = Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(
        &mut self,
    ) -> ::cpp_utils::MutRef<crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfQModelIndexQModelIndexQVectorOfInt<'a> {
    fn default() -> Self {
        SlotOfQModelIndexQModelIndexQVectorOfInt {
            raw_slot: unsafe { crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfQModelIndexQModelIndexQVectorOfInt<'a> {
    type Arguments =
        <crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `*const crate::QModelIndex,*const crate::QModelIndex,*const crate::QVectorOfInt` to a Rust extern function.
///
/// It's recommended to use `SlotOfQModelIndexQModelIndexQVectorOfInt` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```const QModelIndex&, const QModelIndex&, const QVector<int>&```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfQModelIndexQModelIndexQVectorOfInt {
    _unused: u8,
}
impl RawSlotOfQModelIndexQModelIndexQVectorOfInt {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(
        &mut self,
        arg0: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        arg2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfInt>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref_custom_slot(self as *mut crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(arg0).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(arg1).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfInt>>::cast_into(arg2).as_raw_ptr())
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref_metaObject(self as *const crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref_qt_metacall(self as *mut crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt, arg1, arg2, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3).as_mut_raw_ptr())
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref_qt_metacast(self as *mut crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt, ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<
            extern "C" fn(
                *mut ::std::ffi::c_void,
                *const crate::QModelIndex,
                *const crate::QModelIndex,
                *const crate::QVectorOfInt,
            ),
        >,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref_set(self as *mut crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt, func, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data).as_mut_raw_ptr())
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref_tr(::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c).as_raw_ptr(), n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref_trUtf8(::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c).as_raw_ptr(), n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfQListOfQPersistentModelIndex {
    type Arguments = (*const crate::QListOfQPersistentModelIndex,);
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"1custom_slot(const QList< QPersistentModelIndex >&)\0",
                ),
            )
        }
    }
}

/// Binds a Qt signal with arguments `::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```const QList<QPersistentModelIndex>&```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfQListOfQPersistentModelIndex<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfQListOfQPersistentModelIndex>,
    func: ::std::option::Option<
        Box<Box<dyn FnMut(::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>) + 'a>>,
    >,
}

impl<'a> SlotOfQListOfQPersistentModelIndex<'a> {
    /// Constructs a new object.
    pub fn new<F: FnMut(::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>) + 'a>(
        f: F,
    ) -> SlotOfQListOfQPersistentModelIndex<'a> {
        let mut obj = SlotOfQListOfQPersistentModelIndex::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<F: FnMut(::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>) + 'a>(
        &mut self,
        f: F,
    ) {
        extern "C" fn callback(
            data: *mut ::std::ffi::c_void,
            arg0: *const crate::QListOfQPersistentModelIndex,
        ) {
            let func = unsafe {
                &mut *(data as *mut Box<
                    dyn FnMut(::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>),
                >)
            };
            func(unsafe {
                ::cpp_utils::Ref::from_raw(arg0).expect("attempted to construct a null Ref")
            });
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<
            Box<FnMut(::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>) + 'a>,
        > = Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(
        &mut self,
    ) -> ::cpp_utils::MutRef<crate::RawSlotOfQListOfQPersistentModelIndex> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfQListOfQPersistentModelIndex<'a> {
    fn default() -> Self {
        SlotOfQListOfQPersistentModelIndex {
            raw_slot: unsafe { crate::RawSlotOfQListOfQPersistentModelIndex::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfQListOfQPersistentModelIndex<'a> {
    type Arguments = <crate::RawSlotOfQListOfQPersistentModelIndex as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `*const crate::QListOfQPersistentModelIndex` to a Rust extern function.
///
/// It's recommended to use `SlotOfQListOfQPersistentModelIndex` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```const QList<QPersistentModelIndex>&```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfQListOfQPersistentModelIndex {
    _unused: u8,
}
impl RawSlotOfQListOfQPersistentModelIndex {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(
        &mut self,
        arg0: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_custom_slot(self as *mut crate::RawSlotOfQListOfQPersistentModelIndex, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>>::cast_into(arg0).as_raw_ptr())
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_metaObject(self as *const crate::RawSlotOfQListOfQPersistentModelIndex);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfQListOfQPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_qt_metacall(self as *mut crate::RawSlotOfQListOfQPersistentModelIndex, arg1, arg2, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3).as_mut_raw_ptr())
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_qt_metacast(self as *mut crate::RawSlotOfQListOfQPersistentModelIndex, ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<
            extern "C" fn(*mut ::std::ffi::c_void, *const crate::QListOfQPersistentModelIndex),
        >,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_set(self as *mut crate::RawSlotOfQListOfQPersistentModelIndex, func, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data).as_mut_raw_ptr())
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_tr(::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c).as_raw_ptr(), n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_trUtf8(::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c).as_raw_ptr(), n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfQModelIndexInt {
    type Arguments = (*const crate::QModelIndex, ::std::os::raw::c_int);
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"1custom_slot(const QModelIndex&,int)\0",
                ),
            )
        }
    }
}

/// Binds a Qt signal with arguments `::cpp_utils::Ref<crate::QModelIndex>,::std::os::raw::c_int` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```const QModelIndex&, int```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfQModelIndexInt<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfQModelIndexInt>,
    func: ::std::option::Option<
        Box<Box<dyn FnMut(::cpp_utils::Ref<crate::QModelIndex>, ::std::os::raw::c_int) + 'a>>,
    >,
}

impl<'a> SlotOfQModelIndexInt<'a> {
    /// Constructs a new object.
    pub fn new<F: FnMut(::cpp_utils::Ref<crate::QModelIndex>, ::std::os::raw::c_int) + 'a>(
        f: F,
    ) -> SlotOfQModelIndexInt<'a> {
        let mut obj = SlotOfQModelIndexInt::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<F: FnMut(::cpp_utils::Ref<crate::QModelIndex>, ::std::os::raw::c_int) + 'a>(
        &mut self,
        f: F,
    ) {
        extern "C" fn callback(
            data: *mut ::std::ffi::c_void,
            arg0: *const crate::QModelIndex,
            arg1: ::std::os::raw::c_int,
        ) {
            let func = unsafe {
                &mut *(data as *mut Box<
                    dyn FnMut(::cpp_utils::Ref<crate::QModelIndex>, ::std::os::raw::c_int),
                >)
            };
            func(
                unsafe {
                    ::cpp_utils::Ref::from_raw(arg0).expect("attempted to construct a null Ref")
                },
                arg1,
            );
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<
            Box<FnMut(::cpp_utils::Ref<crate::QModelIndex>, ::std::os::raw::c_int) + 'a>,
        > = Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(&mut self) -> ::cpp_utils::MutRef<crate::RawSlotOfQModelIndexInt> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfQModelIndexInt<'a> {
    fn default() -> Self {
        SlotOfQModelIndexInt {
            raw_slot: unsafe { crate::RawSlotOfQModelIndexInt::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfQModelIndexInt<'a> {
    type Arguments = <crate::RawSlotOfQModelIndexInt as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `*const crate::QModelIndex,::std::os::raw::c_int` to a Rust extern function.
///
/// It's recommended to use `SlotOfQModelIndexInt` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```const QModelIndex&, int```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfQModelIndexInt {
    _unused: u8,
}
impl RawSlotOfQModelIndexInt {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(
        &mut self,
        arg0: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        arg1: ::std::os::raw::c_int,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_custom_slot(self as *mut crate::RawSlotOfQModelIndexInt, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(arg0).as_raw_ptr(), arg1)
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_metaObject(self as *const crate::RawSlotOfQModelIndexInt);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfQModelIndexInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_qt_metacall(self as *mut crate::RawSlotOfQModelIndexInt, arg1, arg2, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3).as_mut_raw_ptr())
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_qt_metacast(self as *mut crate::RawSlotOfQModelIndexInt, ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<
            extern "C" fn(
                *mut ::std::ffi::c_void,
                *const crate::QModelIndex,
                ::std::os::raw::c_int,
            ),
        >,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_set(
            self as *mut crate::RawSlotOfQModelIndexInt,
            func,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_tr(
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                    .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                    .as_raw_ptr(),
                n,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_trUtf8(::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c).as_raw_ptr(), n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint {
    type Arguments = (
        *const crate::QListOfQPersistentModelIndex,
        crate::q_abstract_item_model::LayoutChangeHint,
    );
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1custom_slot(const QList< QPersistentModelIndex >&,QAbstractItemModel::LayoutChangeHint)\0")
            )
        }
    }
}

/// Binds a Qt signal with arguments `::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>,crate::q_abstract_item_model::LayoutChangeHint` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```const QList<QPersistentModelIndex>&, QAbstractItemModel::LayoutChangeHint```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfQListOfQPersistentModelIndexLayoutChangeHint<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint>,
    func: ::std::option::Option<
        Box<
            Box<
                dyn FnMut(
                        ::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>,
                        crate::q_abstract_item_model::LayoutChangeHint,
                    ) + 'a,
            >,
        >,
    >,
}

impl<'a> SlotOfQListOfQPersistentModelIndexLayoutChangeHint<'a> {
    /// Constructs a new object.
    pub fn new<
        F: FnMut(
                ::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>,
                crate::q_abstract_item_model::LayoutChangeHint,
            ) + 'a,
    >(
        f: F,
    ) -> SlotOfQListOfQPersistentModelIndexLayoutChangeHint<'a> {
        let mut obj = SlotOfQListOfQPersistentModelIndexLayoutChangeHint::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<
        F: FnMut(
                ::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>,
                crate::q_abstract_item_model::LayoutChangeHint,
            ) + 'a,
    >(
        &mut self,
        f: F,
    ) {
        extern "C" fn callback(
            data: *mut ::std::ffi::c_void,
            arg0: *const crate::QListOfQPersistentModelIndex,
            arg1: crate::q_abstract_item_model::LayoutChangeHint,
        ) {
            let func = unsafe {
                &mut *(data as *mut Box<
                    dyn FnMut(
                        ::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>,
                        crate::q_abstract_item_model::LayoutChangeHint,
                    ),
                >)
            };
            func(
                unsafe {
                    ::cpp_utils::Ref::from_raw(arg0).expect("attempted to construct a null Ref")
                },
                arg1,
            );
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<
            Box<
                FnMut(
                        ::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>,
                        crate::q_abstract_item_model::LayoutChangeHint,
                    ) + 'a,
            >,
        > = Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(
        &mut self,
    ) -> ::cpp_utils::MutRef<crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfQListOfQPersistentModelIndexLayoutChangeHint<'a> {
    fn default() -> Self {
        SlotOfQListOfQPersistentModelIndexLayoutChangeHint {
            raw_slot: unsafe {
                crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint::new()
            },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfQListOfQPersistentModelIndexLayoutChangeHint<'a> {
    type Arguments = <crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `*const crate::QListOfQPersistentModelIndex,crate::q_abstract_item_model::LayoutChangeHint` to a Rust extern function.
///
/// It's recommended to use `SlotOfQListOfQPersistentModelIndexLayoutChangeHint` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```const QList<QPersistentModelIndex>&, QAbstractItemModel::LayoutChangeHint```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint {
    _unused: u8,
}
impl RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(
        &mut self,
        arg0: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>>,
        arg1: crate::q_abstract_item_model::LayoutChangeHint,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint_custom_slot(self as *mut crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>>::cast_into(arg0).as_raw_ptr(), arg1)
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint_metaObject(self as *const crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new(
    ) -> ::cpp_utils::CppBox<crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint_qt_metacall(self as *mut crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint, arg1, arg2, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3).as_mut_raw_ptr())
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint_qt_metacast(self as *mut crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint, ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<
            extern "C" fn(
                *mut ::std::ffi::c_void,
                *const crate::QListOfQPersistentModelIndex,
                crate::q_abstract_item_model::LayoutChangeHint,
            ),
        >,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint_set(self as *mut crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint, func, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data).as_mut_raw_ptr())
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint_tr(::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c).as_raw_ptr(), n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint_trUtf8(::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c).as_raw_ptr(), n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfOrientation {
    type Arguments = (crate::Orientation,);
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1custom_slot(Qt::Orientation)\0"),
            )
        }
    }
}

/// Binds a Qt signal with arguments `crate::Orientation` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```Qt::Orientation```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfOrientation<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfOrientation>,
    func: ::std::option::Option<Box<Box<dyn FnMut(crate::Orientation) + 'a>>>,
}

impl<'a> SlotOfOrientation<'a> {
    /// Constructs a new object.
    pub fn new<F: FnMut(crate::Orientation) + 'a>(f: F) -> SlotOfOrientation<'a> {
        let mut obj = SlotOfOrientation::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<F: FnMut(crate::Orientation) + 'a>(&mut self, f: F) {
        extern "C" fn callback(data: *mut ::std::ffi::c_void, arg0: crate::Orientation) {
            let func = unsafe { &mut *(data as *mut Box<dyn FnMut(crate::Orientation)>) };
            func(arg0);
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<Box<FnMut(crate::Orientation) + 'a>> = Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(&mut self) -> ::cpp_utils::MutRef<crate::RawSlotOfOrientation> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfOrientation<'a> {
    fn default() -> Self {
        SlotOfOrientation {
            raw_slot: unsafe { crate::RawSlotOfOrientation::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfOrientation<'a> {
    type Arguments = <crate::RawSlotOfOrientation as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `crate::Orientation` to a Rust extern function.
///
/// It's recommended to use `SlotOfOrientation` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```Qt::Orientation```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfOrientation {
    _unused: u8,
}
impl RawSlotOfOrientation {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(&mut self, arg0: crate::Orientation) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_custom_slot(
            self as *mut crate::RawSlotOfOrientation,
            arg0,
        )
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_metaObject(
                self as *const crate::RawSlotOfOrientation,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfOrientation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_qt_metacall(
            self as *mut crate::RawSlotOfOrientation,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_qt_metacast(
                self as *mut crate::RawSlotOfOrientation,
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                    .as_raw_ptr(),
            );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<extern "C" fn(*mut ::std::ffi::c_void, crate::Orientation)>,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_set(
            self as *mut crate::RawSlotOfOrientation,
            func,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_tr(
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                    .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                    .as_raw_ptr(),
                n,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_trUtf8(
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                    .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                    .as_raw_ptr(),
                n,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfProcessState {
    type Arguments = (crate::q_process::ProcessState,);
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"1custom_slot(QProcess::ProcessState)\0",
                ),
            )
        }
    }
}

/// Binds a Qt signal with arguments `crate::q_process::ProcessState` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```QProcess::ProcessState```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfProcessState<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfProcessState>,
    func: ::std::option::Option<Box<Box<dyn FnMut(crate::q_process::ProcessState) + 'a>>>,
}

impl<'a> SlotOfProcessState<'a> {
    /// Constructs a new object.
    pub fn new<F: FnMut(crate::q_process::ProcessState) + 'a>(f: F) -> SlotOfProcessState<'a> {
        let mut obj = SlotOfProcessState::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<F: FnMut(crate::q_process::ProcessState) + 'a>(&mut self, f: F) {
        extern "C" fn callback(
            data: *mut ::std::ffi::c_void,
            arg0: crate::q_process::ProcessState,
        ) {
            let func =
                unsafe { &mut *(data as *mut Box<dyn FnMut(crate::q_process::ProcessState)>) };
            func(arg0);
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<Box<FnMut(crate::q_process::ProcessState) + 'a>> =
            Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(&mut self) -> ::cpp_utils::MutRef<crate::RawSlotOfProcessState> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfProcessState<'a> {
    fn default() -> Self {
        SlotOfProcessState {
            raw_slot: unsafe { crate::RawSlotOfProcessState::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfProcessState<'a> {
    type Arguments = <crate::RawSlotOfProcessState as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `crate::q_process::ProcessState` to a Rust extern function.
///
/// It's recommended to use `SlotOfProcessState` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```QProcess::ProcessState```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfProcessState {
    _unused: u8,
}
impl RawSlotOfProcessState {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(&mut self, arg0: crate::q_process::ProcessState) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState_custom_slot(
            self as *mut crate::RawSlotOfProcessState,
            arg0,
        )
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState_metaObject(self as *const crate::RawSlotOfProcessState);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfProcessState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState_qt_metacall(
            self as *mut crate::RawSlotOfProcessState,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState_qt_metacast(self as *mut crate::RawSlotOfProcessState, ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<
            extern "C" fn(*mut ::std::ffi::c_void, crate::q_process::ProcessState),
        >,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState_set(
            self as *mut crate::RawSlotOfProcessState,
            func,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState_tr(
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                    .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                    .as_raw_ptr(),
                n,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState_trUtf8(
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                    .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                    .as_raw_ptr(),
                n,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfBool {
    type Arguments = (bool,);
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1custom_slot(bool)\0"),
            )
        }
    }
}

/// Binds a Qt signal with arguments `bool` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```bool```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfBool<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfBool>,
    func: ::std::option::Option<Box<Box<dyn FnMut(bool) + 'a>>>,
}

impl<'a> SlotOfBool<'a> {
    /// Constructs a new object.
    pub fn new<F: FnMut(bool) + 'a>(f: F) -> SlotOfBool<'a> {
        let mut obj = SlotOfBool::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<F: FnMut(bool) + 'a>(&mut self, f: F) {
        extern "C" fn callback(data: *mut ::std::ffi::c_void, arg0: bool) {
            let func = unsafe { &mut *(data as *mut Box<dyn FnMut(bool)>) };
            func(arg0);
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<Box<FnMut(bool) + 'a>> = Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(&mut self) -> ::cpp_utils::MutRef<crate::RawSlotOfBool> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfBool<'a> {
    fn default() -> Self {
        SlotOfBool {
            raw_slot: unsafe { crate::RawSlotOfBool::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfBool<'a> {
    type Arguments = <crate::RawSlotOfBool as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `bool` to a Rust extern function.
///
/// It's recommended to use `SlotOfBool` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```bool```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfBool {
    _unused: u8,
}
impl RawSlotOfBool {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(&mut self, arg0: bool) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_bool_custom_slot(
            self as *mut crate::RawSlotOfBool,
            arg0,
        )
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_bool_metaObject(
            self as *const crate::RawSlotOfBool,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfBool> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_bool_ctr_qt_core_ffi_slot_wrapper_bool();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_bool_qt_metacall(
            self as *mut crate::RawSlotOfBool,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_bool_qt_metacast(
                self as *mut crate::RawSlotOfBool,
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                    .as_raw_ptr(),
            );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<extern "C" fn(*mut ::std::ffi::c_void, bool)>,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_bool_set(
            self as *mut crate::RawSlotOfBool,
            func,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_bool_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_bool_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_bool_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfQModelIndex {
    type Arguments = (*const crate::QModelIndex,);
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"1custom_slot(const QModelIndex&)\0",
                ),
            )
        }
    }
}

/// Binds a Qt signal with arguments `::cpp_utils::Ref<crate::QModelIndex>` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```const QModelIndex&```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfQModelIndex<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfQModelIndex>,
    func: ::std::option::Option<Box<Box<dyn FnMut(::cpp_utils::Ref<crate::QModelIndex>) + 'a>>>,
}

impl<'a> SlotOfQModelIndex<'a> {
    /// Constructs a new object.
    pub fn new<F: FnMut(::cpp_utils::Ref<crate::QModelIndex>) + 'a>(f: F) -> SlotOfQModelIndex<'a> {
        let mut obj = SlotOfQModelIndex::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<F: FnMut(::cpp_utils::Ref<crate::QModelIndex>) + 'a>(&mut self, f: F) {
        extern "C" fn callback(data: *mut ::std::ffi::c_void, arg0: *const crate::QModelIndex) {
            let func = unsafe {
                &mut *(data as *mut Box<dyn FnMut(::cpp_utils::Ref<crate::QModelIndex>)>)
            };
            func(unsafe {
                ::cpp_utils::Ref::from_raw(arg0).expect("attempted to construct a null Ref")
            });
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<Box<FnMut(::cpp_utils::Ref<crate::QModelIndex>) + 'a>> =
            Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(&mut self) -> ::cpp_utils::MutRef<crate::RawSlotOfQModelIndex> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfQModelIndex<'a> {
    fn default() -> Self {
        SlotOfQModelIndex {
            raw_slot: unsafe { crate::RawSlotOfQModelIndex::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfQModelIndex<'a> {
    type Arguments = <crate::RawSlotOfQModelIndex as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `*const crate::QModelIndex` to a Rust extern function.
///
/// It's recommended to use `SlotOfQModelIndex` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```const QModelIndex&```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfQModelIndex {
    _unused: u8,
}
impl RawSlotOfQModelIndex {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(
        &mut self,
        arg0: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_custom_slot(
            self as *mut crate::RawSlotOfQModelIndex,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(arg0)
                .as_raw_ptr(),
        )
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_metaObject(self as *const crate::RawSlotOfQModelIndex);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_qt_metacall(
            self as *mut crate::RawSlotOfQModelIndex,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_qt_metacast(self as *mut crate::RawSlotOfQModelIndex, ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<
            extern "C" fn(*mut ::std::ffi::c_void, *const crate::QModelIndex),
        >,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_set(
            self as *mut crate::RawSlotOfQModelIndex,
            func,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_tr(
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                    .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                    .as_raw_ptr(),
                n,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_trUtf8(
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                    .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                    .as_raw_ptr(),
                n,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfQAbstractItemModel {
    type Arguments = (*mut crate::QAbstractItemModel,);
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"1custom_slot(QAbstractItemModel*)\0",
                ),
            )
        }
    }
}

/// Binds a Qt signal with arguments `::cpp_utils::MutPtr<crate::QAbstractItemModel>` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```QAbstractItemModel*```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfQAbstractItemModel<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfQAbstractItemModel>,
    func: ::std::option::Option<
        Box<Box<dyn FnMut(::cpp_utils::MutPtr<crate::QAbstractItemModel>) + 'a>>,
    >,
}

impl<'a> SlotOfQAbstractItemModel<'a> {
    /// Constructs a new object.
    pub fn new<F: FnMut(::cpp_utils::MutPtr<crate::QAbstractItemModel>) + 'a>(
        f: F,
    ) -> SlotOfQAbstractItemModel<'a> {
        let mut obj = SlotOfQAbstractItemModel::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<F: FnMut(::cpp_utils::MutPtr<crate::QAbstractItemModel>) + 'a>(&mut self, f: F) {
        extern "C" fn callback(
            data: *mut ::std::ffi::c_void,
            arg0: *mut crate::QAbstractItemModel,
        ) {
            let func = unsafe {
                &mut *(data as *mut Box<dyn FnMut(::cpp_utils::MutPtr<crate::QAbstractItemModel>)>)
            };
            func(unsafe { ::cpp_utils::MutPtr::from_raw(arg0) });
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<Box<FnMut(::cpp_utils::MutPtr<crate::QAbstractItemModel>) + 'a>> =
            Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(&mut self) -> ::cpp_utils::MutRef<crate::RawSlotOfQAbstractItemModel> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfQAbstractItemModel<'a> {
    fn default() -> Self {
        SlotOfQAbstractItemModel {
            raw_slot: unsafe { crate::RawSlotOfQAbstractItemModel::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfQAbstractItemModel<'a> {
    type Arguments = <crate::RawSlotOfQAbstractItemModel as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `*mut crate::QAbstractItemModel` to a Rust extern function.
///
/// It's recommended to use `SlotOfQAbstractItemModel` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```QAbstractItemModel*```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfQAbstractItemModel {
    _unused: u8,
}
impl RawSlotOfQAbstractItemModel {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(
        &mut self,
        arg0: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractItemModel>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr_custom_slot(self as *mut crate::RawSlotOfQAbstractItemModel, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractItemModel>>::cast_into(arg0).as_mut_raw_ptr())
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr_metaObject(self as *const crate::RawSlotOfQAbstractItemModel);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfQAbstractItemModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr_ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr_qt_metacall(self as *mut crate::RawSlotOfQAbstractItemModel, arg1, arg2, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3).as_mut_raw_ptr())
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr_qt_metacast(self as *mut crate::RawSlotOfQAbstractItemModel, ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<
            extern "C" fn(*mut ::std::ffi::c_void, *mut crate::QAbstractItemModel),
        >,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr_set(
            self as *mut crate::RawSlotOfQAbstractItemModel,
            func,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr_tr(
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                    .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                    .as_raw_ptr(),
                n,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr_trUtf8(::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c).as_raw_ptr(), n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfIntInt {
    type Arguments = (::std::os::raw::c_int, ::std::os::raw::c_int);
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1custom_slot(int,int)\0"),
            )
        }
    }
}

/// Binds a Qt signal with arguments `::std::os::raw::c_int,::std::os::raw::c_int` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```int, int```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfIntInt<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfIntInt>,
    func: ::std::option::Option<
        Box<Box<dyn FnMut(::std::os::raw::c_int, ::std::os::raw::c_int) + 'a>>,
    >,
}

impl<'a> SlotOfIntInt<'a> {
    /// Constructs a new object.
    pub fn new<F: FnMut(::std::os::raw::c_int, ::std::os::raw::c_int) + 'a>(
        f: F,
    ) -> SlotOfIntInt<'a> {
        let mut obj = SlotOfIntInt::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<F: FnMut(::std::os::raw::c_int, ::std::os::raw::c_int) + 'a>(&mut self, f: F) {
        extern "C" fn callback(
            data: *mut ::std::ffi::c_void,
            arg0: ::std::os::raw::c_int,
            arg1: ::std::os::raw::c_int,
        ) {
            let func = unsafe {
                &mut *(data as *mut Box<dyn FnMut(::std::os::raw::c_int, ::std::os::raw::c_int)>)
            };
            func(arg0, arg1);
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<Box<FnMut(::std::os::raw::c_int, ::std::os::raw::c_int) + 'a>> =
            Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(&mut self) -> ::cpp_utils::MutRef<crate::RawSlotOfIntInt> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfIntInt<'a> {
    fn default() -> Self {
        SlotOfIntInt {
            raw_slot: unsafe { crate::RawSlotOfIntInt::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfIntInt<'a> {
    type Arguments = <crate::RawSlotOfIntInt as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `::std::os::raw::c_int,::std::os::raw::c_int` to a Rust extern function.
///
/// It's recommended to use `SlotOfIntInt` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```int, int```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfIntInt {
    _unused: u8,
}
impl RawSlotOfIntInt {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(&mut self, arg0: ::std::os::raw::c_int, arg1: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_int_custom_slot(
            self as *mut crate::RawSlotOfIntInt,
            arg0,
            arg1,
        )
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_int_metaObject(
                self as *const crate::RawSlotOfIntInt,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfIntInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_int_ctr_qt_core_ffi_slot_wrapper_int_int();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_int_qt_metacall(
            self as *mut crate::RawSlotOfIntInt,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_int_qt_metacast(
                self as *mut crate::RawSlotOfIntInt,
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                    .as_raw_ptr(),
            );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<
            extern "C" fn(*mut ::std::ffi::c_void, ::std::os::raw::c_int, ::std::os::raw::c_int),
        >,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_int_set(
            self as *mut crate::RawSlotOfIntInt,
            func,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_int_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_int_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_int_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfI64 {
    type Arguments = (i64,);
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1custom_slot(qint64)\0"),
            )
        }
    }
}

/// Binds a Qt signal with arguments `i64` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```qint64```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfI64<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfI64>,
    func: ::std::option::Option<Box<Box<dyn FnMut(i64) + 'a>>>,
}

impl<'a> SlotOfI64<'a> {
    /// Constructs a new object.
    pub fn new<F: FnMut(i64) + 'a>(f: F) -> SlotOfI64<'a> {
        let mut obj = SlotOfI64::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<F: FnMut(i64) + 'a>(&mut self, f: F) {
        extern "C" fn callback(data: *mut ::std::ffi::c_void, arg0: i64) {
            let func = unsafe { &mut *(data as *mut Box<dyn FnMut(i64)>) };
            func(arg0);
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<Box<FnMut(i64) + 'a>> = Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(&mut self) -> ::cpp_utils::MutRef<crate::RawSlotOfI64> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfI64<'a> {
    fn default() -> Self {
        SlotOfI64 {
            raw_slot: unsafe { crate::RawSlotOfI64::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfI64<'a> {
    type Arguments = <crate::RawSlotOfI64 as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `i64` to a Rust extern function.
///
/// It's recommended to use `SlotOfI64` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```qint64```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfI64 {
    _unused: u8,
}
impl RawSlotOfI64 {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(&mut self, arg0: i64) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_qint64_custom_slot(
            self as *mut crate::RawSlotOfI64,
            arg0,
        )
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_qint64_metaObject(
                self as *const crate::RawSlotOfI64,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfI64> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_qint64_ctr_qt_core_ffi_slot_wrapper_qint64();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_qint64_qt_metacall(
            self as *mut crate::RawSlotOfI64,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_qint64_qt_metacast(
                self as *mut crate::RawSlotOfI64,
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                    .as_raw_ptr(),
            );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<
            extern "C" fn(*mut ::std::ffi::c_void, ::std::os::raw::c_longlong),
        >,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_qint64_set(
            self as *mut crate::RawSlotOfI64,
            func,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_qint64_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_qint64_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_qint64_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlot {
    type Arguments = ();
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1custom_slot()\0"),
            )
        }
    }
}

/// Binds a Qt signal with no arguments to a Rust closure.
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct Slot<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlot>,
    func: ::std::option::Option<Box<Box<dyn FnMut() + 'a>>>,
}

impl<'a> Slot<'a> {
    /// Constructs a new object.
    pub fn new<F: FnMut() + 'a>(f: F) -> Slot<'a> {
        let mut obj = Slot::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<F: FnMut() + 'a>(&mut self, f: F) {
        extern "C" fn callback(data: *mut ::std::ffi::c_void) {
            let func = unsafe { &mut *(data as *mut Box<dyn FnMut()>) };
            func();
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<Box<FnMut() + 'a>> = Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(&mut self) -> ::cpp_utils::MutRef<crate::RawSlot> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for Slot<'a> {
    fn default() -> Self {
        Slot {
            raw_slot: unsafe { crate::RawSlot::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b Slot<'a> {
    type Arguments = <crate::RawSlot as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with no arguments to a Rust extern function.
///
/// It's recommended to use `Slot` instead because it provides a more high-level API.
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlot {
    _unused: u8,
}
impl RawSlot {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper__custom_slot(
            self as *mut crate::RawSlot,
        )
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper__metaObject(
            self as *const crate::RawSlot,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlot> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper__ctr_qt_core_ffi_slot_wrapper_();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper__qt_metacall(
            self as *mut crate::RawSlot,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper__qt_metacast(
            self as *mut crate::RawSlot,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<extern "C" fn(*mut ::std::ffi::c_void)>,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper__set(
            self as *mut crate::RawSlot,
            func,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper__staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper__tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper__trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfQObject {
    type Arguments = (*mut crate::QObject,);
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1custom_slot(QObject*)\0"),
            )
        }
    }
}

/// Binds a Qt signal with arguments `::cpp_utils::MutPtr<crate::QObject>` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```QObject*```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfQObject<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfQObject>,
    func: ::std::option::Option<Box<Box<dyn FnMut(::cpp_utils::MutPtr<crate::QObject>) + 'a>>>,
}

impl<'a> SlotOfQObject<'a> {
    /// Constructs a new object.
    pub fn new<F: FnMut(::cpp_utils::MutPtr<crate::QObject>) + 'a>(f: F) -> SlotOfQObject<'a> {
        let mut obj = SlotOfQObject::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<F: FnMut(::cpp_utils::MutPtr<crate::QObject>) + 'a>(&mut self, f: F) {
        extern "C" fn callback(data: *mut ::std::ffi::c_void, arg0: *mut crate::QObject) {
            let func =
                unsafe { &mut *(data as *mut Box<dyn FnMut(::cpp_utils::MutPtr<crate::QObject>)>) };
            func(unsafe { ::cpp_utils::MutPtr::from_raw(arg0) });
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<Box<FnMut(::cpp_utils::MutPtr<crate::QObject>) + 'a>> =
            Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(&mut self) -> ::cpp_utils::MutRef<crate::RawSlotOfQObject> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfQObject<'a> {
    fn default() -> Self {
        SlotOfQObject {
            raw_slot: unsafe { crate::RawSlotOfQObject::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfQObject<'a> {
    type Arguments = <crate::RawSlotOfQObject as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `*mut crate::QObject` to a Rust extern function.
///
/// It's recommended to use `SlotOfQObject` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```QObject*```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfQObject {
    _unused: u8,
}
impl RawSlotOfQObject {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(
        &mut self,
        arg0: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QObject_ptr_custom_slot(
            self as *mut crate::RawSlotOfQObject,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(arg0)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QObject_ptr_metaObject(
                self as *const crate::RawSlotOfQObject,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfQObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QObject_ptr_ctr_qt_core_ffi_slot_wrapper_QObject_ptr();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QObject_ptr_qt_metacall(
            self as *mut crate::RawSlotOfQObject,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QObject_ptr_qt_metacast(
                self as *mut crate::RawSlotOfQObject,
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                    .as_raw_ptr(),
            );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<extern "C" fn(*mut ::std::ffi::c_void, *mut crate::QObject)>,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QObject_ptr_set(
            self as *mut crate::RawSlotOfQObject,
            func,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QObject_ptr_staticMetaObject(
            );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QObject_ptr_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QObject_ptr_trUtf8(
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                    .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                    .as_raw_ptr(),
                n,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfQString {
    type Arguments = (*const crate::QString,);
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1custom_slot(const QString&)\0"),
            )
        }
    }
}

/// Binds a Qt signal with arguments `::cpp_utils::Ref<crate::QString>` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```const QString&```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfQString<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfQString>,
    func: ::std::option::Option<Box<Box<dyn FnMut(::cpp_utils::Ref<crate::QString>) + 'a>>>,
}

impl<'a> SlotOfQString<'a> {
    /// Constructs a new object.
    pub fn new<F: FnMut(::cpp_utils::Ref<crate::QString>) + 'a>(f: F) -> SlotOfQString<'a> {
        let mut obj = SlotOfQString::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<F: FnMut(::cpp_utils::Ref<crate::QString>) + 'a>(&mut self, f: F) {
        extern "C" fn callback(data: *mut ::std::ffi::c_void, arg0: *const crate::QString) {
            let func =
                unsafe { &mut *(data as *mut Box<dyn FnMut(::cpp_utils::Ref<crate::QString>)>) };
            func(unsafe {
                ::cpp_utils::Ref::from_raw(arg0).expect("attempted to construct a null Ref")
            });
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<Box<FnMut(::cpp_utils::Ref<crate::QString>) + 'a>> =
            Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(&mut self) -> ::cpp_utils::MutRef<crate::RawSlotOfQString> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfQString<'a> {
    fn default() -> Self {
        SlotOfQString {
            raw_slot: unsafe { crate::RawSlotOfQString::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfQString<'a> {
    type Arguments = <crate::RawSlotOfQString as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `*const crate::QString` to a Rust extern function.
///
/// It's recommended to use `SlotOfQString` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```const QString&```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfQString {
    _unused: u8,
}
impl RawSlotOfQString {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(
        &mut self,
        arg0: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QString_const_ref_custom_slot(
            self as *mut crate::RawSlotOfQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(arg0).as_raw_ptr(),
        )
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QString_const_ref_metaObject(
                self as *const crate::RawSlotOfQString,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QString_const_ref_ctr_qt_core_ffi_slot_wrapper_QString_const_ref();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QString_const_ref_qt_metacall(
            self as *mut crate::RawSlotOfQString,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QString_const_ref_qt_metacast(self as *mut crate::RawSlotOfQString, ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<extern "C" fn(*mut ::std::ffi::c_void, *const crate::QString)>,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QString_const_ref_set(
            self as *mut crate::RawSlotOfQString,
            func,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QString_const_ref_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QString_const_ref_tr(
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                    .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                    .as_raw_ptr(),
                n,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QString_const_ref_trUtf8(
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                    .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                    .as_raw_ptr(),
                n,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfDirection {
    type Arguments = (crate::q_abstract_animation::Direction,);
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"1custom_slot(QAbstractAnimation::Direction)\0",
                ),
            )
        }
    }
}

/// Binds a Qt signal with arguments `crate::q_abstract_animation::Direction` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```QAbstractAnimation::Direction```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfDirection<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfDirection>,
    func: ::std::option::Option<Box<Box<dyn FnMut(crate::q_abstract_animation::Direction) + 'a>>>,
}

impl<'a> SlotOfDirection<'a> {
    /// Constructs a new object.
    pub fn new<F: FnMut(crate::q_abstract_animation::Direction) + 'a>(f: F) -> SlotOfDirection<'a> {
        let mut obj = SlotOfDirection::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<F: FnMut(crate::q_abstract_animation::Direction) + 'a>(&mut self, f: F) {
        extern "C" fn callback(
            data: *mut ::std::ffi::c_void,
            arg0: crate::q_abstract_animation::Direction,
        ) {
            let func = unsafe {
                &mut *(data as *mut Box<dyn FnMut(crate::q_abstract_animation::Direction)>)
            };
            func(arg0);
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<Box<FnMut(crate::q_abstract_animation::Direction) + 'a>> =
            Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(&mut self) -> ::cpp_utils::MutRef<crate::RawSlotOfDirection> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfDirection<'a> {
    fn default() -> Self {
        SlotOfDirection {
            raw_slot: unsafe { crate::RawSlotOfDirection::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfDirection<'a> {
    type Arguments = <crate::RawSlotOfDirection as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `crate::q_abstract_animation::Direction` to a Rust extern function.
///
/// It's recommended to use `SlotOfDirection` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```QAbstractAnimation::Direction```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfDirection {
    _unused: u8,
}
impl RawSlotOfDirection {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(&mut self, arg0: crate::q_abstract_animation::Direction) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction_custom_slot(self as *mut crate::RawSlotOfDirection, arg0)
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction_metaObject(self as *const crate::RawSlotOfDirection);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfDirection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction_qt_metacall(self as *mut crate::RawSlotOfDirection, arg1, arg2, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3).as_mut_raw_ptr())
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction_qt_metacast(self as *mut crate::RawSlotOfDirection, ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<
            extern "C" fn(*mut ::std::ffi::c_void, crate::q_abstract_animation::Direction),
        >,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction_set(
            self as *mut crate::RawSlotOfDirection,
            func,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction_tr(::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c).as_raw_ptr(), n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction_trUtf8(::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c).as_raw_ptr(), n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfQModelIndexIntIntQModelIndex {
    type Arguments = (
        *const crate::QModelIndex,
        ::std::os::raw::c_int,
        ::std::os::raw::c_int,
        *const crate::QModelIndex,
    );
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"1custom_slot(const QModelIndex&,int,int,const QModelIndex&)\0",
                ),
            )
        }
    }
}

/// Binds a Qt signal with arguments `::cpp_utils::Ref<crate::QModelIndex>,::std::os::raw::c_int,::std::os::raw::c_int,::cpp_utils::Ref<crate::QModelIndex>` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```const QModelIndex&, int, int, const QModelIndex&```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfQModelIndexIntIntQModelIndex<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfQModelIndexIntIntQModelIndex>,
    func: ::std::option::Option<
        Box<
            Box<
                dyn FnMut(
                        ::cpp_utils::Ref<crate::QModelIndex>,
                        ::std::os::raw::c_int,
                        ::std::os::raw::c_int,
                        ::cpp_utils::Ref<crate::QModelIndex>,
                    ) + 'a,
            >,
        >,
    >,
}

impl<'a> SlotOfQModelIndexIntIntQModelIndex<'a> {
    /// Constructs a new object.
    pub fn new<
        F: FnMut(
                ::cpp_utils::Ref<crate::QModelIndex>,
                ::std::os::raw::c_int,
                ::std::os::raw::c_int,
                ::cpp_utils::Ref<crate::QModelIndex>,
            ) + 'a,
    >(
        f: F,
    ) -> SlotOfQModelIndexIntIntQModelIndex<'a> {
        let mut obj = SlotOfQModelIndexIntIntQModelIndex::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<
        F: FnMut(
                ::cpp_utils::Ref<crate::QModelIndex>,
                ::std::os::raw::c_int,
                ::std::os::raw::c_int,
                ::cpp_utils::Ref<crate::QModelIndex>,
            ) + 'a,
    >(
        &mut self,
        f: F,
    ) {
        extern "C" fn callback(
            data: *mut ::std::ffi::c_void,
            arg0: *const crate::QModelIndex,
            arg1: ::std::os::raw::c_int,
            arg2: ::std::os::raw::c_int,
            arg3: *const crate::QModelIndex,
        ) {
            let func = unsafe {
                &mut *(data as *mut Box<
                    dyn FnMut(
                        ::cpp_utils::Ref<crate::QModelIndex>,
                        ::std::os::raw::c_int,
                        ::std::os::raw::c_int,
                        ::cpp_utils::Ref<crate::QModelIndex>,
                    ),
                >)
            };
            func(
                unsafe {
                    ::cpp_utils::Ref::from_raw(arg0).expect("attempted to construct a null Ref")
                },
                arg1,
                arg2,
                unsafe {
                    ::cpp_utils::Ref::from_raw(arg3).expect("attempted to construct a null Ref")
                },
            );
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<
            Box<
                FnMut(
                        ::cpp_utils::Ref<crate::QModelIndex>,
                        ::std::os::raw::c_int,
                        ::std::os::raw::c_int,
                        ::cpp_utils::Ref<crate::QModelIndex>,
                    ) + 'a,
            >,
        > = Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(
        &mut self,
    ) -> ::cpp_utils::MutRef<crate::RawSlotOfQModelIndexIntIntQModelIndex> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfQModelIndexIntIntQModelIndex<'a> {
    fn default() -> Self {
        SlotOfQModelIndexIntIntQModelIndex {
            raw_slot: unsafe { crate::RawSlotOfQModelIndexIntIntQModelIndex::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfQModelIndexIntIntQModelIndex<'a> {
    type Arguments = <crate::RawSlotOfQModelIndexIntIntQModelIndex as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `*const crate::QModelIndex,::std::os::raw::c_int,::std::os::raw::c_int,*const crate::QModelIndex` to a Rust extern function.
///
/// It's recommended to use `SlotOfQModelIndexIntIntQModelIndex` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```const QModelIndex&, int, int, const QModelIndex&```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfQModelIndexIntIntQModelIndex {
    _unused: u8,
}
impl RawSlotOfQModelIndexIntIntQModelIndex {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(
        &mut self,
        arg0: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        arg1: ::std::os::raw::c_int,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_custom_slot(self as *mut crate::RawSlotOfQModelIndexIntIntQModelIndex, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(arg0).as_raw_ptr(), arg1, arg2, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(arg3).as_raw_ptr())
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_metaObject(self as *const crate::RawSlotOfQModelIndexIntIntQModelIndex);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfQModelIndexIntIntQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_qt_metacall(self as *mut crate::RawSlotOfQModelIndexIntIntQModelIndex, arg1, arg2, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3).as_mut_raw_ptr())
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_qt_metacast(self as *mut crate::RawSlotOfQModelIndexIntIntQModelIndex, ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<
            extern "C" fn(
                *mut ::std::ffi::c_void,
                *const crate::QModelIndex,
                ::std::os::raw::c_int,
                ::std::os::raw::c_int,
                *const crate::QModelIndex,
            ),
        >,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_set(self as *mut crate::RawSlotOfQModelIndexIntIntQModelIndex, func, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data).as_mut_raw_ptr())
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_tr(::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c).as_raw_ptr(), n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_trUtf8(::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c).as_raw_ptr(), n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfQModelIndexQModelIndex {
    type Arguments = (*const crate::QModelIndex, *const crate::QModelIndex);
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"1custom_slot(const QModelIndex&,const QModelIndex&)\0",
                ),
            )
        }
    }
}

/// Binds a Qt signal with arguments `::cpp_utils::Ref<crate::QModelIndex>,::cpp_utils::Ref<crate::QModelIndex>` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```const QModelIndex&, const QModelIndex&```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfQModelIndexQModelIndex<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfQModelIndexQModelIndex>,
    func: ::std::option::Option<
        Box<
            Box<
                dyn FnMut(
                        ::cpp_utils::Ref<crate::QModelIndex>,
                        ::cpp_utils::Ref<crate::QModelIndex>,
                    ) + 'a,
            >,
        >,
    >,
}

impl<'a> SlotOfQModelIndexQModelIndex<'a> {
    /// Constructs a new object.
    pub fn new<
        F: FnMut(::cpp_utils::Ref<crate::QModelIndex>, ::cpp_utils::Ref<crate::QModelIndex>) + 'a,
    >(
        f: F,
    ) -> SlotOfQModelIndexQModelIndex<'a> {
        let mut obj = SlotOfQModelIndexQModelIndex::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<
        F: FnMut(::cpp_utils::Ref<crate::QModelIndex>, ::cpp_utils::Ref<crate::QModelIndex>) + 'a,
    >(
        &mut self,
        f: F,
    ) {
        extern "C" fn callback(
            data: *mut ::std::ffi::c_void,
            arg0: *const crate::QModelIndex,
            arg1: *const crate::QModelIndex,
        ) {
            let func = unsafe {
                &mut *(data as *mut Box<
                    dyn FnMut(
                        ::cpp_utils::Ref<crate::QModelIndex>,
                        ::cpp_utils::Ref<crate::QModelIndex>,
                    ),
                >)
            };
            func(
                unsafe {
                    ::cpp_utils::Ref::from_raw(arg0).expect("attempted to construct a null Ref")
                },
                unsafe {
                    ::cpp_utils::Ref::from_raw(arg1).expect("attempted to construct a null Ref")
                },
            );
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<
            Box<
                FnMut(::cpp_utils::Ref<crate::QModelIndex>, ::cpp_utils::Ref<crate::QModelIndex>)
                    + 'a,
            >,
        > = Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(&mut self) -> ::cpp_utils::MutRef<crate::RawSlotOfQModelIndexQModelIndex> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfQModelIndexQModelIndex<'a> {
    fn default() -> Self {
        SlotOfQModelIndexQModelIndex {
            raw_slot: unsafe { crate::RawSlotOfQModelIndexQModelIndex::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfQModelIndexQModelIndex<'a> {
    type Arguments = <crate::RawSlotOfQModelIndexQModelIndex as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `*const crate::QModelIndex,*const crate::QModelIndex` to a Rust extern function.
///
/// It's recommended to use `SlotOfQModelIndexQModelIndex` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```const QModelIndex&, const QModelIndex&```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfQModelIndexQModelIndex {
    _unused: u8,
}
impl RawSlotOfQModelIndexQModelIndex {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(
        &mut self,
        arg0: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_custom_slot(self as *mut crate::RawSlotOfQModelIndexQModelIndex, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(arg0).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(arg1).as_raw_ptr())
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_metaObject(self as *const crate::RawSlotOfQModelIndexQModelIndex);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfQModelIndexQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_qt_metacall(self as *mut crate::RawSlotOfQModelIndexQModelIndex, arg1, arg2, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3).as_mut_raw_ptr())
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_qt_metacast(self as *mut crate::RawSlotOfQModelIndexQModelIndex, ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<
            extern "C" fn(
                *mut ::std::ffi::c_void,
                *const crate::QModelIndex,
                *const crate::QModelIndex,
            ),
        >,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_set(self as *mut crate::RawSlotOfQModelIndexQModelIndex, func, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data).as_mut_raw_ptr())
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_tr(::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c).as_raw_ptr(), n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_trUtf8(::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c).as_raw_ptr(), n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfInt {
    type Arguments = (::std::os::raw::c_int,);
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1custom_slot(int)\0"),
            )
        }
    }
}

/// Binds a Qt signal with arguments `::std::os::raw::c_int` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```int```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfInt<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfInt>,
    func: ::std::option::Option<Box<Box<dyn FnMut(::std::os::raw::c_int) + 'a>>>,
}

impl<'a> SlotOfInt<'a> {
    /// Constructs a new object.
    pub fn new<F: FnMut(::std::os::raw::c_int) + 'a>(f: F) -> SlotOfInt<'a> {
        let mut obj = SlotOfInt::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<F: FnMut(::std::os::raw::c_int) + 'a>(&mut self, f: F) {
        extern "C" fn callback(data: *mut ::std::ffi::c_void, arg0: ::std::os::raw::c_int) {
            let func = unsafe { &mut *(data as *mut Box<dyn FnMut(::std::os::raw::c_int)>) };
            func(arg0);
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<Box<FnMut(::std::os::raw::c_int) + 'a>> = Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(&mut self) -> ::cpp_utils::MutRef<crate::RawSlotOfInt> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfInt<'a> {
    fn default() -> Self {
        SlotOfInt {
            raw_slot: unsafe { crate::RawSlotOfInt::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfInt<'a> {
    type Arguments = <crate::RawSlotOfInt as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `::std::os::raw::c_int` to a Rust extern function.
///
/// It's recommended to use `SlotOfInt` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```int```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfInt {
    _unused: u8,
}
impl RawSlotOfInt {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(&mut self, arg0: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_custom_slot(
            self as *mut crate::RawSlotOfInt,
            arg0,
        )
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_metaObject(
            self as *const crate::RawSlotOfInt,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_ctr_qt_core_ffi_slot_wrapper_int();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_qt_metacall(
            self as *mut crate::RawSlotOfInt,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_qt_metacast(
            self as *mut crate::RawSlotOfInt,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<extern "C" fn(*mut ::std::ffi::c_void, ::std::os::raw::c_int)>,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_set(
            self as *mut crate::RawSlotOfInt,
            func,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfQModelIndexIntInt {
    type Arguments = (
        *const crate::QModelIndex,
        ::std::os::raw::c_int,
        ::std::os::raw::c_int,
    );
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"1custom_slot(const QModelIndex&,int,int)\0",
                ),
            )
        }
    }
}

/// Binds a Qt signal with arguments `::cpp_utils::Ref<crate::QModelIndex>,::std::os::raw::c_int,::std::os::raw::c_int` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```const QModelIndex&, int, int```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfQModelIndexIntInt<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfQModelIndexIntInt>,
    func: ::std::option::Option<
        Box<
            Box<
                dyn FnMut(
                        ::cpp_utils::Ref<crate::QModelIndex>,
                        ::std::os::raw::c_int,
                        ::std::os::raw::c_int,
                    ) + 'a,
            >,
        >,
    >,
}

impl<'a> SlotOfQModelIndexIntInt<'a> {
    /// Constructs a new object.
    pub fn new<
        F: FnMut(::cpp_utils::Ref<crate::QModelIndex>, ::std::os::raw::c_int, ::std::os::raw::c_int)
            + 'a,
    >(
        f: F,
    ) -> SlotOfQModelIndexIntInt<'a> {
        let mut obj = SlotOfQModelIndexIntInt::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<
        F: FnMut(::cpp_utils::Ref<crate::QModelIndex>, ::std::os::raw::c_int, ::std::os::raw::c_int)
            + 'a,
    >(
        &mut self,
        f: F,
    ) {
        extern "C" fn callback(
            data: *mut ::std::ffi::c_void,
            arg0: *const crate::QModelIndex,
            arg1: ::std::os::raw::c_int,
            arg2: ::std::os::raw::c_int,
        ) {
            let func = unsafe {
                &mut *(data as *mut Box<
                    dyn FnMut(
                        ::cpp_utils::Ref<crate::QModelIndex>,
                        ::std::os::raw::c_int,
                        ::std::os::raw::c_int,
                    ),
                >)
            };
            func(
                unsafe {
                    ::cpp_utils::Ref::from_raw(arg0).expect("attempted to construct a null Ref")
                },
                arg1,
                arg2,
            );
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<
            Box<
                FnMut(
                        ::cpp_utils::Ref<crate::QModelIndex>,
                        ::std::os::raw::c_int,
                        ::std::os::raw::c_int,
                    ) + 'a,
            >,
        > = Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(&mut self) -> ::cpp_utils::MutRef<crate::RawSlotOfQModelIndexIntInt> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfQModelIndexIntInt<'a> {
    fn default() -> Self {
        SlotOfQModelIndexIntInt {
            raw_slot: unsafe { crate::RawSlotOfQModelIndexIntInt::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfQModelIndexIntInt<'a> {
    type Arguments = <crate::RawSlotOfQModelIndexIntInt as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `*const crate::QModelIndex,::std::os::raw::c_int,::std::os::raw::c_int` to a Rust extern function.
///
/// It's recommended to use `SlotOfQModelIndexIntInt` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```const QModelIndex&, int, int```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfQModelIndexIntInt {
    _unused: u8,
}
impl RawSlotOfQModelIndexIntInt {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(
        &mut self,
        arg0: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        arg1: ::std::os::raw::c_int,
        arg2: ::std::os::raw::c_int,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_custom_slot(self as *mut crate::RawSlotOfQModelIndexIntInt, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(arg0).as_raw_ptr(), arg1, arg2)
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_metaObject(self as *const crate::RawSlotOfQModelIndexIntInt);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfQModelIndexIntInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_qt_metacall(self as *mut crate::RawSlotOfQModelIndexIntInt, arg1, arg2, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3).as_mut_raw_ptr())
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_qt_metacast(self as *mut crate::RawSlotOfQModelIndexIntInt, ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<
            extern "C" fn(
                *mut ::std::ffi::c_void,
                *const crate::QModelIndex,
                ::std::os::raw::c_int,
                ::std::os::raw::c_int,
            ),
        >,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_set(
            self as *mut crate::RawSlotOfQModelIndexIntInt,
            func,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_tr(::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c).as_raw_ptr(), n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_trUtf8(::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c).as_raw_ptr(), n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfQItemSelectionQItemSelection {
    type Arguments = (*const crate::QItemSelection, *const crate::QItemSelection);
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"1custom_slot(const QItemSelection&,const QItemSelection&)\0",
                ),
            )
        }
    }
}

/// Binds a Qt signal with arguments `::cpp_utils::Ref<crate::QItemSelection>,::cpp_utils::Ref<crate::QItemSelection>` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```const QItemSelection&, const QItemSelection&```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfQItemSelectionQItemSelection<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfQItemSelectionQItemSelection>,
    func: ::std::option::Option<
        Box<
            Box<
                dyn FnMut(
                        ::cpp_utils::Ref<crate::QItemSelection>,
                        ::cpp_utils::Ref<crate::QItemSelection>,
                    ) + 'a,
            >,
        >,
    >,
}

impl<'a> SlotOfQItemSelectionQItemSelection<'a> {
    /// Constructs a new object.
    pub fn new<
        F: FnMut(::cpp_utils::Ref<crate::QItemSelection>, ::cpp_utils::Ref<crate::QItemSelection>)
            + 'a,
    >(
        f: F,
    ) -> SlotOfQItemSelectionQItemSelection<'a> {
        let mut obj = SlotOfQItemSelectionQItemSelection::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<
        F: FnMut(::cpp_utils::Ref<crate::QItemSelection>, ::cpp_utils::Ref<crate::QItemSelection>)
            + 'a,
    >(
        &mut self,
        f: F,
    ) {
        extern "C" fn callback(
            data: *mut ::std::ffi::c_void,
            arg0: *const crate::QItemSelection,
            arg1: *const crate::QItemSelection,
        ) {
            let func = unsafe {
                &mut *(data as *mut Box<
                    dyn FnMut(
                        ::cpp_utils::Ref<crate::QItemSelection>,
                        ::cpp_utils::Ref<crate::QItemSelection>,
                    ),
                >)
            };
            func(
                unsafe {
                    ::cpp_utils::Ref::from_raw(arg0).expect("attempted to construct a null Ref")
                },
                unsafe {
                    ::cpp_utils::Ref::from_raw(arg1).expect("attempted to construct a null Ref")
                },
            );
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<
            Box<
                FnMut(
                        ::cpp_utils::Ref<crate::QItemSelection>,
                        ::cpp_utils::Ref<crate::QItemSelection>,
                    ) + 'a,
            >,
        > = Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(
        &mut self,
    ) -> ::cpp_utils::MutRef<crate::RawSlotOfQItemSelectionQItemSelection> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfQItemSelectionQItemSelection<'a> {
    fn default() -> Self {
        SlotOfQItemSelectionQItemSelection {
            raw_slot: unsafe { crate::RawSlotOfQItemSelectionQItemSelection::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfQItemSelectionQItemSelection<'a> {
    type Arguments = <crate::RawSlotOfQItemSelectionQItemSelection as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `*const crate::QItemSelection,*const crate::QItemSelection` to a Rust extern function.
///
/// It's recommended to use `SlotOfQItemSelectionQItemSelection` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```const QItemSelection&, const QItemSelection&```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfQItemSelectionQItemSelection {
    _unused: u8,
}
impl RawSlotOfQItemSelectionQItemSelection {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(
        &mut self,
        arg0: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelection>>,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelection>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref_custom_slot(self as *mut crate::RawSlotOfQItemSelectionQItemSelection, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelection>>::cast_into(arg0).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelection>>::cast_into(arg1).as_raw_ptr())
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref_metaObject(self as *const crate::RawSlotOfQItemSelectionQItemSelection);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfQItemSelectionQItemSelection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref_qt_metacall(self as *mut crate::RawSlotOfQItemSelectionQItemSelection, arg1, arg2, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3).as_mut_raw_ptr())
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref_qt_metacast(self as *mut crate::RawSlotOfQItemSelectionQItemSelection, ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<
            extern "C" fn(
                *mut ::std::ffi::c_void,
                *const crate::QItemSelection,
                *const crate::QItemSelection,
            ),
        >,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref_set(self as *mut crate::RawSlotOfQItemSelectionQItemSelection, func, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data).as_mut_raw_ptr())
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref_tr(::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c).as_raw_ptr(), n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref_trUtf8(::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c).as_raw_ptr(), n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfDouble {
    type Arguments = (::std::os::raw::c_double,);
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(b"1custom_slot(double)\0"),
            )
        }
    }
}

/// Binds a Qt signal with arguments `::std::os::raw::c_double` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```double```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfDouble<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfDouble>,
    func: ::std::option::Option<Box<Box<dyn FnMut(::std::os::raw::c_double) + 'a>>>,
}

impl<'a> SlotOfDouble<'a> {
    /// Constructs a new object.
    pub fn new<F: FnMut(::std::os::raw::c_double) + 'a>(f: F) -> SlotOfDouble<'a> {
        let mut obj = SlotOfDouble::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<F: FnMut(::std::os::raw::c_double) + 'a>(&mut self, f: F) {
        extern "C" fn callback(data: *mut ::std::ffi::c_void, arg0: ::std::os::raw::c_double) {
            let func = unsafe { &mut *(data as *mut Box<dyn FnMut(::std::os::raw::c_double)>) };
            func(arg0);
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<Box<FnMut(::std::os::raw::c_double) + 'a>> = Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(&mut self) -> ::cpp_utils::MutRef<crate::RawSlotOfDouble> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfDouble<'a> {
    fn default() -> Self {
        SlotOfDouble {
            raw_slot: unsafe { crate::RawSlotOfDouble::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfDouble<'a> {
    type Arguments = <crate::RawSlotOfDouble as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `::std::os::raw::c_double` to a Rust extern function.
///
/// It's recommended to use `SlotOfDouble` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```double```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfDouble {
    _unused: u8,
}
impl RawSlotOfDouble {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(&mut self, arg0: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_double_custom_slot(
            self as *mut crate::RawSlotOfDouble,
            arg0,
        )
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_double_metaObject(
                self as *const crate::RawSlotOfDouble,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfDouble> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_double_ctr_qt_core_ffi_slot_wrapper_double();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_double_qt_metacall(
            self as *mut crate::RawSlotOfDouble,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_double_qt_metacast(
                self as *mut crate::RawSlotOfDouble,
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                    .as_raw_ptr(),
            );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<
            extern "C" fn(*mut ::std::ffi::c_void, ::std::os::raw::c_double),
        >,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_double_set(
            self as *mut crate::RawSlotOfDouble,
            func,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_double_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_double_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_double_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfState2 {
    type Arguments = (crate::q_time_line::State,);
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"1custom_slot(QTimeLine::State)\0",
                ),
            )
        }
    }
}

/// Binds a Qt signal with arguments `crate::q_time_line::State` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```QTimeLine::State```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfState2<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfState2>,
    func: ::std::option::Option<Box<Box<dyn FnMut(crate::q_time_line::State) + 'a>>>,
}

impl<'a> SlotOfState2<'a> {
    /// Constructs a new object.
    pub fn new<F: FnMut(crate::q_time_line::State) + 'a>(f: F) -> SlotOfState2<'a> {
        let mut obj = SlotOfState2::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<F: FnMut(crate::q_time_line::State) + 'a>(&mut self, f: F) {
        extern "C" fn callback(data: *mut ::std::ffi::c_void, arg0: crate::q_time_line::State) {
            let func = unsafe { &mut *(data as *mut Box<dyn FnMut(crate::q_time_line::State)>) };
            func(arg0);
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<Box<FnMut(crate::q_time_line::State) + 'a>> = Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(&mut self) -> ::cpp_utils::MutRef<crate::RawSlotOfState2> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfState2<'a> {
    fn default() -> Self {
        SlotOfState2 {
            raw_slot: unsafe { crate::RawSlotOfState2::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfState2<'a> {
    type Arguments = <crate::RawSlotOfState2 as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `crate::q_time_line::State` to a Rust extern function.
///
/// It's recommended to use `SlotOfState2` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```QTimeLine::State```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfState2 {
    _unused: u8,
}
impl RawSlotOfState2 {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(&mut self, arg0: crate::q_time_line::State) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QTimeLine_State_custom_slot(
            self as *mut crate::RawSlotOfState2,
            arg0,
        )
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QTimeLine_State_metaObject(
                self as *const crate::RawSlotOfState2,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfState2> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QTimeLine_State_ctr_qt_core_ffi_slot_wrapper_QTimeLine_State();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QTimeLine_State_qt_metacall(
            self as *mut crate::RawSlotOfState2,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QTimeLine_State_qt_metacast(
                self as *mut crate::RawSlotOfState2,
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                    .as_raw_ptr(),
            );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<
            extern "C" fn(*mut ::std::ffi::c_void, crate::q_time_line::State),
        >,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QTimeLine_State_set(
            self as *mut crate::RawSlotOfState2,
            func,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QTimeLine_State_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QTimeLine_State_tr(
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                    .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                    .as_raw_ptr(),
                n,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QTimeLine_State_trUtf8(
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                    .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                    .as_raw_ptr(),
                n,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfStateState {
    type Arguments = (
        crate::q_abstract_animation::State,
        crate::q_abstract_animation::State,
    );
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"1custom_slot(QAbstractAnimation::State,QAbstractAnimation::State)\0",
                ),
            )
        }
    }
}

/// Binds a Qt signal with arguments `crate::q_abstract_animation::State,crate::q_abstract_animation::State` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```QAbstractAnimation::State, QAbstractAnimation::State```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfStateState<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfStateState>,
    func: ::std::option::Option<
        Box<
            Box<
                dyn FnMut(crate::q_abstract_animation::State, crate::q_abstract_animation::State)
                    + 'a,
            >,
        >,
    >,
}

impl<'a> SlotOfStateState<'a> {
    /// Constructs a new object.
    pub fn new<
        F: FnMut(crate::q_abstract_animation::State, crate::q_abstract_animation::State) + 'a,
    >(
        f: F,
    ) -> SlotOfStateState<'a> {
        let mut obj = SlotOfStateState::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<
        F: FnMut(crate::q_abstract_animation::State, crate::q_abstract_animation::State) + 'a,
    >(
        &mut self,
        f: F,
    ) {
        extern "C" fn callback(
            data: *mut ::std::ffi::c_void,
            arg0: crate::q_abstract_animation::State,
            arg1: crate::q_abstract_animation::State,
        ) {
            let func = unsafe {
                &mut *(data as *mut Box<
                    dyn FnMut(
                        crate::q_abstract_animation::State,
                        crate::q_abstract_animation::State,
                    ),
                >)
            };
            func(arg0, arg1);
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<
            Box<FnMut(crate::q_abstract_animation::State, crate::q_abstract_animation::State) + 'a>,
        > = Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(&mut self) -> ::cpp_utils::MutRef<crate::RawSlotOfStateState> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfStateState<'a> {
    fn default() -> Self {
        SlotOfStateState {
            raw_slot: unsafe { crate::RawSlotOfStateState::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfStateState<'a> {
    type Arguments = <crate::RawSlotOfStateState as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `crate::q_abstract_animation::State,crate::q_abstract_animation::State` to a Rust extern function.
///
/// It's recommended to use `SlotOfStateState` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```QAbstractAnimation::State, QAbstractAnimation::State```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfStateState {
    _unused: u8,
}
impl RawSlotOfStateState {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(
        &mut self,
        arg0: crate::q_abstract_animation::State,
        arg1: crate::q_abstract_animation::State,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State_custom_slot(self as *mut crate::RawSlotOfStateState, arg0, arg1)
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State_metaObject(self as *const crate::RawSlotOfStateState);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfStateState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State_qt_metacall(self as *mut crate::RawSlotOfStateState, arg1, arg2, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3).as_mut_raw_ptr())
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State_qt_metacast(self as *mut crate::RawSlotOfStateState, ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<
            extern "C" fn(
                *mut ::std::ffi::c_void,
                crate::q_abstract_animation::State,
                crate::q_abstract_animation::State,
            ),
        >,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State_set(self as *mut crate::RawSlotOfStateState, func, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data).as_mut_raw_ptr())
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State_tr(::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c).as_raw_ptr(), n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State_trUtf8(::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c).as_raw_ptr(), n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfProcessError {
    type Arguments = (crate::q_process::ProcessError,);
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"1custom_slot(QProcess::ProcessError)\0",
                ),
            )
        }
    }
}

/// Binds a Qt signal with arguments `crate::q_process::ProcessError` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```QProcess::ProcessError```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfProcessError<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfProcessError>,
    func: ::std::option::Option<Box<Box<dyn FnMut(crate::q_process::ProcessError) + 'a>>>,
}

impl<'a> SlotOfProcessError<'a> {
    /// Constructs a new object.
    pub fn new<F: FnMut(crate::q_process::ProcessError) + 'a>(f: F) -> SlotOfProcessError<'a> {
        let mut obj = SlotOfProcessError::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<F: FnMut(crate::q_process::ProcessError) + 'a>(&mut self, f: F) {
        extern "C" fn callback(
            data: *mut ::std::ffi::c_void,
            arg0: crate::q_process::ProcessError,
        ) {
            let func =
                unsafe { &mut *(data as *mut Box<dyn FnMut(crate::q_process::ProcessError)>) };
            func(arg0);
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<Box<FnMut(crate::q_process::ProcessError) + 'a>> =
            Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(&mut self) -> ::cpp_utils::MutRef<crate::RawSlotOfProcessError> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfProcessError<'a> {
    fn default() -> Self {
        SlotOfProcessError {
            raw_slot: unsafe { crate::RawSlotOfProcessError::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfProcessError<'a> {
    type Arguments = <crate::RawSlotOfProcessError as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `crate::q_process::ProcessError` to a Rust extern function.
///
/// It's recommended to use `SlotOfProcessError` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```QProcess::ProcessError```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfProcessError {
    _unused: u8,
}
impl RawSlotOfProcessError {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(&mut self, arg0: crate::q_process::ProcessError) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError_custom_slot(
            self as *mut crate::RawSlotOfProcessError,
            arg0,
        )
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError_metaObject(self as *const crate::RawSlotOfProcessError);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfProcessError> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError_qt_metacall(
            self as *mut crate::RawSlotOfProcessError,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError_qt_metacast(self as *mut crate::RawSlotOfProcessError, ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<
            extern "C" fn(*mut ::std::ffi::c_void, crate::q_process::ProcessError),
        >,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError_set(
            self as *mut crate::RawSlotOfProcessError,
            func,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError_tr(
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                    .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                    .as_raw_ptr(),
                n,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError_trUtf8(
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                    .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                    .as_raw_ptr(),
                n,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfQItemSelection {
    type Arguments = (*const crate::QItemSelection,);
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"1custom_slot(const QItemSelection&)\0",
                ),
            )
        }
    }
}

/// Binds a Qt signal with arguments `::cpp_utils::Ref<crate::QItemSelection>` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```const QItemSelection&```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfQItemSelection<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfQItemSelection>,
    func: ::std::option::Option<Box<Box<dyn FnMut(::cpp_utils::Ref<crate::QItemSelection>) + 'a>>>,
}

impl<'a> SlotOfQItemSelection<'a> {
    /// Constructs a new object.
    pub fn new<F: FnMut(::cpp_utils::Ref<crate::QItemSelection>) + 'a>(
        f: F,
    ) -> SlotOfQItemSelection<'a> {
        let mut obj = SlotOfQItemSelection::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<F: FnMut(::cpp_utils::Ref<crate::QItemSelection>) + 'a>(&mut self, f: F) {
        extern "C" fn callback(data: *mut ::std::ffi::c_void, arg0: *const crate::QItemSelection) {
            let func = unsafe {
                &mut *(data as *mut Box<dyn FnMut(::cpp_utils::Ref<crate::QItemSelection>)>)
            };
            func(unsafe {
                ::cpp_utils::Ref::from_raw(arg0).expect("attempted to construct a null Ref")
            });
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<Box<FnMut(::cpp_utils::Ref<crate::QItemSelection>) + 'a>> =
            Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(&mut self) -> ::cpp_utils::MutRef<crate::RawSlotOfQItemSelection> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfQItemSelection<'a> {
    fn default() -> Self {
        SlotOfQItemSelection {
            raw_slot: unsafe { crate::RawSlotOfQItemSelection::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfQItemSelection<'a> {
    type Arguments = <crate::RawSlotOfQItemSelection as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `*const crate::QItemSelection` to a Rust extern function.
///
/// It's recommended to use `SlotOfQItemSelection` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```const QItemSelection&```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfQItemSelection {
    _unused: u8,
}
impl RawSlotOfQItemSelection {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(
        &mut self,
        arg0: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelection>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_custom_slot(self as *mut crate::RawSlotOfQItemSelection, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelection>>::cast_into(arg0).as_raw_ptr())
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_metaObject(self as *const crate::RawSlotOfQItemSelection);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfQItemSelection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_qt_metacall(self as *mut crate::RawSlotOfQItemSelection, arg1, arg2, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3).as_mut_raw_ptr())
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_qt_metacast(self as *mut crate::RawSlotOfQItemSelection, ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<
            extern "C" fn(*mut ::std::ffi::c_void, *const crate::QItemSelection),
        >,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_set(
            self as *mut crate::RawSlotOfQItemSelection,
            func,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_tr(
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                    .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                    .as_raw_ptr(),
                n,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_trUtf8(::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c).as_raw_ptr(), n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfQAbstractAnimation {
    type Arguments = (*mut crate::QAbstractAnimation,);
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"1custom_slot(QAbstractAnimation*)\0",
                ),
            )
        }
    }
}

/// Binds a Qt signal with arguments `::cpp_utils::MutPtr<crate::QAbstractAnimation>` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```QAbstractAnimation*```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfQAbstractAnimation<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfQAbstractAnimation>,
    func: ::std::option::Option<
        Box<Box<dyn FnMut(::cpp_utils::MutPtr<crate::QAbstractAnimation>) + 'a>>,
    >,
}

impl<'a> SlotOfQAbstractAnimation<'a> {
    /// Constructs a new object.
    pub fn new<F: FnMut(::cpp_utils::MutPtr<crate::QAbstractAnimation>) + 'a>(
        f: F,
    ) -> SlotOfQAbstractAnimation<'a> {
        let mut obj = SlotOfQAbstractAnimation::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<F: FnMut(::cpp_utils::MutPtr<crate::QAbstractAnimation>) + 'a>(&mut self, f: F) {
        extern "C" fn callback(
            data: *mut ::std::ffi::c_void,
            arg0: *mut crate::QAbstractAnimation,
        ) {
            let func = unsafe {
                &mut *(data as *mut Box<dyn FnMut(::cpp_utils::MutPtr<crate::QAbstractAnimation>)>)
            };
            func(unsafe { ::cpp_utils::MutPtr::from_raw(arg0) });
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<Box<FnMut(::cpp_utils::MutPtr<crate::QAbstractAnimation>) + 'a>> =
            Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(&mut self) -> ::cpp_utils::MutRef<crate::RawSlotOfQAbstractAnimation> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfQAbstractAnimation<'a> {
    fn default() -> Self {
        SlotOfQAbstractAnimation {
            raw_slot: unsafe { crate::RawSlotOfQAbstractAnimation::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfQAbstractAnimation<'a> {
    type Arguments = <crate::RawSlotOfQAbstractAnimation as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `*mut crate::QAbstractAnimation` to a Rust extern function.
///
/// It's recommended to use `SlotOfQAbstractAnimation` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```QAbstractAnimation*```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfQAbstractAnimation {
    _unused: u8,
}
impl RawSlotOfQAbstractAnimation {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(
        &mut self,
        arg0: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractAnimation>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr_custom_slot(self as *mut crate::RawSlotOfQAbstractAnimation, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractAnimation>>::cast_into(arg0).as_mut_raw_ptr())
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr_metaObject(self as *const crate::RawSlotOfQAbstractAnimation);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfQAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr_qt_metacall(self as *mut crate::RawSlotOfQAbstractAnimation, arg1, arg2, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3).as_mut_raw_ptr())
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr_qt_metacast(self as *mut crate::RawSlotOfQAbstractAnimation, ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<
            extern "C" fn(*mut ::std::ffi::c_void, *mut crate::QAbstractAnimation),
        >,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr_set(
            self as *mut crate::RawSlotOfQAbstractAnimation,
            func,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr_tr(
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                    .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                    .as_raw_ptr(),
                n,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr_trUtf8(::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c).as_raw_ptr(), n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfQModelIndexIntIntQModelIndexInt {
    type Arguments = (
        *const crate::QModelIndex,
        ::std::os::raw::c_int,
        ::std::os::raw::c_int,
        *const crate::QModelIndex,
        ::std::os::raw::c_int,
    );
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"1custom_slot(const QModelIndex&,int,int,const QModelIndex&,int)\0",
                ),
            )
        }
    }
}

/// Binds a Qt signal with arguments `::cpp_utils::Ref<crate::QModelIndex>,::std::os::raw::c_int,::std::os::raw::c_int,::cpp_utils::Ref<crate::QModelIndex>,::std::os::raw::c_int` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```const QModelIndex&, int, int, const QModelIndex&, int```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfQModelIndexIntIntQModelIndexInt<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfQModelIndexIntIntQModelIndexInt>,
    func: ::std::option::Option<
        Box<
            Box<
                dyn FnMut(
                        ::cpp_utils::Ref<crate::QModelIndex>,
                        ::std::os::raw::c_int,
                        ::std::os::raw::c_int,
                        ::cpp_utils::Ref<crate::QModelIndex>,
                        ::std::os::raw::c_int,
                    ) + 'a,
            >,
        >,
    >,
}

impl<'a> SlotOfQModelIndexIntIntQModelIndexInt<'a> {
    /// Constructs a new object.
    pub fn new<
        F: FnMut(
                ::cpp_utils::Ref<crate::QModelIndex>,
                ::std::os::raw::c_int,
                ::std::os::raw::c_int,
                ::cpp_utils::Ref<crate::QModelIndex>,
                ::std::os::raw::c_int,
            ) + 'a,
    >(
        f: F,
    ) -> SlotOfQModelIndexIntIntQModelIndexInt<'a> {
        let mut obj = SlotOfQModelIndexIntIntQModelIndexInt::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<
        F: FnMut(
                ::cpp_utils::Ref<crate::QModelIndex>,
                ::std::os::raw::c_int,
                ::std::os::raw::c_int,
                ::cpp_utils::Ref<crate::QModelIndex>,
                ::std::os::raw::c_int,
            ) + 'a,
    >(
        &mut self,
        f: F,
    ) {
        extern "C" fn callback(
            data: *mut ::std::ffi::c_void,
            arg0: *const crate::QModelIndex,
            arg1: ::std::os::raw::c_int,
            arg2: ::std::os::raw::c_int,
            arg3: *const crate::QModelIndex,
            arg4: ::std::os::raw::c_int,
        ) {
            let func = unsafe {
                &mut *(data as *mut Box<
                    dyn FnMut(
                        ::cpp_utils::Ref<crate::QModelIndex>,
                        ::std::os::raw::c_int,
                        ::std::os::raw::c_int,
                        ::cpp_utils::Ref<crate::QModelIndex>,
                        ::std::os::raw::c_int,
                    ),
                >)
            };
            func(
                unsafe {
                    ::cpp_utils::Ref::from_raw(arg0).expect("attempted to construct a null Ref")
                },
                arg1,
                arg2,
                unsafe {
                    ::cpp_utils::Ref::from_raw(arg3).expect("attempted to construct a null Ref")
                },
                arg4,
            );
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<
            Box<
                FnMut(
                        ::cpp_utils::Ref<crate::QModelIndex>,
                        ::std::os::raw::c_int,
                        ::std::os::raw::c_int,
                        ::cpp_utils::Ref<crate::QModelIndex>,
                        ::std::os::raw::c_int,
                    ) + 'a,
            >,
        > = Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(
        &mut self,
    ) -> ::cpp_utils::MutRef<crate::RawSlotOfQModelIndexIntIntQModelIndexInt> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfQModelIndexIntIntQModelIndexInt<'a> {
    fn default() -> Self {
        SlotOfQModelIndexIntIntQModelIndexInt {
            raw_slot: unsafe { crate::RawSlotOfQModelIndexIntIntQModelIndexInt::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfQModelIndexIntIntQModelIndexInt<'a> {
    type Arguments =
        <crate::RawSlotOfQModelIndexIntIntQModelIndexInt as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `*const crate::QModelIndex,::std::os::raw::c_int,::std::os::raw::c_int,*const crate::QModelIndex,::std::os::raw::c_int` to a Rust extern function.
///
/// It's recommended to use `SlotOfQModelIndexIntIntQModelIndexInt` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```const QModelIndex&, int, int, const QModelIndex&, int```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfQModelIndexIntIntQModelIndexInt {
    _unused: u8,
}
impl RawSlotOfQModelIndexIntIntQModelIndexInt {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(
        &mut self,
        arg0: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        arg1: ::std::os::raw::c_int,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        arg4: ::std::os::raw::c_int,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int_custom_slot(self as *mut crate::RawSlotOfQModelIndexIntIntQModelIndexInt, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(arg0).as_raw_ptr(), arg1, arg2, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(arg3).as_raw_ptr(), arg4)
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int_metaObject(self as *const crate::RawSlotOfQModelIndexIntIntQModelIndexInt);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfQModelIndexIntIntQModelIndexInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int_qt_metacall(self as *mut crate::RawSlotOfQModelIndexIntIntQModelIndexInt, arg1, arg2, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3).as_mut_raw_ptr())
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int_qt_metacast(self as *mut crate::RawSlotOfQModelIndexIntIntQModelIndexInt, ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<
            extern "C" fn(
                *mut ::std::ffi::c_void,
                *const crate::QModelIndex,
                ::std::os::raw::c_int,
                ::std::os::raw::c_int,
                *const crate::QModelIndex,
                ::std::os::raw::c_int,
            ),
        >,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int_set(self as *mut crate::RawSlotOfQModelIndexIntIntQModelIndexInt, func, ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data).as_mut_raw_ptr())
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int_tr(::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c).as_raw_ptr(), n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int_trUtf8(::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c).as_raw_ptr(), n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl crate::AsReceiver for crate::RawSlotOfOrientationInt {
    type Arguments = (crate::Orientation, ::std::os::raw::c_int);
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        unsafe {
            crate::Receiver::new(
                ::cpp_utils::Ref::from_raw_ref(self),
                ::std::ffi::CStr::from_bytes_with_nul_unchecked(
                    b"1custom_slot(Qt::Orientation,int)\0",
                ),
            )
        }
    }
}

/// Binds a Qt signal with arguments `crate::Orientation,::std::os::raw::c_int` to a Rust closure.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```Qt::Orientation, int```</span>).
///
/// Create an object using `new()` and bind your closure using `set()`. The closure will be called with the signal's arguments when the slot is invoked. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The closure will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// The slot object takes ownership of the passed closure. If `set()` is called again, previously set closure is dropped. Make sure that the slot object does not outlive objects referenced by the closure.
///
/// If `set()` was not called, slot invocation has no effect.

pub struct SlotOfOrientationInt<'a> {
    raw_slot: ::cpp_utils::CppBox<crate::RawSlotOfOrientationInt>,
    func:
        ::std::option::Option<Box<Box<dyn FnMut(crate::Orientation, ::std::os::raw::c_int) + 'a>>>,
}

impl<'a> SlotOfOrientationInt<'a> {
    /// Constructs a new object.
    pub fn new<F: FnMut(crate::Orientation, ::std::os::raw::c_int) + 'a>(
        f: F,
    ) -> SlotOfOrientationInt<'a> {
        let mut obj = SlotOfOrientationInt::default();
        obj.set(f);
        obj
    }

    /// Sets `f` as the callback closure. If `set()` is called again, previous closure is dropped.
    pub fn set<F: FnMut(crate::Orientation, ::std::os::raw::c_int) + 'a>(&mut self, f: F) {
        extern "C" fn callback(
            data: *mut ::std::ffi::c_void,
            arg0: crate::Orientation,
            arg1: ::std::os::raw::c_int,
        ) {
            let func = unsafe {
                &mut *(data as *mut Box<dyn FnMut(crate::Orientation, ::std::os::raw::c_int)>)
            };
            func(arg0, arg1);
        }

        self.clear();
        // TODO: avoid second Box
        let mut func_box: Box<Box<FnMut(crate::Orientation, ::std::os::raw::c_int) + 'a>> =
            Box::new(Box::new(f));
        unsafe {
            self.raw_slot.set(
                Some(callback),
                ::cpp_utils::MutPtr::from_raw(
                    &mut *func_box as *mut Box<_> as *mut ::std::ffi::c_void,
                ),
            );
        }
        self.func = Some(func_box);
    }

    /// Drops the previously set closure, if any. After this, slot invocation will have no effect
    /// until a new closure is set.
    pub fn clear(&mut self) {
        if self.func.is_some() {
            unsafe {
                self.raw_slot.set(None, ::cpp_utils::NullPtr);
            }
            self.func = None;
        }
    }

    pub unsafe fn as_raw(&mut self) -> ::cpp_utils::MutRef<crate::RawSlotOfOrientationInt> {
        self.raw_slot.as_mut_ref()
    }
}

impl<'a> Default for SlotOfOrientationInt<'a> {
    fn default() -> Self {
        SlotOfOrientationInt {
            raw_slot: unsafe { crate::RawSlotOfOrientationInt::new() },
            func: None,
        }
    }
}

impl<'a, 'b: 'a> crate::AsReceiver for &'b SlotOfOrientationInt<'a> {
    type Arguments = <crate::RawSlotOfOrientationInt as crate::AsReceiver>::Arguments;
    fn as_receiver(&self) -> crate::Receiver<Self::Arguments> {
        self.raw_slot.as_receiver()
    }
}

/// Binds a Qt signal with arguments `crate::Orientation,::std::os::raw::c_int` to a Rust extern function.
///
/// It's recommended to use `SlotOfOrientationInt` instead because it provides a more high-level API.
///
/// Corresponding C++ argument types: (<span style='color: green;'>```Qt::Orientation, int```</span>).
///
/// Create an object using `new()` and bind your function and payload using `set()`. The function will receive the payload as its first arguments, and the rest of arguments will be values passed through the Qt connection system. Use `connect()` method of a `qt_core::Signal` object to connect the signal to this slot. The callback function will be executed each time the slot is invoked until source signals are disconnected or the slot object is destroyed.
///
/// If `set()` was not called, slot invocation has no effect.
#[repr(C)]
pub struct RawSlotOfOrientationInt {
    _unused: u8,
}
impl RawSlotOfOrientationInt {
    /// Calls the slot directly, invoking the assigned handler (if any).
    pub unsafe fn custom_slot(&mut self, arg0: crate::Orientation, arg1: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_custom_slot(
            self as *mut crate::RawSlotOfOrientationInt,
            arg0,
            arg1,
        )
    }

    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_metaObject(self as *const crate::RawSlotOfOrientationInt);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Creates a new object.
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::RawSlotOfOrientationInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_qt_metacall(
            self as *mut crate::RawSlotOfOrientationInt,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_qt_metacast(self as *mut crate::RawSlotOfOrientationInt, ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1).as_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Assigns `func` as the signal handler.
    ///
    /// `func` will be called each time a connected signal is emitted. Any previously assigned function will be deregistered. Passing `None` will deregister the handler without setting a new one.
    pub unsafe fn set(
        &mut self,
        func: ::std::option::Option<
            extern "C" fn(*mut ::std::ffi::c_void, crate::Orientation, ::std::os::raw::c_int),
        >,
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_set(
            self as *mut crate::RawSlotOfOrientationInt,
            func,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(data)
                .as_mut_raw_ptr(),
        )
    }

    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_tr(
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                    .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                    .as_raw_ptr(),
                n,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_trUtf8(
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                    .as_raw_ptr(),
                ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                    .as_raw_ptr(),
                n,
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> class provides a unified view on UTF-16 strings with a read-only subset of the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> API.</p>
///
/// C++ class: <span style='color: green;'>```QStringView```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qstringview.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> class provides a unified view on UTF-16 strings with a read-only subset of the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> API.</p>
/// <p>A <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> references a contiguous portion of a UTF-16 string it does not own. It acts as an interface type to all kinds of UTF-16 string, without the need to construct a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> first.</p>
/// <p>The UTF-16 string may be represented as an array (or an array-compatible data-structure such as <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, std::basic_string, etc.) of <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>, <code>ushort</code>, <code>char16_t</code> (on compilers that support C++11 Unicode strings) or (on platforms, such as Windows, where it is a 16-bit type) <code>wchar_t</code>.</p>
/// <p><a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> is designed as an interface type; its main use-case is as a function parameter type. When QStringViews are used as automatic variables or data members, care must be taken to ensure that the referenced string data (for example, owned by a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>) outlives the <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> on all code paths, lest the string view ends up referencing deleted data.</p>
/// <p>When used as an interface type, <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> allows a single function to accept a wide variety of UTF-16 string data sources. One function accepting <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> thus replaces three function overloads (taking <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, <a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a>, and <code>(const QChar*, int)</code>), while at the same time enabling even more string data sources to be passed to the function, such as <code>u"Hello World"</code>, a <code>char16_t</code> string literal.</p>
/// <p>QStringViews should be passed by value, not by reference-to-const:</p>
/// <pre class="cpp">
///
///   <span class="type">void</span> myfun1(<span class="type"><a href="http://doc.qt.io/qt-5/qstringview.html#QStringView">QStringView</a></span> sv);        <span class="comment">// preferred</span>
///   <span class="type">void</span> myfun2(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstringview.html#QStringView">QStringView</a></span> <span class="operator">&amp;</span>sv); <span class="comment">// compiles and works, but slower</span>
///
/// </pre>
/// <p>If you want to give your users maximum freedom in what strings they can pass to your function, accompany the <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> overload with overloads for</p>
/// <ul>
/// <li><i>QChar</i>: this overload can delegate to the <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> version:<pre class="cpp">
///
///   <span class="type">void</span> fun(<span class="type"><a href="http://doc.qt.io/qt-5/qchar.html">QChar</a></span> ch) { fun(<span class="type"><a href="http://doc.qt.io/qt-5/qstringview.html#QStringView">QStringView</a></span>(<span class="operator">&amp;</span>ch<span class="operator">,</span> <span class="number">1</span>)); }
///
/// </pre>
/// <p>even though, for technical reasons, <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> cannot provide a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> constructor by itself.</p>
/// </li>
/// <li><i>QString</i>: if you store an unmodified copy of the string and thus would like to take advantage of <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>'s implicit sharing.</li>
/// <li><a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>: if you can implement the function without converting the <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> to UTF-16 first; users expect a function overloaded on <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> to perform strictly less memory allocations than the semantically equivalent call of the <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> version, involving construction of a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> from the <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>.</li>
/// </ul>
/// <p><a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> can also be used as the return value of a function. If you call a function returning <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a>, take extra care to not keep the <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> around longer than the function promises to keep the referenced string data alive. If in doubt, obtain a strong reference to the data by calling <a href="http://doc.qt.io/qt-5/qstringview.html#toString">toString</a>() to convert the <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> into a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
/// <p><a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> is a <i>Literal Type</i>, but since it stores data as <code>char16_t</code>, iteration is not <code>constexpr</code> (casts from <code>const char16_t*</code> to <code>const QChar*</code>, which is not allowed in <code>constexpr</code> functions). You can use an indexed loop and/or <a href="http://doc.qt.io/qt-5/qstringview.html#utf16">utf16</a>() in <code>constexpr</code> contexts instead.</p>
/// <p><b>Note: </b>We strongly discourage the use of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a>&gt;, because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is a very inefficient container for QStringViews (it would heap-allocate every element). Use <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> (or std::vector) to hold QStringViews instead.</p></div>
#[repr(C)]
pub struct QStringView {
    _unused: u8,
}
impl QStringView {
    /// <p>Returns the character at position <i>n</i> in this string view.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QStringView::at(long long n) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the character at position <i>n</i> in this string view.</p>
    /// <p>The behavior is undefined if <i>n</i> is negative or not less than <a href="http://doc.qt.io/qt-5/qstringview.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#operator-5b-5d">operator[]</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#front">front</a>(), and <a href="http://doc.qt.io/qt-5/qstringview.html#back">back</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn at(&self, n: ::std::os::raw::c_longlong) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringView_at(self as *const crate::QStringView, n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the last character in the string. Same as <a href="http://doc.qt.io/qt-5/qstringview.html#last">last</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QStringView::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the last character in the string. Same as <a href="http://doc.qt.io/qt-5/qstringview.html#last">last</a>().</p>
    /// <p>This function is provided for STL compatibility.</p>
    /// <p><b>Warning:</b> Calling this function on an empty string view constitutes undefined behavior.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#front">front</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#first">first</a>(), and <a href="http://doc.qt.io/qt-5/qstringview.html#last">last</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn back(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringView_back(self as *const crate::QStringView);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Same as <a href="http://doc.qt.io/qt-5/qstringview.html#begin">begin</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QChar* QStringView::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as <a href="http://doc.qt.io/qt-5/qstringview.html#begin">begin</a>().</p>
    /// <p>This function is provided for STL compatibility.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#cend">cend</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#begin">begin</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#crbegin">crbegin</a>(), and <a href="http://doc.qt.io/qt-5/qstringview.html#data">data</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn cbegin(&self) -> ::cpp_utils::Ptr<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringView_cbegin(self as *const crate::QStringView);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Same as <a href="http://doc.qt.io/qt-5/qstringview.html#end">end</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QChar* QStringView::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as <a href="http://doc.qt.io/qt-5/qstringview.html#end">end</a>().</p>
    /// <p>This function is provided for STL compatibility.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#cbegin">cbegin</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#end">end</a>(), and <a href="http://doc.qt.io/qt-5/qstringview.html#crend">crend</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn cend(&self) -> ::cpp_utils::Ptr<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringView_cend(self as *const crate::QStringView);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Truncates this string view by <i>length</i> characters.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QStringView::chop(long long n)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#chop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Truncates this string view by <i>length</i> characters.</p>
    /// <p>Same as <code>*this = left(size() - length)</code>.</p>
    /// <p><b>Note: </b>The behavior is undefined when <i>length</i> &lt; 0 or <i>length</i> &gt; <a href="http://doc.qt.io/qt-5/qstringview.html#size">size</a>().</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#mid">mid</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#left">left</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#right">right</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#chopped">chopped</a>(), and <a href="http://doc.qt.io/qt-5/qstringview.html#truncate">truncate</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn chop(&mut self, n: ::std::os::raw::c_longlong) {
        crate::__ffi::ctr_qt_core_ffi_QStringView_chop(self as *mut crate::QStringView, n)
    }

    /// <p>Returns the substring of length <a href="http://doc.qt.io/qt-5/qstringview.html#size">size</a>() - <i>length</i> starting at the beginning of this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringView QStringView::chopped(long long n) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#chopped">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the substring of length <a href="http://doc.qt.io/qt-5/qstringview.html#size">size</a>() - <i>length</i> starting at the beginning of this object.</p>
    /// <p>Same as <code>left(size() - length)</code>.</p>
    /// <p><b>Note: </b>The behavior is undefined when <i>length</i> &lt; 0 or <i>length</i> &gt; <a href="http://doc.qt.io/qt-5/qstringview.html#size">size</a>().</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#mid">mid</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#left">left</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#right">right</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#chop">chop</a>(), and <a href="http://doc.qt.io/qt-5/qstringview.html#truncate">truncate</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn chopped(
        &self,
        n: ::std::os::raw::c_longlong,
    ) -> ::cpp_utils::CppBox<crate::QStringView> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringView_chopped(self as *const crate::QStringView, n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Compares this string-view with the <i>other</i> string-view and returns an integer less than, equal to, or greater than zero if this string-view is less than, equal to, or greater than the other string-view.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringView::compare(QStringView other, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#compare">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares this string-view with the <i>other</i> string-view and returns an integer less than, equal to, or greater than zero if this string-view is less than, equal to, or greater than the other string-view.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a>, the comparison is case sensitive; otherwise the comparison is case insensitive.</p>
    /// <p>This function was introduced in  Qt 5.12.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">operator==</a>(), <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt">operator&lt;</a>(), and <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt">operator&gt;</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn compare_2a(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
        cs: crate::CaseSensitivity,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringView_compare(
            self as *const crate::QStringView,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(other)
                .as_raw_ptr(),
            cs,
        )
    }

    /// <p>Compares this string-view with the <i>other</i> string-view and returns an integer less than, equal to, or greater than zero if this string-view is less than, equal to, or greater than the other string-view.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringView::compare(QStringView other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#compare">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares this string-view with the <i>other</i> string-view and returns an integer less than, equal to, or greater than zero if this string-view is less than, equal to, or greater than the other string-view.</p>
    /// <p>If <i>cs</i> is <a href="http://doc.qt.io/qt-5/qt.html#CaseSensitivity-enum">Qt::CaseSensitive</a>, the comparison is case sensitive; otherwise the comparison is case insensitive.</p>
    /// <p>This function was introduced in  Qt 5.12.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">operator==</a>(), <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt">operator&lt;</a>(), and <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt">operator&gt;</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn compare_1a(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringView_compare1(
            self as *const crate::QStringView,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> class provides a unified view on UTF-16 strings with a read-only subset of the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> API.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringView& QStringView::operator=(const QStringView& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> class provides a unified view on UTF-16 strings with a read-only subset of the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> API.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> references a contiguous portion of a UTF-16 string it does not own. It acts as an interface type to all kinds of UTF-16 string, without the need to construct a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> first.</p>
    /// <p>The UTF-16 string may be represented as an array (or an array-compatible data-structure such as <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, std::basic_string, etc.) of <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>, <code>ushort</code>, <code>char16_t</code> (on compilers that support C++11 Unicode strings) or (on platforms, such as Windows, where it is a 16-bit type) <code>wchar_t</code>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> is designed as an interface type; its main use-case is as a function parameter type. When QStringViews are used as automatic variables or data members, care must be taken to ensure that the referenced string data (for example, owned by a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>) outlives the <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> on all code paths, lest the string view ends up referencing deleted data.</p>
    /// <p>When used as an interface type, <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> allows a single function to accept a wide variety of UTF-16 string data sources. One function accepting <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> thus replaces three function overloads (taking <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, <a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a>, and <code>(const QChar*, int)</code>), while at the same time enabling even more string data sources to be passed to the function, such as <code>u"Hello World"</code>, a <code>char16_t</code> string literal.</p>
    /// <p>QStringViews should be passed by value, not by reference-to-const:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">void</span> myfun1(<span class="type"><a href="http://doc.qt.io/qt-5/qstringview.html#QStringView">QStringView</a></span> sv);        <span class="comment">// preferred</span>
    ///   <span class="type">void</span> myfun2(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstringview.html#QStringView">QStringView</a></span> <span class="operator">&amp;</span>sv); <span class="comment">// compiles and works, but slower</span>
    ///
    /// </pre>
    /// <p>If you want to give your users maximum freedom in what strings they can pass to your function, accompany the <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> overload with overloads for</p>
    /// <ul>
    /// <li><i>QChar</i>: this overload can delegate to the <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> version:<pre class="cpp">
    ///
    ///   <span class="type">void</span> fun(<span class="type"><a href="http://doc.qt.io/qt-5/qchar.html">QChar</a></span> ch) { fun(<span class="type"><a href="http://doc.qt.io/qt-5/qstringview.html#QStringView">QStringView</a></span>(<span class="operator">&amp;</span>ch<span class="operator">,</span> <span class="number">1</span>)); }
    ///
    /// </pre>
    /// <p>even though, for technical reasons, <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> cannot provide a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> constructor by itself.</p>
    /// </li>
    /// <li><i>QString</i>: if you store an unmodified copy of the string and thus would like to take advantage of <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>'s implicit sharing.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>: if you can implement the function without converting the <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> to UTF-16 first; users expect a function overloaded on <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> to perform strictly less memory allocations than the semantically equivalent call of the <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> version, involving construction of a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> from the <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>.</li>
    /// </ul>
    /// <p><a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> can also be used as the return value of a function. If you call a function returning <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a>, take extra care to not keep the <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> around longer than the function promises to keep the referenced string data alive. If in doubt, obtain a strong reference to the data by calling <a href="http://doc.qt.io/qt-5/qstringview.html#toString">toString</a>() to convert the <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> into a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> is a <i>Literal Type</i>, but since it stores data as <code>char16_t</code>, iteration is not <code>constexpr</code> (casts from <code>const char16_t*</code> to <code>const QChar*</code>, which is not allowed in <code>constexpr</code> functions). You can use an indexed loop and/or <a href="http://doc.qt.io/qt-5/qstringview.html#utf16">utf16</a>() in <code>constexpr</code> contexts instead.</p>
    /// <p><b>Note: </b>We strongly discourage the use of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a>&gt;, because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is a very inefficient container for QStringViews (it would heap-allocate every element). Use <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> (or std::vector) to hold QStringViews instead.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::cpp_utils::MutRef<crate::QStringView> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringView_operator_(
            self as *mut crate::QStringView,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const pointer to the first character in the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QChar* QStringView::data() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const pointer to the first character in the string.</p>
    /// <p><b>Note: </b>The character array represented by the return value is <i>not</i> null-terminated.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#begin">begin</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#end">end</a>(), and <a href="http://doc.qt.io/qt-5/qstringview.html#utf16">utf16</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn data(&self) -> ::cpp_utils::Ptr<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringView_data(self as *const crate::QStringView);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns whether this string view is empty - that is, whether <code>size() == 0</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringView::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns whether this string view is empty - that is, whether <code>size() == 0</code>.</p>
    /// <p>This function is provided for STL compatibility.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#isEmpty">isEmpty</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#isNull">isNull</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#size">size</a>(), and <a href="http://doc.qt.io/qt-5/qstringview.html#length">length</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringView_empty(self as *const crate::QStringView)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QStringView::endsWith(QStringView s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn ends_with_q_string_view_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringView_endsWith(
            self as *const crate::QStringView,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
            cs,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QStringView::endsWith(QLatin1String s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn ends_with_q_latin1_string_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringView_endsWith1(
            self as *const crate::QStringView,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
            cs,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QStringView::endsWith(QChar c) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn ends_with_q_char(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringView_endsWith2(
            self as *const crate::QStringView,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QStringView::endsWith(QChar c, Qt::CaseSensitivity cs) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn ends_with_q_char_case_sensitivity(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringView_endsWith3(
            self as *const crate::QStringView,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
            cs,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QStringView::endsWith(QStringView s) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn ends_with_q_string_view(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringView_endsWith4(
            self as *const crate::QStringView,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QStringView::endsWith(QLatin1String s) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn ends_with_q_latin1_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringView_endsWith5(
            self as *const crate::QStringView,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the first character in the string. Same as <a href="http://doc.qt.io/qt-5/qstringview.html#front">front</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QStringView::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the first character in the string. Same as <a href="http://doc.qt.io/qt-5/qstringview.html#front">front</a>().</p>
    /// <p>This function is provided for compatibility with other Qt containers.</p>
    /// <p><b>Warning:</b> Calling this function on an empty string view constitutes undefined behavior.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#front">front</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#back">back</a>(), and <a href="http://doc.qt.io/qt-5/qstringview.html#last">last</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn first(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringView_first(self as *const crate::QStringView);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the first character in the string. Same as <a href="http://doc.qt.io/qt-5/qstringview.html#first">first</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QStringView::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the first character in the string. Same as <a href="http://doc.qt.io/qt-5/qstringview.html#first">first</a>().</p>
    /// <p>This function is provided for STL compatibility.</p>
    /// <p><b>Warning:</b> Calling this function on an empty string view constitutes undefined behavior.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#back">back</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#first">first</a>(), and <a href="http://doc.qt.io/qt-5/qstringview.html#last">last</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn front(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringView_front(self as *const crate::QStringView);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the character at position <i>n</i> in this string view.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QStringView::operator[](long long n) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the character at position <i>n</i> in this string view.</p>
    /// <p>The behavior is undefined if <i>n</i> is negative or not less than <a href="http://doc.qt.io/qt-5/qstringview.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#at">at</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#front">front</a>(), and <a href="http://doc.qt.io/qt-5/qstringview.html#back">back</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn index(&self, n: ::std::os::raw::c_longlong) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringView_operator__(
            self as *const crate::QStringView,
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns whether this string view is empty - that is, whether <code>size() == 0</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringView::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns whether this string view is empty - that is, whether <code>size() == 0</code>.</p>
    /// <p>This function is provided for compatibility with other Qt containers.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#empty">empty</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#isNull">isNull</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#size">size</a>(), and <a href="http://doc.qt.io/qt-5/qstringview.html#length">length</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringView_isEmpty(self as *const crate::QStringView)
    }

    /// <p>Returns whether this string view is null - that is, whether <code>data() == nullptr</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringView::isNull() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#isNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns whether this string view is null - that is, whether <code>data() == nullptr</code>.</p>
    /// <p>This functions is provided for compatibility with other Qt containers.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#empty">empty</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#isEmpty">isEmpty</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#size">size</a>(), and <a href="http://doc.qt.io/qt-5/qstringview.html#length">length</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringView_isNull(self as *const crate::QStringView)
    }

    /// <p>Returns <code>true</code> if the string is read right to left.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringView::isRightToLeft() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#isRightToLeft">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the string is read right to left.</p>
    /// <p>This function was introduced in  Qt 5.11.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#isRightToLeft">QString::isRightToLeft</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn is_right_to_left(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringView_isRightToLeft(self as *const crate::QStringView)
    }

    /// <p>Returns the last character in the string. Same as <a href="http://doc.qt.io/qt-5/qstringview.html#back">back</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar QStringView::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the last character in the string. Same as <a href="http://doc.qt.io/qt-5/qstringview.html#back">back</a>().</p>
    /// <p>This function is provided for compatibility with other Qt containers.</p>
    /// <p><b>Warning:</b> Calling this function on an empty string view constitutes undefined behavior.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#back">back</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#front">front</a>(), and <a href="http://doc.qt.io/qt-5/qstringview.html#first">first</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn last(&self) -> ::cpp_utils::CppBox<crate::QChar> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringView_last(self as *const crate::QStringView);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the substring of length <i>length</i> starting at position 0 in this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringView QStringView::left(long long n) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#left">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the substring of length <i>length</i> starting at position 0 in this object.</p>
    /// <p><b>Note: </b>The behavior is undefined when <i>length</i> &lt; 0 or <i>length</i> &gt; <a href="http://doc.qt.io/qt-5/qstringview.html#size">size</a>().</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#mid">mid</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#right">right</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#chopped">chopped</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#chop">chop</a>(), and <a href="http://doc.qt.io/qt-5/qstringview.html#truncate">truncate</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn left(
        &self,
        n: ::std::os::raw::c_longlong,
    ) -> ::cpp_utils::CppBox<crate::QStringView> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringView_left(self as *const crate::QStringView, n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Same as <a href="http://doc.qt.io/qt-5/qstringview.html#size">size</a>(), except returns the result as an <code>int</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QStringView::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as <a href="http://doc.qt.io/qt-5/qstringview.html#size">size</a>(), except returns the result as an <code>int</code>.</p>
    /// <p>This function is provided for compatibility with other Qt containers.</p>
    /// <p><b>Warning:</b> <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> can represent strings with more than 2<sup>31</sup> characters. Calling this function on a string view for which <a href="http://doc.qt.io/qt-5/qstringview.html#size">size</a>() returns a value greater than <code>INT_MAX</code> constitutes undefined behavior.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#empty">empty</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#isEmpty">isEmpty</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#isNull">isNull</a>(), and <a href="http://doc.qt.io/qt-5/qstringview.html#size">size</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QStringView_length(self as *const crate::QStringView)
    }

    /// <p>Returns the substring starting at position <i>start</i> in this object, and extending to the end of the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringView QStringView::mid(long long pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the substring starting at position <i>start</i> in this object, and extending to the end of the string.</p>
    /// <p><b>Note: </b>The behavior is undefined when <i>start</i> &lt; 0 or <i>start</i> &gt; <a href="http://doc.qt.io/qt-5/qstringview.html#size">size</a>().</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#left">left</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#right">right</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#chopped">chopped</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#chop">chop</a>(), and <a href="http://doc.qt.io/qt-5/qstringview.html#truncate">truncate</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_longlong,
    ) -> ::cpp_utils::CppBox<crate::QStringView> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringView_mid(self as *const crate::QStringView, pos);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringView QStringView::mid(long long pos, long long n) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#mid-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the substring of length <i>length</i> starting at position <i>start</i> in this object.</p>
    /// <p><b>Note: </b>The behavior is undefined when <i>start</i> &lt; 0, <i>length</i> &lt; 0, or <i>start</i> + <i>length</i> &gt; <a href="http://doc.qt.io/qt-5/qstringview.html#size">size</a>().</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#left">left</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#right">right</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#chopped">chopped</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#chop">chop</a>(), and <a href="http://doc.qt.io/qt-5/qstringview.html#truncate">truncate</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_longlong,
        n: ::std::os::raw::c_longlong,
    ) -> ::cpp_utils::CppBox<crate::QStringView> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringView_mid1(
            self as *const crate::QStringView,
            pos,
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a null string view.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStringView::QStringView()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#QStringView">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a null string view.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#isNull">isNull</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QStringView> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringView_QStringView();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> class provides a unified view on UTF-16 strings with a read-only subset of the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> API.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QStringView::QStringView(const QStringView& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> class provides a unified view on UTF-16 strings with a read-only subset of the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> API.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> references a contiguous portion of a UTF-16 string it does not own. It acts as an interface type to all kinds of UTF-16 string, without the need to construct a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> first.</p>
    /// <p>The UTF-16 string may be represented as an array (or an array-compatible data-structure such as <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, std::basic_string, etc.) of <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>, <code>ushort</code>, <code>char16_t</code> (on compilers that support C++11 Unicode strings) or (on platforms, such as Windows, where it is a 16-bit type) <code>wchar_t</code>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> is designed as an interface type; its main use-case is as a function parameter type. When QStringViews are used as automatic variables or data members, care must be taken to ensure that the referenced string data (for example, owned by a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>) outlives the <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> on all code paths, lest the string view ends up referencing deleted data.</p>
    /// <p>When used as an interface type, <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> allows a single function to accept a wide variety of UTF-16 string data sources. One function accepting <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> thus replaces three function overloads (taking <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, <a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a>, and <code>(const QChar*, int)</code>), while at the same time enabling even more string data sources to be passed to the function, such as <code>u"Hello World"</code>, a <code>char16_t</code> string literal.</p>
    /// <p>QStringViews should be passed by value, not by reference-to-const:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">void</span> myfun1(<span class="type"><a href="http://doc.qt.io/qt-5/qstringview.html#QStringView">QStringView</a></span> sv);        <span class="comment">// preferred</span>
    ///   <span class="type">void</span> myfun2(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstringview.html#QStringView">QStringView</a></span> <span class="operator">&amp;</span>sv); <span class="comment">// compiles and works, but slower</span>
    ///
    /// </pre>
    /// <p>If you want to give your users maximum freedom in what strings they can pass to your function, accompany the <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> overload with overloads for</p>
    /// <ul>
    /// <li><i>QChar</i>: this overload can delegate to the <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> version:<pre class="cpp">
    ///
    ///   <span class="type">void</span> fun(<span class="type"><a href="http://doc.qt.io/qt-5/qchar.html">QChar</a></span> ch) { fun(<span class="type"><a href="http://doc.qt.io/qt-5/qstringview.html#QStringView">QStringView</a></span>(<span class="operator">&amp;</span>ch<span class="operator">,</span> <span class="number">1</span>)); }
    ///
    /// </pre>
    /// <p>even though, for technical reasons, <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> cannot provide a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> constructor by itself.</p>
    /// </li>
    /// <li><i>QString</i>: if you store an unmodified copy of the string and thus would like to take advantage of <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>'s implicit sharing.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>: if you can implement the function without converting the <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> to UTF-16 first; users expect a function overloaded on <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> to perform strictly less memory allocations than the semantically equivalent call of the <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> version, involving construction of a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> from the <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>.</li>
    /// </ul>
    /// <p><a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> can also be used as the return value of a function. If you call a function returning <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a>, take extra care to not keep the <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> around longer than the function promises to keep the referenced string data alive. If in doubt, obtain a strong reference to the data by calling <a href="http://doc.qt.io/qt-5/qstringview.html#toString">toString</a>() to convert the <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> into a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> is a <i>Literal Type</i>, but since it stores data as <code>char16_t</code>, iteration is not <code>constexpr</code> (casts from <code>const char16_t*</code> to <code>const QChar*</code>, which is not allowed in <code>constexpr</code> functions). You can use an indexed loop and/or <a href="http://doc.qt.io/qt-5/qstringview.html#utf16">utf16</a>() in <code>constexpr</code> contexts instead.</p>
    /// <p><b>Note: </b>We strongly discourage the use of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a>&gt;, because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is a very inefficient container for QStringViews (it would heap-allocate every element). Use <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> (or std::vector) to hold QStringViews instead.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> ::cpp_utils::CppBox<crate::QStringView> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringView_QStringView1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the substring of length <i>length</i> starting at position <a href="http://doc.qt.io/qt-5/qstringview.html#size">size</a>() - <i>length</i> in this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringView QStringView::right(long long n) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#right">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the substring of length <i>length</i> starting at position <a href="http://doc.qt.io/qt-5/qstringview.html#size">size</a>() - <i>length</i> in this object.</p>
    /// <p><b>Note: </b>The behavior is undefined when <i>length</i> &lt; 0 or <i>length</i> &gt; <a href="http://doc.qt.io/qt-5/qstringview.html#size">size</a>().</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#mid">mid</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#left">left</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#chopped">chopped</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#chop">chop</a>(), and <a href="http://doc.qt.io/qt-5/qstringview.html#truncate">truncate</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn right(
        &self,
        n: ::std::os::raw::c_longlong,
    ) -> ::cpp_utils::CppBox<crate::QStringView> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringView_right(self as *const crate::QStringView, n);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the size of this string view, in UTF-16 code points (that is, surrogate pairs count as two for the purposes of this function, the same as in <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and <a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```long long QStringView::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the size of this string view, in UTF-16 code points (that is, surrogate pairs count as two for the purposes of this function, the same as in <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and <a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#empty">empty</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#isEmpty">isEmpty</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#isNull">isNull</a>(), and <a href="http://doc.qt.io/qt-5/qstringview.html#length">length</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn size(&self) -> ::std::os::raw::c_longlong {
        crate::__ffi::ctr_qt_core_ffi_QStringView_size(self as *const crate::QStringView)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QStringView::startsWith(QStringView s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn starts_with_q_string_view_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringView_startsWith(
            self as *const crate::QStringView,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
            cs,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QStringView::startsWith(QLatin1String s, Qt::CaseSensitivity cs = …) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn starts_with_q_latin1_string_case_sensitivity(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringView_startsWith1(
            self as *const crate::QStringView,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
            cs,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QStringView::startsWith(QChar c) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn starts_with_q_char(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringView_startsWith2(
            self as *const crate::QStringView,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QStringView::startsWith(QChar c, Qt::CaseSensitivity cs) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn starts_with_q_char_case_sensitivity(
        &self,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
        cs: crate::CaseSensitivity,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringView_startsWith3(
            self as *const crate::QStringView,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(c).as_raw_ptr(),
            cs,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QStringView::startsWith(QStringView s) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn starts_with_q_string_view(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringView_startsWith4(
            self as *const crate::QStringView,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QStringView::startsWith(QLatin1String s) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn starts_with_q_latin1_string(
        &self,
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QStringView_startsWith5(
            self as *const crate::QStringView,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a Latin-1 representation of the string as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QStringView::toLatin1() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#toLatin1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a Latin-1 representation of the string as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    /// <p>The behavior is undefined if the string contains non-Latin1 characters.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#toUtf8">toUtf8</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#toLocal8Bit">toLocal8Bit</a>(), and <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_latin1(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringView_toLatin1(self as *const crate::QStringView);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a local 8-bit representation of the string as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QStringView::toLocal8Bit() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#toLocal8Bit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a local 8-bit representation of the string as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForLocale">QTextCodec::codecForLocale</a>() is used to perform the conversion from Unicode. If the locale's encoding could not be determined, this function does the same as <a href="http://doc.qt.io/qt-5/qstringview.html#toLatin1">toLatin1</a>().</p>
    /// <p>The behavior is undefined if the string contains characters not supported by the locale's 8-bit encoding.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#toLatin1">toLatin1</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#toUtf8">toUtf8</a>(), and <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_local8_bit(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QStringView_toLocal8Bit(
            self as *const crate::QStringView,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a deep copy of this string view's data as a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QStringView::toString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#toString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a deep copy of this string view's data as a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p>The return value will be the null <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> if and only if this string view is null.</p>
    /// <p><b>Warning:</b> <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> can store strings with more than 2<sup>30</sup> characters while <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> cannot. Calling this function on a string view for which <a href="http://doc.qt.io/qt-5/qstringview.html#size">size</a>() returns a value greater than <code>INT_MAX / 2</code> constitutes undefined behavior.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_string(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringView_toString(self as *const crate::QStringView);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a UCS-4/UTF-32 representation of the string as a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;uint&gt;.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<unsigned int> QStringView::toUcs4() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#toUcs4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a UCS-4/UTF-32 representation of the string as a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;uint&gt;.</p>
    /// <p>UCS-4 is a Unicode codec and therefore it is lossless. All characters from this string will be encoded in UCS-4. Any invalid sequence of code units in this string is replaced by the Unicode replacement character (<a href="http://doc.qt.io/qt-5/qchar.html#SpecialCharacter-enum">QChar::ReplacementCharacter</a>, which corresponds to <code>U+FFFD</code>).</p>
    /// <p>The returned vector is not 0-terminated.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#toUtf8">toUtf8</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#toLatin1">toLatin1</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#toLocal8Bit">toLocal8Bit</a>(), and <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_ucs4(&self) -> ::cpp_utils::CppBox<crate::QVectorOfUint> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringView_toUcs4(self as *const crate::QStringView);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a UTF-8 representation of the string as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QStringView::toUtf8() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#toUtf8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a UTF-8 representation of the string as a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    /// <p>UTF-8 is a Unicode codec and can represent all characters in a Unicode string like <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#toLatin1">toLatin1</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#toLocal8Bit">toLocal8Bit</a>(), and <a href="http://doc.qt.io/qt-5/qtextcodec.html">QTextCodec</a>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn to_utf8(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringView_toUtf8(self as *const crate::QStringView);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Strips leading and trailing whitespace and returns the result.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringView QStringView::trimmed() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#trimmed">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Strips leading and trailing whitespace and returns the result.</p>
    /// <p>Whitespace means any character for which <a href="http://doc.qt.io/qt-5/qchar.html#isSpace">QChar::isSpace</a>() returns <code>true</code>. This includes the ASCII characters '\t', '\n', '\v', '\f', '\r', and ' '.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn trimmed(&self) -> ::cpp_utils::CppBox<crate::QStringView> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QStringView_trimmed(self as *const crate::QStringView);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Truncates this string view to length <i>length</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QStringView::truncate(long long n)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#truncate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Truncates this string view to length <i>length</i>.</p>
    /// <p>Same as <code>*this = left(length)</code>.</p>
    /// <p><b>Note: </b>The behavior is undefined when <i>length</i> &lt; 0 or <i>length</i> &gt; <a href="http://doc.qt.io/qt-5/qstringview.html#size">size</a>().</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#mid">mid</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#left">left</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#right">right</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#chopped">chopped</a>(), and <a href="http://doc.qt.io/qt-5/qstringview.html#chop">chop</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn truncate(&mut self, n: ::std::os::raw::c_longlong) {
        crate::__ffi::ctr_qt_core_ffi_QStringView_truncate(self as *mut crate::QStringView, n)
    }
}

pub mod q_random_generator {
    //! C++ type: <span style='color: green;'>```QRandomGenerator```</span>

    /// C++ enum: <span style='color: green;'>```QRandomGenerator::System```</span>.
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct System(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for System {
        fn from(value: ::std::os::raw::c_int) -> Self {
            System(value)
        }
    }

    impl From<System> for ::std::os::raw::c_int {
        fn from(value: System) -> Self {
            value.0
        }
    }

    impl System {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> class allows one to obtain random values from a high-quality Random Number Generator.</p>
///
/// C++ class: <span style='color: green;'>```QRandomGenerator```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> class allows one to obtain random values from a high-quality Random Number Generator.</p>
/// <p><a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> may be used to generate random values from a high-quality random number generator. Like the C++ random engines, <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> can be seeded with user-provided values through the constructor. When seeded, the sequence of numbers generated by this class is deterministic. That is to say, given the same seed data, <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> will generate the same sequence of numbers. But given different seeds, the results should be considerably different.</p>
/// <p><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#securelySeeded">QRandomGenerator::securelySeeded</a>() can be used to create a <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> that is securely seeded with <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#system">QRandomGenerator::system</a>(), meaning that the sequence of numbers it generates cannot be easily predicted. Additionally, <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#global">QRandomGenerator::global</a>() returns a global instance of <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> that Qt will ensure to be securely seeded. This object is thread-safe, may be shared for most uses, and is always seeded from <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#system">QRandomGenerator::system</a>()</p>
/// <p><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#system">QRandomGenerator::system</a>() may be used to access the system's cryptographically-safe random generator. On Unix systems, it's equivalent to reading from <code>/dev/urandom</code> or the <code>getrandom()</code> or <code>getentropy()</code> system calls.</p>
/// <p>The class can generate 32-bit or 64-bit quantities, or fill an array of those. The most common way of generating new values is to call the <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate">generate</a>(), <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate64">generate64</a>() or <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#fillRange">fillRange</a>() functions. One would use it as:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#quint32-typedef">quint32</a></span> value <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#QRandomGenerator">QRandomGenerator</a></span><span class="operator">::</span>global()<span class="operator">-</span><span class="operator">&gt;</span>generate();
///
/// </pre>
/// <p>Additionally, it provides a floating-point function <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generateDouble">generateDouble</a>() that returns a number in the range [0, 1) (that is, inclusive of zero and exclusive of 1). There's also a set of convenience functions that facilitate obtaining a random number in a bounded, integral range.</p>
/// <a name="seeding-and-determinism"></a></div>
#[repr(C)]
pub struct QRandomGenerator {
    _unused: u8,
}
impl QRandomGenerator {
    /// <p>Generates one random double in the range between 0 (inclusive) and <i>highest</i> (exclusive). This function is equivalent to and is implemented as:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QRandomGenerator::bounded(double highest)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#bounded">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Generates one random double in the range between 0 (inclusive) and <i>highest</i> (exclusive). This function is equivalent to and is implemented as:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">return</span> generateDouble() <span class="operator">*</span> highest;
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generateDouble">generateDouble</a>() and bounded().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn bounded_double(
        &mut self,
        highest: ::std::os::raw::c_double,
    ) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QRandomGenerator_bounded(
            self as *mut crate::QRandomGenerator,
            highest,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```quint32 QRandomGenerator::bounded(quint32 highest)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#bounded-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Generates one random 32-bit quantity in the range between 0 (inclusive) and <i>highest</i> (exclusive). The same result may also be obtained by using <a href="http://en.cppreference.com/w/cpp/numeric/random/uniform_int_distribution"><code>std::uniform_int_distribution</code></a> with parameters 0 and <code>highest - 1</code>. That class can also be used to obtain quantities larger than 32 bits.</p>
    /// <p>For example, to obtain a value between 0 and 255 (inclusive), one would write:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#quint32-typedef">quint32</a></span> v <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#QRandomGenerator">QRandomGenerator</a></span><span class="operator">::</span>bounded(<span class="number">256</span>);
    ///
    /// </pre>
    /// <p>Naturally, the same could also be obtained by masking the result of <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate">generate</a>() to only the lower 8 bits. Either solution is as efficient.</p>
    /// <p>Note that this function cannot be used to obtain values in the full 32-bit range of quint32. Instead, use <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate">generate</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate">generate</a>(), <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate64">generate64</a>(), and <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generateDouble">generateDouble</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn bounded_u32(&mut self, highest: u32) -> u32 {
        crate::__ffi::ctr_qt_core_ffi_QRandomGenerator_bounded1(
            self as *mut crate::QRandomGenerator,
            highest,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRandomGenerator::bounded(int highest)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#bounded-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Generates one random 32-bit quantity in the range between 0 (inclusive) and <i>highest</i> (exclusive). <i>highest</i> must not be negative.</p>
    /// <p>Note that this function cannot be used to obtain values in the full 32-bit range of int. Instead, use <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate">generate</a>() and cast to int.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate">generate</a>(), <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate64">generate64</a>(), and <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generateDouble">generateDouble</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn bounded_int(&mut self, highest: ::std::os::raw::c_int) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRandomGenerator_bounded2(
            self as *mut crate::QRandomGenerator,
            highest,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```quint32 QRandomGenerator::bounded(quint32 lowest, quint32 highest)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#bounded-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Generates one random 32-bit quantity in the range between <i>lowest</i> (inclusive) and <i>highest</i> (exclusive). The same result may also be obtained by using <a href="http://en.cppreference.com/w/cpp/numeric/random/uniform_int_distribution"><code>std::uniform_int_distribution</code></a> with parameters <i>lowest</i> and <code>\a highest - 1</code>. That class can also be used to obtain quantities larger than 32 bits.</p>
    /// <p>For example, to obtain a value between 1000 (incl.) and 2000 (excl.), one would write:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#quint32-typedef">quint32</a></span> v <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#QRandomGenerator">QRandomGenerator</a></span><span class="operator">::</span>bounded(<span class="number">1000</span><span class="operator">,</span> <span class="number">2000</span>);
    ///
    /// </pre>
    /// <p>Note that this function cannot be used to obtain values in the full 32-bit range of quint32. Instead, use <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate">generate</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate">generate</a>(), <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate64">generate64</a>(), and <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generateDouble">generateDouble</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn bounded_2_u32(&mut self, lowest: u32, highest: u32) -> u32 {
        crate::__ffi::ctr_qt_core_ffi_QRandomGenerator_bounded3(
            self as *mut crate::QRandomGenerator,
            lowest,
            highest,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QRandomGenerator::bounded(int lowest, int highest)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#bounded-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Generates one random 32-bit quantity in the range between <i>lowest</i> (inclusive) and <i>highest</i> (exclusive), both of which may be negative.</p>
    /// <p>Note that this function cannot be used to obtain values in the full 32-bit range of int. Instead, use <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate">generate</a>() and cast to int.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate">generate</a>(), <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate64">generate64</a>(), and <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generateDouble">generateDouble</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn bounded_2_int(
        &mut self,
        lowest: ::std::os::raw::c_int,
        highest: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QRandomGenerator_bounded4(
            self as *mut crate::QRandomGenerator,
            lowest,
            highest,
        )
    }

    /// <p>Generates a 32-bit random quantity and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned int QRandomGenerator::operator()()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#operator-28-29">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Generates a 32-bit random quantity and returns it.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate">generate</a>() and <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate64">generate64</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn call(&mut self) -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QRandomGenerator_operator__(
            self as *mut crate::QRandomGenerator,
        )
    }

    /// <p>Copy-assignment operator.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRandomGenerator& QRandomGenerator::operator=(const QRandomGenerator& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Copy-assignment operator.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRandomGenerator>>,
    ) -> ::cpp_utils::MutRef<crate::QRandomGenerator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRandomGenerator_operator_(
            self as *mut crate::QRandomGenerator,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRandomGenerator>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Discards the next <i>z</i> entries from the sequence. This method is equivalent to calling <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate">generate</a>() <i>z</i> times and discarding the result, as in:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRandomGenerator::discard(unsigned long long z)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#discard">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Discards the next <i>z</i> entries from the sequence. This method is equivalent to calling <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate">generate</a>() <i>z</i> times and discarding the result, as in:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">while</span> (z<span class="operator">-</span><span class="operator">-</span>)
    /// &#32;     generator<span class="operator">.</span>generate();
    ///
    /// </pre></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn discard(&mut self, z: ::std::os::raw::c_ulonglong) {
        crate::__ffi::ctr_qt_core_ffi_QRandomGenerator_discard(
            self as *mut crate::QRandomGenerator,
            z,
        )
    }

    /// <p>Generates a 32-bit random quantity and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```quint32 QRandomGenerator::generate()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Generates a 32-bit random quantity and returns it.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#operator-28-29">operator</a>()() and <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate64">generate64</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn generate_0a(&mut self) -> u32 {
        crate::__ffi::ctr_qt_core_ffi_QRandomGenerator_generate(
            self as *mut crate::QRandomGenerator,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QRandomGenerator::generate(quint32* begin, quint32* end)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn generate_2a(
        &mut self,
        begin: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<u32>>,
        end: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<u32>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QRandomGenerator_generate1(
            self as *mut crate::QRandomGenerator,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<u32>>::cast_into(begin).as_mut_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<u32>>::cast_into(end).as_mut_raw_ptr(),
        )
    }

    /// <p>Generates a 64-bit random quantity and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```quint64 QRandomGenerator::generate64()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate64">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Generates a 64-bit random quantity and returns it.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#operator-28-29">operator</a>()() and <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate">generate</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn generate64(&mut self) -> u64 {
        crate::__ffi::ctr_qt_core_ffi_QRandomGenerator_generate64(
            self as *mut crate::QRandomGenerator,
        )
    }

    /// <p>Generates one random qreal in the canonical range [0, 1) (that is, inclusive of zero and exclusive of 1).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QRandomGenerator::generateDouble()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generateDouble">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Generates one random qreal in the canonical range [0, 1) (that is, inclusive of zero and exclusive of 1).</p>
    /// <p>This function is equivalent to:</p>
    /// <pre class="cpp">
    ///
    ///   QRandomGenerator64 rd;
    ///   <span class="keyword">return</span> std<span class="operator">::</span>generate_canonical<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#qreal-typedef">qreal</a></span><span class="operator">,</span> std<span class="operator">::</span>numeric_limits<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#qreal-typedef">qreal</a></span><span class="operator">&gt;</span><span class="operator">::</span>digits<span class="operator">&gt;</span>(rd);
    ///
    /// </pre>
    /// <p>The same may also be obtained by using <a href="http://en.cppreference.com/w/cpp/numeric/random/uniform_real_distribution"><code>std::uniform_real_distribution</code></a> with parameters 0 and 1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate">generate</a>(), <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate64">generate64</a>(), and <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#bounded">bounded</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn generate_double(&mut self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QRandomGenerator_generateDouble(
            self as *mut crate::QRandomGenerator,
        )
    }

    /// <p>Returns a pointer to a shared <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> that was seeded using <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#securelySeeded">securelySeeded</a>(). This function should be used to create random data without the expensive creation of a securely-seeded <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> for a specific use or storing the rather large <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QRandomGenerator* QRandomGenerator::global()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#global">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to a shared <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> that was seeded using <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#securelySeeded">securelySeeded</a>(). This function should be used to create random data without the expensive creation of a securely-seeded <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> for a specific use or storing the rather large <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> object.</p>
    /// <p>For example, the following creates a random RGB color:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">return</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcolor.html">QColor</a></span><span class="operator">::</span>fromRgb(<span class="type"><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#QRandomGenerator">QRandomGenerator</a></span><span class="operator">::</span>global()<span class="operator">-</span><span class="operator">&gt;</span>generate());
    ///
    /// </pre>
    /// <p>Accesses to this object are thread-safe and it may therefore be used in any thread without locks. The object may also be copied and the sequence produced by the copy will be the same as the shared object will produce. Note, however, that if there are other threads accessing the global object, those threads may obtain samples at unpredictable intervals.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#securelySeeded">securelySeeded</a>() and <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#system">system</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn global() -> ::cpp_utils::MutPtr<crate::QRandomGenerator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRandomGenerator_global();
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Returns the maximum value that <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> may ever generate. That is, <code>std::numeric_limits&lt;result_type&gt;::max()</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static unsigned int QRandomGenerator::max()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#max">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the maximum value that <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> may ever generate. That is, <code>std::numeric_limits&lt;result_type&gt;::max()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#min">min</a>() and QRandomGenerator64::max().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn max() -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QRandomGenerator_max()
    }

    /// <p>Returns the minimum value that <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> may ever generate. That is, 0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static unsigned int QRandomGenerator::min()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#min">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the minimum value that <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> may ever generate. That is, 0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#max">max</a>() and QRandomGenerator64::min().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn min() -> ::std::os::raw::c_uint {
        crate::__ffi::ctr_qt_core_ffi_QRandomGenerator_min()
    }

    /// <p>Initializes this <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> object with the value <i>seedValue</i> as the seed. Two objects constructed or reseeded with the same seed value will produce the same number sequence.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRandomGenerator::QRandomGenerator(quint32 seedValue = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#QRandomGenerator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Initializes this <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> object with the value <i>seedValue</i> as the seed. Two objects constructed or reseeded with the same seed value will produce the same number sequence.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#seed">seed</a>() and <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#securelySeeded">securelySeeded</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn from_u32(seed_value: u32) -> ::cpp_utils::CppBox<crate::QRandomGenerator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QRandomGenerator_QRandomGenerator(seed_value);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRandomGenerator::QRandomGenerator(const quint32* seedBuffer, long long len)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn from_u32_longlong(
        seed_buffer: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<u32>>,
        len: ::std::os::raw::c_longlong,
    ) -> ::cpp_utils::CppBox<crate::QRandomGenerator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRandomGenerator_QRandomGenerator1(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<u32>>::cast_into(seed_buffer).as_raw_ptr(),
            len,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRandomGenerator::QRandomGenerator(const quint32* begin, const quint32* end)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#QRandomGenerator-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Initializes this <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> object with the values found in the range from <i>begin</i> to <i>end</i> as the seed. Two objects constructed or reseeded with the same seed value will produce the same number sequence.</p>
    /// <p>This constructor is equivalent to:</p>
    /// <pre class="cpp">
    ///
    ///   std<span class="operator">::</span>seed_seq sseq(begin<span class="operator">,</span> end);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#QRandomGenerator">QRandomGenerator</a></span> generator(sseq);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#seed">seed</a>() and <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#securelySeeded">securelySeeded</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn from_2_u32(
        begin: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<u32>>,
        end: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<u32>>,
    ) -> ::cpp_utils::CppBox<crate::QRandomGenerator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRandomGenerator_QRandomGenerator3(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<u32>>::cast_into(begin).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<u32>>::cast_into(end).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> class allows one to obtain random values from a high-quality Random Number Generator.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRandomGenerator::QRandomGenerator()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> class allows one to obtain random values from a high-quality Random Number Generator.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> may be used to generate random values from a high-quality random number generator. Like the C++ random engines, <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> can be seeded with user-provided values through the constructor. When seeded, the sequence of numbers generated by this class is deterministic. That is to say, given the same seed data, <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> will generate the same sequence of numbers. But given different seeds, the results should be considerably different.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#securelySeeded">QRandomGenerator::securelySeeded</a>() can be used to create a <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> that is securely seeded with <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#system">QRandomGenerator::system</a>(), meaning that the sequence of numbers it generates cannot be easily predicted. Additionally, <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#global">QRandomGenerator::global</a>() returns a global instance of <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> that Qt will ensure to be securely seeded. This object is thread-safe, may be shared for most uses, and is always seeded from <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#system">QRandomGenerator::system</a>()</p>
    /// <p><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#system">QRandomGenerator::system</a>() may be used to access the system's cryptographically-safe random generator. On Unix systems, it's equivalent to reading from <code>/dev/urandom</code> or the <code>getrandom()</code> or <code>getentropy()</code> system calls.</p>
    /// <p>The class can generate 32-bit or 64-bit quantities, or fill an array of those. The most common way of generating new values is to call the <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate">generate</a>(), <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate64">generate64</a>() or <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#fillRange">fillRange</a>() functions. One would use it as:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#quint32-typedef">quint32</a></span> value <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#QRandomGenerator">QRandomGenerator</a></span><span class="operator">::</span>global()<span class="operator">-</span><span class="operator">&gt;</span>generate();
    ///
    /// </pre>
    /// <p>Additionally, it provides a floating-point function <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generateDouble">generateDouble</a>() that returns a number in the range [0, 1) (that is, inclusive of zero and exclusive of 1). There's also a set of convenience functions that facilitate obtaining a random number in a bounded, integral range.</p>
    /// <a name="seeding-and-determinism"></a></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QRandomGenerator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRandomGenerator_QRandomGenerator5();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a copy of the generator state in the <i>other</i> object. If <i>other</i> is <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#system">QRandomGenerator::system</a>() or a copy of that, this object will also read from the operating system random-generating facilities. In that case, the sequences generated by the two objects will be different.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRandomGenerator::QRandomGenerator(const QRandomGenerator& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#QRandomGenerator-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a copy of the generator state in the <i>other</i> object. If <i>other</i> is <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#system">QRandomGenerator::system</a>() or a copy of that, this object will also read from the operating system random-generating facilities. In that case, the sequences generated by the two objects will be different.</p>
    /// <p>In all other cases, the new <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> object will start at the same position in the deterministic sequence as the <i>other</i> object was. Both objects will generate the same sequence from this point on.</p>
    /// <p>For that reason, it is not adviseable to create a copy of <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#global">QRandomGenerator::global</a>(). If one needs an exclusive deterministic generator, consider instead using <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#securelySeeded">securelySeeded</a>() to obtain a new object that shares no relationship with the <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#global">QRandomGenerator::global</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRandomGenerator>>,
    ) -> ::cpp_utils::CppBox<crate::QRandomGenerator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRandomGenerator_QRandomGenerator4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRandomGenerator>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a new <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> object that was securely seeded with <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#system">QRandomGenerator::system</a>(). This function will obtain the ideal seed size for the algorithm that <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> uses and is therefore the recommended way for creating a new <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> object that will be kept for some time.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QRandomGenerator QRandomGenerator::securelySeeded()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#securelySeeded">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a new <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> object that was securely seeded with <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#system">QRandomGenerator::system</a>(). This function will obtain the ideal seed size for the algorithm that <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> uses and is therefore the recommended way for creating a new <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> object that will be kept for some time.</p>
    /// <p>Given the amount of data required to securely seed the deterministic engine, this function is somewhat expensive and should not be used for short-term uses of <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> (using it to generate fewer than 2600 bytes of random data is effectively a waste of resources). If the use doesn't require that much data, consider using <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#global">QRandomGenerator::global</a>() and not storing a <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> object instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#global">global</a>() and <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#system">system</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn securely_seeded() -> ::cpp_utils::CppBox<crate::QRandomGenerator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRandomGenerator_securelySeeded();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reseeds this object using the value <i>seed</i> as the seed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRandomGenerator::seed(quint32 s = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#seed">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reseeds this object using the value <i>seed</i> as the seed.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn seed_1a(&mut self, s: u32) {
        crate::__ffi::ctr_qt_core_ffi_QRandomGenerator_seed(self as *mut crate::QRandomGenerator, s)
    }

    /// <p>Reseeds this object using the value <i>seed</i> as the seed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QRandomGenerator::seed()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#seed">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reseeds this object using the value <i>seed</i> as the seed.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn seed_0a(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QRandomGenerator_seed2(self as *mut crate::QRandomGenerator)
    }

    /// <p>Returns a pointer to a shared <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> that always uses the facilities provided by the operating system to generate random numbers. The system facilities are considered to be cryptographically safe on at least the following operating systems: Apple OSes (Darwin), BSDs, Linux, Windows. That may also be the case on other operating systems.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QRandomGenerator* QRandomGenerator::system()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#system">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to a shared <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> that always uses the facilities provided by the operating system to generate random numbers. The system facilities are considered to be cryptographically safe on at least the following operating systems: Apple OSes (Darwin), BSDs, Linux, Windows. That may also be the case on other operating systems.</p>
    /// <p>They are also possibly backed by a true hardware random number generator. For that reason, the <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> returned by this function should not be used for bulk data generation. Instead, use it to seed <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> or a random engine from the &lt;random&gt; header.</p>
    /// <p>The object returned by this function is thread-safe and may be used in any thread without locks. It may also be copied and the resulting <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> will also access the operating system facilities, but they will not generate the same sequence.</p>
    /// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#securelySeeded">securelySeeded</a>() and <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#global">global</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn system() -> ::cpp_utils::MutPtr<crate::QRandomGenerator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRandomGenerator_system();
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qrandomgenerator64.html">QRandomGenerator64</a> class allows one to obtain 64-bit random values from a high-quality, seed-less Random Number Generator.</p>
///
/// C++ class: <span style='color: green;'>```QRandomGenerator64```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qrandomgenerator64.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qrandomgenerator64.html">QRandomGenerator64</a> class allows one to obtain 64-bit random values from a high-quality, seed-less Random Number Generator.</p>
/// <p><a href="http://doc.qt.io/qt-5/qrandomgenerator64.html">QRandomGenerator64</a> is a simple adaptor class around <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a>, making the <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate64">QRandomGenerator::generate64</a>() function the default for <a href="http://doc.qt.io/qt-5/qrandomgenerator64.html#operator-28-29">operator</a>()(), instead of the function that returns 32-bit quantities. This class is intended to be used in conjunction with Standard Library algorithms that need 64-bit quantities instead of 32-bit ones.</p>
/// <p>In all other aspects, the class is the same. Please refer to <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a>'s documentation for more information.</p></div>
#[repr(C)]
pub struct QRandomGenerator64 {
    _unused: u8,
}
impl QRandomGenerator64 {
    /// <p>Generates a 64-bit random quantity and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```unsigned long long QRandomGenerator64::operator()()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator64.html#operator-28-29">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Generates a 64-bit random quantity and returns it.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate">QRandomGenerator::generate</a>() and <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate64">QRandomGenerator::generate64</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn call(&mut self) -> ::std::os::raw::c_ulonglong {
        crate::__ffi::ctr_qt_core_ffi_QRandomGenerator64_operator__(
            self as *mut crate::QRandomGenerator64,
        )
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qrandomgenerator64.html">QRandomGenerator64</a> class allows one to obtain 64-bit random values from a high-quality, seed-less Random Number Generator.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRandomGenerator64& QRandomGenerator64::operator=(const QRandomGenerator64& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator64.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qrandomgenerator64.html">QRandomGenerator64</a> class allows one to obtain 64-bit random values from a high-quality, seed-less Random Number Generator.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qrandomgenerator64.html">QRandomGenerator64</a> is a simple adaptor class around <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a>, making the <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate64">QRandomGenerator::generate64</a>() function the default for <a href="http://doc.qt.io/qt-5/qrandomgenerator64.html#operator-28-29">operator</a>()(), instead of the function that returns 32-bit quantities. This class is intended to be used in conjunction with Standard Library algorithms that need 64-bit quantities instead of 32-bit ones.</p>
    /// <p>In all other aspects, the class is the same. Please refer to <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a>'s documentation for more information.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRandomGenerator64>>,
    ) -> ::cpp_utils::MutRef<crate::QRandomGenerator64> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRandomGenerator64_operator_(
            self as *mut crate::QRandomGenerator64,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRandomGenerator64>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```void QRandomGenerator64::discard(unsigned long long z)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn discard(&mut self, z: ::std::os::raw::c_ulonglong) {
        crate::__ffi::ctr_qt_core_ffi_QRandomGenerator64_discard(
            self as *mut crate::QRandomGenerator64,
            z,
        )
    }

    /// <p>Generates one 64-bit random value and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```quint64 QRandomGenerator64::generate()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator64.html#generate">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Generates one 64-bit random value and returns it.</p>
    /// <p>Note about casting to a signed integer: all bits returned by this function are random, so there's a 50% chance that the most significant bit will be set. If you wish to cast the returned value to qint64 and keep it positive, you should mask the sign bit off:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#qint64-typedef">qint64</a></span> value <span class="operator">=</span> QRandomGenerator64<span class="operator">::</span>generate() <span class="operator">&amp;</span> std<span class="operator">::</span>numeric_limits<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#qint64-typedef">qint64</a></span><span class="operator">&gt;</span><span class="operator">::</span>max();
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> and <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate64">QRandomGenerator::generate64</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn generate(&mut self) -> u64 {
        crate::__ffi::ctr_qt_core_ffi_QRandomGenerator64_generate(
            self as *mut crate::QRandomGenerator64,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```static QRandomGenerator64* QRandomGenerator64::global()```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn global() -> ::cpp_utils::MutPtr<crate::QRandomGenerator64> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRandomGenerator64_global();
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```static unsigned long long QRandomGenerator64::max()```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn max() -> ::std::os::raw::c_ulonglong {
        crate::__ffi::ctr_qt_core_ffi_QRandomGenerator64_max()
    }

    /// Calls C++ function: <span style='color: green;'>```static unsigned long long QRandomGenerator64::min()```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn min() -> ::std::os::raw::c_ulonglong {
        crate::__ffi::ctr_qt_core_ffi_QRandomGenerator64_min()
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRandomGenerator64::QRandomGenerator64(quint32 seedValue = …)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn from_u32(seed_value: u32) -> ::cpp_utils::CppBox<crate::QRandomGenerator64> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QRandomGenerator64_QRandomGenerator64(seed_value);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRandomGenerator64::QRandomGenerator64(const quint32* seedBuffer, long long len)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn from_u32_longlong(
        seed_buffer: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<u32>>,
        len: ::std::os::raw::c_longlong,
    ) -> ::cpp_utils::CppBox<crate::QRandomGenerator64> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRandomGenerator64_QRandomGenerator641(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<u32>>::cast_into(seed_buffer).as_raw_ptr(),
            len,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRandomGenerator64::QRandomGenerator64(const quint32* begin, const quint32* end)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn from_2_u32(
        begin: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<u32>>,
        end: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<u32>>,
    ) -> ::cpp_utils::CppBox<crate::QRandomGenerator64> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRandomGenerator64_QRandomGenerator643(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<u32>>::cast_into(begin).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<u32>>::cast_into(end).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRandomGenerator64::QRandomGenerator64(const QRandomGenerator& other)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn from_q_random_generator(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRandomGenerator>>,
    ) -> ::cpp_utils::CppBox<crate::QRandomGenerator64> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRandomGenerator64_QRandomGenerator644(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRandomGenerator>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qrandomgenerator64.html">QRandomGenerator64</a> class allows one to obtain 64-bit random values from a high-quality, seed-less Random Number Generator.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRandomGenerator64::QRandomGenerator64()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator64.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qrandomgenerator64.html">QRandomGenerator64</a> class allows one to obtain 64-bit random values from a high-quality, seed-less Random Number Generator.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qrandomgenerator64.html">QRandomGenerator64</a> is a simple adaptor class around <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a>, making the <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate64">QRandomGenerator::generate64</a>() function the default for <a href="http://doc.qt.io/qt-5/qrandomgenerator64.html#operator-28-29">operator</a>()(), instead of the function that returns 32-bit quantities. This class is intended to be used in conjunction with Standard Library algorithms that need 64-bit quantities instead of 32-bit ones.</p>
    /// <p>In all other aspects, the class is the same. Please refer to <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a>'s documentation for more information.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QRandomGenerator64> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRandomGenerator64_QRandomGenerator645();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qrandomgenerator64.html">QRandomGenerator64</a> class allows one to obtain 64-bit random values from a high-quality, seed-less Random Number Generator.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QRandomGenerator64::QRandomGenerator64(const QRandomGenerator64& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator64.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qrandomgenerator64.html">QRandomGenerator64</a> class allows one to obtain 64-bit random values from a high-quality, seed-less Random Number Generator.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qrandomgenerator64.html">QRandomGenerator64</a> is a simple adaptor class around <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a>, making the <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate64">QRandomGenerator::generate64</a>() function the default for <a href="http://doc.qt.io/qt-5/qrandomgenerator64.html#operator-28-29">operator</a>()(), instead of the function that returns 32-bit quantities. This class is intended to be used in conjunction with Standard Library algorithms that need 64-bit quantities instead of 32-bit ones.</p>
    /// <p>In all other aspects, the class is the same. Please refer to <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a>'s documentation for more information.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRandomGenerator64>>,
    ) -> ::cpp_utils::CppBox<crate::QRandomGenerator64> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRandomGenerator64_QRandomGenerator646(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRandomGenerator64>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QRandomGenerator64 QRandomGenerator64::securelySeeded()```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn securely_seeded() -> ::cpp_utils::CppBox<crate::QRandomGenerator64> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRandomGenerator64_securelySeeded();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QRandomGenerator64* QRandomGenerator64::system()```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn system() -> ::cpp_utils::MutPtr<crate::QRandomGenerator64> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QRandomGenerator64_system();
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html">QSemaphoreReleaser</a> class provides exception-safe deferral of a <a href="http://doc.qt.io/qt-5/qsemaphore.html#release">QSemaphore::release</a>() call.</p>
///
/// C++ class: <span style='color: green;'>```QSemaphoreReleaser```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html">QSemaphoreReleaser</a> class provides exception-safe deferral of a <a href="http://doc.qt.io/qt-5/qsemaphore.html#release">QSemaphore::release</a>() call.</p>
/// <p><a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html">QSemaphoreReleaser</a> can be used wherever you would otherwise use <a href="http://doc.qt.io/qt-5/qsemaphore.html#release">QSemaphore::release</a>(). Constructing a <a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html">QSemaphoreReleaser</a> defers the release() call on the semaphore until the <a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html">QSemaphoreReleaser</a> is destroyed (see <a href="http://en.cppreference.com/w/cpp/language/raii">RAII pattern</a>).</p>
/// <p>You can use this to reliably release a semaphore to avoid dead-lock in the face of exceptions or early returns:</p>
/// <pre class="cpp">
///
///   <span class="comment">// ... do something that may throw or return early</span>
///   sem<span class="operator">.</span>release();
///
/// </pre>
/// <p>If an early return is taken or an exception is thrown before the <code>sem.release()</code> call is reached, the semaphore is not released, possibly preventing the thread waiting in the corresponding <code>sem.acquire()</code> call from ever continuing execution.</p>
/// <p>When using RAII instead:</p>
/// <pre class="cpp">
///
///   <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html#QSemaphoreReleaser">QSemaphoreReleaser</a></span> releaser(sem);
///   <span class="comment">// ... do something that may throw or early return</span>
///   <span class="comment">// implicitly calls sem.release() here and at every other return in between</span>
///
/// </pre>
/// <p>this can no longer happen, because the compiler will make sure that the <a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html">QSemaphoreReleaser</a> destructor is always called, and therefore the semaphore is always released.</p>
/// <p><a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html">QSemaphoreReleaser</a> is move-enabled and can therefore be returned from functions to transfer responsibility for releasing a semaphore out of a function or a scope:</p>
/// <pre class="cpp">
///
///   { <span class="comment">// some scope</span>
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html#QSemaphoreReleaser">QSemaphoreReleaser</a></span> releaser; <span class="comment">// does nothing</span>
/// &#32;     <span class="comment">// ...</span>
/// &#32;     <span class="keyword">if</span> (someCondition) {
/// &#32;   &#32;     releaser <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html#QSemaphoreReleaser">QSemaphoreReleaser</a></span>(sem);
/// &#32;   &#32;     <span class="comment">// ...</span>
/// &#32;     }
/// &#32;     <span class="comment">// ...</span>
///   } <span class="comment">// conditionally calls sem.release(), depending on someCondition</span>
///
/// </pre>
/// <p>A <a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html">QSemaphoreReleaser</a> can be canceled by a call to <a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html#cancel">cancel</a>(). A canceled semaphore releaser will no longer call <a href="http://doc.qt.io/qt-5/qsemaphore.html#release">QSemaphore::release</a>() in its destructor.</p></div>
#[repr(C)]
pub struct QSemaphoreReleaser {
    _unused: u8,
}
impl QSemaphoreReleaser {
    /// <p>Cancels this <a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html">QSemaphoreReleaser</a> such that the destructor will no longer call <code>semaphore()-&gt;release()</code>. Returns the value of <a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html#semaphore">semaphore</a>() before this call. After this call, <a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html#semaphore">semaphore</a>() will return <code>nullptr</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSemaphore* QSemaphoreReleaser::cancel()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html#cancel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Cancels this <a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html">QSemaphoreReleaser</a> such that the destructor will no longer call <code>semaphore()-&gt;release()</code>. Returns the value of <a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html#semaphore">semaphore</a>() before this call. After this call, <a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html#semaphore">semaphore</a>() will return <code>nullptr</code>.</p>
    /// <p>To enable again, assign a new <a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html">QSemaphoreReleaser</a>:</p>
    /// <pre class="cpp">
    ///
    ///   releaser<span class="operator">.</span>cancel(); <span class="comment">// avoid releasing old semaphore()</span>
    ///   releaser <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html#QSemaphoreReleaser">QSemaphoreReleaser</a></span>(sem<span class="operator">,</span> <span class="number">42</span>);
    ///   <span class="comment">// now will call sem.release(42) when 'releaser' is destroyed</span>
    ///
    /// </pre></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn cancel(&mut self) -> ::cpp_utils::MutPtr<crate::QSemaphore> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSemaphoreReleaser_cancel(
            self as *mut crate::QSemaphoreReleaser,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Default constructor. Creates a <a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html">QSemaphoreReleaser</a> that does nothing.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSemaphoreReleaser::QSemaphoreReleaser()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html#QSemaphoreReleaser">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Default constructor. Creates a <a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html">QSemaphoreReleaser</a> that does nothing.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QSemaphoreReleaser> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSemaphoreReleaser_QSemaphoreReleaser();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructor. Stores the arguments and calls <i>sem</i>.release(<i>n</i>) in the destructor.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSemaphoreReleaser::QSemaphoreReleaser(QSemaphore& sem, int n = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html#QSemaphoreReleaser-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructor. Stores the arguments and calls <i>sem</i>.release(<i>n</i>) in the destructor.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn from_q_semaphore_int(
        sem: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QSemaphore>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QSemaphoreReleaser> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSemaphoreReleaser_QSemaphoreReleaser1(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QSemaphore>>::cast_into(sem)
                .as_mut_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructor. Stores the arguments and calls <i>sem</i>-&gt;release(<i>n</i>) in the destructor.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSemaphoreReleaser::QSemaphoreReleaser(QSemaphore* sem, int n = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html#QSemaphoreReleaser-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructor. Stores the arguments and calls <i>sem</i>-&gt;release(<i>n</i>) in the destructor.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn from_q_semaphore_int2(
        sem: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QSemaphore>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QSemaphoreReleaser> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSemaphoreReleaser_QSemaphoreReleaser2(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QSemaphore>>::cast_into(sem)
                .as_mut_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructor. Stores the arguments and calls <i>sem</i>.release(<i>n</i>) in the destructor.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSemaphoreReleaser::QSemaphoreReleaser(QSemaphore& sem)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html#QSemaphoreReleaser-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructor. Stores the arguments and calls <i>sem</i>.release(<i>n</i>) in the destructor.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn from_q_semaphore(
        sem: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QSemaphore>>,
    ) -> ::cpp_utils::CppBox<crate::QSemaphoreReleaser> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSemaphoreReleaser_QSemaphoreReleaser5(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QSemaphore>>::cast_into(sem)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructor. Stores the arguments and calls <i>sem</i>-&gt;release(<i>n</i>) in the destructor.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QSemaphoreReleaser::QSemaphoreReleaser(QSemaphore* sem)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html#QSemaphoreReleaser-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructor. Stores the arguments and calls <i>sem</i>-&gt;release(<i>n</i>) in the destructor.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn from_q_semaphore2(
        sem: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QSemaphore>>,
    ) -> ::cpp_utils::CppBox<crate::QSemaphoreReleaser> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSemaphoreReleaser_QSemaphoreReleaser6(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QSemaphore>>::cast_into(sem)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a pointer to the <a href="http://doc.qt.io/qt-5/qsemaphore.html">QSemaphore</a> object provided to the constructor, or by the last move assignment, if any. Otherwise, returns <code>nullptr</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSemaphore* QSemaphoreReleaser::semaphore() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html#semaphore">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the <a href="http://doc.qt.io/qt-5/qsemaphore.html">QSemaphore</a> object provided to the constructor, or by the last move assignment, if any. Otherwise, returns <code>nullptr</code>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn semaphore(&self) -> ::cpp_utils::MutPtr<crate::QSemaphore> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QSemaphoreReleaser_semaphore(
            self as *const crate::QSemaphoreReleaser,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Exchanges the responsibilites of <code>*this</code> and <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QSemaphoreReleaser::swap(QSemaphoreReleaser& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchanges the responsibilites of <code>*this</code> and <i>other</i>.</p>
    /// <p>Unlike move assignment, neither of the two objects ever releases its semaphore, if any, as a consequence of swapping.</p>
    /// <p>Therefore this function is very fast and never fails.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QSemaphoreReleaser>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QSemaphoreReleaser_swap(
            self as *mut crate::QSemaphoreReleaser,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QSemaphoreReleaser>>::cast_into(
                other,
            )
            .as_mut_raw_ptr(),
        )
    }
}

/// <p>This enum contains the possible "Simple Types" for CBOR. Simple Types range from 0 to 255 and are types that carry no further value.</p>
///
/// C++ enum: <span style='color: green;'>```QCborSimpleType```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborSimpleType-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum contains the possible "Simple Types" for CBOR. Simple Types range from 0 to 255 and are types that carry no further value.</p>
/// <p>The following values are currently known:</p>
///
/// <p>Qt CBOR API supports encoding and decoding any Simple Type, whether one of those above or any other value.</p>
/// <p>Applications should only use further values if a corresponding specification has been published, otherwise interpretation and validation by the remote may fail. Values 24 to 31 are reserved and must not be used.</p>
/// <p>The current authoritative list is maintained by IANA in the <a href="https://www.iana.org/assignments/cbor-simple-values/cbor-simple-values.xml">Simple Values registry</a>.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append-8">QCborStreamWriter::append</a>(QCborSimpleType), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isSimpleType">QCborStreamReader::isSimpleType</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toSimpleType">QCborStreamReader::toSimpleType</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isSimpleType">QCborValue::isSimpleType</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toSimpleType">QCborValue::toSimpleType</a>().</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct QCborSimpleType(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for QCborSimpleType {
    fn from(value: ::std::os::raw::c_int) -> Self {
        QCborSimpleType(value)
    }
}

impl From<QCborSimpleType> for ::std::os::raw::c_int {
    fn from(value: QCborSimpleType) -> Self {
        value.0
    }
}

impl QCborSimpleType {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl QCborSimpleType {
    /// A "false" boolean. (C++ enum variant: <span style='color: green;'>```False = 20```</span>)
    #[allow(non_upper_case_globals)]
    pub const False: crate::QCborSimpleType = crate::QCborSimpleType(20);
    /// A "true" boolean. (C++ enum variant: <span style='color: green;'>```True = 21```</span>)
    #[allow(non_upper_case_globals)]
    pub const True: crate::QCborSimpleType = crate::QCborSimpleType(21);
    /// Absence of value (null). (C++ enum variant: <span style='color: green;'>```Null = 22```</span>)
    #[allow(non_upper_case_globals)]
    pub const Null: crate::QCborSimpleType = crate::QCborSimpleType(22);
    /// Missing or deleted value, usually an error. (C++ enum variant: <span style='color: green;'>```Undefined = 23```</span>)
    #[allow(non_upper_case_globals)]
    pub const Undefined: crate::QCborSimpleType = crate::QCborSimpleType(23);
}

/// <p>This enum contains no enumeration and is used only to provide type-safe access to a CBOR tag.</p>
///
/// C++ enum: <span style='color: green;'>```QCborTag```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborTag-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum contains no enumeration and is used only to provide type-safe access to a CBOR tag.</p>
/// <p>CBOR tags are 64-bit numbers that are attached to generic CBOR types to provide further semantic meaning. QCborTag may be constructed from an enumeration found in <a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborKnownTags-enum">QCborKnownTags</a> or directly by providing the numeric representation.</p>
/// <p>For example, the following creates a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> containing a byte array tagged with a tag 2.</p>
/// <pre class="cpp">
///
/// &#32;    <span class="type"><a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a></span>(<span class="type">QCborTag</span>(<span class="number">2</span>)<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span>(<span class="string">"\x01\0\0\0\0\0\0\0\0"</span><span class="operator">,</span> <span class="number">9</span>));
///
/// </pre>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborKnownTags-enum">QCborKnownTags</a>, <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append-6">QCborStreamWriter::append</a>(QCborTag), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isTag">QCborStreamReader::isTag</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toTag">QCborStreamReader::toTag</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isTag">QCborValue::isTag</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#tag">QCborValue::tag</a>().</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct QCborTag(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for QCborTag {
    fn from(value: ::std::os::raw::c_int) -> Self {
        QCborTag(value)
    }
}

impl From<QCborTag> for ::std::os::raw::c_int {
    fn from(value: QCborTag) -> Self {
        value.0
    }
}

impl QCborTag {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

/// <p>This enum contains a list of CBOR tags, known at the time of the Qt implementation. This list is not meant to be complete and contains only tags that are either backed by an RFC or specifically used by the Qt implementation.</p>
///
/// C++ enum: <span style='color: green;'>```QCborKnownTags```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborKnownTags-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum contains a list of CBOR tags, known at the time of the Qt implementation. This list is not meant to be complete and contains only tags that are either backed by an RFC or specifically used by the Qt implementation.</p>
/// <p>The authoritative list is maintained by IANA in the <a href="https://www.iana.org/assignments/cbor-tags/cbor-tags.xhtml">CBOR tag registry</a>.</p>
///
/// <p>The following tags are interpreted by <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> during decoding and will produce objects with extended Qt types, and it will use those tags when encoding the same extended types.</p>
///
/// <p>Additionally, if a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> containing a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> is tagged using one of <code>ExpectedBase64url</code>, <code>ExpectedBase64</code> or <code>ExpectedBase16</code>, <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> will use the expected encoding when converting to JSON (see <a href="http://doc.qt.io/qt-5/qcborvalue.html#toJsonValue">QCborValue::toJsonValue</a>).</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborTag-enum">QCborTag</a>, <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append-6">QCborStreamWriter::append</a>(QCborTag), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isTag">QCborStreamReader::isTag</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toTag">QCborStreamReader::toTag</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isTag">QCborValue::isTag</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#tag">QCborValue::tag</a>().</p></div>
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct QCborKnownTags(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for QCborKnownTags {
    fn from(value: ::std::os::raw::c_int) -> Self {
        QCborKnownTags(value)
    }
}

impl From<QCborKnownTags> for ::std::os::raw::c_int {
    fn from(value: QCborKnownTags) -> Self {
        value.0
    }
}

impl QCborKnownTags {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

impl QCborKnownTags {
    /// A date and time string, formatted according to RFC 3339, as refined by RFC 4287. It is the same format as <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::ISODate</a> and <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::ISODateWithMs</a>. (C++ enum variant: <span style='color: green;'>```DateTimeString = 0```</span>)
    #[allow(non_upper_case_globals)]
    pub const DateTimeString: crate::QCborKnownTags = crate::QCborKnownTags(0);
    /// A numerical representation of seconds elapsed since 1970-01-01T00:00Z. (C++ enum variant: <span style='color: green;'>```UnixTime_t = 1```</span>)
    #[allow(non_upper_case_globals)]
    pub const UnixTimeT: crate::QCborKnownTags = crate::QCborKnownTags(1);
    /// A positive number of arbitrary length, encoded as a byte array in network byte order. For example, the number 2<sup>64</sup> is represented by a byte array containing the byte value 0x01 followed by 8 zero bytes. (C++ enum variant: <span style='color: green;'>```PositiveBignum = 2```</span>)
    #[allow(non_upper_case_globals)]
    pub const PositiveBignum: crate::QCborKnownTags = crate::QCborKnownTags(2);
    /// A negative number of arbirary length, encoded as the absolute value of that number, minus one. For example, a byte array containing byte value 0x02 followed by 8 zero bytes represents the number -2<sup>65</sup> - 1. (C++ enum variant: <span style='color: green;'>```NegativeBignum = 3```</span>)
    #[allow(non_upper_case_globals)]
    pub const NegativeBignum: crate::QCborKnownTags = crate::QCborKnownTags(3);
    /// A decimal fraction, encoded as an array of two integers: the first is the exponent of the power of 10, the second the integral mantissa. The value 273.15 would be encoded as array <code>[-2, 27315]</code>. (C++ enum variant: <span style='color: green;'>```Decimal = 4```</span>)
    #[allow(non_upper_case_globals)]
    pub const Decimal: crate::QCborKnownTags = crate::QCborKnownTags(4);
    /// Similar to Decimal, but the exponent is a power of 2 instead. (C++ enum variant: <span style='color: green;'>```Bigfloat = 5```</span>)
    #[allow(non_upper_case_globals)]
    pub const Bigfloat: crate::QCborKnownTags = crate::QCborKnownTags(5);
    /// An <code>Encrypt0</code> map as specified by <a href="https://tools.ietf.org/html/rfc8152">RFC 8152</a> (CBOR Object Signing and Encryption). (C++ enum variant: <span style='color: green;'>```COSE_Encrypt0 = 16```</span>)
    #[allow(non_upper_case_globals)]
    pub const COSEEncrypt0: crate::QCborKnownTags = crate::QCborKnownTags(16);
    /// A <code>Mac0</code> map as specified by <a href="https://tools.ietf.org/html/rfc8152">RFC 8152</a> (CBOR Object Signing and Encryption). (C++ enum variant: <span style='color: green;'>```COSE_Mac0 = 17```</span>)
    #[allow(non_upper_case_globals)]
    pub const COSEMac0: crate::QCborKnownTags = crate::QCborKnownTags(17);
    /// A <code>Sign1</code> map as specified by <a href="https://tools.ietf.org/html/rfc8152">RFC 8152</a> (CBOR Object Signing and Encryption). (C++ enum variant: <span style='color: green;'>```COSE_Sign1 = 18```</span>)
    #[allow(non_upper_case_globals)]
    pub const COSESign1: crate::QCborKnownTags = crate::QCborKnownTags(18);
    /// Indicates that the byte array should be encoded using Base64url if the stream is converted to JSON. (C++ enum variant: <span style='color: green;'>```ExpectedBase64url = 21```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExpectedBase64URL: crate::QCborKnownTags = crate::QCborKnownTags(21);
    /// Indicates that the byte array should be encoded using Base64 if the stream is converted to JSON. (C++ enum variant: <span style='color: green;'>```ExpectedBase64 = 22```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExpectedBase64: crate::QCborKnownTags = crate::QCborKnownTags(22);
    /// Indicates that the byte array should be encoded using Base16 (hex) if the stream is converted to JSON. (C++ enum variant: <span style='color: green;'>```ExpectedBase16 = 23```</span>)
    #[allow(non_upper_case_globals)]
    pub const ExpectedBase16: crate::QCborKnownTags = crate::QCborKnownTags(23);
    /// Indicates that the byte array contains a CBOR stream. (C++ enum variant: <span style='color: green;'>```EncodedCbor = 24```</span>)
    #[allow(non_upper_case_globals)]
    pub const EncodedCbor: crate::QCborKnownTags = crate::QCborKnownTags(24);
    /// Indicates that the string contains a URL. (C++ enum variant: <span style='color: green;'>```Url = 32```</span>)
    #[allow(non_upper_case_globals)]
    pub const Url: crate::QCborKnownTags = crate::QCborKnownTags(32);
    /// Indicates that the string contains data encoded using Base64url. (C++ enum variant: <span style='color: green;'>```Base64url = 33```</span>)
    #[allow(non_upper_case_globals)]
    pub const Base64URL: crate::QCborKnownTags = crate::QCborKnownTags(33);
    /// Indicates that the string contains data encoded using Base64. (C++ enum variant: <span style='color: green;'>```Base64 = 34```</span>)
    #[allow(non_upper_case_globals)]
    pub const Base64: crate::QCborKnownTags = crate::QCborKnownTags(34);
    /// Indicates that the string contains a Perl-Compatible Regular Expression pattern. (C++ enum variant: <span style='color: green;'>```RegularExpression = 35```</span>)
    #[allow(non_upper_case_globals)]
    pub const RegularExpression: crate::QCborKnownTags = crate::QCborKnownTags(35);
    /// Indicates that the string contains a MIME message (according to <a href="https://tools.ietf.org/html/rfc2045">https://tools.ietf.org/html/rfc2045</a>){RFC 2045}. (C++ enum variant: <span style='color: green;'>```MimeMessage = 36```</span>)
    #[allow(non_upper_case_globals)]
    pub const MimeMessage: crate::QCborKnownTags = crate::QCborKnownTags(36);
    /// Indicates that the byte array contains a UUID. (C++ enum variant: <span style='color: green;'>```Uuid = 37```</span>)
    #[allow(non_upper_case_globals)]
    pub const Uuid: crate::QCborKnownTags = crate::QCborKnownTags(37);
    /// An <code>Encrypt</code> map as specified by <a href="https://tools.ietf.org/html/rfc8152">RFC 8152</a> (CBOR Object Signing and Encryption). (C++ enum variant: <span style='color: green;'>```COSE_Encrypt = 96```</span>)
    #[allow(non_upper_case_globals)]
    pub const COSEEncrypt: crate::QCborKnownTags = crate::QCborKnownTags(96);
    /// A <code>Mac</code> map as specified by <a href="https://tools.ietf.org/html/rfc8152">RFC 8152</a> (CBOR Object Signing and Encryption). (C++ enum variant: <span style='color: green;'>```COSE_Mac = 97```</span>)
    #[allow(non_upper_case_globals)]
    pub const COSEMac: crate::QCborKnownTags = crate::QCborKnownTags(97);
    /// A <code>Sign</code> map as specified by <a href="https://tools.ietf.org/html/rfc8152">RFC 8152</a> (CBOR Object Signing and Encryption). (C++ enum variant: <span style='color: green;'>```COSE_Sign = 98```</span>)
    #[allow(non_upper_case_globals)]
    pub const COSESign: crate::QCborKnownTags = crate::QCborKnownTags(98);
    /// No change in interpretation; this tag can be used as the outermost tag in a CBOR stream as the file header. (C++ enum variant: <span style='color: green;'>```Signature = 55799```</span>)
    #[allow(non_upper_case_globals)]
    pub const Signature: crate::QCborKnownTags = crate::QCborKnownTags(55799);
}

pub mod q_cbor_error {
    //! C++ type: <span style='color: green;'>```QCborError```</span>

    /// <p>This enum contains the possible error condition codes.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QCborError::Code```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborerror.html#Code-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum contains the possible error condition codes.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Code(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Code {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Code(value)
        }
    }

    impl From<Code> for ::std::os::raw::c_int {
        fn from(value: Code) -> Self {
            value.0
        }
    }

    impl Code {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Code {
        /// An unknown error occurred and no further details are available. (C++ enum variant: <span style='color: green;'>```UnknownError = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const UnknownError: crate::q_cbor_error::Code = crate::q_cbor_error::Code(1);
        /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#next">QCborStreamReader::next</a>() was called but there are no more elements in the current context. (C++ enum variant: <span style='color: green;'>```AdvancePastEnd = 3```</span>)
        #[allow(non_upper_case_globals)]
        pub const AdvancePastEnd: crate::q_cbor_error::Code = crate::q_cbor_error::Code(3);
        /// An I/O error with the <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> occurred. (C++ enum variant: <span style='color: green;'>```InputOutputError = 4```</span>)
        #[allow(non_upper_case_globals)]
        pub const InputOutputError: crate::q_cbor_error::Code = crate::q_cbor_error::Code(4);
        /// Data was found in the input stream after the last element. (C++ enum variant: <span style='color: green;'>```GarbageAtEnd = 256```</span>)
        #[allow(non_upper_case_globals)]
        pub const GarbageAtEnd: crate::q_cbor_error::Code = crate::q_cbor_error::Code(256);
        /// The end of the input stream was unexpectedly reached while processing an element. (C++ enum variant: <span style='color: green;'>```EndOfFile = 257```</span>)
        #[allow(non_upper_case_globals)]
        pub const EndOfFile: crate::q_cbor_error::Code = crate::q_cbor_error::Code(257);
        /// The CBOR stream contains a Break where it is not allowed (data is corrupt and the error is not recoverable). (C++ enum variant: <span style='color: green;'>```UnexpectedBreak = 258```</span>)
        #[allow(non_upper_case_globals)]
        pub const UnexpectedBreak: crate::q_cbor_error::Code = crate::q_cbor_error::Code(258);
        /// The CBOR stream contains an unknown/unparseable Type (data is corrupt and the and the error is not recoverable). (C++ enum variant: <span style='color: green;'>```UnknownType = 259```</span>)
        #[allow(non_upper_case_globals)]
        pub const UnknownType: crate::q_cbor_error::Code = crate::q_cbor_error::Code(259);
        /// The CBOR stream contains a known type in a position it is not allowed to exist (data is corrupt and the error is not recoverable). (C++ enum variant: <span style='color: green;'>```IllegalType = 260```</span>)
        #[allow(non_upper_case_globals)]
        pub const IllegalType: crate::q_cbor_error::Code = crate::q_cbor_error::Code(260);
        /// The CBOR stream appears to be encoding a number larger than 64-bit (data is corrupt and the error is not recoverable). (C++ enum variant: <span style='color: green;'>```IllegalNumber = 261```</span>)
        #[allow(non_upper_case_globals)]
        pub const IllegalNumber: crate::q_cbor_error::Code = crate::q_cbor_error::Code(261);
        /// The CBOR stream contains a Simple Type encoded incorrectly (data is corrupt and the error is not recoverable). (C++ enum variant: <span style='color: green;'>```IllegalSimpleType = 262```</span>)
        #[allow(non_upper_case_globals)]
        pub const IllegalSimpleType: crate::q_cbor_error::Code = crate::q_cbor_error::Code(262);
        /// The CBOR stream contains a text string that does not decode properly as UTF-8 (data is corrupt and the error is not recoverable). (C++ enum variant: <span style='color: green;'>```InvalidUtf8String = 516```</span>)
        #[allow(non_upper_case_globals)]
        pub const InvalidUtf8String: crate::q_cbor_error::Code = crate::q_cbor_error::Code(516);
        /// CBOR string, map or array is too big and cannot be parsed by Qt (internal limitation, but the error is not recoverable). (C++ enum variant: <span style='color: green;'>```DataTooLarge = 1024```</span>)
        #[allow(non_upper_case_globals)]
        pub const DataTooLarge: crate::q_cbor_error::Code = crate::q_cbor_error::Code(1024);
        /// Too many levels of arrays or maps encountered while processing the input (internal limitation, but the error is not recoverable). (C++ enum variant: <span style='color: green;'>```NestingTooDeep = 1025```</span>)
        #[allow(non_upper_case_globals)]
        pub const NestingTooDeep: crate::q_cbor_error::Code = crate::q_cbor_error::Code(1025);
        /// The CBOR stream contains a known type that the implementation does not support (internal limitation, but the error is not recoverable). (C++ enum variant: <span style='color: green;'>```UnsupportedType = 1026```</span>)
        #[allow(non_upper_case_globals)]
        pub const UnsupportedType: crate::q_cbor_error::Code = crate::q_cbor_error::Code(1026);
        /// No error was detected. (C++ enum variant: <span style='color: green;'>```NoError = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoError: crate::q_cbor_error::Code = crate::q_cbor_error::Code(0);
    }

}
/// C++ class: <span style='color: green;'>```QCborError```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qcborerror.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'></div>
#[repr(C)]
pub struct QCborError {
    _unused: u8,
}
impl QCborError {
    /// Returns the value of the <span style='color: green;'>```c```</span> field.
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn c(&self) -> crate::q_cbor_error::Code {
        crate::__ffi::ctr_qt_core_ffi_QCborError_c(self as *const crate::QCborError)
    }

    /// Calls C++ function: <span style='color: green;'>```QCborError& QCborError::operator=(const QCborError& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborerror.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborError>>,
    ) -> ::cpp_utils::MutRef<crate::QCborError> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborError_operator_(
            self as *mut crate::QCborError,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborError>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborError::QCborError()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborerror.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QCborError> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborError_QCborError();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborError::QCborError(const QCborError& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborerror.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborError>>,
    ) -> ::cpp_utils::CppBox<crate::QCborError> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborError_QCborError1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborError>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Sets the value of the <span style='color: green;'>```c```</span> field.
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn set_c(&mut self, value: crate::q_cbor_error::Code) {
        crate::__ffi::ctr_qt_core_ffi_set_QCborError_c(self as *mut crate::QCborError, value)
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborError_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```QCborError::Code QCborError::operator QCborError::Code() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_code(&self) -> crate::q_cbor_error::Code {
        crate::__ffi::ctr_qt_core_ffi_QCborError_operator_QCborError__Code(
            self as *const crate::QCborError,
        )
    }

    /// <p>Returns a text string that matches the error code in this <a href="http://doc.qt.io/qt-5/qcborerror.html">QCborError</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QCborError::toString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborerror.html#toString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a text string that matches the error code in this <a href="http://doc.qt.io/qt-5/qcborerror.html">QCborError</a> object.</p>
    /// <p>Note: the string is not translated. Applications whose interface allow users to parse CBOR streams need to provide their own, translated strings.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborerror.html#Code-enum">QCborError::Code</a>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_string(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborError_toString(self as *const crate::QCborError);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qcborparsererror.html">QCborParserError</a> is used by <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> to report a parsing error.</p>
///
/// C++ class: <span style='color: green;'>```QCborParserError```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qcborparsererror.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qcborparsererror.html">QCborParserError</a> is used by <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> to report a parsing error.</p>
/// <p>This class is used by <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromCbor-1">QCborValue::fromCbor</a>(const QByteArray &amp;ba, QCborParserError *error) to report a parser error and the byte offset where the error was detected.</p></div>
#[repr(C)]
pub struct QCborParserError {
    _unused: u8,
}
impl QCborParserError {
    /// <p>The <a href="http://doc.qt.io/qt-5/qcborparsererror.html">QCborParserError</a> is used by <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> to report a parsing error.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborParserError& QCborParserError::operator=(const QCborParserError& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborparsererror.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qcborparsererror.html">QCborParserError</a> is used by <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> to report a parsing error.</p>
    /// <p>This class is used by <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromCbor-1">QCborValue::fromCbor</a>(const QByteArray &amp;ba, QCborParserError *error) to report a parser error and the byte offset where the error was detected.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborParserError>>,
    ) -> ::cpp_utils::MutRef<crate::QCborParserError> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborParserError_operator_(
            self as *mut crate::QCborParserError,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborParserError>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Returns a reference to the <span style='color: green;'>```error```</span> field.
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn error(&self) -> ::cpp_utils::Ref<crate::QCborError> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborParserError_error(
            self as *const crate::QCborParserError,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Returns a mutable reference to the <span style='color: green;'>```error```</span> field.
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn error_mut(&mut self) -> ::cpp_utils::MutRef<crate::QCborError> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborParserError_error_mut(
            self as *mut crate::QCborParserError,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a string representation of the error code. This string is not translated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QCborParserError::errorString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborparsererror.html#errorString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string representation of the error code. This string is not translated.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborerror.html#toString">QCborError::toString</a>() and <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromCbor">QCborValue::fromCbor</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn error_string(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborParserError_errorString(
            self as *const crate::QCborParserError,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qcborparsererror.html">QCborParserError</a> is used by <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> to report a parsing error.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborParserError::QCborParserError()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborparsererror.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qcborparsererror.html">QCborParserError</a> is used by <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> to report a parsing error.</p>
    /// <p>This class is used by <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromCbor-1">QCborValue::fromCbor</a>(const QByteArray &amp;ba, QCborParserError *error) to report a parser error and the byte offset where the error was detected.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QCborParserError> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborParserError_QCborParserError();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qcborparsererror.html">QCborParserError</a> is used by <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> to report a parsing error.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborParserError::QCborParserError(const QCborParserError& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborparsererror.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qcborparsererror.html">QCborParserError</a> is used by <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> to report a parsing error.</p>
    /// <p>This class is used by <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromCbor-1">QCborValue::fromCbor</a>(const QByteArray &amp;ba, QCborParserError *error) to report a parser error and the byte offset where the error was detected.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborParserError>>,
    ) -> ::cpp_utils::CppBox<crate::QCborParserError> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborParserError_QCborParserError1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborParserError>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Returns the value of the <span style='color: green;'>```offset```</span> field.
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn offset(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QCborParserError_offset(
            self as *const crate::QCborParserError,
        )
    }

    /// Sets the value of the <span style='color: green;'>```error```</span> field.
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn set_error(
        &mut self,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborError>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_set_QCborParserError_error(
            self as *mut crate::QCborParserError,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborError>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// Sets the value of the <span style='color: green;'>```offset```</span> field.
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn set_offset(&mut self, value: i64) {
        crate::__ffi::ctr_qt_core_ffi_set_QCborParserError_offset(
            self as *mut crate::QCborParserError,
            value,
        )
    }
}

pub mod q_cbor_value {
    //! C++ type: <span style='color: green;'>```QCborValue```</span>

    /// <p>This enum is used in the options argument to <a href="http://doc.qt.io/qt-5/qcborvalue.html#toCbor">toCbor</a>(), modifying the behavior of the encoder.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QCborValue::EncodingOption```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#EncodingOption-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum is used in the options argument to <a href="http://doc.qt.io/qt-5/qcborvalue.html#toCbor">toCbor</a>(), modifying the behavior of the encoder.</p>
    ///
    /// <p>The use of <code>UseFloat16</code> is required to encode the stream in Canonical Format, but is not otherwise necessary.</p>
    /// <p>The EncodingOptions type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;EncodingOption&gt;. It stores an OR combination of EncodingOption values.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#toCbor">toCbor</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct EncodingOption(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for EncodingOption {
        fn from(value: ::std::os::raw::c_int) -> Self {
            EncodingOption(value)
        }
    }

    impl From<EncodingOption> for ::std::os::raw::c_int {
        fn from(value: EncodingOption) -> Self {
            value.0
        }
    }

    impl EncodingOption {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl EncodingOption {
        /// C++ enum variant: <span style='color: green;'>```SortKeysInMaps = 1```</span>
        #[allow(non_upper_case_globals)]
        pub const SortKeysInMaps: crate::q_cbor_value::EncodingOption =
            crate::q_cbor_value::EncodingOption(1);
        /// Tells the encoder to use IEEE 754 single-precision floating point (that is, <code>float</code>) whenever possible. (C++ enum variant: <span style='color: green;'>```UseFloat = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const UseFloat: crate::q_cbor_value::EncodingOption =
            crate::q_cbor_value::EncodingOption(2);
        /// Tells the encoder to use IEEE 754 half-precision floating point (that is, <code>qfloat16</code>), whenever possible. Implies <code>UseFloat</code>. (C++ enum variant: <span style='color: green;'>```UseFloat16 = 6```</span>)
        #[allow(non_upper_case_globals)]
        pub const UseFloat16: crate::q_cbor_value::EncodingOption =
            crate::q_cbor_value::EncodingOption(6);
        /// Tells the encoder to use integers whenever a value of type <a href="http://doc.qt.io/qt-5/qcborvalue.html#Type-enum">Double</a> contains an integer. (C++ enum variant: <span style='color: green;'>```UseIntegers = 8```</span>)
        #[allow(non_upper_case_globals)]
        pub const UseIntegers: crate::q_cbor_value::EncodingOption =
            crate::q_cbor_value::EncodingOption(8);
        /// (Default) Performs no transformations. (C++ enum variant: <span style='color: green;'>```NoTransformation = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const NoTransformation: crate::q_cbor_value::EncodingOption =
            crate::q_cbor_value::EncodingOption(0);
    }

    /// <p>This enum is used in the option argument to <a href="http://doc.qt.io/qt-5/qcborvalue.html#toDiagnosticNotation">toDiagnosticNotation</a>(), to modify the output format.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QCborValue::DiagnosticNotationOption```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#DiagnosticNotationOption-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum is used in the option argument to <a href="http://doc.qt.io/qt-5/qcborvalue.html#toDiagnosticNotation">toDiagnosticNotation</a>(), to modify the output format.</p>
    ///
    /// <p>Currently, <code>ExtendedFormat</code> will change how byte arrays are represented. Without it, they are always hex-encoded and without spaces. With it, <a href="http://doc.qt.io/qt-5/qcborvalue.html#toCbor">QCborValue::toCbor</a>() will either use hex with spaces, base64 or base64url encoding, depending on the context.</p>
    /// <p>The DiagnosticNotationOptions type is a typedef for <a href="http://doc.qt.io/qt-5/qflags.html">QFlags</a>&lt;DiagnosticNotationOption&gt;. It stores an OR combination of DiagnosticNotationOption values.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#toDiagnosticNotation">toDiagnosticNotation</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct DiagnosticNotationOption(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for DiagnosticNotationOption {
        fn from(value: ::std::os::raw::c_int) -> Self {
            DiagnosticNotationOption(value)
        }
    }

    impl From<DiagnosticNotationOption> for ::std::os::raw::c_int {
        fn from(value: DiagnosticNotationOption) -> Self {
            value.0
        }
    }

    impl DiagnosticNotationOption {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl DiagnosticNotationOption {
        /// Does not use any line-breaks, producing a compact representation. (C++ enum variant: <span style='color: green;'>```Compact = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const Compact: crate::q_cbor_value::DiagnosticNotationOption =
            crate::q_cbor_value::DiagnosticNotationOption(0);
        /// Uses line-breaks, one <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> per line. (C++ enum variant: <span style='color: green;'>```LineWrapped = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const LineWrapped: crate::q_cbor_value::DiagnosticNotationOption =
            crate::q_cbor_value::DiagnosticNotationOption(1);
        /// Uses some different options to represent values, not found in RFC 7049. Those options are subject to change. (C++ enum variant: <span style='color: green;'>```ExtendedFormat = 2```</span>)
        #[allow(non_upper_case_globals)]
        pub const ExtendedFormat: crate::q_cbor_value::DiagnosticNotationOption =
            crate::q_cbor_value::DiagnosticNotationOption(2);
    }

    /// <p>This enum represents the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> type. It is returned by the <a href="http://doc.qt.io/qt-5/qcborvalue.html#type">type</a>() function.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QCborValue::Type```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#Type-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum represents the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> type. It is returned by the <a href="http://doc.qt.io/qt-5/qcborvalue.html#type">type</a>() function.</p>
    /// <p>The CBOR built-in types are:</p>
    ///
    /// <p>Additionally, <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> can represent extended types:</p>
    ///
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#type">type</a>().</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Type(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Type {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Type(value)
        }
    }

    impl From<Type> for ::std::os::raw::c_int {
        fn from(value: Type) -> Self {
            value.0
        }
    }

    impl Type {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Type {
        /// <code>qint64</code>: An integer value (C++ enum variant: <span style='color: green;'>```Integer = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const Integer: crate::q_cbor_value::Type = crate::q_cbor_value::Type(0);
        /// <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>: a byte array ("byte string") (C++ enum variant: <span style='color: green;'>```ByteArray = 64```</span>)
        #[allow(non_upper_case_globals)]
        pub const ByteArray: crate::q_cbor_value::Type = crate::q_cbor_value::Type(64);
        /// <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>: a Unicode string ("text string") (C++ enum variant: <span style='color: green;'>```String = 96```</span>)
        #[allow(non_upper_case_globals)]
        pub const String: crate::q_cbor_value::Type = crate::q_cbor_value::Type(96);
        /// <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>: an array of QCborValues (C++ enum variant: <span style='color: green;'>```Array = 128```</span>)
        #[allow(non_upper_case_globals)]
        pub const Array: crate::q_cbor_value::Type = crate::q_cbor_value::Type(128);
        /// <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>: an associative container of QCborValues (C++ enum variant: <span style='color: green;'>```Map = 160```</span>)
        #[allow(non_upper_case_globals)]
        pub const Map: crate::q_cbor_value::Type = crate::q_cbor_value::Type(160);
        /// An unknown or unrecognized extended type, represented by its tag (a <a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborTag-enum">QCborTag</a>) and the tagged value (a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>) (C++ enum variant: <span style='color: green;'>```Tag = 192```</span>)
        #[allow(non_upper_case_globals)]
        pub const Tag: crate::q_cbor_value::Type = crate::q_cbor_value::Type(192);
        /// <a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborSimpleType-enum">QCborSimpleType</a>: one of several simple types/values (C++ enum variant: <span style='color: green;'>```SimpleType = 256```</span>)
        #[allow(non_upper_case_globals)]
        pub const SimpleType: crate::q_cbor_value::Type = crate::q_cbor_value::Type(256);
        /// <code>bool</code>: the simple type for value <code>false</code> (C++ enum variant: <span style='color: green;'>```False = 276```</span>)
        #[allow(non_upper_case_globals)]
        pub const False: crate::q_cbor_value::Type = crate::q_cbor_value::Type(276);
        /// <code>bool</code>: the simple type for value <code>true</code> (C++ enum variant: <span style='color: green;'>```True = 277```</span>)
        #[allow(non_upper_case_globals)]
        pub const True: crate::q_cbor_value::Type = crate::q_cbor_value::Type(277);
        /// <code>std::nullptr_t</code>: the simple type for the null value (C++ enum variant: <span style='color: green;'>```Null = 278```</span>)
        #[allow(non_upper_case_globals)]
        pub const Null: crate::q_cbor_value::Type = crate::q_cbor_value::Type(278);
        /// (no type) the simple type for the undefined value (C++ enum variant: <span style='color: green;'>```Undefined = 279```</span>)
        #[allow(non_upper_case_globals)]
        pub const Undefined: crate::q_cbor_value::Type = crate::q_cbor_value::Type(279);
        /// <code>double</code>: a double-precision floating point (C++ enum variant: <span style='color: green;'>```Double = 514```</span>)
        #[allow(non_upper_case_globals)]
        pub const Double: crate::q_cbor_value::Type = crate::q_cbor_value::Type(514);
        /// <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>: a date and time stamp (C++ enum variant: <span style='color: green;'>```DateTime = 65536```</span>)
        #[allow(non_upper_case_globals)]
        pub const DateTime: crate::q_cbor_value::Type = crate::q_cbor_value::Type(65536);
        /// <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a>: a URL or URI (C++ enum variant: <span style='color: green;'>```Url = 65568```</span>)
        #[allow(non_upper_case_globals)]
        pub const Url: crate::q_cbor_value::Type = crate::q_cbor_value::Type(65568);
        /// <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a>: the pattern of a regular expression (C++ enum variant: <span style='color: green;'>```RegularExpression = 65571```</span>)
        #[allow(non_upper_case_globals)]
        pub const RegularExpression: crate::q_cbor_value::Type = crate::q_cbor_value::Type(65571);
        /// <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>: a UUID (C++ enum variant: <span style='color: green;'>```Uuid = 65573```</span>)
        #[allow(non_upper_case_globals)]
        pub const Uuid: crate::q_cbor_value::Type = crate::q_cbor_value::Type(65573);
        /// Not a valid value, this usually indicates a CBOR decoding error (C++ enum variant: <span style='color: green;'>```Invalid = -1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Invalid: crate::q_cbor_value::Type = crate::q_cbor_value::Type(-1);
    }

    impl From<crate::q_cbor_value::EncodingOption>
        for crate::QFlags<crate::q_cbor_value::EncodingOption>
    {
        fn from(value: crate::q_cbor_value::EncodingOption) -> Self {
            Self::from(value.to_int())
        }
    }

    impl<T: Into<crate::QFlags<crate::q_cbor_value::EncodingOption>>> std::ops::BitOr<T>
        for crate::q_cbor_value::EncodingOption
    {
        type Output = crate::QFlags<crate::q_cbor_value::EncodingOption>;
        fn bitor(self, rhs: T) -> crate::QFlags<crate::q_cbor_value::EncodingOption> {
            Into::<crate::QFlags<crate::q_cbor_value::EncodingOption>>::into(self) | rhs
        }
    }

    impl From<crate::q_cbor_value::DiagnosticNotationOption>
        for crate::QFlags<crate::q_cbor_value::DiagnosticNotationOption>
    {
        fn from(value: crate::q_cbor_value::DiagnosticNotationOption) -> Self {
            Self::from(value.to_int())
        }
    }

    impl<T: Into<crate::QFlags<crate::q_cbor_value::DiagnosticNotationOption>>> std::ops::BitOr<T>
        for crate::q_cbor_value::DiagnosticNotationOption
    {
        type Output = crate::QFlags<crate::q_cbor_value::DiagnosticNotationOption>;
        fn bitor(self, rhs: T) -> crate::QFlags<crate::q_cbor_value::DiagnosticNotationOption> {
            Into::<crate::QFlags<crate::q_cbor_value::DiagnosticNotationOption>>::into(self) | rhs
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> class encapsulates a value in CBOR.</p>
///
/// C++ class: <span style='color: green;'>```QCborValue```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qcborvalue.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> class encapsulates a value in CBOR.</p>
/// <p>This class can be used to hold one of the many types available in CBOR. CBOR is the Concise Binary Object Representation, a very compact form of binary data encoding that is a superset of JSON. It was created by the IETF Constrained RESTful Environments (CoRE) WG, which has used it in many new RFCs. It is meant to be used alongside the <a href="https://tools.ietf.org/html/rfc7252">CoAP protocol</a>.</p>
/// <p>CBOR has three groups of built-in types:</p>
/// <ul>
/// <li>Basic types: integers, floating point (double), boolean, null, etc.</li>
/// <li>String-like types: strings and byte arrays</li>
/// <li>Containers: arrays and maps</li>
/// </ul>
/// <p>Additionally, CBOR supports a form of type extensibility by associating a "tag" to one of the above types to convey more information. For example, a UUID is represented by a tag and a byte array containing the 16 bytes of the UUID content. <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> supports creating and decoding several of those extended types directly with Qt classes (like <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>).</p>
/// <p>For the complete list, see <a href="http://doc.qt.io/qt-5/qcborvalue.html#Type-enum">QCborValue::Type</a>. The type of a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> can be queried using <a href="http://doc.qt.io/qt-5/qcborvalue.html#type">type</a>() or one of the "isXxxx" functions.</p>
/// <a name="extended-types-and-tagged-values"></a></div>
#[repr(C)]
pub struct QCborValue {
    _unused: u8,
}
impl QCborValue {
    /// <p>Compares this value and <i>other</i>, and returns an integer that indicates whether this value should be sorted prior to (if the result is negative) or after <i>other</i> (if the result is positive). If this function returns 0, the two values are equal and hold the same contents.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QCborValue::compare(const QCborValue& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#compare">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares this value and <i>other</i>, and returns an integer that indicates whether this value should be sorted prior to (if the result is negative) or after <i>other</i> (if the result is positive). If this function returns 0, the two values are equal and hold the same contents.</p>
    /// <p>If each <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> contains an array or map, the comparison is recursive to elements contained in them.</p>
    /// <a name="extended-types"></a>
    /// <h6>Extended types</h6>
    /// <p><a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> compares equal a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> containing an extended type, like <a href="http://doc.qt.io/qt-5/qcborvalue.html#Type-enum">Url</a> and <a href="http://doc.qt.io/qt-5/qcborvalue.html#Type-enum">Url</a> and its equivalent tagged representation. So, for example, the following expression is true:</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcborvalue.html#QCborValue">QCborValue</a></span>(<span class="type"><a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a></span>(<span class="string">"https://example.com"</span>)) <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcborvalue.html#QCborValue">QCborValue</a></span>(<span class="type"><a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborKnownTags-enum">QCborKnownTags</a></span><span class="operator">::</span>Url<span class="operator">,</span> <span class="string">"https://example.com"</span>);
    ///
    /// </pre>
    /// <p>Do note that Qt types like <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a> and <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a> will normalize and otherwise modify their arguments. The expression above is true only because the string on the right side is the normalized value that the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> on the left would take. If, for example, the "https" part were uppercase in both sides, the comparison would fail. For information on normalizations performed by <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, please consult the documentation of the constructor taking the Qt type in question.</p>
    /// <a name="sorting-order"></a>
    /// <h6>Sorting order</h6>
    /// <p>Sorting order in CBOR is defined in RFC 7049 {https://tools.ietf.org/html/rfc7049#section-3.9}{section 3.9}, which discusses the sorting of keys in a map when following the Canonical encoding. According to the specification, "sorting is performed on the bytes of the representation of the key data items" and lists as consequences that:</p>
    /// <ul>
    /// <li>"If two keys have different lengths, the shorter one sorts earlier;"</li>
    /// <li>"If two keys have the same length, the one with the lower value in (byte-wise) lexical order sorts earlier."</li>
    /// </ul>
    /// <p>This results in surprising sorting of QCborValues, where the result of this function is different from that which would later be retrieved by comparing the contained elements. For example, the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> containing string "zzz" sorts before the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> with string "foobar", even though when comparing as <a href="http://doc.qt.io/qt-5/qstring.html#compare">QStrings</a> or <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArrays</a> the "zzz" sorts after "foobar" (dictionary order).</p>
    /// <p>The specification does not clearly indicate what sorting order should be done for values of different types (it says sorting should not pay "attention to the 3/5 bit splitting for major types"). <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> makes the assumption that types should be sorted too. The numeric values of the <a href="http://doc.qt.io/qt-5/qcborvalue.html#Type-enum">QCborValue::Type</a> enumeration are in that order, with the exception of the extended types, which compare as their tagged equivalents.</p>
    /// <p><b>Note: </b>Sorting order is preliminary and is subject to change. Applications should not depend on the order returned by this function for the time being.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#compare">QCborArray::compare</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#compare">QCborMap::compare</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#operator-eq-eq">operator==</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn compare(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_compare(
            self as *const crate::QCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Replaces the contents of this QCborObject with a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue& QCborValue::operator=(const QCborValue& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the contents of this QCborObject with a copy of <i>other</i>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::cpp_utils::MutRef<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_operator_(
            self as *mut crate::QCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Decodes one item from the CBOR stream found in <i>reader</i> and returns the equivalent representation. This function is recursive: if the item is a map or array, it will decode all items found in that map or array, until the outermost object is finished.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QCborValue QCborValue::fromCbor(QCborStreamReader& reader)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromCbor">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Decodes one item from the CBOR stream found in <i>reader</i> and returns the equivalent representation. This function is recursive: if the item is a map or array, it will decode all items found in that map or array, until the outermost object is finished.</p>
    /// <p>This function need not be used on the root element of a <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a>. For example, the following code illustrates how to skip the CBOR signature tag from the beginning of a file:</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="keyword">if</span> (reader<span class="operator">.</span>isTag() <span class="operator">&amp;</span><span class="operator">&amp;</span> reader<span class="operator">.</span>toTag() <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborKnownTags-enum">QCborKnownTags</a></span><span class="operator">::</span>Signature)
    /// &#32;   &#32;     reader<span class="operator">.</span>next();
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcborvalue.html#QCborValue">QCborValue</a></span> contents <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcborvalue.html#QCborValue">QCborValue</a></span><span class="operator">::</span>fromCbor(reader);
    ///
    /// </pre>
    /// <p>The returned value may be partially complete and indistinguishable from a valid <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> even if the decoding failed. To determine if there was an error, check if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#lastError">reader.lastError()</a> is indicating an error condition. This function stops decoding immediately after the first error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#toCbor">toCbor</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#toDiagnosticNotation">toDiagnosticNotation</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#toVariant">toVariant</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toJsonValue">toJsonValue</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_cbor_q_cbor_stream_reader(
        reader: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QCborStreamReader>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_fromCbor(
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QCborStreamReader>>::cast_into(
                reader,
            )
            .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QCborValue QCborValue::fromCbor(const QByteArray& ba, QCborParserError* error = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromCbor-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Decodes one item from the CBOR stream found in the byte array <i>ba</i> and returns the equivalent representation. This function is recursive: if the item is a map or array, it will decode all items found in that map or array, until the outermost object is finished.</p>
    /// <p>This function stores the error state, if any, in the object pointed to by <i>error</i>, along with the offset of where the error occurred. If no error happened, it stores <a href="http://doc.qt.io/qt-5/qtcborcommon.html">NoError</a> in the error state and the number of bytes that it consumed (that is, it stores the offset for the first unused byte). Using that information makes it possible to parse further data that may exist in the same byte array.</p>
    /// <p>The returned value may be partially complete and indistinguishable from a valid <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> even if the decoding failed. To determine if there was an error, check if there was an error stored in <i>error</i>. This function stops decoding immediately after the first error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#toCbor">toCbor</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#toDiagnosticNotation">toDiagnosticNotation</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#toVariant">toVariant</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toJsonValue">toJsonValue</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_cbor_q_byte_array_q_cbor_parser_error(
        ba: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
        error: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QCborParserError>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_fromCbor1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(ba)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QCborParserError>>::cast_into(error)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QCborValue QCborValue::fromCbor(const char* data, long long len, QCborParserError* error = …)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_cbor_char_longlong_q_cbor_parser_error(
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        len: ::std::os::raw::c_longlong,
        error: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QCborParserError>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_fromCbor2(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(data)
                .as_raw_ptr(),
            len,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QCborParserError>>::cast_into(error)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QCborValue QCborValue::fromCbor(const quint8* data, long long len, QCborParserError* error = …)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_cbor_u8_longlong_q_cbor_parser_error(
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<u8>>,
        len: ::std::os::raw::c_longlong,
        error: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QCborParserError>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_fromCbor3(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<u8>>::cast_into(data).as_raw_ptr(),
            len,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QCborParserError>>::cast_into(error)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QCborValue QCborValue::fromCbor(const QByteArray& ba)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromCbor-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Decodes one item from the CBOR stream found in the byte array <i>ba</i> and returns the equivalent representation. This function is recursive: if the item is a map or array, it will decode all items found in that map or array, until the outermost object is finished.</p>
    /// <p>This function stores the error state, if any, in the object pointed to by <i>error</i>, along with the offset of where the error occurred. If no error happened, it stores <a href="http://doc.qt.io/qt-5/qtcborcommon.html">NoError</a> in the error state and the number of bytes that it consumed (that is, it stores the offset for the first unused byte). Using that information makes it possible to parse further data that may exist in the same byte array.</p>
    /// <p>The returned value may be partially complete and indistinguishable from a valid <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> even if the decoding failed. To determine if there was an error, check if there was an error stored in <i>error</i>. This function stops decoding immediately after the first error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#toCbor">toCbor</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#toDiagnosticNotation">toDiagnosticNotation</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#toVariant">toVariant</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toJsonValue">toJsonValue</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_cbor_q_byte_array(
        ba: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_fromCbor4(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(ba)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QCborValue QCborValue::fromCbor(const char* data, long long len)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_cbor_char_longlong(
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        len: ::std::os::raw::c_longlong,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_fromCbor5(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(data)
                .as_raw_ptr(),
            len,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QCborValue QCborValue::fromCbor(const quint8* data, long long len)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_cbor_u8_longlong(
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<u8>>,
        len: ::std::os::raw::c_longlong,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_fromCbor6(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<u8>>::cast_into(data).as_raw_ptr(),
            len,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts the JSON value contained in <i>v</i> into its corresponding CBOR value and returns it. There is no data loss in converting from JSON to CBOR, as the CBOR type set is richer than JSON's. Additionally, values converted to CBOR using this function can be converted back to JSON using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toJsonValue">toJsonValue</a>() with no data loss.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QCborValue QCborValue::fromJsonValue(const QJsonValue& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromJsonValue">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the JSON value contained in <i>v</i> into its corresponding CBOR value and returns it. There is no data loss in converting from JSON to CBOR, as the CBOR type set is richer than JSON's. Additionally, values converted to CBOR using this function can be converted back to JSON using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toJsonValue">toJsonValue</a>() with no data loss.</p>
    /// <p>The following table lists the mapping of JSON types to CBOR types:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>JSON Type</th><th>CBOR Type</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>Bool</td><td>Bool</td></tr>
    /// <tr valign="top" class="even"><td>Number</td><td>Integer (if the number has no fraction and is in the <a href="http://doc.qt.io/qt-5/qtglobal.html#qint64-typedef">qint64</a> range) or Double</td></tr>
    /// <tr valign="top" class="odd"><td>String</td><td>String</td></tr>
    /// <tr valign="top" class="even"><td>Array</td><td>Array</td></tr>
    /// <tr valign="top" class="odd"><td>Object</td><td>Map</td></tr>
    /// <tr valign="top" class="even"><td>Null</td><td>Null</td></tr>
    /// </tbody></table></div>
    /// <p><a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> can also be undefined, indicating a previous operation that failed to complete (for example, searching for a key not present in an object). Undefined values are not JSON types and may not appear in JSON arrays and objects, but this function does return the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> undefined value if the corresponding <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> is undefined.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#toJsonValue">toJsonValue</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromVariant">fromVariant</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#fromJsonArray">QCborArray::fromJsonArray</a>(), and <a href="http://doc.qt.io/qt-5/qcbormap.html#fromJsonObject">QCborMap::fromJsonObject</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_json_value(
        v: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonValue>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_fromJsonValue(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonValue>>::cast_into(v).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts the <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> <i>variant</i> into <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QCborValue QCborValue::fromVariant(const QVariant& variant)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromVariant">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> <i>variant</i> into <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> and returns it.</p>
    /// <p>QVariants may contain a large list of different meta types, many of which have no corresponding representation in CBOR. That includes all user-defined meta types. When preparing transmission using CBOR, it is suggested to encode carefully each value to prevent loss of representation.</p>
    /// <p>The following table lists the conversion this function will apply:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Qt (C++) type</th><th>CBOR type</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>invalid (QVariant())</td><td>Undefined</td></tr>
    /// <tr valign="top" class="even"><td><code>bool</code></td><td>Bool</td></tr>
    /// <tr valign="top" class="odd"><td><code>std::nullptr_t</code></td><td>Null</td></tr>
    /// <tr valign="top" class="even"><td><code>short</code>, <code>ushort</code>, <code>int</code>, <code>uint</code>, <a href="http://doc.qt.io/qt-5/qtglobal.html#qint64-typedef">qint64</a></td><td>Integer</td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qtglobal.html#quint64-typedef">quint64</a></td><td>Integer, but they are cast to <code>qint64</code> first so values higher than 2<sup>63</sup>-1 (<code>INT64_MAX</code>) will be wrapped to negative</td></tr>
    /// <tr valign="top" class="even"><td><code>float</code>, <code>double</code></td><td>Double</td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></td><td><a href="http://doc.qt.io/qt-5/qcborvalue.html#Type-enum">ByteArray</a></td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a></td><td><a href="http://doc.qt.io/qt-5/qcborvalue.html#Type-enum">DateTime</a></td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborSimpleType-enum">QCborSimpleType</a></td><td>Simple type</td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a></td><td>Array, converted using QCborArray::formJsonArray()</td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qjsondocument.html">QJsonDocument</a></td><td>Array or Map</td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a></td><td>Map, converted using <a href="http://doc.qt.io/qt-5/qcbormap.html#fromJsonObject">QCborMap::fromJsonObject</a>()</td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a></td><td>converted using <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromJsonValue">fromJsonValue</a>()</td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a></td><td><a href="http://doc.qt.io/qt-5/qcborvalue.html#Type-enum">RegularExpression</a></td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></td><td>String</td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></td><td>Array</td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qvariant.html#QVariantHash-typedef">QVariantHash</a></td><td>Map</td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a></td><td>Array</td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qvariant.html#QVariantMap-typedef">QVariantMap</a></td><td>Map</td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a></td><td>Url</td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a></td><td>Uuid</td></tr>
    /// </tbody></table></div>
    /// <p>For any other types, this function will return Null if the <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> itself is null, and otherwise will try to convert to string using <a href="http://doc.qt.io/qt-5/qvariant.html#toString">QVariant::toString</a>(). If the conversion to string fails, this function returns Undefined.</p>
    /// <p>Please note that the conversions via <a href="http://doc.qt.io/qt-5/qvariant.html#toString">QVariant::toString</a>() are subject to change at any time. <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> may be extended in the future to support more types, which will result in a change in how this function performs conversions.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#toVariant">toVariant</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromJsonValue">fromJsonValue</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#toVariantList">QCborArray::toVariantList</a>(), and <a href="http://doc.qt.io/qt-5/qcbormap.html#toVariantMap">QCborMap::toVariantMap</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_variant(
        variant: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_fromVariant(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(variant)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>If this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>, searches elements for the value whose key matches <i>key</i>. If there's no key matching <i>key</i> in the map or if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object is not a map, returns the undefined value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborValue::operator[](const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>, searches elements for the value whose key matches <i>key</i>. If there's no key matching <i>key</i> in the map or if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object is not a map, returns the undefined value.</p>
    /// <p>This function is equivalent to:</p>
    /// <pre class="cpp">
    ///
    /// &#32;     value<span class="operator">.</span>toMap()<span class="operator">.</span>value(key);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#operator-5b-5d-2">operator[]</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d">QCborMap::operator[]</a>, <a href="http://doc.qt.io/qt-5/qcbormap.html#value">QCborMap::value</a>(), and <a href="http://doc.qt.io/qt-5/qcbormap.html#find">QCborMap::find</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn index_q_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_operator__(
            self as *const crate::QCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborValue::operator[](QLatin1String key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>, searches elements for the value whose key matches <i>key</i>. If there's no key matching <i>key</i> in the map or if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object is not a map, returns the undefined value.</p>
    /// <p>This function is equivalent to:</p>
    /// <pre class="cpp">
    ///
    /// &#32;     value<span class="operator">.</span>toMap()<span class="operator">.</span>value(key);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#operator-5b-5d-2">operator[]</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d">QCborMap::operator[]</a>, <a href="http://doc.qt.io/qt-5/qcbormap.html#value">QCborMap::value</a>(), and <a href="http://doc.qt.io/qt-5/qcbormap.html#find">QCborMap::find</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn index_q_latin1_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_operator__1(
            self as *const crate::QCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>If this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>, searches elements for the value whose key matches <i>key</i>. If this is an array, returns the element whose index is <i>key</i>. If there's no matching value in the array or map, or if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object is not an array or map, returns the undefined value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborValue::operator[](qint64 key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#operator-5b-5d-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>, searches elements for the value whose key matches <i>key</i>. If this is an array, returns the element whose index is <i>key</i>. If there's no matching value in the array or map, or if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object is not an array or map, returns the undefined value.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#operator-5b-5d">operator[]</a>, <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d">QCborMap::operator[]</a>, <a href="http://doc.qt.io/qt-5/qcbormap.html#value">QCborMap::value</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#find">QCborMap::find</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-5b-5d">QCborArray::operator[]</a>, and <a href="http://doc.qt.io/qt-5/qcborarray.html#at">QCborArray::at</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn index_i64(&self, key: i64) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_operator__2(
            self as *const crate::QCborValue,
            key,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValueRef QCborValue::operator[](qint64 key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#operator-5b-5d-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> that can be used to read or modify the entry in this, as a map or array, with the given <i>key</i>. When this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> or, for 0 &lt;= key &lt; 0x10000, a <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>, this function is equivalent to the matching operator[] on that map or array.</p>
    /// <p>Before returning the reference: if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> was an array but the key is out of range, the array is first converted to a map (so that <code>map[i]</code> is <code>array[i]</code> for each index, <code>i</code>, with valid <code>array[i]</code>); otherwise, if it was not a map it will be over-written with an empty map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#operator-5b-5d">operator[]</a>, <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d">QCborMap::operator[]</a>, <a href="http://doc.qt.io/qt-5/qcbormap.html#value">QCborMap::value</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#find">QCborMap::find</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-5b-5d">QCborArray::operator[]</a>, and <a href="http://doc.qt.io/qt-5/qcborarray.html#at">QCborArray::at</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn index_i64_mut(&mut self, key: i64) -> ::cpp_utils::CppBox<crate::QCborValueRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_operator__10(
            self as *mut crate::QCborValue,
            key,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValueRef QCborValue::operator[](QLatin1String key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#operator-5b-5d-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> that can be used to read or modify the entry in this, as a map, with the given <i>key</i>. When this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>, this function is equivalent to the matching operator[] on that map.</p>
    /// <p>Before returning the reference: if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> was an array, it is first converted to a map (so that <code>map[i]</code> is <code>array[i]</code> for each index, <code>i</code>, with valid <code>array[i]</code>); otherwise, if it was not a map it will be over-written with an empty map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#operator-5b-5d-2">operator[]</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d">QCborMap::operator[]</a>, <a href="http://doc.qt.io/qt-5/qcbormap.html#value">QCborMap::value</a>(), and <a href="http://doc.qt.io/qt-5/qcbormap.html#find">QCborMap::find</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn index_q_latin1_string_mut(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValueRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_operator__11(
            self as *mut crate::QCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> that can be used to read or modify the entry in this, as a map, with the given <i>key</i>. When this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>, this function is equivalent to the matching operator[] on that map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValueRef QCborValue::operator[](const QString& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#operator-5b-5d-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> that can be used to read or modify the entry in this, as a map, with the given <i>key</i>. When this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>, this function is equivalent to the matching operator[] on that map.</p>
    /// <p>Before returning the reference: if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> was an array, it is first converted to a map (so that <code>map[i]</code> is <code>array[i]</code> for each index, <code>i</code>, with valid <code>array[i]</code>); otherwise, if it was not a map it will be over-written with an empty map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#operator-5b-5d-2">operator[]</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d">QCborMap::operator[]</a>, <a href="http://doc.qt.io/qt-5/qcbormap.html#value">QCborMap::value</a>(), and <a href="http://doc.qt.io/qt-5/qcbormap.html#find">QCborMap::find</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn index_q_string_mut(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValueRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_operator__12(
            self as *mut crate::QCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of the array type. The array value can be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toArray">toArray</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborValue::isArray() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#isArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of the array type. The array value can be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toArray">toArray</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#type">type</a>() and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toArray">toArray</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_array(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_isArray(self as *const crate::QCborValue)
    }

    /// <p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is a boolean. The value can be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toBool">toBool</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborValue::isBool() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#isBool">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is a boolean. The value can be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toBool">toBool</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#toBool">toBool</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isTrue">isTrue</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#isFalse">isFalse</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_bool(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_isBool(self as *const crate::QCborValue)
    }

    /// <p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of the byte array type. The byte array value can be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toByteArray">toByteArray</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborValue::isByteArray() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#isByteArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of the byte array type. The byte array value can be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toByteArray">toByteArray</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#type">type</a>() and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toByteArray">toByteArray</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_byte_array(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_isByteArray(self as *const crate::QCborValue)
    }

    /// <p>This convenience function returns true if the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is either an array or a map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborValue::isContainer() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#isContainer">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This convenience function returns true if the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is either an array or a map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isArray">isArray</a>() and <a href="http://doc.qt.io/qt-5/qcborvalue.html#isMap">isMap</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_container(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_isContainer(self as *const crate::QCborValue)
    }

    /// <p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of the date/time type. The value can be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toDateTime">toDateTime</a>(). Date/times are extended types that use the tag <a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborKnownTags-enum">DateTime</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborValue::isDateTime() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#isDateTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of the date/time type. The value can be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toDateTime">toDateTime</a>(). Date/times are extended types that use the tag <a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborKnownTags-enum">DateTime</a>.</p>
    /// <p>Additionally, when decoding from a CBOR stream, <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> will interpret tags of value <a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborKnownTags-enum">UnixTime_t</a> and convert them to the equivalent date/time.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#type">type</a>() and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toDateTime">toDateTime</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_date_time(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_isDateTime(self as *const crate::QCborValue)
    }

    /// <p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of the floating-point type. The value can be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toDouble">toDouble</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborValue::isDouble() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#isDouble">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of the floating-point type. The value can be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toDouble">toDouble</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#type">type</a>() and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toDouble">toDouble</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_double(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_isDouble(self as *const crate::QCborValue)
    }

    /// <p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is a boolean with false value. This function exists because, internally, CBOR booleans are stored as two separate types, one for true and one for false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborValue::isFalse() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#isFalse">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is a boolean with false value. This function exists because, internally, CBOR booleans are stored as two separate types, one for true and one for false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isBool">isBool</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isTrue">isTrue</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toBool">toBool</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_false(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_isFalse(self as *const crate::QCborValue)
    }

    /// <p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of the integer type. The integer value can be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toInteger">toInteger</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborValue::isInteger() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#isInteger">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of the integer type. The integer value can be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toInteger">toInteger</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#type">type</a>() and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toInteger">toInteger</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_integer(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_isInteger(self as *const crate::QCborValue)
    }

    /// <p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is not of any valid type. Invalid QCborValues are distinct from those with undefined values and they usually represent a decoding error.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborValue::isInvalid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#isInvalid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is not of any valid type. Invalid QCborValues are distinct from those with undefined values and they usually represent a decoding error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isUndefined">isUndefined</a>() and <a href="http://doc.qt.io/qt-5/qcborvalue.html#isNull">isNull</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_invalid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_isInvalid(self as *const crate::QCborValue)
    }

    /// <p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of the map type. The map value can be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toMap">toMap</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborValue::isMap() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#isMap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of the map type. The map value can be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toMap">toMap</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#type">type</a>() and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toMap">toMap</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_map(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_isMap(self as *const crate::QCborValue)
    }

    /// <p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of the null type.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborValue::isNull() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#isNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of the null type.</p>
    /// <p>CBOR null values are used to indicate optional values that were not provided. They are distinct from undefined values, in that null values are usually not the result of an earlier error or problem.</p>
    /// <p>Null values are distinct from undefined values and from invalid <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> objects. The API will not create invalid QCborValues, but they may exist as a result of a parsing error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isUndefined">isUndefined</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#isInvalid">isInvalid</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_isNull(self as *const crate::QCborValue)
    }

    /// <p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> contains a regular expression's pattern. The pattern can be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toRegularExpression">toRegularExpression</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborValue::isRegularExpression() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#isRegularExpression">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> contains a regular expression's pattern. The pattern can be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toRegularExpression">toRegularExpression</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#type">type</a>() and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toRegularExpression">toRegularExpression</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_regular_expression(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_isRegularExpression(
            self as *const crate::QCborValue,
        )
    }

    /// <p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of one of the CBOR simple types. The type itself can later be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#type">type</a>(), even for types that don't have an enumeration in the API. They can also be checked with the <a href="http://doc.qt.io/qt-5/qcborvalue.html#isSimpleType-1">isSimpleType</a>(QCborSimpleType) overload.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborValue::isSimpleType() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#isSimpleType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of one of the CBOR simple types. The type itself can later be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#type">type</a>(), even for types that don't have an enumeration in the API. They can also be checked with the <a href="http://doc.qt.io/qt-5/qcborvalue.html#isSimpleType-1">isSimpleType</a>(QCborSimpleType) overload.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborSimpleType-enum">QCborSimpleType</a>, <a href="http://doc.qt.io/qt-5/qcborvalue.html#isSimpleType-1">isSimpleType</a>(QCborSimpleType), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toSimpleType">toSimpleType</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_simple_type_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_isSimpleType(self as *const crate::QCborValue)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborValue::isSimpleType(QCborSimpleType st) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#isSimpleType-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of a simple type and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toSimpleType">toSimpleType</a>() would return <i>st</i>, false otherwise. This function can be used to check for any CBOR simple type, even those for which there is no enumeration in the API. For example, for the simple type of value 12, you could write:</p>
    /// <pre class="cpp">
    ///
    /// &#32;     value<span class="operator">.</span>isSimpleType(<span class="type"><a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborSimpleType-enum">QCborSimpleType</a></span>(<span class="number">12</span>));
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#QCborValue-6">QCborValue::QCborValue</a>(QCborSimpleType), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isSimpleType">isSimpleType</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isFalse">isFalse</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isTrue">isTrue</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isNull">isNull</a>, <a href="http://doc.qt.io/qt-5/qcborvalue.html#isUndefined">isUndefined</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toSimpleType">toSimpleType</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_simple_type_1a(&self, st: crate::QCborSimpleType) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_isSimpleType1(self as *const crate::QCborValue, st)
    }

    /// <p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of the string type. The string value can be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toString">toString</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborValue::isString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#isString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of the string type. The string value can be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toString">toString</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#type">type</a>() and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_string(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_isString(self as *const crate::QCborValue)
    }

    /// <p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of the tag type. The tag value can be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#tag">tag</a>() and the tagged value using <a href="http://doc.qt.io/qt-5/qcborvalue.html#taggedValue">taggedValue</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborValue::isTag() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#isTag">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of the tag type. The tag value can be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#tag">tag</a>() and the tagged value using <a href="http://doc.qt.io/qt-5/qcborvalue.html#taggedValue">taggedValue</a>().</p>
    /// <p>This function also returns true for extended types that the API recognizes. For code that handles extended types directly before the Qt API is updated to support them, it is possible to recreate the tag + tagged value pair by using <a href="http://doc.qt.io/qt-5/qcborvalue.html#taggedValue">taggedValue</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#tag">tag</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#taggedValue">taggedValue</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#taggedValue">taggedValue</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_tag(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_isTag(self as *const crate::QCborValue)
    }

    /// <p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is a boolean with true value. This function exists because, internally, CBOR booleans are stored as two separate types, one for false and one for true.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborValue::isTrue() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#isTrue">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is a boolean with true value. This function exists because, internally, CBOR booleans are stored as two separate types, one for false and one for true.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isBool">isBool</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isFalse">isFalse</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toBool">toBool</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_true(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_isTrue(self as *const crate::QCborValue)
    }

    /// <p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of the undefined type.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborValue::isUndefined() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#isUndefined">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of the undefined type.</p>
    /// <p>CBOR undefined values are used to indicate missing information, usually as a result of a previous operation that did not complete as expected. They are also used by the <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> and <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> API to indicate the searched item was not found.</p>
    /// <p>Undefined values are distinct from null values.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> objects with undefined values are also different from invalid <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> objects. The API will not create invalid QCborValues, but they may exist as a result of a parsing error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isNull">isNull</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#isInvalid">isInvalid</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_undefined(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_isUndefined(self as *const crate::QCborValue)
    }

    /// <p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of the URL type. The URL value can be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toUrl">toUrl</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborValue::isUrl() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#isUrl">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of the URL type. The URL value can be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toUrl">toUrl</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#type">type</a>() and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toUrl">toUrl</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_url(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_isUrl(self as *const crate::QCborValue)
    }

    /// <p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> contains a UUID. The value can be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toUuid">toUuid</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborValue::isUuid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#isUuid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> contains a UUID. The value can be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toUuid">toUuid</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#type">type</a>() and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toUuid">toUuid</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_uuid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_isUuid(self as *const crate::QCborValue)
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> of the <a href="http://doc.qt.io/qt-5/qcborvalue.html#Type-enum">Undefined</a> type.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborValue::QCborValue()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#QCborValue">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> of the <a href="http://doc.qt.io/qt-5/qcborvalue.html#Type-enum">Undefined</a> type.</p>
    /// <p>CBOR undefined values are used to indicate missing information, usually as a result of a previous operation that did not complete as expected. They are also used by the <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> and <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> API to indicate the searched item was not found.</p>
    /// <p>Undefined values are represented by the <a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborSimpleType-enum">Undefined simple type</a>. Because of that, QCborValues with undefined values will also return true for <a href="http://doc.qt.io/qt-5/qcborvalue.html#isSimpleType">isSimpleType</a>() and <code>isSimpleType(QCborSimpleType::Undefined)</code>.</p>
    /// <p>Undefined values are different from null values.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> objects with undefined values are also different from invalid <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> objects. The API will not create invalid QCborValues, but they may exist as a result of a parsing error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isUndefined">isUndefined</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isNull">isNull</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#isSimpleType">isSimpleType</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_QCborValue();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> of type <i>t_</i>. The value associated with such a type (if any) will be default constructed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborValue::QCborValue(QCborValue::Type t_)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#QCborValue-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> of type <i>t_</i>. The value associated with such a type (if any) will be default constructed.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#type">type</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_type(
        t: crate::q_cbor_value::Type,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_QCborValue1(t);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> with boolean value <i>b</i>. The value can later be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toBool">toBool</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborValue::QCborValue(bool b_)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#QCborValue-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> with boolean value <i>b</i>. The value can later be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toBool">toBool</a>().</p>
    /// <p>Internally, CBOR booleans are represented by a pair of types, one for true and one for false. For that reason, boolean QCborValues will return true for <a href="http://doc.qt.io/qt-5/qcborvalue.html#isSimpleType">isSimpleType</a>() and one of <code>isSimpleType(QCborSimpleType::False)</code> or <code>isSimpleType(QCborSimpleType::True)</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#toBool">toBool</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isBool">isBool</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isTrue">isTrue</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isFalse">isFalse</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#isSimpleType">isSimpleType</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_bool(b: bool) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_QCborValue2(b);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborValue::QCborValue(int i)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_int(i: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_QCborValue3(i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborValue::QCborValue(unsigned int u)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_uint(u: ::std::os::raw::c_uint) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_QCborValue4(u);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> with integer value <i>i</i>. The value can later be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toInteger">toInteger</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborValue::QCborValue(qint64 i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#QCborValue-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> with integer value <i>i</i>. The value can later be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toInteger">toInteger</a>().</p>
    /// <p>CBOR integer values are distinct from floating point values. Therefore, <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> objects with integers will compare differently to <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> objects containing floating-point, even if the values contained in the objects are equivalent.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#toInteger">toInteger</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isInteger">isInteger</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#isDouble">isDouble</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_i64(i: i64) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_QCborValue5(i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> with floating point value <i>d</i>. The value can later be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toDouble">toDouble</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborValue::QCborValue(double v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#QCborValue-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> with floating point value <i>d</i>. The value can later be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toDouble">toDouble</a>().</p>
    /// <p>CBOR floating point values are distinct from integer values. Therefore, <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> objects with integers will compare differently to <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> objects containing floating-point, even if the values contained in the objects are equivalent.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#toDouble">toDouble</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isDouble">isDouble</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#isInteger">isInteger</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_double(
        v: ::std::os::raw::c_double,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_QCborValue6(v);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> of simple type <i>st</i>. The type can later later be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toSimpleType">toSimpleType</a>() as well as <a href="http://doc.qt.io/qt-5/qcborvalue.html#isSimpleType">isSimpleType</a>(st).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborValue::QCborValue(QCborSimpleType st)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#QCborValue-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> of simple type <i>st</i>. The type can later later be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toSimpleType">toSimpleType</a>() as well as <a href="http://doc.qt.io/qt-5/qcborvalue.html#isSimpleType">isSimpleType</a>(st).</p>
    /// <p>CBOR simple types are types that do not have any associated value, like C++'s <code>std::nullptr_t</code> type, whose only possible value is <code>nullptr</code>.</p>
    /// <p>If <i>st</i> is <code>QCborSimpleType::Null</code>, the resulting <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> will be of the <a href="http://doc.qt.io/qt-5/qcborvalue.html#Type-enum">Null</a> type and similarly for <code>QCborSimpleType::Undefined</code>. If <i>st</i> is <code>QCborSimpleType::False</code> or <code>QCborSimpleType::True</code>, the created <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> will be a boolean containing a value of false or true, respectively.</p>
    /// <p>This function can be used with simple types not defined in the API. For example, to create a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> with simple type 12, one could write:</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcborvalue.html#QCborValue">QCborValue</a></span> value(<span class="type"><a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborSimpleType-enum">QCborSimpleType</a></span>(<span class="number">12</span>));
    ///
    /// </pre>
    /// <p>Simple types should not be used until a specification for them has been published, since other implementations may not support them properly. Simple type values 24 to 31 are reserved and must not be used.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qcborvalue.html#isSimpleType">isSimpleType</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isNull">isNull</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isUndefined">isUndefined</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isTrue">isTrue</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isFalse">isFalse</a>()</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_q_cbor_simple_type(
        st: crate::QCborSimpleType,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_QCborValue7(st);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> with byte array value <i>ba</i>. The value can later be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toByteArray">toByteArray</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborValue::QCborValue(const QByteArray& ba)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#QCborValue-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> with byte array value <i>ba</i>. The value can later be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toByteArray">toByteArray</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#toByteArray">toByteArray</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isByteArray">isByteArray</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#isString">isString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_q_byte_array(
        ba: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_QCborValue8(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(ba)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> with string value <i>s</i>. The value can later be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toString">toString</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborValue::QCborValue(const QString& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#QCborValue-8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> with string value <i>s</i>. The value can later be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toString">toString</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#toString">toString</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isString">isString</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#isByteArray">isByteArray</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_q_string(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_QCborValue9(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(s).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborValue::QCborValue(QLatin1String s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#QCborValue-9">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> with string value <i>s</i>. The value can later be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toString">toString</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#toString">toString</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isString">isString</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#isByteArray">isByteArray</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_q_latin1_string(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_QCborValue10(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(s)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Default constructs an instance of QCborValue.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborValue::QCborValue(const char* s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#QCborValue-10">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Default constructs an instance of QCborValue.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_char(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_QCborValue11(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborValue::QCborValue(const QCborArray& a)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_q_cbor_array(
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborArray>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_QCborValue12(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborArray>>::cast_into(a).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborValue::QCborValue(const QCborMap& m)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_q_cbor_map(
        m: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborMap>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_QCborValue14(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborMap>>::cast_into(m).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborValue::QCborValue(QCborTag tag, const QCborValue& taggedValue = …)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_q_cbor_tag_q_cbor_value(
        tag: crate::QCborTag,
        tagged_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_QCborValue16(
            tag,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(tagged_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborValue::QCborValue(QCborKnownTags t_, const QCborValue& tv = …)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_q_cbor_known_tags_q_cbor_value(
        t: crate::QCborKnownTags,
        tv: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_QCborValue17(
            t,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(tv)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object of the date/time extended type and containing the value represented by <i>dt</i>. The value can later be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toDateTime">toDateTime</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborValue::QCborValue(const QDateTime& dt)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#QCborValue-17">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object of the date/time extended type and containing the value represented by <i>dt</i>. The value can later be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toDateTime">toDateTime</a>().</p>
    /// <p>The CBOR date/time types are extension types using tags: either a string (in ISO date format) tagged as a <a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborKnownTags-enum">DateTime</a> or a number (of seconds since the start of 1970, UTC) tagged as a <a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborKnownTags-enum">UnixTime_t</a>. When parsing CBOR streams, <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> will convert <a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborKnownTags-enum">UnixTime_t</a> to the string-based type.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#toDateTime">toDateTime</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isDateTime">isDateTime</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#taggedValue">taggedValue</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_q_date_time(
        dt: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_QCborValue18(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(dt).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object of the URL extended type and containing the value represented by <i>url</i>. The value can later be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toUrl">toUrl</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborValue::QCborValue(const QUrl& url)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#QCborValue-18">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object of the URL extended type and containing the value represented by <i>url</i>. The value can later be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toUrl">toUrl</a>().</p>
    /// <p>The CBOR URL type is an extended type represented by a string tagged as an <a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborKnownTags-enum">Url</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#toUrl">toUrl</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isUrl">isUrl</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#taggedValue">taggedValue</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_q_url(
        url: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_QCborValue19(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(url).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object of the regular expression pattern extended type and containing the value represented by <i>rx</i>. The value can later be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toRegularExpression">toRegularExpression</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborValue::QCborValue(const QRegularExpression& rx)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#QCborValue-19">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object of the regular expression pattern extended type and containing the value represented by <i>rx</i>. The value can later be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toRegularExpression">toRegularExpression</a>().</p>
    /// <p>The CBOR regular expression type is an extended type represented by a string tagged as an <a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborKnownTags-enum">RegularExpression</a>. Note that CBOR regular expressions only store the patterns, so any flags that the <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> object may carry will be lost.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#toRegularExpression">toRegularExpression</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isRegularExpression">isRegularExpression</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#taggedValue">taggedValue</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_q_regular_expression(
        rx: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_QCborValue20(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(rx)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object of the UUID extended type and containing the value represented by <i>uuid</i>. The value can later be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toUuid">toUuid</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborValue::QCborValue(const QUuid& uuid)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#QCborValue-20">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object of the UUID extended type and containing the value represented by <i>uuid</i>. The value can later be retrieved using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toUuid">toUuid</a>().</p>
    /// <p>The CBOR UUID type is an extended type represented by a byte array tagged as an <a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborKnownTags-enum">Uuid</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#toUuid">toUuid</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isUuid">isUuid</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#taggedValue">taggedValue</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_q_uuid(
        uuid: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUuid>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_QCborValue21(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUuid>>::cast_into(uuid).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborValue::QCborValue(QCborTag tag)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_q_cbor_tag(tag: crate::QCborTag) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_QCborValue25(tag);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborValue::QCborValue(QCborKnownTags t_)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_q_cbor_known_tags(
        t: crate::QCborKnownTags,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_QCborValue26(t);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Copies the contents of <i>other</i> into this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborValue::QCborValue(const QCborValue& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#QCborValue-22">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Copies the contents of <i>other</i> into this object.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_QCborValue23(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Swaps the contents of this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object and <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborValue::swap(QCborValue& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps the contents of this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object and <i>other</i>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QCborValue>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_swap(
            self as *mut crate::QCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QCborValue>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the tag of this extended <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object, if it is of the tag type, <i>defaultValue</i> otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborTag QCborValue::tag(QCborTag defaultValue = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#tag">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the tag of this extended <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object, if it is of the tag type, <i>defaultValue</i> otherwise.</p>
    /// <p>CBOR represents extended types by associating a number (the tag) with a stored representation. This function returns that number. To retrieve the representation, use <a href="http://doc.qt.io/qt-5/qcborvalue.html#taggedValue">taggedValue</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isTag">isTag</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#taggedValue">taggedValue</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isDateTime">isDateTime</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isUrl">isUrl</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isRegularExpression">isRegularExpression</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#isUuid">isUuid</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn tag_1a(&self, default_value: crate::QCborTag) -> crate::QCborTag {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_tag(
            self as *const crate::QCborValue,
            default_value,
        )
    }

    /// <p>Returns the tag of this extended <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object, if it is of the tag type, <i>defaultValue</i> otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborTag QCborValue::tag() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#tag">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the tag of this extended <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object, if it is of the tag type, <i>defaultValue</i> otherwise.</p>
    /// <p>CBOR represents extended types by associating a number (the tag) with a stored representation. This function returns that number. To retrieve the representation, use <a href="http://doc.qt.io/qt-5/qcborvalue.html#taggedValue">taggedValue</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isTag">isTag</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#taggedValue">taggedValue</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isDateTime">isDateTime</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isUrl">isUrl</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isRegularExpression">isRegularExpression</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#isUuid">isUuid</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn tag_0a(&self) -> crate::QCborTag {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_tag1(self as *const crate::QCborValue)
    }

    /// <p>Returns the tagged value of this extended <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object, if it is of the tag type, <i>defaultValue</i> otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborValue::taggedValue(const QCborValue& defaultValue = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#taggedValue">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the tagged value of this extended <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object, if it is of the tag type, <i>defaultValue</i> otherwise.</p>
    /// <p>CBOR represents extended types by associating a number (the tag) with a stored representation. This function returns that representation. To retrieve the tag, use <a href="http://doc.qt.io/qt-5/qcborvalue.html#tag">tag</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isTag">isTag</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#tag">tag</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isDateTime">isDateTime</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isUrl">isUrl</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isRegularExpression">isRegularExpression</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#isUuid">isUuid</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn tagged_value_1a(
        &self,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_taggedValue(
            self as *const crate::QCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the tagged value of this extended <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object, if it is of the tag type, <i>defaultValue</i> otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborValue::taggedValue() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#taggedValue">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the tagged value of this extended <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object, if it is of the tag type, <i>defaultValue</i> otherwise.</p>
    /// <p>CBOR represents extended types by associating a number (the tag) with a stored representation. This function returns that representation. To retrieve the tag, use <a href="http://doc.qt.io/qt-5/qcborvalue.html#tag">tag</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isTag">isTag</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#tag">tag</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isDateTime">isDateTime</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isUrl">isUrl</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isRegularExpression">isRegularExpression</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#isUuid">isUuid</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn tagged_value_0a(&self) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborValue_taggedValue1(self as *const crate::QCborValue);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the array value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the array type. Otherwise, it returns <i>defaultValue</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborArray QCborValue::toArray() const```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qcborvalue.html#toArray">C++ documentation</a> for <span style='color: green;'>```QCborArray QCborValue::toArray(const QCborArray &defaultValue) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the array value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the array type. Otherwise, it returns <i>defaultValue</i>.</p>
    /// <p>Note that this function performs no conversion from other types to <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isArray">isArray</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isByteArray">isByteArray</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isMap">isMap</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isContainer">isContainer</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toMap">toMap</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_array_0a(&self) -> ::cpp_utils::CppBox<crate::QCborArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborValue_toArray(self as *const crate::QCborValue);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the array value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the array type. Otherwise, it returns <i>defaultValue</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborArray QCborValue::toArray(const QCborArray& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the array value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the array type. Otherwise, it returns <i>defaultValue</i>.</p>
    /// <p>Note that this function performs no conversion from other types to <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isArray">isArray</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isByteArray">isByteArray</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isMap">isMap</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isContainer">isContainer</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toMap">toMap</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_array_1a(
        &self,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborArray>>,
    ) -> ::cpp_utils::CppBox<crate::QCborArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_toArray1(
            self as *const crate::QCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborArray>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the boolean value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of a boolean type. Otherwise, it returns <i>defaultValue</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborValue::toBool(bool defaultValue = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toBool">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the boolean value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of a boolean type. Otherwise, it returns <i>defaultValue</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isBool">isBool</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isTrue">isTrue</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#isFalse">isFalse</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_bool_1a(&self, default_value: bool) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_toBool(
            self as *const crate::QCborValue,
            default_value,
        )
    }

    /// <p>Returns the boolean value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of a boolean type. Otherwise, it returns <i>defaultValue</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborValue::toBool() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toBool">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the boolean value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of a boolean type. Otherwise, it returns <i>defaultValue</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isBool">isBool</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isTrue">isTrue</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#isFalse">isFalse</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_bool_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_toBool1(self as *const crate::QCborValue)
    }

    /// <p>Returns the byte array value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the byte array type. Otherwise, it returns <i>defaultValue</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QCborValue::toByteArray(const QByteArray& defaultValue = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toByteArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the byte array type. Otherwise, it returns <i>defaultValue</i>.</p>
    /// <p>Note that this function performs no conversion from other types to <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isByteArray">isByteArray</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isString">isString</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_byte_array_1a(
        &self,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_toByteArray(
            self as *const crate::QCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the byte array value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the byte array type. Otherwise, it returns <i>defaultValue</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QCborValue::toByteArray() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toByteArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the byte array value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the byte array type. Otherwise, it returns <i>defaultValue</i>.</p>
    /// <p>Note that this function performs no conversion from other types to <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isByteArray">isByteArray</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isString">isString</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toString">toString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_byte_array_0a(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborValue_toByteArray1(self as *const crate::QCborValue);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Encodes this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object to its CBOR representation, using the options specified in <i>opt</i>, and return the byte array containing that representation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QCborValue::toCbor(QFlags<QCborValue::EncodingOption> opt = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toCbor">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Encodes this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object to its CBOR representation, using the options specified in <i>opt</i>, and return the byte array containing that representation.</p>
    /// <p>This function will not fail, except if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> or any of the contained items, if this is a map or array, are invalid. Invalid types are not produced normally by the API, but can result from decoding errors.</p>
    /// <p>By default, this function performs no transformation on the values in the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, writing all floating point directly as double-precision (<code>double</code>) types. If the <a href="http://doc.qt.io/qt-5/qcborvalue.html#EncodingOption-enum">UseFloat</a> option is specified, it will use single precision (<code>float</code>) for any floating point value for which there's no loss of precision in using that representation. That includes infinities and NaN values.</p>
    /// <p>Similarly, if <a href="http://doc.qt.io/qt-5/qcborvalue.html#EncodingOption-enum">UseFloat16</a> is specified, this function will try to use half-precision (<code>qfloat16</code>) floating point if the conversion to that results in no loss of precision. This is always true for infinities and NaN.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qcborvalue.html#EncodingOption-enum">UseIntegers</a> is specified, it will use integers for any floating point value that contains an actual integer.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#fromCbor">fromCbor</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromVariant">fromVariant</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromJsonValue">fromJsonValue</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_cbor_q_flags_encoding_option(
        &mut self,
        opt: crate::QFlags<crate::q_cbor_value::EncodingOption>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_toCbor(
            self as *mut crate::QCborValue,
            opt.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborValue::toCbor(QCborStreamWriter& writer, QFlags<QCborValue::EncodingOption> opt = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toCbor-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Encodes this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object to its CBOR representation, using the options specified in <i>opt</i>, to the writer specified by <i>writer</i>. The same writer can be used by multiple QCborValues, for example, in order to encode different elements in a larger array.</p>
    /// <p>This function will not fail, except if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> or any of the contained items, if this is a map or array, are invalid. Invalid types are not produced normally by the API, but can result from decoding errors.</p>
    /// <p>By default, this function performs no transformation on the values in the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, writing all floating point directly as double-precision (binary64) types. If the <a href="http://doc.qt.io/qt-5/qcborvalue.html#EncodingOption-enum">UseFloat</a> option is specified, it will use single precision (binary32) for any floating point value for which there's no loss of precision in using that representation. That includes infinities and NaN values.</p>
    /// <p>Similarly, if <a href="http://doc.qt.io/qt-5/qcborvalue.html#EncodingOption-enum">UseFloat16</a> is specified, this function will try to use half-precision (binary16) floating point if the conversion to that results in no loss of precision. This is always true for infinities and NaN.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qcborvalue.html#EncodingOption-enum">UseIntegers</a> is specified, it will use integers for any floating point value that contains an actual integer.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#fromCbor">fromCbor</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromVariant">fromVariant</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromJsonValue">fromJsonValue</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_cbor_q_cbor_stream_writer_q_flags_encoding_option(
        &mut self,
        writer: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QCborStreamWriter>>,
        opt: crate::QFlags<crate::q_cbor_value::EncodingOption>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_toCbor1(
            self as *mut crate::QCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QCborStreamWriter>>::cast_into(
                writer,
            )
            .as_mut_raw_ptr(),
            opt.to_int(),
        )
    }

    /// <p>Encodes this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object to its CBOR representation, using the options specified in <i>opt</i>, and return the byte array containing that representation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray QCborValue::toCbor()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toCbor">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Encodes this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object to its CBOR representation, using the options specified in <i>opt</i>, and return the byte array containing that representation.</p>
    /// <p>This function will not fail, except if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> or any of the contained items, if this is a map or array, are invalid. Invalid types are not produced normally by the API, but can result from decoding errors.</p>
    /// <p>By default, this function performs no transformation on the values in the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, writing all floating point directly as double-precision (<code>double</code>) types. If the <a href="http://doc.qt.io/qt-5/qcborvalue.html#EncodingOption-enum">UseFloat</a> option is specified, it will use single precision (<code>float</code>) for any floating point value for which there's no loss of precision in using that representation. That includes infinities and NaN values.</p>
    /// <p>Similarly, if <a href="http://doc.qt.io/qt-5/qcborvalue.html#EncodingOption-enum">UseFloat16</a> is specified, this function will try to use half-precision (<code>qfloat16</code>) floating point if the conversion to that results in no loss of precision. This is always true for infinities and NaN.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qcborvalue.html#EncodingOption-enum">UseIntegers</a> is specified, it will use integers for any floating point value that contains an actual integer.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#fromCbor">fromCbor</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromVariant">fromVariant</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromJsonValue">fromJsonValue</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_cbor(&mut self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborValue_toCbor2(self as *mut crate::QCborValue);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborValue::toCbor(QCborStreamWriter& writer)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toCbor-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Encodes this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object to its CBOR representation, using the options specified in <i>opt</i>, to the writer specified by <i>writer</i>. The same writer can be used by multiple QCborValues, for example, in order to encode different elements in a larger array.</p>
    /// <p>This function will not fail, except if this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> or any of the contained items, if this is a map or array, are invalid. Invalid types are not produced normally by the API, but can result from decoding errors.</p>
    /// <p>By default, this function performs no transformation on the values in the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, writing all floating point directly as double-precision (binary64) types. If the <a href="http://doc.qt.io/qt-5/qcborvalue.html#EncodingOption-enum">UseFloat</a> option is specified, it will use single precision (binary32) for any floating point value for which there's no loss of precision in using that representation. That includes infinities and NaN values.</p>
    /// <p>Similarly, if <a href="http://doc.qt.io/qt-5/qcborvalue.html#EncodingOption-enum">UseFloat16</a> is specified, this function will try to use half-precision (binary16) floating point if the conversion to that results in no loss of precision. This is always true for infinities and NaN.</p>
    /// <p>If <a href="http://doc.qt.io/qt-5/qcborvalue.html#EncodingOption-enum">UseIntegers</a> is specified, it will use integers for any floating point value that contains an actual integer.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#fromCbor">fromCbor</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromVariant">fromVariant</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromJsonValue">fromJsonValue</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_cbor_q_cbor_stream_writer(
        &mut self,
        writer: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QCborStreamWriter>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_toCbor3(
            self as *mut crate::QCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QCborStreamWriter>>::cast_into(
                writer,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Returns the date/time value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the date/time extended type. Otherwise, it returns <i>defaultValue</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDateTime QCborValue::toDateTime(const QDateTime& defaultValue = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toDateTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the date/time value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the date/time extended type. Otherwise, it returns <i>defaultValue</i>.</p>
    /// <p>Note that this function performs no conversion from other types to <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isDateTime">isDateTime</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isTag">isTag</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#taggedValue">taggedValue</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_date_time_1a(
        &self,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_toDateTime(
            self as *const crate::QCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the date/time value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the date/time extended type. Otherwise, it returns <i>defaultValue</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDateTime QCborValue::toDateTime() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toDateTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the date/time value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the date/time extended type. Otherwise, it returns <i>defaultValue</i>.</p>
    /// <p>Note that this function performs no conversion from other types to <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isDateTime">isDateTime</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isTag">isTag</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#taggedValue">taggedValue</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_date_time_0a(&self) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborValue_toDateTime1(self as *const crate::QCborValue);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates the diagnostic notation equivalent of this CBOR object and returns it. The <i>opts</i> parameter controls the dialect of the notation. Diagnostic notation is useful in debugging, to aid the developer in understanding what value is stored in the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> or in a CBOR stream. For that reason, the Qt API provides no support for parsing the diagnostic back into the in-memory format or CBOR stream, though the representation is unique and it would be possible.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QCborValue::toDiagnosticNotation(QFlags<QCborValue::DiagnosticNotationOption> opts = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toDiagnosticNotation">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates the diagnostic notation equivalent of this CBOR object and returns it. The <i>opts</i> parameter controls the dialect of the notation. Diagnostic notation is useful in debugging, to aid the developer in understanding what value is stored in the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> or in a CBOR stream. For that reason, the Qt API provides no support for parsing the diagnostic back into the in-memory format or CBOR stream, though the representation is unique and it would be possible.</p>
    /// <p>CBOR diagnostic notation is specified by <a href="https://tools.ietf.org/html/rfc7049#section-6">section 6</a> of RFC 7049. It is a text representation of the CBOR stream and it is very similar to JSON, but it supports the CBOR types not found in JSON. The extended format enabled by the <a href="http://doc.qt.io/qt-5/qcborvalue.html#DiagnosticNotationOption-enum">ExtendedFormat</a> flag is currently in some IETF drafts and its format is subject to change.</p>
    /// <p>This function produces the equivalent representation of the stream that <a href="http://doc.qt.io/qt-5/qcborvalue.html#toCbor">toCbor</a>() would produce, without any transformation option provided there. This also implies this function may not produce a representation of the stream that was used to create the object, if it was created using <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromCbor">fromCbor</a>(), as that function may have applied transformations. For a high-fidelity notation of a stream, without transformation, see the <code>cbordump</code> example.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#toCbor">toCbor</a>() and <a href="http://doc.qt.io/qt-5/qjsondocument.html#toJson">QJsonDocument::toJson</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_diagnostic_notation_1a(
        &self,
        opts: crate::QFlags<crate::q_cbor_value::DiagnosticNotationOption>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_toDiagnosticNotation(
            self as *const crate::QCborValue,
            opts.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates the diagnostic notation equivalent of this CBOR object and returns it. The <i>opts</i> parameter controls the dialect of the notation. Diagnostic notation is useful in debugging, to aid the developer in understanding what value is stored in the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> or in a CBOR stream. For that reason, the Qt API provides no support for parsing the diagnostic back into the in-memory format or CBOR stream, though the representation is unique and it would be possible.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QCborValue::toDiagnosticNotation() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toDiagnosticNotation">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates the diagnostic notation equivalent of this CBOR object and returns it. The <i>opts</i> parameter controls the dialect of the notation. Diagnostic notation is useful in debugging, to aid the developer in understanding what value is stored in the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> or in a CBOR stream. For that reason, the Qt API provides no support for parsing the diagnostic back into the in-memory format or CBOR stream, though the representation is unique and it would be possible.</p>
    /// <p>CBOR diagnostic notation is specified by <a href="https://tools.ietf.org/html/rfc7049#section-6">section 6</a> of RFC 7049. It is a text representation of the CBOR stream and it is very similar to JSON, but it supports the CBOR types not found in JSON. The extended format enabled by the <a href="http://doc.qt.io/qt-5/qcborvalue.html#DiagnosticNotationOption-enum">ExtendedFormat</a> flag is currently in some IETF drafts and its format is subject to change.</p>
    /// <p>This function produces the equivalent representation of the stream that <a href="http://doc.qt.io/qt-5/qcborvalue.html#toCbor">toCbor</a>() would produce, without any transformation option provided there. This also implies this function may not produce a representation of the stream that was used to create the object, if it was created using <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromCbor">fromCbor</a>(), as that function may have applied transformations. For a high-fidelity notation of a stream, without transformation, see the <code>cbordump</code> example.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#toCbor">toCbor</a>() and <a href="http://doc.qt.io/qt-5/qjsondocument.html#toJson">QJsonDocument::toJson</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_diagnostic_notation_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_toDiagnosticNotation1(
            self as *const crate::QCborValue,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the floating point value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the Double type. If it is of the Integer type, this function returns the integer value converted to double. In any other case, it returns <i>defaultValue</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QCborValue::toDouble(double defaultValue = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toDouble">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the floating point value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the Double type. If it is of the Integer type, this function returns the integer value converted to double. In any other case, it returns <i>defaultValue</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isDouble">isDouble</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isInteger">isInteger</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toInteger">toInteger</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_double_1a(
        &self,
        default_value: ::std::os::raw::c_double,
    ) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_toDouble(
            self as *const crate::QCborValue,
            default_value,
        )
    }

    /// <p>Returns the floating point value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the Double type. If it is of the Integer type, this function returns the integer value converted to double. In any other case, it returns <i>defaultValue</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QCborValue::toDouble() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toDouble">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the floating point value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the Double type. If it is of the Integer type, this function returns the integer value converted to double. In any other case, it returns <i>defaultValue</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isDouble">isDouble</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isInteger">isInteger</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toInteger">toInteger</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_double_0a(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_toDouble1(self as *const crate::QCborValue)
    }

    /// <p>Returns the integer value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the integer type. If it is of the Double type, this function returns the floating point value converted to integer. In any other case, it returns <i>defaultValue</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QCborValue::toInteger(qint64 defaultValue = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toInteger">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the integer value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the integer type. If it is of the Double type, this function returns the floating point value converted to integer. In any other case, it returns <i>defaultValue</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isInteger">isInteger</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isDouble">isDouble</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toDouble">toDouble</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_integer_1a(&self, default_value: i64) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_toInteger(
            self as *const crate::QCborValue,
            default_value,
        )
    }

    /// <p>Returns the integer value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the integer type. If it is of the Double type, this function returns the floating point value converted to integer. In any other case, it returns <i>defaultValue</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QCborValue::toInteger() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toInteger">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the integer value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the integer type. If it is of the Double type, this function returns the floating point value converted to integer. In any other case, it returns <i>defaultValue</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isInteger">isInteger</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isDouble">isDouble</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toDouble">toDouble</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_integer_0a(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_toInteger1(self as *const crate::QCborValue)
    }

    /// <p>Converts this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object to an equivalent representation in JSON and returns it as a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonValue QCborValue::toJsonValue() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toJsonValue">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object to an equivalent representation in JSON and returns it as a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>.</p>
    /// <p>Please note that CBOR contains a richer and wider type set than JSON, so some information may be lost in this conversion. The following table compares CBOR types to JSON types and indicates whether information may be lost or not.</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>CBOR Type</th><th>JSON Type</th><th>Comments</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>Bool</td><td>Bool</td><td>No data loss possible</td></tr>
    /// <tr valign="top" class="even"><td>Double</td><td>Number</td><td>Infinities and NaN will be converted to Null; no data loss for other values</td></tr>
    /// <tr valign="top" class="odd"><td>Integer</td><td>Number</td><td>Data loss possible in the conversion if the integer is larger than 2<sup>53</sup> or smaller than -2<sup>53</sup>.</td></tr>
    /// <tr valign="top" class="even"><td>Null</td><td>Null</td><td>No data loss possible</td></tr>
    /// <tr valign="top" class="odd"><td>Undefined</td><td>Null</td><td>Type information lost</td></tr>
    /// <tr valign="top" class="even"><td>String</td><td>String</td><td>No data loss possible</td></tr>
    /// <tr valign="top" class="odd"><td>Byte Array</td><td>String</td><td>Converted to a lossless encoding like Base64url, but the distinction between strings and byte arrays is lost</td></tr>
    /// <tr valign="top" class="even"><td>Other simple types</td><td>String</td><td>Type information lost</td></tr>
    /// <tr valign="top" class="odd"><td>Array</td><td>Array</td><td>Conversion applies to each contained value</td></tr>
    /// <tr valign="top" class="even"><td>Map</td><td>Object</td><td>Keys are converted to string; values converted according to this table</td></tr>
    /// <tr valign="top" class="odd"><td>Tags and extended types</td><td>Special</td><td>The tag number itself is lost and the tagged value is converted to JSON</td></tr>
    /// </tbody></table></div>
    /// <p>For information on the conversion of CBOR map keys to string, see <a href="http://doc.qt.io/qt-5/qcbormap.html#toJsonObject">QCborMap::toJsonObject</a>().</p>
    /// <p>If this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> contains the undefined value, this function will return an undefined <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a> too. Note that JSON does not support undefined values and undefined QJsonValues are an extension to the specification. They cannot be held in a <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> or <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>, but can be returned from functions to indicate a failure. For all other intents and purposes, they are the same as null.</p>
    /// <a name="special-handling-of-tags-and-extended-types"></a>
    /// <h6>Special handling of tags and extended types</h6>
    /// <p>Some tags are handled specially and change the transformation of the tagged value from CBOR to JSON. The following table lists those special cases:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Tag</th><th>CBOR type</th><th>Transformation</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborKnownTags-enum">ExpectedBase64url</a></td><td>Byte array</td><td>Encodes the byte array as Base64url</td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborKnownTags-enum">ExpectedBase64</a></td><td>Byte array</td><td>Encodes the byte array as Base64</td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborKnownTags-enum">ExpectedBase16</a></td><td>Byte array</td><td>Encodes the byte array as hex</td></tr>
    /// <tr valign="top" class="even"><td>Url</td><td>Url and String</td><td>Uses <a href="http://doc.qt.io/qt-5/qurl.html#toEncoded">QUrl::toEncoded</a>() to normalize the encoding to the URL's fully encoded format</td></tr>
    /// <tr valign="top" class="odd"><td>Uuid</td><td>Uuid and Byte array</td><td>Uses <a href="http://doc.qt.io/qt-5/quuid.html#toString">QUuid::toString</a>() to create the string representation</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#fromJsonValue">fromJsonValue</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#toVariant">toVariant</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#toJsonArray">QCborArray::toJsonArray</a>(), and <a href="http://doc.qt.io/qt-5/qcbormap.html#toJsonObject">QCborMap::toJsonObject</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_json_value(&self) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborValue_toJsonValue(self as *const crate::QCborValue);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the map value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the map type. Otherwise, it returns <i>defaultValue</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborMap QCborValue::toMap() const```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qcborvalue.html#toMap">C++ documentation</a> for <span style='color: green;'>```QCborMap QCborValue::toMap(const QCborMap &defaultValue) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the map value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the map type. Otherwise, it returns <i>defaultValue</i>.</p>
    /// <p>Note that this function performs no conversion from other types to <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isMap">isMap</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isArray">isArray</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isContainer">isContainer</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toArray">toArray</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_map_0a(&self) -> ::cpp_utils::CppBox<crate::QCborMap> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborValue_toMap(self as *const crate::QCborValue);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the map value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the map type. Otherwise, it returns <i>defaultValue</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborMap QCborValue::toMap(const QCborMap& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toMap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the map value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the map type. Otherwise, it returns <i>defaultValue</i>.</p>
    /// <p>Note that this function performs no conversion from other types to <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isMap">isMap</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isArray">isArray</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isContainer">isContainer</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toArray">toArray</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_map_1a(
        &self,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborMap>>,
    ) -> ::cpp_utils::CppBox<crate::QCborMap> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_toMap1(
            self as *const crate::QCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborMap>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the regular expression value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the regular expression pattern extended type. Otherwise, it returns <i>defaultValue</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpression QCborValue::toRegularExpression(const QRegularExpression& defaultValue = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toRegularExpression">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the regular expression value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the regular expression pattern extended type. Otherwise, it returns <i>defaultValue</i>.</p>
    /// <p>Note that this function performs no conversion from other types to <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isRegularExpression">isRegularExpression</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isTag">isTag</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#taggedValue">taggedValue</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_regular_expression_1a(
        &self,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
    ) -> ::cpp_utils::CppBox<crate::QRegularExpression> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_toRegularExpression(
            self as *const crate::QCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(
                default_value,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the regular expression value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the regular expression pattern extended type. Otherwise, it returns <i>defaultValue</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRegularExpression QCborValue::toRegularExpression() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toRegularExpression">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the regular expression value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the regular expression pattern extended type. Otherwise, it returns <i>defaultValue</i>.</p>
    /// <p>Note that this function performs no conversion from other types to <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isRegularExpression">isRegularExpression</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isTag">isTag</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#taggedValue">taggedValue</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_regular_expression_0a(
        &self,
    ) -> ::cpp_utils::CppBox<crate::QRegularExpression> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_toRegularExpression1(
            self as *const crate::QCborValue,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the simple type this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of, if it is a simple type. If it is not a simple type, it returns <i>defaultValue</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborSimpleType QCborValue::toSimpleType(QCborSimpleType defaultValue = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toSimpleType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the simple type this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of, if it is a simple type. If it is not a simple type, it returns <i>defaultValue</i>.</p>
    /// <p>The following types are simple types and this function will return the listed values:</p>
    /// <div class="table"><table class="generic">
    ///  <tbody><tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qcborvalue.html#Type-enum">QCborValue::False</a></td><td>QCborSimpleType::False</td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qcborvalue.html#Type-enum">QCborValue::True</a></td><td>QCborSimpleType::True</td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qcborvalue.html#Type-enum">QCborValue::Null</a></td><td>QCborSimpleType::Null</td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qcborvalue.html#Type-enum">QCborValue::Undefined</a></td><td>QCborSimpleType::Undefined</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isSimpleType">isSimpleType</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isBool">isBool</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isTrue">isTrue</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isFalse">isFalse</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isTrue">isTrue</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isNull">isNull</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#isUndefined">isUndefined</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_simple_type_1a(
        &self,
        default_value: crate::QCborSimpleType,
    ) -> crate::QCborSimpleType {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_toSimpleType(
            self as *const crate::QCborValue,
            default_value,
        )
    }

    /// <p>Returns the simple type this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of, if it is a simple type. If it is not a simple type, it returns <i>defaultValue</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborSimpleType QCborValue::toSimpleType() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toSimpleType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the simple type this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is of, if it is a simple type. If it is not a simple type, it returns <i>defaultValue</i>.</p>
    /// <p>The following types are simple types and this function will return the listed values:</p>
    /// <div class="table"><table class="generic">
    ///  <tbody><tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qcborvalue.html#Type-enum">QCborValue::False</a></td><td>QCborSimpleType::False</td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qcborvalue.html#Type-enum">QCborValue::True</a></td><td>QCborSimpleType::True</td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qcborvalue.html#Type-enum">QCborValue::Null</a></td><td>QCborSimpleType::Null</td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qcborvalue.html#Type-enum">QCborValue::Undefined</a></td><td>QCborSimpleType::Undefined</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isSimpleType">isSimpleType</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isBool">isBool</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isTrue">isTrue</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isFalse">isFalse</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isTrue">isTrue</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isNull">isNull</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#isUndefined">isUndefined</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_simple_type_0a(&self) -> crate::QCborSimpleType {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_toSimpleType1(self as *const crate::QCborValue)
    }

    /// <p>Returns the string value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the string type. Otherwise, it returns <i>defaultValue</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QCborValue::toString(const QString& defaultValue = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the string type. Otherwise, it returns <i>defaultValue</i>.</p>
    /// <p>Note that this function performs no conversion from other types to <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isString">isString</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isByteArray">isByteArray</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toByteArray">toByteArray</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_string_1a(
        &self,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_toString(
            self as *const crate::QCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the string value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the string type. Otherwise, it returns <i>defaultValue</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString QCborValue::toString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the string value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the string type. Otherwise, it returns <i>defaultValue</i>.</p>
    /// <p>Note that this function performs no conversion from other types to <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isString">isString</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isByteArray">isByteArray</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#toByteArray">toByteArray</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_string_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborValue_toString1(self as *const crate::QCborValue);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the URL value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the URL extended type. Otherwise, it returns <i>defaultValue</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QUrl QCborValue::toUrl(const QUrl& defaultValue = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toUrl">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the URL value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the URL extended type. Otherwise, it returns <i>defaultValue</i>.</p>
    /// <p>Note that this function performs no conversion from other types to <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isUrl">isUrl</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isTag">isTag</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#taggedValue">taggedValue</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_url_1a(
        &self,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
    ) -> ::cpp_utils::CppBox<crate::QUrl> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_toUrl(
            self as *const crate::QCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the URL value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the URL extended type. Otherwise, it returns <i>defaultValue</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QUrl QCborValue::toUrl() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toUrl">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the URL value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the URL extended type. Otherwise, it returns <i>defaultValue</i>.</p>
    /// <p>Note that this function performs no conversion from other types to <a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isUrl">isUrl</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isTag">isTag</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#taggedValue">taggedValue</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_url_0a(&self) -> ::cpp_utils::CppBox<crate::QUrl> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborValue_toUrl1(self as *const crate::QCborValue);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the UUID value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the UUID extended type. Otherwise, it returns <i>defaultValue</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QUuid QCborValue::toUuid(const QUuid& defaultValue = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toUuid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the UUID value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the UUID extended type. Otherwise, it returns <i>defaultValue</i>.</p>
    /// <p>Note that this function performs no conversion from other types to <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isUuid">isUuid</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isTag">isTag</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#taggedValue">taggedValue</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_uuid_1a(
        &self,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUuid>>,
    ) -> ::cpp_utils::CppBox<crate::QUuid> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValue_toUuid(
            self as *const crate::QCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUuid>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the UUID value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the UUID extended type. Otherwise, it returns <i>defaultValue</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QUuid QCborValue::toUuid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toUuid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the UUID value stored in this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, if it is of the UUID extended type. Otherwise, it returns <i>defaultValue</i>.</p>
    /// <p>Note that this function performs no conversion from other types to <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isUuid">isUuid</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isTag">isTag</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#taggedValue">taggedValue</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_uuid_0a(&self) -> ::cpp_utils::CppBox<crate::QUuid> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborValue_toUuid1(self as *const crate::QCborValue);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts this value to a native Qt type and returns the corresponding <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVariant QCborValue::toVariant() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#toVariant">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts this value to a native Qt type and returns the corresponding <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
    /// <p>The following table lists the mapping performed between <a href="http://doc.qt.io/qt-5/qcborvalue.html#Type-enum">QCborValue types</a> and <a href="http://doc.qt.io/qt-5/qmetatype.html#Type-enum">Qt meta types</a>.</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>CBOR Type</th><th>Qt or C++ type</th><th>Notes</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>Integer</td><td><a href="http://doc.qt.io/qt-5/qtglobal.html#qint64-typedef">qint64</a></td><td></td></tr>
    /// <tr valign="top" class="even"><td>Double</td><td><code>double</code></td><td></td></tr>
    /// <tr valign="top" class="odd"><td>Bool</td><td><code>bool</code></td><td></td></tr>
    /// <tr valign="top" class="even"><td>Null</td><td><code>std::nullptr_t</code></td><td></td></tr>
    /// <tr valign="top" class="odd"><td>Undefined</td><td>no type (QVariant())</td><td></td></tr>
    /// <tr valign="top" class="even"><td>Byte array</td><td><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></td><td></td></tr>
    /// <tr valign="top" class="odd"><td>String</td><td><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></td><td></td></tr>
    /// <tr valign="top" class="even"><td>Array</td><td><a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a></td><td>Recursively converts all values</td></tr>
    /// <tr valign="top" class="odd"><td>Map</td><td><a href="http://doc.qt.io/qt-5/qvariant.html#QVariantMap-typedef">QVariantMap</a></td><td>Key types are "stringified"</td></tr>
    /// <tr valign="top" class="even"><td>Other simple types</td><td><a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborSimpleType-enum">QCborSimpleType</a></td><td></td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qcborvalue.html#Type-enum">DateTime</a></td><td><a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a></td><td></td></tr>
    /// <tr valign="top" class="even"><td>Url</td><td><a href="http://doc.qt.io/qt-5/qurl.html">QUrl</a></td><td></td></tr>
    /// <tr valign="top" class="odd"><td><a href="http://doc.qt.io/qt-5/qcborvalue.html#Type-enum">RegularExpression</a></td><td><a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a></td><td></td></tr>
    /// <tr valign="top" class="even"><td>Uuid</td><td><a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a></td><td></td></tr>
    /// <tr valign="top" class="odd"><td>Other tags</td><td>Special</td><td>The tag is ignored and the tagged value is converted using this function</td></tr>
    /// </tbody></table></div>
    /// <p>Note that values in both CBOR Maps and Arrays are converted recursively using this function too and placed in <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantMap-typedef">QVariantMap</a> and <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a> instead. You will not find <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> and <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> stored inside the QVariants.</p>
    /// <p>QVariantMaps have string keys, unlike CBOR, so the conversion of a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> to <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantMap-typedef">QVariantMap</a> will imply a step of "stringification" of the key values. See <a href="http://doc.qt.io/qt-5/qcbormap.html#toJsonObject">QCborMap::toJsonObject</a>() for details.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#fromVariant">fromVariant</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#toJsonValue">toJsonValue</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#toVariantList">QCborArray::toVariantList</a>(), and <a href="http://doc.qt.io/qt-5/qcbormap.html#toVariantMap">QCborMap::toVariantMap</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_variant(&self) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborValue_toVariant(self as *const crate::QCborValue);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the type of this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>. The type can also later be retrieved by one of the "isXxx" functions.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue::Type QCborValue::type() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#type">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the type of this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>. The type can also later be retrieved by one of the "isXxx" functions.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#isInteger">isInteger</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isByteArray">isByteArray</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isString">isString</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isArray">isArray</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isMap">isMap</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isTag">isTag</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isFalse">isFalse</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isTrue">isTrue</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isBool">isBool</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isNull">isNull</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isUndefined">isUndefined</a>, <a href="http://doc.qt.io/qt-5/qcborvalue.html#isDouble">isDouble</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isDateTime">isDateTime</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isUrl">isUrl</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#isRegularExpression">isRegularExpression</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#isUuid">isUuid</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn type_(&self) -> crate::q_cbor_value::Type {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_type(self as *const crate::QCborValue)
    }
}

/// C++ class: <span style='color: green;'>```QCborValueRef```</span>.
#[repr(C)]
pub struct QCborValueRef {
    _unused: u8,
}
impl QCborValueRef {
    /// Calls C++ function: <span style='color: green;'>```int QCborValueRef::compare(const QCborValue& other) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn compare(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_compare(
            self as *const crate::QCborValueRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```QCborValueRef& QCborValueRef::operator=(const QCborValue& other)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn copy_from_q_cbor_value(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::cpp_utils::MutRef<crate::QCborValueRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_operator_(
            self as *mut crate::QCborValueRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```QCborValueRef& QCborValueRef::operator=(const QCborValueRef& other)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn copy_from_q_cbor_value_ref(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValueRef>>,
    ) -> ::cpp_utils::MutRef<crate::QCborValueRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_operator_2(
            self as *mut crate::QCborValueRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValueRef>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>If this <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> refers to a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>, searches elements for the value whose key matches <i>key</i>. If there's no key matching <i>key</i> in the map or if this <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> object is not a map, returns the undefined value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborValueRef::operator[](const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalueref.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>If this <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> refers to a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>, searches elements for the value whose key matches <i>key</i>. If there's no key matching <i>key</i> in the map or if this <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> object is not a map, returns the undefined value.</p>
    /// <p>This function is equivalent to:</p>
    /// <pre class="cpp">
    ///
    ///   value<span class="operator">.</span>toMap()<span class="operator">.</span>value(key);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalueref.html#operator-5b-5d-2">operator[]</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d">QCborMap::operator[]</a>, <a href="http://doc.qt.io/qt-5/qcbormap.html#value">QCborMap::value</a>(), and <a href="http://doc.qt.io/qt-5/qcbormap.html#find">QCborMap::find</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn index_q_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_operator__4(
            self as *const crate::QCborValueRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborValueRef::operator[](QLatin1String key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalueref.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If this <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> refers to a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>, searches elements for the value whose key matches <i>key</i>. If there's no key matching <i>key</i> in the map or if this <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> object is not a map, returns the undefined value.</p>
    /// <p>This function is equivalent to:</p>
    /// <pre class="cpp">
    ///
    ///   value<span class="operator">.</span>toMap()<span class="operator">.</span>value(key);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalueref.html#operator-5b-5d-2">operator[]</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d">QCborMap::operator[]</a>, <a href="http://doc.qt.io/qt-5/qcbormap.html#value">QCborMap::value</a>(), and <a href="http://doc.qt.io/qt-5/qcbormap.html#find">QCborMap::find</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn index_q_latin1_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_operator__5(
            self as *const crate::QCborValueRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborValueRef::operator[](qint64 key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalueref.html#operator-5b-5d-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If this <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> refers to a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>, searches elements for the value whose key matches <i>key</i>. If this is a <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>, returns the element whose index is <i>key</i>. If there's no matching value in the array or map, or if this <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> object is not an array or map, returns the undefined value.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalueref.html#operator-5b-5d">operator[]</a>, <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d">QCborMap::operator[]</a>, <a href="http://doc.qt.io/qt-5/qcbormap.html#value">QCborMap::value</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#find">QCborMap::find</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-5b-5d">QCborArray::operator[]</a>, and <a href="http://doc.qt.io/qt-5/qcborarray.html#at">QCborArray::at</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn index_i64(&self, key: i64) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_operator__6(
            self as *const crate::QCborValueRef,
            key,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValueRef QCborValueRef::operator[](qint64 key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalueref.html#operator-5b-5d-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> that can be used to read or modify the entry in this, as a map or array, with the given <i>key</i>. When this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> or, for 0 &lt;= key &lt; 0x10000, a <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>, this function is equivalent to the matching operator[] on that map or array.</p>
    /// <p>Before returning the reference: if the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> referenced was an array but the key is out of range, the array is first converted to a map (so that <code>map[i]</code> is <code>array[i]</code> for each index, <code>i</code>, with valid <code>array[i]</code>); otherwise, if it was not a map it will be over-written with an empty map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalueref.html#operator-5b-5d">operator[]</a>, <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d">QCborMap::operator[]</a>, <a href="http://doc.qt.io/qt-5/qcbormap.html#value">QCborMap::value</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#find">QCborMap::find</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-5b-5d">QCborArray::operator[]</a>, and <a href="http://doc.qt.io/qt-5/qcborarray.html#at">QCborArray::at</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn index_i64_mut(&mut self, key: i64) -> ::cpp_utils::CppBox<crate::QCborValueRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_operator__7(
            self as *mut crate::QCborValueRef,
            key,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValueRef QCborValueRef::operator[](QLatin1String key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalueref.html#operator-5b-5d-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> that can be used to read or modify the entry in this, as a map, with the given <i>key</i>. When this <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> is a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>, this function is equivalent to the matching operator[] on that map.</p>
    /// <p>Before returning the reference: if the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> referenced was an array, it is first converted to a map (so that <code>map[i]</code> is <code>array[i]</code> for each index, <code>i</code>, with valid <code>array[i]</code>); otherwise, if it was not a map it will be over-written with an empty map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalueref.html#operator-5b-5d-2">operator[]</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d">QCborMap::operator[]</a>, <a href="http://doc.qt.io/qt-5/qcbormap.html#value">QCborMap::value</a>(), and <a href="http://doc.qt.io/qt-5/qcbormap.html#find">QCborMap::find</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn index_q_latin1_string_mut(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValueRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_operator__8(
            self as *mut crate::QCborValueRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> that can be used to read or modify the entry in this, as a map, with the given <i>key</i>. When this <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> refers to a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>, this function is equivalent to the matching operator[] on that map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValueRef QCborValueRef::operator[](const QString& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalueref.html#operator-5b-5d-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> that can be used to read or modify the entry in this, as a map, with the given <i>key</i>. When this <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> refers to a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>, this function is equivalent to the matching operator[] on that map.</p>
    /// <p>Before returning the reference: if the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> referenced was an array, it is first converted to a map (so that <code>map[i]</code> is <code>array[i]</code> for each index, <code>i</code>, with valid <code>array[i]</code>); otherwise, if it was not a map it will be over-written with an empty map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalueref.html#operator-5b-5d-2">operator[]</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d">QCborMap::operator[]</a>, <a href="http://doc.qt.io/qt-5/qcbormap.html#value">QCborMap::value</a>(), and <a href="http://doc.qt.io/qt-5/qcbormap.html#find">QCborMap::find</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn index_q_string_mut(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValueRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_operator__9(
            self as *mut crate::QCborValueRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCborValueRef::isArray() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_array(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_isArray(self as *const crate::QCborValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCborValueRef::isBool() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_bool(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_isBool(self as *const crate::QCborValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCborValueRef::isByteArray() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_byte_array(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_isByteArray(self as *const crate::QCborValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCborValueRef::isContainer() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_container(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_isContainer(self as *const crate::QCborValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCborValueRef::isDateTime() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_date_time(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_isDateTime(self as *const crate::QCborValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCborValueRef::isDouble() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_double(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_isDouble(self as *const crate::QCborValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCborValueRef::isFalse() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_false(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_isFalse(self as *const crate::QCborValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCborValueRef::isInteger() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_integer(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_isInteger(self as *const crate::QCborValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCborValueRef::isInvalid() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_invalid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_isInvalid(self as *const crate::QCborValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCborValueRef::isMap() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_map(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_isMap(self as *const crate::QCborValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCborValueRef::isNull() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_isNull(self as *const crate::QCborValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCborValueRef::isRegularExpression() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_regular_expression(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_isRegularExpression(
            self as *const crate::QCborValueRef,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCborValueRef::isSimpleType() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_simple_type_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_isSimpleType(
            self as *const crate::QCborValueRef,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCborValueRef::isSimpleType(QCborSimpleType st) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_simple_type_1a(&self, st: crate::QCborSimpleType) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_isSimpleType1(
            self as *const crate::QCborValueRef,
            st,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCborValueRef::isString() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_string(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_isString(self as *const crate::QCborValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCborValueRef::isTag() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_tag(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_isTag(self as *const crate::QCborValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCborValueRef::isTrue() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_true(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_isTrue(self as *const crate::QCborValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCborValueRef::isUndefined() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_undefined(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_isUndefined(self as *const crate::QCborValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCborValueRef::isUrl() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_url(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_isUrl(self as *const crate::QCborValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCborValueRef::isUuid() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_uuid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_isUuid(self as *const crate::QCborValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborValueRef::QCborValueRef(const QCborValueRef& arg1)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn new_copy(
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValueRef>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValueRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_QCborValueRef(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValueRef>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QCborTag QCborValueRef::tag(QCborTag defaultValue = …) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn tag_1a(&self, default_value: crate::QCborTag) -> crate::QCborTag {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_tag(
            self as *const crate::QCborValueRef,
            default_value,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```QCborTag QCborValueRef::tag() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn tag_0a(&self) -> crate::QCborTag {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_tag1(self as *const crate::QCborValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborValueRef::taggedValue(const QCborValue& defaultValue = …) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn tagged_value_1a(
        &self,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_taggedValue(
            self as *const crate::QCborValueRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborValueRef::taggedValue() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn tagged_value_0a(&self) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_taggedValue1(
            self as *const crate::QCborValueRef,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QCborArray QCborValueRef::toArray() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_array_0a(&self) -> ::cpp_utils::CppBox<crate::QCborArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toArray(
            self as *const crate::QCborValueRef,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QCborArray QCborValueRef::toArray(const QCborArray& a) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_array_1a(
        &self,
        a: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborArray>>,
    ) -> ::cpp_utils::CppBox<crate::QCborArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toArray1(
            self as *const crate::QCborValueRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborArray>>::cast_into(a).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCborValueRef::toBool(bool defaultValue = …) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_bool_1a(&self, default_value: bool) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toBool(
            self as *const crate::QCborValueRef,
            default_value,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QCborValueRef::toBool() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_bool_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toBool1(self as *const crate::QCborValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```QByteArray QCborValueRef::toByteArray(const QByteArray& defaultValue = …) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_byte_array_1a(
        &self,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toByteArray(
            self as *const crate::QCborValueRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QByteArray QCborValueRef::toByteArray() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_byte_array_0a(&self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toByteArray1(
            self as *const crate::QCborValueRef,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QByteArray QCborValueRef::toCbor(QFlags<QCborValue::EncodingOption> opt = …)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_cbor_q_flags_encoding_option(
        &mut self,
        opt: crate::QFlags<crate::q_cbor_value::EncodingOption>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toCbor(
            self as *mut crate::QCborValueRef,
            opt.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```void QCborValueRef::toCbor(QCborStreamWriter& writer, QFlags<QCborValue::EncodingOption> opt = …)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_cbor_q_cbor_stream_writer_q_flags_encoding_option(
        &mut self,
        writer: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QCborStreamWriter>>,
        opt: crate::QFlags<crate::q_cbor_value::EncodingOption>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toCbor1(
            self as *mut crate::QCborValueRef,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QCborStreamWriter>>::cast_into(
                writer,
            )
            .as_mut_raw_ptr(),
            opt.to_int(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```QByteArray QCborValueRef::toCbor()```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_cbor(&mut self) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toCbor2(self as *mut crate::QCborValueRef);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```void QCborValueRef::toCbor(QCborStreamWriter& writer)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_cbor_q_cbor_stream_writer(
        &mut self,
        writer: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QCborStreamWriter>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toCbor3(
            self as *mut crate::QCborValueRef,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QCborStreamWriter>>::cast_into(
                writer,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```QDateTime QCborValueRef::toDateTime(const QDateTime& defaultValue = …) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_date_time_1a(
        &self,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
    ) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toDateTime(
            self as *const crate::QCborValueRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QDateTime QCborValueRef::toDateTime() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_date_time_0a(&self) -> ::cpp_utils::CppBox<crate::QDateTime> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toDateTime1(
            self as *const crate::QCborValueRef,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QString QCborValueRef::toDiagnosticNotation(QFlags<QCborValue::DiagnosticNotationOption> opt = …)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_diagnostic_notation_1a(
        &mut self,
        opt: crate::QFlags<crate::q_cbor_value::DiagnosticNotationOption>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toDiagnosticNotation(
            self as *mut crate::QCborValueRef,
            opt.to_int(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QString QCborValueRef::toDiagnosticNotation()```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_diagnostic_notation_0a(&mut self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toDiagnosticNotation1(
            self as *mut crate::QCborValueRef,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```double QCborValueRef::toDouble(double defaultValue = …) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_double_1a(
        &self,
        default_value: ::std::os::raw::c_double,
    ) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toDouble(
            self as *const crate::QCborValueRef,
            default_value,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```double QCborValueRef::toDouble() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_double_0a(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toDouble1(self as *const crate::QCborValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```qint64 QCborValueRef::toInteger(qint64 defaultValue = …) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_integer_1a(&self, default_value: i64) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toInteger(
            self as *const crate::QCborValueRef,
            default_value,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```qint64 QCborValueRef::toInteger() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_integer_0a(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toInteger1(self as *const crate::QCborValueRef)
    }

    /// Calls C++ function: <span style='color: green;'>```QJsonValue QCborValueRef::toJsonValue() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_json_value(&self) -> ::cpp_utils::CppBox<crate::QJsonValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toJsonValue(
            self as *const crate::QCborValueRef,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QCborMap QCborValueRef::toMap() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_map_0a(&self) -> ::cpp_utils::CppBox<crate::QCborMap> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toMap(self as *const crate::QCborValueRef);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QCborMap QCborValueRef::toMap(const QCborMap& m) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_map_1a(
        &self,
        m: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborMap>>,
    ) -> ::cpp_utils::CppBox<crate::QCborMap> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toMap1(
            self as *const crate::QCborValueRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborMap>>::cast_into(m).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborValueRef::operator QCborValue() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_q_cbor_value(&self) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_operator_QCborValue(
            self as *const crate::QCborValueRef,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QRegularExpression QCborValueRef::toRegularExpression(const QRegularExpression& defaultValue = …) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_regular_expression_1a(
        &self,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
    ) -> ::cpp_utils::CppBox<crate::QRegularExpression> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toRegularExpression(
            self as *const crate::QCborValueRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(
                default_value,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QRegularExpression QCborValueRef::toRegularExpression() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_regular_expression_0a(
        &self,
    ) -> ::cpp_utils::CppBox<crate::QRegularExpression> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toRegularExpression1(
            self as *const crate::QCborValueRef,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QString QCborValueRef::toString(const QString& defaultValue = …) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_string_1a(
        &self,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toString(
            self as *const crate::QCborValueRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QString QCborValueRef::toString() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_string_0a(&self) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toString1(
            self as *const crate::QCborValueRef,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QUrl QCborValueRef::toUrl(const QUrl& defaultValue = …) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_url_1a(
        &self,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
    ) -> ::cpp_utils::CppBox<crate::QUrl> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toUrl(
            self as *const crate::QCborValueRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QUrl QCborValueRef::toUrl() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_url_0a(&self) -> ::cpp_utils::CppBox<crate::QUrl> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toUrl1(self as *const crate::QCborValueRef);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QUuid QCborValueRef::toUuid(const QUuid& defaultValue = …) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_uuid_1a(
        &self,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUuid>>,
    ) -> ::cpp_utils::CppBox<crate::QUuid> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toUuid(
            self as *const crate::QCborValueRef,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUuid>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QUuid QCborValueRef::toUuid() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_uuid_0a(&self) -> ::cpp_utils::CppBox<crate::QUuid> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toUuid1(
            self as *const crate::QCborValueRef,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QVariant QCborValueRef::toVariant() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_variant(&self) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborValueRef_toVariant(
            self as *const crate::QCborValueRef,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QCborValue::Type QCborValueRef::type() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn type_(&self) -> crate::q_cbor_value::Type {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_type(self as *const crate::QCborValueRef)
    }
}

pub mod q_cbor_array {
    //! C++ type: <span style='color: green;'>```QCborArray```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html">QCborArray::Iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QCborArray::Iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html">QCborArray::Iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qcborarray-iterator.html">QCborArray::Iterator</a> allows you to iterate over a <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> and to modify the array item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>, use <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html">QCborArray::ConstIterator</a> instead. It is generally a good practice to use <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html">QCborArray::ConstIterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> through the iterator. Const iterators are slightly faster and improve code readability.</p>
    /// <p>Iterators are initialized by using a <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> function like <a href="http://doc.qt.io/qt-5/qcborarray.html#begin">QCborArray::begin</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#end">QCborArray::end</a>(), or <a href="http://doc.qt.io/qt-5/qcborarray.html#insert">QCborArray::insert</a>(). Iteration is only possible after that.</p>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>Multiple iterators can be used on the same array. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> will render all existing iterators undefined.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// Calls C++ function: <span style='color: green;'>```QCborArray::Iterator& QCborArray::Iterator::operator=(const QCborArray::Iterator& other)```</span>.
        ///
        ///
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_cbor_array::Iterator>>,
        ) -> ::cpp_utils::MutRef<crate::q_cbor_array::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_Iterator_operator_(self as *mut crate::q_cbor_array::Iterator, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_cbor_array::Iterator>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborArray::Iterator QCborArray::Iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator, <code>it--</code>, makes the preceding item current and returns an iterator to the previously current item.</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_cbor_array::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_Iterator_operator__13(
                self as *mut crate::q_cbor_array::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborArray::Iterator QCborArray::Iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator, <code>it++</code>, advances the iterator to the next item in the array and returns an iterator to the previously current item.</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_cbor_array::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_Iterator_operator__11(
                self as *mut crate::q_cbor_array::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a modifiable reference to the item at a position <i>j</i> steps forward from the item pointed to by this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborValueRef QCborArray::Iterator::operator[](long long j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the item at a position <i>j</i> steps forward from the item pointed to by this iterator.</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> iterators behave like C++ pointers.</p>
        /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> and <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>. If you assign to it, the assignment will apply to the element in the <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> or <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> from which you got the reference.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-2b">operator+</a>().</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn index(
            &mut self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::QCborValueRef> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_Iterator_operator__1(
                self as *mut crate::q_cbor_array::Iterator,
                j,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborArray::Iterator::Iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#Iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#begin">QCborArray::begin</a>() and <a href="http://doc.qt.io/qt-5/qcborarray.html#end">QCborArray::end</a>().</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_cbor_array::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_Iterator_Iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Makes a copy of <i>other</i>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborArray::Iterator::Iterator(const QCborArray::Iterator& arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#Iterator-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes a copy of <i>other</i>.</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn new_copy(
            arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_cbor_array::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_cbor_array::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_Iterator_Iterator1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_cbor_array::Iterator>>::cast_into(arg1).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to a modifiable reference to the current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborValueRef* QCborArray::Iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to a modifiable reference to the current item.</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::MutPtr<crate::QCborValueRef> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_Iterator_operator__(
                self as *const crate::q_cbor_array::Iterator,
            );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html">QCborArray::ConstIterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QCborArray::ConstIterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html">QCborArray::ConstIterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html">QCborArray::ConstIterator</a> allows you to iterate over a <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>. If you want to modify the <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html">QCborArray::Iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html">QCborArray::ConstIterator</a>, even on a non-const <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>, when you don't need to change the <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> through the iterator. Const iterators are slightly faster and improves code readability.</p>
    /// <p>Iterators are initialized by using a <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> function like <a href="http://doc.qt.io/qt-5/qcborarray.html#begin">QCborArray::begin</a>() or <a href="http://doc.qt.io/qt-5/qcborarray.html#end">QCborArray::end</a>(). Iteration is only possible after that.</p>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>Multiple iterators can be used on the same array. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> will render all existing iterators undefined.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>Makes this iterator a copy of <i>other</i> and returns a reference to this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborArray::ConstIterator& QCborArray::ConstIterator::operator=(const QCborArray::ConstIterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes this iterator a copy of <i>other</i> and returns a reference to this iterator.</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>>,
        ) -> ::cpp_utils::MutRef<crate::q_cbor_array::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_ConstIterator_operator_(self as *mut crate::q_cbor_array::ConstIterator, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborArray::ConstIterator QCborArray::ConstIterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator, <code>it--</code>, makes the preceding item current and returns an iterator to the previously current item.</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_cbor_array::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_ConstIterator_operator__13(
                self as *mut crate::q_cbor_array::ConstIterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborArray::ConstIterator QCborArray::ConstIterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator, <code>it++</code>, advances the iterator to the next item in the array and returns an iterator to the previously current item.</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_cbor_array::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_ConstIterator_operator__11(
                self as *mut crate::q_cbor_array::ConstIterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the item at a position <i>j</i> steps forward from the item pointed to by this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborValueRef QCborArray::ConstIterator::operator[](long long j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at a position <i>j</i> steps forward from the item pointed to by this iterator.</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> iterators behave like C++ pointers.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-2b">operator+</a>().</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn index(
            &mut self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::QCborValueRef> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_ConstIterator_operator__1(
                self as *mut crate::q_cbor_array::ConstIterator,
                j,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborArray::ConstIterator::ConstIterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#ConstIterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#constBegin">QCborArray::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qcborarray.html#constEnd">QCborArray::constEnd</a>().</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_cbor_array::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_ConstIterator_ConstIterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs a copy of <i>other</i>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborArray::ConstIterator::ConstIterator(const QCborArray::ConstIterator& arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#ConstIterator-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn new_copy(
            arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_cbor_array::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_ConstIterator_ConstIterator1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>>::cast_into(arg1).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QCborValueRef* QCborArray::ConstIterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item.</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::Ptr<crate::QCborValueRef> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_ConstIterator_operator__(
                self as *const crate::q_cbor_array::ConstIterator,
            );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::ops::Indirection for &crate::q_cbor_array::Iterator {
        type Output = ::cpp_utils::CppBox<crate::QCborValueRef>;
        /// <p>Returns a modifiable reference to the current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborValueRef QCborArray::Iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item.</p>
        /// <p>You can change the value of an item by using operator*() on the left side of an assignment.</p>
        /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> and <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>. If you assign to it, the assignment will apply to the element in the <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> or <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> from which you got the reference.</p></div>
        fn indirection(self) -> ::cpp_utils::CppBox<crate::QCborValueRef> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_Iterator_operator_1(
                    self as *const crate::q_cbor_array::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_cbor_array::Iterator>>
        for crate::q_cbor_array::Iterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborArray::Iterator::operator==(const QCborArray::Iterator& o) const```</span>.
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_cbor_array::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_Iterator_operator__2(
                    self as *const crate::q_cbor_array::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_cbor_array::Iterator>>
        for crate::q_cbor_array::Iterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborArray::Iterator::operator<(const QCborArray::Iterator& other) const```</span>.
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_cbor_array::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_Iterator_operator_2(
                    self as *const crate::q_cbor_array::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_cbor_array::Iterator>>
        for crate::q_cbor_array::Iterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborArray::Iterator::operator<=(const QCborArray::Iterator& other) const```</span>.
        fn le(&self, other: &::cpp_utils::Ref<crate::q_cbor_array::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_Iterator_operator__4(
                    self as *const crate::q_cbor_array::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_cbor_array::Iterator>>
        for crate::q_cbor_array::Iterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborArray::Iterator::operator>(const QCborArray::Iterator& other) const```</span>.
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_cbor_array::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_Iterator_operator_3(
                    self as *const crate::q_cbor_array::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_cbor_array::Iterator>>
        for crate::q_cbor_array::Iterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborArray::Iterator::operator>=(const QCborArray::Iterator& other) const```</span>.
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_cbor_array::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_Iterator_operator__5(
                    self as *const crate::q_cbor_array::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>>
        for crate::q_cbor_array::Iterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborArray::Iterator::operator==(const QCborArray::ConstIterator& o) const```</span>.
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_Iterator_operator__6(
                    self as *const crate::q_cbor_array::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>>
        for crate::q_cbor_array::Iterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborArray::Iterator::operator<(const QCborArray::ConstIterator& other) const```</span>.
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_Iterator_operator_4(
                    self as *const crate::q_cbor_array::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>>
        for crate::q_cbor_array::Iterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborArray::Iterator::operator<=(const QCborArray::ConstIterator& other) const```</span>.
        fn le(&self, other: &::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_Iterator_operator__8(
                    self as *const crate::q_cbor_array::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>>
        for crate::q_cbor_array::Iterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborArray::Iterator::operator>(const QCborArray::ConstIterator& other) const```</span>.
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_Iterator_operator_5(
                    self as *const crate::q_cbor_array::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>>
        for crate::q_cbor_array::Iterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborArray::Iterator::operator>=(const QCborArray::ConstIterator& other) const```</span>.
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_Iterator_operator__9(
                    self as *const crate::q_cbor_array::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::ops::Increment for &mut crate::q_cbor_array::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_cbor_array::Iterator>;
        /// <p>The prefix ++ operator, <code>++it</code>, advances the iterator to the next item in the array and returns this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborArray::Iterator& QCborArray::Iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator, <code>++it</code>, advances the iterator to the next item in the array and returns this iterator.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qcborarray.html#end">QCborArray::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_cbor_array::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_Iterator_operator__10(
                    self as *mut crate::q_cbor_array::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::ops::Decrement for &mut crate::q_cbor_array::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_cbor_array::Iterator>;
        /// <p>The prefix -- operator, <code>--it</code>, makes the preceding item current and returns this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborArray::Iterator& QCborArray::Iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator, <code>--it</code>, makes the preceding item current and returns this iterator.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qcborarray.html#begin">QCborArray::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_cbor_array::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_Iterator_operator__12(
                    self as *mut crate::q_cbor_array::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::ops::AddAssign<::std::os::raw::c_longlong> for crate::q_cbor_array::Iterator {
        /// <p>Advances the iterator by <i>j</i> positions. If <i>j</i> is negative, the iterator goes backward. Returns a reference to this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborArray::Iterator& QCborArray::Iterator::operator+=(long long j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> positions. If <i>j</i> is negative, the iterator goes backward. Returns a reference to this iterator.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_Iterator_operator__14(
                    self as *mut crate::q_cbor_array::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::ops::SubAssign<::std::os::raw::c_longlong> for crate::q_cbor_array::Iterator {
        /// <p>Makes the iterator go back by <i>j</i> positions. If <i>j</i> is negative, the iterator goes forward. Returns a reference to this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborArray::Iterator& QCborArray::Iterator::operator-=(long long j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> positions. If <i>j</i> is negative, the iterator goes forward. Returns a reference to this iterator.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_Iterator_operator__15(
                    self as *mut crate::q_cbor_array::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_cbor_array::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_cbor_array::Iterator>;
        /// <p>Returns an iterator to the item at position <i>j</i> steps forward from this iterator. If <i>j</i> is negative, the iterator goes backward.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborArray::Iterator QCborArray::Iterator::operator+(long long j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at position <i>j</i> steps forward from this iterator. If <i>j</i> is negative, the iterator goes backward.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-">operator-</a>() and <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-2b-eq">operator+=</a>().</p></div>
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_cbor_array::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_Iterator_operator_6(
                    self as *const crate::q_cbor_array::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_cbor_array::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_cbor_array::Iterator>;
        /// <p>Returns an iterator to the item at position <i>j</i> steps backward from this iterator. If <i>j</i> is negative, the iterator goes forward.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborArray::Iterator QCborArray::Iterator::operator-(long long j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at position <i>j</i> steps backward from this iterator. If <i>j</i> is negative, the iterator goes forward.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator--eq">operator-=</a>().</p></div>
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_cbor_array::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_Iterator_operator_7(
                    self as *const crate::q_cbor_array::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_cbor_array::Iterator>>
        for &crate::q_cbor_array::Iterator
    {
        type Output = ::std::os::raw::c_longlong;
        /// <p>Returns the offset of this iterator relative to <i>other</i>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```long long QCborArray::Iterator::operator-(QCborArray::Iterator j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator--1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the offset of this iterator relative to <i>other</i>.</p></div>
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_cbor_array::Iterator>,
        ) -> ::std::os::raw::c_longlong {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_Iterator_operator_8(
                    self as *const crate::q_cbor_array::Iterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::ops::Indirection for &crate::q_cbor_array::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::QCborValueRef>;
        /// <p>Returns the current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborValueRef QCborArray::ConstIterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current item.</p></div>
        fn indirection(self) -> ::cpp_utils::CppBox<crate::QCborValueRef> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_ConstIterator_operator_1(
                    self as *const crate::q_cbor_array::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_cbor_array::Iterator>>
        for crate::q_cbor_array::ConstIterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborArray::ConstIterator::operator==(const QCborArray::Iterator& o) const```</span>.
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_cbor_array::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_ConstIterator_operator__2(
                    self as *const crate::q_cbor_array::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_cbor_array::Iterator>>
        for crate::q_cbor_array::ConstIterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborArray::ConstIterator::operator<(const QCborArray::Iterator& other) const```</span>.
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_cbor_array::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_ConstIterator_operator_2(
                    self as *const crate::q_cbor_array::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_cbor_array::Iterator>>
        for crate::q_cbor_array::ConstIterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborArray::ConstIterator::operator<=(const QCborArray::Iterator& other) const```</span>.
        fn le(&self, other: &::cpp_utils::Ref<crate::q_cbor_array::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_ConstIterator_operator__4(
                    self as *const crate::q_cbor_array::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_cbor_array::Iterator>>
        for crate::q_cbor_array::ConstIterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborArray::ConstIterator::operator>(const QCborArray::Iterator& other) const```</span>.
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_cbor_array::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_ConstIterator_operator_3(
                    self as *const crate::q_cbor_array::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_cbor_array::Iterator>>
        for crate::q_cbor_array::ConstIterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborArray::ConstIterator::operator>=(const QCborArray::Iterator& other) const```</span>.
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_cbor_array::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_ConstIterator_operator__5(
                    self as *const crate::q_cbor_array::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>>
        for crate::q_cbor_array::ConstIterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborArray::ConstIterator::operator==(const QCborArray::ConstIterator& o) const```</span>.
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_ConstIterator_operator__6(
                    self as *const crate::q_cbor_array::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>>
        for crate::q_cbor_array::ConstIterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborArray::ConstIterator::operator<(const QCborArray::ConstIterator& other) const```</span>.
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_ConstIterator_operator_4(
                    self as *const crate::q_cbor_array::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>>
        for crate::q_cbor_array::ConstIterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborArray::ConstIterator::operator<=(const QCborArray::ConstIterator& other) const```</span>.
        fn le(&self, other: &::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_ConstIterator_operator__8(
                    self as *const crate::q_cbor_array::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>>
        for crate::q_cbor_array::ConstIterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborArray::ConstIterator::operator>(const QCborArray::ConstIterator& other) const```</span>.
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_ConstIterator_operator_5(
                    self as *const crate::q_cbor_array::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>>
        for crate::q_cbor_array::ConstIterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborArray::ConstIterator::operator>=(const QCborArray::ConstIterator& other) const```</span>.
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_ConstIterator_operator__9(
                    self as *const crate::q_cbor_array::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::ops::Increment for &mut crate::q_cbor_array::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_cbor_array::ConstIterator>;
        /// <p>The prefix ++ operator, <code>++it</code>, advances the iterator to the next item in the array and returns this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborArray::ConstIterator& QCborArray::ConstIterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator, <code>++it</code>, advances the iterator to the next item in the array and returns this iterator.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qcborarray.html#end">QCborArray::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_cbor_array::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_ConstIterator_operator__10(
                    self as *mut crate::q_cbor_array::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::ops::Decrement for &mut crate::q_cbor_array::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_cbor_array::ConstIterator>;
        /// <p>The prefix -- operator, <code>--it</code>, makes the preceding item current and returns this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborArray::ConstIterator& QCborArray::ConstIterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator, <code>--it</code>, makes the preceding item current and returns this iterator.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qcborarray.html#begin">QCborArray::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_cbor_array::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_ConstIterator_operator__12(
                    self as *mut crate::q_cbor_array::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::ops::AddAssign<::std::os::raw::c_longlong> for crate::q_cbor_array::ConstIterator {
        /// <p>Advances the iterator by <i>j</i> positions. If <i>j</i> is negative, the iterator goes backward. Returns a reference to this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborArray::ConstIterator& QCborArray::ConstIterator::operator+=(long long j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> positions. If <i>j</i> is negative, the iterator goes backward. Returns a reference to this iterator.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_ConstIterator_operator__14(
                    self as *mut crate::q_cbor_array::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::ops::SubAssign<::std::os::raw::c_longlong> for crate::q_cbor_array::ConstIterator {
        /// <p>Makes the iterator go back by <i>j</i> positions. If <i>j</i> is negative, the iterator goes forward. Returns a reference to this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborArray::ConstIterator& QCborArray::ConstIterator::operator-=(long long j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> positions. If <i>j</i> is negative, the iterator goes forward. Returns a reference to this iterator.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_ConstIterator_operator__15(
                    self as *mut crate::q_cbor_array::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_cbor_array::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_cbor_array::ConstIterator>;
        /// <p>Returns an iterator to the item at a position <i>j</i> steps forward from this iterator. If <i>j</i> is negative, the iterator goes backward.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborArray::ConstIterator QCborArray::ConstIterator::operator+(long long j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at a position <i>j</i> steps forward from this iterator. If <i>j</i> is negative, the iterator goes backward.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-">operator-</a>() and <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-2b-eq">operator+=</a>().</p></div>
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_cbor_array::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_ConstIterator_operator_6(
                    self as *const crate::q_cbor_array::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_cbor_array::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_cbor_array::ConstIterator>;
        /// <p>Returns an iterator to the item at a position <i>j</i> steps backward from this iterator. If <i>j</i> is negative, the iterator goes forward.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborArray::ConstIterator QCborArray::ConstIterator::operator-(long long j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at a position <i>j</i> steps backward from this iterator. If <i>j</i> is negative, the iterator goes forward.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator--eq">operator-=</a>().</p></div>
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_cbor_array::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_ConstIterator_operator_7(
                    self as *const crate::q_cbor_array::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>>
        for &crate::q_cbor_array::ConstIterator
    {
        type Output = ::std::os::raw::c_longlong;
        /// <p>Returns the offset of this iterator relative to <i>other</i>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```long long QCborArray::ConstIterator::operator-(QCborArray::ConstIterator j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator--1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the offset of this iterator relative to <i>other</i>.</p></div>
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>,
        ) -> ::std::os::raw::c_longlong {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborArray_ConstIterator_operator_8(
                    self as *const crate::q_cbor_array::ConstIterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::CppDeletable for crate::q_cbor_array::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html">QCborArray::Iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QCborArray::Iterator::~Iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html">QCborArray::Iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qcborarray-iterator.html">QCborArray::Iterator</a> allows you to iterate over a <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> and to modify the array item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>, use <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html">QCborArray::ConstIterator</a> instead. It is generally a good practice to use <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html">QCborArray::ConstIterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> through the iterator. Const iterators are slightly faster and improve code readability.</p>
        /// <p>Iterators are initialized by using a <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> function like <a href="http://doc.qt.io/qt-5/qcborarray.html#begin">QCborArray::begin</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#end">QCborArray::end</a>(), or <a href="http://doc.qt.io/qt-5/qcborarray.html#insert">QCborArray::insert</a>(). Iteration is only possible after that.</p>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>Multiple iterators can be used on the same array. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> will render all existing iterators undefined.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QCborArray_Iterator_dIterator(
                self as *mut crate::q_cbor_array::Iterator,
            )
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::CppDeletable for crate::q_cbor_array::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html">QCborArray::ConstIterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QCborArray::ConstIterator::~ConstIterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html">QCborArray::ConstIterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html">QCborArray::ConstIterator</a> allows you to iterate over a <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>. If you want to modify the <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html">QCborArray::Iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html">QCborArray::ConstIterator</a>, even on a non-const <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>, when you don't need to change the <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> through the iterator. Const iterators are slightly faster and improves code readability.</p>
        /// <p>Iterators are initialized by using a <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> function like <a href="http://doc.qt.io/qt-5/qcborarray.html#begin">QCborArray::begin</a>() or <a href="http://doc.qt.io/qt-5/qcborarray.html#end">QCborArray::end</a>(). Iteration is only possible after that.</p>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>Multiple iterators can be used on the same array. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> will render all existing iterators undefined.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QCborArray_ConstIterator_dConstIterator(
                self as *mut crate::q_cbor_array::ConstIterator,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> class is used to hold an array of CBOR elements.</p>
///
/// C++ class: <span style='color: green;'>```QCborArray```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qcborarray.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> class is used to hold an array of CBOR elements.</p>
/// <p>This class can be used to hold one sequential container in CBOR (an array). CBOR is the Concise Binary Object Representation, a very compact form of binary data encoding that is a superset of JSON. It was created by the IETF Constrained RESTful Environments (CoRE) WG, which has used it in many new RFCs. It is meant to be used alongside the <a href="https://tools.ietf.org/html/rfc7252">CoAP protocol</a>.</p>
/// <p><a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> is very similar to <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a> and <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and its API is almost identical to those two classes. It can also be converted to and from those two, though there may be loss of information in some conversions.</p></div>
#[repr(C)]
pub struct QCborArray {
    _unused: u8,
}
impl QCborArray {
    /// Calls C++ function: <span style='color: green;'>```void QCborArray::append(const QCborValue& value)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn append(
        &mut self,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborArray_append(
            self as *mut crate::QCborArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> element at position <i>i</i> in the array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborArray::at(long long i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> element at position <i>i</i> in the array.</p>
    /// <p>If the array is smaller than <i>i</i> elements, this function returns a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> containing an undefined value. For that reason, it is not possible with this function to tell apart the situation where the array is not large enough from the case where the array starts with an undefined value.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#operator-5b-5d">operator[]</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#first">first</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#last">last</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#removeAt">removeAt</a>(), and <a href="http://doc.qt.io/qt-5/qcborarray.html#takeAt">takeAt</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn at(
        &self,
        i: ::std::os::raw::c_longlong,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborArray_at(self as *const crate::QCborArray, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns an array iterator pointing to the first item in this array. If the array is empty, then this function returns the same as <a href="http://doc.qt.io/qt-5/qcborarray.html#end">end</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborArray::ConstIterator QCborArray::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an array iterator pointing to the first item in this array. If the array is empty, then this function returns the same as <a href="http://doc.qt.io/qt-5/qcborarray.html#end">end</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qcborarray.html#constEnd">constEnd</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn cbegin(&self) -> ::cpp_utils::CppBox<crate::q_cbor_array::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborArray_cbegin(self as *const crate::QCborArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns an array iterator pointing to just after the last element in this array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborArray::ConstIterator QCborArray::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an array iterator pointing to just after the last element in this array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qcborarray.html#constEnd">constEnd</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn cend(&self) -> ::cpp_utils::CppBox<crate::q_cbor_array::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborArray_cend(self as *const crate::QCborArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Empties this array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborArray::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Empties this array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#isEmpty">isEmpty</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCborArray_clear(self as *mut crate::QCborArray)
    }

    /// <p>Compares this array and <i>other</i>, comparing each element in sequence, and returns an integer that indicates whether this array should be sorted before (if the result is negative) or after <i>other</i> (if the result is positive). If this function returns 0, the two arrays are equal and contain the same elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QCborArray::compare(const QCborArray& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#compare">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares this array and <i>other</i>, comparing each element in sequence, and returns an integer that indicates whether this array should be sorted before (if the result is negative) or after <i>other</i> (if the result is positive). If this function returns 0, the two arrays are equal and contain the same elements.</p>
    /// <p>For more information on CBOR sorting order, see <a href="http://doc.qt.io/qt-5/qcborvalue.html#compare">QCborValue::compare</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#compare">QCborValue::compare</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#compare">QCborMap::compare</a>(), and <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-eq-eq">operator==</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn compare(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborArray>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QCborArray_compare(
            self as *const crate::QCborArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborArray>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns an array iterator pointing to the first item in this array. If the array is empty, then this function returns the same as <a href="http://doc.qt.io/qt-5/qcborarray.html#end">end</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborArray::ConstIterator QCborArray::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an array iterator pointing to the first item in this array. If the array is empty, then this function returns the same as <a href="http://doc.qt.io/qt-5/qcborarray.html#end">end</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qcborarray.html#constEnd">constEnd</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn const_begin(&self) -> ::cpp_utils::CppBox<crate::q_cbor_array::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborArray_constBegin(self as *const crate::QCborArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns an array iterator pointing to just after the last element in this array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborArray::ConstIterator QCborArray::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an array iterator pointing to just after the last element in this array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qcborarray.html#end">end</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn const_end(&self) -> ::cpp_utils::CppBox<crate::q_cbor_array::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborArray_constEnd(self as *const crate::QCborArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns true if this array contains an element that is equal to <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborArray::contains(const QCborValue& value) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if this array contains an element that is equal to <i>value</i>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn contains(
        &self,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborArray_contains(
            self as *const crate::QCborArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>Replaces the contents of this array with that found in <i>other</i>, then returns a reference to this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborArray& QCborArray::operator=(const QCborArray& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the contents of this array with that found in <i>other</i>, then returns a reference to this object.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborArray>>,
    ) -> ::cpp_utils::MutRef<crate::QCborArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_operator_(
            self as *mut crate::QCborArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborArray>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Synonym for <a href="http://doc.qt.io/qt-5/qcborarray.html#isEmpty">isEmpty</a>(). This function is provided for compatibility with generic code that uses the Standard Library API.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborArray::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Synonym for <a href="http://doc.qt.io/qt-5/qcborarray.html#isEmpty">isEmpty</a>(). This function is provided for compatibility with generic code that uses the Standard Library API.</p>
    /// <p>Returns true if this array is empty (<a href="http://doc.qt.io/qt-5/qcborarray.html#size">size</a>() == 0).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qcborarray.html#size">size</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborArray_empty(self as *const crate::QCborArray)
    }

    /// Calls C++ function: <span style='color: green;'>```QCborArray::Iterator QCborArray::erase(QCborArray::Iterator it)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn erase_iterator(
        &mut self,
        it: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_cbor_array::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_cbor_array::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_erase(
            self as *mut crate::QCborArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_cbor_array::Iterator>>::cast_into(it)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QCborArray::Iterator QCborArray::erase(QCborArray::ConstIterator it)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn erase_const_iterator(
        &mut self,
        it: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_cbor_array::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_erase1(self as *mut crate::QCborArray, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>>::cast_into(it).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborArray::extract(QCborArray::ConstIterator it)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn extract_const_iterator(
        &mut self,
        it: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_extract(self as *mut crate::QCborArray, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>>::cast_into(it).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborArray::extract(QCborArray::Iterator it)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn extract_iterator(
        &mut self,
        it: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_cbor_array::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_extract1(
            self as *mut crate::QCborArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_cbor_array::Iterator>>::cast_into(it)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the first <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> of this array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborArray::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the first <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> of this array.</p>
    /// <p>If the array is empty, this function returns a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> containing an undefined value. For that reason, it is not possible with this function to tell apart the situation where the array is not large enough from the case where the array ends with an undefined value.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#operator-5b-5d">operator[]</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#at">at</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#last">last</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#removeAt">removeAt</a>(), and <a href="http://doc.qt.io/qt-5/qcborarray.html#takeAt">takeAt</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn first(&self) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborArray_first(self as *const crate::QCborArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the first <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> of this array. The array must not be empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValueRef QCborArray::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> of this array. The array must not be empty.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> has the exact same API as <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, with one important difference: if you assign new values to it, this map will be updated with that new value.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#operator-5b-5d">operator[]</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#at">at</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#last">last</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#removeAt">removeAt</a>(), and <a href="http://doc.qt.io/qt-5/qcborarray.html#takeAt">takeAt</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::CppBox<crate::QCborValueRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborArray_first1(self as *mut crate::QCborArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts all JSON items found in the <i>array</i> array to CBOR using QCborValue::fromJson(), and returns the CBOR array composed of those elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QCborArray QCborArray::fromJsonArray(const QJsonArray& array)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#fromJsonArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts all JSON items found in the <i>array</i> array to CBOR using QCborValue::fromJson(), and returns the CBOR array composed of those elements.</p>
    /// <p>This conversion is lossless, as the CBOR type system is a superset of JSON's. Moreover, the array returned by this function can be converted back to the original <i>array</i> by using <a href="http://doc.qt.io/qt-5/qcborarray.html#toJsonArray">toJsonArray</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#toJsonArray">toJsonArray</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#toVariantList">toVariantList</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromJsonValue">QCborValue::fromJsonValue</a>(), and <a href="http://doc.qt.io/qt-5/qcbormap.html#fromJsonObject">QCborMap::fromJsonObject</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_json_array(
        array: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonArray>>,
    ) -> ::cpp_utils::CppBox<crate::QCborArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_fromJsonArray(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonArray>>::cast_into(array)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> containing all the strings found in the <i>list</i> list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QCborArray QCborArray::fromStringList(const QStringList& list)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#fromStringList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> containing all the strings found in the <i>list</i> list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#fromVariantList">fromVariantList</a>() and <a href="http://doc.qt.io/qt-5/qcborarray.html#fromJsonArray">fromJsonArray</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_string_list(
        list: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringList>>,
    ) -> ::cpp_utils::CppBox<crate::QCborArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_fromStringList(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringList>>::cast_into(list)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts all the items in the <i>list</i> to CBOR using <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromVariant">QCborValue::fromVariant</a>() and returns the array composed of those elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QCborArray QCborArray::fromVariantList(const QList<QVariant>& list)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#fromVariantList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts all the items in the <i>list</i> to CBOR using <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromVariant">QCborValue::fromVariant</a>() and returns the array composed of those elements.</p>
    /// <p>Conversion from <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> is not completely lossless. Please see the documentation in <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromVariant">QCborValue::fromVariant</a>() for more information.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#toVariantList">toVariantList</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#fromStringList">fromStringList</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#fromJsonArray">fromJsonArray</a>(), and <a href="http://doc.qt.io/qt-5/qcbormap.html#fromVariantMap">QCborMap::fromVariantMap</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_variant_list(
        list: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QCborArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_fromVariantList(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQVariant>>::cast_into(list)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> element at position <i>i</i> in the array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborArray::operator[](long long i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> element at position <i>i</i> in the array.</p>
    /// <p>If the array is smaller than <i>i</i> elements, this function returns a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> containing an undefined value. For that reason, it is not possible with this function to tell apart the situation where the array is not large enough from the case where the array contains an undefined value at position <i>i</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#at">at</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#first">first</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#last">last</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#removeAt">removeAt</a>(), and <a href="http://doc.qt.io/qt-5/qcborarray.html#takeAt">takeAt</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn index(
        &self,
        i: ::std::os::raw::c_longlong,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_operator__(
            self as *const crate::QCborArray,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> element at position <i>i</i> in the array. Indices beyond the end of the array will grow the array, filling with undefined entries, until it has an entry at the specified index.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValueRef QCborArray::operator[](long long i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> element at position <i>i</i> in the array. Indices beyond the end of the array will grow the array, filling with undefined entries, until it has an entry at the specified index.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> has the exact same API as <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, with one important difference: if you assign new values to it, this map will be updated with that new value.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#at">at</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#first">first</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#last">last</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#removeAt">removeAt</a>(), and <a href="http://doc.qt.io/qt-5/qcborarray.html#takeAt">takeAt</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_longlong,
    ) -> ::cpp_utils::CppBox<crate::QCborValueRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborArray_operator__1(self as *mut crate::QCborArray, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```void QCborArray::insert(long long i, const QCborValue& value)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn insert_longlong_q_cbor_value(
        &mut self,
        i: ::std::os::raw::c_longlong,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborArray_insert(
            self as *mut crate::QCborArray,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```QCborArray::Iterator QCborArray::insert(QCborArray::Iterator before, const QCborValue& value)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn insert_iterator_q_cbor_value(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_cbor_array::Iterator>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::cpp_utils::CppBox<crate::q_cbor_array::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_insert2(
            self as *mut crate::QCborArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_cbor_array::Iterator>>::cast_into(
                before,
            )
            .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QCborArray::Iterator QCborArray::insert(QCborArray::ConstIterator before, const QCborValue& value)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn insert_const_iterator_q_cbor_value(
        &mut self,
        before: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::cpp_utils::CppBox<crate::q_cbor_array::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_insert3(self as *mut crate::QCborArray, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_cbor_array::ConstIterator>>::cast_into(before).as_raw_ptr(), ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(value).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> is empty (that is if <a href="http://doc.qt.io/qt-5/qcborarray.html#size">size</a>() is 0).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborArray::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if this <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> is empty (that is if <a href="http://doc.qt.io/qt-5/qcborarray.html#size">size</a>() is 0).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qcborarray.html#clear">clear</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborArray_isEmpty(self as *const crate::QCborArray)
    }

    /// <p>Returns the last <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> of this array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborArray::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the last <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> of this array.</p>
    /// <p>If the array is empty, this function returns a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> containing an undefined value. For that reason, it is not possible with this function to tell apart the situation where the array is not large enough from the case where the array ends with an undefined value.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#operator-5b-5d">operator[]</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#at">at</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#first">first</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#removeAt">removeAt</a>(), and <a href="http://doc.qt.io/qt-5/qcborarray.html#takeAt">takeAt</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn last(&self) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborArray_last(self as *const crate::QCborArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the last <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> of this array. The array must not be empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValueRef QCborArray::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> of this array. The array must not be empty.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> has the exact same API as <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, with one important difference: if you assign new values to it, this map will be updated with that new value.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#operator-5b-5d">operator[]</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#at">at</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#first">first</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#removeAt">removeAt</a>(), and <a href="http://doc.qt.io/qt-5/qcborarray.html#takeAt">takeAt</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::CppBox<crate::QCborValueRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborArray_last1(self as *mut crate::QCborArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs an empty <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborArray::QCborArray()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#QCborArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QCborArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_QCborArray();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Copies the contents of <i>other</i> into this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborArray::QCborArray(const QCborArray& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#QCborArray-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Copies the contents of <i>other</i> into this object.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborArray>>,
    ) -> ::cpp_utils::CppBox<crate::QCborArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_QCborArray1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborArray>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Synonym for <a href="http://doc.qt.io/qt-5/qcborarray.html#removeLast">removeLast</a>(). This function is provided for compatibility with generic code that uses the Standard Library API.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborArray::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Synonym for <a href="http://doc.qt.io/qt-5/qcborarray.html#removeLast">removeLast</a>(). This function is provided for compatibility with generic code that uses the Standard Library API.</p>
    /// <p>Removes the last element of this array. The array must not be empty before the removal</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#removeLast">removeLast</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#pop_front">pop_front</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#push_back">push_back</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#append">append</a>(), and <a href="http://doc.qt.io/qt-5/qcborarray.html#insert">insert</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCborArray_pop_back(self as *mut crate::QCborArray)
    }

    /// <p>Synonym for <a href="http://doc.qt.io/qt-5/qcborarray.html#removeFirst">removeFirst</a>(). This function is provided for compatibility with generic code that uses the Standard Library API.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborArray::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Synonym for <a href="http://doc.qt.io/qt-5/qcborarray.html#removeFirst">removeFirst</a>(). This function is provided for compatibility with generic code that uses the Standard Library API.</p>
    /// <p>Removes the first element of this array. The array must not be empty before the removal</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#pop_back">pop_back</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#push_front">push_front</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qcborarray.html#insert">insert</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCborArray_pop_front(self as *mut crate::QCborArray)
    }

    /// Calls C++ function: <span style='color: green;'>```void QCborArray::prepend(const QCborValue& value)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn prepend(
        &mut self,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborArray_prepend(
            self as *mut crate::QCborArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>Synonym for <a href="http://doc.qt.io/qt-5/qcborarray.html#append">append</a>(). This function is provided for compatibility with generic code that uses the Standard Library API.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborArray::push_back(const QCborValue& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Synonym for <a href="http://doc.qt.io/qt-5/qcborarray.html#append">append</a>(). This function is provided for compatibility with generic code that uses the Standard Library API.</p>
    /// <p>Appends the element <i>t</i> to this array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#push_front">push_front</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#pop_back">pop_back</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qcborarray.html#insert">insert</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborArray_push_back(
            self as *mut crate::QCborArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Synonym for <a href="http://doc.qt.io/qt-5/qcborarray.html#prepend">prepend</a>(). This function is provided for compatibility with generic code that uses the Standard Library API.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborArray::push_front(const QCborValue& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Synonym for <a href="http://doc.qt.io/qt-5/qcborarray.html#prepend">prepend</a>(). This function is provided for compatibility with generic code that uses the Standard Library API.</p>
    /// <p>Prepends the element <i>t</i> to this array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#push_back">push_back</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#pop_front">pop_front</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#append">append</a>(), and <a href="http://doc.qt.io/qt-5/qcborarray.html#insert">insert</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborArray_push_front(
            self as *mut crate::QCborArray,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes the item at position <i>i</i> from the array. The array must have more than <i>i</i> elements before the removal.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborArray::removeAt(long long i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at position <i>i</i> from the array. The array must have more than <i>i</i> elements before the removal.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#removeLast">removeLast</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#at">at</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-5b-5d">operator[]</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qcborarray.html#append">append</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_longlong) {
        crate::__ffi::ctr_qt_core_ffi_QCborArray_removeAt(self as *mut crate::QCborArray, i)
    }

    /// <p>Removes the first item in the array, making the second element become the first. The array must not be empty before this call.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborArray::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the array, making the second element become the first. The array must not be empty before this call.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#removeLast">removeLast</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#at">at</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-5b-5d">operator[]</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qcborarray.html#append">append</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCborArray_removeFirst(self as *mut crate::QCborArray)
    }

    /// <p>Removes the last item in the array. The array must not be empty before this call.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborArray::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the array. The array must not be empty before this call.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#at">at</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-5b-5d">operator[]</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qcborarray.html#append">append</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCborArray_removeLast(self as *mut crate::QCborArray)
    }

    /// <p>Returns the size of this array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```long long QCborArray::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the size of this array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#isEmpty">isEmpty</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn size(&self) -> ::std::os::raw::c_longlong {
        crate::__ffi::ctr_qt_core_ffi_QCborArray_size(self as *const crate::QCborArray)
    }

    /// <p>Swaps the contents of this object and <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborArray::swap(QCborArray& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps the contents of this object and <i>other</i>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QCborArray>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborArray_swap(
            self as *mut crate::QCborArray,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QCborArray>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Removes the item at position <i>i</i> from the array and returns it. The array must have more than <i>i</i> elements before the removal.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborArray::takeAt(long long i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at position <i>i</i> from the array and returns it. The array must have more than <i>i</i> elements before the removal.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#removeLast">removeLast</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#at">at</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-5b-5d">operator[]</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qcborarray.html#append">append</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_longlong,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborArray_takeAt(self as *mut crate::QCborArray, i);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the first item in the array and returns it, making the second element become the first. The array must not be empty before this call.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborArray::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the array and returns it, making the second element become the first. The array must not be empty before this call.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#removeLast">removeLast</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#at">at</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-5b-5d">operator[]</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qcborarray.html#append">append</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn take_first(&mut self) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborArray_takeFirst(self as *mut crate::QCborArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the last item in the array and returns it. The array must not be empty before this call.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborArray::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the array and returns it. The array must not be empty before this call.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#removeLast">removeLast</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#at">at</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-5b-5d">operator[]</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qcborarray.html#append">append</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn take_last(&mut self) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborArray_takeLast(self as *mut crate::QCborArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Explicitly construcuts a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object that represents this array. This function is usually not necessary since <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> has a constructor for <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>, so the conversion is implicit.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborArray::toCborValue() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#toCborValue">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Explicitly construcuts a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object that represents this array. This function is usually not necessary since <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> has a constructor for <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a>, so the conversion is implicit.</p>
    /// <p>Converting <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> to <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> allows it to be used in any context where QCborValues can be used, including as items in QCborArrays and as keys and mapped types in <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>. Converting an array to <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> allows access to <a href="http://doc.qt.io/qt-5/qcborvalue.html#toCbor">QCborValue::toCbor</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#QCborValue-11">QCborValue::QCborValue</a>(const QCborArray &amp;).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_cbor_value(&self) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborArray_toCborValue(self as *const crate::QCborArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Recursively converts every <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> element in this array to JSON using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toJsonValue">QCborValue::toJsonValue</a>() and returns the corresponding <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> composed of those elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonArray QCborArray::toJsonArray() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#toJsonArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Recursively converts every <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> element in this array to JSON using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toJsonValue">QCborValue::toJsonValue</a>() and returns the corresponding <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> composed of those elements.</p>
    /// <p>Please note that CBOR contains a richer and wider type set than JSON, so some information may be lost in this conversion. For more details on what conversions are applied, see <a href="http://doc.qt.io/qt-5/qcborvalue.html#toJsonValue">QCborValue::toJsonValue</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#fromJsonArray">fromJsonArray</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#toJsonValue">QCborValue::toJsonValue</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#toJsonObject">QCborMap::toJsonObject</a>(), and <a href="http://doc.qt.io/qt-5/qcborarray.html#toVariantList">toVariantList</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_json_array(&self) -> ::cpp_utils::CppBox<crate::QJsonArray> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborArray_toJsonArray(self as *const crate::QCborArray);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Recursively converts each <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> in this array using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toVariant">QCborValue::toVariant</a>() and returns the <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a> composed of the converted items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant> QCborArray::toVariantList() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#toVariantList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Recursively converts each <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> in this array using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toVariant">QCborValue::toVariant</a>() and returns the <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a> composed of the converted items.</p>
    /// <p>Conversion to <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> is not completely lossless. Please see the documentation in <a href="http://doc.qt.io/qt-5/qcborvalue.html#toVariant">QCborValue::toVariant</a>() for more information.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#fromVariantList">fromVariantList</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#fromStringList">fromStringList</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#toJsonArray">toJsonArray</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#toVariant">QCborValue::toVariant</a>(), and <a href="http://doc.qt.io/qt-5/qcbormap.html#toVariantMap">QCborMap::toVariantMap</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_variant_list(&self) -> ::cpp_utils::CppBox<crate::QListOfQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborArray_toVariantList(
            self as *const crate::QCborArray,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_cbor_map {
    //! C++ type: <span style='color: green;'>```QCborMap```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qcbormap-iterator.html">QCborMap::Iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QCborMap::Iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qcbormap-iterator.html">QCborMap::Iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qcbormap-iterator.html">QCborMap::Iterator</a> allows you to iterate over a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> and to modify the value (but not the key) stored under a particular key. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>, you should use <a href="http://doc.qt.io/qt-5/qcbormap-constiterator.html">QCborMap::ConstIterator</a>. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qcbormap-constiterator.html">QCborMap::ConstIterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> through the iterator. Const iterators are slightly faster, and improve code readability.</p>
    /// <p>You must initialize the iterator using a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> function like <a href="http://doc.qt.io/qt-5/qcbormap.html#begin">QCborMap::begin</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#end">QCborMap::end</a>(), or <a href="http://doc.qt.io/qt-5/qcbormap.html#find">QCborMap::find</a>() before you can start iterating..</p>
    /// <p>Multiple iterators can be used on the same object. Existing iterators will however become dangling once the object gets modified.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// Calls C++ function: <span style='color: green;'>```QCborMap::Iterator& QCborMap::Iterator::operator=(const QCborMap::Iterator& other)```</span>.
        ///
        ///
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_cbor_map::Iterator>>,
        ) -> ::cpp_utils::MutRef<crate::q_cbor_map::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_Iterator_operator_(
                self as *mut crate::q_cbor_map::Iterator,
                ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_cbor_map::Iterator>>::cast_into(
                    other,
                )
                .as_raw_ptr(),
            );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborMap::Iterator QCborMap::Iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator, <code>it--</code>, makes the preceding item current and returns an iterator to the previously current item.</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_cbor_map::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_Iterator_operator__12(
                self as *mut crate::q_cbor_map::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborMap::Iterator QCborMap::Iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator, <code>it++</code>, advances the iterator to the next item in the array and returns an iterator to the previously current item.</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_cbor_map::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_Iterator_operator__10(
                self as *mut crate::q_cbor_map::Iterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the current item's key.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborValue QCborMap::Iterator::key() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcbormap-iterator.html#key">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current item's key.</p>
        /// <p>There is no direct way of changing an item's key through an iterator, although it can be done by calling <a href="http://doc.qt.io/qt-5/qcbormap.html#erase">QCborMap::erase</a>() followed by <a href="http://doc.qt.io/qt-5/qcbormap.html#insert">QCborMap::insert</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap-iterator.html#value">value</a>().</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn key(&self) -> ::cpp_utils::CppBox<crate::QCborValue> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_Iterator_key(
                self as *const crate::q_cbor_map::Iterator,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborMap::Iterator::Iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcbormap-iterator.html#Iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like <a href="http://doc.qt.io/qt-5/qcbormap-iterator.html#key">key</a>(), <a href="http://doc.qt.io/qt-5/qcbormap-iterator.html#value">value</a>(), and operator++() must not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#begin">QCborMap::begin</a>() and <a href="http://doc.qt.io/qt-5/qcbormap.html#end">QCborMap::end</a>().</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_cbor_map::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_Iterator_Iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs an iterator as a copy of <i>other</i>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborMap::Iterator::Iterator(const QCborMap::Iterator& arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcbormap-iterator.html#Iterator-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an iterator as a copy of <i>other</i>.</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn new_copy(
            arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_cbor_map::Iterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_cbor_map::Iterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_Iterator_Iterator1(
                ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_cbor_map::Iterator>>::cast_into(
                    arg1,
                )
                .as_raw_ptr(),
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to a modifiable reference to the current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborValueRef* QCborMap::Iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to a modifiable reference to the current item.</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::MutPtr<crate::QCborValueRef> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_Iterator_operator__(
                self as *const crate::q_cbor_map::Iterator,
            );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }

        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborValueRef QCborMap::Iterator::value() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcbormap-iterator.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>You can change the value for a key by using value() on the left side of an assignment.</p>
        /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> and <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>. If you assign to it, the assignment will apply to the element in the <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> or <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> from which you got the reference.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap-iterator.html#key">key</a>() and <a href="http://doc.qt.io/qt-5/qcbormap-iterator.html#operator-2a">operator*</a>().</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn value(&self) -> ::cpp_utils::CppBox<crate::QCborValueRef> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_Iterator_value(
                self as *const crate::q_cbor_map::Iterator,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qcbormap-constiterator.html">QCborMap::ConstIterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QCborMap::ConstIterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap-constiterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qcbormap-constiterator.html">QCborMap::ConstIterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qcbormap-constiterator.html">QCborMap::ConstIterator</a> allows you to iterate over a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>. If you want to modify the <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> as you iterate over it, you must use <a href="http://doc.qt.io/qt-5/qcbormap-iterator.html">QCborMap::Iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qcbormap-constiterator.html">QCborMap::ConstIterator</a>, even on a non-const <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>, when you don't need to change the <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> through the iterator. Const iterators are slightly faster and improve code readability.</p>
    /// <p>You must initialize the iterator using a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> function like <a href="http://doc.qt.io/qt-5/qcbormap.html#begin">QCborMap::begin</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#end">QCborMap::end</a>(), or <a href="http://doc.qt.io/qt-5/qcbormap.html#find">QCborMap::find</a>() before you can start iterating..</p>
    /// <p>Multiple iterators can be used on the same object. Existing iterators will however become dangling if the object gets modified.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>Makes this iterator a copy of <i>other</i> and returns a reference to this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborMap::ConstIterator& QCborMap::ConstIterator::operator=(const QCborMap::ConstIterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes this iterator a copy of <i>other</i> and returns a reference to this iterator.</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>>,
        ) -> ::cpp_utils::MutRef<crate::q_cbor_map::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_ConstIterator_operator_(self as *mut crate::q_cbor_map::ConstIterator, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborMap::ConstIterator QCborMap::ConstIterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator, <code>it--</code>, makes the preceding item current and returns an iterator to the previously current item.</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_cbor_map::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_ConstIterator_operator__12(
                self as *mut crate::q_cbor_map::ConstIterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborMap::ConstIterator QCborMap::ConstIterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator, <code>it++</code>, advances the iterator to the next item in the array and returns an iterator to the previously current item.</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_cbor_map::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_ConstIterator_operator__10(
                self as *mut crate::q_cbor_map::ConstIterator,
                arg1,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the current item's key.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborValue QCborMap::ConstIterator::key() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcbormap-constiterator.html#key">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current item's key.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap-constiterator.html#value">value</a>().</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn key(&self) -> ::cpp_utils::CppBox<crate::QCborValue> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_ConstIterator_key(
                self as *const crate::q_cbor_map::ConstIterator,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborMap::ConstIterator::ConstIterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcbormap-constiterator.html#ConstIterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like <a href="http://doc.qt.io/qt-5/qcbormap-constiterator.html#key">key</a>(), <a href="http://doc.qt.io/qt-5/qcbormap-constiterator.html#value">value</a>(), and operator++() must not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#constBegin">QCborMap::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qcbormap.html#constEnd">QCborMap::constEnd</a>().</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn new() -> ::cpp_utils::CppBox<crate::q_cbor_map::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_ConstIterator_ConstIterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Constructs an iterator as a copy of <i>other</i>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborMap::ConstIterator::ConstIterator(const QCborMap::ConstIterator& arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcbormap-constiterator.html#ConstIterator-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an iterator as a copy of <i>other</i>.</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn new_copy(
            arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>>,
        ) -> ::cpp_utils::CppBox<crate::q_cbor_map::ConstIterator> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_ConstIterator_ConstIterator1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>>::cast_into(arg1).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QCborValueRef* QCborMap::ConstIterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item.</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::Ptr<crate::QCborValueRef> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_ConstIterator_operator__(
                self as *const crate::q_cbor_map::ConstIterator,
            );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }

        /// <p>Returns the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborValueRef QCborMap::ConstIterator::value() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcbormap-constiterator.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current item's value.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap-constiterator.html#key">key</a>() and <a href="http://doc.qt.io/qt-5/qcbormap-constiterator.html#operator-2a">operator*</a>().</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn value(&self) -> ::cpp_utils::CppBox<crate::QCborValueRef> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_ConstIterator_value(
                self as *const crate::q_cbor_map::ConstIterator,
            );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::ops::Indirection for &crate::q_cbor_map::Iterator {
        type Output = ::cpp_utils::CppBox<crate::QPairOfQCborValueRefQCborValueRef>;
        /// <p>Returns a modifiable reference to the current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QPair<QCborValueRef, QCborValueRef> QCborMap::Iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item.</p>
        /// <p>You can change the value of an item by using operator*() on the left side of an assignment.</p>
        /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> and <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>. If you assign to it, the assignment will apply to the element in the <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> or <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> from which you got the reference.</p></div>
        fn indirection(self) -> ::cpp_utils::CppBox<crate::QPairOfQCborValueRefQCborValueRef> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_Iterator_operator_1(
                    self as *const crate::q_cbor_map::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_cbor_map::Iterator>>
        for crate::q_cbor_map::Iterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborMap::Iterator::operator==(const QCborMap::Iterator& o) const```</span>.
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_cbor_map::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_Iterator_operator__1(
                    self as *const crate::q_cbor_map::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_cbor_map::Iterator>>
        for crate::q_cbor_map::Iterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborMap::Iterator::operator<(const QCborMap::Iterator& other) const```</span>.
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_cbor_map::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_Iterator_operator_2(
                    self as *const crate::q_cbor_map::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_cbor_map::Iterator>>
        for crate::q_cbor_map::Iterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborMap::Iterator::operator<=(const QCborMap::Iterator& other) const```</span>.
        fn le(&self, other: &::cpp_utils::Ref<crate::q_cbor_map::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_Iterator_operator__3(
                    self as *const crate::q_cbor_map::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_cbor_map::Iterator>>
        for crate::q_cbor_map::Iterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborMap::Iterator::operator>(const QCborMap::Iterator& other) const```</span>.
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_cbor_map::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_Iterator_operator_3(
                    self as *const crate::q_cbor_map::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_cbor_map::Iterator>>
        for crate::q_cbor_map::Iterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborMap::Iterator::operator>=(const QCborMap::Iterator& other) const```</span>.
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_cbor_map::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_Iterator_operator__4(
                    self as *const crate::q_cbor_map::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>>
        for crate::q_cbor_map::Iterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborMap::Iterator::operator==(const QCborMap::ConstIterator& o) const```</span>.
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_Iterator_operator__5(
                    self as *const crate::q_cbor_map::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>>
        for crate::q_cbor_map::Iterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborMap::Iterator::operator<(const QCborMap::ConstIterator& other) const```</span>.
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_Iterator_operator_4(
                    self as *const crate::q_cbor_map::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>>
        for crate::q_cbor_map::Iterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborMap::Iterator::operator<=(const QCborMap::ConstIterator& other) const```</span>.
        fn le(&self, other: &::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_Iterator_operator__7(
                    self as *const crate::q_cbor_map::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>>
        for crate::q_cbor_map::Iterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborMap::Iterator::operator>(const QCborMap::ConstIterator& other) const```</span>.
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_Iterator_operator_5(
                    self as *const crate::q_cbor_map::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>>
        for crate::q_cbor_map::Iterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborMap::Iterator::operator>=(const QCborMap::ConstIterator& other) const```</span>.
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_Iterator_operator__8(
                    self as *const crate::q_cbor_map::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::ops::Increment for &mut crate::q_cbor_map::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_cbor_map::Iterator>;
        /// <p>The prefix ++ operator, <code>++it</code>, advances the iterator to the next item in the array and returns this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborMap::Iterator& QCborMap::Iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator, <code>++it</code>, advances the iterator to the next item in the array and returns this iterator.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qcborarray.html#end">QCborArray::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_cbor_map::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_Iterator_operator__9(
                    self as *mut crate::q_cbor_map::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::ops::Decrement for &mut crate::q_cbor_map::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_cbor_map::Iterator>;
        /// <p>The prefix -- operator, <code>--it</code>, makes the preceding item current and returns this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborMap::Iterator& QCborMap::Iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator, <code>--it</code>, makes the preceding item current and returns this iterator.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qcborarray.html#begin">QCborArray::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_cbor_map::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_Iterator_operator__11(
                    self as *mut crate::q_cbor_map::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::ops::AddAssign<::std::os::raw::c_longlong> for crate::q_cbor_map::Iterator {
        /// <p>Advances the iterator by <i>j</i> positions. If <i>j</i> is negative, the iterator goes backward. Returns a reference to this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborMap::Iterator& QCborMap::Iterator::operator+=(long long j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> positions. If <i>j</i> is negative, the iterator goes backward. Returns a reference to this iterator.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_Iterator_operator__13(
                    self as *mut crate::q_cbor_map::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::ops::SubAssign<::std::os::raw::c_longlong> for crate::q_cbor_map::Iterator {
        /// <p>Makes the iterator go back by <i>j</i> positions. If <i>j</i> is negative, the iterator goes forward. Returns a reference to this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborMap::Iterator& QCborMap::Iterator::operator-=(long long j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> positions. If <i>j</i> is negative, the iterator goes forward. Returns a reference to this iterator.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_Iterator_operator__14(
                    self as *mut crate::q_cbor_map::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_cbor_map::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_cbor_map::Iterator>;
        /// <p>Returns an iterator to the item at position <i>j</i> steps forward from this iterator. If <i>j</i> is negative, the iterator goes backward.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborMap::Iterator QCborMap::Iterator::operator+(long long j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at position <i>j</i> steps forward from this iterator. If <i>j</i> is negative, the iterator goes backward.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-">operator-</a>() and <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-2b-eq">operator+=</a>().</p></div>
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_cbor_map::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_Iterator_operator_6(
                    self as *const crate::q_cbor_map::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_cbor_map::Iterator {
        type Output = ::cpp_utils::CppBox<crate::q_cbor_map::Iterator>;
        /// <p>Returns an iterator to the item at position <i>j</i> steps backward from this iterator. If <i>j</i> is negative, the iterator goes forward.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborMap::Iterator QCborMap::Iterator::operator-(long long j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at position <i>j</i> steps backward from this iterator. If <i>j</i> is negative, the iterator goes forward.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator--eq">operator-=</a>().</p></div>
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_cbor_map::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_Iterator_operator_7(
                    self as *const crate::q_cbor_map::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_cbor_map::Iterator>>
        for &crate::q_cbor_map::Iterator
    {
        type Output = ::std::os::raw::c_longlong;
        /// <p>Returns the offset of this iterator relative to <i>other</i>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```long long QCborMap::Iterator::operator-(QCborMap::Iterator j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-iterator.html#operator--1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the offset of this iterator relative to <i>other</i>.</p></div>
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_cbor_map::Iterator>,
        ) -> ::std::os::raw::c_longlong {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_Iterator_operator_8(
                    self as *const crate::q_cbor_map::Iterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::ops::Indirection for &crate::q_cbor_map::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::QPairOfQCborValueRefQCborValueRef>;
        /// <p>Returns the current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QPair<QCborValueRef, QCborValueRef> QCborMap::ConstIterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current item.</p></div>
        fn indirection(self) -> ::cpp_utils::CppBox<crate::QPairOfQCborValueRefQCborValueRef> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_ConstIterator_operator_1(
                    self as *const crate::q_cbor_map::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_cbor_map::Iterator>>
        for crate::q_cbor_map::ConstIterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborMap::ConstIterator::operator==(const QCborMap::Iterator& o) const```</span>.
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_cbor_map::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_ConstIterator_operator__1(
                    self as *const crate::q_cbor_map::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_cbor_map::Iterator>>
        for crate::q_cbor_map::ConstIterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborMap::ConstIterator::operator<(const QCborMap::Iterator& other) const```</span>.
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_cbor_map::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_ConstIterator_operator_2(
                    self as *const crate::q_cbor_map::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_cbor_map::Iterator>>
        for crate::q_cbor_map::ConstIterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborMap::ConstIterator::operator<=(const QCborMap::Iterator& other) const```</span>.
        fn le(&self, other: &::cpp_utils::Ref<crate::q_cbor_map::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_ConstIterator_operator__3(
                    self as *const crate::q_cbor_map::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_cbor_map::Iterator>>
        for crate::q_cbor_map::ConstIterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborMap::ConstIterator::operator>(const QCborMap::Iterator& other) const```</span>.
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_cbor_map::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_ConstIterator_operator_3(
                    self as *const crate::q_cbor_map::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_cbor_map::Iterator>>
        for crate::q_cbor_map::ConstIterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborMap::ConstIterator::operator>=(const QCborMap::Iterator& other) const```</span>.
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_cbor_map::Iterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_ConstIterator_operator__4(
                    self as *const crate::q_cbor_map::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>>
        for crate::q_cbor_map::ConstIterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborMap::ConstIterator::operator==(const QCborMap::ConstIterator& o) const```</span>.
        fn eq(&self, o: &::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_ConstIterator_operator__5(
                    self as *const crate::q_cbor_map::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>>
        for crate::q_cbor_map::ConstIterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborMap::ConstIterator::operator<(const QCborMap::ConstIterator& other) const```</span>.
        fn lt(&self, other: &::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_ConstIterator_operator_4(
                    self as *const crate::q_cbor_map::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>>
        for crate::q_cbor_map::ConstIterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborMap::ConstIterator::operator<=(const QCborMap::ConstIterator& other) const```</span>.
        fn le(&self, other: &::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_ConstIterator_operator__7(
                    self as *const crate::q_cbor_map::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>>
        for crate::q_cbor_map::ConstIterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborMap::ConstIterator::operator>(const QCborMap::ConstIterator& other) const```</span>.
        fn gt(&self, other: &::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_ConstIterator_operator_5(
                    self as *const crate::q_cbor_map::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>>
        for crate::q_cbor_map::ConstIterator
    {
        /// Calls C++ function: <span style='color: green;'>```bool QCborMap::ConstIterator::operator>=(const QCborMap::ConstIterator& other) const```</span>.
        fn ge(&self, other: &::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_ConstIterator_operator__8(
                    self as *const crate::q_cbor_map::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::ops::Increment for &mut crate::q_cbor_map::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_cbor_map::ConstIterator>;
        /// <p>The prefix ++ operator, <code>++it</code>, advances the iterator to the next item in the array and returns this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborMap::ConstIterator& QCborMap::ConstIterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator, <code>++it</code>, advances the iterator to the next item in the array and returns this iterator.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qcborarray.html#end">QCborArray::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator--">operator--</a>().</p></div>
        fn inc(self) -> ::cpp_utils::MutRef<crate::q_cbor_map::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_ConstIterator_operator__9(
                    self as *mut crate::q_cbor_map::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::ops::Decrement for &mut crate::q_cbor_map::ConstIterator {
        type Output = ::cpp_utils::MutRef<crate::q_cbor_map::ConstIterator>;
        /// <p>The prefix -- operator, <code>--it</code>, makes the preceding item current and returns this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborMap::ConstIterator& QCborMap::ConstIterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator, <code>--it</code>, makes the preceding item current and returns this iterator.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qcborarray.html#begin">QCborArray::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(self) -> ::cpp_utils::MutRef<crate::q_cbor_map::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_ConstIterator_operator__11(
                    self as *mut crate::q_cbor_map::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::ops::AddAssign<::std::os::raw::c_longlong> for crate::q_cbor_map::ConstIterator {
        /// <p>Advances the iterator by <i>j</i> positions. If <i>j</i> is negative, the iterator goes backward. Returns a reference to this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborMap::ConstIterator& QCborMap::ConstIterator::operator+=(long long j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> positions. If <i>j</i> is negative, the iterator goes backward. Returns a reference to this iterator.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_ConstIterator_operator__13(
                    self as *mut crate::q_cbor_map::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::ops::SubAssign<::std::os::raw::c_longlong> for crate::q_cbor_map::ConstIterator {
        /// <p>Makes the iterator go back by <i>j</i> positions. If <i>j</i> is negative, the iterator goes forward. Returns a reference to this iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborMap::ConstIterator& QCborMap::ConstIterator::operator-=(long long j)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> positions. If <i>j</i> is negative, the iterator goes forward. Returns a reference to this iterator.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_ConstIterator_operator__14(
                    self as *mut crate::q_cbor_map::ConstIterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::ops::Add<::std::os::raw::c_longlong> for &crate::q_cbor_map::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_cbor_map::ConstIterator>;
        /// <p>Returns an iterator to the item at a position <i>j</i> steps forward from this iterator. If <i>j</i> is negative, the iterator goes backward.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborMap::ConstIterator QCborMap::ConstIterator::operator+(long long j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at a position <i>j</i> steps forward from this iterator. If <i>j</i> is negative, the iterator goes backward.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-">operator-</a>() and <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-2b-eq">operator+=</a>().</p></div>
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_cbor_map::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_ConstIterator_operator_6(
                    self as *const crate::q_cbor_map::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::ops::Sub<::std::os::raw::c_longlong> for &crate::q_cbor_map::ConstIterator {
        type Output = ::cpp_utils::CppBox<crate::q_cbor_map::ConstIterator>;
        /// <p>Returns an iterator to the item at a position <i>j</i> steps backward from this iterator. If <i>j</i> is negative, the iterator goes forward.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborMap::ConstIterator QCborMap::ConstIterator::operator-(long long j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an iterator to the item at a position <i>j</i> steps backward from this iterator. If <i>j</i> is negative, the iterator goes forward.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator--eq">operator-=</a>().</p></div>
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_cbor_map::ConstIterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_ConstIterator_operator_7(
                    self as *const crate::q_cbor_map::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>>
        for &crate::q_cbor_map::ConstIterator
    {
        type Output = ::std::os::raw::c_longlong;
        /// <p>Returns the offset of this iterator relative to <i>other</i>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```long long QCborMap::ConstIterator::operator-(QCborMap::ConstIterator j) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborarray-constiterator.html#operator--1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the offset of this iterator relative to <i>other</i>.</p></div>
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>,
        ) -> ::std::os::raw::c_longlong {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QCborMap_ConstIterator_operator_8(
                    self as *const crate::q_cbor_map::ConstIterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::CppDeletable for crate::q_cbor_map::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qcbormap-iterator.html">QCborMap::Iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QCborMap::Iterator::~Iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcbormap-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qcbormap-iterator.html">QCborMap::Iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qcbormap-iterator.html">QCborMap::Iterator</a> allows you to iterate over a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> and to modify the value (but not the key) stored under a particular key. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>, you should use <a href="http://doc.qt.io/qt-5/qcbormap-constiterator.html">QCborMap::ConstIterator</a>. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qcbormap-constiterator.html">QCborMap::ConstIterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> through the iterator. Const iterators are slightly faster, and improve code readability.</p>
        /// <p>You must initialize the iterator using a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> function like <a href="http://doc.qt.io/qt-5/qcbormap.html#begin">QCborMap::begin</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#end">QCborMap::end</a>(), or <a href="http://doc.qt.io/qt-5/qcbormap.html#find">QCborMap::find</a>() before you can start iterating..</p>
        /// <p>Multiple iterators can be used on the same object. Existing iterators will however become dangling once the object gets modified.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QCborMap_Iterator_dIterator(
                self as *mut crate::q_cbor_map::Iterator,
            )
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::CppDeletable for crate::q_cbor_map::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qcbormap-constiterator.html">QCborMap::ConstIterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QCborMap::ConstIterator::~ConstIterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcbormap-constiterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qcbormap-constiterator.html">QCborMap::ConstIterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qcbormap-constiterator.html">QCborMap::ConstIterator</a> allows you to iterate over a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>. If you want to modify the <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> as you iterate over it, you must use <a href="http://doc.qt.io/qt-5/qcbormap-iterator.html">QCborMap::Iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qcbormap-constiterator.html">QCborMap::ConstIterator</a>, even on a non-const <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>, when you don't need to change the <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> through the iterator. Const iterators are slightly faster and improve code readability.</p>
        /// <p>You must initialize the iterator using a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> function like <a href="http://doc.qt.io/qt-5/qcbormap.html#begin">QCborMap::begin</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#end">QCborMap::end</a>(), or <a href="http://doc.qt.io/qt-5/qcbormap.html#find">QCborMap::find</a>() before you can start iterating..</p>
        /// <p>Multiple iterators can be used on the same object. Existing iterators will however become dangling if the object gets modified.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QCborMap_ConstIterator_dConstIterator(
                self as *mut crate::q_cbor_map::ConstIterator,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> class is used to hold an associative container representable in CBOR.</p>
///
/// C++ class: <span style='color: green;'>```QCborMap```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qcbormap.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> class is used to hold an associative container representable in CBOR.</p>
/// <p>This class can be used to hold an associative container in CBOR, a map between a key and a value type. CBOR is the Concise Binary Object Representation, a very compact form of binary data encoding that is a superset of JSON. It was created by the IETF Constrained RESTful Environments (CoRE) WG, which has used it in many new RFCs. It is meant to be used alongside the <a href="https://tools.ietf.org/html/rfc7252">CoAP protocol</a>.</p>
/// <p>Unlike JSON and <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantMap-typedef">QVariantMap</a>, CBOR map keys can be of any type, not just strings. For that reason, <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> is effectively a map between <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> keys to <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> value elements.</p>
/// <p>However, for all member functions that take a key parameter, <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> provides overloads that will work efficiently with integers and strings. In fact, the use of integer keys is encouraged, since they occupy fewer bytes to transmit and are simpler to encode and decode. Newer protocols designed by the IETF CoRE WG to work specifically with CBOR are known to use them.</p>
/// <p><a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> is not sorted, because of that, searching for keys has linear complexity (O(n)). <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> actually keeps the elements in the order that they were inserted, which means that it is possible to make sorted QCborMaps by carefully inserting elements in sorted order. CBOR does not require sorting, but recommends it.</p>
/// <p><a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> can also be converted to and from <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantMap-typedef">QVariantMap</a> and <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. However, when performing the conversion, any non-string keys will be stringified using a one-way method that the conversion back to <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> will not undo.</p></div>
#[repr(C)]
pub struct QCborMap {
    _unused: u8,
}
impl QCborMap {
    /// <p>Returns a map iterator pointing to the first key-value pair of this map. If this map is empty, the returned iterator will be the same as <a href="http://doc.qt.io/qt-5/qcbormap.html#constEnd">constEnd</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborMap::ConstIterator QCborMap::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a map iterator pointing to the first key-value pair of this map. If this map is empty, the returned iterator will be the same as <a href="http://doc.qt.io/qt-5/qcbormap.html#constEnd">constEnd</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qcbormap.html#constEnd">constEnd</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn cbegin(&self) -> ::cpp_utils::CppBox<crate::q_cbor_map::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborMap_cbegin(self as *const crate::QCborMap);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a map iterator representing an element just past the last element in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborMap::ConstIterator QCborMap::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a map iterator representing an element just past the last element in the map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#begin">begin</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#constBegin">constBegin</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#find">find</a>(), and <a href="http://doc.qt.io/qt-5/qcbormap.html#constFind">constFind</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn cend(&self) -> ::cpp_utils::CppBox<crate::q_cbor_map::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborMap_cend(self as *const crate::QCborMap);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Empties this map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborMap::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Empties this map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#isEmpty">isEmpty</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCborMap_clear(self as *mut crate::QCborMap)
    }

    /// <p>Compares this map and <i>other</i>, comparing each element in sequence, and returns an integer that indicates whether this map should be sorted prior to (if the result is negative) or after <i>other</i> (if the result is positive). If this function returns 0, the two maps are equal and contain the same elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QCborMap::compare(const QCborMap& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#compare">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares this map and <i>other</i>, comparing each element in sequence, and returns an integer that indicates whether this map should be sorted prior to (if the result is negative) or after <i>other</i> (if the result is positive). If this function returns 0, the two maps are equal and contain the same elements.</p>
    /// <p>Note that CBOR maps are unordered, which means that two maps containing the very same pairs but in different order will still compare differently. To avoid this, it is recommended to insert elements into the map in a predictable order, such as by ascending key value. In fact, maps with keys in sorted order are required for Canonical CBOR representation.</p>
    /// <p>For more information on CBOR sorting order, see <a href="http://doc.qt.io/qt-5/qcborvalue.html#compare">QCborValue::compare</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#compare">QCborValue::compare</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#compare">QCborArray::compare</a>(), and <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-eq-eq">operator==</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn compare(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborMap>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QCborMap_compare(
            self as *const crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborMap>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a map iterator pointing to the first key-value pair of this map. If this map is empty, the returned iterator will be the same as <a href="http://doc.qt.io/qt-5/qcbormap.html#constEnd">constEnd</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborMap::ConstIterator QCborMap::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a map iterator pointing to the first key-value pair of this map. If this map is empty, the returned iterator will be the same as <a href="http://doc.qt.io/qt-5/qcbormap.html#constEnd">constEnd</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qcbormap.html#constEnd">constEnd</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn const_begin(&self) -> ::cpp_utils::CppBox<crate::q_cbor_map::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborMap_constBegin(self as *const crate::QCborMap);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a map iterator representing an element just past the last element in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborMap::ConstIterator QCborMap::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a map iterator representing an element just past the last element in the map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#begin">begin</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#constBegin">constBegin</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#find">find</a>(), and <a href="http://doc.qt.io/qt-5/qcbormap.html#constFind">constFind</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn const_end(&self) -> ::cpp_utils::CppBox<crate::q_cbor_map::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborMap_constEnd(self as *const crate::QCborMap);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a map iterator to the key-value pair whose key is <i>key</i>, if the map contains such a pair. If it doesn't, this function returns <a href="http://doc.qt.io/qt-5/qcbormap.html#constEnd">constEnd</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborMap::ConstIterator QCborMap::constFind(qint64 key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#constFind">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a map iterator to the key-value pair whose key is <i>key</i>, if the map contains such a pair. If it doesn't, this function returns <a href="http://doc.qt.io/qt-5/qcbormap.html#constEnd">constEnd</a>().</p>
    /// <p>CBOR recommends using integer keys, since they occupy less space and are simpler to encode and decode.</p>
    /// <p>If the map contains more than one key equal to <i>key</i>, it is undefined which one this function will find. <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> does not allow inserting duplicate keys, but it is possible to create such a map by decoding a CBOR stream with them. They are usually not permitted and having duplicate keys is usually an indication of a problem in the sender.</p>
    /// <p>value(<a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>), value(const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;), value(const <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> &amp;)</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#value">value</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d">operator[]</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#find">find</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove">remove</a>(qint64), and <a href="http://doc.qt.io/qt-5/qcbormap.html#contains">contains</a>(qint64).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn const_find_i64(
        &self,
        key: i64,
    ) -> ::cpp_utils::CppBox<crate::q_cbor_map::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborMap_constFind(self as *const crate::QCborMap, key);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborMap::ConstIterator QCborMap::constFind(QLatin1String key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#constFind-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a map iterator to the key-value pair whose key is <i>key</i>, if the map contains such a pair. If it doesn't, this function returns <a href="http://doc.qt.io/qt-5/qcbormap.html#constEnd">constEnd</a>().</p>
    /// <p>If the map contains more than one key equal to <i>key</i>, it is undefined which one this function will find. <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> does not allow inserting duplicate keys, but it is possible to create such a map by decoding a CBOR stream with them. They are usually not permitted and having duplicate keys is usually an indication of a problem in the sender.</p>
    /// <p>value(qint64), value(const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;), value(const <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> &amp;)</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#value-1">value</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-1">operator[]</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#find-1">find</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-1">remove</a>(QLatin1String), and <a href="http://doc.qt.io/qt-5/qcbormap.html#contains-1">contains</a>(QLatin1String).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn const_find_q_latin1_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::q_cbor_map::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_constFind1(
            self as *const crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborMap::ConstIterator QCborMap::constFind(const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#constFind-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a map iterator to the key-value pair whose key is <i>key</i>, if the map contains such a pair. If it doesn't, this function returns <a href="http://doc.qt.io/qt-5/qcbormap.html#constEnd">constEnd</a>().</p>
    /// <p>If the map contains more than one key equal to <i>key</i>, it is undefined which one this function will find. <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> does not allow inserting duplicate keys, but it is possible to create such a map by decoding a CBOR stream with them. They are usually not permitted and having duplicate keys is usually an indication of a problem in the sender.</p>
    /// <p>value(qint64), value(<a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>), value(const <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> &amp;)</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#value-2">value</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-2">operator[]</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#find-2">find</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-2">remove</a>(const QString &amp;), and <a href="http://doc.qt.io/qt-5/qcbormap.html#contains-2">contains</a>(const QString &amp;).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn const_find_q_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::q_cbor_map::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_constFind2(
            self as *const crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborMap::ConstIterator QCborMap::constFind(const QCborValue& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#constFind-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a map iterator to the key-value pair whose key is <i>key</i>, if the map contains such a pair. If it doesn't, this function returns <a href="http://doc.qt.io/qt-5/qcbormap.html#constEnd">constEnd</a>().</p>
    /// <p>If the map contains more than one key equal to <i>key</i>, it is undefined which one this function will find. <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> does not allow inserting duplicate keys, but it is possible to create such a map by decoding a CBOR stream with them. They are usually not permitted and having duplicate keys is usually an indication of a problem in the sender.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#value-3">value</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-3">operator[]</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#find-3">find</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-3">remove</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#contains-3">contains</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#value">value</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#value-1">value</a>(QLatin1String), and <a href="http://doc.qt.io/qt-5/qcbormap.html#value-2">value</a>(const QString &amp;).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn const_find_q_cbor_value(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::cpp_utils::CppBox<crate::q_cbor_map::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_constFind3(
            self as *const crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns true if this map contains a key-value pair identified by key <i>key</i>. CBOR recommends using integer keys, since they occupy less space and are simpler to encode and decode.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborMap::contains(qint64 key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if this map contains a key-value pair identified by key <i>key</i>. CBOR recommends using integer keys, since they occupy less space and are simpler to encode and decode.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#value">value</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d">operator[]</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#find">find</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove">remove</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#contains-1">contains</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-2">remove</a>(const QString &amp;), and <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-3">remove</a>(const QCborValue &amp;).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn contains_i64(&self, key: i64) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborMap_contains(self as *const crate::QCborMap, key)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborMap::contains(QLatin1String key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#contains-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns true if this map contains a key-value pair identified by key <i>key</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#value-1">value</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-1">operator[]</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#find-1">find</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-1">remove</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#contains">contains</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-2">remove</a>(const QString &amp;), and <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-3">remove</a>(const QCborValue &amp;).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn contains_q_latin1_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborMap_contains1(
            self as *const crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(key)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborMap::contains(const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#contains-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns true if this map contains a key-value pair identified by key <i>key</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#value-2">value</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-2">operator[]</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#find-2">find</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-2">remove</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#contains">contains</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-1">remove</a>(QLatin1String), and <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-3">remove</a>(const QCborValue &amp;).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn contains_q_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborMap_contains2(
            self as *const crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        )
    }

    /// <p>Returns true if this map contains a key-value pair identified by key <i>key</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborMap::contains(const QCborValue& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#contains-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if this map contains a key-value pair identified by key <i>key</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#value-3">value</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-3">operator[]</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#find-3">find</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-3">remove</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#contains">contains</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-1">remove</a>(QLatin1String), and <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-2">remove</a>(const QString &amp;).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn contains_q_cbor_value(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborMap_contains3(
            self as *const crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(key)
                .as_raw_ptr(),
        )
    }

    /// <p>Replaces the contents of this object with a copy of <i>other</i>, then returns a reference to this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborMap& QCborMap::operator=(const QCborMap& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the contents of this object with a copy of <i>other</i>, then returns a reference to this object.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborMap>>,
    ) -> ::cpp_utils::MutRef<crate::QCborMap> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_operator_(
            self as *mut crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborMap>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Synonym for <a href="http://doc.qt.io/qt-5/qcbormap.html#isEmpty">isEmpty</a>(). This function is provided for compatibility with generic code that uses the Standard Library API.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborMap::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Synonym for <a href="http://doc.qt.io/qt-5/qcbormap.html#isEmpty">isEmpty</a>(). This function is provided for compatibility with generic code that uses the Standard Library API.</p>
    /// <p>Returns true if this map is empty (<a href="http://doc.qt.io/qt-5/qcbormap.html#size">size</a>() == 0).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qcbormap.html#size">size</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborMap_empty(self as *const crate::QCborMap)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborMap::Iterator QCborMap::erase(QCborMap::Iterator it)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#erase-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes the key-value pair pointed to by the map iterator <i>it</i> and returns a pointer to the next element, after removal.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#begin">begin</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#end">end</a>(), and <a href="http://doc.qt.io/qt-5/qcbormap.html#insert">insert</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn erase_iterator(
        &mut self,
        it: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_cbor_map::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_cbor_map::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_erase(
            self as *mut crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_cbor_map::Iterator>>::cast_into(it)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the key-value pair pointed to by the map iterator <i>it</i> and returns a pointer to the next element, after removal.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborMap::Iterator QCborMap::erase(QCborMap::ConstIterator it)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the key-value pair pointed to by the map iterator <i>it</i> and returns a pointer to the next element, after removal.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#begin">begin</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#end">end</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#insert">insert</a>(), and <a href="http://doc.qt.io/qt-5/qcbormap.html#extract">extract</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn erase_const_iterator(
        &mut self,
        it: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>>,
    ) -> ::cpp_utils::CppBox<crate::q_cbor_map::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_erase1(
            self as *mut crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>>::cast_into(
                it,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborMap::extract(QCborMap::Iterator it)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn extract_iterator(
        &mut self,
        it: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_cbor_map::Iterator>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_extract(
            self as *mut crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_cbor_map::Iterator>>::cast_into(it)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborMap::extract(QCborMap::ConstIterator it)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn extract_const_iterator(
        &mut self,
        it: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_extract1(
            self as *mut crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_cbor_map::ConstIterator>>::cast_into(
                it,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QCborMap::Iterator QCborMap::find(qint64 key)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn find_i64_mut(
        &mut self,
        key: i64,
    ) -> ::cpp_utils::CppBox<crate::q_cbor_map::Iterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborMap_find(self as *mut crate::QCborMap, key);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QCborMap::Iterator QCborMap::find(QLatin1String key)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn find_q_latin1_string_mut(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::q_cbor_map::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_find1(
            self as *mut crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QCborMap::Iterator QCborMap::find(const QString& key)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn find_q_string_mut(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::q_cbor_map::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_find2(
            self as *mut crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QCborMap::Iterator QCborMap::find(const QCborValue& key)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn find_q_cbor_value_mut(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::cpp_utils::CppBox<crate::q_cbor_map::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_find3(
            self as *mut crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QCborMap::ConstIterator QCborMap::find(qint64 key) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn find_i64(
        &self,
        key: i64,
    ) -> ::cpp_utils::CppBox<crate::q_cbor_map::ConstIterator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborMap_find4(self as *const crate::QCborMap, key);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QCborMap::ConstIterator QCborMap::find(QLatin1String key) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn find_q_latin1_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::q_cbor_map::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_find5(
            self as *const crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QCborMap::ConstIterator QCborMap::find(const QString& key) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn find_q_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::q_cbor_map::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_find6(
            self as *const crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```QCborMap::ConstIterator QCborMap::find(const QCborValue& key) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn find_q_cbor_value(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::cpp_utils::CppBox<crate::q_cbor_map::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_find7(
            self as *const crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts all JSON items found in the <i>obj</i> object to CBOR using QCborValue::fromJson(), and returns the map composed of those elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QCborMap QCborMap::fromJsonObject(const QJsonObject& o)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#fromJsonObject">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts all JSON items found in the <i>obj</i> object to CBOR using QCborValue::fromJson(), and returns the map composed of those elements.</p>
    /// <p>This conversion is lossless, as the CBOR type system is a superset of JSON's. Moreover, the map returned by this function can be converted back to the original <i>obj</i> by using <a href="http://doc.qt.io/qt-5/qcbormap.html#toJsonObject">toJsonObject</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#toJsonObject">toJsonObject</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#toVariantMap">toVariantMap</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromJsonValue">QCborValue::fromJsonValue</a>(), and <a href="http://doc.qt.io/qt-5/qcborarray.html#fromJsonArray">QCborArray::fromJsonArray</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_json_object(
        o: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonObject>>,
    ) -> ::cpp_utils::CppBox<crate::QCborMap> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_fromJsonObject(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonObject>>::cast_into(o)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts all the items in <i>hash</i> to CBOR using <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromVariant">QCborValue::fromVariant</a>() and returns the map composed of those elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QCborMap QCborMap::fromVariantHash(const QHash<QString, QVariant>& hash)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#fromVariantHash">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts all the items in <i>hash</i> to CBOR using <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromVariant">QCborValue::fromVariant</a>() and returns the map composed of those elements.</p>
    /// <p>Conversion from <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> is not completely lossless. Please see the documentation in <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromVariant">QCborValue::fromVariant</a>() for more information.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#toVariantHash">toVariantHash</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#fromVariantMap">fromVariantMap</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#fromJsonObject">fromJsonObject</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromVariant">QCborValue::fromVariant</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_variant_hash(
        hash: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QHashOfQStringQVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QCborMap> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_fromVariantHash(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QHashOfQStringQVariant>>::cast_into(
                hash,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts all the items in <i>map</i> to CBOR using <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromVariant">QCborValue::fromVariant</a>() and returns the map composed of those elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```static QCborMap QCborMap::fromVariantMap(const QMap<QString, QVariant>& map)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#fromVariantMap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts all the items in <i>map</i> to CBOR using <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromVariant">QCborValue::fromVariant</a>() and returns the map composed of those elements.</p>
    /// <p>Conversion from <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> is not completely lossless. Please see the documentation in <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromVariant">QCborValue::fromVariant</a>() for more information.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#toVariantMap">toVariantMap</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#fromVariantHash">fromVariantHash</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#fromJsonObject">fromJsonObject</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromVariant">QCborValue::fromVariant</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_variant_map(
        map: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMapOfQStringQVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QCborMap> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_fromVariantMap(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMapOfQStringQVariant>>::cast_into(map)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> element in this map that corresponds to key <i>key</i>, if there is one. CBOR recommends using integer keys, since they occupy less space and are simpler to encode and decode.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborMap::operator[](qint64 key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> element in this map that corresponds to key <i>key</i>, if there is one. CBOR recommends using integer keys, since they occupy less space and are simpler to encode and decode.</p>
    /// <p>If the map does not contain key <i>key</i>, this function returns a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> containing an undefined value. For that reason, it is not possible with this function to tell apart the situation where the key was not present from the situation where the key was mapped to an undefined value.</p>
    /// <p>If the map contains more than one key equal to <i>key</i>, it is undefined which one this function will return. <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> does not allow inserting duplicate keys, but it is possible to create such a map by decoding a CBOR stream with them. They are usually not permitted and having duplicate keys is usually an indication of a problem in the sender.</p>
    /// <p>operator[](<a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>), operator[](const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;), operator[](const QCborOperator[] &amp;)</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#value">value</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#find">find</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#constFind">constFind</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove">remove</a>(qint64), and <a href="http://doc.qt.io/qt-5/qcbormap.html#contains">contains</a>(qint64).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn index_i64(&self, key: i64) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborMap_operator__(self as *const crate::QCborMap, key);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborMap::operator[](QLatin1String key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> element in this map that corresponds to key <i>key</i>, if there is one.</p>
    /// <p>If the map does not contain key <i>key</i>, this function returns a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> containing an undefined value. For that reason, it is not possible with this function to tell apart the situation where the key was not present from the situation where the key was mapped to an undefined value.</p>
    /// <p>If the map contains more than one key equal to <i>key</i>, it is undefined which one this function will return. <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> does not allow inserting duplicate keys, but it is possible to create such a map by decoding a CBOR stream with them. They are usually not permitted and having duplicate keys is usually an indication of a problem in the sender.</p>
    /// <p>operator[](qint64), operator[](const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;), operator[](const QCborOperator[] &amp;)</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#value-1">value</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#find-1">find</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#constFind-1">constFind</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-1">remove</a>(QLatin1String), and <a href="http://doc.qt.io/qt-5/qcbormap.html#contains-1">contains</a>(QLatin1String).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn index_q_latin1_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_operator__1(
            self as *const crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborMap::operator[](const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> element in this map that corresponds to key <i>key</i>, if there is one.</p>
    /// <p>If the map does not contain key <i>key</i>, this function returns a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> containing an undefined value. For that reason, it is not possible with this function to tell apart the situation where the key was not present from the situation where the key was mapped to an undefined value.</p>
    /// <p>If the map contains more than one key equal to <i>key</i>, it is undefined which one this function will return. <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> does not allow inserting duplicate keys, but it is possible to create such a map by decoding a CBOR stream with them. They are usually not permitted and having duplicate keys is usually an indication of a problem in the sender.</p>
    /// <p>operator[](qint64), operator[](<a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>), operator[](const QCborOperator[] &amp;)</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#value-2">value</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#find-2">find</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#constFind-2">constFind</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-2">remove</a>(const QString &amp;), and <a href="http://doc.qt.io/qt-5/qcbormap.html#contains-2">contains</a>(const QString &amp;).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn index_q_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_operator__2(
            self as *const crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> element in this map that corresponds to key <i>key</i>, if there is one.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborMap::operator[](const QCborValue& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> element in this map that corresponds to key <i>key</i>, if there is one.</p>
    /// <p>If the map does not contain key <i>key</i>, this function returns a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> containing an undefined value. For that reason, it is not possible with this function to tell apart the situation where the key was not present from the situation where the key was mapped to an undefined value.</p>
    /// <p>If the map contains more than one key equal to <i>key</i>, it is undefined which one this function will return. <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> does not allow inserting duplicate keys, but it is possible to create such a map by decoding a CBOR stream with them. They are usually not permitted and having duplicate keys is usually an indication of a problem in the sender.</p>
    /// <p>operator[](qint64), operator[](<a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>), operator[](const QCborOperator[] &amp;)</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#value-3">value</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#find-3">find</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#constFind-3">constFind</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-3">remove</a>(const QCborValue &amp;), and <a href="http://doc.qt.io/qt-5/qcbormap.html#contains-3">contains</a>(const QCborValue &amp;).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn index_q_cbor_value(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_operator__3(
            self as *const crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> to the value in this map that corresponds to key <i>key</i>. CBOR recommends using integer keys, since they occupy less space and are simpler to encode and decode.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValueRef QCborMap::operator[](qint64 key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> to the value in this map that corresponds to key <i>key</i>. CBOR recommends using integer keys, since they occupy less space and are simpler to encode and decode.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> has the exact same API as <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, with one important difference: if you assign new values to it, this map will be updated with that new value.</p>
    /// <p>If the map did not have a key equal to <i>key</i>, one is inserted and this function returns a reference to the new value, which will be a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> with an undefined value. For that reason, it is not possible with this function to tell apart the situation where the key was not present from the situation where the key was mapped to an undefined value.</p>
    /// <p>If the map contains more than one key equal to <i>key</i>, it is undefined which one the return will reference. <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> does not allow inserting duplicate keys, but it is possible to create such a map by decoding a CBOR stream with them. They are usually not permitted and having duplicate keys is usually an indication of a problem in the sender.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#value">value</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#find">find</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#contains">contains</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove">remove</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-1">operator[]</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-2">operator[]</a>(const QString &amp;), and <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-3">operator[]</a>(const QCborValue &amp;).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn index_i64_mut(&mut self, key: i64) -> ::cpp_utils::CppBox<crate::QCborValueRef> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborMap_operator__4(self as *mut crate::QCborMap, key);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValueRef QCborMap::operator[](QLatin1String key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> to the value in this map that corresponds to key <i>key</i>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> has the exact same API as <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, with one important difference: if you assign new values to it, this map will be updated with that new value.</p>
    /// <p>If the map did not have a key equal to <i>key</i>, one is inserted and this function returns a reference to the new value, which will be a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> with an undefined value. For that reason, it is not possible with this function to tell apart the situation where the key was not present from the situation where the key was mapped to an undefined value.</p>
    /// <p>If the map contains more than one key equal to <i>key</i>, it is undefined which one the return will reference. <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> does not allow inserting duplicate keys, but it is possible to create such a map by decoding a CBOR stream with them. They are usually not permitted and having duplicate keys is usually an indication of a problem in the sender.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#value-1">value</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#find-1">find</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#contains-1">contains</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-1">remove</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d">operator[]</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-2">operator[]</a>(const QString &amp;), and <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-3">operator[]</a>(const QCborValue &amp;).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn index_q_latin1_string_mut(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValueRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_operator__5(
            self as *mut crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValueRef QCborMap::operator[](const QString& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> to the value in this map that corresponds to key <i>key</i>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> has the exact same API as <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, with one important difference: if you assign new values to it, this map will be updated with that new value.</p>
    /// <p>If the map did not have a key equal to <i>key</i>, one is inserted and this function returns a reference to the new value, which will be a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> with an undefined value. For that reason, it is not possible with this function to tell apart the situation where the key was not present from the situation where the key was mapped to an undefined value.</p>
    /// <p>If the map contains more than one key equal to <i>key</i>, it is undefined which one the return will reference. <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> does not allow inserting duplicate keys, but it is possible to create such a map by decoding a CBOR stream with them. They are usually not permitted and having duplicate keys is usually an indication of a problem in the sender.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#value-2">value</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#find-2">find</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#contains-2">contains</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-2">remove</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d">operator[]</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-1">operator[]</a>(QLatin1String), and <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-3">operator[]</a>(const QCborValue &amp;).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn index_q_string_mut(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValueRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_operator__6(
            self as *mut crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValueRef QCborMap::operator[](const QCborValue& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> to the value in this map that corresponds to key <i>key</i>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qcborvalue.html#qcborvalueref">QCborValueRef</a> has the exact same API as <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a>, with one important difference: if you assign new values to it, this map will be updated with that new value.</p>
    /// <p>If the map did not have a key equal to <i>key</i>, one is inserted and this function returns a reference to the new value, which will be a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> with an undefined value. For that reason, it is not possible with this function to tell apart the situation where the key was not present from the situation where the key was mapped to an undefined value.</p>
    /// <p>If the map contains more than one key equal to <i>key</i>, it is undefined which one the return will reference. <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> does not allow inserting duplicate keys, but it is possible to create such a map by decoding a CBOR stream with them. They are usually not permitted and having duplicate keys is usually an indication of a problem in the sender.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#value-3">value</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#find-3">find</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#contains-3">contains</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-3">remove</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d">operator[]</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-1">operator[]</a>(QLatin1String), and <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-2">operator[]</a>(const QString &amp;).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn index_q_cbor_value_mut(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValueRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_operator__7(
            self as *mut crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborMap::Iterator QCborMap::insert(qint64 key, const QCborValue& value_)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts the key <i>key</i> and value <i>value</i> into this map and returns a map iterator pointing to the newly inserted pair.</p>
    /// <p>If the map already had a key equal to <i>key</i>, its value will be overwritten by <i>value</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#erase">erase</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove">remove</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#value">value</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d">operator[]</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#find">find</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#contains">contains</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#take">take</a>(qint64), and <a href="http://doc.qt.io/qt-5/qcbormap.html#extract">extract</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn insert_i64_q_cbor_value(
        &mut self,
        key: i64,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::cpp_utils::CppBox<crate::q_cbor_map::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_insert(
            self as *mut crate::QCborMap,
            key,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborMap::Iterator QCborMap::insert(QLatin1String key, const QCborValue& value_)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts the key <i>key</i> and value <i>value</i> into this map and returns a map iterator pointing to the newly inserted pair.</p>
    /// <p>If the map already had a key equal to <i>key</i>, its value will be overwritten by <i>value</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#erase">erase</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-1">remove</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#value-1">value</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-1">operator[]</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#find-1">find</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#contains-1">contains</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#take-1">take</a>(QLatin1String), and <a href="http://doc.qt.io/qt-5/qcbormap.html#extract">extract</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn insert_q_latin1_string_q_cbor_value(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::cpp_utils::CppBox<crate::q_cbor_map::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_insert1(
            self as *mut crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(key)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborMap::Iterator QCborMap::insert(const QString& key, const QCborValue& value_)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#insert-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts the key <i>key</i> and value <i>value</i> into this map and returns a map iterator pointing to the newly inserted pair.</p>
    /// <p>If the map already had a key equal to <i>key</i>, its value will be overwritten by <i>value</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#erase">erase</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-2">remove</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#value-2">value</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-2">operator[]</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#find-2">find</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#contains-2">contains</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#take-2">take</a>(const QString &amp;), and <a href="http://doc.qt.io/qt-5/qcbormap.html#extract">extract</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn insert_q_string_q_cbor_value(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::cpp_utils::CppBox<crate::q_cbor_map::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_insert2(
            self as *mut crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborMap::Iterator QCborMap::insert(const QCborValue& key, const QCborValue& value_)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#insert-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts the key <i>key</i> and value <i>value</i> into this map and returns a map iterator pointing to the newly inserted pair.</p>
    /// <p>If the map already had a key equal to <i>key</i>, its value will be overwritten by <i>value</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#erase">erase</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-3">remove</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#value-3">value</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-3">operator[]</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#find-3">find</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#contains-3">contains</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#take-3">take</a>(const QCborValue &amp;), and <a href="http://doc.qt.io/qt-5/qcbormap.html#extract">extract</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn insert_2_q_cbor_value(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::cpp_utils::CppBox<crate::q_cbor_map::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_insert3(
            self as *mut crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(key)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborMap::Iterator QCborMap::insert(QPair<QCborValue, QCborValue> v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#insert-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts the key-value pair in <i>v</i> into this map and returns a map iterator pointing to the newly inserted pair.</p>
    /// <p>If the map already had a key equal to <code>v.first</code>, its value will be overwritten by <code>v.second</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d">operator[]</a>, <a href="http://doc.qt.io/qt-5/qcbormap.html#erase">erase</a>(), and <a href="http://doc.qt.io/qt-5/qcbormap.html#extract">extract</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn insert_q_pair_of_q_cbor_value_q_cbor_value(
        &mut self,
        v: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQCborValueQCborValue>>,
    ) -> ::cpp_utils::CppBox<crate::q_cbor_map::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_insert4(self as *mut crate::QCborMap, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQCborValueQCborValue>>::cast_into(v).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns true if this map is empty (that is, <a href="http://doc.qt.io/qt-5/qcbormap.html#size">size</a>() is 0).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborMap::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if this map is empty (that is, <a href="http://doc.qt.io/qt-5/qcbormap.html#size">size</a>() is 0).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qcbormap.html#clear">clear</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborMap_isEmpty(self as *const crate::QCborMap)
    }

    /// <p>Returns a list of all keys in this map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QCborValue> QCborMap::keys() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#keys">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list of all keys in this map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#keys">QMap::keys</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#keys">QHash::keys</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn keys(&self) -> ::cpp_utils::CppBox<crate::QVectorOfQCborValue> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborMap_keys(self as *const crate::QCborMap);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs an empty CBOR Map object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborMap::QCborMap()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#QCborMap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty CBOR Map object.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#isEmpty">isEmpty</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QCborMap> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_QCborMap();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> object that is a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborMap::QCborMap(const QCborMap& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#QCborMap-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> object that is a copy of <i>other</i>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborMap>>,
    ) -> ::cpp_utils::CppBox<crate::QCborMap> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_QCborMap1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborMap>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the key <i>key</i> and the corresponding value from the map, if it is found. If the map contains no such key, this function does nothing.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborMap::remove(qint64 key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#remove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the key <i>key</i> and the corresponding value from the map, if it is found. If the map contains no such key, this function does nothing.</p>
    /// <p>If the map contains more than one key equal to <i>key</i>, it is undefined which one this function will remove. <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> does not allow inserting duplicate keys, but it is possible to create such a map by decoding a CBOR stream with them. They are usually not permitted and having duplicate keys is usually an indication of a problem in the sender.</p>
    /// <p>remove(<a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>), remove(const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;), remove(const <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> &amp;)</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#value">value</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d">operator[]</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#find">find</a>(qint64), and <a href="http://doc.qt.io/qt-5/qcbormap.html#contains">contains</a>(qint64).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn remove_i64(&mut self, key: i64) {
        crate::__ffi::ctr_qt_core_ffi_QCborMap_remove(self as *mut crate::QCborMap, key)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborMap::remove(QLatin1String key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes the key <i>key</i> and the corresponding value from the map, if it is found. If the map contains no such key, this function does nothing.</p>
    /// <p>If the map contains more than one key equal to <i>key</i>, it is undefined which one this function will remove. <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> does not allow inserting duplicate keys, but it is possible to create such a map by decoding a CBOR stream with them. They are usually not permitted and having duplicate keys is usually an indication of a problem in the sender.</p>
    /// <p>remove(qint64), remove(const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;), remove(const <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> &amp;)</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#value-1">value</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-1">operator[]</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#find-1">find</a>(QLatin1String), and <a href="http://doc.qt.io/qt-5/qcbormap.html#contains-1">contains</a>(QLatin1String).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn remove_q_latin1_string(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborMap_remove1(
            self as *mut crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(key)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborMap::remove(const QString& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes the key <i>key</i> and the corresponding value from the map, if it is found. If the map contains no such key, this function does nothing.</p>
    /// <p>If the map contains more than one key equal to <i>key</i>, it is undefined which one this function will remove. <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> does not allow inserting duplicate keys, but it is possible to create such a map by decoding a CBOR stream with them. They are usually not permitted and having duplicate keys is usually an indication of a problem in the sender.</p>
    /// <p>remove(qint64), remove(<a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>), remove(const <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> &amp;)</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#value-2">value</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-2">operator[]</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#find-2">find</a>(const QString &amp;), and <a href="http://doc.qt.io/qt-5/qcbormap.html#contains-2">contains</a>(const QString &amp;).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn remove_q_string(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborMap_remove2(
            self as *mut crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        )
    }

    /// <p>Removes the key <i>key</i> and the corresponding value from the map, if it is found. If the map contains no such key, this function does nothing.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborMap::remove(const QCborValue& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the key <i>key</i> and the corresponding value from the map, if it is found. If the map contains no such key, this function does nothing.</p>
    /// <p>If the map contains more than one key equal to <i>key</i>, it is undefined which one this function will remove. <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> does not allow inserting duplicate keys, but it is possible to create such a map by decoding a CBOR stream with them. They are usually not permitted and having duplicate keys is usually an indication of a problem in the sender.</p>
    /// <p>remove(qint64), remove(<a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>), remove(const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;)</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#value-3">value</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-3">operator[]</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#find-3">find</a>(const QCborValue &amp;), and <a href="http://doc.qt.io/qt-5/qcbormap.html#contains-3">contains</a>(const QCborValue &amp;).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn remove_q_cbor_value(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborMap_remove3(
            self as *mut crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(key)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the number of elements in this map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```long long QCborMap::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of elements in this map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#isEmpty">isEmpty</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn size(&self) -> ::std::os::raw::c_longlong {
        crate::__ffi::ctr_qt_core_ffi_QCborMap_size(self as *const crate::QCborMap)
    }

    /// <p>Swaps the contents of this map and <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborMap::swap(QCborMap& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps the contents of this map and <i>other</i>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QCborMap>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborMap_swap(
            self as *mut crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QCborMap>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Removes the key <i>key</i> and the corresponding value from the map and returns the value, if it is found. If the map contains no such key, this function does nothing.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborMap::take(qint64 key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#take">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the key <i>key</i> and the corresponding value from the map and returns the value, if it is found. If the map contains no such key, this function does nothing.</p>
    /// <p>If the map contains more than one key equal to <i>key</i>, it is undefined which one this function will remove. <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> does not allow inserting duplicate keys, but it is possible to create such a map by decoding a CBOR stream with them. They are usually not permitted and having duplicate keys is usually an indication of a problem in the sender.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#value">value</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d">operator[]</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#find">find</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#contains">contains</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#take-1">take</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#take-2">take</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#take-3">take</a>(const QCborValue &amp;), and <a href="http://doc.qt.io/qt-5/qcbormap.html#insert">insert</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn take_i64(&mut self, key: i64) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborMap_take(self as *mut crate::QCborMap, key);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the key <i>key</i> and the corresponding value from the map and returns the value, if it is found. If the map contains no such key, this function does nothing.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborMap::take(QLatin1String key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#take-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the key <i>key</i> and the corresponding value from the map and returns the value, if it is found. If the map contains no such key, this function does nothing.</p>
    /// <p>If the map contains more than one key equal to <i>key</i>, it is undefined which one this function will remove. <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> does not allow inserting duplicate keys, but it is possible to create such a map by decoding a CBOR stream with them. They are usually not permitted and having duplicate keys is usually an indication of a problem in the sender.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#value-1">value</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-1">operator[]</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#find-1">find</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#contains-1">contains</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#take">take</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#take-2">take</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#take-3">take</a>(const QCborValue &amp;), and <a href="http://doc.qt.io/qt-5/qcbormap.html#insert">insert</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn take_q_latin1_string(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_take1(
            self as *mut crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the key <i>key</i> and the corresponding value from the map and returns the value, if it is found. If the map contains no such key, this function does nothing.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborMap::take(const QString& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#take-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the key <i>key</i> and the corresponding value from the map and returns the value, if it is found. If the map contains no such key, this function does nothing.</p>
    /// <p>If the map contains more than one key equal to <i>key</i>, it is undefined which one this function will remove. <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> does not allow inserting duplicate keys, but it is possible to create such a map by decoding a CBOR stream with them. They are usually not permitted and having duplicate keys is usually an indication of a problem in the sender.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#value-2">value</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-2">operator[]</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#find-2">find</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#contains-2">contains</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#take-1">take</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#take">take</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#take-3">take</a>(const QCborValue &amp;), and <a href="http://doc.qt.io/qt-5/qcbormap.html#insert">insert</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn take_q_string(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_take2(
            self as *mut crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the key <i>key</i> and the corresponding value from the map and returns the value, if it is found. If the map contains no such key, this function does nothing.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborMap::take(const QCborValue& key)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#take-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the key <i>key</i> and the corresponding value from the map and returns the value, if it is found. If the map contains no such key, this function does nothing.</p>
    /// <p>If the map contains more than one key equal to <i>key</i>, it is undefined which one this function will remove. <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> does not allow inserting duplicate keys, but it is possible to create such a map by decoding a CBOR stream with them. They are usually not permitted and having duplicate keys is usually an indication of a problem in the sender.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#value-3">value</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-3">operator[]</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#find-3">find</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#contains-3">contains</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#take-1">take</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#take-2">take</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#take">take</a>(qint64), and <a href="http://doc.qt.io/qt-5/qcbormap.html#insert">insert</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn take_q_cbor_value(
        &mut self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_take3(
            self as *mut crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Explicitly constructs a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object that represents this map. This function is usually not necessary since <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> has a constructor for <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>, so the conversion is implicit.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborMap::toCborValue() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#toCborValue">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Explicitly constructs a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object that represents this map. This function is usually not necessary since <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> has a constructor for <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>, so the conversion is implicit.</p>
    /// <p>Converting <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> to <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> allows it to be used in any context where QCborValues can be used, including as keys and mapped types in <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a>, as well as <a href="http://doc.qt.io/qt-5/qcborvalue.html#toCbor">QCborValue::toCbor</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#QCborValue-13">QCborValue::QCborValue</a>(const QCborMap &amp;).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_cbor_value(&self) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborMap_toCborValue(self as *const crate::QCborMap);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Recursively converts every <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> value in this array to JSON using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toJsonValue">QCborValue::toJsonValue</a>() and creates a string key for all keys that aren't strings, then returns the corresponding <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> composed of those associations.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonObject QCborMap::toJsonObject() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#toJsonObject">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Recursively converts every <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> value in this array to JSON using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toJsonValue">QCborValue::toJsonValue</a>() and creates a string key for all keys that aren't strings, then returns the corresponding <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> composed of those associations.</p>
    /// <p>Please note that CBOR contains a richer and wider type set than JSON, so some information may be lost in this conversion. For more details on what conversions are applied, see <a href="http://doc.qt.io/qt-5/qcborvalue.html#toJsonValue">QCborValue::toJsonValue</a>().</p>
    /// <a name="map-key-conversion-to-string"></a>
    /// <h6>Map key conversion to string</h6>
    /// <p>JSON objects are defined as having string keys, unlike CBOR, so the conversion of a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> to <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> will imply a step of "stringification" of the key values. The conversion will use the special handling of tags and extended types from above and will also convert the rest of the types as follows:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Type</th><th>Transformation</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>Bool</td><td>"true" and "false"</td></tr>
    /// <tr valign="top" class="even"><td>Null</td><td>"null"</td></tr>
    /// <tr valign="top" class="odd"><td>Undefined</td><td>"undefined"</td></tr>
    /// <tr valign="top" class="even"><td>Integer</td><td>The decimal string form of the number</td></tr>
    /// <tr valign="top" class="odd"><td>Double</td><td>The decimal string form of the number</td></tr>
    /// <tr valign="top" class="even"><td>Byte array</td><td>Unless tagged differently (see above), encoded as Base64url</td></tr>
    /// <tr valign="top" class="odd"><td>Array</td><td>Replaced by the compact form of its <a href="http://doc.qt.io/qt-5/qcborvalue.html#toDiagnosticNotation">Diagnostic notation</a></td></tr>
    /// <tr valign="top" class="even"><td>Map</td><td>Replaced by the compact form of its <a href="http://doc.qt.io/qt-5/qcborvalue.html#toDiagnosticNotation">Diagnostic notation</a></td></tr>
    /// <tr valign="top" class="odd"><td>Tags and extended types</td><td>Tag number is dropped and the tagged value is converted to string</td></tr>
    /// </tbody></table></div>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#fromJsonObject">fromJsonObject</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#toJsonValue">QCborValue::toJsonValue</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#toJsonArray">QCborArray::toJsonArray</a>(), and <a href="http://doc.qt.io/qt-5/qcbormap.html#toVariantMap">toVariantMap</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_json_object(&self) -> ::cpp_utils::CppBox<crate::QJsonObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborMap_toJsonObject(self as *const crate::QCborMap);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts the CBOR values to <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toVariant">QCborValue::toVariant</a>() and "stringifies" all the CBOR keys in this map, returning the <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantHash-typedef">QVariantHash</a> that results from that association list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant> QCborMap::toVariantHash() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#toVariantHash">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the CBOR values to <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toVariant">QCborValue::toVariant</a>() and "stringifies" all the CBOR keys in this map, returning the <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantHash-typedef">QVariantHash</a> that results from that association list.</p>
    /// <p>QVariantMaps have string keys, unlike CBOR, so the conversion of a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> to <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantMap-typedef">QVariantMap</a> will imply a step of "stringification" of the key values. See <a href="http://doc.qt.io/qt-5/qcbormap.html#toJsonObject">QCborMap::toJsonObject</a>() for details.</p>
    /// <p>In addition, the conversion to <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> is not completely lossless. Please see the documentation in <a href="http://doc.qt.io/qt-5/qcborvalue.html#toVariant">QCborValue::toVariant</a>() for more information.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#fromVariantHash">fromVariantHash</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#toVariantMap">toVariantMap</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#toJsonObject">toJsonObject</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#toVariant">QCborValue::toVariant</a>(), and <a href="http://doc.qt.io/qt-5/qcborarray.html#toVariantList">QCborArray::toVariantList</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_variant_hash(&self) -> ::cpp_utils::CppBox<crate::QHashOfQStringQVariant> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborMap_toVariantHash(self as *const crate::QCborMap);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Converts the CBOR values to <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toVariant">QCborValue::toVariant</a>() and "stringifies" all the CBOR keys in this map, returning the <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantMap-typedef">QVariantMap</a> that results from that association list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant> QCborMap::toVariantMap() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#toVariantMap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts the CBOR values to <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> using <a href="http://doc.qt.io/qt-5/qcborvalue.html#toVariant">QCborValue::toVariant</a>() and "stringifies" all the CBOR keys in this map, returning the <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantMap-typedef">QVariantMap</a> that results from that association list.</p>
    /// <p>QVariantMaps have string keys, unlike CBOR, so the conversion of a <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> to <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantMap-typedef">QVariantMap</a> will imply a step of "stringification" of the key values. See <a href="http://doc.qt.io/qt-5/qcbormap.html#toJsonObject">QCborMap::toJsonObject</a>() for details.</p>
    /// <p>In addition, the conversion to <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> is not completely lossless. Please see the documentation in <a href="http://doc.qt.io/qt-5/qcborvalue.html#toVariant">QCborValue::toVariant</a>() for more information.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#fromVariantMap">fromVariantMap</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#toVariantHash">toVariantHash</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#toJsonObject">toJsonObject</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#toVariant">QCborValue::toVariant</a>(), and <a href="http://doc.qt.io/qt-5/qcborarray.html#toVariantList">QCborArray::toVariantList</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_variant_map(&self) -> ::cpp_utils::CppBox<crate::QMapOfQStringQVariant> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborMap_toVariantMap(self as *const crate::QCborMap);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> element in this map that corresponds to key <i>key</i>, if there is one. CBOR recommends using integer keys, since they occupy less space and are simpler to encode and decode.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborMap::value(qint64 key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> element in this map that corresponds to key <i>key</i>, if there is one. CBOR recommends using integer keys, since they occupy less space and are simpler to encode and decode.</p>
    /// <p>If the map does not contain key <i>key</i>, this function returns a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> containing an undefined value. For that reason, it is not possible with this function to tell apart the situation where the key was not present from the situation where the key was mapped to an undefined value.</p>
    /// <p>If the map contains more than one key equal to <i>key</i>, it is undefined which one the return from function will reference. <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> does not allow inserting duplicate keys, but it is possible to create such a map by decoding a CBOR stream with them. They are usually not permitted and having duplicate keys is usually an indication of a problem in the sender.</p>
    /// <p>value(<a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>), value(const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;), value(const <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> &amp;)</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d">operator[]</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#find">find</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#constFind">constFind</a>(qint64), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove">remove</a>(qint64), and <a href="http://doc.qt.io/qt-5/qcbormap.html#contains">contains</a>(qint64).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn value_i64(&self, key: i64) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QCborMap_value(self as *const crate::QCborMap, key);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborMap::value(QLatin1String key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> element in this map that corresponds to key <i>key</i>, if there is one.</p>
    /// <p>If the map does not contain key <i>key</i>, this function returns a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> containing an undefined value. For that reason, it is not possible with this function to tell apart the situation where the key was not present from the situation where the key was mapped to an undefined value.</p>
    /// <p>If the map contains more than one key equal to <i>key</i>, it is undefined which one this function will return. <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> does not allow inserting duplicate keys, but it is possible to create such a map by decoding a CBOR stream with them. They are usually not permitted and having duplicate keys is usually an indication of a problem in the sender.</p>
    /// <p>value(qint64), value(const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;), value(const <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> &amp;)</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-1">operator[]</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#find-1">find</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#constFind-1">constFind</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-1">remove</a>(QLatin1String), and <a href="http://doc.qt.io/qt-5/qcbormap.html#contains-1">contains</a>(QLatin1String).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn value_q_latin1_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_value1(
            self as *const crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborMap::value(const QString& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#value-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> element in this map that corresponds to key <i>key</i>, if there is one.</p>
    /// <p>If the map does not contain key <i>key</i>, this function returns a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> containing an undefined value. For that reason, it is not possible with this function to tell apart the situation where the key was not present from the situation where the key was mapped to an undefined value.</p>
    /// <p>If the map contains more than one key equal to <i>key</i>, it is undefined which one this function will return. <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> does not allow inserting duplicate keys, but it is possible to create such a map by decoding a CBOR stream with them. They are usually not permitted and having duplicate keys is usually an indication of a problem in the sender.</p>
    /// <p>value(qint64), value(<a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>), value(const <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> &amp;)</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-2">operator[]</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#find-2">find</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#constFind-2">constFind</a>(const QString &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-2">remove</a>(const QString &amp;), and <a href="http://doc.qt.io/qt-5/qcbormap.html#contains-2">contains</a>(const QString &amp;).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn value_q_string(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_value2(
            self as *const crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> element in this map that corresponds to key <i>key</i>, if there is one.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QCborMap::value(const QCborValue& key) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#value-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> element in this map that corresponds to key <i>key</i>, if there is one.</p>
    /// <p>If the map does not contain key <i>key</i>, this function returns a <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> containing an undefined value. For that reason, it is not possible with this function to tell apart the situation where the key was not present from the situation where the key was mapped to an undefined value.</p>
    /// <p>If the map contains more than one key equal to <i>key</i>, it is undefined which one this function will return. <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> does not allow inserting duplicate keys, but it is possible to create such a map by decoding a CBOR stream with them. They are usually not permitted and having duplicate keys is usually an indication of a problem in the sender.</p>
    /// <p>value(qint64), value(<a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>), value(const <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> &amp;)</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#operator-5b-5d-3">operator[]</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#find-3">find</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#constFind-3">constFind</a>(const QCborValue &amp;), <a href="http://doc.qt.io/qt-5/qcbormap.html#remove-3">remove</a>(const QCborValue &amp;), and <a href="http://doc.qt.io/qt-5/qcbormap.html#contains-3">contains</a>(const QCborValue &amp;).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn value_q_cbor_value(
        &self,
        key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborMap_value3(
            self as *const crate::QCborMap,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(key)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>This header file provides support for half-precision (16-bit) floating point data with the class <code>qfloat16</code>. It is fully compliant with IEEE 754 as a storage type. This implies that any arithmetic operation on a <code>qfloat16</code> instance results in the value first being converted to a <code>float</code>. This conversion to and from <code>float</code> is performed by hardware when possible, but on processors that do not natively support half-precision, the conversion is performed through a sequence of lookup table operations.</p>
///
/// C++ class: <span style='color: green;'>```qfloat16```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qfloat16.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This header file provides support for half-precision (16-bit) floating point data with the class <code>qfloat16</code>. It is fully compliant with IEEE 754 as a storage type. This implies that any arithmetic operation on a <code>qfloat16</code> instance results in the value first being converted to a <code>float</code>. This conversion to and from <code>float</code> is performed by hardware when possible, but on processors that do not natively support half-precision, the conversion is performed through a sequence of lookup table operations.</p>
/// <p><code>qfloat16</code> should be treated as if it were a POD (plain old data) type. Consequently, none of the supported operations need any elaboration beyond stating that it supports all arithmetic operators incident to floating point types.</p></div>
#[repr(C)]
pub struct Qfloat16 {
    _unused: u8,
}
impl Qfloat16 {
    /// Calls C++ function: <span style='color: green;'>```float operator+(qfloat16 lhs, float rhs)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn add(&self, rhs: ::std::os::raw::c_float) -> ::std::os::raw::c_float {
        crate::__ffi::ctr_qt_core_ffi_operator_457(self as *const crate::Qfloat16, rhs)
    }

    /// Calls C++ function: <span style='color: green;'>```qfloat16& operator+=(qfloat16& lhs, float rhs)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn add_assign(
        &mut self,
        rhs: ::std::os::raw::c_float,
    ) -> ::cpp_utils::MutRef<crate::Qfloat16> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_operator__1146(self as *mut crate::Qfloat16, rhs);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This header file provides support for half-precision (16-bit) floating point data with the class <code>qfloat16</code>. It is fully compliant with IEEE 754 as a storage type. This implies that any arithmetic operation on a <code>qfloat16</code> instance results in the value first being converted to a <code>float</code>. This conversion to and from <code>float</code> is performed by hardware when possible, but on processors that do not natively support half-precision, the conversion is performed through a sequence of lookup table operations.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qfloat16& qfloat16::operator=(const qfloat16& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfloat16.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This header file provides support for half-precision (16-bit) floating point data with the class <code>qfloat16</code>. It is fully compliant with IEEE 754 as a storage type. This implies that any arithmetic operation on a <code>qfloat16</code> instance results in the value first being converted to a <code>float</code>. This conversion to and from <code>float</code> is performed by hardware when possible, but on processors that do not natively support half-precision, the conversion is performed through a sequence of lookup table operations.</p>
    /// <p><code>qfloat16</code> should be treated as if it were a POD (plain old data) type. Consequently, none of the supported operations need any elaboration beyond stating that it supports all arithmetic operators incident to floating point types.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::Qfloat16>>,
    ) -> ::cpp_utils::MutRef<crate::Qfloat16> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_qfloat16_operator_(
            self as *mut crate::Qfloat16,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::Qfloat16>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```float operator/(qfloat16 lhs, float rhs)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn div(&self, rhs: ::std::os::raw::c_float) -> ::std::os::raw::c_float {
        crate::__ffi::ctr_qt_core_ffi_operator_463(self as *const crate::Qfloat16, rhs)
    }

    /// Calls C++ function: <span style='color: green;'>```qfloat16& operator/=(qfloat16& lhs, float rhs)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn div_assign(
        &mut self,
        rhs: ::std::os::raw::c_float,
    ) -> ::cpp_utils::MutRef<crate::Qfloat16> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_operator__1149(self as *mut crate::Qfloat16, rhs);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(qfloat16 lhs, float rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn eq(&self, rhs: ::std::os::raw::c_float) -> bool {
        crate::__ffi::ctr_qt_core_ffi_operator__1174(self as *const crate::Qfloat16, rhs)
    }

    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(qfloat16 lhs, float rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn ge(&self, rhs: ::std::os::raw::c_float) -> bool {
        crate::__ffi::ctr_qt_core_ffi_operator__1170(self as *const crate::Qfloat16, rhs)
    }

    /// Calls C++ function: <span style='color: green;'>```bool operator>(qfloat16 lhs, float rhs)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn gt(&self, rhs: ::std::os::raw::c_float) -> bool {
        crate::__ffi::ctr_qt_core_ffi_operator_485(self as *const crate::Qfloat16, rhs)
    }

    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(qfloat16 lhs, float rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn le(&self, rhs: ::std::os::raw::c_float) -> bool {
        crate::__ffi::ctr_qt_core_ffi_operator__1172(self as *const crate::Qfloat16, rhs)
    }

    /// Calls C++ function: <span style='color: green;'>```bool operator<(qfloat16 lhs, float rhs)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn lt(&self, rhs: ::std::os::raw::c_float) -> bool {
        crate::__ffi::ctr_qt_core_ffi_operator_483(self as *const crate::Qfloat16, rhs)
    }

    /// Calls C++ function: <span style='color: green;'>```float operator*(qfloat16 lhs, float rhs)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn mul(&self, rhs: ::std::os::raw::c_float) -> ::std::os::raw::c_float {
        crate::__ffi::ctr_qt_core_ffi_operator_461(self as *const crate::Qfloat16, rhs)
    }

    /// Calls C++ function: <span style='color: green;'>```qfloat16& operator*=(qfloat16& lhs, float rhs)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn mul_assign(
        &mut self,
        rhs: ::std::os::raw::c_float,
    ) -> ::cpp_utils::MutRef<crate::Qfloat16> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_operator__1148(self as *mut crate::Qfloat16, rhs);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void qfloat16::qfloat16()```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::Qfloat16> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_qfloat16_qfloat16();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```[constructor] void qfloat16::qfloat16(float f)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn new_1a(f: ::std::os::raw::c_float) -> ::cpp_utils::CppBox<crate::Qfloat16> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_qfloat16_qfloat161(f);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This header file provides support for half-precision (16-bit) floating point data with the class <code>qfloat16</code>. It is fully compliant with IEEE 754 as a storage type. This implies that any arithmetic operation on a <code>qfloat16</code> instance results in the value first being converted to a <code>float</code>. This conversion to and from <code>float</code> is performed by hardware when possible, but on processors that do not natively support half-precision, the conversion is performed through a sequence of lookup table operations.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void qfloat16::qfloat16(const qfloat16& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfloat16.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This header file provides support for half-precision (16-bit) floating point data with the class <code>qfloat16</code>. It is fully compliant with IEEE 754 as a storage type. This implies that any arithmetic operation on a <code>qfloat16</code> instance results in the value first being converted to a <code>float</code>. This conversion to and from <code>float</code> is performed by hardware when possible, but on processors that do not natively support half-precision, the conversion is performed through a sequence of lookup table operations.</p>
    /// <p><code>qfloat16</code> should be treated as if it were a POD (plain old data) type. Consequently, none of the supported operations need any elaboration beyond stating that it supports all arithmetic operators incident to floating point types.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::Qfloat16>>,
    ) -> ::cpp_utils::CppBox<crate::Qfloat16> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_qfloat16_qfloat162(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::Qfloat16>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```float operator-(qfloat16 lhs, float rhs)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn sub(&self, rhs: ::std::os::raw::c_float) -> ::std::os::raw::c_float {
        crate::__ffi::ctr_qt_core_ffi_operator_459(self as *const crate::Qfloat16, rhs)
    }

    /// Calls C++ function: <span style='color: green;'>```qfloat16& operator-=(qfloat16& lhs, float rhs)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn sub_assign(
        &mut self,
        rhs: ::std::os::raw::c_float,
    ) -> ::cpp_utils::MutRef<crate::Qfloat16> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_operator__1147(self as *mut crate::Qfloat16, rhs);
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```float qfloat16::operator float() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_float(&self) -> ::std::os::raw::c_float {
        crate::__ffi::ctr_qt_core_ffi_qfloat16_operator_float(self as *const crate::Qfloat16)
    }
}

/// C++ enum: <span style='color: green;'>```QCborNegativeInteger```</span>.
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
pub struct QCborNegativeInteger(::std::os::raw::c_int);

impl From<::std::os::raw::c_int> for QCborNegativeInteger {
    fn from(value: ::std::os::raw::c_int) -> Self {
        QCborNegativeInteger(value)
    }
}

impl From<QCborNegativeInteger> for ::std::os::raw::c_int {
    fn from(value: QCborNegativeInteger) -> Self {
        value.0
    }
}

impl QCborNegativeInteger {
    pub fn to_int(&self) -> ::std::os::raw::c_int {
        self.0
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> class is a simple CBOR encoder operating on a one-way stream.</p>
///
/// C++ class: <span style='color: green;'>```QCborStreamWriter```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> class is a simple CBOR encoder operating on a one-way stream.</p>
/// <p>This class can be used to quickly encode a stream of CBOR content directly to either a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> or <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>. CBOR is the Concise Binary Object Representation, a very compact form of binary data encoding that is compatible with JSON. It was created by the IETF Constrained RESTful Environments (CoRE) WG, which has used it in many new RFCs. It is meant to be used alongside the <a href="https://tools.ietf.org/html/rfc7252">CoAP protocol</a>.</p>
/// <p><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> provides a StAX-like API, similar to that of <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a>. It is rather low-level and requires a bit of knowledge of CBOR encoding. For a simpler API, see <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> and especially the encoding function <a href="http://doc.qt.io/qt-5/qcborvalue.html#toCbor">QCborValue::toCbor</a>().</p>
/// <p>The typical use of <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> is to create the object on the target <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> or <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>, then call one of the <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append">append</a>() overloads with the desired type to be encoded. To create arrays and maps, <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> provides <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startArray">startArray</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startMap">startMap</a>() overloads, which must be terminated by the corresponding <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#endArray">endArray</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#endMap">endMap</a>() functions.</p>
/// <p>The following example encodes the equivalent of this JSON content:</p>
/// <div class="pre"><p>{ "label": "journald", "autoDetect": false, "condition": "libs.journald", "output": [ "privateFeature" ] }</p>
/// </div><pre class="cpp">
///
/// &#32;    writer<span class="operator">.</span>startMap(<span class="number">4</span>);&#32;   <span class="comment">// 4 elements in the map</span>
///
/// &#32;    writer<span class="operator">.</span>append(QLatin1String(<span class="string">"label"</span>));
/// &#32;    writer<span class="operator">.</span>append(QLatin1String(<span class="string">"journald"</span>));
///
/// &#32;    writer<span class="operator">.</span>append(QLatin1String(<span class="string">"autoDetect"</span>));
/// &#32;    writer<span class="operator">.</span>append(<span class="keyword">false</span>);
///
/// &#32;    writer<span class="operator">.</span>append(QLatin1String(<span class="string">"condition"</span>));
/// &#32;    writer<span class="operator">.</span>append(QLatin1String(<span class="string">"libs.journald"</span>));
///
/// &#32;    writer<span class="operator">.</span>append(QLatin1String(<span class="string">"output"</span>));
/// &#32;    writer<span class="operator">.</span>startArray(<span class="number">1</span>);
/// &#32;    writer<span class="operator">.</span>append(QLatin1String(<span class="string">"privateFeature"</span>));
/// &#32;    writer<span class="operator">.</span>endArray();
///
/// &#32;    writer<span class="operator">.</span>endMap();
///
/// </pre>
/// <a name="cbor-support"></a></div>
#[repr(C)]
pub struct QCborStreamWriter {
    _unused: u8,
}
impl QCborStreamWriter {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamWriter::append(quint64 u)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the 64-bit unsigned value <i>u</i> to the CBOR stream, creating a CBOR Unsigned Integer value. In the following example, we write the values 0, 2<sup>32</sup> and <code>UINT64_MAX</code>:</p>
    /// <pre class="cpp">
    ///
    /// &#32;    writer<span class="operator">.</span>append(<span class="number">0U</span>);
    /// &#32;    writer<span class="operator">.</span>append(Q_UINT64_C(<span class="number">4294967296</span>));
    /// &#32;    writer<span class="operator">.</span>append(std<span class="operator">::</span>numeric_limits<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#quint64-typedef">quint64</a></span><span class="operator">&gt;</span><span class="operator">::</span>max());
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isUnsignedInteger">QCborStreamReader::isUnsignedInteger</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toUnsignedInteger">QCborStreamReader::toUnsignedInteger</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn append_u64(&mut self, u: u64) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_append(
            self as *mut crate::QCborStreamWriter,
            u,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamWriter::append(qint64 i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the 64-bit signed value <i>i</i> to the CBOR stream. This will create either a CBOR Unsigned Integer or CBOR NegativeInteger value based on the sign of the parameter. In the following example, we write the values 0, -1, 2<sup>32</sup> and <code>INT64_MAX</code>:</p>
    /// <pre class="cpp">
    ///
    /// &#32;    writer<span class="operator">.</span>append(<span class="number">0</span>);
    /// &#32;    writer<span class="operator">.</span>append(<span class="operator">-</span><span class="number">1</span>);
    /// &#32;    writer<span class="operator">.</span>append(Q_INT64_C(<span class="number">4294967296</span>));
    /// &#32;    writer<span class="operator">.</span>append(std<span class="operator">::</span>numeric_limits<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#qint64-typedef">qint64</a></span><span class="operator">&gt;</span><span class="operator">::</span>max());
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isInteger">QCborStreamReader::isInteger</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toInteger">QCborStreamReader::toInteger</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn append_i64(&mut self, i: i64) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_append1(
            self as *mut crate::QCborStreamWriter,
            i,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamWriter::append(QCborNegativeInteger n)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the 64-bit negative value <i>n</i> to the CBOR stream. QCborNegativeInteger is a 64-bit enum that holds the absolute value of the negative number we want to write. If n is zero, the value written will be equivalent to 2<sup>64</sup> (that is, -18,446,744,073,709,551,616).</p>
    /// <p>In the following example, we write the values -1, -2<sup>32</sup> and INT64_MIN:</p>
    /// <pre class="cpp">
    ///
    /// &#32;    writer<span class="operator">.</span>append(<span class="type">QCborNegativeInteger</span>(<span class="number">1</span>));
    /// &#32;    writer<span class="operator">.</span>append(<span class="type">QCborNegativeInteger</span>(Q_INT64_C(<span class="number">4294967296</span>)));
    /// &#32;    writer<span class="operator">.</span>append(<span class="type">QCborNegativeInteger</span>(<span class="operator">-</span><span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#quint64-typedef">quint64</a></span>(std<span class="operator">::</span>numeric_limits<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#qint64-typedef">qint64</a></span><span class="operator">&gt;</span><span class="operator">::</span>min())));
    ///
    /// </pre>
    /// <p>Note how this function can be used to encode numbers that cannot fit a standard computer's 64-bit signed integer like <a href="http://doc.qt.io/qt-5/qtglobal.html#qint64-typedef">qint64</a>. That is, if <i>n</i> is larger than <code>std::numeric_limits&lt;qint64&gt;::max()</code> or is 0, this will represent a negative number smaller than <code>std::numeric_limits&lt;qint64&gt;::min()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isNegativeInteger">QCborStreamReader::isNegativeInteger</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toNegativeInteger">QCborStreamReader::toNegativeInteger</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn append_q_cbor_negative_integer(&mut self, n: crate::QCborNegativeInteger) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_append2(
            self as *mut crate::QCborStreamWriter,
            n,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamWriter::append(const QByteArray& ba)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the byte array <i>ba</i> to the stream, creating a CBOR Byte String value. <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> will attempt to write the entire string in one chunk.</p>
    /// <p>The following example will load and append the contents of a file to the stream:</p>
    /// <pre class="cpp">
    ///
    /// &#32;    <span class="type">void</span> writeFile(<span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#QCborStreamWriter">QCborStreamWriter</a></span> <span class="operator">&amp;</span>writer<span class="operator">,</span> <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>fileName)
    /// &#32;    {
    /// &#32;   &#32;    <span class="type"><a href="http://doc.qt.io/qt-5/qfile.html">QFile</a></span> f(fileName);
    /// &#32;   &#32;    <span class="keyword">if</span> (f<span class="operator">.</span>open(<span class="type"><a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a></span><span class="operator">::</span>ReadOnly))
    /// &#32;   &#32;   &#32;    writer<span class="operator">.</span>append(f<span class="operator">.</span>readAll());
    /// &#32;    }
    ///
    /// </pre>
    /// <p>As the example shows, unlike JSON, CBOR requires no escaping for binary content.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#appendByteString">appendByteString</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isByteArray">QCborStreamReader::isByteArray</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">QCborStreamReader::readByteArray</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn append_q_byte_array(
        &mut self,
        ba: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_append3(
            self as *mut crate::QCborStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(ba)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamWriter::append(QLatin1String str)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the text string <i>str</i> to the stream, creating a CBOR Text String value. <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> will attempt to write the entire string in one chunk.</p>
    /// <p>The following example appends a simple string to the stream:</p>
    /// <pre class="cpp">
    ///
    /// &#32;    writer<span class="operator">.</span>append(QLatin1String(<span class="string">"Hello, World"</span>));
    ///
    /// </pre>
    /// <p><b>Performance note</b>: CBOR requires that all Text Strings be encoded in UTF-8, so this function will iterate over the characters in the string to determine whether the contents are US-ASCII or not. If the string is found to contain characters outside of US-ASCII, it will allocate memory and convert to UTF-8. If this check is unnecessary, use <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#appendTextString">appendTextString</a>() instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isString">QCborStreamReader::isString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">QCborStreamReader::readString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn append_q_latin1_string(
        &mut self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_append4(
            self as *mut crate::QCborStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(str)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamWriter::append(QStringView str)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the text string <i>str</i> to the stream, creating a CBOR Text String value. <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> will attempt to write the entire string in one chunk.</p>
    /// <p>The following example writes an arbitrary <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> to the stream:</p>
    /// <pre class="cpp">
    ///
    /// &#32;    <span class="type">void</span> writeString(<span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#QCborStreamWriter">QCborStreamWriter</a></span> <span class="operator">&amp;</span>writer<span class="operator">,</span> <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>str)
    /// &#32;    {
    /// &#32;   &#32;    writer<span class="operator">.</span>append(str);
    /// &#32;    }
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isString">QCborStreamReader::isString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">QCborStreamReader::readString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn append_q_string_view(
        &mut self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_append5(
            self as *mut crate::QCborStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(str)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamWriter::append(QCborTag tag)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the CBOR tag <i>tag</i> to the stream, creating a CBOR Tag value. All tags must be followed by another type which they provide meaning for.</p>
    /// <p>In the following example, we append a CBOR Tag 36 (Regular Expression) and a <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a>'s pattern to the stream:</p>
    /// <pre class="cpp">
    ///
    /// &#32;    <span class="type">void</span> writeRxPattern(<span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#QCborStreamWriter">QCborStreamWriter</a></span> <span class="operator">&amp;</span>writer<span class="operator">,</span> <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a></span> <span class="operator">&amp;</span>rx)
    /// &#32;    {
    /// &#32;   &#32;    writer<span class="operator">.</span>append(<span class="type"><a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborTag-enum">QCborTag</a></span>(<span class="number">36</span>));
    /// &#32;   &#32;    writer<span class="operator">.</span>append(rx<span class="operator">.</span>pattern());
    /// &#32;    }
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isTag">QCborStreamReader::isTag</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toTag">QCborStreamReader::toTag</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn append_q_cbor_tag(&mut self, tag: crate::QCborTag) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_append6(
            self as *mut crate::QCborStreamWriter,
            tag,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamWriter::append(QCborKnownTags tag)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the CBOR tag <i>tag</i> to the stream, creating a CBOR Tag value. All tags must be followed by another type which they provide meaning for.</p>
    /// <p>In the following example, we append a CBOR Tag 1 (Unix <code>time_t</code>) and an integer representing the current time to the stream, obtained using the <code>time()</code> function:</p>
    /// <pre class="cpp">
    ///
    /// &#32;    <span class="type">void</span> writeCurrentTime(<span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#QCborStreamWriter">QCborStreamWriter</a></span> <span class="operator">&amp;</span>writer)
    /// &#32;    {
    /// &#32;   &#32;    writer<span class="operator">.</span>append(<span class="type"><a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborKnownTags-enum">QCborKnownTags</a></span><span class="operator">::</span>UnixTime_t);
    /// &#32;   &#32;    writer<span class="operator">.</span>append(time(nullptr));
    /// &#32;    }
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isTag">QCborStreamReader::isTag</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toTag">QCborStreamReader::toTag</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn append_q_cbor_known_tags(&mut self, tag: crate::QCborKnownTags) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_append7(
            self as *mut crate::QCborStreamWriter,
            tag,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamWriter::append(QCborSimpleType st)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append-8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the CBOR simple type <i>st</i> to the stream, creating a CBOR Simple Type value. In the following example, we write the simple type for Null as well as for type 32, which Qt has no support for.</p>
    /// <pre class="cpp">
    ///
    /// &#32;   writer<span class="operator">.</span>append(<span class="type"><a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborSimpleType-enum">QCborSimpleType</a></span><span class="operator">::</span>Null);
    /// &#32;   writer<span class="operator">.</span>append(<span class="type"><a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborSimpleType-enum">QCborSimpleType</a></span>(<span class="number">32</span>));
    ///
    /// </pre>
    /// <p><b>Note: </b>Using Simple Types for which there is no specification can lead to validation errors by the remote receiver. In addition, simple type values 24 through 31 (inclusive) are reserved and must not be used.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isSimpleType">QCborStreamReader::isSimpleType</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toSimpleType">QCborStreamReader::toSimpleType</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn append_q_cbor_simple_type(&mut self, st: crate::QCborSimpleType) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_append8(
            self as *mut crate::QCborStreamWriter,
            st,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamWriter::append(qfloat16 f)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append-10">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the floating point number <i>f</i> to the stream, creating a CBOR 16-bit Half-Precision Floating Point value. The following code can be used to convert a C++ <code>float</code> to <code>qfloat16</code> if there's no loss of precision and append it, or instead append the <code>float</code>.</p>
    /// <pre class="cpp">
    ///
    /// &#32;    <span class="type">void</span> writeFloat(<span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#QCborStreamWriter">QCborStreamWriter</a></span> <span class="operator">&amp;</span>writer<span class="operator">,</span> <span class="type">float</span> f)
    /// &#32;    {
    /// &#32;   &#32;    qfloat16 f16 <span class="operator">=</span> f;
    /// &#32;   &#32;    <span class="keyword">if</span> (qIsNaN(f) <span class="operator">|</span><span class="operator">|</span> f16 <span class="operator">=</span><span class="operator">=</span> f)
    /// &#32;   &#32;   &#32;    writer<span class="operator">.</span>append(f16);
    /// &#32;   &#32;    <span class="keyword">else</span>
    /// &#32;   &#32;   &#32;    writer<span class="operator">.</span>append(f);
    /// &#32;    }
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isFloat16">QCborStreamReader::isFloat16</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toFloat16">QCborStreamReader::toFloat16</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn append_qfloat16(
        &mut self,
        f: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::Qfloat16>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_append9(
            self as *mut crate::QCborStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::Qfloat16>>::cast_into(f).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamWriter::append(float f)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append-11">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the floating point number <i>f</i> to the stream, creating a CBOR 32-bit Single-Precision Floating Point value. The following code can be used to convert a C++ <code>double</code> to <code>float</code> if there's no loss of precision and append it, or instead append the <code>double</code>.</p>
    /// <pre class="cpp">
    ///
    /// &#32;    <span class="type">void</span> writeFloat(<span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#QCborStreamWriter">QCborStreamWriter</a></span> <span class="operator">&amp;</span>writer<span class="operator">,</span> <span class="type">double</span> d)
    /// &#32;    {
    /// &#32;   &#32;    <span class="type">float</span> f <span class="operator">=</span> d;
    /// &#32;   &#32;    <span class="keyword">if</span> (qIsNaN(d) <span class="operator">|</span><span class="operator">|</span> d <span class="operator">=</span><span class="operator">=</span> f)
    /// &#32;   &#32;   &#32;    writer<span class="operator">.</span>append(f);
    /// &#32;   &#32;    <span class="keyword">else</span>
    /// &#32;   &#32;   &#32;    writer<span class="operator">.</span>append(d);
    /// &#32;    }
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isFloat">QCborStreamReader::isFloat</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toFloat">QCborStreamReader::toFloat</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn append_float(&mut self, f: ::std::os::raw::c_float) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_append10(
            self as *mut crate::QCborStreamWriter,
            f,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamWriter::append(double d)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append-12">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the floating point number <i>d</i> to the stream, creating a CBOR 64-bit Double-Precision Floating Point value. <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> always appends the number as-is, performing no check for whether the number is the canonical form for NaN, an infinite, whether it is denormal or if it could be written with a shorter format.</p>
    /// <p>The following code performs all those checks, except for the denormal one, which is expected to be taken into account by the system FPU or floating point emulation directly.</p>
    /// <pre class="cpp">
    ///
    /// &#32;    <span class="type">void</span> writeDouble(<span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#QCborStreamWriter">QCborStreamWriter</a></span> <span class="operator">&amp;</span>writer<span class="operator">,</span> <span class="type">double</span> d)
    /// &#32;    {
    /// &#32;   &#32;    <span class="type">float</span> f;
    /// &#32;   &#32;    <span class="keyword">if</span> (qIsNaN(d)) {
    /// &#32;   &#32;   &#32;    writer<span class="operator">.</span>append(qfloat16(qQNaN()));
    /// &#32;   &#32;    } <span class="keyword">else</span> <span class="keyword">if</span> (<a href="http://doc.qt.io/qt-5/qtglobal.html#qIsInf">qIsInf</a>(d)) {
    /// &#32;   &#32;   &#32;    writer<span class="operator">.</span>append(d <span class="operator">&lt;</span> <span class="number">0</span> <span class="operator">?</span> <span class="operator">-</span><a href="http://doc.qt.io/qt-5/qtglobal.html#qInf">qInf</a>() : <a href="http://doc.qt.io/qt-5/qtglobal.html#qInf">qInf</a>());
    /// &#32;   &#32;    } <span class="keyword">else</span> <span class="keyword">if</span> ((f <span class="operator">=</span> d) <span class="operator">=</span><span class="operator">=</span> d) {
    /// &#32;   &#32;   &#32;    qfloat16 f16 <span class="operator">=</span> f;
    /// &#32;   &#32;   &#32;    <span class="keyword">if</span> (f16 <span class="operator">=</span><span class="operator">=</span> f)
    /// &#32;   &#32;   &#32;   &#32;    writer<span class="operator">.</span>append(f16);
    /// &#32;   &#32;   &#32;    <span class="keyword">else</span>
    /// &#32;   &#32;   &#32;   &#32;    writer<span class="operator">.</span>append(f);
    /// &#32;   &#32;    } <span class="keyword">else</span> {
    /// &#32;   &#32;   &#32;    writer<span class="operator">.</span>append(d);
    /// &#32;   &#32;    }
    /// &#32;    }
    ///
    /// </pre>
    /// <p>Determining if a double can be converted to an integral with no loss of precision is left as an exercise to the reader.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isDouble">QCborStreamReader::isDouble</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toDouble">QCborStreamReader::toDouble</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn append_double(&mut self, d: ::std::os::raw::c_double) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_append11(
            self as *mut crate::QCborStreamWriter,
            d,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamWriter::append(bool b)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append-13">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the boolean value <i>b</i> to the stream, creating either a CBOR False value or a CBOR True value. This function is equivalent to (and implemented as):</p>
    /// <pre class="cpp">
    ///
    /// &#32;    writer<span class="operator">.</span>append(b <span class="operator">?</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborSimpleType-enum">QCborSimpleType</a></span><span class="operator">::</span>True : <span class="type"><a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborSimpleType-enum">QCborSimpleType</a></span><span class="operator">::</span>False);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#appendNull">appendNull</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#appendUndefined">appendUndefined</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isBool">QCborStreamReader::isBool</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toBool">QCborStreamReader::toBool</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn append_bool(&mut self, b: bool) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_append12(
            self as *mut crate::QCborStreamWriter,
            b,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QCborStreamWriter::append(int i)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn append_int(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_append13(
            self as *mut crate::QCborStreamWriter,
            i,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QCborStreamWriter::append(unsigned int u)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn append_uint(&mut self, u: ::std::os::raw::c_uint) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_append14(
            self as *mut crate::QCborStreamWriter,
            u,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamWriter::append(const char* str, long long size = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append-14">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>size</i> bytes of text starting from <i>str</i> to the stream, creating a CBOR Text String value. <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> will attempt to write the entire string in one chunk. If <i>size</i> is -1, this function will write <code>strlen(\a str)</code> bytes.</p>
    /// <p>The string pointed to by <i>str</i> is expected to be properly encoded UTF-8. <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> performs no validation that this is the case.</p>
    /// <p>Unlike the <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> overload of <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append">append</a>(), this function is not limited to 2 GB. However, note that neither <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> nor <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> support reading CBOR streams with text strings larger than 2 GB.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append-4">append</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append-5">append</a>(QStringView), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isString">QCborStreamReader::isString</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">QCborStreamReader::readString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn append_char_longlong(
        &mut self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        size: ::std::os::raw::c_longlong,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_append15(
            self as *mut crate::QCborStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(str)
                .as_raw_ptr(),
            size,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamWriter::append(const char* str)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append-14">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>size</i> bytes of text starting from <i>str</i> to the stream, creating a CBOR Text String value. <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> will attempt to write the entire string in one chunk. If <i>size</i> is -1, this function will write <code>strlen(\a str)</code> bytes.</p>
    /// <p>The string pointed to by <i>str</i> is expected to be properly encoded UTF-8. <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> performs no validation that this is the case.</p>
    /// <p>Unlike the <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> overload of <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append">append</a>(), this function is not limited to 2 GB. However, note that neither <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> nor <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> support reading CBOR streams with text strings larger than 2 GB.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append-4">append</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append-5">append</a>(QStringView), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isString">QCborStreamReader::isString</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">QCborStreamReader::readString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn append_char(
        &mut self,
        str: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_append16(
            self as *mut crate::QCborStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(str)
                .as_raw_ptr(),
        )
    }

    /// <p>Appends <i>len</i> bytes of data starting from <i>data</i> to the stream, creating a CBOR Byte String value. <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> will attempt to write the entire string in one chunk.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamWriter::appendByteString(const char* data, long long len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#appendByteString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>len</i> bytes of data starting from <i>data</i> to the stream, creating a CBOR Byte String value. <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> will attempt to write the entire string in one chunk.</p>
    /// <p>Unlike the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> overload of <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append">append</a>(), this function is not limited by <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>'s size limits. However, note that neither <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">QCborStreamReader::readByteArray</a>() nor <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> support reading CBOR streams with byte arrays larger than 2 GB.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#appendTextString">appendTextString</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isByteArray">QCborStreamReader::isByteArray</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">QCborStreamReader::readByteArray</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn append_byte_string(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        len: ::std::os::raw::c_longlong,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_appendByteString(
            self as *mut crate::QCborStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(data)
                .as_raw_ptr(),
            len,
        )
    }

    /// <p>Appends a CBOR Null value to the stream. This function is equivalent to (and implemented as):</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamWriter::appendNull()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#appendNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends a CBOR Null value to the stream. This function is equivalent to (and implemented as):</p>
    /// <pre class="cpp">
    ///
    /// &#32;    writer<span class="operator">.</span>append(<span class="type"><a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborSimpleType-enum">QCborSimpleType</a></span><span class="operator">::</span>Null);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append-9">append</a>(std::nullptr_t), <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append-8">append</a>(QCborSimpleType), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isNull">QCborStreamReader::isNull</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn append_null(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_appendNull(
            self as *mut crate::QCborStreamWriter,
        )
    }

    /// <p>Appends <i>len</i> bytes of text starting from <i>utf8</i> to the stream, creating a CBOR Text String value. <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> will attempt to write the entire string in one chunk.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamWriter::appendTextString(const char* utf8, long long len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#appendTextString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>len</i> bytes of text starting from <i>utf8</i> to the stream, creating a CBOR Text String value. <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> will attempt to write the entire string in one chunk.</p>
    /// <p>The string pointed to by <i>utf8</i> is expected to be properly encoded UTF-8. <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> performs no validation that this is the case.</p>
    /// <p>Unlike the <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> overload of <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append">append</a>(), this function is not limited to 2 GB. However, note that neither <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">QCborStreamReader::readString</a>() nor <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> support reading CBOR streams with text strings larger than 2 GB.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append-4">append</a>(QLatin1String), <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append-5">append</a>(QStringView), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isString">QCborStreamReader::isString</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">QCborStreamReader::readString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn append_text_string(
        &mut self,
        utf8: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        len: ::std::os::raw::c_longlong,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_appendTextString(
            self as *mut crate::QCborStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(utf8)
                .as_raw_ptr(),
            len,
        )
    }

    /// <p>Appends a CBOR Undefined value to the stream. This function is equivalent to (and implemented as):</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamWriter::appendUndefined()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#appendUndefined">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends a CBOR Undefined value to the stream. This function is equivalent to (and implemented as):</p>
    /// <pre class="cpp">
    ///
    /// &#32;    writer<span class="operator">.</span>append(<span class="type"><a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborSimpleType-enum">QCborSimpleType</a></span><span class="operator">::</span>Undefined);
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append-8">append</a>(QCborSimpleType) and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isUndefined">QCborStreamReader::isUndefined</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn append_undefined(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_appendUndefined(
            self as *mut crate::QCborStreamWriter,
        )
    }

    /// <p>Returns the <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> that this <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> object is writing to. The device must have previously been set with either the constructor or with <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#setDevice">setDevice</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QIODevice* QCborStreamWriter::device() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#device">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> that this <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> object is writing to. The device must have previously been set with either the constructor or with <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#setDevice">setDevice</a>().</p>
    /// <p>If this object was created by writing to a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>, this function will return an internal instance of <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a>, which is owned by <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#setDevice">setDevice</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn device(&self) -> ::cpp_utils::MutPtr<crate::QIODevice> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_device(
            self as *const crate::QCborStreamWriter,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Terminates the array started by either overload of <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startArray">startArray</a>() and returns true if the correct number of elements was added to the array. This function must be called for every <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startArray">startArray</a>() used.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamWriter::endArray()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#endArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Terminates the array started by either overload of <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startArray">startArray</a>() and returns true if the correct number of elements was added to the array. This function must be called for every <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startArray">startArray</a>() used.</p>
    /// <p>A return of false indicates error in the application and an unrecoverable error in this stream. <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> also writes a warning using <a href="http://doc.qt.io/qt-5/qtglobal.html#qWarning">qWarning</a>() if that happens.</p>
    /// <p>Calling this function when the current container is not an array is also an error, though <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> cannot currently detect this condition.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startArray">startArray</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startArray-1">startArray</a>(quint64), and <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#endMap">endMap</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn end_array(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_endArray(
            self as *mut crate::QCborStreamWriter,
        )
    }

    /// <p>Terminates the map started by either overload of <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startMap">startMap</a>() and returns true if the correct number of elements was added to the array. This function must be called for every <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startMap">startMap</a>() used.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamWriter::endMap()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#endMap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Terminates the map started by either overload of <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startMap">startMap</a>() and returns true if the correct number of elements was added to the array. This function must be called for every <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startMap">startMap</a>() used.</p>
    /// <p>A return of false indicates error in the application and an unrecoverable error in this stream. <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> also writes a warning using <a href="http://doc.qt.io/qt-5/qtglobal.html#qWarning">qWarning</a>() if that happens.</p>
    /// <p>Calling this function when the current container is not a map is also an error, though <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> cannot currently detect this condition.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startMap">startMap</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startMap-1">startMap</a>(quint64), and <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#endArray">endArray</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn end_map(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_endMap(
            self as *mut crate::QCborStreamWriter,
        )
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> object that will write the stream to <i>device</i>. The device must be opened before the first <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append">append</a>() call is made. This constructor can be used with any class that derives from <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>, such as <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a>, <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> or <a href="http://doc.qt.io/qt-5/qtcpsocket.html">QTcpSocket</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborStreamWriter::QCborStreamWriter(QIODevice* device)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#QCborStreamWriter">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> object that will write the stream to <i>device</i>. The device must be opened before the first <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append">append</a>() call is made. This constructor can be used with any class that derives from <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>, such as <a href="http://doc.qt.io/qt-5/qfile.html">QFile</a>, <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> or <a href="http://doc.qt.io/qt-5/qtcpsocket.html">QTcpSocket</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> has no buffering, so every <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#append">append</a>() call will result in one or more calls to the device's <a href="http://doc.qt.io/qt-5/qiodevice.html#write">write()</a> method.</p>
    /// <p>The following example writes an empty map to a file:</p>
    /// <pre class="cpp">
    ///
    /// &#32;    <span class="type"><a href="http://doc.qt.io/qt-5/qfile.html">QFile</a></span> f(<span class="string">"output"</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a></span><span class="operator">::</span>WriteOnly);
    /// &#32;    <span class="type">QCborStreamWriter</span> writer(<span class="operator">&amp;</span>f);
    /// &#32;    writer<span class="operator">.</span>startMap(<span class="number">0</span>);
    /// &#32;    writer<span class="operator">.</span>endMap();
    ///
    /// </pre>
    /// <p><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> does not take ownership of <i>device</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#device">device</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#setDevice">setDevice</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_q_io_device(
        device: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QIODevice>>,
    ) -> ::cpp_utils::CppBox<crate::QCborStreamWriter> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_QCborStreamWriter(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QIODevice>>::cast_into(device)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> object that will append the stream to <i>data</i>. All streaming is done immediately to the byte array, without the need for flushing any buffers.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborStreamWriter::QCborStreamWriter(QByteArray* data)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#QCborStreamWriter-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> object that will append the stream to <i>data</i>. All streaming is done immediately to the byte array, without the need for flushing any buffers.</p>
    /// <p>The following example writes a number to a byte array then returns it.</p>
    /// <pre class="cpp">
    ///
    /// &#32;    <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span> encodedNumber(<span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#qint64-typedef">qint64</a></span> value)
    /// &#32;    {
    /// &#32;   &#32;    <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span> ba;
    /// &#32;   &#32;    <span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#QCborStreamWriter">QCborStreamWriter</a></span> writer(<span class="operator">&amp;</span>ba);
    /// &#32;   &#32;    writer<span class="operator">.</span>append(value);
    /// &#32;   &#32;    <span class="keyword">return</span> ba;
    /// &#32;    }
    ///
    /// </pre>
    /// <p><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> does not take ownership of <i>data</i>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_q_byte_array(
        data: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QCborStreamWriter> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_QCborStreamWriter1(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QByteArray>>::cast_into(data)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Replaces the device or byte array that this <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> object is writing to with <i>device</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamWriter::setDevice(QIODevice* device)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#setDevice">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the device or byte array that this <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> object is writing to with <i>device</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#device">device</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn set_device(
        &mut self,
        device: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QIODevice>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_setDevice(
            self as *mut crate::QCborStreamWriter,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QIODevice>>::cast_into(device)
                .as_mut_raw_ptr(),
        )
    }

    /// <p>Starts a CBOR Array with indeterminate length in the CBOR stream. Each startArray() call must be paired with one <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#endArray">endArray</a>() call and the current CBOR element extends until the end of the array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamWriter::startArray()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Starts a CBOR Array with indeterminate length in the CBOR stream. Each startArray() call must be paired with one <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#endArray">endArray</a>() call and the current CBOR element extends until the end of the array.</p>
    /// <p>The array created by this function has no explicit length. Instead, its length is implied by the elements contained in it. Note, however, that use of indeterminate-length arrays is not compliant with canonical CBOR encoding.</p>
    /// <p>The following example appends elements from the linked list of strings passed as input:</p>
    /// <pre class="cpp">
    ///
    /// &#32;    <span class="type">void</span> appendList(<span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#QCborStreamWriter">QCborStreamWriter</a></span> <span class="operator">&amp;</span>writer<span class="operator">,</span> <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> <span class="operator">&amp;</span>list)
    /// &#32;    {
    /// &#32;   &#32;    writer<span class="operator">.</span>startArray();
    /// &#32;   &#32;    <span class="keyword">for</span> (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>s : list)
    /// &#32;   &#32;   &#32;    writer<span class="operator">.</span>append(s);
    /// &#32;   &#32;    writer<span class="operator">.</span>endArray();
    /// &#32;    }
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startArray-1">startArray</a>(quint64), <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#endArray">endArray</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startMap">startMap</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isArray">QCborStreamReader::isArray</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isLengthKnown">QCborStreamReader::isLengthKnown</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn start_array_0a(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_startArray(
            self as *mut crate::QCborStreamWriter,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamWriter::startArray(quint64 count)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startArray-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Starts a CBOR Array with explicit length of <i>count</i> items in the CBOR stream. Each <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startArray">startArray</a> call must be paired with one <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#endArray">endArray</a>() call and the current CBOR element extends until the end of the array.</p>
    /// <p>The array created by this function has an explicit length and therefore exactly <i>count</i> items must be added to the CBOR stream. Adding fewer or more items will result in failure during <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#endArray">endArray</a>() and the CBOR stream will be corrupt. However, explicit-length arrays are required by canonical CBOR encoding.</p>
    /// <p>The following example appends all strings found in the <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> passed as input:</p>
    /// <pre class="cpp">
    ///
    /// &#32;    <span class="type">void</span> appendList(<span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#QCborStreamWriter">QCborStreamWriter</a></span> <span class="operator">&amp;</span>writer<span class="operator">,</span> <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> <span class="operator">&amp;</span>list)
    /// &#32;    {
    /// &#32;   &#32;    writer<span class="operator">.</span>startArray(list<span class="operator">.</span>size());
    /// &#32;   &#32;    <span class="keyword">for</span> (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>s : list)
    /// &#32;   &#32;   &#32;    writer<span class="operator">.</span>append(s);
    /// &#32;   &#32;    writer<span class="operator">.</span>endArray();
    /// &#32;    }
    ///
    /// </pre>
    /// <p><b>Size limitations</b>: The parameter to this function is quint64, which would seem to allow up to 2<sup>64</sup>-1 elements in the array. However, both <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> are currently limited to 2<sup>32</sup>-2 items on 32-bit systems and 2<sup>64</sup>-2 items on 64-bit ones. Also note that <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> is currently limited to 2<sup>27</sup> elements in any platform.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startArray">startArray</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#endArray">endArray</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startMap">startMap</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isArray">QCborStreamReader::isArray</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isLengthKnown">QCborStreamReader::isLengthKnown</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn start_array_1a(&mut self, count: u64) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_startArray1(
            self as *mut crate::QCborStreamWriter,
            count,
        )
    }

    /// <p>Starts a CBOR Map with indeterminate length in the CBOR stream. Each startMap() call must be paired with one <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#endMap">endMap</a>() call and the current CBOR element extends until the end of the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamWriter::startMap()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startMap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Starts a CBOR Map with indeterminate length in the CBOR stream. Each startMap() call must be paired with one <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#endMap">endMap</a>() call and the current CBOR element extends until the end of the map.</p>
    /// <p>The map created by this function has no explicit length. Instead, its length is implied by the elements contained in it. Note, however, that use of indeterminate-length maps is not compliant with canonical CBOR encoding (canonical encoding also requires keys to be unique and in sorted order).</p>
    /// <p>The following example appends elements from the linked list of int and string pairs passed as input:</p>
    /// <pre class="cpp">
    ///
    /// &#32;    <span class="type">void</span> appendMap(<span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#QCborStreamWriter">QCborStreamWriter</a></span> <span class="operator">&amp;</span>writer<span class="operator">,</span> <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">&gt;</span> <span class="operator">&amp;</span>list)
    /// &#32;    {
    /// &#32;   &#32;    writer<span class="operator">.</span>startMap();
    /// &#32;   &#32;    <span class="keyword">for</span> (<span class="keyword">const</span> <span class="keyword">auto</span> pair : list) {
    /// &#32;   &#32;   &#32;    writer<span class="operator">.</span>append(pair<span class="operator">.</span>first)
    /// &#32;   &#32;   &#32;    writer<span class="operator">.</span>append(pair<span class="operator">.</span>second);
    /// &#32;   &#32;    }
    /// &#32;   &#32;    writer<span class="operator">.</span>endMap();
    /// &#32;    }
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startMap-1">startMap</a>(quint64), <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#endMap">endMap</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startArray">startArray</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isMap">QCborStreamReader::isMap</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isLengthKnown">QCborStreamReader::isLengthKnown</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn start_map_0a(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_startMap(
            self as *mut crate::QCborStreamWriter,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamWriter::startMap(quint64 count)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startMap-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Starts a CBOR Map with explicit length of <i>count</i> items in the CBOR stream. Each <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startMap">startMap</a> call must be paired with one <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#endMap">endMap</a>() call and the current CBOR element extends until the end of the map.</p>
    /// <p>The map created by this function has an explicit length and therefore exactly <i>count</i> pairs of items must be added to the CBOR stream. Adding fewer or more items will result in failure during <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#endMap">endMap</a>() and the CBOR stream will be corrupt. However, explicit-length map are required by canonical CBOR encoding.</p>
    /// <p>The following example appends all strings found in the <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a> passed as input:</p>
    /// <pre class="cpp">
    ///
    /// &#32;    <span class="type">void</span> appendMap(<span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#QCborStreamWriter">QCborStreamWriter</a></span> <span class="operator">&amp;</span>writer<span class="operator">,</span> <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qmap.html">QMap</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> <span class="operator">&amp;</span>map)
    /// &#32;    {
    /// &#32;   &#32;    writer<span class="operator">.</span>startMap(map<span class="operator">.</span>size());
    /// &#32;   &#32;    <span class="keyword">for</span> (<span class="keyword">auto</span> it <span class="operator">=</span> map<span class="operator">.</span>begin(); it <span class="operator">!</span><span class="operator">=</span> map<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>it) {
    /// &#32;   &#32;   &#32;    writer<span class="operator">.</span>append(it<span class="operator">.</span>key());
    /// &#32;   &#32;   &#32;    writer<span class="operator">.</span>append(it<span class="operator">.</span>value());
    /// &#32;   &#32;    }
    /// &#32;   &#32;    writer<span class="operator">.</span>endMap();
    /// &#32;    }
    ///
    /// </pre>
    /// <p><b>Size limitations</b>: The parameter to this function is quint64, which would seem to allow up to 2<sup>64</sup>-1 pairs in the map. However, both <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> are currently limited to 2<sup>31</sup>-1 items on 32-bit systems and 2<sup>63</sup>-1 items on 64-bit ones. Also note that <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> is currently limited to 2<sup>26</sup> elements in any platform.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startMap">startMap</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#endMap">endMap</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startArray">startArray</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isMap">QCborStreamReader::isMap</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isLengthKnown">QCborStreamReader::isLengthKnown</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn start_map_1a(&mut self, count: u64) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_startMap1(
            self as *mut crate::QCborStreamWriter,
            count,
        )
    }
}

pub mod q_cbor_stream_reader {
    //! C++ type: <span style='color: green;'>```QCborStreamReader```</span>

    /// <p>This enumeration contains all possible CBOR types as decoded by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a>. CBOR has 7 major types, plus a number of simple types carrying no value, and floating point values.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QCborStreamReader::Type```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enumeration contains all possible CBOR types as decoded by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a>. CBOR has 7 major types, plus a number of simple types carrying no value, and floating point values.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct Type(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for Type {
        fn from(value: ::std::os::raw::c_int) -> Self {
            Type(value)
        }
    }

    impl From<Type> for ::std::os::raw::c_int {
        fn from(value: Type) -> Self {
            value.0
        }
    }

    impl Type {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl Type {
        /// (Major type 0) Ranges from 0 to 2<sup>64</sup> - 1 (18,446,744,073,709,551,616) (C++ enum variant: <span style='color: green;'>```UnsignedInteger = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const UnsignedInteger: crate::q_cbor_stream_reader::Type =
            crate::q_cbor_stream_reader::Type(0);
        /// (Major type 1) Ranges from -1 to -2<sup>64</sup> (-18,446,744,073,709,551,616) (C++ enum variant: <span style='color: green;'>```NegativeInteger = 32```</span>)
        #[allow(non_upper_case_globals)]
        pub const NegativeInteger: crate::q_cbor_stream_reader::Type =
            crate::q_cbor_stream_reader::Type(32);
        /// An alias to ByteArray. (C++ enum variant: <span style='color: green;'>```ByteString = 64```</span>)
        #[allow(non_upper_case_globals)]
        pub const ByteString: crate::q_cbor_stream_reader::Type =
            crate::q_cbor_stream_reader::Type(64);
        /// (Major type 2) Arbitrary binary data. (C++ enum variant: <span style='color: green;'>```ByteArray = 64```</span>)
        #[allow(non_upper_case_globals)]
        pub const ByteArray: crate::q_cbor_stream_reader::Type =
            crate::q_cbor_stream_reader::Type(64);
        /// An alias to String (C++ enum variant: <span style='color: green;'>```TextString = 96```</span>)
        #[allow(non_upper_case_globals)]
        pub const TextString: crate::q_cbor_stream_reader::Type =
            crate::q_cbor_stream_reader::Type(96);
        /// (Major type 3) Unicode text, possibly containing NULs. (C++ enum variant: <span style='color: green;'>```String = 96```</span>)
        #[allow(non_upper_case_globals)]
        pub const String: crate::q_cbor_stream_reader::Type = crate::q_cbor_stream_reader::Type(96);
        /// (Major type 4) Array of heterogeneous items. (C++ enum variant: <span style='color: green;'>```Array = -128```</span>)
        #[allow(non_upper_case_globals)]
        pub const Array: crate::q_cbor_stream_reader::Type =
            crate::q_cbor_stream_reader::Type(-128);
        /// (Major type 5) Map/dictionary of heterogeneous items. (C++ enum variant: <span style='color: green;'>```Map = -96```</span>)
        #[allow(non_upper_case_globals)]
        pub const Map: crate::q_cbor_stream_reader::Type = crate::q_cbor_stream_reader::Type(-96);
        /// (Major type 6) Numbers giving further semantic value to generic CBOR items. See <a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborTag-enum">QCborTag</a> for more information. (C++ enum variant: <span style='color: green;'>```Tag = -64```</span>)
        #[allow(non_upper_case_globals)]
        pub const Tag: crate::q_cbor_stream_reader::Type = crate::q_cbor_stream_reader::Type(-64);
        /// (Major type 7) Types carrying no further value. Includes booleans (true and false), null, undefined. (C++ enum variant: <span style='color: green;'>```SimpleType = -32```</span>)
        #[allow(non_upper_case_globals)]
        pub const SimpleType: crate::q_cbor_stream_reader::Type =
            crate::q_cbor_stream_reader::Type(-32);
        /// An alias to Float16. (C++ enum variant: <span style='color: green;'>```HalfFloat = -7```</span>)
        #[allow(non_upper_case_globals)]
        pub const HalfFloat: crate::q_cbor_stream_reader::Type =
            crate::q_cbor_stream_reader::Type(-7);
        /// IEEE 754 half-precision floating point (<code>qfloat16</code>). (C++ enum variant: <span style='color: green;'>```Float16 = -7```</span>)
        #[allow(non_upper_case_globals)]
        pub const Float16: crate::q_cbor_stream_reader::Type =
            crate::q_cbor_stream_reader::Type(-7);
        /// IEEE 754 single-precision floating point (<code>float</code>). (C++ enum variant: <span style='color: green;'>```Float = -6```</span>)
        #[allow(non_upper_case_globals)]
        pub const Float: crate::q_cbor_stream_reader::Type = crate::q_cbor_stream_reader::Type(-6);
        /// IEEE 754 double-precision floating point (<code>double</code>). (C++ enum variant: <span style='color: green;'>```Double = -5```</span>)
        #[allow(non_upper_case_globals)]
        pub const Double: crate::q_cbor_stream_reader::Type = crate::q_cbor_stream_reader::Type(-5);
        /// Not a valid type, either due to parsing error or due to reaching the end of an array or map. (C++ enum variant: <span style='color: green;'>```Invalid = -1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Invalid: crate::q_cbor_stream_reader::Type =
            crate::q_cbor_stream_reader::Type(-1);
    }

    /// <p>This enum is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>() and is used to indicate what the status of the parsing is.</p>
    ///
    /// C++ enum: <span style='color: green;'>```QCborStreamReader::StringResultCode```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#StringResultCode-enum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This enum is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>() and is used to indicate what the status of the parsing is.</p></div>
    #[derive(Debug, Clone, Copy, PartialEq, Eq)]
    #[repr(transparent)]
    pub struct StringResultCode(::std::os::raw::c_int);

    impl From<::std::os::raw::c_int> for StringResultCode {
        fn from(value: ::std::os::raw::c_int) -> Self {
            StringResultCode(value)
        }
    }

    impl From<StringResultCode> for ::std::os::raw::c_int {
        fn from(value: StringResultCode) -> Self {
            value.0
        }
    }

    impl StringResultCode {
        pub fn to_int(&self) -> ::std::os::raw::c_int {
            self.0
        }
    }

    impl StringResultCode {
        /// The parsing for the string is complete, with no error. (C++ enum variant: <span style='color: green;'>```EndOfString = 0```</span>)
        #[allow(non_upper_case_globals)]
        pub const EndOfString: crate::q_cbor_stream_reader::StringResultCode =
            crate::q_cbor_stream_reader::StringResultCode(0);
        /// The function returned data; there was no error. (C++ enum variant: <span style='color: green;'>```Ok = 1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Ok: crate::q_cbor_stream_reader::StringResultCode =
            crate::q_cbor_stream_reader::StringResultCode(1);
        /// Parsing failed with an error. (C++ enum variant: <span style='color: green;'>```Error = -1```</span>)
        #[allow(non_upper_case_globals)]
        pub const Error: crate::q_cbor_stream_reader::StringResultCode =
            crate::q_cbor_stream_reader::StringResultCode(-1);
    }

    /// <p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
    ///
    /// C++ class: <span style='color: green;'>```QCborStreamReader::StringResult<QString>```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
    /// <p>The contents of <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#data-var">data</a> are valid only if <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#status-var">status</a> is <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#StringResultCode-enum">Ok</a>. Otherwise, it should be null.</p></div>
    #[repr(C)]
    pub struct StringResultOfQString {
        _unused: u8,
    }
    impl StringResultOfQString {
        /// <p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborStreamReader::StringResult<QString>& QCborStreamReader::StringResult<QString>::operator=(const QCborStreamReader::StringResult<QString>& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
        /// <p>The contents of <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#data-var">data</a> are valid only if <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#status-var">status</a> is <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#StringResultCode-enum">Ok</a>. Otherwise, it should be null.</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_cbor_stream_reader::StringResultOfQString>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_cbor_stream_reader::StringResultOfQString> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_StringResult_QString_operator_(
                    self as *mut crate::q_cbor_stream_reader::StringResultOfQString,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_cbor_stream_reader::StringResultOfQString>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborStreamReader::StringResult<QString>::StringResult()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
        /// <p>The contents of <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#data-var">data</a> are valid only if <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#status-var">status</a> is <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#StringResultCode-enum">Ok</a>. Otherwise, it should be null.</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn new(
        ) -> ::cpp_utils::CppBox<crate::q_cbor_stream_reader::StringResultOfQString> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_StringResult_QString_StringResult();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborStreamReader::StringResult<QString>::StringResult(const QCborStreamReader::StringResult<QString>& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
        /// <p>The contents of <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#data-var">data</a> are valid only if <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#status-var">status</a> is <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#StringResultCode-enum">Ok</a>. Otherwise, it should be null.</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn new_copy(
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_cbor_stream_reader::StringResultOfQString>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_cbor_stream_reader::StringResultOfQString> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_StringResult_QString_StringResult1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_cbor_stream_reader::StringResultOfQString>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }

    /// <p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
    ///
    /// C++ class: <span style='color: green;'>```QCborStreamReader::StringResult<QByteArray>```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
    /// <p>The contents of <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#data-var">data</a> are valid only if <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#status-var">status</a> is <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#StringResultCode-enum">Ok</a>. Otherwise, it should be null.</p></div>
    #[repr(C)]
    pub struct StringResultOfQByteArray {
        _unused: u8,
    }
    impl StringResultOfQByteArray {
        /// <p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborStreamReader::StringResult<QByteArray>& QCborStreamReader::StringResult<QByteArray>::operator=(const QCborStreamReader::StringResult<QByteArray>& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
        /// <p>The contents of <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#data-var">data</a> are valid only if <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#status-var">status</a> is <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#StringResultCode-enum">Ok</a>. Otherwise, it should be null.</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_cbor_stream_reader::StringResultOfQByteArray>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_cbor_stream_reader::StringResultOfQByteArray> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_StringResult_QByteArray_operator_(
                    self as *mut crate::q_cbor_stream_reader::StringResultOfQByteArray,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_cbor_stream_reader::StringResultOfQByteArray>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborStreamReader::StringResult<QByteArray>::StringResult()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
        /// <p>The contents of <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#data-var">data</a> are valid only if <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#status-var">status</a> is <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#StringResultCode-enum">Ok</a>. Otherwise, it should be null.</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn new(
        ) -> ::cpp_utils::CppBox<crate::q_cbor_stream_reader::StringResultOfQByteArray> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_StringResult_QByteArray_StringResult();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborStreamReader::StringResult<QByteArray>::StringResult(const QCborStreamReader::StringResult<QByteArray>& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
        /// <p>The contents of <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#data-var">data</a> are valid only if <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#status-var">status</a> is <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#StringResultCode-enum">Ok</a>. Otherwise, it should be null.</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn new_copy(
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_cbor_stream_reader::StringResultOfQByteArray>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_cbor_stream_reader::StringResultOfQByteArray> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_StringResult_QByteArray_StringResult1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_cbor_stream_reader::StringResultOfQByteArray>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }

    /// <p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
    ///
    /// C++ class: <span style='color: green;'>```QCborStreamReader::StringResult<long long>```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
    /// <p>The contents of <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#data-var">data</a> are valid only if <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#status-var">status</a> is <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#StringResultCode-enum">Ok</a>. Otherwise, it should be null.</p></div>
    #[repr(C)]
    pub struct StringResultOfLonglong {
        _unused: u8,
    }
    impl StringResultOfLonglong {
        /// <p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QCborStreamReader::StringResult<long long>& QCborStreamReader::StringResult<long long>::operator=(const QCborStreamReader::StringResult<long long>& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
        /// <p>The contents of <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#data-var">data</a> are valid only if <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#status-var">status</a> is <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#StringResultCode-enum">Ok</a>. Otherwise, it should be null.</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_cbor_stream_reader::StringResultOfLonglong>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_cbor_stream_reader::StringResultOfLonglong> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_StringResult_long_long_operator_(
                    self as *mut crate::q_cbor_stream_reader::StringResultOfLonglong,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_cbor_stream_reader::StringResultOfLonglong>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborStreamReader::StringResult<long long>::StringResult()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
        /// <p>The contents of <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#data-var">data</a> are valid only if <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#status-var">status</a> is <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#StringResultCode-enum">Ok</a>. Otherwise, it should be null.</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn new(
        ) -> ::cpp_utils::CppBox<crate::q_cbor_stream_reader::StringResultOfLonglong> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_StringResult_long_long_StringResult(
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborStreamReader::StringResult<long long>::StringResult(const QCborStreamReader::StringResult<long long>& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
        /// <p>The contents of <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#data-var">data</a> are valid only if <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#status-var">status</a> is <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#StringResultCode-enum">Ok</a>. Otherwise, it should be null.</p></div>
        ///
        /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
        #[cfg(any(
            any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
            ritual_rustdoc
        ))]
        pub unsafe fn new_copy(
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_cbor_stream_reader::StringResultOfLonglong>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_cbor_stream_reader::StringResultOfLonglong> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_StringResult_long_long_StringResult1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_cbor_stream_reader::StringResultOfLonglong>>::cast_into(other).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::CppDeletable for crate::q_cbor_stream_reader::StringResultOfQString {
        /// <p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QCborStreamReader::StringResult<QString>::~StringResult()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
        /// <p>The contents of <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#data-var">data</a> are valid only if <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#status-var">status</a> is <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#StringResultCode-enum">Ok</a>. Otherwise, it should be null.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_StringResult_QString_dStringResult(
                self as *mut crate::q_cbor_stream_reader::StringResultOfQString,
            )
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::CppDeletable for crate::q_cbor_stream_reader::StringResultOfQByteArray {
        /// <p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QCborStreamReader::StringResult<QByteArray>::~StringResult()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
        /// <p>The contents of <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#data-var">data</a> are valid only if <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#status-var">status</a> is <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#StringResultCode-enum">Ok</a>. Otherwise, it should be null.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_StringResult_QByteArray_dStringResult(
                self as *mut crate::q_cbor_stream_reader::StringResultOfQByteArray,
            )
        }
    }

    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    impl ::cpp_utils::CppDeletable for crate::q_cbor_stream_reader::StringResultOfLonglong {
        /// <p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QCborStreamReader::StringResult<long long>::~StringResult()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This class is returned by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), with either the contents of the string that was read or an indication that the parsing is done or found an error.</p>
        /// <p>The contents of <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#data-var">data</a> are valid only if <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#status-var">status</a> is <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#StringResultCode-enum">Ok</a>. Otherwise, it should be null.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_StringResult_long_long_dStringResult(
                self as *mut crate::q_cbor_stream_reader::StringResultOfLonglong,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> class is a simple CBOR stream decoder, operating on either a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> or <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>.</p>
///
/// C++ class: <span style='color: green;'>```QCborStreamReader```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> class is a simple CBOR stream decoder, operating on either a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> or <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>.</p>
/// <p>This class can be used to decode a stream of CBOR content directly from either a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> or a <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>. CBOR is the Concise Binary Object Representation, a very compact form of binary data encoding that is compatible with JSON. It was created by the IETF Constrained RESTful Environments (CoRE) WG, which has used it in many new RFCs. It is meant to be used alongside the <a href="https://tools.ietf.org/html/rfc7252">CoAP protocol</a>.</p>
/// <p><a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> provides a StAX-like API, similar to that of <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html">QXmlStreamReader</a>. Using it requires a bit of knowledge of CBOR encoding. For a simpler API, see <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> and especially the decoding function <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromCbor">QCborValue::fromCbor</a>().</p>
/// <p>Typically, one creates a <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> by passing the source <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> or <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> as a parameter to the constructor, then pop elements off the stream if there were no errors in decoding. There are three kinds of CBOR types:</p>
/// <div class="table"><table class="generic">
///  <thead><tr class="qt-style"><th>Kind</th><th>Types</th><th>Behavior</th></tr></thead>
/// <tbody><tr valign="top" class="odd"><td>Fixed-width</td><td>Integers, Tags, Simple types, Floating point</td><td>Value is pre-parsed by <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a>, so accessor functions are <code>const</code>. Must call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#next">next</a>() to advance.</td></tr>
/// <tr valign="top" class="even"><td>Strings</td><td>Byte arrays, Text strings</td><td>Length (if known) is pre-parsed, but the string itself is not. The accessor functions are not const and may allocate memory. Once called, the accessor functions automatically advance to the next element.</td></tr>
/// <tr valign="top" class="odd"><td>Containers</td><td>Arrays, Maps</td><td>Length (if known) is pre-parsed. To access the elements, you must call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#enterContainer">enterContainer</a>(), read all elements, then call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#leaveContainer">leaveContainer</a>(). That function advances to the next element.</td></tr>
/// </tbody></table></div>
/// <p>So a processor function typically looks like this:</p>
/// <pre class="cpp">
///
/// &#32;    <span class="type">void</span> handleStream(<span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#QCborStreamReader">QCborStreamReader</a></span> <span class="operator">&amp;</span>reader)
/// &#32;    {
/// &#32;   &#32;    <span class="keyword">switch</span> (reader<span class="operator">.</span>type())
/// &#32;   &#32;    <span class="keyword">case</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#QCborStreamReader">QCborStreamReader</a></span><span class="operator">::</span>UnsignedInteger:
/// &#32;   &#32;    <span class="keyword">case</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#QCborStreamReader">QCborStreamReader</a></span><span class="operator">::</span>NegativeInteger:
/// &#32;   &#32;    <span class="keyword">case</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#QCborStreamReader">QCborStreamReader</a></span><span class="operator">::</span>SimpleType:
/// &#32;   &#32;    <span class="keyword">case</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#QCborStreamReader">QCborStreamReader</a></span><span class="operator">::</span>Float16:
/// &#32;   &#32;    <span class="keyword">case</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#QCborStreamReader">QCborStreamReader</a></span><span class="operator">::</span>Float:
/// &#32;   &#32;    <span class="keyword">case</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#QCborStreamReader">QCborStreamReader</a></span><span class="operator">::</span>Double:
/// &#32;   &#32;   &#32;    handleFixedWidth(reader);
/// &#32;   &#32;   &#32;    reader<span class="operator">.</span>next();
/// &#32;   &#32;   &#32;    <span class="keyword">break</span>;
/// &#32;   &#32;    <span class="keyword">case</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#QCborStreamReader">QCborStreamReader</a></span><span class="operator">::</span>ByteArray:
/// &#32;   &#32;    <span class="keyword">case</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#QCborStreamReader">QCborStreamReader</a></span><span class="operator">::</span>String:
/// &#32;   &#32;   &#32;    handleString(reader);
/// &#32;   &#32;   &#32;    <span class="keyword">break</span>;
/// &#32;   &#32;    <span class="keyword">case</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#QCborStreamReader">QCborStreamReader</a></span><span class="operator">::</span>Array:
/// &#32;   &#32;    <span class="keyword">case</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#QCborStreamReader">QCborStreamReader</a></span><span class="operator">::</span>Map:
/// &#32;   &#32;   &#32;    reader<span class="operator">.</span>enterContainer();
/// &#32;   &#32;   &#32;    <span class="keyword">while</span> (reader<span class="operator">.</span>lastError() <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtcborcommon.html">QCborError</a></span><span class="operator">::</span>NoError)
/// &#32;   &#32;   &#32;   &#32;    handleStream(reader);
/// &#32;   &#32;   &#32;    <span class="keyword">if</span> (reader<span class="operator">.</span>lastError() <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtcborcommon.html">QCborError</a></span><span class="operator">::</span>NoError)
/// &#32;   &#32;   &#32;   &#32;    reader<span class="operator">.</span>leaveContainer();
/// &#32;   &#32;    }
/// &#32;    }
///
/// </pre>
/// <a name="cbor-support"></a></div>
#[repr(C)]
pub struct QCborStreamReader {
    _unused: u8,
}
impl QCborStreamReader {
    /// <p>Adds <i>data</i> to the CBOR stream and reparses the current element. This function is useful if the end of the data was previously reached while processing the stream, but now more data is available.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamReader::addData(const QByteArray& data)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#addData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds <i>data</i> to the CBOR stream and reparses the current element. This function is useful if the end of the data was previously reached while processing the stream, but now more data is available.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn add_data_q_byte_array(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_addData(
            self as *mut crate::QCborStreamReader,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(data)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamReader::addData(const char* data, long long len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#addData-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Adds <i>len</i> bytes of data starting at <i>data</i> to the CBOR stream and reparses the current element. This function is useful if the end of the data was previously reached while processing the stream, but now more data is available.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn add_data_char_longlong(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        len: ::std::os::raw::c_longlong,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_addData1(
            self as *mut crate::QCborStreamReader,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(data)
                .as_raw_ptr(),
            len,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamReader::addData(const quint8* data, long long len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#addData-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Adds <i>len</i> bytes of data starting at <i>data</i> to the CBOR stream and reparses the current element. This function is useful if the end of the data was previously reached while processing the stream, but now more data is available.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn add_data_u8_longlong(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<u8>>,
        len: ::std::os::raw::c_longlong,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_addData2(
            self as *mut crate::QCborStreamReader,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<u8>>::cast_into(data).as_raw_ptr(),
            len,
        )
    }

    /// <p>Clears the decoder state and resets the input source data to an empty byte array. After this function is called, <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> will be indicating an error parsing.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamReader::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Clears the decoder state and resets the input source data to an empty byte array. After this function is called, <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> will be indicating an error parsing.</p>
    /// <p>Call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#addData">addData</a>() to add more data to be parsed.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#reset">reset</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#setDevice">setDevice</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_clear(self as *mut crate::QCborStreamReader)
    }

    /// <p>Returns the number of containers that this stream has entered with <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#enterContainer">enterContainer</a>() but not yet left.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QCborStreamReader::containerDepth() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#containerDepth">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of containers that this stream has entered with <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#enterContainer">enterContainer</a>() but not yet left.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#enterContainer">enterContainer</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#leaveContainer">leaveContainer</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn container_depth(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_containerDepth(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns the offset in the input stream of the item currently being decoded. The current offset is the number of decoded bytes so far only if the source data is a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> or it is a <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> that was positioned at its beginning when decoding started.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QCborStreamReader::currentOffset() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#currentOffset">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the offset in the input stream of the item currently being decoded. The current offset is the number of decoded bytes so far only if the source data is a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> or it is a <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> that was positioned at its beginning when decoding started.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#reset">reset</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#clear">clear</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#device">device</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn current_offset(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_currentOffset(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns the size of the current text or byte string chunk. If the CBOR stream contains a non-chunked string (that is, if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isLengthKnown">isLengthKnown</a>() returns <code>true</code>), this function returns the size of the entire string, the same as <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#length">length</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```long long QCborStreamReader::currentStringChunkSize() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#currentStringChunkSize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the size of the current text or byte string chunk. If the CBOR stream contains a non-chunked string (that is, if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isLengthKnown">isLengthKnown</a>() returns <code>true</code>), this function returns the size of the entire string, the same as <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#length">length</a>().</p>
    /// <p>This function is useful to pre-allocate the buffer whose pointer can be passed to <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readStringChunk">readStringChunk</a>() later.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readStringChunk">readStringChunk</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn current_string_chunk_size(&self) -> ::std::os::raw::c_longlong {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_currentStringChunkSize(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns the <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> that was set with either <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#setDevice">setDevice</a>() or the <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> constructor. If this object was reading from a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>, this function returns nullptr instead.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QIODevice* QCborStreamReader::device() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#device">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> that was set with either <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#setDevice">setDevice</a>() or the <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> constructor. If this object was reading from a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>, this function returns nullptr instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#setDevice">setDevice</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn device(&self) -> ::cpp_utils::MutPtr<crate::QIODevice> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_device(
            self as *const crate::QCborStreamReader,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Enters the array or map that is the current item and prepares for iterating the elements contained in the container. Returns true if entering the container succeeded, false otherwise (usually, a parsing error). Each call to enterContainer() must be paired with a call to <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#leaveContainer">leaveContainer</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::enterContainer()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#enterContainer">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Enters the array or map that is the current item and prepares for iterating the elements contained in the container. Returns true if entering the container succeeded, false otherwise (usually, a parsing error). Each call to enterContainer() must be paired with a call to <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#leaveContainer">leaveContainer</a>().</p>
    /// <p>This function may only be called if the current item is an array or a map (that is, if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isArray">isArray</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isMap">isMap</a>() or <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isContainer">isContainer</a>() is true). Calling it in any other condition is an error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#leaveContainer">leaveContainer</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isContainer">isContainer</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isArray">isArray</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isMap">isMap</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn enter_container(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_enterContainer(
            self as *mut crate::QCborStreamReader,
        )
    }

    /// <p>Returns true if there are more items to be decoded in the current container or false of we've reached its end. If we're parsing the root element, hasNext() returning false indicates the parsing is complete; otherwise, if the container depth is non-zero, then the outer code needs to call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#leaveContainer">leaveContainer</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::hasNext() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#hasNext">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if there are more items to be decoded in the current container or false of we've reached its end. If we're parsing the root element, hasNext() returning false indicates the parsing is complete; otherwise, if the container depth is non-zero, then the outer code needs to call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#leaveContainer">leaveContainer</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#parentContainerType">parentContainerType</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#containerDepth">containerDepth</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#leaveContainer">leaveContainer</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn has_next(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_hasNext(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns true if the type of the current element is an array (that is, if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>() returns <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::Array</a>). If this function returns true, you may call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#enterContainer">enterContainer</a>() to begin parsing that container.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::isArray() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the type of the current element is an array (that is, if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>() returns <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::Array</a>). If this function returns true, you may call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#enterContainer">enterContainer</a>() to begin parsing that container.</p>
    /// <p>When the current element is an array, you may also call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isLengthKnown">isLengthKnown</a>() to find out if the array's size is explicit in the CBOR stream. If it is, that size can be obtained by calling <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#length">length</a>().</p>
    /// <p>The following example pre-allocates a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a> given the array's size for more efficient decoding:</p>
    /// <pre class="cpp">
    ///
    /// &#32;    <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a></span> populateFromCbor(<span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#QCborStreamReader">QCborStreamReader</a></span> <span class="operator">&amp;</span>reader)
    /// &#32;    {
    /// &#32;   &#32;    <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a></span> list;
    /// &#32;   &#32;    <span class="keyword">if</span> (reader<span class="operator">.</span>isLengthKnown())
    /// &#32;   &#32;   &#32;    list<span class="operator">.</span>reserve(reader<span class="operator">.</span>length());
    ///
    /// &#32;   &#32;    reader<span class="operator">.</span>enterContainer();
    /// &#32;   &#32;    <span class="keyword">while</span> (reader<span class="operator">.</span>lastError() <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtcborcommon.html">QCborError</a></span><span class="operator">::</span>NoError <span class="operator">&amp;</span><span class="operator">&amp;</span> reader<span class="operator">.</span>hasNext())
    /// &#32;   &#32;   &#32;    list<span class="operator">.</span>append(readOneElement(reader));
    /// &#32;   &#32;    <span class="keyword">if</span> (reader<span class="operator">.</span>lastError() <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtcborcommon.html">QCborError</a></span><span class="operator">::</span>NoError)
    /// &#32;   &#32;   &#32;    reader<span class="operator">.</span>leaveContainer();
    /// &#32;    }
    ///
    /// </pre>
    /// <p><b>Note: </b>The code above does not validate that the length is a sensible value. If the input stream reports that the length is 1 billion elements, the above function will try to allocate some 16 GB or more of RAM, which can lead to a crash.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isMap">isMap</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isLengthKnown">isLengthKnown</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#length">length</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#enterContainer">enterContainer</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#leaveContainer">leaveContainer</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_array(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_isArray(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns true if the current element is a boolean value (<code>true</code> or <code>false</code>), false if it is anything else. If this function returns true, you may call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toBool">toBool</a>() to retrieve the value of the boolean. You may also call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toSimpleType">toSimpleType</a>() and compare to either QCborSimpleValue::True or QCborSimpleValue::False.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::isBool() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isBool">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the current element is a boolean value (<code>true</code> or <code>false</code>), false if it is anything else. If this function returns true, you may call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toBool">toBool</a>() to retrieve the value of the boolean. You may also call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toSimpleType">toSimpleType</a>() and compare to either QCborSimpleValue::True or QCborSimpleValue::False.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isFalse">isFalse</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isTrue">isTrue</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toBool">toBool</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isSimpleType">isSimpleType</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toSimpleType">toSimpleType</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_bool(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_isBool(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns true if the type of the current element is a byte array (that is, if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>() returns <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::ByteArray</a>). If this function returns true, you may call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>() to read that data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::isByteArray() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isByteArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the type of the current element is a byte array (that is, if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>() returns <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::ByteArray</a>). If this function returns true, you may call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>() to read that data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isString">isString</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_byte_array(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_isByteArray(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns true if the current element is a container (that is, an array or a map), false if it is anything else. If the current element is a container, the <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isLengthKnown">isLengthKnown</a>() function may be used to find out if the container's size is explicit in the stream and, if so, <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#length">length</a>() can be used to get that size.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::isContainer() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isContainer">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the current element is a container (that is, an array or a map), false if it is anything else. If the current element is a container, the <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isLengthKnown">isLengthKnown</a>() function may be used to find out if the container's size is explicit in the stream and, if so, <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#length">length</a>() can be used to get that size.</p>
    /// <p>More importantly, for a container, the <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#enterContainer">enterContainer</a>() function is available to begin iterating through the elements contained therein.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isArray">isArray</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isMap">isMap</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isLengthKnown">isLengthKnown</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#length">length</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#enterContainer">enterContainer</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#leaveContainer">leaveContainer</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#containerDepth">containerDepth</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_container(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_isContainer(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns true if the type of the current element is an IEEE 754 double-precision floating point (that is, if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>() returns <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::Double</a>). If this function returns true, you may call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toDouble">toDouble</a>() to read that data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::isDouble() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isDouble">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the type of the current element is an IEEE 754 double-precision floating point (that is, if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>() returns <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::Double</a>). If this function returns true, you may call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toDouble">toDouble</a>() to read that data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toDouble">toDouble</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isFloat16">isFloat16</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isFloat">isFloat</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_double(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_isDouble(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns true if the current element is the <code>false</code> value, false if it is anything else.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::isFalse() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isFalse">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the current element is the <code>false</code> value, false if it is anything else.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isTrue">isTrue</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isBool">isBool</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toBool">toBool</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isSimpleType">isSimpleType</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toSimpleType">toSimpleType</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_false(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_isFalse(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns true if the type of the current element is an IEEE 754 single-precision floating point (that is, if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>() returns <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::Float</a>). If this function returns true, you may call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toFloat">toFloat</a>() to read that data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::isFloat() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isFloat">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the type of the current element is an IEEE 754 single-precision floating point (that is, if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>() returns <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::Float</a>). If this function returns true, you may call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toFloat">toFloat</a>() to read that data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toFloat">toFloat</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isFloat16">isFloat16</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isDouble">isDouble</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_float(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_isFloat(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns true if the type of the current element is an IEEE 754 half-precision floating point (that is, if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>() returns <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::Float16</a>). If this function returns true, you may call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toFloat16">toFloat16</a>() to read that data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::isFloat16() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isFloat16">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the type of the current element is an IEEE 754 half-precision floating point (that is, if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>() returns <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::Float16</a>). If this function returns true, you may call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toFloat16">toFloat16</a>() to read that data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toFloat16">toFloat16</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isFloat">isFloat</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isDouble">isDouble</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_float16(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_isFloat16(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns true if the type of the current element is either an unsigned integer or a negative one (that is, if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>() returns <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::UnsignedInteger</a> or <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::NegativeInteger</a>). If this function returns true, you may call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toInteger">toInteger</a>() to read that value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::isInteger() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isInteger">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the type of the current element is either an unsigned integer or a negative one (that is, if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>() returns <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::UnsignedInteger</a> or <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::NegativeInteger</a>). If this function returns true, you may call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toInteger">toInteger</a>() to read that value.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toInteger">toInteger</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toUnsignedInteger">toUnsignedInteger</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toNegativeInteger">toNegativeInteger</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isUnsignedInteger">isUnsignedInteger</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isNegativeInteger">isNegativeInteger</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_integer(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_isInteger(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns true if the current element is invalid, false otherwise. The current element may be invalid if there was a decoding error or we've just parsed the last element in an array or map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::isInvalid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isInvalid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the current element is invalid, false otherwise. The current element may be invalid if there was a decoding error or we've just parsed the last element in an array or map.</p>
    /// <p><b>Note: </b>This function is not to be confused with <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isNull">isNull</a>(). Null is a normal CBOR type that must be handled by the application.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isValid">isValid</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_invalid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_isInvalid(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns true if the length of the current array, map, byte array or string is known (explicit in the CBOR stream), false otherwise. This function should only be called if the element is one of those.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::isLengthKnown() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isLengthKnown">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the length of the current array, map, byte array or string is known (explicit in the CBOR stream), false otherwise. This function should only be called if the element is one of those.</p>
    /// <p>If the length is known, it may be obtained by calling <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#length">length</a>().</p>
    /// <p>If the length of a map or an array is not known, it is implied by the number of elements present in the stream. <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> has no API to calculate the length in that condition.</p>
    /// <p>Strings and byte arrays may also have indeterminate length (that is, they may be transmitted in multiple chunks). Those cannot currently be created with <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a>, but they could be with other encoders, so <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> supports them.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#length">length</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startArray">QCborStreamWriter::startArray</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startMap">QCborStreamWriter::startMap</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_length_known(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_isLengthKnown(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns true if the type of the current element is a map (that is, if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>() returns <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::Map</a>). If this function returns true, you may call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#enterContainer">enterContainer</a>() to begin parsing that container.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::isMap() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isMap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the type of the current element is a map (that is, if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>() returns <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::Map</a>). If this function returns true, you may call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#enterContainer">enterContainer</a>() to begin parsing that container.</p>
    /// <p>When the current element is a map, you may also call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isLengthKnown">isLengthKnown</a>() to find out if the map's size is explicit in the CBOR stream. If it is, that size can be obtained by calling <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#length">length</a>().</p>
    /// <p>The following example pre-allocates a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantMap-typedef">QVariantMap</a> given the map's size for more efficient decoding:</p>
    /// <pre class="cpp">
    ///
    /// &#32;    <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariantMap-typedef">QVariantMap</a></span> populateFromCbor(<span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#QCborStreamReader">QCborStreamReader</a></span> <span class="operator">&amp;</span>reader)
    /// &#32;    {
    /// &#32;   &#32;    <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariantMap-typedef">QVariantMap</a></span> map;
    /// &#32;   &#32;    <span class="keyword">if</span> (reader<span class="operator">.</span>isLengthKnown())
    /// &#32;   &#32;   &#32;    map<span class="operator">.</span>reserve(reader<span class="operator">.</span>length());
    ///
    /// &#32;   &#32;    reader<span class="operator">.</span>enterContainer();
    /// &#32;   &#32;    <span class="keyword">while</span> (reader<span class="operator">.</span>lastError() <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtcborcommon.html">QCborError</a></span><span class="operator">::</span>NoError <span class="operator">&amp;</span><span class="operator">&amp;</span> reader<span class="operator">.</span>hasNext()) {
    /// &#32;   &#32;   &#32;    <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> key <span class="operator">=</span> readElementAsString(reader);
    /// &#32;   &#32;   &#32;    map<span class="operator">.</span>insert(key<span class="operator">,</span> readOneElement(reader));
    /// &#32;   &#32;    }
    /// &#32;   &#32;    <span class="keyword">if</span> (reader<span class="operator">.</span>lastError() <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtcborcommon.html">QCborError</a></span><span class="operator">::</span>NoError)
    /// &#32;   &#32;   &#32;    reader<span class="operator">.</span>leaveContainer();
    /// &#32;    }
    ///
    /// </pre>
    /// <p>The example above uses a function called <code>readElementAsString</code> to read the map's keys and obtain a string. That is because CBOR maps may contain any type as keys, not just strings. User code needs to either perform this conversion, reject non-string keys, or instead use a different container besides <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantMap-typedef">QVariantMap</a> and <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantHash-typedef">QVariantHash</a>. For example, if the map is expected to contain integer keys, which is recommended as it reduces stream size and parsing, the correct container would be <code>\l{QMap}&lt;int, QVariant&gt;</code> or <code>\l{QHash}&lt;int, QVariant&gt;</code>.</p>
    /// <p><b>Note: </b>The code above does not validate that the length is a sensible value. If the input stream reports that the length is 1 billion elements, the above function will try to allocate some 24 GB or more of RAM, which can lead to a crash.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isArray">isArray</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isLengthKnown">isLengthKnown</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#length">length</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#enterContainer">enterContainer</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#leaveContainer">leaveContainer</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_map(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_isMap(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns true if the type of the current element is a negative integer (that is if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>() returns <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::NegativeInteger</a>). If this function returns true, you may call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toNegativeInteger">toNegativeInteger</a>() or <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toInteger">toInteger</a>() to read that value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::isNegativeInteger() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isNegativeInteger">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the type of the current element is a negative integer (that is if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>() returns <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::NegativeInteger</a>). If this function returns true, you may call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toNegativeInteger">toNegativeInteger</a>() or <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toInteger">toInteger</a>() to read that value.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toNegativeInteger">toNegativeInteger</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toInteger">toInteger</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isInteger">isInteger</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isUnsignedInteger">isUnsignedInteger</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_negative_integer(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_isNegativeInteger(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns true if the current element is the <code>null</code> value, false if it is anything else. Null values may be used to indicate the absence of some optional data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::isNull() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the current element is the <code>null</code> value, false if it is anything else. Null values may be used to indicate the absence of some optional data.</p>
    /// <p><b>Note: </b>This function is not the opposite of <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isValid">isValid</a>(). A Null value is a valid CBOR value.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isSimpleType">isSimpleType</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toSimpleType">toSimpleType</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_null(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_isNull(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns true if the type of the current element is any CBOR simple type, including a boolean value (true and false) as well as null and undefined. To find out which simple type this is, call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toSimpleType">toSimpleType</a>(). Alternatively, to test for one specific simple type, call the overload that takes a <a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborSimpleType-enum">QCborSimpleType</a> parameter.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::isSimpleType() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isSimpleType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the type of the current element is any CBOR simple type, including a boolean value (true and false) as well as null and undefined. To find out which simple type this is, call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toSimpleType">toSimpleType</a>(). Alternatively, to test for one specific simple type, call the overload that takes a <a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborSimpleType-enum">QCborSimpleType</a> parameter.</p>
    /// <p>CBOR simple types are types that do not carry extra value. There are 255 possibilities, but there are currently only four values that have defined meaning. Code is not expected to cope with unknown simple types and may simply discard the stream as invalid if it finds an unknown one.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborSimpleType-enum">QCborSimpleType</a>, <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isSimpleType-1">isSimpleType</a>(QCborSimpleType), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toSimpleType">toSimpleType</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_simple_type_0a(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_isSimpleType(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns true if the type of the current element is the simple type <i>st</i>, false otherwise. If this function returns true, then <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toSimpleType">toSimpleType</a>() will return <i>st</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::isSimpleType(QCborSimpleType st) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isSimpleType-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the type of the current element is the simple type <i>st</i>, false otherwise. If this function returns true, then <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toSimpleType">toSimpleType</a>() will return <i>st</i>.</p>
    /// <p>CBOR simple types are types that do not carry extra value. There are 255 possibilities, but there are currently only four values that have defined meaning. Code is not expected to cope with unknown simple types and may simply discard the stream as invalid if it finds an unknown one.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborSimpleType-enum">QCborSimpleType</a>, <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isSimpleType">isSimpleType</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toSimpleType">toSimpleType</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_simple_type_1a(&self, st: crate::QCborSimpleType) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_isSimpleType1(
            self as *const crate::QCborStreamReader,
            st,
        )
    }

    /// <p>Returns true if the type of the current element is a text string (that is, if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>() returns <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::String</a>). If this function returns true, you may call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() to read that data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::isString() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the type of the current element is a text string (that is, if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>() returns <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::String</a>). If this function returns true, you may call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() to read that data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isByteArray">isByteArray</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_string(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_isString(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns true if the type of the current element is a CBOR tag (that is, if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>() returns <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::Tag</a>). If this function returns true, you may call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toTag">toTag</a>() to read that data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::isTag() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isTag">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the type of the current element is a CBOR tag (that is, if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>() returns <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::Tag</a>). If this function returns true, you may call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toTag">toTag</a>() to read that data.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toTag">toTag</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_tag(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_isTag(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns true if the current element is the <code>true</code> value, false if it is anything else.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::isTrue() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isTrue">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the current element is the <code>true</code> value, false if it is anything else.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isFalse">isFalse</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isBool">isBool</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toBool">toBool</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isSimpleType">isSimpleType</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toSimpleType">toSimpleType</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_true(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_isTrue(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns true if the current element is the <code>undefined</code> value, false if it is anything else. Undefined values may be encoded to indicate that some conversion failed or was not possible when creating the stream. <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> never performs any replacement and this function will only return true if the stream contains an explicit undefined value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::isUndefined() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isUndefined">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the current element is the <code>undefined</code> value, false if it is anything else. Undefined values may be encoded to indicate that some conversion failed or was not possible when creating the stream. <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> never performs any replacement and this function will only return true if the stream contains an explicit undefined value.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isSimpleType">isSimpleType</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toSimpleType">toSimpleType</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_undefined(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_isUndefined(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns true if the type of the current element is an unsigned integer (that is if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>() returns <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::UnsignedInteger</a>). If this function returns true, you may call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toUnsignedInteger">toUnsignedInteger</a>() or <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toInteger">toInteger</a>() to read that value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::isUnsignedInteger() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isUnsignedInteger">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the type of the current element is an unsigned integer (that is if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>() returns <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::UnsignedInteger</a>). If this function returns true, you may call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toUnsignedInteger">toUnsignedInteger</a>() or <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toInteger">toInteger</a>() to read that value.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toUnsignedInteger">toUnsignedInteger</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toInteger">toInteger</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isInteger">isInteger</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isNegativeInteger">isNegativeInteger</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_unsigned_integer(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_isUnsignedInteger(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns true if the current element is valid, false otherwise. The current element may be invalid if there was a decoding error or we've just parsed the last element in an array or map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::isValid() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isValid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the current element is valid, false otherwise. The current element may be invalid if there was a decoding error or we've just parsed the last element in an array or map.</p>
    /// <p><b>Note: </b>This function is not the opposite of <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isNull">isNull</a>(). Null is a normal CBOR type that must be handled by the application.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isInvalid">isInvalid</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_valid(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_isValid(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns the last error in decoding the stream, if any. If no error was encountered, this returns an <a href="http://doc.qt.io/qt-5/qcborerror.html#Code-enum">QCborError::NoError</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborError QCborStreamReader::lastError()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#lastError">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the last error in decoding the stream, if any. If no error was encountered, this returns an <a href="http://doc.qt.io/qt-5/qcborerror.html#Code-enum">QCborError::NoError</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isValid">isValid</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn last_error(&mut self) -> ::cpp_utils::CppBox<crate::QCborError> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_lastError(
            self as *mut crate::QCborStreamReader,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Leaves the array or map whose items were being processed and positions the decoder at the next item after the end of the container. Returns true if leaving the container succeeded, false otherwise (usually, a parsing error). Each call to <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#enterContainer">enterContainer</a>() must be paired with a call to leaveContainer().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::leaveContainer()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#leaveContainer">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Leaves the array or map whose items were being processed and positions the decoder at the next item after the end of the container. Returns true if leaving the container succeeded, false otherwise (usually, a parsing error). Each call to <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#enterContainer">enterContainer</a>() must be paired with a call to leaveContainer().</p>
    /// <p>This function may only be called if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#hasNext">hasNext</a>() has returned false and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#containerDepth">containerDepth</a>() is not zero. Calling it in any other condition is an error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#enterContainer">enterContainer</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#parentContainerType">parentContainerType</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#containerDepth">containerDepth</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn leave_container(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_leaveContainer(
            self as *mut crate::QCborStreamReader,
        )
    }

    /// <p>Returns the length of the string or byte array, or the number of items in an array or the number, of item pairs in a map, if known. This function must not be called if the length is unknown (that is, if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isLengthKnown">isLengthKnown</a>() returned false). It is an error to do that and it will cause <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> to stop parsing the input stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```quint64 QCborStreamReader::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the length of the string or byte array, or the number of items in an array or the number, of item pairs in a map, if known. This function must not be called if the length is unknown (that is, if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isLengthKnown">isLengthKnown</a>() returned false). It is an error to do that and it will cause <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> to stop parsing the input stream.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isLengthKnown">isLengthKnown</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startArray">QCborStreamWriter::startArray</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#startMap">QCborStreamWriter::startMap</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn length(&self) -> u64 {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_length(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> object with no source data. After construction, <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> will report an error parsing.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborStreamReader::QCborStreamReader()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#QCborStreamReader">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Creates a <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> object with no source data. After construction, <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> will report an error parsing.</p>
    /// <p>You can add more data by calling <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#addData">addData</a>() or by setting a different source device using <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#setDevice">setDevice</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#addData">addData</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isValid">isValid</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QCborStreamReader> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_QCborStreamReader();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborStreamReader::QCborStreamReader(const char* data, long long len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#QCborStreamReader-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> object with <i>len</i> bytes of data starting at <i>data</i>. The pointer must remain valid until <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> is destroyed.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_char_longlong(
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        len: ::std::os::raw::c_longlong,
    ) -> ::cpp_utils::CppBox<crate::QCborStreamReader> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_QCborStreamReader1(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(data)
                .as_raw_ptr(),
            len,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborStreamReader::QCborStreamReader(const quint8* data, long long len)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#QCborStreamReader-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> object with <i>len</i> bytes of data starting at <i>data</i>. The pointer must remain valid until <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> is destroyed.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_u8_longlong(
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<u8>>,
        len: ::std::os::raw::c_longlong,
    ) -> ::cpp_utils::CppBox<crate::QCborStreamReader> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_QCborStreamReader2(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<u8>>::cast_into(data).as_raw_ptr(),
            len,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborStreamReader::QCborStreamReader(const QByteArray& data)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#QCborStreamReader-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> object that will parse the CBOR stream found in <i>data</i>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_q_byte_array(
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    ) -> ::cpp_utils::CppBox<crate::QCborStreamReader> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_QCborStreamReader3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(data)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QCborStreamReader::QCborStreamReader(QIODevice* device)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#QCborStreamReader-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Creates a <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> object that will parse the CBOR stream found by reading from <i>device</i>. <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> does not take ownership of <i>device</i>, so it must remain valid until this oject is destroyed.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn from_q_io_device(
        device: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QIODevice>>,
    ) -> ::cpp_utils::CppBox<crate::QCborStreamReader> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_QCborStreamReader4(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QIODevice>>::cast_into(device)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Advance the CBOR stream decoding one element. You should usually call this function when parsing fixed-width basic elements (that is, integers, simple values, tags and floating point values). But this function can be called when the current item is a string, array or map too and it will skip over that entire element, including all contained elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::next(int maxRecursion = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#next">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advance the CBOR stream decoding one element. You should usually call this function when parsing fixed-width basic elements (that is, integers, simple values, tags and floating point values). But this function can be called when the current item is a string, array or map too and it will skip over that entire element, including all contained elements.</p>
    /// <p>This function returns true if advancing was successful, false otherwise. It may fail if the stream is corrupt, incomplete or if the nesting level of arrays and maps exceeds <i>maxRecursion</i>. Calling this function when <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#hasNext">hasNext</a>() has returned false is also an error. If this function returns false, <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#lastError">lastError</a>() will return the error code detailing what the failure was.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#lastError">lastError</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isValid">isValid</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#hasNext">hasNext</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn next_1a(&mut self, max_recursion: ::std::os::raw::c_int) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_next(
            self as *mut crate::QCborStreamReader,
            max_recursion,
        )
    }

    /// <p>Advance the CBOR stream decoding one element. You should usually call this function when parsing fixed-width basic elements (that is, integers, simple values, tags and floating point values). But this function can be called when the current item is a string, array or map too and it will skip over that entire element, including all contained elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::next()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#next">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advance the CBOR stream decoding one element. You should usually call this function when parsing fixed-width basic elements (that is, integers, simple values, tags and floating point values). But this function can be called when the current item is a string, array or map too and it will skip over that entire element, including all contained elements.</p>
    /// <p>This function returns true if advancing was successful, false otherwise. It may fail if the stream is corrupt, incomplete or if the nesting level of arrays and maps exceeds <i>maxRecursion</i>. Calling this function when <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#hasNext">hasNext</a>() has returned false is also an error. If this function returns false, <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#lastError">lastError</a>() will return the error code detailing what the failure was.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#lastError">lastError</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isValid">isValid</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#hasNext">hasNext</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn next_0a(&mut self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_next1(self as *mut crate::QCborStreamReader)
    }

    /// <p>Returns either <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::Array</a> or <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::Map</a>, indicating whether the container that contains the current item was an array or map, respectively. If we're currently parsing the root element, this function returns <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::Invalid</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborStreamReader::Type QCborStreamReader::parentContainerType() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#parentContainerType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns either <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::Array</a> or <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::Map</a>, indicating whether the container that contains the current item was an array or map, respectively. If we're currently parsing the root element, this function returns <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">QCborStreamReader::Invalid</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#containerDepth">containerDepth</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#enterContainer">enterContainer</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn parent_container_type(&self) -> crate::q_cbor_stream_reader::Type {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_parentContainerType(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Decodes one byte array chunk from the CBOR string and returns it. This function is used for both regular and chunked contents, so the caller must always loop around calling this function, even if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isLengthKnown">isLengthKnown</a>() has is true. The typical use of this function is as follows:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborStreamReader::StringResult<QByteArray> QCborStreamReader::readByteArray()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Decodes one byte array chunk from the CBOR string and returns it. This function is used for both regular and chunked contents, so the caller must always loop around calling this function, even if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isLengthKnown">isLengthKnown</a>() has is true. The typical use of this function is as follows:</p>
    /// <pre class="cpp">
    ///
    /// &#32;    <span class="type">QBytearray</span> decodeBytearray(<span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#QCborStreamReader">QCborStreamReader</a></span> <span class="operator">&amp;</span>reader)
    /// &#32;    {
    /// &#32;   &#32;    <span class="type">QBytearray</span> result;
    /// &#32;   &#32;    <span class="keyword">auto</span> r <span class="operator">=</span> reader<span class="operator">.</span>readBytearray();
    /// &#32;   &#32;    <span class="keyword">while</span> (r<span class="operator">.</span>code <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#QCborStreamReader">QCborStreamReader</a></span><span class="operator">::</span>Ok) {
    /// &#32;   &#32;   &#32;    result <span class="operator">+</span><span class="operator">=</span> r<span class="operator">.</span>data;
    /// &#32;   &#32;   &#32;    r <span class="operator">=</span> reader<span class="operator">.</span>readByteArray();
    /// &#32;   &#32;    }
    ///
    /// &#32;   &#32;    <span class="keyword">if</span> (r<span class="operator">.</span>code <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#QCborStreamReader">QCborStreamReader</a></span><span class="operator">::</span>Error) {
    /// &#32;   &#32;   &#32;    <span class="comment">// handle error condition</span>
    /// &#32;   &#32;   &#32;    result<span class="operator">.</span>clear();
    /// &#32;   &#32;    }
    /// &#32;   &#32;    <span class="keyword">return</span> result;
    /// &#32;    }
    ///
    /// </pre>
    /// <p>This function does not perform any type conversions, including from integers or from strings. Therefore, it may only be called if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isByteArray">isByteArray</a>() is true; calling it in any other condition is an error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isByteArray">isByteArray</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readStringChunk">readStringChunk</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn read_byte_array(
        &mut self,
    ) -> ::cpp_utils::CppBox<crate::q_cbor_stream_reader::StringResultOfQByteArray> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_readByteArray(
            self as *mut crate::QCborStreamReader,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Decodes one string chunk from the CBOR string and returns it. This function is used for both regular and chunked string contents, so the caller must always loop around calling this function, even if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isLengthKnown">isLengthKnown</a>() has is true. The typical use of this function is as follows:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborStreamReader::StringResult<QString> QCborStreamReader::readString()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Decodes one string chunk from the CBOR string and returns it. This function is used for both regular and chunked string contents, so the caller must always loop around calling this function, even if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isLengthKnown">isLengthKnown</a>() has is true. The typical use of this function is as follows:</p>
    /// <pre class="cpp">
    ///
    /// &#32;    <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> decodeString(<span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#QCborStreamReader">QCborStreamReader</a></span> <span class="operator">&amp;</span>reader)
    /// &#32;    {
    /// &#32;   &#32;    <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> result;
    /// &#32;   &#32;    <span class="keyword">auto</span> r <span class="operator">=</span> reader<span class="operator">.</span>readString();
    /// &#32;   &#32;    <span class="keyword">while</span> (r<span class="operator">.</span>code <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#QCborStreamReader">QCborStreamReader</a></span><span class="operator">::</span>Ok) {
    /// &#32;   &#32;   &#32;    result <span class="operator">+</span><span class="operator">=</span> r<span class="operator">.</span>data;
    /// &#32;   &#32;   &#32;    r <span class="operator">=</span> reader<span class="operator">.</span>readString();
    /// &#32;   &#32;    }
    ///
    /// &#32;   &#32;    <span class="keyword">if</span> (r<span class="operator">.</span>code <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#QCborStreamReader">QCborStreamReader</a></span><span class="operator">::</span>Error) {
    /// &#32;   &#32;   &#32;    <span class="comment">// handle error condition</span>
    /// &#32;   &#32;   &#32;    result<span class="operator">.</span>clear();
    /// &#32;   &#32;    }
    /// &#32;   &#32;    <span class="keyword">return</span> result;
    /// &#32;    }
    ///
    /// </pre>
    /// <p>This function does not perform any type conversions, including from integers or from byte arrays. Therefore, it may only be called if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isString">isString</a>() returned true; calling it in any other condition is an error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isString">isString</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readStringChunk">readStringChunk</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn read_string(
        &mut self,
    ) -> ::cpp_utils::CppBox<crate::q_cbor_stream_reader::StringResultOfQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_readString(
            self as *mut crate::QCborStreamReader,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reads the current string chunk into the buffer pointed to by <i>ptr</i>, whose size is <i>maxlen</i>. This function returns a <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html">StringResult</a> object, with the number of bytes copied into <i>ptr</i> saved in the <code>\l</code> <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#data-var">StringResult::data</a> member. The <code>\l</code> <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#status-var">StringResult::status</a> member indicates whether there was an error reading the string, whether data was copied or whether this was the last chunk.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborStreamReader::StringResult<long long> QCborStreamReader::readStringChunk(char* ptr, long long maxlen)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readStringChunk">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads the current string chunk into the buffer pointed to by <i>ptr</i>, whose size is <i>maxlen</i>. This function returns a <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html">StringResult</a> object, with the number of bytes copied into <i>ptr</i> saved in the <code>\l</code> <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#data-var">StringResult::data</a> member. The <code>\l</code> <a href="http://doc.qt.io/qt-5/qcborstreamreader-stringresult.html#status-var">StringResult::status</a> member indicates whether there was an error reading the string, whether data was copied or whether this was the last chunk.</p>
    /// <p>This function can be called for both <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">String</a> and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#Type-enum">ByteArray</a> types. For the latter, this function will read the same data that <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>() would have returned. For strings, it returns the UTF-8 equivalent of the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> that would have been returned.</p>
    /// <p>This function is usually used alongside <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#currentStringChunkSize">currentStringChunkSize</a>() in a loop. For example:</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#QCborStreamReader">QCborStreamReader</a></span><span class="operator">&lt;</span>qsizetype<span class="operator">&gt;</span> result;
    /// &#32;     <span class="keyword">do</span> {
    /// &#32;   &#32;     qsizetype size <span class="operator">=</span> reader<span class="operator">.</span>currentStringChunkSize();
    /// &#32;   &#32;     qsizetype oldsize <span class="operator">=</span> buffer<span class="operator">.</span>size();
    /// &#32;   &#32;     buffer<span class="operator">.</span>resize(oldsize <span class="operator">+</span> size);
    /// &#32;   &#32;     result <span class="operator">=</span> reader<span class="operator">.</span>readStringChunk(buffer<span class="operator">.</span>data() <span class="operator">+</span> oldsize<span class="operator">,</span> size);
    /// &#32;     } <span class="keyword">while</span> (result<span class="operator">.</span>status() <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#QCborStreamReader">QCborStreamReader</a></span><span class="operator">::</span>Ok);
    ///
    /// </pre>
    /// <p>Unlike <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>(), this function is not limited by implementation limits of <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> and <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p><b>Note: </b>This function does not perform verification that the UTF-8 contents are properly formatted. That means this function does not produce the <a href="http://doc.qt.io/qt-5/qcborerror.html#Code-enum">QCborError::InvalidUtf8String</a> error, even when <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>() does.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#currentStringChunkSize">currentStringChunkSize</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readString">readString</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#readByteArray">readByteArray</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isString">isString</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isByteArray">isByteArray</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn read_string_chunk(
        &mut self,
        ptr: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_char>>,
        maxlen: ::std::os::raw::c_longlong,
    ) -> ::cpp_utils::CppBox<crate::q_cbor_stream_reader::StringResultOfLonglong> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_readStringChunk(
            self as *mut crate::QCborStreamReader,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_char>>::cast_into(ptr)
                .as_mut_raw_ptr(),
            maxlen,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reparses the current element. This function must be called when more data becomes available in the source <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> after parsing failed due to reaching the end of the input data before the end of the CBOR stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamReader::reparse()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#reparse">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reparses the current element. This function must be called when more data becomes available in the source <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> after parsing failed due to reaching the end of the input data before the end of the CBOR stream.</p>
    /// <p>When reading from QByteArray(), the <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#addData">addData</a>() function automatically calls this function. Calling it when the reading had not failed is a no-op.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn reparse(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_reparse(
            self as *mut crate::QCborStreamReader,
        )
    }

    /// <p>Resets the source back to the beginning and clears the decoder state. If the source data was a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>, <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> will restart from the beginning of the array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamReader::reset()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#reset">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Resets the source back to the beginning and clears the decoder state. If the source data was a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a>, <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> will restart from the beginning of the array.</p>
    /// <p>If the source data is a <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>, this function will call <a href="http://doc.qt.io/qt-5/qiodevice.html#reset">QIODevice::reset</a>(), which will seek to byte position 0. If the CBOR stream is not found at the beginning of the device (e.g., beginning of a file), then this function will likely do the wrong thing. Instead, position the <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> to the right offset and call <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#setDevice">setDevice</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#clear">clear</a>() and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#setDevice">setDevice</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn reset(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_reset(self as *mut crate::QCborStreamReader)
    }

    /// <p>Sets the source of data to <i>device</i>, resetting the decoder to its initial state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QCborStreamReader::setDevice(QIODevice* device)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#setDevice">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the source of data to <i>device</i>, resetting the decoder to its initial state.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#device">device</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn set_device(
        &mut self,
        device: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QIODevice>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_setDevice(
            self as *mut crate::QCborStreamReader,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QIODevice>>::cast_into(device)
                .as_mut_raw_ptr(),
        )
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the boolean value of the current element.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborStreamReader::toBool() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toBool">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the boolean value of the current element.</p>
    /// <p>This function does not perform any type conversions, including from integer. Therefore, it may only be called if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isTrue">isTrue</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isFalse">isFalse</a>() or <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isBool">isBool</a>() returned true; calling it in any other condition is an error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isBool">isBool</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isTrue">isTrue</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isFalse">isFalse</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toInteger">toInteger</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_bool(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_toBool(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns the 64-bit double-precision floating point value of the current element.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```double QCborStreamReader::toDouble() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toDouble">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the 64-bit double-precision floating point value of the current element.</p>
    /// <p>This function does not perform any type conversions, including from other floating point types or from integer values. Therefore, it may only be called if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isDouble">isDouble</a>() is true; calling it in any other condition is an error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isDouble">isDouble</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toFloat16">toFloat16</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toFloat">toFloat</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_double(&self) -> ::std::os::raw::c_double {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_toDouble(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns the 32-bit single-precision floating point value of the current element.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```float QCborStreamReader::toFloat() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toFloat">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the 32-bit single-precision floating point value of the current element.</p>
    /// <p>This function does not perform any type conversions, including from other floating point types or from integer values. Therefore, it may only be called if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isFloat">isFloat</a>() is true; calling it in any other condition is an error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isFloat">isFloat</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toFloat16">toFloat16</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toDouble">toDouble</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_float(&self) -> ::std::os::raw::c_float {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_toFloat(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns the 16-bit half-precision floating point value of the current element.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qfloat16 QCborStreamReader::toFloat16() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toFloat16">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the 16-bit half-precision floating point value of the current element.</p>
    /// <p>This function does not perform any type conversions, including from other floating point types or from integer values. Therefore, it may only be called if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isFloat16">isFloat16</a>() is true; calling it in any other condition is an error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isFloat16">isFloat16</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toFloat">toFloat</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toDouble">toDouble</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_float16(&self) -> ::cpp_utils::CppBox<crate::Qfloat16> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_toFloat16(
            self as *const crate::QCborStreamReader,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the integer value of the current element, be it negative, positive or zero. If the value is larger than 2<sup>63</sup> - 1 or smaller than -2<sup>63</sup>, the returned value will overflow and will have an incorrect sign. If handling those values is required, use <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toUnsignedInteger">toUnsignedInteger</a>() or <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toNegativeInteger">toNegativeInteger</a>() instead.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```qint64 QCborStreamReader::toInteger() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toInteger">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the integer value of the current element, be it negative, positive or zero. If the value is larger than 2<sup>63</sup> - 1 or smaller than -2<sup>63</sup>, the returned value will overflow and will have an incorrect sign. If handling those values is required, use <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toUnsignedInteger">toUnsignedInteger</a>() or <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toNegativeInteger">toNegativeInteger</a>() instead.</p>
    /// <p>This function does not perform any type conversions, including from boolean or CBOR tag. Therefore, it may only be called if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isInteger">isInteger</a>() is true; calling it in any other condition is an error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isInteger">isInteger</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toUnsignedInteger">toUnsignedInteger</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toNegativeInteger">toNegativeInteger</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_integer(&self) -> i64 {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_toInteger(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns the negative integer value of the current element. QCborNegativeValue is a 64-bit unsigned integer containing the absolute value of the negative number that was stored in the CBOR stream. Additionally, QCborNegativeValue(0) represents the number -2<sup>64</sup>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborNegativeInteger QCborStreamReader::toNegativeInteger() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toNegativeInteger">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the negative integer value of the current element. QCborNegativeValue is a 64-bit unsigned integer containing the absolute value of the negative number that was stored in the CBOR stream. Additionally, QCborNegativeValue(0) represents the number -2<sup>64</sup>.</p>
    /// <p>This function does not perform any type conversions, including from boolean or CBOR tag. Therefore, it may only be called if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isNegativeInteger">isNegativeInteger</a>() is true; calling it in any other condition is an error.</p>
    /// <p>This function may be used to obtain numbers beyond the range of the return type of <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toInteger">toInteger</a>(). However, use of negative numbers smaller than -2<sup>63</sup> is extremely discouraged.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toInteger">toInteger</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isNegativeInteger">isNegativeInteger</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isUnsignedInteger">isUnsignedInteger</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_negative_integer(&self) -> crate::QCborNegativeInteger {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_toNegativeInteger(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns value of the current simple type.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborSimpleType QCborStreamReader::toSimpleType() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toSimpleType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns value of the current simple type.</p>
    /// <p>This function does not perform any type conversions, including from integer. Therefore, it may only be called if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isSimpleType">isSimpleType</a>() is true; calling it in any other condition is an error.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isSimpleType">isSimpleType</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isTrue">isTrue</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isFalse">isFalse</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isBool">isBool</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isNull">isNull</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isUndefined">isUndefined</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_simple_type(&self) -> crate::QCborSimpleType {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_toSimpleType(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns the tag value of the current element.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborTag QCborStreamReader::toTag() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toTag">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the tag value of the current element.</p>
    /// <p>This function does not perform any type conversions, including from integer. Therefore, it may only be called if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isTag">isTag</a>() is true; calling it in any other condition is an error.</p>
    /// <p>Tags are 64-bit numbers attached to generic CBOR types that give them further meaning. For a list of known tags, see the <a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborKnownTags-enum">QCborKnownTags</a> enumeration.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isTag">isTag</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toInteger">toInteger</a>(), and <a href="http://doc.qt.io/qt-5/qtcborcommon.html#QCborKnownTags-enum">QCborKnownTags</a>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_tag(&self) -> crate::QCborTag {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_toTag(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns the unsigned integer value of the current element.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```quint64 QCborStreamReader::toUnsignedInteger() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toUnsignedInteger">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the unsigned integer value of the current element.</p>
    /// <p>This function does not perform any type conversions, including from boolean or CBOR tag. Therefore, it may only be called if <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isUnsignedInteger">isUnsignedInteger</a>() is true; calling it in any other condition is an error.</p>
    /// <p>This function may be used to obtain numbers beyond the range of the return type of <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toInteger">toInteger</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">type</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#toInteger">toInteger</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isUnsignedInteger">isUnsignedInteger</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isNegativeInteger">isNegativeInteger</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn to_unsigned_integer(&self) -> u64 {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_toUnsignedInteger(
            self as *const crate::QCborStreamReader,
        )
    }

    /// <p>Returns the type of the current element. It is one of the valid types or Invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborStreamReader::Type QCborStreamReader::type() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#type">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the type of the current element. It is one of the valid types or Invalid.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isValid">isValid</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isUnsignedInteger">isUnsignedInteger</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isNegativeInteger">isNegativeInteger</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isInteger">isInteger</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isByteArray">isByteArray</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isString">isString</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isArray">isArray</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isMap">isMap</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isTag">isTag</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isSimpleType">isSimpleType</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isBool">isBool</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isFalse">isFalse</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isTrue">isTrue</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isNull">isNull</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isUndefined">isUndefined</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isFloat16">isFloat16</a>(), <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isFloat">isFloat</a>(), and <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#isDouble">isDouble</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn type_(&self) -> crate::q_cbor_stream_reader::Type {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_type(
            self as *const crate::QCborStreamReader,
        )
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
///
/// C++ class: <span style='color: green;'>```QPair<QCborValueRef, QCborValueRef>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qpair.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
/// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; can be used in your application if the STL <code>pair</code> type is not available. It stores one value of type T1 and one value of type T2. It can be used as a return value for a function that needs to return two values, or as the value type of a <a href="http://doc.qt.io/qt-5/containers.html">generic container</a>.</p>
/// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> that stores one <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and one <code>double</code> value:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qpair.html#QPair">QPair</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> pair;
///
/// </pre>
/// <p>The components are accessible as public data members called <a href="http://doc.qt.io/qt-5/qpair.html#first-var">first</a> and <a href="http://doc.qt.io/qt-5/qpair.html#second-var">second</a>. For example:</p>
/// <pre class="cpp">
///
///   pair<span class="operator">.</span>first <span class="operator">=</span> <span class="string">"pi"</span>;
///   pair<span class="operator">.</span>second <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
///
/// </pre>
/// <p>Note, however, that it is almost always preferable to define a small struct to hold the result of a function with multiple return values. A struct trivially generalizes to more than two values, and allows more descriptive member names than <code>first</code> and <code>second</code>:</p>
/// <pre class="cpp">
///
///   <span class="keyword">struct</span> Variable {
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> name;
/// &#32;     <span class="type">double</span> value;
///   };
///   Variable v;
///   v<span class="operator">.</span>name <span class="operator">=</span> <span class="string">"pi"</span>;
///   v<span class="operator">.</span>value <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
///
/// </pre>
/// <p>The advent of C++11 automatic variable type deduction (<code>auto</code>) shifts the emphasis from the type name to the name of functions and members. Thus, <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>, like <code>std::pair</code> and <code>std::tuple</code>, is mostly useful in generic (template) code, where defining a dedicated type is not possible.</p>
/// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>'s template data types (T1 and T2) must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; these requirements are documented on a per-function basis.</p></div>
#[repr(C)]
pub struct QPairOfQCborValueRefQCborValueRef {
    _unused: u8,
}
impl QPairOfQCborValueRefQCborValueRef {
    /// <p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPair<QCborValueRef, QCborValueRef>& QPair<QCborValueRef, QCborValueRef>::operator=(const QPair<QCborValueRef, QCborValueRef>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; can be used in your application if the STL <code>pair</code> type is not available. It stores one value of type T1 and one value of type T2. It can be used as a return value for a function that needs to return two values, or as the value type of a <a href="http://doc.qt.io/qt-5/containers.html">generic container</a>.</p>
    /// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> that stores one <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and one <code>double</code> value:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpair.html#QPair">QPair</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> pair;
    ///
    /// </pre>
    /// <p>The components are accessible as public data members called <a href="http://doc.qt.io/qt-5/qpair.html#first-var">first</a> and <a href="http://doc.qt.io/qt-5/qpair.html#second-var">second</a>. For example:</p>
    /// <pre class="cpp">
    ///
    ///   pair<span class="operator">.</span>first <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   pair<span class="operator">.</span>second <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>Note, however, that it is almost always preferable to define a small struct to hold the result of a function with multiple return values. A struct trivially generalizes to more than two values, and allows more descriptive member names than <code>first</code> and <code>second</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">struct</span> Variable {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> name;
    /// &#32;     <span class="type">double</span> value;
    ///   };
    ///   Variable v;
    ///   v<span class="operator">.</span>name <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   v<span class="operator">.</span>value <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>The advent of C++11 automatic variable type deduction (<code>auto</code>) shifts the emphasis from the type name to the name of functions and members. Thus, <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>, like <code>std::pair</code> and <code>std::tuple</code>, is mostly useful in generic (template) code, where defining a dedicated type is not possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>'s template data types (T1 and T2) must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; these requirements are documented on a per-function basis.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQCborValueRefQCborValueRef>>,
    ) -> ::cpp_utils::MutRef<crate::QPairOfQCborValueRefQCborValueRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPair_QCborValueRef_QCborValueRef_operator_(self as *mut crate::QPairOfQCborValueRefQCborValueRef, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQCborValueRefQCborValueRef>>::cast_into(other).as_raw_ptr());
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Constructs a pair and initializes the <code>first</code> element with <i>value1</i> and the <code>second</code> element with <i>value2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPair<QCborValueRef, QCborValueRef>::QPair(const QCborValueRef& t1, const QCborValueRef& t2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html#QPair-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a pair and initializes the <code>first</code> element with <i>value1</i> and the <code>second</code> element with <i>value2</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpair.html#qMakePair">qMakePair</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn new(
        t1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValueRef>>,
        t2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValueRef>>,
    ) -> ::cpp_utils::CppBox<crate::QPairOfQCborValueRefQCborValueRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPair_QCborValueRef_QCborValueRef_QPair1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValueRef>>::cast_into(t1)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValueRef>>::cast_into(t2)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPair<QCborValueRef, QCborValueRef>::QPair(const QPair<QCborValueRef, QCborValueRef>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; can be used in your application if the STL <code>pair</code> type is not available. It stores one value of type T1 and one value of type T2. It can be used as a return value for a function that needs to return two values, or as the value type of a <a href="http://doc.qt.io/qt-5/containers.html">generic container</a>.</p>
    /// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> that stores one <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and one <code>double</code> value:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpair.html#QPair">QPair</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> pair;
    ///
    /// </pre>
    /// <p>The components are accessible as public data members called <a href="http://doc.qt.io/qt-5/qpair.html#first-var">first</a> and <a href="http://doc.qt.io/qt-5/qpair.html#second-var">second</a>. For example:</p>
    /// <pre class="cpp">
    ///
    ///   pair<span class="operator">.</span>first <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   pair<span class="operator">.</span>second <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>Note, however, that it is almost always preferable to define a small struct to hold the result of a function with multiple return values. A struct trivially generalizes to more than two values, and allows more descriptive member names than <code>first</code> and <code>second</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">struct</span> Variable {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> name;
    /// &#32;     <span class="type">double</span> value;
    ///   };
    ///   Variable v;
    ///   v<span class="operator">.</span>name <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   v<span class="operator">.</span>value <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>The advent of C++11 automatic variable type deduction (<code>auto</code>) shifts the emphasis from the type name to the name of functions and members. Thus, <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>, like <code>std::pair</code> and <code>std::tuple</code>, is mostly useful in generic (template) code, where defining a dedicated type is not possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>'s template data types (T1 and T2) must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; these requirements are documented on a per-function basis.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQCborValueRefQCborValueRef>>,
    ) -> ::cpp_utils::CppBox<crate::QPairOfQCborValueRefQCborValueRef> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPair_QCborValueRef_QCborValueRef_QPair2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQCborValueRefQCborValueRef>>::cast_into(other).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Swaps this pair with <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QPair<QCborValueRef, QCborValueRef>::swap(QPair<QCborValueRef, QCborValueRef>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps this pair with <i>other</i>.</p>
    /// <p>Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms-obsolete.html#qSwap-1">qSwap</a>(<span class="keyword">this</span><span class="operator">-</span><span class="operator">&gt;</span>first<span class="operator">,</span> other<span class="operator">.</span>first);
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms-obsolete.html#qSwap-1">qSwap</a>(<span class="keyword">this</span><span class="operator">-</span><span class="operator">&gt;</span>second<span class="operator">,</span> other<span class="operator">.</span>second);
    ///
    /// </pre>
    /// <p>Swap overloads are found in namespace <code>std</code> as well as via argument-dependent lookup (ADL) in the namespace of <code>T</code> .</p>
    /// <p>This function was introduced in  Qt 5.5.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QPairOfQCborValueRefQCborValueRef>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QPair_QCborValueRef_QCborValueRef_swap(self as *mut crate::QPairOfQCborValueRefQCborValueRef, ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QPairOfQCborValueRefQCborValueRef>>::cast_into(other).as_mut_raw_ptr())
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
///
/// C++ class: <span style='color: green;'>```QPair<QCborValue, QCborValue>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qpair.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
/// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; can be used in your application if the STL <code>pair</code> type is not available. It stores one value of type T1 and one value of type T2. It can be used as a return value for a function that needs to return two values, or as the value type of a <a href="http://doc.qt.io/qt-5/containers.html">generic container</a>.</p>
/// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> that stores one <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and one <code>double</code> value:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qpair.html#QPair">QPair</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> pair;
///
/// </pre>
/// <p>The components are accessible as public data members called <a href="http://doc.qt.io/qt-5/qpair.html#first-var">first</a> and <a href="http://doc.qt.io/qt-5/qpair.html#second-var">second</a>. For example:</p>
/// <pre class="cpp">
///
///   pair<span class="operator">.</span>first <span class="operator">=</span> <span class="string">"pi"</span>;
///   pair<span class="operator">.</span>second <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
///
/// </pre>
/// <p>Note, however, that it is almost always preferable to define a small struct to hold the result of a function with multiple return values. A struct trivially generalizes to more than two values, and allows more descriptive member names than <code>first</code> and <code>second</code>:</p>
/// <pre class="cpp">
///
///   <span class="keyword">struct</span> Variable {
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> name;
/// &#32;     <span class="type">double</span> value;
///   };
///   Variable v;
///   v<span class="operator">.</span>name <span class="operator">=</span> <span class="string">"pi"</span>;
///   v<span class="operator">.</span>value <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
///
/// </pre>
/// <p>The advent of C++11 automatic variable type deduction (<code>auto</code>) shifts the emphasis from the type name to the name of functions and members. Thus, <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>, like <code>std::pair</code> and <code>std::tuple</code>, is mostly useful in generic (template) code, where defining a dedicated type is not possible.</p>
/// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>'s template data types (T1 and T2) must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; these requirements are documented on a per-function basis.</p></div>
#[repr(C)]
pub struct QPairOfQCborValueQCborValue {
    _unused: u8,
}
impl QPairOfQCborValueQCborValue {
    /// <p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPair<QCborValue, QCborValue>& QPair<QCborValue, QCborValue>::operator=(const QPair<QCborValue, QCborValue>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; can be used in your application if the STL <code>pair</code> type is not available. It stores one value of type T1 and one value of type T2. It can be used as a return value for a function that needs to return two values, or as the value type of a <a href="http://doc.qt.io/qt-5/containers.html">generic container</a>.</p>
    /// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> that stores one <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and one <code>double</code> value:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpair.html#QPair">QPair</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> pair;
    ///
    /// </pre>
    /// <p>The components are accessible as public data members called <a href="http://doc.qt.io/qt-5/qpair.html#first-var">first</a> and <a href="http://doc.qt.io/qt-5/qpair.html#second-var">second</a>. For example:</p>
    /// <pre class="cpp">
    ///
    ///   pair<span class="operator">.</span>first <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   pair<span class="operator">.</span>second <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>Note, however, that it is almost always preferable to define a small struct to hold the result of a function with multiple return values. A struct trivially generalizes to more than two values, and allows more descriptive member names than <code>first</code> and <code>second</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">struct</span> Variable {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> name;
    /// &#32;     <span class="type">double</span> value;
    ///   };
    ///   Variable v;
    ///   v<span class="operator">.</span>name <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   v<span class="operator">.</span>value <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>The advent of C++11 automatic variable type deduction (<code>auto</code>) shifts the emphasis from the type name to the name of functions and members. Thus, <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>, like <code>std::pair</code> and <code>std::tuple</code>, is mostly useful in generic (template) code, where defining a dedicated type is not possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>'s template data types (T1 and T2) must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; these requirements are documented on a per-function basis.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQCborValueQCborValue>>,
    ) -> ::cpp_utils::MutRef<crate::QPairOfQCborValueQCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPair_QCborValue_QCborValue_operator_(self as *mut crate::QPairOfQCborValueQCborValue, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQCborValueQCborValue>>::cast_into(other).as_raw_ptr());
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Constructs an empty pair. The <code>first</code> and <code>second</code> elements are initialized with <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>s.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPair<QCborValue, QCborValue>::QPair()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html#QPair">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty pair. The <code>first</code> and <code>second</code> elements are initialized with <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>s.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QPairOfQCborValueQCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPair_QCborValue_QCborValue_QPair();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a pair and initializes the <code>first</code> element with <i>value1</i> and the <code>second</code> element with <i>value2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPair<QCborValue, QCborValue>::QPair(const QCborValue& t1, const QCborValue& t2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html#QPair-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a pair and initializes the <code>first</code> element with <i>value1</i> and the <code>second</code> element with <i>value2</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpair.html#qMakePair">qMakePair</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn new_2a(
        t1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
        t2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::cpp_utils::CppBox<crate::QPairOfQCborValueQCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPair_QCborValue_QCborValue_QPair1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(t1)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(t2)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPair<QCborValue, QCborValue>::QPair(const QPair<QCborValue, QCborValue>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; can be used in your application if the STL <code>pair</code> type is not available. It stores one value of type T1 and one value of type T2. It can be used as a return value for a function that needs to return two values, or as the value type of a <a href="http://doc.qt.io/qt-5/containers.html">generic container</a>.</p>
    /// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> that stores one <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and one <code>double</code> value:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpair.html#QPair">QPair</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> pair;
    ///
    /// </pre>
    /// <p>The components are accessible as public data members called <a href="http://doc.qt.io/qt-5/qpair.html#first-var">first</a> and <a href="http://doc.qt.io/qt-5/qpair.html#second-var">second</a>. For example:</p>
    /// <pre class="cpp">
    ///
    ///   pair<span class="operator">.</span>first <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   pair<span class="operator">.</span>second <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>Note, however, that it is almost always preferable to define a small struct to hold the result of a function with multiple return values. A struct trivially generalizes to more than two values, and allows more descriptive member names than <code>first</code> and <code>second</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">struct</span> Variable {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> name;
    /// &#32;     <span class="type">double</span> value;
    ///   };
    ///   Variable v;
    ///   v<span class="operator">.</span>name <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   v<span class="operator">.</span>value <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>The advent of C++11 automatic variable type deduction (<code>auto</code>) shifts the emphasis from the type name to the name of functions and members. Thus, <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>, like <code>std::pair</code> and <code>std::tuple</code>, is mostly useful in generic (template) code, where defining a dedicated type is not possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>'s template data types (T1 and T2) must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; these requirements are documented on a per-function basis.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQCborValueQCborValue>>,
    ) -> ::cpp_utils::CppBox<crate::QPairOfQCborValueQCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPair_QCborValue_QCborValue_QPair2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQCborValueQCborValue>>::cast_into(other).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Swaps this pair with <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QPair<QCborValue, QCborValue>::swap(QPair<QCborValue, QCborValue>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps this pair with <i>other</i>.</p>
    /// <p>Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms-obsolete.html#qSwap-1">qSwap</a>(<span class="keyword">this</span><span class="operator">-</span><span class="operator">&gt;</span>first<span class="operator">,</span> other<span class="operator">.</span>first);
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms-obsolete.html#qSwap-1">qSwap</a>(<span class="keyword">this</span><span class="operator">-</span><span class="operator">&gt;</span>second<span class="operator">,</span> other<span class="operator">.</span>second);
    ///
    /// </pre>
    /// <p>Swap overloads are found in namespace <code>std</code> as well as via argument-dependent lookup (ADL) in the namespace of <code>T</code> .</p>
    /// <p>This function was introduced in  Qt 5.5.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QPairOfQCborValueQCborValue>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QPair_QCborValue_QCborValue_swap(self as *mut crate::QPairOfQCborValueQCborValue, ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QPairOfQCborValueQCborValue>>::cast_into(other).as_mut_raw_ptr())
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> class is a template class that provides a dynamic array.</p>
///
/// C++ class: <span style='color: green;'>```QVector<QCborValue>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qvector.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> class is a template class that provides a dynamic array.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores its items in adjacent memory locations and provides fast index-based access.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Here's an example of a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> that stores <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerVector;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> stringVector;
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> stores its items in a vector (array). Typically, vectors are created with an initial size. For example, the following code constructs a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> with 200 elements:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">200</span>);
///
/// </pre>
/// <p>The elements are automatically initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you want to initialize the vector with a different value, pass that value as the second argument to the constructor:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">200</span><span class="operator">,</span> <span class="string">"Pass"</span>);
///
/// </pre>
/// <p>You can also call <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>() at any time to fill the vector with a value.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const vectors, operator[]() returns a reference to the item that can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (vector<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Liz"</span>)
/// &#32;     vector<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Elizabeth"</span>;
///
/// </pre>
/// <p>For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> vector<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (vector<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Alfonso"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Alfonso at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>Another way to access the data stored in a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> is to call <a href="http://doc.qt.io/qt-5/qvector.html#data">data</a>(). The function returns a pointer to the first item in the vector. You can use the pointer to directly access and modify the elements stored in the vector. The pointer is also useful if you need to pass a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to a function that accepts a plain C++ array.</p>
/// <p>If you want to find all occurrences of a particular value in a vector, use <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of the matching item if they found one; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> vector<span class="operator">.</span>indexOf(<span class="string">"Harumi"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Harumi is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a vector contains a particular value, use <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the vector, use <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(). With the exception of <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), these functions can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>) for large vectors, because they require moving many items in the vector by one position in memory. If you want a container class that provides fast insertion/removal in the middle, use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> or <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
/// <p>Unlike plain C++ arrays, QVectors can be resized at any time by calling <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>(). If the new size is larger than the old size, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> might need to reallocate the whole vector. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> tries to reduce the number of reallocations by preallocating up to twice as much memory as the actual data needs.</p>
/// <p>If you know in advance approximately how many items the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> will contain, you can call <a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>(), asking <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to preallocate a certain amount of memory. You can also call <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>() to find out how much memory <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> actually allocated.</p>
/// <p>Note that using non-const operators and functions can cause <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to do a deep copy of the data. This is due to <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicit sharing</a>.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qvectoriterator.html">QVectorIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablevectoriterator.html">QMutableVectorIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qvector.html#const_iterator-typedef">QVector::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qvector.html#iterator-typedefx">QVector::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>.</p>
/// <p>In addition to <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>, Qt also provides <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a>, a very low-level class with little functionality that is optimized for speed.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <a name="more-information-on-using-qt-containers"></a>
/// <h4>More Information on Using Qt Containers</h4>
/// <p>For a detailed discussion comparing Qt containers with each other and with STL containers, see <a href="http://marcmutz.wordpress.com/effective-qt/containers/">Understand the Qt Containers</a>.</p></div>
#[repr(C)]
pub struct QVectorOfQCborValue {
    _unused: u8,
}
impl QVectorOfQCborValue {
    /// <p>Inserts <i>value</i> at the end of the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QCborValue>::append(const QCborValue& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   vector<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> three <span class="operator">=</span> <span class="string">"three"</span>;
    ///   vector<span class="operator">.</span>append(three);
    ///   <span class="comment">// vector: ["one", "two", "three"]</span>
    ///   <span class="comment">// three: "three"</span>
    ///
    /// </pre>
    /// <p>This is the same as calling resize(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() + 1) and assigning <i>value</i> to the new last element in the vector.</p>
    /// <p>This operation is relatively fast, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> typically allocates more memory than necessary, so it can grow without reallocating the entire vector each time.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn append_q_cbor_value(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_append(
            self as *mut crate::QVectorOfQCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QCborValue>::append(const QVector<QCborValue>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#append-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> vector to this vector.</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn append_q_vector_of_q_cbor_value(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQCborValue>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_append2(
            self as *mut crate::QVectorOfQCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQCborValue>>::cast_into(l)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the item at index position <i>i</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QCborValue& QVector<QCborValue>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the vector.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn at(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_at(
            self as *const crate::QVectorOfQCborValue,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue& QVector<QCborValue>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::MutRef<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_back(
            self as *mut crate::QVectorOfQCborValue,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QCborValue& QVector<QCborValue>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_back1(
            self as *const crate::QVectorOfQCborValue,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the maximum number of items that can be stored in the vector without forcing a reallocation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QCborValue>::capacity() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#capacity">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the maximum number of items that can be stored in the vector without forcing a reallocation.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s memory usage. In general, you will rarely ever need to call this function. If you want to know how many items are in the vector, call <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn capacity(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_capacity(
            self as *const crate::QVectorOfQCborValue,
        )
    }

    /// <p>Removes all the elements from the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QCborValue>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all the elements from the vector.</p>
    /// <p><b>Note: </b>Until Qt 5.6, this also released the memory used by the vector. From Qt 5.7, the capacity is preserved. To shed all capacity, swap with a default-constructed vector:</p><pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span>T<span class="operator">&gt;</span> v <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span>T<span class="operator">&gt;</span>()<span class="operator">.</span>swap(v);
    ///   Q_ASSERT(v<span class="operator">.</span>capacity() <span class="operator">=</span><span class="operator">=</span> <span class="number">0</span>);
    ///
    /// </pre>
    /// <p>or call <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_clear(
            self as *mut crate::QVectorOfQCborValue,
        )
    }

    /// <p>Returns a const pointer to the data stored in the vector. The pointer can be used to access the items in the vector. The pointer remains valid as long as the vector isn't reallocated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QCborValue* QVector<QCborValue>::constData() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const pointer to the data stored in the vector. The pointer can be used to access the items in the vector. The pointer remains valid as long as the vector isn't reallocated.</p>
    /// <p>This function is mostly useful to pass a vector to a function that accepts a plain C++ array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#data">data</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn const_data(&self) -> ::cpp_utils::Ptr<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_constData(
            self as *const crate::QVectorOfQCborValue,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns a const reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QCborValue& QVector<QCborValue>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_constFirst(
            self as *const crate::QVectorOfQCborValue,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QCborValue& QVector<QCborValue>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_constLast(
            self as *const crate::QVectorOfQCborValue,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the vector contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QCborValue>::contains(const QCborValue& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the vector contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn contains(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_contains(
            self as *const crate::QVectorOfQCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QCborValue>& QVector<QCborValue>::operator=(const QVector<QCborValue>& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this vector and returns a reference to this vector.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn copy_from(
        &mut self,
        v: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQCborValue>>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_operator_(
            self as *mut crate::QVectorOfQCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQCborValue>>::cast_into(v)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of occurrences of <i>value</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QCborValue>::count(const QCborValue& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of occurrences of <i>value</i> in the vector.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn count_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_count(
            self as *const crate::QVectorOfQCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QCborValue>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_count1(
            self as *const crate::QVectorOfQCborValue,
        )
    }

    /// <p>Returns a pointer to the data stored in the vector. The pointer can be used to access and modify the items in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue* QVector<QCborValue>::data()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the data stored in the vector. The pointer can be used to access and modify the items in the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> vector(<span class="number">10</span>);
    ///   <span class="type">int</span> <span class="operator">*</span>data <span class="operator">=</span> vector<span class="operator">.</span>data();
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> <span class="number">10</span>; <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     data<span class="operator">[</span>i<span class="operator">]</span> <span class="operator">=</span> <span class="number">2</span> <span class="operator">*</span> i;
    ///
    /// </pre>
    /// <p>The pointer remains valid as long as the vector isn't reallocated.</p>
    /// <p>This function is mostly useful to pass a vector to a function that accepts a plain C++ array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constData">constData</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn data_mut(&mut self) -> ::cpp_utils::MutPtr<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_data(
            self as *mut crate::QVectorOfQCborValue,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QCborValue* QVector<QCborValue>::data() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#data-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn data(&self) -> ::cpp_utils::Ptr<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_data1(
            self as *const crate::QVectorOfQCborValue,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```void QVector<QCborValue>::detach()```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_detach(
            self as *mut crate::QVectorOfQCborValue,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), returning <code>true</code> if the vector is empty; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QCborValue>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), returning <code>true</code> if the vector is empty; otherwise returns <code>false</code>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_empty(
            self as *const crate::QVectorOfQCborValue,
        )
    }

    /// <p>Returns <code>true</code> if this vector is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QCborValue>::endsWith(const QCborValue& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this vector is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn ends_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_endsWith(
            self as *const crate::QVectorOfQCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QCborValue>& QVector<QCborValue>::fill(const QCborValue& t, int size = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">3</span>);
    ///   vector<span class="operator">.</span>fill(<span class="string">"Yes"</span>);
    ///   <span class="comment">// vector: ["Yes", "Yes", "Yes"]</span>
    ///
    ///   vector<span class="operator">.</span>fill(<span class="string">"oh"</span><span class="operator">,</span> <span class="number">5</span>);
    ///   <span class="comment">// vector: ["oh", "oh", "oh", "oh", "oh"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn fill_2a(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_fill(
            self as *mut crate::QVectorOfQCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(t).as_raw_ptr(),
            size,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QCborValue>& QVector<QCborValue>::fill(const QCborValue& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">3</span>);
    ///   vector<span class="operator">.</span>fill(<span class="string">"Yes"</span>);
    ///   <span class="comment">// vector: ["Yes", "Yes", "Yes"]</span>
    ///
    ///   vector<span class="operator">.</span>fill(<span class="string">"oh"</span><span class="operator">,</span> <span class="number">5</span>);
    ///   <span class="comment">// vector: ["oh", "oh", "oh", "oh", "oh"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn fill_1a(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_fill1(
            self as *mut crate::QVectorOfQCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(t).as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue& QVector<QCborValue>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#constFirst">constFirst</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_first(
            self as *mut crate::QVectorOfQCborValue,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QCborValue& QVector<QCborValue>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_first1(
            self as *const crate::QVectorOfQCborValue,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue& QVector<QCborValue>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::MutRef<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_front(
            self as *mut crate::QVectorOfQCborValue,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QCborValue& QVector<QCborValue>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_front1(
            self as *const crate::QVectorOfQCborValue,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue& QVector<QCborValue>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p>Note that using non-const operators can cause <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to do a deep copy.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#value">value</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_operator__2(
            self as *mut crate::QVectorOfQCborValue,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QCborValue& QVector<QCborValue>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as at(<i>i</i>).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn index(&self, i: ::std::os::raw::c_int) -> ::cpp_utils::Ref<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_operator__3(
            self as *const crate::QVectorOfQCborValue,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QCborValue>::indexOf(const QCborValue& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span>);            <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);         <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);         <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"X"</span>);            <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_indexOf(
            self as *const crate::QVectorOfQCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(t).as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QCborValue>::indexOf(const QCborValue& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span>);            <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);         <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);         <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"X"</span>);            <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_indexOf1(
            self as *const crate::QVectorOfQCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the vector. If <i>i</i> is 0, the value is prepended to the vector. If <i>i</i> is <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), the value is appended to the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QCborValue>::insert(int i, const QCborValue& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the vector. If <i>i</i> is 0, the value is prepended to the vector. If <i>i</i> is <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), the value is appended to the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   vector<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// vector: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p>For large vectors, this operation can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because it requires moving all the items at indexes <i>i</i> and above by one position further in memory. If you want a container class that provides a fast insert() function, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn insert_2a(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_insert(
            self as *mut crate::QVectorOfQCborValue,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QCborValue>::insert(int i, int n, const QCborValue& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>count</i> copies of <i>value</i> at index position <i>i</i> in the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">double</span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">2.718</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">1.442</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">0.4342</span>;
    ///   vector<span class="operator">.</span>insert(<span class="number">1</span><span class="operator">,</span> <span class="number">3</span><span class="operator">,</span> <span class="number">9.9</span>);
    ///   <span class="comment">// vector: [2.718, 9.9, 9.9, 9.9, 1.442, 0.4342]</span>
    ///
    /// </pre></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn insert_3a(
        &mut self,
        i: ::std::os::raw::c_int,
        n: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_insert1(
            self as *mut crate::QVectorOfQCborValue,
            i,
            n,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QVector<QCborValue>::isDetached() const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_isDetached(
            self as *const crate::QVectorOfQCborValue,
        )
    }

    /// <p>Returns <code>true</code> if the vector has size 0; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QCborValue>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the vector has size 0; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_isEmpty(
            self as *const crate::QVectorOfQCborValue,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QVector<QCborValue>::isSharedWith(const QVector<QCborValue>& other) const```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQCborValue>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_isSharedWith(
            self as *const crate::QVectorOfQCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQCborValue>>::cast_into(other)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns a reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue& QVector<QCborValue>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#constLast">constLast</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_last(
            self as *mut crate::QVectorOfQCborValue,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QCborValue& QVector<QCborValue>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_last1(
            self as *const crate::QVectorOfQCborValue,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QCborValue>::lastIndexOf(const QCborValue& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);        <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);     <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);     <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn last_index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_lastIndexOf(
            self as *const crate::QVectorOfQCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(t).as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QCborValue>::lastIndexOf(const QCborValue& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);        <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);     <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);     <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn last_index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_lastIndexOf1(
            self as *const crate::QVectorOfQCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QCborValue>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#length">QList::length</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_length(
            self as *const crate::QVectorOfQCborValue,
        )
    }

    /// <p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QCborValue> QVector<QCborValue>::mid(int pos, int len = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        len: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_mid(
            self as *const crate::QVectorOfQCborValue,
            pos,
            len,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QCborValue> QVector<QCborValue>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_mid1(
            self as *const crate::QVectorOfQCborValue,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QCborValue>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">QList::move</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_move(
            self as *mut crate::QVectorOfQCborValue,
            from,
            to,
        )
    }

    /// <p>Constructs an empty vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QCborValue>::QVector()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QVectorOfQCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_QVector();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a vector with an initial size of <i>size</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QCborValue>::QVector(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a vector with an initial size of <i>size</i> elements.</p>
    /// <p>The elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn new_1a(
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_QVector1(size);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a vector with an initial size of <i>size</i> elements. Each element is initialized with <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QCborValue>::QVector(int size, const QCborValue& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a vector with an initial size of <i>size</i> elements. Each element is initialized with <i>value</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn new_2a(
        size: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_QVector2(
            size,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(t).as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QCborValue>::QVector(const QVector<QCborValue>& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-eq">operator=</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn new_copy(
        v: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQCborValue>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_QVector3(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQCborValue>>::cast_into(v)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QCborValue>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_pop_back(
            self as *mut crate::QVectorOfQCborValue,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QCborValue>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_pop_front(
            self as *mut crate::QVectorOfQCborValue,
        )
    }

    /// <p>Inserts <i>value</i> at the beginning of the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QCborValue>::prepend(const QCborValue& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   vector<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   vector<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// vector: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as vector.insert(0, <i>value</i>).</p>
    /// <p>For large vectors, this operation can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because it requires moving all the items in the vector by one position further in memory. If you want a container class that provides a fast prepend() function, use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> or <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_prepend(
            self as *mut crate::QVectorOfQCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to append(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QCborValue>::push_back(const QCborValue& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to append(<i>value</i>).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_push_back(
            self as *mut crate::QVectorOfQCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to prepend(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QCborValue>::push_front(const QCborValue& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to prepend(<i>value</i>).</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_push_front(
            self as *mut crate::QVectorOfQCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QCborValue>::remove(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#remove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes the element at index position <i>i</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn remove_1a(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_remove(
            self as *mut crate::QVectorOfQCborValue,
            i,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QCborValue>::remove(int i, int n)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#remove-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes <i>count</i> elements from the middle of the vector, starting at index position <i>i</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn remove_2a(&mut self, i: ::std::os::raw::c_int, n: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_remove1(
            self as *mut crate::QVectorOfQCborValue,
            i,
            n,
        )
    }

    /// <p>Removes all elements that compare equal to <i>t</i> from the vector. Returns the number of elements removed, if any.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QCborValue>::removeAll(const QCborValue& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all elements that compare equal to <i>t</i> from the vector. Returns the number of elements removed, if any.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#removeOne">removeOne</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">QList::removeAll</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn remove_all(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_removeAll(
            self as *mut crate::QVectorOfQCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Removes the element at index position <i>i</i>. Equivalent to</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QCborValue>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the element at index position <i>i</i>. Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   remove(i);
    ///
    /// </pre>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">QList::removeAt</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_removeAt(
            self as *mut crate::QVectorOfQCborValue,
            i,
        )
    }

    /// <p>Removes the first item in the vector. Calling this function is equivalent to calling remove(0). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QCborValue>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the vector. Calling this function is equivalent to calling remove(0). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_removeFirst(
            self as *mut crate::QVectorOfQCborValue,
        )
    }

    /// <p>Removes the last item in the vector. Calling this function is equivalent to calling remove(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() - 1). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QCborValue>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the vector. Calling this function is equivalent to calling remove(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() - 1). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_removeLast(
            self as *mut crate::QVectorOfQCborValue,
        )
    }

    /// <p>Removes the first element that compares equal to <i>t</i> from the vector. Returns whether an element was, in fact, removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QCborValue>::removeOne(const QCborValue& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeOne">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first element that compares equal to <i>t</i> from the vector. Returns whether an element was, in fact, removed.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#removeAll">removeAll</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">QList::removeOne</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn remove_one(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_removeOne(
            self as *mut crate::QVectorOfQCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QCborValue>::replace(int i, const QCborValue& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_replace(
            self as *mut crate::QVectorOfQCborValue,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Attempts to allocate memory for at least <i>size</i> elements. If you know in advance how large the vector will be, you should call this function to prevent reallocations and memory fragmentation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QCborValue>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to allocate memory for at least <i>size</i> elements. If you know in advance how large the vector will be, you should call this function to prevent reallocations and memory fragmentation.</p>
    /// <p>If <i>size</i> is an underestimate, the worst that will happen is that the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> will be a bit slower. If <i>size</i> is an overestimate, you may have used more memory than the normal <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> growth strategy would have allocated—or you may have used less.</p>
    /// <p>An alternative to reserve() is calling <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>(). Whether or not that is faster than reserve() depends on the element type, because <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() default-constructs all elements, and requires assignment to existing entries rather than calling <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(), which copy- or move-constructs. For simple types, like <code>int</code> or <code>double</code>, <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() is typically faster, but for anything more complex, you should prefer reserve().</p>
    /// <p><b>Warning:</b> If the size passed to <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() was underestimated, you run out of allocated space and into undefined behavior. This problem does not exist with reserve(), because it treats the size as just a hint.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_reserve(
            self as *mut crate::QVectorOfQCborValue,
            size,
        )
    }

    /// <p>Sets the size of the vector to <i>size</i>. If <i>size</i> is greater than the current size, elements are added to the end; the new elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If <i>size</i> is less than the current size, elements are removed from the end.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QCborValue>::resize(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#resize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the size of the vector to <i>size</i>. If <i>size</i> is greater than the current size, elements are added to the end; the new elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If <i>size</i> is less than the current size, elements are removed from the end.</p>
    /// <p>Since Qt 5.6, resize() doesn't shrink the capacity anymore. To shed excess capacity, use <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn resize(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_resize(
            self as *mut crate::QVectorOfQCborValue,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QVector<QCborValue>::setSharable(bool sharable)```</span>.
    ///
    ///
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_setSharable(
            self as *mut crate::QVectorOfQCborValue,
            sharable,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QCborValue>::shrink_to_fit()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#shrink_to_fit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn shrink_to_fit(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_shrink_to_fit(
            self as *mut crate::QVectorOfQCborValue,
        )
    }

    /// <p>Returns the number of items in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QCborValue>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_size(
            self as *const crate::QVectorOfQCborValue,
        )
    }

    /// <p>Releases any memory not required to store the items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QCborValue>::squeeze()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Releases any memory not required to store the items.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s memory usage. In general, you will rarely ever need to call this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn squeeze(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_squeeze(
            self as *mut crate::QVectorOfQCborValue,
        )
    }

    /// <p>Returns <code>true</code> if this vector is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QCborValue>::startsWith(const QCborValue& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this vector is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn starts_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_startsWith(
            self as *const crate::QVectorOfQCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(t).as_raw_ptr(),
        )
    }

    /// <p>Swaps vector <i>other</i> with this vector. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QCborValue>::swap(QVector<QCborValue>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps vector <i>other</i> with this vector. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QVectorOfQCborValue>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_swap(
            self as *mut crate::QVectorOfQCborValue,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QVectorOfQCborValue>>::cast_into(
                other,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Removes the element at index position <i>i</i> and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QVector<QCborValue>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the element at index position <i>i</i> and returns it.</p>
    /// <p>Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   T t <span class="operator">=</span> at(i);
    ///   remove(i);
    ///   <span class="keyword">return</span> t;
    ///
    /// </pre>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">QList::takeAt</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_takeAt(
            self as *mut crate::QVectorOfQCborValue,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the first item in the vector and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QVector<QCborValue>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the vector and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn take_first(&mut self) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_takeFirst(
            self as *mut crate::QVectorOfQCborValue,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the last item in the list and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QVector<QCborValue>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn take_last(&mut self) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_takeLast(
            self as *mut crate::QVectorOfQCborValue,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value at index position <i>i</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QVector<QCborValue>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the vector.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that <i>i</i> is within bounds, you can use <a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn value_1a(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_value(
            self as *const crate::QVectorOfQCborValue,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborValue QVector<QCborValue>::value(int i, const QCborValue& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn value_2a(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    ) -> ::cpp_utils::CppBox<crate::QCborValue> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_value1(
            self as *const crate::QVectorOfQCborValue,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(default_value)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
///
/// C++ class: <span style='color: green;'>```QPair<qint64, unsigned int>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qpair.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
/// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; can be used in your application if the STL <code>pair</code> type is not available. It stores one value of type T1 and one value of type T2. It can be used as a return value for a function that needs to return two values, or as the value type of a <a href="http://doc.qt.io/qt-5/containers.html">generic container</a>.</p>
/// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> that stores one <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and one <code>double</code> value:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qpair.html#QPair">QPair</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> pair;
///
/// </pre>
/// <p>The components are accessible as public data members called <a href="http://doc.qt.io/qt-5/qpair.html#first-var">first</a> and <a href="http://doc.qt.io/qt-5/qpair.html#second-var">second</a>. For example:</p>
/// <pre class="cpp">
///
///   pair<span class="operator">.</span>first <span class="operator">=</span> <span class="string">"pi"</span>;
///   pair<span class="operator">.</span>second <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
///
/// </pre>
/// <p>Note, however, that it is almost always preferable to define a small struct to hold the result of a function with multiple return values. A struct trivially generalizes to more than two values, and allows more descriptive member names than <code>first</code> and <code>second</code>:</p>
/// <pre class="cpp">
///
///   <span class="keyword">struct</span> Variable {
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> name;
/// &#32;     <span class="type">double</span> value;
///   };
///   Variable v;
///   v<span class="operator">.</span>name <span class="operator">=</span> <span class="string">"pi"</span>;
///   v<span class="operator">.</span>value <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
///
/// </pre>
/// <p>The advent of C++11 automatic variable type deduction (<code>auto</code>) shifts the emphasis from the type name to the name of functions and members. Thus, <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>, like <code>std::pair</code> and <code>std::tuple</code>, is mostly useful in generic (template) code, where defining a dedicated type is not possible.</p>
/// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>'s template data types (T1 and T2) must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; these requirements are documented on a per-function basis.</p></div>
#[repr(C)]
pub struct QPairOfI64Uint {
    _unused: u8,
}
impl QPairOfI64Uint {
    /// <p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPair<qint64, unsigned int>& QPair<qint64, unsigned int>::operator=(const QPair<qint64, unsigned int>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; can be used in your application if the STL <code>pair</code> type is not available. It stores one value of type T1 and one value of type T2. It can be used as a return value for a function that needs to return two values, or as the value type of a <a href="http://doc.qt.io/qt-5/containers.html">generic container</a>.</p>
    /// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> that stores one <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and one <code>double</code> value:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpair.html#QPair">QPair</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> pair;
    ///
    /// </pre>
    /// <p>The components are accessible as public data members called <a href="http://doc.qt.io/qt-5/qpair.html#first-var">first</a> and <a href="http://doc.qt.io/qt-5/qpair.html#second-var">second</a>. For example:</p>
    /// <pre class="cpp">
    ///
    ///   pair<span class="operator">.</span>first <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   pair<span class="operator">.</span>second <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>Note, however, that it is almost always preferable to define a small struct to hold the result of a function with multiple return values. A struct trivially generalizes to more than two values, and allows more descriptive member names than <code>first</code> and <code>second</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">struct</span> Variable {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> name;
    /// &#32;     <span class="type">double</span> value;
    ///   };
    ///   Variable v;
    ///   v<span class="operator">.</span>name <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   v<span class="operator">.</span>value <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>The advent of C++11 automatic variable type deduction (<code>auto</code>) shifts the emphasis from the type name to the name of functions and members. Thus, <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>, like <code>std::pair</code> and <code>std::tuple</code>, is mostly useful in generic (template) code, where defining a dedicated type is not possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>'s template data types (T1 and T2) must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; these requirements are documented on a per-function basis.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn copy_from(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfI64Uint>>,
    ) -> ::cpp_utils::MutRef<crate::QPairOfI64Uint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPair_qint64_unsigned_int_operator_(
            self as *mut crate::QPairOfI64Uint,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfI64Uint>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Constructs an empty pair. The <code>first</code> and <code>second</code> elements are initialized with <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>s.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPair<qint64, unsigned int>::QPair()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html#QPair">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty pair. The <code>first</code> and <code>second</code> elements are initialized with <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>s.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QPairOfI64Uint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPair_qint64_unsigned_int_QPair();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a pair and initializes the <code>first</code> element with <i>value1</i> and the <code>second</code> element with <i>value2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPair<qint64, unsigned int>::QPair(const qint64& t1, const unsigned int& t2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html#QPair-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a pair and initializes the <code>first</code> element with <i>value1</i> and the <code>second</code> element with <i>value2</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpair.html#qMakePair">qMakePair</a>().</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn new_2a(
        t1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<i64>>,
        t2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_uint>>,
    ) -> ::cpp_utils::CppBox<crate::QPairOfI64Uint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPair_qint64_unsigned_int_QPair1(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<i64>>::cast_into(t1).as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_uint>>::cast_into(t2)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QPair<qint64, unsigned int>::QPair(const QPair<qint64, unsigned int>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; can be used in your application if the STL <code>pair</code> type is not available. It stores one value of type T1 and one value of type T2. It can be used as a return value for a function that needs to return two values, or as the value type of a <a href="http://doc.qt.io/qt-5/containers.html">generic container</a>.</p>
    /// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> that stores one <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and one <code>double</code> value:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpair.html#QPair">QPair</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> pair;
    ///
    /// </pre>
    /// <p>The components are accessible as public data members called <a href="http://doc.qt.io/qt-5/qpair.html#first-var">first</a> and <a href="http://doc.qt.io/qt-5/qpair.html#second-var">second</a>. For example:</p>
    /// <pre class="cpp">
    ///
    ///   pair<span class="operator">.</span>first <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   pair<span class="operator">.</span>second <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>Note, however, that it is almost always preferable to define a small struct to hold the result of a function with multiple return values. A struct trivially generalizes to more than two values, and allows more descriptive member names than <code>first</code> and <code>second</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">struct</span> Variable {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> name;
    /// &#32;     <span class="type">double</span> value;
    ///   };
    ///   Variable v;
    ///   v<span class="operator">.</span>name <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   v<span class="operator">.</span>value <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>The advent of C++11 automatic variable type deduction (<code>auto</code>) shifts the emphasis from the type name to the name of functions and members. Thus, <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>, like <code>std::pair</code> and <code>std::tuple</code>, is mostly useful in generic (template) code, where defining a dedicated type is not possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>'s template data types (T1 and T2) must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; these requirements are documented on a per-function basis.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn new_copy(
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfI64Uint>>,
    ) -> ::cpp_utils::CppBox<crate::QPairOfI64Uint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QPair_qint64_unsigned_int_QPair2(
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfI64Uint>>::cast_into(other)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Swaps this pair with <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QPair<qint64, unsigned int>::swap(QPair<qint64, unsigned int>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps this pair with <i>other</i>.</p>
    /// <p>Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms-obsolete.html#qSwap-1">qSwap</a>(<span class="keyword">this</span><span class="operator">-</span><span class="operator">&gt;</span>first<span class="operator">,</span> other<span class="operator">.</span>first);
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms-obsolete.html#qSwap-1">qSwap</a>(<span class="keyword">this</span><span class="operator">-</span><span class="operator">&gt;</span>second<span class="operator">,</span> other<span class="operator">.</span>second);
    ///
    /// </pre>
    /// <p>Swap overloads are found in namespace <code>std</code> as well as via argument-dependent lookup (ADL) in the namespace of <code>T</code> .</p>
    /// <p>This function was introduced in  Qt 5.5.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
        ritual_rustdoc
    ))]
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QPairOfI64Uint>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QPair_qint64_unsigned_int_swap(
            self as *mut crate::QPairOfI64Uint,
            ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QPairOfI64Uint>>::cast_into(other)
                .as_mut_raw_ptr(),
        )
    }
}

/// <p><a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html">QConcatenateTablesProxyModel</a> takes multiple source models and concatenates their rows.</p>
///
/// C++ class: <span style='color: green;'>```QConcatenateTablesProxyModel```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p><a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html">QConcatenateTablesProxyModel</a> takes multiple source models and concatenates their rows.</p>
/// <p>In other words, the proxy will have all rows of the first source model, followed by all rows of the second source model, and so on.</p>
/// <p>If the source models don't have the same number of columns, the proxy will only have as many columns as the source model with the smallest number of columns. Additional columns in other source models will simply be ignored.</p>
/// <p>Source models can be added and removed at runtime, and the column count is adjusted accordingly.</p>
/// <p>This proxy does not inherit from <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html">QAbstractProxyModel</a> because it uses multiple source models, rather than a single one.</p>
/// <p>Only flat models (lists and tables) are supported, tree models are not.</p></div>
#[repr(C)]
pub struct QConcatenateTablesProxyModel {
    _unused: u8,
}
impl QConcatenateTablesProxyModel {
    /// <p>Adds a source model <i>sourceModel</i>, below all previously added source models.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QConcatenateTablesProxyModel::addSourceModel(QAbstractItemModel* sourceModel)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#addSourceModel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds a source model <i>sourceModel</i>, below all previously added source models.</p>
    /// <p>The ownership of <i>sourceModel</i> is not affected by this.</p>
    /// <p>The same source model cannot be added more than once.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn add_source_model(
        &mut self,
        source_model: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractItemModel>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_addSourceModel(
            self as *mut crate::QConcatenateTablesProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractItemModel>>::cast_into(
                source_model,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#canDropMimeData">QAbstractItemModel::canDropMimeData</a>(const QMimeData *data, Qt::DropAction action, int row, int column, const QModelIndex &amp;parent) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QConcatenateTablesProxyModel::canDropMimeData(const QMimeData* data, Qt::DropAction action, int row, int column, const QModelIndex& parent) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#canDropMimeData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#canDropMimeData">QAbstractItemModel::canDropMimeData</a>(const QMimeData *data, Qt::DropAction action, int row, int column, const QModelIndex &amp;parent) const.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn can_drop_mime_data(
        &self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QMimeData>>,
        action: crate::DropAction,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_canDropMimeData(
            self as *const crate::QConcatenateTablesProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QMimeData>>::cast_into(data)
                .as_raw_ptr(),
            action,
            row,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnCount">QAbstractItemModel::columnCount</a>(const QModelIndex &amp;parent) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual int QConcatenateTablesProxyModel::columnCount(const QModelIndex& parent = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#columnCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnCount">QAbstractItemModel::columnCount</a>(const QModelIndex &amp;parent) const.</p>
    /// <p>This method returns the column count of the source model with the smallest number of columns.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn column_count_1a(
        &self,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_columnCount(
            self as *const crate::QConcatenateTablesProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnCount">QAbstractItemModel::columnCount</a>(const QModelIndex &amp;parent) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual int QConcatenateTablesProxyModel::columnCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#columnCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnCount">QAbstractItemModel::columnCount</a>(const QModelIndex &amp;parent) const.</p>
    /// <p>This method returns the column count of the source model with the smallest number of columns.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn column_count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_columnCount1(
            self as *const crate::QConcatenateTablesProxyModel,
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">QAbstractItemModel::data</a>(const QModelIndex &amp;index, int role) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QVariant QConcatenateTablesProxyModel::data(const QModelIndex& index, int role = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">QAbstractItemModel::data</a>(const QModelIndex &amp;index, int role) const.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#setData">setData</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn data_2a(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        role: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_data(
            self as *const crate::QConcatenateTablesProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
            role,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">QAbstractItemModel::data</a>(const QModelIndex &amp;index, int role) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QVariant QConcatenateTablesProxyModel::data(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#data">QAbstractItemModel::data</a>(const QModelIndex &amp;index, int role) const.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#setData">setData</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn data_1a(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_data1(
            self as *const crate::QConcatenateTablesProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dropMimeData">QAbstractItemModel::dropMimeData</a>(const QMimeData *data, Qt::DropAction action, int row, int column, const QModelIndex &amp;parent).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QConcatenateTablesProxyModel::dropMimeData(const QMimeData* data, Qt::DropAction action, int row, int column, const QModelIndex& parent)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#dropMimeData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dropMimeData">QAbstractItemModel::dropMimeData</a>(const QMimeData *data, Qt::DropAction action, int row, int column, const QModelIndex &amp;parent).</p>
    /// <p><a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html">QConcatenateTablesProxyModel</a> handles dropping onto an item, between items, and after the last item. In all cases the call is forwarded to the underlying source model. When dropping onto an item, the source model for this item is called. When dropping between items, the source model immediately below the drop position is called. When dropping after the last item, the last source model is called.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn drop_mime_data(
        &mut self,
        data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QMimeData>>,
        action: crate::DropAction,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_dropMimeData(
            self as *mut crate::QConcatenateTablesProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QMimeData>>::cast_into(data)
                .as_raw_ptr(),
            action,
            row,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#flags">QAbstractItemModel::flags</a>(const QModelIndex &amp;index) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QFlags<Qt::ItemFlag> QConcatenateTablesProxyModel::flags(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#flags">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#flags">QAbstractItemModel::flags</a>(const QModelIndex &amp;index) const.</p>
    /// <p>Returns the flags for the given index. If the <i>index</i> is valid, the flags come from the source model for this <i>index</i>. If the <i>index</i> is invalid (as used to determine if dropping onto an empty area in the view is allowed, for instance), the flags from the first model are returned.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn flags(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> crate::QFlags<crate::ItemFlag> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_flags(
            self as *const crate::QConcatenateTablesProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        );
        crate::QFlags::from(ffi_result)
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerData">QAbstractItemModel::headerData</a>(int section, Qt::Orientation orientation, int role) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QVariant QConcatenateTablesProxyModel::headerData(int section, Qt::Orientation orientation, int role = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#headerData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerData">QAbstractItemModel::headerData</a>(int section, Qt::Orientation orientation, int role) const.</p>
    /// <p>This method returns the horizontal header data for the first source model, and the vertical header data for the source model corresponding to each row.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn header_data_3a(
        &self,
        section: ::std::os::raw::c_int,
        orientation: crate::Orientation,
        role: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_headerData(
            self as *const crate::QConcatenateTablesProxyModel,
            section,
            orientation,
            role,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerData">QAbstractItemModel::headerData</a>(int section, Qt::Orientation orientation, int role) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QVariant QConcatenateTablesProxyModel::headerData(int section, Qt::Orientation orientation) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#headerData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#headerData">QAbstractItemModel::headerData</a>(int section, Qt::Orientation orientation, int role) const.</p>
    /// <p>This method returns the horizontal header data for the first source model, and the vertical header data for the source model corresponding to each row.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn header_data_2a(
        &self,
        section: ::std::os::raw::c_int,
        orientation: crate::Orientation,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_headerData1(
            self as *const crate::QConcatenateTablesProxyModel,
            section,
            orientation,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>(int row, int column, const QModelIndex &amp;parent) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QConcatenateTablesProxyModel::index(int row, int column, const QModelIndex& parent = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#index">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>(int row, int column, const QModelIndex &amp;parent) const.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn index_3a(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_index(
            self as *const crate::QConcatenateTablesProxyModel,
            row,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>(int row, int column, const QModelIndex &amp;parent) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QConcatenateTablesProxyModel::index(int row, int column) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#index">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>(int row, int column, const QModelIndex &amp;parent) const.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn index_2a(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_index1(
            self as *const crate::QConcatenateTablesProxyModel,
            row,
            column,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#itemData">QAbstractItemModel::itemData</a>(const QModelIndex &amp;index) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QMap<int, QVariant> QConcatenateTablesProxyModel::itemData(const QModelIndex& proxyIndex) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#itemData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#itemData">QAbstractItemModel::itemData</a>(const QModelIndex &amp;index) const.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#setItemData">setItemData</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn item_data(
        &self,
        proxy_index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QMapOfIntQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_itemData(
            self as *const crate::QConcatenateTablesProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(proxy_index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the proxy index for a given <i>sourceIndex</i>, which can be from any of the source models.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QModelIndex QConcatenateTablesProxyModel::mapFromSource(const QModelIndex& sourceIndex) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#mapFromSource">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the proxy index for a given <i>sourceIndex</i>, which can be from any of the source models.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn map_from_source(
        &self,
        source_index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_mapFromSource(
            self as *const crate::QConcatenateTablesProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(source_index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the source index for a given <i>proxyIndex</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QModelIndex QConcatenateTablesProxyModel::mapToSource(const QModelIndex& proxyIndex) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#mapToSource">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the source index for a given <i>proxyIndex</i>.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn map_to_source(
        &self,
        proxy_index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_mapToSource(
            self as *const crate::QConcatenateTablesProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(proxy_index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QConcatenateTablesProxyModel::metaObject() const```</span>.
    ///
    ///
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_metaObject(
            self as *const crate::QConcatenateTablesProxyModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeData">QAbstractItemModel::mimeData</a>(const QModelIndexList &amp;indexes) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QMimeData* QConcatenateTablesProxyModel::mimeData(const QList<QModelIndex>& indexes) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#mimeData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeData">QAbstractItemModel::mimeData</a>(const QModelIndexList &amp;indexes) const.</p>
    /// <p>The call is forwarded to the source model of the first index in the list of <i>indexes</i>.</p>
    /// <p>Important: please note that this proxy only supports dragging a single row. It will assert if called with indexes from multiple rows, because dragging rows that might come from different source models cannot be implemented generically by this proxy model. Each piece of data in the <a href="http://doc.qt.io/qt-5/qmimedata.html">QMimeData</a> needs to be merged, which is data-type-specific. Reimplement this method in a subclass if you want to support dragging multiple rows.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn mime_data(
        &self,
        indexes: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQModelIndex>>,
    ) -> ::cpp_utils::MutPtr<crate::QMimeData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_mimeData(
            self as *const crate::QConcatenateTablesProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQModelIndex>>::cast_into(
                indexes,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeTypes">QAbstractItemModel::mimeTypes</a>() const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QStringList QConcatenateTablesProxyModel::mimeTypes() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#mimeTypes">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#mimeTypes">QAbstractItemModel::mimeTypes</a>() const.</p>
    /// <p>This method returns the mime types for the first source model.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn mime_types(&self) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_mimeTypes(
            self as *const crate::QConcatenateTablesProxyModel,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a concatenate-rows proxy model with the given <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QConcatenateTablesProxyModel::QConcatenateTablesProxyModel(QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#QConcatenateTablesProxyModel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a concatenate-rows proxy model with the given <i>parent</i>.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn new_1a(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QConcatenateTablesProxyModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_QConcatenateTablesProxyModel(
                ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                    .as_mut_raw_ptr(),
            );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p><a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html">QConcatenateTablesProxyModel</a> takes multiple source models and concatenates their rows.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QConcatenateTablesProxyModel::QConcatenateTablesProxyModel()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p><a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html">QConcatenateTablesProxyModel</a> takes multiple source models and concatenates their rows.</p>
    /// <p>In other words, the proxy will have all rows of the first source model, followed by all rows of the second source model, and so on.</p>
    /// <p>If the source models don't have the same number of columns, the proxy will only have as many columns as the source model with the smallest number of columns. Additional columns in other source models will simply be ignored.</p>
    /// <p>Source models can be added and removed at runtime, and the column count is adjusted accordingly.</p>
    /// <p>This proxy does not inherit from <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html">QAbstractProxyModel</a> because it uses multiple source models, rather than a single one.</p>
    /// <p>Only flat models (lists and tables) are supported, tree models are not.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QConcatenateTablesProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_QConcatenateTablesProxyModel1();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#parent">QAbstractItemModel::parent</a>(const QModelIndex &amp;index) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QConcatenateTablesProxyModel::parent(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#parent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#parent">QAbstractItemModel::parent</a>(const QModelIndex &amp;index) const.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn parent(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_parent(
            self as *const crate::QConcatenateTablesProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QConcatenateTablesProxyModel::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    ///
    ///
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_qt_metacall(
            self as *mut crate::QConcatenateTablesProxyModel,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QConcatenateTablesProxyModel::qt_metacast(const char* arg1)```</span>.
    ///
    ///
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_qt_metacast(
            self as *mut crate::QConcatenateTablesProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Removes the source model <i>sourceModel</i>, which was previously added to this proxy.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QConcatenateTablesProxyModel::removeSourceModel(QAbstractItemModel* sourceModel)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#removeSourceModel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the source model <i>sourceModel</i>, which was previously added to this proxy.</p>
    /// <p>The ownership of <i>sourceModel</i> is not affected by this.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn remove_source_model(
        &mut self,
        source_model: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractItemModel>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_removeSourceModel(
            self as *mut crate::QConcatenateTablesProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractItemModel>>::cast_into(
                source_model,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">QAbstractItemModel::rowCount</a>(const QModelIndex &amp;parent) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual int QConcatenateTablesProxyModel::rowCount(const QModelIndex& parent = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#rowCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">QAbstractItemModel::rowCount</a>(const QModelIndex &amp;parent) const.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn row_count_1a(
        &self,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_rowCount(
            self as *const crate::QConcatenateTablesProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">QAbstractItemModel::rowCount</a>(const QModelIndex &amp;parent) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual int QConcatenateTablesProxyModel::rowCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#rowCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">QAbstractItemModel::rowCount</a>(const QModelIndex &amp;parent) const.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn row_count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_rowCount1(
            self as *const crate::QConcatenateTablesProxyModel,
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">QAbstractItemModel::setData</a>(const QModelIndex &amp;index, const QVariant &amp;value, int role).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QConcatenateTablesProxyModel::setData(const QModelIndex& index, const QVariant& value, int role = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#setData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">QAbstractItemModel::setData</a>(const QModelIndex &amp;index, const QVariant &amp;value, int role).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#data">data</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn set_data_3a(
        &mut self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
        role: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_setData(
            self as *mut crate::QConcatenateTablesProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
            role,
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">QAbstractItemModel::setData</a>(const QModelIndex &amp;index, const QVariant &amp;value, int role).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QConcatenateTablesProxyModel::setData(const QModelIndex& index, const QVariant& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#setData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setData">QAbstractItemModel::setData</a>(const QModelIndex &amp;index, const QVariant &amp;value, int role).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#data">data</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn set_data_2a(
        &mut self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_setData1(
            self as *mut crate::QConcatenateTablesProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setItemData">QAbstractItemModel::setItemData</a>(const QModelIndex &amp;index, const QMap&lt;int, QVariant&gt; &amp;roles).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QConcatenateTablesProxyModel::setItemData(const QModelIndex& index, const QMap<int, QVariant>& roles)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#setItemData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#setItemData">QAbstractItemModel::setItemData</a>(const QModelIndex &amp;index, const QMap&lt;int, QVariant&gt; &amp;roles).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#itemData">itemData</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn set_item_data(
        &mut self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        roles: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMapOfIntQVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_setItemData(
            self as *mut crate::QConcatenateTablesProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMapOfIntQVariant>>::cast_into(roles)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#span">QAbstractItemModel::span</a>(const QModelIndex &amp;index) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QSize QConcatenateTablesProxyModel::span(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#span">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#span">QAbstractItemModel::span</a>(const QModelIndex &amp;index) const.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn span(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QSize> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_span(
            self as *const crate::QConcatenateTablesProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QConcatenateTablesProxyModel::tr(const char* s, const char* c, int n)```</span>.
    ///
    ///
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QConcatenateTablesProxyModel::trUtf8(const char* s, const char* c, int n)```</span>.
    ///
    ///
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>This model will make the rows of the source model become columns of the proxy model and vice-versa.</p>
///
/// C++ class: <span style='color: green;'>```QTransposeProxyModel```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This model will make the rows of the source model become columns of the proxy model and vice-versa.</p>
/// <p>If the model is a tree, the parents will be transposed as well. For example, if an index in the source model had parent `index(2,0)`, it will have parent `index(0,2)` in the proxy.</p></div>
#[repr(C)]
pub struct QTransposeProxyModel {
    _unused: u8,
}
impl QTransposeProxyModel {
    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnCount">QAbstractItemModel::columnCount</a>(const QModelIndex &amp;parent) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual int QTransposeProxyModel::columnCount(const QModelIndex& parent = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#columnCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnCount">QAbstractItemModel::columnCount</a>(const QModelIndex &amp;parent) const.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn column_count_1a(
        &self,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_columnCount(
            self as *const crate::QTransposeProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnCount">QAbstractItemModel::columnCount</a>(const QModelIndex &amp;parent) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual int QTransposeProxyModel::columnCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#columnCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#columnCount">QAbstractItemModel::columnCount</a>(const QModelIndex &amp;parent) const.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn column_count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_columnCount1(
            self as *const crate::QTransposeProxyModel,
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#headerData">QAbstractProxyModel::headerData</a>(int section, Qt::Orientation orientation, int role) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QVariant QTransposeProxyModel::headerData(int section, Qt::Orientation orientation, int role = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#headerData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#headerData">QAbstractProxyModel::headerData</a>(int section, Qt::Orientation orientation, int role) const.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#setHeaderData">setHeaderData</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn header_data_3a(
        &self,
        section: ::std::os::raw::c_int,
        orientation: crate::Orientation,
        role: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_headerData(
            self as *const crate::QTransposeProxyModel,
            section,
            orientation,
            role,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#headerData">QAbstractProxyModel::headerData</a>(int section, Qt::Orientation orientation, int role) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QVariant QTransposeProxyModel::headerData(int section, Qt::Orientation orientation) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#headerData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#headerData">QAbstractProxyModel::headerData</a>(int section, Qt::Orientation orientation, int role) const.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#setHeaderData">setHeaderData</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn header_data_2a(
        &self,
        section: ::std::os::raw::c_int,
        orientation: crate::Orientation,
    ) -> ::cpp_utils::CppBox<crate::QVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_headerData1(
            self as *const crate::QTransposeProxyModel,
            section,
            orientation,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>(int row, int column, const QModelIndex &amp;parent) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QTransposeProxyModel::index(int row, int column, const QModelIndex& parent = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#index">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>(int row, int column, const QModelIndex &amp;parent) const.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn index_3a(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_index(
            self as *const crate::QTransposeProxyModel,
            row,
            column,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>(int row, int column, const QModelIndex &amp;parent) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QTransposeProxyModel::index(int row, int column) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#index">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>(int row, int column, const QModelIndex &amp;parent) const.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn index_2a(
        &self,
        row: ::std::os::raw::c_int,
        column: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_index1(
            self as *const crate::QTransposeProxyModel,
            row,
            column,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumns">QAbstractItemModel::insertColumns</a>(int column, int count, const QModelIndex &amp;parent).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QTransposeProxyModel::insertColumns(int column, int count, const QModelIndex& parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#insertColumns">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumns">QAbstractItemModel::insertColumns</a>(int column, int count, const QModelIndex &amp;parent).</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn insert_columns_3a(
        &mut self,
        column: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_insertColumns(
            self as *mut crate::QTransposeProxyModel,
            column,
            count,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumns">QAbstractItemModel::insertColumns</a>(int column, int count, const QModelIndex &amp;parent).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QTransposeProxyModel::insertColumns(int column, int count)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#insertColumns">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertColumns">QAbstractItemModel::insertColumns</a>(int column, int count, const QModelIndex &amp;parent).</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn insert_columns_2a(
        &mut self,
        column: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_insertColumns1(
            self as *mut crate::QTransposeProxyModel,
            column,
            count,
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">QAbstractItemModel::insertRows</a>(int row, int count, const QModelIndex &amp;parent).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QTransposeProxyModel::insertRows(int row, int count, const QModelIndex& parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#insertRows">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">QAbstractItemModel::insertRows</a>(int row, int count, const QModelIndex &amp;parent).</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn insert_rows_3a(
        &mut self,
        row: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_insertRows(
            self as *mut crate::QTransposeProxyModel,
            row,
            count,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">QAbstractItemModel::insertRows</a>(int row, int count, const QModelIndex &amp;parent).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QTransposeProxyModel::insertRows(int row, int count)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#insertRows">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#insertRows">QAbstractItemModel::insertRows</a>(int row, int count, const QModelIndex &amp;parent).</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn insert_rows_2a(
        &mut self,
        row: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_insertRows1(
            self as *mut crate::QTransposeProxyModel,
            row,
            count,
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#itemData">QAbstractProxyModel::itemData</a>(const QModelIndex &amp;proxyIndex) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QMap<int, QVariant> QTransposeProxyModel::itemData(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#itemData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#itemData">QAbstractProxyModel::itemData</a>(const QModelIndex &amp;proxyIndex) const.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#setItemData">setItemData</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn item_data(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QMapOfIntQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_itemData(
            self as *const crate::QTransposeProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapFromSource">QAbstractProxyModel::mapFromSource</a>(const QModelIndex &amp;sourceIndex) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QTransposeProxyModel::mapFromSource(const QModelIndex& sourceIndex) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#mapFromSource">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapFromSource">QAbstractProxyModel::mapFromSource</a>(const QModelIndex &amp;sourceIndex) const.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn map_from_source(
        &self,
        source_index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_mapFromSource(
            self as *const crate::QTransposeProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(source_index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapToSource">QAbstractProxyModel::mapToSource</a>(const QModelIndex &amp;proxyIndex) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QTransposeProxyModel::mapToSource(const QModelIndex& proxyIndex) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#mapToSource">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#mapToSource">QAbstractProxyModel::mapToSource</a>(const QModelIndex &amp;proxyIndex) const.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn map_to_source(
        &self,
        proxy_index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_mapToSource(
            self as *const crate::QTransposeProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(proxy_index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual const QMetaObject* QTransposeProxyModel::metaObject() const```</span>.
    ///
    ///
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn meta_object(&self) -> ::cpp_utils::Ptr<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_metaObject(
            self as *const crate::QTransposeProxyModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#moveColumns">QAbstractItemModel::moveColumns</a>(const QModelIndex &amp;sourceParent, int sourceColumn, int count, const QModelIndex &amp;destinationParent, int destinationChild).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QTransposeProxyModel::moveColumns(const QModelIndex& sourceParent, int sourceColumn, int count, const QModelIndex& destinationParent, int destinationChild)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#moveColumns">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#moveColumns">QAbstractItemModel::moveColumns</a>(const QModelIndex &amp;sourceParent, int sourceColumn, int count, const QModelIndex &amp;destinationParent, int destinationChild).</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn move_columns(
        &mut self,
        source_parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        source_column: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
        destination_parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        destination_child: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_moveColumns(
            self as *mut crate::QTransposeProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(source_parent)
                .as_raw_ptr(),
            source_column,
            count,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(
                destination_parent,
            )
            .as_raw_ptr(),
            destination_child,
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#moveRows">QAbstractItemModel::moveRows</a>(const QModelIndex &amp;sourceParent, int sourceRow, int count, const QModelIndex &amp;destinationParent, int destinationChild).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QTransposeProxyModel::moveRows(const QModelIndex& sourceParent, int sourceRow, int count, const QModelIndex& destinationParent, int destinationChild)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#moveRows">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#moveRows">QAbstractItemModel::moveRows</a>(const QModelIndex &amp;sourceParent, int sourceRow, int count, const QModelIndex &amp;destinationParent, int destinationChild).</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn move_rows(
        &mut self,
        source_parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        source_row: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
        destination_parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        destination_child: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_moveRows(
            self as *mut crate::QTransposeProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(source_parent)
                .as_raw_ptr(),
            source_row,
            count,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(
                destination_parent,
            )
            .as_raw_ptr(),
            destination_child,
        )
    }

    /// <p>Constructs a new proxy model with the given <i>parent</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTransposeProxyModel::QTransposeProxyModel(QObject* parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#QTransposeProxyModel-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a new proxy model with the given <i>parent</i>.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn new_1a(
        parent: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QObject>>,
    ) -> ::cpp_utils::CppBox<crate::QTransposeProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_QTransposeProxyModel(
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QObject>>::cast_into(parent)
                .as_mut_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This model will make the rows of the source model become columns of the proxy model and vice-versa.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QTransposeProxyModel::QTransposeProxyModel()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This model will make the rows of the source model become columns of the proxy model and vice-versa.</p>
    /// <p>If the model is a tree, the parents will be transposed as well. For example, if an index in the source model had parent `index(2,0)`, it will have parent `index(0,2)` in the proxy.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QTransposeProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_QTransposeProxyModel1();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#parent">QAbstractItemModel::parent</a>(const QModelIndex &amp;index) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QModelIndex QTransposeProxyModel::parent(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#parent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#parent">QAbstractItemModel::parent</a>(const QModelIndex &amp;index) const.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn parent(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_parent(
            self as *const crate::QTransposeProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```virtual int QTransposeProxyModel::qt_metacall(QMetaObject::Call arg1, int arg2, void** arg3)```</span>.
    ///
    ///
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn qt_metacall(
        &mut self,
        arg1: crate::q_meta_object::Call,
        arg2: ::std::os::raw::c_int,
        arg3: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_qt_metacall(
            self as *mut crate::QTransposeProxyModel,
            arg1,
            arg2,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<*mut ::std::ffi::c_void>>::cast_into(arg3)
                .as_mut_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```virtual void* QTransposeProxyModel::qt_metacast(const char* arg1)```</span>.
    ///
    ///
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn qt_metacast(
        &mut self,
        arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_qt_metacast(
            self as *mut crate::QTransposeProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumns">QAbstractItemModel::removeColumns</a>(int column, int count, const QModelIndex &amp;parent).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QTransposeProxyModel::removeColumns(int column, int count, const QModelIndex& parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#removeColumns">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumns">QAbstractItemModel::removeColumns</a>(int column, int count, const QModelIndex &amp;parent).</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn remove_columns_3a(
        &mut self,
        column: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_removeColumns(
            self as *mut crate::QTransposeProxyModel,
            column,
            count,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumns">QAbstractItemModel::removeColumns</a>(int column, int count, const QModelIndex &amp;parent).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QTransposeProxyModel::removeColumns(int column, int count)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#removeColumns">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeColumns">QAbstractItemModel::removeColumns</a>(int column, int count, const QModelIndex &amp;parent).</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn remove_columns_2a(
        &mut self,
        column: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_removeColumns1(
            self as *mut crate::QTransposeProxyModel,
            column,
            count,
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">QAbstractItemModel::removeRows</a>(int row, int count, const QModelIndex &amp;parent).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QTransposeProxyModel::removeRows(int row, int count, const QModelIndex& parent = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#removeRows">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">QAbstractItemModel::removeRows</a>(int row, int count, const QModelIndex &amp;parent).</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn remove_rows_3a(
        &mut self,
        row: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_removeRows(
            self as *mut crate::QTransposeProxyModel,
            row,
            count,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">QAbstractItemModel::removeRows</a>(int row, int count, const QModelIndex &amp;parent).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QTransposeProxyModel::removeRows(int row, int count)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#removeRows">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#removeRows">QAbstractItemModel::removeRows</a>(int row, int count, const QModelIndex &amp;parent).</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn remove_rows_2a(
        &mut self,
        row: ::std::os::raw::c_int,
        count: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_removeRows1(
            self as *mut crate::QTransposeProxyModel,
            row,
            count,
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">QAbstractItemModel::rowCount</a>(const QModelIndex &amp;parent) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual int QTransposeProxyModel::rowCount(const QModelIndex& parent = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#rowCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">QAbstractItemModel::rowCount</a>(const QModelIndex &amp;parent) const.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn row_count_1a(
        &self,
        parent: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_rowCount(
            self as *const crate::QTransposeProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(parent)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">QAbstractItemModel::rowCount</a>(const QModelIndex &amp;parent) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual int QTransposeProxyModel::rowCount() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#rowCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#rowCount">QAbstractItemModel::rowCount</a>(const QModelIndex &amp;parent) const.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn row_count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_rowCount1(
            self as *const crate::QTransposeProxyModel,
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#setHeaderData">QAbstractProxyModel::setHeaderData</a>(int section, Qt::Orientation orientation, const QVariant &amp;value, int role).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QTransposeProxyModel::setHeaderData(int section, Qt::Orientation orientation, const QVariant& value, int role = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#setHeaderData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#setHeaderData">QAbstractProxyModel::setHeaderData</a>(int section, Qt::Orientation orientation, const QVariant &amp;value, int role).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#headerData">headerData</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn set_header_data_4a(
        &mut self,
        section: ::std::os::raw::c_int,
        orientation: crate::Orientation,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
        role: ::std::os::raw::c_int,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_setHeaderData(
            self as *mut crate::QTransposeProxyModel,
            section,
            orientation,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
            role,
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#setHeaderData">QAbstractProxyModel::setHeaderData</a>(int section, Qt::Orientation orientation, const QVariant &amp;value, int role).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QTransposeProxyModel::setHeaderData(int section, Qt::Orientation orientation, const QVariant& value)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#setHeaderData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#setHeaderData">QAbstractProxyModel::setHeaderData</a>(int section, Qt::Orientation orientation, const QVariant &amp;value, int role).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#headerData">headerData</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn set_header_data_3a(
        &mut self,
        section: ::std::os::raw::c_int,
        orientation: crate::Orientation,
        value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_setHeaderData1(
            self as *mut crate::QTransposeProxyModel,
            section,
            orientation,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(value)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#setItemData">QAbstractProxyModel::setItemData</a>(const QModelIndex &amp;index, const QMap&lt;int, QVariant&gt; &amp;roles).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual bool QTransposeProxyModel::setItemData(const QModelIndex& index, const QMap<int, QVariant>& roles)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#setItemData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#setItemData">QAbstractProxyModel::setItemData</a>(const QModelIndex &amp;index, const QMap&lt;int, QVariant&gt; &amp;roles).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#itemData">itemData</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn set_item_data(
        &mut self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
        roles: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMapOfIntQVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_setItemData(
            self as *mut crate::QTransposeProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMapOfIntQVariant>>::cast_into(roles)
                .as_raw_ptr(),
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#setSourceModel">QAbstractProxyModel::setSourceModel</a>(QAbstractItemModel *sourceModel).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual void QTransposeProxyModel::setSourceModel(QAbstractItemModel* newSourceModel)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#setSourceModel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#setSourceModel">QAbstractProxyModel::setSourceModel</a>(QAbstractItemModel *sourceModel).</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn set_source_model(
        &mut self,
        new_source_model: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::QAbstractItemModel>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_setSourceModel(
            self as *mut crate::QTransposeProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::QAbstractItemModel>>::cast_into(
                new_source_model,
            )
            .as_mut_raw_ptr(),
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#sort">QAbstractProxyModel::sort</a>(int column, Qt::SortOrder order).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual void QTransposeProxyModel::sort(int column, Qt::SortOrder order = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#sort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#sort">QAbstractProxyModel::sort</a>(int column, Qt::SortOrder order).</p>
    /// <p>This method will perform no action. Use a <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html">QSortFilterProxyModel</a> on top of this one if you require sorting.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn sort_2a(&mut self, column: ::std::os::raw::c_int, order: crate::SortOrder) {
        crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_sort(
            self as *mut crate::QTransposeProxyModel,
            column,
            order,
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#sort">QAbstractProxyModel::sort</a>(int column, Qt::SortOrder order).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual void QTransposeProxyModel::sort(int column)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#sort">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#sort">QAbstractProxyModel::sort</a>(int column, Qt::SortOrder order).</p>
    /// <p>This method will perform no action. Use a <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html">QSortFilterProxyModel</a> on top of this one if you require sorting.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn sort_1a(&mut self, column: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_sort1(
            self as *mut crate::QTransposeProxyModel,
            column,
        )
    }

    /// <p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#span">QAbstractProxyModel::span</a>(const QModelIndex &amp;index) const.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual QSize QTransposeProxyModel::span(const QModelIndex& index) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#span">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reimplements: <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#span">QAbstractProxyModel::span</a>(const QModelIndex &amp;index) const.</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn span(
        &self,
        index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
    ) -> ::cpp_utils::CppBox<crate::QSize> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_span(
            self as *const crate::QTransposeProxyModel,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Returns a reference to the <span style='color: green;'>```staticMetaObject```</span> field.
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn static_meta_object() -> ::cpp_utils::Ref<crate::QMetaObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_staticMetaObject();
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QTransposeProxyModel::tr(const char* s, const char* c, int n)```</span>.
    ///
    ///
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn tr(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_tr(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```static QString QTransposeProxyModel::trUtf8(const char* s, const char* c, int n)```</span>.
    ///
    ///
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn tr_utf8(
        s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        c: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
        n: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_trUtf8(
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
                .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(c)
                .as_raw_ptr(),
            n,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

/// <p>The <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> class is a template class that provides a dynamic array.</p>
///
/// C++ class: <span style='color: green;'>```QVector<QPair<double, QVariant>>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qvector.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> class is a template class that provides a dynamic array.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores its items in adjacent memory locations and provides fast index-based access.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Here's an example of a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> that stores <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerVector;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> stringVector;
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> stores its items in a vector (array). Typically, vectors are created with an initial size. For example, the following code constructs a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> with 200 elements:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">200</span>);
///
/// </pre>
/// <p>The elements are automatically initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you want to initialize the vector with a different value, pass that value as the second argument to the constructor:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">200</span><span class="operator">,</span> <span class="string">"Pass"</span>);
///
/// </pre>
/// <p>You can also call <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>() at any time to fill the vector with a value.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const vectors, operator[]() returns a reference to the item that can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (vector<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Liz"</span>)
/// &#32;     vector<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Elizabeth"</span>;
///
/// </pre>
/// <p>For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> vector<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (vector<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Alfonso"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Alfonso at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>Another way to access the data stored in a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> is to call <a href="http://doc.qt.io/qt-5/qvector.html#data">data</a>(). The function returns a pointer to the first item in the vector. You can use the pointer to directly access and modify the elements stored in the vector. The pointer is also useful if you need to pass a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to a function that accepts a plain C++ array.</p>
/// <p>If you want to find all occurrences of a particular value in a vector, use <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of the matching item if they found one; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> vector<span class="operator">.</span>indexOf(<span class="string">"Harumi"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Harumi is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a vector contains a particular value, use <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the vector, use <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(). With the exception of <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), these functions can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>) for large vectors, because they require moving many items in the vector by one position in memory. If you want a container class that provides fast insertion/removal in the middle, use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> or <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
/// <p>Unlike plain C++ arrays, QVectors can be resized at any time by calling <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>(). If the new size is larger than the old size, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> might need to reallocate the whole vector. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> tries to reduce the number of reallocations by preallocating up to twice as much memory as the actual data needs.</p>
/// <p>If you know in advance approximately how many items the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> will contain, you can call <a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>(), asking <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to preallocate a certain amount of memory. You can also call <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>() to find out how much memory <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> actually allocated.</p>
/// <p>Note that using non-const operators and functions can cause <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to do a deep copy of the data. This is due to <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicit sharing</a>.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qvectoriterator.html">QVectorIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablevectoriterator.html">QMutableVectorIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qvector.html#const_iterator-typedef">QVector::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qvector.html#iterator-typedefx">QVector::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>.</p>
/// <p>In addition to <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>, Qt also provides <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a>, a very low-level class with little functionality that is optimized for speed.</p>
/// <p><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <a name="more-information-on-using-qt-containers"></a>
/// <h4>More Information on Using Qt Containers</h4>
/// <p>For a detailed discussion comparing Qt containers with each other and with STL containers, see <a href="http://marcmutz.wordpress.com/effective-qt/containers/">Understand the Qt Containers</a>.</p></div>
#[repr(C)]
pub struct QVectorOfQPairOfDoubleQVariant {
    _unused: u8,
}
impl QVectorOfQPairOfDoubleQVariant {
    /// <p>Inserts <i>value</i> at the end of the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPair<double, QVariant>>::append(const QPair<double, QVariant>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   vector<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> three <span class="operator">=</span> <span class="string">"three"</span>;
    ///   vector<span class="operator">.</span>append(three);
    ///   <span class="comment">// vector: ["one", "two", "three"]</span>
    ///   <span class="comment">// three: "three"</span>
    ///
    /// </pre>
    /// <p>This is the same as calling resize(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() + 1) and assigning <i>value</i> to the new last element in the vector.</p>
    /// <p>This operation is relatively fast, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> typically allocates more memory than necessary, so it can grow without reallocating the entire vector each time.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>().</p></div>
    pub unsafe fn append_q_pair_of_double_q_variant(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_append(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPair<double, QVariant>>::append(const QVector<QPair<double, QVariant>>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#append-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> vector to this vector.</p>
    /// <p>This function was introduced in  Qt 5.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_vector_of_q_pair_of_double_q_variant(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_append2(self as *mut crate::QVectorOfQPairOfDoubleQVariant, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>>::cast_into(l).as_raw_ptr())
    }

    /// <p>Returns the item at index position <i>i</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPair<double, QVariant>& QVector<QPair<double, QVariant>>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the vector.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ref<crate::QPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_at(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPair<double, QVariant>& QVector<QPair<double, QVariant>>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::MutRef<crate::QPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_back(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPair<double, QVariant>& QVector<QPair<double, QVariant>>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<crate::QPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_back1(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the maximum number of items that can be stored in the vector without forcing a reallocation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QPair<double, QVariant>>::capacity() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#capacity">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the maximum number of items that can be stored in the vector without forcing a reallocation.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s memory usage. In general, you will rarely ever need to call this function. If you want to know how many items are in the vector, call <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p></div>
    pub unsafe fn capacity(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_capacity(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
        )
    }

    /// <p>Removes all the elements from the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPair<double, QVariant>>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all the elements from the vector.</p>
    /// <p><b>Note: </b>Until Qt 5.6, this also released the memory used by the vector. From Qt 5.7, the capacity is preserved. To shed all capacity, swap with a default-constructed vector:</p><pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span>T<span class="operator">&gt;</span> v <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span>T<span class="operator">&gt;</span>()<span class="operator">.</span>swap(v);
    ///   Q_ASSERT(v<span class="operator">.</span>capacity() <span class="operator">=</span><span class="operator">=</span> <span class="number">0</span>);
    ///
    /// </pre>
    /// <p>or call <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_clear(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
        )
    }

    /// <p>Returns a const pointer to the data stored in the vector. The pointer can be used to access the items in the vector. The pointer remains valid as long as the vector isn't reallocated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPair<double, QVariant>* QVector<QPair<double, QVariant>>::constData() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const pointer to the data stored in the vector. The pointer can be used to access the items in the vector. The pointer remains valid as long as the vector isn't reallocated.</p>
    /// <p>This function is mostly useful to pass a vector to a function that accepts a plain C++ array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#data">data</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn const_data(&self) -> ::cpp_utils::Ptr<crate::QPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_constData(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// <p>Returns a const reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPair<double, QVariant>& QVector<QPair<double, QVariant>>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<crate::QPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_constFirst(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPair<double, QVariant>& QVector<QPair<double, QVariant>>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<crate::QPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_constLast(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the vector contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QPair<double, QVariant>>::contains(const QPair<double, QVariant>& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the vector contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p></div>
    pub unsafe fn contains(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_contains(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QPair<double, QVariant>>& QVector<QPair<double, QVariant>>::operator=(const QVector<QPair<double, QVariant>>& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this vector and returns a reference to this vector.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        v: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_operator_(self as *mut crate::QVectorOfQPairOfDoubleQVariant, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>>::cast_into(v).as_raw_ptr());
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of occurrences of <i>value</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QPair<double, QVariant>>::count(const QPair<double, QVariant>& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of occurrences of <i>value</i> in the vector.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_count(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QPair<double, QVariant>>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_count1(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
        )
    }

    /// <p>Returns a pointer to the data stored in the vector. The pointer can be used to access and modify the items in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPair<double, QVariant>* QVector<QPair<double, QVariant>>::data()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#data">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the data stored in the vector. The pointer can be used to access and modify the items in the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> vector(<span class="number">10</span>);
    ///   <span class="type">int</span> <span class="operator">*</span>data <span class="operator">=</span> vector<span class="operator">.</span>data();
    ///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> <span class="number">10</span>; <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     data<span class="operator">[</span>i<span class="operator">]</span> <span class="operator">=</span> <span class="number">2</span> <span class="operator">*</span> i;
    ///
    /// </pre>
    /// <p>The pointer remains valid as long as the vector isn't reallocated.</p>
    /// <p>This function is mostly useful to pass a vector to a function that accepts a plain C++ array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#constData">constData</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn data_mut(&mut self) -> ::cpp_utils::MutPtr<crate::QPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_data(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPair<double, QVariant>* QVector<QPair<double, QVariant>>::data() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#data-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn data(&self) -> ::cpp_utils::Ptr<crate::QPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_data1(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```void QVector<QPair<double, QVariant>>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_detach(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), returning <code>true</code> if the vector is empty; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QPair<double, QVariant>>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), returning <code>true</code> if the vector is empty; otherwise returns <code>false</code>.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_empty(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
        )
    }

    /// <p>Returns <code>true</code> if this vector is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QPair<double, QVariant>>::endsWith(const QPair<double, QVariant>& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this vector is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>().</p></div>
    pub unsafe fn ends_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_endsWith(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QPair<double, QVariant>>& QVector<QPair<double, QVariant>>::fill(const QPair<double, QVariant>& t, int size = …)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">3</span>);
    ///   vector<span class="operator">.</span>fill(<span class="string">"Yes"</span>);
    ///   <span class="comment">// vector: ["Yes", "Yes", "Yes"]</span>
    ///
    ///   vector<span class="operator">.</span>fill(<span class="string">"oh"</span><span class="operator">,</span> <span class="number">5</span>);
    ///   <span class="comment">// vector: ["oh", "oh", "oh", "oh", "oh"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_2a(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>,
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_fill(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>::cast_into(t)
                .as_raw_ptr(),
            size,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QPair<double, QVariant>>& QVector<QPair<double, QVariant>>::fill(const QPair<double, QVariant>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#fill">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>value</i> to all items in the vector. If <i>size</i> is different from -1 (the default), the vector is resized to size <i>size</i> beforehand.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector(<span class="number">3</span>);
    ///   vector<span class="operator">.</span>fill(<span class="string">"Yes"</span>);
    ///   <span class="comment">// vector: ["Yes", "Yes", "Yes"]</span>
    ///
    ///   vector<span class="operator">.</span>fill(<span class="string">"oh"</span><span class="operator">,</span> <span class="number">5</span>);
    ///   <span class="comment">// vector: ["oh", "oh", "oh", "oh", "oh"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn fill_1a(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_fill1(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>::cast_into(t)
                .as_raw_ptr(),
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPair<double, QVariant>& QVector<QPair<double, QVariant>>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#last">last</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#constFirst">constFirst</a>().</p></div>
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<crate::QPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_first(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPair<double, QVariant>& QVector<QPair<double, QVariant>>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::QPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_first1(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPair<double, QVariant>& QVector<QPair<double, QVariant>>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::MutRef<crate::QPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_front(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPair<double, QVariant>& QVector<QPair<double, QVariant>>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<crate::QPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_front1(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPair<double, QVariant>& QVector<QPair<double, QVariant>>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p>Note that using non-const operators can cause <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> to do a deep copy.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_operator__2(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPair<double, QVariant>& QVector<QPair<double, QVariant>>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as at(<i>i</i>).</p></div>
    pub unsafe fn index(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ref<crate::QPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_operator__3(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QPair<double, QVariant>>::indexOf(const QPair<double, QVariant>& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span>);            <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);         <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);         <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"X"</span>);            <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_indexOf(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>::cast_into(t)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QPair<double, QVariant>>::indexOf(const QPair<double, QVariant>& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the vector, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span>);            <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);         <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);         <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>indexOf(<span class="string">"X"</span>);            <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_indexOf1(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the vector. If <i>i</i> is 0, the value is prepended to the vector. If <i>i</i> is <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), the value is appended to the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPair<double, QVariant>>::insert(int i, const QPair<double, QVariant>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the vector. If <i>i</i> is 0, the value is prepended to the vector. If <i>i</i> is <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), the value is appended to the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   vector<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// vector: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p>For large vectors, this operation can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because it requires moving all the items at indexes <i>i</i> and above by one position further in memory. If you want a container class that provides a fast insert() function, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>().</p></div>
    pub unsafe fn insert_2a(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_insert(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPair<double, QVariant>>::insert(int i, int n, const QPair<double, QVariant>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>count</i> copies of <i>value</i> at index position <i>i</i> in the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type">double</span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">2.718</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">1.442</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">0.4342</span>;
    ///   vector<span class="operator">.</span>insert(<span class="number">1</span><span class="operator">,</span> <span class="number">3</span><span class="operator">,</span> <span class="number">9.9</span>);
    ///   <span class="comment">// vector: [2.718, 9.9, 9.9, 9.9, 1.442, 0.4342]</span>
    ///
    /// </pre></div>
    pub unsafe fn insert_3a(
        &mut self,
        i: ::std::os::raw::c_int,
        n: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_insert1(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
            i,
            n,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QVector<QPair<double, QVariant>>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_isDetached(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
        )
    }

    /// <p>Returns <code>true</code> if the vector has size 0; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QPair<double, QVariant>>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the vector has size 0; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_isEmpty(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QVector<QPair<double, QVariant>>::isSharedWith(const QVector<QPair<double, QVariant>>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_isSharedWith(self as *const crate::QVectorOfQPairOfDoubleQVariant, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>>::cast_into(other).as_raw_ptr())
    }

    /// <p>Returns a reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPair<double, QVariant>& QVector<QPair<double, QVariant>>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the vector. This function assumes that the vector isn't empty.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#constLast">constLast</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<crate::QPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_last(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPair<double, QVariant>& QVector<QPair<double, QVariant>>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::QPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_last1(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QPair<double, QVariant>>::lastIndexOf(const QPair<double, QVariant>& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);        <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);     <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);     <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_lastIndexOf(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>::cast_into(t)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QPair<double, QVariant>>::lastIndexOf(const QPair<double, QVariant>& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of the value <i>value</i> in the vector, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);        <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);     <span class="comment">// returns 3</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);     <span class="comment">// returns 1</span>
    ///   vector<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);        <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_lastIndexOf1(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QPair<double, QVariant>>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>().</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#count-1">count</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#length">QList::length</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_length(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
        )
    }

    /// <p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QPair<double, QVariant>> QVector<QPair<double, QVariant>>::mid(int pos, int len = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        len: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_mid(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
            pos,
            len,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QPair<double, QVariant>> QVector<QPair<double, QVariant>>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-vector which contains elements from this vector, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements after <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_mid1(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPair<double, QVariant>>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">QList::move</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_move(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
            from,
            to,
        )
    }

    /// <p>Constructs an empty vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QPair<double, QVariant>>::QVector()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn new_0a() -> ::cpp_utils::CppBox<crate::QVectorOfQPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_QVector();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a vector with an initial size of <i>size</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QPair<double, QVariant>>::QVector(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a vector with an initial size of <i>size</i> elements.</p>
    /// <p>The elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn new_1a(
        size: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_QVector1(size);
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a vector with an initial size of <i>size</i> elements. Each element is initialized with <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QPair<double, QVariant>>::QVector(int size, const QPair<double, QVariant>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a vector with an initial size of <i>size</i> elements. Each element is initialized with <i>value</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn new_2a(
        size: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_QVector2(
            size,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>::cast_into(t)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QVector<QPair<double, QVariant>>::QVector(const QVector<QPair<double, QVariant>>& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#QVector-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        v: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_QVector3(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>>::cast_into(v).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPair<double, QVariant>>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_pop_back(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPair<double, QVariant>>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_pop_front(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
        )
    }

    /// <p>Inserts <i>value</i> at the beginning of the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPair<double, QVariant>>::prepend(const QPair<double, QVariant>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the vector.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvector.html#QVector">QVector</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> vector;
    ///   vector<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   vector<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   vector<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// vector: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as vector.insert(0, <i>value</i>).</p>
    /// <p>For large vectors, this operation can be slow (<a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>), because it requires moving all the items in the vector by one position further in memory. If you want a container class that provides a fast prepend() function, use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> or <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> instead.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_prepend(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to append(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPair<double, QVariant>>::push_back(const QPair<double, QVariant>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to append(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_push_back(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to prepend(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPair<double, QVariant>>::push_front(const QPair<double, QVariant>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to prepend(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_push_front(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPair<double, QVariant>>::remove(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#remove">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes the element at index position <i>i</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn remove_1a(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_remove(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
            i,
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPair<double, QVariant>>::remove(int i, int n)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#remove-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes <i>count</i> elements from the middle of the vector, starting at index position <i>i</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#fill">fill</a>().</p></div>
    pub unsafe fn remove_2a(&mut self, i: ::std::os::raw::c_int, n: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_remove1(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
            i,
            n,
        )
    }

    /// <p>Removes all elements that compare equal to <i>t</i> from the vector. Returns the number of elements removed, if any.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QPair<double, QVariant>>::removeAll(const QPair<double, QVariant>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all elements that compare equal to <i>t</i> from the vector. Returns the number of elements removed, if any.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#removeOne">removeOne</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">QList::removeAll</a>().</p></div>
    pub unsafe fn remove_all(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_removeAll(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes the element at index position <i>i</i>. Equivalent to</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPair<double, QVariant>>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the element at index position <i>i</i>. Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   remove(i);
    ///
    /// </pre>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">QList::removeAt</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_removeAt(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
            i,
        )
    }

    /// <p>Removes the first item in the vector. Calling this function is equivalent to calling remove(0). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPair<double, QVariant>>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the vector. Calling this function is equivalent to calling remove(0). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_removeFirst(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
        )
    }

    /// <p>Removes the last item in the vector. Calling this function is equivalent to calling remove(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() - 1). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPair<double, QVariant>>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the vector. Calling this function is equivalent to calling remove(<a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>() - 1). The vector must not be empty. If the vector can be empty, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_removeLast(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
        )
    }

    /// <p>Removes the first element that compares equal to <i>t</i> from the vector. Returns whether an element was, in fact, removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QPair<double, QVariant>>::removeOne(const QPair<double, QVariant>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#removeOne">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first element that compares equal to <i>t</i> from the vector. Returns whether an element was, in fact, removed.</p>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#removeAll">removeAll</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">QList::removeOne</a>().</p></div>
    pub unsafe fn remove_one(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_removeOne(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPair<double, QVariant>>::replace(int i, const QPair<double, QVariant>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>.</p>
    /// <p><i>i</i> must be a valid index position in the vector (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#remove">remove</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_replace(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Attempts to allocate memory for at least <i>size</i> elements. If you know in advance how large the vector will be, you should call this function to prevent reallocations and memory fragmentation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPair<double, QVariant>>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Attempts to allocate memory for at least <i>size</i> elements. If you know in advance how large the vector will be, you should call this function to prevent reallocations and memory fragmentation.</p>
    /// <p>If <i>size</i> is an underestimate, the worst that will happen is that the <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> will be a bit slower. If <i>size</i> is an overestimate, you may have used more memory than the normal <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> growth strategy would have allocated—or you may have used less.</p>
    /// <p>An alternative to reserve() is calling <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>(). Whether or not that is faster than reserve() depends on the element type, because <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() default-constructs all elements, and requires assignment to existing entries rather than calling <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>(), which copy- or move-constructs. For simple types, like <code>int</code> or <code>double</code>, <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() is typically faster, but for anything more complex, you should prefer reserve().</p>
    /// <p><b>Warning:</b> If the size passed to <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>() was underestimated, you run out of allocated space and into undefined behavior. This problem does not exist with reserve(), because it treats the size as just a hint.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>().</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_reserve(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
            size,
        )
    }

    /// <p>Sets the size of the vector to <i>size</i>. If <i>size</i> is greater than the current size, elements are added to the end; the new elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If <i>size</i> is less than the current size, elements are removed from the end.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPair<double, QVariant>>::resize(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#resize">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the size of the vector to <i>size</i>. If <i>size</i> is greater than the current size, elements are added to the end; the new elements are initialized with a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If <i>size</i> is less than the current size, elements are removed from the end.</p>
    /// <p>Since Qt 5.6, resize() doesn't shrink the capacity anymore. To shed excess capacity, use <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#size">size</a>().</p></div>
    pub unsafe fn resize(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_resize(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QVector<QPair<double, QVariant>>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_setSharable(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
            sharable,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPair<double, QVariant>>::shrink_to_fit()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#shrink_to_fit">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">squeeze</a>().</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    ///
    /// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
    #[cfg(any(
        any(
            cpp_lib_version = "5.11.3",
            cpp_lib_version = "5.12.2",
            cpp_lib_version = "5.13.0"
        ),
        ritual_rustdoc
    ))]
    pub unsafe fn shrink_to_fit(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_shrink_to_fit(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
        )
    }

    /// <p>Returns the number of items in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QVector<QPair<double, QVariant>>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#resize">resize</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_size(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
        )
    }

    /// <p>Releases any memory not required to store the items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPair<double, QVariant>>::squeeze()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#squeeze">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Releases any memory not required to store the items.</p>
    /// <p>The sole purpose of this function is to provide a means of fine tuning <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>'s memory usage. In general, you will rarely ever need to call this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#reserve">reserve</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#capacity">capacity</a>().</p></div>
    pub unsafe fn squeeze(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_squeeze(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
        )
    }

    /// <p>Returns <code>true</code> if this vector is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QPair<double, QVariant>>::startsWith(const QPair<double, QVariant>& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this vector is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#first">first</a>().</p></div>
    pub unsafe fn starts_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_startsWith(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Swaps vector <i>other</i> with this vector. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QVector<QPair<double, QVariant>>::swap(QVector<QPair<double, QVariant>>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps vector <i>other</i> with this vector. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QVectorOfQPairOfDoubleQVariant>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_swap(self as *mut crate::QVectorOfQPairOfDoubleQVariant, ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QVectorOfQPairOfDoubleQVariant>>::cast_into(other).as_mut_raw_ptr())
    }

    /// <p>Removes the element at index position <i>i</i> and returns it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPair<double, QVariant> QVector<QPair<double, QVariant>>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the element at index position <i>i</i> and returns it.</p>
    /// <p>Equivalent to</p>
    /// <pre class="cpp">
    ///
    ///   T t <span class="operator">=</span> at(i);
    ///   remove(i);
    ///   <span class="keyword">return</span> t;
    ///
    /// </pre>
    /// <p>Provided for compatibility with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p>This function was introduced in  Qt 5.2.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">QList::takeAt</a>().</p></div>
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_takeAt(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the first item in the vector and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPair<double, QVariant> QVector<QPair<double, QVariant>>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the vector and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeLast">takeLast</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> ::cpp_utils::CppBox<crate::QPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_takeFirst(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the last item in the list and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPair<double, QVariant> QVector<QPair<double, QVariant>>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This function assumes the vector is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qvector.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#takeFirst">takeFirst</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> ::cpp_utils::CppBox<crate::QPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_takeLast(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value at index position <i>i</i> in the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPair<double, QVariant> QVector<QPair<double, QVariant>>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the vector.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that <i>i</i> is within bounds, you can use <a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_value(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPair<double, QVariant> QVector<QPair<double, QVariant>>::value(int i, const QPair<double, QVariant>& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>,
    ) -> ::cpp_utils::CppBox<crate::QPairOfDoubleQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_value1(
            self as *const crate::QVectorOfQPairOfDoubleQVariant,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>::cast_into(
                default_value,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

pub mod q_list_of_q_pair_of_q_string_q_string {
    //! C++ type: <span style='color: green;'>```QList<QPair<QString, QString>>```</span>

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QPair<QString, QString>>::iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct Iterator {
        _unused: u8,
    }
    impl Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::iterator& QList<QPair<QString, QString>>::iterator::operator=(const QList<QPair<QString, QString>>::iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_pair_of_q_string_q_string::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_iterator_operator_8(
                    self as *mut crate::q_list_of_q_pair_of_q_string_q_string::Iterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::iterator QList<QPair<QString, QString>>::iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>i--</code>) makes the preceding item current and returns an iterator pointing to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_iterator_operator__13(
                    self as *mut crate::q_list_of_q_pair_of_q_string_q_string::Iterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::iterator QList<QPair<QString, QString>>::iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>i++</code>) advances the iterator to the next item in the hash and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_iterator_operator__11(
                    self as *mut crate::q_list_of_q_pair_of_q_string_q_string::Iterator,
                    arg1,
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QPair<QString, QString>& QList<QPair<QString, QString>>::iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValueRef iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p>The return value is of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, a helper class for <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> and <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a>. When you get an object of type <a href="http://doc.qt.io/qt-5/qjsonvalue.html#qjsonvalueref">QJsonValueRef</a>, you can use it as if it were a reference to a <a href="http://doc.qt.io/qt-5/qjsonvalue.html">QJsonValue</a>. If you assign to it, the assignment will apply to the character in the <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> of <a href="http://doc.qt.io/qt-5/qjsonobject.html">QJsonObject</a> from which you got the reference.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::MutRef<crate::QPairOfQStringQString> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_iterator_operator__1(
                    self as *const crate::q_list_of_q_pair_of_q_string_q_string::Iterator,
                    j,
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QPair<QString, QString>>::iterator::iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html#iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>().</p></div>
        pub unsafe fn new(
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_iterator_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QPair<QString, QString>>::iterator::iterator(const QList<QPair<QString, QString>>::iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::Iterator> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_iterator_iterator1(
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>,
                    >::cast_into(o)
                    .as_raw_ptr(),
                );
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QPair<QString, QString>* QList<QPair<QString, QString>>::iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current item's value.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::MutPtr<crate::QPairOfQStringQString> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_iterator_operator__(
                    self as *const crate::q_list_of_q_pair_of_q_string_q_string::Iterator,
                );
            ::cpp_utils::MutPtr::from_raw(ffi_result)
        }
    }

    /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    ///
    /// C++ class: <span style='color: green;'>```QList<QPair<QString, QString>>::const_iterator```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
    /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
    ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
    /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
    /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
    ///
    /// </pre>
    /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
    /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
    #[repr(C)]
    pub struct ConstIterator {
        _unused: u8,
    }
    impl ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::const_iterator& QList<QPair<QString, QString>>::const_iterator::operator=(const QList<QPair<QString, QString>>::const_iterator& other)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        pub unsafe fn copy_from(
            &mut self,
            other: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>,
            >,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>
        {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_const_iterator_operator_6(
                    self as *mut crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator,
                    ::cpp_utils::CastInto::<
                        ::cpp_utils::Ref<
                            crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator,
                        >,
                    >::cast_into(other)
                    .as_raw_ptr(),
                );
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::const_iterator QList<QPair<QString, QString>>::const_iterator::operator--(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator---1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix -- operator (<code>it--</code>) makes the preceding item current and returns an iterator to the previously current item.</p></div>
        pub unsafe fn dec_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>
        {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_const_iterator_operator__9(self as *mut crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator, arg1);
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>This is an overloaded function.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::const_iterator QList<QPair<QString, QString>>::const_iterator::operator++(int arg1)```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
        /// <p>The postfix ++ operator (<code>it++</code>) advances the iterator to the next item in the container and returns an iterator to the previously current item.</p></div>
        pub unsafe fn inc_postfix(
            &mut self,
            arg1: ::std::os::raw::c_int,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>
        {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_const_iterator_operator__7(self as *mut crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator, arg1);
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QPair<QString, QString>& QList<QPair<QString, QString>>::const_iterator::operator[](long long j) const```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-5b-5d">C++ documentation</a> for <span style='color: green;'>```QJsonValue const_iterator::operator[](int j) const```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at offset <i>j</i> from the item pointed to by this iterator (the item at position <code>*this + j</code>).</p>
        /// <p>This function is provided to make <a href="http://doc.qt.io/qt-5/qjsonarray.html">QJsonArray</a> iterators behave like C++ pointers.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-2b">operator+</a>().</p></div>
        pub unsafe fn index(
            &self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::Ref<crate::QPairOfQStringQString> {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_const_iterator_operator__1(self as *const crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator, j);
            ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }

        /// <p>Constructs an uninitialized iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QPair<QString, QString>>::const_iterator::const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html#const_iterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an uninitialized iterator.</p>
        /// <p>Functions like operator*() and operator++() should not be called on an uninitialized iterator. Use operator=() to assign a value to it before using it.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>().</p></div>
        pub unsafe fn new_0a(
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>
        {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_const_iterator_const_iterator();
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QPair<QString, QString>>::const_iterator::const_iterator(const QList<QPair<QString, QString>>::iterator& o)```</span>.
        pub unsafe fn new_1a(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>
        {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_const_iterator_const_iterator2(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QPair<QString, QString>>::const_iterator::const_iterator(const QList<QPair<QString, QString>>::const_iterator& o)```</span>.
        pub unsafe fn new_copy(
            o: impl ::cpp_utils::CastInto<
                ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>,
            >,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>
        {
            let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_const_iterator_const_iterator1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>>::cast_into(o).as_raw_ptr());
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }

        /// <p>Returns a pointer to the current result.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QPair<QString, QString>* QList<QPair<QString, QString>>::const_iterator::operator->() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html#operator--gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a pointer to the current result.</p></div>
        pub unsafe fn struct_deref(&self) -> ::cpp_utils::Ptr<crate::QPairOfQStringQString> {
            let ffi_result =
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_const_iterator_operator__(
                    self as *const crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator,
                );
            ::cpp_utils::Ptr::from_raw(ffi_result)
        }
    }

    impl ::cpp_utils::ops::Indirection for &crate::q_list_of_q_pair_of_q_string_q_string::Iterator {
        type Output = ::cpp_utils::MutRef<crate::QPairOfQStringQString>;
        /// <p>Returns a modifiable reference to the current item's value.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QPair<QString, QString>& QList<QPair<QString, QString>>::iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a modifiable reference to the current item's value.</p>
        /// <p>Same as <a href="http://doc.qt.io/qt-5/qhash-iterator.html#value">value</a>().</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#key">key</a>().</p></div>
        fn indirection(self) -> ::cpp_utils::MutRef<crate::QPairOfQStringQString> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_iterator_operator_(
                    self as *const crate::q_list_of_q_pair_of_q_string_q_string::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl
        ::std::cmp::PartialEq<
            ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>,
        > for crate::q_list_of_q_pair_of_q_string_q_string::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPair<QString, QString>>::iterator::operator==(const QList<QPair<QString, QString>>::iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_iterator_operator__2(
                    self as *const crate::q_list_of_q_pair_of_q_string_q_string::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Lt<
            ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>,
        > for crate::q_list_of_q_pair_of_q_string_q_string::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPair<QString, QString>>::iterator::operator<(const QList<QPair<QString, QString>>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_iterator_operator_1(
                    self as *const crate::q_list_of_q_pair_of_q_string_q_string::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Le<
            ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>,
        > for crate::q_list_of_q_pair_of_q_string_q_string::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPair<QString, QString>>::iterator::operator<=(const QList<QPair<QString, QString>>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_iterator_operator__4(
                    self as *const crate::q_list_of_q_pair_of_q_string_q_string::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Gt<
            ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>,
        > for crate::q_list_of_q_pair_of_q_string_q_string::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPair<QString, QString>>::iterator::operator>(const QList<QPair<QString, QString>>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_iterator_operator_2(
                    self as *const crate::q_list_of_q_pair_of_q_string_q_string::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Ge<
            ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>,
        > for crate::q_list_of_q_pair_of_q_string_q_string::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPair<QString, QString>>::iterator::operator>=(const QList<QPair<QString, QString>>::iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_iterator_operator__5(
                    self as *const crate::q_list_of_q_pair_of_q_string_q_string::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::std::cmp::PartialEq<
            ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>,
        > for crate::q_list_of_q_pair_of_q_string_q_string::Iterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPair<QString, QString>>::iterator::operator==(const QList<QPair<QString, QString>>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_iterator_operator__6(
                    self as *const crate::q_list_of_q_pair_of_q_string_q_string::Iterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Lt<
            ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>,
        > for crate::q_list_of_q_pair_of_q_string_q_string::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPair<QString, QString>>::iterator::operator<(const QList<QPair<QString, QString>>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_iterator_operator_3(
                    self as *const crate::q_list_of_q_pair_of_q_string_q_string::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Le<
            ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>,
        > for crate::q_list_of_q_pair_of_q_string_q_string::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPair<QString, QString>>::iterator::operator<=(const QList<QPair<QString, QString>>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-lt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_iterator_operator__8(
                    self as *const crate::q_list_of_q_pair_of_q_string_q_string::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Gt<
            ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>,
        > for crate::q_list_of_q_pair_of_q_string_q_string::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPair<QString, QString>>::iterator::operator>(const QList<QPair<QString, QString>>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_iterator_operator_4(
                    self as *const crate::q_list_of_q_pair_of_q_string_q_string::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Ge<
            ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>,
        > for crate::q_list_of_q_pair_of_q_string_q_string::Iterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPair<QString, QString>>::iterator::operator>=(const QList<QPair<QString, QString>>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-iterator.html#operator-gt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_iterator_operator__9(
                    self as *const crate::q_list_of_q_pair_of_q_string_q_string::Iterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment for &mut crate::q_list_of_q_pair_of_q_string_q_string::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>;
        /// <p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::iterator& QList<QPair<QString, QString>>::iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++i</code>) advances the iterator to the next item in the hash and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#end">QHash::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(
            self,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_pair_of_q_string_q_string::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_iterator_operator__10(
                    self as *mut crate::q_list_of_q_pair_of_q_string_q_string::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement for &mut crate::q_list_of_q_pair_of_q_string_q_string::Iterator {
        type Output = ::cpp_utils::MutRef<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>;
        /// <p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::iterator& QList<QPair<QString, QString>>::iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--i</code>) makes the preceding item current and returns an iterator pointing to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qhash.html#begin">QHash::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(
            self,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_pair_of_q_string_q_string::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_iterator_operator__12(
                    self as *mut crate::q_list_of_q_pair_of_q_string_q_string::Iterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_pair_of_q_string_q_string::Iterator
    {
        /// <p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::iterator& QList<QPair<QString, QString>>::iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items. (If <i>j</i> is negative, the iterator goes backward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_iterator_operator__14(
                    self as *mut crate::q_list_of_q_pair_of_q_string_q_string::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_pair_of_q_string_q_string::Iterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::iterator& QList<QPair<QString, QString>>::iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```iterator &iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items. (If <i>j</i> is negative, the iterator goes forward.)</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qhash-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_iterator_operator__15(
                    self as *mut crate::q_list_of_q_pair_of_q_string_q_string::Iterator,
                    j,
                )
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_pair_of_q_string_q_string::Iterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::iterator QList<QPair<QString, QString>>::iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_iterator_operator_5(
                    self as *const crate::q_list_of_q_pair_of_q_string_q_string::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_pair_of_q_string_q_string::Iterator
    {
        type Output = ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::iterator QList<QPair<QString, QString>>::iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::Iterator> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_iterator_operator_6(
                    self as *const crate::q_list_of_q_pair_of_q_string_q_string::Iterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>>
        for &crate::q_list_of_q_pair_of_q_string_q_string::Iterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QPair<QString, QString>>::iterator::operator-(QList<QPair<QString, QString>>::iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_iterator_operator_7(
                    self as *const crate::q_list_of_q_pair_of_q_string_q_string::Iterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Indirection
        for &crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator
    {
        type Output = ::cpp_utils::Ref<crate::QPairOfQStringQString>;
        /// <p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```const QPair<QString, QString>& QList<QPair<QString, QString>>::const_iterator::operator*() const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current value, converted to a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p></div>
        fn indirection(self) -> ::cpp_utils::Ref<crate::QPairOfQStringQString> {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_const_iterator_operator_(
                    self as *const crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
            }
        }
    }

    impl
        ::std::cmp::PartialEq<
            ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>,
        > for crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator
    {
        /// <p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPair<QString, QString>>::const_iterator::operator==(const QList<QPair<QString, QString>>::const_iterator& o) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> points to the same item as this iterator; otherwise returns <code>false</code>.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-not-eq">operator!=</a>().</p></div>
        fn eq(
            &self,
            o: &::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_const_iterator_operator__2(
                    self as *const crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator,
                    o.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Lt<
            ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>,
        > for crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPair<QString, QString>>::const_iterator::operator<(const QList<QPair<QString, QString>>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than the item pointed to by the <i>other</i> iterator.</p></div>
        fn lt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_const_iterator_operator_1(
                    self as *const crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Le<
            ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>,
        > for crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPair<QString, QString>>::const_iterator::operator<=(const QList<QPair<QString, QString>>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is less than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn le(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_const_iterator_operator__4(
                    self as *const crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Gt<
            ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>,
        > for crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPair<QString, QString>>::const_iterator::operator>(const QList<QPair<QString, QString>>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than the item pointed to by the <i>other</i> iterator.</p></div>
        fn gt(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_const_iterator_operator_2(
                    self as *const crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl
        ::cpp_utils::cmp::Ge<
            ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>,
        > for crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator
    {
        /// <p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```bool QList<QPair<QString, QString>>::const_iterator::operator>=(const QList<QPair<QString, QString>>::const_iterator& other) const```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qjsonarray-const-iterator.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the item pointed to by this iterator is greater than or equal to the item pointed to by the <i>other</i> iterator.</p></div>
        fn ge(
            &self,
            other: &::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>,
        ) -> bool {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_const_iterator_operator__5(
                    self as *const crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator,
                    other.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::ops::Increment
        for &mut crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator
    {
        type Output =
            ::cpp_utils::MutRef<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>;
        /// <p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::const_iterator& QList<QPair<QString, QString>>::const_iterator::operator++()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix ++ operator (<code>++it</code>) advances the iterator to the next item in the container and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">QAssociativeIterable::end</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">operator--</a>().</p></div>
        fn inc(
            self,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>
        {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_const_iterator_operator__6(
                    self as *mut crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::cpp_utils::ops::Decrement
        for &mut crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator
    {
        type Output =
            ::cpp_utils::MutRef<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>;
        /// <p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::const_iterator& QList<QPair<QString, QString>>::const_iterator::operator--()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The prefix -- operator (<code>--it</code>) makes the preceding item current and returns an iterator to the new current item.</p>
        /// <p>Calling this function on <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">QAssociativeIterable::begin</a>() leads to undefined results.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-2b">operator++</a>().</p></div>
        fn dec(
            self,
        ) -> ::cpp_utils::MutRef<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>
        {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_const_iterator_operator__8(
                    self as *mut crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator,
                )
            };
            unsafe {
                ::cpp_utils::MutRef::from_raw(ffi_result)
                    .expect("attempted to construct a null Ref")
            }
        }
    }

    impl ::std::ops::AddAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator
    {
        /// <p>Advances the iterator by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::const_iterator& QList<QPair<QString, QString>>::const_iterator::operator+=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator+=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Advances the iterator by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">operator-=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b">operator+</a>().</p></div>
        fn add_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_const_iterator_operator__10(self as *mut crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator, j)
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::SubAssign<::std::os::raw::c_longlong>
        for crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator
    {
        /// <p>Makes the iterator go back by <i>j</i> items.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::const_iterator& QList<QPair<QString, QString>>::const_iterator::operator-=(long long j)```</span>.
        ///
        /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator--eq">C++ documentation</a> for <span style='color: green;'>```const_iterator &const_iterator::operator-=(int j)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Makes the iterator go back by <i>j</i> items.</p>
        /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html#operator-">operator-</a>().</p></div>
        fn sub_assign(&mut self, j: ::std::os::raw::c_longlong) {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_const_iterator_operator__11(self as *mut crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator, j)
            };
            let _ = ffi_result;
        }
    }

    impl ::std::ops::Add<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator
    {
        type Output =
            ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::const_iterator QList<QPair<QString, QString>>::const_iterator::operator+(long long j) const```</span>.
        fn add(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>
        {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_const_iterator_operator_3(
                    self as *const crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl ::std::ops::Sub<::std::os::raw::c_longlong>
        for &crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator
    {
        type Output =
            ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>;
        /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::const_iterator QList<QPair<QString, QString>>::const_iterator::operator-(long long j) const```</span>.
        fn sub(
            self,
            j: ::std::os::raw::c_longlong,
        ) -> ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>
        {
            let ffi_result = unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_const_iterator_operator_4(
                    self as *const crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator,
                    j,
                )
            };
            unsafe {
                ::cpp_utils::CppBox::from_raw(ffi_result)
                    .expect("attempted to construct a null CppBox")
            }
        }
    }

    impl
        ::std::ops::Sub<
            ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>,
        > for &crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator
    {
        type Output = ::std::os::raw::c_int;
        /// Calls C++ function: <span style='color: green;'>```int QList<QPair<QString, QString>>::const_iterator::operator-(QList<QPair<QString, QString>>::const_iterator j) const```</span>.
        fn sub(
            self,
            j: ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>,
        ) -> ::std::os::raw::c_int {
            unsafe {
                crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_const_iterator_operator_5(
                    self as *const crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator,
                    j.as_raw_ptr(),
                )
            }
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_pair_of_q_string_q_string::Iterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QPair<QString, QString>>::iterator::~iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> class provides an STL-style non-const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> features both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;) and to modify the list item associated with the iterator. If you want to iterate over a const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#begin">QList::begin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#end">QList::end</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Let's see a few examples of things we can do with a <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> that we cannot do with a <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a>. Here's an example that increments every value stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;int&gt; by 2:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span><span class="operator">::</span>iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>begin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>end(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     <span class="operator">*</span>i <span class="operator">+</span><span class="operator">=</span> <span class="number">2</span>;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>begin()<span class="operator">,</span> list<span class="operator">.</span>end());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_iterator_diterator(
                self as *mut crate::q_list_of_q_pair_of_q_string_q_string::Iterator,
            )
        }
    }

    impl ::cpp_utils::CppDeletable for crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator {
        /// <p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        ///
        /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QPair<QString, QString>>::const_iterator::~const_iterator()```</span>.
        ///
        /// <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> class provides an STL-style const iterator for <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> and <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides both <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> and <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a>. The STL-style iterators are more low-level and more cumbersome to use; on the other hand, they are slightly faster and, for developers who already know STL, have the advantage of familiarity.</p>
        /// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;::const_iterator allows you to iterate over a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; (or a <a href="http://doc.qt.io/qt-5/qqueue.html">QQueue</a>&lt;T&gt;). If you want to modify the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as you iterate over it, use <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a> instead. It is generally good practice to use <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> on a non-const <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as well, unless you need to change the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> through the iterator. Const iterators are slightly faster, and can improve code readability.</p>
        /// <p>The default <a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> constructor creates an uninitialized iterator. You must initialize it using a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> function like <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">QList::constBegin</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">QList::constEnd</a>(), or <a href="http://doc.qt.io/qt-5/qlist.html#insert">QList::insert</a>() before you can start iterating. Here's a typical loop that prints all the items stored in a list:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
        ///   list<span class="operator">.</span>append(<span class="string">"January"</span>);
        ///   list<span class="operator">.</span>append(<span class="string">"February"</span>);
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   list<span class="operator">.</span>append(<span class="string">"December"</span>);
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i;
        ///   <span class="keyword">for</span> (i <span class="operator">=</span> list<span class="operator">.</span>constBegin(); i <span class="operator">!</span><span class="operator">=</span> list<span class="operator">.</span>constEnd(); <span class="operator">+</span><span class="operator">+</span>i)
        /// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
        ///
        /// </pre>
        /// <p>Most <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> functions accept an integer index rather than an iterator. For that reason, iterators are rarely useful in connection with <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. One place where STL-style iterators do make sense is as arguments to <a href="http://doc.qt.io/qt-5/qtalgorithms.html">generic algorithms</a>.</p>
        /// <p>For example, here's how to delete all the widgets stored in a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt;:</p>
        /// <pre class="cpp">
        ///
        ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
        ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
        ///   <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qDeleteAll">qDeleteAll</a>(list<span class="operator">.</span>constBegin()<span class="operator">,</span> list<span class="operator">.</span>constEnd());
        ///
        /// </pre>
        /// <p>Multiple iterators can be used on the same list. However, be aware that any non-const function call performed on the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> will render all existing iterators undefined. If you need to keep iterators over a long period of time, we recommend that you use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> rather than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>.</p>
        /// <p><b>Warning:</b> Iterators on implicitly shared containers do not work exactly like STL-iterators. You should avoid copying a container while iterators are active on that container. For more information, read <a href="http://doc.qt.io/qt-5/containers.html#implicit-sharing-iterator-problem">Implicit sharing iterator problem</a>.</p></div>
        unsafe fn delete(&mut self) {
            crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_const_iterator_dconst_iterator(
                self as *mut crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator,
            )
        }
    }

}
/// <p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
///
/// C++ class: <span style='color: green;'>```QList<QPair<QString, QString>>```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qlist.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> class is a template class that provides lists.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores items in a list that provides fast index-based access and index-based insertions and removals.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>&lt;T&gt;, and <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; provide similar APIs and functionality. They are often interchangeable, but there are performance consequences. Here is an overview of use cases:</p>
/// <ul>
/// <li><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> should be your default first choice. <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; will usually give better performance than <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt;, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a>&lt;T&gt; always stores its items sequentially in memory, where <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; will allocate its items on the heap unless <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. See the <a href="http://marcmutz.wordpress.com/effective-qt/containers/#containers-qlist">Pros and Cons of Using QList</a> for an explanation.</li>
/// <li>However, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is used throughout the Qt APIs for passing parameters and for returning values. Use <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> to interface with those APIs.</li>
/// <li>If you need a real linked list, which guarantees <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> insertions mid-list and uses iterators to items rather than indexes, use <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a>.</li>
/// </ul>
/// <p><b>Note: </b><a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and <a href="http://doc.qt.io/qt-5/qvarlengtharray.html">QVarLengthArray</a> both guarantee C-compatible array layout. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does not. This might be important if your application must interface with a C API.</p><p><b>Note: </b>Iterators into a <a href="http://doc.qt.io/qt-5/qlinkedlist.html">QLinkedList</a> and references into heap-allocating QLists remain valid as long as the referenced items remain in the container. This is not true for iterators and references into a <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> and non-heap-allocating QLists.</p><p>Internally, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T if <code>sizeof(T) &lt;= sizeof(void*)</code> and T has been declared to be either a <code>Q_MOVABLE_TYPE</code> or a <code>Q_PRIMITIVE_TYPE</code> using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_DECLARE_TYPEINFO">Q_DECLARE_TYPEINFO</a>. Otherwise, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;T&gt; is represented as an array of T* and the items are allocated on the heap.</p>
/// <p>The array representation allows very fast insertions and index-based access. The <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() operations are also very fast because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates memory at both ends of its internal array. (See <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">Algorithmic Complexity</a> for details.</p>
/// <p>Note, however, that when the conditions specified above are not met, each append or insert of a new item requires allocating the new item on the heap, and this per item allocation will make <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> a better choice for use cases that do a lot of appending or inserting, because <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> can allocate memory for many items in a single heap allocation.</p>
/// <p>Note that the internal array only ever gets bigger over the life of the list. It never shrinks. The internal array is deallocated by the destructor and by the assignment operator, when one list is assigned to another.</p>
/// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores integers and a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> that stores <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> values:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> integerList;
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a></span><span class="operator">&gt;</span> dateList;
///
/// </pre>
/// <p>Qt includes a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class that inherits <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>&gt; and adds a few convenience functions, such as <a href="http://doc.qt.io/qt-5/qstringlist.html#join">QStringList::join</a>() and <a href="http://doc.qt.io/qt-5/qstringlist.html#filter">QStringList::filter</a>(). <a href="http://doc.qt.io/qt-5/qstring.html#split">QString::split</a>() creates QStringLists from strings.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> stores a list of items. The default constructor creates an empty list. To insert items into the list, you can use operator&lt;&lt;():</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"one"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"two"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"three"</span>;
///   <span class="comment">// list: ["one", "two", "three"]</span>
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides these basic functions to add, move, and remove items: <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(). In addition, it provides the following convenience functions: <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. To access the item at a particular index position, you can use operator[](). On non-const lists, operator[]() returns a reference to the item and can be used on the left side of an assignment:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span><span class="operator">=</span> <span class="string">"Bob"</span>)
/// &#32;     list<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="string">"Robert"</span>;
///
/// </pre>
/// <p>Because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented as an array of pointers for types that are larger than a pointer or are not movable, this operation requires (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>). For read-only access, an alternative syntax is to use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>():</p>
/// <pre class="cpp">
///
///   <span class="keyword">for</span> (<span class="type">int</span> i <span class="operator">=</span> <span class="number">0</span>; i <span class="operator">&lt;</span> list<span class="operator">.</span>size(); <span class="operator">+</span><span class="operator">+</span>i) {
/// &#32;     <span class="keyword">if</span> (list<span class="operator">.</span>at(i) <span class="operator">=</span><span class="operator">=</span> <span class="string">"Jane"</span>)
/// &#32;   &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Found Jane at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///   }
///
/// </pre>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() can be faster than operator[](), because it never causes a <a href="http://doc.qt.io/qt-5/implicit-sharing.html#deep-copy">deep copy</a> to occur.</p>
/// <p>A common requirement is to remove an item from a list and do something with it. For this, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(). Here's a loop that removes the items from a list one at a time and calls <code>delete</code> on them:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> list;
///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
///   <span class="keyword">while</span> (<span class="operator">!</span>list<span class="operator">.</span>isEmpty())
/// &#32;     <span class="keyword">delete</span> list<span class="operator">.</span>takeFirst();
///
/// </pre>
/// <p>Inserting and removing items at either end of the list is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a> in most cases), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
/// <p>If you want to find all occurrences of a particular value in a list, use <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() or <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>(). The former searches forward starting from a given index position, the latter searches backward. Both return the index of a matching item if they find it; otherwise, they return -1. For example:</p>
/// <pre class="cpp">
///
///   <span class="type">int</span> i <span class="operator">=</span> list<span class="operator">.</span>indexOf(<span class="string">"Jane"</span>);
///   <span class="keyword">if</span> (i <span class="operator">!</span><span class="operator">=</span> <span class="operator">-</span><span class="number">1</span>)
/// &#32;     cout <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"First occurrence of Jane is at position "</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> i <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
///
/// </pre>
/// <p>If you simply want to check whether a list contains a particular value, use <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>(). If you want to find out how many times a particular value occurs in the list, use <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>(). If you want to replace all occurrences of a particular value with another, use <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s value type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. This covers most data types that are commonly used, but the compiler won't let you, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; for example, <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() expect the value type to support <code>operator==()</code>. These requirements are documented on a per-function basis.</p>
/// <p>Like the other container classes, <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> provides <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlistiterator.html">QListIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablelistiterator.html">QMutableListIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qlist-const-iterator.html">QList::const_iterator</a> and <a href="http://doc.qt.io/qt-5/qlist-iterator.html">QList::iterator</a>). In practice, these are rarely used, because you can use indexes into the <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>. <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is implemented in such a way that direct index-based access is just as fast as using iterators.</p>
/// <p><a href="http://doc.qt.io/qt-5/qlist.html">QList</a> does <i>not</i> support inserting, prepending, appending or replacing with references to its own values. Doing so will cause your application to abort with an error message.</p>
/// <p>To make <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> as efficient as possible, its member functions don't validate their input before using it. Except for <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), member functions always assume the list is <i>not</i> empty. Member functions that take index values as parameters always assume their index value parameters are in the valid range. This means <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> member functions can fail. If you define QT_NO_DEBUG when you compile, failures will not be detected. If you <i>don't</i> define QT_NO_DEBUG, failures will be detected using <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT">Q_ASSERT</a>() or <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_ASSERT_X">Q_ASSERT_X</a>() with an appropriate message.</p>
/// <p>To avoid failures when your list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling other member functions. If you must pass an index value that might not be in the valid range, check that it is less than the value returned by <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() but <i>not</i> less than 0.</p>
/// <a name="more-members"></a></div>
#[repr(C)]
pub struct QListOfQPairOfQStringQString {
    _unused: u8,
}
impl QListOfQPairOfQStringQString {
    /// <p>Inserts <i>value</i> at the end of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPair<QString, QString>>::append(const QPair<QString, QString>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the end of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>append(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>append(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["one", "two", "three"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn append_q_pair_of_q_string_q_string(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQStringQString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_append(
            self as *mut crate::QListOfQPairOfQStringQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQStringQString>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPair<QString, QString>>::append(const QList<QPair<QString, QString>>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#append-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the items of the <i>value</i> list to this list.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    pub unsafe fn append_q_list_of_q_pair_of_q_string_q_string(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_append1(self as *mut crate::QListOfQPairOfQStringQString, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>>::cast_into(t).as_raw_ptr())
    }

    /// <p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPair<QString, QString>& QList<QPair<QString, QString>>::at(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#at">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> in the list. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>This function is very fast (<a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn at(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ref<crate::QPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_at(
            self as *const crate::QListOfQPairOfQStringQString,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPair<QString, QString>& QList<QPair<QString, QString>>::back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn back_mut(&mut self) -> ::cpp_utils::MutRef<crate::QPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_back(
            self as *mut crate::QListOfQPairOfQStringQString,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPair<QString, QString>& QList<QPair<QString, QString>>::back() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#back-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn back(&self) -> ::cpp_utils::Ref<crate::QPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_back1(
            self as *const crate::QListOfQPairOfQStringQString,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::const_iterator QList<QPair<QString, QString>>::cbegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cbegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#cend">cend</a>().</p></div>
    pub unsafe fn cbegin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_cbegin(
            self as *const crate::QListOfQPairOfQStringQString,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::const_iterator QList<QPair<QString, QString>>::cend() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#cend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p>This function was introduced in  Qt 5.0.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#cbegin">cbegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn cend(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_cend(
            self as *const crate::QListOfQPairOfQStringQString,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes all items from the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPair<QString, QString>>::clear()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#clear">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all items from the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>().</p></div>
    pub unsafe fn clear(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_clear(
            self as *mut crate::QListOfQPairOfQStringQString,
        )
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::const_iterator QList<QPair<QString, QString>>::constBegin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constBegin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    pub unsafe fn const_begin(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_constBegin(
            self as *const crate::QListOfQPairOfQStringQString,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::const_iterator QList<QPair<QString, QString>>::constEnd() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    pub unsafe fn const_end(
        &self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_constEnd(
            self as *const crate::QListOfQPairOfQStringQString,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPair<QString, QString>& QList<QPair<QString, QString>>::constFirst() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>().</p></div>
    pub unsafe fn const_first(&self) -> ::cpp_utils::Ref<crate::QPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_constFirst(
            self as *const crate::QListOfQPairOfQStringQString,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPair<QString, QString>& QList<QPair<QString, QString>>::constLast() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#constLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>This function was introduced in  Qt 5.6.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>().</p></div>
    pub unsafe fn const_last(&self) -> ::cpp_utils::Ref<crate::QPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_constLast(
            self as *const crate::QListOfQPairOfQStringQString,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QPair<QString, QString>>::contains(const QPair<QString, QString>& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#contains">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains an occurrence of <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn contains(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQStringQString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_contains(
            self as *const crate::QListOfQPairOfQStringQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQStringQString>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Assigns <i>other</i> to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>& QList<QPair<QString, QString>>::operator=(const QList<QPair<QString, QString>>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Assigns <i>other</i> to this list and returns a reference to this list.</p></div>
    pub unsafe fn copy_from(
        &mut self,
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_operator_(self as *mut crate::QListOfQPairOfQStringQString, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>>::cast_into(l).as_raw_ptr());
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the number of occurrences of <i>value</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QPair<QString, QString>>::count(const QPair<QString, QString>& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of occurrences of <i>value</i> in the list.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn count_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQStringQString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_count(
            self as *const crate::QListOfQPairOfQStringQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQStringQString>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QPair<QString, QString>>::count() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#count-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list. This is effectively the same as <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn count_0a(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_count1(
            self as *const crate::QListOfQPairOfQStringQString,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QPair<QString, QString>>::detach()```</span>.
    pub unsafe fn detach(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_detach(
            self as *mut crate::QListOfQPairOfQStringQString,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QPair<QString, QString>>::detachShared()```</span>.
    pub unsafe fn detach_shared(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_detachShared(
            self as *mut crate::QListOfQPairOfQStringQString,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QPair<QString, QString>>::empty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#empty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and returns <code>true</code> if the list is empty.</p></div>
    pub unsafe fn empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_empty(
            self as *const crate::QListOfQPairOfQStringQString,
        )
    }

    /// <p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QPair<QString, QString>>::endsWith(const QPair<QString, QString>& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#endsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its last item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn ends_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQStringQString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_endsWith(
            self as *const crate::QListOfQPairOfQStringQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQStringQString>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::iterator QList<QPair<QString, QString>>::erase(QList<QPair<QString, QString>>::iterator pos)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item associated with the iterator <i>pos</i> from the list, and returns an iterator to the next item in the list (which may be <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn erase_1a(
        &mut self,
        pos: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>,
        >,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_erase(
            self as *mut crate::QListOfQPairOfQStringQString,
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>,
            >::cast_into(pos)
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::iterator QList<QPair<QString, QString>>::erase(QList<QPair<QString, QString>>::iterator first, QList<QPair<QString, QString>>::iterator last)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#erase-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Removes all the items from <i>begin</i> up to (but not including) <i>end</i>. Returns an iterator to the same item that <i>end</i> referred to before the call.</p></div>
    pub unsafe fn erase_2a(
        &mut self,
        first: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>,
        >,
        last: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>,
        >,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_erase1(
            self as *mut crate::QListOfQPairOfQStringQString,
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>,
            >::cast_into(first)
            .as_raw_ptr(),
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>,
            >::cast_into(last)
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPair<QString, QString>& QList<QPair<QString, QString>>::first()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the first item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constFirst">constFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#last">last</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn first_mut(&mut self) -> ::cpp_utils::MutRef<crate::QPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_first(
            self as *mut crate::QListOfQPairOfQStringQString,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPair<QString, QString>& QList<QPair<QString, QString>>::first() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#first-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn first(&self) -> ::cpp_utils::Ref<crate::QPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_first1(
            self as *const crate::QListOfQPairOfQStringQString,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPair<QString, QString>& QList<QPair<QString, QString>>::front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn front_mut(&mut self) -> ::cpp_utils::MutRef<crate::QPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_front(
            self as *mut crate::QListOfQPairOfQStringQString,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPair<QString, QString>& QList<QPair<QString, QString>>::front() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#front-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn front(&self) -> ::cpp_utils::Ref<crate::QPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_front1(
            self as *const crate::QListOfQPairOfQStringQString,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPair<QString, QString>& QList<QPair<QString, QString>>::operator[](int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Same as <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>(). This function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p></div>
    pub unsafe fn index(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::Ref<crate::QPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_operator__2(
            self as *const crate::QListOfQPairOfQStringQString,
            i,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPair<QString, QString>& QList<QPair<QString, QString>>::operator[](int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the item at index position <i>i</i> as a modifiable reference. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If this function is called on a list that is currently being shared, it will trigger a copy of all elements. Otherwise, this function runs in <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>. If you do not want to modify the list you should use <a href="http://doc.qt.io/qt-5/qlist.html#at">QList::at</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#value">value</a>().</p></div>
    pub unsafe fn index_mut(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::MutRef<crate::QPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_operator__3(
            self as *mut crate::QListOfQPairOfQStringQString,
            i,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QPair<QString, QString>>::indexOf(const QPair<QString, QString>& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQStringQString>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_indexOf(
            self as *const crate::QListOfQPairOfQStringQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQStringQString>>::cast_into(t)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QPair<QString, QString>>::indexOf(const QPair<QString, QString>& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#indexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the first occurrence of <i>value</i> in the list, searching forward from index position <i>from</i>. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span>);          <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">1</span>);       <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);       <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>indexOf(<span class="string">"X"</span>);          <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">lastIndexOf</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQStringQString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_indexOf1(
            self as *const crate::QListOfQPairOfQStringQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQStringQString>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPair<QString, QString>>::insert(int i, const QPair<QString, QString>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at index position <i>i</i> in the list. If <i>i</i> &lt;= 0, the value is prepended to the list. If <i>i</i> &gt;= <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(), the value is appended to the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"alpha"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"beta"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"delta"</span>;
    ///   list<span class="operator">.</span>insert(<span class="number">2</span><span class="operator">,</span> <span class="string">"gamma"</span>);
    ///   <span class="comment">// list: ["alpha", "beta", "gamma", "delta"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn insert_int_q_pair_of_q_string_q_string(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQStringQString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_insert(
            self as *mut crate::QListOfQPairOfQStringQString,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQStringQString>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::iterator QList<QPair<QString, QString>>::insert(QList<QPair<QString, QString>>::iterator before, const QPair<QString, QString>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#insert-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Inserts <i>value</i> in front of the item pointed to by the iterator <i>before</i>. Returns an iterator pointing at the inserted item. Note that the iterator passed to the function will be invalid after the call; the returned iterator should be used instead.</p></div>
    pub unsafe fn insert_iterator_q_pair_of_q_string_q_string(
        &mut self,
        before: impl ::cpp_utils::CastInto<
            ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>,
        >,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQStringQString>>,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::Iterator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_insert1(
            self as *mut crate::QListOfQPairOfQStringQString,
            ::cpp_utils::CastInto::<
                ::cpp_utils::Ref<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>,
            >::cast_into(before)
            .as_raw_ptr(),
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQStringQString>>::cast_into(t)
                .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QPair<QString, QString>>::isDetached() const```</span>.
    pub unsafe fn is_detached(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_isDetached(
            self as *const crate::QListOfQPairOfQStringQString,
        )
    }

    /// <p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QPair<QString, QString>>::isEmpty() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the list contains no items; otherwise returns false.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p></div>
    pub unsafe fn is_empty(&self) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_isEmpty(
            self as *const crate::QListOfQPairOfQStringQString,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```bool QList<QPair<QString, QString>>::isSharedWith(const QList<QPair<QString, QString>>& other) const```</span>.
    pub unsafe fn is_shared_with(
        &self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_isSharedWith(self as *const crate::QListOfQPairOfQStringQString, ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>>::cast_into(other).as_raw_ptr())
    }

    /// <p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPair<QString, QString>& QList<QPair<QString, QString>>::last()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a reference to the last item in the list. The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constLast">constLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#first">first</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>().</p></div>
    pub unsafe fn last_mut(&mut self) -> ::cpp_utils::MutRef<crate::QPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_last(
            self as *mut crate::QListOfQPairOfQStringQString,
        );
        ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QPair<QString, QString>& QList<QPair<QString, QString>>::last() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#last-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    pub unsafe fn last(&self) -> ::cpp_utils::Ref<crate::QPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_last1(
            self as *const crate::QListOfQPairOfQStringQString,
        );
        ::cpp_utils::Ref::from_raw(ffi_result).expect("attempted to construct a null Ref")
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QPair<QString, QString>>::lastIndexOf(const QPair<QString, QString>& t, int from = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_2a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQStringQString>>,
        from: ::std::os::raw::c_int,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_lastIndexOf(
            self as *const crate::QListOfQPairOfQStringQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQStringQString>>::cast_into(t)
                .as_raw_ptr(),
            from,
        )
    }

    /// <p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QPair<QString, QString>>::lastIndexOf(const QPair<QString, QString>& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#lastIndexOf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the index position of the last occurrence of <i>value</i> in the list, searching backward from index position <i>from</i>. If <i>from</i> is -1 (the default), the search starts at the last item. Returns -1 if no item matched.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span>;
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span>);      <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">3</span>);   <span class="comment">// returns 3</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"B"</span><span class="operator">,</span> <span class="number">2</span>);   <span class="comment">// returns 1</span>
    ///   list<span class="operator">.</span>lastIndexOf(<span class="string">"X"</span>);      <span class="comment">// returns -1</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>Note that <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> uses 0-based indexes, just like C++ arrays. Negative indexes are not supported with the exception of the value mentioned above.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#indexOf">indexOf</a>().</p></div>
    pub unsafe fn last_index_of_1a(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQStringQString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_lastIndexOf1(
            self as *const crate::QListOfQPairOfQStringQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQStringQString>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QPair<QString, QString>>::length() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#length">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is identical to <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn length(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_length(
            self as *const crate::QListOfQPairOfQStringQString,
        )
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>> QList<QPair<QString, QString>>::mid(int pos, int length = …) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_2a(
        &self,
        pos: ::std::os::raw::c_int,
        length: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_mid(
            self as *const crate::QListOfQPairOfQStringQString,
            pos,
            length,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>> QList<QPair<QString, QString>>::mid(int pos) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#mid">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a sub-list which includes elements from this list, starting at position <i>pos</i>. If <i>length</i> is -1 (the default), all elements from <i>pos</i> are included; otherwise <i>length</i> elements (or all remaining elements if there are less than <i>length</i> elements) are included.</p></div>
    pub unsafe fn mid_1a(
        &self,
        pos: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QListOfQPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_mid1(
            self as *const crate::QListOfQPairOfQStringQString,
            pos,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPair<QString, QString>>::move(int from, int to)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#move">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Moves the item at index position <i>from</i> to index position <i>to</i>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>move(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "C", "D", "E", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This is the same as insert(<i>to</i>, <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<i>from</i>)).This function assumes that both <i>from</i> and <i>to</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>from</i> and <i>to</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#swap">swap</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>().</p></div>
    pub unsafe fn move_(&mut self, from: ::std::os::raw::c_int, to: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_move(
            self as *mut crate::QListOfQPairOfQStringQString,
            from,
            to,
        )
    }

    /// <p>Constructs an empty list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QPair<QString, QString>>::QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs an empty list.</p></div>
    pub unsafe fn new() -> ::cpp_utils::CppBox<crate::QListOfQPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_QList();
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Constructs a copy of <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[constructor] void QList<QPair<QString, QString>>::QList(const QList<QPair<QString, QString>>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#QList-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Constructs a copy of <i>other</i>.</p>
    /// <p>This operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>, because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. This makes returning a <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> from a function very fast. If a shared instance is modified, it will be copied (copy-on-write), and that takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">linear time</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-eq">operator=</a>().</p></div>
    pub unsafe fn new_copy(
        l: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_QList1(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>>::cast_into(l).as_raw_ptr());
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPair<QString, QString>>::pop_back()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_back(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_pop_back(
            self as *mut crate::QListOfQPairOfQStringQString,
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPair<QString, QString>>::pop_front()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#pop_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p></div>
    pub unsafe fn pop_front(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_pop_front(
            self as *mut crate::QListOfQPairOfQStringQString,
        )
    }

    /// <p>Inserts <i>value</i> at the beginning of the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPair<QString, QString>>::prepend(const QPair<QString, QString>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#prepend">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Inserts <i>value</i> at the beginning of the list.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list<span class="operator">.</span>prepend(<span class="string">"one"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"two"</span>);
    ///   list<span class="operator">.</span>prepend(<span class="string">"three"</span>);
    ///   <span class="comment">// list: ["three", "two", "one"]</span>
    ///
    /// </pre>
    /// <p>This is the same as list.insert(0, <i>value</i>).</p>
    /// <p>If this list is not shared, this operation is typically very fast (amortized <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>), because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> preallocates extra space on both sides of its internal buffer to allow for fast growth at both ends of the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>().</p></div>
    pub unsafe fn prepend(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQStringQString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_prepend(
            self as *mut crate::QListOfQPairOfQStringQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQStringQString>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPair<QString, QString>>::push_back(const QPair<QString, QString>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_back">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(<i>value</i>).</p></div>
    pub unsafe fn push_back(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQStringQString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_push_back(
            self as *mut crate::QListOfQPairOfQStringQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQStringQString>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPair<QString, QString>>::push_front(const QPair<QString, QString>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#push_front">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function is provided for STL compatibility. It is equivalent to <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(<i>value</i>).</p></div>
    pub unsafe fn push_front(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQStringQString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_push_front(
            self as *mut crate::QListOfQPairOfQStringQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQStringQString>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QPair<QString, QString>>::removeAll(const QPair<QString, QString>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes all occurrences of <i>value</i> in the list and returns the number of entries removed.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeAll(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_all(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQStringQString>>,
    ) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_removeAll(
            self as *mut crate::QListOfQPairOfQStringQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQStringQString>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPair<QString, QString>>::removeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>().</p></div>
    pub unsafe fn remove_at(&mut self, i: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_removeAt(
            self as *mut crate::QListOfQPairOfQStringQString,
            i,
        )
    }

    /// <p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPair<QString, QString>>::removeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(0). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>().</p></div>
    pub unsafe fn remove_first(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_removeFirst(
            self as *mut crate::QListOfQPairOfQStringQString,
        )
    }

    /// <p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPair<QString, QString>>::removeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list. Calling this function is equivalent to calling <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). The list must not be empty. If the list can be empty, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn remove_last(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_removeLast(
            self as *mut crate::QListOfQPairOfQStringQString,
        )
    }

    /// <p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QPair<QString, QString>>::removeOne(const QPair<QString, QString>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first occurrence of <i>value</i> in the list and returns true on success; otherwise returns <code>false</code>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"cloud"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"sun"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"rain"</span>;
    ///   list<span class="operator">.</span>removeOne(<span class="string">"sun"</span>);
    ///   <span class="comment">// list: ["cloud", ,"sun", "rain"]</span>
    ///
    /// </pre>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p>This function was introduced in  Qt 4.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>().</p></div>
    pub unsafe fn remove_one(
        &mut self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQStringQString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_removeOne(
            self as *mut crate::QListOfQPairOfQStringQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQStringQString>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPair<QString, QString>>::replace(int i, const QPair<QString, QString>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#replace">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Replaces the item at index position <i>i</i> with <i>value</i>. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>().</p></div>
    pub unsafe fn replace(
        &mut self,
        i: ::std::os::raw::c_int,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQStringQString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_replace(
            self as *mut crate::QListOfQPairOfQStringQString,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQStringQString>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Reserve space for <i>alloc</i> elements.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPair<QString, QString>>::reserve(int size)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#reserve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reserve space for <i>alloc</i> elements.</p>
    /// <p>If <i>alloc</i> is smaller than the current size of the list, nothing will happen.</p>
    /// <p>Use this function to avoid repetetive reallocation of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s internal data if you can predict how many elements will be appended. Note that the reservation applies only to the internal pointer array.</p>
    /// <p>This function was introduced in  Qt 4.7.</p></div>
    pub unsafe fn reserve(&mut self, size: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_reserve(
            self as *mut crate::QListOfQPairOfQStringQString,
            size,
        )
    }

    /// Calls C++ function: <span style='color: green;'>```void QList<QPair<QString, QString>>::setSharable(bool sharable)```</span>.
    pub unsafe fn set_sharable(&mut self, sharable: bool) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_setSharable(
            self as *mut crate::QListOfQPairOfQStringQString,
            sharable,
        )
    }

    /// <p>Returns the number of items in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```int QList<QPair<QString, QString>>::size() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#size">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of items in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#count-1">count</a>().</p></div>
    pub unsafe fn size(&self) -> ::std::os::raw::c_int {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_size(
            self as *const crate::QListOfQPairOfQStringQString,
        )
    }

    /// <p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QPair<QString, QString>>::startsWith(const QPair<QString, QString>& t) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#startsWith">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this list is not empty and its first item is equal to <i>value</i>; otherwise returns <code>false</code>.</p>
    /// <p>This function was introduced in  Qt 4.5.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#contains">contains</a>().</p></div>
    pub unsafe fn starts_with(
        &self,
        t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQStringQString>>,
    ) -> bool {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_startsWith(
            self as *const crate::QListOfQPairOfQStringQString,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQStringQString>>::cast_into(t)
                .as_raw_ptr(),
        )
    }

    /// <p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPair<QString, QString>>::swap(QList<QPair<QString, QString>>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Swaps list <i>other</i> with this list. This operation is very fast and never fails.</p>
    /// <p>This function was introduced in  Qt 4.8.</p></div>
    pub unsafe fn swap_1a(
        &mut self,
        other: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QListOfQPairOfQStringQString>>,
    ) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_swap(self as *mut crate::QListOfQPairOfQStringQString, ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QListOfQPairOfQStringQString>>::cast_into(other).as_mut_raw_ptr())
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPair<QString, QString>>::swap(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swap-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swap(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    pub unsafe fn swap_2a(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_swap1(
            self as *mut crate::QListOfQPairOfQStringQString,
            i,
            j,
        )
    }

    /// <p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```void QList<QPair<QString, QString>>::swapItemsAt(int i, int j)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#swapItemsAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Exchange the item at index position <i>i</i> with the item at index position <i>j</i>. This function assumes that both <i>i</i> and <i>j</i> are at least 0 but less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>(). To avoid failure, test that both <i>i</i> and <i>j</i> are at least 0 and less than <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>().</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html#QList">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> list;
    ///   list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"A"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"B"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"C"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"D"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"E"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"F"</span>;
    ///   list<span class="operator">.</span>swapItemsAt(<span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   <span class="comment">// list: ["A", "E", "C", "D", "B", "F"]</span>
    ///
    /// </pre>
    /// <p>This function was introduced in  Qt 5.13.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#move">move</a>().</p></div>
    ///
    /// This item is available if `cpp_lib_version="5.13.0"`.
    #[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
    pub unsafe fn swap_items_at(&mut self, i: ::std::os::raw::c_int, j: ::std::os::raw::c_int) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_swapItemsAt(
            self as *mut crate::QListOfQPairOfQStringQString,
            i,
            j,
        )
    }

    /// <p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPair<QString, QString> QList<QPair<QString, QString>>::takeAt(int i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the item at index position <i>i</i> and returns it. <i>i</i> must be a valid index position in the list (i.e., 0 &lt;= <i>i</i> &lt; <a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>()).</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>().</p></div>
    pub unsafe fn take_at(
        &mut self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_takeAt(
            self as *mut crate::QListOfQPairOfQStringQString,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPair<QString, QString> QList<QPair<QString, QString>>::takeFirst()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the first item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(0). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeLast">takeLast</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>().</p></div>
    pub unsafe fn take_first(&mut self) -> ::cpp_utils::CppBox<crate::QPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_takeFirst(
            self as *mut crate::QListOfQPairOfQStringQString,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPair<QString, QString> QList<QPair<QString, QString>>::takeLast()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#takeLast">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the last item in the list and returns it. This is the same as <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(<a href="http://doc.qt.io/qt-5/qlist.html#size">size</a>() - 1). This function assumes the list is not empty. To avoid failure, call <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() before calling this function.</p>
    /// <p>If this list is not shared, this operation takes <a href="http://doc.qt.io/qt-5/containers.html#algorithmic-complexity">constant time</a>.</p>
    /// <p>If you don't use the return value, <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>() is more efficient.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#takeFirst">takeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#takeAt">takeAt</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>().</p></div>
    pub unsafe fn take_last(&mut self) -> ::cpp_utils::CppBox<crate::QPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_takeLast(
            self as *mut crate::QListOfQPairOfQStringQString,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>Returns the value at index position <i>i</i> in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPair<QString, QString> QList<QPair<QString, QString>>::value(int i) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value at index position <i>i</i> in the list.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns a <a href="http://doc.qt.io/qt-5/containers.html#default-constructed-value">default-constructed value</a>. If you are certain that the index is going to be within bounds, you can use <a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() instead, which is slightly faster.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#at">at</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-5b-5d">operator[]</a>().</p></div>
    pub unsafe fn value_1a(
        &self,
        i: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_value(
            self as *const crate::QListOfQPairOfQStringQString,
            i,
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }

    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPair<QString, QString> QList<QPair<QString, QString>>::value(int i, const QPair<QString, QString>& defaultValue) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#value-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>If the index <i>i</i> is out of bounds, the function returns <i>defaultValue</i>.</p></div>
    pub unsafe fn value_2a(
        &self,
        i: ::std::os::raw::c_int,
        default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQStringQString>>,
    ) -> ::cpp_utils::CppBox<crate::QPairOfQStringQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_value1(
            self as *const crate::QListOfQPairOfQStringQString,
            i,
            ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQStringQString>>::cast_into(
                default_value,
            )
            .as_raw_ptr(),
        );
        ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QChar>> for crate::QChar {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, c2: &::cpp_utils::Ref<crate::QChar>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__(self as *const crate::QChar, c2.as_raw_ptr())
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QChar>> for crate::QChar {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<(QChar c1, QChar c2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn lt(&self, c2: &::cpp_utils::Ref<crate::QChar>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_(self as *const crate::QChar, c2.as_raw_ptr())
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QChar>> for crate::QChar {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, c2: &::cpp_utils::Ref<crate::QChar>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__2(self as *const crate::QChar, c2.as_raw_ptr())
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QChar>> for crate::QChar {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>(QChar c1, QChar c2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn gt(&self, c2: &::cpp_utils::Ref<crate::QChar>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_1(self as *const crate::QChar, c2.as_raw_ptr())
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QChar>> for crate::QChar {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, c2: &::cpp_utils::Ref<crate::QChar>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__3(self as *const crate::QChar, c2.as_raw_ptr())
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QChar>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, QChar arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QChar>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__4(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QChar>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QChar& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QChar>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__5(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QByteArray {
    /// <p>Destroys the byte array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QByteArray::~QByteArray()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#dtor.QByteArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the byte array.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QByteArray_dQByteArray(self as *mut crate::QByteArray)
    }
}

impl ::std::ops::AddAssign<::std::os::raw::c_char> for crate::QByteArray {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::operator+=(char c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#operator-2b-eq-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the character <i>ch</i> onto the end of this byte array and returns a reference to this byte array.</p></div>
    fn add_assign(&mut self, c: ::std::os::raw::c_char) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QByteArray_operator__4(self as *mut crate::QByteArray, c)
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ptr<::std::os::raw::c_char>> for crate::QByteArray {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::operator+=(const char* s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the string <i>str</i> onto the end of this byte array and returns a reference to this byte array.</p></div>
    fn add_assign(&mut self, s: ::cpp_utils::Ptr<::std::os::raw::c_char>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QByteArray_operator__5(
                self as *mut crate::QByteArray,
                s.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QByteArray>> for crate::QByteArray {
    /// <p>Appends the byte array <i>ba</i> onto the end of this byte array and returns a reference to this byte array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::operator+=(const QByteArray& a)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the byte array <i>ba</i> onto the end of this byte array and returns a reference to this byte array.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> x(<span class="string">"free"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html#QByteArray">QByteArray</a></span> y(<span class="string">"dom"</span>);
    ///   x <span class="operator">+</span><span class="operator">=</span> y;
    ///   <span class="comment">// x == "freedom"</span>
    ///
    /// </pre>
    /// <p>Note: <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> is an <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a> class. Consequently, if you append to an empty byte array, then the byte array will just share the data held in <i>ba</i>. In this case, no copying of data is done, taking <a href="http://doc.qt.io/qt-5/containers.html#constant-time">constant time</a>. If a shared instance is modified, it will be copied (copy-on-write), taking <a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>.</p>
    /// <p>If the byte array being appended to is not empty, a deep copy of the data is performed, taking <a href="http://doc.qt.io/qt-5/containers.html#linear-time">linear time</a>.</p>
    /// <p>This operation typically does not suffer from allocation overhead, because <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> preallocates extra space at the end of the data so that it may grow without reallocating for each append operation.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#prepend">prepend</a>().</p></div>
    fn add_assign(&mut self, a: ::cpp_utils::Ref<crate::QByteArray>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QByteArray_operator__6(
                self as *mut crate::QByteArray,
                a.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QString>> for crate::QByteArray {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray& QByteArray::operator+=(const QString& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#operator-2b-eq-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the string <i>str</i> onto the end of this byte array and returns a reference to this byte array. The Unicode data is converted into 8-bit characters using <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">QString::toUtf8</a>().</p>
    /// <p>You can disable this function by defining <code>QT_NO_CAST_TO_ASCII</code> when you compile your applications. You then need to call <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">QString::toUtf8</a>() (or <a href="http://doc.qt.io/qt-5/qstring.html#toLatin1">QString::toLatin1</a>() or <a href="http://doc.qt.io/qt-5/qstring.html#toLocal8Bit">QString::toLocal8Bit</a>()) explicitly if you want to convert the data to <code>const char *</code>.</p></div>
    fn add_assign(&mut self, s: ::cpp_utils::Ref<crate::QString>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QByteArray_operator__7(
                self as *mut crate::QByteArray,
                s.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QString>> for crate::QByteArray {
    /// <p>Returns <code>true</code> if this byte array is equal to string <i>str</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QByteArray::operator==(const QString& s2) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this byte array is equal to string <i>str</i>; otherwise returns <code>false</code>.</p>
    /// <p>The Unicode data is converted into 8-bit characters using <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">QString::toUtf8</a>().</p>
    /// <p>The comparison is case sensitive.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. You then need to call <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">QString::fromUtf8</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#fromLatin1">QString::fromLatin1</a>(), or <a href="http://doc.qt.io/qt-5/qstring.html#fromLocal8Bit">QString::fromLocal8Bit</a>() explicitly if you want to convert the byte array to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> before doing the comparison.</p></div>
    fn eq(&self, s2: &::cpp_utils::Ref<crate::QString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QByteArray_operator__8(
                self as *const crate::QByteArray,
                s2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QString>> for crate::QByteArray {
    /// <p>Returns <code>true</code> if this byte array is lexically less than string <i>str</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QByteArray::operator<(const QString& s2) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this byte array is lexically less than string <i>str</i>; otherwise returns <code>false</code>.</p>
    /// <p>The Unicode data is converted into 8-bit characters using <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">QString::toUtf8</a>().</p>
    /// <p>The comparison is case sensitive.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. You then need to call <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">QString::fromUtf8</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#fromLatin1">QString::fromLatin1</a>(), or <a href="http://doc.qt.io/qt-5/qstring.html#fromLocal8Bit">QString::fromLocal8Bit</a>() explicitly if you want to convert the byte array to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> before doing the comparison.</p></div>
    fn lt(&self, s2: &::cpp_utils::Ref<crate::QString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QByteArray_operator_3(
                self as *const crate::QByteArray,
                s2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QString>> for crate::QByteArray {
    /// <p>Returns <code>true</code> if this byte array is lexically greater than string <i>str</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QByteArray::operator>(const QString& s2) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this byte array is lexically greater than string <i>str</i>; otherwise returns <code>false</code>.</p>
    /// <p>The Unicode data is converted into 8-bit characters using <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">QString::toUtf8</a>().</p>
    /// <p>The comparison is case sensitive.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. You then need to call <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">QString::fromUtf8</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#fromLatin1">QString::fromLatin1</a>(), or <a href="http://doc.qt.io/qt-5/qstring.html#fromLocal8Bit">QString::fromLocal8Bit</a>() explicitly if you want to convert the byte array to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> before doing the comparison.</p></div>
    fn gt(&self, s2: &::cpp_utils::Ref<crate::QString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QByteArray_operator_4(
                self as *const crate::QByteArray,
                s2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QString>> for crate::QByteArray {
    /// <p>Returns <code>true</code> if this byte array is lexically less than or equal to string <i>str</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QByteArray::operator<=(const QString& s2) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this byte array is lexically less than or equal to string <i>str</i>; otherwise returns <code>false</code>.</p>
    /// <p>The Unicode data is converted into 8-bit characters using <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">QString::toUtf8</a>().</p>
    /// <p>The comparison is case sensitive.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. You then need to call <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">QString::fromUtf8</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#fromLatin1">QString::fromLatin1</a>(), or <a href="http://doc.qt.io/qt-5/qstring.html#fromLocal8Bit">QString::fromLocal8Bit</a>() explicitly if you want to convert the byte array to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> before doing the comparison.</p></div>
    fn le(&self, s2: &::cpp_utils::Ref<crate::QString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QByteArray_operator__10(
                self as *const crate::QByteArray,
                s2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QString>> for crate::QByteArray {
    /// <p>Returns <code>true</code> if this byte array is greater than or equal to string <i>str</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QByteArray::operator>=(const QString& s2) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this byte array is greater than or equal to string <i>str</i>; otherwise returns <code>false</code>.</p>
    /// <p>The Unicode data is converted into 8-bit characters using <a href="http://doc.qt.io/qt-5/qstring.html#toUtf8">QString::toUtf8</a>().</p>
    /// <p>The comparison is case sensitive.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. You then need to call <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">QString::fromUtf8</a>(), <a href="http://doc.qt.io/qt-5/qstring.html#fromLatin1">QString::fromLatin1</a>(), or <a href="http://doc.qt.io/qt-5/qstring.html#fromLocal8Bit">QString::fromLocal8Bit</a>() explicitly if you want to convert the byte array to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> before doing the comparison.</p></div>
    fn ge(&self, s2: &::cpp_utils::Ref<crate::QString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QByteArray_operator__11(
                self as *const crate::QByteArray,
                s2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QByteArray {
    type Output = ::cpp_utils::MutPtr<::std::os::raw::c_char>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first character in the byte-array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```char* QByteArray::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first character in the byte-array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::MutPtr<::std::os::raw::c_char> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QByteArray_begin(self as *mut crate::QByteArray)
        };
        unsafe { ::cpp_utils::MutPtr::from_raw(ffi_result) }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QByteArray {
    type Output = ::cpp_utils::Ptr<::std::os::raw::c_char>;
    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qbytearray.html#begin">begin</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QByteArray::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qbytearray.html#begin">begin</a>().</p></div>
    fn begin(self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QByteArray_begin1(self as *const crate::QByteArray)
        };
        unsafe { ::cpp_utils::Ptr::from_raw(ffi_result) }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QByteArray {
    type Output = ::cpp_utils::MutPtr<::std::os::raw::c_char>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary character after the last character in the byte-array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```char* QByteArray::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary character after the last character in the byte-array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qbytearray.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::MutPtr<::std::os::raw::c_char> {
        let ffi_result =
            unsafe { crate::__ffi::ctr_qt_core_ffi_QByteArray_end(self as *mut crate::QByteArray) };
        unsafe { ::cpp_utils::MutPtr::from_raw(ffi_result) }
    }
}

impl ::cpp_utils::ops::End for &crate::QByteArray {
    type Output = ::cpp_utils::Ptr<::std::os::raw::c_char>;
    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qbytearray.html#end">end</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QByteArray::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qbytearray.html#end">end</a>().</p></div>
    fn end(self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QByteArray_end1(self as *const crate::QByteArray)
        };
        unsafe { ::cpp_utils::Ptr::from_raw(ffi_result) }
    }
}

impl ::std::cmp::PartialEq<::std::os::raw::c_char> for crate::QByteRef {
    /// Calls C++ function: <span style='color: green;'>```bool QByteRef::operator==(char c) const```</span>.
    fn eq(&self, c: &::std::os::raw::c_char) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QByteRef_operator__(self as *const crate::QByteRef, *c)
        }
    }
}

impl ::cpp_utils::cmp::Gt<::std::os::raw::c_char> for crate::QByteRef {
    /// Calls C++ function: <span style='color: green;'>```bool QByteRef::operator>(char c) const```</span>.
    fn gt(&self, c: &::std::os::raw::c_char) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QByteRef_operator_2(self as *const crate::QByteRef, *c)
        }
    }
}

impl ::cpp_utils::cmp::Ge<::std::os::raw::c_char> for crate::QByteRef {
    /// Calls C++ function: <span style='color: green;'>```bool QByteRef::operator>=(char c) const```</span>.
    fn ge(&self, c: &::std::os::raw::c_char) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QByteRef_operator__2(self as *const crate::QByteRef, *c)
        }
    }
}

impl ::cpp_utils::cmp::Lt<::std::os::raw::c_char> for crate::QByteRef {
    /// Calls C++ function: <span style='color: green;'>```bool QByteRef::operator<(char c) const```</span>.
    fn lt(&self, c: &::std::os::raw::c_char) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QByteRef_operator_3(self as *const crate::QByteRef, *c)
        }
    }
}

impl ::cpp_utils::cmp::Le<::std::os::raw::c_char> for crate::QByteRef {
    /// Calls C++ function: <span style='color: green;'>```bool QByteRef::operator<=(char c) const```</span>.
    fn le(&self, c: &::std::os::raw::c_char) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QByteRef_operator__3(self as *const crate::QByteRef, *c)
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QByteArray>> for crate::QByteArray {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(const QByteArray& a1, const QByteArray& a2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, a2: &::cpp_utils::Ref<crate::QByteArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__6(
                self as *const crate::QByteArray,
                a2.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ptr<::std::os::raw::c_char>> for crate::QByteArray {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(const QByteArray& a1, const char* a2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, a2: &::cpp_utils::Ptr<::std::os::raw::c_char>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__7(
                self as *const crate::QByteArray,
                a2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QByteArray>> for crate::QByteArray {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QByteArray& a1, const QByteArray& a2)```</span>.
    fn lt(&self, a2: &::cpp_utils::Ref<crate::QByteArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_2(
                self as *const crate::QByteArray,
                a2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ptr<::std::os::raw::c_char>> for crate::QByteArray {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QByteArray& a1, const char* a2)```</span>.
    fn lt(&self, a2: &::cpp_utils::Ptr<::std::os::raw::c_char>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_3(
                self as *const crate::QByteArray,
                a2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QByteArray>> for crate::QByteArray {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QByteArray& a1, const QByteArray& a2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, a2: &::cpp_utils::Ref<crate::QByteArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__12(
                self as *const crate::QByteArray,
                a2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ptr<::std::os::raw::c_char>> for crate::QByteArray {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QByteArray& a1, const char* a2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, a2: &::cpp_utils::Ptr<::std::os::raw::c_char>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__13(
                self as *const crate::QByteArray,
                a2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QByteArray>> for crate::QByteArray {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QByteArray& a1, const QByteArray& a2)```</span>.
    fn gt(&self, a2: &::cpp_utils::Ref<crate::QByteArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_5(
                self as *const crate::QByteArray,
                a2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ptr<::std::os::raw::c_char>> for crate::QByteArray {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QByteArray& a1, const char* a2)```</span>.
    fn gt(&self, a2: &::cpp_utils::Ptr<::std::os::raw::c_char>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_6(
                self as *const crate::QByteArray,
                a2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QByteArray>> for crate::QByteArray {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QByteArray& a1, const QByteArray& a2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, a2: &::cpp_utils::Ref<crate::QByteArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__15(
                self as *const crate::QByteArray,
                a2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ptr<::std::os::raw::c_char>> for crate::QByteArray {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QByteArray& a1, const char* a2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, a2: &::cpp_utils::Ptr<::std::os::raw::c_char>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__16(
                self as *const crate::QByteArray,
                a2.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QByteArray>> for &crate::QByteArray {
    type Output = ::cpp_utils::CppBox<crate::QByteArray>;
    /// <p>Returns a byte array that is the result of concatenating byte array <i>a1</i> and byte array <i>a2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray operator+(const QByteArray& a1, const QByteArray& a2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a byte array that is the result of concatenating byte array <i>a1</i> and byte array <i>a2</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#operator-2b-eq">QByteArray::operator+=</a>().</p></div>
    fn add(
        self,
        a2: ::cpp_utils::Ref<crate::QByteArray>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_8(
                self as *const crate::QByteArray,
                a2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ptr<::std::os::raw::c_char>> for &crate::QByteArray {
    type Output = ::cpp_utils::CppBox<crate::QByteArray>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray operator+(const QByteArray& a1, const char* a2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#operator-2b-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a byte array that is the result of concatenating byte array <i>a1</i> and string <i>a2</i>.</p></div>
    fn add(
        self,
        a2: ::cpp_utils::Ptr<::std::os::raw::c_char>,
    ) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_9(
                self as *const crate::QByteArray,
                a2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::std::os::raw::c_char> for &crate::QByteArray {
    type Output = ::cpp_utils::CppBox<crate::QByteArray>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QByteArray operator+(const QByteArray& a1, char a2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearray.html#operator-2b-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a byte array that is the result of concatenating byte array <i>a1</i> and character <i>a2</i>.</p></div>
    fn add(self, a2: ::std::os::raw::c_char) -> ::cpp_utils::CppBox<crate::QByteArray> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_10(self as *const crate::QByteArray, a2)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QByteArray>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, const QByteArray& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        arg2: ::cpp_utils::Ref<crate::QByteArray>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__18(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QByteArray>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QByteArray& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QByteArray>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__19(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QString>> for crate::QLatin1String {
    /// <p>Returns <code>true</code> if this string is equal to string <i>other</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::operator==(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this string is equal to string <i>other</i>; otherwise returns <code>false</code>.</p>
    /// <p>The comparison is based exclusively on the numeric Unicode values of the characters and is very fast, but is not what a human would expect. Consider sorting user-interface strings with <a href="http://doc.qt.io/qt-5/qstring.html#localeAwareCompare">QString::localeAwareCompare</a>().</p></div>
    fn eq(&self, s: &::cpp_utils::Ref<crate::QString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QLatin1String_operator__1(
                self as *const crate::QLatin1String,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QString>> for crate::QLatin1String {
    /// <p>Returns <code>true</code> if this string is lexically greater than string <i>other</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::operator>(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this string is lexically greater than string <i>other</i>; otherwise returns <code>false</code>.</p>
    /// <p>The comparison is based exclusively on the numeric Unicode values of the characters and is very fast, but is not what a human would expect. Consider sorting user-interface strings with <a href="http://doc.qt.io/qt-5/qstring.html#localeAwareCompare">QString::localeAwareCompare</a>().</p></div>
    fn gt(&self, s: &::cpp_utils::Ref<crate::QString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QLatin1String_operator_(
                self as *const crate::QLatin1String,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QString>> for crate::QLatin1String {
    /// <p>Returns <code>true</code> if this string is lexically less than the <i>other</i> string; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::operator<(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this string is lexically less than the <i>other</i> string; otherwise returns <code>false</code>.</p>
    /// <p>The comparison is based exclusively on the numeric Unicode values of the characters and is very fast, but is not what a human would expect. Consider sorting user-interface strings using the <a href="http://doc.qt.io/qt-5/qstring.html#localeAwareCompare">QString::localeAwareCompare</a>() function.</p></div>
    fn lt(&self, s: &::cpp_utils::Ref<crate::QString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QLatin1String_operator_1(
                self as *const crate::QLatin1String,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QString>> for crate::QLatin1String {
    /// <p>Returns <code>true</code> if this string is lexically greater than or equal to string <i>other</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::operator>=(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this string is lexically greater than or equal to string <i>other</i>; otherwise returns <code>false</code>.</p>
    /// <p>The comparison is based exclusively on the numeric Unicode values of the characters and is very fast, but is not what a human would expect. Consider sorting user-interface strings with <a href="http://doc.qt.io/qt-5/qstring.html#localeAwareCompare">QString::localeAwareCompare</a>().</p></div>
    fn ge(&self, s: &::cpp_utils::Ref<crate::QString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QLatin1String_operator__3(
                self as *const crate::QLatin1String,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QString>> for crate::QLatin1String {
    /// <p>Returns <code>true</code> if this string is lexically less than or equal to string <i>other</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::operator<=(const QString& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this string is lexically less than or equal to string <i>other</i>; otherwise returns <code>false</code>.</p>
    /// <p>The comparison is based exclusively on the numeric Unicode values of the characters and is very fast, but is not what a human would expect. Consider sorting user-interface strings with <a href="http://doc.qt.io/qt-5/qstring.html#localeAwareCompare">QString::localeAwareCompare</a>().</p></div>
    fn le(&self, s: &::cpp_utils::Ref<crate::QString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QLatin1String_operator__4(
                self as *const crate::QLatin1String,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ptr<::std::os::raw::c_char>> for crate::QLatin1String {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::operator==(const char* s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>The <i>other</i> const char pointer is converted to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">QString::fromUtf8</a>() function.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p>
    /// <p>This function was introduced in  Qt 4.3.</p></div>
    fn eq(&self, s: &::cpp_utils::Ptr<::std::os::raw::c_char>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QLatin1String_operator__5(
                self as *const crate::QLatin1String,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ptr<::std::os::raw::c_char>> for crate::QLatin1String {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::operator<(const char* s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#operator-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>The <i>other</i> const char pointer is converted to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">QString::fromUtf8</a>() function.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p>
    /// <p>This function was introduced in  Qt 4.3.</p></div>
    fn lt(&self, s: &::cpp_utils::Ptr<::std::os::raw::c_char>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QLatin1String_operator_2(
                self as *const crate::QLatin1String,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ptr<::std::os::raw::c_char>> for crate::QLatin1String {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::operator>(const char* s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#operator-gt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>The <i>other</i> const char pointer is converted to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">QString::fromUtf8</a>() function.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p>
    /// <p>This function was introduced in  Qt 4.3.</p></div>
    fn gt(&self, s: &::cpp_utils::Ptr<::std::os::raw::c_char>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QLatin1String_operator_3(
                self as *const crate::QLatin1String,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ptr<::std::os::raw::c_char>> for crate::QLatin1String {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::operator<=(const char* s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#operator-lt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>The <i>other</i> const char pointer is converted to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">QString::fromUtf8</a>() function.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p>
    /// <p>This function was introduced in  Qt 4.3.</p></div>
    fn le(&self, s: &::cpp_utils::Ptr<::std::os::raw::c_char>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QLatin1String_operator__7(
                self as *const crate::QLatin1String,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ptr<::std::os::raw::c_char>> for crate::QLatin1String {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::operator>=(const char* s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#operator-gt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>The <i>other</i> const char pointer is converted to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">QString::fromUtf8</a>() function.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p>
    /// <p>This function was introduced in  Qt 4.3.</p></div>
    fn ge(&self, s: &::cpp_utils::Ptr<::std::os::raw::c_char>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QLatin1String_operator__8(
                self as *const crate::QLatin1String,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QByteArray>> for crate::QLatin1String {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::operator==(const QByteArray& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#operator-eq-eq-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>The <i>other</i> byte array is converted to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">QString::fromUtf8</a>() function.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    fn eq(&self, s: &::cpp_utils::Ref<crate::QByteArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QLatin1String_operator__9(
                self as *const crate::QLatin1String,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QByteArray>> for crate::QLatin1String {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::operator<(const QByteArray& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#operator-lt-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>The <i>other</i> const char pointer is converted to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">QString::fromUtf8</a>() function.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    fn lt(&self, s: &::cpp_utils::Ref<crate::QByteArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QLatin1String_operator_4(
                self as *const crate::QLatin1String,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QByteArray>> for crate::QLatin1String {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::operator>(const QByteArray& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#operator-gt-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>The <i>other</i> const char pointer is converted to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">QString::fromUtf8</a>() function.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    fn gt(&self, s: &::cpp_utils::Ref<crate::QByteArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QLatin1String_operator_5(
                self as *const crate::QLatin1String,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QByteArray>> for crate::QLatin1String {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::operator<=(const QByteArray& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#operator-lt-eq-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>The <i>other</i> array is converted to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">QString::fromUtf8</a>() function.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    fn le(&self, s: &::cpp_utils::Ref<crate::QByteArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QLatin1String_operator__11(
                self as *const crate::QLatin1String,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QByteArray>> for crate::QLatin1String {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLatin1String::operator>=(const QByteArray& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#operator-gt-eq-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>The <i>other</i> array is converted to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">QString::fromUtf8</a>() function.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p>
    /// <p>This function was introduced in  Qt 5.0.</p></div>
    fn ge(&self, s: &::cpp_utils::Ref<crate::QByteArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QLatin1String_operator__12(
                self as *const crate::QLatin1String,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QString {
    /// <p>Destroys the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QString::~QString()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#dtor.QString">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the string.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QString_dQString(self as *mut crate::QString)
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QChar>> for crate::QString {
    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-2b-eq">operator+=</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::operator+=(QChar c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-2b-eq-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-2b-eq">operator+=</a>().</p>
    /// <p>Appends the character <i>ch</i> to the string.</p></div>
    fn add_assign(&mut self, c: ::cpp_utils::Ref<crate::QChar>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QString_operator__4(
                self as *mut crate::QString,
                c.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<crate::q_char::SpecialCharacter> for crate::QString {
    /// Calls C++ function: <span style='color: green;'>```QString& QString::operator+=(QChar::SpecialCharacter c)```</span>.
    fn add_assign(&mut self, c: crate::q_char::SpecialCharacter) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QString_operator__5(self as *mut crate::QString, c)
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QString>> for crate::QString {
    /// <p>Appends the string <i>other</i> onto the end of this string and returns a reference to this string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::operator+=(const QString& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the string <i>other</i> onto the end of this string and returns a reference to this string.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> x <span class="operator">=</span> <span class="string">"free"</span>;
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html#QString">QString</a></span> y <span class="operator">=</span> <span class="string">"dom"</span>;
    ///   x <span class="operator">+</span><span class="operator">=</span> y;
    ///   <span class="comment">// x == "freedom"</span>
    ///
    /// </pre>
    /// <p>This operation is typically very fast (<a href="http://doc.qt.io/qt-5/containers.html#constant-time">constant time</a>), because <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> preallocates extra space at the end of the string data so it can grow without reallocating the entire string each time.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#prepend">prepend</a>().</p></div>
    fn add_assign(&mut self, s: ::cpp_utils::Ref<crate::QString>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QString_operator__6(
                self as *mut crate::QString,
                s.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QStringRef>> for crate::QString {
    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-2b-eq">operator+=</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::operator+=(const QStringRef& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-2b-eq-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-2b-eq">operator+=</a>().</p>
    /// <p>Appends the string section referenced by <i>str</i> to this string.</p></div>
    fn add_assign(&mut self, s: ::cpp_utils::Ref<crate::QStringRef>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QString_operator__7(
                self as *mut crate::QString,
                s.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QLatin1String>> for crate::QString {
    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-2b-eq">operator+=</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::operator+=(QLatin1String s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-2b-eq-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-2b-eq">operator+=</a>().</p>
    /// <p>Appends the Latin-1 string <i>str</i> to this string.</p></div>
    fn add_assign(&mut self, s: ::cpp_utils::Ref<crate::QLatin1String>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QString_operator__8(
                self as *mut crate::QString,
                s.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QLatin1String>> for crate::QString {
    /// <p>This function overloads operator==().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::operator==(QLatin1String s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads operator==().</p>
    /// <p>Returns <code>true</code> if this string is equal to <i>other</i>; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, s: &::cpp_utils::Ref<crate::QLatin1String>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QString_operator__9(
                self as *const crate::QString,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QLatin1String>> for crate::QString {
    /// <p>This function overloads operator&lt;().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::operator<(QLatin1String s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads operator&lt;().</p>
    /// <p>Returns <code>true</code> if this string is lexically less than the parameter string called <i>other</i>; otherwise returns <code>false</code>.</p></div>
    fn lt(&self, s: &::cpp_utils::Ref<crate::QLatin1String>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QString_operator_4(
                self as *const crate::QString,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QLatin1String>> for crate::QString {
    /// <p>This function overloads operator&gt;().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::operator>(QLatin1String s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads operator&gt;().</p>
    /// <p>Returns <code>true</code> if this string is lexically greater than the parameter string <i>other</i>; otherwise returns <code>false</code>.</p></div>
    fn gt(&self, s: &::cpp_utils::Ref<crate::QLatin1String>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QString_operator_5(
                self as *const crate::QString,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QLatin1String>> for crate::QString {
    /// <p>Returns <code>true</code> if this string is lexically less than or equal to parameter string <i>other</i>. Otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::operator<=(QLatin1String s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this string is lexically less than or equal to parameter string <i>other</i>. Otherwise returns <code>false</code>.</p>
    /// <p>This function overloads operator&lt;=().</p></div>
    fn le(&self, s: &::cpp_utils::Ref<crate::QLatin1String>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QString_operator__11(
                self as *const crate::QString,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QLatin1String>> for crate::QString {
    /// <p>Returns <code>true</code> if this string is lexically greater than or equal to parameter string <i>other</i>. Otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::operator>=(QLatin1String s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this string is lexically greater than or equal to parameter string <i>other</i>. Otherwise returns <code>false</code>.</p>
    /// <p>This function overloads operator&gt;=().</p></div>
    fn ge(&self, s: &::cpp_utils::Ref<crate::QLatin1String>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QString_operator__12(
                self as *const crate::QString,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ptr<::std::os::raw::c_char>> for crate::QString {
    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-2b-eq">operator+=</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::operator+=(const char* s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-2b-eq-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-2b-eq">operator+=</a>().</p>
    /// <p>Appends the string <i>str</i> to this string. The const char pointer is converted to Unicode using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>() function.</p>
    /// <p>You can disable this function by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p></div>
    fn add_assign(&mut self, s: ::cpp_utils::Ptr<::std::os::raw::c_char>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QString_operator__13(
                self as *mut crate::QString,
                s.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QByteArray>> for crate::QString {
    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-2b-eq">operator+=</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::operator+=(const QByteArray& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-2b-eq-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-2b-eq">operator+=</a>().</p>
    /// <p>Appends the byte array <i>ba</i> to this string. The byte array is converted to Unicode using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>() function. If any NUL characters ('\0') are embedded in the <i>ba</i> byte array, they will be included in the transformation.</p>
    /// <p>You can disable this function by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p></div>
    fn add_assign(&mut self, s: ::cpp_utils::Ref<crate::QByteArray>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QString_operator__14(
                self as *mut crate::QString,
                s.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::std::os::raw::c_char> for crate::QString {
    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-2b-eq">operator+=</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QString& QString::operator+=(char c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-2b-eq-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-2b-eq">operator+=</a>().</p>
    /// <p>Appends the character <i>ch</i> to this string. Note that the character is converted to Unicode using the <a href="http://doc.qt.io/qt-5/qstring.html#fromLatin1">fromLatin1</a>() function, unlike other 8-bit functions that operate on UTF-8 data.</p>
    /// <p>You can disable this function by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p></div>
    fn add_assign(&mut self, c: ::std::os::raw::c_char) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QString_operator__15(self as *mut crate::QString, c)
        };
        let _ = ffi_result;
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ptr<::std::os::raw::c_char>> for crate::QString {
    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-eq-eq">operator==</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::operator==(const char* s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-eq-eq">operator==</a>().</p>
    /// <p>The <i>other</i> const char pointer is converted to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>() function.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#QT_NO_CAST_FROM_ASCII">QT_NO_CAST_FROM_ASCII</a>.</p></div>
    fn eq(&self, s: &::cpp_utils::Ptr<::std::os::raw::c_char>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QString_operator__16(
                self as *const crate::QString,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ptr<::std::os::raw::c_char>> for crate::QString {
    /// <p>Returns <code>true</code> if this string is lexically less than string <i>other</i>. Otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::operator<(const char* s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this string is lexically less than string <i>other</i>. Otherwise returns <code>false</code>.</p>
    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-lt">operator&lt;</a>().</p>
    /// <p>The <i>other</i> const char pointer is converted to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>() function.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#QT_NO_CAST_FROM_ASCII">QT_NO_CAST_FROM_ASCII</a>.</p></div>
    fn lt(&self, s: &::cpp_utils::Ptr<::std::os::raw::c_char>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QString_operator_9(
                self as *const crate::QString,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ptr<::std::os::raw::c_char>> for crate::QString {
    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-lt-eq">operator&lt;=</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::operator<=(const char* s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-lt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-lt-eq">operator&lt;=</a>().</p>
    /// <p>The <i>other</i> const char pointer is converted to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>() function.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#QT_NO_CAST_FROM_ASCII">QT_NO_CAST_FROM_ASCII</a>.</p></div>
    fn le(&self, s: &::cpp_utils::Ptr<::std::os::raw::c_char>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QString_operator__18(
                self as *const crate::QString,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ptr<::std::os::raw::c_char>> for crate::QString {
    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-gt">operator&gt;</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::operator>(const char* s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-gt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-gt">operator&gt;</a>().</p>
    /// <p>The <i>other</i> const char pointer is converted to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>() function.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#QT_NO_CAST_FROM_ASCII">QT_NO_CAST_FROM_ASCII</a>.</p></div>
    fn gt(&self, s: &::cpp_utils::Ptr<::std::os::raw::c_char>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QString_operator_10(
                self as *const crate::QString,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ptr<::std::os::raw::c_char>> for crate::QString {
    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-gt-eq">operator&gt;=</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::operator>=(const char* s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-gt-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-gt-eq">operator&gt;=</a>().</p>
    /// <p>The <i>other</i> const char pointer is converted to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>() function.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#QT_NO_CAST_FROM_ASCII">QT_NO_CAST_FROM_ASCII</a>.</p></div>
    fn ge(&self, s: &::cpp_utils::Ptr<::std::os::raw::c_char>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QString_operator__19(
                self as *const crate::QString,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QByteArray>> for crate::QString {
    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-eq-eq">operator==</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::operator==(const QByteArray& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-eq-eq-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-eq-eq">operator==</a>().</p>
    /// <p>The <i>other</i> byte array is converted to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>() function. This function stops conversion at the first NUL character found, or the end of the byte array.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p>
    /// <p>Returns <code>true</code> if this string is lexically equal to the parameter string <i>other</i>. Otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#QT_NO_CAST_FROM_ASCII">QT_NO_CAST_FROM_ASCII</a>.</p></div>
    fn eq(&self, s: &::cpp_utils::Ref<crate::QByteArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QString_operator__20(
                self as *const crate::QString,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QByteArray>> for crate::QString {
    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-lt">operator&lt;</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::operator<(const QByteArray& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-lt-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-lt">operator&lt;</a>().</p>
    /// <p>The <i>other</i> byte array is converted to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>() function. If any NUL characters ('\0') are embedded in the byte array, they will be included in the transformation.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#QT_NO_CAST_FROM_ASCII">QT_NO_CAST_FROM_ASCII</a>.</p></div>
    fn lt(&self, s: &::cpp_utils::Ref<crate::QByteArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QString_operator_11(
                self as *const crate::QString,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QByteArray>> for crate::QString {
    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-gt">operator&gt;</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::operator>(const QByteArray& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-gt-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-gt">operator&gt;</a>().</p>
    /// <p>The <i>other</i> byte array is converted to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>() function. If any NUL characters ('\0') are embedded in the byte array, they will be included in the transformation.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#QT_NO_CAST_FROM_ASCII">QT_NO_CAST_FROM_ASCII</a>.</p></div>
    fn gt(&self, s: &::cpp_utils::Ref<crate::QByteArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QString_operator_12(
                self as *const crate::QString,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QByteArray>> for crate::QString {
    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-lt-eq">operator&lt;=</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::operator<=(const QByteArray& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-lt-eq-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-lt-eq">operator&lt;=</a>().</p>
    /// <p>The <i>other</i> byte array is converted to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>() function. If any NUL characters ('\0') are embedded in the byte array, they will be included in the transformation.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#QT_NO_CAST_FROM_ASCII">QT_NO_CAST_FROM_ASCII</a>.</p></div>
    fn le(&self, s: &::cpp_utils::Ref<crate::QByteArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QString_operator__22(
                self as *const crate::QString,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QByteArray>> for crate::QString {
    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-gt-eq">operator&gt;=</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QString::operator>=(const QByteArray& s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#operator-gt-eq-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#operator-gt-eq">operator&gt;=</a>().</p>
    /// <p>The <i>other</i> byte array is converted to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> using the <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">fromUtf8</a>() function. If any NUL characters ('\0') are embedded in the byte array, they will be included in the transformation.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#QT_NO_CAST_FROM_ASCII">QT_NO_CAST_FROM_ASCII</a>.</p></div>
    fn ge(&self, s: &::cpp_utils::Ref<crate::QByteArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QString_operator__23(
                self as *const crate::QString,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QString {
    type Output = ::cpp_utils::MutPtr<crate::QChar>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first character in the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar* QString::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first character in the string.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::MutPtr<crate::QChar> {
        let ffi_result =
            unsafe { crate::__ffi::ctr_qt_core_ffi_QString_begin(self as *mut crate::QString) };
        unsafe { ::cpp_utils::MutPtr::from_raw(ffi_result) }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QString {
    type Output = ::cpp_utils::Ptr<crate::QChar>;
    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#begin">begin</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QChar* QString::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#begin">begin</a>().</p></div>
    fn begin(self) -> ::cpp_utils::Ptr<crate::QChar> {
        let ffi_result =
            unsafe { crate::__ffi::ctr_qt_core_ffi_QString_begin1(self as *const crate::QString) };
        unsafe { ::cpp_utils::Ptr::from_raw(ffi_result) }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QString {
    type Output = ::cpp_utils::MutPtr<crate::QChar>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary character after the last character in the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QChar* QString::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary character after the last character in the string.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qstring.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::MutPtr<crate::QChar> {
        let ffi_result =
            unsafe { crate::__ffi::ctr_qt_core_ffi_QString_end(self as *mut crate::QString) };
        unsafe { ::cpp_utils::MutPtr::from_raw(ffi_result) }
    }
}

impl ::cpp_utils::ops::End for &crate::QString {
    type Output = ::cpp_utils::Ptr<crate::QChar>;
    /// <p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#end">end</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QChar* QString::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstring.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads <a href="http://doc.qt.io/qt-5/qstring.html#end">end</a>().</p></div>
    fn end(self) -> ::cpp_utils::Ptr<crate::QChar> {
        let ffi_result =
            unsafe { crate::__ffi::ctr_qt_core_ffi_QString_end1(self as *const crate::QString) };
        unsafe { ::cpp_utils::Ptr::from_raw(ffi_result) }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QLatin1String>> for crate::QLatin1String {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(QLatin1String s1, QLatin1String s2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, s2: &::cpp_utils::Ref<crate::QLatin1String>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__20(
                self as *const crate::QLatin1String,
                s2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QLatin1String>> for crate::QLatin1String {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(QLatin1String s1, QLatin1String s2)```</span>.
    fn lt(&self, s2: &::cpp_utils::Ref<crate::QLatin1String>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_13(
                self as *const crate::QLatin1String,
                s2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QLatin1String>> for crate::QLatin1String {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(QLatin1String s1, QLatin1String s2)```</span>.
    fn gt(&self, s2: &::cpp_utils::Ref<crate::QLatin1String>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_14(
                self as *const crate::QLatin1String,
                s2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QLatin1String>> for crate::QLatin1String {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(QLatin1String s1, QLatin1String s2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, s2: &::cpp_utils::Ref<crate::QLatin1String>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__22(
                self as *const crate::QLatin1String,
                s2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QLatin1String>> for crate::QLatin1String {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(QLatin1String s1, QLatin1String s2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, s2: &::cpp_utils::Ref<crate::QLatin1String>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__23(
                self as *const crate::QLatin1String,
                s2.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QString>> for &crate::QString {
    type Output = ::cpp_utils::CppBox<crate::QString>;
    /// Calls C++ function: <span style='color: green;'>```QString operator+(const QString& s1, const QString& s2)```</span>.
    fn add(self, s2: ::cpp_utils::Ref<crate::QString>) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_19(
                self as *const crate::QString,
                s2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QChar>> for &crate::QString {
    type Output = ::cpp_utils::CppBox<crate::QString>;
    /// Calls C++ function: <span style='color: green;'>```QString operator+(const QString& s1, QChar s2)```</span>.
    fn add(self, s2: ::cpp_utils::Ref<crate::QChar>) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_20(
                self as *const crate::QString,
                s2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QString>> for &crate::QChar {
    type Output = ::cpp_utils::CppBox<crate::QString>;
    /// Calls C++ function: <span style='color: green;'>```QString operator+(QChar s1, const QString& s2)```</span>.
    fn add(self, s2: ::cpp_utils::Ref<crate::QString>) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_21(self as *const crate::QChar, s2.as_raw_ptr())
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ptr<::std::os::raw::c_char>> for &crate::QString {
    type Output = ::cpp_utils::CppBox<crate::QString>;
    /// Calls C++ function: <span style='color: green;'>```QString operator+(const QString& s1, const char* s2)```</span>.
    fn add(
        self,
        s2: ::cpp_utils::Ptr<::std::os::raw::c_char>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_22(
                self as *const crate::QString,
                s2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::std::os::raw::c_char> for &crate::QString {
    type Output = ::cpp_utils::CppBox<crate::QString>;
    /// Calls C++ function: <span style='color: green;'>```QString operator+(const QString& s, char c)```</span>.
    fn add(self, c: ::std::os::raw::c_char) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result =
            unsafe { crate::__ffi::ctr_qt_core_ffi_operator_25(self as *const crate::QString, c) };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QString>> for &crate::QByteArray {
    type Output = ::cpp_utils::CppBox<crate::QString>;
    /// Calls C++ function: <span style='color: green;'>```QString operator+(const QByteArray& ba, const QString& s)```</span>.
    fn add(self, s: ::cpp_utils::Ref<crate::QString>) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_26(
                self as *const crate::QByteArray,
                s.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QByteArray>> for &crate::QString {
    type Output = ::cpp_utils::CppBox<crate::QString>;
    /// Calls C++ function: <span style='color: green;'>```QString operator+(const QString& s, const QByteArray& ba)```</span>.
    fn add(self, ba: ::cpp_utils::Ref<crate::QByteArray>) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_27(
                self as *const crate::QString,
                ba.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QString>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, const QString& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        arg2: ::cpp_utils::Ref<crate::QString>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__32(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QString>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QString& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QString>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__33(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QStringRef {
    /// <p>Destroys the string reference.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QStringRef::~QStringRef()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#dtor.QStringRef">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the string reference.</p>
    /// <p>Since this class is only used to refer to string data, and does not take ownership of it, no memory is freed when instances are destroyed.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QStringRef_dQStringRef(self as *mut crate::QStringRef)
    }
}

impl ::cpp_utils::ops::Begin for &crate::QStringRef {
    type Output = ::cpp_utils::Ptr<crate::QChar>;
    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first character in the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QChar* QStringRef::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first character in the string.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#cbegin">cbegin</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#constBegin">constBegin</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#end">end</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#constEnd">constEnd</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#rbegin">rbegin</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#rend">rend</a>().</p></div>
    fn begin(self) -> ::cpp_utils::Ptr<crate::QChar> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QStringRef_begin(self as *const crate::QStringRef)
        };
        unsafe { ::cpp_utils::Ptr::from_raw(ffi_result) }
    }
}

impl ::cpp_utils::ops::End for &crate::QStringRef {
    type Output = ::cpp_utils::Ptr<crate::QChar>;
    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary character after the last character in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QChar* QStringRef::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary character after the last character in the list.</p>
    /// <p>This function was introduced in  Qt 5.4.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringref.html#cbegin">cbegin</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#constBegin">constBegin</a>(), end(), <a href="http://doc.qt.io/qt-5/qstringref.html#constEnd">constEnd</a>(), <a href="http://doc.qt.io/qt-5/qstringref.html#rbegin">rbegin</a>(), and <a href="http://doc.qt.io/qt-5/qstringref.html#rend">rend</a>().</p></div>
    fn end(self) -> ::cpp_utils::Ptr<crate::QChar> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QStringRef_end(self as *const crate::QStringRef)
        };
        unsafe { ::cpp_utils::Ptr::from_raw(ffi_result) }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ptr<::std::os::raw::c_char>> for crate::QStringRef {
    /// <p>This function overloads operator==().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::operator==(const char* s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads operator==().</p>
    /// <p>The <i>s</i> byte array is converted to a <a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a> using the fromUtf8() function. This function stops conversion at the first NUL character found, or the end of the byte array.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p>
    /// <p>Returns <code>true</code> if this string is lexically equal to the parameter string <i>s</i>. Otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#QT_NO_CAST_FROM_ASCII">QT_NO_CAST_FROM_ASCII</a>.</p></div>
    fn eq(&self, s: &::cpp_utils::Ptr<::std::os::raw::c_char>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QStringRef_operator__1(
                self as *const crate::QStringRef,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ptr<::std::os::raw::c_char>> for crate::QStringRef {
    /// <p>This function overloads operator&lt;().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::operator<(const char* s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads operator&lt;().</p>
    /// <p>The <i>s</i> const char pointer is converted to a <a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a> using the fromUtf8() function.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p>
    /// <p>Returns <code>true</code> if this string is lexically smaller than the parameter string <i>s</i>. Otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#QT_NO_CAST_FROM_ASCII">QT_NO_CAST_FROM_ASCII</a>.</p></div>
    fn lt(&self, s: &::cpp_utils::Ptr<::std::os::raw::c_char>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QStringRef_operator_3(
                self as *const crate::QStringRef,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ptr<::std::os::raw::c_char>> for crate::QStringRef {
    /// <p>This function overloads operator&lt;=().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::operator<=(const char* s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads operator&lt;=().</p>
    /// <p>The <i>s</i> const char pointer is converted to a <a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a> using the fromUtf8() function.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p>
    /// <p>Returns <code>true</code> if this string is lexically smaller than or equal to the parameter string <i>s</i>. Otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#QT_NO_CAST_FROM_ASCII">QT_NO_CAST_FROM_ASCII</a>.</p></div>
    fn le(&self, s: &::cpp_utils::Ptr<::std::os::raw::c_char>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QStringRef_operator__3(
                self as *const crate::QStringRef,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ptr<::std::os::raw::c_char>> for crate::QStringRef {
    /// <p>This function overloads operator&gt;().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::operator>(const char* s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads operator&gt;().</p>
    /// <p>The <i>s</i> const char pointer is converted to a <a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a> using the fromUtf8() function.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p>
    /// <p>Returns <code>true</code> if this string is lexically greater than the parameter string <i>s</i>. Otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#QT_NO_CAST_FROM_ASCII">QT_NO_CAST_FROM_ASCII</a>.</p></div>
    fn gt(&self, s: &::cpp_utils::Ptr<::std::os::raw::c_char>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QStringRef_operator_4(
                self as *const crate::QStringRef,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ptr<::std::os::raw::c_char>> for crate::QStringRef {
    /// <p>This function overloads operator&gt;=().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QStringRef::operator>=(const char* s) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringref.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function overloads operator&gt;=().</p>
    /// <p>The <i>s</i> const char pointer is converted to a <a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a> using the fromUtf8() function.</p>
    /// <p>You can disable this operator by defining <code>QT_NO_CAST_FROM_ASCII</code> when you compile your applications. This can be useful if you want to ensure that all user-visible strings go through <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), for example.</p>
    /// <p>Returns <code>true</code> if this string is lexically greater than or equal to the parameter string <i>s</i>. Otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#QT_NO_CAST_FROM_ASCII">QT_NO_CAST_FROM_ASCII</a>.</p></div>
    fn ge(&self, s: &::cpp_utils::Ptr<::std::os::raw::c_char>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QStringRef_operator__4(
                self as *const crate::QStringRef,
                s.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QStringRef>> for crate::QStringRef {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(const QStringRef& s1, const QStringRef& s2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, s2: &::cpp_utils::Ref<crate::QStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__34(
                self as *const crate::QStringRef,
                s2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QStringRef>> for crate::QStringRef {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QStringRef& s1, const QStringRef& s2)```</span>.
    fn lt(&self, s2: &::cpp_utils::Ref<crate::QStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_28(
                self as *const crate::QStringRef,
                s2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QStringRef>> for crate::QStringRef {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QStringRef& s1, const QStringRef& s2)```</span>.
    fn gt(&self, s2: &::cpp_utils::Ref<crate::QStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_29(
                self as *const crate::QStringRef,
                s2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QStringRef>> for crate::QStringRef {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QStringRef& s1, const QStringRef& s2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, s2: &::cpp_utils::Ref<crate::QStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__36(
                self as *const crate::QStringRef,
                s2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QStringRef>> for crate::QStringRef {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QStringRef& s1, const QStringRef& s2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, s2: &::cpp_utils::Ref<crate::QStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__37(
                self as *const crate::QStringRef,
                s2.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QStringRef>> for crate::QString {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(const QString& lhs, const QStringRef& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, rhs: &::cpp_utils::Ref<crate::QStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__38(
                self as *const crate::QString,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QStringRef>> for crate::QString {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QString& lhs, const QStringRef& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_30(
                self as *const crate::QString,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QStringRef>> for crate::QString {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QString& lhs, const QStringRef& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_31(
                self as *const crate::QString,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QStringRef>> for crate::QString {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QString& lhs, const QStringRef& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__40(
                self as *const crate::QString,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QStringRef>> for crate::QString {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QString& lhs, const QStringRef& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__41(
                self as *const crate::QString,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QString>> for crate::QStringRef {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(const QStringRef& lhs, const QString& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, rhs: &::cpp_utils::Ref<crate::QString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__42(
                self as *const crate::QStringRef,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QString>> for crate::QStringRef {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QStringRef& lhs, const QString& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_32(
                self as *const crate::QStringRef,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QString>> for crate::QStringRef {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QStringRef& lhs, const QString& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_33(
                self as *const crate::QStringRef,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QString>> for crate::QStringRef {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QStringRef& lhs, const QString& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__44(
                self as *const crate::QStringRef,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QString>> for crate::QStringRef {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QStringRef& lhs, const QString& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__45(
                self as *const crate::QStringRef,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QStringRef>> for crate::QLatin1String {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(QLatin1String lhs, const QStringRef& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, rhs: &::cpp_utils::Ref<crate::QStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__46(
                self as *const crate::QLatin1String,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QStringRef>> for crate::QLatin1String {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(QLatin1String lhs, const QStringRef& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_34(
                self as *const crate::QLatin1String,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QStringRef>> for crate::QLatin1String {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(QLatin1String lhs, const QStringRef& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_35(
                self as *const crate::QLatin1String,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QStringRef>> for crate::QLatin1String {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(QLatin1String lhs, const QStringRef& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__48(
                self as *const crate::QLatin1String,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QStringRef>> for crate::QLatin1String {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(QLatin1String lhs, const QStringRef& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__49(
                self as *const crate::QLatin1String,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QLatin1String>> for crate::QStringRef {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(const QStringRef& lhs, QLatin1String rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, rhs: &::cpp_utils::Ref<crate::QLatin1String>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__50(
                self as *const crate::QStringRef,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QLatin1String>> for crate::QStringRef {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QStringRef& lhs, QLatin1String rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QLatin1String>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_36(
                self as *const crate::QStringRef,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QLatin1String>> for crate::QStringRef {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QStringRef& lhs, QLatin1String rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QLatin1String>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_37(
                self as *const crate::QStringRef,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QLatin1String>> for crate::QStringRef {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QStringRef& lhs, QLatin1String rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QLatin1String>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__52(
                self as *const crate::QStringRef,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QLatin1String>> for crate::QStringRef {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QStringRef& lhs, QLatin1String rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QLatin1String>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__53(
                self as *const crate::QStringRef,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QString>> for crate::QChar {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(QChar lhs, const QString& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, rhs: &::cpp_utils::Ref<crate::QString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__54(
                self as *const crate::QChar,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QString>> for crate::QChar {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(QChar lhs, const QString& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_38(self as *const crate::QChar, rhs.as_raw_ptr())
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QString>> for crate::QChar {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(QChar lhs, const QString& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_39(self as *const crate::QChar, rhs.as_raw_ptr())
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QString>> for crate::QChar {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(QChar lhs, const QString& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__56(
                self as *const crate::QChar,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QString>> for crate::QChar {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(QChar lhs, const QString& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__57(
                self as *const crate::QChar,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QChar>> for crate::QString {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(const QString& lhs, QChar rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, rhs: &::cpp_utils::Ref<crate::QChar>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__58(
                self as *const crate::QString,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QChar>> for crate::QString {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QString& lhs, QChar rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QChar>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_40(
                self as *const crate::QString,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QChar>> for crate::QString {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QString& lhs, QChar rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QChar>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_41(
                self as *const crate::QString,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QChar>> for crate::QString {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QString& lhs, QChar rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QChar>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__60(
                self as *const crate::QString,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QChar>> for crate::QString {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QString& lhs, QChar rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QChar>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__61(
                self as *const crate::QString,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QStringRef>> for crate::QChar {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(QChar lhs, const QStringRef& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, rhs: &::cpp_utils::Ref<crate::QStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__62(
                self as *const crate::QChar,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QStringRef>> for crate::QChar {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(QChar lhs, const QStringRef& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_42(self as *const crate::QChar, rhs.as_raw_ptr())
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QStringRef>> for crate::QChar {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(QChar lhs, const QStringRef& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_43(self as *const crate::QChar, rhs.as_raw_ptr())
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QStringRef>> for crate::QChar {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(QChar lhs, const QStringRef& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__64(
                self as *const crate::QChar,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QStringRef>> for crate::QChar {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(QChar lhs, const QStringRef& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__65(
                self as *const crate::QChar,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QChar>> for crate::QStringRef {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(const QStringRef& lhs, QChar rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, rhs: &::cpp_utils::Ref<crate::QChar>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__66(
                self as *const crate::QStringRef,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QChar>> for crate::QStringRef {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QStringRef& lhs, QChar rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QChar>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_44(
                self as *const crate::QStringRef,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QChar>> for crate::QStringRef {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QStringRef& lhs, QChar rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QChar>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_45(
                self as *const crate::QStringRef,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QChar>> for crate::QStringRef {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QStringRef& lhs, QChar rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QChar>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__68(
                self as *const crate::QStringRef,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QChar>> for crate::QStringRef {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QStringRef& lhs, QChar rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QChar>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__69(
                self as *const crate::QStringRef,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QLatin1String>> for crate::QChar {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(QChar lhs, QLatin1String rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, rhs: &::cpp_utils::Ref<crate::QLatin1String>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__70(
                self as *const crate::QChar,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QLatin1String>> for crate::QChar {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(QChar lhs, QLatin1String rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QLatin1String>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_46(self as *const crate::QChar, rhs.as_raw_ptr())
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QLatin1String>> for crate::QChar {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(QChar lhs, QLatin1String rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QLatin1String>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_47(self as *const crate::QChar, rhs.as_raw_ptr())
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QLatin1String>> for crate::QChar {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(QChar lhs, QLatin1String rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QLatin1String>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__72(
                self as *const crate::QChar,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QLatin1String>> for crate::QChar {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(QChar lhs, QLatin1String rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QLatin1String>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__73(
                self as *const crate::QChar,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QChar>> for crate::QLatin1String {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(QLatin1String lhs, QChar rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, rhs: &::cpp_utils::Ref<crate::QChar>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__74(
                self as *const crate::QLatin1String,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QChar>> for crate::QLatin1String {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(QLatin1String lhs, QChar rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QChar>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_48(
                self as *const crate::QLatin1String,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QChar>> for crate::QLatin1String {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(QLatin1String lhs, QChar rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QChar>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_49(
                self as *const crate::QLatin1String,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QChar>> for crate::QLatin1String {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(QLatin1String lhs, QChar rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QChar>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__76(
                self as *const crate::QLatin1String,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QChar>> for crate::QLatin1String {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(QLatin1String lhs, QChar rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QChar>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__77(
                self as *const crate::QLatin1String,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QByteArray>> for crate::QStringRef {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(const QStringRef& lhs, const QByteArray& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, rhs: &::cpp_utils::Ref<crate::QByteArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__78(
                self as *const crate::QStringRef,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QByteArray>> for crate::QStringRef {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QStringRef& lhs, const QByteArray& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QByteArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_50(
                self as *const crate::QStringRef,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QByteArray>> for crate::QStringRef {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QStringRef& lhs, const QByteArray& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QByteArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_51(
                self as *const crate::QStringRef,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QByteArray>> for crate::QStringRef {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QStringRef& lhs, const QByteArray& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QByteArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__80(
                self as *const crate::QStringRef,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QByteArray>> for crate::QStringRef {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QStringRef& lhs, const QByteArray& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QByteArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__81(
                self as *const crate::QStringRef,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QStringRef>> for crate::QByteArray {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(const QByteArray& lhs, const QStringRef& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, rhs: &::cpp_utils::Ref<crate::QStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__82(
                self as *const crate::QByteArray,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QStringRef>> for crate::QByteArray {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QByteArray& lhs, const QStringRef& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_52(
                self as *const crate::QByteArray,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QStringRef>> for crate::QByteArray {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QByteArray& lhs, const QStringRef& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_53(
                self as *const crate::QByteArray,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QStringRef>> for crate::QByteArray {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QByteArray& lhs, const QStringRef& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__84(
                self as *const crate::QByteArray,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QStringRef>> for crate::QByteArray {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QByteArray& lhs, const QStringRef& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__85(
                self as *const crate::QByteArray,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QStringRef>> for &crate::QString {
    type Output = ::cpp_utils::CppBox<crate::QString>;
    /// Calls C++ function: <span style='color: green;'>```QString operator+(const QString& s1, const QStringRef& s2)```</span>.
    fn add(self, s2: ::cpp_utils::Ref<crate::QStringRef>) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_56(
                self as *const crate::QString,
                s2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QString>> for &crate::QStringRef {
    type Output = ::cpp_utils::CppBox<crate::QString>;
    /// Calls C++ function: <span style='color: green;'>```QString operator+(const QStringRef& s1, const QString& s2)```</span>.
    fn add(self, s2: ::cpp_utils::Ref<crate::QString>) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_57(
                self as *const crate::QStringRef,
                s2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QLatin1String>> for &crate::QStringRef {
    type Output = ::cpp_utils::CppBox<crate::QString>;
    /// Calls C++ function: <span style='color: green;'>```QString operator+(const QStringRef& s1, QLatin1String s2)```</span>.
    fn add(
        self,
        s2: ::cpp_utils::Ref<crate::QLatin1String>,
    ) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_58(
                self as *const crate::QStringRef,
                s2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QStringRef>> for &crate::QLatin1String {
    type Output = ::cpp_utils::CppBox<crate::QString>;
    /// Calls C++ function: <span style='color: green;'>```QString operator+(QLatin1String s1, const QStringRef& s2)```</span>.
    fn add(self, s2: ::cpp_utils::Ref<crate::QStringRef>) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_59(
                self as *const crate::QLatin1String,
                s2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QStringRef>> for &crate::QStringRef {
    type Output = ::cpp_utils::CppBox<crate::QString>;
    /// Calls C++ function: <span style='color: green;'>```QString operator+(const QStringRef& s1, const QStringRef& s2)```</span>.
    fn add(self, s2: ::cpp_utils::Ref<crate::QStringRef>) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_60(
                self as *const crate::QStringRef,
                s2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QChar>> for &crate::QStringRef {
    type Output = ::cpp_utils::CppBox<crate::QString>;
    /// Calls C++ function: <span style='color: green;'>```QString operator+(const QStringRef& s1, QChar s2)```</span>.
    fn add(self, s2: ::cpp_utils::Ref<crate::QChar>) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_61(
                self as *const crate::QStringRef,
                s2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QStringRef>> for &crate::QChar {
    type Output = ::cpp_utils::CppBox<crate::QString>;
    /// Calls C++ function: <span style='color: green;'>```QString operator+(QChar s1, const QStringRef& s2)```</span>.
    fn add(self, s2: ::cpp_utils::Ref<crate::QStringRef>) -> ::cpp_utils::CppBox<crate::QString> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_62(self as *const crate::QChar, s2.as_raw_ptr())
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QRegExp {
    /// <p>Destroys the regular expression and cleans up its internal data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QRegExp::~QRegExp()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#dtor.QRegExp">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the regular expression and cleans up its internal data.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QRegExp_dQRegExp(self as *mut crate::QRegExp)
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QRegExp>> for crate::QRegExp {
    /// <p>Returns <code>true</code> if this regular expression is equal to <i>rx</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRegExp::operator==(const QRegExp& rx) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregexp.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this regular expression is equal to <i>rx</i>; otherwise returns <code>false</code>.</p>
    /// <p>Two <a href="http://doc.qt.io/qt-5/qregexp.html">QRegExp</a> objects are equal if they have the same pattern strings and the same settings for case sensitivity, wildcard and minimal matching.</p></div>
    fn eq(&self, rx: &::cpp_utils::Ref<crate::QRegExp>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QRegExp_operator__(
                self as *const crate::QRegExp,
                rx.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QRegExp>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& out, const QRegExp& regExp)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        reg_exp: ::cpp_utils::Ref<crate::QRegExp>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__90(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                reg_exp.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QRegExp>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& in, QRegExp& regExp)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        reg_exp: ::cpp_utils::MutRef<crate::QRegExp>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__91(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                reg_exp.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QRegExp>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QRegExp& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QRegExp>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__92(
                self as *const crate::QDebug,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QStringMatcher {
    /// <p>Destroys the string matcher.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QStringMatcher::~QStringMatcher()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringmatcher.html#dtor.QStringMatcher">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the string matcher.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QStringMatcher_dQStringMatcher(
            self as *mut crate::QStringMatcher,
        )
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QStringList>> for &crate::QStringList {
    type Output = ::cpp_utils::CppBox<crate::QStringList>;
    /// <p>Returns a string list that is the concatenation of this string list with the <i>other</i> string list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList QStringList::operator+(const QStringList& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a string list that is the concatenation of this string list with the <i>other</i> string list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn add(
        self,
        other: ::cpp_utils::Ref<crate::QStringList>,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QStringList_operator_2(
                self as *const crate::QStringList,
                other.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QString>> for &crate::QStringList {
    type Output = ::cpp_utils::MutRef<crate::QStringList>;
    /// <p>Appends the given string, <i>str</i>, to this string list and returns a reference to the string list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList& QStringList::operator<<(const QString& str)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the given string, <i>str</i>, to this string list and returns a reference to the string list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn shl(self, str: ::cpp_utils::Ref<crate::QString>) -> ::cpp_utils::MutRef<crate::QStringList> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QStringList_operator__(
                self as *const crate::QStringList as *mut crate::QStringList,
                str.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QStringList>> for &crate::QStringList {
    type Output = ::cpp_utils::MutRef<crate::QStringList>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList& QStringList::operator<<(const QStringList& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the <i>other</i> string list to the string list and returns a reference to the latter string list.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QStringList>,
    ) -> ::cpp_utils::MutRef<crate::QStringList> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QStringList_operator__1(
                self as *const crate::QStringList as *mut crate::QStringList,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQString>> for &crate::QStringList {
    type Output = ::cpp_utils::MutRef<crate::QStringList>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QStringList& QStringList::operator<<(const QList<QString>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html#operator-lt-lt-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends the <i>other</i> string list to the string list and returns a reference to the latter string list.</p>
    /// <p>This function was introduced in  Qt 5.4.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQString>,
    ) -> ::cpp_utils::MutRef<crate::QStringList> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QStringList_operator__2(
                self as *const crate::QStringList as *mut crate::QStringList,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QStringList>> for &crate::QListOfQString {
    type Output = ::cpp_utils::CppBox<crate::QStringList>;
    /// Calls C++ function: <span style='color: green;'>```QStringList operator+(const QList<QString>& one, const QStringList& other)```</span>.
    fn add(
        self,
        other: ::cpp_utils::Ref<crate::QStringList>,
    ) -> ::cpp_utils::CppBox<crate::QStringList> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_63(
                self as *const crate::QListOfQString,
                other.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QMetaType {
    /// <p>Destructs this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QMetaType::~QMetaType()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetatype.html#dtor.QMetaType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destructs this object.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMetaType_dQMetaType(self as *mut crate::QMetaType)
    }
}

impl ::cpp_utils::CppDeletable for crate::QObject {
    /// <p>Destroys the object, deleting all its child objects.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QObject::~QObject()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobject.html#dtor.QObject">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the object, deleting all its child objects.</p>
    /// <p>All signals to and from the object are automatically disconnected, and any pending posted events for the object are removed from the event queue. However, it is often safer to use <a href="http://doc.qt.io/qt-5/qobject.html#deleteLater">deleteLater</a>() rather than deleting a <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> subclass directly.</p>
    /// <p><b>Warning:</b> All child objects are deleted. If any of these objects are on the stack or global, sooner or later your program will crash. We do not recommend holding pointers to child objects from outside the parent. If you still do, the <a href="http://doc.qt.io/qt-5/qobject.html#destroyed">destroyed</a>() signal gives you an opportunity to detect when an object is destroyed.</p>
    /// <p><b>Warning:</b> Deleting a <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> while pending events are waiting to be delivered can cause a crash. You must not delete the <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> directly if it exists in a different thread than the one currently executing. Use <a href="http://doc.qt.io/qt-5/qobject.html#deleteLater">deleteLater</a>() instead, which will cause the event loop to delete the object after all pending events have been delivered to it.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#deleteLater">deleteLater</a>().</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QObject_dQObject(self as *mut crate::QObject)
    }
}

impl ::std::ops::Shl<::cpp_utils::Ptr<crate::QObject>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QObject* arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ptr<crate::QObject>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__93(
                self as *const crate::QDebug,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QSignalBlocker {
    /// <p>Destructor. Restores the <a href="http://doc.qt.io/qt-5/qobject.html#signalsBlocked">QObject::signalsBlocked</a>() state to what it was before the constructor ran, unless <a href="http://doc.qt.io/qt-5/qsignalblocker.html#unblock">unblock</a>() has been called without a following <a href="http://doc.qt.io/qt-5/qsignalblocker.html#reblock">reblock</a>(), in which case it does nothing.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QSignalBlocker::~QSignalBlocker()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignalblocker.html#dtor.QSignalBlocker">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destructor. Restores the <a href="http://doc.qt.io/qt-5/qobject.html#signalsBlocked">QObject::signalsBlocked</a>() state to what it was before the constructor ran, unless <a href="http://doc.qt.io/qt-5/qsignalblocker.html#unblock">unblock</a>() has been called without a following <a href="http://doc.qt.io/qt-5/qsignalblocker.html#reblock">reblock</a>(), in which case it does nothing.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSignalBlocker_dQSignalBlocker(
            self as *mut crate::QSignalBlocker,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QAbstractAnimation {
    /// <p>Stops the animation if it's running, then destroys the <a href="http://doc.qt.io/qt-5/qabstractanimation.html">QAbstractAnimation</a>. If the animation is part of a <a href="http://doc.qt.io/qt-5/qanimationgroup.html">QAnimationGroup</a>, it is automatically removed before it's destroyed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QAbstractAnimation::~QAbstractAnimation()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractanimation.html#dtor.QAbstractAnimation">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Stops the animation if it's running, then destroys the <a href="http://doc.qt.io/qt-5/qabstractanimation.html">QAbstractAnimation</a>. If the animation is part of a <a href="http://doc.qt.io/qt-5/qanimationgroup.html">QAnimationGroup</a>, it is automatically removed before it's destroyed.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractAnimation_dQAbstractAnimation(
            self as *mut crate::QAbstractAnimation,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QAnimationDriver {
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QAnimationDriver::~QAnimationDriver()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QAnimationDriver_dQAnimationDriver(
            self as *mut crate::QAnimationDriver,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QEventLoop {
    /// <p>Destroys the event loop object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QEventLoop::~QEventLoop()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventloop.html#dtor.QEventLoop">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the event loop object.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QEventLoop_dQEventLoop(self as *mut crate::QEventLoop)
    }
}

impl ::cpp_utils::CppDeletable for crate::QEventLoopLocker {
    /// <p>Destroys this event loop locker object</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QEventLoopLocker::~QEventLoopLocker()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventlooplocker.html#dtor.QEventLoopLocker">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys this event loop locker object</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QEventLoopLocker_dQEventLoopLocker(
            self as *mut crate::QEventLoopLocker,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QAbstractEventDispatcher {
    /// <p>Destroys the event dispatcher.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QAbstractEventDispatcher::~QAbstractEventDispatcher()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracteventdispatcher.html#dtor.QAbstractEventDispatcher">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the event dispatcher.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractEventDispatcher_dQAbstractEventDispatcher(
            self as *mut crate::QAbstractEventDispatcher,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QVariant {
    /// <p>Destroys the <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> and the contained object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QVariant::~QVariant()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#dtor.QVariant">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> and the contained object.</p>
    /// <p>Note that subclasses that reimplement <a href="http://doc.qt.io/qt-5/qvariant.html#clear">clear</a>() should reimplement the destructor to call <a href="http://doc.qt.io/qt-5/qvariant.html#clear">clear</a>(). This destructor calls <a href="http://doc.qt.io/qt-5/qvariant.html#clear">clear</a>(), but because it is the destructor, <a href="http://doc.qt.io/qt-5/qvariant.html#clear">QVariant::clear</a>() is called rather than a subclass's <a href="http://doc.qt.io/qt-5/qvariant.html#clear">clear</a>().</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVariant_dQVariant(self as *mut crate::QVariant)
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QVariant>> for crate::QVariant {
    /// <p>Compares this <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> with <i>v</i> and returns <code>true</code> if they are equal; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVariant::operator==(const QVariant& v) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares this <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> with <i>v</i> and returns <code>true</code> if they are equal; otherwise returns <code>false</code>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> uses the equality operator of the <a href="http://doc.qt.io/qt-5/qvariant.html#type">type</a>() it contains to check for equality. <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> will try to <a href="http://doc.qt.io/qt-5/qvariant.html#convert">convert</a>() <i>v</i> if its type is not the same as this variant's type. See <a href="http://doc.qt.io/qt-5/qvariant.html#canConvert-1">canConvert</a>() for a list of possible conversions.</p>
    /// <p><b>Warning:</b> To make this function work with a custom type registered with <a href="http://doc.qt.io/qt-5/qmetatype.html#qRegisterMetaType-1">qRegisterMetaType</a>(), its comparison operator must be registered using <a href="http://doc.qt.io/qt-5/qmetatype.html#registerComparators">QMetaType::registerComparators</a>().</p></div>
    fn eq(&self, v: &::cpp_utils::Ref<crate::QVariant>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVariant_operator__(
                self as *const crate::QVariant,
                v.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QVariant>> for crate::QVariant {
    /// <p>Compares this <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> with <i>v</i> and returns <code>true</code> if this is less than <i>v</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVariant::operator<(const QVariant& v) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares this <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> with <i>v</i> and returns <code>true</code> if this is less than <i>v</i>.</p>
    /// <p><b>Note: </b>Comparability might not be availabe for the type stored in this <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> or in <i>v</i>.</p><p><b>Warning:</b> To make this function work with a custom type registered with <a href="http://doc.qt.io/qt-5/qmetatype.html#qRegisterMetaType-1">qRegisterMetaType</a>(), its comparison operator must be registered using <a href="http://doc.qt.io/qt-5/qmetatype.html#registerComparators">QMetaType::registerComparators</a>().</p></div>
    fn lt(&self, v: &::cpp_utils::Ref<crate::QVariant>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVariant_operator_2(
                self as *const crate::QVariant,
                v.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QVariant>> for crate::QVariant {
    /// <p>Compares this <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> with <i>v</i> and returns <code>true</code> if this is less or equal than <i>v</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVariant::operator<=(const QVariant& v) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares this <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> with <i>v</i> and returns <code>true</code> if this is less or equal than <i>v</i>.</p>
    /// <p><b>Note: </b>Comparability might not be available for the type stored in this <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> or in <i>v</i>.</p><p><b>Warning:</b> To make this function work with a custom type registered with <a href="http://doc.qt.io/qt-5/qmetatype.html#qRegisterMetaType-1">qRegisterMetaType</a>(), its comparison operator must be registered using <a href="http://doc.qt.io/qt-5/qmetatype.html#registerComparators">QMetaType::registerComparators</a>().</p></div>
    fn le(&self, v: &::cpp_utils::Ref<crate::QVariant>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVariant_operator__2(
                self as *const crate::QVariant,
                v.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QVariant>> for crate::QVariant {
    /// <p>Compares this <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> with <i>v</i> and returns <code>true</code> if this is larger than <i>v</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVariant::operator>(const QVariant& v) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares this <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> with <i>v</i> and returns <code>true</code> if this is larger than <i>v</i>.</p>
    /// <p><b>Note: </b>Comparability might not be available for the type stored in this <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> or in <i>v</i>.</p><p><b>Warning:</b> To make this function work with a custom type registered with <a href="http://doc.qt.io/qt-5/qmetatype.html#qRegisterMetaType-1">qRegisterMetaType</a>(), its comparison operator must be registered using <a href="http://doc.qt.io/qt-5/qmetatype.html#registerComparators">QMetaType::registerComparators</a>().</p></div>
    fn gt(&self, v: &::cpp_utils::Ref<crate::QVariant>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVariant_operator_3(
                self as *const crate::QVariant,
                v.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QVariant>> for crate::QVariant {
    /// <p>Compares this <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> with <i>v</i> and returns <code>true</code> if this is larger or equal than <i>v</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVariant::operator>=(const QVariant& v) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariant.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares this <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> with <i>v</i> and returns <code>true</code> if this is larger or equal than <i>v</i>.</p>
    /// <p><b>Note: </b>Comparability might not be available for the type stored in this <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> or in <i>v</i>.</p><p><b>Warning:</b> To make this function work with a custom type registered with <a href="http://doc.qt.io/qt-5/qmetatype.html#qRegisterMetaType-1">qRegisterMetaType</a>(), its comparison operator must be registered using <a href="http://doc.qt.io/qt-5/qmetatype.html#registerComparators">QMetaType::registerComparators</a>().</p></div>
    fn ge(&self, v: &::cpp_utils::Ref<crate::QVariant>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVariant_operator__3(
                self as *const crate::QVariant,
                v.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QVariant>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& s, QVariant& p)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        p: ::cpp_utils::MutRef<crate::QVariant>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__94(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                p.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVariant>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QVariant& p)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, p: ::cpp_utils::Ref<crate::QVariant>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__95(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                p.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::q_variant::Type>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& s, QVariant::Type& p)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        p: ::cpp_utils::MutRef<crate::q_variant::Type>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__96(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                p.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::q_variant::Type> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QVariant::Type p)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, p: crate::q_variant::Type) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__97(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                p,
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QSequentialIterable {
    type Output = ::cpp_utils::CppBox<crate::q_sequential_iterable::ConstIterator>;
    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qsequentialiterable-const-iterator.html">QSequentialIterable::const_iterator</a> for the beginning of the container. This can be used in stl-style iteration.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSequentialIterable::const_iterator QSequentialIterable::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsequentialiterable.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qsequentialiterable-const-iterator.html">QSequentialIterable::const_iterator</a> for the beginning of the container. This can be used in stl-style iteration.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsequentialiterable.html#end">end</a>().</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_sequential_iterable::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QSequentialIterable_begin(
                self as *const crate::QSequentialIterable,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QSequentialIterable {
    type Output = ::cpp_utils::CppBox<crate::q_sequential_iterable::ConstIterator>;
    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qsequentialiterable-const-iterator.html">QSequentialIterable::const_iterator</a> for the end of the container. This can be used in stl-style iteration.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSequentialIterable::const_iterator QSequentialIterable::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsequentialiterable.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qsequentialiterable-const-iterator.html">QSequentialIterable::const_iterator</a> for the end of the container. This can be used in stl-style iteration.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsequentialiterable.html#begin">begin</a>().</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_sequential_iterable::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QSequentialIterable_end(
                self as *const crate::QSequentialIterable,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QAssociativeIterable {
    type Output = ::cpp_utils::CppBox<crate::q_associative_iterable::ConstIterator>;
    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html">QAssociativeIterable::const_iterator</a> for the beginning of the container. This can be used in stl-style iteration.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAssociativeIterable::const_iterator QAssociativeIterable::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html">QAssociativeIterable::const_iterator</a> for the beginning of the container. This can be used in stl-style iteration.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">end</a>().</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_associative_iterable::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QAssociativeIterable_begin(
                self as *const crate::QAssociativeIterable,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QAssociativeIterable {
    type Output = ::cpp_utils::CppBox<crate::q_associative_iterable::ConstIterator>;
    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html">QAssociativeIterable::const_iterator</a> for the end of the container. This can be used in stl-style iteration.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QAssociativeIterable::const_iterator QAssociativeIterable::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qassociativeiterable.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qassociativeiterable-const-iterator.html">QAssociativeIterable::const_iterator</a> for the end of the container. This can be used in stl-style iteration.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qassociativeiterable.html#begin">begin</a>().</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_associative_iterable::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QAssociativeIterable_end(
                self as *const crate::QAssociativeIterable,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVariant>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QVariant& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QVariant>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__98(
                self as *const crate::QDebug,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::q_variant::Type> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, QVariant::Type arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: crate::q_variant::Type) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__99(self as *const crate::QDebug, arg2)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QModelIndex>> for crate::QModelIndex {
    /// <p>Returns <code>true</code> if this model index refers to the same location as the <i>other</i> model index; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QModelIndex::operator==(const QModelIndex& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmodelindex.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this model index refers to the same location as the <i>other</i> model index; otherwise returns <code>false</code>.</p>
    /// <p>All values in the model index are used when comparing with another model index.</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QModelIndex>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QModelIndex_operator__(
                self as *const crate::QModelIndex,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QModelIndex>> for crate::QModelIndex {
    /// <p>Returns <code>true</code> if this model index is smaller than the <i>other</i> model index; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QModelIndex::operator<(const QModelIndex& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmodelindex.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this model index is smaller than the <i>other</i> model index; otherwise returns <code>false</code>.</p>
    /// <p>The less than calculation is not directly useful to developers - the way that indexes with different parents compare is not defined. This operator only exists so that the class can be used with <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>.</p>
    /// <p>This function was introduced in  Qt 4.1.</p></div>
    fn lt(&self, other: &::cpp_utils::Ref<crate::QModelIndex>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QModelIndex_operator_(
                self as *const crate::QModelIndex,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QModelIndex>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QModelIndex& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QModelIndex>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__100(
                self as *const crate::QDebug,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QPersistentModelIndex {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QPersistentModelIndex::~QPersistentModelIndex()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QPersistentModelIndex_dQPersistentModelIndex(
            self as *mut crate::QPersistentModelIndex,
        )
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QPersistentModelIndex>>
    for crate::QPersistentModelIndex
{
    /// <p>Returns <code>true</code> if this persistent model index is smaller than the <i>other</i> persistent model index; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QPersistentModelIndex::operator<(const QPersistentModelIndex& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this persistent model index is smaller than the <i>other</i> persistent model index; otherwise returns <code>false</code>.</p>
    /// <p>All values in the persistent model index are used when comparing with another persistent model index.</p>
    /// <p>This function was introduced in  Qt 4.1.</p></div>
    fn lt(&self, other: &::cpp_utils::Ref<crate::QPersistentModelIndex>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QPersistentModelIndex_operator_(
                self as *const crate::QPersistentModelIndex,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QPersistentModelIndex>>
    for crate::QPersistentModelIndex
{
    /// <p>Returns <code>true</code> if this persistent model index is equal to the <i>other</i> persistent model index; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QPersistentModelIndex::operator==(const QPersistentModelIndex& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this persistent model index is equal to the <i>other</i> persistent model index; otherwise returns <code>false</code>.</p>
    /// <p>All values in the persistent model index are used when comparing with another persistent model index.</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QPersistentModelIndex>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QPersistentModelIndex_operator__(
                self as *const crate::QPersistentModelIndex,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QModelIndex>> for crate::QPersistentModelIndex {
    /// <p>Returns <code>true</code> if this persistent model index refers to the same location as the <i>other</i> model index; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QPersistentModelIndex::operator==(const QModelIndex& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html#operator-eq-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this persistent model index refers to the same location as the <i>other</i> model index; otherwise returns <code>false</code>.</p>
    /// <p>All values in the persistent model index are used when comparing with another model index.</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QModelIndex>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QPersistentModelIndex_operator__2(
                self as *const crate::QPersistentModelIndex,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QPersistentModelIndex>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QPersistentModelIndex& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        arg2: ::cpp_utils::Ref<crate::QPersistentModelIndex>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__101(
                self as *const crate::QDebug,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QAbstractItemModel {
    /// <p>Destroys the abstract item model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QAbstractItemModel::~QAbstractItemModel()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#dtor.QAbstractItemModel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the abstract item model.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractItemModel_dQAbstractItemModel(
            self as *mut crate::QAbstractItemModel,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QAbstractTableModel {
    /// <p>Destroys the abstract table model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QAbstractTableModel::~QAbstractTableModel()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracttablemodel.html#dtor.QAbstractTableModel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the abstract table model.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractTableModel_dQAbstractTableModel(
            self as *mut crate::QAbstractTableModel,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QAbstractListModel {
    /// <p>Destroys the abstract list model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QAbstractListModel::~QAbstractListModel()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractlistmodel.html#dtor.QAbstractListModel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the abstract list model.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractListModel_dQAbstractListModel(
            self as *mut crate::QAbstractListModel,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QAbstractNativeEventFilter {
    /// <p>Destroys the native event filter.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QAbstractNativeEventFilter::~QAbstractNativeEventFilter()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractnativeeventfilter.html#dtor.QAbstractNativeEventFilter">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the native event filter.</p>
    /// <p>This automatically removes it from the application.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractNativeEventFilter_dQAbstractNativeEventFilter(
            self as *mut crate::QAbstractNativeEventFilter,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QAbstractProxyModel {
    /// <p>Destroys the proxy model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QAbstractProxyModel::~QAbstractProxyModel()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractproxymodel.html#dtor.QAbstractProxyModel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the proxy model.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractProxyModel_dQAbstractProxyModel(
            self as *mut crate::QAbstractProxyModel,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QAbstractState {
    /// <p>Destroys this state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QAbstractState::~QAbstractState()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstractstate.html#dtor.QAbstractState">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys this state.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractState_dQAbstractState(
            self as *mut crate::QAbstractState,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QAbstractTransition {
    /// <p>Destroys this transition.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QAbstractTransition::~QAbstractTransition()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qabstracttransition.html#dtor.QAbstractTransition">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys this transition.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QAbstractTransition_dQAbstractTransition(
            self as *mut crate::QAbstractTransition,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QAnimationGroup {
    /// <p>Destroys the animation group. It will also destroy all its animations.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QAnimationGroup::~QAnimationGroup()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qanimationgroup.html#dtor.QAnimationGroup">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the animation group. It will also destroy all its animations.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QAnimationGroup_dQAnimationGroup(
            self as *mut crate::QAnimationGroup,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QBasicTimer {
    /// <p>Destroys the basic timer.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QBasicTimer::~QBasicTimer()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbasictimer.html#dtor.QBasicTimer">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the basic timer.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QBasicTimer_dQBasicTimer(self as *mut crate::QBasicTimer)
    }
}

impl ::std::ops::BitAndAssign<::cpp_utils::Ref<crate::QBitArray>> for crate::QBitArray {
    /// <p>Performs the AND operation between all bits in this bit array and <i>other</i>. Assigns the result to this bit array, and returns a reference to it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QBitArray& QBitArray::operator&=(const QBitArray& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-and-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Performs the AND operation between all bits in this bit array and <i>other</i>. Assigns the result to this bit array, and returns a reference to it.</p>
    /// <p>The result has the length of the longest of the two bit arrays, with any missing bits (if one array is shorter than the other) taken to be 0.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span> a(<span class="number">3</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span> b(<span class="number">2</span>);
    ///   a<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>; a<span class="operator">[</span><span class="number">1</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">0</span>; a<span class="operator">[</span><span class="number">2</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>;   <span class="comment">// a: [ 1, 0, 1 ]</span>
    ///   b<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>; b<span class="operator">[</span><span class="number">1</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>;             <span class="comment">// b: [ 1, 1 ]</span>
    ///   a <span class="operator">&amp;</span><span class="operator">=</span> b;                         <span class="comment">// a: [ 1, 0, 0 ]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbitarray.html#operator-and">operator&amp;</a>(), <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-7c-eq">operator|=</a>(), <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-5e-eq">operator^=</a>(), and <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-7e">operator~</a>().</p></div>
    fn bitand_assign(&mut self, arg1: ::cpp_utils::Ref<crate::QBitArray>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QBitArray_operator__4(
                self as *mut crate::QBitArray,
                arg1.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::BitOrAssign<::cpp_utils::Ref<crate::QBitArray>> for crate::QBitArray {
    /// <p>Performs the OR operation between all bits in this bit array and <i>other</i>. Assigns the result to this bit array, and returns a reference to it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QBitArray& QBitArray::operator|=(const QBitArray& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-7c-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Performs the OR operation between all bits in this bit array and <i>other</i>. Assigns the result to this bit array, and returns a reference to it.</p>
    /// <p>The result has the length of the longest of the two bit arrays, with any missing bits (if one array is shorter than the other) taken to be 0.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span> a(<span class="number">3</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span> b(<span class="number">2</span>);
    ///   a<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>; a<span class="operator">[</span><span class="number">1</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">0</span>; a<span class="operator">[</span><span class="number">2</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>;   <span class="comment">// a: [ 1, 0, 1 ]</span>
    ///   b<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>; b<span class="operator">[</span><span class="number">1</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>;             <span class="comment">// b: [ 1, 1 ]</span>
    ///   a <span class="operator">|</span><span class="operator">=</span> b;                         <span class="comment">// a: [ 1, 1, 1 ]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbitarray.html#operator-7c">operator|</a>(), <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-and-eq">operator&amp;=</a>(), <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-5e-eq">operator^=</a>(), and <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-7e">operator~</a>().</p></div>
    fn bitor_assign(&mut self, arg1: ::cpp_utils::Ref<crate::QBitArray>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QBitArray_operator__5(
                self as *mut crate::QBitArray,
                arg1.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::BitXorAssign<::cpp_utils::Ref<crate::QBitArray>> for crate::QBitArray {
    /// <p>Performs the XOR operation between all bits in this bit array and <i>other</i>. Assigns the result to this bit array, and returns a reference to it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QBitArray& QBitArray::operator^=(const QBitArray& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-5e-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Performs the XOR operation between all bits in this bit array and <i>other</i>. Assigns the result to this bit array, and returns a reference to it.</p>
    /// <p>The result has the length of the longest of the two bit arrays, with any missing bits (if one array is shorter than the other) taken to be 0.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span> a(<span class="number">3</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span> b(<span class="number">2</span>);
    ///   a<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>; a<span class="operator">[</span><span class="number">1</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">0</span>; a<span class="operator">[</span><span class="number">2</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>;   <span class="comment">// a: [ 1, 0, 1 ]</span>
    ///   b<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>; b<span class="operator">[</span><span class="number">1</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>;             <span class="comment">// b: [ 1, 1 ]</span>
    ///   a <span class="operator">^</span><span class="operator">=</span> b;                         <span class="comment">// a: [ 0, 1, 1 ]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbitarray.html#operator-5e">operator^</a>(), <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-and-eq">operator&amp;=</a>(), <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-7c-eq">operator|=</a>(), and <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-7e">operator~</a>().</p></div>
    fn bitxor_assign(&mut self, arg1: ::cpp_utils::Ref<crate::QBitArray>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QBitArray_operator__6(
                self as *mut crate::QBitArray,
                arg1.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QBitArray>> for crate::QBitArray {
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this bit array; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QBitArray::operator==(const QBitArray& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this bit array; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbitarray.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QBitArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QBitArray_operator__7(
                self as *const crate::QBitArray,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::BitAnd<::cpp_utils::Ref<crate::QBitArray>> for &crate::QBitArray {
    type Output = ::cpp_utils::CppBox<crate::QBitArray>;
    /// <p>Returns a bit array that is the AND of the bit arrays <i>a1</i> and <i>a2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QBitArray operator&(const QBitArray& arg1, const QBitArray& arg2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-and">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a bit array that is the AND of the bit arrays <i>a1</i> and <i>a2</i>.</p>
    /// <p>The result has the length of the longest of the two bit arrays, with any missing bits (if one array is shorter than the other) taken to be 0.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a></span> a(<span class="number">3</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a></span> b(<span class="number">2</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a></span> c;
    ///   a<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>; a<span class="operator">[</span><span class="number">1</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">0</span>; a<span class="operator">[</span><span class="number">2</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>;   <span class="comment">// a: [ 1, 0, 1 ]</span>
    ///   b<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>; b<span class="operator">[</span><span class="number">1</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>;             <span class="comment">// b: [ 1, 1 ]</span>
    ///   c <span class="operator">=</span> a <span class="operator">&amp;</span> b;                      <span class="comment">// c: [ 1, 0, 0 ]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbitarray.html#operator-and-eq">operator&amp;=</a>(), <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-7c">operator|</a>(), and <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-5e">operator^</a>().</p></div>
    fn bitand(
        self,
        arg2: ::cpp_utils::Ref<crate::QBitArray>,
    ) -> ::cpp_utils::CppBox<crate::QBitArray> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_64(
                self as *const crate::QBitArray,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::BitOr<::cpp_utils::Ref<crate::QBitArray>> for &crate::QBitArray {
    type Output = ::cpp_utils::CppBox<crate::QBitArray>;
    /// <p>Returns a bit array that is the OR of the bit arrays <i>a1</i> and <i>a2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QBitArray operator|(const QBitArray& arg1, const QBitArray& arg2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-7c">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a bit array that is the OR of the bit arrays <i>a1</i> and <i>a2</i>.</p>
    /// <p>The result has the length of the longest of the two bit arrays, with any missing bits (if one array is shorter than the other) taken to be 0.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a></span> a(<span class="number">3</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a></span> b(<span class="number">2</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a></span> c;
    ///   a<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>; a<span class="operator">[</span><span class="number">1</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">0</span>; a<span class="operator">[</span><span class="number">2</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>;   <span class="comment">// a: [ 1, 0, 1 ]</span>
    ///   b<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>; b<span class="operator">[</span><span class="number">1</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>;             <span class="comment">// b: [ 1, 1 ]</span>
    ///   c <span class="operator">=</span> a <span class="operator">|</span> b;                      <span class="comment">// c: [ 1, 1, 1 ]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbitarray.html#operator-7c-eq">QBitArray::operator|=</a>(), <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-and">operator&amp;</a>(), and <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-5e">operator^</a>().</p></div>
    fn bitor(
        self,
        arg2: ::cpp_utils::Ref<crate::QBitArray>,
    ) -> ::cpp_utils::CppBox<crate::QBitArray> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_65(
                self as *const crate::QBitArray,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::BitXor<::cpp_utils::Ref<crate::QBitArray>> for &crate::QBitArray {
    type Output = ::cpp_utils::CppBox<crate::QBitArray>;
    /// <p>Returns a bit array that is the XOR of the bit arrays <i>a1</i> and <i>a2</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QBitArray operator^(const QBitArray& arg1, const QBitArray& arg2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-5e">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a bit array that is the XOR of the bit arrays <i>a1</i> and <i>a2</i>.</p>
    /// <p>The result has the length of the longest of the two bit arrays, with any missing bits (if one array is shorter than the other) taken to be 0.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a></span> a(<span class="number">3</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a></span> b(<span class="number">2</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a></span> c;
    ///   a<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>; a<span class="operator">[</span><span class="number">1</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">0</span>; a<span class="operator">[</span><span class="number">2</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>;   <span class="comment">// a: [ 1, 0, 1 ]</span>
    ///   b<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>; b<span class="operator">[</span><span class="number">1</span><span class="operator">]</span> <span class="operator">=</span> <span class="number">1</span>;             <span class="comment">// b: [ 1, 1 ]</span>
    ///   c <span class="operator">=</span> a <span class="operator">^</span> b;                      <span class="comment">// c: [ 0, 1, 1 ]</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbitarray.html">operator^=</a>(), <a href="http://doc.qt.io/qt-5/qbitarray.html">operator&amp;</a>(), and <a href="http://doc.qt.io/qt-5/qbitarray.html">operator|</a>().</p></div>
    fn bitxor(
        self,
        arg2: ::cpp_utils::Ref<crate::QBitArray>,
    ) -> ::cpp_utils::CppBox<crate::QBitArray> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_66(
                self as *const crate::QBitArray,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Not for &crate::QBitRef {
    type Output = bool;
    /// Calls C++ function: <span style='color: green;'>```bool QBitRef::operator!() const```</span>.
    fn not(self) -> bool {
        unsafe { crate::__ffi::ctr_qt_core_ffi_QBitRef_operator_(self as *const crate::QBitRef) }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QBitArray>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, const QBitArray& arg2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        arg2: ::cpp_utils::Ref<crate::QBitArray>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__102(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QBitArray>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QBitArray& arg2)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QBitArray>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__103(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QBitArray>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QBitArray& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QBitArray>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__104(
                self as *const crate::QDebug,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QIODevice {
    /// <p>The destructor is virtual, and <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> is an abstract base class. This destructor does not call <a href="http://doc.qt.io/qt-5/qiodevice.html#close">close</a>(), but the subclass destructor might. If you are in doubt, call <a href="http://doc.qt.io/qt-5/qiodevice.html#close">close</a>() before destroying the <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QIODevice::~QIODevice()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qiodevice.html#dtor.QIODevice">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The destructor is virtual, and <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> is an abstract base class. This destructor does not call <a href="http://doc.qt.io/qt-5/qiodevice.html#close">close</a>(), but the subclass destructor might. If you are in doubt, call <a href="http://doc.qt.io/qt-5/qiodevice.html#close">close</a>() before destroying the <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a>.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QIODevice_dQIODevice(self as *mut crate::QIODevice)
    }
}

impl ::cpp_utils::CppDeletable for crate::QBuffer {
    /// <p>Destroys the buffer.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QBuffer::~QBuffer()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbuffer.html#dtor.QBuffer">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the buffer.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QBuffer_dQBuffer(self as *mut crate::QBuffer)
    }
}

impl ::cpp_utils::CppDeletable for crate::QByteArrayMatcher {
    /// <p>Destroys the byte array matcher.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QByteArrayMatcher::~QByteArrayMatcher()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbytearraymatcher.html#dtor.QByteArrayMatcher">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the byte array matcher.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QByteArrayMatcher_dQByteArrayMatcher(
            self as *mut crate::QByteArrayMatcher,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QLocale {
    /// <p>Destructor</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QLocale::~QLocale()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#dtor.QLocale">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destructor</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QLocale_dQLocale(self as *mut crate::QLocale)
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QLocale>> for crate::QLocale {
    /// <p>Returns <code>true</code> if the <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object is the same as the <i>other</i> locale specified; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLocale::operator==(const QLocale& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlocale.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the <a href="http://doc.qt.io/qt-5/qlocale.html">QLocale</a> object is the same as the <i>other</i> locale specified; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QLocale>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QLocale_operator__(
                self as *const crate::QLocale,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QLocale>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, const QLocale& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        arg2: ::cpp_utils::Ref<crate::QLocale>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__105(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QLocale>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QLocale& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QLocale>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__106(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QLocale>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QLocale& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QLocale>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__107(
                self as *const crate::QDebug,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QCollatorSortKey {
    /// <p>Destroys the collator key.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QCollatorSortKey::~QCollatorSortKey()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcollatorsortkey.html#dtor.QCollatorSortKey">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the collator key.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCollatorSortKey_dQCollatorSortKey(
            self as *mut crate::QCollatorSortKey,
        )
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QCollatorSortKey>> for crate::QCollatorSortKey {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QCollatorSortKey& lhs, const QCollatorSortKey& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QCollatorSortKey>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_67(
                self as *const crate::QCollatorSortKey,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QCollator {
    /// <p>Destroys the collator.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QCollator::~QCollator()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcollator.html#dtor.QCollator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the collator.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCollator_dQCollator(self as *mut crate::QCollator)
    }
}

impl ::cpp_utils::CppDeletable for crate::QCommandLineOption {
    /// <p>Destroys the command line option object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QCommandLineOption::~QCommandLineOption()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineoption.html#dtor.QCommandLineOption">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the command line option object.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCommandLineOption_dQCommandLineOption(
            self as *mut crate::QCommandLineOption,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QEvent {
    /// <p>Destroys the event. If it was <a href="http://doc.qt.io/qt-5/qcoreapplication.html#postEvent">posted</a>, it will be removed from the list of events to be posted.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QEvent::~QEvent()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qevent.html#dtor.QEvent">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the event. If it was <a href="http://doc.qt.io/qt-5/qcoreapplication.html#postEvent">posted</a>, it will be removed from the list of events to be posted.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QEvent_dQEvent(self as *mut crate::QEvent)
    }
}

impl ::cpp_utils::CppDeletable for crate::QTimerEvent {
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QTimerEvent::~QTimerEvent()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QTimerEvent_dQTimerEvent(self as *mut crate::QTimerEvent)
    }
}

impl ::cpp_utils::CppDeletable for crate::QChildEvent {
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QChildEvent::~QChildEvent()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QChildEvent_dQChildEvent(self as *mut crate::QChildEvent)
    }
}

impl ::cpp_utils::CppDeletable for crate::QDynamicPropertyChangeEvent {
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QDynamicPropertyChangeEvent::~QDynamicPropertyChangeEvent()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QDynamicPropertyChangeEvent_dQDynamicPropertyChangeEvent(
            self as *mut crate::QDynamicPropertyChangeEvent,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QDeferredDeleteEvent {
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QDeferredDeleteEvent::~QDeferredDeleteEvent()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QDeferredDeleteEvent_dQDeferredDeleteEvent(
            self as *mut crate::QDeferredDeleteEvent,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QCoreApplication {
    /// <p>Destroys the <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QCoreApplication::~QCoreApplication()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#dtor.QCoreApplication">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> object.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCoreApplication_dQCoreApplication(
            self as *mut crate::QCoreApplication,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QCommandLineParser {
    /// <p>Destroys the command line parser object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QCommandLineParser::~QCommandLineParser()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcommandlineparser.html#dtor.QCommandLineParser">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the command line parser object.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCommandLineParser_dQCommandLineParser(
            self as *mut crate::QCommandLineParser,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QCryptographicHash {
    /// <p>Destroys the object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QCryptographicHash::~QCryptographicHash()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcryptographichash.html#dtor.QCryptographicHash">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the object.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCryptographicHash_dQCryptographicHash(
            self as *mut crate::QCryptographicHash,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QDataStream {
    /// <p>Destroys the data stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QDataStream::~QDataStream()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#dtor.QDataStream">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the data stream.</p>
    /// <p>The destructor will not affect the current I/O device, unless it is an internal I/O device (e.g. a <a href="http://doc.qt.io/qt-5/qbuffer.html">QBuffer</a>) processing a <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> passed in the <i>constructor</i>, in which case the internal I/O device is destroyed.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QDataStream_dQDataStream(self as *mut crate::QDataStream)
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<i8>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a signed byte from the stream into <i>i</i>, and returns a reference to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::operator>>(qint8& i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#operator-gt-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a signed byte from the stream into <i>i</i>, and returns a reference to the stream.</p></div>
    fn shr(self, i: ::cpp_utils::MutRef<i8>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDataStream_operator__(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                i.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<u8>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::operator>>(quint8& i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#operator-gt-gt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Reads an unsigned byte from the stream into <i>i</i>, and returns a reference to the stream.</p></div>
    fn shr(self, i: ::cpp_utils::MutRef<u8>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDataStream_operator__1(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                i.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<i16>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::operator>>(qint16& i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#operator-gt-gt-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Reads a signed 16-bit integer from the stream into <i>i</i>, and returns a reference to the stream.</p></div>
    fn shr(self, i: ::cpp_utils::MutRef<i16>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDataStream_operator__2(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                i.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<u16>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::operator>>(quint16& i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#operator-gt-gt-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Reads an unsigned 16-bit integer from the stream into <i>i</i>, and returns a reference to the stream.</p></div>
    fn shr(self, i: ::cpp_utils::MutRef<u16>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDataStream_operator__3(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                i.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<i32>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::operator>>(qint32& i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#operator-gt-gt-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Reads a signed 32-bit integer from the stream into <i>i</i>, and returns a reference to the stream.</p></div>
    fn shr(self, i: ::cpp_utils::MutRef<i32>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDataStream_operator__4(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                i.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<u32>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::operator>>(quint32& i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#operator-gt-gt-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Reads an unsigned 32-bit integer from the stream into <i>i</i>, and returns a reference to the stream.</p></div>
    fn shr(self, i: ::cpp_utils::MutRef<u32>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDataStream_operator__5(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                i.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<i64>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::operator>>(qint64& i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#operator-gt-gt-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Reads a signed 64-bit integer from the stream into <i>i</i>, and returns a reference to the stream.</p></div>
    fn shr(self, i: ::cpp_utils::MutRef<i64>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDataStream_operator__6(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                i.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<u64>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::operator>>(quint64& i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#operator-gt-gt-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Reads an unsigned 64-bit integer from the stream, into <i>i</i>, and returns a reference to the stream.</p></div>
    fn shr(self, i: ::cpp_utils::MutRef<u64>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDataStream_operator__7(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                i.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<bool>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a boolean value from the stream into <i>i</i>. Returns a reference to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::operator>>(bool& i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#operator-gt-gt-9">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a boolean value from the stream into <i>i</i>. Returns a reference to the stream.</p></div>
    fn shr(self, i: ::cpp_utils::MutRef<bool>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDataStream_operator__8(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                i.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::Qfloat16>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::operator>>(qfloat16& f)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#operator-gt-gt-10">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Reads a floating point number from the stream into <i>f</i>, using the standard IEEE 754 format. Returns a reference to the stream.</p>
    /// <p>This function was introduced in  Qt 5.9.</p></div>
    fn shr(
        self,
        f: ::cpp_utils::MutRef<crate::Qfloat16>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDataStream_operator__9(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                f.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<::std::os::raw::c_float>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::operator>>(float& f)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#operator-gt-gt-11">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Reads a floating point number from the stream into <i>f</i>, using the standard IEEE 754 format. Returns a reference to the stream.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatastream.html#setFloatingPointPrecision">setFloatingPointPrecision</a>().</p></div>
    fn shr(
        self,
        f: ::cpp_utils::MutRef<::std::os::raw::c_float>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDataStream_operator__10(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                f.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<*mut ::std::os::raw::c_char>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::operator>>(char*& str)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#operator-gt-gt-13">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Reads the '\0'-terminated string <i>s</i> from the stream and returns a reference to the stream.</p>
    /// <p>The string is deserialized using <code>readBytes()</code>.</p>
    /// <p>Space for the string is allocated using <code>new []</code> -- the caller must destroy it with <code>delete []</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatastream.html#readBytes">readBytes</a>() and <a href="http://doc.qt.io/qt-5/qdatastream.html#readRawData">readRawData</a>().</p></div>
    fn shr(
        self,
        str: ::cpp_utils::MutRef<*mut ::std::os::raw::c_char>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDataStream_operator__12(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                str.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<i8> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes a signed byte, <i>i</i>, to the stream and returns a reference to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::operator<<(qint8 i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes a signed byte, <i>i</i>, to the stream and returns a reference to the stream.</p></div>
    fn shl(self, i: i8) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDataStream_operator__13(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                i,
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<u8> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::operator<<(quint8 i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes an unsigned byte, <i>i</i>, to the stream and returns a reference to the stream.</p></div>
    fn shl(self, i: u8) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDataStream_operator__14(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                i,
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<i16> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::operator<<(qint16 i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#operator-lt-lt-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes a signed 16-bit integer, <i>i</i>, to the stream and returns a reference to the stream.</p></div>
    fn shl(self, i: i16) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDataStream_operator__15(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                i,
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<u16> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::operator<<(quint16 i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#operator-lt-lt-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes an unsigned 16-bit integer, <i>i</i>, to the stream and returns a reference to the stream.</p></div>
    fn shl(self, i: u16) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDataStream_operator__16(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                i,
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<i32> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::operator<<(qint32 i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#operator-lt-lt-4">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes a signed 32-bit integer, <i>i</i>, to the stream and returns a reference to the stream.</p></div>
    fn shl(self, i: i32) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDataStream_operator__17(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                i,
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<u32> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::operator<<(quint32 i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#operator-lt-lt-5">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes an unsigned integer, <i>i</i>, to the stream as a 32-bit unsigned integer (quint32). Returns a reference to the stream.</p></div>
    fn shl(self, i: u32) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDataStream_operator__18(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                i,
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<i64> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::operator<<(qint64 i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#operator-lt-lt-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes a signed 64-bit integer, <i>i</i>, to the stream and returns a reference to the stream.</p></div>
    fn shl(self, i: i64) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDataStream_operator__19(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                i,
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<u64> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::operator<<(quint64 i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#operator-lt-lt-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes an unsigned 64-bit integer, <i>i</i>, to the stream and returns a reference to the stream.</p></div>
    fn shl(self, i: u64) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDataStream_operator__20(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                i,
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<bool> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes a boolean value, <i>i</i>, to the stream. Returns a reference to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::operator<<(bool i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#operator-lt-lt-9">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes a boolean value, <i>i</i>, to the stream. Returns a reference to the stream.</p></div>
    fn shl(self, i: bool) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDataStream_operator__21(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                i,
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::Qfloat16>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::operator<<(qfloat16 f)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#operator-lt-lt-10">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes a floating point number, <i>f</i>, to the stream using the standard IEEE 754 format. Returns a reference to the stream.</p>
    /// <p>This function was introduced in  Qt 5.9.</p></div>
    fn shl(self, f: ::cpp_utils::Ref<crate::Qfloat16>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDataStream_operator__22(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                f.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::std::os::raw::c_float> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::operator<<(float f)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#operator-lt-lt-11">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes a floating point number, <i>f</i>, to the stream using the standard IEEE 754 format. Returns a reference to the stream.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatastream.html#setFloatingPointPrecision">setFloatingPointPrecision</a>().</p></div>
    fn shl(self, f: ::std::os::raw::c_float) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDataStream_operator__23(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                f,
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ptr<::std::os::raw::c_char>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& QDataStream::operator<<(const char* str)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatastream.html#operator-lt-lt-13">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes the '\0'-terminated string <i>s</i> to the stream and returns a reference to the stream.</p>
    /// <p>The string is serialized using <code>writeBytes()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatastream.html#writeBytes">writeBytes</a>() and <a href="http://doc.qt.io/qt-5/qdatastream.html#writeRawData">writeRawData</a>().</p></div>
    fn shl(
        self,
        str: ::cpp_utils::Ptr<::std::os::raw::c_char>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDataStream_operator__25(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                str.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QDate>> for crate::QDate {
    /// <p>Returns <code>true</code> if this date is equal to <i>d</i>; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDate::operator==(const QDate& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this date is equal to <i>d</i>; otherwise returns false.</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QDate>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDate_operator__(
                self as *const crate::QDate,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QDate>> for crate::QDate {
    /// <p>Returns <code>true</code> if this date is earlier than <i>d</i>; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDate::operator<(const QDate& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this date is earlier than <i>d</i>; otherwise returns false.</p></div>
    fn lt(&self, other: &::cpp_utils::Ref<crate::QDate>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDate_operator_(
                self as *const crate::QDate,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QDate>> for crate::QDate {
    /// <p>Returns <code>true</code> if this date is earlier than or equal to <i>d</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDate::operator<=(const QDate& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this date is earlier than or equal to <i>d</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, other: &::cpp_utils::Ref<crate::QDate>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDate_operator__2(
                self as *const crate::QDate,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QDate>> for crate::QDate {
    /// <p>Returns <code>true</code> if this date is later than <i>d</i>; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDate::operator>(const QDate& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this date is later than <i>d</i>; otherwise returns false.</p></div>
    fn gt(&self, other: &::cpp_utils::Ref<crate::QDate>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDate_operator_1(
                self as *const crate::QDate,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QDate>> for crate::QDate {
    /// <p>Returns <code>true</code> if this date is later than or equal to <i>d</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDate::operator>=(const QDate& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this date is later than or equal to <i>d</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, other: &::cpp_utils::Ref<crate::QDate>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDate_operator__3(
                self as *const crate::QDate,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QTime>> for crate::QTime {
    /// <p>Returns <code>true</code> if this time is equal to <i>t</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTime::operator==(const QTime& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this time is equal to <i>t</i>; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QTime>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QTime_operator__(
                self as *const crate::QTime,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QTime>> for crate::QTime {
    /// <p>Returns <code>true</code> if this time is earlier than <i>t</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTime::operator<(const QTime& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this time is earlier than <i>t</i>; otherwise returns <code>false</code>.</p></div>
    fn lt(&self, other: &::cpp_utils::Ref<crate::QTime>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QTime_operator_(
                self as *const crate::QTime,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QTime>> for crate::QTime {
    /// <p>Returns <code>true</code> if this time is earlier than or equal to <i>t</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTime::operator<=(const QTime& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this time is earlier than or equal to <i>t</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, other: &::cpp_utils::Ref<crate::QTime>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QTime_operator__2(
                self as *const crate::QTime,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QTime>> for crate::QTime {
    /// <p>Returns <code>true</code> if this time is later than <i>t</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTime::operator>(const QTime& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this time is later than <i>t</i>; otherwise returns <code>false</code>.</p></div>
    fn gt(&self, other: &::cpp_utils::Ref<crate::QTime>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QTime_operator_1(
                self as *const crate::QTime,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QTime>> for crate::QTime {
    /// <p>Returns <code>true</code> if this time is later than or equal to <i>t</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTime::operator>=(const QTime& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this time is later than or equal to <i>t</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, other: &::cpp_utils::Ref<crate::QTime>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QTime_operator__3(
                self as *const crate::QTime,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QDateTime {
    /// <p>Destroys the datetime.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QDateTime::~QDateTime()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#dtor.QDateTime">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the datetime.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QDateTime_dQDateTime(self as *mut crate::QDateTime)
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QDateTime>> for crate::QDateTime {
    /// <p>Returns <code>true</code> if this datetime is equal to the <i>other</i> datetime; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDateTime::operator==(const QDateTime& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this datetime is equal to the <i>other</i> datetime; otherwise returns <code>false</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdatetime.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QDateTime>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDateTime_operator__(
                self as *const crate::QDateTime,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QDateTime>> for crate::QDateTime {
    /// <p>Returns <code>true</code> if this datetime is earlier than the <i>other</i> datetime; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDateTime::operator<(const QDateTime& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this datetime is earlier than the <i>other</i> datetime; otherwise returns <code>false</code>.</p></div>
    fn lt(&self, other: &::cpp_utils::Ref<crate::QDateTime>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDateTime_operator_2(
                self as *const crate::QDateTime,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QDateTime>> for crate::QDateTime {
    /// <p>Returns <code>true</code> if this datetime is earlier than or equal to the <i>other</i> datetime; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDateTime::operator<=(const QDateTime& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#operator-lt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this datetime is earlier than or equal to the <i>other</i> datetime; otherwise returns <code>false</code>.</p></div>
    fn le(&self, other: &::cpp_utils::Ref<crate::QDateTime>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDateTime_operator__2(
                self as *const crate::QDateTime,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QDateTime>> for crate::QDateTime {
    /// <p>Returns <code>true</code> if this datetime is later than the <i>other</i> datetime; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDateTime::operator>(const QDateTime& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this datetime is later than the <i>other</i> datetime; otherwise returns <code>false</code>.</p></div>
    fn gt(&self, other: &::cpp_utils::Ref<crate::QDateTime>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDateTime_operator_3(
                self as *const crate::QDateTime,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QDateTime>> for crate::QDateTime {
    /// <p>Returns <code>true</code> if this datetime is later than or equal to the <i>other</i> datetime; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDateTime::operator>=(const QDateTime& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdatetime.html#operator-gt-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this datetime is later than or equal to the <i>other</i> datetime; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, other: &::cpp_utils::Ref<crate::QDateTime>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDateTime_operator__3(
                self as *const crate::QDateTime,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QDate>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, const QDate& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QDate>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__108(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QDate>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QDate& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QDate>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__109(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QTime>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, const QTime& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QTime>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__110(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QTime>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QTime& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QTime>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__111(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QDateTime>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, const QDateTime& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        arg2: ::cpp_utils::Ref<crate::QDateTime>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__112(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QDateTime>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QDateTime& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QDateTime>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__113(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QDate>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QDate& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QDate>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__114(
                self as *const crate::QDebug,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QTime>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QTime& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QTime>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__115(
                self as *const crate::QDebug,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QDateTime>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QDateTime& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QDateTime>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__116(
                self as *const crate::QDebug,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QElapsedTimer>> for crate::QElapsedTimer {
    /// <p>Returns <code>true</code> if this object and <i>other</i> contain the same time.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QElapsedTimer::operator==(const QElapsedTimer& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this object and <i>other</i> contain the same time.</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QElapsedTimer>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QElapsedTimer_operator__(
                self as *const crate::QElapsedTimer,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::AddAssign<i64> for crate::QDeadlineTimer {
    /// <p>Extends this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object by <i>msecs</i> milliseconds and returns itself. If this object is set to never expire, this function does nothing.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDeadlineTimer& QDeadlineTimer::operator+=(qint64 msecs)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Extends this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object by <i>msecs</i> milliseconds and returns itself. If this object is set to never expire, this function does nothing.</p>
    /// <p>To add times of precision greater than 1 millisecond, use <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#addNSecs">addNSecs</a>().</p></div>
    fn add_assign(&mut self, msecs: i64) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_operator__(
                self as *mut crate::QDeadlineTimer,
                msecs,
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::SubAssign<i64> for crate::QDeadlineTimer {
    /// <p>Shortens this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object by <i>msecs</i> milliseconds and returns itself. If this object is set to never expire, this function does nothing.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDeadlineTimer& QDeadlineTimer::operator-=(qint64 msecs)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Shortens this <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> object by <i>msecs</i> milliseconds and returns itself. If this object is set to never expire, this function does nothing.</p>
    /// <p>To subtract times of precision greater than 1 millisecond, use <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#addNSecs">addNSecs</a>().</p></div>
    fn sub_assign(&mut self, msecs: i64) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_operator__1(
                self as *mut crate::QDeadlineTimer,
                msecs,
            )
        };
        let _ = ffi_result;
    }
}

impl ::cpp_utils::CppDeletable for crate::QTextStream {
    /// <p>Destroys the <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QTextStream::~QTextStream()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#dtor.QTextStream">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>.</p>
    /// <p>If the stream operates on a device, <a href="http://doc.qt.io/qt-5/qtextstream.html#flush">flush</a>() will be called implicitly. Otherwise, the device is unaffected.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QTextStream_dQTextStream(self as *mut crate::QTextStream)
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QChar>> for &crate::QTextStream {
    type Output = ::cpp_utils::MutRef<crate::QTextStream>;
    /// <p>Reads a character from the stream and stores it in <i>c</i>. Returns a reference to the <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>, so several operators can be nested. Example:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator>>(QChar& ch)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-gt-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a character from the stream and stores it in <i>c</i>. Returns a reference to the <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>, so several operators can be nested. Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtextstream.html#QTextStream">QTextStream</a></span> in(file);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qchar.html">QChar</a></span> ch1<span class="operator">,</span> ch2<span class="operator">,</span> ch3;
    ///   in <span class="operator">&gt;</span><span class="operator">&gt;</span> ch1 <span class="operator">&gt;</span><span class="operator">&gt;</span> ch2 <span class="operator">&gt;</span><span class="operator">&gt;</span> ch3;
    ///
    /// </pre>
    /// <p>Whitespace is <i>not</i> skipped.</p></div>
    fn shr(self, ch: ::cpp_utils::MutRef<crate::QChar>) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__(
                self as *const crate::QTextStream as *mut crate::QTextStream,
                ch.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<::std::os::raw::c_char>> for &crate::QTextStream {
    type Output = ::cpp_utils::MutRef<crate::QTextStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator>>(char& ch)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-gt-gt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Reads a character from the stream and stores it in <i>c</i>. The character from the stream is converted to ISO-5589-1 before it is stored.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qchar.html#toLatin1">QChar::toLatin1</a>().</p></div>
    fn shr(
        self,
        ch: ::cpp_utils::MutRef<::std::os::raw::c_char>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__1(
                self as *const crate::QTextStream as *mut crate::QTextStream,
                ch.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<i64>> for &crate::QTextStream {
    type Output = ::cpp_utils::MutRef<crate::QTextStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator>>(qlonglong& i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-gt-gt-8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Stores the integer in the qlonglong <i>i</i>.</p></div>
    fn shr(self, i: ::cpp_utils::MutRef<i64>) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__8(
                self as *const crate::QTextStream as *mut crate::QTextStream,
                i.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<::std::os::raw::c_float>> for &crate::QTextStream {
    type Output = ::cpp_utils::MutRef<crate::QTextStream>;
    /// <p>Reads a real number from the stream and stores it in <i>f</i>, then returns a reference to the <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>. The number is cast to the correct type. If no real number is detect on the stream, <i>f</i> is set to 0.0.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator>>(float& f)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-gt-gt-10">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a real number from the stream and stores it in <i>f</i>, then returns a reference to the <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>. The number is cast to the correct type. If no real number is detect on the stream, <i>f</i> is set to 0.0.</p>
    /// <p>As a special exception, <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> allows the strings "nan" and "inf" to represent NAN and INF floats or doubles.</p>
    /// <p>Leading whitespace is skipped.</p></div>
    fn shr(
        self,
        f: ::cpp_utils::MutRef<::std::os::raw::c_float>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__10(
                self as *const crate::QTextStream as *mut crate::QTextStream,
                f.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QString>> for &crate::QTextStream {
    type Output = ::cpp_utils::MutRef<crate::QTextStream>;
    /// <p>Reads a word from the stream and stores it in <i>str</i>, then returns a reference to the stream. Words are separated by whitespace (i.e., all characters for which <a href="http://doc.qt.io/qt-5/qchar.html#isSpace">QChar::isSpace</a>() returns <code>true</code>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator>>(QString& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-gt-gt-12">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a word from the stream and stores it in <i>str</i>, then returns a reference to the stream. Words are separated by whitespace (i.e., all characters for which <a href="http://doc.qt.io/qt-5/qchar.html#isSpace">QChar::isSpace</a>() returns <code>true</code>).</p>
    /// <p>Leading whitespace is skipped.</p></div>
    fn shr(
        self,
        s: ::cpp_utils::MutRef<crate::QString>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__12(
                self as *const crate::QTextStream as *mut crate::QTextStream,
                s.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QByteArray>> for &crate::QTextStream {
    type Output = ::cpp_utils::MutRef<crate::QTextStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator>>(QByteArray& array)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-gt-gt-13">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Converts the word to ISO-8859-1, then stores it in <i>array</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstring.html#toLatin1">QString::toLatin1</a>().</p></div>
    fn shr(
        self,
        array: ::cpp_utils::MutRef<crate::QByteArray>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__13(
                self as *const crate::QTextStream as *mut crate::QTextStream,
                array.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutPtr<::std::os::raw::c_char>> for &crate::QTextStream {
    type Output = ::cpp_utils::MutRef<crate::QTextStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator>>(char* c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-gt-gt-14">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Stores the word in <i>c</i>, terminated by a '\0' character. If no word is available, only the '\0' character is stored.</p>
    /// <p>Warning: Although convenient, this operator is dangerous and must be used with care. <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> assumes that <i>c</i> points to a buffer with enough space to hold the word. If the buffer is too small, your application may crash.</p>
    /// <p>If possible, use the <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> operator instead.</p></div>
    fn shr(
        self,
        c: ::cpp_utils::MutPtr<::std::os::raw::c_char>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__14(
                self as *const crate::QTextStream as *mut crate::QTextStream,
                c.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QChar>> for &crate::QTextStream {
    type Output = ::cpp_utils::MutRef<crate::QTextStream>;
    /// <p>Writes the character <i>c</i> to the stream, then returns a reference to the <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator<<(QChar ch)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes the character <i>c</i> to the stream, then returns a reference to the <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#setFieldWidth">setFieldWidth</a>().</p></div>
    fn shl(self, ch: ::cpp_utils::Ref<crate::QChar>) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__15(
                self as *const crate::QTextStream as *mut crate::QTextStream,
                ch.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::std::os::raw::c_char> for &crate::QTextStream {
    type Output = ::cpp_utils::MutRef<crate::QTextStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator<<(char ch)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Converts <i>c</i> from ASCII to a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>, then writes it to the stream.</p></div>
    fn shl(self, ch: ::std::os::raw::c_char) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__16(
                self as *const crate::QTextStream as *mut crate::QTextStream,
                ch,
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<i64> for &crate::QTextStream {
    type Output = ::cpp_utils::MutRef<crate::QTextStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator<<(qlonglong i)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-lt-lt-8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes the qlonglong <i>i</i> to the stream.</p></div>
    fn shl(self, i: i64) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__23(
                self as *const crate::QTextStream as *mut crate::QTextStream,
                i,
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::std::os::raw::c_float> for &crate::QTextStream {
    type Output = ::cpp_utils::MutRef<crate::QTextStream>;
    /// <p>Writes the real number <i>f</i> to the stream, then returns a reference to the <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>. By default, <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> stores it using <a href="http://doc.qt.io/qt-5/qtextstream.html#RealNumberNotation-enum">SmartNotation</a>, with up to 6 digits of precision. You can change the textual representation <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> will use for real numbers by calling <a href="http://doc.qt.io/qt-5/qtextstream.html#setRealNumberNotation">setRealNumberNotation</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#setRealNumberPrecision">setRealNumberPrecision</a>() and <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">setNumberFlags</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator<<(float f)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-lt-lt-10">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes the real number <i>f</i> to the stream, then returns a reference to the <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>. By default, <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> stores it using <a href="http://doc.qt.io/qt-5/qtextstream.html#RealNumberNotation-enum">SmartNotation</a>, with up to 6 digits of precision. You can change the textual representation <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> will use for real numbers by calling <a href="http://doc.qt.io/qt-5/qtextstream.html#setRealNumberNotation">setRealNumberNotation</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#setRealNumberPrecision">setRealNumberPrecision</a>() and <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">setNumberFlags</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#setFieldWidth">setFieldWidth</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#setRealNumberNotation">setRealNumberNotation</a>(), <a href="http://doc.qt.io/qt-5/qtextstream.html#setRealNumberPrecision">setRealNumberPrecision</a>(), and <a href="http://doc.qt.io/qt-5/qtextstream.html#setNumberFlags">setNumberFlags</a>().</p></div>
    fn shl(self, f: ::std::os::raw::c_float) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__25(
                self as *const crate::QTextStream as *mut crate::QTextStream,
                f,
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QString>> for &crate::QTextStream {
    type Output = ::cpp_utils::MutRef<crate::QTextStream>;
    /// <p>Writes the string <i>string</i> to the stream, and returns a reference to the <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>. The string is first encoded using the assigned codec (the default codec is <a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForLocale">QTextCodec::codecForLocale</a>()) before it is written to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator<<(const QString& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-lt-lt-12">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes the string <i>string</i> to the stream, and returns a reference to the <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>. The string is first encoded using the assigned codec (the default codec is <a href="http://doc.qt.io/qt-5/qtextcodec.html#codecForLocale">QTextCodec::codecForLocale</a>()) before it is written to the stream.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtextstream.html#setFieldWidth">setFieldWidth</a>() and <a href="http://doc.qt.io/qt-5/qtextstream.html#setCodec">setCodec</a>().</p></div>
    fn shl(self, s: ::cpp_utils::Ref<crate::QString>) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__27(
                self as *const crate::QTextStream as *mut crate::QTextStream,
                s.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QLatin1String>> for &crate::QTextStream {
    type Output = ::cpp_utils::MutRef<crate::QTextStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator<<(QLatin1String s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-lt-lt-13">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes <i>string</i> to the stream, and returns a reference to the <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>.</p></div>
    fn shl(
        self,
        s: ::cpp_utils::Ref<crate::QLatin1String>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__28(
                self as *const crate::QTextStream as *mut crate::QTextStream,
                s.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QStringRef>> for &crate::QTextStream {
    type Output = ::cpp_utils::MutRef<crate::QTextStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator<<(const QStringRef& s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-lt-lt-14">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes <i>string</i> to the stream, and returns a reference to the <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>.</p>
    /// <p>This function was introduced in  Qt 5.6.</p></div>
    fn shl(
        self,
        s: ::cpp_utils::Ref<crate::QStringRef>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__29(
                self as *const crate::QTextStream as *mut crate::QTextStream,
                s.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QByteArray>> for &crate::QTextStream {
    type Output = ::cpp_utils::MutRef<crate::QTextStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator<<(const QByteArray& array)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-lt-lt-15">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes <i>array</i> to the stream. The contents of <i>array</i> are converted with <a href="http://doc.qt.io/qt-5/qstring.html#fromUtf8">QString::fromUtf8</a>().</p></div>
    fn shl(
        self,
        array: ::cpp_utils::Ref<crate::QByteArray>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__30(
                self as *const crate::QTextStream as *mut crate::QTextStream,
                array.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ptr<::std::os::raw::c_char>> for &crate::QTextStream {
    type Output = ::cpp_utils::MutRef<crate::QTextStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator<<(const char* c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-lt-lt-16">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes the constant string pointed to by <i>string</i> to the stream. <i>string</i> is assumed to be in ISO-8859-1 encoding. This operator is convenient when working with constant string data. Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtextstream.html#QTextStream">QTextStream</a></span> out(stdout);
    ///   out <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"Qt rocks!"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> endl;
    ///
    /// </pre>
    /// <p>Warning: <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a> assumes that <i>string</i> points to a string of text, terminated by a '\0' character. If there is no terminating '\0' character, your application may crash.</p></div>
    fn shl(
        self,
        c: ::cpp_utils::Ptr<::std::os::raw::c_char>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__31(
                self as *const crate::QTextStream as *mut crate::QTextStream,
                c.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ptr<::std::ffi::c_void>> for &crate::QTextStream {
    type Output = ::cpp_utils::MutRef<crate::QTextStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator<<(const void* ptr)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-lt-lt-17">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes <i>ptr</i> to the stream as a hexadecimal number with a base.</p></div>
    fn shl(
        self,
        ptr: ::cpp_utils::Ptr<::std::ffi::c_void>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__32(
                self as *const crate::QTextStream as *mut crate::QTextStream,
                ptr.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QTextStreamManipulator>> for &crate::QTextStream {
    type Output = ::cpp_utils::MutRef<crate::QTextStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& operator<<(QTextStream& s, QTextStreamManipulator m)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        m: ::cpp_utils::Ref<crate::QTextStreamManipulator>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__119(
                self as *const crate::QTextStream as *mut crate::QTextStream,
                m.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QDebug {
    /// <p>Flushes any pending data to be written and destroys the debug stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QDebug::~QDebug()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#dtor.QDebug">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Flushes any pending data to be written and destroys the debug stream.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QDebug_dQDebug(self as *mut crate::QDebug)
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QChar>> for &crate::QDebug {
    type Output = ::cpp_utils::MutRef<crate::QDebug>;
    /// <p>Writes the character, <i>t</i>, to the stream and returns a reference to the stream. Normally, <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> prints control characters and non-US-ASCII characters as their C escape sequences or their Unicode value (\u1234). To print non-printable characters without transformation, enable the <a href="http://doc.qt.io/qt-5/qdebug.html#noquote">noquote</a>() functionality, but note that some <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> backends may not be 8-bit clean and may not be able to represent <code>t</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::operator<<(QChar t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes the character, <i>t</i>, to the stream and returns a reference to the stream. Normally, <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> prints control characters and non-US-ASCII characters as their C escape sequences or their Unicode value (\u1234). To print non-printable characters without transformation, enable the <a href="http://doc.qt.io/qt-5/qdebug.html#noquote">noquote</a>() functionality, but note that some <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> backends may not be 8-bit clean and may not be able to represent <code>t</code>.</p></div>
    fn shl(self, t: ::cpp_utils::Ref<crate::QChar>) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDebug_operator__(
                self as *const crate::QDebug as *mut crate::QDebug,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<bool> for &crate::QDebug {
    type Output = ::cpp_utils::MutRef<crate::QDebug>;
    /// <p>Writes the boolean value, <i>t</i>, to the stream and returns a reference to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::operator<<(bool t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes the boolean value, <i>t</i>, to the stream and returns a reference to the stream.</p></div>
    fn shl(self, t: bool) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDebug_operator__1(
                self as *const crate::QDebug as *mut crate::QDebug,
                t,
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::std::os::raw::c_char> for &crate::QDebug {
    type Output = ::cpp_utils::MutRef<crate::QDebug>;
    /// <p>Writes the character, <i>t</i>, to the stream and returns a reference to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::operator<<(char t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#operator-lt-lt-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes the character, <i>t</i>, to the stream and returns a reference to the stream.</p></div>
    fn shl(self, t: ::std::os::raw::c_char) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDebug_operator__2(
                self as *const crate::QDebug as *mut crate::QDebug,
                t,
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<i64> for &crate::QDebug {
    type Output = ::cpp_utils::MutRef<crate::QDebug>;
    /// <p>Writes the signed 64-bit integer, <i>i</i>, to the stream and returns a reference to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::operator<<(qint64 t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#operator-lt-lt-11">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes the signed 64-bit integer, <i>i</i>, to the stream and returns a reference to the stream.</p></div>
    fn shl(self, t: i64) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDebug_operator__11(
                self as *const crate::QDebug as *mut crate::QDebug,
                t,
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::std::os::raw::c_float> for &crate::QDebug {
    type Output = ::cpp_utils::MutRef<crate::QDebug>;
    /// <p>Writes the 32-bit floating point number, <i>f</i>, to the stream and returns a reference to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::operator<<(float t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#operator-lt-lt-13">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes the 32-bit floating point number, <i>f</i>, to the stream and returns a reference to the stream.</p></div>
    fn shl(self, t: ::std::os::raw::c_float) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDebug_operator__13(
                self as *const crate::QDebug as *mut crate::QDebug,
                t,
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ptr<::std::os::raw::c_char>> for &crate::QDebug {
    type Output = ::cpp_utils::MutRef<crate::QDebug>;
    /// <p>Writes the '\0'-terminated string, <i>s</i>, to the stream and returns a reference to the stream. The string is never quoted nor transformed to the output, but note that some <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> backends might not be 8-bit clean.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::operator<<(const char* t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#operator-lt-lt-15">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes the '\0'-terminated string, <i>s</i>, to the stream and returns a reference to the stream. The string is never quoted nor transformed to the output, but note that some <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> backends might not be 8-bit clean.</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ptr<::std::os::raw::c_char>,
    ) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDebug_operator__15(
                self as *const crate::QDebug as *mut crate::QDebug,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QString>> for &crate::QDebug {
    type Output = ::cpp_utils::MutRef<crate::QDebug>;
    /// <p>Writes the string, <i>s</i>, to the stream and returns a reference to the stream. Normally, <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> prints the string inside quotes and transforms non-printable characters to their Unicode values (\u1234).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::operator<<(const QString& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#operator-lt-lt-16">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes the string, <i>s</i>, to the stream and returns a reference to the stream. Normally, <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> prints the string inside quotes and transforms non-printable characters to their Unicode values (\u1234).</p>
    /// <p>To print non-printable characters without transformation, enable the <a href="http://doc.qt.io/qt-5/qdebug.html#noquote">noquote</a>() functionality. Note that some <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> backends might not be 8-bit clean.</p>
    /// <p>Output examples:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> s;
    ///
    ///   s <span class="operator">=</span> <span class="string">"a"</span>;
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebugx">qDebug</a>()<span class="operator">.</span>noquote() <span class="operator">&lt;</span><span class="operator">&lt;</span> s;    <span class="comment">// prints: a</span>
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebugx">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> s;              <span class="comment">// prints: "a"</span>
    ///
    ///   s <span class="operator">=</span> <span class="string">"\"a\r\n\""</span>;
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebugx">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> s;              <span class="comment">// prints: "\"a\r\n\""</span>
    ///
    ///   s <span class="operator">=</span> <span class="string">"\033"</span>;                 <span class="comment">// escape character</span>
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebugx">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> s;              <span class="comment">// prints: "\u001B"</span>
    ///
    ///   s <span class="operator">=</span> <span class="string">"\u00AD"</span>;               <span class="comment">// SOFT HYPHEN</span>
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebugx">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> s;              <span class="comment">// prints: "\u00AD"</span>
    ///
    ///   s <span class="operator">=</span> <span class="string">"\u00E1"</span>;               <span class="comment">// LATIN SMALL LETTER A WITH ACUTE</span>
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebugx">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> s;              <span class="comment">// prints: "á"</span>
    ///
    ///   s <span class="operator">=</span> <span class="string">"a\u0301"</span>;              <span class="comment">// "a" followed by COMBINING ACUTE ACCENT</span>
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebugx">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> s;              <span class="comment">// prints: "á";</span>
    ///
    ///   s <span class="operator">=</span> <span class="string">"\u0430\u0301"</span>;         <span class="comment">// CYRILLIC SMALL LETTER A followed by COMBINING ACUTE ACCENT</span>
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebugx">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> s;              <span class="comment">// prints: "а́"</span>
    ///
    /// </pre></div>
    fn shl(self, t: ::cpp_utils::Ref<crate::QString>) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDebug_operator__16(
                self as *const crate::QDebug as *mut crate::QDebug,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QStringRef>> for &crate::QDebug {
    type Output = ::cpp_utils::MutRef<crate::QDebug>;
    /// <p>Writes the string, <i>s</i>, to the stream and returns a reference to the stream. Normally, <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> prints the string inside quotes and transforms non-printable characters to their Unicode values (\u1234).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::operator<<(const QStringRef& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#operator-lt-lt-17">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes the string, <i>s</i>, to the stream and returns a reference to the stream. Normally, <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> prints the string inside quotes and transforms non-printable characters to their Unicode values (\u1234).</p>
    /// <p>To print non-printable characters without transformation, enable the <a href="http://doc.qt.io/qt-5/qdebug.html#noquote">noquote</a>() functionality. Note that some <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> backends might not be 8-bit clean.</p>
    /// <p>See the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> overload for examples.</p></div>
    fn shl(self, t: ::cpp_utils::Ref<crate::QStringRef>) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDebug_operator__17(
                self as *const crate::QDebug as *mut crate::QDebug,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QLatin1String>> for &crate::QDebug {
    type Output = ::cpp_utils::MutRef<crate::QDebug>;
    /// <p>Writes the string, <i>s</i>, to the stream and returns a reference to the stream. Normally, <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> prints the string inside quotes and transforms non-printable characters to their Unicode values (\u1234).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::operator<<(QLatin1String t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#operator-lt-lt-18">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes the string, <i>s</i>, to the stream and returns a reference to the stream. Normally, <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> prints the string inside quotes and transforms non-printable characters to their Unicode values (\u1234).</p>
    /// <p>To print non-printable characters without transformation, enable the <a href="http://doc.qt.io/qt-5/qdebug.html#noquote">noquote</a>() functionality. Note that some <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> backends might not be 8-bit clean.</p>
    /// <p>See the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> overload for examples.</p></div>
    fn shl(self, t: ::cpp_utils::Ref<crate::QLatin1String>) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDebug_operator__18(
                self as *const crate::QDebug as *mut crate::QDebug,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QByteArray>> for &crate::QDebug {
    type Output = ::cpp_utils::MutRef<crate::QDebug>;
    /// <p>Writes the byte array, <i>b</i>, to the stream and returns a reference to the stream. Normally, <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> prints the array inside quotes and transforms control or non-US-ASCII characters to their C escape sequences (\xAB). This way, the output is always 7-bit clean and the string can be copied from the output and pasted back into C++ sources, if necessary.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::operator<<(const QByteArray& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#operator-lt-lt-19">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes the byte array, <i>b</i>, to the stream and returns a reference to the stream. Normally, <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> prints the array inside quotes and transforms control or non-US-ASCII characters to their C escape sequences (\xAB). This way, the output is always 7-bit clean and the string can be copied from the output and pasted back into C++ sources, if necessary.</p>
    /// <p>To print non-printable characters without transformation, enable the <a href="http://doc.qt.io/qt-5/qdebug.html#noquote">noquote</a>() functionality. Note that some <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> backends might not be 8-bit clean.</p>
    /// <p>Output examples:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span> ba;
    ///
    ///   ba <span class="operator">=</span> <span class="string">"a"</span>;
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebugx">qDebug</a>()<span class="operator">.</span>noquote() <span class="operator">&lt;</span><span class="operator">&lt;</span> ba;    <span class="comment">// prints: a</span>
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebugx">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> ba;              <span class="comment">// prints: "a"</span>
    ///
    ///   ba <span class="operator">=</span> <span class="string">"\"a\r\n\""</span>;
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebugx">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> ba;              <span class="comment">// prints: "\"a\r\n\""</span>
    ///
    ///   ba <span class="operator">=</span> <span class="string">"\033"</span>;                 <span class="comment">// escape character</span>
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebugx">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> ba;              <span class="comment">// prints: "\x1B"</span>
    ///
    ///   ba <span class="operator">=</span> <span class="string">"\xC3\xA1"</span>;
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebugx">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> ba;              <span class="comment">// prints: "\xC3\xA1"</span>
    ///
    ///   ba <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span>(<span class="string">"a\0b"</span><span class="operator">,</span> <span class="number">3</span>);
    ///   <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebugx">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> ba               <span class="comment">// prints: "\a\x00""b"</span>
    ///
    /// </pre>
    /// <p>Note how <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> needed to close and reopen the string in the way C and C++ languages concatenate string literals so that the letter 'b' is not interpreted as part of the previous hexadecimal escape sequence.</p></div>
    fn shl(self, t: ::cpp_utils::Ref<crate::QByteArray>) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDebug_operator__19(
                self as *const crate::QDebug as *mut crate::QDebug,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ptr<::std::ffi::c_void>> for &crate::QDebug {
    type Output = ::cpp_utils::MutRef<crate::QDebug>;
    /// <p>Writes a pointer, <i>p</i>, to the stream and returns a reference to the stream.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::operator<<(const void* t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#operator-lt-lt-20">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes a pointer, <i>p</i>, to the stream and returns a reference to the stream.</p></div>
    fn shl(self, t: ::cpp_utils::Ptr<::std::ffi::c_void>) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDebug_operator__20(
                self as *const crate::QDebug as *mut crate::QDebug,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QTextStreamManipulator>> for &crate::QDebug {
    type Output = ::cpp_utils::MutRef<crate::QDebug>;
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::operator<<(QTextStreamManipulator m)```</span>.
    fn shl(
        self,
        m: ::cpp_utils::Ref<crate::QTextStreamManipulator>,
    ) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDebug_operator__22(
                self as *const crate::QDebug as *mut crate::QDebug,
                m.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QDebugStateSaver {
    /// <p>Destroys a <a href="http://doc.qt.io/qt-5/qdebugstatesaver.html">QDebugStateSaver</a> instance, which restores the settings used when the <a href="http://doc.qt.io/qt-5/qdebugstatesaver.html">QDebugStateSaver</a> instance was created.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QDebugStateSaver::~QDebugStateSaver()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebugstatesaver.html#dtor.QDebugStateSaver">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys a <a href="http://doc.qt.io/qt-5/qdebugstatesaver.html">QDebugStateSaver</a> instance, which restores the settings used when the <a href="http://doc.qt.io/qt-5/qdebugstatesaver.html">QDebugStateSaver</a> instance was created.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qdebug.html#setAutoInsertSpaces">QDebug::setAutoInsertSpaces</a>() and <a href="http://doc.qt.io/qt-5/qdebug.html#autoInsertSpaces">QDebug::autoInsertSpaces</a>().</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QDebugStateSaver_dQDebugStateSaver(
            self as *mut crate::QDebugStateSaver,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QFileDevice {
    /// <p>Destroys the file device, closing it if necessary.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QFileDevice::~QFileDevice()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfiledevice.html#dtor.QFileDevice">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the file device, closing it if necessary.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QFileDevice_dQFileDevice(self as *mut crate::QFileDevice)
    }
}

impl ::cpp_utils::CppDeletable for crate::QFile {
    /// <p>Destroys the file object, closing it if necessary.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QFile::~QFile()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfile.html#dtor.QFile">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the file object, closing it if necessary.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QFile_dQFile(self as *mut crate::QFile)
    }
}

impl ::cpp_utils::CppDeletable for crate::QFileInfo {
    /// <p>Destroys the <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> and frees its resources.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QFileInfo::~QFileInfo()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#dtor.QFileInfo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> and frees its resources.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QFileInfo_dQFileInfo(self as *mut crate::QFileInfo)
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QFileInfo>> for crate::QFileInfo {
    /// <p>Returns <code>true</code> if this <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> object refers to a file in the same location as <i>fileinfo</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFileInfo::operator==(const QFileInfo& fileinfo) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileinfo.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> object refers to a file in the same location as <i>fileinfo</i>; otherwise returns <code>false</code>.</p>
    /// <p>Note that the result of comparing two empty <a href="http://doc.qt.io/qt-5/qfileinfo.html">QFileInfo</a> objects, containing no file references (file paths that do not exist or are empty), is undefined.</p>
    /// <p><b>Warning:</b> This will not compare two different symbolic links pointing to the same file.</p>
    /// <p><b>Warning:</b> Long and short file names that refer to the same file on Windows are treated as if they referred to different files.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfileinfo.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, fileinfo: &::cpp_utils::Ref<crate::QFileInfo>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QFileInfo_operator__(
                self as *const crate::QFileInfo,
                fileinfo.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QDir {
    /// <p>Destroys the <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> object frees up its resources. This has no effect on the underlying directory in the file system.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QDir::~QDir()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#dtor.QDir">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the <a href="http://doc.qt.io/qt-5/qdir.html">QDir</a> object frees up its resources. This has no effect on the underlying directory in the file system.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QDir_dQDir(self as *mut crate::QDir)
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QDir>> for crate::QDir {
    /// <p>Returns <code>true</code> if directory <i>dir</i> and this directory have the same path and their sort and filter settings are the same; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QDir::operator==(const QDir& dir) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdir.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if directory <i>dir</i> and this directory have the same path and their sort and filter settings are the same; otherwise returns <code>false</code>.</p>
    /// <p>Example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="comment">// The current directory is "/usr/local"</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdir.html#QDir">QDir</a></span> d1(<span class="string">"/usr/local/bin"</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qdir.html#QDir">QDir</a></span> d2(<span class="string">"bin"</span>);
    ///   <span class="keyword">if</span> (d1 <span class="operator">=</span><span class="operator">=</span> d2)
    /// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>(<span class="string">"They're the same"</span>);
    ///
    /// </pre></div>
    fn eq(&self, dir: &::cpp_utils::Ref<crate::QDir>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDir_operator__1(
                self as *const crate::QDir,
                dir.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QDir>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QDir& dir)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, dir: ::cpp_utils::Ref<crate::QDir>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__120(
                self as *const crate::QDebug,
                dir.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QDirIterator {
    /// <p>Destroys the <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QDirIterator::~QDirIterator()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdiriterator.html#dtor.QDirIterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the <a href="http://doc.qt.io/qt-5/qdiriterator.html">QDirIterator</a>.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QDirIterator_dQDirIterator(self as *mut crate::QDirIterator)
    }
}

impl ::cpp_utils::CppDeletable for crate::QEasingCurve {
    /// <p>Destructor.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QEasingCurve::~QEasingCurve()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeasingcurve.html#dtor.QEasingCurve">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destructor.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QEasingCurve_dQEasingCurve(self as *mut crate::QEasingCurve)
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QEasingCurve>> for crate::QEasingCurve {
    /// <p>Compare this easing curve with <i>other</i> and returns <code>true</code> if they are equal. It will also compare the properties of a curve.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QEasingCurve::operator==(const QEasingCurve& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeasingcurve.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compare this easing curve with <i>other</i> and returns <code>true</code> if they are equal. It will also compare the properties of a curve.</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QEasingCurve>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QEasingCurve_operator__(
                self as *const crate::QEasingCurve,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QEasingCurve>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QEasingCurve& item)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        item: ::cpp_utils::Ref<crate::QEasingCurve>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__121(
                self as *const crate::QDebug,
                item.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QEasingCurve>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, const QEasingCurve& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        arg2: ::cpp_utils::Ref<crate::QEasingCurve>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__122(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QEasingCurve>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QEasingCurve& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QEasingCurve>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__123(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QEventTransition {
    /// <p>Destroys this <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> event transition.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QEventTransition::~QEventTransition()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qeventtransition.html#dtor.QEventTransition">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys this <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> event transition.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QEventTransition_dQEventTransition(
            self as *mut crate::QEventTransition,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QUnhandledException {
    /// <p>Destroys the instance of QUnhandledException. The destructor is virtual.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QUnhandledException::~QUnhandledException()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qunhandledexception.html#dtor.QUnhandledException">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the instance of QUnhandledException. The destructor is virtual.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QUnhandledException_dQUnhandledException(
            self as *mut crate::QUnhandledException,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QFactoryInterface {
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QFactoryInterface::~QFactoryInterface()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QFactoryInterface_dQFactoryInterface(
            self as *mut crate::QFactoryInterface,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QFileSelector {
    /// <p>Destroys this selector instance.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QFileSelector::~QFileSelector()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfileselector.html#dtor.QFileSelector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys this selector instance.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QFileSelector_dQFileSelector(
            self as *mut crate::QFileSelector,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QFileSystemWatcher {
    /// <p>Destroys the file system watcher.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QFileSystemWatcher::~QFileSystemWatcher()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfilesystemwatcher.html#dtor.QFileSystemWatcher">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the file system watcher.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QFileSystemWatcher_dQFileSystemWatcher(
            self as *mut crate::QFileSystemWatcher,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QFinalState {
    /// <p>Destroys this final state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QFinalState::~QFinalState()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfinalstate.html#dtor.QFinalState">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys this final state.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QFinalState_dQFinalState(self as *mut crate::QFinalState)
    }
}

impl ::cpp_utils::CppDeletable for crate::QRunnable {
    /// <p><a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a> virtual destructor.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QRunnable::~QRunnable()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrunnable.html#dtor.QRunnable">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p><a href="http://doc.qt.io/qt-5/qrunnable.html">QRunnable</a> virtual destructor.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QRunnable_dQRunnable(self as *mut crate::QRunnable)
    }
}

impl ::cpp_utils::CppDeletable for crate::QMutex {
    /// <p>Destroys the mutex.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QMutex::~QMutex()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmutex.html#dtor.QMutex">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the mutex.</p>
    /// <p><b>Warning:</b> Destroying a locked mutex may result in undefined behavior.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMutex_dQMutex(self as *mut crate::QMutex)
    }
}

impl ::cpp_utils::CppDeletable for crate::QMutexLocker {
    /// <p>Destroys the <a href="http://doc.qt.io/qt-5/qmutexlocker.html">QMutexLocker</a> and unlocks the mutex that was locked in the constructor.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QMutexLocker::~QMutexLocker()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmutexlocker.html#dtor.QMutexLocker">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the <a href="http://doc.qt.io/qt-5/qmutexlocker.html">QMutexLocker</a> and unlocks the mutex that was locked in the constructor.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmutex.html#unlock">QMutex::unlock</a>().</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMutexLocker_dQMutexLocker(self as *mut crate::QMutexLocker)
    }
}

impl ::cpp_utils::CppDeletable for crate::QHistoryState {
    /// <p>Destroys this history state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QHistoryState::~QHistoryState()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhistorystate.html#dtor.QHistoryState">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys this history state.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QHistoryState_dQHistoryState(
            self as *mut crate::QHistoryState,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QIdentityProxyModel {
    /// <p>Destroys this identity model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QIdentityProxyModel::~QIdentityProxyModel()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qidentityproxymodel.html#dtor.QIdentityProxyModel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys this identity model.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QIdentityProxyModel_dQIdentityProxyModel(
            self as *mut crate::QIdentityProxyModel,
        )
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QItemSelectionRange>>
    for crate::QItemSelectionRange
{
    /// <p>Returns <code>true</code> if the selection range is exactly the same as the <i>other</i> range given; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QItemSelectionRange::operator==(const QItemSelectionRange& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the selection range is exactly the same as the <i>other</i> range given; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QItemSelectionRange>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QItemSelectionRange_operator__(
                self as *const crate::QItemSelectionRange,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QItemSelectionRange>>
    for crate::QItemSelectionRange
{
    /// <p>Returns <code>true</code> if the selection range is less than the <i>other</i> range given; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QItemSelectionRange::operator<(const QItemSelectionRange& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the selection range is less than the <i>other</i> range given; otherwise returns <code>false</code>.</p>
    /// <p>The less than calculation is not directly useful to developers - the way that ranges with different parents compare is not defined. This operator only exists so that the class can be used with <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>.</p></div>
    fn lt(&self, other: &::cpp_utils::Ref<crate::QItemSelectionRange>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QItemSelectionRange_operator_2(
                self as *const crate::QItemSelectionRange,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QItemSelectionModel {
    /// <p>Destroys the selection model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QItemSelectionModel::~QItemSelectionModel()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#dtor.QItemSelectionModel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the selection model.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionModel_dQItemSelectionModel(
            self as *mut crate::QItemSelectionModel,
        )
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QItemSelectionRange>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QItemSelectionRange& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        arg2: ::cpp_utils::Ref<crate::QItemSelectionRange>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__124(
                self as *const crate::QDebug,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QJsonValue {
    /// <p>Destroys the value.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QJsonValue::~QJsonValue()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#dtor.QJsonValue">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the value.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QJsonValue_dQJsonValue(self as *mut crate::QJsonValue)
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QJsonValue>> for crate::QJsonValue {
    /// <p>Returns <code>true</code> if the value is equal to <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QJsonValue::operator==(const QJsonValue& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonvalue.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the value is equal to <i>other</i>.</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QJsonValue>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QJsonValue_operator__(
                self as *const crate::QJsonValue,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QJsonValue>> for crate::QJsonValueRef {
    /// Calls C++ function: <span style='color: green;'>```bool QJsonValueRef::operator==(const QJsonValue& other) const```</span>.
    fn eq(&self, other: &::cpp_utils::Ref<crate::QJsonValue>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QJsonValueRef_operator__(
                self as *const crate::QJsonValueRef,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QJsonValue>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QJsonValue& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QJsonValue>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__125(
                self as *const crate::QDebug,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QJsonArray {
    /// <p>Deletes the array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QJsonArray::~QJsonArray()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#dtor.QJsonArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Deletes the array.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QJsonArray_dQJsonArray(self as *mut crate::QJsonArray)
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QJsonArray>> for crate::QJsonArray {
    /// <p>Returns <code>true</code> if this array is equal to <i>other</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QJsonArray::operator==(const QJsonArray& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this array is equal to <i>other</i>.</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QJsonArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QJsonArray_operator__2(
                self as *const crate::QJsonArray,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QJsonArray {
    type Output = ::cpp_utils::CppBox<crate::q_json_array::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonArray::iterator QJsonArray::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qjsonarray.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_json_array::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QJsonArray_begin(self as *mut crate::QJsonArray)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QJsonArray {
    type Output = ::cpp_utils::CppBox<crate::q_json_array::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonArray::const_iterator QJsonArray::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_json_array::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QJsonArray_begin1(self as *const crate::QJsonArray)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QJsonArray {
    type Output = ::cpp_utils::CppBox<crate::q_json_array::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonArray::iterator QJsonArray::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qjsonarray.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_json_array::Iterator> {
        let ffi_result =
            unsafe { crate::__ffi::ctr_qt_core_ffi_QJsonArray_end(self as *mut crate::QJsonArray) };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QJsonArray {
    type Output = ::cpp_utils::CppBox<crate::q_json_array::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonArray::const_iterator QJsonArray::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_json_array::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QJsonArray_end1(self as *const crate::QJsonArray)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QJsonValue>> for &crate::QJsonArray {
    type Output = ::cpp_utils::CppBox<crate::QJsonArray>;
    /// <p>Returns an array that contains all the items in this array followed by the provided <i>value</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonArray QJsonArray::operator+(const QJsonValue& v) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an array that contains all the items in this array followed by the provided <i>value</i>.</p>
    /// <p>This function was introduced in  Qt 5.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(self, v: ::cpp_utils::Ref<crate::QJsonValue>) -> ::cpp_utils::CppBox<crate::QJsonArray> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QJsonArray_operator_1(
                self as *const crate::QJsonArray,
                v.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QJsonValue>> for crate::QJsonArray {
    /// <p>Appends <i>value</i> to the array, and returns a reference to the array itself.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonArray& QJsonArray::operator+=(const QJsonValue& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>value</i> to the array, and returns a reference to the array itself.</p>
    /// <p>This function was introduced in  Qt 5.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qjsonarray.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, v: ::cpp_utils::Ref<crate::QJsonValue>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QJsonArray_operator__4(
                self as *mut crate::QJsonArray,
                v.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QJsonValue>> for &crate::QJsonArray {
    type Output = ::cpp_utils::MutRef<crate::QJsonArray>;
    /// <p>Appends <i>value</i> to the array, and returns a reference to the array itself.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonArray& QJsonArray::operator<<(const QJsonValue& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonarray.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>value</i> to the array, and returns a reference to the array itself.</p>
    /// <p>This function was introduced in  Qt 5.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonarray.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qjsonarray.html#append">append</a>().</p></div>
    fn shl(self, v: ::cpp_utils::Ref<crate::QJsonValue>) -> ::cpp_utils::MutRef<crate::QJsonArray> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QJsonArray_operator__5(
                self as *const crate::QJsonArray as *mut crate::QJsonArray,
                v.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QJsonArray>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QJsonArray& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QJsonArray>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__126(
                self as *const crate::QDebug,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QJsonDocument {
    /// <p>Deletes the document.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QJsonDocument::~QJsonDocument()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#dtor.QJsonDocument">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Deletes the document.</p>
    /// <p>Binary data set with <a href="http://doc.qt.io/qt-5/qjsondocument.html#fromRawData">fromRawData</a> is not freed.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QJsonDocument_dQJsonDocument(
            self as *mut crate::QJsonDocument,
        )
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QJsonDocument>> for crate::QJsonDocument {
    /// <p>Returns <code>true</code> if the <i>other</i> document is equal to this document.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QJsonDocument::operator==(const QJsonDocument& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsondocument.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the <i>other</i> document is equal to this document.</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QJsonDocument>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QJsonDocument_operator__(
                self as *const crate::QJsonDocument,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QJsonDocument>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QJsonDocument& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        arg2: ::cpp_utils::Ref<crate::QJsonDocument>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__127(
                self as *const crate::QDebug,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QJsonObject {
    /// <p>Destroys the object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QJsonObject::~QJsonObject()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#dtor.QJsonObject">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the object.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QJsonObject_dQJsonObject(self as *mut crate::QJsonObject)
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QJsonObject>> for crate::QJsonObject {
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QJsonObject::operator==(const QJsonObject& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this object.</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QJsonObject>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QJsonObject_operator__4(
                self as *const crate::QJsonObject,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QJsonObject {
    type Output = ::cpp_utils::CppBox<crate::q_json_object::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonObject::iterator QJsonObject::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the object.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonobject.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qjsonobject.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_json_object::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QJsonObject_begin(self as *mut crate::QJsonObject)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QJsonObject {
    type Output = ::cpp_utils::CppBox<crate::q_json_object::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonObject::const_iterator QJsonObject::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_json_object::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QJsonObject_begin1(self as *const crate::QJsonObject)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QJsonObject {
    type Output = ::cpp_utils::CppBox<crate::q_json_object::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonObject::iterator QJsonObject::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the object.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qjsonobject.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qjsonobject.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_json_object::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QJsonObject_end(self as *mut crate::QJsonObject)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QJsonObject {
    type Output = ::cpp_utils::CppBox<crate::q_json_object::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QJsonObject::const_iterator QJsonObject::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonobject.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_json_object::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QJsonObject_end1(self as *const crate::QJsonObject)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QJsonObject>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QJsonObject& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QJsonObject>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__128(
                self as *const crate::QDebug,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QLibrary {
    /// <p>Destroys the <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QLibrary::~QLibrary()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibrary.html#dtor.QLibrary">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the <a href="http://doc.qt.io/qt-5/qlibrary.html">QLibrary</a> object.</p>
    /// <p>Unless <a href="http://doc.qt.io/qt-5/qlibrary.html#unload">unload</a>() was called explicitly, the library stays in memory until the application terminates.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlibrary.html#isLoaded">isLoaded</a>() and <a href="http://doc.qt.io/qt-5/qlibrary.html#unload">unload</a>().</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QLibrary_dQLibrary(self as *mut crate::QLibrary)
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVersionNumber>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& out, const QVersionNumber& version)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        version: ::cpp_utils::Ref<crate::QVersionNumber>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__129(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                version.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QVersionNumber>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& in, QVersionNumber& version)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        version: ::cpp_utils::MutRef<crate::QVersionNumber>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__130(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                version.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVersionNumber>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QVersionNumber& version)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        version: ::cpp_utils::Ref<crate::QVersionNumber>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__131(
                self as *const crate::QDebug,
                version.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QVersionNumber>> for crate::QVersionNumber {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QVersionNumber& lhs, const QVersionNumber& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QVersionNumber>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_68(
                self as *const crate::QVersionNumber,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QVersionNumber>> for crate::QVersionNumber {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QVersionNumber& lhs, const QVersionNumber& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QVersionNumber>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__132(
                self as *const crate::QVersionNumber,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QVersionNumber>> for crate::QVersionNumber {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QVersionNumber& lhs, const QVersionNumber& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QVersionNumber>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_69(
                self as *const crate::QVersionNumber,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QVersionNumber>> for crate::QVersionNumber {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QVersionNumber& lhs, const QVersionNumber& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QVersionNumber>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__133(
                self as *const crate::QVersionNumber,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QVersionNumber>> for crate::QVersionNumber {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(const QVersionNumber& lhs, const QVersionNumber& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, rhs: &::cpp_utils::Ref<crate::QVersionNumber>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__134(
                self as *const crate::QVersionNumber,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QPoint>> for crate::QPoint {
    /// <p>Adds the given <i>point</i> to this point and returns a reference to this point. For example:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPoint& QPoint::operator+=(const QPoint& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpoint.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds the given <i>point</i> to this point and returns a reference to this point. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html#QPoint">QPoint</a></span> p( <span class="number">3</span><span class="operator">,</span> <span class="number">7</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html#QPoint">QPoint</a></span> q(<span class="operator">-</span><span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   p <span class="operator">+</span><span class="operator">=</span> q;    <span class="comment">// p becomes (2, 11)</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpoint.html#operator--eq">operator-=</a>().</p></div>
    fn add_assign(&mut self, p: ::cpp_utils::Ref<crate::QPoint>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QPoint_operator__(
                self as *mut crate::QPoint,
                p.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::SubAssign<::cpp_utils::Ref<crate::QPoint>> for crate::QPoint {
    /// <p>Subtracts the given <i>point</i> from this point and returns a reference to this point. For example:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPoint& QPoint::operator-=(const QPoint& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpoint.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Subtracts the given <i>point</i> from this point and returns a reference to this point. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html#QPoint">QPoint</a></span> p( <span class="number">3</span><span class="operator">,</span> <span class="number">7</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html#QPoint">QPoint</a></span> q(<span class="operator">-</span><span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   p <span class="operator">-</span><span class="operator">=</span> q;    <span class="comment">// p becomes (4, 3)</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpoint.html#operator-2b-eq">operator+=</a>().</p></div>
    fn sub_assign(&mut self, p: ::cpp_utils::Ref<crate::QPoint>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QPoint_operator__1(
                self as *mut crate::QPoint,
                p.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::MulAssign<::std::os::raw::c_float> for crate::QPoint {
    /// <p>Multiplies this point's coordinates by the given <i>factor</i>, and returns a reference to this point.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPoint& QPoint::operator*=(float factor)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpoint.html#operator-2a-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Multiplies this point's coordinates by the given <i>factor</i>, and returns a reference to this point.</p>
    /// <p>Note that the result is rounded to the nearest integer as points are held as integers. Use <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> for floating point accuracy.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpoint.html#operator-2f-eq">operator/=</a>().</p></div>
    fn mul_assign(&mut self, factor: ::std::os::raw::c_float) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QPoint_operator__2(self as *mut crate::QPoint, factor)
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::MulAssign<::std::os::raw::c_int> for crate::QPoint {
    /// <p>Multiplies this point's coordinates by the given <i>factor</i>, and returns a reference to this point.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPoint& QPoint::operator*=(int factor)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpoint.html#operator-2a-eq-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Multiplies this point's coordinates by the given <i>factor</i>, and returns a reference to this point.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpoint.html#operator-2f-eq">operator/=</a>().</p></div>
    fn mul_assign(&mut self, factor: ::std::os::raw::c_int) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QPoint_operator__4(self as *mut crate::QPoint, factor)
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::DivAssign<::std::os::raw::c_double> for crate::QPoint {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPoint& QPoint::operator/=(double divisor)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpoint.html#operator-2f-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Divides both x and y by the given <i>divisor</i>, and returns a reference to this point. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html#QPoint">QPoint</a></span> p(<span class="operator">-</span><span class="number">3</span><span class="operator">,</span> <span class="number">10</span>);
    ///   p <span class="operator">/</span><span class="operator">=</span> <span class="number">2.5</span>;           <span class="comment">// p becomes (-1, 4)</span>
    ///
    /// </pre>
    /// <p>Note that the result is rounded to the nearest integer as points are held as integers. Use <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> for floating point accuracy.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpoint.html#operator-2a-eq">operator*=</a>().</p></div>
    fn div_assign(&mut self, divisor: ::std::os::raw::c_double) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QPoint_operator__5(self as *mut crate::QPoint, divisor)
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QPoint>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, const QPoint& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QPoint>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__136(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QPoint>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QPoint& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QPoint>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__137(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QPoint>> for crate::QPoint {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(const QPoint& p1, const QPoint& p2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, p2: &::cpp_utils::Ref<crate::QPoint>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__138(
                self as *const crate::QPoint,
                p2.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QPoint>> for &crate::QPoint {
    type Output = ::cpp_utils::CppBox<crate::QPoint>;
    /// Calls C++ function: <span style='color: green;'>```QPoint operator+(const QPoint& p1, const QPoint& p2)```</span>.
    fn add(self, p2: ::cpp_utils::Ref<crate::QPoint>) -> ::cpp_utils::CppBox<crate::QPoint> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_70(self as *const crate::QPoint, p2.as_raw_ptr())
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Sub<::cpp_utils::Ref<crate::QPoint>> for &crate::QPoint {
    type Output = ::cpp_utils::CppBox<crate::QPoint>;
    /// Calls C++ function: <span style='color: green;'>```QPoint operator-(const QPoint& p1, const QPoint& p2)```</span>.
    fn sub(self, p2: ::cpp_utils::Ref<crate::QPoint>) -> ::cpp_utils::CppBox<crate::QPoint> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_71(self as *const crate::QPoint, p2.as_raw_ptr())
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Mul<::std::os::raw::c_float> for &crate::QPoint {
    type Output = ::cpp_utils::CppBox<crate::QPoint>;
    /// Calls C++ function: <span style='color: green;'>```QPoint operator*(const QPoint& p, float factor)```</span>.
    fn mul(self, factor: ::std::os::raw::c_float) -> ::cpp_utils::CppBox<crate::QPoint> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_72(self as *const crate::QPoint, factor)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Mul<::std::os::raw::c_int> for &crate::QPoint {
    type Output = ::cpp_utils::CppBox<crate::QPoint>;
    /// Calls C++ function: <span style='color: green;'>```QPoint operator*(const QPoint& p, int factor)```</span>.
    fn mul(self, factor: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QPoint> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_74(self as *const crate::QPoint, factor)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Neg for &crate::QPoint {
    type Output = ::cpp_utils::CppBox<crate::QPoint>;
    /// Calls C++ function: <span style='color: green;'>```QPoint operator-(const QPoint& p)```</span>.
    fn neg(self) -> ::cpp_utils::CppBox<crate::QPoint> {
        let ffi_result =
            unsafe { crate::__ffi::ctr_qt_core_ffi_operator_79(self as *const crate::QPoint) };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Div<::std::os::raw::c_double> for &crate::QPoint {
    type Output = ::cpp_utils::CppBox<crate::QPoint>;
    /// Calls C++ function: <span style='color: green;'>```QPoint operator/(const QPoint& p, double c)```</span>.
    fn div(self, c: ::std::os::raw::c_double) -> ::cpp_utils::CppBox<crate::QPoint> {
        let ffi_result =
            unsafe { crate::__ffi::ctr_qt_core_ffi_operator_80(self as *const crate::QPoint, c) };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QPoint>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QPoint& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QPoint>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__140(
                self as *const crate::QDebug,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QPointF>> for crate::QPointF {
    /// <p>Adds the given <i>point</i> to this point and returns a reference to this point. For example:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPointF& QPointF::operator+=(const QPointF& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpointf.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds the given <i>point</i> to this point and returns a reference to this point. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpointf.html#QPointF">QPointF</a></span> p( <span class="number">3.1</span><span class="operator">,</span> <span class="number">7.1</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpointf.html#QPointF">QPointF</a></span> q(<span class="operator">-</span><span class="number">1.0</span><span class="operator">,</span> <span class="number">4.1</span>);
    ///   p <span class="operator">+</span><span class="operator">=</span> q;    <span class="comment">// p becomes (2.1, 11.2)</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpointf.html#operator--eq">operator-=</a>().</p></div>
    fn add_assign(&mut self, p: ::cpp_utils::Ref<crate::QPointF>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QPointF_operator__(
                self as *mut crate::QPointF,
                p.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::SubAssign<::cpp_utils::Ref<crate::QPointF>> for crate::QPointF {
    /// <p>Subtracts the given <i>point</i> from this point and returns a reference to this point. For example:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPointF& QPointF::operator-=(const QPointF& p)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpointf.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Subtracts the given <i>point</i> from this point and returns a reference to this point. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpointf.html#QPointF">QPointF</a></span> p( <span class="number">3.1</span><span class="operator">,</span> <span class="number">7.1</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpointf.html#QPointF">QPointF</a></span> q(<span class="operator">-</span><span class="number">1.0</span><span class="operator">,</span> <span class="number">4.1</span>);
    ///   p <span class="operator">-</span><span class="operator">=</span> q;    <span class="comment">// p becomes (4.1, 3.0)</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpointf.html#operator-2b-eq">operator+=</a>().</p></div>
    fn sub_assign(&mut self, p: ::cpp_utils::Ref<crate::QPointF>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QPointF_operator__1(
                self as *mut crate::QPointF,
                p.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::MulAssign<::std::os::raw::c_double> for crate::QPointF {
    /// <p>Multiplies this point's coordinates by the given <i>factor</i>, and returns a reference to this point. For example:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPointF& QPointF::operator*=(double c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpointf.html#operator-2a-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Multiplies this point's coordinates by the given <i>factor</i>, and returns a reference to this point. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpointf.html#QPointF">QPointF</a></span> p(<span class="operator">-</span><span class="number">1.1</span><span class="operator">,</span> <span class="number">4.1</span>);
    ///   p <span class="operator">*</span><span class="operator">=</span> <span class="number">2.5</span>;    <span class="comment">// p becomes (-2.75, 10.25)</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpointf.html#operator-2f-eq">operator/=</a>().</p></div>
    fn mul_assign(&mut self, c: ::std::os::raw::c_double) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QPointF_operator__2(self as *mut crate::QPointF, c)
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::DivAssign<::std::os::raw::c_double> for crate::QPointF {
    /// <p>Divides both x and y by the given <i>divisor</i>, and returns a reference to this point. For example:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QPointF& QPointF::operator/=(double c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpointf.html#operator-2f-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Divides both x and y by the given <i>divisor</i>, and returns a reference to this point. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpointf.html#QPointF">QPointF</a></span> p(<span class="operator">-</span><span class="number">2.75</span><span class="operator">,</span> <span class="number">10.25</span>);
    ///   p <span class="operator">/</span><span class="operator">=</span> <span class="number">2.5</span>;           <span class="comment">// p becomes (-1.1, 4.1)</span>
    ///
    /// </pre>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpointf.html#operator-2a-eq">operator*=</a>().</p></div>
    fn div_assign(&mut self, c: ::std::os::raw::c_double) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QPointF_operator__3(self as *mut crate::QPointF, c)
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QPointF>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, const QPointF& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        arg2: ::cpp_utils::Ref<crate::QPointF>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__141(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QPointF>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QPointF& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QPointF>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__142(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QPointF>> for crate::QPointF {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(const QPointF& p1, const QPointF& p2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, p2: &::cpp_utils::Ref<crate::QPointF>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__143(
                self as *const crate::QPointF,
                p2.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QPointF>> for &crate::QPointF {
    type Output = ::cpp_utils::CppBox<crate::QPointF>;
    /// Calls C++ function: <span style='color: green;'>```QPointF operator+(const QPointF& p1, const QPointF& p2)```</span>.
    fn add(self, p2: ::cpp_utils::Ref<crate::QPointF>) -> ::cpp_utils::CppBox<crate::QPointF> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_81(
                self as *const crate::QPointF,
                p2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Sub<::cpp_utils::Ref<crate::QPointF>> for &crate::QPointF {
    type Output = ::cpp_utils::CppBox<crate::QPointF>;
    /// Calls C++ function: <span style='color: green;'>```QPointF operator-(const QPointF& p1, const QPointF& p2)```</span>.
    fn sub(self, p2: ::cpp_utils::Ref<crate::QPointF>) -> ::cpp_utils::CppBox<crate::QPointF> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_82(
                self as *const crate::QPointF,
                p2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Mul<::std::os::raw::c_double> for &crate::QPointF {
    type Output = ::cpp_utils::CppBox<crate::QPointF>;
    /// Calls C++ function: <span style='color: green;'>```QPointF operator*(const QPointF& p, double c)```</span>.
    fn mul(self, c: ::std::os::raw::c_double) -> ::cpp_utils::CppBox<crate::QPointF> {
        let ffi_result =
            unsafe { crate::__ffi::ctr_qt_core_ffi_operator_83(self as *const crate::QPointF, c) };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Neg for &crate::QPointF {
    type Output = ::cpp_utils::CppBox<crate::QPointF>;
    /// Calls C++ function: <span style='color: green;'>```QPointF operator-(const QPointF& p)```</span>.
    fn neg(self) -> ::cpp_utils::CppBox<crate::QPointF> {
        let ffi_result =
            unsafe { crate::__ffi::ctr_qt_core_ffi_operator_86(self as *const crate::QPointF) };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Div<::std::os::raw::c_double> for &crate::QPointF {
    type Output = ::cpp_utils::CppBox<crate::QPointF>;
    /// Calls C++ function: <span style='color: green;'>```QPointF operator/(const QPointF& p, double divisor)```</span>.
    fn div(self, divisor: ::std::os::raw::c_double) -> ::cpp_utils::CppBox<crate::QPointF> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_87(self as *const crate::QPointF, divisor)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QPointF>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug d, const QPointF& p)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, p: ::cpp_utils::Ref<crate::QPointF>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__145(
                self as *const crate::QDebug,
                p.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QLine>> for crate::QLine {
    /// <p>Returns <code>true</code> if the given <i>line</i> is the same as <i>this</i> line.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLine::operator==(const QLine& d) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qline.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the given <i>line</i> is the same as <i>this</i> line.</p>
    /// <p>A line is identical to another line if the start and end points are identical, and the internal order of the points is the same.</p></div>
    fn eq(&self, d: &::cpp_utils::Ref<crate::QLine>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QLine_operator__(
                self as *const crate::QLine,
                d.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QLine>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug d, const QLine& p)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, p: ::cpp_utils::Ref<crate::QLine>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__146(
                self as *const crate::QDebug,
                p.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QLine>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, const QLine& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QLine>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__147(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QLine>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QLine& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QLine>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__148(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QLineF>> for crate::QLineF {
    /// <p>Returns <code>true</code> if the given <i>line</i> is the same as this line.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QLineF::operator==(const QLineF& d) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the given <i>line</i> is the same as this line.</p>
    /// <p>A line is identical to another line if the start and end points are identical, and the internal order of the points is the same.</p></div>
    fn eq(&self, d: &::cpp_utils::Ref<crate::QLineF>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QLineF_operator__(
                self as *const crate::QLineF,
                d.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QLineF>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug d, const QLineF& p)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, p: ::cpp_utils::Ref<crate::QLineF>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__149(
                self as *const crate::QDebug,
                p.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QLineF>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, const QLineF& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QLineF>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__150(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QLineF>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QLineF& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QLineF>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__151(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QLockFile {
    /// <p>Destroys the lock file object. If the lock was acquired, this will release the lock, by deleting the lock file.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QLockFile::~QLockFile()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlockfile.html#dtor.QLockFile">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the lock file object. If the lock was acquired, this will release the lock, by deleting the lock file.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QLockFile_dQLockFile(self as *mut crate::QLockFile)
    }
}

impl ::cpp_utils::CppDeletable for crate::QLoggingCategory {
    /// <p>Destructs a <a href="http://doc.qt.io/qt-5/qloggingcategory.html">QLoggingCategory</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QLoggingCategory::~QLoggingCategory()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qloggingcategory.html#dtor.QLoggingCategory">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destructs a <a href="http://doc.qt.io/qt-5/qloggingcategory.html">QLoggingCategory</a> object.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QLoggingCategory_dQLoggingCategory(
            self as *mut crate::QLoggingCategory,
        )
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QMargins>> for crate::QMargins {
    /// <p>Add each component of <i>margins</i> to the respective component of this object and returns a reference to it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMargins& QMargins::operator+=(const QMargins& margins)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmargins.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Add each component of <i>margins</i> to the respective component of this object and returns a reference to it.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmargins.html#operator--eq">operator-=</a>().</p></div>
    fn add_assign(&mut self, margins: ::cpp_utils::Ref<crate::QMargins>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QMargins_operator__(
                self as *mut crate::QMargins,
                margins.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::SubAssign<::cpp_utils::Ref<crate::QMargins>> for crate::QMargins {
    /// <p>Subtract each component of <i>margins</i> from the respective component of this object and returns a reference to it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMargins& QMargins::operator-=(const QMargins& margins)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmargins.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Subtract each component of <i>margins</i> from the respective component of this object and returns a reference to it.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmargins.html#operator-2b-eq">operator+=</a>().</p></div>
    fn sub_assign(&mut self, margins: ::cpp_utils::Ref<crate::QMargins>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QMargins_operator__1(
                self as *mut crate::QMargins,
                margins.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::std::os::raw::c_int> for crate::QMargins {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMargins& QMargins::operator+=(int arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmargins.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Adds the <i>addend</i> to each component of this object and returns a reference to it.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmargins.html#operator--eq">operator-=</a>().</p></div>
    fn add_assign(&mut self, arg1: ::std::os::raw::c_int) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QMargins_operator__2(self as *mut crate::QMargins, arg1)
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::SubAssign<::std::os::raw::c_int> for crate::QMargins {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMargins& QMargins::operator-=(int arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmargins.html#operator--eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Subtracts the <i>subtrahend</i> from each component of this object and returns a reference to it.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmargins.html#operator-2b-eq">operator+=</a>().</p></div>
    fn sub_assign(&mut self, arg1: ::std::os::raw::c_int) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QMargins_operator__3(self as *mut crate::QMargins, arg1)
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::MulAssign<::std::os::raw::c_int> for crate::QMargins {
    /// <p>Multiplies each component of this object by <i>factor</i> and returns a reference to it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMargins& QMargins::operator*=(int arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmargins.html#operator-2a-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Multiplies each component of this object by <i>factor</i> and returns a reference to it.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmargins.html#operator-2f-eq">operator/=</a>().</p></div>
    fn mul_assign(&mut self, arg1: ::std::os::raw::c_int) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QMargins_operator__4(self as *mut crate::QMargins, arg1)
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::DivAssign<::std::os::raw::c_int> for crate::QMargins {
    /// <p>Divides each component of this object by <i>divisor</i> and returns a reference to it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMargins& QMargins::operator/=(int arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmargins.html#operator-2f-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Divides each component of this object by <i>divisor</i> and returns a reference to it.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmargins.html#operator-2a-eq">operator*=</a>().</p></div>
    fn div_assign(&mut self, arg1: ::std::os::raw::c_int) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QMargins_operator__5(self as *mut crate::QMargins, arg1)
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::MulAssign<::std::os::raw::c_double> for crate::QMargins {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMargins& QMargins::operator*=(double arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmargins.html#operator-2a-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Multiplies each component of this object by <i>factor</i> and returns a reference to it.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmargins.html#operator-2f-eq">operator/=</a>().</p></div>
    fn mul_assign(&mut self, arg1: ::std::os::raw::c_double) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QMargins_operator__6(self as *mut crate::QMargins, arg1)
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::DivAssign<::std::os::raw::c_double> for crate::QMargins {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMargins& QMargins::operator/=(double arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmargins.html#operator-2f-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmargins.html#operator-2a-eq">operator*=</a>().</p></div>
    fn div_assign(&mut self, arg1: ::std::os::raw::c_double) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QMargins_operator__7(self as *mut crate::QMargins, arg1)
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QMargins>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, const QMargins& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        arg2: ::cpp_utils::Ref<crate::QMargins>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__152(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QMargins>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QMargins& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QMargins>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__153(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QMargins>> for crate::QMargins {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(const QMargins& m1, const QMargins& m2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, m2: &::cpp_utils::Ref<crate::QMargins>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__154(
                self as *const crate::QMargins,
                m2.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QMargins>> for &crate::QMargins {
    type Output = ::cpp_utils::CppBox<crate::QMargins>;
    /// Calls C++ function: <span style='color: green;'>```QMargins operator+(const QMargins& m1, const QMargins& m2)```</span>.
    fn add(self, m2: ::cpp_utils::Ref<crate::QMargins>) -> ::cpp_utils::CppBox<crate::QMargins> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_88(
                self as *const crate::QMargins,
                m2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Sub<::cpp_utils::Ref<crate::QMargins>> for &crate::QMargins {
    type Output = ::cpp_utils::CppBox<crate::QMargins>;
    /// Calls C++ function: <span style='color: green;'>```QMargins operator-(const QMargins& m1, const QMargins& m2)```</span>.
    fn sub(self, m2: ::cpp_utils::Ref<crate::QMargins>) -> ::cpp_utils::CppBox<crate::QMargins> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_89(
                self as *const crate::QMargins,
                m2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::std::os::raw::c_int> for &crate::QMargins {
    type Output = ::cpp_utils::CppBox<crate::QMargins>;
    /// Calls C++ function: <span style='color: green;'>```QMargins operator+(const QMargins& lhs, int rhs)```</span>.
    fn add(self, rhs: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QMargins> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_90(self as *const crate::QMargins, rhs)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Sub<::std::os::raw::c_int> for &crate::QMargins {
    type Output = ::cpp_utils::CppBox<crate::QMargins>;
    /// Calls C++ function: <span style='color: green;'>```QMargins operator-(const QMargins& lhs, int rhs)```</span>.
    fn sub(self, rhs: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QMargins> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_92(self as *const crate::QMargins, rhs)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Mul<::std::os::raw::c_int> for &crate::QMargins {
    type Output = ::cpp_utils::CppBox<crate::QMargins>;
    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qmargins.html">QMargins</a> object that is formed by multiplying each component of the given <i>margins</i> by <i>factor</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMargins operator*(const QMargins& margins, int factor)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmargins.html#operator-2a">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qmargins.html">QMargins</a> object that is formed by multiplying each component of the given <i>margins</i> by <i>factor</i>.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmargins.html#operator-2a-eq">QMargins::operator*=</a>() and <a href="http://doc.qt.io/qt-5/qmargins.html#operator-2f-eq">QMargins::operator/=</a>().</p></div>
    fn mul(self, factor: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QMargins> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_93(self as *const crate::QMargins, factor)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Mul<::std::os::raw::c_double> for &crate::QMargins {
    type Output = ::cpp_utils::CppBox<crate::QMargins>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMargins operator*(const QMargins& margins, double factor)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmargins.html#operator-2a-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qmargins.html">QMargins</a> object that is formed by multiplying each component of the given <i>margins</i> by <i>factor</i>.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmargins.html#operator-2a-eq">QMargins::operator*=</a>() and <a href="http://doc.qt.io/qt-5/qmargins.html#operator-2f-eq">QMargins::operator/=</a>().</p></div>
    fn mul(self, factor: ::std::os::raw::c_double) -> ::cpp_utils::CppBox<crate::QMargins> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_95(self as *const crate::QMargins, factor)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Div<::std::os::raw::c_int> for &crate::QMargins {
    type Output = ::cpp_utils::CppBox<crate::QMargins>;
    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qmargins.html">QMargins</a> object that is formed by dividing the components of the given <i>margins</i> by the given <i>divisor</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMargins operator/(const QMargins& margins, int divisor)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmargins.html#operator-2f">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qmargins.html">QMargins</a> object that is formed by dividing the components of the given <i>margins</i> by the given <i>divisor</i>.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmargins.html#operator-2a-eq">QMargins::operator*=</a>() and <a href="http://doc.qt.io/qt-5/qmargins.html#operator-2f-eq">QMargins::operator/=</a>().</p></div>
    fn div(self, divisor: ::std::os::raw::c_int) -> ::cpp_utils::CppBox<crate::QMargins> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_97(self as *const crate::QMargins, divisor)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Div<::std::os::raw::c_double> for &crate::QMargins {
    type Output = ::cpp_utils::CppBox<crate::QMargins>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMargins operator/(const QMargins& margins, double divisor)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmargins.html#operator-2f-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a <a href="http://doc.qt.io/qt-5/qmargins.html">QMargins</a> object that is formed by dividing the components of the given <i>margins</i> by the given <i>divisor</i>.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmargins.html#operator-2a-eq">QMargins::operator*=</a>() and <a href="http://doc.qt.io/qt-5/qmargins.html#operator-2f-eq">QMargins::operator/=</a>().</p></div>
    fn div(self, divisor: ::std::os::raw::c_double) -> ::cpp_utils::CppBox<crate::QMargins> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_98(self as *const crate::QMargins, divisor)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Neg for &crate::QMargins {
    type Output = ::cpp_utils::CppBox<crate::QMargins>;
    /// Calls C++ function: <span style='color: green;'>```QMargins operator-(const QMargins& margins)```</span>.
    fn neg(self) -> ::cpp_utils::CppBox<crate::QMargins> {
        let ffi_result =
            unsafe { crate::__ffi::ctr_qt_core_ffi_operator_100(self as *const crate::QMargins) };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QMargins>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QMargins& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QMargins>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__156(
                self as *const crate::QDebug,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QMarginsF>> for crate::QMarginsF {
    /// <p>Add each component of <i>margins</i> to the respective component of this object and returns a reference to it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMarginsF& QMarginsF::operator+=(const QMarginsF& margins)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmarginsf.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Add each component of <i>margins</i> to the respective component of this object and returns a reference to it.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmarginsf.html#operator--eq">operator-=</a>().</p></div>
    fn add_assign(&mut self, margins: ::cpp_utils::Ref<crate::QMarginsF>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QMarginsF_operator__(
                self as *mut crate::QMarginsF,
                margins.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::SubAssign<::cpp_utils::Ref<crate::QMarginsF>> for crate::QMarginsF {
    /// <p>Subtract each component of <i>margins</i> from the respective component of this object and returns a reference to it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMarginsF& QMarginsF::operator-=(const QMarginsF& margins)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmarginsf.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Subtract each component of <i>margins</i> from the respective component of this object and returns a reference to it.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmarginsf.html#operator-2b-eq">operator+=</a>().</p></div>
    fn sub_assign(&mut self, margins: ::cpp_utils::Ref<crate::QMarginsF>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QMarginsF_operator__1(
                self as *mut crate::QMarginsF,
                margins.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::std::os::raw::c_double> for crate::QMarginsF {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMarginsF& QMarginsF::operator+=(double addend)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmarginsf.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Adds the <i>addend</i> to each component of this object and returns a reference to it.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmarginsf.html#operator--eq">operator-=</a>().</p></div>
    fn add_assign(&mut self, addend: ::std::os::raw::c_double) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QMarginsF_operator__2(
                self as *mut crate::QMarginsF,
                addend,
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::SubAssign<::std::os::raw::c_double> for crate::QMarginsF {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMarginsF& QMarginsF::operator-=(double subtrahend)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmarginsf.html#operator--eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Subtracts the <i>subtrahend</i> from each component of this object and returns a reference to it.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmarginsf.html#operator-2b-eq">operator+=</a>().</p></div>
    fn sub_assign(&mut self, subtrahend: ::std::os::raw::c_double) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QMarginsF_operator__3(
                self as *mut crate::QMarginsF,
                subtrahend,
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::MulAssign<::std::os::raw::c_double> for crate::QMarginsF {
    /// <p>Multiplies each component of this object by <i>factor</i> and returns a reference to it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMarginsF& QMarginsF::operator*=(double factor)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmarginsf.html#operator-2a-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Multiplies each component of this object by <i>factor</i> and returns a reference to it.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmarginsf.html#operator-2f-eq">operator/=</a>().</p></div>
    fn mul_assign(&mut self, factor: ::std::os::raw::c_double) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QMarginsF_operator__4(
                self as *mut crate::QMarginsF,
                factor,
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::DivAssign<::std::os::raw::c_double> for crate::QMarginsF {
    /// <p>Divides each component of this object by <i>divisor</i> and returns a reference to it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMarginsF& QMarginsF::operator/=(double divisor)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmarginsf.html#operator-2f-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Divides each component of this object by <i>divisor</i> and returns a reference to it.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmarginsf.html#operator-2a-eq">operator*=</a>().</p></div>
    fn div_assign(&mut self, divisor: ::std::os::raw::c_double) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QMarginsF_operator__5(
                self as *mut crate::QMarginsF,
                divisor,
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QMarginsF>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, const QMarginsF& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        arg2: ::cpp_utils::Ref<crate::QMarginsF>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__157(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QMarginsF>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QMarginsF& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QMarginsF>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__158(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QMarginsF>> for crate::QMarginsF {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(const QMarginsF& lhs, const QMarginsF& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, rhs: &::cpp_utils::Ref<crate::QMarginsF>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__159(
                self as *const crate::QMarginsF,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QMarginsF>> for &crate::QMarginsF {
    type Output = ::cpp_utils::CppBox<crate::QMarginsF>;
    /// Calls C++ function: <span style='color: green;'>```QMarginsF operator+(const QMarginsF& lhs, const QMarginsF& rhs)```</span>.
    fn add(self, rhs: ::cpp_utils::Ref<crate::QMarginsF>) -> ::cpp_utils::CppBox<crate::QMarginsF> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_101(
                self as *const crate::QMarginsF,
                rhs.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Sub<::cpp_utils::Ref<crate::QMarginsF>> for &crate::QMarginsF {
    type Output = ::cpp_utils::CppBox<crate::QMarginsF>;
    /// Calls C++ function: <span style='color: green;'>```QMarginsF operator-(const QMarginsF& lhs, const QMarginsF& rhs)```</span>.
    fn sub(self, rhs: ::cpp_utils::Ref<crate::QMarginsF>) -> ::cpp_utils::CppBox<crate::QMarginsF> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_102(
                self as *const crate::QMarginsF,
                rhs.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::std::os::raw::c_double> for &crate::QMarginsF {
    type Output = ::cpp_utils::CppBox<crate::QMarginsF>;
    /// Calls C++ function: <span style='color: green;'>```QMarginsF operator+(const QMarginsF& lhs, double rhs)```</span>.
    fn add(self, rhs: ::std::os::raw::c_double) -> ::cpp_utils::CppBox<crate::QMarginsF> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_103(self as *const crate::QMarginsF, rhs)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Sub<::std::os::raw::c_double> for &crate::QMarginsF {
    type Output = ::cpp_utils::CppBox<crate::QMarginsF>;
    /// Calls C++ function: <span style='color: green;'>```QMarginsF operator-(const QMarginsF& lhs, double rhs)```</span>.
    fn sub(self, rhs: ::std::os::raw::c_double) -> ::cpp_utils::CppBox<crate::QMarginsF> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_105(self as *const crate::QMarginsF, rhs)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Mul<::std::os::raw::c_double> for &crate::QMarginsF {
    type Output = ::cpp_utils::CppBox<crate::QMarginsF>;
    /// Calls C++ function: <span style='color: green;'>```QMarginsF operator*(const QMarginsF& lhs, double rhs)```</span>.
    fn mul(self, rhs: ::std::os::raw::c_double) -> ::cpp_utils::CppBox<crate::QMarginsF> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_106(self as *const crate::QMarginsF, rhs)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Div<::std::os::raw::c_double> for &crate::QMarginsF {
    type Output = ::cpp_utils::CppBox<crate::QMarginsF>;
    /// Calls C++ function: <span style='color: green;'>```QMarginsF operator/(const QMarginsF& lhs, double divisor)```</span>.
    fn div(self, divisor: ::std::os::raw::c_double) -> ::cpp_utils::CppBox<crate::QMarginsF> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_108(self as *const crate::QMarginsF, divisor)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Neg for &crate::QMarginsF {
    type Output = ::cpp_utils::CppBox<crate::QMarginsF>;
    /// Calls C++ function: <span style='color: green;'>```QMarginsF operator-(const QMarginsF& margins)```</span>.
    fn neg(self) -> ::cpp_utils::CppBox<crate::QMarginsF> {
        let ffi_result =
            unsafe { crate::__ffi::ctr_qt_core_ffi_operator_110(self as *const crate::QMarginsF) };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QMarginsF>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QMarginsF& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QMarginsF>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__161(
                self as *const crate::QDebug,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QMessageAuthenticationCode {
    /// <p>Destroys the object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QMessageAuthenticationCode::~QMessageAuthenticationCode()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmessageauthenticationcode.html#dtor.QMessageAuthenticationCode">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the object.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMessageAuthenticationCode_dQMessageAuthenticationCode(
            self as *mut crate::QMessageAuthenticationCode,
        )
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QMetaMethod>> for crate::QMetaMethod {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(const QMetaMethod& m1, const QMetaMethod& m2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, m2: &::cpp_utils::Ref<crate::QMetaMethod>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__162(
                self as *const crate::QMetaMethod,
                m2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QMimeData {
    /// <p>Destroys the MIME data object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QMimeData::~QMimeData()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedata.html#dtor.QMimeData">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the MIME data object.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMimeData_dQMimeData(self as *mut crate::QMimeData)
    }
}

impl ::cpp_utils::CppDeletable for crate::QMimeType {
    /// <p>Destroys the <a href="http://doc.qt.io/qt-5/qmimetype.html">QMimeType</a> object, and releases the d pointer.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QMimeType::~QMimeType()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimetype.html#dtor.QMimeType">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the <a href="http://doc.qt.io/qt-5/qmimetype.html">QMimeType</a> object, and releases the d pointer.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMimeType_dQMimeType(self as *mut crate::QMimeType)
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QMimeType>> for crate::QMimeType {
    /// <p>Returns <code>true</code> if <i>other</i> equals this <a href="http://doc.qt.io/qt-5/qmimetype.html">QMimeType</a> object, otherwise returns <code>false</code>. The name is the unique identifier for a mimetype, so two mimetypes with the same name, are equal.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMimeType::operator==(const QMimeType& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimetype.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> equals this <a href="http://doc.qt.io/qt-5/qmimetype.html">QMimeType</a> object, otherwise returns <code>false</code>. The name is the unique identifier for a mimetype, so two mimetypes with the same name, are equal.</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QMimeType>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QMimeType_operator__(
                self as *const crate::QMimeType,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QMimeType>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QMimeType& mime)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, mime: ::cpp_utils::Ref<crate::QMimeType>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__164(
                self as *const crate::QDebug,
                mime.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QMimeDatabase {
    /// <p>Destroys the <a href="http://doc.qt.io/qt-5/qmimedatabase.html">QMimeDatabase</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QMimeDatabase::~QMimeDatabase()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmimedatabase.html#dtor.QMimeDatabase">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the <a href="http://doc.qt.io/qt-5/qmimedatabase.html">QMimeDatabase</a> object.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMimeDatabase_dQMimeDatabase(
            self as *mut crate::QMimeDatabase,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QObjectCleanupHandler {
    /// <p>Destroys the cleanup handler. All objects in this cleanup handler will be deleted.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QObjectCleanupHandler::~QObjectCleanupHandler()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qobjectcleanuphandler.html#dtor.QObjectCleanupHandler">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the cleanup handler. All objects in this cleanup handler will be deleted.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobjectcleanuphandler.html#clear">clear</a>().</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QObjectCleanupHandler_dQObjectCleanupHandler(
            self as *mut crate::QObjectCleanupHandler,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QParallelAnimationGroup {
    /// <p>Destroys the animation group. It will also destroy all its animations.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QParallelAnimationGroup::~QParallelAnimationGroup()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qparallelanimationgroup.html#dtor.QParallelAnimationGroup">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the animation group. It will also destroy all its animations.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QParallelAnimationGroup_dQParallelAnimationGroup(
            self as *mut crate::QParallelAnimationGroup,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QPauseAnimation {
    /// <p>Destroys the pause animation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QPauseAnimation::~QPauseAnimation()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpauseanimation.html#dtor.QPauseAnimation">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the pause animation.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QPauseAnimation_dQPauseAnimation(
            self as *mut crate::QPauseAnimation,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QPluginLoader {
    /// <p>Destroys the <a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QPluginLoader::~QPluginLoader()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpluginloader.html#dtor.QPluginLoader">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the <a href="http://doc.qt.io/qt-5/qpluginloader.html">QPluginLoader</a> object.</p>
    /// <p>Unless <a href="http://doc.qt.io/qt-5/qpluginloader.html#unload">unload</a>() was called explicitly, the plugin stays in memory until the application terminates.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qpluginloader.html#isLoaded">isLoaded</a>() and <a href="http://doc.qt.io/qt-5/qpluginloader.html#unload">unload</a>().</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QPluginLoader_dQPluginLoader(
            self as *mut crate::QPluginLoader,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QProcessEnvironment {
    /// <p>Frees the resources associated with this <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QProcessEnvironment::~QProcessEnvironment()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#dtor.QProcessEnvironment">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Frees the resources associated with this <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> object.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QProcessEnvironment_dQProcessEnvironment(
            self as *mut crate::QProcessEnvironment,
        )
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QProcessEnvironment>>
    for crate::QProcessEnvironment
{
    /// <p>Returns <code>true</code> if this and the <i>other</i> <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> objects are equal.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QProcessEnvironment::operator==(const QProcessEnvironment& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this and the <i>other</i> <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> objects are equal.</p>
    /// <p>Two <a href="http://doc.qt.io/qt-5/qprocessenvironment.html">QProcessEnvironment</a> objects are considered equal if they have the same set of key=value pairs. The comparison of keys is done case-sensitive on platforms where the environment is case-sensitive.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qprocessenvironment.html#operator-not-eq">operator!=</a>() and <a href="http://doc.qt.io/qt-5/qprocessenvironment.html#contains">contains</a>().</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QProcessEnvironment>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QProcessEnvironment_operator__(
                self as *const crate::QProcessEnvironment,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QProcess {
    /// <p>Destructs the <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> object, i.e., killing the process.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QProcess::~QProcess()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qprocess.html#dtor.QProcess">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destructs the <a href="http://doc.qt.io/qt-5/qprocess.html">QProcess</a> object, i.e., killing the process.</p>
    /// <p>Note that this function will not return until the process is terminated.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QProcess_dQProcess(self as *mut crate::QProcess)
    }
}

impl ::cpp_utils::CppDeletable for crate::QVariantAnimation {
    /// <p>Destroys the animation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QVariantAnimation::~QVariantAnimation()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvariantanimation.html#dtor.QVariantAnimation">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the animation.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVariantAnimation_dQVariantAnimation(
            self as *mut crate::QVariantAnimation,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QPropertyAnimation {
    /// <p>Destroys the <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a> instance.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QPropertyAnimation::~QPropertyAnimation()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpropertyanimation.html#dtor.QPropertyAnimation">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the <a href="http://doc.qt.io/qt-5/qpropertyanimation.html">QPropertyAnimation</a> instance.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QPropertyAnimation_dQPropertyAnimation(
            self as *mut crate::QPropertyAnimation,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QReadWriteLock {
    /// <p>Destroys the <a href="http://doc.qt.io/qt-5/qreadwritelock.html">QReadWriteLock</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QReadWriteLock::~QReadWriteLock()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qreadwritelock.html#dtor.QReadWriteLock">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the <a href="http://doc.qt.io/qt-5/qreadwritelock.html">QReadWriteLock</a> object.</p>
    /// <p><b>Warning:</b> Destroying a read-write lock that is in use may result in undefined behavior.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QReadWriteLock_dQReadWriteLock(
            self as *mut crate::QReadWriteLock,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QReadLocker {
    /// <p>Destroys the <a href="http://doc.qt.io/qt-5/qreadlocker.html">QReadLocker</a> and unlocks the lock that was passed to the constructor.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QReadLocker::~QReadLocker()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qreadlocker.html#dtor.QReadLocker">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the <a href="http://doc.qt.io/qt-5/qreadlocker.html">QReadLocker</a> and unlocks the lock that was passed to the constructor.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qreadwritelock.html#unlock">QReadWriteLock::unlock</a>().</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QReadLocker_dQReadLocker(self as *mut crate::QReadLocker)
    }
}

impl ::cpp_utils::CppDeletable for crate::QWriteLocker {
    /// <p>Destroys the <a href="http://doc.qt.io/qt-5/qwritelocker.html">QWriteLocker</a> and unlocks the lock that was passed to the constructor.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QWriteLocker::~QWriteLocker()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qwritelocker.html#dtor.QWriteLocker">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the <a href="http://doc.qt.io/qt-5/qwritelocker.html">QWriteLocker</a> and unlocks the lock that was passed to the constructor.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qreadwritelock.html#unlock">QReadWriteLock::unlock</a>().</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QWriteLocker_dQWriteLocker(self as *mut crate::QWriteLocker)
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QSize>> for crate::QSize {
    /// <p>Adds the given <i>size</i> to <i>this</i> size, and returns a reference to this size. For example:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSize& QSize::operator+=(const QSize& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsize.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds the given <i>size</i> to <i>this</i> size, and returns a reference to this size. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsize.html#QSize">QSize</a></span> s( <span class="number">3</span><span class="operator">,</span> <span class="number">7</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsize.html#QSize">QSize</a></span> r(<span class="operator">-</span><span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   s <span class="operator">+</span><span class="operator">=</span> r;
    ///
    ///   <span class="comment">// s becomes (2,11)</span>
    ///
    /// </pre></div>
    fn add_assign(&mut self, arg1: ::cpp_utils::Ref<crate::QSize>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QSize_operator__(
                self as *mut crate::QSize,
                arg1.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::SubAssign<::cpp_utils::Ref<crate::QSize>> for crate::QSize {
    /// <p>Subtracts the given <i>size</i> from <i>this</i> size, and returns a reference to this size. For example:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSize& QSize::operator-=(const QSize& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsize.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Subtracts the given <i>size</i> from <i>this</i> size, and returns a reference to this size. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsize.html#QSize">QSize</a></span> s( <span class="number">3</span><span class="operator">,</span> <span class="number">7</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsize.html#QSize">QSize</a></span> r(<span class="operator">-</span><span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   s <span class="operator">-</span><span class="operator">=</span> r;
    ///
    ///   <span class="comment">// s becomes (4,3)</span>
    ///
    /// </pre></div>
    fn sub_assign(&mut self, arg1: ::cpp_utils::Ref<crate::QSize>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QSize_operator__1(
                self as *mut crate::QSize,
                arg1.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::MulAssign<::std::os::raw::c_double> for crate::QSize {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSize& QSize::operator*=(double c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsize.html#operator-2a-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Multiplies both the width and height by the given <i>factor</i>, and returns a reference to the size.</p>
    /// <p>Note that the result is rounded to the nearest integer.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsize.html#scale">scale</a>().</p></div>
    fn mul_assign(&mut self, c: ::std::os::raw::c_double) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QSize_operator__2(self as *mut crate::QSize, c)
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::DivAssign<::std::os::raw::c_double> for crate::QSize {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSize& QSize::operator/=(double c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsize.html#operator-2f-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Divides both the width and height by the given <i>divisor</i>, and returns a reference to the size.</p>
    /// <p>Note that the result is rounded to the nearest integer.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsize.html#scale">QSize::scale</a>().</p></div>
    fn div_assign(&mut self, c: ::std::os::raw::c_double) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QSize_operator__3(self as *mut crate::QSize, c)
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QSize>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, const QSize& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QSize>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__165(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QSize>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QSize& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QSize>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__166(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QSize>> for crate::QSize {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(const QSize& s1, const QSize& s2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, s2: &::cpp_utils::Ref<crate::QSize>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__167(
                self as *const crate::QSize,
                s2.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QSize>> for &crate::QSize {
    type Output = ::cpp_utils::CppBox<crate::QSize>;
    /// Calls C++ function: <span style='color: green;'>```QSize operator+(const QSize& s1, const QSize& s2)```</span>.
    fn add(self, s2: ::cpp_utils::Ref<crate::QSize>) -> ::cpp_utils::CppBox<crate::QSize> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_111(self as *const crate::QSize, s2.as_raw_ptr())
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Sub<::cpp_utils::Ref<crate::QSize>> for &crate::QSize {
    type Output = ::cpp_utils::CppBox<crate::QSize>;
    /// Calls C++ function: <span style='color: green;'>```QSize operator-(const QSize& s1, const QSize& s2)```</span>.
    fn sub(self, s2: ::cpp_utils::Ref<crate::QSize>) -> ::cpp_utils::CppBox<crate::QSize> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_112(self as *const crate::QSize, s2.as_raw_ptr())
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Mul<::std::os::raw::c_double> for &crate::QSize {
    type Output = ::cpp_utils::CppBox<crate::QSize>;
    /// Calls C++ function: <span style='color: green;'>```QSize operator*(const QSize& s, double c)```</span>.
    fn mul(self, c: ::std::os::raw::c_double) -> ::cpp_utils::CppBox<crate::QSize> {
        let ffi_result =
            unsafe { crate::__ffi::ctr_qt_core_ffi_operator_113(self as *const crate::QSize, c) };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Div<::std::os::raw::c_double> for &crate::QSize {
    type Output = ::cpp_utils::CppBox<crate::QSize>;
    /// Calls C++ function: <span style='color: green;'>```QSize operator/(const QSize& s, double c)```</span>.
    fn div(self, c: ::std::os::raw::c_double) -> ::cpp_utils::CppBox<crate::QSize> {
        let ffi_result =
            unsafe { crate::__ffi::ctr_qt_core_ffi_operator_115(self as *const crate::QSize, c) };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QSize>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QSize& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QSize>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__169(
                self as *const crate::QDebug,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QSizeF>> for crate::QSizeF {
    /// <p>Adds the given <i>size</i> to this size and returns a reference to this size. For example:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSizeF& QSizeF::operator+=(const QSizeF& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds the given <i>size</i> to this size and returns a reference to this size. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsizef.html#QSizeF">QSizeF</a></span> s( <span class="number">3</span><span class="operator">,</span> <span class="number">7</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsizef.html#QSizeF">QSizeF</a></span> r(<span class="operator">-</span><span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   s <span class="operator">+</span><span class="operator">=</span> r;
    ///
    ///   <span class="comment">// s becomes (2,11)</span>
    ///
    /// </pre></div>
    fn add_assign(&mut self, arg1: ::cpp_utils::Ref<crate::QSizeF>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QSizeF_operator__(
                self as *mut crate::QSizeF,
                arg1.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::SubAssign<::cpp_utils::Ref<crate::QSizeF>> for crate::QSizeF {
    /// <p>Subtracts the given <i>size</i> from this size and returns a reference to this size. For example:</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSizeF& QSizeF::operator-=(const QSizeF& arg1)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Subtracts the given <i>size</i> from this size and returns a reference to this size. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsizef.html#QSizeF">QSizeF</a></span> s( <span class="number">3</span><span class="operator">,</span> <span class="number">7</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsizef.html#QSizeF">QSizeF</a></span> r(<span class="operator">-</span><span class="number">1</span><span class="operator">,</span> <span class="number">4</span>);
    ///   s <span class="operator">-</span><span class="operator">=</span> r;
    ///
    ///   <span class="comment">// s becomes (4,3)</span>
    ///
    /// </pre></div>
    fn sub_assign(&mut self, arg1: ::cpp_utils::Ref<crate::QSizeF>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QSizeF_operator__1(
                self as *mut crate::QSizeF,
                arg1.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::MulAssign<::std::os::raw::c_double> for crate::QSizeF {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSizeF& QSizeF::operator*=(double c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html#operator-2a-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Multiplies both the width and height by the given <i>factor</i> and returns a reference to the size.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsizef.html#scale">scale</a>().</p></div>
    fn mul_assign(&mut self, c: ::std::os::raw::c_double) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QSizeF_operator__2(self as *mut crate::QSizeF, c)
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::DivAssign<::std::os::raw::c_double> for crate::QSizeF {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSizeF& QSizeF::operator/=(double c)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html#operator-2f-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Divides both the width and height by the given <i>divisor</i> and returns a reference to the size.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsizef.html#scale">scale</a>().</p></div>
    fn div_assign(&mut self, c: ::std::os::raw::c_double) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QSizeF_operator__3(self as *mut crate::QSizeF, c)
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QSizeF>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, const QSizeF& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QSizeF>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__170(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QSizeF>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QSizeF& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QSizeF>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__171(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QSizeF>> for crate::QSizeF {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(const QSizeF& s1, const QSizeF& s2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, s2: &::cpp_utils::Ref<crate::QSizeF>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__172(
                self as *const crate::QSizeF,
                s2.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QSizeF>> for &crate::QSizeF {
    type Output = ::cpp_utils::CppBox<crate::QSizeF>;
    /// Calls C++ function: <span style='color: green;'>```QSizeF operator+(const QSizeF& s1, const QSizeF& s2)```</span>.
    fn add(self, s2: ::cpp_utils::Ref<crate::QSizeF>) -> ::cpp_utils::CppBox<crate::QSizeF> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_116(
                self as *const crate::QSizeF,
                s2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Sub<::cpp_utils::Ref<crate::QSizeF>> for &crate::QSizeF {
    type Output = ::cpp_utils::CppBox<crate::QSizeF>;
    /// Calls C++ function: <span style='color: green;'>```QSizeF operator-(const QSizeF& s1, const QSizeF& s2)```</span>.
    fn sub(self, s2: ::cpp_utils::Ref<crate::QSizeF>) -> ::cpp_utils::CppBox<crate::QSizeF> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_117(
                self as *const crate::QSizeF,
                s2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Mul<::std::os::raw::c_double> for &crate::QSizeF {
    type Output = ::cpp_utils::CppBox<crate::QSizeF>;
    /// Calls C++ function: <span style='color: green;'>```QSizeF operator*(const QSizeF& s, double c)```</span>.
    fn mul(self, c: ::std::os::raw::c_double) -> ::cpp_utils::CppBox<crate::QSizeF> {
        let ffi_result =
            unsafe { crate::__ffi::ctr_qt_core_ffi_operator_118(self as *const crate::QSizeF, c) };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Div<::std::os::raw::c_double> for &crate::QSizeF {
    type Output = ::cpp_utils::CppBox<crate::QSizeF>;
    /// Calls C++ function: <span style='color: green;'>```QSizeF operator/(const QSizeF& s, double c)```</span>.
    fn div(self, c: ::std::os::raw::c_double) -> ::cpp_utils::CppBox<crate::QSizeF> {
        let ffi_result =
            unsafe { crate::__ffi::ctr_qt_core_ffi_operator_120(self as *const crate::QSizeF, c) };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QSizeF>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QSizeF& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QSizeF>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__174(
                self as *const crate::QDebug,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::BitOr<::cpp_utils::Ref<crate::QRect>> for &crate::QRect {
    type Output = ::cpp_utils::CppBox<crate::QRect>;
    /// <p>Returns the bounding rectangle of this rectangle and the given <i>rectangle</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRect QRect::operator|(const QRect& r) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#operator-7c">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the bounding rectangle of this rectangle and the given <i>rectangle</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#operator-7c-eq">operator|=</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#united">united</a>().</p></div>
    fn bitor(self, r: ::cpp_utils::Ref<crate::QRect>) -> ::cpp_utils::CppBox<crate::QRect> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QRect_operator_(
                self as *const crate::QRect,
                r.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::BitAnd<::cpp_utils::Ref<crate::QRect>> for &crate::QRect {
    type Output = ::cpp_utils::CppBox<crate::QRect>;
    /// <p>Returns the intersection of this rectangle and the given <i>rectangle</i>. Returns an empty rectangle if there is no intersection.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRect QRect::operator&(const QRect& r) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#operator-and">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the intersection of this rectangle and the given <i>rectangle</i>. Returns an empty rectangle if there is no intersection.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#operator-and-eq">operator&amp;=</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#intersected">intersected</a>().</p></div>
    fn bitand(self, r: ::cpp_utils::Ref<crate::QRect>) -> ::cpp_utils::CppBox<crate::QRect> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QRect_operator_1(
                self as *const crate::QRect,
                r.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::BitOrAssign<::cpp_utils::Ref<crate::QRect>> for crate::QRect {
    /// <p>Unites this rectangle with the given <i>rectangle</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRect& QRect::operator|=(const QRect& r)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#operator-7c-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Unites this rectangle with the given <i>rectangle</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#united">united</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#operator-7c">operator|</a>().</p></div>
    fn bitor_assign(&mut self, r: ::cpp_utils::Ref<crate::QRect>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QRect_operator__(
                self as *mut crate::QRect,
                r.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::BitAndAssign<::cpp_utils::Ref<crate::QRect>> for crate::QRect {
    /// <p>Intersects this rectangle with the given <i>rectangle</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRect& QRect::operator&=(const QRect& r)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#operator-and-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Intersects this rectangle with the given <i>rectangle</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#intersected">intersected</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#operator-and">operator&amp;</a>().</p></div>
    fn bitand_assign(&mut self, r: ::cpp_utils::Ref<crate::QRect>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QRect_operator__1(
                self as *mut crate::QRect,
                r.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QMargins>> for crate::QRect {
    /// <p>Adds the <i>margins</i> to the rectangle, growing it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRect& QRect::operator+=(const QMargins& margins)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds the <i>margins</i> to the rectangle, growing it.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#marginsAdded">marginsAdded</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#marginsRemoved">marginsRemoved</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#operator--eq">operator-=</a>().</p></div>
    fn add_assign(&mut self, margins: ::cpp_utils::Ref<crate::QMargins>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QRect_operator__2(
                self as *mut crate::QRect,
                margins.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::SubAssign<::cpp_utils::Ref<crate::QMargins>> for crate::QRect {
    /// <p>Returns a rectangle shrunk by the <i>margins</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRect& QRect::operator-=(const QMargins& margins)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a rectangle shrunk by the <i>margins</i>.</p>
    /// <p>This function was introduced in  Qt 5.1.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrect.html#marginsRemoved">marginsRemoved</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#operator-2b-eq">operator+=</a>(), and <a href="http://doc.qt.io/qt-5/qrect.html#marginsAdded">marginsAdded</a>().</p></div>
    fn sub_assign(&mut self, margins: ::cpp_utils::Ref<crate::QMargins>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QRect_operator__3(
                self as *mut crate::QRect,
                margins.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QRect>> for crate::QRect {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(const QRect& arg1, const QRect& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, arg2: &::cpp_utils::Ref<crate::QRect>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__175(
                self as *const crate::QRect,
                arg2.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QRect>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, const QRect& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QRect>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__177(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QRect>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QRect& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QRect>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__178(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QMargins>> for &crate::QRect {
    type Output = ::cpp_utils::CppBox<crate::QRect>;
    /// Calls C++ function: <span style='color: green;'>```QRect operator+(const QRect& rectangle, const QMargins& margins)```</span>.
    fn add(self, margins: ::cpp_utils::Ref<crate::QMargins>) -> ::cpp_utils::CppBox<crate::QRect> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_121(
                self as *const crate::QRect,
                margins.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QRect>> for &crate::QMargins {
    type Output = ::cpp_utils::CppBox<crate::QRect>;
    /// Calls C++ function: <span style='color: green;'>```QRect operator+(const QMargins& margins, const QRect& rectangle)```</span>.
    fn add(self, rectangle: ::cpp_utils::Ref<crate::QRect>) -> ::cpp_utils::CppBox<crate::QRect> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_122(
                self as *const crate::QMargins,
                rectangle.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Sub<::cpp_utils::Ref<crate::QMargins>> for &crate::QRect {
    type Output = ::cpp_utils::CppBox<crate::QRect>;
    /// Calls C++ function: <span style='color: green;'>```QRect operator-(const QRect& lhs, const QMargins& rhs)```</span>.
    fn sub(self, rhs: ::cpp_utils::Ref<crate::QMargins>) -> ::cpp_utils::CppBox<crate::QRect> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_123(
                self as *const crate::QRect,
                rhs.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QRect>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QRect& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QRect>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__179(
                self as *const crate::QDebug,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::BitOr<::cpp_utils::Ref<crate::QRectF>> for &crate::QRectF {
    type Output = ::cpp_utils::CppBox<crate::QRectF>;
    /// <p>Returns the bounding rectangle of this rectangle and the given <i>rectangle</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRectF QRectF::operator|(const QRectF& r) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#operator-7c">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the bounding rectangle of this rectangle and the given <i>rectangle</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#united">united</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#operator-7c-eq">operator|=</a>().</p></div>
    fn bitor(self, r: ::cpp_utils::Ref<crate::QRectF>) -> ::cpp_utils::CppBox<crate::QRectF> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QRectF_operator_(
                self as *const crate::QRectF,
                r.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::BitAnd<::cpp_utils::Ref<crate::QRectF>> for &crate::QRectF {
    type Output = ::cpp_utils::CppBox<crate::QRectF>;
    /// <p>Returns the intersection of this rectangle and the given <i>rectangle</i>. Returns an empty rectangle if there is no intersection.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRectF QRectF::operator&(const QRectF& r) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#operator-and">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the intersection of this rectangle and the given <i>rectangle</i>. Returns an empty rectangle if there is no intersection.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#operator-and-eq">operator&amp;=</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#intersected">intersected</a>().</p></div>
    fn bitand(self, r: ::cpp_utils::Ref<crate::QRectF>) -> ::cpp_utils::CppBox<crate::QRectF> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QRectF_operator_1(
                self as *const crate::QRectF,
                r.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::BitOrAssign<::cpp_utils::Ref<crate::QRectF>> for crate::QRectF {
    /// <p>Unites this rectangle with the given <i>rectangle</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRectF& QRectF::operator|=(const QRectF& r)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#operator-7c-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Unites this rectangle with the given <i>rectangle</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#united">united</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#operator-7c">operator|</a>().</p></div>
    fn bitor_assign(&mut self, r: ::cpp_utils::Ref<crate::QRectF>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QRectF_operator__(
                self as *mut crate::QRectF,
                r.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::BitAndAssign<::cpp_utils::Ref<crate::QRectF>> for crate::QRectF {
    /// <p>Intersects this rectangle with the given <i>rectangle</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRectF& QRectF::operator&=(const QRectF& r)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#operator-and-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Intersects this rectangle with the given <i>rectangle</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#intersected">intersected</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#operator-7c-eq">operator|=</a>().</p></div>
    fn bitand_assign(&mut self, r: ::cpp_utils::Ref<crate::QRectF>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QRectF_operator__1(
                self as *mut crate::QRectF,
                r.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QMarginsF>> for crate::QRectF {
    /// <p>Adds the <i>margins</i> to the rectangle, growing it.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRectF& QRectF::operator+=(const QMarginsF& margins)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds the <i>margins</i> to the rectangle, growing it.</p>
    /// <p>This function was introduced in  Qt 5.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#marginsAdded">marginsAdded</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#marginsRemoved">marginsRemoved</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#operator--eq">operator-=</a>().</p></div>
    fn add_assign(&mut self, margins: ::cpp_utils::Ref<crate::QMarginsF>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QRectF_operator__2(
                self as *mut crate::QRectF,
                margins.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::SubAssign<::cpp_utils::Ref<crate::QMarginsF>> for crate::QRectF {
    /// <p>Returns a rectangle shrunk by the <i>margins</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QRectF& QRectF::operator-=(const QMarginsF& margins)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a rectangle shrunk by the <i>margins</i>.</p>
    /// <p>This function was introduced in  Qt 5.3.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qrectf.html#marginsRemoved">marginsRemoved</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#operator-2b-eq">operator+=</a>(), and <a href="http://doc.qt.io/qt-5/qrectf.html#marginsAdded">marginsAdded</a>().</p></div>
    fn sub_assign(&mut self, margins: ::cpp_utils::Ref<crate::QMarginsF>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QRectF_operator__3(
                self as *mut crate::QRectF,
                margins.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QRectF>> for crate::QRectF {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(const QRectF& arg1, const QRectF& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, arg2: &::cpp_utils::Ref<crate::QRectF>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__180(
                self as *const crate::QRectF,
                arg2.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QRectF>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, const QRectF& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QRectF>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__182(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QRectF>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QRectF& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QRectF>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__183(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QMarginsF>> for &crate::QRectF {
    type Output = ::cpp_utils::CppBox<crate::QRectF>;
    /// Calls C++ function: <span style='color: green;'>```QRectF operator+(const QRectF& lhs, const QMarginsF& rhs)```</span>.
    fn add(self, rhs: ::cpp_utils::Ref<crate::QMarginsF>) -> ::cpp_utils::CppBox<crate::QRectF> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_124(
                self as *const crate::QRectF,
                rhs.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QRectF>> for &crate::QMarginsF {
    type Output = ::cpp_utils::CppBox<crate::QRectF>;
    /// Calls C++ function: <span style='color: green;'>```QRectF operator+(const QMarginsF& lhs, const QRectF& rhs)```</span>.
    fn add(self, rhs: ::cpp_utils::Ref<crate::QRectF>) -> ::cpp_utils::CppBox<crate::QRectF> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_125(
                self as *const crate::QMarginsF,
                rhs.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Sub<::cpp_utils::Ref<crate::QMarginsF>> for &crate::QRectF {
    type Output = ::cpp_utils::CppBox<crate::QRectF>;
    /// Calls C++ function: <span style='color: green;'>```QRectF operator-(const QRectF& lhs, const QMarginsF& rhs)```</span>.
    fn sub(self, rhs: ::cpp_utils::Ref<crate::QMarginsF>) -> ::cpp_utils::CppBox<crate::QRectF> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_126(
                self as *const crate::QRectF,
                rhs.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QRectF>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QRectF& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QRectF>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__184(
                self as *const crate::QDebug,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QRegularExpression {
    /// <p>Destroys the <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QRegularExpression::~QRegularExpression()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#dtor.QRegularExpression">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> object.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpression_dQRegularExpression(
            self as *mut crate::QRegularExpression,
        )
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QRegularExpression>>
    for crate::QRegularExpression
{
    /// <p>Returns <code>true</code> if the regular expression is equal to <i>re</i>, or false otherwise. Two <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> objects are equal if they have the same pattern string and the same pattern options.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QRegularExpression::operator==(const QRegularExpression& re) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpression.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the regular expression is equal to <i>re</i>, or false otherwise. Two <a href="http://doc.qt.io/qt-5/qregularexpression.html">QRegularExpression</a> objects are equal if they have the same pattern string and the same pattern options.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qregularexpression.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, re: &::cpp_utils::Ref<crate::QRegularExpression>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QRegularExpression_operator__(
                self as *const crate::QRegularExpression,
                re.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QRegularExpression>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& out, const QRegularExpression& re)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        re: ::cpp_utils::Ref<crate::QRegularExpression>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__185(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                re.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QRegularExpression>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& in, QRegularExpression& re)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        re: ::cpp_utils::MutRef<crate::QRegularExpression>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__186(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                re.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QRegularExpression>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QRegularExpression& re)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        re: ::cpp_utils::Ref<crate::QRegularExpression>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__187(
                self as *const crate::QDebug,
                re.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QRegularExpressionMatch {
    /// <p>Destroys the match result.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QRegularExpressionMatch::~QRegularExpressionMatch()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatch.html#dtor.QRegularExpressionMatch">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the match result.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatch_dQRegularExpressionMatch(
            self as *mut crate::QRegularExpressionMatch,
        )
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QRegularExpressionMatch>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QRegularExpressionMatch& match)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        match_: ::cpp_utils::Ref<crate::QRegularExpressionMatch>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__188(
                self as *const crate::QDebug,
                match_.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QRegularExpressionMatchIterator {
    /// <p>Destroys the <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QRegularExpressionMatchIterator::~QRegularExpressionMatchIterator()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html#dtor.QRegularExpressionMatchIterator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the <a href="http://doc.qt.io/qt-5/qregularexpressionmatchiterator.html">QRegularExpressionMatchIterator</a> object.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QRegularExpressionMatchIterator_dQRegularExpressionMatchIterator(self as *mut crate::QRegularExpressionMatchIterator)
    }
}

impl ::cpp_utils::CppDeletable for crate::QResource {
    /// <p>Releases the resources of the <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QResource::~QResource()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qresource.html#dtor.QResource">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Releases the resources of the <a href="http://doc.qt.io/qt-5/qresource.html">QResource</a> object.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QResource_dQResource(self as *mut crate::QResource)
    }
}

impl ::cpp_utils::CppDeletable for crate::QSaveFile {
    /// <p>Destroys the file object, discarding the saved contents unless <a href="http://doc.qt.io/qt-5/qsavefile.html#commit">commit</a>() was called.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QSaveFile::~QSaveFile()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsavefile.html#dtor.QSaveFile">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the file object, discarding the saved contents unless <a href="http://doc.qt.io/qt-5/qsavefile.html#commit">commit</a>() was called.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSaveFile_dQSaveFile(self as *mut crate::QSaveFile)
    }
}

impl ::cpp_utils::CppDeletable for crate::QSemaphore {
    /// <p>Destroys the semaphore.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QSemaphore::~QSemaphore()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsemaphore.html#dtor.QSemaphore">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the semaphore.</p>
    /// <p><b>Warning:</b> Destroying a semaphore that is in use may result in undefined behavior.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSemaphore_dQSemaphore(self as *mut crate::QSemaphore)
    }
}

impl ::cpp_utils::CppDeletable for crate::QSequentialAnimationGroup {
    /// <p>Destroys the animation group. It will also destroy all its animations.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QSequentialAnimationGroup::~QSequentialAnimationGroup()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsequentialanimationgroup.html#dtor.QSequentialAnimationGroup">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the animation group. It will also destroy all its animations.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSequentialAnimationGroup_dQSequentialAnimationGroup(
            self as *mut crate::QSequentialAnimationGroup,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QSettings {
    /// <p>Destroys the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QSettings::~QSettings()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsettings.html#dtor.QSettings">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the <a href="http://doc.qt.io/qt-5/qsettings.html">QSettings</a> object.</p>
    /// <p>Any unsaved changes will eventually be written to permanent storage.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsettings.html#sync">sync</a>().</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSettings_dQSettings(self as *mut crate::QSettings)
    }
}

impl ::cpp_utils::CppDeletable for crate::QSharedMemory {
    /// <p>The destructor clears the key, which forces the shared memory object to <a href="http://doc.qt.io/qt-5/qsharedmemory.html#detach">detach</a> from its underlying shared memory segment. If this shared memory object is the last one connected to the shared memory segment, the <a href="http://doc.qt.io/qt-5/qsharedmemory.html#detach">detach</a>() operation destroys the shared memory segment.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QSharedMemory::~QSharedMemory()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsharedmemory.html#dtor.QSharedMemory">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The destructor clears the key, which forces the shared memory object to <a href="http://doc.qt.io/qt-5/qsharedmemory.html#detach">detach</a> from its underlying shared memory segment. If this shared memory object is the last one connected to the shared memory segment, the <a href="http://doc.qt.io/qt-5/qsharedmemory.html#detach">detach</a>() operation destroys the shared memory segment.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qsharedmemory.html#detach">detach</a>() and <a href="http://doc.qt.io/qt-5/qsharedmemory.html#isAttached">isAttached</a>().</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSharedMemory_dQSharedMemory(
            self as *mut crate::QSharedMemory,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QSignalMapper {
    /// <p>Destroys the <a href="http://doc.qt.io/qt-5/qsignalmapper.html">QSignalMapper</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QSignalMapper::~QSignalMapper()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignalmapper.html#dtor.QSignalMapper">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the <a href="http://doc.qt.io/qt-5/qsignalmapper.html">QSignalMapper</a>.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSignalMapper_dQSignalMapper(
            self as *mut crate::QSignalMapper,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QSignalTransition {
    /// <p>Destroys this signal transition.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QSignalTransition::~QSignalTransition()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsignaltransition.html#dtor.QSignalTransition">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys this signal transition.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSignalTransition_dQSignalTransition(
            self as *mut crate::QSignalTransition,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QSocketNotifier {
    /// <p>Destroys this socket notifier.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QSocketNotifier::~QSocketNotifier()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsocketnotifier.html#dtor.QSocketNotifier">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys this socket notifier.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSocketNotifier_dQSocketNotifier(
            self as *mut crate::QSocketNotifier,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QSortFilterProxyModel {
    /// <p>Destroys this sorting filter model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QSortFilterProxyModel::~QSortFilterProxyModel()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsortfilterproxymodel.html#dtor.QSortFilterProxyModel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys this sorting filter model.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSortFilterProxyModel_dQSortFilterProxyModel(
            self as *mut crate::QSortFilterProxyModel,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QState {
    /// <p>Destroys this state.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QState::~QState()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstate.html#dtor.QState">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys this state.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QState_dQState(self as *mut crate::QState)
    }
}

impl ::cpp_utils::CppDeletable for crate::QStateMachine {
    /// <p>Destroys this state machine.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QStateMachine::~QStateMachine()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstatemachine.html#dtor.QStateMachine">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys this state machine.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QStateMachine_dQStateMachine(
            self as *mut crate::QStateMachine,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QStorageInfo {
    /// <p>Destroys the <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> object and frees its resources.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QStorageInfo::~QStorageInfo()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstorageinfo.html#dtor.QStorageInfo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the <a href="http://doc.qt.io/qt-5/qstorageinfo.html">QStorageInfo</a> object and frees its resources.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QStorageInfo_dQStorageInfo(self as *mut crate::QStorageInfo)
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QStorageInfo>> for crate::QStorageInfo {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(const QStorageInfo& first, const QStorageInfo& second)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, second: &::cpp_utils::Ref<crate::QStorageInfo>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__189(
                self as *const crate::QStorageInfo,
                second.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QSystemSemaphore {
    /// <p>The destructor destroys the <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html">QSystemSemaphore</a> object, but the underlying system semaphore is not removed from the system unless this instance of <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html">QSystemSemaphore</a> is the last one existing for that system semaphore.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QSystemSemaphore::~QSystemSemaphore()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#dtor.QSystemSemaphore">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The destructor destroys the <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html">QSystemSemaphore</a> object, but the underlying system semaphore is not removed from the system unless this instance of <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html">QSystemSemaphore</a> is the last one existing for that system semaphore.</p>
    /// <p>Two important side effects of the destructor depend on the system. In Windows, if <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#acquire">acquire</a>() has been called for this semaphore but not <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#release">release</a>(), <a href="http://doc.qt.io/qt-5/qsystemsemaphore.html#release">release</a>() will not be called by the destructor, nor will the resource be released when the process exits normally. This would be a program bug which could be the cause of a deadlock in another process trying to acquire the same resource. In Unix, acquired resources that are not released before the destructor is called are automatically released when the process exits.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSystemSemaphore_dQSystemSemaphore(
            self as *mut crate::QSystemSemaphore,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QTemporaryDir {
    /// <p>Destroys the temporary directory object. If auto remove mode was set, it will automatically delete the directory including all its contents.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QTemporaryDir::~QTemporaryDir()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtemporarydir.html#dtor.QTemporaryDir">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the temporary directory object. If auto remove mode was set, it will automatically delete the directory including all its contents.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtemporarydir.html#autoRemove">autoRemove</a>().</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QTemporaryDir_dQTemporaryDir(
            self as *mut crate::QTemporaryDir,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QTemporaryFile {
    /// <p>Destroys the temporary file object, the file is automatically closed if necessary and if in auto remove mode it will automatically delete the file.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QTemporaryFile::~QTemporaryFile()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtemporaryfile.html#dtor.QTemporaryFile">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the temporary file object, the file is automatically closed if necessary and if in auto remove mode it will automatically delete the file.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtemporaryfile.html#autoRemove">autoRemove</a>().</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QTemporaryFile_dQTemporaryFile(
            self as *mut crate::QTemporaryFile,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QTextBoundaryFinder {
    /// <p>Destructs the <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QTextBoundaryFinder::~QTextBoundaryFinder()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html#dtor.QTextBoundaryFinder">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destructs the <a href="http://doc.qt.io/qt-5/qtextboundaryfinder.html">QTextBoundaryFinder</a> object.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QTextBoundaryFinder_dQTextBoundaryFinder(
            self as *mut crate::QTextBoundaryFinder,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QTextEncoder {
    /// <p>Destroys the encoder.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QTextEncoder::~QTextEncoder()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextencoder.html#dtor.QTextEncoder">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the encoder.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QTextEncoder_dQTextEncoder(self as *mut crate::QTextEncoder)
    }
}

impl ::cpp_utils::CppDeletable for crate::QTextDecoder {
    /// <p>Destroys the decoder.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QTextDecoder::~QTextDecoder()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextdecoder.html#dtor.QTextDecoder">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the decoder.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QTextDecoder_dQTextDecoder(self as *mut crate::QTextDecoder)
    }
}

impl ::cpp_utils::CppDeletable for crate::QThread {
    /// <p>Destroys the <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QThread::~QThread()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthread.html#dtor.QThread">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a>.</p>
    /// <p>Note that deleting a <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a> object will not stop the execution of the thread it manages. Deleting a running <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a> (i.e. <a href="http://doc.qt.io/qt-5/qthread.html#isFinished">isFinished</a>() returns <code>false</code>) will result in a program crash. Wait for the <a href="http://doc.qt.io/qt-5/qthread.html#finished">finished</a>() signal before deleting the <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a>.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QThread_dQThread(self as *mut crate::QThread)
    }
}

impl ::cpp_utils::CppDeletable for crate::QThreadPool {
    /// <p>Destroys the <a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a>. This function will block until all runnables have been completed.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QThreadPool::~QThreadPool()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qthreadpool.html#dtor.QThreadPool">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the <a href="http://doc.qt.io/qt-5/qthreadpool.html">QThreadPool</a>. This function will block until all runnables have been completed.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QThreadPool_dQThreadPool(self as *mut crate::QThreadPool)
    }
}

impl ::cpp_utils::CppDeletable for crate::QTimeLine {
    /// <p>Destroys the timeline.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QTimeLine::~QTimeLine()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimeline.html#dtor.QTimeLine">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the timeline.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QTimeLine_dQTimeLine(self as *mut crate::QTimeLine)
    }
}

impl ::cpp_utils::CppDeletable for crate::QTimer {
    /// <p>Destroys the timer.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QTimer::~QTimer()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimer.html#dtor.QTimer">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the timer.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QTimer_dQTimer(self as *mut crate::QTimer)
    }
}

impl ::cpp_utils::CppDeletable for crate::QTimeZone {
    /// <p>Destroys the time zone.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QTimeZone::~QTimeZone()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#dtor.QTimeZone">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the time zone.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QTimeZone_dQTimeZone(self as *mut crate::QTimeZone)
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QTimeZone>> for crate::QTimeZone {
    /// <p>Returns <code>true</code> if this time zone is equal to the <i>other</i> time zone.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QTimeZone::operator==(const QTimeZone& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtimezone.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this time zone is equal to the <i>other</i> time zone.</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QTimeZone>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QTimeZone_operator__(
                self as *const crate::QTimeZone,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QTimeZone>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& ds, const QTimeZone& tz)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        tz: ::cpp_utils::Ref<crate::QTimeZone>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__191(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                tz.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QTimeZone>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& ds, QTimeZone& tz)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        tz: ::cpp_utils::MutRef<crate::QTimeZone>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__192(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                tz.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QTimeZone>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug dbg, const QTimeZone& tz)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, tz: ::cpp_utils::Ref<crate::QTimeZone>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__193(
                self as *const crate::QDebug,
                tz.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QTranslator {
    /// <p>Destroys the object and frees any allocated resources.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QTranslator::~QTranslator()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtranslator.html#dtor.QTranslator">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the object and frees any allocated resources.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QTranslator_dQTranslator(self as *mut crate::QTranslator)
    }
}

impl ::cpp_utils::CppDeletable for crate::QUrl {
    /// <p>Destructor; called immediately before the object is deleted.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QUrl::~QUrl()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#dtor.QUrl">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destructor; called immediately before the object is deleted.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QUrl_dQUrl(self as *mut crate::QUrl)
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QUrl>> for crate::QUrl {
    /// Calls C++ function: <span style='color: green;'>```bool QUrl::operator<(const QUrl& url) const```</span>.
    fn lt(&self, url: &::cpp_utils::Ref<crate::QUrl>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QUrl_operator_3(
                self as *const crate::QUrl,
                url.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QUrl>> for crate::QUrl {
    /// <p>Returns <code>true</code> if this URL and the given <i>url</i> are equal; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QUrl::operator==(const QUrl& url) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurl.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this URL and the given <i>url</i> are equal; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, url: &::cpp_utils::Ref<crate::QUrl>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QUrl_operator__(
                self as *const crate::QUrl,
                url.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::BitOrAssign<crate::QFlags<crate::q_url::ComponentFormattingOption>>
    for crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption
{
    /// Calls C++ function: <span style='color: green;'>```QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>& operator|=(QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>& i, QFlags<QUrl::ComponentFormattingOption> f)```</span>.
    fn bitor_assign(&mut self, f: crate::QFlags<crate::q_url::ComponentFormattingOption>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__194(
                self as *mut crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption,
                f.to_int(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::BitOr<crate::QFlags<crate::q_url::ComponentFormattingOption>>
    for &crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption
{
    type Output =
        ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>;
    /// Calls C++ function: <span style='color: green;'>```QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption> operator|(QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption> i, QFlags<QUrl::ComponentFormattingOption> f)```</span>.
    fn bitor(
        self,
        f: crate::QFlags<crate::q_url::ComponentFormattingOption>,
    ) -> ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>
    {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_133(
                self as *const crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption,
                f.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QUrl>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, const QUrl& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QUrl>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__195(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QUrl>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QUrl& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QUrl>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__196(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QUrl>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QUrl& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QUrl>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__197(
                self as *const crate::QDebug,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QUrlQuery {
    /// <p>Destroys this <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QUrlQuery::~QUrlQuery()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#dtor.QUrlQuery">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys this <a href="http://doc.qt.io/qt-5/qurlquery.html">QUrlQuery</a> object.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QUrlQuery_dQUrlQuery(self as *mut crate::QUrlQuery)
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QUrlQuery>> for crate::QUrlQuery {
    /// <p>Returns <code>true</code> if this object and the <i>other</i> object contain the same contents, in the same order, and use the same query delimiters.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QUrlQuery::operator==(const QUrlQuery& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qurlquery.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this object and the <i>other</i> object contain the same contents, in the same order, and use the same query delimiters.</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QUrlQuery>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QUrlQuery_operator__(
                self as *const crate::QUrlQuery,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QUuid>> for crate::QUuid {
    /// <p>Returns <code>true</code> if this <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> and the <i>other</i> <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> are identical; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QUuid::operator==(const QUuid& orig) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> and the <i>other</i> <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> are identical; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, orig: &::cpp_utils::Ref<crate::QUuid>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QUuid_operator__(
                self as *const crate::QUuid,
                orig.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QUuid>> for crate::QUuid {
    /// <p>Returns <code>true</code> if this <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> has the same <a href="http://doc.qt.io/qt-5/quuid.html#variant-field">variant field</a> as the <i>other</i> <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> and is lexicographically <i>before</i> the <i>other</i> <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>. If the <i>other</i> <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> has a different variant field, the return value is determined by comparing the two <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">variants</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QUuid::operator<(const QUuid& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> has the same <a href="http://doc.qt.io/qt-5/quuid.html#variant-field">variant field</a> as the <i>other</i> <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> and is lexicographically <i>before</i> the <i>other</i> <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>. If the <i>other</i> <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> has a different variant field, the return value is determined by comparing the two <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">variants</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/quuid.html#variant">variant</a>().</p></div>
    fn lt(&self, other: &::cpp_utils::Ref<crate::QUuid>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QUuid_operator_(
                self as *const crate::QUuid,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QUuid>> for crate::QUuid {
    /// <p>Returns <code>true</code> if this <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> has the same <a href="http://doc.qt.io/qt-5/quuid.html#variant-field">variant field</a> as the <i>other</i> <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> and is lexicographically <i>after</i> the <i>other</i> <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>. If the <i>other</i> <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> has a different variant field, the return value is determined by comparing the two <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">variants</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QUuid::operator>(const QUuid& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html#operator-gt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if this <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> has the same <a href="http://doc.qt.io/qt-5/quuid.html#variant-field">variant field</a> as the <i>other</i> <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> and is lexicographically <i>after</i> the <i>other</i> <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>. If the <i>other</i> <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> has a different variant field, the return value is determined by comparing the two <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">variants</a>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/quuid.html#variant">variant</a>().</p></div>
    fn gt(&self, other: &::cpp_utils::Ref<crate::QUuid>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QUuid_operator_1(
                self as *const crate::QUuid,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QUuid>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, const QUuid& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QUuid>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__198(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QUuid>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QUuid& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QUuid>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__199(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QUuid>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QUuid& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QUuid>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__200(
                self as *const crate::QDebug,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QUuid>> for crate::QUuid {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QUuid& lhs, const QUuid& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QUuid>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__201(
                self as *const crate::QUuid,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QUuid>> for crate::QUuid {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QUuid& lhs, const QUuid& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QUuid>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__202(
                self as *const crate::QUuid,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QWaitCondition {
    /// <p>Destroys the wait condition object.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QWaitCondition::~QWaitCondition()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qwaitcondition.html#dtor.QWaitCondition">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the wait condition object.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QWaitCondition_dQWaitCondition(
            self as *mut crate::QWaitCondition,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QXmlStreamStringRef {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QXmlStreamStringRef::~QXmlStreamStringRef()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamStringRef_dQXmlStreamStringRef(
            self as *mut crate::QXmlStreamStringRef,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QXmlStreamAttribute {
    /// <p>Destructs an attribute.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QXmlStreamAttribute::~QXmlStreamAttribute()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html#dtor.QXmlStreamAttribute">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destructs an attribute.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamAttribute_dQXmlStreamAttribute(
            self as *mut crate::QXmlStreamAttribute,
        )
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QXmlStreamAttribute>>
    for crate::QXmlStreamAttribute
{
    /// <p>Compares this attribute with <i>other</i> and returns <code>true</code> if they are equal; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QXmlStreamAttribute::operator==(const QXmlStreamAttribute& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares this attribute with <i>other</i> and returns <code>true</code> if they are equal; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QXmlStreamAttribute>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QXmlStreamAttribute_operator__(
                self as *const crate::QXmlStreamAttribute,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QXmlStreamNamespaceDeclaration {
    /// <p>Destructs this namespace declaration.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QXmlStreamNamespaceDeclaration::~QXmlStreamNamespaceDeclaration()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamnamespacedeclaration.html#dtor.QXmlStreamNamespaceDeclaration">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destructs this namespace declaration.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamNamespaceDeclaration_dQXmlStreamNamespaceDeclaration(
            self as *mut crate::QXmlStreamNamespaceDeclaration,
        )
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>
    for crate::QXmlStreamNamespaceDeclaration
{
    /// <p>Compares this namespace declaration with <i>other</i> and returns <code>true</code> if they are equal; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QXmlStreamNamespaceDeclaration::operator==(const QXmlStreamNamespaceDeclaration& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamnamespacedeclaration.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares this namespace declaration with <i>other</i> and returns <code>true</code> if they are equal; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QXmlStreamNamespaceDeclaration_operator__(
                self as *const crate::QXmlStreamNamespaceDeclaration,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QXmlStreamNotationDeclaration {
    /// <p>Destructs this notation declaration.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QXmlStreamNotationDeclaration::~QXmlStreamNotationDeclaration()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamnotationdeclaration.html#dtor.QXmlStreamNotationDeclaration">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destructs this notation declaration.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamNotationDeclaration_dQXmlStreamNotationDeclaration(
            self as *mut crate::QXmlStreamNotationDeclaration,
        )
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>
    for crate::QXmlStreamNotationDeclaration
{
    /// <p>Compares this notation declaration with <i>other</i> and returns <code>true</code> if they are equal; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QXmlStreamNotationDeclaration::operator==(const QXmlStreamNotationDeclaration& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamnotationdeclaration.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares this notation declaration with <i>other</i> and returns <code>true</code> if they are equal; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QXmlStreamNotationDeclaration_operator__(
                self as *const crate::QXmlStreamNotationDeclaration,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QXmlStreamEntityDeclaration {
    /// <p>Destructs this entity declaration.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QXmlStreamEntityDeclaration::~QXmlStreamEntityDeclaration()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamentitydeclaration.html#dtor.QXmlStreamEntityDeclaration">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destructs this entity declaration.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamEntityDeclaration_dQXmlStreamEntityDeclaration(
            self as *mut crate::QXmlStreamEntityDeclaration,
        )
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>
    for crate::QXmlStreamEntityDeclaration
{
    /// <p>Compares this entity declaration with <i>other</i> and returns <code>true</code> if they are equal; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QXmlStreamEntityDeclaration::operator==(const QXmlStreamEntityDeclaration& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamentitydeclaration.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares this entity declaration with <i>other</i> and returns <code>true</code> if they are equal; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QXmlStreamEntityDeclaration_operator__(
                self as *const crate::QXmlStreamEntityDeclaration,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QXmlStreamEntityResolver {
    /// <p>Destroys the entity resolver.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QXmlStreamEntityResolver::~QXmlStreamEntityResolver()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamentityresolver.html#dtor.QXmlStreamEntityResolver">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the entity resolver.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamEntityResolver_dQXmlStreamEntityResolver(
            self as *mut crate::QXmlStreamEntityResolver,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QXmlStreamReader {
    /// <p>Destructs the reader.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QXmlStreamReader::~QXmlStreamReader()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#dtor.QXmlStreamReader">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destructs the reader.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamReader_dQXmlStreamReader(
            self as *mut crate::QXmlStreamReader,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QXmlStreamWriter {
    /// <p>Destructor.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QXmlStreamWriter::~QXmlStreamWriter()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#dtor.QXmlStreamWriter">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destructor.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamWriter_dQXmlStreamWriter(
            self as *mut crate::QXmlStreamWriter,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QSysInfo {
    /// <p>The <a href="http://doc.qt.io/qt-5/qsysinfo.html">QSysInfo</a> class provides information about the system.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QSysInfo::~QSysInfo()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsysinfo.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsysinfo.html">QSysInfo</a> class provides information about the system.</p>
    /// <ul>
    /// <li><a href="http://doc.qt.io/qt-5/qsysinfo.html#Sizes-enum">WordSize</a> specifies the size of a pointer for the platform on which the application is compiled.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qsysinfo.html#Endian-enum">ByteOrder</a> specifies whether the platform is big-endian or little-endian.</li>
    /// </ul>
    /// <p>Some constants are defined only on certain platforms. You can use the preprocessor symbols <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_OS_WIN">Q_OS_WIN</a> and <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_OS_MACOS">Q_OS_MACOS</a> to test that the application is compiled under Windows or <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSysInfo_dQSysInfo(self as *mut crate::QSysInfo)
    }
}

impl ::cpp_utils::CppDeletable for crate::QMessageLogContext {
    /// <p>The <a href="http://doc.qt.io/qt-5/qmessagelogcontext.html">QMessageLogContext</a> class provides additional information about a log message.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QMessageLogContext::~QMessageLogContext()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmessagelogcontext.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmessagelogcontext.html">QMessageLogContext</a> class provides additional information about a log message.</p>
    /// <p>The class provides information about the source code location a <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>(), <a href="http://doc.qt.io/qt-5/qtglobal.html#qInfo">qInfo</a>(), <a href="http://doc.qt.io/qt-5/qtglobal.html#qWarning">qWarning</a>(), <a href="http://doc.qt.io/qt-5/qtglobal.html#qCritical">qCritical</a>() or <a href="http://doc.qt.io/qt-5/qtglobal.html#qFatal">qFatal</a>() message was generated.</p>
    /// <p><b>Note: </b>By default, this information is recorded only in debug builds. You can overwrite this explicitly by defining <code>QT_MESSAGELOGCONTEXT</code> or <code>QT_NO_MESSAGELOGCONTEXT</code>.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMessageLogContext_dQMessageLogContext(
            self as *mut crate::QMessageLogContext,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QMessageLogger {
    /// <p>The <a href="http://doc.qt.io/qt-5/qmessagelogger.html">QMessageLogger</a> class generates log messages.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QMessageLogger::~QMessageLogger()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmessagelogger.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmessagelogger.html">QMessageLogger</a> class generates log messages.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qmessagelogger.html">QMessageLogger</a> is used to generate messages for the Qt logging framework. Usually one uses it through <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>(), <a href="http://doc.qt.io/qt-5/qtglobal.html#qInfo">qInfo</a>(), <a href="http://doc.qt.io/qt-5/qtglobal.html#qWarning">qWarning</a>(), <a href="http://doc.qt.io/qt-5/qtglobal.html#qCritical">qCritical</a>, or <a href="http://doc.qt.io/qt-5/qtglobal.html#qFatal">qFatal</a>() functions, which are actually macros: For example <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() expands to <a href="http://doc.qt.io/qt-5/qmessagelogger.html">QMessageLogger</a>(__FILE__, __LINE__, <a href="http://doc.qt.io/qt-5/qtglobal.html#Q_FUNC_INFO">Q_FUNC_INFO</a>).<a href="http://doc.qt.io/qt-5/qmessagelogger.html#debug-3">debug</a>() for debug builds, and <a href="http://doc.qt.io/qt-5/qmessagelogger.html">QMessageLogger</a>(0, 0, 0).<a href="http://doc.qt.io/qt-5/qmessagelogger.html#debug-3">debug</a>() for release builds.</p>
    /// <p>One example of direct use is to forward errors that stem from a scripting language, e.g. QML:</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="type">void</span> statusChanged(<span class="type">QDeclarativeComponent</span><span class="operator">::</span>Status status) {
    /// &#32;   &#32;     <span class="keyword">if</span> (status <span class="operator">=</span><span class="operator">=</span> <span class="type">QDeclarativeComponent</span><span class="operator">::</span>Error) {
    /// &#32;   &#32;   &#32;     foreach (<span class="keyword">const</span> <span class="type">QDeclarativeError</span> <span class="operator">&amp;</span>error<span class="operator">,</span> component<span class="operator">-</span><span class="operator">&gt;</span>errors()) {
    /// &#32;   &#32;   &#32;   &#32;     <span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a></span> file <span class="operator">=</span> error<span class="operator">.</span>url()<span class="operator">.</span>toEncoded();
    /// &#32;   &#32;   &#32;   &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qmessagelogger.html#QMessageLogger">QMessageLogger</a></span>(file<span class="operator">.</span>constData()<span class="operator">,</span> error<span class="operator">.</span>line()<span class="operator">,</span> <span class="number">0</span>)<span class="operator">.</span>debug() <span class="operator">&lt;</span><span class="operator">&lt;</span> error<span class="operator">.</span>description();
    /// &#32;   &#32;   &#32;     }
    /// &#32;   &#32;     }
    /// &#32;     }
    ///
    /// </pre></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMessageLogger_dQMessageLogger(
            self as *mut crate::QMessageLogger,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QMetaObject {
    /// <p>The <a href="http://doc.qt.io/qt-5/qmetaobject.html">QMetaObject</a> class contains meta-information about Qt objects.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QMetaObject::~QMetaObject()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaobject.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmetaobject.html">QMetaObject</a> class contains meta-information about Qt objects.</p>
    /// <p>The Qt <a href="http://doc.qt.io/qt-5/metaobjects.html">Meta-Object System</a> in Qt is responsible for the signals and slots inter-object communication mechanism, runtime type information, and the Qt property system. A single <a href="http://doc.qt.io/qt-5/qmetaobject.html">QMetaObject</a> instance is created for each <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> subclass that is used in an application, and this instance stores all the meta-information for the <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> subclass. This object is available as <a href="http://doc.qt.io/qt-5/qobject.html#metaObject">QObject::metaObject</a>().</p>
    /// <p>This class is not normally required for application programming, but it is useful if you write meta-applications, such as scripting engines or GUI builders.</p>
    /// <p>The functions you are most likely to find useful are these:</p>
    /// <ul>
    /// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#className">className</a>() returns the name of a class.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#superClass">superClass</a>() returns the superclass's meta-object.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#method">method</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#methodCount">methodCount</a>() provide information about a class's meta-methods (signals, slots and other <a href="http://doc.qt.io/qt-5/qobject.html#Q_INVOKABLE">invokable</a> member functions).</li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#enumerator">enumerator</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#enumeratorCount">enumeratorCount</a>() and provide information about a class's enumerators.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#propertyCount">propertyCount</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#property">property</a>() provide information about a class's properties.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qmetaobject.html#constructor">constructor</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#constructorCount">constructorCount</a>() provide information about a class's meta-constructors.</li>
    /// </ul>
    /// <p>The index functions <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfConstructor">indexOfConstructor</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfMethod">indexOfMethod</a>(), <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfEnumerator">indexOfEnumerator</a>(), and <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfProperty">indexOfProperty</a>() map names of constructors, member functions, enumerators, or properties to indexes in the meta-object. For example, Qt uses <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfMethod">indexOfMethod</a>() internally when you connect a signal to a slot.</p>
    /// <p>Classes can also have a list of <i>name</i>--<i>value</i> pairs of additional class information, stored in <a href="http://doc.qt.io/qt-5/qmetaclassinfo.html">QMetaClassInfo</a> objects. The number of pairs is returned by <a href="http://doc.qt.io/qt-5/qmetaobject.html#classInfoCount">classInfoCount</a>(), single pairs are returned by <a href="http://doc.qt.io/qt-5/qmetaobject.html#classInfo">classInfo</a>(), and you can search for pairs with <a href="http://doc.qt.io/qt-5/qmetaobject.html#indexOfClassInfo">indexOfClassInfo</a>().</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMetaObject_dQMetaObject(self as *mut crate::QMetaObject)
    }
}

impl ::cpp_utils::CppDeletable for crate::QLatin1Char {
    /// <p>The <a href="http://doc.qt.io/qt-5/qlatin1char.html">QLatin1Char</a> class provides an 8-bit ASCII/Latin-1 character.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QLatin1Char::~QLatin1Char()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1char.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlatin1char.html">QLatin1Char</a> class provides an 8-bit ASCII/Latin-1 character.</p>
    /// <p>This class is only useful to construct a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> with 8-bit character.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QLatin1Char_dQLatin1Char(self as *mut crate::QLatin1Char)
    }
}

impl ::cpp_utils::CppDeletable for crate::QChar {
    /// <p>The <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> class provides a 16-bit Unicode character.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QChar::~QChar()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qchar.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> class provides a 16-bit Unicode character.</p>
    /// <p>In Qt, Unicode characters are 16-bit entities without any markup or structure. This class represents such an entity. It is lightweight, so it can be used everywhere. Most compilers treat it like an <code>unsigned short</code>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> provides a full complement of testing/classification functions, converting to and from other formats, converting from composed to decomposed Unicode, and trying to compare and case-convert if you ask it to.</p>
    /// <p>The classification functions include functions like those in the standard C++ header &lt;cctype&gt; (formerly &lt;ctype.h&gt;), but operating on the full range of Unicode characters, not just for the ASCII range. They all return true if the character is a certain type of character; otherwise they return false. These classification functions are <a href="http://doc.qt.io/qt-5/qchar.html#isNull">isNull</a>() (returns <code>true</code> if the character is '\0'), <a href="http://doc.qt.io/qt-5/qchar.html#isPrint">isPrint</a>() (true if the character is any sort of printable character, including whitespace), <a href="http://doc.qt.io/qt-5/qchar.html#isPunct">isPunct</a>() (any sort of punctation), <a href="http://doc.qt.io/qt-5/qchar.html#isMark">isMark</a>() (Unicode Mark), <a href="http://doc.qt.io/qt-5/qchar.html#isLetter">isLetter</a>() (a letter), <a href="http://doc.qt.io/qt-5/qchar.html#isNumber">isNumber</a>() (any sort of numeric character, not just 0-9), <a href="http://doc.qt.io/qt-5/qchar.html#isLetterOrNumber">isLetterOrNumber</a>(), and <a href="http://doc.qt.io/qt-5/qchar.html#isDigit">isDigit</a>() (decimal digits). All of these are wrappers around <a href="http://doc.qt.io/qt-5/qchar.html#category">category</a>() which return the Unicode-defined category of each character. Some of these also calculate the derived properties (for example <a href="http://doc.qt.io/qt-5/qchar.html#isSpace">isSpace</a>() returns <code>true</code> if the character is of category Separator_* or an exceptional code point from <a href="http://doc.qt.io/qt-5/qchar.html#Category-enum">Other_Control</a> category).</p>
    /// <p><a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> also provides <a href="http://doc.qt.io/qt-5/qchar.html#direction">direction</a>(), which indicates the "natural" writing direction of this character. The <a href="http://doc.qt.io/qt-5/qchar.html#joiningType">joiningType</a>() function indicates how the character joins with it's neighbors (needed mostly for Arabic or Syriac) and finally <a href="http://doc.qt.io/qt-5/qchar.html#hasMirrored">hasMirrored</a>(), which indicates whether the character needs to be mirrored when it is printed in it's "unnatural" writing direction.</p>
    /// <p>Composed Unicode characters (like <i>ring</i>) can be converted to decomposed Unicode ("a" followed by "ring above") by using <a href="http://doc.qt.io/qt-5/qchar.html#decomposition">decomposition</a>().</p>
    /// <p>In Unicode, comparison is not necessarily possible and case conversion is very difficult at best. Unicode, covering the "entire" world, also includes most of the world's case and sorting problems. operator==() and friends will do comparison based purely on the numeric Unicode value (code point) of the characters, and <a href="http://doc.qt.io/qt-5/qchar.html#toUpper">toUpper</a>() and <a href="http://doc.qt.io/qt-5/qchar.html#toLower">toLower</a>() will do case changes when the character has a well-defined uppercase/lowercase equivalent. For locale-dependent comparisons, use <a href="http://doc.qt.io/qt-5/qstring.html#localeAwareCompare">QString::localeAwareCompare</a>().</p>
    /// <p>The conversion functions include <a href="http://doc.qt.io/qt-5/qchar.html#unicode">unicode</a>() (to a scalar), <a href="http://doc.qt.io/qt-5/qchar.html#toLatin1">toLatin1</a>() (to scalar, but converts all non-Latin-1 characters to 0), <a href="http://doc.qt.io/qt-5/qchar.html#row">row</a>() (gives the Unicode row), <a href="http://doc.qt.io/qt-5/qchar.html#cell">cell</a>() (gives the Unicode cell), <a href="http://doc.qt.io/qt-5/qchar.html#digitValue">digitValue</a>() (gives the integer value of any of the numerous digit characters), and a host of constructors.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> provides constructors and cast operators that make it easy to convert to and from traditional 8-bit <code>char</code>s. If you defined <code>QT_NO_CAST_FROM_ASCII</code> and <code>QT_NO_CAST_TO_ASCII</code>, as explained in the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> documentation, you will need to explicitly call <a href="http://doc.qt.io/qt-5/qchar.html#fromLatin1">fromLatin1</a>(), or use <a href="http://doc.qt.io/qt-5/qlatin1char.html">QLatin1Char</a>, to construct a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> from an 8-bit <code>char</code>, and you will need to call <a href="http://doc.qt.io/qt-5/qchar.html#toLatin1">toLatin1</a>() to get the 8-bit value back.</p>
    /// <p>For more information see <a href="http://www.unicode.org/ucd/">"About the Unicode Character Database"</a>.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QChar_dQChar(self as *mut crate::QChar)
    }
}

impl ::cpp_utils::CppDeletable for crate::QByteRef {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QByteRef::~QByteRef()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QByteRef_dQByteRef(self as *mut crate::QByteRef)
    }
}

impl ::cpp_utils::CppDeletable for crate::QLatin1String {
    /// <p>The <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> class provides a thin wrapper around an US-ASCII/Latin-1 encoded string literal.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QLatin1String::~QLatin1String()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> class provides a thin wrapper around an US-ASCII/Latin-1 encoded string literal.</p>
    /// <p>Many of <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>'s member functions are overloaded to accept <code>const char *</code> instead of <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>. This includes the copy constructor, the assignment operator, the comparison operators, and various other functions such as <a href="http://doc.qt.io/qt-5/qstring.html#insert">insert()</a>, <a href="http://doc.qt.io/qt-5/qstring.html#replace">replace()</a>, and <a href="http://doc.qt.io/qt-5/qstring.html#indexOf">indexOf()</a>. These functions are usually optimized to avoid constructing a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> object for the <code>const char *</code> data. For example, assuming <code>str</code> is a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>,</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">if</span> (str <span class="operator">=</span><span class="operator">=</span> <span class="string">"auto"</span> <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> <span class="string">"extern"</span>
    /// &#32;   &#32;     <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> <span class="string">"static"</span> <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> <span class="string">"register"</span>) {
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   }
    ///
    /// </pre>
    /// <p>is much faster than</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">if</span> (str <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>(<span class="string">"auto"</span>) <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>(<span class="string">"extern"</span>)
    /// &#32;   &#32;     <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>(<span class="string">"static"</span>) <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span>(<span class="string">"register"</span>)) {
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   }
    ///
    /// </pre>
    /// <p>because it doesn't construct four temporary <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> objects and make a deep copy of the character data.</p>
    /// <p>Applications that define <code>QT_NO_CAST_FROM_ASCII</code> (as explained in the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> documentation) don't have access to <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>'s <code>const char *</code> API. To provide an efficient way of specifying constant Latin-1 strings, Qt provides the <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>, which is just a very thin wrapper around a <code>const char *</code>. Using <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>, the example code above becomes</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">if</span> (str <span class="operator">=</span><span class="operator">=</span> QLatin1String(<span class="string">"auto"</span>)
    /// &#32;   &#32;     <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> QLatin1String(<span class="string">"extern"</span>)
    /// &#32;   &#32;     <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> QLatin1String(<span class="string">"static"</span>)
    /// &#32;   &#32;     <span class="operator">|</span><span class="operator">|</span> str <span class="operator">=</span><span class="operator">=</span> QLatin1String(<span class="string">"register"</span>) {
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   }
    ///
    /// </pre>
    /// <p>This is a bit longer to type, but it provides exactly the same benefits as the first version of the code, and is faster than converting the Latin-1 strings using <a href="http://doc.qt.io/qt-5/qstring.html#fromLatin1">QString::fromLatin1</a>().</p>
    /// <p>Thanks to the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>(<a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>) constructor, <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> can be used everywhere a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> is expected. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlabel.html">QLabel</a></span> <span class="operator">*</span>label <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qlabel.html">QLabel</a></span>(QLatin1String(<span class="string">"MOD"</span>)<span class="operator">,</span> <span class="keyword">this</span>);
    ///
    /// </pre>
    /// <p><b>Note: </b>If the function you're calling with a <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> argument isn't actually overloaded to take <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>, the implicit conversion to <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> will trigger a memory allocation, which is usually what you want to avoid by using <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> in the first place. In those cases, using <a href="http://doc.qt.io/qt-5/qstring.html#QStringLiteral">QStringLiteral</a> may be the better option.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QLatin1String_dQLatin1String(
            self as *mut crate::QLatin1String,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QCharRef {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QCharRef::~QCharRef()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCharRef_dQCharRef(self as *mut crate::QCharRef)
    }
}

impl ::cpp_utils::CppDeletable for crate::QStringList {
    /// <p>The <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class provides a list of strings.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QStringList::~QStringList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlist.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> class provides a list of strings.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> inherits from <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstring.html">QString</a>&gt;. Like <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>, <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> is <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicitly shared</a>. It provides fast index-based access as well as fast insertions and removals. Passing string lists as value parameters is both fast and safe.</p>
    /// <p>All of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>'s functionality also applies to <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a>. For example, you can use <a href="http://doc.qt.io/qt-5/qlist.html#isEmpty">isEmpty</a>() to test whether the list is empty, and you can call functions like <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#prepend">prepend</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#replace">replace</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAll">removeAll</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeAt">removeAt</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeFirst">removeFirst</a>(), <a href="http://doc.qt.io/qt-5/qlist.html#removeLast">removeLast</a>(), and <a href="http://doc.qt.io/qt-5/qlist.html#removeOne">removeOne</a>() to modify a <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a>. In addition, <a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a> provides a few convenience functions that make handling lists of strings easier:</p>
    /// <a name="adding-strings"></a></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QStringList_dQStringList(self as *mut crate::QStringList)
    }
}

impl ::cpp_utils::CppDeletable for crate::QSequentialIterable {
    /// <p>The <a href="http://doc.qt.io/qt-5/qsequentialiterable.html">QSequentialIterable</a> class is an iterable interface for a container in a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QSequentialIterable::~QSequentialIterable()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsequentialiterable.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsequentialiterable.html">QSequentialIterable</a> class is an iterable interface for a container in a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
    /// <p>This class allows several methods of accessing the elements of a container held within a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>. An instance of <a href="http://doc.qt.io/qt-5/qsequentialiterable.html">QSequentialIterable</a> can be extracted from a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> if it can be converted to a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> intList <span class="operator">=</span> {<span class="number">7</span><span class="operator">,</span> <span class="number">11</span><span class="operator">,</span> <span class="number">42</span>};
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> variant <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span><span class="operator">::</span>fromValue(intList);
    ///   <span class="keyword">if</span> (variant<span class="operator">.</span>canConvert<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariantList-typedef">QVariantList</a></span><span class="operator">&gt;</span>()) {
    /// &#32;     <span class="type">QSequentialIterable</span> iterable <span class="operator">=</span> variant<span class="operator">.</span>value<span class="operator">&lt;</span><span class="type">QSequentialIterable</span><span class="operator">&gt;</span>();
    /// &#32;     <span class="comment">// Can use foreach:</span>
    /// &#32;     foreach (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> <span class="operator">&amp;</span>v<span class="operator">,</span> iterable) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> v;
    /// &#32;     }
    /// &#32;     <span class="comment">// Can use C++11 range-for:</span>
    /// &#32;     <span class="keyword">for</span> (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> <span class="operator">&amp;</span>v : iterable) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> v;
    /// &#32;     }
    /// &#32;     <span class="comment">// Can use iterators:</span>
    /// &#32;     <span class="type">QSequentialIterable</span><span class="operator">::</span>const_iterator it <span class="operator">=</span> iterable<span class="operator">.</span>begin();
    /// &#32;     <span class="keyword">const</span> <span class="type">QSequentialIterable</span><span class="operator">::</span>const_iterator end <span class="operator">=</span> iterable<span class="operator">.</span>end();
    /// &#32;     <span class="keyword">for</span> ( ; it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>it;
    /// &#32;     }
    ///   }
    ///
    /// </pre>
    /// <p>The container itself is not copied before iterating over it.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSequentialIterable_dQSequentialIterable(
            self as *mut crate::QSequentialIterable,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QAssociativeIterable {
    /// <p>The <a href="http://doc.qt.io/qt-5/qassociativeiterable.html">QAssociativeIterable</a> class is an iterable interface for an associative container in a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QAssociativeIterable::~QAssociativeIterable()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qassociativeiterable.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qassociativeiterable.html">QAssociativeIterable</a> class is an iterable interface for an associative container in a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>.</p>
    /// <p>This class allows several methods of accessing the elements of an associative container held within a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a>. An instance of <a href="http://doc.qt.io/qt-5/qassociativeiterable.html">QAssociativeIterable</a> can be extracted from a <a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a> if it can be converted to a <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantHash-typedef">QVariantHash</a> or <a href="http://doc.qt.io/qt-5/qvariant.html#QVariantMap-typedef">QVariantMap</a>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qhash.html">QHash</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> mapping;
    ///   mapping<span class="operator">.</span>insert(<span class="number">7</span><span class="operator">,</span> <span class="string">"Seven"</span>);
    ///   mapping<span class="operator">.</span>insert(<span class="number">11</span><span class="operator">,</span> <span class="string">"Eleven"</span>);
    ///   mapping<span class="operator">.</span>insert(<span class="number">42</span><span class="operator">,</span> <span class="string">"Forty-two"</span>);
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> variant <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span><span class="operator">::</span>fromValue(mapping);
    ///   <span class="keyword">if</span> (variant<span class="operator">.</span>canConvert<span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html#QVariantHash-typedef">QVariantHash</a></span><span class="operator">&gt;</span>()) {
    /// &#32;     <span class="type">QAssociativeIterable</span> iterable <span class="operator">=</span> variant<span class="operator">.</span>value<span class="operator">&lt;</span><span class="type">QAssociativeIterable</span><span class="operator">&gt;</span>();
    /// &#32;     <span class="comment">// Can use foreach over the values:</span>
    /// &#32;     foreach (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> <span class="operator">&amp;</span>v<span class="operator">,</span> iterable) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> v;
    /// &#32;     }
    /// &#32;     <span class="comment">// Can use C++11 range-for over the values:</span>
    /// &#32;     <span class="keyword">for</span> (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> <span class="operator">&amp;</span>v : iterable) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> v;
    /// &#32;     }
    /// &#32;     <span class="comment">// Can use iterators:</span>
    /// &#32;     <span class="type">QAssociativeIterable</span><span class="operator">::</span>const_iterator it <span class="operator">=</span> iterable<span class="operator">.</span>begin();
    /// &#32;     <span class="keyword">const</span> <span class="type">QAssociativeIterable</span><span class="operator">::</span>const_iterator end <span class="operator">=</span> iterable<span class="operator">.</span>end();
    /// &#32;     <span class="keyword">for</span> ( ; it <span class="operator">!</span><span class="operator">=</span> end; <span class="operator">+</span><span class="operator">+</span>it) {
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>it; <span class="comment">// The current value</span>
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>key();
    /// &#32;   &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> it<span class="operator">.</span>value();
    /// &#32;     }
    ///   }
    ///
    /// </pre>
    /// <p>The container itself is not copied before iterating over it.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QAssociativeIterable_dQAssociativeIterable(
            self as *mut crate::QAssociativeIterable,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QModelIndex {
    /// <p>The <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> class is used to locate data in a data model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QModelIndex::~QModelIndex()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmodelindex.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> class is used to locate data in a data model.</p>
    /// <p>This class is used as an index into item models derived from <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a>. The index is used by item views, delegates, and selection models to locate an item in the model.</p>
    /// <p>New <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> objects are created by the model using the <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#createIndex">QAbstractItemModel::createIndex</a>() function. An <i>invalid</i> model index can be constructed with the <a href="http://doc.qt.io/qt-5/qmodelindex.html">QModelIndex</a> constructor. Invalid indexes are often used as parent indexes when referring to top-level items in a model.</p>
    /// <p>Model indexes refer to items in models, and contain all the information required to specify their locations in those models. Each index is located in a given row and column, and may have a parent index; use <a href="http://doc.qt.io/qt-5/qmodelindex.html#row">row</a>(), <a href="http://doc.qt.io/qt-5/qmodelindex.html#column">column</a>(), and <a href="http://doc.qt.io/qt-5/qmodelindex.html#parent">parent</a>() to obtain this information. Each top-level item in a model is represented by a model index that does not have a parent index - in this case, <a href="http://doc.qt.io/qt-5/qmodelindex.html#parent">parent</a>() will return an invalid model index, equivalent to an index constructed with the zero argument form of the <a href="http://doc.qt.io/qt-5/qmodelindex.html#QModelIndex">QModelIndex</a>() constructor.</p>
    /// <p>To obtain a model index that refers to an existing item in a model, call <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html#index">QAbstractItemModel::index</a>() with the required row and column values, and the model index of the parent. When referring to top-level items in a model, supply <a href="http://doc.qt.io/qt-5/qmodelindex.html#QModelIndex">QModelIndex</a>() as the parent index.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qmodelindex.html#model">model</a>() function returns the model that the index references as a <a href="http://doc.qt.io/qt-5/qabstractitemmodel.html">QAbstractItemModel</a>. The child() function is used to examine items held under the index in the model. The <a href="http://doc.qt.io/qt-5/qmodelindex.html#sibling">sibling</a>() function allows you to traverse items in the model on the same level as the index.</p>
    /// <p><b>Note: </b>Model indexes should be used immediately and then discarded. You should not rely on indexes to remain valid after calling model functions that change the structure of the model or delete items. If you need to keep a model index over time use a <a href="http://doc.qt.io/qt-5/qpersistentmodelindex.html">QPersistentModelIndex</a>.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QModelIndex_dQModelIndex(self as *mut crate::QModelIndex)
    }
}

impl ::cpp_utils::CppDeletable for crate::QBitArray {
    /// <p>The <a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a> class provides an array of bits.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QBitArray::~QBitArray()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qbitarray.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a> class provides an array of bits.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a> is an array that gives access to individual bits and provides operators (<a href="http://doc.qt.io/qt-5/qbitarray.html#operator-and">AND</a>, <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-7c">OR</a>, <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-5e">XOR</a>, and <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-7e">NOT</a>) that work on entire arrays of bits. It uses <a href="http://doc.qt.io/qt-5/implicit-sharing.html">implicit sharing</a> (copy-on-write) to reduce memory usage and to avoid the needless copying of data.</p>
    /// <p>The following code constructs a <a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a> containing 200 bits initialized to false (0):</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span> ba(<span class="number">200</span>);
    ///
    /// </pre>
    /// <p>To initialize the bits to true, either pass <code>true</code> as second argument to the constructor, or call <a href="http://doc.qt.io/qt-5/qbitarray.html#fill">fill</a>() later on.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a> uses 0-based indexes, just like C++ arrays. To access the bit at a particular index position, you can use operator[](). On non-const bit arrays, operator[]() returns a reference to a bit that can be used on the left side of an assignment. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span> ba;
    ///   ba<span class="operator">.</span>resize(<span class="number">3</span>);
    ///   ba<span class="operator">[</span><span class="number">0</span><span class="operator">]</span> <span class="operator">=</span> <span class="keyword">true</span>;
    ///   ba<span class="operator">[</span><span class="number">1</span><span class="operator">]</span> <span class="operator">=</span> <span class="keyword">false</span>;
    ///   ba<span class="operator">[</span><span class="number">2</span><span class="operator">]</span> <span class="operator">=</span> <span class="keyword">true</span>;
    ///
    /// </pre>
    /// <p>For technical reasons, it is more efficient to use <a href="http://doc.qt.io/qt-5/qbitarray.html#testBit">testBit</a>() and <a href="http://doc.qt.io/qt-5/qbitarray.html#setBit">setBit</a>() to access bits in the array than operator[](). For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span> ba(<span class="number">3</span>);
    ///   ba<span class="operator">.</span>setBit(<span class="number">0</span><span class="operator">,</span> <span class="keyword">true</span>);
    ///   ba<span class="operator">.</span>setBit(<span class="number">1</span><span class="operator">,</span> <span class="keyword">false</span>);
    ///   ba<span class="operator">.</span>setBit(<span class="number">2</span><span class="operator">,</span> <span class="keyword">true</span>);
    ///
    /// </pre>
    /// <p><a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a> supports <code>&amp;</code> (<a href="http://doc.qt.io/qt-5/qbitarray.html#operator-and">AND</a>), <code>|</code> (<a href="http://doc.qt.io/qt-5/qbitarray.html#operator-7c">OR</a>), <code>^</code> (<a href="http://doc.qt.io/qt-5/qbitarray.html#operator-5e">XOR</a>), <code>~</code> (<a href="http://doc.qt.io/qt-5/qbitarray.html#operator-7e">NOT</a>), as well as <code>&amp;=</code>, <code>|=</code>, and <code>^=</code>. These operators work in the same way as the built-in C++ bitwise operators of the same name. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span> x(<span class="number">5</span>);
    ///   x<span class="operator">.</span>setBit(<span class="number">3</span><span class="operator">,</span> <span class="keyword">true</span>);
    ///   <span class="comment">// x: [ 0, 0, 0, 1, 0 ]</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span> y(<span class="number">5</span>);
    ///   y<span class="operator">.</span>setBit(<span class="number">4</span><span class="operator">,</span> <span class="keyword">true</span>);
    ///   <span class="comment">// y: [ 0, 0, 0, 0, 1 ]</span>
    ///
    ///   x <span class="operator">|</span><span class="operator">=</span> y;
    ///   <span class="comment">// x: [ 0, 0, 0, 1, 1 ]</span>
    ///
    /// </pre>
    /// <p>For historical reasons, <a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a> distinguishes between a null bit array and an empty bit array. A <i>null</i> bit array is a bit array that is initialized using <a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a>'s default constructor. An <i>empty</i> bit array is any bit array with size 0. A null bit array is always empty, but an empty bit array isn't necessarily null:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span>()<span class="operator">.</span>isNull();           <span class="comment">// returns true</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span>()<span class="operator">.</span>isEmpty();          <span class="comment">// returns true</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span>(<span class="number">0</span>)<span class="operator">.</span>isNull();          <span class="comment">// returns false</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span>(<span class="number">0</span>)<span class="operator">.</span>isEmpty();         <span class="comment">// returns true</span>
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span>(<span class="number">3</span>)<span class="operator">.</span>isNull();          <span class="comment">// returns false</span>
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a></span>(<span class="number">3</span>)<span class="operator">.</span>isEmpty();         <span class="comment">// returns false</span>
    ///
    /// </pre>
    /// <p>All functions except <a href="http://doc.qt.io/qt-5/qbitarray.html#isNull">isNull</a>() treat null bit arrays the same as empty bit arrays; for example, <a href="http://doc.qt.io/qt-5/qbitarray.html#QBitArray">QBitArray</a>() compares equal to <a href="http://doc.qt.io/qt-5/qbitarray.html">QBitArray</a>(0). We recommend that you always use <a href="http://doc.qt.io/qt-5/qbitarray.html#isEmpty">isEmpty</a>() and avoid <a href="http://doc.qt.io/qt-5/qbitarray.html#isNull">isNull</a>().</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QBitArray_dQBitArray(self as *mut crate::QBitArray)
    }
}

impl ::cpp_utils::CppDeletable for crate::QBitRef {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QBitRef::~QBitRef()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QBitRef_dQBitRef(self as *mut crate::QBitRef)
    }
}

impl ::cpp_utils::CppDeletable for crate::QStaticByteArrayMatcherBase {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QStaticByteArrayMatcherBase::~QStaticByteArrayMatcherBase()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QStaticByteArrayMatcherBase_dQStaticByteArrayMatcherBase(
            self as *mut crate::QStaticByteArrayMatcherBase,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QDate {
    /// <p>The <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> class provides date functions.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QDate::~QDate()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdate.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> class provides date functions.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> object contains a calendar date, i.e. year, month, and day numbers, in the Gregorian calendar. It can read the current date from the system clock. It provides functions for comparing dates, and for manipulating dates. For example, it is possible to add and subtract days, months, and years to dates.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> object is typically created by giving the year, month, and day numbers explicitly. Note that <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> interprets two digit years as is, i.e., years 0 - 99. A <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> can also be constructed with the static function <a href="http://doc.qt.io/qt-5/qdate.html#currentDate">currentDate</a>(), which creates a <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> object containing the system clock's date. An explicit date can also be set using <a href="http://doc.qt.io/qt-5/qdate.html#setDate">setDate</a>(). The <a href="http://doc.qt.io/qt-5/qdate.html#fromString">fromString</a>() function returns a <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> given a string and a date format which is used to interpret the date within the string.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qdate.html#year">year</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#month">month</a>(), and <a href="http://doc.qt.io/qt-5/qdate.html#day">day</a>() functions provide access to the year, month, and day numbers. Also, <a href="http://doc.qt.io/qt-5/qdate.html#dayOfWeek">dayOfWeek</a>() and <a href="http://doc.qt.io/qt-5/qdate.html#dayOfYear">dayOfYear</a>() functions are provided. The same information is provided in textual format by the <a href="http://doc.qt.io/qt-5/qdate.html#toString">toString</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#shortDayName">shortDayName</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#longDayName">longDayName</a>(), <a href="http://doc.qt.io/qt-5/qdate.html#shortMonthName">shortMonthName</a>(), and <a href="http://doc.qt.io/qt-5/qdate.html#longMonthName">longMonthName</a>() functions.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> provides a full set of operators to compare two <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> objects where smaller means earlier, and larger means later.</p>
    /// <p>You can increment (or decrement) a date by a given number of days using <a href="http://doc.qt.io/qt-5/qdate.html#addDays">addDays</a>(). Similarly you can use <a href="http://doc.qt.io/qt-5/qdate.html#addMonths">addMonths</a>() and <a href="http://doc.qt.io/qt-5/qdate.html#addYears">addYears</a>(). The <a href="http://doc.qt.io/qt-5/qdate.html#daysTo">daysTo</a>() function returns the number of days between two dates.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qdate.html#daysInMonth">daysInMonth</a>() and <a href="http://doc.qt.io/qt-5/qdate.html#daysInYear">daysInYear</a>() functions return how many days there are in this date's month and year, respectively. The <a href="http://doc.qt.io/qt-5/qdate.html#isLeapYear">isLeapYear</a>() function indicates whether a date is in a leap year.</p>
    /// <a name="no-year-0"></a>
    /// <a name="no-year-0"></a>
    /// <h4>No Year 0</h4>
    /// <p>There is no year 0. Dates in that year are considered invalid. The year -1 is the year "1 before Christ" or "1 before current era." The day before 1 January 1 CE is 31 December 1 BCE.</p>
    /// <a name="range-of-valid-dates"></a>
    /// <h4>Range of Valid Dates</h4>
    /// <p>Dates are stored internally as a Julian Day number, an integer count of every day in a contiguous range, with 24 November 4714 BCE in the Gregorian calendar being Julian Day 0 (1 January 4713 BCE in the Julian calendar). As well as being an efficient and accurate way of storing an absolute date, it is suitable for converting a Date into other calendar systems such as Hebrew, Islamic or Chinese. The Julian Day number can be obtained using <a href="http://doc.qt.io/qt-5/qdate.html#toJulianDay">QDate::toJulianDay</a>() and can be set using <a href="http://doc.qt.io/qt-5/qdate.html#fromJulianDay">QDate::fromJulianDay</a>().</p>
    /// <p>The range of dates able to be stored by <a href="http://doc.qt.io/qt-5/qdate.html">QDate</a> as a Julian Day number is for technical reasons limited to between -784350574879 and 784354017364, which means from before 2 billion BCE to after 2 billion CE.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QDate_dQDate(self as *mut crate::QDate)
    }
}

impl ::cpp_utils::CppDeletable for crate::QTime {
    /// <p>The <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> class provides clock time functions.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QTime::~QTime()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtime.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> class provides clock time functions.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> object contains a clock time, i.e. the number of hours, minutes, seconds, and milliseconds since midnight. It can read the current time from the system clock and measure a span of elapsed time. It provides functions for comparing times and for manipulating a time by adding a number of milliseconds.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> uses the 24-hour clock format; it has no concept of AM/PM. Unlike <a href="http://doc.qt.io/qt-5/qdatetime.html">QDateTime</a>, <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> knows nothing about time zones or daylight-saving time (DST).</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> object is typically created either by giving the number of hours, minutes, seconds, and milliseconds explicitly, or by using the static function <a href="http://doc.qt.io/qt-5/qtime.html#currentTime">currentTime</a>(), which creates a <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> object that contains the system's local time. Note that the accuracy depends on the accuracy of the underlying operating system; not all systems provide 1-millisecond accuracy.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qtime.html#hour">hour</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#minute">minute</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#second">second</a>(), and <a href="http://doc.qt.io/qt-5/qtime.html#msec">msec</a>() functions provide access to the number of hours, minutes, seconds, and milliseconds of the time. The same information is provided in textual format by the <a href="http://doc.qt.io/qt-5/qtime.html#toString">toString</a>() function.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> provides a full set of operators to compare two <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> objects. <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> A is considered smaller than <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> B if A is earlier than B.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qtime.html#addSecs">addSecs</a>() and <a href="http://doc.qt.io/qt-5/qtime.html#addMSecs">addMSecs</a>() functions provide the time a given number of seconds or milliseconds later than a given time. Correspondingly, the number of seconds or milliseconds between two times can be found using <a href="http://doc.qt.io/qt-5/qtime.html#secsTo">secsTo</a>() or <a href="http://doc.qt.io/qt-5/qtime.html#msecsTo">msecsTo</a>().</p>
    /// <p><a href="http://doc.qt.io/qt-5/qtime.html">QTime</a> can be used to measure a span of elapsed time using the <a href="http://doc.qt.io/qt-5/qtime.html#start">start</a>(), <a href="http://doc.qt.io/qt-5/qtime.html#restart">restart</a>(), and <a href="http://doc.qt.io/qt-5/qtime.html#elapsed">elapsed</a>() functions.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QTime_dQTime(self as *mut crate::QTime)
    }
}

impl ::cpp_utils::CppDeletable for crate::QElapsedTimer {
    /// <p>The <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> class provides a fast way to calculate elapsed times.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QElapsedTimer::~QElapsedTimer()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> class provides a fast way to calculate elapsed times.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> class is usually used to quickly calculate how much time has elapsed between two events. Its API is similar to that of <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a>, so code that was using that can be ported quickly to the new class.</p>
    /// <p>However, unlike <a href="http://doc.qt.io/qt-5/qtime.html">QTime</a>, <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> tries to use monotonic clocks if possible. This means it's not possible to convert <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> objects to a human-readable time.</p>
    /// <p>The typical use-case for the class is to determine how much time was spent in a slow operation. The simplest example of such a case is for debugging purposes, as in the following example:</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#QElapsedTimer">QElapsedTimer</a></span> timer;
    /// &#32;     timer<span class="operator">.</span>start();
    ///
    /// &#32;     slowOperation1();
    ///
    /// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"The slow operation took"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> timer<span class="operator">.</span>elapsed() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"milliseconds"</span>;
    ///
    /// </pre>
    /// <p>In this example, the timer is started by a call to <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#start">start</a>() and the elapsed timer is calculated by the <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#elapsed">elapsed</a>() function.</p>
    /// <p>The time elapsed can also be used to recalculate the time available for another operation, after the first one is complete. This is useful when the execution must complete within a certain time period, but several steps are needed. The <code>waitFor</code>-type functions in <a href="http://doc.qt.io/qt-5/qiodevice.html">QIODevice</a> and its subclasses are good examples of such need. In that case, the code could be as follows:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">void</span> executeSlowOperations(<span class="type">int</span> timeout)
    ///   {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#QElapsedTimer">QElapsedTimer</a></span> timer;
    /// &#32;     timer<span class="operator">.</span>start();
    /// &#32;     slowOperation1();
    ///
    /// &#32;     <span class="type">int</span> remainingTime <span class="operator">=</span> timeout <span class="operator">-</span> timer<span class="operator">.</span>elapsed();
    /// &#32;     <span class="keyword">if</span> (remainingTime <span class="operator">&gt;</span> <span class="number">0</span>)
    /// &#32;   &#32;     slowOperation2(remainingTime);
    ///   }
    ///
    /// </pre>
    /// <p>Another use-case is to execute a certain operation for a specific timeslice. For this, <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a> provides the <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#hasExpired">hasExpired</a>() convenience function, which can be used to determine if a certain number of milliseconds has already elapsed:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">void</span> executeOperationsForTime(<span class="type">int</span> ms)
    ///   {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qelapsedtimer.html#QElapsedTimer">QElapsedTimer</a></span> timer;
    /// &#32;     timer<span class="operator">.</span>start();
    ///
    /// &#32;     <span class="keyword">while</span> (<span class="operator">!</span>timer<span class="operator">.</span>hasExpired(ms))
    /// &#32;   &#32;     slowOperation1();
    ///   }
    ///
    /// </pre>
    /// <p>It is often more convenient to use <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> in this case, which counts towards a timeout in the future instead of tracking elapsed time.</p>
    /// <a name="reference-clocks"></a></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QElapsedTimer_dQElapsedTimer(
            self as *mut crate::QElapsedTimer,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QDeadlineTimer {
    /// <p>The <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> class marks a deadline in the future.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QDeadlineTimer::~QDeadlineTimer()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> class marks a deadline in the future.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> class is usually used to calculate future deadlines and verify whether the deadline has expired. <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> can also be used for deadlines without expiration ("forever"). It forms a counterpart to <a href="http://doc.qt.io/qt-5/qelapsedtimer.html">QElapsedTimer</a>, which calculates how much time has elapsed since <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#start">QElapsedTimer::start</a>() was called.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> provides a more convenient API compared to <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#hasExpired">QElapsedTimer::hasExpired</a>().</p>
    /// <p>The typical use-case for the class is to create a <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> before the operation in question is started, and then use <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#remainingTime">remainingTime</a>() or <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#hasExpired">hasExpired</a>() to determine whether to continue trying the operation. <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> objects can be passed to functions being called to execute this operation so they know how long to still operate.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">void</span> executeOperation(<span class="type">int</span> msecs)
    ///   {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qdeadlinetimer.html#QDeadlineTimer">QDeadlineTimer</a></span> deadline(msecs);
    /// &#32;     <span class="keyword">do</span> {
    /// &#32;   &#32;     <span class="keyword">if</span> (readFromDevice(deadline<span class="operator">.</span>remainingTime())
    /// &#32;   &#32;   &#32;     <span class="keyword">break</span>;
    /// &#32;   &#32;     waitForReadyRead(deadline);
    /// &#32;     } <span class="keyword">while</span> (<span class="operator">!</span>deadline<span class="operator">.</span>hasExpired());
    ///   }
    ///
    /// </pre>
    /// <p>Many <a href="http://doc.qt.io/qt-5/qdeadlinetimer.html">QDeadlineTimer</a> functions deal with time out values, which all are measured in milliseconds. There are two special values, the same as many other Qt functions named <code>waitFor</code> or similar:</p>
    /// <ul>
    /// <li>0: no time left, expired</li>
    /// <li>-1: infinite time left, timer never expires</li>
    /// </ul>
    /// <a name="reference-clocks"></a></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QDeadlineTimer_dQDeadlineTimer(
            self as *mut crate::QDeadlineTimer,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QTextStreamManipulator {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QTextStreamManipulator::~QTextStreamManipulator()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QTextStreamManipulator_dQTextStreamManipulator(
            self as *mut crate::QTextStreamManipulator,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QBasicMutex {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QBasicMutex::~QBasicMutex()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QBasicMutex_dQBasicMutex(self as *mut crate::QBasicMutex)
    }
}

impl ::cpp_utils::CppDeletable for crate::QFutureWatcherBase {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QFutureWatcherBase::~QFutureWatcherBase()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QFutureWatcherBase_dQFutureWatcherBase(
            self as *mut crate::QFutureWatcherBase,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QItemSelectionRange {
    /// <p>The <a href="http://doc.qt.io/qt-5/qitemselectionrange.html">QItemSelectionRange</a> class manages information about a range of selected items in a model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QItemSelectionRange::~QItemSelectionRange()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselectionrange.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qitemselectionrange.html">QItemSelectionRange</a> class manages information about a range of selected items in a model.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qitemselectionrange.html">QItemSelectionRange</a> contains information about a range of selected items in a model. A range of items is a contiguous array of model items, extending to cover a number of adjacent rows and columns with a common parent item; this can be visualized as a two-dimensional block of cells in a table. A selection range has a <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#top">top</a>(), <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#left">left</a>() a <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#bottom">bottom</a>(), <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#right">right</a>() and a <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#parent">parent</a>().</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qitemselectionrange.html">QItemSelectionRange</a> class is one of the <a href="http://doc.qt.io/qt-5/model-view-programming.html#model-view-classes">Model/View Classes</a> and is part of Qt's <a href="http://doc.qt.io/qt-5/model-view-programming.html">model/view framework</a>.</p>
    /// <p>The model items contained in the selection range can be obtained using the <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#indexes">indexes</a>() function. Use <a href="http://doc.qt.io/qt-5/qitemselectionmodel.html#selectedIndexes">QItemSelectionModel::selectedIndexes</a>() to get a list of all selected items for a view.</p>
    /// <p>You can determine whether a given model item lies within a particular range by using the <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#contains">contains</a>() function. Ranges can also be compared using the overloaded operators for equality and inequality, and the <a href="http://doc.qt.io/qt-5/qitemselectionrange.html#intersects">intersects</a>() function allows you to determine whether two ranges overlap.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QItemSelectionRange_dQItemSelectionRange(
            self as *mut crate::QItemSelectionRange,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QItemSelection {
    /// <p>The <a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a> class manages information about selected items in a model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QItemSelection::~QItemSelection()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qitemselection.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a> class manages information about selected items in a model.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a> describes the items in a model that have been selected by the user. A <a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a> is basically a list of selection ranges, see <a href="http://doc.qt.io/qt-5/qitemselectionrange.html">QItemSelectionRange</a>. It provides functions for creating and manipulating selections, and selecting a range of items from a model.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a> class is one of the <a href="http://doc.qt.io/qt-5/model-view-programming.html#model-view-classes">Model/View Classes</a> and is part of Qt's <a href="http://doc.qt.io/qt-5/model-view-programming.html">model/view framework</a>.</p>
    /// <p>An item selection can be constructed and initialized to contain a range of items from an existing model. The following example constructs a selection that contains a range of items from the given <code>model</code>, beginning at the <code>topLeft</code>, and ending at the <code>bottomRight</code>.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qitemselection.html#QItemSelection">QItemSelection</a></span> <span class="operator">*</span>selection <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qitemselection.html#QItemSelection">QItemSelection</a></span>(topLeft<span class="operator">,</span> bottomRight);
    ///
    /// </pre>
    /// <p>An empty item selection can be constructed, and later populated as required. So, if the model is going to be unavailable when we construct the item selection, we can rewrite the above code in the following way:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qitemselection.html#QItemSelection">QItemSelection</a></span> <span class="operator">*</span>selection <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qitemselection.html#QItemSelection">QItemSelection</a></span>();
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   selection<span class="operator">-</span><span class="operator">&gt;</span>select(topLeft<span class="operator">,</span> bottomRight);
    ///
    /// </pre>
    /// <p><a href="http://doc.qt.io/qt-5/qitemselection.html">QItemSelection</a> saves memory, and avoids unnecessary work, by working with selection ranges rather than recording the model item index for each item in the selection. Generally, an instance of this class will contain a list of non-overlapping selection ranges.</p>
    /// <p>Use <a href="http://doc.qt.io/qt-5/qitemselection.html#merge">merge</a>() to merge one item selection into another without making overlapping ranges. Use <a href="http://doc.qt.io/qt-5/qitemselection.html#split">split</a>() to split one selection range into smaller ranges based on a another selection range.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QItemSelection_dQItemSelection(
            self as *mut crate::QItemSelection,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QJsonValueRef {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QJsonValueRef::~QJsonValueRef()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QJsonValueRef_dQJsonValueRef(
            self as *mut crate::QJsonValueRef,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QJsonParseError {
    /// <p>The <a href="http://doc.qt.io/qt-5/qjsonparseerror.html">QJsonParseError</a> class is used to report errors during JSON parsing.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QJsonParseError::~QJsonParseError()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qjsonparseerror.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qjsonparseerror.html">QJsonParseError</a> class is used to report errors during JSON parsing.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QJsonParseError_dQJsonParseError(
            self as *mut crate::QJsonParseError,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QVersionNumber {
    /// <p>The <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> class contains a version number with an arbitrary number of segments.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QVersionNumber::~QVersionNumber()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qversionnumber.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a> class contains a version number with an arbitrary number of segments.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qversionnumber.html#QVersionNumber">QVersionNumber</a></span> version(<span class="number">1</span><span class="operator">,</span> <span class="number">2</span><span class="operator">,</span> <span class="number">3</span>);  <span class="comment">// 1.2.3</span>
    ///
    /// </pre></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVersionNumber_dQVersionNumber(
            self as *mut crate::QVersionNumber,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QLibraryInfo {
    /// <p>The <a href="http://doc.qt.io/qt-5/qlibraryinfo.html">QLibraryInfo</a> class provides information about the Qt library.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QLibraryInfo::~QLibraryInfo()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlibraryinfo.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlibraryinfo.html">QLibraryInfo</a> class provides information about the Qt library.</p>
    /// <p>Many pieces of information are established when Qt is configured and built. This class provides an abstraction for accessing that information. By using the static functions of this class, an application can obtain information about the instance of the Qt library which the application is using at run-time.</p>
    /// <p>You can also use a <code>qt.conf</code> file to override the hard-coded paths that are compiled into the Qt library. For more information, see the <a href="http://doc.qt.io/qt-5/qt-conf.html">Using qt.conf</a> documentation.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QLibraryInfo_dQLibraryInfo(self as *mut crate::QLibraryInfo)
    }
}

impl ::cpp_utils::CppDeletable for crate::QPoint {
    /// <p>The <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> class defines a point in the plane using integer precision.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QPoint::~QPoint()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpoint.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> class defines a point in the plane using integer precision.</p>
    /// <p>A point is specified by a x coordinate and an y coordinate which can be accessed using the <a href="http://doc.qt.io/qt-5/qpoint.html#x">x</a>() and <a href="http://doc.qt.io/qt-5/qpoint.html#y">y</a>() functions. The <a href="http://doc.qt.io/qt-5/qpoint.html#isNull">isNull</a>() function returns <code>true</code> if both x and y are set to 0. The coordinates can be set (or altered) using the <a href="http://doc.qt.io/qt-5/qpoint.html#setX">setX</a>() and <a href="http://doc.qt.io/qt-5/qpoint.html#setY">setY</a>() functions, or alternatively the <a href="http://doc.qt.io/qt-5/qpoint.html#rx">rx</a>() and <a href="http://doc.qt.io/qt-5/qpoint.html#ry">ry</a>() functions which return references to the coordinates (allowing direct manipulation).</p>
    /// <p>Given a point <i>p</i>, the following statements are all equivalent:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html#QPoint">QPoint</a></span> p;
    ///
    ///   p<span class="operator">.</span>setX(p<span class="operator">.</span>x() <span class="operator">+</span> <span class="number">1</span>);
    ///   p <span class="operator">+</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html#QPoint">QPoint</a></span>(<span class="number">1</span><span class="operator">,</span> <span class="number">0</span>);
    ///   p<span class="operator">.</span>rx()<span class="operator">+</span><span class="operator">+</span>;
    ///
    /// </pre>
    /// <p>A <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> object can also be used as a vector: Addition and subtraction are defined as for vectors (each component is added separately). A <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> object can also be divided or multiplied by an <code>int</code> or a <code>qreal</code>.</p>
    /// <p>In addition, the <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> class provides the <a href="http://doc.qt.io/qt-5/qpoint.html#manhattanLength">manhattanLength</a>() function which gives an inexpensive approximation of the length of the <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> object interpreted as a vector. Finally, <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> objects can be streamed as well as compared.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QPoint_dQPoint(self as *mut crate::QPoint)
    }
}

impl ::cpp_utils::CppDeletable for crate::QPointF {
    /// <p>The <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> class defines a point in the plane using floating point precision.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QPointF::~QPointF()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpointf.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> class defines a point in the plane using floating point precision.</p>
    /// <p>A point is specified by a x coordinate and an y coordinate which can be accessed using the <a href="http://doc.qt.io/qt-5/qpointf.html#x">x</a>() and <a href="http://doc.qt.io/qt-5/qpointf.html#y">y</a>() functions. The coordinates of the point are specified using floating point numbers for accuracy. The <a href="http://doc.qt.io/qt-5/qpointf.html#isNull">isNull</a>() function returns <code>true</code> if both x and y are set to 0.0. The coordinates can be set (or altered) using the <a href="http://doc.qt.io/qt-5/qpointf.html#setX">setX</a>() and <a href="http://doc.qt.io/qt-5/qpointf.html#setY">setY</a>() functions, or alternatively the <a href="http://doc.qt.io/qt-5/qpointf.html#rx">rx</a>() and <a href="http://doc.qt.io/qt-5/qpointf.html#ry">ry</a>() functions which return references to the coordinates (allowing direct manipulation).</p>
    /// <p>Given a point <i>p</i>, the following statements are all equivalent:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpointf.html#QPointF">QPointF</a></span> p;
    ///
    ///   p<span class="operator">.</span>setX(p<span class="operator">.</span>x() <span class="operator">+</span> <span class="number">1.0</span>);
    ///   p <span class="operator">+</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qpointf.html#QPointF">QPointF</a></span>(<span class="number">1.0</span><span class="operator">,</span> <span class="number">0.0</span>);
    ///   p<span class="operator">.</span>rx()<span class="operator">+</span><span class="operator">+</span>;
    ///
    /// </pre>
    /// <p>A <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> object can also be used as a vector: Addition and subtraction are defined as for vectors (each component is added separately). A <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> object can also be divided or multiplied by an <code>int</code> or a <code>qreal</code>.</p>
    /// <p>In addition, the <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> class provides a constructor converting a <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> object into a <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> object, and a corresponding <a href="http://doc.qt.io/qt-5/qpointf.html#toPoint">toPoint</a>() function which returns a <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> copy of <i>this</i> point. Finally, <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> objects can be streamed as well as compared.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QPointF_dQPointF(self as *mut crate::QPointF)
    }
}

impl ::cpp_utils::CppDeletable for crate::QLine {
    /// <p>The <a href="http://doc.qt.io/qt-5/qline.html">QLine</a> class provides a two-dimensional vector using integer precision.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QLine::~QLine()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qline.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qline.html">QLine</a> class provides a two-dimensional vector using integer precision.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qline.html">QLine</a> describes a finite length line (or a line segment) on a two-dimensional surface. The start and end points of the line are specified using integer point accuracy for coordinates. Use the <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> constructor to retrieve a floating point copy.</p>
    /// <div class="table"><table class="generic">
    ///  <tbody><tr valign="top" class="odd"><td><img src="http://doc.qt.io/qt-5/images/qline-point.png" alt=""></td><td><img src="http://doc.qt.io/qt-5/images/qline-coordinates.png" alt=""></td></tr>
    /// </tbody></table></div>
    /// <p>The positions of the line's start and end points can be retrieved using the <a href="http://doc.qt.io/qt-5/qline.html#p1">p1</a>(), <a href="http://doc.qt.io/qt-5/qline.html#x1">x1</a>(), <a href="http://doc.qt.io/qt-5/qline.html#y1">y1</a>(), <a href="http://doc.qt.io/qt-5/qline.html#p2">p2</a>(), <a href="http://doc.qt.io/qt-5/qline.html#x2">x2</a>(), and <a href="http://doc.qt.io/qt-5/qline.html#y2">y2</a>() functions. The <a href="http://doc.qt.io/qt-5/qline.html#dx">dx</a>() and <a href="http://doc.qt.io/qt-5/qline.html#dy">dy</a>() functions return the horizontal and vertical components of the line. Use <a href="http://doc.qt.io/qt-5/qline.html#isNull">isNull</a>() to determine whether the <a href="http://doc.qt.io/qt-5/qline.html">QLine</a> represents a valid line or a null line.</p>
    /// <p>Finally, the line can be translated a given offset using the <a href="http://doc.qt.io/qt-5/qline.html#translate">translate</a>() function.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QLine_dQLine(self as *mut crate::QLine)
    }
}

impl ::cpp_utils::CppDeletable for crate::QLineF {
    /// <p>The <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> class provides a two-dimensional vector using floating point precision.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QLineF::~QLineF()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlinef.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> class provides a two-dimensional vector using floating point precision.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> describes a finite length line (or line segment) on a two-dimensional surface. <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> defines the start and end points of the line using floating point accuracy for coordinates. Use the <a href="http://doc.qt.io/qt-5/qlinef.html#toLine">toLine</a>() function to retrieve an integer based copy of this line.</p>
    /// <div class="table"><table class="generic">
    ///  <tbody><tr valign="top" class="odd"><td><img src="http://doc.qt.io/qt-5/images/qline-point.png" alt=""></td><td><img src="http://doc.qt.io/qt-5/images/qline-coordinates.png" alt=""></td></tr>
    /// </tbody></table></div>
    /// <p>The positions of the line's start and end points can be retrieved using the <a href="http://doc.qt.io/qt-5/qlinef.html#p1">p1</a>(), <a href="http://doc.qt.io/qt-5/qlinef.html#x1">x1</a>(), <a href="http://doc.qt.io/qt-5/qlinef.html#y1">y1</a>(), <a href="http://doc.qt.io/qt-5/qlinef.html#p2">p2</a>(), <a href="http://doc.qt.io/qt-5/qlinef.html#x2">x2</a>(), and <a href="http://doc.qt.io/qt-5/qlinef.html#y2">y2</a>() functions. The <a href="http://doc.qt.io/qt-5/qlinef.html#dx">dx</a>() and <a href="http://doc.qt.io/qt-5/qlinef.html#dy">dy</a>() functions return the horizontal and vertical components of the line, respectively.</p>
    /// <p>The line's length can be retrieved using the <a href="http://doc.qt.io/qt-5/qlinef.html#length">length</a>() function, and altered using the <a href="http://doc.qt.io/qt-5/qlinef.html#setLength">setLength</a>() function. Similarly, <a href="http://doc.qt.io/qt-5/qlinef.html#angle">angle</a>() and <a href="http://doc.qt.io/qt-5/qlinef.html#setAngle">setAngle</a>() are respectively used for retrieving and altering the angle of the line. Use the <a href="http://doc.qt.io/qt-5/qlinef.html#isNull">isNull</a>() function to determine whether the <a href="http://doc.qt.io/qt-5/qlinef.html">QLineF</a> represents a valid line or a null line.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qlinef.html#intersect">intersect</a>() function determines the <a href="http://doc.qt.io/qt-5/qlinef.html#IntersectType-enum">IntersectType</a> for this line and a given line, while the <a href="http://doc.qt.io/qt-5/qlinef.html#angleTo">angleTo</a>() function returns the angle between the lines. In addition, the <a href="http://doc.qt.io/qt-5/qlinef.html#unitVector">unitVector</a>() function returns a line that has the same starting point as this line, but with a length of only 1, while the <a href="http://doc.qt.io/qt-5/qlinef.html#normalVector">normalVector</a>() function returns a line that is perpendicular to this line with the same starting point and length.</p>
    /// <p>Finally, the line can be translated a given offset using the <a href="http://doc.qt.io/qt-5/qlinef.html#translate">translate</a>() function, and can be traversed using the <a href="http://doc.qt.io/qt-5/qlinef.html#pointAt">pointAt</a>() function.</p>
    /// <a name="constraints"></a></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QLineF_dQLineF(self as *mut crate::QLineF)
    }
}

impl ::cpp_utils::CppDeletable for crate::QMargins {
    /// <p>The <a href="http://doc.qt.io/qt-5/qmargins.html">QMargins</a> class defines the four margins of a rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QMargins::~QMargins()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmargins.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmargins.html">QMargins</a> class defines the four margins of a rectangle.</p>
    /// <p>QMargin defines a set of four margins; left, top, right and bottom, that describe the size of the borders surrounding a rectangle.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qmargins.html#isNull">isNull</a>() function returns <code>true</code> only if all margins are set to zero.</p>
    /// <p>QMargin objects can be streamed as well as compared.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMargins_dQMargins(self as *mut crate::QMargins)
    }
}

impl ::cpp_utils::CppDeletable for crate::QMarginsF {
    /// <p>The <a href="http://doc.qt.io/qt-5/qmarginsf.html">QMarginsF</a> class defines the four margins of a rectangle.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QMarginsF::~QMarginsF()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmarginsf.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmarginsf.html">QMarginsF</a> class defines the four margins of a rectangle.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qmarginsf.html">QMarginsF</a> defines a set of four margins; left, top, right and bottom, that describe the size of the borders surrounding a rectangle.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qmarginsf.html#isNull">isNull</a>() function returns <code>true</code> only if all margins are set to zero.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qmarginsf.html">QMarginsF</a> objects can be streamed as well as compared.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMarginsF_dQMarginsF(self as *mut crate::QMarginsF)
    }
}

impl ::cpp_utils::CppDeletable for crate::QMetaMethod {
    /// <p>The <a href="http://doc.qt.io/qt-5/qmetamethod.html">QMetaMethod</a> class provides meta-data about a member function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QMetaMethod::~QMetaMethod()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetamethod.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmetamethod.html">QMetaMethod</a> class provides meta-data about a member function.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qmetamethod.html">QMetaMethod</a> has a <a href="http://doc.qt.io/qt-5/qmetamethod.html#methodType">methodType</a>(), a <a href="http://doc.qt.io/qt-5/qmetamethod.html#methodSignature">methodSignature</a>(), a list of <a href="http://doc.qt.io/qt-5/qmetamethod.html#parameterTypes">parameterTypes</a>() and <a href="http://doc.qt.io/qt-5/qmetamethod.html#parameterNames">parameterNames</a>(), a return <a href="http://doc.qt.io/qt-5/qmetamethod.html#typeName">typeName</a>(), a <a href="http://doc.qt.io/qt-5/qmetamethod.html#tag">tag</a>(), and an <a href="http://doc.qt.io/qt-5/qmetamethod.html#access">access</a>() specifier. You can use <a href="http://doc.qt.io/qt-5/qmetamethod.html#invoke">invoke</a>() to invoke the method on an arbitrary <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a>.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMetaMethod_dQMetaMethod(self as *mut crate::QMetaMethod)
    }
}

impl ::cpp_utils::CppDeletable for crate::QMetaEnum {
    /// <p>The <a href="http://doc.qt.io/qt-5/qmetaenum.html">QMetaEnum</a> class provides meta-data about an enumerator.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QMetaEnum::~QMetaEnum()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaenum.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmetaenum.html">QMetaEnum</a> class provides meta-data about an enumerator.</p>
    /// <p>Use <a href="http://doc.qt.io/qt-5/qmetaenum.html#name">name</a>() for the enumerator's name. The enumerator's keys (names of each enumerated item) are returned by <a href="http://doc.qt.io/qt-5/qmetaenum.html#key">key</a>(); use <a href="http://doc.qt.io/qt-5/qmetaenum.html#keyCount">keyCount</a>() to find the number of keys. <a href="http://doc.qt.io/qt-5/qmetaenum.html#isFlag">isFlag</a>() returns whether the enumerator is meant to be used as a flag, meaning that its values can be combined using the OR operator.</p>
    /// <p>The conversion functions <a href="http://doc.qt.io/qt-5/qmetaenum.html#keyToValue">keyToValue</a>(), <a href="http://doc.qt.io/qt-5/qmetaenum.html#valueToKey">valueToKey</a>(), <a href="http://doc.qt.io/qt-5/qmetaenum.html#keysToValue">keysToValue</a>(), and <a href="http://doc.qt.io/qt-5/qmetaenum.html#valueToKeys">valueToKeys</a>() allow conversion between the integer representation of an enumeration or set value and its literal representation. The <a href="http://doc.qt.io/qt-5/qmetaenum.html#scope">scope</a>() function returns the class scope this enumerator was declared in.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMetaEnum_dQMetaEnum(self as *mut crate::QMetaEnum)
    }
}

impl ::cpp_utils::CppDeletable for crate::QMetaProperty {
    /// <p>The <a href="http://doc.qt.io/qt-5/qmetaproperty.html">QMetaProperty</a> class provides meta-data about a property.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QMetaProperty::~QMetaProperty()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaproperty.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmetaproperty.html">QMetaProperty</a> class provides meta-data about a property.</p>
    /// <p>Property meta-data is obtained from an object's meta-object. See <a href="http://doc.qt.io/qt-5/qmetaobject.html#property">QMetaObject::property</a>() and <a href="http://doc.qt.io/qt-5/qmetaobject.html#propertyCount">QMetaObject::propertyCount</a>() for details.</p>
    /// <a name="property-meta-data"></a></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMetaProperty_dQMetaProperty(
            self as *mut crate::QMetaProperty,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QMetaClassInfo {
    /// <p>The <a href="http://doc.qt.io/qt-5/qmetaclassinfo.html">QMetaClassInfo</a> class provides additional information about a class.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QMetaClassInfo::~QMetaClassInfo()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmetaclassinfo.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qmetaclassinfo.html">QMetaClassInfo</a> class provides additional information about a class.</p>
    /// <p>Class information items are simple <i>name</i>--<i>value</i> pairs that are specified using <a href="http://doc.qt.io/qt-5/qobject.html#Q_CLASSINFO">Q_CLASSINFO</a>() in the source code. The information can be retrieved using <a href="http://doc.qt.io/qt-5/qmetaclassinfo.html#name">name</a>() and <a href="http://doc.qt.io/qt-5/qmetaclassinfo.html#value">value</a>(). For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">class</span> MyClass
    ///   {
    /// &#32;     Q_OBJECT
    /// &#32;     Q_CLASSINFO(<span class="string">"author"</span><span class="operator">,</span> <span class="string">"Sabrina Schweinsteiger"</span>)
    /// &#32;     Q_CLASSINFO(<span class="string">"url"</span><span class="operator">,</span> <span class="string">"http://doc.moosesoft.co.uk/1.0/"</span>)
    ///
    ///   <span class="keyword">public</span>:
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   };
    ///
    /// </pre>
    /// <p>This mechanism is free for you to use in your Qt applications. Qt doesn't use it for any of its classes.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMetaClassInfo_dQMetaClassInfo(
            self as *mut crate::QMetaClassInfo,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QOperatingSystemVersion {
    /// <p>The <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> class provides information about the operating system version.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QOperatingSystemVersion::~QOperatingSystemVersion()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> class provides information about the operating system version.</p>
    /// <p>Unlike other version functions in <a href="http://doc.qt.io/qt-5/qsysinfo.html">QSysInfo</a>, <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> provides access to the full version number that <i>developers</i> typically use to vary behavior or determine whether to enable APIs or features based on the operating system version (as opposed to the kernel version number or marketing version).</p>
    /// <p>This class is also a complete replacement for QSysInfo::macVersion and QSysInfo::windowsVersion, additionally providing access to the third (micro) version number component.</p>
    /// <p>Presently, Android, Apple Platforms (iOS, <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, tvOS, and watchOS), and Windows are supported.</p>
    /// <p>The <i>majorVersion()</i>, <i>minorVersion()</i>, and <i>microVersion()</i> functions return the parts of the operating system version number based on:</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>Platforms</th><th>Value</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>Android</td><td>result of parsing <a href="https://developer.android.com/reference/android/os/Build.VERSION.html#RELEASE">android.os.Build.VERSION.RELEASE</a> using <a href="http://doc.qt.io/qt-5/qversionnumber.html">QVersionNumber</a>, with a fallback to <a href="https://developer.android.com/reference/android/os/Build.VERSION.html#SDK_INT">android.os.Build.VERSION.SDK_INT</a> to determine the major and minor version component if the former fails</td></tr>
    /// <tr valign="top" class="even"><td>Apple Platforms</td><td><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#majorVersion">majorVersion</a>, <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#minorVersion">minorVersion</a>, and patchVersion from <a href="https://developer.apple.com/reference/foundation/nsprocessinfo/1410906-operatingsystemversion?language=objc">NSProcessInfo.operatingSystemVersion</a></td></tr>
    /// <tr valign="top" class="odd"><td>Windows</td><td>dwMajorVersion, dwMinorVersion, and dwBuildNumber from <a href="https://msdn.microsoft.com/en-us/library/mt723418.aspx">RtlGetVersion</a> - note that this function ALWAYS return the version number of the underlying operating system, as opposed to the shim underneath GetVersionEx that hides the real version number if the application is not manifested for that version of the OS</td></tr>
    /// </tbody></table></div>
    /// <p>Because <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> stores both a version number and an OS type, the OS type can be taken into account when performing comparisons. For example, on a <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> system running <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a> Sierra (v10.12), the following expression will return <code>false</code> even though the major version number component of the object on the left hand side of the expression (10) is greater than that of the object on the right (9):</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span><span class="operator">::</span>current() <span class="operator">&gt;</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span>(<span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span><span class="operator">::</span>IOS<span class="operator">,</span> <span class="number">9</span>)
    ///
    /// </pre>
    /// <p>This allows expressions for multiple operating systems to be joined with a logical OR operator and still work as expected. For example:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">auto</span> current <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span><span class="operator">::</span>current();
    ///   <span class="keyword">if</span> (current <span class="operator">&gt;</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span><span class="operator">::</span>OSXYosemite <span class="operator">|</span><span class="operator">|</span>
    /// &#32;     current <span class="operator">&gt;</span><span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span>(<span class="type"><a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html#QOperatingSystemVersion">QOperatingSystemVersion</a></span><span class="operator">::</span>IOS<span class="operator">,</span> <span class="number">8</span>)) {
    /// &#32;     <span class="comment">// returns true on macOS &gt;= 10.10 and iOS &gt;= 8.0, but false on macOS &lt; 10.10 and iOS &lt; 8.0</span>
    ///   }
    ///
    /// </pre>
    /// <p>A more naive comparison algorithm might incorrectly return true on all versions of <a href="http://doc.qt.io/qt-5/internationalization.html#macos">macOS</a>, including Mac OS 9. This behavior is achieved by overloading the comparison operators to return <code>false</code> whenever the OS types of the <a href="http://doc.qt.io/qt-5/qoperatingsystemversion.html">QOperatingSystemVersion</a> instances being compared do not match. Be aware that due to this it can be the case <code>x</code> &gt;= y and <code>x</code> &lt; y are BOTH <code>false</code> for the same instances of <code>x</code> and <code>y</code>.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QOperatingSystemVersion_dQOperatingSystemVersion(
            self as *mut crate::QOperatingSystemVersion,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QStaticPlugin {
    /// <p><a href="http://doc.qt.io/qt-5/qstaticplugin.html">QStaticPlugin</a> is a struct containing a reference to a static plugin instance together with its meta data.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QStaticPlugin::~QStaticPlugin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstaticplugin.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p><a href="http://doc.qt.io/qt-5/qstaticplugin.html">QStaticPlugin</a> is a struct containing a reference to a static plugin instance together with its meta data.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QStaticPlugin_dQStaticPlugin(
            self as *mut crate::QStaticPlugin,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QSize {
    /// <p>The <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a> class defines the size of a two-dimensional object using integer point precision.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QSize::~QSize()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsize.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a> class defines the size of a two-dimensional object using integer point precision.</p>
    /// <p>A size is specified by a <a href="http://doc.qt.io/qt-5/qsize.html#width">width</a>() and a <a href="http://doc.qt.io/qt-5/qsize.html#height">height</a>(). It can be set in the constructor and changed using the <a href="http://doc.qt.io/qt-5/qsize.html#setWidth">setWidth</a>(), <a href="http://doc.qt.io/qt-5/qsize.html#setHeight">setHeight</a>(), or <a href="http://doc.qt.io/qt-5/qsize.html#scale">scale</a>() functions, or using arithmetic operators. A size can also be manipulated directly by retrieving references to the width and height using the <a href="http://doc.qt.io/qt-5/qsize.html#rwidth">rwidth</a>() and <a href="http://doc.qt.io/qt-5/qsize.html#rheight">rheight</a>() functions. Finally, the width and height can be swapped using the <a href="http://doc.qt.io/qt-5/qsize.html#transpose">transpose</a>() function.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qsize.html#isValid">isValid</a>() function determines if a size is valid (a valid size has both width and height greater than or equal to zero). The <a href="http://doc.qt.io/qt-5/qsize.html#isEmpty">isEmpty</a>() function returns <code>true</code> if either of the width and height is less than, or equal to, zero, while the <a href="http://doc.qt.io/qt-5/qsize.html#isNull">isNull</a>() function returns <code>true</code> only if both the width and the height is zero.</p>
    /// <p>Use the <a href="http://doc.qt.io/qt-5/qsize.html#expandedTo">expandedTo</a>() function to retrieve a size which holds the maximum height and width of <i>this</i> size and a given size. Similarly, the <a href="http://doc.qt.io/qt-5/qsize.html#boundedTo">boundedTo</a>() function returns a size which holds the minimum height and width of <i>this</i> size and a given size.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qsize.html">QSize</a> objects can be streamed as well as compared.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSize_dQSize(self as *mut crate::QSize)
    }
}

impl ::cpp_utils::CppDeletable for crate::QSizeF {
    /// <p>The <a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a> class defines the size of a two-dimensional object using floating point precision.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QSizeF::~QSizeF()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsizef.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a> class defines the size of a two-dimensional object using floating point precision.</p>
    /// <p>A size is specified by a <a href="http://doc.qt.io/qt-5/qsizef.html#width">width</a>() and a <a href="http://doc.qt.io/qt-5/qsizef.html#height">height</a>(). It can be set in the constructor and changed using the <a href="http://doc.qt.io/qt-5/qsizef.html#setWidth">setWidth</a>(), <a href="http://doc.qt.io/qt-5/qsizef.html#setHeight">setHeight</a>(), or <a href="http://doc.qt.io/qt-5/qsizef.html#scale">scale</a>() functions, or using arithmetic operators. A size can also be manipulated directly by retrieving references to the width and height using the <a href="http://doc.qt.io/qt-5/qsizef.html#rwidth">rwidth</a>() and <a href="http://doc.qt.io/qt-5/qsizef.html#rheight">rheight</a>() functions. Finally, the width and height can be swapped using the <a href="http://doc.qt.io/qt-5/qsizef.html#transpose">transpose</a>() function.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qsizef.html#isValid">isValid</a>() function determines if a size is valid. A valid size has both width and height greater than or equal to zero. The <a href="http://doc.qt.io/qt-5/qsizef.html#isEmpty">isEmpty</a>() function returns <code>true</code> if either of the width and height is <i>less</i> than (or equal to) zero, while the <a href="http://doc.qt.io/qt-5/qsizef.html#isNull">isNull</a>() function returns <code>true</code> only if both the width and the height is zero.</p>
    /// <p>Use the <a href="http://doc.qt.io/qt-5/qsizef.html#expandedTo">expandedTo</a>() function to retrieve a size which holds the maximum height and width of this size and a given size. Similarly, the <a href="http://doc.qt.io/qt-5/qsizef.html#boundedTo">boundedTo</a>() function returns a size which holds the minimum height and width of this size and a given size.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a> class also provides the <a href="http://doc.qt.io/qt-5/qsizef.html#toSize">toSize</a>() function returning a <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a> copy of this size, constructed by rounding the width and height to the nearest integers.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a> objects can be streamed as well as compared.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSizeF_dQSizeF(self as *mut crate::QSizeF)
    }
}

impl ::cpp_utils::CppDeletable for crate::QRect {
    /// <p>The <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> class defines a rectangle in the plane using integer precision.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QRect::~QRect()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrect.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> class defines a rectangle in the plane using integer precision.</p>
    /// <p>A rectangle is normally expressed as an upper-left corner and a size. The size (width and height) of a <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> is always equivalent to the mathematical rectangle that forms the basis for its rendering.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> can be constructed with a set of left, top, width and height integers, or from a <a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a> and a <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a>. The following code creates two identical rectangles.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qrect.html#QRect">QRect</a></span> r1(<span class="number">100</span><span class="operator">,</span> <span class="number">200</span><span class="operator">,</span> <span class="number">11</span><span class="operator">,</span> <span class="number">16</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qrect.html#QRect">QRect</a></span> r2(<span class="type"><a href="http://doc.qt.io/qt-5/qpoint.html">QPoint</a></span>(<span class="number">100</span><span class="operator">,</span> <span class="number">200</span>)<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsize.html">QSize</a></span>(<span class="number">11</span><span class="operator">,</span> <span class="number">16</span>));
    ///
    /// </pre>
    /// <p>There is a third constructor that creates a <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> using the top-left and bottom-right coordinates, but we recommend that you avoid using it. The rationale is that for historical reasons the values returned by the <a href="http://doc.qt.io/qt-5/qrect.html#bottom">bottom</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#right">right</a>() functions deviate from the true bottom-right corner of the rectangle.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> class provides a collection of functions that return the various rectangle coordinates, and enable manipulation of these. <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> also provide functions to move the rectangle relative to the various coordinates. In addition there is a <a href="http://doc.qt.io/qt-5/qrect.html#moveTo">moveTo</a>() function that moves the rectangle, leaving its top left corner at the given coordinates. Alternatively, the <a href="http://doc.qt.io/qt-5/qrect.html#translate">translate</a>() function moves the rectangle the given offset relative to the current position, and the <a href="http://doc.qt.io/qt-5/qrect.html#translated">translated</a>() function returns a translated copy of this rectangle.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qrect.html#size">size</a>() function returns the rectange's dimensions as a <a href="http://doc.qt.io/qt-5/qsize.html">QSize</a>. The dimensions can also be retrieved separately using the <a href="http://doc.qt.io/qt-5/qrect.html#width">width</a>() and <a href="http://doc.qt.io/qt-5/qrect.html#height">height</a>() functions. To manipulate the dimensions use the <a href="http://doc.qt.io/qt-5/qrect.html#setSize">setSize</a>(), <a href="http://doc.qt.io/qt-5/qrect.html#setWidth">setWidth</a>() or <a href="http://doc.qt.io/qt-5/qrect.html#setHeight">setHeight</a>() functions. Alternatively, the size can be changed by applying either of the functions setting the rectangle coordinates, for example, <a href="http://doc.qt.io/qt-5/qrect.html#setBottom">setBottom</a>() or <a href="http://doc.qt.io/qt-5/qrect.html#setRight">setRight</a>().</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qrect.html#contains">contains</a>() function tells whether a given point is inside the rectangle or not, and the <a href="http://doc.qt.io/qt-5/qrect.html#intersects">intersects</a>() function returns <code>true</code> if this rectangle intersects with a given rectangle. The <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> class also provides the <a href="http://doc.qt.io/qt-5/qrect.html#intersected">intersected</a>() function which returns the intersection rectangle, and the <a href="http://doc.qt.io/qt-5/qrect.html#united">united</a>() function which returns the rectangle that encloses the given rectangle and this:</p>
    /// <div class="table"><table class="generic">
    ///  <tbody><tr valign="top" class="odd"><td><img src="http://doc.qt.io/qt-5/images/qrect-intersect.png" alt=""></td><td><img src="http://doc.qt.io/qt-5/images/qrect-unite.png" alt=""></td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qrect.html#intersected">intersected</a>()</td><td><a href="http://doc.qt.io/qt-5/qrect.html#united">united</a>()</td></tr>
    /// </tbody></table></div>
    /// <p>The <a href="http://doc.qt.io/qt-5/qrect.html#isEmpty">isEmpty</a>() function returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qrect.html#left">left</a>() &gt; <a href="http://doc.qt.io/qt-5/qrect.html#right">right</a>() or <a href="http://doc.qt.io/qt-5/qrect.html#top">top</a>() &gt; <a href="http://doc.qt.io/qt-5/qrect.html#bottom">bottom</a>(). Note that an empty rectangle is not valid: The <a href="http://doc.qt.io/qt-5/qrect.html#isValid">isValid</a>() function returns <code>true</code> if <a href="http://doc.qt.io/qt-5/qrect.html#left">left</a>() &lt;= <a href="http://doc.qt.io/qt-5/qrect.html#right">right</a>() <i>and</i> <a href="http://doc.qt.io/qt-5/qrect.html#top">top</a>() &lt;= <a href="http://doc.qt.io/qt-5/qrect.html#bottom">bottom</a>(). A null rectangle (<a href="http://doc.qt.io/qt-5/qrect.html#isNull">isNull</a>() == true) on the other hand, has both width and height set to 0.</p>
    /// <p>Note that due to the way <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> and <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> are defined, an empty <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> is defined in essentially the same way as <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a>.</p>
    /// <p>Finally, <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> objects can be streamed as well as compared.</p>
    /// <a name="rendering"></a></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QRect_dQRect(self as *mut crate::QRect)
    }
}

impl ::cpp_utils::CppDeletable for crate::QRectF {
    /// <p>The <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> class defines a rectangle in the plane using floating point precision.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QRectF::~QRectF()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrectf.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> class defines a rectangle in the plane using floating point precision.</p>
    /// <p>A rectangle is normally expressed as an upper-left corner and a size. The size (width and height) of a <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> is always equivalent to the mathematical rectangle that forms the basis for its rendering.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> can be constructed with a set of left, top, width and height coordinates, or from a <a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a> and a <a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a>. The following code creates two identical rectangles.</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qrectf.html#QRectF">QRectF</a></span> r1(<span class="number">100.0</span><span class="operator">,</span> <span class="number">200.1</span><span class="operator">,</span> <span class="number">11.2</span><span class="operator">,</span> <span class="number">16.3</span>);
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qrectf.html#QRectF">QRectF</a></span> r2(<span class="type"><a href="http://doc.qt.io/qt-5/qpointf.html">QPointF</a></span>(<span class="number">100.0</span><span class="operator">,</span> <span class="number">200.1</span>)<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a></span>(<span class="number">11.2</span><span class="operator">,</span> <span class="number">16.3</span>));
    ///
    /// </pre>
    /// <p>There is also a third constructor creating a <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> from a <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a>, and a corresponding <a href="http://doc.qt.io/qt-5/qrectf.html#toRect">toRect</a>() function that returns a <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> object based on the values of this rectangle (note that the coordinates in the returned rectangle are rounded to the nearest integer).</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> class provides a collection of functions that return the various rectangle coordinates, and enable manipulation of these. <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> also provide functions to move the rectangle relative to the various coordinates. In addition there is a <a href="http://doc.qt.io/qt-5/qrectf.html#moveTo">moveTo</a>() function that moves the rectangle, leaving its top left corner at the given coordinates. Alternatively, the <a href="http://doc.qt.io/qt-5/qrectf.html#translate">translate</a>() function moves the rectangle the given offset relative to the current position, and the <a href="http://doc.qt.io/qt-5/qrectf.html#translated">translated</a>() function returns a translated copy of this rectangle.</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qrectf.html#size">size</a>() function returns the rectange's dimensions as a <a href="http://doc.qt.io/qt-5/qsizef.html">QSizeF</a>. The dimensions can also be retrieved separately using the <a href="http://doc.qt.io/qt-5/qrectf.html#width">width</a>() and <a href="http://doc.qt.io/qt-5/qrectf.html#height">height</a>() functions. To manipulate the dimensions use the <a href="http://doc.qt.io/qt-5/qrectf.html#setSize">setSize</a>(), <a href="http://doc.qt.io/qt-5/qrectf.html#setWidth">setWidth</a>() or <a href="http://doc.qt.io/qt-5/qrectf.html#setHeight">setHeight</a>() functions. Alternatively, the size can be changed by applying either of the functions setting the rectangle coordinates, for example, <a href="http://doc.qt.io/qt-5/qrectf.html#setBottom">setBottom</a>() or <a href="http://doc.qt.io/qt-5/qrectf.html#setRight">setRight</a>().</p>
    /// <p>The <a href="http://doc.qt.io/qt-5/qrectf.html#contains">contains</a>() function tells whether a given point is inside the rectangle or not, and the <a href="http://doc.qt.io/qt-5/qrectf.html#intersects">intersects</a>() function returns <code>true</code> if this rectangle intersects with a given rectangle (otherwise false). The <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> class also provides the <a href="http://doc.qt.io/qt-5/qrectf.html#intersected">intersected</a>() function which returns the intersection rectangle, and the <a href="http://doc.qt.io/qt-5/qrectf.html#united">united</a>() function which returns the rectangle that encloses the given rectangle and this:</p>
    /// <div class="table"><table class="generic">
    ///  <tbody><tr valign="top" class="odd"><td><img src="http://doc.qt.io/qt-5/images/qrect-intersect.png" alt=""></td><td><img src="http://doc.qt.io/qt-5/images/qrect-unite.png" alt=""></td></tr>
    /// <tr valign="top" class="even"><td><a href="http://doc.qt.io/qt-5/qrectf.html#intersected">intersected</a>()</td><td><a href="http://doc.qt.io/qt-5/qrectf.html#united">united</a>()</td></tr>
    /// </tbody></table></div>
    /// <p>The <a href="http://doc.qt.io/qt-5/qrectf.html#isEmpty">isEmpty</a>() function returns <code>true</code> if the rectangle's width or height is less than, or equal to, 0. Note that an empty rectangle is not valid: The <a href="http://doc.qt.io/qt-5/qrectf.html#isValid">isValid</a>() function returns <code>true</code> if both width and height is larger than 0. A null rectangle (<a href="http://doc.qt.io/qt-5/qrectf.html#isNull">isNull</a>() == true) on the other hand, has both width and height set to 0.</p>
    /// <p>Note that due to the way <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a> and <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> are defined, an empty <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> is defined in essentially the same way as <a href="http://doc.qt.io/qt-5/qrect.html">QRect</a>.</p>
    /// <p>Finally, <a href="http://doc.qt.io/qt-5/qrectf.html">QRectF</a> objects can be streamed as well as compared.</p>
    /// <a name="rendering"></a></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QRectF_dQRectF(self as *mut crate::QRectF)
    }
}

impl ::cpp_utils::CppDeletable for crate::QStringListModel {
    /// <p>The <a href="http://doc.qt.io/qt-5/qstringlistmodel.html">QStringListModel</a> class provides a model that supplies strings to views.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QStringListModel::~QStringListModel()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringlistmodel.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qstringlistmodel.html">QStringListModel</a> class provides a model that supplies strings to views.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qstringlistmodel.html">QStringListModel</a> is an editable model that can be used for simple cases where you need to display a number of strings in a view widget, such as a <a href="http://doc.qt.io/qt-5/qlistview.html">QListView</a> or a <a href="http://doc.qt.io/qt-5/qcombobox.html">QComboBox</a>.</p>
    /// <p>The model provides all the standard functions of an editable model, representing the data in the string list as a model with one column and a number of rows equal to the number of items in the list.</p>
    /// <p>Model indexes corresponding to items are obtained with the <a href="http://doc.qt.io/qt-5/qabstractlistmodel.html#index">index()</a> function, and item flags are obtained with <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#flags">flags</a>(). Item data is read with the <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#data">data</a>() function and written with <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#setData">setData</a>(). The number of rows (and number of items in the string list) can be found with the <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#rowCount">rowCount</a>() function.</p>
    /// <p>The model can be constructed with an existing string list, or strings can be set later with the <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#setStringList">setStringList</a>() convenience function. Strings can also be inserted in the usual way with the <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#insertRows">insertRows</a>() function, and removed with <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#removeRows">removeRows</a>(). The contents of the string list can be retrieved with the <a href="http://doc.qt.io/qt-5/qstringlistmodel.html#stringList">stringList</a>() convenience function.</p>
    /// <p>An example usage of <a href="http://doc.qt.io/qt-5/qstringlistmodel.html">QStringListModel</a>:</p>
    /// <pre class="cpp">
    ///
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstringlistmodel.html#QStringListModel">QStringListModel</a></span> <span class="operator">*</span>model <span class="operator">=</span> <span class="keyword">new</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstringlistmodel.html#QStringListModel">QStringListModel</a></span>();
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstringlist.html">QStringList</a></span> list;
    /// &#32;     list <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"a"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"b"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"c"</span>;
    /// &#32;     model<span class="operator">-</span><span class="operator">&gt;</span>setStringList(list);
    ///
    /// </pre></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QStringListModel_dQStringListModel(
            self as *mut crate::QStringListModel,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QUuid {
    /// <p>The <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> class stores a Universally Unique Identifier (UUID).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QUuid::~QUuid()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/quuid.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> class stores a Universally Unique Identifier (UUID).</p>
    /// <p>Using <i>U</i>niversally <i>U</i>nique <i>ID</i>entifiers (UUID) is a standard way to uniquely identify entities in a distributed computing environment. A UUID is a 16-byte (128-bit) number generated by some algorithm that is meant to guarantee that the UUID will be unique in the distributed computing environment where it is used. The acronym GUID is often used instead, <i>G</i>lobally <i>U</i>nique <i>ID</i>entifiers, but it refers to the same thing.</p>
    /// <a name="variant-field"></a><p>Actually, the GUID is one <i>variant</i> of UUID. Multiple variants are in use. Each UUID contains a bit field that specifies which type (variant) of UUID it is. Call <a href="http://doc.qt.io/qt-5/quuid.html#variant">variant</a>() to discover which type of UUID an instance of <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> contains. It extracts the three most significant bits of byte 8 of the 16 bytes. In <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a>, byte 8 is <code>QUuid::data4[0]</code>. If you create instances of <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> using the constructor that accepts all the numeric values as parameters, use the following table to set the three most significant bits of parameter <code>b1</code>, which becomes <code>QUuid::data4[0]</code> and contains the variant field in its three most significant bits. In the table, 'x' means <i>don't care</i>.</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>msb0</th><th>msb1</th><th>msb2</th><th>Variant</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>0</td><td>x</td><td>x</td><td>NCS (Network Computing System)</td></tr>
    /// <tr valign="top" class="even"><td>1</td><td>0</td><td>x</td><td>DCE (Distributed Computing Environment)</td></tr>
    /// <tr valign="top" class="odd"><td>1</td><td>1</td><td>0</td><td>Microsoft (GUID)</td></tr>
    /// <tr valign="top" class="even"><td>1</td><td>1</td><td>1</td><td>Reserved for future expansion</td></tr>
    /// </tbody></table></div>
    /// <a name="version-field"></a><p>If <a href="http://doc.qt.io/qt-5/quuid.html#variant">variant</a>() returns <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::DCE</a>, the UUID also contains a <i>version</i> field in the four most significant bits of <code>QUuid::data3</code>, and you can call <a href="http://doc.qt.io/qt-5/quuid.html#version">version</a>() to discover which version your <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> contains. If you create instances of <a href="http://doc.qt.io/qt-5/quuid.html">QUuid</a> using the constructor that accepts all the numeric values as parameters, use the following table to set the four most significant bits of parameter <code>w2</code>, which becomes <code>QUuid::data3</code> and contains the version field in its four most significant bits.</p>
    /// <div class="table"><table class="generic">
    ///  <thead><tr class="qt-style"><th>msb0</th><th>msb1</th><th>msb2</th><th>msb3</th><th>Version</th></tr></thead>
    /// <tbody><tr valign="top" class="odd"><td>0</td><td>0</td><td>0</td><td>1</td><td>Time</td></tr>
    /// <tr valign="top" class="even"><td>0</td><td>0</td><td>1</td><td>0</td><td>Embedded POSIX</td></tr>
    /// <tr valign="top" class="odd"><td>0</td><td>0</td><td>1</td><td>1</td><td>Md5(Name)</td></tr>
    /// <tr valign="top" class="even"><td>0</td><td>1</td><td>0</td><td>0</td><td>Random</td></tr>
    /// <tr valign="top" class="odd"><td>0</td><td>1</td><td>0</td><td>1</td><td>Sha1</td></tr>
    /// </tbody></table></div>
    /// <p>The field layouts for the DCE versions listed in the table above are specified in the <a href="http://www.ietf.org/rfc/rfc4122.txt">Network Working Group UUID Specification</a>.</p>
    /// <p>Most platforms provide a tool for generating new UUIDs, e.g. <code>uuidgen</code> and <code>guidgen</code>. You can also use <a href="http://doc.qt.io/qt-5/quuid.html#createUuid">createUuid</a>(). UUIDs generated by <a href="http://doc.qt.io/qt-5/quuid.html#createUuid">createUuid</a>() are of the random type. Their <a href="http://doc.qt.io/qt-5/quuid.html#Version-enum">QUuid::Version</a> bits are set to <a href="http://doc.qt.io/qt-5/quuid.html#Version-enum">QUuid::Random</a>, and their <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::Variant</a> bits are set to <a href="http://doc.qt.io/qt-5/quuid.html#Variant-enum">QUuid::DCE</a>. The rest of the UUID is composed of random numbers. Theoretically, this means there is a small chance that a UUID generated by <a href="http://doc.qt.io/qt-5/quuid.html#createUuid">createUuid</a>() will not be unique. But it is <a href="http://en.wikipedia.org/wiki/Universally_Unique_Identifier#Random_UUID_probability_of_duplicates">a <i>very</i> small chance</a>.</p>
    /// <p>UUIDs can be constructed from numeric values or from strings, or using the static <a href="http://doc.qt.io/qt-5/quuid.html#createUuid">createUuid</a>() function. They can be converted to a string with <a href="http://doc.qt.io/qt-5/quuid.html#toString">toString</a>(). UUIDs have a <a href="http://doc.qt.io/qt-5/quuid.html#variant">variant</a>() and a <a href="http://doc.qt.io/qt-5/quuid.html#version">version</a>(), and null UUIDs return true from <a href="http://doc.qt.io/qt-5/quuid.html#isNull">isNull</a>().</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QUuid_dQUuid(self as *mut crate::QUuid)
    }
}

impl ::cpp_utils::CppDeletable for crate::QXmlStreamAttributes {
    /// <p>The <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html">QXmlStreamAttributes</a> class represents a vector of <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html">QXmlStreamAttribute</a>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QXmlStreamAttributes::~QXmlStreamAttributes()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html">QXmlStreamAttributes</a> class represents a vector of <a href="http://doc.qt.io/qt-5/qxmlstreamattribute.html">QXmlStreamAttribute</a>.</p>
    /// <p>Attributes are returned by a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html">QXmlStreamReader</a> in <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#attributes">attributes()</a> when the reader reports a <a href="http://doc.qt.io/qt-5/qxmlstreamreader.html#TokenType-enum">start element</a>. The class can also be used with a <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html">QXmlStreamWriter</a> as an argument to <a href="http://doc.qt.io/qt-5/qxmlstreamwriter.html#writeAttributes">writeAttributes()</a>.</p>
    /// <p>The convenience function <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html#value">value</a>() loops over the vector and returns an attribute value for a given namespaceUri and an attribute's name.</p>
    /// <p>New attributes can be added with <a href="http://doc.qt.io/qt-5/qxmlstreamattributes.html#append">append</a>().</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QXmlStreamAttributes_dQXmlStreamAttributes(
            self as *mut crate::QXmlStreamAttributes,
        )
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QPairOfQStringQJsonValue>>
    for crate::QPairOfQStringQJsonValue
{
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(const QPair<QString, QJsonValue>& p1, const QPair<QString, QJsonValue>& p2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, p2: &::cpp_utils::Ref<crate::QPairOfQStringQJsonValue>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__203(
                self as *const crate::QPairOfQStringQJsonValue,
                p2.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>
    for crate::QPairOfDoubleQVariant
{
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(const QPair<double, QVariant>& p1, const QPair<double, QVariant>& p2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, p2: &::cpp_utils::Ref<crate::QPairOfDoubleQVariant>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__204(
                self as *const crate::QPairOfDoubleQVariant,
                p2.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QPairOfQStringQString>>
    for crate::QPairOfQStringQString
{
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(const QPair<QString, QString>& p1, const QPair<QString, QString>& p2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, p2: &::cpp_utils::Ref<crate::QPairOfQStringQString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__205(
                self as *const crate::QPairOfQStringQString,
                p2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>
    for crate::QPairOfDoubleQVariant
{
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QPair<double, QVariant>& p1, const QPair<double, QVariant>& p2)```</span>.
    fn lt(&self, p2: &::cpp_utils::Ref<crate::QPairOfDoubleQVariant>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_137(
                self as *const crate::QPairOfDoubleQVariant,
                p2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QPairOfQStringQString>>
    for crate::QPairOfQStringQString
{
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QPair<QString, QString>& p1, const QPair<QString, QString>& p2)```</span>.
    fn lt(&self, p2: &::cpp_utils::Ref<crate::QPairOfQStringQString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_138(
                self as *const crate::QPairOfQStringQString,
                p2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>
    for crate::QPairOfDoubleQVariant
{
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QPair<double, QVariant>& p1, const QPair<double, QVariant>& p2)```</span>.
    fn gt(&self, p2: &::cpp_utils::Ref<crate::QPairOfDoubleQVariant>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_140(
                self as *const crate::QPairOfDoubleQVariant,
                p2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QPairOfQStringQString>>
    for crate::QPairOfQStringQString
{
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QPair<QString, QString>& p1, const QPair<QString, QString>& p2)```</span>.
    fn gt(&self, p2: &::cpp_utils::Ref<crate::QPairOfQStringQString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_141(
                self as *const crate::QPairOfQStringQString,
                p2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>
    for crate::QPairOfDoubleQVariant
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QPair<double, QVariant>& p1, const QPair<double, QVariant>& p2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, p2: &::cpp_utils::Ref<crate::QPairOfDoubleQVariant>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__210(
                self as *const crate::QPairOfDoubleQVariant,
                p2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QPairOfQStringQString>>
    for crate::QPairOfQStringQString
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QPair<QString, QString>& p1, const QPair<QString, QString>& p2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, p2: &::cpp_utils::Ref<crate::QPairOfQStringQString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__211(
                self as *const crate::QPairOfQStringQString,
                p2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>
    for crate::QPairOfDoubleQVariant
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QPair<double, QVariant>& p1, const QPair<double, QVariant>& p2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, p2: &::cpp_utils::Ref<crate::QPairOfDoubleQVariant>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__213(
                self as *const crate::QPairOfDoubleQVariant,
                p2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QPairOfQStringQString>>
    for crate::QPairOfQStringQString
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QPair<QString, QString>& p1, const QPair<QString, QString>& p2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, p2: &::cpp_utils::Ref<crate::QPairOfQStringQString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__214(
                self as *const crate::QPairOfQStringQString,
                p2.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QListOfQString {
    /// <p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QString>::~QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#dtor.QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QString_dQList(self as *mut crate::QListOfQString)
    }
}

impl ::cpp_utils::CppDeletable for crate::QListOfQItemSelectionRange {
    /// <p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QItemSelectionRange>::~QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#dtor.QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_dQList(
            self as *mut crate::QListOfQItemSelectionRange,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QListOfQObject {
    /// <p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QObject*>::~QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#dtor.QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_dQList(self as *mut crate::QListOfQObject)
    }
}

impl ::cpp_utils::CppDeletable for crate::QListOfQVariant {
    /// <p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QVariant>::~QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#dtor.QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QVariant_dQList(self as *mut crate::QListOfQVariant)
    }
}

impl ::cpp_utils::CppDeletable for crate::QListOfQByteArray {
    /// <p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QByteArray>::~QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#dtor.QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_dQList(self as *mut crate::QListOfQByteArray)
    }
}

impl ::cpp_utils::CppDeletable for crate::QListOfTimerInfo {
    /// <p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QAbstractEventDispatcher::TimerInfo>::~QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#dtor.QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_dQList(
            self as *mut crate::QListOfTimerInfo,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QListOfQModelIndex {
    /// <p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QModelIndex>::~QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#dtor.QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_dQList(
            self as *mut crate::QListOfQModelIndex,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QListOfQPersistentModelIndex {
    /// <p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QPersistentModelIndex>::~QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#dtor.QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_dQList(
            self as *mut crate::QListOfQPersistentModelIndex,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QListOfQAbstractState {
    /// <p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QAbstractState*>::~QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#dtor.QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_dQList(
            self as *mut crate::QListOfQAbstractState,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QListOfQAbstractAnimation {
    /// <p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QAbstractAnimation*>::~QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#dtor.QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_dQList(
            self as *mut crate::QListOfQAbstractAnimation,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QListOfDayOfWeek {
    /// <p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<Qt::DayOfWeek>::~QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#dtor.QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_dQList(
            self as *mut crate::QListOfDayOfWeek,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QListOfQLocale {
    /// <p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QLocale>::~QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#dtor.QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_dQList(self as *mut crate::QListOfQLocale)
    }
}

impl ::cpp_utils::CppDeletable for crate::QListOfCountry {
    /// <p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QLocale::Country>::~QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#dtor.QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_dQList(
            self as *mut crate::QListOfCountry,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QListOfQCommandLineOption {
    /// <p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QCommandLineOption>::~QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#dtor.QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_dQList(
            self as *mut crate::QListOfQCommandLineOption,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QListOfQFileInfo {
    /// <p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QFileInfo>::~QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#dtor.QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_dQList(self as *mut crate::QListOfQFileInfo)
    }
}

impl ::cpp_utils::CppDeletable for crate::QListOfQUrl {
    /// <p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QUrl>::~QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#dtor.QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QUrl_dQList(self as *mut crate::QListOfQUrl)
    }
}

impl ::cpp_utils::CppDeletable for crate::QListOfQMimeType {
    /// <p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QMimeType>::~QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#dtor.QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_dQList(self as *mut crate::QListOfQMimeType)
    }
}

impl ::cpp_utils::CppDeletable for crate::QListOfQAbstractTransition {
    /// <p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QAbstractTransition*>::~QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#dtor.QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_dQList(
            self as *mut crate::QListOfQAbstractTransition,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QListOfQStorageInfo {
    /// <p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QStorageInfo>::~QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#dtor.QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_dQList(
            self as *mut crate::QListOfQStorageInfo,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QListOfInt {
    /// <p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<int>::~QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#dtor.QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_int_dQList(self as *mut crate::QListOfInt)
    }
}

impl ::cpp_utils::CppDeletable for crate::QListOfQPairOfQStringQString {
    /// <p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QList<QPair<QString, QString>>::~QList()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#dtor.QList">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the list. References to the values in the list and all iterators of this list become invalid.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_dQList(
            self as *mut crate::QListOfQPairOfQStringQString,
        )
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QListOfQString>> for crate::QListOfQString {
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QString>::operator==(const QList<QString>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    /// <p>Two lists are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, l: &::cpp_utils::Ref<crate::QListOfQString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QString_operator__(
                self as *const crate::QListOfQString,
                l.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QListOfQItemSelectionRange>>
    for crate::QListOfQItemSelectionRange
{
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QItemSelectionRange>::operator==(const QList<QItemSelectionRange>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    /// <p>Two lists are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, l: &::cpp_utils::Ref<crate::QListOfQItemSelectionRange>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_operator__(
                self as *const crate::QListOfQItemSelectionRange,
                l.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QListOfQObject>> for crate::QListOfQObject {
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QObject*>::operator==(const QList<QObject*>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    /// <p>Two lists are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, l: &::cpp_utils::Ref<crate::QListOfQObject>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_operator__(
                self as *const crate::QListOfQObject,
                l.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QListOfQVariant>> for crate::QListOfQVariant {
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QVariant>::operator==(const QList<QVariant>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    /// <p>Two lists are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, l: &::cpp_utils::Ref<crate::QListOfQVariant>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QVariant_operator__(
                self as *const crate::QListOfQVariant,
                l.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QListOfQByteArray>>
    for crate::QListOfQByteArray
{
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QByteArray>::operator==(const QList<QByteArray>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    /// <p>Two lists are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, l: &::cpp_utils::Ref<crate::QListOfQByteArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_operator__(
                self as *const crate::QListOfQByteArray,
                l.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QListOfQModelIndex>>
    for crate::QListOfQModelIndex
{
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QModelIndex>::operator==(const QList<QModelIndex>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    /// <p>Two lists are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, l: &::cpp_utils::Ref<crate::QListOfQModelIndex>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_operator__(
                self as *const crate::QListOfQModelIndex,
                l.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>>
    for crate::QListOfQPersistentModelIndex
{
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QPersistentModelIndex>::operator==(const QList<QPersistentModelIndex>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    /// <p>Two lists are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, l: &::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_operator__(
                self as *const crate::QListOfQPersistentModelIndex,
                l.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QListOfQAbstractState>>
    for crate::QListOfQAbstractState
{
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractState*>::operator==(const QList<QAbstractState*>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    /// <p>Two lists are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, l: &::cpp_utils::Ref<crate::QListOfQAbstractState>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_operator__(
                self as *const crate::QListOfQAbstractState,
                l.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QListOfQAbstractAnimation>>
    for crate::QListOfQAbstractAnimation
{
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractAnimation*>::operator==(const QList<QAbstractAnimation*>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    /// <p>Two lists are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, l: &::cpp_utils::Ref<crate::QListOfQAbstractAnimation>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_operator__(
                self as *const crate::QListOfQAbstractAnimation,
                l.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QListOfDayOfWeek>> for crate::QListOfDayOfWeek {
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<Qt::DayOfWeek>::operator==(const QList<Qt::DayOfWeek>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    /// <p>Two lists are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, l: &::cpp_utils::Ref<crate::QListOfDayOfWeek>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_operator__(
                self as *const crate::QListOfDayOfWeek,
                l.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QListOfQLocale>> for crate::QListOfQLocale {
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale>::operator==(const QList<QLocale>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    /// <p>Two lists are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, l: &::cpp_utils::Ref<crate::QListOfQLocale>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_operator__(
                self as *const crate::QListOfQLocale,
                l.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QListOfCountry>> for crate::QListOfCountry {
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QLocale::Country>::operator==(const QList<QLocale::Country>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    /// <p>Two lists are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, l: &::cpp_utils::Ref<crate::QListOfCountry>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_operator__(
                self as *const crate::QListOfCountry,
                l.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QListOfQFileInfo>> for crate::QListOfQFileInfo {
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QFileInfo>::operator==(const QList<QFileInfo>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    /// <p>Two lists are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, l: &::cpp_utils::Ref<crate::QListOfQFileInfo>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_operator__(
                self as *const crate::QListOfQFileInfo,
                l.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QListOfQUrl>> for crate::QListOfQUrl {
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QUrl>::operator==(const QList<QUrl>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    /// <p>Two lists are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, l: &::cpp_utils::Ref<crate::QListOfQUrl>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_operator__(
                self as *const crate::QListOfQUrl,
                l.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QListOfQMimeType>> for crate::QListOfQMimeType {
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QMimeType>::operator==(const QList<QMimeType>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    /// <p>Two lists are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, l: &::cpp_utils::Ref<crate::QListOfQMimeType>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_operator__(
                self as *const crate::QListOfQMimeType,
                l.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QListOfQAbstractTransition>>
    for crate::QListOfQAbstractTransition
{
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QAbstractTransition*>::operator==(const QList<QAbstractTransition*>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    /// <p>Two lists are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, l: &::cpp_utils::Ref<crate::QListOfQAbstractTransition>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_operator__(
                self as *const crate::QListOfQAbstractTransition,
                l.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QListOfQStorageInfo>>
    for crate::QListOfQStorageInfo
{
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QStorageInfo>::operator==(const QList<QStorageInfo>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    /// <p>Two lists are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, l: &::cpp_utils::Ref<crate::QListOfQStorageInfo>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_operator__(
                self as *const crate::QListOfQStorageInfo,
                l.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QListOfInt>> for crate::QListOfInt {
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<int>::operator==(const QList<int>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    /// <p>Two lists are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, l: &::cpp_utils::Ref<crate::QListOfInt>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_int_operator__(
                self as *const crate::QListOfInt,
                l.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>>
    for crate::QListOfQPairOfQStringQString
{
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QList<QPair<QString, QString>>::operator==(const QList<QPair<QString, QString>>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this list; otherwise returns false.</p>
    /// <p>Two lists are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, l: &::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_operator__(
                self as *const crate::QListOfQPairOfQStringQString,
                l.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QListOfQString {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_string::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QString>::iterator QList<QString>::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_string::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QString_begin(self as *mut crate::QListOfQString)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QListOfQItemSelectionRange {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::iterator QList<QItemSelectionRange>::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_begin(
                self as *mut crate::QListOfQItemSelectionRange,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QListOfQObject {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_object::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::iterator QList<QObject*>::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_object::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_begin(
                self as *mut crate::QListOfQObject,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QListOfQVariant {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_variant::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::iterator QList<QVariant>::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_variant::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QVariant_begin(self as *mut crate::QListOfQVariant)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QListOfQByteArray {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::iterator QList<QByteArray>::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_begin(
                self as *mut crate::QListOfQByteArray,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QListOfTimerInfo {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_timer_info::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::iterator QList<QAbstractEventDispatcher::TimerInfo>::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_timer_info::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_begin(
                self as *mut crate::QListOfTimerInfo,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QListOfQModelIndex {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_model_index::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::iterator QList<QModelIndex>::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_model_index::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_begin(
                self as *mut crate::QListOfQModelIndex,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QListOfQPersistentModelIndex {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::iterator QList<QPersistentModelIndex>::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_begin(
                self as *mut crate::QListOfQPersistentModelIndex,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QListOfQAbstractState {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::iterator QList<QAbstractState*>::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_begin(
                self as *mut crate::QListOfQAbstractState,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QListOfQAbstractAnimation {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::iterator QList<QAbstractAnimation*>::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_begin(
                self as *mut crate::QListOfQAbstractAnimation,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QListOfDayOfWeek {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_day_of_week::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::iterator QList<Qt::DayOfWeek>::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_day_of_week::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_begin(
                self as *mut crate::QListOfDayOfWeek,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QListOfQLocale {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_locale::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::iterator QList<QLocale>::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_locale::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_begin(self as *mut crate::QListOfQLocale)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QListOfCountry {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_country::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::iterator QList<QLocale::Country>::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_country::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_begin(
                self as *mut crate::QListOfCountry,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QListOfQCommandLineOption {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::iterator QList<QCommandLineOption>::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_begin(
                self as *mut crate::QListOfQCommandLineOption,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QListOfQFileInfo {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_file_info::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::iterator QList<QFileInfo>::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_file_info::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_begin(
                self as *mut crate::QListOfQFileInfo,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QListOfQUrl {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_url::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::iterator QList<QUrl>::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_url::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_begin(self as *mut crate::QListOfQUrl)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QListOfQMimeType {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::iterator QList<QMimeType>::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_begin(
                self as *mut crate::QListOfQMimeType,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QListOfQAbstractTransition {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::iterator QList<QAbstractTransition*>::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_begin(
                self as *mut crate::QListOfQAbstractTransition,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QListOfQStorageInfo {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::iterator QList<QStorageInfo>::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_begin(
                self as *mut crate::QListOfQStorageInfo,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QListOfInt {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_int::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<int>::iterator QList<int>::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_int::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_int_begin(self as *mut crate::QListOfInt)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QListOfQPairOfQStringQString {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::iterator QList<QPair<QString, QString>>::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#end">end</a>().</p></div>
    fn begin_mut(
        self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_begin(
                self as *mut crate::QListOfQPairOfQStringQString,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QListOfQString {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_string::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QString>::const_iterator QList<QString>::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_string::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QString_begin1(self as *const crate::QListOfQString)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QListOfQItemSelectionRange {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::const_iterator QList<QItemSelectionRange>::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_begin1(
                self as *const crate::QListOfQItemSelectionRange,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QListOfQObject {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_object::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::const_iterator QList<QObject*>::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_object::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_begin1(
                self as *const crate::QListOfQObject,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QListOfQVariant {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_variant::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::const_iterator QList<QVariant>::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_variant::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QVariant_begin1(
                self as *const crate::QListOfQVariant,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QListOfQByteArray {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::const_iterator QList<QByteArray>::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_begin1(
                self as *const crate::QListOfQByteArray,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QListOfTimerInfo {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_timer_info::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::const_iterator QList<QAbstractEventDispatcher::TimerInfo>::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_list_of_timer_info::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_begin1(
                self as *const crate::QListOfTimerInfo,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QListOfQModelIndex {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_model_index::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::const_iterator QList<QModelIndex>::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_model_index::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_begin1(
                self as *const crate::QListOfQModelIndex,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QListOfQPersistentModelIndex {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::const_iterator QList<QPersistentModelIndex>::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn begin(
        self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_begin1(
                self as *const crate::QListOfQPersistentModelIndex,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QListOfQAbstractState {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::const_iterator QList<QAbstractState*>::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_begin1(
                self as *const crate::QListOfQAbstractState,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QListOfQAbstractAnimation {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::const_iterator QList<QAbstractAnimation*>::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_begin1(
                self as *const crate::QListOfQAbstractAnimation,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QListOfDayOfWeek {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_day_of_week::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::const_iterator QList<Qt::DayOfWeek>::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_list_of_day_of_week::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_begin1(
                self as *const crate::QListOfDayOfWeek,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QListOfQLocale {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_locale::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::const_iterator QList<QLocale>::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_locale::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_begin1(self as *const crate::QListOfQLocale)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QListOfCountry {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_country::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::const_iterator QList<QLocale::Country>::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_list_of_country::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_begin1(
                self as *const crate::QListOfCountry,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QListOfQCommandLineOption {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::const_iterator QList<QCommandLineOption>::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_begin1(
                self as *const crate::QListOfQCommandLineOption,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QListOfQFileInfo {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_file_info::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::const_iterator QList<QFileInfo>::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_file_info::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_begin1(
                self as *const crate::QListOfQFileInfo,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QListOfQUrl {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_url::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::const_iterator QList<QUrl>::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_url::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_begin1(self as *const crate::QListOfQUrl)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QListOfQMimeType {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::const_iterator QList<QMimeType>::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_begin1(
                self as *const crate::QListOfQMimeType,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QListOfQAbstractTransition {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::const_iterator QList<QAbstractTransition*>::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_begin1(
                self as *const crate::QListOfQAbstractTransition,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QListOfQStorageInfo {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::const_iterator QList<QStorageInfo>::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_begin1(
                self as *const crate::QListOfQStorageInfo,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QListOfInt {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_int::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<int>::const_iterator QList<int>::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_list_of_int::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_int_begin1(self as *const crate::QListOfInt)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QListOfQPairOfQStringQString {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::const_iterator QList<QPair<QString, QString>>::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn begin(
        self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_begin1(
                self as *const crate::QListOfQPairOfQStringQString,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QListOfQString {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_string::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QString>::iterator QList<QString>::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_string::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QString_end(self as *mut crate::QListOfQString)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QListOfQItemSelectionRange {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::iterator QList<QItemSelectionRange>::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_end(
                self as *mut crate::QListOfQItemSelectionRange,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QListOfQObject {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_object::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::iterator QList<QObject*>::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_object::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_end(self as *mut crate::QListOfQObject)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QListOfQVariant {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_variant::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::iterator QList<QVariant>::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_variant::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QVariant_end(self as *mut crate::QListOfQVariant)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QListOfQByteArray {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::iterator QList<QByteArray>::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_end(
                self as *mut crate::QListOfQByteArray,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QListOfTimerInfo {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_timer_info::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::iterator QList<QAbstractEventDispatcher::TimerInfo>::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_timer_info::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_end(
                self as *mut crate::QListOfTimerInfo,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QListOfQModelIndex {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_model_index::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::iterator QList<QModelIndex>::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_model_index::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_end(
                self as *mut crate::QListOfQModelIndex,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QListOfQPersistentModelIndex {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::iterator QList<QPersistentModelIndex>::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_end(
                self as *mut crate::QListOfQPersistentModelIndex,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QListOfQAbstractState {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::iterator QList<QAbstractState*>::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_end(
                self as *mut crate::QListOfQAbstractState,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QListOfQAbstractAnimation {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::iterator QList<QAbstractAnimation*>::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_end(
                self as *mut crate::QListOfQAbstractAnimation,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QListOfDayOfWeek {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_day_of_week::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::iterator QList<Qt::DayOfWeek>::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_day_of_week::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_end(
                self as *mut crate::QListOfDayOfWeek,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QListOfQLocale {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_locale::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::iterator QList<QLocale>::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_locale::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_end(self as *mut crate::QListOfQLocale)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QListOfCountry {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_country::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::iterator QList<QLocale::Country>::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_country::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_end(
                self as *mut crate::QListOfCountry,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QListOfQCommandLineOption {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::iterator QList<QCommandLineOption>::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_end(
                self as *mut crate::QListOfQCommandLineOption,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QListOfQFileInfo {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_file_info::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::iterator QList<QFileInfo>::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_file_info::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_end(self as *mut crate::QListOfQFileInfo)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QListOfQUrl {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_url::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::iterator QList<QUrl>::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_url::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_end(self as *mut crate::QListOfQUrl)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QListOfQMimeType {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::iterator QList<QMimeType>::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_end(self as *mut crate::QListOfQMimeType)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QListOfQAbstractTransition {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::iterator QList<QAbstractTransition*>::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_end(
                self as *mut crate::QListOfQAbstractTransition,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QListOfQStorageInfo {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::iterator QList<QStorageInfo>::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_end(
                self as *mut crate::QListOfQStorageInfo,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QListOfInt {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_int::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<int>::iterator QList<int>::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_list_of_int::Iterator> {
        let ffi_result =
            unsafe { crate::__ffi::ctr_qt_core_ffi_QList_int_end(self as *mut crate::QListOfInt) };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QListOfQPairOfQStringQString {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::iterator QList<QPair<QString, QString>>::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(
        self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_end(
                self as *mut crate::QListOfQPairOfQStringQString,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QListOfQString {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_string::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QString>::const_iterator QList<QString>::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_string::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QString_end1(self as *const crate::QListOfQString)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QListOfQItemSelectionRange {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>::const_iterator QList<QItemSelectionRange>::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_item_selection_range::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_end1(
                self as *const crate::QListOfQItemSelectionRange,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QListOfQObject {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_object::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QObject*>::const_iterator QList<QObject*>::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_object::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_end1(
                self as *const crate::QListOfQObject,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QListOfQVariant {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_variant::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant>::const_iterator QList<QVariant>::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_variant::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QVariant_end1(self as *const crate::QListOfQVariant)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QListOfQByteArray {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>::const_iterator QList<QByteArray>::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_byte_array::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_end1(
                self as *const crate::QListOfQByteArray,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QListOfTimerInfo {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_timer_info::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>::const_iterator QList<QAbstractEventDispatcher::TimerInfo>::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_list_of_timer_info::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_end1(
                self as *const crate::QListOfTimerInfo,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QListOfQModelIndex {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_model_index::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>::const_iterator QList<QModelIndex>::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_model_index::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_end1(
                self as *const crate::QListOfQModelIndex,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QListOfQPersistentModelIndex {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>::const_iterator QList<QPersistentModelIndex>::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_persistent_model_index::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_end1(
                self as *const crate::QListOfQPersistentModelIndex,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QListOfQAbstractState {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>::const_iterator QList<QAbstractState*>::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_state::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_end1(
                self as *const crate::QListOfQAbstractState,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QListOfQAbstractAnimation {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>::const_iterator QList<QAbstractAnimation*>::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_animation::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_end1(
                self as *const crate::QListOfQAbstractAnimation,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QListOfDayOfWeek {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_day_of_week::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>::const_iterator QList<Qt::DayOfWeek>::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_list_of_day_of_week::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_end1(
                self as *const crate::QListOfDayOfWeek,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QListOfQLocale {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_locale::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale>::const_iterator QList<QLocale>::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_locale::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_end1(self as *const crate::QListOfQLocale)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QListOfCountry {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_country::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>::const_iterator QList<QLocale::Country>::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_list_of_country::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_end1(
                self as *const crate::QListOfCountry,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QListOfQCommandLineOption {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>::const_iterator QList<QCommandLineOption>::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_command_line_option::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_end1(
                self as *const crate::QListOfQCommandLineOption,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QListOfQFileInfo {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_file_info::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>::const_iterator QList<QFileInfo>::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_file_info::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_end1(
                self as *const crate::QListOfQFileInfo,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QListOfQUrl {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_url::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QUrl>::const_iterator QList<QUrl>::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_url::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_end1(self as *const crate::QListOfQUrl)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QListOfQMimeType {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>::const_iterator QList<QMimeType>::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_mime_type::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_end1(
                self as *const crate::QListOfQMimeType,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QListOfQAbstractTransition {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>::const_iterator QList<QAbstractTransition*>::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_abstract_transition::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_end1(
                self as *const crate::QListOfQAbstractTransition,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QListOfQStorageInfo {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>::const_iterator QList<QStorageInfo>::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_list_of_q_storage_info::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_end1(
                self as *const crate::QListOfQStorageInfo,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QListOfInt {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_int::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<int>::const_iterator QList<int>::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_list_of_int::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_int_end1(self as *const crate::QListOfInt)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QListOfQPairOfQStringQString {
    type Output = ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>::const_iterator QList<QPair<QString, QString>>::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn end(
        self,
    ) -> ::cpp_utils::CppBox<crate::q_list_of_q_pair_of_q_string_q_string::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_end1(
                self as *const crate::QListOfQPairOfQStringQString,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QListOfQString>> for crate::QListOfQString {
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QString>& QList<QString>::operator+=(const QList<QString>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QListOfQString>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QString_operator__4(
                self as *mut crate::QListOfQString,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QListOfQItemSelectionRange>>
    for crate::QListOfQItemSelectionRange
{
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>& QList<QItemSelectionRange>::operator+=(const QList<QItemSelectionRange>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QListOfQItemSelectionRange>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_operator__4(
                self as *mut crate::QListOfQItemSelectionRange,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QListOfQObject>> for crate::QListOfQObject {
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QObject*>& QList<QObject*>::operator+=(const QList<QObject*>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QListOfQObject>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_operator__4(
                self as *mut crate::QListOfQObject,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QListOfQVariant>> for crate::QListOfQVariant {
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant>& QList<QVariant>::operator+=(const QList<QVariant>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QListOfQVariant>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QVariant_operator__4(
                self as *mut crate::QListOfQVariant,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QListOfQByteArray>>
    for crate::QListOfQByteArray
{
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>& QList<QByteArray>::operator+=(const QList<QByteArray>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QListOfQByteArray>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_operator__4(
                self as *mut crate::QListOfQByteArray,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QListOfTimerInfo>> for crate::QListOfTimerInfo {
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>& QList<QAbstractEventDispatcher::TimerInfo>::operator+=(const QList<QAbstractEventDispatcher::TimerInfo>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QListOfTimerInfo>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_operator__4(
                self as *mut crate::QListOfTimerInfo,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QListOfQModelIndex>>
    for crate::QListOfQModelIndex
{
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>& QList<QModelIndex>::operator+=(const QList<QModelIndex>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QListOfQModelIndex>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_operator__4(
                self as *mut crate::QListOfQModelIndex,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>>
    for crate::QListOfQPersistentModelIndex
{
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>& QList<QPersistentModelIndex>::operator+=(const QList<QPersistentModelIndex>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_operator__4(
                self as *mut crate::QListOfQPersistentModelIndex,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QListOfQAbstractState>>
    for crate::QListOfQAbstractState
{
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>& QList<QAbstractState*>::operator+=(const QList<QAbstractState*>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QListOfQAbstractState>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_operator__4(
                self as *mut crate::QListOfQAbstractState,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QListOfQAbstractAnimation>>
    for crate::QListOfQAbstractAnimation
{
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>& QList<QAbstractAnimation*>::operator+=(const QList<QAbstractAnimation*>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QListOfQAbstractAnimation>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_operator__4(
                self as *mut crate::QListOfQAbstractAnimation,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QListOfDayOfWeek>> for crate::QListOfDayOfWeek {
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>& QList<Qt::DayOfWeek>::operator+=(const QList<Qt::DayOfWeek>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QListOfDayOfWeek>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_operator__4(
                self as *mut crate::QListOfDayOfWeek,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QListOfQLocale>> for crate::QListOfQLocale {
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale>& QList<QLocale>::operator+=(const QList<QLocale>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QListOfQLocale>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_operator__4(
                self as *mut crate::QListOfQLocale,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QListOfCountry>> for crate::QListOfCountry {
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>& QList<QLocale::Country>::operator+=(const QList<QLocale::Country>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QListOfCountry>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_operator__4(
                self as *mut crate::QListOfCountry,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QListOfQCommandLineOption>>
    for crate::QListOfQCommandLineOption
{
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>& QList<QCommandLineOption>::operator+=(const QList<QCommandLineOption>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QListOfQCommandLineOption>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_operator__4(
                self as *mut crate::QListOfQCommandLineOption,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QListOfQFileInfo>> for crate::QListOfQFileInfo {
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>& QList<QFileInfo>::operator+=(const QList<QFileInfo>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QListOfQFileInfo>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_operator__4(
                self as *mut crate::QListOfQFileInfo,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QListOfQUrl>> for crate::QListOfQUrl {
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QUrl>& QList<QUrl>::operator+=(const QList<QUrl>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QListOfQUrl>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_operator__4(
                self as *mut crate::QListOfQUrl,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QListOfQMimeType>> for crate::QListOfQMimeType {
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>& QList<QMimeType>::operator+=(const QList<QMimeType>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QListOfQMimeType>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_operator__4(
                self as *mut crate::QListOfQMimeType,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QListOfQAbstractTransition>>
    for crate::QListOfQAbstractTransition
{
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>& QList<QAbstractTransition*>::operator+=(const QList<QAbstractTransition*>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QListOfQAbstractTransition>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_operator__4(
                self as *mut crate::QListOfQAbstractTransition,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QListOfQStorageInfo>>
    for crate::QListOfQStorageInfo
{
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>& QList<QStorageInfo>::operator+=(const QList<QStorageInfo>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QListOfQStorageInfo>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_operator__4(
                self as *mut crate::QListOfQStorageInfo,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QListOfInt>> for crate::QListOfInt {
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<int>& QList<int>::operator+=(const QList<int>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QListOfInt>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_int_operator__4(
                self as *mut crate::QListOfInt,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>>
    for crate::QListOfQPairOfQStringQString
{
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>& QList<QPair<QString, QString>>::operator+=(const QList<QPair<QString, QString>>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_operator__4(
                self as *mut crate::QListOfQPairOfQStringQString,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QListOfQString>> for &crate::QListOfQString {
    type Output = ::cpp_utils::CppBox<crate::QListOfQString>;
    /// <p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QString> QList<QString>::operator+(const QList<QString>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQString>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQString> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QString_operator_2(
                self as *const crate::QListOfQString,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QListOfQItemSelectionRange>>
    for &crate::QListOfQItemSelectionRange
{
    type Output = ::cpp_utils::CppBox<crate::QListOfQItemSelectionRange>;
    /// <p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange> QList<QItemSelectionRange>::operator+(const QList<QItemSelectionRange>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQItemSelectionRange>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQItemSelectionRange> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_operator_2(
                self as *const crate::QListOfQItemSelectionRange,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QListOfQObject>> for &crate::QListOfQObject {
    type Output = ::cpp_utils::CppBox<crate::QListOfQObject>;
    /// <p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QObject*> QList<QObject*>::operator+(const QList<QObject*>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQObject>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQObject> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_operator_2(
                self as *const crate::QListOfQObject,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QListOfQVariant>> for &crate::QListOfQVariant {
    type Output = ::cpp_utils::CppBox<crate::QListOfQVariant>;
    /// <p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant> QList<QVariant>::operator+(const QList<QVariant>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQVariant>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQVariant> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QVariant_operator_2(
                self as *const crate::QListOfQVariant,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QListOfQByteArray>> for &crate::QListOfQByteArray {
    type Output = ::cpp_utils::CppBox<crate::QListOfQByteArray>;
    /// <p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QByteArray> QList<QByteArray>::operator+(const QList<QByteArray>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQByteArray>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQByteArray> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_operator_2(
                self as *const crate::QListOfQByteArray,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QListOfTimerInfo>> for &crate::QListOfTimerInfo {
    type Output = ::cpp_utils::CppBox<crate::QListOfTimerInfo>;
    /// <p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo> QList<QAbstractEventDispatcher::TimerInfo>::operator+(const QList<QAbstractEventDispatcher::TimerInfo>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QListOfTimerInfo>,
    ) -> ::cpp_utils::CppBox<crate::QListOfTimerInfo> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_operator_2(
                self as *const crate::QListOfTimerInfo,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QListOfQModelIndex>> for &crate::QListOfQModelIndex {
    type Output = ::cpp_utils::CppBox<crate::QListOfQModelIndex>;
    /// <p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex> QList<QModelIndex>::operator+(const QList<QModelIndex>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQModelIndex>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQModelIndex> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_operator_2(
                self as *const crate::QListOfQModelIndex,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>>
    for &crate::QListOfQPersistentModelIndex
{
    type Output = ::cpp_utils::CppBox<crate::QListOfQPersistentModelIndex>;
    /// <p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex> QList<QPersistentModelIndex>::operator+(const QList<QPersistentModelIndex>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQPersistentModelIndex> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_operator_2(
                self as *const crate::QListOfQPersistentModelIndex,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QListOfQAbstractState>>
    for &crate::QListOfQAbstractState
{
    type Output = ::cpp_utils::CppBox<crate::QListOfQAbstractState>;
    /// <p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*> QList<QAbstractState*>::operator+(const QList<QAbstractState*>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQAbstractState>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractState> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_operator_2(
                self as *const crate::QListOfQAbstractState,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QListOfQAbstractAnimation>>
    for &crate::QListOfQAbstractAnimation
{
    type Output = ::cpp_utils::CppBox<crate::QListOfQAbstractAnimation>;
    /// <p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*> QList<QAbstractAnimation*>::operator+(const QList<QAbstractAnimation*>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQAbstractAnimation>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractAnimation> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_operator_2(
                self as *const crate::QListOfQAbstractAnimation,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QListOfDayOfWeek>> for &crate::QListOfDayOfWeek {
    type Output = ::cpp_utils::CppBox<crate::QListOfDayOfWeek>;
    /// <p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek> QList<Qt::DayOfWeek>::operator+(const QList<Qt::DayOfWeek>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QListOfDayOfWeek>,
    ) -> ::cpp_utils::CppBox<crate::QListOfDayOfWeek> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_operator_2(
                self as *const crate::QListOfDayOfWeek,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QListOfQLocale>> for &crate::QListOfQLocale {
    type Output = ::cpp_utils::CppBox<crate::QListOfQLocale>;
    /// <p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale> QList<QLocale>::operator+(const QList<QLocale>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQLocale>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQLocale> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_operator_2(
                self as *const crate::QListOfQLocale,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QListOfCountry>> for &crate::QListOfCountry {
    type Output = ::cpp_utils::CppBox<crate::QListOfCountry>;
    /// <p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country> QList<QLocale::Country>::operator+(const QList<QLocale::Country>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QListOfCountry>,
    ) -> ::cpp_utils::CppBox<crate::QListOfCountry> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_operator_2(
                self as *const crate::QListOfCountry,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QListOfQCommandLineOption>>
    for &crate::QListOfQCommandLineOption
{
    type Output = ::cpp_utils::CppBox<crate::QListOfQCommandLineOption>;
    /// <p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption> QList<QCommandLineOption>::operator+(const QList<QCommandLineOption>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQCommandLineOption>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQCommandLineOption> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_operator_2(
                self as *const crate::QListOfQCommandLineOption,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QListOfQFileInfo>> for &crate::QListOfQFileInfo {
    type Output = ::cpp_utils::CppBox<crate::QListOfQFileInfo>;
    /// <p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo> QList<QFileInfo>::operator+(const QList<QFileInfo>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQFileInfo>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQFileInfo> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_operator_2(
                self as *const crate::QListOfQFileInfo,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QListOfQUrl>> for &crate::QListOfQUrl {
    type Output = ::cpp_utils::CppBox<crate::QListOfQUrl>;
    /// <p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QUrl> QList<QUrl>::operator+(const QList<QUrl>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQUrl>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQUrl> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_operator_2(
                self as *const crate::QListOfQUrl,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QListOfQMimeType>> for &crate::QListOfQMimeType {
    type Output = ::cpp_utils::CppBox<crate::QListOfQMimeType>;
    /// <p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType> QList<QMimeType>::operator+(const QList<QMimeType>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQMimeType>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQMimeType> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_operator_2(
                self as *const crate::QListOfQMimeType,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QListOfQAbstractTransition>>
    for &crate::QListOfQAbstractTransition
{
    type Output = ::cpp_utils::CppBox<crate::QListOfQAbstractTransition>;
    /// <p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*> QList<QAbstractTransition*>::operator+(const QList<QAbstractTransition*>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQAbstractTransition>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQAbstractTransition> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_operator_2(
                self as *const crate::QListOfQAbstractTransition,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QListOfQStorageInfo>> for &crate::QListOfQStorageInfo {
    type Output = ::cpp_utils::CppBox<crate::QListOfQStorageInfo>;
    /// <p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo> QList<QStorageInfo>::operator+(const QList<QStorageInfo>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQStorageInfo>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQStorageInfo> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_operator_2(
                self as *const crate::QListOfQStorageInfo,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QListOfInt>> for &crate::QListOfInt {
    type Output = ::cpp_utils::CppBox<crate::QListOfInt>;
    /// <p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<int> QList<int>::operator+(const QList<int>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(self, l: ::cpp_utils::Ref<crate::QListOfInt>) -> ::cpp_utils::CppBox<crate::QListOfInt> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_int_operator_2(
                self as *const crate::QListOfInt,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>>
    for &crate::QListOfQPairOfQStringQString
{
    type Output = ::cpp_utils::CppBox<crate::QListOfQPairOfQStringQString>;
    /// <p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>> QList<QPair<QString, QString>>::operator+(const QList<QPair<QString, QString>>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a list that contains all the items in this list followed by all the items in the <i>other</i> list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>,
    ) -> ::cpp_utils::CppBox<crate::QListOfQPairOfQStringQString> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_operator_2(
                self as *const crate::QListOfQPairOfQStringQString,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QString>> for crate::QListOfQString {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QString>& QList<QString>::operator+=(const QString& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<crate::QString>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QString_operator__5(
                self as *mut crate::QListOfQString,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QItemSelectionRange>>
    for crate::QListOfQItemSelectionRange
{
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>& QList<QItemSelectionRange>::operator+=(const QItemSelectionRange& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<crate::QItemSelectionRange>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_operator__5(
                self as *mut crate::QListOfQItemSelectionRange,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QVariant>> for crate::QListOfQVariant {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant>& QList<QVariant>::operator+=(const QVariant& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<crate::QVariant>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QVariant_operator__5(
                self as *mut crate::QListOfQVariant,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QByteArray>> for crate::QListOfQByteArray {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>& QList<QByteArray>::operator+=(const QByteArray& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<crate::QByteArray>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_operator__5(
                self as *mut crate::QListOfQByteArray,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo>>
    for crate::QListOfTimerInfo
{
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>& QList<QAbstractEventDispatcher::TimerInfo>::operator+=(const QAbstractEventDispatcher::TimerInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_operator__5(
                self as *mut crate::QListOfTimerInfo,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QModelIndex>> for crate::QListOfQModelIndex {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>& QList<QModelIndex>::operator+=(const QModelIndex& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<crate::QModelIndex>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_operator__5(
                self as *mut crate::QListOfQModelIndex,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QPersistentModelIndex>>
    for crate::QListOfQPersistentModelIndex
{
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>& QList<QPersistentModelIndex>::operator+=(const QPersistentModelIndex& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<crate::QPersistentModelIndex>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_operator__5(
                self as *mut crate::QListOfQPersistentModelIndex,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::DayOfWeek>> for crate::QListOfDayOfWeek {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>& QList<Qt::DayOfWeek>::operator+=(const Qt::DayOfWeek& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<crate::DayOfWeek>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_operator__5(
                self as *mut crate::QListOfDayOfWeek,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QLocale>> for crate::QListOfQLocale {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale>& QList<QLocale>::operator+=(const QLocale& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<crate::QLocale>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_operator__5(
                self as *mut crate::QListOfQLocale,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::q_locale::Country>> for crate::QListOfCountry {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>& QList<QLocale::Country>::operator+=(const QLocale::Country& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<crate::q_locale::Country>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_operator__5(
                self as *mut crate::QListOfCountry,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QCommandLineOption>>
    for crate::QListOfQCommandLineOption
{
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>& QList<QCommandLineOption>::operator+=(const QCommandLineOption& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<crate::QCommandLineOption>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_operator__5(
                self as *mut crate::QListOfQCommandLineOption,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QFileInfo>> for crate::QListOfQFileInfo {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>& QList<QFileInfo>::operator+=(const QFileInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<crate::QFileInfo>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_operator__5(
                self as *mut crate::QListOfQFileInfo,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QUrl>> for crate::QListOfQUrl {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QUrl>& QList<QUrl>::operator+=(const QUrl& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<crate::QUrl>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_operator__5(
                self as *mut crate::QListOfQUrl,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QMimeType>> for crate::QListOfQMimeType {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>& QList<QMimeType>::operator+=(const QMimeType& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<crate::QMimeType>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_operator__5(
                self as *mut crate::QListOfQMimeType,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QStorageInfo>> for crate::QListOfQStorageInfo {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>& QList<QStorageInfo>::operator+=(const QStorageInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<crate::QStorageInfo>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_operator__5(
                self as *mut crate::QListOfQStorageInfo,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<::std::os::raw::c_int>> for crate::QListOfInt {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<int>& QList<int>::operator+=(const int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<::std::os::raw::c_int>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_int_operator__5(
                self as *mut crate::QListOfInt,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QPairOfQStringQString>>
    for crate::QListOfQPairOfQStringQString
{
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>& QList<QPair<QString, QString>>::operator+=(const QPair<QString, QString>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<crate::QPairOfQStringQString>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_operator__5(
                self as *mut crate::QListOfQPairOfQStringQString,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QString>> for &crate::QListOfQString {
    type Output = ::cpp_utils::MutRef<crate::QListOfQString>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QString>& QList<QString>::operator<<(const QString& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<crate::QString>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQString> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QString_operator__6(
                self as *const crate::QListOfQString as *mut crate::QListOfQString,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QItemSelectionRange>>
    for &crate::QListOfQItemSelectionRange
{
    type Output = ::cpp_utils::MutRef<crate::QListOfQItemSelectionRange>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>& QList<QItemSelectionRange>::operator<<(const QItemSelectionRange& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<crate::QItemSelectionRange>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQItemSelectionRange> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_operator__6(
                self as *const crate::QListOfQItemSelectionRange
                    as *mut crate::QListOfQItemSelectionRange,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVariant>> for &crate::QListOfQVariant {
    type Output = ::cpp_utils::MutRef<crate::QListOfQVariant>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant>& QList<QVariant>::operator<<(const QVariant& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<crate::QVariant>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQVariant> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QVariant_operator__6(
                self as *const crate::QListOfQVariant as *mut crate::QListOfQVariant,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QByteArray>> for &crate::QListOfQByteArray {
    type Output = ::cpp_utils::MutRef<crate::QListOfQByteArray>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>& QList<QByteArray>::operator<<(const QByteArray& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<crate::QByteArray>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQByteArray> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_operator__6(
                self as *const crate::QListOfQByteArray as *mut crate::QListOfQByteArray,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo>>
    for &crate::QListOfTimerInfo
{
    type Output = ::cpp_utils::MutRef<crate::QListOfTimerInfo>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>& QList<QAbstractEventDispatcher::TimerInfo>::operator<<(const QAbstractEventDispatcher::TimerInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<crate::q_abstract_event_dispatcher::TimerInfo>,
    ) -> ::cpp_utils::MutRef<crate::QListOfTimerInfo> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_operator__6(
                self as *const crate::QListOfTimerInfo as *mut crate::QListOfTimerInfo,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QModelIndex>> for &crate::QListOfQModelIndex {
    type Output = ::cpp_utils::MutRef<crate::QListOfQModelIndex>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>& QList<QModelIndex>::operator<<(const QModelIndex& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<crate::QModelIndex>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQModelIndex> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_operator__6(
                self as *const crate::QListOfQModelIndex as *mut crate::QListOfQModelIndex,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QPersistentModelIndex>>
    for &crate::QListOfQPersistentModelIndex
{
    type Output = ::cpp_utils::MutRef<crate::QListOfQPersistentModelIndex>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>& QList<QPersistentModelIndex>::operator<<(const QPersistentModelIndex& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<crate::QPersistentModelIndex>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQPersistentModelIndex> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_operator__6(
                self as *const crate::QListOfQPersistentModelIndex
                    as *mut crate::QListOfQPersistentModelIndex,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::DayOfWeek>> for &crate::QListOfDayOfWeek {
    type Output = ::cpp_utils::MutRef<crate::QListOfDayOfWeek>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>& QList<Qt::DayOfWeek>::operator<<(const Qt::DayOfWeek& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<crate::DayOfWeek>,
    ) -> ::cpp_utils::MutRef<crate::QListOfDayOfWeek> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_operator__6(
                self as *const crate::QListOfDayOfWeek as *mut crate::QListOfDayOfWeek,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QLocale>> for &crate::QListOfQLocale {
    type Output = ::cpp_utils::MutRef<crate::QListOfQLocale>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale>& QList<QLocale>::operator<<(const QLocale& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<crate::QLocale>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQLocale> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_operator__6(
                self as *const crate::QListOfQLocale as *mut crate::QListOfQLocale,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::q_locale::Country>> for &crate::QListOfCountry {
    type Output = ::cpp_utils::MutRef<crate::QListOfCountry>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>& QList<QLocale::Country>::operator<<(const QLocale::Country& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<crate::q_locale::Country>,
    ) -> ::cpp_utils::MutRef<crate::QListOfCountry> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_operator__6(
                self as *const crate::QListOfCountry as *mut crate::QListOfCountry,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QCommandLineOption>>
    for &crate::QListOfQCommandLineOption
{
    type Output = ::cpp_utils::MutRef<crate::QListOfQCommandLineOption>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>& QList<QCommandLineOption>::operator<<(const QCommandLineOption& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<crate::QCommandLineOption>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQCommandLineOption> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_operator__6(
                self as *const crate::QListOfQCommandLineOption
                    as *mut crate::QListOfQCommandLineOption,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QFileInfo>> for &crate::QListOfQFileInfo {
    type Output = ::cpp_utils::MutRef<crate::QListOfQFileInfo>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>& QList<QFileInfo>::operator<<(const QFileInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<crate::QFileInfo>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQFileInfo> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_operator__6(
                self as *const crate::QListOfQFileInfo as *mut crate::QListOfQFileInfo,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QUrl>> for &crate::QListOfQUrl {
    type Output = ::cpp_utils::MutRef<crate::QListOfQUrl>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QUrl>& QList<QUrl>::operator<<(const QUrl& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p></div>
    fn shl(self, t: ::cpp_utils::Ref<crate::QUrl>) -> ::cpp_utils::MutRef<crate::QListOfQUrl> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_operator__6(
                self as *const crate::QListOfQUrl as *mut crate::QListOfQUrl,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QMimeType>> for &crate::QListOfQMimeType {
    type Output = ::cpp_utils::MutRef<crate::QListOfQMimeType>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>& QList<QMimeType>::operator<<(const QMimeType& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<crate::QMimeType>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQMimeType> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_operator__6(
                self as *const crate::QListOfQMimeType as *mut crate::QListOfQMimeType,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QStorageInfo>> for &crate::QListOfQStorageInfo {
    type Output = ::cpp_utils::MutRef<crate::QListOfQStorageInfo>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>& QList<QStorageInfo>::operator<<(const QStorageInfo& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<crate::QStorageInfo>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQStorageInfo> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_operator__6(
                self as *const crate::QListOfQStorageInfo as *mut crate::QListOfQStorageInfo,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<::std::os::raw::c_int>> for &crate::QListOfInt {
    type Output = ::cpp_utils::MutRef<crate::QListOfInt>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<int>& QList<int>::operator<<(const int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<::std::os::raw::c_int>,
    ) -> ::cpp_utils::MutRef<crate::QListOfInt> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_int_operator__6(
                self as *const crate::QListOfInt as *mut crate::QListOfInt,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QPairOfQStringQString>>
    for &crate::QListOfQPairOfQStringQString
{
    type Output = ::cpp_utils::MutRef<crate::QListOfQPairOfQStringQString>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>& QList<QPair<QString, QString>>::operator<<(const QPair<QString, QString>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the list.</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<crate::QPairOfQStringQString>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQPairOfQStringQString> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_operator__6(
                self as *const crate::QListOfQPairOfQStringQString
                    as *mut crate::QListOfQPairOfQStringQString,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQString>> for &crate::QListOfQString {
    type Output = ::cpp_utils::MutRef<crate::QListOfQString>;
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QString>& QList<QString>::operator<<(const QList<QString>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQString>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQString> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QString_operator__7(
                self as *const crate::QListOfQString as *mut crate::QListOfQString,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQItemSelectionRange>>
    for &crate::QListOfQItemSelectionRange
{
    type Output = ::cpp_utils::MutRef<crate::QListOfQItemSelectionRange>;
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>& QList<QItemSelectionRange>::operator<<(const QList<QItemSelectionRange>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQItemSelectionRange>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQItemSelectionRange> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QItemSelectionRange_operator__7(
                self as *const crate::QListOfQItemSelectionRange
                    as *mut crate::QListOfQItemSelectionRange,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQObject>> for &crate::QListOfQObject {
    type Output = ::cpp_utils::MutRef<crate::QListOfQObject>;
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QObject*>& QList<QObject*>::operator<<(const QList<QObject*>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQObject>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQObject> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QObject_ptr_operator__7(
                self as *const crate::QListOfQObject as *mut crate::QListOfQObject,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQVariant>> for &crate::QListOfQVariant {
    type Output = ::cpp_utils::MutRef<crate::QListOfQVariant>;
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QVariant>& QList<QVariant>::operator<<(const QList<QVariant>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQVariant>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQVariant> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QVariant_operator__7(
                self as *const crate::QListOfQVariant as *mut crate::QListOfQVariant,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQByteArray>> for &crate::QListOfQByteArray {
    type Output = ::cpp_utils::MutRef<crate::QListOfQByteArray>;
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QByteArray>& QList<QByteArray>::operator<<(const QList<QByteArray>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQByteArray>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQByteArray> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QByteArray_operator__7(
                self as *const crate::QListOfQByteArray as *mut crate::QListOfQByteArray,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfTimerInfo>> for &crate::QListOfTimerInfo {
    type Output = ::cpp_utils::MutRef<crate::QListOfTimerInfo>;
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractEventDispatcher::TimerInfo>& QList<QAbstractEventDispatcher::TimerInfo>::operator<<(const QList<QAbstractEventDispatcher::TimerInfo>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfTimerInfo>,
    ) -> ::cpp_utils::MutRef<crate::QListOfTimerInfo> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractEventDispatcher_TimerInfo_operator__7(
                self as *const crate::QListOfTimerInfo as *mut crate::QListOfTimerInfo,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQModelIndex>> for &crate::QListOfQModelIndex {
    type Output = ::cpp_utils::MutRef<crate::QListOfQModelIndex>;
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QModelIndex>& QList<QModelIndex>::operator<<(const QList<QModelIndex>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQModelIndex>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQModelIndex> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QModelIndex_operator__7(
                self as *const crate::QListOfQModelIndex as *mut crate::QListOfQModelIndex,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>>
    for &crate::QListOfQPersistentModelIndex
{
    type Output = ::cpp_utils::MutRef<crate::QListOfQPersistentModelIndex>;
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPersistentModelIndex>& QList<QPersistentModelIndex>::operator<<(const QList<QPersistentModelIndex>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQPersistentModelIndex> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QPersistentModelIndex_operator__7(
                self as *const crate::QListOfQPersistentModelIndex
                    as *mut crate::QListOfQPersistentModelIndex,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQAbstractState>>
    for &crate::QListOfQAbstractState
{
    type Output = ::cpp_utils::MutRef<crate::QListOfQAbstractState>;
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractState*>& QList<QAbstractState*>::operator<<(const QList<QAbstractState*>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQAbstractState>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQAbstractState> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractState_ptr_operator__7(
                self as *const crate::QListOfQAbstractState as *mut crate::QListOfQAbstractState,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQAbstractAnimation>>
    for &crate::QListOfQAbstractAnimation
{
    type Output = ::cpp_utils::MutRef<crate::QListOfQAbstractAnimation>;
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractAnimation*>& QList<QAbstractAnimation*>::operator<<(const QList<QAbstractAnimation*>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQAbstractAnimation>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQAbstractAnimation> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractAnimation_ptr_operator__7(
                self as *const crate::QListOfQAbstractAnimation
                    as *mut crate::QListOfQAbstractAnimation,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfDayOfWeek>> for &crate::QListOfDayOfWeek {
    type Output = ::cpp_utils::MutRef<crate::QListOfDayOfWeek>;
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<Qt::DayOfWeek>& QList<Qt::DayOfWeek>::operator<<(const QList<Qt::DayOfWeek>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfDayOfWeek>,
    ) -> ::cpp_utils::MutRef<crate::QListOfDayOfWeek> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_Qt_DayOfWeek_operator__7(
                self as *const crate::QListOfDayOfWeek as *mut crate::QListOfDayOfWeek,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQLocale>> for &crate::QListOfQLocale {
    type Output = ::cpp_utils::MutRef<crate::QListOfQLocale>;
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale>& QList<QLocale>::operator<<(const QList<QLocale>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQLocale>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQLocale> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_operator__7(
                self as *const crate::QListOfQLocale as *mut crate::QListOfQLocale,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfCountry>> for &crate::QListOfCountry {
    type Output = ::cpp_utils::MutRef<crate::QListOfCountry>;
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QLocale::Country>& QList<QLocale::Country>::operator<<(const QList<QLocale::Country>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfCountry>,
    ) -> ::cpp_utils::MutRef<crate::QListOfCountry> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QLocale_Country_operator__7(
                self as *const crate::QListOfCountry as *mut crate::QListOfCountry,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQCommandLineOption>>
    for &crate::QListOfQCommandLineOption
{
    type Output = ::cpp_utils::MutRef<crate::QListOfQCommandLineOption>;
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QCommandLineOption>& QList<QCommandLineOption>::operator<<(const QList<QCommandLineOption>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQCommandLineOption>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQCommandLineOption> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QCommandLineOption_operator__7(
                self as *const crate::QListOfQCommandLineOption
                    as *mut crate::QListOfQCommandLineOption,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQFileInfo>> for &crate::QListOfQFileInfo {
    type Output = ::cpp_utils::MutRef<crate::QListOfQFileInfo>;
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QFileInfo>& QList<QFileInfo>::operator<<(const QList<QFileInfo>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQFileInfo>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQFileInfo> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QFileInfo_operator__7(
                self as *const crate::QListOfQFileInfo as *mut crate::QListOfQFileInfo,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQUrl>> for &crate::QListOfQUrl {
    type Output = ::cpp_utils::MutRef<crate::QListOfQUrl>;
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QUrl>& QList<QUrl>::operator<<(const QList<QUrl>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQUrl>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQUrl> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QUrl_operator__7(
                self as *const crate::QListOfQUrl as *mut crate::QListOfQUrl,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQMimeType>> for &crate::QListOfQMimeType {
    type Output = ::cpp_utils::MutRef<crate::QListOfQMimeType>;
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QMimeType>& QList<QMimeType>::operator<<(const QList<QMimeType>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQMimeType>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQMimeType> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QMimeType_operator__7(
                self as *const crate::QListOfQMimeType as *mut crate::QListOfQMimeType,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQAbstractTransition>>
    for &crate::QListOfQAbstractTransition
{
    type Output = ::cpp_utils::MutRef<crate::QListOfQAbstractTransition>;
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QAbstractTransition*>& QList<QAbstractTransition*>::operator<<(const QList<QAbstractTransition*>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQAbstractTransition>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQAbstractTransition> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QAbstractTransition_ptr_operator__7(
                self as *const crate::QListOfQAbstractTransition
                    as *mut crate::QListOfQAbstractTransition,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQStorageInfo>> for &crate::QListOfQStorageInfo {
    type Output = ::cpp_utils::MutRef<crate::QListOfQStorageInfo>;
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QStorageInfo>& QList<QStorageInfo>::operator<<(const QList<QStorageInfo>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQStorageInfo>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQStorageInfo> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QStorageInfo_operator__7(
                self as *const crate::QListOfQStorageInfo as *mut crate::QListOfQStorageInfo,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfInt>> for &crate::QListOfInt {
    type Output = ::cpp_utils::MutRef<crate::QListOfInt>;
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<int>& QList<int>::operator<<(const QList<int>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn shl(self, l: ::cpp_utils::Ref<crate::QListOfInt>) -> ::cpp_utils::MutRef<crate::QListOfInt> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_int_operator__7(
                self as *const crate::QListOfInt as *mut crate::QListOfInt,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>>
    for &crate::QListOfQPairOfQStringQString
{
    type Output = ::cpp_utils::MutRef<crate::QListOfQPairOfQStringQString>;
    /// <p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QList<QPair<QString, QString>>& QList<QPair<QString, QString>>::operator<<(const QList<QPair<QString, QString>>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlist.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> list to this list and returns a reference to this list.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlist.html#operator-2b-eq">operator+=</a>() and <a href="http://doc.qt.io/qt-5/qlist.html#append">append</a>().</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>,
    ) -> ::cpp_utils::MutRef<crate::QListOfQPairOfQStringQString> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QList_QPair_QString_QString_operator__7(
                self as *const crate::QListOfQPairOfQStringQString
                    as *mut crate::QListOfQPairOfQStringQString,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QListOfQString>> for crate::QListOfQString {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QList<QString>& lhs, const QList<QString>& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QListOfQString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_142(
                self as *const crate::QListOfQString,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QListOfQItemSelectionRange>>
    for crate::QListOfQItemSelectionRange
{
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QList<QItemSelectionRange>& lhs, const QList<QItemSelectionRange>& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QListOfQItemSelectionRange>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_143(
                self as *const crate::QListOfQItemSelectionRange,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QListOfQObject>> for crate::QListOfQObject {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QList<QObject*>& lhs, const QList<QObject*>& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QListOfQObject>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_144(
                self as *const crate::QListOfQObject,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QListOfQVariant>> for crate::QListOfQVariant {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QList<QVariant>& lhs, const QList<QVariant>& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QListOfQVariant>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_145(
                self as *const crate::QListOfQVariant,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QListOfQByteArray>> for crate::QListOfQByteArray {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QList<QByteArray>& lhs, const QList<QByteArray>& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QListOfQByteArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_146(
                self as *const crate::QListOfQByteArray,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QListOfQModelIndex>>
    for crate::QListOfQModelIndex
{
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QList<QModelIndex>& lhs, const QList<QModelIndex>& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QListOfQModelIndex>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_148(
                self as *const crate::QListOfQModelIndex,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>>
    for crate::QListOfQPersistentModelIndex
{
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QList<QPersistentModelIndex>& lhs, const QList<QPersistentModelIndex>& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_149(
                self as *const crate::QListOfQPersistentModelIndex,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QListOfQAbstractState>>
    for crate::QListOfQAbstractState
{
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QList<QAbstractState*>& lhs, const QList<QAbstractState*>& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QListOfQAbstractState>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_150(
                self as *const crate::QListOfQAbstractState,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QListOfQAbstractAnimation>>
    for crate::QListOfQAbstractAnimation
{
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QList<QAbstractAnimation*>& lhs, const QList<QAbstractAnimation*>& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QListOfQAbstractAnimation>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_151(
                self as *const crate::QListOfQAbstractAnimation,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QListOfDayOfWeek>> for crate::QListOfDayOfWeek {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QList<Qt::DayOfWeek>& lhs, const QList<Qt::DayOfWeek>& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QListOfDayOfWeek>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_152(
                self as *const crate::QListOfDayOfWeek,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QListOfCountry>> for crate::QListOfCountry {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QList<QLocale::Country>& lhs, const QList<QLocale::Country>& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QListOfCountry>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_154(
                self as *const crate::QListOfCountry,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QListOfQUrl>> for crate::QListOfQUrl {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QList<QUrl>& lhs, const QList<QUrl>& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QListOfQUrl>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_157(
                self as *const crate::QListOfQUrl,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QListOfQAbstractTransition>>
    for crate::QListOfQAbstractTransition
{
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QList<QAbstractTransition*>& lhs, const QList<QAbstractTransition*>& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QListOfQAbstractTransition>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_159(
                self as *const crate::QListOfQAbstractTransition,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QListOfInt>> for crate::QListOfInt {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QList<int>& lhs, const QList<int>& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QListOfInt>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_161(
                self as *const crate::QListOfInt,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>>
    for crate::QListOfQPairOfQStringQString
{
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QList<QPair<QString, QString>>& lhs, const QList<QPair<QString, QString>>& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_162(
                self as *const crate::QListOfQPairOfQStringQString,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QListOfQString>> for crate::QListOfQString {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QList<QString>& lhs, const QList<QString>& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QListOfQString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_163(
                self as *const crate::QListOfQString,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QListOfQItemSelectionRange>>
    for crate::QListOfQItemSelectionRange
{
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QList<QItemSelectionRange>& lhs, const QList<QItemSelectionRange>& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QListOfQItemSelectionRange>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_164(
                self as *const crate::QListOfQItemSelectionRange,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QListOfQObject>> for crate::QListOfQObject {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QList<QObject*>& lhs, const QList<QObject*>& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QListOfQObject>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_165(
                self as *const crate::QListOfQObject,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QListOfQVariant>> for crate::QListOfQVariant {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QList<QVariant>& lhs, const QList<QVariant>& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QListOfQVariant>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_166(
                self as *const crate::QListOfQVariant,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QListOfQByteArray>> for crate::QListOfQByteArray {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QList<QByteArray>& lhs, const QList<QByteArray>& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QListOfQByteArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_167(
                self as *const crate::QListOfQByteArray,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QListOfQModelIndex>>
    for crate::QListOfQModelIndex
{
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QList<QModelIndex>& lhs, const QList<QModelIndex>& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QListOfQModelIndex>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_169(
                self as *const crate::QListOfQModelIndex,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>>
    for crate::QListOfQPersistentModelIndex
{
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QList<QPersistentModelIndex>& lhs, const QList<QPersistentModelIndex>& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_170(
                self as *const crate::QListOfQPersistentModelIndex,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QListOfQAbstractState>>
    for crate::QListOfQAbstractState
{
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QList<QAbstractState*>& lhs, const QList<QAbstractState*>& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QListOfQAbstractState>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_171(
                self as *const crate::QListOfQAbstractState,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QListOfQAbstractAnimation>>
    for crate::QListOfQAbstractAnimation
{
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QList<QAbstractAnimation*>& lhs, const QList<QAbstractAnimation*>& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QListOfQAbstractAnimation>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_172(
                self as *const crate::QListOfQAbstractAnimation,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QListOfDayOfWeek>> for crate::QListOfDayOfWeek {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QList<Qt::DayOfWeek>& lhs, const QList<Qt::DayOfWeek>& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QListOfDayOfWeek>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_173(
                self as *const crate::QListOfDayOfWeek,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QListOfCountry>> for crate::QListOfCountry {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QList<QLocale::Country>& lhs, const QList<QLocale::Country>& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QListOfCountry>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_175(
                self as *const crate::QListOfCountry,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QListOfQUrl>> for crate::QListOfQUrl {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QList<QUrl>& lhs, const QList<QUrl>& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QListOfQUrl>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_178(
                self as *const crate::QListOfQUrl,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QListOfQAbstractTransition>>
    for crate::QListOfQAbstractTransition
{
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QList<QAbstractTransition*>& lhs, const QList<QAbstractTransition*>& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QListOfQAbstractTransition>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_180(
                self as *const crate::QListOfQAbstractTransition,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QListOfInt>> for crate::QListOfInt {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QList<int>& lhs, const QList<int>& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QListOfInt>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_182(
                self as *const crate::QListOfInt,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>>
    for crate::QListOfQPairOfQStringQString
{
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QList<QPair<QString, QString>>& lhs, const QList<QPair<QString, QString>>& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_183(
                self as *const crate::QListOfQPairOfQStringQString,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QListOfQString>> for crate::QListOfQString {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QList<QString>& lhs, const QList<QString>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QListOfQString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__215(
                self as *const crate::QListOfQString,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QListOfQItemSelectionRange>>
    for crate::QListOfQItemSelectionRange
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QList<QItemSelectionRange>& lhs, const QList<QItemSelectionRange>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QListOfQItemSelectionRange>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__216(
                self as *const crate::QListOfQItemSelectionRange,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QListOfQObject>> for crate::QListOfQObject {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QList<QObject*>& lhs, const QList<QObject*>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QListOfQObject>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__217(
                self as *const crate::QListOfQObject,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QListOfQVariant>> for crate::QListOfQVariant {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QList<QVariant>& lhs, const QList<QVariant>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QListOfQVariant>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__218(
                self as *const crate::QListOfQVariant,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QListOfQByteArray>> for crate::QListOfQByteArray {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QList<QByteArray>& lhs, const QList<QByteArray>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QListOfQByteArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__219(
                self as *const crate::QListOfQByteArray,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QListOfQModelIndex>>
    for crate::QListOfQModelIndex
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QList<QModelIndex>& lhs, const QList<QModelIndex>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QListOfQModelIndex>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__221(
                self as *const crate::QListOfQModelIndex,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>>
    for crate::QListOfQPersistentModelIndex
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QList<QPersistentModelIndex>& lhs, const QList<QPersistentModelIndex>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__222(
                self as *const crate::QListOfQPersistentModelIndex,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QListOfQAbstractState>>
    for crate::QListOfQAbstractState
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QList<QAbstractState*>& lhs, const QList<QAbstractState*>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QListOfQAbstractState>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__223(
                self as *const crate::QListOfQAbstractState,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QListOfQAbstractAnimation>>
    for crate::QListOfQAbstractAnimation
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QList<QAbstractAnimation*>& lhs, const QList<QAbstractAnimation*>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QListOfQAbstractAnimation>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__224(
                self as *const crate::QListOfQAbstractAnimation,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QListOfDayOfWeek>> for crate::QListOfDayOfWeek {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QList<Qt::DayOfWeek>& lhs, const QList<Qt::DayOfWeek>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QListOfDayOfWeek>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__225(
                self as *const crate::QListOfDayOfWeek,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QListOfCountry>> for crate::QListOfCountry {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QList<QLocale::Country>& lhs, const QList<QLocale::Country>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QListOfCountry>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__227(
                self as *const crate::QListOfCountry,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QListOfQUrl>> for crate::QListOfQUrl {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QList<QUrl>& lhs, const QList<QUrl>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QListOfQUrl>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__230(
                self as *const crate::QListOfQUrl,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QListOfQAbstractTransition>>
    for crate::QListOfQAbstractTransition
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QList<QAbstractTransition*>& lhs, const QList<QAbstractTransition*>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QListOfQAbstractTransition>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__232(
                self as *const crate::QListOfQAbstractTransition,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QListOfInt>> for crate::QListOfInt {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QList<int>& lhs, const QList<int>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QListOfInt>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__234(
                self as *const crate::QListOfInt,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>>
    for crate::QListOfQPairOfQStringQString
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QList<QPair<QString, QString>>& lhs, const QList<QPair<QString, QString>>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__235(
                self as *const crate::QListOfQPairOfQStringQString,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QListOfQString>> for crate::QListOfQString {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QList<QString>& lhs, const QList<QString>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QListOfQString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__236(
                self as *const crate::QListOfQString,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QListOfQItemSelectionRange>>
    for crate::QListOfQItemSelectionRange
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QList<QItemSelectionRange>& lhs, const QList<QItemSelectionRange>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QListOfQItemSelectionRange>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__237(
                self as *const crate::QListOfQItemSelectionRange,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QListOfQObject>> for crate::QListOfQObject {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QList<QObject*>& lhs, const QList<QObject*>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QListOfQObject>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__238(
                self as *const crate::QListOfQObject,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QListOfQVariant>> for crate::QListOfQVariant {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QList<QVariant>& lhs, const QList<QVariant>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QListOfQVariant>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__239(
                self as *const crate::QListOfQVariant,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QListOfQByteArray>> for crate::QListOfQByteArray {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QList<QByteArray>& lhs, const QList<QByteArray>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QListOfQByteArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__240(
                self as *const crate::QListOfQByteArray,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QListOfQModelIndex>>
    for crate::QListOfQModelIndex
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QList<QModelIndex>& lhs, const QList<QModelIndex>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QListOfQModelIndex>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__242(
                self as *const crate::QListOfQModelIndex,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>>
    for crate::QListOfQPersistentModelIndex
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QList<QPersistentModelIndex>& lhs, const QList<QPersistentModelIndex>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__243(
                self as *const crate::QListOfQPersistentModelIndex,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QListOfQAbstractState>>
    for crate::QListOfQAbstractState
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QList<QAbstractState*>& lhs, const QList<QAbstractState*>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QListOfQAbstractState>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__244(
                self as *const crate::QListOfQAbstractState,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QListOfQAbstractAnimation>>
    for crate::QListOfQAbstractAnimation
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QList<QAbstractAnimation*>& lhs, const QList<QAbstractAnimation*>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QListOfQAbstractAnimation>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__245(
                self as *const crate::QListOfQAbstractAnimation,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QListOfDayOfWeek>> for crate::QListOfDayOfWeek {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QList<Qt::DayOfWeek>& lhs, const QList<Qt::DayOfWeek>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QListOfDayOfWeek>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__246(
                self as *const crate::QListOfDayOfWeek,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QListOfCountry>> for crate::QListOfCountry {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QList<QLocale::Country>& lhs, const QList<QLocale::Country>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QListOfCountry>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__248(
                self as *const crate::QListOfCountry,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QListOfQUrl>> for crate::QListOfQUrl {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QList<QUrl>& lhs, const QList<QUrl>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QListOfQUrl>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__251(
                self as *const crate::QListOfQUrl,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QListOfQAbstractTransition>>
    for crate::QListOfQAbstractTransition
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QList<QAbstractTransition*>& lhs, const QList<QAbstractTransition*>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QListOfQAbstractTransition>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__253(
                self as *const crate::QListOfQAbstractTransition,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QListOfInt>> for crate::QListOfInt {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QList<int>& lhs, const QList<int>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QListOfInt>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__255(
                self as *const crate::QListOfInt,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>>
    for crate::QListOfQPairOfQStringQString
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QList<QPair<QString, QString>>& lhs, const QList<QPair<QString, QString>>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__256(
                self as *const crate::QListOfQPairOfQStringQString,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QMapOfQStringQVariant {
    /// <p>Destroys the map. References to the values in the map, and all iterators over this map, become invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QMap<QString, QVariant>::~QMap()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#dtor.QMap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the map. References to the values in the map, and all iterators over this map, become invalid.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_dQMap(
            self as *mut crate::QMapOfQStringQVariant,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QMapOfIntQVariant {
    /// <p>Destroys the map. References to the values in the map, and all iterators over this map, become invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QMap<int, QVariant>::~QMap()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#dtor.QMap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the map. References to the values in the map, and all iterators over this map, become invalid.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_dQMap(self as *mut crate::QMapOfIntQVariant)
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QMapOfQStringQVariant>>
    for crate::QMapOfQStringQVariant
{
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this map; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMap<QString, QVariant>::operator==(const QMap<QString, QVariant>& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this map; otherwise returns false.</p>
    /// <p>Two maps are considered equal if they contain the same (key, value) pairs.</p>
    /// <p>This function requires the value type to implement <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QMapOfQStringQVariant>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_operator__(
                self as *const crate::QMapOfQStringQVariant,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QMapOfIntQVariant>>
    for crate::QMapOfIntQVariant
{
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this map; otherwise returns false.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QMap<int, QVariant>::operator==(const QMap<int, QVariant>& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this map; otherwise returns false.</p>
    /// <p>Two maps are considered equal if they contain the same (key, value) pairs.</p>
    /// <p>This function requires the value type to implement <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QMapOfIntQVariant>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_operator__(
                self as *const crate::QMapOfIntQVariant,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QMapOfQStringQVariant {
    type Output = ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::iterator QMap<QString, QVariant>::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_begin(
                self as *mut crate::QMapOfQStringQVariant,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QMapOfIntQVariant {
    type Output = ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::iterator QMap<int, QVariant>::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_begin(
                self as *mut crate::QMapOfIntQVariant,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QMapOfQStringQVariant {
    type Output = ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::const_iterator QMap<QString, QVariant>::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_begin1(
                self as *const crate::QMapOfQStringQVariant,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QMapOfIntQVariant {
    type Output = ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::const_iterator QMap<int, QVariant>::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_begin1(
                self as *const crate::QMapOfIntQVariant,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QMapOfQStringQVariant {
    type Output = ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::iterator QMap<QString, QVariant>::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_end(
                self as *mut crate::QMapOfQStringQVariant,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QMapOfIntQVariant {
    type Output = ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::iterator QMap<int, QVariant>::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qmap.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qmap.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_end(
                self as *mut crate::QMapOfIntQVariant,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QMapOfQStringQVariant {
    type Output = ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<QString, QVariant>::const_iterator QMap<QString, QVariant>::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_map_of_q_string_q_variant::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QMap_QString_QVariant_end1(
                self as *const crate::QMapOfQStringQVariant,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QMapOfIntQVariant {
    type Output = ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QMap<int, QVariant>::const_iterator QMap<int, QVariant>::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qmap.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_map_of_int_q_variant::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QMap_int_QVariant_end1(
                self as *const crate::QMapOfIntQVariant,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QHashOfQStringQVariant {
    /// <p>Destroys the hash. References to the values in the hash and all iterators of this hash become invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QHash<QString, QVariant>::~QHash()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#dtor.QHash">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the hash. References to the values in the hash and all iterators of this hash become invalid.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_dQHash(
            self as *mut crate::QHashOfQStringQVariant,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QHashOfIntQByteArray {
    /// <p>Destroys the hash. References to the values in the hash and all iterators of this hash become invalid.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QHash<int, QByteArray>::~QHash()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#dtor.QHash">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the hash. References to the values in the hash and all iterators of this hash become invalid.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_dQHash(
            self as *mut crate::QHashOfIntQByteArray,
        )
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QHashOfQStringQVariant {
    type Output = ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::iterator QHash<QString, QVariant>::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the hash.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_begin(
                self as *mut crate::QHashOfQStringQVariant,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QHashOfIntQByteArray {
    type Output = ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::iterator QHash<int, QByteArray>::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first item in the hash.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_begin(
                self as *mut crate::QHashOfIntQByteArray,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QHashOfQStringQVariant {
    type Output = ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::const_iterator QHash<QString, QVariant>::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_begin1(
                self as *const crate::QHashOfQStringQVariant,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QHashOfIntQByteArray {
    type Output = ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::const_iterator QHash<int, QByteArray>::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_begin1(
                self as *const crate::QHashOfIntQByteArray,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QHashOfQStringQVariant {
    type Output = ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::iterator QHash<QString, QVariant>::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the hash.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_end(
                self as *mut crate::QHashOfQStringQVariant,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QHashOfIntQByteArray {
    type Output = ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::Iterator>;
    /// <p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the hash.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::iterator QHash<int, QByteArray>::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the hash.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_end(
                self as *mut crate::QHashOfIntQByteArray,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QHashOfQStringQVariant {
    type Output = ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<QString, QVariant>::const_iterator QHash<QString, QVariant>::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_hash_of_q_string_q_variant::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QHash_QString_QVariant_end1(
                self as *const crate::QHashOfQStringQVariant,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QHashOfIntQByteArray {
    type Output = ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::ConstIterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QHash<int, QByteArray>::const_iterator QHash<int, QByteArray>::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qhash.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_hash_of_int_q_byte_array::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QHash_int_QByteArray_end1(
                self as *const crate::QHashOfIntQByteArray,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QVectorOfQXmlStreamAttribute {
    /// <p>Destroys the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QVector<QXmlStreamAttribute>::~QVector()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#dtor.QVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the vector.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_dQVector(
            self as *mut crate::QVectorOfQXmlStreamAttribute,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QVectorOfQStringRef {
    /// <p>Destroys the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QVector<QStringRef>::~QVector()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#dtor.QVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the vector.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_dQVector(
            self as *mut crate::QVectorOfQStringRef,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QVectorOfUint {
    /// <p>Destroys the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QVector<unsigned int>::~QVector()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#dtor.QVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the vector.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_dQVector(
            self as *mut crate::QVectorOfUint,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QVectorOfInt {
    /// <p>Destroys the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QVector<int>::~QVector()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#dtor.QVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the vector.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_int_dQVector(self as *mut crate::QVectorOfInt)
    }
}

impl ::cpp_utils::CppDeletable for crate::QVectorOfQPointF {
    /// <p>Destroys the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QVector<QPointF>::~QVector()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#dtor.QVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the vector.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_dQVector(self as *mut crate::QVectorOfQPointF)
    }
}

impl ::cpp_utils::CppDeletable for crate::QVectorOfQStaticPlugin {
    /// <p>Destroys the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QVector<QStaticPlugin>::~QVector()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#dtor.QVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the vector.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_dQVector(
            self as *mut crate::QVectorOfQStaticPlugin,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QVectorOfQPairOfDoubleQVariant {
    /// <p>Destroys the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QVector<QPair<double, QVariant>>::~QVector()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#dtor.QVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the vector.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_dQVector(
            self as *mut crate::QVectorOfQPairOfDoubleQVariant,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QVectorOfOffsetData {
    /// <p>Destroys the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QVector<QTimeZone::OffsetData>::~QVector()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#dtor.QVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the vector.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_dQVector(
            self as *mut crate::QVectorOfOffsetData,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QVectorOfQXmlStreamNamespaceDeclaration {
    /// <p>Destroys the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QVector<QXmlStreamNamespaceDeclaration>::~QVector()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#dtor.QVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the vector.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_dQVector(
            self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QVectorOfQXmlStreamNotationDeclaration {
    /// <p>Destroys the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QVector<QXmlStreamNotationDeclaration>::~QVector()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#dtor.QVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the vector.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_dQVector(
            self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QVectorOfQXmlStreamEntityDeclaration {
    /// <p>Destroys the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QVector<QXmlStreamEntityDeclaration>::~QVector()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#dtor.QVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the vector.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_dQVector(
            self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
        )
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QVectorOfQXmlStreamAttribute>>
    for crate::QVectorOfQXmlStreamAttribute
{
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this vector; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamAttribute>::operator==(const QVector<QXmlStreamAttribute>& v) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this vector; otherwise returns <code>false</code>.</p>
    /// <p>Two vectors are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, v: &::cpp_utils::Ref<crate::QVectorOfQXmlStreamAttribute>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_operator__(
                self as *const crate::QVectorOfQXmlStreamAttribute,
                v.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QVectorOfQStringRef>>
    for crate::QVectorOfQStringRef
{
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this vector; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QStringRef>::operator==(const QVector<QStringRef>& v) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this vector; otherwise returns <code>false</code>.</p>
    /// <p>Two vectors are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, v: &::cpp_utils::Ref<crate::QVectorOfQStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_operator__(
                self as *const crate::QVectorOfQStringRef,
                v.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QVectorOfUint>> for crate::QVectorOfUint {
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this vector; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<unsigned int>::operator==(const QVector<unsigned int>& v) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this vector; otherwise returns <code>false</code>.</p>
    /// <p>Two vectors are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, v: &::cpp_utils::Ref<crate::QVectorOfUint>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_operator__(
                self as *const crate::QVectorOfUint,
                v.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QVectorOfInt>> for crate::QVectorOfInt {
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this vector; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<int>::operator==(const QVector<int>& v) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this vector; otherwise returns <code>false</code>.</p>
    /// <p>Two vectors are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, v: &::cpp_utils::Ref<crate::QVectorOfInt>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_int_operator__(
                self as *const crate::QVectorOfInt,
                v.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QVectorOfQPointF>> for crate::QVectorOfQPointF {
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this vector; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QPointF>::operator==(const QVector<QPointF>& v) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this vector; otherwise returns <code>false</code>.</p>
    /// <p>Two vectors are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, v: &::cpp_utils::Ref<crate::QVectorOfQPointF>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_operator__(
                self as *const crate::QVectorOfQPointF,
                v.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>>
    for crate::QVectorOfQPairOfDoubleQVariant
{
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this vector; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QPair<double, QVariant>>::operator==(const QVector<QPair<double, QVariant>>& v) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this vector; otherwise returns <code>false</code>.</p>
    /// <p>Two vectors are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, v: &::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_operator__(
                self as *const crate::QVectorOfQPairOfDoubleQVariant,
                v.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QVectorOfQXmlStreamNamespaceDeclaration>>
    for crate::QVectorOfQXmlStreamNamespaceDeclaration
{
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this vector; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamNamespaceDeclaration>::operator==(const QVector<QXmlStreamNamespaceDeclaration>& v) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this vector; otherwise returns <code>false</code>.</p>
    /// <p>Two vectors are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, v: &::cpp_utils::Ref<crate::QVectorOfQXmlStreamNamespaceDeclaration>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_operator__(
                self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration,
                v.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QVectorOfQXmlStreamNotationDeclaration>>
    for crate::QVectorOfQXmlStreamNotationDeclaration
{
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this vector; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamNotationDeclaration>::operator==(const QVector<QXmlStreamNotationDeclaration>& v) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this vector; otherwise returns <code>false</code>.</p>
    /// <p>Two vectors are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, v: &::cpp_utils::Ref<crate::QVectorOfQXmlStreamNotationDeclaration>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_operator__(
                self as *const crate::QVectorOfQXmlStreamNotationDeclaration,
                v.as_raw_ptr(),
            )
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QVectorOfQXmlStreamEntityDeclaration>>
    for crate::QVectorOfQXmlStreamEntityDeclaration
{
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this vector; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QXmlStreamEntityDeclaration>::operator==(const QVector<QXmlStreamEntityDeclaration>& v) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this vector; otherwise returns <code>false</code>.</p>
    /// <p>Two vectors are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, v: &::cpp_utils::Ref<crate::QVectorOfQXmlStreamEntityDeclaration>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_operator__(
                self as *const crate::QVectorOfQXmlStreamEntityDeclaration,
                v.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QVectorOfQXmlStreamAttribute>>
    for crate::QVectorOfQXmlStreamAttribute
{
    /// <p>Appends the items of the <i>other</i> vector to this vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamAttribute>& QVector<QXmlStreamAttribute>::operator+=(const QVector<QXmlStreamAttribute>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> vector to this vector and returns a reference to this vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QVectorOfQXmlStreamAttribute>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_operator__4(
                self as *mut crate::QVectorOfQXmlStreamAttribute,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QVectorOfQStringRef>>
    for crate::QVectorOfQStringRef
{
    /// <p>Appends the items of the <i>other</i> vector to this vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStringRef>& QVector<QStringRef>::operator+=(const QVector<QStringRef>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> vector to this vector and returns a reference to this vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QVectorOfQStringRef>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_operator__4(
                self as *mut crate::QVectorOfQStringRef,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QVectorOfUint>> for crate::QVectorOfUint {
    /// <p>Appends the items of the <i>other</i> vector to this vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<unsigned int>& QVector<unsigned int>::operator+=(const QVector<unsigned int>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> vector to this vector and returns a reference to this vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QVectorOfUint>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_operator__4(
                self as *mut crate::QVectorOfUint,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QVectorOfInt>> for crate::QVectorOfInt {
    /// <p>Appends the items of the <i>other</i> vector to this vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<int>& QVector<int>::operator+=(const QVector<int>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> vector to this vector and returns a reference to this vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QVectorOfInt>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_int_operator__4(
                self as *mut crate::QVectorOfInt,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QVectorOfQPointF>> for crate::QVectorOfQPointF {
    /// <p>Appends the items of the <i>other</i> vector to this vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QPointF>& QVector<QPointF>::operator+=(const QVector<QPointF>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> vector to this vector and returns a reference to this vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QVectorOfQPointF>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_operator__4(
                self as *mut crate::QVectorOfQPointF,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QVectorOfQStaticPlugin>>
    for crate::QVectorOfQStaticPlugin
{
    /// <p>Appends the items of the <i>other</i> vector to this vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStaticPlugin>& QVector<QStaticPlugin>::operator+=(const QVector<QStaticPlugin>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> vector to this vector and returns a reference to this vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QVectorOfQStaticPlugin>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_operator__4(
                self as *mut crate::QVectorOfQStaticPlugin,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>>
    for crate::QVectorOfQPairOfDoubleQVariant
{
    /// <p>Appends the items of the <i>other</i> vector to this vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QPair<double, QVariant>>& QVector<QPair<double, QVariant>>::operator+=(const QVector<QPair<double, QVariant>>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> vector to this vector and returns a reference to this vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_operator__4(
                self as *mut crate::QVectorOfQPairOfDoubleQVariant,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QVectorOfOffsetData>>
    for crate::QVectorOfOffsetData
{
    /// <p>Appends the items of the <i>other</i> vector to this vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QTimeZone::OffsetData>& QVector<QTimeZone::OffsetData>::operator+=(const QVector<QTimeZone::OffsetData>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> vector to this vector and returns a reference to this vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QVectorOfOffsetData>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_operator__4(
                self as *mut crate::QVectorOfOffsetData,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QVectorOfQXmlStreamNamespaceDeclaration>>
    for crate::QVectorOfQXmlStreamNamespaceDeclaration
{
    /// <p>Appends the items of the <i>other</i> vector to this vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamNamespaceDeclaration>& QVector<QXmlStreamNamespaceDeclaration>::operator+=(const QVector<QXmlStreamNamespaceDeclaration>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> vector to this vector and returns a reference to this vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QVectorOfQXmlStreamNamespaceDeclaration>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_operator__4(
                self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QVectorOfQXmlStreamNotationDeclaration>>
    for crate::QVectorOfQXmlStreamNotationDeclaration
{
    /// <p>Appends the items of the <i>other</i> vector to this vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamNotationDeclaration>& QVector<QXmlStreamNotationDeclaration>::operator+=(const QVector<QXmlStreamNotationDeclaration>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> vector to this vector and returns a reference to this vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QVectorOfQXmlStreamNotationDeclaration>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_operator__4(
                self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QVectorOfQXmlStreamEntityDeclaration>>
    for crate::QVectorOfQXmlStreamEntityDeclaration
{
    /// <p>Appends the items of the <i>other</i> vector to this vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamEntityDeclaration>& QVector<QXmlStreamEntityDeclaration>::operator+=(const QVector<QXmlStreamEntityDeclaration>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> vector to this vector and returns a reference to this vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QVectorOfQXmlStreamEntityDeclaration>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_operator__4(
                self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QVectorOfQXmlStreamAttribute>>
    for &crate::QVectorOfQXmlStreamAttribute
{
    type Output = ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamAttribute>;
    /// <p>Returns a vector that contains all the items in this vector followed by all the items in the <i>other</i> vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamAttribute> QVector<QXmlStreamAttribute>::operator+(const QVector<QXmlStreamAttribute>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a vector that contains all the items in this vector followed by all the items in the <i>other</i> vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QVectorOfQXmlStreamAttribute>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamAttribute> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_operator_2(
                self as *const crate::QVectorOfQXmlStreamAttribute,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QVectorOfQStringRef>> for &crate::QVectorOfQStringRef {
    type Output = ::cpp_utils::CppBox<crate::QVectorOfQStringRef>;
    /// <p>Returns a vector that contains all the items in this vector followed by all the items in the <i>other</i> vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStringRef> QVector<QStringRef>::operator+(const QVector<QStringRef>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a vector that contains all the items in this vector followed by all the items in the <i>other</i> vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QVectorOfQStringRef>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStringRef> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_operator_2(
                self as *const crate::QVectorOfQStringRef,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QVectorOfUint>> for &crate::QVectorOfUint {
    type Output = ::cpp_utils::CppBox<crate::QVectorOfUint>;
    /// <p>Returns a vector that contains all the items in this vector followed by all the items in the <i>other</i> vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<unsigned int> QVector<unsigned int>::operator+(const QVector<unsigned int>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a vector that contains all the items in this vector followed by all the items in the <i>other</i> vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QVectorOfUint>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfUint> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_operator_2(
                self as *const crate::QVectorOfUint,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QVectorOfInt>> for &crate::QVectorOfInt {
    type Output = ::cpp_utils::CppBox<crate::QVectorOfInt>;
    /// <p>Returns a vector that contains all the items in this vector followed by all the items in the <i>other</i> vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<int> QVector<int>::operator+(const QVector<int>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a vector that contains all the items in this vector followed by all the items in the <i>other</i> vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QVectorOfInt>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfInt> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_int_operator_2(
                self as *const crate::QVectorOfInt,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QVectorOfQPointF>> for &crate::QVectorOfQPointF {
    type Output = ::cpp_utils::CppBox<crate::QVectorOfQPointF>;
    /// <p>Returns a vector that contains all the items in this vector followed by all the items in the <i>other</i> vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QPointF> QVector<QPointF>::operator+(const QVector<QPointF>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a vector that contains all the items in this vector followed by all the items in the <i>other</i> vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QVectorOfQPointF>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQPointF> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_operator_2(
                self as *const crate::QVectorOfQPointF,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QVectorOfQStaticPlugin>>
    for &crate::QVectorOfQStaticPlugin
{
    type Output = ::cpp_utils::CppBox<crate::QVectorOfQStaticPlugin>;
    /// <p>Returns a vector that contains all the items in this vector followed by all the items in the <i>other</i> vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStaticPlugin> QVector<QStaticPlugin>::operator+(const QVector<QStaticPlugin>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a vector that contains all the items in this vector followed by all the items in the <i>other</i> vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QVectorOfQStaticPlugin>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQStaticPlugin> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_operator_2(
                self as *const crate::QVectorOfQStaticPlugin,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>>
    for &crate::QVectorOfQPairOfDoubleQVariant
{
    type Output = ::cpp_utils::CppBox<crate::QVectorOfQPairOfDoubleQVariant>;
    /// <p>Returns a vector that contains all the items in this vector followed by all the items in the <i>other</i> vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QPair<double, QVariant>> QVector<QPair<double, QVariant>>::operator+(const QVector<QPair<double, QVariant>>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a vector that contains all the items in this vector followed by all the items in the <i>other</i> vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQPairOfDoubleQVariant> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_operator_2(
                self as *const crate::QVectorOfQPairOfDoubleQVariant,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QVectorOfOffsetData>> for &crate::QVectorOfOffsetData {
    type Output = ::cpp_utils::CppBox<crate::QVectorOfOffsetData>;
    /// <p>Returns a vector that contains all the items in this vector followed by all the items in the <i>other</i> vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QTimeZone::OffsetData> QVector<QTimeZone::OffsetData>::operator+(const QVector<QTimeZone::OffsetData>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a vector that contains all the items in this vector followed by all the items in the <i>other</i> vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QVectorOfOffsetData>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfOffsetData> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_operator_2(
                self as *const crate::QVectorOfOffsetData,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QVectorOfQXmlStreamNamespaceDeclaration>>
    for &crate::QVectorOfQXmlStreamNamespaceDeclaration
{
    type Output = ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamNamespaceDeclaration>;
    /// <p>Returns a vector that contains all the items in this vector followed by all the items in the <i>other</i> vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamNamespaceDeclaration> QVector<QXmlStreamNamespaceDeclaration>::operator+(const QVector<QXmlStreamNamespaceDeclaration>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a vector that contains all the items in this vector followed by all the items in the <i>other</i> vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QVectorOfQXmlStreamNamespaceDeclaration>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamNamespaceDeclaration> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_operator_2(
                self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QVectorOfQXmlStreamNotationDeclaration>>
    for &crate::QVectorOfQXmlStreamNotationDeclaration
{
    type Output = ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamNotationDeclaration>;
    /// <p>Returns a vector that contains all the items in this vector followed by all the items in the <i>other</i> vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamNotationDeclaration> QVector<QXmlStreamNotationDeclaration>::operator+(const QVector<QXmlStreamNotationDeclaration>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a vector that contains all the items in this vector followed by all the items in the <i>other</i> vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QVectorOfQXmlStreamNotationDeclaration>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamNotationDeclaration> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_operator_2(
                self as *const crate::QVectorOfQXmlStreamNotationDeclaration,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QVectorOfQXmlStreamEntityDeclaration>>
    for &crate::QVectorOfQXmlStreamEntityDeclaration
{
    type Output = ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamEntityDeclaration>;
    /// <p>Returns a vector that contains all the items in this vector followed by all the items in the <i>other</i> vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamEntityDeclaration> QVector<QXmlStreamEntityDeclaration>::operator+(const QVector<QXmlStreamEntityDeclaration>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a vector that contains all the items in this vector followed by all the items in the <i>other</i> vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QVectorOfQXmlStreamEntityDeclaration>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQXmlStreamEntityDeclaration> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_operator_2(
                self as *const crate::QVectorOfQXmlStreamEntityDeclaration,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QXmlStreamAttribute>>
    for crate::QVectorOfQXmlStreamAttribute
{
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamAttribute>& QVector<QXmlStreamAttribute>::operator+=(const QXmlStreamAttribute& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<crate::QXmlStreamAttribute>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_operator__5(
                self as *mut crate::QVectorOfQXmlStreamAttribute,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QStringRef>> for crate::QVectorOfQStringRef {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStringRef>& QVector<QStringRef>::operator+=(const QStringRef& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<crate::QStringRef>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_operator__5(
                self as *mut crate::QVectorOfQStringRef,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<::std::os::raw::c_uint>> for crate::QVectorOfUint {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<unsigned int>& QVector<unsigned int>::operator+=(const unsigned int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<::std::os::raw::c_uint>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_operator__5(
                self as *mut crate::QVectorOfUint,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<::std::os::raw::c_int>> for crate::QVectorOfInt {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<int>& QVector<int>::operator+=(const int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<::std::os::raw::c_int>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_int_operator__5(
                self as *mut crate::QVectorOfInt,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QPointF>> for crate::QVectorOfQPointF {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QPointF>& QVector<QPointF>::operator+=(const QPointF& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<crate::QPointF>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_operator__5(
                self as *mut crate::QVectorOfQPointF,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QStaticPlugin>>
    for crate::QVectorOfQStaticPlugin
{
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStaticPlugin>& QVector<QStaticPlugin>::operator+=(const QStaticPlugin& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<crate::QStaticPlugin>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_operator__5(
                self as *mut crate::QVectorOfQStaticPlugin,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>
    for crate::QVectorOfQPairOfDoubleQVariant
{
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QPair<double, QVariant>>& QVector<QPair<double, QVariant>>::operator+=(const QPair<double, QVariant>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<crate::QPairOfDoubleQVariant>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_operator__5(
                self as *mut crate::QVectorOfQPairOfDoubleQVariant,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>
    for crate::QVectorOfOffsetData
{
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QTimeZone::OffsetData>& QVector<QTimeZone::OffsetData>::operator+=(const QTimeZone::OffsetData& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<crate::q_time_zone::OffsetData>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_operator__5(
                self as *mut crate::QVectorOfOffsetData,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>
    for crate::QVectorOfQXmlStreamNamespaceDeclaration
{
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamNamespaceDeclaration>& QVector<QXmlStreamNamespaceDeclaration>::operator+=(const QXmlStreamNamespaceDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_operator__5(
                self as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>
    for crate::QVectorOfQXmlStreamNotationDeclaration
{
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamNotationDeclaration>& QVector<QXmlStreamNotationDeclaration>::operator+=(const QXmlStreamNotationDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_operator__5(
                self as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>
    for crate::QVectorOfQXmlStreamEntityDeclaration
{
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamEntityDeclaration>& QVector<QXmlStreamEntityDeclaration>::operator+=(const QXmlStreamEntityDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_operator__5(
                self as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QXmlStreamAttribute>>
    for &crate::QVectorOfQXmlStreamAttribute
{
    type Output = ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamAttribute>;
    /// <p>Appends <i>value</i> to the vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamAttribute>& QVector<QXmlStreamAttribute>::operator<<(const QXmlStreamAttribute& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>value</i> to the vector and returns a reference to this vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<crate::QXmlStreamAttribute>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamAttribute> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_operator__6(
                self as *const crate::QVectorOfQXmlStreamAttribute
                    as *mut crate::QVectorOfQXmlStreamAttribute,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QStringRef>> for &crate::QVectorOfQStringRef {
    type Output = ::cpp_utils::MutRef<crate::QVectorOfQStringRef>;
    /// <p>Appends <i>value</i> to the vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStringRef>& QVector<QStringRef>::operator<<(const QStringRef& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>value</i> to the vector and returns a reference to this vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<crate::QStringRef>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQStringRef> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_operator__6(
                self as *const crate::QVectorOfQStringRef as *mut crate::QVectorOfQStringRef,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<::std::os::raw::c_uint>> for &crate::QVectorOfUint {
    type Output = ::cpp_utils::MutRef<crate::QVectorOfUint>;
    /// <p>Appends <i>value</i> to the vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<unsigned int>& QVector<unsigned int>::operator<<(const unsigned int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>value</i> to the vector and returns a reference to this vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<::std::os::raw::c_uint>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfUint> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_operator__6(
                self as *const crate::QVectorOfUint as *mut crate::QVectorOfUint,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<::std::os::raw::c_int>> for &crate::QVectorOfInt {
    type Output = ::cpp_utils::MutRef<crate::QVectorOfInt>;
    /// <p>Appends <i>value</i> to the vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<int>& QVector<int>::operator<<(const int& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>value</i> to the vector and returns a reference to this vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<::std::os::raw::c_int>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfInt> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_int_operator__6(
                self as *const crate::QVectorOfInt as *mut crate::QVectorOfInt,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QPointF>> for &crate::QVectorOfQPointF {
    type Output = ::cpp_utils::MutRef<crate::QVectorOfQPointF>;
    /// <p>Appends <i>value</i> to the vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QPointF>& QVector<QPointF>::operator<<(const QPointF& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>value</i> to the vector and returns a reference to this vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<crate::QPointF>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQPointF> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_operator__6(
                self as *const crate::QVectorOfQPointF as *mut crate::QVectorOfQPointF,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QStaticPlugin>> for &crate::QVectorOfQStaticPlugin {
    type Output = ::cpp_utils::MutRef<crate::QVectorOfQStaticPlugin>;
    /// <p>Appends <i>value</i> to the vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStaticPlugin>& QVector<QStaticPlugin>::operator<<(const QStaticPlugin& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>value</i> to the vector and returns a reference to this vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<crate::QStaticPlugin>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQStaticPlugin> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_operator__6(
                self as *const crate::QVectorOfQStaticPlugin as *mut crate::QVectorOfQStaticPlugin,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>>
    for &crate::QVectorOfQPairOfDoubleQVariant
{
    type Output = ::cpp_utils::MutRef<crate::QVectorOfQPairOfDoubleQVariant>;
    /// <p>Appends <i>value</i> to the vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QPair<double, QVariant>>& QVector<QPair<double, QVariant>>::operator<<(const QPair<double, QVariant>& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>value</i> to the vector and returns a reference to this vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<crate::QPairOfDoubleQVariant>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQPairOfDoubleQVariant> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_operator__6(
                self as *const crate::QVectorOfQPairOfDoubleQVariant
                    as *mut crate::QVectorOfQPairOfDoubleQVariant,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::q_time_zone::OffsetData>>
    for &crate::QVectorOfOffsetData
{
    type Output = ::cpp_utils::MutRef<crate::QVectorOfOffsetData>;
    /// <p>Appends <i>value</i> to the vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QTimeZone::OffsetData>& QVector<QTimeZone::OffsetData>::operator<<(const QTimeZone::OffsetData& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>value</i> to the vector and returns a reference to this vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<crate::q_time_zone::OffsetData>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfOffsetData> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_operator__6(
                self as *const crate::QVectorOfOffsetData as *mut crate::QVectorOfOffsetData,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>>
    for &crate::QVectorOfQXmlStreamNamespaceDeclaration
{
    type Output = ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamNamespaceDeclaration>;
    /// <p>Appends <i>value</i> to the vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamNamespaceDeclaration>& QVector<QXmlStreamNamespaceDeclaration>::operator<<(const QXmlStreamNamespaceDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>value</i> to the vector and returns a reference to this vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<crate::QXmlStreamNamespaceDeclaration>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamNamespaceDeclaration> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_operator__6(
                self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration
                    as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>>
    for &crate::QVectorOfQXmlStreamNotationDeclaration
{
    type Output = ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamNotationDeclaration>;
    /// <p>Appends <i>value</i> to the vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamNotationDeclaration>& QVector<QXmlStreamNotationDeclaration>::operator<<(const QXmlStreamNotationDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>value</i> to the vector and returns a reference to this vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<crate::QXmlStreamNotationDeclaration>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamNotationDeclaration> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_operator__6(
                self as *const crate::QVectorOfQXmlStreamNotationDeclaration
                    as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>>
    for &crate::QVectorOfQXmlStreamEntityDeclaration
{
    type Output = ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamEntityDeclaration>;
    /// <p>Appends <i>value</i> to the vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamEntityDeclaration>& QVector<QXmlStreamEntityDeclaration>::operator<<(const QXmlStreamEntityDeclaration& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>value</i> to the vector and returns a reference to this vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<crate::QXmlStreamEntityDeclaration>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamEntityDeclaration> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_operator__6(
                self as *const crate::QVectorOfQXmlStreamEntityDeclaration
                    as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVectorOfQXmlStreamAttribute>>
    for &crate::QVectorOfQXmlStreamAttribute
{
    type Output = ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamAttribute>;
    /// <p>Appends <i>other</i> to the vector and returns a reference to the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamAttribute>& QVector<QXmlStreamAttribute>::operator<<(const QVector<QXmlStreamAttribute>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>other</i> to the vector and returns a reference to the vector.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QVectorOfQXmlStreamAttribute>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamAttribute> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamAttribute_operator__7(
                self as *const crate::QVectorOfQXmlStreamAttribute
                    as *mut crate::QVectorOfQXmlStreamAttribute,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVectorOfQStringRef>> for &crate::QVectorOfQStringRef {
    type Output = ::cpp_utils::MutRef<crate::QVectorOfQStringRef>;
    /// <p>Appends <i>other</i> to the vector and returns a reference to the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStringRef>& QVector<QStringRef>::operator<<(const QVector<QStringRef>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>other</i> to the vector and returns a reference to the vector.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QVectorOfQStringRef>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQStringRef> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QStringRef_operator__7(
                self as *const crate::QVectorOfQStringRef as *mut crate::QVectorOfQStringRef,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVectorOfUint>> for &crate::QVectorOfUint {
    type Output = ::cpp_utils::MutRef<crate::QVectorOfUint>;
    /// <p>Appends <i>other</i> to the vector and returns a reference to the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<unsigned int>& QVector<unsigned int>::operator<<(const QVector<unsigned int>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>other</i> to the vector and returns a reference to the vector.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QVectorOfUint>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfUint> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_unsigned_int_operator__7(
                self as *const crate::QVectorOfUint as *mut crate::QVectorOfUint,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVectorOfInt>> for &crate::QVectorOfInt {
    type Output = ::cpp_utils::MutRef<crate::QVectorOfInt>;
    /// <p>Appends <i>other</i> to the vector and returns a reference to the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<int>& QVector<int>::operator<<(const QVector<int>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>other</i> to the vector and returns a reference to the vector.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QVectorOfInt>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfInt> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_int_operator__7(
                self as *const crate::QVectorOfInt as *mut crate::QVectorOfInt,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVectorOfQPointF>> for &crate::QVectorOfQPointF {
    type Output = ::cpp_utils::MutRef<crate::QVectorOfQPointF>;
    /// <p>Appends <i>other</i> to the vector and returns a reference to the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QPointF>& QVector<QPointF>::operator<<(const QVector<QPointF>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>other</i> to the vector and returns a reference to the vector.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QVectorOfQPointF>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQPointF> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QPointF_operator__7(
                self as *const crate::QVectorOfQPointF as *mut crate::QVectorOfQPointF,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVectorOfQStaticPlugin>>
    for &crate::QVectorOfQStaticPlugin
{
    type Output = ::cpp_utils::MutRef<crate::QVectorOfQStaticPlugin>;
    /// <p>Appends <i>other</i> to the vector and returns a reference to the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QStaticPlugin>& QVector<QStaticPlugin>::operator<<(const QVector<QStaticPlugin>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>other</i> to the vector and returns a reference to the vector.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QVectorOfQStaticPlugin>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQStaticPlugin> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QStaticPlugin_operator__7(
                self as *const crate::QVectorOfQStaticPlugin as *mut crate::QVectorOfQStaticPlugin,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>>
    for &crate::QVectorOfQPairOfDoubleQVariant
{
    type Output = ::cpp_utils::MutRef<crate::QVectorOfQPairOfDoubleQVariant>;
    /// <p>Appends <i>other</i> to the vector and returns a reference to the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QPair<double, QVariant>>& QVector<QPair<double, QVariant>>::operator<<(const QVector<QPair<double, QVariant>>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>other</i> to the vector and returns a reference to the vector.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQPairOfDoubleQVariant> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QPair_double_QVariant_operator__7(
                self as *const crate::QVectorOfQPairOfDoubleQVariant
                    as *mut crate::QVectorOfQPairOfDoubleQVariant,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVectorOfOffsetData>> for &crate::QVectorOfOffsetData {
    type Output = ::cpp_utils::MutRef<crate::QVectorOfOffsetData>;
    /// <p>Appends <i>other</i> to the vector and returns a reference to the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QTimeZone::OffsetData>& QVector<QTimeZone::OffsetData>::operator<<(const QVector<QTimeZone::OffsetData>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>other</i> to the vector and returns a reference to the vector.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QVectorOfOffsetData>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfOffsetData> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QTimeZone_OffsetData_operator__7(
                self as *const crate::QVectorOfOffsetData as *mut crate::QVectorOfOffsetData,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVectorOfQXmlStreamNamespaceDeclaration>>
    for &crate::QVectorOfQXmlStreamNamespaceDeclaration
{
    type Output = ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamNamespaceDeclaration>;
    /// <p>Appends <i>other</i> to the vector and returns a reference to the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamNamespaceDeclaration>& QVector<QXmlStreamNamespaceDeclaration>::operator<<(const QVector<QXmlStreamNamespaceDeclaration>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>other</i> to the vector and returns a reference to the vector.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QVectorOfQXmlStreamNamespaceDeclaration>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamNamespaceDeclaration> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNamespaceDeclaration_operator__7(
                self as *const crate::QVectorOfQXmlStreamNamespaceDeclaration
                    as *mut crate::QVectorOfQXmlStreamNamespaceDeclaration,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVectorOfQXmlStreamNotationDeclaration>>
    for &crate::QVectorOfQXmlStreamNotationDeclaration
{
    type Output = ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamNotationDeclaration>;
    /// <p>Appends <i>other</i> to the vector and returns a reference to the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamNotationDeclaration>& QVector<QXmlStreamNotationDeclaration>::operator<<(const QVector<QXmlStreamNotationDeclaration>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>other</i> to the vector and returns a reference to the vector.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QVectorOfQXmlStreamNotationDeclaration>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamNotationDeclaration> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamNotationDeclaration_operator__7(
                self as *const crate::QVectorOfQXmlStreamNotationDeclaration
                    as *mut crate::QVectorOfQXmlStreamNotationDeclaration,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVectorOfQXmlStreamEntityDeclaration>>
    for &crate::QVectorOfQXmlStreamEntityDeclaration
{
    type Output = ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamEntityDeclaration>;
    /// <p>Appends <i>other</i> to the vector and returns a reference to the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamEntityDeclaration>& QVector<QXmlStreamEntityDeclaration>::operator<<(const QVector<QXmlStreamEntityDeclaration>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>other</i> to the vector and returns a reference to the vector.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QVectorOfQXmlStreamEntityDeclaration>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQXmlStreamEntityDeclaration> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QXmlStreamEntityDeclaration_operator__7(
                self as *const crate::QVectorOfQXmlStreamEntityDeclaration
                    as *mut crate::QVectorOfQXmlStreamEntityDeclaration,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QVectorOfQStringRef>>
    for crate::QVectorOfQStringRef
{
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QVector<QStringRef>& lhs, const QVector<QStringRef>& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QVectorOfQStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_185(
                self as *const crate::QVectorOfQStringRef,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QVectorOfUint>> for crate::QVectorOfUint {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QVector<unsigned int>& lhs, const QVector<unsigned int>& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QVectorOfUint>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_186(
                self as *const crate::QVectorOfUint,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QVectorOfInt>> for crate::QVectorOfInt {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QVector<int>& lhs, const QVector<int>& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QVectorOfInt>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_187(
                self as *const crate::QVectorOfInt,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>>
    for crate::QVectorOfQPairOfDoubleQVariant
{
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QVector<QPair<double, QVariant>>& lhs, const QVector<QPair<double, QVariant>>& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_191(
                self as *const crate::QVectorOfQPairOfDoubleQVariant,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QVectorOfQStringRef>>
    for crate::QVectorOfQStringRef
{
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QVector<QStringRef>& lhs, const QVector<QStringRef>& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QVectorOfQStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_197(
                self as *const crate::QVectorOfQStringRef,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QVectorOfUint>> for crate::QVectorOfUint {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QVector<unsigned int>& lhs, const QVector<unsigned int>& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QVectorOfUint>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_198(
                self as *const crate::QVectorOfUint,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QVectorOfInt>> for crate::QVectorOfInt {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QVector<int>& lhs, const QVector<int>& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QVectorOfInt>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_199(
                self as *const crate::QVectorOfInt,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>>
    for crate::QVectorOfQPairOfDoubleQVariant
{
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QVector<QPair<double, QVariant>>& lhs, const QVector<QPair<double, QVariant>>& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_203(
                self as *const crate::QVectorOfQPairOfDoubleQVariant,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QVectorOfQStringRef>>
    for crate::QVectorOfQStringRef
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QVector<QStringRef>& lhs, const QVector<QStringRef>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QVectorOfQStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__258(
                self as *const crate::QVectorOfQStringRef,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QVectorOfUint>> for crate::QVectorOfUint {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QVector<unsigned int>& lhs, const QVector<unsigned int>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QVectorOfUint>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__259(
                self as *const crate::QVectorOfUint,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QVectorOfInt>> for crate::QVectorOfInt {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QVector<int>& lhs, const QVector<int>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QVectorOfInt>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__260(
                self as *const crate::QVectorOfInt,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>>
    for crate::QVectorOfQPairOfDoubleQVariant
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QVector<QPair<double, QVariant>>& lhs, const QVector<QPair<double, QVariant>>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__264(
                self as *const crate::QVectorOfQPairOfDoubleQVariant,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QVectorOfQStringRef>>
    for crate::QVectorOfQStringRef
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QVector<QStringRef>& lhs, const QVector<QStringRef>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QVectorOfQStringRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__270(
                self as *const crate::QVectorOfQStringRef,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QVectorOfUint>> for crate::QVectorOfUint {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QVector<unsigned int>& lhs, const QVector<unsigned int>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QVectorOfUint>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__271(
                self as *const crate::QVectorOfUint,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QVectorOfInt>> for crate::QVectorOfInt {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QVector<int>& lhs, const QVector<int>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QVectorOfInt>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__272(
                self as *const crate::QVectorOfInt,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>>
    for crate::QVectorOfQPairOfDoubleQVariant
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QVector<QPair<double, QVariant>>& lhs, const QVector<QPair<double, QVariant>>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__276(
                self as *const crate::QVectorOfQPairOfDoubleQVariant,
                rhs.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::MouseButton>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<Qt::MouseButton> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, e: crate::QFlags<crate::MouseButton>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__281(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::Orientation>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<Qt::Orientation> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, e: crate::QFlags<crate::Orientation>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__282(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::KeyboardModifier>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<Qt::KeyboardModifier> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::KeyboardModifier>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__283(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::WindowType>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<Qt::WindowType> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, e: crate::QFlags<crate::WindowType>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__284(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::AlignmentFlag>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<Qt::AlignmentFlag> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::AlignmentFlag>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__285(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::Edge>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<Qt::Edge> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, e: crate::QFlags<crate::Edge>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__286(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::ImageConversionFlag>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<Qt::ImageConversionFlag> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::ImageConversionFlag>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__287(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::DockWidgetArea>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<Qt::DockWidgetArea> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::DockWidgetArea>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__288(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::ToolBarArea>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<Qt::ToolBarArea> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, e: crate::QFlags<crate::ToolBarArea>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__289(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::WindowState>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<Qt::WindowState> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, e: crate::QFlags<crate::WindowState>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__290(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::ScreenOrientation>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<Qt::ScreenOrientation> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::ScreenOrientation>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__291(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::DropAction>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<Qt::DropAction> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, e: crate::QFlags<crate::DropAction>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__292(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::ItemFlag>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<Qt::ItemFlag> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, e: crate::QFlags<crate::ItemFlag>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__293(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::MatchFlag>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<Qt::MatchFlag> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, e: crate::QFlags<crate::MatchFlag>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__294(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::TextInteractionFlag>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<Qt::TextInteractionFlag> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::TextInteractionFlag>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__295(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::InputMethodQuery>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<Qt::InputMethodQuery> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::InputMethodQuery>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__296(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::InputMethodHint>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<Qt::InputMethodHint> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::InputMethodHint>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__297(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::TouchPointState>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<Qt::TouchPointState> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::TouchPointState>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__298(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::MouseEventFlag>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<Qt::MouseEventFlag> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::MouseEventFlag>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__299(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::GestureFlag>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<Qt::GestureFlag> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, e: crate::QFlags<crate::GestureFlag>) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__300(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_byte_array::Base64Option>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<QByteArray::Base64Option> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::q_byte_array::Base64Option>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__302(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_string::SectionFlag>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<QString::SectionFlag> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::q_string::SectionFlag>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__303(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_meta_type::TypeFlag>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<QMetaType::TypeFlag> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::q_meta_type::TypeFlag>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__304(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::FindChildOption>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<Qt::FindChildOption> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::FindChildOption>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__305(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_event_loop::ProcessEventsFlag>>
    for &crate::QDataStream
{
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<QEventLoop::ProcessEventsFlag> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::q_event_loop::ProcessEventsFlag>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__306(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_io_device::OpenModeFlag>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<QIODevice::OpenModeFlag> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::q_io_device::OpenModeFlag>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__307(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_locale::NumberOption>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<QLocale::NumberOption> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::q_locale::NumberOption>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__308(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_command_line_option::Flag>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<QCommandLineOption::Flag> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::q_command_line_option::Flag>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__309(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_text_stream::NumberFlag>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<QTextStream::NumberFlag> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::q_text_stream::NumberFlag>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__310(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_file_device::Permission>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<QFileDevice::Permission> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::q_file_device::Permission>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__311(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_file_device::FileHandleFlag>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<QFileDevice::FileHandleFlag> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::q_file_device::FileHandleFlag>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__312(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_dir::SortFlag>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<QDir::SortFlag> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::q_dir::SortFlag>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__313(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_dir::Filter>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<QDir::Filter> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::q_dir::Filter>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__314(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_dir_iterator::IteratorFlag>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<QDirIterator::IteratorFlag> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::q_dir_iterator::IteratorFlag>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__315(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_item_selection_model::SelectionFlag>>
    for &crate::QDataStream
{
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<QItemSelectionModel::SelectionFlag> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::q_item_selection_model::SelectionFlag>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__316(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_library::LoadHint>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<QLibrary::LoadHint> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::q_library::LoadHint>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__317(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_regular_expression::PatternOption>>
    for &crate::QDataStream
{
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<QRegularExpression::PatternOption> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::q_regular_expression::PatternOption>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__318(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_regular_expression::MatchOption>>
    for &crate::QDataStream
{
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<QRegularExpression::MatchOption> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::q_regular_expression::MatchOption>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__319(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_standard_paths::LocateOption>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<QStandardPaths::LocateOption> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::q_standard_paths::LocateOption>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__320(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_text_boundary_finder::BoundaryReason>>
    for &crate::QDataStream
{
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<QTextBoundaryFinder::BoundaryReason> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::q_text_boundary_finder::BoundaryReason>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__321(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_text_codec::ConversionFlag>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<QTextCodec::ConversionFlag> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::q_text_codec::ConversionFlag>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__322(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_url::UserInputResolutionOption>>
    for &crate::QDataStream
{
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<QUrl::UserInputResolutionOption> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::q_url::UserInputResolutionOption>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__323(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_url::ComponentFormattingOption>>
    for &crate::QDataStream
{
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<QUrl::ComponentFormattingOption> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::q_url::ComponentFormattingOption>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__324(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QListOfQString>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& s, QList<QString>& l)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        l: ::cpp_utils::MutRef<crate::QListOfQString>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__369(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                l.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QListOfQVariant>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& s, QList<QVariant>& l)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        l: ::cpp_utils::MutRef<crate::QListOfQVariant>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__372(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                l.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QListOfQByteArray>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& s, QList<QByteArray>& l)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        l: ::cpp_utils::MutRef<crate::QListOfQByteArray>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__373(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                l.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QListOfQLocale>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& s, QList<QLocale>& l)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        l: ::cpp_utils::MutRef<crate::QListOfQLocale>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__380(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                l.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QListOfQUrl>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& s, QList<QUrl>& l)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        l: ::cpp_utils::MutRef<crate::QListOfQUrl>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__384(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                l.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QListOfInt>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& s, QList<int>& l)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        l: ::cpp_utils::MutRef<crate::QListOfInt>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__388(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                l.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QListOfQPairOfQStringQString>>
    for &crate::QDataStream
{
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& s, QList<QPair<QString, QString>>& l)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        l: ::cpp_utils::MutRef<crate::QListOfQPairOfQStringQString>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__389(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                l.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQString>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QList<QString>& l)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQString>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__390(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQObject>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QList<QObject*>& l)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQObject>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__392(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQVariant>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QList<QVariant>& l)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQVariant>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__393(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQByteArray>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QList<QByteArray>& l)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQByteArray>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__394(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQModelIndex>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QList<QModelIndex>& l)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQModelIndex>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__396(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>>
    for &crate::QDataStream
{
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QList<QPersistentModelIndex>& l)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__397(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQAbstractState>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QList<QAbstractState*>& l)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQAbstractState>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__398(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQAbstractAnimation>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QList<QAbstractAnimation*>& l)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQAbstractAnimation>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__399(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfDayOfWeek>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QList<Qt::DayOfWeek>& l)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfDayOfWeek>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__400(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQLocale>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QList<QLocale>& l)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQLocale>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__401(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfCountry>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QList<QLocale::Country>& l)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfCountry>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__402(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQUrl>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QList<QUrl>& l)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQUrl>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__405(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQAbstractTransition>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QList<QAbstractTransition*>& l)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQAbstractTransition>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__407(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfInt>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QList<int>& l)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfInt>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__409(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>>
    for &crate::QDataStream
{
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QList<QPair<QString, QString>>& l)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__410(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QVectorOfUint>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& s, QVector<unsigned int>& v)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        v: ::cpp_utils::MutRef<crate::QVectorOfUint>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__413(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                v.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QVectorOfInt>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& s, QVector<int>& v)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        v: ::cpp_utils::MutRef<crate::QVectorOfInt>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__414(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                v.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QVectorOfQPointF>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& s, QVector<QPointF>& v)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        v: ::cpp_utils::MutRef<crate::QVectorOfQPointF>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__415(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                v.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QVectorOfQPairOfDoubleQVariant>>
    for &crate::QDataStream
{
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& s, QVector<QPair<double, QVariant>>& v)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        v: ::cpp_utils::MutRef<crate::QVectorOfQPairOfDoubleQVariant>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__418(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                v.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVectorOfUint>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QVector<unsigned int>& v)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        v: ::cpp_utils::Ref<crate::QVectorOfUint>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__425(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                v.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVectorOfInt>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QVector<int>& v)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        v: ::cpp_utils::Ref<crate::QVectorOfInt>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__426(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                v.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVectorOfQPointF>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QVector<QPointF>& v)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        v: ::cpp_utils::Ref<crate::QVectorOfQPointF>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__427(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                v.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>>
    for &crate::QDataStream
{
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QVector<QPair<double, QVariant>>& v)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        v: ::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__430(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                v.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QSetOfQAbstractState>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QSet<QAbstractState*>& set)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        set: ::cpp_utils::Ref<crate::QSetOfQAbstractState>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__436(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                set.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QHashOfQStringQVariant>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& s, QHash<QString, QVariant>& hash)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        hash: ::cpp_utils::MutRef<crate::QHashOfQStringQVariant>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__437(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                hash.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QHashOfIntQByteArray>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& s, QHash<int, QByteArray>& hash)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        hash: ::cpp_utils::MutRef<crate::QHashOfIntQByteArray>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__438(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                hash.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QHashOfQStringQVariant>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QHash<QString, QVariant>& hash)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        hash: ::cpp_utils::Ref<crate::QHashOfQStringQVariant>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__439(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                hash.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QHashOfIntQByteArray>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QHash<int, QByteArray>& hash)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        hash: ::cpp_utils::Ref<crate::QHashOfIntQByteArray>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__440(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                hash.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QMapOfQStringQVariant>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& s, QMap<QString, QVariant>& map)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        map: ::cpp_utils::MutRef<crate::QMapOfQStringQVariant>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__441(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                map.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QMapOfIntQVariant>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& s, QMap<int, QVariant>& map)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        map: ::cpp_utils::MutRef<crate::QMapOfIntQVariant>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__442(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                map.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QMapOfQStringQVariant>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QMap<QString, QVariant>& map)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        map: ::cpp_utils::Ref<crate::QMapOfQStringQVariant>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__443(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                map.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QMapOfIntQVariant>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QMap<int, QVariant>& map)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        map: ::cpp_utils::Ref<crate::QMapOfIntQVariant>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__444(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                map.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QPairOfQStringQJsonValue>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& s, QPair<QString, QJsonValue>& p)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        p: ::cpp_utils::MutRef<crate::QPairOfQStringQJsonValue>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__445(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                p.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QPairOfDoubleQVariant>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& s, QPair<double, QVariant>& p)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        p: ::cpp_utils::MutRef<crate::QPairOfDoubleQVariant>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__446(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                p.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QPairOfQStringQString>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& s, QPair<QString, QString>& p)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        p: ::cpp_utils::MutRef<crate::QPairOfQStringQString>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__447(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                p.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QPairOfQStringQJsonValue>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QPair<QString, QJsonValue>& p)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        p: ::cpp_utils::Ref<crate::QPairOfQStringQJsonValue>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__448(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                p.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QPair<double, QVariant>& p)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        p: ::cpp_utils::Ref<crate::QPairOfDoubleQVariant>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__449(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                p.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QPairOfQStringQString>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QPair<QString, QString>& p)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        p: ::cpp_utils::Ref<crate::QPairOfQStringQString>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__450(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                p.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QSetOfQAbstractState>>
    for crate::QSetOfQAbstractState
{
    /// <p>Returns <code>true</code> if the <i>other</i> set is equal to this set; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QSet<QAbstractState*>::operator==(const QSet<QAbstractState*>& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the <i>other</i> set is equal to this set; otherwise returns <code>false</code>.</p>
    /// <p>Two sets are considered equal if they contain the same elements.</p>
    /// <p>This function requires the value type to implement <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QSetOfQAbstractState>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_operator__(
                self as *const crate::QSetOfQAbstractState,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::cpp_utils::ops::BeginMut for &mut crate::QSetOfQAbstractState {
    type Output = ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::Iterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::iterator QSet<QAbstractState*>::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a non-const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> positioned at the first item in the set.</p>
    /// <p>This function was introduced in  Qt 4.2.</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_begin(
                self as *mut crate::QSetOfQAbstractState,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::Begin for &crate::QSetOfQAbstractState {
    type Output = ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::ConstIterator>;
    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> positioned at the first item in the set.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::const_iterator QSet<QAbstractState*>::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> positioned at the first item in the set.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qset.html#end">end</a>().</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_begin1(
                self as *const crate::QSetOfQAbstractState,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::EndMut for &mut crate::QSetOfQAbstractState {
    type Output = ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::Iterator>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::iterator QSet<QAbstractState*>::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Returns a non-const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary item after the last item in the set.</p>
    /// <p>This function was introduced in  Qt 4.2.</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_end(
                self as *mut crate::QSetOfQAbstractState,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::ops::End for &crate::QSetOfQAbstractState {
    type Output = ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::ConstIterator>;
    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> positioned at the imaginary item after the last item in the set.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>::const_iterator QSet<QAbstractState*>::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> positioned at the imaginary item after the last item in the set.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#constEnd">constEnd</a>() and <a href="http://doc.qt.io/qt-5/qset.html#begin">begin</a>().</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_set_of_q_abstract_state::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_end1(
                self as *const crate::QSetOfQAbstractState,
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::BitOrAssign<::cpp_utils::Ref<crate::QSetOfQAbstractState>>
    for crate::QSetOfQAbstractState
{
    /// <p>Same as unite(<i>other</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>& QSet<QAbstractState*>::operator|=(const QSet<QAbstractState*>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#operator-7c-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as unite(<i>other</i>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#operator-7c">operator|</a>(), <a href="http://doc.qt.io/qt-5/qset.html#operator-and-eq">operator&amp;=</a>(), and <a href="http://doc.qt.io/qt-5/qset.html#operator--eq">operator-=</a>().</p></div>
    fn bitor_assign(&mut self, other: ::cpp_utils::Ref<crate::QSetOfQAbstractState>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_operator__3(
                self as *mut crate::QSetOfQAbstractState,
                other.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::BitAndAssign<::cpp_utils::Ref<crate::QSetOfQAbstractState>>
    for crate::QSetOfQAbstractState
{
    /// <p>Same as intersect(<i>other</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>& QSet<QAbstractState*>::operator&=(const QSet<QAbstractState*>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#operator-and-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as intersect(<i>other</i>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#operator-and">operator&amp;</a>(), <a href="http://doc.qt.io/qt-5/qset.html#operator-7c-eq">operator|=</a>(), and <a href="http://doc.qt.io/qt-5/qset.html#operator--eq">operator-=</a>().</p></div>
    fn bitand_assign(&mut self, other: ::cpp_utils::Ref<crate::QSetOfQAbstractState>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_operator__5(
                self as *mut crate::QSetOfQAbstractState,
                other.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QSetOfQAbstractState>>
    for crate::QSetOfQAbstractState
{
    /// <p>Same as unite(<i>other</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>& QSet<QAbstractState*>::operator+=(const QSet<QAbstractState*>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as unite(<i>other</i>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#operator-7c">operator|</a>(), <a href="http://doc.qt.io/qt-5/qset.html#operator-and-eq">operator&amp;=</a>(), and <a href="http://doc.qt.io/qt-5/qset.html#operator--eq">operator-=</a>().</p></div>
    fn add_assign(&mut self, other: ::cpp_utils::Ref<crate::QSetOfQAbstractState>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_operator__7(
                self as *mut crate::QSetOfQAbstractState,
                other.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::SubAssign<::cpp_utils::Ref<crate::QSetOfQAbstractState>>
    for crate::QSetOfQAbstractState
{
    /// <p>Same as subtract(<i>other</i>).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*>& QSet<QAbstractState*>::operator-=(const QSet<QAbstractState*>& other)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#operator--eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Same as subtract(<i>other</i>).</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#operator-">operator-</a>(), <a href="http://doc.qt.io/qt-5/qset.html#operator-7c-eq">operator|=</a>(), and <a href="http://doc.qt.io/qt-5/qset.html#operator-and-eq">operator&amp;=</a>().</p></div>
    fn sub_assign(&mut self, other: ::cpp_utils::Ref<crate::QSetOfQAbstractState>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_operator__9(
                self as *mut crate::QSetOfQAbstractState,
                other.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::BitOr<::cpp_utils::Ref<crate::QSetOfQAbstractState>>
    for &crate::QSetOfQAbstractState
{
    type Output = ::cpp_utils::CppBox<crate::QSetOfQAbstractState>;
    /// <p>Returns a new <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> that is the union of this set and the <i>other</i> set.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*> QSet<QAbstractState*>::operator|(const QSet<QAbstractState*>& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#operator-7c">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a new <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> that is the union of this set and the <i>other</i> set.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#unite">unite</a>(), <a href="http://doc.qt.io/qt-5/qset.html#operator-7c-eq">operator|=</a>(), <a href="http://doc.qt.io/qt-5/qset.html#operator-and">operator&amp;</a>(), and <a href="http://doc.qt.io/qt-5/qset.html#operator-">operator-</a>().</p></div>
    fn bitor(
        self,
        other: ::cpp_utils::Ref<crate::QSetOfQAbstractState>,
    ) -> ::cpp_utils::CppBox<crate::QSetOfQAbstractState> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_operator_(
                self as *const crate::QSetOfQAbstractState,
                other.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::BitAnd<::cpp_utils::Ref<crate::QSetOfQAbstractState>>
    for &crate::QSetOfQAbstractState
{
    type Output = ::cpp_utils::CppBox<crate::QSetOfQAbstractState>;
    /// <p>Returns a new <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> that is the intersection of this set and the <i>other</i> set.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*> QSet<QAbstractState*>::operator&(const QSet<QAbstractState*>& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#operator-and">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a new <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> that is the intersection of this set and the <i>other</i> set.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#intersect">intersect</a>(), <a href="http://doc.qt.io/qt-5/qset.html#operator-and-eq">operator&amp;=</a>(), <a href="http://doc.qt.io/qt-5/qset.html#operator-7c">operator|</a>(), and <a href="http://doc.qt.io/qt-5/qset.html#operator-">operator-</a>().</p></div>
    fn bitand(
        self,
        other: ::cpp_utils::Ref<crate::QSetOfQAbstractState>,
    ) -> ::cpp_utils::CppBox<crate::QSetOfQAbstractState> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_operator_1(
                self as *const crate::QSetOfQAbstractState,
                other.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Add<::cpp_utils::Ref<crate::QSetOfQAbstractState>>
    for &crate::QSetOfQAbstractState
{
    type Output = ::cpp_utils::CppBox<crate::QSetOfQAbstractState>;
    /// <p>Returns a new <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> that is the union of this set and the <i>other</i> set.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*> QSet<QAbstractState*>::operator+(const QSet<QAbstractState*>& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a new <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> that is the union of this set and the <i>other</i> set.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#unite">unite</a>(), <a href="http://doc.qt.io/qt-5/qset.html#operator-7c-eq">operator|=</a>(), <a href="http://doc.qt.io/qt-5/qset.html#operator-and">operator&amp;</a>(), and <a href="http://doc.qt.io/qt-5/qset.html#operator-">operator-</a>().</p></div>
    fn add(
        self,
        other: ::cpp_utils::Ref<crate::QSetOfQAbstractState>,
    ) -> ::cpp_utils::CppBox<crate::QSetOfQAbstractState> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_operator_2(
                self as *const crate::QSetOfQAbstractState,
                other.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Sub<::cpp_utils::Ref<crate::QSetOfQAbstractState>>
    for &crate::QSetOfQAbstractState
{
    type Output = ::cpp_utils::CppBox<crate::QSetOfQAbstractState>;
    /// <p>Returns a new <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> that is the set difference of this set and the <i>other</i> set, i.e., this set - <i>other</i> set.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QSet<QAbstractState*> QSet<QAbstractState*>::operator-(const QSet<QAbstractState*>& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html#operator-">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a new <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> that is the set difference of this set and the <i>other</i> set, i.e., this set - <i>other</i> set.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qset.html#subtract">subtract</a>(), <a href="http://doc.qt.io/qt-5/qset.html#operator--eq">operator-=</a>(), <a href="http://doc.qt.io/qt-5/qset.html#operator-7c">operator|</a>(), and <a href="http://doc.qt.io/qt-5/qset.html#operator-and">operator&amp;</a>().</p></div>
    fn sub(
        self,
        other: ::cpp_utils::Ref<crate::QSetOfQAbstractState>,
    ) -> ::cpp_utils::CppBox<crate::QSetOfQAbstractState> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_operator_3(
                self as *const crate::QSetOfQAbstractState,
                other.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQString>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QList<QString>& list)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        list: ::cpp_utils::Ref<crate::QListOfQString>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__451(
                self as *const crate::QDebug,
                list.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQItemSelectionRange>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QList<QItemSelectionRange>& list)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        list: ::cpp_utils::Ref<crate::QListOfQItemSelectionRange>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__452(
                self as *const crate::QDebug,
                list.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQObject>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QList<QObject*>& list)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        list: ::cpp_utils::Ref<crate::QListOfQObject>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__453(
                self as *const crate::QDebug,
                list.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQVariant>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QList<QVariant>& list)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        list: ::cpp_utils::Ref<crate::QListOfQVariant>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__454(
                self as *const crate::QDebug,
                list.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQByteArray>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QList<QByteArray>& list)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        list: ::cpp_utils::Ref<crate::QListOfQByteArray>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__455(
                self as *const crate::QDebug,
                list.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQModelIndex>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QList<QModelIndex>& list)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        list: ::cpp_utils::Ref<crate::QListOfQModelIndex>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__457(
                self as *const crate::QDebug,
                list.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QList<QPersistentModelIndex>& list)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        list: ::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__458(
                self as *const crate::QDebug,
                list.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQAbstractState>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QList<QAbstractState*>& list)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        list: ::cpp_utils::Ref<crate::QListOfQAbstractState>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__459(
                self as *const crate::QDebug,
                list.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQAbstractAnimation>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QList<QAbstractAnimation*>& list)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        list: ::cpp_utils::Ref<crate::QListOfQAbstractAnimation>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__460(
                self as *const crate::QDebug,
                list.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfDayOfWeek>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QList<Qt::DayOfWeek>& list)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        list: ::cpp_utils::Ref<crate::QListOfDayOfWeek>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__461(
                self as *const crate::QDebug,
                list.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQLocale>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QList<QLocale>& list)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        list: ::cpp_utils::Ref<crate::QListOfQLocale>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__462(
                self as *const crate::QDebug,
                list.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfCountry>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QList<QLocale::Country>& list)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        list: ::cpp_utils::Ref<crate::QListOfCountry>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__463(
                self as *const crate::QDebug,
                list.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQFileInfo>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QList<QFileInfo>& list)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        list: ::cpp_utils::Ref<crate::QListOfQFileInfo>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__465(
                self as *const crate::QDebug,
                list.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQUrl>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QList<QUrl>& list)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, list: ::cpp_utils::Ref<crate::QListOfQUrl>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__466(
                self as *const crate::QDebug,
                list.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQMimeType>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QList<QMimeType>& list)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        list: ::cpp_utils::Ref<crate::QListOfQMimeType>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__467(
                self as *const crate::QDebug,
                list.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQAbstractTransition>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QList<QAbstractTransition*>& list)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        list: ::cpp_utils::Ref<crate::QListOfQAbstractTransition>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__468(
                self as *const crate::QDebug,
                list.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQStorageInfo>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QList<QStorageInfo>& list)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        list: ::cpp_utils::Ref<crate::QListOfQStorageInfo>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__469(
                self as *const crate::QDebug,
                list.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfInt>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QList<int>& list)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, list: ::cpp_utils::Ref<crate::QListOfInt>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__470(
                self as *const crate::QDebug,
                list.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QList<QPair<QString, QString>>& list)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        list: ::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__471(
                self as *const crate::QDebug,
                list.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVectorOfQStringRef>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QVector<QStringRef>& vec)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        vec: ::cpp_utils::Ref<crate::QVectorOfQStringRef>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__473(
                self as *const crate::QDebug,
                vec.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVectorOfUint>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QVector<unsigned int>& vec)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        vec: ::cpp_utils::Ref<crate::QVectorOfUint>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__474(
                self as *const crate::QDebug,
                vec.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVectorOfInt>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QVector<int>& vec)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, vec: ::cpp_utils::Ref<crate::QVectorOfInt>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__475(
                self as *const crate::QDebug,
                vec.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVectorOfQPointF>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QVector<QPointF>& vec)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        vec: ::cpp_utils::Ref<crate::QVectorOfQPointF>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__476(
                self as *const crate::QDebug,
                vec.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QVector<QPair<double, QVariant>>& vec)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        vec: ::cpp_utils::Ref<crate::QVectorOfQPairOfDoubleQVariant>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__479(
                self as *const crate::QDebug,
                vec.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QMapOfQStringQVariant>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QMap<QString, QVariant>& map)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        map: ::cpp_utils::Ref<crate::QMapOfQStringQVariant>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__484(
                self as *const crate::QDebug,
                map.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QMapOfIntQVariant>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QMap<int, QVariant>& map)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        map: ::cpp_utils::Ref<crate::QMapOfIntQVariant>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__485(
                self as *const crate::QDebug,
                map.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QHashOfQStringQVariant>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QHash<QString, QVariant>& hash)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        hash: ::cpp_utils::Ref<crate::QHashOfQStringQVariant>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__486(
                self as *const crate::QDebug,
                hash.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QHashOfIntQByteArray>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QHash<int, QByteArray>& hash)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        hash: ::cpp_utils::Ref<crate::QHashOfIntQByteArray>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__487(
                self as *const crate::QDebug,
                hash.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QPairOfQStringQJsonValue>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QPair<QString, QJsonValue>& pair)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        pair: ::cpp_utils::Ref<crate::QPairOfQStringQJsonValue>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__488(
                self as *const crate::QDebug,
                pair.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QPairOfDoubleQVariant>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QPair<double, QVariant>& pair)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        pair: ::cpp_utils::Ref<crate::QPairOfDoubleQVariant>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__489(
                self as *const crate::QDebug,
                pair.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QPairOfQStringQString>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QPair<QString, QString>& pair)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        pair: ::cpp_utils::Ref<crate::QPairOfQStringQString>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__490(
                self as *const crate::QDebug,
                pair.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<::cpp_utils::Ref<crate::QSetOfQAbstractState>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QSet<QAbstractState*>& set)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        set: ::cpp_utils::Ref<crate::QSetOfQAbstractState>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__491(
                self as *const crate::QDebug,
                set.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::MouseButton>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<Qt::MouseButton>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, flags: crate::QFlags<crate::MouseButton>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__492(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::Orientation>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<Qt::Orientation>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, flags: crate::QFlags<crate::Orientation>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__493(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::KeyboardModifier>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<Qt::KeyboardModifier>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::KeyboardModifier>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__494(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::WindowType>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<Qt::WindowType>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, flags: crate::QFlags<crate::WindowType>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__495(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::AlignmentFlag>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<Qt::AlignmentFlag>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, flags: crate::QFlags<crate::AlignmentFlag>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__496(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::Edge>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<Qt::Edge>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, flags: crate::QFlags<crate::Edge>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__497(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::ImageConversionFlag>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<Qt::ImageConversionFlag>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::ImageConversionFlag>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__498(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::DockWidgetArea>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<Qt::DockWidgetArea>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::DockWidgetArea>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__499(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::ToolBarArea>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<Qt::ToolBarArea>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, flags: crate::QFlags<crate::ToolBarArea>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__500(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::WindowState>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<Qt::WindowState>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, flags: crate::QFlags<crate::WindowState>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__501(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::ScreenOrientation>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<Qt::ScreenOrientation>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::ScreenOrientation>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__502(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::DropAction>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<Qt::DropAction>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, flags: crate::QFlags<crate::DropAction>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__503(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::ItemFlag>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<Qt::ItemFlag>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, flags: crate::QFlags<crate::ItemFlag>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__504(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::MatchFlag>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<Qt::MatchFlag>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, flags: crate::QFlags<crate::MatchFlag>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__505(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::TextInteractionFlag>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<Qt::TextInteractionFlag>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::TextInteractionFlag>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__506(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::InputMethodQuery>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<Qt::InputMethodQuery>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::InputMethodQuery>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__507(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::InputMethodHint>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<Qt::InputMethodHint>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::InputMethodHint>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__508(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::TouchPointState>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<Qt::TouchPointState>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::TouchPointState>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__509(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::MouseEventFlag>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<Qt::MouseEventFlag>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::MouseEventFlag>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__510(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::GestureFlag>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<Qt::GestureFlag>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, flags: crate::QFlags<crate::GestureFlag>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__511(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_byte_array::Base64Option>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<QByteArray::Base64Option>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::q_byte_array::Base64Option>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__513(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_string::SectionFlag>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<QString::SectionFlag>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::q_string::SectionFlag>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__514(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_meta_type::TypeFlag>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<QMetaType::TypeFlag>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::q_meta_type::TypeFlag>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__515(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::FindChildOption>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<Qt::FindChildOption>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::FindChildOption>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__516(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_event_loop::ProcessEventsFlag>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<QEventLoop::ProcessEventsFlag>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::q_event_loop::ProcessEventsFlag>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__517(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_io_device::OpenModeFlag>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<QIODevice::OpenModeFlag>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::q_io_device::OpenModeFlag>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__518(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_locale::NumberOption>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<QLocale::NumberOption>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::q_locale::NumberOption>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__519(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_command_line_option::Flag>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<QCommandLineOption::Flag>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::q_command_line_option::Flag>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__520(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_text_stream::NumberFlag>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<QTextStream::NumberFlag>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::q_text_stream::NumberFlag>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__521(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_file_device::Permission>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<QFileDevice::Permission>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::q_file_device::Permission>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__522(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_file_device::FileHandleFlag>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<QFileDevice::FileHandleFlag>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::q_file_device::FileHandleFlag>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__523(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_dir::SortFlag>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<QDir::SortFlag>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::q_dir::SortFlag>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__524(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_dir::Filter>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<QDir::Filter>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, flags: crate::QFlags<crate::q_dir::Filter>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__525(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_dir_iterator::IteratorFlag>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<QDirIterator::IteratorFlag>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::q_dir_iterator::IteratorFlag>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__526(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_item_selection_model::SelectionFlag>>
    for &crate::QDebug
{
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<QItemSelectionModel::SelectionFlag>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::q_item_selection_model::SelectionFlag>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__527(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_library::LoadHint>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<QLibrary::LoadHint>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::q_library::LoadHint>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__528(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_regular_expression::PatternOption>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<QRegularExpression::PatternOption>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::q_regular_expression::PatternOption>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__529(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_regular_expression::MatchOption>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<QRegularExpression::MatchOption>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::q_regular_expression::MatchOption>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__530(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_standard_paths::LocateOption>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<QStandardPaths::LocateOption>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::q_standard_paths::LocateOption>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__531(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_text_boundary_finder::BoundaryReason>>
    for &crate::QDebug
{
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<QTextBoundaryFinder::BoundaryReason>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::q_text_boundary_finder::BoundaryReason>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__532(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_text_codec::ConversionFlag>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<QTextCodec::ConversionFlag>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::q_text_codec::ConversionFlag>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__533(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_url::UserInputResolutionOption>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<QUrl::UserInputResolutionOption>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::q_url::UserInputResolutionOption>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__534(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::Shl<crate::QFlags<crate::q_url::ComponentFormattingOption>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<QUrl::ComponentFormattingOption>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::q_url::ComponentFormattingOption>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__535(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QFutureOf>> for crate::QFutureOf {
    /// <p>Returns <code>true</code> if <i>other</i> is a copy of this future; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QFuture<void>::operator==(const QFuture<void>& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfuture.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is a copy of this future; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QFutureOf>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QFuture_void_operator__(
                self as *const crate::QFutureOf,
                other.as_raw_ptr(),
            )
        }
    }
}

impl ::std::ops::BitAndAssign<::std::os::raw::c_int>
    for crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption
{
    /// Calls C++ function: <span style='color: green;'>```QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>& QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::operator&=(int mask)```</span>.
    fn bitand_assign(&mut self, mask: ::std::os::raw::c_int) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_operator__(self as *mut crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption, mask)
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::BitAndAssign<::std::os::raw::c_uint>
    for crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption
{
    /// Calls C++ function: <span style='color: green;'>```QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>& QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::operator&=(unsigned int mask)```</span>.
    fn bitand_assign(&mut self, mask: ::std::os::raw::c_uint) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_operator__1(self as *mut crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption, mask)
        };
        let _ = ffi_result;
    }
}

impl
    ::std::ops::BitOrAssign<
        ::cpp_utils::Ref<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>,
    > for crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption
{
    /// Calls C++ function: <span style='color: green;'>```QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>& QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::operator|=(QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption> f)```</span>.
    fn bitor_assign(
        &mut self,
        f: ::cpp_utils::Ref<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>,
    ) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_operator__2(self as *mut crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption, f.as_raw_ptr())
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::BitOrAssign<crate::q_url::UrlFormattingOption>
    for crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption
{
    /// Calls C++ function: <span style='color: green;'>```QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>& QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::operator|=(QUrl::UrlFormattingOption f)```</span>.
    fn bitor_assign(&mut self, f: crate::q_url::UrlFormattingOption) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_operator__3(self as *mut crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption, f)
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::BitOrAssign<crate::q_url::ComponentFormattingOption>
    for crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption
{
    /// Calls C++ function: <span style='color: green;'>```QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>& QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::operator|=(QUrl::ComponentFormattingOption f)```</span>.
    fn bitor_assign(&mut self, f: crate::q_url::ComponentFormattingOption) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_operator__4(self as *mut crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption, f)
        };
        let _ = ffi_result;
    }
}

impl
    ::std::ops::BitXorAssign<
        ::cpp_utils::Ref<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>,
    > for crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption
{
    /// Calls C++ function: <span style='color: green;'>```QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>& QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::operator^=(QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption> f)```</span>.
    fn bitxor_assign(
        &mut self,
        f: ::cpp_utils::Ref<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>,
    ) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_operator__5(self as *mut crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption, f.as_raw_ptr())
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::BitXorAssign<crate::q_url::UrlFormattingOption>
    for crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption
{
    /// Calls C++ function: <span style='color: green;'>```QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>& QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::operator^=(QUrl::UrlFormattingOption f)```</span>.
    fn bitxor_assign(&mut self, f: crate::q_url::UrlFormattingOption) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_operator__6(self as *mut crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption, f)
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::BitXorAssign<crate::q_url::ComponentFormattingOption>
    for crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption
{
    /// Calls C++ function: <span style='color: green;'>```QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>& QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::operator^=(QUrl::ComponentFormattingOption f)```</span>.
    fn bitxor_assign(&mut self, f: crate::q_url::ComponentFormattingOption) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_operator__7(self as *mut crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption, f)
        };
        let _ = ffi_result;
    }
}

impl ::std::ops::Not for &crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption {
    type Output = bool;
    /// Calls C++ function: <span style='color: green;'>```bool QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::operator!() const```</span>.
    fn not(self) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_operator_(self as *const crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption)
        }
    }
}

impl
    ::std::ops::BitOr<
        ::cpp_utils::Ref<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>,
    > for &crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption
{
    type Output =
        ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>;
    /// Calls C++ function: <span style='color: green;'>```QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption> QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::operator|(QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption> f) const```</span>.
    fn bitor(
        self,
        f: ::cpp_utils::Ref<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>,
    ) -> ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>
    {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_operator_1(self as *const crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption, f.as_raw_ptr())
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::BitOr<crate::q_url::UrlFormattingOption>
    for &crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption
{
    type Output =
        ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>;
    /// Calls C++ function: <span style='color: green;'>```QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption> QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::operator|(QUrl::UrlFormattingOption f) const```</span>.
    fn bitor(
        self,
        f: crate::q_url::UrlFormattingOption,
    ) -> ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>
    {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_operator_2(self as *const crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption, f)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::BitOr<crate::q_url::ComponentFormattingOption>
    for &crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption
{
    type Output =
        ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>;
    /// Calls C++ function: <span style='color: green;'>```QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption> QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::operator|(QUrl::ComponentFormattingOption f) const```</span>.
    fn bitor(
        self,
        f: crate::q_url::ComponentFormattingOption,
    ) -> ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>
    {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_operator_3(self as *const crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption, f)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl
    ::std::ops::BitXor<
        ::cpp_utils::Ref<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>,
    > for &crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption
{
    type Output =
        ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>;
    /// Calls C++ function: <span style='color: green;'>```QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption> QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::operator^(QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption> f) const```</span>.
    fn bitxor(
        self,
        f: ::cpp_utils::Ref<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>,
    ) -> ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>
    {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_operator_4(self as *const crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption, f.as_raw_ptr())
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::BitXor<crate::q_url::UrlFormattingOption>
    for &crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption
{
    type Output =
        ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>;
    /// Calls C++ function: <span style='color: green;'>```QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption> QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::operator^(QUrl::UrlFormattingOption f) const```</span>.
    fn bitxor(
        self,
        f: crate::q_url::UrlFormattingOption,
    ) -> ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>
    {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_operator_5(self as *const crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption, f)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::BitXor<crate::q_url::ComponentFormattingOption>
    for &crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption
{
    type Output =
        ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>;
    /// Calls C++ function: <span style='color: green;'>```QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption> QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::operator^(QUrl::ComponentFormattingOption f) const```</span>.
    fn bitxor(
        self,
        f: crate::q_url::ComponentFormattingOption,
    ) -> ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>
    {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_operator_6(self as *const crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption, f)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::BitAnd<::std::os::raw::c_int>
    for &crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption
{
    type Output =
        ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>;
    /// Calls C++ function: <span style='color: green;'>```QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption> QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::operator&(int mask) const```</span>.
    fn bitand(
        self,
        mask: ::std::os::raw::c_int,
    ) -> ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>
    {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_operator_7(self as *const crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption, mask)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::BitAnd<::std::os::raw::c_uint>
    for &crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption
{
    type Output =
        ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>;
    /// Calls C++ function: <span style='color: green;'>```QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption> QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::operator&(unsigned int mask) const```</span>.
    fn bitand(
        self,
        mask: ::std::os::raw::c_uint,
    ) -> ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>
    {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_operator_8(self as *const crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption, mask)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::BitAnd<crate::q_url::UrlFormattingOption>
    for &crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption
{
    type Output =
        ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>;
    /// Calls C++ function: <span style='color: green;'>```QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption> QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::operator&(QUrl::UrlFormattingOption f) const```</span>.
    fn bitand(
        self,
        f: crate::q_url::UrlFormattingOption,
    ) -> ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>
    {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_operator_9(self as *const crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption, f)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::std::ops::BitAnd<crate::q_url::ComponentFormattingOption>
    for &crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption
{
    type Output =
        ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>;
    /// Calls C++ function: <span style='color: green;'>```QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption> QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::operator&(QUrl::ComponentFormattingOption f) const```</span>.
    fn bitand(
        self,
        f: crate::q_url::ComponentFormattingOption,
    ) -> ::cpp_utils::CppBox<crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption>
    {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_operator_10(self as *const crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption, f)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

impl ::cpp_utils::CppDeletable for crate::QPairOfQStringQJsonValue {
    /// <p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QPair<QString, QJsonValue>::~QPair()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; can be used in your application if the STL <code>pair</code> type is not available. It stores one value of type T1 and one value of type T2. It can be used as a return value for a function that needs to return two values, or as the value type of a <a href="http://doc.qt.io/qt-5/containers.html">generic container</a>.</p>
    /// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> that stores one <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and one <code>double</code> value:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpair.html#QPair">QPair</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> pair;
    ///
    /// </pre>
    /// <p>The components are accessible as public data members called <a href="http://doc.qt.io/qt-5/qpair.html#first-var">first</a> and <a href="http://doc.qt.io/qt-5/qpair.html#second-var">second</a>. For example:</p>
    /// <pre class="cpp">
    ///
    ///   pair<span class="operator">.</span>first <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   pair<span class="operator">.</span>second <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>Note, however, that it is almost always preferable to define a small struct to hold the result of a function with multiple return values. A struct trivially generalizes to more than two values, and allows more descriptive member names than <code>first</code> and <code>second</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">struct</span> Variable {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> name;
    /// &#32;     <span class="type">double</span> value;
    ///   };
    ///   Variable v;
    ///   v<span class="operator">.</span>name <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   v<span class="operator">.</span>value <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>The advent of C++11 automatic variable type deduction (<code>auto</code>) shifts the emphasis from the type name to the name of functions and members. Thus, <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>, like <code>std::pair</code> and <code>std::tuple</code>, is mostly useful in generic (template) code, where defining a dedicated type is not possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>'s template data types (T1 and T2) must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; these requirements are documented on a per-function basis.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QPair_QString_QJsonValue_dQPair(
            self as *mut crate::QPairOfQStringQJsonValue,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QPairOfDoubleQVariant {
    /// <p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QPair<double, QVariant>::~QPair()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; can be used in your application if the STL <code>pair</code> type is not available. It stores one value of type T1 and one value of type T2. It can be used as a return value for a function that needs to return two values, or as the value type of a <a href="http://doc.qt.io/qt-5/containers.html">generic container</a>.</p>
    /// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> that stores one <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and one <code>double</code> value:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpair.html#QPair">QPair</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> pair;
    ///
    /// </pre>
    /// <p>The components are accessible as public data members called <a href="http://doc.qt.io/qt-5/qpair.html#first-var">first</a> and <a href="http://doc.qt.io/qt-5/qpair.html#second-var">second</a>. For example:</p>
    /// <pre class="cpp">
    ///
    ///   pair<span class="operator">.</span>first <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   pair<span class="operator">.</span>second <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>Note, however, that it is almost always preferable to define a small struct to hold the result of a function with multiple return values. A struct trivially generalizes to more than two values, and allows more descriptive member names than <code>first</code> and <code>second</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">struct</span> Variable {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> name;
    /// &#32;     <span class="type">double</span> value;
    ///   };
    ///   Variable v;
    ///   v<span class="operator">.</span>name <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   v<span class="operator">.</span>value <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>The advent of C++11 automatic variable type deduction (<code>auto</code>) shifts the emphasis from the type name to the name of functions and members. Thus, <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>, like <code>std::pair</code> and <code>std::tuple</code>, is mostly useful in generic (template) code, where defining a dedicated type is not possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>'s template data types (T1 and T2) must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; these requirements are documented on a per-function basis.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QPair_double_QVariant_dQPair(
            self as *mut crate::QPairOfDoubleQVariant,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QPairOfQStringQString {
    /// <p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QPair<QString, QString>::~QPair()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; can be used in your application if the STL <code>pair</code> type is not available. It stores one value of type T1 and one value of type T2. It can be used as a return value for a function that needs to return two values, or as the value type of a <a href="http://doc.qt.io/qt-5/containers.html">generic container</a>.</p>
    /// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> that stores one <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and one <code>double</code> value:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpair.html#QPair">QPair</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> pair;
    ///
    /// </pre>
    /// <p>The components are accessible as public data members called <a href="http://doc.qt.io/qt-5/qpair.html#first-var">first</a> and <a href="http://doc.qt.io/qt-5/qpair.html#second-var">second</a>. For example:</p>
    /// <pre class="cpp">
    ///
    ///   pair<span class="operator">.</span>first <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   pair<span class="operator">.</span>second <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>Note, however, that it is almost always preferable to define a small struct to hold the result of a function with multiple return values. A struct trivially generalizes to more than two values, and allows more descriptive member names than <code>first</code> and <code>second</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">struct</span> Variable {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> name;
    /// &#32;     <span class="type">double</span> value;
    ///   };
    ///   Variable v;
    ///   v<span class="operator">.</span>name <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   v<span class="operator">.</span>value <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>The advent of C++11 automatic variable type deduction (<code>auto</code>) shifts the emphasis from the type name to the name of functions and members. Thus, <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>, like <code>std::pair</code> and <code>std::tuple</code>, is mostly useful in generic (template) code, where defining a dedicated type is not possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>'s template data types (T1 and T2) must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; these requirements are documented on a per-function basis.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QPair_QString_QString_dQPair(
            self as *mut crate::QPairOfQStringQString,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QSetOfQAbstractState {
    /// <p>The <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> class is a template class that provides a hash-table-based set.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QSet<QAbstractState*>::~QSet()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qset.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> class is a template class that provides a hash-table-based set.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qset.html">QSet</a>&lt;T&gt; is one of Qt's generic <a href="http://doc.qt.io/qt-5/containers.html">container classes</a>. It stores values in an unspecified order and provides very fast lookup of the values. Internally, <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>&lt;T&gt; is implemented as a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>.</p>
    /// <p>Here's an example <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> with <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> values:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> set;
    ///
    /// </pre>
    /// <p>To insert a value into the set, use <a href="http://doc.qt.io/qt-5/qset.html#insert">insert</a>():</p>
    /// <pre class="cpp">
    ///
    ///   set<span class="operator">.</span>insert(<span class="string">"one"</span>);
    ///   set<span class="operator">.</span>insert(<span class="string">"three"</span>);
    ///   set<span class="operator">.</span>insert(<span class="string">"seven"</span>);
    ///
    /// </pre>
    /// <p>Another way to insert items into the set is to use operator&lt;&lt;():</p>
    /// <pre class="cpp">
    ///
    ///   set <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"twelve"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"fifteen"</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="string">"nineteen"</span>;
    ///
    /// </pre>
    /// <p>To test whether an item belongs to the set or not, use <a href="http://doc.qt.io/qt-5/qset.html#contains">contains</a>():</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">if</span> (<span class="operator">!</span>set<span class="operator">.</span>contains(<span class="string">"ninety-nine"</span>))
    /// &#32;     <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///
    /// </pre>
    /// <p>If you want to navigate through all the values stored in a <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>, you can use an iterator. <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> supports both <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterators</a> (<a href="http://doc.qt.io/qt-5/qsetiterator.html">QSetIterator</a> and <a href="http://doc.qt.io/qt-5/qmutablesetiterator.html">QMutableSetIterator</a>) and <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterators</a> (<a href="http://doc.qt.io/qt-5/qset-iterator.html">QSet::iterator</a> and <a href="http://doc.qt.io/qt-5/qset-const-iterator.html">QSet::const_iterator</a>). Here's how to iterate over a <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>&lt;<a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *&gt; using a Java-style iterator:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qsetiterator.html">QSetIterator</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span> i(set);
    ///   <span class="keyword">while</span> (i<span class="operator">.</span>hasNext())
    /// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> i<span class="operator">.</span>next();
    ///
    /// </pre>
    /// <p>Here's the same code, but using an STL-style iterator:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a></span> <span class="operator">*</span><span class="operator">&gt;</span><span class="operator">::</span>const_iterator i <span class="operator">=</span> set<span class="operator">.</span>constBegin();
    ///   <span class="keyword">while</span> (i <span class="operator">!</span><span class="operator">=</span> set<span class="operator">.</span>constEnd()) {
    /// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="operator">*</span>i;
    /// &#32;     <span class="operator">+</span><span class="operator">+</span>i;
    ///   }
    ///
    /// </pre>
    /// <p><a href="http://doc.qt.io/qt-5/qset.html">QSet</a> is unordered, so an iterator's sequence cannot be assumed to be predictable. If ordering by key is required, use a <a href="http://doc.qt.io/qt-5/qmap.html">QMap</a>.</p>
    /// <p>To navigate through a <a href="http://doc.qt.io/qt-5/qset.html">QSet</a>, you can also use <a href="http://doc.qt.io/qt-5/containers.html#foreach">foreach</a>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qset.html#QSet">QSet</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">&gt;</span> set;
    ///   <span class="operator">.</span><span class="operator">.</span><span class="operator">.</span>
    ///   foreach (<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> <span class="operator">&amp;</span>value<span class="operator">,</span> set)
    /// &#32;     <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>() <span class="operator">&lt;</span><span class="operator">&lt;</span> value;
    ///
    /// </pre>
    /// <p>Items can be removed from the set using <a href="http://doc.qt.io/qt-5/qset.html#remove">remove</a>(). There is also a <a href="http://doc.qt.io/qt-5/qset.html#clear">clear</a>() function that removes all items.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qset.html">QSet</a>'s value data type must be an <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-type">assignable data type</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. In addition, the type must provide <code>operator==()</code>, and there must also be a global <a href="http://doc.qt.io/qt-5/qhash.html#qHash">qHash</a>() function that returns a hash value for an argument of the key's type. See the <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> documentation for a list of types supported by <a href="http://doc.qt.io/qt-5/qhash.html#qHash">qHash</a>().</p>
    /// <p>Internally, <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> uses a hash table to perform lookups. The hash table automatically grows and shrinks to provide fast lookups without wasting memory. You can still control the size of the hash table by calling <a href="http://doc.qt.io/qt-5/qset.html#reserve">reserve</a>(), if you already know approximately how many elements the <a href="http://doc.qt.io/qt-5/qset.html">QSet</a> will contain, but this isn't necessary to obtain good performance. You can also call <a href="http://doc.qt.io/qt-5/qset.html#capacity">capacity</a>() to retrieve the hash table's size.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSet_QAbstractState_ptr_dQSet(
            self as *mut crate::QSetOfQAbstractState,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::QFutureOf {
    /// <p>The <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> class represents the result of an asynchronous computation.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QFuture<void>::~QFuture()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfuture.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> class represents the result of an asynchronous computation.</p>
    /// <p>To start a computation, use one of the APIs in the <a href="http://doc.qt.io/qt-5/qtconcurrent-index.html">Qt Concurrent</a> framework.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> allows threads to be synchronized against one or more results which will be ready at a later point in time. The result can be of any type that has a default constructor and a copy constructor. If a result is not available at the time of calling the <a href="http://doc.qt.io/qt-5/qfuture.html#result">result</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#resultAt">resultAt</a>(), or <a href="http://doc.qt.io/qt-5/qfuture.html#results">results</a>() functions, <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> will wait until the result becomes available. You can use the <a href="http://doc.qt.io/qt-5/qfuture.html#isResultReadyAt">isResultReadyAt</a>() function to determine if a result is ready or not. For <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> objects that report more than one result, the <a href="http://doc.qt.io/qt-5/qfuture.html#resultCount">resultCount</a>() function returns the number of continuous results. This means that it is always safe to iterate through the results from 0 to <a href="http://doc.qt.io/qt-5/qfuture.html#resultCount">resultCount</a>().</p>
    /// <p><a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> provides a <a href="http://doc.qt.io/qt-5/containers.html#java-style-iterators">Java-style iterator</a> (<a href="http://doc.qt.io/qt-5/qfutureiterator.html">QFutureIterator</a>) and an <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> (<a href="http://doc.qt.io/qt-5/qfuture-const-iterator.html">QFuture::const_iterator</a>). Using these iterators is another way to access results in the future.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> also offers ways to interact with a runnning computation. For instance, the computation can be canceled with the <a href="http://doc.qt.io/qt-5/qfuture.html#cancel">cancel</a>() function. To pause the computation, use the <a href="http://doc.qt.io/qt-5/qfuture.html#setPaused">setPaused</a>() function or one of the <a href="http://doc.qt.io/qt-5/qfuture.html#pause">pause</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#resume">resume</a>(), or <a href="http://doc.qt.io/qt-5/qfuture.html#togglePaused">togglePaused</a>() convenience functions. Be aware that not all asynchronous computations can be canceled or paused. For example, the future returned by <a href="http://doc.qt.io/qt-5/qtconcurrent.html#run">QtConcurrent::run</a>() cannot be canceled; but the future returned by <a href="http://doc.qt.io/qt-5/qtconcurrent.html#mappedReduced-1">QtConcurrent::mappedReduced</a>() can.</p>
    /// <p>Progress information is provided by the <a href="http://doc.qt.io/qt-5/qfuture.html#progressValue">progressValue</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#progressMinimum">progressMinimum</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#progressMaximum">progressMaximum</a>(), and <a href="http://doc.qt.io/qt-5/qfuture.html#progressText">progressText</a>() functions. The <a href="http://doc.qt.io/qt-5/qfuture.html#waitForFinished">waitForFinished</a>() function causes the calling thread to block and wait for the computation to finish, ensuring that all results are available.</p>
    /// <p>The state of the computation represented by a <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> can be queried using the <a href="http://doc.qt.io/qt-5/qfuture.html#isCanceled">isCanceled</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#isStarted">isStarted</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#isFinished">isFinished</a>(), <a href="http://doc.qt.io/qt-5/qfuture.html#isRunning">isRunning</a>(), or <a href="http://doc.qt.io/qt-5/qfuture.html#isPaused">isPaused</a>() functions.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a> is a lightweight reference counted class that can be passed by value.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a>&lt;void&gt; is specialized to not contain any of the result fetching functions. Any <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a>&lt;T&gt; can be assigned or copied into a <a href="http://doc.qt.io/qt-5/qfuture.html">QFuture</a>&lt;void&gt; as well. This is useful if only status or progress information is needed - not the actual result data.</p>
    /// <p>To interact with running tasks using signals and slots, use <a href="http://doc.qt.io/qt-5/qfuturewatcher.html">QFutureWatcher</a>.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QFuture_void_dQFuture(self as *mut crate::QFutureOf)
    }
}

impl ::cpp_utils::CppDeletable
    for crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption
{
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QUrlTwoFlags<QUrl::UrlFormattingOption, QUrl::ComponentFormattingOption>::~QUrlTwoFlags()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QUrlTwoFlags_QUrl_UrlFormattingOption_QUrl_ComponentFormattingOption_dQUrlTwoFlags(self as *mut crate::QUrlTwoFlagsOfUrlFormattingOptionComponentFormattingOption)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QStringList> for crate::QListOfQString {
    /// Calls C++ function: <span style='color: green;'>```QStringList* static_cast<QStringList*>(QList<QString>* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QListOfQString>,
    ) -> ::cpp_utils::Ptr<crate::QStringList> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QStringList_ptr(
            ptr.as_raw_ptr() as *mut crate::QListOfQString,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QStringList* static_cast<QStringList*>(QList<QString>* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QListOfQString>,
    ) -> ::cpp_utils::MutPtr<crate::QStringList> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QStringList_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QListOfQString> for crate::QStringList {
    /// Calls C++ function: <span style='color: green;'>```QList<QString>* static_cast<QList<QString>*>(QStringList* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QStringList>,
    ) -> ::cpp_utils::Ptr<crate::QListOfQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QList_QString_ptr(
            ptr.as_raw_ptr() as *mut crate::QStringList,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QList<QString>* static_cast<QList<QString>*>(QStringList* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QStringList>,
    ) -> ::cpp_utils::MutPtr<crate::QListOfQString> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QList_QString_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QStringList {
    type Target = crate::QListOfQString;
    /// Calls C++ function: <span style='color: green;'>```QList<QString>* static_cast<QList<QString>*>(QStringList* ptr)```</span>.
    fn deref(&self) -> &crate::QListOfQString {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QList_QString_ptr(
                self as *const crate::QStringList as *mut crate::QStringList,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QStringList {
    /// Calls C++ function: <span style='color: green;'>```QList<QString>* static_cast<QList<QString>*>(QStringList* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QListOfQString {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QList_QString_ptr(
                self as *mut crate::QStringList,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::StaticDowncast<crate::QAbstractAnimation> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* static_cast<QAbstractAnimation*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractAnimation_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* static_cast<QAbstractAnimation*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractAnimation> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractAnimation_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QAbstractAnimation {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAbstractAnimation* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractAnimation>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractAnimation
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAbstractAnimation* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractAnimation>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QAbstractAnimation {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAbstractAnimation* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr(
                self as *const crate::QAbstractAnimation as *mut crate::QAbstractAnimation,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QAbstractAnimation {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAbstractAnimation* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr(
                self as *mut crate::QAbstractAnimation,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QAbstractAnimation> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* dynamic_cast<QAbstractAnimation*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAbstractAnimation_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* dynamic_cast<QAbstractAnimation*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractAnimation> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAbstractAnimation_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QAnimationDriver> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QAnimationDriver* static_cast<QAnimationDriver*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QAnimationDriver> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAnimationDriver_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAnimationDriver* static_cast<QAnimationDriver*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QAnimationDriver> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAnimationDriver_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QAnimationDriver {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAnimationDriver* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QAnimationDriver>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr1(
            ptr.as_raw_ptr() as *mut crate::QAnimationDriver
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAnimationDriver* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAnimationDriver>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QAnimationDriver {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAnimationDriver* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr1(
                self as *const crate::QAnimationDriver as *mut crate::QAnimationDriver,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QAnimationDriver {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAnimationDriver* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr1(
                self as *mut crate::QAnimationDriver,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QAnimationDriver> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QAnimationDriver* dynamic_cast<QAnimationDriver*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QAnimationDriver> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAnimationDriver_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAnimationDriver* dynamic_cast<QAnimationDriver*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QAnimationDriver> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAnimationDriver_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QEventLoop> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QEventLoop* static_cast<QEventLoop*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QEventLoop> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QEventLoop_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QEventLoop* static_cast<QEventLoop*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QEventLoop> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QEventLoop_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QEventLoop {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QEventLoop* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QEventLoop>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr2(
            ptr.as_raw_ptr() as *mut crate::QEventLoop
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QEventLoop* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QEventLoop>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr2(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QEventLoop {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QEventLoop* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr2(
                self as *const crate::QEventLoop as *mut crate::QEventLoop,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QEventLoop {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QEventLoop* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr2(self as *mut crate::QEventLoop)
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QEventLoop> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QEventLoop* dynamic_cast<QEventLoop*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QEventLoop> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QEventLoop_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QEventLoop* dynamic_cast<QEventLoop*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QEventLoop> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QEventLoop_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QAbstractEventDispatcher> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QAbstractEventDispatcher* static_cast<QAbstractEventDispatcher*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractEventDispatcher> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractEventDispatcher_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractEventDispatcher* static_cast<QAbstractEventDispatcher*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractEventDispatcher> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractEventDispatcher_ptr(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QAbstractEventDispatcher {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAbstractEventDispatcher* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractEventDispatcher>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr3(
            ptr.as_raw_ptr() as *mut crate::QAbstractEventDispatcher
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAbstractEventDispatcher* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractEventDispatcher>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr3(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QAbstractEventDispatcher {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAbstractEventDispatcher* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr3(
                self as *const crate::QAbstractEventDispatcher
                    as *mut crate::QAbstractEventDispatcher,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QAbstractEventDispatcher {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAbstractEventDispatcher* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr3(
                self as *mut crate::QAbstractEventDispatcher,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QAbstractEventDispatcher> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QAbstractEventDispatcher* dynamic_cast<QAbstractEventDispatcher*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractEventDispatcher> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAbstractEventDispatcher_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractEventDispatcher* dynamic_cast<QAbstractEventDispatcher*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractEventDispatcher> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAbstractEventDispatcher_ptr(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QAbstractItemModel> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* static_cast<QAbstractItemModel*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractItemModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractItemModel_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* static_cast<QAbstractItemModel*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractItemModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractItemModel_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QAbstractItemModel {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr4(
            ptr.as_raw_ptr() as *mut crate::QAbstractItemModel
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr4(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QAbstractItemModel {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAbstractItemModel* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr4(
                self as *const crate::QAbstractItemModel as *mut crate::QAbstractItemModel,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QAbstractItemModel {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAbstractItemModel* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr4(
                self as *mut crate::QAbstractItemModel,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QAbstractItemModel> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* dynamic_cast<QAbstractItemModel*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractItemModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAbstractItemModel_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* dynamic_cast<QAbstractItemModel*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractItemModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAbstractItemModel_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QAbstractTableModel> for crate::QAbstractItemModel {
    /// Calls C++ function: <span style='color: green;'>```QAbstractTableModel* static_cast<QAbstractTableModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractTableModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractTableModel_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractItemModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractTableModel* static_cast<QAbstractTableModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractTableModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractTableModel_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QAbstractItemModel> for crate::QAbstractTableModel {
    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* static_cast<QAbstractItemModel*>(QAbstractTableModel* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractTableModel>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractItemModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractItemModel_ptr1(
            ptr.as_raw_ptr() as *mut crate::QAbstractTableModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* static_cast<QAbstractItemModel*>(QAbstractTableModel* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractTableModel>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractItemModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractItemModel_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QAbstractTableModel {
    type Target = crate::QAbstractItemModel;
    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* static_cast<QAbstractItemModel*>(QAbstractTableModel* ptr)```</span>.
    fn deref(&self) -> &crate::QAbstractItemModel {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractItemModel_ptr1(
                self as *const crate::QAbstractTableModel as *mut crate::QAbstractTableModel,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QAbstractTableModel {
    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* static_cast<QAbstractItemModel*>(QAbstractTableModel* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QAbstractItemModel {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractItemModel_ptr1(
                self as *mut crate::QAbstractTableModel,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QAbstractTableModel> for crate::QAbstractItemModel {
    /// Calls C++ function: <span style='color: green;'>```QAbstractTableModel* dynamic_cast<QAbstractTableModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractTableModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAbstractTableModel_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractItemModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractTableModel* dynamic_cast<QAbstractTableModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractTableModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAbstractTableModel_ptr(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QAbstractTableModel> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QAbstractTableModel* static_cast<QAbstractTableModel*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractTableModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractTableModel_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractTableModel* static_cast<QAbstractTableModel*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractTableModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractTableModel_ptr1(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QAbstractTableModel {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAbstractTableModel* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractTableModel>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr5(
            ptr.as_raw_ptr() as *mut crate::QAbstractTableModel
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAbstractTableModel* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractTableModel>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr5(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QAbstractTableModel> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QAbstractTableModel* dynamic_cast<QAbstractTableModel*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractTableModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAbstractTableModel_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractTableModel* dynamic_cast<QAbstractTableModel*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractTableModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAbstractTableModel_ptr1(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QAbstractListModel> for crate::QAbstractItemModel {
    /// Calls C++ function: <span style='color: green;'>```QAbstractListModel* static_cast<QAbstractListModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractListModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractListModel_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractItemModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractListModel* static_cast<QAbstractListModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractListModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractListModel_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QAbstractItemModel> for crate::QAbstractListModel {
    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* static_cast<QAbstractItemModel*>(QAbstractListModel* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractListModel>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractItemModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractItemModel_ptr2(
            ptr.as_raw_ptr() as *mut crate::QAbstractListModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* static_cast<QAbstractItemModel*>(QAbstractListModel* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractListModel>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractItemModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractItemModel_ptr2(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QAbstractListModel {
    type Target = crate::QAbstractItemModel;
    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* static_cast<QAbstractItemModel*>(QAbstractListModel* ptr)```</span>.
    fn deref(&self) -> &crate::QAbstractItemModel {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractItemModel_ptr2(
                self as *const crate::QAbstractListModel as *mut crate::QAbstractListModel,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QAbstractListModel {
    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* static_cast<QAbstractItemModel*>(QAbstractListModel* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QAbstractItemModel {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractItemModel_ptr2(
                self as *mut crate::QAbstractListModel,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QAbstractListModel> for crate::QAbstractItemModel {
    /// Calls C++ function: <span style='color: green;'>```QAbstractListModel* dynamic_cast<QAbstractListModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractListModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAbstractListModel_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractItemModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractListModel* dynamic_cast<QAbstractListModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractListModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAbstractListModel_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QAbstractListModel> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QAbstractListModel* static_cast<QAbstractListModel*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractListModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractListModel_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractListModel* static_cast<QAbstractListModel*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractListModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractListModel_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QAbstractListModel {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAbstractListModel* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractListModel>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr6(
            ptr.as_raw_ptr() as *mut crate::QAbstractListModel
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAbstractListModel* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractListModel>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr6(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QAbstractListModel> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QAbstractListModel* dynamic_cast<QAbstractListModel*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractListModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAbstractListModel_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractListModel* dynamic_cast<QAbstractListModel*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractListModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAbstractListModel_ptr1(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QAbstractProxyModel> for crate::QAbstractItemModel {
    /// Calls C++ function: <span style='color: green;'>```QAbstractProxyModel* static_cast<QAbstractProxyModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractProxyModel_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractItemModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractProxyModel* static_cast<QAbstractProxyModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractProxyModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractProxyModel_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QAbstractItemModel> for crate::QAbstractProxyModel {
    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* static_cast<QAbstractItemModel*>(QAbstractProxyModel* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractProxyModel>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractItemModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractItemModel_ptr3(
            ptr.as_raw_ptr() as *mut crate::QAbstractProxyModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* static_cast<QAbstractItemModel*>(QAbstractProxyModel* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractProxyModel>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractItemModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractItemModel_ptr3(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QAbstractProxyModel {
    type Target = crate::QAbstractItemModel;
    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* static_cast<QAbstractItemModel*>(QAbstractProxyModel* ptr)```</span>.
    fn deref(&self) -> &crate::QAbstractItemModel {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractItemModel_ptr3(
                self as *const crate::QAbstractProxyModel as *mut crate::QAbstractProxyModel,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QAbstractProxyModel {
    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* static_cast<QAbstractItemModel*>(QAbstractProxyModel* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QAbstractItemModel {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractItemModel_ptr3(
                self as *mut crate::QAbstractProxyModel,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QAbstractProxyModel> for crate::QAbstractItemModel {
    /// Calls C++ function: <span style='color: green;'>```QAbstractProxyModel* dynamic_cast<QAbstractProxyModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAbstractProxyModel_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractItemModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractProxyModel* dynamic_cast<QAbstractProxyModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAbstractProxyModel_ptr(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QAbstractProxyModel> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QAbstractProxyModel* static_cast<QAbstractProxyModel*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractProxyModel_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractProxyModel* static_cast<QAbstractProxyModel*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractProxyModel_ptr1(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QAbstractProxyModel {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAbstractProxyModel* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractProxyModel>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr7(
            ptr.as_raw_ptr() as *mut crate::QAbstractProxyModel
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAbstractProxyModel* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractProxyModel>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr7(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QAbstractProxyModel> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QAbstractProxyModel* dynamic_cast<QAbstractProxyModel*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAbstractProxyModel_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractProxyModel* dynamic_cast<QAbstractProxyModel*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAbstractProxyModel_ptr1(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QAbstractState> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QAbstractState* static_cast<QAbstractState*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractState_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractState* static_cast<QAbstractState*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractState> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractState_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QAbstractState {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAbstractState* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractState>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr8(
            ptr.as_raw_ptr() as *mut crate::QAbstractState
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAbstractState* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractState>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr8(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QAbstractState {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAbstractState* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr8(
                self as *const crate::QAbstractState as *mut crate::QAbstractState,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QAbstractState {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAbstractState* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr8(
                self as *mut crate::QAbstractState,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QAbstractState> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QAbstractState* dynamic_cast<QAbstractState*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAbstractState_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractState* dynamic_cast<QAbstractState*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractState> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAbstractState_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QAbstractTransition> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QAbstractTransition* static_cast<QAbstractTransition*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractTransition_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractTransition* static_cast<QAbstractTransition*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractTransition> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractTransition_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QAbstractTransition {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAbstractTransition* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractTransition>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr9(
            ptr.as_raw_ptr() as *mut crate::QAbstractTransition
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAbstractTransition* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractTransition>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr9(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QAbstractTransition {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAbstractTransition* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr9(
                self as *const crate::QAbstractTransition as *mut crate::QAbstractTransition,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QAbstractTransition {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAbstractTransition* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr9(
                self as *mut crate::QAbstractTransition,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QAbstractTransition> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QAbstractTransition* dynamic_cast<QAbstractTransition*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAbstractTransition_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractTransition* dynamic_cast<QAbstractTransition*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAbstractTransition_ptr(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QAnimationGroup> for crate::QAbstractAnimation {
    /// Calls C++ function: <span style='color: green;'>```QAnimationGroup* static_cast<QAnimationGroup*>(QAbstractAnimation* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractAnimation>,
    ) -> ::cpp_utils::Ptr<crate::QAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAnimationGroup_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractAnimation,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAnimationGroup* static_cast<QAnimationGroup*>(QAbstractAnimation* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractAnimation>,
    ) -> ::cpp_utils::MutPtr<crate::QAnimationGroup> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAnimationGroup_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QAbstractAnimation> for crate::QAnimationGroup {
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* static_cast<QAbstractAnimation*>(QAnimationGroup* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QAnimationGroup>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractAnimation_ptr1(
            ptr.as_raw_ptr() as *mut crate::QAnimationGroup,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* static_cast<QAbstractAnimation*>(QAnimationGroup* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAnimationGroup>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractAnimation> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractAnimation_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QAnimationGroup {
    type Target = crate::QAbstractAnimation;
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* static_cast<QAbstractAnimation*>(QAnimationGroup* ptr)```</span>.
    fn deref(&self) -> &crate::QAbstractAnimation {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractAnimation_ptr1(
                self as *const crate::QAnimationGroup as *mut crate::QAnimationGroup,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QAnimationGroup {
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* static_cast<QAbstractAnimation*>(QAnimationGroup* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QAbstractAnimation {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractAnimation_ptr1(
                self as *mut crate::QAnimationGroup,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QAnimationGroup> for crate::QAbstractAnimation {
    /// Calls C++ function: <span style='color: green;'>```QAnimationGroup* dynamic_cast<QAnimationGroup*>(QAbstractAnimation* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractAnimation>,
    ) -> ::cpp_utils::Ptr<crate::QAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAnimationGroup_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractAnimation,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAnimationGroup* dynamic_cast<QAnimationGroup*>(QAbstractAnimation* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractAnimation>,
    ) -> ::cpp_utils::MutPtr<crate::QAnimationGroup> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAnimationGroup_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QAnimationGroup> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QAnimationGroup* static_cast<QAnimationGroup*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAnimationGroup_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAnimationGroup* static_cast<QAnimationGroup*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QAnimationGroup> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAnimationGroup_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QAnimationGroup {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAnimationGroup* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QAnimationGroup>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr10(
            ptr.as_raw_ptr() as *mut crate::QAnimationGroup
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QAnimationGroup* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAnimationGroup>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr10(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QAnimationGroup> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QAnimationGroup* dynamic_cast<QAnimationGroup*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAnimationGroup_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAnimationGroup* dynamic_cast<QAnimationGroup*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QAnimationGroup> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QAnimationGroup_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QIODevice> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QIODevice* static_cast<QIODevice*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QIODevice> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QIODevice_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QIODevice* static_cast<QIODevice*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QIODevice> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QIODevice_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QIODevice {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QIODevice* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QIODevice>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr11(
            ptr.as_raw_ptr() as *mut crate::QIODevice
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QIODevice* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QIODevice>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr11(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QIODevice {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QIODevice* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr11(
                self as *const crate::QIODevice as *mut crate::QIODevice,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QIODevice {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QIODevice* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr11(self as *mut crate::QIODevice)
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QIODevice> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QIODevice* dynamic_cast<QIODevice*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QIODevice> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QIODevice_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QIODevice* dynamic_cast<QIODevice*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QIODevice> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QIODevice_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QBuffer> for crate::QIODevice {
    /// Calls C++ function: <span style='color: green;'>```QBuffer* static_cast<QBuffer*>(QIODevice* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QIODevice>,
    ) -> ::cpp_utils::Ptr<crate::QBuffer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QBuffer_ptr(
            ptr.as_raw_ptr() as *mut crate::QIODevice
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QBuffer* static_cast<QBuffer*>(QIODevice* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QIODevice>,
    ) -> ::cpp_utils::MutPtr<crate::QBuffer> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QBuffer_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QIODevice> for crate::QBuffer {
    /// Calls C++ function: <span style='color: green;'>```QIODevice* static_cast<QIODevice*>(QBuffer* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QBuffer>,
    ) -> ::cpp_utils::Ptr<crate::QIODevice> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QIODevice_ptr1(
            ptr.as_raw_ptr() as *mut crate::QBuffer
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QIODevice* static_cast<QIODevice*>(QBuffer* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QBuffer>,
    ) -> ::cpp_utils::MutPtr<crate::QIODevice> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QIODevice_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QBuffer {
    type Target = crate::QIODevice;
    /// Calls C++ function: <span style='color: green;'>```QIODevice* static_cast<QIODevice*>(QBuffer* ptr)```</span>.
    fn deref(&self) -> &crate::QIODevice {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QIODevice_ptr1(
                self as *const crate::QBuffer as *mut crate::QBuffer,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QBuffer {
    /// Calls C++ function: <span style='color: green;'>```QIODevice* static_cast<QIODevice*>(QBuffer* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QIODevice {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QIODevice_ptr1(self as *mut crate::QBuffer)
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QBuffer> for crate::QIODevice {
    /// Calls C++ function: <span style='color: green;'>```QBuffer* dynamic_cast<QBuffer*>(QIODevice* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QIODevice>,
    ) -> ::cpp_utils::Ptr<crate::QBuffer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QBuffer_ptr(
            ptr.as_raw_ptr() as *mut crate::QIODevice
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QBuffer* dynamic_cast<QBuffer*>(QIODevice* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QIODevice>,
    ) -> ::cpp_utils::MutPtr<crate::QBuffer> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QBuffer_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QBuffer> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QBuffer* static_cast<QBuffer*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QBuffer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QBuffer_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QBuffer* static_cast<QBuffer*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QBuffer> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QBuffer_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QBuffer {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QBuffer* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QBuffer>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr12(
            ptr.as_raw_ptr() as *mut crate::QBuffer
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QBuffer* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QBuffer>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr12(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QBuffer> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QBuffer* dynamic_cast<QBuffer*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QBuffer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QBuffer_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QBuffer* dynamic_cast<QBuffer*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QBuffer> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QBuffer_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QTimerEvent> for crate::QEvent {
    /// Calls C++ function: <span style='color: green;'>```QTimerEvent* static_cast<QTimerEvent*>(QEvent* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QEvent>,
    ) -> ::cpp_utils::Ptr<crate::QTimerEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QTimerEvent_ptr(
            ptr.as_raw_ptr() as *mut crate::QEvent,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QTimerEvent* static_cast<QTimerEvent*>(QEvent* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QEvent>,
    ) -> ::cpp_utils::MutPtr<crate::QTimerEvent> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QTimerEvent_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QEvent> for crate::QTimerEvent {
    /// Calls C++ function: <span style='color: green;'>```QEvent* static_cast<QEvent*>(QTimerEvent* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QTimerEvent>,
    ) -> ::cpp_utils::Ptr<crate::QEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QEvent_ptr(
            ptr.as_raw_ptr() as *mut crate::QTimerEvent
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QEvent* static_cast<QEvent*>(QTimerEvent* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QTimerEvent>,
    ) -> ::cpp_utils::MutPtr<crate::QEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QEvent_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QTimerEvent {
    type Target = crate::QEvent;
    /// Calls C++ function: <span style='color: green;'>```QEvent* static_cast<QEvent*>(QTimerEvent* ptr)```</span>.
    fn deref(&self) -> &crate::QEvent {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QEvent_ptr(
                self as *const crate::QTimerEvent as *mut crate::QTimerEvent,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QTimerEvent {
    /// Calls C++ function: <span style='color: green;'>```QEvent* static_cast<QEvent*>(QTimerEvent* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QEvent {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QEvent_ptr(self as *mut crate::QTimerEvent)
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QTimerEvent> for crate::QEvent {
    /// Calls C++ function: <span style='color: green;'>```QTimerEvent* dynamic_cast<QTimerEvent*>(QEvent* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QEvent>,
    ) -> ::cpp_utils::Ptr<crate::QTimerEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QTimerEvent_ptr(
            ptr.as_raw_ptr() as *mut crate::QEvent,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QTimerEvent* dynamic_cast<QTimerEvent*>(QEvent* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QEvent>,
    ) -> ::cpp_utils::MutPtr<crate::QTimerEvent> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QTimerEvent_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QChildEvent> for crate::QEvent {
    /// Calls C++ function: <span style='color: green;'>```QChildEvent* static_cast<QChildEvent*>(QEvent* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QEvent>,
    ) -> ::cpp_utils::Ptr<crate::QChildEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QChildEvent_ptr(
            ptr.as_raw_ptr() as *mut crate::QEvent,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QChildEvent* static_cast<QChildEvent*>(QEvent* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QEvent>,
    ) -> ::cpp_utils::MutPtr<crate::QChildEvent> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QChildEvent_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QEvent> for crate::QChildEvent {
    /// Calls C++ function: <span style='color: green;'>```QEvent* static_cast<QEvent*>(QChildEvent* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QChildEvent>,
    ) -> ::cpp_utils::Ptr<crate::QEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QEvent_ptr1(
            ptr.as_raw_ptr() as *mut crate::QChildEvent
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QEvent* static_cast<QEvent*>(QChildEvent* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QChildEvent>,
    ) -> ::cpp_utils::MutPtr<crate::QEvent> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QEvent_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QChildEvent {
    type Target = crate::QEvent;
    /// Calls C++ function: <span style='color: green;'>```QEvent* static_cast<QEvent*>(QChildEvent* ptr)```</span>.
    fn deref(&self) -> &crate::QEvent {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QEvent_ptr1(
                self as *const crate::QChildEvent as *mut crate::QChildEvent,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QChildEvent {
    /// Calls C++ function: <span style='color: green;'>```QEvent* static_cast<QEvent*>(QChildEvent* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QEvent {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QEvent_ptr1(self as *mut crate::QChildEvent)
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QChildEvent> for crate::QEvent {
    /// Calls C++ function: <span style='color: green;'>```QChildEvent* dynamic_cast<QChildEvent*>(QEvent* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QEvent>,
    ) -> ::cpp_utils::Ptr<crate::QChildEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QChildEvent_ptr(
            ptr.as_raw_ptr() as *mut crate::QEvent,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QChildEvent* dynamic_cast<QChildEvent*>(QEvent* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QEvent>,
    ) -> ::cpp_utils::MutPtr<crate::QChildEvent> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QChildEvent_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QDynamicPropertyChangeEvent> for crate::QEvent {
    /// Calls C++ function: <span style='color: green;'>```QDynamicPropertyChangeEvent* static_cast<QDynamicPropertyChangeEvent*>(QEvent* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QEvent>,
    ) -> ::cpp_utils::Ptr<crate::QDynamicPropertyChangeEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QDynamicPropertyChangeEvent_ptr(
            ptr.as_raw_ptr() as *mut crate::QEvent,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QDynamicPropertyChangeEvent* static_cast<QDynamicPropertyChangeEvent*>(QEvent* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QEvent>,
    ) -> ::cpp_utils::MutPtr<crate::QDynamicPropertyChangeEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QDynamicPropertyChangeEvent_ptr(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QEvent> for crate::QDynamicPropertyChangeEvent {
    /// Calls C++ function: <span style='color: green;'>```QEvent* static_cast<QEvent*>(QDynamicPropertyChangeEvent* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QDynamicPropertyChangeEvent>,
    ) -> ::cpp_utils::Ptr<crate::QEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QEvent_ptr2(
            ptr.as_raw_ptr() as *mut crate::QDynamicPropertyChangeEvent
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QEvent* static_cast<QEvent*>(QDynamicPropertyChangeEvent* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QDynamicPropertyChangeEvent>,
    ) -> ::cpp_utils::MutPtr<crate::QEvent> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QEvent_ptr2(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QDynamicPropertyChangeEvent {
    type Target = crate::QEvent;
    /// Calls C++ function: <span style='color: green;'>```QEvent* static_cast<QEvent*>(QDynamicPropertyChangeEvent* ptr)```</span>.
    fn deref(&self) -> &crate::QEvent {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QEvent_ptr2(
                self as *const crate::QDynamicPropertyChangeEvent
                    as *mut crate::QDynamicPropertyChangeEvent,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QDynamicPropertyChangeEvent {
    /// Calls C++ function: <span style='color: green;'>```QEvent* static_cast<QEvent*>(QDynamicPropertyChangeEvent* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QEvent {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QEvent_ptr2(
                self as *mut crate::QDynamicPropertyChangeEvent,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QDynamicPropertyChangeEvent> for crate::QEvent {
    /// Calls C++ function: <span style='color: green;'>```QDynamicPropertyChangeEvent* dynamic_cast<QDynamicPropertyChangeEvent*>(QEvent* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QEvent>,
    ) -> ::cpp_utils::Ptr<crate::QDynamicPropertyChangeEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QDynamicPropertyChangeEvent_ptr(
            ptr.as_raw_ptr() as *mut crate::QEvent,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QDynamicPropertyChangeEvent* dynamic_cast<QDynamicPropertyChangeEvent*>(QEvent* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QEvent>,
    ) -> ::cpp_utils::MutPtr<crate::QDynamicPropertyChangeEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QDynamicPropertyChangeEvent_ptr(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QDeferredDeleteEvent> for crate::QEvent {
    /// Calls C++ function: <span style='color: green;'>```QDeferredDeleteEvent* static_cast<QDeferredDeleteEvent*>(QEvent* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QEvent>,
    ) -> ::cpp_utils::Ptr<crate::QDeferredDeleteEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QDeferredDeleteEvent_ptr(
            ptr.as_raw_ptr() as *mut crate::QEvent,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QDeferredDeleteEvent* static_cast<QDeferredDeleteEvent*>(QEvent* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QEvent>,
    ) -> ::cpp_utils::MutPtr<crate::QDeferredDeleteEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QDeferredDeleteEvent_ptr(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QEvent> for crate::QDeferredDeleteEvent {
    /// Calls C++ function: <span style='color: green;'>```QEvent* static_cast<QEvent*>(QDeferredDeleteEvent* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QDeferredDeleteEvent>,
    ) -> ::cpp_utils::Ptr<crate::QEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QEvent_ptr3(
            ptr.as_raw_ptr() as *mut crate::QDeferredDeleteEvent
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QEvent* static_cast<QEvent*>(QDeferredDeleteEvent* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QDeferredDeleteEvent>,
    ) -> ::cpp_utils::MutPtr<crate::QEvent> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QEvent_ptr3(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QDeferredDeleteEvent {
    type Target = crate::QEvent;
    /// Calls C++ function: <span style='color: green;'>```QEvent* static_cast<QEvent*>(QDeferredDeleteEvent* ptr)```</span>.
    fn deref(&self) -> &crate::QEvent {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QEvent_ptr3(
                self as *const crate::QDeferredDeleteEvent as *mut crate::QDeferredDeleteEvent,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QDeferredDeleteEvent {
    /// Calls C++ function: <span style='color: green;'>```QEvent* static_cast<QEvent*>(QDeferredDeleteEvent* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QEvent {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QEvent_ptr3(
                self as *mut crate::QDeferredDeleteEvent,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QDeferredDeleteEvent> for crate::QEvent {
    /// Calls C++ function: <span style='color: green;'>```QDeferredDeleteEvent* dynamic_cast<QDeferredDeleteEvent*>(QEvent* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QEvent>,
    ) -> ::cpp_utils::Ptr<crate::QDeferredDeleteEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QDeferredDeleteEvent_ptr(
            ptr.as_raw_ptr() as *mut crate::QEvent,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QDeferredDeleteEvent* dynamic_cast<QDeferredDeleteEvent*>(QEvent* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QEvent>,
    ) -> ::cpp_utils::MutPtr<crate::QDeferredDeleteEvent> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QDeferredDeleteEvent_ptr(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QCoreApplication> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QCoreApplication* static_cast<QCoreApplication*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QCoreApplication> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QCoreApplication_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QCoreApplication* static_cast<QCoreApplication*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QCoreApplication> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QCoreApplication_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QCoreApplication {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QCoreApplication* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QCoreApplication>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr13(
            ptr.as_raw_ptr() as *mut crate::QCoreApplication
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QCoreApplication* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QCoreApplication>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr13(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QCoreApplication {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QCoreApplication* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr13(
                self as *const crate::QCoreApplication as *mut crate::QCoreApplication,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QCoreApplication {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QCoreApplication* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr13(
                self as *mut crate::QCoreApplication,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QCoreApplication> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QCoreApplication* dynamic_cast<QCoreApplication*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QCoreApplication> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QCoreApplication_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QCoreApplication* dynamic_cast<QCoreApplication*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QCoreApplication> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QCoreApplication_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QFileDevice> for crate::QIODevice {
    /// Calls C++ function: <span style='color: green;'>```QFileDevice* static_cast<QFileDevice*>(QIODevice* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QIODevice>,
    ) -> ::cpp_utils::Ptr<crate::QFileDevice> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QFileDevice_ptr(
            ptr.as_raw_ptr() as *mut crate::QIODevice,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QFileDevice* static_cast<QFileDevice*>(QIODevice* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QIODevice>,
    ) -> ::cpp_utils::MutPtr<crate::QFileDevice> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QFileDevice_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QIODevice> for crate::QFileDevice {
    /// Calls C++ function: <span style='color: green;'>```QIODevice* static_cast<QIODevice*>(QFileDevice* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QFileDevice>,
    ) -> ::cpp_utils::Ptr<crate::QIODevice> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QIODevice_ptr2(
            ptr.as_raw_ptr() as *mut crate::QFileDevice
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QIODevice* static_cast<QIODevice*>(QFileDevice* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QFileDevice>,
    ) -> ::cpp_utils::MutPtr<crate::QIODevice> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QIODevice_ptr2(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QFileDevice {
    type Target = crate::QIODevice;
    /// Calls C++ function: <span style='color: green;'>```QIODevice* static_cast<QIODevice*>(QFileDevice* ptr)```</span>.
    fn deref(&self) -> &crate::QIODevice {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QIODevice_ptr2(
                self as *const crate::QFileDevice as *mut crate::QFileDevice,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QFileDevice {
    /// Calls C++ function: <span style='color: green;'>```QIODevice* static_cast<QIODevice*>(QFileDevice* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QIODevice {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QIODevice_ptr2(
                self as *mut crate::QFileDevice,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QFileDevice> for crate::QIODevice {
    /// Calls C++ function: <span style='color: green;'>```QFileDevice* dynamic_cast<QFileDevice*>(QIODevice* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QIODevice>,
    ) -> ::cpp_utils::Ptr<crate::QFileDevice> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QFileDevice_ptr(
            ptr.as_raw_ptr() as *mut crate::QIODevice,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QFileDevice* dynamic_cast<QFileDevice*>(QIODevice* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QIODevice>,
    ) -> ::cpp_utils::MutPtr<crate::QFileDevice> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QFileDevice_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QFileDevice> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QFileDevice* static_cast<QFileDevice*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QFileDevice> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QFileDevice_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QFileDevice* static_cast<QFileDevice*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QFileDevice> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QFileDevice_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QFileDevice {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QFileDevice* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QFileDevice>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr14(
            ptr.as_raw_ptr() as *mut crate::QFileDevice
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QFileDevice* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QFileDevice>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr14(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QFileDevice> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QFileDevice* dynamic_cast<QFileDevice*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QFileDevice> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QFileDevice_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QFileDevice* dynamic_cast<QFileDevice*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QFileDevice> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QFileDevice_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QFile> for crate::QFileDevice {
    /// Calls C++ function: <span style='color: green;'>```QFile* static_cast<QFile*>(QFileDevice* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QFileDevice>,
    ) -> ::cpp_utils::Ptr<crate::QFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QFile_ptr(
            ptr.as_raw_ptr() as *mut crate::QFileDevice
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QFile* static_cast<QFile*>(QFileDevice* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QFileDevice>,
    ) -> ::cpp_utils::MutPtr<crate::QFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QFile_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QFileDevice> for crate::QFile {
    /// Calls C++ function: <span style='color: green;'>```QFileDevice* static_cast<QFileDevice*>(QFile* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QFile>,
    ) -> ::cpp_utils::Ptr<crate::QFileDevice> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QFileDevice_ptr2(
            ptr.as_raw_ptr() as *mut crate::QFile,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QFileDevice* static_cast<QFileDevice*>(QFile* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QFile>,
    ) -> ::cpp_utils::MutPtr<crate::QFileDevice> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QFileDevice_ptr2(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QFile {
    type Target = crate::QFileDevice;
    /// Calls C++ function: <span style='color: green;'>```QFileDevice* static_cast<QFileDevice*>(QFile* ptr)```</span>.
    fn deref(&self) -> &crate::QFileDevice {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QFileDevice_ptr2(
                self as *const crate::QFile as *mut crate::QFile,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QFile {
    /// Calls C++ function: <span style='color: green;'>```QFileDevice* static_cast<QFileDevice*>(QFile* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QFileDevice {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QFileDevice_ptr2(self as *mut crate::QFile)
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QFile> for crate::QFileDevice {
    /// Calls C++ function: <span style='color: green;'>```QFile* dynamic_cast<QFile*>(QFileDevice* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QFileDevice>,
    ) -> ::cpp_utils::Ptr<crate::QFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QFile_ptr(
            ptr.as_raw_ptr() as *mut crate::QFileDevice
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QFile* dynamic_cast<QFile*>(QFileDevice* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QFileDevice>,
    ) -> ::cpp_utils::MutPtr<crate::QFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QFile_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QFile> for crate::QIODevice {
    /// Calls C++ function: <span style='color: green;'>```QFile* static_cast<QFile*>(QIODevice* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QIODevice>,
    ) -> ::cpp_utils::Ptr<crate::QFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QFile_ptr1(
            ptr.as_raw_ptr() as *mut crate::QIODevice
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QFile* static_cast<QFile*>(QIODevice* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QIODevice>,
    ) -> ::cpp_utils::MutPtr<crate::QFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QFile_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QIODevice> for crate::QFile {
    /// Calls C++ function: <span style='color: green;'>```QIODevice* static_cast<QIODevice*>(QFile* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QFile>,
    ) -> ::cpp_utils::Ptr<crate::QIODevice> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QIODevice_ptr3(
            ptr.as_raw_ptr() as *mut crate::QFile
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QIODevice* static_cast<QIODevice*>(QFile* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QFile>,
    ) -> ::cpp_utils::MutPtr<crate::QIODevice> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QIODevice_ptr3(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QFile> for crate::QIODevice {
    /// Calls C++ function: <span style='color: green;'>```QFile* dynamic_cast<QFile*>(QIODevice* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QIODevice>,
    ) -> ::cpp_utils::Ptr<crate::QFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QFile_ptr1(
            ptr.as_raw_ptr() as *mut crate::QIODevice
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QFile* dynamic_cast<QFile*>(QIODevice* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QIODevice>,
    ) -> ::cpp_utils::MutPtr<crate::QFile> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QFile_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QFile> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QFile* static_cast<QFile*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QFile_ptr2(
            ptr.as_raw_ptr() as *mut crate::QObject
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QFile* static_cast<QFile*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QFile_ptr2(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QFile {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QFile* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QFile>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr15(
            ptr.as_raw_ptr() as *mut crate::QFile
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QFile* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QFile>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr15(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QFile> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QFile* dynamic_cast<QFile*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QFile_ptr2(
            ptr.as_raw_ptr() as *mut crate::QObject
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QFile* dynamic_cast<QFile*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QFile> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QFile_ptr2(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QEventTransition> for crate::QAbstractTransition {
    /// Calls C++ function: <span style='color: green;'>```QEventTransition* static_cast<QEventTransition*>(QAbstractTransition* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractTransition>,
    ) -> ::cpp_utils::Ptr<crate::QEventTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QEventTransition_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractTransition,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QEventTransition* static_cast<QEventTransition*>(QAbstractTransition* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractTransition>,
    ) -> ::cpp_utils::MutPtr<crate::QEventTransition> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QEventTransition_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QAbstractTransition> for crate::QEventTransition {
    /// Calls C++ function: <span style='color: green;'>```QAbstractTransition* static_cast<QAbstractTransition*>(QEventTransition* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QEventTransition>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractTransition_ptr1(
            ptr.as_raw_ptr() as *mut crate::QEventTransition,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractTransition* static_cast<QAbstractTransition*>(QEventTransition* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QEventTransition>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractTransition_ptr1(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QEventTransition {
    type Target = crate::QAbstractTransition;
    /// Calls C++ function: <span style='color: green;'>```QAbstractTransition* static_cast<QAbstractTransition*>(QEventTransition* ptr)```</span>.
    fn deref(&self) -> &crate::QAbstractTransition {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractTransition_ptr1(
                self as *const crate::QEventTransition as *mut crate::QEventTransition,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QEventTransition {
    /// Calls C++ function: <span style='color: green;'>```QAbstractTransition* static_cast<QAbstractTransition*>(QEventTransition* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QAbstractTransition {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractTransition_ptr1(
                self as *mut crate::QEventTransition,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QEventTransition> for crate::QAbstractTransition {
    /// Calls C++ function: <span style='color: green;'>```QEventTransition* dynamic_cast<QEventTransition*>(QAbstractTransition* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractTransition>,
    ) -> ::cpp_utils::Ptr<crate::QEventTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QEventTransition_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractTransition,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QEventTransition* dynamic_cast<QEventTransition*>(QAbstractTransition* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractTransition>,
    ) -> ::cpp_utils::MutPtr<crate::QEventTransition> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QEventTransition_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QEventTransition> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QEventTransition* static_cast<QEventTransition*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QEventTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QEventTransition_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QEventTransition* static_cast<QEventTransition*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QEventTransition> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QEventTransition_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QEventTransition {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QEventTransition* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QEventTransition>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr16(
            ptr.as_raw_ptr() as *mut crate::QEventTransition
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QEventTransition* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QEventTransition>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr16(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QEventTransition> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QEventTransition* dynamic_cast<QEventTransition*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QEventTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QEventTransition_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QEventTransition* dynamic_cast<QEventTransition*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QEventTransition> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QEventTransition_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QFileSelector> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QFileSelector* static_cast<QFileSelector*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QFileSelector> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QFileSelector_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QFileSelector* static_cast<QFileSelector*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QFileSelector> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QFileSelector_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QFileSelector {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QFileSelector* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QFileSelector>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr17(
            ptr.as_raw_ptr() as *mut crate::QFileSelector
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QFileSelector* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QFileSelector>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr17(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QFileSelector {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QFileSelector* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr17(
                self as *const crate::QFileSelector as *mut crate::QFileSelector,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QFileSelector {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QFileSelector* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr17(
                self as *mut crate::QFileSelector,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QFileSelector> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QFileSelector* dynamic_cast<QFileSelector*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QFileSelector> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QFileSelector_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QFileSelector* dynamic_cast<QFileSelector*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QFileSelector> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QFileSelector_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QFileSystemWatcher> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QFileSystemWatcher* static_cast<QFileSystemWatcher*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QFileSystemWatcher> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QFileSystemWatcher_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QFileSystemWatcher* static_cast<QFileSystemWatcher*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QFileSystemWatcher> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QFileSystemWatcher_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QFileSystemWatcher {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QFileSystemWatcher* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QFileSystemWatcher>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr18(
            ptr.as_raw_ptr() as *mut crate::QFileSystemWatcher
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QFileSystemWatcher* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QFileSystemWatcher>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr18(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QFileSystemWatcher {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QFileSystemWatcher* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr18(
                self as *const crate::QFileSystemWatcher as *mut crate::QFileSystemWatcher,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QFileSystemWatcher {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QFileSystemWatcher* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr18(
                self as *mut crate::QFileSystemWatcher,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QFileSystemWatcher> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QFileSystemWatcher* dynamic_cast<QFileSystemWatcher*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QFileSystemWatcher> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QFileSystemWatcher_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QFileSystemWatcher* dynamic_cast<QFileSystemWatcher*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QFileSystemWatcher> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QFileSystemWatcher_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QFinalState> for crate::QAbstractState {
    /// Calls C++ function: <span style='color: green;'>```QFinalState* static_cast<QFinalState*>(QAbstractState* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractState>,
    ) -> ::cpp_utils::Ptr<crate::QFinalState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QFinalState_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractState,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QFinalState* static_cast<QFinalState*>(QAbstractState* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractState>,
    ) -> ::cpp_utils::MutPtr<crate::QFinalState> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QFinalState_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QAbstractState> for crate::QFinalState {
    /// Calls C++ function: <span style='color: green;'>```QAbstractState* static_cast<QAbstractState*>(QFinalState* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QFinalState>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractState_ptr1(
            ptr.as_raw_ptr() as *mut crate::QFinalState,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractState* static_cast<QAbstractState*>(QFinalState* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QFinalState>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractState> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractState_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QFinalState {
    type Target = crate::QAbstractState;
    /// Calls C++ function: <span style='color: green;'>```QAbstractState* static_cast<QAbstractState*>(QFinalState* ptr)```</span>.
    fn deref(&self) -> &crate::QAbstractState {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractState_ptr1(
                self as *const crate::QFinalState as *mut crate::QFinalState,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QFinalState {
    /// Calls C++ function: <span style='color: green;'>```QAbstractState* static_cast<QAbstractState*>(QFinalState* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QAbstractState {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractState_ptr1(
                self as *mut crate::QFinalState,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QFinalState> for crate::QAbstractState {
    /// Calls C++ function: <span style='color: green;'>```QFinalState* dynamic_cast<QFinalState*>(QAbstractState* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractState>,
    ) -> ::cpp_utils::Ptr<crate::QFinalState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QFinalState_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractState,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QFinalState* dynamic_cast<QFinalState*>(QAbstractState* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractState>,
    ) -> ::cpp_utils::MutPtr<crate::QFinalState> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QFinalState_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QFinalState> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QFinalState* static_cast<QFinalState*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QFinalState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QFinalState_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QFinalState* static_cast<QFinalState*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QFinalState> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QFinalState_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QFinalState {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QFinalState* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QFinalState>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr19(
            ptr.as_raw_ptr() as *mut crate::QFinalState
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QFinalState* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QFinalState>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr19(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QFinalState> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QFinalState* dynamic_cast<QFinalState*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QFinalState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QFinalState_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QFinalState* dynamic_cast<QFinalState*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QFinalState> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QFinalState_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QMutex> for crate::QBasicMutex {
    /// Calls C++ function: <span style='color: green;'>```QMutex* static_cast<QMutex*>(QBasicMutex* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QBasicMutex>,
    ) -> ::cpp_utils::Ptr<crate::QMutex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QMutex_ptr(
            ptr.as_raw_ptr() as *mut crate::QBasicMutex
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QMutex* static_cast<QMutex*>(QBasicMutex* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QBasicMutex>,
    ) -> ::cpp_utils::MutPtr<crate::QMutex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QMutex_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QBasicMutex> for crate::QMutex {
    /// Calls C++ function: <span style='color: green;'>```QBasicMutex* static_cast<QBasicMutex*>(QMutex* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QMutex>,
    ) -> ::cpp_utils::Ptr<crate::QBasicMutex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QBasicMutex_ptr(
            ptr.as_raw_ptr() as *mut crate::QMutex,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QBasicMutex* static_cast<QBasicMutex*>(QMutex* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QMutex>,
    ) -> ::cpp_utils::MutPtr<crate::QBasicMutex> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QBasicMutex_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QMutex {
    type Target = crate::QBasicMutex;
    /// Calls C++ function: <span style='color: green;'>```QBasicMutex* static_cast<QBasicMutex*>(QMutex* ptr)```</span>.
    fn deref(&self) -> &crate::QBasicMutex {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QBasicMutex_ptr(
                self as *const crate::QMutex as *mut crate::QMutex,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QMutex {
    /// Calls C++ function: <span style='color: green;'>```QBasicMutex* static_cast<QBasicMutex*>(QMutex* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QBasicMutex {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QBasicMutex_ptr(self as *mut crate::QMutex)
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::StaticDowncast<crate::QFutureWatcherBase> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QFutureWatcherBase* static_cast<QFutureWatcherBase*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QFutureWatcherBase> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QFutureWatcherBase_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QFutureWatcherBase* static_cast<QFutureWatcherBase*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QFutureWatcherBase> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QFutureWatcherBase_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QFutureWatcherBase {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QFutureWatcherBase* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QFutureWatcherBase>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr20(
            ptr.as_raw_ptr() as *mut crate::QFutureWatcherBase
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QFutureWatcherBase* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QFutureWatcherBase>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr20(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QFutureWatcherBase {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QFutureWatcherBase* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr20(
                self as *const crate::QFutureWatcherBase as *mut crate::QFutureWatcherBase,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QFutureWatcherBase {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QFutureWatcherBase* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr20(
                self as *mut crate::QFutureWatcherBase,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QFutureWatcherBase> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QFutureWatcherBase* dynamic_cast<QFutureWatcherBase*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QFutureWatcherBase> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QFutureWatcherBase_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QFutureWatcherBase* dynamic_cast<QFutureWatcherBase*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QFutureWatcherBase> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QFutureWatcherBase_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QHistoryState> for crate::QAbstractState {
    /// Calls C++ function: <span style='color: green;'>```QHistoryState* static_cast<QHistoryState*>(QAbstractState* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractState>,
    ) -> ::cpp_utils::Ptr<crate::QHistoryState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QHistoryState_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractState,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QHistoryState* static_cast<QHistoryState*>(QAbstractState* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractState>,
    ) -> ::cpp_utils::MutPtr<crate::QHistoryState> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QHistoryState_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QAbstractState> for crate::QHistoryState {
    /// Calls C++ function: <span style='color: green;'>```QAbstractState* static_cast<QAbstractState*>(QHistoryState* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QHistoryState>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractState_ptr2(
            ptr.as_raw_ptr() as *mut crate::QHistoryState,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractState* static_cast<QAbstractState*>(QHistoryState* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QHistoryState>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractState> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractState_ptr2(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QHistoryState {
    type Target = crate::QAbstractState;
    /// Calls C++ function: <span style='color: green;'>```QAbstractState* static_cast<QAbstractState*>(QHistoryState* ptr)```</span>.
    fn deref(&self) -> &crate::QAbstractState {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractState_ptr2(
                self as *const crate::QHistoryState as *mut crate::QHistoryState,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QHistoryState {
    /// Calls C++ function: <span style='color: green;'>```QAbstractState* static_cast<QAbstractState*>(QHistoryState* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QAbstractState {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractState_ptr2(
                self as *mut crate::QHistoryState,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QHistoryState> for crate::QAbstractState {
    /// Calls C++ function: <span style='color: green;'>```QHistoryState* dynamic_cast<QHistoryState*>(QAbstractState* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractState>,
    ) -> ::cpp_utils::Ptr<crate::QHistoryState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QHistoryState_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractState,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QHistoryState* dynamic_cast<QHistoryState*>(QAbstractState* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractState>,
    ) -> ::cpp_utils::MutPtr<crate::QHistoryState> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QHistoryState_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QHistoryState> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QHistoryState* static_cast<QHistoryState*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QHistoryState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QHistoryState_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QHistoryState* static_cast<QHistoryState*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QHistoryState> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QHistoryState_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QHistoryState {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QHistoryState* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QHistoryState>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr21(
            ptr.as_raw_ptr() as *mut crate::QHistoryState
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QHistoryState* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QHistoryState>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr21(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QHistoryState> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QHistoryState* dynamic_cast<QHistoryState*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QHistoryState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QHistoryState_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QHistoryState* dynamic_cast<QHistoryState*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QHistoryState> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QHistoryState_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QIdentityProxyModel> for crate::QAbstractProxyModel {
    /// Calls C++ function: <span style='color: green;'>```QIdentityProxyModel* static_cast<QIdentityProxyModel*>(QAbstractProxyModel* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractProxyModel>,
    ) -> ::cpp_utils::Ptr<crate::QIdentityProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QIdentityProxyModel_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractProxyModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QIdentityProxyModel* static_cast<QIdentityProxyModel*>(QAbstractProxyModel* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractProxyModel>,
    ) -> ::cpp_utils::MutPtr<crate::QIdentityProxyModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QIdentityProxyModel_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QAbstractProxyModel> for crate::QIdentityProxyModel {
    /// Calls C++ function: <span style='color: green;'>```QAbstractProxyModel* static_cast<QAbstractProxyModel*>(QIdentityProxyModel* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QIdentityProxyModel>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractProxyModel_ptr2(
            ptr.as_raw_ptr() as *mut crate::QIdentityProxyModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractProxyModel* static_cast<QAbstractProxyModel*>(QIdentityProxyModel* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QIdentityProxyModel>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractProxyModel_ptr2(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QIdentityProxyModel {
    type Target = crate::QAbstractProxyModel;
    /// Calls C++ function: <span style='color: green;'>```QAbstractProxyModel* static_cast<QAbstractProxyModel*>(QIdentityProxyModel* ptr)```</span>.
    fn deref(&self) -> &crate::QAbstractProxyModel {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractProxyModel_ptr2(
                self as *const crate::QIdentityProxyModel as *mut crate::QIdentityProxyModel,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QIdentityProxyModel {
    /// Calls C++ function: <span style='color: green;'>```QAbstractProxyModel* static_cast<QAbstractProxyModel*>(QIdentityProxyModel* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QAbstractProxyModel {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractProxyModel_ptr2(
                self as *mut crate::QIdentityProxyModel,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QIdentityProxyModel> for crate::QAbstractProxyModel {
    /// Calls C++ function: <span style='color: green;'>```QIdentityProxyModel* dynamic_cast<QIdentityProxyModel*>(QAbstractProxyModel* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractProxyModel>,
    ) -> ::cpp_utils::Ptr<crate::QIdentityProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QIdentityProxyModel_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractProxyModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QIdentityProxyModel* dynamic_cast<QIdentityProxyModel*>(QAbstractProxyModel* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractProxyModel>,
    ) -> ::cpp_utils::MutPtr<crate::QIdentityProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QIdentityProxyModel_ptr(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QIdentityProxyModel> for crate::QAbstractItemModel {
    /// Calls C++ function: <span style='color: green;'>```QIdentityProxyModel* static_cast<QIdentityProxyModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::Ptr<crate::QIdentityProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QIdentityProxyModel_ptr1(
            ptr.as_raw_ptr() as *mut crate::QAbstractItemModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QIdentityProxyModel* static_cast<QIdentityProxyModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::MutPtr<crate::QIdentityProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QIdentityProxyModel_ptr1(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QAbstractItemModel> for crate::QIdentityProxyModel {
    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* static_cast<QAbstractItemModel*>(QIdentityProxyModel* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QIdentityProxyModel>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractItemModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractItemModel_ptr4(
            ptr.as_raw_ptr() as *mut crate::QIdentityProxyModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* static_cast<QAbstractItemModel*>(QIdentityProxyModel* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QIdentityProxyModel>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractItemModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractItemModel_ptr4(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QIdentityProxyModel> for crate::QAbstractItemModel {
    /// Calls C++ function: <span style='color: green;'>```QIdentityProxyModel* dynamic_cast<QIdentityProxyModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::Ptr<crate::QIdentityProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QIdentityProxyModel_ptr1(
            ptr.as_raw_ptr() as *mut crate::QAbstractItemModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QIdentityProxyModel* dynamic_cast<QIdentityProxyModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::MutPtr<crate::QIdentityProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QIdentityProxyModel_ptr1(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QIdentityProxyModel> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QIdentityProxyModel* static_cast<QIdentityProxyModel*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QIdentityProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QIdentityProxyModel_ptr2(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QIdentityProxyModel* static_cast<QIdentityProxyModel*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QIdentityProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QIdentityProxyModel_ptr2(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QIdentityProxyModel {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QIdentityProxyModel* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QIdentityProxyModel>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr22(
            ptr.as_raw_ptr() as *mut crate::QIdentityProxyModel
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QIdentityProxyModel* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QIdentityProxyModel>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr22(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QIdentityProxyModel> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QIdentityProxyModel* dynamic_cast<QIdentityProxyModel*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QIdentityProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QIdentityProxyModel_ptr2(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QIdentityProxyModel* dynamic_cast<QIdentityProxyModel*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QIdentityProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QIdentityProxyModel_ptr2(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QItemSelectionModel> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QItemSelectionModel* static_cast<QItemSelectionModel*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QItemSelectionModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QItemSelectionModel_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QItemSelectionModel* static_cast<QItemSelectionModel*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QItemSelectionModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QItemSelectionModel_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QItemSelectionModel {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QItemSelectionModel* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QItemSelectionModel>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr23(
            ptr.as_raw_ptr() as *mut crate::QItemSelectionModel
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QItemSelectionModel* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QItemSelectionModel>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr23(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QItemSelectionModel {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QItemSelectionModel* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr23(
                self as *const crate::QItemSelectionModel as *mut crate::QItemSelectionModel,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QItemSelectionModel {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QItemSelectionModel* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr23(
                self as *mut crate::QItemSelectionModel,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QItemSelectionModel> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QItemSelectionModel* dynamic_cast<QItemSelectionModel*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QItemSelectionModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QItemSelectionModel_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QItemSelectionModel* dynamic_cast<QItemSelectionModel*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QItemSelectionModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QItemSelectionModel_ptr(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QItemSelection> for crate::QListOfQItemSelectionRange {
    /// Calls C++ function: <span style='color: green;'>```QItemSelection* static_cast<QItemSelection*>(QList<QItemSelectionRange>* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QListOfQItemSelectionRange>,
    ) -> ::cpp_utils::Ptr<crate::QItemSelection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QItemSelection_ptr(
            ptr.as_raw_ptr() as *mut crate::QListOfQItemSelectionRange,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QItemSelection* static_cast<QItemSelection*>(QList<QItemSelectionRange>* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QListOfQItemSelectionRange>,
    ) -> ::cpp_utils::MutPtr<crate::QItemSelection> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QItemSelection_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QListOfQItemSelectionRange> for crate::QItemSelection {
    /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>* static_cast<QList<QItemSelectionRange>*>(QItemSelection* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QItemSelection>,
    ) -> ::cpp_utils::Ptr<crate::QListOfQItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QList_QItemSelectionRange_ptr(
            ptr.as_raw_ptr() as *mut crate::QItemSelection,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>* static_cast<QList<QItemSelectionRange>*>(QItemSelection* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QItemSelection>,
    ) -> ::cpp_utils::MutPtr<crate::QListOfQItemSelectionRange> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QList_QItemSelectionRange_ptr(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QItemSelection {
    type Target = crate::QListOfQItemSelectionRange;
    /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>* static_cast<QList<QItemSelectionRange>*>(QItemSelection* ptr)```</span>.
    fn deref(&self) -> &crate::QListOfQItemSelectionRange {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QList_QItemSelectionRange_ptr(
                self as *const crate::QItemSelection as *mut crate::QItemSelection,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QItemSelection {
    /// Calls C++ function: <span style='color: green;'>```QList<QItemSelectionRange>* static_cast<QList<QItemSelectionRange>*>(QItemSelection* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QListOfQItemSelectionRange {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QList_QItemSelectionRange_ptr(
                self as *mut crate::QItemSelection,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::StaticDowncast<crate::QLibrary> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QLibrary* static_cast<QLibrary*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QLibrary> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QLibrary_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QLibrary* static_cast<QLibrary*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QLibrary> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QLibrary_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QLibrary {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QLibrary* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QLibrary>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr24(
            ptr.as_raw_ptr() as *mut crate::QLibrary
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QLibrary* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QLibrary>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr24(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QLibrary {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QLibrary* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr24(
                self as *const crate::QLibrary as *mut crate::QLibrary,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QLibrary {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QLibrary* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr24(self as *mut crate::QLibrary)
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QLibrary> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QLibrary* dynamic_cast<QLibrary*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QLibrary> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QLibrary_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QLibrary* dynamic_cast<QLibrary*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QLibrary> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QLibrary_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QMimeData> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QMimeData* static_cast<QMimeData*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QMimeData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QMimeData_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QMimeData* static_cast<QMimeData*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QMimeData> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QMimeData_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QMimeData {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QMimeData* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QMimeData>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr25(
            ptr.as_raw_ptr() as *mut crate::QMimeData
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QMimeData* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QMimeData>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr25(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QMimeData {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QMimeData* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr25(
                self as *const crate::QMimeData as *mut crate::QMimeData,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QMimeData {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QMimeData* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr25(self as *mut crate::QMimeData)
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QMimeData> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QMimeData* dynamic_cast<QMimeData*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QMimeData> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QMimeData_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QMimeData* dynamic_cast<QMimeData*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QMimeData> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QMimeData_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QObjectCleanupHandler> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QObjectCleanupHandler* static_cast<QObjectCleanupHandler*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QObjectCleanupHandler> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObjectCleanupHandler_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObjectCleanupHandler* static_cast<QObjectCleanupHandler*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QObjectCleanupHandler> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObjectCleanupHandler_ptr(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QObjectCleanupHandler {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QObjectCleanupHandler* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QObjectCleanupHandler>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr26(
            ptr.as_raw_ptr() as *mut crate::QObjectCleanupHandler
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QObjectCleanupHandler* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObjectCleanupHandler>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr26(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QObjectCleanupHandler {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QObjectCleanupHandler* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr26(
                self as *const crate::QObjectCleanupHandler as *mut crate::QObjectCleanupHandler,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QObjectCleanupHandler {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QObjectCleanupHandler* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr26(
                self as *mut crate::QObjectCleanupHandler,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QObjectCleanupHandler> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QObjectCleanupHandler* dynamic_cast<QObjectCleanupHandler*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QObjectCleanupHandler> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QObjectCleanupHandler_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObjectCleanupHandler* dynamic_cast<QObjectCleanupHandler*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QObjectCleanupHandler> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QObjectCleanupHandler_ptr(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QParallelAnimationGroup> for crate::QAnimationGroup {
    /// Calls C++ function: <span style='color: green;'>```QParallelAnimationGroup* static_cast<QParallelAnimationGroup*>(QAnimationGroup* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QAnimationGroup>,
    ) -> ::cpp_utils::Ptr<crate::QParallelAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QParallelAnimationGroup_ptr(
            ptr.as_raw_ptr() as *mut crate::QAnimationGroup,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QParallelAnimationGroup* static_cast<QParallelAnimationGroup*>(QAnimationGroup* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAnimationGroup>,
    ) -> ::cpp_utils::MutPtr<crate::QParallelAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QParallelAnimationGroup_ptr(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QAnimationGroup> for crate::QParallelAnimationGroup {
    /// Calls C++ function: <span style='color: green;'>```QAnimationGroup* static_cast<QAnimationGroup*>(QParallelAnimationGroup* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QParallelAnimationGroup>,
    ) -> ::cpp_utils::Ptr<crate::QAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAnimationGroup_ptr2(
            ptr.as_raw_ptr() as *mut crate::QParallelAnimationGroup,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAnimationGroup* static_cast<QAnimationGroup*>(QParallelAnimationGroup* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QParallelAnimationGroup>,
    ) -> ::cpp_utils::MutPtr<crate::QAnimationGroup> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAnimationGroup_ptr2(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QParallelAnimationGroup {
    type Target = crate::QAnimationGroup;
    /// Calls C++ function: <span style='color: green;'>```QAnimationGroup* static_cast<QAnimationGroup*>(QParallelAnimationGroup* ptr)```</span>.
    fn deref(&self) -> &crate::QAnimationGroup {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAnimationGroup_ptr2(
                self as *const crate::QParallelAnimationGroup
                    as *mut crate::QParallelAnimationGroup,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QParallelAnimationGroup {
    /// Calls C++ function: <span style='color: green;'>```QAnimationGroup* static_cast<QAnimationGroup*>(QParallelAnimationGroup* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QAnimationGroup {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAnimationGroup_ptr2(
                self as *mut crate::QParallelAnimationGroup,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QParallelAnimationGroup> for crate::QAnimationGroup {
    /// Calls C++ function: <span style='color: green;'>```QParallelAnimationGroup* dynamic_cast<QParallelAnimationGroup*>(QAnimationGroup* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QAnimationGroup>,
    ) -> ::cpp_utils::Ptr<crate::QParallelAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QParallelAnimationGroup_ptr(
            ptr.as_raw_ptr() as *mut crate::QAnimationGroup,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QParallelAnimationGroup* dynamic_cast<QParallelAnimationGroup*>(QAnimationGroup* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAnimationGroup>,
    ) -> ::cpp_utils::MutPtr<crate::QParallelAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QParallelAnimationGroup_ptr(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QParallelAnimationGroup> for crate::QAbstractAnimation {
    /// Calls C++ function: <span style='color: green;'>```QParallelAnimationGroup* static_cast<QParallelAnimationGroup*>(QAbstractAnimation* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractAnimation>,
    ) -> ::cpp_utils::Ptr<crate::QParallelAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QParallelAnimationGroup_ptr1(
            ptr.as_raw_ptr() as *mut crate::QAbstractAnimation,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QParallelAnimationGroup* static_cast<QParallelAnimationGroup*>(QAbstractAnimation* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractAnimation>,
    ) -> ::cpp_utils::MutPtr<crate::QParallelAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QParallelAnimationGroup_ptr1(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QAbstractAnimation> for crate::QParallelAnimationGroup {
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* static_cast<QAbstractAnimation*>(QParallelAnimationGroup* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QParallelAnimationGroup>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractAnimation_ptr2(
            ptr.as_raw_ptr() as *mut crate::QParallelAnimationGroup,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* static_cast<QAbstractAnimation*>(QParallelAnimationGroup* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QParallelAnimationGroup>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractAnimation> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractAnimation_ptr2(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QParallelAnimationGroup> for crate::QAbstractAnimation {
    /// Calls C++ function: <span style='color: green;'>```QParallelAnimationGroup* dynamic_cast<QParallelAnimationGroup*>(QAbstractAnimation* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractAnimation>,
    ) -> ::cpp_utils::Ptr<crate::QParallelAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QParallelAnimationGroup_ptr1(
            ptr.as_raw_ptr() as *mut crate::QAbstractAnimation,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QParallelAnimationGroup* dynamic_cast<QParallelAnimationGroup*>(QAbstractAnimation* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractAnimation>,
    ) -> ::cpp_utils::MutPtr<crate::QParallelAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QParallelAnimationGroup_ptr1(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QParallelAnimationGroup> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QParallelAnimationGroup* static_cast<QParallelAnimationGroup*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QParallelAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QParallelAnimationGroup_ptr2(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QParallelAnimationGroup* static_cast<QParallelAnimationGroup*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QParallelAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QParallelAnimationGroup_ptr2(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QParallelAnimationGroup {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QParallelAnimationGroup* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QParallelAnimationGroup>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr27(
            ptr.as_raw_ptr() as *mut crate::QParallelAnimationGroup
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QParallelAnimationGroup* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QParallelAnimationGroup>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr27(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QParallelAnimationGroup> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QParallelAnimationGroup* dynamic_cast<QParallelAnimationGroup*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QParallelAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QParallelAnimationGroup_ptr2(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QParallelAnimationGroup* dynamic_cast<QParallelAnimationGroup*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QParallelAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QParallelAnimationGroup_ptr2(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QPauseAnimation> for crate::QAbstractAnimation {
    /// Calls C++ function: <span style='color: green;'>```QPauseAnimation* static_cast<QPauseAnimation*>(QAbstractAnimation* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractAnimation>,
    ) -> ::cpp_utils::Ptr<crate::QPauseAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QPauseAnimation_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractAnimation,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QPauseAnimation* static_cast<QPauseAnimation*>(QAbstractAnimation* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractAnimation>,
    ) -> ::cpp_utils::MutPtr<crate::QPauseAnimation> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QPauseAnimation_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QAbstractAnimation> for crate::QPauseAnimation {
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* static_cast<QAbstractAnimation*>(QPauseAnimation* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QPauseAnimation>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractAnimation_ptr3(
            ptr.as_raw_ptr() as *mut crate::QPauseAnimation,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* static_cast<QAbstractAnimation*>(QPauseAnimation* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QPauseAnimation>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractAnimation> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractAnimation_ptr3(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QPauseAnimation {
    type Target = crate::QAbstractAnimation;
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* static_cast<QAbstractAnimation*>(QPauseAnimation* ptr)```</span>.
    fn deref(&self) -> &crate::QAbstractAnimation {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractAnimation_ptr3(
                self as *const crate::QPauseAnimation as *mut crate::QPauseAnimation,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QPauseAnimation {
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* static_cast<QAbstractAnimation*>(QPauseAnimation* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QAbstractAnimation {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractAnimation_ptr3(
                self as *mut crate::QPauseAnimation,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QPauseAnimation> for crate::QAbstractAnimation {
    /// Calls C++ function: <span style='color: green;'>```QPauseAnimation* dynamic_cast<QPauseAnimation*>(QAbstractAnimation* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractAnimation>,
    ) -> ::cpp_utils::Ptr<crate::QPauseAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QPauseAnimation_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractAnimation,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QPauseAnimation* dynamic_cast<QPauseAnimation*>(QAbstractAnimation* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractAnimation>,
    ) -> ::cpp_utils::MutPtr<crate::QPauseAnimation> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QPauseAnimation_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QPauseAnimation> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QPauseAnimation* static_cast<QPauseAnimation*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QPauseAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QPauseAnimation_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QPauseAnimation* static_cast<QPauseAnimation*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QPauseAnimation> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QPauseAnimation_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QPauseAnimation {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QPauseAnimation* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QPauseAnimation>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr28(
            ptr.as_raw_ptr() as *mut crate::QPauseAnimation
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QPauseAnimation* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QPauseAnimation>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr28(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QPauseAnimation> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QPauseAnimation* dynamic_cast<QPauseAnimation*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QPauseAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QPauseAnimation_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QPauseAnimation* dynamic_cast<QPauseAnimation*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QPauseAnimation> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QPauseAnimation_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QPluginLoader> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QPluginLoader* static_cast<QPluginLoader*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QPluginLoader> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QPluginLoader_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QPluginLoader* static_cast<QPluginLoader*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QPluginLoader> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QPluginLoader_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QPluginLoader {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QPluginLoader* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QPluginLoader>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr29(
            ptr.as_raw_ptr() as *mut crate::QPluginLoader
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QPluginLoader* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QPluginLoader>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr29(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QPluginLoader {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QPluginLoader* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr29(
                self as *const crate::QPluginLoader as *mut crate::QPluginLoader,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QPluginLoader {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QPluginLoader* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr29(
                self as *mut crate::QPluginLoader,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QPluginLoader> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QPluginLoader* dynamic_cast<QPluginLoader*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QPluginLoader> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QPluginLoader_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QPluginLoader* dynamic_cast<QPluginLoader*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QPluginLoader> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QPluginLoader_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QProcess> for crate::QIODevice {
    /// Calls C++ function: <span style='color: green;'>```QProcess* static_cast<QProcess*>(QIODevice* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QIODevice>,
    ) -> ::cpp_utils::Ptr<crate::QProcess> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QProcess_ptr(
            ptr.as_raw_ptr() as *mut crate::QIODevice
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QProcess* static_cast<QProcess*>(QIODevice* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QIODevice>,
    ) -> ::cpp_utils::MutPtr<crate::QProcess> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QProcess_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QIODevice> for crate::QProcess {
    /// Calls C++ function: <span style='color: green;'>```QIODevice* static_cast<QIODevice*>(QProcess* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QProcess>,
    ) -> ::cpp_utils::Ptr<crate::QIODevice> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QIODevice_ptr4(
            ptr.as_raw_ptr() as *mut crate::QProcess
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QIODevice* static_cast<QIODevice*>(QProcess* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QProcess>,
    ) -> ::cpp_utils::MutPtr<crate::QIODevice> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QIODevice_ptr4(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QProcess {
    type Target = crate::QIODevice;
    /// Calls C++ function: <span style='color: green;'>```QIODevice* static_cast<QIODevice*>(QProcess* ptr)```</span>.
    fn deref(&self) -> &crate::QIODevice {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QIODevice_ptr4(
                self as *const crate::QProcess as *mut crate::QProcess,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QProcess {
    /// Calls C++ function: <span style='color: green;'>```QIODevice* static_cast<QIODevice*>(QProcess* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QIODevice {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QIODevice_ptr4(self as *mut crate::QProcess)
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QProcess> for crate::QIODevice {
    /// Calls C++ function: <span style='color: green;'>```QProcess* dynamic_cast<QProcess*>(QIODevice* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QIODevice>,
    ) -> ::cpp_utils::Ptr<crate::QProcess> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QProcess_ptr(
            ptr.as_raw_ptr() as *mut crate::QIODevice
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QProcess* dynamic_cast<QProcess*>(QIODevice* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QIODevice>,
    ) -> ::cpp_utils::MutPtr<crate::QProcess> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QProcess_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QProcess> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QProcess* static_cast<QProcess*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QProcess> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QProcess_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QProcess* static_cast<QProcess*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QProcess> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QProcess_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QProcess {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QProcess* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QProcess>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr30(
            ptr.as_raw_ptr() as *mut crate::QProcess
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QProcess* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QProcess>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr30(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QProcess> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QProcess* dynamic_cast<QProcess*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QProcess> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QProcess_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QProcess* dynamic_cast<QProcess*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QProcess> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QProcess_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QVariantAnimation> for crate::QAbstractAnimation {
    /// Calls C++ function: <span style='color: green;'>```QVariantAnimation* static_cast<QVariantAnimation*>(QAbstractAnimation* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractAnimation>,
    ) -> ::cpp_utils::Ptr<crate::QVariantAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QVariantAnimation_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractAnimation,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QVariantAnimation* static_cast<QVariantAnimation*>(QAbstractAnimation* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractAnimation>,
    ) -> ::cpp_utils::MutPtr<crate::QVariantAnimation> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QVariantAnimation_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QAbstractAnimation> for crate::QVariantAnimation {
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* static_cast<QAbstractAnimation*>(QVariantAnimation* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QVariantAnimation>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractAnimation_ptr4(
            ptr.as_raw_ptr() as *mut crate::QVariantAnimation,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* static_cast<QAbstractAnimation*>(QVariantAnimation* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QVariantAnimation>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractAnimation> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractAnimation_ptr4(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QVariantAnimation {
    type Target = crate::QAbstractAnimation;
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* static_cast<QAbstractAnimation*>(QVariantAnimation* ptr)```</span>.
    fn deref(&self) -> &crate::QAbstractAnimation {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractAnimation_ptr4(
                self as *const crate::QVariantAnimation as *mut crate::QVariantAnimation,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QVariantAnimation {
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* static_cast<QAbstractAnimation*>(QVariantAnimation* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QAbstractAnimation {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractAnimation_ptr4(
                self as *mut crate::QVariantAnimation,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QVariantAnimation> for crate::QAbstractAnimation {
    /// Calls C++ function: <span style='color: green;'>```QVariantAnimation* dynamic_cast<QVariantAnimation*>(QAbstractAnimation* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractAnimation>,
    ) -> ::cpp_utils::Ptr<crate::QVariantAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QVariantAnimation_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractAnimation,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QVariantAnimation* dynamic_cast<QVariantAnimation*>(QAbstractAnimation* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractAnimation>,
    ) -> ::cpp_utils::MutPtr<crate::QVariantAnimation> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QVariantAnimation_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QVariantAnimation> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QVariantAnimation* static_cast<QVariantAnimation*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QVariantAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QVariantAnimation_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QVariantAnimation* static_cast<QVariantAnimation*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QVariantAnimation> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QVariantAnimation_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QVariantAnimation {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QVariantAnimation* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QVariantAnimation>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr31(
            ptr.as_raw_ptr() as *mut crate::QVariantAnimation
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QVariantAnimation* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QVariantAnimation>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr31(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QVariantAnimation> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QVariantAnimation* dynamic_cast<QVariantAnimation*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QVariantAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QVariantAnimation_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QVariantAnimation* dynamic_cast<QVariantAnimation*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QVariantAnimation> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QVariantAnimation_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QPropertyAnimation> for crate::QVariantAnimation {
    /// Calls C++ function: <span style='color: green;'>```QPropertyAnimation* static_cast<QPropertyAnimation*>(QVariantAnimation* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QVariantAnimation>,
    ) -> ::cpp_utils::Ptr<crate::QPropertyAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QPropertyAnimation_ptr(
            ptr.as_raw_ptr() as *mut crate::QVariantAnimation,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QPropertyAnimation* static_cast<QPropertyAnimation*>(QVariantAnimation* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QVariantAnimation>,
    ) -> ::cpp_utils::MutPtr<crate::QPropertyAnimation> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QPropertyAnimation_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QVariantAnimation> for crate::QPropertyAnimation {
    /// Calls C++ function: <span style='color: green;'>```QVariantAnimation* static_cast<QVariantAnimation*>(QPropertyAnimation* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QPropertyAnimation>,
    ) -> ::cpp_utils::Ptr<crate::QVariantAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QVariantAnimation_ptr2(
            ptr.as_raw_ptr() as *mut crate::QPropertyAnimation,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QVariantAnimation* static_cast<QVariantAnimation*>(QPropertyAnimation* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QPropertyAnimation>,
    ) -> ::cpp_utils::MutPtr<crate::QVariantAnimation> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QVariantAnimation_ptr2(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QPropertyAnimation {
    type Target = crate::QVariantAnimation;
    /// Calls C++ function: <span style='color: green;'>```QVariantAnimation* static_cast<QVariantAnimation*>(QPropertyAnimation* ptr)```</span>.
    fn deref(&self) -> &crate::QVariantAnimation {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QVariantAnimation_ptr2(
                self as *const crate::QPropertyAnimation as *mut crate::QPropertyAnimation,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QPropertyAnimation {
    /// Calls C++ function: <span style='color: green;'>```QVariantAnimation* static_cast<QVariantAnimation*>(QPropertyAnimation* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QVariantAnimation {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QVariantAnimation_ptr2(
                self as *mut crate::QPropertyAnimation,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QPropertyAnimation> for crate::QVariantAnimation {
    /// Calls C++ function: <span style='color: green;'>```QPropertyAnimation* dynamic_cast<QPropertyAnimation*>(QVariantAnimation* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QVariantAnimation>,
    ) -> ::cpp_utils::Ptr<crate::QPropertyAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QPropertyAnimation_ptr(
            ptr.as_raw_ptr() as *mut crate::QVariantAnimation,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QPropertyAnimation* dynamic_cast<QPropertyAnimation*>(QVariantAnimation* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QVariantAnimation>,
    ) -> ::cpp_utils::MutPtr<crate::QPropertyAnimation> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QPropertyAnimation_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QPropertyAnimation> for crate::QAbstractAnimation {
    /// Calls C++ function: <span style='color: green;'>```QPropertyAnimation* static_cast<QPropertyAnimation*>(QAbstractAnimation* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractAnimation>,
    ) -> ::cpp_utils::Ptr<crate::QPropertyAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QPropertyAnimation_ptr1(
            ptr.as_raw_ptr() as *mut crate::QAbstractAnimation,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QPropertyAnimation* static_cast<QPropertyAnimation*>(QAbstractAnimation* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractAnimation>,
    ) -> ::cpp_utils::MutPtr<crate::QPropertyAnimation> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QPropertyAnimation_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QAbstractAnimation> for crate::QPropertyAnimation {
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* static_cast<QAbstractAnimation*>(QPropertyAnimation* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QPropertyAnimation>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractAnimation_ptr5(
            ptr.as_raw_ptr() as *mut crate::QPropertyAnimation,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* static_cast<QAbstractAnimation*>(QPropertyAnimation* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QPropertyAnimation>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractAnimation> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractAnimation_ptr5(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QPropertyAnimation> for crate::QAbstractAnimation {
    /// Calls C++ function: <span style='color: green;'>```QPropertyAnimation* dynamic_cast<QPropertyAnimation*>(QAbstractAnimation* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractAnimation>,
    ) -> ::cpp_utils::Ptr<crate::QPropertyAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QPropertyAnimation_ptr1(
            ptr.as_raw_ptr() as *mut crate::QAbstractAnimation,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QPropertyAnimation* dynamic_cast<QPropertyAnimation*>(QAbstractAnimation* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractAnimation>,
    ) -> ::cpp_utils::MutPtr<crate::QPropertyAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QPropertyAnimation_ptr1(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QPropertyAnimation> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QPropertyAnimation* static_cast<QPropertyAnimation*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QPropertyAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QPropertyAnimation_ptr2(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QPropertyAnimation* static_cast<QPropertyAnimation*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QPropertyAnimation> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QPropertyAnimation_ptr2(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QPropertyAnimation {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QPropertyAnimation* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QPropertyAnimation>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr32(
            ptr.as_raw_ptr() as *mut crate::QPropertyAnimation
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QPropertyAnimation* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QPropertyAnimation>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr32(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QPropertyAnimation> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QPropertyAnimation* dynamic_cast<QPropertyAnimation*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QPropertyAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QPropertyAnimation_ptr2(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QPropertyAnimation* dynamic_cast<QPropertyAnimation*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QPropertyAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QPropertyAnimation_ptr2(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QSaveFile> for crate::QFileDevice {
    /// Calls C++ function: <span style='color: green;'>```QSaveFile* static_cast<QSaveFile*>(QFileDevice* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QFileDevice>,
    ) -> ::cpp_utils::Ptr<crate::QSaveFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QSaveFile_ptr(
            ptr.as_raw_ptr() as *mut crate::QFileDevice
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSaveFile* static_cast<QSaveFile*>(QFileDevice* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QFileDevice>,
    ) -> ::cpp_utils::MutPtr<crate::QSaveFile> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QSaveFile_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QFileDevice> for crate::QSaveFile {
    /// Calls C++ function: <span style='color: green;'>```QFileDevice* static_cast<QFileDevice*>(QSaveFile* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QSaveFile>,
    ) -> ::cpp_utils::Ptr<crate::QFileDevice> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QFileDevice_ptr3(
            ptr.as_raw_ptr() as *mut crate::QSaveFile,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QFileDevice* static_cast<QFileDevice*>(QSaveFile* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QSaveFile>,
    ) -> ::cpp_utils::MutPtr<crate::QFileDevice> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QFileDevice_ptr3(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QSaveFile {
    type Target = crate::QFileDevice;
    /// Calls C++ function: <span style='color: green;'>```QFileDevice* static_cast<QFileDevice*>(QSaveFile* ptr)```</span>.
    fn deref(&self) -> &crate::QFileDevice {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QFileDevice_ptr3(
                self as *const crate::QSaveFile as *mut crate::QSaveFile,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QSaveFile {
    /// Calls C++ function: <span style='color: green;'>```QFileDevice* static_cast<QFileDevice*>(QSaveFile* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QFileDevice {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QFileDevice_ptr3(
                self as *mut crate::QSaveFile,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QSaveFile> for crate::QFileDevice {
    /// Calls C++ function: <span style='color: green;'>```QSaveFile* dynamic_cast<QSaveFile*>(QFileDevice* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QFileDevice>,
    ) -> ::cpp_utils::Ptr<crate::QSaveFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSaveFile_ptr(
            ptr.as_raw_ptr() as *mut crate::QFileDevice
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSaveFile* dynamic_cast<QSaveFile*>(QFileDevice* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QFileDevice>,
    ) -> ::cpp_utils::MutPtr<crate::QSaveFile> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSaveFile_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QSaveFile> for crate::QIODevice {
    /// Calls C++ function: <span style='color: green;'>```QSaveFile* static_cast<QSaveFile*>(QIODevice* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QIODevice>,
    ) -> ::cpp_utils::Ptr<crate::QSaveFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QSaveFile_ptr1(
            ptr.as_raw_ptr() as *mut crate::QIODevice
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSaveFile* static_cast<QSaveFile*>(QIODevice* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QIODevice>,
    ) -> ::cpp_utils::MutPtr<crate::QSaveFile> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QSaveFile_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QIODevice> for crate::QSaveFile {
    /// Calls C++ function: <span style='color: green;'>```QIODevice* static_cast<QIODevice*>(QSaveFile* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QSaveFile>,
    ) -> ::cpp_utils::Ptr<crate::QIODevice> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QIODevice_ptr5(
            ptr.as_raw_ptr() as *mut crate::QSaveFile
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QIODevice* static_cast<QIODevice*>(QSaveFile* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QSaveFile>,
    ) -> ::cpp_utils::MutPtr<crate::QIODevice> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QIODevice_ptr5(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QSaveFile> for crate::QIODevice {
    /// Calls C++ function: <span style='color: green;'>```QSaveFile* dynamic_cast<QSaveFile*>(QIODevice* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QIODevice>,
    ) -> ::cpp_utils::Ptr<crate::QSaveFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSaveFile_ptr1(
            ptr.as_raw_ptr() as *mut crate::QIODevice,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSaveFile* dynamic_cast<QSaveFile*>(QIODevice* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QIODevice>,
    ) -> ::cpp_utils::MutPtr<crate::QSaveFile> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSaveFile_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QSaveFile> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QSaveFile* static_cast<QSaveFile*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QSaveFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QSaveFile_ptr2(
            ptr.as_raw_ptr() as *mut crate::QObject
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSaveFile* static_cast<QSaveFile*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QSaveFile> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QSaveFile_ptr2(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QSaveFile {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QSaveFile* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QSaveFile>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr33(
            ptr.as_raw_ptr() as *mut crate::QSaveFile
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QSaveFile* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QSaveFile>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr33(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QSaveFile> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QSaveFile* dynamic_cast<QSaveFile*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QSaveFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSaveFile_ptr2(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSaveFile* dynamic_cast<QSaveFile*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QSaveFile> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSaveFile_ptr2(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QSequentialAnimationGroup> for crate::QAnimationGroup {
    /// Calls C++ function: <span style='color: green;'>```QSequentialAnimationGroup* static_cast<QSequentialAnimationGroup*>(QAnimationGroup* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QAnimationGroup>,
    ) -> ::cpp_utils::Ptr<crate::QSequentialAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QSequentialAnimationGroup_ptr(
            ptr.as_raw_ptr() as *mut crate::QAnimationGroup,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSequentialAnimationGroup* static_cast<QSequentialAnimationGroup*>(QAnimationGroup* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAnimationGroup>,
    ) -> ::cpp_utils::MutPtr<crate::QSequentialAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QSequentialAnimationGroup_ptr(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QAnimationGroup> for crate::QSequentialAnimationGroup {
    /// Calls C++ function: <span style='color: green;'>```QAnimationGroup* static_cast<QAnimationGroup*>(QSequentialAnimationGroup* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QSequentialAnimationGroup>,
    ) -> ::cpp_utils::Ptr<crate::QAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAnimationGroup_ptr3(
            ptr.as_raw_ptr() as *mut crate::QSequentialAnimationGroup,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAnimationGroup* static_cast<QAnimationGroup*>(QSequentialAnimationGroup* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QSequentialAnimationGroup>,
    ) -> ::cpp_utils::MutPtr<crate::QAnimationGroup> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAnimationGroup_ptr3(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QSequentialAnimationGroup {
    type Target = crate::QAnimationGroup;
    /// Calls C++ function: <span style='color: green;'>```QAnimationGroup* static_cast<QAnimationGroup*>(QSequentialAnimationGroup* ptr)```</span>.
    fn deref(&self) -> &crate::QAnimationGroup {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAnimationGroup_ptr3(
                self as *const crate::QSequentialAnimationGroup
                    as *mut crate::QSequentialAnimationGroup,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QSequentialAnimationGroup {
    /// Calls C++ function: <span style='color: green;'>```QAnimationGroup* static_cast<QAnimationGroup*>(QSequentialAnimationGroup* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QAnimationGroup {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAnimationGroup_ptr3(
                self as *mut crate::QSequentialAnimationGroup,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QSequentialAnimationGroup> for crate::QAnimationGroup {
    /// Calls C++ function: <span style='color: green;'>```QSequentialAnimationGroup* dynamic_cast<QSequentialAnimationGroup*>(QAnimationGroup* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QAnimationGroup>,
    ) -> ::cpp_utils::Ptr<crate::QSequentialAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSequentialAnimationGroup_ptr(
            ptr.as_raw_ptr() as *mut crate::QAnimationGroup,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSequentialAnimationGroup* dynamic_cast<QSequentialAnimationGroup*>(QAnimationGroup* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAnimationGroup>,
    ) -> ::cpp_utils::MutPtr<crate::QSequentialAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSequentialAnimationGroup_ptr(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QSequentialAnimationGroup> for crate::QAbstractAnimation {
    /// Calls C++ function: <span style='color: green;'>```QSequentialAnimationGroup* static_cast<QSequentialAnimationGroup*>(QAbstractAnimation* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractAnimation>,
    ) -> ::cpp_utils::Ptr<crate::QSequentialAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QSequentialAnimationGroup_ptr1(
            ptr.as_raw_ptr() as *mut crate::QAbstractAnimation,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSequentialAnimationGroup* static_cast<QSequentialAnimationGroup*>(QAbstractAnimation* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractAnimation>,
    ) -> ::cpp_utils::MutPtr<crate::QSequentialAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QSequentialAnimationGroup_ptr1(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QAbstractAnimation> for crate::QSequentialAnimationGroup {
    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* static_cast<QAbstractAnimation*>(QSequentialAnimationGroup* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QSequentialAnimationGroup>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractAnimation_ptr6(
            ptr.as_raw_ptr() as *mut crate::QSequentialAnimationGroup,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractAnimation* static_cast<QAbstractAnimation*>(QSequentialAnimationGroup* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QSequentialAnimationGroup>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractAnimation> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractAnimation_ptr6(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QSequentialAnimationGroup> for crate::QAbstractAnimation {
    /// Calls C++ function: <span style='color: green;'>```QSequentialAnimationGroup* dynamic_cast<QSequentialAnimationGroup*>(QAbstractAnimation* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractAnimation>,
    ) -> ::cpp_utils::Ptr<crate::QSequentialAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSequentialAnimationGroup_ptr1(
            ptr.as_raw_ptr() as *mut crate::QAbstractAnimation,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSequentialAnimationGroup* dynamic_cast<QSequentialAnimationGroup*>(QAbstractAnimation* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractAnimation>,
    ) -> ::cpp_utils::MutPtr<crate::QSequentialAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSequentialAnimationGroup_ptr1(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QSequentialAnimationGroup> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QSequentialAnimationGroup* static_cast<QSequentialAnimationGroup*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QSequentialAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QSequentialAnimationGroup_ptr2(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSequentialAnimationGroup* static_cast<QSequentialAnimationGroup*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QSequentialAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QSequentialAnimationGroup_ptr2(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QSequentialAnimationGroup {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QSequentialAnimationGroup* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QSequentialAnimationGroup>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr34(
            ptr.as_raw_ptr() as *mut crate::QSequentialAnimationGroup
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QSequentialAnimationGroup* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QSequentialAnimationGroup>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr34(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QSequentialAnimationGroup> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QSequentialAnimationGroup* dynamic_cast<QSequentialAnimationGroup*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QSequentialAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSequentialAnimationGroup_ptr2(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSequentialAnimationGroup* dynamic_cast<QSequentialAnimationGroup*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QSequentialAnimationGroup> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSequentialAnimationGroup_ptr2(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QSettings> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QSettings* static_cast<QSettings*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QSettings> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QSettings_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSettings* static_cast<QSettings*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QSettings> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QSettings_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QSettings {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QSettings* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QSettings>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr35(
            ptr.as_raw_ptr() as *mut crate::QSettings
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QSettings* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QSettings>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr35(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QSettings {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QSettings* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr35(
                self as *const crate::QSettings as *mut crate::QSettings,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QSettings {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QSettings* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr35(self as *mut crate::QSettings)
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QSettings> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QSettings* dynamic_cast<QSettings*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QSettings> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSettings_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSettings* dynamic_cast<QSettings*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QSettings> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSettings_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QSharedMemory> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QSharedMemory* static_cast<QSharedMemory*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QSharedMemory> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QSharedMemory_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSharedMemory* static_cast<QSharedMemory*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QSharedMemory> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QSharedMemory_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QSharedMemory {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QSharedMemory* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QSharedMemory>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr36(
            ptr.as_raw_ptr() as *mut crate::QSharedMemory
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QSharedMemory* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QSharedMemory>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr36(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QSharedMemory {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QSharedMemory* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr36(
                self as *const crate::QSharedMemory as *mut crate::QSharedMemory,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QSharedMemory {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QSharedMemory* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr36(
                self as *mut crate::QSharedMemory,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QSharedMemory> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QSharedMemory* dynamic_cast<QSharedMemory*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QSharedMemory> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSharedMemory_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSharedMemory* dynamic_cast<QSharedMemory*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QSharedMemory> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSharedMemory_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QSignalMapper> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QSignalMapper* static_cast<QSignalMapper*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QSignalMapper> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QSignalMapper_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSignalMapper* static_cast<QSignalMapper*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QSignalMapper> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QSignalMapper_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QSignalMapper {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QSignalMapper* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QSignalMapper>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr37(
            ptr.as_raw_ptr() as *mut crate::QSignalMapper
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QSignalMapper* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QSignalMapper>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr37(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QSignalMapper {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QSignalMapper* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr37(
                self as *const crate::QSignalMapper as *mut crate::QSignalMapper,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QSignalMapper {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QSignalMapper* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr37(
                self as *mut crate::QSignalMapper,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QSignalMapper> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QSignalMapper* dynamic_cast<QSignalMapper*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QSignalMapper> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSignalMapper_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSignalMapper* dynamic_cast<QSignalMapper*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QSignalMapper> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSignalMapper_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QSignalTransition> for crate::QAbstractTransition {
    /// Calls C++ function: <span style='color: green;'>```QSignalTransition* static_cast<QSignalTransition*>(QAbstractTransition* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractTransition>,
    ) -> ::cpp_utils::Ptr<crate::QSignalTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QSignalTransition_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractTransition,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSignalTransition* static_cast<QSignalTransition*>(QAbstractTransition* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractTransition>,
    ) -> ::cpp_utils::MutPtr<crate::QSignalTransition> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QSignalTransition_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QAbstractTransition> for crate::QSignalTransition {
    /// Calls C++ function: <span style='color: green;'>```QAbstractTransition* static_cast<QAbstractTransition*>(QSignalTransition* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QSignalTransition>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractTransition_ptr2(
            ptr.as_raw_ptr() as *mut crate::QSignalTransition,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractTransition* static_cast<QAbstractTransition*>(QSignalTransition* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QSignalTransition>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractTransition_ptr2(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QSignalTransition {
    type Target = crate::QAbstractTransition;
    /// Calls C++ function: <span style='color: green;'>```QAbstractTransition* static_cast<QAbstractTransition*>(QSignalTransition* ptr)```</span>.
    fn deref(&self) -> &crate::QAbstractTransition {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractTransition_ptr2(
                self as *const crate::QSignalTransition as *mut crate::QSignalTransition,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QSignalTransition {
    /// Calls C++ function: <span style='color: green;'>```QAbstractTransition* static_cast<QAbstractTransition*>(QSignalTransition* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QAbstractTransition {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractTransition_ptr2(
                self as *mut crate::QSignalTransition,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QSignalTransition> for crate::QAbstractTransition {
    /// Calls C++ function: <span style='color: green;'>```QSignalTransition* dynamic_cast<QSignalTransition*>(QAbstractTransition* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractTransition>,
    ) -> ::cpp_utils::Ptr<crate::QSignalTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSignalTransition_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractTransition,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSignalTransition* dynamic_cast<QSignalTransition*>(QAbstractTransition* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractTransition>,
    ) -> ::cpp_utils::MutPtr<crate::QSignalTransition> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSignalTransition_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QSignalTransition> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QSignalTransition* static_cast<QSignalTransition*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QSignalTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QSignalTransition_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSignalTransition* static_cast<QSignalTransition*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QSignalTransition> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QSignalTransition_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QSignalTransition {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QSignalTransition* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QSignalTransition>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr38(
            ptr.as_raw_ptr() as *mut crate::QSignalTransition
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QSignalTransition* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QSignalTransition>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr38(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QSignalTransition> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QSignalTransition* dynamic_cast<QSignalTransition*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QSignalTransition> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSignalTransition_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSignalTransition* dynamic_cast<QSignalTransition*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QSignalTransition> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSignalTransition_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QSocketNotifier> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QSocketNotifier* static_cast<QSocketNotifier*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QSocketNotifier> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QSocketNotifier_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSocketNotifier* static_cast<QSocketNotifier*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QSocketNotifier> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QSocketNotifier_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QSocketNotifier {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QSocketNotifier* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QSocketNotifier>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr39(
            ptr.as_raw_ptr() as *mut crate::QSocketNotifier
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QSocketNotifier* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QSocketNotifier>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr39(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QSocketNotifier {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QSocketNotifier* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr39(
                self as *const crate::QSocketNotifier as *mut crate::QSocketNotifier,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QSocketNotifier {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QSocketNotifier* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr39(
                self as *mut crate::QSocketNotifier,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QSocketNotifier> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QSocketNotifier* dynamic_cast<QSocketNotifier*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QSocketNotifier> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSocketNotifier_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSocketNotifier* dynamic_cast<QSocketNotifier*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QSocketNotifier> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSocketNotifier_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QSortFilterProxyModel> for crate::QAbstractProxyModel {
    /// Calls C++ function: <span style='color: green;'>```QSortFilterProxyModel* static_cast<QSortFilterProxyModel*>(QAbstractProxyModel* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractProxyModel>,
    ) -> ::cpp_utils::Ptr<crate::QSortFilterProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QSortFilterProxyModel_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractProxyModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSortFilterProxyModel* static_cast<QSortFilterProxyModel*>(QAbstractProxyModel* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractProxyModel>,
    ) -> ::cpp_utils::MutPtr<crate::QSortFilterProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QSortFilterProxyModel_ptr(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QAbstractProxyModel> for crate::QSortFilterProxyModel {
    /// Calls C++ function: <span style='color: green;'>```QAbstractProxyModel* static_cast<QAbstractProxyModel*>(QSortFilterProxyModel* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QSortFilterProxyModel>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractProxyModel_ptr3(
            ptr.as_raw_ptr() as *mut crate::QSortFilterProxyModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractProxyModel* static_cast<QAbstractProxyModel*>(QSortFilterProxyModel* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QSortFilterProxyModel>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractProxyModel_ptr3(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QSortFilterProxyModel {
    type Target = crate::QAbstractProxyModel;
    /// Calls C++ function: <span style='color: green;'>```QAbstractProxyModel* static_cast<QAbstractProxyModel*>(QSortFilterProxyModel* ptr)```</span>.
    fn deref(&self) -> &crate::QAbstractProxyModel {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractProxyModel_ptr3(
                self as *const crate::QSortFilterProxyModel as *mut crate::QSortFilterProxyModel,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QSortFilterProxyModel {
    /// Calls C++ function: <span style='color: green;'>```QAbstractProxyModel* static_cast<QAbstractProxyModel*>(QSortFilterProxyModel* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QAbstractProxyModel {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractProxyModel_ptr3(
                self as *mut crate::QSortFilterProxyModel,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QSortFilterProxyModel> for crate::QAbstractProxyModel {
    /// Calls C++ function: <span style='color: green;'>```QSortFilterProxyModel* dynamic_cast<QSortFilterProxyModel*>(QAbstractProxyModel* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractProxyModel>,
    ) -> ::cpp_utils::Ptr<crate::QSortFilterProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSortFilterProxyModel_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractProxyModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSortFilterProxyModel* dynamic_cast<QSortFilterProxyModel*>(QAbstractProxyModel* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractProxyModel>,
    ) -> ::cpp_utils::MutPtr<crate::QSortFilterProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSortFilterProxyModel_ptr(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QSortFilterProxyModel> for crate::QAbstractItemModel {
    /// Calls C++ function: <span style='color: green;'>```QSortFilterProxyModel* static_cast<QSortFilterProxyModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::Ptr<crate::QSortFilterProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QSortFilterProxyModel_ptr1(
            ptr.as_raw_ptr() as *mut crate::QAbstractItemModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSortFilterProxyModel* static_cast<QSortFilterProxyModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::MutPtr<crate::QSortFilterProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QSortFilterProxyModel_ptr1(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QAbstractItemModel> for crate::QSortFilterProxyModel {
    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* static_cast<QAbstractItemModel*>(QSortFilterProxyModel* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QSortFilterProxyModel>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractItemModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractItemModel_ptr5(
            ptr.as_raw_ptr() as *mut crate::QSortFilterProxyModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* static_cast<QAbstractItemModel*>(QSortFilterProxyModel* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QSortFilterProxyModel>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractItemModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractItemModel_ptr5(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QSortFilterProxyModel> for crate::QAbstractItemModel {
    /// Calls C++ function: <span style='color: green;'>```QSortFilterProxyModel* dynamic_cast<QSortFilterProxyModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::Ptr<crate::QSortFilterProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSortFilterProxyModel_ptr1(
            ptr.as_raw_ptr() as *mut crate::QAbstractItemModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSortFilterProxyModel* dynamic_cast<QSortFilterProxyModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::MutPtr<crate::QSortFilterProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSortFilterProxyModel_ptr1(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QSortFilterProxyModel> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QSortFilterProxyModel* static_cast<QSortFilterProxyModel*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QSortFilterProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QSortFilterProxyModel_ptr2(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSortFilterProxyModel* static_cast<QSortFilterProxyModel*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QSortFilterProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QSortFilterProxyModel_ptr2(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QSortFilterProxyModel {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QSortFilterProxyModel* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QSortFilterProxyModel>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr40(
            ptr.as_raw_ptr() as *mut crate::QSortFilterProxyModel
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QSortFilterProxyModel* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QSortFilterProxyModel>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr40(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QSortFilterProxyModel> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QSortFilterProxyModel* dynamic_cast<QSortFilterProxyModel*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QSortFilterProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSortFilterProxyModel_ptr2(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QSortFilterProxyModel* dynamic_cast<QSortFilterProxyModel*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QSortFilterProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QSortFilterProxyModel_ptr2(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QState> for crate::QAbstractState {
    /// Calls C++ function: <span style='color: green;'>```QState* static_cast<QState*>(QAbstractState* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractState>,
    ) -> ::cpp_utils::Ptr<crate::QState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QState_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractState
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QState* static_cast<QState*>(QAbstractState* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractState>,
    ) -> ::cpp_utils::MutPtr<crate::QState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QState_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QAbstractState> for crate::QState {
    /// Calls C++ function: <span style='color: green;'>```QAbstractState* static_cast<QAbstractState*>(QState* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QState>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractState_ptr3(
            ptr.as_raw_ptr() as *mut crate::QState,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractState* static_cast<QAbstractState*>(QState* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QState>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractState> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractState_ptr3(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QState {
    type Target = crate::QAbstractState;
    /// Calls C++ function: <span style='color: green;'>```QAbstractState* static_cast<QAbstractState*>(QState* ptr)```</span>.
    fn deref(&self) -> &crate::QAbstractState {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractState_ptr3(
                self as *const crate::QState as *mut crate::QState,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QState {
    /// Calls C++ function: <span style='color: green;'>```QAbstractState* static_cast<QAbstractState*>(QState* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QAbstractState {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractState_ptr3(
                self as *mut crate::QState,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QState> for crate::QAbstractState {
    /// Calls C++ function: <span style='color: green;'>```QState* dynamic_cast<QState*>(QAbstractState* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractState>,
    ) -> ::cpp_utils::Ptr<crate::QState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QState_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractState
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QState* dynamic_cast<QState*>(QAbstractState* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractState>,
    ) -> ::cpp_utils::MutPtr<crate::QState> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QState_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QState> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QState* static_cast<QState*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QState_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QState* static_cast<QState*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QState> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QState_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QState {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QState* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QState>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr41(
            ptr.as_raw_ptr() as *mut crate::QState
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QState* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QState>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr41(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QState> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QState* dynamic_cast<QState*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QState_ptr1(
            ptr.as_raw_ptr() as *mut crate::QObject
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QState* dynamic_cast<QState*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QState> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QState_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QStateMachine> for crate::QState {
    /// Calls C++ function: <span style='color: green;'>```QStateMachine* static_cast<QStateMachine*>(QState* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QState>,
    ) -> ::cpp_utils::Ptr<crate::QStateMachine> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QStateMachine_ptr(
            ptr.as_raw_ptr() as *mut crate::QState,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QStateMachine* static_cast<QStateMachine*>(QState* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QState>,
    ) -> ::cpp_utils::MutPtr<crate::QStateMachine> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QStateMachine_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QState> for crate::QStateMachine {
    /// Calls C++ function: <span style='color: green;'>```QState* static_cast<QState*>(QStateMachine* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QStateMachine>,
    ) -> ::cpp_utils::Ptr<crate::QState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QState_ptr2(
            ptr.as_raw_ptr() as *mut crate::QStateMachine
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QState* static_cast<QState*>(QStateMachine* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QStateMachine>,
    ) -> ::cpp_utils::MutPtr<crate::QState> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QState_ptr2(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QStateMachine {
    type Target = crate::QState;
    /// Calls C++ function: <span style='color: green;'>```QState* static_cast<QState*>(QStateMachine* ptr)```</span>.
    fn deref(&self) -> &crate::QState {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QState_ptr2(
                self as *const crate::QStateMachine as *mut crate::QStateMachine,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QStateMachine {
    /// Calls C++ function: <span style='color: green;'>```QState* static_cast<QState*>(QStateMachine* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QState {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QState_ptr2(self as *mut crate::QStateMachine)
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QStateMachine> for crate::QState {
    /// Calls C++ function: <span style='color: green;'>```QStateMachine* dynamic_cast<QStateMachine*>(QState* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QState>,
    ) -> ::cpp_utils::Ptr<crate::QStateMachine> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QStateMachine_ptr(
            ptr.as_raw_ptr() as *mut crate::QState,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QStateMachine* dynamic_cast<QStateMachine*>(QState* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QState>,
    ) -> ::cpp_utils::MutPtr<crate::QStateMachine> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QStateMachine_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QStateMachine> for crate::QAbstractState {
    /// Calls C++ function: <span style='color: green;'>```QStateMachine* static_cast<QStateMachine*>(QAbstractState* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractState>,
    ) -> ::cpp_utils::Ptr<crate::QStateMachine> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QStateMachine_ptr1(
            ptr.as_raw_ptr() as *mut crate::QAbstractState,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QStateMachine* static_cast<QStateMachine*>(QAbstractState* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractState>,
    ) -> ::cpp_utils::MutPtr<crate::QStateMachine> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QStateMachine_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QAbstractState> for crate::QStateMachine {
    /// Calls C++ function: <span style='color: green;'>```QAbstractState* static_cast<QAbstractState*>(QStateMachine* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QStateMachine>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractState_ptr4(
            ptr.as_raw_ptr() as *mut crate::QStateMachine,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractState* static_cast<QAbstractState*>(QStateMachine* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QStateMachine>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractState> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractState_ptr4(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QStateMachine> for crate::QAbstractState {
    /// Calls C++ function: <span style='color: green;'>```QStateMachine* dynamic_cast<QStateMachine*>(QAbstractState* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractState>,
    ) -> ::cpp_utils::Ptr<crate::QStateMachine> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QStateMachine_ptr1(
            ptr.as_raw_ptr() as *mut crate::QAbstractState,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QStateMachine* dynamic_cast<QStateMachine*>(QAbstractState* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractState>,
    ) -> ::cpp_utils::MutPtr<crate::QStateMachine> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QStateMachine_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QStateMachine> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QStateMachine* static_cast<QStateMachine*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QStateMachine> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QStateMachine_ptr2(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QStateMachine* static_cast<QStateMachine*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QStateMachine> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QStateMachine_ptr2(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QStateMachine {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QStateMachine* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QStateMachine>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr42(
            ptr.as_raw_ptr() as *mut crate::QStateMachine
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QStateMachine* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QStateMachine>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr42(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QStateMachine> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QStateMachine* dynamic_cast<QStateMachine*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QStateMachine> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QStateMachine_ptr2(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QStateMachine* dynamic_cast<QStateMachine*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QStateMachine> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QStateMachine_ptr2(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QStringListModel> for crate::QAbstractListModel {
    /// Calls C++ function: <span style='color: green;'>```QStringListModel* static_cast<QStringListModel*>(QAbstractListModel* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractListModel>,
    ) -> ::cpp_utils::Ptr<crate::QStringListModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QStringListModel_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractListModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QStringListModel* static_cast<QStringListModel*>(QAbstractListModel* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractListModel>,
    ) -> ::cpp_utils::MutPtr<crate::QStringListModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QStringListModel_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QAbstractListModel> for crate::QStringListModel {
    /// Calls C++ function: <span style='color: green;'>```QAbstractListModel* static_cast<QAbstractListModel*>(QStringListModel* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QStringListModel>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractListModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractListModel_ptr2(
            ptr.as_raw_ptr() as *mut crate::QStringListModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractListModel* static_cast<QAbstractListModel*>(QStringListModel* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QStringListModel>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractListModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractListModel_ptr2(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QStringListModel {
    type Target = crate::QAbstractListModel;
    /// Calls C++ function: <span style='color: green;'>```QAbstractListModel* static_cast<QAbstractListModel*>(QStringListModel* ptr)```</span>.
    fn deref(&self) -> &crate::QAbstractListModel {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractListModel_ptr2(
                self as *const crate::QStringListModel as *mut crate::QStringListModel,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QStringListModel {
    /// Calls C++ function: <span style='color: green;'>```QAbstractListModel* static_cast<QAbstractListModel*>(QStringListModel* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QAbstractListModel {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractListModel_ptr2(
                self as *mut crate::QStringListModel,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QStringListModel> for crate::QAbstractListModel {
    /// Calls C++ function: <span style='color: green;'>```QStringListModel* dynamic_cast<QStringListModel*>(QAbstractListModel* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractListModel>,
    ) -> ::cpp_utils::Ptr<crate::QStringListModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QStringListModel_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractListModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QStringListModel* dynamic_cast<QStringListModel*>(QAbstractListModel* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractListModel>,
    ) -> ::cpp_utils::MutPtr<crate::QStringListModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QStringListModel_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QStringListModel> for crate::QAbstractItemModel {
    /// Calls C++ function: <span style='color: green;'>```QStringListModel* static_cast<QStringListModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::Ptr<crate::QStringListModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QStringListModel_ptr1(
            ptr.as_raw_ptr() as *mut crate::QAbstractItemModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QStringListModel* static_cast<QStringListModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::MutPtr<crate::QStringListModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QStringListModel_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QAbstractItemModel> for crate::QStringListModel {
    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* static_cast<QAbstractItemModel*>(QStringListModel* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QStringListModel>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractItemModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractItemModel_ptr6(
            ptr.as_raw_ptr() as *mut crate::QStringListModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* static_cast<QAbstractItemModel*>(QStringListModel* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QStringListModel>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractItemModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractItemModel_ptr6(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QStringListModel> for crate::QAbstractItemModel {
    /// Calls C++ function: <span style='color: green;'>```QStringListModel* dynamic_cast<QStringListModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::Ptr<crate::QStringListModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QStringListModel_ptr1(
            ptr.as_raw_ptr() as *mut crate::QAbstractItemModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QStringListModel* dynamic_cast<QStringListModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::MutPtr<crate::QStringListModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QStringListModel_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QStringListModel> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QStringListModel* static_cast<QStringListModel*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QStringListModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QStringListModel_ptr2(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QStringListModel* static_cast<QStringListModel*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QStringListModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QStringListModel_ptr2(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QStringListModel {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QStringListModel* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QStringListModel>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr43(
            ptr.as_raw_ptr() as *mut crate::QStringListModel
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QStringListModel* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QStringListModel>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr43(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QStringListModel> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QStringListModel* dynamic_cast<QStringListModel*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QStringListModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QStringListModel_ptr2(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QStringListModel* dynamic_cast<QStringListModel*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QStringListModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QStringListModel_ptr2(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QTemporaryFile> for crate::QFile {
    /// Calls C++ function: <span style='color: green;'>```QTemporaryFile* static_cast<QTemporaryFile*>(QFile* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QFile>,
    ) -> ::cpp_utils::Ptr<crate::QTemporaryFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QTemporaryFile_ptr(
            ptr.as_raw_ptr() as *mut crate::QFile,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QTemporaryFile* static_cast<QTemporaryFile*>(QFile* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QFile>,
    ) -> ::cpp_utils::MutPtr<crate::QTemporaryFile> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QTemporaryFile_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QFile> for crate::QTemporaryFile {
    /// Calls C++ function: <span style='color: green;'>```QFile* static_cast<QFile*>(QTemporaryFile* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QTemporaryFile>,
    ) -> ::cpp_utils::Ptr<crate::QFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QFile_ptr3(
            ptr.as_raw_ptr() as *mut crate::QTemporaryFile
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QFile* static_cast<QFile*>(QTemporaryFile* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QTemporaryFile>,
    ) -> ::cpp_utils::MutPtr<crate::QFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QFile_ptr3(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QTemporaryFile {
    type Target = crate::QFile;
    /// Calls C++ function: <span style='color: green;'>```QFile* static_cast<QFile*>(QTemporaryFile* ptr)```</span>.
    fn deref(&self) -> &crate::QFile {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QFile_ptr3(
                self as *const crate::QTemporaryFile as *mut crate::QTemporaryFile,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QTemporaryFile {
    /// Calls C++ function: <span style='color: green;'>```QFile* static_cast<QFile*>(QTemporaryFile* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QFile {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QFile_ptr3(self as *mut crate::QTemporaryFile)
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QTemporaryFile> for crate::QFile {
    /// Calls C++ function: <span style='color: green;'>```QTemporaryFile* dynamic_cast<QTemporaryFile*>(QFile* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QFile>,
    ) -> ::cpp_utils::Ptr<crate::QTemporaryFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QTemporaryFile_ptr(
            ptr.as_raw_ptr() as *mut crate::QFile,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QTemporaryFile* dynamic_cast<QTemporaryFile*>(QFile* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QFile>,
    ) -> ::cpp_utils::MutPtr<crate::QTemporaryFile> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QTemporaryFile_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QTemporaryFile> for crate::QFileDevice {
    /// Calls C++ function: <span style='color: green;'>```QTemporaryFile* static_cast<QTemporaryFile*>(QFileDevice* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QFileDevice>,
    ) -> ::cpp_utils::Ptr<crate::QTemporaryFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QTemporaryFile_ptr1(
            ptr.as_raw_ptr() as *mut crate::QFileDevice,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QTemporaryFile* static_cast<QTemporaryFile*>(QFileDevice* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QFileDevice>,
    ) -> ::cpp_utils::MutPtr<crate::QTemporaryFile> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QTemporaryFile_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QFileDevice> for crate::QTemporaryFile {
    /// Calls C++ function: <span style='color: green;'>```QFileDevice* static_cast<QFileDevice*>(QTemporaryFile* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QTemporaryFile>,
    ) -> ::cpp_utils::Ptr<crate::QFileDevice> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QFileDevice_ptr4(
            ptr.as_raw_ptr() as *mut crate::QTemporaryFile,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QFileDevice* static_cast<QFileDevice*>(QTemporaryFile* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QTemporaryFile>,
    ) -> ::cpp_utils::MutPtr<crate::QFileDevice> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QFileDevice_ptr4(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QTemporaryFile> for crate::QFileDevice {
    /// Calls C++ function: <span style='color: green;'>```QTemporaryFile* dynamic_cast<QTemporaryFile*>(QFileDevice* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QFileDevice>,
    ) -> ::cpp_utils::Ptr<crate::QTemporaryFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QTemporaryFile_ptr1(
            ptr.as_raw_ptr() as *mut crate::QFileDevice,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QTemporaryFile* dynamic_cast<QTemporaryFile*>(QFileDevice* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QFileDevice>,
    ) -> ::cpp_utils::MutPtr<crate::QTemporaryFile> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QTemporaryFile_ptr1(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QTemporaryFile> for crate::QIODevice {
    /// Calls C++ function: <span style='color: green;'>```QTemporaryFile* static_cast<QTemporaryFile*>(QIODevice* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QIODevice>,
    ) -> ::cpp_utils::Ptr<crate::QTemporaryFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QTemporaryFile_ptr2(
            ptr.as_raw_ptr() as *mut crate::QIODevice,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QTemporaryFile* static_cast<QTemporaryFile*>(QIODevice* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QIODevice>,
    ) -> ::cpp_utils::MutPtr<crate::QTemporaryFile> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QTemporaryFile_ptr2(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QIODevice> for crate::QTemporaryFile {
    /// Calls C++ function: <span style='color: green;'>```QIODevice* static_cast<QIODevice*>(QTemporaryFile* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QTemporaryFile>,
    ) -> ::cpp_utils::Ptr<crate::QIODevice> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QIODevice_ptr6(
            ptr.as_raw_ptr() as *mut crate::QTemporaryFile
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QIODevice* static_cast<QIODevice*>(QTemporaryFile* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QTemporaryFile>,
    ) -> ::cpp_utils::MutPtr<crate::QIODevice> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QIODevice_ptr6(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QTemporaryFile> for crate::QIODevice {
    /// Calls C++ function: <span style='color: green;'>```QTemporaryFile* dynamic_cast<QTemporaryFile*>(QIODevice* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QIODevice>,
    ) -> ::cpp_utils::Ptr<crate::QTemporaryFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QTemporaryFile_ptr2(
            ptr.as_raw_ptr() as *mut crate::QIODevice,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QTemporaryFile* dynamic_cast<QTemporaryFile*>(QIODevice* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QIODevice>,
    ) -> ::cpp_utils::MutPtr<crate::QTemporaryFile> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QTemporaryFile_ptr2(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QTemporaryFile> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QTemporaryFile* static_cast<QTemporaryFile*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QTemporaryFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QTemporaryFile_ptr3(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QTemporaryFile* static_cast<QTemporaryFile*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QTemporaryFile> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QTemporaryFile_ptr3(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QTemporaryFile {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QTemporaryFile* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QTemporaryFile>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr44(
            ptr.as_raw_ptr() as *mut crate::QTemporaryFile
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QTemporaryFile* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QTemporaryFile>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr44(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::DynamicCast<crate::QTemporaryFile> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QTemporaryFile* dynamic_cast<QTemporaryFile*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QTemporaryFile> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QTemporaryFile_ptr3(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QTemporaryFile* dynamic_cast<QTemporaryFile*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QTemporaryFile> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QTemporaryFile_ptr3(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QThread> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QThread* static_cast<QThread*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QThread> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QThread_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QThread* static_cast<QThread*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QThread> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QThread_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QThread {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QThread* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QThread>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr45(
            ptr.as_raw_ptr() as *mut crate::QThread
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QThread* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QThread>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr45(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QThread {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QThread* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr45(
                self as *const crate::QThread as *mut crate::QThread,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QThread {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QThread* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr45(self as *mut crate::QThread)
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QThread> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QThread* dynamic_cast<QThread*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QThread> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QThread_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QThread* dynamic_cast<QThread*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QThread> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QThread_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QThreadPool> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QThreadPool* static_cast<QThreadPool*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QThreadPool> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QThreadPool_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QThreadPool* static_cast<QThreadPool*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QThreadPool> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QThreadPool_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QThreadPool {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QThreadPool* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QThreadPool>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr46(
            ptr.as_raw_ptr() as *mut crate::QThreadPool
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QThreadPool* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QThreadPool>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr46(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QThreadPool {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QThreadPool* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr46(
                self as *const crate::QThreadPool as *mut crate::QThreadPool,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QThreadPool {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QThreadPool* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr46(self as *mut crate::QThreadPool)
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QThreadPool> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QThreadPool* dynamic_cast<QThreadPool*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QThreadPool> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QThreadPool_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QThreadPool* dynamic_cast<QThreadPool*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QThreadPool> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QThreadPool_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QTimeLine> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QTimeLine* static_cast<QTimeLine*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QTimeLine> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QTimeLine_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QTimeLine* static_cast<QTimeLine*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QTimeLine> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QTimeLine_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QTimeLine {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QTimeLine* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QTimeLine>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr47(
            ptr.as_raw_ptr() as *mut crate::QTimeLine
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QTimeLine* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QTimeLine>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr47(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QTimeLine {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QTimeLine* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr47(
                self as *const crate::QTimeLine as *mut crate::QTimeLine,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QTimeLine {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QTimeLine* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr47(self as *mut crate::QTimeLine)
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QTimeLine> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QTimeLine* dynamic_cast<QTimeLine*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QTimeLine> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QTimeLine_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QTimeLine* dynamic_cast<QTimeLine*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QTimeLine> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QTimeLine_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QTimer> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QTimer* static_cast<QTimer*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QTimer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QTimer_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QTimer* static_cast<QTimer*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QTimer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QTimer_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QTimer {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QTimer* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QTimer>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr48(
            ptr.as_raw_ptr() as *mut crate::QTimer
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QTimer* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QTimer>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr48(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QTimer {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QTimer* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr48(
                self as *const crate::QTimer as *mut crate::QTimer,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QTimer {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QTimer* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr48(self as *mut crate::QTimer)
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QTimer> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QTimer* dynamic_cast<QTimer*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QTimer> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QTimer_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QTimer* dynamic_cast<QTimer*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QTimer> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QTimer_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QTranslator> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QTranslator* static_cast<QTranslator*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QTranslator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QTranslator_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QTranslator* static_cast<QTranslator*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QTranslator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QTranslator_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QTranslator {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QTranslator* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QTranslator>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr49(
            ptr.as_raw_ptr() as *mut crate::QTranslator
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QTranslator* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QTranslator>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr49(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QTranslator {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QTranslator* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr49(
                self as *const crate::QTranslator as *mut crate::QTranslator,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QTranslator {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QTranslator* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr49(self as *mut crate::QTranslator)
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::QTranslator> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QTranslator* dynamic_cast<QTranslator*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QTranslator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QTranslator_ptr(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QTranslator* dynamic_cast<QTranslator*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QTranslator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QTranslator_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::QXmlStreamAttributes>
    for crate::QVectorOfQXmlStreamAttribute
{
    /// Calls C++ function: <span style='color: green;'>```QXmlStreamAttributes* static_cast<QXmlStreamAttributes*>(QVector<QXmlStreamAttribute>* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QVectorOfQXmlStreamAttribute>,
    ) -> ::cpp_utils::Ptr<crate::QXmlStreamAttributes> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QXmlStreamAttributes_ptr(
            ptr.as_raw_ptr() as *mut crate::QVectorOfQXmlStreamAttribute,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QXmlStreamAttributes* static_cast<QXmlStreamAttributes*>(QVector<QXmlStreamAttribute>* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QVectorOfQXmlStreamAttribute>,
    ) -> ::cpp_utils::MutPtr<crate::QXmlStreamAttributes> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QXmlStreamAttributes_ptr(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QVectorOfQXmlStreamAttribute>
    for crate::QXmlStreamAttributes
{
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamAttribute>* static_cast<QVector<QXmlStreamAttribute>*>(QXmlStreamAttributes* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QXmlStreamAttributes>,
    ) -> ::cpp_utils::Ptr<crate::QVectorOfQXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QVector_QXmlStreamAttribute_ptr(
            ptr.as_raw_ptr() as *mut crate::QXmlStreamAttributes,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamAttribute>* static_cast<QVector<QXmlStreamAttribute>*>(QXmlStreamAttributes* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QXmlStreamAttributes>,
    ) -> ::cpp_utils::MutPtr<crate::QVectorOfQXmlStreamAttribute> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QVector_QXmlStreamAttribute_ptr(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::QXmlStreamAttributes {
    type Target = crate::QVectorOfQXmlStreamAttribute;
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamAttribute>* static_cast<QVector<QXmlStreamAttribute>*>(QXmlStreamAttributes* ptr)```</span>.
    fn deref(&self) -> &crate::QVectorOfQXmlStreamAttribute {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QVector_QXmlStreamAttribute_ptr(
                self as *const crate::QXmlStreamAttributes as *mut crate::QXmlStreamAttributes,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::QXmlStreamAttributes {
    /// Calls C++ function: <span style='color: green;'>```QVector<QXmlStreamAttribute>* static_cast<QVector<QXmlStreamAttribute>*>(QXmlStreamAttributes* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QVectorOfQXmlStreamAttribute {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QVector_QXmlStreamAttribute_ptr(
                self as *mut crate::QXmlStreamAttributes,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfOrientationIntInt {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int::~ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int_dctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int(self as *mut crate::RawSlotOfOrientationIntInt)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfIntExitStatus {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus::~ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus_dctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus(self as *mut crate::RawSlotOfIntExitStatus)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfQVariant {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref::~ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref_dctr_qt_core_ffi_slot_wrapper_QVariant_const_ref(self as *mut crate::RawSlotOfQVariant)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfState {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State::~ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_dctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State(self as *mut crate::RawSlotOfState)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfIntI64 {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_int_qint64::~ctr_qt_core_ffi_slot_wrapper_int_qint64()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_qint64_dctr_qt_core_ffi_slot_wrapper_int_qint64(self as *mut crate::RawSlotOfIntI64)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref::~ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref_dctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref(self as *mut crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfQListOfQPersistentModelIndex {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref::~ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_dctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref(self as *mut crate::RawSlotOfQListOfQPersistentModelIndex)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfQModelIndexInt {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int::~ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_dctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int(self as *mut crate::RawSlotOfQModelIndexInt)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint::~ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint_dctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint(self as *mut crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfOrientation {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_Qt_Orientation::~ctr_qt_core_ffi_slot_wrapper_Qt_Orientation()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_dctr_qt_core_ffi_slot_wrapper_Qt_Orientation(self as *mut crate::RawSlotOfOrientation)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfProcessState {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState::~ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState_dctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState(self as *mut crate::RawSlotOfProcessState)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfBool {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_bool::~ctr_qt_core_ffi_slot_wrapper_bool()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_bool_dctr_qt_core_ffi_slot_wrapper_bool(self as *mut crate::RawSlotOfBool)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfQModelIndex {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref::~ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_dctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref(self as *mut crate::RawSlotOfQModelIndex)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfQAbstractItemModel {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr::~ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr_dctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr(self as *mut crate::RawSlotOfQAbstractItemModel)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfIntInt {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_int_int::~ctr_qt_core_ffi_slot_wrapper_int_int()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_int_dctr_qt_core_ffi_slot_wrapper_int_int(self as *mut crate::RawSlotOfIntInt)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfI64 {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_qint64::~ctr_qt_core_ffi_slot_wrapper_qint64()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_qint64_dctr_qt_core_ffi_slot_wrapper_qint64(self as *mut crate::RawSlotOfI64)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlot {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_::~ctr_qt_core_ffi_slot_wrapper_()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper__dctr_qt_core_ffi_slot_wrapper_(
            self as *mut crate::RawSlot,
        )
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfQObject {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_QObject_ptr::~ctr_qt_core_ffi_slot_wrapper_QObject_ptr()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QObject_ptr_dctr_qt_core_ffi_slot_wrapper_QObject_ptr(self as *mut crate::RawSlotOfQObject)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfQString {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_QString_const_ref::~ctr_qt_core_ffi_slot_wrapper_QString_const_ref()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QString_const_ref_dctr_qt_core_ffi_slot_wrapper_QString_const_ref(self as *mut crate::RawSlotOfQString)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfDirection {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction::~ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction_dctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction(self as *mut crate::RawSlotOfDirection)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfQModelIndexIntIntQModelIndex {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref::~ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_dctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref(self as *mut crate::RawSlotOfQModelIndexIntIntQModelIndex)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfQModelIndexQModelIndex {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref::~ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_dctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref(self as *mut crate::RawSlotOfQModelIndexQModelIndex)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfInt {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_int::~ctr_qt_core_ffi_slot_wrapper_int()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_int_dctr_qt_core_ffi_slot_wrapper_int(self as *mut crate::RawSlotOfInt)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfQModelIndexIntInt {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int::~ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_dctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int(self as *mut crate::RawSlotOfQModelIndexIntInt)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfQItemSelectionQItemSelection {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref::~ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref_dctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref(self as *mut crate::RawSlotOfQItemSelectionQItemSelection)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfDouble {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_double::~ctr_qt_core_ffi_slot_wrapper_double()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_double_dctr_qt_core_ffi_slot_wrapper_double(self as *mut crate::RawSlotOfDouble)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfState2 {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_QTimeLine_State::~ctr_qt_core_ffi_slot_wrapper_QTimeLine_State()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QTimeLine_State_dctr_qt_core_ffi_slot_wrapper_QTimeLine_State(self as *mut crate::RawSlotOfState2)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfStateState {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State::~ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State_dctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State(self as *mut crate::RawSlotOfStateState)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfProcessError {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError::~ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError_dctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError(self as *mut crate::RawSlotOfProcessError)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfQItemSelection {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref::~ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_dctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref(self as *mut crate::RawSlotOfQItemSelection)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfQAbstractAnimation {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr::~ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr_dctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr(self as *mut crate::RawSlotOfQAbstractAnimation)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfQModelIndexIntIntQModelIndexInt {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int::~ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int_dctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int(self as *mut crate::RawSlotOfQModelIndexIntIntQModelIndexInt)
    }
}

impl ::cpp_utils::CppDeletable for crate::RawSlotOfOrientationInt {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int::~ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_dctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int(self as *mut crate::RawSlotOfOrientationInt)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfOrientationIntInt> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int* static_cast<ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfOrientationIntInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int* static_cast<ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfOrientationIntInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfOrientationIntInt {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfOrientationIntInt>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr100(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfOrientationIntInt
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfOrientationIntInt>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr100(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfOrientationIntInt {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr100(
                self as *const crate::RawSlotOfOrientationIntInt
                    as *mut crate::RawSlotOfOrientationIntInt,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfOrientationIntInt {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr100(
                self as *mut crate::RawSlotOfOrientationIntInt,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfOrientationIntInt> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfOrientationIntInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfOrientationIntInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_int_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfIntExitStatus> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus* static_cast<ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfIntExitStatus> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus* static_cast<ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfIntExitStatus> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfIntExitStatus {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfIntExitStatus>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr101(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfIntExitStatus
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfIntExitStatus>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr101(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfIntExitStatus {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr101(
                self as *const crate::RawSlotOfIntExitStatus as *mut crate::RawSlotOfIntExitStatus,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfIntExitStatus {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr101(
                self as *mut crate::RawSlotOfIntExitStatus,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfIntExitStatus> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfIntExitStatus> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfIntExitStatus> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_int_QProcess_ExitStatus_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfQVariant> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref* static_cast<ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref* static_cast<ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfQVariant {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfQVariant>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr102(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfQVariant
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfQVariant>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr102(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfQVariant {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr102(
                self as *const crate::RawSlotOfQVariant as *mut crate::RawSlotOfQVariant,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfQVariant {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr102(
                self as *mut crate::RawSlotOfQVariant,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfQVariant> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQVariant> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QVariant_const_ref_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfState> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State* static_cast<ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State* static_cast<ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfState {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfState>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr103(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfState
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfState>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr103(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfState {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr103(
                self as *const crate::RawSlotOfState as *mut crate::RawSlotOfState,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfState {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr103(
                self as *mut crate::RawSlotOfState,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfState> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfIntI64> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_int_qint64* static_cast<ctr_qt_core_ffi_slot_wrapper_int_qint64*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfIntI64> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_int_qint64_ptr(
                ptr.as_raw_ptr() as *mut crate::QObject,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_int_qint64* static_cast<ctr_qt_core_ffi_slot_wrapper_int_qint64*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfIntI64> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_int_qint64_ptr(
                ptr.as_mut_raw_ptr(),
            );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfIntI64 {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_int_qint64* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfIntI64>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr104(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfIntI64
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_int_qint64* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfIntI64>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr104(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfIntI64 {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_int_qint64* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr104(
                self as *const crate::RawSlotOfIntI64 as *mut crate::RawSlotOfIntI64,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfIntI64 {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_int_qint64* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr104(
                self as *mut crate::RawSlotOfIntI64,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfIntI64> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_int_qint64* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_int_qint64*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfIntI64> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_int_qint64_ptr(
                ptr.as_raw_ptr() as *mut crate::QObject,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_int_qint64* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_int_qint64*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfIntI64> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_int_qint64_ptr(
                ptr.as_mut_raw_ptr(),
            );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt>
    for crate::QObject
{
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref* static_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref* static_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject>
    for crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt
{
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr105(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr105(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr105(
                self as *const crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt
                    as *mut crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr105(
                self as *mut crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt>
    for crate::QObject
{
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQModelIndexQModelIndexQVectorOfInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_QVector_int_const_ref_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfQListOfQPersistentModelIndex> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref* static_cast<ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQListOfQPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref* static_cast<ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQListOfQPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfQListOfQPersistentModelIndex {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfQListOfQPersistentModelIndex>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr106(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfQListOfQPersistentModelIndex
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfQListOfQPersistentModelIndex>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr106(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfQListOfQPersistentModelIndex {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr106(
                self as *const crate::RawSlotOfQListOfQPersistentModelIndex
                    as *mut crate::RawSlotOfQListOfQPersistentModelIndex,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfQListOfQPersistentModelIndex {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr106(
                self as *mut crate::RawSlotOfQListOfQPersistentModelIndex,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfQListOfQPersistentModelIndex> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQListOfQPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQListOfQPersistentModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfQModelIndexInt> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int* static_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQModelIndexInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int* static_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQModelIndexInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfQModelIndexInt {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfQModelIndexInt>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr107(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfQModelIndexInt
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfQModelIndexInt>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr107(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfQModelIndexInt {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr107(
                self as *const crate::RawSlotOfQModelIndexInt
                    as *mut crate::RawSlotOfQModelIndexInt,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfQModelIndexInt {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr107(
                self as *mut crate::RawSlotOfQModelIndexInt,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfQModelIndexInt> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQModelIndexInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQModelIndexInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint>
    for crate::QObject
{
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint* static_cast<ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint* static_cast<ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject>
    for crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint
{
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr108(ptr.as_raw_ptr()
                as *mut crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr108(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr108(
                self as *const crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint
                    as *mut crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr108(
                self as *mut crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint>
    for crate::QObject
{
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQListOfQPersistentModelIndexLayoutChangeHint> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QList_QPersistentModelIndex_const_ref_QAbstractItemModel_LayoutChangeHint_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfOrientation> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_Qt_Orientation* static_cast<ctr_qt_core_ffi_slot_wrapper_Qt_Orientation*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfOrientation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_Qt_Orientation* static_cast<ctr_qt_core_ffi_slot_wrapper_Qt_Orientation*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfOrientation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfOrientation {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_Qt_Orientation* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfOrientation>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr109(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfOrientation
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_Qt_Orientation* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfOrientation>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr109(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfOrientation {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_Qt_Orientation* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr109(
                self as *const crate::RawSlotOfOrientation as *mut crate::RawSlotOfOrientation,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfOrientation {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_Qt_Orientation* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr109(
                self as *mut crate::RawSlotOfOrientation,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfOrientation> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_Qt_Orientation* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_Qt_Orientation*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfOrientation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_Qt_Orientation* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_Qt_Orientation*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfOrientation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfProcessState> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState* static_cast<ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfProcessState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState* static_cast<ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfProcessState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfProcessState {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfProcessState>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr110(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfProcessState
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfProcessState>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr110(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfProcessState {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr110(
                self as *const crate::RawSlotOfProcessState as *mut crate::RawSlotOfProcessState,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfProcessState {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr110(
                self as *mut crate::RawSlotOfProcessState,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfProcessState> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfProcessState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfProcessState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessState_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfBool> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_bool* static_cast<ctr_qt_core_ffi_slot_wrapper_bool*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfBool> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_bool_ptr(
                ptr.as_raw_ptr() as *mut crate::QObject,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_bool* static_cast<ctr_qt_core_ffi_slot_wrapper_bool*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfBool> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_bool_ptr(
                ptr.as_mut_raw_ptr(),
            );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfBool {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_bool* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfBool>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr111(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfBool
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_bool* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfBool>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr111(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfBool {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_bool* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr111(
                self as *const crate::RawSlotOfBool as *mut crate::RawSlotOfBool,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfBool {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_bool* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr111(
                self as *mut crate::RawSlotOfBool,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfBool> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_bool* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_bool*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfBool> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_bool_ptr(
                ptr.as_raw_ptr() as *mut crate::QObject,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_bool* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_bool*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfBool> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_bool_ptr(
                ptr.as_mut_raw_ptr(),
            );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfQModelIndex> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref* static_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref* static_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfQModelIndex {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfQModelIndex>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr112(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfQModelIndex
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfQModelIndex>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr112(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfQModelIndex {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr112(
                self as *const crate::RawSlotOfQModelIndex as *mut crate::RawSlotOfQModelIndex,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfQModelIndex {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr112(
                self as *mut crate::RawSlotOfQModelIndex,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfQModelIndex> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfQAbstractItemModel> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr* static_cast<ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQAbstractItemModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr* static_cast<ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQAbstractItemModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfQAbstractItemModel {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfQAbstractItemModel>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr113(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfQAbstractItemModel
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfQAbstractItemModel>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr113(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfQAbstractItemModel {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr113(
                self as *const crate::RawSlotOfQAbstractItemModel
                    as *mut crate::RawSlotOfQAbstractItemModel,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfQAbstractItemModel {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr113(
                self as *mut crate::RawSlotOfQAbstractItemModel,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfQAbstractItemModel> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQAbstractItemModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQAbstractItemModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QAbstractItemModel_ptr_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfIntInt> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_int_int* static_cast<ctr_qt_core_ffi_slot_wrapper_int_int*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfIntInt> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_int_int_ptr(
                ptr.as_raw_ptr() as *mut crate::QObject,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_int_int* static_cast<ctr_qt_core_ffi_slot_wrapper_int_int*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfIntInt> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_int_int_ptr(
                ptr.as_mut_raw_ptr(),
            );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfIntInt {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_int_int* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfIntInt>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr114(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfIntInt
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_int_int* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfIntInt>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr114(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfIntInt {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_int_int* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr114(
                self as *const crate::RawSlotOfIntInt as *mut crate::RawSlotOfIntInt,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfIntInt {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_int_int* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr114(
                self as *mut crate::RawSlotOfIntInt,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfIntInt> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_int_int* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_int_int*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfIntInt> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_int_int_ptr(
                ptr.as_raw_ptr() as *mut crate::QObject,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_int_int* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_int_int*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfIntInt> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_int_int_ptr(
                ptr.as_mut_raw_ptr(),
            );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfI64> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_qint64* static_cast<ctr_qt_core_ffi_slot_wrapper_qint64*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfI64> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_qint64_ptr(
                ptr.as_raw_ptr() as *mut crate::QObject,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_qint64* static_cast<ctr_qt_core_ffi_slot_wrapper_qint64*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfI64> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_qint64_ptr(
                ptr.as_mut_raw_ptr(),
            );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfI64 {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_qint64* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfI64>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr115(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfI64
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_qint64* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfI64>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr115(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfI64 {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_qint64* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr115(
                self as *const crate::RawSlotOfI64 as *mut crate::RawSlotOfI64,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfI64 {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_qint64* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr115(
                self as *mut crate::RawSlotOfI64,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfI64> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_qint64* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_qint64*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfI64> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_qint64_ptr(
                ptr.as_raw_ptr() as *mut crate::QObject,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_qint64* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_qint64*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfI64> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_qint64_ptr(
                ptr.as_mut_raw_ptr(),
            );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlot> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_* static_cast<ctr_qt_core_ffi_slot_wrapper_*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlot> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper__ptr(
                ptr.as_raw_ptr() as *mut crate::QObject,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_* static_cast<ctr_qt_core_ffi_slot_wrapper_*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlot> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper__ptr(
                ptr.as_mut_raw_ptr(),
            );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlot {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlot>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr116(
            ptr.as_raw_ptr() as *mut crate::RawSlot
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlot>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr116(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlot {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr116(
                self as *const crate::RawSlot as *mut crate::RawSlot,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlot {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr116(self as *mut crate::RawSlot)
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlot> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlot> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper__ptr(
                ptr.as_raw_ptr() as *mut crate::QObject,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlot> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper__ptr(
                ptr.as_mut_raw_ptr(),
            );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfQObject> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QObject_ptr* static_cast<ctr_qt_core_ffi_slot_wrapper_QObject_ptr*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QObject_ptr_ptr(
                ptr.as_raw_ptr() as *mut crate::QObject,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QObject_ptr* static_cast<ctr_qt_core_ffi_slot_wrapper_QObject_ptr*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QObject_ptr_ptr(
                ptr.as_mut_raw_ptr(),
            );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfQObject {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QObject_ptr* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfQObject>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr117(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfQObject
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QObject_ptr* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfQObject>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr117(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfQObject {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QObject_ptr* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr117(
                self as *const crate::RawSlotOfQObject as *mut crate::RawSlotOfQObject,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfQObject {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QObject_ptr* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr117(
                self as *mut crate::RawSlotOfQObject,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfQObject> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QObject_ptr* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QObject_ptr*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QObject_ptr_ptr(
                ptr.as_raw_ptr() as *mut crate::QObject,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QObject_ptr* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QObject_ptr*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QObject_ptr_ptr(
                ptr.as_mut_raw_ptr(),
            );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfQString> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QString_const_ref* static_cast<ctr_qt_core_ffi_slot_wrapper_QString_const_ref*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QString_const_ref_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QString_const_ref* static_cast<ctr_qt_core_ffi_slot_wrapper_QString_const_ref*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QString_const_ref_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfQString {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QString_const_ref* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfQString>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr118(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfQString
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QString_const_ref* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfQString>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr118(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfQString {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QString_const_ref* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr118(
                self as *const crate::RawSlotOfQString as *mut crate::RawSlotOfQString,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfQString {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QString_const_ref* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr118(
                self as *mut crate::RawSlotOfQString,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfQString> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QString_const_ref* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QString_const_ref*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QString_const_ref_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QString_const_ref* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QString_const_ref*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQString> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QString_const_ref_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfDirection> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction* static_cast<ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfDirection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction* static_cast<ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfDirection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfDirection {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfDirection>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr119(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfDirection
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfDirection>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr119(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfDirection {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr119(
                self as *const crate::RawSlotOfDirection as *mut crate::RawSlotOfDirection,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfDirection {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr119(
                self as *mut crate::RawSlotOfDirection,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfDirection> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfDirection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfDirection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_Direction_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfQModelIndexIntIntQModelIndex> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref* static_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQModelIndexIntIntQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref* static_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQModelIndexIntIntQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfQModelIndexIntIntQModelIndex {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfQModelIndexIntIntQModelIndex>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr120(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfQModelIndexIntIntQModelIndex
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfQModelIndexIntIntQModelIndex>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr120(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfQModelIndexIntIntQModelIndex {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr120(
                self as *const crate::RawSlotOfQModelIndexIntIntQModelIndex
                    as *mut crate::RawSlotOfQModelIndexIntIntQModelIndex,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfQModelIndexIntIntQModelIndex {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr120(
                self as *mut crate::RawSlotOfQModelIndexIntIntQModelIndex,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfQModelIndexIntIntQModelIndex> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQModelIndexIntIntQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQModelIndexIntIntQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfQModelIndexQModelIndex> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref* static_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQModelIndexQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref* static_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQModelIndexQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfQModelIndexQModelIndex {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfQModelIndexQModelIndex>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr121(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfQModelIndexQModelIndex
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfQModelIndexQModelIndex>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr121(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfQModelIndexQModelIndex {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr121(
                self as *const crate::RawSlotOfQModelIndexQModelIndex
                    as *mut crate::RawSlotOfQModelIndexQModelIndex,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfQModelIndexQModelIndex {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr121(
                self as *mut crate::RawSlotOfQModelIndexQModelIndex,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfQModelIndexQModelIndex> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQModelIndexQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQModelIndexQModelIndex> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_QModelIndex_const_ref_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfInt> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_int* static_cast<ctr_qt_core_ffi_slot_wrapper_int*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfInt> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_int_ptr(
                ptr.as_raw_ptr() as *mut crate::QObject,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_int* static_cast<ctr_qt_core_ffi_slot_wrapper_int*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfInt> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_int_ptr(
                ptr.as_mut_raw_ptr(),
            );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfInt {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_int* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfInt>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr122(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfInt
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_int* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfInt>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr122(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfInt {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_int* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr122(
                self as *const crate::RawSlotOfInt as *mut crate::RawSlotOfInt,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfInt {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_int* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr122(
                self as *mut crate::RawSlotOfInt,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfInt> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_int* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_int*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfInt> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_int_ptr(
                ptr.as_raw_ptr() as *mut crate::QObject,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_int* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_int*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfInt> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_int_ptr(
                ptr.as_mut_raw_ptr(),
            );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfQModelIndexIntInt> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int* static_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQModelIndexIntInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int* static_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQModelIndexIntInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfQModelIndexIntInt {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfQModelIndexIntInt>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr123(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfQModelIndexIntInt
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfQModelIndexIntInt>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr123(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfQModelIndexIntInt {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr123(
                self as *const crate::RawSlotOfQModelIndexIntInt
                    as *mut crate::RawSlotOfQModelIndexIntInt,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfQModelIndexIntInt {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr123(
                self as *mut crate::RawSlotOfQModelIndexIntInt,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfQModelIndexIntInt> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQModelIndexIntInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQModelIndexIntInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfQItemSelectionQItemSelection> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref* static_cast<ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQItemSelectionQItemSelection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref* static_cast<ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQItemSelectionQItemSelection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfQItemSelectionQItemSelection {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfQItemSelectionQItemSelection>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr124(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfQItemSelectionQItemSelection
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfQItemSelectionQItemSelection>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr124(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfQItemSelectionQItemSelection {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr124(
                self as *const crate::RawSlotOfQItemSelectionQItemSelection
                    as *mut crate::RawSlotOfQItemSelectionQItemSelection,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfQItemSelectionQItemSelection {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr124(
                self as *mut crate::RawSlotOfQItemSelectionQItemSelection,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfQItemSelectionQItemSelection> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQItemSelectionQItemSelection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQItemSelectionQItemSelection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_QItemSelection_const_ref_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfDouble> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_double* static_cast<ctr_qt_core_ffi_slot_wrapper_double*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfDouble> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_double_ptr(
                ptr.as_raw_ptr() as *mut crate::QObject,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_double* static_cast<ctr_qt_core_ffi_slot_wrapper_double*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfDouble> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_double_ptr(
                ptr.as_mut_raw_ptr(),
            );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfDouble {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_double* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfDouble>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr126(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfDouble
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_double* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfDouble>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr126(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfDouble {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_double* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr126(
                self as *const crate::RawSlotOfDouble as *mut crate::RawSlotOfDouble,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfDouble {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_double* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr126(
                self as *mut crate::RawSlotOfDouble,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfDouble> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_double* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_double*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfDouble> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_double_ptr(
                ptr.as_raw_ptr() as *mut crate::QObject,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_double* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_double*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfDouble> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_double_ptr(
                ptr.as_mut_raw_ptr(),
            );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfState2> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QTimeLine_State* static_cast<ctr_qt_core_ffi_slot_wrapper_QTimeLine_State*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfState2> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QTimeLine_State_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QTimeLine_State* static_cast<ctr_qt_core_ffi_slot_wrapper_QTimeLine_State*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfState2> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QTimeLine_State_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfState2 {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QTimeLine_State* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfState2>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr127(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfState2
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QTimeLine_State* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfState2>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr127(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfState2 {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QTimeLine_State* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr127(
                self as *const crate::RawSlotOfState2 as *mut crate::RawSlotOfState2,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfState2 {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QTimeLine_State* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr127(
                self as *mut crate::RawSlotOfState2,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfState2> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QTimeLine_State* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QTimeLine_State*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfState2> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QTimeLine_State_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QTimeLine_State* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QTimeLine_State*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfState2> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QTimeLine_State_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfStateState> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State* static_cast<ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfStateState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State* static_cast<ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfStateState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfStateState {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfStateState>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr128(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfStateState
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfStateState>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr128(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfStateState {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr128(
                self as *const crate::RawSlotOfStateState as *mut crate::RawSlotOfStateState,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfStateState {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr128(
                self as *mut crate::RawSlotOfStateState,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfStateState> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfStateState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfStateState> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_State_QAbstractAnimation_State_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfProcessError> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError* static_cast<ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfProcessError> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError* static_cast<ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfProcessError> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfProcessError {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfProcessError>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr129(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfProcessError
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfProcessError>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr129(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfProcessError {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr129(
                self as *const crate::RawSlotOfProcessError as *mut crate::RawSlotOfProcessError,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfProcessError {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr129(
                self as *mut crate::RawSlotOfProcessError,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfProcessError> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfProcessError> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfProcessError> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QProcess_ProcessError_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfQItemSelection> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref* static_cast<ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQItemSelection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref* static_cast<ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQItemSelection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfQItemSelection {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfQItemSelection>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr130(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfQItemSelection
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfQItemSelection>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr130(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfQItemSelection {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr130(
                self as *const crate::RawSlotOfQItemSelection
                    as *mut crate::RawSlotOfQItemSelection,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfQItemSelection {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr130(
                self as *mut crate::RawSlotOfQItemSelection,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfQItemSelection> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQItemSelection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQItemSelection> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QItemSelection_const_ref_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfQAbstractAnimation> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr* static_cast<ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr* static_cast<ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfQAbstractAnimation {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfQAbstractAnimation>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr131(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfQAbstractAnimation
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfQAbstractAnimation>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr131(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfQAbstractAnimation {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr131(
                self as *const crate::RawSlotOfQAbstractAnimation
                    as *mut crate::RawSlotOfQAbstractAnimation,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfQAbstractAnimation {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr131(
                self as *mut crate::RawSlotOfQAbstractAnimation,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfQAbstractAnimation> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQAbstractAnimation> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QAbstractAnimation_ptr_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfQModelIndexIntIntQModelIndexInt>
    for crate::QObject
{
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int* static_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQModelIndexIntIntQModelIndexInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int* static_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQModelIndexIntIntQModelIndexInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfQModelIndexIntIntQModelIndexInt {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfQModelIndexIntIntQModelIndexInt>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr132(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfQModelIndexIntIntQModelIndexInt
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfQModelIndexIntIntQModelIndexInt>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr132(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfQModelIndexIntIntQModelIndexInt {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr132(
                self as *const crate::RawSlotOfQModelIndexIntIntQModelIndexInt
                    as *mut crate::RawSlotOfQModelIndexIntIntQModelIndexInt,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfQModelIndexIntIntQModelIndexInt {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr132(
                self as *mut crate::RawSlotOfQModelIndexIntIntQModelIndexInt,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfQModelIndexIntIntQModelIndexInt> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfQModelIndexIntIntQModelIndexInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfQModelIndexIntIntQModelIndexInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_QModelIndex_const_ref_int_int_QModelIndex_const_ref_int_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticDowncast<crate::RawSlotOfOrientationInt> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int* static_cast<ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfOrientationInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int* static_cast<ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfOrientationInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::RawSlotOfOrientationInt {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::RawSlotOfOrientationInt>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr133(
            ptr.as_raw_ptr() as *mut crate::RawSlotOfOrientationInt
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::RawSlotOfOrientationInt>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr133(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

impl ::std::ops::Deref for crate::RawSlotOfOrientationInt {
    type Target = crate::QObject;
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int* ptr)```</span>.
    fn deref(&self) -> &crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr133(
                self as *const crate::RawSlotOfOrientationInt
                    as *mut crate::RawSlotOfOrientationInt,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::std::ops::DerefMut for crate::RawSlotOfOrientationInt {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QObject {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr133(
                self as *mut crate::RawSlotOfOrientationInt,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

impl ::cpp_utils::DynamicCast<crate::RawSlotOfOrientationInt> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::RawSlotOfOrientationInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_ptr(ptr.as_raw_ptr() as *mut crate::QObject);
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int* dynamic_cast<ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::RawSlotOfOrientationInt> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_ctr_qt_core_ffi_slot_wrapper_Qt_Orientation_int_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::ops::Begin for &crate::QStringView {
    type Output = ::cpp_utils::Ptr<crate::QChar>;
    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first character in the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QChar* QStringView::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first character in the string.</p>
    /// <p>This function is provided for STL compatibility.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#end">end</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#cbegin">cbegin</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#rbegin">rbegin</a>(), and <a href="http://doc.qt.io/qt-5/qstringview.html#data">data</a>().</p></div>
    fn begin(self) -> ::cpp_utils::Ptr<crate::QChar> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QStringView_begin(self as *const crate::QStringView)
        };
        unsafe { ::cpp_utils::Ptr::from_raw(ffi_result) }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::ops::End for &crate::QStringView {
    type Output = ::cpp_utils::Ptr<crate::QChar>;
    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary character after the last character in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const QChar* QStringView::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary character after the last character in the list.</p>
    /// <p>This function is provided for STL compatibility.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qstringview.html#begin">begin</a>(), <a href="http://doc.qt.io/qt-5/qstringview.html#cend">cend</a>(), and <a href="http://doc.qt.io/qt-5/qstringview.html#rend">rend</a>().</p></div>
    fn end(self) -> ::cpp_utils::Ptr<crate::QChar> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QStringView_end(self as *const crate::QStringView)
        };
        unsafe { ::cpp_utils::Ptr::from_raw(ffi_result) }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::ops::Begin for &crate::QLatin1String {
    type Output = ::cpp_utils::Ptr<::std::os::raw::c_char>;
    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first character in the string.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QLatin1String::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the first character in the string.</p>
    /// <p>This function is provided for STL compatibility.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlatin1string.html#end">end</a>(), <a href="http://doc.qt.io/qt-5/qlatin1string.html#cbegin">cbegin</a>(), <a href="http://doc.qt.io/qt-5/qlatin1string.html#rbegin">rbegin</a>(), and <a href="http://doc.qt.io/qt-5/qlatin1string.html#data">data</a>().</p></div>
    fn begin(self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QLatin1String_begin(self as *const crate::QLatin1String)
        };
        unsafe { ::cpp_utils::Ptr::from_raw(ffi_result) }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::ops::End for &crate::QLatin1String {
    type Output = ::cpp_utils::Ptr<::std::os::raw::c_char>;
    /// <p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary character after the last character in the list.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```const char* QLatin1String::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qlatin1string.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a const <a href="http://doc.qt.io/qt-5/containers.html#stl-style-iterators">STL-style iterator</a> pointing to the imaginary character after the last character in the list.</p>
    /// <p>This function is provided for STL compatibility.</p>
    /// <p>This function was introduced in  Qt 5.10.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qlatin1string.html#begin">begin</a>(), <a href="http://doc.qt.io/qt-5/qlatin1string.html#cend">cend</a>(), and <a href="http://doc.qt.io/qt-5/qlatin1string.html#rend">rend</a>().</p></div>
    fn end(self) -> ::cpp_utils::Ptr<::std::os::raw::c_char> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QLatin1String_end(self as *const crate::QLatin1String)
        };
        unsafe { ::cpp_utils::Ptr::from_raw(ffi_result) }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QStringView>> for crate::QStringView {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(QStringView lhs, QStringView rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, rhs: &::cpp_utils::Ref<crate::QStringView>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1070(
                self as *const crate::QStringView,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QStringView>> for crate::QStringView {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(QStringView lhs, QStringView rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QStringView>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_416(
                self as *const crate::QStringView,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QStringView>> for crate::QStringView {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(QStringView lhs, QStringView rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QStringView>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1072(
                self as *const crate::QStringView,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QStringView>> for crate::QStringView {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(QStringView lhs, QStringView rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QStringView>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_417(
                self as *const crate::QStringView,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QStringView>> for crate::QStringView {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(QStringView lhs, QStringView rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QStringView>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1073(
                self as *const crate::QStringView,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QChar>> for crate::QStringView {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(QStringView lhs, QChar rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, rhs: &::cpp_utils::Ref<crate::QChar>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1074(
                self as *const crate::QStringView,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QChar>> for crate::QStringView {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(QStringView lhs, QChar rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QChar>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_418(
                self as *const crate::QStringView,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QChar>> for crate::QStringView {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(QStringView lhs, QChar rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QChar>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1076(
                self as *const crate::QStringView,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QChar>> for crate::QStringView {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(QStringView lhs, QChar rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QChar>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_419(
                self as *const crate::QStringView,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QChar>> for crate::QStringView {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(QStringView lhs, QChar rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QChar>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1077(
                self as *const crate::QStringView,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QStringView>> for crate::QChar {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(QChar lhs, QStringView rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, rhs: &::cpp_utils::Ref<crate::QStringView>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1078(
                self as *const crate::QChar,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QStringView>> for crate::QChar {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(QChar lhs, QStringView rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QStringView>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_420(
                self as *const crate::QChar,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QStringView>> for crate::QChar {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(QChar lhs, QStringView rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QStringView>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1080(
                self as *const crate::QChar,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QStringView>> for crate::QChar {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(QChar lhs, QStringView rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QStringView>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_421(
                self as *const crate::QChar,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QStringView>> for crate::QChar {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(QChar lhs, QStringView rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QStringView>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1081(
                self as *const crate::QChar,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QLatin1String>> for crate::QStringView {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(QStringView lhs, QLatin1String rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, rhs: &::cpp_utils::Ref<crate::QLatin1String>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1082(
                self as *const crate::QStringView,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QLatin1String>> for crate::QStringView {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(QStringView lhs, QLatin1String rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QLatin1String>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_422(
                self as *const crate::QStringView,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QLatin1String>> for crate::QStringView {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(QStringView lhs, QLatin1String rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QLatin1String>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1084(
                self as *const crate::QStringView,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QLatin1String>> for crate::QStringView {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(QStringView lhs, QLatin1String rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QLatin1String>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_423(
                self as *const crate::QStringView,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QLatin1String>> for crate::QStringView {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(QStringView lhs, QLatin1String rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QLatin1String>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1085(
                self as *const crate::QStringView,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QStringView>> for crate::QLatin1String {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(QLatin1String lhs, QStringView rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, rhs: &::cpp_utils::Ref<crate::QStringView>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1086(
                self as *const crate::QLatin1String,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QStringView>> for crate::QLatin1String {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(QLatin1String lhs, QStringView rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QStringView>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_424(
                self as *const crate::QLatin1String,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QStringView>> for crate::QLatin1String {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(QLatin1String lhs, QStringView rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QStringView>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1088(
                self as *const crate::QLatin1String,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QStringView>> for crate::QLatin1String {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(QLatin1String lhs, QStringView rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QStringView>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_425(
                self as *const crate::QLatin1String,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QStringView>> for crate::QLatin1String {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(QLatin1String lhs, QStringView rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QStringView>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1089(
                self as *const crate::QLatin1String,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QStringView>> for &crate::QDebug {
    type Output = ::cpp_utils::MutRef<crate::QDebug>;
    /// <p>Writes the string view, <i>s</i>, to the stream and returns a reference to the stream. Normally, <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> prints the string inside quotes and transforms non-printable characters to their Unicode values (\u1234).</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug& QDebug::operator<<(QStringView s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qdebug.html#operator-lt-lt-18">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes the string view, <i>s</i>, to the stream and returns a reference to the stream. Normally, <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> prints the string inside quotes and transforms non-printable characters to their Unicode values (\u1234).</p>
    /// <p>To print non-printable characters without transformation, enable the <a href="http://doc.qt.io/qt-5/qdebug.html#noquote">noquote</a>() functionality. Note that some <a href="http://doc.qt.io/qt-5/qdebug.html">QDebug</a> backends might not be 8-bit clean.</p>
    /// <p>See the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> overload for examples.</p>
    /// <p>This function was introduced in  Qt 5.10.</p></div>
    fn shl(self, s: ::cpp_utils::Ref<crate::QStringView>) -> ::cpp_utils::MutRef<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QDebug_operator__45(
                self as *const crate::QDebug as *mut crate::QDebug,
                s.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QFileInfo>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QFileInfo& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, arg2: ::cpp_utils::Ref<crate::QFileInfo>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1090(
                self as *const crate::QDebug,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QOperatingSystemVersion>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QOperatingSystemVersion& ov)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        ov: ::cpp_utils::Ref<crate::QOperatingSystemVersion>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1091(
                self as *const crate::QDebug,
                ov.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::CppDeletable for crate::QSemaphoreReleaser {
    /// <p>Unless canceled, calls <a href="http://doc.qt.io/qt-5/qsemaphore.html#release">QSemaphore::release</a>() with the arguments provided to the constructor, or by the last move assignment.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QSemaphoreReleaser::~QSemaphoreReleaser()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qsemaphorereleaser.html#dtor.QSemaphoreReleaser">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Unless canceled, calls <a href="http://doc.qt.io/qt-5/qsemaphore.html#release">QSemaphore::release</a>() with the arguments provided to the constructor, or by the last move assignment.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QSemaphoreReleaser_dQSemaphoreReleaser(
            self as *mut crate::QSemaphoreReleaser,
        )
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QStorageInfo>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QStorageInfo& arg2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        arg2: ::cpp_utils::Ref<crate::QStorageInfo>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1092(
                self as *const crate::QDebug,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::CppDeletable for crate::QStringView {
    /// <p>The <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> class provides a unified view on UTF-16 strings with a read-only subset of the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> API.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QStringView::~QStringView()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qstringview.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> class provides a unified view on UTF-16 strings with a read-only subset of the <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> API.</p>
    /// <p>A <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> references a contiguous portion of a UTF-16 string it does not own. It acts as an interface type to all kinds of UTF-16 string, without the need to construct a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> first.</p>
    /// <p>The UTF-16 string may be represented as an array (or an array-compatible data-structure such as <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, std::basic_string, etc.) of <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a>, <code>ushort</code>, <code>char16_t</code> (on compilers that support C++11 Unicode strings) or (on platforms, such as Windows, where it is a 16-bit type) <code>wchar_t</code>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> is designed as an interface type; its main use-case is as a function parameter type. When QStringViews are used as automatic variables or data members, care must be taken to ensure that the referenced string data (for example, owned by a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>) outlives the <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> on all code paths, lest the string view ends up referencing deleted data.</p>
    /// <p>When used as an interface type, <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> allows a single function to accept a wide variety of UTF-16 string data sources. One function accepting <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> thus replaces three function overloads (taking <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>, <a href="http://doc.qt.io/qt-5/qstringref.html">QStringRef</a>, and <code>(const QChar*, int)</code>), while at the same time enabling even more string data sources to be passed to the function, such as <code>u"Hello World"</code>, a <code>char16_t</code> string literal.</p>
    /// <p>QStringViews should be passed by value, not by reference-to-const:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type">void</span> myfun1(<span class="type"><a href="http://doc.qt.io/qt-5/qstringview.html#QStringView">QStringView</a></span> sv);        <span class="comment">// preferred</span>
    ///   <span class="type">void</span> myfun2(<span class="keyword">const</span> <span class="type"><a href="http://doc.qt.io/qt-5/qstringview.html#QStringView">QStringView</a></span> <span class="operator">&amp;</span>sv); <span class="comment">// compiles and works, but slower</span>
    ///
    /// </pre>
    /// <p>If you want to give your users maximum freedom in what strings they can pass to your function, accompany the <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> overload with overloads for</p>
    /// <ul>
    /// <li><i>QChar</i>: this overload can delegate to the <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> version:<pre class="cpp">
    ///
    ///   <span class="type">void</span> fun(<span class="type"><a href="http://doc.qt.io/qt-5/qchar.html">QChar</a></span> ch) { fun(<span class="type"><a href="http://doc.qt.io/qt-5/qstringview.html#QStringView">QStringView</a></span>(<span class="operator">&amp;</span>ch<span class="operator">,</span> <span class="number">1</span>)); }
    ///
    /// </pre>
    /// <p>even though, for technical reasons, <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> cannot provide a <a href="http://doc.qt.io/qt-5/qchar.html">QChar</a> constructor by itself.</p>
    /// </li>
    /// <li><i>QString</i>: if you store an unmodified copy of the string and thus would like to take advantage of <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>'s implicit sharing.</li>
    /// <li><a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>: if you can implement the function without converting the <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> to UTF-16 first; users expect a function overloaded on <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a> to perform strictly less memory allocations than the semantically equivalent call of the <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> version, involving construction of a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> from the <a href="http://doc.qt.io/qt-5/qlatin1string.html">QLatin1String</a>.</li>
    /// </ul>
    /// <p><a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> can also be used as the return value of a function. If you call a function returning <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a>, take extra care to not keep the <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> around longer than the function promises to keep the referenced string data alive. If in doubt, obtain a strong reference to the data by calling <a href="http://doc.qt.io/qt-5/qstringview.html#toString">toString</a>() to convert the <a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> into a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a>.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a> is a <i>Literal Type</i>, but since it stores data as <code>char16_t</code>, iteration is not <code>constexpr</code> (casts from <code>const char16_t*</code> to <code>const QChar*</code>, which is not allowed in <code>constexpr</code> functions). You can use an indexed loop and/or <a href="http://doc.qt.io/qt-5/qstringview.html#utf16">utf16</a>() in <code>constexpr</code> contexts instead.</p>
    /// <p><b>Note: </b>We strongly discourage the use of <a href="http://doc.qt.io/qt-5/qlist.html">QList</a>&lt;<a href="http://doc.qt.io/qt-5/qstringview.html">QStringView</a>&gt;, because <a href="http://doc.qt.io/qt-5/qlist.html">QList</a> is a very inefficient container for QStringViews (it would heap-allocate every element). Use <a href="http://doc.qt.io/qt-5/qvector.html">QVector</a> (or std::vector) to hold QStringViews instead.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QStringView_dQStringView(self as *mut crate::QStringView)
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::CppDeletable for crate::QRandomGenerator {
    /// <p>The <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> class allows one to obtain random values from a high-quality Random Number Generator.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QRandomGenerator::~QRandomGenerator()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> class allows one to obtain random values from a high-quality Random Number Generator.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> may be used to generate random values from a high-quality random number generator. Like the C++ random engines, <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> can be seeded with user-provided values through the constructor. When seeded, the sequence of numbers generated by this class is deterministic. That is to say, given the same seed data, <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> will generate the same sequence of numbers. But given different seeds, the results should be considerably different.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#securelySeeded">QRandomGenerator::securelySeeded</a>() can be used to create a <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> that is securely seeded with <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#system">QRandomGenerator::system</a>(), meaning that the sequence of numbers it generates cannot be easily predicted. Additionally, <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#global">QRandomGenerator::global</a>() returns a global instance of <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a> that Qt will ensure to be securely seeded. This object is thread-safe, may be shared for most uses, and is always seeded from <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#system">QRandomGenerator::system</a>()</p>
    /// <p><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#system">QRandomGenerator::system</a>() may be used to access the system's cryptographically-safe random generator. On Unix systems, it's equivalent to reading from <code>/dev/urandom</code> or the <code>getrandom()</code> or <code>getentropy()</code> system calls.</p>
    /// <p>The class can generate 32-bit or 64-bit quantities, or fill an array of those. The most common way of generating new values is to call the <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate">generate</a>(), <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate64">generate64</a>() or <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#fillRange">fillRange</a>() functions. One would use it as:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#quint32-typedef">quint32</a></span> value <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qrandomgenerator.html#QRandomGenerator">QRandomGenerator</a></span><span class="operator">::</span>global()<span class="operator">-</span><span class="operator">&gt;</span>generate();
    ///
    /// </pre>
    /// <p>Additionally, it provides a floating-point function <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generateDouble">generateDouble</a>() that returns a number in the range [0, 1) (that is, inclusive of zero and exclusive of 1). There's also a set of convenience functions that facilitate obtaining a random number in a bounded, integral range.</p>
    /// <a name="seeding-and-determinism"></a></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QRandomGenerator_dQRandomGenerator(
            self as *mut crate::QRandomGenerator,
        )
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::CppDeletable for crate::QRandomGenerator64 {
    /// <p>The <a href="http://doc.qt.io/qt-5/qrandomgenerator64.html">QRandomGenerator64</a> class allows one to obtain 64-bit random values from a high-quality, seed-less Random Number Generator.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QRandomGenerator64::~QRandomGenerator64()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qrandomgenerator64.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qrandomgenerator64.html">QRandomGenerator64</a> class allows one to obtain 64-bit random values from a high-quality, seed-less Random Number Generator.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qrandomgenerator64.html">QRandomGenerator64</a> is a simple adaptor class around <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a>, making the <a href="http://doc.qt.io/qt-5/qrandomgenerator.html#generate64">QRandomGenerator::generate64</a>() function the default for <a href="http://doc.qt.io/qt-5/qrandomgenerator64.html#operator-28-29">operator</a>()(), instead of the function that returns 32-bit quantities. This class is intended to be used in conjunction with Standard Library algorithms that need 64-bit quantities instead of 32-bit ones.</p>
    /// <p>In all other aspects, the class is the same. Please refer to <a href="http://doc.qt.io/qt-5/qrandomgenerator.html">QRandomGenerator</a>'s documentation for more information.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QRandomGenerator64_dQRandomGenerator64(
            self as *mut crate::QRandomGenerator64,
        )
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::std::ops::Shl<crate::QFlags<crate::q_abstract_item_model::CheckIndexOption>>
    for &crate::QDataStream
{
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<QAbstractItemModel::CheckIndexOption> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::q_abstract_item_model::CheckIndexOption>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1093(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::std::ops::Shl<crate::QFlags<crate::q_locale::DataSizeFormat>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<QLocale::DataSizeFormat> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::q_locale::DataSizeFormat>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1094(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::std::ops::Shl<crate::QFlags<crate::q_abstract_item_model::CheckIndexOption>>
    for &crate::QDebug
{
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<QAbstractItemModel::CheckIndexOption>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::q_abstract_item_model::CheckIndexOption>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1097(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::std::ops::Shl<crate::QFlags<crate::q_locale::DataSizeFormat>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<QLocale::DataSizeFormat>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::q_locale::DataSizeFormat>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1098(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::StaticDowncast<crate::QRandomGenerator64> for crate::QRandomGenerator {
    /// Calls C++ function: <span style='color: green;'>```QRandomGenerator64* static_cast<QRandomGenerator64*>(QRandomGenerator* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QRandomGenerator>,
    ) -> ::cpp_utils::Ptr<crate::QRandomGenerator64> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QRandomGenerator64_ptr(
            ptr.as_raw_ptr() as *mut crate::QRandomGenerator,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QRandomGenerator64* static_cast<QRandomGenerator64*>(QRandomGenerator* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QRandomGenerator>,
    ) -> ::cpp_utils::MutPtr<crate::QRandomGenerator64> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QRandomGenerator64_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::cpp_utils::StaticUpcast<crate::QRandomGenerator> for crate::QRandomGenerator64 {
    /// Calls C++ function: <span style='color: green;'>```QRandomGenerator* static_cast<QRandomGenerator*>(QRandomGenerator64* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QRandomGenerator64>,
    ) -> ::cpp_utils::Ptr<crate::QRandomGenerator> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QRandomGenerator_ptr(
            ptr.as_raw_ptr() as *mut crate::QRandomGenerator64,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QRandomGenerator* static_cast<QRandomGenerator*>(QRandomGenerator64* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QRandomGenerator64>,
    ) -> ::cpp_utils::MutPtr<crate::QRandomGenerator> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QRandomGenerator_ptr(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::std::ops::Deref for crate::QRandomGenerator64 {
    type Target = crate::QRandomGenerator;
    /// Calls C++ function: <span style='color: green;'>```QRandomGenerator* static_cast<QRandomGenerator*>(QRandomGenerator64* ptr)```</span>.
    fn deref(&self) -> &crate::QRandomGenerator {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QRandomGenerator_ptr(
                self as *const crate::QRandomGenerator64 as *mut crate::QRandomGenerator64,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
impl ::std::ops::DerefMut for crate::QRandomGenerator64 {
    /// Calls C++ function: <span style='color: green;'>```QRandomGenerator* static_cast<QRandomGenerator*>(QRandomGenerator64* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QRandomGenerator {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QRandomGenerator_ptr(
                self as *mut crate::QRandomGenerator64,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QStringView>> for &crate::QTextStream {
    type Output = ::cpp_utils::MutRef<crate::QTextStream>;
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QTextStream& QTextStream::operator<<(QStringView s)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtextstream.html#operator-lt-lt-13">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Writes <i>string</i> to the stream, and returns a reference to the <a href="http://doc.qt.io/qt-5/qtextstream.html">QTextStream</a>.</p>
    /// <p>This function was introduced in  Qt 5.12.</p></div>
    fn shl(
        self,
        s: ::cpp_utils::Ref<crate::QStringView>,
    ) -> ::cpp_utils::MutRef<crate::QTextStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QTextStream_operator__66(
                self as *const crate::QTextStream as *mut crate::QTextStream,
                s.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Shl<crate::QCborSimpleType> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, QCborSimpleType st)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, st: crate::QCborSimpleType) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1132(self as *const crate::QDebug, st)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Shl<crate::QCborKnownTags> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, QCborKnownTags tg)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, tg: crate::QCborKnownTags) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1133(self as *const crate::QDebug, tg)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Shl<crate::QCborTag> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, QCborTag tg)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, tg: crate::QCborTag) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1134(self as *const crate::QDebug, tg)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::CppDeletable for crate::QCborValue {
    /// <p>Disposes of the current <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object and frees any associated resources.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QCborValue::~QCborValue()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#dtor.QCborValue">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Disposes of the current <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> object and frees any associated resources.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCborValue_dQCborValue(self as *mut crate::QCborValue)
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QCborValue>> for crate::QCborValue {
    /// <p>Compares this value and <i>other</i>, and returns true if they hold the same contents, false otherwise. If each <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> contains an array or map, the comparison is recursive to elements contained in them.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborValue::operator==(const QCborValue& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares this value and <i>other</i>, and returns true if they hold the same contents, false otherwise. If each <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> contains an array or map, the comparison is recursive to elements contained in them.</p>
    /// <p>For more information on CBOR equality in Qt, see, <a href="http://doc.qt.io/qt-5/qcborvalue.html#compare">compare</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#compare">compare</a>(), QCborValue::operator==(), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-eq-eq">QCborMap::operator==</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#operator-not-eq">operator!=</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#operator-lt">operator&lt;</a>().</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QCborValue>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QCborValue_operator__3(
                self as *const crate::QCborValue,
                other.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QCborValue>> for crate::QCborValue {
    /// <p>Compares this value and <i>other</i>, and returns true if this value should be sorted before <i>other</i>, false otherwise. If each <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> contains an array or map, the comparison is recursive to elements contained in them.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborValue::operator<(const QCborValue& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborvalue.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares this value and <i>other</i>, and returns true if this value should be sorted before <i>other</i>, false otherwise. If each <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> contains an array or map, the comparison is recursive to elements contained in them.</p>
    /// <p>For more information on CBOR sorting order, see <a href="http://doc.qt.io/qt-5/qcborvalue.html#compare">QCborValue::compare</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborvalue.html#compare">compare</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#operator-eq-eq">QCborValue::operator==</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-eq-eq">QCborMap::operator==</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#operator-eq-eq">operator==</a>(), and <a href="http://doc.qt.io/qt-5/qcborvalue.html#operator-not-eq">operator!=</a>().</p></div>
    fn lt(&self, other: &::cpp_utils::Ref<crate::QCborValue>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QCborValue_operator_2(
                self as *const crate::QCborValue,
                other.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QCborValue>> for crate::QCborValueRef {
    /// Calls C++ function: <span style='color: green;'>```bool QCborValueRef::operator==(const QCborValue& other) const```</span>.
    fn eq(&self, other: &::cpp_utils::Ref<crate::QCborValue>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QCborValueRef_operator__(
                self as *const crate::QCborValueRef,
                other.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QCborValue>> for crate::QCborValueRef {
    /// Calls C++ function: <span style='color: green;'>```bool QCborValueRef::operator<(const QCborValue& other) const```</span>.
    fn lt(&self, other: &::cpp_utils::Ref<crate::QCborValue>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QCborValueRef_operator_3(
                self as *const crate::QCborValueRef,
                other.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QCborValue>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QCborValue& v)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, v: ::cpp_utils::Ref<crate::QCborValue>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1135(
                self as *const crate::QDebug,
                v.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::CppDeletable for crate::QCborArray {
    /// <p>Destroys this <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> and frees any associated resources.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QCborArray::~QCborArray()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#dtor.QCborArray">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys this <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> and frees any associated resources.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCborArray_dQCborArray(self as *mut crate::QCborArray)
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QCborArray>> for crate::QCborArray {
    /// <p>Compares this array and <i>other</i>, comparing each element in sequence, and returns true if both arrays contains the same elements, false otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborArray::operator==(const QCborArray& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares this array and <i>other</i>, comparing each element in sequence, and returns true if both arrays contains the same elements, false otherwise.</p>
    /// <p>For more information on CBOR equality in Qt, see, <a href="http://doc.qt.io/qt-5/qcborvalue.html#compare">QCborValue::compare</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#compare">compare</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#operator-eq-eq">QCborValue::operator==</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-eq-eq">QCborMap::operator==</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-not-eq">operator!=</a>(), and <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-lt">operator&lt;</a>().</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QCborArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QCborArray_operator__2(
                self as *const crate::QCborArray,
                other.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QCborArray>> for crate::QCborArray {
    /// <p>Compares this array and <i>other</i>, comparing each element in sequence, and returns true if this array should be sorted before <i>other</i>, false otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborArray::operator<(const QCborArray& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares this array and <i>other</i>, comparing each element in sequence, and returns true if this array should be sorted before <i>other</i>, false otherwise.</p>
    /// <p>For more information on CBOR sorting order, see <a href="http://doc.qt.io/qt-5/qcborvalue.html#compare">QCborValue::compare</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#compare">compare</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#operator-eq-eq">QCborValue::operator==</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-eq-eq">QCborMap::operator==</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-eq-eq">operator==</a>(), and <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-not-eq">operator!=</a>().</p></div>
    fn lt(&self, other: &::cpp_utils::Ref<crate::QCborArray>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QCborArray_operator_1(
                self as *const crate::QCborArray,
                other.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::ops::BeginMut for &mut crate::QCborArray {
    type Output = ::cpp_utils::CppBox<crate::q_cbor_array::Iterator>;
    /// <p>Returns an array iterator pointing to the first item in this array. If the array is empty, then this function returns the same as <a href="http://doc.qt.io/qt-5/qcborarray.html#end">end</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborArray::Iterator QCborArray::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an array iterator pointing to the first item in this array. If the array is empty, then this function returns the same as <a href="http://doc.qt.io/qt-5/qcborarray.html#end">end</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qcborarray.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_cbor_array::Iterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QCborArray_begin(self as *mut crate::QCborArray)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::ops::Begin for &crate::QCborArray {
    type Output = ::cpp_utils::CppBox<crate::q_cbor_array::ConstIterator>;
    /// <p>Returns an array iterator pointing to the first item in this array. If the array is empty, then this function returns the same as <a href="http://doc.qt.io/qt-5/qcborarray.html#end">end</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborArray::ConstIterator QCborArray::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an array iterator pointing to the first item in this array. If the array is empty, then this function returns the same as <a href="http://doc.qt.io/qt-5/qcborarray.html#end">end</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qcborarray.html#constEnd">constEnd</a>().</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_cbor_array::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QCborArray_begin1(self as *const crate::QCborArray)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::ops::EndMut for &mut crate::QCborArray {
    type Output = ::cpp_utils::CppBox<crate::q_cbor_array::Iterator>;
    /// <p>Returns an array iterator pointing to just after the last element in this array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborArray::Iterator QCborArray::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an array iterator pointing to just after the last element in this array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qcborarray.html#constEnd">constEnd</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_cbor_array::Iterator> {
        let ffi_result =
            unsafe { crate::__ffi::ctr_qt_core_ffi_QCborArray_end(self as *mut crate::QCborArray) };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::ops::End for &crate::QCborArray {
    type Output = ::cpp_utils::CppBox<crate::q_cbor_array::ConstIterator>;
    /// <p>Returns an array iterator pointing to just after the last element in this array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborArray::ConstIterator QCborArray::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns an array iterator pointing to just after the last element in this array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qcborarray.html#constEnd">constEnd</a>().</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_cbor_array::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QCborArray_end1(self as *const crate::QCborArray)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Add<::cpp_utils::Ref<crate::QCborValue>> for &crate::QCborArray {
    type Output = ::cpp_utils::CppBox<crate::QCborArray>;
    /// <p>Returns a new <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> containing the same elements as this array, plus <i>v</i> appended as the last element.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborArray QCborArray::operator+(const QCborValue& v) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a new <a href="http://doc.qt.io/qt-5/qcborarray.html">QCborArray</a> containing the same elements as this array, plus <i>v</i> appended as the last element.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#operator-2b-eq">operator+=</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-lt-lt">operator&lt;&lt;</a>(), and <a href="http://doc.qt.io/qt-5/qcborarray.html#append">append</a>().</p></div>
    fn add(self, v: ::cpp_utils::Ref<crate::QCborValue>) -> ::cpp_utils::CppBox<crate::QCborArray> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QCborArray_operator_2(
                self as *const crate::QCborArray,
                v.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QCborValue>> for crate::QCborArray {
    /// <p>Appends <i>v</i> to this array and returns a reference to this array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborArray& QCborArray::operator+=(const QCborValue& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>v</i> to this array and returns a reference to this array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-2b">operator+</a>(), and <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, v: ::cpp_utils::Ref<crate::QCborValue>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QCborArray_operator__4(
                self as *mut crate::QCborArray,
                v.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QCborValue>> for &crate::QCborArray {
    type Output = ::cpp_utils::MutRef<crate::QCborArray>;
    /// <p>Appends <i>v</i> to this array and returns a reference to this array.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborArray& QCborArray::operator<<(const QCborValue& v)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>v</i> to this array and returns a reference to this array.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcborarray.html#append">append</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#insert">insert</a>(), <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-2b">operator+</a>(), and <a href="http://doc.qt.io/qt-5/qcborarray.html#operator-2b-eq">operator+=</a>().</p></div>
    fn shl(self, v: ::cpp_utils::Ref<crate::QCborValue>) -> ::cpp_utils::MutRef<crate::QCborArray> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QCborArray_operator__5(
                self as *const crate::QCborArray as *mut crate::QCborArray,
                v.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QCborArray>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QCborArray& a)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, a: ::cpp_utils::Ref<crate::QCborArray>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1136(
                self as *const crate::QDebug,
                a.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::CppDeletable for crate::QCborMap {
    /// <p>Destroys this <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> object and frees any associated resources it owns.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QCborMap::~QCborMap()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#dtor.QCborMap">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys this <a href="http://doc.qt.io/qt-5/qcbormap.html">QCborMap</a> object and frees any associated resources it owns.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCborMap_dQCborMap(self as *mut crate::QCborMap)
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QCborMap>> for crate::QCborMap {
    /// <p>Compares this map and <i>other</i>, comparing each element in sequence, and returns true if the two maps contains the same elements in the same order, false otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborMap::operator==(const QCborMap& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares this map and <i>other</i>, comparing each element in sequence, and returns true if the two maps contains the same elements in the same order, false otherwise.</p>
    /// <p>Note that CBOR maps are unordered, which means that two maps containing the very same pairs but in different order will still compare differently. To avoid this, it is recommended to insert elements into the map in a predictable order, such as by ascending key value. In fact, maps with keys in sorted order are required for Canonical CBOR representation.</p>
    /// <p>For more information on CBOR equality in Qt, see, <a href="http://doc.qt.io/qt-5/qcborvalue.html#compare">QCborValue::compare</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#compare">compare</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#operator-eq-eq">QCborValue::operator==</a>(), QCborMap::operator==(), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-not-eq">operator!=</a>(), and <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-lt">operator&lt;</a>().</p></div>
    fn eq(&self, other: &::cpp_utils::Ref<crate::QCborMap>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QCborMap_operator__8(
                self as *const crate::QCborMap,
                other.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QCborMap>> for crate::QCborMap {
    /// <p>Compares this map and <i>other</i>, comparing each element in sequence, and returns true if this map should be sorted before <i>other</i>, false otherwise.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QCborMap::operator<(const QCborMap& other) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares this map and <i>other</i>, comparing each element in sequence, and returns true if this map should be sorted before <i>other</i>, false otherwise.</p>
    /// <p>Note that CBOR maps are unordered, which means that two maps containing the very same pairs but in different order will still compare differently. To avoid this, it is recommended to insert elements into the map in a predictable order, such as by ascending key value. In fact, maps with keys in sorted order are required for Canonical CBOR representation.</p>
    /// <p>For more information on CBOR sorting order, see <a href="http://doc.qt.io/qt-5/qcborvalue.html#compare">QCborValue::compare</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#compare">compare</a>(), <a href="http://doc.qt.io/qt-5/qcborvalue.html#operator-eq-eq">QCborValue::operator==</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-eq-eq">QCborMap::operator==</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-eq-eq">operator==</a>(), and <a href="http://doc.qt.io/qt-5/qcbormap.html#operator-not-eq">operator!=</a>().</p></div>
    fn lt(&self, other: &::cpp_utils::Ref<crate::QCborMap>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QCborMap_operator_1(
                self as *const crate::QCborMap,
                other.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::ops::BeginMut for &mut crate::QCborMap {
    type Output = ::cpp_utils::CppBox<crate::q_cbor_map::Iterator>;
    /// <p>Returns a map iterator pointing to the first key-value pair of this map. If this map is empty, the returned iterator will be the same as <a href="http://doc.qt.io/qt-5/qcbormap.html#end">end</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborMap::Iterator QCborMap::begin()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#begin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a map iterator pointing to the first key-value pair of this map. If this map is empty, the returned iterator will be the same as <a href="http://doc.qt.io/qt-5/qcbormap.html#end">end</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#constBegin">constBegin</a>() and <a href="http://doc.qt.io/qt-5/qcbormap.html#end">end</a>().</p></div>
    fn begin_mut(self) -> ::cpp_utils::CppBox<crate::q_cbor_map::Iterator> {
        let ffi_result =
            unsafe { crate::__ffi::ctr_qt_core_ffi_QCborMap_begin(self as *mut crate::QCborMap) };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::ops::Begin for &crate::QCborMap {
    type Output = ::cpp_utils::CppBox<crate::q_cbor_map::ConstIterator>;
    /// <p>Returns a map iterator pointing to the first key-value pair of this map. If this map is empty, the returned iterator will be the same as <a href="http://doc.qt.io/qt-5/qcbormap.html#constEnd">constEnd</a>().</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborMap::ConstIterator QCborMap::begin() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#begin-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a map iterator pointing to the first key-value pair of this map. If this map is empty, the returned iterator will be the same as <a href="http://doc.qt.io/qt-5/qcbormap.html#constEnd">constEnd</a>().</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#begin">begin</a>() and <a href="http://doc.qt.io/qt-5/qcbormap.html#constEnd">constEnd</a>().</p></div>
    fn begin(self) -> ::cpp_utils::CppBox<crate::q_cbor_map::ConstIterator> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QCborMap_begin1(self as *const crate::QCborMap)
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::ops::EndMut for &mut crate::QCborMap {
    type Output = ::cpp_utils::CppBox<crate::q_cbor_map::Iterator>;
    /// <p>Returns a map iterator representing an element just past the last element in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborMap::Iterator QCborMap::end()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#end">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a map iterator representing an element just past the last element in the map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#begin">begin</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#constBegin">constBegin</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#find">find</a>(), and <a href="http://doc.qt.io/qt-5/qcbormap.html#constFind">constFind</a>().</p></div>
    fn end_mut(self) -> ::cpp_utils::CppBox<crate::q_cbor_map::Iterator> {
        let ffi_result =
            unsafe { crate::__ffi::ctr_qt_core_ffi_QCborMap_end(self as *mut crate::QCborMap) };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::ops::End for &crate::QCborMap {
    type Output = ::cpp_utils::CppBox<crate::q_cbor_map::ConstIterator>;
    /// <p>Returns a map iterator representing an element just past the last element in the map.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QCborMap::ConstIterator QCborMap::end() const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcbormap.html#end-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a map iterator representing an element just past the last element in the map.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcbormap.html#begin">begin</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#constBegin">constBegin</a>(), <a href="http://doc.qt.io/qt-5/qcbormap.html#find">find</a>(), and <a href="http://doc.qt.io/qt-5/qcbormap.html#constFind">constFind</a>().</p></div>
    fn end(self) -> ::cpp_utils::CppBox<crate::q_cbor_map::ConstIterator> {
        let ffi_result =
            unsafe { crate::__ffi::ctr_qt_core_ffi_QCborMap_end1(self as *const crate::QCborMap) };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QCborMap>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug arg1, const QCborMap& m)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(self, m: ::cpp_utils::Ref<crate::QCborMap>) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1137(
                self as *const crate::QDebug,
                m.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Neg for &crate::Qfloat16 {
    type Output = ::cpp_utils::CppBox<crate::Qfloat16>;
    /// Calls C++ function: <span style='color: green;'>```qfloat16 operator-(qfloat16 a)```</span>.
    fn neg(self) -> ::cpp_utils::CppBox<crate::Qfloat16> {
        let ffi_result =
            unsafe { crate::__ffi::ctr_qt_core_ffi_operator_436(self as *const crate::Qfloat16) };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Add<::cpp_utils::Ref<crate::Qfloat16>> for &crate::Qfloat16 {
    type Output = ::cpp_utils::CppBox<crate::Qfloat16>;
    /// Calls C++ function: <span style='color: green;'>```qfloat16 operator+(qfloat16 a, qfloat16 b)```</span>.
    fn add(self, b: ::cpp_utils::Ref<crate::Qfloat16>) -> ::cpp_utils::CppBox<crate::Qfloat16> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_437(
                self as *const crate::Qfloat16,
                b.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Sub<::cpp_utils::Ref<crate::Qfloat16>> for &crate::Qfloat16 {
    type Output = ::cpp_utils::CppBox<crate::Qfloat16>;
    /// Calls C++ function: <span style='color: green;'>```qfloat16 operator-(qfloat16 a, qfloat16 b)```</span>.
    fn sub(self, b: ::cpp_utils::Ref<crate::Qfloat16>) -> ::cpp_utils::CppBox<crate::Qfloat16> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_438(
                self as *const crate::Qfloat16,
                b.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Mul<::cpp_utils::Ref<crate::Qfloat16>> for &crate::Qfloat16 {
    type Output = ::cpp_utils::CppBox<crate::Qfloat16>;
    /// Calls C++ function: <span style='color: green;'>```qfloat16 operator*(qfloat16 a, qfloat16 b)```</span>.
    fn mul(self, b: ::cpp_utils::Ref<crate::Qfloat16>) -> ::cpp_utils::CppBox<crate::Qfloat16> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_439(
                self as *const crate::Qfloat16,
                b.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Div<::cpp_utils::Ref<crate::Qfloat16>> for &crate::Qfloat16 {
    type Output = ::cpp_utils::CppBox<crate::Qfloat16>;
    /// Calls C++ function: <span style='color: green;'>```qfloat16 operator/(qfloat16 a, qfloat16 b)```</span>.
    fn div(self, b: ::cpp_utils::Ref<crate::Qfloat16>) -> ::cpp_utils::CppBox<crate::Qfloat16> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_440(
                self as *const crate::Qfloat16,
                b.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Add<::std::os::raw::c_double> for &crate::Qfloat16 {
    type Output = ::std::os::raw::c_double;
    /// Calls C++ function: <span style='color: green;'>```double operator+(qfloat16 lhs, double rhs)```</span>.
    fn add(self, rhs: ::std::os::raw::c_double) -> ::std::os::raw::c_double {
        unsafe { crate::__ffi::ctr_qt_core_ffi_operator_449(self as *const crate::Qfloat16, rhs) }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Sub<::std::os::raw::c_double> for &crate::Qfloat16 {
    type Output = ::std::os::raw::c_double;
    /// Calls C++ function: <span style='color: green;'>```double operator-(qfloat16 lhs, double rhs)```</span>.
    fn sub(self, rhs: ::std::os::raw::c_double) -> ::std::os::raw::c_double {
        unsafe { crate::__ffi::ctr_qt_core_ffi_operator_451(self as *const crate::Qfloat16, rhs) }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Mul<::std::os::raw::c_double> for &crate::Qfloat16 {
    type Output = ::std::os::raw::c_double;
    /// Calls C++ function: <span style='color: green;'>```double operator*(qfloat16 lhs, double rhs)```</span>.
    fn mul(self, rhs: ::std::os::raw::c_double) -> ::std::os::raw::c_double {
        unsafe { crate::__ffi::ctr_qt_core_ffi_operator_453(self as *const crate::Qfloat16, rhs) }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Div<::std::os::raw::c_double> for &crate::Qfloat16 {
    type Output = ::std::os::raw::c_double;
    /// Calls C++ function: <span style='color: green;'>```double operator/(qfloat16 lhs, double rhs)```</span>.
    fn div(self, rhs: ::std::os::raw::c_double) -> ::std::os::raw::c_double {
        unsafe { crate::__ffi::ctr_qt_core_ffi_operator_455(self as *const crate::Qfloat16, rhs) }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::AddAssign<::std::os::raw::c_double> for crate::Qfloat16 {
    /// Calls C++ function: <span style='color: green;'>```qfloat16& operator+=(qfloat16& lhs, double rhs)```</span>.
    fn add_assign(&mut self, rhs: ::std::os::raw::c_double) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1142(self as *mut crate::Qfloat16, rhs)
        };
        let _ = ffi_result;
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::SubAssign<::std::os::raw::c_double> for crate::Qfloat16 {
    /// Calls C++ function: <span style='color: green;'>```qfloat16& operator-=(qfloat16& lhs, double rhs)```</span>.
    fn sub_assign(&mut self, rhs: ::std::os::raw::c_double) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1143(self as *mut crate::Qfloat16, rhs)
        };
        let _ = ffi_result;
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::MulAssign<::std::os::raw::c_double> for crate::Qfloat16 {
    /// Calls C++ function: <span style='color: green;'>```qfloat16& operator*=(qfloat16& lhs, double rhs)```</span>.
    fn mul_assign(&mut self, rhs: ::std::os::raw::c_double) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1144(self as *mut crate::Qfloat16, rhs)
        };
        let _ = ffi_result;
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::DivAssign<::std::os::raw::c_double> for crate::Qfloat16 {
    /// Calls C++ function: <span style='color: green;'>```qfloat16& operator/=(qfloat16& lhs, double rhs)```</span>.
    fn div_assign(&mut self, rhs: ::std::os::raw::c_double) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1145(self as *mut crate::Qfloat16, rhs)
        };
        let _ = ffi_result;
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Add<::std::os::raw::c_int> for &crate::Qfloat16 {
    type Output = ::std::os::raw::c_double;
    /// Calls C++ function: <span style='color: green;'>```double operator+(qfloat16 lhs, int rhs)```</span>.
    fn add(self, rhs: ::std::os::raw::c_int) -> ::std::os::raw::c_double {
        unsafe { crate::__ffi::ctr_qt_core_ffi_operator_465(self as *const crate::Qfloat16, rhs) }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Sub<::std::os::raw::c_int> for &crate::Qfloat16 {
    type Output = ::std::os::raw::c_double;
    /// Calls C++ function: <span style='color: green;'>```double operator-(qfloat16 lhs, int rhs)```</span>.
    fn sub(self, rhs: ::std::os::raw::c_int) -> ::std::os::raw::c_double {
        unsafe { crate::__ffi::ctr_qt_core_ffi_operator_467(self as *const crate::Qfloat16, rhs) }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Mul<::std::os::raw::c_int> for &crate::Qfloat16 {
    type Output = ::std::os::raw::c_double;
    /// Calls C++ function: <span style='color: green;'>```double operator*(qfloat16 lhs, int rhs)```</span>.
    fn mul(self, rhs: ::std::os::raw::c_int) -> ::std::os::raw::c_double {
        unsafe { crate::__ffi::ctr_qt_core_ffi_operator_469(self as *const crate::Qfloat16, rhs) }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Div<::std::os::raw::c_int> for &crate::Qfloat16 {
    type Output = ::std::os::raw::c_double;
    /// Calls C++ function: <span style='color: green;'>```double operator/(qfloat16 lhs, int rhs)```</span>.
    fn div(self, rhs: ::std::os::raw::c_int) -> ::std::os::raw::c_double {
        unsafe { crate::__ffi::ctr_qt_core_ffi_operator_471(self as *const crate::Qfloat16, rhs) }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::Qfloat16>> for crate::Qfloat16 {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(qfloat16 a, qfloat16 b)```</span>.
    fn gt(&self, b: &::cpp_utils::Ref<crate::Qfloat16>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_473(
                self as *const crate::Qfloat16,
                b.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::Qfloat16>> for crate::Qfloat16 {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(qfloat16 a, qfloat16 b)```</span>.
    fn lt(&self, b: &::cpp_utils::Ref<crate::Qfloat16>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_474(
                self as *const crate::Qfloat16,
                b.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::Qfloat16>> for crate::Qfloat16 {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(qfloat16 a, qfloat16 b)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, b: &::cpp_utils::Ref<crate::Qfloat16>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1150(
                self as *const crate::Qfloat16,
                b.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::Qfloat16>> for crate::Qfloat16 {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(qfloat16 a, qfloat16 b)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, b: &::cpp_utils::Ref<crate::Qfloat16>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1151(
                self as *const crate::Qfloat16,
                b.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::Qfloat16>> for crate::Qfloat16 {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(qfloat16 a, qfloat16 b)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, b: &::cpp_utils::Ref<crate::Qfloat16>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1152(
                self as *const crate::Qfloat16,
                b.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Lt<::std::os::raw::c_double> for crate::Qfloat16 {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(qfloat16 lhs, double rhs)```</span>.
    fn lt(&self, rhs: &::std::os::raw::c_double) -> bool {
        unsafe { crate::__ffi::ctr_qt_core_ffi_operator_479(self as *const crate::Qfloat16, *rhs) }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Gt<::std::os::raw::c_double> for crate::Qfloat16 {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(qfloat16 lhs, double rhs)```</span>.
    fn gt(&self, rhs: &::std::os::raw::c_double) -> bool {
        unsafe { crate::__ffi::ctr_qt_core_ffi_operator_481(self as *const crate::Qfloat16, *rhs) }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Ge<::std::os::raw::c_double> for crate::Qfloat16 {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(qfloat16 lhs, double rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::std::os::raw::c_double) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1162(self as *const crate::Qfloat16, *rhs)
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Le<::std::os::raw::c_double> for crate::Qfloat16 {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(qfloat16 lhs, double rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::std::os::raw::c_double) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1164(self as *const crate::Qfloat16, *rhs)
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::cmp::PartialEq<::std::os::raw::c_double> for crate::Qfloat16 {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(qfloat16 lhs, double rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, rhs: &::std::os::raw::c_double) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1166(self as *const crate::Qfloat16, *rhs)
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Gt<::std::os::raw::c_int> for crate::Qfloat16 {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(qfloat16 a, int b)```</span>.
    fn gt(&self, b: &::std::os::raw::c_int) -> bool {
        unsafe { crate::__ffi::ctr_qt_core_ffi_operator_487(self as *const crate::Qfloat16, *b) }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Lt<::std::os::raw::c_int> for crate::Qfloat16 {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(qfloat16 a, int b)```</span>.
    fn lt(&self, b: &::std::os::raw::c_int) -> bool {
        unsafe { crate::__ffi::ctr_qt_core_ffi_operator_489(self as *const crate::Qfloat16, *b) }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Ge<::std::os::raw::c_int> for crate::Qfloat16 {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(qfloat16 a, int b)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, b: &::std::os::raw::c_int) -> bool {
        unsafe { crate::__ffi::ctr_qt_core_ffi_operator__1178(self as *const crate::Qfloat16, *b) }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Le<::std::os::raw::c_int> for crate::Qfloat16 {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(qfloat16 a, int b)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, b: &::std::os::raw::c_int) -> bool {
        unsafe { crate::__ffi::ctr_qt_core_ffi_operator__1180(self as *const crate::Qfloat16, *b) }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::cmp::PartialEq<::std::os::raw::c_int> for crate::Qfloat16 {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(qfloat16 a, int b)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, b: &::std::os::raw::c_int) -> bool {
        unsafe { crate::__ffi::ctr_qt_core_ffi_operator__1182(self as *const crate::Qfloat16, *b) }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::CppDeletable for crate::QCborStreamWriter {
    /// <p>Destroys this <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> object and frees any resources associated.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QCborStreamWriter::~QCborStreamWriter()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html#dtor.QCborStreamWriter">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys this <a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> object and frees any resources associated.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qcborstreamwriter.html">QCborStreamWriter</a> does not perform error checking to see if all required items were written to the stream prior to the object being destroyed. It is the programmer's responsibility to ensure that it was done.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamWriter_dQCborStreamWriter(
            self as *mut crate::QCborStreamWriter,
        )
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::CppDeletable for crate::QCborStreamReader {
    /// <p>Destroys this <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> object and frees any associated resources.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QCborStreamReader::~QCborStreamReader()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborstreamreader.html#dtor.QCborStreamReader">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys this <a href="http://doc.qt.io/qt-5/qcborstreamreader.html">QCborStreamReader</a> object and frees any associated resources.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCborStreamReader_dQCborStreamReader(
            self as *mut crate::QCborStreamReader,
        )
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::CppDeletable for crate::QCborError {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QCborError::~QCborError()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborerror.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCborError_dQCborError(self as *mut crate::QCborError)
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::CppDeletable for crate::QCborParserError {
    /// <p>The <a href="http://doc.qt.io/qt-5/qcborparsererror.html">QCborParserError</a> is used by <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> to report a parsing error.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QCborParserError::~QCborParserError()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qcborparsererror.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qcborparsererror.html">QCborParserError</a> is used by <a href="http://doc.qt.io/qt-5/qcborvalue.html">QCborValue</a> to report a parsing error.</p>
    /// <p>This class is used by <a href="http://doc.qt.io/qt-5/qcborvalue.html#fromCbor-1">QCborValue::fromCbor</a>(const QByteArray &amp;ba, QCborParserError *error) to report a parser error and the byte offset where the error was detected.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCborParserError_dQCborParserError(
            self as *mut crate::QCborParserError,
        )
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::CppDeletable for crate::QCborValueRef {
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QCborValueRef::~QCborValueRef()```</span>.
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QCborValueRef_dQCborValueRef(
            self as *mut crate::QCborValueRef,
        )
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::CppDeletable for crate::Qfloat16 {
    /// <p>This header file provides support for half-precision (16-bit) floating point data with the class <code>qfloat16</code>. It is fully compliant with IEEE 754 as a storage type. This implies that any arithmetic operation on a <code>qfloat16</code> instance results in the value first being converted to a <code>float</code>. This conversion to and from <code>float</code> is performed by hardware when possible, but on processors that do not natively support half-precision, the conversion is performed through a sequence of lookup table operations.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void qfloat16::~qfloat16()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qfloat16.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This header file provides support for half-precision (16-bit) floating point data with the class <code>qfloat16</code>. It is fully compliant with IEEE 754 as a storage type. This implies that any arithmetic operation on a <code>qfloat16</code> instance results in the value first being converted to a <code>float</code>. This conversion to and from <code>float</code> is performed by hardware when possible, but on processors that do not natively support half-precision, the conversion is performed through a sequence of lookup table operations.</p>
    /// <p><code>qfloat16</code> should be treated as if it were a POD (plain old data) type. Consequently, none of the supported operations need any elaboration beyond stating that it supports all arithmetic operators incident to floating point types.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_qfloat16_dqfloat16(self as *mut crate::Qfloat16)
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QPairOfQCborValueRefQCborValueRef>>
    for crate::QPairOfQCborValueRefQCborValueRef
{
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(const QPair<QCborValueRef, QCborValueRef>& p1, const QPair<QCborValueRef, QCborValueRef>& p2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, p2: &::cpp_utils::Ref<crate::QPairOfQCborValueRefQCborValueRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1186(
                self as *const crate::QPairOfQCborValueRefQCborValueRef,
                p2.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QPairOfQCborValueQCborValue>>
    for crate::QPairOfQCborValueQCborValue
{
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(const QPair<QCborValue, QCborValue>& p1, const QPair<QCborValue, QCborValue>& p2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, p2: &::cpp_utils::Ref<crate::QPairOfQCborValueQCborValue>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1187(
                self as *const crate::QPairOfQCborValueQCborValue,
                p2.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QPairOfI64Uint>> for crate::QPairOfI64Uint {
    /// <p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator==(const QPair<qint64, unsigned int>& p1, const QPair<qint64, unsigned int>& p2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-eq-eq">C++ documentation</a> for <span style='color: green;'>```bool operator==(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>c1</i> and <i>c2</i> are the same Unicode character; otherwise returns <code>false</code>.</p></div>
    fn eq(&self, p2: &::cpp_utils::Ref<crate::QPairOfI64Uint>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1188(
                self as *const crate::QPairOfI64Uint,
                p2.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QPairOfQCborValueRefQCborValueRef>>
    for crate::QPairOfQCborValueRefQCborValueRef
{
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QPair<QCborValueRef, QCborValueRef>& p1, const QPair<QCborValueRef, QCborValueRef>& p2)```</span>.
    fn lt(&self, p2: &::cpp_utils::Ref<crate::QPairOfQCborValueRefQCborValueRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_491(
                self as *const crate::QPairOfQCborValueRefQCborValueRef,
                p2.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QPairOfQCborValueQCborValue>>
    for crate::QPairOfQCborValueQCborValue
{
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QPair<QCborValue, QCborValue>& p1, const QPair<QCborValue, QCborValue>& p2)```</span>.
    fn lt(&self, p2: &::cpp_utils::Ref<crate::QPairOfQCborValueQCborValue>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_492(
                self as *const crate::QPairOfQCborValueQCborValue,
                p2.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QPairOfI64Uint>> for crate::QPairOfI64Uint {
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QPair<qint64, unsigned int>& p1, const QPair<qint64, unsigned int>& p2)```</span>.
    fn lt(&self, p2: &::cpp_utils::Ref<crate::QPairOfI64Uint>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_493(
                self as *const crate::QPairOfI64Uint,
                p2.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QPairOfQCborValueRefQCborValueRef>>
    for crate::QPairOfQCborValueRefQCborValueRef
{
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QPair<QCborValueRef, QCborValueRef>& p1, const QPair<QCborValueRef, QCborValueRef>& p2)```</span>.
    fn gt(&self, p2: &::cpp_utils::Ref<crate::QPairOfQCborValueRefQCborValueRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_494(
                self as *const crate::QPairOfQCborValueRefQCborValueRef,
                p2.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QPairOfQCborValueQCborValue>>
    for crate::QPairOfQCborValueQCborValue
{
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QPair<QCborValue, QCborValue>& p1, const QPair<QCborValue, QCborValue>& p2)```</span>.
    fn gt(&self, p2: &::cpp_utils::Ref<crate::QPairOfQCborValueQCborValue>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_495(
                self as *const crate::QPairOfQCborValueQCborValue,
                p2.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QPairOfI64Uint>> for crate::QPairOfI64Uint {
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QPair<qint64, unsigned int>& p1, const QPair<qint64, unsigned int>& p2)```</span>.
    fn gt(&self, p2: &::cpp_utils::Ref<crate::QPairOfI64Uint>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_496(
                self as *const crate::QPairOfI64Uint,
                p2.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QPairOfQCborValueRefQCborValueRef>>
    for crate::QPairOfQCborValueRefQCborValueRef
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QPair<QCborValueRef, QCborValueRef>& p1, const QPair<QCborValueRef, QCborValueRef>& p2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, p2: &::cpp_utils::Ref<crate::QPairOfQCborValueRefQCborValueRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1192(
                self as *const crate::QPairOfQCborValueRefQCborValueRef,
                p2.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QPairOfQCborValueQCborValue>>
    for crate::QPairOfQCborValueQCborValue
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QPair<QCborValue, QCborValue>& p1, const QPair<QCborValue, QCborValue>& p2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, p2: &::cpp_utils::Ref<crate::QPairOfQCborValueQCborValue>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1193(
                self as *const crate::QPairOfQCborValueQCborValue,
                p2.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QPairOfI64Uint>> for crate::QPairOfI64Uint {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QPair<qint64, unsigned int>& p1, const QPair<qint64, unsigned int>& p2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, p2: &::cpp_utils::Ref<crate::QPairOfI64Uint>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1194(
                self as *const crate::QPairOfI64Uint,
                p2.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QPairOfQCborValueRefQCborValueRef>>
    for crate::QPairOfQCborValueRefQCborValueRef
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QPair<QCborValueRef, QCborValueRef>& p1, const QPair<QCborValueRef, QCborValueRef>& p2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, p2: &::cpp_utils::Ref<crate::QPairOfQCborValueRefQCborValueRef>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1195(
                self as *const crate::QPairOfQCborValueRefQCborValueRef,
                p2.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QPairOfQCborValueQCborValue>>
    for crate::QPairOfQCborValueQCborValue
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QPair<QCborValue, QCborValue>& p1, const QPair<QCborValue, QCborValue>& p2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, p2: &::cpp_utils::Ref<crate::QPairOfQCborValueQCborValue>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1196(
                self as *const crate::QPairOfQCborValueQCborValue,
                p2.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QPairOfI64Uint>> for crate::QPairOfI64Uint {
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QPair<qint64, unsigned int>& p1, const QPair<qint64, unsigned int>& p2)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, p2: &::cpp_utils::Ref<crate::QPairOfI64Uint>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1197(
                self as *const crate::QPairOfI64Uint,
                p2.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::CppDeletable for crate::QVectorOfQCborValue {
    /// <p>Destroys the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QVector<QCborValue>::~QVector()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#dtor.QVector">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys the vector.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_dQVector(
            self as *mut crate::QVectorOfQCborValue,
        )
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::cmp::PartialEq<::cpp_utils::Ref<crate::QVectorOfQCborValue>>
    for crate::QVectorOfQCborValue
{
    /// <p>Returns <code>true</code> if <i>other</i> is equal to this vector; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool QVector<QCborValue>::operator==(const QVector<QCborValue>& v) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-eq-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if <i>other</i> is equal to this vector; otherwise returns <code>false</code>.</p>
    /// <p>Two vectors are considered equal if they contain the same values in the same order.</p>
    /// <p>This function requires the value type to have an implementation of <code>operator==()</code>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-not-eq">operator!=</a>().</p></div>
    fn eq(&self, v: &::cpp_utils::Ref<crate::QVectorOfQCborValue>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_operator__(
                self as *const crate::QVectorOfQCborValue,
                v.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QVectorOfQCborValue>>
    for crate::QVectorOfQCborValue
{
    /// <p>Appends the items of the <i>other</i> vector to this vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QCborValue>& QVector<QCborValue>::operator+=(const QVector<QCborValue>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends the items of the <i>other</i> vector to this vector and returns a reference to this vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-2b">operator+</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>().</p></div>
    fn add_assign(&mut self, l: ::cpp_utils::Ref<crate::QVectorOfQCborValue>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_operator__4(
                self as *mut crate::QVectorOfQCborValue,
                l.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Add<::cpp_utils::Ref<crate::QVectorOfQCborValue>> for &crate::QVectorOfQCborValue {
    type Output = ::cpp_utils::CppBox<crate::QVectorOfQCborValue>;
    /// <p>Returns a vector that contains all the items in this vector followed by all the items in the <i>other</i> vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QCborValue> QVector<QCborValue>::operator+(const QVector<QCborValue>& l) const```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a vector that contains all the items in this vector followed by all the items in the <i>other</i> vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    fn add(
        self,
        l: ::cpp_utils::Ref<crate::QVectorOfQCborValue>,
    ) -> ::cpp_utils::CppBox<crate::QVectorOfQCborValue> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_operator_2(
                self as *const crate::QVectorOfQCborValue,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::AddAssign<::cpp_utils::Ref<crate::QCborValue>> for crate::QVectorOfQCborValue {
    /// <p>This is an overloaded function.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QCborValue>& QVector<QCborValue>::operator+=(const QCborValue& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
    /// <p>Appends <i>value</i> to the vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">operator&lt;&lt;</a>().</p></div>
    fn add_assign(&mut self, t: ::cpp_utils::Ref<crate::QCborValue>) {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_operator__5(
                self as *mut crate::QVectorOfQCborValue,
                t.as_raw_ptr(),
            )
        };
        let _ = ffi_result;
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QCborValue>> for &crate::QVectorOfQCborValue {
    type Output = ::cpp_utils::MutRef<crate::QVectorOfQCborValue>;
    /// <p>Appends <i>value</i> to the vector and returns a reference to this vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QCborValue>& QVector<QCborValue>::operator<<(const QCborValue& t)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>value</i> to the vector and returns a reference to this vector.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvector.html#append">append</a>() and <a href="http://doc.qt.io/qt-5/qvector.html#operator-2b-eq">operator+=</a>().</p></div>
    fn shl(
        self,
        t: ::cpp_utils::Ref<crate::QCborValue>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQCborValue> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_operator__6(
                self as *const crate::QVectorOfQCborValue as *mut crate::QVectorOfQCborValue,
                t.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVectorOfQCborValue>> for &crate::QVectorOfQCborValue {
    type Output = ::cpp_utils::MutRef<crate::QVectorOfQCborValue>;
    /// <p>Appends <i>other</i> to the vector and returns a reference to the vector.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QVector<QCborValue>& QVector<QCborValue>::operator<<(const QVector<QCborValue>& l)```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qvector.html#operator-lt-lt-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Appends <i>other</i> to the vector and returns a reference to the vector.</p></div>
    fn shl(
        self,
        l: ::cpp_utils::Ref<crate::QVectorOfQCborValue>,
    ) -> ::cpp_utils::MutRef<crate::QVectorOfQCborValue> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_QVector_QCborValue_operator__7(
                self as *const crate::QVectorOfQCborValue as *mut crate::QVectorOfQCborValue,
                l.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Lt<::cpp_utils::Ref<crate::QVectorOfQCborValue>>
    for crate::QVectorOfQCborValue
{
    /// Calls C++ function: <span style='color: green;'>```bool operator<(const QVector<QCborValue>& lhs, const QVector<QCborValue>& rhs)```</span>.
    fn lt(&self, rhs: &::cpp_utils::Ref<crate::QVectorOfQCborValue>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_497(
                self as *const crate::QVectorOfQCborValue,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Gt<::cpp_utils::Ref<crate::QVectorOfQCborValue>>
    for crate::QVectorOfQCborValue
{
    /// Calls C++ function: <span style='color: green;'>```bool operator>(const QVector<QCborValue>& lhs, const QVector<QCborValue>& rhs)```</span>.
    fn gt(&self, rhs: &::cpp_utils::Ref<crate::QVectorOfQCborValue>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator_498(
                self as *const crate::QVectorOfQCborValue,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Le<::cpp_utils::Ref<crate::QVectorOfQCborValue>>
    for crate::QVectorOfQCborValue
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator<=(const QVector<QCborValue>& lhs, const QVector<QCborValue>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-lt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator<=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is less than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn le(&self, rhs: &::cpp_utils::Ref<crate::QVectorOfQCborValue>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1198(
                self as *const crate::QVectorOfQCborValue,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::cmp::Ge<::cpp_utils::Ref<crate::QVectorOfQCborValue>>
    for crate::QVectorOfQCborValue
{
    /// <p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```bool operator>=(const QVector<QCborValue>& lhs, const QVector<QCborValue>& rhs)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qchar.html#operator-gt-eq">C++ documentation</a> for <span style='color: green;'>```bool operator>=(QChar c1, QChar c2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns <code>true</code> if the numeric Unicode value of <i>c1</i> is greater than or equal to that of <i>c2</i>; otherwise returns <code>false</code>.</p></div>
    fn ge(&self, rhs: &::cpp_utils::Ref<crate::QVectorOfQCborValue>) -> bool {
        unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1199(
                self as *const crate::QVectorOfQCborValue,
                rhs.as_raw_ptr(),
            )
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Shl<crate::QFlags<crate::q_cbor_value::EncodingOption>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<QCborValue::EncodingOption> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::q_cbor_value::EncodingOption>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1200(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Shl<crate::QFlags<crate::q_cbor_value::DiagnosticNotationOption>>
    for &crate::QDataStream
{
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, QFlags<QCborValue::DiagnosticNotationOption> e)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        e: crate::QFlags<crate::q_cbor_value::DiagnosticNotationOption>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1201(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                e.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QVectorOfQCborValue>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& s, QVector<QCborValue>& v)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        v: ::cpp_utils::MutRef<crate::QVectorOfQCborValue>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1204(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                v.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVectorOfQCborValue>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QVector<QCborValue>& v)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        v: ::cpp_utils::Ref<crate::QVectorOfQCborValue>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1205(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                v.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QPairOfQCborValueQCborValue>>
    for &crate::QDataStream
{
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& s, QPair<QCborValue, QCborValue>& p)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        p: ::cpp_utils::MutRef<crate::QPairOfQCborValueQCborValue>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1207(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                p.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QPairOfI64Uint>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& s, QPair<qint64, unsigned int>& p)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-gt-gt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator>>(QDataStream &in, QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Reads a bit array into <i>ba</i> from stream <i>in</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shr(
        self,
        p: ::cpp_utils::MutRef<crate::QPairOfI64Uint>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1208(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                p.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QPairOfQCborValueRefQCborValueRef>>
    for &crate::QDataStream
{
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QPair<QCborValueRef, QCborValueRef>& p)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        p: ::cpp_utils::Ref<crate::QPairOfQCborValueRefQCborValueRef>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1209(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                p.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QPairOfQCborValueQCborValue>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QPair<QCborValue, QCborValue>& p)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        p: ::cpp_utils::Ref<crate::QPairOfQCborValueQCborValue>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1210(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                p.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QPairOfI64Uint>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& s, const QPair<qint64, unsigned int>& p)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        p: ::cpp_utils::Ref<crate::QPairOfI64Uint>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1211(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                p.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QVectorOfQCborValue>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QVector<QCborValue>& vec)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        vec: ::cpp_utils::Ref<crate::QVectorOfQCborValue>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1212(
                self as *const crate::QDebug,
                vec.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QPairOfQCborValueRefQCborValueRef>>
    for &crate::QDebug
{
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QPair<QCborValueRef, QCborValueRef>& pair)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        pair: ::cpp_utils::Ref<crate::QPairOfQCborValueRefQCborValueRef>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1213(
                self as *const crate::QDebug,
                pair.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QPairOfQCborValueQCborValue>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QPair<QCborValue, QCborValue>& pair)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        pair: ::cpp_utils::Ref<crate::QPairOfQCborValueQCborValue>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1214(
                self as *const crate::QDebug,
                pair.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QPairOfI64Uint>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QPair<qint64, unsigned int>& pair)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        pair: ::cpp_utils::Ref<crate::QPairOfI64Uint>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1215(
                self as *const crate::QDebug,
                pair.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Shl<crate::QFlags<crate::q_cbor_value::EncodingOption>> for &crate::QDebug {
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<QCborValue::EncodingOption>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::q_cbor_value::EncodingOption>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1216(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::std::ops::Shl<crate::QFlags<crate::q_cbor_value::DiagnosticNotationOption>>
    for &crate::QDebug
{
    type Output = ::cpp_utils::CppBox<crate::QDebug>;
    /// <p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```QDebug operator<<(QDebug debug, const QFlags<QCborValue::DiagnosticNotationOption>& flags)```</span>.
    ///
    /// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbitarray.html#operator-lt-lt">C++ documentation</a> for <span style='color: green;'>```QDataStream &operator<<(QDataStream &out, const QBitArray &ba)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Writes bit array <i>ba</i> to stream <i>out</i>.</p>
    /// <p><b>See also </b><a href="http://doc.qt.io/qt-5/datastreamformat.html">Format of the QDataStream operators</a>.</p></div>
    fn shl(
        self,
        flags: crate::QFlags<crate::q_cbor_value::DiagnosticNotationOption>,
    ) -> ::cpp_utils::CppBox<crate::QDebug> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1217(
                self as *const crate::QDebug,
                flags.to_int(),
            )
        };
        unsafe {
            ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
        }
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::CppDeletable for crate::QPairOfQCborValueRefQCborValueRef {
    /// <p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QPair<QCborValueRef, QCborValueRef>::~QPair()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; can be used in your application if the STL <code>pair</code> type is not available. It stores one value of type T1 and one value of type T2. It can be used as a return value for a function that needs to return two values, or as the value type of a <a href="http://doc.qt.io/qt-5/containers.html">generic container</a>.</p>
    /// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> that stores one <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and one <code>double</code> value:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpair.html#QPair">QPair</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> pair;
    ///
    /// </pre>
    /// <p>The components are accessible as public data members called <a href="http://doc.qt.io/qt-5/qpair.html#first-var">first</a> and <a href="http://doc.qt.io/qt-5/qpair.html#second-var">second</a>. For example:</p>
    /// <pre class="cpp">
    ///
    ///   pair<span class="operator">.</span>first <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   pair<span class="operator">.</span>second <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>Note, however, that it is almost always preferable to define a small struct to hold the result of a function with multiple return values. A struct trivially generalizes to more than two values, and allows more descriptive member names than <code>first</code> and <code>second</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">struct</span> Variable {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> name;
    /// &#32;     <span class="type">double</span> value;
    ///   };
    ///   Variable v;
    ///   v<span class="operator">.</span>name <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   v<span class="operator">.</span>value <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>The advent of C++11 automatic variable type deduction (<code>auto</code>) shifts the emphasis from the type name to the name of functions and members. Thus, <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>, like <code>std::pair</code> and <code>std::tuple</code>, is mostly useful in generic (template) code, where defining a dedicated type is not possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>'s template data types (T1 and T2) must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; these requirements are documented on a per-function basis.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QPair_QCborValueRef_QCborValueRef_dQPair(
            self as *mut crate::QPairOfQCborValueRefQCborValueRef,
        )
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::CppDeletable for crate::QPairOfQCborValueQCborValue {
    /// <p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QPair<QCborValue, QCborValue>::~QPair()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; can be used in your application if the STL <code>pair</code> type is not available. It stores one value of type T1 and one value of type T2. It can be used as a return value for a function that needs to return two values, or as the value type of a <a href="http://doc.qt.io/qt-5/containers.html">generic container</a>.</p>
    /// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> that stores one <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and one <code>double</code> value:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpair.html#QPair">QPair</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> pair;
    ///
    /// </pre>
    /// <p>The components are accessible as public data members called <a href="http://doc.qt.io/qt-5/qpair.html#first-var">first</a> and <a href="http://doc.qt.io/qt-5/qpair.html#second-var">second</a>. For example:</p>
    /// <pre class="cpp">
    ///
    ///   pair<span class="operator">.</span>first <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   pair<span class="operator">.</span>second <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>Note, however, that it is almost always preferable to define a small struct to hold the result of a function with multiple return values. A struct trivially generalizes to more than two values, and allows more descriptive member names than <code>first</code> and <code>second</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">struct</span> Variable {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> name;
    /// &#32;     <span class="type">double</span> value;
    ///   };
    ///   Variable v;
    ///   v<span class="operator">.</span>name <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   v<span class="operator">.</span>value <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>The advent of C++11 automatic variable type deduction (<code>auto</code>) shifts the emphasis from the type name to the name of functions and members. Thus, <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>, like <code>std::pair</code> and <code>std::tuple</code>, is mostly useful in generic (template) code, where defining a dedicated type is not possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>'s template data types (T1 and T2) must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; these requirements are documented on a per-function basis.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QPair_QCborValue_QCborValue_dQPair(
            self as *mut crate::QPairOfQCborValueQCborValue,
        )
    }
}

#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
impl ::cpp_utils::CppDeletable for crate::QPairOfI64Uint {
    /// <p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```[destructor] void QPair<qint64, unsigned int>::~QPair()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qpair.html">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> class is a template class that stores a pair of items.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; can be used in your application if the STL <code>pair</code> type is not available. It stores one value of type T1 and one value of type T2. It can be used as a return value for a function that needs to return two values, or as the value type of a <a href="http://doc.qt.io/qt-5/containers.html">generic container</a>.</p>
    /// <p>Here's an example of a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a> that stores one <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> and one <code>double</code> value:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="type"><a href="http://doc.qt.io/qt-5/qpair.html#QPair">QPair</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> pair;
    ///
    /// </pre>
    /// <p>The components are accessible as public data members called <a href="http://doc.qt.io/qt-5/qpair.html#first-var">first</a> and <a href="http://doc.qt.io/qt-5/qpair.html#second-var">second</a>. For example:</p>
    /// <pre class="cpp">
    ///
    ///   pair<span class="operator">.</span>first <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   pair<span class="operator">.</span>second <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>Note, however, that it is almost always preferable to define a small struct to hold the result of a function with multiple return values. A struct trivially generalizes to more than two values, and allows more descriptive member names than <code>first</code> and <code>second</code>:</p>
    /// <pre class="cpp">
    ///
    ///   <span class="keyword">struct</span> Variable {
    /// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> name;
    /// &#32;     <span class="type">double</span> value;
    ///   };
    ///   Variable v;
    ///   v<span class="operator">.</span>name <span class="operator">=</span> <span class="string">"pi"</span>;
    ///   v<span class="operator">.</span>value <span class="operator">=</span> <span class="number">3.14159265358979323846</span>;
    ///
    /// </pre>
    /// <p>The advent of C++11 automatic variable type deduction (<code>auto</code>) shifts the emphasis from the type name to the name of functions and members. Thus, <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>, like <code>std::pair</code> and <code>std::tuple</code>, is mostly useful in generic (template) code, where defining a dedicated type is not possible.</p>
    /// <p><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>'s template data types (T1 and T2) must be <a href="http://doc.qt.io/qt-5/containers.html#assignable-data-types">assignable data types</a>. You cannot, for example, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> as a value; instead, store a <a href="http://doc.qt.io/qt-5/qwidget.html">QWidget</a> *. A few functions have additional requirements; these requirements are documented on a per-function basis.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QPair_qint64_unsigned_int_dQPair(
            self as *mut crate::QPairOfI64Uint,
        )
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QCborValue>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, const QCborValue& arg2)```</span>.
    fn shl(
        self,
        arg2: ::cpp_utils::Ref<crate::QCborValue>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1310(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QCborValue>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QCborValue& arg2)```</span>.
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QCborValue>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1311(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QCborArray>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, const QCborArray& arg2)```</span>.
    fn shl(
        self,
        arg2: ::cpp_utils::Ref<crate::QCborArray>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1312(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QCborArray>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QCborArray& arg2)```</span>.
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QCborArray>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1313(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QCborMap>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, const QCborMap& arg2)```</span>.
    fn shl(
        self,
        arg2: ::cpp_utils::Ref<crate::QCborMap>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1314(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QCborMap>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QCborMap& arg2)```</span>.
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QCborMap>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1315(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::cpp_utils::CppDeletable for crate::QConcatenateTablesProxyModel {
    /// <p>Destroys this proxy model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QConcatenateTablesProxyModel::~QConcatenateTablesProxyModel()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qconcatenatetablesproxymodel.html#dtor.QConcatenateTablesProxyModel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destroys this proxy model.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QConcatenateTablesProxyModel_dQConcatenateTablesProxyModel(
            self as *mut crate::QConcatenateTablesProxyModel,
        )
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QJsonValue>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, const QJsonValue& arg2)```</span>.
    fn shl(
        self,
        arg2: ::cpp_utils::Ref<crate::QJsonValue>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1316(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QJsonValue>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QJsonValue& arg2)```</span>.
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QJsonValue>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1317(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QJsonArray>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, const QJsonArray& arg2)```</span>.
    fn shl(
        self,
        arg2: ::cpp_utils::Ref<crate::QJsonArray>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1318(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QJsonArray>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QJsonArray& arg2)```</span>.
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QJsonArray>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1319(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QJsonDocument>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, const QJsonDocument& arg2)```</span>.
    fn shl(
        self,
        arg2: ::cpp_utils::Ref<crate::QJsonDocument>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1320(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QJsonDocument>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QJsonDocument& arg2)```</span>.
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QJsonDocument>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1321(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::std::ops::Shl<::cpp_utils::Ref<crate::QJsonObject>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator<<(QDataStream& arg1, const QJsonObject& arg2)```</span>.
    fn shl(
        self,
        arg2: ::cpp_utils::Ref<crate::QJsonObject>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1322(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::std::ops::Shr<::cpp_utils::MutRef<crate::QJsonObject>> for &crate::QDataStream {
    type Output = ::cpp_utils::MutRef<crate::QDataStream>;
    /// Calls C++ function: <span style='color: green;'>```QDataStream& operator>>(QDataStream& arg1, QJsonObject& arg2)```</span>.
    fn shr(
        self,
        arg2: ::cpp_utils::MutRef<crate::QJsonObject>,
    ) -> ::cpp_utils::MutRef<crate::QDataStream> {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_operator__1323(
                self as *const crate::QDataStream as *mut crate::QDataStream,
                arg2.as_mut_raw_ptr(),
            )
        };
        unsafe {
            ::cpp_utils::MutRef::from_raw(ffi_result).expect("attempted to construct a null Ref")
        }
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::cpp_utils::CppDeletable for crate::QTransposeProxyModel {
    /// <p>Destructs the proxy model.</p>
    ///
    /// Calls C++ function: <span style='color: green;'>```virtual [destructor] void QTransposeProxyModel::~QTransposeProxyModel()```</span>.
    ///
    /// <a href="http://doc.qt.io/qt-5/qtransposeproxymodel.html#dtor.QTransposeProxyModel">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Destructs the proxy model.</p></div>
    unsafe fn delete(&mut self) {
        crate::__ffi::ctr_qt_core_ffi_QTransposeProxyModel_dQTransposeProxyModel(
            self as *mut crate::QTransposeProxyModel,
        )
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::cpp_utils::StaticDowncast<crate::QConcatenateTablesProxyModel>
    for crate::QAbstractItemModel
{
    /// Calls C++ function: <span style='color: green;'>```QConcatenateTablesProxyModel* static_cast<QConcatenateTablesProxyModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::Ptr<crate::QConcatenateTablesProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QConcatenateTablesProxyModel_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractItemModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QConcatenateTablesProxyModel* static_cast<QConcatenateTablesProxyModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::MutPtr<crate::QConcatenateTablesProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QConcatenateTablesProxyModel_ptr(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::cpp_utils::StaticUpcast<crate::QAbstractItemModel> for crate::QConcatenateTablesProxyModel {
    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* static_cast<QAbstractItemModel*>(QConcatenateTablesProxyModel* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QConcatenateTablesProxyModel>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractItemModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractItemModel_ptr14(
            ptr.as_raw_ptr() as *mut crate::QConcatenateTablesProxyModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* static_cast<QAbstractItemModel*>(QConcatenateTablesProxyModel* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QConcatenateTablesProxyModel>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractItemModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractItemModel_ptr14(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::std::ops::Deref for crate::QConcatenateTablesProxyModel {
    type Target = crate::QAbstractItemModel;
    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* static_cast<QAbstractItemModel*>(QConcatenateTablesProxyModel* ptr)```</span>.
    fn deref(&self) -> &crate::QAbstractItemModel {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractItemModel_ptr14(
                self as *const crate::QConcatenateTablesProxyModel
                    as *mut crate::QConcatenateTablesProxyModel,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::std::ops::DerefMut for crate::QConcatenateTablesProxyModel {
    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* static_cast<QAbstractItemModel*>(QConcatenateTablesProxyModel* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QAbstractItemModel {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractItemModel_ptr14(
                self as *mut crate::QConcatenateTablesProxyModel,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::cpp_utils::DynamicCast<crate::QConcatenateTablesProxyModel> for crate::QAbstractItemModel {
    /// Calls C++ function: <span style='color: green;'>```QConcatenateTablesProxyModel* dynamic_cast<QConcatenateTablesProxyModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::Ptr<crate::QConcatenateTablesProxyModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QConcatenateTablesProxyModel_ptr(
                ptr.as_raw_ptr() as *mut crate::QAbstractItemModel,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QConcatenateTablesProxyModel* dynamic_cast<QConcatenateTablesProxyModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::MutPtr<crate::QConcatenateTablesProxyModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QConcatenateTablesProxyModel_ptr(
                ptr.as_mut_raw_ptr(),
            );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::cpp_utils::StaticDowncast<crate::QConcatenateTablesProxyModel> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QConcatenateTablesProxyModel* static_cast<QConcatenateTablesProxyModel*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QConcatenateTablesProxyModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QConcatenateTablesProxyModel_ptr1(
                ptr.as_raw_ptr() as *mut crate::QObject,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QConcatenateTablesProxyModel* static_cast<QConcatenateTablesProxyModel*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QConcatenateTablesProxyModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QConcatenateTablesProxyModel_ptr1(
                ptr.as_mut_raw_ptr(),
            );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QConcatenateTablesProxyModel {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QConcatenateTablesProxyModel* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QConcatenateTablesProxyModel>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr168(
            ptr.as_raw_ptr() as *mut crate::QConcatenateTablesProxyModel
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QConcatenateTablesProxyModel* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QConcatenateTablesProxyModel>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr168(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::cpp_utils::DynamicCast<crate::QConcatenateTablesProxyModel> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QConcatenateTablesProxyModel* dynamic_cast<QConcatenateTablesProxyModel*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QConcatenateTablesProxyModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QConcatenateTablesProxyModel_ptr1(
                ptr.as_raw_ptr() as *mut crate::QObject,
            );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QConcatenateTablesProxyModel* dynamic_cast<QConcatenateTablesProxyModel*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QConcatenateTablesProxyModel> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QConcatenateTablesProxyModel_ptr1(
                ptr.as_mut_raw_ptr(),
            );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::cpp_utils::StaticDowncast<crate::QTransposeProxyModel> for crate::QAbstractProxyModel {
    /// Calls C++ function: <span style='color: green;'>```QTransposeProxyModel* static_cast<QTransposeProxyModel*>(QAbstractProxyModel* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractProxyModel>,
    ) -> ::cpp_utils::Ptr<crate::QTransposeProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QTransposeProxyModel_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractProxyModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QTransposeProxyModel* static_cast<QTransposeProxyModel*>(QAbstractProxyModel* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractProxyModel>,
    ) -> ::cpp_utils::MutPtr<crate::QTransposeProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QTransposeProxyModel_ptr(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::cpp_utils::StaticUpcast<crate::QAbstractProxyModel> for crate::QTransposeProxyModel {
    /// Calls C++ function: <span style='color: green;'>```QAbstractProxyModel* static_cast<QAbstractProxyModel*>(QTransposeProxyModel* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QTransposeProxyModel>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractProxyModel_ptr8(
            ptr.as_raw_ptr() as *mut crate::QTransposeProxyModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractProxyModel* static_cast<QAbstractProxyModel*>(QTransposeProxyModel* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QTransposeProxyModel>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractProxyModel_ptr8(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::std::ops::Deref for crate::QTransposeProxyModel {
    type Target = crate::QAbstractProxyModel;
    /// Calls C++ function: <span style='color: green;'>```QAbstractProxyModel* static_cast<QAbstractProxyModel*>(QTransposeProxyModel* ptr)```</span>.
    fn deref(&self) -> &crate::QAbstractProxyModel {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractProxyModel_ptr8(
                self as *const crate::QTransposeProxyModel as *mut crate::QTransposeProxyModel,
            )
        };
        unsafe { ffi_result.as_ref() }.expect("Attempted to convert null pointer to reference")
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::std::ops::DerefMut for crate::QTransposeProxyModel {
    /// Calls C++ function: <span style='color: green;'>```QAbstractProxyModel* static_cast<QAbstractProxyModel*>(QTransposeProxyModel* ptr)```</span>.
    fn deref_mut(&mut self) -> &mut crate::QAbstractProxyModel {
        let ffi_result = unsafe {
            crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractProxyModel_ptr8(
                self as *mut crate::QTransposeProxyModel,
            )
        };
        unsafe { ffi_result.as_mut() }.expect("Attempted to convert null pointer to reference")
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::cpp_utils::DynamicCast<crate::QTransposeProxyModel> for crate::QAbstractProxyModel {
    /// Calls C++ function: <span style='color: green;'>```QTransposeProxyModel* dynamic_cast<QTransposeProxyModel*>(QAbstractProxyModel* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractProxyModel>,
    ) -> ::cpp_utils::Ptr<crate::QTransposeProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QTransposeProxyModel_ptr(
            ptr.as_raw_ptr() as *mut crate::QAbstractProxyModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QTransposeProxyModel* dynamic_cast<QTransposeProxyModel*>(QAbstractProxyModel* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractProxyModel>,
    ) -> ::cpp_utils::MutPtr<crate::QTransposeProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QTransposeProxyModel_ptr(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::cpp_utils::StaticDowncast<crate::QTransposeProxyModel> for crate::QAbstractItemModel {
    /// Calls C++ function: <span style='color: green;'>```QTransposeProxyModel* static_cast<QTransposeProxyModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::Ptr<crate::QTransposeProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QTransposeProxyModel_ptr1(
            ptr.as_raw_ptr() as *mut crate::QAbstractItemModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QTransposeProxyModel* static_cast<QTransposeProxyModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::MutPtr<crate::QTransposeProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QTransposeProxyModel_ptr1(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::cpp_utils::StaticUpcast<crate::QAbstractItemModel> for crate::QTransposeProxyModel {
    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* static_cast<QAbstractItemModel*>(QTransposeProxyModel* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QTransposeProxyModel>,
    ) -> ::cpp_utils::Ptr<crate::QAbstractItemModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractItemModel_ptr15(
            ptr.as_raw_ptr() as *mut crate::QTransposeProxyModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QAbstractItemModel* static_cast<QAbstractItemModel*>(QTransposeProxyModel* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QTransposeProxyModel>,
    ) -> ::cpp_utils::MutPtr<crate::QAbstractItemModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QAbstractItemModel_ptr15(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::cpp_utils::DynamicCast<crate::QTransposeProxyModel> for crate::QAbstractItemModel {
    /// Calls C++ function: <span style='color: green;'>```QTransposeProxyModel* dynamic_cast<QTransposeProxyModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::Ptr<crate::QTransposeProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QTransposeProxyModel_ptr1(
            ptr.as_raw_ptr() as *mut crate::QAbstractItemModel,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QTransposeProxyModel* dynamic_cast<QTransposeProxyModel*>(QAbstractItemModel* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QAbstractItemModel>,
    ) -> ::cpp_utils::MutPtr<crate::QTransposeProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QTransposeProxyModel_ptr1(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::cpp_utils::StaticDowncast<crate::QTransposeProxyModel> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QTransposeProxyModel* static_cast<QTransposeProxyModel*>(QObject* ptr)```</span>.
    unsafe fn static_downcast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QTransposeProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QTransposeProxyModel_ptr2(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QTransposeProxyModel* static_cast<QTransposeProxyModel*>(QObject* ptr)```</span>.
    unsafe fn static_downcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QTransposeProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QTransposeProxyModel_ptr2(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::cpp_utils::StaticUpcast<crate::QObject> for crate::QTransposeProxyModel {
    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QTransposeProxyModel* ptr)```</span>.
    unsafe fn static_upcast(
        ptr: ::cpp_utils::Ptr<crate::QTransposeProxyModel>,
    ) -> ::cpp_utils::Ptr<crate::QObject> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr169(
            ptr.as_raw_ptr() as *mut crate::QTransposeProxyModel
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QObject* static_cast<QObject*>(QTransposeProxyModel* ptr)```</span>.
    unsafe fn static_upcast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QTransposeProxyModel>,
    ) -> ::cpp_utils::MutPtr<crate::QObject> {
        let ffi_result =
            crate::__ffi::ctr_qt_core_ffi_static_cast_QObject_ptr169(ptr.as_mut_raw_ptr());
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

#[cfg(any(cpp_lib_version = "5.13.0", ritual_rustdoc))]
impl ::cpp_utils::DynamicCast<crate::QTransposeProxyModel> for crate::QObject {
    /// Calls C++ function: <span style='color: green;'>```QTransposeProxyModel* dynamic_cast<QTransposeProxyModel*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast(
        ptr: ::cpp_utils::Ptr<crate::QObject>,
    ) -> ::cpp_utils::Ptr<crate::QTransposeProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QTransposeProxyModel_ptr2(
            ptr.as_raw_ptr() as *mut crate::QObject,
        );
        ::cpp_utils::Ptr::from_raw(ffi_result)
    }

    /// Calls C++ function: <span style='color: green;'>```QTransposeProxyModel* dynamic_cast<QTransposeProxyModel*>(QObject* ptr)```</span>.
    unsafe fn dynamic_cast_mut(
        ptr: ::cpp_utils::MutPtr<crate::QObject>,
    ) -> ::cpp_utils::MutPtr<crate::QTransposeProxyModel> {
        let ffi_result = crate::__ffi::ctr_qt_core_ffi_dynamic_cast_QTransposeProxyModel_ptr2(
            ptr.as_mut_raw_ptr(),
        );
        ::cpp_utils::MutPtr::from_raw(ffi_result)
    }
}

/// <p>Returns the arccosine of <i>v</i> as an angle in radians. Arccosine is the inverse operation of cosine.</p>
///
/// Calls C++ function: <span style='color: green;'>```double qAcos(double v)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtmath.html#qAcos">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the arccosine of <i>v</i> as an angle in radians. Arccosine is the inverse operation of cosine.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtmath.html#qAtan">qAtan</a>(), <a href="http://doc.qt.io/qt-5/qtmath.html#qAsin">qAsin</a>(), and <a href="http://doc.qt.io/qt-5/qtmath.html#qCos">qCos</a>().</p></div>
pub unsafe fn q_acos(v: ::std::os::raw::c_double) -> ::std::os::raw::c_double {
    crate::__ffi::ctr_qt_core_ffi_qAcos(v)
}

/// <p>Adds a global routine that will be called from the <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> destructor. This function is normally used to add cleanup routines for program-wide functionality.</p>
///
/// Calls C++ function: <span style='color: green;'>```void qAddPostRoutine(void (*FN_PTR)() arg1)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#qAddPostRoutine">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Adds a global routine that will be called from the <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> destructor. This function is normally used to add cleanup routines for program-wide functionality.</p>
/// <p>The cleanup routines are called in the reverse order of their addition.</p>
/// <p>The function specified by <i>ptr</i> should take no arguments and should return nothing. For example:</p>
/// <pre class="cpp">
///
///   <span class="keyword">static</span> <span class="type">int</span> <span class="operator">*</span>global_ptr <span class="operator">=</span> <span class="number">0</span>;
///
///   <span class="keyword">static</span> <span class="type">void</span> cleanup_ptr()
///   {
/// &#32;     <span class="keyword">delete</span> <span class="operator">[</span><span class="operator">]</span> global_ptr;
/// &#32;     global_ptr <span class="operator">=</span> <span class="number">0</span>;
///   }
///
///   <span class="type">void</span> init_ptr()
///   {
/// &#32;     global_ptr <span class="operator">=</span> <span class="keyword">new</span> <span class="type">int</span><span class="operator">[</span><span class="number">100</span><span class="operator">]</span>;&#32;     <span class="comment">// allocate data</span>
/// &#32;     qAddPostRoutine(cleanup_ptr);   <span class="comment">// delete later</span>
///   }
///
/// </pre>
/// <p>Note that for an application- or module-wide cleanup, qaddPostRoutine() is often not suitable. For example, if the program is split into dynamically loaded modules, the relevant module may be unloaded long before the <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> destructor is called. In such cases, if using qaddPostRoutine() is still desirable, <a href="http://doc.qt.io/qt-5/qcoreapplication.html#qRemovePostRoutine">qRemovePostRoutine</a>() can be used to prevent a routine from being called by the <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> destructor. For example, if that routine was called before the module was unloaded.</p>
/// <p>For modules and libraries, using a reference-counted initialization manager or Qt's parent-child deletion mechanism may be better. Here is an example of a private class that uses the parent-child mechanism to call a cleanup function at the right time:</p>
/// <pre class="cpp">
///
///   <span class="keyword">class</span> MyPrivateInitStuff : <span class="keyword">public</span> <span class="type"><a href="http://doc.qt.io/qt-5/qobject.html">QObject</a></span>
///   {
///   <span class="keyword">public</span>:
/// &#32;     <span class="keyword">static</span> MyPrivateInitStuff <span class="operator">*</span>initStuff(<span class="type"><a href="http://doc.qt.io/qt-5/qobject.html">QObject</a></span> <span class="operator">*</span>parent)
/// &#32;     {
/// &#32;   &#32;     <span class="keyword">if</span> (<span class="operator">!</span>p)
/// &#32;   &#32;   &#32;     p <span class="operator">=</span> <span class="keyword">new</span> MyPrivateInitStuff(parent);
/// &#32;   &#32;     <span class="keyword">return</span> p;
/// &#32;     }
///
/// &#32;     <span class="operator">~</span>MyPrivateInitStuff()
/// &#32;     {
/// &#32;   &#32;     <span class="comment">// cleanup goes here</span>
/// &#32;     }
///
///   <span class="keyword">private</span>:
/// &#32;     MyPrivateInitStuff(<span class="type"><a href="http://doc.qt.io/qt-5/qobject.html">QObject</a></span> <span class="operator">*</span>parent)
/// &#32;   &#32;     : <span class="type"><a href="http://doc.qt.io/qt-5/qobject.html">QObject</a></span>(parent)
/// &#32;     {
/// &#32;   &#32;     <span class="comment">// initialization goes here</span>
/// &#32;     }
///
/// &#32;     MyPrivateInitStuff <span class="operator">*</span>p;
///   };
///
/// </pre>
/// <p>By selecting the right parent object, this can often be made to clean up the module's data at the right moment.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#qRemovePostRoutine">qRemovePostRoutine</a>().</p></div>
pub unsafe fn q_add_post_routine(arg1: ::std::option::Option<extern "C" fn()>) {
    crate::__ffi::ctr_qt_core_ffi_qAddPostRoutine(arg1)
}

/// Calls C++ function: <span style='color: green;'>```void qAddPreRoutine(void (*FN_PTR)() arg1)```</span>.
pub unsafe fn q_add_pre_routine(arg1: ::std::option::Option<extern "C" fn()>) {
    crate::__ffi::ctr_qt_core_ffi_qAddPreRoutine(arg1)
}

/// Calls C++ function: <span style='color: green;'>```QString qAppName()```</span>.
pub unsafe fn q_app_name() -> ::cpp_utils::CppBox<crate::QString> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qAppName();
    ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
}

/// <p>Returns the arcsine of <i>v</i> as an angle in radians. Arcsine is the inverse operation of sine.</p>
///
/// Calls C++ function: <span style='color: green;'>```double qAsin(double v)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtmath.html#qAsin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the arcsine of <i>v</i> as an angle in radians. Arcsine is the inverse operation of sine.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtmath.html#qSin">qSin</a>(), <a href="http://doc.qt.io/qt-5/qtmath.html#qAtan">qAtan</a>(), and <a href="http://doc.qt.io/qt-5/qtmath.html#qAcos">qAcos</a>().</p></div>
pub unsafe fn q_asin(v: ::std::os::raw::c_double) -> ::std::os::raw::c_double {
    crate::__ffi::ctr_qt_core_ffi_qAsin(v)
}

/// <p>Returns the arctangent of <i>v</i> as an angle in radians. Arctangent is the inverse operation of tangent.</p>
///
/// Calls C++ function: <span style='color: green;'>```double qAtan(double v)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtmath.html#qAtan">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the arctangent of <i>v</i> as an angle in radians. Arctangent is the inverse operation of tangent.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtmath.html#qTan">qTan</a>(), <a href="http://doc.qt.io/qt-5/qtmath.html#qAcos">qAcos</a>(), and <a href="http://doc.qt.io/qt-5/qtmath.html#qAsin">qAsin</a>().</p></div>
pub unsafe fn q_atan(v: ::std::os::raw::c_double) -> ::std::os::raw::c_double {
    crate::__ffi::ctr_qt_core_ffi_qAtan(v)
}

/// <p>Returns the arctangent of a point specified by the coordinates <i>y</i> and <i>x</i>. This function will return the angle (argument) of that point.</p>
///
/// Calls C++ function: <span style='color: green;'>```double qAtan2(double y, double x)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtmath.html#qAtan2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the arctangent of a point specified by the coordinates <i>y</i> and <i>x</i>. This function will return the angle (argument) of that point.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtmath.html#qAtan">qAtan</a>().</p></div>
pub unsafe fn q_atan2(
    y: ::std::os::raw::c_double,
    x: ::std::os::raw::c_double,
) -> ::std::os::raw::c_double {
    crate::__ffi::ctr_qt_core_ffi_qAtan2(y, x)
}

/// <p>Return the ceiling of the value <i>v</i>.</p>
///
/// Calls C++ function: <span style='color: green;'>```int qCeil(double v)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtmath.html#qCeil">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Return the ceiling of the value <i>v</i>.</p>
/// <p>The ceiling is the smallest integer that is not less than <i>v</i>. For example, if <i>v</i> is 41.2, then the ceiling is 42.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtmath.html#qFloor">qFloor</a>().</p></div>
pub unsafe fn q_ceil(v: ::std::os::raw::c_double) -> ::std::os::raw::c_int {
    crate::__ffi::ctr_qt_core_ffi_qCeil(v)
}

/// <p>Returns the CRC-16 checksum of the first <i>len</i> bytes of <i>data</i>.</p>
///
/// Calls C++ function: <span style='color: green;'>```quint16 qChecksum(const char* s, unsigned int len)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qbytearray.html#qChecksum">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the CRC-16 checksum of the first <i>len</i> bytes of <i>data</i>.</p>
/// <p>The checksum is independent of the byte order (endianness) and will be calculated accorded to the algorithm published in ISO 3309 (<a href="http://doc.qt.io/qt-5/qt.html#ChecksumType-enum">Qt::ChecksumIso3309</a>).</p>
/// <p><b>Note: </b>This function is a 16-bit cache conserving (16 entry table) implementation of the CRC-16-CCITT algorithm.</p></div>
pub unsafe fn q_checksum_2a(
    s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    len: ::std::os::raw::c_uint,
) -> u16 {
    crate::__ffi::ctr_qt_core_ffi_qChecksum(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
            .as_raw_ptr(),
        len,
    )
}

/// <p>Returns the CRC-16 checksum of the first <i>len</i> bytes of <i>data</i>.</p>
///
/// Calls C++ function: <span style='color: green;'>```quint16 qChecksum(const char* s, unsigned int len, Qt::ChecksumType standard)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qbytearray.html#qChecksum-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the CRC-16 checksum of the first <i>len</i> bytes of <i>data</i>.</p>
/// <p>The checksum is independent of the byte order (endianness) and will be calculated accorded to the algorithm published in <i>standard</i>.</p>
/// <p><b>Note: </b>This function is a 16-bit cache conserving (16 entry table) implementation of the CRC-16-CCITT algorithm.</p><p>This function was introduced in  Qt 5.9.</p></div>
pub unsafe fn q_checksum_3a(
    s: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    len: ::std::os::raw::c_uint,
    standard: crate::ChecksumType,
) -> u16 {
    crate::__ffi::ctr_qt_core_ffi_qChecksum1(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(s)
            .as_raw_ptr(),
        len,
        standard,
    )
}

/// <p>This is an overloaded function.</p>
///
/// Calls C++ function: <span style='color: green;'>```QByteArray qCompress(const unsigned char* data, int nbytes, int compressionLevel = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qbytearray.html#qCompress">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
/// <p>Compresses the first <i>nbytes</i> of <i>data</i> at compression level <i>compressionLevel</i> and returns the compressed data in a new byte array.</p></div>
pub unsafe fn q_compress_uchar2_int(
    data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_uchar>>,
    nbytes: ::std::os::raw::c_int,
    compression_level: ::std::os::raw::c_int,
) -> ::cpp_utils::CppBox<crate::QByteArray> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qCompress(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_uchar>>::cast_into(data)
            .as_raw_ptr(),
        nbytes,
        compression_level,
    );
    ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
}

/// <p>Compresses the <i>data</i> byte array and returns the compressed data in a new byte array.</p>
///
/// Calls C++ function: <span style='color: green;'>```QByteArray qCompress(const QByteArray& data, int compressionLevel = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qbytearray.html#qCompress">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compresses the <i>data</i> byte array and returns the compressed data in a new byte array.</p>
/// <p>The <i>compressionLevel</i> parameter specifies how much compression should be used. Valid values are between 0 and 9, with 9 corresponding to the greatest compression (i.e. smaller compressed data) at the cost of using a slower algorithm. Smaller values (8, 7, ..., 1) provide successively less compression at slightly faster speeds. The value 0 corresponds to no compression at all. The default value is -1, which specifies zlib's default compression.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#qUncompress">qUncompress</a>().</p></div>
pub unsafe fn q_compress_q_byte_array_int(
    data: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    compression_level: ::std::os::raw::c_int,
) -> ::cpp_utils::CppBox<crate::QByteArray> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qCompress1(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(data).as_raw_ptr(),
        compression_level,
    );
    ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
}

/// <p>This is an overloaded function.</p>
///
/// Calls C++ function: <span style='color: green;'>```QByteArray qCompress(const unsigned char* data, int nbytes)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qbytearray.html#qCompress">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
/// <p>Compresses the first <i>nbytes</i> of <i>data</i> at compression level <i>compressionLevel</i> and returns the compressed data in a new byte array.</p></div>
pub unsafe fn q_compress_uchar_int(
    data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_uchar>>,
    nbytes: ::std::os::raw::c_int,
) -> ::cpp_utils::CppBox<crate::QByteArray> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qCompress2(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_uchar>>::cast_into(data)
            .as_raw_ptr(),
        nbytes,
    );
    ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
}

/// <p>Compresses the <i>data</i> byte array and returns the compressed data in a new byte array.</p>
///
/// Calls C++ function: <span style='color: green;'>```QByteArray qCompress(const QByteArray& data)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qbytearray.html#qCompress">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compresses the <i>data</i> byte array and returns the compressed data in a new byte array.</p>
/// <p>The <i>compressionLevel</i> parameter specifies how much compression should be used. Valid values are between 0 and 9, with 9 corresponding to the greatest compression (i.e. smaller compressed data) at the cost of using a slower algorithm. Smaller values (8, 7, ..., 1) provide successively less compression at slightly faster speeds. The value 0 corresponds to no compression at all. The default value is -1, which specifies zlib's default compression.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#qUncompress">qUncompress</a>().</p></div>
pub unsafe fn q_compress_q_byte_array(
    data: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
) -> ::cpp_utils::CppBox<crate::QByteArray> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qCompress3(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(data).as_raw_ptr(),
    );
    ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
}

/// <p>Returns the cosine of an angle <i>v</i> in radians.</p>
///
/// Calls C++ function: <span style='color: green;'>```double qCos(double v)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtmath.html#qCos">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the cosine of an angle <i>v</i> in radians.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtmath.html#qSin">qSin</a>() and <a href="http://doc.qt.io/qt-5/qtmath.html#qTan">qTan</a>().</p></div>
pub unsafe fn q_cos(v: ::std::os::raw::c_double) -> ::std::os::raw::c_double {
    crate::__ffi::ctr_qt_core_ffi_qCos(v)
}

/// <p>Returns the number of consecutive zero bits in <i>v</i>, when searching from the MSB. For example, qCountLeadingZeroBits(quint32(1)) returns 31 and qCountLeadingZeroBits(quint32(8)) returns 28.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qCountLeadingZeroBits(quint32 v)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qCountLeadingZeroBits">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of consecutive zero bits in <i>v</i>, when searching from the MSB. For example, qCountLeadingZeroBits(quint32(1)) returns 31 and qCountLeadingZeroBits(quint32(8)) returns 28.</p>
/// <p>This function was introduced in  Qt 5.6.</p></div>
pub unsafe fn q_count_leading_zero_bits_u32(v: u32) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qCountLeadingZeroBits(v)
}

/// <p>Returns the number of consecutive zero bits in <i>v</i>, when searching from the MSB. For example, qCountLeadingZeroBits(quint32(1)) returns 31 and qCountLeadingZeroBits(quint32(8)) returns 28.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qCountLeadingZeroBits(quint8 v)```</span>.
///
/// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qCountLeadingZeroBits">C++ documentation</a> for <span style='color: green;'>```uint qCountLeadingZeroBits(quint32 v)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of consecutive zero bits in <i>v</i>, when searching from the MSB. For example, qCountLeadingZeroBits(quint32(1)) returns 31 and qCountLeadingZeroBits(quint32(8)) returns 28.</p>
/// <p>This function was introduced in  Qt 5.6.</p></div>
pub unsafe fn q_count_leading_zero_bits_u8(v: u8) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qCountLeadingZeroBits1(v)
}

/// <p>Returns the number of consecutive zero bits in <i>v</i>, when searching from the MSB. For example, qCountLeadingZeroBits(quint32(1)) returns 31 and qCountLeadingZeroBits(quint32(8)) returns 28.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qCountLeadingZeroBits(quint16 v)```</span>.
///
/// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qCountLeadingZeroBits">C++ documentation</a> for <span style='color: green;'>```uint qCountLeadingZeroBits(quint32 v)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of consecutive zero bits in <i>v</i>, when searching from the MSB. For example, qCountLeadingZeroBits(quint32(1)) returns 31 and qCountLeadingZeroBits(quint32(8)) returns 28.</p>
/// <p>This function was introduced in  Qt 5.6.</p></div>
pub unsafe fn q_count_leading_zero_bits_u16(v: u16) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qCountLeadingZeroBits2(v)
}

/// <p>Returns the number of consecutive zero bits in <i>v</i>, when searching from the MSB. For example, qCountLeadingZeroBits(quint32(1)) returns 31 and qCountLeadingZeroBits(quint32(8)) returns 28.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qCountLeadingZeroBits(quint64 v)```</span>.
///
/// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qCountLeadingZeroBits">C++ documentation</a> for <span style='color: green;'>```uint qCountLeadingZeroBits(quint32 v)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of consecutive zero bits in <i>v</i>, when searching from the MSB. For example, qCountLeadingZeroBits(quint32(1)) returns 31 and qCountLeadingZeroBits(quint32(8)) returns 28.</p>
/// <p>This function was introduced in  Qt 5.6.</p></div>
pub unsafe fn q_count_leading_zero_bits_u64(v: u64) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qCountLeadingZeroBits3(v)
}

/// <p>Returns the number of consecutive zero bits in <i>v</i>, when searching from the MSB. For example, qCountLeadingZeroBits(quint32(1)) returns 31 and qCountLeadingZeroBits(quint32(8)) returns 28.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qCountLeadingZeroBits(unsigned long v)```</span>.
///
/// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qCountLeadingZeroBits">C++ documentation</a> for <span style='color: green;'>```uint qCountLeadingZeroBits(quint32 v)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of consecutive zero bits in <i>v</i>, when searching from the MSB. For example, qCountLeadingZeroBits(quint32(1)) returns 31 and qCountLeadingZeroBits(quint32(8)) returns 28.</p>
/// <p>This function was introduced in  Qt 5.6.</p></div>
pub unsafe fn q_count_leading_zero_bits_ulong(
    v: ::std::os::raw::c_ulong,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qCountLeadingZeroBits4(v)
}

/// <p>This is an overloaded function.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qCountTrailingZeroBits(quint32 v)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qCountTrailingZeroBits">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
/// <p>This function was introduced in  Qt 5.6.</p></div>
pub unsafe fn q_count_trailing_zero_bits_u32(v: u32) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qCountTrailingZeroBits(v)
}

/// <p>Returns the number of consecutive zero bits in <i>v</i>, when searching from the LSB. For example, qCountTrailingZeroBits(1) returns 0 and qCountTrailingZeroBits(8) returns 3.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qCountTrailingZeroBits(quint8 v)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qCountTrailingZeroBits">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of consecutive zero bits in <i>v</i>, when searching from the LSB. For example, qCountTrailingZeroBits(1) returns 0 and qCountTrailingZeroBits(8) returns 3.</p>
/// <p>This function was introduced in  Qt 5.6.</p></div>
pub unsafe fn q_count_trailing_zero_bits_u8(v: u8) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qCountTrailingZeroBits1(v)
}

/// <p>This is an overloaded function.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qCountTrailingZeroBits(quint16 v)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qCountTrailingZeroBits-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
/// <p>This function was introduced in  Qt 5.6.</p></div>
pub unsafe fn q_count_trailing_zero_bits_u16(v: u16) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qCountTrailingZeroBits2(v)
}

/// <p>This is an overloaded function.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qCountTrailingZeroBits(quint64 v)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qCountTrailingZeroBits-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
/// <p>This function was introduced in  Qt 5.6.</p></div>
pub unsafe fn q_count_trailing_zero_bits_u64(v: u64) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qCountTrailingZeroBits3(v)
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qCountTrailingZeroBits(unsigned long v)```</span>.
pub unsafe fn q_count_trailing_zero_bits_ulong(
    v: ::std::os::raw::c_ulong,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qCountTrailingZeroBits4(v)
}

/// <p>This function converts the <i>degrees</i> in float to radians.</p>
///
/// Calls C++ function: <span style='color: green;'>```float qDegreesToRadians(float degrees)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtmath.html#qDegreesToRadians">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function converts the <i>degrees</i> in float to radians.</p>
/// <p>Example:</p>
/// <pre class="cpp">
///
///   <span class="type">float</span> degrees <span class="operator">=</span> <span class="number">180.0f</span>
///   <span class="type">float</span> radians <span class="operator">=</span> qDegreesToRadians(degrees)
///
/// </pre>
/// <p>This function was introduced in  Qt 5.1.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtmath.html#qRadiansToDegrees">qRadiansToDegrees</a>().</p></div>
pub unsafe fn q_degrees_to_radians_float(
    degrees: ::std::os::raw::c_float,
) -> ::std::os::raw::c_float {
    crate::__ffi::ctr_qt_core_ffi_qDegreesToRadians(degrees)
}

/// <p>This function converts the <i>degrees</i> in double to radians.</p>
///
/// Calls C++ function: <span style='color: green;'>```double qDegreesToRadians(double degrees)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtmath.html#qDegreesToRadians-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function converts the <i>degrees</i> in double to radians.</p>
/// <p>Example:</p>
/// <pre class="cpp">
///
///   <span class="type">double</span> degrees <span class="operator">=</span> <span class="number">180.0</span>
///   <span class="type">double</span> radians <span class="operator">=</span> <a href="http://doc.qt.io/qt-5/qtmath.html#qDegreesToRadians">qDegreesToRadians</a>(degrees)
///
/// </pre>
/// <p>This function was introduced in  Qt 5.1.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtmath.html#qRadiansToDegrees">qRadiansToDegrees</a>().</p></div>
pub unsafe fn q_degrees_to_radians_double(
    degrees: ::std::os::raw::c_double,
) -> ::std::os::raw::c_double {
    crate::__ffi::ctr_qt_core_ffi_qDegreesToRadians1(degrees)
}

/// Calls C++ function: <span style='color: green;'>```QString qEnvironmentVariable(const char* varName)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
pub unsafe fn q_environment_variable_1a(
    var_name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
) -> ::cpp_utils::CppBox<crate::QString> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qEnvironmentVariable(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(var_name)
            .as_raw_ptr(),
    );
    ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
}

/// Calls C++ function: <span style='color: green;'>```QString qEnvironmentVariable(const char* varName, const QString& defaultValue)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
pub unsafe fn q_environment_variable_2a(
    var_name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    default_value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
) -> ::cpp_utils::CppBox<crate::QString> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qEnvironmentVariable1(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(var_name)
            .as_raw_ptr(),
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(default_value)
            .as_raw_ptr(),
    );
    ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
}

/// <p>Returns the numerical value of the environment variable <i>varName</i>. If <i>ok</i> is not null, sets <code>*ok</code> to <code>true</code> or <code>false</code> depending on the success of the conversion.</p>
///
/// Calls C++ function: <span style='color: green;'>```int qEnvironmentVariableIntValue(const char* varName, bool* ok = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtglobal.html#qEnvironmentVariableIntValue">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the numerical value of the environment variable <i>varName</i>. If <i>ok</i> is not null, sets <code>*ok</code> to <code>true</code> or <code>false</code> depending on the success of the conversion.</p>
/// <p>Equivalent to</p>
/// <pre class="cpp">
///
///   qgetenv(varName)<span class="operator">.</span>toInt(ok<span class="operator">,</span> <span class="number">0</span>)
///
/// </pre>
/// <p>except that it's much faster, and can't throw exceptions.</p>
/// <p><b>Note: </b>there's a limit on the length of the value, which is sufficient for all valid values of int, not counting leading zeroes or spaces. Values that are too long will either be truncated or this function will set <i>ok</i> to <code>false</code>.</p><p>This function was introduced in  Qt 5.5.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtglobal.html#qgetenv">qgetenv</a>() and <a href="http://doc.qt.io/qt-5/qtglobal.html#qEnvironmentVariableIsSet">qEnvironmentVariableIsSet</a>().</p></div>
pub unsafe fn q_environment_variable_int_value_2a(
    var_name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    ok: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<bool>>,
) -> ::std::os::raw::c_int {
    crate::__ffi::ctr_qt_core_ffi_qEnvironmentVariableIntValue(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(var_name)
            .as_raw_ptr(),
        ::cpp_utils::CastInto::<::cpp_utils::MutPtr<bool>>::cast_into(ok).as_mut_raw_ptr(),
    )
}

/// <p>Returns the numerical value of the environment variable <i>varName</i>. If <i>ok</i> is not null, sets <code>*ok</code> to <code>true</code> or <code>false</code> depending on the success of the conversion.</p>
///
/// Calls C++ function: <span style='color: green;'>```int qEnvironmentVariableIntValue(const char* varName)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtglobal.html#qEnvironmentVariableIntValue">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the numerical value of the environment variable <i>varName</i>. If <i>ok</i> is not null, sets <code>*ok</code> to <code>true</code> or <code>false</code> depending on the success of the conversion.</p>
/// <p>Equivalent to</p>
/// <pre class="cpp">
///
///   qgetenv(varName)<span class="operator">.</span>toInt(ok<span class="operator">,</span> <span class="number">0</span>)
///
/// </pre>
/// <p>except that it's much faster, and can't throw exceptions.</p>
/// <p><b>Note: </b>there's a limit on the length of the value, which is sufficient for all valid values of int, not counting leading zeroes or spaces. Values that are too long will either be truncated or this function will set <i>ok</i> to <code>false</code>.</p><p>This function was introduced in  Qt 5.5.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtglobal.html#qgetenv">qgetenv</a>() and <a href="http://doc.qt.io/qt-5/qtglobal.html#qEnvironmentVariableIsSet">qEnvironmentVariableIsSet</a>().</p></div>
pub unsafe fn q_environment_variable_int_value_1a(
    var_name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
) -> ::std::os::raw::c_int {
    crate::__ffi::ctr_qt_core_ffi_qEnvironmentVariableIntValue1(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(var_name)
            .as_raw_ptr(),
    )
}

/// <p>Returns whether the environment variable <i>varName</i> is empty.</p>
///
/// Calls C++ function: <span style='color: green;'>```bool qEnvironmentVariableIsEmpty(const char* varName)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtglobal.html#qEnvironmentVariableIsEmpty">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns whether the environment variable <i>varName</i> is empty.</p>
/// <p>Equivalent to</p>
/// <pre class="cpp">
///
///   qgetenv(varName)<span class="operator">.</span>isEmpty()
///
/// </pre>
/// <p>except that it's potentially much faster, and can't throw exceptions.</p>
/// <p>This function was introduced in  Qt 5.1.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtglobal.html#qgetenv">qgetenv</a>() and <a href="http://doc.qt.io/qt-5/qtglobal.html#qEnvironmentVariableIsSet">qEnvironmentVariableIsSet</a>().</p></div>
pub unsafe fn q_environment_variable_is_empty(
    var_name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
) -> bool {
    crate::__ffi::ctr_qt_core_ffi_qEnvironmentVariableIsEmpty(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(var_name)
            .as_raw_ptr(),
    )
}

/// <p>Returns whether the environment variable <i>varName</i> is set.</p>
///
/// Calls C++ function: <span style='color: green;'>```bool qEnvironmentVariableIsSet(const char* varName)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtglobal.html#qEnvironmentVariableIsSet">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns whether the environment variable <i>varName</i> is set.</p>
/// <p>Equivalent to</p>
/// <pre class="cpp">
///
///   <span class="operator">!</span>qgetenv(varName)<span class="operator">.</span>isNull()
///
/// </pre>
/// <p>except that it's potentially much faster, and can't throw exceptions.</p>
/// <p>This function was introduced in  Qt 5.1.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtglobal.html#qgetenv">qgetenv</a>() and <a href="http://doc.qt.io/qt-5/qtglobal.html#qEnvironmentVariableIsEmpty">qEnvironmentVariableIsEmpty</a>().</p></div>
pub unsafe fn q_environment_variable_is_set(
    var_name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
) -> bool {
    crate::__ffi::ctr_qt_core_ffi_qEnvironmentVariableIsSet(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(var_name)
            .as_raw_ptr(),
    )
}

/// <p>Returns the exponential function of <code>e</code> to the power of <i>v</i>.</p>
///
/// Calls C++ function: <span style='color: green;'>```double qExp(double v)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtmath.html#qExp">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the exponential function of <code>e</code> to the power of <i>v</i>.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtmath.html#qLn">qLn</a>().</p></div>
pub unsafe fn q_exp(v: ::std::os::raw::c_double) -> ::std::os::raw::c_double {
    crate::__ffi::ctr_qt_core_ffi_qExp(v)
}

/// <p>Returns the absolute value of <i>v</i> as a qreal.</p>
///
/// Calls C++ function: <span style='color: green;'>```double qFabs(double v)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtmath.html#qFabs">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the absolute value of <i>v</i> as a qreal.</p></div>
pub unsafe fn q_fabs(v: ::std::os::raw::c_double) -> ::std::os::raw::c_double {
    crate::__ffi::ctr_qt_core_ffi_qFabs(v)
}

/// Calls C++ function: <span style='color: green;'>```double qFastCos(double x)```</span>.
pub unsafe fn q_fast_cos(x: ::std::os::raw::c_double) -> ::std::os::raw::c_double {
    crate::__ffi::ctr_qt_core_ffi_qFastCos(x)
}

/// Calls C++ function: <span style='color: green;'>```double qFastSin(double x)```</span>.
pub unsafe fn q_fast_sin(x: ::std::os::raw::c_double) -> ::std::os::raw::c_double {
    crate::__ffi::ctr_qt_core_ffi_qFastSin(x)
}

/// <p>Returns the number of representable floating-point numbers between <i>a</i> and <i>b</i>.</p>
///
/// Calls C++ function: <span style='color: green;'>```quint32 qFloatDistance(float a, float b)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtglobal.html#qFloatDistance">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of representable floating-point numbers between <i>a</i> and <i>b</i>.</p>
/// <p>This function provides an alternative way of doing approximated comparisons of floating-point numbers similar to <a href="http://doc.qt.io/qt-5/qfloat16.html#qFuzzyCompare">qFuzzyCompare</a>(). However, it returns the distance between two numbers, which gives the caller a possibility to choose the accepted error. Errors are relative, so for instance the distance between 1.0E-5 and 1.00001E-5 will give 110, while the distance between 1.0E36 and 1.00001E36 will give 127.</p>
/// <p>This function is useful if a floating point comparison requires a certain precision. Therefore, if <i>a</i> and <i>b</i> are equal it will return 0. The maximum value it will return for 32-bit floating point numbers is 4,278,190,078. This is the distance between <code>-FLT_MAX</code> and <code>+FLT_MAX</code>.</p>
/// <p>The function does not give meaningful results if any of the arguments are <code>Infinite</code> or <code>NaN</code>. You can check for this by calling <a href="http://doc.qt.io/qt-5/qfloat16.html#qIsFinite">qIsFinite</a>().</p>
/// <p>The return value can be considered as the "error", so if you for instance want to compare two 32-bit floating point numbers and all you need is an approximated 24-bit precision, you can use this function like this:</p>
/// <pre class="cpp">
///
///   <span class="keyword">if</span> (qFloatDistance(a<span class="operator">,</span> b) <span class="operator">&lt;</span> (<span class="number">1</span> <span class="operator">&lt;</span><span class="operator">&lt;</span> <span class="number">7</span>)) {   <span class="comment">// The last 7 bits are not</span>
/// &#32;   &#32;   &#32;   &#32;   &#32;   &#32;   &#32;   &#32;   &#32;   &#32;     <span class="comment">// significant</span>
/// &#32;     <span class="comment">// precise enough</span>
///   }
///
/// </pre>
/// <p>This function was introduced in  Qt 5.2.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfloat16.html#qFuzzyCompare">qFuzzyCompare</a>().</p></div>
pub unsafe fn q_float_distance_2_float(
    a: ::std::os::raw::c_float,
    b: ::std::os::raw::c_float,
) -> u32 {
    crate::__ffi::ctr_qt_core_ffi_qFloatDistance(a, b)
}

/// <p>Returns the number of representable floating-point numbers between <i>a</i> and <i>b</i>.</p>
///
/// Calls C++ function: <span style='color: green;'>```quint64 qFloatDistance(double a, double b)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtglobal.html#qFloatDistance-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of representable floating-point numbers between <i>a</i> and <i>b</i>.</p>
/// <p>This function serves the same purpose as <code>qFloatDistance(float, float)</code>, but returns the distance between two <code>double</code> numbers. Since the range is larger than for two <code>float</code> numbers (<code>[-DBL_MAX,DBL_MAX]</code>), the return type is quint64.</p>
/// <p>This function was introduced in  Qt 5.2.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qfloat16.html#qFuzzyCompare">qFuzzyCompare</a>().</p></div>
pub unsafe fn q_float_distance_2_double(
    a: ::std::os::raw::c_double,
    b: ::std::os::raw::c_double,
) -> u64 {
    crate::__ffi::ctr_qt_core_ffi_qFloatDistance1(a, b)
}

/// <p>Converts <i>len</i> qfloat16 from <i>in</i> to floats and stores them in <i>out</i>. Both <i>in</i> and <i>out</i> must have <i>len</i> allocated entries.</p>
///
/// Calls C++ function: <span style='color: green;'>```void qFloatFromFloat16(float* arg1, const qfloat16* arg2, long long length)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qfloat16.html#qFloatFromFloat16">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts <i>len</i> qfloat16 from <i>in</i> to floats and stores them in <i>out</i>. Both <i>in</i> and <i>out</i> must have <i>len</i> allocated entries.</p>
/// <p>This function was introduced in  Qt 5.11.</p></div>
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_float_from_float16(
    arg1: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_float>>,
    arg2: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::Qfloat16>>,
    length: ::std::os::raw::c_longlong,
) {
    crate::__ffi::ctr_qt_core_ffi_qFloatFromFloat16(
        ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_float>>::cast_into(arg1)
            .as_mut_raw_ptr(),
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::Qfloat16>>::cast_into(arg2).as_raw_ptr(),
        length,
    )
}

/// <p>Converts <i>len</i> floats from <i>in</i> to qfloat16 and stores them in <i>out</i>. Both <i>in</i> and <i>out</i> must have <i>len</i> allocated entries.</p>
///
/// Calls C++ function: <span style='color: green;'>```void qFloatToFloat16(qfloat16* arg1, const float* arg2, long long length)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qfloat16.html#qFloatToFloat16">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Converts <i>len</i> floats from <i>in</i> to qfloat16 and stores them in <i>out</i>. Both <i>in</i> and <i>out</i> must have <i>len</i> allocated entries.</p>
/// <p>This function was introduced in  Qt 5.11.</p></div>
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_float_to_float16(
    arg1: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<crate::Qfloat16>>,
    arg2: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_float>>,
    length: ::std::os::raw::c_longlong,
) {
    crate::__ffi::ctr_qt_core_ffi_qFloatToFloat16(
        ::cpp_utils::CastInto::<::cpp_utils::MutPtr<crate::Qfloat16>>::cast_into(arg1)
            .as_mut_raw_ptr(),
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_float>>::cast_into(arg2)
            .as_raw_ptr(),
        length,
    )
}

/// <p>Return the floor of the value <i>v</i>.</p>
///
/// Calls C++ function: <span style='color: green;'>```int qFloor(double v)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtmath.html#qFloor">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Return the floor of the value <i>v</i>.</p>
/// <p>The floor is the largest integer that is not greater than <i>v</i>. For example, if <i>v</i> is 41.2, then the floor is 41.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtmath.html#qCeil">qCeil</a>().</p></div>
pub unsafe fn q_floor(v: ::std::os::raw::c_double) -> ::std::os::raw::c_int {
    crate::__ffi::ctr_qt_core_ffi_qFloor(v)
}

/// <p>Generates a formatted string out of the <i>type</i>, <i>context</i>, <i>str</i> arguments.</p>
///
/// Calls C++ function: <span style='color: green;'>```QString qFormatLogMessage(QtMsgType type, const QMessageLogContext& context, const QString& buf)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtglobal.html#qFormatLogMessage">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Generates a formatted string out of the <i>type</i>, <i>context</i>, <i>str</i> arguments.</p>
/// <p>qFormatLogMessage returns a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> that is formatted according to the current message pattern. It can be used by custom message handlers to format output similar to Qt's default message handler.</p>
/// <p>The function is thread-safe.</p>
/// <p>This function was introduced in  Qt 5.4.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtglobal.html#qInstallMessageHandler">qInstallMessageHandler</a>() and <a href="http://doc.qt.io/qt-5/qtglobal.html#qSetMessagePattern">qSetMessagePattern</a>().</p></div>
pub unsafe fn q_format_log_message(
    type_: crate::QtMsgType,
    context: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMessageLogContext>>,
    buf: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
) -> ::cpp_utils::CppBox<crate::QString> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qFormatLogMessage(
        type_,
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMessageLogContext>>::cast_into(context)
            .as_raw_ptr(),
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(buf).as_raw_ptr(),
    );
    ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
}

/// Calls C++ function: <span style='color: green;'>```void qFreeAligned(void* ptr)```</span>.
pub unsafe fn q_free_aligned(
    ptr: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
) {
    crate::__ffi::ctr_qt_core_ffi_qFreeAligned(
        ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(ptr)
            .as_mut_raw_ptr(),
    )
}

/// <p>Compares the floating point value <i>p1</i> and <i>p2</i> and returns <code>true</code> if they are considered equal, otherwise <code>false</code>.</p>
///
/// Calls C++ function: <span style='color: green;'>```bool qFuzzyCompare(double p1, double p2)```</span>.
///
/// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qfloat16.html#qFuzzyCompare">C++ documentation</a> for <span style='color: green;'>```bool qFuzzyCompare(qfloat16 p1, qfloat16 p2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares the floating point value <i>p1</i> and <i>p2</i> and returns <code>true</code> if they are considered equal, otherwise <code>false</code>.</p>
/// <p>The two numbers are compared in a relative way, where the exactness is stronger the smaller the numbers are.</p></div>
pub unsafe fn q_fuzzy_compare_2_double(
    p1: ::std::os::raw::c_double,
    p2: ::std::os::raw::c_double,
) -> bool {
    crate::__ffi::ctr_qt_core_ffi_qFuzzyCompare(p1, p2)
}

/// <p>Compares the floating point value <i>p1</i> and <i>p2</i> and returns <code>true</code> if they are considered equal, otherwise <code>false</code>.</p>
///
/// Calls C++ function: <span style='color: green;'>```bool qFuzzyCompare(float p1, float p2)```</span>.
///
/// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qfloat16.html#qFuzzyCompare">C++ documentation</a> for <span style='color: green;'>```bool qFuzzyCompare(qfloat16 p1, qfloat16 p2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares the floating point value <i>p1</i> and <i>p2</i> and returns <code>true</code> if they are considered equal, otherwise <code>false</code>.</p>
/// <p>The two numbers are compared in a relative way, where the exactness is stronger the smaller the numbers are.</p></div>
pub unsafe fn q_fuzzy_compare_2_float(
    p1: ::std::os::raw::c_float,
    p2: ::std::os::raw::c_float,
) -> bool {
    crate::__ffi::ctr_qt_core_ffi_qFuzzyCompare1(p1, p2)
}

/// <p>Compares the floating point value <i>p1</i> and <i>p2</i> and returns <code>true</code> if they are considered equal, otherwise <code>false</code>.</p>
///
/// Calls C++ function: <span style='color: green;'>```bool qFuzzyCompare(qfloat16 p1, qfloat16 p2)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qfloat16.html#qFuzzyCompare">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Compares the floating point value <i>p1</i> and <i>p2</i> and returns <code>true</code> if they are considered equal, otherwise <code>false</code>.</p>
/// <p>The two numbers are compared in a relative way, where the exactness is stronger the smaller the numbers are.</p></div>
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_fuzzy_compare_2_qfloat16(
    p1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::Qfloat16>>,
    p2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::Qfloat16>>,
) -> bool {
    crate::__ffi::ctr_qt_core_ffi_qFuzzyCompare4(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::Qfloat16>>::cast_into(p1).as_raw_ptr(),
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::Qfloat16>>::cast_into(p2).as_raw_ptr(),
    )
}

/// <p>Returns true if the absolute value of <i>d</i> is within 0.000000000001 of 0.0.</p>
///
/// Calls C++ function: <span style='color: green;'>```bool qFuzzyIsNull(double d)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtglobal.html#qFuzzyIsNull">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the absolute value of <i>d</i> is within 0.000000000001 of 0.0.</p>
/// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p>
/// <p>This function was introduced in  Qt 4.4.</p></div>
pub unsafe fn q_fuzzy_is_null_double(d: ::std::os::raw::c_double) -> bool {
    crate::__ffi::ctr_qt_core_ffi_qFuzzyIsNull(d)
}

/// <p>Returns true if the absolute value of <i>f</i> is within 0.00001f of 0.0.</p>
///
/// Calls C++ function: <span style='color: green;'>```bool qFuzzyIsNull(float f)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtglobal.html#qFuzzyIsNull-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the absolute value of <i>f</i> is within 0.00001f of 0.0.</p>
/// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">thread-safe</a>.</p>
/// <p>This function was introduced in  Qt 4.4.</p></div>
pub unsafe fn q_fuzzy_is_null_float(f: ::std::os::raw::c_float) -> bool {
    crate::__ffi::ctr_qt_core_ffi_qFuzzyIsNull1(f)
}

/// Calls C++ function: <span style='color: green;'>```bool qFuzzyIsNull(qfloat16 f)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_fuzzy_is_null_qfloat16(
    f: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::Qfloat16>>,
) -> bool {
    crate::__ffi::ctr_qt_core_ffi_qFuzzyIsNull4(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::Qfloat16>>::cast_into(f).as_raw_ptr(),
    )
}

/// <p>Returns the current global <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> seed.</p>
///
/// Calls C++ function: <span style='color: green;'>```int qGlobalQHashSeed()```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qGlobalQHashSeed">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the current global <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> seed.</p>
/// <p>The seed is set in any newly created <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>. See <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a> about how this seed is being used by <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>.</p>
/// <p>This function was introduced in  Qt 5.6.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#qSetGlobalQHashSeed">qSetGlobalQHashSeed</a>.</p></div>
pub unsafe fn q_global_q_hash_seed() -> ::std::os::raw::c_int {
    crate::__ffi::ctr_qt_core_ffi_qGlobalQHashSeed()
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(char key, unsigned int seed = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_char_uint(
    key: ::std::os::raw::c_char,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash(key, seed)
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(unsigned char key, unsigned int seed = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_uchar_uint(
    key: ::std::os::raw::c_uchar,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash1(key, seed)
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(signed char key, unsigned int seed = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_schar_uint(
    key: ::std::os::raw::c_schar,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash2(key, seed)
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(unsigned short key, unsigned int seed = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-9">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_ushort_uint(
    key: ::std::os::raw::c_ushort,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash3(key, seed)
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(short key, unsigned int seed = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-10">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_short_uint(
    key: ::std::os::raw::c_short,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash4(key, seed)
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(unsigned int key, unsigned int seed = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-11">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_2_uint(
    key: ::std::os::raw::c_uint,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash5(key, seed)
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(int key, unsigned int seed = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-12">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_int_uint(
    key: ::std::os::raw::c_int,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash6(key, seed)
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(unsigned long key, unsigned int seed = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-13">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_ulong_uint(
    key: ::std::os::raw::c_ulong,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash7(key, seed)
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(long key, unsigned int seed = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-14">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_long_uint(
    key: ::std::os::raw::c_long,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash8(key, seed)
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(quint64 key, unsigned int seed = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-15">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_u64_uint(key: u64, seed: ::std::os::raw::c_uint) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash9(key, seed)
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(qint64 key, unsigned int seed = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-16">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_i64_uint(key: i64, seed: ::std::os::raw::c_uint) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash10(key, seed)
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(float key, unsigned int seed = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-17">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.3.</p></div>
pub unsafe fn q_hash_float_uint(
    key: ::std::os::raw::c_float,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash11(key, seed)
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(double key, unsigned int seed = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-18">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.3.</p></div>
pub unsafe fn q_hash_double_uint(
    key: ::std::os::raw::c_double,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash12(key, seed)
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(QChar key, unsigned int seed = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-19">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_q_char_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash14(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(key).as_raw_ptr(),
        seed,
    )
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QByteArray& key, unsigned int seed = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-20">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_q_byte_array_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash15(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(key).as_raw_ptr(),
        seed,
    )
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QString& key, unsigned int seed = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-22">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_q_string_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash16(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
        seed,
    )
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QStringRef& key, unsigned int seed = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-23">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_q_string_ref_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash17(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(key).as_raw_ptr(),
        seed,
    )
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QBitArray& key, unsigned int seed = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-21">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_q_bit_array_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QBitArray>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash18(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QBitArray>>::cast_into(key).as_raw_ptr(),
        seed,
    )
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(QLatin1String key, unsigned int seed = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-24">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_q_latin1_string_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash19(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(key)
            .as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QRegExp& key, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_reg_exp_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash20(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(key).as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QPersistentModelIndex& index, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_persistent_model_index_uint(
    index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPersistentModelIndex>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash21(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPersistentModelIndex>>::cast_into(index)
            .as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QModelIndex& index)```</span>.
pub unsafe fn q_hash_q_model_index(
    index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QModelIndex>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash22(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QModelIndex>>::cast_into(index)
            .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QLocale& key, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_locale_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash23(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(key).as_raw_ptr(),
        seed,
    )
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QDateTime& key, unsigned int seed = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-1x">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_q_date_time_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash24(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(key).as_raw_ptr(),
        seed,
    )
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QDate& key, unsigned int seed = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-2x">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_q_date_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDate>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash25(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDate>>::cast_into(key).as_raw_ptr(),
        seed,
    )
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QTime& key, unsigned int seed = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-3x">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_q_time_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTime>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash26(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTime>>::cast_into(key).as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QItemSelectionRange& arg1)```</span>.
pub unsafe fn q_hash_q_item_selection_range(
    arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QItemSelectionRange>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash27(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QItemSelectionRange>>::cast_into(arg1)
            .as_raw_ptr(),
    )
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QVersionNumber& key, unsigned int seed = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-28">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.6.</p></div>
pub unsafe fn q_hash_q_version_number_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVersionNumber>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash28(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVersionNumber>>::cast_into(key)
            .as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QMimeType& key, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_mime_type_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMimeType>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash29(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMimeType>>::cast_into(key).as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QRegularExpression& key, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_regular_expression_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash30(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(key)
            .as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QUrl& url, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_url_uint(
    url: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash31(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(url).as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QUrlQuery& key, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_url_query_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrlQuery>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash32(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrlQuery>>::cast_into(key).as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QUuid& uuid, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_uuid_uint(
    uuid: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUuid>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash33(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUuid>>::cast_into(uuid).as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QPair<QString, QJsonValue>& key, unsigned int seed = …)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_pair_of_q_string_q_json_value_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQStringQJsonValue>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash34(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQStringQJsonValue>>::cast_into(key)
            .as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QPair<QString, QString>& key, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_pair_of_q_string_q_string_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQStringQString>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash36(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQStringQString>>::cast_into(key)
            .as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QString>& key, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_list_of_q_string_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQString>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash37(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQString>>::cast_into(key)
            .as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QItemSelectionRange>& key, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_list_of_q_item_selection_range_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQItemSelectionRange>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash38(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQItemSelectionRange>>::cast_into(
            key,
        )
        .as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QObject*>& key, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_list_of_q_object_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQObject>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash39(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQObject>>::cast_into(key)
            .as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QByteArray>& key, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_list_of_q_byte_array_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQByteArray>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash41(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQByteArray>>::cast_into(key)
            .as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QModelIndex>& key, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_list_of_q_model_index_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQModelIndex>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash43(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQModelIndex>>::cast_into(key)
            .as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QPersistentModelIndex>& key, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_list_of_q_persistent_model_index_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash44(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>>::cast_into(
            key,
        )
        .as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QAbstractState*>& key, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_list_of_q_abstract_state_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQAbstractState>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash45(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQAbstractState>>::cast_into(key)
            .as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QAbstractAnimation*>& key, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_list_of_q_abstract_animation_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQAbstractAnimation>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash46(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQAbstractAnimation>>::cast_into(key)
            .as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<Qt::DayOfWeek>& key, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_list_of_day_of_week_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfDayOfWeek>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash47(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfDayOfWeek>>::cast_into(key)
            .as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QLocale>& key, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_list_of_q_locale_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQLocale>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash48(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQLocale>>::cast_into(key)
            .as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QLocale::Country>& key, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_list_of_country_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfCountry>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash49(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfCountry>>::cast_into(key)
            .as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QUrl>& key, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_list_of_q_url_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQUrl>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash52(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQUrl>>::cast_into(key).as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QMimeType>& key, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_list_of_q_mime_type_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQMimeType>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash53(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQMimeType>>::cast_into(key)
            .as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QAbstractTransition*>& key, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_list_of_q_abstract_transition_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQAbstractTransition>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash54(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQAbstractTransition>>::cast_into(
            key,
        )
        .as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<int>& key, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_list_of_int_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfInt>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash56(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfInt>>::cast_into(key).as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QPair<QString, QString>>& key, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_list_of_q_pair_of_q_string_q_string_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash57(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>>::cast_into(
            key,
        )
        .as_raw_ptr(),
        seed,
    )
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QHash<int, QByteArray>& key, unsigned int seed = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-26">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>Type <code>T</code> must be supported by <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>().</p>
/// <p>This function was introduced in  Qt 5.8.</p></div>
pub unsafe fn q_hash_q_hash_of_int_q_byte_array_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QHashOfIntQByteArray>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash59(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QHashOfIntQByteArray>>::cast_into(key)
            .as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QVector<QStringRef>& key, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_vector_of_q_string_ref_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQStringRef>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash61(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQStringRef>>::cast_into(key)
            .as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QVector<unsigned int>& key, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_vector_of_uint_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfUint>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash62(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfUint>>::cast_into(key)
            .as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QVector<int>& key, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_vector_of_int_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfInt>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash63(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfInt>>::cast_into(key).as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QSet<QAbstractState*>& key, unsigned int seed = …)```</span>.
pub unsafe fn q_hash_q_set_of_q_abstract_state_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSetOfQAbstractState>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash72(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSetOfQAbstractState>>::cast_into(key)
            .as_raw_ptr(),
        seed,
    )
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(char key)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-6">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_char(key: ::std::os::raw::c_char) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash73(key)
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(unsigned char key)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-7">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_uchar(key: ::std::os::raw::c_uchar) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash74(key)
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(signed char key)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-8">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_schar(key: ::std::os::raw::c_schar) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash75(key)
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(unsigned short key)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-9">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_ushort(key: ::std::os::raw::c_ushort) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash76(key)
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(short key)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-10">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_short(key: ::std::os::raw::c_short) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash77(key)
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(unsigned int key)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-11">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_uint(key: ::std::os::raw::c_uint) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash78(key)
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(int key)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-12">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_int(key: ::std::os::raw::c_int) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash79(key)
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(unsigned long key)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-13">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_ulong(key: ::std::os::raw::c_ulong) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash80(key)
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(long key)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-14">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_long(key: ::std::os::raw::c_long) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash81(key)
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(quint64 key)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-15">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_u64(key: u64) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash82(key)
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(qint64 key)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-16">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_i64(key: i64) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash83(key)
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(float key)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-17">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.3.</p></div>
pub unsafe fn q_hash_float(key: ::std::os::raw::c_float) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash84(key)
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(double key)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-18">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.3.</p></div>
pub unsafe fn q_hash_double(key: ::std::os::raw::c_double) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash85(key)
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(QChar key)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-19">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_q_char(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QChar>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash87(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QChar>>::cast_into(key).as_raw_ptr(),
    )
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QByteArray& key)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-20">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_q_byte_array(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash88(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(key).as_raw_ptr(),
    )
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QString& key)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-22">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_q_string(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash89(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(key).as_raw_ptr(),
    )
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QStringRef& key)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-23">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_q_string_ref(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringRef>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash90(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringRef>>::cast_into(key).as_raw_ptr(),
    )
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QBitArray& key)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-21">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_q_bit_array(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QBitArray>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash91(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QBitArray>>::cast_into(key).as_raw_ptr(),
    )
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(QLatin1String key)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-24">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_q_latin1_string(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLatin1String>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash92(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLatin1String>>::cast_into(key)
            .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QRegExp& key)```</span>.
pub unsafe fn q_hash_q_reg_exp(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegExp>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash93(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegExp>>::cast_into(key).as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QPersistentModelIndex& index)```</span>.
pub unsafe fn q_hash_q_persistent_model_index(
    index: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPersistentModelIndex>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash94(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPersistentModelIndex>>::cast_into(index)
            .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QLocale& key)```</span>.
pub unsafe fn q_hash_q_locale(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QLocale>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash95(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QLocale>>::cast_into(key).as_raw_ptr(),
    )
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QDateTime& key)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-1x">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_q_date_time(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDateTime>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash96(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDateTime>>::cast_into(key).as_raw_ptr(),
    )
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QDate& key)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-2x">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_q_date(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QDate>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash97(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QDate>>::cast_into(key).as_raw_ptr(),
    )
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QTime& key)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-3x">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_hash_q_time(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QTime>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash98(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QTime>>::cast_into(key).as_raw_ptr(),
    )
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QVersionNumber& key)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-28">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>This function was introduced in  Qt 5.6.</p></div>
pub unsafe fn q_hash_q_version_number(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVersionNumber>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash99(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVersionNumber>>::cast_into(key)
            .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QMimeType& key)```</span>.
pub unsafe fn q_hash_q_mime_type(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMimeType>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash100(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMimeType>>::cast_into(key).as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QRegularExpression& key)```</span>.
pub unsafe fn q_hash_q_regular_expression(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QRegularExpression>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash101(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QRegularExpression>>::cast_into(key)
            .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QUrl& url)```</span>.
pub unsafe fn q_hash_q_url(
    url: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrl>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash102(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrl>>::cast_into(url).as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QUrlQuery& key)```</span>.
pub unsafe fn q_hash_q_url_query(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUrlQuery>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash103(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUrlQuery>>::cast_into(key).as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QUuid& uuid)```</span>.
pub unsafe fn q_hash_q_uuid(
    uuid: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QUuid>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash104(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QUuid>>::cast_into(uuid).as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QPair<QString, QJsonValue>& key)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_pair_of_q_string_q_json_value(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQStringQJsonValue>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash105(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQStringQJsonValue>>::cast_into(key)
            .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QPair<QString, QString>& key)```</span>.
pub unsafe fn q_hash_q_pair_of_q_string_q_string(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQStringQString>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash107(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQStringQString>>::cast_into(key)
            .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QString>& key)```</span>.
pub unsafe fn q_hash_q_list_of_q_string(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQString>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash108(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQString>>::cast_into(key)
            .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QItemSelectionRange>& key)```</span>.
pub unsafe fn q_hash_q_list_of_q_item_selection_range(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQItemSelectionRange>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash109(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQItemSelectionRange>>::cast_into(
            key,
        )
        .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QObject*>& key)```</span>.
pub unsafe fn q_hash_q_list_of_q_object(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQObject>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash110(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQObject>>::cast_into(key)
            .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QByteArray>& key)```</span>.
pub unsafe fn q_hash_q_list_of_q_byte_array(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQByteArray>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash112(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQByteArray>>::cast_into(key)
            .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QModelIndex>& key)```</span>.
pub unsafe fn q_hash_q_list_of_q_model_index(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQModelIndex>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash114(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQModelIndex>>::cast_into(key)
            .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QPersistentModelIndex>& key)```</span>.
pub unsafe fn q_hash_q_list_of_q_persistent_model_index(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash115(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQPersistentModelIndex>>::cast_into(
            key,
        )
        .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QAbstractState*>& key)```</span>.
pub unsafe fn q_hash_q_list_of_q_abstract_state(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQAbstractState>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash116(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQAbstractState>>::cast_into(key)
            .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QAbstractAnimation*>& key)```</span>.
pub unsafe fn q_hash_q_list_of_q_abstract_animation(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQAbstractAnimation>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash117(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQAbstractAnimation>>::cast_into(key)
            .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<Qt::DayOfWeek>& key)```</span>.
pub unsafe fn q_hash_q_list_of_day_of_week(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfDayOfWeek>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash118(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfDayOfWeek>>::cast_into(key)
            .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QLocale>& key)```</span>.
pub unsafe fn q_hash_q_list_of_q_locale(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQLocale>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash119(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQLocale>>::cast_into(key)
            .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QLocale::Country>& key)```</span>.
pub unsafe fn q_hash_q_list_of_country(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfCountry>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash120(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfCountry>>::cast_into(key)
            .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QUrl>& key)```</span>.
pub unsafe fn q_hash_q_list_of_q_url(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQUrl>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash123(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQUrl>>::cast_into(key).as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QMimeType>& key)```</span>.
pub unsafe fn q_hash_q_list_of_q_mime_type(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQMimeType>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash124(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQMimeType>>::cast_into(key)
            .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QAbstractTransition*>& key)```</span>.
pub unsafe fn q_hash_q_list_of_q_abstract_transition(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQAbstractTransition>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash125(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQAbstractTransition>>::cast_into(
            key,
        )
        .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<int>& key)```</span>.
pub unsafe fn q_hash_q_list_of_int(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfInt>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash127(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfInt>>::cast_into(key).as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QList<QPair<QString, QString>>& key)```</span>.
pub unsafe fn q_hash_q_list_of_q_pair_of_q_string_q_string(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash128(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QListOfQPairOfQStringQString>>::cast_into(
            key,
        )
        .as_raw_ptr(),
    )
}

/// <p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QHash<int, QByteArray>& key)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHash-26">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the <i>key</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>Type <code>T</code> must be supported by <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>().</p>
/// <p>This function was introduced in  Qt 5.8.</p></div>
pub unsafe fn q_hash_q_hash_of_int_q_byte_array(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QHashOfIntQByteArray>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash130(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QHashOfIntQByteArray>>::cast_into(key)
            .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QVector<QStringRef>& key)```</span>.
pub unsafe fn q_hash_q_vector_of_q_string_ref(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQStringRef>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash132(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQStringRef>>::cast_into(key)
            .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QVector<unsigned int>& key)```</span>.
pub unsafe fn q_hash_q_vector_of_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfUint>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash133(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfUint>>::cast_into(key)
            .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QVector<int>& key)```</span>.
pub unsafe fn q_hash_q_vector_of_int(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfInt>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash134(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfInt>>::cast_into(key).as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QSet<QAbstractState*>& key)```</span>.
pub unsafe fn q_hash_q_set_of_q_abstract_state(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QSetOfQAbstractState>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash143(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QSetOfQAbstractState>>::cast_into(key)
            .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(QStringView key, unsigned int seed = …)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_string_view_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash288(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(key).as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(QStringView key)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.11.3", cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(
        cpp_lib_version = "5.11.3",
        cpp_lib_version = "5.12.2",
        cpp_lib_version = "5.13.0"
    ),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_string_view(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStringView>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash289(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStringView>>::cast_into(key).as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(QCborSimpleType tag, unsigned int seed = …)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_cbor_simple_type_uint(
    tag: crate::QCborSimpleType,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash292(tag, seed)
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(QCborTag tag, unsigned int seed = …)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_cbor_tag_uint(
    tag: crate::QCborTag,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash293(tag, seed)
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QCborValue& value, unsigned int seed = …)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_cbor_value_uint(
    value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash294(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(value).as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QCborArray& array, unsigned int seed = …)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_cbor_array_uint(
    array: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborArray>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash295(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborArray>>::cast_into(array).as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QCborMap& map, unsigned int seed = …)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_cbor_map_uint(
    map: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborMap>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash296(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborMap>>::cast_into(map).as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QJsonValue& value, unsigned int seed = …)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_json_value_uint(
    value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonValue>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash297(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonValue>>::cast_into(value).as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QJsonArray& array, unsigned int seed = …)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_json_array_uint(
    array: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonArray>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash298(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonArray>>::cast_into(array).as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QJsonObject& object, unsigned int seed = …)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_json_object_uint(
    object: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonObject>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash299(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonObject>>::cast_into(object)
            .as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QPair<QCborValueRef, QCborValueRef>& key, unsigned int seed = …)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_pair_of_q_cbor_value_ref_q_cbor_value_ref_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQCborValueRefQCborValueRef>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash300(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQCborValueRefQCborValueRef>>::cast_into(key).as_raw_ptr(), seed)
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QPair<QCborValue, QCborValue>& key, unsigned int seed = …)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_pair_of_q_cbor_value_q_cbor_value_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQCborValueQCborValue>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash301(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQCborValueQCborValue>>::cast_into(
            key,
        )
        .as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QPair<qint64, unsigned int>& key, unsigned int seed = …)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_pair_of_i64_uint_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfI64Uint>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash302(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfI64Uint>>::cast_into(key)
            .as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QVector<QCborValue>& key, unsigned int seed = …)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_vector_of_q_cbor_value_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQCborValue>>,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash303(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQCborValue>>::cast_into(key)
            .as_raw_ptr(),
        seed,
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QPair<QCborValueRef, QCborValueRef>& key)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_pair_of_q_cbor_value_ref_q_cbor_value_ref(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQCborValueRefQCborValueRef>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash304(::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQCborValueRefQCborValueRef>>::cast_into(key).as_raw_ptr())
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QPair<QCborValue, QCborValue>& key)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_pair_of_q_cbor_value_q_cbor_value(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfQCborValueQCborValue>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash305(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfQCborValueQCborValue>>::cast_into(
            key,
        )
        .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QPair<qint64, unsigned int>& key)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_pair_of_i64_uint(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QPairOfI64Uint>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash306(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QPairOfI64Uint>>::cast_into(key)
            .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QVector<QCborValue>& key)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_vector_of_q_cbor_value(
    key: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVectorOfQCborValue>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash307(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVectorOfQCborValue>>::cast_into(key)
            .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(QCborSimpleType tag)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_cbor_simple_type(tag: crate::QCborSimpleType) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash308(tag)
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(QCborTag tag)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_cbor_tag(tag: crate::QCborTag) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash309(tag)
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QCborValue& value)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_cbor_value(
    value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash310(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(value).as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QCborArray& array)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_cbor_array(
    array: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborArray>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash311(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborArray>>::cast_into(array).as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QCborMap& map)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_cbor_map(
    map: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborMap>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash312(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborMap>>::cast_into(map).as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QJsonValue& value)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_json_value(
    value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonValue>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash313(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonValue>>::cast_into(value).as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QJsonArray& array)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_json_array(
    array: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonArray>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash314(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonArray>>::cast_into(array).as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qHash(const QJsonObject& object)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_hash_q_json_object(
    object: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonObject>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHash315(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonObject>>::cast_into(object)
            .as_raw_ptr(),
    )
}

/// <p>Returns the hash value for the memory block of size <i>len</i> pointed to by <i>p</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHashBits(const void* p, unsigned long size, unsigned int seed = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHashBits">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the memory block of size <i>len</i> pointed to by <i>p</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>Use this function only to implement <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() for your own custom types. For example, here's how you could implement a <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() overload for std::vector&lt;int&gt;:</p>
/// <pre class="cpp">
///
///   <span class="keyword">inline</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(<span class="keyword">const</span> std<span class="operator">::</span>vector<span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> <span class="operator">&amp;</span>key<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> seed <span class="operator">=</span> <span class="number">0</span>)
///   {
/// &#32;     <span class="keyword">if</span> (key<span class="operator">.</span>empty())
/// &#32;   &#32;     <span class="keyword">return</span> seed;
/// &#32;     <span class="keyword">else</span>
/// &#32;   &#32;     <span class="keyword">return</span> qHashBits(<span class="operator">&amp;</span>key<span class="operator">.</span>front()<span class="operator">,</span> key<span class="operator">.</span>size() <span class="operator">*</span> <span class="keyword">sizeof</span>(<span class="type">int</span>)<span class="operator">,</span> seed);
///   }
///
/// </pre>
/// <p>This takes advantage of the fact that std::vector lays out its data contiguously. If that is not the case, or the contained type has padding, you should use <a href="http://doc.qt.io/qt-5/qhash.html#qHashRange">qHashRange</a>() instead.</p>
/// <p>It bears repeating that the implementation of qHashBits() - like the <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() overloads offered by Qt - may change at any time. You <b>must not</b> rely on the fact that qHashBits() will give the same results (for the same inputs) across different Qt versions.</p>
/// <p>This function was introduced in  Qt 5.4.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#qHashRange">qHashRange</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#qHashRangeCommutative">qHashRangeCommutative</a>().</p></div>
pub unsafe fn q_hash_bits_3a(
    p: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::ffi::c_void>>,
    size: ::std::os::raw::c_ulong,
    seed: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHashBits(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::ffi::c_void>>::cast_into(p).as_raw_ptr(),
        size,
        seed,
    )
}

/// <p>Returns the hash value for the memory block of size <i>len</i> pointed to by <i>p</i>, using <i>seed</i> to seed the calculation.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qHashBits(const void* p, unsigned long size)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qHashBits">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the hash value for the memory block of size <i>len</i> pointed to by <i>p</i>, using <i>seed</i> to seed the calculation.</p>
/// <p>Use this function only to implement <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() for your own custom types. For example, here's how you could implement a <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() overload for std::vector&lt;int&gt;:</p>
/// <pre class="cpp">
///
///   <span class="keyword">inline</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>(<span class="keyword">const</span> std<span class="operator">::</span>vector<span class="operator">&lt;</span><span class="type">int</span><span class="operator">&gt;</span> <span class="operator">&amp;</span>key<span class="operator">,</span> <span class="type"><a href="http://doc.qt.io/qt-5/qtglobal.html#uint-typedef">uint</a></span> seed <span class="operator">=</span> <span class="number">0</span>)
///   {
/// &#32;     <span class="keyword">if</span> (key<span class="operator">.</span>empty())
/// &#32;   &#32;     <span class="keyword">return</span> seed;
/// &#32;     <span class="keyword">else</span>
/// &#32;   &#32;     <span class="keyword">return</span> qHashBits(<span class="operator">&amp;</span>key<span class="operator">.</span>front()<span class="operator">,</span> key<span class="operator">.</span>size() <span class="operator">*</span> <span class="keyword">sizeof</span>(<span class="type">int</span>)<span class="operator">,</span> seed);
///   }
///
/// </pre>
/// <p>This takes advantage of the fact that std::vector lays out its data contiguously. If that is not the case, or the contained type has padding, you should use <a href="http://doc.qt.io/qt-5/qhash.html#qHashRange">qHashRange</a>() instead.</p>
/// <p>It bears repeating that the implementation of qHashBits() - like the <a href="http://doc.qt.io/qt-5/qhash.html#qHashx">qHash</a>() overloads offered by Qt - may change at any time. You <b>must not</b> rely on the fact that qHashBits() will give the same results (for the same inputs) across different Qt versions.</p>
/// <p>This function was introduced in  Qt 5.4.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#qHashRange">qHashRange</a>() and <a href="http://doc.qt.io/qt-5/qhash.html#qHashRangeCommutative">qHashRangeCommutative</a>().</p></div>
pub unsafe fn q_hash_bits_2a(
    p: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::ffi::c_void>>,
    size: ::std::os::raw::c_ulong,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qHashBits1(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::ffi::c_void>>::cast_into(p).as_raw_ptr(),
        size,
    )
}

/// <p>Returns the bit pattern for an infinite number as a double.</p>
///
/// Calls C++ function: <span style='color: green;'>```double qInf()```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtglobal.html#qInf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the bit pattern for an infinite number as a double.</p></div>
pub unsafe fn q_inf() -> ::std::os::raw::c_double {
    crate::__ffi::ctr_qt_core_ffi_qInf()
}

/// Calls C++ function: <span style='color: green;'>```int qIntCast(double f)```</span>.
pub unsafe fn q_int_cast_double(f: ::std::os::raw::c_double) -> ::std::os::raw::c_int {
    crate::__ffi::ctr_qt_core_ffi_qIntCast(f)
}

/// Calls C++ function: <span style='color: green;'>```int qIntCast(float f)```</span>.
pub unsafe fn q_int_cast_float(f: ::std::os::raw::c_float) -> ::std::os::raw::c_int {
    crate::__ffi::ctr_qt_core_ffi_qIntCast1(f)
}

/// Calls C++ function: <span style='color: green;'>```int qIntCast(qfloat16 f)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_int_cast_qfloat16(
    f: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::Qfloat16>>,
) -> ::std::os::raw::c_int {
    crate::__ffi::ctr_qt_core_ffi_qIntCast4(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::Qfloat16>>::cast_into(f).as_raw_ptr(),
    )
}

/// <p>Returns true if the <code>qfloat16</code> <i>f</i> is a finite number.</p>
///
/// Calls C++ function: <span style='color: green;'>```bool qIsFinite(double d)```</span>.
///
/// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qfloat16.html#qIsFinite">C++ documentation</a> for <span style='color: green;'>```bool qIsFinite(qfloat16 f)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the <code>qfloat16</code> <i>f</i> is a finite number.</p>
/// <p><b>See also </b>qIsFinite.</p></div>
pub unsafe fn q_is_finite_double(d: ::std::os::raw::c_double) -> bool {
    crate::__ffi::ctr_qt_core_ffi_qIsFinite(d)
}

/// <p>Returns true if the <code>qfloat16</code> <i>f</i> is a finite number.</p>
///
/// Calls C++ function: <span style='color: green;'>```bool qIsFinite(float f)```</span>.
///
/// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qfloat16.html#qIsFinite">C++ documentation</a> for <span style='color: green;'>```bool qIsFinite(qfloat16 f)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the <code>qfloat16</code> <i>f</i> is a finite number.</p>
/// <p><b>See also </b>qIsFinite.</p></div>
pub unsafe fn q_is_finite_float(f: ::std::os::raw::c_float) -> bool {
    crate::__ffi::ctr_qt_core_ffi_qIsFinite1(f)
}

/// <p>Returns true if the <code>qfloat16</code> <i>f</i> is a finite number.</p>
///
/// Calls C++ function: <span style='color: green;'>```bool qIsFinite(qfloat16 f)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qfloat16.html#qIsFinite">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the <code>qfloat16</code> <i>f</i> is a finite number.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtglobal.html#qIsFinite">qIsFinite</a>.</p></div>
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_is_finite_qfloat16(
    f: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::Qfloat16>>,
) -> bool {
    crate::__ffi::ctr_qt_core_ffi_qIsFinite4(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::Qfloat16>>::cast_into(f).as_raw_ptr(),
    )
}

/// <p>Returns true if the <code>qfloat16</code> <i>f</i> is equivalent to infinity.</p>
///
/// Calls C++ function: <span style='color: green;'>```bool qIsInf(double d)```</span>.
///
/// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qfloat16.html#qIsInf">C++ documentation</a> for <span style='color: green;'>```bool qIsInf(qfloat16 f)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the <code>qfloat16</code> <i>f</i> is equivalent to infinity.</p>
/// <p><b>See also </b>qIsInf.</p></div>
pub unsafe fn q_is_inf_double(d: ::std::os::raw::c_double) -> bool {
    crate::__ffi::ctr_qt_core_ffi_qIsInf(d)
}

/// <p>Returns true if the <code>qfloat16</code> <i>f</i> is equivalent to infinity.</p>
///
/// Calls C++ function: <span style='color: green;'>```bool qIsInf(float f)```</span>.
///
/// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qfloat16.html#qIsInf">C++ documentation</a> for <span style='color: green;'>```bool qIsInf(qfloat16 f)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the <code>qfloat16</code> <i>f</i> is equivalent to infinity.</p>
/// <p><b>See also </b>qIsInf.</p></div>
pub unsafe fn q_is_inf_float(f: ::std::os::raw::c_float) -> bool {
    crate::__ffi::ctr_qt_core_ffi_qIsInf1(f)
}

/// <p>Returns true if the <code>qfloat16</code> <i>f</i> is equivalent to infinity.</p>
///
/// Calls C++ function: <span style='color: green;'>```bool qIsInf(qfloat16 f)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qfloat16.html#qIsInf">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the <code>qfloat16</code> <i>f</i> is equivalent to infinity.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtglobal.html#qIsInf">qIsInf</a>.</p></div>
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_is_inf_qfloat16(
    f: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::Qfloat16>>,
) -> bool {
    crate::__ffi::ctr_qt_core_ffi_qIsInf4(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::Qfloat16>>::cast_into(f).as_raw_ptr(),
    )
}

/// <p>Returns true if the <code>qfloat16</code> <i>f</i> is not a number (NaN).</p>
///
/// Calls C++ function: <span style='color: green;'>```bool qIsNaN(double d)```</span>.
///
/// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qfloat16.html#qIsNaN">C++ documentation</a> for <span style='color: green;'>```bool qIsNaN(qfloat16 f)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the <code>qfloat16</code> <i>f</i> is not a number (NaN).</p>
/// <p><b>See also </b>qIsNaN.</p></div>
pub unsafe fn q_is_nan_double(d: ::std::os::raw::c_double) -> bool {
    crate::__ffi::ctr_qt_core_ffi_qIsNaN(d)
}

/// <p>Returns true if the <code>qfloat16</code> <i>f</i> is not a number (NaN).</p>
///
/// Calls C++ function: <span style='color: green;'>```bool qIsNaN(float f)```</span>.
///
/// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qfloat16.html#qIsNaN">C++ documentation</a> for <span style='color: green;'>```bool qIsNaN(qfloat16 f)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the <code>qfloat16</code> <i>f</i> is not a number (NaN).</p>
/// <p><b>See also </b>qIsNaN.</p></div>
pub unsafe fn q_is_nan_float(f: ::std::os::raw::c_float) -> bool {
    crate::__ffi::ctr_qt_core_ffi_qIsNaN1(f)
}

/// <p>Returns true if the <code>qfloat16</code> <i>f</i> is not a number (NaN).</p>
///
/// Calls C++ function: <span style='color: green;'>```bool qIsNaN(qfloat16 f)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qfloat16.html#qIsNaN">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns true if the <code>qfloat16</code> <i>f</i> is not a number (NaN).</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtglobal.html#qIsNaN">qIsNaN</a>.</p></div>
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_is_nan_qfloat16(
    f: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::Qfloat16>>,
) -> bool {
    crate::__ffi::ctr_qt_core_ffi_qIsNaN4(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::Qfloat16>>::cast_into(f).as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```bool qIsNull(double d)```</span>.
pub unsafe fn q_is_null_double(d: ::std::os::raw::c_double) -> bool {
    crate::__ffi::ctr_qt_core_ffi_qIsNull(d)
}

/// Calls C++ function: <span style='color: green;'>```bool qIsNull(float f)```</span>.
pub unsafe fn q_is_null_float(f: ::std::os::raw::c_float) -> bool {
    crate::__ffi::ctr_qt_core_ffi_qIsNull1(f)
}

/// Calls C++ function: <span style='color: green;'>```bool qIsNull(qfloat16 f)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_is_null_qfloat16(
    f: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::Qfloat16>>,
) -> bool {
    crate::__ffi::ctr_qt_core_ffi_qIsNull4(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::Qfloat16>>::cast_into(f).as_raw_ptr(),
    )
}

/// <p>Returns the natural logarithm of <i>v</i>. Natural logarithm uses base e.</p>
///
/// Calls C++ function: <span style='color: green;'>```double qLn(double v)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtmath.html#qLn">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the natural logarithm of <i>v</i>. Natural logarithm uses base e.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtmath.html#qExp">qExp</a>().</p></div>
pub unsafe fn q_ln(v: ::std::os::raw::c_double) -> ::std::os::raw::c_double {
    crate::__ffi::ctr_qt_core_ffi_qLn(v)
}

/// <p>Returns a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; that contains <i>value1</i> and <i>value2</i>. Example:</p>
///
/// Calls C++ function: <span style='color: green;'>```QPair<QString, QJsonValue> qMakePair(const QString& x, const QJsonValue& y)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qpair.html#qMakePair">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; that contains <i>value1</i> and <i>value2</i>. Example:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> <span class="operator">&gt;</span> list;
///   list<span class="operator">.</span>append(qMakePair(<span class="number">66</span><span class="operator">,</span> <span class="number">3.14159</span>));
///
/// </pre>
/// <p>This is equivalent to <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt;(<i>value1</i>, <i>value2</i>), but usually requires less typing.</p></div>
pub unsafe fn q_make_pair_q_string_q_json_value(
    x: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    y: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QJsonValue>>,
) -> ::cpp_utils::CppBox<crate::QPairOfQStringQJsonValue> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qMakePair(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(x).as_raw_ptr(),
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QJsonValue>>::cast_into(y).as_raw_ptr(),
    );
    ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
}

/// <p>Returns a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; that contains <i>value1</i> and <i>value2</i>. Example:</p>
///
/// Calls C++ function: <span style='color: green;'>```QPair<double, QVariant> qMakePair(const double& x, const QVariant& y)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qpair.html#qMakePair">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; that contains <i>value1</i> and <i>value2</i>. Example:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> <span class="operator">&gt;</span> list;
///   list<span class="operator">.</span>append(qMakePair(<span class="number">66</span><span class="operator">,</span> <span class="number">3.14159</span>));
///
/// </pre>
/// <p>This is equivalent to <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt;(<i>value1</i>, <i>value2</i>), but usually requires less typing.</p></div>
pub unsafe fn q_make_pair_double_q_variant(
    x: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_double>>,
    y: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
) -> ::cpp_utils::CppBox<crate::QPairOfDoubleQVariant> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qMakePair1(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_double>>::cast_into(x)
            .as_raw_ptr(),
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(y).as_raw_ptr(),
    );
    ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
}

/// <p>Returns a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; that contains <i>value1</i> and <i>value2</i>. Example:</p>
///
/// Calls C++ function: <span style='color: green;'>```QPair<QString, QString> qMakePair(const QString& x, const QString& y)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qpair.html#qMakePair">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; that contains <i>value1</i> and <i>value2</i>. Example:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> <span class="operator">&gt;</span> list;
///   list<span class="operator">.</span>append(qMakePair(<span class="number">66</span><span class="operator">,</span> <span class="number">3.14159</span>));
///
/// </pre>
/// <p>This is equivalent to <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt;(<i>value1</i>, <i>value2</i>), but usually requires less typing.</p></div>
pub unsafe fn q_make_pair_2_q_string(
    x: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
    y: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
) -> ::cpp_utils::CppBox<crate::QPairOfQStringQString> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qMakePair2(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(x).as_raw_ptr(),
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(y).as_raw_ptr(),
    );
    ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
}

/// <p>Returns a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; that contains <i>value1</i> and <i>value2</i>. Example:</p>
///
/// Calls C++ function: <span style='color: green;'>```QPair<QCborValueRef, QCborValueRef> qMakePair(const QCborValueRef& x, const QCborValueRef& y)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qpair.html#qMakePair">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; that contains <i>value1</i> and <i>value2</i>. Example:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> <span class="operator">&gt;</span> list;
///   list<span class="operator">.</span>append(qMakePair(<span class="number">66</span><span class="operator">,</span> <span class="number">3.14159</span>));
///
/// </pre>
/// <p>This is equivalent to <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt;(<i>value1</i>, <i>value2</i>), but usually requires less typing.</p></div>
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_make_pair_2_q_cbor_value_ref(
    x: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValueRef>>,
    y: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValueRef>>,
) -> ::cpp_utils::CppBox<crate::QPairOfQCborValueRefQCborValueRef> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qMakePair6(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValueRef>>::cast_into(x).as_raw_ptr(),
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValueRef>>::cast_into(y).as_raw_ptr(),
    );
    ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
}

/// <p>Returns a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; that contains <i>value1</i> and <i>value2</i>. Example:</p>
///
/// Calls C++ function: <span style='color: green;'>```QPair<QCborValue, QCborValue> qMakePair(const QCborValue& x, const QCborValue& y)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qpair.html#qMakePair">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; that contains <i>value1</i> and <i>value2</i>. Example:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> <span class="operator">&gt;</span> list;
///   list<span class="operator">.</span>append(qMakePair(<span class="number">66</span><span class="operator">,</span> <span class="number">3.14159</span>));
///
/// </pre>
/// <p>This is equivalent to <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt;(<i>value1</i>, <i>value2</i>), but usually requires less typing.</p></div>
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_make_pair_2_q_cbor_value(
    x: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
    y: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QCborValue>>,
) -> ::cpp_utils::CppBox<crate::QPairOfQCborValueQCborValue> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qMakePair7(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(x).as_raw_ptr(),
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QCborValue>>::cast_into(y).as_raw_ptr(),
    );
    ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
}

/// <p>Returns a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; that contains <i>value1</i> and <i>value2</i>. Example:</p>
///
/// Calls C++ function: <span style='color: green;'>```QPair<qint64, unsigned int> qMakePair(const qint64& x, const unsigned int& y)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qpair.html#qMakePair">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt; that contains <i>value1</i> and <i>value2</i>. Example:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qlist.html">QList</a></span><span class="operator">&lt;</span><span class="type"><a href="http://doc.qt.io/qt-5/qpair.html">QPair</a></span><span class="operator">&lt;</span><span class="type">int</span><span class="operator">,</span> <span class="type">double</span><span class="operator">&gt;</span> <span class="operator">&gt;</span> list;
///   list<span class="operator">.</span>append(qMakePair(<span class="number">66</span><span class="operator">,</span> <span class="number">3.14159</span>));
///
/// </pre>
/// <p>This is equivalent to <a href="http://doc.qt.io/qt-5/qpair.html">QPair</a>&lt;T1, T2&gt;(<i>value1</i>, <i>value2</i>), but usually requires less typing.</p></div>
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_make_pair_i64_uint(
    x: impl ::cpp_utils::CastInto<::cpp_utils::Ref<i64>>,
    y: impl ::cpp_utils::CastInto<::cpp_utils::Ref<::std::os::raw::c_uint>>,
) -> ::cpp_utils::CppBox<crate::QPairOfI64Uint> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qMakePair8(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<i64>>::cast_into(x).as_raw_ptr(),
        ::cpp_utils::CastInto::<::cpp_utils::Ref<::std::os::raw::c_uint>>::cast_into(y)
            .as_raw_ptr(),
    );
    ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
}

/// Calls C++ function: <span style='color: green;'>```void* qMallocAligned(unsigned long size, unsigned long alignment)```</span>.
pub unsafe fn q_malloc_aligned(
    size: ::std::os::raw::c_ulong,
    alignment: ::std::os::raw::c_ulong,
) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qMallocAligned(size, alignment);
    ::cpp_utils::MutPtr::from_raw(ffi_result)
}

/// <p>This function returns the nearest power of two greater than <i>value</i>. For 0 it returns 1, and for values larger than or equal to 2^31 it returns 0.</p>
///
/// Calls C++ function: <span style='color: green;'>```quint32 qNextPowerOfTwo(quint32 v)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtmath.html#qNextPowerOfTwo">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function returns the nearest power of two greater than <i>value</i>. For 0 it returns 1, and for values larger than or equal to 2^31 it returns 0.</p>
/// <p>This function was introduced in  Qt 5.4.</p></div>
pub unsafe fn q_next_power_of_two_u32(v: u32) -> u32 {
    crate::__ffi::ctr_qt_core_ffi_qNextPowerOfTwo(v)
}

/// <p>This function returns the nearest power of two greater than <i>value</i>. For 0 it returns 1, and for values larger than or equal to 2^63 it returns 0.</p>
///
/// Calls C++ function: <span style='color: green;'>```quint64 qNextPowerOfTwo(quint64 v)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtmath.html#qNextPowerOfTwo-2">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function returns the nearest power of two greater than <i>value</i>. For 0 it returns 1, and for values larger than or equal to 2^63 it returns 0.</p>
/// <p>This function was introduced in  Qt 5.4.</p></div>
pub unsafe fn q_next_power_of_two_u64(v: u64) -> u64 {
    crate::__ffi::ctr_qt_core_ffi_qNextPowerOfTwo1(v)
}

/// <p>This is an overloaded function.</p>
///
/// Calls C++ function: <span style='color: green;'>```quint32 qNextPowerOfTwo(qint32 v)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtmath.html#qNextPowerOfTwo-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
/// <p>This function returns the nearest power of two greater than <i>value</i>. For negative values it returns 0.</p>
/// <p>This function was introduced in  Qt 5.4.</p></div>
pub unsafe fn q_next_power_of_two_i32(v: i32) -> u32 {
    crate::__ffi::ctr_qt_core_ffi_qNextPowerOfTwo2(v)
}

/// <p>This is an overloaded function.</p>
///
/// Calls C++ function: <span style='color: green;'>```quint64 qNextPowerOfTwo(qint64 v)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtmath.html#qNextPowerOfTwo-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
/// <p>This function returns the nearest power of two greater than <i>value</i>. For negative values it returns 0.</p>
/// <p>This function was introduced in  Qt 5.4.</p></div>
pub unsafe fn q_next_power_of_two_i64(v: i64) -> u64 {
    crate::__ffi::ctr_qt_core_ffi_qNextPowerOfTwo3(v)
}

/// Calls C++ function: <span style='color: green;'>```unsigned char qPluginArchRequirements()```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_plugin_arch_requirements() -> ::std::os::raw::c_uchar {
    crate::__ffi::ctr_qt_core_ffi_qPluginArchRequirements()
}

/// <p>This is an overloaded function.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qPopulationCount(quint32 v)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qPopulationCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
/// <p>This function was introduced in  Qt 5.2.</p></div>
pub unsafe fn q_population_count_u32(v: u32) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qPopulationCount(v)
}

/// <p>Returns the number of bits set in <i>v</i>. This number is also called the Hamming Weight of <i>v</i>.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qPopulationCount(quint8 v)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qPopulationCount">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the number of bits set in <i>v</i>. This number is also called the Hamming Weight of <i>v</i>.</p>
/// <p>This function was introduced in  Qt 5.2.</p></div>
pub unsafe fn q_population_count_u8(v: u8) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qPopulationCount1(v)
}

/// <p>This is an overloaded function.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qPopulationCount(quint16 v)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qPopulationCount-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
/// <p>This function was introduced in  Qt 5.2.</p></div>
pub unsafe fn q_population_count_u16(v: u16) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qPopulationCount2(v)
}

/// <p>This is an overloaded function.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qPopulationCount(quint64 v)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtalgorithms.html#qPopulationCount-3">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This is an overloaded function.</p>
/// <p>This function was introduced in  Qt 5.2.</p></div>
pub unsafe fn q_population_count_u64(v: u64) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qPopulationCount3(v)
}

/// Calls C++ function: <span style='color: green;'>```unsigned int qPopulationCount(unsigned long v)```</span>.
pub unsafe fn q_population_count_ulong(v: ::std::os::raw::c_ulong) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qPopulationCount4(v)
}

/// <p>Returns the value of <i>x</i> raised to the power of <i>y</i>. That is, <i>x</i> is the base and <i>y</i> is the exponent.</p>
///
/// Calls C++ function: <span style='color: green;'>```double qPow(double x, double y)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtmath.html#qPow">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value of <i>x</i> raised to the power of <i>y</i>. That is, <i>x</i> is the base and <i>y</i> is the exponent.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtmath.html#qSqrt">qSqrt</a>().</p></div>
pub unsafe fn q_pow(
    x: ::std::os::raw::c_double,
    y: ::std::os::raw::c_double,
) -> ::std::os::raw::c_double {
    crate::__ffi::ctr_qt_core_ffi_qPow(x, y)
}

/// <p>Returns the bit pattern of a quiet NaN as a double.</p>
///
/// Calls C++ function: <span style='color: green;'>```double qQNaN()```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtglobal.html#qQNaN">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the bit pattern of a quiet NaN as a double.</p></div>
pub unsafe fn q_q_nan() -> ::std::os::raw::c_double {
    crate::__ffi::ctr_qt_core_ffi_qQNaN()
}

/// <p>This function converts the <i>radians</i> in float to degrees.</p>
///
/// Calls C++ function: <span style='color: green;'>```float qRadiansToDegrees(float radians)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtmath.html#qRadiansToDegrees">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function converts the <i>radians</i> in float to degrees.</p>
/// <p>Example:</p>
/// <pre class="cpp">
///
///   <span class="type">float</span> radians <span class="operator">=</span> <span class="type">float</span>(M_PI)
///   <span class="type">float</span> degrees <span class="operator">=</span> qRadiansToDegrees(radians)
///
/// </pre>
/// <p>This function was introduced in  Qt 5.1.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtmath.html#qDegreesToRadians">qDegreesToRadians</a>().</p></div>
pub unsafe fn q_radians_to_degrees_float(
    radians: ::std::os::raw::c_float,
) -> ::std::os::raw::c_float {
    crate::__ffi::ctr_qt_core_ffi_qRadiansToDegrees(radians)
}

/// <p>This function converts the <i>radians</i> in double to degrees.</p>
///
/// Calls C++ function: <span style='color: green;'>```double qRadiansToDegrees(double radians)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtmath.html#qRadiansToDegrees-1">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function converts the <i>radians</i> in double to degrees.</p>
/// <p>Example:</p>
/// <pre class="cpp">
///
///   <span class="type">double</span> radians <span class="operator">=</span> M_PI
///   <span class="type">double</span> degrees <span class="operator">=</span> <a href="http://doc.qt.io/qt-5/qtmath.html#qRadiansToDegrees">qRadiansToDegrees</a>(radians)
///
/// </pre>
/// <p>This function was introduced in  Qt 5.1.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtmath.html#qDegreesToRadians">qDegreesToRadians</a>().</p></div>
pub unsafe fn q_radians_to_degrees_double(
    radians: ::std::os::raw::c_double,
) -> ::std::os::raw::c_double {
    crate::__ffi::ctr_qt_core_ffi_qRadiansToDegrees1(radians)
}

/// Calls C++ function: <span style='color: green;'>```void* qReallocAligned(void* ptr, unsigned long size, unsigned long oldsize, unsigned long alignment)```</span>.
pub unsafe fn q_realloc_aligned(
    ptr: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
    size: ::std::os::raw::c_ulong,
    oldsize: ::std::os::raw::c_ulong,
    alignment: ::std::os::raw::c_ulong,
) -> ::cpp_utils::MutPtr<::std::ffi::c_void> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qReallocAligned(
        ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(ptr)
            .as_mut_raw_ptr(),
        size,
        oldsize,
        alignment,
    );
    ::cpp_utils::MutPtr::from_raw(ffi_result)
}

/// <p>Registers the <i>plugin</i> specified with the plugin loader, and is used by <a href="http://doc.qt.io/qt-5/qtplugin.html#Q_IMPORT_PLUGIN">Q_IMPORT_PLUGIN</a>().</p>
///
/// Calls C++ function: <span style='color: green;'>```void qRegisterStaticPluginFunction(QStaticPlugin staticPlugin)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qpluginloader.html#qRegisterStaticPluginFunction">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Registers the <i>plugin</i> specified with the plugin loader, and is used by <a href="http://doc.qt.io/qt-5/qtplugin.html#Q_IMPORT_PLUGIN">Q_IMPORT_PLUGIN</a>().</p>
/// <p>This function was introduced in  Qt 5.0.</p></div>
pub unsafe fn q_register_static_plugin_function(
    static_plugin: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QStaticPlugin>>,
) {
    crate::__ffi::ctr_qt_core_ffi_qRegisterStaticPluginFunction(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QStaticPlugin>>::cast_into(static_plugin)
            .as_raw_ptr(),
    )
}

/// <p>Removes the cleanup routine specified by <i>ptr</i> from the list of routines called by the <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> destructor. The routine must have been previously added to the list by a call to <a href="http://doc.qt.io/qt-5/qcoreapplication.html#qAddPostRoutine">qAddPostRoutine</a>(), otherwise this function has no effect.</p>
///
/// Calls C++ function: <span style='color: green;'>```void qRemovePostRoutine(void (*FN_PTR)() arg1)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qcoreapplication.html#qRemovePostRoutine">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Removes the cleanup routine specified by <i>ptr</i> from the list of routines called by the <a href="http://doc.qt.io/qt-5/qcoreapplication.html">QCoreApplication</a> destructor. The routine must have been previously added to the list by a call to <a href="http://doc.qt.io/qt-5/qcoreapplication.html#qAddPostRoutine">qAddPostRoutine</a>(), otherwise this function has no effect.</p>
/// <p>This function was introduced in  Qt 5.3.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qcoreapplication.html#qAddPostRoutine">qAddPostRoutine</a>().</p></div>
pub unsafe fn q_remove_post_routine(arg1: ::std::option::Option<extern "C" fn()>) {
    crate::__ffi::ctr_qt_core_ffi_qRemovePostRoutine(arg1)
}

/// <p>Rounds <i>value</i> to the nearest integer.</p>
///
/// Calls C++ function: <span style='color: green;'>```int qRound(double d)```</span>.
///
/// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qfloat16.html#qRound">C++ documentation</a> for <span style='color: green;'>```int qRound(qfloat16 value)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Rounds <i>value</i> to the nearest integer.</p>
/// <p><b>See also </b>qRound.</p></div>
pub unsafe fn q_round_double(d: ::std::os::raw::c_double) -> ::std::os::raw::c_int {
    crate::__ffi::ctr_qt_core_ffi_qRound(d)
}

/// <p>Rounds <i>value</i> to the nearest integer.</p>
///
/// Calls C++ function: <span style='color: green;'>```int qRound(float d)```</span>.
///
/// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qfloat16.html#qRound">C++ documentation</a> for <span style='color: green;'>```int qRound(qfloat16 value)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Rounds <i>value</i> to the nearest integer.</p>
/// <p><b>See also </b>qRound.</p></div>
pub unsafe fn q_round_float(d: ::std::os::raw::c_float) -> ::std::os::raw::c_int {
    crate::__ffi::ctr_qt_core_ffi_qRound1(d)
}

/// <p>Rounds <i>value</i> to the nearest integer.</p>
///
/// Calls C++ function: <span style='color: green;'>```int qRound(qfloat16 d)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qfloat16.html#qRound">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Rounds <i>value</i> to the nearest integer.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtglobal.html#qRound">qRound</a>.</p></div>
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_round_qfloat16(
    d: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::Qfloat16>>,
) -> ::std::os::raw::c_int {
    crate::__ffi::ctr_qt_core_ffi_qRound4(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::Qfloat16>>::cast_into(d).as_raw_ptr(),
    )
}

/// <p>Rounds <i>value</i> to the nearest 64-bit integer.</p>
///
/// Calls C++ function: <span style='color: green;'>```qint64 qRound64(double d)```</span>.
///
/// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qfloat16.html#qRound64">C++ documentation</a> for <span style='color: green;'>```qint64 qRound64(qfloat16 value)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Rounds <i>value</i> to the nearest 64-bit integer.</p>
/// <p><b>See also </b>qRound64.</p></div>
pub unsafe fn q_round64_double(d: ::std::os::raw::c_double) -> i64 {
    crate::__ffi::ctr_qt_core_ffi_qRound64(d)
}

/// <p>Rounds <i>value</i> to the nearest 64-bit integer.</p>
///
/// Calls C++ function: <span style='color: green;'>```qint64 qRound64(float d)```</span>.
///
/// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qfloat16.html#qRound64">C++ documentation</a> for <span style='color: green;'>```qint64 qRound64(qfloat16 value)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Rounds <i>value</i> to the nearest 64-bit integer.</p>
/// <p><b>See also </b>qRound64.</p></div>
pub unsafe fn q_round64_float(d: ::std::os::raw::c_float) -> i64 {
    crate::__ffi::ctr_qt_core_ffi_qRound641(d)
}

/// <p>Rounds <i>value</i> to the nearest 64-bit integer.</p>
///
/// Calls C++ function: <span style='color: green;'>```qint64 qRound64(qfloat16 d)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qfloat16.html#qRound64">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Rounds <i>value</i> to the nearest 64-bit integer.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtglobal.html#qRound64">qRound64</a>.</p></div>
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn q_round64_qfloat16(
    d: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::Qfloat16>>,
) -> i64 {
    crate::__ffi::ctr_qt_core_ffi_qRound644(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::Qfloat16>>::cast_into(d).as_raw_ptr(),
    )
}

/// <p>Returns the bit pattern of a signalling NaN as a double.</p>
///
/// Calls C++ function: <span style='color: green;'>```double qSNaN()```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtglobal.html#qSNaN">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the bit pattern of a signalling NaN as a double.</p></div>
pub unsafe fn q_s_nan() -> ::std::os::raw::c_double {
    crate::__ffi::ctr_qt_core_ffi_qSNaN()
}

/// <p>Sets the global <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> seed to <i>newSeed</i>.</p>
///
/// Calls C++ function: <span style='color: green;'>```void qSetGlobalQHashSeed(int newSeed)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qhash.html#qSetGlobalQHashSeed">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the global <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> seed to <i>newSeed</i>.</p>
/// <p>Manually setting the global <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> seed value should be done only for testing and debugging purposes, when deterministic and reproducible behavior on a <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> is needed. We discourage to do it in production code as it can make your application susceptible to <a href="http://doc.qt.io/qt-5/qhash.html#algorithmic-complexity-attacks">algorithmic complexity attacks</a>.</p>
/// <p>The seed is set in any newly created <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>. See <a href="http://doc.qt.io/qt-5/qhash.html#qhash">qHash</a> about how this seed is being used by <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a>.</p>
/// <p>If the environment variable <code>QT_HASH_SEED</code> is set, calling this function will result in a no-op.</p>
/// <p>Passing the value -1 will reinitialize the global <a href="http://doc.qt.io/qt-5/qhash.html#qhash">QHash</a> seed to a random value.</p>
/// <p>This function was introduced in  Qt 5.6.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qhash.html#qGlobalQHashSeed">qGlobalQHashSeed</a>.</p></div>
pub unsafe fn q_set_global_q_hash_seed(new_seed: ::std::os::raw::c_int) {
    crate::__ffi::ctr_qt_core_ffi_qSetGlobalQHashSeed(new_seed)
}

/// <p>Changes the output of the default message handler.</p>
///
/// Calls C++ function: <span style='color: green;'>```void qSetMessagePattern(const QString& messagePattern)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtglobal.html#qSetMessagePattern">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Changes the output of the default message handler.</p>
/// <p>Allows to tweak the output of <a href="http://doc.qt.io/qt-5/qtglobal.html#qDebug">qDebug</a>(), <a href="http://doc.qt.io/qt-5/qtglobal.html#qInfo">qInfo</a>(), <a href="http://doc.qt.io/qt-5/qtglobal.html#qWarning">qWarning</a>(), <a href="http://doc.qt.io/qt-5/qtglobal.html#qCritical">qCritical</a>(), and <a href="http://doc.qt.io/qt-5/qtglobal.html#qFatal">qFatal</a>(). The category logging output of <a href="http://doc.qt.io/qt-5/qloggingcategory.html#qCDebug">qCDebug</a>(), <a href="http://doc.qt.io/qt-5/qloggingcategory.html#qCInfo">qCInfo</a>(), <a href="http://doc.qt.io/qt-5/qloggingcategory.html#qCWarning">qCWarning</a>(), and <a href="http://doc.qt.io/qt-5/qloggingcategory.html#qCCritical">qCCritical</a>() is formatted, too.</p>
/// <p>Following placeholders are supported:</p>
/// <div class="table"><table class="generic">
///  <thead><tr class="qt-style"><th>Placeholder</th><th>Description</th></tr></thead>
/// <tbody><tr valign="top" class="odd"><td><code>%{appname}</code></td><td><a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationName-prop">QCoreApplication::applicationName</a>()</td></tr>
/// <tr valign="top" class="even"><td><code>%{category}</code></td><td>Logging category</td></tr>
/// <tr valign="top" class="odd"><td><code>%{file}</code></td><td>Path to source file</td></tr>
/// <tr valign="top" class="even"><td><code>%{function}</code></td><td>Function</td></tr>
/// <tr valign="top" class="odd"><td><code>%{line}</code></td><td>Line in source file</td></tr>
/// <tr valign="top" class="even"><td><code>%{message}</code></td><td>The actual message</td></tr>
/// <tr valign="top" class="odd"><td><code>%{pid}</code></td><td><a href="http://doc.qt.io/qt-5/qcoreapplication.html#applicationPid">QCoreApplication::applicationPid</a>()</td></tr>
/// <tr valign="top" class="even"><td><code>%{threadid}</code></td><td>The system-wide ID of current thread (if it can be obtained)</td></tr>
/// <tr valign="top" class="odd"><td><code>%{qthreadptr}</code></td><td>A pointer to the current <a href="http://doc.qt.io/qt-5/qthread.html">QThread</a> (result of <a href="http://doc.qt.io/qt-5/qthread.html#currentThread">QThread::currentThread</a>())</td></tr>
/// <tr valign="top" class="even"><td><code>%{type}</code></td><td>"debug", "warning", "critical" or "fatal"</td></tr>
/// <tr valign="top" class="odd"><td><code>%{time process}</code></td><td>time of the message, in seconds since the process started (the token "process" is literal)</td></tr>
/// <tr valign="top" class="even"><td><code>%{time boot}</code></td><td>the time of the message, in seconds since the system boot if that can be determined (the token "boot" is literal). If the time since boot could not be obtained, the output is indeterminate (see <a href="http://doc.qt.io/qt-5/qelapsedtimer.html#msecsSinceReference">QElapsedTimer::msecsSinceReference</a>()).</td></tr>
/// <tr valign="top" class="odd"><td><code>%{time [format]}</code></td><td>system time when the message occurred, formatted by passing the <code>format</code> to <a href="http://doc.qt.io/qt-5/qdatetime.html#toString">QDateTime::toString</a>(). If the format is not specified, the format of <a href="http://doc.qt.io/qt-5/qt.html#DateFormat-enum">Qt::ISODate</a> is used.</td></tr>
/// <tr valign="top" class="even"><td><code>%{backtrace [depth=N] [separator="..."]}</code></td><td>A backtrace with the number of frames specified by the optional <code>depth</code> parameter (defaults to 5), and separated by the optional <code>separator</code> parameter (defaults to "|"). This expansion is available only on some platforms (currently only platfoms using glibc). Names are only known for exported functions. If you want to see the name of every function in your application, use <code>QMAKE_LFLAGS += -rdynamic</code>. When reading backtraces, take into account that frames might be missing due to inlining or tail call optimization.</td></tr>
/// </tbody></table></div>
/// <p>You can also use conditionals on the type of the message using <code>%{if-debug}</code>, <code>%{if-info}</code> <code>%{if-warning}</code>, <code>%{if-critical}</code> or <code>%{if-fatal}</code> followed by an <code>%{endif}</code>. What is inside the <code>%{if-*}</code> and <code>%{endif}</code> will only be printed if the type matches.</p>
/// <p>Finally, text inside <code>%{if-category}</code> ... <code>%{endif}</code> is only printed if the category is not the default one.</p>
/// <p>Example:</p>
/// <pre class="cpp">
///
///   QT_MESSAGE_PATTERN<span class="operator">=</span><span class="string">"[%{time yyyyMMdd h:mm:ss.zzz t} %{if-debug}D%{endif}%{if-info}I%{endif}%{if-warning}W%{endif}%{if-critical}C%{endif}%{if-fatal}F%{endif}] %{file}:%{line} - %{message}"</span>
///
/// </pre>
/// <p>The default <i>pattern</i> is "%{if-category}%{category}: %{endif}%{message}".</p>
/// <p>The <i>pattern</i> can also be changed at runtime by setting the QT_MESSAGE_PATTERN environment variable; if both qSetMessagePattern() is called and QT_MESSAGE_PATTERN is set, the environment variable takes precedence.</p>
/// <p>Custom message handlers can use <a href="http://doc.qt.io/qt-5/qtglobal.html#qFormatLogMessage">qFormatLogMessage</a>() to take <i>pattern</i> into account.</p>
/// <p>This function was introduced in  Qt 5.0.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtglobal.html#qInstallMessageHandler">qInstallMessageHandler</a>(), <a href="http://doc.qt.io/qt-5/testing-and-debugging.html#debugging-techniques">Debugging Techniques</a>, and <a href="http://doc.qt.io/qt-5/qloggingcategory.html">QLoggingCategory</a>.</p></div>
pub unsafe fn q_set_message_pattern(
    message_pattern: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
) {
    crate::__ffi::ctr_qt_core_ffi_qSetMessagePattern(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(message_pattern)
            .as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```bool qSharedBuild()```</span>.
pub unsafe fn q_shared_build() -> bool {
    crate::__ffi::ctr_qt_core_ffi_qSharedBuild()
}

/// <p>Returns the sine of the angle <i>v</i> in radians.</p>
///
/// Calls C++ function: <span style='color: green;'>```double qSin(double v)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtmath.html#qSin">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the sine of the angle <i>v</i> in radians.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtmath.html#qCos">qCos</a>() and <a href="http://doc.qt.io/qt-5/qtmath.html#qTan">qTan</a>().</p></div>
pub unsafe fn q_sin(v: ::std::os::raw::c_double) -> ::std::os::raw::c_double {
    crate::__ffi::ctr_qt_core_ffi_qSin(v)
}

/// <p>Returns the square root of <i>v</i>. This function returns a NaN if <i>v</i> is a negative number.</p>
///
/// Calls C++ function: <span style='color: green;'>```double qSqrt(double v)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtmath.html#qSqrt">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the square root of <i>v</i>. This function returns a NaN if <i>v</i> is a negative number.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtmath.html#qPow">qPow</a>().</p></div>
pub unsafe fn q_sqrt(v: ::std::os::raw::c_double) -> ::std::os::raw::c_double {
    crate::__ffi::ctr_qt_core_ffi_qSqrt(v)
}

/// <p>Returns the tangent of an angle <i>v</i> in radians.</p>
///
/// Calls C++ function: <span style='color: green;'>```double qTan(double v)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtmath.html#qTan">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the tangent of an angle <i>v</i> in radians.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtmath.html#qSin">qSin</a>() and <a href="http://doc.qt.io/qt-5/qtmath.html#qCos">qCos</a>().</p></div>
pub unsafe fn q_tan(v: ::std::os::raw::c_double) -> ::std::os::raw::c_double {
    crate::__ffi::ctr_qt_core_ffi_qTan(v)
}

/// <p>Uncompresses the <i>data</i> byte array and returns a new byte array with the uncompressed data.</p>
///
/// Calls C++ function: <span style='color: green;'>```QByteArray qUncompress(const unsigned char* data, int nbytes)```</span>.
///
/// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbytearray.html#qUncompress">C++ documentation</a> for <span style='color: green;'>```QByteArray qUncompress(const QByteArray &data)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Uncompresses the <i>data</i> byte array and returns a new byte array with the uncompressed data.</p>
/// <p>Returns an empty <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> if the input data was corrupt.</p>
/// <p>This function will uncompress data compressed with <a href="http://doc.qt.io/qt-5/qbytearray.html#qCompress">qCompress</a>() from this and any earlier Qt version, back to Qt 3.1 when this feature was added.</p>
/// <p><b>Note:</b> If you want to use this function to uncompress external data that was compressed using zlib, you first need to prepend a four byte header to the byte array containing the data. The header must contain the expected length (in bytes) of the uncompressed data, expressed as an unsigned, big-endian, 32-bit integer.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#qCompress">qCompress</a>().</p></div>
pub unsafe fn q_uncompress_2a(
    data: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_uchar>>,
    nbytes: ::std::os::raw::c_int,
) -> ::cpp_utils::CppBox<crate::QByteArray> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qUncompress(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_uchar>>::cast_into(data)
            .as_raw_ptr(),
        nbytes,
    );
    ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
}

/// <p>Uncompresses the <i>data</i> byte array and returns a new byte array with the uncompressed data.</p>
///
/// Calls C++ function: <span style='color: green;'>```QByteArray qUncompress(const QByteArray& data)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qbytearray.html#qUncompress">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Uncompresses the <i>data</i> byte array and returns a new byte array with the uncompressed data.</p>
/// <p>Returns an empty <a href="http://doc.qt.io/qt-5/qbytearray.html">QByteArray</a> if the input data was corrupt.</p>
/// <p>This function will uncompress data compressed with <a href="http://doc.qt.io/qt-5/qbytearray.html#qCompress">qCompress</a>() from this and any earlier Qt version, back to Qt 3.1 when this feature was added.</p>
/// <p><b>Note:</b> If you want to use this function to uncompress external data that was compressed using zlib, you first need to prepend a four byte header to the byte array containing the data. The header must contain the expected length (in bytes) of the uncompressed data, expressed as an unsigned, big-endian, 32-bit integer.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#qCompress">qCompress</a>().</p></div>
pub unsafe fn q_uncompress_1a(
    data: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
) -> ::cpp_utils::CppBox<crate::QByteArray> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qUncompress1(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(data).as_raw_ptr(),
    );
    ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
}

/// <p>Returns a variant containing a copy of the given <i>value</i> with template type <code>T</code>.</p>
///
/// Calls C++ function: <span style='color: green;'>```QVariant qVariantFromValue(const QVariant& t)```</span>.
///
/// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qvariant-obsolete.html#qVariantFromValue-1">C++ documentation</a> for <span style='color: green;'>```QVariant qVariantFromValue(const T &value)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a variant containing a copy of the given <i>value</i> with template type <code>T</code>.</p>
/// <p>This function is equivalent to <a href="http://doc.qt.io/qt-5/qvariant.html#fromValue">QVariant::fromValue</a>(<i>value</i>).</p>
/// <p><b>Note: </b>This function was provided as a workaround for MSVC 6 which did not support member template functions. It is advised to use the other form in new code.</p><p>For example, a <a href="http://doc.qt.io/qt-5/qobject.html">QObject</a> pointer can be stored in a variant with the following code:</p>
/// <pre class="cpp">
///
///   <span class="type"><a href="http://doc.qt.io/qt-5/qobject.html">QObject</a></span> <span class="operator">*</span>object <span class="operator">=</span> getObjectFromSomewhere();
///   <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span> data <span class="operator">=</span> <span class="type"><a href="http://doc.qt.io/qt-5/qvariant.html">QVariant</a></span><span class="operator">::</span>fromValue(object);
///
/// </pre>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#fromValue">QVariant::fromValue</a>().</p></div>
pub unsafe fn q_variant_from_value(
    t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
) -> ::cpp_utils::CppBox<crate::QVariant> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qVariantFromValue(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(t).as_raw_ptr(),
    );
    ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
}

/// <p>Sets the contents of the given <i>variant</i> to a copy of the <i>value</i> with the specified template type <code>T</code>.</p>
///
/// Calls C++ function: <span style='color: green;'>```void qVariantSetValue(QVariant& v, const QVariant& t)```</span>.
///
/// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qvariant-obsolete.html#qVariantSetValue-1">C++ documentation</a> for <span style='color: green;'>```void qVariantSetValue(QVariant &variant, const T &value)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Sets the contents of the given <i>variant</i> to a copy of the <i>value</i> with the specified template type <code>T</code>.</p>
/// <p>This function is equivalent to <a href="http://doc.qt.io/qt-5/qvariant.html#setValue">QVariant::setValue</a>(<i>value</i>).</p>
/// <p><b>Note: </b>This function was provided as a workaround for MSVC 6 which did not support member template functions. It is advised to use the other form in new code.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qvariant.html#setValue">QVariant::setValue</a>().</p></div>
pub unsafe fn q_variant_set_value(
    v: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QVariant>>,
    t: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QVariant>>,
) {
    crate::__ffi::ctr_qt_core_ffi_qVariantSetValue(
        ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QVariant>>::cast_into(v)
            .as_mut_raw_ptr(),
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QVariant>>::cast_into(t).as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```quint64 qbswap(quint64 source)```</span>.
pub unsafe fn qbswap_u64(source: u64) -> u64 {
    crate::__ffi::ctr_qt_core_ffi_qbswap(source)
}

/// Calls C++ function: <span style='color: green;'>```quint32 qbswap(quint32 source)```</span>.
pub unsafe fn qbswap_u32(source: u32) -> u32 {
    crate::__ffi::ctr_qt_core_ffi_qbswap1(source)
}

/// Calls C++ function: <span style='color: green;'>```void qbswap(quint64 source, void* dest)```</span>.
pub unsafe fn qbswap_u64_void(
    source: u64,
    dest: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
) {
    crate::__ffi::ctr_qt_core_ffi_qbswap2(
        source,
        ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(dest)
            .as_mut_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```void qbswap(quint32 source, void* dest)```</span>.
pub unsafe fn qbswap_u32_void(
    source: u32,
    dest: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
) {
    crate::__ffi::ctr_qt_core_ffi_qbswap3(
        source,
        ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(dest)
            .as_mut_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```quint16 qbswap(quint16 source)```</span>.
pub unsafe fn qbswap_u16(source: u16) -> u16 {
    crate::__ffi::ctr_qt_core_ffi_qbswap4(source)
}

/// Calls C++ function: <span style='color: green;'>```void qbswap(quint16 source, void* dest)```</span>.
pub unsafe fn qbswap_u16_void(
    source: u16,
    dest: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
) {
    crate::__ffi::ctr_qt_core_ffi_qbswap5(
        source,
        ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(dest)
            .as_mut_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```qint64 qbswap(qint64 source)```</span>.
pub unsafe fn qbswap_i64(source: i64) -> i64 {
    crate::__ffi::ctr_qt_core_ffi_qbswap6(source)
}

/// Calls C++ function: <span style='color: green;'>```qint32 qbswap(qint32 source)```</span>.
pub unsafe fn qbswap_i32(source: i32) -> i32 {
    crate::__ffi::ctr_qt_core_ffi_qbswap7(source)
}

/// Calls C++ function: <span style='color: green;'>```qint16 qbswap(qint16 source)```</span>.
pub unsafe fn qbswap_i16(source: i16) -> i16 {
    crate::__ffi::ctr_qt_core_ffi_qbswap8(source)
}

/// Calls C++ function: <span style='color: green;'>```void qbswap(qint64 source, void* dest)```</span>.
pub unsafe fn qbswap_i64_void(
    source: i64,
    dest: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
) {
    crate::__ffi::ctr_qt_core_ffi_qbswap9(
        source,
        ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(dest)
            .as_mut_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```void qbswap(qint32 source, void* dest)```</span>.
pub unsafe fn qbswap_i32_void(
    source: i32,
    dest: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
) {
    crate::__ffi::ctr_qt_core_ffi_qbswap10(
        source,
        ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(dest)
            .as_mut_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```void qbswap(qint16 source, void* dest)```</span>.
pub unsafe fn qbswap_i16_void(
    source: i16,
    dest: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::ffi::c_void>>,
) {
    crate::__ffi::ctr_qt_core_ffi_qbswap11(
        source,
        ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::ffi::c_void>>::cast_into(dest)
            .as_mut_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```quint8 qbswap(quint8 source)```</span>.
pub unsafe fn qbswap_u8(source: u8) -> u8 {
    crate::__ffi::ctr_qt_core_ffi_qbswap12(source)
}

/// Calls C++ function: <span style='color: green;'>```qint8 qbswap(qint8 source)```</span>.
pub unsafe fn qbswap_i8(source: i8) -> i8 {
    crate::__ffi::ctr_qt_core_ffi_qbswap13(source)
}

/// Calls C++ function: <span style='color: green;'>```qfloat16 qbswap(qfloat16 source)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn qbswap_qfloat16(
    source: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::Qfloat16>>,
) -> ::cpp_utils::CppBox<crate::Qfloat16> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qbswap28(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::Qfloat16>>::cast_into(source).as_raw_ptr(),
    );
    ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
}

/// Calls C++ function: <span style='color: green;'>```float qbswap(float source)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn qbswap_float(source: ::std::os::raw::c_float) -> ::std::os::raw::c_float {
    crate::__ffi::ctr_qt_core_ffi_qbswap29(source)
}

/// Calls C++ function: <span style='color: green;'>```double qbswap(double source)```</span>.
///
///
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn qbswap_double(source: ::std::os::raw::c_double) -> ::std::os::raw::c_double {
    crate::__ffi::ctr_qt_core_ffi_qbswap30(source)
}

/// <p>Returns the value of the environment variable with name <i>varName</i>. To get the variable string, use <a href="http://doc.qt.io/qt-5/qbytearray.html#constData">QByteArray::constData</a>(). To convert the data to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> use <a href="http://doc.qt.io/qt-5/qstring.html#fromLocal8Bit">QString::fromLocal8Bit</a>().</p>
///
/// Calls C++ function: <span style='color: green;'>```QByteArray qgetenv(const char* varName)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtglobal.html#qgetenv">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns the value of the environment variable with name <i>varName</i>. To get the variable string, use <a href="http://doc.qt.io/qt-5/qbytearray.html#constData">QByteArray::constData</a>(). To convert the data to a <a href="http://doc.qt.io/qt-5/qstring.html">QString</a> use <a href="http://doc.qt.io/qt-5/qstring.html#fromLocal8Bit">QString::fromLocal8Bit</a>().</p>
/// <p><b>Note: </b>qgetenv() was introduced because getenv() from the standard C library was deprecated in VC2005 (and later versions). qgetenv() uses the new replacement function in VC, and calls the standard C library's implementation on all other platforms.</p><p><b>Warning:</b> Don't use qgetenv on Windows if the content may contain non-US-ASCII characters, like file paths.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtglobal.html#qputenv">qputenv</a>(), <a href="http://doc.qt.io/qt-5/qtglobal.html#qEnvironmentVariableIsSet">qEnvironmentVariableIsSet</a>(), and <a href="http://doc.qt.io/qt-5/qtglobal.html#qEnvironmentVariableIsEmpty">qEnvironmentVariableIsEmpty</a>().</p></div>
pub unsafe fn qgetenv(
    var_name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
) -> ::cpp_utils::CppBox<crate::QByteArray> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qgetenv(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(var_name)
            .as_raw_ptr(),
    );
    ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
}

/// <p>This function sets the <i>value</i> of the environment variable named <i>varName</i>. It will create the variable if it does not exist. It returns 0 if the variable could not be set.</p>
///
/// Calls C++ function: <span style='color: green;'>```bool qputenv(const char* varName, const QByteArray& value)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtglobal.html#qputenv">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function sets the <i>value</i> of the environment variable named <i>varName</i>. It will create the variable if it does not exist. It returns 0 if the variable could not be set.</p>
/// <p>Calling qputenv with an empty value removes the environment variable on Windows, and makes it set (but empty) on Unix. Prefer using <a href="http://doc.qt.io/qt-5/qtglobal.html#qunsetenv">qunsetenv</a>() for fully portable behavior.</p>
/// <p><b>Note: </b>qputenv() was introduced because putenv() from the standard C library was deprecated in VC2005 (and later versions). qputenv() uses the replacement function in VC, and calls the standard C library's implementation on all other platforms.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qtglobal.html#qgetenv">qgetenv</a>().</p></div>
pub unsafe fn qputenv(
    var_name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    value: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
) -> bool {
    crate::__ffi::ctr_qt_core_ffi_qputenv(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(var_name)
            .as_raw_ptr(),
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(value).as_raw_ptr(),
    )
}

/// <p>Thread-safe version of the standard C++ <code>rand()</code> function.</p>
///
/// Calls C++ function: <span style='color: green;'>```int qrand()```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtglobal.html#qrand">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Thread-safe version of the standard C++ <code>rand()</code> function.</p>
/// <p>Returns a value between 0 and <code>RAND_MAX</code> (defined in <code>&lt;cstdlib&gt;</code> and <code>&lt;stdlib.h&gt;</code>), the next number in the current sequence of pseudo-random integers.</p>
/// <p>Use <code>qsrand()</code> to initialize the pseudo-random number generator with a seed value.</p>
/// <p>This function was introduced in  Qt 4.2.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtglobal.html#qsrand">qsrand</a>().</p></div>
pub unsafe fn qrand() -> ::std::os::raw::c_int {
    crate::__ffi::ctr_qt_core_ffi_qrand()
}

/// <p>Thread-safe version of the standard C++ <code>srand()</code> function.</p>
///
/// Calls C++ function: <span style='color: green;'>```void qsrand(unsigned int seed)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtglobal.html#qsrand">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Thread-safe version of the standard C++ <code>srand()</code> function.</p>
/// <p>Sets the argument <i>seed</i> to be used to generate a new random number sequence of pseudo random integers to be returned by <a href="http://doc.qt.io/qt-5/qtglobal.html#qrand">qrand</a>().</p>
/// <p>The sequence of random numbers generated is deterministic per thread. For example, if two threads call qsrand(1) and subsequently call <a href="http://doc.qt.io/qt-5/qtglobal.html#qrand">qrand</a>(), the threads will get the same random number sequence.</p>
/// <p>This function was introduced in  Qt 4.2.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtglobal.html#qrand">qrand</a>().</p></div>
pub unsafe fn qsrand(seed: ::std::os::raw::c_uint) {
    crate::__ffi::ctr_qt_core_ffi_qsrand(seed)
}

/// <p>A safe <code>strcmp()</code> function.</p>
///
/// Calls C++ function: <span style='color: green;'>```int qstrcmp(const char* str1, const char* str2)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qbytearray.html#qstrcmp">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>A safe <code>strcmp()</code> function.</p>
/// <p>Compares <i>str1</i> and <i>str2</i>. Returns a negative value if <i>str1</i> is less than <i>str2</i>, 0 if <i>str1</i> is equal to <i>str2</i> or a positive value if <i>str1</i> is greater than <i>str2</i>.</p>
/// <p>Special case 1: Returns 0 if <i>str1</i> and <i>str2</i> are both 0.</p>
/// <p>Special case 2: Returns an arbitrary non-zero value if <i>str1</i> is 0 or <i>str2</i> is 0 (but not both).</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#qstrncmp">qstrncmp</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#qstricmp">qstricmp</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#qstrnicmp">qstrnicmp</a>(), and <a href="http://doc.qt.io/qt-5/qbytearray.html#8-bit-character-comparisons">8-bit Character Comparisons</a>.</p></div>
pub unsafe fn qstrcmp_2_char(
    str1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    str2: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
) -> ::std::os::raw::c_int {
    crate::__ffi::ctr_qt_core_ffi_qstrcmp(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(str1)
            .as_raw_ptr(),
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(str2)
            .as_raw_ptr(),
    )
}

/// <p>A safe <code>strcmp()</code> function.</p>
///
/// Calls C++ function: <span style='color: green;'>```int qstrcmp(const QByteArray& str1, const QByteArray& str2)```</span>.
///
/// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbytearray.html#qstrcmp">C++ documentation</a> for <span style='color: green;'>```int qstrcmp(const char *str1, const char *str2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>A safe <code>strcmp()</code> function.</p>
/// <p>Compares <i>str1</i> and <i>str2</i>. Returns a negative value if <i>str1</i> is less than <i>str2</i>, 0 if <i>str1</i> is equal to <i>str2</i> or a positive value if <i>str1</i> is greater than <i>str2</i>.</p>
/// <p>Special case 1: Returns 0 if <i>str1</i> and <i>str2</i> are both 0.</p>
/// <p>Special case 2: Returns an arbitrary non-zero value if <i>str1</i> is 0 or <i>str2</i> is 0 (but not both).</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#qstrncmp">qstrncmp</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#qstricmp">qstricmp</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#qstrnicmp">qstrnicmp</a>(), and <a href="http://doc.qt.io/qt-5/qbytearray.html#8-bit-character-comparisons">8-bit Character Comparisons</a>.</p></div>
pub unsafe fn qstrcmp_2_q_byte_array(
    str1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    str2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
) -> ::std::os::raw::c_int {
    crate::__ffi::ctr_qt_core_ffi_qstrcmp1(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(str1).as_raw_ptr(),
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(str2).as_raw_ptr(),
    )
}

/// <p>A safe <code>strcmp()</code> function.</p>
///
/// Calls C++ function: <span style='color: green;'>```int qstrcmp(const QByteArray& str1, const char* str2)```</span>.
///
/// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbytearray.html#qstrcmp">C++ documentation</a> for <span style='color: green;'>```int qstrcmp(const char *str1, const char *str2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>A safe <code>strcmp()</code> function.</p>
/// <p>Compares <i>str1</i> and <i>str2</i>. Returns a negative value if <i>str1</i> is less than <i>str2</i>, 0 if <i>str1</i> is equal to <i>str2</i> or a positive value if <i>str1</i> is greater than <i>str2</i>.</p>
/// <p>Special case 1: Returns 0 if <i>str1</i> and <i>str2</i> are both 0.</p>
/// <p>Special case 2: Returns an arbitrary non-zero value if <i>str1</i> is 0 or <i>str2</i> is 0 (but not both).</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#qstrncmp">qstrncmp</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#qstricmp">qstricmp</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#qstrnicmp">qstrnicmp</a>(), and <a href="http://doc.qt.io/qt-5/qbytearray.html#8-bit-character-comparisons">8-bit Character Comparisons</a>.</p></div>
pub unsafe fn qstrcmp_q_byte_array_char(
    str1: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
    str2: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
) -> ::std::os::raw::c_int {
    crate::__ffi::ctr_qt_core_ffi_qstrcmp2(
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(str1).as_raw_ptr(),
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(str2)
            .as_raw_ptr(),
    )
}

/// <p>A safe <code>strcmp()</code> function.</p>
///
/// Calls C++ function: <span style='color: green;'>```int qstrcmp(const char* str1, const QByteArray& str2)```</span>.
///
/// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbytearray.html#qstrcmp">C++ documentation</a> for <span style='color: green;'>```int qstrcmp(const char *str1, const char *str2)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>A safe <code>strcmp()</code> function.</p>
/// <p>Compares <i>str1</i> and <i>str2</i>. Returns a negative value if <i>str1</i> is less than <i>str2</i>, 0 if <i>str1</i> is equal to <i>str2</i> or a positive value if <i>str1</i> is greater than <i>str2</i>.</p>
/// <p>Special case 1: Returns 0 if <i>str1</i> and <i>str2</i> are both 0.</p>
/// <p>Special case 2: Returns an arbitrary non-zero value if <i>str1</i> is 0 or <i>str2</i> is 0 (but not both).</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#qstrncmp">qstrncmp</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#qstricmp">qstricmp</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#qstrnicmp">qstrnicmp</a>(), and <a href="http://doc.qt.io/qt-5/qbytearray.html#8-bit-character-comparisons">8-bit Character Comparisons</a>.</p></div>
pub unsafe fn qstrcmp_char_q_byte_array(
    str1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    str2: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QByteArray>>,
) -> ::std::os::raw::c_int {
    crate::__ffi::ctr_qt_core_ffi_qstrcmp3(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(str1)
            .as_raw_ptr(),
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QByteArray>>::cast_into(str2).as_raw_ptr(),
    )
}

/// <p>Copies all the characters up to and including the '\0' from <i>src</i> into <i>dst</i> and returns a pointer to <i>dst</i>. If <i>src</i> is 0, it immediately returns 0.</p>
///
/// Calls C++ function: <span style='color: green;'>```char* qstrcpy(char* dst, const char* src)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qbytearray.html#qstrcpy">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Copies all the characters up to and including the '\0' from <i>src</i> into <i>dst</i> and returns a pointer to <i>dst</i>. If <i>src</i> is 0, it immediately returns 0.</p>
/// <p>This function assumes that <i>dst</i> is large enough to hold the contents of <i>src</i>.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#qstrncpy">qstrncpy</a>().</p></div>
pub unsafe fn qstrcpy(
    dst: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_char>>,
    src: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
) -> ::cpp_utils::MutPtr<::std::os::raw::c_char> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qstrcpy(
        ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_char>>::cast_into(dst)
            .as_mut_raw_ptr(),
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(src)
            .as_raw_ptr(),
    );
    ::cpp_utils::MutPtr::from_raw(ffi_result)
}

/// <p>Returns a duplicate string.</p>
///
/// Calls C++ function: <span style='color: green;'>```char* qstrdup(const char* arg1)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qbytearray.html#qstrdup">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>Returns a duplicate string.</p>
/// <p>Allocates space for a copy of <i>src</i>, copies it, and returns a pointer to the copy. If <i>src</i> is 0, it immediately returns 0.</p>
/// <p>Ownership is passed to the caller, so the returned string must be deleted using <code>delete[]</code>.</p></div>
pub unsafe fn qstrdup(
    arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
) -> ::cpp_utils::MutPtr<::std::os::raw::c_char> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qstrdup(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
            .as_raw_ptr(),
    );
    ::cpp_utils::MutPtr::from_raw(ffi_result)
}

/// <p>A safe <code>stricmp()</code> function.</p>
///
/// Calls C++ function: <span style='color: green;'>```int qstricmp(const char* arg1, const char* arg2)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qbytearray.html#qstricmp">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>A safe <code>stricmp()</code> function.</p>
/// <p>Compares <i>str1</i> and <i>str2</i> ignoring the case of the characters. The encoding of the strings is assumed to be Latin-1.</p>
/// <p>Returns a negative value if <i>str1</i> is less than <i>str2</i>, 0 if <i>str1</i> is equal to <i>str2</i> or a positive value if <i>str1</i> is greater than <i>str2</i>.</p>
/// <p>Special case 1: Returns 0 if <i>str1</i> and <i>str2</i> are both 0.</p>
/// <p>Special case 2: Returns a random non-zero value if <i>str1</i> is 0 or <i>str2</i> is 0 (but not both).</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#qstrcmp">qstrcmp</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#qstrncmp">qstrncmp</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#qstrnicmp">qstrnicmp</a>(), and <a href="http://doc.qt.io/qt-5/qbytearray.html#8-bit-character-comparisons">8-bit Character Comparisons</a>.</p></div>
pub unsafe fn qstricmp(
    arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    arg2: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
) -> ::std::os::raw::c_int {
    crate::__ffi::ctr_qt_core_ffi_qstricmp(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
            .as_raw_ptr(),
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg2)
            .as_raw_ptr(),
    )
}

/// <p>A safe <code>strlen()</code> function.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qstrlen(const char* str)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qbytearray.html#qstrlen">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>A safe <code>strlen()</code> function.</p>
/// <p>Returns the number of characters that precede the terminating '\0', or 0 if <i>str</i> is 0.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#qstrnlen">qstrnlen</a>().</p></div>
pub unsafe fn qstrlen(
    str: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qstrlen(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(str)
            .as_raw_ptr(),
    )
}

/// <p>A safe <code>strncmp()</code> function.</p>
///
/// Calls C++ function: <span style='color: green;'>```int qstrncmp(const char* str1, const char* str2, unsigned int len)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qbytearray.html#qstrncmp">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>A safe <code>strncmp()</code> function.</p>
/// <p>Compares at most <i>len</i> bytes of <i>str1</i> and <i>str2</i>.</p>
/// <p>Returns a negative value if <i>str1</i> is less than <i>str2</i>, 0 if <i>str1</i> is equal to <i>str2</i> or a positive value if <i>str1</i> is greater than <i>str2</i>.</p>
/// <p>Special case 1: Returns 0 if <i>str1</i> and <i>str2</i> are both 0.</p>
/// <p>Special case 2: Returns a random non-zero value if <i>str1</i> is 0 or <i>str2</i> is 0 (but not both).</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#qstrcmp">qstrcmp</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#qstricmp">qstricmp</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#qstrnicmp">qstrnicmp</a>(), and <a href="http://doc.qt.io/qt-5/qbytearray.html#8-bit-character-comparisons">8-bit Character Comparisons</a>.</p></div>
pub unsafe fn qstrncmp(
    str1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    str2: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    len: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_int {
    crate::__ffi::ctr_qt_core_ffi_qstrncmp(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(str1)
            .as_raw_ptr(),
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(str2)
            .as_raw_ptr(),
        len,
    )
}

/// <p>A safe <code>strncpy()</code> function.</p>
///
/// Calls C++ function: <span style='color: green;'>```char* qstrncpy(char* dst, const char* src, unsigned int len)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qbytearray.html#qstrncpy">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>A safe <code>strncpy()</code> function.</p>
/// <p>Copies at most <i>len</i> bytes from <i>src</i> (stopping at <i>len</i> or the terminating '\0' whichever comes first) into <i>dst</i> and returns a pointer to <i>dst</i>. Guarantees that <i>dst</i> is '\0'-terminated. If <i>src</i> or <i>dst</i> is 0, returns 0 immediately.</p>
/// <p>This function assumes that <i>dst</i> is at least <i>len</i> characters long.</p>
/// <p><b>Note: </b>When compiling with Visual C++ compiler version 14.00 (Visual C++ 2005) or later, internally the function strncpy_s will be used.</p><p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#qstrcpy">qstrcpy</a>().</p></div>
pub unsafe fn qstrncpy(
    dst: impl ::cpp_utils::CastInto<::cpp_utils::MutPtr<::std::os::raw::c_char>>,
    src: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    len: ::std::os::raw::c_uint,
) -> ::cpp_utils::MutPtr<::std::os::raw::c_char> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qstrncpy(
        ::cpp_utils::CastInto::<::cpp_utils::MutPtr<::std::os::raw::c_char>>::cast_into(dst)
            .as_mut_raw_ptr(),
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(src)
            .as_raw_ptr(),
        len,
    );
    ::cpp_utils::MutPtr::from_raw(ffi_result)
}

/// <p>A safe <code>strnicmp()</code> function.</p>
///
/// Calls C++ function: <span style='color: green;'>```int qstrnicmp(const char* arg1, const char* arg2, unsigned int len)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qbytearray.html#qstrnicmp">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>A safe <code>strnicmp()</code> function.</p>
/// <p>Compares at most <i>len</i> bytes of <i>str1</i> and <i>str2</i> ignoring the case of the characters. The encoding of the strings is assumed to be Latin-1.</p>
/// <p>Returns a negative value if <i>str1</i> is less than <i>str2</i>, 0 if <i>str1</i> is equal to <i>str2</i> or a positive value if <i>str1</i> is greater than <i>str2</i>.</p>
/// <p>Special case 1: Returns 0 if <i>str1</i> and <i>str2</i> are both 0.</p>
/// <p>Special case 2: Returns a random non-zero value if <i>str1</i> is 0 or <i>str2</i> is 0 (but not both).</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#qstrcmp">qstrcmp</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#qstrncmp">qstrncmp</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#qstricmp">qstricmp</a>(), and <a href="http://doc.qt.io/qt-5/qbytearray.html#8-bit-character-comparisons">8-bit Character Comparisons</a>.</p></div>
pub unsafe fn qstrnicmp_2_char_uint(
    arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    arg2: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    len: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_int {
    crate::__ffi::ctr_qt_core_ffi_qstrnicmp(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
            .as_raw_ptr(),
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg2)
            .as_raw_ptr(),
        len,
    )
}

/// <p>A safe <code>strnicmp()</code> function.</p>
///
/// Calls C++ function: <span style='color: green;'>```int qstrnicmp(const char* arg1, long long arg2, const char* arg3, long long arg4 = …)```</span>.
///
/// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbytearray.html#qstrnicmp">C++ documentation</a> for <span style='color: green;'>```int qstrnicmp(const char *str1, const char *str2, uint len)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>A safe <code>strnicmp()</code> function.</p>
/// <p>Compares at most <i>len</i> bytes of <i>str1</i> and <i>str2</i> ignoring the case of the characters. The encoding of the strings is assumed to be Latin-1.</p>
/// <p>Returns a negative value if <i>str1</i> is less than <i>str2</i>, 0 if <i>str1</i> is equal to <i>str2</i> or a positive value if <i>str1</i> is greater than <i>str2</i>.</p>
/// <p>Special case 1: Returns 0 if <i>str1</i> and <i>str2</i> are both nullptr.</p>
/// <p>Special case 2: Returns a random non-zero value if <i>str1</i> is nullptr or <i>str2</i> is nullptr (but not both).</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#qstrcmp">qstrcmp</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#qstrncmp">qstrncmp</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#qstricmp">qstricmp</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#8-bit-character-comparisons">8-bit Character Comparisons</a>, and <a href="http://doc.qt.io/qt-5/qbytearray.html#compare">QByteArray::compare</a>().</p></div>
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn qstrnicmp_char_longlong_char_longlong(
    arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    arg2: ::std::os::raw::c_longlong,
    arg3: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    arg4: ::std::os::raw::c_longlong,
) -> ::std::os::raw::c_int {
    crate::__ffi::ctr_qt_core_ffi_qstrnicmp2(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
            .as_raw_ptr(),
        arg2,
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg3)
            .as_raw_ptr(),
        arg4,
    )
}

/// <p>A safe <code>strnicmp()</code> function.</p>
///
/// Calls C++ function: <span style='color: green;'>```int qstrnicmp(const char* arg1, long long arg2, const char* arg3)```</span>.
///
/// Warning: no exact match found in C++ documentation. Below is the <a href="http://doc.qt.io/qt-5/qbytearray.html#qstrnicmp">C++ documentation</a> for <span style='color: green;'>```int qstrnicmp(const char *str1, const char *str2, uint len)```</span>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>A safe <code>strnicmp()</code> function.</p>
/// <p>Compares at most <i>len</i> bytes of <i>str1</i> and <i>str2</i> ignoring the case of the characters. The encoding of the strings is assumed to be Latin-1.</p>
/// <p>Returns a negative value if <i>str1</i> is less than <i>str2</i>, 0 if <i>str1</i> is equal to <i>str2</i> or a positive value if <i>str1</i> is greater than <i>str2</i>.</p>
/// <p>Special case 1: Returns 0 if <i>str1</i> and <i>str2</i> are both nullptr.</p>
/// <p>Special case 2: Returns a random non-zero value if <i>str1</i> is nullptr or <i>str2</i> is nullptr (but not both).</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#qstrcmp">qstrcmp</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#qstrncmp">qstrncmp</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#qstricmp">qstricmp</a>(), <a href="http://doc.qt.io/qt-5/qbytearray.html#8-bit-character-comparisons">8-bit Character Comparisons</a>, and <a href="http://doc.qt.io/qt-5/qbytearray.html#compare">QByteArray::compare</a>().</p></div>
///
/// This item is available if `any(cpp_lib_version="5.12.2", cpp_lib_version="5.13.0")`.
#[cfg(any(
    any(cpp_lib_version = "5.12.2", cpp_lib_version = "5.13.0"),
    ritual_rustdoc
))]
pub unsafe fn qstrnicmp_char_longlong_char(
    arg1: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    arg2: ::std::os::raw::c_longlong,
    arg3: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
) -> ::std::os::raw::c_int {
    crate::__ffi::ctr_qt_core_ffi_qstrnicmp3(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg1)
            .as_raw_ptr(),
        arg2,
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(arg3)
            .as_raw_ptr(),
    )
}

/// <p>A safe <code>strnlen()</code> function.</p>
///
/// Calls C++ function: <span style='color: green;'>```unsigned int qstrnlen(const char* str, unsigned int maxlen)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qbytearray.html#qstrnlen">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>A safe <code>strnlen()</code> function.</p>
/// <p>Returns the number of characters that precede the terminating '\0', but at most <i>maxlen</i>. If <i>str</i> is 0, returns 0.</p>
/// <p>This function was introduced in  Qt 4.2.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qbytearray.html#qstrlen">qstrlen</a>().</p></div>
pub unsafe fn qstrnlen(
    str: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    maxlen: ::std::os::raw::c_uint,
) -> ::std::os::raw::c_uint {
    crate::__ffi::ctr_qt_core_ffi_qstrnlen(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(str)
            .as_raw_ptr(),
        maxlen,
    )
}

/// Calls C++ function: <span style='color: green;'>```const QMetaObject* qt_getQtMetaObject()```</span>.
pub unsafe fn qt_get_qt_meta_object() -> ::cpp_utils::Ptr<crate::QMetaObject> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qt_getQtMetaObject();
    ::cpp_utils::Ptr::from_raw(ffi_result)
}

/// Calls C++ function: <span style='color: green;'>```void qt_message_output(QtMsgType arg1, const QMessageLogContext& context, const QString& message)```</span>.
pub unsafe fn qt_message_output(
    arg1: crate::QtMsgType,
    context: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QMessageLogContext>>,
    message: impl ::cpp_utils::CastInto<::cpp_utils::Ref<crate::QString>>,
) {
    crate::__ffi::ctr_qt_core_ffi_qt_message_output(
        arg1,
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QMessageLogContext>>::cast_into(context)
            .as_raw_ptr(),
        ::cpp_utils::CastInto::<::cpp_utils::Ref<crate::QString>>::cast_into(message).as_raw_ptr(),
    )
}

/// Calls C++ function: <span style='color: green;'>```QDebug qt_QMetaEnum_debugOperator(QDebug& arg1, int value, const QMetaObject* meta, const char* name)```</span>.
pub unsafe fn qt_q_meta_enum_debug_operator(
    arg1: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QDebug>>,
    value: ::std::os::raw::c_int,
    meta: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QMetaObject>>,
    name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
) -> ::cpp_utils::CppBox<crate::QDebug> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qt_QMetaEnum_debugOperator(
        ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QDebug>>::cast_into(arg1)
            .as_mut_raw_ptr(),
        value,
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QMetaObject>>::cast_into(meta).as_raw_ptr(),
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(name)
            .as_raw_ptr(),
    );
    ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
}

/// Calls C++ function: <span style='color: green;'>```void qt_QMetaEnum_flagDebugOperator(QDebug& debug, unsigned long sizeofT, int value)```</span>.
pub unsafe fn qt_q_meta_enum_flag_debug_operator_3a(
    debug: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QDebug>>,
    sizeof_t: ::std::os::raw::c_ulong,
    value: ::std::os::raw::c_int,
) {
    crate::__ffi::ctr_qt_core_ffi_qt_QMetaEnum_flagDebugOperator(
        ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QDebug>>::cast_into(debug)
            .as_mut_raw_ptr(),
        sizeof_t,
        value,
    )
}

/// Calls C++ function: <span style='color: green;'>```QDebug qt_QMetaEnum_flagDebugOperator(QDebug& dbg, quint64 value, const QMetaObject* meta, const char* name)```</span>.
pub unsafe fn qt_q_meta_enum_flag_debug_operator_4a(
    dbg: impl ::cpp_utils::CastInto<::cpp_utils::MutRef<crate::QDebug>>,
    value: u64,
    meta: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<crate::QMetaObject>>,
    name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
) -> ::cpp_utils::CppBox<crate::QDebug> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qt_QMetaEnum_flagDebugOperator1(
        ::cpp_utils::CastInto::<::cpp_utils::MutRef<crate::QDebug>>::cast_into(dbg)
            .as_mut_raw_ptr(),
        value,
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<crate::QMetaObject>>::cast_into(meta).as_raw_ptr(),
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(name)
            .as_raw_ptr(),
    );
    ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
}

/// <p>The qtTrId function finds and returns a translated string.</p>
///
/// Calls C++ function: <span style='color: green;'>```QString qtTrId(const char* id, int n = …)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtglobal.html#qtTrId">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The qtTrId function finds and returns a translated string.</p>
/// <p>Returns a translated string identified by <i>id</i>. If no matching string is found, the id itself is returned. This should not happen under normal conditions.</p>
/// <p>If <i>n</i> &gt;= 0, all occurrences of <code>%n</code> in the resulting string are replaced with a decimal representation of <i>n</i>. In addition, depending on <i>n</i>'s value, the translation text may vary.</p>
/// <p>Meta data and comments can be passed as documented for <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(). In addition, it is possible to supply a source string template like that:</p>
/// <p><code>//% &lt;C string&gt;</code></p>
/// <p>or</p>
/// <p><code>\begincomment% &lt;C string&gt; \endcomment</code></p>
/// <p>Example:</p>
/// <pre class="cpp">
///
/// &#32;     <span class="comment">//% "%n fooish bar(s) found.\n"</span>
/// &#32;     <span class="comment">//% "Do you want to continue?"</span>
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> text <span class="operator">=</span> qtTrId(<span class="string">"qtn_foo_bar"</span><span class="operator">,</span> n);
///
/// </pre>
/// <p>Creating QM files suitable for use with this function requires passing the <code>-idbased</code> option to the <code>lrelease</code> tool.</p>
/// <p><b>Warning:</b> This method is reentrant only if all translators are installed <i>before</i> calling this method. Installing or removing translators while performing translations is not supported. Doing so will probably result in crashes or other undesirable behavior.</p>
/// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">reentrant</a>.</p>
/// <p>This function was introduced in  Qt 4.6.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), <a href="http://doc.qt.io/qt-5/qcoreapplication.html#translate">QCoreApplication::translate</a>(), and <a href="http://doc.qt.io/qt-5/internationalization.html">Internationalization with Qt</a>.</p></div>
pub unsafe fn qt_tr_id_2a(
    id: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
    n: ::std::os::raw::c_int,
) -> ::cpp_utils::CppBox<crate::QString> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qtTrId(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(id)
            .as_raw_ptr(),
        n,
    );
    ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
}

/// <p>The qtTrId function finds and returns a translated string.</p>
///
/// Calls C++ function: <span style='color: green;'>```QString qtTrId(const char* id)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtglobal.html#qtTrId">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>The qtTrId function finds and returns a translated string.</p>
/// <p>Returns a translated string identified by <i>id</i>. If no matching string is found, the id itself is returned. This should not happen under normal conditions.</p>
/// <p>If <i>n</i> &gt;= 0, all occurrences of <code>%n</code> in the resulting string are replaced with a decimal representation of <i>n</i>. In addition, depending on <i>n</i>'s value, the translation text may vary.</p>
/// <p>Meta data and comments can be passed as documented for <a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(). In addition, it is possible to supply a source string template like that:</p>
/// <p><code>//% &lt;C string&gt;</code></p>
/// <p>or</p>
/// <p><code>\begincomment% &lt;C string&gt; \endcomment</code></p>
/// <p>Example:</p>
/// <pre class="cpp">
///
/// &#32;     <span class="comment">//% "%n fooish bar(s) found.\n"</span>
/// &#32;     <span class="comment">//% "Do you want to continue?"</span>
/// &#32;     <span class="type"><a href="http://doc.qt.io/qt-5/qstring.html">QString</a></span> text <span class="operator">=</span> qtTrId(<span class="string">"qtn_foo_bar"</span><span class="operator">,</span> n);
///
/// </pre>
/// <p>Creating QM files suitable for use with this function requires passing the <code>-idbased</code> option to the <code>lrelease</code> tool.</p>
/// <p><b>Warning:</b> This method is reentrant only if all translators are installed <i>before</i> calling this method. Installing or removing translators while performing translations is not supported. Doing so will probably result in crashes or other undesirable behavior.</p>
/// <p><b>Note:</b> This function is <a href="http://doc.qt.io/qt-5/threads-reentrancy.html">reentrant</a>.</p>
/// <p>This function was introduced in  Qt 4.6.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qobject.html#tr">QObject::tr</a>(), <a href="http://doc.qt.io/qt-5/qcoreapplication.html#translate">QCoreApplication::translate</a>(), and <a href="http://doc.qt.io/qt-5/internationalization.html">Internationalization with Qt</a>.</p></div>
pub unsafe fn qt_tr_id_1a(
    id: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
) -> ::cpp_utils::CppBox<crate::QString> {
    let ffi_result = crate::__ffi::ctr_qt_core_ffi_qtTrId1(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(id)
            .as_raw_ptr(),
    );
    ::cpp_utils::CppBox::from_raw(ffi_result).expect("attempted to construct a null CppBox")
}

/// <p>This function deletes the variable <i>varName</i> from the environment.</p>
///
/// Calls C++ function: <span style='color: green;'>```bool qunsetenv(const char* varName)```</span>.
///
/// <a href="http://doc.qt.io/qt-5/qtglobal.html#qunsetenv">C++ documentation</a>:<div style='border: 1px solid #5CFF95; background: #D6FFE4; padding: 16px;'><p>This function deletes the variable <i>varName</i> from the environment.</p>
/// <p>Returns <code>true</code> on success.</p>
/// <p>This function was introduced in  Qt 5.1.</p>
/// <p><b>See also </b><a href="http://doc.qt.io/qt-5/qtglobal.html#qputenv">qputenv</a>() and <a href="http://doc.qt.io/qt-5/qtglobal.html#qgetenv">qgetenv</a>().</p>
///
/// <h2>Macro Documentation</h2></div>
pub unsafe fn qunsetenv(
    var_name: impl ::cpp_utils::CastInto<::cpp_utils::Ptr<::std::os::raw::c_char>>,
) -> bool {
    crate::__ffi::ctr_qt_core_ffi_qunsetenv(
        ::cpp_utils::CastInto::<::cpp_utils::Ptr<::std::os::raw::c_char>>::cast_into(var_name)
            .as_raw_ptr(),
    )
}
